SET(CMAKE_MODULE_PATH ${CMAKE_MODULE_PATH} ${CMAKE_SOURCE_DIR}/cmake/Modules)
#MESSAGE("CMAKE_CURRENT_SOURCE_DIR=${CMAKE_CURRENT_SOURCE_DIR}")

## Boost
set(Boost_USE_STATIC_LIBS ON)
set(Boost_USE_MULTITHREADED ON)
set(Boost_USE_STATIC_RUNTIME OFF)
#set(Boost_DEBUG TRUE)
# Disable the config mode of find_package(Boost)
#set(Boost_NO_BOOST_CMAKE ON)
# Required boost modules
find_package(Boost #REQUIRED
    COMPONENTS system filesystem thread date_time iostreams
    )
if(Boost_FOUND)
    # Obtain diagnostic information about Boost's automatic linking outputted 
    # during compilation time.
    add_definitions(${Boost_LIB_DIAGNOSTIC_DEFINITIONS})
    include_directories(SYSTEM ${Boost_INCLUDE_DIRS})
    link_directories(${Boost_LIBRARY_DIRS})
    SET(Boost_DLL_Core
        "boost_date_time"
        "boost_system"
        "boost_thread"
        "boost_chrono"
        )
    SET(Boost_DLL_DEBUG
        #"boost_date_time-vc110-mt-gd-1_55.dll"
        #"boost_system-vc110-mt-gd-1_55"
        )
    SET(Boost_DLL_RELEASE
        #"boost_date_time-vc110-mt-1_55.dll"
        #"boost_system-vc110-mt-1_55.dll"
        )
else (Boost_FOUND)
    MESSAGE(FATAL_ERROR "Boost was not found.")
endif(Boost_FOUND)

## CGAL
find_package(CGAL QUIET #REQUIRED
    COMPONENTS Core
    )
IF(CGAL_FOUND)
	MESSAGE("CGAL found   ${CGAL_USE_FILE}")
	INCLUDE( ${CGAL_USE_FILE} )
	SET( CMAKE_VERBOSE_MAKEFILE OFF )
ELSE(CGAL_FOUND)
	MESSAGE("Error : CGAL not found")
ENDIF(CGAL_FOUND)

## CLAPACK
SET (CLAPACK_INCLUDE "${CMAKE_SOURCE_DIR}/modules/thirdParty/CLAPACK/include")
SET (CLAPACK_LIBRARY
    "${CMAKE_SOURCE_DIR}/modules/thirdParty/CLAPACK/lib/${PlatformName}/${CMAKE_CFG_INTDIR}/blas.lib"
    "${CMAKE_SOURCE_DIR}/modules/thirdParty/CLAPACK/lib/${PlatformName}/${CMAKE_CFG_INTDIR}/lapack.lib"
    "${CMAKE_SOURCE_DIR}/modules/thirdParty/CLAPACK/lib/${PlatformName}/${CMAKE_CFG_INTDIR}/libf2c.lib"
    )

## CUDA
if(USE_CUDA)
    #include(${CMAKE_SOURCE_DIR}/cmake/find_cuda.cmake)
    find_package(CUDA)
    if(CUDA_FOUND)
        ADD_DEFINITIONS(-DUSE_CUDA)
    else(CUDA_FOUND)
        SET(USE_CUDA OFF)
    endif(CUDA_FOUND)
endif(USE_CUDA)

## Eigen
#find_package(Eigen3 #REQUIRED)
SET (Eigen_INCLUDE "${CMAKE_SOURCE_DIR}/modules/thirdParty/Eigen")

## Fastest Fourier Transform in the West (short name: FFTW)
SET (FFTW_BIN "${CMAKE_SOURCE_DIR}/modules/thirdParty/libfftw3/bin/${PlatformName}")
SET (FFTW_LIBRARY
    "${CMAKE_SOURCE_DIR}/modules/thirdParty/libfftw3/lib/${PlatformName}/libfftw.lib"
    "${CMAKE_SOURCE_DIR}/modules/thirdParty/libfftw3/lib/${PlatformName}/libfftwf.lib"
    )
SET (FFTW_DLL
    "${FFTW_BIN}/libfftw_64.dll"
    "${FFTW_BIN}/libfftwf_64.dll"
    )

## Mosek
# SET (Mosek_BIN "${CMAKE_SOURCE_DIR}/modules/thirdParty/mosek/bin/${PlatformName}")
# SET (Mosek_LIBRARY
#    "${CMAKE_SOURCE_DIR}/modules/thirdParty/mosek/lib/${PlatformName}/mosek64_6_0.lib"
#    )
#SET (Mosek_DLL
#    "${Mosek_BIN}/mosek64_6_0.dll"
#    )

## Nvidia CG
FIND_PACKAGE(Cg REQUIRED)
IF (NOT CG_FOUND)
    MESSAGE(FATAL_ERROR "Cg was not found.")
ENDIF(NOT CG_FOUND)

## Open Asset Import Library (short name: Assimp)
SET(ASSIMP_BIN "${CMAKE_SOURCE_DIR}/modules/thirdParty/OpenAsset/bin/${PlatformName}")
SET (ASSIMP_INCLUDE "${CMAKE_SOURCE_DIR}/modules/thirdParty/OpenAsset/include")
SET (ASSIMP_LIBRARY "${CMAKE_SOURCE_DIR}/modules/thirdParty/OpenAsset/lib/${PlatformName}/assimp.lib")

## OpenCV
#SET(BUILD_SHARED_LIBS OFF)
FIND_PACKAGE(OpenCV REQUIRED)
IF (NOT OpenCV_FOUND)
    MESSAGE(FATAL_ERROR "OpenCV was not found.")
ENDIF(NOT OpenCV_FOUND)
SET(OpenCV_DLL
    "opencv_core"
    "opencv_highgui"
    )

## OpenGL
FIND_PACKAGE(OpenGL REQUIRED)

## OpenMP
find_package(OpenMP)
if (OPENMP_FOUND)
    set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} ${OpenMP_C_FLAGS}")
    set (CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ${OpenMP_CXX_FLAGS}")
endif()

## Qt
IF (QT_VERSION EQUAL 4)
    ## QT4
    SET(QT_USE_QTOPENGL TRUE)
    SET(QT_USE_QTXML TRUE)
    SET(QT_BIN "${QT_DIR}/bin")
    SET(CMAKE_PREFIX_PATH
        "${QT_BIN}"
        )
    FIND_PACKAGE(QT4 REQUIRED)
    IF (NOT QT4_FOUND)
        MESSAGE(FATAL_ERROR "Qt4 was not found.")
    ENDIF(NOT QT4_FOUND)
    INCLUDE(${QT_USE_FILE})
    ADD_DEFINITIONS(${QT_DEFINITIONS})

    SET(QT_DLL_DEBUG
        "QtCored4.dll"
        "QtGuid4.dll"
        "QtOpenGLd4.dll"
        "QtXmld4.dll"
        )
    SET(QT_DLL_RELEASE
        "QtCore4.dll"
        "QtGui4.dll"
        "QtOpenGL4.dll"
        "QtXml4.dll"
        )
ELSEIF (QT_VERSION EQUAL 5)
    ##QT5
    set(CMAKE_INCLUDE_CURRENT_DIR ON)
    #set(CMAKE_AUTOMOC ON)
    SET(QT_BIN "${QT_DIR}/bin")
    set(CMAKE_PREFIX_PATH
        "${QT_DIR}/lib/cmake"
        "C:/Program Files (x86)/Windows Kits/8.0/Lib/win8/um/x64"
        )
    find_package(Qt5Widgets)
    find_package(Qt5Core)
    find_package(Qt5Gui)
    find_package(Qt5OpenGL)
    find_package(Qt5Xml)
    set(QT_LIBRARIES "Qt5::Widgets;Qt5::Core;Qt5::Gui;Qt5::OpenGL;Qt5::Xml")

    SET(QT_DLL_DEBUG
        "Qt5Cored.dll"
        "Qt5Guid.dll"
        "Qt5OpenGLd.dll"
        "Qt5Xml.dll"
        )
    SET(QT_DLL_RELEASE
        "Qt5Core.dll"
        "Qt5Gui.dll"
        "Qt5OpenGL.dll"
        "Qt5Xmld.dll"
        )
ENDIF ()

