//////////////////////////////////////////////////////////////////////////

/// author: Michael Wand
/// last modified: March 2011

//////////////////////////////////////////////////////////////////////////

#ifndef GlobalDefinitionsH
#define GlobalDefinitionsH

//////////////////////////////////////////////////////////////////////////

/// we always use namespace std
namespace std {}
using namespace std;

#ifdef X4_PLATFORM_MSVC
   /// define compiler type ms visual c++
   #define COMPILER_MSVC
   /// enable qt gui support
   #define PLATFORM_QT
   /// use win32 specific low-level routines
   #define PLATFORM_WINDOWS
   /// 32/64 bit switch
   #ifdef _WIN64
      #define X4_64BIT
   #else
      #define X4_32BIT
   #endif

   // (art) - disabled anything like secure_scl, since this makes problems with external libraries
   // since vs2010 there is a new concept ITERATOR_DEBUG_LEVEl, which is when _DEBUG is defined is set to 2
   // In release mode it set to 0, except when _SECURE_SCL is defined. However most of the external libraries
   // are just use either 0 or 2 and nothing like 1 for the ITERATOR_DEBUG_LEVEL. Hence I've just disabled
   // this option here. Hence some range checks are not performed in the FastDebug mode anymore !
#if 0
   // platform specific settings
   // default / deprecated in vs 2010
   // /// iterator range checking: problems cause exceptions
   // #define _SECURE_SCL_THROWS 1
   #if defined(X4_DEBUG)
      /// enable checking in debug/fastdebug
      #define _SECURE_SCL 1
   #else
      ///disable otherwise (release/fullopt)
      #define _SECURE_SCL 0
   #endif
#endif

   // dll-export warning for private members without dll-interface
   #pragma warning ( disable : 4251 )

   // dll-export warning for baseclasses without dll-interface
   #pragma warning ( disable : 4275 )

   // we don't care about this... :-)
	#ifndef _CRT_SECURE_NO_WARNINGS
		#define _CRT_SECURE_NO_WARNINGS
	#endif
	//#ifndef D_SCL_SECURE_NO_WARNINGS
	//	#define D_SCL_SECURE_NO_WARNINGS
	//#endif

   
#endif


/// default target for GCC / QT / LINUX (32 bit)
#ifdef X4_PLATFORM_LINUX_GCC

   /// define compiler type gcc
   #define COMPILER_GCC

   /// enable qt gui support
   #define PLATFORM_QT
   /// use linux (32 bit) specific low-level routines
   #define PLATFORM_LINUX32

   /// change this for a linux-64bit port
   #define X4_32BIT
#endif

/// default target for GCC / QT / WINDOWS (32/64 bit)
#ifdef X4_PLATFORM_MINGW_GCC

	/// define compiler type gcc
	#define COMPILER_GCC

	/// enable qt gui support
	#define PLATFORM_QT

	/// use win32 specific low-level routines
	#define PLATFORM_WINDOWS

	/// 32/64 bit switch
	#ifdef _WIN64
		#define X4_64BIT
	#else
		#define X4_32BIT
	#endif
#endif

#ifdef PLATFORM_WINDOWS
	 #include <direct.h>
	 #define GetCurrentDir _getcwd
#else
	 #include <unistd.h>
	 #define GetCurrentDir getcwd
#endif


#if (!defined X4_PLATFORM_MSVC && !defined X4_PLATFORM_LINUX_GCC && !defined X4_PLATFORM_MINGW_GCC)
#error "Unknown build platform. Stop here."
#endif


// -- general definitions
#if (__cplusplus == 201103L)
#define X4CPP11
#else
#undef X4CPP11
#endif

#ifndef X4CPP11
/// final keyword missing, so just forget about it...
#define x4_method_final
#else
#define x4_method_final final

#endif


//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
