//////////////////////////////////////////////////////////////////////////

/// author: Michael Wand, WSI/GRIS
/// started: 2000
/// last modified: March 2010
/// license: GPL V2 or 3

//////////////////////////////////////////////////////////////////////////

#ifndef X4TypesH
#define X4TypesH

//////////////////////////////////////////////////////////////////////////

#include "x4basics.h"

//////////////////////////////////////////////////////////////////////////

/*--------------------------------------------------------------------------------*/
/* ptypes.h: elementary datatypes to enable portability between                   */
/*           multiple platforms                                                   */
/*--------------------------------------------------------------------------------*/

namespace X4 {

	// native platform int-types, maximum sized ints that can be used for addressing (indexing, pointers)
	typedef intptr_t                  max_platform_int;
	typedef uintptr_t                 max_platform_card;

	//---------------------------------------------------------------------------
#if (defined(X4_PLATFORM_MSVC) || defined(X4_PLATFORM_MINGW_GCC)) && defined(X4_32BIT)
	typedef signed   char            int8;
	typedef signed   short int       int16;
	typedef signed   int             int32;
	typedef signed   __int64         int64;

	typedef unsigned char            card8;
	typedef unsigned short int       card16;
	typedef unsigned int             card32;
	typedef unsigned __int64         card64;

	typedef float                    float32;
	typedef double                   float64;

	typedef ptrdiff_t						pointer_diff;

	#elif (defined(X4_PLATFORM_MSVC) || defined(X4_PLATFORM_MINGW_GCC)) && defined(X4_64BIT)

	typedef signed   char            int8;
	typedef signed   short int       int16;
	typedef signed   int             int32;
	typedef signed   __int64         int64;

	typedef unsigned char            card8;
	typedef unsigned short int       card16;
	typedef unsigned int             card32;
	typedef unsigned __int64         card64;

	typedef float                    float32;
	typedef double                   float64;

	typedef ptrdiff_t						pointer_diff;

	#else
	#error "Undefined platform!"
	#endif

	//---------------------------------------------------------------------------

	// shortcuts
	typedef max_platform_card mpcard;
	typedef max_platform_int  mpint;
	typedef char byte;

}




#endif

