//---------------------------------------------------------------------------
#ifndef AttachedDataH
#define AttachedDataH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "HashTables.h"
#include "X4Assert.h"
#include <string>
#include <vector>
#include <Tools.h>
//---------------------------------------------------------------------------

namespace X4 {


    class MemoryManager;
    class MemoryManagedObject;
    class AttachedDataContainer;



    class X4CORE_API AttachedData : public Persistent{
        X4_CLASS (AttachedData)
    private:
        std::string m_name;
        card32 m_flags;
        MemoryManagedObject* m_memManObj;
        AttachedDataContainer   *  m_MyContainer;

    protected:

        static void delCallback(std::string name, AttachedData *data);

        friend AttachedDataContainer;

    public:
        enum AttachedDataFlags { ADF_PERSISTENT = 1, ADF_DELETEONUPDATE = 2, ADF_QUERY_DELETE_ON_UPDATE = 4 };

        enum ChangeType { CT_ATTRIBUTE_CHANGED,    // some attribute channel other than "position" changed
            CT_POSITION_CHANGED,     // some attribute channel including "position" changed
            CT_ADD_DELETE,           // objects have been added/removed
            CT_EVERYTHING_CHANGED }; // more involved/unknown changes

        AttachedData();
        virtual ~AttachedData();
        /// setup instance: flags & ADF_PERSISTENT --> data will be saved along with container
        /// flags & ADF_DELETEONUPDATE --> data will be deleted if the object the data is attached to changes
        /// flags & ADF_QUERY_DELETE_ON_UPDATE --> attachment will be informed of changes and can decide if it should be deleted
        /// MemoryManager: if not null, the object will be deleted if memory pool runs out of resources
        /// hint: usually ADF_PERSISTENT and (ADF_DELETEONUPDATE || memManager) are used mutually exclusive
        virtual void setup(const std::string& name, const card32 flags, MemoryManager *memManager = NULL);

        virtual bool queryDelete(ChangeType whatChanged) {forget(whatChanged); return false;}

        inline card32 getFlags() const {return m_flags;}
        inline void setFlags(card32 f) {m_flags = f;}

        inline const std::string& getName() const {return m_name;}

        virtual void assign(const Object* obj, X4CopyContext *context = NULL);
        /// size of this attachment for memory management
        /// (to-do: currently static over att. lifetime)
        /// (default = 1, allows simple instance counting)
        virtual card32 getSize() const {return 1;}


        friend MemoryManager;
        friend AttachedDataContainer;
    };





}


#endif
