//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "AttachedDataContainer.h"
//---------------------------------------------------------------------------
#include <X4Assert.h>
#include <Tools.h>
#include <HashTables.inline.h>
#include "MemoryManager.h"
#include "NumericalClassProperty.h"
#include "StringClassProperty.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


IMPLEMENT_X4_CLASS( AttachedDataContainer, 1 )
{
    BEGIN_CLASS_INIT(AttachedDataContainer);
}

// ---


max_platform_card AttachedDataContainer::MyHashTable::hashFunction(std::string key) const {
   card32 hk = 0;
   const card32 n = (card32)key.size();
   for (card32 i=0; i<n; i++) {
      const card8 c = key[i];
      hk <<= 1;
      hk ^=c;
   }
   return (max_platform_card)hk;
}

AttachedDataContainer::MyHashTable::MyHashTable(card32 fillFactor, card32 startSizeIndex)
: HashTable<std::string, AttachedData*, StringComp>(fillFactor, startSizeIndex) {}


AttachedDataContainer::AttachedDataContainer() {
   m_dataHashTable = new MyHashTable(4 /* fill factor */, 1 /* initial size */);
}

static void writeCallback(void *context, std::string name, AttachedData *data) {
   OutputObjectStream *s = (OutputObjectStream*)context;
   if( data->getFlags() & AttachedData::ADF_PERSISTENT )
      s->writeObject((Persistent*)data);
}

static void countDataCallback( void * context, std::string name, AttachedData * data )
{
   card32 * numData = (card32*)context;
   if( data->getFlags() & AttachedData::ADF_PERSISTENT )
      numData[0]++;
}

void AttachedDataContainer::write(OutputObjectStream *s) const
{
   card32 numData=0;
   m_dataHashTable->forEachEntry(&numData, countDataCallback);
   s->write(numData);
   m_dataHashTable->forEachEntry(s, writeCallback);
}

void AttachedDataContainer::read(InputObjectStream *s) {
   clear();
   card32 count;
   s->read(count);
   for (card32 i=0; i<count; i++) {
      AttachedData *d;
      s->readObject((Persistent*&)d);
      attachData(d);
   }
}


static void copyCallback(void* context, std::string name, AttachedData *data) {
   AttachedDataContainer *thisInst = (AttachedDataContainer*)context;
   if ((data->getFlags() & AttachedData::ADF_PERSISTENT) != 0) {
      thisInst->attachData((AttachedData*)(data->copy()));
   }
}

void AttachedDataContainer::clear() {
   m_dataHashTable->forEachEntry(AttachedData::delCallback);
   delete m_dataHashTable;
   m_dataHashTable = new MyHashTable(4 /* fill factor */, 1 /* initial size */);
}

void AttachedDataContainer::assign(const Object* obj, X4CopyContext *context) {
   const AttachedDataContainer *other = dynamic_cast<const AttachedDataContainer*>(obj);
   if (other == NULL) {
      x4Assert(false);
   } else {
      clear();
      other->m_dataHashTable->forEachEntry(this, copyCallback);
   }
   forget(context);
}

static void deleteOnUpdateCallBack(void* context, std::string name, AttachedData *data) {
   AttachedDataContainer *cThis = (AttachedDataContainer*)context;
   AttachedData::ChangeType change = cThis->temp_change_for_callback;
   if ((data->getFlags() & AttachedData::ADF_DELETEONUPDATE) != 0
       || (((data->getFlags() & AttachedData::ADF_QUERY_DELETE_ON_UPDATE) != 0)) && data->queryDelete(change)) {
      cThis->deleteData(name);
   }
}

void AttachedDataContainer::deleteDelOnWriteData(AttachedData::ChangeType change) {
   temp_change_for_callback = change;
   m_dataHashTable->forEachEntry(this, deleteOnUpdateCallBack);
}

void getAllCallback( void * context, std::string key, AttachedData * data )
{
   std::vector<AttachedData*> * pVec = (std::vector<AttachedData*> *)context;
   pVec->push_back( data );
   forget(key);
}

void AttachedDataContainer::getAll( std::vector<AttachedData*> &ret )
{
   m_dataHashTable->forEachEntry( (void *)&ret, getAllCallback );
}


AttachedData* AttachedDataContainer::getData(const std::string& name) const {
   AttachedData *result = NULL;
   bool success = m_dataHashTable->getData(name, result);
   if (success) {
      return result;
   } else {
      return NULL;
   }
}

void AttachedDataContainer::attachData(AttachedData* data)
{
	if (data == NULL) return;
   x4Assert( data->m_MyContainer == NULL );
   const std::string name = data->getName();
   AttachedData *oldData = NULL;
   if (m_dataHashTable->getData(name, oldData)) {
      oldData->m_MyContainer = NULL;
      delete oldData;
   }
   data->m_MyContainer = this;
   m_dataHashTable->addElement(data->getName(), data);
}

void AttachedDataContainer::deleteData(const std::string &name) {
   AttachedData *oldData = NULL;
   if (m_dataHashTable->getData(name, oldData)) {
      oldData->m_MyContainer = NULL;
      delete oldData;
      m_dataHashTable->deleteElement(name);
   } else {
      x4Assert(false);
   }
}

void AttachedDataContainer::removeData(const std::string &name)
{
   AttachedData *oldData = NULL;
   if (m_dataHashTable->getData(name, oldData)) {
      m_dataHashTable->deleteElement(name);
   } else {
      x4Assert(false);
   }
}

AttachedDataContainer::~AttachedDataContainer() {
   m_dataHashTable->forEachEntry(AttachedData::delCallback);
   delete m_dataHashTable;
}

mpcard AttachedDataContainer::getSizeInMemory()
{
	std::vector<AttachedData*> att;
	getAll(att);
	card32 ret = 0;
	for( unsigned i=0;i<att.size();i++ )
	{
		ret += att[i]->getSize();
	}
	return ret;
}
