//---------------------------------------------------------------------------
#ifndef AttachedDataContainerH
#define AttachedDataContainerH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "HashTables.h"
#include "X4Assert.h"
#include <string>
#include <vector>
#include "AttachedData.h"
//---------------------------------------------------------------------------

namespace X4 {
   

class MemoryManager;
class MemoryManagedObject;
class AttachedDataContainer;



class X4CORE_API AttachedDataContainer : public Persistent {
    X4_CLASS (AttachedDataContainer)

 private:
   class X4CORE_API StringComp {
     public:
       static bool equals(std::string obj1, std::string obj2) { return obj1 == obj2; }
   };

   class MyHashTable : public HashTable<std::string, AttachedData*, StringComp> {
    protected:
      virtual max_platform_card hashFunction(std::string key) const;
    public:
      MyHashTable(card32 fillFactor = 1, card32 startSizeIndex = 8);

   };
   mutable MyHashTable *m_dataHashTable;

 public:
   AttachedDataContainer();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   AttachedData* getData(const std::string& name) const;
   void attachData(AttachedData* data);
   void deleteData(const std::string &name);
   /// works like deleteData, but without deleting attachment
   void removeData(const std::string &name);

   void getAll( std::vector<AttachedData*> &ret );

   void clear();

   /// deletes all attached data which is marked "ADF_DELETEONUPDATE" and possibly/checks for "ADF_QUERY_DELETE_ON_UPDATE"
   void deleteDelOnWriteData(AttachedData::ChangeType change = AttachedData::CT_EVERYTHING_CHANGED);
   /// do not use!
   AttachedData::ChangeType temp_change_for_callback;

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual ~AttachedDataContainer();

	/// sum all attachment data
	mpcard getSizeInMemory();
};




}


#endif
