//---------------------------------------------------------------------------
#ifndef _BufferStreamTest_H
#define _BufferStreamTest_H
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include <ObjectStreams.h>
#include <BufferObjectStreams.h>
//---------------------------------------------------------------------------
#include <map>
//---------------------------------------------------------------------------


/*
The buffer object streams provide object streams that store the serialized data in an in-memory
buffer or deserialize objects from a pre-existing in-memory buffer respectively.
*/
namespace X4 {

	class X4CORE_API BufferOutputObjectStreamComparison : public BufferOutputObjectStream {
		private:
			std::map<mpcard, card64> _objPtrMap;
			typedef std::map<mpcard, card64>::iterator ObjPtrMapIter;
			card32 _objPtrCounter;
			
			bool _getObjectID(card64 & id, const Object * pObj) 
			{
				ObjPtrMapIter iter = _objPtrMap.find((mpcard)pObj);
				if (iter == _objPtrMap.end()) {
					// new object ptr
					_objPtrCounter++; // get new id
					_objPtrMap[(mpcard)pObj] = _objPtrCounter;
					id = _objPtrCounter;
					return false;
				}
				else { // object ptr is already known
					id = iter->second;
					return true;
				}
			}
			card32 _getNumRegisteredObjectPointers() const { return _objPtrCounter; }

		public:
		
			BufferOutputObjectStreamComparison(bool perObjectClassVersion = false, StreamClassTable *classTable = NULL, bool doWriteClassTable = true);
			virtual void writeObject(const Persistent *obj);
			/// remove already written object from cache, so that it can be written again even if it is the same object
			virtual void removeFromCache(Persistent *obj);
			/// clean the whole cache of already written objects
			virtual void clearCache();

	};

	//! just for convenience also define the input reader (same as the BufferInputObjectStream)
	typedef BufferInputObjectStream BufferInputObjectStreamComparison;

	//! compare 2 persistent objects whether they contain the same data (invariant to pointers)
	//  Beware, this comparison is not very efficient!
	bool operator==(const X4::Persistent & obj1, const X4::Persistent & obj2);

}//namespace X4

#endif
