//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "BufferObjectStreams.h"
//---------------------------------------------------------------------------
#include <ConsoleOutput.h>
#include <X4Exception.h>
#include "MetaClass.h"
#include "StreamClassTable.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------

BufferOutputObjectStream::BufferOutputObjectStream(bool perObjectClassVersion, StreamClassTable *classTable, bool doWriteClassTable) :
		OutputObjectStream(classTable, doWriteClassTable)
{
	classTabPointerPos = 0;
	offset = 0;
	finalized = false;
	initStream();
}


void BufferOutputObjectStream::writeClassTableHeader() {
    // Place holder for pointer to class table, will be filled later,
	// in method writeClassTable()
	classTabPointerPos = offset;
	write<card64>(0);
}

card64 BufferOutputObjectStream::seek(int64 rel)
{ 
	if (static_cast<int64>(offset) + rel < 0 || static_cast<int64>(offset) + rel > static_cast<int64>(buffer.size()))
		throw EIOException("BufferOutputObjectStream::seek() - Attempted to seek out of range.");

	offset = static_cast<size_t>(static_cast<int64>(offset) + rel);
	return offset;
}

void BufferOutputObjectStream::writeBuffer(const void* buf, mpcard length) {
	if (finalized)
		throw EIOException("BinaryOutputObjectStream::writeBuffer() - stream is already finalized");
	x4Assert(offset <= buffer.size());

	if (buffer.size() < offset + length)
		buffer.resize(offset + length, 0);
	memcpy(&buffer[offset], buf, length);
	offset += length;
}

void BufferOutputObjectStream::writeClassTable() {
   card64 classTablePos = offset;

   classTable->write(this);

   size_t offsetBackup = offset;
   offset = classTabPointerPos;
   write<card64>(classTablePos);
   offset = offsetBackup;
}

OutputStream* BufferOutputObjectStream::createSiblingStream(card64 uid, bool raw)
{
	//return new BufferOutputObjectStream(*this);
	return this;
}

//---------------------------------------------------------------------------

BufferInputObjectStream::BufferInputObjectStream(const byte *buffer, size_t bufferSize) : 
	InputObjectStream(), buffer(buffer), bufferSize(bufferSize)
{
	offset = 0;
	initStream();
}

void BufferInputObjectStream::readClassTable() {
   card64 classTablePos;
   read<card64>(classTablePos);

   size_t offsetBackup = offset;
   offset = classTablePos;
   classTable->read(this);
   offset = offsetBackup;
}

void BufferInputObjectStream::readBuffer(void* buf, mpcard length) {
   if (length==0) return;

   if (offset + length > bufferSize)
   {
	   throw EIOException("BufferInputObjectStream::readBuffer - could not read data.");
   }

   memcpy(buf, &buffer[offset], length);
   offset += length;
}

card64 BufferInputObjectStream::seek(int64 rel)
{ 
	if (static_cast<int64>(offset) + rel < 0 || static_cast<int64>(offset) + rel > static_cast<int64>(bufferSize))
		throw EIOException("BufferInputObjectStream::seek() - Attempted to seek out of buffer range.");

	offset = static_cast<size_t>(static_cast<int64>(offset) + rel);
	return offset;
}


InputStream* BufferInputObjectStream::createSiblingStream(card64 uid, bool raw)
{
	//return new BufferInputObjectStream(*this);
	return this;
}
