//---------------------------------------------------------------------------
#ifndef BufferObjectStreamsH
#define BufferObjectStreamsH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "ObjectStreams.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------


/*
The buffer object streams provide object streams that store the serialized data in an in-memory
buffer or deserialize objects from a pre-existing in-memory buffer respectively.
*/

namespace X4 {

class X4CORE_API BufferOutputObjectStream : public OutputObjectStream {
 private:
   std::vector<byte> buffer;
   size_t offset;
   bool finalized;
   card64 classTabPointerPos;

 protected:
   virtual void writeClassTableHeader();
   virtual void writeClassTable();

 public:
   BufferOutputObjectStream(bool perObjectClassVersion = false, StreamClassTable *classTable = NULL, bool doWriteClassTable = true);
   virtual bool canSeek() const { return true; }
   virtual card64 seek(int64 rel);
   virtual void writeBuffer(const void* buf, mpcard length);
   // The returned pointer remains valid as long as the BufferOutputObjectStream object exists and as soon as the stream has been finalized.
   const byte *getBuffer() const { x4Assert(finalized); return buffer.data(); }
   void finalize() { x4Assert(!finalized); if(doWriteClassTable) writeClassTable(); finalized = true; }
   size_t getBufferSize() const { return buffer.size(); }
   //! create a new stream (overridden)
   virtual OutputStream* createSiblingStream(card64 uid, bool raw = false);
};

class X4CORE_API BufferInputObjectStream : public InputObjectStream {
private:
   const byte *buffer;
   size_t bufferSize;
   size_t offset;

protected:
   virtual void readClassTable();

public:
   BufferInputObjectStream(const byte *buffer, size_t bufferSize);
   virtual void readBuffer(void* buf, mpcard length);
   virtual bool canSeek() const { return true; }
   virtual card64 seek(int64 rel);
   //! create a new stream (overridden)
   virtual InputStream* createSiblingStream(card64 uid, bool raw = false);
};

}

#endif
