//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ClassMethods.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


ClassMethod::ClassMethod(std::string name, std::string description, std::string icon, bool visible)
{
	this->name = name;
	this->description = description;
	this->icon = icon;
	this->visible = visible;
	this->packageAsCmdObjHint = false;
}

std::string ClassMethod::getDescription() const {
   return description;
}


std::string ClassMethod::getName() const {
	return name;
}


std::string ClassMethod::getIcon() const {
	return icon;
}

bool ClassMethod::getVisible() const {
	return visible;
}

void ClassMethod::setVisible(bool v) {
   visible = v;
}


bool ClassMethod::getPackageAsCmdObjHint() const {
   return packageAsCmdObjHint;
}

void ClassMethod::setPackageAsCmdObjHint(bool h) {
   packageAsCmdObjHint = h;
}


// ---

NoArgsClassMethod::NoArgsClassMethod(MethodPointer method, std::string name, std::string description, std::string icon, bool visible) : ClassMethod(name, description, icon, visible) {
   this->method = method;
}

void NoArgsClassMethod::call(Object *obj) const {
   (obj->*method)();
}

ClassMethod *NoArgsClassMethod::copyMethod() const {
   ClassMethod *result = new NoArgsClassMethod(method, getName(), getDescription(), getIcon());
   return result;
}

// ---
NoArgsClassMethodDispatcher::NoArgsClassMethodDispatcher(DispatcherMethodPointer dispatcher, ClassMethod* method, std::string name, std::string description, std::string icon, bool visible ) :
    NoArgsClassMethod(NULL, name, description, icon, visible)
{
    this->dispatcher = dispatcher;
    this->method = method;
}

NoArgsClassMethodDispatcher::~NoArgsClassMethodDispatcher()
{
    if (this->method) delete this->method;
}

void NoArgsClassMethodDispatcher::call(Object *obj) const
{
    x4Assert(method != NULL && dispatcher != NULL);
    (obj->*dispatcher)(method);
}

ClassMethod *NoArgsClassMethodDispatcher::copyMethod() const
{
    ClassMethod *result = new NoArgsClassMethodDispatcher(dispatcher, method, getName(), getDescription(), getIcon(), getVisible());
    return result;
}


// ---

BasicArgsClassMethod::BasicArgsClassMethod(
   MethodPointer invoker,
   std::string name,
   const VariantTypeList parameterTypes, const Variant::Type resultType,
   std::string description, std::string icon, bool visible)
: ClassMethod(name, description, icon, visible)
{
   this->method = invoker;
   this->parameterTypes = parameterTypes;
   this->resultType = resultType;
}


void BasicArgsClassMethod::call( Object *obj, VariantList &params, Variant &result ) const
{
   x4Assert(obj != NULL);
   result = (obj->*method)(params);
}

ClassMethod *BasicArgsClassMethod::copyMethod() const
{
   ClassMethod *cpy = new BasicArgsClassMethod(method, getName(), parameterTypes, (Variant::Type)resultType, getDescription(), getIcon());
   return cpy;
}

