//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ClassPropertyParameters.h"
#include "ClassPropertyParameters.inline.h"
//---------------------------------------------------------------------------
#include "EmbeddedObjectClassProperty.h"
#include "EmbeddedObjectClassProperty.inline.h"
#include "ObjectClassProperty.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


static std::vector<std::string> parsePath(std::string path) {
   std::vector<std::string> result;
   std::string *current = NULL;
   const card32 n = (card32)path.size();
   for (card32 i=0; i<n; i++) {
      if (path[i] == '.') {
         current = NULL;
      } else {
         if (current == NULL) {
            result.push_back(std::string());
            current = &(result[result.size()-1]);
         }
         (*current).push_back(path[i]);
      }
   }
   return result;
}

bool ClassPropertyParameters::getAccessData(card32 index, const Object *root, const ClassProperty* &cp, Object* &target) const {
   x4Assert(index < parameters.size());
   std::vector<std::string> path = parsePath(parameters[index]);
   const card32 numPath = (card32)path.size();

   if (numPath == 0) return false;

   Object *currentObject = const_cast<Object*>(root);
   for (card32 iPath=0; iPath<numPath; iPath++) {

      const MetaClass *mc = currentObject->getInstanceClass();
      const card32 numProps = mc->getNumProperties();

      for (card32 iProp = 0; iProp<numProps; iProp++) {
         const ClassProperty *currentCP = mc->getProperty(iProp);

         if (currentCP->getCName() == path[iPath]) {

            if (iPath == numPath-1) {
               target = currentObject;
               cp = currentCP;
               return true;
            } else {
               const ObjectClassProperty *objProp = dynamic_cast<const ObjectClassProperty*>(currentCP);
               if (objProp != NULL) {
                  currentObject = const_cast<ObjectClassProperty*>(objProp)->getValue(currentObject,NULL);
               } else {
                  const EmbeddedObjectClassProperty *embProp = dynamic_cast<const EmbeddedObjectClassProperty*>(currentCP);
                  if (embProp != NULL) {
                     currentObject = const_cast<EmbeddedObjectClassProperty*>(embProp)->getSubObjectAdress(currentObject,NULL);
                  } else {
                     return false;
                  }
               }
            }

         }

      }

   }
   return false;
}


#if 0
#include "foundation/math/linearAlgebra/DynamicLinearAlgebra.inline.h"

CPPFloatParameterAccess::CPPFloatParameterAccess(const ClassPropertyParameters &parameters, Object *target) {
   const card32 n = parameters.getNumParameters();
   for (card32 i=0; i<n; i++) {
      Access ac;
      if (parameters.getAccessData(i, target, ac.prop, ac.target)) {
         access.push_back(ac);
      }
   }
}
#endif 

