//---------------------------------------------------------------------------
#ifndef ClassPropertyParametersH
#define ClassPropertyParametersH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "X4Assert.h"
//#include "foundation/math/linearAlgebra/DynamicLinearAlgebra.h"
//---------------------------------------------------------------------------


namespace X4 {

class ClassProperty;


/** This class specifies a set of class properties by std::string. An access path via class property and target
    object can be established by specifying a root object object to start searching for the given properties.

    The properties are specified by their "cName" (getCName(), see ClassProperty.h).
    Hierarchical Property access is provided by using the dot (".") operator.
    Access is provided by searching for ObjectClassProperties and EmbeddedObjectClassProperties.


    Example, for a PointCloudScene object:

    renderState.cam.up - std::vector of 3 floats           (target of type:  Camera*,       ClassProperty of type:  ArrayClassProperty<float32>)
    renderState.viewFrustum - complex object property (target of type:  RenderState*,  ClassProperty of type:  EmbeddedObjectClassProperty<ViewFrustum>)
    renderState.ctm - 4x4 transformation matrix       (target of type:  RenderState*,  ClassProperty of type:  ArrayClassProperty<float32>)

*/
class X4CORE_API ClassPropertyParameters : public Persistent {
 private:
   std::vector<std::string> parameters;

 public:
   ClassPropertyParameters() {}

   ClassPropertyParameters(const ClassPropertyParameters &c);
   const ClassPropertyParameters operator=(const ClassPropertyParameters &c);
   bool operator==(const ClassPropertyParameters &c);

   void addParameter(std::string name);
   card32 getNumParameters() const;
   void setNumParameters(card32 n);
   const std::string &operator[](card32 index) const;
         std::string &operator[](card32 index);
   bool getAccessData(card32 index, const Object *root, const ClassProperty* &cp, Object* &target) const;
};

#if 0
// TODO (art) : check where required and put this into maybe math-module
class X4CORE_API CPPFloatParameterAccess {
 private:
   struct Access {
      const ClassProperty *prop;
      Object *target;
   };
   std::vector<Access> access;

 public:
   CPPFloatParameterAccess(const ClassPropertyParameters &parameters, Object *target);

   template <typename FloatType>
   const DynamicVector<FloatType> getParameterVector();

   template <typename FloatType>
   void setParameterVector(const DynamicVector<FloatType> &params);
};
#endif 


}


#endif
