//---------------------------------------------------------------------------
#ifndef FixedArrayClassPropertyH
#define FixedArrayClassPropertyH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "CompoundClassProperty.h"
#include "NumericalClassProperty.h"
//---------------------------------------------------------------------------

namespace X4
{

class X4CORE_API FixedArrayClassProperty : public CompoundClassProperty
{
    X4_CLASS(FixedArrayClassProperty);

private:

	pointer_diff calculateElementDiff( unsigned i ) const;
	
	ClassProperty	*	m_ElementProperty;
	pointer_diff		m_Offset;
	card32				m_EntriesPerRow;
	card32				m_ArraySize;
	card32				m_Stride;

protected:
	FixedArrayClassProperty() {}

public:
	/// contructs an array property, becomes owner of prop
	FixedArrayClassProperty(const std::string cName, const Version introducedInVersion,
			pointer_diff offset, ClassProperty * prop, card32 arraySize, card32 stride, card32 entriesPerRow = 1);
	~FixedArrayClassProperty(void);

	pointer_diff getOffset() const {return m_Offset;}
	card32 getEntriesPerRow() const {return m_EntriesPerRow;}
	card32 getArraySize() const {return m_ArraySize;}
	card32 getStride() const {return m_Stride;}
	ClassProperty * getElementProperty() const {return m_ElementProperty;}
	
    virtual const std::string getTypeName() const;
    virtual void writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const;
    virtual void readProperty(Object* obj, pointer_diff offset, InputObjectStream *in, card64 expectedSize) const;
    virtual std::string toString(const Object* obj, pointer_diff offset) const;
    virtual void fromString(Object* obj, pointer_diff offset, const std::string str) const;
	virtual void assignProperty(const Object *from, Object *to, pointer_diff offset) const;

    virtual ClassProperty* copyClassProperty();

};

#define ADD_FLOAT32_FIXED_ARRAY_PROP( propName, version, count, entriesPerRow ) \
	((MetaClass*)getClass())->addProperty( new FixedArrayClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
	(pointer_diff)((pointer_diff)&( obj_null_pointer->##propName ) - obj_null_offset) , new Float32ClassProperty("", version , NULL) \
	,count, sizeof(float32), entriesPerRow ));

#define ADD_FLOAT64_FIXED_ARRAY_PROP( propName, version, count, entriesPerRow ) \
	((MetaClass*)getClass())->addProperty( new FixedArrayClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
	(pointer_diff)((pointer_diff)&( obj_null_pointer->##propName ) - obj_null_offset) , new Float64ClassProperty("", version , NULL) \
	,count, sizeof(float64), entriesPerRow ));

#define ADD_INT32_FIXED_ARRAY_PROP( propName, version, count, entriesPerRow ) \
	((MetaClass*)getClass())->addProperty( new FixedArrayClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
	(pointer_diff)((pointer_diff)&( obj_null_pointer->##propName ) - obj_null_offset) , new Int32ClassProperty("", version , NULL) \
	,count, sizeof(int32), entriesPerRow ));

#define ADD_CARD32_FIXED_ARRAY_PROP( propName, version, count, entriesPerRow ) \
	((MetaClass*)getClass())->addProperty( new FixedArrayClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
	(pointer_diff)((pointer_diff)&( obj_null_pointer->##propName ) - obj_null_offset) , new Card32ClassProperty("", version , NULL) \
	,count, sizeof(card32), entriesPerRow ));

#define ADD_CARD8_FIXED_ARRAY_PROP( propName, version, count, entriesPerRow ) \
	((MetaClass*)getClass())->addProperty( new FixedArrayClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
	(pointer_diff)((pointer_diff)&( obj_null_pointer->##propName ) - obj_null_offset) , new Card8ClassProperty("", version , NULL) \
	,count, sizeof(card8), entriesPerRow ));

#define ADD_BOOLEAN_FIXED_ARRAY_PROP( propName, version, count, entriesPerRow ) \
	((MetaClass*)getClass())->addProperty( new FixedArrayClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
	(pointer_diff)((pointer_diff)&( obj_null_pointer->##propName ) - obj_null_offset) , new BooleanClassProperty("", version , NULL) \
	,count, sizeof(bool), entriesPerRow )); 

#define ADD_OBJECT_FIXED_ARRAY_PROP( propName, version, count, baseClass, is_owner ) \
	((MetaClass*)getClass())->addProperty( new FixedArrayClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
	(pointer_diff)((pointer_diff)&( obj_null_pointer->##propName ) - obj_null_offset) , new ObjectClassProperty("", version , NULL, \
	baseClass , is_owner ) ,count, sizeof(card32), 1 ));
}

#endif