//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MemoryManager.h"
//---------------------------------------------------------------------------
#include <X4Assert.h>
#include "AttachedData.h"
#include <ConsoleOutput.h>
//---------------------------------------------------------------------------

namespace X4
{
	QMutex MemoryManager::m_mutex;

#if 0
	std::pair<size_t,float> MemoryManager::m_minMemoryGrowingParam(0,2.0f);
	std::pair<size_t,float> MemoryManager::m_maxMemoryGrowingParam(1,2.0f);
	std::pair<double,double>  MemoryManager::m_MemoryGrowingParam(0,2.0f);

	//---------------------------------------------------------------------------
	bool MemoryManager::ds_setupFactors(size_t minMem, float minFactor, size_t maxMem, float maxFactor)
	{
		if (minMem > maxMem || minMem == maxMem) return false;
		if (minFactor < 1 || maxFactor < 1) return false;

		m_minMemoryGrowingParam.first = minMem;
		m_minMemoryGrowingParam.second = minFactor;

		m_maxMemoryGrowingParam.first = maxMem;
		m_maxMemoryGrowingParam.second = maxFactor;

		m_MemoryGrowingParam.first  = (double)(minFactor - maxFactor) / ((double)minMem - (double)maxMem);
		m_MemoryGrowingParam.second = ((double)maxMem * (double)minFactor - (double)minMem * (double)maxFactor) / (double)(maxMem - minMem);

		debugOutput << "a=" << m_MemoryGrowingParam.first << ", b=" << m_MemoryGrowingParam.second << "\n";
		debugOutput.flush();

		return true;
	}

#endif

	//---------------------------------------------------------------------------
	void MemoryManager::ds_free(byte*& arr)
	{
		m_mutex.lock();

		if (arr) ::free(arr);
		arr = NULL;

		m_mutex.unlock();
	}

	//---------------------------------------------------------------------------
	byte* MemoryManager::ds_alloc(size_t numBytes)
	{
		m_mutex.lock();
		byte* data = (byte*)malloc(numBytes);
		m_mutex.unlock();

		return data;
	}

	//---------------------------------------------------------------------------
	byte* MemoryManager::ds_realloc(byte* arr, size_t elemSize, size_t usedElements, size_t& allocatedElements, size_t neededElements)
	{
		if (neededElements == 0)
		{
			ds_free(arr);
			return NULL;
		}

		m_mutex.lock();

		size_t allocMem = allocatedElements * elemSize;
		size_t neededMem = neededElements * elemSize;

		byte* dd = NULL;

#if 0
		// factor to be used for resizing allocated elements
		double factor = (long double)neededMem * (long double)m_MemoryGrowingParam.first + m_MemoryGrowingParam.second;

		// possible amount of memory if the factor would be applied to the allocated memory
		size_t hintAllocMemMax = (long double)allocMem * (long double)factor;
		size_t hintAllocMemMin = (long double)allocMem / (long double)factor;

		// need less then the factor of memory, then just reallocate
		if ((neededMem >= hintAllocMemMax)
	     || (neededMem < hintAllocMemMin))
		{
			dd = (byte*)realloc(arr, neededMem);
			allocatedElements = neededElements;		

		// the amount of memory is just slightly larger then we have allocated
		}else if (neededMem > allocMem)
		{					
			size_t newMem = (size_t((long double)neededMem * (long double)factor) / elemSize + 1) * elemSize; // round to proper height 

			dd = (byte*)realloc(arr, newMem);
			allocatedElements = newMem / elemSize;		

			//debugOutput << "need=" << neededMem << ", alloc=" << allocMem << "[" << allocatedElements << "], new=" << newMem << ", factor=" << factor << "\n";
			//debugOutput.flush();

		// in all other cases do nothing
		}else
			dd = arr;
#endif

#if 1
		// if the amount of required memory is much larger then we have preallocated
		// or if the demanded memory size is way to small
		// then reallocate the memory to the exact number of bytes
		if ((neededMem >= (allocMem << 1))
	     || (neededMem <= (allocMem >> 1)))
		{
			dd = (byte*)realloc(arr, neededMem);
			allocatedElements = neededElements;		

		// if the amount of demanded memory is just slightly larger, then
		// allocate the memory plus some additional space
		}else if (neededMem > allocMem)
		{			
			size_t newMem = (allocMem << 1); // double as big

			// if the amount of new empty memory is to large, then we just resize to the demanded size and add
			// some fixed amount of bytes at the end. This additional memory fits at least one element
			// We do this, in order to reduce memory reallocation on consecutive calls.
			//if (newMem - neededMem > PAD_SPACE)
			//	newMem = ((neededMem + PAD_SPACE) / elemSize + 1) * elemSize; // round to proper height (integer div!)

			dd = (byte*)realloc(arr, newMem);
			allocatedElements = newMem / elemSize;		
			neededMem = newMem;

		// if no memory reallocation is required
		}else
			dd = arr;

#endif

		m_mutex.unlock();

		if (dd == NULL)
			return NULL;

		size_t usedMem  = usedElements * elemSize;
		if (neededMem > usedMem)
			memset(dd + usedMem, 0, neededMem - usedMem);

		return dd;
	}


	MemoryManager::MemoryManager( card32 maxMemory )
	{
	   m_UsedMemory = 0;
	   m_MaxMemory = maxMemory;
	}

	MemoryManager::~MemoryManager()
	{
	   while( !m_Objects.empty() )
		  delete  *m_Objects.begin();
	}

	void MemoryManager::setMaxMemory( card32 maxMemory )
	{
	   free( maxMemory );
	   m_MaxMemory = maxMemory;
	}

	void MemoryManager::free( card32 maxUsedMemory )
	{
	   while( m_UsedMemory > maxUsedMemory )
		  if( m_Objects.size() )
			 delete (*m_Objects.begin());
	}

	void MemoryManager::add( MemoryManagedObject * obj )
	{
	   x4Assert( obj->m_MyManager == NULL );

	   x4Assert( m_MaxMemory > obj->getSize() );

	   free( m_MaxMemory - obj->getSize() );

	   m_UsedMemory += obj->getSize();

	   m_Objects.push_back( obj );
	   obj->m_MyManager = this;
	   obj->m_MyIterator = m_Objects.end();
	   obj->m_MyIterator--;
	}

	void MemoryManager::remove( MemoryManagedObject * obj )
	{
	   m_UsedMemory -= obj->getSize();
	   m_Objects.erase( obj->m_MyIterator );
	}

	void MemoryManager::touched( MemoryManagedObject * obj )
	{
	   m_Objects.erase( obj->m_MyIterator );
	   m_Objects.push_back( obj );
	   obj->m_MyIterator = m_Objects.end();
	   obj->m_MyIterator--;
	}

	void MemoryManager::resized( MemoryManagedObject * obj, card32 oldSize, card32 newSize )
	{
	   x4Assert( m_UsedMemory >= oldSize );
	   m_UsedMemory -= oldSize;
	   m_UsedMemory += newSize;
	   touched( obj );
	}

	MemoryManagedObject* MemoryManager::createNewMMObject(const AttachedData* data){
	   MemoryManagedObject* mmo=new MemoryManagedObject(((AttachedData*)data)->getSize(), this);
	   return mmo;
	}
                                                         

	MemoryManagedObject::MemoryManagedObject( card32 size, MemoryManager * manager )
	{
	   m_MyManager = NULL;
	   m_MySize = size;
	   if( manager )
		  manager->add(this);
	   m_MyOwner = NULL;
	}

	MemoryManagedObject::~MemoryManagedObject()
	{
	   if( m_MyOwner )
		  m_MyOwner->onDeleteMMO(this);
	   if( m_MyManager )
		  m_MyManager->remove( this );
	}

	void MemoryManagedObject::touched()
	{
	   if( m_MyManager )
		  m_MyManager->touched( this );
	}

	void MemoryManagedObject::setSize(card32 size)
	{
	   card32 oldSize = m_MySize;
	   m_MySize = size;
	   if( m_MyManager )
		  m_MyManager->resized( this, oldSize, size );
	}

	// ----------------------------------------------

	/*MemoryManagedParameter::MemoryManagedParameter( card32 size, MemoryManager * manager, AttachedData * data )
	:MemoryManagedObject(size,manager)
	{
	   m_AttachedData = data;
	}

	MemoryManagedParameter::~MemoryManagedParameter()
	{

	}*/

}

