//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ObjectClassProperty.h"
//---------------------------------------------------------------------------
#include "Tools.h"
#include "ObjectStreams.h"
#include "MetaClass.h"
#include "Persistent.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_ABSTRACT_CLASS(ObjectClassProperty, 0)
{
    BEGIN_CLASS_INIT(ObjectClassProperty);
}

//---------------------------------------------------------------------------
ObjectClassProperty::ObjectClassProperty(const std::string CName, const Version introducedInVersion,
                                         ObjectGetPropertyMethod getMethod, ObjectSetPropertyMethod setMethod,
                                         const MetaClass *baseClass, bool isOwner)
                                         : SingleClassProperty(CName, introducedInVersion) 
{
    this->getMethod = getMethod;
    this->setMethod = setMethod;
    this->updateAfterWrite = NULL;
    this->baseClass = baseClass;
    this->owner = isOwner;
    useGetSetMethods = true;
    setCompoundAble( false );
}

//---------------------------------------------------------------------------
ObjectClassProperty::ObjectClassProperty(const std::string CName, const Version introducedInVersion,
                                         pointer_diff offset, const MetaClass *baseClass, bool isOwner,UpdateMethod updateAfterWrite)
                                         : SingleClassProperty(CName, introducedInVersion)
{
    this->getMethod = NULL;
    this->setMethod = NULL;
    this->updateAfterWrite = updateAfterWrite;
    this->baseClass = baseClass;
    this->owner = isOwner;
    setOffset( offset );
    useGetSetMethods = false;
}

//---------------------------------------------------------------------------
Object *ObjectClassProperty::getValue(const Object *obj, pointer_diff offset) const
{
    if (useGetSetMethods)
        return (obj->*getMethod)();
    else
        return *((Object **)((char *)obj + getOffset() + offset));
}

//---------------------------------------------------------------------------
void ObjectClassProperty::setValue(Object *obj, pointer_diff offset, Object *v) const
{
    if (useGetSetMethods)
        (obj->*setMethod)(v);
    else {
        *((Object **)((char *)obj + getOffset() + offset)) = v;
        if (updateAfterWrite != NULL)
            (obj->*updateAfterWrite)();
    }
}

//---------------------------------------------------------------------------
void ObjectClassProperty::writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const 
{
    out->writeObject((Persistent*)getValue(obj,offset) );
}

//---------------------------------------------------------------------------
void ObjectClassProperty::readProperty(Object* obj, pointer_diff offset, InputObjectStream *in, card64 expectedSize) const 
{
    Persistent* inobj;
    in->readObject(inobj);
    setValue( obj, offset, inobj );
}

#if 0
//---------------------------------------------------------------------------
ClassProperty* ObjectClassProperty::copyClassProperty() 
{
    ObjectClassProperty *result = new ObjectClassProperty();
    *result = *this;
    return result;
}
#endif 

//---------------------------------------------------------------------------
void ObjectClassProperty::assignProperty(const Object *from, Object *to, pointer_diff offset) const 
{
    if (owner && !useGetSetMethods ) 
    {
        delete getValue(to,offset);
        setValue(to, offset, NULL);
    }

    if (getObjectIsOwnerOfSubobject()) 
    {
        if (getValue(from, offset) != NULL) 
            setValue(to, offset, getValue(from, offset)->copy());
        else
            setValue(to, offset, NULL);
    } else
        setValue(to, offset, getValue(from ,offset));
}




//---------------------------------------------------------------------------
