//---------------------------------------------------------------------------
#ifndef ObjectClassPropertyH
#define ObjectClassPropertyH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "SingleClassProperty.h"
#include <Tools.h>
//---------------------------------------------------------------------------


namespace X4 {


    class X4CORE_API ObjectClassProperty : public SingleClassProperty {
        X4_CLASS(ObjectClassProperty);

    private:
        ObjectClassProperty() : SingleClassProperty("", 1) {}
        const MetaClass *baseClass;
        bool owner;

    public:
        typedef Object* (Object::* ObjectGetPropertyMethod)() const;
        typedef void (Object::* ObjectSetPropertyMethod)(Object*);
        typedef void (Object::* UpdateMethod)();

    protected:
        ObjectGetPropertyMethod getMethod;
        ObjectSetPropertyMethod setMethod;
        UpdateMethod updateAfterWrite;
        bool useGetSetMethods;

    public:
        ObjectClassProperty(const std::string CName, const Version introducedInVersion,
            ObjectGetPropertyMethod getMethod, ObjectSetPropertyMethod setMethod,
            const MetaClass *baseClass, bool isOwner = false);

        ObjectClassProperty(const std::string CName, const Version introducedInVersion,
            pointer_diff offset, const MetaClass *baseClass, bool isOwner = false,
            UpdateMethod updateAfterWrite = NULL);

        Object* getValue(const Object *obj, pointer_diff offset) const;
        void    setValue(Object *obj, pointer_diff offset, Object *v) const;

        inline const MetaClass *getBaseClass() const { return baseClass; }
        inline void setBaseClass(const MetaClass *baseClass) { this->baseClass = baseClass; }
        inline bool getObjectIsOwnerOfSubobject() const { return owner; }
        inline void setObjectIsOwnerOfSubobject(bool isOwner) { owner = isOwner; }

        inline bool getUseGetSetMethods() const {return useGetSetMethods;}

        virtual void writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const;
        virtual void readProperty(Object* obj, pointer_diff offset, InputObjectStream *in, card64 expectedSize) const;
        virtual std::string toString(const Object* obj, pointer_diff offset) const {return makeObjectDescription( getValue( obj,offset ) );}
        virtual void fromString(Object* obj, pointer_diff offset, const std::string str) const {forget(obj); forget(offset); forget(str);}
        virtual const std::string getTypeName() const { return baseClass->getClassName() + std::string("*"); }

        void assignProperty(const Object *from, Object *to, pointer_diff offset) const;

        inline ClassProperty* copyClassProperty() { return new ObjectClassProperty(*this); }
    };


    typedef ObjectClassProperty::ObjectSetPropertyMethod SetObjectPropMethod;
    typedef ObjectClassProperty::ObjectGetPropertyMethod GetObjectPropMethod;
    typedef ObjectClassProperty::UpdateMethod ObjectUpdateMethod;

#define ADD_OBJECT_PROP_UPDATE( propName, version, baseClass, is_owner, updateMethodPtr ) \
    ((MetaClass*)getClass())->addProperty( new ObjectClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
    (pointer_diff)((pointer_diff)&( obj_null_pointer->##propName ) - obj_null_offset), baseClass, is_owner, (ObjectUpdateMethod)updateMethodPtr) );

#define ADD_OBJECT_PROP( propName, version, baseClass, is_owner ) \
    ADD_OBJECT_PROP_UPDATE( propName, version, baseClass, is_owner, NULL )

#define ADD_OBJECT_PROP_METHOD( propName, version, get, set, baseClass, is_owner ) \
    ((MetaClass*)getClass())->addProperty( new ObjectClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
    (GetObjectPropMethod)get, (SetObjectPropMethod)set, baseClass, is_owner) );

}


#endif
