//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------
#include "ClassProperty.h"
#include "ObjectStreams.h"
#include "PropertyTable.h"
#include "UnknownObject.h"
#include <Tools.h>
#include <StringHelper.h>
#include <ConsoleOutput.h>
//---------------------------------------------------------------------------
#undef DEBUG_STREAMS
//#define DEBUG_STREAMS
//---------------------------------------------------------------------------
using namespace X4;
// --------------------------------------------------------------------------



IMPLEMENT_X4_CLASS_DESCR(Persistent, 0, "Base Class for all serializable classes.")
{
	BEGIN_CLASS_INIT(Persistent);
}


void Persistent::write(OutputObjectStream *s) const
{
	s->includeClass(getInstanceClass());
	writeObjectProperties(s);
}

void Persistent::read(InputObjectStream *s)
{
	readObjectProperties(s);
}

void Persistent::writeObjectProperties(OutputObjectStream *s) const
{
	const MetaClass *mc = getInstanceClass();
	#ifdef DEBUG_STREAMS
	x4console << "**** " << mc->getClassName(); x4console.cr();
	#endif
	card32 n = mc->getNumProperties();

	// write all properties as they are
	for (card32 i=0; i<n; ++i)
	{
		const ClassProperty *cp = mc->getProperty(i);
		if (cp->getStreamProperty())
		{
			#ifdef DEBUG_STREAMS
			x4console << cp->getTypeName() << " : " << cp->getCName(); x4console.cr();
			#endif
			cp->write(this, NULL, s);
		}
	}
}

void Persistent::readObjectProperties(InputObjectStream *s)
{
	const MetaClass *mc = getInstanceClass();
	#ifdef DEBUG_STREAMS
	x4console << "**** " << mc->getClassName(); x4console.cr();
	#endif

	// get property table associated with this class
	// mw: unused // const PropertyTable* ownTable = mc->getPropertyTable();
	const PropertyTable* streamTable = s->getPropertyTable(mc);

   if (streamTable == NULL) {
      throw EIOException(std::string("error reading object property: the class ")+mc->getClassName()+std::string(" has no stream table. Typically, this means that it has not been registered (please check the tags .file.h of the corresponding module.)"));
   }
	

	// based on the property table found in the stream, read it
	for (unsigned i=0; i < streamTable->getTableSize(); i++)
	{
		// TODO (daniel): This is just debugging code
		if (streamTable != s->getPropertyTable(mc))
		{
			x4Assert(false && "Memory has been corrupted!");
		}

		const PropertyTable::TableEntry& entry = streamTable->getTableEntry(i);
		const ClassProperty* cp = mc->getProperty(entry.propertyName);

		// ok property is in the stream but not in the current class definition, hence jump over it
		if (cp == NULL)
		{
			consoleWarning(std::string("Cannot find property ") + entry.propertyName + std::string(" of type ")
					+ entry.typeName + std::string(" introduced in the class ") + mc->getClassName() + std::string(":") + intToStr(entry.introducedInVersion)
					+ std::string(" (current version ") + intToStr(mc->getClassVersion())
					+ std::string("). The property will be ignored."));

			// we need to create a temporary property to hold the data
			getOrCreateUnknownPropertyContainer()->readAndAddProperty(this, entry, s);

			continue;
		}

		// there is right property in the stream, however it is of wrong type, hence jump over it
		if (cp->getTypeName() != entry.typeName)
		{
			consoleWarning(std::string("Property ") + entry.propertyName + std::string(" of type ") + entry.typeName
					+ std::string(" from class ") + mc->getClassName()
					+ std::string(" found in the stream is incompatible to the actual type. Property is ignored."));

			getOrCreateUnknownPropertyContainer()->readAndAddProperty(this, entry, s);
			continue;
		}

		// if property is readable, then read it
		if (cp->getStreamProperty())
		{
			//bool readProperty = (cp->getIntroducedInVersion() <= s->getClassVersion(cp->getOwner()));
			//if (readProperty)
			{
				#ifdef DEBUG_STREAMS
				x4console << cp->getTypeName() << " : " << cp->getCName(); x4console.cr();
				#endif
				cp->read(this, NULL, s);
			}
		}
	}

}





