//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PersistentSharedPtr.h"
//---------------------------------------------------------------------------
#include <ObjectClassProperty.h>
#include <CopyObjectProperties.h>
#include <EmbeddedObjectClassProperty.h>
#include <Tools.h>
#include <BinaryObjectStreams.h>
#include <Persistent.h>
//---------------------------------------------------------------------------

namespace X4 {

	//X4::PtrHashTable _ptrDebugHashTable;

	//---------------------------------------------------------------------------
	//BEGIN_TEMPLATE_CLASS(PersistentSharedPtr, PersistentSharedPtr_Base<Persistent>)
	//	ADD_OBJECT_PROP(_rawPtr, 0, Persistent::getClass(), true);
	//END_CLASS

	// -----------------------------------------------------------------------------------------------
	IMPLEMENT_X4_CLASS(PersistentSharedPtr, 0)
	{
		BEGIN_CLASS_INIT(PersistentSharedPtr);
		ADD_OBJECT_PROP(_rawPtr, 0, Persistent::getClass(), true);
	}


	// -----------------------------------------------------------------------------------------------
	//override from X4::Object
	void 
	PersistentSharedPtr::assign(const Object* obj, X4CopyContext *context) 
	{
		x4Assert(obj);
		if (obj->getInstanceClass() != this->getInstanceClass()) {
			throw ECannotAssign("Object::assign does not implement assignment between different types.");
		} else {
			*this = dynamic_cast<const PersistentSharedPtr&>(*obj);
		}
		forget(context);
	}


	// -----------------------------------------------------------------------------------------------
	IMPLEMENT_X4_CLASS(PersistentSharedArray, 0)
	{
		BEGIN_CLASS_INIT(PersistentSharedArray);
	}

	void 
	PersistentSharedArray::assign(const Object* obj, X4CopyContext *context) //override X4::Object
	{
		x4Assert(obj);
		if (obj->getInstanceClass() != this->getInstanceClass()) {
			throw ECannotAssign("Object::assign does not implement assignment between different types.");
		} else {
			*this = dynamic_cast<const PersistentSharedArray&>(*obj);
		}
		forget(context);
	}

	void 
	PersistentSharedArray::write(OutputObjectStream *s) const
	{
		Persistent::write(s);
		s->write(_arraySize);
		s->writeBuffer(_rawPtr, _arraySize);
	}
	
	void
	PersistentSharedArray::read(InputObjectStream *s)
	{
		Persistent::read(s);
		s->read(_arraySize);
		_sharedArray.reset(new byte[_arraySize]);
		_rawPtr = _sharedArray.get();
		s->readBuffer(_rawPtr, _arraySize);
	}


} // namespace X4
