//---------------------------------------------------------------------------
#ifndef _PersistentSharedPtr_H
#define _PersistentSharedPtr_H
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "Object.h"
#include "MetaClass.h"
#include "X4GlobalDefinitions.h"
#include "X4Types.h"
#include "Persistent.h"
//---------------------------------------------------------------------------
#include <boost/shared_ptr.hpp>
#include <boost/shared_array.hpp>
//---------------------------------------------------------------------------


namespace X4 {

#if 0
		struct X4CORE_API PtrKeyComp {
		   static bool equals(intptr_t p1, intptr_t p2) {
			  return p1 == p2;
		   }
		};
		struct PtrHashData
		{
			card32 count;
		};

		class PtrHashTable : public HashTable<intptr_t, PtrHashData, PtrKeyComp> {
		 public:
		   virtual max_platform_card hashFunction(intptr_t key) const {
			  return (mpcard)key;
		   }
		   PtrHashTable(max_platform_card fillFactor = 1, max_platform_card startSizeIndex = 0) : HashTable<intptr_t, PtrHashData, PtrKeyComp>(fillFactor, startSizeIndex) {}
		};

		extern X4CORE_API PtrHashTable _ptrDebugHashTable;
#endif

		//------------------------------------------------------------------------------
		class X4CORE_API PersistentSharedPtr : public Persistent
		{
			X4_CLASS(PersistentSharedPtr)
		public:
			typedef boost::shared_ptr<Persistent> SharedPtr;
		protected:
			//! the raw pointer to the persistent object is the actual property
			//! being written/read to/from the stream.
			Persistent* _rawPtr;
			boost::shared_ptr<Persistent> _sharedPtr;
			
		public:
			PersistentSharedPtr() : _rawPtr(0), _sharedPtr()
			{}
			PersistentSharedPtr(Persistent* obj) : _rawPtr(obj), _sharedPtr(obj)
			{}
			virtual ~PersistentSharedPtr() {
			#ifdef _DEBUG
				if (_sharedPtr) {
					x4Assert(_sharedPtr.get() == _rawPtr);
				}
			#endif
			}
			// don't need a destructor as shared ptr destructs itself.
			//template<class T> T * getRawPtr() const { return dynamic_cast<T*>(_rawPtr); }
			template<class T> const T * getRawPtr() const { return dynamic_cast<const T*>(_sharedPtr.get()); }
			template<class T> T * getRawPtr() { return dynamic_cast<T*>(_sharedPtr.get()); }
			template<class T> boost::shared_ptr<T> get() { return boost::dynamic_pointer_cast<T>(_sharedPtr); }
			template<> boost::shared_ptr<Persistent> get() { return _sharedPtr; }
			template<class T> const boost::shared_ptr<T> get() const { return boost::dynamic_pointer_cast<T>(_sharedPtr); }
			template<> const boost::shared_ptr<Persistent> get() const { return _sharedPtr; }
			const boost::shared_ptr<Persistent> & operator= (const boost::shared_ptr<Persistent> & p) 
			{ _sharedPtr = p; _rawPtr = _sharedPtr.get(); x4Assert(_sharedPtr.get() == _rawPtr); return _sharedPtr; }
			PersistentSharedPtr & operator= (const PersistentSharedPtr & p) 
			{
				_sharedPtr = p._sharedPtr; _rawPtr = p._rawPtr; 
				x4Assert(_sharedPtr.get() == _rawPtr);
				return *this; 
			}
			operator bool() const { return _sharedPtr.get() != NULL; }
			void set(Persistent* p) { _sharedPtr.reset(p); _rawPtr = _sharedPtr.get(); }
			void reset(Persistent* p) { set(p); }
			operator Persistent*() { 
				return _sharedPtr.get(); 
			}
			operator const Persistent*() const { 
				return _sharedPtr.get(); 
			}
			boost::shared_ptr<Persistent> operator -> () { return _sharedPtr; }
			const boost::shared_ptr<Persistent> operator -> () const { return _sharedPtr; }

			//we need to override the method defined in X4::Object as we have pointers to copy in this class.
			virtual void assign(const Object* obj, X4CopyContext *context = NULL); //override X4::Object
			virtual void write(OutputObjectStream *s) const
			{
				Persistent::write(s);
			}
			virtual void read(InputObjectStream *s)
			{
				Persistent::read(s);
				_sharedPtr.reset(_rawPtr);// have to set the shared_ptr manually
			}
		}; // class PersistentSharedPtr

		//------------------------------------------------------------------------------
		template<class PersistentType>
		class PersistentSharedPtr_Tmpl : public PersistentSharedPtr
		{
			X_CLASS;
		public:
			typedef boost::shared_ptr<PersistentType> SharedPtr;

			inline operator PersistentType*() {  return getRawPtr();  }
			inline operator const PersistentType*() const { return getRawPtr(); }

			inline boost::shared_ptr<PersistentType> operator -> () { return _sharedPtr; }
			inline const boost::shared_ptr<PersistentType> operator -> () const { return _sharedPtr; }

			inline void operator= (const boost::shared_ptr<PersistentType> & p) 
			{ 
				_sharedPtr = p; 
				_rawPtr = _sharedPtr.get(); 
				x4Assert(_sharedPtr.get() == _rawPtr); 
				//return _sharedPtr;
			}
		};

		//------------------------------------------------------------------------------
		class X4CORE_API PersistentSharedArray : public Persistent
		{
			X4_CLASS(PersistentSharedArray)
		public:
			typedef boost::shared_array<byte> SharedPtr;
		private:
			//! the raw pointer to the persistent object is the actual property
			//! being written/read to/from the stream.
			card32 _arraySize;
			byte * _rawPtr;
			boost::shared_array<byte> _sharedArray;
		public:
			PersistentSharedArray() : _arraySize(0), _rawPtr(0), _sharedArray()
			{}
			PersistentSharedArray(byte * obj, const card32 size) : _arraySize(size), _rawPtr(obj), _sharedArray(obj)
			{}
			virtual ~PersistentSharedArray() {
			#ifdef _DEBUG
				if (_sharedArray)
					x4Assert(_sharedArray.get() == _rawPtr);
			#endif
			}
			// don't need a destructor as shared ptr destructs itself.
			template<class T> const T * getRawPtr() const { return dynamic_cast<const T*>(_sharedArray.get()); }
			//template<class T> boost::shared_array<T> get() { return reinterpret_cast<boost::shared_array<T> >(_sharedArray); }
			boost::shared_array<byte> get() { return _sharedArray; }
			const boost::shared_array<byte> get() const { return _sharedArray; }
			const boost::shared_array<byte> & operator= (const boost::shared_array<byte> & p) 
			{ _sharedArray = p; _rawPtr = _sharedArray.get(); x4Assert(_sharedArray.get() == _rawPtr); return _sharedArray; }
			PersistentSharedArray & operator= (const PersistentSharedArray & p) 
			{
				_sharedArray = p._sharedArray; _rawPtr = p._rawPtr; 
				x4Assert(_sharedArray.get() == _rawPtr);
				return *this; 
			}
			void set(byte * p, const card32 size) { _arraySize = size; _sharedArray.reset(p); _rawPtr = _sharedArray.get(); }
			void reset(byte * p, const card32 size) { set(p, size); }
			//we need to override the method defined in X4::Object as we have pointers to copy in this class.
			virtual void assign(const Object* obj, X4CopyContext *context = NULL); //override X4::Object
			virtual void write(OutputObjectStream *s) const;
			virtual void read(InputObjectStream *s);
		}; // class PersistentSharedArray

}//namespace X4

#endif // end of file
