//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PropertyTable.h"
//---------------------------------------------------------------------------
#include "OutputStream.h"
#include "InputStream.h"
//---------------------------------------------------------------------------
#include <StringHelper.h>
#include <X4Exception.h>
//---------------------------------------------------------------------------

namespace X4
{

	//---------------------------------------------------------------------------
	PropertyTable::PropertyTable()
	{
	}

	//---------------------------------------------------------------------------
	void PropertyTable::merge(const PropertyTable* tbl)
	{
		for (unsigned i=0; i < tbl->m_Table.size(); i++)
			m_Table.push_back(tbl->m_Table[i]);
	}

	//---------------------------------------------------------------------------
	void PropertyTable::add(card32 version, const std::string& type, const std::string& name)
	{
		TableEntry e;
		e.introducedInVersion = version;
		e.typeName = type;
		e.propertyName = name;

		m_Table.push_back(e);
	}

	//---------------------------------------------------------------------------
	void PropertyTable::write(OutputStream* out) const
	{
		x4Assert(m_Table.size() < 0xFFFFFFFF);

		// first we write our own version number
		out->write<card32>(tableVersion);
		out->write<card32>((card32)m_Table.size());

		// next write information over all properties
		for (unsigned i=0; i < m_Table.size(); i++)
		{
			out->write<MetaClass::ClassVersion>(m_Table[i].introducedInVersion);
			out->writeString(m_Table[i].propertyName);
			out->writeString(m_Table[i].typeName);
		}
	}

	//---------------------------------------------------------------------------
	void PropertyTable::read(InputStream* in)
	{
		// first table version of the property must be readed
		card32 tabver = 0;
		in->read<card32>(tabver);

		// for later changes we should be able to make corresponding reads for different table version
		if (tabver != tableVersion)
		{
			throw EIOException("Property table has an unsupported version number. This is a fatal error!");
			return;
		}

		// read all properties into an internal property table
		card32 numProperties = 0;
		in->read<card32>(numProperties);
		m_Table.resize(numProperties);
		for (unsigned i=0; i < numProperties; i++)
		{
			in->read<MetaClass::ClassVersion>(m_Table[i].introducedInVersion);
			in->readString(m_Table[i].propertyName);
			in->readString(m_Table[i].typeName);
		}

	}

};



