//---------------------------------------------------------------------------
#ifndef PropertyTable__H
#define PropertyTable__H
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "MetaClass.h"
//---------------------------------------------------------------------------

namespace X4
{
	class X4CORE_API OutputStream;
	class X4CORE_API InputStream;

	/**
	* Simple table holding information about used properties in a meta class
	**/
	class X4CORE_API PropertyTable
	{
	public:
		struct TableEntry
		{
			MetaClass::ClassVersion introducedInVersion;
			std::string typeName;
			std::string propertyName;
		};

		PropertyTable();
		PropertyTable(const PropertyTable& tbl) : m_Table(tbl.m_Table) {}

		void merge(const PropertyTable* tbl);
		void merge(const PropertyTable& tbl) { merge(&tbl); }

		void add(card32 version, const std::string& type, const std::string& name);

		size_t getTableSize() const { return m_Table.size(); }
		const TableEntry& getTableEntry(size_t i) const { return m_Table[i]; }

		void write(OutputStream* s) const;
		void read(InputStream* s);

	protected:
		// magic version number
		static const card32 tableVersion = ('p' << 24) | ('t' << 16) | 2;

		mutable std::vector<TableEntry> m_Table;
	};

}




#endif
