//---------------------------------------------------------------------------
#ifndef PropertyTablePropertyH
#define StringClassPropertyH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "SingleClassProperty.h"
#include "ObjectStreams.h"
//---------------------------------------------------------------------------

namespace X4 {


	class X4CORE_API PropertyTableProperty : public ClassProperty {
	public:
		struct TableEntry
		{
			Version introducedInVersion;
			std::string typeName;
			std::string propertyName;
		};

		PropertyTableProperty();

		virtual void writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const;
		virtual void readProperty(Object* obj, pointer_diff offset, InputObjectStream *in, card64 expectedSize) const;
		virtual std::string toString(const Object* obj, pointer_diff offset) const;
		virtual void fromString(Object* obj, pointer_diff offset, const std::string str) const;
		virtual const std::string getTypeName() const;
		virtual ClassProperty* copyClassProperty(); // comment out by CL @ 2014/02/17

		void assignProperty(const Object *from, Object *to, pointer_diff offset) const;
		static std::string getPropertyTablePropertyName() { return "+PropertyTable+"; }

		size_t getTableSize() const { return m_Table.size(); }
		const TableEntry& getTableEntry(unsigned i) const { return m_Table[i]; }

		/// usefull for converting old files to new format (ignore on read, but not on write)
		inline void setIgnoreTableOnRead(bool ignore) const { bIgnoreOnRead = ignore; }
		inline bool getIgnoreTableOnRead() const { return bIgnoreOnRead; }

	protected:
		// magic version number
		static const card32 tableVersion = ('p' << 24) | ('t' << 16) | 1;

		mutable std::vector<TableEntry> m_Table;
		mutable bool bIgnoreOnRead;
	};

#define INIT_PROPERTY_TABLE() \
	((MetaClass*)getClass())->addProperty( new PropertyTableProperty() );\
	SET_USER_EDIT(false);
#define SET_PROPERY_TABLE_IGNORE_ONREAD(ignoreOnRead)\
	dynamic_cast<const PropertyTableProperty*>(((MetaClass*)getClass())->getProperty(((MetaClass*)getClass())->getNumProperties()-1))->setIgnoreTableOnRead(ignoreOnRead);

}




#endif
