//---------------------------------------------------------------------------
#ifndef STLHelperH
#define STLHelperH
//---------------------------------------------------------------------------
#include "x4core.h"
#include "X4Types.h"
#include "ObjectStreams.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	some stl helper functions
*/
class X4CORE_API STLHelper 
{
public:

	/// fast but order distructive delete - deletes std::vector element i by swap/pop_back
	template<class T>
	static void vecFastElementDelete(std::vector<T> &vec, mpcard i);

	/// merge two vectors - appendVec will be added to vec 
	template<class T>
	static void mergeVectors( std::vector<T> &vec, const std::vector<T> &appendVec);

	/// deletes every std::vector entry and clears std::vector
	template<class T>
	static void deleteVectorEntries( std::vector<T*> &v );

	/// writes std::vector<T> via stream (T must not contain pointers!)
	template<class T>
	static void streamVector( OutputObjectStream *s, const std::vector<T> &v);

	/// reads std::vector<T> via stream (T must not contain pointers!)
	template<class T>
	static void streamVector( InputObjectStream *s, std::vector<T> &v);

	/// writes std::vector<T> via stream (T must not contain pointers!)
	template<class T>
	static void streamVector( std::ofstream &s, const std::vector<T> &v);

	/// reads std::vector<T> via stream (T must not contain pointers!)
	template<class T>
	static void streamVector( std::ifstream &s, std::vector<T> &v);


	struct TIndexScorePair
	{
		TIndexScorePair(mpint ind, float scr) {index = ind;score=scr;}
		TIndexScorePair() {}
		mpint index;
		float score;
		bool operator < (const TIndexScorePair &p ) const {return score < p.score;}
	};

	template<class T>
	static void killVector( std::vector<T> &v )
	{
		std::vector<T> tmp;
		v.swap(tmp);
	}


};


template<class T>
void X4::STLHelper::vecFastElementDelete(std::vector<T> &vec, mpcard i)
{
	vec[i] = vec.back();
	vec.pop_back();
}


template<class T>
void X4::STLHelper::mergeVectors( std::vector<T> &vec, const std::vector<T> &appendVec)
{
	vec.insert(vec.end(),appendVec.begin(),appendVec.end());
}

template<class T>
void X4::STLHelper::deleteVectorEntries( std::vector<T*> &v )
{
	for( std::vector<T*>::size_type i=0;i<v.size();i++ )
		delete v[i];
	v.clear();
}

template<class T>
static void X4::STLHelper::streamVector( OutputObjectStream *s, const std::vector<T> &v)
{
	mpcard numEntries = v.size();
	s->writeMaxPlatformCard(numEntries);
	if( numEntries > 0 )
		s->writeBuffer(&v[0],numEntries*sizeof(T));
}

template<class T>
static void X4::STLHelper::streamVector( InputObjectStream *s, std::vector<T> &v)
{
	mpcard numEntries;
	s->readMaxPlatformCard(numEntries);
	v = std::vector<T>(numEntries);
	if( numEntries > 0 )
		s->readBuffer(&v[0],numEntries*sizeof(T));
}

template<class T>
static void X4::STLHelper::streamVector( std::ofstream &s, const std::vector<T> &v)
{
	card64 numEntries = v.size();
	s.write( (char*)&numEntries,sizeof(card64));
	if( numEntries > 0 )
		s.write((char*)&v[0],numEntries*sizeof(T));
}

template<class T>
static void X4::STLHelper::streamVector( std::ifstream &s, std::vector<T> &v)
{
	card64 numEntries;
	s.read( (char*)&numEntries,sizeof(card64));
	v = std::vector<T>(numEntries);
	if( numEntries > 0 )
		s.read((char*)&v[0],numEntries*sizeof(T));
}


}

#endif