//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


IMPLEMENT_X4_CLASS(StringClassProperty, 0) { BEGIN_CLASS_INIT(StringClassProperty); }


StringClassProperty::StringClassProperty(const std::string CName, const Version introducedInVersion,
                       StringGetPropertyMethod getMethod, StringSetPropertyMethod setMethod)
: SingleClassProperty(CName, introducedInVersion) {
   this->getMethod = getMethod;
   this->setMethod = setMethod;
   useGetSetMethods = true;
	setCompoundAble( false );
	//addDefaultEditor(this);
}

StringClassProperty::StringClassProperty(const std::string CName, const Version introducedInVersion,
                                         pointer_diff offset)
: SingleClassProperty(CName, introducedInVersion) {
   this->getMethod = NULL;
   this->setMethod = NULL;
	setOffset( offset );
   useGetSetMethods = false;
	//addDefaultEditor(this);
}


std::string StringClassProperty::getValue(const Object *obj, pointer_diff offset) const {
   if (useGetSetMethods) 
	{
      return (obj->*getMethod)();
   } 
	else 
	{
      return *((std::string *)((char *)obj + getOffset() + offset));
   }
}

void   StringClassProperty::setValue(Object *obj, pointer_diff offset, std::string v) const {
   if (useGetSetMethods) 
	{
		if( setMethod != NULL )
			(obj->*setMethod)(v);
   } 
	else 
	{
      *((std::string *)((char *)obj + getOffset() + offset)) = v;
   }
}

void StringClassProperty::writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const {
	out->writeString( getValue( obj,offset ) );
}

void StringClassProperty::readProperty(Object* obj, pointer_diff offset, InputObjectStream *in, card64 expectedSize) const {
	std::string s;
	in->readString(s);
	setValue( obj,offset, s );
}


std::string StringClassProperty::toString(const Object* obj, pointer_diff offset) const {
	return getValue(obj,offset);
}

void StringClassProperty::fromString(Object* obj, pointer_diff offset, const std::string str) const {
	setValue( obj,offset, str );
}

const std::string StringClassProperty::getTypeName() const {
   return "string";
}

void StringClassProperty::assignProperty(const Object *from, Object *to, pointer_diff offset) const {
   setValue(to,offset, getValue(from,offset));
}


ClassProperty* StringClassProperty::copyClassProperty() {
   StringClassProperty *result = new StringClassProperty();
   *result = *this;
   return result;
}

