//---------------------------------------------------------------------------
#ifndef StringClassPropertyH
#define StringClassPropertyH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "SingleClassProperty.h"
#include "ObjectStreams.h"
//---------------------------------------------------------------------------

namespace X4 {


class X4CORE_API StringClassProperty : public SingleClassProperty {
	X4_CLASS(StringClassProperty);
private:
   StringClassProperty() {}

 public:
   typedef std::string (Object::* StringGetPropertyMethod)() const;
   typedef void (Object::* StringSetPropertyMethod)(std::string);
//   typedef std::string Object::* ElementPointer;

 protected:
   StringGetPropertyMethod getMethod;
   StringSetPropertyMethod setMethod;
//   ElementPointer elementPtr;
   bool useGetSetMethods;

 public:
   StringClassProperty(const std::string CName, const Version introducedInVersion,
                       StringGetPropertyMethod getMethod, StringSetPropertyMethod setMethod);
   StringClassProperty(const std::string CName, const Version introducedInVersion,
                       pointer_diff offset);

   std::string getValue(const Object *obj, pointer_diff offset) const;
   void   setValue(Object *obj, pointer_diff offset, std::string v) const;

   virtual void writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const;
   virtual void readProperty(Object* obj, pointer_diff offset, InputObjectStream *in, card64 expectedSize) const;
   virtual std::string toString(const Object* obj, pointer_diff offset) const;
   virtual void fromString(Object* obj, pointer_diff offset, const std::string str) const;
   virtual const std::string getTypeName() const;
   virtual ClassProperty* copyClassProperty();

   void assignProperty(const Object *from, Object *to, pointer_diff offset) const;
};

typedef StringClassProperty::StringSetPropertyMethod SetStringPropMethod;
typedef StringClassProperty::StringGetPropertyMethod GetStringPropMethod;


#define ADD_STRING_PROP( propName, version ) \
	((MetaClass*)getClass())->addProperty( new StringClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
	(pointer_diff)((pointer_diff)&( obj_null_pointer->##propName ) - obj_null_offset) ) );

#define ADD_STRING_PROP_METHOD( propName, version, get, set ) \
	((MetaClass*)getClass())->addProperty( new StringClassProperty( #propName , MetaClass::FIRST_VERSION + version, \
	(GetStringPropMethod) get, (SetStringPropMethod) set ) );
}




#endif
