//---------------------------------------------------------------------------
#ifndef TimerH
#define TimerH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "X4Types.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------


namespace X4 {

class X4CORE_API Timer {
 private:
   card32 lastTime;

 public:
   Timer();
   card32 getCurrentValue();
   card32 getDeltaValue(card32 lastValue);
   card32 getDeltaValue();
   ~Timer();
};

#define PEAK_TIMER_START( id ) \
	Timer timer_##id ; 

#define PEAK_TIMER_END( id ) \
	static card32 peaktime_##id = 0; card32 tmptime_##id = timer_##id .getDeltaValue(); \
	if( peaktime_##id < tmptime_##id ) \
{ x4console << "peak time[ " << #id << "]" << convertTimeToString( tmptime_##id ) << "\n"; peaktime_##id = tmptime_##id; x4console.flush(); }


#define X4_TIMER_START( id ) \
	Timer X4_timer_##id ; \
	X4_timer_##id##.getDeltaValue();

#define X4_TIMER_END( id ) \
	x4console << "Timings[" << #id << "]: " << convertTimeToString(  X4_timer_##id .getDeltaValue() ) << "\n";

#define X4_TIMER_STOP( id ) X4_TIMER_END( id )

#define X4_AVERAGE_TIMER_DEFINE(id) \
	static unsigned X4_average_timer_variable_##id = 0; \
	static unsigned X4_average_timer_numMeasurements_##id = 0; \
	static unsigned X4_average_timer_begin_##id; \
	static Timer X4_average_timer_##id ; 

#define X4_AVERAGE_TIMER_START(id) \
	X4_average_timer_begin_##id = X4_average_timer_##id##.getCurrentValue();
	
#define X4_AVERAGE_TIMER_STOP(id) \
	X4_average_timer_variable_##id += (X4_average_timer_##id##.getCurrentValue() - X4_average_timer_begin_##id); \
	X4_average_timer_numMeasurements_##id ++;

#define X4_AVERAGE_TIMER_REPORT_AND_RESET(id) \
	if( X4_average_timer_numMeasurements_##id > 0 ) \
	x4console << "Timer[ " << #id << "] - Report " << convertTimeToString( X4_average_timer_variable_##id / X4_average_timer_numMeasurements_##id ) << \
	" total time:" << convertTimeToString( X4_average_timer_variable_##id ) << "\n"; \
	else \
	x4console << "Timer[ " << #id << "] - No measurements were done!\n"; \
	X4_average_timer_variable_##id = 0; \
	X4_average_timer_numMeasurements_##id = 0; 

#define STATUSREPORT_DEFINE( statusName ) \
	Timer StatusReportTimer_##statusName ; \
	unsigned StatusReportRT_##statusName = StatusReportTimer_##statusName . getCurrentValue();

#define STATUSREPORT_REPORT( statusName, reportInterval, reportMessage ) \
{	unsigned currentTime = StatusReportTimer_##statusName . getCurrentValue();	\
	if( currentTime - StatusReportRT_##statusName > (reportInterval) ) \
{	x4console << "StatusReport[" << #statusName << "] " << (reportMessage) << "\n"; \
	x4console.flush(); 	StatusReportRT_##statusName = currentTime; \
} \
}

}

#endif
 