//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "UnknownObject.h"
//---------------------------------------------------------------------------
#include "ObjectClassProperty.h"
#include "ObjectStreams.h"
#include "NumericalClassProperty.h"
#include "CopyObjectProperties.h"
#include "StringClassProperty.h"
#include "UnknownProperty.h"
//---------------------------------------------------------------------------

namespace X4
{
	std::vector<const MetaClass*> UnknownObject::unknownClasses;

	//---------------------------------------------------------------------------
	IMPLEMENT_X4_CLASS(UnknownObject, 0)
	{
		BEGIN_CLASS_INIT(UnknownObject);

		ADD_CARD32_PROP(classID, 0);
		SET_USER_EDIT(false);
		ADD_STRING_PROP(unknownClassName, 0);
		ADD_CARD64_PROP(size, 0);
		SET_USER_EDIT(false);
		ADD_CARD8_VAR_ARRAY_PROP(rawData, 0);
	}

	//---------------------------------------------------------------------------
	void UnknownObject::shutdownUnknownClasses()
	{
		for (unsigned i=0; i < unknownClasses.size(); i++)
			delete unknownClasses[i];
		unknownClasses.clear();
	}

	//---------------------------------------------------------------------------
	void UnknownObject::readFromRaw(InputObjectStream *s, card32 id, size_t size)
	{
		classID = id;
		rawData.resize(size);
		this->size = size;
		s->readBuffer(&rawData[0], size);
	}

	//---------------------------------------------------------------------------
	void UnknownObject::write(OutputObjectStream *s) const
	{
		x4Assert(rawData.size() == size);

		// just dump out the memory dump of the other object
		s->writeBuffer(&rawData[0], size);
	}

	//---------------------------------------------------------------------------
	void UnknownPropertyContainer::readAndAddProperty(Object* obj, const PropertyTable::TableEntry& entry, InputObjectStream* in)
	{
		// read memory dump of the property
		card64 size = 0;
		in->read<card64>(size);
		
		properties.push_back(Property());
		Property& prop = properties.back();

		prop.entry = entry;
		prop.rawData.resize(size);
		in->readBuffer(&(prop.rawData[0]), size);

		// add new property to the metaclass of this object, to make sure, that this property is known as UnknownProperty from now on :)
		const_cast<MetaClass*>(obj->getInstanceClass())->addProperty( new UnknownClassProperty(entry, properties.size()-1) );
	}

	//---------------------------------------------------------------------------
	void UnknownPropertyContainer::syncPropertyList(Object* obj)
	{
		const MetaClass* mc = obj->getInstanceClass();

		std::vector<const UnknownClassProperty*> cps;
		for (unsigned i=0; i < mc->getNumProperties(); i++)
		{
			const UnknownClassProperty* ucp = dynamic_cast<const UnknownClassProperty*>(mc->getProperty(i));
			if (ucp != NULL) cps.push_back(ucp);
		}

		UnknownPropertyContainer* pr = const_cast<UnknownPropertyContainer*>(obj->getOrCreateUnknownPropertyContainer());

		pr->properties.resize(cps.size());
		for (unsigned i=0; i < cps.size(); i++)
			pr->properties[i].entry = cps[i]->getEntry();
	}

} // end namespace
