//---------------------------------------------------------------------------
#ifndef UnknownObject__H
#define UnknownObject__H
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include <X4Types.h>
#include "VariableArrayClassProperty.h"
#include "PropertyTable.h"
//---------------------------------------------------------------------------

namespace X4
{

	/**
	* Unknown object represents an object found in the stream for which no classes
	* exists in the currently compiled version. The unknown object does contain
	* enough information to be reconstructed on the fly, if required
	**/
	class X4CORE_API UnknownObject : public Persistent
	{
		X4_CLASS(UnknownObject)
	protected:
		std::vector<byte> rawData;
		card32 classID;
		card64 size;
		std::string unknownClassName;
		IMPLEMENT_VAR_ARRAY_METHODS_STL(rawData);

		void write(OutputObjectStream *s) const;

	public:
		UnknownObject() : size(0) {}
		~UnknownObject() {}

		const std::vector<byte>& getRawData() const { return rawData; }
		const card32 getClassID() const { return classID; }
		const card64 getSize() const { return size; }
		const std::string& getUnknownClassName() const { return unknownClassName; }

		void setUnknownClassName(const std::string& name) { unknownClassName = name; }

		//! get class which is unknown to the application, however was assigned (e.g. during reading from the stream) to this object
		const MetaClass* getUnknownClass() const { return MetaClass::getClassByName(unknownClassName); }

		void readFromRaw(InputObjectStream *s, card32 classID, size_t size);


		// --- Handling of unknown classes --
		static void shutdownUnknownClasses();
		static std::vector<const MetaClass*> unknownClasses;
	};

	/**
	* A special proprety object which acts as a container for all unknown properties (i.e. memory dumps)
	* of every object. Every object should have a pointer to such a class where it will store all unknown properties.
	**/
	class X4CORE_API UnknownPropertyContainer
	{
	public:
		struct Property
		{
			std::vector<byte> rawData;
			PropertyTable::TableEntry entry;
		};

		UnknownPropertyContainer() {}
		~UnknownPropertyContainer() {}

		// read unknown property from the stream, this assumes that the next to follow in the stream is the size of the property
		void readAndAddProperty(Object* obj, const PropertyTable::TableEntry& entry, InputObjectStream* in);

		Property& getProperty(size_t i) { return properties[i]; }
		const Property& getProperty(size_t i) const { return properties[i]; }

		size_t getNumProperties() const { return properties.size(); }

		// prepare container to match with the list of unknown properties of the metaclass of the given object
		static void syncPropertyList(Object* obj);

	private:
		std::vector<Property> properties;

	};

}

#endif
