//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "UnknownProperty.h"
//---------------------------------------------------------------------------
#include "Tools.h"
#include "ObjectStreams.h"
#include "MetaClass.h"
#include "Persistent.h"
#include "UnknownObject.h"
#include <ConsoleOutput.h>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


IMPLEMENT_X4_ABSTRACT_CLASS(UnknownClassProperty, 0)
{
	BEGIN_CLASS_INIT(UnknownClassProperty);
}

//---------------------------------------------------------------------------
UnknownClassProperty::UnknownClassProperty(const PropertyTable::TableEntry& entry, size_t offset)
	: SingleClassProperty(entry.propertyName, entry.introducedInVersion), 
	  propEntry(entry)
{
	setOffset(offset);

	x4Assert(offset >= 0);
}

//---------------------------------------------------------------------------
void UnknownClassProperty::writeProperty(const Object* obj, pointer_diff, OutputObjectStream *out) const
{
	// just dump out the content of the unknown property from the object
	const UnknownPropertyContainer::Property& prop = obj->getOrCreateUnknownPropertyContainer()->getProperty(getOffset());
	out->writeBuffer(&(prop.rawData[0]), prop.rawData.size());
}

//---------------------------------------------------------------------------
void UnknownClassProperty::readProperty(Object* obj, pointer_diff, InputObjectStream *in, card64 expectedSize) const
{
	// add new unknown properties fields to the object, if such are not already known
	UnknownPropertyContainer* prop = obj->getOrCreateUnknownPropertyContainer();
	
	if ((size_t)getOffset() >= prop->getNumProperties())
		prop->syncPropertyList(obj);

	UnknownPropertyContainer::Property& p = prop->getProperty(getOffset());
	p.rawData.resize(expectedSize);
	in->readBuffer(&(p.rawData[0]), expectedSize);
}

//---------------------------------------------------------------------------
void UnknownClassProperty::assignProperty(const Object *from, Object *to, pointer_diff ) const
{
	// here we check if such property does really exists, then copy it to this property
	const SingleClassProperty* cp = dynamic_cast<const SingleClassProperty*>(to->getInstanceClass()->getProperty(propEntry.propertyName));
		
	if (cp == NULL || cp->getOffset() != getOffset())
		throw(std::string("UnknownClassProperty::assignProperty() - assigning of an unknown property to an object which doesn't even now anything about this unknown property. This means something is really screwed up! Object = ") + to->getInstanceClass()->getClassName());

	x4Assert(cp != NULL);

	to->getOrCreateUnknownPropertyContainer()->getProperty(getOffset()) = from->getOrCreateUnknownPropertyContainer()->getProperty(getOffset());
}

