//---------------------------------------------------------------------------
#ifndef UnknownClassPropertyH
#define UnknownClassPropertyH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "SingleClassProperty.h"
#include "ObjectStreams.h"
#include "PropertyTable.h"
//---------------------------------------------------------------------------

namespace X4 {


	/**
	* Property which represents an unknown property. 
	* An unknown property is added during the runtime to be able to deal
	* with properties found in streams which cannot be handled by the application.
	* It will emulate the behavior of the property which it represents, so that it can safely be stored
	* to streams without breaking the class definitions found in the streams.
	**/
	class X4CORE_API UnknownClassProperty : public SingleClassProperty 
	{
		X4_CLASS(UnknownClassProperty)
		private:
			UnknownClassProperty() {}

			PropertyTable::TableEntry propEntry;

		public:

			UnknownClassProperty(const PropertyTable::TableEntry& property, size_t offset);

			const PropertyTable::TableEntry& getEntry() const { return propEntry; }

			void writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const;
			void readProperty(Object* obj, pointer_diff offset, InputObjectStream *in, card64 expectedSize) const;

			std::string toString(const Object* obj, pointer_diff offset) const { throw X4Exception("UnknownClassProperty::toString() - is not implemented"); }
			void fromString(Object* obj, pointer_diff offset, const std::string str) const  { throw X4Exception("UnknownClassProperty::toString() - is not implemented"); }

			const std::string getTypeName() const { return propEntry.typeName; }
			const std::string     getCName() const { return propEntry.propertyName; }
			const std::string     getDescription() const { return "Unknown property"; }
			const Version    getIntroducedInVersion() const { return propEntry.introducedInVersion; }

			inline ClassProperty* copyClassProperty() { return new UnknownClassProperty(propEntry, getOffset()); }

			void assignProperty(const Object *from, Object *to, pointer_diff offset) const;

	};


}




#endif
