//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "VariableArrayClassProperty.h"
//---------------------------------------------------------------------------
#include "ClassProperty.h"
#include "ObjectStreams.h"
#include <StringHelper.h>
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_ABSTRACT_CLASS(VariableArrayClassProperty, 0)
{
	BEGIN_CLASS_INIT(VariableArrayClassProperty);
}


VariableArrayClassProperty::VariableArrayClassProperty(const std::string cName, const Version introducedInVersion, ClassProperty * cp,
			SetSizeMethod setSize, DeleteMethod del, GetMethod get, GetNumMethod getNum, GetObjectNameMethod getName)
:CompoundClassProperty(cName, introducedInVersion)
{
	setSizeMethod = setSize;
	delMethod = del;
	getMethod = get;
	getNumMethod = getNum;
	getNameMethod = getName;
	m_ElementClassProperty = cp;
}

VariableArrayClassProperty::~VariableArrayClassProperty(void)
{
}

const std::string VariableArrayClassProperty::getTypeName() const
{
	std::string ret = "Variable Array of ";
	ret += m_ElementClassProperty->getCName();
	return ret;
}

void VariableArrayClassProperty::writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const
{
	mpcard numElements = getNumElements( (Object*)obj );
	out->writeMaxPlatformCard( numElements );

	for( max_platform_card i=0;i<numElements;i++ )
	{
		pointer_diff diff = getDiff((Object*)obj,i);
		m_ElementClassProperty->write( (Object*)obj, diff, out );
	}
}

void VariableArrayClassProperty::readProperty(Object* obj, pointer_diff offset, InputObjectStream *in, card64 expectedSize) const
{
	max_platform_card num_elements;
	in->readMaxPlatformCard(num_elements);
	setSize( obj, num_elements );
	for( max_platform_card i=0;i<num_elements;i++ )
	{
		pointer_diff diff = getDiff(obj,i);
		m_ElementClassProperty->read( obj, diff, in );
	}
}


std::string VariableArrayClassProperty::toString(const Object* obj, pointer_diff offset) const
{
	return "not implemented.";
}

void VariableArrayClassProperty::fromString(Object* obj, pointer_diff offset, const std::string str) const
{
}

void VariableArrayClassProperty::assignProperty(const Object *from, Object *to, pointer_diff offset) const
{
	setSize( to, getNumElements( (Object*)from ) );
	for( mpcard i=0;i<getNumElements(to);i++ )
	{
		//  pointer_diff diff = getDiff(to,i);
		//  m_ElementClassProperty->assignProperty( from, to, diff );		  // fix by oliver burghard
		m_ElementClassProperty->assignProperty( (Object*)((Object*)from->*getMethod)(i), (Object*)(to->*getMethod)(i), 0 );
	}
}


pointer_diff VariableArrayClassProperty::getDiff( Object * obj, mpcard index ) const
{
	char* objind = getElement(obj,index);
	return objind - (char*)obj;
}


std::string VariableArrayClassProperty::getObjectName( Object * obj, mpcard index ) const
{
	if( getNameMethod == NULL )
	{
		return intToStr( index );
	}
	else
	{
		return (obj->*getNameMethod)(index);
	}
}

ClassProperty*   VariableArrayClassProperty::copyClassProperty()
{
	VariableArrayClassProperty * ret = new VariableArrayClassProperty( getCName(), 
		getIntroducedInVersion(), m_ElementClassProperty, setSizeMethod, delMethod,
		getMethod, getNumMethod, getNameMethod );
	ret->setOwner(this->getOwner());
	return ret;
}


//---------------------------------------------------------------------------
