#ifndef VariableArrayClassPropertyH
#define VariableArrayClassPropertyH
//---------------------------------------------------------------------------
#include "x4core.h"
//---------------------------------------------------------------------------
#include "CompoundClassProperty.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
#include "NumericalClassProperty.h"
#include "BooleanClassProperty.h"
#include "EmbeddedObjectClassProperty.h"
#include "ObjectClassProperty.h"

//---------------------------------------------------------------------------

namespace X4
{

	class ClassProperty;
	class VariableArrayElementEditor;

	class X4CORE_API VariableArrayClassProperty : public CompoundClassProperty
	{
		X4_CLASS(VariableArrayClassProperty);

		friend VariableArrayElementEditor;

		public:
		   // property-access-methods
		   typedef void (Object::* SetSizeMethod)(mpcard);
		   typedef void (Object::* DeleteMethod)(mpcard);
		   typedef char* (Object::* GetMethod)(mpcard);
		   typedef mpcard (Object::* GetNumMethod)();
		   typedef std::string (Object::* GetObjectNameMethod)(mpcard );

		public:
			VariableArrayClassProperty(const std::string cName, const Version introducedInVersion, ClassProperty * elementProperty,
					SetSizeMethod setSize, DeleteMethod del, GetMethod get, GetNumMethod getNum, GetObjectNameMethod getName );
			~VariableArrayClassProperty(void);

			virtual const std::string     getTypeName() const;
			virtual void             writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const;
			virtual void             readProperty(Object* obj, pointer_diff offset, InputObjectStream *in, card64 expectedSize) const;
			virtual std::string           toString(const Object* obj, pointer_diff offset) const;
			virtual void             fromString(Object* obj, pointer_diff offset, const std::string str) const;
			virtual void             assignProperty(const Object *from, Object *to, pointer_diff offset) const;
			virtual ClassProperty*   copyClassProperty();

			ClassProperty * getElementClassProperty() const {return m_ElementClassProperty;}

		private:

			ClassProperty		*		m_ElementClassProperty;
			SetSizeMethod				setSizeMethod;
			DeleteMethod				delMethod;
			GetMethod					getMethod;
			GetNumMethod				getNumMethod;
			GetObjectNameMethod		getNameMethod;

		public:

			void setSize( Object * obj, mpcard size) const
			{
				(obj->*setSizeMethod)(size);
			}

			void deleteElement( Object * obj, mpcard index ) const
			{
				(obj->*delMethod)(index);
			}

			char * getElement( Object * obj, mpcard index ) const
			{
				return (obj->*getMethod)(index);
			}

			mpcard getNumElements( Object * obj ) const
			{
				return (obj->*getNumMethod)();
			}

			std::string getObjectName( Object * obj, mpcard index ) const;

			pointer_diff getDiff( Object * obj, mpcard index ) const;
	};

	#if 0
	template<typename FloatType>
	class OBJECTS_API DynamicVectorClassProperty : public CompoundClassProperty
	{
		public:
			// property-access-methods
			typedef void (Object::* SetDimMethod)(mpcard);
			typedef mpcard (Object::* GetDimMethod)();
			typedef void (Object::* SetElementMethod)(FloatType element);
			typedef FloatType (Object::* GetElementMethod)(mpcard);

		public:

			DynamicVectorClassProperty(const std::string cName, const Version introducedInVersion, ClassProperty * elementProperty,
				SetDimMethod setDim, GetDimMethod getDim, SetElementMethod setElement, GetElementMethod getElement);
			~DynamicVectorClassProperty(void);

			virtual const std::string     getTypeName() const;
			virtual void             writeProperty(const Object* obj, pointer_diff offset, OutputObjectStream *out) const;
			virtual void             readProperty(Object* obj, pointer_diff offset, InputObjectStream *in) const;
			virtual std::string           toString(const Object* obj, pointer_diff offset) const;
			virtual void             fromString(Object* obj, pointer_diff offset, const std::string str) const;
			virtual void             assignProperty(const Object *from, Object *to, pointer_diff offset) const;
			virtual ClassProperty*   copyClassProperty();

	};
	#endif 

	#define ADD_VAR_ARRAY_PROP( propName, version, elementProperty ) \
		((MetaClass*)getClass())->addProperty( new VariableArrayClassProperty( std::string(#propName) , MetaClass::FIRST_VERSION + version, \
		 elementProperty,\
		 (VariableArrayClassProperty::SetSizeMethod)&setNum##propName##s, \
		 (VariableArrayClassProperty::DeleteMethod)&delete##propName, \
		 (VariableArrayClassProperty::GetMethod) &get##propName, \
		 (VariableArrayClassProperty::GetNumMethod) &getNum##propName##s, \
		 (VariableArrayClassProperty::GetObjectNameMethod) &get##propName##Name ));

	#define ADD_STRING_VAR_ARRAY_PROP( propName, version ) \
		ADD_VAR_ARRAY_PROP( propName, version, new StringClassProperty("element", MetaClass::FIRST_VERSION + version,NULL))
	#define ADD_CARD8_VAR_ARRAY_PROP( propName, version ) \
		ADD_VAR_ARRAY_PROP( propName, version, new Card8ClassProperty("element", MetaClass::FIRST_VERSION + version,NULL))
	#define ADD_INT32_VAR_ARRAY_PROP( propName, version ) \
		ADD_VAR_ARRAY_PROP( propName, version, new Int32ClassProperty("element", MetaClass::FIRST_VERSION + version,NULL))
	#define ADD_CARD32_VAR_ARRAY_PROP( propName, version ) \
		ADD_VAR_ARRAY_PROP( propName, version, new Card32ClassProperty("element", MetaClass::FIRST_VERSION + version,NULL))
	#define ADD_FLOAT32_VAR_ARRAY_PROP( propName, version ) \
		ADD_VAR_ARRAY_PROP( propName, version, new Float32ClassProperty("element", MetaClass::FIRST_VERSION + version,NULL))
	#define ADD_MPCARD_VAR_ARRAY_PROP( propName, version ) \
		ADD_VAR_ARRAY_PROP( propName, version, new MaxPlatformCardClassProperty("element", MetaClass::FIRST_VERSION + version,NULL))
	#define ADD_OBJECT_VAR_ARRAY_PROP( propName, baseClass, is_owner, version ) \
		ADD_VAR_ARRAY_PROP( propName, version, new ObjectClassProperty("element", MetaClass::FIRST_VERSION + version, NULL, baseClass, is_owner, NULL))
	
	#define ADD_EMBEDDED_OBJECT_VAR_ARRAY_PROP( propName, Type, version ) \
		ADD_VAR_ARRAY_PROP( propName, version, new EmbeddedObjectClassProperty(#Type, MetaClass::FIRST_VERSION + version, NULL, Type::getClass()))

	//#define ADD_OBJECT_INSTANCE_VAR_ARRAY_PROP( propName, Type, version ) \
	//	ADD_VAR_ARRAY_PROP( propName, version, new ObjectInstanceClassProperty<Type>(#Type, MetaClass::FIRST_VERSION + version, NULL, Type::getClass(), NULL))

	#define IMPLEMENT_VAR_ARRAY_METHODS_STL( propName ) \
		void setNum##propName##s(mpcard size) \
			{ propName##.resize(size);} \
		void delete##propName##( mpcard index ) \
			{x4Assert( index < propName##.size()); propName##.erase( propName##.begin()+index );}\
		char * get##propName##( mpcard index ) \
			{x4Assert( index < propName##.size()); return (char*)&propName##[index];} \
		mpcard getNum##propName##s() \
			{return propName##.size();} \
		std::string get##propName##Name(mpcard index ) \
			{x4Assert( index < propName##.size()); forget(index); return "Object";}

	#define IMPLEMENT_VAR_ARRAY_METHODS_STL_UPDATE( propName, callOnChange ) \
		void setNum##propName##s(mpcard size) \
			{ propName##.resize(size); callOnChange##();} \
		void delete##propName##( mpcard index ) \
			{x4Assert( index < propName##.size()); propName##.erase( propName##.begin()+index ); callOnChange##();}\
		char * get##propName##( mpcard index ) \
			{x4Assert( index < propName##.size()); return (char*)&propName##[index];} \
		mpcard getNum##propName##s() \
			{return propName##.size();} \
		std::string get##propName##Name(mpcard index ) \
			{x4Assert( index < propName##.size()); return "Object";}

	#define IMPLEMENT_VAR_STRING_ARRAY_METHODS_STL_UPDATE( propName, callOnChange ) \
		void setNum##propName##s(mpcard size) \
			{ propName##.resize(size); callOnChange##();} \
		void delete##propName##( mpcard index ) \
			{x4Assert( index < propName##.size()); propName##.erase( propName##.begin()+index ); callOnChange##();}\
		char * get##propName##( mpcard index ) \
			{x4Assert( index < propName##.size()); return (char*)&propName##[index];} \
		mpcard getNum##propName##s() \
			{return propName##.size();} \
		std::string get##propName##Name(mpcard index ) \
			{x4Assert( index < propName##.size()); return propName##[index];}

}

#endif
