#include "StdAfx.h"

#include "Variant.h"

#include <ConsoleOutput.h>
#include "NumericalClassProperty.h"
#include "ObjectStreams.h"
#include "ObjectClassProperty.h"
#include "StringClassProperty.h"
#include "Object.h"

namespace X4
{

	static std::string descrType(card32 v) {
	   switch (v) {
		  case Variant::X4T_VOID    : return "X4T_VOID";
		  case Variant::X4T_CARD8   : return "X4T_CARD8";
		  case Variant::X4T_INT8    : return "X4T_INT8";
		  case Variant::X4T_CARD16  : return "X4T_CARD16";
		  case Variant::X4T_INT16   : return "X4T_INT16";
		  case Variant::X4T_CARD32  : return "X4T_CARD32";
		  case Variant::X4T_INT32   : return "X4T_INT32";
		  case Variant::X4T_CARD64  : return "X4T_CARD64"; 
		  case Variant::X4T_INT64   : return "X4T_INT64";
		  case Variant::X4T_MPCARD  : return "X4T_MPCARD"; 
		  case Variant::X4T_MPINT   : return "X4T_MPINT";
		  case Variant::X4T_FLOAT32 : return "X4T_FLOAT32"; 
		  case Variant::X4T_FLOAT64 : return "X4T_FLOAT64";
		  case Variant::X4T_OBJECT  : return "X4T_OBJECT";
		  case Variant::X4T_STRING  : return "X4T_STRING";
		  default: return "unkown";          
	   }                                     
	}                                        


	// ------------------------------------------------------------------------------
	IMPLEMENT_X4_CLASS(Variant, 0)
	{
	   BEGIN_CLASS_INIT(Variant)
	   ADD_CARD32_PROP_UPDATE_DESCR(type, 0, NULL, descrType, 32, NULL)
	   ADD_INT64_PROP(intVal, 0)
	   ADD_FLOAT64_PROP(floatVal, 0)
	   ADD_OBJECT_PROP(obj, 0, Object::getClass(), false);
	   ADD_STRING_PROP(str, 0);
	}

	// ------------------------------------------------------------------------------
	Variant::Variant()
	{
	   type = X4T_VOID;
	   intVal = 0;
	   floatVal = 0;
	   obj = NULL;
	}

	// ------------------------------------------------------------------------------
	Variant::~Variant()
	{
	}

	// ------------------------------------------------------------------------------
	void Variant::assign( const Object* obj, X4CopyContext *context /*= NULL*/ )
	{
	   const Variant *other = dynamic_cast<const Variant*>(obj);
	   if (other == NULL) throw EInvalidArgument("Variant::assign - incompatible type.");
	   this->type = other->type;
	   this->intVal = other->intVal;
	   this->floatVal = other->floatVal;
	   this->obj = other->obj;
	   this->str = other->str;
		forget(context);
	}

	// ------------------------------------------------------------------------------
	void Variant::write( OutputObjectStream *s ) const
	{
	   s->write(type);
	   if (type > X4T_VOID || type <= X4T_MAX_TYPE_INDEX)
	   {
		  if (type <= X4T_MPINT) {
			 s->write<int64>(intVal);
		  } else if (type <= X4T_FLOAT64) {
			 s->write<float64>(floatVal);
		  } else if (type == X4T_OBJECT) {
			 Persistent *per = dynamic_cast<Persistent*>(obj);
			 if ((Object*)per != obj) consoleError("Variant::write - trying to write non-persistent object, exchanged with NULL-pointer.");
			 s->writeObject(per);
		  } else if (type == X4T_STRING) {
			 s->writeString(str);
		  }else {
			 x4Assert(false);
		  }
	   }
	}

	// ------------------------------------------------------------------------------
	void Variant::read( InputObjectStream *s )
	{
	   s->read(type);
	   if (type > X4T_VOID || type <= X4T_MAX_TYPE_INDEX)
	   {
		  if (type <= X4T_MPINT) {
			 s->read<int64>(intVal);
		  } else if (type <= X4T_FLOAT64) {
			 s->read<float64>(floatVal);
		  } else if (type == X4T_OBJECT) {
			 Persistent *per = NULL;
			 s->readObject(per);
			 obj = per;
		  } else if (type == X4T_STRING) {
			 s->readString(str);
		  } else {
			x4Assert(false);
		  }
	   }
	}
}
