//---------------------------------------------------------------------------
#ifndef ClassEditorH
#define ClassEditorH
//---------------------------------------------------------------------------
#include "x4interface.h"
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include "Object.h"
#include "IEditor.h"
#include "ObjectViewsTable.h"
//---------------------------------------------------------------------------

class QWidget;

namespace X4 {

class X4INTERFACE_API ClassEditor : public Object, public IEditor {
 private:
   static MetaClass *ClassEditorMetaClass;

 public:

   ClassEditor();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void update() = 0;
	virtual card32 getPreferredWidth() = 0;
   virtual card32 getPreferredHeight() = 0;

	virtual QWidget * makeEditor( QWidget * parent, Object * obj ) = 0;
	virtual Object * getObject() = 0;

	virtual bool standAloneEditorApplication() {return false;}
};

}; // end namespace

/**
	declares a class editor
*/
#define MAKE_CLASS_EDITOR_EX( api, editorName, widgetName ) \
class api editorName : public X4::ClassEditor \
{\
private:\
	static X4::MetaClass * editorName##MetaClass;\
   static X4::Object* newInstance();\
\
	widgetName  *m_Widget;\
	X4::Object	*	m_Object;\
\
public:\
   virtual const X4::MetaClass *getInstanceClass() const;\
   static const X4::MetaClass *getClass();\
   static void init(const X4::MetaClass *parentClass);\
   static void shutdown();\
\
	editorName##() {m_Widget = NULL;} \
\
   virtual QWidget *makeEditor(QWidget *parent, X4::Object *obj);\
	virtual void apply() {}\
   virtual void update();\
   virtual X4::card32 getPreferredWidth();\
   virtual X4::card32 getPreferredHeight();\
   virtual X4::Object *getObject();\
\
	virtual bool standAloneEditorApplication();\
\
	virtual ~##editorName##();\
};

#define MAKE_CLASS_EDITOR( api, className ) MAKE_CLASS_EDITOR_EX( api, className##Editor, className##EditorWidget )


/**
	implements a class editor ( param = editorName )
*/
#define IMPLEMENT_CLASS_EDITOR_EX( className, editorName, widgetName, standAlone ) \
X4::MetaClass* editorName##::editorName##MetaClass = NULL;\
\
const X4::MetaClass* editorName##::getInstanceClass() const {\
   return getClass();\
}\
\
X4::Object* editorName##::newInstance() {\
   return new editorName##;\
}\
\
const X4::MetaClass* editorName##::getClass() {		\
   if (editorName##MetaClass==NULL) {		\
	editorName##MetaClass = new X4::MetaClass( #editorName , "no description.",		\
                                  newInstance, sizeof( editorName ));		\
   }		\
   return editorName##MetaClass;		\
}		\
		\
void editorName##::init(const X4::MetaClass *parentClass) {		\
   ((X4::MetaClass*)getClass())->init(parentClass, X4::MetaClass::FIRST_VERSION + 0);		\
   editorName##MetaClass->copyParentProperties();		\
}		\
		\
void editorName##::shutdown() {		\
   delete editorName##MetaClass;		\
}		\
void editorName##::update() {m_Widget->updateEditor();}\
X4::card32 editorName##::getPreferredWidth() {return m_Widget->width();}\
X4::card32 editorName##::getPreferredHeight()  {return m_Widget->height();}\
QWidget * editorName##::makeEditor( QWidget * parent, X4::Object * obj ) { \
	m_Widget = new widgetName##( parent ); \
	m_Object = obj; \
	m_Widget->setup( ( className *)m_Object ); \
	m_Widget->updateEditor(); \
	X4::ObjectViewsTable::bindView(m_Object, this);\
	return m_Widget; }\
X4::Object * editorName##::getObject() {return m_Object;}\
\
bool editorName##::standAloneEditorApplication() {return standAlone;}\
\
editorName##::~##editorName() {\
	X4::ObjectViewsTable::releaseView(m_Object, this);\
}

/// use this macro for widget style editor
#define IMPLEMENT_CLASS_EDITOR( className ) IMPLEMENT_CLASS_EDITOR_EX( className, className##Editor , className##EditorWidget, false )
/// use this macro for application style editors
#define IMPLEMENT_CLASS_EDITOR_STANDALONE( className ) IMPLEMENT_CLASS_EDITOR_EX( className, className##Editor , className##EditorWidget, true )

//}

#endif
