#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ClassTypeClassPropertyEditorWidget.h"
//---------------------------------------------------------------------------
#include "ClassTypeClassProperty.h"
#include "StringHelper.h"
#include "ConsoleOutput.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_CLASS_EDITOR( ClassTypeClassProperty )

ClassTypeClassPropertyEditorWidget::ClassTypeClassPropertyEditorWidget(QWidget * parent, Qt::WindowFlags f)
:QComboBox(parent), callBack(0)
{
	 m_Object=NULL;
	 initialized = false;
   QObject::connect(this, SIGNAL(currentIndexChanged(int)), this, SLOT(selectionChanged(int)));
}

ClassTypeClassPropertyEditorWidget::~ClassTypeClassPropertyEditorWidget(void)
{
}

void ClassTypeClassPropertyEditorWidget::setup( ClassTypeClassProperty * obj )
{
	m_Object = obj;


}

void ClassTypeClassPropertyEditorWidget::updateEditor()
{

//	const MetaClass *mc = ((ClassTypeClassProperty*)m_Object)->getValue((Object*)obj);
   if (!initialized) {
      initialize(0, ((ClassTypeClassProperty*)m_Object)->getBaseClass());
		setCurrentIndex(0);
      initialized = true;
   }
//   setSelection(mc);
}



std::string ClassTypeClassPropertyEditorWidget::handlePrefix(const std::string s)
{
	if (!ignorePrefix.empty() && s.length() > ignorePrefix.length()) {
		return s.substr(ignorePrefix.length(), s.length()-ignorePrefix.length());
	} else {
	   return s;
	}
}


void ClassTypeClassPropertyEditorWidget::collectClasses(const std::string baseClass, bool sortByName, std::string ignorePrefix )
{
	this->ignorePrefix = ignorePrefix;
	const MetaClass * mc = MetaClass::getClassByName( baseClass );
	QStringList list;
	if( !mc->isAbstract() ) {
		std::string name = baseClass; // not for base classes - handlePrefix(baseClass);
		list << name.c_str();
	}

	collectClassList( baseClass, list);
	if( sortByName )
		list.sort();

	addItems( list );
}

void ClassTypeClassPropertyEditorWidget::collectClassList(const std::string baseClass,QStringList &list )
{
	const MetaClass * mc = MetaClass::getClassByName( baseClass );
	for( card32 i=0;i<mc->getNumDescendants();i++ )
	{
		std::string className = mc->getDescendant(i)->getClassName();
		std::string name = handlePrefix(className);
		list << name.c_str();
		collectClassList( className, list );
	}
}

static int findTextInQComboBox(QComboBox *box, const QString &x, const std::string &ignorePrefix) {
   int num = box->count();
   for (int i=0; i<num; i++) {
      QString other = box->itemText(i);
      std::string wpref = ignorePrefix.c_str()+qString2STLString(x);
      QString x2 = wpref.c_str();
      if (other == x) return i;
      if (other == x2) return i;
   }
   consoleWarning("findTextInQComboBox failed.");
   return 0;
}

void ClassTypeClassPropertyEditorWidget::setSelection(const std::string className){
	std::string name = handlePrefix(className);
	QString x = name.c_str();
	int index = findTextInQComboBox(this, x, ignorePrefix);
	setCurrentIndex(index);
}


void ClassTypeClassPropertyEditorWidget::initialize(card32 ident, const MetaClass *mc) {
   std::string identStr;
   for (card32 i=0; i<ident; i++) {
      identStr += "  ";
   }

	// Qt dosen't support sotring mc pointer, using classnames (=> identStr="")
	identStr="";


   if (mc->isAbstract()) {
		//addItem((identStr+mc->getClassName() + std::string(" (abstract)")).c_str());
      //cbClasses->Items->AddObject((identStr+mc->getClassName() + std::string(" (abstract)")).c_str(), (System::TObject*)mc);
   } else {
		addItem((identStr+mc->getClassName()).c_str());
      //cbClasses->Items->AddObject((identStr+mc->getClassName()).c_str(), (System::TObject*)mc);
   }
   const card32 numDescendants = mc->getNumDescendants();

   for (card32 i=0; i<numDescendants; i++) {
      initialize(ident+1, mc->getDescendant(i));
   }
	this->setCurrentIndex(0);

}

const MetaClass* ClassTypeClassPropertyEditorWidget::getSelected() {
   card32 i = currentIndex();
	const MetaClass *mc = MetaClass::getClassByName(ignorePrefix + qString2STLString(this->itemText(i)));
   
	 return mc;
}

void ClassTypeClassPropertyEditorWidget::setSelection(const MetaClass *mc) {
	setSelection(handlePrefix(mc->getClassName()));
}
              
void ClassTypeClassPropertyEditorWidget::selectionChanged(int index) {
   if (callBack != NULL) {
      callBack();
      emit sigModified();
   }
}





#include "ClassTypeClassPropertyEditorWidget_moc.h"
