#ifndef ConsoleWidgetH
#define ConsoleWidgetH
//---------------------------------------------------------------------------
#include "x4interface.h"
//---------------------------------------------------------------------------
#include <QWidget>
#include <QtCore/QMutex.h>
#include "X4Types.h"
//---------------------------------------------------------------------------
//this file is generated from the ui file by a custom buildstep, include it here
#include "x4interface/ConsoleWidget_ui.h"
#include "ConsoleOutput.h"
//---------------------------------------------------------------------------

namespace X4
{

class EventDisplayerEvent;
class EventViewerWidget;


/// set this variable to true to see debug output flushes immediately. default is false (because "true" processes the event loop which is risky)
extern bool X4INTERFACE_API ALWAYS_PROCESS_EVENTS_ON_FLUSH;

class X4INTERFACE_API ConsoleWidget : public QWidget, ConsoleHandler
{
	Q_OBJECT;

protected:
	Ui_ConsoleWidget ui;

private:

	QTimer *timer;
	QColor stdBackground;
	QColor stdColor;
	EventViewerWidget *eventViewer;
	std::string textBuffer;
    QMutex *bufferMutex;
	std::string* loggedOutput;

public:
    ConsoleWidget(QWidget *parent);
    ~ConsoleWidget(void);

    // output
    void operator<<(bool b);
    void operator<<(int i);
    void operator<<(unsigned int i);
    void operator<<(int64 i);
    void operator<<(card64 i);
    void operator<<(long double d);
    void operator<<(std::string s);
    void operator<<(char* s);

    void displayEvent(const MetaClass *eventType /* of type EventDisplayer */, std::string name, const EventDisplayerEvent *event);
    void cr();

    virtual void flush();
    void warningMessage(std::string message);
    void errorMessage(std::string message);
	void setLoggedOutput(std::string* whereToLog) { loggedOutput = whereToLog; }

	virtual void outputText(const std::string& text) {ui.TEConsoleOutput->append(text.c_str());}
	virtual void outputHTML(const std::string& html) {ui.TEConsoleOutput->insertHtml(html.c_str());};
	virtual void outputObject(const void*, size_t) {(*this)<<"warning: object output rejected";}
	virtual void showConsole() {/*show();*/}
	virtual void hideConsole() {hide();}

    enum Page
    {
        PAGE_CONSOLE = 0,
        PAGE_EVENTS = 1,
        PAGE_PROTOCOL = 2,
        PAGE_ERRORS = 3,

        LAST_PAGE = PAGE_ERRORS,
        NUM_PAGES = LAST_PAGE+1
    };
    //static const card32 PAGE_CONSOLE = 0;
    //static const card32 PAGE_EVENTS = 1;
    //static const card32 PAGE_PROTOCOL = 2;
    //static const card32 PAGE_ERRORS = 3;
    //static const card32 PAGE_CONSOLE_HIDE_INPUT = 4;

    void showPage(Page page);

private slots:
	void textTimer();
	
public slots:
	// /// button execute clicked
	// void sltBTExecClicked();
	/// checkbox log console to file toggled
	void sltCBLogConsoleStateChanged(int);
	/// checkbox log events to file toggled
	void sltCBLogEventsStateChanged(int);

    /// flush the content of the buffer to the debug window
    /// the method is used as a slot to make it thread-safe (through queued connection)
    void sltFlush();

    void sltWarningMessage(const std::string&  message);
    void sltErrorMessage(const std::string&  message);

signals:

    void sigFlush();
    void sigWarningMessage(const std::string& msg);
    void sigErrorMessage(const std::string& msg);

protected:

	void timerEvent(QTimerEvent * e);
	void flushText();


};


}

#endif
