#include "StdAfx.h"

#include "ConsoleWindowProcess.h"
#include "X4Assert.h"

#include <QtCore/qiodevice.h>
#include <iostream>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>

using namespace X4;

enum X4ConsoleOutputTypeInterProc {OTIP_CONSOLE_CTRL, OTIP_X4_OBJECT, OTIP_ASCII, OTIP_HTML, OTIP_TERMINATE_CONSOLE, OTIP_PARENT_HANDLE};

struct ConsoleCtrl {
   bool resize;
   bool move;
   bool hide;
   bool show;
   bool flush;
   int x,y;
   ConsoleCtrl() {resize=0; move=0; hide=0; show=0; flush=0; x=0; y=0;}
};

ConsoleWindowProcess *X4::globalConsoleWindowProcess = NULL;

ConsoleWindowProcess *ConsoleWindowProcess::createGlobalConsoleWindowProcess(QIODevice *consoleProc)
{
   if (!globalConsoleWindowProcess ) {
      globalConsoleWindowProcess = new ConsoleWindowProcess(consoleProc);
   } else {
      x4Assert(false);
   }
   return globalConsoleWindowProcess;
}

ConsoleWindowProcess *ConsoleWindowProcess::getGlobalConsoleWindowProcess() 
{
   return globalConsoleWindowProcess;
}


ConsoleWindowProcess::ConsoleWindowProcess(QIODevice *consoleProcess)
{
   this->consoleProcess = consoleProcess;
   if (consoleProcess->openMode() & QIODevice::Text) {
      x4Assert(false);
   }
}

void ConsoleWindowProcess::outputText(const string &text)
{
   int32 tag = OTIP_ASCII;
   consoleProcess->write((char*)&tag, sizeof(tag));
   int size = (int)text.size()+1;
   consoleProcess->write((char*)&size, sizeof(size));
   consoleProcess->write(&(text[0]), size-1);
   char n=0;
   consoleProcess->write(&n, 1);
	consoleProcess->waitForBytesWritten(100);
}

void ConsoleWindowProcess::outputHTML(const string &html)
{
   int32 tag = OTIP_HTML;
   consoleProcess->write((char*)&tag, sizeof(tag));
   int size = (int)html.size()+1;
   consoleProcess->write((char*)&size, sizeof(size));
   consoleProcess->write(&(html[0]), size-1);
   char n=0;
   consoleProcess->write(&n, 1);
	consoleProcess->waitForBytesWritten(100);
}

void ConsoleWindowProcess::outputObject(const void* data, mpcard sizeParam)
{
   int size = sizeParam;
   consoleProcess->write((char*)&size, sizeof(size));
   consoleProcess->write((char*)data, size);
	consoleProcess->waitForBytesWritten(100);
}

void ConsoleWindowProcess::flush() 
{
   ConsoleCtrl ct;
   ct.flush = true;
   int32 tag = OTIP_CONSOLE_CTRL;
   consoleProcess->write((char*)&tag, sizeof(tag));
   consoleProcess->write((char*)&ct, sizeof(ct));

   consoleProcess->waitForBytesWritten(-1);
}

void ConsoleWindowProcess::resizeWin(int x, int y)
{
   ConsoleCtrl ct;
   ct.resize = true;
   ct.x=x;
   ct.y=y;
   int32 tag = OTIP_CONSOLE_CTRL;
   consoleProcess->write((char*)&tag, sizeof(tag));
   consoleProcess->write((char*)&ct, sizeof(ct));
	consoleProcess->waitForBytesWritten(100);
}

void ConsoleWindowProcess::moveWin(int x, int y)
{
   ConsoleCtrl ct;
   ct.move = true;
   ct.x=x;
   ct.y=y;
   int32 tag = OTIP_CONSOLE_CTRL;
   consoleProcess->write((char*)&tag, sizeof(tag));
   consoleProcess->write((char*)&ct, sizeof(ct));
	consoleProcess->waitForBytesWritten(100);
}

void ConsoleWindowProcess::showWin()
{
   ConsoleCtrl ct;
   ct.show = true;
   int32 tag = OTIP_CONSOLE_CTRL;
   consoleProcess->write((char*)&tag, sizeof(tag));
   consoleProcess->write((char*)&ct, sizeof(ct));
	consoleProcess->waitForBytesWritten(100);
}

void ConsoleWindowProcess::showHideWinToggle()
{
	ConsoleCtrl ct;
	ct.hide = true;
	ct.show = true;
	int32 tag = OTIP_CONSOLE_CTRL;
	consoleProcess->write((char*)&tag, sizeof(tag));
	consoleProcess->write((char*)&ct, sizeof(ct));
	consoleProcess->waitForBytesWritten(100);
}

void ConsoleWindowProcess::hideWin()
{
   ConsoleCtrl ct;
   ct.hide = true;
   int32 tag = OTIP_CONSOLE_CTRL;
   consoleProcess->write((char*)&tag, sizeof(tag));
   consoleProcess->write((char*)&ct, sizeof(ct));
	consoleProcess->waitForBytesWritten(100);
}

void ConsoleWindowProcess::sendParentID(HANDLE targetProcess) {
	int32 tag = OTIP_PARENT_HANDLE;
// 	HANDLE duplicate;
// 	DuplicateHandle(
// 		GetCurrentProcess(), GetCurrentProcess(), targetProcess,
// 		&duplicate, 0, true, DUPLICATE_SAME_ACCESS);
	DWORD cpid = GetCurrentProcessId();
	consoleProcess->write((char*)&tag, sizeof(tag));
	consoleProcess->write((char*)&cpid, sizeof(cpid));
	consoleProcess->waitForBytesWritten(100);
}


ConsoleWindowProcess::~ConsoleWindowProcess()
{
   int32 tag = OTIP_TERMINATE_CONSOLE;
   consoleProcess->write((char*)&tag, sizeof(tag));
	consoleProcess->waitForBytesWritten(100);
}



void ConsoleWindowProcessReceiver::run()
{
   QObject::moveToThread(this);

   if (_setmode(_fileno(stdin), _O_BINARY) != -1) {
      bool termConsole = false;
      while (!termConsole && cin.good() && !cin.eof() && !(!cin)) {
         int32 tag;
         cin.read((char*)&tag, sizeof(int32));
         switch ((X4ConsoleOutputTypeInterProc)tag) {
            
            case OTIP_CONSOLE_CTRL: {
               ConsoleCtrl rec;
               cin.read((char*)&rec, sizeof(ConsoleCtrl));
					if (rec.show && rec.hide) {
						emit showHideWinToggle();
					} else if (rec.show) {
                  emit showWin();
               } else if (rec.hide) {
                  emit hideWin();
               } else if (rec.move) {
                  emit moveWin(rec.x, rec.y);
               } else if (rec.resize) {
                  emit resizeWin(rec.x, rec.y);
               } else if (rec.flush) {
                  emit flush();
               }
               break;
            }

            case OTIP_X4_OBJECT: {
               int size;
               cin.read((char*)&size, sizeof(int));
               char *bin = new char[size];
               emit newBinary(bin, size);
               break;
            }

            case OTIP_ASCII: {
               int size;
               cin.read((char*)&size, sizeof(int));
               char *ascii = new char[size];
               cin.read(ascii, size);
					emit newText(ascii);
               break;
            }
            
            case OTIP_HTML: {
               int size;
               cin.read((char*)&size, sizeof(int));
               char *ascii = new char[size];
               cin.read(ascii, size);
               emit newHTML(ascii);
               break;
            }

            case OTIP_TERMINATE_CONSOLE: {
               termConsole = true;
               emit closeAll();
               break;
            }
				case OTIP_PARENT_HANDLE: {
					//HANDLE parentProcess;
					DWORD parentProcess;
					cin.read((char*)&parentProcess, sizeof(parentProcess));
					emit parentProcessIDReceived((card64)parentProcess);
				}
         }
      }
   } else {
      emit newText("error - cannot reopen stdin in binary mode!");
   }
}

#include "x4interface/ConsoleWindowProcess_moc.h"