#ifndef ConsoleWindowProcessH
#define ConsoleWindowProcessH

#include <ConsoleOutput.h>
#include <QtCore/qthread.h>

class QIODevice;

namespace X4 {


class X4INTERFACE_API ConsoleWindowProcess : public ConsoleHandler {
 private:
   QIODevice *consoleProcess;

 public:
   ConsoleWindowProcess(QIODevice *consoleProcess);

   virtual void outputText(const string &text);
   virtual void outputHTML(const string &html);
   virtual void outputObject(const void* data, mpcard size);
	virtual void flush();

   void resizeWin(int x, int y);
   void moveWin(int x, int y);
   void showWin();
   void hideWin();
	void showHideWinToggle();
	void sendParentID(Qt::HANDLE targetProcess);

	virtual void showConsole() {showWin();}
	virtual void hideConsole() {hideWin();}
	virtual void toggleVisible() {showHideWinToggle();}
	virtual void moveConsoleWindow(int x, int y) {moveWin(x,y);}
	virtual void resizeConsoleWindow(int w, int h) {resizeWin(w,h);}


   ~ConsoleWindowProcess();

   static ConsoleWindowProcess *createGlobalConsoleWindowProcess(QIODevice *consoleProcess);
   static ConsoleWindowProcess *getGlobalConsoleWindowProcess();
};

extern ConsoleWindowProcess *globalConsoleWindowProcess;



class X4INTERFACE_API ConsoleWindowProcessReceiver : public QThread {
   Q_OBJECT
 public:
   virtual void run();

 signals:
   void newText(char *text); // delete text after receive, only one receiver
   void newHTML(char *HTML); // delete text after receive, only one receiver
   void newBinary(char *data, int size); // delete data after receive, only one receiver

   void resizeWin(int x, int y);
   void moveWin(int x, int y);
   void showWin();
   void hideWin();
	void showHideWinToggle();
   void closeAll();
   void flush();

	void parentProcessIDReceived(card64);
};


}

#endif