//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
//#define DEBUGOUTPUT_CPP
#include "DebugOutput.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.inline.h"
#include "DynamicLinearAlgebra.inline.h"
#include "UserSettings.h"
#include "Configuration.h"
#include "SparseLinearAlgebra.inline.h"
#include <opencv\cxcore.h>
//---------------------------------------------------------------------------

using namespace X4;

DebugOutput X4::debugOutput;
DOutput_TalkToPointCloudProject * X4::doutput_talkToPointCloudProject = NULL;

DebugOutput::DebugOutput() {outputEnabled=true;}

DebugOutput::~DebugOutput() {}


DebugOutput& DebugOutput::operator<<(bool b)         {if (outputEnabled) x4console << b; return *this;}
DebugOutput& DebugOutput::operator<<(int i)          {if (outputEnabled) x4console << i; return *this;}
DebugOutput& DebugOutput::operator<<(unsigned int i) {if (outputEnabled) x4console << i; return *this;}
DebugOutput& DebugOutput::operator<<(X4::int64 i)    {if (outputEnabled) x4console << i; return *this;}
DebugOutput& DebugOutput::operator<<(X4::card64 i)   {if (outputEnabled) x4console << i; return *this;}
DebugOutput& DebugOutput::operator<<(long double d)  {if (outputEnabled) x4console << d; return *this;}
DebugOutput& DebugOutput::operator<<(float f)        {if (outputEnabled) x4console << f; return *this;}
DebugOutput& DebugOutput::operator<<(double d)       {if (outputEnabled) x4console << d; return *this;}
DebugOutput& DebugOutput::operator<<(std::string s)  {if (outputEnabled) x4console << s; return *this;}
DebugOutput& DebugOutput::operator<<(char* s)        {if (outputEnabled) x4console << s; return *this;}


DebugOutput& DebugOutput::operator<<(const CvMat &m)
{
	if (outputEnabled) {
		for (int32 i=0;i<m.rows;i++ )
		{
			(*this).cr();
			for (int32 j=0;j<m.cols;j++)
			{
				if( j )
					(*this) << ",   ";
				(*this) << cvmGet(&m,i,j);
			}
		}
		(*this).cr();
	}
	return *this;
}


DebugOutput& DebugOutput::operator<<(const BoundingBox3f &bbox )
{
	if (outputEnabled) {
		(*this) << "(" << bbox.lowerCorner[0] << "," << bbox.lowerCorner[1] << "," << bbox.lowerCorner[2] << ")/";
		(*this) << "(" << bbox.upperCorner[0] << "," << bbox.upperCorner[1] << "," << bbox.upperCorner[2] << ")";
	}
	return *this;
}







DebugOutput& DebugOutput::operator<<(SparseMatrixF &A)
{
	if (outputEnabled) {
		float32 value;
		mpcard index;
		for ( mpcard r = 0; r < A.getRows(); r++ ){
			SparseVectorF& row = A[r];
			for ( mpcard i = 0; i < row.getNumSparseEntries(); i++ ){
				row.getSparseEntry( i, index, value );
				(*this) << index << ": " << value << ", ";
			}
			(*this).cr();
		}
	}
	return *this;
}


void DebugOutput::writeIdented(unsigned ident, std::string s)
{
	if (outputEnabled) {
		std::string identsp;
		for (unsigned i = 0; i<ident; i++) identsp += " ";
		(*this) << (identsp + s);
	}
}

void DebugOutput::cr() 
{
	if (outputEnabled) x4console.cr();
}

void DebugOutput::flush()
{
	if (outputEnabled) x4console.flush();
}


void DebugOutput::displayEvent(const MetaClass *eventType /* of type EventDisplayer */, std::string name, const EventDisplayerEvent *event)
{
// 	bool doDisplay = true;
// 	UserSettings * u = dynamic_cast<UserSettings*>( Configuration::getConfiguration()->getData( "USER_SETTINGS" ) );
// 	if( u )
// 	{
// 		doDisplay = u->getDisplayDebugEvents();
// 	}
// 	if (doDisplay) {
// 		(*debugWindow).displayEvent(eventType, name, event);
// 	}
	warning("display event is not yet supported.");
}


void X4::warning(std::string msg) {	
	x4console.outputHTML(string("<p><span style=\"font-weight:600; color:#9bb030;\">warning</span><span style=\"font-weight:600; color:#00000;\">: </span>") + msg + string("<br/></p>"));
	x4console.show();
}
   

void X4::error(std::string msg) {
	x4console.outputHTML(string("<p><span style=\"font-weight:600; color:#902040;\">error</span><span style=\"font-weight:600; color:#00000;\">: </span>") + msg + string("<br/></p>"));
	x4console.show();
}

void X4::message(std::string msg) {
   debugOutput << msg;
   x4console.show();
}

void X4::DebugOutput::displayRGBAFloatImage( RGBAFloat32Image * image, std::string name )
{
	if( doutput_talkToPointCloudProject )
		doutput_talkToPointCloudProject->displayRGBAFloatImage(image,name);
}

void X4::DebugOutput::displayQImage( QImage * image, std::string name )
{
	if( doutput_talkToPointCloudProject )
		doutput_talkToPointCloudProject->displayQImage(image,name);
}

void X4::DebugOutput::disable()
{
	outputEnabled = false; 
}

void X4::DebugOutput::enable()
{
	outputEnabled = true;
}
