//---------------------------------------------------------------------------
#ifndef DebugOutputH
#define DebugOutputH
//---------------------------------------------------------------------------
#include "x4interface.h"
//---------------------------------------------------------------------------
#include <string>
#include "LinearAlgebra.h"
#include "DynamicLinearAlgebra.h"
#include "BoundingBox.h"
#include "SparseLinearAlgebra.h"
#include <QtCore/QMutex>
#include "ConsoleOutput.h"
//---------------------------------------------------------------------------

class QImage;
struct CvMat;

namespace X4 {

class EventDisplayerEvent;
class MetaClass;
class RGBAFloat32Image;


class X4INTERFACE_API DebugOutput { 
 private:
	 bool outputEnabled;
 public:	  
	DebugOutput();
	~DebugOutput();
	DebugOutput& operator<<(bool b);
   DebugOutput& operator<<(int i);
   DebugOutput& operator<<(unsigned int i);
   // this is necessary for 64-bit builds
   DebugOutput& operator<<(int64 i);
   DebugOutput& operator<<(card64 i); 
   DebugOutput& operator<<(long double d);
   DebugOutput& operator<<(float f);
   DebugOutput& operator<<(double d); 
   DebugOutput& operator<<(std::string s);
   DebugOutput& operator<<(char* s);
	DebugOutput& operator<<(const BoundingBox3f &bbox);

   template<typename FloatType, int dim>
   DebugOutput& operator<<(const StaticVector<FloatType,dim> &v) {
	   if (outputEnabled) (*this) << v.toString();
	   return *this;
   }
   template<typename FloatType, int dim1, int dim2>
   DebugOutput& operator<<(const StaticMatrix<FloatType,dim1,dim2> &m) {
	   if (outputEnabled) (*this) << m.toString();
	   return *this;
   }
   template<typename FloatType>
   DebugOutput& operator<<(const DynamicVector<FloatType> &v) {
	   if (outputEnabled)  {
			(*this) << "(";
			for( unsigned i=0;i<v.size();i++ )
			{
				if( i )
					(*this) << ",";
				(*this) << v[i];
			}
			(*this) << ")";
		}
		return *this;	
   }
   template<typename FloatType>
   DebugOutput& operator<<(const DynamicMatrix<FloatType> &m) {
	   if (outputEnabled) (*this) << m.toString();
	   return *this;
   }
   
   DebugOutput& operator<<(const CvMat &m);
   DebugOutput& operator<<(SparseMatrixF &A);

   void writeIdented(unsigned ident, std::string s);
   void cr();
	void flush();

   void displayEvent(const MetaClass *eventType /* of type EventDisplayer */, std::string name, const EventDisplayerEvent *event);
   void displayRGBAFloatImage(RGBAFloat32Image * image, std::string name);
   void displayQImage(QImage * image, std::string name);

   /// locking the output makes sure that every consecutive write to the output in a thread is protected until output is unlocked
   void lock() {x4console.lock();}
   void unlock() {x4console.unlock();}

   /// disable temporary debugging (all calls to debug output would be just ignored)
   void disable();
   void enable();

   virtual void outputText(const std::string& text) {x4console.outputText(text);}
   /// output formatted text
   virtual void outputHTML(const std::string& html) {x4console.outputHTML(html);}
   /// output anything else
   virtual void outputObject(const void* data, size_t size) {x4console.outputObject(data,size);}
};

extern X4INTERFACE_API DebugOutput debugOutput;

/// printout warning message on debug console, pop-up window if non-visible.
void X4INTERFACE_API error(std::string msg);

/// printout error message on debug console, pop-up window if non-visible.
void X4INTERFACE_API warning(std::string msg);

/// printout message on debug console, pop-up window if non-visible.
void X4INTERFACE_API message(std::string msg);

/// intermediate class for calling stuff from pointclouds project
class X4INTERFACE_API DOutput_TalkToPointCloudProject
{
public:
	virtual void displayRGBAFloatImage(RGBAFloat32Image * image, std::string name) {}
	virtual void displayQImage(QImage * image, std::string name) {}
};

extern X4INTERFACE_API DOutput_TalkToPointCloudProject * doutput_talkToPointCloudProject;



}

#endif
