//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "EventCounterDisplay.h"
#include "EventDisplayerBaseWidget.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
#include <fstream>
#include <MMSystem.h>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* EventCounterDisplay::EventCounterDisplayMetaClass = NULL;

const MetaClass* EventCounterDisplay::getInstanceClass() const {
   return getClass();
}

Object* EventCounterDisplay::newInstance() {
   return new EventCounterDisplay;
}

const MetaClass* EventCounterDisplay::getClass() {
   if (EventCounterDisplayMetaClass==NULL) {
      EventCounterDisplayMetaClass = new MetaClass("EventCounterDisplay", "no description.",
                                  newInstance, sizeof(EventCounterDisplay));
   }
   return EventCounterDisplayMetaClass;
}

void EventCounterDisplay::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   EventCounterDisplayMetaClass->copyParentProperties();
}

void EventCounterDisplay::shutdown() {
   delete EventCounterDisplayMetaClass;
}

EventCounterDisplay::EventCounterDisplay() {
	incrementCounter = 0;
   lastValue = -1234578901;
	myCounterGraph = NULL;
}

EventCounterDisplay::~EventCounterDisplay() {
}


QWidget* EventCounterDisplay::createSpecialWidget(QWidget *parent) {
	QWidget* myWidget = new QWidget(parent);

	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(myWidget);

	myCounterGraph = new PerformanceGraphWidget();
	myCounterGraph->setGraphName(baseWidget->getName());

	QVBoxLayout *layout = new QVBoxLayout;
	layout->setMargin(1);
	layout->setSpacing(0);
	layout->addWidget(myCounterGraph);
	ui.FrameGraph->setLayout(layout);

	baseWidget->setValue("");
	return myWidget;
};


void EventCounterDisplay::displayEvent(const EventDisplayerEvent *event, const char *logfile) {
	x4Assert(myCounterGraph!=NULL);
   bool doLog = false;
   int value = -1;
   {
      const EventCounterDisplay::NewValue *e = dynamic_cast<const EventCounterDisplay::NewValue *>(event);
      if (e != NULL) {
         if (dynamic_cast<const EventCounterDisplay::NewValueNoStep *>(event) != NULL) {
            incrementCounter = e->value;
         } else {
            bool update = true;
            if (dynamic_cast<const EventCounterDisplay::NewValueStepOnChange *>(event) != NULL) {
               if (e->value == lastValue) update = false;
            }
            if (update) {
	            incrementCounter = e->value;	// mh - added 04/25/2006 - hope its right
					myCounterGraph->setNextValue(e->value);
					myCounterGraph->doStep();
					if (baseWidget->isMinimized()) baseWidget->setValue(intToStr(e->value));
						else baseWidget->setValue("");
					ui.LabelValue->setNum(e->value);
					ui.LabelValue->repaint();
	            value = e->value;
               lastValue = value;
               doLog = true;
            }
         }
      }
   }
   {
      const EventCounterDisplay::Increment *e = dynamic_cast<const EventCounterDisplay::Increment *>(event);
      if (e != NULL) {
         incrementCounter++;
      }
   }
   {
      const EventCounterDisplay::Decrement *e = dynamic_cast<const EventCounterDisplay::Decrement *>(event);
      if (e != NULL) {
         incrementCounter--;
      }
   }
   {
      const EventCounterDisplay::Display *e = dynamic_cast<const EventCounterDisplay::Display *>(event);
      if (e != NULL) {
			myCounterGraph->setNextValue(incrementCounter);
			myCounterGraph->doStep();
			if (baseWidget->isMinimized()) baseWidget->setValue(intToStr(incrementCounter));
				else baseWidget->setValue("");
         ui.LabelValue->setNum(incrementCounter);
			ui.LabelValue->repaint();
         value = incrementCounter;
         lastValue = value;
         doLog = true;
         const EventCounterDisplay::DisplayAndReset *e = dynamic_cast<const EventCounterDisplay::DisplayAndReset *>(event);
         if (e != NULL) {
            incrementCounter = 0;
         }
      }
   }

   if (logfile != NULL && doLog) {
      try {
         std::ofstream out(logfile, ios_base::app);
         out << baseWidget->getName();
         out << "\tsystime\t";
         out << timeGetTime();
         out << "\tvalue\t";
         out << value << "\n";
      } catch (...) {
      }
   }

}
