//---------------------------------------------------------------------------
#ifndef FileDialogsH 
#define FileDialogsH
//---------------------------------------------------------------------------
#include "x4interface.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AttachedData.h"
//---------------------------------------------------------------------------
#include <string>
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {
	
class StandardDirectoriesConfig : public AttachedData {
	X4_CLASS(StandardDirectoriesConfig)
 public:
	std::string openDir;
	std::string saveDir;
	std::string importDir;
	std::string exportDir; 

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
};
	
/// encapsulates QT file dialogs	
class X4INTERFACE_API FileDialogs {
 public:
	/// default directories to initialize file dialog with
	/// stored in configuration (i.e. per user / computer / app)
	enum InitDirType {																															 
		CURRENT_DIR,		
		OPEN_DIR,
		SAVE_DIR,
		IMPORT_DIR,
		EXPORT_DIR
	};

 private:
	static void getDir(std::string &dir, InitDirType t);
	static void storeDir(std::string dir, InitDirType t);
	static std::string getFileNameIntern(QWidget *parent, std::string caption, 
                                   std::string filter, std::string *selectedFilter, std::string dir, 
                                   InitDirType initDir, bool open, std::string defaultName);
	
 public:
	/// get file name to open 
	/// returns "" if canceled
	/// if dir == "" the directory is initialized to the selected default location
	static std::string getOpenFileName(QWidget *parent, std::string caption, 
                                 std::string filter, std::string *selectedFilter = NULL, std::string dir = "", 
                                 InitDirType initDir = OPEN_DIR, std::string defaultName = "");
	/// get multiple file names to open 
	/// returns empty list if canceled
	/// if dir == "" the directory is initialized to the selected default location
	static std::vector<std::string> getOpenFileNames(QWidget *parent, std::string caption, 
													   std::string filter, std::string *selectedFilter = NULL, std::string dir = "", 
														InitDirType initDir = OPEN_DIR, std::string defaultName = "");
	/// get file name to open 
	/// returns "" if canceled
	/// if dir == "" the directory is initialized to the selected default location
	static std::string getSaveFileName(QWidget *parent, std::string caption, 
                                 std::string filter, std::string *selectedFilter = NULL, std::string dir = "", 
                                 InitDirType initDir = SAVE_DIR, std::string defaultName = "");

	/// get existing directory, returns empty std::string if canceled
	static std::string getExistingDirectory(QWidget *parent, const std::string &caption, const std::string &initialDir="", InitDirType initDir = CURRENT_DIR);
	
};

}

#endif