#include "StdAfx.h"
// -------------------------------------------------------------------
#include "MenuWidget.h"
// -------------------------------------------------------------------
#include <QMenu>
#include <qtCore/qstring.h>
#include <qlayout>
#include <qpushbutton>
#include <qtreewidget>
#include <qpalette>
#include <iostream>
#include "StringHelper.h"
#include "X4Assert.h"
// -------------------------------------------------------------------
using namespace X4;

MenuWidget::MenuWidget(QWidget * parent, Qt::WindowFlags f)
:QMainWindow(parent, f)
{
	connect( menuBar(), SIGNAL(triggered(QAction*)),SLOT(sltMenuHandler(QAction*)));

	QWidget * cw = new QWidget(this);
	setCentralWidget( cw );

}

MenuWidget::~MenuWidget(void)
{
}


// ---------- user functions ----------

void MenuWidget::menuItemTriggered(std::string id)
{
}

void MenuWidget::addMenuEntry(std::string id, bool checkAble)
{
	if( m_Actions.find(id) != m_Actions.end() )
		throw "Error!";
	std::vector<QString> entries = splitID( id );
	QString entry;
	QMenu * actMenu = NULL;
	for( unsigned i=0;i<entries.size()-1;i++ )
	{
		QString act_entry = entries[i];
		entry += act_entry;
		std::string entry_stl = entry.toLatin1().constData();
		if( m_Menu.find( entry_stl ) == m_Menu.end() )
		{
			if( actMenu == NULL )
				actMenu = menuBar()->addMenu( act_entry );
			else
				actMenu = actMenu->addMenu( act_entry );
			m_Menu[ entry_stl ] = actMenu;
		}
		actMenu = m_Menu[ entry_stl ];
	}

	if( entries.back() == "-" )
	{
		x4Assert( actMenu != NULL );
		actMenu->addSeparator();
		return;
	}
	if( actMenu != NULL)
	{
		QAction * action = actMenu->addAction( entries.back() );
		action->setCheckable( checkAble );
		m_Actions[ id ] = action;
		m_ActionsMap[ action ] = id;
	}
}

void MenuWidget::setMenuEntryChecked(std::string id, bool checked)
{
	if( m_Actions.find( id ) != m_Actions.end() )
	{
		m_Actions[id]->setChecked( checked );
	}
	else
		throw "error";
}

bool MenuWidget::menuEntryChecked(std::string id)
{
	if( m_Actions.find( id ) != m_Actions.end() )
	{
		return m_Actions[id]->isChecked();
	}
	else
		throw "error";
	return false;
}

void MenuWidget::setMenuEntryIcon(std::string id, QIcon &icon)
{
	if( m_Actions.find( id ) != m_Actions.end() )
	{
		m_Actions[id]->setIcon(icon);
	}
	else
		throw "error";
};


// ---------- slots ----------

void MenuWidget::sltMenuHandler( QAction * action )
{
	menuItemTriggered( m_ActionsMap[ action ] );
}


std::vector<QString> MenuWidget::splitID(std::string id)
{
	std::vector<QString> ret;

	std::vector<std::string> tmp = splitString( id, "/" );
	for( unsigned i=0;i<tmp.size();i++ )
		ret.push_back( QString( tmp[i].c_str() ) );
	
	return ret;
}

#include "MenuWidget_moc.h"
