//////////////////////////////////////////////////////////////////////////
#ifndef ModuleManagerWindowH
#define ModuleManagerWindowH
//////////////////////////////////////////////////////////////////////////

#include "X4GlobalDefinitions.h"
#include <TagList.h>
#include "x4interface/ModuleManagerWindow_ui.h"

#include <QDialog>
#include <string>
#include <vector>

//////////////////////////////////////////////////////////////////////////

class QTreeWidgetItem;

namespace X4 {


class X4INTERFACE_API ModuleManagerWindow : public QDialog {
	Q_OBJECT

 private:
	Ui_ModuleManagerWindow ui;
   struct AvailableModule {
      TagList *tags;
      std::string name;

      AvailableModule() {tags=0;}
   };
   
   std::vector<AvailableModule> availableModules;
   bool changedLOS;

   QTreeWidgetItem *createTreeWidgetItemLoaded(std::string modName, unsigned depth, bool &cyclic, bool &error);
   QTreeWidgetItem *createTreeWidgetItemAvailable(std::string modName, unsigned depth, bool &cyclic, bool &error);
   int getAvModIndex(std::string name);
   void scanForAvailableModules();
   void displayAvailableModules();
   void loadStartupModulesLoadList();
   void losTextChanged();
   void losTextUnchanged();
   std::vector<std::string> getStartupModulesList();

 protected:
   void outputError(std::string caption, std::string errMsg);
   void outputMsg(std::string caption, std::string msg);

   void updateLoaded();
   void updateAvailable();
   void updateDependenciesLoaded(std::string modName);
   void updateDescrLoaded(std::string modName);
   void updateDependenciesAvailable(std::string modName);
   void updateDescrAvailable(std::string modName);

   //void hideEvent(QHideEvent *event);

 public:
	ModuleManagerWindow(QWidget *parent=NULL, Qt::WindowFlags f=0);
	~ModuleManagerWindow(void);

   void update();
   void loadStartupModules();

 public slots:
   void on_pbClose_clicked();
   void on_pbUpdateLoaded_clicked();
   void on_pbUnloadMod_clicked();
   void on_lwModulesLoaded_clicked(const QModelIndex & index);
   void on_lwModulesAvailable_clicked(const QModelIndex & index);
   void on_twDependenciesLoaded_itemDoubleClicked(QTreeWidgetItem *item, int column);
   void on_twDependenciesAvailable_itemDoubleClicked(QTreeWidgetItem *item, int column);
   void on_pbViewDependentAvailable_clicked();
   void on_pbViewDependentLoaded_clicked();
   void on_pbUpdateAvailable_clicked();
   void on_cbTechnicalLoaded_clicked();
   void on_cbTechnicalAvailable_clicked();
   void on_pbLoadModule_clicked();
   void on_pbLOSUpdate_clicked();
   void on_pteLoadOnStartup_textChanged();
   void on_pbSaveLOS_clicked();
   void on_pbLOSall_clicked();

   //void on_pbTestModuleParser_clicked();

};


}

#endif