//---------------------------------------------------------------------------
#ifndef ObjectBrowserWidgetH
#define ObjectBrowserWidgetH
//---------------------------------------------------------------------------
#include "x4interface.h"
//---------------------------------------------------------------------------
#include "x4interface/ObjectBrowserWidget_ui.h"
//---------------------------------------------------------------------------
#include <QDialog>
#include <map>
//---------------------------------------------------------------------------

class QHBoxLayout;

namespace X4 {

class ClassEditor;
class Object;
class MetaClass;


/// a dialog that displays an object editor, with some additional options
class X4INTERFACE_API ObjectBrowserWidget : public QDialog {
   Q_OBJECT
 
 private:
   Ui_ObjectBrowserWidget ui;
   Object *obj;
   QHBoxLayout *edLayout;  
   QWidget *edWidget;
   ClassEditor *clEd;

	QAction	*	m_SaveObjectAction;
	QAction	*	m_UpdateAction;
	QMenu		*		m_Menu;
	QMenu		*		m_EditorsMenu;

	struct TMetaInfo
	{
		MetaClass * mc;
		Object	*	obj;
		std::string		name;
		bool			isDefault;
	};
	std::map<QAction*, TMetaInfo >	m_EditorClasses;
   
 protected:
	void keyPressEvent(QKeyEvent *event);

 public:
   /// create object specifying the object to be edited and the parent widget this dialog is attached to
   ObjectBrowserWidget(Object *obj, ClassEditor *clEd, QWidget *parent = 0, Qt::WindowFlags f = 0);
   /// after creating the widget, this method can be called to obtain the reference to the container the new editor is to be placed into
   QWidget *getEditorContainer();
   /// and then, setupEditor must be called on the widget to layout it correctly and register it for editor exchange
   void setupEditor(QWidget *ed);
   
 public slots:
   void sltHandleMenu(QAction *action);
   void on_editButton_clicked();
   void sltUpdate(bool x);
   
};


}

#endif