//---------------------------------------------------------------------------
#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "PerformanceGraphWidget.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


PerformanceGraphWidget::PerformanceGraphWidget(QWidget* parent)
:QWidget(parent)
{
	gridSize = 10;
	minValue = 0;
	maxValue = 10;
	legendOffset = 10;
	graphName = "name";
	nextValue = 0;
	stepSize = 3;

	widgetSize = QSize(40,40);
	backgroundColor = QColor(10,30,10);
	gridColor = QColor(0,120,35);
	legendColor = QColor(0,240,0);
	legend2Color = QColor(250,250,0);
	graphColor = QColor(5,255,5);

	// set background color
	QPalette pal = palette();
	pal.setColor(QPalette::Window, backgroundColor);
	setPalette(pal);
	setBackgroundRole(QPalette::Window);
	setAutoFillBackground(true);
}

PerformanceGraphWidget::~PerformanceGraphWidget() {
}


QSize PerformanceGraphWidget::minimumSizeHint() const {
	return widgetSize;
}

QSize PerformanceGraphWidget::sizeHint() const {
	return widgetSize;
}


void PerformanceGraphWidget::drawGrid(QPainter &painter) {
	card32 w = width();
	card32 h = height();
	painter.setPen(gridColor);
	// draw horizontal lines
	for (int32 i=h; i>0; i-=gridSize) {
		painter.drawLine(legendOffset,i,w,i);
	}
	// draw vertical lines
	for (card32 j=legendOffset; j<w; j+=gridSize) {
		painter.drawLine(j,0,j,h);
	}
	// draw first line stronger
	painter.setPen(legendColor);
	painter.drawLine(legendOffset-1, 0, legendOffset-1, h);
}


void PerformanceGraphWidget::drawLegend(QPainter &painter) {
	// set font
	QFont f = painter.font();
	f.setPointSize(7);
	painter.setFont(f);
	painter.setPen(legendColor);
	// estimate text size
	QRect r = painter.boundingRect(QRect(0,0,10,10), Qt::TextSingleLine | Qt::AlignRight, QString().setNum(maxValue));
	if ((card32)(r.width()+6)>legendOffset) 
		legendOffset = r.width()+6;
	// draw legend text right-aligned towards graph
	painter.drawText(0, 0, legendOffset-3, r.height(), Qt::TextSingleLine | Qt::AlignRight, QString().setNum(maxValue));
	painter.drawText(0, height()-r.height(), legendOffset-3, r.height(), Qt::TextSingleLine | Qt::TextDontClip | Qt::AlignRight, QString().setNum(minValue));
	// draw legend2 text
	painter.setPen(legend2Color);
	r = painter.boundingRect(QRect(0,0,10,10), Qt::TextSingleLine|Qt::AlignLeft, QString(graphName.c_str()));
	painter.drawText(legendOffset+3, 0, r.width(), r.height(), Qt::TextSingleLine | Qt::AlignLeft, QString(graphName.c_str()));
}


void PerformanceGraphWidget::drawGraph(QPainter &painter) {
	card32 w = width();
	card32 h = height();
	// scale graph height according to window height and maxValue
	float32 scaleY = ((float32)h)/((float32)maxValue);
	// set pen
	painter.setPen(QPen(QBrush(graphColor), 1.2f));
	// paint graph
	QPointF p1, p2;
	int32 prevXPos;
	card32 numPoints = (card32)graphPoints.size();
	for (card32 i=1; i<numPoints; i++) {
		prevXPos = w - i*stepSize;
		if (prevXPos<(int32)legendOffset) {
			// left border clipping
			float32 deltaY = (((float32)graphPoints[numPoints-i]-(float32)graphPoints[numPoints-i-1])/(float32)stepSize) * ((float32)legendOffset-prevXPos);
			p1 = QPointF(legendOffset, h-((graphPoints[numPoints-i-1]+deltaY)*scaleY));
			p2 = QPointF(prevXPos+stepSize, h-(graphPoints[numPoints-i]*scaleY));
			painter.drawLine(p1, p2);
			// last visible point so break for loop
			break;
		}
		// draw line p(i-1) to p(i)
		p1 = QPointF(prevXPos, h-(graphPoints[numPoints-i-1]*scaleY));
		p2 = QPointF(prevXPos+stepSize, h-(graphPoints[numPoints-i]*scaleY));
		painter.drawLine(p1, p2);
	}
}


void PerformanceGraphWidget::paintEvent(QPaintEvent* event) {
	QPainter painter(this);
	drawGrid(painter);
	painter.setRenderHint(QPainter::TextAntialiasing);
	drawLegend(painter);
	painter.setRenderHint(QPainter::Antialiasing);
	drawGraph(painter);
}



// ---------- slots ----------


#include "PerformanceGraphWidget_moc.h"
