/***************************************************************************
*   Copyright (c) 2009   Art Tevs                                         *
*                                                                         *
*   This library is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation; either version 3 of        *
*   the License, or (at your option) any later version.                   *
*                                                                         *
*   This library is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU Lesse General Public License for more details.                    *
*                                                                         *
*   The full license is in LICENSE file included with this distribution.  *
***************************************************************************/

//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ProgressFrameWidget.h"
//---------------------------------------------------------------------------
#include "UserSettings.h"
#include "Configuration.h"
#include <ctime>
#include <qtcore/QTime>
//---------------------------------------------------------------------------
using namespace X4;

//---------------------------------------------------------------------------
ProgressFrameWidget::ProgressFrameWidget(QWidget *parent) : ProgressWidget(parent)
{
    ui.setupUi(this);
    this->setWindowFlags(Qt::Widget | Qt::FramelessWindowHint);

    QObject::connect(ui.cancelButton, SIGNAL(clicked(bool)), this, SLOT(cancel_clicked(bool)));
}

//---------------------------------------------------------------------------
ProgressFrameWidget::~ProgressFrameWidget(void)
{
}

//---------------------------------------------------------------------------
void ProgressFrameWidget::cancel_clicked(bool)
{
    emit progressCanceled();
}

#include <QtCore/QTimer>
//---------------------------------------------------------------------------
void ProgressFrameWidget::showForm()
{
    // if visible, then do nothing
    if (isVisible()) return;

    // update 04/12/2007: show of ProgressWindow could be enabled/disabled in UserSettings now
    bool doShow = true;
    UserSettings * u = dynamic_cast<UserSettings*>( Configuration::getConfiguration()->getData( "USER_SETTINGS" ) );
    if( u )
    {
        doShow = u->getShowProgressWindow();
    }
    if (doShow)
    {
        emit progressShowed();
        show();
	}
};

//---------------------------------------------------------------------------
void ProgressFrameWidget::setToolStartTime(time_t timeInSeconds)
{
	if (timeInSeconds == 0)
	{
		ui.ToolStartTimeLabel->setText(QString("n.a."));
		return;
	}
	struct tm * timeinfo;
	timeinfo = localtime(&timeInSeconds);
	toolTimeStartedInSeconds = timeInSeconds;

	QTime qtime(timeinfo->tm_hour, timeinfo->tm_min, timeinfo->tm_sec);
	ui.ToolStartTimeLabel->setText(qtime.toString("hh:mm:ss"));

}

//---------------------------------------------------------------------------
void ProgressFrameWidget::setToolEstimatedTime(card64 estTimeInSeconds, time_t currentTimeInSeconds)
{
	card64 seconds = estTimeInSeconds % 60;
	card64 minutes = (estTimeInSeconds / 60) % 60;
	card64 hours = (estTimeInSeconds/(60*60));

	// setup estimated running time
	if (estTimeInSeconds == 0)
	{
		ui.ToolEstimatedTimeLabel->setText(QString("n.a."));
	}else if (estTimeInSeconds < 60)
	{
		ui.ToolEstimatedTimeLabel->setText(QString::number(seconds) + QString("s"));
	}else if (estTimeInSeconds < 60 * 60)
	{
		ui.ToolEstimatedTimeLabel->setText(QString::number(minutes) + QString("m") 
			+ QString::number(seconds) + QString("s"));
	}else
	{
		ui.ToolEstimatedTimeLabel->setText(QString::number(hours) + QString("h") 
			+ QString::number(minutes) + QString("m") 
			+ QString::number(seconds) + QString("s"));
	}

	// setup estimated remaining time
	//struct tm * endTimeInfo;
	time_t endtime = static_cast<time_t>(estTimeInSeconds) - (currentTimeInSeconds - toolTimeStartedInSeconds);

	// compute estimated end time
	QTime qtime(endtime / (60*60), (endtime / 60) % 60, endtime % 60);
	ui.ToolEstimatedRemainingTimeLabel->setText(qtime.toString("hh:mm:ss"));
}

//---------------------------------------------------------------------------
void ProgressFrameWidget::setToolProgress(card32 position)
{
    ui.ToolProgress->setValue(position);
};

//---------------------------------------------------------------------------
void ProgressFrameWidget::setOverallProgress(card32 position)
{
    ui.OverallProgress->setValue(position);
};

//---------------------------------------------------------------------------
void ProgressFrameWidget::setToolProgressAvailable(bool available)
{
    ui.ToolProgress->setEnabled(available);
};

//---------------------------------------------------------------------------
void ProgressFrameWidget::setOverallProgressAvailable(bool available)
{
    ui.OverallProgress->setEnabled(available);
};

//---------------------------------------------------------------------------
void ProgressFrameWidget::setToolName(const std::string& name)
{
    if (!name.empty())
        ui.ToolBox->setText(QString((name + " progress: ").c_str()));
    else
        ui.ToolBox->setText(QString( "Tool progress: "));

};

//---------------------------------------------------------------------------
void ProgressFrameWidget::setToolMessage(const std::string& msg)
{
	ui.MessageLabel->setText(QString( msg.c_str())); 
};

//---------------------------------------------------------------------------
void ProgressFrameWidget::drawUpdates()
{
    update();
	updateGeometry();
};

//---------------------------------------------------------------------------
void ProgressFrameWidget::closeForm()
{
    if (isVisible() == false) return;

    emit progressHided();
    hide();
};



#include "ProgressFrameWidget_moc.h"

