//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ProgressWindowWidget.h"
//---------------------------------------------------------------------------
#include "UserSettings.h"
#include "Configuration.h"
#include <ctime>
#include <qtcore/QTime>
//---------------------------------------------------------------------------
using namespace X4;

//---------------------------------------------------------------------------
ProgressWindowWidget *X4::progressWindowWidget = NULL;

ProgressWindowWidget::ProgressWindowWidget(QWidget *parent) : ProgressWidget(parent)
{
	toolTimeStartedInSeconds = 0;
	ui.setupUi(this);
    this->setWindowFlags(Qt::Tool | Qt::WindowStaysOnTopHint);
}

ProgressWindowWidget::~ProgressWindowWidget(void)
{
}


void ProgressWindowWidget::showForm()
{

// changed 08/08/2006 (mh), hopefully works on multi-monitor systems now
//	int sWidth = qApp->desktop()->screen()->width();
//	int sHeight = qApp->desktop()->screen()->height();

	int sWidth = qApp->desktop()->screenGeometry(this).width();
	int sHeight = qApp->desktop()->screenGeometry(this).height();

	int left = sWidth - width() - 50;
	int top = sHeight - height() - 80;

	move(left, top);

// update 04/12/2007: show of ProgressWindow could be enabled/disabled in UserSettings now
	bool doShow = true;
	UserSettings * u = dynamic_cast<UserSettings*>( Configuration::getConfiguration()->getData( "USER_SETTINGS" ) );
	if( u )
	{
		doShow = u->getShowProgressWindow();
	}
	if (doShow) show();

};

void ProgressWindowWidget::setToolStartTime(time_t timeInSeconds)
{
	if (timeInSeconds == 0)
	{
		ui.ToolStartTimeLabel->setText(QString("n.a."));
		return;
	}
	struct tm * timeinfo;
	timeinfo = localtime(&timeInSeconds);
	toolTimeStartedInSeconds = timeInSeconds;

	QTime qtime(timeinfo->tm_hour, timeinfo->tm_min, timeinfo->tm_sec);
	ui.ToolStartTimeLabel->setText(qtime.toString("hh:mm:ss"));
};

void ProgressWindowWidget::setToolEstimatedTime(card64 estTimeInSeconds, time_t currentTimeInSeconds)
{
	card64 seconds = estTimeInSeconds % 60;
	card64 minutes = (estTimeInSeconds / 60) % 60;
	card64 hours = (estTimeInSeconds/(60*60));

	// setup estimated running time
	if (estTimeInSeconds == 0)
	{
		ui.ToolEstimatedTimeLabel->setText(QString("n.a."));
	}else if (estTimeInSeconds < 60)
	{
		ui.ToolEstimatedTimeLabel->setText(QString::number(seconds) + QString("s"));
	}else if (estTimeInSeconds < 60 * 60)
	{
		ui.ToolEstimatedTimeLabel->setText(QString::number(minutes) + QString("m") 
			+ QString::number(seconds) + QString("s"));
	}else
	{
		ui.ToolEstimatedTimeLabel->setText(QString::number(hours) + QString("h") 
			+ QString::number(minutes) + QString("m") 
			+ QString::number(seconds) + QString("s"));
	}

	// setup estimated remaining time
//	struct tm * endTimeInfo;
	time_t endtime = static_cast<time_t>(estTimeInSeconds) - (currentTimeInSeconds - toolTimeStartedInSeconds);

	// compute estimated end time
	QTime qtime(endtime / (60*60), (endtime / 60) % 60, endtime % 60);
	ui.ToolEstimatedRemainingTimeLabel->setText(qtime.toString("hh:mm:ss"));
}

void ProgressWindowWidget::setToolProgress(card32 position)
{
	ui.ToolProgress->setValue(position);
};

void ProgressWindowWidget::setOverallProgress(card32 position)
{
	ui.OverallProgress->setValue(position);
};

void ProgressWindowWidget::setToolProgressAvailable(bool available)
{
	
	ui.ToolProgress->setEnabled(available);

};

void ProgressWindowWidget::setOverallProgressAvailable(bool available)
{
	ui.OverallProgress->setEnabled(available);
};

void ProgressWindowWidget::setToolName(const std::string& name)
{
	if (!name.empty())
		ui.ToolBox->setTitle(QString((name + " progress: ").c_str()));
	else
		ui.ToolBox->setTitle(QString( "Tool progress: "));

};

void ProgressWindowWidget::setToolMessage(const std::string& msg)
{
	ui.MessageLabel->setText(QString( msg.c_str())); 
};
void ProgressWindowWidget::drawUpdates()
{
	update();
};
void ProgressWindowWidget::closeForm()
{
	hide();
};

#include "ProgressWindowWidget_moc.h"
