//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "StandardTimeController.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
#include "BooleanClassProperty.h"
//---------------------------------------------------------------------------


using namespace X4;


MetaClass* StandardTimeController::StandardTimeControllerMetaClass = NULL;

const MetaClass* StandardTimeController::getInstanceClass() const {
   return getClass();
}

Object* StandardTimeController::newInstance() {
   return new StandardTimeController;
}

const MetaClass* StandardTimeController::getClass() {
   if (StandardTimeControllerMetaClass==NULL) {
      StandardTimeControllerMetaClass = new MetaClass("StandardTimeController", "no description.",
                                  newInstance, sizeof(StandardTimeController));
   }
   return StandardTimeControllerMetaClass;
}

void StandardTimeController::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   StandardTimeControllerMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT( StandardTimeController );
	ADD_FLOAT32_PROP( startTime, 0 );
	ADD_FLOAT32_PROP( endTime, 0 );
	ADD_FLOAT32_PROP( increment, 0 );
	ADD_BOOLEAN_PROP( cyclic, 0 );
	ADD_BOOLEAN_PROP( followRealTime, 0 );
	ADD_FLOAT32_PROP( realTimeFactor, 0 );
	ADD_BOOLEAN_PROP( limitRTDelta, 0 );
	ADD_FLOAT32_PROP( maxDelta, 0 );

}

void StandardTimeController::shutdown() {
   delete StandardTimeControllerMetaClass;
}


// ----


StandardTimeController::StandardTimeController() {
   startTime = 0.0f;
   endTime = 1.0f;
   increment = 0.05f;
   cyclic = true;
   followRealTime = false;
   realTimeFactor = 1.0f;
   limitRTDelta = true;
   maxDelta = 1.0f;
}

float32 StandardTimeController::getNewTime(float32 lastTime, float32 deltaTReal) {
   float32 result = lastTime;
   if (limitRTDelta && deltaTReal > maxDelta) deltaTReal = maxDelta;
   if (followRealTime) {
      result += deltaTReal * realTimeFactor;
   } else {
      result += increment;
   }
   if (cyclic && endTime > startTime && result > endTime) {
      float32 diff = result - startTime;
      diff = fmod(diff, endTime-startTime);
      result = startTime + diff;
   }
   return result;
}
