//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ColorTools.h"
//---------------------------------------------------------------------------
#include <qtGUI/QColor>
#include <algorithm>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


Vector3f X4MATH_API X4::convertRGBtoHSV( const Vector3f &rgb )
{
	QColor col(rgb[0]*255.0f,rgb[1]*255.0f,rgb[2]*255.0f);
	int h,s,v;
	col.getHsv(&h,&s,&v);
	return makeVector3f(h,s,v) / 255.0f;
}

Vector3f X4MATH_API X4::convertHSVtoRGB( const Vector3f &hsv )
{
	QColor col;
	col.setHsv(hsv[0]*255.0f,hsv[1]*255.0f,hsv[2]*255.0f);
	return makeVector3f(col.red(),col.green(),col.blue()) / 255.0f;
}


void X4::ColorGradient::addColor( const Vector3f &color, float t )
{
	TColor c;
	c.color = color;
	c.t = min(1.0f,max(0.0f,t));
	m_Colors.push_back(c);
	sort(m_Colors.begin(),m_Colors.end());
	m_Table.clear();
}

void X4::ColorGradient::setupTable( mpcard numEntries/*=1000*/ )
{
	m_Table = std::vector<Vector3f>(numEntries);
	for( mpcard i=0;i<numEntries;i++)
	{
		m_Table[i] = getColor( (float)i/(float)(numEntries-1));
	}
}


X4::Vector3f X4::ColorGradient::getColor( float t ) const
{
	if( m_Colors.size() == 0 )
		return NULL_VECTOR3F;
	if( m_Colors.size() == 1)
		return m_Colors.front().color;
	
	// binary search for color range
	t = min(1.0f,max(0.0f,t));
	mpcard istart = 0;
	mpcard istop = m_Colors.size()-1;
	float tstart = m_Colors[istart].t;
	float tstop = m_Colors[istop].t;
	bool report = rand() % 100 == 0;
	while (istart+1<istop)
	{
		mpcard imiddle = (istart+istop)/2;
		float tmiddle = m_Colors[imiddle].t;
		if( tmiddle < t)
		{
			istart = imiddle;
			tstart = tmiddle;
		}
		else
		{
			istop = imiddle;
			tstop = tmiddle;
		}
	}
	float interval = tstop-tstart;
	if( interval < 1e-10f)
		return m_Colors[istart].color;
	float blendfactor = (t-tstart)/interval;
	return m_Colors[istart].color*(1.0f-blendfactor)+m_Colors[istop].color*blendfactor;
}

X4::Vector3f X4::ColorGradient::getColorFromTable( float t )
{
	if(m_Table.size()== 0)
		setupTable();
	return m_Table[ min((int)m_Table.size()-1,max(0,(int)(t*(m_Table.size()-1)))) ];
}