//---------------------------------------------------------------------------
#ifndef DynamicLinearAlgebraToolsH
#define DynamicLinearAlgebraToolsH
//---------------------------------------------------------------------------
#include "x4math.h"
//---------------------------------------------------------------------------
#include "DynamicLinearAlgebra.h"
//---------------------------------------------------------------------------
#include "random.h"
//---------------------------------------------------------------------------

namespace X4 {

/// Calc fast Covariance Matrix and SVD on DynamicVectors using OpenCV
/// currently instantiated explicitly - allows only float32 and float64
template <typename FloatType>
class X4MATH_API DynamicPCA {

 private:
   DynamicVector<FloatType> * dataPointer;
   card32 numberOfVecs;
   card32 dimension;
   int type; //OpenCV DataType eg.  float32 = CV_32FC1

	DynamicMatrix<FloatType>	eigenVectors;
	DynamicVector<FloatType> userDefinedMean;
	bool useUserDefinedMean;
	DynamicVector<FloatType>	centroid;

 public:

   DynamicPCA(DynamicVector<FloatType> * dataPointer, card32 numberOfVecs);
   DynamicPCA();

   static const int getType();

   DynamicVector<FloatType> calcMean();
   /// this method can be called to replace average by user defined std::vector (this is not used in conventional PCA)
   void setUserDefinedMean(const DynamicVector<FloatType> &userMean);
   
	/// [mw 10/18/06] parameter useU: if set to false, the columns of the matrix contain the eigenvectors (each column is one ev)
	DynamicVector<FloatType> calcEigenValues(bool useU=true);
	
	/// returns eigenvectors calculated in calcEigenValues()
	DynamicMatrix<FloatType> getEigenVectors() {return eigenVectors;}

	/// returns centroid calculated in calcEigenValues()
	DynamicVector<FloatType> getCentroid() {return centroid;}

	void calcCovarMatrix(DynamicMatrix<FloatType> & target);
   void calcCovarMatrixB(DynamicMatrix<FloatType> & target);

   static void computeSVD(const DynamicMatrix<FloatType> &input,
      DynamicMatrix<FloatType> &u,
      DynamicVector<FloatType> &singularValues,
      DynamicMatrix<FloatType> &v,
	   bool transposeU = false,
		bool transposeV = false
   );
};


template <typename FloatType>
void calcBoundingBox(const std::vector< DynamicVector<FloatType> > &data, DynamicVector<FloatType> &lower, DynamicVector<FloatType> &upper);

/// computes a random std::vector with uniform, independent ("rand()") entries from the range [0..1]
template <typename FloatType>
void randomDVector(DynamicVector<FloatType> &v, card32 dim);

/// computes a random std::vector with uniform, independent ("rand()") entries from the range [-1..1]
template <typename FloatType>
void randomSignedDVector(DynamicVector<FloatType> &v, card32 dim);

template <typename VectorTypeSource, typename VectorTypeDest>
VectorTypeDest convertVectorType(const VectorTypeSource &source, unsigned dim);

template <typename VectorTypeSource, typename VectorTypeDest>
void convertVectorType(const VectorTypeSource &source, VectorTypeDest &dest, unsigned dim);

template <typename MatrixTypeSource, typename MatrixTypeDest>
MatrixTypeDest convertMatrixType(const MatrixTypeSource &source, unsigned cols, unsigned rows);

template <typename MatrixTypeSource, typename MatrixTypeDest>
void convertMatrixType(const MatrixTypeSource &source, MatrixTypeDest &dest, unsigned cols, unsigned rows);

}

#endif
