//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "Plane3f.h"
#include "Line3f.h"
#include "Plane3f.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( Line3f, 0)
{
	BEGIN_CLASS_INIT( Line3f );

	ADD_VECTOR3F_PROP(point, 0);
	ADD_VECTOR3F_PROP(dir, 0);
}

Line3f::Line3f()
{
	clear();
}

Line3f::Line3f( Vector3f d, Vector3f p )
{ 
	clear();
	dir = d;
	point = p; 
}

Line3f::~Line3f()
{
}

void Line3f::clear()
{
	for ( unsigned int n = 0; n < 3; n++ ){
		point[n] = 0;
		dir[n] = 0;
	}
}
float Line3f::computeDistance(const Vector3f &p) const 
{
	Vector3f dir_n = dir/norm(dir);
	float t = (p-point)*dir_n;
	return norm(point + dir_n*t - p);
}

float Line3f::computeProjection( const Vector3f &p) const 
{
	return (p-point)*dir/normQuad(dir);
}

float Line3f::computeMinDistance( const Line3f &line0, const Line3f &line1, float &t0, float &t1 )
{
	if( fabs(normalize(line0.getDirection())*normalize(line1.getDirection())) > 0.99f )
	{
		throw X4Exception("computeMinDistance() - lines are parallel!");
	}

	// solve: min (line0(t0)-line1(t1))^2
	const Vector3f d0 = line0.getDirection();
	const Vector3f d1 = line1.getDirection();
	const Vector3f diff = line0.getPoint()-line1.getPoint();
	Matrix2f M;	Vector2f rhs;

	M[0][0] = d0*d0;
	M[0][1] = -d0*d1;
	M[1][0] = d0*d1;
	M[1][1] = -d1*d1;
	rhs[0] = -diff * d0;
	rhs[1] = -diff * d1;

	Vector2f sol;
	linsolve2f(M,rhs,sol);
	t0 = sol[0];
	t1 = sol[1];
	return norm(line0.getPoint() + d0 * t0 - line1.getPoint()-d1*t1);
}

/**
 * Projection of a point onto a line
 */
void X4::projectPointLine( const Line3f& line, const Vector3f& point, Vector3f& projectedPoint )
{
	Vector3f normal = line.getDirection();
	normal = normalize( normal );
	Plane3f plane = Plane3f( line.getDirection(), point );
	intersectPlaneLine( plane, line, projectedPoint );
}
