#ifndef Line3f_H
#define Line3f_H
//---------------------------------------------------------------------------
#include "x4math.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{

/// PS_INFRONT: entire object is on the positive side of plane
/// PS_BEHIND: entire object is on the negative side of plane
/// PS_COINCIDING: entire object is on the plane
/// PS_SPANNING: plane splits object into two parts
#ifndef EPLANESIDE
#define EPLANESIDE
enum EPlaneSide { PS_INFRONT = 0, PS_BEHIND = 1, PS_COINCIDING = 2, PS_SPANNING = 3};
#endif

/// Container for a plane defined by a normal and one point within the plane.
class X4MATH_API Line3f : public Persistent
{
	X4_CLASS(Line3f);
	protected:
		Vector3f point;
		Vector3f dir;

	public:
		Line3f();
		Line3f( Vector3f d, Vector3f p );
		~Line3f();
		Line3f( const Line3f &other ) { dir = other.getDirection(); point = other.getPoint(); }
		
		void operator = ( const Line3f &other ){ dir = other.getDirection(); point = other.getPoint(); }
		void clear();

		inline Vector3f getPoint() const { return point; }
		inline Vector3f getDirection() const { return dir; }
		inline void setDirection( Vector3f d ){ dir = d; }
		inline void setPoint( Vector3f p ){ point = p; }
		inline float length() const {return norm(dir);}
		
		/// compute distance to line
        float computeDistance(const Vector3f &point) const;

        /// compute projection onto line segment
        float computeProjection( const Vector3f &point) const;

        /// compute distance between two non-parallel lines
        /// returns closest points as parameter values t0,t1
        static float computeMinDistance( const Line3f &line0, const Line3f &line1, float &t0, float &t1);
        static float computeMinDistance( const Line3f &line0, const Line3f &line1){float a,b; return computeMinDistance(line0,line1,a,b);}
};

/// Projection of a point onto a line
extern X4MATH_API void projectPointLine( const Line3f& line, const Vector3f& point, Vector3f& projectedPoint );


}

#endif
