//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "NormalDistribution.h"
//------------------------------------------------------------------------------
#include "LinearAlgebra.inline.h"
#include <ConsoleOutput.h>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

/**
 * Normal random variate generator by mean and variance
 */
float32 NormalDistribution::randomNumber( const float32 mean, const float32 stdDeviation )
{
	float32 v1, v2, w, z1;
   float32 z2 = 0.0; 
	int use_z2 = 1;

	/// use value from previous call
	if ( use_z2 && z2 != 0.0 ){
		z1 = z2;
		z2 = 0.0;
	} else {
		do {
			v1 = 2.0 * ( rand()%1000/1000.0f ) - 1.0;
			v2 = 2.0 * ( rand()%1000/1000.0f ) - 1.0;
			w = v1 * v1 + v2 * v2;
		} while ( w >= 1.0 || w < 1e-15f);

		w = sqrt( (-2.0 * log( w ) ) / w );
		z1 = v1 * w;
		z2 = v2 * w;
	}
	return( mean + z1 * stdDeviation );
}

float X4::NormalDistribution::expMultiplier;
float * X4::NormalDistribution::expTable;


void X4::NormalDistribution::initExpTable()
{
	expTable = new float[expTableSize];
	float expRange = 9.0f;
	
	for( int x=0;x<expTableSize;x++ )
	{
		float xf = (float)x / expTableSize * expRange;
		expTable[x] = exp(-xf);
	}

	expMultiplier = (float)expTableSize / expRange;
}

void X4::NormalDistribution::clearExpTable()
{
	delete[] expTable;
}