//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "ParametricPlane.h"
//---------------------------------------------------------------------------
using namespace X4;

/**
 * Create new object, init members
 */
ParametricPlane::ParametricPlane()
{
	clear();
}

/**
 * Create new object, from initial values
 */
ParametricPlane::ParametricPlane( Vector3f n, Vector3f p, Vector3f u0, Vector3f v0 )
{
	clear();
	normal = n;
	point = p;
	u = u0;
	v = v0;
}

/**
 * Delete object
 */
ParametricPlane::~ParametricPlane()
{
	clear();
}

/**
 * Reset all members to the initial state
 */
void ParametricPlane::clear()
{
	bounded = false;
	clearIntervals();
	u[0] = u[1] = u[2] = 0;
    v[0] = v[1] = v[2] = 0;
    Plane::clear();
}

/**
 * Test if a point 'p' is within the interval region
 */
bool ParametricPlane::isInIntervalRegion( Vector3f p )
{
	float s,t;
	computeParameters( p, s, t );
	return ( s >= s0 ) && ( s <= s1 ) && ( t >= t0 ) && ( t <= t1 );
}

/**
 * Solve linear system in order to find parameters of point in plane
 * Point p is assumed to lie in the plane!
 * Return true, if computation was successful.
 */
bool ParametricPlane::computeParameters( Vector3f p, float& s, float& t )
{
	s = t = 0;
	Matrix2f A;
	Vector2f x;
	Vector2f b;

	A = makeMatrix2f( u[0], v[0], u[1], v[1] );
	if ( fabs( A.getDeterminant() ) > 1e-5 ){
		b = makeVector2f( p[0] - point[0], p[1] - point[1] );
	} else {
		A = makeMatrix2f( u[0], v[0], u[2], v[2] );
		if ( fabs( A.getDeterminant() ) > 1e-5 ){
			b = makeVector2f( p[0] - point[0], p[2] - point[2] );
		} else {
			A = makeMatrix2f( u[1], v[1], u[2], v[2] );
			if ( fabs( A.getDeterminant() ) > 1e-5 ){
				b = makeVector2f( p[1] - point[1], p[2] - point[2] );
			} else {
				std::cout << "Point is not on plane of spanning vectors are invalid!" << endl;
				return false;
			}
		}
	}

	linsolve2f( A, b, x );
	s = x[0];
	t = x[1];
	return true;
}


/**
 * Add interval point (adjust intervals to contains new point)
 */
void ParametricPlane::addIntervalPoint( Vector3f p )
{
	float s, t;
	computeParameters( p, s, t );
	if ( s < s0 ) s0 = s;
	if ( s > s1 ) s1 = s;
	if ( t < t0 ) t0 = t;
	if ( t > t1 ) t1 = t;
}
