#ifndef PARAMETRIC_PLANE_H
#define PARAMETRIC_PLANE_H

// CGTool
#include "MathTools.h"
#include "MathTools.inline.h"
#include "Plane.h"

namespace X4
{

/// Container for a plane that is parameterized other to linear independent vectors.
class X4MATH_API ParametricPlane : public Plane
{
	protected:
		/// First major plane direction
		Vector3f u;
		/// Second major plane direction
		Vector3f v;
		/// Set flag if plane has bounded parameter domain
		bool bounded;
		/// Boundaries of the bounded parameterization
		float s0;
		/// Boundaries of the bounded parameterization
		float s1;
		/// Boundaries of the bounded parameterization
		float t0;
		/// Boundaries of the bounded parameterization
		float t1;

	public:
		/// Constructor
		ParametricPlane();
		/// Constructor (normal, point, u, v)
		ParametricPlane( Vector3f n, Vector3f p, Vector3f u0, Vector3f v0 );
		/// Destructor
		~ParametricPlane();
		/// Clear members
		void clear();
      /// Test if point p is in interval region. Point has to lie within the plane
      bool isInIntervalRegion( Vector3f p );
		/// Compute parameters in the plane domain
		bool computeParameters( Vector3f p, float& s, float& t );
		/// Getter: u
		inline Vector3f getU() const { return u; }
		/// Getter: v
		inline Vector3f getV() const { return v; }
		/// Setter: u
		inline void setU( Vector3f newU ){ u = newU; u.normalize(); }
		/// Setter: v
		inline void setV( Vector3f newV ){ v = newV; v.normalize(); }
		/// Getter: parametric intervals
		inline void getParamIntervals( float& sMin, float& sMax, float& tMin, float& tMax ){ sMin = s0; sMax = s1; tMin = t0; tMax = t1; }
		/// Setter: parametric intervals
		inline void setParamIntervals( float sMin, float sMax, float tMin, float tMax ){ s0 = sMin; s1 = sMax; t0 = tMin; t1 = tMax; }
		/// Clear intervals
		inline void clearIntervals(){ s0 = s1 = t0 = t1 = 0; }
		/// Add interval point (adjust intervals to contains new point). Point has to lie in plane!
		void addIntervalPoint( Vector3f p );
	  
};

}

#endif
