#ifndef Plane3fH
#define Plane3fH
//---------------------------------------------------------------------------
#include "x4math.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include <Line3f.h>
#include <Persistent.h>
//---------------------------------------------------------------------------

namespace X4
{

    class Line3f;

    /// Container for a plane defined by a normal and one point within the plane (in 3D, float32 precision).
    class X4MATH_API Plane3f : public Persistent
    {
        X4_CLASS( Plane3f )
    public:
        Vector3f point;
        Vector3f normal;

        Plane3f();
        Plane3f( Vector3f n, Vector3f p );
        ~Plane3f();
        inline void setup( Vector3f n, Vector3f p ){ normal = n; point = p; }
        void clear();

        // 3D only
        EPlaneSide classify( const Line3f &line, float epsilon = 1e-20f ) const;
        void split( const Line3f &line, Line3f &positive, Line3f &negative ) const;
        Vector3f calculateIntersection( const Line3f &plane ) const;
        Line3f calculateIntersection(const Plane3f& plane) const;

        inline void operator=( const Plane3f& p ){ normal = p.getNormal(); point = p.getPoint(); }
        float calculateSignedDistance( const Vector3f &p ) const;
        float calculateLineIntersection( const Vector3f &a, const Vector3f &b ) const;

        Vector3f calculateMirrorPoint( const Vector3f &p);
        Vector3f projectPointOntoPlane( const Vector3f &p) const;

        Plane3f transform(const Matrix4f &T) const;
        bool equal(const Plane3f &p, float angle_threshold, float spatial_threshold) const;

        Plane3f flip() const {
            return Plane3f( -normal, point );			  
        }

        void movePoint( const float& dist );

        /*** MEMBER ACCESS ***/
        float getDistanceFromPlane( const Vector3f p ) const;
        inline Vector3f getPoint() const { return point; }
        inline Vector3f getNormal() const { return normal; }
        inline float getDistance() const { return normal * point; }
        inline void setNormal( Vector3f n ){ normal = n; normal.normalize(); }
        inline void setPoint( Vector3f p ){ point = p; }
    };


    /// Intersection between tweo planes; return true, if possible
    extern X4MATH_API bool intersectPlanePlane( const Plane3f plane1, const Plane3f plane2, Line3f& intersectionLine );

    /// Intersection between plane and line; return true if possible
    extern X4MATH_API bool intersectPlaneLine( const Plane3f& plane, const Line3f& line, Vector3f& intersectionPoint );

    /// Projection of a point onto a plane
    extern X4MATH_API void projectPointPlane( const Plane3f& plane, const Vector3f point, Vector3f& projectedPoint );
}

#endif
