//---------------------------------------------------------------------------
#ifndef RandomH
#define RandomH
//---------------------------------------------------------------------------
#include "x4math.h"
#include "X4Types.h"
//---------------------------------------------------------------------------
#include <stdlib.h>
//---------------------------------------------------------------------------

namespace X4 {

inline float32 rnd01() {
#ifdef X4_PLATFORM_MSVC
   return (float32)qrand() / ((float32)(RAND_MAX+1));
#endif
}

inline card32 randomNumber(card32 maxNumberPlusOne) {
#ifdef X4_PLATFORM_MSVC
	float32 ret = rnd01() * (float32)maxNumberPlusOne;
	return (card32)ret;
#endif
}

inline card32 random32bit(card32 maxNumberPlusOne = 0xffffffff)
{
	// rand() produces a pseudo random number between 0 and 2^15
	card32 ret = (qrand() | 
		((qrand() & 0x7fff) << 15 ) |
		((qrand() & 0x7fff) << 30 )) % maxNumberPlusOne;
	return ret;
}

inline card64 random64bit(card64 maxNumberPlusOne = 0xffffffffffffffff)
{
   // rand() produces a pseudo random number between 0 and 2^15
   card64 ret = (qrand() | 
      (((card64)(qrand() & 0x7fff)) << 15 ) |
      (((card64)(qrand() & 0x7fff)) << 30 ) |
      (((card64)(qrand() & 0x7fff)) << 45 ) |
      (((card64)(qrand() & 0x7fff)) << 60 ) ) % maxNumberPlusOne;
   return ret;
}

inline card32 random(card32 maxNumberPlusOne)
{
	return randomNumber(maxNumberPlusOne);
}

inline void srandom(unsigned seed)
{
	qsrand(seed);
}

class X4MATH_API LotteryDraw
{
	std::vector<unsigned>	m_Indices;
	unsigned		    m_NumIndices;

public:

	LotteryDraw() {m_NumIndices = 0;}

	/// setup array of indices, if customIndices is specified the array is filled with
	/// those custom indices. Otherwise it's filled with 0,1,2,...,n-1
	void setup( unsigned numIndices, unsigned * customIndices = NULL )
	{
		m_Indices = std::vector<unsigned>(size_t(numIndices));
		m_NumIndices = numIndices;
		if( customIndices )
		{
			for( unsigned i=0;i<numIndices;i++ )
				m_Indices[i] = customIndices[i];
		}
		else
		{
			for( unsigned i=0;i<numIndices;i++ )
				m_Indices[i] = i;
		}
	}

	void reset() {m_NumIndices = unsigned(m_Indices.size());}

	bool empty() const {return m_NumIndices == 0;}

	unsigned draw()
	{
		if( m_NumIndices == 0)
			throw X4Exception("LotteryDraw::draw() - array is empty!");

		unsigned rIndex = random32bit(m_NumIndices);
		m_NumIndices--;
		swap(m_Indices[rIndex],m_Indices[m_NumIndices]);
		return m_Indices[m_NumIndices];
	}



};


}


//---------------------------------------------------------------------------
#endif
 