//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ScalarDerivative.h"
#include "ScalarDerivative.inline.h"
#include <ConsoleOutput.h>
#include "MathTools.h"
#include "MathTools.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


void X4::ScalarDerivative_unitTest() {
	x4console << "\nTest1:\n";
	x4console << "x" <<"\t"<< "val" <<"\t"<< "autoval" <<"\t"<< "deriv" <<"\t"<< "autoderiv" << "\n";
	for (card32 i=0; i<=20; i++) {
		float64 x=i/10.0 - 1;
		float64 value      = sqr(x) + 2*x - 5;
		float64 derivative = 2*x + 2;
		DerivativeD xDD = DerivativeD::createVariable(x);
		DerivativeD yDD = sqr(xDD) + xDD*2 - 5;
		x4console << x <<"\t"<< value <<"\t"<< yDD.getValue() <<"\t"<< derivative <<"\t"<< yDD.getDerivative() << "\n";
		if (fabs(value-yDD.getValue()) != 0) {
			x4console << "VAULE does not match\n";
		}
		if (fabs(derivative-yDD.getDerivative()) != 0) {
			x4console << "DERIVATIVE does not match\n";
		}
	}	
	x4console << "\n";

	x4console << "\nTest2:\n";
	x4console << "x" <<"\t"<< "val" <<"\t"<< "autoval" <<"\t"<< "deriv" <<"\t"<< "autoderiv" << "\n";
	for (card32 i=0; i<=20; i++) {
		float64 x=i/10.0 - 1;
		float64 value      = (x*x*x*42 - x/2) / (x*x+1) - 5;
		float64 derivative = ((3*x*x*42 - 1.0/2.0)*(x*x+1) - (x*x*x*42 - x/2)*(2*x))/((x*x+1)*(x*x+1));
		DerivativeD xDD = DerivativeD::createVariable(x);
		DerivativeD yDD = (xDD*xDD*xDD*42 - xDD/2) / (xDD*xDD+1) - 5;
		x4console << x <<"\t"<< value <<"\t"<< yDD.getValue() <<"\t"<< derivative <<"\t"<< yDD.getDerivative() << "\n";
		if (fabs(value-yDD.getValue()) != 0) {
			x4console << "VAULE does not match\n";
			x4console << "relative error: " << fabs(value-yDD.getValue())/value << "\n";
		}
		if (fabs(derivative-yDD.getDerivative()) != 0) {
			x4console << "DERIVATIVE does not match\n";		
			x4console << "relative error: " << fabs(derivative-yDD.getDerivative())/derivative << "\n";
		}
	}	
	x4console << "\n";


	x4console << "\nTest3:\n";
	x4console << "x" <<"\t"<< "val" <<"\t"<< "autoval" <<"\t"<< "deriv" <<"\t"<< "autoderiv" <<"\t"<< "2nd deriv" <<"\t"<< "2nd a.d." << "\n";
	for (card32 i=0; i<=20; i++) {
		float64 x=i/10.0 - 1;
		float64 value      = x*x*x + sqr(x) + 2*x - 5;
		float64 derivative = 3*x*x + 2*x + 2;
		float64 secondDerivative = 6*x + 2;
		Derivative2ndOrderD xDD = createVariable2ndOrderD(x);
		Derivative2ndOrderD yDD = xDD*xDD*xDD +sqr(xDD) + xDD*2 - 5;
		float64 autoValue, autoDeriv, auto2ndDeriv;
		extract2ndOrderDerivativesD(yDD, autoValue, autoDeriv, auto2ndDeriv);
		x4console << x <<"\t"<< value <<"\t"<< autoValue 
		            <<"\t"<< derivative <<"\t"<< autoDeriv
		            <<"\t"<< secondDerivative <<"\t"<< auto2ndDeriv << "\n";
	}	
	x4console << "\n";

	x4console << "\nTest4:\n";
	x4console << "x" <<"\t"<< "val" <<"\t"<< "autoval" <<"\t"<< "deriv" <<"\t"<< "autoderiv" << "\n";
	for (card32 i=0; i<=20; i++) {
		float64 x=i/10.0 - 1;
		float64 value      = sin(x*2+3) + cos(x);
		float64 derivative = cos(x*2+3)*2 - sin(x);
		DerivativeD xDD = DerivativeD::createVariable(x);
		DerivativeD yDD = sin(xDD*2+3) + cos(xDD);
		x4console << x <<"\t"<< value <<"\t"<< yDD.getValue() <<"\t"<< derivative <<"\t"<< yDD.getDerivative() << "\n";
		if (fabs(value-yDD.getValue()) != 0) {
			x4console << "VAULE does not match\n";
			x4console << "relative error: " << fabs(value-yDD.getValue())/value << "\n";
		}
		if (fabs(derivative-yDD.getDerivative()) != 0) {
			x4console << "DERIVATIVE does not match\n";		
			x4console << "relative error: " << fabs(derivative-yDD.getDerivative())/derivative << "\n";
		}	 
	}	
	x4console << "\n";

	x4console << "\nTest5:\n";
	x4console << "x" <<"\t"<< "val" <<"\t"<< "autoval" <<"\t"<< "deriv" <<"\t"<< "autoderiv\t" << "2ndDeriv\t" <<"auto2nd\t" <<"\n";
	for (card32 i=0; i<=20; i++) {
		float64 x=i/10.0 - 1;
		float64 value       = sin(x*2+3) + cos(x);
		float64 derivative  = cos(x*2+3)*2 - sin(x);
		float64 secondDeriv = -sin(x*2+3)*4 - cos(x);
		Derivative2ndOrderD xDD = createVariable2ndOrderD(x);
		Derivative2ndOrderD yDD = sin(xDD*2+3) + cos(xDD);
		float64 av, ad, a2ndd;
		extract2ndOrderDerivativesD(yDD, av, ad, a2ndd);
		x4console << x << "\t"
		            << value <<"\t"<< av <<"\t"
		            << derivative <<"\t"<< ad << "\t"
		            << secondDeriv <<"\t"<< a2ndd << "\t"
		            << "\n";
	}	
	x4console << "\n";

	x4console << "\nTest6:\n";
	x4console << "x" <<"\t"<< "val" <<"\t"<< "autoval" <<"\t"<< "deriv" <<"\t"<< "autoderiv" <<"\t"<< "2nd deriv" <<"\t"<< "2nd a.d." << "\n";
	for (card32 i=1; i<=20; i++) {
		float64 x=i/10.0;
		float64 value      = x*x*x + sqr(x) + 2*x - 5;
		float64 derivative = 3*x*x + 2*x + 2;
		float64 secondDerivative = 6*x + 2;
		Derivative2ndOrderD xDD = createVariable2ndOrderD(x);
		Derivative2ndOrderD yDD = 
		   pow(xDD, (Derivative2ndOrderD)3.0)
		 + pow(xDD, (Derivative2ndOrderD)2.0) + xDD*2.0 - (Derivative2ndOrderD)5.0;
		float64 autoValue, autoDeriv, auto2ndDeriv;
		extract2ndOrderDerivativesD(yDD, autoValue, autoDeriv, auto2ndDeriv);
		x4console << x <<"\t"<< value <<"\t"<< autoValue 
		            <<"\t"<< derivative <<"\t"<< autoDeriv
		            <<"\t"<< secondDerivative <<"\t"<< auto2ndDeriv << "\n";
	}	
	x4console << "\n";
}

