#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "CameraPathRecorderEditorWidget.h"
#include "FreeCameraController.h"
#include "ExaminerCameraController.h"
//---------------------------------------------------------------------------
#include "PropertyBrowser.h"
//---------------------------------------------------------------------------

using namespace X4;

IMPLEMENT_CLASS_EDITOR( CameraPathRecorder )


CameraPathRecorderEditorWidget::CameraPathRecorderEditorWidget( QWidget * parent)
:QWidget(parent) // don't forget to call the base constructor with parent !
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	//easy access to QtDesigner buttons, etc. by ui object
	QWidget::connect(ui.IEKeyFrame, SIGNAL(valueChanged(int)), this, SLOT(sltIEKeyFrameValueChanged(int)));
	QWidget::connect(ui.BTGoto, SIGNAL(clicked()), this, SLOT(sltGotoKeyFrame()));
	QWidget::connect(ui.BTAdd, SIGNAL(clicked()), this, SLOT(sltAddKeyFrame()));
	QWidget::connect(ui.BTRemove, SIGNAL(clicked()), this, SLOT(sltRemoveKeyFrame()));
	QWidget::connect(ui.BTEdSubContrl, SIGNAL(clicked()), this, SLOT(sltEditSubController()));
	QWidget::connect(ui.BTSetExaminerCamController, SIGNAL(clicked()), this, SLOT(sltSetExaminerCamController()));
	QWidget::connect(ui.BTSetFreeCamController, SIGNAL(clicked()), this, SLOT(sltSetFreeCamController()));
	QWidget::connect(ui.FEPathTime, SIGNAL(valueChanged(double)), this, SLOT(sltFEPathTimeValueChanged(double)));
	QWidget::connect(ui.RBLinear, SIGNAL(toggled(bool)), this, SLOT(sltInterpolationToggled(bool)));
	QWidget::connect(ui.RBHermite, SIGNAL(toggled(bool)), this, SLOT(sltInterpolationToggled(bool)));
	QWidget::connect(ui.RBXRotate, SIGNAL(toggled(bool)), this, SLOT(sltInterpolationToggled(bool)));
	QWidget::connect(ui.RBYRotate, SIGNAL(toggled(bool)), this, SLOT(sltInterpolationToggled(bool)));
	QWidget::connect(ui.RBZRotate, SIGNAL(toggled(bool)), this, SLOT(sltInterpolationToggled(bool)));
	QWidget::connect(ui.RBBspline, SIGNAL(toggled(bool)), this, SLOT(sltInterpolationToggled(bool)));
	QWidget::connect(ui.CBRecordFrames, SIGNAL(stateChanged(int)), this, SLOT(sltCBRecordFramesStateChanged(int)));

	// button icons
	ui.BTGoto->setIcon(QIcon(":/foundation/images/Buttons/MovieFrame.bmp"));
	ui.BTAdd->setIcon(QIcon(":/foundation/images/Buttons/add.png"));
	ui.BTRemove->setIcon(QIcon(":/foundation/images/Buttons/remove.png"));
   ui.BTEdSubContrl->setIcon(QIcon(":/foundation/images/Icons/smallClass.png"));
}


void CameraPathRecorderEditorWidget::setup( CameraPathRecorder * obj )
{
	m_Recorder=obj;
	changing = false;
}


void CameraPathRecorderEditorWidget::updateEditor()
{
   setNumFrames(m_Recorder->getNumKeyFrames());
   limitFrameEd(m_Recorder->getNumKeyFrames()-1);
   ui.FEPathTime->setValue(m_Recorder->getPathTime());
   if (m_Recorder->getInterpolationType() == CameraPathRecorder::interpLinear) {
		ui.RBLinear->setChecked(true);
   } else if (m_Recorder->getInterpolationType() == CameraPathRecorder::interpHermite) {
		ui.RBHermite->setChecked(true);
   } else if (m_Recorder->getInterpolationType() == CameraPathRecorder::interpBSpline) {
	   ui.RBBspline->setChecked(true);
   } else if (m_Recorder->getInterpolationType() == CameraPathRecorder::interpRotX) {
	   ui.RBXRotate->setChecked(true);
   } else if (m_Recorder->getInterpolationType() == CameraPathRecorder::interpRotY) {
	   ui.RBYRotate->setChecked(true);
   } else {
		ui.RBZRotate->setChecked(true);
   }
	
}


// ------- user functions -------

void CameraPathRecorderEditorWidget::setNumFrames(int numFrames) {
	QString str;
   str.setNum(numFrames);
   ui.LBNumFrames->setText(str);
}


void CameraPathRecorderEditorWidget::limitFrameEd(int limit) {
   if (!changing) {
      changing = true;
      if (ui.IEKeyFrame->value() < 0) ui.IEKeyFrame->setValue(0);
      if (ui.IEKeyFrame->value() > limit) ui.IEKeyFrame->setValue(limit);
      changing = false;
   }
}


// ------- slots -------

void CameraPathRecorderEditorWidget::sltIEKeyFrameValueChanged(int) {
   limitFrameEd(m_Recorder->getNumKeyFrames()-1);
}


void CameraPathRecorderEditorWidget::sltGotoKeyFrame() {
   m_Recorder->gotoKeyFrame(ui.IEKeyFrame->value());
}


void CameraPathRecorderEditorWidget::sltAddKeyFrame() {
   m_Recorder->addKeyFrame(ui.IEKeyFrame->value());
   ui.IEKeyFrame->setValue(ui.IEKeyFrame->value() + 1);
   setNumFrames(m_Recorder->getNumKeyFrames());
}


void CameraPathRecorderEditorWidget::sltRemoveKeyFrame() {
   if (m_Recorder->getNumKeyFrames() == 0) return;
   m_Recorder->removeKeyFrame(ui.IEKeyFrame->value());
   ui.IEKeyFrame->setValue(ui.IEKeyFrame->value() - 1);
   setNumFrames(m_Recorder->getNumKeyFrames());
}


void CameraPathRecorderEditorWidget::sltEditSubController() {
   if (m_Recorder != NULL) {
      if (m_Recorder->getSubController() != NULL) {
//         const MetaClass *edClass = m_Recorder->getSubController()->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor();
//         if (edClass != NULL) {
            PropertyBrowserWidget *objBr = new PropertyBrowserWidget(NULL);
				objBr->setup(m_Recorder->getSubController());
//            objBr->pushObject(m_Recorder->getSubController(), (ClassEditor*)(edClass->newInstance()));
            objBr->show();
//         }
      }
   }
}

void CameraPathRecorderEditorWidget::sltSetExaminerCamController() 
{
	if ( m_Recorder == NULL ) return;
	m_Recorder->setSubController( new ExaminerCameraController() );
}

void CameraPathRecorderEditorWidget::sltSetFreeCamController() 
{
	if ( m_Recorder == NULL ) return;
	m_Recorder->setSubController( new FreeCameraController() );
}


void CameraPathRecorderEditorWidget::sltFEPathTimeValueChanged(double val) {
   m_Recorder->setPathTime(val);
}


void CameraPathRecorderEditorWidget::sltInterpolationToggled(bool toggle) {
	if (ui.RBLinear->isChecked()) {
		m_Recorder->setInterpolationType(CameraPathRecorder::interpLinear);
	} else if ( ui.RBHermite->isChecked() ) {
		m_Recorder->setInterpolationType(CameraPathRecorder::interpHermite);
	} else if ( ui.RBBspline->isChecked() ) {
		m_Recorder->setInterpolationType(CameraPathRecorder::interpBSpline);
	} else if ( ui.RBXRotate->isChecked() ) {
		m_Recorder->setInterpolationType(CameraPathRecorder::interpRotX);
	} else if ( ui.RBYRotate->isChecked() ) {
		m_Recorder->setInterpolationType(CameraPathRecorder::interpRotY);
	} else if ( ui.RBZRotate->isChecked() ){
		m_Recorder->setInterpolationType(CameraPathRecorder::interpRotZ);
	}
}



void CameraPathRecorderEditorWidget::sltCBRecordFramesStateChanged(int state) {
   ui.BTAdd->setEnabled(state==Qt::Checked);
   ui.BTRemove->setEnabled(state==Qt::Checked);
   m_Recorder->setRecordFrames(state==Qt::Checked);
}


#include "CameraPathRecorderEditorWidget_moc.h"
