//---------------------------------------------------------------------------
#ifndef ColorToolsH
#define ColorToolsH
//---------------------------------------------------------------------------
#include "x4miscFoundation.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "Random.h"
//---------------------------------------------------------------------------

namespace X4 {



inline Vector4f rainbowGradient(float32 v) {
	if (v<0.0f) v = 0.0f;
	if (v>1.0f) v = 1.0f;
	if (v <= 0.333333333333333333) {
		v *= 4.0f;
		return makeVector4f(0.0f, 1.0f, 0.0f, 1.0f) * v + makeVector4f(0.0f, 0.0f, 1.0f, 1.0f) * (1.0f-v);
	} else if (v <= 0.6666666666666666666) {
		v *= 3.0f;
		v -= 1.0f;
		return makeVector4f(1.0f, 1.0f, 0.0f, 1.0f) * v + makeVector4f(0.0f, 1.0f, 0.0f, 1.0f) * (1.0f-v);
	}  else {
		v *= 3.0f;
		v -= 2.0f;
		return makeVector4f(1.0f, 0.0f, 0.0f, 1.0f) * v + makeVector4f(1.0f, 1.0f, 0.0f, 1.0f) * (1.0f-v);
	} 
}

inline Vector3f rainbowGradient3f(float32 v){
	return shrink4To3(rainbowGradient(v));
}

inline Vector4f blueRedGradient(float32 v, float32 alpha) {
	if (v<0.0f) v = 0.0f;
	if (v>1.0f) v = 1.0f;
   return makeVector4f(0.2f, 0.2f, 1.0f, alpha)*(1.0f-v) + makeVector4f(1.0f, 0.2f, 0.2f, alpha)*v;
}

inline Vector3f randomColor()
{
	return makeVector3f(rnd01(),rnd01(),rnd01());
}

/// returns random color with fixed seed
inline Vector3f randomColorPersistent(mpcard seed)
{
	unsigned int rescueRandom = rand();
	srand(seed);
	return makeVector3f(rnd01(),rnd01(),rnd01());
	srand(rescueRandom);
}
/**
	class for arbitary color gradients within [0,1]
*/
class X4MISCFOUNDATION_API ColorGradient
{
public:

	/// add color at position t
	void addColor( const Vector3f &color, float t);

	/// compute color at t
	Vector3f getColor( float t) const;

	/// lookup color at t (creates&uses color table)
	Vector3f getColorFromTable( float t);

	/// setup color table
	void setupTable(mpcard numEntries=1000);

private:


	struct TColor
	{
		float t;
		Vector3f color;
		bool operator < (const TColor &o) const {return t < o.t;}
	};

	vector<TColor>		m_Colors;
	vector<Vector3f>	m_Table;
};

Vector3f  convertRGBtoHSV( const Vector3f &rgb);
Vector3f  convertHSVtoRGB( const Vector3f &hsv);

}


#endif