#ifndef CommandHistoryEditorWidgetH 
#define CommandHistoryEditorWidgetH
//---------------------------------------------------------------------------
#include "x4miscFoundation.h"
//---------------------------------------------------------------------------
#include "ClassEditor.h"
#include "Object.h"
#include <QDialog>
#include <QRadioButton>
#include <QTreeWidget>
#include "x4miscFoundation/CommandHistoryEditorForm_ui.h"
//---------------------------------------------------------------------------

namespace X4
{

// ******************** Import Dialog ********************

/// helper class to open an CommandHistory-ImportDialog
class X4MISCFOUNDATION_API CommandHistoryImportDialog : public QDialog
{
	Q_OBJECT

public:
	CommandHistoryImportDialog(QWidget *parent=0);

private slots:
	void sltOkButtonClicked();

private:
	QRadioButton *radioButton1;
	QRadioButton *radioButton2;
	QRadioButton *radioButton3;
	QVBoxLayout *mainLayout;
};



// ******************** CommandHistoryEditor ********************

class CommandHistory;
class CommandObject;
class CommandObjectContextInterface;

class X4MISCFOUNDATION_API CommandHistoryEditorWidget : public QWidget
{
	Q_OBJECT

private:
	///used to access buttons, etc. from QtDesigner
	Ui_CommandHistoryEditorForm ui;

   /// object to be edited
	CommandHistory	*	m_CmdHist;
   /// context in which actions take place - must be set manually
   Object *context;
	/// context interface
   CommandObjectContextInterface *coiContext;

	/// class editor of command object selected in DoList
   ClassEditor *doEditor;
	/// class editor widget of command object selected in DoList
   QWidget *doEditorWidget;
	/// class editor of command object selected in UndoList
   ClassEditor *undoEditor;
	/// class editor widget of command object selected in UndoList
   QWidget *undoEditorWidget;

	/// update list view
	void fillList(const std::vector<CommandObject*> &cmds, QTreeWidget *treeview);
	/// udates/changes the class editor of the selected command object which is shown in tab
   void updateSelectedItemEditor(ClassEditor *&selEditor, QWidget *&selEditorWidget, QWidget* tab, CommandObject *obj);

public:
	CommandHistoryEditorWidget(QWidget * parent = NULL, Qt::WindowFlags f=0);
	~CommandHistoryEditorWidget(void);

	void setup( CommandHistory * obj );

	/// update editor content with object data
	void updateEditor();

   /// call this method to set context (additional functionality will be provided)
	void setContext(Object *cont);


public slots:
	/// load button clicked
	void sltBTLoadClicked();
	/// save button clicked
	void sltBTSaveClicked();
	/// checkbox save with scene state changed
	void sltCBSaveWithSceneStateChanged(int);
	/// make macro button clicked
	void sltBTMakeMacroClicked();
	/// do list view selected item changed
	void sltLVDoChanged(QModelIndex);
	/// undo list view selected item changed
	void sltLVUndoChanged(QModelIndex);
	/// check box to show line number was clicked
	void sltCBShowLineNumberChanged(int);

};


MAKE_CLASS_EDITOR( X4MISCFOUNDATION_API , CommandHistory )

}

#endif