//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "CommandObjMacro.h"
//---------------------------------------------------------------------------
#include "ObjectStreams.h"
#include "MetaClass.h"
#include "Object.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* CommandObjMacro::CommandObjMacroMetaClass = NULL;

const MetaClass* CommandObjMacro::getInstanceClass() const
{
  return getClass();
}

Object* CommandObjMacro::newInstance()
{
  return new CommandObjMacro;
}

const MetaClass* CommandObjMacro::getClass()
{
  if (CommandObjMacroMetaClass == NULL) {
    CommandObjMacroMetaClass = new MetaClass("CommandObjMacro", "no description.",
                                      newInstance, sizeof(CommandObjMacro));
  }
  return CommandObjMacroMetaClass;
}

void CommandObjMacro::init(const MetaClass *parentClass)
{
  ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION);
  CommandObjMacroMetaClass->copyParentProperties();
}

void CommandObjMacro::shutdown()
{
  delete CommandObjMacroMetaClass;
}

// ---

void CommandObjMacro::addCommand(const CommandObject* command)
{
	CommandObject* newCommand = (CommandObject*)command->copy();
	commands.push_back(newCommand);
}

void CommandObjMacro::write(OutputObjectStream *s) const
{
  card32 numCommands = (card32)commands.size();
  s->write(numCommands);
  for (card32 i = 0; i != numCommands; ++i)
    s->writeObject(commands[i]);
}

void CommandObjMacro::read(InputObjectStream *s)
{
  card32 numCommands;
  s->read(numCommands);
  commands.resize(numCommands);

  for (card32 i = 0; i != numCommands; ++i) {
    s->readObject((Persistent*&)commands[i]);
  }
}

CommandObjMacro::~CommandObjMacro()
{
  for (card32 i = 0; i != commands.size(); ++i)
    delete commands[i];
}
