//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "CubeMap.h"
#include "CubeMap.inline.h"
//---------------------------------------------------------------------------
#include "X4Exception.h"
#include "ObjectClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "ClassMethods.h"
#include "LinearAlgebra.inline.h"
#include "TemplatedImage.inline.h"
#include "FileDialogs.h"
#include "InterfaceGlobals.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS(RGBACubeMap, 0)
{

	BEGIN_CLASS_INIT( RGBACubeMap );
	//ADD_OBJECT_FIXED_ARRAY_PROP( sides, 0, 6, RGBAImage::getClass(), true );
	ADD_OBJECT_PROP(sides[0],0, RGBAImage::getClass(), true );
	ADD_OBJECT_PROP(sides[1],0, RGBAImage::getClass(), true );
	ADD_OBJECT_PROP(sides[2],0, RGBAImage::getClass(), true );
	ADD_OBJECT_PROP(sides[3],0, RGBAImage::getClass(), true );
	ADD_OBJECT_PROP(sides[4],0, RGBAImage::getClass(), true );
	ADD_OBJECT_PROP(sides[5],0, RGBAImage::getClass(), true );

	ADD_NOARGS_METHOD_EX(interactiveLoadCubeMapFromFile, "loadCubeMap...", "open file dialog and load cube std::map from file (cross format)", ":/foundation/images/Buttons/open.png", true);
	ADD_NOARGS_METHOD_DESC(setupTestMap, "create a simple test cube std::map with colored sides");
}


// ----

RGBACubeMap::RGBACubeMap() {
   for (card32 i=0; i<6; i++) {
      delete sides[i]; // remove "TemplatedImage<Vector4ub>"
      sides[i] = new RGBAImage();
   }
}

void RGBACubeMap::assignSide(card32 index, const TemplatedImage<Vector4ub> *img) {
   x4Assert(dynamic_cast<const RGBAImage*>(img) != NULL);
   CubeMap<Vector4ub>::assignSide(index, img);
}


void RGBACubeMap::loadFromCrossBMP(const char *filename) {

	QImage* img = new QImage(QImage(QString(filename)).convertToFormat(QImage::Format_RGB32));

   card32 w = img->width();
   card32 h = img->height();

   card32 sw = w/3;
   card32 sh = h/4;

   if (sw == 0 || sh == 0 || w%3 != 0 || h%4 != 0) {
      throw X4Exception("RGBACubeMap::loadFromCrossBMP() - size does not match.");
   }

   // coordinates:
   //
   //        +---.
   //        |-y |
   //    +---+---+---.
   //    |-x |+z |+x |
   //    .---+---.---.
   //        |+y |
   //        +---.  + = reference point
   //        |-z |
   //        .---.

   const Vector2i refNX = makeVector2i(0,sh);
   const Vector2i refPX = makeVector2i(2*sw,sh);
   const Vector2i refNY = makeVector2i(sw,0);
   const Vector2i refPY = makeVector2i(sw,2*sh);
   const Vector2i refNZ = makeVector2i(sw,3*sh);
   const Vector2i refPZ = makeVector2i(sw,sh);

   ((RGBAImage*)(sides[CMS_POSITIVE_X]))->resize(sw, sh);
   ((RGBAImage*)(sides[CMS_POSITIVE_X]))->copyFromBMP(img, refPX[0], refPX[1], sw, sh);
   ((RGBAImage*)(sides[CMS_NEGATIVE_X]))->resize(sw, sh);
   ((RGBAImage*)(sides[CMS_NEGATIVE_X]))->copyFromBMP(img, refNX[0], refNX[1], sw, sh);
   ((RGBAImage*)(sides[CMS_POSITIVE_Y]))->resize(sw, sh);
   ((RGBAImage*)(sides[CMS_POSITIVE_Y]))->copyFromBMP(img, refPY[0], refPY[1], sw, sh);
   ((RGBAImage*)(sides[CMS_NEGATIVE_Y]))->resize(sw, sh);
   ((RGBAImage*)(sides[CMS_NEGATIVE_Y]))->copyFromBMP(img, refNY[0], refNY[1], sw, sh);
   ((RGBAImage*)(sides[CMS_POSITIVE_Z]))->resize(sw, sh);
   ((RGBAImage*)(sides[CMS_POSITIVE_Z]))->copyFromBMP(img, refPZ[0], refPZ[1], sw, sh);
   ((RGBAImage*)(sides[CMS_NEGATIVE_Z]))->resize(sw, sh);
   ((RGBAImage*)(sides[CMS_NEGATIVE_Z]))->copyFromBMP(img, refNZ[0], refNZ[1], sw, sh);

	delete img;
}

void RGBACubeMap::interactiveLoadCubeMapFromFile() {
	std::string filename = FileDialogs::getOpenFileName(InterfaceGlobals::getMainWindow(), "Open Cube Map Image (Cross Layout)",
	                                               "Bitmap Files (*.bmp; *.jpg; *.png)", NULL, "", FileDialogs::IMPORT_DIR);

   if (filename != "") {
      try {
         loadFromCrossBMP(filename.c_str());
      } catch (...) {
         error("error loading std::map.");
      }
   }
}


Vector4ub calcAngular( QImage *img, Vector3f &vec )
{
   float vAngle = acos( vec[1] ) / M_PI * 180.0f;
   Vector2f tmp = normalize( makeVector2f( vec[0], vec[2] ) );
   float hAngle = atan2( tmp[0], tmp[1] ) / M_PI * 180.0f;

   if( hAngle < 0 )
      hAngle += 360.0f;
   hAngle = max( 0.0f, min( 360.0f, hAngle ) );

   float x = min((float)img->width()-1, max( 0.0f, hAngle / 360.0f * (float)img->width() ));
   float y = min((float)img->height()-1, max( 0.0f, vAngle / 180.0f * (float)img->height() ));

	QRgb rgb = img->pixel(x,y);
   Vector4ub ret = makeVector4ub(qBlue(rgb), qGreen(rgb), qRed(rgb), 0xFF);
   return ret;
}


void RGBACubeMap::loadFromAngularBMP(const char * filename )
{
	QImage* img = new QImage(QImage(QString(filename)).convertToFormat(QImage::Format_RGB32));

   card32 w = img->width();
   card32 h = img->height();

   int size = 1;
   while( size < (int)w/2 && size < (int)h/2 )
      size *= 2;

   for( card32 i=0;i<6;i++ )
      sides[i]->resize(size,size);

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(0.5f,  -yf, -xf );
            vec = normalize( vec );

            sides[ CMS_POSITIVE_X ]->setPixel(x,y, calcAngular( img, vec) );
		  }

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(xf, -yf,0.5f );
				vec = normalize( vec );
            sides[ CMS_POSITIVE_Z ]->setPixel(x,y, calcAngular( img, vec) );
		  }

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(-0.5f, -yf,xf );
				vec = normalize( vec );
            sides[ CMS_NEGATIVE_X ]->setPixel(x,y, calcAngular( img, vec) );
		  }

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(xf, yf,-0.5f );
				vec = normalize( vec );
            sides[ CMS_NEGATIVE_Z ]->setPixel(x,y, calcAngular( img, vec) );
		  }

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(xf, -0.5f,-yf );
				vec = normalize( vec );
            sides[ CMS_POSITIVE_Y ]->setPixel(x,y, calcAngular( img, vec) );
		  }

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(xf, 0.5f,yf );
				vec = normalize( vec );
            sides[ CMS_NEGATIVE_Y ]->setPixel(x,y, calcAngular( img, vec) );
		  }

}


void RGBACubeMap::setupTestMap() {
   delete sides[0];
   sides[0] = new RGBAImage(64,64);
   sides[0]->clear(makeVector4ub(255,0,0,0));
   delete sides[1];
   sides[1] = new RGBAImage(64,64);
   sides[1]->clear(makeVector4ub(0,255,0,0));
   delete sides[2];
   sides[2] = new RGBAImage(64,64);
   sides[2]->clear(makeVector4ub(0,0,255,0));
   delete sides[3];
   sides[3] = new RGBAImage(64,64);
   sides[3]->clear(makeVector4ub(255,255,0,0));
   delete sides[4];
   sides[4] = new RGBAImage(64,64);
   sides[4]->clear(makeVector4ub(255,0,255,0));
   delete sides[5];
   sides[5] = new RGBAImage(64,64);
   sides[5]->clear(makeVector4ub(0,255,255,0));
}

void RGBACubeMap::prefilterDiffuse(card32 mapSize) {
   for (card32 i=0; i<6; i++) {
      RGBAImage *newSide = new RGBAImage(mapSize, mapSize);
      ((RGBAImage*)(sides[i]))->downsampleAA(sides[i]->getSizeX()/mapSize, newSide);
      delete sides[i];
      sides[i] = newSide;
   }

   for (card32 id=0; id<6; id++) {
      for (card32 xd=0; xd<mapSize; xd++) {
         for (card32 yd=0; yd<mapSize; yd++) {

            Vector4f col = NULL_VECTOR4F;
            float32 wSum = 0.0f;

            for (card32 is=0; is<6; is++) {
               for (card32 xs=0; xs<mapSize; xs++) {
                  for (card32 ys=0; ys<mapSize; ys++) {

                     Vector3f sourceVector = getPixelDirection(is,xs,ys);
                     Vector3f destVector = getPixelDirection(id,xd,yd);
                     float32 w = sourceVector * destVector;
                     if (w > 0) {
                        w = w*w*w*w*w;
                        wSum += w;
                        Vector4ub cs = lookupCubeMap(getPixelDirection(is,xs,ys));
                        col += makeVector4f(cs[0], cs[1], cs[2], cs[3])*w;
                     }
                  }
               }
            }

            col /= wSum;
            Vector4ub iCol = makeVector4ub((card8)(col[0]), (card8)(col[1]), (card8)(col[2]), (card8)(col[3]));
            sides[id]->setPixel(xd,yd, iCol);
         }
      }
   }

}



template class X4MISCFOUNDATION_API CubeMap< Vector3ub>;
template class X4MISCFOUNDATION_API CubeMap< Vector4ub>;
template class X4MISCFOUNDATION_API CubeMap< StaticVector<card8,1> >;
template class X4MISCFOUNDATION_API CubeMap< Vector3f >;
template class X4MISCFOUNDATION_API CubeMap< Vector4f >;
template class X4MISCFOUNDATION_API CubeMap< StaticVector<float32,1> >;
