#ifndef DVectorObjectH
#define DVectorObjectH

//---------------------------------------------------------------------------
#include "x4miscFoundation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//#include "LinearAlgebra.h"
#include "DynamicLinearAlgebra.h"
#include "AttachedData.h"
#include "ObjectListProperty.h"
//---------------------------------------------------------------------------

namespace X4
{

	template<typename Base>
	class DVectorObjectTemplate : public Persistent
	{
	public:
		typedef DynamicVector<Base> DVector;
		DVector data;


	public:
		DVectorObjectTemplate(void) {}
		DVectorObjectTemplate(const DVector& value) { data = value; }
		virtual ~DVectorObjectTemplate(void) {}

		virtual void assign(const Object* obj, X4CopyContext *context = NULL) {
			// Persistent::assign(obj,context);
			const DVectorObjectTemplate<Base> * o = dynamic_cast<const DVectorObjectTemplate<Base>*>(obj);
			x4Assert( o != NULL );

			data = o->data;
		}

		Base& operator[](const mpcard &index) { return data[index]; }
		const Base& operator[](const mpcard &index) const { return data[index]; }
		mpcard getDim() const { return data.getDim(); }

		inline std::vector<Base> toStd() const 
		{
			std::vector<Base> res(data.getDim());
			for (unsigned i=0; i < data.getDim(); i++) res[i] = data[i];
			return res;
		}
		inline void fromStd(const std::vector<Base>& elems) 
		{
			data.setDim(elems.size());
			for (unsigned i=0; i < elems.size(); i++) data[i] = elems[i];
		}
		virtual void write(OutputObjectStream *s) const {
			Persistent::write(s);
			data.write(s);
		}
		virtual void read(InputObjectStream *s) {
			Persistent::read(s);
			data.read(s);
		}


		// stl compatible interface
	public:
		typedef typename DVector::iterator iterator;
		typedef typename DVector::const_iterator const_iterator;

		iterator begin() { return data.begin(); }
		const_iterator begin() const { return data.begin(); }
		iterator end() { return data.end(); }
		const_iterator end() const { return data.end(); }
		size_t size() const { return data.size(); }
		bool empty() const { return data.empty(); }
	};




	class X4MISCFOUNDATION_API DVectorUBObject : public DVectorObjectTemplate<card8>
	{
		X4_CLASS( DVectorUBObject )

	public:
		DVectorUBObject(void) {}
		DVectorUBObject(const DVector& value) : DVectorObjectTemplate<card8>( value ) {}
	};

	class X4MISCFOUNDATION_API DVectorFObject : public DVectorObjectTemplate<float>
	{
		X4_CLASS( DVectorFObject )

	public:
		DVectorFObject(void) {}
		DVectorFObject(const DVector& value) : DVectorObjectTemplate<float>( value ) {}
	};

	class X4MISCFOUNDATION_API DVectorIObject : public DVectorObjectTemplate<int>
	{
		X4_CLASS( DVectorIObject )

	public:
		DVectorIObject(void) {}
		DVectorIObject(const DVector& value) : DVectorObjectTemplate<int>( value ) {}
	};

	class X4MISCFOUNDATION_API DVectorUObject : public DVectorObjectTemplate<card32>
	{
		X4_CLASS( DVectorUObject )

	public:
		DVectorUObject(void) {}
		DVectorUObject(const DVector& value) : DVectorObjectTemplate<card32>( value ) {}
	};

	class X4MISCFOUNDATION_API DVectorUObjectList : public AttachedData
	{
		X4_CLASS( DVectorUObjectList )

		std::vector<DVectorUObject*> m_List;

		IMPLEMENT_OBJECT_LIST_METHODS_STL(List, DVectorUObject);
	public:
		DVectorUObjectList() {}
		virtual ~DVectorUObjectList() { for (unsigned i=0; i < m_List.size(); i++) delete m_List[i]; }

		virtual void assign(const Object* obj, X4CopyContext *context = NULL)
		{
			//Persistent::assign(obj, context);
			const DVectorUObjectList* o = dynamic_cast<const DVectorUObjectList*>(obj);
			x4Assert( o != NULL );
			for (unsigned i=0; i < m_List.size(); i++) delete m_List[i];
			m_List.resize(o->m_List.size());
			for (unsigned i=0; i < m_List.size(); i++) m_List[i] = (DVectorUObject*)o->m_List[i]->copy();
		}

		static std::string getDefaultName() { return "DVectorUObjectList"; }

	};
}

#endif