//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "MarchingCubes.h"
//---------------------------------------------------------------------------
#include "Polygon.h"
#include "Polygon.inline.h"
#include "LinearAlgebra.inline.h"
#include "BoundingBox.inline.h"
#include "ProgressWindow.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace X4;

int indexList[256 * 15] = {
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 8, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 1, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 8, 3, 9, 8, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 2, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 8, 3, 1, 2, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        9, 2, 11, 0, 2, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        2, 8, 3, 2, 11, 8, 11, 9, 8, -1, -1, -1, -1, -1, -1,
        3, 10, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 10, 2, 8, 10, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 9, 0, 2, 3, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 10, 2, 1, 9, 10, 9, 8, 10, -1, -1, -1, -1, -1, -1,
        3, 11, 1, 10, 11, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 11, 1, 0, 8, 11, 8, 10, 11, -1, -1, -1, -1, -1, -1,
        3, 9, 0, 3, 10, 9, 10, 11, 9, -1, -1, -1, -1, -1, -1,
        9, 8, 11, 11, 8, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        4, 7, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        4, 3, 0, 7, 3, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 1, 9, 8, 4, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        4, 1, 9, 4, 7, 1, 7, 3, 1, -1, -1, -1, -1, -1, -1,
        1, 2, 11, 8, 4, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        3, 4, 7, 3, 0, 4, 1, 2, 11, -1, -1, -1, -1, -1, -1,
        9, 2, 11, 9, 0, 2, 8, 4, 7, -1, -1, -1, -1, -1, -1,
        2, 11, 9, 2, 9, 7, 2, 7, 3, 7, 9, 4, -1, -1, -1,
        8, 4, 7, 3, 10, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        10, 4, 7, 10, 2, 4, 2, 0, 4, -1, -1, -1, -1, -1, -1,
        9, 0, 1, 8, 4, 7, 2, 3, 10, -1, -1, -1, -1, -1, -1,
        4, 7, 10, 9, 4, 10, 9, 10, 2, 9, 2, 1, -1, -1, -1,
        3, 11, 1, 3, 10, 11, 7, 8, 4, -1, -1, -1, -1, -1, -1,
        1, 10, 11, 1, 4, 10, 1, 0, 4, 7, 10, 4, -1, -1, -1,
        4, 7, 8, 9, 0, 10, 9, 10, 11, 10, 0, 3, -1, -1, -1,
        4, 7, 10, 4, 10, 9, 9, 10, 11, -1, -1, -1, -1, -1, -1,
        9, 5, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        9, 5, 4, 0, 8, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 5, 4, 1, 5, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        8, 5, 4, 8, 3, 5, 3, 1, 5, -1, -1, -1, -1, -1, -1,
        1, 2, 11, 9, 5, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        3, 0, 8, 1, 2, 11, 4, 9, 5, -1, -1, -1, -1, -1, -1,
        5, 2, 11, 5, 4, 2, 4, 0, 2, -1, -1, -1, -1, -1, -1,
        2, 11, 5, 3, 2, 5, 3, 5, 4, 3, 4, 8, -1, -1, -1,
        9, 5, 4, 2, 3, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 10, 2, 0, 8, 10, 4, 9, 5, -1, -1, -1, -1, -1, -1,
        0, 5, 4, 0, 1, 5, 2, 3, 10, -1, -1, -1, -1, -1, -1,
        2, 1, 5, 2, 5, 8, 2, 8, 10, 4, 8, 5, -1, -1, -1,
        11, 3, 10, 11, 1, 3, 9, 5, 4, -1, -1, -1, -1, -1, -1,
        4, 9, 5, 0, 8, 1, 8, 11, 1, 8, 10, 11, -1, -1, -1,
        5, 4, 0, 5, 0, 10, 5, 10, 11, 10, 0, 3, -1, -1, -1,
        5, 4, 8, 5, 8, 11, 11, 8, 10, -1, -1, -1, -1, -1, -1,
        9, 7, 8, 5, 7, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        9, 3, 0, 9, 5, 3, 5, 7, 3, -1, -1, -1, -1, -1, -1,
        0, 7, 8, 0, 1, 7, 1, 5, 7, -1, -1, -1, -1, -1, -1,
        1, 5, 3, 3, 5, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        9, 7, 8, 9, 5, 7, 11, 1, 2, -1, -1, -1, -1, -1, -1,
        11, 1, 2, 9, 5, 0, 5, 3, 0, 5, 7, 3, -1, -1, -1,
        8, 0, 2, 8, 2, 5, 8, 5, 7, 11, 5, 2, -1, -1, -1,
        2, 11, 5, 2, 5, 3, 3, 5, 7, -1, -1, -1, -1, -1, -1,
        7, 9, 5, 7, 8, 9, 3, 10, 2, -1, -1, -1, -1, -1, -1,
        9, 5, 7, 9, 7, 2, 9, 2, 0, 2, 7, 10, -1, -1, -1,
        2, 3, 10, 0, 1, 8, 1, 7, 8, 1, 5, 7, -1, -1, -1,
        10, 2, 1, 10, 1, 7, 7, 1, 5, -1, -1, -1, -1, -1, -1,
        9, 5, 8, 8, 5, 7, 11, 1, 3, 11, 3, 10, -1, -1, -1,
        5, 7, 0, 5, 0, 9, 7, 10, 0, 1, 0, 11, 10, 11, 0,
        10, 11, 0, 10, 0, 3, 11, 5, 0, 8, 0, 7, 5, 7, 0,
        10, 11, 5, 7, 10, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        11, 6, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 8, 3, 5, 11, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        9, 0, 1, 5, 11, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 8, 3, 1, 9, 8, 5, 11, 6, -1, -1, -1, -1, -1, -1,
        1, 6, 5, 2, 6, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 6, 5, 1, 2, 6, 3, 0, 8, -1, -1, -1, -1, -1, -1,
        9, 6, 5, 9, 0, 6, 0, 2, 6, -1, -1, -1, -1, -1, -1,
        5, 9, 8, 5, 8, 2, 5, 2, 6, 3, 2, 8, -1, -1, -1,
        2, 3, 10, 11, 6, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        10, 0, 8, 10, 2, 0, 11, 6, 5, -1, -1, -1, -1, -1, -1,
        0, 1, 9, 2, 3, 10, 5, 11, 6, -1, -1, -1, -1, -1, -1,
        5, 11, 6, 1, 9, 2, 9, 10, 2, 9, 8, 10, -1, -1, -1,
        6, 3, 10, 6, 5, 3, 5, 1, 3, -1, -1, -1, -1, -1, -1,
        0, 8, 10, 0, 10, 5, 0, 5, 1, 5, 10, 6, -1, -1, -1,
        3, 10, 6, 0, 3, 6, 0, 6, 5, 0, 5, 9, -1, -1, -1,
        6, 5, 9, 6, 9, 10, 10, 9, 8, -1, -1, -1, -1, -1, -1,
        5, 11, 6, 4, 7, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        4, 3, 0, 4, 7, 3, 6, 5, 11, -1, -1, -1, -1, -1, -1,
        1, 9, 0, 5, 11, 6, 8, 4, 7, -1, -1, -1, -1, -1, -1,
        11, 6, 5, 1, 9, 7, 1, 7, 3, 7, 9, 4, -1, -1, -1,
        6, 1, 2, 6, 5, 1, 4, 7, 8, -1, -1, -1, -1, -1, -1,
        1, 2, 5, 5, 2, 6, 3, 0, 4, 3, 4, 7, -1, -1, -1,
        8, 4, 7, 9, 0, 5, 0, 6, 5, 0, 2, 6, -1, -1, -1,
        7, 3, 9, 7, 9, 4, 3, 2, 9, 5, 9, 6, 2, 6, 9,
        3, 10, 2, 7, 8, 4, 11, 6, 5, -1, -1, -1, -1, -1, -1,
        5, 11, 6, 4, 7, 2, 4, 2, 0, 2, 7, 10, -1, -1, -1,
        0, 1, 9, 4, 7, 8, 2, 3, 10, 5, 11, 6, -1, -1, -1,
        9, 2, 1, 9, 10, 2, 9, 4, 10, 7, 10, 4, 5, 11, 6,
        8, 4, 7, 3, 10, 5, 3, 5, 1, 5, 10, 6, -1, -1, -1,
        5, 1, 10, 5, 10, 6, 1, 0, 10, 7, 10, 4, 0, 4, 10,
        0, 5, 9, 0, 6, 5, 0, 3, 6, 10, 6, 3, 8, 4, 7,
        6, 5, 9, 6, 9, 10, 4, 7, 9, 7, 10, 9, -1, -1, -1,
        11, 4, 9, 6, 4, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        4, 11, 6, 4, 9, 11, 0, 8, 3, -1, -1, -1, -1, -1, -1,
        11, 0, 1, 11, 6, 0, 6, 4, 0, -1, -1, -1, -1, -1, -1,
        8, 3, 1, 8, 1, 6, 8, 6, 4, 6, 1, 11, -1, -1, -1,
        1, 4, 9, 1, 2, 4, 2, 6, 4, -1, -1, -1, -1, -1, -1,
        3, 0, 8, 1, 2, 9, 2, 4, 9, 2, 6, 4, -1, -1, -1,
        0, 2, 4, 4, 2, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        8, 3, 2, 8, 2, 4, 4, 2, 6, -1, -1, -1, -1, -1, -1,
        11, 4, 9, 11, 6, 4, 10, 2, 3, -1, -1, -1, -1, -1, -1,
        0, 8, 2, 2, 8, 10, 4, 9, 11, 4, 11, 6, -1, -1, -1,
        3, 10, 2, 0, 1, 6, 0, 6, 4, 6, 1, 11, -1, -1, -1,
        6, 4, 1, 6, 1, 11, 4, 8, 1, 2, 1, 10, 8, 10, 1,
        9, 6, 4, 9, 3, 6, 9, 1, 3, 10, 6, 3, -1, -1, -1,
        8, 10, 1, 8, 1, 0, 10, 6, 1, 9, 1, 4, 6, 4, 1,
        3, 10, 6, 3, 6, 0, 0, 6, 4, -1, -1, -1, -1, -1, -1,
        6, 4, 8, 10, 6, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        7, 11, 6, 7, 8, 11, 8, 9, 11, -1, -1, -1, -1, -1, -1,
        0, 7, 3, 0, 11, 7, 0, 9, 11, 6, 7, 11, -1, -1, -1,
        11, 6, 7, 1, 11, 7, 1, 7, 8, 1, 8, 0, -1, -1, -1,
        11, 6, 7, 11, 7, 1, 1, 7, 3, -1, -1, -1, -1, -1, -1,
        1, 2, 6, 1, 6, 8, 1, 8, 9, 8, 6, 7, -1, -1, -1,
        2, 6, 9, 2, 9, 1, 6, 7, 9, 0, 9, 3, 7, 3, 9,
        7, 8, 0, 7, 0, 6, 6, 0, 2, -1, -1, -1, -1, -1, -1,
        7, 3, 2, 6, 7, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        2, 3, 10, 11, 6, 8, 11, 8, 9, 8, 6, 7, -1, -1, -1,
        2, 0, 7, 2, 7, 10, 0, 9, 7, 6, 7, 11, 9, 11, 7,
        1, 8, 0, 1, 7, 8, 1, 11, 7, 6, 7, 11, 2, 3, 10,
        10, 2, 1, 10, 1, 7, 11, 6, 1, 6, 7, 1, -1, -1, -1,
        8, 9, 6, 8, 6, 7, 9, 1, 6, 10, 6, 3, 1, 3, 6,
        0, 9, 1, 10, 6, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        7, 8, 0, 7, 0, 6, 3, 10, 0, 10, 6, 0, -1, -1, -1,
        7, 10, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        7, 6, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        3, 0, 8, 10, 7, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 1, 9, 10, 7, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        8, 1, 9, 8, 3, 1, 10, 7, 6, -1, -1, -1, -1, -1, -1,
        11, 1, 2, 6, 10, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 2, 11, 3, 0, 8, 6, 10, 7, -1, -1, -1, -1, -1, -1,
        2, 9, 0, 2, 11, 9, 6, 10, 7, -1, -1, -1, -1, -1, -1,
        6, 10, 7, 2, 11, 3, 11, 8, 3, 11, 9, 8, -1, -1, -1,
        7, 2, 3, 6, 2, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        7, 0, 8, 7, 6, 0, 6, 2, 0, -1, -1, -1, -1, -1, -1,
        2, 7, 6, 2, 3, 7, 0, 1, 9, -1, -1, -1, -1, -1, -1,
        1, 6, 2, 1, 8, 6, 1, 9, 8, 8, 7, 6, -1, -1, -1,
        11, 7, 6, 11, 1, 7, 1, 3, 7, -1, -1, -1, -1, -1, -1,
        11, 7, 6, 1, 7, 11, 1, 8, 7, 1, 0, 8, -1, -1, -1,
        0, 3, 7, 0, 7, 11, 0, 11, 9, 6, 11, 7, -1, -1, -1,
        7, 6, 11, 7, 11, 8, 8, 11, 9, -1, -1, -1, -1, -1, -1,
        6, 8, 4, 10, 8, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        3, 6, 10, 3, 0, 6, 0, 4, 6, -1, -1, -1, -1, -1, -1,
        8, 6, 10, 8, 4, 6, 9, 0, 1, -1, -1, -1, -1, -1, -1,
        9, 4, 6, 9, 6, 3, 9, 3, 1, 10, 3, 6, -1, -1, -1,
        6, 8, 4, 6, 10, 8, 2, 11, 1, -1, -1, -1, -1, -1, -1,
        1, 2, 11, 3, 0, 10, 0, 6, 10, 0, 4, 6, -1, -1, -1,
        4, 10, 8, 4, 6, 10, 0, 2, 9, 2, 11, 9, -1, -1, -1,
        11, 9, 3, 11, 3, 2, 9, 4, 3, 10, 3, 6, 4, 6, 3,
        8, 2, 3, 8, 4, 2, 4, 6, 2, -1, -1, -1, -1, -1, -1,
        0, 4, 2, 4, 6, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 9, 0, 2, 3, 4, 2, 4, 6, 4, 3, 8, -1, -1, -1,
        1, 9, 4, 1, 4, 2, 2, 4, 6, -1, -1, -1, -1, -1, -1,
        8, 1, 3, 8, 6, 1, 8, 4, 6, 6, 11, 1, -1, -1, -1,
        11, 1, 0, 11, 0, 6, 6, 0, 4, -1, -1, -1, -1, -1, -1,
        4, 6, 3, 4, 3, 8, 6, 11, 3, 0, 3, 9, 11, 9, 3,
        11, 9, 4, 6, 11, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        4, 9, 5, 7, 6, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 8, 3, 4, 9, 5, 10, 7, 6, -1, -1, -1, -1, -1, -1,
        5, 0, 1, 5, 4, 0, 7, 6, 10, -1, -1, -1, -1, -1, -1,
        10, 7, 6, 8, 3, 4, 3, 5, 4, 3, 1, 5, -1, -1, -1,
        9, 5, 4, 11, 1, 2, 7, 6, 10, -1, -1, -1, -1, -1, -1,
        6, 10, 7, 1, 2, 11, 0, 8, 3, 4, 9, 5, -1, -1, -1,
        7, 6, 10, 5, 4, 11, 4, 2, 11, 4, 0, 2, -1, -1, -1,
        3, 4, 8, 3, 5, 4, 3, 2, 5, 11, 5, 2, 10, 7, 6,
        7, 2, 3, 7, 6, 2, 5, 4, 9, -1, -1, -1, -1, -1, -1,
        9, 5, 4, 0, 8, 6, 0, 6, 2, 6, 8, 7, -1, -1, -1,
        3, 6, 2, 3, 7, 6, 1, 5, 0, 5, 4, 0, -1, -1, -1,
        6, 2, 8, 6, 8, 7, 2, 1, 8, 4, 8, 5, 1, 5, 8,
        9, 5, 4, 11, 1, 6, 1, 7, 6, 1, 3, 7, -1, -1, -1,
        1, 6, 11, 1, 7, 6, 1, 0, 7, 8, 7, 0, 9, 5, 4,
        4, 0, 11, 4, 11, 5, 0, 3, 11, 6, 11, 7, 3, 7, 11,
        7, 6, 11, 7, 11, 8, 5, 4, 11, 4, 8, 11, -1, -1, -1,
        6, 9, 5, 6, 10, 9, 10, 8, 9, -1, -1, -1, -1, -1, -1,
        3, 6, 10, 0, 6, 3, 0, 5, 6, 0, 9, 5, -1, -1, -1,
        0, 10, 8, 0, 5, 10, 0, 1, 5, 5, 6, 10, -1, -1, -1,
        6, 10, 3, 6, 3, 5, 5, 3, 1, -1, -1, -1, -1, -1, -1,
        1, 2, 11, 9, 5, 10, 9, 10, 8, 10, 5, 6, -1, -1, -1,
        0, 10, 3, 0, 6, 10, 0, 9, 6, 5, 6, 9, 1, 2, 11,
        10, 8, 5, 10, 5, 6, 8, 0, 5, 11, 5, 2, 0, 2, 5,
        6, 10, 3, 6, 3, 5, 2, 11, 3, 11, 5, 3, -1, -1, -1,
        5, 8, 9, 5, 2, 8, 5, 6, 2, 3, 8, 2, -1, -1, -1,
        9, 5, 6, 9, 6, 0, 0, 6, 2, -1, -1, -1, -1, -1, -1,
        1, 5, 8, 1, 8, 0, 5, 6, 8, 3, 8, 2, 6, 2, 8,
        1, 5, 6, 2, 1, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 3, 6, 1, 6, 11, 3, 8, 6, 5, 6, 9, 8, 9, 6,
        11, 1, 0, 11, 0, 6, 9, 5, 0, 5, 6, 0, -1, -1, -1,
        0, 3, 8, 5, 6, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        11, 5, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        10, 5, 11, 7, 5, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        10, 5, 11, 10, 7, 5, 8, 3, 0, -1, -1, -1, -1, -1, -1,
        5, 10, 7, 5, 11, 10, 1, 9, 0, -1, -1, -1, -1, -1, -1,
        11, 7, 5, 11, 10, 7, 9, 8, 1, 8, 3, 1, -1, -1, -1,
        10, 1, 2, 10, 7, 1, 7, 5, 1, -1, -1, -1, -1, -1, -1,
        0, 8, 3, 1, 2, 7, 1, 7, 5, 7, 2, 10, -1, -1, -1,
        9, 7, 5, 9, 2, 7, 9, 0, 2, 2, 10, 7, -1, -1, -1,
        7, 5, 2, 7, 2, 10, 5, 9, 2, 3, 2, 8, 9, 8, 2,
        2, 5, 11, 2, 3, 5, 3, 7, 5, -1, -1, -1, -1, -1, -1,
        8, 2, 0, 8, 5, 2, 8, 7, 5, 11, 2, 5, -1, -1, -1,
        9, 0, 1, 5, 11, 3, 5, 3, 7, 3, 11, 2, -1, -1, -1,
        9, 8, 2, 9, 2, 1, 8, 7, 2, 11, 2, 5, 7, 5, 2,
        1, 3, 5, 3, 7, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 8, 7, 0, 7, 1, 1, 7, 5, -1, -1, -1, -1, -1, -1,
        9, 0, 3, 9, 3, 5, 5, 3, 7, -1, -1, -1, -1, -1, -1,
        9, 8, 7, 5, 9, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        5, 8, 4, 5, 11, 8, 11, 10, 8, -1, -1, -1, -1, -1, -1,
        5, 0, 4, 5, 10, 0, 5, 11, 10, 10, 3, 0, -1, -1, -1,
        0, 1, 9, 8, 4, 11, 8, 11, 10, 11, 4, 5, -1, -1, -1,
        11, 10, 4, 11, 4, 5, 10, 3, 4, 9, 4, 1, 3, 1, 4,
        2, 5, 1, 2, 8, 5, 2, 10, 8, 4, 5, 8, -1, -1, -1,
        0, 4, 10, 0, 10, 3, 4, 5, 10, 2, 10, 1, 5, 1, 10,
        0, 2, 5, 0, 5, 9, 2, 10, 5, 4, 5, 8, 10, 8, 5,
        9, 4, 5, 2, 10, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        2, 5, 11, 3, 5, 2, 3, 4, 5, 3, 8, 4, -1, -1, -1,
        5, 11, 2, 5, 2, 4, 4, 2, 0, -1, -1, -1, -1, -1, -1,
        3, 11, 2, 3, 5, 11, 3, 8, 5, 4, 5, 8, 0, 1, 9,
        5, 11, 2, 5, 2, 4, 1, 9, 2, 9, 4, 2, -1, -1, -1,
        8, 4, 5, 8, 5, 3, 3, 5, 1, -1, -1, -1, -1, -1, -1,
        0, 4, 5, 1, 0, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        8, 4, 5, 8, 5, 3, 9, 0, 5, 0, 3, 5, -1, -1, -1,
        9, 4, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        4, 10, 7, 4, 9, 10, 9, 11, 10, -1, -1, -1, -1, -1, -1,
        0, 8, 3, 4, 9, 7, 9, 10, 7, 9, 11, 10, -1, -1, -1,
        1, 11, 10, 1, 10, 4, 1, 4, 0, 7, 4, 10, -1, -1, -1,
        3, 1, 4, 3, 4, 8, 1, 11, 4, 7, 4, 10, 11, 10, 4,
        4, 10, 7, 9, 10, 4, 9, 2, 10, 9, 1, 2, -1, -1, -1,
        9, 7, 4, 9, 10, 7, 9, 1, 10, 2, 10, 1, 0, 8, 3,
        10, 7, 4, 10, 4, 2, 2, 4, 0, -1, -1, -1, -1, -1, -1,
        10, 7, 4, 10, 4, 2, 8, 3, 4, 3, 2, 4, -1, -1, -1,
        2, 9, 11, 2, 7, 9, 2, 3, 7, 7, 4, 9, -1, -1, -1,
        9, 11, 7, 9, 7, 4, 11, 2, 7, 8, 7, 0, 2, 0, 7,
        3, 7, 11, 3, 11, 2, 7, 4, 11, 1, 11, 0, 4, 0, 11,
        1, 11, 2, 8, 7, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        4, 9, 1, 4, 1, 7, 7, 1, 3, -1, -1, -1, -1, -1, -1,
        4, 9, 1, 4, 1, 7, 0, 8, 1, 8, 7, 1, -1, -1, -1,
        4, 0, 3, 7, 4, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        4, 8, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        9, 11, 8, 11, 10, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        3, 0, 9, 3, 9, 10, 10, 9, 11, -1, -1, -1, -1, -1, -1,
        0, 1, 11, 0, 11, 8, 8, 11, 10, -1, -1, -1, -1, -1, -1,
        3, 1, 11, 10, 3, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 2, 10, 1, 10, 9, 9, 10, 8, -1, -1, -1, -1, -1, -1,
        3, 0, 9, 3, 9, 10, 1, 2, 9, 2, 10, 9, -1, -1, -1,
        0, 2, 10, 8, 0, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        3, 2, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        2, 3, 8, 2, 8, 11, 11, 8, 9, -1, -1, -1, -1, -1, -1,
        9, 11, 2, 0, 9, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        2, 3, 8, 2, 8, 11, 0, 1, 8, 1, 11, 8, -1, -1, -1,
        1, 11, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        1, 3, 8, 9, 1, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 9, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        0, 3, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
 };

unsigned int edgeTable[24] = {
   // bottom
   0, 1,
   1, 2,
   2, 3,
   3, 0,

   // top
   4, 5,
   5, 6,
   6, 7,
   7, 4,

   // vertical edges
   0, 4,
   1, 5,
   3, 7,
   2, 6
};

Vector3i calcLocalCoords( card32 index )
{
   switch( index )
   {
   case 0:
      return makeVector3i( 0,0,0 );
   case 1:
      return makeVector3i( 1,0,0 );
   case 2:
      return makeVector3i( 1,1,0 );
   case 3:
      return makeVector3i( 0,1,0 );
   case 4:
      return makeVector3i( 0,0,1 );
   case 5:
      return makeVector3i( 1,0,1 );
   case 6:
      return makeVector3i( 1,1,1 );
   case 7:
      return makeVector3i( 0,1,1 );
   }
   x4Assert( false );
   return makeVector3i(0,0,0);
}

Vector3f calcLocalCoords3f( card32 index )
{
   switch( index )
   {
   case 0:
      return makeVector3f( 0,0,0 );
   case 1:
      return makeVector3f( 1,0,0 );
   case 2:
      return makeVector3f( 1,1,0 );
   case 3:
      return makeVector3f( 0,1,0 );
   case 4:
      return makeVector3f( 0,0,1 );
   case 5:
      return makeVector3f( 1,0,1 );
   case 6:
      return makeVector3f( 1,1,1 );
   case 7:
      return makeVector3f( 0,1,1 );
   }
   x4Assert( false );
   return makeVector3f(0,0,0);
}

MarchingCubes::MarchingCubes()
{
   m_VolumeData = NULL;
	m_GridSnapTolerance = 0.1f;
}

void MarchingCubes::clearMesh()
{
	m_Vertices.clear();
	m_Triangles.clear();
}

void MarchingCubes::calculateVolume( float * data, BoundingBox3f bb, Vector3i dim, float s, bool newMesh, float undefValue )
{
   m_VolumeData = data;
   m_VolumeBB = bb;
   m_VolumeDim = dim;
   m_Scale = makeVector3f(
      (bb.upperCorner[0] - bb.lowerCorner[0]) / (float)(dim[0]-1),
      (bb.upperCorner[1] - bb.lowerCorner[1]) / (float)(dim[1]-1),
      (bb.upperCorner[2] - bb.lowerCorner[2]) / (float)(dim[2]-1)
   );

	if( newMesh )
	{
		clearMesh();
	}
	m_EdgeMap.clear();

   card32 incX = 1;
   card32 incY = m_VolumeDim[0];
   card32 incZ = m_VolumeDim[1]*m_VolumeDim[0];

   int maxCase = 0;

   for( card32 x=0;x<(card32)m_VolumeDim[0]-1;x++ )
   {
      for( card32 y=0;y<(card32)m_VolumeDim[1]-1;y++ )
      {
         for( card32 z=0;z<(card32)m_VolumeDim[2]-1;z++ )
         {
            Vector3i cellCoord = makeVector3i( x,y,z);
            card32 offset = getVertexIndex( cellCoord );
            card32 index = ( m_VolumeData[offset] < s ) ? 0 : 0x1;
            index += ( m_VolumeData[offset+incX] < s ) ? 0 : 0x2;
            index += ( m_VolumeData[offset+incX+incY] < s ) ? 0 : 0x4;
            index += ( m_VolumeData[offset+incY] < s ) ? 0 : 0x8;
            index += ( m_VolumeData[offset+incZ] < s ) ? 0 : 0x10;
            index += ( m_VolumeData[offset+incZ+incX] < s ) ? 0 : 0x20;
            index += ( m_VolumeData[offset+incZ+incX+incY] < s ) ? 0 : 0x40;
            index += ( m_VolumeData[offset+incZ+incY] < s ) ? 0 : 0x80;

            bool undef = false;
            if (undefValue != 0) {
               undef |= (m_VolumeData[offset] == undefValue);
               undef |= (m_VolumeData[offset+incX] == undefValue);
               undef |= (m_VolumeData[offset+incX+incY] == undefValue);
               undef |= (m_VolumeData[offset+incY] == undefValue);
               undef |= (m_VolumeData[offset+incZ] == undefValue);
               undef |= (m_VolumeData[offset+incZ+incX] == undefValue);
               undef |= (m_VolumeData[offset+incZ+incX+incY] == undefValue);
               undef |= (m_VolumeData[offset+incZ+incY] == undefValue);
            }
            if (!undef) {
               int testCase = 0;
               for( card32 i=0;(i<15)&&(indexList[index*15+i] != -1);i+=3 )
               {
                  addTriangle( cellCoord,
                     indexList[index*15+i+0],
                     indexList[index*15+i+1],
                     indexList[index*15+i+2],s );
                  testCase++;
               }
               maxCase = max( testCase, maxCase );
            }
         }
      }
   }

}

void MarchingCubes::calculateVolume( EvaluateFunctionMethod evalMethod, void *context, BoundingBox3f bb, Vector3i dim, float s, bool newMesh )
{
	if( newMesh )
	{
		m_Vertices.clear();
		m_Triangles.clear();
	}

   Vector3f scale = makeVector3f(
      (bb.upperCorner[0] - bb.lowerCorner[0]) / (float)(dim[0]),
      (bb.upperCorner[1] - bb.lowerCorner[1]) / (float)(dim[1]),
      (bb.upperCorner[2] - bb.lowerCorner[2]) / (float)(dim[2])
   );
   Vector3f scaleSamples = makeVector3f(
      (bb.upperCorner[0] - bb.lowerCorner[0]) / (float)(dim[0]-1),
      (bb.upperCorner[1] - bb.lowerCorner[1]) / (float)(dim[1]-1),
      (bb.upperCorner[2] - bb.lowerCorner[2]) / (float)(dim[2]-1)
   );

	
	card32 blockSize = 3;
	card32 numBlockSamples = blockSize+1;
	std::vector<float> volume( numBlockSamples*numBlockSamples*numBlockSamples );

	card32 actBlockIndex = 0;
	card32 numTotalBlocks = ((dim[0]-1)/blockSize+1)
									*((dim[1]-1)/blockSize+1)
									*((dim[2]-1)/blockSize+1);

	progressWindow->pushStep( true, "mc" );
	bool aborted = false;

	for( card32 blockX=0;blockX<(dim[0]-1)/blockSize+1 && !aborted;blockX++ )
	{
		for( card32 blockY=0;blockY<(dim[1]-1)/blockSize+1 && !aborted;blockY++ )
		{
			for( card32 blockZ=0;blockZ<(dim[2]-1)/blockSize+1 && !aborted;blockZ++ )
			{
				// sample block
				card32 numBlockSamplesX=min( dim[0]-blockX*blockSize+1, numBlockSamples );
				card32 numBlockSamplesY=min( dim[1]-blockY*blockSize+1, numBlockSamples );
				card32 numBlockSamplesZ=min( dim[2]-blockZ*blockSize+1, numBlockSamples );
				Vector3i blockDim = makeVector3i(numBlockSamplesX,numBlockSamplesY,numBlockSamplesZ); 
				BoundingBox3f blockBB;
				blockBB.lowerCorner = scale.componentProduct( makeVector3f(blockX, blockY, blockZ)) 
					* (float)(numBlockSamples) + bb.lowerCorner;
				blockBB.upperCorner = blockBB.lowerCorner + scale.componentProduct( 
					makeVector3f(numBlockSamplesX,numBlockSamplesY,numBlockSamplesZ) );

				float px,py,pz;
				for( card32 x=0;x<numBlockSamplesX;x++ )
				{
					px = (float)x * scaleSamples[0] + blockBB.lowerCorner[0];
					for( card32 y=0;y<numBlockSamplesY;y++ )
					{
						py = (float)y * scaleSamples[1] + blockBB.lowerCorner[1];
						for( card32 z=0;z<numBlockSamplesZ;z++ )
						{
							pz = (float)z * scaleSamples[2] + blockBB.lowerCorner[2];
							volume[ (z*numBlockSamplesY + y)*numBlockSamplesX + x] =
								(*evalMethod)(context, px,py,pz);
						}
					}
				}

				// triangulate block
				calculateVolume( &volume[0], blockBB, blockDim, s, false );
			
				aborted = !progressWindow->progressf( (float)actBlockIndex++/(float)numTotalBlocks*100.0f );

			}
		} 
	}

	progressWindow->popSteps( &aborted );


}


void MarchingCubes::calculateHierarchical( EvaluateFunctionMethod evalMethod, void *context, IsBoundingBoxEmptyMethod isEmptyMethod, 
														BoundingBox3f bb, Vector3i dim, float s, bool newMesh, float undefValue) 
{
	m_OriginalBB = bb;

	if( newMesh )
	{
		m_Vertices.clear();
		m_Triangles.clear();
	}

	// make power-of-two cube (much easier for hierarchical mc)
	BoundingBox3f	pot_bb=bb;
	Vector3i			pot_dim = makeVector3i(1,1,1);
	while( pot_dim[0] < dim[0] || pot_dim[1] < dim[1] || pot_dim[2] < dim[2] )
	{
		pot_dim *= 2;
	}
	for( card32 i=0;i<3;i++ )
		pot_bb.upperCorner[i] += ((float)(pot_dim[i]-dim[i]))/(float)dim[i] * bb.getSideLength(i);


	progressWindow->pushStep( "mc" );
	m_CurrentBox = 0;
	m_NumBoxes = dim[0] * dim[1] * dim[2];
	m_Aborted = false;

	calcHierarchical( evalMethod, isEmptyMethod, context, pot_bb, pot_dim, s, undefValue );

	progressWindow->popSteps( &m_Aborted );
}

Vector3f MarchingCubes::findSurface( EvaluateFunctionMethod evalMethod, void *context, Vector3f v1,
   Vector3f v2, float f1, float f2, float s, card32 maxIterations )
{
   Vector3f ret;
   if( f2 > f1 )
   {
      float tmp = f1;
      f1 = f2;
      f2 = tmp;
      Vector3f tmpV = v1;
      v1 = v2;
      v2 = tmpV;
   }

   float scale_start = 1.0f / (f2-f1);

   for( card32 i=0;i<maxIterations;i++ )
   {
      Vector3f middle;
      float scale = 1.0f / (f2-f1);
      float tmpI = (s-f1) * scale;
      middle = v2 * tmpI + v1 * (1.0f - tmpI );
      ret = middle;


      float fMiddle = (*evalMethod)( context, middle[0],middle[1], middle[2] );

      if( fabs( fMiddle - s )*scale_start < 0.01f )
         return middle;

      if( fMiddle < s )
      {
         f1 = fMiddle;
         v1 = middle;
      }
      else
      {
         f2 = fMiddle;
         v2 = middle;
      }

   }

   return ret;
}


//void MarchingCubes::calculateCube( const Object * obj, EvaluateFunctionMethod eval, const BoundingBox3f &bb, float s )
void MarchingCubes::calculateCube( EvaluateFunctionMethod eval, void *context, const BoundingBox3f &bb, float s,
   card32 maxIterations, bool newMesh, float undefValue )
{
   float v[8];
/*   v[0] = (obj->*eval)( bb.lowerCorner[0], bb.lowerCorner[1], bb.lowerCorner[2] ) ;
   v[1] = (obj->*eval)( bb.upperCorner[0], bb.lowerCorner[1], bb.lowerCorner[2] );
   v[2] = (obj->*eval)( bb.upperCorner[0], bb.upperCorner[1], bb.lowerCorner[2] );
   v[3] = (obj->*eval)( bb.lowerCorner[0], bb.upperCorner[1], bb.lowerCorner[2] );
   v[4] = (obj->*eval)( bb.lowerCorner[0], bb.lowerCorner[1], bb.upperCorner[2] );
   v[5] = (obj->*eval)( bb.upperCorner[0], bb.lowerCorner[1], bb.upperCorner[2] );
   v[6] = (obj->*eval)( bb.upperCorner[0], bb.upperCorner[1], bb.upperCorner[2] );
   v[7] = (obj->*eval)( bb.lowerCorner[0], bb.upperCorner[1], bb.upperCorner[2] );
*/
   v[0] = (*eval)( context, bb.lowerCorner[0], bb.lowerCorner[1], bb.lowerCorner[2] ) ;
   v[1] = (*eval)( context, bb.upperCorner[0], bb.lowerCorner[1], bb.lowerCorner[2] );
   v[2] = (*eval)( context, bb.upperCorner[0], bb.upperCorner[1], bb.lowerCorner[2] );
   v[3] = (*eval)( context, bb.lowerCorner[0], bb.upperCorner[1], bb.lowerCorner[2] );
   v[4] = (*eval)( context, bb.lowerCorner[0], bb.lowerCorner[1], bb.upperCorner[2] );
   v[5] = (*eval)( context, bb.upperCorner[0], bb.lowerCorner[1], bb.upperCorner[2] );
   v[6] = (*eval)( context, bb.upperCorner[0], bb.upperCorner[1], bb.upperCorner[2] );
   v[7] = (*eval)( context, bb.lowerCorner[0], bb.upperCorner[1], bb.upperCorner[2] );

   if (undefValue != 0.0f) {
      if (   v[0] == undefValue || v[1] == undefValue || v[2] == undefValue || v[3] == undefValue
          || v[4] == undefValue || v[5] == undefValue || v[6] == undefValue || v[7] == undefValue) {
         return;
      }
   }

   if( newMesh )
	{
		m_Vertices.clear();
		m_Triangles.clear();
	}

   card32
   index  = (v[0] < s) ? 0 : 0x1;
   index |= (v[1] < s) ? 0 : 0x2;
   index |= (v[2] < s) ? 0 : 0x4;
   index |= (v[3] < s) ? 0 : 0x8;
   index |= (v[4] < s) ? 0 : 0x10;
   index |= (v[5] < s) ? 0 : 0x20;
   index |= (v[6] < s) ? 0 : 0x40;
   index |= (v[7] < s) ? 0 : 0x80;

   for( card32 i=0;(i<15)&&(indexList[index*15+i] != -1); )
   {
		mpcard triIndex = m_Vertices.size();
      m_Triangles.push_back( makeVector3i( triIndex, triIndex+1, triIndex+2 ) );

      for( card32 e=0;e<3;e++ )
      {
         card32 edge = indexList[index*15+i+e];
         if( edge > 11 )
            edge = 0;
         Vector3f v1 = calcLocalCoords3f( edgeTable[ edge*2+0] );
         Vector3f v2 = calcLocalCoords3f( edgeTable[ edge*2+1] );

         float f1 = v[ edgeTable[ edge*2+0] ];
         float f2 = v[ edgeTable[ edge*2+1] ];


         Vector3f vec;
         if( maxIterations == 0 )
         {
            float i1,i2;
				if( f1 > f2 )
				{
					i1 = (s-f1) / (f2-f1);
				}
				else
				{
					i1 = 1.0f - (s-f2) / (f1-f2);
				}

				if( i1 < m_GridSnapTolerance )
					i1 = 0.0f;
				if( i1 > 1.0f - m_GridSnapTolerance )
					i1 = 1.0f;
				i2 = 1.0f - i1;

				vec = v2 * i1 + v1 * i2;

				vec = vec.componentProduct(bb.upperCorner-bb.lowerCorner) + bb.lowerCorner;
         }
         else
         {
            v1 = v1.componentProduct(bb.upperCorner-bb.lowerCorner) + bb.lowerCorner;
            v2 = v2.componentProduct(bb.upperCorner-bb.lowerCorner) + bb.lowerCorner;

            vec = findSurface( eval, context, v1,v2,f1,f2, s, maxIterations );
         }
         m_Vertices.push_back( vec );
      }

		i += 3;

   }

}

void MarchingCubes::calculateCube( float *v, const BoundingBox3f &bb, float s, bool newMesh )
{
	if( newMesh )
	{
		m_Vertices.clear();
		m_Triangles.clear();
	}
	Vector3i startIndices = makeVector3i(1,1,1) * (int32)m_Vertices.size();

   card32
   index  = (v[0] < s) ? 0 : 0x1;
   index |= (v[1] < s) ? 0 : 0x2;
   index |= (v[2] < s) ? 0 : 0x4;
   index |= (v[3] < s) ? 0 : 0x8;
   index |= (v[4] < s) ? 0 : 0x10;
   index |= (v[5] < s) ? 0 : 0x20;
   index |= (v[6] < s) ? 0 : 0x40;
   index |= (v[7] < s) ? 0 : 0x80;

   for( card32 i=0;(i<15)&&(indexList[index*15+i] != -1);i+=3 )
   {
      m_Triangles.push_back( makeVector3i( i, i+1, i+2 ) + startIndices );

      for( card32 e=0;e<3;e++ )
      {
         card32 edge = indexList[index*15+i+e];
         if( edge > 11 )
            edge = 0;
         Vector3f v1 = calcLocalCoords3f( edgeTable[ edge*2+0] );
         Vector3f v2 = calcLocalCoords3f( edgeTable[ edge*2+1] );

         float f1 = v[ edgeTable[ edge*2+0] ];
         float f2 = v[ edgeTable[ edge*2+1] ];

         Vector3f vec;
         if( f1 > f2 )
            vec = v2 * (s-f1) / (f2-f1) + v1 * (1.0f - (s-f1) / (f2-f1));
         else
            vec = v1 * (s-f2) / (f1-f2) + v2 * (1.0f - (s-f2) / (f1-f2));

			Vector3f ret;
			float i1,i2;
			if( f1 > f2 )
			{
				i1 = (s-f1) / (f2-f1);
			}
			else
			{
				i1 = 1.0f - (s-f2) / (f1-f2);
			}

			if( i1 < m_GridSnapTolerance )
				i1 = 0.0f;
			if( i1 > 1.0f - m_GridSnapTolerance )
				i1 = 1.0f;
			i2 = 1.0f - i1;

			ret = v2 * i1 + v1 * i2;

			
			ret = ret.componentProduct(bb.upperCorner-bb.lowerCorner) + bb.lowerCorner;
         m_Vertices.push_back( ret );

      }

   }

}


card32 MarchingCubes::getVertexIndex( const Vector3i &coord )
{
    return (coord[2] * m_VolumeDim[1] + coord[1])*m_VolumeDim[0] + coord[0];
}

int MarchingCubes::getEdgeVertexIndex( card32 cellIndex, card32 edge )
{
   makeUnique(cellIndex,edge);

   if( m_EdgeMap.find( cellIndex ) == m_EdgeMap.end() )
      return -1;
   return m_EdgeMap[ cellIndex ].edge_vertex[edge];
}

void MarchingCubes::setEdgeVertexIndex( card32 cellIndex, card32 edge, card32 index )
{
   makeUnique(cellIndex,edge);
   m_EdgeMap[cellIndex].edge_vertex[edge] = index;
}


Vector3f MarchingCubes::calcEdgeVertex( const Vector3i &cellCoord, card32 edge,float s )
{
   Vector3i v1i = cellCoord + calcLocalCoords( edgeTable[edge*2+0] );
   Vector3i v2i = cellCoord + calcLocalCoords( edgeTable[edge*2+1] );
   Vector3f v1 = makeVector3f( (float)v1i[0],(float)v1i[1],(float)v1i[2] );
   Vector3f v2 = makeVector3f( (float)v2i[0],(float)v2i[1],(float)v2i[2] );

   card32 offset1 = getVertexIndex( v1i );
   card32 offset2 = getVertexIndex( v2i );
   float f1 = m_VolumeData[offset1];
   float f2 = m_VolumeData[offset2];


   Vector3f ret;
	float i1,i2;
   if( f1 > f2 )
	{
		i1 = (s-f1) / (f2-f1);
	}
   else
	{
		i1 = 1.0f - (s-f2) / (f1-f2);
	}

	if( i1 < m_GridSnapTolerance )
		i1 = 0.0f;
	if( i1 > 1.0f - m_GridSnapTolerance )
		i1 = 1.0f;
	i2 = 1.0f - i1;

	ret = v2 * i1 + v1 * i2;

   for( card32 i=0;i<3;i++ )
   {
      ret[i] *= m_Scale[i];
   }

   ret += m_VolumeBB.lowerCorner;
	return ret;
}

void MarchingCubes::addTriangle( const Vector3i &cellCoord, card32 e1,card32 e2,card32 e3,float s )
{
   card32 cellIndex = getVertexIndex( cellCoord );
   int id1 = getEdgeVertexIndex(cellIndex,e1);
   int id2 = getEdgeVertexIndex(cellIndex,e2);
   int id3 = getEdgeVertexIndex(cellIndex,e3);

   Vector3i triangle;


   if( id1 == -1 )
   {
      // push_back new vertex
      triangle[0] = (int32)m_Vertices.size();
      setEdgeVertexIndex( cellIndex, e1, triangle[0] );
      m_Vertices.push_back( calcEdgeVertex( cellCoord, e1, s) );
   }
   else
      triangle[0] = id1; // get index of existing vertex


   if( id2 == -1 )
   {
      // push_back new vertex
      triangle[1] = (int32)m_Vertices.size();
      setEdgeVertexIndex( cellIndex, e2, triangle[1] );
      m_Vertices.push_back( calcEdgeVertex( cellCoord, e2, s) );
   }
   else
      triangle[1] = id2; // get index of existing vertex

   if( id3 == -1 )
   {
      // push_back new vertex
      triangle[2] = (int32)m_Vertices.size();
      setEdgeVertexIndex( cellIndex, e3, triangle[2] );
      m_Vertices.push_back( calcEdgeVertex( cellCoord, e3, s) );
   }
   else
      triangle[2] = id3; // get index of existing vertex

   //m_VertexPointer.clear();
   m_Triangles.push_back( triangle );
}

void MarchingCubes::makeUnique(card32 &cellIndex,card32 &edge)
{
   // change cellIndex & edge to create a unique access id for shared vertices
   // details at: http://www.essi.fr/~lingrand/MarchingCubes/algo.html
   // take a look at the single-cube figure
   int xInc = 1;
   int yInc = m_VolumeDim[0];
   int zInc = m_VolumeDim[0]*m_VolumeDim[1];
   edge++;
   switch( edge )
   {
   case 1:
   break;
   case 2:
      edge = 4;
      cellIndex += xInc;
   break;
   case 3:
      edge = 1;
      cellIndex += yInc;
   break;
   case 4:
   break;
   case 5:
      edge = 1;
      cellIndex += zInc;
   break;
   case 6:
      edge = 4;
      cellIndex += zInc + xInc;
   break;
   case 7:
      edge = 1;
      cellIndex += zInc + yInc;
   break;
   case 8:
      edge = 4;
      cellIndex += zInc;
   break;
   case 9:
   break;
   case 10:
      edge = 9;
      cellIndex += xInc;
   break;
   case 11:
      edge = 9;
      cellIndex += yInc;
   break;
   case 12:
      edge = 9;
      cellIndex += xInc + yInc;
   break;
   }
   edge--;
}

void MarchingCubes::cutTriangles( const Plane3f &p )
{
   std::vector<Vector3f>  vertices;
   std::vector<Vector3i>  triangles;
   for( card32 i=0;i<getNumTriangles();i++ )
   {
      Polygon<TMC_Vertex> poly;
      for( card32 j=0;j<3;j++ )
      {
         TMC_Vertex v;
         v.pos = m_Vertices[ m_Triangles[i][j] ];
         poly.addVertex( v );
      }
      //Plane testPlane = poly.calculatePlane();
      EPlaneSide side = poly.classify( p );
      if( side == PS_SPANNING )
      {
         Polygon<TMC_Vertex> poly_tmp = poly.cut( p );
         poly = poly_tmp;
      }

      if( poly.isValid() && side != PS_BEHIND)
      {
         for( card32 k=0;k<poly.getNumVertices()-2;k++ )
         {
            triangles.push_back( makeVector3i( (int)vertices.size(),
               (int)vertices.size()+1,(int)vertices.size()+2 ));

            vertices.push_back( poly[0].pos );
            vertices.push_back( poly[k+1].pos );
            vertices.push_back( poly[k+2].pos );
         }
      }

   }

   m_Vertices = vertices;
   m_Triangles = triangles;

}

void MarchingCubes::cutTriangles( const std::vector<Plane3f> &planes, bool complex_case )
{
   std::vector<Polygon<TMC_Vertex> > polys;
   for( card32 i=0;i<getNumTriangles();i++ )
   {
      Polygon<TMC_Vertex> poly;
      for( card32 j=0;j<3;j++ )
      {
         TMC_Vertex v;
         v.pos = m_Vertices[ m_Triangles[i][j] ];
         poly.addVertex( v );
      }
      polys.push_back( poly );
   }

   if( complex_case == false )
   {
      for( card32 j=0;j<polys.size();j++ )
      {
         if( polys[j].isValid() )
            for( card32 i=0;i<planes.size();i++ )
            {
               EPlaneSide side = polys[j].classify( planes[i] );
               if( side == PS_BEHIND )
               {
                  polys[j].clear();
                  break;
               }
               else
                  if( side == PS_SPANNING )
                  {
                     Polygon<TMC_Vertex> poly_tmp = polys[j].cut( planes[i] );
                     polys[j] = poly_tmp;
                  }
            }
      }
   }

   if( complex_case && planes.size())
   {
      std::vector<Polygon<TMC_Vertex> > goodPolys;
      for( card32 i=0;i<polys.size();i++ )
      {
         Polygon<TMC_Vertex> poly = polys[i];
         for( card32 j=0;j<planes.size();j++ )
         {
            EPlaneSide side = poly.classify( planes[j] );
            std::vector<Polygon<TMC_Vertex> > tmpPolys;
            if( side == PS_INFRONT )
            {
               tmpPolys.push_back( poly );
            }
            else
               if( side == PS_SPANNING )
               {
                  Polygon<TMC_Vertex> positivePoly = poly.cut( planes[j] );
                  Plane3f inv_plane = planes[j];
                  inv_plane.normal = -inv_plane.normal;
                  Polygon<TMC_Vertex> negativePoly = poly.cut( inv_plane );
                  poly = negativePoly;
                  tmpPolys.push_back( positivePoly );
               }
            if( tmpPolys.size() > 0 )
            {
               for( card32 k=j+1;k<planes.size();k++ )
               {
                  card32 numTmpPolys = (card32)tmpPolys.size();
                  for( card32 m=0;m<numTmpPolys;m++ )
                     if( tmpPolys[m].classify( planes[k] ) == PS_SPANNING )
                     {
                        Polygon<TMC_Vertex> positivePoly = tmpPolys[m].cut( planes[k] );
                        Plane3f inv_plane = planes[k];
                        inv_plane.normal = -inv_plane.normal;
                        Polygon<TMC_Vertex> negativePoly = tmpPolys[m].cut( inv_plane );
                        tmpPolys[m] = negativePoly;
                        tmpPolys.push_back( positivePoly );
                     }

               }
            }
            for( card32 k=0;k<tmpPolys.size();k++ )
               goodPolys.push_back( tmpPolys[k] );
            tmpPolys.clear();
         }
      }
      polys = goodPolys;
   }

   m_Vertices.clear();
   m_Triangles.clear();
   for( card32 i=0;i<polys.size();i++ )
      if( polys[i].isValid() )
      {
         for( card32 k=0;k<polys[i].getNumVertices()-2;k++ )
         {
            m_Triangles.push_back( makeVector3i( (int)m_Vertices.size(),
               (int)m_Vertices.size()+1,(int)m_Vertices.size()+2 ));

            m_Vertices.push_back( polys[i][0].pos );
            m_Vertices.push_back( polys[i][k+1].pos );
            m_Vertices.push_back( polys[i][k+2].pos );
         }
      }
}

MarchingCubes::TMC_Vertex MarchingCubes::TMC_Vertex::operator + ( const TMC_Vertex &v ) const
{
  TMC_Vertex ret;
  ret.pos = pos + v.pos;
  return ret;
}

MarchingCubes::TMC_Vertex MarchingCubes::TMC_Vertex::operator - ( const TMC_Vertex &v ) const
{
  TMC_Vertex ret;
  ret.pos = pos - v.pos;
  return ret;
}

MarchingCubes::TMC_Vertex MarchingCubes::TMC_Vertex::operator * ( float f ) const
{
  TMC_Vertex ret;
  ret.pos = pos*f;
  return ret;
}


void MarchingCubes::calcHierarchical( EvaluateFunctionMethod evalMethod, IsBoundingBoxEmptyMethod isEmptyMethod, void *context, const BoundingBox3f &bb, const Vector3i &dim, float s, float undefValue ) 
{
	if( m_Aborted )
		return;

	bool intersect = false;
	for( card32 i=0;i<3;i++ )
	{
		if( bb.lowerCorner[i] < m_OriginalBB.upperCorner[i] )
			intersect = true;
	}

	if( !intersect )
	{
		m_CurrentBox += dim[0] * dim[1] * dim[2];
		m_Aborted = !progressWindow->progress( (float)m_CurrentBox / (float)m_NumBoxes * 100.0f );
		return;
	}

	if( dim[0] > 1 )
	{
		if( (*isEmptyMethod)(context, bb) )
		{
			m_CurrentBox += dim[0] * dim[1] * dim[2];
			m_Aborted = !progressWindow->progress( (float)m_CurrentBox / (float)m_NumBoxes * 100.0f );
			return;	// empty box
		}

		// traverse children
		Vector3i child_dim = dim / 2;
		BoundingBox3f child_bb;
		float side_length_2 = bb.getSideLength(0) / 2.0f;
		for( card32 i=0;i<8;i++ )
		{
			child_bb = bb;
			if( i&1 )
				child_bb.lowerCorner[0] += side_length_2;
			else
				child_bb.upperCorner[0] -= side_length_2;
			if( i&2 )
				child_bb.lowerCorner[1] += side_length_2;
			else
				child_bb.upperCorner[1] -= side_length_2;
			if( i&4 )
				child_bb.lowerCorner[2] += side_length_2;
			else
				child_bb.upperCorner[2] -= side_length_2;

			calcHierarchical(evalMethod, isEmptyMethod, context, child_bb, child_dim, s, undefValue);
		}
	}
	else
	{
		// process single cube
		calculateCube( evalMethod, context, bb, s, 0, false, undefValue );
		m_CurrentBox++;
		m_Aborted = !progressWindow->progress( (float)m_CurrentBox / (float)m_NumBoxes * 100.0f );

	}


}
