//---------------------------------------------------------------------------
#ifndef RGBAImageH
#define RGBAImageH
//---------------------------------------------------------------------------
#include "x4miscFoundation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "LinearAlgebra.h"
#include "TemplatedImage.h"
#include <QtGui/QImage>
//---------------------------------------------------------------------------
namespace X4 {


class RGBImage;


class X4MISCFOUNDATION_API RGBAImage : public TemplatedImage<Vector4ub> {
 private:
	X4_CLASS(RGBAImage);

   inline bool checkBorder(card32 x, card32 y, Vector4ub &avCol);
   void _interactiveLoadFromFile();

 public:

   static RGBAImage* interactiveLoadFromFile();
   static RGBAImage* loadFromFile(const std::string& filename);

   RGBAImage() {}
   RGBAImage(card32 sizeX, card32 sizeY) : TemplatedImage<Vector4ub>(sizeX, sizeY) {}

   RGBAImage(const RGBAImage &t) { *this = t; }
   RGBAImage &operator=(const RGBAImage &t) { assign(&t); return *this; }


   // mark all pixels with color cMask as transparent, alpha component == index 3
   void chromaMask(Vector4ub cMask);
   // set color of pixels at the border to neighboring color values
   void maskBorder();
   // zoom down image with anti-aliasing, ignoring alpha == 0 entries
   void downsampleWBorder(card32 factor, RGBAImage *dest, Vector4ub maskColor);
   // zoom down with anti-aliasing, no chroma keying
   void downsampleAA(card32 factor, RGBAImage *dest);
   // drop alpha channel (color index 3)
   RGBImage *convertToRGB();
   // change from rgba to argb format (in place)
   void convertToBGRA();
   // create q image
   QImage * getQImage() const;

   // mb - converts image to normal + height std::map
   // RGB = normal, A = height
   // height = (R + G + B) / 3
   void convertToNormalMap();

   void loadGLTexture();
   void loadGLTextureMipmapped();
   void loadGLSubTexture(card32 x, card32 y);
   void readPixelsGL(card32 x=0, card32 y=0);
   void drawPixelsGL(float32 x=0.0f, float32 y=0.0f);
   void loadFromFile(const char* bmpFile);
   void loadFromMemory(unsigned w, unsigned h, char* data);
   void copyFromBMP(const QImage *img, card32 x, card32 y, card32 width, card32 height);
   RGBAImage* flipY();
};

class X4MISCFOUNDATION_API RGBImage : public TemplatedImage<Vector3ub> {
	X4_CLASS(RGBImage);

 public:
   RGBImage() {}
   RGBImage(card32 sizeX, card32 sizeY) : TemplatedImage<Vector3ub>(sizeX, sizeY) {}

   // change from rgb to bgr format (in place)
   void convertToBGR();

   void loadGLTexture();
   void loadGLTextureMipmapped();
   void loadGLSubTexture(card32 x, card32 y);
};

class X4MISCFOUNDATION_API OneByteImage : public TemplatedImage<card8> {
	X4_CLASS(OneByteImage);

 public:
   OneByteImage() {}
   OneByteImage(card32 sizeX, card32 sizeY) : TemplatedImage<card8>(sizeX, sizeY) {}

   void loadGLTexture();
   void loadGLTextureMipmapped();
   void loadGLSubTexture(card32 x, card32 y);
   void readPixelsFromDepthGL(card32 x=0, card32 y=0);
   void drawPixelsGL(float32 x=0.0f, float32 y=0.0f);
};

class X4MISCFOUNDATION_API RGBAFloat32Image : public TemplatedImage<Vector4f>
{
	X4_CLASS(RGBAFloat32Image);

public:

   RGBAFloat32Image() {}
   RGBAFloat32Image( card32 sizeX, card32 sizeY ) : TemplatedImage<Vector4f>(sizeX, sizeY) {}

   bool loadImage(const std::string &filename);
};


}

#endif

