#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SimpleAttachment.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "VariableArrayClassProperty.h"
#include "Object.h"
#include "AttachedDataContainer.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( SimpleAttachment , 0)
{
	BEGIN_CLASS_INIT( SimpleAttachment );
	ADD_VAR_ARRAY_PROP( Byte, 0, new Int8ClassProperty("element", MetaClass::FIRST_VERSION,NULL) );
	SET_EDIT_READONLY(true);
}

SimpleAttachment::SimpleAttachment(void)
{
}

SimpleAttachment::~SimpleAttachment(void)
{
}

void SimpleAttachment::assign(const Object* obj, X4CopyContext *context)
{
	const SimpleAttachment * o = dynamic_cast<const SimpleAttachment*>(obj);
	x4Assert( o != NULL );

	//copyObjectProperties(obj,this);
	m_Data = o->m_Data;

}


bool X4::SimpleAttachment::exists( AttachedDataContainer * att, const std::string &name )
{
	return dynamic_cast<SimpleAttachment*>(att->getData(name));
}

void * X4::SimpleAttachment::get( AttachedDataContainer * att, const std::string &name )
{
	SimpleAttachment * s = dynamic_cast<SimpleAttachment*>(att->getData(name));
	if( s == NULL )
		throw X4Exception(std::string("SimpleAttachment::get() - Attachment ") + name  + " doesn't exists!");
	return &s->m_Data[0];
}

void X4::SimpleAttachment::set( AttachedDataContainer * att, const std::string &name, const void * ptr, card32 numBytes, bool persistent )
{
	SimpleAttachment * s = dynamic_cast<SimpleAttachment*>(att->getData(name));
	if( s != NULL )
	{
		if( s->m_Data.size() != numBytes )
		{
			throw X4Exception(std::string("SimpleAttachment::set() - Attachment ") + name  + " already exists, but with a different number of bytes!");
		}
	}
	else
	{
		s = new SimpleAttachment;
		s->setup(name, persistent ? AttachedData::ADF_PERSISTENT:0,NULL );
		s->m_Data.resize(numBytes);
		att->attachData(s);
	}
	memcpy(&s->m_Data[0],ptr,numBytes);
}

template<class T>
T simpleAttachment_templated_get(AttachedDataContainer * att, const std::string &name )
{
	T * ret = (T*)SimpleAttachment::get(att,name);
	return *ret;
}

template<class T>
void simpleAttachment_templated_set(AttachedDataContainer * att, const std::string &name,const T &val,bool persistent )
{
	SimpleAttachment::set(att,name, &val, sizeof(T), persistent);
}


X4::float32 X4::SimpleAttachment::get1f( AttachedDataContainer * att, const std::string &name )
{
	return simpleAttachment_templated_get<float32>(att,name);
}

void X4::SimpleAttachment::set1f( AttachedDataContainer * att, const std::string &name, const float32 &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}

X4::Vector2f X4::SimpleAttachment::get2f( AttachedDataContainer * att, const std::string &name )
{
	return simpleAttachment_templated_get<Vector2f>(att,name);
}

void X4::SimpleAttachment::set2f( AttachedDataContainer * att, const std::string &name, const Vector2f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}

X4::Vector3f X4::SimpleAttachment::get3f( AttachedDataContainer * att, const std::string &name )
{
	return simpleAttachment_templated_get<Vector3f>(att,name);

}

void X4::SimpleAttachment::set3f( AttachedDataContainer * att, const std::string &name, const Vector3f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}

X4::Vector4f X4::SimpleAttachment::get4f( AttachedDataContainer * att, const std::string &name )
{	
	return simpleAttachment_templated_get<Vector4f>(att,name);
}

void X4::SimpleAttachment::set4f( AttachedDataContainer * att, const std::string &name, const Vector4f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}

X4::Matrix3f X4::SimpleAttachment::getMatrix3f( AttachedDataContainer * att, const std::string &name )
{
	return simpleAttachment_templated_get<Matrix3f>(att,name);

}

void X4::SimpleAttachment::setMatrix3f( AttachedDataContainer * att, const std::string &name, const Matrix3f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);

}

X4::Matrix4f X4::SimpleAttachment::getMatrix4f( AttachedDataContainer * att, const std::string &name )
{
	return simpleAttachment_templated_get<Matrix4f>(att,name);
}

void X4::SimpleAttachment::setMatrix4f( AttachedDataContainer * att, const std::string &name, const Matrix4f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}

X4::int32 X4::SimpleAttachment::get1i( AttachedDataContainer * att, const std::string &name )
{
	return simpleAttachment_templated_get<int32>(att,name);
}

void X4::SimpleAttachment::set1i( AttachedDataContainer * att, const std::string &name, const int32 &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}