#ifndef SimpleAttachmentH
#define SimpleAttachmentH
//---------------------------------------------------------------------------
#include "x4miscFoundation.h"
//---------------------------------------------------------------------------
#include "AttachedData.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	simple fixed-size memory attachment
*/
class X4MISCFOUNDATION_API SimpleAttachment : public AttachedData
{
	X4_CLASS( SimpleAttachment )

private:

	std::vector<char> m_Data;

	void setNumBytes(card32 size) {m_Data.resize(size);}
	void deleteByte( card32 index ) {m_Data.erase( m_Data.begin() + index);}
	char * getByte( card32 index ) {return &m_Data[index];}
	mpcard getNumBytes() {return m_Data.size();}
	std::string getByteName(card32 index ) {return "";}

public:
	SimpleAttachment(void);
	~SimpleAttachment(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	// basic access methods
	static bool exists( AttachedDataContainer * att, const std::string &name);
	static void * get( AttachedDataContainer * att, const std::string &name);
	static void set( AttachedDataContainer * att, const std::string &name, const void * ptr, card32 numBytes,bool persistent=true);

	// specialized attachments
	static float32 get1f(AttachedDataContainer * att, const std::string &name);
	static void set1f(AttachedDataContainer * att, const std::string &name, const float32 &v,bool persistent=true);
	static Vector2f get2f(AttachedDataContainer * att, const std::string &name);
	static void set2f(AttachedDataContainer * att, const std::string &name, const Vector2f &v,bool persistent=true);
	static Vector3f get3f(AttachedDataContainer * att, const std::string &name);
	static void set3f(AttachedDataContainer * att, const std::string &name, const Vector3f &v,bool persistent=true);
	static Vector4f get4f(AttachedDataContainer * att, const std::string &name);
	static void set4f(AttachedDataContainer * att, const std::string &name, const Vector4f &v,bool persistent=true);
	static int32 get1i(AttachedDataContainer * att, const std::string &name);
	static void set1i(AttachedDataContainer * att, const std::string &name, const int32 &v,bool persistent=true);

	static Matrix3f getMatrix3f(AttachedDataContainer * att, const std::string &name);
	static void setMatrix3f(AttachedDataContainer * att, const std::string &name, const Matrix3f &v,bool persistent=true);
	static Matrix4f getMatrix4f(AttachedDataContainer * att, const std::string &name);
	static void setMatrix4f(AttachedDataContainer * att, const std::string &name, const Matrix4f &v,bool persistent=true);

	// templates for DynamicVectors
	template<class FloatType>
	static DynamicVector<FloatType> getDVector(AttachedDataContainer * att, const std::string &name)
	{
		SimpleAttachment * s = dynamic_cast<SimpleAttachment*>(att->getData(name));
		if( s == NULL )
			throw X4Exception(std::string("SimpleAttachment::get() - Attachment ") + name  + " doesn't exists!");

		mpcard numElements = s->m_Data.size() / sizeof(FloatType);
		DynamicVector<FloatType> ret(numElements);
		memcpy(&ret[0],&s->m_Data[0],s->m_Data.size());
		return ret;
	}
	template<class FloatType>
	static void setDVector(AttachedDataContainer * att, const std::string &name, const DynamicVector<FloatType> &v)
	{
		set(att,name, v.dataPtr(),v.getDim()*sizeof(FloatType));
	}

	// templates for fixed-size elements
	template<class SimpleType>
	static SimpleType get_generic(AttachedDataContainer * att, const std::string &name)
	{
		SimpleType * ret = (SimpleType*)get(att,name);
		return *ret;
	}

	template<class SimpleType>
	static void set_generic(AttachedDataContainer * att, const std::string &name, const SimpleType &v,bool persistent=true)
	{
		set(att,name, &v, sizeof(SimpleType), persistent);
	}


};



}

#endif