#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "Statistics.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "Object.h"
#include "ObjectListProperty.h"
#include "StringClassProperty.h"
//---------------------------------------------------------------------------
using namespace X4;

Statistics * X4::globalStats = new Statistics;
Statistics * X4::stats = globalStats;

IMPLEMENT_X4_CLASS( Statistics ,0)
{
	BEGIN_CLASS_INIT( Statistics );
	ADD_OBJECT_LIST_PROP(Stats,0,StatisticEntry::getClass());
}

IMPLEMENT_X4_CLASS( StatisticEntry ,1)
{
	BEGIN_CLASS_INIT( StatisticEntry );

	ADD_STRING_PROP(m_Name, 1);
}

Statistics::Statistics(void)
{
	m_CurrentStats = 0;
	StatisticEntry* entry = new StatisticEntry;
	entry->setup("_default_");	// global
	m_Stats.push_back(entry);
}

Statistics::~Statistics(void)
{
	for( unsigned i=0;i<m_Stats.size();i++ )
		delete m_Stats[i];
}

void Statistics::assign(const Object* obj, X4CopyContext *context)
{
	// Persistent::assign(obj,context);
	const Statistics * o = dynamic_cast<const Statistics*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void Statistics::setCurrentMeasurementID(const std::string &id) 
{
	if( id == "_default_" )
	{
		m_CurrentStats = 0;
		return;
	}
	if( m_Stats[m_CurrentStats]->getName() == id)
		return;

	// search
	for( unsigned i=0;i<m_Stats.size();i++ )
	{
		if( m_Stats[i]->getName() == id )
		{
			m_CurrentStats = i;
			return;
		}
	}

	// create new
	StatisticEntry* entry = new StatisticEntry;
	entry->setup(id);
	m_CurrentStats = (int)m_Stats.size();
	m_Stats.push_back(entry);
}

void Statistics::clearTimer(const std::string &timer_id )	/// deletes all existing measurements 
{
	m_Stats[m_CurrentStats]->clearTimer(timer_id);
}

void Statistics::startTimer(const std::string &timer_id ) 
{
	m_Stats[m_CurrentStats]->startTimer(timer_id);
}

void Statistics::stopTimer(const std::string &timer_id ) 
{
	m_Stats[m_CurrentStats]->stopTimer(timer_id);
}

void Statistics::resetAverageTimer(const std::string &timer_id) 
{
	m_Stats[m_CurrentStats]->resetAverageTimer(timer_id);
}

void Statistics::startAverageTimer(const std::string &timer_id) 
{
	m_Stats[m_CurrentStats]->startAverageTimer(timer_id);
}

void Statistics::stopAverageTimer(const std::string &timer_id) 
{
	m_Stats[m_CurrentStats]->stopAverageTimer(timer_id);
}

void Statistics::resetCounter(const std::string &counter_id) 
{
	m_Stats[m_CurrentStats]->resetCounter(counter_id);
}

void Statistics::incCounter(const std::string &counter_id, unsigned inc) 
{
	m_Stats[m_CurrentStats]->incCounter(counter_id,inc);
}

X4::StatisticEntry::StatisticEntry( void )
{

}

X4::StatisticEntry::~StatisticEntry( void )
{

}

void X4::StatisticEntry::assign( const Object* obj, X4CopyContext *context /*= NULL*/ )
{
	const StatisticEntry * o = dynamic_cast<const StatisticEntry*>(obj);
	m_SimpleTimer = o->m_SimpleTimer;
	m_AverageTimer = o->m_AverageTimer;
	m_Counter = o->m_Counter;

	copyObjectProperties(obj,this);
}

void X4::StatisticEntry::setup( std::string id )
{
	m_Name = id;
}

void X4::StatisticEntry::clearTimer( const std::string &timer_id )
{
	m_SimpleTimer[timer_id].clear();
}

void X4::StatisticEntry::startTimer(const std::string &timer_id )
{
	std::string tmpID = ">>";
	tmpID += timer_id;
	m_TempTimer[tmpID] = m_Timer.getCurrentValue();
}

void X4::StatisticEntry::stopTimer(const std::string &timer_id )
{
	std::string tmpID = ">>";
	tmpID += timer_id;
	m_SimpleTimer[timer_id].push_back(m_Timer.getCurrentValue() - m_TempTimer[tmpID]);
}

void X4::StatisticEntry::resetAverageTimer(const std::string &timer_id)
{
	if( m_AverageTimer.find(timer_id) != m_AverageTimer.end())
		m_AverageTimer.erase(timer_id);
}

void X4::StatisticEntry::startAverageTimer(const std::string &timer_id)
{
	std::string tmpID = "||";
	tmpID += timer_id;
	m_TempTimer[tmpID] = m_Timer.getCurrentValue();
}

void X4::StatisticEntry::stopAverageTimer(const std::string &timer_id)
{
	std::string tmpID = "||";
	tmpID += timer_id;
	unsigned measurement = m_Timer.getCurrentValue() - m_TempTimer[tmpID];


	if( m_AverageTimer.find(timer_id) == m_AverageTimer.end())
	{
		m_AverageTimer[timer_id].minEntry = measurement;
		m_AverageTimer[timer_id].maxEntry = measurement;
		m_AverageTimer[timer_id].total = measurement;
		m_AverageTimer[timer_id].numMeasurements = 1;
	}
	else
	{
		m_AverageTimer[timer_id].minEntry = min(measurement,m_AverageTimer[timer_id].minEntry);
		m_AverageTimer[timer_id].maxEntry = max(measurement,m_AverageTimer[timer_id].maxEntry);
		m_AverageTimer[timer_id].total += measurement;
		m_AverageTimer[timer_id].numMeasurements += 1;
	}
}

void X4::StatisticEntry::resetCounter(const std::string &counter_id)
{
	if( m_Counter.find(counter_id) != m_Counter.end())
		m_Counter.erase(counter_id);
}

void X4::StatisticEntry::incCounter(const std::string &counter_id, unsigned inc)
{
	m_Counter[counter_id] += inc;
}

std::string X4::StatisticEntry::report()
{
	std::string ret;
	ret  = "=============================\n";
	ret += "id: ";
	if( m_Name == "")
		ret += "Global Statistics";
	else
		ret += m_Name;
	ret += "\n=============================\n";

	// simple timers
	for( stdext::hash_map<std::string, std::vector<unsigned> >::iterator iter = m_SimpleTimer.begin();
		iter != m_SimpleTimer.end(); ++iter )
	{
		ret += "Timer: ";
		ret += iter->first + "\n";
		const std::vector<unsigned> m = iter->second;
		if( m.size() == 0)
		{
			ret += "no measurements\n";
		}
		else
		if( m.size() == 1)
		{
			ret += convertTimeToString(m[0]) + "\n";
		}
		else
		{
			unsigned total = 0;
			for( unsigned i=0;i<m.size();i++ )
				total += m[i];
			ret += "Total: ";
			ret += convertTimeToString(total);
			ret += "\tAverage: ";
			ret += convertTimeToString(total / card32(m.size()) ) + "\n";
		}
	}

	// average timers
	for( stdext::hash_map<std::string, TAverageTimer >::iterator iter = m_AverageTimer.begin();
		iter != m_AverageTimer.end(); ++iter)
	{
		ret += "AverageTimer: ";
		ret += iter->first + "\n";
		
		ret += "\tNum Measurements: ";
		ret += intToStr(iter->second.numMeasurements);
		ret += "\n\tAverage: ";
		ret += convertTimeToStringFloat(iter->second.total / iter->second.numMeasurements);
		ret += "\n\tTotal: ";
		ret += convertTimeToString(iter->second.total);
		ret += "\n\tMin: ";
		ret += convertTimeToString(iter->second.minEntry);
		ret += "\n\tMax: ";
		ret += convertTimeToString(iter->second.maxEntry) + "\n";
	}

	// counters
	for( stdext::hash_map<std::string, card64>::iterator iter = m_Counter.begin();
		iter != m_Counter.end(); ++iter )
	{
		ret += "Counter: ";
		ret += iter->first + " = " + intToStr(iter->second) + "\n";
	}

	return ret;
}

void X4::StatisticEntry::write( OutputObjectStream *s ) const
{
	card64 numSimpleTimers = m_SimpleTimer.size();
	s->write<card64>(numSimpleTimers);
	for( stdext::hash_map<std::string, std::vector<unsigned> >::const_iterator iter = m_SimpleTimer.begin();
		iter != m_SimpleTimer.end(); ++iter )
	{
		s->writeString( iter->first );
		card64 numEntries = iter->second.size();
		s->write<card64>( numEntries );
		for( unsigned i=0;i<numEntries;i++ )
		{
			card64 entry = iter->second[i];
			s->write<card64>(entry);
		}
	}

	card64 numAverageTimers = m_AverageTimer.size();
	s->write<card64>(numAverageTimers);
	for( stdext::hash_map<std::string, TAverageTimer >::const_iterator iter = m_AverageTimer.begin();
		iter != m_AverageTimer.end(); ++iter)
	{
		s->writeString(iter->first);
		s->write<TAverageTimer>(iter->second);
	}

	card64 numCounters = m_Counter.size();
	s->write<card64>(numCounters);
	for( stdext::hash_map<std::string, card64 >::const_iterator iter = m_Counter.begin();
		iter != m_Counter.end(); ++iter)
	{
		s->writeString(iter->first);
		s->write<card64>(iter->second);
	}

}

void X4::StatisticEntry::read( InputObjectStream *s )
{
	card64 numSimpleTimers;
	s->read<card64>(numSimpleTimers);
	for( unsigned j=0;j<numSimpleTimers;j++ )
	{
		std::string tID;
		s->readString(tID);
		std::vector<unsigned> entries;
		card64 numEntries;
		s->read<card64>(numEntries);
		for( unsigned i=0;i<numEntries;i++ )
		{
			card64 e;
			s->read<card64>(e);
			entries.push_back(e);
		}
		m_SimpleTimer[tID] = entries;
	}

	card64 numAverageTimers;
	s->read<card64>(numAverageTimers);
	for( unsigned j=0;j<numAverageTimers;j++ )
	{
		std::string tID;
		s->readString(tID);
		TAverageTimer at;
		s->read<TAverageTimer>(at);
		m_AverageTimer[tID] = at;
	}

	card64 numCounters;
	s->read<card64>(numCounters);
	for( unsigned j=0;j<numCounters;j++ )
	{
		std::string cID;
		s->readString(cID);
		card64 t;
		s->read<card64>(t);
		m_Counter[cID] = t;
	}

}

std::string X4::Statistics::outputAll()
{
	std::string ret;
	for( unsigned i=0;i<m_Stats.size();i++ )
	{
		ret += m_Stats[i]->report();
	}
	return ret;
}
void StatisticEntry::clearEntries() 
{
	m_SimpleTimer.clear();
	m_AverageTimer.clear();
	m_Counter.clear();
	m_TempTimer.clear();
}

void Statistics::clearCurrentMeasurements() 
{
	m_Stats[m_CurrentStats]->clearEntries();
}


std::string Statistics::outputCurrentCompactSummary() 
{
	return m_Stats[m_CurrentStats]->reportCompact();
}


std::string StatisticEntry::reportCompact() 
{
	std::string ret;
	ret  = "=============================\n";
	ret += "id: ";
	if( m_Name == "")
		ret += "Global Statistics";
	else
		ret += m_Name;
	ret += "\n=============================\n";

	// simple timers
	for( stdext::hash_map<std::string, std::vector<unsigned> >::iterator iter = m_SimpleTimer.begin();
		iter != m_SimpleTimer.end(); ++iter )
	{
		ret += "Timer[";
		ret += iter->first + "] ";
		const std::vector<unsigned> m = iter->second;
		if( m.size() == 0)
		{
			ret += "no measurements\n";
		}
		else
		if( m.size() == 1)
		{
			ret += convertTimeToString(m[0]) + "\n";
		}
		else
		{
			unsigned total = 0;
			for( unsigned i=0;i<m.size();i++ )
				total += m[i];
			ret += "(AV)" +convertTimeToStringFloat(total / (float32)(m.size()) ) + "\n";
		}
	}

	// average timers
	for( stdext::hash_map<std::string, TAverageTimer >::iterator iter = m_AverageTimer.begin();
		iter != m_AverageTimer.end(); ++iter)
	{
		ret += "ATimer[";
		ret += iter->first + "] ";
		
		ret += "(AV) ";
		ret += convertTimeToStringFloat(iter->second.total / iter->second.numMeasurements);
		ret += "(T) ";
		ret += convertTimeToString(iter->second.total);
		ret += "\n";
	}

	// counters
	for( stdext::hash_map<std::string, card64>::iterator iter = m_Counter.begin();
		iter != m_Counter.end(); ++iter )
	{
		ret += "Counter[";
		ret += iter->first + "] " + intToStr(iter->second) + "\n";
	}

	return ret;
}
