#ifndef PCH_X4MISCFOUNDATION_H
#define PCH_X4MISCFOUNDATION_H


// note: the header file is not located in the subdirectory x4basics because the msvc compiler does not 
//       handle directory prefixes on precompiled header files

/// precompiled header file that includes common headers from the standard C++ library, Windows, and QT Core,Gui
/// precompiled headers can be shared among several projects

////////////////////////////////////////////////
//                                            //
//                  Global                    //
//                                            //
////////////////////////////////////////////////

#include <X4GlobalDefinitions.h>


////////////////////////////////////////////////
//                                            //
//                  Windows                   //
//                                            //
////////////////////////////////////////////////

#if defined(WIN32) || defined(WIN64)

	#ifndef WINVER                          // Specifies that the minimum required platform is Windows Vista.
	#define WINVER 0x0600           // Change this to the appropriate value to target other versions of Windows.
	#endif

	#ifndef _WIN32_WINNT            // Specifies that the minimum required platform is Windows Vista.
	#define _WIN32_WINNT 0x0600     // Change this to the appropriate value to target other versions of Windows.
	#endif

	#ifndef _WIN32_WINDOWS          // Specifies that the minimum required platform is Windows 98.
	#define _WIN32_WINDOWS 0x0410 // Change this to the appropriate value to target Windows Me or later.
	#endif

	#ifndef _WIN32_IE                       // Specifies that the minimum required platform is Internet Explorer 7.0.
	#define _WIN32_IE 0x0700        // Change this to the appropriate value to target other versions of IE.
	#endif

	#define NOMINMAX
	#define WIN32_LEAN_AND_MEAN             // Exclude rarely-used stuff from Windows headers
	// Windows Header Files:
	#include <windows.h>

#endif


////////////////////////////////////////////////
//                                            //
//                 C++ / STD                  //
//                                            //
////////////////////////////////////////////////


// C RunTime Header Files
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <tchar.h>

// standard template library
#include <string>
#include <vector>

////////////////////////////////////////////////
//                                            //
//                    QT                      //
//                                            //
////////////////////////////////////////////////

#include <QtCore/QtCore>
#include <QtGui/QtGui>


////////////////////////////////////////////////
//                                            //
//                    X4                      //
//                                            //
////////////////////////////////////////////////

#include "DynamicLinearAlgebra.inline.h"

#include "Camera.h"
#include "CameraController.h"
#include "CameraPathRecorder.h"
#include "CameraPathRecorderEditorWidget.h"
#include "CmdObjMemberMethod.h"
#include "CommandHistory.h"
#include "CommandHistoryEditorWidget.h"
#include "CommandObject.h"
#include "CommandObjMacro.h"
#include "CommandObjMacroEditorWidget.h"
#include "CubeMap.h"
#include "CubeMap.inline.h"
#include "DVectorObject.h"
#include "ExaminerCameraController.h"
#include "FileBlockAllocator.h"
#include "FileBlockChainAllocator.h"
#include "FreeCameraController.h"
#include "GloballyUniqueID.h"
#include "GlobalMacros.h"
#include "Histogram.h"
#include "ImportTools.h"
#include "LargeBinaryFile.h"
#include "ManagedMemory.h"
#include "MarchingCubes.h"
#include "MemoryManagedAttachment.h"
#include "ObjectAttachment.h"
#include "RGBACubeMapEditorWidget.h"
#include "RGBAImage.h"
#include "RGBAImageEditorWidget.h"
#include "Statistics.h"
#include "TableOutputTools.h"
#include "TemplatedImage.h"
#include "TemplatedImage.inline.h"
#include "TimeDependentController.h"
#include "x4miscFoundation.h"


#endif
