#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "TableOutputTools.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
#include <qtCore/qstring>
//---------------------------------------------------------------------------
using namespace X4;

TableOutputTools * X4::tableOutputTool = new TableOutputTools;

TableOutputTools::TableOutputTools(void)
{
}

TableOutputTools::~TableOutputTools(void)
{
}

void TableOutputTools::clear() 
{
	m_Tables.clear();
}

void TableOutputTools::setData( std::string tableName, int row, int column, float data ) 
{
	setupTable( tableName, row+1,column+1);
	std::map<std::string,TTable>::iterator iter = m_Tables.find( tableName );
	iter->second.table[column*iter->second.numRows+row] = data;
}

void TableOutputTools::addData( std::string tableName, int row, int column, float data ) 
{
	setupTable( tableName, row+1,column+1);
	std::map<std::string,TTable>::iterator iter = m_Tables.find( tableName );
	iter->second.table[column*iter->second.numRows+row] += data;
}

void TableOutputTools::setRowName( std::string tableName, int row, std::string rowName ) 
{
	setupTable( tableName, row+1, 1 );
	std::map<std::string,TTable>::iterator iter = m_Tables.find( tableName );
	iter->second.rowNames[row] = rowName;	
}

void TableOutputTools::setColumnName( std::string tableName, int column, std::string columnName ) 
{
	setupTable( tableName, 1, column+1 );
	std::map<std::string,TTable>::iterator iter = m_Tables.find( tableName );
	iter->second.columnNames[column] = columnName;	
}



std::string TableOutputTools::outputSimpleTable( std::string tableName, DataEntryFormat format ) 
{
	std::map<std::string,TTable>::iterator iter = m_Tables.find( tableName );
	if( iter == m_Tables.end() )
		return std::string("Table ") + tableName + " not found!\n";

	std::string ret = std::string("Table: ") + tableName + "\nColumns:";
	for( int c=0;c<iter->second.numColumns;c++ )
	{
		ret += std::string("\t") + iter->second.columnNames[c];
	}
	ret += "\n";

	for( int r=0;r<iter->second.numRows;r++ )
	{
		ret += iter->second.rowNames[r];
		for( int c=0;c<iter->second.numColumns;c++ )
		{
			float n = iter->second.table[ c*iter->second.numRows+r];
			ret += "\t";
			ret += convertNumber( n, format );
		}
		ret += "\n";
	}
	return ret;
}


std::string TableOutputTools::outputLatexTable( std::string tableName, DataEntryFormat format ) 
{
	std::map<std::string,TTable>::iterator iter = m_Tables.find( tableName );
	if( iter == m_Tables.end() )
		return std::string("Table ") + tableName + " not found!\n";

	std::string ret = 
		"\\begin{table}\n"
		"\\begin{tabular}{|l|";
	for(  int c=0;c<iter->second.numColumns;c++ )
	{
		ret += "c|";
	}
	ret += "}\n\\hline\n";
	for(  int c=0;c<iter->second.numColumns;c++ )
	{
		ret += std::string("& ") + iter->second.columnNames[c];
	}
	ret += "\\\\ \\hline \n";
	for( int r=0;r<iter->second.numRows;r++ )
	{
		ret += iter->second.rowNames[r];
		for( int c=0;c<iter->second.numColumns;c++ )
		{
			float n = iter->second.table[ c*iter->second.numRows+r];
			ret += " & ";
			ret += std::string("$") + convertNumber( n, format ) + "$";
		}
		ret += "\\\\ \\hline \n";
	}
	ret += "\\end{tabular}\n";
	ret += std::string("\\caption{") + tableName + "}\n";
	ret += "\\end{table}\n";
	return ret;
}


std::string TableOutputTools::convertNumber(float n, DataEntryFormat format ) 
{
	switch( format )
	{
	case DF_FloatPoint:
		return numToStr(n);
	case DF_FloatComma:
		{
			std::string tmp = QString::number( n ).toLatin1().constBegin();
			for( card32 i=0;i<tmp.length();i++ )
				if( tmp[i] == '.' )
					tmp[i] = ',';

			return tmp;
		}
	case DF_Int:
		return intToStr( (int)n );

	}

	return "#"; // unknown format
}

void TableOutputTools::setupTable( std::string tableName, int numRows, int numColumns ) 
{
	if( m_Tables.find( tableName ) == m_Tables.end() )
	{
		TTable table;
		table.numColumns = numColumns;
		table.numRows = numRows;
		table.table = std::vector<float>( table.numColumns * table.numRows, 0);
		table.rowNames.resize(numRows);
		table.columnNames.resize(numColumns);
		m_Tables[ tableName ] = table;
	}

	std::map<std::string,TTable>::iterator iter = m_Tables.find( tableName );
	if( iter->second.numColumns < numColumns || iter->second.numRows < numRows )
	{
		TTable table;
		table.numColumns = max(numColumns,iter->second.numColumns);
		table.numRows = max(numRows,iter->second.numRows);
		table.table = std::vector<float>( table.numColumns * table.numRows, 0);
		table.rowNames = iter->second.rowNames;
		table.columnNames = iter->second.columnNames;
		table.rowNames.resize(table.numRows);
		table.columnNames.resize(table.numColumns);
		for( int c=0;c<iter->second.numColumns;c++ )
		{
			for( int r=0;r<iter->second.numRows;r++ )
			{
				table.table[ c*table.numRows+r] = iter->second.table[c*iter->second.numRows+r];
			}
		}
		iter->second = table;
	}
}
