#ifndef TableOutputToolsH
#define TableOutputToolsH
//---------------------------------------------------------------------------
#include "x4miscFoundation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	this is a helper class for table data. holds multiple tables accessed by a unique name
	and offers std::string output for different table formats (latex, simple table for exel copy/paste).
*/
class X4MISCFOUNDATION_API TableOutputTools 
{

public:
	TableOutputTools(void);
	~TableOutputTools(void);

	/// erase every table
	void clear();

	/// set table entry (you don't have to care about table dimensions)
	void setData( std::string tableName, int row, int column, float data );

	/// adds to existing table entry (which is initially zero)
	void addData( std::string tableName, int row, int column, float data );

	void setRowName( std::string tableName, int row, std::string rowName );
	void setColumnName( std::string tableName, int column, std::string columnName );


	enum DataEntryFormat
	{
		DF_FloatPoint, // 1.5
		DF_FloatComma,	// 1,5
		DF_Int			// 1
	};

	/// generates an output std::string with simple table format
	/// which looks like this:
	/// Tabelle: tableName
	/// 1 0 1 2 1 0
	/// 0 1 0 0 1 1
	/// 6 8 2 7 2 1 
	/// ...
	std::string outputSimpleTable( std::string tableName, DataEntryFormat dataEntryFormat );

	/// generates an output std::string for latex tables
	std::string outputLatexTable( std::string tableName, DataEntryFormat dataEntryFormat );

protected:

	void setupTable( std::string tableName, int numRows, int numColumns );
	std::string convertNumber(float n, DataEntryFormat format );

	struct TTable
	{
		std::vector<float> table;
		int numRows, numColumns;
		std::vector<std::string> rowNames, columnNames;
	};

	std::map<std::string,TTable> m_Tables;

};

extern X4MISCFOUNDATION_API TableOutputTools * tableOutputTool;

}

#endif