//---------------------------------------------------------------------------
#ifndef ComparableSharedPtrH
#define ComparableSharedPtrH
//---------------------------------------------------------------------------
#include "x4tools.h"
//---------------------------------------------------------------------------
#include <boost/shared_ptr.hpp>
//---------------------------------------------------------------------------


namespace X4 {

	/// Allows the usage of boost::shared_ptr in data structures that require operator< (e.g. an std::map)
	template<typename T> class ComparableSharedPtr {
	public:
		typedef boost::shared_ptr<T> SharedPtrType;

		ComparableSharedPtr(boost::shared_ptr<T> ptr) : ptr(ptr) {}
		bool operator< (const ComparableSharedPtr<T> &o) const { return *ptr < *o.ptr; }

		operator boost::shared_ptr<T>() { return ptr; }
		operator const boost::shared_ptr<T>() const { return ptr; }

		boost::shared_ptr<T> operator -> () { return ptr; }
		const boost::shared_ptr<T> operator -> () const { return ptr; }
		
		T& operator*() { return *ptr; }
		const T&  operator*() const { return *ptr; }

		T* get() { return ptr.get(); }
		const T* get() const { return ptr.get(); }

	private:
		boost::shared_ptr<T> ptr;
	};

} /* namespace X4 */

#endif /* ComparableSharedPtrH */
