//---------------------------------------------------------------------------
#ifndef __GUI_ASSERT_H__
#define __GUI_ASSERT_H__
//---------------------------------------------------------------------------
#include "x4tools.h"
//---------------------------------------------------------------------------
#include "X4Types.h"
//---------------------------------------------------------------------------
#include <boost/thread.hpp>
#include <string>
#include <QMessageBox>
#include <stdarg.h>
//---------------------------------------------------------------------------


namespace X4
{
	
	//---------------------------------------------------------------------------
	/// assert message and return flag regarding user decision.
	/// callFlag set if repeating assertions shall be ignored
	inline int HandleAsserting(char *testStr, char *fileStr, 
							   int line, int *callFlag, char * errorMsg = NULL )
	{
		static boost::mutex g_assertion_mutex;
		boost::unique_lock<boost::mutex> lock(g_assertion_mutex);

		static char s_text[512];
		if (errorMsg)
			sprintf(s_text,"FAILED: %s\n"
					"Error: ( %s )\n"
					"File '%s', Line %d\n"
					"Abort execution, allow assert Retry, or Ignore in future?",
					errorMsg, testStr, fileStr, line);
		else
			sprintf(s_text,"Error: ( %s )\n"
					"File '%s', Line %d\n"
					"Abort execution, allow assert Retry, or Ignore in future?",
					testStr, fileStr, line);
		//switch ( ::MessageBox(NULL, s_text, "ASSERTION ERROR", MB_ABORTRETRYIGNORE) )
		switch (QMessageBox::warning(NULL, "Assertion failed", s_text,
				QMessageBox::Ignore | QMessageBox::Abort | QMessageBox::Retry))
		{
		case QMessageBox::Ignore: // prevent calling again - turn off flag
			*callFlag=0; // never call again
			break;
		case QMessageBox::Abort:  // return flag and break
			return 1;    // abort/break
		}
		return 0;
	}

//#undef DEBUG_INCLUDE_ASSERTIONS
#ifdef DEBUG_INCLUDE_ASSERTIONS
	#define NUM_ASSERTION_ARGS(...) (sizeof((char *[]){__VA_ARGS__})/sizeof(char *))
	//#define NUM_ASSERTION_ARGS(arr) ((sizeof (arr))/(sizeof (arr)[0]))
		
	#define ASSERTING(test, ...) { \
		static int __callIt=1;\
		if (!(test)) { \
			if (__callIt) { \
				/* The ## operator on __VA_ARGS__ swallows the preceding Comma in case __VA_ARGS__ is emtpy */\
				if (HandleAsserting( #test, __FILE__, __LINE__, &__callIt, ##__VA_ARGS__)) { \
					/* TODO: end processes properly */ \
					exit(EXIT_FAILURE); /*__asm int 3;*/ \
				} \
			} \
		} \
	}
#else
	#define ASSERTING(test, ...)
#endif

}//end of namespace X4


#endif
