//---------------------------------------------------------------------------
#ifndef MouseButtonsH
#define MouseButtonsH
//---------------------------------------------------------------------------
#include "x4tools.h"
//---------------------------------------------------------------------------
#include "X4Types.h"
#include <QtGui/QMouseEvent>
//---------------------------------------------------------------------------

namespace X4 {


class MouseButtons {
 private:
   static const card8 leftMB = 1;
   static const card8 rightMB = 2;
   static const card8 leftMBdouble = 4;
   static const card8 middleMB = 8;
   card8 buttonstate;
 public:
   inline MouseButtons() {buttonstate =0;}
   inline MouseButtons(bool left, bool right, bool leftDouble = false, bool middle = false) {
      buttonstate = 0;
      if (left)  buttonstate |=  leftMB;
      if (middle) buttonstate |= middleMB;
      if (right) buttonstate |= rightMB;
      if (leftDouble) buttonstate |= leftMBdouble;
   }
   inline MouseButtons(const Qt::MouseButtons &qtButtons) {
	  buttonstate = 0;
      if (qtButtons & Qt::LeftButton)  buttonstate |=  leftMB;
      if (qtButtons & Qt::MidButton) buttonstate |= middleMB;
      if (qtButtons & Qt::RightButton) buttonstate |= rightMB;
      // TODO (daniel): How to extract leftMBdouble?
   }

   inline bool getLeft() const {
      return ((buttonstate & leftMB) != 0);
   }
   inline bool getMiddle() const {
      return ((buttonstate & middleMB) != 0);
   }
   inline bool getRight() const {
      return ((buttonstate & rightMB) != 0);
   }
   inline bool getLeftDouble() const {
      return ((buttonstate & leftMBdouble) != 0);
   }
   inline bool isAnyPressed() const {
	   return buttonstate != 0;
   }
};

}

#endif