//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
#include "X4Exception.h"
#include <sstream>
#include <iostream>
#include <QtCore/QString>
#include <iomanip>
//---------------------------------------------------------------------------


bool X4::strToNum(std::string s, long double& num)
{
  std::istringstream is(s);
  is >> num;
  if (!is.eof()) return false;
  bool result = !(is.fail());
  return result;
}

bool X4::strToInt(std::string s, int& num){
  std::istringstream is(s);
  is >> num;
  if (!is.eof()) return false;
  bool result = !(is.fail());
  return result;
}

bool X4::strToInt(std::string s, card32& num){
	std::istringstream is(s);
	is >> num;
	if (!is.eof()) return false;
	bool result = !(is.fail());
	return result;
}


bool X4::strToInt(std::string s, card64& num){
	std::istringstream is(s);
	is >> num;
	if (!is.eof()) return false;
	bool result = !(is.fail());
	return result;
}

bool X4::strToFloat(std::string s, float& num){
  std::istringstream is(s);
  is >> num;
  if (!is.eof()) return false;
  bool result = !(is.fail());
  return result;
}

std::string X4::numToStr(long double num)
{
  std::string s;
  std::ostringstream os;
  os << num;
  s = os.str();
  return s;
}

std::string X4::floatToStrPre( float f, X4::card32 precision )
{
	return QString::number(f, 'f', precision).toStdString();
}
std::string X4::lTrimStr(std::string s)
{
	if( s.empty()) // added back by CL @ 2014/02/16
		return s;
  while (s[0] == ' ') s.erase(0, 1);
  return s;
}

std::string X4::rTrimStr(std::string s)
{
  if (s.size() > 0) {
    while (s[s.size()-1] == ' ') {
      s.erase(s.size()-1, 1);
      if (s.size() == 0) break;
    }
  }
  return s;
}

std::string X4::trimStr(std::string s)
{
  s = lTrimStr(rTrimStr(s));
  return s;
}


std::string X4::removeDblSpace(std::string s)
{
  unsigned pos = 0;
  bool lastSpace = false;
  while (pos < s.size()) {
    if (s[pos] == ' ') {
      if (lastSpace) {
        s.erase(pos, 1);
      } else {
        lastSpace = true;
        ++pos;
      }
    } else {
      lastSpace = false;
      ++pos;
    }
  }

  return s;
}


/// splits s into substring
std::vector<std::string> X4::splitString(const std::string &s, const std::string &splitchars)
{
	std::vector<std::string>	ret;
	std::string tmp;
	for( unsigned int i=0;i<s.length();i++ )
	{
		bool splitter = false;
		for( unsigned int j=0;j<splitchars.length();j++ )
			if( s[i] == splitchars[j] )
			{
				splitter = true;
				break;
			}
		if( splitter )
		{
			if( tmp.length() )
			{
				ret.push_back( tmp );
				tmp = "";
			}
		}
		else
			tmp += s[i];
	}

	if( tmp.length() )
		ret.push_back( tmp );

	return ret;
}


std::string X4::cutString( const std::string str, card32 maxLength )
{
   if( str.length() <= maxLength )
      return str;

   std::string ret = str;
   ret.resize( maxLength );
   return ret;
}

std::string X4::replaceSubStrings( const std::string str, const std::string substring, const std::string rStr )
{
   std::string ret = str;
   if( substring == rStr )
      return ret;
   int index = (int)ret.find( substring );
   while( index >= 0 )
   {
      ret = ret.replace( index, substring.length(), rStr );
      index = (int)ret.find( substring );
   }

   return ret;
}

std::string X4::getTimeString( card32 time, bool milliseconds )
{
   card32 ms = time % 1000;
   card32 seconds = (time / 1000) % 60;
   card32 minutes = (time / 60 / 1000) % 60;
   card32 hours = (time / 60 / 60 / 1000) % 24;
   card32 days = (time / 60 / 60 / 1000 / 24 ) % 365;
   card32 years = (time / 60 / 60 / 1000 / 24 / 365 );

   std::string ret;
   bool b = false;
   if( years > 0 )
   {
      ret += intToStr(years) + " years ";
      b = true;
   }
   if( days > 0 || b)
   {
      ret += intToStr(days) + " days";
      b = true;
   }
   if( hours > 0 || b )
   {
      ret += intToStr(hours) + ":";
      b = true;
   }
   if( minutes > 0 || b )
   {
      std::string tmpMin = intToStr(minutes);
      if( tmpMin.length() < 2 )
         tmpMin = std::string("0") + tmpMin;
      ret += tmpMin + ":";
      b = true;
   }
   std::string tmpSec = intToStr(seconds);
   if( tmpSec.length() < 2 )
      tmpSec = std::string("0") + tmpSec;
   ret += tmpSec;
   if( milliseconds )
   {
      ret += ".";
      ret += intToStr(ms);
   }
   return ret;
}


/// returns std::string with the file name ending or "." if no ending exists, e.g. input:"dir/myFile.txt" output:".txt"
std::string X4::extractFileExt( const std::string &s ) {
	int index = (int)s.find_last_of(".");
	if (index>0) {
		return s.substr(index,(int)s.size()-index);
	}
	return ".";
}


std::string X4::allDirSeperatorsToDos(const std::string filename) {
	std::string result = filename;
	for (unsigned i=0; i<result.size(); i++) {
		if (result[i] == '/') {
			result[i] = '\\';
		}
	}
	return result;
}

std::string X4::allDirSeperatorsToUnix(const std::string filename) {
	std::string result = filename;
	for (unsigned i=0; i<result.size(); i++) {
		if (result[i] == '\\') {
			result[i] = '/';
		}
	}
	return result;
}

std::string  X4::fillNumberStringWithZeros(const std::string number, unsigned numZeros) {
	std::string result = number;
	while (result.size() < numZeros) {
		result = std::string("0") + result;
	}
	return result;
}

std::string X4::convertTimeToString( card32 delta_time )
{
	std::string ret;
	card32 h = delta_time / 1000 / 60 / 60;
	card32 m = (delta_time / 1000 / 60) % 60;
	card32 s = (delta_time / 1000 ) % 60;
	card32 ms = delta_time % 1000;
	if( h > 0 )
		ret += intToStr(h) + "h ";
	if( m > 0 || h > 0)
		ret += intToStr(m) + "m ";
	if( s > 0 || h > 0 || m > 0 )
		ret += intToStr(s) + "s ";
	ret += intToStr(ms) + "ms";
	return ret;
}

std::string X4::convertWStringToString(const std::wstring& str)
{
#ifdef _NATIVE_WCHAR_T_DEFINED
	std::string result;
	const size_t n = str.size();
	result.resize(n);
	for (unsigned i=0; i<n; i++) {
		if (str[i] < 256) {
			result[i] = str[i];
		} else {
			throw X4Exception("convertWStringToString() - out of ascii character range.");
		}
	}
	return result;

	//QString qstr = QString::fromStdWString ( str );
#else
	QString qstr = QString::fromUtf16 ( (wchar_t*)str ); // TODO: check if really UTF16, it might be something else, however wchar_t is of ushort type
	return qstr.toStdString();
#endif
}

std::wstring X4::convertStringToWString(const std::string& str)
{
	std::wstring temp(str.length(),L' ');
	std::copy(str.begin(), str.end(), temp.begin());
	return temp;
	//QString qstr = QString((const QChar*)str.c_str(), str.length());//QString qstr = QString::fromStdString ( str );
	//return qstr.toStdWString();
}

std::string X4::convertTimeToStringFloat( float delta_time_f )
{
   card32 delta_time = delta_time_f;
   std::string ret;
   card32 h = delta_time / 1000 / 60 / 60;
   card32 m = (delta_time / 1000 / 60) % 60;
   card32 s = (delta_time / 1000 ) % 60;
   card32 ms = delta_time % 1000;
   float fms = (float)ms + (delta_time_f - (float)delta_time);
   if( h > 0 )
      ret += intToStr(h) + "h ";
   if( m > 0 || h > 0)
      ret += intToStr(m) + "m ";
   if( s > 0 || h > 0 || m > 0 )
      ret += intToStr(s) + "s ";
   ret += floatToStr(fms);
   ret += "ms";
   return ret;
}
