//---------------------------------------------------------------------------
#ifndef StringHelperH
#define StringHelperH
//---------------------------------------------------------------------------
#include "x4tools.h"
//---------------------------------------------------------------------------
#include <X4Types.h>
#include <BasicStringUtils.h>
//---------------------------------------------------------------------------
#include <string>
#include <vector>
//---------------------------------------------------------------------------

class QString;

namespace X4 {

	/// Already defined in BasicStringUtils.h/.cpp
	//	/// Converts int to a std::string
	//	std::string X4MODULES_API intToStr(int64 i, int64 minWidth = 0);
	//	/// Converts unsigned 32 bit int to a std::string
	//	std::string X4MODULES_API mpcardToStr(mpcard i);
	//	/// Converts float to a std::string
	//	std::string X4MODULES_API floatToStr(float32 f);


	//	/// Converts the std::string into lower case and returns the result
	//	std::string X4MODULES_API strLCase(std::string s);
	//	/// Converts the std::string into upper case and returns the result
	//	std::string X4MODULES_API strUCase(std::string s);
	//	/// Extracts the characters up to a certain character (see detailed descr.)
	//	std::string X4MODULES_API extractToChar(std::string& s, char c);


	//	/// Example: "dir/test.abc" -> "test.abc"
	//	std::string X4MODULES_API removePathFromFilename( const std::string &filename );
	//	/// Example: "dir/test.abc" -> "dir/"
	//	std::string X4MODULES_API removeFilenameFromPath( const std::string &filename );


	//	/// Converts QString to an STL std::string (std::string)
	//	std::string X4MODULES_API qString2STLString(const QString &str);



	/// Convert the std::string given to a number. Returns true if successful.
	bool X4TOOLS_API strToNum(std::string s, long double& num);
	/// Converts the number given to a std::string.
	std::string X4TOOLS_API numToStr(long double num);
	/// Convert the std::string given to a int. Returns true if successful.
	bool X4TOOLS_API strToInt(std::string s, int& num);
	bool X4TOOLS_API strToInt(std::string s, card32& num);
	bool X4TOOLS_API strToInt(std::string s, card64& num);
	/// Convert the std::string given to a float. Returns true if successful.
	bool X4TOOLS_API strToFloat(std::string s, float& num);

	/// Converts float to a std::string with given precision
	std::string X4TOOLS_API floatToStrPre( float f, card32 precision );
	/// Trims the std::string (deletes spaces at the beginning and the end)
	std::string X4TOOLS_API trimStr(std::string s);
	/// Deletes the leading spaces in the std::string and returns the result
	std::string X4TOOLS_API lTrimStr(std::string s);
	/// Deletes the trailing spaces in the std::string and returns the result
	std::string X4TOOLS_API rTrimStr(std::string s);
	/// Removes all double spaces from the std::string (i.e. makes them single spaces)
	std::string X4TOOLS_API removeDblSpace(std::string s);

	/// Splits s into substrings, splitted by splitchars
	/// Example: splitString( "hello world!\n bye", " \n" ) ----> "hello", "world!", "bye"
	std::vector<std::string> X4TOOLS_API splitString( const std::string &s, const std::string &splitchars);


	/// Cuts std::string ( by removing all characters behind maxLength), if length > maxLength 
	std::string X4TOOLS_API cutString( const std::string str, card32 maxLength );

	/// Replaces every substd::string in str with rStr
	std::string X4TOOLS_API replaceSubStrings( const std::string str, const std::string substring, const std::string rStr );

	/// returns a std::string with format [y years][d days ][h:][m:]s.ms calculated from time, time in ms
	std::string X4TOOLS_API getTimeString( card32 time, bool milliseconds=false );

	/// returns std::string with the file name ending or "." if no ending exists, e.g. input:"dir/myFile.txt" output:".txt"
	std::string X4TOOLS_API extractFileExt( const std::string &s );

	/// converts all '/' characters to '\'
	std::string X4TOOLS_API allDirSeperatorsToDos(const std::string filename);

	/// converts all '\' characters to '/'
	std::string X4TOOLS_API allDirSeperatorsToUnix(const std::string filename);

	/// pads std::string with zeros in front if length is smaller than numZeroes
	std::string X4TOOLS_API fillNumberStringWithZeros(const std::string number, unsigned numZeros);

	/// converts time (in ms) into "[h hours] [m minutes] [s seconds] ms ms"
	std::string X4TOOLS_API convertTimeToString( card32 delta_time );

   /// converts time (in ms) into "[h hours] [m minutes] [s seconds] ms.ffff ms"
   std::string X4TOOLS_API convertTimeToStringFloat( float delta_time );

	/// convert Wstd::string to std::string(goes over Qt's std::string implementation)
	std::string X4TOOLS_API convertWStringToString(const std::wstring& str);
	std::wstring X4TOOLS_API convertStringToWString(const std::string& str);

}

#endif
