//---------------------------------------------------------------------------
#ifndef TimerH
#define TimerH
//---------------------------------------------------------------------------
#include "x4tools.h"
//---------------------------------------------------------------------------
#include "X4Types.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------


namespace X4 {

class X4TOOLS_API Timer {
 private:
   card32 lastTime;

 public:
   Timer();
   card32 getCurrentValue();
   card32 getDeltaValue(card32 lastValue);
   card32 getDeltaValue();
   ~Timer();
};

#define PEAK_TIMER_START( id ) \
	Timer timer_##id ; 

#define PEAK_TIMER_END( id ) \
	static card32 peaktime_##id = 0; card32 tmptime_##id = timer_##id .getDeltaValue(); \
	if( peaktime_##id < tmptime_##id ) \
{ debugOutput << "peak time[ " << #id << "]" << convertTimeToString( tmptime_##id ) << "\n"; peaktime_##id = tmptime_##id; debugOutput.flush(); }


#define X4_TIMER_START( id ) \
	Timer x4_timer_##id ; \
	x4_timer_##id##.getDeltaValue();

#define X4_TIMER_END( id ) \
	debugOutput << "Timings[" << #id << "]: " << convertTimeToString(  x4_timer_##id .getDeltaValue() ) << "\n";

#define X4_TIMER_STOP( id ) X4_TIMER_END( id )

#define X4_AVERAGE_TIMER_DEFINE(id) \
	static unsigned x4_average_timer_variable_##id = 0; \
	static unsigned x4_average_timer_numMeasurements_##id = 0; \
	static unsigned x4_average_timer_begin_##id; \
	static Timer x4_average_timer_##id ; 

#define X4_AVERAGE_TIMER_START(id) \
	x4_average_timer_begin_##id = x4_average_timer_##id##.getCurrentValue();
	
#define X4_AVERAGE_TIMER_STOP(id) \
	x4_average_timer_variable_##id += (x4_average_timer_##id##.getCurrentValue() - x4_average_timer_begin_##id); \
	x4_average_timer_numMeasurements_##id ++;

#define X4_AVERAGE_TIMER_REPORT_AND_RESET(id) \
	if( x4_average_timer_numMeasurements_##id > 0 ) \
	debugOutput << "Timer[ " << #id << "] - Report " << convertTimeToString( x4_average_timer_variable_##id / x4_average_timer_numMeasurements_##id ) << \
	" total time:" << convertTimeToString( x4_average_timer_variable_##id ) << "\n"; \
	else \
	debugOutput << "Timer[ " << #id << "] - No measurements were done!\n"; \
	x4_average_timer_variable_##id = 0; \
	x4_average_timer_numMeasurements_##id = 0; 

#define STATUSREPORT_DEFINE( statusName ) \
	Timer StatusReportTimer_##statusName ; \
	unsigned StatusReportRT_##statusName = StatusReportTimer_##statusName . getCurrentValue();

#define STATUSREPORT_REPORT( statusName, reportInterval, reportMessage ) \
{	unsigned currentTime = StatusReportTimer_##statusName . getCurrentValue();	\
	if( currentTime - StatusReportRT_##statusName > (reportInterval) ) \
{	debugOutput << "StatusReport[" << #statusName << "] " << (reportMessage) << "\n"; \
	debugOutput.flush(); 	StatusReportRT_##statusName = currentTime; \
} \
}

}

#endif
 
