#include <precompiledHeadersBasics.h>
#include "CreateClassRegistration.h"
#include <TagList.h>
#include <BasicStringUtils.h>

#include <iostream>

using namespace X4;


static const std::string header =
"// automatically generated by x4ModuleParser\n"
"// do not edit manually - file will be overwritten in each rebuild\n"
"\n"
"\n";

static const std::string initStart1 =  
"\n"
"\n"
"static void init_";
static const std::string initStart2 =  
"()\n"
"{\n";
static const std::string initEnd = 
"}\n";

static const std::string shutdownStart1 =  
"\n"
"\n"
"static void shutdown_";
static const std::string shutdownStart2 =  
"()\n"
"{\n";
static const std::string shutdownEnd = 
"}\n";



std::string X4::createClassRegistration(TagList *tags, std::string moduleName)
{
	std::vector<std::string> classes = tags->getAllValues("CLASSES");

	std::string includes;
	std::string inits;
	std::string shutdowns;
	for (size_t i=0; i<classes.size(); i++)
	{      
		QString qcls(classes[i].c_str());
		QStringList parts = qcls.trimmed().split(" ");
      
		std::string className;
		std::string baseClassName;
		std::string includeFile;

		if (parts.size() == 5)
		{
			className = qString2STLString(parts[0]);
			
			if (parts[1].toLower() == "in") 
			{
				includeFile = qString2STLString(parts[2]);
			}else
			{
				std::cerr << "Error: class entry " << i << ": middle entry is not \"in\" \n"
						"entry: " << classes[i] << "\n";
				return "";
			}

			if (parts[3].toLower() == "inherits") 
			{
				baseClassName = qString2STLString(parts[4])+std::string("::getClass()");
			}else
			{
				std::cerr << "Error: class entry " << i << ": middle entry is not \"inherits\" \n"
						"entry: " << classes[i] << "\n";
				return "";
			}

		}else if (parts.size() == 3)
		{
			if (parts[1].toLower() == "inherits") 
			{
				className = qString2STLString(parts[0]);
				baseClassName = qString2STLString(parts[2])+std::string("::getClass()");
			} else 
			{
				std::cerr << "Error: class entry " << i << ": middle entry is not \"inherits\" \n"
						"entry: " << classes[i] << "\n";
				return "";
			}
		} else if (parts.size() == 1) 
		{
			className = qString2STLString(parts[0]);
			baseClassName = "NULL";
		} else 
		{
			std::cerr << "Error: class entry " << i << " is invalid\n"
					"entry: " << classes[i] << "\n";
			return "";
		}

		//if (includeFile.length())
		//	includes += std::string("#include \"") + moduleName + std::string("/") + includeFile + std::string("\"\n");
		//else
		//	includes += std::string("#include \"") + moduleName + std::string("/") + className + std::string(".h\"\n");
      
      /// changed to simple include (without project names)
      /// to make porting of code easier
      if (includeFile.length())
         includes += std::string("#include \"") + includeFile + std::string("\"\n");
      else
         includes += std::string("#include \"") + className + std::string(".h\"\n");

		inits += std::string("   ") + className + std::string("::init( ") + baseClassName + std::string(" );\n");
		shutdowns += std::string("   ") +className+std::string("::shutdown();\n");
   
	}

	return   header + includes 
			+ std::string("\n\n")
			+ initStart1+moduleName+initStart2 + inits + initEnd
			+ std::string("\n\n")
			+ shutdownStart1+moduleName+shutdownStart2 + shutdowns + shutdownEnd;
}
