#include <precompiledHeadersBasics.h>

#include "CreateClassRegistration.h"

#include <iostream>
#include <string>
#include <fstream>
#include <TagHeaderParser.h>
#include <TagList.h>
#include <BasicStringUtils.h>

using namespace X4;

#ifdef _MSC_VER
#        pragma comment(linker, "/subsystem:console")
//#        pragma comment(linker, "/entry:mainCRTStartup")
#endif

static void outputUsageInfo()
{
   std::cout << "usage: x4ModuleParser -command <parameters>\n"
            "where -command offers the following options:\n"
            "\n"
            "x4ModuleParser -createTagsInfo <InputFile> <OutputFile>\n"
				"x4ModuleParser -createClassReg <InputFile> <OutputFile>\n"
            "x4ModuleParser -rebuild <InputDirectory> <OutputDirectory>\n"
            "\n";
}

// QT creator template
// #include <QCoreApplication>
//
// int main(int argc, char *argv[])
// {
//    QCoreApplication a(argc, argv);
// 
//    return a.exec();
// }

//

//int WINAPI WinMain(HINSTANCE hInstance,
//							HINSTANCE hPrevInstance,
//							LPSTR lpCmdLine,
//							int nCmdShow) {}
#if defined(X4_PLATFORM_MINGW_GCC)
int main(int argc, char **argv)
#elif defined(X4_PLATFORM_MSVC)
int _tmain(int argc, TCHAR* argv[])
#else
#error("unsupported platform")
#endif
{
	std::vector< std::string > cmdLine;
	for (int i=0; i<argc; i++) {
		#if defined(X4_PLATFORM_MINGW_GCC)
		std::string ws(argv[i]);
		#elif defined(X4_PLATFORM_MSVC)
		std::wstring ws(argv[i]);
		#else
		#error("unsupported platform")
		#endif
		std::string s(ws.begin(), ws.end());
      cmdLine.push_back(s);
//		std::cout << "Command line (" << i << "):" << s << "\n";
	}
//	for (int i=0; i<argc; i++) {
//		std::cout << "DBN: ||" << cmdLine[i] << "||\n\n\n";
//	}
	if (argc <= 1) {
      outputUsageInfo();
      return 0;
   }
   if (cmdLine[1] == "-createTagsInfo") {
      if (argc != 4) {
         outputUsageInfo();
         return 1;
      } else {
         TagHeaderParser parser;
         TagList *tagList = parser.parseTagFile(cmdLine[2]);
         if (tagList == NULL || parser.errorLevel) {
            std::cerr << "Parser error:\n";
            std::cerr << parser.errors;
            delete tagList;
            return 1;            
         }                
         tagList->writeTextToFile(cmdLine[3]);
         delete tagList;
         return 0;         
      }
   } else if (cmdLine[1] == "-createClassReg") {
      if (argc != 4 && argc != 5) {
         outputUsageInfo();
         return 1;
      } else {
         TagHeaderParser parser;
         TagList *tagList = parser.parseTagFile(cmdLine[2]);
         if (tagList == NULL || parser.errorLevel) {
            std::cerr << "Parser error:\n";
            std::cerr << parser.errors;
            delete tagList;
            return 1;            
         }                
         if (tagList->getAllValues("CLASSES").empty()) {
            std::cerr << "warning - no classes found. Proxy file created (with no actual registrations).\n";
         }
			std::string noPath = removePathFromFilename(cmdLine[2]);
			std::string reg = createClassRegistration( tagList, extractToChar(noPath,'.') );
         if (reg == "") {
            std::cerr << "error, registration file could not be created.\n";
            return 1;
         } else {
            std::ofstream out(cmdLine[3].c_str());
            out << reg;
         }
         delete tagList;
         return 0;         
      }
   } else if (cmdLine[1] == "-rebuild") {
      std::cerr << "not implemented yet.\n";
      return 1;      
   } else {
      std::cerr << "unknown command\n";
      outputUsageInfo();
	   return 1;
   }
}

