//---------------------------------------------------------------------------
#include "precompiledHeadersBasics.h"
//---------------------------------------------------------------------------
#include "ApplicationDirectories.h"
//---------------------------------------------------------------------------
#include "BasicStringUtils.h"
#include "ConsoleOutput.h"
#include <X4Exception.h>
//---------------------------------------------------------------------------
#include <QtCore/QCoreApplication>
#include <QtCore/QDir>
//---------------------------------------------------------------------------

using namespace X4;


bool ApplicationDirectories::isSetup = false;

std::string ApplicationDirectories::applicationDir;

std::string ApplicationDirectories::resourcesDir;

std::string ApplicationDirectories::configDir;

std::string ApplicationDirectories::docsDir;

std::string ApplicationDirectories::binDir;

std::string ApplicationDirectories::logDir;


void ApplicationDirectories::setup() 
{
   if (isSetup) {
      throw EInvalidState("ApplicationDirectories::setupFromCurrentDirectory() - called twice.");
   }
   QDir binDirQ = QCoreApplication::applicationDirPath();
   binDir = qString2STLString(binDirQ.absolutePath());
   QDir dir(QCoreApplication::applicationDirPath());
   if (!dir.cdUp()) {
      throw EInvalidState(std::string("ApplicationDirectories::setup() - unexpected directory layout."
                                 "Cannot move up from binary directory. Binary directory was: ") + binDir);
   }
   if (!dir.cdUp()) {
      throw EInvalidState(std::string("ApplicationDirectories::setup() - unexpected directory layout."
                                 "Cannot move up twice from binary directory. Binary directory was: ") + binDir);
   }
   if (!dir.cdUp()) {
      throw EInvalidState(std::string("ApplicationDirectories::setup() - unexpected directory layout."
                                 "Cannot move up 3x from binary directory. Binary directory was: ") + binDir);
   }
   applicationDir = qString2STLString(dir.absolutePath())+std::string("/");

   QDir resDirQ = dir;
   if (!resDirQ.cd("resources")) {
      consoleWarning("ApplicationDirectories::setup() - resources directory not found.");
   }
   resourcesDir = qString2STLString(resDirQ.absolutePath())+std::string("/");
   
   QDir cfgDirQ = dir;
   if (!cfgDirQ.cd("config")) {
      consoleWarning("ApplicationDirectories::setup() - config directory not found.");
   }
   configDir = qString2STLString(cfgDirQ.absolutePath())+std::string("/");

   QDir docsDirQ = dir;
   if (!docsDirQ.cd("docs")) {
      consoleWarning("ApplicationDirectories::setup() - docs directory not found.");
   }
   docsDir = qString2STLString(docsDirQ.absolutePath())+std::string("/");

   QDir logDirQ = dir;
   if (!logDirQ.cd("log")) {
      consoleWarning("ApplicationDirectories::setup() - log directory not found.");
   }
   logDir = qString2STLString(logDirQ.absolutePath())+std::string("/");

   isSetup = true;
}


std::string ApplicationDirectories::getApplicationDir() 
{
   return applicationDir;
}


std::string ApplicationDirectories::getBinDir() 
{
   return binDir;
}


std::string ApplicationDirectories::getResourcesDir() 
{
   return resourcesDir;
}


std::string ApplicationDirectories::getDocsDir() 
{
   return docsDir;
}


std::string ApplicationDirectories::getLogDir() 
{
   return logDir;
}


std::string ApplicationDirectories::getStartupModulesListFilename() 
{
   return std::string("StartupModules.config");
}


std::string ApplicationDirectories::getStartupModulesListFilenameAndPath() 
{
   return configDir+getStartupModulesListFilename();
}


std::string ApplicationDirectories::getConfigDir() 
{
   return configDir;
}
