//---------------------------------------------------------------------------
#include "precompiledHeadersBasics.h"
//---------------------------------------------------------------------------
#include "ConsoleOutput.h"
//---------------------------------------------------------------------------
#include "BasicStringUtils.h"
#include <sstream>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



X4MODULES_API X4Console X4::x4console;
//X4MODULES_API X4Console &X4::debugOutput = x4console;

void X4MODULES_API X4::consoleError(std::string msg)
{
   x4console.outputHTML("<span color=\"#CC3333\"><b>error:</b></span>\n");
   x4console << msg << "\n";
   x4console.show();
}

void X4MODULES_API X4::consoleWarning(std::string msg) {
   x4console.outputHTML("<span color=\"#CCCC33\"><b>warning:</b></span>\n");
   x4console << msg << "\n";
   x4console.show();
}

void X4MODULES_API X4::consoleMessage(std::string msg) {
   x4console.outputHTML("<span color=\"#A0A0A0\"><b>message:</b></span>\n");
   x4console << msg << "\n";
   x4console.show();
}



X4Console::X4Console()
{
	myMutex = new QMutex(QMutex::Recursive);
}

X4Console& X4Console::operator<<(bool b) {
   if (b) {
      (*this) << "true";
   } else {
      (*this) << "false";
   }
   return *this;
}

X4Console& X4Console::operator<<(int i)  {
   (*this) << intToStr(i);
   return *this;
}

X4Console& X4Console::operator<<(unsigned int i)  {
   (*this) << intToStr(i);
   return *this;
}

X4Console& X4Console::operator<<(X4::int64 i)  {
   (*this) << intToStr(i);
   return *this;
}

X4Console& X4Console::operator<<(card64 i)  {
   (*this) << intToStr((int64)i);
   return *this;
}

X4Console& X4Console::operator<<(long double d)  {
   std::string s;
   std::ostringstream os;
   os << d;
   s = os.str();
   (*this) << s;
   return *this;
}


X4Console& X4Console::operator<<(float f)  {
   (*this) << (long double)f;
   return *this;
}

X4Console& X4Console::operator<<(double d)  {
   (*this) << (long double)d;
   return *this;
}

X4Console& X4Console::operator<<(char* s)
{
   std::string tmpS = s;
   (*this) << tmpS;
   return *this;
}



X4Console& X4Console::operator<<(const std::string &msg)
{
   lock();
   for (size_t i=0; i<handlers.size(); i++) {
      handlers[i]->outputText(msg);
   }
	if (handlers.empty()) unhandledMessages += msg;
   unlock();
   return *this;
}

void X4Console::outputHTML(const std::string &text)
{
   lock();
   for (size_t i=0; i<handlers.size(); i++) {
      handlers[i]->outputHTML(text);
   }
	if (handlers.empty()) unhandledMessages += text;
   unlock();
}

void X4Console::outputObject(const void* data, mpcard size)
{
   lock();
   for (size_t i=0; i<handlers.size(); i++) {
      handlers[i]->outputObject(data, size);
   }
   unlock();
}

void X4Console::cr() 
{
   (*this) << "\n";
}

void X4Console::flush()
{
   lock();
   for (size_t i=0; i<handlers.size(); i++) {
      handlers[i]->flush();
   }
   unlock();
}

void X4Console::addHandler(ConsoleHandler *cHandler)
{
   lock();
   handlers.push_back(cHandler);
   unlock();
}

void X4Console::clearHandlers()
{
   for (size_t i=0; i<handlers.size(); i++) {
      delete handlers[i];
   }
   handlers.clear();
}

X4Console::~X4Console()
{
   //for (size_t i=0; i<handlers.size(); i++) {
   //   delete handlers[i];
   //}
   delete myMutex;
}

void X4Console::show()
{
   for (size_t i=0; i<handlers.size(); i++) {
      handlers[i]->showConsole();
   }
}

void X4Console::hide()
{
   for (size_t i=0; i<handlers.size(); i++) {
      handlers[i]->hideConsole();
   }
}


void X4Console::toggleVisible() 
{
	for (size_t i=0; i<handlers.size(); i++) {
		handlers[i]->toggleVisible();
	}
}

void X4Console::moveConsoleWindow(int x, int y) 
{
	for (size_t i=0; i<handlers.size(); i++) {
		handlers[i]->moveConsoleWindow(x,y);
	}
}

void X4Console::resizeConsoleWindow(int w, int h)
{
	for (size_t i=0; i<handlers.size(); i++) {
		handlers[i]->resizeConsoleWindow(w,h);
	}

}

