//---------------------------------------------------------------------------
#ifndef ExceptionHandlingH
#define ExceptionHandlingH
//---------------------------------------------------------------------------
#include "x4modules.h"
#include <X4GlobalDefinitions.h>
#include <X4Exception.h>
//---------------------------------------------------------------------------
#include <exception>
//---------------------------------------------------------------------------


namespace X4 {

	// -- helper functions that display exceptions on the console

	void X4MODULES_API displayPException(X4Exception *e);

	#ifdef X4_PLATFORM_MSVC
	void X4MODULES_API displayOSException(unsigned int code, EXCEPTION_RECORD info);
	#endif

	void X4MODULES_API displayGeneralException();

	void X4MODULES_API displaySTLException(std::exception e);


	// -- structured win32 exception wrappper

	#ifdef X4_PLATFORM_MSVC
	class SE_Exception {
	 public:
	   unsigned int nSE;
		EXCEPTION_RECORD info;
	
	   SE_Exception(unsigned int n, EXCEPTION_RECORD exceptionInfo) : nSE(n), info(exceptionInfo) {}
	   ~SE_Exception() {}
	};

	X4MODULES_API void SE_wrapper_trans_func( unsigned int u, _EXCEPTION_POINTERS* pExp );
	#endif

	/// call this once in main to setup handling of structured exception (i.e. WIN32-Exceptions)
	X4MODULES_API void setStructureExceptionHandler();


	// -- macros for "global" exception handling (i.e. in qt-slots that receive input from qapp directly)


	#define STD_GLOBAL_EXCEPTION_HANDLER_START \
	try {\



	#ifdef TARGET_PLATFORM_MSVC2005FF

	#define STD_GLOBAL_EXCEPTION_HANDLER_END \
	} catch (X4Exception &e) {\
		displayPException(&e);\
	} catch (SE_Exception e) {\
		displayOSException(e.nSE, e.info);\
	} catch (std::exception e) {\
		displaySTLException(e);\
	} catch (...) {\
		displayGeneralException();\
	}

	#else

	#define STD_GLOBAL_EXCEPTION_HANDLER_END \
	} catch (X4Exception &e) {\
		displayPException(&e);\
	} catch (...) {\
		displayGeneralException();\
	}

	#endif

}

#endif
