#include "precompiledHeadersBasics.h"

#include "Module.h"
#include "BasicStringUtils.h"

using namespace X4;


Module::Module(std::string name) 
{
   moduleName = name;
}

void Module::addTag(std::string tagname, std::string value) 
{
   moduleTags.addTag(tagname, value);
}

void Module::loadTagFile(const std::string& filename) 
{
   moduleTags.clear();
   std::string tmp =  removePathFromFilename(filename);
   moduleName = extractToChar(tmp, '.');
   moduleTags.readTextFromFile(filename);
}

void Module::parseModuleDeclaration(const std::string& filename) 
{
   moduleTags.clear();
   std::string tmp =  removePathFromFilename(filename);
   moduleName = extractToChar(tmp, '.');
   moduleTags.parseModuleDeclaration(filename);
}


std::string Module::getIconResStr() const 
{
   return moduleTags.getValue("ICON");
}


std::string Module::createHTMLDescrForMod(const TagList *moduleTags, bool includeTechnicalDetails)
{
   static const std::string header = 
      "<HTML>\n"
      "<HEAD>\n"
      "<TITLE>Module Description</TITLE>\n"
      "</HEAD>\n"
      "<BODY>\n";

   static const std::string footer =
      "</BODY>\n"
      "</HTML>\n";

   std::string tags;
   for (unsigned i=0; i<moduleTags->getNumTags(); i++) {
      if (moduleTags->getTag(i) == "ICON" && !includeTechnicalDetails) continue;
      if (moduleTags->getTag(i) == "DEPENDENCIES" && !includeTechnicalDetails) continue;
      if (moduleTags->getTag(i) == "DEPENDENCIES_BUILD" && !includeTechnicalDetails) continue;
      tags += std::string("<p> <font color=\"#445588\"><b>") + moduleTags->getTag(i) + std::string("</b></font></p>\n");
      tags += "<ul>\n";
      for (unsigned j=0; j<moduleTags->getNumValues(i); j++) {
         tags += std::string("<li> ") + moduleTags->getValue(i,j) + std::string(" </li>\n");
      }
      tags += "</ul>\n";
   }

   return header + tags + footer;
}


std::string Module::createHTMLDescr(bool includeTechnicalDetails) const {
   return createHTMLDescrForMod(&moduleTags, includeTechnicalDetails);
}


std::vector<std::string> Module::getDependencies() const 
{
   return moduleTags.getAllValues("DEPENDENCIES");
}
