#ifndef ModuleH
#define ModuleH

#include "x4modules.h"
#include "TagList.h"

#include <string>

class QObject;


namespace X4 {


/// This class represents a loadable X4 module.
class X4MODULES_API Module {
 private:
   std::string moduleName;
   TagList moduleTags;

 public:
   Module(std::string name);
   
   void addTag(std::string tagname, std::string value);
   TagList &getTags() {return moduleTags;}
   const TagList &getTags() const {return moduleTags;}
   std::string getName() const {return moduleName;}
   
   void loadTagFile(const std::string& filename);
   void parseModuleDeclaration(const std::string& filename);
   
   // helper functions
   std::string getIconResStr() const;
   static std::string createHTMLDescrForMod(const TagList *moduleTags, bool includeTechnicalDetails = false);
   std::string createHTMLDescr(bool includeTechnicalDetails = false) const;
   std::vector<std::string> getDependencies() const;
};


}

#endif
