#ifndef ModuleEventsH
#define ModuleEventsH

#include "x4modules.h"

#include <vector>


namespace X4 {


/// This class provides a set of events that are handled during initialization and shutdown of modules
/// mw-28/07/12
///
class X4MODULES_API ModuleEvents {
 public:
	typedef void (*EventProc)(void);

 private:
	static std::vector<EventProc> postInitEvents;
	static std::vector<EventProc> preShutdownEvents;

 public:
	// -- more events can be added if necessary...

	 /// add an event; prost init events are called after all (default) modules are loaded, just before the main interface loop starts
	 static void addPostInitEvent(EventProc ev);
	 /// call this just before going into the main loop of your application
	 static void callPostInitEvents();


	/// add an event; pre shutdown events are called before the individual modules are unloaded (to clean up things before destroying the class registration)
	static void addPreShutdownEvent(EventProc ev);
	/// call this method when the main application ends (before unregistering any individual modules)
	static void callPreShutdownEvents();
};


}

#endif