#include "precompiledHeadersBasics.h"
#include "ModuleList.h"

#include "Module.h"
#include "BasicStringUtils.h"
#include <algorithm>

using namespace X4;

X4MODULES_API ModuleList X4::x4modules;

void ModuleList::addModule(Module *mod) 
{
   modules.push_back(mod);
}

unsigned ModuleList::getNumModules() const 
{
   return (unsigned)modules.size();
}

int ModuleList::findModule(std::string name) const
{
   for (size_t i=0; i<modules.size(); i++) {
      std::string a = modules[i]->getName().c_str();
      std::string b = name.c_str();
		std::transform(a.begin(), a.end(), a.begin(), ::tolower);
		std::transform(b.begin(), b.end(), b.begin(), ::tolower);
      if (a == b) {
         return (int)i;
      }
   }
   return -1;
}


Module *ModuleList::getModule(unsigned i)
{
   return modules[i];
}

Module *ModuleList::getModule(std::string name)
{
   int i=findModule(name);
   if (i==-1) {
      return NULL;
   } else {
      return modules[i];
   }
}

const Module *ModuleList::getModule(unsigned i) const
{
   return modules[i];
}

const Module *ModuleList::getModule(std::string name) const
{
   int i=findModule(name);
   if (i==-1) {
      return NULL;
   } else {
      return modules[i];
   }
}

ModuleList::~ModuleList() {
   for (size_t i=0; i<modules.size(); i++) {
      delete modules[i];
   }
}

bool ModuleList::tryLoadModule(std::string filename, std::string &error) 
{
   #if defined(WIN32) || defined(WIN64)
   std::wstring mod;
   mod.assign(filename.begin(), filename.end());
   HMODULE dllHandle = LoadLibrary(mod.c_str());
   if (dllHandle == NULL)
   {
       DWORD lastError = GetLastError();

       error = std::string("error loading module \"")+filename+std::string("\": error code ")+intToStr(lastError)+std::string("; ");
       LPTSTR lpBuffer = 0;
       if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, 0, lastError, 0, lpBuffer, 0, 0)) {
         std::wstring ws(lpBuffer);
         std::string a(ws.begin(), ws.end());
         error += a;
			if (lastError == 126) error += " (error 126: dependent DLL could not be loaded)";
			if (lastError == 126) error += " (error 3765269347 == 0xE06D7363: DLL crashed during intialization due to unhandled exception. Check you class registration.)";
       }
       return false;
   } else {
      error = "";
      return true;
   }
   #else
   throw X4Exception("not implemented for this platform");
   #endif
}

bool ModuleList::tryLoadModules(const std::vector<std::string> &filenames, std::vector<std::string> &errors) 
{
   bool success = true;
   errors.clear();

   for (size_t i=0; i<filenames.size(); i++) {
      std::string err;
      bool modOk = tryLoadModule(filenames[i], err);
      if (!modOk) {
         success = false;
         errors.push_back(err);
      } else {
         errors.push_back("");
      }
   }
   return success;
}
