#include "precompiledHeadersBasics.h"
#include <X4Exception.h>
#include <X4Assert.h>
#include "TagHeaderParser.h"
#include "TagList.h"
#include "BasicStringUtils.h"

#include <QtCore/qfile.h>
#include <QtCore/qtextstream.h>

using namespace X4;

// finds characters before first "(", excludes comments
// returns "" if nothing is found
// "eats" the characters parsed
QString TagHeaderParser::findKeyword(QString &s)
{                                     
   int l = s.size();
   QString result;
   int i=0;
   for (i=0; i<l; i++) {
      if (s[i] != ' ' && s[i] != '\t') {
         // comment detected
         if (s[i] == '/' || s[i] == '(') break;
      }
      result += s[i];
   }
   s = s.right(l-i-1);
   return result;
}

// finds strings enclosed in ""
// overreads commas (for multiple arguments) and whitespaces
// "eats" the characters parsed

std::string TagHeaderParser::getString(QString &s)
{                                     
   int l = s.size();
   std::string test = qString2STLString(s);
   QString result;
   int i=0;
   int stage = 0;
   for (i=0; i<l; i++) {
      if (stage == 0 && (s[i] == ' ' || s[i] == '\t' || s[i] == ',')) continue;
      // comment detected
      if (stage == 0 && (s[i] == '/' || s[i] == '(') ) break;
      if (s[i] == '\"') {
         if (stage == 1) {
            stage = 2;
            break;
         }
         stage = 1;
         continue;
      }
      // inside std::string
      if (stage == 1) {
         if (s[i] == '\\' && i+1 < l) {
            i++;
            QChar qc = s[i+1];
            char ch = qc.toLatin1();
            switch (ch) {
               case '\\': result += '\\'; break;
               case '\"': result += '\"'; break;
               case '\'': result += '\''; break;
               case '\n': result += '\n'; break;
               case '\r': result += '\r'; break;
               case '\b': result += '\b'; break;
               case '\t': result += '\t'; break;
               case '\f': result += '\f'; break;
               case '\a': result += '\a'; break;
               case '\v': result += '\v'; break;
               case '\?': result += '\?'; break;
            }
         } else {
            result += s[i];
         }
      }
   }
   if (stage != 2) {
      if (stage == 0) {
         addError("error while parsing std::string - opening quotes (\") not found");
      } else {
         addError("error while parsing std::string - closing quotes (\") not found");
      }
      return "";
   } else {
      s = s.right(l-i-1);
      #ifdef _DEBUG
      std::string test = qString2STLString(result);
      test;
      #endif
      return qString2STLString(result);
   }
}

enum MacroType {INVALID, ADD_DEPENDENCY, /*ADD_DEPENDENCY_BUILD, */ ADD_LICENSE, ADD_AUTHOR, ADD_INSTITUITION, ADD_ICON, ADD_CLASS, ADD_CLASS_INC, TAG_MODULE, SET_MODULE_TYPE_LOADABLE_MODULE, SET_MODULE_TYPE_APPLICATION};


TagList *TagHeaderParser::parseTagFile(const std::string& filename) 
{
   #ifdef _DEBUG
   QDir dir;
   std::string test = qString2STLString(dir.currentPath());
   #endif
   lineNumber = 0;
   QFile inFile(filename.c_str());
   if( !inFile.open(QIODevice::ReadOnly) ) {
      addError("TagHeaderParser::parseTagFile() - file not found.");
      return NULL;
   }
   QTextStream inStream(&inFile);
   TagList *result = new TagList();
   while (!inStream.atEnd()) {
      QString line = inStream.readLine().trimmed();
      lastLineRead = qString2STLString(line);
      lineNumber++;
      QString keywordStr = findKeyword(line);
      MacroType keyword = INVALID;

      if (keywordStr == "ADD_DEPENDENCY") {
         keyword = ADD_DEPENDENCY;
      //} else if (keywordStr == "ADD_DEPENDENCY_BUILD") {
      //   keyword = ADD_DEPENDENCY_BUILD;
      } else if (keywordStr == "ADD_LICENSE") {
         keyword = ADD_LICENSE;
      } else if (keywordStr == "ADD_AUTHOR") {
         keyword = ADD_AUTHOR;
      } else if (keywordStr == "ADD_INSTITUITION") {
         keyword = ADD_INSTITUITION;
      } else if (keywordStr == "ADD_ICON") {
         keyword = ADD_ICON;
      } else if (keywordStr == "TAG_MODULE") {
         keyword = TAG_MODULE;
      } else if (keywordStr == "ADD_CLASS") {
         keyword = ADD_CLASS;
      } else if (keywordStr == "ADD_CLASS_INC") {
         keyword = ADD_CLASS_INC;
      } else if (keywordStr == "SET_MODULE_TYPE_LOADABLE_MODULE") {
         keyword = SET_MODULE_TYPE_LOADABLE_MODULE;
      } else if (keywordStr == "SET_MODULE_TYPE_APPLICATION") {
         keyword = SET_MODULE_TYPE_APPLICATION;
      } else {
         continue;
      }
      std::string arg1, arg2, arg3;
      switch (keyword) {
         case ADD_DEPENDENCY:
//         case ADD_DEPENDENCY_BUILD:
         case ADD_LICENSE:
         case ADD_AUTHOR:
         case ADD_INSTITUITION: 
         {
            arg1 = getString(line);
            break;
         }

         case ADD_ICON: {
            continue;
         }

         case TAG_MODULE:
         case ADD_CLASS: 
         {
            arg1 = getString(line);
            arg2 = getString(line);
            break;
         }
         case ADD_CLASS_INC: 
         {
            arg1 = getString(line);
            arg2 = getString(line);
            arg3 = getString(line);
            break;
         }
         case SET_MODULE_TYPE_LOADABLE_MODULE:
         case SET_MODULE_TYPE_APPLICATION:
         {
            break;
         }
         default: x4Assert(false);
      };

      switch (keyword) {
			case ADD_ICON: break;
         case ADD_DEPENDENCY:       result->addTag("DEPENDENCIES",       arg1);                               break;
//         case ADD_DEPENDENCY_BUILD: result->addTag("DEPENDENCIES_BUILD", arg1);                               break;
         case ADD_LICENSE:          result->addTag("LICENSE",            arg1);                               break;                                          
         case ADD_AUTHOR:           result->addTag("AUTHORS",            arg1);                               break;
         case ADD_INSTITUITION:     result->addTag("INSTITUTION",        arg1);                               break;
         case ADD_CLASS:            result->addTag("CLASSES",            arg1 + std::string(" inherits ") + arg2); break;
         case ADD_CLASS_INC:        result->addTag("CLASSES",            arg1 + std::string(" in ") + arg3 + std::string(" inherits ") + arg2); break;
         case TAG_MODULE:           result->addTag(arg1, arg2);                                               break;
         case SET_MODULE_TYPE_LOADABLE_MODULE: result->addTag("MODULE_TYPE", "LOADABLE_MODULE");              break;
         case SET_MODULE_TYPE_APPLICATION:     result->addTag("MODULE_TYPE", "APPLICATION");                  break;
			case INVALID: addError(std::string("TagHeaderParser::parseTagFile() - invalid keyworkd encountered. Line: ")+intToStr(lineNumber)); break;
      };
   }
   if (errorLevel) {
      delete result;
      return NULL;
   } else {
      return result;
   }
}

TagHeaderParser::TagHeaderParser() 
{
   errorLevel = 0;
}

void TagHeaderParser::addError(const std::string& error) 
{
   errorLevel = 1;
   errors += std::string("[line ") + intToStr(lineNumber) + std::string("] ") + lastLineRead + std::string("\n");
   errors += error;
   errors += "\n";
}
