#ifndef TaggingMacrosH
#define TaggingMacrosH

#include <X4GlobalDefinitions.h>
#include "x4modules.h"
#include "Module.h"
#include "ModuleList.h"
#include <X4Exception.h>
#include "ConsoleOutput.h"

// --- tags


#define TAG_MODULE(tagname, value)       mod->addTag( tagname , value );

#define ADD_DEPENDENCY(moduleName)       TAG_MODULE("DEPENDENCIES", moduleName )
//#define ADD_DEPENDENCY_BUILD(moduleName) TAG_MODULE("DEPENDENCIES_BUILD", moduleName )
#define ADD_LICENSE(license)             TAG_MODULE("LICENSE", license )
#define ADD_AUTHOR(author)               TAG_MODULE("AUTHORS", author )
#define ADD_INSTITUITION(institution)    TAG_MODULE("INSTITUTION", institution )
#define ADD_ICON(icon)                   TAG_MODULE("ICON", icon )
#define ADD_CLASS(newClass,parentClass)  TAG_MODULE("CLASSES", newClass##" inherits "##parentClass )
#define ADD_CLASS_INC(newClass,parentClass,includeFile)  TAG_MODULE("CLASSES", newClass##" in "##includeFile##" inherits "##parentClass )
#define SET_MODULE_TYPE_LOADABLE_MODULE  TAG_MODULE("MODULE_TYPE","LOADABLE_MODULE")
#define SET_MODULE_TYPE_APPLICATION      TAG_MODULE("MODULE_TYPE","APPLICATION")
#define ADD_REMARK(remark)					  TAG_MODULE("REMARK", remark )



// these definitions only work for visual studio / windows
#if defined(X4_PLATFORM_MSVC)

   // --- loadable (DLL) modules

   #define BEGIN_X4_LOADABLE_MODULE(moduleName) static void register_##moduleName##() { \
      X4::Module *mod = new X4::Module( #moduleName ); 

   #define END_X4_LOADABLE_MODULE(moduleName)    X4::x4modules.addModule(mod);} \
   \
   BOOL APIENTRY DllMain(HMODULE hModule,\
                         DWORD  ul_reason_for_call,\
                         LPVOID lpReserved )\
	{\
		UNREFERENCED_PARAMETER(hModule);\
		UNREFERENCED_PARAMETER(lpReserved);\
		switch (ul_reason_for_call)\
		{\
		case DLL_PROCESS_ATTACH:\
			try {\
				register_##moduleName##(); \
				init_##moduleName##();\
				postInit();\
			} catch (X4::X4Exception &e) { \
				X4::x4console << std::string("Exception during init of ")+ std::string( #moduleName ) \
				+ std::string(": X4 Exception of type ")+ e.getTypeString()+std::string(". Error message: ") + e.getMsg() + std::string("\n"); \
				throw e;\
			} catch (...) {\
				X4::x4console << std::string("Generic exception during init of ") + std::string( #moduleName ); \
				throw; \
			} \
			break;\
		case DLL_THREAD_ATTACH:\
			break;\
		case DLL_THREAD_DETACH:\
			break;\
		case DLL_PROCESS_DETACH:\
			try {\
				preShutdown();\
				shutdown_##moduleName##();\
			} catch (X4::X4Exception &e) { \
				X4::x4console << std::string("Exception during shutdown of ")+ std::string( #moduleName ) \
				+ std::string(": X4 Exception of type ")+ e.getTypeString()+std::string(". Error message: ") + e.getMsg() + std::string("\n"); \
				throw e;\
			} catch (...) {\
				X4::x4console << std::string("Generic exception during shutdown of ") + std::string( #moduleName ); \
				throw; \
			} \
			break;\
		};\
		return TRUE;\
	}



   // --- Application (EXE) modules

   #define BEGIN_X4_APPLICATION_MODULE(moduleName) static void register_##moduleName##() { \
      X4::Module *mod = new X4::Module( #moduleName ); 

   #define END_X4_APPLICATION_MODULE(moduleName)    X4::x4modules.addModule(mod);}

   #define BEGIN_INIT_MODULE static void postInit() {
   #define END_INIT_MODULE } 

   #define BEGIN_SHUTDOWN_MODULE static void preShutdown() {
   #define END_SHUTDOWN_MODULE } 


// these definitions only work for g++ on windows
#elif defined(X4_PLATFORM_MINGW_GCC)
	template <class T>
	void UNREFERENCED_PARAMETER_GCC(T) {}

	// --- loadable (DLL) modules

	#define BEGIN_X4_LOADABLE_MODULE(moduleName) static void register_##moduleName() { \
		X4::Module *mod = new X4::Module( #moduleName );

	#define END_X4_LOADABLE_MODULE(moduleName)    X4::x4modules.addModule(mod);} \
	\
	BOOL APIENTRY DllMain(HMODULE hModule,\
								 DWORD  ul_reason_for_call,\
								 LPVOID lpReserved )\
	{\
		UNREFERENCED_PARAMETER_GCC(hModule);\
		UNREFERENCED_PARAMETER_GCC(lpReserved);\
		switch (ul_reason_for_call)\
		{\
		case DLL_PROCESS_ATTACH:\
			try {\
				register_##moduleName(); \
				init_##moduleName();\
				postInit();\
			} catch (X4::X4Exception &e) { \
				X4::debugOutput << std::string("Exception during init of ")+ std::string( #moduleName ) \
				+ std::string(": X4 Exception of type ")+ e.getTypeString()+std::string(". Error message: ") + e.getMsg() + std::string("\n"); \
				throw e;\
			} catch (...) {\
				X4::debugOutput << std::string("Generic exception during init of ") + std::string( #moduleName ); \
				throw; \
			} \
			break;\
		case DLL_THREAD_ATTACH:\
			break;\
		case DLL_THREAD_DETACH:\
			break;\
		case DLL_PROCESS_DETACH:\
			try {\
				preShutdown();\
				shutdown_##moduleName();\
			} catch (X4::X4Exception &e) { \
				X4::debugOutput << std::string("Exception during shutdown of ")+ std::string( #moduleName ) \
				+ std::string(": X4 Exception of type ")+ e.getTypeString()+std::string(". Error message: ") + e.getMsg() + std::string("\n"); \
				throw e;\
			} catch (...) {\
				X4::debugOutput << std::string("Generic exception during shutdown of ") + std::string( #moduleName ); \
				throw; \
			} \
			break;\
		};\
		return TRUE;\
	}



	// --- Application (EXE) modules

	#define BEGIN_X4_APPLICATION_MODULE(moduleName) static void register_##moduleName##() { \
		X4::Module *mod = new X4::Module( #moduleName );

	#define END_X4_APPLICATION_MODULE(moduleName)    X4::x4modules.addModule(mod);}

	#define BEGIN_INIT_MODULE static void postInit() {
	#define END_INIT_MODULE }

	#define BEGIN_SHUTDOWN_MODULE static void preShutdown() {
	#define END_SHUTDOWN_MODULE }

#endif


#endif
