//---------------------------------------------------------------------------
#ifndef UnitTestsH
#define UnitTestsH
//---------------------------------------------------------------------------
#include "x4modules.h"
#include <X4Types.h>
//---------------------------------------------------------------------------
#include <vector>
#include <string>
//---------------------------------------------------------------------------

// namespace X4
// {
// 
// 	struct UnitTest
// 	{
// 		typedef void (*Method)();
// 
// 		Method exec;
// 		std::string name;
// 		std::string module;
// 
// 		UnitTest(Method exec, const std::string& name, const std::string& module) : exec(exec), name(name), module(module) {}
// 	};
// 
// 	class X4MODULES_API UnitTests
// 	{
// 	private:
// 		std::map<std::string, std::vector<UnitTest> > tests;
// 		std::string currentTestGroup;
// 
// 	public:
// 
// 		void     addUnitTest(UnitTest::Method method, const std::string& name, const std::string& module);
// 		size_t   getNumUnitTests(const std::string& module);
// 		UnitTest getUnitTest(const std::string& module, size_t i);
// 
// 		typedef std::map<std::string, std::vector<UnitTest> > UnitTestMap;
// 
// 		const UnitTestMap& getUnitTestMap() const { return tests; }
// 
// 		void setCurrentTestGroup(const std::string& name) { currentTestGroup = name; }
// 		const std::string& getCurrentTestGroup() const { return currentTestGroup; }
// 
// 		static void addTest(UnitTest::Method test, const std::string& name);
// 		static void addTest(UnitTest::Method test, const std::string& name, const std::string& module);
// 	};
// 
// 	extern X4MODULES_API UnitTests unitTests;
// 
// 	#define UNITTEST_ASSERT(expr, errormsg, action) {\
// 	if (!expr)\
// 	{\
// 		error(errormsg);\
// 		action;\
// 	}\
// 	}
// };

namespace X4 {

   typedef void (*UnitTest)();

   class X4MODULES_API UnitTests {
   private:
      static std::vector<UnitTest> tests;
      static std::vector<std::string> names;

   public:
      static void addTest(UnitTest test, std::string name);
      static card32 getNumTests();
      static UnitTest getTest(card32 i);
      static std::string getTestName(card32 i);
   };

};


#endif

