//---------------------------------------------------------------------------
#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "MainWindow.h"
//---------------------------------------------------------------------------
#include "ObjectDictionary.h"
#include "MetaClass.h"
#include "ClassEditor.h"
#include "OnlineHelp.h"
#include "ProgressWindow.h"
#include "InfoDialogsWidget.h"
#include "ClassCreator.h"
#include "ObjectDictionaryEditorWidget.h"
#include "UserSettings.h"
#include "ExceptionHandling.h"
#include "BinaryObjectStreams.h"
#include "CommandHistory.h"
#include "CommandObjMacro.h"
#include "DebugOutput.h"
#include "Timer.h"
#include "InterfaceGlobals.h"
#include "ModuleManagerWindow.h"
#include "ConsoleOutput.h"
//---------------------------------------------------------------------------
#include "MainWindow_moc.h"
//---------------------------------------------------------------------------
#include <QBoxlayout>
#include <QEvent>
#include <QPushButton>
#include <QMenuBar>
#include <QMessageBox>
//---------------------------------------------------------------------------

using namespace X4;


MainWindow::MainWindow(QWidget * parent, Qt::WindowFlags f)
:QMainWindow(parent, f)
{
	ui.setupUi(this);

	setWindowTitle( "GeoXL 3.5" );

	setAttribute(Qt::WA_DeleteOnClose);

	m_MainObject = new ObjectDictionary;
	m_MainEditor = (ClassEditor*)m_MainObject->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();


	QWidget * edWidget = m_MainEditor->makeEditor( ui.memoryFrame, m_MainObject );
	m_ObjectDictEditor = (ObjectDictionaryEditorWidget*)edWidget;

	QVBoxLayout *layout = new QVBoxLayout(ui.memoryFrame);
	layout->addWidget(edWidget);
	layout->setMargin(3);

   card32 n = UnitTests::getNumTests();
   for (card32 i=0; i<n; i++) {
      UnitTest t = UnitTests::getTest(i);
      std::string name = UnitTests::getTestName(i);
      UnitTestEntry e;
      e.test = t;
      e.action = ui.menuUnit_Tests_and_Experiments->addAction(name.c_str()); 
      unitTestEntries.push_back(e);
   }
	move(0,0);

	moduleManager = new ModuleManagerWindow(this);
	moduleManager->setVisible(false);
	moduleManager->loadStartupModules();

// x4 version - work in progress
//    UnitTests::UnitTestMap um = UnitTests::getUnitTestMap();
//    UnitTests::UnitTestMap::iterator uit = um.begin();
//    UnitTests::UnitTestMap::iterator ue = um.end();
// 
//    while (uit != ue) {
//       const card32 n = uit->size();
//       for (card32 i=0; i<n; i++) {
//          UnitTest t = uit->second()[i];
//          std::string name = UnitTests::getTestName(i) + std::string(" [") + ui->first() + std::string("]");
//          UnitTestEntry e;
//          e.test = t;
//          e.action = ui.menuUnit_Tests_and_Experiments->addAction(name.c_str()); 
//          unitTestEntries.push_back(e);
//       }
//       uit++;
//    }


	ui.actionShow_Debug_Window->setShortcutContext( Qt::ApplicationShortcut );
	m_ConfigurationEditor = NULL;
	m_ConfigurationWidget = NULL;

	//on_actionCreate_New_Object_triggered();

//     m_idleTimer = new QTimer(this);
//     QWidget::connect(m_idleTimer, SIGNAL(timeout()), this, SLOT(idleTimer()));

//        m_idleTimer->setSingleShot(false);
//    m_idleTimer->start(500);

}

void MainWindow::initConfigEditor()
{
	m_ConfigurationEditor = (ClassEditor *)Configuration::getConfiguration()->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
	m_ConfigurationWidget = m_ConfigurationEditor->makeEditor( this , Configuration::getConfiguration() );
	m_ConfigurationWidget->hide();
	m_ConfigurationWidget->setWindowFlags( Qt::Window );
}

MainWindow::~MainWindow()
{
	delete m_ConfigurationEditor;
   //delete m_idleTimer;
}

// void MainWindow::idleTimer()
// {
//     // update application, so that it does not get stalled
//     InterfaceGlobals::getApplication()->processEvents(QEventLoop::AllEvents);
// }

void MainWindow::on_menubar_triggered(QAction *action)
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
   size_t n = unitTestEntries.size();
   for (card32 i=0; i<n; i++) {
      UnitTestEntry e = unitTestEntries[i];
      if (e.action == action) {
         if (e.test != NULL) {
            (e.test)();
         }
         return;      
      }
   }
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}


void MainWindow::on_newButton_clicked(){
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_actionCreate_New_Object_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_exitButton_clicked(){
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_actionE_xit_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_openButton_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_action_Open_Object_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_saveButton_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_actionSave_Object_As_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_actionCreate_New_Object_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	const MetaClass * mc = MetaClass::getClassByName( "Scene" );
   if (mc) {
	ClassCreator * cc = (ClassCreator*)mc->getGUIDescriptor()->getDefaultClassCreator()->newInstance();
	x4Assert(cc);
	Object * newObj = cc->makeObject();
	delete cc;
	if (newObj != NULL) {
		m_MainObject->addObject(newObj);
	}
	m_MainEditor->update();
   } else {
      warning("class Scene not found.");
   }
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_action_Open_Object_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_ObjectDictEditor->load();		
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_actionSave_Object_As_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_ObjectDictEditor->save();		
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_actionConfiguration_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (m_ConfigurationEditor && m_ConfigurationWidget) {
		m_ConfigurationWidget->hide();
		m_ConfigurationEditor->update();
		m_ConfigurationWidget->show();
	}
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_actionSafe_Configuration_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
   Configuration::writeConfiguration();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::cleanupAllocs() {
// 	// force clean shutdown for all multi object stream objects
// 
// 	#ifndef _DEBUG
// 	try {
// 	#endif
// 		
// 		std::vector<HierarchicalOutOfCorePointCloud*> remainingOOCPCs = HierarchicalOutOfCorePointCloud::getActiveOOCPCs();
// 		for (card32 i=0; i<remainingOOCPCs.size(); i++) {
// 			remainingOOCPCs[i]->shutdownAllocator();
// 		}
// 
// 	#ifndef _DEBUG
// 	} catch (...) {}
// 	
// 	try {
// 	#endif
// 		bool goOn = false;
// 		do {
// 			std::vector<MultiObjectStream*> remainingStreams = MultiObjectStream::getActiveStreams();
// 			if (!remainingStreams.empty()) {
// 				remainingStreams[0]->sync();
// 				delete remainingStreams[0];
// 				goOn = true;
// 			} else {
// 				goOn = false;
// 			}
// 		} while (goOn);
// 	#ifndef _DEBUG
// 	} catch (...) {}
// 	#endif
}


void MainWindow::on_actionE_xit_triggered()
{
	close();
}

void MainWindow::closeEvent(QCloseEvent *event) {

	#if !defined(_DEBUG) && !defined(X4_MAIN_FAST_DEBUG)

	UserSettings * u = dynamic_cast<UserSettings*>( Configuration::getConfiguration()->getData( "USER_SETTINGS" ) );
	bool askBeforeQuit = false;
	if( u )
		askBeforeQuit = u->getAskBeforeQuit();
	if( askBeforeQuit )
	{
		if (QMessageBox::question(this, "Confirm closing...", "Do you want to close GeoXL?", QMessageBox::Ok, QMessageBox::Cancel) == QMessageBox::Ok) {
			event->setAccepted(true);
			cleanupAllocs();
			// fix qt413 bug
			QApplication::exit(0);
		} else {
			event->setAccepted(false);
		}
	} else {
		cleanupAllocs();
		// fix qt413 bug
		QApplication::exit(0);
	}

	#else
		event->setAccepted(true);
		cleanupAllocs();
		// fix qt413 bug
		QApplication::exit(0);
	#endif
}

void MainWindow::on_actionShow_Debug_Window_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	static bool first = true;
	//if (first) {
	//   first = false;
	//   x4console.moveConsoleWindow(310,50);
	//	x4console.show();
	//} else {
	x4console.toggleVisible();
	//}
	//ui.actionShow_Debug_Window->setChecked(!x4console->isHidden());
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_action_Online_Help_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	OnlineHelp::newTopic("index.html");
	OnlineHelp::show();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_action_Index_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	OnlineHelp::showIndex();
	OnlineHelp::show();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_action_Info_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	InfoDialogsWidget *info = new InfoDialogsWidget;
	info->show();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

// void X4::MainWindow::setBatchMode( std::string in, std::string out, std::string script )
// {
// 	inputFile=in;
// 	outputFile=out;
// 	scriptFile=script;
// 	QTimer::singleShot(3000, this, SLOT(startBatch()));
// 	
// }

// void X4::MainWindow::startBatch()
// {
// 	
// 	debugOutput <<"\n-runbatch\n=============\n"; 
// 	
// 	BinaryInputObjectStream in(inputFile.c_str());
// 	Persistent *instance = NULL;
// 	debugOutput << ("loading " + inputFile +"\n");
// 	debugOutput.flush();
// 	in.readObject(instance);
// 	Scene* inputScene= dynamic_cast<Scene*>(instance);
// 
// 	BinaryInputObjectStream inScript(scriptFile.c_str());
// 	Persistent *obj = NULL;
// 	inScript.readObject(obj);
// 
// 	debugOutput << ("processing " + scriptFile +" :\n");
// 
// 	Timer wholeT; card32 vt;
// 	Timer T;
// 
// 
// 	CommandObject* cmdObj = dynamic_cast<CommandObject*>(obj);
// 	if (cmdObj == NULL) {
// 		CommandObjMacro* macro = dynamic_cast<CommandObjMacro*>(obj);
// 		if (macro == NULL) {
// 			QMessageBox::critical(this, "Wrong object type", "Only command objects and macros can be executed");
// 			return;
// 		} else {
// 			STD_GLOBAL_EXCEPTION_HANDLER_START
// 				wholeT.getDeltaValue();
// 				for (card32 i = 0; i < macro->numCommands(); ++i) {
// 					CommandObject* cmdObj = const_cast<CommandObject*>(macro->getCommand(i));
// 					debugOutput << "-> "<< cmdObj->getDescription()<<"\n";
// 					debugOutput.flush();
// 					T.getDeltaValue();
// 					inputScene->getCommandHistory()->executeCommand(inputScene, cmdObj);
// 					vt = T.getDeltaValue();
// 					if (vt > 5)
// 						debugOutput << "  operation time: "<< vt << " ms\n";
// 				}
// 				vt = wholeT.getDeltaValue();
// 				debugOutput << "Preprocessing time: "<< vt << " ms\n";
// 			STD_GLOBAL_EXCEPTION_HANDLER_END
// 		}
// 	} else {
// 		CmdObjScriptProgram* script = dynamic_cast<CmdObjScriptProgram*>(cmdObj);
// 		if (script != NULL) {
// 			try {
// 				debugOutput << "-> "<< cmdObj->getDescription()<<"\n";
// 				debugOutput.flush();
// 				inputScene->getCommandHistory()->executeCommand(inputScene, cmdObj);
// 			} catch(X4Exception &e) {
// 				std::string msg = "Error in line " + intToStr(script->getScript()->getLine() + 1) + ": \n" + e.getMsg();
// 				QMessageBox::critical( this, "GeoXL script error", QString(msg.c_str()));
// 			}
// 		} else {
// 			STD_GLOBAL_EXCEPTION_HANDLER_START
// 				debugOutput << "-> "<< cmdObj->getDescription()<<"\n";
// 				debugOutput.flush();
// 				inputScene->getCommandHistory()->executeCommand(inputScene, cmdObj);
// 			STD_GLOBAL_EXCEPTION_HANDLER_END
// 		}
// 	}
// 
// 	debugOutput << ("writing to " + outputFile +"\n");
// 	debugOutput.flush();
// 	BinaryOutputObjectStream out(outputFile.c_str());
// 	out.writeObject((Persistent*)inputScene);
// 
// 
// 	delete inputScene;
// 
// 	debugOutput << "done\n";
// 	debugOutput.flush();
// 
// 	close();
// }

void X4::MainWindow::loadFile( std::string inputFile )
{
	this->inputFile=inputFile;
	QTimer::singleShot(100, this, SLOT(openFileAndSwitchToTool()));
}

void X4::MainWindow::openFileAndSwitchToTool()
{
	
	if (FileExists(inputFile.c_str()))
	{
		m_ObjectDictEditor->loadFile(inputFile);
		m_ObjectDictEditor->openFirstItem();
	}
	else
		debugOutput << "File: " << inputFile <<" not found!\n";
}

void X4::MainWindow::on_modulesButton_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_action_Module_Manager_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void X4::MainWindow::on_action_Module_Manager_triggered()
{
	moduleManager->update();
	moduleManager->setVisible(true);
}
