//---------------------------------------------------------------------------
#ifndef MainWindowH
#define MainWindowH
//---------------------------------------------------------------------------
#include "X4GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MenuWidget.h"
#include "MainWindow_ui.h"
#include "UnitTests.h"
#include <QtCore/QTimer>
//---------------------------------------------------------------------------

class QPushButton;

namespace X4
{
class Object;
class ClassEditor;
class ObjectDictionary;
class ObjectDictionaryEditorWidget;
class ModuleManagerWindow;

class MainWindow : public QMainWindow {
	Q_OBJECT
 private:
	Ui_MainWindow ui;
	struct UnitTestEntry {
	   UnitTest test;
	   QAction *action;
	};
	std::vector<UnitTestEntry> unitTestEntries;

	ObjectDictionary		*	m_MainObject;
	ClassEditor				*	m_MainEditor;
	ObjectDictionaryEditorWidget * m_ObjectDictEditor;
	ModuleManagerWindow  * moduleManager;

	ClassEditor				*	m_ConfigurationEditor;
	QWidget					*	m_ConfigurationWidget;
   // QTimer                  *   m_idleTimer;

	void cleanupAllocs();
 protected:
	void closeEvent(QCloseEvent *event);

 public:
	MainWindow(QWidget * parent=NULL, Qt::WindowFlags f=0);
	/// call after configuration has been read (which can only been done after the main window has initialized and thereby loaded the modules; this is
	/// currently a design flaw).
	void initConfigEditor();

	~MainWindow(void);

 private slots:
	void on_newButton_clicked();
	void on_exitButton_clicked();
	void on_openButton_clicked();
	void on_saveButton_clicked();
	void on_modulesButton_clicked();
	void on_menubar_triggered(QAction* action);

	void on_actionCreate_New_Object_triggered();
	void on_action_Open_Object_triggered();
	void on_actionSave_Object_As_triggered();
	void on_actionConfiguration_triggered();
	void on_actionSafe_Configuration_triggered();
	void on_actionE_xit_triggered();
	void on_actionShow_Debug_Window_triggered();
	void on_action_Online_Help_triggered();
	void on_action_Index_triggered();
	void on_action_Info_triggered();
	void on_action_Module_Manager_triggered();
	// void startBatch();
	void openFileAndSwitchToTool();

   // void idleTimer();

 protected:
	 std::string inputFile;
	 std::string outputFile;
	 std::string scriptFile;

public:
	// void setBatchMode(std::string inputFile, std::string outputFile, std::string scriptFile);
	void loadFile(std::string inputFile);


};
}

#endif