//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "X4GlobalDefinitions.h"
//---------------------------------------------------------------------------
// // Using the Memory Leak Detector can be turned on and off in the following include file (only in Debug Mode)
// #include "VLDLeakDetection.h"
// #if defined(_DEBUG) && defined(DO_VLD_LEAK_DETECTION)
//    #include "externalIncludes/vld/vld.h"
// #endif
//---------------------------------------------------------------------------
#include <QApplication>
#include <QBoxlayout>
#include <QtGui/QMotifStyle>
#include <QtGui/QCDEStyle>
#include <QtGui/QWindowsStyle>
#include <QtGui/QPlastiqueStyle>
#include <QtGui/QCleanlooksStyle>
#include <QtGui/QWindowsXPStyle>
#include <QtCore/QString>
#include <QtCore/QMetaType>
//---------------------------------------------------------------------------
#include <Psapi.h>
#include <Shellapi.h>
#include <direct.h>
#include "SIGNAL.H."
//---------------------------------------------------------------------------
#include <iostream>
//---------------------------------------------------------------------------
#include "InterfaceGlobals.h"
//---------------------------------------------------------------------------
#include "MainWindow.h"
#include "ClassSelectionDialog.h"
#include "Persistent.h"
#include "BinaryObjectStreams.h"
#include "Configuration.h"
#include "ObjectDictionary.h"
//---------------------------------------------------------------------------
#include "Tools.h"
#include "UserSettings.h"
#include "ExceptionHandling.h"
#include "ProgressWindow.h"
#include "StringHelper.h"
#include "DebugOutput.h"
#include "StringAttachment.h"
#include "ApplicationDirectories.h"
#include "ConsoleWindowProcess.h"
#include "ModuleEvents.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

//#ifdef _MSC_VER
//#    ifdef NDEBUG
//#        pragma comment(linker, "/subsystem:windows")
////#        pragma comment(linker, "/entry:WinMainCRTStartup")
//#    else
//#        pragma comment(linker, "/subsystem:console")
////#        pragma comment(linker, "/entry:WinMainCRTStartup")
//#    endif
//#endif

#define GEOXL35_MAXPATH 10000

static std::string wString2String(std::wstring x) {
    std::string result;
    const size_t n = x.size();
    result.resize(n);
    for (unsigned i=0; i<n; i++) {
        if (x[i] < 256) {
            result[i] = x[i];
        } else {
            throw X4Exception("wString2String() - out of ascii character range.");
        }
    }
    return result;
}

std::wstring string2wString(const std::string& s)
{
    std::wstring temp(s.length(),L' ');
    std::copy(s.begin(), s.end(), temp.begin());
    return temp;
}

static void setStyle(card32 i) {
   switch (i) {
      case 1: QApplication::setStyle(new QMotifStyle); break;
      case 2: QApplication::setStyle(new QCDEStyle); break;
      case 3: QApplication::setStyle(new QWindowsStyle); break;
      case 4: QApplication::setStyle(new QCleanlooksStyle); break;
      case 5: QApplication::setStyle(new QPlastiqueStyle); break;
      case 6: QApplication::setStyle(new QWindowsXPStyle); break;
	   default: break;
   }
}

static void writeBackConfig() {
   try {
      Configuration::writeConfiguration();
   } catch (...) {
      consoleWarning("x4core.cpp (module init) - failed to write back configuration file config.x4obj at default location.");
   }	
   Configuration::deleteConfiguration();
}

static void loadConfig() {
   try {
      Configuration::loadConfiguration();
      if (Configuration::getConfiguration() == NULL) throw 1;
   } catch (...) {
      consoleWarning(std::string("x4core.cpp (module init) - failed to load configuration file config.x4obj from default location(")
         + ApplicationDirectories::getConfigDir() + std::string("config.x4obj") + std::string("). creating empty (default) object."));
      Configuration::createConfiguration();
   }
}


void X4_SIGABRT_Handler(int signal) {
	throw X4Exception(std::string("SIGABRT caught (signal #")+intToStr(signal)+std::string(")."));
}

//#ifdef _MSC_VER
//#    ifdef NDEBUG
int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);
	UNREFERENCED_PARAMETER(nCmdShow);
//#    else
//int _tmain(int argc, TCHAR* argv[])
//{
//#    endif
//#endif

	// signal handlers

	typedef void (*SignalHandlerPointer)(int);

   SignalHandlerPointer previousSIGABRTHandler;
   previousSIGABRTHandler = signal(SIGABRT, X4_SIGABRT_Handler);



   // Qt - Initialization
   //register default types by Qt, in order to have queued signal/slot support
   qRegisterMetaType<card32>("card32");
   qRegisterMetaType<std::string>("std::string");
   qRegisterMetaType<card64>("card64");
   qRegisterMetaType<time_t>("time_t");
    
   #if defined(_DEBUG) && defined(DO_VLD_LEAK_DETECTION)
      // turn off leak detection by default (turn on again manually where needed)
      VLDDisable();
   #endif

	// mw - activate heavy heap validation here ;-)
	#ifdef _DEBUG
	// int tmpFlag = _CrtSetDbgFlag( _CRTDBG_REPORT_FLAG );

	// tmpFlag |= _CRTDBG_ALLOC_MEM_DF;		  // turn on debugging
	// tmpFlag |= _CRTDBG_DELAY_FREE_MEM_DF; // never free any memory
	// tmpFlag |= _CRTDBG_CHECK_ALWAYS_DF;	  // heap check at every allocation operation
	// tmpFlag |= _CRTDBG_CHECK_CRT_DF;		  // include crt (runtime) memory
	// tmpFlag |= _CRTDBG_LEAK_CHECK_DF;     // leak check at programm exit

	// turn this into a comment this line if none of the above are uncommented
	// std::cout << "Currently using debug heap - may cause unwanted effects; uncomment in geoxl35.cpp to turn off\n";

	// _CrtSetDbgFlag( tmpFlag );	
	#endif

	setStructureExceptionHandler();


	LPWSTR commandLine  = GetCommandLineW();
	int numCmdLineItems = 0;
	LPWSTR *cmdLineItems = CommandLineToArgvW(commandLine, &numCmdLineItems);

   int n = 0;
   QApplication qapp(n, 0);
	InterfaceGlobals::setApplication(&qapp);

	ApplicationDirectories::setup();

   // store working directory
   WCHAR exechar[GEOXL35_MAXPATH];
	GetModuleFileName(NULL, exechar, GEOXL35_MAXPATH);
	std::wstring exePath = exechar;
	exePath.resize(exePath.find_last_of('\\')); // removes the execution file name, so goes just to the path
	std::string dir = wString2String(exePath);

   std::cout << "BIN_PATH = "<< dir << "\n";
   
	// ------ create debug window in separate thread -------
	
	// start console as an additional process
	QProcess *consoleProc = new QProcess();
	string prog = ApplicationDirectories::getBinDir() + string("/x4console.exe");
	QStringList arguments;
	consoleProc->start(prog.c_str(), arguments);
	ConsoleWindowProcess *consoleProcHandler = ConsoleWindowProcess::createGlobalConsoleWindowProcess(consoleProc);
	consoleProcHandler->sendParentID(consoleProc->pid());
	x4console.addHandler(consoleProcHandler);
	// -----------------------------------------------------


   UserSettings * userSettings = dynamic_cast<UserSettings*>( 
      Configuration::getConfiguration()->getData( "USER_SETTINGS" ));
   if( userSettings == NULL )
   {
      userSettings = new UserSettings;
      userSettings->setup( "USER_SETTINGS", AttachedData::ADF_PERSISTENT);
      Configuration::getConfiguration()->attachData( userSettings );
   }

   setStyle(userSettings->getInterfaceStyle());

	StringAttachment * binPath = new StringAttachment;
	binPath->setup( "BIN_PATH", 0 );
	binPath->setString( dir );
	Configuration::getConfiguration()->attachData( binPath );

	// create main window	
	MainWindow * mw = new MainWindow;
	mw->show();
	InterfaceGlobals::setMainWindow(mw);
	qapp.setActiveWindow( mw );
	consoleProcHandler->moveWin(mw->width()+10,0);

	// create progress window (child of main window)
	progressWindow = new ProgressWindow();
	
	//Close all windows with main window
	QApplication::connect(mw, SIGNAL(destroyed()), &qapp, SLOT(closeAllWindows()));

	// output machine information
	char com_name[255];
	unsigned nSize = sizeof(com_name);
	GetComputerNameA(com_name, (LPDWORD)&nSize); 
	debugOutput << "ComputerName: "<< com_name << "\n";

	SYSTEM_INFO info;
	GetSystemInfo( &info );
	debugOutput << "Machine information: #Processors:" << (unsigned)info.dwNumberOfProcessors;
	switch( info.wProcessorArchitecture )
	{
	case PROCESSOR_ARCHITECTURE_AMD64:
		debugOutput << "|x86|AMD64|";
		break;
	case PROCESSOR_ARCHITECTURE_INTEL:
		debugOutput << "|x86|Intel|";
			break;
	case PROCESSOR_ARCHITECTURE_IA64:
		debugOutput << "|IPF|";
		break;
	default:
		debugOutput << "|Unknown Architecture|";
		break;
	}

	debugOutput << "\n";
   debugOutput << "Executable: " << wString2String(cmdLineItems[0]) << "\n\n";

   if (numCmdLineItems >2 ) {
	   std::string secondArg(wString2String(cmdLineItems[1]));
	   if (secondArg == "-load") {
		   std::string inputFile (wString2String(cmdLineItems[2]));
		   //std::string PCC_PCI(wString2String(cmdLineItems[3]));
		   mw->loadFile(inputFile);
	   }
   }

   loadConfig();
	mw->initConfigEditor();

	ModuleEvents::callPostInitEvents();

	// run it
	qapp.exec();

	ModuleEvents::callPreShutdownEvents();

	writeBackConfig();

	LocalFree(cmdLineItems);

	return 0;
}

// include icons & other resources
#include "geoxl35_rcc.h"
