//---------------------------------------------------------------------------
#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "X4ConsoleWindow.h"
//---------------------------------------------------------------------------
#include "ConsoleWidget.h"
#include "ModuleManagerWindow.h"
//---------------------------------------------------------------------------
#include "X4ConsoleWindow_moc.h"
//---------------------------------------------------------------------------
#include <QMenuBar>
#include "ExceptionHandling.h"
#include "InfoDialogsWidget.h"
#include "BasicStringUtils.h"
//---------------------------------------------------------------------------

using namespace X4;


X4ConsoleWindow::X4ConsoleWindow(QWidget * parent, Qt::WindowFlags f) : QMainWindow(parent, f)
{
	shutdown = false;

	ui.setupUi(this);

	setWindowTitle( "GeoXL 3.5.1 Console Window (separate process)" );

	//setAttribute(Qt::WA_DeleteOnClose);

	moduleManager = new ModuleManagerWindow(this);
	moduleManager->setVisible(false);
	moduleManager->loadStartupModules();
	autoCloseCheck = NULL;
}

X4ConsoleWindow::~X4ConsoleWindow() {}

void X4ConsoleWindow::on_exitButton_clicked(){
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_actionE_xit_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void X4ConsoleWindow::on_actionE_xit_triggered()
{
	close();
}

void X4ConsoleWindow::on_action_Info_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	InfoDialogsWidget *info = new InfoDialogsWidget;
	info->show();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void X4ConsoleWindow::on_action_Module_Manager_triggered()
{
	moduleManager->update();
	moduleManager->setVisible(true);
}

ConsoleWidget* X4ConsoleWindow::getConsoleWidget()
{
	return ui.consoleWidget;
}

void X4::X4ConsoleWindow::resizeWin( int w, int h )
{
	resize(w,h);
}

void X4::X4ConsoleWindow::moveWin( int x, int y )
{
	move(x,y);
}

void X4::X4ConsoleWindow::showWin()
{
	show();
}

void X4::X4ConsoleWindow::hideWin()
{
	hide();
}

void X4::X4ConsoleWindow::showHideWinToggle()
{
	if (this->isVisible()) hide(); else show();
}

void X4::X4ConsoleWindow::closeAll()
{
	close();
}

void X4::X4ConsoleWindow::flush()
{
	ui.consoleWidget->flush();
}

void X4::X4ConsoleWindow::newText( char *text )
{
	(*ui.consoleWidget) << text;
}

void X4::X4ConsoleWindow::newHTML( char *text )
{
	ui.consoleWidget->outputHTML(text);
}

void X4::X4ConsoleWindow::newBinary( char *buffer, int size )
{

}

static BOOL isProcessRunning(DWORD pid)
{
	HANDLE process = OpenProcess(SYNCHRONIZE, FALSE, pid);
	DWORD ret = WaitForSingleObject(process, 0);
	CloseHandle(process);
	return ret == WAIT_TIMEOUT;
}

void X4::X4ConsoleWindow::parentProcessIDReceived(card64 pid)
{
	parentProcessID = (DWORD)pid;
	autoCloseCheck = new QTimer(this);
	autoCloseCheck->setSingleShot(false);
	autoCloseCheck->setInterval(250);
	QObject::connect(autoCloseCheck, SIGNAL(timeout()), this, SLOT(checkParentProcess()));
	autoCloseCheck->start(250);
}

void X4::X4ConsoleWindow::checkParentProcess() {
	if (!isProcessRunning(parentProcessID)) {
		shutdown = true;
		close();
	}
}


void X4ConsoleWindow::closeEvent(QCloseEvent *event)
{
	if (!shutdown) {
		hide();
		event->ignore();
	}
}