//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "X4GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <QApplication>
//#include <QtGui/QMotifStyle>
//#include <QtGui/QCDEStyle>
//#include <QtGui/QWindowsStyle>
//#include <QtGui/QPlastiqueStyle>
//#include <QtGui/QCleanlooksStyle>
//#include <QtGui/QWindowsXPStyle>
#include <QtCore/QString>
#include <QtCore/QMetaType>
//---------------------------------------------------------------------------
#include <Psapi.h>
#include <Shellapi.h>
#include <direct.h>
#include "SIGNAL.H."
//---------------------------------------------------------------------------
#include <iostream>
//---------------------------------------------------------------------------
#include "InterfaceGlobals.h"
//---------------------------------------------------------------------------
#include "BinaryObjectStreams.h"
#include "Tools.h"
#include "ConsoleWidget.h"
#include "ExceptionHandling.h"
#include "StringHelper.h"
#include "DebugOutput.h"
#include "ApplicationDirectories.h"
#include "X4ConsoleWindow.h"
#include "ConsoleWindowProcess.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



//static void setStyle(card32 i) {
//   switch (i) {
//      //case 1: QApplication::setStyle(new QMotifStyle); break;
//      //case 2: QApplication::setStyle(new QCDEStyle); break;
//      case 3: QApplication::setStyle(new QWindowsStyle); break;
//      case 4: QApplication::setStyle(new QCleanlooksStyle); break;
//      case 5: QApplication::setStyle(new QPlastiqueStyle); break;
//      case 6: QApplication::setStyle(new QWindowsXPStyle); break;
//	   default: break;
//   }
//}


void X4_SIGABRT_Handler(int signal) {
	throw X4Exception(std::string("SIGABRT caught (signal #")+intToStr(signal)+std::string(")."));
}

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);
	UNREFERENCED_PARAMETER(nCmdShow);
	// signal handlers

	typedef void (*SignalHandlerPointer)(int);

   SignalHandlerPointer previousSIGABRTHandler;
   previousSIGABRTHandler = signal(SIGABRT, X4_SIGABRT_Handler);

   // Qt - Initialization
   //register default types by Qt, in order to have queued signal/slot support
   qRegisterMetaType<card32>("card32");
   qRegisterMetaType<std::string>("std::string");
   qRegisterMetaType<card64>("card64");
   qRegisterMetaType<time_t>("time_t");
    
	setStructureExceptionHandler();


	LPWSTR commandLine  = GetCommandLineW();
	int numCmdLineItems = 0;
	LPWSTR *cmdLineItems = CommandLineToArgvW(commandLine, &numCmdLineItems);

   int n = 0;
   QApplication qapp(n, 0);
	InterfaceGlobals::setApplication(&qapp);

	ApplicationDirectories::setup();

	// create main window	
	X4ConsoleWindow * console = new X4ConsoleWindow(NULL);
	console->hide();
	InterfaceGlobals::setMainWindow(console);
	qapp.setActiveWindow( console );

	// Close all windows with main window
	QApplication::connect(console, SIGNAL(destroyed()), &qapp, SLOT(closeAllWindows()));

	// create receiver thread
	ConsoleWindowProcessReceiver *rec = new ConsoleWindowProcessReceiver();
	
	rec->moveToThread(rec);

	QObject::connect(rec, SIGNAL(resizeWin(int,int)), console, SLOT(resizeWin(int,int)), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(moveWin(int,int)), console, SLOT(moveWin(int,int)), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(showWin()), console, SLOT(showWin()), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(hideWin()), console, SLOT(hideWin()), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(showHideWinToggle()), console, SLOT(showHideWinToggle()), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(closeAll()), console, SLOT(closeAll()), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(flush()), console, SLOT(flush()), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(newText(char*)), console, SLOT(newText(char*)), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(newHTML(char*)), console, SLOT(newHTML(char*)), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(newBinary(char*,int)), console, SLOT(newBinary(char*,int)), Qt::QueuedConnection);
	QObject::connect(rec, SIGNAL(parentProcessIDReceived(card64)), console, SLOT(parentProcessIDReceived(card64)), Qt::QueuedConnection);
	
	rec->start();

	// run it
	qapp.exec();

	delete rec;

	LocalFree(cmdLineItems);

	return 0;
}

// include icons & other resources
#include "x4console_rcc.h"
