//---------------------------------------------------------------------------
#ifndef ARDeformationOptimizationContextH
#define ARDeformationOptimizationContextH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "AROptimizationContext.h"
//---------------------------------------------------------------------------

namespace X4 {

class SGEDeformationSequence;
class ARSettings;
class DeformationSolver;
class DeformationEvaluator;


///   --- mw 11/27/2007
///
///   This specific AROptimizationContext deals with deformation optimization
///   this means, the deformation of a single shape ("urshape", "parametrization domain")
///   over a time interval is optimized according to some criteria.
///
class X3DEFORMATION_API ARDeformationOptimizationContext : public AROptimizationContext {
	X4_ABSTRACT_CLASS( ARDeformationOptimizationContext )
 public:
	/// initializes the shape optimization data structures from a candidate shape
	virtual void initializeFromShape(const SceneObject *obj, Vector2f timeInterval, const ARSettings *params) = 0;

	// we might want to visualize the context in the scene graph.
	virtual void visualize(Scene * scene) {}
	
	/// returns the time interval covered (i.e., [0...getDuration()])
	/// typically, this is the number of frames-1
	virtual float32 getDuration() = 0;

   /// quickhack
	virtual void writeNodesVisToSceneGraph(Scene *scene, std::string path, std::string subgraphName) {}
	
	/// create a compatible solver
	virtual DeformationSolver *createSolver() = 0;
	/// create a compatible evaluator
	virtual DeformationEvaluator *createEvaluator() = 0;
	
	/// returns the base resolution (e.g. base grid spacing) at which the discretization
	/// of the deformation function resolves the mapping
	virtual float32 getBaseResolution() = 0;
};

}

#endif