//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARSettings.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SeparatorClassProperty.h"
#include "ICPConstraints.h"
#include "MMSampler.h"
#include "ClassTypeClassProperty.h"
#include "MMGridSampler.h"
#include "MMTopologyGridSampler.h"
#include "LocalICP.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

static std::string getShapeSolverSolverDescr( const card32 index )
{
	if ( index == SHAPE_SOLVER_STEEPEST_DESCENT ) return "Steepest Descent";
	else if ( index == SHAPE_SOLVER_NEWTON ) return "Newton method";
	else if ( index == SHAPE_SOLVER_DIRECT ) return "Direct";
	else return "Undefined";
}

static std::string getShapeSolverConsistencyModeDescr( const card32 index )
{
	if ( index == CONSISTENCY_MODE_SINGLE_NEIGHBOR ) return "Single Neighbors";
	else if ( index == CONSISTENCY_MODE_INTEGRATION ) return "Integration";
	else if ( index == CONSISTENCY_MODE_NONE ) return "None";
	else if ( index == CONSISTENCY_MODE_NORMALS_PLANE ) return "Normals/Plane";
	else return "Undefined";
}

static std::string getICPConstraintsModeDescr(const card32 index) {
   if (index == ICP_CONSTRAINT_POINT_TO_PLANE) {
      return "ICP_CONSTRAINT_POINT_TO_PLANE";
   } else {
      return "ICP_CONSTRAINT_POINT_TO_POINT";
   }
}

static std::string descrMMSMethod(card32 index) {
	if (index == 0) return "MMS_BFGS";
	if (index == 1) return "MMS_NON_LINEAR_CG";
	if (index == ARSettings::MMS_LBFGS) return "MMS_LBFGS";
	return "undefined";
}

static std::string descrTSDeformationModel( const card32 index )
{
	if ( index == TS_MODEL_LAPLACE ) return "TS_MODEL_LAPLACE";
	else if ( index == TS_MODEL_THINPLATESPLINES ) return "TS_MODEL_THINPLATESPLINES";
	else if ( index == TS_MODEL_MIXED_MODE ) return "TS_MODEL_MIXED_MODE";
	else if ( index == TS_MODEL_THINPLATESPLINES_2D ) return "TS_MODEL_THINPLATESPLINES_2D";
	else return "Undefined";
}

static std::string describeTSSBasisFunctions( const card32 index )
{
	if ( index == TSS_COSINE ) return "TSS_COSINE";
	else if ( index == TSS_GAUSSIAN ) return "TSS_GAUSSIAN";
	else if ( index == TSS_POLYSPLINE ) return "TSS_POLYSPLINE";
	else return "Undefined";
}

static std::string describeAsapLinSolver( const card32 index )
{
	if ( index == ASAP_LIN_SOLVER_CG ) return "ASAP_LIN_SOLVER_CG";
	else if ( index == ASAP_LIN_SOLVER_GS ) return "ASAP_LIN_SOLVER_GS";
	else if ( index == ASAP_LIN_SOLVER_DPCG ) return "ASAP_LIN_SOLVER_DPCG";
	else if ( index == ASAP_LIN_SOLVER_CGS ) return "ASAP_LIN_SOLVER_CGS";
	else if ( index == ASAP_LIN_SOLVER_DPCGS ) return "ASAP_LIN_SOLVER_DPCGS";
	else if ( index == ASAP_LIN_SOLVER_DIRECT_INTEL ) return "ASAP_LIN_SOLVER_DIRECT_INTEL";
	else return "Undefined";
}


IMPLEMENT_X4_CLASS( ARSettings, 45 ) {
	BEGIN_CLASS_INIT( ARSettings );
   ADD_SEPARATOR( "Topology" )
   ADD_FLOAT32_PROP( topSurfelSpacing, 2 )
   ADD_FLOAT32_PROP( topResamplingSurfelSpacing, 10 )
   ADD_FLOAT32_PROP( topRelRadius, 9 )
   ADD_CARD32_PROP( topMaxNumNeighbors, 9 )
   ADD_FLOAT32_PROP( topMaxRelAddLengthChange, 11)
   SET_USER_EDIT(false)
   ADD_FLOAT32_PROP( topMaxMultLengthChange, 11 )
   SET_USER_EDIT(false)
   ADD_FLOAT32_PROP( topMinRelLengthRelCheck, 11 )
   SET_USER_EDIT(false)
   ADD_FLOAT32_PROP( topCutMaxStressFactor, 27 )
   ADD_FLOAT32_PROP( topCutMinRelativeLength, 27 )
   ADD_CARD32_PROP( topCutBorder, 27 )
   ADD_BOOLEAN_PROP( topCheckWholeAnimation, 28 )
   ADD_BOOLEAN_PROP( resampleInitialUrshapes, 15 )
   ADD_BOOLEAN_PROP( resampleMergedUrshapes, 15 )
	ADD_CARD32_PROP( outlier_minNumberOfPoints, 17 )
	ADD_BOOLEAN_PROP( outlier_removeSmallPatches, 17 )

	ADD_SEPARATOR( "Numerics" );
	ADD_CARD32_PROP(maxNumICPIterations, 8)
	ADD_CARD32_PROP(maxNumShapeOptIterations, 10)
   ADD_CARD32_PROP(numNewtonIterations, 0)
   ADD_CARD32_PROP(numCGIterations, 0)
   ADD_FLOAT32_PROP(convThreshICP, 8)
   ADD_FLOAT32_PROP(convThreshNewton, 0)
   ADD_FLOAT32_PROP(convThreshCG, 0)
   ADD_FLOAT32_PROP(newtonRegularization, 0)
	ADD_CARD32_PROP_UPDATE_DESCR(mmSolverMethod, 14, NULL, descrMMSMethod, 3, NULL); 

   ADD_SEPARATOR( "Reconstruction" )
   ADD_FLOAT32_PROP(pjDeprecated01, 2 )
   SET_USER_EDIT(false)
   ADD_FLOAT32_PROP(pjDeprecated02, 2 )
   SET_USER_EDIT(false)
   ADD_FLOAT32_PROP(pjDeprecated03, 2 )
   SET_USER_EDIT(false)

   ADD_SEPARATOR( "Shape Solver" )
   ADD_CARD32_PROP_UPDATE_DESCR( shapeSolveSolver, 3, NULL, getShapeSolverSolverDescr, 3, NULL )
   ADD_FLOAT32_PROP(weightSmoothNormal, 4)
   ADD_FLOAT32_PROP(weightSmoothPlane, 16)
   ADD_CARD32_PROP_UPDATE_DESCR( shapeSolveConsistencyMethod, 5, NULL, getShapeSolverConsistencyModeDescr, 4, NULL )
   ADD_FLOAT32_PROP(shapeSolveStepSize, 3)
   ADD_CARD32_PROP(shapeSolveMaxNumIterations, 6)
   ADD_FLOAT32_PROP(shapeSolveConvergence, 6)
   ADD_BOOLEAN_PROP(shapeSolveLocallyWeightedICP, 7)

   ADD_SEPARATOR( "MM Deformation Solver")
   ADD_FLOAT32_PROP(nodalRadius, 0)
   ADD_FLOAT32_PROP(nodalOffset, 0)
   ADD_FLOAT32_PROP(rigidityParam, 0)
   ADD_FLOAT32_PROP(correspondenceParam, 0)
   ADD_FLOAT32_PROP(volumeParam, 0)
   ADD_FLOAT32_PROP(accelerationParam, 0)
   ADD_FLOAT32_PROP(velocityParam, 0)
	ADD_FLOAT32_PROP(halfSpaceParam, 16)
   ADD_CARD32_PROP(gridSize, 0)
   ADD_FLOAT32_PROP(topReach, 25)
   ADD_CARD32_PROP(windowSize,0)
   ADD_BOOLEAN_PROP(d2aLinesearch, 0)
	ADD_METACLASS_PROP(mmNodesSampler, 13, MMSampler::getClass())
	ADD_BOOLEAN_PROP(nearestBaseIndexHack, 15)

	ADD_SEPARATOR( "FD Deformation Solver" )
   ADD_FLOAT32_PROP(fdDeformationWeight, 1)
   ADD_FLOAT32_PROP(fdDiscretization, 1)

	ADD_SEPARATOR( "TS Deformation Solver" )
   ADD_FLOAT32_PROP(tsGridSpacing, 33)
   ADD_FLOAT32_PROP(tsRegularizerWeight, 33)
   ADD_FLOAT32_PROP(tsMixedModeLPComp, 34)
   ADD_BOOLEAN_PROP(tsSlowTest, 35)
   ADD_BOOLEAN_PROP(tsCachedCellMatrices, 36)
   ADD_FLOAT32_PROP(tsIdentityWeight, 33)
   ADD_FLOAT32_PROP(tsConstraintsWeight, 33);
   ADD_CARD32_PROP_UPDATE_DESCR(tsDeformationModel, 33, NULL, descrTSDeformationModel, 3, NULL);
   ADD_BOOLEAN_PROP(tsMode2D, 38)
   

	ADD_SEPARATOR( "TS Smooth Deformation Solver" )
	 ADD_CARD32_PROP_UPDATE_DESCR(tssBasisFunction, 37, NULL, describeTSSBasisFunctions, 3, NULL);


	ADD_SEPARATOR( "ASAP Deformation Solver" )
	 ADD_BOOLEAN_PROP(asapCorotation, 39)
     ADD_BOOLEAN_PROP(asapUseSubSpaces, 41)
	 ADD_FLOAT32_PROP(asapSubSpaceCoordinatesPenalty, 43)
	 ADD_FLOAT32_PROP(asapSubSpaceAdapterCoordinatesPenalty, 44)
     ADD_BOOLEAN_PROP(asapPerPartCorrotation, 42)
     ADD_CARD32_PROP_UPDATE_DESCR(asapLinSolver, 40, NULL, describeAsapLinSolver, 6, NULL)
     ADD_FLOAT32_PROP(asapBaseShapeWeight, 40)
	 ADD_BOOLEAN_PROP(asapUseCotangentWeights, 45)

	ADD_SEPARATOR( "ASAP Def Solver L1 Norm Test" )
     ADD_BOOLEAN_PROP(asapL1Norm, 39)
     ADD_FLOAT32_PROP(asapL1NormSmoothing, 39)
   

   ADD_SEPARATOR( "ICP Constraints" )
   ADD_CARD32_PROP_UPDATE_DESCR(icpConstraintsMode, 10, NULL, getICPConstraintsModeDescr, 2, NULL)
   ADD_BOOLEAN_PROP(icpPercentileCulling, 10)
	ADD_FLOAT32_PROP(icpOutlierPercentile, 10)
	ADD_BOOLEAN_PROP(icpNormalCulling, 10)
	ADD_FLOAT32_PROP(icpNormalCosAngleCutOff, 10)
	ADD_BOOLEAN_PROP(icpNearFieldCulling, 10)
	ADD_FLOAT32_PROP(icpNearFieldRelativeCutOff, 10)
	ADD_FLOAT32_PROP(icpSkipPercentage, 18)
	ADD_BOOLEAN_PROP(icpUseDataErrorQuads, 22)
	ADD_FLOAT32_PROP(icpRigidityInitialFactor, 29)
	ADD_FLOAT32_PROP(icpRigidityMultiPassFactor, 23)
   ADD_CARD32_PROP(icpNumRPasses, 23)
   ADD_BOOLEAN_PROP(icpTwoWayMatch, 30)

	ADD_SEPARATOR( "Patch-ICP Constraints" )
	ADD_OBJECT_PROP(patchICP_localICP, 24,LocalICP::getClass(),true)
	ADD_BOOLEAN_PROP(patchICP_active, 24)
	ADD_FLOAT32_PROP(patchICP_weight, 24)
	ADD_FLOAT32_PROP(patchICP_maxDist, 24)

	ADD_SEPARATOR("residuals");
	ADD_FLOAT32_PROP(residualSupportSigma, 21)
	ADD_FLOAT32_PROP(residualUpperLimit, 24)
	ADD_FLOAT32_PROP(residualLowerLimit, 24)
	ADD_FLOAT32_PROP(residualRefittingRigidity, 24)
	
	ADD_SEPARATOR( "DepthImageHalfSpaceConstraint")
	ADD_BOOLEAN_PROP(useHalfSpaceConstraints, 26)
	ADD_FLOAT32_PROP(depthTolerance, 9)
	ADD_CARD32_PROP(gridResolution,  9)

	ADD_SEPARATOR( "Unsupported Points Culling")
	ADD_CARD32_PROP(cullStartNumFrames, 12)
	ADD_CARD32_PROP(cullBelowNumFrames, 12)
	ADD_FLOAT32_PROP(cullBelowPercentage, 12)

   ADD_SEPARATOR( "Final Output" )
   ADD_CARD32_PROP(outFramesPerFrame, 10)
   ADD_CARD32_PROP(outPointsPerPoint, 10)

   ADD_SEPARATOR( "Debugging" )
	ADD_BOOLEAN_PROP(mergeDebugOutput, 18)
	ADD_BOOLEAN_PROP(mmDebugOutput, 18)
	ADD_BOOLEAN_PROP( surfaceRecDebugOutput, 20)
	ADD_BOOLEAN_PROP(singleStepDebugVis, 19)
	ADD_CARD32_PROP(dataToUsNumIterations, 26)
	ADD_BOOLEAN_PROP(debugOutputStressTensors, 27)

	ADD_SEPARATOR( "Constraints OOC Settings" )
	// --- Constraints OOC settings
	ADD_BOOLEAN_PROP( useConstraintsOOC, 31);
	ADD_CARD32_PROP ( maxConstraintsInMem, 31);
	ADD_STRING_PROP ( constraintsTempFileDir, 31);
}

// ----

ARSettings::ARSettings() 
{
	initDefaultValues();
}

void ARSettings::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

void ARSettings::initDefaultValues() 
{
   // --- Topology computation parameters
	topSurfelSpacing = 0.075f;
   topResamplingSurfelSpacing = 0.05f;
   topRelRadius = 3.0f; 
   topMaxNumNeighbors = 12;
	topMaxRelAddLengthChange = 1.0e30f;
	topMaxMultLengthChange = 1.4f;
	topMinRelLengthRelCheck = 0.3f;
	topCutMaxStressFactor = 1.2f;
	topCutMinRelativeLength = 0.01f;
	topCutBorder = 0;
	topCheckWholeAnimation = false;
	resampleInitialUrshapes = false;				// reset, mw, 2007-01-17
	resampleMergedUrshapes = true;
	outlier_minNumberOfPoints = 100;			// set to 100 pj, 2007-01-14 (call after resampling)
	outlier_removeSmallPatches = true;			// set true pj, 2007-01-14

	// --- Numerics
	maxNumICPIterations = 25;
	maxNumShapeOptIterations = 5;
	numNewtonIterations = 1;
	numCGIterations = 10000;
	convThreshICP = 1e-4f;
	convThreshNewton = 1e-5f;
	convThreshCG = 1e-5f;
	newtonRegularization = 0.0f;
	mmSolverMethod = MMS_BFGS;
	
	// --- Reconstruction parameters
	pjDeprecated01 = -1;
	pjDeprecated02 = -1;
	pjDeprecated04 = -1;

	// --- Shape solver parameters
	weightSmoothNormal = 5.0f;
	weightSmoothPlane = 5.0f;
	shapeSolveConsistencyMethod = CONSISTENCY_MODE_NORMALS_PLANE;
	shapeSolveSolver = SHAPE_SOLVER_NEWTON;
	shapeSolveStepSize = 0.01f;
	shapeSolveMaxNumIterations = 300;
	shapeSolveConvergence = 1e-12f;
	shapeSolveLocallyWeightedICP = true;

	// --- MMSolver parameters
	nodalRadius = 0.5f;
	nodalOffset = 0.2f*nodalRadius;
		
	rigidityParam = 5.0f;
	correspondenceParam = 0.1f;
	volumeParam = 0.00001f;
	accelerationParam = 0;
	velocityParam = 0;
	halfSpaceParam = 1;
	windowSize = 1;
	gridSize = 2; 
	topReach = 2;                        
	mmNodesSampler = MMTopologyGridSampler::getClass();
	nearestBaseIndexHack = true;
	d2aLinesearch = false;

    // --- FDSolver parameters
	fdDeformationWeight = 1e-8f;
	fdDiscretization = 8;
	
   // --- TSSolver parameters
	tsGridSpacing = 0.2f;
	tsRegularizerWeight = 1.0f;
	tsMixedModeLPComp = 0.05f;
	tsIdentityWeight = 0.0f;
	tsConstraintsWeight = 100.0f;
	tsDeformationModel = TS_MODEL_THINPLATESPLINES;
	tssBasisFunction = TSS_POLYSPLINE ;
	//tssSupport = 2.0f;
	tsSlowTest = false;
	tsCachedCellMatrices = true;
	tsMode2D = false;

   // --- ASAPSolver parameters
   asapCorotation = false;
   asapL1Norm = false;
   asapL1NormSmoothing = 0.0001f;
   asapLinSolver = ASAP_LIN_SOLVER_CG;
   asapBaseShapeWeight = 0.0f;
   asapUseSubSpaces = true;
   asapSubSpaceCoordinatesPenalty = 0.1f;
   asapSubSpaceAdapterCoordinatesPenalty = 0.07f;
   asapPerPartCorrotation = true;
   asapUseCotangentWeights = false;
	
   // --- ICP constraints parameters
   icpConstraintsMode = ICP_CONSTRAINT_POINT_TO_PLANE;
   icpPercentileCulling = true;
	icpOutlierPercentile = 0.2f;
	icpNormalCulling = true;
	icpNearFieldRelativeCutOff = 3.0f;
	icpNearFieldCulling = true;
	icpNormalCosAngleCutOff = 0.7f;
	icpSkipPercentage = 0.0f; // off by default
   icpUseDataErrorQuads = false;
   icpRigidityInitialFactor = 1.0f; // off by default
   icpRigidityMultiPassFactor = 1.0f; // off by default
   icpNumRPasses = 1;
   icpTwoWayMatch = false;

	// --- Patch ICP constraints parameters
	patchICP_localICP = new LocalICP;
	patchICP_active = false;
	patchICP_weight = 1.0f;
	patchICP_maxDist = .2f;

	// --- Residual parameters
	residualSupportSigma = 1.5f;
	residualUpperLimit = 0.95f;
	residualLowerLimit = 0.75f;
	residualRefittingRigidity = 0.5f;

	// --- Unsupported Points Culling
	cullStartNumFrames = 100000;
	cullBelowNumFrames = 1;
	cullBelowPercentage  = 0.1f;
	
	// --- DepthImageHalfSpaceConstraint settings
	useHalfSpaceConstraints = false;
	depthTolerance = 1.0f;					
	gridResolution = 64;

   // --- Final output parameters
   outFramesPerFrame = 1;
   outPointsPerPoint = 1;
   
   // --- Debugging
   mergeDebugOutput = true;
	mmDebugOutput = false;
	surfaceRecDebugOutput = false;
	singleStepDebugVis = false;
	dataToUsNumIterations = 10;
	debugOutputStressTensors = false;

	// --- Constraints OOC settings
	useConstraintsOOC = true;
	maxConstraintsInMem = 10; 
	constraintsTempFileDir = "c:/temp/AR2tmpConstr_";

}

void ARSettings::setPresets(ARDefaultSettings::DataType presets) {
	initDefaultValues();
	// --- add code for different data sets
	switch (presets) {

      case ARDefaultSettings::SIMPLE_CUBE: {
		  //topSurfelSpacing = 0.1f;
		  //topResamplingSurfelSpacing = 0.1f;
		  topSurfelSpacing = 0.025f;
		  topResamplingSurfelSpacing = 0.025f;
		  topRelRadius = 3;
		  break;
      }

	  case ARDefaultSettings::XFACE: {
		  topSurfelSpacing = 0.002f;
		  topResamplingSurfelSpacing = 0.002f;
		  break;
	  }
	
	  case ARDefaultSettings::FLAG: {
		  topSurfelSpacing = 0.03f;
		  topResamplingSurfelSpacing = 0.03f;
		  break;
	  }

//	  case ARDefaultSettings::SPOK: {
//		  topSurfelSpacing = 0.02f;
//		  topResamplingSurfelSpacing = 0.02f;
//		  break;
//	  }

	  case ARDefaultSettings::PCA_TEST: {
		  topSurfelSpacing = 0.02f;
		  topResamplingSurfelSpacing = 0.02f;
		  break;
	 }


		case ARDefaultSettings::DRESDEN:
		case ARDefaultSettings::DRESDEN_RIGID: {
		   topSurfelSpacing = 0.0125f;
		   topResamplingSurfelSpacing = 0.0125f;
			topMaxNumNeighbors = 12;
		   break;
		}
		case ARDefaultSettings::MSR_DANCERS: {
         notImplemented();
			break;
		}
	}
	switch (presets) {
		case ARDefaultSettings::DRESDEN: {
		   gridSize = 12;
		   rigidityParam = 30.0f;
		   break;
	   }
		case ARDefaultSettings::DRESDEN_RIGID: {
		   gridSize = 2;
		   rigidityParam = 1000.0f;
		   break;
		}
	}
}


ARSettings::~ARSettings() {
	delete patchICP_localICP;
}



