//---------------------------------------------------------------------------
#ifndef DeformationToolsH
#define DeformationToolsH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "RegularData.h"
#include <set>
//---------------------------------------------------------------------------


// DeformationTools - mw 10/11/2007
//
// misc. routines and helpers for the deformation project


namespace X4 {

class InCorePCTopologyGraph;
class ARSequence;
class ARSettings;


bool X3DEFORMATION_API compareDescr( 
   const VertexDescriptor *other, 
   const VertexDescriptor *mustHave
);

X3DEFORMATION_API std::string endWithSlash(const std::string &str);
X3DEFORMATION_API std::string endWithBackSlash(const std::string &str);
   
/// fetches an attached octree (creates it if not present)
/// works only for UnstructuredInCorePointClouds and 
/// GenericSwapAllOutOfCorePointCloud that contain an UnstructuredInCorePointCloud
X3DEFORMATION_API AttachedIndexedOctree *fetchAndAttachOctree(PointCloud *pc, card32 createNumPtsPerLeaf = 20);
   
/// converts the position channel of the point cloud to a dynamic std::vector
X3DEFORMATION_API DVectorF convertPCToVector(PointCloud *pc);
   
/// fetch attached topology
X3DEFORMATION_API InCorePCTopologyGraph *getTopology(PointCloud *pc);   
   
/// put / replace attached topology
X3DEFORMATION_API void attachTopology(PointCloud *pc, InCorePCTopologyGraph *tg, bool visible = true);


/// replaces attached topology of target with that attached to source
/// if source has no topology, target will end up with no topology as well
X3DEFORMATION_API void assignTopology(PointCloud *source, PointCloud *target);   
   
X3DEFORMATION_API Matrix4d svdPseudoInvert4d(Matrix4d m, const float64 zeroTolerance = 1e-5);
X3DEFORMATION_API Matrix3d svdPseudoInvert3d(Matrix3d m, const float64 zeroTolerance = 1e-5);
   
X3DEFORMATION_API void prunePoints(
   UnstructuredInCorePointCloud *inPC, InCorePCTopologyGraph *inTG,
   UnstructuredInCorePointCloud *&outPC, InCorePCTopologyGraph *&outTG,
   const std::vector<bool> &toKeep
);

X3DEFORMATION_API void prunePoints(
   PointCloud *pc, InCorePCTopologyGraph *tg,
   const std::vector<bool> &toKeep
);

X3DEFORMATION_API void prunePointsAndReplace(ARSequence *seq, const std::vector<bool> &toKeep);

/// merges the two point clouds and their topology, including new edges between pc1 and pc2 
X3DEFORMATION_API PointCloud *mergePointClouds(PointCloud *pc1, PointCloud *pc2, ARSettings *settings);

X3DEFORMATION_API void removePointsAtEnd(PointCloud *pc, card32 endPtsStartIndex);

/// recomputes and unifies normals and reference frames
/// assumes topology attachment
X3DEFORMATION_API void recomputeNormalsAndFrames(PointCloud *pc, ARSettings *settings);

/// Helper for recomputeNormalsAndFrames: Estimate normals from topological neighborhood via PCA
X3DEFORMATION_API void recomputeFrames( PointCloud *pc, ARSettings *settings );

/// Helper for recomputeNormalsAndFrames: unify normal directions via region growing
X3DEFORMATION_API void recomputeUnifyNormals( PointCloud *pc, ARSettings *settings );

/// Helper for recomputeNormalsAndFrames: unify normal directions via Hoppe's minimal spanning tree
X3DEFORMATION_API void recomputeUnifyNormalsHoppe( PointCloud *pc, ARSettings *settings );

/// Helper for recomputeNormalsAndFrames: Find all topological neighbors of point 
/// 'pIndex' (directly connected or within 'influenceRadius')
X3DEFORMATION_API std::set<card32> getNeighborIndices(
	const card32 index,
	PointSet* ps,
	InCorePCTopologyGraph* topology,
	const float32 influenceRadius = -1 );	// if < 0, only directly connected neighbors are used

X3DEFORMATION_API PointSet *copySegment(PointSet *ps, mpcard startIndex, mpcard endIndex);

   
}

#endif