#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PhotometricContraints.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "CameraParameterAttachment.h"
#include "UnstructuredInCorePointCloud.h"
#include "CVisionTools.h"
#include "ParametricInCorePointCloud.h"
#include "DebugRenderer.h"
#include "RGBAImage.h"
#include "ImageDisplayer.h"
#include "SuperIterator_templCode.h"
#include "ProgressWindow.h"
#include <QtGui/QImage>
#include "ARConstraintsInterface.h"
#include "ARConstraints.h"
#include "StringHelper.h"
#include <QtGui/QImage>
#include <QtGui/QPainter>
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PhotometricContraints ,0)
{
	BEGIN_CLASS_INIT( PhotometricContraints );

	ADD_FLOAT32_PROP( m_MinCorrelationThreshold, 0);
	ADD_CARD32_PROP(m_WindowSize,0);
	ADD_FLOAT32_PROP(m_MinEdgeThreshold,0);
	ADD_FLOAT32_PROP(m_MinQuadCoeff,0);
}

PhotometricContraints::PhotometricContraints(void)
{
	m_MinCorrelationThreshold = 0.4f;
	m_WindowSize = 5;
	m_MinEdgeThreshold = 0.025f;
	m_MinQuadCoeff = 0.1f;
}

PhotometricContraints::~PhotometricContraints(void)
{
}

void PhotometricContraints::assign(const Object* obj, X4CopyContext *context)
{
	// Persistent::assign(obj,context);
	const PhotometricContraints * o = dynamic_cast<const PhotometricContraints*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::PhotometricContraints::buildPhotometricConstraints( const ARSettings *settings, ARConstraintsInterface *receiver, 
																				  PointCloud *referenceShape, PointCloud *deformedReference, 
																				  std::vector<ParametricInCorePointCloud*> images, 
																				  std::vector<CameraParameterAttachment*> cameraAttachments, bool visualize /*= false */)
{
	// preprocessing (should be cached...)
	// find points with high amount of horizontal edges
	// also reject points with a depth discontinuity
	std::vector<OneByteImage*> mask;
	progressWindow->pushStep(true,"preprocess");
	for( unsigned i=0;i<images.size();i++ )
	{
		mask.push_back( preprocessImage(images[i]));
		progressWindow->progress(i*100/(card32)images.size());
	}
	progressWindow->popSteps();

	
	bool debugOutputPointCloud = true;
	UnstructuredInCorePointCloud * constr_pc = NULL;
	if( debugOutputPointCloud )
	{
		constr_pc = new UnstructuredInCorePointCloud;
		VertexDescriptor * vd = new VertexDescriptor;
		vd->pushAttrib(VAD("position",3,VAD::DATA_FORMAT_FLOAT32));
		vd->pushAttrib(VAD("color",3,VAD::DATA_FORMAT_FLOAT32));
		vd->pushAttrib(VAD("flags",1,VAD::DATA_FORMAT_INT32));
		constr_pc->clearAndSetup(vd, 0);
	}
	
	
	// compute contraints
	SuperIterator it(deformedReference->createIterator(0), deformedReference->getDescr());
	//SuperIterator it_reference(referenceShape->createIterator(0), referenceShape->getDescr());
	
	AAT posAAT = deformedReference->getAAT("position");
	//AAT refPosAAT = referenceShape->getAAT("position");
	
	debugRenderer->beginRenderJob("constraint-test2");
	int numConstraintComputations = 0;
	progressWindow->pushStep(true,"contraints");
	int index = 0;
	while( !it.atEnd() )
	{
		Vector3f pos = it.get<Vector3f>(posAAT);
		//Vector3f anchor = it_reference.get<Vector3f>(refPosAAT);

		//pos = testPoint;

		// test masks for good constraint candidates
		int numViews = 0;
		std::vector<bool> validView(mask.size(),false);
		for( unsigned i=0;i<mask.size();i++ )
		{
			Vector2f projPos = cameraAttachments[i]->projectWorldtoImage(pos);
			if( projPos[0] < m_WindowSize || 
				projPos[1] < m_WindowSize ||
				projPos[0] > mask[i]->getSizeX() - m_WindowSize ||
				projPos[1] > mask[i]->getSizeY() - m_WindowSize
				)
				continue;
			card8 test = mask[i]->getPixel( projPos[0], projPos[1] );
			if( test > 0)
			{
				numViews++;
				validView[i] = true;
			}
		}
		
		int centerView = 4;

		// compute contraint
		if( numViews > 2 && validView[centerView])
		//if( rand() % 100 == 0)
		{
			numConstraintComputations++;

			Matrix3f m = IDENTITY3F * 0;
			Vector3f b = NULL_VECTOR3F;
			//for( unsigned i=0;i<images.size();i++ )
			{
				int i = centerView;
				if( !mask[i])
					continue;

				for( int j=0;j<(int)images.size();j++ )
				{
					Vector3f retpos;
					Matrix3f retmat;
					if( abs(i-j) < 1 )
						continue;

					if( !validView[j] )
						continue;

					float disp = computeDisparityConstraint( pos, images[i], cameraAttachments[i],
						images[j],cameraAttachments[j], retpos, retmat);

					if( disp > m_MinCorrelationThreshold )
					{
						Vector3f tmpB = retmat * retpos * (-2.0f);
						b += tmpB;
						m += retmat;
					}
				}
			}

			if( norm(b) > 0.0f )
			{
				//debugRenderer->addPositionConstraint( pos, center, bestM);
				Vector3f center;
				if( linsolve3f(m*-2.0f,b,center) )
				{
					//debugRenderer->addPoint(center, makeVector3f(1,0.5f,0));
					if( debugOutputPointCloud )
					{
						std::vector<char> buffer(constr_pc->getDescr()->getSize(),0);
						Vector3f* posPtr = (Vector3f*)(&buffer[0]);
						*posPtr = center;
						Vector3f* colorPtr = (Vector3f*)(&buffer[4*3]);

						Vector2f projPoint1 = cameraAttachments[centerView]->projectWorldtoImage(center);
						Vector3f cam_pos0 = cameraAttachments[centerView]->projectImageToWorld( makeVector3f(projPoint1[0],projPoint1[1],0));
						Vector3f d = normalize(cam_pos0 - center);

						float coeff = norm(m*d);
	

						*colorPtr = makeVector3f(coeff,0,0);
						constr_pc->getPointSet()->addPoint( &buffer[0]);
					}

					PositionARConstraint pconst;
					//pconst.setup(anchor, center, m);
					//receiver->addPositionConstraint( )
				}
				else
				{
					debugOutput << "Couldn't solve!\n"
						<< m << "\nb = " << b << "\n";
					float32 det = m[2][2]*m[0][0]*m[1][1] + m[0][2]*m[2][1]*m[1][0] - m[0][2]*m[2][0]*m[1][1]
					-m[0][0]*m[1][2]*m[2][1] + m[0][1]*m[1][2]*m[2][0] - m[2][2]*m[0][1]*m[1][0];
					debugOutput << "det(m) = " << det << "\n";
				}

			}

		}

		progressWindow->progress(index*100/deformedReference->getNumPoints());
		it.next();
		//it_reference.next();
		index++;
	}
	progressWindow->popSteps();

	debugRenderer->endRenderJob();

	debugOutput << "num candidates: " << numConstraintComputations
		<< "\npercentage: " << 100.0f * numConstraintComputations / deformedReference->getNumPoints() << "%\n";

	if( debugOutputPointCloud )
		addPointCloud(debugScene,constr_pc, "photometric");

	// delete masks
	for( unsigned i=0;i<mask.size();i++)
		delete mask[i];

}

float X4::PhotometricContraints::computeDisparityConstraint( const Vector3f &point, ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1, 
																		ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2, 
																		Vector3f &constraint_pos, Matrix3f &contraint_matrix, float confidenceMode )
{
	// compute epipolarline
	Vector2f projPoint1 = camAtt1->projectWorldtoImage(point);
	Vector3f cam_pos0 = camAtt1->projectImageToWorld( makeVector3f(projPoint1[0],projPoint1[1],0));

	Vector2f projPoint2 = camAtt2->projectWorldtoImage(point);
	Vector2f epipol = camAtt2->projectWorldtoImage((cam_pos0 + point) * 0.5f);
	Vector3f cam_pos1 = camAtt2->projectImageToWorld( makeVector3f(projPoint2[0],projPoint2[1],0));

	Vector3f pointToCam0 = normalize(cam_pos0 - point);

	unsigned numSamples = 10;
	float pixelRange = 5.0f;
	Vector2f normedEpipolDirection = normalize( projPoint2 - epipol );
	std::vector<Vector3f> result_vec;
	std::vector<float> result_corr;
	size_t maxCorrIndex = 0;
	constraint_pos = NULL_VECTOR3F;
	contraint_matrix *= 0.0f;

	std::vector<Vector2f> data(numSamples);	// keep data for fit
	for( unsigned i=0;i<numSamples;i++ )
	{
		float t = ((float)i/(numSamples-1.0f) - 0.5f)*2.0f*pixelRange;
		Vector2f testPoint = projPoint2 + normedEpipolDirection * t;
		//debugOutput << "testpoint: " << testPoint << "\n";

		Vector3f reprojectedTestPoint = camAtt2->projectImageToWorld( makeVector3f(testPoint[0],testPoint[1], point[2]));

		// compute "intersection" between cam1-ray and cam2-ray
		// defines a plane parallel to cam2-ray and computes the intersection between cam1-ray and the plane
		Vector3f tmpVec = (cam_pos0 - point).crossProduct(reprojectedTestPoint - cam_pos1);
		Plane3f p( normalize( tmpVec.crossProduct(reprojectedTestPoint - cam_pos1)), reprojectedTestPoint);
		Line3f l(cam_pos0 - point,point);

		Vector3f finalTestPoint = p.calculateIntersection( l );

		float correlation = computeDisparity( finalTestPoint, p1, camAtt1, p2, camAtt2 );
		result_vec.push_back(finalTestPoint);
		result_corr.push_back(correlation);

		data[i] = makeVector2f( pointToCam0 * (finalTestPoint - point), 0.5f - correlation*0.5f );

		if( result_corr.back() > result_corr[maxCorrIndex] )
		{
			if( correlation > m_MinCorrelationThreshold)
				constraint_pos = finalTestPoint;
			maxCorrIndex = result_corr.size()-1;
		}

		//	debugOutput << "correlation: " << correlation << "\n";
	}

	// scale interval
	float scale = max( fabs(data[0][0]),fabs(data.back()[0]));
	for( unsigned i=0;i<data.size();i++ )
	{
		data[i][0] /= scale;
	}

	if( ! confidenceMode )
	{
		Vector3f coeffs;
		computeQuadraticFit(data,coeffs);

		if( coeffs[2] < m_MinQuadCoeff ) // bad fit
			return -1.0f;

		// compute minimum
		float minPos = -coeffs[1] / coeffs[2] / 2.0f;

		// reject constraint if minimum is out of bounds
		float lowerBound = min(data[0][0],data.back()[0]);
		float upperBound = max(data[0][0],data.back()[0]);
		if( lowerBound > minPos || upperBound < minPos )
			return -1.0f;

		constraint_pos = point + pointToCam0 * minPos * scale;
		contraint_matrix = outerProduct(pointToCam0,pointToCam0) * coeffs[2] * scale;

		Matrix3f contraint_frame = calcTangentSystem(pointToCam0);
		contraint_matrix += outerProduct(contraint_frame[0],contraint_frame[0]) * scale;
		contraint_matrix += outerProduct(contraint_frame[1],contraint_frame[1]) * scale;
	}
	else
	{
		Vector2f coeffs;
		computeQuadraticFitSpecial(data,coeffs);
		//computeQuadraticFit(data,coeffs);

		//debugOutput << "coeffs: " << coeffs << "\n";

		/*Function1DImageGenerator fgen;

		DVectorF dv_coeff(3);
		dv_coeff[0] = coeffs[0];
		dv_coeff[1] = 0;
		dv_coeff[2] = coeffs[1];
		fgen.addPolynomialFunction(dv_coeff, makeVector3f(1,0,0));
		fgen.addDataPointFunction( data, makeVector3f(0,1,0), false);
		dv_coeff[0] = 0;
		dv_coeff[1] = 1.0f;
		fgen.addPolynomialFunction(dv_coeff, makeVector3f(0,0,1));

		BoundingBox2f vp;
		vp.lowerCorner = makeVector2f(-1.0f,-1.0f);
		vp.upperCorner = makeVector2f( 1.0f, 1.0f);
		RGBAFloat32Image * fimg = fgen.createImage( vp, 200, 50);
		ImageDisplayer::DisplayRGBAFloat32Image ev(fimg);
		debugOutput.displayEvent(ImageDisplayer::getClass(), std::string("fit1d"), &ev);
		delete fimg;*/

		if( coeffs[1] < m_MinQuadCoeff ) // bad fit
			return -1.0f;

		constraint_pos = point;
		contraint_matrix = outerProduct(pointToCam0,pointToCam0) * coeffs[1] * scale;
	}
	
	
	return result_corr[maxCorrIndex];

}

float X4::PhotometricContraints::computeDisparity( const Vector3f &point, ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1, ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2 )
{
	Vector2f projPoint1 = camAtt1->projectWorldtoImage( point );
	Vector2f projPoint2 = camAtt2->projectWorldtoImage( point );
	Vector2i point1i = makeVector2i(projPoint1[0],projPoint1[1]);
	const int windowSize = m_WindowSize;

	// compute projection
	Vector2f quad1,quad2,quad3,quad4;
	float depth = point[2];
	quad1 = camAtt2->projectWorldtoImage(
		camAtt1->projectImageToWorld( makeVector3f(point1i[0] - windowSize,point1i[1] - windowSize, depth)));
	quad2 = camAtt2->projectWorldtoImage(
		camAtt1->projectImageToWorld( makeVector3f(point1i[0] + windowSize,point1i[1] - windowSize, depth)));
	quad3 = camAtt2->projectWorldtoImage(
		camAtt1->projectImageToWorld( makeVector3f(point1i[0] + windowSize,point1i[1] + windowSize, depth)));
	quad4 = camAtt2->projectWorldtoImage(
		camAtt1->projectImageToWorld( makeVector3f(point1i[0] - windowSize,point1i[1] + windowSize, depth)));

	if( projPoint1[0] < 2.0f * m_WindowSize  + 1.0f ||
		projPoint1[1] < 2.0f * m_WindowSize  + 1.0f ||
		projPoint1[0] > p1->getPointSet()->getWidth()- 2.0f * m_WindowSize  + 1.0f ||
		projPoint1[1] > p2->getPointSet()->getHeight()- 2.0f * m_WindowSize  + 1.0f
		)
		return 0.0f;
	if( projPoint2[0] < 2.0f * m_WindowSize  + 1.0f ||
		projPoint2[1] < 2.0f * m_WindowSize  + 1.0f ||
		projPoint2[0] > p1->getPointSet()->getWidth()- 2.0f * m_WindowSize  + 1.0f ||
		projPoint2[1] > p2->getPointSet()->getHeight()- 2.0f * m_WindowSize  + 1.0f
		)
		return 0.0f;

	float dist = compareImagePatches(p1, point1i, windowSize,p2,quad1,
		quad2,quad3,quad4, CI_NCC );


	return dist;
}

void X4::PhotometricContraints::computeConfidence( PointCloud * data, std::vector<ParametricInCorePointCloud*> images, std::vector<CameraParameterAttachment*> cameraAttachments )
{
	checkAttribute(data, "errorQuad",6,VAD::DATA_FORMAT_FLOAT32);
	SuperIterator it(data->createIterator(0), data->getDescr());
	AAT posAAT = data->getAAT("position");
	AAT resultAAT = data->getAAT("errorQuad");

	progressWindow->pushStep(true,"conf");
	unsigned index=0;
	while( !it.atEnd() )
	{
		if( rand() % 100 == 0)
		{
			Vector3f pos = it.get<Vector3f>(posAAT);

			Matrix3f mat = IDENTITY3F * 0;
			int centerView = 4;
			for( unsigned i=0;i<images.size();i++ )
			{
				if( i == centerView) 
					continue;
				Vector3f cpos;
				Matrix3f cmat;
				float disp = computeDisparityConstraint(pos, images[centerView], cameraAttachments[centerView], images[i],cameraAttachments[i],cpos,cmat,true);
				if( disp > 0.0f )
					mat += cmat;
			}

			Vector6f errQuad = makeVector6f(mat[0][0],mat[1][1],mat[2][2],mat[1][0],mat[2][1],mat[2][2]);
			it.set<Vector6f>(resultAAT, errQuad);
			//break;
		}
		index++;
		progressWindow->progress(index*100/data->getNumPoints());
		it.flush();
		it.next();
	}
	progressWindow->popSteps();

}

void X4::PhotometricContraints::computeQuadraticFitSpecial( const std::vector<Vector2f> &data, Vector2f &coeffs )
{
	const size_t NUM_POINTS = data.size();
	Vector2f cd;
	Matrix2f dm = IDENTITY2F * 0;
	coeffs *= 0;

	for (card32 pIndex=0; pIndex<NUM_POINTS; pIndex++){
		const float32 u = data[pIndex][0];
		const float32 n = data[pIndex][1];
		const float32 w = 1.0f / (float32)NUM_POINTS;

		// c[j] += b[j](uv(i)) * n(i) * w(uv(i));
		cd[0] =       w;
		cd[1] = u*u * w;

		coeffs[0] += cd[0] * n;
		coeffs[1] += cd[1] * n;

		// 1 u uu

		dm[0][0] += w;     dm[0][1] += u*u*w;     
		dm[1][0] += w*u*u; dm[1][1] += u*u*w*u*u; 
	}

	Matrix2f invDm;
	bool success;
	invDm = invertMatrix(dm, &success, 1e-7f);
	if (success) {
		coeffs = invDm * coeffs;
	} else {
		coeffs[0] = 0; coeffs[1] = 0;
	}

}

void PhotometricContraints::computeQuadraticFit( const std::vector<Vector2f> &data, Vector3f &coeffs ) 
{
	const size_t NUM_POINTS = data.size();
	coeffs = NULL_VECTOR3F;
	Vector3f cd;
	Matrix3f dm = IDENTITY3F * 0;

	for (card32 pIndex=0; pIndex<NUM_POINTS; pIndex++){
		const float32 u = data[pIndex][0];
		const float32 n = data[pIndex][1];
		const float32 w = 1.0f / (float32)NUM_POINTS;

		// c[j] += b[j](uv(i)) * n(i) * w(uv(i));
		cd[0] =       w;
		cd[1] = u   * w;
		cd[2] = u*u * w;

		coeffs[0] += cd[0] * n;
		coeffs[1] += cd[1] * n;
		coeffs[2] += cd[2] * n;

		// 1 u uu

		dm[0][0] += w;     dm[0][1] += u*w;     dm[0][2] += u*u*w;     
		dm[1][0] += w*u;   dm[1][1] += u*w*u;   dm[1][2] += u*u*w*u;   
		dm[2][0] += w*u*u; dm[2][1] += u*w*u*u; dm[2][2] += u*u*w*u*u; 
	}

	Matrix3f invDm;
	bool success;
	invDm = invertMatrix(dm, &success, 1e-7f);
	if (success) {
		coeffs = invDm * coeffs;
	} else {
		coeffs[0] = 0; coeffs[1] = 0; coeffs[2] = 0;
	}

}

OneByteImage *PhotometricContraints::preprocessImage( ParametricInCorePointCloud * image ) 
{
	PointSet * ps = image->getPointSet();
	OneByteImage * retImage = new OneByteImage(ps->getWidth(),ps->getHeight());
	retImage->clear(0);

	// copy both channels into 2 images
	AAT colorAAT = ps->getAAT("color");
	AAT depthAAT = ps->getAAT("depth");
	RGBAFloat32Image colorImage(ps->getWidth(),ps->getHeight());
	RGBAFloat32Image depthImage(ps->getWidth(),ps->getHeight());
	for( unsigned x=0;x<ps->getWidth();x++ )
	{
		for( unsigned y=0;y<ps->getHeight();y++ )
		{
			Vector3f color = ps->get3f(x,y,colorAAT);
			float depth = ps->get1f(x,y,depthAAT);
			colorImage.setPixel(x,y,expand3To4(color));
			depthImage.setPixel(x,y,makeVector4f(1,1,1,1) * depth);
		}
	}

	// detect horizontal edges in color image
	Matrix3f vsobel;
	vsobel[0][0] = -3.0f;		vsobel[0][1] = 0.0f;		vsobel[0][2] = 3.0f;
	vsobel[1][0] = -10.0f;		vsobel[1][1] = 0.0f;		vsobel[1][2] = 10.0f;
	vsobel[2][0] = -3.0f;		vsobel[2][1] = 0.0f;		vsobel[2][2] = 3.0f;
	vsobel /= 26.0f;
	Matrix3f hsobel = vsobel.transpose();

	Timer timer;
	colorImage.filterMask(3,3,vsobel.data(),false);

	// detect edges in depth image
	RGBAFloat32Image *vertEdges = (RGBAFloat32Image*)depthImage.copy();
	vsobel /= 64.0f;
	hsobel /= 64.0f;
	vertEdges->filterMask(3,3,vsobel.data(),false);
	depthImage.filterMask(3,3,hsobel.data(),false);

	debugOutput << "filter timing1: " << convertTimeToString( timer.getDeltaValue()) << "\n";

	// combine edge images
	RGBAFloat32Image * dOutImage = NULL;
	QImage * storeImage = NULL;
	bool showOutputImage = false;
	bool storeOutputImage = false;
	if( showOutputImage )
	{
		dOutImage = new RGBAFloat32Image(ps->getWidth(),ps->getHeight());
		dOutImage->clear( makeVector4f(0,0,0,0) );
		if( storeOutputImage )
		{
			storeImage = new QImage(ps->getWidth(),ps->getHeight(),QImage::Format_RGB32);
			storeImage->fill(0);
		}
	}

	// prepare color + depth edges
	for( int x=0;x<(int)ps->getWidth();x++ )
	{
		for( int y=0;y<(int)ps->getHeight();y++ )
		{
			Vector4f col = colorImage.getPixel(x,y);
			int colEdge = (max(fabs(col[0]),
				max(fabs(col[1]),fabs(col[2]))) > m_MinEdgeThreshold) ? 1 : 0;
			float depthEdge1 = depthImage.getPixel(x,y)[0];
			float depthEdge2 = vertEdges->getPixel(x,y)[0];
			int depthEdge = sqrt( pow(depthEdge1,2.0f) + pow(depthEdge2,2.0f)) > m_MinEdgeThreshold ? 1:0;
			colorImage.setPixel(x,y,makeVector4f(colEdge,depthEdge,0,0));
		}
	}

	// count color & depth edges in y direction
	int windowSize = max((int)m_WindowSize - 2,2);
	RGBAFloat32Image temporalResult(ps->getWidth(),ps->getHeight());
	for( int x=windowSize;x<(int)ps->getWidth() - windowSize;x++ )
	{
		for( int y=windowSize;y<(int)ps->getHeight() - windowSize;y++ )
		{
			int numEdgePixels = 0;
			int numDepthDisc = 0;
			for( int my=-windowSize;my<=windowSize;my++ )
			{
				Vector4f tmp = colorImage.getPixel(x,y+my); // both results are now in colorImage
				// color edges
				if( abs(my)<windowSize )
				{
					numEdgePixels += tmp[0];
				}

				numDepthDisc += tmp[1];
			}

			temporalResult.setPixel(x,y,makeVector4f(numEdgePixels,numDepthDisc,0,0));

		}
	}
	
	debugOutput << "filter timing2: " << convertTimeToString( timer.getDeltaValue()) << "\n";

	for( int x=windowSize;x<(int)ps->getWidth() - windowSize;x++ )
	{
		for( int y=windowSize;y<(int)ps->getHeight() - windowSize;y++ )
		{
			int numEdgePixels = 0;
			int numDepthDisc = 0;
			for( int mx=-windowSize;mx<=windowSize;mx++ )
			{
				Vector4f tmp = temporalResult.getPixel(x+mx,y);
				if( abs(mx)<windowSize )
					numEdgePixels += tmp[0];
				numDepthDisc += tmp[1];
			}

			int mask = (numEdgePixels > 6) && (numDepthDisc < 2) ? 255 : 0;
			//int mask = (numEdgePixels > 5)? 255 : 0;
			retImage->setPixel(x,y,(card8)mask);

			Vector4f col = colorImage.getPixel(x,y);
			int edge = (max(fabs(col[0]),
				max(fabs(col[1]),fabs(col[2]))) > m_MinEdgeThreshold) ? 1 : 0;
			float depthEdge1 = fabs( depthImage.getPixel(x,y)[0] );
			float depthEdge2 = fabs( vertEdges->getPixel(x,y)[0] );
			int depthDisc = sqrt( pow(depthEdge1,2.0f) + pow(depthEdge2,2.0f)) > m_MinEdgeThreshold ? 1:0;

			if( showOutputImage )
			{
				dOutImage->setPixel(x,y,makeVector4f(mask / 255.0f, edge, depthDisc, 0));
				storeImage->setPixel(x,y,qRgb(mask,mask,mask));

			}

			
		}

	}

	debugOutput << "filter timing3: " << convertTimeToString( timer.getDeltaValue()) << "\n";

	if( showOutputImage )
	{
		ImageDisplayer::DisplayRGBAFloat32Image ev(dOutImage);
		static int c;
		c++;
		debugOutput.displayEvent(ImageDisplayer::getClass(), std::string("preprocessed images") + (char)('0' + (c % 10)), &ev);
	}
	if( storeImage)
	{
		static int c;
		c++;
		storeImage->save( QString("c:/precrossed") + QString::number(c) + ".png", "PNG");
	}
	delete vertEdges;
	delete dOutImage;
	delete storeImage;

	return retImage;
}

void X4::Function1DImageGenerator::addPolynomialFunction( const DVectorF &coeffs, const Vector3f &col )
{
	TPolynomialFunction f;
	f.coeff = coeffs;
	f.color = col;
	m_PolynomialFunction.push_back(f);
}

void X4::Function1DImageGenerator::addDataPointFunction( const std::vector<Vector2f> &dataPoints, const Vector3f &col, bool drawLines /*= true */ )
{
	TDataPointFunction f;
	f.dataPoints = dataPoints;
	f.color = col;
	f.drawLines = drawLines;
	m_DataPointFunction.push_back(f);
}



RGBAFloat32Image * X4::Function1DImageGenerator::createImage( const BoundingBox2f &viewport, int imageWidth, int imageHeight )
{
	// draw into a qimage	
	QImage destImage(imageWidth,imageHeight,QImage::Format_RGB32);
	destImage.fill(0);
	QPainter painter;
	painter.begin(&destImage);

	// draw polynomial function
	for( unsigned i=0;i<m_PolynomialFunction.size();i++ )
	{
		TPolynomialFunction * f = &m_PolynomialFunction[i];
		QPoint lastPoint;
		painter.setPen(QColor(f->color[0]*255.0f,f->color[1]*255.0f,f->color[2]*255.0f));
		for( int x=0;x<imageWidth;x++ )
		{
			// x in viewport space
			float xf = (float)x/(float)imageWidth*viewport.getSideLength(0) + viewport.lowerCorner[0];

			// evaluate
			float yf = 0;
			for( unsigned d=0;d<f->coeff.getDim();d++ )
			{
				yf += f->coeff[d] * pow(xf,(float)d);
			}

			// retransform into image space and draw line
			int y = (float)imageHeight - 1.0f - (yf - viewport.lowerCorner[1]) / viewport.getSideLength(1) * (float)imageHeight;
			QPoint nextPoint(x,y);
			if(x>0)
			{
				painter.drawLine(lastPoint,nextPoint);
			}
			lastPoint = nextPoint;
		}
	}

	// draw datapoint functions
	for( unsigned i=0;i<m_DataPointFunction.size();i++ )
	{
		TDataPointFunction * f = &m_DataPointFunction[i];
		painter.setPen(QColor(f->color[0]*255.0f,f->color[1]*255.0f,f->color[2]*255.0f));
		QPoint lastPoint;
		for( int p=0;p<(int)f->dataPoints.size();p++ )
		{
			// transform into image space
			int x = (f->dataPoints[p][0] - viewport.lowerCorner[0]) / viewport.getSideLength(0) * (float)imageWidth;
			int y = (float)imageHeight - 1.0f - (f->dataPoints[p][1] - viewport.lowerCorner[1]) / viewport.getSideLength(1) * (float)imageHeight;
			QPoint nextPoint(x,y);

			// draw
			if( f->drawLines )
			{
				if(p>0)
				{
					painter.drawLine(lastPoint,nextPoint);
				}
			}
			else
				painter.drawPoint(nextPoint);
			lastPoint = nextPoint;
		}
	}

	painter.end();
	
	// convert qimage into RGBAFloat32Image and return result
	RGBAFloat32Image * ret = new RGBAFloat32Image(imageWidth,imageHeight);
	for( unsigned x=0;(int)x<imageWidth;x++)
	{
		for( unsigned y=0;(int)y<imageHeight;y++ )
		{
			QRgb rgb = destImage.pixel(x,y);
			Vector4f col = makeVector4f( qRed(rgb),qGreen(rgb), qBlue(rgb),255) / 255.0f;
			ret->setPixel(x,y,col);
		}
	}

	return ret;

}