#ifndef PhotometricContraintsH
#define PhotometricContraintsH
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "ARConstraintsInterface.h"
#include "AROptToolWithFeedback.h"
//---------------------------------------------------------------------------

namespace X4
{
	class ARSettings;
	class PointCloud;
	class ARConstraintsInterface;
	class ParametricInCorePointCloud;
	class CameraParameterAttachment;
	class OneByteImage;
	class RGBAFloat32Image;
/**
	computes photometric contraints for multi view scene
*/
class X3DEFORMATION_API PhotometricContraints : public Persistent, public AROptToolWithFeedback 
{
	X4_CLASS( PhotometricContraints )

public:
	PhotometricContraints(void);
	~PhotometricContraints(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	/// builds constraints that move points from referenceShape
	/// to match matchingShape
	void buildPhotometricConstraints(
		const ARSettings *settings,
		ARConstraintsInterface *receiver,
		/// parametrization (urshape)
		PointCloud *referenceShape,
		/// urshape as in its current form ("best guess"), used for correspondences
		PointCloud *deformedReference,
		/// image data
		std::vector<ParametricInCorePointCloud*> images,
		/// projection information for each image
		std::vector<CameraParameterAttachment*> cameraAttachments,
		/// visualization
		bool visualize = false
		);

	float computeDisparityConstraint( const Vector3f &point, ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1, 
		ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2, Vector3f &constraint_pos, Matrix3f &contraint_matrix,
		float confidenceMode = false);

	/// compute confidence for each point in pointcloud 'data'
	void computeConfidence( PointCloud * data, std::vector<ParametricInCorePointCloud*> images, std::vector<CameraParameterAttachment*> cameraAttachments );

	// -------------------------------
	// property access methods
	float32 getMinCorrelationThreshold() const {return m_MinCorrelationThreshold;}
	void setMinCorrelationThreshold(float32 v) {m_MinCorrelationThreshold=v;}

	card32 getWindowSize() const {return m_WindowSize;}
	void setWindowSize(card32 v) {m_WindowSize=v;}

	float32 getMinEdgeThreshold() const {return m_MinEdgeThreshold;}
	void setMinEdgeThreshold(float32 v) {m_MinEdgeThreshold=v;}

	Vector3f testPoint;
	Scene * debugScene;

private:

	float computeDisparity( const Vector3f &point, ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1, ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2 );

	/// fit c2*x^2 + c1*x + c0 onto data points
	void computeQuadraticFit( const std::vector<Vector2f> &data, Vector3f &coeffs );
	
	/// fit c1*x^2 + c0 onto data points
	void computeQuadraticFitSpecial( const std::vector<Vector2f> &data, Vector2f &coeffs );

	/// marks every good point
	OneByteImage * preprocessImage( ParametricInCorePointCloud * image );

	float32	m_MinCorrelationThreshold;
	card32	m_WindowSize;
	float32	m_MinEdgeThreshold;
	float32	m_MinQuadCoeff;






};

class Function1DImageGenerator
{
public:

	/// add a polynomial function c0 + c1*x c2*x^2 + ...
	void addPolynomialFunction( const DVectorF &coeffs, const Vector3f &col);

	/// add a function consisting of a set of data points.
	/// draw a line between two sequent points when drawLines is activated
	void addDataPointFunction( const std::vector<Vector2f> &dataPoints, const Vector3f &col, bool drawLines = true );

	/// create image
	RGBAFloat32Image * createImage( const BoundingBox2f &viewport, int imageWidth, int imageHeight);

private:

	struct TPolynomialFunction
	{
		DVectorF coeff;
		Vector3f color;
	};

	struct TDataPointFunction
	{
		std::vector<Vector2f> dataPoints;
		bool drawLines;
		Vector3f color;
	};

	std::vector<TPolynomialFunction> m_PolynomialFunction;
	std::vector<TDataPointFunction> m_DataPointFunction;
};

}

#endif