//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "DeformationEvaluator.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
#include <fstream>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

#define MAKEASLASH(s) /##s
// instrument code for debugging?
// define to "//" to turn off
#define DBG_INSTR MAKEASLASH(/)


IMPLEMENT_X4_ABSTRACT_CLASS( DeformationEvaluator , 0 ) {
	BEGIN_CLASS_INIT( DeformationEvaluator );
}

// ----

DeformationEvaluator::DeformationEvaluator() {
}

DeformationEvaluator::~DeformationEvaluator() {
}




Vector3f DeformationEvaluator::invertDeformation(
   const Vector3f &position, 
   const ARConstraintAnchor &closeUrshapePoint,
   Matrix3f *linApprox,
   Matrix3f *invLinApprox,
   float32 precision, 
   card32 maxIterations) 
{
   DBG_INSTR std::ofstream stat("c:\\temp\\xgrt_stat.txt", ios_base::app);       

   Matrix3f linF = getSpatialGradient(closeUrshapePoint);
   Matrix3f linFInv = invertMatrix(linF);
   Vector3f currentGuess = closeUrshapePoint.pos + linFInv * (position - getPosition(closeUrshapePoint));
   
   float32 resQuad = normQuad(currentGuess - closeUrshapePoint.pos);
   const float32 precisionQuad = sqr(precision);
   card32 numIt = 0;
   DBG_INSTR std::vector<float32> resOut;
   DBG_INSTR std::vector<float32> deltaOut;
   DBG_INSTR resOut.push_back(sqrt(resQuad));
   while (resQuad > precisionQuad && numIt < maxIterations) {
      linF = getSpatialGradient(ARConstraintAnchor(currentGuess));
      linFInv = invertMatrix(linF);
      Vector3f delta = linFInv * (position - getPosition(ARConstraintAnchor(currentGuess)));
      currentGuess = currentGuess + delta;
      resQuad = normQuad(delta);
      numIt++;
      DBG_INSTR resOut.push_back(sqrt(resQuad));
      DBG_INSTR deltaOut.push_back(norm(delta));
   }
   DBG_INSTR stat << "it:\t" << numIt << "\t";
   DBG_INSTR stat << "res:";
   DBG_INSTR for (card32 i=0; i<resOut.size(); i++) {
   DBG_INSTR    stat << "\t" << sqrt(resQuad);
   DBG_INSTR }
   DBG_INSTR stat << "\tdelta:";
   DBG_INSTR for (card32 i=0; i<resOut.size(); i++) {
   DBG_INSTR    stat << "\t" << sqrt(resQuad);
   DBG_INSTR }
   DBG_INSTR stat << endl;
   
   
   
   if (linApprox) {
      *linApprox = linF;
   }
   if (invLinApprox) {
      *invLinApprox = linFInv;
   }
   
   return currentGuess;
}
