//---------------------------------------------------------------------------
#ifndef DeformationEvaluatorH
#define DeformationEvaluatorH
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include "AROptimizationTool.h"
#include "ARConstraints.h"
//---------------------------------------------------------------------------

namespace X4 {

    ///   mw - 10/11/2007
    ///
    ///   abstract base class of deformation evaluators
    ///   the purpose of this class is to evaluate the deformation field computed
    ///   by a DeformationSolver continuously, at any point within the deformed domain
    ///
    class X3DEFORMATION_API DeformationEvaluator : public AROptimizationTool {
        X4_ABSTRACT_CLASS( DeformationEvaluator )
    private:

    public:
        DeformationEvaluator();

        /// start querying for deformation results
        /// must be called before any query
        /// override to cache iterators, AATs etc...
        virtual void beginQuery(float32 time) = 0;

        /// this method just calculates the postion
        virtual Vector3f getPosition(const ARConstraintAnchor &at) = 0;
        /// this method evalute at position by interpolation
        virtual Vector3f Evaluate (Vector3f const& position) { return position; }
        /// this method just calculates the velocity
        virtual Vector3f getVelocity(const ARConstraintAnchor &at) = 0;
        /// and this one the spatial gradient
        virtual Matrix3f getSpatialGradient(const ARConstraintAnchor &at) = 0;
        /// this method computes the inverse of getPosition()
        /// closeUrshapePoint is used to initialize a non-linear optimization (if necessary)
        /// the parameter linApprox returns the gradient of the deformation at the inverted point (if pointer is not NULL)
        /// the parameter invLinApprox returns the inverse of the gradient at the inverted point (if pointer is not NULL)
        virtual Vector3f invertDeformation(
            const Vector3f &position, 
            const ARConstraintAnchor &closeUrshapePoint, 
            Matrix3f *linApprox = NULL,
            Matrix3f *invLinApprox = NULL,
            float32 precision = 1e-5,
            card32 maxIterations = 0); // mw - turned off additional newton steps (just one linear correction step)

        /// end queries, throw away chached helper objects if necessary
        virtual void endQuery() {};

        ~DeformationEvaluator();
    };

}

#endif