//---------------------------------------------------------------------------
#ifndef DeformationSolverH
#define DeformationSolverH
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include "AROptimizationTool.h"
#include "Delegate.h"
#include "ARConstraintsInterface.h"
//---------------------------------------------------------------------------

namespace X4 {

class ARSettings;
class DeformationEvaluator;
class GroupDeformationContext;

///   mw - 09/27/2007
///
///   abstract base class of all deformation solvers
///
///   given a set of derivative constraints and boundary conditions (domain with regularization weights),
///   an optimal deformation field is computed
///   boundary conditions are taken from the "domain" property (see superclass)
///
class X3DEFORMATION_API DeformationSolver : public AROptimizationTool, public ARTimeVariantConstraintsInterface {
	X4_ABSTRACT_CLASS( DeformationSolver )
 private:

 public:
	/// create an evaluator that is compatible with this solver (will work on the same context)
	virtual DeformationEvaluator *createEvaluator() = 0;
	
	/// call this method to begin specifying constraints
	virtual void beginConstraints() = 0;
	
	// --- constraints are added through TimeVariantARConstraintsInterface methods
	virtual void addGeneralConstraint(const GeneralARConstraint *c);
	
	/// solve with given constraints and boundary conditions
	/// has to pair with "beginConstraints" (this is implicitly "endConstraints") 
	virtual void solve() = 0;
	
	/// solves a number of problems that are coupled as specified in the group context
	/// group specifies all the _other_ problems to be solved (not including this one)
	/// the default behavior is to ignore these constraints and solve the problems separately
	virtual void solveGroup(GroupDeformationContext *groupContext, std::vector<DeformationSolver*> group);
	
	/// calling this method instead of beginConstraints....solve causes the deformation context to be initialized
	/// with a (low complexity) identity std::map that does not transform the urshape points in any way
	virtual void setIdentityMap();

   /// call anytime to set a method-of-object pointer to be called back to give feedback on
   /// current solution (e.g. redraw scene, output scores, etc).
	virtual void addUserFeedbackCallback(Delegate d) {}
	virtual void clearUserFeedbackCallbacks() {}
};

}

#endif