//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "FDDeformationContext.h"
//---------------------------------------------------------------------------
#include "FiniteDifferencesDeformation.h"
#include "FiniteDifferencesDeformation.inline.h"
#include "SGEShape.h"
#include "SGEVolumeGridDeformationSequence.h"
#include "SGAnimationNode.h"
#include "SGObjectNode.h"
#include "StringHelper.h"
#include "RegularInCoreVolume.h"
#include "RegularVolume.h"
#include "RegularVolume.inline.h"
#include "ARSettings.h"
#include "SGRelativeTimeAnimationNode.h"
#include "FDDeformationSolver.h"
#include "FDDeformationEvaluator.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( FDDeformationContext , 0 ) {
	BEGIN_CLASS_INIT( FDDeformationContext );
}

// ----

FDDeformationContext::FDDeformationContext() {
	urshape = NULL;
	baseResolution = 0;
}

FDDeformationContext::~FDDeformationContext() {
	for (unsigned i=0; i<vols.size(); i++) {
		delete vols[i];
	}
	delete urshape;
}



RegularInCoreVolume *FDDeformationContext::createDeformationStorageObject(PointCloud *pc, const ARSettings *settings) 
{
   BoundingBox3f bb(NULL_VECTOR3F, makeVector3f(1,1,1));
   SceneObjectIterator *it = pc->createIterator(SOIT::CAP_HIERARCHICAL);
   HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(it);
   if (hit) {
      bb = hit->getBoundingBox();
      bb.makeBoundingCube();
      bb.enlarge(1.5f);
   }
   delete it;

   VertexDescriptor vd;
   vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
   vd.pushAttrib(mVAD("nonRigidWeight", 1, VAD::DATA_FORMAT_FLOAT32));
   RegularVolume *resultVol = new RegularVolume();
   resultVol->clearAndSetup(settings->fdDiscretization,
                            settings->fdDiscretization,
                            settings->fdDiscretization, 
                            &vd, bb);
   AAT posAAT = resultVol->getDescr()->getToken("position");
   float32 w = resultVol->getWidth();
   float32 h = resultVol->getHeight();
   float32 d = resultVol->getDepth();
   FOR_XYZ(resultVol)
      float32 xf = x / w;
      float32 yf = y / h;
      float32 zf = z / d;
      Vector3f initPos = makeVector3f(xf,yf,zf).componentProduct(bb.upperCorner-bb.lowerCorner)
                       + bb.lowerCorner;
      resultVol->setVoxel(x,y,z, posAAT, initPos);
   END_FOR_XYZ()
   RegularInCoreVolume *result = new RegularInCoreVolume();
   result->clearAndSetup(resultVol);
   return result;
}

void FDDeformationContext::initializeFromShape(const SceneObject *obj, Vector2f timeInterval, const ARSettings *settings) {
	const PointCloud *pc = dynamic_cast<const PointCloud*>(obj);
	if (!pc) {
		throw X4Exception("FDDeformationContext::initializeFromShape() - obj is not of correct type.");
	}
	delete urshape;
	urshape = (PointCloud*)(pc->copy());
	int numFrames = (int)(timeInterval[1] - timeInterval[0])+1;
	if (numFrames < 1) {
		throw ERangeCheck("FDDeformationContext::initializeFromShape() - timeIntervall invalid.");
	}
	
	for (unsigned i=0; i<vols.size(); i++) {
		delete vols[i];
	}
	vols.clear();
	
	vols.resize(numFrames);
	for (int i=0; i<numFrames; i++) {
		vols[i] = createDeformationStorageObject(urshape, settings);
	}
	baseResolution = settings->fdDiscretization;
}

float32 FDDeformationContext::getDuration() 
{
   card32 n = getNumVolumes();
   return (n==0) ? 0 : n-1;
}

DeformationSolver *FDDeformationContext::createSolver() 
{
   return new FDDeformationSolver();
}

DeformationEvaluator *FDDeformationContext::createEvaluator() 
{
   return new FDDeformationEvaluator();
}

float32 FDDeformationContext::getBaseResolution() 
{
   return baseResolution;
}
