//---------------------------------------------------------------------------
#ifndef FDDeformationContextH
#define FDDeformationContextH
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include "ARDeformationOptimizationContext.h"
//---------------------------------------------------------------------------

namespace X4 {

class RegularInCoreVolume;

///
///   this class stores all the data to describe a deformation as computed by the FDDeformationSolver/Evaluator
///
class X3DEFORMATION_API FDDeformationContext : public ARDeformationOptimizationContext {
	X4_CLASS( FDDeformationContext )
 private:
	PointCloud *urshape;
	
   std::vector<RegularInCoreVolume*> vols;
   float32 baseResolution;
   
	/// helper function
	RegularInCoreVolume *createDeformationStorageObject(PointCloud *pc, const ARSettings *settings);

 public:
	FDDeformationContext();

	virtual void initializeFromShape(const SceneObject *obj, Vector2f timeInterval, const ARSettings *settings);
	
	virtual float32 getDuration();

	inline card32 getNumVolumes() {return (card32)vols.size();}
	inline RegularInCoreVolume *getVolume(card32 i) {return vols[i];}

	virtual DeformationSolver *createSolver();
	virtual DeformationEvaluator *createEvaluator();

	virtual float32 getBaseResolution();

   ~FDDeformationContext();
};

}

#endif