//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "FDDeformationEvaluator.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "FiniteDifferencesDeformation.h"
#include "ARConstraints.h"
#include "FiniteDifferencesDeformation.h"
#include "FiniteDifferencesDeformation.inline.h"
#include "RegularVolume.h"
#include "RegularVolume.inline.h"
#include "SGEVolumeGridDeformationSequence.h"
#include "RegularInCoreVolume.h"
#include "FDDeformationEvaluator.h"
#include "FDDeformationContext.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( FDDeformationEvaluator , 0 ) {
	BEGIN_CLASS_INIT( FDDeformationEvaluator );
}

// ----

FDDeformationEvaluator::FDDeformationEvaluator() {
	context = NULL;
}

FDDeformationEvaluator::~FDDeformationEvaluator() {}



AROptimizationContext *FDDeformationEvaluator::createOptContext() 
{
	throw X4Exception("FDDeformationEvaluator::createDeformationOptContext() - please use FDDeformationSolver to create context.");
}

void FDDeformationEvaluator::setup(AROptimizationContext *context, const ARSettings *settings) 
{
	if (!dynamic_cast<FDDeformationContext*>(context)) {
		throw ETypeMismatch("FDDeformationSolver::setup() - context does not have the correct dynamic type.");
	}
	this->context = (FDDeformationContext*)context;
	this->settings = settings;	
}



Vector3f FDDeformationEvaluator::getPosition(const ARConstraintAnchor &at) {
   x4Assert(regVol);
   return regVol->interpolateAttribute3f(transformVector3f(transform, at.getPos()), posAAT);
}

Vector3f FDDeformationEvaluator::getVelocity(const ARConstraintAnchor &at){
   if (!prevVol && !nextVol) {
      throw ERangeCheck("only one frame - no velocity defined.");
   }
   Vector3f tPos = transformVector3f(transform, at.getPos());
   if (prevVol) {
      return   regVol->interpolateAttribute3f(tPos, posAAT)
             - prevVol->interpolateAttribute3f(tPos, posAAT);
   } else {
      return   nextVol->interpolateAttribute3f(tPos, posAAT)
             - regVol->interpolateAttribute3f(tPos, posAAT);
   }
}

Vector3f FDDeformationEvaluator::getAcceleration(const ARConstraintAnchor &at){
   if (!(prevVol && nextVol)) {
      throw ERangeCheck("need two frames to define acceleration.");
   }
   Vector3f tPos = transformVector3f(transform, at.getPos());
   return   prevVol->interpolateAttribute3f(tPos, posAAT)
          - regVol->interpolateAttribute3f(tPos, posAAT)*2
          + nextVol->interpolateAttribute3f(tPos, posAAT);
}

Matrix3f FDDeformationEvaluator::getSpatialGradient(const ARConstraintAnchor &at) {
   Vector3f tPos = transformVector3f(transform, at.getPos());
   card32 w = regVol->getWidth();
   card32 h = regVol->getHeight();
   card32 d = regVol->getDepth();
   Vector3i pi = makeVector3i(
      max<int>(min<int>(tPos[0], 0), w-2),
      max<int>(min<int>(tPos[1], 0), h-2),
      max<int>(min<int>(tPos[2], 0), d-2)
   );
   Matrix3f result;
   PointSet *ps = regVol->getVoxelData();
   result[0] = ps->get3f(pi[0]+1 + w*pi[1]   + w*h*pi[2],   posAAT) -
               ps->get3f(pi[0]   + w*pi[1]   + w*h*pi[2],   posAAT);
   result[1] = ps->get3f(pi[0]   + w*pi[1]+1 + w*h*pi[2],   posAAT) -
               ps->get3f(pi[0]   + w*pi[1]   + w*h*pi[2],   posAAT);
   result[2] = ps->get3f(pi[0]   + w*pi[1]   + w*h*pi[2]+1, posAAT) -
               ps->get3f(pi[0]   + w*pi[1]   + w*h*pi[2],   posAAT);
   return result;
}


void FDDeformationEvaluator::beginQuery(float32 time) 
{
	card32 frame = (card32)time;
	if (!context) {
		throw EInvalidState("FDDeformationEvaluator::beginQuery() - context is not setup.");
	}
   RegularInCoreVolume *vol = context->getVolume(frame);
   regVol = vol->getVoxels();

   transform = bbToVoxelTransform(regVol);

   if (frame > 0) {
      vol = context->getVolume(frame-1);
      prevVol = vol->getVoxels();
   }
   if (frame+1 < context->getNumVolumes()) {
      vol = context->getVolume(frame+1);
      nextVol = vol->getVoxels();
   }
   posAAT = regVol->getDescr()->getToken("position", 3, VAD::DATA_FORMAT_FLOAT32);
   x4Assert(posAAT != NULL_AAT);
}

void FDDeformationEvaluator::endQuery() {}

