//---------------------------------------------------------------------------
#ifndef FDDeformationEvaluatorH
#define FDDeformationEvaluatorH
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include "DeformationEvaluator.h"
#include "ARConstraints.h"
//---------------------------------------------------------------------------

namespace X4 {

class RegularVolume;
class FDDeformationContext;

///   mw - 10/11/2007
///
///
///   Brain dead super-simple deformation solver based on finite differences
///   and linear interpolation (evaluator module)
///
class X3DEFORMATION_API FDDeformationEvaluator : public DeformationEvaluator {
	X4_CLASS( FDDeformationEvaluator )
 private:
	FDDeformationContext *context;
	const ARSettings *settings;
	
   RegularVolume *regVol;
   RegularVolume *prevVol;
   RegularVolume *nextVol;
   Matrix4f transform;
   AAT posAAT;
      
 public:
	FDDeformationEvaluator();

	AROptimizationContext *createOptContext();
	virtual void setup(AROptimizationContext *context, const ARSettings *settings);

   virtual void beginQuery(float32 time);

   virtual Vector3f getPosition(const ARConstraintAnchor &at);
   virtual Vector3f getVelocity(const ARConstraintAnchor &at);
   virtual Vector3f getAcceleration(const ARConstraintAnchor &at);
   virtual Matrix3f getSpatialGradient(const ARConstraintAnchor &at);

   virtual void endQuery();

   ~FDDeformationEvaluator();
};

}

#endif