//---------------------------------------------------------------------------
#ifndef FDDeformationSolverH
#define FDDeformationSolverH
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include "DeformationSolver.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {
class RegularVolume;
class SGEDeformationSequence;
class SGEShape;
class FDDeformationContext;


///   mw - 10/11/2007
///
///   Brain-dead super-simple deformation solver based on finite differences
///   and linear interpolation
///
class X3DEFORMATION_API FDDeformationSolver : public DeformationSolver {
	X4_CLASS( FDDeformationSolver )
 private:
   std::vector<Delegate> callbacks;
	FDDeformationContext *context;
	const ARSettings *settings;
	
   // --- variables used between beginConstraints() and solve() ---
   AAT positionAAT;
   /// quadratic part of the (quadratic) constraint energy        
   SparseMatrixF constraintsQuadric;
   /// linear part of the (quadratic) constraint energy           
   DVectorF constraintsLinear;
   /// current extends of the volumes                             
   card32 volWidth, volHeight, volDepth;
   // -------------------------------------------------------------
   
 protected:
   void callCallbacks();
   
 public:
	FDDeformationSolver();

	virtual DeformationEvaluator *createEvaluator();
	AROptimizationContext *createOptContext();
	virtual void setup(AROptimizationContext *context, const ARSettings *settings);

	virtual void addUserFeedbackCallback(Delegate d);
	virtual void clearUserFeedbackCallbacks();

	virtual void beginConstraints();
   virtual void addPositionConstraint(const PositionARConstraint *c);
	virtual void addHalfSpaceConstraint(const ARHalfSpaceConstraint *c);
   virtual void addVelocityConstraint(const VelocityVectorARConstraint *c);
   virtual void addInstantaneousVelocityConstraint(const InstantaneousVelocityARConstraint *c);
   virtual void addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c);
   virtual void addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c);
   
   /// test code
   void addVoxelConstraint(card32 x, card32 y, card32 z, Vector3f pos);
   void addExampleConstraints();

	virtual void solve();


	~FDDeformationSolver();
};

}

#endif