//---------------------------------------------------------------------------
#ifndef FiniteDifferencesDeformationH
#define FiniteDifferencesDeformationH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "PointCloudTools.h"
#include "VertexDescriptor.h"
//---------------------------------------------------------------------------

///   mw - 10/11/2007
///
///   This module is a collection of helper classes and functions for the
///   finite differences deformation solver (the test solver)

namespace X4 {

class PointSet;
class SGEShape;
class SGEDeformationSequence;
class RegularVolume;
class ARSettings;


// ------ helper functions
                          
/// create SGE template for FD solver
X3DEFORMATION_API SGEShape* createFDDeformSGEShapeTemplate();

/// create SGE template for FD solver
X3DEFORMATION_API SGEDeformationSequence* createFDDeformSGEDefSeqTemplate();

/// computes a transformation that maps world coordinates to voxel coordinates
/// using the bounding box of the volume object
X3DEFORMATION_API Matrix4f bbToVoxelTransform(RegularVolume *vol);


// ------ deformation engine

class X3DEFORMATION_API FDDeformSolverTools {
 public:
   /// computes gradient and Hessian of the regularization score
   /// (deformation energy, acceleration energy)
   static void buildRegScoreTaylorApprox(
      RegularVolume *fVol,
      float32 &score,
      DVectorF &gradient,
      SparseMatrixF &hessian,
      const ARSettings *settings);

   static void convertVolToVec(const RegularVolume *vol, DVectorF &vec);
   static void convertVecToVol(const DVectorF &vec, RegularVolume *vol);
};

}

#endif