//---------------------------------------------------------------------------
#ifndef MMDeformationEvaluatorH
#define MMDeformationEvaluatorH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "DeformationEvaluator.h"
#include "MMDeformationContext.h"
#include "ARConstraints.h"
//---------------------------------------------------------------------------

namespace X4 {

///   ba-mo - 12/6/2007
///
///   abstract base class of deformation evaluators
///   the purpose of this class is to evaluate the deformation field computed
///   by a DeformationSolver continuously, at any point within the deformed domain
///
class X3DEFORMATION_API MMDeformationEvaluator : public DeformationEvaluator {
	X4_CLASS( MMDeformationEvaluator )
 private:

	 MMDeformationContext * context;
	 const ARSettings *settings;
	 float32 time;

 public:
	MMDeformationEvaluator();

	AROptimizationContext *createOptContext();
	virtual void setup(AROptimizationContext *context, const ARSettings *settings);


   /// start querying for deformation results
   /// must be called before any query
   /// override to cache iterators, AATs etc...
   virtual void beginQuery(float32 time);

   /// this method just calculates the postion
	virtual Vector3f getPosition(const ARConstraintAnchor &at) ;
	/// this method just calculates the velocity
   virtual Vector3f getVelocity(const ARConstraintAnchor &at) ;
   /// and this one the spatial gradient
   virtual Vector3f getAcceleration(const ARConstraintAnchor &at);
   virtual Matrix3f getSpatialGradient(const ARConstraintAnchor &at);

   /// end queries, throw away chached helper objects if necessary
   virtual void endQuery();

   ~MMDeformationEvaluator();
};

}

#endif