//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MMDeformationSolver.h"
#include "SGEMeshlessDeformationSequence.h"
#include "UnstructuredInCorePointCloud.h"
#include "SGEPointCloud.h"
#include "ARConstraints.h"
#include "SGEMeshlessUrshape.h"
#include "VertexDescriptor.h"
#include "MeshlessCachedConstraints.h"
#include "AttachedNodeInformation.h"
#include "MMDeformationSolverAdapter.h"
#include "NonLinearIterativeSolvers.inline.h"
#include "MMDeformationEvaluator.h"
#include "ARSettings.h"
#include "Timer.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( MMDeformationSolver ,0)
{
	BEGIN_CLASS_INIT( MMDeformationSolver );
}

MMDeformationSolver::MMDeformationSolver() {
	//_rigidityParam = 0.01f;
	//_correspondenceParam = 10.0f;
	//_volumeParam = 0.05f;
	//_temporalParam = 0;
	context = NULL;
}


MMDeformationSolver::~MMDeformationSolver() {}

AROptimizationContext *MMDeformationSolver::createOptContext() {
	return new MMDeformationContext();
}

void MMDeformationSolver::setup(AROptimizationContext *context, const ARSettings *settings) {
	if (!dynamic_cast<MMDeformationContext*>(context)) {
		throw ETypeMismatch("MMDeformationSolver::setup() - context does not have the correct dynamic type.");
	}
	this->context = (MMDeformationContext*)context;
	this->settings = settings;
	
	/*const card32 nbframes = this->context->nbFrames();
	card32 totalConstraints = 0;
	for (card32 frame=0; frame<nbframes; frame++) {
		MMConstraints * constraints = this->context->getConstraints(frame);
		totalConstraints += constraints->nbConstraints();
	}
	
	if(totalConstraints > 0)
		tempCorrespondenceParam = settings->correspondenceParam / totalConstraints;
	else
		tempCorrespondenceParam = settings->correspondenceParam;*/
		
	tempCorrespondenceParam = settings->correspondenceParam;

	std::cout << "Using settings" << endl;
	std::cout << "rigidity: " << settings->rigidityParam << endl;
	std::cout << "volume: " << settings->volumeParam << endl;
	std::cout << "correspondences: " << tempCorrespondenceParam << endl;
	std::cout << "acceleration: " << settings->accelerationParam << endl;
	std::cout << "velocity: " << settings->velocityParam << endl;
	
	
}


void MMDeformationSolver::addPositionConstraint(const PositionARConstraint *c) {
   if (context->isIdentity()) return;
   int32 t = (int32)c->getTime();
   if (t >= (int32)context->nbFrames() || t<0) throw ERangeCheck("time out of range - time: " + intToStr(t)+" context->nbFrames() " + intToStr(context->nbFrames()) + "\n");
   Vector3f posAt = c->getAnchor().getPos();
   Vector3f posTo = c->getToPoint();
   //std::cout << "Adding constraint: " << posAt[0] << " " << posAt[1] << " "  << posAt[2]  << endl;
   //std::cout << "going to: " << posTo[0] << " " << posTo[1] << " "  << posTo[2]  << endl;
   // TODO: compute neighboring nodes and evaluate their shape functions
   bool owner;
   MMNeighborContainer * neighbors = context->getNeighbors(c->getAnchor().getTopologicalAnchor(), posAt, owner);
   //std::cout << "constraint neighbors: " << neighbors->nbNeighbors() << endl;
   if (!neighbors->getShapeValuesInitialized()) {
	   bool success;
	   context->getNodes()->evaluateNodeShapeFunctions(neighbors, success);
   }
   MMPositionConstraint * constraint = new MMPositionConstraint((const PositionARConstraint*)c->copy(), neighbors, owner);
   //if (owner) delete neighbors;
   //MMConstraint * constraint = new MMConstraint((const PositionARConstraint*)c->copy());
   /*context->getNodes()->computeNeighbors(constraint);
   bool success;
   context->getNodes()->evaluateNodeShapeFunctions(constraint, success);
   //for (card32 i=0; i<constraint->nbNeighbors(); i++) {
	   //std::cout << constraint->getNeighborIndex(i) << " " << constraint->getShapeValue(i) << endl;
   //}
   //std::cout << "Total: " << constraint->nbNeighbors() << endl;
   if (success) {
      context->getConstraints(t)->addConstraint(constraint);
   }
   else {
	//   std::cout << "Not adding constraint, shape function evaluation failed" << endl;
	   delete constraint;
   }*/
   context->getConstraints(t)->setNumNodes(context->nbNodes());  
   context->getConstraints(t)->addPositionConstraint(constraint);
}

double MMDeformationSolver::d2a(){

	double d2a = 0;
	
	d2a += d2aCorrespondences();
	
	d2a += d2aRigidAndVolume();

	return d2a;
}

double MMDeformationSolver::d2aRigidAndVolume(){
	float32 rigidSum=0;
	float32 volumeSum=0;
	
	double alpha = context->getAlpha();

	const card32 nbframes = context->nbFrames();
	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations * deformations = context->getDeformations(frame);
		if (!deformations->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;

		//const card32 offset = context->getOffset(frame);
		const card32 nbnodes = context->nbNodes();
		for (card32 i=0; i < nbnodes; i++) {

			float32 J11 = 0, J12 = 0, J13 = 0, J21 = 0,  J22 = 0, J23 = 0, J31 = 0, J32 = 0, J33 = 0;
			float32 J11_a = 0, J12_a = 0, J13_a = 0, J21_a = 0,  J22_a = 0, J23_a = 0, J31_a = 0, J32_a = 0, J33_a = 0;

			const card32 real_index = context->getDeformations(frame)->getRealIndex(i);
			const bool active = context->getDeformations(frame)->isActive(i);
			const Vector3f &deformation = context->getDeformations(frame)->getDeformation(i);

			const float32 ui = deformation[0];
			const float32 vi = deformation[1];
			const float32 wi = deformation[2];

			const Vector3f &ddeformation = context->getDDeformations(frame)->getDeformation(i);

			const float32 dui = ddeformation[0];
			const float32 dvi = ddeformation[1];
			const float32 dwi = ddeformation[2];

			MMNode * node = context->getNodes()->getNode(i);

			for (card32 j=0; j < node->nbNeighbors(); j++) {
				const card32 neighbor_index = node->getNeighborIndex(j);
				
				const Vector3f &neighbor_deformation = context->getDeformations(frame)->getDeformation(neighbor_index);
				const float32 aij = node->getGradientShapeValues(j)[0];
				const float32 bij = node->getGradientShapeValues(j)[1];
				const float32 cij = node->getGradientShapeValues(j)[2];

				const float32 uj = neighbor_deformation[0];
				const float32 vj = neighbor_deformation[1];
				const float32 wj = neighbor_deformation[2];
				
			   const Vector3f &neighbor_ddeformation = context->getDDeformations(frame)->getDeformation(neighbor_index);
      
      		const float32 duj = neighbor_ddeformation[0];
				const float32 dvj = neighbor_ddeformation[1];
				const float32 dwj = neighbor_ddeformation[2];
      
				J11 += aij*(uj-ui + alpha*(duj-dui)); J11_a += aij*(duj - dui); 
				J12 += aij*(vj-vi + alpha*(dvj-dvi)); J12_a += aij*(dvj - dvi);
				J13 += aij*(wj-wi + alpha*(dwj-dwi)); J13_a += aij*(dwj - dwi);
				J21 += bij*(uj-ui + alpha*(duj-dui)); J21_a += bij*(duj - dui);
				J22 += bij*(vj-vi + alpha*(dvj-dvi)); J22_a += bij*(dvj - dvi);
				J23 += bij*(wj-wi + alpha*(dwj-dwi)); J23_a += bij*(dwj - dwi);
				J31 += cij*(uj-ui + alpha*(duj-dui)); J31_a += cij*(duj - dui);
				J32 += cij*(vj-vi + alpha*(dvj-dvi)); J32_a += cij*(dvj - dvi);
				J33 += cij*(wj-wi + alpha*(dwj-dwi)); J33_a += cij*(dwj - dwi);
			}
			
			float32 divE=0, divE_a = 0, divE_aa = 0;
			if (settings->volumeParam>0) {
				divE =  J11 + J22 + J33 + 
						J11*J22 + J22*J33 + J11*J33 - J31*J13 - J23*J32-J12*J21  
						+ J11*J22*J33
						+ J12*J23*J31
						+ J13*J21*J32
						- J11*J23*J32
						- J13*J22*J31
						- J12*J21*J33;
				
				divE_a = J11_a + J22_a + J33_a + J11_a*J22 + J11*J22_a + J22_a*J33 + J22*J33_a + J11_a*J33 + J11*J33_a - J31_a*J13 - J31*J13_a - J23_a*J32 - J23*J32_a - J12_a*J21 - J12*J21_a 
							+ J11_a*J22*J33 + J11*J22_a*J33 + J11*J22*J33_a 
							+ J12_a*J23*J31 + J12*J23_a*J31 + J12*J23*J31_a
							+ J13_a*J21*J32 + J13*J21_a*J32 + J13*J21*J32_a
							- J11_a*J23*J32 - J11*J23_a*J32 - J11*J23*J32_a
							- J13_a*J22*J31 - J13*J22_a*J31 - J13*J22*J31_a
							- J12_a*J21*J33 - J12*J21_a*J33 - J12*J21*J33_a;
							
				divE_aa = 2*J11_a*J22_a + 2*J22_a*J33_a + 2*J11_a*J33_a - 2*J31_a*J13_a - 2*J23_a*J32_a - 2*J12_a*J21_a 
							+ 2*(J11_a*J22_a*J33 + J11*J22_a*J33_a + J11*J22_a*J33_a
							+ J12_a*J23_a*J31 + J12*J23_a*J31_a + J12*J23_a*J31_a
							+ J13_a*J21_a*J32 + J13*J21_a*J32_a + J13*J21_a*J32_a
							- J11_a*J23_a*J32 - J11*J23_a*J32_a - J11*J23_a*J32_a
							- J13_a*J22_a*J31 - J13*J22_a*J31_a - J13*J22_a*J31_a
							- J12_a*J21_a*J33 - J12*J21_a*J33_a - J12*J21_a*J33_a);
			}
			
			// This is rigidity
			float S11 = (J11*J11 + J21*J21 + J31*J31 + 2*J11);     
			float S12 = (J11*J12 + J21*J22 + J31*J32 + J12 + J21); 
			float S13 = (J11*J13 + J21*J23 + J31*J33 + J13 + J31); 
			float S22 = (J12*J12 + J22*J22 + J32*J32 + 2*J22);     
			float S23 = (J12*J13 + J22*J23 + J32*J33 + J32 + J23); 
			float S33 = (J13*J13 + J23*J23 + J33*J33 + 2*J33); 
			
			if (active) { // don't take derivatives to constants
				//d^(S11^2)/d alpha^2
				rigidSum += 2*(2*J11*J11_a + 2*J21*J21_a + 2*J31*J31_a + 2*J11_a)*(2*J11*J11_a + 2*J21*J21_a + 2*J31*J31_a + 2*J11_a);
				rigidSum += 2*S11*(2*J11_a*J11_a + 2*J21_a*J21_a + 2*J31_a*J31_a);
				
				//d(S22^2)/d alpha^2
				rigidSum += 2*(2*J12*J12_a + 2*J22*J22_a + 2*J32*J32_a + 2*J22_a)*(2*J12*J12_a + 2*J22*J22_a + 2*J32*J32_a + 2*J22_a);
				rigidSum += 2*S22*(2*J12_a*J12_a + 2*J22_a*J22_a + 2*J32_a*J32_a);
				
				//d(S33^2)/d alpha^2
				rigidSum += 2*(2*J13*J13_a + 2*J23*J23_a + 2*J33*J33_a + 2*J33_a)*(2*J13*J13_a + 2*J23*J23_a + 2*J33*J33_a + 2*J33_a);
				rigidSum += 2*S33*(2*J13_a*J13_a + 2*J23_a*J23_a + 2*J33_a*J33_a);
				
				//2*d(S12^2)/d alpha^2
				rigidSum += 2*2*(J11*J12_a + J11_a*J12 + J21*J22_a + J21_a*J22 + J31*J32_a + J31_a*J32 + J12_a + J21_a)*(J11*J12_a + J11_a*J12 + J21*J22_a + J21_a*J22 + J31*J32_a + J31_a*J32 + J21_a + J12_a);
				rigidSum += 2*2*S12*(2*J11_a*J12_a + 2*J21_a*J22_a + 2*J31_a*J32_a);
				
				//2*d(S13^2)/d alpha^2
				rigidSum += 2*2*(J11*J13_a + J11_a*J13 + J21*J23_a + J21_a*J23 + J31*J33_a + J31_a*J33 + J13_a + J31_a)*(J11*J13_a + J11_a*J13 + J21*J23_a + J21_a*J23 + J31*J33_a + J31_a*J33 + J13_a + J31_a);
				rigidSum += 2*2*S13*(2*J11_a*J13_a + 2*J21_a*J23_a + 2*J31_a*J33_a);

				//2*d(S23^2)/d alpha^2
				rigidSum += 2*2*(J12*J13_a + J12_a*J13 + J22*J23_a + J22_a*J23 + J32*J33_a + J32_a*J33 + J32_a + J23_a)*(J12*J13_a + J12_a*J13 + J22*J23_a + J22_a*J23 + J32*J33_a + J32_a*J33 + J32_a + J23_a);
				rigidSum += 2*2*S23*(2*J12_a*J13_a + 2*J22_a*J23_a + 2*J32_a*J33_a);
								
				if (settings->volumeParam>0) {
					// This is volume 
					volumeSum += 2*divE*divE_aa;
					volumeSum += 2*divE_a*divE_a;
				}

			}
		}
	}

	return settings->rigidityParam*rigidSum+settings->volumeParam*volumeSum;
}

double MMDeformationSolver::d2aCorrespondences(){
	float32 d2aConstraintSum = 0;
	
	const card32 nbframes = context->nbFrames();
	
	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations * deformations = context->getDeformations(frame);
		if (!deformations->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;

		MMConstraints * constraints = context->getConstraints(frame);
		const card32 nbconstraints = constraints->nbPositionConstraints();
		
		//correspondences: sum_j F(x) uj - u(x)
		for(card32 i=0; i<nbconstraints; i++){
			MMPositionConstraint * constraint = constraints->getPositionConstraint(i);		
			Matrix3f covariance = constraint->getCovariance();
			
			Vector3f sum; sum.setZero();
 		MMNeighborContainer * neighbors = constraint->getNeighbors();
		
			const card32 nbvalues = neighbors->nbNeighbors();
			for (card32 j=0; j< nbvalues; j++) {
				const card32 neighbor_index = neighbors->getNeighborIndex(j);
		
				const Vector3f &ddeformation = context->getDDeformations(frame)->getDeformation(neighbor_index);
				sum+=ddeformation*neighbors->getShapeValue(j);				
		    }
			     
			d2aConstraintSum += 2*(sum*(covariance*sum));
		}
	}
	
	return tempCorrespondenceParam*d2aConstraintSum;
}

void MMDeformationSolver::addVelocityConstraint(const VelocityVectorARConstraint *c) {
	notImplemented();
}
   
void MMDeformationSolver::addInstantaneousVelocityConstraint(const InstantaneousVelocityARConstraint *c) {
	notImplemented();
}

void MMDeformationSolver::solve() {

	card32 totConstraints = 0;
	card32 tframes = context->nbFrames();
		
	if (settings->mmDebugOutput) debugOutput << "Solving between frames 0 and " << context->nbFrames()-1 << "\n";

	GradientDescentOptimization<float32>::Parameters param;
	param.stepSize = 0.001f;
	param.maxGradConv = 1e-5f;
	param.maxNumSteps = 100;

	card32 windowIndex = 0;
	//the size of the window 3 = number of Frames to optimize for during each parse.
	card32 windowOffset = settings->windowSize;

	
	
	while((windowIndex==0) || (windowIndex + windowOffset -1 < context->nbFrames())){
		//if (context->isIdentity()) return;
		std::cout << " MMSolver at window " << "[" << windowIndex << "..." << min<int>(windowIndex+windowOffset-1,context->nbFrames()-1) << "]\n" << flush;
		Timer T;

		if (true) {debugOutput << "Setting Window to [" << windowIndex << "..." << min<int>(windowIndex+windowOffset-1,context->nbFrames()-1) << "]\n" ; debugOutput.flush();}
		context->setWindow(windowIndex, windowIndex+windowOffset-1);		
		context->computeRealIndices();


		// now call non-linear conjugate gradient
		const card32 nbunknowns = context->nbNodes()*context->nbFrames();
		const card32 realNbUnknowns = context->getRealNbUnknowns();

		if (settings->mmDebugOutput) debugOutput << "MM solve (total unknowns: " << nbunknowns*3 << ", active unknowns: " << 3*realNbUnknowns << ")...\n";
		if (settings->mmDebugOutput) debugOutput << "      context identity? : " << context->isIdentity() << "\n";
			
		totConstraints = 0;
		/*
		for (card32 i=0; i<context->nbConstraints(); i++) {
			if(context->getFrameActive(i))
				totConstraints += context->getConstraints(i)->nbConstraints();
		}*/
		
		if (settings->mmDebugOutput) debugOutput << "Number of active Constraints: " << totConstraints << "\n";
		if (settings->mmDebugOutput) debugOutput.flush();
		 
		//Vector3f startEstimateVector = makeVector3f(0,0,0);
		//-context->getConstraints(0)->getConstraint(0)->getPos();
		DynamicVector<float32> startEstimate(realNbUnknowns*3);
		//int index = 0;
		for (card32 i=0; i<context->nbFrames(); i++) {
			MMDeformations * deformations = context->getDeformations(i);
		if (!deformations->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;
			//if(context->getFrameActive(i)){
			//	MMDeformations * deformations = context->getDeformations(i);
				for (card32 j=0; j<context->nbNodes(); j++) {
					const bool active = deformations->isActive(j);
					if (active) {				
						const card32 index = deformations->getRealIndex(j);
						//std::cout << "active: " << index << endl;
						const Vector3f &deformation = deformations->getDeformation(j);
						for (card32 k=0; k<3; k++) 
							//startEstimate[3*index+k] = context->getConstraints(0)->getPositionConstraint(0)->getDeformation()[k];
							startEstimate[3*index+k] = deformation[k];
					}
					else {
						//std::cout << "not active" << endl;
					}
				}
			//}
		}

		if (settings->mmDebugOutput) debugOutput  << "Func before = " << func() << "\n";
		if (settings->mmDebugOutput) std::cout << fRigidAndVolume() << "\t" << fCorrespondences() << "\t" << fAcceleration() << "\t" << fVelocity() << endl;

		if (settings->mmDebugOutput) std::cout << "Func before = " << func() << "\n";
		if (settings->mmSolverMethod == ARSettings::MMS_BFGS) {
			if (settings->mmDebugOutput) debugOutput << "Starting BFGS\n ";
			if (settings->mmDebugOutput) debugOutput.flush();
			MMDeformationSolverAdapter_BFGS adapter(this, realNbUnknowns*3);
			adapter.setx(startEstimate);
			adapter.BFGSoptimize();
			if (settings->mmDebugOutput)std::cout << "     cnls =  " << adapter.cnls << " cnf = " << adapter.cnf << " cng =  " << adapter.cng << endl;
			adapter.setx();
		} 
		else
		if( settings->mmSolverMethod == ARSettings::MMS_LBFGS )
		{
			if (settings->mmDebugOutput) debugOutput << "Starting LBFGS\n ";
			if (settings->mmDebugOutput) debugOutput.flush();

			MMDeformationSolverAdapter_LBFGS adapter(this,realNbUnknowns*3);
			adapter.setx(startEstimate);
			adapter.solve();
			if (settings->mmDebugOutput)std::cout << "     cnls =  " << adapter.cnls << " cnf = " << adapter.cnf << " cng =  " << adapter.cng << endl;
			adapter.setx();
		}
		else {
			MMDeformationSolverAdapter_NLCG adapter(this, realNbUnknowns*3);
			adapter.setx(startEstimate);
			//adapter.CGoptimize(settings->d2aLinesearch);
			adapter.CGoptimize(settings->d2aLinesearch);
			if (settings->mmDebugOutput)std::cout << "     cnls =  " << adapter.cnls << " cnf = " << adapter.cnf << " cng =  " << adapter.cng << endl;
			adapter.setx();
		}
		if (settings->mmDebugOutput) debugOutput << "   ...done (" << T.getDeltaValue() << " ms)\n";
	
		if (settings->mmDebugOutput) debugOutput  << "Func after = " << func() << "\n";
		if (settings->mmDebugOutput) debugOutput.flush();

		if (settings->mmDebugOutput)std::cout << "Func after = " << func() << "\n";
		if (settings->mmDebugOutput)std::cout << fRigidAndVolume() << "\t" << fCorrespondences() << "\t" << fAcceleration() << "\t" << fVelocity() << endl;
		
		windowIndex += 1;//(windowOffset==1?1:windowOffset-1);
	}
/*
	GradientDescentOptimization<float32>::optimizeSimple(param,
		&adapter,
		startEstimate,
		result,
		false);*/
	
		

//	adapter.result(result);

	
	// it should use f() and g() and the temporary
	// solution is stored in unknowns and the gradient
	// in gradient.

	// update the deformed positions with the solved deformations
	/*for (card32 i=0; i<nbframes; i++) {		
		PointSet * deformedPositions = dSeq->getDeformedNodePositions(i)->getPointSet();
		for (card32 j=0; j<nbnodes; j++) {			
			const Vector3f &originalPosition = originalPositions->get3f(j, positionAAT);			
			deformedPositions->set3f(j, positionAAT, originalPosition+unknowns[offsets[i]+j]);		
		}
	}*/
}

float32 MMDeformationSolver::func() {
	// First compute the real deformations
	context->computeRealDeformations();
	
	//std::cout << fRigidAndVolume() << "\t" << fCorrespondences() << "\t" << fAcceleration() << "\t" << fVelocity() << endl;
	return fRigidAndVolume()+fCorrespondences()+fAcceleration()+fVelocity()+fHalfSpace();
}

void MMDeformationSolver::grad(std::vector<Vector3f> &gradients) {
	// First compute the real deformations
	context->computeRealDeformations();
	// first initialize to zero
	gradients.clear();
	Vector3f zero; zero.setZero();
	const int nbunknowns = context->nbNodes()*context->nbFrames();
	const int realNbUnknowns = context->getRealNbUnknowns();
	gradients.resize(realNbUnknowns, zero);
	// compute the gradient of the different parts of the objective function
	gRigidAndVolume(gradients);
	
	//std::cout << "gRigidAndVolume" << "\t" << gradients. << endl;
	gCorrespondences(gradients);
	//std::cout << "gCorrespondences" << "\t" << norm(gradients) << endl;
	gAcceleration(gradients);
	gVelocity(gradients);
	gHalfSpace(gradients);
}

void MMDeformationSolver::precon(std::vector<Vector3f> &conditioner) {
	// First compute the real deformations
	context->computeRealDeformations();
	// first initialize to zero
	conditioner.clear();
	Vector3f ones; ones[0] = ones[1] = ones[2] = 0.f;
	
	const int nbunknowns = context->nbNodes()*context->nbFrames();
	const int realNbUnknowns = context->getRealNbUnknowns();
	
	conditioner.resize(realNbUnknowns, ones);
	// compute the gradient of the different parts of the objective function
	//gRigidAndVolume(condtioner);
	
	//std::cout << "gRigidAndVolume" << "\t" << gradients. << endl;
	mCorrespondences(conditioner);
	
	mRigidAndVolume(conditioner);
	//std::cout << "gCorrespondences" << "\t" << norm(gradients) << endl;
	//gAcceleration(gradients);
	//gVelocity(gradients);
	
	for(int i=0;i<realNbUnknowns; i++){
		for(int j=0;j<3;j++){
			if(conditioner[i][j] <= 0.00000001)
				conditioner[i][j] = 1.0f;		
		}
	}
}

void MMDeformationSolver::mCorrespondences(std::vector<Vector3f> &conditioner) {
	const card32 nbframes = context->nbFrames();
	
	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations * deformations = context->getDeformations(frame);
		if (!deformations->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;

		MMConstraints * constraints = context->getConstraints(frame);		
		const card32 nbconstraints = constraints->nbPositionConstraints();

		for(card32 i=0; i<nbconstraints; i++){
			MMPositionConstraint * constraint = constraints->getPositionConstraint(i);		
			
			Vector3f sum; sum.setZero();

			MMNeighborContainer * neighbors = constraint->getNeighbors();
			const card32 nbvalues = neighbors->nbNeighbors();

			for (card32 j=0; j< nbvalues; j++) {
				const card32 neighbor_index = neighbors->getNeighborIndex(j);
				const bool active = context->getDeformations(frame)->isActive(neighbor_index);

				if (!active) continue;
							
				const Matrix3f &covariance  = constraint->getCovariance();
				
				float32 Phij = neighbors->getShapeValue(j);
				
				sum[0] = sum[1] = sum[2] = 2*Phij*Phij;
				
				const card32 real_index = context->getDeformations(frame)->getRealIndex(neighbor_index);
				
				conditioner[real_index] += covariance*sum*tempCorrespondenceParam;
				//std::cout << _correspondenceParam << " " << sum.getSqrNorm() << " " << constraint->getShapeValue(j) << " " << gradients[offset+neighbor_index].getSqrNorm() << endl;
			}
		}
	}
}


void MMDeformationSolver::mRigidAndVolume(std::vector<Vector3f> &conditioner) {
	const float32 _rigidityParam = settings->rigidityParam;
	const float32 _volumeParam = settings->volumeParam;

	const card32 nbframes = context->nbFrames();
	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations * deformations = context->getDeformations(frame);
		if (!deformations->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;

		//const card32 offset = context->getOffset(frame);
		const card32 nbnodes = context->nbNodes();
		for (card32 i=0; i < nbnodes; i++) {

			float32 J11 = 0, J12 = 0, J13 = 0, J21 = 0,  J22 = 0, J23 = 0, J31 = 0, J32 = 0, J33 = 0;
			float32 sumAij = 0, sumBij = 0, sumCij = 0;

			const card32 real_index = context->getDeformations(frame)->getRealIndex(i);
			const bool active = context->getDeformations(frame)->isActive(i);
			const Vector3f &deformation = context->getDeformations(frame)->getDeformation(i);

			const float32 ui = deformation[0];
			const float32 vi = deformation[1];
			const float32 wi = deformation[2];

			MMNode * node = context->getNodes()->getNode(i);

			for (card32 j=0; j < node->nbNeighbors(); j++) {
				const card32 neighbor_index = node->getNeighborIndex(j);
				
				const Vector3f &neighbor_deformation = context->getDeformations(frame)->getDeformation(neighbor_index);
				const float32 aij = node->getGradientShapeValues(j)[0];
				const float32 bij = node->getGradientShapeValues(j)[1];
				const float32 cij = node->getGradientShapeValues(j)[2];

				const float32 uj = neighbor_deformation[0];
				const float32 vj = neighbor_deformation[1];
				const float32 wj = neighbor_deformation[2];

				sumAij += aij;
				sumBij += bij;
				sumCij += cij;
      
				J11 += aij*(uj-ui);
				J12 += aij*(vj-vi);
				J13 += aij*(wj-wi);
				J21 += bij*(uj-ui);
				J22 += bij*(vj-vi);
				J23 += bij*(wj-wi);
				J31 += cij*(uj-ui);
				J32 += cij*(vj-vi);
				J33 += cij*(wj-wi);
			}
			float32 divE=0;
			if (_volumeParam>0) {
				divE =  J11 + J22 + J33 + J11*J22 + J22*J33 + J11*J33 - J31*J13 - J23*J32-J12*J21+J11*J22*J33+J12*J23*J31+J13*J21*J32-J11*J23*J32-J13*J22*J31-J12*J21*J33;
			}
			// This is rigidity
			float S11 = (J11*J11 + J21*J21 + J31*J31 + 2*J11);     
			float S12 = (J11*J12 + J21*J22 + J31*J32 + J12 + J21); 
			float S13 = (J11*J13 + J21*J23 + J31*J33 + J13 + J31); 
			float S22 = (J12*J12 + J22*J22 + J32*J32 + 2*J22);     
			float S23 = (J12*J13 + J22*J23 + J32*J33 + J32 + J23); 
			float S33 = (J13*J13 + J23*J23 + J33*J33 + 2*J33); 
			if (active) { // don't take derivatives to constants
				const float32 J11_u = -sumAij;
				const float32 J12_v = -sumAij;
				const float32 J13_w = -sumAij;
				const float32 J21_u = -sumBij;
				const float32 J22_v = -sumBij;
				const float32 J23_w = -sumBij;
				const float32 J31_u = -sumCij;
				const float32 J32_v = -sumCij;
				const float32 J33_w = -sumCij;
				
				if (_volumeParam>0) {
					// This is volume 
					conditioner[real_index][0] += _volumeParam*2*(J11_u+J22*J11_u+J33*J11_u-J13*J31_u-J12*J21_u+J22*J33*J11_u+J12*J23*J31_u+J13*J32*J21_u-J23*J32*J11_u-J13*J22*J31_u-J12*J33*J21_u)*(J11_u+J22*J11_u+J33*J11_u-J13*J31_u-J12*J21_u+J22*J33*J11_u+J12*J23*J31_u+J13*J32*J21_u-J23*J32*J11_u-J13*J22*J31_u-J12*J33*J21_u);
					conditioner[real_index][1] += _volumeParam*2*(J22_v+J11*J22_v+J33*J22_v-J23*J32_v-J21*J12_v+J11*J33*J22_v+J23*J31*J12_v+J13*J21*J32_v-J11*J23*J32_v-J13*J31*J22_v-J21*J33*J12_v)*(J22_v+J11*J22_v+J33*J22_v-J23*J32_v-J21*J12_v+J11*J33*J22_v+J23*J31*J12_v+J13*J21*J32_v-J11*J23*J32_v-J13*J31*J22_v-J21*J33*J12_v);
					conditioner[real_index][2] += _volumeParam*2*(J33_w+J11*J33_w+J22*J33_w-J31*J13_w-J32*J23_w+J11*J22*J33_w+J12*J31*J23_w+J21*J32*J13_w-J11*J32*J23_w-J22*J31*J13_w-J12*J21*J33_w)*(J33_w+J11*J33_w+J22*J33_w-J31*J13_w-J32*J23_w+J11*J22*J33_w+J12*J31*J23_w+J21*J32*J13_w-J11*J32*J23_w-J22*J31*J13_w-J12*J21*J33_w);
				}

	  
				conditioner[real_index][0] += _rigidityParam*2*(2*J11*J11_u+2*J21*J21_u+2*J31*J31_u+2*J11_u)*(2*J11*J11_u+2*J21*J21_u+2*J31*J31_u+2*J11_u);
				conditioner[real_index][0] += _rigidityParam*2*S11*(2*J11_u*J11_u+2*J21_u*J21_u+2*J31_u*J31_u);
				
				conditioner[real_index][1] += _rigidityParam*2*(2*J12*J12_v+2*J22*J22_v+2*J32*J32_v+2*J22_v)*(2*J12*J12_v+2*J22*J22_v+2*J32*J32_v+2*J22_v);
				conditioner[real_index][1] += _rigidityParam*2*S22*(2*J12_v*J12_v+2*J22_v*J22_v+2*J32_v*J32_v);
				
				conditioner[real_index][2] += _rigidityParam*2*(2*J13*J13_w+2*J23*J23_w+2*J33*J33_w+2*J33_w)*(2*J13*J13_w+2*J23*J23_w+2*J33*J33_w+2*J33_w);
				conditioner[real_index][2] += _rigidityParam*2*S33*(2*J13_w*J13_w+2*J23_w*J23_w+2*J33_w*J33_w);
				
				conditioner[real_index][0] += 2*_rigidityParam*2*(J12*J11_u+J22*J21_u+J32*J31_u+J21_u)*(J12*J11_u+J22*J21_u+J32*J31_u+J21_u);
				conditioner[real_index][1] += 2*_rigidityParam*2*(J11*J12_v+J21*J22_v+J31*J32_v+J12_v)*(J11*J12_v+J21*J22_v+J31*J32_v+J12_v);

				conditioner[real_index][0] += 2*_rigidityParam*2*(J13*J11_u+J23*J21_u+J33*J31_u+J31_u)*(J13*J11_u+J23*J21_u+J33*J31_u+J31_u);
				conditioner[real_index][2] += 2*_rigidityParam*2*(J11*J13_w+J21*J23_w+J31*J33_w+J13_w)*(J11*J13_w+J21*J23_w+J31*J33_w+J13_w);

				conditioner[real_index][1] += 2*_rigidityParam*2*(J13*J12_v+J23*J22_v+J33*J32_v+J32_v)*(J13*J12_v+J23*J22_v+J33*J32_v+J32_v);
				conditioner[real_index][2] += 2*_rigidityParam*2*(J12*J13_w+J22*J23_w+J32*J33_w+J23_w)*(J12*J13_w+J22*J23_w+J32*J33_w+J23_w);

			}

			for (card32 j=0; j < node->nbNeighbors(); j++) {
				const card32 jindex = node->getNeighborIndex(j);
				const bool neighbor_active = context->getDeformations(frame)->isActive(jindex);		
				if (!neighbor_active) continue; // don't take derivatives to constants

				const card32 real_neighbor_index = context->getDeformations(frame)->getRealIndex(jindex);										
				const Vector3f &neighbor_deformation = context->getDeformations(frame)->getDeformation(jindex);
				const float32 aij = node->getGradientShapeValues(j)[0];
				const float32 bij = node->getGradientShapeValues(j)[1];
				const float32 cij = node->getGradientShapeValues(j)[2];

				const float J11_u = aij;
				const float J12_v = aij;
				const float J13_w = aij;
				const float J21_u = bij;
				const float J22_v = bij;
				const float J23_w = bij;
				const float J31_u = cij;
				const float J32_v = cij;
				const float J33_w = cij;

				// This is volume 
				if (_volumeParam>0) {
					conditioner[real_neighbor_index][0] += _volumeParam*2*(J11_u+J22*J11_u+J33*J11_u-J13*J31_u-J12*J21_u+J22*J33*J11_u+J12*J23*J31_u+J13*J32*J21_u-J23*J32*J11_u-J13*J22*J31_u-J12*J33*J21_u)*(J11_u+J22*J11_u+J33*J11_u-J13*J31_u-J12*J21_u+J22*J33*J11_u+J12*J23*J31_u+J13*J32*J21_u-J23*J32*J11_u-J13*J22*J31_u-J12*J33*J21_u);
					conditioner[real_neighbor_index][1] += _volumeParam*2*(J22_v+J11*J22_v+J33*J22_v-J23*J32_v-J21*J12_v+J11*J33*J22_v+J23*J31*J12_v+J13*J21*J32_v-J11*J23*J32_v-J13*J31*J22_v-J21*J33*J12_v)*(J22_v+J11*J22_v+J33*J22_v-J23*J32_v-J21*J12_v+J11*J33*J22_v+J23*J31*J12_v+J13*J21*J32_v-J11*J23*J32_v-J13*J31*J22_v-J21*J33*J12_v);
					conditioner[real_neighbor_index][2] += _volumeParam*2*(J33_w+J11*J33_w+J22*J33_w-J31*J13_w-J32*J23_w+J11*J22*J33_w+J12*J31*J23_w+J21*J32*J13_w-J11*J32*J23_w-J22*J31*J13_w-J12*J21*J33_w)*(J33_w+J11*J33_w+J22*J33_w-J31*J13_w-J32*J23_w+J11*J22*J33_w+J12*J31*J23_w+J21*J32*J13_w-J11*J32*J23_w-J22*J31*J13_w-J12*J21*J33_w);
				}				
				

				// This is rigidity
				conditioner[real_neighbor_index][0] += _rigidityParam*2*(2*J11*J11_u+2*J21*J21_u+2*J31*J31_u+2*J11_u)*(2*J11*J11_u+2*J21*J21_u+2*J31*J31_u+2*J11_u);
				conditioner[real_neighbor_index][0] += _rigidityParam*2*S11*(2*J11_u*J11_u+2*J21_u*J21_u+2*J31_u*J31_u);

				conditioner[real_neighbor_index][1] += _rigidityParam*2*(2*J12*J12_v+2*J22*J22_v+2*J32*J32_v+2*J22_v)*(2*J12*J12_v+2*J22*J22_v+2*J32*J32_v+2*J22_v);
				conditioner[real_neighbor_index][1] += _rigidityParam*2*S22*(2*J12_v*J12_v+2*J22_v*J22_v+2*J32_v*J32_v);

				conditioner[real_neighbor_index][2] += _rigidityParam*2*(2*J13*J13_w+2*J23*J23_w+2*J33*J33_w+2*J33_w)*(2*J13*J13_w+2*J23*J23_w+2*J33*J33_w+2*J33_w);
				conditioner[real_neighbor_index][2] += _rigidityParam*2*S33*(2*J13_w*J13_w+2*J23_w*J23_w+2*J33_w*J33_w);

				conditioner[real_neighbor_index][0] += 2*_rigidityParam*2*(J12*J11_u+J22*J21_u+J32*J31_u+J21_u)*(J12*J11_u+J22*J21_u+J32*J31_u+J21_u);
				conditioner[real_neighbor_index][1] += 2*_rigidityParam*2*(J11*J12_v+J21*J22_v+J31*J32_v+J12_v)*(J11*J12_v+J21*J22_v+J31*J32_v+J12_v);

				conditioner[real_neighbor_index][0] += 2*_rigidityParam*2*(J13*J11_u+J23*J21_u+J33*J31_u+J31_u)*(J13*J11_u+J23*J21_u+J33*J31_u+J31_u);
				conditioner[real_neighbor_index][2] += 2*_rigidityParam*2*(J11*J13_w+J21*J23_w+J31*J33_w+J13_w)*(J11*J13_w+J21*J23_w+J31*J33_w+J13_w);

				conditioner[real_neighbor_index][1] += 2*_rigidityParam*2*(J13*J12_v+J23*J22_v+J33*J32_v+J32_v)*(J13*J12_v+J23*J22_v+J33*J32_v+J32_v);
				conditioner[real_neighbor_index][2] += 2*_rigidityParam*2*(J12*J13_w+J22*J23_w+J32*J33_w+J23_w)*(J12*J13_w+J22*J23_w+J32*J33_w+J23_w);
			}        
		}
	}
}


static inline float standardkernel(const float R, const float r) {
   const float RR = R*R;
   const float qq = r*r/RR;

      if (qq > 1)
         return 0;
      else
      {
         const float dd = 1 - qq;
         return 315.0/(64.0*M_PI*RR*R)*dd*dd*dd;
      }

}


float32 MMDeformationSolver::fRigidAndVolume() {
	float32 rigidSum=0;
	float32 volumeSum=0;

	const card32 nbframes = context->nbFrames();

	float32 M11 = 0, M12 = 0, M13=0, M22 = 0, M23=0, M33=0;
	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations * deformations = context->getDeformations(frame);
		if (!deformations->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;

		const card32 nbnodes = context->nbNodes();
		for (card32 i=0; i < nbnodes; i++) {
			float32 N11 = 0, N12 = 0, N13 = 0, N21 = 0,  N22 = 0, N23 = 0, N31 = 0, N32 = 0, N33 = 0;			
			const Vector3f &deformation = context->getDeformations(frame)->getDeformation(i);

			const float32 ui = deformation[0];
			const float32 vi = deformation[1];
			const float32 wi = deformation[2];

			MMNode * node = context->getNodes()->getNode(i);
			
			const float32 nodeRadius = node->getRadius();
			float32 radiusScale = 0;

			for (card32 j=0; j < node->nbNeighbors(); j++) {
				const card32 neighbor_index = node->getNeighborIndex(j);
				
				const float32 neighborRadius = context->getNodes()->getNode(neighbor_index)->getRadius();
				
				const Vector3f &neighbor_deformation = context->getDeformations(frame)->getDeformation(neighbor_index);
				const float32 aij = node->getGradientShapeValues(j)[0];
				const float32 bij = node->getGradientShapeValues(j)[1];
				const float32 cij = node->getGradientShapeValues(j)[2];

				const float32 uj = neighbor_deformation[0];
				const float32 vj = neighbor_deformation[1];
				const float32 wj = neighbor_deformation[2];

				 N11 += aij*(uj - ui);
				 N12 += aij*(vj - vi);
				 N13 += aij*(wj - wi);
				 N21 += bij*(uj - ui);
				 N22 += bij*(vj - vi);
				 N23 += bij*(wj - wi);
				 N31 += cij*(uj - ui);
				 N32 += cij*(vj - vi);
				 N33 += cij*(wj - wi);
				 
				 radiusScale += standardkernel(neighborRadius,node->getDistance(j));
			}
			
			if(fabs(radiusScale)>0.000001)
				radiusScale = nodeRadius*nodeRadius*nodeRadius/radiusScale;
			else
				radiusScale = 1;
				
			//radiusScale = 1;
			//if(i == 20)
			//	debugOutput << "radius Scale = for node " << i << " = " << radiusScale << "\n";

			M11 = (N11*N11 + N21*N21 + N31*N31 + 2*N11);     M11*=M11;
			M12 = (N11*N12 + N21*N22 + N31*N32 + N12 + N21); M12*=M12;
			M13 = (N11*N13 + N21*N23 + N31*N33 + N13 + N31); M13*=M13;
			M22 = (N12*N12 + N22*N22 + N32*N32 + 2*N22);     M22*=M22;
			M23 = (N12*N13 + N22*N23 + N32*N33 + N32 + N23); M23*=M23;
			M33 = (N13*N13 + N23*N23 + N33*N33 + 2*N33);     M33*=M33;    
      
			rigidSum += radiusScale*(M11 + 2*M12 + 2*M13 + M22 + 2*M23 + M33);

			float32 divE = 0;
			if (settings->volumeParam>0) {
				divE =  N11+N22+N33+N11*N22+N22*N33+N11*N33-N31*N13-N23*N32-N12*N21+N11*N22*N33+N12*N23*N31+N13*N21*N32-N11*N23*N32-N13*N22*N31-N12*N21*N33;
			}
			
			volumeSum += radiusScale*(divE*divE);			
		}
	}

	return settings->rigidityParam*rigidSum+settings->volumeParam*volumeSum;
}

float32 MMDeformationSolver::fCorrespondences() {
	//float32 constraintSum = 0;
	float32 constraintSum2 = 0;
	const card32 nbframes = context->nbFrames();
	
	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations * deformations = context->getDeformations(frame);
		if (!deformations->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;

		MMConstraints * constraints = context->getConstraints(frame);
		const card32 nbconstraints = constraints->nbPositionConstraints();
//		MMDeformations * deformations = context->getDeformations(frame);

		constraintSum2+=constraints->fCorrespondences(deformations, tempCorrespondenceParam);
		
		//correspondences: sum_j F(x) uj - u(x)
		/*for(card32 i=0; i<nbconstraints; i++){
			MMPositionConstraint * constraint = constraints->getPositionConstraint(i);		
			Matrix3f covariance = constraint->getCovariance();
			
			Vector3f sum; sum.setZero();
 			MMNeighborContainer * neighbors = constraint->getNeighbors();
			const card32 nbvalues = neighbors->nbNeighbors();
			for (card32 j=0; j< nbvalues; j++) {
				const card32 neighbor_index = neighbors->getNeighborIndex(j);
				const Vector3f &deformation = deformations->getDeformation(neighbor_index);
				sum+=deformation*neighbors->getShapeValue(j);				
		    }
			sum-=constraint->getDeformation();     
			     
			constraintSum += sum*(covariance*sum);
		}*/
	}
	
	//std::cout << "EQUAL? : " << constraintSum << " " << constraintSum2 << endl;
	return constraintSum2;
}

float32 MMDeformationSolver::fAcceleration() {
	if (settings->accelerationParam==0) return 0;
	// First compute current (real not ficitious) deformations of the nodes.
	// context->computeRealDeformations();

	float32 result = 0;
	// Now add up acceleration penalties
	const card32 nbframes = context->nbFrames();

	for (card32 frame=0; frame<nbframes; frame++) {

		const card32 index0 = frame==0?0:frame-1;
		const card32 index1 = frame;
		const card32 index2 = frame==nbframes-1?nbframes-1:frame+1;

		if (index0==index1 && index1==index2) continue;

		MMDeformations * deformations0 = context->getDeformations(index0);
		MMDeformations * deformations1 = context->getDeformations(index1);
		MMDeformations * deformations2 = context->getDeformations(index2);
		if (!deformations0->isActive()) continue;
		if (!deformations1->isActive()) continue;
		if (!deformations2->isActive()) continue;
		//if(!context->getFrameActive(index0) || !context->getFrameActive(index1) || !context->getFrameActive(index2))
		//	continue;
			
		const card32 nbnodes = context->nbNodes();
		for (card32 i=0; i < nbnodes; i++) {
			Vector3f acceleration = context->getDeformations(index0)->getRealDeformation(i)
								   -context->getDeformations(index1)->getRealDeformation(i)*2
								   +context->getDeformations(index2)->getRealDeformation(i);			
			result+=acceleration.getSqrNorm();
		}
		//std::cout << index0 << " " << index1 <<" " << index2 <<" " << result << endl;
	}
	
	return settings->accelerationParam*result;
}

float32 MMDeformationSolver::fVelocity() {
	if (settings->velocityParam==0) return 0;
	// First compute current (real not ficitious) deformations of the nodes.
	// context->computeRealDeformations();

	float32 result = 0;
	// Now add up velocity penalties
	const card32 nbframes = context->nbFrames();

	for (card32 frame=0; frame<nbframes; frame++) {
		const card32 index1 = frame;
		const card32 index2 = frame==nbframes-1?nbframes-1:frame+1;

		if (index1==index2) continue;

		MMDeformations * deformations1 = context->getDeformations(index1);
		if (!deformations1->isActive()) continue;
		MMDeformations * deformations2 = context->getDeformations(index2);
		if (!deformations2->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;
		
		const card32 nbnodes = context->nbNodes();
		for (card32 i=0; i < nbnodes; i++) {
			Vector3f velocity = deformations2->getRealDeformation(i)
								-deformations1->getRealDeformation(i);								 		
			result+=velocity.getSqrNorm();
		}
	}
	return settings->velocityParam*result;
}

float32 MMDeformationSolver::fHalfSpace() {
	/*float32 result = 0;
	// Now add up half space penalties
	const card32 nbconstraints = context->nbHalfSpaceConstraints();
	const card32 nbnodes = context->nbNodes();

	for (card32 k=0; k<nbconstraints; k++) {
		ARHalfSpaceConstraint * c = context->getHalfSpaceConstraint(k);
		const card32 frame = c->time;		

		if(!context->getFrameActive(frame))
			continue;

		for (card32 i=0; i < nbnodes; i++) {
			MMNode * node = context->getNodes()->getNode(i);
			Vector3f pos = node->getPos() + context->getDeformations(frame)->getRealDeformation(i);
			/// penalty = w*[(x-fp)*n]^2, if (x-fp)*n>0, zero otherwise
			float32 dot = (pos-c->halfSpaceFootPoint)*c->halfSpaceNormal;
			if (dot<0) continue;
			result+=c->penaltyWeight*dot*dot;
		}
	}
	return settings->halfSpaceParam*result;*/


	///


	float32 result = 0;
	const card32 nbframes = context->nbFrames();
	
	for (card32 frame=0; frame<nbframes; frame++) {
		MMConstraints * constraints = context->getConstraints(frame);
		const card32 nbconstraints = constraints->nbHalfSpaceConstraints();
		
		for(card32 i=0; i<nbconstraints; i++){
			MMHalfSpaceConstraint * constraint = constraints->getHalfSpaceConstraint(i);	
			const ARHalfSpaceConstraint * c = constraint->getConstraint();

			MMNeighborContainer * neighbors = constraint->getNeighbors();
			Vector3f pos = neighbors->getPos();
						
			const card32 nbvalues = neighbors->nbNeighbors();
			for (card32 j=0; j< nbvalues; j++) {
				const card32 neighbor_index = neighbors->getNeighborIndex(j);
				const Vector3f &deformation = context->getDeformations(frame)->getDeformation(neighbor_index);
				pos+=deformation*neighbors->getShapeValue(j);				
		    }

			/// penalty = w*[(x-fp)*n]^2, if (x-fp)*n>0, zero otherwise
			float32 dot = (pos-c->halfSpaceFootPoint)*c->halfSpaceNormal;
			if (dot<0) 
				continue;
			result+=c->penaltyWeight*dot*dot;						    			
		}
	}
	//debugOutput << "MMDeformationSolver::fHalfSpace() = " <<result<<"\n";
	return settings->halfSpaceParam*result;
}



void MMDeformationSolver::gRigidAndVolume(std::vector<Vector3f> &gradients) {
	const float32 _rigidityParam = settings->rigidityParam;
	const float32 _volumeParam = settings->volumeParam;

	const card32 nbframes = context->nbFrames();
	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations * deformations = context->getDeformations(frame);
		if (!deformations->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;


		//const card32 offset = context->getOffset(frame);
		const card32 nbnodes = context->nbNodes();
		for (card32 i=0; i < nbnodes; i++) {
			
			const float32 nodeRadius = context->getNodes()->getNode(i)->getRadius();
			float32 radiusScale = 0;

			float32 J11 = 0, J12 = 0, J13 = 0, J21 = 0,  J22 = 0, J23 = 0, J31 = 0, J32 = 0, J33 = 0;
			float32 sumAij = 0, sumBij = 0, sumCij = 0;

			const card32 real_index = context->getDeformations(frame)->getRealIndex(i);
			const bool active = context->getDeformations(frame)->isActive(i);
			const Vector3f &deformation = context->getDeformations(frame)->getDeformation(i);

			const float32 ui = deformation[0];
			const float32 vi = deformation[1];
			const float32 wi = deformation[2];

			MMNode * node = context->getNodes()->getNode(i);

			for (card32 j=0; j < node->nbNeighbors(); j++) {				
				const card32 neighbor_index = node->getNeighborIndex(j);
				
				const float32 neighborRadius = context->getNodes()->getNode(neighbor_index)->getRadius();
				
				const Vector3f &neighbor_deformation = context->getDeformations(frame)->getDeformation(neighbor_index);
				const float32 aij = node->getGradientShapeValues(j)[0];
				const float32 bij = node->getGradientShapeValues(j)[1];
				const float32 cij = node->getGradientShapeValues(j)[2];

				const float32 uj = neighbor_deformation[0];
				const float32 vj = neighbor_deformation[1];
				const float32 wj = neighbor_deformation[2];

				sumAij += aij;
				sumBij += bij;
				sumCij += cij;
      
				J11 += aij*(uj-ui);
				J12 += aij*(vj-vi);
				J13 += aij*(wj-wi);
				J21 += bij*(uj-ui);
				J22 += bij*(vj-vi);
				J23 += bij*(wj-wi);
				J31 += cij*(uj-ui);
				J32 += cij*(vj-vi);
				J33 += cij*(wj-wi);
				
				radiusScale += standardkernel(neighborRadius,node->getDistance(j));
			}
			
			if(fabs(radiusScale)>0.000001)
				radiusScale = nodeRadius*nodeRadius*nodeRadius/radiusScale;
			else
				radiusScale = 1;
			//radiusScale = 1;
			float32 divE=0;
			if (_volumeParam>0) {
				divE =  J11+J22+J33+J11*J22+J22*J33+J11*J33-J31*J13-J23*J32-J12*J21+J11*J22*J33+J12*J23*J31+J13*J21*J32-J11*J23*J32-J13*J22*J31-J12*J21*J33;
			}
			// This is rigidity
			float S11 = (J11*J11 + J21*J21 + J31*J31 + 2*J11);     
			float S12 = (J11*J12 + J21*J22 + J31*J32 + J12 + J21); 
			float S13 = (J11*J13 + J21*J23 + J31*J33 + J13 + J31); 
			float S22 = (J12*J12 + J22*J22 + J32*J32 + 2*J22);     
			float S23 = (J12*J13 + J22*J23 + J32*J33 + J32 + J23); 
			float S33 = (J13*J13 + J23*J23 + J33*J33 + 2*J33); 
			if (active) { // don't take derivatives to constants
				const float32 J11_u = -sumAij;
				const float32 J12_v = -sumAij;
				const float32 J13_w = -sumAij;
				const float32 J21_u = -sumBij;
				const float32 J22_v = -sumBij;
				const float32 J23_w = -sumBij;
				const float32 J31_u = -sumCij;
				const float32 J32_v = -sumCij;
				const float32 J33_w = -sumCij;
				
				if (_volumeParam>0) {
					// This is volume 
					gradients[real_index][0] += radiusScale*_volumeParam*2*divE*(J11_u+J22*J11_u+J33*J11_u-J13*J31_u-J12*J21_u+J22*J33*J11_u+J12*J23*J31_u+J13*J32*J21_u-J23*J32*J11_u-J13*J22*J31_u-J12*J33*J21_u);
					gradients[real_index][1] += radiusScale*_volumeParam*2*divE*(J22_v+J11*J22_v+J33*J22_v-J23*J32_v-J21*J12_v+J11*J33*J22_v+J23*J31*J12_v+J13*J21*J32_v-J11*J23*J32_v-J13*J31*J22_v-J21*J33*J12_v);
					gradients[real_index][2] += radiusScale*_volumeParam*2*divE*(J33_w+J11*J33_w+J22*J33_w-J31*J13_w-J32*J23_w+J11*J22*J33_w+J12*J31*J23_w+J21*J32*J13_w-J11*J32*J23_w-J22*J31*J13_w-J12*J21*J33_w);
				}
				    
	  
				gradients[real_index][0] += radiusScale*_rigidityParam*2*S11*(2*J11*J11_u+2*J21*J21_u+2*J31*J31_u+2*J11_u);
				gradients[real_index][1] += radiusScale*_rigidityParam*2*S22*(2*J12*J12_v+2*J22*J22_v+2*J32*J32_v+2*J22_v);
				gradients[real_index][2] += radiusScale*_rigidityParam*2*S33*(2*J13*J13_w+2*J23*J23_w+2*J33*J33_w+2*J33_w);

				gradients[real_index][0] += radiusScale*2*_rigidityParam*2*S12*(J12*J11_u+J22*J21_u+J32*J31_u+J21_u);
				gradients[real_index][1] += radiusScale*2*_rigidityParam*2*S12*(J11*J12_v+J21*J22_v+J31*J32_v+J12_v);

				gradients[real_index][0] += radiusScale*2*_rigidityParam*2*S13*(J13*J11_u+J23*J21_u+J33*J31_u+J31_u);
				gradients[real_index][2] += radiusScale*2*_rigidityParam*2*S13*(J11*J13_w+J21*J23_w+J31*J33_w+J13_w);

				gradients[real_index][1] += radiusScale*2*_rigidityParam*2*S23*(J13*J12_v+J23*J22_v+J33*J32_v+J32_v);
				gradients[real_index][2] += radiusScale*2*_rigidityParam*2*S23*(J12*J13_w+J22*J23_w+J32*J33_w+J23_w);
			}

			for (card32 j=0; j < node->nbNeighbors(); j++) {
				const card32 jindex = node->getNeighborIndex(j);
				const bool neighbor_active = context->getDeformations(frame)->isActive(jindex);		
				if (!neighbor_active) continue; // don't take derivatives to constants

				const card32 real_neighbor_index = context->getDeformations(frame)->getRealIndex(jindex);										
				const Vector3f &neighbor_deformation = context->getDeformations(frame)->getDeformation(jindex);
				const float32 aij = node->getGradientShapeValues(j)[0];
				const float32 bij = node->getGradientShapeValues(j)[1];
				const float32 cij = node->getGradientShapeValues(j)[2];

				const float J11_u = aij;
				const float J12_v = aij;
				const float J13_w = aij;
				const float J21_u = bij;
				const float J22_v = bij;
				const float J23_w = bij;
				const float J31_u = cij;
				const float J32_v = cij;
				const float J33_w = cij;

         
			// This is volume 
			 if (_volumeParam>0) {
				gradients[real_neighbor_index][0] += radiusScale*_volumeParam*2*divE*(J11_u+J22*J11_u+J33*J11_u-J13*J31_u-J12*J21_u+J22*J33*J11_u+J12*J23*J31_u+J13*J32*J21_u-J23*J32*J11_u-J13*J22*J31_u-J12*J33*J21_u);
				gradients[real_neighbor_index][1] += radiusScale*_volumeParam*2*divE*(J22_v+J11*J22_v+J33*J22_v-J23*J32_v-J21*J12_v+J11*J33*J22_v+J23*J31*J12_v+J13*J21*J32_v-J11*J23*J32_v-J13*J31*J22_v-J21*J33*J12_v);
				gradients[real_neighbor_index][2] += radiusScale*_volumeParam*2*divE*(J33_w+J11*J33_w+J22*J33_w-J31*J13_w-J32*J23_w+J11*J22*J33_w+J12*J31*J23_w+J21*J32*J13_w-J11*J32*J23_w-J22*J31*J13_w-J12*J21*J33_w);
			}
			// This is rigidity
			gradients[real_neighbor_index][0] += radiusScale*_rigidityParam*2*S11*(2*J11*J11_u+2*J21*J21_u+2*J31*J31_u+2*J11_u);
			gradients[real_neighbor_index][1] += radiusScale*_rigidityParam*2*S22*(2*J12*J12_v+2*J22*J22_v+2*J32*J32_v+2*J22_v);
			gradients[real_neighbor_index][2] += radiusScale*_rigidityParam*2*S33*(2*J13*J13_w+2*J23*J23_w+2*J33*J33_w+2*J33_w);

			gradients[real_neighbor_index][0] += radiusScale*2*_rigidityParam*2*S12*(J12*J11_u+J22*J21_u+J32*J31_u+J21_u);
			gradients[real_neighbor_index][1] += radiusScale*2*_rigidityParam*2*S12*(J11*J12_v+J21*J22_v+J31*J32_v+J12_v);

			gradients[real_neighbor_index][0] += radiusScale*2*_rigidityParam*2*S13*(J13*J11_u+J23*J21_u+J33*J31_u+J31_u);
			gradients[real_neighbor_index][2] += radiusScale*2*_rigidityParam*2*S13*(J11*J13_w+J21*J23_w+J31*J33_w+J13_w);

			gradients[real_neighbor_index][1] += radiusScale*2*_rigidityParam*2*S23*(J13*J12_v+J23*J22_v+J33*J32_v+J32_v);
			gradients[real_neighbor_index][2] += radiusScale*2*_rigidityParam*2*S23*(J12*J13_w+J22*J23_w+J32*J33_w+J23_w);

			}        
		}
	}
}

void MMDeformationSolver::gCorrespondences(std::vector<Vector3f> &gradients) {
	const card32 nbframes = context->nbFrames();
	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations * deformations = context->getDeformations(frame);
		if (!deformations->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;


		//card32 offset = context->getOffset(frame);
		MMConstraints * constraints = context->getConstraints(frame);		
		const card32 nbconstraints = constraints->nbPositionConstraints();
//		MMDeformations * deformations = context->getDeformations(frame);
		constraints->gCorrespondences(gradients, deformations, tempCorrespondenceParam);
		//correspondences: sum_j F(x) uj - u(x)
		/*for(card32 i=0; i<nbconstraints; i++){
			MMPositionConstraint * constraint = constraints->getPositionConstraint(i);		
			
			Vector3f sum; sum.setZero();
			MMNeighborContainer * neighbors = constraint->getNeighbors();
			const card32 nbvalues = neighbors->nbNeighbors();
			for (card32 j=0; j< nbvalues; j++) {
				
				const card32 neighbor_index = neighbors->getNeighborIndex(j);
				const Vector3f &deformation = deformations->getDeformation(neighbor_index);
				//std::cout << j << " " << nbvalues << " " << deformation.getSqrNorm() << endl;
				sum+=deformation*neighbors->getShapeValue(j);				
		    }
			sum-=constraint->getDeformation();  
			//std::cout << "Constraint deformation: " << constraint->getDeformation()[0] << " " <<constraint->getDeformation()[1] << " " << constraint->getDeformation()[1] << endl;
			for (card32 j=0; j< nbvalues; j++) {
				const card32 neighbor_index = neighbors->getNeighborIndex(j);
				const bool active = deformations->isActive(neighbor_index);
				
				const Matrix3f &covariance  = constraint->getCovariance();
				
				if (!active) continue;
				
				const card32 real_index = deformations->getRealIndex(neighbor_index);
				
				gradients[real_index] += covariance*sum*neighbors->getShapeValue(j)*tempCorrespondenceParam*2;
				//std::cout << _correspondenceParam << " " << sum.getSqrNorm() << " " << constraint->getShapeValue(j) << " " << gradients[offset+neighbor_index].getSqrNorm() << endl;
			}			
		}*/
	}	
}

void MMDeformationSolver::gAcceleration(std::vector<Vector3f> &gradients) {
	if (settings->accelerationParam==0) return;
	// First compute current (real not ficitious) deformations of the nodes.
	// context->computeRealDeformations();

	float32 result = 0;
	// Now add up acceleration penalties
	const card32 nbframes = context->nbFrames();
	for (card32 frame=0; frame<nbframes; frame++) {
		card32 index[3]; //card32 offset[3];
		index[0] = (frame==0)?frame:frame-1;
		index[1] = frame;
		index[2] = (frame==nbframes-1)?frame:frame+1;
		
		if (index[0]==index[1] && index[1]==index[2]) continue;

		MMDeformations * deformations0 = context->getDeformations(index[0]);
		MMDeformations * deformations1 = context->getDeformations(index[1]);
		MMDeformations * deformations2 = context->getDeformations(index[2]);
		if (!deformations0->isActive()) continue;
		if (!deformations1->isActive()) continue;
		if (!deformations2->isActive()) continue;


		
		//for (card32 k=0; k<3; k++) offset[k] = context->getOffset(index[k]);		
		const card32 nbnodes = context->nbNodes();
		for (card32 i=0; i < nbnodes; i++) {
			//const Vector3f acceleration = real_deformations[offset[0]+i] - real_deformations[offset[1]+i]*2 + real_deformations[offset[2]+i];
			Vector3f acceleration = deformations0->getRealDeformation(i)
								   -deformations1->getRealDeformation(i)*2
								   +deformations2->getRealDeformation(i);			
			// loop over the node's neighbors and take derivatives
			MMNode  * node = context->getNodes()->getNode(i);
		
			for (card32 j=0; j < node->nbNeighbors(); j++) {
				const card32 neighbor_index = node->getNeighborIndex(j);
				bool active[3];
				card32 realIndex[3];
				for (card32 k=0; k<3; k++) {
					active[k] = context->getDeformations(index[k])->isActive(neighbor_index);
					realIndex[k] = context->getDeformations(index[k])->getRealIndex(neighbor_index);
				}
				const float32 shapeValue = node->getShapeValue(j);
				const Vector3f temp = acceleration*settings->accelerationParam*2*shapeValue;
				if (active[0]) gradients[realIndex[0]] += temp;
				if (active[1]) gradients[realIndex[1]] += temp*(-2);
				if (active[2]) gradients[realIndex[2]] += temp;
			}			
		}
	}
}

void MMDeformationSolver::gVelocity(std::vector<Vector3f> &gradients) {
	if (settings->velocityParam==0) return;
	// First compute current (real not ficitious) deformations of the nodes.
	// context->computeRealDeformations();

	float32 result = 0;
	// Now add up acceleration penalties
	const card32 nbframes = context->nbFrames();
	for (card32 frame=0; frame<nbframes; frame++) {
		card32 index[3]; //card32 offset[3];
		index[0] = frame;
		index[1] = (frame==nbframes-1)?frame:frame+1;

		if (index[0]==index[1]) continue;

		MMDeformations * deformations0 = context->getDeformations(index[0]);
		MMDeformations * deformations1 = context->getDeformations(index[1]);
		if (!deformations0->isActive()) continue;
		if (!deformations1->isActive()) continue;
		//if(!context->getFrameActive(frame))
		//	continue;

		
		
		//for (card32 k=0; k<3; k++) offset[k] = context->getOffset(index[k]);		
		const card32 nbnodes = context->nbNodes();
		for (card32 i=0; i < nbnodes; i++) {
			//const Vector3f acceleration = real_deformations[offset[0]+i] - real_deformations[offset[1]+i]*2 + real_deformations[offset[2]+i];
			Vector3f velocity = context->getDeformations(index[1])->getRealDeformation(i)
								   -context->getDeformations(index[0])->getRealDeformation(i);		
			// loop over the node's neighbors and take derivatives
			MMNode  * node = context->getNodes()->getNode(i);
		
			for (card32 j=0; j < node->nbNeighbors(); j++) {
				const card32 neighbor_index = node->getNeighborIndex(j);
				bool active[2];
				card32 realIndex[2];
				for (card32 k=0; k<2; k++) {
					active[k] = context->getDeformations(index[k])->isActive(neighbor_index);
					realIndex[k] = context->getDeformations(index[k])->getRealIndex(neighbor_index);
				}
				const float32 shapeValue = node->getShapeValue(j);
				const Vector3f temp = velocity*settings->velocityParam*2*shapeValue;
				if (active[1]) gradients[realIndex[0]] += temp;
				if (active[0]) gradients[realIndex[1]] += temp*(-1);
			}			
		}
	}
}

void MMDeformationSolver::gHalfSpace(std::vector<Vector3f> &gradients) {
	/*float32 result = 0;
	// Now add up half space penalties
	const card32 nbconstraints = context->nbHalfSpaceConstraints();
	const card32 nbnodes = context->nbNodes();

	for (card32 k=0; k<nbconstraints; k++) {
		ARHalfSpaceConstraint * c = context->getHalfSpaceConstraint(k);
		const card32 frame = c->time;		

		if(!context->getFrameActive(frame))
			continue;

		MMDeformations * deformations = context->getDeformations(frame);
		for (card32 i=0; i < nbnodes; i++) {
			MMNode * node = context->getNodes()->getNode(i);
			Vector3f pos = node->getPos() + deformations->getRealDeformation(i);
			/// penalty = w*[(x-fp)*n]^2, if (x-fp)*n>0, zero otherwise
			float32 dot = (pos-c->halfSpaceFootPoint)*c->halfSpaceNormal;
			if (dot<0) continue;
			for (card32 j=0; j < node->nbNeighbors(); j++) {
				const card32 neighbor_index = node->getNeighborIndex(j);
				if (deformations->isActive(neighbor_index)) continue;
				gradients[neighbor_index]+= c->halfSpaceNormal*settings->halfSpaceParam*2*c->penaltyWeight*dot*node->getShapeValue(j);
			}
		}
	}*/
	Vector3f gradient_0_before=gradients[0];

	///
	const card32 nbframes = context->nbFrames();
	
	for (card32 frame=0; frame<nbframes; frame++) {
		MMConstraints * constraints = context->getConstraints(frame);
		const card32 nbconstraints = constraints->nbHalfSpaceConstraints();

		MMDeformations * deformations = context->getDeformations(frame);
		
		for(card32 i=0; i<nbconstraints; i++){
			MMHalfSpaceConstraint * constraint = constraints->getHalfSpaceConstraint(i);	
			const ARHalfSpaceConstraint * c = constraint->getConstraint();

			MMNeighborContainer * neighbors = constraint->getNeighbors();
			Vector3f pos = neighbors->getPos();
						
			const card32 nbvalues = neighbors->nbNeighbors();
			for (card32 j=0; j< nbvalues; j++) {
				const card32 neighbor_index = neighbors->getNeighborIndex(j);
				const Vector3f &deformation = deformations->getDeformation(neighbor_index);
				pos+=deformation*neighbors->getShapeValue(j);				
		    }

			/// penalty = w*[(x-fp)*n]^2, if (x-fp)*n>0, zero otherwise
			float32 dot = (pos-c->halfSpaceFootPoint)*c->halfSpaceNormal;
			if (dot<0) continue;

			for (card32 j=0; j < neighbors->nbNeighbors(); j++) {
				const card32 neighbor_index = neighbors->getNeighborIndex(j);
				//---
				if (!deformations->isActive(neighbor_index)) // changed -ab:  "if (deformations->isActive(neighbor_index))" 
					continue;
				//---
				gradients[neighbor_index]+= c->halfSpaceNormal*settings->halfSpaceParam*2*c->penaltyWeight*dot*neighbors->getShapeValue(j);
			}					    			
		}
	}
	//debugOutput << "MMDeformationSolver::gHalfSpace()norm(gradients[0]) = " << norm(gradients[0]-gradient_0_before) <<"\n";
}

DeformationEvaluator * MMDeformationSolver::createEvaluator() {
	return new MMDeformationEvaluator();
}

void X4::MMDeformationSolver::addHalfSpaceConstraint( const ARHalfSpaceConstraint *c )
{
   if (context->isIdentity()) return;
   int32 t = (int32)c->getTime();
   if (t >= (int32)context->nbFrames() || t<0) throw ERangeCheck("time out of range");
   Vector3f posAt = c->getAnchor().getPos();
   bool owner;
   MMNeighborContainer * neighbors = context->getNeighbors(c->getAnchor().getTopologicalAnchor(), posAt, owner);
   //std::cout << "constraint neighbors: " << neighbors->nbNeighbors() << endl;
   if (!neighbors->getShapeValuesInitialized()) {
	   bool success;
	   context->getNodes()->evaluateNodeShapeFunctions(neighbors, success);
   }
   MMHalfSpaceConstraint * constraint = new MMHalfSpaceConstraint((const ARHalfSpaceConstraint*)c->copy(), neighbors, owner);
   //if (owner) delete neighbors;
   //MMConstraint * constraint = new MMConstraint((const PositionARConstraint*)c->copy());
   /*context->getNodes()->computeNeighbors(constraint);
   bool success;
   context->getNodes()->evaluateNodeShapeFunctions(constraint, success);
   //for (card32 i=0; i<constraint->nbNeighbors(); i++) {
	   //std::cout << constraint->getNeighborIndex(i) << " " << constraint->getShapeValue(i) << endl;
   //}
   //std::cout << "Total: " << constraint->nbNeighbors() << endl;
   if (success) {
      context->getConstraints(t)->addConstraint(constraint);
   }
   else {
	//   std::cout << "Not adding constraint, shape function evaluation failed" << endl;
	   delete constraint;
   }*/
   context->getConstraints(t)->addHalfSpaceConstraint(constraint);
	//context->addHalfSpaceConstraint((ARHalfSpaceConstraint *)c->copy());
}

void MMDeformationSolver::addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c) {
   notImplemented();
}

void MMDeformationSolver::addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c) {
   notImplemented();
}


void MMDeformationSolver::beginConstraints() 
{
	context->clearConstraints();
}
