#include "StdAfx.h"
#include "MMDeformationSolverAdapter.h"


using namespace X4;


template <class BaseClass>
void MMDeformationSolverAdapter<BaseClass>::setx(){
	// first copy the pos to our actual unknows
	MMDeformationContext * context = solver->getContext();
	const card32 nbframes = context->nbFrames();
	const card32 nbnodes = context->nbNodes();

	for (card32 frame=0; frame<nbframes; frame++) {
		//if(!context->getFrameActive(frame))
		//	continue;
		MMDeformations* deformations = context->getDeformations(frame);
		if (!deformations->isActive()) continue;
		for (card32 i=0; i<nbnodes; i++) {
			const bool active = deformations->isActive(i);
			if (active) {
				Vector3f & deformation = deformations->getDeformation(i);
				const card32 realIndex = deformations->getRealIndex(i);
				for (card32 k=0; k<3; k++) deformation[k] = x[3*realIndex+k];
			}
		}
	} 			
}

template <class BaseClass>
void MMDeformationSolverAdapter<BaseClass>::setd(DVectorD *d){
	// first copy the pos to our actual unknows
	MMDeformationContext * context = solver->getContext();
	const card32 nbframes = context->nbFrames();
	const card32 nbnodes = context->nbNodes();
	
	context->createDDeformations();

	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations* deformations = context->getDDeformations(frame);
		if (!deformations->isActive()) continue;
		
		for (card32 i=0; i<nbnodes; i++) {
			const bool active = deformations->isActive(i);
			if (active) {
				Vector3f & ddeformation = deformations->getDeformation(i);
				const card32 realIndex = deformations->getRealIndex(i);
				for (card32 k=0; k<3; k++) ddeformation[k] = (*d)[3*realIndex+k];
			}
		}
	} 	
}

template <class BaseClass>
void MMDeformationSolverAdapter<BaseClass>::setAlpha(double alpha){
	MMDeformationContext * context = solver->getContext();
	context->setAlpha(alpha);
}

template <class BaseClass>
void MMDeformationSolverAdapter<BaseClass>::setx(DynamicVector<float32> &startEstimate){
	for(unsigned j=0; j<startEstimate.size(); j++){
		x[j] = startEstimate[j];
	}
}

template <class BaseClass>
void MMDeformationSolverAdapter<BaseClass>::F(){
	//std::cout << "Function start" << endl;
	setx();
		
	f = solver->func();
	//std::cout << "Function: " <<f << endl;
}

template <class BaseClass>
void MMDeformationSolverAdapter<BaseClass>::G(){
	//std::cout << "Gradient start" << endl;
	setx();

	std::vector<Vector3f> gradients;
	solver->grad(gradients);

	// copy gradient into the given std::vector
	for (card32 i=0; i<gradients.size(); i++) {
		g[3*i+0] = gradients[i][0];
		g[3*i+1] = gradients[i][1];
		g[3*i+2] = gradients[i][2];
		//if (gradients[i].getSqrNorm()>0) std::cout << gradients[i].getSqrNorm() << endl;
	}
	//std::cout << "Gradient end" << endl;
}

template <class BaseClass>
void MMDeformationSolverAdapter<BaseClass>::M(){
	//std::cout << "Gradient start" << endl;
	setx();

	std::vector<Vector3f> conditioner;
	solver->precon(conditioner);

	// copy gradient into the given std::vector
	for (card32 i=0; i<conditioner.size(); i++) {
		m[3*i+0] = conditioner[i][0];
		m[3*i+1] = conditioner[i][1];
		m[3*i+2] = conditioner[i][2];
		//if (gradients[i].getSqrNorm()>0) std::cout << gradients[i].getSqrNorm() << endl;
	}
	//std::cout << "Gradient end" << endl;
}


template <class BaseClass>
double MMDeformationSolverAdapter<BaseClass>::D2A(DVectorD *d, double alpha){
	//std::cout << "Gradient start" << endl;
	setx();
	
	setd(d);
	
	setAlpha(alpha);

	return solver->d2a();
}


template <class BaseClass>
void MMDeformationSolverAdapter<BaseClass>::result(const DynamicVector<float32> &pos) const { 
	// first copy the pos to our actual unknows
	MMDeformationContext * context = solver->getContext();
	const card32 nbframes = context->nbFrames();
	const card32 nbnodes = context->nbNodes();

	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations* deformations = context->getDeformations(frame);
		for (card32 i=0; i<nbnodes; i++) {
			const bool active = deformations->isActive(i);
			if (active) {
				Vector3f & deformation = deformations->getDeformation(i);
				const card32 realIndex = deformations->getRealIndex(i);
				for (card32 k=0; k<3; k++) deformation[k] = pos[3*realIndex+k];
			}
			//Vector3f & deformation = deformations->getDeformation(i);
			//for (card32 k=0; k<3; k++) deformation[k] = pos[3*frame*nbnodes+3*i+k];
		}
	} 		
}

template <class BaseClass>
float32 MMDeformationSolverAdapter<BaseClass>::eval(const DynamicVector<float32> &pos) const { 
	std::cout << "objective" << endl;
	// first copy the pos to our actual unknows
	result(pos);

	// get objective function
	float result = solver->func(); 		
	std::cout << "Objective function: " << result << endl;
	return result;
}
		
template <class BaseClass>
void MMDeformationSolverAdapter<BaseClass>::gradient(const DynamicVector<float32> &pos, DynamicVector<float32> &gradient) const {
	std::cout << "gradient" << endl;
	// first copy the pos to our actual unknows
	result(pos);

	eval(pos);

	// get gradient
	std::vector<Vector3f> gradients;
	solver->grad(gradients);

	// copy gradient into the given std::vector
	for (card32 i=0; i<gradients.size(); i++) {
		gradient[3*i+0] = gradients[i][0];
		gradient[3*i+1] = gradients[i][1];
		gradient[3*i+2] = gradients[i][2];
		//if (gradients[i].getSqrNorm()>0) std::cout << gradients[i].getSqrNorm() << endl;
	}
	std::cout << "Gradient norm: " << norm(gradient) << endl;
	//int bla; cin >> bla;
}

/// explicit instantiation (no need to include the code then)
template MMDeformationSolverAdapter<UnconstrainedOptim>;
template MMDeformationSolverAdapter<MMSimpleCG>;
template MMDeformationSolverAdapter<MMLBFGS>;
