#ifndef MMDeformationSolverAdapterH
#define MMDeformationSolverAdapterH

#include "x3deformation.h"
#include "NonLinearIterativeSolvers.h"
#include "MMDeformationSolver.h"
#include "uncoptim.h"
#include "MMSimpleCG.h"
#include "MMLBFGS.h"

namespace X4 {

	template <class BaseClass>
	class MMDeformationSolverAdapter : public ObjectiveFunctionF, public BaseClass {
	private:
	
		MMDeformationSolver * solver;

	public:

		MMDeformationSolverAdapter(MMDeformationSolver * s, int32 numUnknowns) : solver(s), BaseClass(numUnknowns,0,0) {}
		
		//compute the function value and store it in the local variable f
		void F();
		
		//compute the function value and store it in the local variable g (array of doubles)
		void G();
		
		//compute the diagonal preconditioner and store it in the local variable m (array of doubles)
		void M();
		
		//compute the approximate second order derivative d^2 f(x + alpha*d)/ dalpha^2 and store it in the local variable d2a (double)
		double D2A(DVectorD *d, double alpha);
		 
		void setx();
		
		void setd(DVectorD *d);
		
		void setAlpha(double alpha);
		
		void setx(DynamicVector<float32> &startEstimate);
		

		void result(const DynamicVector<float32> &pos) const;

		virtual float32 eval(const DynamicVector<float32> &pos) const;
		
		virtual void gradient(const DynamicVector<float32> &pos, DynamicVector<float32> &gradient) const;
	};
	
	typedef MMDeformationSolverAdapter<UnconstrainedOptim> MMDeformationSolverAdapter_BFGS;
	typedef MMDeformationSolverAdapter<MMLBFGS> MMDeformationSolverAdapter_LBFGS;
	typedef MMDeformationSolverAdapter<MMSimpleCG> MMDeformationSolverAdapter_NLCG;
	
}

#endif