//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MMFurthestPointSampler.h"
#include "UnstructuredInCorePointCloud.h"
#include "ARSettings.h"
#include "MMNodes.h"
#include "PointWithDist.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( MMFurthestPointSampler , 0 ) {
	BEGIN_CLASS_INIT( MMFurthestPointSampler );
}

bool MMFurthestPointSampler_compare_xyz(PointWithDist &p1, PointWithDist &p2) {
      if (p1.pos[0]<p2.pos[0]) return true;
	  else if (p1.pos[0]==p2.pos[0]) {
		  if (p1.pos[1]<p2.pos[1]) return true;
		  else if (p1.pos[1]==p2.pos[1]) {
			  return p1.pos[2]<p2.pos[2];
		  }
		  else return false;
	  }
	  else return false;
}

bool MMFurthestPointSampler_compare_dist(PointWithDist &p0, PointWithDist &p1) {
     return p0.sqrDist>p1.sqrDist;
}

MMNodes* MMFurthestPointSampler::createNodes(const ARSettings *settings, UnstructuredInCorePointCloud * geometry) {
	std::vector<Vector3f> nodes;
	float32 radius = settings->nodalRadius;
	// get the vertex with the smallest (x,y,z) coordinate
	// make this a node
	//const UnstructuredInCorePointCloud *pc = dynamic_cast<const UnstructuredInCorePointCloud*>(urshape);
	AAT posAAT = geometry->getAAT("position");
	AAT normalAAT = geometry->getAAT("normal");
	const PointSet *ps = geometry->getPointSet();
	const card32 nbpoints = ps->getNumEntries();

	std::vector<PointWithDist> uncoveredPoints;
	for (card32 i=0; i<nbpoints; i++) {
		uncoveredPoints.push_back(PointWithDist(ps->get3f(i, posAAT), ps->get3f(i, normalAAT), 100000.0f));
	}
	sort(uncoveredPoints.begin(), uncoveredPoints.end(), MMFurthestPointSampler_compare_xyz);

	while (!uncoveredPoints.empty()) {
		// create new node from first position
		const Vector3f &newpos = uncoveredPoints[0].pos;
		const Vector3f &normal = uncoveredPoints[0].normal;
		nodes.push_back(newpos);
		nodes.push_back(newpos + normal*(0.5f*radius));
		nodes.push_back(newpos - normal*(0.5f*radius));
		// go over all points and update dist and add if not too close to new node
		std::vector<PointWithDist> temp;
		for (card32 i=0; i<uncoveredPoints.size(); i++)  {
			PointWithDist &p = uncoveredPoints[i];
			float32 sqrDist = (p.pos-newpos).getSqrNorm();
			if (sqrDist>radius*radius) {
				p.sqrDist = p.sqrDist < sqrDist?p.sqrDist:sqrDist;
				temp.push_back(p);
			}
		}
		uncoveredPoints = temp;
		sort(uncoveredPoints.begin(), uncoveredPoints.end(), MMFurthestPointSampler_compare_dist);
	}
	MMNodes * result = new MMNodes(nodes, 2.5*radius);
	return result;
}