//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MMGridSampler.h"
#include "UnstructuredInCorePointCloud.h"
#include "ARSettings.h"
#include "MMNodes.h"
#include "SparseGrid.h"
#include <cmath>
#include <cfloat>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( MMGridSampler , 0 ) {
	BEGIN_CLASS_INIT( MMGridSampler );
}



MMNodes* MMGridSampler::createNodes(const ARSettings *settings, UnstructuredInCorePointCloud * geometry) {
	PointSet * ps = geometry->getPointSet();
	BoundingBox3f bb = ps->getBoundingBox();

	Vector3f m = bb.lowerCorner;
	Vector3f M = bb.upperCorner;

	const uint dimension = settings->gridSize;

	float32 maxLength = bb.getMaxSideLength();
	float32 scale = 1.0/maxLength;
	float32 cellsize = maxLength/float32(dimension-1);

	// to avoid geometry coinciding with the cell boundaries
	// gives nicer samplings for planes etc.
	m = m - makeVector3f(cellsize/2, cellsize/2, cellsize/2);

	// Conversion from 3D positions to grid coordinates
	// x <- (x-m)*scale; this is in [0,1]
	// x <- x*(dimension-1)+1; this is in [1, d]
	// x <- round(x); this is to its nearest cell

	SparseGrid<3, bool, Object> grid;
	std::vector<Vector3f> positions;
	const card32 numPoints = ps->getNumEntries();

	AAT posAAT = ps->getAAT("position");
	for (card32 i=0; i<numPoints; i++) {
		const Vector3f &p = ps->get3f(i, posAAT);
		Vector3f index = (p-m)*scale;
		index = index*(dimension-1)+makeVector3f(1,1,1);
		for (card32 k=0; k<3; k++) index[k] = floor(index[k]+0.5);
		Vector3f indexPos = (index-makeVector3f(1,1,1))*(1.0f/float32(dimension-1));
		indexPos = indexPos*(1.0/scale)+m;
		bool entry;
		const int startx = indexPos[0]<p[0]?0:-1;
		const int starty = indexPos[1]<p[1]?0:-1;
		const int startz = indexPos[2]<p[2]?0:-1;
		const int endx = startx+1;
		const int endy = starty+1;
		const int endz = startz+1;
		for (int dx=startx; dx<=endx; dx++) {
			for (int dy=starty; dy<=endy; dy++) {
				for (int dz=startz; dz<=endz; dz++) {					
					const Vector3i cellIndex = makeVector3i(index[0]+dx,index[1]+dy,index[2]+dz);
					const Vector3f cellPos = indexPos+makeVector3f(dx*cellsize, dy*cellsize, dz*cellsize);
					const bool alreadyThere = grid.getObject(cellIndex, entry);
					//std::cout << alreadyThere << endl;
					if (!alreadyThere) {			
						entry = true;
						positions.push_back(cellPos);
						//std::cout << "Adding node: " << cellPos[0] << " " << cellPos[1] << " " << cellPos[2] << endl;
						grid.setObject(cellIndex, entry);
					}
				}
			}
		}		
	}
	MMNodes * nodes = new MMNodes(positions, 2.5*cellsize);
	return nodes;
}