#ifndef MMNeighborContainerH
#define MMNeighborContainerH

#include "x3deformation.h"
#include <vector>
#include "NeighborInformation.h"
#include <assert.h>

namespace X4 {

	class MMNodes;


class MMNeighborContainer : public Persistent {
   X4_CLASS(MMNeighborContainer)
 private:
   std::vector<NeighborInformation> neighbors;
   Vector3f pos;
   card32 baseIndex;

   bool shapeValuesInitialized;
   bool gradientShapeValuesInitialized;

  //   IMPLEMENT_VAR_ARRAY_METHODS_STL(neighbors)	

 public:
	 MMNeighborContainer() : baseIndex(-1), shapeValuesInitialized(false), gradientShapeValuesInitialized(false) {
   }
	MMNeighborContainer(const Vector3f &p) : pos(p), baseIndex(-1), shapeValuesInitialized(false), gradientShapeValuesInitialized(false) {}
	MMNeighborContainer(const Vector3f &p, const card32 b) : pos(p), baseIndex(b), shapeValuesInitialized(false), gradientShapeValuesInitialized(false) {}

	virtual ~MMNeighborContainer() {
		neighbors.clear();
	}

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

	Vector3f& getPos() { return pos; }
	const Vector3f& getPos() const { return pos; }

	mpcard nbNeighbors() const {
		return neighbors.size();
	}

	void clearNeighbors() {
		neighbors.clear();
	}

	void addNeighbor(card32 index, float32 distance, float32 weight, const Vector3f &gradientWeight) {
		if(weight<0)
			std::cout << "Negative weight for index " << index << " weight = " << weight << endl;
			
		neighbors.push_back(NeighborInformation(index, distance, weight, gradientWeight));
	}

	void setShapeValue(card32 index, float32 value) {
		neighbors[index].shapeValue = value;
	}

	void setGradientShapeValue(card32 index, const Vector3f &value) {
		neighbors[index].gradientShapeValue = value;
	}

	void setGradientShapeValue2(card32 index, const Vector3f &value) {
		neighbors[index].gradientShapeValue2 = value;
	}

	void setShapeValuesInitialized() {
		shapeValuesInitialized = true;
	}

	void setGradientShapeValuesInitialized() {
		gradientShapeValuesInitialized = true;
	}

	bool getShapeValuesInitialized() const {
		return shapeValuesInitialized;
	}

	bool getGradientShapeValuesInitialized() const {
		return gradientShapeValuesInitialized;
	}
	
	card32 getNeighborIndex(card32 index) const {
		return neighbors[index].index;
	}

	float32 getWeight(card32 index) const {
		return neighbors[index].weight;
	}

	float32 getDistance(card32 index) const {
		return neighbors[index].distance;
	}

	const Vector3f &getGradientWeight(card32 index) const {
		return neighbors[index].gradientWeight;
	}

	float32 getShapeValue(card32 index) const {
		return neighbors[index].shapeValue;
	}

	const Vector3f& getGradientShapeValues(card32 index) const {
		//std::cout << neighbors.size() << " " << index << " " << neighbors[index].gradientShapeValue[0] << " " <<  neighbors[index].gradientShapeValue[1] << " " << neighbors[index].gradientShapeValue[2]<< endl;
		assert(index < neighbors.size());
		return neighbors[index].gradientShapeValue;
	}	

	const Vector3f& getGradientShapeValues2(card32 index) const {
		//std::cout << neighbors.size() << " " << index << " " << neighbors[index].gradientShapeValue[0] << " " <<  neighbors[index].gradientShapeValue[1] << " " << neighbors[index].gradientShapeValue[2]<< endl;
		assert(index < neighbors.size());
		return neighbors[index].gradientShapeValue2;
	}

	void clearShapeValues(const Vector3f &pos, MMNodes *nodes);

	void setBaseIndex(card32 index) { baseIndex = index; }
	card32 getBaseIndex() { return baseIndex; }
};


}

#endif
