#ifndef MMNodeH
#define MMNodeH

#include "x3deformation.h"
#include <vector>
#include "MMNeighborContainer.h"


namespace X4 {


/**
  * Holds for a node, the indices of its neighbors,
  * together with the shape functions evaluated at this node
  * together with the coefficients for the gradient of the displacement
  * field evaluated at this node. We cache this information for efficient
  * solving.
  */
class X3DEFORMATION_API MMNode : public MMNeighborContainer {
   X4_CLASS( MMNode )

 private:
	float32 radius;
	std::set<card32> reachableBaseIndices;
	
 public:
   MMNode() {radius = 0;}
   MMNode(const Vector3f &p, const float32 &r) : MMNeighborContainer(p), radius(r) {}
   MMNode(const Vector3f &p, const card32 baseIndex, const float32 &r) : MMNeighborContainer(p, baseIndex), radius(r) {}

   std::set<card32> &getReachableBaseIndices() { return reachableBaseIndices; }

   //virtual ~MMNode() {}

	float32& getRadius() {
		return radius;
	}
};

}

#endif