//---------------------------------------------------------------------------
#ifndef MMNodesH
#define MMNodesH
//---------------------------------------------------------------------------
#include "x3deformation.h"
#include "UnstructuredInCorePointCloud.h"
#include "PointSetSphericalQuery.h"
//---------------------------------------------------------------------------
#include "MMNode.h"
#include <vector>
#include <set>
#include "MMTopology.h"
//---------------------------------------------------------------------------

namespace X4 {


/**
  * Holds for every node, additional information which
  * holds the node's neighbors and the evaluated shape functions
  * and gradient coefficients.
  */
class X3DEFORMATION_API MMNodes : public Persistent {
	X4_CLASS( MMNodes )

 private:
	std::vector<MMNode*> nodes;
	UnstructuredInCorePointCloud * pointcloud;
	AttachedIndexedOctree * octree;
	PointSetSphericalQuery * query;
    PointSet *queryPS; // mw - added (fixed a memory leak: ps is not owned by query, i.e. will not be deleted)
	float32 maxRadius;
	float32 averageRadius;
	MMTopology * topology;
	

   IMPLEMENT_VAR_ARRAY_METHODS_STL(nodes)	


	void createOctree();
 public:
	MMNodes();
	~MMNodes();

	MMNodes(UnstructuredInCorePointCloud * pc, float32 radius);
	MMNodes(std::vector<Vector3f> &positions, float32 radius);
	MMNodes(std::vector<Vector3f> &positions, std::vector<card32> &baseIndices, float32 radius);
	MMNodes(std::vector<Vector3f> &positions, std::vector<card32> &baseIndices, std::vector<float32> &radii);
	MMNodes(const MMNodes &other); // mw - added to fix init bug

	mpcard nbNodes() const {
		return nodes.size();
	}

	void addNode(MMNode * ni) {
		nodes.push_back(ni);
	}

	MMNode* getNode(card32 index) {
		return nodes[index];
	}	

	// compute node neighbors, evaluate shape functions, etc.
   void initializeNodes();

   UnstructuredInCorePointCloud* getPointCloud() {
	   return pointcloud;
   }

   PointSetSphericalQuery * getQuery() { return query; }

   void setTopology(MMTopology * topology) { this->topology = topology; }

	// Compute node neighbors
	void computeNodeNeighbors();
	// For each node, compute the shape functions (normal and gradient)
	void evaluateNodeShapeFunctions();
	// Normal shape functions for the point x
	void evaluateNodeShapeFunctions(MMNeighborContainer * node, bool &success);
	// Gradient shape functions for the ponit x
	void evaluateNodeGradientShapeFunctions(MMNeighborContainer * node, bool &success);
	// compute the neighbors
	void computeNeighbors(card32 index);
	// also for general neighborcontainers
	void computeNeighbors(MMNeighborContainer* node);

	card32 getClosestBaseIndex(const Vector3f &pos) {
		if (!topology) return -1;
		else return topology->getClosestBaseIndex(pos);
	}

   


};

}

#endif