#ifndef MeshlessCachedConstraintH
#define MeshlessCachedConstraintH

#include "x3deformation.h"
#include "ARConstraints.h"

#include <vector>

namespace X4 {

/**
  * Holds for a constraint anchor point position, the indices of the nodal neighbors,
  * together with the shape functions evaluated at the anchor point.
  * We cache this information for efficient solving.
  */
class MeshlessCachedConstraint {
	private: 
		std::vector<card32> neighborIndices;
		std::vector<float32> shapeValues;
		const PositionARConstraint * constraint;

	public:

		MeshlessCachedConstraint(const PositionARConstraint *c) : constraint(c) {}

		mpcard nbNeighbors() const {
			return neighborIndices.size();
		}

		void addNeighbor(card32 index) {
			neighborIndices.push_back(index);
		}

		void addShapeValue(float32 value) {
			shapeValues.push_back(value);
		}
	
		card32 getNeighborIndex(card32 index) const {
			return neighborIndices[index];
		}

		float32 getShapeValue(card32 index) const {
			return shapeValues[index];
		}

		Vector3f getDeformation() const {
			const Vector3f &posAt = constraint->getAnchor().getPos();
			const Vector3f &posTo = constraint->getToPoint();
			return posTo-posAt;
		}
	
};


}


#endif