//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "RWDeformationContext.h"
//---------------------------------------------------------------------------
#include "RWDeformationSolver.h"
#include "RWDeformationEvaluator.h"
#include "ARSettings.h"
#include "DeformationTools.h"
#include "InCorePCTopologyGraph.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( RWDeformationContext , 0 ) {
	BEGIN_CLASS_INIT( RWDeformationContext );
}

// ----


RWDeformationContext::RWDeformationContext() 
{
   urshape = NULL;
}

RWDeformationContext::~RWDeformationContext() {
}

DeformationSolver *RWDeformationContext::createSolver() 
{
   return new RWDeformationSolver();
}

DeformationEvaluator *RWDeformationContext::createEvaluator() 
{
   return new RWDeformationEvaluator();
}

void RWDeformationContext::initializeFromShape(const SceneObject *obj, Vector2f timeInterval, const ARSettings *settings)
{
   startTime = (int32)timeInterval[0];
   endTime = (int32)timeInterval[1];   
   if (startTime != timeInterval[0] || endTime != timeInterval[1])
   {
      warning("RWDeformationContext::initializeFromShape - requires integral time; ignoring fractional values.");
   }
   SceneObject *ucObj = const_cast<SceneObject*>(obj);
   UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(ucObj);
   InCorePCTopologyGraph *tg = getTopology(upc);
   if (!upc || !tg) {
      throw X4Exception("RWDeformationContext::initializeFromShape - need unstructured pc with topology.");
   }
   delete urshape;
   urshape = new UnstructuredInCorePointCloud();
   urshape->setPointSet((PointSet*)upc->getPointSet()->copy());
   attachTopology(urshape, (InCorePCTopologyGraph*)tg->copy());
}

float32 RWDeformationContext::getDuration() 
{
   return endTime - startTime + 1;
}

float32 RWDeformationContext::getBaseResolution() 
{
   return 0;//settings->tsGridSpacing;
}


