//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "RWDeformationDiscretization.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ARTopologicalResampling.h"
#include "RWDeformationGlobal.h"
#include "InCorePCTopologyGraph.h"
#include "DeformationTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS(RWDeformationDiscretization, 0)
{
	BEGIN_CLASS_INIT(RWDeformationDiscretization)
}

//---------------------------------------------------------------------------

RWDeformationDiscretization::RWDeformationDiscretization()
{
   controlPoints = NULL;
   topology = NULL;
}

RWDeformationDiscretization::~RWDeformationDiscretization() 
{
   delete topology;
   delete controlPoints;
}

void RWDeformationDiscretization::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj, this);
}

//---------------------------------------------------------------------------

void RWDeformationDiscretization::clearAndSetup(const UnstructuredInCorePointCloud *ptsWTop, float32 eps) 
{
   UnstructuredInCorePointCloud *base = 
      ARTopologicalResampling::resample((UnstructuredInCorePointCloud*)ptsWTop, eps, RW_DEFORMATION_REL_TOP_RADIUS, RW_DEFORMATION_TOP_KNN);
   mpcard numPts = base->getPointSet()->getNumEntries();
   delete controlPoints;
   controlPoints = new PointSet();
   VertexDescriptor vd;
   vd.pushAttrib( mVAD("position",         3, VAD::DATA_FORMAT_FLOAT32) );
   vd.pushAttrib( mVAD("weight",           1, VAD::DATA_FORMAT_FLOAT32) );
   vd.pushAttrib( mVAD("localRigidTransf", 9, VAD::DATA_FORMAT_FLOAT32) );
   positionAAT = vd.getToken("position");
   weightAAT = vd.getToken("weight");
   localRigidTransfAAT = vd.getToken("localRigidTransf");
   controlPoints->clearAndSetup(1, numPts, &vd);
   AAT basePositionAAT = base->getAAT("position");
   for (mpcard i=0; i<numPts; i++) {
      controlPoints->set3f(i, positionAAT, base->getPointSet()->get3f(i, basePositionAAT));
      controlPoints->set1f(i, weightAAT, 1.0f);
      controlPoints->set9f(i, localRigidTransfAAT, IDENTITY3F);
   }
   topology = (InCorePCTopologyGraph*)(getTopology(base)->copy());
   delete base;
}

UnstructuredInCorePointCloud *RWDeformationDiscretization::createPC() 
{
   UnstructuredInCorePointCloud *result = new UnstructuredInCorePointCloud();
   result->setPointSet((PointSet*)controlPoints->copy());
   attachTopology(result, (InCorePCTopologyGraph*)topology->copy());
   return result;
}
