//---------------------------------------------------------------------------
#ifndef RWDeformationSolverH
#define RWDeformationSolverH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "DeformationSolver.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {
class RWDeformationContext;


///   mw - 02/03/2010
///
///   Elastic deformation solver with iteratively reweighted least squares
///
class X3DEFORMATION_API RWDeformationSolver : public DeformationSolver {
	X4_CLASS( RWDeformationSolver )
 private:
   std::vector<Delegate> callbacks;
	RWDeformationContext *context;
	const ARSettings *settings;

	
 protected:
   void callCallbacks();
   
 public:
	RWDeformationSolver();

	virtual DeformationEvaluator *createEvaluator();
	AROptimizationContext *createOptContext();
	virtual void setup(AROptimizationContext *context, const ARSettings *settings);

	virtual void addUserFeedbackCallback(Delegate d);
	virtual void clearUserFeedbackCallbacks();

	virtual void beginConstraints();
   virtual void addPositionConstraint(const PositionARConstraint *c);
	virtual void addHalfSpaceConstraint(const ARHalfSpaceConstraint *c);
   virtual void addVelocityConstraint(const VelocityVectorARConstraint *c);
   virtual void addInstantaneousVelocityConstraint(const InstantaneousVelocityARConstraint *c);
   virtual void addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c);
   virtual void addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c);
   
	virtual void solve();


	~RWDeformationSolver();
};

}

#endif