//---------------------------------------------------------------------------
#ifndef RigidDeformationSolverH
#define RigidDeformationSolverH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "DeformationSolver.h"
//---------------------------------------------------------------------------

namespace X4 {

class RigidDeformationContext;


///
///   simple deformation solver that just computes the best-fitting
///   rigid transformation out of the given position constraints
///   (ignoring velocity and silhouette constraints, as well as covariance matrices)
///
class X3DEFORMATION_API RigidDeformationSolver : public DeformationSolver {
	X4_CLASS( RigidDeformationSolver )
 private:
	RigidDeformationContext *context;
	std::vector< std::vector< pair<Vector3f, Vector3f> > > constraints;
 public:
	static Matrix4f calculateTransformation(const std::vector<pair<Vector3f, Vector3f> > &pairs); 

	RigidDeformationSolver();

	virtual AROptimizationContext *createOptContext();
	virtual void setup(AROptimizationContext *context, const ARSettings *settings);
	virtual DeformationEvaluator *createEvaluator();

	virtual void beginConstraints();
   virtual void addPositionConstraint(const PositionARConstraint *c);
	virtual void addHalfSpaceConstraint(const ARHalfSpaceConstraint *c);
   virtual void addVelocityConstraint(const VelocityVectorARConstraint *c);
   virtual void addInstantaneousVelocityConstraint(const InstantaneousVelocityARConstraint *c);
   virtual void addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c);
   virtual void addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c);
	virtual void solve();

	virtual void addUserFeedbackCallback(Delegate d) {}
	virtual void clearUserFeedbackCallbacks() {}

   ~RigidDeformationSolver();
};

}

#endif