//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "TSDeformationSolver.h"
//---------------------------------------------------------------------------
#include "SGEVolumeGridDeformationSequence.h"
#include "SGEPointCloud.h"
#include "RegularVolume.h"
#include "RegularVolume.inline.h"
#include "RegularInCoreVolume.h"
#include "DynamicLinearAlgebraTools.h"
#include "DynamicLinearAlgebraTools.inline.h"
#include "BoundingBox.h"
#include "BoundingBox.inline.h"
#include "IterativeSolvers.h"
#include "IterativeSolvers.inline.h"
#include "ARSettings.h"
#include "ARConstraints.h"
#include "TSDeformationContext.h"
#include "TSDeformationEvaluator.h"
#include "Timer.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( TSDeformationSolver ,0)
{
	BEGIN_CLASS_INIT( TSDeformationSolver );
}

TSDeformationSolver::TSDeformationSolver()
{
	context = NULL;
	settings = NULL;
}

TSDeformationSolver::~TSDeformationSolver() {}



AROptimizationContext *TSDeformationSolver::createOptContext() {
	return new TSDeformationContext();
}

void TSDeformationSolver::setup(AROptimizationContext *context, const ARSettings *settings) {
	if (!dynamic_cast<TSDeformationContext*>(context)) {
		throw ETypeMismatch("TSDeformationSolver::setup() - context does not have the correct dynamic type.");
	}
	this->context = (TSDeformationContext*)context;
	this->settings = settings;
}

void TSDeformationSolver::solveGroup(GroupDeformationContext *groupContext, std::vector<DeformationSolver*> group) {
 //  group.push_back(this);
 //  
 //  std::vector<mpcard> dimensions(linTerms.size(), 0);
 //  std::vector<SparseMatrixF> matrices;
 //  std::vector<DVectorF> rhSides;
 //  size_t numTimeSteps = linTerms.size());

 //  for (size_t t=0; t<numTimeSteps; t++) {
 //     for (size_t solverIndex=0; solverIndex<group.size(); solverIndex++) {
 //        TSDeformationSolver *otherSolver = dynamic_cast<TSDeformationSolver*>(group[solverIndex]);
 //        if (!otherSolver) {
 //           throw ETypeMismatch("TSDeformationSolver::solveGroup() - all solvers need to be of type TSDeformationSolver.");
 //        }
 //        x4Assert(otherSolver->linTerms.size() == numTimeSteps);
 //        if (numTimeSteps != otherSolver->linTerms.size()) {
 //           throw ERangeCheck("TSDeformationSolver::solveGroup() - this works only if all solvers have the same number of time steps.");
 //        }
 //        for (size_t t=0; t<numTimeSteps; t++) {
 //           dimensions[t] += otherSolver->linTerms[t].size();
 //        }
 //     }
 //  }   


 //  for (size_t solverIndex=0; solverIndex<group.size(); solverIndex++) {
 //     TSDeformationSolver *otherSolver = dynamic_cast<TSDeformationSolver*>(group[solverIndex]);
	//   if(otherSolver->settings->tsCachedCellMatrices) {
	//	   for( unsigned i=0; i<otherSolver->context->getDuration(); i++) {
	//		   otherSolver->context->transferCachedPositionConstraints(otherSolver->context->getStartTime() + i, otherSolver->quadTerms[i]);
	//	   }
	//   }
	//}
 //     
 //  size_t numTimeSteps = otherSolver->quadTerms.size();
 //  x4Assert(numTimeSteps == otherSolver->linTerms.size());
 //  for (size_t t=0; t<numTimeSteps; t++) {
 //     for (size_t solverIndex=0; solverIndex<group.size(); solverIndex++) {
 //        TSDeformationSolver *otherSolver = dynamic_cast<TSDeformationSolver*>(group[solverIndex]);
 //        DVectorF result;
 //        otherSolver->context->getLastResult(t-otherSolver->context->getStartTime(), result);
 //        card32 numIterations = otherSolver->settings->numCGIterations;
 //        conjugateGradientsSolve<float32>(
 //           otherSolver->quadTerms[t],
 //           result,
 //           otherSolver->linTerms[t],
 //           numIterations,
 //           otherSolver->settings->convThreshCG,
 //           otherSolver->settings->mmDebugOutput
 //        );
 //        otherSolver->context->setResult(t-context->getStartTime(), result);


 //        // xxx distribute matrices
 //     }
 //     otherSolver->quadTerms = std::vector<SparseMatrixF>();
 //     otherSolver->linTerms = std::vector<DVectorF>();
 //  }
}


void TSDeformationSolver::beginConstraints() 
{
   quadTerms = std::vector<SparseMatrixF>();
   linTerms = std::vector<DVectorF>();
   
   int32 numTimeSteps = context->getNumTimeSteps();
   quadTerms.resize(numTimeSteps);
   linTerms.resize(numTimeSteps);
   
   for (int32 i=0; i<numTimeSteps; i++) {
      context->getRegularizer(context->getStartTime()+i, quadTerms[i], linTerms[i]);
   }
}


void TSDeformationSolver::addPositionConstraint(const PositionARConstraint *c) {
   int32 t = (int32)c->getTime();
   if (t > context->getEndTime() || t<context->getStartTime()) throw ERangeCheck("time out of range");
   card32 index = t - context->getStartTime();

	if( !settings->tsCachedCellMatrices )
		context->addPositionConstraint(t, c, quadTerms[index], linTerms[index]);
	else
		context->addCachedPositionConstraint(t,c,quadTerms[index], linTerms[index]);
}

void TSDeformationSolver::addVelocityConstraint(const VelocityVectorARConstraint *c)
{
   notImplemented();
}

void TSDeformationSolver::addInstantaneousVelocityConstraint(const InstantaneousVelocityARConstraint *c) 
{
   notImplemented();
}

void TSDeformationSolver::addHalfSpaceConstraint( const ARHalfSpaceConstraint *c )
{
   notImplemented();
}

void TSDeformationSolver::addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c)
{
   int32 t = (int32)c->getTime();
   if (t > context->getEndTime() || t<context->getStartTime()) throw ERangeCheck("time out of range");
   card32 index = t - context->getStartTime();

   context->addSpatialDerivativeConstraint(t, c, quadTerms[index], linTerms[index]);
}

void TSDeformationSolver::addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c)
{
   int32 t = (int32)c->getTime();
   if (t > context->getEndTime() || t<context->getStartTime()) throw ERangeCheck("time out of range");
   card32 index = t - context->getStartTime();

   context->addSimpleSpatialDerivativeConstraint(t, c, quadTerms[index], linTerms[index]);
}

void TSDeformationSolver::solve() 
{
	// mb - transfer cached matrices and clear cache
	if( settings->tsCachedCellMatrices )
	{
		for( unsigned i=0;i<context->getDuration();i++ )
		{
			context->transferCachedPositionConstraints(context->getStartTime() + i,quadTerms[i]);
		}
	}


   size_t numTimeSteps = quadTerms.size();
   x4Assert(numTimeSteps == linTerms.size());
   for (size_t t=0; t<numTimeSteps; t++) {
      DVectorF result;
      context->getLastResult((int32)t-context->getStartTime(), result);
      card32 numIterations = settings->numCGIterations;
      conjugateGradientsSolve<float32>(
         quadTerms[t],
         result,
         linTerms[t],
         numIterations,
         settings->convThreshCG,
         settings->mmDebugOutput
      );
      //debugOutput << "num CG iterations: " << numIterations << "\n";
      context->setResult((int32)t-context->getStartTime(), result);
   }
   quadTerms = std::vector<SparseMatrixF>();
   linTerms = std::vector<DVectorF>();
}



void TSDeformationSolver::callCallbacks() 
{
   for (unsigned i=0; i<callbacks.size(); i++) {
      callbacks[i]();
   }
}

void TSDeformationSolver::addUserFeedbackCallback(Delegate d) {
   callbacks.push_back(d);
}

void TSDeformationSolver::clearUserFeedbackCallbacks() 
{
   callbacks.clear();
}

DeformationEvaluator *TSDeformationSolver::createEvaluator() 
{
	return new TSDeformationEvaluator();
}

