//---------------------------------------------------------------------------
#ifndef TSDeformationSolverH
#define TSDeformationSolverH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "DeformationSolver.h"
#include "SparseLinearAlgebra.h"
#include "DynamicLinearAlgebra.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {
class RegularVolume;
class SGEDeformationSequence;
class SGEShape;
class TSDeformationContext;


///   mw - 12/12/2008
///
///   Deformation solver that uses a 3D->3D thin plate spline energy
///   on a sparse regular grid
///
class X3DEFORMATION_API TSDeformationSolver : public DeformationSolver {
	X4_CLASS( TSDeformationSolver )
 private:
   std::vector<Delegate> callbacks;
	TSDeformationContext *context;
	const ARSettings *settings;

   // --- temporary
   std::vector<SparseMatrixF> quadTerms;
   std::vector<DVectorF> linTerms;
	
 protected:
   void callCallbacks();
   
 public:
	TSDeformationSolver();

	virtual DeformationEvaluator *createEvaluator();
	AROptimizationContext *createOptContext();
	virtual void setup(AROptimizationContext *context, const ARSettings *settings);
	virtual void solveGroup(GroupDeformationContext *groupContext, std::vector<DeformationSolver*> group);

	virtual void addUserFeedbackCallback(Delegate d);
	virtual void clearUserFeedbackCallbacks();

	virtual void beginConstraints();
   virtual void addPositionConstraint(const PositionARConstraint *c);
	virtual void addHalfSpaceConstraint(const ARHalfSpaceConstraint *c);
   virtual void addVelocityConstraint(const VelocityVectorARConstraint *c);
   virtual void addInstantaneousVelocityConstraint(const InstantaneousVelocityARConstraint *c);
   virtual void addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c);
   virtual void addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c);
   
	virtual void solve();


	~TSDeformationSolver();
};

}

#endif