//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ConstraintsDialog.h"
#include "LinearAlgebra.inline.h"
#include "ARConstraints.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------





ConstraintsDialog::ConstraintsDialog(QWidget *parent, Qt::WindowFlags f) 
{
	ui.setupUi(this);
}

ConstraintsDialog::~ConstraintsDialog() 
{
}

bool ConstraintsDialog::createConstraint(QWidget *parent, GeneralARConstraint *&result) 
{
	ConstraintsDialog *w = new ConstraintsDialog(parent, Qt::Dialog);
	w->setModal(true);
	bool r = (w->exec() == Accepted);
	if (r) {
		ARConstraintAnchor anchor;
		Vector3f toVec;

		anchor.setup(
			makeVector3f(
				(float32)w->ui.dsbPosAtX->value(),
				(float32)w->ui.dsbPosAtX->value(),
				(float32)w->ui.dsbPosAtX->value()
			), 
			ARConstraintAnchor::NO_TOP_ANCHOR
		);
		toVec[0] = (float32)w->ui.dsbPosVectorX->value();
		toVec[1] = (float32)w->ui.dsbPosVectorY->value();
		toVec[2] = (float32)w->ui.dsbPosVectorZ->value();

		float32 timeStart = w->ui.sbTime->value();
		float32 timeEnd   = w->ui.sbVelocityTime->value();

		Matrix3f quadEnergy;
		quadEnergy.setIdentity();
		quadEnergy *= w->ui.dsbVariance->value();

		if (w->ui.cbVelocity->isChecked()) {
			if (w->ui.cbInstVelocity->isChecked()) {
				InstantaneousVelocityARConstraint *ret = new InstantaneousVelocityARConstraint();
				ret->setup(anchor, timeStart, toVec, quadEnergy);
				result = ret;
			} else {
				VelocityVectorARConstraint	*ret = new VelocityVectorARConstraint();
				ret->setup(anchor, timeStart, timeEnd, toVec, quadEnergy);
				result = ret;
			}
		} else {
			PositionARConstraint *ret = new PositionARConstraint();
			ret->setup(anchor, toVec, quadEnergy, timeStart);
				result = ret;
		}
	}
	delete w;
	return r;
}


void ConstraintsDialog::on_pbOk_clicked() 
{
	done(Accepted);
}

void ConstraintsDialog::on_pbCancel_clicked() 
{
	done(Rejected);
}


#include "ConstraintsDialog_moc.h"

