#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SilhouetteImageEditorWidget.h"
//---------------------------------------------------------------------------
#include "SilhouetteImage.h"
#include "PointSet.inline.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_CLASS_EDITOR( SilhouetteImage )

SilhouetteImageEditorWidget::SilhouetteImageEditorWidget(QWidget * parent, Qt::WindowFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));

}

SilhouetteImageEditorWidget::~SilhouetteImageEditorWidget(void)
{
}

void SilhouetteImageEditorWidget::setup( SilhouetteImage * obj )
{
	m_Object = obj;
}

void SilhouetteImageEditorWidget::updateEditor()
{
	int w=m_Object->getSizeX();
	int h=m_Object->getSizeY();

	QImage img(QSize(m_Object->getSizeX(),m_Object->getSizeY()),QImage::Format_RGB32);

	for( card32 y=0;y<m_Object->getSizeY();y++ )
	{
		card32 * ptr = (card32 *)img.scanLine(y);  ;
	
		for( card32 x=0;x<m_Object->getSizeX();x++ )
		{
			float32 val = m_Object->getPixel(x,y);
			
			ptr[x] = getPixelRGB(val);

			if (m_Object->isBorder(x,y))
			{
				ptr[x] = qRgb(255,20,20);
			}
		}
	}


	ui.imgLabel->resize(w,h);

	pm=pm.fromImage(img.scaled(w,h,Qt::KeepAspectRatio,Qt::SmoothTransformation));
	ui.imgLabel->setPixmap(pm);



	ui.imgLabel->setPixmap(pm);
}

#include "SilhouetteImageEditorWidget_moc.h"

X4::card32 X4::SilhouetteImageEditorWidget::getPixelRGB( float32 z )
{
	float32 MinZ = 42.0, MaxZ = 130.0;
	float32 d;
	float32 range=(1.0f/MinZ - 1.0f/MaxZ);
	float32 max=1.0/MaxZ;

	d   = (((1.0/z)-max)/range)*255.0;

	card8 gray= (card8)(d+0.5f);
	return qRgb(gray,gray,gray);
}

void X4::SilhouetteImageEditorWidget::mousePressEvent( QMouseEvent* e )
{
	QPoint qp=mapToGlobal(e->pos());
	qp=ui.imgLabel->mapFromGlobal(qp);
	Vector2f p=makeVector2f(qp.x(),qp.y());

	float32 depth=m_Object->getPixel(qp.x(),qp.y())-5;

	if  (e->button()==Qt::LeftButton ) 
	{
		Vector2i targetPixel= m_Object->getNearestCoveringPixel(qp.x(),qp.y(),depth,60);

		//std::vector<Vector2i> knns= m_Object->getNearestCoveringPixels(qp.x(),qp.y(),depth,60);

		QPainter pnt(&pm);
			pnt.setPen(Qt::red);
			//for(card32 i=0; i<knns.size();i++)
			//	pnt.drawPoint(knns[i][0], knns[i][1]);

			pnt.drawLine(targetPixel[0]-4,targetPixel[1]-4,targetPixel[0]+4,targetPixel[1]+4);
			pnt.drawLine(targetPixel[0]-4,targetPixel[1]+4,targetPixel[0]+4,targetPixel[1]-4);
			pnt.setPen(Qt::green);
			pnt.drawPoint(targetPixel[0], targetPixel[1]);
		pnt.end();
		ui.imgLabel->setPixmap(pm);
	}
	else
	{
		updateEditor();
	}
	
}

void SilhouetteImageEditorWidget::on_normalImageButton_clicked(){
	m_Object->markEdges();

}