#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "CVisionTools.h"
//---------------------------------------------------------------------------
#include "ParametricInCorePointCloud.h"
#include "PointSet.inline.h"
#include "LinearAlgebra.inline.h"
#include "RGBAImage.h"
#include "ImageDisplayer.h"

//---------------------------------------------------------------------------
using namespace X4;

//#define DEBUG_IMAGE_COMPARE

float X3DEFORMATION_API X4::compareImagePatches( ParametricInCorePointCloud * image1, const Vector2i &point1, int windowSize, 
																ParametricInCorePointCloud * image2, const Vector2f &quad1, const Vector2f &quad2,
																const Vector2f &quad3, const Vector2f &quad4,   int compareType,
																std::string channel)
{
	AAT chan = image1->getAAT(channel);
	static std::vector<Vector3f> samples1, samples2;
	unsigned numEntries = (2*windowSize+1) * (2*windowSize+1);
	samples1.resize(numEntries);
	samples2.resize(numEntries);
	PointSet * ps1 = image1->getPointSet();
#ifdef DEBUG_IMAGE_COMPARE
	RGBAFloat32Image testImage(4*windowSize+2,windowSize*2+1);

#endif
	unsigned i=0;
	for( int x=-windowSize;x<=windowSize;x++ )
	{
		for( int y=-windowSize;y<=windowSize;y++ )
		{
			samples1[i] = image1->getBilinearInterpolatedAttribute3f(x+point1[0],y+point1[1],chan);//ps1->get3f(x+point1[0],y+point1[1],chan);
			
			// compute position in image2
			float w1 = ((float)x + windowSize) / (float)(windowSize * 2.0f);
			float w2 = ((float)y + windowSize) / (float)(windowSize * 2.0f);
			Vector2f pos = (quad1 * (1.0f - w1) + quad2 * w1) * (1.0f - w2)
				+ (quad4 * (1.0f - w1) + quad3 * w1) * w2;
			samples2[i] = image2->getBilinearInterpolatedAttribute3f(pos[0],pos[1],chan);
#ifdef DEBUG_IMAGE_COMPARE
			testImage.setPixel( x+windowSize,y+windowSize, expand3To4(samples1[i]));
			testImage.setPixel( x+3*windowSize+1,y+windowSize, expand3To4(samples2[i]));
#endif
			i++;
		}
	}
#ifdef DEBUG_IMAGE_COMPARE
	static int imageNumber = 0;
	ImageDisplayer::DisplayRGBAFloat32Image ev(&testImage);
	char c = '0' + imageNumber;
	debugOutput.displayEvent(ImageDisplayer::getClass(), std::string("test") + c, &ev);
	imageNumber = (imageNumber+1) % 10;
#endif

	switch( compareType )
	{
	case CI_SSD:
		{
			float ret = 0;
			for( unsigned i=0;i<samples1.size();i++)
				ret += normQuad(samples1[i] - samples2[i]);
			return ret;
		}
		break;
	case CI_NCC:
		{
			// gray
			/*DVectorF vec1(samples1.size()),vec2(samples1.size());
			float average1 = 0;
			float average2 = 0;
			for( unsigned i=0;i<samples1.size();i++)
			{
				vec1[i] = samples1[i]* makeVector3f(11,16,5);
				average1 += vec1[i];
				vec2[i] = samples2[i]* makeVector3f(11,16,5);
				average2 += vec2[i];
			}
			average1 /= (float)samples1.size();
			average2 /= (float)samples1.size();
			for( unsigned i=0;i<samples1.size();i++)
			{
				vec1[i] -= average1;
				vec2[i] -= average2;
			}

			vec1 /= vec1.getNorm();
			vec2 /= vec2.getNorm();

			return vec1 * vec2;*/
			

			
			Vector3f average1 = NULL_VECTOR3F;
			Vector3f average2 = NULL_VECTOR3F;
			for( unsigned i=0;i<samples1.size();i++)
			{
				average1 += samples1[i];
				average2 += samples2[i];
			}
			average1 /= (float)samples1.size();
			average2 /= (float)samples1.size();

			float ret = 0;
			float ret_norm1 = 0;
			float ret_norm2 = 0;
			for( unsigned i=0;i<samples1.size();i++)
			{
				ret += (samples1[i]-average1)*(samples2[i]-average2);
				ret_norm1 += normQuad(samples1[i]-average1);
				ret_norm2 += normQuad(samples2[i]-average2);
			}

			return ret / sqrt(ret_norm1*ret_norm2);
		}
		break;
	}

	return 0;
}

float X4::computeImagePatchVariance( ParametricInCorePointCloud * image, int windowSize,
												  const Vector2f &quad1, const Vector2f &quad2,// quadrangle (see createImagePatches for details)
												  const Vector2f &quad3, const Vector2f &quad4,
													std::string channel )
{
	AAT chan = image->getAAT(channel);
	static std::vector<Vector3f> samples;
	unsigned numEntries = (2*windowSize+1) * (2*windowSize+1);
	samples.resize(numEntries);

	unsigned i=0;
	for( int x=-windowSize;x<=windowSize;x++ )
	{
		for( int y=-windowSize;y<=windowSize;y++ )
		{
			// compute position
			float w1 = ((float)x + windowSize) / (float)(windowSize * 2.0f);
			float w2 = ((float)y + windowSize) / (float)(windowSize * 2.0f);
			Vector2f pos = (quad1 * (1.0f - w1) + quad2 * w1) * (1.0f - w2)
				+ (quad4 * (1.0f - w1) + quad3 * w1) * w2;
			samples[i] = image->getBilinearInterpolatedAttribute3f(pos[0],pos[1],chan);

			i++;
		}
	}

	DVectorF vec((card32)samples.size());
	float average = 0;
	for( unsigned i=0;i<samples.size();i++)
	{
		vec[i] = samples[i]* makeVector3f(11,16,5);
		average += vec[i];
	}
	average /= (float)samples.size();
	float ret = 0;
	for( unsigned i=0;i<samples.size();i++)
	{
		ret += pow( vec[i] - average,2.0f);
	}
	return ret / (float)samples.size();
}
