#ifndef CVisionToolsH
#define CVisionToolsH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------

namespace X4
{
class ParametricInCorePointCloud;

enum ECompareImagePatchType
{
	CI_SSD = 1,
	CI_NCC = 2
};

/** 
compare two image patches with SSD/NCC
warning: NCC -> [0,1]    1 is optimal
         SSD -> [0,inf)  0 is optimal

assumes an attribute channel with (float,3)

window definition: 
	point1 + (-windowSize,-windowSize) -> quad1 
	point1 + (windowSize,-windowSize) -> quad2 
	point1 + (windowSize,windowSize) -> quad3 
	point1 + (-windowSize,windowSize) -> quad4

	1 ------ 2
	|        |
	|        |
	|        |
	4 ------ 3


	*/
float X3DEFORMATION_API compareImagePatches( ParametricInCorePointCloud * image1,			// first image
														const Vector2i &point1, int windowSize,		// quadratic window in first image (e.g.5)
														ParametricInCorePointCloud * image2,			// second image
														const Vector2f &quad1, const Vector2f &quad2,// quadrangle in second frame
														const Vector2f &quad3, const Vector2f &quad4,
														int compareType = CI_SSD,							// type of comparison
														std::string channel = "color"							// attribute channel for comparison
														);

float X3DEFORMATION_API computeImagePatchVariance( ParametricInCorePointCloud * image,
																int windowSize,
																const Vector2f &quad1, const Vector2f &quad2,// quadrangle (see createImagePatches for details)
																const Vector2f &quad3, const Vector2f &quad4,
																std::string channel = "color"
																);


}

#endif