#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "DepthImageHalfSpaceConstraintCreator.h"
//---------------------------------------------------------------------------
#include "PCCImportDepthAnimation.h"
#include "ARSettings.h"
#include "BitVector8.h"
#include "PointSet.inline.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( DepthImageHalfSpaceConstraintCreator ,0)
{
	BEGIN_CLASS_INIT( DepthImageHalfSpaceConstraintCreator );
}

DepthImageHalfSpaceConstraintCreator::DepthImageHalfSpaceConstraintCreator(void)
{
}

DepthImageHalfSpaceConstraintCreator::~DepthImageHalfSpaceConstraintCreator(void)
{
}

void DepthImageHalfSpaceConstraintCreator::assign(const Object* obj, X4CopyContext *context)
{
	Object::assign(obj,context);
	const DepthImageHalfSpaceConstraintCreator * o = dynamic_cast<const DepthImageHalfSpaceConstraintCreator*>(obj);
	x4Assert( o != NULL );

}

ARHalfSpaceConstraint* X4::DepthImageHalfSpaceConstraintCreator::getConstraint( const Vector3f &position )
{

	x4Assert(false);
	/*for (card32 cam=0; cam<8;cam++){
		if(depthMaps[cam]->isInShadow(position))
			return NULL; //positionIsOk
	}*/
	ARHalfSpaceConstraint* result=new ARHalfSpaceConstraint();
	/*ARConstraintAnchor anchor;
	BitVector8 cams;
	Vector3f footPoint=volGrid.getNearestPositionAndCams(position, cams);
	//TODO: refine by using pictures
	Vector3f normal=normalize(footPoint-position);

	/// penalty = w*[(x-fp)*n]^2, if (x-fp)*n>0, zero otherwise
	float32 weight= 1.0f;


	result->setup(anchor, currentFrameNo, footPoint, normal, weight);*/
	return result;
}

void X4::DepthImageHalfSpaceConstraintCreator::setup( card32 frameNo, Scene* scene, card32 minVolGridRes, float32 depthTol )
{
	
	notImplemented();

	reset();
	currentFrameNo=frameNo;
	volGrid.setup(getScenePCBBox(scene),minVolGridRes, depthTol);
	volGrid.processFrame(frameNo, scene);
	
	for (card32 cam=0; cam<8;cam++)
	{
		SilhouetteImage* si= new SilhouetteImage();
		si->setup(getPointCloud(scene, PCCImportDepthAnimation::getPointCloudName(frameNo, cam ,true)), depthTol);
		depthMaps.push_back(si);
	}

}

void X4::DepthImageHalfSpaceConstraintCreator::reset()
{
	for (card32 i=0;i<depthMaps.size();i++)
		delete depthMaps[i];
	depthMaps.clear();
	volGrid.reset();
}

bool X4::DepthImageHalfSpaceConstraintCreator::constraintNeeded( Vector3f pos )
{
	for (card32 cam=0; cam<8;cam++){
		if(depthMaps[cam]->isInShadow( pos )){
			return false; 
		}
	}
	return true;
}

X4::Vector3f X4::DepthImageHalfSpaceConstraintCreator::getBestFootPointAndNormal( Vector3f pos, Vector3f &returnedNormal, bool &error )
{
	error=false;
	BitVector8 cams;
	Vector3f footPoint;

	if (!constraintNeeded(pos))	
	{


		footPoint=pos;   //;volGrid.getNearestPositionAndCams(pos, cams);
		cams.setAllBitsTrue();
		std::vector<card32> interestingCams=cams.getTrueBits();
		if (interestingCams.empty())
		{
			//error("DepthImageHalfSpaceConstraintCreator::buildSilhouetteConstraints() - Serious error!");
			//debugOutput << "warning";
			error=true;

		}

		float32 maxDist=999999999999.0f;
		int32 bestCam= -1;

		for(card32 i=0; i<interestingCams.size(); i++){
			Vector3f nearestPos= depthMaps[interestingCams[i]]->getNearestForbiddenPosition(pos, error);
			if (nearestPos== makeVector3f(-10000, -10000, -10000))
				continue;			
			float32 dist=norm(nearestPos-pos);
			if(dist<maxDist){
				footPoint=nearestPos;
				bestCam=interestingCams[i];
				maxDist=dist;
			}
		}

		if (maxDist==999999999999.0f)
		{
			error=true;
		}


		returnedNormal=normalize(footPoint-pos);
		//TODO: use picture normal



	}else{

		footPoint=volGrid.getNearestPositionAndCams(pos, cams);
		std::vector<card32> interestingCams=cams.getTrueBits();
		if (interestingCams.empty())
		{
			//error("DepthImageHalfSpaceConstraintCreator::buildSilhouetteConstraints() - Serious error!");
			//debugOutput << "warning";
			error=true;

		}
		
		float32 maxDist=999999999999.0f;
		int32 bestCam= -1;

		for(card32 i=0; i<interestingCams.size(); i++){
			Vector3f nearestPos= depthMaps[interestingCams[i]]->getNearestAllowedPosition(pos, error);
			float32 dist=norm(nearestPos-pos);
			if(dist<maxDist){
				footPoint=nearestPos;
				bestCam=interestingCams[i];
				maxDist=dist;
			}
		}

		
		returnedNormal=normalize(pos-footPoint);
		//TODO: use picture normal

	}

	return footPoint;
}




void DepthImageHalfSpaceConstraintCreator::buildSilhouetteConstraints(
   const ARSettings *settings,
   ARConstraintsInterface *receiver,
   PointCloud *referenceShape,
   PointCloud *deformedReference,
	card32 frameNo,
	Scene* scene,
   Delegate callForRepaint,
   bool showConstr)
{

	//this->setup(frameNo, scene, settings->gridResolution, settings->depthTolerance);
	debugOutput << "buildSilhouetteConstraints: setup with grid: " << settings->gridResolution << " and depthTolerance "<< settings->depthTolerance<< "\n"; debugOutput.flush();
	reset();

	SceneObjectIterator *refIt = referenceShape->createIterator(SOIT::CAP_BASIC_PC);
	SceneObjectIterator *defRefIt = deformedReference->createIterator(SOIT::CAP_BASIC_PC);
	BasicPointCloudIterator *bRefIt = dynamic_cast<BasicPointCloudIterator*>(refIt);
	BasicPointCloudIterator *bDefRefIt = dynamic_cast<BasicPointCloudIterator*>(defRefIt);
	if (!bRefIt || !bDefRefIt ) {
		delete refIt; delete defRefIt;
		error("something went wrong.");
		return;
	}

	const PointSet *refPS = bRefIt->lockBlockRead();
	const PointSet *defRefPS = bDefRefIt->lockBlockRead();

	if (!refPS->getNumEntries() == defRefPS->getNumEntries()) {
		error("buildSilhouetteConstraints() - deformed reference and reference should have the same number of points");
		return;
	}


	currentFrameNo=frameNo;
	BoundingBox3f bb=defRefPS->getBoundingBox();
	bb.addBB(refPS->getBoundingBox());
	bb.addBorder(1);
	volGrid.setup(bb ,settings->gridResolution, 0);//settings->depthTolerance);
	volGrid.processFrame(frameNo, scene);

	for (card32 cam=0; cam<8;cam++)
	{
		SilhouetteImage* si= new SilhouetteImage();
		si->setup(getPointCloud(scene, PCCImportDepthAnimation::getPointCloudName(frameNo, cam ,true)),  0);//settings->depthTolerance);
		depthMaps.push_back(si);
	}

	debugOutput << "buildSilhouetteConstraints: setup done.\n"; debugOutput.flush();

	
	AAT REFERENCE_POSITION_AAT			   = refPS->getAAT( "position" );
	AAT DEFORMED_REFERENCE_POSITION_AAT = defRefPS->getAAT( "position" );

	card32 numConsAdded=0;

	for (card32 matchIndex = 0; matchIndex<defRefPS->getNumEntries(); matchIndex++) {
		const Vector3f& deformedPos = defRefPS->get3f(matchIndex, DEFORMED_REFERENCE_POSITION_AAT);

		
		//Test if constraint is used
		//if (!constraintNeeded(deformedPos))
		//	continue;

		//we need a constraint
		numConsAdded++;
		ARHalfSpaceConstraint* result=new ARHalfSpaceConstraint();
		ARConstraintAnchor anchor;
		anchor.setup(refPS->get3f(matchIndex, REFERENCE_POSITION_AAT), matchIndex);


		Vector3f normal;
		bool error=false;
		Vector3f footPoint=getBestFootPointAndNormal(deformedPos, normal, error);
		if (error)
		{
			continue;
		}


		/// penalty = w*[(x-fp)*n]^2, if (x-fp)*n>0, zero otherwise
		float32 weight= 1.0f;

		result->setup(anchor, 0, footPoint, normal, weight);

		receiver->addHalfSpaceConstraint(result);
		
		//debug
			calculatedConstr.push_back(result);
	}

	

	bRefIt->unlock();
	bDefRefIt->unlock();
	
	delete refIt;
	delete defRefIt;

	debugOutput << "buildSilhouetteConstraints: added " << numConsAdded << " constraints.\n";
	
	if (showConstr) 
		callForRepaint.callIfNotNull();

	//clean up
	reset();
}




void DepthImageHalfSpaceConstraintCreator::glDrawFeedback(GLContext *glContext, ARVisualizationSettings *visSettings) {
//   if (corrVis.empty()) return;
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glEnable(GL_BLEND);
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);





   glBegin(GL_LINES);
		for (unsigned i=0; i<calculatedConstr.size(); i++) {
			   glColor4f(0.5f, 1.0f, 0.50f, 0.5f);
			glVertex3fv(calculatedConstr[i]->getHalfSpaceFootPoint().data());
			   glColor4f(1.0f, 0.50f, 0.50f, 0.5f);
			glVertex3fv(calculatedConstr[i]->getAnchor().getPos().data());
			//glVertex3fv((calculatedConstr[i]->getHalfSpaceFootPoint()-calculatedConstr[i]->getHalfSpaceNormal()).data());
			//drawGradientLine(corrVis[i].first, corrVis[i].second);

			float32 no=norm(calculatedConstr[i]->getHalfSpaceFootPoint() - (calculatedConstr[i]->getAnchor().getPos()) );

				if (no>1.0f)
				{
				//	debugOutput << no << "\n";
				}
		}
   glEnd();

   glPopAttrib();
}
