#ifndef DepthImageHalfSpaceConstraintCreatorH
#define DepthImageHalfSpaceConstraintCreatorH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Object.h"
#include "ARConstraints.h"
#include "ShadowVolumeGrid.h"
#include "SilhouetteImage.h"
#include "ARConstraintsInterface.h"
#include "AROptToolWithFeedback.h"
#include "Delegate.h"
//---------------------------------------------------------------------------

namespace X4
{
	class ARSettings;
	class PointCloud;
	class ARConstraintsInterface;

	/**
	plz comment your class
	*/
	class X3DEFORMATION_API DepthImageHalfSpaceConstraintCreator : public Persistent, public AROptToolWithFeedback 
	{
		X4_CLASS( DepthImageHalfSpaceConstraintCreator )

	private:
		ShadowVolumeGrid volGrid;
		std::vector<SilhouetteImage*> depthMaps;
		card32 currentFrameNo;
		std::vector<ARHalfSpaceConstraint*> calculatedConstr;

		void setup(card32 frameNo, Scene* scene, card32 minVolGridRes, float32 depthTol);
		bool constraintNeeded(Vector3f pos);
		Vector3f getBestFootPointAndNormal(Vector3f pos, Vector3f &returnedNormal, bool &error);
		bool debugCams[8];

	public:
		DepthImageHalfSpaceConstraintCreator(void);
		~DepthImageHalfSpaceConstraintCreator(void);

		virtual void assign(const Object* obj, X4CopyContext *context = NULL);


		void reset();

		ARHalfSpaceConstraint* getConstraint(const Vector3f &position);

		virtual void glDrawFeedback(GLContext *glContext, ARVisualizationSettings *visSettings);

		/// builds constraints that move points from referenceShape
		void buildSilhouetteConstraints(
			const ARSettings *settings,
			ARConstraintsInterface *receiver,
			/// parametrization (urshape)
			PointCloud *referenceShape,
			/// urshape as in its current form ("best guess")
			PointCloud *deformedReference,
			/// current frame number
			card32 frameNo,
			///scene pointer
			Scene* scene,
			/// redraw scene, calling 
			Delegate callForRepaint = NULL,
			/// show halfspace constraints
			bool showConstr = false
			);



	};

}

#endif