#ifndef ShadowVolumeGridH
#define ShadowVolumeGridH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "CameraParameterAttachment.h"
#include "BitVector8.h"
//---------------------------------------------------------------------------
//#define USE_VOXELGRID 
#ifdef USE_VOXELGRID
	#include "private/projects_ab/projects_ab.h"
	#include "private/projects_ab/VoxelReconstruction/PCVoxel.h"
	#include "private/projects_ab/VoxelReconstruction/VoxelGrid.h"
#endif // USE_VOXELGRID
//---------------------------------------------------------------------------
 
namespace X4
{
	class PointSetKNNQuery;
	class AttachedIndexedOctree;
/**
	plz comment your class
*/
class X3DEFORMATION_API ShadowVolumeGrid : public Persistent
{
	X4_CLASS( ShadowVolumeGrid )

private:
	card32 resX,resY,resZ;
	BitVector8* voxelData;
	//bool* nearShadow;
	PointSet* nearShadowPointSet;
	float32 sideLength;
	Vector3f upper;
	Vector3f lower;
	CameraParameterAttachment* myCam;
	const PointSet* depthPicturePointset;
	float32 depthTolerance;
	AAT DEPTH;
	AAT CAM;
	AAT NSH_POS;
	PointSetKNNQuery* knnNearShadow;
	AttachedIndexedOctree* octree;

	bool inShadowVolume(Vector3f worldCoordinate);


public:
	ShadowVolumeGrid(void);
	~ShadowVolumeGrid(void);

	void reset();
	void setup(BoundingBox3f bb, card32 minRes, float32 tol);
	void processFrame(card32 frameNo, Scene* scene);
	void processDepthImage(card32 camNo, CameraParameterAttachment* cam,const PointSet* picturePointset);
	card8 voxelNearShadow(const int32 &myX, const int32 &myY, const int32 &myZ);
	Vector3f getVoxelCenter(const card32 &x, const card32 &y, const card32 &z);
	void addCamToVoxel(const card32 &camNo, const card32 &x, const card32 &y, const card32 &z);
	BitVector8* getVoxelPointer(const card32 &x, const card32 &y, const card32 &z);
	card8 getVoxelValue(const card32 &x, const card32 &y, const card32 &z);
	bool voxelIsInShadow(const card32 &x, const card32 &y, const card32 &z);
   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
	Vector3f getNearestPositionAndCams(Vector3f pointPos, BitVector8 &returnedCams);


	
	
	
	void visualizeVolumeInScene(card8 th, Scene* scene, card32 numFrame ){
		#ifdef USE_VOXELGRID
			displayVoxelGrid( th, scene );
		#endif // USE_VOXELGRID
		
		#ifndef USE_VOXELGRID
			displayRegularVolume( th,  scene, numFrame );
		#endif
	}
	
	void displayRegularVolume(card8 th, Scene* scene, card32 frameNo );

	#ifdef USE_VOXELGRID
		void displayVoxelGrid(card8 th, Scene* scene );
	#endif // USE_VOXELGRID

};

}

#endif