#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SilhouetteImage.h"
//---------------------------------------------------------------------------
#include "PointSet.inline.h"
//---------------------------------------------------------------------------
using namespace X4;


IMPLEMENT_X4_CLASS( XYWNormalFloat32Image ,0)
{
	BEGIN_CLASS_INIT( XYWNormalFloat32Image );
}



IMPLEMENT_X4_CLASS( SilhouetteImage ,0)
{
	BEGIN_CLASS_INIT( SilhouetteImage );
	ADD_FLOAT32_PROP(depthTolerance,0);
}

SilhouetteImage::SilhouetteImage(void)
{
	depthTolerance=1.0f;
	borderMargin=1;
}

SilhouetteImage::~SilhouetteImage(void)
{
}

void SilhouetteImage::assign(const Object* obj, X4CopyContext *context)
{
	TemplatedImage<float32> ::assign(obj,context);
	const SilhouetteImage * o = dynamic_cast<const SilhouetteImage*>(obj);
	x4Assert( o != NULL );

}


X4::Vector2i X4::SilhouetteImage::getNearestUnCoveringPixel( card32 myX, card32 myY, float32 myDepth, card32 maxRad )
{

	float32 otherDepth=getPixel(myX,myY);
	x4Assert(myDepth>=otherDepth);

	CoordinatesAndDistance bestFound(0,0,9999999999999999.0f);
	bool found=false;

	int32 spuareSize=maxRad*maxRad;
	for (int32 y=myY-maxRad; y<(int32)(myY+maxRad); y++)
		for (int32 x=myX-maxRad; x<(int32)(myX+maxRad); x++){
			if (x<borderMargin || y<borderMargin || x>=(int32)sizeX-borderMargin || y >= (int32)sizeY-borderMargin )
				continue;
			float32 pictDepth=getPixel(x,y);
			if(myDepth>=pictDepth)
				continue;

			int32 xDif=x-(int32)myX;
			int32 yDif=y-(int32)myY;

			int32 sqrDist=xDif*xDif+yDif*yDif;
			if (sqrDist <spuareSize && sqrDist<bestFound.sqrDist)
			{
				found=true;
				bestFound.x=x; bestFound.y=y; bestFound.sqrDist=sqrDist;
			}

		}

		if (!found)
		{
			return makeVector2i( 0, 0);
		}

		return makeVector2i(bestFound.x, bestFound.y);

}

X4::Vector2i X4::SilhouetteImage::getNearestCoveringPixel( card32 myX, card32 myY, float32 myDepth, card32 maxRad )
{

	float32 otherDepth=getPixel(myX,myY);
	x4Assert(myDepth<otherDepth);

	CoordinatesAndDistance bestFound(0,0,9999999999999999.0f);
	bool found=false;

	int32 spuareSize=maxRad*maxRad;
	for (int32 y=myY-maxRad; y<(int32)(myY+maxRad); y++)
		for (int32 x=myX-maxRad; x<(int32)(myX+maxRad); x++){
			if (x<borderMargin || y<borderMargin || x>=(int32)sizeX-borderMargin || y >= (int32)sizeY-borderMargin )
				continue;
			float32 pictDepth=getPixel(x,y);
			if(myDepth<pictDepth)
				continue;
			
			int32 xDif=x-(int32)myX;
			int32 yDif=y-(int32)myY;

			int32 sqrDist=xDif*xDif+yDif*yDif;
			if (sqrDist <spuareSize && sqrDist<bestFound.sqrDist)
			{
				found=true;
				bestFound.x=x; bestFound.y=y; bestFound.sqrDist=sqrDist;
			}
			
		}

	if (!found)
	{
		return makeVector2i( 0, 0);
	}
	
	return makeVector2i(bestFound.x, bestFound.y);

}

void X4::SilhouetteImage::setup( PointCloud* pc, float32 tol)
{
	if (!pc->supportsIteratorType( SOIT::CAP_PARAMETRIC_PC)) {
		x4Assert(false);
		return;
	}

	depthTolerance=tol;

	SceneObjectIterator* iter=pc->createIterator(SOIT::CAP_BASIC_PC|SOIT::CAP_PARAMETRIC_PC );
	BasicPointCloudIterator* basicIt=dynamic_cast<BasicPointCloudIterator*>(iter);
	const PointSet *ps = basicIt->lockBlockRead();
	camForThisImage.assign(pc->getAttachments()->getData(CameraParameterAttachment::getAttachmentName()));
	AAT DEPTH= ps->getAAT("depth");
	card32 h=ps->getHeight();
	card32 w= ps->getWidth();
	resize(w,h);
	for (card32 y=0; y<h; y++)
		for (card32 x=0; x<w; x++)
		{
			this->setPixel(x,y,ps->get1f(x,y,DEPTH));
		}
	basicIt->unlock();
	delete iter;
}


std::vector<Vector2i> X4::SilhouetteImage::getNearestCoveringPixels( card32 myX, card32 myY, float32 myDepth, card32 maxRad )
{
	x4Assert(myDepth<getPixel(myX,myY));

	std::vector<CoordinatesAndDistance> hits;

	int32 spuareSize=maxRad*maxRad;
	for (int32 y=myY-maxRad; y<(int32)(myY+maxRad); y++)
		for (int32 x=myX-maxRad; x<(int32)(myX+maxRad); x++){
			if (x<borderMargin || y<borderMargin || x>=(int32)sizeX-borderMargin || y >= (int32)sizeY-borderMargin )
				continue;
			float32 pictDepth=getPixel(x,y);
			if(myDepth<pictDepth)
				continue;

			int32 xDif=x-(int32)myX;
			int32 yDif=y-(int32)myY;

			int32 sqrDist=xDif*xDif+yDif*yDif;
			if (sqrDist <spuareSize)
			{
				hits.push_back(CoordinatesAndDistance(x,y,sqrDist));
			}

		}

		std::vector<Vector2i> result;
		if (hits.empty())
		{
			result.push_back(makeVector2i( 0, 0));
			return result;
		}


		sort(hits.begin(),hits.end());


		for (card32 i=0;i<hits.size(); i++)
		{
			Vector2i n=makeVector2i(hits[i].x, hits[i].y);
			result.push_back(n);
		}
		return result;
}

bool X4::SilhouetteImage::isInShadow( const Vector3f &worldCoordinate )
{
	Vector2f imagePos = camForThisImage.projectWorldtoImage(worldCoordinate);
	float32 x=imagePos[0], y=imagePos[1];
	if (x<borderMargin || y<borderMargin || x>=(int32)sizeX-borderMargin || y >= (int32)sizeY-borderMargin )
		return false;

	float32 depth=getPixel( x+0.5f , y+0.5f);
	float32 depth3D=worldCoordinate[2];
	if (depth3D > (depth-depthTolerance))
		return true;
	else
		return false;

}

X4::Vector3f X4::SilhouetteImage::getNearestAllowedPosition( const Vector3f &pos, bool &error )
{
	x4Assert(!isInShadow(pos));

	
	Vector2f imagePos = camForThisImage.projectWorldtoImage(pos);
	Vector2i bestImagePos= getNearestCoveringPixel(imagePos[0]+0.5f, imagePos[1]+0.5f, pos[2]+depthTolerance, 100);//TODO: PARAMETERS xxx !
	if (bestImagePos==makeVector2i( 0, 0))
	{
		error=true;
	}
	Vector3f result= camForThisImage.projectImageToWorld(makeVector3f(bestImagePos[0], bestImagePos[1], getPixel(bestImagePos[0], bestImagePos[1]))) ; // z-Co-Problem!! xxx TODO

	return result;
}


X4::Vector3f X4::SilhouetteImage::getNearestForbiddenPosition( const Vector3f &pos, bool &error )
{
	if(!isInShadow(pos)){
		return makeVector3f(-10000, -10000, -10000);
	}


	Vector2f imagePos = camForThisImage.projectWorldtoImage(pos);
	Vector2i bestImagePos= getNearestUnCoveringPixel(imagePos[0]+0.5f, imagePos[1]+0.5f, pos[2]+depthTolerance, 100);//TODO: PARAMETERS xxx !
	if (bestImagePos==makeVector2i( 0, 0))
	{
		error=true;
	}
	Vector3f result= camForThisImage.projectImageToWorld(makeVector3f(bestImagePos[0], bestImagePos[1], getPixel(bestImagePos[0], bestImagePos[1]))) ; // z-Co-Problem!! xxx TODO

	return result;
}

bool X4::SilhouetteImage::isBorder( const int32 &myX, const int32 &myY )
{
	//float32 boarderDepthTol=0.5f;
	
	if (myX<2 || myY<2 || myX>1021 || myY>766)
		return false;

	float32 h=getPixel(myX,myY);

	for (int32 x=myX-1; x<=myX+1; x++ )
		for (int32 y=myY-1; y<=myY+1; y++)
		{
			if (getPixel(x,y)<h)
				return true;
		}

	return false;
}

void SilhouetteImage::markEdges(){


}
