#ifndef SilhouetteImageH
#define SilhouetteImageH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "TemplatedImage.h"
#include "TemplatedImage.inline.h"
#include "ParametricInCorePointCloud.h"
#include "CameraParameterAttachment.h"
//---------------------------------------------------------------------------

namespace X4
{

class CoordinatesAndDistance 
{
public:
	card32 x,y;
	float32 sqrDist;
	CoordinatesAndDistance(card32 sx, card32 sy, float32 r){
		x=sx;y=sy;sqrDist=r;
	}
	inline bool operator<(const CoordinatesAndDistance &other) const {
		return sqrDist < other.sqrDist;
	}
};


class X3DEFORMATION_API XYWNormalFloat32Image : public Vector3fImage
{
	X4_CLASS( XYWNormalFloat32Image )

	XYWNormalFloat32Image() {}
	XYWNormalFloat32Image( card32 sizeX, card32 sizeY ) : TemplatedImage<Vector3f>(sizeX, sizeY) {}

};

/**
	plz comment your class
*/
class X3DEFORMATION_API SilhouetteImage : public Float32Image 
{
	X4_CLASS( SilhouetteImage )

private:
	CameraParameterAttachment camForThisImage;
	float32 depthTolerance;
	int32 borderMargin;
	
public:
	SilhouetteImage(void);
	~SilhouetteImage(void);

	void setup(PointCloud* pc, float32 tol);
	Vector2i getNearestCoveringPixel(card32 x, card32 y, float32 myDepth, card32 maxRad);
	Vector2i getNearestUnCoveringPixel(card32 x, card32 y, float32 myDepth, card32 maxRad);
	std::vector<Vector2i> getNearestCoveringPixels(card32 x, card32 y, float32 myDepth, card32 maxRad);
	bool isInShadow(const Vector3f &pos);

	void markEdges();
	bool isBorder(const int32 &x, const int32 &y);


	Vector3f getNearestAllowedPosition(const Vector3f &pos, bool &error);
	Vector3f getNearestForbiddenPosition(const Vector3f &pos, bool &error);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

};

}

#endif
