//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "BackendSettings.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "NumericalClassProperty.h"
#include "SeparatorClassProperty.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

static std::string getParamerizationModeDescr( const card32 index )
{
	if ( index == MESH_PARAMETERIZATION_MODE_SPHERE ) return "Sphere";
	else if ( index == MESH_PARAMETERIZATION_MODE_PLANE ) return "Plane";
	else return "Undefined";
}

static std::string getModeDesc( const card32 index )
{
	if ( index == MC_REGULAR ) return "Regular";
	else if ( index == MC_HIERARCHICAL ) return "Hierarchical";
	else return "Undefined";
}

IMPLEMENT_X4_CLASS( BackendSettings, 4 ) 
{
	BEGIN_CLASS_INIT( BackendSettings )
	ADD_SEPARATOR( "Upsampling" )
	ADD_CARD32_PROP( numUpsamplingIterations, 1 )
	ADD_BOOLEAN_PROP( doUpsample, 0 )
	ADD_CARD32_PROP( numSurfRecIterations, 1 )
	ADD_BOOLEAN_PROP( doNewSurfaceReconstruction, 0 )
	ADD_CARD32_PROP( dataSubsampling, 2 )
	ADD_SEPARATOR( "PCA Reconstruction" )
	ADD_FLOAT32_PROP( patchSamplingSpacing, 0 )
	ADD_CARD32_PROP( numEigenVectors, 0 )
	ADD_FLOAT32_PROP( dataPointsInfluence, 3 )
	ADD_CARD32_PROP( maxNumFrames, 3 )
	ADD_BOOLEAN_PROP( doPCAReconstruction, 0 )
	//ADD_BOOLEAN_PROP( debugSavePatchPC, 0 )
	//ADD_BOOLEAN_PROP( debugSaveParts, 0 )
	//ADD_BOOLEAN_PROP( debugSaveDeformedDataPoints, 0 )
	//ADD_BOOLEAN_PROP( debugSaveResiduals, 0 )
	//ADD_BOOLEAN_PROP( debugSaveEigenVectors, 0 )
	ADD_SEPARATOR( "Triangulation" )
	ADD_CARD32_PROP_UPDATE_DESCR( mcMode, 0, NULL, getModeDesc, 2, NULL )
	ADD_BOOLEAN_PROP( doMarchingCubes, 0 )
	ADD_BOOLEAN_PROP( doParameterization, 0 )
	ADD_BOOLEAN_PROP( doClipBorder, 0 )
	ADD_BOOLEAN_PROP( doExtrapolation, 0 )
	ADD_CARD32_PROP( mcOctreeLevel, 0 )
	ADD_FLOAT32_PROP( influence, 0 )
	ADD_CARD32_PROP( materialIndex, 0 )
	ADD_FLOAT32_PROP( texCoordScale, 0 )
	ADD_CARD32_PROP_UPDATE_DESCR( paramerizationMode, 0, NULL, getParamerizationModeDescr, 2, NULL )
	ADD_FLOAT32_PROP( clipInfluence, 0 )
	ADD_CARD32_PROP( borderNumCones, 0 )
	ADD_CARD32_PROP( borderConesThreshold, 0 )
	ADD_FLOAT32_PROP( borderInfluence, 0 )
	ADD_FLOAT32_PROP( borderTopoInfluence, 0 )
	ADD_CARD32_PROP( connectedComponentSize, 0 )
	ADD_BOOLEAN_PROP( saveMeshOOC, 4 )
	ADD_STRING_PROP( oocFilename, 4 )
}

BackendSettings::BackendSettings()
{
	// Upsampling
	numUpsamplingIterations = 0;
	doUpsample = true;
	numSurfRecIterations = 0;
	doNewSurfaceReconstruction = false;
	dataSubsampling = 3;

	// PCA-Reconstruction
	patchSamplingSpacing = 0.3f;
	numEigenVectors = 2;
	dataPointsInfluence = 2.0f;
	maxNumFrames = 1000;
	doPCAReconstruction = true;
	debugSavePatchPC = false;
	debugSaveParts = false;
	debugSaveDeformedDataPoints = false;
	debugSaveResiduals = false;
	debugSaveEigenVectors = false;

	// Triangulation
	doMarchingCubes = false;
	doParameterization = false;
	doClipBorder = false;
	doExtrapolation = false;
	meshNodeName = "meshes";
	mcMode = MC_REGULAR;
	mcOctreeLevel = 4;
	influence = 2.5f;
	normalInfluence = 4.0f;
	materialIndex = 3;
	texCoordScale = 1.0f;
	paramerizationMode = MESH_PARAMETERIZATION_MODE_SPHERE;
	clipInfluence = 2.0f;
	borderNumCones = 7;
	borderConesThreshold = 6;
	borderInfluence = 4.0f;
	borderTopoInfluence = 2.0f;
	connectedComponentSize = 10;
	saveMeshOOC = true;
	oocFilename = "c:/temp/";
}

BackendSettings::~BackendSettings()
{

}




