//---------------------------------------------------------------------------
#ifndef BackendSettingsH
#define BackendSettingsH
//---------------------------------------------------------------------------
#include "x3deformation.h"
#include "TriangleMeshTools.h"
//---------------------------------------------------------------------------

namespace X4 {

#define MC_REGULAR 0
#define MC_HIERARCHICAL 1

///
///   Settings for the triangulation functionality
///
class X3DEFORMATION_API BackendSettings : public Persistent 
{
	X4_CLASS( BackendSettings )
 
	public:
		BackendSettings();
		~BackendSettings();

		// Upsampling
		card32 numUpsamplingIterations;		// Number of upsampling runs (sampleSpacing /= 2)
		bool doUpsample;
		card32 numSurfRecIterations;		// Number of reconstruction runs
		bool doNewSurfaceReconstruction;
		card32 dataSubsampling;				// Only use every nth data frame

		// Small scale PCA reconstruction
		float32 patchSamplingSpacing;		// Sampling spacing of seed points for patches
		card32 numEigenVectors;				// Number of eigenvectors used in projection
		float32 dataPointsInfluence;		// Influence of data points; used to compute residuals std::vector
		card32 maxNumFrames;				// Limits the numbre of frames considered (debugging)
		bool doPCAReconstruction;			// Enable/disable PCA reconstruction
		bool debugSavePatchPC;				// These flags were used during development; not used any more
		bool debugSaveParts;
		bool debugSaveDeformedDataPoints;
		bool debugSaveResiduals;
		bool debugSaveEigenVectors;

		// Triangulation
		std::string meshNodeName;			// Name of mesh animation node
		card32 mcMode;					// Regular vs. Hierarchical
		bool doMarchingCubes;			// Run marching cubes
		card32 mcOctreeLevel;			// Subdivision level of mc octree
		float32 influence;				// Influence radius (times sample spacing) used in MC
		float32 normalInfluence;		// Influence of normals
		bool doParameterization;		// Run parameterization of urshape
		card32 materialIndex;			// Index of assigned material
		float32 texCoordScale;			// Scaling factor of texture coordinates
		card32 paramerizationMode;		// sphere vs. plane
		bool doClipBorder;				// Clip at border 
		float32 clipInfluence;			// Max distance to closest clipping border point
		float32 borderInfluence;		// Influence radius in surfel candidate detection (times sampling spacing)
		card32 borderNumCones;			// Number of cones used to detect border surfels; default = 6
		card32 borderConesThreshold;	// Number of filled cones for non-border surfel; default = 5
		float32 borderTopoInfluence;	// topological connection influence between border points
		card32 connectedComponentSize;	// Minimum number of topologically connected points in valid border component
		bool doExtrapolation;			// Extrapolate mesh frames
		bool saveMeshOOC;
		std::string oocFilename;
};

}

#endif