#ifndef PCCDeformationSurfaceReconstructionH
#define PCCDeformationSurfaceReconstructionH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
#include "InCorePCTopologyGraph.h"
#include "RenderingTool.h"
#include "SurfelGraphInit2.h"
#include "SurfelOptimizer.h"
#include "SGEPointCloud.h"
#include "SGESurfelGraph.h"
#include "ICPConstraints.h"
#include "SurfelGraphTriangulator.h"
//---------------------------------------------------------------------------

namespace X4
{

/** Test surface reconstruction module **/
class X3DEFORMATION_API PCCDeformationSurfaceReconstruction : public PCCommand
{
	X4_CLASS( PCCDeformationSurfaceReconstruction )

	private:
		ARSettings* settings;
		BackendSettings* triangulationSettings;
		SurfelGraphInit2* initializer;
		SurfelOptimizer* optimizer;
		SGESurfelGraph* surfels;
		SGEPointCloud* data;
		ICPConstraints* icpConstraints;
		SurfelGraphTriangulator* triangulator;

		// Rendering
		int32 monitorPoint;
		RenderingTool* rt;
		float32 renderScale;

		// Show
		bool showSurfelGraph;
		bool showSurfelFrame;
		bool showSurfelFunction;
		bool showHSConstraints;
		bool showBorderClip;
		bool showMC;

		int32 visI, visJ, visK;

		// Debug
		bool createICPConstraints;
		bool createHSContraint;

		Vector3f hsPoint;				// Central point of constraint
		Vector3f hsNormal;				// Normal of constraint	
		float32 hsWeight;				// Weight
		card32 hsNumSamples;			// Number of constraints within
		float32 hsRadius;				// Spreading of constraint samples
		std::vector<Plane3f> hsConstraints;	// Container for the constraints

	public:
		PCCDeformationSurfaceReconstruction();
		~PCCDeformationSurfaceReconstruction();

		void assign( const Object* obj, X4CopyContext* context = NULL );
		void execute( Scene *scene );
		void glDrawTool( GLContext* context );
		void connectScene( Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget ); 

		// Init surfel graph
		void initialize();
		// Optimize surfelgraph
		void optimize();
		// Create triangle mesh
		void triangulate();
		// Upsample
		void upsample();
};

}

#endif