//---------------------------------------------------------------------------
#ifndef ShapeReconstructionH
#define ShapeReconstructionH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "AROptimizationTool.h"
#include "ARConstraintsInterface.h"
//---------------------------------------------------------------------------

namespace X4 {

class SGEShape;
class ARSettings;
class ShapeEvaluator;

///
///   this is the base class of all surface reconstruction modules 
///   in the animation reconstruction 2.0 framework
///
class X3DEFORMATION_API ShapeReconstruction : public AROptimizationTool, public ARConstraintsInterface {
	X4_ABSTRACT_CLASS( ShapeReconstruction )

 public:
	ShapeReconstruction();

	/// create an evaluator that is compatible with this solver (will work on the same context)
	virtual ShapeEvaluator *createEvaluator() = 0;

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	/// call this method to begin specifying constraints
	virtual void beginConstraints() = 0;
	
	// --- actual add constraint methods are in interface ARConstraintsInterface
	
	/// solve with given constraints and boundary conditions
	/// has to pair with "beginConstraints" (this is implicitly "endConstraints") 
	virtual void solve() = 0;

   ~ShapeReconstruction();
};

}

#endif