//---------------------------------------------------------------------------
#ifndef SmallScaleReconstructionH
#define SmallScaleReconstructionH
//---------------------------------------------------------------------------
#include "x3deformation.h"
#include "TriangleMeshTools.h"
#include "ARSequence.h"
#include "ArSettings.h"
#include "SGSimpleAnimationNode.h"
#include "PointSetKNNQuery.h"
#include "BackendSettings.h"
#include "InCorePCTopologyGraph.h"
//---------------------------------------------------------------------------

namespace X4 {

///
/// Reconstruct small scale features in the data using principle component analysis
///
class X3DEFORMATION_API SmallScaleReconstruction : public Persistent 
{
	X4_CLASS( SmallScaleReconstruction )

	private:
		BackendSettings* backendSettings;
			
		// Compute residuum field for all points in 'sample' for frame 'time'
		DynamicVector<float32> computeResidualVector( 
			const card32 time,
			PointSet* deformedDataPS,
			PointSet* samplesPS,
			const float32 samplingSpacing );
		// Deform a point into frame 'time' from urshape frame
		void deformFromUrshapeToFrame( 
			Vector3f& deformedPoint,		// deformed points
			Vector3f& deformedNormal,		// deformed normals
			const card32 time, 
			const Vector3f p, 
			DeformationEvaluator *evaluator,
			PointSet* urShapePS,
			InCorePCTopologyGraph* urShapeTopology,
			PointSetKNNQuery& urShapeKNN, 
			const float32 samplingSpacing );


		// DEBUGGING
		void debugAppendDeformedDataPoints( 
			SGListNode* listNode, 
			std::vector<PointSet*>& deformedDataPoints );
		void debugAppendEigenVectors( 
			SGListNode* listNode, 
			std::vector<PointSet*>& parts,
			std::vector< std::vector< DynamicVector<float32> > >& eigenVectors, 
			std::vector< DynamicVector<float32> >& eigenValues );
		void debugAppendResiduals( 
			SGListNode* listNode, 
			std::vector<PointSet*>& parts, 
			std::vector< DynamicVector<float32>* >& partResiduals,
			const card32 numFrames );
		void debugAppendPatchPC( 
			SGListNode* listNode, 
			UnstructuredInCorePointCloud* resampledPatchPC );
		void debugAppendParts( 
			SGListNode* listNode, 
			std::vector<PointSet*>& parts );

	public:
		SmallScaleReconstruction();
		~SmallScaleReconstruction();

		// Apply reconstruction; returns a vertor with a reconstructed point set for each frame
		void reconstruct(
			UnstructuredInCorePointCloud* originalInUrshapeFrame,		// Original point set, the reconstruction is built upon
			card32 startFrame,
			card32 endFrame,
			ARSequence* seq, 
			std::vector<PointCloud*>& dataPoints,
			DeformationEvaluator *evaluator,
			const float32 samplingSpacing,
			SGListNode* listNode
		);

		// Deform all data points in 'dataPoints' of frame 'time' into urshape's local coordinate system
		static PointSet* deformDataPoints(
			PointCloud* dataPoints,
			const card32 time,
			PointSet* urShapePS,
			DeformationEvaluator *evaluator, 
			const float32 samplingSpacing );

		/** MEMBER ACCESS **/
		inline void setBackendSettings( BackendSettings* backendSettings ){ this->backendSettings = backendSettings; }
};

}

#endif

/*
static void testDeform( PointSet* ps, const card32 time );
static void testDeformHighFrequency( PointSet* ps, const card32 time );
static SGSimpleAnimationNode* createTestExample( const card32 numFrames, const card32 numPoints );
*/