//---------------------------------------------------------------------------
#ifndef SurfelGraphInitH
#define SurfelGraphInitH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "TensorVoting.h"
#include "ARSettings.h"
#include "SGESurfelGraph.h"
#include "SGEPointCloud.h"
//---------------------------------------------------------------------------

namespace X4 {

//
// Creates a surfel graph entity from a data point cloud
//
class X3DEFORMATION_API SurfelGraphInit : public Persistent {
	
	X4_CLASS( SurfelGraphInit )

	private:
		ARSettings* settings;
			
		bool doPreprocessData;		// Apply preprocessing in the data
		bool doCreateSurfelGraph;	// Create a valid surfel graph data structure including all required attachments
		bool doCreate;				// Create initial surfel graph from data
		bool doResample;			// Resample the surfel graph according to the sampling spacing parameter
		bool doInitFrames;			// Init local coordinate systems

		void createSurfelGraph( SGESurfelGraph* surfels );
		bool applyTensorVoting( SGEPointCloud* data );
		bool createSurfels( SGESurfelGraph* surfels, SGEPointCloud* data );
		bool resample( SGESurfelGraph* surfels );
		bool initFrames( SGESurfelGraph* surfels );

	public:
		SurfelGraphInit();
		~SurfelGraphInit();

		// Apply the complete initialization routine; return true on success
		bool initialize( SGESurfelGraph* surfels, SGEPointCloud* data );

		/*** MEMBER ACCESS ***/
		inline void setSettings( ARSettings* settings ){ this->settings = settings; }

	   bool getDoPreprocessData() const {return doPreprocessData;}
	   void setDoPreprocessData(bool v) {doPreprocessData=v;}

	   bool getDoCreateSurfelGraph() const {return doCreateSurfelGraph;}
	   void setDoCreateSurfelGraph(bool v) {doCreateSurfelGraph=v;}

	   bool getDoCreate() const {return doCreate;}
	   void setDoCreate(bool v) {doCreate=v;}

	   bool getDoResample() const {return doResample;}
	   void setDoResample(bool v) {doResample=v;}

	   bool getDoInitFrames() const {return doInitFrames;}
	   void setDoInitFrames(bool v) {doInitFrames=v;}
};

}

#endif
