//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SurfelOptEvaluator.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SurfelOptimizationContext.h"
#include "SGESurfelGraph.h"
#include "LocalSurfaceFunction.h"
#include "ARSettings.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( SurfelOptEvaluator , 0 ) {
	BEGIN_CLASS_INIT( SurfelOptEvaluator );
}

// ----

SurfelOptEvaluator::SurfelOptEvaluator() {
}

SurfelOptEvaluator::~SurfelOptEvaluator() {
}




AROptimizationContext *SurfelOptEvaluator::createOptContext() 
{
   notImplemented();
   return NULL;
}

void SurfelOptEvaluator::setup(AROptimizationContext *context, const ARSettings *settings)
{
   SurfelOptimizationContext *so_context = dynamic_cast<SurfelOptimizationContext*>(context);
   if (!so_context) {
      throw X4Exception("SurfelOptEvaluator::setup() - context NULL or of wrong type.");
   }
   this->context = so_context;
   this->settings = settings;
}


PointCloud *SurfelOptEvaluator::evaluate() 
{
	PointSet* surfelPS = context->getSurfels();
	if (   !surfelPS 
	    || !surfelPS->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)
	    || !surfelPS->getDescr()->providesAttribute("normal", 3, VAD::DATA_FORMAT_FLOAT32)
	    || !surfelPS->getDescr()->providesAttribute("tangent_u", 3, VAD::DATA_FORMAT_FLOAT32)
	    || !surfelPS->getDescr()->providesAttribute("tangent_v", 3, VAD::DATA_FORMAT_FLOAT32)
	    || !surfelPS->getDescr()->providesAttribute("coeffs") )
	{
	   throw X4Exception("SurfelOptEvaluator::evaluate() - wrong surfels format in context.");
	}
	AAT SURFEL_POSITION = surfelPS->getAAT( "position" );
	AAT SURFEL_NORMAL = surfelPS->getAAT( "normal" );
	AAT SURFEL_TANGENT_U = surfelPS->getAAT( "tangent_u" );
	AAT SURFEL_TANGENT_V = surfelPS->getAAT( "tangent_v" );
	AAT SURFEL_COEFFS = surfelPS->getAAT( "coeffs" );

   PointSet *resultPS = (PointSet*)(surfelPS->copy());

	// Project points
	for ( unsigned int pIndex = 0; pIndex < surfelPS->getNumEntries(); pIndex++ ){
		StaticVector<float32,SURFEL_NUM_BASIS_FUNCTIONS>& coeffs = surfelPS->get<float32,SURFEL_NUM_BASIS_FUNCTIONS>( pIndex, SURFEL_COEFFS );
		const Vector3f& p = surfelPS->get3f( pIndex, SURFEL_POSITION );
		Matrix3f T;
		T[0] = surfelPS->get3f( pIndex, SURFEL_TANGENT_U );
		T[1] = surfelPS->get3f( pIndex, SURFEL_TANGENT_V );
		T[2] = surfelPS->get3f( pIndex, SURFEL_NORMAL );
		const Vector3f& newP = ( T * makeVector3f( 0, 0, LocalSurfaceFunction<SURFEL_NUM_BASIS_FUNCTIONS>::f( coeffs, 0, 0 ) ) ) * settings->topSurfelSpacing + p;
		const Vector3f& localNormal = LocalSurfaceFunction<SURFEL_NUM_BASIS_FUNCTIONS>::getNormal( coeffs, 0, 0 );
		Vector3f newNormal = T * localNormal;
		Vector3f newTangentU = T[1].crossProduct( newNormal );
		Vector3f& newTangentV = newNormal.crossProduct( newTangentU );
		newNormal.normalize();
		newTangentU.normalize();
		newTangentV.normalize();
		resultPS->set3f( pIndex, SURFEL_POSITION, newP );
		resultPS->set3f( pIndex, SURFEL_NORMAL, newNormal );
		resultPS->set3f( pIndex, SURFEL_TANGENT_U, newTangentU );
		resultPS->set3f( pIndex, SURFEL_TANGENT_V, newTangentV );
		coeffs[0] = 0;
		resultPS->set<float32,SURFEL_NUM_BASIS_FUNCTIONS>( pIndex, SURFEL_COEFFS, coeffs );
	}
	
	UnstructuredInCorePointCloud *result = new UnstructuredInCorePointCloud;
	result->setPointSet(resultPS);
	return result;
}
