//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SurfelOptimizationContext.h"
#include "TopologyAttachment.h"
#include "SGEPointCloud.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_ABSTRACT_CLASS( SurfelOptimizationContext , 0 ) 
{
	BEGIN_CLASS_INIT( SurfelOptimizationContext );
}

SurfelOptimizationContext::SurfelOptimizationContext() 
{
	surfels = NULL;
	topologyGraph = NULL;
}


SurfelOptimizationContext::~SurfelOptimizationContext() 
{
}

void SurfelOptimizationContext::initializeFromShape( SceneObject *obj, const ARSettings *params )
{
	UnstructuredInCorePointCloud* upc = dynamic_cast<UnstructuredInCorePointCloud*>( obj );
	if ( upc == NULL ) {
		throw X4Exception("SurfelOptimizationContext::initializeFromShape() - only works on UnstructuredInCorePointClouds");
	}

	// Extract point set
	surfels = upc->getPointSet();

	// Extract topology
	AttachedData *att = upc->getAttachments()->getData( "TopologyAtt" );
	std::string msg;
	if ( !att ) {
		throw X4Exception("SurfelOptimizationContext::initializeFromShape() - TopologyAttachment is missing");
	}
	TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(att);
	if (!topAtt) {
      throw X4Exception("SurfelOptimizationContext::initializeFromShape() - TopologyAttachment is of wrong type (not TopologyAttachment)");
	}
	topologyGraph = topAtt->getTopology();
	if ( topologyGraph == NULL ) {
      throw X4Exception("SurfelOptimizationContext::initializeFromShape() - InCorePCTopologyGraph is NULL");
	}
}




