#ifndef DebugAnimationRendererH
#define DebugAnimationRendererH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Renderer.h"
//---------------------------------------------------------------------------

namespace X4
{
class SceneObject;
class SGObjectNode;
class MemoryManager;
class RenderingTool;
/**
	debug renderer for animation reconstruction
*/
class X3DEFORMATION_API DebugAnimationRenderer : public Renderer
{
	X4_CLASS( DebugAnimationRenderer )

public:
	DebugAnimationRenderer(void);
	~DebugAnimationRenderer(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual void connect(Scene *scene, GLContext* ctxt);
	virtual void render(card32 viewportWidth, card32 viewportHeight);
	virtual void disconnect(GLContext* ctxt);


private:

	/// every scene object of the animation within the range [a,b] is rendered with this method
	/// a ~= currentframe - numrenderedFrames/2, b = currentframe + numRenderedFrames/2
	/// each frame lives in a given coordinate system
	void renderSceneObject( SGObjectNode *node, SceneObject * so, Matrix4f frame);

	/// renderes stuff between two frames (e.g. correspondences)
	/// both scene objects and theirs frames are provided 
	void renderInBetweenStuff(SceneObject * so1, Matrix4f frame1, SceneObject * so2, Matrix4f frame2);


	// properties
	card32	m_NumRenderedFrames;
	float32	m_RenderScale;
	bool		m_DrawFeatureCorrespondences;
	bool		m_DrawFeaturePoints;


	// members
	card32	m_ScreenWidth, m_ScreenHeight;
	MemoryManager * m_MemoryManager;
	RenderingTool	*	m_RenderingTool;

};

}

#endif