//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SGEMeshlessDeformationSequence.h"
//---------------------------------------------------------------------------
//#include "CopyObjectProperties.h"
#include "VertexDescriptor.h"
#include "DeformationTools.h"
#include "PointCloud.h"
#include "SGListNode.h"
#include "SGObjectNode.h"
//#include "VolumeObject.h"
//#include "RegularInCoreVolume.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( SGEMeshlessDeformationSequence , 0 ) {
	BEGIN_CLASS_INIT( SGEMeshlessDeformationSequence );
    ADD_OBJECT_PROP(deformationChannels, 0, VertexDescriptor::getClass(), true)
}

// ----

SGEMeshlessDeformationSequence::SGEMeshlessDeformationSequence() {
   deformationChannels = NULL;
}

SGEMeshlessDeformationSequence::~SGEMeshlessDeformationSequence() {
   delete deformationChannels;
}




bool SGEMeshlessDeformationSequence::check(std::string &msg) const
{
   if (!SGEDeformationSequence::check(msg)) return false;
   SceneGraphNode *node = getSceneGraphNode();
   SGListNode *list = dynamic_cast<SGListNode*>(node);
   x4Assert(list);
   
   mpcard n = list->getNumChildNodes(NULL);
   
   for (mpcard i=0; i<n; i++) {
      SceneGraphNode *node = list->getChildNode(NULL, i);
      SGObjectNode *on = dynamic_cast<SGObjectNode*>(node);
      x4Assert(on);
      SceneObject *so = on->getSceneObject();
      x4Assert(so);
      UnstructuredInCorePointCloud *pc = dynamic_cast<UnstructuredInCorePointCloud*>(so);
      if (!pc) {
         msg = "one of the objects in the sequence is not a point cloud";
         return false;
      }
      if (deformationChannels) {
         if (!compareDescr(pc->getDescr(), deformationChannels )) {
            msg = "vertex descriptor of deformed point cloud does not match";
            return false;
         }
      }
   }
   return true;	
}

SceneObject *SGEMeshlessDeformationSequence::getSceneObject(mpcard frame) const
{
   SceneGraphNode *node = getSceneGraphNode();

   SGListNode *list = dynamic_cast<SGListNode*>(node);
   if (!list) 
      throw EInvalidState("SGEMeshlessDeformationSequence::getSceneObject - expect path to SGListNode.");

   mpcard n = list->getNumChildNodes(NULL);
   if (frame >= n) 
      throw ERangeCheck("SGEMeshlessDeformationSequence::getSceneObject - index out of range.");

   SceneGraphNode *subnode = list->getChildNode(NULL, frame);
   SGObjectNode *on = dynamic_cast<SGObjectNode*>(subnode);
   if (!on) 
      throw EInvalidState("SGEMeshlessDeformationSequence::getSceneObject - list contains non-volume node.");

   SceneObject *so = dynamic_cast<UnstructuredInCorePointCloud*>(on->getSceneObject());
   if (!so) 
      throw EInvalidState("SGEMeshlessDeformationSequence::getSceneObject - the scene object at the specified index is either NULL or of wrong type (not volume object).");
   UnstructuredInCorePointCloud *pc = dynamic_cast<UnstructuredInCorePointCloud*>(so);
   return pc;
}

UnstructuredInCorePointCloud *SGEMeshlessDeformationSequence::getDeformedNodePositions(mpcard frame) const 
{
   return (UnstructuredInCorePointCloud*)getSceneObject(frame);
}

SceneObjectIterator *SGEMeshlessDeformationSequence::createIterator(mpcard frame, SceneObjectIteratorType type) const
{
   SceneObject *so = getSceneObject(frame);
   return so->createIterator(type);
}

void SGEMeshlessDeformationSequence::setDescriptor(VertexDescriptor *vd) 
{
   deformationChannels = vd;
}

const VertexDescriptor *SGEMeshlessDeformationSequence::getDescriptor() const
{
   return deformationChannels;
}
