//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SGESurfelGraph.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "AttachedData.h"
#include "TopologyAttachment.h"
#include "UnstructuredInCorePointCloud.h"
#include "InCorePCTopologyGraph.h"
#include "DeformationTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( SGESurfelGraph , 0 ) {
	BEGIN_CLASS_INIT( SGESurfelGraph );
	ADD_OBJECT_PROP(surfelChannels, 0, VertexDescriptor::getClass(), true)
	ADD_OBJECT_PROP(graphChannels, 0, VertexDescriptor::getClass(), true)
}

// ----

SGESurfelGraph::SGESurfelGraph() 
{
	graphChannels = new VertexDescriptor();
	graphChannels->pushAttrib( VAD( "vindex", 2, VAD::DATA_FORMAT_INT32 ) );
	surfelChannels = new VertexDescriptor();
	surfelChannels->pushAttrib( mVAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
	surfelChannels->pushAttrib( mVAD( "normal", 3, VAD::DATA_FORMAT_FLOAT32 ) );
	surfelChannels->pushAttrib( mVAD( "flags", 1, VAD::DATA_FORMAT_INT32 ) );
	surfelChannels->pushAttrib( mVAD( "tangent_u", 3, VAD::DATA_FORMAT_FLOAT32 ) );
	surfelChannels->pushAttrib( mVAD( "tangent_v", 3, VAD::DATA_FORMAT_FLOAT32 ) );
	surfelChannels->pushAttrib( mVAD( "coeffs", SURFEL_NUM_BASIS_FUNCTIONS, VAD::DATA_FORMAT_FLOAT32 ) );
}

SGESurfelGraph::~SGESurfelGraph() {
	delete surfelChannels;
	delete graphChannels;
}

bool SGESurfelGraph::check(std::string &msg) const
{
	if (!SGEShape::check(msg)) {
		return false;
	}
	SceneObject *so = getSceneObject();
	UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(so);
	if (!upc) {
		msg = "scene object is not an UnstructureInCorePointCloud (or descendant)";
		return false;
	}
	if (!compareDescr(upc->getDescr(), surfelChannels)) {
		msg = "attribute channels of Point Cloud do not match the requirements.";
		return false;
	} 
	AttachedData *att = upc->getAttachments()->getData("TopologyAtt");
	if (!att) {
		msg = "TopologyAttachment is missing";
		return false;
	}
	TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(att);
	if (!topAtt) {
		msg = "TopologyAttachment is of wrong type (not TopologyAttachment)";
		return false;
	}
	InCorePCTopologyGraph *top = topAtt->getTopology();
	if (top == NULL) {
		msg = "InCorePCTopologyGraph is NULL";
		return false;	
	}
	if (!compareDescr(top->getEdges()->getDescr(), graphChannels)) {
		msg = "attribute channels of topology graph do not match the requirements.";
		return false;
	} 
	return true;
}

UnstructuredInCorePointCloud *SGESurfelGraph::getPointCloud() const 
{
	SceneObject *so = getSceneObject();
	UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(so);
	if (upc == NULL) {
		throw EInvalidState("SGESurfelGraph::getPointSet() - point cloud of wrong type (call check before using)");
	}
	return upc;
}

PointSet *SGESurfelGraph::getPointSet() const
{
	return getPointCloud()->getPointSet();
}

InCorePCTopologyGraph *SGESurfelGraph::getTopology() const
{
	TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(getPointCloud()->getAttachments()->getData("TopologyAtt"));
	if (!topAtt) {
		throw EInvalidState("SGESurfelGraph::getTopology() - TopologyAttachment is missing");
	}
	return topAtt->getTopology();
}

void SGESurfelGraph::setSurfelChannels(VertexDescriptor *cn)
{
   surfelChannels = cn;
}

const VertexDescriptor *SGESurfelGraph::getSurfelChannels() const
{
   return surfelChannels;
}

void SGESurfelGraph::setGraphChannels(VertexDescriptor *cn) 
{
   graphChannels = cn;
}

const VertexDescriptor *SGESurfelGraph::getGraphChannels() const
{
   return graphChannels;
}

/*
void SGESurfelGraph::setPointSet( PointSet* ps )
{
	UnstructuredInCorePointCloud* pc = getPointCloud();
	if ( pc == NULL ){
		debugOutput << "Invalid point cloud.\n"; debugOutput.flush();
		return;
	}
	pc->setPointSet( ps );
}
*/

/*
void SGESurfelGraph::setTopology( InCorePCTopologyGraph* topology )
{
	TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(getPointCloud()->getAttachments()->getData("TopologyAtt"));
	if (!topAtt) {
		throw EInvalidState("SGESurfelGraph::setTopology() - TopologyAttachment is missing");
	}
	topAtt->setTopology( topology );
}
*/

/*
void SGESurfelGraph::setOctree( AttachedIndexedOctree* newOctree )
{
	AttachedIndexedOctree* octree = dynamic_cast<AttachedIndexedOctree*>(getPointCloud()->getAttachments()->getData("SimpleOctree"));
	if ( !octree ) {
		throw EInvalidState("SGESurfelGraph::getOctree() - Octree attachment is missing");
	} else {
		delete octree;
		octree = newOctree;
	}
}
*/