//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SGEVolumeGridDeformationSequence.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "VertexDescriptor.h"
#include "DeformationTools.h"
#include "VolumeObject.h"
#include "SGListNode.h"
#include "SGObjectNode.h"
#include "VolumeObject.h"
#include "RegularInCoreVolume.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( SGEVolumeGridDeformationSequence , 0 ) {
	BEGIN_CLASS_INIT( SGEVolumeGridDeformationSequence );
   ADD_OBJECT_PROP(volumeChannels, 0, VertexDescriptor::getClass(), true)
}

// ----

SGEVolumeGridDeformationSequence::SGEVolumeGridDeformationSequence() {
   volumeChannels = NULL;
}

SGEVolumeGridDeformationSequence::~SGEVolumeGridDeformationSequence() {
   delete volumeChannels;
}




bool SGEVolumeGridDeformationSequence::check(std::string &msg) const
{
   if (!SGEDeformationSequence::check(msg)) return false;
   SceneGraphNode *node = getSceneGraphNode();
   SGListNode *list = dynamic_cast<SGListNode*>(node);
   x4Assert(list);
   
   mpcard n = list->getNumChildNodes(NULL);
   
   for (mpcard i=0; i<n; i++) {
      SceneGraphNode *node = list->getChildNode(NULL, i);
      SGObjectNode *on = dynamic_cast<SGObjectNode*>(node);
      x4Assert(on);
      SceneObject *so = on->getSceneObject();
      x4Assert(so);
      VolumeObject *vo = dynamic_cast<VolumeObject*>(so);
      if (!vo) {
         msg = "one of the objects in the sequence is not a volume object";
         return false;
      }
      if (volumeChannels) {
         if (!compareDescr(vo->getDescr(), volumeChannels )) {
            msg = "vertex descriptor of volume object does not match";
            return false;
         }
      }
   }
   return true;	
}

SceneObject *SGEVolumeGridDeformationSequence::getSceneObject(mpcard frame) const
{
   SceneGraphNode *node = getSceneGraphNode();

   SGListNode *list = dynamic_cast<SGListNode*>(node);
   if (!list) 
      throw EInvalidState("SGEVolumeGridDeformationSequence::getSceneObject - expect path to SGListNode.");

   mpcard n = list->getNumChildNodes(NULL);
   if (frame >= n) 
      throw ERangeCheck("SGEVolumeGridDeformationSequence::getSceneObject - index out of range.");

   SceneGraphNode *subnode = list->getChildNode(NULL, frame);
   SGObjectNode *on = dynamic_cast<SGObjectNode*>(subnode);
   if (!on) 
      throw EInvalidState("SGEVolumeGridDeformationSequence::getSceneObject - list contains non-volume node.");

   SceneObject *so = dynamic_cast<VolumeObject*>(on->getSceneObject());
   if (!so) 
      throw EInvalidState("SGEVolumeGridDeformationSequence::getSceneObject - the scene object at the specified index is either NULL or of wrong type (not volume object).");
   RegularInCoreVolume *vo = dynamic_cast<RegularInCoreVolume*>(so);
   return vo;
}

RegularInCoreVolume *SGEVolumeGridDeformationSequence::getRegularInCoreVolume(mpcard frame) const 
{
   return (RegularInCoreVolume*)getSceneObject(frame);
}

SceneObjectIterator *SGEVolumeGridDeformationSequence::createIterator(mpcard frame, SceneObjectIteratorType type) const
{
   SceneObject *so = getSceneObject(frame);
   return so->createIterator(type);
}

void SGEVolumeGridDeformationSequence::setDescriptor(VertexDescriptor *vd) 
{
   volumeChannels = vd;
}

const VertexDescriptor *SGEVolumeGridDeformationSequence::getDescriptor() const
{
   return volumeChannels;
}
