//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARSequence.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ARDeformationOptimizationContext.h"
#include "DeformationEvaluator.h"
#include "PointCloud.h"
#include "ShapeReconstruction.h"
#include "DeformationSolver.h"
#include "SGEDeformationSequence.h"
#include "SGEShape.h"
#include "AREvaluateDeformation.h"
#include "SGDeformationNodeAR.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( ARSequence , 0 ) {
	BEGIN_CLASS_INIT( ARSequence );
   ADD_OBJECT_PROP(deformation, 0, ARDeformationOptimizationContext::getClass(), false)
   ADD_OBJECT_PROP(urshape, 0, PointCloud::getClass(), false)
}

// ----

ARSequence::ARSequence() {
   deformation = NULL;
   urshape = NULL;
}

void ARSequence::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

void ARSequence::clearAndSetup(
   ARDeformationOptimizationContext *deformation,
   PointCloud *urshape)
{
   this->deformation = deformation;
   this->urshape = urshape;
}

void ARSequence::setupFromSceneGraph(
   Scene *scene,
   std::string urshapePath, std::string deformationPath, 
   ShapeReconstruction *shapeSolver, DeformationSolver *deformSolver)
{
   SGDeformationNodeAR *deformationNode = dynamic_cast<SGDeformationNodeAR*>(getSceneGraphNodeByName(scene, deformationPath));
   if (!deformationNode) {
      throw X4Exception("ARSequence::setupFromSceneGraph() - deformation node not found.");
   }
   deformation = deformationNode->getDeformation();
   x4Assert(deformation);
   urshape = getPointCloud(scene, urshapePath);
   if (!urshape) {
      throw ETypeMismatch("ARSequence::setupFromSceneGraph() - urshape is not a point cloud.");
   }
}

float32 ARSequence::getDuration() {
   return deformation->getDuration();
}

PointCloud *ARSequence::evaluate(float32 time, DeformationEvaluator *evaluator, ARSettings *settings) {
   return AREvaluateDeformation::evaluatePC(urshape, deformation, evaluator, time, settings);
}

PointCloud *ARSequence::evaluateStart(DeformationEvaluator *evaluator, ARSettings *settings) 
{
   return evaluate(0, evaluator, settings);
}

PointCloud *ARSequence::evaluateEnd(DeformationEvaluator *evaluator, ARSettings *settings) 
{
   return evaluate(getDuration(), evaluator, settings);
}






