//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARChangeUrshape.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ARDeformationOptimizationContext.h"
#include "DeformationEvaluator.h"
#include "DeformationSolver.h"
#include "PointCloud.h"
#include "ARSettings.h"
#include "PointSetKNNQuery.h"
#include "MMDeformationSolver.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



ARDeformationOptimizationContext *ARChangeUrshape::recomputeDeformation(
   PointCloud *newUrshape,
   PointCloud *oldUrshape,
   ARDeformationOptimizationContext *deformation,
   DeformationSolver *solver,
   ARSettings *settings)
{
   DeformationEvaluator *eval = solver->createEvaluator();
   eval->setup(deformation, settings);
   ARDeformationOptimizationContext *result = (ARDeformationOptimizationContext*)solver->createOptContext();
   debugOutput << "recreating nodes...\n";
   result->initializeFromShape(newUrshape, makeVector2f(0, deformation->getDuration()), settings);
   solver->setup(result, settings);
   debugOutput << "created " << ((MMDeformationSolver*)solver)->getContext()->nbNodes() << " deformation nodes per Frame.\n";
   
//         delete eval;
// 		return result;


   const card32 numFrames = (card32)(deformation->getDuration()+1);

   // inserted by pj
   bool useDeformationInterpolation = ( oldUrshape != NULL );

	AttachedIndexedOctree* oldUrshapeOctree = NULL;
	UnstructuredInCorePointCloud* oldUrshapePC = NULL;
	PointSet* oldUrshapePS = NULL;
	AAT OLD_URSHAPE_POSITION;
	AAT OLD_URSHAPE_NORMAL;
	AAT OLD_URSHAPE_TANGENT_U;
	AAT OLD_URSHAPE_TANGENT_V;
	PointSetKNNQuery* oldUrshapeKNN = NULL;
	if ( useDeformationInterpolation ){
		oldUrshapePC = dynamic_cast<UnstructuredInCorePointCloud*>( oldUrshape );
		oldUrshapePS = oldUrshapePC->getPointSet();
		OLD_URSHAPE_POSITION = oldUrshapePS->getAAT( "position" );
		OLD_URSHAPE_NORMAL = oldUrshapePS->getAAT( "normal" );
		OLD_URSHAPE_TANGENT_U = oldUrshapePS->getAAT( "tangent_u" );
		OLD_URSHAPE_TANGENT_V = oldUrshapePS->getAAT( "tangent_v" );
		oldUrshapeOctree = PointSetKNNQuery::createOctree( oldUrshapePS );
		oldUrshapeKNN = new PointSetKNNQuery( oldUrshapePS, oldUrshapeOctree, makeVector3f( 0,0,0 ), OLD_URSHAPE_POSITION );
	}
	
   
   SceneObjectIterator *it = newUrshape->createIterator(SOIT::CAP_BASIC_PC);
   BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
   if (!bit) {
      delete it;
      delete eval;
      delete result;
      throw X4Exception("ARChangeUrshape::recomputeDeformation() - new urshape does not provide the necessary basic iterator.");
   }
   AAT newPositionAAT = newUrshape->getDescr()->getToken("position", 3, VAD::DATA_FORMAT_FLOAT32);

   solver->beginConstraints();

   for (card32 frame=0; frame<numFrames; frame++) {

	   if (numFrames>50)
	   {
		   std::cout << " building constraints for frame " << frame << endl;
	   }

      eval->beginQuery(frame);
      card32 index = 0;
      srand(42);
      while (!bit->atEnd()) {
	      if (settings->icpSkipPercentage != 0.0f) {
	         if (rnd01()<settings->icpSkipPercentage) {
	            bit->next();
					index++;
	            continue;
	         }
	      }
         Vector3f newPos = bit->get3f( newPositionAAT );
         Vector3f newPosDeformed = makeVector3f( 0,0,0 );	// dummy; set in the next block

          // inserted by pj
         if ( !useDeformationInterpolation ) {
            newPosDeformed = eval->getPosition(ARConstraintAnchor(newPos));
         } else {
            oldUrshapeKNN->setTargetPoint( newPos );
            card32 nIndex;
            bool found;
            oldUrshapeKNN->getNextNearestPointIndex( nIndex, found );
            Vector3f np = oldUrshapePS->get3f( nIndex, OLD_URSHAPE_POSITION );
            float32 summedWeight = 0;
            newPosDeformed = makeVector3f( 0,0,0 );
            const float32 sqrInfluence = sqr( settings->topSurfelSpacing );
            for ( unsigned int n = 0; n < 5; n++ ){
	            ARConstraintAnchor at;
	            at.pos = np;
	            at.topologicalAnchor = nIndex;
	            const float32 sqrD = ( np - newPos ) * ( np - newPos );
	            const float32 weight = exp( - sqrD / sqrInfluence );
	            summedWeight += weight;
	            const Matrix3f jac = eval->getSpatialGradient( at );
	            const Matrix3f transform = invertMatrix( jac ).transpose();
	            Matrix3f T_urshape, T_frame;
	            T_urshape[0] = oldUrshapePS->get3f( nIndex, OLD_URSHAPE_TANGENT_U );
	            T_urshape[1] = oldUrshapePS->get3f( nIndex, OLD_URSHAPE_TANGENT_V );
	            T_urshape[2] = oldUrshapePS->get3f( nIndex, OLD_URSHAPE_NORMAL );
	            T_frame[0] = jac * oldUrshapePS->get3f( nIndex, OLD_URSHAPE_TANGENT_U );
	            T_frame[1] = jac * oldUrshapePS->get3f( nIndex, OLD_URSHAPE_TANGENT_V );
	            T_frame[2] = transform * oldUrshapePS->get3f( nIndex, OLD_URSHAPE_NORMAL );
	            Vector3f offset = newPos - np;
	            offset = T_frame * T_urshape.transpose() * offset;
	            newPosDeformed += ( eval->getPosition( at ) + offset ) * weight;
	            oldUrshapeKNN->getNextNearestPointIndex( nIndex, found );
	            np = oldUrshapePS->get3f( nIndex, OLD_URSHAPE_POSITION );
            }
            newPosDeformed /= summedWeight;
         }

         PositionARConstraint constr;
         constr.anchor = ARConstraintAnchor(newPos, index);
         constr.toPoint = newPosDeformed;
         constr.time = frame;
         constr.covariance = IDENTITY3F;
         solver->addPositionConstraint(&constr);
         index++;
         bit->next();
      }
      eval->endQuery();
      bit->reset();
	  //debugOutput << "Constraints-Memory usage for frame " << frame << ":\n";
	  //((MMDeformationSolver*)solver)->getContext()->getConstraints(frame)->printMemUsage();
	 // debugOutput.flush();
   }
   solver->solve();
   

   delete it;
   delete eval;
   // inserted by pj
   if ( useDeformationInterpolation ){
 	   delete oldUrshapeOctree;
 	   delete oldUrshapeKNN;
   }

   return result;
}
