#ifndef ARComputeGlobalResidualH
#define ARComputeGlobalResidualH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{
class PointCloud;
class ARDeformationOptimizationContext;
class DeformationEvaluator;
class ARSettings;
/**
	computes residual for every frame - iterates over all
	input point clouds:
	 - computes deformed urshape
	 - estimates signed distance from urshape to pc[x] 
	 - computes weighted average squared distance of all frames
   residuals are stored in urshape, channel "residual" (2f)
	first float represents summed residuals, second float represents 
	summed weights
*/
class X3DEFORMATION_API ARComputeGlobalResidual 
{
public:
	static void computeGlobalResidual(
		PointCloud * urshape,			// reference shape
		ARDeformationOptimizationContext *deformation, 
		DeformationEvaluator *evaluator,
		std::vector<PointCloud*> data,		// pointcloud sequence (input data).
												// assumes data[0] represents time=0
		ARSettings * settings
		);

	static PointCloud * adaptRigidityToResiduals(
		PointCloud * urshape,				// reference shape, has to contain a channel "residual"
		ARSettings * settings,
		float32 medianFilterRadius
		);

};

}

#endif