//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARPruneUnsupportedOutliers.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "ARSettings.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



      
ARPruneUnsupportedOutliers::ARPruneUnsupportedOutliers(const ARSettings *settings, card32 numFrames) 
{
   this->settings = settings;
   this->numFrames = numFrames;
}

void ARPruneUnsupportedOutliers::addNextFrame(PointCloud *pc) 
{
   UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
   if (!upc) {
      notImplemented();
   }
   PointSet *ps = upc->getPointSet();
   AAT numDataPtsAAT = ps->getAAT("numDataPts");
   if (numSupportedFrames.empty()) {
      numSupportedFrames.resize(ps->getNumEntries(), 0);
   }
   for (card32 i=0; i<ps->getNumEntries(); i++) {
      if (ps->get1i(i, numDataPtsAAT) > 0) {
         numSupportedFrames[i]++;
      }
   }
}

void ARPruneUnsupportedOutliers::getResult(std::vector<bool> &result) 
{
   card32 numFramesNecessary = settings->cullBelowNumFrames;
   numFramesNecessary = max(numFramesNecessary, ((card32)(settings->cullBelowPercentage*numFrames)));
   if (settings->cullStartNumFrames > numFrames) numFramesNecessary = 0;
   
   result.resize(numSupportedFrames.size());
   for (unsigned i=0; i<result.size(); i++) {
      result[i] = numSupportedFrames[i] >= numFramesNecessary;
   }
}

void ARPruneUnsupportedOutliers::resetCounters(PointCloud *pc) {
   UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
   if (!upc) {
      notImplemented();
   }
   PointSet *ps = upc->getPointSet();
   AAT numDataPtsAAT = ps->getAAT("numDataPts");
   for (card32 i=0; i<ps->getNumEntries(); i++) {
      ps->set1i(i, numDataPtsAAT, 0);
   }   
}
