//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARSetUrshapeDataWeight.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointSetKNNQuery.h"
#include "ARSequence.h"
#include "DeformationEvaluator.h"
#include "ARSettings.h"
#include "AREvaluateDeformation.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------






void ARSetUrshapeDataWeight::accumDataWeight(
   ARSequence *seq, 
   const std::vector<PointCloud*> &data,
   DeformationEvaluator *eval,
   ARSettings *settings) 
{
   checkAttribute(seq->getUrshape(), "dataWeight", 1, VAD::DATA_FORMAT_FLOAT32);
   UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(seq->getUrshape());
   if (!upc) {
      warning("not implemented");
      return;
   }
   PointSet *refPS = upc->getPointSet();
   AAT dataWeightAAT = seq->getUrshape()->getAAT("dataWeight");
   for (card32 i=0; i<refPS->getNumEntries(); i++) {
      refPS->set1f(i, dataWeightAAT, 0);
   }
   const card32 numFrames = (card32)(seq->getDuration()+1);
   for (card32 i=0; i<numFrames; i++) {
      UnstructuredInCorePointCloud *deformedReference = AREvaluateDeformation::evaluatePC(
         seq->getUrshape(), 
         seq->getDeformation(), 
         eval,
         i,
         settings);
	   
      PointSet *ps = deformedReference->getPointSet();
	   AttachedIndexedOctree *octree = PointSetKNNQuery::createOctree(ps, 20);
      AAT urshapePosAAT = ps->getAAT("position");
      PointSetKNNQuery knn(ps, octree, NULL_VECTOR3F, urshapePosAAT);
      SceneObjectIterator *it = data[i]->createIterator(SOIT::CAP_BASIC_PC);
      BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
      AAT dataPosAAT = data[i]->getAAT("position");
      while (!bit->atEnd()) {
         Vector3f dPos = bit->get3f(dataPosAAT);
         knn.setTargetPoint(dPos);
         bool found;
         card32 nIndex;
         knn.getNextNearestPointIndex(nIndex, found);
         if (found) {
            Vector3f uPos = ps->get3f(nIndex, urshapePosAAT);
            float32 sqrDist = normQuad(uPos - dPos);
            float32 weight = exp(-sqrDist/sqr(settings->topSurfelSpacing*settings->topRelRadius/2));
            refPS->set1f(nIndex, dataWeightAAT, weight + ps->get1f(nIndex, dataWeightAAT));
         }
         bit->next();
      }
      upc->getAttachments()->deleteDelOnWriteData();
      delete it;

	   delete octree;
	   delete deformedReference;
   }
}
