//---------------------------------------------------------------------------
#ifndef ARTopologicalResamplingH
#define ARTopologicalResamplingH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4 {

	class PointCloud;
	class UnstructuredInCorePointCloud;
	class ARSettings;

	///
	///   removes points until sampling condition is met
	///   takes into account topology
	///
	class X3DEFORMATION_API ARTopologicalResampling {
	public:

		static UnstructuredInCorePointCloud *resample(
			/// input to be resampled (topology assumed to be attached)
			UnstructuredInCorePointCloud* pc, 
			/// new sample spacing
			float32 maxDist,
			/// relative radius (x maxDist) for new topology graph
			float32 reconnectTopologyRelativeRadius,
			/// maximum number of nearest neighbors for new topology graph
			card32 reconnectTopologyMaxNumNeighbors,
			// possible output of delete vertices indexes
			std::vector<bool>* deletePtn_User = NULL
			);

		static void resampleInPlace(PointCloud *pc, ARSettings *settings);






		/*
		this is a copy and a refactorized version of
		ARTopologicalResampling::resample. The function is split in two parts, one for computing the toDelete vertices
		and one for resampling the mesh and its topology
		*/
		static void resampleMeshFindToDeleteVertices(
			UnstructuredInCorePointCloud* pc, 
			float32 maxDist, 
			//	   float32 reconnectTopologyRelativeRadius,
			//	   card32  reconnectTopologyMaxNumNeighbors,
			std::vector<bool>& toDelete );
		static UnstructuredInCorePointCloud* resampleMeshAndTopology(
			UnstructuredInCorePointCloud* pc, 
			float32 maxDist, 
			float32 reconnectTopologyRelativeRadius,
			card32  reconnectTopologyMaxNumNeighbors,
			/// if bridgeEdges is set, all erased edges are replaced by a bridging edge - thus preserving the topological connections
			bool reconnectTopologyInsertBridgeEdges,
			const std::vector<bool>& toDelete);
	};

}

#endif