//---------------------------------------------------------------------------
#ifndef ARDataConstraintsSettingsH
#define ARDataConstraintsSettingsH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "ARDefaultSettings.h"
//---------------------------------------------------------------------------

namespace X4 {

///
///   This class controls which data constraints are used in the merging process
///
class X3DEFORMATION_API ARDataConstraintsSettings : public Persistent {
	X4_CLASS( ARDataConstraintsSettings )
 private:
   // --- geometry matching
	bool useGeometry;
	//std::vector<std::string> geometryDataPaths; - use path in ARMergeSettings

   // --- feature matching
	bool useFeatures;
	std::vector<std::string> featureCorrPaths;

   // --- photometric matching
	bool usePhotoConsistency;
	bool useOpticalFlow;
	std::vector<std::string> imageDataPaths;

   // --- silhouette constraints
	bool useSilhouettes;
	std::vector<std::string> depthImagesPaths;

	//IMPLEMENT_VAR_ARRAY_METHODS_STL(geometryDataPaths)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(featureCorrPaths)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(imageDataPaths)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(depthImagesPaths)

 public:
	ARDataConstraintsSettings();

	void setPresets(ARDefaultSettings::DataType presets);

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   ~ARDataConstraintsSettings();
};

}

#endif