//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARMergeSettings.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ARPipelineOptions.h"
#include "ARSettings.h"
#include "ARDataConstraintsSettings.h"
#include "SeparatorClassProperty.h"
#include "RigidDeformationSolver.h"
#include "MMDeformationSolver.h"
#include "SurfelOptimizer.h"
#include "SurfelGraphInit2.h"
#include "SGListNode.h"
#include "ARVisualizationSettings.h"
#include "ARSequenceEvaluator.h"
#include "ARPointCloudSeqEval.h"
#include "ARGlobalOptSettings.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( ARMergeSettings , 5 ) {
	BEGIN_CLASS_INIT( ARMergeSettings );
	ADD_SEPARATOR("input & output")
	ADD_STRING_PROP(constraintsDataPath, 0)
	ADD_STRING_PROP(urshapesDataPath, 3)
	ADD_STRING_PROP(globalOptDataPath, 2)
	ADD_STRING_PROP(urshapesPath, 0)
	ADD_STRING_PROP(deformationsPath, 0)
	ADD_STRING_PROP(resultsPath, 0)
	ADD_SEPARATOR("configuration")
	ADD_OBJECT_PROP(pipelineOptions, 0, ARPipelineOptions::getClass(), true)
	ADD_OBJECT_PROP(optimizationSettings, 0, ARSettings::getClass(), true)
	ADD_OBJECT_PROP(dataConstraintsSettings, 0, ARDataConstraintsSettings::getClass(), true)
	SET_USER_EDIT(false)
	ADD_OBJECT_PROP(globalOptSettings, 5, ARGlobalOptSettings::getClass(), true)
	ADD_OBJECT_PROP(visualizationSettings, 0, ARVisualizationSettings::getClass(), true)

	ADD_OBJECT_PROP( backendSettings, 4, BackendSettings::getClass(), true )
	ADD_SEPARATOR("solvers")
	ADD_OBJECT_PROP(deformationSolver, 0, DeformationSolver::getClass(), true)
	ADD_OBJECT_PROP(shapeSolver, 0, ShapeReconstruction::getClass(), true)
	ADD_OBJECT_PROP(initUrshapes, 0 , SurfelGraphInit2::getClass(), true)
	ADD_OBJECT_PROP(finalOutput, 1, ARSequenceEvaluator::getClass(), true)
}

// ----

ARMergeSettings::ARMergeSettings() {
	constraintsDataPath = "root/data";
	urshapesDataPath = "root/data";
	globalOptDataPath = "root/data";
	urshapesPath = "root/urshapes";
	deformationsPath = "root/deformations";
	resultsPath = "root/results";

	pipelineOptions = new ARPipelineOptions();
	optimizationSettings = new ARSettings();
	dataConstraintsSettings = new ARDataConstraintsSettings();
	globalOptSettings = new ARGlobalOptSettings();
	visualizationSettings = new ARVisualizationSettings();
	backendSettings = new BackendSettings();

	//deformationSolver = new RigidDeformationSolver();
	deformationSolver = new MMDeformationSolver();
	shapeSolver = new SurfelOptimizer();
	initUrshapes = new SurfelGraphInit2();
	finalOutput = new ARPointCloudSeqEval();
}

void ARMergeSettings::setPresets(ARDefaultSettings::DataType presets) {
	pipelineOptions->setPresets(presets);
	optimizationSettings->setPresets(presets);
	dataConstraintsSettings->setPresets(presets);
}

void ARMergeSettings::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

ARMergeSettings::~ARMergeSettings() {
	delete pipelineOptions;
	delete optimizationSettings;
	delete dataConstraintsSettings;
	delete globalOptSettings;
	delete backendSettings;

	delete deformationSolver;
	delete shapeSolver;
	delete initUrshapes;
	delete finalOutput;
}



void ARMergeSettings::getPathIntern(std::string whichPath, Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node) {
   SGListNode *list = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(scene, whichPath));
   if (!list) {
      throw X4Exception("ARMergeSettings::getPathIntern() - path not found.");
   }
   if (frame >= list->getNumChildNodes(NULL)) {
      throw ERangeCheck("ARMergeSettings::getPathIntern() - frame out of range.");
   }
   node = list->getChildNode(NULL, frame);
   framePath = whichPath + ((whichPath[whichPath.size()-1] != '/')?std::string("/"):string("")) + node->getName();
}

void ARMergeSettings::getConstraintsDataPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node)
{
	getPathIntern(constraintsDataPath, scene, frame, framePath, node);
}

void ARMergeSettings::getUrshapesDataPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node)
{
	getPathIntern(urshapesDataPath, scene, frame, framePath, node);
}

void ARMergeSettings::getGlobalOptDataPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node)
{
	getPathIntern(globalOptDataPath, scene, frame, framePath, node);
}

void ARMergeSettings::getUrshapesPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node)
{
	getPathIntern(urshapesPath, scene, frame, framePath, node);
}

void ARMergeSettings::getDeformationsPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node)
{
	getPathIntern(deformationsPath, scene, frame, framePath, node);
}

void ARMergeSettings::getResultsPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node)
{
	getPathIntern(resultsPath, scene, frame, framePath, node);
}