//---------------------------------------------------------------------------
#ifndef ARMergeSettingsH
#define ARMergeSettingsH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "ARDefaultSettings.h"
#include "BackendSettings.h"
//---------------------------------------------------------------------------

namespace X4 {

class ARPipelineOptions;
class ARSettings;
class ARDataConstraintsSettings;
class DeformationSolver;
class ShapeReconstruction;
class SurfelGraphInit2;
class ARConstraintsSource;
class ARVisualizationSettings;
class ARSequenceEvaluator;
class ARGlobalOptSettings;

///
///   this class contains all parameters necessary to control merging of sequences
///
class X3DEFORMATION_API ARMergeSettings : public Persistent {
	X4_CLASS( ARMergeSettings )
 private:
	void getPathIntern(std::string whichPath, Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node);
 
 public:
	// -- input & output
	std::string constraintsDataPath;   // folder with data to be used in optimization
	std::string urshapesDataPath;      // folder with data to be used to create initial urshapes
	std::string globalOptDataPath;     // folder with data to be used in global optimization
	std::string urshapesPath;          // folder that stores the computed urshapes
	std::string deformationsPath;      // folder that stores the computed deformations
	std::string resultsPath;           // folder in which to put the results
	// -- configuration
	ARPipelineOptions         *pipelineOptions;
	ARSettings                *optimizationSettings;
	ARDataConstraintsSettings *dataConstraintsSettings;
	ARGlobalOptSettings       *globalOptSettings;
	ARVisualizationSettings   *visualizationSettings;
	BackendSettings     *backendSettings;
	// -- solvers
	DeformationSolver			  *deformationSolver;
	ShapeReconstruction		  *shapeSolver;
	SurfelGraphInit2  		  *initUrshapes;
	ARSequenceEvaluator       *finalOutput;

	// ----

	ARMergeSettings();

	void setPresets(ARDefaultSettings::DataType presets);
	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   void getConstraintsDataPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node);
   void getUrshapesDataPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node);
   void getGlobalOptDataPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node);
   void getUrshapesPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node);
   void getDeformationsPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node);
   void getResultsPath(Scene *scene, card32 frame, std::string &framePath, SceneGraphNode *&node);
	
   ~ARMergeSettings();
};

}

#endif