#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIVisionTests.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ParametricInCorePointCloud.h"
#include "StandardSceneGraphIterator.h"
#include "RGBAImage.h"
#include "CVisionTools.h"
#include "CameraParameterAttachment.h"
#include "ImageDisplayer.h"
#include "SGObjectNode.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
#include "PointSet.inline.h"
#include "LinearAlgebra.inline.h"
#include "ClassMethods.h"
#include "RegularVolume.h"
#include "GLVolumeDisplayer.h"
#include "PointSet.inline.h"
#include "ProgressWindow.h"
#include "RegularInCoreVolume.h"
#include "Random.h"
#include "VertexArray.h"
#include "DebugRenderer.h"
#include "PointCloudTools.h"
#include "PhotometricContraints.h"
#include "InCorePCTopologyGraph.h"
#include "SurfelGraphInit2.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCIVisionTests ,0)
{
	BEGIN_CLASS_INIT( PCIVisionTests );
	ADD_CARD32_PROP(m_VolumeGrid,0);
	ADD_CARD32_PROP(m_WindowSize,0);
	ADD_BOOLEAN_PROP(m_TransformPatches,0);
	ADD_FLOAT32_PROP(m_MinCorrelationThreshold,0);
	ADD_FLOAT32_PROP(m_MinEdgeThreshold,0);

	ADD_NOARGS_METHOD(computeDisparityVolume);
	ADD_NOARGS_METHOD(testImageCompare);
	ADD_NOARGS_METHOD(visualizeDisparity);
	ADD_NOARGS_METHOD(computeFrameConstraints);
	ADD_NOARGS_METHOD(computeConfidence);
	ADD_NOARGS_METHOD(testRemoveSmallComponents);}

PCIVisionTests::PCIVisionTests(void)
{
	m_VolumeGrid = 16;
	m_WindowSize = 5;
	m_TransformPatches = true;
	m_MinCorrelationThreshold = .5f;
	m_MinEdgeThreshold = 0.025f;
}

PCIVisionTests::~PCIVisionTests(void)
{
	dropCurrentFrame();
}

void PCIVisionTests::assign(const Object* obj, X4CopyContext *context)
{
/*	PCInteractionTool::assign(obj,context);
	const PCIVisionTests * o = dynamic_cast<const PCIVisionTests*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);*/
}

static Vector3f clickedTestPoint;

void X4::PCIVisionTests::mouseDown( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState )
{
	Vector3f point;
	PointCloud * pc;
	std::string pcName;
	if( getIntersectionFromCursor( x,y, point, pcName, &pc ) )
	{
		clickedTestPoint = point;
		visualizeDisparity();
		computeFrameConstraints();
	}

/*	StandardSceneGraphIterator sIt;
	sIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(),
		SGDynamicState::tmAllNodes);
		while( !sIt.atEnd())
		{
			SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sIt.getNode());
			if( objNode )
			{
				GenericSwapAllOutOfCorePointCloud * test = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(objNode->getSceneObject());
				if( test )
				{
					CameraParameterAttachment * camAtt = dynamic_cast<CameraParameterAttachment*>(
						test->getAttachments()->getData(CameraParameterAttachment::getAttachmentName()));

					ParametricInCorePointCloud * pc = dynamic_cast<ParametricInCorePointCloud*>(test->fetchPC());
					if( pc && camAtt)
					{
						clickedOnParametricCloud(pc, point, camAtt);
						return;
					}
				}
			}

			sIt.next();
		}
	}*/
}

void X4::PCIVisionTests::clickedOnParametricCloud( ParametricInCorePointCloud* pc, const Vector3f &pos,CameraParameterAttachment * camAtt )
{

	if( !camAtt )
		return;

	RGBAFloat32Image testImage;
	testImage.resize(100,100);
	
	Vector2f p = camAtt->projectWorldtoImage( pos );
	AAT aat = pc->getAAT("color");

	debugOutput << "original point:" << pos << "projected point: " << p << "\n";


	for( int x=0;x<(int)testImage.getSizeX();x++ )
	{
		for( int y=0;y<(int)testImage.getSizeY();y++ )
		{
			Vector2f tmp = p + makeVector2f((float)x - (float)testImage.getSizeX()/2.0f,(float)y - (float)testImage.getSizeY()/2.0f) * 0.5f;
			Vector3f col = pc->getBilinearInterpolatedAttribute3f(tmp[0],tmp[1],aat);
			testImage.setPixel(x,y,expand3To4(col));
		}
	}

	static int imageNumber = 0;
	imageNumber = (imageNumber+1)%8;

	ImageDisplayer::DisplayRGBAFloat32Image ev(&testImage);
	char c = '0';// + imageNumber;
	debugOutput.displayEvent(ImageDisplayer::getClass(), std::string("test") + c, &ev);


}

void X4::PCIVisionTests::computeDisparityVolume()
{
	StandardSceneGraphIterator sIt;
	sIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(),
		SGDynamicState::tmRendering);

	std::vector<ParametricInCorePointCloud*> clouds;
	std::vector<CameraParameterAttachment*> attachments;
	BoundingBox3f sceneBB;
	
	while( !sIt.atEnd())
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sIt.getNode());
		if( objNode )
		{
			GenericSwapAllOutOfCorePointCloud * test = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(objNode->getSceneObject());
			if( test )
			{
				CameraParameterAttachment * camAtt = dynamic_cast<CameraParameterAttachment*>(
					test->getAttachments()->getData(CameraParameterAttachment::getAttachmentName()));

				UnstructuredInCorePointCloud *upc =dynamic_cast<UnstructuredInCorePointCloud*>(test->fetchPC());
				if( clouds.size() == 1 && upc)
					sceneBB = getPCBBox(upc);

				ParametricInCorePointCloud * pc = dynamic_cast<ParametricInCorePointCloud*>(test->fetchPC());
				if( pc && camAtt)
				{
					clouds.push_back((ParametricInCorePointCloud*)pc->copy());
					attachments.push_back(camAtt);
				}
			}
		}

		sIt.next();
	}

	RegularInCoreVolume *testVol = new RegularInCoreVolume();
	const card32 SIZE = m_VolumeGrid;
	VertexDescriptor vd;
	vd.pushAttrib(VAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(VAD("intensity", 1, VAD::DATA_FORMAT_FLOAT32));
	testVol->clearAndSetup(SIZE, SIZE, SIZE, &vd, sceneBB  );

	RegularVolume * voxels = testVol->getVoxels();
	PointSet *ps = voxels->getVoxelData();
	AAT POS = ps->getAAT("position");
	AAT INTEN = ps->getAAT("intensity");
	StaticVector<float,1> v;
	v[0] = 0.0f;

	for (card32 z=0; z<SIZE; z++) {
		for (card32 y=0; y<SIZE; y++) {
			for (card32 x=0; x<SIZE; x++) {
				ps->set<float32, 1>(x + SIZE*y + SIZE*SIZE*z, INTEN, v);
				ps->set<float32, 3>(x + SIZE*y + SIZE*SIZE*z, POS,makeVector3f(x,y,z).componentProduct(sceneBB.upperCorner - sceneBB.lowerCorner)/SIZE + sceneBB.lowerCorner);
			}
		}
	}

	progressWindow->pushStep();
	for( unsigned i=0;i<clouds.size()-1;i++ )
	{
		progressWindow->progress(i*100.0f/max<float>(1,clouds.size()-1));
		for( unsigned j=i+1;j<clouds.size() && j<i+3;j++ )
		{
			computeDisparityVolume(clouds[i],attachments[i], clouds[j], attachments[j],voxels, sceneBB );
		}
	}
	progressWindow->popSteps();

	/*GLVolumeDisplayer *voldisp = new GLVolumeDisplayer(testVol);
	GLEventDisplayer::SetNewGLEventObject ev2(voldisp);
	debugOutput.displayEvent(GLEventDisplayer::getClass(), "glVolDispTest", &ev2);	*/

	testVol->setMaterialIndex(5);
	addPointCloud( getScene(), testVol, "volume" );

	for( unsigned i=0;i<clouds.size();i++ )
		delete clouds[i];

}

void X4::PCIVisionTests::computeDisparityVolume( ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1,
																  ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2, RegularVolume * vol,
																  BoundingBox3f &bb)
{

	int windowSize = m_WindowSize;
	PointSet *ps = vol->getVoxelData();
	AAT INTEN = ps->getAAT("intensity");
	const card32 SIZE = vol->getExtends()[0];

	for (card32 z=0; z<SIZE; z++) 
	{
		for (card32 y=0; y<SIZE; y++) {
			for (card32 x=0; x<SIZE; x++) {
				StaticVector<float32, 1> v = ps->get<float32,1>(x + SIZE*y + SIZE*SIZE*z,INTEN);

				Vector3f scenePoint = makeVector3f(x,y,z).componentProduct( bb.upperCorner - bb.lowerCorner )/SIZE + bb.lowerCorner;

				float dist = computeDisparity(scenePoint, p1, camAtt1, p2, camAtt2 );

				/*Vector2f projPoint1 = camAtt1->projectWorldtoImage( scenePoint );
				Vector2f projPoint2 = camAtt2->projectWorldtoImage( scenePoint );
				Vector2i point1i = makeVector2i(projPoint1[0],projPoint1[1]);
				
				
				float dist = compareImagePatches(p1, point1i, windowSize,p2,projPoint2 + makeVector2f(-windowSize, -windowSize),
					projPoint2 + makeVector2f( windowSize, -windowSize),
					projPoint2 + makeVector2f( windowSize,  windowSize),
					projPoint2 + makeVector2f(-windowSize,  windowSize), CI_NCC );*/

				if( dist > m_MinCorrelationThreshold)
					v[0] += (dist);
				ps->set<float32, 1>(x + SIZE*y + SIZE*SIZE*z, INTEN, v);

			}
		}
	}


	
}

void X4::PCIVisionTests::testImageCompare()
{
	StandardSceneGraphIterator sIt;
	sIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(),
		SGDynamicState::tmRendering);

	std::vector<ParametricInCorePointCloud*> clouds;
	std::vector<CameraParameterAttachment*> attachments;
	UnstructuredInCorePointCloud * upc = NULL;
	BoundingBox3f sceneBB;

	while( !sIt.atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sIt.getNode());
		if( objNode )
		{
			GenericSwapAllOutOfCorePointCloud * test = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(objNode->getSceneObject());
			if( test )
			{
				CameraParameterAttachment * camAtt = dynamic_cast<CameraParameterAttachment*>(
					test->getAttachments()->getData(CameraParameterAttachment::getAttachmentName()));

				UnstructuredInCorePointCloud *upc_tmp =dynamic_cast<UnstructuredInCorePointCloud*>(test->fetchPC());
				if( !upc && upc_tmp)
					upc = (UnstructuredInCorePointCloud*)upc_tmp->copy();

				ParametricInCorePointCloud * pc = dynamic_cast<ParametricInCorePointCloud*>(test->fetchPC());
				if( pc && camAtt)
				{
					clouds.push_back((ParametricInCorePointCloud*)pc->copy());
					attachments.push_back(camAtt);
				}
			}
		}

		sIt.next();
	}

	//select random point
	VertexArray array(upc->getPointSet());
	Vector3f rndPoint = array.getPosition3f( random32bit(upc->getNumPoints()));

	debugRenderer->clearDebugData("test-image-compare");
	debugRenderer->beginRenderJob("test-image-compare");
	debugRenderer->addSphere(rndPoint, getMedianPointDistance(upc) * 7.0f, makeVector3f(1,0,0));
	debugRenderer->endRenderJob();

	for( unsigned i=0;i<clouds.size()-1;i++ )
	{
		CameraParameterAttachment * camAtt1 = attachments[i];
		CameraParameterAttachment * camAtt2 = attachments[i+1];
		int windowSize = m_WindowSize;
		Vector2f projPoint1 = camAtt1->projectWorldtoImage( rndPoint );
		Vector2f projPoint2 = camAtt2->projectWorldtoImage( rndPoint );
		Vector2i point1i = makeVector2i(projPoint1[0],projPoint1[1]);

		debugOutput << "projected point1:" << projPoint1 << 
			"projected point2:" << projPoint2 << "\n";


		float dist = compareImagePatches(clouds[i], point1i, windowSize,clouds[i+1],projPoint2 + makeVector2f(-windowSize, -windowSize),
			projPoint2 + makeVector2f( windowSize, -windowSize),
			projPoint2 + makeVector2f( windowSize,  windowSize),
			projPoint2 + makeVector2f(-windowSize,  windowSize), CI_NCC );

		debugOutput << "dist[" << i << "," << i+1 << "] = " << dist << "\n";

	}


	for( unsigned i=0;i<clouds.size();i++ )
		delete clouds[i];
	delete upc;

}

void X4::PCIVisionTests::fetchCurrentFrame()
{
	dropCurrentFrame();
	StandardSceneGraphIterator sIt;
	sIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(),
		SGDynamicState::tmRendering);

	while( !sIt.atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sIt.getNode());
		if( objNode )
		{
			GenericSwapAllOutOfCorePointCloud * test = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(objNode->getSceneObject());
			if( test )
			{
				CameraParameterAttachment * camAtt = dynamic_cast<CameraParameterAttachment*>(
					test->getAttachments()->getData(CameraParameterAttachment::getAttachmentName()));

				ParametricInCorePointCloud * pc = dynamic_cast<ParametricInCorePointCloud*>(test->fetchPC());
				if( pc && camAtt)
				{
					m_Clouds.push_back((ParametricInCorePointCloud*)pc->copy());
					m_CamAtt.push_back(camAtt);
				}
			}
		}

		sIt.next();
	}
}

void X4::PCIVisionTests::dropCurrentFrame()
{
	for( unsigned i=0;i<m_Clouds.size();i++ )
	{
		delete m_Clouds[i];
	}
	m_Clouds.clear();
	m_CamAtt.clear();
}

PointCloud * X4::PCIVisionTests::getSelectedCloud()
{
	std::vector<std::string> sel = getScene()->getRootState()->staticState->getSelectedNodes();
	for( unsigned i=0;i<sel.size();i++ )
	{
		PointCloud * ret = getPointCloud(getScene(),sel[i]);
		if( ret )
			return ret;
	}
	return NULL;
}

void X4::PCIVisionTests::visualizeDisparity()
{
	GenericSwapAllOutOfCorePointCloud * gpc = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(getSelectedCloud());
	if( !gpc )
		return;

	UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(gpc->fetchPC());
	if( !upc )
		return;
	upc = (UnstructuredInCorePointCloud*)upc->copy();

	PhotometricContraints pconstraint;
	pconstraint.setMinCorrelationThreshold(m_MinCorrelationThreshold);
	pconstraint.setWindowSize(m_WindowSize);
	pconstraint.setMinEdgeThreshold(m_MinEdgeThreshold);

	UnstructuredInCorePointCloud * return_pc = new UnstructuredInCorePointCloud;
	VertexDescriptor * vd = new VertexDescriptor;
	vd->pushAttrib(VAD("position",3,VAD::DATA_FORMAT_FLOAT32));
	vd->pushAttrib(VAD("color",3,VAD::DATA_FORMAT_FLOAT32));
	vd->pushAttrib(VAD("flags",1,VAD::DATA_FORMAT_FLOAT32));
	return_pc->clearAndSetup( vd, 0);
	PointSet * ps = return_pc->getPointSet();
	AAT posAAT = ps->getAAT("position");
	AAT colorAAT = ps->getAAT("color");
	std::vector<char> buffer(vd->getSize(),0);
	
	fetchCurrentFrame();

	bool testOnePointOnly = true;

	VertexArray array(upc->getPointSet());
	for( unsigned i=0;i<array.getNumElements();i++ )
	{
		Vector3f pos = array.getPosition3f(i);
		if( !testOnePointOnly )
		{
			if( rand() % 10 != 0)
				continue;
		}
		else
		{
			pos = clickedTestPoint;
		}



		int windowSize = m_WindowSize;
		unsigned pc1Index = 4;

		Vector3f b = NULL_VECTOR3F;
		Matrix3f m = IDENTITY3F * 0;

		for( unsigned j=0;j<m_Clouds.size();j++ )
		{
			Vector3f retpos;
			Matrix3f retmat;
			if( j == pc1Index )
				continue;

			float disp = pconstraint.computeDisparityConstraint( pos, m_Clouds[pc1Index], m_CamAtt[pc1Index],
				m_Clouds[j],m_CamAtt[j], retpos, retmat);

			if( disp > m_MinCorrelationThreshold )
			{
				Vector3f tmpB = retmat * retpos * (-2.0f);
				b += tmpB;
				m += retmat;
				debugOutput << "\nretpos: " << retpos;
			}
		}

		debugOutput << m << "b = " << b << "\n";

		if( norm(b) > 0.0f )
		{
			Vector3f center;
			linsolve3f(m*2.0f,b,center);
			debugOutput << "\ncenter: " << center << "\n";
			//debugRenderer->clearDebugData("constraint-test");
			debugRenderer->beginRenderJob("constraint-test");
				debugRenderer->addPoint(center, makeVector3f(1,0,0));
				debugRenderer->addPositionConstraint(pos,center,m);
			debugRenderer->endRenderJob();
		}


		if( testOnePointOnly )
			break;
	}

	if( !testOnePointOnly)
		addPointCloud(getScene(),return_pc, "test");
	else
		delete return_pc;

	dropCurrentFrame();
	delete upc;
}

float X4::PCIVisionTests::computeDisparity( const Vector3f &point, ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1, ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2 )
{
	Vector2f projPoint1 = camAtt1->projectWorldtoImage( point );
	Vector2f projPoint2 = camAtt2->projectWorldtoImage( point );
	Vector2i point1i = makeVector2i(projPoint1[0],projPoint1[1]);
	const int windowSize = m_WindowSize;

	// compute projection
	Vector2f quad1,quad2,quad3,quad4;
	if( !m_TransformPatches )
	{
		quad1 = projPoint2 + makeVector2f(-windowSize, -windowSize);
		quad2 = projPoint2 + makeVector2f( windowSize, -windowSize);
		quad3 = projPoint2 + makeVector2f( windowSize,  windowSize);
		quad4 = projPoint2 + makeVector2f(-windowSize,  windowSize);
	}
	else
	{

		float depth = point[2];
		quad1 = camAtt2->projectWorldtoImage(
			camAtt1->projectImageToWorld( makeVector3f(point1i[0] - windowSize,point1i[1] - windowSize, depth)));
		quad2 = camAtt2->projectWorldtoImage(
			camAtt1->projectImageToWorld( makeVector3f(point1i[0] + windowSize,point1i[1] - windowSize, depth)));
		quad3 = camAtt2->projectWorldtoImage(
			camAtt1->projectImageToWorld( makeVector3f(point1i[0] + windowSize,point1i[1] + windowSize, depth)));
		quad4 = camAtt2->projectWorldtoImage(
			camAtt1->projectImageToWorld( makeVector3f(point1i[0] - windowSize,point1i[1] + windowSize, depth)));


	}

	if( projPoint1[0] < 2.0f * m_WindowSize  + 1.0f ||
		projPoint1[1] < 2.0f * m_WindowSize  + 1.0f ||
		projPoint1[0] > p1->getPointSet()->getWidth()- 2.0f * m_WindowSize  + 1.0f ||
		projPoint1[1] > p2->getPointSet()->getHeight()- 2.0f * m_WindowSize  + 1.0f
		)
		return 0.0f;
	if( projPoint2[0] < 2.0f * m_WindowSize  + 1.0f ||
		projPoint2[1] < 2.0f * m_WindowSize  + 1.0f ||
		projPoint2[0] > p1->getPointSet()->getWidth()- 2.0f * m_WindowSize  + 1.0f ||
		projPoint2[1] > p2->getPointSet()->getHeight()- 2.0f * m_WindowSize  + 1.0f
		)
		return 0.0f;

	float dist = compareImagePatches(p1, point1i, windowSize,p1,quad1,
		quad2,quad3,quad4, CI_NCC );


	return dist;
}

void X4::PCIVisionTests::computeDisparityConstraint( const Vector3f &point, ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1, 
																		ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2, Vector3f &constraint_pos, Matrix3f &contraint_matrix )
{
	// compute epipolarline
	Vector2f projPoint1 = camAtt1->projectWorldtoImage(point);
	Vector3f cam_pos0 = camAtt1->projectImageToWorld( makeVector3f(projPoint1[0],projPoint1[1],0));

	Vector2f projPoint2 = camAtt2->projectWorldtoImage(point);
	Vector2f epipol = camAtt2->projectWorldtoImage((cam_pos0 + point) * 0.5f);
	Vector3f cam_pos1 = camAtt2->projectImageToWorld( makeVector3f(projPoint2[0],projPoint2[1],0));

	unsigned numSamples = 10;
	float pixelRange = 5.0f;
	Vector2f normedEpipolDirection = normalize( projPoint2 - epipol );
	std::vector<Vector3f> result_vec;
	std::vector<float> result_corr;
	size_t maxCorrIndex = 0;
	constraint_pos = point;
	for( unsigned i=0;i<numSamples;i++ )
	{
		float t = ((float)i/(numSamples-1.0f) - 0.5f)*2.0f*pixelRange;
		Vector2f testPoint = projPoint2 + normedEpipolDirection * t;
		//debugOutput << "testpoint: " << testPoint << "\n";

		Vector3f reprojectedTestPoint = camAtt2->projectImageToWorld( makeVector3f(testPoint[0],testPoint[1], point[2]));

		// compute "intersection" between cam1-ray and cam2-ray
		// defines a plane parallel to cam2-ray and computes the intersection between cam1-ray and the plane
		Vector3f tmpVec = (cam_pos0 - point).crossProduct(reprojectedTestPoint - cam_pos1);
		Plane3f p( normalize( tmpVec.crossProduct(reprojectedTestPoint - cam_pos1)), reprojectedTestPoint);
		Line3f l(cam_pos0 - point,point);

		Vector3f finalTestPoint = p.calculateIntersection( l );

		float correlation = computeDisparity( finalTestPoint, p1, camAtt1, p2, camAtt2 );
		result_vec.push_back(finalTestPoint);
		result_corr.push_back(correlation);

		if( result_corr.back() > result_corr[maxCorrIndex] )
		{
			if( correlation > m_MinCorrelationThreshold)
				constraint_pos = finalTestPoint;
			maxCorrIndex = result_corr.size()-1;
		}

	//	debugOutput << "correlation: " << correlation << "\n";
	}

	/*debugRenderer->beginRenderJob("testCorr");
	for( unsigned i=0;i<result_vec.size()-1;i++)
	{
		Vector3f col1,col2;
		if( result_corr[i] < m_MinCorrelationThreshold )
			col1 = makeVector3f(255,0,0);
		else
			if( i == maxCorrIndex )
				col1 = makeVector3f(0,255,0);
			else
			{
				float t = (result_corr[i]-m_MinCorrelationThreshold) / (1.0f - m_MinCorrelationThreshold);
				col1 = makeVector3f(t,t,1.0f-t);
			}
		if( result_corr[i+1] < m_MinCorrelationThreshold )
			col2 = makeVector3f(255,0,0);
		else
			if( i+1 == maxCorrIndex )
				col2 = makeVector3f(0,255,0);
			else
			{
				float t = (result_corr[i+1]-m_MinCorrelationThreshold) / (1.0f - m_MinCorrelationThreshold);
				col2 = makeVector3f(t,t,1.0f-t);
			}
			

		debugRenderer->addLine(result_vec[i],result_vec[i+1],col1,col2,2 );
	}
	debugRenderer->endRenderJob();*/

}

void X4::PCIVisionTests::computeConfidence()
{
	fetchCurrentFrame();
	PointCloud * ref_pc = getSelectedCloud();
	if( !ref_pc || !ref_pc->providesAttribute("position"))
		return;

	
	PhotometricContraints pcon;
	pcon.setMinCorrelationThreshold(m_MinCorrelationThreshold);
	pcon.setWindowSize(m_WindowSize);
	pcon.setMinEdgeThreshold(m_MinEdgeThreshold);
	pcon.testPoint = clickedTestPoint;
	pcon.debugScene = getScene();
	pcon.computeConfidence( ref_pc, m_Clouds, m_CamAtt);
}

void X4::PCIVisionTests::testRemoveSmallComponents()
{
	card32 minNumPoints = m_WindowSize;
	UnstructuredInCorePointCloud * pc = dynamic_cast<UnstructuredInCorePointCloud*>(getSelectedCloud());
	HierarchicalKNNIterator hIt(pc,32,NULL);
	VertexArray varr(pc);
	AAT posAAT = pc->getAAT("position");

	InCorePCTopologyGraph *top = new InCorePCTopologyGraph();
	VertexDescriptor vd;
	vd.pushAttrib(VAD("vindex", 2, VAD::DATA_FORMAT_INT32));
	vd.pushAttrib(VAD("color", 4, VAD::DATA_FORMAT_FLOAT32));
	top->clearAndSetup(&vd);

	float medPointDistance = getMedianPointDistance(pc);


	for( unsigned i=0;i<varr.getNumElements();i++ )
	{
		Vector3f pos = varr.getPosition3f(i);
		hIt.setSeekPointAndReset(pos);
		for( unsigned j=0;j<9 && !hIt.atEnd();j++ )
		{
			card32 n = hIt.getCurrentPointIndex();
			Vector3f npos = hIt.get3f(posAAT);
			if( i != n && norm(npos-pos)<medPointDistance*5.0f)
				top->addEdge(i,n);
			hIt.next();
		}
	}

	SceneObjectIterator * iter = pc->createIterator(SOIT::CAP_BASIC_PC|SOIT::CAP_MODIFYABLE_PC);


	ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	PointSet * ps = mIt->lockBlockReadWrite();
	SurfelGraphInit2::removeSmallComponents(ps, top, minNumPoints, NULL);
	bIt->unlock();
}

void PCIVisionTests::computeFrameConstraints() 
{
	fetchCurrentFrame();
	PointCloud * ref_pc = getSelectedCloud();
	if( !ref_pc || !ref_pc->providesAttribute("position"))
		return;

	PhotometricContraints pcon;
	pcon.setMinCorrelationThreshold(m_MinCorrelationThreshold);
	pcon.setWindowSize(m_WindowSize);
	pcon.setMinEdgeThreshold(m_MinEdgeThreshold);
	pcon.testPoint = clickedTestPoint;
	pcon.debugScene = getScene();
	pcon.buildPhotometricConstraints(NULL,NULL,NULL,ref_pc,m_Clouds,m_CamAtt,true);
}
