#ifndef PCIVisionTestsH
#define PCIVisionTestsH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------

namespace X4
{
class ParametricInCorePointCloud;
class CameraParameterAttachment;
class RegularVolume;

/**
	cvision test tool (disparity volume etc.)
*/
class X3DEFORMATION_API PCIVisionTests : public PCInteractionTool
{
	X4_CLASS( PCIVisionTests )

private:

	void clickedOnParametricCloud(ParametricInCorePointCloud* pc, const Vector3f &pos,CameraParameterAttachment * camAtt);
	void computeDisparityVolume();
	void computeDisparityVolume( ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1,
		ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2,RegularVolume * vol, BoundingBox3f &bb );
	void testImageCompare();
	void visualizeDisparity();
	void computeFrameConstraints();
	void computeConfidence();
	void testRemoveSmallComponents();
	
	void fetchCurrentFrame();
	void dropCurrentFrame();
	PointCloud * getSelectedCloud();

	float computeDisparity( const Vector3f &point, ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1,
		ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2  );
	void computeDisparityConstraint( const Vector3f &point, ParametricInCorePointCloud * p1, CameraParameterAttachment * camAtt1,
		ParametricInCorePointCloud * p2, CameraParameterAttachment * camAtt2, Vector3f &constraint_pos, Matrix3f &contraint_matrix );

	// properties
	card32	m_VolumeGrid;
	card32	m_WindowSize;
	bool		m_TransformPatches;
	float32	m_MinCorrelationThreshold;
	float32	m_MinEdgeThreshold;

	// members
	std::vector<ParametricInCorePointCloud*> m_Clouds;
	std::vector<CameraParameterAttachment*>	m_CamAtt;


public:
	PCIVisionTests(void);
	~PCIVisionTests(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual void keyDown(GeneralKey key) {}
	virtual void keyUp(GeneralKey key) {}

	virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
	virtual void mouseMoved(int32 x, int32 y) {}
	virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {}
	virtual void areaResize(card32 width, card32 height) {}

};

}

#endif