//---------------------------------------------------------------------------
#ifndef PCCDataResamplingH
#define PCCDataResamplingH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------

namespace X4 {

///
///   This class creates a resampled version of input data nodes for initial matching
///
class X3DEFORMATION_API PCCDataResampling : public PCCommand {
	X4_CLASS( PCCDataResampling )

 public:
	/// point removal - good results but slow
	static const card32 METHOD_POINT_REMOVAL = 0;
	/// grid decimation - fast and inaccurate
	static const card32 METHOD_GRID = 1;
	/// grid decimation followed by point removal
	static const card32 METHOD_HYBRID = 2;
	/// add offset before quantization?
	bool avoidZeroCrossing;
	
 private:	
	/// where to read the input from
	std::string sourceFolder;
	/// where to put the result (must exists)
	std::string targetFolder;
	/// additional prefix to filenames in case of OOC PCs (no effect otherwise)
	std::string filenamePrefix;
	/// new (guaranteed) point distance
   float32 maxDist;
   /// frames to skip (every k-th)
   card32 frameResampling;
	/// how to perform the resampling (default: grid - it's really fast)
   card32 method;
   
 public:
	PCCDataResampling();
	
	void setup(
		std::string sourceFolder,
		std::string targetFolder,
		float32 maxDist,
		card32 frameResampling
	);

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);
	virtual void execute(Scene *scene);

	std::string getSourceFolder() const {return sourceFolder;}
	void setSourceFolder(std::string v) {sourceFolder=v;}
	std::string getTargetFolder() const {return targetFolder;}
	void setTargetFolder(std::string v) {targetFolder=v;}
	float32 getMaxDist() const {return maxDist;}
	void setMaxDist(float32 v) {maxDist=v;}
	card32 getMethod() const {return method;}
	void setMethod(card32 v) {method=v;}
	card32 getFrameResampling() const {return frameResampling;}
	void setFrameResampling(card32 v) {frameResampling=v;}
	static void gridResampling(PointSet *ps, float32 cellSize, bool avoidZeroCrossing);

   ~PCCDataResampling();
};

}

#endif