//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCDeformation.h"
//---------------------------------------------------------------------------
//#define _CRTDBG_MAP_ALLOC
//#include <stdlib.h>
//#include <crtdbg.h>

#include "CopyObjectProperties.h"
#include "SGObjectNode.h"
#include "SGListNode.h"
#include "SceneObject.h"
#include "SceneGraphNode.h"
#include "DeformationSolver.h"
#include "DeformationEvaluator.h"
#include "FDDeformationEvaluator.h"
#include "FDDeformationSolver.h"
#include "MMDeformationSolver.h"		
#include "SGDeformationConstraintsList.h"
#include "SGEShape.h"
#include "SGEDeformationSequence.h"
#include "ARSettings.h"
#include "ARDeformationOptimizationContext.h"
#include "Timer.h"
#include "TSDeformationSolver.h"
#include "TSDeformationContext.h"
#include <fstream>
//#include "FastSparseMatrix.h"
#include "DynamicLinearAlgebra.h"
#include "DynamicLinearAlgebra.inline.h"
#include "ProgressWindow.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( PCCDeformation , 3 ) {
	BEGIN_CLASS_INIT( PCCDeformation );
   ADD_STRING_PROP(sourceNode, 0)
   ADD_STRING_PROP(targetNode, 0)
   ADD_STRING_PROP(constraintsNode, 0)
   ADD_STRING_PROP(deformationStorageNode_deprecated, 0)
   SET_USER_EDIT(false)
   ADD_BOOLEAN_PROP(transformNormals, 2)
   ADD_OBJECT_PROP(settings, 1, ARSettings::getClass(), true)
   ADD_OBJECT_PROP(solver, 0, DeformationSolver::getClass(), true)
   ADD_OBJECT_PROP(evaluator_deprecated, 0, DeformationEvaluator::getClass(), true)
   SET_USER_EDIT(false)
   ADD_NOARGS_METHOD(initializeNodesFromSource)
   SET_METHOD_AS_CMD_OBJ( true )
   ADD_NOARGS_METHOD(clearAndInitFromSource)
   SET_METHOD_AS_CMD_OBJ( true )
   ADD_NOARGS_METHOD(executeWithExampleConstraints)
   SET_METHOD_AS_CMD_OBJ( true )
   ADD_NOARGS_METHOD(testTSSolver)
   ADD_NOARGS_METHOD(tuneSparseMatrixSolver)

   ADD_STRING_PROP(matrixName, 3)
   ADD_STRING_PROP(bVecName ,  3)
}

// ----

PCCDeformation::PCCDeformation() {
   sourceNode = "root/pc1";
   targetNode = "root/deformed";
   constraintsNode = "root/constraints";
   settings = new ARSettings();
   solver = new MMDeformationSolver();		
   //solver = new FDDeformationSolver();		
   evaluator_deprecated = NULL;
   transformNormals = false;
   useExampleConstraints = false;

   matrixName = "F:\\aberner\\stefan\\data\\heq512Matrix.txt";
   bVecName   = "F:\\aberner\\stefan\\data\\lenna512.txt";

   matrixName = "F:\\aberner\\stefan\\data\\heq2048Matrix.txt";
   bVecName   = "F:\\aberner\\stefan\\data\\earth2048.txt";
}

void PCCDeformation::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

PCCDeformation::~PCCDeformation() {
	delete evaluator_deprecated;
}





void PCCDeformation::initializeNodesFromSource()
{
   {  // --- constraintsNode
      std::string::size_type offset = constraintsNode.find_last_of('/');
      static const std::string::size_type npos = -1;
      if (offset == npos) {
         error("PCCDeformation::initializeNodes() - constraintsNode is not a valid path (no separator char '/' found).");
         return;
      }
      std::string nodeName = constraintsNode.substr(offset+1, constraintsNode.length()-offset);
      std::string pathName = constraintsNode.substr(0, offset);
      SceneGraphNode *node = getSceneGraphNodeByName(getScene(), constraintsNode);
      if (node != NULL) {
         message("PCCDeformation::initializeNodes() - constraintsNode does already exists. Continuing with creating the other nodes...");
      } else {
         SceneGraphNode *parent = getSceneGraphNodeByName(getScene(), pathName);	
         if (!parent) {
            error("PCCDeformation::initializeNodes() - containing node for 'constraintsNode' not found, please check path.");
            return;
         }
         SGDeformationConstraintsList *result = new SGDeformationConstraintsList();
         result->setName(nodeName);
         if (!parent->addChildNode(result)) {
            error("PCCDeformation::initializeNodes() - specified parent does not take additional child nodes");
            delete result;
            return;
         }
      }   
   }
   
   {  // --- targetNode
      std::string::size_type offset = targetNode.find_last_of('/');
      static const std::string::size_type npos = -1;
      if (offset == npos) {
         error("PCCDeformation::initializeNodes() - targetNode is not a valid path (no separator char '/' found).");
         return;
      }
      std::string nodeName = targetNode.substr(offset+1, targetNode.length()-offset);
      std::string pathName = targetNode.substr(0, offset);
      SceneGraphNode *node = getSceneGraphNodeByName(getScene(), targetNode);
      if (node != NULL) {
         warning("PCCDeformation::initializeNodes() - targetNode does already exists. Please check the name/path. Continuing creating the other nodes...");
      } else {
         SceneGraphNode *parent = getSceneGraphNodeByName(getScene(), pathName);	
         if (!parent) {
            error("PCCDeformation::initializeNodes() - containing node for 'targetNode' not found, please check path.");
            return;
         }
         
         SGObjectNode *sgoSource = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), sourceNode));
         if (!sgoSource) {
				error("PCCDeformation::initializeNodes() - SGObjectNode for source node not found.");
				return;
         }
         
         SceneObject *so = sgoSource->getSceneObject();
			if (!so) {
				error("PCCDeformation::initializeNodes() - SGObjectNode for source is empty.");
				return;
			}
         SceneObject *copySO = (SceneObject*)so->copy();
         BoundingBox3f soBB(NULL_VECTOR3F, makeVector3f(1,1,1));
         if (dynamic_cast<PointCloud*>(copySO)) {
            soBB = getPCBBox((PointCloud*)copySO);
         }
         //copySO->setTransformation(makeTranslation4f(0,1.2*soBB.getSideLength(1),0));
         SGObjectNode *result = new SGObjectNode();
         result->clearAndSetup(copySO);
         result->setName(nodeName);
         if (!parent->addChildNode(result)) {
            error("PCCDeformation::initializeNodes() - parent specified for targetNode does not take additional child nodes");
            delete result;
            return;
         }
      }   
   }   
}

void PCCDeformation::execute(Scene *scene) 
{
   SGDeformationConstraintsList *cList = dynamic_cast<SGDeformationConstraintsList*>
      (getSceneGraphNodeByName(scene, constraintsNode));
   if (!cList) {
      error("PCCDeformation::execute() - constraintsNode not found or of wrong type.");
      return;
   }
   mpcard n = cList->getNumConstraints();

   // Changed by pj - 4.12.2007
   AROptimizationContext *cont = solver->createOptContext();
	ARDeformationOptimizationContext *context = dynamic_cast<ARDeformationOptimizationContext*>( cont );
   
   Timer t;
   PointCloud *pcDomain = getPointCloud(scene, sourceNode);
   if (!pcDomain) throw X4Exception("PCCDeformation::execute() - can't find sourceNode.");
   context->initializeFromShape(pcDomain, makeVector2f(0,0), settings);
   std::cout << "initializeFromSceneGraph: " << t.getDeltaValue() << endl;
   solver->addUserFeedbackCallback(Delegate(this, &PCCDeformation::redrawScene));
   solver->setup(context, settings);
   solver->beginConstraints();
   if (useExampleConstraints && dynamic_cast<FDDeformationSolver*>(solver)) {
      dynamic_cast<FDDeformationSolver*>(solver)->addExampleConstraints();
   } else {
      for (mpcard i=0; i<n; i++) {
         solver->addGeneralConstraint(cList->getConstraint(i));
      }
   }
   std::cout << "solver->addGeneralConstraint: " << t.getDeltaValue() << endl;
   
   solver->solve();
   std::cout << "solver->solve: " << t.getDeltaValue() << endl;
   
   SGObjectNode *sgoSource = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), sourceNode));
   if (!sgoSource) {
		error("PCCDeformation::initializeNodes() - SGObjectNode for source node not found.");
		return;
   }
   
   PointCloud *pc_s = dynamic_cast<PointCloud*>(sgoSource->getSceneObject());
	if (!pc_s) {
		error("PCCDeformation::initializeNodes() - SGObjectNode for source is not a PointCloud.");
		return;
	}

   PointCloud *pc_t = getPointCloud(scene, targetNode);
   if (!pc_t) {
	  pc_t = dynamic_cast<PointCloud*>(pc_s->copy());
	  addPointCloud(scene,pc_t,targetNode);
      //error("PCCDeformation::execute() - cannot apply deformation - targetNode not found or not a point cloud.");
      //return;
   }


   SceneObjectIterator *it_t = pc_t->createIterator(SOIT::CAP_BASIC_PC);
   BasicPointCloudIterator *bit_t = dynamic_cast<BasicPointCloudIterator*>(it_t);
   ModifyablePointCloudIterator *mit_t = dynamic_cast<ModifyablePointCloudIterator*>(it_t);
   if (!bit_t || !mit_t) {
      error("PCCDeformation::execute() - cannot apply deformation - targetNode does not provide basic and modifyable point cloud iterator interface.");
      delete it_t;
      return;
   }
   SceneObjectIterator *it_s = pc_s->createIterator(SOIT::CAP_BASIC_PC);
   BasicPointCloudIterator *bit_s = dynamic_cast<BasicPointCloudIterator*>(it_s);
   if (!bit_s) {
      error("PCCDeformation::execute() - cannot apply deformation - sourceNode does not provide basic point cloud iterator interface.");
      delete it_t;
      delete it_s;
      return;
   }
   AAT posAAT_S;
   AAT posAAT_T;
   AAT normalAAT_S = NULL_AAT;
   AAT normalAAT_T = NULL_AAT;
   bool normals = false;
   if (pc_s->getDescr()->providesAttribute("position",3, VAD::DATA_FORMAT_FLOAT32)) {
      posAAT_S = pc_s->getAAT("position");
   } else {
      error("PCCDeformation::execute() - cannot apply deformation - sourceNode does not have a position channel.");
      delete it_s;
      delete it_t;
      return;
   }
   if (pc_t->getDescr()->providesAttribute("position",3, VAD::DATA_FORMAT_FLOAT32)) {
      posAAT_T = pc_t->getAAT("position");
   } else {
      error("PCCDeformation::execute() - cannot apply deformation - targetNode does not have a position channel.");
      delete it_s;
      delete it_t;
      return;
   }
   if (   pc_s->getDescr()->providesAttribute("normal", 3, VAD::DATA_FORMAT_FLOAT32)
       && pc_t->getDescr()->providesAttribute("normal", 3, VAD::DATA_FORMAT_FLOAT32) ) 
   {
      normals = true;
      normalAAT_S = pc_s->getAAT("normal");
      normalAAT_T = pc_t->getAAT("normal");
   }
   std::cout << "Before evaluator: " << t.getDeltaValue() << endl;
   DeformationEvaluator *evaluator = solver->createEvaluator();

   evaluator->setup(context, settings);
   evaluator->beginQuery(0);
   ARConstraintAnchor at;
   card32 id=0;
   while (!bit_s->atEnd() && !bit_t->atEnd()) {
      at.setPos(bit_s->get3f(posAAT_S));
      //at.setTopologicalAnchor(ARConstraintAnchor::NO_TOP_ANCHOR);
	  at.setTopologicalAnchor(id);
      mit_t->set3f(posAAT_T, evaluator->getPosition(at));
      if (normals && transformNormals) {
         Matrix3f jac = evaluator->getSpatialGradient(at);
		 /*std::cout << "-J-" << endl;
		 std::cout << jac[0][0] << " " << jac[0][1] << " " <<jac[0][2] << endl;
		 std::cout << jac[1][0] << " " <<jac[1][1] << " " <<jac[1][2] << endl;
		 std::cout << jac[2][0] << " " <<jac[2][1] << " " <<jac[2][2] << endl;
         jac = invertMatrix(jac).transpose();
		 std::cout << "-J-1-" << endl;
		 std::cout << jac[0][0] << " " << jac[0][1] << " " <<jac[0][2] << endl;
		 std::cout << jac[1][0] << " " <<jac[1][1] << " " <<jac[1][2] << endl;
		 std::cout << jac[2][0] << " " <<jac[2][1] << " " <<jac[2][2] << endl;*/
         Vector3f normal = jac * bit_s->get3f(normalAAT_S);
         mit_t->set3f(normalAAT_S, normal);
      }
      bit_s->next();
      bit_t->next();
	  id++;
   }
   delete it_s;
   delete it_t;
   std::cout << "Deform geometry: " << t.getDeltaValue() << endl;

   context->visualize(scene);
   
   delete context;

   //_CrtDumpMemoryLeaks();
}

void PCCDeformation::clearAndInitFromSource() 
{
   deleteSceneGraphNode(getScene(), targetNode);
   initializeNodesFromSource();
}

void PCCDeformation::executeWithExampleConstraints() 
{
   useExampleConstraints = true;
   execute(getScene());
   useExampleConstraints = false;
}

void PCCDeformation::testTSSolver() 
{
   PointCloud *pcDomain = getPointCloud(scene, sourceNode);
   if (!pcDomain) throw X4Exception("PCCDeformation::execute() - can't find sourceNode.");

   TSDeformationSolver solver;
   TSDeformationContext *ctxt = dynamic_cast<TSDeformationContext*>(solver.createOptContext());
   ctxt->initializeFromShape(pcDomain, NULL_VECTOR2F, settings);
   ctxt->debugOutputVis();
   delete ctxt;
}

void X4::PCCDeformation::tuneSparseMatrixSolver(){

		// parameters:


/*

		debugOutput << "loading: " << matrixName << "\n";
		debugOutput.flush();



		//load matrix
		std::ifstream in;
		in.clear();
		in.open(matrixName.c_str(),std::ifstream::in);
		if (!in.good()) { 
			warning("\nError opening file \"" + matrixName + "\"!"); 
			return ;
		};

		int m, n, numElements;
		in >> m; in >> n; in >> numElements;
		float scaling;
		in >> scaling;
		int row, col, value;

		SparseMatrixF sm(n);

		int tenPercent = numElements/20;
		
		progressWindow->pushStep(true,"loadFromFile");
		int i = 0;
		progressWindow->progress(0);
		while( !in.eof() ) {
			in >> row; in >> col; in >> value;
			sm[row-1].addEntryBinary(col-1, value);
			if(i%tenPercent == 0)
				progressWindow->progress(i*100/numElements);
			i++;
		}
		progressWindow->progress(100);
		progressWindow->popSteps();

		in.close();


		//loadVector b
		debugOutput << "loading: " << bVecName << "\n";
		debugOutput.flush();

		in.clear();
		in.open(bVecName.c_str(),std::ifstream::in);
		if (!in.good()) { 
			warning("\nError opening file \"" + bVecName + "\"!"); 
			return ;
		};

		in >> m; in >> n;
		in >> scaling;

		DVectorF bVec;
		bVec.setDim(n);

		while( !in.eof() ) {
			in >> row;  in >> value;
			bVec[row-1] = value;

		}

		in.close();


		// ------------------------------------

		FastSparseMatrix fsm;

		fsm.initUpperHalfMatrixFromFullSymmetricSparseMatrixF(sm);
		DVectorF xVec; xVec.setDim(n);

		
	debugOutput << "sysSolve(): A ("  << sm.getNumRows() << "x" << sm.getNumRows() << ") x(" 
	<< xVec.getDim() << ") = b(" << bVec.getDim() << ")\n" ;
	xVec.setZero();
	
	Timer t;
	for(mpcard i = 0; i < 20; i++){
		fsm.solveWithIntel(xVec, bVec);
 		debugOutput << "solveLinearSystem: "  << t.getDeltaValue() << "\n";
		xVec.setZero();
	}
		
	fsm.solveWithIntel(xVec, bVec);
	for(mpcard i = 0; i < 10; i++){
		debugOutput << xVec[i] << "\n";
	}
	}
	}*/


}
