#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCImportDepthAnimation.h"
//---------------------------------------------------------------------------
#include "ParametricInCorePointCloud.h"
#include "SGObjectNode.h"
#include "SGListNode.h"
#include "PointCloudImporterReaderMSDepthBMP.h"
#include "ProgressWindow.h"
#include "StringHelper.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
#include "SGRelativeTimeAnimationNode.h"
#include "PointSet.inline.h"
#include "LinearAlgebra.inline.h"
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCCImportDepthAnimation ,2)
{
	BEGIN_CLASS_INIT( PCCImportDepthAnimation );
	ADD_STRING_PROP(importPath, 0);
	ADD_STRING_PROP(camCalibFile, 0);
	ADD_FLOAT32_PROP(MinZ,0);
	ADD_FLOAT32_PROP(MaxZ,0);
	ADD_STRING_PROP(oocSwapFileNamePath,0);
	ADD_CARD32_PROP(firstFrame,1);
	ADD_CARD32_PROP(lastFrame,1);
	ADD_BOOLEAN_PROP(mergePointClouds,2);
}

PCCImportDepthAnimation::PCCImportDepthAnimation(void)
{
	importPath="C:\\localhome\\bokeloh\\datasets\\MSR3DVideo-Ballet\\";
	camCalibFile="calibParams-ballet.txt";
	oocSwapFileNamePath="C:\\localhome\\bokeloh\\datasets\\ballet_merged\\";
	MinZ = 42.0, MaxZ = 130.0;
	firstFrame=0;
	lastFrame=1;
	mergePointClouds = true;
}

PCCImportDepthAnimation::~PCCImportDepthAnimation(void)
{
}

void PCCImportDepthAnimation::assign(const Object* obj, X4CopyContext *context)
{
	const PCCImportDepthAnimation * o = dynamic_cast<const PCCImportDepthAnimation*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::PCCImportDepthAnimation::execute( Scene *scene )
{

	SGListNode *rootNode = dynamic_cast<SGListNode*>(scene->getRootNode());
	std::vector<SceneGraphNode*> animNodes3D;
	std::vector<SceneGraphNode*> animNodes2D;
	for (card32 cam=0; cam<8;cam++)
	{
		if( !mergePointClouds )
		{
			SceneGraphNode *AnimNode = (SceneGraphNode*)(new SGRelativeTimeAnimationNode());
			animNodes3D.push_back(AnimNode);
			AnimNode->setName("3D-Cam_"+intToStr(cam));
			rootNode->addChildNode(AnimNode);
		}

		SceneGraphNode *AnimNode = (SceneGraphNode*)(new SGRelativeTimeAnimationNode());
		animNodes2D.push_back(AnimNode);
		AnimNode->setName("2D-Cam_"+intToStr(cam));
		rootNode->addChildNode(AnimNode);

	}

	SceneGraphNode *dataNode = (SceneGraphNode*)(new SGRelativeTimeAnimationNode());
	dataNode->setName("data");
	rootNode->addChildNode(dataNode);



	VertexDescriptor* colorVD = new VertexDescriptor;
	colorVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	colorVD->pushAttrib(mVAD("depth", 1, VAD::DATA_FORMAT_FLOAT32));

	VertexDescriptor* m_vd = new VertexDescriptor;
	m_vd->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
//	m_vd->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
	m_vd->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	m_vd->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));

	card32 positionOffset = m_vd->getAttributeOffset(m_vd->getIndex("position"));
	card32 colorOffset = m_vd->getAttributeOffset(m_vd->getIndex("color"));

	card32 colParaOffset=   colorVD->getAttributeOffset(colorVD->getIndex("color"));
	card32 depthParaOffset= colorVD->getAttributeOffset(colorVD->getIndex("depth"));


	card32 camNum=8;
	card32 numFrames=lastFrame-firstFrame+1;
	card32 numFiles=camNum*numFrames;
	card32 fileNumber=0;
	// init progress window
	progressWindow->pushStep(true, (std::string)"import " + intToStr(numFrames)+ " frames");
	progressWindow->progress(0);


	for (card32 frameNo=firstFrame; frameNo<=lastFrame; frameNo++)
	{
		UnstructuredInCorePointCloud * mergedPC = NULL;

		card64 numPoints = 0;
		float32 allFramePoints=(1024.0f*768.0f*8.0f);
		for (card32 cam=0; cam<camNum;cam++)
		{
			
			// update progress window
			fileNumber++;
			if (!progressWindow->progress((float32)fileNumber/(float32)numFiles*100.0f)) 
				{ 
					// escape pressed: abort
					return;
				};
			
			
			
			ParametricInCorePointCloud* para_pc = new ParametricInCorePointCloud();
			((ParametricInCorePointCloud*)para_pc)->clearAndSetup(0, 0, colorVD);
			
			UnstructuredInCorePointCloud* pc;
			if( !mergePointClouds )
			{
				pc =new UnstructuredInCorePointCloud();
				((UnstructuredInCorePointCloud*)pc)->clearAndSetup(m_vd,0);
			}
			else
			{
				if( cam > 0)
					pc = mergedPC;
				else
				{
					mergedPC = new UnstructuredInCorePointCloud;
					mergedPC->clearAndSetup(m_vd,0);
					pc = mergedPC;
				}
			}
			
		
			SceneObjectIterator *newIt = pc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PC);
			//SceneObjectIterator *newIt_para = pc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PARAMETRIC_PC);
			if (newIt == NULL) return;
			//BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
			//if (basicIt == NULL) {delete newIt; return ;}
			//DynamicParametricPointCloudIterator *dynparIt = dynamic_cast<DynamicParametricPointCloudIterator*>(newIt);
			//if (dynparIt == NULL) {delete newIt; return ;}
			DynamicPointCloudIterator* dynIt= dynamic_cast<DynamicPointCloudIterator*>(newIt);


			SceneObjectIterator *newIt_para = para_pc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PARAMETRIC_PC);
			if (newIt_para == NULL) return;
			BasicPointCloudIterator *basicIt_para = dynamic_cast<BasicPointCloudIterator*>(newIt_para);
			if (basicIt_para == NULL) {delete newIt; return ;}
			ModifyablePointCloudIterator *modIt_para = dynamic_cast<ModifyablePointCloudIterator*>(newIt_para);
			if (modIt_para == NULL) {delete newIt; return ;}
			DynamicParametricPointCloudIterator *dynparIt = dynamic_cast<DynamicParametricPointCloudIterator*>(newIt_para);
			if (dynparIt == NULL) {delete newIt; return ;}


			card32 width = 1024;
			card32 height = 768;
			dynparIt->setSize(width, height);

			std::vector<card8> card8vector;

			std::string frameNumStr=intToStr(frameNo);
			if (frameNumStr.size()==1)
			{
				frameNumStr.insert(0,1,'0');
			}
			std::string filnam=	importPath+"cam"+"0"+"\\"+"depth-cam1-f0"+frameNumStr+".bmp";
			std::string file = removePathFromFilename( filnam );
			std::string path = removeFilenameFromPath( filnam );


			std::string fileNumber = file.substr( file.length() - 7, 3 );
			


			QImage* m_ColorImage = new QImage(QString( (path + "../cam" + intToStr(cam) + "/color-cam" + intToStr(cam) + "-f" + fileNumber + ".bmp").c_str() ) );
			QImage* m_DepthImage = new QImage(QString( (path + "../cam" + intToStr(cam) + "/depth-cam" + intToStr(cam) + "-f" + fileNumber + ".bmp").c_str() ) );
			
			
			if( !readCalibrationFile( (char*)(importPath + camCalibFile).c_str() ) )
				return;
			computeProjectionMatrices();
			
			card8* buffer = new card8[m_vd->getSize()];
			for( card32 i=0;i<m_vd->getSize();i++ )
				buffer[i] = 0;
			card8* buffer_para = new card8[colorVD->getSize()];
			for( card32 i=0;i<colorVD->getSize();i++ )
				buffer_para[i] = 0;

			card32 x=0;
			card32 y=0;

			bool ImportFinished=false;

			while (!ImportFinished) 
			{


				// color data
				QRgb pixel;
				pixel = m_ColorImage->pixel(x,y);
				Vector3f color = makeVector3f(qRed(pixel),qGreen(pixel),qBlue(pixel)) / 255.0f;
				//Vector3f color = makeVector3f(1,1,1);

				// position data
				pixel = m_DepthImage->pixel(x,y);
				double depth = DepthLevelToZ( qGray(pixel) );

				double u = (double)x;
				double v = (double)y;

				double worldX, worldY;
				projUVZtoXY( m_CalibParams[cam].m_ProjMatrix, u,v, depth, &worldX, &worldY );

				//projXYZtoUV( m_CalibParams[4].m_ProjMatrix, worldX, worldY, depth, &v, &u );

				Vector3f pos;
				pos[0] = worldX;
				pos[1] = worldY;
				pos[2] = depth;

				StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

				*((StaticVector<float32, 3>*)(buffer + positionOffset)) = pos;
				*((StaticVector<float32, 3>*)(buffer + colorOffset )) = color;
				//		*((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;

				*((StaticVector<float32, 3>*)(buffer_para + colParaOffset )) = color;
				*(float*)(buffer_para + depthParaOffset) = (float)depth;


				if (buffer!=NULL) {
					//3D
					dynIt->add((const char*)&buffer[0]);
					//basicIt->next();
					//2D
					modIt_para->setVertex((const char*)&buffer_para[0]);
					basicIt_para->next();
				}

				x++;
				if (x >= 1024) {
					x = 0;
					y++;
					if (y >= 768) 
						ImportFinished = true;
				}

			}

			delete newIt;


			
			card32 stdMaterialIndex=0;
			if( !mergePointClouds)
			{
				SGObjectNode *n = new SGObjectNode();
				std::string name= "cam_"+intToStr(cam)+"_frame_"+intToStr(frameNo)+"_3D";
				n->setName( name );
				GenericSwapAllOutOfCorePointCloud *newGSA = new GenericSwapAllOutOfCorePointCloud();
				pc->setMaterialIndex(stdMaterialIndex);
				newGSA->clearAndSetup(pc, oocSwapFileNamePath+n->getName()+"_anim.swap.object", true);
				newGSA->getAttachments()->attachData((CameraParameterAttachment*)camParameters[cam]->copy());
				newGSA->setMaterialIndex(stdMaterialIndex);
				n->clearAndSetup(newGSA);
				((SGRelativeTimeAnimationNode*)animNodes3D[cam])->addNode( n );
			}
			
			SGObjectNode *n = new SGObjectNode();
			std::string name= "cam_"+intToStr(cam)+"_frame_"+intToStr(frameNo)+"_2D";
			n->setName( name );
			n->setVisible(false);
			GenericSwapAllOutOfCorePointCloud *newGSA = new GenericSwapAllOutOfCorePointCloud();
			para_pc->setMaterialIndex(stdMaterialIndex);
			newGSA->clearAndSetup(para_pc, oocSwapFileNamePath+n->getName()+"_anim.swap.object", true);
			newGSA->getAttachments()->attachData((CameraParameterAttachment*)camParameters[cam]->copy());
			newGSA->setMaterialIndex(stdMaterialIndex);
			n->clearAndSetup(newGSA);
			((SGRelativeTimeAnimationNode*)animNodes2D[cam])->addNode( n );

			if( !mergePointClouds)
				delete pc;
			delete para_pc;
		
			delete m_ColorImage;
			delete m_DepthImage;
			delete buffer;
			delete buffer_para;
		}
		//		imp->endImport();
		// finish progress window

		if( mergePointClouds )
		{
			SGObjectNode *n = new SGObjectNode();
			std::string name= "frame_"+intToStr(frameNo)+"_3D";
			n->setName( name );
			GenericSwapAllOutOfCorePointCloud *newGSA = new GenericSwapAllOutOfCorePointCloud();
			mergedPC->setMaterialIndex(0);
			newGSA->clearAndSetup(mergedPC, oocSwapFileNamePath+n->getName()+"_anim.swap.object", true);
			//newGSA->getAttachments()->attachData((CameraParameterAttachment*)camParameters[cam]->copy());
			newGSA->setMaterialIndex(0);
			n->clearAndSetup(newGSA);
			((SGRelativeTimeAnimationNode*)dataNode)->addNode( n );
			delete mergedPC;
		}


	}

	progressWindow->popSteps();
	delete colorVD;
	delete m_vd;


}

void X4::PCCImportDepthAnimation::projUVZtoXY( double projMatrix[4][4], double u, double v, double z, double *x, double *y )
{
	double c0, c1, c2;

	// image (0,0) is bottom lefthand corner
	v = (double) 768 - v - 1.0;
	//u = (double) m_Width - u - 1.0;

	c0 = z*projMatrix[0][2] + projMatrix[0][3];
	c1 = z*projMatrix[1][2] + projMatrix[1][3];
	c2 = z*projMatrix[2][2] + projMatrix[2][3];

	*y = u*(c1*projMatrix[2][0] - projMatrix[1][0]*c2) +
		v*(c2*projMatrix[0][0] - projMatrix[2][0]*c0) +
		projMatrix[1][0]*c0 - c1*projMatrix[0][0];

	*y /= v*(projMatrix[2][0]*projMatrix[0][1] - projMatrix[2][1]*projMatrix[0][0]) +
		u*(projMatrix[1][0]*projMatrix[2][1] - projMatrix[1][1]*projMatrix[2][0]) +
		projMatrix[0][0]*projMatrix[1][1] - projMatrix[1][0]*projMatrix[0][1];

	*x = (*y)*(projMatrix[0][1] - projMatrix[2][1]*u) + c0 - c2*u;
	*x /= projMatrix[2][0]*u - projMatrix[0][0];
	*x = - *x;
}

X4::float32 X4::PCCImportDepthAnimation::DepthLevelToZ( unsigned char d )
{
	double z;

	z = 1.0/((d/255.0)*(1.0/MinZ - 1.0/MaxZ) + 1.0/MaxZ);
	return z;
}

X4::card8 X4::PCCImportDepthAnimation::ZToDepthLevel( float32 z )
{
	float32 d;
	float32 range=(1.0f/MinZ - 1.0f/MaxZ);
	float32 max=1.0/MaxZ;

	d   = (((1.0/z)-max)/range)*255.0;

	return (card8)(d+0.5f);
}

std::string X4::PCCImportDepthAnimation::getPointCloudName( card32 time, card32 cam, bool picture )
{
	if (picture)
	{
		std::string path="2D-Cam_"+intToStr(cam)+"/";
		std::string name= "cam_"+intToStr(cam)+"_frame_"+intToStr(time)+"_2D";
		return "root/"+path+name;
	}
	
	std::string path="3D-Cam_"+intToStr(cam)+"/";
	std::string name= "cam_"+intToStr(cam)+"_frame_"+intToStr(time)+"_3D";
	return "root/"+path+name;
}

bool PCCImportDepthAnimation::readCalibrationFile(char *fileName)
{
	int i, j, k;
	FILE *pIn;
	double dIn; // dummy variable
	int camIdx;

	if(pIn = fopen(fileName, "r"))
	{
		for(k=0; k<8; k++)
		{
			// camera index
			fscanf(pIn, "%d", &camIdx);

			// camera intrinsics
			for (i=0; i<3; i++)
				fscanf(pIn, "%lf\t%lf\t%lf", &(m_CalibParams[camIdx].m_K[i][0]),
				&(m_CalibParams[camIdx].m_K[i][1]), &(m_CalibParams[camIdx].m_K[i][2]));

			// read barrel distortion params (assume 0)
			fscanf(pIn, "%lf", &dIn);
			fscanf(pIn, "%lf", &dIn);

			// read extrinsics
			for(i=0;i<3;i++)
			{
				for(j=0;j<3;j++)
				{
					fscanf(pIn, "%lf", &dIn);
					m_CalibParams[camIdx].m_RotMatrix[i][j] = dIn;
				}

				fscanf(pIn, "%lf", &dIn);
				m_CalibParams[camIdx].m_Trans[i] = dIn;
			}

		}

		fclose(pIn);
	}
	else
		return false;
	return true;
} // readCalibrationFile

void PCCImportDepthAnimation::computeProjectionMatrices()
{
	int i, j, k, camIdx;
	double (*inMat)[3];
	double exMat[3][4];

	for(camIdx=0; camIdx<8; camIdx++)
	{
		// The intrinsic matrix
		inMat = m_CalibParams[camIdx].m_K;

		// The extrinsic matrix
		for(i=0;i<3;i++)
		{
			for(j=0;j<3;j++)
			{
				exMat[i][j] = m_CalibParams[camIdx].m_RotMatrix[i][j];
			}
		}

		for(i=0;i<3;i++)
		{
			exMat[i][3] = m_CalibParams[camIdx].m_Trans[i];
		}

		// Multiply the intrinsic matrix by the extrinsic matrix to find our projection matrix
		for(i=0;i<3;i++)
		{
			for(j=0;j<4;j++)
			{
				m_CalibParams[camIdx].m_ProjMatrix[i][j] = 0.0;

				for(k=0;k<3;k++)
				{
					m_CalibParams[camIdx].m_ProjMatrix[i][j] += inMat[i][k]*exMat[k][j];
				}
			}
		}

		m_CalibParams[camIdx].m_ProjMatrix[3][0] = 0.0;
		m_CalibParams[camIdx].m_ProjMatrix[3][1] = 0.0;
		m_CalibParams[camIdx].m_ProjMatrix[3][2] = 0.0;
		m_CalibParams[camIdx].m_ProjMatrix[3][3] = 1.0;
	}


	//copy to attachment
	camParameters.clear();

	for(camIdx=0; camIdx<8; camIdx++)
	{
		Matrix3f intrinsicMatrix;
		Matrix3f rotationMatrix;
		Vector3f translationVector;
		Matrix4f projectionMatrix;

		for (card32 i=0; i<3; i++)
			for (card32 j=0; j<3; j++)
				intrinsicMatrix[i][j]= (float)m_CalibParams[camIdx].m_K[i][j];
			
		for (card32 i=0; i<3; i++)
			for (card32 j=0; j<3; j++)
				rotationMatrix[i][j]= (float)m_CalibParams[camIdx].m_RotMatrix[i][j];		
		
		for (card32 i=0; i<3; i++)
			translationVector[i]= (float)m_CalibParams[camIdx].m_Trans[i];

		for (card32 i=0; i<4; i++)
			for (card32 j=0; j<4; j++)
				projectionMatrix[i][j]= (float)m_CalibParams[camIdx].m_ProjMatrix[i][j];	

		CameraParameterAttachment* camPara= new CameraParameterAttachment(intrinsicMatrix, rotationMatrix, translationVector, projectionMatrix);
		camPara->setup(CameraParameterAttachment::getAttachmentName(), AttachedData::ADF_PERSISTENT);
		camParameters.push_back(camPara);

	}

}
