//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCImportMPIIScannerASC.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ImportTools.h"
#include "PointCloudTools.h"
#include "FileNamePropertyEditor.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
#include <fstream>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( PCCImportMPIIScannerASC , 0 ) {
	BEGIN_CLASS_INIT( PCCImportMPIIScannerASC );
   ADD_STRING_PROP(filenameWithoutExtension, 0)
	SET_EDITOR(FileNamePropertyEditor::getClass())
}

// ----

PCCImportMPIIScannerASC::PCCImportMPIIScannerASC() {
}

PCCImportMPIIScannerASC::~PCCImportMPIIScannerASC() {
}



struct VType {
   Vector3f pos;
   int32 flags;
   Vector4f color;
};

void PCCImportMPIIScannerASC::execute(Scene *scene) 
{
   std::string ascFile = filenameWithoutExtension+".asc";
   std::string pngFile = filenameWithoutExtension+".png";
   std::string path = removePathFromFilename(filenameWithoutExtension);
   std::ifstream in(ascFile.c_str());
   if (!in.is_open()) {
      throw X4Exception("PCCImportMPIIScannerASC::execute() - file not found");
      return;
   }
   
   
   QImage img(QString(pngFile.c_str()));
   if (!(img.width() == 640 && img.height() == 480)) {
      throw X4Exception("PCCImportMPIIScannerASC::execute() - image file not found");
      return;
   }
   skipToEOL( in );
   skipToEOL( in );
   skipToEOL( in );
   skipToEOL( in );
   skipToEOL( in );
   skipToEOL( in );
   UnstructuredInCorePointCloud *result = new UnstructuredInCorePointCloud();
   VertexDescriptor vd;
   vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
   vd.pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
   vd.pushAttrib(mVAD("color", 4, VAD::DATA_FORMAT_FLOAT32));
   result->clearAndSetup(&vd, 640*480);
   VType v;
   v.flags = 0;
   PointCloudIterator *it = result->createIterator(SOIT::CAP_MODIFYABLE_PC);
   BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
   ModifyablePointCloudIterator *mit = dynamic_cast<ModifyablePointCloudIterator*>(it);
   x4Assert(bit && mit);
   AAT posAAT = vd.getToken("position");
   AAT flagsAAT = vd.getToken("flags");
   AAT colorAAT = vd.getToken("color");
   while (!in.eof()) {
      in >> v.pos[0];
      in >> v.pos[1];
      in >> v.pos[2];
      int index;
      in >> index;
      QRgb pix = img.pixel(index%640, index/640);
      v.color = makeVector4f(qRed(pix)/255.0f, qGreen(pix)/255.0f, qBlue(pix)/255.0f, 1);
      mit->set3f(posAAT, v.pos);
      mit->set1i(flagsAAT, 0);
      mit->set4f(colorAAT, v.color);
      bit->next();
   }
   delete it;
   result->setMaterialIndex(2);
   addPointCloud(scene, result, path);
}
