//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SGEMeshlessUrshape.h"
#include "SGListNode.h"
#include "SGobjectNode.h"
#include "DeformationTools.h"
#include "AttachedData.h"
#include "TopologyAttachment.h"
#include "UnstructuredInCorePointCloud.h"
#include "InCorePCTopologyGraph.h"
#include "PCCNodesTools.h"
#include "CopyObjectProperties.h"
#include "TopologyRangeSearch.h"
#include "PointSetKNNQuery.h"
#include "PointCloudTools.h"
#include "PointSetSphericalQuery.h"

#include <iostream>
#include <fstream>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

static std::string describeType(card32 q) {
	if ( q == RANDOM ) {
		return "RANDOM";
	} else if ( q == TOP_SEARCH) {
		return "TOPOLOGICAL SEARCH";
	} else if ( q == TOP_SEARCH2) {
		return "TOPOLOGICAL SEARCH 2";
	} else {
		return "UNDEFINED";
	}
}

IMPLEMENT_X4_CLASS( PCCNodesTools , 0 ) {
	BEGIN_CLASS_INIT( PCCNodesTools );
	ADD_INT32_PROP(numNodes,0);
	ADD_FLOAT32_PROP(offset,0);	
	ADD_FLOAT32_PROP(radius,0);	
	ADD_INT32_PROP(minNeighbors,0);	
	ADD_CARD32_PROP_UPDATE_DESCR( type, 0, NULL, &describeType, 3, NULL )	
	ADD_NOARGS_METHOD( output )

}

// ----

PCCNodesTools::PCCNodesTools() {
	offset = 0.1f;
	numNodes = 1000;
	radius = 0.2f;
	minNeighbors = 2;
	type = TOP_SEARCH;
}

PCCNodesTools::PCCNodesTools(float32 offset, float32 radius, int32 minNeighbors, int32 numNodes, card32 type){
	this->offset = offset;
	this->radius = radius;
	this->numNodes = numNodes;
	this->type = type;
	this->minNeighbors = minNeighbors;
}


PCCNodesTools::~PCCNodesTools() {
}

void PCCNodesTools::output(){

	std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
         
   PointCloud *pc = getPointCloud( scene, selection[0] );

	UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	
	PointSet *ps = upc->getPointSet();		
		
	AttachedDataContainer *attachments = upc->getAttachments();
		
	TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
		
	AAT positionAAT;
		
	if (topAtt == NULL) {
			error("Need a topology attachement to the pointCloud");
			return;
	}
				
	if(ps->providesAttribute("position")){
		positionAAT = ps->getAAT("position");
	}
	else{
		error("No position channel in geometry");
		return;
	}

	InCorePCTopologyGraph*	topGraph  = topAtt->getTopology();

	PointSet *t = topGraph->getEdges();
	PointSet *edges = dynamic_cast<PointSet*>(t->copy());
		
		
	std::ofstream out;

	out.open("vertices.txt"); 

	if(!out) { 
		error("Cannot open file"); 
		return ; 
	} 

	out << radius << endl;

	out << ps->getHeight() << endl;
	out << edges->getHeight() << endl;

	for(unsigned j=0;j< ps->getHeight(); j++){
		Vector3f tempVertex = ps->get3f(j,positionAAT);

		out << tempVertex[0] << "\t" << tempVertex[1] << "\t" << tempVertex[2] << endl;		
	}

	for(unsigned j=0;j< edges->getHeight(); j++){
		Vector2i tempEdge = edges->get2i(j,positionAAT);

		out << tempEdge[0] << "\t" << tempEdge[1] << endl;		
	}

	out.close();

	return ;
}



void PCCNodesTools::execute(Scene *scene) {
   unsigned int m;
	
	std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
         
   for ( m = 0; m < selection.size(); m++ ){
   
      PointCloud *pc = getPointCloud( scene, selection[m] );

		if (pc == NULL) 
			continue;
			
		UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
		
		if (upc == NULL) 
			continue;
			
		if (upc) {
			PointSet *ps = upc->getPointSet();

			UnstructuredInCorePointCloud* result;
				
			if(type == TOP_SEARCH || type == TOP_SEARCH2){
				result = edgeContractionSampleMMNodesPointCloud(upc,offset,radius,true);
				
				addVerifiedTopology(result,radius, 1.5*radius);
			}
			
			else if(type == RANDOM){
				result = randomSampleMMNodesPointCloud(upc,offset,numNodes);
			}

			if(result){
				setRadii(result,20);

				//	testNodes(result,upc, 1, radius);

				float32 fourRadius = testNodes(result,upc, minNeighbors, radius);
			}
			
			if(result != NULL)
				addPointCloud(scene,result,"Deformation Nodes");			
		} 
	}
}

void PCCNodesTools::addVerifiedTopology(UnstructuredInCorePointCloud *geometry, float32 euclRadius, float32 topRadius){

		std::cout << "Adding verified Toplogy" << endl;
		
		if(geometry==NULL){
			return;
		}
		
		AAT positionAAT;
		
		PointSet *ps = geometry->getPointSet();
		
		if(geometry->providesAttribute("position")){
			positionAAT = geometry->getAAT("position");
		}
		else{
			error("No position channel for geometry in addVerifiedTopology");
			return;		
		}
		

		InCorePCTopologyGraph *top = new InCorePCTopologyGraph();
		VertexDescriptor vd;
		vd.pushAttrib(VAD("vindex", 2, VAD::DATA_FORMAT_INT32));
		top->clearAndSetup(&vd);
		
		int32 numNodes = ps->getHeight();

		AttachedIndexedOctree * octree;
		PointSetSphericalQuery * query;
		
		octree = new AttachedIndexedOctree();
		octree->setup("SimpleOctree", AttachedData::ADF_DELETEONUPDATE,  NULL);
		octree->octreesetup(ps, 20);

		// TODO: we should keep this, for the correspondences.
		query = new PointSetSphericalQuery(ps, octree, Vector3f(), euclRadius);

		bool found;
		card32 nIndex;

		set<card32> verified;
		verified.clear();	
		
		for(int32 j=0;j<(int32)ps->getHeight();j++){
				Vector3f pos = ps->get3f( j, positionAAT );
				
				query->setTargetSphere(pos, euclRadius);
				
				query->getNextPointIndex(nIndex, found);
				
				set<card32> euclNeighbors;
				euclNeighbors.clear();
				
				while(found){
					euclNeighbors.insert(nIndex);
					query->getNextPointIndex(nIndex, found);
				}
				
				verified.clear();
				verify(geometry, pos, topRadius, &euclNeighbors, &verified);
				
				for(set<card32>::iterator it = verified.begin(); it != verified.end(); it++){
					top->addEdge(j,*it);
				
				}
		}
		
		delete query;
		delete octree;
		
			
		AttachedDataContainer *attachments = geometry->getAttachments();

		TopologyAttachment *topAtt = NULL;
		topAtt = new TopologyAttachment();
		topAtt->setup("VerifiedTopologyAtt", AttachedData::ADF_PERSISTENT);
		attachments->attachData(topAtt);

		topAtt->setTopology(top);	
		
		std::cout << "done" << endl;
}

int32 PCCNodesTools::getTopAnchor(PointSet *ps, AttachedIndexedOctree* octree, Vector3f position, AAT positionAAT, AAT baseAAT){
		
		bool found;
		card32 nIndex;
		
		PointSetKNNQuery knn( ps, octree, NULL_VECTOR3F, positionAAT );	
		
		knn.setTargetPoint( position );
		
		knn.getNextNearestPointIndex( nIndex, found );
		
		if(found){
			int32 basePoint = ps->get1i(nIndex,baseAAT);			
			return basePoint;
		}
		else
			return -1;
}

void PCCNodesTools::verify(UnstructuredInCorePointCloud* nodes, Vector3f position, float32 radius, set<card32> *euclNeighbors, set<card32> *verified){

		PointSet *ps = nodes->getPointSet();		
		
		AttachedDataContainer *attachments = nodes->getAttachments();
		
		TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
		
		AAT positionAAT;
		
		*verified = *euclNeighbors;
		
		if (topAtt == NULL) {
			error("Need a topology attachement to the pointCloud");
			return ;
		}
				
		if(ps->providesAttribute("position")){
			positionAAT = ps->getAAT("position");
		}
		else{
			error("No position channel in geometry");
			return;
		}
		
		AAT baseAAT;
		
		if(ps->providesAttribute("baseIndex")){
			baseAAT = ps->getAAT("baseIndex");
		}
		else{
			error("Need baseIndex AAT in the Nodes point cloud");
			return;
		}

		AttachedIndexedOctree* octree = PointSetKNNQuery::createOctree( ps );

		InCorePCTopologyGraph*	topGraph  = topAtt->getTopology();
		
		TopNeighborhoodSphereSearch* topSearch = new TopNeighborhoodSphereSearch();	
		
		int32 anchor = getTopAnchor(ps, octree, position, positionAAT, baseAAT);
		
		if(anchor != -1){
			Vector3f anchorPosition = ps->get3f(anchor,positionAAT);

			topSearch->setup(topGraph,ps,anchor,anchorPosition,radius,positionAAT);

			set<card32> topNeighbors;

			while((!topSearch->atEnd()) ){
				card32 k = topSearch->getCurrentIndex();
				topNeighbors.insert(k);
				topSearch->next();
			}

			delete topSearch;

			verified->clear();
			
			for(set<card32>::iterator it=euclNeighbors->begin(); it!= euclNeighbors->end(); it++){
				card32 basePoint = ps->get1i(*it,baseAAT);	
				
				if(topNeighbors.find(basePoint) != topNeighbors.end()){
					verified->insert(*it);
				}
			}	
		}
}

void PCCNodesTools::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}


	
	/// get point cloud object for the mmNodes part
	UnstructuredInCorePointCloud* PCCNodesTools::createMMNodesPointCloud() const {
		UnstructuredInCorePointCloud *uc = new UnstructuredInCorePointCloud();
		
		return uc;
	
	}	
	
	// get point cloud object for the mmNodes part
	UnstructuredInCorePointCloud* PCCNodesTools::randomSampleMMNodesPointCloud(UnstructuredInCorePointCloud *geometry, float32 level, int32 numNodes)	{
		PointSet *ps = geometry->getPointSet();		
		
		std::vector<bool> to_keep(ps->getHeight(),false);
		std::vector<int32> permute(ps->getHeight());
		
		for(unsigned i=0;i<ps->getHeight(); ++i){
			permute[i] = i;
		}
		
		random_shuffle(permute.begin(), permute.end());
		
		for(unsigned i=0;i<(unsigned)numNodes;++i){
			to_keep[permute[i]] = true;
		}
		
		UnstructuredInCorePointCloud *uc = getToKeep(geometry,to_keep,offset);

		return uc;		
	}
	
	float32 PCCNodesTools::testNodes(UnstructuredInCorePointCloud *mmNodes, UnstructuredInCorePointCloud *geometry, int32 nNeighbors, float32 radius){
 		PointSet *ps = mmNodes->getPointSet();	
						
		AAT radiusAAT = mmNodes->getAAT("radius");
		
		AAT positionAAT = mmNodes->getAAT("position");
		
		int32 numNodes = ps->getHeight();

		AttachedIndexedOctree* octree = PointSetKNNQuery::createOctree( ps );
		PointSetKNNQuery knn( ps, octree, NULL_VECTOR3F, positionAAT );	
		
		bool found;
		card32 nIndex, badCount = 0, maxNeighbors = 0;
		float32 avgWithinR = 0, dmax = 0;
		
		PointSet *ps2 = geometry->getPointSet();
		AAT geometryAAT = ps2->getAAT("position");
		
		
		for(unsigned j=0;j<ps2->getHeight();j++){
				Vector3f pos = ps2->get3f( j, geometryAAT );
				Vector3f postemp;
				
				knn.setTargetPoint( pos );
				
				float32 dtemp = 0;	
				found = true;
				unsigned withinRadius = 0;
	
				for(int32 k=0;(k<nNeighbors) && (found);k++){
					knn.getNextNearestPointIndex( nIndex, found );
					
					if(found){
						postemp = ps->get3f( nIndex, positionAAT );
						
						dtemp = norm( postemp - pos );
						if(dtemp<=radius){
							withinRadius++;
						}
						
						dmax = max(dmax,dtemp);
					}
				}
				
				maxNeighbors = max(withinRadius,maxNeighbors);
				
				avgWithinR += ((float32)withinRadius)/((float32)ps2->getHeight());
				
				//ps->set1f(j,radiusAAT,d);
				if(withinRadius<(unsigned)nNeighbors){
					badCount++;
					//	std::cout << j << "\t" << d << endl;
				}
		}
		
		std::cout << " Found " << badCount << " bad geometry points" << endl;
		std::cout << " Radius " << radius << " each geometry point has " << avgWithinR << " avg. neighbors " << endl;
		std::cout << " Maximum number of geometry neighbors for this radius is " << maxNeighbors << endl;
		std::cout << " Suggested Radius: " << dmax << endl;
		
		delete octree;
		
		return dmax;
	
	}
		
	void PCCNodesTools::setRadii(UnstructuredInCorePointCloud *mmNodes,int32 nNeighbors){
	
		if(mmNodes==NULL){
			return;
		}
		
		if(!mmNodes->providesAttribute("radius")){
			error ("Need radius");
			return;
		}
		
		if(!mmNodes->providesAttribute("position")){
			error ("Need position");
			return;
		}
		
		PointSet *ps = mmNodes->getPointSet();	
						
		AAT radiusAAT = mmNodes->getAAT("radius");
		
		AAT positionAAT = mmNodes->getAAT("position");
		
		int32 numNodes = ps->getHeight();

		AttachedIndexedOctree* octree = PointSetKNNQuery::createOctree( ps );
		PointSetKNNQuery knn( ps, octree, NULL_VECTOR3F, positionAAT );	
		
		bool found;
		card32 nIndex;
		
		for(int32 j=0;j<numNodes;j++){
				Vector3f pos = ps->get3f( j, positionAAT );
				Vector3f postemp;
				
				knn.setTargetPoint( pos );
				
				float32 d=0,dtemp = 0;	
				found = true;
	
				for(int32 k=0;(k<nNeighbors) && (found);k++){
					knn.getNextNearestPointIndex( nIndex, found );
					
					if(found){
						postemp = ps->get3f( nIndex, positionAAT );
						
						dtemp = norm( postemp - pos );
						d = max(dtemp,d);
					}
				}
				
				ps->set1f(j,radiusAAT,d);
				//std::cout << j << "\t" << d << endl;
		}
		
		delete octree;
	}
	
	bool MyDataSortPredicate(const edgeType& lhs, const edgeType& rhs)
	{
		return lhs.first < rhs.first;
	}

	unsigned PCCNodesTools::getOther(Vector2i both, unsigned given) const{
		x4Assert((both[0] == given) || (both[1] == given));
		
		if(both[0] == given)
			return both[1];
		else
			return both[0];
	}
	
	
		
	//Merge vertex2 onto vertex1 through edge edgeIndex
	bool PCCNodesTools::vertexMerge(int32 v1, int32 v2, int32 edgeIndex, PointSet *edges, std::vector< set<card32> > *edgeTable, float32 radius, PointSet *geometry, priority_queue< edgeType, std::vector<edgeType>, Prioritize> *edgeIndices, std::vector<bool> *to_keep, std::vector<card32> *counter, bool useQueue){

		if(!(edges->providesAttribute("vindex"))){
			error("No vindex attribute for edge pointset in Vertex Merge");
			return false;
		}
		AAT edgeAAT = edges->getAAT("vindex");
		
  		set<card32>::iterator it,temp_it;
  		
  		//card32 t;
		
		set<card32> v1Edges = (*edgeTable)[v1];
		set<card32> v2Edges = (*edgeTable)[v2];
	
		std::vector<int> int_set;
		std::vector<int> diff_set;
		std::vector<card32> int_set_count;
		std::vector<card32> diff_set_count;
		
		set<card32> v1Neighbors;
		set<card32> v2Neighbors;

		for(it=v1Edges.begin(); it != v1Edges.end(); ++it){
			Vector2i tempEdge = edges->get2i(*it,edgeAAT);

			card32 v1Neighbor = getOther(tempEdge,v1);
			if(v1Neighbor != v2)
				v1Neighbors.insert(v1Neighbor);
		}


		for(it=v2Edges.begin(); it != v2Edges.end(); ++it){
			Vector2i tempEdge = edges->get2i(*it,edgeAAT);

			card32 v2Neighbor = getOther(tempEdge,v2);
			
			if(v2Neighbor != v1)
				v2Neighbors.insert(v2Neighbor);
		}
		
		set_intersection(v1Neighbors.begin(),v1Neighbors.end(), v2Neighbors.begin(), v2Neighbors.end(), back_inserter(int_set));
		set_difference(v2Neighbors.begin(),v2Neighbors.end(), v1Neighbors.begin(), v1Neighbors.end(), back_inserter(diff_set));
		
		for(unsigned j=0;j<int_set.size();j++){
			int_set_count.push_back((*counter)[int_set[j]]);
		}
		
		for(unsigned j=0;j<diff_set.size();j++){
			diff_set_count.push_back((*counter)[diff_set[j]]);
		}

		AAT positionAAT = geometry->getAAT("position");
		
		Vector3f tempPoint,vertex1 = geometry->get3f(v1,positionAAT);
		float32 dist=0;
		//card32 other;
		set<card32> others, others2;
		
		
		
		for(unsigned j=0;j<diff_set.size();j++){
			tempPoint = geometry->get3f(diff_set[j],positionAAT);
			
			dist = norm(tempPoint-vertex1);
			//if there is at least one neighbor, creating a bad edge, this contraction does not work
			
			if(dist>radius)
				return false;
		}
		
		//check if each of the neighbors 
		card32 othersAlive = 0;
		if(true){
			for(unsigned j=0;j<int_set.size();j++){
				if( ((*counter)[int_set[j]] <= 1) || (((*counter)[int_set[j]] <= (unsigned)minNeighbors) && (*to_keep)[int_set[j]]))
					return false;
			}
			
			for(unsigned j=0;j<diff_set.size();j++){
				if((*to_keep)[diff_set[j]])
					othersAlive++;
			}
							
			if((*counter)[v1]+othersAlive-1 < (unsigned)minNeighbors)
				return false;
		}
		
		(*counter)[v1]--;

		//it's ok
		
		for(unsigned j=0;j<int_set.size();j++){
			(*counter)[int_set[j]] = (*counter)[int_set[j]]-1;
		}
		
		for(unsigned j=0;j<diff_set.size();j++){
			Vector2i newEdge;
			card32 newNeighbor = diff_set[j];
			
			newEdge[0] = v1;
			newEdge[1] = newNeighbor;
			
			card32 tempHeight = edges->getHeight();
			
			edges->changeHeight(tempHeight+1);
			card32 newIndex = edges->getHeight()-1;
			
			edges->set2i(newIndex,edgeAAT,newEdge);
			edgeType newEdgePair;
				
			if(useQueue && (*to_keep)[newNeighbor]){
				Vector3f newNeighborVertex = geometry->get3f(newNeighbor,positionAAT);
				newEdgePair.first = norm(vertex1-newNeighborVertex);
				newEdgePair.second = newIndex;
				
				edgeIndices->push(newEdgePair);
			}
				
			(*edgeTable)[v1].insert(newIndex);	
			(*edgeTable)[newNeighbor].insert(newIndex);
				
			if((*to_keep)[newNeighbor])
				(*counter)[v1] = (*counter)[v1] + 1;
		}
		
		if((*counter)[v1] < (unsigned)minNeighbors)
			std::cout << "should't happen" << endl;
		
		//(*edgeTable)[v1].erase(edgeIndex);
		
		//(*edgeTable)[v2].erase(edgeIndex);
		
		return true;
	}
	
	void PCCNodesTools::copyConnections(card32 oldPoint, card32 newNeighbor, std::vector< set<card32> > *edgeTable , PointSet *edges){
	
		AAT edgeAAT;
		
		if(edges){
			edgeAAT = edges->getAAT("vindex");		
		}	

		edges->changeHeight(edges->getHeight()+1);
		card32 newIndex = edges->getHeight()-1;

		//make a connection between newNeighbor and oldPoint
		Vector2i tempEdge;

		tempEdge[0] = newNeighbor;
		tempEdge[1] = oldPoint;

		edges->set2i(newIndex, edgeAAT, tempEdge);

		(*edgeTable)[newNeighbor].insert(newIndex);

		(*edgeTable)[oldPoint].insert(newIndex);		

		//copy all connections between oldPoint and others to newNeighbor
		for(set<card32>::iterator it=(*edgeTable)[oldPoint].begin(); it != (*edgeTable)[oldPoint].end(); it++){
			tempEdge = edges->get2i(*it, edgeAAT);
			card32 other = getOther(tempEdge, oldPoint);
			
			tempEdge[0] = newNeighbor;
			tempEdge[1] = other;
			
			edges->changeHeight(edges->getHeight()+1);
			newIndex = edges->getHeight()-1;

			edges->set2i(newIndex, edgeAAT, tempEdge);

			(*edgeTable)[newNeighbor].insert(newIndex);

			(*edgeTable)[other].insert(newIndex);				
		}
	}
	
	UnstructuredInCorePointCloud * PCCNodesTools::getToKeep(UnstructuredInCorePointCloud *geometry, std::vector<bool> to_keep, float32 offset, std::map<int32,int32> *keepers , std::vector< set<card32> > *edgeTable , PointSet *edges){	
	
		PointSet *ps = geometry->getPointSet();	
		
	
		
		AAT positionAAT;
		AAT normalAAT;
		bool isNormal = true;
				
		if(ps->providesAttribute("position")){
			positionAAT = ps->getAAT("position");
		}
		else{
			error("No position channel in geometry");
			return NULL;
		}
		
		if(ps->providesAttribute("normal")){
			normalAAT = ps->getAAT("normal");
		}
		else{
			isNormal = false;
		}
		
		int32 nNodes = ps->getNumEntries();
		
		UnstructuredInCorePointCloud *uc = NULL;	
		
		if(uc == NULL){
			uc = createMMNodesPointCloud();
		}
		
		VertexDescriptor vd;
		vd.pushAttrib(mVAD("position",3,VAD::DATA_FORMAT_FLOAT32));	
		vd.pushAttrib(mVAD("baseIndex", 1, VAD::DATA_FORMAT_INT32));
		vd.pushAttrib(mVAD("radius",1,VAD::DATA_FORMAT_FLOAT32));	
		vd.pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));

		uc->clearAndSetup(&vd,0);
		
		PointSet* ps2 = uc->getPointSet();

		AAT positionAAT2 = ps2->getAAT("position");			
		AAT colorAAT2 = ps2->getAAT("color");	
		AAT radiusAAT2 = ps2->getAAT("radius");	
		AAT baseIndexAAT2 = ps2->getAAT("baseIndex");	
	
		Vector3f col3f;
		
		for(int32 j=0;j<nNodes;j++){
			if(to_keep[j]){
				Vector3f position = ps->get3f(j,positionAAT);
				Vector3f normal = ps->get3f(j,normalAAT);
				
				Vector3f tempPosition;
				
				ps2->changeHeight(ps2->getHeight()+1);
				
				col3f = makeVector3f( 255 , 10 , 10)  / 255.0f;
				
				int32 baseIndex = ps2->getHeight()-1;
				
				if(keepers != NULL){
					(*keepers)[j] = baseIndex;
				}
				
			/*	position[0] += rnd01()*0.1f;
				position[0] += rnd01()*0.1f;
				position[0] += rnd01()*0.1f;*/
				
				ps2->set3f(ps2->getHeight()-1,positionAAT2,position);
				ps2->set3f(ps2->getHeight()-1,colorAAT2,col3f);
				ps2->set1i(baseIndex,baseIndexAAT2,baseIndex);
				
				if((norm(normal)>0) && (offset != 0) && (isNormal)){
					normal = normal*1/norm(normal);
					col3f = makeVector3f( 255, 255 , 255  )  / 255.0f;
					
					tempPosition = position + normal*offset;
					
					ps2->changeHeight(ps2->getHeight()+1);
					ps2->set3f(ps2->getHeight()-1 ,positionAAT2,tempPosition);
					ps2->set3f(ps2->getHeight()-1 ,colorAAT2,col3f);
					ps2->set1i(ps2->getHeight()-1 ,baseIndexAAT2,baseIndex);
					
					if(edges != NULL){				
						unsigned newNeighbor = (unsigned)(*edgeTable).size();
						(*keepers)[newNeighbor] = ps2->getHeight()-1;
						
						set<card32> newConnections ;
						(*edgeTable).push_back(newConnections);

						copyConnections(j,newNeighbor,edgeTable,edges);
					}
					
					tempPosition = position - normal*offset;
					
					ps2->changeHeight(ps2->getHeight()+1);
					ps2->set3f(ps2->getHeight()-1 ,positionAAT2,tempPosition);
					ps2->set3f(ps2->getHeight()-1 ,colorAAT2,col3f);
					ps2->set1i(ps2->getHeight()-1 ,baseIndexAAT2,baseIndex);

					if(edges != NULL){				
						unsigned newNeighbor = (unsigned)(*edgeTable).size();
						(*keepers)[newNeighbor] = ps2->getHeight()-1;
						
						set<card32> newConnections ;
						(*edgeTable).push_back(newConnections);

						copyConnections(j,newNeighbor,edgeTable,edges);
					}
				}
			}
		}

		
		return uc;
	}
	
	void PCCNodesTools::setTopology(UnstructuredInCorePointCloud* uc, std::vector< set<card32> > *edgeTable, PointSet *edges, std::map<int32,int32> *keepers ){
			InCorePCTopologyGraph *top = new InCorePCTopologyGraph();
			VertexDescriptor vd;
			vd.pushAttrib(VAD("vindex", 2, VAD::DATA_FORMAT_INT32));
			top->clearAndSetup(&vd);
			
			if(!(edges->providesAttribute("vindex"))){
				error("Cannot access the toplogy channel in Edges pointset");
				return ;
			}
			AAT edgeAAT = edges->getAAT("vindex");

			for(unsigned i=0; i<(*edgeTable).size(); ++i){
				//is this point alive
				if((*keepers).find(i) != (*keepers).end()){
					for(set<card32>::iterator it = (*edgeTable)[i].begin(); it!= (*edgeTable)[i].end(); it++){
						Vector2i tempEdge = edges->get2i(*it,edgeAAT);

						unsigned other = getOther(tempEdge,i);

						//are both points still alive?
						if(((*keepers).find(other) != (*keepers).end()))
							top->addEdge((*keepers)[tempEdge[0]],(*keepers)[tempEdge[1]]);
					}
				}
			}

			
			AttachedDataContainer *attachments = uc->getAttachments();

			TopologyAttachment *topAtt = NULL;
			topAtt = new TopologyAttachment();
			topAtt->setup("TopologyAtt", AttachedData::ADF_PERSISTENT);
			attachments->attachData(topAtt);

			topAtt->setTopology(top);	

	}

	
	UnstructuredInCorePointCloud* PCCNodesTools::edgeContractionSampleMMNodesPointCloud(UnstructuredInCorePointCloud *geometry, float32 offset, float32 radius, bool useQueue){

		PointSet *ps = geometry->getPointSet();		
		
		AttachedDataContainer *attachments = geometry->getAttachments();
		
		std::cout << "Starting Edge Contraction" << endl;
		std::cout << "radius = " << radius << "\t offset = " << offset << "\t minNeighbors = " << minNeighbors << endl;
		
		TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
		
		AAT positionAAT;
		
		if (topAtt == NULL) {
			error("Need a topology attachement to the pointCloud");
			return NULL;
		}
				
		if(ps->providesAttribute("position")){
			positionAAT = ps->getAAT("position");
		}
		else{
			error("No position channel in geometry");
			return NULL;
		}

		InCorePCTopologyGraph*	topGraph  = topAtt->getTopology();

		PointSet *t = topGraph->getEdges();
		PointSet *edges = dynamic_cast<PointSet*>(t->copy());
		
		if(!(edges->providesAttribute("vindex"))){
			error("No vindex attribute for edge pointset");
			return NULL;
		}
				
		int32 nNodes = ps->getNumEntries();
		std::vector<bool> to_keep, candidates;
		to_keep.resize(nNodes,true);
		candidates.resize(nNodes,false);
		
		std::vector<card32> numNeighbors;
		
		numNeighbors.resize(nNodes,0);
		
		//do a toplogical search of the geometry
		
		std::vector< set<card32> > *edgeTable = new std::vector<set<card32>>(ps->getHeight()) ;
		
		AAT vertexAAT = edges->getAAT("vindex");
		
		std::vector< pair<float32,card32> > * edgeLengths = new std::vector< pair<float32,card32> >(edges->getHeight());

		priority_queue< edgeType, std::vector<edgeType>, Prioritize> *edgeIndices = new priority_queue< edgeType, std::vector<edgeType>, Prioritize>();
		
		std::vector<card32> counter((ps->getHeight()));
	
		//std::vector<int32> verticesLengths(ps->getHeight(),0);
		
		for(unsigned j=0;j<edges->getHeight();j++){
			edgeType edge;
			Vector2i edgePair = edges->get2i(j,vertexAAT);
			
			Vector3f v1 = ps->get3f(edgePair[0],positionAAT);
			Vector3f v2 = ps->get3f(edgePair[1],positionAAT);
			
			(*edgeTable)[edgePair[0]].insert(j);
			(*edgeTable)[edgePair[1]].insert(j);
			
			float32 dist = norm(v1-v2);
			
			//verticesLengths[edgePair[0]] = max(verticesLengths[edgePair[0]],dist);
			//verticesLengths[edgePair[1]] = max(verticesLengths[edgePair[1]],dist);
			
			edge.first = dist;
			edge.second = j;
		
			if(useQueue)	
				edgeIndices->push(edge);
			
			else
				(*edgeLengths)[j] = edge;
		}
		
		for(unsigned j=0;j<(*edgeTable).size(); j++){
			counter[j] = (card32)(*edgeTable)[j].size();
		
				//std::cout << "crap" << endl;
		}
		
		bool check = true;
		
		std::vector<pair<float32,card32> >::iterator edgeIterator;
		
		if(useQueue){
			check = (edgeIndices->size()>0);
		}
		else{
			std::sort(edgeLengths->begin(), edgeLengths->end(), MyDataSortPredicate);
			
			edgeIterator = edgeLengths->begin();
			check = (edgeIterator != edgeLengths->end());
		}
		
		unsigned numDeleted = 0;
		
		if(type == TOP_SEARCH2){
			radius = 0;
		}
		
		while(check){
			int32 index;
			
			edgeType edge;
			
			if(useQueue){
				edge = edgeIndices->top();
				edgeIndices->pop();
				//go through each of the edges in order and try to delete it
				index = edge.second;
				check = (edgeIndices->size()>0);
			}
			else{
				edge = *edgeIterator;
				index = edge.second;
				edgeIterator++;
				check = (edgeIterator != edgeLengths->end());
			}
			
			Vector2i edgePair = edges->get2i(index,vertexAAT);
			
			
			int32 vertexIndex1, vertexIndex2;
			
			if(rand() % 2){
				vertexIndex1 = edgePair[0];
				vertexIndex2 = edgePair[1];
			}
			else{
				vertexIndex1 = edgePair[1];
				vertexIndex2 = edgePair[0];		
			}
			
			
			if(useQueue){
				if(norm(ps->get3f(vertexIndex1,positionAAT)-ps->get3f(vertexIndex2,positionAAT)) != edge.first)
					continue;
			}
			
			//int32 toDelete;
			bool v1t = to_keep[vertexIndex1], v2t = to_keep[vertexIndex2];
			
			//verticesLengths[vertexIndex1]>verticesLengths[vertexIndex2] && 
			if(to_keep[vertexIndex1] && to_keep[vertexIndex2] && (vertexIndex1 != vertexIndex2)){
				//delete Vertex2 and merge its connections to Vertex1
				
				
						
				if(vertexMerge(vertexIndex1,vertexIndex2,index,edges,edgeTable,radius,ps,edgeIndices,&to_keep,&counter,useQueue)){
					numDeleted++;

				/*	for(set<card32>::iterator tit = (*edgeTable)[vertexIndex2].begin(); tit != (*edgeTable)[vertexIndex2].end(); tit++){
						card32 other = getOther(edges->get2i(*tit, vertexAAT), vertexIndex2);
						card32 numV = 0;
						set<card32> otherNeighbors;
						for(set<card32>::iterator tit2 = (*edgeTable)[other].begin(); tit2 != (*edgeTable)[other].end(); tit2++){
							card32 third = getOther(edges->get2i(*tit2, vertexAAT),other);	
							if(to_keep[third]){
								otherNeighbors.insert(third);
								numV++;
							}
						}
						if(numV != counter[other]){
							card32 i1 = counter[other], i2 = numV;
							std::cout << "HERE" << endl;
						}
					}*/
					
					to_keep[vertexIndex2] = false;					
				
					/*for(set<card32>::iterator tit = (*edgeTable)[vertexIndex2].begin(); tit != (*edgeTable)[vertexIndex2].end(); tit++){
						card32 other = getOther(edges->get2i(*tit, vertexAAT), vertexIndex2);
						card32 numV = 0;
						set<card32> otherNeighbors;
						for(set<card32>::iterator tit2 = (*edgeTable)[other].begin(); tit2 != (*edgeTable)[other].end(); tit2++){
							card32 third = getOther(edges->get2i(*tit2, vertexAAT),other);	
							if(to_keep[third]){
								otherNeighbors.insert(third);
								numV++;
							}
						}
						if(numV != counter[other]){
							card32 i1 = counter[other], i2 = numV;
							std::cout << "HERE" << endl;
						}
					}
					
					for(set<card32>::iterator tit = (*edgeTable)[vertexIndex1].begin(); tit != (*edgeTable)[vertexIndex1].end(); tit++){
						card32 other = getOther(edges->get2i(*tit, vertexAAT), vertexIndex1);
						card32 numV = 0;
						for(set<card32>::iterator tit2 = (*edgeTable)[other].begin(); tit2 != (*edgeTable)[other].end(); tit2++){
							card32 third = getOther(edges->get2i(*tit2, vertexAAT),other);	
							if(to_keep[third])
								numV++;
						}
						if(numV != counter[other]){
							std::cout << "HERE" << endl;
						}
					}*/
					
				}
				//delete Vertex1 and merge its connections to Vertex2
				else if(vertexMerge(vertexIndex2,vertexIndex1,index,edges,edgeTable,radius,ps,edgeIndices,&to_keep,&counter,useQueue)){
					numDeleted++;
					to_keep[vertexIndex1] = false;
					
					/*for(set<card32>::iterator tit = (*edgeTable)[vertexIndex1].begin(); tit != (*edgeTable)[vertexIndex1].end(); tit++){
						card32 other = getOther(edges->get2i(*tit, vertexAAT), vertexIndex1);
						card32 numV = 0;
						for(set<card32>::iterator tit2 = (*edgeTable)[other].begin(); tit2 != (*edgeTable)[other].end(); tit2++){
							card32 third = getOther(edges->get2i(*tit2, vertexAAT),other);	
							if(to_keep[third])
								numV++;
						}
						if(numV != counter[other]){
							std::cout << "HERE" << endl;
						}
					}
					
					for(set<card32>::iterator tit = (*edgeTable)[vertexIndex2].begin(); tit != (*edgeTable)[vertexIndex2].end(); tit++){
						card32 other = getOther(edges->get2i(*tit, vertexAAT), vertexIndex2);
						card32 numV = 0;
						set<card32> otherNeighbors;
						for(set<card32>::iterator tit2 = (*edgeTable)[other].begin(); tit2 != (*edgeTable)[other].end(); tit2++){
							card32 third = getOther(edges->get2i(*tit2, vertexAAT),other);	
							if(to_keep[third]){
								otherNeighbors.insert(third);
								numV++;
							}
						}
						if(numV != counter[other]){
							card32 i1 = counter[other], i2 = numV;
							std::cout << "HERE" << endl;
						}
					}*/
					
				}
				else{
					continue;		
				}
			}
			if(type == TOP_SEARCH2){
				check = (check && (ps->getHeight()-numDeleted > (unsigned)numNodes));
			}
		}
		
		std::map<int32, int32> *keepers = new std::map<int32,int32>();
		
		UnstructuredInCorePointCloud *uc = getToKeep(geometry,to_keep,offset,keepers);
	
		
		setTopology(uc,edgeTable,edges,keepers);
		
		card32 maxNeighbors = 0, minNeighbors = 1000000;			

		for(card32 j=0;j<counter.size();j++){
			if(to_keep[j]){
				minNeighbors = min(counter[j],minNeighbors);
				maxNeighbors = max(counter[j],maxNeighbors);
			}
		}

		std::cout << "Max neighbors = " << maxNeighbors << endl;
		std::cout << "Min neighbors = " << minNeighbors << endl;
		
		std::cout << "Total nodes: " << uc->getPointSet()->getHeight() << endl;
		

		delete edgeLengths;
		
		delete edgeIndices;

		delete keepers;
		delete edgeTable;
		delete edges;
		return uc;
		
		std::cout << "Done" << endl;
	}
	
	
	// get the topological sampling. Assumes that a InCorePCTopologyGraph exists as an attachment to the geometry
	UnstructuredInCorePointCloud* PCCNodesTools::topSampleMMNodesPointCloud(UnstructuredInCorePointCloud *geometry, float32 offset, float32 radius) 	{
		PointSet *ps = geometry->getPointSet();		
		
		AttachedDataContainer *attachments = geometry->getAttachments();
		
		TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
		
		AAT positionAAT;
		AAT normalAAT;
		
		if (topAtt == NULL) {
			error("Need a topology attachement to the pointCloud");
			return NULL;
		}
				
		if(ps->providesAttribute("position")){
			positionAAT = ps->getAAT("position");
		}
		else{
			error("No position channel in geometry");
			return NULL;
		}
		if(ps->providesAttribute("normal")){
			normalAAT = ps->getAAT("normal");
		}
		else{
			error("No normal channel in geometry");
			return NULL;
		}
		
		int32 nNodes = ps->getNumEntries();
		std::vector<bool> to_keep, candidates;
		to_keep.resize(nNodes,true);
		candidates.resize(nNodes,false);
		
		std::vector< std::vector<card32> > neighbors;
		std::vector<card32> numNeighbors;
		
		neighbors.resize(nNodes);
		numNeighbors.resize(nNodes,0);
		
		//do a toplogical search of the geometry
		
		InCorePCTopologyGraph*	topGraph  = topAtt->getTopology();
		TopNeighborhoodSphereSearch* topSearch = new TopNeighborhoodSphereSearch();	
			
		for(int j=0;j<nNodes;j++){
			Vector3f position = ps->get3f(j,positionAAT);
			
			topSearch->setup(topGraph,ps,j,position,radius,positionAAT);
			
			while((!topSearch->atEnd()) && ((int32)numNeighbors[j]<nNodes/100)){
				card32 k = topSearch->getCurrentIndex();
				if(k != j){
					candidates[j] = true;
					
					neighbors[j].push_back(k);
					numNeighbors[j] = numNeighbors[j]+1;
					
					candidates[k] = true;
					
					neighbors[k].push_back(j);
					numNeighbors[k] = numNeighbors[k]+1;					
				}
				topSearch->next();
			}
			
			//delete topSearch;
		}
		delete topSearch;
		

		if(type == TOP_SEARCH2){
			for(int j=0;j<nNodes;j++){
				if(candidates[j] && to_keep[j]){
						for(unsigned k=0;k<neighbors[j].size();k++){
							int32 m = neighbors[j][k];
							if(m>j && to_keep[m])
								to_keep[m] = false;
						}
				}
			}
		}

		else if(type == TOP_SEARCH){
 			for(int j=0;j<nNodes;j++){
				if(candidates[j]){
					bool check = true;
					if(numNeighbors[j]>1){
						// for each neighbor check whether it has more than one remaining neighbors
						for(unsigned k=0;k<neighbors[j].size();k++){
							if(numNeighbors[neighbors[j][k]] <= 1){
								check = false;
								break;
							}
						}
						
						//everything is fine, can delete
						if(check){
							to_keep[j] = false;
							for(unsigned k=0;k<neighbors[j].size();k++){
								numNeighbors[neighbors[j][k]] = numNeighbors[neighbors[j][k]]-1;
							}
						}
					}
				}
			}
		}
		
		UnstructuredInCorePointCloud *uc = getToKeep(geometry,to_keep,offset);
		
		return uc;
		
	}