//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIAnimationReconstruction2.h"
//---------------------------------------------------------------------------
#if defined(_DEBUG) && defined(DO_VLD_LEAK_DETECTION)
   #include "externalIncludes/vld/vld.h"
#endif
#include "CopyObjectProperties.h"
#include "ARSettings.h"
#include "BinaryObjectStreams.h"
#include "FileDialogs.h"
#include "ARMergeSequences.h"
#include "SeparatorClassProperty.h"
#include "SGListNode.h"
#include "SGEPointCloud.h"
#include "SurfelGraphInit2.h"
#include "StringHelper.h"
#include "ARMergeSettings.h"
#include "AREvaluateDeformation.h"
#include "ARPairwiseMatching.h"
#include "ARDeformationOptimizationContext.h"
#include "DeformationEvaluator.h"
#include "DeformationSolver.h"
#include "AROptToolWithFeedback.h"
#include "DeformationTools.h"
#include "SGObjectNode.h"
#include "ARSequence.h"
#include "DeformationTools.h"
#include "ARPipelineOptions.h"
#include "Timer.h"
#include "MMDeformationSolver.h"
#include "ARSequenceEvaluator.h"
#include "ARUrshapeOptimization.h"
#include "ProgressWindow.h"
#include "PCCDataResampling.h"
#include "SGESurfelGraph.h"
#include "ARGlobalOptimization.h"
#include "SGDeformationNodeAR.h"
#include "BinaryObjectStreams.h"
#include "ARSetUrshapeDataWeight.h"
#include "SurfelGraphTriangulator.h"
#include "GLShaderMaterial.h"
#include "ARComputeGlobalResidual.h"
#include "SmallScaleReconstruction.h"
#include "MathTools.h"
#include "ARGlobalOptSettings.h"
#include "Resampling.h"
#include "ARShapeOptimizationContext.h"
#include "ICPConstraints.h"
#include "ARTransfConstraintsToUrshape.h"
#include "ShapeReconstruction.h"
#include "SurfelOptEvaluator.h"
#include "SurfelOptimizer.h"
#include "time.h"
#include "ARChangeUrshape.h"
#include "DataToUrshape.h"
#include "DebugRenderer.h"
#include "MMConstraintsOOC.h"
#include "StringHelper.h"
#include "GeometricTools.h"
#include "SuperIterator_templCode.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

static card32 statistics_initTopology_time = 0;
static card32 statistics_initShapeOpt_time = 0;

static void debugOutputTimingsPCIAR2() {
   debugOutput << "initTopology_time:\t" << statistics_initTopology_time << "\n";
   debugOutput << "initShapeOpt_time:\t" << statistics_initShapeOpt_time << "\n";
}
				 
IMPLEMENT_X4_CLASS( PCIAnimationReconstruction2 , 0 ) {
	BEGIN_CLASS_INIT( PCIAnimationReconstruction2 );
	ADD_SEPARATOR("parameters")
	ADD_OBJECT_PROP(mergeSettings, 0, ARMergeSettings::getClass(), true)
	ADD_SEPARATOR("processing range")
	ADD_CARD32_PROP(startFrame, 0)
	ADD_CARD32_PROP(endFrame, 0)
	ADD_SEPARATOR("setPresets (click on method to set)")
	ADD_CARD32_PROP_UPDATE_DESCR(presets, 0, NULL, ARDefaultSettings::descrPresets, ARDefaultSettings::numPresets, NULL)

	ADD_NOARGS_METHOD(clean)
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(setPresets)
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(mergeRange)
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(globalOpt)
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(computeGlobalResidual);
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(testRigidities);
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(convertResidualsToRigidities)
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(adaptDeformationToRigidities)
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(loadSettings)
	SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(saveSettings)
	SET_METHOD_AS_CMD_OBJ( false )
   ADD_NOARGS_METHOD(outputResult)
   SET_METHOD_AS_CMD_OBJ( false )
   ADD_NOARGS_METHOD(upsampleUrshape)
   SET_METHOD_AS_CMD_OBJ( false )
   ADD_NOARGS_METHOD(backend)
   SET_METHOD_AS_CMD_OBJ( false )
   ADD_NOARGS_METHOD(outputDefNodes)
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(upsampleAndGlobalOpt)
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(dataToUrshape)
   SET_METHOD_AS_CMD_OBJ( false )
	ADD_NOARGS_METHOD(test);
   SET_METHOD_VISIBLE( false )
	ADD_NOARGS_METHOD(computeFrameResiduals);
   SET_METHOD_AS_CMD_OBJ( false )
}

void PCIAnimationReconstruction2::test() {
   static int count = 0;
   if (count == 0) {
      debugRenderer->beginRenderJob(std::string("topCheck"));
   } else {
   }
   static int i = 0;
   debugRenderer->addFastEllipse(NULL_VECTOR3F+makeVector3f(i++,0,0), IDENTITY3F, makeVector3f(0.3f,0.6f,0.3f), makeVector3f(0.5f,1,0.5f), false);
   debugRenderer->endRenderJob();   
}


// ----

PCIAnimationReconstruction2::PCIAnimationReconstruction2() {
	startFrame = 0;
	endFrame = 1;
	presets = ARDefaultSettings::DRESDEN;
	mergeSettings = new ARMergeSettings();
	currentToolToDraw = NULL;
	mergeAborted = false;

	setPresets();
}

void PCIAnimationReconstruction2::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

PCIAnimationReconstruction2::~PCIAnimationReconstruction2() {
	delete mergeSettings;
}



void PCIAnimationReconstruction2::glDrawTool(GLContext *glContext) {
	if (currentToolToDraw) {
		currentToolToDraw->glDrawFeedback(glContext, mergeSettings->visualizationSettings);
	}
}

bool PCIAnimationReconstruction2::drawGLSceneRendering() {
   return currentToolToDraw == NULL;
}

card32 PCIAnimationReconstruction2::splitRange(card32 startFrame, card32 endFrame) {
//    if (mergeSettings->pipelineOptions->mergeInPowersOfTwo) {
//       if (startFrame == endFrame) {
//          x4Assert(false);
//          return startFrame;
//       }
//       int32 dist = endFrame - startFrame;
//       if (dist == 1) return startFrame;
//       int32 largestP2 = 1 << (simpleLog2(dist-1));
//       if (startFrame + largestP2 >= endFrame) throw X4Exception("does not work");
//       return startFrame + largestP2;
//    } else {
//   return (startFrame + endFrame)/2;
//    }

	if( mergeSettings->pipelineOptions->mergeInPowersOfTwo )
	{
		int32 diff = endFrame - startFrame;
		if( diff == 1)
			return startFrame;

		int32 biggestPowerOfTwo = 1;
		while(biggestPowerOfTwo <= diff)
			biggestPowerOfTwo *= 2;
		biggestPowerOfTwo /= 2;

		//debugOutput << "split( " << startFrame << "," << endFrame << ") = " << 
		//	startFrame + biggestPowerOfTwo - 1<< "\n";

		return startFrame + biggestPowerOfTwo - 1;
	}
	else
	{
		return (startFrame + endFrame)/2;
	}
}


void PCIAnimationReconstruction2::recursiveMerge(
   card32 startFrame, card32 endFrame, 
   PointCloud *&urshapePropagation)
{

	//_CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
   if (mergeAborted) return;
   if (progressWindow->getIsAborted()) return;
   if (endFrame < startFrame) return;
   if (startFrame == endFrame) {
      initUrshape(startFrame);
      if (progressWindow->getIsAborted()) {
         message("\n*** mergeRange() aborted ***\n\n");
         mergeAborted = true;
         return;
      }
   }
   if (endFrame > startFrame) {
      PointCloud *leftHalfPUrshape = NULL;
      card32 lastOfFirst = splitRange(startFrame, endFrame);
      
      if (!urshapeExists(startFrame, lastOfFirst)) {
         recursiveMerge(startFrame, lastOfFirst, urshapePropagation);
         card32 numFrames = lastOfFirst - startFrame + 1;
      }
      if (progressWindow->getIsAborted()) {
         if (!mergeAborted) {
            message("\n*** mergeRange() aborted ***\n\n");
            mergeAborted = true;
         }
         return;
      }
      if (!urshapeExists(lastOfFirst+1, endFrame)) {
         recursiveMerge(lastOfFirst+1, endFrame, urshapePropagation);
         card32 numFrames = endFrame - lastOfFirst;
      }
      if (progressWindow->getIsAborted()) {
         if (!mergeAborted) {
            message("\n*** mergeRange() aborted ***\n\n");
            mergeAborted = true;
         }
         return;
      }
      
      if (endFrame - startFrame + 1 <= mergeSettings->pipelineOptions->maximumMergeRange) {
         PointCloud *urshapePropagationOutput = NULL;
         if (endFrame - startFrame == 1) {
            // in this case: use propagated us and get a new one for next frame (next in time)
            mergeSequences(
               startFrame, lastOfFirst, 
               lastOfFirst+1, endFrame,
               urshapePropagation,
               urshapePropagationOutput);
            /// not needed anylonger...
            delete urshapePropagation;
            urshapePropagation = urshapePropagationOutput;
         } else {
            // we don't care need a propagated input urshape; it has already been merged into the sequence
            PointCloud *urshapePropagationOutput = NULL;
            mergeSequences(
               startFrame, lastOfFirst, 
               lastOfFirst+1, endFrame,
               NULL,
               urshapePropagationOutput);
            delete urshapePropagation;
            urshapePropagation = urshapePropagationOutput;
         }
      }
      if (progressWindow->getIsAborted()) {
         if (!mergeAborted) {
            message("\n*** mergeRange() aborted ***\n\n");
            mergeAborted = true;
            return;
         }
      }
   }
}

void PCIAnimationReconstruction2::mergeRange() 
{
   message("\n----- PCIAnimationReconstruction2::mergeRange() - started\n");
   Timer timer;
	progressWindow->pushStep(false, "mergeRange");   
   #if defined(_DEBUG) && defined(DO_VLD_LEAK_DETECTION)
	   VLDEnable();
   #endif
   PointCloud *propagatedUrshape = NULL;
   recursiveMerge(startFrame, endFrame, propagatedUrshape);
   if (propagatedUrshape && !mergeSettings->pipelineOptions->deletePartialResults) {
      addPointCloud(scene, propagatedUrshape, "root/propagatedUrshape");
   }
   delete propagatedUrshape;

   #if defined(_DEBUG) && defined(DO_VLD_LEAK_DETECTION)
      VLDDisable();
   #endif
   mergeAborted = false;
   progressWindow->popSteps();
   redrawScene();
   message("----- PCIAnimationReconstruction2::mergeRange() - operation completed.\n");
   debugOutput << "----- overall time: " << timer.getDeltaValue() << "ms.\n\n";
   debugOutput.flush();
   debugOutputTimingsPCIAR2();
   ARMergeSequences_DebugOutputTimings();
}


std::string PCIAnimationReconstruction2::makeUrshapeName(card32 startIndex, card32 endIndex) {
   if (startIndex != endIndex) {
      return std::string("urshape_frame_")+intToStr(startIndex)+std::string("_to_")+intToStr(endIndex);
   } else {
      return std::string("urshape_frame_")+intToStr(startIndex);
   }
}


std::string PCIAnimationReconstruction2::makeDeformationName(card32 startIndex, card32 endIndex) {
   if (startIndex != endIndex) {
      return std::string("deformations_frame_")+intToStr(startIndex)+std::string("_to_")+intToStr(endIndex);
   } else {
      return std::string("deformations_frame_")+intToStr(startIndex);
   }
}

bool PCIAnimationReconstruction2::urshapeExists(card32 startIndex, card32 endIndex) {
	SGListNode *urshapeNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->urshapesPath));
	if (!urshapeNode) {
		throw X4Exception("PCIAnimationReconstruction2::urshapeExists() - urshapesPath does not point to a list node (initialization aborted).");
	}
	std::string urshapesPathPlusSlash = endWithSlash(mergeSettings->urshapesPath);
	return urshapeNode->getChildIndex(makeUrshapeName(startIndex, endIndex)) != -1;
}


void PCIAnimationReconstruction2::initUrshape(card32 frame) {
   if (mergeAborted) return;
   debugOutput << "initializing urshape " << frame << "\n";
   debugOutput.flush();
   Timer timer;
	SGListNode *dataNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->urshapesDataPath));
	if (!dataNode) {
		throw X4Exception("PCIAnimationReconstruction2::initUrshape() - urshapesDataPath does not point to a list node (initialization aborted).");
	}
	std::string urshapesDataPathPlusSlash = endWithSlash(mergeSettings->urshapesDataPath);
	
	SGListNode *constraintsDataNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->constraintsDataPath));
	if (!dataNode && mergeSettings->pipelineOptions->mode == ARPipelineOptions::mmExtractSurfacesFirst) {
		throw X4Exception("PCIAnimationReconstruction2::initUrshape() - constraintsDataPath (used for initial shape opt) does not point to a list node (initialization aborted).");
	}
	std::string constraintsDataPathPlusSlash = endWithSlash(mergeSettings->constraintsDataPath);

	SGListNode *urshapeNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->urshapesPath));
	if (!urshapeNode) {
		throw X4Exception("PCIAnimationReconstruction2::initUrshape() - urshapesPath does not point to a list node (initialization aborted).");
	}
	std::string urshapesPathPlusSlash = endWithSlash(mergeSettings->urshapesPath);
   
	SGListNode *deformationNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->deformationsPath));
	if (!deformationNode) {
		throw X4Exception("PCIAnimationReconstruction2::initUrshape() - deformationsPath does not point to a list node (initialization aborted).");
	}
	std::string deformationsPathPlusSlash = endWithSlash(mergeSettings->deformationsPath);

   /// create initial urshape from data
   std::string urshapeName = makeUrshapeName(frame, frame);
   if (urshapeNode->getChildIndex(urshapeName) == -1) {
	   SGEPointCloud sgeSource;
	   if (dataNode->getNumChildNodes(NULL) <= frame) {
	      warning("PCIAnimationReconstruction2::initUrshape() - data out of range - no new urshape created.");
	      return;
	   }
	   sgeSource.setup(getScene(), urshapesDataPathPlusSlash+dataNode->getChildNode(NULL, frame)->getName());
	   std::string msg;
	   if (!sgeSource.check(msg)) {
	      warning(std::string("PCIAnimationReconstruction2::initUrshape() - data frame ")+intToStr(frame)+std::string(" not found; msg: ")+msg);
   	   return;
	   }

	   SGESurfelGraph sgeDest;
	   sgeDest.setup(getScene(), urshapesPathPlusSlash + urshapeName);

	   mergeSettings->initUrshapes->initialize(&sgeDest, &sgeSource, mergeSettings->optimizationSettings);
	
      card32 initTopologyTime = timer.getDeltaValue();
	   debugOutput << "finished init [surfel graph setup] (" << initTopologyTime << " ms)\n";
      debugOutput.flush();
      statistics_initTopology_time += initTopologyTime;

	   if (mergeSettings->pipelineOptions->mode == ARPipelineOptions::mmExtractSurfacesFirst) {

	      SGEPointCloud sgeConstraintsSource;
	      if (constraintsDataNode->getNumChildNodes(NULL) <= frame) {
	         warning("PCIAnimationReconstruction2::initUrshape() - constraints data (used for initial shape opt) out of range - no new urshape created.");
	         return;
	      }
	      sgeConstraintsSource.setup(getScene(), constraintsDataPathPlusSlash+constraintsDataNode->getChildNode(NULL, frame)->getName());
	      std::string msg;
	      if (!sgeSource.check(msg)) {
	         warning(std::string("PCIAnimationReconstruction2::initUrshape() - constraints data frame ")+intToStr(frame)+std::string(" not found; msg: ")+msg);
   	      return;
	      }

	      ARUrshapeOptimization opt;
	      PointCloud *pc = sgeDest.getPointCloud();
	      PointCloud *constrPC = sgeConstraintsSource.getPointCloud();
	      PointCloud *filtered = opt.optimizeSingleFrame(
	         pc,
	         mergeSettings->shapeSolver, 
	         constrPC, 
	         mergeSettings->optimizationSettings,
            Delegate(this, (Delegate::MethodPtr)&PCIAnimationReconstruction2::redrawScene)
	      );
	      pc->assign(filtered);
	      delete filtered;
	      card32 usOptTime = timer.getDeltaValue();
		   debugOutput << "finished urshape optimization (" << usOptTime << " ms)\n";
         debugOutput.flush();
		   statistics_initShapeOpt_time = usOptTime;
	   }
   }
   
   std::string deformationName = makeDeformationName(frame, frame);
   if (deformationNode->getChildIndex(deformationName) == -1) {
      ARDeformationOptimizationContext *context = (ARDeformationOptimizationContext*)mergeSettings->deformationSolver->createOptContext();
      SGObjectNode *urshapeSGNode = dynamic_cast<SGObjectNode*>(urshapeNode->getChildNode(NULL, urshapeNode->getChildIndex(urshapeName)));
      if (!urshapeSGNode || !(urshapeSGNode->getSceneObject())) {
         throw EInvalidState("PCIAnimationReconstruction2::initUrshape() - urshape not found (this should not happen).");
      }
	  
      // TODO: CHECK IF THIS IS FINE FOR THE OTHER SOLVERS?
      mergeSettings->deformationSolver->setup(context, mergeSettings->optimizationSettings);
	  mergeSettings->deformationSolver->setIdentityMap();
	  context->initializeFromShape(urshapeSGNode->getSceneObject(), makeVector2f(0,0), mergeSettings->optimizationSettings);
      
      SGDeformationNodeAR *deformationContextNode = new SGDeformationNodeAR();
      deformationContextNode->setDeformation(context);
      deformationContextNode->setName(deformationName);
      deformationNode->addChildNode(deformationContextNode);
   }
   debugOutput << "created id deformation ... all done (" << timer.getDeltaValue() << " ms)\n";
   debugOutput.flush();
}


static std::string getUniqueFileName(std::string path, std::string name) {
   path = endWithBackSlash(path);
   static int number = 0;
   std::string fn = path+name+intToStr(number);
   while (FileExists((fn+std::string(".scene.object")).c_str())) {
      number++;
      fn = path+name+intToStr(number);
   }
   return fn+std::string(".scene.object");
}


void PCIAnimationReconstruction2::mergeSequences(
   card32 seqAFirstFrame, card32 seqALastFrame, 
   card32 seqBFirstFrame, card32 seqBLastFrame,
   PointCloud *urshapePropagationInput, PointCloud *&urshapePropagationOutput)
{
   urshapePropagationOutput = NULL;

   debugOutput << "merging sequences [" << seqAFirstFrame << ".." << seqALastFrame << "] and [" << seqBFirstFrame << ".." << seqBLastFrame << "].\n";
   debugOutput.flush();
   Timer timer;
   x4Assert(seqAFirstFrame <= seqALastFrame);
   x4Assert(seqBFirstFrame <= seqBLastFrame);
   std::string urshapeNameA = makeUrshapeName(seqAFirstFrame, seqALastFrame);
   std::string urshapeNameB = makeUrshapeName(seqBFirstFrame, seqBLastFrame);
   std::string deformationNameA = makeDeformationName(seqAFirstFrame, seqALastFrame);
   std::string deformationNameB = makeDeformationName(seqBFirstFrame, seqBLastFrame);
   
   if (getSceneGraphNodeByName(getScene(), endWithSlash(mergeSettings->urshapesPath)+urshapeNameA) == NULL) {
      warning(std::string("PCIAnimationReconstruction2::mergeSequences() - urshape named ")+urshapeNameA
             +std::string(" has not been found. The merge is not performed."));
      return;
   }
   if (getSceneGraphNodeByName(getScene(), endWithSlash(mergeSettings->urshapesPath)+urshapeNameB) == NULL) {
      warning(std::string("PCIAnimationReconstruction2::mergeSequences() - urshape named ")+urshapeNameB
             +std::string(" has not been found. The merge is not performed."));
      return;
   }
   if (getSceneGraphNodeByName(getScene(), endWithSlash(mergeSettings->deformationsPath)+deformationNameA) == NULL) {
      warning(std::string("PCIAnimationReconstruction2::mergeSequences() - deformation sequence named ")+deformationNameA
             +std::string(" has not been found. The merge is not performed."));
      return;
   }
   if (getSceneGraphNodeByName(getScene(), endWithSlash(mergeSettings->deformationsPath)+deformationNameB) == NULL) {
      warning(std::string("PCIAnimationReconstruction2::mergeSequences() - deformation sequence named ")+deformationNameB
             +std::string(" has not been found. The merge is not performed."));
      return;
   }
   
   ARSequence seqA;
   seqA.setupFromSceneGraph(
      getScene(), 
      endWithSlash(mergeSettings->urshapesPath) + urshapeNameA,
      endWithSlash(mergeSettings->deformationsPath) + deformationNameA,
      mergeSettings->shapeSolver,
      mergeSettings->deformationSolver
   );
   ARSequence seqB;
   seqB.setupFromSceneGraph(
      getScene(), 
      endWithSlash(mergeSettings->urshapesPath) + urshapeNameB,
      endWithSlash(mergeSettings->deformationsPath) + deformationNameB,
      mergeSettings->shapeSolver,
      mergeSettings->deformationSolver
   );
   
   std::vector<PointCloud*> dataPointClouds;
   SGListNode *dataPointsList = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->constraintsDataPath));
   if (!dataPointsList) {
      throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - constraintsDataPath does not point to a list node.");
   }
   for (card32 i=seqAFirstFrame; i<=seqALastFrame; i++) {
      x4Assert(i < dataPointsList->getNumChildNodes(NULL));
      SGObjectNode *sg = dynamic_cast<SGObjectNode*>(dataPointsList->getChildNode(NULL, i));
      if (!sg) {
         throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - one of the data nodes is not an SGObjectNode.");
      }
      PointCloud *pcData = dynamic_cast<PointCloud*>(sg->getSceneObject());
      if (!pcData) {
         throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - no point cloud found in data node.");
      }
      dataPointClouds.push_back(pcData);
   }
   for (card32 i=seqBFirstFrame; i<=seqBLastFrame; i++) {
      x4Assert(i < dataPointsList->getNumChildNodes(NULL));
      SGObjectNode *sg = dynamic_cast<SGObjectNode*>(dataPointsList->getChildNode(NULL, i));
      if (!sg) {
         throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - one of the data nodes is not an SGObjectNode.");
      }
      PointCloud *pcData = dynamic_cast<PointCloud*>(sg->getSceneObject());
      if (!pcData) {
         throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - no point cloud found in data node.");
      }
      dataPointClouds.push_back(pcData);
   }
   
   /// this block of statement just calls the merger safely...
   ARMergeSequences merger;
   currentToolToDraw = &merger;
   ARSequence *result = NULL;
   try {
      result = merger.mergeSequences(
			scene,
         &seqA, &seqB, 
         seqAFirstFrame, seqBFirstFrame,
         dataPointClouds,
         urshapePropagationInput, urshapePropagationOutput,
         mergeSettings, 
         Delegate(this, (Delegate::MethodPtr)&PCIAnimationReconstruction2::redrawScene)
      );
   } catch (...) {
      currentToolToDraw = NULL;
      throw;
   }
   currentToolToDraw = NULL;
   if (result == NULL) return; // can only happen if debugging options are set
   
   /// now it's time to put the results back into the scene graph...
   PointCloud *resultPC = result->getUrshape();
   
   std::string newUrshapeName = makeUrshapeName(seqAFirstFrame, seqBLastFrame);
   std::string newDeformationName = makeDeformationName(seqAFirstFrame, seqBLastFrame);
   
   SGObjectNode *newSG = new SGObjectNode();
   newSG->clearAndSetup(resultPC);
   newSG->setName(newUrshapeName);
   SGListNode *urshapesContainer = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->urshapesPath));
   if (!urshapesContainer) {
      throw X4Exception("unexpectedly, something went wrong.");
   }
   urshapesContainer->addChildNode(newSG);
   
   ARDeformationOptimizationContext *resultDeform = result->getDeformation();

   SGDeformationNodeAR *deformationContextNode = new SGDeformationNodeAR();
   deformationContextNode->setDeformation(resultDeform);
   deformationContextNode->setName(newDeformationName);
	SGListNode *deformationNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->deformationsPath));
	if (!deformationNode) {
		throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - deformationsPath does not point to a list node (aborted).");
	}
   deformationNode->addChildNode(deformationContextNode);
   
   delete result; // just the container (ARSequence)...
   // -- this changed (mw 01/12/07) delete resultDeform; // and the deformation as well, which is not needed anylonger (the scene graph contains a copy)
   
   if (mergeSettings->pipelineOptions->deletePartialResults 
   || min(seqALastFrame-seqAFirstFrame, seqBLastFrame-seqBFirstFrame) < mergeSettings->pipelineOptions->minKeepPartialResultsSize) {
      deleteSceneGraphNode(getScene(), endWithSlash(mergeSettings->urshapesPath) + urshapeNameA);
      deleteSceneGraphNode(getScene(), endWithSlash(mergeSettings->deformationsPath) + deformationNameA);
      deleteSceneGraphNode(getScene(), endWithSlash(mergeSettings->urshapesPath) + urshapeNameB);
      deleteSceneGraphNode(getScene(), endWithSlash(mergeSettings->deformationsPath) + deformationNameB);
   }
   if (mergeSettings->pipelineOptions->savePartialResults
   || min(seqALastFrame-seqAFirstFrame, seqBLastFrame-seqBFirstFrame) > mergeSettings->pipelineOptions->minKeepPartialResultsSize) {
      try {
         BinaryOutputObjectStream out(getUniqueFileName(mergeSettings->pipelineOptions->savePartialResultsPath, "AR2PartialResult").c_str());
         out.writeObject(getScene());
      } catch (...) {
         debugOutput << "PCIAnimationReconstrution2 - was not able to save partial result.\n";
      }
   }

   // -- this changed (mw 01/12/07) // not used anymore and not deleted automatically
   // delete seqA.getDeformation();
   // delete seqB.getDeformation();

   debugOutput << "finished merge sequences (" << timer.getDeltaValue() << " ms)\n";
   debugOutput.flush();
}


void PCIAnimationReconstruction2::loadSettings() 
{
	std::string filename = FileDialogs::getOpenFileName(sceneEditorWidget, "load reconstruction settings", "*.object");
	if (!filename.empty()) {
		ARMergeSettings *s = NULL;
		BinaryInputObjectStream in(filename.c_str());
		in.readAnyObject(s);
		delete mergeSettings;
		mergeSettings = s;
	}	
}

void PCIAnimationReconstruction2::saveSettings() 
{
	std::string filename = FileDialogs::getSaveFileName(sceneEditorWidget, "save reconstruction settings to file", "*.object");
	if (!filename.empty()) {
		BinaryOutputObjectStream out(filename.c_str());
		out.writeObject(mergeSettings);
	}	
}

void PCIAnimationReconstruction2::keyDown(GeneralKey key) {}
void PCIAnimationReconstruction2::keyUp(GeneralKey key) {}
void PCIAnimationReconstruction2::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {}
void PCIAnimationReconstruction2::mouseMoved(int32 x, int32 y) {}
void PCIAnimationReconstruction2::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {}
void PCIAnimationReconstruction2::areaResize(card32 width, card32 height) {}


void PCIAnimationReconstruction2::setPresets() 
{
	mergeSettings->setPresets(presets);
}

void PCIAnimationReconstruction2::clean() 
{
   #if defined(_DEBUG) && defined(DO_VLD_LEAK_DETECTION)
	   VLDEnable();
   #endif
   SGListNode *node = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(scene, mergeSettings->urshapesPath));
   if (node) {
      node->clear();
   } else {
      warning("urshapesPath not found.");
   }
   node = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(scene, mergeSettings->deformationsPath));
   if (node) {
      node->clear();
   } else {
      warning("deformationsPath not found.");
   }
   node = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(scene, mergeSettings->resultsPath));
   if (node) {
      node->clear();
   } else {
      warning("resultsPath not found.");
   }
   redrawScene();
   #if defined(_DEBUG) && defined(DO_VLD_LEAK_DETECTION)
	   VLDDisable();
   #endif
}

//void PCIAnimationReconstruction2::mmSolver() 
//{
//   delete mergeSettings->deformationSolver;
//   mergeSettings->deformationSolver = new MMDeformationSolver();
//}

void PCIAnimationReconstruction2::outputResult() 
{
   debugOutput << "----- creating final output [frames " << startFrame << ".." << endFrame << "].\n";
   debugOutput.flush();
   Timer timer;
   x4Assert(startFrame <= endFrame);
   std::string urshapeName = makeUrshapeName(startFrame, endFrame);
   std::string deformationName = makeDeformationName(startFrame, endFrame);
   ARSequence seq;
   seq.setupFromSceneGraph(
      getScene(), 
      endWithSlash(mergeSettings->urshapesPath) + urshapeName,
      endWithSlash(mergeSettings->deformationsPath) + deformationName,
      mergeSettings->shapeSolver,
      mergeSettings->deformationSolver
   );
   SGListNode *targetNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->resultsPath));
   if (!targetNode) {
      error("PCIAnimationReconstruction2::outputResult() - resultPath is not pointing to an SGListNode.");
      return;
   }
   DeformationEvaluator *evaluator = mergeSettings->deformationSolver->createEvaluator();
   mergeSettings->finalOutput->evalSeq(
      &seq, 
      targetNode, 
      evaluator,
      mergeSettings->optimizationSettings);
   delete evaluator;
   debugOutput << "----- final output finished.\n---- time: " << timer.getDeltaValue() << "ms.\n\n";
   debugOutput.flush();
   redrawScene();
}


void PCIAnimationReconstruction2::globalOpt() 
{
   MMConstraintsOOC::useConstraintsOOC=true;
   std::string urshapeName = makeUrshapeName(startFrame, endFrame);
   std::string deformationName = makeDeformationName(startFrame, endFrame);

   debugOutput << "----- globalOpt() [frames " << startFrame << ".." << endFrame << "].\n";
   debugOutput.flush();
   Timer timer;
   card32 initialValue = timer.getCurrentValue();
   
   ARSequence seq;
   seq.setupFromSceneGraph(
      getScene(), 
      endWithSlash(mergeSettings->urshapesPath) + urshapeName,
      endWithSlash(mergeSettings->deformationsPath) + deformationName,
      mergeSettings->shapeSolver,
      mergeSettings->deformationSolver
   );

   ARGlobalOptimization opt;
   opt.performGlobalOpt(
      &seq, 
      mergeSettings->deformationSolver, 
      mergeSettings->shapeSolver, 
      getScene(), 
      mergeSettings->globalOptDataPath, 
      mergeSettings->optimizationSettings,
      startFrame,
      endFrame,
      Delegate(this, (Delegate::MethodPtr)&PCIAnimationReconstruction2::redrawScene),
      mergeSettings->globalOptSettings->optimizeUrshapeDeformation,
      mergeSettings->globalOptSettings->optimizeShape,
      mergeSettings->globalOptSettings->optimizeDeformation,
      mergeSettings->globalOptSettings->removeUnsupportedPoints
   );

   debugOutput << "----- globalOpt() finished.\n---- time: " << timer.getCurrentValue()-initialValue << "ms.\n\n";
   debugOutput.flush();

   std::string defNodePath = endWithSlash(mergeSettings->deformationsPath) + deformationName;
   SGDeformationNodeAR *deformationNode = 
      dynamic_cast<SGDeformationNodeAR*>(getSceneGraphNodeByName(scene, defNodePath));
   if (!deformationNode) {
      throw X4Exception("ARSequence::setupFromSceneGraph() - deformation node not found.");
   }
   // this is ugly but necessary - the deformation might have changed and assign is not possible for deformation objects
   deformationNode->removeDeformation();
   deformationNode->setDeformation(seq.getDeformation());

   
   MMConstraintsOOC::useConstraintsOOC=false;

   redrawScene();
}

//void PCIAnimationReconstruction2::globalOptWindowed() 
//{
//   warning("not implemented yet.");
//}

void PCIAnimationReconstruction2::computeDataWeight() 
{
   std::string urshapeName = makeUrshapeName(startFrame, endFrame);
   std::string deformationName = makeDeformationName(startFrame, endFrame);

   ARSequence seq;
   seq.setupFromSceneGraph(
      getScene(), 
      endWithSlash(mergeSettings->urshapesPath) + urshapeName,
      endWithSlash(mergeSettings->deformationsPath) + deformationName,
      mergeSettings->shapeSolver,
      mergeSettings->deformationSolver
   );
   std::vector<PointCloud*> dataPointClouds;
   SGListNode *dataPointsList = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->constraintsDataPath));
   if (!dataPointsList) {
      throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - constraintsDataPath does not point to a list node.");
   }
   for (card32 i=startFrame; i<=endFrame; i++) {
      if (i >= dataPointsList->getNumChildNodes(NULL)) {
         error("frames out of range");
         return;
      }
      SGObjectNode *sg = dynamic_cast<SGObjectNode*>(dataPointsList->getChildNode(NULL, i));
      if (!sg) {
         throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - one of the data nodes is not an SGObjectNode.");
      }
      PointCloud *pcData = dynamic_cast<PointCloud*>(sg->getSceneObject());
      if (!pcData) {
         throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - no point cloud found in data node.");
      }
      dataPointClouds.push_back(pcData);
   }

   DeformationEvaluator *eval = mergeSettings->deformationSolver->createEvaluator();
   ARSetUrshapeDataWeight::accumDataWeight(
      &seq, 
      dataPointClouds,
      eval,
      mergeSettings->optimizationSettings);
   delete eval;
}

void PCIAnimationReconstruction2::backendOptimizeShape( 
	UnstructuredInCorePointCloud* pc, 
	ARSettings* settings, 
	ARSequence* seq,
	DeformationEvaluator* evaluator, 
	std::vector<PointCloud*>& dataPoints )
{
	std::string msg;
	if ( pc == NULL ) return;
	if ( settings == NULL ) return;
	SurfelOptimizer* optimizer = new SurfelOptimizer();
	SurfelOptimizationContext* context = dynamic_cast<SurfelOptimizationContext*>( optimizer->createOptContext() );
	context->initializeFromShape( pc, settings );
	optimizer->setup( context, settings );
	optimizer->beginConstraints();

	debugOutput << "  Using every " << mergeSettings->backendSettings->dataSubsampling << "th data frame.\n"; debugOutput.flush();

	progressWindow->pushStep( true, "Shape Optimization" );
	for ( unsigned int time = startFrame; time <= endFrame; time++ ){
		if ( time%(mergeSettings->backendSettings->dataSubsampling) == 0 ){
			PointSet* deformedDataPS = SmallScaleReconstruction::deformDataPoints( 
				dataPoints[time], 
				time, 
				dynamic_cast<UnstructuredInCorePointCloud*>( seq->getUrshape() )->getPointSet(),
				evaluator, 
				settings->topSurfelSpacing );
			UnstructuredInCorePointCloud* deformedDataPC = new UnstructuredInCorePointCloud();
			deformedDataPC->setPointSet( deformedDataPS );
			ICPConstraints ICPConstraints;
			ICPConstraints.setConstraintMode(settings->icpConstraintsMode);
			ICPConstraints.setOutliersDistanceFactor(settings->icpNearFieldRelativeCutOff);
			ICPConstraints.setDistanceOutliersMode(settings->icpNearFieldCulling?ICP_OUTLIERS_DISTANCE:ICP_OUTLIERS_NONE);
			ICPConstraints.setCullNormalOutliers(false);	
			ICPConstraints.setOutlierNormals(-1.0f);
			ICPConstraints.buildICPConstraints( settings, optimizer, pc, pc, deformedDataPC, 0 );
			delete deformedDataPC;
		}
		progressWindow->progress( float32( time - startFrame ) * 100 / float32( endFrame - startFrame ) );
	}
	progressWindow->popSteps();
	
	optimizer->solve();

	ShapeEvaluator *shapeEval = optimizer->createEvaluator();
	shapeEval->setup( context, settings );
	PointCloud *newSurfels = shapeEval->evaluate();

	InCorePCTopologyGraph *tg = getTopology( pc );
	InCorePCTopologyGraph *copyTG = NULL;
	if (!tg) {
		warning("urshape without topology");
	} else {
		copyTG = tg->copy();
	}
	pc->assign( newSurfels );
	if (copyTG) {
		attachTopology( pc, copyTG );
	}

	delete newSurfels;
	delete shapeEval;
	delete context;
	delete optimizer;
}

void PCIAnimationReconstruction2::upsampleUrshape()
{
   MMConstraintsOOC::useConstraintsOOC=true;
	
	std::string urshapeName = makeUrshapeName(startFrame, endFrame);
	std::string deformationName = makeDeformationName(startFrame, endFrame);
	ARSequence seq;
	seq.setupFromSceneGraph(
		getScene(), 
		endWithSlash( mergeSettings->urshapesPath ) + urshapeName,
		endWithSlash( mergeSettings->deformationsPath ) + deformationName,
		mergeSettings->shapeSolver,
		mergeSettings->deformationSolver
		);
	PointCloud* oldUrshape = seq.getUrshape();
	UnstructuredInCorePointCloud* newUrshape = dynamic_cast<UnstructuredInCorePointCloud*>( oldUrshape->copy() );

	float32 newSamplingSpacing = mergeSettings->optimizationSettings->topSurfelSpacing;

	// Upsampling
	const float32 startTime = clock();
	debugOutput << "Upsampling\n"; debugOutput.flush();
	Resampling::upsample2( newUrshape->getPointSet(), getTopology( newUrshape ), newSamplingSpacing );
	newSamplingSpacing /= 2.0f;
	Resampling::downsample( newUrshape->getPointSet(), getTopology( newUrshape ), newSamplingSpacing );
	debugOutput << "  Upsampled from " << card32( oldUrshape->getNumPoints() ) << " surfels to " << card32( newUrshape->getNumPoints() ) << " surfels.\n"; debugOutput.flush();
	debugOutput << "Time required: " << float32( ( clock() - startTime ) / CLK_TCK ) << ".\n"; debugOutput.flush();

	// Replace urshape
	DeformationSolver * deformOpt = mergeSettings->deformationSolver;
	deformOpt->setup( seq.getDeformation(), mergeSettings->optimizationSettings );
	ARDeformationOptimizationContext * newDeformContext = 
		ARChangeUrshape::recomputeDeformation( newUrshape, oldUrshape, seq.getDeformation(), mergeSettings->deformationSolver, mergeSettings->optimizationSettings);
	oldUrshape->assign( newUrshape );
	delete newUrshape;
	// Tell scene about newDeformContext
	SGDeformationNodeAR *defNode = 
		dynamic_cast<SGDeformationNodeAR*>(getSceneGraphNodeByName(scene, endWithSlash(mergeSettings->deformationsPath) + deformationName));
	x4Assert(defNode);
	defNode->setDeformation(newDeformContext);

	// Update settings
	mergeSettings->optimizationSettings->topSurfelSpacing = newSamplingSpacing;
	mergeSettings->optimizationSettings->topResamplingSurfelSpacing = newSamplingSpacing;

	// free mem for old constraints
	deformOpt->beginConstraints();

	MMConstraintsOOC::useConstraintsOOC=false;
}

void PCIAnimationReconstruction2::backend()
{
	x4Assert( startFrame <= endFrame );
	std::string urshapeName = makeUrshapeName(startFrame, endFrame);
	std::string deformationName = makeDeformationName(startFrame, endFrame);
	ARSequence seq;
	seq.setupFromSceneGraph(
		getScene(), 
		endWithSlash( mergeSettings->urshapesPath ) + urshapeName,
		endWithSlash( mergeSettings->deformationsPath ) + deformationName,
		mergeSettings->shapeSolver,
		mergeSettings->deformationSolver
		);

	// Read data points
	std::vector<PointCloud*> dataPoints;
	SGListNode *dataPointsList = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->constraintsDataPath));
	if (!dataPointsList) {
		throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - constraintsDataPath does not point to a list node.");
	}
	for (card32 i=startFrame; i<=endFrame; i++) {
		if (i >= dataPointsList->getNumChildNodes(NULL)) {
			error("frames out of range");
			return;
		}
		SGObjectNode *sg = dynamic_cast<SGObjectNode*>(dataPointsList->getChildNode(NULL, i));
		if (!sg) {
			throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - one of the data nodes is not an SGObjectNode.");
		}
		PointCloud *pcData = dynamic_cast<PointCloud*>(sg->getSceneObject());
		if (!pcData) {
			throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - no point cloud found in data node.");
		}
		dataPoints.push_back(pcData);
	}
	DeformationEvaluator *evaluator = mergeSettings->deformationSolver->createEvaluator();
	evaluator->setup( seq.getDeformation(), mergeSettings->optimizationSettings );

	SceneGraphNode* nodeCand = getSceneGraphNodeByName( scene, "root/backend" );
	SGListNode* backEndListNode = dynamic_cast<SGListNode*>( nodeCand );
	if ( backEndListNode == NULL ){
		backEndListNode = new SGListNode();
		backEndListNode->setName( "backend" );
		scene->getRootNode()->addChildNode( backEndListNode );
	}

	// Init point set to operate on from urshape
	deleteSceneGraphNode( scene, "root/backend/original" );
	PointCloud* urShape = seq.getUrshape();
	UnstructuredInCorePointCloud* originalPC = dynamic_cast<UnstructuredInCorePointCloud*>( urShape->copy() );
	float32 samplingSpacing = mergeSettings->optimizationSettings->topSurfelSpacing;
	SGObjectNode* originalNode = new SGObjectNode();
	originalNode->setName( "original" );
	originalNode->clearAndSetup( originalPC );
	originalNode->setVisible( false );
	backEndListNode->addChildNode( originalNode );

	//// DEPRECATED: done via resampleUrshape()
	/*
	// Upsampling
	if ( mergeSettings->backendSettings->doUpsample ){
		for ( unsigned int n = 0; n < mergeSettings->backendSettings->numUpsamplingIterations; n++ ){
			const float32 startTime = clock();
			debugOutput << "Backend: Upsampling\n"; debugOutput.flush();
			Resampling::upsample2( originalPC->getPointSet(), getTopology( originalPC ), samplingSpacing );
			samplingSpacing /= 2.0f;
			Resampling::downsample( originalPC->getPointSet(), getTopology( originalPC ), samplingSpacing );
			debugOutput << "Time required: " << float32( ( clock() - startTime ) / CLK_TCK ) << ".\n"; debugOutput.flush();
		}
	}
	*/

	/// DEPRECATED: done via globalOpt()
	/*
	// Additional run of surface reconstruction routine
	if ( mergeSettings->backendSettings->doNewSurfaceReconstruction ){
		for ( unsigned int n = 0; n < mergeSettings->backendSettings->numSurfRecIterations; n++  ){
			float32 startTime = clock();
			debugOutput << "Backend: Surface reconstruction\n"; debugOutput.flush();
			ARSettings* set = dynamic_cast<ARSettings*>( mergeSettings->optimizationSettings->copy() );
			set->topSurfelSpacing = set->topResamplingSurfelSpacing = samplingSpacing;
			recomputeNormalsAndFrames( originalPC, set );
			backendOptimizeShape( originalPC, set, &seq, evaluator, dataPoints );
			delete set;
			debugOutput << "Time required: " << float32( ( clock() - startTime ) / CLK_TCK ) << ".\n"; debugOutput.flush();
		}
	}
	*/
	
	// Reconstruct small scale feature via PCA
	if ( mergeSettings->backendSettings->doPCAReconstruction ){
		deleteSceneGraphNode( scene, "root/backend/pca_rec_result" );
		float32 startTime = clock();
		debugOutput << "Backend: PCA reconstruction\n"; debugOutput.flush();
		SmallScaleReconstruction scr;
		scr.setBackendSettings( mergeSettings->backendSettings );
		scr.reconstruct( 
			originalPC,
			startFrame,
			endFrame,
			&seq, 
			dataPoints, 
			evaluator, 
			samplingSpacing,
			backEndListNode );
		debugOutput << "Time required: " << float32( ( clock() - startTime ) / CLK_TCK ) << ".\n"; debugOutput.flush();
	}

	// Triangulate
	float32 startTime = clock();
	debugOutput << "Backend: Triangulation\n"; debugOutput.flush();
	SurfelGraphTriangulator triangulator;
	triangulator.setBackendSettings( mergeSettings->backendSettings );
	SGSimpleAnimationNode* meshAnimNode = triangulator.triangulate( 
		startFrame, 
		endFrame,
		originalPC, 
		samplingSpacing, 
		&seq, 
		evaluator, scene  );
	if ( meshAnimNode != NULL ){
		deleteSceneGraphNode( scene, "root/backend/meshes" );
		backEndListNode->addChildNode( meshAnimNode );
	}
	debugOutput << "Time required: " << float32( ( clock() - startTime ) / CLK_TCK ) << ".\n"; debugOutput.flush();

	delete evaluator;

	// Rebuild render objects
	for ( unsigned int n = 0; n < scene->getNumGLMaterials(); n++ ){
		GLShaderMaterial* mat = dynamic_cast<GLShaderMaterial*>( scene->getGLMaterial( n ) );
		if ( mat != NULL ){
			mat->increaseVersion();
		}
	}

	redrawScene();
}

void X4::PCIAnimationReconstruction2::computeGlobalResidual()
{
	std::string urshapeName = makeUrshapeName(startFrame, endFrame);
	std::string deformationName = makeDeformationName(startFrame, endFrame);

	debugOutput << "----- computeGlobalResidual() [frames " << startFrame << ".." << endFrame << "].\n";
	debugOutput.flush();
	Timer timer;
	card32 initialValue = timer.getCurrentValue();

	ARSequence seq;
	seq.setupFromSceneGraph(
		getScene(), 
		endWithSlash(mergeSettings->urshapesPath) + urshapeName,
		endWithSlash(mergeSettings->deformationsPath) + deformationName,
		mergeSettings->shapeSolver,
		mergeSettings->deformationSolver
		);

	std::vector<PointCloud*> dataPointClouds;
	SGListNode *dataPointsList = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->constraintsDataPath));
	if (!dataPointsList) {
		throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - constraintsDataPath does not point to a list node.");
	}
	for (card32 i=startFrame; i<=endFrame; i++) {
		if (i >= dataPointsList->getNumChildNodes(NULL)) {
			error("frames out of range");
			return;
		}
		SGObjectNode *sg = dynamic_cast<SGObjectNode*>(dataPointsList->getChildNode(NULL, i));
		if (!sg) {
			throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - one of the data nodes is not an SGObjectNode.");
		}
		PointCloud *pcData = dynamic_cast<PointCloud*>(sg->getSceneObject());
		if (!pcData) {
			throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - no point cloud found in data node.");
		}
		dataPointClouds.push_back(pcData);
	}

	DeformationSolver * deformOpt = mergeSettings->deformationSolver;
	deformOpt->setup(seq.getDeformation(), mergeSettings->optimizationSettings);
	DeformationEvaluator *deformEval = deformOpt->createEvaluator();
	deformEval->setup(seq.getDeformation(), mergeSettings->optimizationSettings);

	ARComputeGlobalResidual::computeGlobalResidual(
		seq.getUrshape(),
		seq.getDeformation(),
		deformEval,
		dataPointClouds,
		mergeSettings->optimizationSettings	);

	debugOutput << convertTimeToString(timer.getDeltaValue()) << "\n";
}

void X4::PCIAnimationReconstruction2::convertResidualsToRigidities()
{
	MMConstraintsOOC::useConstraintsOOC=true;
	std::string urshapeName = makeUrshapeName(startFrame, endFrame);
	std::string deformationName = makeDeformationName(startFrame, endFrame);

	debugOutput << "----- convertResidualsToRigidities() [frames " << startFrame << ".." << endFrame << "].\n";
	debugOutput.flush();
	Timer timer;
	card32 initialValue = timer.getCurrentValue();

	ARSequence seq;
	seq.setupFromSceneGraph(
		getScene(), 
		endWithSlash(mergeSettings->urshapesPath) + urshapeName,
		endWithSlash(mergeSettings->deformationsPath) + deformationName,
		mergeSettings->shapeSolver,
		mergeSettings->deformationSolver
		);


	PointCloud * urshape = seq.getUrshape();
	PointCloud * newUrshape = ARComputeGlobalResidual::adaptRigidityToResiduals( urshape, 
		mergeSettings->optimizationSettings,
		mergeSettings->optimizationSettings->topSurfelSpacing * 3.0f
		);

	DeformationSolver * deformOpt = mergeSettings->deformationSolver;
	deformOpt->setup(seq.getDeformation(), mergeSettings->optimizationSettings);

	ARDeformationOptimizationContext * newDeformContext = 
		ARChangeUrshape::recomputeDeformation( newUrshape, urshape, seq.getDeformation(), mergeSettings->deformationSolver, mergeSettings->optimizationSettings);
	urshape->assign(newUrshape);
	delete newUrshape;

	// TODO: tell scene about newDeformContext
	SGDeformationNodeAR *defNode = 
	   dynamic_cast<SGDeformationNodeAR*>(getSceneGraphNodeByName(scene, endWithSlash(mergeSettings->deformationsPath) + deformationName));
	x4Assert(defNode);
	defNode->setDeformation(newDeformContext);


	// free mem for old constraints
	deformOpt->beginConstraints();

	MMConstraintsOOC::useConstraintsOOC=false;
	debugOutput << convertTimeToString(timer.getDeltaValue()) << "\n";
}

void X4::PCIAnimationReconstruction2::testRigidities()
{
	std::string urshapeName = makeUrshapeName(startFrame, endFrame);
	std::string deformationName = makeDeformationName(startFrame, endFrame);

	debugOutput << "----- globalOpt() [frames " << startFrame << ".." << endFrame << "].\n";
	debugOutput.flush();
	Timer timer;
	card32 initialValue = timer.getCurrentValue();

	ARSequence seq;
	seq.setupFromSceneGraph(
		getScene(), 
		endWithSlash(mergeSettings->urshapesPath) + urshapeName,
		endWithSlash(mergeSettings->deformationsPath) + deformationName,
		mergeSettings->shapeSolver,
		mergeSettings->deformationSolver
		);


	PointCloud * urshape = seq.getUrshape();
	PointCloud * newUrshape = ARComputeGlobalResidual::adaptRigidityToResiduals( urshape, 
		mergeSettings->optimizationSettings,
		mergeSettings->optimizationSettings->topSurfelSpacing * 3.0f
		);

	urshape->assign(newUrshape);
	delete newUrshape;

}

void X4::PCIAnimationReconstruction2::upsampleAndGlobalOpt()
{
	upsampleUrshape();
	globalOpt();
}

void X4::PCIAnimationReconstruction2::adaptDeformationToRigidities()
{
	MMConstraintsOOC::useConstraintsOOC=true;
	{
		std::string urshapeName = makeUrshapeName(startFrame, endFrame);
		std::string deformationName = makeDeformationName(startFrame, endFrame);

		debugOutput << "----- recompute deformation() [frames " << startFrame << ".." << endFrame << "].\n";
		debugOutput.flush();
		Timer timer;
		card32 initialValue = timer.getCurrentValue();

		ARSequence seq;
		seq.setupFromSceneGraph(
			getScene(), 
			endWithSlash(mergeSettings->urshapesPath) + urshapeName,
			endWithSlash(mergeSettings->deformationsPath) + deformationName,
			mergeSettings->shapeSolver,
			mergeSettings->deformationSolver
			);


		PointCloud * urshape = seq.getUrshape();

		DeformationSolver * deformOpt = mergeSettings->deformationSolver;
		deformOpt->setup(seq.getDeformation(), mergeSettings->optimizationSettings);

		PointCloud * newUrshape = (PointCloud*)urshape->copy();
		ARDeformationOptimizationContext * newDeformContext = 
			ARChangeUrshape::recomputeDeformation( newUrshape, urshape, seq.getDeformation(), mergeSettings->deformationSolver, mergeSettings->optimizationSettings);
		urshape->assign(newUrshape);
		delete newUrshape;

		// TODO: tell scene about newDeformContext
		SGDeformationNodeAR *defNode = 
			dynamic_cast<SGDeformationNodeAR*>(getSceneGraphNodeByName(scene, endWithSlash(mergeSettings->deformationsPath) + deformationName));
		x4Assert(defNode);
		defNode->setDeformation(newDeformContext);

		// free mem for old constraints
		deformOpt->beginConstraints();

		debugOutput << convertTimeToString(timer.getDeltaValue()) << "\n";
	}
	MMConstraintsOOC::useConstraintsOOC=false;
}

void X4::PCIAnimationReconstruction2::computeFrameResiduals()
{
	std::string urshapeName = makeUrshapeName(startFrame, endFrame);
	std::string deformationName = makeDeformationName(startFrame, endFrame);

	debugOutput << "----- computeFrameResiduals() [frames " << startFrame << ".." << endFrame << "].\n";
	debugOutput.flush();
	Timer timer;
	card32 initialValue = timer.getCurrentValue();

	ARSequence seq;
	seq.setupFromSceneGraph(
		getScene(), 
		endWithSlash(mergeSettings->urshapesPath) + urshapeName,
		endWithSlash(mergeSettings->deformationsPath) + deformationName,
		mergeSettings->shapeSolver,
		mergeSettings->deformationSolver
		);

	std::vector<PointCloud*> dataPointClouds;
	SGListNode *dataPointsList = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->constraintsDataPath));
	if (!dataPointsList) {
		throw X4Exception("PCIAnimationReconstruction2::computeFrameResiduals() - constraintsDataPath does not point to a list node.");
	}
	for (card32 i=startFrame; i<=endFrame; i++) {
		if (i >= dataPointsList->getNumChildNodes(NULL)) {
			error("frames out of range");
			return;
		}
		SGObjectNode *sg = dynamic_cast<SGObjectNode*>(dataPointsList->getChildNode(NULL, i));
		if (!sg) {
			throw X4Exception("PCIAnimationReconstruction2::computeFrameResiduals() - one of the data nodes is not an SGObjectNode.");
		}
		PointCloud *pcData = dynamic_cast<PointCloud*>(sg->getSceneObject());
		if (!pcData) {
			throw X4Exception("PCIAnimationReconstruction2::computeFrameResiduals() - no point cloud found in data node.");
		}
		dataPointClouds.push_back(pcData);
	}

	float medPointDistance = 0;
	for( unsigned i=0;i<dataPointClouds.size();i++ )
	{
		PointCloud * dataPC = dataPointClouds[i];
		
		std::string resultPCName = "root/results/result_";
		resultPCName += intToStr(i);
		UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(getPointCloud(getScene(),resultPCName));
		if( !upc )
		{
			warning(std::string("Couldn't find result point cloud! (") + resultPCName + ")\n");
			break;
		}

		if( i == 0 )
			medPointDistance = getMedianPointDistance(upc);

		checkAttribute(upc,"residual",1,VAD::DATA_FORMAT_FLOAT32);
		SceneObjectIterator * iter = upc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC );
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);

		AAT resposAAT = upc->getAAT("position");
		AAT resAAT = upc->getAAT("residual");
		HierarchicalKNNIterator  hIt(dataPC,32,NULL);
		float sigma = medPointDistance * 0.5f;

		while(!bIt->atEnd())
		{
			Vector3f pos = bIt->get3f(resposAAT);
			Matrix3f frame;
			Vector3f center;
			computeLocalFrame(pos,sigma,dataPC,&hIt,frame,center);

			Vector3f distVec = center - pos;
			distVec -= frame[2] * (frame[2]*distVec);
			float residual = norm(distVec);
			mIt->set1f(resAAT,residual);
			bIt->next();
		}

		delete iter;
		

	}

	debugOutput << convertTimeToString(timer.getDeltaValue()) << "\n";
}

void PCIAnimationReconstruction2::outputDefNodes() 
{
   std::string defPath = endWithSlash(mergeSettings->deformationsPath);
	std::string deformationName = makeDeformationName(startFrame, endFrame);

	SGDeformationNodeAR *defNode = 
	   dynamic_cast<SGDeformationNodeAR*>(getSceneGraphNodeByName(scene, endWithSlash(mergeSettings->deformationsPath) + deformationName));

   defNode->getDeformation()->writeNodesVisToSceneGraph(scene, "root", "nodes");
}

void PCIAnimationReconstruction2::dataToUrshape() 
{
   std::vector<PointCloud*> dataPointClouds;
   SGListNode *dataPointsList = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), mergeSettings->globalOptDataPath));
   if (!dataPointsList) {
      throw X4Exception("PCIAnimationReconstruction2::dataToUrshape() - constraintsDataPath does not point to a list node.");
   }
   for (card32 i=startFrame; i<=endFrame; i++) {
      x4Assert(i < dataPointsList->getNumChildNodes(NULL));
      SGObjectNode *sg = dynamic_cast<SGObjectNode*>(dataPointsList->getChildNode(NULL, i));
      if (!sg) {
         throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - one of the data nodes is not an SGObjectNode.");
      }
      PointCloud *pcData = dynamic_cast<PointCloud*>(sg->getSceneObject());
      if (!pcData) {
         throw X4Exception("PCIAnimationReconstruction2::mergeSequences() - no point cloud found in data node.");
      }
      dataPointClouds.push_back(pcData);
   }




   debugOutput << "transforming data to urshape domain [" << startFrame << ".." << endFrame << "].\n";
   debugOutput.flush();
   Timer timer;
   std::string urshapeName = makeUrshapeName(startFrame, endFrame);
   std::string deformationName = makeDeformationName(startFrame, endFrame);

   if (getSceneGraphNodeByName(getScene(), endWithSlash(mergeSettings->urshapesPath)+urshapeName) == NULL) {
      warning(std::string("PCIAnimationReconstruction2::dataToUrshape() - urshape named ")+urshapeName
         +std::string(" has not been found. The operation is not performed."));
      return;
   }
   if (getSceneGraphNodeByName(getScene(), endWithSlash(mergeSettings->deformationsPath)+deformationName) == NULL) {
      warning(std::string("PCIAnimationReconstruction2::dataToUrshape() - deformation sequence named ")+deformationName
         +std::string(" has not been found. The operation is not performed."));
      return;
   }

   ARSequence seq;
   seq.setupFromSceneGraph(
      getScene(), 
      endWithSlash(mergeSettings->urshapesPath) + urshapeName,
      endWithSlash(mergeSettings->deformationsPath) + deformationName,
      mergeSettings->shapeSolver,
      mergeSettings->deformationSolver
   );
   
   DeformationEvaluator *deformEval = mergeSettings->deformationSolver->createEvaluator();
   UnstructuredInCorePointCloud *result = DataToUrshape::transfD2U(&seq, dataPointClouds, deformEval, mergeSettings->optimizationSettings);
   delete deformEval;
   addPointCloud(scene, result, "dataToUrshape");
}
