//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIDeformationConstraintEditor.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ClassMethods.h"
#include "Handle3D.h"
#include "PointCloudTools.h"
#include "ConstraintsDialog.h"
#include "SGDeformationConstraintsList.h"
#include "RegularData.inline.h"
#include "SelectionTools.h"
#include "PCCDeformation.h"
#include "SceneEditorWidget.h"
#include "ARConstraints.h"
#include "SGObjectNode.h"
#include "RenderingTool.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( PCIDeformationConstraintEditor , 2 ) {
	BEGIN_CLASS_INIT( PCIDeformationConstraintEditor );
	ADD_STRING_PROP_METHOD(constraintsNode, 0, &getConstraintsNode, &setConstraintsNode)
	ADD_BOOLEAN_PROP(showFromNow, 0)
	ADD_BOOLEAN_PROP(showToNow, 0)
	ADD_BOOLEAN_PROP(showAll, 0)
	ADD_BOOLEAN_PROP(executeDeformOnline, 1)
	ADD_VECTOR3F_PROP(halfSpaceNormal, 2)
	ADD_VECTOR3F_PROP(halfSpaceFootPoint, 2);

	ADD_NOARGS_METHOD(addConstraint)
	ADD_NOARGS_METHOD(deleteConstraint)
	ADD_NOARGS_METHOD(createConstraintsNode)
	ADD_NOARGS_METHOD(addHalfSpaceConstraint)
	ADD_NOARGS_METHOD(addPlaneConstraints)
}

// ----

PCIDeformationConstraintEditor::PCIDeformationConstraintEditor() {
	constraintsNode = "root/constraints";
	hhandler.addChangesListener(this);
	showFromNow = true;
	showAll = true;
	showToNow = true;
   executeDeformOnline = false;
	usingHalfSpaceConstraints=false;
	halfSpaceNormal=makeVector3f(1,0,0);
	halfSpaceFootPoint=makeVector3f(1,0,0);
}

void PCIDeformationConstraintEditor::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

PCIDeformationConstraintEditor::~PCIDeformationConstraintEditor() {
	clearHandles();
}


// ----


PCIDeformationConstraintEditor::HandleData::HandleData(int32 startTime, int32 secondaryTime1, int32 secondaryTime2, int32 timeOrder) {
	this->startTime = startTime;
	this->secondaryTime1 = secondaryTime1;
	this->secondaryTime2 = secondaryTime2;
	this->timeOrder = timeOrder;
}

void PCIDeformationConstraintEditor::activateTool() 
{
	updateConstraintsNode();
}

void PCIDeformationConstraintEditor::deactivateTool() {}

void PCIDeformationConstraintEditor::glDrawTool(GLContext *glContext) 
{
	for (unsigned i=0; i<handles.size(); i++) {
		handles[i]->glDraw(glContext);
	}

	RenderingTool * rt = NULL;
	if( m_RenderingTools.find(glContext)==m_RenderingTools.end())
	{
		rt = new RenderingTool;
		rt->init();
		m_RenderingTools[glContext] = rt;
	}
	else
		rt = m_RenderingTools[glContext];


	if (usingHalfSpaceConstraints){
		rt->setRenderColor(makeVector4f(1.0,0.3f,0.3f,1));
 		Plane3f realPlane(halfSpaceNormal, halfSpaceFootPoint);
		rt->renderPlane(realPlane,2);
	}

	else
		rt->setRenderColor(makeVector4f(1.0,1,0.5f,0.5));



}

void PCIDeformationConstraintEditor::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
	if (!setParams()) return;
	if (modifiersState.getCtrl()) {
	   Vector3f pos;
	   std::string pcID;
      //bool hit = getClosestPointPositionFromAllPoints(
      //   x, y, viewportWidth, viewportHeight, 5, 
      //   &(getScene()->getRootState()->staticState->camera), 
      //   &(getScene()->getRootState()->staticState->viewFrustum),
      //   getScene(), pos, pcID);

      PointCloudIterator *it = getClosestPointIteratorFromAllPoints(
         x, y, viewportWidth, viewportHeight, 5, 
         &(getScene()->getRootState()->staticState->camera), 
         &(getScene()->getRootState()->staticState->viewFrustum),
         getScene(),
         pcID);
      if (it) {
         
         PointCloud *pc = getPointCloud(getScene(), pcID);
         if (pc == NULL) {
            x4Assert(false);
            delete it;
            return;
         }
         AAT posAAT = pc->getAAT("position");
         BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
         x4Assert(bit);
         pos = bit->get3f(posAAT);		 
         delete it;
         
         PositionARConstraint *constr = new PositionARConstraint();
         ARConstraintAnchor anchor;
         anchor.setPos(pos);
         anchor.setTopologicalAnchor(ARConstraintAnchor::NO_TOP_ANCHOR); // xxx add selected point
         constr->setup(anchor, pos, IDENTITY3F, getScene()->getRootState()->dynamicState->time);
			addConstraintObj(constr);
			hhandler.setHandles(&handles);
			hhandler.manuallySelectLatest(x,y);
      }
   } else {
      hhandler.setHandles(&handles);
      hhandler.mouseDown(x, y, buttonsState, modifiersState);
   }
}

void PCIDeformationConstraintEditor::mouseMoved(int32 x, int32 y) 
{
	if (!setParams()) return;
	hhandler.setHandles(&handles);
	hhandler.mouseMoved(x, y);
	if (executeDeformOnline && hhandler.active()) {
	   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
	   if (w) {
   	   w->executeCommandClass(PCCDeformation::getClass());
	   }
	}
}

void PCIDeformationConstraintEditor::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
	if (!setParams()) return;
	hhandler.setHandles(&handles);
	hhandler.mouseUp(x, y, buttonsState, modifiersState);
}

void PCIDeformationConstraintEditor::areaResize(card32 width, card32 height) 
{
   viewportWidth = width;
   viewportHeight = height;
	setParams();
	hhandler.areaResize(width, height);
}

void PCIDeformationConstraintEditor::keyDown(GeneralKey key) 
{
	if (!setParams()) return;
	hhandler.setHandles(&handles);
	hhandler.keyDown(key);
}

void PCIDeformationConstraintEditor::keyUp(GeneralKey key) 
{
	if (!setParams()) return;
	hhandler.setHandles(&handles);
	hhandler.keyUp(key);
}

void PCIDeformationConstraintEditor::addConstraintObj(GeneralARConstraint *constr) 
{
	SGDeformationConstraintsList *node = dynamic_cast<SGDeformationConstraintsList*>(getSceneGraphNodeByName(getScene(), constraintsNode));
	if (node == NULL) {
		error("PCIDeformationConstraintEditor::addConstraint() - constraintsNode not found (or of wrong type).");
		return;
	}
	node->addConstraint(constr);
	updateConstraintsNode();
}

void PCIDeformationConstraintEditor::addConstraint() 
{
   GeneralARConstraint *constr;
	if (ConstraintsDialog::createConstraint(sceneEditorWidget, constr)) {
		addConstraintObj(constr);
	}
}

void PCIDeformationConstraintEditor::deleteConstraint() 
{
	unsigned lastSelection = hhandler.getLastSelectedHandle();
	SGDeformationConstraintsList *node = dynamic_cast<SGDeformationConstraintsList*>(getSceneGraphNodeByName(getScene(), constraintsNode));
	if (node == NULL) {
		error("PCIDeformationConstraintEditor::addConstraint() - constraintsNode not found (or of wrong type).");
		return;
	}
	if (lastSelection < node->getNumConstraints()) {
      node->deleteConstraint(lastSelection);
   }
	
	updateConstraintsNode();
}

bool PCIDeformationConstraintEditor::setParams() 
{
	SceneGraphNode *node = NULL;
	Matrix4f transf;
	getSGNodeTransformation(getScene(), constraintsNode, transf, node);
	if (node) {
		hhandler.setParams(getScene()->getRootState()->staticState->camera, getScene()->getRootState()->staticState->viewFrustum, transf);
		return true;
	}
	return false; 
}

void PCIDeformationConstraintEditor::createConstraintsNode() 
{
   std::string::size_type offset = constraintsNode.find_last_of('/');
	static const std::string::size_type npos = -1;
   if (offset == npos) {
		error("PCIDeformationConstraintEditor::createConstraintsNode() - constraintsNode is not a valid path (no separator char '/' found).");
		return;
   }
   std::string nodeName = constraintsNode.substr(offset+1, constraintsNode.length()-offset);
   std::string pathName = constraintsNode.substr(0, offset);
   SceneGraphNode *node = getSceneGraphNodeByName(getScene(), constraintsNode);
   if (node != NULL) {
		error("PCIDeformationConstraintEditor::createConstraintsNode() - target does already exists. Please check the name/path.");
		return;
   }
	SceneGraphNode *parent = getSceneGraphNodeByName(getScene(), pathName);	
	if (!parent) {
		error("PCIDeformationConstraintEditor::createConstraintsNode() - containing node not found, please check path.");
		return;
	}
	SGDeformationConstraintsList *result = new SGDeformationConstraintsList();
	result->setName(nodeName);
	if (!parent->addChildNode(result)) {
		error("PCIDeformationConstraintEditor::createConstraintsNode() - specified parent does not take additional child nodes");
		delete result;
		return;
	}
	updateConstraintsNode();
}


std::string PCIDeformationConstraintEditor::getConstraintsNode() const
{
	return constraintsNode;
}

void PCIDeformationConstraintEditor::setConstraintsNode(std::string s) 
{
	constraintsNode = s;
	updateConstraintsNode();
}

void PCIDeformationConstraintEditor::updateConstraintsNode()
{
	clearHandles();	
	if (getScene() == NULL)
		return;
	SGDeformationConstraintsList *node = dynamic_cast<SGDeformationConstraintsList*>(getSceneGraphNodeByName(getScene(), constraintsNode));
	if (node != NULL) {
		mpcard nConstr = node->getNumConstraints();
		for (mpcard i=0; i<nConstr; i++) {
			HandleData data;
			Vector3f toPos;
			GeneralARConstraint *constr = node->getConstraint(i);
			if (dynamic_cast<PositionARConstraint*>(constr)) {
				data.timeOrder = 0;
				data.startTime = dynamic_cast<PositionARConstraint*>(constr)->getTime();
				data.secondaryTime1 = -1;
				data.secondaryTime2 = -1;
				toPos = dynamic_cast<PositionARConstraint*>(constr)->getToPoint();

			} else if (dynamic_cast<VelocityVectorARConstraint*>(constr)) {
				data.timeOrder = 1;
				data.startTime = dynamic_cast<VelocityVectorARConstraint*>(constr)->getFromTime();
				data.secondaryTime1 = dynamic_cast<VelocityVectorARConstraint*>(constr)->getToTime();
				data.secondaryTime2 = -1;
				toPos = dynamic_cast<VelocityVectorARConstraint*>(constr)->getVelocity();

			} else if (dynamic_cast<InstantaneousVelocityARConstraint*>(constr)) {
				data.timeOrder = 1;
				data.startTime = dynamic_cast<InstantaneousVelocityARConstraint*>(constr)->getTime();
				data.secondaryTime1 = -1;
				data.secondaryTime2 = -1;
				toPos = dynamic_cast<InstantaneousVelocityARConstraint*>(constr)->getVelocity();
			} else {
				continue;
			}
			handlesData.push_back(data);
			
			Handle3D *handle = Handle3D::makeVectorHandle(constr->getAnchor().getPos(), toPos);
			handles.push_back(handle);
		}
	}
}

void PCIDeformationConstraintEditor::clearHandles() 
{
	for (unsigned i=0; i<handles.size(); i++) {
		delete handles[i];
	}
	handles.clear();
	handlesData.clear();
}

void PCIDeformationConstraintEditor::handleChanged(card32 which, card32 point) 
{
	if (which >= handles.size()) {
		x4Assert(false);
		return;
	}
	
	SGDeformationConstraintsList *node = dynamic_cast<SGDeformationConstraintsList*>(getSceneGraphNodeByName(getScene(), constraintsNode));
	if (node == NULL) {
		error("PCIDeformationConstraintEditor::addConstraint() - constraintsNode not found (or of wrong type). Internal error, this should never happen at this point.");
		return;
	}
	if (which >= node->getNumConstraints()) {
		error("PCIDeformationConstraintEditor::addConstraint() - handle index too large. Internal error, this should never happen at this point.");
		return;
	}
	node->getConstraint(which)->anchor.pos = handles[which]->getPoint(0);
	if (handles[which]->getNumPoints() == 2) {
		Vector3f x = handles[which]->getPoint(1);
		VectorValuedARConstraint *vc = dynamic_cast<VectorValuedARConstraint*>(node->getConstraint(which));
		if (vc) {
			vc->setConstrVector(x);
		}
	}
}

void X4::PCIDeformationConstraintEditor::addPlaneConstraints()
{
	SGObjectNode *sgoSource = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), "root/pc1"));
	if (!sgoSource) {
		error("PCCDeformation::initializeNodes() - SGObjectNode for source node not found.");
		return;
	}
	PointCloud *pc_s = dynamic_cast<PointCloud*>(sgoSource->getSceneObject());
	if (!pc_s) {
		error("PCCDeformation::initializeNodes() - SGObjectNode for source is not a PointCloud.");
		return;
	}
	SceneObjectIterator *it_s = pc_s->createIterator(SOIT::CAP_BASIC_PC);
	BasicPointCloudIterator *bit_s = dynamic_cast<BasicPointCloudIterator*>(it_s);
	if (!bit_s) {
		error("PCCDeformation::execute() - cannot apply deformation - sourceNode does not provide basic point cloud iterator interface.");
		delete it_s;
		return;
	}
	AAT posAAT_S = pc_s->getAAT("position");

	card32 index=0;
	while(!bit_s->atEnd()){
		Vector3f pos=bit_s->get3f(posAAT_S);
		
		PositionARConstraint *constr = new PositionARConstraint();
		
		ARConstraintAnchor anchor;
		anchor.setup(pos, index);
		index++;
		
		if(pos[0] <= 10){
			constr->setup(anchor, pos, IDENTITY3F, getScene()->getRootState()->dynamicState->time);
			addConstraintObj(constr);
		}
		else if(pos[0] >=240){
			Vector3f toPos = makeVector3f(pos[0]-50,pos[1],pos[2]+100);
			constr->setup(anchor, toPos, IDENTITY3F, getScene()->getRootState()->dynamicState->time);
			addConstraintObj(constr);		
		}

		bit_s->next();
	}

	delete it_s;

	ObjectViewsTable::update(getScene());
}


void X4::PCIDeformationConstraintEditor::addHalfSpaceConstraint()
{
	SGObjectNode *sgoSource = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), "root/pc1"));
	if (!sgoSource) {
		error("PCCDeformation::initializeNodes() - SGObjectNode for source node not found.");
		return;
	}
	PointCloud *pc_s = dynamic_cast<PointCloud*>(sgoSource->getSceneObject());
	if (!pc_s) {
		error("PCCDeformation::initializeNodes() - SGObjectNode for source is not a PointCloud.");
		return;
	}
	SceneObjectIterator *it_s = pc_s->createIterator(SOIT::CAP_BASIC_PC);
	BasicPointCloudIterator *bit_s = dynamic_cast<BasicPointCloudIterator*>(it_s);
	if (!bit_s) {
		error("PCCDeformation::execute() - cannot apply deformation - sourceNode does not provide basic point cloud iterator interface.");
		delete it_s;
		return;
	}
	AAT posAAT_S = pc_s->getAAT("position");

	card32 index=0;
	while(!bit_s->atEnd()){
		Vector3f pos=bit_s->get3f(posAAT_S);
		ARHalfSpaceConstraint* result=new ARHalfSpaceConstraint();
		ARConstraintAnchor anchor;
		anchor.setup(pos, index);
		index++;


		/// penalty = w*[(x-fp)*n]^2, if (x-fp)*n>0, zero otherwise
		float32 weight= 1.0f;

		result->setup(anchor, 0, halfSpaceFootPoint, halfSpaceNormal, weight);

		addConstraintObj(result);
		bit_s->next();
	}

	usingHalfSpaceConstraints=true;
	delete it_s;

	ObjectViewsTable::update(getScene());
}

