//---------------------------------------------------------------------------
#ifndef PCIDeformationConstraintEditorH
#define PCIDeformationConstraintEditorH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "Handle3DHandler.h"
//---------------------------------------------------------------------------

namespace X4 {

class Handle3D;
class GeneralARConstraint;
class RenderingTool;

///   mw - sept 24 2007
///
///   Editor for interactive editing of SGDeformationConstraint objects
///
class X3DEFORMATION_API PCIDeformationConstraintEditor : public PCInteractionTool, public Handle3DUpdatesListener {
	X4_CLASS( PCIDeformationConstraintEditor )
 protected: 
	struct HandleData {
		int32 startTime;
		int32 secondaryTime1;
		int32 secondaryTime2;
		int32 timeOrder;		
	
		HandleData() {}
		HandleData(int32 startTime, int32 secondaryTime1, int32 secondaryTime2, int32 timeOrder);
	};
	
	std::string constraintsNode;
	bool showFromNow;
	bool showToNow;
	bool showAll;
	bool executeDeformOnline;

	bool usingHalfSpaceConstraints;
	Vector3f halfSpaceNormal;
	Vector3f halfSpaceFootPoint;
	
	std::vector<Handle3D*> handles;
	std::vector<HandleData> handlesData;
	Handle3DHandler hhandler;
	card32 viewportWidth;
	card32 viewportHeight;
	std::map<GLContext*,RenderingTool *> m_RenderingTools;

	void addConstraint();
	void addHalfSpaceConstraint();
	void addPlaneConstraints();
	
	void deleteConstraint();
	void createConstraintsNode();
	
	bool setParams();
	std::string getConstraintsNode() const;
	void setConstraintsNode(std::string s);
	void updateConstraintsNode();
	void clearHandles();
	void addConstraintObj(GeneralARConstraint *constr);

 public:
	PCIDeformationConstraintEditor();

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void activateTool();
   virtual void deactivateTool();
   virtual void glDrawTool(GLContext *glContext);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

	virtual void handleChanged(card32 which, card32 point);
	

   ~PCIDeformationConstraintEditor();
};

}

#endif