//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PCIShapeMatching.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "DeformationSolver.h"
#include "MMDeformationSolver.h"
#include "ClassTypeClassProperty.h"
#include "SeparatorClassProperty.h"
#include "ARSettings.h"
#include "UnstructuredInCorePointCloud.h"
#include "ARDeformationOptimizationContext.h"
#include "SGDeformationNodeAR.h"
#include "SceneGraphTools.h"
#include "DeformationSolver.h"
#include "DeformationEvaluator.h"
#include "SceneEditorWidget.h"
#include "ICPConstraints.h"
#include "PointSetANNQuery.h"
#include "PickFromPointSet.h"
#include "TSDeformationSolver.h"
#include "Timer.h"
#include "AREvaluateDeformation.h"
#include "CopyObjectProperties.h"
#include "BinaryObjectStreams.h"
#include "PointCloudTools.h"
#include "FileDialogs.h"
#include <QtGui/QMessageBox>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

#define SHOW_TIMINGS

static const float32 MAX_POINT_DIST = 5.0f;


IMPLEMENT_X4_CLASS(ShapeMatchingHandle3D, 0)
{
	BEGIN_CLASS_INIT(ShapeMatchingHandle3D);

	ADD_CARD32_PROP(index, 0);
}

IMPLEMENT_X4_CLASS(ShapeMatchingPersistentHandles, 0)
{
	BEGIN_CLASS_INIT(ShapeMatchingPersistentHandles);

	ADD_OBJECT_LIST_PROP( handles, 0, ShapeMatchingHandle3D::getClass() );
}


IMPLEMENT_X4_CLASS(PCIShapeMatching, 2)
{
	BEGIN_CLASS_INIT(PCIShapeMatching)

	ADD_SEPARATOR("Input & Output")
	ADD_STRING_PROP(urshapeNode, 0)
	ADD_STRING_PROP(deformationNode, 0)
	ADD_STRING_PROP(resultNode, 0)
	ADD_STRING_PROP(matchingNode, 0)
	ADD_BOOLEAN_PROP(m_WriteGradients,1);
	SET_NAME("writeGradients");

	ADD_SEPARATOR("Solver")
	ADD_METACLASS_PROP(deformationSolver, 0, DeformationSolver::getClass())
	ADD_OBJECT_PROP(settings, 0, ARSettings::getClass(), true);
   // alias property
	ADD_CARD32_PROP_METHOD(gridSize, 0, &getGridSize, &setGridSize)
	SET_STREAM_PROPERTY(false)
   // alias property
	ADD_FLOAT32_PROP_METHOD(rigidityParam, 0, &getRigidityParam, &setRigidityParam)
	SET_STREAM_PROPERTY(false)
	ADD_FLOAT32_PROP(pointWeight, 0)
	ADD_FLOAT32_PROP(dataWeight, 0)
	ADD_FLOAT32_PROP(boundaryWeight, 0)

	
	ADD_SEPARATOR("State")
	ADD_BOOLEAN_PROP(running, 0)
	ADD_BOOLEAN_PROP_METHOD(updateDeformation, 0, &getUpdateDeformation, &setUpdateDeformation)
	ADD_BOOLEAN_PROP_METHOD(shapeMatching, 0, &getShapeMatching, &setShapeMatching)
	ADD_BOOLEAN_PROP_METHOD(boundaryToBoundary, 0, &getBoundaryToBoundary, &setBoundaryToBoundary)
	
	
	//ADD_NOARGS_METHOD(start)
	//ADD_NOARGS_METHOD(stop)
   ADD_NOARGS_METHOD(writeDeformation)
   ADD_NOARGS_METHOD(recomputeDeformation)

	ADD_NOARGS_METHOD(loadHandles);
	ADD_NOARGS_METHOD(saveHandles);
	ADD_NOARGS_METHOD(clearHandles);
}

//---------------------------------------------------------------------------

PCIShapeMatching::PCIShapeMatching() {
   urshapeNode = "root/pc1";
   deformationNode = "root/deformation";
   resultNode = "root/deformed";
   matchingNode = "root/data";

   deformationSolver = MMDeformationSolver::getClass();
   settings = new ARSettings();
   settings->gridSize = 2;
   settings->correspondenceParam = 1;
   settings->rigidityParam = 1;
   settings->icpPercentileCulling = false;
   settings->correspondenceParam = 1;
   pointWeight = 1000;
   dataWeight = 1;
   boundaryWeight = 1;

   running = false;
   updateDeformation = true;
   shapeMatching = false;
   boundaryToBoundary = false;

   // -- dynamic state
   currentUrshape = NULL;
   currentResult = NULL;
   cachedCtxt = NULL;
   lastGridSize = 0xFFFFFFFF;
   lastTSGridSpacing = -12345;
   lastSolver = NULL;

	hhandler.setHandles(&handles);
	hhandler.addChangesListener(this);
	m_WriteGradients = false;
}

PCIShapeMatching::~PCIShapeMatching() {
   delete settings;
   delete cachedCtxt;
}

void PCIShapeMatching::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj, this);
}

//---------------------------------------------------------------------------

bool PCIShapeMatching::fetchNodes(
   UnstructuredInCorePointCloud *&urshape, 
   UnstructuredInCorePointCloud *&result)
{
   urshape = dynamic_cast<UnstructuredInCorePointCloud*>(getPointCloud(getScene(), urshapeNode));
   if (!urshape) {
      return false;
   }
   
   result = dynamic_cast<UnstructuredInCorePointCloud*>(getPointCloud(getScene(), resultNode));
   if (!result) {
      result = (UnstructuredInCorePointCloud*)(urshape->copy());
      addPointCloud(getScene(), result, resultNode);
   }
   return true;
}


void PCIShapeMatching::keyDown(GeneralKey key) 
{
   keyDownPassToCam(key);
}

void PCIShapeMatching::keyUp(GeneralKey key) 
{
   keyUpPassToCam(key);
}



ShapeMatchingHandle3D *ShapeMatchingHandle3D::makeMyHandle(
	const Vector3f &startPos, 
	const Vector3f &pos, 
	const mpint index) 
{
	ShapeMatchingHandle3D *result = new ShapeMatchingHandle3D();
	result->points.push_back(Point(pos, makeVector4f(0.5f, 1.0f, 0.4f, 1.0f)));
	result->points.push_back(Point(startPos, NULL_VECTOR4F, false));
	result->lines.push_back(Line(0,1,makeVector4f(1,1,1,0.3f)));
	result->index = index;
	return result;
}

void ShapeMatchingHandle3D::assign(
	const Object* obj, X4CopyContext *context ) 
{
	Handle3D::assign(obj, context);

	const ShapeMatchingHandle3D * o = dynamic_cast<const ShapeMatchingHandle3D*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}



X4::ShapeMatchingPersistentHandles::ShapeMatchingPersistentHandles( const std::vector<Handle3D*>& handles ) {
	for( int i1 = 0; i1 < (int)handles.size(); ++ i1 )
		this->m_handles.push_back( (Handle3D*) handles[i1]->copy() );
}

X4::ShapeMatchingPersistentHandles::~ShapeMatchingPersistentHandles() {
	for( int i1 = 0; i1 < (int)m_handles.size(); ++ i1 )
		delete m_handles[i1];
	m_handles.clear();
}

void X4::ShapeMatchingPersistentHandles::assign(const Object* obj, X4CopyContext *context) {
	const ShapeMatchingPersistentHandles * o = dynamic_cast<const ShapeMatchingPersistentHandles*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}




void X4::PCIShapeMatching::saveHandles() {
	std::string ext( ".handle.object" );
	std::string filename = FileDialogs::getSaveFileName( getToolWidget(), "save file", "*" + ext );
	if( !filename.empty() ) {
		if( std::search( filename.begin(), filename.end(), ext.begin(), ext.end() ) == filename.end() )
			filename = filename + ext;
		BinaryOutputObjectStream out( filename.c_str() );
		ShapeMatchingPersistentHandles* p = new ShapeMatchingPersistentHandles( handles );
		out.writeObject( p );
	}
}

void X4::PCIShapeMatching::loadHandles() {
	std::string filename = FileDialogs::getOpenFileName( getToolWidget(), "open file", "*.handle.object" );
	if (!filename.empty()) {
		BinaryInputObjectStream in(filename.c_str());
		Persistent *instance = NULL;
		in.readObject(instance);
		ShapeMatchingPersistentHandles* hInstance = dynamic_cast<ShapeMatchingPersistentHandles*>(instance);
		if( hInstance == NULL ) {
			QMessageBox::information( getToolWidget(), "Error", "this is not a handle object" );
			delete instance;
		} else {
			handles.clear();
			for( int i1 = 0; i1 < (int)hInstance->m_handles.size(); ++ i1 ) {
				Handle3D* h = dynamic_cast<Handle3D*>( hInstance->m_handles[i1]->copy() );

				ShapeMatchingHandle3D* smh = dynamic_cast<ShapeMatchingHandle3D*>( h );
				if( smh ) {
					// disable topological constraints after load, because they might vary
					smh->index = ARConstraintAnchor::NO_TOP_ANCHOR;
				}

				handles.push_back( h );
			}

			ObjectViewsTable::update(this);
		}
	}
}

void X4::PCIShapeMatching::clearHandles() {
	handles.clear();

	ObjectViewsTable::update(this);
}

bool PCIShapeMatching::pickPoint(int32 x, int32 y, PointCloud *pc, mpcard &index) {
   UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
   if (!upc) return false; 
   PointSet *ps = upc->getPointSet();

//    if (false)
// 	{   float32 dist = computeMinOnScreenDistancePoint(
// 		   x,y,ps, 
// 		   getScene()->getRootState()->staticState->viewFrustum,
// 		   getScene()->getRootState()->staticState->camera,
// 		   screenSizeX, screenSizeY, 
// 		   index);
// 		   if (dist <= MAX_POINT_DIST) return true;
// 		   return false;
//    }
   Vector3f clickPoint;
   if( guessClickPoint(x, y,3 , clickPoint) )
   {
	   AAT POS = upc->getAAT("position");
	   HierarchicalKNNIterator knn(pc, 16, NULL);
	   knn.setSeekPointAndReset(clickPoint);
	   float32 dist = norm(knn.get3f(POS)-clickPoint);
	   index = knn.getPointSetPointNumber();
	   if (dist <= MAX_POINT_DIST) 
		   return true;
	   return false;
   }
   else
	   return false;

   
   //std::string pcname;
   //PointCloudIterator *returnedIt = getClosestPointIteratorFromAllPoints(
   //   x, y, screenSizeX, screenSizeY, 5,
   //   &(getScene()->getRootState()->staticState->camera),
   //   &(getScene()->getRootState()->staticState->viewFrustum),
   //   getScene(), 
   //   pcname);
   //if (returnedIt==NULL) return false;

   //pc = getPointCloud(getScene(), pcname);

   //IndexedPointCloudIterator *idxIt = dynamic_cast<IndexedPointCloudIterator*>(returnedIt);
   //if (idxIt) {
   //   index = (mpcard)(idxIt->getCurrentPointIndex());
   //   delete returnedIt;
   //   return true;
   //} else {
   //   delete returnedIt;
   //   return false;
   //}
}


void PCIShapeMatching::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
   
	if ( buttonsState.getMiddle() )
      mouseDownPassToCam(x,y,buttonsState,modifiersState);
	
	if (!running) {
      //PointCloud *pc;
      mpcard index;
      PointCloud *resultPC = getPointCloud(getScene(), resultNode);
      PointCloud *urshapePC = getPointCloud(getScene(), urshapeNode);
      if (  (urshapePC != NULL && pickPoint(x,y, urshapePC, index))
         || (resultPC != NULL && pickPoint(x,y, resultPC, index)) ) 
      {
         UnstructuredInCorePointCloud *forgetit1;
         UnstructuredInCorePointCloud *forgetit2;
         if (fetchNodes(forgetit1, forgetit2)) {
            start();
         } else {
            mouseDownPassToCam(x,y,buttonsState,modifiersState);
            return;
         }
      } else {
         mouseDownPassToCam(x,y,buttonsState,modifiersState);
         return;
      }
   }

	hhandler.setParams(
	   getScene()->getRootState()->staticState->camera, 
	   getScene()->getRootState()->staticState->viewFrustum,
	   IDENTITY4F);
   
   hhandler.mouseDown(x, y, MouseButtons(buttonsState.getLeft() || buttonsState.getRight(), false, false), modifiersState);
   if (buttonsState.getLeft()) {
      if (hhandler.selected()) {
         return;
      }
      mpcard index;
      if (currentResult != NULL) {
         if (pickPoint(x,y, currentResult, index)) {
            if (index < currentResult->getPointSet()->getNumEntries()) {
               AAT positionAAT = currentResult->getAAT("position");
               AAT positionAATUS = currentUrshape->getAAT("position");
               Vector3f pos = currentResult->getPointSet()->get3f(index, positionAAT);
               ShapeMatchingHandle3D *newPoint = ShapeMatchingHandle3D::makeMyHandle(currentUrshape->getPointSet()->get3f(index, positionAATUS), pos, index);
               handles.push_back(newPoint);
               hhandler.mouseDown(x, y, buttonsState, modifiersState);
            } else {
               mouseDownPassToCam(x,y,buttonsState,modifiersState);
            }
         } else {
            mouseDownPassToCam(x,y,buttonsState,modifiersState);
         }
      } else {
         mouseDownPassToCam(x,y,buttonsState,modifiersState);
      }
   } else if (buttonsState.getRight()) {
      bool sel;
      card32 h;
      card32 p;
      hhandler.getSelection(sel, h, p);
      if (sel && h<handles.size()) {
         delete handles[h];
         handles.erase(handles.begin()+h);
      } else {
         mouseDownPassToCam(x,y,buttonsState,modifiersState);
      }
   }
}

void PCIShapeMatching::mouseMoved(int32 x, int32 y) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      if (w->mapGLAreaToGlobal(QPoint(x,y)) != QCursor::pos()) {
         return;
      }
   }
   handleChangedFlag = false;
	hhandler.setParams(
	   getScene()->getRootState()->staticState->camera, 
	   getScene()->getRootState()->staticState->viewFrustum,
	   IDENTITY4F
   );
   hhandler.mouseMoved(x, y);
   if (!handleChangedFlag) {
      mouseMovedPassToCam(x,y);
   }
   //handleChangedFlag = false;
}

void PCIShapeMatching::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
   bool working = hhandler.selected();
	hhandler.setParams(
	   getScene()->getRootState()->staticState->camera, 
	   getScene()->getRootState()->staticState->viewFrustum,
	   IDENTITY4F
   );
   hhandler.mouseUp(x, y, MouseButtons(buttonsState.getLeft() || buttonsState.getRight(),false, false), modifiersState);
   if (!working) {
      mouseUpPassToCam(x, y, buttonsState, modifiersState);
   }
}

void PCIShapeMatching::areaResize(card32 width, card32 height) 
{
	hhandler.setParams(
	   getScene()->getRootState()->staticState->camera, 
	   getScene()->getRootState()->staticState->viewFrustum,
	   IDENTITY4F
   );
   hhandler.areaResize(width, height);
   areaResizePassToCam(width, height);
}

void PCIShapeMatching::mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState) 
{
	//hhandler.setParams(
	//   getScene()->getRootState()->staticState->camera, 
	//   getScene()->getRootState()->staticState->viewFrustum,
	//   IDENTITY4F
   //);
   //hhandler.mouseWheelRotated(rotatedDelta, modifiersState);
   mouseWheelRotatedPassToCam(rotatedDelta, modifiersState);
}

void PCIShapeMatching::start() 
{
   if (!running) {
      if (!fetchNodes(currentUrshape, currentResult)) {
         warning("path(s) not found");
         stop();
         return;
      }
      running = true;
      SceneGraphNode *urshapeSGNode = getSceneGraphNode(getScene(), urshapeNode);
      if (urshapeSGNode) {
         urshapeSGNode->setVisible(false);
      }
      SceneGraphNode *resultSGNode = getSceneGraphNode(getScene(), resultNode);
      if (resultSGNode) {
         resultSGNode->setVisible(true);
      }
   } else {
      warning("already running");
   }
   ObjectViewsTable::update(this);
}

void PCIShapeMatching::stop() 
{
   currentUrshape = NULL;
   currentResult = NULL;
   running = false;
   SceneGraphNode *urshapeSGNode = getSceneGraphNode(getScene(), urshapeNode);
   if (urshapeSGNode) {
      urshapeSGNode->setVisible(true);
   }
   ObjectViewsTable::update(this);
}

void PCIShapeMatching::handleChanged(card32 which, card32 point) 
{
  // recomputeDeformation();
   handleChangedFlag = true;
}

void PCIShapeMatching::glDrawTool(GLContext *glContext) 
{
	for (unsigned i=0; i<handles.size(); i++) {
		handles[i]->glDraw(glContext);
	}
}

bool PCIShapeMatching::getUpdateDeformation() const
{
   return updateDeformation;
}

void PCIShapeMatching::setUpdateDeformation(bool b) 
{
   updateDeformation = b;
   recomputeDeformation();
}

bool PCIShapeMatching::getShapeMatching() const
{
   return shapeMatching;
}

void PCIShapeMatching::setShapeMatching(bool b) 
{
   shapeMatching = b;
   recomputeDeformation();
}

bool PCIShapeMatching::getBoundaryToBoundary() const
{
   return boundaryToBoundary;
}

void PCIShapeMatching::setBoundaryToBoundary(bool b) 
{
   boundaryToBoundary = b;
   recomputeDeformation();
}

void PCIShapeMatching::recomputeDeformation() {
   if (!running) {
      start();
   }
   if (!running) {
      return;
   }                             
   if (currentUrshape && currentResult) {
      DeformationSolver *solver = (DeformationSolver*)(deformationSolver->newInstance());
      DeformationEvaluator *evaluator = solver->createEvaluator();
      if (cachedCtxt) {
         if (deformationSolver != lastSolver || lastGridSize != getGridSize() || lastTSGridSpacing != settings->tsGridSpacing) {
            delete cachedCtxt;
            cachedCtxt = NULL;
         }
      }
      if (!cachedCtxt) {
         cachedCtxt = (ARDeformationOptimizationContext*)(solver->createOptContext());
         cachedCtxt->initializeFromShape(currentUrshape, NULL_VECTOR2F, settings);
         lastSolver = deformationSolver;
         lastGridSize = getGridSize();
         lastTSGridSpacing = settings->tsGridSpacing;
      }

      recomputeDeformation(solver, evaluator, cachedCtxt);
      
      delete solver;
      delete evaluator;
   }
}

PointSet *PCIShapeMatching::extractBoundary(PointCloud *pc) {
   UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
   if (!upc) return NULL;
   PointSet *ps = upc->getPointSet();
   if (!ps->providesAttribute("boundary", 1, VAD::DATA_FORMAT_FLOAT32)) return NULL;
   AAT boundaryAAT = ps->getAAT("boundary");
   mpcard numB = 0;
   for (mpcard i=0; i<ps->getNumEntries(); i++) {
      float32 boundary = ps->get1f(i, boundaryAAT);
      if (boundary > 0.5f) numB++;
   }
   if (numB==0) return NULL;
   PointSet *result = new PointSet();
   result->clearAndSetup(1, numB, ps->getDescr());
   mpcard index = 0;
   for (mpcard i=0; i<ps->getNumEntries(); i++) {
      float32 boundary = ps->get1f(i, boundaryAAT);
      if (boundary > 0.5f) {
         result->setVertex(index, ps->getDataPointer(i));
         index++;
      }      
   }   
   return result;
}


void PCIShapeMatching::recomputeDeformation(DeformationSolver *solver, DeformationEvaluator *evaluator, ARDeformationOptimizationContext *ctxt) {
   if (currentUrshape && currentResult) {
      
      AAT urshapePositionAAT = currentUrshape->getAAT("position");
      AAT resultPositionAAT = currentResult->getAAT("position");
      PointSet *urshapePS = currentUrshape->getPointSet();
      PointSet *resultPS = currentResult->getPointSet();
      if (urshapePositionAAT == NULL_AAT || resultPositionAAT == NULL_AAT) {
         return;
      }
      
      #ifdef SHOW_TIMINGS
         debugOutput << "constraints adding:\n";
         X4_TIMER_START(pointContr);
      #endif
      
      solver->setup(cachedCtxt, settings);
      solver->beginConstraints();

      extraConstraints();

      for (size_t i=0; i<handles.size(); i++) {
         ShapeMatchingHandle3D* handle = (ShapeMatchingHandle3D*)(handles[i]);
         PositionARConstraint c;
		 if( handle->index == ARConstraintAnchor::NO_TOP_ANCHOR ) {
			 c.anchor = ARConstraintAnchor( handle->getPoint(1), ARConstraintAnchor::NO_TOP_ANCHOR );
		 } else {
			 c.anchor = ARConstraintAnchor( urshapePS->get3f(handle->index, urshapePositionAAT), handle->index );
		 }
         c.covariance = IDENTITY3F*pointWeight;
         c.time = 0;
         c.toPoint = handle->getPoint(0);
         solver->addPositionConstraint(&c);
      }
      
      #ifdef SHOW_TIMINGS
         X4_TIMER_END(pointContr);
         debugOutput << "icpConstr:\n";
         X4_TIMER_START(icpConstr);
      #endif
            
      if (shapeMatching) {
         PointCloud *data = getPointCloud(getScene(), matchingNode);
         if (!data) {
            warning("matchingNode not found - shape matching not employed.");
         } else {
		      ICPConstraints icp;
            icp.setConstraintMode(settings->icpConstraintsMode);
            icp.setDistanceOutliersMode(settings->icpPercentileCulling?ICP_OUTLIERS_PERCENTILE:ICP_OUTLIERS_NONE);
            icp.setOutliersDistanceFactor(settings->icpNearFieldRelativeCutOff);
            icp.setOutliersPercentile(settings->icpOutlierPercentile);
            icp.setCullNormalOutliers(settings->icpNormalCulling);
            icp.setOutlierNormals(settings->icpNormalCosAngleCutOff);
            
  		      icp.buildICPConstraints(
  		         settings,
  		         solver,
  		         currentUrshape,
  		         currentResult, 
  		         data, 0);
            
         }
      }
      if (boundaryToBoundary) {
         PointCloud *data = getPointCloud(getScene(), matchingNode);
         PointSet *dataBoundary = extractBoundary(data);

         if (dataBoundary 
             && currentResult->getPointSet()->providesAttribute("boundary", 1, VAD::DATA_FORMAT_FLOAT32)
             && dataBoundary->providesAttribute("boundaryTangent", 3, VAD::DATA_FORMAT_FLOAT32)) 
         {
            PointSetANNQuery query(dataBoundary, 1);
            PointSet *currentResultPS = currentResult->getPointSet();
            PointSet *currentUrshapePS = currentUrshape->getPointSet();
            AAT currentUrshape_positionAAT = currentUrshapePS->getAAT("position");
            AAT currentResult_boundaryAAT = currentResultPS->getAAT("boundary");
            AAT currentResult_positionAAT = currentResultPS->getAAT("position");
            AAT dataBoundary_positionAAT = dataBoundary->getAAT("position");
            AAT dataBoundary_boundaryTangentAAT = dataBoundary->getAAT("boundaryTangent");
            const mpcard numResultPts = currentResultPS->getNumEntries();
            for (mpcard i=0; i<numResultPts; i++) {
               float32 boundaryVal = currentResultPS->get1f(i, currentResult_boundaryAAT);
               if (boundaryVal > 0.5f) {
                  int32 p = query.getNearestPointIndex(currentResultPS->get3f(i, currentResult_positionAAT));
                  Vector3f targetPos = dataBoundary->get3f(p, dataBoundary_positionAAT);
                  Vector3f targetTangent = normalize(dataBoundary->get3f(p, dataBoundary_boundaryTangentAAT));
                  Vector3f urshapePos = currentUrshapePS->get3f(i, currentUrshape_positionAAT);
                  
                  PositionARConstraint c;
                  c.anchor = ARConstraintAnchor(urshapePos, i);
                  c.covariance = (IDENTITY3F - outerProduct(targetTangent, targetTangent))*boundaryWeight;
                  c.time = 0;
                  c.toPoint = targetPos;
                  solver->addPositionConstraint(&c);
               }
            }
         }

         delete dataBoundary;
      }
      #ifdef SHOW_TIMINGS
         X4_TIMER_END(icpConstr);
         debugOutput << "solve:\n";
         X4_TIMER_START(solve);
      #endif
      
      solver->solve();

      #ifdef SHOW_TIMINGS
         X4_TIMER_END(solve);
         debugOutput << "eval:\n";
         X4_TIMER_START(eval)
      #endif
      
      PointCloud *pc_ev = AREvaluateDeformation::evaluatePC(currentUrshape, cachedCtxt, evaluator, 0, settings, m_WriteGradients);
      UnstructuredInCorePointCloud *upc_ev = dynamic_cast<UnstructuredInCorePointCloud*>(pc_ev);
      if (upc_ev) {
         currentResult->getPointSet()->assign(upc_ev->getPointSet());
      }
      delete pc_ev;
      
      //evaluator->setup(cachedCtxt, settings);
      //evaluator->beginQuery(0);
      //const mpcard n = urshapePS->getNumEntries();
      //resultPS->changeHeight(n);
      //for (mpcard i=0; i<n; i++) {
      //   Vector3f newPos = evaluator->getPosition(ARConstraintAnchor(urshapePS->get3f(i, urshapePositionAAT), i));
      //   resultPS->set3f(i, resultPositionAAT, newPos);
      //}
      //evaluator->endQuery();

      #ifdef SHOW_TIMINGS
         X4_TIMER_END(eval);
         debugOutput << "\n\n";
      #endif
      currentResult->deleteDelOnWriteAttachments();
   }
}

card32 PCIShapeMatching::getGridSize() const 
{
   return settings->gridSize;
}

void PCIShapeMatching::setGridSize(card32 g) 
{
   settings->gridSize = g;
   //recomputeDeformation();
}

float32 PCIShapeMatching::getRigidityParam() const 
{
   return settings->rigidityParam;
}

void PCIShapeMatching::setRigidityParam(float32 p) 
{
   settings->rigidityParam = p;
   //recomputeDeformation();
}

void PCIShapeMatching::writeDeformation() 
{
   SGDeformationNodeAR *deform = dynamic_cast<SGDeformationNodeAR*>(getSceneGraphNodeByName(getScene(), deformationNode));
   if (!deform) {
      deform = new SGDeformationNodeAR();
      deform->setDeformation(NULL);
      deform->setName(extractSGNode(deformationNode));
      try {
         addSceneGraphNode(scene, extractSGPath(deformationNode), deform);
      } catch (...) {
         delete deform;
         throw;
      }
   }
   if (deform) {
      DeformationSolver *solver = (DeformationSolver*)(deformationSolver->newInstance());
      DeformationEvaluator *evaluator = solver->createEvaluator();
      ARDeformationOptimizationContext *newCtxt = (ARDeformationOptimizationContext*)(solver->createOptContext());
      recomputeDeformation(solver, evaluator, newCtxt);
      deform->setDeformation(newCtxt);
      delete solver;
      delete evaluator;
   }
}

void X4::PCIShapeMatching::deactivateTool()
{
	if (autoSaveAndRestoreToolParameters)
	{
		bool r = running;
		running = false;
		savePCISettings();
		running = r;
	}
	
}

void X4::PCIShapeMatching::addHandle( const Vector3f& pos, const mpcard index )
{
	ShapeMatchingHandle3D *newPoint = ShapeMatchingHandle3D::makeMyHandle(pos, pos, index);
	handles.push_back(newPoint);
}


