//---------------------------------------------------------------------------
#ifndef PCIShapeMatching_H
#define PCIShapeMatching_H
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "Handle3DHandler.h"
#include "Handle3D.h"
#include "ObjectListProperty.h"
//---------------------------------------------------------------------------


namespace X4 {

class ARSettings;
class UnstructuredInCorePointCloud;
class SGDeformationNodeAR;
class ARDeformationOptimizationContext;
class DeformationSolver;
class DeformationEvaluator;


class ShapeMatchingHandle3D : public Handle3D {
	X4_CLASS( ShapeMatchingHandle3D )

public:
	mpint index;
	static ShapeMatchingHandle3D *makeMyHandle(const Vector3f &startPos, const Vector3f &pos, const mpint index);

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);
};

class ShapeMatchingPersistentHandles : public Persistent {
	X4_CLASS( ShapeMatchingPersistentHandles );

	std::vector<Handle3D*> m_handles;	
	IMPLEMENT_OBJECT_LIST_METHODS_STL( handles, Handle3D );


public:
	ShapeMatchingPersistentHandles() {
	}

	ShapeMatchingPersistentHandles( const std::vector<Handle3D*>& handles );
	virtual ~ShapeMatchingPersistentHandles();

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);
};

/// This is a tool to deform objects by user input
class X3DEFORMATION_API PCIShapeMatching : public PCInteractionTool, public Handle3DUpdatesListener {
	X4_CLASS( PCIShapeMatching )
 protected:
   
   // --------------------
   // interface properties
   // --------------------

   // === Input & Output ===
   std::string urshapeNode;
   std::string deformationNode;
   std::string resultNode;
   std::string matchingNode;

   // === Solver ===
   const MetaClass *deformationSolver;
   ARSettings *settings;

   bool fetchNodes(
      UnstructuredInCorePointCloud *&urshape, 
      UnstructuredInCorePointCloud *&result
   );
   float32 pointWeight;
   float32 dataWeight;
   float32 boundaryWeight;
   
   // === State ===
   bool running;
   bool updateDeformation; 
   bool shapeMatching; 
   bool boundaryToBoundary; 
   
   // === push buttons ===
   void start();
   void stop();       
   void writeDeformation();
   
   // --------------
   // dynamic state
   // --------------
   
   // --- data & solvers
   UnstructuredInCorePointCloud *currentUrshape;
   UnstructuredInCorePointCloud *currentResult;
   ARDeformationOptimizationContext *cachedCtxt;
   card32 lastGridSize;
   float32 lastTSGridSpacing;
   const MetaClass *lastSolver;
   bool handleChangedFlag;
	bool m_WriteGradients;
   
   // --- constraints
public:

protected:
	std::vector<Handle3D*> handles;
	Handle3DHandler hhandler;
   
      
   // --- helper methods
   void dragPoint(int32 x, int32 y);
	virtual void handleChanged(card32 which, card32 point);
	bool getUpdateDeformation() const;
	void setUpdateDeformation(bool b);
	bool getShapeMatching() const;
	void setShapeMatching(bool b);
	bool getBoundaryToBoundary() const;
	void setBoundaryToBoundary(bool b);
	virtual void recomputeDeformation();
	void recomputeDeformation(DeformationSolver *solver, DeformationEvaluator *evaluator, ARDeformationOptimizationContext *ctxt);
	card32 getGridSize() const;
	void setGridSize(card32 g);
	float32 getRigidityParam() const;
	void setRigidityParam(float32 p);
	PointSet *extractBoundary(PointCloud *pc);

   virtual void extraConstraints() {}
      

   void saveHandles();
   void loadHandles();
   void clearHandles();

 protected:
   bool pickPoint(int32 x, int32 y, PointCloud *pc, mpcard &index);
   
 public:
	PCIShapeMatching();

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);
   virtual void mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState);

   virtual void glDrawTool(GLContext *glContext);
   virtual void deactivateTool();

   bool handleChanged() const{return handleChangedFlag;}
   void addHandle(const Vector3f& pos, const mpcard index);


	~PCIShapeMatching();
};


}

#endif
