#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIVisionConstraints.h"
//---------------------------------------------------------------------------
#include "ClassMethods.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
#include "CameraParameterAttachment.h"
#include "ParametricInCorePointCloud.h"
#include "RegularInCoreVolume.h"
#include "RegularVolume.h"
#include "ProgressWindow.h"
#include "ShadowVolumeGrid.h"
#include "PCCImportDepthAnimation.h"
#include "SilhouetteImage.h"
#include "SilhouetteImageEditorWidget.h"
#include "DepthImageHalfSpaceConstraintCreator.h"
#include "MMDeformationSolver.h"
#include "PCCImportDepthAnimation.h"
#include "ARSettings.h"
#include "ARVisualizationSettings.h"
#include "PointSet.inline.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------

using namespace X4;

IMPLEMENT_X4_CLASS( PCIVisionConstraints ,1)
{
	BEGIN_CLASS_INIT( PCIVisionConstraints );
	ADD_FLOAT32_PROP(m_depthTolerance,0);
	ADD_CARD32_PROP(m_volumeGridRes,0)
	ADD_CARD32_PROP(m_minCamShadowVote,1)

	ADD_NOARGS_METHOD(testShadowVolume);
	ADD_NOARGS_METHOD(testDepthImage);
	ADD_NOARGS_METHOD(testHSConstraints);
}

PCIVisionConstraints::PCIVisionConstraints(void)
{
	m_depthTolerance=2.0f;
	m_volumeGridRes=64;
	m_minCamShadowVote=4;
	constrCreator=NULL;
}

PCIVisionConstraints::~PCIVisionConstraints(void)
{
	delete constrCreator;
}

void PCIVisionConstraints::assign(const Object* obj, X4CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCIVisionConstraints * o = dynamic_cast<const PCIVisionConstraints*>(obj);
	x4Assert( o != NULL );

}

void X4::PCIVisionConstraints::testShadowVolume()
{

	BoundingBox3f sceneBB = getScenePCBBox(scene);
//	GenericSwapAllOutOfCorePointCloud* spacePCGeneric= dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(getPointCloud(scene,PCCImportDepthAnimation::getPointCloudName(0,0,false)));
//	UnstructuredInCorePointCloud * pc3D=dynamic_cast<UnstructuredInCorePointCloud*>(spacePCGeneric->fetchPC());

	ShadowVolumeGrid shadow;
	shadow.setup(sceneBB, m_volumeGridRes, m_depthTolerance);

	card32 frameNum=0;

	shadow.processFrame(frameNum, scene);
	/*

	for (card32 camNo=0; camNo<8; camNo++)
	{
		GenericSwapAllOutOfCorePointCloud* picturePCGen  = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(getPointCloud(scene,PCCImportDepthAnimation::getPointCloudName(frameNum,camNo,true)));
		camAtt=dynamic_cast<CameraParameterAttachment*>(picturePCGen->getAttachments()->getData(CameraParameterAttachment::getAttachmentName()));
		ParametricInCorePointCloud * pc2D=dynamic_cast<ParametricInCorePointCloud*>(picturePCGen->fetchPC());
		picturePointset=pc2D->getPointSet();

		shadow.processDepthImage(camAtt, picturePointset);
	}
	shadow.visualizeVolumeInScene(m_minCamShadowVote, scene);

	*/
}

void X4::PCIVisionConstraints::testDepthImage()
{
	
	SilhouetteImage* si=new SilhouetteImage;
	si->setup(getPointCloud(scene,PCCImportDepthAnimation::getPointCloudName(0,0,true)), m_depthTolerance);
	SilhouetteImageEditorWidget* siw=new SilhouetteImageEditorWidget();
	siw->setup(si);
	siw->updateEditor();
	siw->show();

}

void X4::PCIVisionConstraints::testHSConstraints()
{
   delete constrCreator;

	ARSettings settings;
	settings.gridResolution=m_volumeGridRes;
	settings.depthTolerance=m_depthTolerance;

	MMDeformationSolver solv;

	constrCreator= new DepthImageHalfSpaceConstraintCreator();
	constrCreator->buildSilhouetteConstraints(&settings, 
		&solv,  getPointCloud(scene, PCCImportDepthAnimation::getPointCloudName(0, 0 ,false)), 
		getPointCloud(scene, PCCImportDepthAnimation::getPointCloudName(0, 0 ,false)), 0, scene);
}

void X4::PCIVisionConstraints::glDrawTool( GLContext *glContext )
{
	if (constrCreator)
	{
		ARVisualizationSettings s;
		constrCreator->glDrawFeedback(glContext, &s);
	}
	
}