//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "Handle3D.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


const Vector4f Handle3D::POINT_HANDLE_COL   = makeVector4f(0.2f, 0.3f, 0.4f, 1.0f);
const Vector4f Handle3D::TO_POINT_HANDLE_COL= makeVector4f(0.2f, 0.6f, 0.2f, 1.0f);
const Vector4f Handle3D::LINE_HANDLE_COL    = makeVector4f(0.4f, 0.4f, 0.4f, 1.0f);
const Vector4f Handle3D::COORD_X_HANDLE_COL = makeVector4f(0.7f, 0.5f, 0.5f, 1.0f);
const Vector4f Handle3D::COORD_Y_HANDLE_COL = makeVector4f(0.5f, 0.7f, 0.5f, 1.0f);
const Vector4f Handle3D::COORD_Z_HANDLE_COL = makeVector4f(0.5f, 0.5f, 0.7f, 1.0f);
const Vector4f Handle3D::BOX_HANDLE_COL     = makeVector4f(0.6f, 0.6f, 0.6f, 1.0f);
const float32  Handle3D::HANDLE_POINT_SIZE  = 10;
const float32  Handle3D::HANDLE_LINE_WIDTH  = 2;


IMPLEMENT_X4_ABSTRACT_CLASS( Handle3D , 0 ) {
	BEGIN_CLASS_INIT( Handle3D );
}

// ----

Handle3D::Handle3D() {
	selected = false;
}

void Handle3D::assign(const Object* obj, X4CopyContext *context) {
	const Handle3D* o = dynamic_cast<const Handle3D*>( obj );
	x4Assert(o);

	points = o->points;
	lines = o->lines;
	selected = o->selected;
	hover = o->hover;
//	copyObjectProperties(obj, this);
}

Handle3D::~Handle3D() {}




void Handle3D::addPoint(const Point &p) {
	points.push_back(p);
	hover.push_back(false);
}

void Handle3D::addLine(const Line &l) {
	lines.push_back(l);
}

mpcard Handle3D::getNumPoints() 
{
	return points.size();
}

Vector3f Handle3D::getPoint(card32 index)  const
{
	if (index >= points.size()) {
		throw ERangeCheck("Handle3D::getPoint() - index out of range.");
	}
	return points[index].pos;
}

bool Handle3D::getPointMovable(card32 index)
{
    if (index >= points.size()) {
        throw ERangeCheck("Handle3D::getPointMovable() - index out of range.");
    }
    return points[index].movable;
}

bool Handle3D::getPointVisible(card32 index)
{
	if (index >= points.size()) {
		throw ERangeCheck("Handle3D::getPointVisible() - index out of range.");
	}
	return points[index].visible;
}


void Handle3D::glDraw(GLContext *glContext) 
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glDisable(GL_LIGHTING);
	glPointSize(HANDLE_POINT_SIZE);
	glEnable(GL_POINT_SMOOTH);
	glEnable(GL_LINE_SMOOTH);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);

	glBegin(GL_POINTS);
	for (unsigned p=0; p<points.size(); p++) {
	   if (!points[p].visible) continue;
		Vector4f col = points[p].col;
		if (hover.size() > p) {
			if (hover[p]) col += makeVector4f(0.35f, 0.2f, 0.2f, 0.0f);
		}
		glColor4fv(col.data());
		glVertex3fv(points[p].pos.data());
	}	
	glEnd();

	glLineWidth(HANDLE_LINE_WIDTH);
	glBegin(GL_LINES);	
	for (unsigned l=0; l<lines.size(); l++) {
		Vector4f col = lines[l].col;
		if (selected) {
			col += makeVector4f(0.2f, 0.2f, 0.2f, 0.0f);
		}
		glColor4fv(col.data());
		glVertex3fv(points[ lines[l].ptn1 ].pos.data());
		glVertex3fv(points[ lines[l].ptn2 ].pos.data());
	}	
	glEnd();
	glPopAttrib();
}

void Handle3D::select() {
	selected = true;
}

void Handle3D::hoverPoint(card32 index) {
	hover[index] = true;
}

void Handle3D::clearHighlights() {
	selected = false;
	hover.clear();
	hover.resize(points.size(), false);
}

void Handle3D::movePoint(card32 index, const Vector3f &newPos, Modifiers modifiersState) 
{
	points[index].pos = newPos;
	forget(modifiersState);
}

Handle3D *Handle3D::makePointHandle(const Vector3f &initialPos) 
{
	Handle3D *result = new Handle3D();
	result->points.push_back(Point(initialPos, POINT_HANDLE_COL));
	return result;
}

Handle3D *Handle3D::makeVectorHandle(const Vector3f &initialPos1, const Vector3f &initialPos2) 
{
	Handle3D *result = new Handle3D();
	result->points.push_back(Point(initialPos1, POINT_HANDLE_COL));
	result->points.push_back(Point(initialPos2, TO_POINT_HANDLE_COL));
	result->lines.push_back(Line(0, 1, LINE_HANDLE_COL));	
	return result;
}

Handle3D *Handle3D::makeGradHandle(const Vector3f &initialPos, const Matrix3f &initialGrad) 
{
	Handle3D *result = new Handle3D();
	result->points.push_back(Point(initialPos, POINT_HANDLE_COL));
	result->points.push_back(Point(initialPos+initialGrad[0], COORD_X_HANDLE_COL));
	result->points.push_back(Point(initialPos+initialGrad[1], COORD_Y_HANDLE_COL));
	result->points.push_back(Point(initialPos+initialGrad[2], COORD_Z_HANDLE_COL));
	result->lines.push_back(Line(0, 1, COORD_X_HANDLE_COL*0.75f));	
	result->lines.push_back(Line(0, 2, COORD_Y_HANDLE_COL*0.75f));	
	result->lines.push_back(Line(0, 3, COORD_Z_HANDLE_COL*0.75f));	
	return result;
}

//Handle3D *Handle3D::makeVecGradHandle(const Vector3f &refPos, const Vector3f &initialPos, const Matrix3f &initialGrad) 
//{
//	Handle3D *result = new Handle3D();
//	result->points.push_back(Point(refPos, POINT_HANDLE_COL));
//	result->points.push_back(Point(initialPos, POINT_HANDLE_COL));
//	result->points.push_back(Point(initialPos+initialGrad[0], COORD_X_HANDLE_COL));
//	result->points.push_back(Point(initialPos+initialGrad[1], COORD_Y_HANDLE_COL));
//	result->points.push_back(Point(initialPos+initialGrad[2], COORD_Z_HANDLE_COL));
//	result->lines.push_back(Line(0, 1, LINE_HANDLE_COL));	
//	result->lines.push_back(Line(1, 2, COORD_X_HANDLE_COL*0.75f));	
//	result->lines.push_back(Line(1, 3, COORD_Y_HANDLE_COL*0.75f));	
//	result->lines.push_back(Line(1, 4, COORD_Z_HANDLE_COL*0.75f));	
//	return result;
//}

void Handle3D::write(OutputObjectStream *s) const
{
	s->write((mpcard) points.size());
	for( int i1 = 0; i1 < (int)points.size(); ++ i1 ) {
		points[i1].pos.write(s);
		points[i1].col.write(s);
        s->write( points[i1].movable );
		s->write( points[i1].visible );
	}
	s->write((mpcard) lines.size());
	for( int i1 = 0; i1 < (int)lines.size(); ++ i1 ) {
		s->write( lines[i1].ptn1 );
		s->write( lines[i1].ptn2 );
		lines[i1].col.write(s);
	}
	s->write(selected);
	s->write(hover.size());
	for( int i1 = 0; i1 < (int)hover.size(); ++ i1 ) {
		s->write( (bool) hover[i1] );
	}
}

void Handle3D::read(InputObjectStream *s)
{
	points.clear();
	mpcard pSize;
	s->read( pSize );
	for( int i1 = 0; i1 < (int)pSize; ++ i1 ) {
		Vector3f pos; Vector4f col; bool movable; bool visible;
		pos.read(s); col.read(s); s->read(movable); s->read(visible);
		points.push_back(Point(pos,col,movable,visible));
	}

	lines.clear();
	mpcard lSize;
	s->read( lSize );
	for( int i1 = 0; i1 < (int)lSize; ++ i1 ) {
		card32 ptn1, ptn2; Vector4f col;
		s->read(ptn1); s->read(ptn2); col.read(s);
		lines.push_back(Line(ptn1,ptn2,col));
	}

	s->read(selected);

	mpcard hSize;
	s->read( hSize );
	hover.resize(hSize);
	for( int i1 = 0; i1 < (int)hSize; ++ i1 ) {
		s->read((bool&) hover[i1]);
	}
}