//---------------------------------------------------------------------------
#ifndef Handle3DH
#define Handle3DH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "Modifiers.h"
//---------------------------------------------------------------------------

namespace X4 {

class GLContext;


///   mw - sept 24 2007
///
///   This abstract class describes a set of 3d points forming a handle the user can interact with
///
class X3DEFORMATION_API Handle3D : public Persistent {
	X4_ABSTRACT_CLASS( Handle3D )

 protected:
	struct Point {
		Vector3f pos;
		Vector4f col;
        bool movable;
		bool visible;
		Point(Vector3f p, Vector4f c, bool m=true, bool v=true) {pos=p;col=c;movable=m;visible=v;}
	};
	struct Line {
		card32 ptn1;
		card32 ptn2;
		Vector4f col;
		Line(card32 p1, card32 p2, Vector4f c) {ptn1=p1;ptn2=p2;col=c;}
	};
 protected:
	std::vector<Point> points;
	std::vector<Line> lines;
	bool selected;
	std::vector<bool> hover;

	void addPoint(const Point &p);
	void addLine(const Line &l);
	
	static const Vector4f POINT_HANDLE_COL;
	static const Vector4f TO_POINT_HANDLE_COL;
	static const Vector4f LINE_HANDLE_COL;
	static const Vector4f COORD_X_HANDLE_COL;
	static const Vector4f COORD_Y_HANDLE_COL;
	static const Vector4f COORD_Z_HANDLE_COL;
	static const Vector4f BOX_HANDLE_COL;
	static const float32  HANDLE_POINT_SIZE;
	static const float32  HANDLE_LINE_WIDTH;

 public:
	static Handle3D *makePointHandle(const Vector3f &initialPos);
	static Handle3D *makeVectorHandle(const Vector3f &initialPos1, const Vector3f &initialPos2);
	static Handle3D *makeGradHandle(const Vector3f &initialPos, const Matrix3f &initialGrad);
	//static Handle3D *makeVecGradHandle(const Vector3f &refPos, const Vector3f &initialPos, const Matrix3f &initialGrad);
 
	Handle3D();

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	/// get number of active handle points that can be moved around by the user
	virtual mpcard getNumPoints();
	/// get the point in world coordinates
	virtual Vector3f getPoint(card32 index) const;
    /// is the point movable?
    virtual bool getPointMovable(card32 index);
   /// is the point visible? (or just line endpoint)
	virtual bool getPointVisible(card32 index);
	/// draw a handle image using gl
	virtual void glDraw(GLContext *glContext);

	/// select the whole thing
	virtual void select();
	/// mouse is over one of the points, in click-radius
	virtual void hoverPoint(card32 index);
	/// mouse is not over any of the points in click-radius, and the handle is also not selected (anymore)
	virtual void clearHighlights();

	/// a point has been moved to a new position 
	/// override the method to reproject the point according to some constraint
	virtual void movePoint(card32 index, const Vector3f &newPos, Modifiers modifiersState);

   ~Handle3D();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);
};

}

#endif