//---------------------------------------------------------------------------
#ifndef Handle3DHandlerH
#define Handle3DHandlerH
//---------------------------------------------------------------------------
#include "x3deformation.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4 {

class Handle3D;

/// helper interface for communicating changes
class Handle3DUpdatesListener { // INTERFACE CLASS
 public:
	virtual void handleChanged(card32 which, card32 point) = 0;
};


///   mw - sept 24 2007
///
///   this class handles the user interaction for Handle3D objects
///
class X3DEFORMATION_API Handle3DHandler : public Persistent, 
													 public MouseController, public KeyboardController {
	X4_CLASS( Handle3DHandler )
 private:
	std::vector<Handle3D*> *handles;
	std::vector<Handle3DUpdatesListener*> listeners; 
	Matrix4f mvMatrix;
	int32 width;
	int32 height;
	bool selection;
	unsigned selected_h;
	card32 selected_p;
	int32 last_x;
	int32 last_y;
	float32 last_prj_depth;
	Modifiers last_modifiers;
	float32 tanOfHalfViewingAngle;
	Vector3f camViewDir, camRightDir, camUpDir, camPos;

	float32 rayQuery(Vector3f pos, int32 x, int32 y, float32 &depth);
	bool pick(int32 x, int32 y, unsigned &handle, card32 &point); 
	Vector3f reproject(float32 x, float32 y, float32 depth);

 public:
	Handle3DHandler();

	void setHandles(std::vector<Handle3D*> *handles);
	void setParams(const Camera &cam, const ViewFrustum &vf, const Matrix4f &baseTransform);
	void addChangesListener(Handle3DUpdatesListener *x);

   bool selected();
   void getSelection(bool &selected, card32 &handle, card32 &point);
   void deselect();

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);
   virtual void mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState){};

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);
	
	/// this method sets the handler into the state of having just selected (mouse down) the
	/// last handle of the most recently added (end of list) handle
	/// used for example for <CTRL+Click> in PCIDeformationEditor
	void manuallySelectLatest(int x, int y);
	
	/// returns true if a user operation is currently in progress
	/// false if nothing particular is going on
	bool active() {return selection;}
	
	/// returns index of the last handle that has been selected
	/// default is zero (if no selection has been made so far)
	unsigned getLastSelectedHandle();
		
   ~Handle3DHandler();
};

}

#endif