//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "VarianceHandle3D.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( VarianceHandle3D , 0 ) {
	BEGIN_CLASS_INIT( VarianceHandle3D );
}

// ----


const Vector4f VarianceHandle3D::VARIANCE_HANDLE_COL = makeVector4f(0.5f, 0.2f, 0.2f, 1.0f);


VarianceHandle3D::VarianceHandle3D() {
}

VarianceHandle3D::~VarianceHandle3D() {
}




void VarianceHandle3D::addVarianceHandle(Line varHandle) 
{
	varEntries.push_back(varHandle);
}

void VarianceHandle3D::glDraw(GLContext *glContext) 
{
	Handle3D::glDraw(glContext);
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glDisable(GL_LIGHTING);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LINE_SMOOTH);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLineWidth(1);
   glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	GLUquadricObj *quad = gluNewQuadric();
	for (unsigned v=0; v<varEntries.size(); v++) {
		Vector4f col = varEntries[v].col;
		if (selected) {
			col += makeVector4f(0.5f, 0.5f, 0.5f, 0.0f);
		}
		glColor4fv(col.data());
		Vector3f pos = points[ varEntries[v].ptn1 ].pos;
		float32 r = norm(pos-points[ varEntries[v].ptn2 ].pos);
		glTranslatef(pos[0], pos[1], pos[2]);
		gluSphere(quad, r, 8, 8);
		glTranslatef(-pos[0], -pos[1], -pos[2]);
	}	
	
	glPopMatrix();
	glPopAttrib();
	gluDeleteQuadric(quad);
}

VarianceHandle3D *VarianceHandle3D::makeVectorHandle(const Vector3f &initialPos1, const Vector3f &initialPos2,
																	  const float32 initialVariance) 
{
	VarianceHandle3D *result = new VarianceHandle3D();
	result->points.push_back(Point(initialPos1, POINT_HANDLE_COL*0.9f));
	result->points.push_back(Point(initialPos2, POINT_HANDLE_COL*1.1f));
	result->lines.push_back(Line(0, 1, LINE_HANDLE_COL));	
	result->points.push_back(Point(initialPos1+YAXIS_VECTOR3F*initialVariance, POINT_HANDLE_COL*0.6f));
	result->addVarianceHandle(Line(1, 2, VARIANCE_HANDLE_COL));
	
	return result;
}

VarianceHandle3D *VarianceHandle3D::makeGradHandle(const Vector3f &initialPos, const Matrix3f &initialGrad,
															      const Vector3f &initialVariances) 
{
	VarianceHandle3D *result = new VarianceHandle3D();
	result->points.push_back(Point(initialPos, POINT_HANDLE_COL));
	result->points.push_back(Point(initialPos+initialGrad[0], COORD_X_HANDLE_COL));
	result->points.push_back(Point(initialPos+initialGrad[1], COORD_Y_HANDLE_COL));
	result->points.push_back(Point(initialPos+initialGrad[2], COORD_Z_HANDLE_COL));
	result->lines.push_back(Line(0, 1, COORD_X_HANDLE_COL*0.75f));	
	result->lines.push_back(Line(0, 2, COORD_Y_HANDLE_COL*0.75f));	
	result->lines.push_back(Line(0, 3, COORD_Z_HANDLE_COL*0.75f));	
	result->points.push_back(Point(initialPos+initialGrad[0]+YAXIS_VECTOR3F*initialVariances[0], POINT_HANDLE_COL*0.6f));
	result->points.push_back(Point(initialPos+initialGrad[1]+YAXIS_VECTOR3F*initialVariances[1], POINT_HANDLE_COL*0.6f));
	result->points.push_back(Point(initialPos+initialGrad[2]+YAXIS_VECTOR3F*initialVariances[2], POINT_HANDLE_COL*0.6f));
	result->addVarianceHandle(Line(1, 4, VARIANCE_HANDLE_COL));
	result->addVarianceHandle(Line(2, 5, VARIANCE_HANDLE_COL));
	result->addVarianceHandle(Line(3, 6, VARIANCE_HANDLE_COL));
	return result;
}
