#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "AddToToolbarWidget.h"
//---------------------------------------------------------------------------
#include "Configuration.h"
#include <QFileDialog>
#include <QMessageBox>
#include <QPixmap>
#include "SceneInterface.h"
#include <QImage>
//---------------------------------------------------------------------------

using namespace X4;


AddToToolbarWidget::AddToToolbarWidget(QWidget *parent)
:QDialog(parent)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	m_Icon = new RGBAImage(16,16);
	m_Icon->clear( makeVector4ub( 255,0,0,0));
}

AddToToolbarWidget::~AddToToolbarWidget(void)
{

}

void AddToToolbarWidget::setup( std::string className, bool et_object, bool et_command, EEventType init_eventType )
{
	x4Assert( !et_object || init_eventType != Object );
	x4Assert( !et_command || init_eventType != Command );
	x4Assert( et_object || !et_command );

	ui.labelClassName->setText( className.c_str() );
	ui.cbEventType->clear();
	ui.cbEventType->addItem( "Class" );
	if( et_object ) 
		ui.cbEventType->addItem( "Object" );
	if( et_command ) 
		ui.cbEventType->addItem( "Command" );

	ui.leDesc->clear();
	ui.leTooltip->clear();
}

void AddToToolbarWidget::on_pbLoad_clicked()
{
	std::string image_path = getConfigurationString( "IMAGE_PATH" );
	QString filename = QFileDialog::getOpenFileName( this, "load icon", image_path.c_str(), "*.bmp;*.png" );
	if( !filename.isEmpty() )
	{
		QPixmap pixmap;
		if( pixmap.load( filename ) )
		{
			ui.labelIcon->setPixmap( pixmap );
			//m_Icon->loadFromFile( (char *)filename.toLatin1().constData() );
			m_Icon->resize( pixmap.width(), pixmap.height() );
			QImage img = pixmap.toImage();
			QImage alpha = pixmap.alphaChannel().toImage();
			for( unsigned int x=0;x<m_Icon->getSizeX();x++ )
			{
				for( unsigned int y=0;y<m_Icon->getSizeY();y++ )
				{
					QRgb rgb = img.pixel(x,y);
					QRgb a =	alpha.pixel(x,y);
					m_Icon->setPixel( x,y, makeVector4ub(
										(rgb >> 16) & 0xff,
										(rgb >> 8) & 0xff,
										(rgb >> 0) & 0xff, a & 0xff ));
				}
			}
		}
		else
			QMessageBox::information( this, "error", QString("couldn't read file: ") + filename );
	}
}

int AddToToolbarWidget::getEventType()
{
	int ret = -1;
	switch( ui.cbEventType->currentIndex() )
	{
	case 0:
		ret = Class;
		break;
	case 1:
		ret = Object;
		break;
	case 2:
		ret = Command;
		break;
	}
	return ret;
}

RGBAImage * AddToToolbarWidget::getIcon()
{
	return (RGBAImage *)m_Icon->copy();
}


std::string AddToToolbarWidget::getShortDesc()
{
	return ui.leDesc->text().toLatin1().constData();
}


std::string AddToToolbarWidget::getTooltip()
{
	return ui.leTooltip->text().toLatin1().constData();
}

int AddToToolbarWidget::getShortCutKey()
{
	if( ui.leKey->text().isEmpty() )
		return -1;
	QString t = ui.leKey->text();
	QChar c = t[0];
	if( c.isDigit() )
		return Qt::Key_0 + c.digitValue();
	if( c.isSpace() )
		return Qt::Key_Space;
	if( c.isLetter() )
		return Qt::Key_A + c.toLower().toLatin1() - 'a';
	
	return -1;
}

int AddToToolbarWidget::getShortCutModifer()
{
	switch( ui.cbModifer->currentIndex() )
	{
	case 0:
		return SceneToolbarElement::SCM_None;
	case 1:
		return SceneToolbarElement::SCM_Ctrl;
	case 2:
		return SceneToolbarElement::SCM_Alt;
	case 3:
		return SceneToolbarElement::SCM_Shift;
	}

	return SceneToolbarElement::SCM_None;
}

#include "AddToToolbarWidget_moc.h"
