#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "AnimationBarWidget.h"
//---------------------------------------------------------------------------
#include "SceneEditorWidget.h"
#include <QMenuBar>
#include "OnlineHelp.h"
#include "Timer.h"
#include "DebugOutput.h"
#include "Configuration.h"
#include "SceneGraphState.h"
//---------------------------------------------------------------------------

using namespace X4;


AnimationBarWidget::AnimationBarWidget(QWidget *parent)
:QFrame(parent)
{

	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Icons
	//ui.playButton->setIcon(QIcon(QString::fromUtf8("../images/Play.bmp")));
	//ui.stopButton->setIcon(QIcon(QString::fromUtf8("../images/Stop.bmp")));
	//ui.oneBackButton->setIcon(QIcon(QString::fromUtf8("../images/PlayOneBackward.bmp")));
	//ui.oneForwardButton->setIcon(QIcon(QString::fromUtf8("../images/PlayOneFrame.bmp")));

	ui.traversalModeBox->addItem("all frames");
	ui.traversalModeBox->addItem("actual frame");
	ui.traversalModeBox->setCurrentIndex(0);

	//Init Slider
	ui.frameSlider->setMaximum(ui.endBox->value());
	ui.frameSlider->setMinimum(ui.startBox->value());
	ui.frameSlider->setSingleStep(ui.stepBox->value());

	//Init Timer
	timer = new QTimer(this);
	timer->setInterval((1.0f/(float)ui.fpsBox->value())*1000);
	QWidget::connect(timer,SIGNAL(timeout()), this, SLOT(onTimerTick()));
	sliderPosBefore=ui.frameSlider->value();

}

AnimationBarWidget::~AnimationBarWidget(void)
{

}

void AnimationBarWidget::on_frameSlider_valueChanged(int at){
	
	int corrected = at - (at - getStartFrameNumber())%getAnimStep();
	if (corrected!=at)
	{
		ui.frameSlider->setSliderPosition(corrected);
	}

	if (sliderPosBefore!=corrected)
	{
		emit frameChanged(corrected);
		sliderPosBefore=corrected;
	}

}

void AnimationBarWidget::setTime(float32 time) {
	ui.frameSlider->setSliderPosition((int)time - ui.frameSlider->minimum());
}


int AnimationBarWidget::getFrameNumber(){
	return ui.frameSlider->value();
}
void AnimationBarWidget::setFrameNumber(int x){
	on_frameSlider_valueChanged(x);
}

card32 AnimationBarWidget::getStartFrameNumber(){
	return ui.frameSlider->minimum();
}

card32 AnimationBarWidget::getEndFrameNumber(){
	return ui.frameSlider->maximum();
}

int32  AnimationBarWidget::getAnimStep(){
	return ui.frameSlider->singleStep();
}
card32 AnimationBarWidget::getFramesCount(){
	return ((getEndFrameNumber() - getStartFrameNumber()) / getAnimStep())+1;
}

void AnimationBarWidget::updateTraversalModeBox(SGStaticState::TraversalModeType mode) {
	card32 boxMode = mode;
	switch (mode) {
		case SGStaticState::tmAllNodes: boxMode = 0; break;
		case SGStaticState::tmRendering: boxMode = 1; break;
		default: break;
	};
	ui.traversalModeBox->setCurrentIndex(boxMode);
}



void AnimationBarWidget::on_startBox_valueChanged(int v){
	if (v>=ui.endBox->value())
	{
		ui.endBox->setValue(v+1);
	}

	ui.frameSlider->setMinimum(v);
   ui.timeBox->setMinimum(v);
	on_frameSlider_valueChanged(ui.frameSlider->value());
}

void AnimationBarWidget::on_endBox_valueChanged(int v){
	if (v<=ui.startBox->value())
	{
		if (ui.startBox->value()==0)
		{
			ui.endBox->setValue(1);
		}else{
			ui.startBox->setValue(v-1);
		}
	}
   ui.frameSlider->setMaximum(v);
   ui.timeBox->setMaximum(v);
	on_frameSlider_valueChanged(ui.frameSlider->value());
}

void AnimationBarWidget::on_stepBox_valueChanged(int v){
	ui.frameSlider->setSingleStep(v);
	on_frameSlider_valueChanged(ui.frameSlider->value());
}

void AnimationBarWidget::on_playButton_clicked(){
	if (ui.playButton->isChecked())
	{
		timer->start();
	}else{
		timer->stop();
	}

}

void AnimationBarWidget::on_stopButton_clicked(){
	ui.playButton->setChecked(false);
	timer->stop();
}

void AnimationBarWidget::on_oneBackButton_clicked(){
	ui.frameSlider->triggerAction(QAbstractSlider::SliderSingleStepSub);
}

void AnimationBarWidget::on_oneForwardButton_clicked(){
	ui.frameSlider->triggerAction(QAbstractSlider::SliderSingleStepAdd);
}

void AnimationBarWidget::on_fpsBox_valueChanged(int v){
	timer->setInterval((1.0f/(float)v)*1000);
}

void AnimationBarWidget::onTimerTick(){
	if (ui.frameSlider->maximum()<(ui.frameSlider->value()+ui.frameSlider->singleStep()))
	{
		ui.frameSlider->setValue(ui.frameSlider->minimum());
	}else{
		ui.frameSlider->triggerAction(QAbstractSlider::SliderSingleStepAdd);	
	}
}

void AnimationBarWidget::on_traversalModeBox_currentIndexChanged(int mode) {
	SGStaticState::TraversalModeType emitMode = (SGStaticState::TraversalModeType)mode;
	switch (mode) {
		case 0: emitMode = SGStaticState::tmAllNodes; break;
		case 1: emitMode = SGStaticState::tmRendering; break;
		default: break;
	};
	emit traversalModeChanged(emitMode);
}


#include "AnimationBarWidget_moc.h"
