#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "AreaSelectionTool.h"
//---------------------------------------------------------------------------
#include "RGBAImage.h"
#include "GLContext.h"
#include "GLHelper.h"
//---------------------------------------------------------------------------
using namespace X4;


AreaSelectionTool::AreaSelectionTool(void)
{
	m_GLContext = NULL;
	m_SelectionMask = NULL;
	m_SelectionColor = makeVector3f(0.2f,0.5f,1.0f);
	m_SelectionActive = false;
}

AreaSelectionTool::~AreaSelectionTool(void)
{
	delete m_SelectionMask;
}

void AreaSelectionTool::selectPaint(mpcard x, mpcard y, mpcard rad) 
{
	m_SelectionActive = true;
	m_SelectionMask->drawSolidCircle(x,m_SelectionMask->getSizeY()-1-y,
		rad,makeVector4ub(m_SelectionColor[0]*255.0f,m_SelectionColor[1]*255.0f,
		m_SelectionColor[2]*255.0f,128));
}

RGBAImage *AreaSelectionTool::finishedSelection() 
{
	RGBAImage * sel = (RGBAImage*)m_SelectionMask->copy();
	m_SelectionMask->clear(makeVector4ub(0,0,0,0));
	m_SelectionActive = false;
	return sel;
}

void AreaSelectionTool::areaResize(card32 width, card32 height) 
{
	delete m_SelectionMask;
	m_SelectionMask = new RGBAImage(width,height);
	m_SelectionMask->clear(makeVector4ub(0,0,0,0));
}

void AreaSelectionTool::draw(GLContext * context) 
{
	m_GLContext = context;
	if( !m_SelectionActive )
		return;
	
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	
	GLHelper::begin2D(m_SelectionMask->getSizeX(),m_SelectionMask->getSizeY());
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glRasterPos2f(0, 0);
	glDrawPixels(m_SelectionMask->getSizeX(),m_SelectionMask->getSizeY(), GL_RGBA, GL_UNSIGNED_BYTE, m_SelectionMask->getPixels());
	GLHelper::end2D();

	glPopAttrib();

}

void AreaSelectionTool::selectPaintLine(mpcard x, mpcard y, mpcard rad, bool firstClick) 
{
	selectPaint(x,y,rad);
	if( !firstClick )
	{
		float deltaX = ((float)x - (float)m_LastX);
		float deltaY = ((float)y - (float)m_LastY);
		if( deltaX == 0 && deltaY == 0 )
		{
			m_LastX = x;
			m_LastY = y;
			return;
		}
		float maxDelta = max(fabs(deltaX),fabs(deltaY));
		float xf = m_LastX;
		float yf = m_LastY;
		float stepSize = 3.0f;
		mpcard numSteps = 0.5f + (float)(maxDelta / stepSize);

		deltaX /= (float)numSteps;
		deltaY /= (float)numSteps;
		for( mpcard i=0;i<numSteps;i++ )
		{
			selectPaint(xf,yf,rad);
			xf += deltaX;
			yf += deltaY;
		}
	}
	m_LastX = x;
	m_LastY = y;
}
