#ifndef AreaSelectionToolH
#define AreaSelectionToolH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------

namespace X4
{

class GLContext;
class RGBAImage;

/**
	a helper class for painting area selections
	usage (in your PCI-Tool):
		call draw(glContext) in PCI::glDrawTool() -> this also initialized the context
		call areaResize() in PCI::areaResize() to pass screen dimensions
		call selectPaint() in PCI::mouseMoved() whenever you want to select sth
		call finishSelection() in PCI::mouseUp() to retrieve the selection
	
	as an alternative to selectPaint you may use selectPaintLine() to draw a thick line between two points:
		call selectPaintLine(,,,true) in PCI::mouseDown()
		call selectPaintLine(,,,false) in PCI::mouseMoved()
*/
class X3EDITOR_API AreaSelectionTool
{

public:
	AreaSelectionTool(void);
	~AreaSelectionTool(void);

	/// paint solid sphere in selection image (call in PCI::mouseMoved)
	void selectPaint(mpcard x, mpcard y, mpcard rad);

	/// paint a sequence of solid spheres in selection image (thick line), call in PCI::mouseDown and PCI::mouseMoved)
	/// method will draw line between last point and this point unless you set firstClick=true (for PCI::mouseDown)
	void selectPaintLine(mpcard x, mpcard y, mpcard rad, bool firstClick);
	
	/// finish and retrieve selection, returns copy of selection mask (call in PCI::mouseUp)
	RGBAImage * finishedSelection();

	/// pass screen dimensions (call in PCI::areaResize)
	void areaResize(card32 width, card32 height);

	/// draw selection (call in PCI::glDrawTool)
	void draw(GLContext * context);

	bool isActive() const {return m_SelectionActive;}

private:

//	GLuint				m_TextureID;	// gl texture id
	GLContext		*	m_GLContext;
	RGBAImage		*	m_SelectionMask;
	Vector3f			m_SelectionColor;	// for rendering
	bool				m_SelectionActive;	// indicates that selection is not empty
	mpcard			m_LastX,m_LastY;


};

}

#endif