//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "CmdObjDrawOnClick.h"
//---------------------------------------------------------------------------
#include "FixedArrayClassProperty.h"
#include "StringClassProperty.h"
#include "ObjectClassProperty.h"
#include "PCCommand.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "EmbeddedObjectClassProperty.h"

#include "PointCloud.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "PointCloudIterators.h"
#include "IteratorAdaptors.h"
#include "SelectionIteratorAdaptor.h"
#include "GeometricRange.h"
#include "GeometricRangeIteratorAdaptor.h"
#include "UnstructuredInCorePointCloud.h"
#include "VertexDescriptor.h"
#include "ObjectViewsTable.h"
#include "DebugOutput.h"
#include "MHGeoRangeIteratorAdaptor.h"
#include "VertexDescriptor.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "MathTools.h"
#include "MathTools.inline.h"
#include "PointCloudTools.h"
#include "SelectionTools.h"
#include "LinearAlgebra.inline.h"
#include "BooleanClassProperty.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* CmdObjDrawOnClick::CmdObjDrawOnClickMetaClass = NULL;

const MetaClass* CmdObjDrawOnClick::getInstanceClass() const {
	return getClass();
}

Object* CmdObjDrawOnClick::newInstance() {
	return new CmdObjDrawOnClick;
}

const MetaClass* CmdObjDrawOnClick::getClass() {
	if (CmdObjDrawOnClickMetaClass==NULL) {
		CmdObjDrawOnClickMetaClass = new MetaClass("CmdObjDrawOnClick", "no description.",
			newInstance, sizeof(CmdObjDrawOnClick));
	}
	return CmdObjDrawOnClickMetaClass;
}


void CmdObjDrawOnClick::init(const MetaClass *parentClass) {
	((MetaClass*)getClass())->init(parentClass,MetaClass::FIRST_VERSION + 3);  // mb version +3
	CmdObjDrawOnClickMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT(CmdObjDrawOnClick);
	ADD_EMBEDDED_OBJECT_PROP( cam,2, Camera::getClass() );
	SET_USER_EDIT( false );
	ADD_EMBEDDED_OBJECT_PROP( viewFrust,2, ViewFrustum::getClass() );
	SET_USER_EDIT( false );
	ADD_CARD32_PROP(areaWidth ,0);
	ADD_CARD32_PROP(areaHeight ,0);
	ADD_CARD32_PROP(selectMode ,0);
	ADD_CARD32_PROP(tolerance ,0);
	ADD_INT32_PROP(clickX ,0);
	ADD_INT32_PROP(clickY ,0);
	ADD_BOOLEAN_PROP( fade , 0);
	ADD_FLOAT32_FIXED_ARRAY_PROP( m_col, 0 , 3, 1);
	ADD_BOOLEAN_PROP(m_VariableAttributePainting  , 0);
	ADD_STRING_PROP(m_AttributeName , 0 );
	ADD_BOOLEAN_FIXED_ARRAY_PROP(m_Mask,0,4,1);
	ADD_FLOAT32_PROP(m_BrushIntensity  ,0);
	ADD_BOOLEAN_PROP(m_AdditiveBlending  , 0);
	ADD_BOOLEAN_PROP(m_noBackfacing, 3);
}

void CmdObjDrawOnClick::shutdown() {
	delete CmdObjDrawOnClickMetaClass;
}

void CmdObjDrawOnClick::assign(const Object* obj, X4CopyContext *context) {
	CommandObject::assign(obj, context);
	const CmdObjDrawOnClick *tobj = dynamic_cast<const CmdObjDrawOnClick*>(obj);
	x4Assert(tobj != NULL);
	clickList = tobj->clickList;
}

void CmdObjDrawOnClick::write(OutputObjectStream *s) const {
	CommandObject::write(s);
	card32 n = (card32)clickList.size();
	s->write(n);
	for (card32 i=0; i<n; i++) {
		s->write(clickList[i].x);
		s->write(clickList[i].y);
	}
}

void CmdObjDrawOnClick::read(InputObjectStream *s) {
	CommandObject::read(s);
	card32 n;
	s->read(n);
	clickList.resize(n);
	for (card32 i=0; i<n; i++) {
		s->read(clickList[i].x);
		s->read(clickList[i].y);
	}
}



// ----
void CmdObjDrawOnClick::setup(int32 x, int32 y, card32 tol, card32 mode, Camera &ca, ViewFrustum &vfr, card32 w, card32 h, Vector3f color,bool f,
							  bool variableAttributePainting, std::string attributeName, bool mask1,bool mask2,bool mask3,bool mask4,
							  float32 brushIntensity, bool additiveBlending, bool noBackfacing)
{
	clickX = x;
	clickY = y;

	clickPoint p;
	p.x = clickX;
	p.y = clickY;
	clickList.push_back(p);


	tolerance = tol;
	selectMode = mode;
	cam = ca;
	viewFrust = vfr;
	areaWidth = w;
	areaHeight = h;

	m_col=color;
	fade=f;

	m_VariableAttributePainting = variableAttributePainting;
	m_AttributeName = attributeName;
	m_Mask[0] = mask1;
	m_Mask[1] = mask2;
	m_Mask[2] = mask3;
	m_Mask[3] = mask4;
	m_BrushIntensity = brushIntensity;
	m_AdditiveBlending = additiveBlending;
	this->m_noBackfacing = noBackfacing;

};


void CmdObjDrawOnClick::doAction(Object *context) {

	Scene *scene = dynamic_cast<Scene*>(context); x4Assert(scene);
	//   checkSceneAllAttributes(scene); //slow but stable

	{
		SGLegacyPCSceneAdaptor sceneLA(scene);

		for (card32 i=0; i<sceneLA.getNumPointClouds(); i++) {
			std::string pcName = sceneLA.getNameWithPath( i );
			checkAttribute( scene, pcName, "color", 3, VAD::DATA_FORMAT_FLOAT32 );
			checkAttribute( scene, pcName, "flags", 1, VAD::DATA_FORMAT_INT32 );
			checkAttribute( scene, pcName, "position", 3, VAD::DATA_FORMAT_FLOAT32 );
		}
	}

	SGLegacyPCSceneAdaptor sceneLA(scene);

	for (card32 i=0; i<clickList.size(); i++)
	{
		clickX = clickList[i].x;
		clickY = clickList[i].y;

		if (selectMode==0){
			RayConeRange* range = (RayConeRange*)getRayConeRangeFromMouseClick(clickX,clickY,areaWidth,areaHeight,tolerance,&cam,&viewFrust);
			if (range==NULL) return; //nothing selected

			for (card32 i=0; i<sceneLA.getNumPointClouds(); i++)
			{
				PointCloud *pc = sceneLA.getPointCloud(i);
				if (!sceneLA.visible(i)) continue;
				SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL)); // modified - mw - 22.12.04
				PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
				if (newIt == NULL) {
					delete newItSO;
					return;
				}

				range->setTransformation(sceneLA.getState(i)->dynamicState->ctm);
				//IteratorAdaptor *it=new MHGeoRangeIteratorAdaptor(newIt,pc->getDescr(), range);
				IteratorAdaptor *it=getGeoRangeIteratorAdaptor(pc, newIt, range);
				//MHGeoRangeIteratorAdaptor *it=new GeometricRangeIteratorAdaptor(newIt,pc->getDescr(), range);


				bool instancePainting=false;

				//#define INST_PAINT 0
#ifdef INST_PAINT


				SceneObjectIterator *inst1_newIt = getPointCloud(scene,"root/instance1")->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL)); // modified - mw - 22.12.04
				BasicPointCloudIterator *inst1_basicIt = dynamic_cast<BasicPointCloudIterator*>(inst1_newIt);
				ModifyablePointCloudIterator *inst1_modyIt = dynamic_cast<ModifyablePointCloudIterator*>(inst1_newIt);
				PointSet* inst1_ps=inst1_modyIt->lockBlockReadWrite();

				SceneObjectIterator *inst2_newIt = getPointCloud(scene,"root/instance2")->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL)); // modified - mw - 22.12.04
				BasicPointCloudIterator *inst2_basicIt = dynamic_cast<BasicPointCloudIterator*>(inst2_newIt);
				ModifyablePointCloudIterator *inst2_modyIt = dynamic_cast<ModifyablePointCloudIterator*>(inst2_newIt);
				PointSet* inst2_ps=inst2_modyIt->lockBlockReadWrite();
#endif

				AAT POSITION = pc->getAAT("position");
				AAT NORMAL = NULL_AAT;
				if( m_noBackfacing && pc->providesAttribute("normal") )
					NORMAL = pc->getAAT("normal");

				AAT ATTRIBUTE;
				card32 numAttributeComponents = 0;
				if( m_VariableAttributePainting )
				{
					if( !pc->providesAttribute( m_AttributeName ) )
						continue;     // do nothing if attribute doesn't exists
					ATTRIBUTE = pc->getAAT( m_AttributeName );
					numAttributeComponents = pc->getDescr()->getAttribute( m_AttributeName ).numComponents;
				}
				else
				{
					ATTRIBUTE = pc->getAAT("color");
				}


				while (!it->atEnd() )
				{
					bool skip = m_noBackfacing && NORMAL != NULL_AAT && ((it->get3f(NORMAL) * (it->get3f(POSITION)-range->getRayOrigin())) >= 0);
					if( !skip ) {
						if( m_VariableAttributePainting )
						{
							//float32 fading=range->getDistNorm(it->get3f(POSITION));
							switch( numAttributeComponents )
							{
							case 1:
								{
									it->set1f( ATTRIBUTE, m_BrushIntensity );
								}
								break;
							case 2:
								{
									it->set2f( ATTRIBUTE, makeVector2f(m_BrushIntensity, m_BrushIntensity) );
									//it->set2f( ATTRIBUTE, modifyAttribute<float32,2>( it->get2f(ATTRIBUTE), fading ) );
								}
								break;
							case 3:
								{
									it->set3f( ATTRIBUTE,  makeVector3f(m_BrushIntensity, m_BrushIntensity, m_BrushIntensity) );
									//it->set3f( ATTRIBUTE, modifyAttribute<float32,3>( it->get3f(ATTRIBUTE), fading ) );
								}
								break;
							case 4:
								{
									it->set4f( ATTRIBUTE, makeVector4f(m_BrushIntensity, m_BrushIntensity, m_BrushIntensity, m_BrushIntensity) );
									//it->set4f( ATTRIBUTE, modifyAttribute<float32,4>( it->get4f(ATTRIBUTE), fading ) );
								}
								break;
							}
						}
						else
						{
							if (fade ){
								float32 fading=range->getDistNorm(it->get3f(POSITION));
								it->set3f(ATTRIBUTE, blend(it->get3f(ATTRIBUTE),m_col,fading));
								mpint ind= it->getCurrentPointIndex();
#ifdef INST_PAINT
								inst1_ps->set3f(ind, ATTRIBUTE, blend(inst1_ps->get3f(ind, ATTRIBUTE),m_col,fading));
								inst2_ps->set3f(ind, ATTRIBUTE, blend(inst2_ps->get3f(ind, ATTRIBUTE),m_col,fading));
#endif					 
							}
							else
								it->set3f(ATTRIBUTE, m_col);
						}
					}
					it->next();
				}
				delete it;
				delete newIt;

#ifdef INST_PAINT

				delete inst1_newIt;
				delete inst2_newIt;

#endif
			}; // end for

			delete range;

		} // if mode

		if (selectMode==1){
			SphereRange* range = (SphereRange*)getNearestSphereRangeFromMouseClick(clickX,clickY,areaWidth,areaHeight,tolerance,&cam,&viewFrust,scene);
			if (range==NULL) return; //nothing selected

			float32 r2=range->getRadius();
			r2=r2*r2; //sqrRad is used frequently

			for (card32 i=0; i<sceneLA.getNumPointClouds(); i++)
			{
				PointCloud *pc = sceneLA.getPointCloud(i);


				SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL));
				PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
				if (newIt == NULL) {
					delete newItSO;
					return;
				}
				range->setTransformation(sceneLA.getState(i)->dynamicState->ctm);
				IteratorAdaptor *it=getGeoRangeIteratorAdaptor(pc, newIt, range);

				AAT POSITION = pc->getAAT("position");
				AAT ATTRIBUTE;
				card32 numAttributeComponents = 0;
				if( m_VariableAttributePainting )
				{
					if( !pc->providesAttribute( m_AttributeName ) )
						continue;     // do nothing if attribute doesn't exists
					ATTRIBUTE = pc->getAAT( m_AttributeName );
					numAttributeComponents = pc->getDescr()->getAttribute( m_AttributeName ).numComponents;
				}
				else
				{
					ATTRIBUTE = pc->getAAT("color");
				}


				Vector3f tCenter=range->getTransCenter();
				while (!it->atEnd() )
				{
					if( m_VariableAttributePainting )
					{
						float32 fading=normQuad(tCenter - it->get3f(POSITION))/r2;
						switch( numAttributeComponents )
						{
						case 2:
							{
								it->set2f( ATTRIBUTE, modifyAttribute<float,2>( it->get2f(ATTRIBUTE), fading ) );
							}
							break;
						case 3:
							{
								it->set3f( ATTRIBUTE, modifyAttribute<float,3>( it->get3f(ATTRIBUTE), fading ) );
							}
							break;
						case 4:
							{
								it->set4f( ATTRIBUTE, modifyAttribute<float,4>( it->get4f(ATTRIBUTE), fading ) );
							}
							break;
						}
					}
					else
						if (fade ){
							float32 fading=normQuad(tCenter - it->get3f(POSITION))/r2;
							it->set3f(ATTRIBUTE, blend(it->get3f(ATTRIBUTE),m_col,fading));
						}
						else
							it->set3f(ATTRIBUTE, m_col);
					it->next() ;
				}
				delete it;
				delete newIt;

			}; // end for

			delete range;

		} // end if mode  
	}; // end for clicklist
}



void CmdObjDrawOnClick::undoAction(Object *context) {
	forget(context);
	throw ECannotUndo("CmdObjDrawOnClick does not provide any undo.");
}


bool CmdObjDrawOnClick::canUndo() {
	return false;
}


std::string CmdObjDrawOnClick::getDescription() {
	std::string result;
	result = "draw ";
	for (card32 i=0; i<clickList.size(); i++) {
		result += " (";
		char buff[1000];
		std::string sX = itoa(clickList[i].x,buff,10);
		result += sX;
		result += ",";
		std::string sY = itoa(clickList[i].y,buff,10);
		result += sY;
		result += ")";
	}
	return result;
}




bool CmdObjDrawOnClick::mergeCommand(const CommandObject *nextCmd) {
	if (nextCmd->getInstanceClass() == CmdObjDrawOnClick::getClass()) {
		const CmdObjDrawOnClick *nextCmdMyType = dynamic_cast<const CmdObjDrawOnClick*>(nextCmd);
		x4Assert(nextCmdMyType != NULL);

		if ((this->m_col==nextCmdMyType->m_col) &&
			(this->tolerance==nextCmdMyType->tolerance) &&
			(this->selectMode==nextCmdMyType->selectMode) &&
			(this->cam==nextCmdMyType->cam) &&
			(this->viewFrust==nextCmdMyType->viewFrust) &&
			(this->areaWidth==nextCmdMyType->areaWidth) &&
			(this->areaHeight==nextCmdMyType->areaHeight) &&
			(this->fade==nextCmdMyType->fade))
		{
			clickPoint p;
			p.x = nextCmdMyType->clickX;
			p.y = nextCmdMyType->clickY;
			this->clickList.push_back(p);
			return true;
		}
		else return false;

	} else {
		return false;
	}
}

///----






