//---------------------------------------------------------------------------
#ifndef CmdObjDrawOnClickH
#define CmdObjDrawOnClickH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
#include "PointCloud.h"
#include "ViewFrustum.h"
#include "Camera.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace X4 {


class X3EDITOR_API CmdObjDrawOnClick : public CommandObject {

 private:
   static MetaClass *CmdObjDrawOnClickMetaClass;
   static Object* newInstance();

   struct clickPoint {
      int32 x;
      int32 y;
   };


 protected:
   std::vector<clickPoint> clickList;

   int32 clickX, clickY;
   card32 tolerance, selectMode, areaWidth, areaHeight;

   Camera cam;
   ViewFrustum viewFrust;
   Vector3f m_col;
   bool fade;

   // ------- changes mb: added variable attribute + component-mask ----------------------
   // monochrome painting only ( TODO :-) )
   bool     m_VariableAttributePainting;  /// activates this modus
   std::string   m_AttributeName;
   bool     m_Mask[4];           /// works like OpenGL's colormask - false = don't modify this component
   float32  m_BrushIntensity;    /// scales brush ( usefull when paintng "position" )
   bool     m_AdditiveBlending;  /// new value = old value + brushIntensity

   template<class FloatType, int numComponents>
   StaticVector<FloatType,numComponents>
      modifyAttribute( const StaticVector<FloatType,numComponents> &attrib , float32 dist)
   {
      StaticVector<FloatType,numComponents> brushVector;
      StaticVector<FloatType,numComponents> ret;
      for( card32 i=0;i<numComponents;i++ )
         brushVector[i] = m_BrushIntensity;
      if( fade )
      {
         dist = max(0.0f, min(1.0f, dist ) );
         if( m_AdditiveBlending )
            ret = attrib + brushVector * (1.0f - dist );
         else
            ret = attrib * dist + brushVector * (1.0f - dist );
      }
      else
      {
         if( m_AdditiveBlending )
            ret = attrib + brushVector;
         else
            ret = brushVector;
      }

      for( card32 i=0;i<numComponents;i++ )
      {
         ret[i] = m_Mask[i] ? ret[i] : attrib[i];
      }
      return ret;

   }

   bool m_noBackfacing;
   // -------------------------------------------------------------------------------------


 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual void setup(int32 x, int32 y, card32 tol, card32 mode, Camera &ca, ViewFrustum &vfr, card32 w, card32 h, Vector3f color, bool fade,
      bool variableAttributePainting, std::string attributeName, bool mask1,bool mask2,bool mask3,bool mask4,
      float32 brushIntensity, bool additiveBlending, bool noBackfacing);


   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();

   virtual bool mergeCommand(const CommandObject *nextCmd);

};


}//ns

#endif
