//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "CmdObjFilter.h"
//---------------------------------------------------------------------------
#include "FixedArrayClassProperty.h"
#include "StringClassProperty.h"
#include "ObjectClassProperty.h"
#include "PCCommand.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "EmbeddedObjectClassProperty.h"
#include "PointCloud.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "PointCloudIterators.h"
#include "IteratorAdaptors.h"
#include "SelectionIteratorAdaptor.h"
#include "GeometricRange.h"
#include "GeometricRangeIteratorAdaptor.h"
#include "UnstructuredInCorePointCloud.h"
#include "VertexDescriptor.h"
#include "ObjectViewsTable.h"
#include "DebugOutput.h"
#include "HierarchicalKNNIterator.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "PointCloudTools.h"
#include "SelectionTools.h"
//---------------------------------------------------------------------------
#include <algorithm>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* CmdObjFilter::CmdObjFilterMetaClass = NULL;

const MetaClass* CmdObjFilter::getInstanceClass() const {
   return getClass();
}

Object* CmdObjFilter::newInstance() {
   return new CmdObjFilter;
}

const MetaClass* CmdObjFilter::getClass() {
   if (CmdObjFilterMetaClass==NULL) {
      CmdObjFilterMetaClass = new MetaClass("CmdObjFilter", "Filter.",
                                  newInstance, sizeof(CmdObjFilter));
   }
   return CmdObjFilterMetaClass;
}


void CmdObjFilter::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 3);
   CmdObjFilterMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(CmdObjFilter)
	ADD_EMBEDDED_OBJECT_PROP( cam,3, Camera::getClass() );
	SET_USER_EDIT( false );
	ADD_EMBEDDED_OBJECT_PROP( viewFrust,3, ViewFrustum::getClass() );
	SET_USER_EDIT( false );
	ADD_CARD32_PROP(areaWidth ,0)
		 ADD_CARD32_PROP(areaHeight ,0)
		 ADD_CARD32_PROP(mode ,0)
		 ADD_CARD32_PROP(channel ,0)
		 ADD_CARD32_PROP(tolerance ,0)
		 ADD_CARD32_PROP(strength ,0)
		 ADD_INT32_PROP(clickX ,0)
		 ADD_INT32_PROP(clickY ,0)
		 ADD_BOOLEAN_PROP( drawScene , 0)
		 ADD_BOOLEAN_PROP(use_kNN , 0)
		 ADD_CARD32_PROP(kNN_numPoints ,0)
		 ADD_BOOLEAN_PROP(use_Heightmap , 0)


// *** clickList extra behandelt, siehe assign/write/read

}

void CmdObjFilter::shutdown() {
   delete CmdObjFilterMetaClass;
}

void CmdObjFilter::assign(const Object* obj, X4CopyContext *context) {
   CommandObject::assign(obj, context);
   const CmdObjFilter *tobj = dynamic_cast<const CmdObjFilter*>(obj);
   x4Assert(tobj != NULL);
   clickList = tobj->clickList;
}

void CmdObjFilter::write(OutputObjectStream *s) const {
   CommandObject::write(s);
   card32 n = (card32)clickList.size();
   s->write(n);
   for (card32 i=0; i<n; i++) {
      s->write(clickList[i].x);
      s->write(clickList[i].y);
   }
}

void CmdObjFilter::read(InputObjectStream *s) {
   CommandObject::read(s);
   card32 n;
   s->read(n);
   clickList.resize(n);
   for (card32 i=0; i<n; i++) {
      s->read(clickList[i].x);
      s->read(clickList[i].y);
   }
}



// ----

void CmdObjFilter::setup(const int32 &x, const int32 &y, const card32 &tol, const card32 &m, const card32 &ch,
                      Camera &ca, ViewFrustum &vfr, const card32 &w, const card32 &h, const float32 &stre,
                      const bool &scn, const bool &use_kNN_it, const card32 &kNN_numPoints_var, const bool &use_Height)
{
  clickX = x;
  clickY = y;

  clickPoint p;
  p.x = clickX;
  p.y = clickY;
  clickList.push_back(p);


  tolerance = tol;
  mode = m;
  channel = ch;
  cam = ca;
  viewFrust = vfr;
  areaWidth = w;
  areaHeight = h;

  strength=stre;               
  drawScene=scn;

  use_kNN=use_kNN_it;
  kNN_numPoints=kNN_numPoints_var;

  use_Heightmap= use_Height;

};


void CmdObjFilter::filterHightField(Scene *scene){
     SGLegacyPCSceneAdaptor sceneLA(scene);

     //Select Points with GeometricRange Iterator
     //Filter Points in inner Range with Points from outer range
     SphereRange* innerRange = (SphereRange*) getNearestSphereRangeFromMouseClick(clickX,clickY,areaWidth,areaHeight,tolerance,&cam,&viewFrust,scene);
     if (innerRange==NULL) return; //nothing is in Range
     SphereRange* outerRange = new SphereRange(innerRange->getCenter(),1);
     //Both ranges must have same center, outerRange has double size
     outerRange->setRadius(2.0f*innerRange->getRadius());

     //  float32 sqrRad=innerRange->getRadius() * innerRange->getRadius();

     card32 i=0;
     for (i=0; i<sceneLA.getNumPointClouds(); i++) //for all Clouds
     {
        Matrix4f ctm = sceneLA.getState(i)->dynamicState->ctm;

        //copy outerRange Points to temp Cloud
        PointCloud *pc = sceneLA.getPointCloud(i);
        if (!sceneLA.visible(i)) continue;
        innerRange->setTransformation(ctm);
        outerRange->setTransformation(ctm);

        SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL));
        PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
        if (newIt == NULL) {delete newItSO; continue;}

        IteratorAdaptor *outerIter = getGeoRangeIteratorAdaptor(pc, newIt, outerRange);
        IteratorAdaptor *innerIter = getGeoRangeIteratorAdaptor(pc, newIt, innerRange);


        AAT POSITION = pc->getAAT("position");
        AAT COLOR    = pc->getAAT("color");
        AAT NORMAL   = pc->getAAT("normal");
        AAT FLAGS   = pc->getAAT("flags");

        //for every Point in outer Range copy Attribs to tempCloud
        tempCloud=new UnstructuredInCorePointCloud();
        tempCloud->clearAndSetup(pc->getDescr(), 0);
        tmpIt = tempCloud->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PC));
        DynamicPointCloudIterator *tmpDynIt = dynamic_cast<DynamicPointCloudIterator*>(tmpIt);
        x4Assert (tmpDynIt!=NULL);
        char *buffer = new char[pc->getDescr()->getSize()];

        card32 tstC=0;
        while (!outerIter->atEnd() )  // for each Point in selection
        {
           Vector3f test=outerIter->get3f(COLOR);

           outerIter->getVertex(buffer);
           tmpDynIt->add(buffer);
           outerIter->next();
           tstC++;
        }



        AAT TMP_POSITION = pc->getAAT("position");
        AAT TMP_COLOR    = pc->getAAT("color");
        AAT TMP_NORMAL   = pc->getAAT("normal");
        //End - for every Point in outer Range copy Attribs to tempCloud


         kNN_Iterator_TempCloud = new HierarchicalKNNIterator(tempCloud, 32 , NULL);
         x4Assert(kNN_Iterator_TempCloud);



        //======= make Plane
        hightField.clear();

         // Calc Local Coord System on Cloud

       AllPointsIteratorAdaptor *allItTmp =new AllPointsIteratorAdaptor(tmpIt, tempCloud->getDescr());

        //Find Covariance and Mean for Selected Points
       card32 count;
       Matrix3f eigenVec;

       Vector3f eigenVal;

       Matrix3f cov=createCovarianceAndMean(tempCloud,  allItTmp, mean, count, ctm);
       if (count >=3) { //Min 3 Pts selected
            cov.computeEigenStructure( eigenVal, eigenVec );
            //eigenVec= createEigenVec(cov, eigenVal);
       } else{
         //Less than 3 Pts selected - no Est. possible
         //localValid=false;
         continue;
       }

       //get u,v,n from Matrix3f
       u=normalize(eigenVec[0]);
       v=normalize(eigenVec[1]);
       n=normalize(eigenVec[2]);


        /// ctm beachten
        inv = invertMatrix(ctm);


        Matrix3f ctm3f;  //3f transpose
        for(int k=0; k<3; k++)
          for(int l=0; l<3; l++){
             ctm3f[k][l]=ctm[l][k];
          }


        allItTmp->reset();
        //Normale n umkehren, falls sie falsch liegt
        Vector3f firstNormal;
        if (!allItTmp->atEnd())firstNormal=allItTmp->get3f(NORMAL);
        float32 dir=firstNormal * (ctm3f*n);    //ctm beachtewn
        if(dir>0.0f){
           n= -n;
           debugOutput << "flip n "  <<n;
           debugOutput.cr();
        }else{
           v= -v;
        }


        allItTmp->reset();
       //hoehenfeld speichern
        while (!allItTmp->atEnd() )  // for each Point in range
        {



           // transform to local coords
           Vector4f hPos = expand3To4(allItTmp->get3f(POSITION));
           hPos = ctm*hPos;
           float32 w = hPos[3];
           if (fabs(w) > 0.000001f) {
             Vector3f p = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);


              Vector3f c; // Erg
              c[0]=(mean-p)* u;
              c[1]=(mean-p)* v;
              c[2]=(mean-p)* n;


              hightField.push_back(c[2]);

             /* it->set3f(POSITION,  transformVector3f(inv,
                            p-n*colTex->getZ())        );
                        */
          } // if w > 0.00001
           allItTmp->next() ;
        } //while next Point in Selection



        delete allItTmp;
        //======= make Plane & get hightfield



        switch( channel )
        {
           case 0:
              ATTRIB = pc->getAAT("color");
           break;
           case 1:
              ATTRIB = pc->getAAT("position");
           break;
           case 2:
              ATTRIB = pc->getAAT("normal");
           break;
        }


     //   MedianFilter3f avFilter(ATTRIB,  tempCloud, tmpIt);

       // float32 intensity=innerRange->getRadius()*strength;

        //Filter every Point in inner Range
        innerIter->reset();


        while (!innerIter->atEnd())
        {


          switch( mode )
          {
             case 0:
                averageFilter1f(innerIter);
             break;
             case 1:
                medianFilter1f(innerIter);
             break;
             case 2:
                gaussFilter1f(innerIter);
             break;
          }

           innerIter->next();
        }

       innerIter->reset();
       regenerateNormals(innerIter, pc);

       delete innerIter;
       delete outerIter;
       delete tmpIt;
       delete tempCloud;
       delete newIt;
     }//every cloudnewIt

     delete innerRange;
     delete outerRange;               
}

void CmdObjFilter::regenerateNormals(IteratorAdaptor *it, PointCloud * pc){
   //it->reset();

      /*const*/ card32 m_maxNumPoints=20;
      Vector3f* position=new Vector3f[m_maxNumPoints];
      AAT POSITION= pc->getAAT("position");
      AAT NORMAL  = pc->getAAT("normal");

      HierarchicalKNNIterator* nni=new HierarchicalKNNIterator(pc, 32, NULL);


      while (!it->atEnd())
      {
         Vector3f seekPoint=it->get3f(POSITION);
         Vector3f oldNormal=it->get3f(NORMAL);
         Vector3f pos=NULL_VECTOR3F;
         Vector3f mean=pos;

         nni->setSeekPointAndReset(seekPoint);

         card32 max_points=0;
         //calculate mean, buffer points for covariance calculation

         while(max_points<m_maxNumPoints && !nni->atEnd()){
            pos=nni->get3f(POSITION);
            nni->next();

            mean+=pos;
            position[max_points]=pos;
            max_points++;
         }

         if (max_points!=0)
            mean/=(float) max_points;
         Matrix3f sigma;
         float covXX ; float covXY ; float covXZ ;
         float covYY ; float covYZ ; float covZZ ;
         //calculate covariance based on buffered points
         for (int i=0;i<(int)max_points;i++)
         {
            pos=position[i];
            covXX=(pos[0]-mean[0])*(pos[0]-mean[0]);
            covXY=(pos[0]-mean[0])*(pos[1]-mean[1]);
            covXZ=(pos[0]-mean[0])*(pos[2]-mean[2]);
            covYY=(pos[1]-mean[1])*(pos[1]-mean[1]);
            covYZ=(pos[1]-mean[1])*(pos[2]-mean[2]);
            covZZ=(pos[2]-mean[2])*(pos[2]-mean[2]);

            sigma[0]+=makeVector3f(covXX,covXY,covXZ);
            sigma[1]+=makeVector3f(covXY,covYY,covYZ);
            sigma[2]+=makeVector3f(covXZ,covYZ,covZZ);
         }

         if (max_points!=0)
            sigma/=max_points;

         Vector3f eigenVal;
         Matrix3f eigenVec;
         sigma.computeEigenStructure( eigenVal, eigenVec );
         ///old: Matrix3f eigenVec =createEigenVec(sigma, eigenVal);

         Vector3f normal=normalize(eigenVec[2]);

         if (((normal*(oldNormal)) < 0.0f))
            normal= -normal;


         it->set3f(NORMAL, normal);

         it->next();
      }
      
      delete nni;
      //delete newIt2;
      delete[] position;





}


void CmdObjFilter::doAction(Object *context) {

   Scene *scene = dynamic_cast<Scene*>(context); x4Assert(scene);

   checkSceneAllAttributes(scene); //slow but stable



  if (drawScene) filterScene(scene);
  else

  for (card32 i=0; i<clickList.size(); i++)
   {
      clickX = clickList[i].x;
      clickY = clickList[i].y;

      if (use_Heightmap){
         filterHightField(scene);
         continue;
      }
     //Select Points with GeometricRange Iterator
     //Filter Points in inner Range with Points from outer range
     SphereRange* innerRange = (SphereRange*) getNearestSphereRangeFromMouseClick(clickX,clickY,areaWidth,areaHeight,tolerance,&cam,&viewFrust,scene);
     if (innerRange==NULL) return; //nothing is in Range
     SphereRange* outerRange = new SphereRange(innerRange->getCenter(),1);
     //Both ranges must have same center, outerRange has double size
     outerRange->setRadius(2.0f*innerRange->getRadius());

   //  float32 sqrRad=innerRange->getRadius() * innerRange->getRadius();


     SGLegacyPCSceneAdaptor sceneLA(scene);
     for (i=0; i<sceneLA.getNumPointClouds(); i++) //for all Clouds
     {
        //copy outerRange Points to temp Cloud
        PointCloud *pc = sceneLA.getPointCloud(i);
        if (!sceneLA.visible(i)) continue;
        innerRange->setTransformation(sceneLA.getState(i)->dynamicState->ctm);
        outerRange->setTransformation(sceneLA.getState(i)->dynamicState->ctm);

        SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL));
        PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
        if (newIt == NULL) {delete newItSO; continue;}

        IteratorAdaptor *outerIter = getGeoRangeIteratorAdaptor(pc, newIt, outerRange);
        IteratorAdaptor *innerIter = getGeoRangeIteratorAdaptor(pc, newIt, innerRange);


        AAT POSITION = pc->getAAT("position");
        AAT COLOR    = pc->getAAT("color");
        AAT NORMAL   = pc->getAAT("normal");
        AAT FLAGS   = pc->getAAT("flags");

        //for every Point in outer Range copy Attribs to tempCloud
        tempCloud=new UnstructuredInCorePointCloud();
        tempCloud->clearAndSetup(pc->getDescr(), 0);
        tmpIt = tempCloud->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PC));
        DynamicPointCloudIterator *tmpDynIt = dynamic_cast<DynamicPointCloudIterator*>(tmpIt);
        x4Assert (tmpDynIt!=NULL);
        char *buffer = new char[pc->getDescr()->getSize()];

        card32 tstC=0;
        while (!outerIter->atEnd() )  // for each Point in selection
        {
           Vector3f test=outerIter->get3f(COLOR);

           outerIter->getVertex(buffer);
           tmpDynIt->add(buffer);
           outerIter->next();
           tstC++;
        }



        AAT TMP_POSITION = pc->getAAT("position");
        AAT TMP_COLOR    = pc->getAAT("color");
        AAT TMP_NORMAL   = pc->getAAT("normal");
        //End - for every Point in outer Range copy Attribs to tempCloud

        if (use_kNN){
         kNN_Iterator_TempCloud = new HierarchicalKNNIterator(tempCloud, 32 , NULL);
         x4Assert(kNN_Iterator_TempCloud);
        }

        switch( channel )
        {
           case 0:
              ATTRIB = pc->getAAT("color");
           break;
           case 1:
              ATTRIB = pc->getAAT("position");
           break;
           case 2:
              ATTRIB = pc->getAAT("normal");
           break;
        }


     //   MedianFilter3f avFilter(ATTRIB,  tempCloud, tmpIt);

        float32 intensity=innerRange->getRadius()*strength;

        //Filter every Point in inner Range
        innerIter->reset();
        while (!innerIter->atEnd())
        {

          Vector3f t= innerIter->get3f(TMP_POSITION);
          switch( mode )
          {
             case 0:
                averageFilter3f(innerIter, intensity);
             break;
             case 1:
                medianFilter3f(innerIter, intensity);
             break;
             case 2:
                gaussFilter3f(innerIter, intensity);
             break;
          }

           innerIter->next();
        }

        //Normalize if needed
        if(channel==2){ //normals channel
           innerIter->reset();
           while (!innerIter->atEnd())
           {
              innerIter->set3f(ATTRIB,normalize(innerIter->get3f(ATTRIB)));
              innerIter->next();
           }
        }


       delete innerIter;
       delete outerIter;
       delete tmpIt;
       delete tempCloud;
       delete newIt;
     }//every cloudnewIt

     delete innerRange;
     delete outerRange;




   }; // end for clicklist
}


void CmdObjFilter::filterScene(Scene *scene){

     card32 i=0;
     SGLegacyPCSceneAdaptor sceneLA(scene);
     for (i=0; i<sceneLA.getNumPointClouds(); i++) //for all Clouds
     {
        //copy outerRange Points to temp Cloud
        PointCloud *pc = sceneLA.getPointCloud(i);

        SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
        PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
        if (newIt == NULL) {delete newItSO; continue;}

        AllPointsIteratorAdaptor *allIt = new AllPointsIteratorAdaptor(newIt,pc->getDescr());

        AAT POSITION = pc->getAAT("position");
        AAT COLOR    = pc->getAAT("color");
        AAT NORMAL   = pc->getAAT("normal");
        AAT FLAGS   = pc->getAAT("flags");

        //for every Point in outer Range copy Attribs to tempCloud     //TODO: needen Attribs ONLY
        tempCloud=new UnstructuredInCorePointCloud();
        tempCloud->clearAndSetup(pc->getDescr(), 0);
        tmpIt = tempCloud->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
        DynamicPointCloudIterator *tmpDynIt = dynamic_cast<DynamicPointCloudIterator*>(tmpIt);
        x4Assert (tmpDynIt!=NULL);
        char *buffer = new char[pc->getDescr()->getSize()];

        card32 tstC=0;
        while (!allIt->atEnd() )  // for each Point in selection
        {
           allIt->getVertex(buffer);
           tmpDynIt->add(buffer);
           allIt->next();
           tstC++;
        }



        AAT TMP_POSITION = pc->getAAT("position");
        AAT TMP_COLOR    = pc->getAAT("color");
        AAT TMP_NORMAL   = pc->getAAT("normal");
        //End - for every Point in outer Range copy Attribs to tempCloud

        if (use_kNN){
         kNN_Iterator_TempCloud = new HierarchicalKNNIterator(tempCloud, 32 , NULL);
         x4Assert(kNN_Iterator_TempCloud);
        }

        switch( channel )
        {
           case 0:
              ATTRIB = pc->getAAT("color");
           break;
           case 1:
              ATTRIB = pc->getAAT("position");
           break;
           case 2:
              ATTRIB = pc->getAAT("normal");
           break;
        }


     //   MedianFilter3f avFilter(ATTRIB,  tempCloud, tmpIt);

        float32 intensity=strength;

        //Filter every Point in inner Range
        allIt->reset();
        while (!allIt->atEnd())
        {

          switch( mode )
          {
             case 0:
                averageFilter3f(allIt, intensity);
             break;
             case 1:
                medianFilter3f(allIt, intensity);
             break;
             case 2:
                gaussFilter3f(allIt, intensity);
             break;
          }
           allIt->next();
        }



       delete tmpIt;
       delete tempCloud;
       delete newIt;
     }//every cloudnewIt



}



void CmdObjFilter::averageFilter3f(IteratorAdaptor *it, float32 radius){
   AAT POSITION=tempCloud->getAAT("position");
   Vector3f center=it->get3f(POSITION);
   // get Points in Sphere around this Point
   SphereRange filterRange(center, radius*radius);// Constructor wants sqrRad
   // ?... filterRange.setTransformation(scene->getRenderState(i).ctm);

   GeometricRangeIteratorAdaptor* filterIter=NULL;
   if (!use_kNN){
      filterIter=new GeometricRangeIteratorAdaptor(tmpIt,tempCloud->getDescr(), &filterRange);
   }else
      kNN_Iterator_TempCloud->setSeekPointAndReset(center);


   //filter func
   Vector3f medVal=makeVector3f(0.0f, 0.0f, 0.0f);
   card32 count=0;

   if (!use_kNN){  //use geoRange
      while (!filterIter->atEnd())
      {
         medVal+=filterIter->get3f(ATTRIB);
         count++;
         filterIter->next();
      }
   }else{   //use knn
      while (!kNN_Iterator_TempCloud->atEnd() && count< kNN_numPoints )
      {
         medVal+=kNN_Iterator_TempCloud->get3f(ATTRIB);
         count++;
         kNN_Iterator_TempCloud->next();
      }
   }

   if(count!=0){
      medVal/=(float)count;
      it->set3f(ATTRIB, medVal );
   }else{
      //do nothing for this Point
   }
   //filter func


   delete filterIter;

}

void CmdObjFilter::medianFilter3f(IteratorAdaptor *it, float32 radius){
   AAT POSITION=tempCloud->getAAT("position");
   Vector3f center=it->get3f(POSITION);
   // get Points in Sphere around this Point
   SphereRange filterRange(center, radius*radius);// Constructor wants sqrRad

   GeometricRangeIteratorAdaptor* filterIter=NULL;
   if (!use_kNN){
      filterIter=new GeometricRangeIteratorAdaptor(tmpIt,tempCloud->getDescr(), &filterRange);
   }else
      kNN_Iterator_TempCloud->setSeekPointAndReset(center);


   //filter func
   card32 count=0;

   std::vector<float32> vx;
   std::vector<float32> vy;
   std::vector<float32> vz;

   if (!use_kNN){  //use geoRange
      while (!filterIter->atEnd())
      {
         Vector3f p =filterIter->get3f(ATTRIB);
         vx.push_back(p[0]);
         vy.push_back(p[1]);
         vz.push_back(p[2]);

         count++;
         filterIter->next();
      }
   }else{   //use knn
      while (!kNN_Iterator_TempCloud->atEnd() && count< kNN_numPoints )
      {
         Vector3f p = kNN_Iterator_TempCloud->get3f(ATTRIB);
         vx.push_back(p[0]);
         vy.push_back(p[1]);
         vz.push_back(p[2]);

         count++;
         kNN_Iterator_TempCloud->next();
      }
   }




    if(count!=0){
       sort(vx.begin(), vx.end());
       sort(vy.begin(), vy.end());
       sort(vz.begin(), vz.end());
       card32 median=count/2;
       it->set3f(ATTRIB, makeVector3f(vx[median], vy[median], vz[median]));
   }else{
      //do nothing for this Point
   }
   //filter func

   delete filterIter;

}

void CmdObjFilter::gaussFilter3f(IteratorAdaptor *it, float32 radius){
   AAT POSITION=tempCloud->getAAT("position");
   Vector3f center=it->get3f(POSITION);
   // get Points in Sphere around this Point
   SphereRange filterRange(center, radius*radius);// Constructor wants sqrRad
   // ?... filterRange.setTransformation(scene->getRenderState(i).ctm);

   GeometricRangeIteratorAdaptor* filterIter=NULL;
   if (!use_kNN){
      filterIter=new GeometricRangeIteratorAdaptor(tmpIt,tempCloud->getDescr(), &filterRange);
   }else
      kNN_Iterator_TempCloud->setSeekPointAndReset(center);


   //filter func
   Vector3f medVal=makeVector3f(0.0f, 0.0f, 0.0f);
   float32 weightSum=0.0f;
   card32 count=0;



   if (!use_kNN){  //use geoRange
      while (!filterIter->atEnd())
      {
         Vector3f pos=filterIter->get3f(POSITION);
         float32 dist=normQuad(pos-center);
         if (dist > 0.00000001f) dist=sqrt(dist);
         float weight=exp(- 2* dist/radius);

         medVal+=filterIter->get3f(ATTRIB)*weight;
         weightSum+=weight;
         count++;
         filterIter->next();
      }
   }else{   //use knn
      while (!kNN_Iterator_TempCloud->atEnd() && count< kNN_numPoints )
      {
         Vector3f pos=kNN_Iterator_TempCloud->get3f(POSITION);
         float32 dist=normQuad(pos-center);
         if (dist > 0.00000001f) dist=sqrt(dist);
         float weight=exp(- 2* dist/radius);

         medVal+=kNN_Iterator_TempCloud->get3f(ATTRIB)*weight;
         weightSum+=weight;
         count++;
         kNN_Iterator_TempCloud->next();
      }
   }

   if(count!=0){
      medVal/=weightSum;
      it->set3f(ATTRIB, medVal );
   }else{
      //do nothing for this Point
   }
   //filter func

   delete filterIter;

}



void CmdObjFilter::averageFilter1f(IteratorAdaptor *it){

   AAT POSITION=tempCloud->getAAT("position");
   Vector3f center=it->get3f(POSITION);
   kNN_Iterator_TempCloud->setSeekPointAndReset(center);

   //filter func
   float32 medVal= 0.0f;
   card32 count=0;

      while (!kNN_Iterator_TempCloud->atEnd() && count< kNN_numPoints )
      {
         card32 index= kNN_Iterator_TempCloud->getPointSetPointNumber();
         medVal+=hightField[index];
         count++;
         kNN_Iterator_TempCloud->next();
      }

   if(count!=0){
   // transform to local coords
     float32 hight;
     Vector4f hPos = expand3To4(center);
     hPos = ctm*hPos;
     float32 w = hPos[3];
     Vector3f p;
     if (fabs(w) > 0.000001f) {
       p = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);
       hight =(mean-p)* n;
     } // if w > 0.00001

      medVal/=(float)count;
      Vector3f newPos=p - n*(medVal-hight);
      it->set3f(POSITION, transformVector3f(inv, newPos) );
   }else{
      //do nothing for this Point
   }
   //filter func

}

void  CmdObjFilter::medianFilter1f(IteratorAdaptor *it){
   AAT POSITION=tempCloud->getAAT("position");
   Vector3f center=it->get3f(POSITION);

   kNN_Iterator_TempCloud->setSeekPointAndReset(center);


   //filter func
   card32 count=0;

   std::vector<float32> vx;

   while (!kNN_Iterator_TempCloud->atEnd() && count< kNN_numPoints )
      {
         card32 index= kNN_Iterator_TempCloud->getPointSetPointNumber();
         vx.push_back(hightField[index]);

         count++;
         kNN_Iterator_TempCloud->next();
      }


    if(count!=0){
       sort(vx.begin(), vx.end());
       card32 median=count/2;

   // transform to local coords
     float32 hight;
     Vector4f hPos = expand3To4(center);
     hPos = ctm*hPos;
     float32 w = hPos[3];
     Vector3f p;
     if (fabs(w) > 0.000001f) {
       p = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);
       hight =(mean-p)* n;
     } // if w > 0.00001


      Vector3f newPos=p - n*(vx[median] -hight);
      it->set3f(POSITION, transformVector3f(inv, newPos) );



   }else{
      //do nothing for this Point
   }
   //filter func
}

void  CmdObjFilter::gaussFilter1f(IteratorAdaptor *it){

   AAT POSITION=tempCloud->getAAT("position");
   Vector3f center=it->get3f(POSITION);

   kNN_Iterator_TempCloud->setSeekPointAndReset(center);


   //filter func
   float32 medVal=0.0f;
   float32 weightSum=0.0f;
   card32 count=0;

   while (!kNN_Iterator_TempCloud->atEnd() && count< kNN_numPoints )
      {
         Vector3f pos=kNN_Iterator_TempCloud->get3f(POSITION);
         float32 dist=normQuad(pos-center);
         if (dist > 0.00000001f) dist=sqrt(dist);
         float weight=exp(- 2* dist/strength);

         card32 index= kNN_Iterator_TempCloud->getPointSetPointNumber();
         medVal+=hightField[index]*weight;
         weightSum+=weight;
         count++;
         kNN_Iterator_TempCloud->next();
      }



   if(count!=0){
   // transform to local coords
     float32 hight;
     Vector4f hPos = expand3To4(center);
     hPos = ctm*hPos;
     float32 w = hPos[3];
     Vector3f p;
     if (fabs(w) > 0.000001f) {
       p = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);
       hight =(mean-p)* n;
     } // if w > 0.00001

      medVal/=weightSum;
      Vector3f newPos=p - n*(medVal-hight);
      it->set3f(POSITION, transformVector3f(inv, newPos) );

   }else{
      //do nothing for this Point
   }
   //filter func




}








void CmdObjFilter::undoAction(Object *context) {
   forget(context);
   throw ECannotUndo("CmdObjFilter does not provide any undo.");
}


bool CmdObjFilter::canUndo() {
   return false;
}


std::string CmdObjFilter::getDescription() {
   std::string result;
   result = "FilterTool:  ";
   if (drawScene) result += "scene ";
   else result += "local ";

   switch (mode)
   {
      case 0: result += "average filter:"; break;
      case 1: result += "median filter:"; break;
      case 2: result += "gauss filter:"; break;
      default: result += ":"; break;
   };

   if (!drawScene) for (card32 i=0; i<clickList.size(); i++) {
      result += " (";
      char buff[1000];
      std::string sX = itoa(clickList[i].x,buff,10);
      result += sX;
      result += ",";
      std::string sY = itoa(clickList[i].y,buff,10);
      result += sY;
      result += ")";
   }
   return result;
}




bool CmdObjFilter::mergeCommand(const CommandObject *nextCmd) {
   if (nextCmd->getInstanceClass() == CmdObjFilter::getClass()) {
      const CmdObjFilter *nextCmdMyType = dynamic_cast<const CmdObjFilter*>(nextCmd);
      x4Assert(nextCmdMyType != NULL);

      if (!drawScene &&
         (this->channel==nextCmdMyType->channel) &&
         (this->tolerance==nextCmdMyType->tolerance) &&
         (this->mode==nextCmdMyType->mode) &&
         (this->cam==nextCmdMyType->cam) &&
         (this->viewFrust==nextCmdMyType->viewFrust) &&
         (this->areaWidth==nextCmdMyType->areaWidth) &&
         (this->areaHeight==nextCmdMyType->areaHeight) )
         {
            clickPoint p;
            p.x = nextCmdMyType->clickX;
            p.y = nextCmdMyType->clickY;
            this->clickList.push_back(p);
            return true;
         }
       else return false;

   } else {
      return false;
   }
}

///----
