//---------------------------------------------------------------------------
#ifndef CMD_OBJ_GENERATE_PC_H
#define CMD_OBJ_GENERATE_PC_H
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
#include "PointCloud.h"

#include "UnstructuredInCorePointCloud.h"
#include "IteratorAdaptors.h"
#include "AttachedMemoryManager.h"
#include "HierarchicalKNNIterator.h"
#include "Scene.h"
//---------------------------------------------------------------------------

namespace X4 {

enum CONSTRUCTION_TYPE { 
	GENERATE_BOX, 
	GENERATE_SPHERE, 
	GENERATE_PLANE,
	GENERATE_DOWNWARD_SPIRAL,
	GENERATE_SINUS,
	GENERATE_PARABEL,
	GENERATE_CYLINDER,
	GENERATE_MERLONS
};

/// Generate new point clouds
class X3EDITOR_API CmdObjGeneratePC : public CommandObject {
	X4_CLASS( CmdObjGeneratePC );

	private:
		/// Name of old pc or newly created pc
		std::string pcName;
		/// Number of points to be created
		card32 numPoints;
		/// Type of point cloud: 0 = box, 1 = sphere, 2 = plane.
		card32 type;

	public:
		/// Constructor
		CmdObjGeneratePC();
		/// Apply operation
		virtual void doAction(Object *context);
		/// Undo operation if possible
		virtual void undoAction(Object *context);
		/// Test for undoability
		virtual bool canUndo();
		/// Return description of command object
		virtual std::string getDescription();
		/// Setup object
		void setup( unsigned int type, 
			        unsigned int numPoints, 
					std::string pcName );

		/// Create a spiral object with decreasing radius - really depressing ;-)
		static PointSet* createDownwardSpiral( const card32& numPoints );
		/// Create a sphere
		static PointSet* createSphere( const card32& numPoints );
		/// Create a box
		static PointSet* createBox( const card32& numPoints );
		/// Create a plane
		static PointSet* createPlane( const card32& numPoints );
		/// Create a parabel (plane with f(x,y) = z = x^2
		static PointSet* createParabel( const card32& numPoints);
		/// Create a sinus plane
		static PointSet* createSinusPlane( const card32& numPoints );
		/// Create a cylinder
		static PointSet* createCylinder( const card32& numPoints );
		/// Create some merlons
		static PointSet* createMerlons( const card32& numMerlons );

};


}//ns






#endif
