//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "CmdObjMappingOnClick.h"
//---------------------------------------------------------------------------
#include "FixedArrayClassProperty.h"
#include "StringClassProperty.h"
#include "ObjectClassProperty.h"
#include "PCCommand.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "EmbeddedObjectClassProperty.h"
#include "PointCloud.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "PointCloudIterators.h"
#include "IteratorAdaptors.h"
#include "SelectionIteratorAdaptor.h"
#include "GeometricRange.h"
#include "GeometricRangeIteratorAdaptor.h"
#include "UnstructuredInCorePointCloud.h"
#include "VertexDescriptor.h"
#include "ObjectViewsTable.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "SGObjectNode.h"
#include "MathTools.h"
#include "MathTools.inline.h"
#include "PointCloudTools.h"
#include "SelectionTools.h"
#include "LinearAlgebra.inline.h"
#include "PCTexture.h"
//---------------------------------------------------------------------------
#include "DebugOutput.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* CmdObjMappingOnClick::CmdObjMappingOnClickMetaClass = NULL;

const MetaClass* CmdObjMappingOnClick::getInstanceClass() const {
   return getClass();
}

Object* CmdObjMappingOnClick::newInstance() {
   return new CmdObjMappingOnClick;
}

const MetaClass* CmdObjMappingOnClick::getClass() {
   if (CmdObjMappingOnClickMetaClass==NULL) {
      CmdObjMappingOnClickMetaClass = new MetaClass("CmdObjMappingOnClick", "no description.",
                                  newInstance, sizeof(CmdObjMappingOnClick));
   }
   return CmdObjMappingOnClickMetaClass;
}


void CmdObjMappingOnClick::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 2);
   CmdObjMappingOnClickMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(CmdObjMappingOnClick)
	ADD_EMBEDDED_OBJECT_PROP( cam,2, Camera::getClass() );
	SET_USER_EDIT( false );
	ADD_EMBEDDED_OBJECT_PROP( viewFrust,2, ViewFrustum::getClass() );
	SET_USER_EDIT( false );
	    ADD_CARD32_PROP(areaWidth ,0)
		 ADD_CARD32_PROP(areaHeight ,0)
		 ADD_CARD32_PROP(selectMode ,0)
		 ADD_CARD32_PROP(tolerance ,0)
		 ADD_INT32_PROP(clickX ,0)
		 ADD_INT32_PROP(clickY ,0)
		 ADD_BOOLEAN_PROP( fade , 0)
		 ADD_BOOLEAN_PROP( pos , 0)
		 ADD_BOOLEAN_PROP( pnorm , 0)
		 ADD_BOOLEAN_PROP( col , 0)
		 ADD_BOOLEAN_PROP( raise , 0)
		 ADD_BOOLEAN_PROP( drag , 0)
		 ADD_STRING_PROP(texName,0)
		 ADD_FLOAT32_PROP(scale,0)
}

void CmdObjMappingOnClick::shutdown() {
   delete CmdObjMappingOnClickMetaClass;
}

void CmdObjMappingOnClick::assign(const Object* obj, X4CopyContext *context) {
   CommandObject::assign(obj, context);
   const CmdObjMappingOnClick *tobj = dynamic_cast<const CmdObjMappingOnClick*>(obj);
   x4Assert(tobj != NULL);
   clickList = tobj->clickList;
}

void CmdObjMappingOnClick::write(OutputObjectStream *s) const {
   CommandObject::write(s);
   card32 n = (card32)clickList.size();
   s->write(n);
   for (card32 i=0; i<n; i++) {
      s->write(clickList[i].x);
      s->write(clickList[i].y);
   }
}

void CmdObjMappingOnClick::read(InputObjectStream *s) {
   CommandObject::read(s);
   card32 n;
   s->read(n);
   clickList.resize(n);
   for (card32 i=0; i<n; i++) {
      s->read(clickList[i].x);
      s->read(clickList[i].y);
   }
}



// ----
void CmdObjMappingOnClick::setup(int32 x, int32 y, card32 tol, card32 mode, Camera &ca, ViewFrustum &vfr, card32 w, card32 h, std::string tex, bool f, bool c, bool po, bool pn, bool r , bool d,float32 scl)
{
  clickX = x;
  clickY = y;

  clickPoint p;
  p.x = clickX;
  p.y = clickY;
  clickList.push_back(p);


  tolerance = tol;
  selectMode = mode;
  cam = ca;
  viewFrust = vfr;
  areaWidth = w;
  areaHeight = h;

  fade=f;
  col=c;
  pos=po;
  pnorm=pn;
  raise=r;
  scale=scl;
  drag=d;

  texName=tex;
};


void CmdObjMappingOnClick::doAction(Object *context) {

  Scene *scene = dynamic_cast<Scene*>(context); x4Assert(scene);
  SGLegacyPCSceneAdaptor sceneLA(scene);

  checkSceneAllAttributes(scene); //slow but stable

  PCTexture* colTex = new PCTexture();

  PointCloud* tempPC = sceneLA.getPointCloud(texName);

  for (card32 i=0; i<sceneLA.getNumPointClouds(); i++) {
     if ((sceneLA.getNode(i)->getName() == texName)) {
        tempPC = sceneLA.getPointCloud(i);
     }
  }


  ParametricInCorePointCloud* tpc=dynamic_cast<ParametricInCorePointCloud*>(tempPC);
  if (!tpc) return;
  colTex->setPC(tpc);
  colTex->setDisplacementScale(scale); //slow - perhaps merge?
  colTex->setDisplacementMode(raise);

  for (card32 i=0; i<clickList.size(); i++)
   {
      clickX = clickList[i].x;
      clickY = clickList[i].y;

       //Estimate Plane
       //Select Points for Plane with GeometricRange Iterator
       SphereRange* range = NULL;
       if (drag)
         range = (SphereRange*) getSelectionSphereRangeNotMod(scene);
       else
         range = (SphereRange*) getNearestSphereRangeFromMouseClick(clickX,clickY,areaWidth,areaHeight,tolerance,&cam,&viewFrust,scene);
       if (range==NULL) return;

       //for drag
       SphereRange outerRange(range->getCenter(),1);
       outerRange.setRadius(2.0f*range->getRadius());



       for (i=0; i<sceneLA.getNumPointClouds(); i++) //for all Clouds
       {
           Matrix4f ctm = sceneLA.getState(i)->dynamicState->ctm;
           // Calc Local Coord System on Cloud
           Vector3f u,v,n;
           bool localCoSys=makeLocalCoords(scene,range,drag,i ,u,v,n);
           PointCloud *pc = sceneLA.getPointCloud(i);
           if(!localCoSys || !sceneLA.visible(i)){
             //No Plane possible for this Cloud - too less Points found
           }else{

             /// rotate u and v to match cam up and right  //TODO
             considerCamPos(scene,u,v,n);
             range->setTransformation(ctm);

             /// Paint on Selection
             SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL));
             PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
             if (newIt == NULL) {delete newItSO; return;}
             // GeometricRangeIteratorAdaptor *it=new GeometricRangeIteratorAdaptor(newIt,pc->getDescr(), range);
             IteratorAdaptor *it=getGeoRangeIteratorAdaptor(pc, newIt, range);



             AAT POSITION = pc->getAAT("position");
             AAT COLOR    = pc->getAAT("color");
             AAT NORMAL   = pc->getAAT("normal");
             AAT FLAGS    = pc->getAAT("flags");


             //Consider dimension of selection
             float32 selRad= range->getRadius();
             colTex->setRadius(selRad); //scale Texture


             /// ctm beachten
             Matrix4f inv = invertMatrix(ctm);


             Matrix3f ctm3f;  //3f transpose
             for(int k=0; k<3; k++)
               for(int l=0; l<3; l++){
                  ctm3f[k][l]=ctm[l][k];
               }



             //Normale n umkehren, falls sie falsch liegt
             Vector3f firstNormal;
             if (!it->atEnd())firstNormal=it->get3f(NORMAL);
             float32 dir=firstNormal * (ctm3f*n);    //ctm beachtewn
             if(dir>0.0f){
                n= -n;
//                debugOutput << "flip n "  <<n;
//                debugOutput.cr();
             }else{
                v= -v;
             }



             //Normalen transformieren
             Matrix3f transNorm;
             transNorm[0]=u;
             transNorm[1]=v;
             transNorm[2]=n;

             float32 maxSqr= selRad*selRad;

             Vector3f simpleNormal=makeVector3f(0.0f, 0.0f , -1.0f);


              while (!it->atEnd() )  // for each Point in range
              {
                 Vector3f firstNormal=it->get3f(POSITION);

                 Vector4f hPos = expand3To4(it->get3f(POSITION));
                 hPos = ctm*hPos;
                 float32 w = hPos[3];
                 if (fabs(w) > 0.000001f) {
                   Vector3f p = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);


                    Vector3f c; // Erg
                    c[0]=(mean-p)* u;
                    c[1]=(mean-p)* v;
                    c[2]=(mean-p)* n;

                    ///TEX//////////
                       colTex->transformLookUp(c[0], c[1]);

                       if(col){
                          Vector3f color=colTex->getCol();
                          if(fade){ //fade
                             //calc dist for Fading
                             float32 fading = (c[0]*c[0] + c[1]*c[1])/maxSqr;
                             it->set3f(COLOR, blend(it->get3f(COLOR),color,fading));
                          }
                          else     //set
                             it->set3f(COLOR, color) ;
                       }

                       bool pcIsTexture=(pc==colTex->getPC());
                       if(pos && !pcIsTexture){
                        it->set3f(POSITION,  transformVector3f(inv,
                                  p-n*colTex->getZ())        );
                          if (drag){
                             it->set1i(FLAGS, it->get1i(FLAGS) | PF_FLAG_MODIFIED);
                            // it->set3f(COLOR,makeVector3f(1.0,0.0,0.0));//debug
                          }
                        }

                       // fade position Test
                       //if(pos)it->set3f(POSITION,  transformVector3f(inv,
                       //           p+n*modifyAttrib(0.0f, colTex->getZ(),fading) )  );

                       if(pnorm){
                           if(!(colTex->getNormal()==simpleNormal))
                           it->set3f(NORMAL,
                              ctm3f*  transNorm* (colTex->getNormal())      );
                       }
                    ///TEX//////////

                } // if w > 0.00001
                 it->next() ;
              } //while next Point in Selection

             delete it;
             delete newIt;
          } //if localCoSys
       } //for all Clouds


      delete range;




   }; // end for clicklist

  // delete colTex;
}

void CmdObjMappingOnClick::undoAction(Object *context) {
   forget(context);
   throw ECannotUndo("CmdObjMappingOnClick does not provide any undo.");
}


bool CmdObjMappingOnClick::canUndo() {
   return false;
}


std::string CmdObjMappingOnClick::getDescription() {
   std::string result="";
   if (fade)result+="soft ";
   result += "maping at ";
   for (card32 i=0; i<clickList.size(); i++) {
      result += " (";
      char buff[1000];
      std::string sX = itoa(clickList[i].x,buff,10);
      result += sX;
      result += ",";
      std::string sY = itoa(clickList[i].y,buff,10);
      result += sY;
      result += ")";
   }
  result += " on ";
  if (col)result+="color ";
  if (pos)result+="position ";
  if (pnorm)result+="normal ";

   return result;
}




bool CmdObjMappingOnClick::mergeCommand(const CommandObject *nextCmd) {
   if (nextCmd->getInstanceClass() == CmdObjMappingOnClick::getClass()) {
      const CmdObjMappingOnClick *nextCmdMyType = dynamic_cast<const CmdObjMappingOnClick*>(nextCmd);
      x4Assert(nextCmdMyType != NULL);

      if ((this->m_col==nextCmdMyType->m_col) &&
         (this->tolerance==nextCmdMyType->tolerance) &&
         (this->selectMode==nextCmdMyType->selectMode) &&
         (this->cam==nextCmdMyType->cam) &&
         (this->viewFrust==nextCmdMyType->viewFrust) &&
         (this->areaWidth==nextCmdMyType->areaWidth) &&
         (this->areaHeight==nextCmdMyType->areaHeight) &&
         (this->fade==nextCmdMyType->fade))
         {
            clickPoint p;
            p.x = nextCmdMyType->clickX;
            p.y = nextCmdMyType->clickY;
            this->clickList.push_back(p);
            return true;
         }
       else return false;

   } else {
      return false;
   }
}

///----

// Calc Local Coord System
bool CmdObjMappingOnClick::makeLocalCoords(Scene* scene,GeometricRange* range, bool notMod,card32 pcnumber,Vector3f &u,Vector3f &v,Vector3f &n){
       SGLegacyPCSceneAdaptor sceneLA(scene);

       PointCloud *pc = sceneLA.getPointCloud(pcnumber);
       SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL));
       PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
       if (newIt == NULL) {delete newItSO; return false;}

       Matrix4f ctm = sceneLA.getState(pcnumber)->dynamicState->ctm;

       range->setTransformation(ctm);
       IteratorAdaptor *it=NULL;
       if(notMod)
         it=new NotModGeometricRangeIteratorAdaptor(newIt,pc->getDescr(), range);
       else
         //it=new GeometricRangeIteratorAdaptor(newIt,pc->getDescr(), range);
         it=getGeoRangeIteratorAdaptor(pc, newIt, range);


       //Find Covariance and Mean for Selected Points

       card32 count;
       Vector3f tempMean;

       Matrix3f cov=createCovarianceAndMean(pc,  it, tempMean ,count, ctm);
       if (count >=3) { //Min 3 Pts selected
          mean= tempMean;
          //eigenVec = createEigenVec(cov, eigenVal);
          cov.computeEigenStructure( eigenVal, eigenVec );
       } else
       {
         //Less than 3 Pts selected - no Est. possible
         localValid=false;
         return false;
       }



      //get u,v,n from Matrix3f
      // Matrix3f show=eigenVec;
       u=normalize(eigenVec[0]);
       v=normalize(eigenVec[1]);
       n=normalize(eigenVec[2]);

       delete it;
       delete newIt;

       localValid=true;
       return true;
}






GeometricRange* CmdObjMappingOnClick::getSelectionSphereRangeNotMod(Scene* scene){
   SGLegacyPCSceneAdaptor sceneLA(scene);

   float32 nearest_z = 1.0f;
   RayConeRange *range = (RayConeRange*)getRayConeRangeFromMouseClick(clickX,clickY,areaWidth,areaHeight,tolerance,&cam,&viewFrust);

   for (card32 i=0; i<sceneLA.getNumPointClouds(); i++)
   {
      Matrix4f ctm = sceneLA.getState(i)->dynamicState->ctm;
      PointCloud *pc = sceneLA.getPointCloud(i);
      if (!pc->providesAttribute("position")) return NULL;
      AAT POSITION = pc->getAAT("position");
      AAT FLAGS = pc->getAAT("flags");

      SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_HIERARCHICAL));
      PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
      if (newIt == NULL) {delete newItSO; return NULL;}

      BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
      if (basicIt == NULL) {delete newIt; return NULL;}

      // set cloud transformation matrix and make range iterator adaptor
      range->setTransformation(ctm);
      IteratorAdaptor *rIt = getGeoRangeIteratorAdaptor(pc, newIt, range);
      if (rIt == NULL) {delete newItSO; return NULL;}

      while(!rIt->atEnd()) {

         if(!(rIt->get1i(FLAGS) & PF_FLAG_MODIFIED)){
            float32 PointZ = range->getPointZInRayCoords(basicIt->get3f(POSITION));
            if (PointZ!=1.0f) {
               if ((PointZ>nearest_z) || (nearest_z==1.0f)) {
                  nearest_z = PointZ;
               };
            };
         }
         rIt->next();
      };

      delete rIt;
      delete newItSO;

   } // end for


//   delete range;
   if (nearest_z==1.0f) return NULL;
   return new SphereRange((cam.getPosition()+((range->getRayDir())*nearest_z)), (nearest_z*nearest_z*(range->getSqrRadiusAt1())));

};


//==============================================================================


MetaClass* CmdObjLoadMap::CmdObjLoadMapMetaClass = NULL;

const MetaClass* CmdObjLoadMap::getInstanceClass() const {
   return getClass();
}

Object* CmdObjLoadMap::newInstance() {
   return new CmdObjLoadMap;
}

const MetaClass* CmdObjLoadMap::getClass() {
   if (CmdObjLoadMapMetaClass==NULL) {
      CmdObjLoadMapMetaClass = new MetaClass("CmdObjLoadMap", "no description.",
                                  newInstance, sizeof(CmdObjLoadMap));
   }
   return CmdObjLoadMapMetaClass;
}


void CmdObjLoadMap::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   CmdObjLoadMapMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(CmdObjLoadMap)
	ADD_STRING_PROP(fileName,0)
}

void CmdObjLoadMap::shutdown() {
   delete CmdObjLoadMapMetaClass;
}


// ----


void CmdObjLoadMap::setup(std::string file)
{
   fileName=file;
};


void CmdObjLoadMap::doAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); x4Assert(scene);
   SGLegacyPCSceneAdaptor sceneLA(scene);

   //test if pc with this name exists
   for (card32 i=0; i<sceneLA.getNumPointClouds(); i++)
   {
      std::string scenePCName = sceneLA.getNode(i)->getName();
      if (scenePCName == fileName) return;
   }




	ParametricInCorePointCloud* texture = PCTexture::loadFromFile(fileName);
   SGObjectNode *sgn = new SGObjectNode();
   sgn->clearAndSetup(texture);
   sgn->setName(fileName);
   sgn->setVisible(false);
   sceneLA.addNode(sgn);

   Matrix4f tmpM = makeMatrix4f(1,0,0,3,  // move away TODO
                             0,1,0,3,
                             0,0,1,3,
                             0,0,0,1);
   texture->setTransformation(tmpM * texture->getTransformation());
   ObjectViewsTable::update(scene);
}

void CmdObjLoadMap::undoAction(Object *context) {
   forget(context);
   throw ECannotUndo("CmdObjLoadMap does not provide any undo.");
}

bool CmdObjLoadMap::canUndo() {
   return false;
}


std::string CmdObjLoadMap::getDescription() {
   std::string result;
   result = "load texture " + fileName;
   return result;
}



