//---------------------------------------------------------------------------
#ifndef CmdObjPointCloudOperationsH
#define CmdObjPointCloudOperationsH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
#include "PointCloud.h"
#include "ViewFrustum.h"
#include "Camera.h"
#include "Scene.h"
//---------------------------------------------------------------------------


namespace X4 {


// identifiers
const card32 OP_DELETE = 0;
const card32 OP_COPY = 1;
const card32 OP_SPLIT = 2;
const card32 OP_JOIN = 3;
const card32 OP_COPY_TO_ONE = 4;
const card32 OP_SPLIT_TO_ONE = 5;
const card32 OP_REMOVE_CHANNEL =6;

const card32 OLDP_NOTHING = 0;
const card32 OLDP_DESELECT = 1;
const card32 OLDP_DELETE = 2;



/// command object for duplication, split or join of point clouds
class X3EDITOR_API CmdObjPointCloudOperations : public CommandObject {
 private:
   static MetaClass *CmdObjPointCloudOperationsMetaClass;
   static Object* newInstance();

 protected:

   // properties
   /// defines what operation is done. 0=delete_points, 1=copy_points_to_different_clouds, 2=split_to_different_clouds, 3=join_clouds, 4=copy_to_one_target_cloud, 5=split_to_one_target_cloud
   card32 Operation;
   ///for removeChannel only: channel to remove;
   std::string ChannelName;


   // user functions
   /// delete selected points from clouds
   void deletePoints(Scene *scene);
   /// copy selected points to new clouds, oldp says what to do with used selected points: 0=nothing, 1=deselect, 2=delete
   void copyPointsToNewCloud(Scene *scene, card32 oldp);
   /// join two clouds
   bool joinTwoClouds(PointCloud* basePC, PointCloud* addPC);
   /// join all selected clouds
   void joinPointClouds(Scene* scene);
   /// check if point cloud type is dynamic. needed to check if output cloud is able to incorporate points.
   inline bool checkCloudTypeIsDynamic(PointCloud* pc) {
      if ( pc->supportsIteratorType(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PC)) ) return true;
		// todo: maybe some other warning and return false!
         else throw X4Exception("CmdObjPointCloudOperations::checkCloudTypeIsDynamic() - PointCloud ist not dynamic, cannot add points");
   };
   void removeChannel(Scene *scene);
   /// returns point cloud priority. 4=HierarchicalOutOfCorePointCloud, 3=HierarchicalInCorePointCloud, 2=UnstructuredInCorePointCloud, 1=other
   card32 calcPCPriority(const PointCloud *pc);


 public:

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   /// setup parameters: see properties and implementation
   virtual void setup(card32 mode);
   virtual void setup(std::string remChannelName);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();

};


}

#endif