//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "CmdObjTransfEdit.h"
//---------------------------------------------------------------------------
#include "SceneGraphState.h"
#include "EmbeddedObjectClassProperty.h"
#include "MathTools.h"
#include "MathTools.inline.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "SelectionTools.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* CmdObjTransfEdit::CmdObjTransfEditMetaClass = NULL;

const MetaClass* CmdObjTransfEdit::getInstanceClass() const {
   return getClass();
}

Object* CmdObjTransfEdit::newInstance() {
   return new CmdObjTransfEdit;
}

const MetaClass* CmdObjTransfEdit::getClass() {
   if (CmdObjTransfEditMetaClass==NULL) {
      CmdObjTransfEditMetaClass = new MetaClass("CmdObjTransfEdit", "no description.",
                                  newInstance, sizeof(CmdObjTransfEdit));
   }
   return CmdObjTransfEditMetaClass;
}

void CmdObjTransfEdit::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   CmdObjTransfEditMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( CmdObjTransfEdit )
   ADD_MATRIX4F_PROP(TransfMatrix, 0)
	ADD_BOOLEAN_PROP(transPoints, 0)
	ADD_BOOLEAN_PROP(merge, 0)
}

void CmdObjTransfEdit::shutdown() {
   delete CmdObjTransfEditMetaClass;
}


void CmdObjTransfEdit::setup(Matrix4f trmatr, bool trP, bool merg) {
   TransfMatrix = trmatr;
   transPoints = trP;
   merge = merg;
}


void CmdObjTransfEdit::doAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); x4Assert(scene);
   applyTransformation(scene, TransfMatrix);
}


void CmdObjTransfEdit::undoAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); x4Assert(scene);
   applyTransformation(scene, invertMatrix(TransfMatrix));
}

bool CmdObjTransfEdit::canUndo() {
   // check if transformation matrix is invertible
   Matrix3f tmpM = makeMatrix3f(TransfMatrix[0][0],TransfMatrix[0][1],TransfMatrix[0][2],
                                TransfMatrix[1][0],TransfMatrix[1][1],TransfMatrix[1][2],
                                TransfMatrix[2][0],TransfMatrix[2][1],TransfMatrix[2][2]).transpose();

   if ((!((TransfMatrix[0][3]==0) &&
          (TransfMatrix[1][3]==0) &&
          (TransfMatrix[2][3]==0) &&
          (TransfMatrix[3][3]==1)))
       || (fabs(determinant(tmpM)) < (1E-8)))
      return false;

   return true;
}


void CmdObjTransfEdit::applyTransformation(Scene *scene, Matrix4f TrMatrix) {

   StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
   sceneIterator->clearAndSetup(scene->getRootNode(), scene->getRootState());

   while (!sceneIterator->atEnd()) {

      SGObjectNode* SGN = dynamic_cast<SGObjectNode*>(sceneIterator->getNode());
      if ((SGN==NULL) || !SGN->visible()) { sceneIterator->next(); continue; }
      PointCloud* PC = dynamic_cast<PointCloud*>(SGN->getSceneObject());
      if (PC==NULL) { sceneIterator->next(); continue; }

      if (!transPoints){
         // affect selected clouds only
         if (!testPointCloudIsSelected(scene, sceneIterator->getNameWithPath())) { sceneIterator->next(); continue; }
         // transform cloud
         Matrix4f ctm = sceneIterator->getState()->dynamicState->ctm * invertMatrix(PC->getTransformation());
         Matrix4f TM = invertMatrix(ctm) * TrMatrix * ctm;
         PC->setTransformation(TM * PC->getTransformation());

      } else {
         // affect selected points
         SceneObjectIterator *newItSO = PC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_HIERARCHICAL));
         PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
         if ((newIt==NULL)||(newItSO==NULL)) { delete newItSO; sceneIterator->next(); continue; };

         IteratorAdaptor *sIt = getSelectionIteratorAdaptor(PC, newIt);
         if (sIt==NULL) { delete newItSO; sceneIterator->next(); continue; };

         // consider cloud transformation
         Matrix4f ctm = sceneIterator->getState()->dynamicState->ctm;
         Matrix4f ctmInv = invertMatrix(ctm);
         Matrix4f TM = ctmInv * TrMatrix * ctm;
		   Matrix3f TMRot = shrink4To3(TM);
         AAT POSITION = PC->getAAT("position");
			AAT NORMAL;
			bool hasnormal = PC->providesAttribute("normal");
			if (hasnormal) NORMAL = PC->getAAT("normal");

         // transform points
         while(!sIt->atEnd()) {
            sIt->set3f(POSITION, transformVector3f(TM,sIt->get3f(POSITION)));
	         if (hasnormal) sIt->set3f(NORMAL, normalize(TMRot*(sIt->get3f(NORMAL))));
            sIt->next();
         };

         delete sIt;
         delete newItSO;

      }; // end else transPoints

      sceneIterator->next();
   }; // end while sceneIterator

   delete sceneIterator;
}



std::string CmdObjTransfEdit::getDescription() {
   std::string result = "PCITransfEdit: ";
   if (transPoints) result += "points";
      else result += "clouds";
   return result;
}


bool CmdObjTransfEdit::mergeCommand(const CommandObject *nextCmd) {
   if (nextCmd->getInstanceClass() == CmdObjTransfEdit::getClass()) {
      const CmdObjTransfEdit *nextCmdMyType = dynamic_cast<const CmdObjTransfEdit*>(nextCmd);
      x4Assert(nextCmdMyType != NULL);
      if ((nextCmdMyType->merge))
      {
         this->TransfMatrix = nextCmdMyType->TransfMatrix * this->TransfMatrix;
         return true;
      }
      else return false;

   } else {
      return false;
   }
}

