//---------------------------------------------------------------------------
#ifndef CmdObjTransfEditH
#define CmdObjTransfEditH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
#include "PointCloud.h"
#include "ViewFrustum.h"
#include "Camera.h"
#include "Scene.h"
//---------------------------------------------------------------------------


namespace X4 {


/// command object for transformation (movement/rotation/scaling) of clouds or points
class X3EDITOR_API CmdObjTransfEdit : public CommandObject {

 private:
   static MetaClass *CmdObjTransfEditMetaClass;
   static Object* newInstance();

 protected:

   // properties
   /// transformation matrix which is used for transformation
   Matrix4f TransfMatrix;
   /// true if points should be affected, else clouds are affected
   bool transPoints;
   /// says wether merging of consecutive transformations is allowed or not (used in PCITransfEdit to avoid multiple command objects on mouse movement).
   bool merge;

   // user functions
   /// helper function, which does the real transform action on the clouds or points. TrMatrix is the tranformation matrix used.
   void applyTransformation(Scene *scene, Matrix4f TrMatrix);


 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   /// setup parameters: see properties and implementation
   virtual void setup(Matrix4f trmatr, bool trP, bool merg);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();

   virtual bool mergeCommand(const CommandObject *nextCmd);
};


}

#endif