#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ImportMeshOpenAsset.h"
//---------------------------------------------------------------------------
/// fixme - 32bit version of assimp.lib is missing!
//#define X4_64BIT
#ifdef X4_64BIT
#include "assimp.h"
#include "aiPostProcess.h"
#include "aiScene.h"
#include "VertexDescriptor.h"
#include "ColorTools.h"
#endif
#include "Random.h"
//---------------------------------------------------------------------------
using namespace X4;

/// fixme - 32bit version of assimp.lib is missing!
#ifdef X4_64BIT

struct OA_SceneGraphState
{
	Matrix4f	T;
	mpcard currentPolyIndex;
};

Vector3f convertOAtoX4(const aiVector3D &v)
{
	return makeVector3f(v.x,v.y,v.z);
}

void parseOpenAssetScene(const aiScene *scene, const aiNode* nd, OA_SceneGraphState * state, UnstructuredInCoreTriangleMesh * mesh )
{
	aiMatrix4x4 aiT = nd->mTransformation;	
	Matrix4f * T = (Matrix4f*)&aiT;
	Matrix4f currentT = state->T;
	state->T = state->T * T->transpose();
	Matrix3f T3f = shrink4To3(state->T);

	AAT posAAT = mesh->getAAT("position");
	AAT normalAAT = mesh->getAAT("normal");
	AAT colorAAT = mesh->getAAT("color");
	AAT meshidAAT = mesh->getAAT("meshid");

	PointSet * points = mesh->getPointSet();
	PointSet * triangles = mesh->getTriangles();
	AAT indexAAT = triangles->getAAT("index");
	AAT polyIndexAAT = triangles->getAAT("polyIndex");

	// instanciate meshes
	for( mpcard i=0;i<nd->mNumMeshes;i++ )
	{
		const aiMesh * mesh = scene->mMeshes[nd->mMeshes[i]];
		const aiMaterial * mtl = scene->mMaterials[mesh->mMaterialIndex];

		aiColor4D diffuse;
		diffuse.r = 1;
		diffuse.g = 1;
		diffuse.b = 1;
		aiGetMaterialColor(mtl, AI_MATKEY_COLOR_DIFFUSE, &diffuse);
		const Vector3f matColor = makeVector3f(rnd01(),rnd01(),rnd01());//randomColor();//makeVector3f(diffuse.r, diffuse.g, diffuse.b);

		stdext::hash_map<mpcard,mpcard> vertexMap;

		for( mpcard j=0;j<mesh->mNumFaces;j++ )
		{
			const aiFace * face = &mesh->mFaces[j];
			if( face->mNumIndices < 3 )
				continue;	// ignore lines and points

			// add vertices
			std::vector<mpcard> indexTable(face->mNumIndices);	// maps local face index 'k' to global vertex index
			for( mpcard k=0;k<face->mNumIndices;k++ )
			{
				mpcard index = face->mIndices[k];
				if( vertexMap.find(index) != vertexMap.end() )
				{
					indexTable[k] = vertexMap[index];
					continue;
				}

				mpcard vi = points->getNumEntries();
				vertexMap[index] = indexTable[k] = vi;
				points->addPoint(NULL);
				points->set3f(vi,posAAT,transformVector3f(state->T,convertOAtoX4(mesh->mVertices[index])));
				points->set1u(vi,meshidAAT,i);
				if( mesh->mNormals )
					points->set3f(vi,normalAAT,T3f * convertOAtoX4(mesh->mNormals[index]));
				Vector3f col = matColor;
				if( mesh->mColors[0] )
				{
					aiColor4D aiCol = mesh->mColors[0][index];
					col = col.componentProduct(makeVector3f(aiCol.r,aiCol.g,aiCol.b));
				}
				points->set3f(vi,colorAAT,col);
			}

			// compute normals if required
			if( mesh->mNormals == NULL )
			{
				// compute polygon normal (choose stable triangle within poly)
				const Vector3f p0 = points->get3f(indexTable[0],posAAT);
				const Vector3f p1 = points->get3f(indexTable[1],posAAT);
				const Vector3f dir0 = p1-p0;

				// pick third point with maximum distance to line
				Line3f l(dir0,p0);
				float maxDist = 0;
				Vector3f maxDistPos;
				for( mpcard k=2;k<face->mNumIndices;k++ )
				{
					const Vector3f p = points->get3f(indexTable[k],posAAT);
					float dist = l.computeDistance(p);
					if( dist > maxDist )
					{
						maxDist = dist;
						maxDistPos = p;
					}
				}

				// compute triangle normal
				Vector3f normal = normalize( dir0.crossProduct(maxDistPos-p0) );

				// assign normal vectors
				for( mpcard k=0;k<face->mNumIndices;k++ )
				{
					points->set3f(indexTable[k],normalAAT,normal);
				}
			}

			// add triangles
			mpcard index0 = indexTable[0];
			for( mpcard k=2;k<face->mNumIndices;k++ )
			{
				mpcard index1 = indexTable[k-1];
				mpcard index2 = indexTable[k];
				mpcard ti = triangles->getNumEntries();
				triangles->addPoint(NULL);
				triangles->set3i( ti, indexAAT, makeVector3i(index0,index1,index2));
				triangles->set1i( ti, polyIndexAAT, state->currentPolyIndex );
			}
			state->currentPolyIndex++;
		}

	}

	// traverse further down in hierarchy
	for (mpcard i = 0; i < nd->mNumChildren; i++) 
	{
		parseOpenAssetScene(scene, nd->mChildren[i],state,mesh);
	}

	state->T = currentT;
}
#endif

UnstructuredInCoreTriangleMesh *ImportMeshOpenAsset::import( const std::string &filename ) 
{
   /// fixme - 32bit version of assimp.lib is missing!
#ifdef X4_64BIT
	// try to import
	const aiScene* scene = aiImportFile(filename.c_str(),aiProcessPreset_TargetRealtime_Quality);
	if( !scene )
		return NULL;

	// prepare mesh
	UnstructuredInCoreTriangleMesh * mesh = new UnstructuredInCoreTriangleMesh;
	VertexDescriptor * vdVertex = createDefaultVertexDescriptor(true,true,true,true);
	vdVertex->pushAttrib(VAD("meshid", 1, VAD::DATA_FORMAT_CARD32));

	VertexDescriptor vdTri;
	vdTri.pushAttrib(VAD("index",3, VAD::DATA_FORMAT_INT32));
	vdTri.pushAttrib(VAD("polyIndex",1,VAD::DATA_FORMAT_INT32));
	mesh->clearAndSetup(vdVertex,0,&vdTri,0);
	delete vdVertex;

	// parse scene
	OA_SceneGraphState state;
	state.T = IDENTITY4F;
	state.currentPolyIndex = 0;
	parseOpenAssetScene(scene,scene->mRootNode,&state,mesh);
	return mesh;
#else
	return NULL;
#endif
}
