#include "StdAfx.h"
//--------------------------------
#include "IndexGraph.h"

using namespace X4;

/****************************
* IGGraph
****************************/
IGGraph::IGGraph(void){
	rootIndex = 0;
	edgeCounter = 0;
}

IGGraph::~IGGraph(void){
	for(unsigned i=0;i<nodes.size();i++){
		delete nodes[i];
	}
}

void IGGraph::createNodes(int count){
	for(int i=0;i<count;i++){
		nodes.push_back(new IGNode(i));
	}
}

void IGGraph::addNodes(int count){
	for(int i=1;i<=count;i++){
		nodes.push_back(new IGNode(nodes[nodes.size()]->getIndex()+i));
	}
}

void IGGraph::createEdge(int node1, int node2, float costs){
	//Make sure nodes-std::vector is large enough
	x4Assert((int)nodes.size() > node1);
	x4Assert((int)nodes.size() > node2);
	//Add edges
	IGEdge* e1 = new IGEdge(costs,node1,node2,edgeCounter++);
	IGEdge* e2 = new IGEdge(costs,node2,node1,edgeCounter++);
	nodes[node1]->addEdge(e1);
	nodes[node2]->addEdge(e2);
}

/******************
* IGNode
******************/

IGNode::IGNode(void){
	
}

IGNode::IGNode(int index){
	this->index = index;
	this->generalPurposeFlag = false;
}

IGNode::~IGNode(void){
	list<IGEdge*>::iterator it;
	for(it=l.begin();it!=l.end();it++){
		delete *it;
	}
}

void IGNode::addEdge(IGEdge* e){
	list<IGEdge*>::iterator it;
	IGEdge* oldEdge;
	for(it=l.begin();it!=l.end();it++){
		oldEdge = *it;
		//If edge already exists, return
		if(oldEdge->getTargetNode() == e->getTargetNode()){
			delete e;
			return;
		}
	}
	l.push_back(e);
}

list<IGEdge*>::iterator IGNode::getEdgeIterator(){
	return l.begin();
}

/*********************
* IGEdge
*********************/

IGEdge::IGEdge(void){

}
IGEdge::IGEdge(float costs, int from, int to, int uid){
	this->costs = costs;
	this->from = from;
	this->to = to;
	this->uid = uid;
}
IGEdge::~IGEdge(void){

}