#ifndef IndexGraphH
#define IndexGraphH
//------------------------------------
#include "x3editor.h"
#include <list>
#include <vector>

/*****************************************************
* Data structure for an index-based neighborhood graph
******************************************************/
namespace X4{
	//forward declaration
	class IGEdge;

//Node class
class X3EDITOR_API IGNode
{
private:
	int index;
	bool generalPurposeFlag; //flag for graph algorithms
	list<IGEdge*> l;
public:
	IGNode(void);
	IGNode(int index);
	~IGNode(void);

	inline int getIndex() { return index; };
	inline void setFlag(bool val) { generalPurposeFlag = val; };
	inline bool isFlagSet() { return generalPurposeFlag; };
	inline mpcard getEdgeCount() { return l.size(); };
	void addEdge(IGEdge* e);
	list<IGEdge*>::iterator getEdgeIterator();
};

//Edge class
class X3EDITOR_API IGEdge
{
private:
	float costs;
	int from;
	int to;
public:
	int uid;

	IGEdge(void);
	IGEdge(float costs, int from, int to, int uid);
	~IGEdge(void);

	bool operator < (const IGEdge* other) const{
		return (uid < other->uid);
	}

	bool operator == (const IGEdge* other) const{
		return (uid == other->uid);
	}

	inline float getCosts() { return costs; };
	inline int getStartNode() { return from; };
	inline int getTargetNode() { return to; };
};

//Graph class
class X3EDITOR_API IGGraph
{
private:
	int rootIndex;
	int edgeCounter;
	std::vector<IGNode*> nodes;
public:
	IGGraph(void);
	~IGGraph(void);

	void createNodes(int count);
	void addNodes(int count);
	void createEdge(int node1, int node2, float costs);
	inline void setRoot(int nodeIndex) { rootIndex = nodeIndex; };
	inline int getRoot() { return rootIndex; };
	inline IGNode* getNode(int index) { x4Assert((int)nodes.size() > index); return nodes[index]; };
	inline mpcard getNodeCount() { return nodes.size(); };
};

}

#endif