#include "StdAfx.h"
#include "MacroEditorWidget.h"
#include "Timer.h"
#include "DebugOutput.h"
#include "Configuration.h"
#include "FileDialogs.h"
#include "BinaryObjectStreams.h"
#include "ExceptionHandling.h"
#include "ObjectDictionaryEditorWidget.h"
#include "CommandObjMacro.h"
#include <QLayout>
#include "SceneEditorWidget.h"
#include "AttachedDataTypes.h"
#include "CommandHistory.h"
//---------------------------------------------------------------------------

using namespace X4;

bool MacroEditorWidget::scriptRunning = false;
SceneEditorWidget *MacroEditorWidget::activeEditorWidget = NULL;

/*MacroScriptEditorWidget *X4::macroScriptEditorWidget = NULL;*/


MacroEditorWidget::MacroEditorWidget(Scene *scene, SceneEditorWidget *editorWidget)
:QMainWindow()
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	// used Qt auto-connect feature for slots

	this->scene = scene;
	this->editorWidget = editorWidget;
	
   // Create dictionary attachment if it does not exist yet
   AttachedDataContainer* attachments = scene->getAttachments();
   if (attachments->getData("dictionary") == NULL) {
      AttachedDictionaryObject* attachedDic = new AttachedDictionaryObject();
      attachedDic->setup("dictionary", AttachedData::ADF_PERSISTENT);
      attachments->attachData(attachedDic);
   }

   AttachedDictionaryObject* attachedDic =
     (AttachedDictionaryObject*)scene->getAttachments()->getData("dictionary");
   dictionary = attachedDic->getObjectDictionary();

   dictionaryEditor = new ObjectDictionaryEditor;
   dictionaryEditorWidget = (ObjectDictionaryEditorWidget*)dictionaryEditor->makeEditor(ui.frame, dictionary);
   QHBoxLayout* editorLayout = new QHBoxLayout(ui.frame);
   editorLayout->addWidget(dictionaryEditorWidget);

   dictionaryEditor->update();
}

void MacroEditorWidget::update() {
   dictionaryEditor->update();
}

MacroEditorWidget::~MacroEditorWidget(void)
{
   delete dictionaryEditor;
}

void MacroEditorWidget::on_action_Load_triggered(bool checked)
{
   dictionaryEditorWidget->load();
}

void MacroEditorWidget::on_action_Save_triggered(bool checked)
{
   dictionaryEditorWidget->save();
}

void MacroEditorWidget::on_action_Update_triggered(bool checked)
{
   dictionaryEditor->update();
}

void MacroEditorWidget::on_BTExecute_clicked()
{
   Object* obj = dictionaryEditorWidget->getSelectedObject();
   if (obj == NULL) {
      QMessageBox::critical(this, "Error", "No object selected");
      return;
   }

   CommandObject* cmdObj = dynamic_cast<CommandObject*>(obj);
   if (cmdObj == NULL) {
      CommandObjMacro* macro = dynamic_cast<CommandObjMacro*>(obj);
      if (macro == NULL) {
         QMessageBox::critical(this, "Wrong object type", "Only command objects and macros can be executed");
         return;
      } else {
         STD_GLOBAL_EXCEPTION_HANDLER_START

            scriptRunning = true;
            activeEditorWidget = editorWidget;
            for (card32 i = 0; i < macro->numCommands(); ++i) {
               CommandObject* cmdObj = const_cast<CommandObject*>(macro->getCommand(i));
               scene->getCommandHistory()->executeCommand(scene, cmdObj);
            }
            scriptRunning = false;
            activeEditorWidget = NULL;

         STD_GLOBAL_EXCEPTION_HANDLER_END
      }
	} else {
		STD_GLOBAL_EXCEPTION_HANDLER_START
		scriptRunning = true;
		activeEditorWidget = editorWidget;
		scene->getCommandHistory()->executeCommand(scene, cmdObj);
		scriptRunning = false;
		activeEditorWidget = NULL;
		STD_GLOBAL_EXCEPTION_HANDLER_END
	}
}

bool MacroEditorWidget::getScriptRunning() 
{
   return scriptRunning;
}

void MacroEditorWidget::repaintInScript() 
{
   if (activeEditorWidget != NULL) {
      activeEditorWidget->repaintGLArea();
   	//QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
   } else {
      x4Assert(false);
   }
}


#include "MacroEditorWidget_moc.h"
