//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "MeshExporter.h"
//---------------------------------------------------------------------------
#include "UnstructuredInCoreTriangleMesh.h"
#include "PointSet.inline.h"
#include "LinearAlgebra.inline.h"
#include "MathTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------
#include "StringHelper.h"



MeshExporter::MeshExporter(PointCloud* pc, std::ofstream* outf)
{
  this->pc = pc;
  this->outf = outf;
  exportObjNormals=false;

  exportSmallTriangles = false;
  smallTriangleSize = 0.001f;
}

void MeshExporter::exportPointCloud()
{
	UnstructuredInCoreTriangleMesh *mesh = dynamic_cast<UnstructuredInCoreTriangleMesh*>(pc);

	if ( mesh != NULL ){
		// Export triangle mesh
		PointSet* vertex   = mesh->getPointSet();
		card32 numVert= vertex->getNumEntries();
		const VertexDescriptor* vertexDesc = vertex->getDescr();
		AAT POSITION = vertexDesc->getToken("position");
		for (card32 i=0; i<numVert; i++){
			Vector3f vec3f = vertex->get<float32,3>(i, POSITION);
			*outf << "v " << vec3f[0] << " " << vec3f[1] << " " << vec3f[2] << endl;
		}
		PointSet* triangle = mesh->getTriangles();
		card32 numTri= triangle->getNumEntries();
		const VertexDescriptor* triDesc = triangle->getDescr();
		AAT INDEX = triDesc->getToken("index");
		for (card32 i=0; i<numTri; i++){
			Vector3i vec3i = triangle->get<int32,3>(i, INDEX);
			*outf << "f " << intToStr(vec3i[0]+1) << " " << intToStr(vec3i[1]+1) << " " << intToStr(vec3i[2]+1) << endl;
		}
	} else {
		// Export point cloud
		if ( pc == NULL ){
			return;
		}

		if(exportSmallTriangles){
			doExportSmallTriangles(pc, smallTriangleSize);
			return;
		}

		SceneObjectIterator *newItSO = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC ) );
		BasicPointCloudIterator* basicIt = dynamic_cast<BasicPointCloudIterator*>( newItSO );
		ModifyablePointCloudIterator* modIt = dynamic_cast<ModifyablePointCloudIterator*>( newItSO );
		AAT POSITION = pc->getAAT( "position" );
		basicIt->reset();
		while( !basicIt->atEnd() ){
			Vector3f vec3f = basicIt->get3f( POSITION );
			*outf << "v " << vec3f[0] << " " << vec3f[1] << " " << vec3f[2] << endl;
			basicIt->next();
		}
		if (exportObjNormals)
		{
			AAT NORM = pc->getAAT( "normal" );
			basicIt->reset();
			while( !basicIt->atEnd() ){
				Vector3f vec3f = normalize(basicIt->get3f( NORM ));
				*outf << "vn " << vec3f[0] << " " << vec3f[1] << " " << vec3f[2] << endl;
				basicIt->next();
			}		
		}
		delete newItSO;
	}
}


void X4::MeshExporter::setExportObjNormals( bool b )
{
	exportObjNormals=b;
}

void X4::MeshExporter::setExportObjSmallTriangles( bool b )
{
	exportSmallTriangles = b;
}

void X4::MeshExporter::doExportSmallTriangles( PointCloud* pc, float32 size )
{

	SceneObjectIterator *newItSO = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC ) );
	BasicPointCloudIterator* basicIt = dynamic_cast<BasicPointCloudIterator*>( newItSO );
	ModifyablePointCloudIterator* modIt = dynamic_cast<ModifyablePointCloudIterator*>( newItSO );
	AAT POSITION = pc->getAAT( "position" );
	AAT NORM = pc->getAAT( "normal" );
	basicIt->reset();
	while( !basicIt->atEnd() ){
		Vector3f pos3f	  = basicIt->get3f( POSITION );
		Vector3f normal3f = normalize(basicIt->get3f( NORM ));
		Matrix3f tanSys =  calcTangentSystem(normal3f);
		x4Assert (tanSys[2] == normal3f);
		Vector3f b = pos3f + tanSys[1] * smallTriangleSize;
		Vector3f c = pos3f + tanSys[0] * smallTriangleSize;
		
		*outf << "v " << pos3f[0] << " " << pos3f[1] << " " << pos3f[2] << endl;

		*outf << "v " << c[0] << " " << c[1] << " " << c[2] << endl;
				*outf << "v " << b[0] << " " << b[1] << " " << b[2] << endl;
		basicIt->next();
	}
		
// 	basicIt->reset();
// 	while( !basicIt->atEnd() ){
// 		Vector3f vec3f = normalize(basicIt->get3f( NORM ));
// 		*outf << "vn " << vec3f[0] << " " << vec3f[1] << " " << vec3f[2] << endl;
// 		*outf << "vn " << vec3f[0] << " " << vec3f[1] << " " << vec3f[2] << endl;
// 		*outf << "vn " << vec3f[0] << " " << vec3f[1] << " " << vec3f[2] << endl;
// 		basicIt->next();
// 	}		
	
	delete newItSO;

	// add pseudo faces
	for (mpcard i = 0; i < pc->getNumPoints(); i++)
	{
		*outf << "f " << i*3+1 << " " << i*3+2 << " " << i*3+3 << endl;
	}

}