#include "StdAfx.h"
#include "MovieStudioWidget.h"
#include "SceneEditorWidget.h"
#include <QMenuBar>
#include "OnlineHelp.h"
#include "Timer.h"
#include "DebugOutput.h"
#include "Configuration.h"
#include "FileDialogs.h"
//---------------------------------------------------------------------------

using namespace X4;

MovieStudioWidget *X4::movieStudioWidget = NULL;


MovieStudioWidget::MovieStudioWidget(SceneEditorWidget *sceneEd)
:QWidget(NULL)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	// used Qt auto-connect feature for slots

	setWindowIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/X4.png").c_str()));

	// setup the menubar
	menuBar = new QMenuBar(this);
	QMenu* fileMenu = menuBar->addMenu(tr("&File"));
	QMenu* helpMenu = menuBar->addMenu(tr("&Help"));

	QAction* ActClose = new QAction(tr("Close"), this);
   connect(ActClose, SIGNAL(triggered()), this, SLOT(hide()));
	fileMenu->addAction(ActClose);

	QAction* ActOnlineHelp = new QAction(tr("Online Help"), this);
   connect(ActOnlineHelp, SIGNAL(triggered()), this, SLOT(sltOnlineHelp()));
	helpMenu->addAction(ActOnlineHelp);

	ui.vboxLayout->setMenuBar(menuBar);

	// init the rest
	this->sceneEd = sceneEd;
	int numFrames = sceneEd->getAnimationBar()->getFramesCount();
	ui.SBNumberOfFrames->setValue(numFrames);
}

MovieStudioWidget::~MovieStudioWidget(void)
{
	delete menuBar;
}



// ---------- user functions ----------

void MovieStudioWidget::BTRecOrTimeNowClicked(bool doRec) {
   hide();
   Timer someTimer;
   float32 mint, maxt, stept;
	if (ui.CBMainWinTimeline->isChecked()) {

		mint = sceneEd->getAnimationBar()->getStartFrameNumber(); 
      maxt = sceneEd->getAnimationBar()->getEndFrameNumber();
      stept = sceneEd->getAnimationBar()->getAnimStep(); 
		sceneEd->recordMovieNow(qPrintable(ui.LEFileName->text()), mint, maxt, stept, true, doRec);

   } else {
      mint = 0.0f;
		maxt = ui.SBNumberOfFrames->value()-1.0f;
      stept = 1.0f;
		sceneEd->recordMovieNow(qPrintable(ui.LEFileName->text()), mint, maxt, stept, ui.CBIncrSceneGraphTime->isChecked(), doRec);
   }
   card32 deltaT = someTimer.getDeltaValue();
   if (!doRec) show();
   message("Timing result:\n");
   int numFrames = (int)((maxt-mint)/stept) + 1;
   debugOutput << "No. frames:   " << numFrames << "\n";
   debugOutput << "Time [ms]:    " << deltaT << "\n";
   if (numFrames > 0) {
      float32 avTime = (float)deltaT / (float)numFrames;
      debugOutput << "Average [ms]: " << avTime << "\n";
      if (avTime > 0) {
         debugOutput << "FPS:          " << 1000.0f / avTime << "\n";
      }
   }
}



// ---------- slots ----------

void MovieStudioWidget::sltOnlineHelp(){
   OnlineHelp::newTopic("MovieStudioWidget.html");
   OnlineHelp::show();
}


void MovieStudioWidget::on_BTClose_clicked() {
	hide();
}


void MovieStudioWidget::on_BTFileOpen_clicked() {
	std::string fname = FileDialogs::getSaveFileName(this, "Select FileName for BMP export", "All Files (*.*)", NULL, "", FileDialogs::EXPORT_DIR);
	if (fname != "") ui.LEFileName->setText(fname.c_str());
};


void MovieStudioWidget::on_CBMatchRealTime_stateChanged(int state) {
	ui.SBFrameRate->setEnabled(state==Qt::Checked);
	ui.SBMaxRepl->setEnabled(state==Qt::Checked);
}


void MovieStudioWidget::on_CBMainWinTimeline_stateChanged(int state) {
	ui.SBNumberOfFrames->setEnabled(state==Qt::Unchecked);
   if (state==Qt::Checked) {
		int numFrames = sceneEd->getAnimationBar()->getFramesCount();
		ui.SBNumberOfFrames->setValue(numFrames);
   }
}


void MovieStudioWidget::on_BTRecNow_clicked() {
	BTRecOrTimeNowClicked(true);
}


void MovieStudioWidget::on_BTTimeNow_clicked() {
	BTRecOrTimeNowClicked(false);
}


#include "MovieStudioWidget_moc.h"
void MovieStudioWidget::on_imageFormat_currentIndexChanged(int index) 
{
	
}
