//---------------------------------------------------------------------------
#include "StdAfx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PCCAnimationImporter.h"
//---------------------------------------------------------------------------
#include "SGRelativeTimeAnimationNode.h"
#include "ImportTools.h"
#include "StringHelper.h"
#include "CopyObjectProperties.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
#include "SGObjectNode.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
#include <QtCore/QDir> 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* PCCAnimationImporter::PCCAnimationImporterMetaClass = NULL;

const MetaClass* PCCAnimationImporter::getInstanceClass() const {
   return getClass();
}

Object* PCCAnimationImporter::newInstance() {
   return new PCCAnimationImporter;
}

const MetaClass* PCCAnimationImporter::getClass() {
   if (PCCAnimationImporterMetaClass == NULL) {
      PCCAnimationImporterMetaClass = new MetaClass("PCCAnimationImporter", "no description.",
                                       newInstance, sizeof(PCCAnimationImporter));
   }
   return PCCAnimationImporterMetaClass;
}

void PCCAnimationImporter::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 2);
   PCCAnimationImporterMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( PCCAnimationImporter )
	ADD_OBJECT_PROP(m_FramePCCreator, 0, PCCPointCloudCreator::getClass(), true );
	ADD_BOOLEAN_PROP(m_importAnimationOOC, 1)
	ADD_BOOLEAN_PROP(m_relativePath, 1)
	ADD_STRING_PROP(m_oocSwapFileNamePath, 1)
   ADD_BOOLEAN_PROP(m_entireDirectory, 2)
}

void PCCAnimationImporter::shutdown() {
   delete PCCAnimationImporterMetaClass;
}

PCCAnimationImporter::PCCAnimationImporter() {
   m_TargetName = "root";
	m_FramePCCreator = new PCCPointCloudCreator();
	m_FramePCCreator->setName("N/A (named automatically)");
	m_importAnimationOOC=false;
	m_relativePath=true;
	m_oocSwapFileNamePath="C:\\AnimationSwapFiles\\";
	m_entireDirectory = false;
};


PCCAnimationImporter::~PCCAnimationImporter() {
	delete m_FramePCCreator;
}


// ----


void PCCAnimationImporter::execute(Scene *scene) {

   // functions from PointCloudTools
   SceneGraphNode* SGNode = getSceneGraphNodeByName(scene, m_TargetName);
	if (SGNode==NULL) {
		WARNINGMESSAGE("PCCAnimationImporter::execute()\nTarget \"" + m_TargetName + "\" not found in scene graph!");
		return;
	};

   QDir dir;
   QStringList dirEntries;
   if (!m_entireDirectory) {
	   if (!X4::FileExists(m_InputFileName.c_str())) {
		   WARNINGMESSAGE("PCCAnimationImporter::execute()\nFile \"" + m_InputFileName + "\" not found. Please input first numerated file of animation sequence.");
		   return;
	   };
	} else {
      dir = QDir((removeFilenameFromPath(m_InputFileName)).c_str());
      if (!dir.exists()) {
         WARNINGMESSAGE("PCCAnimationImporter::execute()\nDirectory \"" + m_InputFileName + "\" not found. Please specify a path to the directory to be imported.");
         return;
      }
      if (dir.count() == 0) {
         WARNINGMESSAGE("PCCAnimationImporter::execute()\nDirectory \"" + m_InputFileName + "\" does not contain any files.");
         return;
      }
      //dir.setFilter(QDir::Files | QDir::Readable | QDir::NoDotAndDotDot);
      //dir.setSorting(QDir::Name);
      dirEntries = dir.entryList(QDir::Files | QDir::Readable | QDir::NoDotAndDotDot, QDir::Name);
	}

	SceneGraphNode *AnimNode = (SceneGraphNode*)(new SGRelativeTimeAnimationNode());
   AnimNode->setName("data");
	if (!SGNode->addChildNode(AnimNode)) {
		WARNINGMESSAGE("PCCAnimationImporter::execute()\nParentNode \"" + m_TargetName + "\" does not accept SGSimpleAnimationNode as child node!");
		return;
	};

	card32 counter = 0;
	std::string tmpCloudPath = m_TargetName + "/" + AnimNode->getName() + "/";
	std::string currentFileName;
	if (m_entireDirectory) {
	   currentFileName = qString2STLString(dirEntries[0]);
	} else {
	   currentFileName = m_InputFileName;
	}
	int nextIndex = 1;
	while (currentFileName!="") {

		if (strUCase(extractFileExt(currentFileName))==".x4obj") {
			// execute X4 .x4obj importer
			execX4ObjectImport(dynamic_cast<SGListNode*>(AnimNode));

		} else {
			// execute import by reader/adder
			std::string tmpCloudName = tmpCloudPath + AnimNode->getName() + "_Frame" + intToStr(counter);
			
			m_FramePCCreator->setName(tmpCloudName);
			m_FramePCCreator->executeCreator(scene);

			PointCloud* pc = getPointCloud(scene, tmpCloudName);
			SGObjectNode *node = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(scene, tmpCloudName));
			std::string t = m_InputFileName;
			m_InputFileName = qString2STLString(dir.absoluteFilePath(currentFileName.c_str()));
			execImporter(pc);
			m_InputFileName = t;
			if (m_entireDirectory) {
			   node->setName(currentFileName);
			}
			counter++;

			if(m_importAnimationOOC){
				x4Assert(pc && node) ;

				if (dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pc)) {
					warning("pc is already of type GenericSwapAllOutOfCorePointCloud - item not converted.");
					continue;
				}
				GenericSwapAllOutOfCorePointCloud *newGSA = new GenericSwapAllOutOfCorePointCloud();
				newGSA->clearAndSetup(pc, m_oocSwapFileNamePath+node->getName()+"_anim.swap.x4obj", m_relativePath);
				//node->removeSceneObject();
				node->clearAndSetup(newGSA);
			}
		};

      if (m_entireDirectory) {
         if (nextIndex < dirEntries.count()) {
            currentFileName = qString2STLString(dirEntries[nextIndex]);
            nextIndex++;
         } else {
            currentFileName = "";
         }
      } else {
   		currentFileName = getExistingNextNumberedFileName(currentFileName);
	  }
	}

   //ObjectViewsTable::update(scene);

}


