//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCAnimationImporterEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCCAnimationImporter.h"
#include "PointCloudImporterReader.h"
#include "StringHelper.h"
#include "FileDialogs.h"
#include "ObjectBrowserWidget.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


IMPLEMENT_CLASS_EDITOR( PCCAnimationImporter )

PCCAnimationImporterEditorWidget::PCCAnimationImporterEditorWidget(QWidget * parent, Qt::WindowFlags f)
:QWidget(parent,f)
{
	m_Object = NULL;

	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	// used Qt auto-connect feature for slots

	// init PointCloudImporterReader combobox
	ui.CBImporterReader->collectClasses("PointCloudImporterReader");
}

PCCAnimationImporterEditorWidget::~PCCAnimationImporterEditorWidget(void)
{
}

void PCCAnimationImporterEditorWidget::setup( PCCAnimationImporter * obj )
{
	m_Object = obj;
}

void PCCAnimationImporterEditorWidget::updateEditor()
{
	ui.LEFileName->setText(QString(m_Object->getInputFileName().c_str()));
	ui.LESwapFileName->setText(QString(m_Object->getOocSwapFileName().c_str()));
	ui.LETarget->setText(QString(m_Object->getTargetName().c_str()));
	ui.CBImporterReader->setSelection(m_Object->getImporterReader()->getInstanceClass()->getClassName()); 
	ui.GBParametricWrite->setChecked(m_Object->getParamWriteByCoords());
	ui.LEParamCoordsChannel->setText(QString(m_Object->getParamCoordsChannel().c_str()));
	ui.SBParamScaleX->setValue(m_Object->getPCoordsScaleX());
	ui.SBParamScaleY->setValue(m_Object->getPCoordsScaleY());
	ui.SBParamOffsetX->setValue(m_Object->getPCoordsOffsetX());
	ui.SBParamOffsetY->setValue(m_Object->getPCoordsOffsetY());
	ui.cbOOC->setChecked(m_Object->getImportAnimationOOC());
	ui.cbRelativePath->setChecked(m_Object->getRelativePath());
	ui.cbEntireDirectory->setChecked(m_Object->getImportEntireDirectory());
}


// ---------- user functions ----------

void PCCAnimationImporterEditorWidget::switchImporterToFileExtension(std::string ext){

	if (!ui.cbDetectByExtension->isChecked()) return;

	if (ext==".X4OBJ") {
		// X4 object importer
		ui.PBFramePCEditSubType->setEnabled(false);
		ui.CBImporterReader->setEnabled(false);
		ui.PBEdit->setEnabled(false);
		ui.GBParametricWrite->setEnabled(false);

	} else {
		// other importer
		ui.PBFramePCEditSubType->setEnabled(true);
		ui.CBImporterReader->setEnabled(true);
		ui.PBEdit->setEnabled(true);
		ui.GBParametricWrite->setEnabled(true);

		if ((ext==".BMP")||(ext==".JPG")||(ext==".PNG"))
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReader2DImages"));
		}
		else if (ext==".WRL")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderVRML"));
		}
		else if (ext==".PLY")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderPLY"));
		}
		else if ((ext==".OBJ")||(ext==".SMF"))
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderSMFOBJ"));
		}
		else if (ext==".TXT")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderCustomASCII"));
		}
		else if (ext==".OFF")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderOFF"));
		}
		else if (ext==".PNT")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderCustom3DScan"));
		}

		ui.CBImporterReader->setSelection(m_Object->getImporterReader()->getInstanceClass()->getClassName());

	};

}



// ---------- slots ----------

void PCCAnimationImporterEditorWidget::on_LEFileName_textEdited(QString s){
	m_Object->setInputFileName(qPrintable(s));
}


void PCCAnimationImporterEditorWidget::on_LEFileName_editingFinished(){
	switchImporterToFileExtension(strUCase(extractFileExt(m_Object->getInputFileName())));
}


void PCCAnimationImporterEditorWidget::on_PBLoadFile_clicked(){
	std::string dir = removeFilenameFromPath(qPrintable(ui.LEFileName->text()));
	std::string s = FileDialogs::getOpenFileName(this, "Choose a file", "All Files (*.*)", NULL, dir, FileDialogs::IMPORT_DIR );
	if (s != ""){ 
		//set filename
		ui.LEFileName->setText(s.c_str());
		m_Object->setInputFileName(s);
		switchImporterToFileExtension(strUCase(extractFileExt(s)));
	}
}


void PCCAnimationImporterEditorWidget::on_CBImporterReader_currentIndexChanged(QString v){
	if (m_Object==NULL) return;
	std::string  s = v.toLatin1().constData();
	const MetaClass *newImporterMetaClass = MetaClass::getClassByName(s);
	x4Assert (newImporterMetaClass != NULL );

	m_Object->setImporterReader((PointCloudImporterReader*)(newImporterMetaClass->newInstance()));
}


void PCCAnimationImporterEditorWidget::on_PBEdit_clicked(){
	ClassEditor * ce = (ClassEditor*)m_Object->getImporterReader()->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
	ObjectViewsTable::bindView( m_Object->getImporterReader(), ce );
   ObjectBrowserWidget *browser = new ObjectBrowserWidget(m_Object->getImporterReader(), ce, this, Qt::Window);
	QWidget * w = ce->makeEditor( browser->getEditorContainer(), m_Object->getImporterReader() );
   browser->setupEditor(w);
   browser->show();
   
//	w->setWindowFlags( Qt::Window );
//	w->setWindowTitle( objectName.c_str() );
//	w->move( QCursor::pos() );
//	w->setAttribute(Qt::WA_DeleteOnClose);
//	w->show();

}


void PCCAnimationImporterEditorWidget::on_LETarget_textEdited(QString s){
	m_Object->setTargetName(qPrintable(s));
}


void  PCCAnimationImporterEditorWidget::on_GBParametricWrite_toggled(bool state) {
	m_Object->setParamWriteByCoords(state);
}

void X4::PCCAnimationImporterEditorWidget::on_cbOOC_toggled(bool t)
{
	m_Object->setImportAnimationOOC(t);
}

void X4::PCCAnimationImporterEditorWidget::on_cbRelativePath_toggled(bool t)
{
	m_Object->setRelativePath(t);
}

void X4::PCCAnimationImporterEditorWidget::on_LESwapFileName_textEdited( QString s)
{
	m_Object->setOocSwapFileName(qPrintable(s));
}

void PCCAnimationImporterEditorWidget::on_LEParamCoordsChannel_textEdited(QString text) {
	m_Object->setParamCoordsChannel(qPrintable(text));
}


void PCCAnimationImporterEditorWidget::on_SBParamScaleX_valueChanged(double value) {
	m_Object->setPCoordsScaleX(value);
}


void PCCAnimationImporterEditorWidget::on_SBParamScaleY_valueChanged(double value) {
	m_Object->setPCoordsScaleY(value);
}


void PCCAnimationImporterEditorWidget::on_SBParamOffsetX_valueChanged(double value) {
	m_Object->setPCoordsOffsetX(value);
}


void PCCAnimationImporterEditorWidget::on_SBParamOffsetY_valueChanged(double value) {
	m_Object->setPCoordsOffsetY(value);
}


void PCCAnimationImporterEditorWidget::on_PBFramePCEditSubType_clicked() {
	m_Object->getFramePCCreator();

	ClassEditor * ce = (ClassEditor*)m_Object->getFramePCCreator()->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
	ObjectViewsTable::bindView( m_Object->getFramePCCreator(), ce );
   ObjectBrowserWidget *browser = new ObjectBrowserWidget(m_Object->getFramePCCreator(), ce, this, Qt::Window);
	QWidget * w = ce->makeEditor( browser->getEditorContainer() , m_Object->getFramePCCreator() );
   browser->setupEditor(w);
   browser->show();
//	w->setWindowFlags( Qt::Window );
//	w->setWindowTitle( objectName.c_str() );
//	w->move( QCursor::pos() );
//	w->setAttribute(Qt::WA_DeleteOnClose);
//	w->show();

}


#include "PCCAnimationImporterEditorWidget_moc.h"

void PCCAnimationImporterEditorWidget::on_cbEntireDirectory_toggled(bool) 
{
   m_Object->setImportEntireDirectory(ui.cbEntireDirectory->isChecked());
}
