//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCAutoAlign.h"
//---------------------------------------------------------------------------
#include "BoundingBox.h"
#include "BoundingBox.inline.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra.inline.h"
#include "BooleanClassProperty.h"

#include "PointCloudIterators.h"
#include "PointCloudTools.h"

#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "ClassMethods.h"
#include "SGListNode.h"
#include "SGObjectNode.h"
#include "SceneGraphTools.h"
//---------------------------------------------------------------------------
 

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


static std::string LevelParamDescription( mpcard q ) {
	switch(q) {
		default:
		case 2:
			return "z-axis";
		case 1:
			return "y-axis";
		case 0:
			return "x-axis";
	}
}

//---------------------------------------------------------------------------


MetaClass* PCCAutoAlign::PCCAutoAlignMetaClass = NULL;

const MetaClass* PCCAutoAlign::getInstanceClass() const {
   return getClass();
}

Object* PCCAutoAlign::newInstance() {
   return new PCCAutoAlign;
}

const MetaClass* PCCAutoAlign::getClass() {
   if (PCCAutoAlignMetaClass==NULL) {
      PCCAutoAlignMetaClass = new MetaClass("PCCAutoAlign", "no description.",
                                  newInstance, sizeof(PCCAutoAlign));
   }
   return PCCAutoAlignMetaClass;
}

void PCCAutoAlign::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 3);
   PCCAutoAlignMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT(PCCAutoAlign)

	ADD_CARD32_PROP_UPDATE_DESCR( LevelParam, 2 , &levelParamUpdateFunc, LevelParamDescription, 3, NULL );
   
	ADD_BOOLEAN_PROP( center , 0)
	ADD_BOOLEAN_PROP( rescale , 0)
	ADD_BOOLEAN_PROP( adjustCamera , 0)

	ADD_FLOAT32_PROP(GridWidth, 3)
	ADD_BOOLEAN_PROP(readFromList, 3)
	ADD_STRING_PROP(sourceList, 3)

	// ------------------
	ADD_NOARGS_METHOD(level)
	ADD_NOARGS_METHOD(alignToGrid)
}

void PCCAutoAlign::shutdown() {
   delete PCCAutoAlignMetaClass;
}

void PCCAutoAlign::levelParamUpdateFunc() {}

// ------------------------------------------------------------

PCCAutoAlign::PCCAutoAlign() {
   center = true;
   rescale = true;
   adjustCamera = false;
   LevelParam = 2;
   GridWidth = 1;
   readFromList = false;
   sourceList = "root/data";
}


void PCCAutoAlign::alignToGrid() {
	Scene* scene = getScene();
	std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
	if (readFromList) {
		SGListNode *list = dynamic_cast<SGListNode*>(getSceneGraphNode(getScene(),sourceList));
		if (list) {
			selection.clear();
			for (mpcard i=0; i<list->getNumChildNodes(NULL); i++) {
				SGObjectNode *objNode = dynamic_cast<SGObjectNode*>(list->getChildNode(NULL, i));
				if (objNode) {
					std::string pre = sourceList;
					if (pre.size() >0 && pre[pre.size()-1] != '/') {
						pre += "/";
					}
					pre += objNode->getName();
					selection.push_back(pre);
				}
			}
		}
	}
	for (unsigned m = 0; m < selection.size(); m++ ) {
		alignToGrid(getPointCloud( scene, selection[m] ), m, selection.size());
	}
}

void PCCAutoAlign::alignToGrid(PointCloud* pc, mpcard index, mpcard numGridEntries) {

	if (!pc->providesAttribute("position")) {
		warning("Selected pointcloud does not provide attribute 'position'");
		return;
	}

	AAT vertexAAT = pc->getAAT("position");

	SceneObjectIterator *it = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	if (it == NULL) {
		warning("Cannot create iterator");
		return;
	}
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(it);
    ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(it);

	if(basicIt->atEnd()) {
		warning("Empty pointcloud");
		return;
	}

	x4Assert(index < numGridEntries);

	Vector3f u;
	Vector3f v;
	u.setZero();
	v.setZero();
	switch(LevelParam) {
		default:
		case 2: 
			u[0] = 1;
			v[1] = 1;
			break;
		case 1: 
			u[0] = 1;
			v[2] = 1;
			break;
		case 0: 
			u[1] = 1;
			v[2] = 1;
			break;
	}

	mpcard offset = (mpcard) sqrt((float32) numGridEntries);

	mpcard du = index/offset;
	mpcard dv = index%offset;
	x4Assert(du*offset+dv == index);

	Vector3f t = u*(du*GridWidth) + v*(dv*GridWidth);


	while(!basicIt->atEnd()) {
		Vector3f& pos = basicIt->get3f(vertexAAT);

		pos += t;

		modIt->set3f(vertexAAT, pos);
		basicIt->next();

	} // end of while

	delete it;

}



void PCCAutoAlign::level() {
	Scene* scene = getScene();
	std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
	for (unsigned m = 0; m < selection.size(); m++ )
	{
		level(getPointCloud( scene, selection[m] ));
	}
}

void PCCAutoAlign::level(PointCloud* pc) {

	if (!pc->providesAttribute("position")) {
		warning("Selected pointcloud does not provide attribute 'position'");
		return;
	}

	AAT vertexAAT = pc->getAAT("position");

	SceneObjectIterator *it = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	if (it == NULL) {
		warning("Cannot create iterator");
		return;
	}
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(it);
    ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(it);

	if(basicIt->atEnd()) {
		warning("Empty pointcloud");
		return;
	}

	float32 minHeight = basicIt->get3f(vertexAAT)[LevelParam];
	while(!basicIt->atEnd()) {
		Vector3f& pos = basicIt->get3f(vertexAAT);
		float height = pos[LevelParam];
		if(height < minHeight) minHeight = height;
		basicIt->next();
	}
	basicIt->reset();

	while(!basicIt->atEnd()) {
		Vector3f& pos = basicIt->get3f(vertexAAT);
		pos[LevelParam] -= minHeight;
		modIt->set3f(vertexAAT, pos);
		basicIt->next();
	}

	delete it;

}



void PCCAutoAlign::execute(Scene* scene)
{
	std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
	for (unsigned m = 0; m < selection.size(); m++ )
	{
		execute_at(getPointCloud( scene, selection[m] ));
	}
}

void PCCAutoAlign::execute_at(PointCloud* pc)
{
   if (!rescale && !center) return;

   // calc bb
   BoundingBox3f bb;
   bool posset = false;

   //unsigned int m;
   //std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
   //PointCloud* pc;
   //for ( m = 0; m < selection.size(); m++ )
   {
      //pc = getPointCloud( scene, selection[m] );

      if (!pc->providesAttribute("position")) return;
      AAT POSITION = pc->getAAT("position");
      SceneObjectIterator *newIt = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
      if (newIt == NULL) return;
      BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
      //if (basicIt == NULL) {delete newIt; continue;}

      while(!basicIt->atEnd()) {
         Vector3f pos = basicIt->get3f(POSITION);
         if (posset) {
            bb.addPoint(pos);
         } else {
            bb = BoundingBox3f(pos);
            posset = true;
         }
         basicIt->next();
      };

      delete newIt;
   }

   // compute transform
   Matrix4f transform = IDENTITY4F;
   Vector3f mean = bb.getCenter();
   if (center) {
      transform = makeTranslation4f(-mean);
   }
   float32 diameter = bb.getMaxSideLength();
   if (rescale && diameter > 1E-10) {
      float32 s = 2.0/diameter;
      transform = makeScale4f(s,s,s) * transform;
   }

   if ( adjustCamera ){
      /// Only adjust camera
      Camera& cam = scene->getRootState()->staticState->camera;
      cam.setup( cam.getPosition(), mean, cam.getUp() );
   }
   
   {
      // Modify point data of selectec points
      //for ( m = 0; m < selection.size(); m++ ){
         //pc = getPointCloud( scene, selection[m] );

         //if (!pc->providesAttribute("position")) continue;
         AAT POSITION = pc->getAAT("position");

         SceneObjectIterator *newIt = pc->createIterator(SOIT(SOIT::CAP_MODIFYABLE_PC));
         //if (newIt == NULL) continue;
         BasicPointCloudIterator *baseIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
         //if (baseIt == NULL) {delete newIt; continue;}
         ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
         //if (modIt == NULL) {delete newIt; continue;}

         while(!baseIt->atEnd()) {
            Vector3f pos = baseIt->get3f(POSITION);
            Vector3f newPos = transformVector3f(transform, pos);
            modIt->set3f(POSITION, newPos);
            baseIt->next();
         };

         delete newIt;
      //}
   }
}

