//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCCComputeFlags.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "Scene.h"
#include "PointCloudIterators.h"
#include "BooleanClassProperty.h"
#include "StringClassProperty.h"
#include "NumericalClassProperty.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "SceneGraphState.h"
#include "PointCloudTools.h"
#include "LinearAlgebra.inline.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* PCCComputeFlags::PCCComputeFlagsMetaClass = NULL;

const MetaClass* PCCComputeFlags::getInstanceClass() const {
   return getClass();
}

Object* PCCComputeFlags::newInstance() {
   return new PCCComputeFlags;
}

const MetaClass* PCCComputeFlags::getClass() {
   if (PCCComputeFlagsMetaClass==NULL) {
      PCCComputeFlagsMetaClass = new MetaClass("PCCComputeFlags", "no description.",
                                  newInstance, sizeof(PCCComputeFlags));
   }
   return PCCComputeFlagsMetaClass;
}

void PCCComputeFlags::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   PCCComputeFlagsMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT( PCCComputeFlags )
	ADD_FLOAT32_PROP(referenceValue, 0)
	ADD_STRING_PROP(channel,0)
	ADD_CARD32_PROP(channelIndex,0)
	ADD_INT32_PROP(flagsToBeSet,0)
	ADD_BOOLEAN_PROP(testOnGreater,0)

}

void PCCComputeFlags::shutdown() {
   delete PCCComputeFlagsMetaClass;
}


// ----


PCCComputeFlags::PCCComputeFlags() {
   referenceValue = 25;
   channel = "intensity";
   channelIndex = 0;
   flagsToBeSet = PF_FLAG_INVALID;
   testOnGreater = false;
}

void PCCComputeFlags::execute(Scene *scene) {
   unsigned int m;
   std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
   PointCloud* pc;
   for ( m = 0; m < selection.size(); m++ ){
      pc = getPointCloud( scene, selection[m] );

      checkAttribute( scene, selection[m], "flags", 1, VAD::DATA_FORMAT_INT32 );

      if (!pc->providesAttribute(channel)) continue;

      VAD atDescr = pc->getDescr()->getAttribute(channel);
      AAT CHANNEL_AAT = pc->getAAT(channel);

      if (atDescr.numComponents <= channelIndex) continue;

      AAT FLAGS = pc->getAAT("flags");

      SceneObjectIterator *it = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
      BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(it);
      ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(it);
      if (modIt != NULL) {
         while (!bIt->atEnd()) {
            float32 value = 0;
            switch (atDescr.dataFormat) {
             case VAD::DATA_FORMAT_CARD8:
               if (atDescr.numComponents == 1) value = bIt->get1ub(CHANNEL_AAT);
               if (atDescr.numComponents == 2) value = bIt->get2ub(CHANNEL_AAT)[channelIndex];
               if (atDescr.numComponents == 3) value = bIt->get3ub(CHANNEL_AAT)[channelIndex];
               if (atDescr.numComponents == 4) value = bIt->get4ub(CHANNEL_AAT)[channelIndex];
             break;
             case VAD::DATA_FORMAT_FLOAT32:
               if (atDescr.numComponents == 1) value = bIt->get1f(CHANNEL_AAT);
               if (atDescr.numComponents == 2) value = bIt->get2f(CHANNEL_AAT)[channelIndex];
               if (atDescr.numComponents == 3) value = bIt->get3f(CHANNEL_AAT)[channelIndex];
               if (atDescr.numComponents == 4) value = bIt->get4f(CHANNEL_AAT)[channelIndex];
             break;
             case VAD::DATA_FORMAT_INT32:
               if (atDescr.numComponents == 1) value = bIt->get1i(CHANNEL_AAT);
               if (atDescr.numComponents == 2) value = bIt->get2i(CHANNEL_AAT)[channelIndex];
               if (atDescr.numComponents == 3) value = bIt->get3i(CHANNEL_AAT)[channelIndex];
               if (atDescr.numComponents == 4) value = bIt->get4i(CHANNEL_AAT)[channelIndex];
             break;
            }
            bool setFlagMask = ((value <= referenceValue) ^ testOnGreater);
            int32 flags = bIt->get1i(FLAGS);
            if (setFlagMask) {
               flags |= flagsToBeSet;
            } else {
               flags &= 0xFFFFFFFF ^ flagsToBeSet;
            }
            modIt->set1i(FLAGS, flags);
            bIt->next();
         }
      }
      delete it;
   }

}
