//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCDirectoryImporter.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PCCImporter.h"
#include "StringHelper.h"
#include "SceneGraphTools.h"
#include "VertexArray.h"
#include "SGRelativeTimeAnimationNode.h"
#include <Windows.h>
#include <fstream>
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS(PCCDirectoryImporter, 0)
{
	BEGIN_CLASS_INIT(PCCDirectoryImporter);
   ADD_STRING_PROP(directory, 0)
   ADD_STRING_PROP(fileEnding, 0)
   ADD_OBJECT_PROP(importer, 0, PCCImporter::getClass(), true);
   ADD_BOOLEAN_PROP(ignoreExceptions, 0)
   ADD_BOOLEAN_PROP(outputFiles, 0)
}

PCCDirectoryImporter::PCCDirectoryImporter()
{
   fileEnding = ".dat";
   importer = new PCCImporter();
   ignoreExceptions = false;
   outputFiles = true;
}

PCCDirectoryImporter::~PCCDirectoryImporter()
{
   delete importer;
}

void PCCDirectoryImporter::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj, this);
}


void PCCDirectoryImporter::execute(Scene *scene) 
{
   importer->connectScene(scene, getCallback(), sceneEditorWidget);
   recursiveCollect(directory);
}

void PCCDirectoryImporter::recursiveCollect(std::string file) 
{
   QFileInfo info(file.c_str());
   if (info.isDir()) {
      if (outputFiles) debugOutput << file << "\n";
      //debugOutput.flush();
      QDir dir(file.c_str());
      for (unsigned i=0; i<dir.count(); i++) {
         if (!(dir[i] == QString(".") || dir[i] == QString(".."))) {
            recursiveCollect(qString2STLString(dir.absoluteFilePath(dir[i])));
         }
      }
   } else {
      if (info.isFile() && info.exists() && info.isReadable()) {
         bool readme = ((file.size() > 0) && (file.size() >= fileEnding.size()));
         if (readme) {
            for (unsigned i=0; i<fileEnding.size(); i++) {
               if (file[file.size()-fileEnding.size()+i] != fileEnding[i]) {
                  return;
               }
            }
         }
         if (outputFiles) debugOutput << file << "\n";
         //debugOutput.flush();
         try {
            importer->setInputFileName(qString2STLString(QDir::toNativeSeparators(file.c_str())));
            importer->execute(scene);
         } catch (...) {
            if (!ignoreExceptions) {
               throw;
            } else {
               warning("PCCDirectoryImporter::recursiveCollect(): importer has thrown an exception - ignored.");
            }
         }
      }
   }
}



void X4::PCCDirectoryImporter::importHannover()
{
	// parameters:
	std::string folderName = "I:\\aberner\\kirche\\Marktkirche";
	std::string targetNodeName = "root/tri";




	SceneGraphNode *node = getSceneGraphNode(scene, targetNodeName);
	if (!node) {
		SGRelativeTimeAnimationNode *nn = new SGRelativeTimeAnimationNode();
		nn->setName(extractSGNode(targetNodeName));
		addSceneGraphNode(scene, extractSGPath(targetNodeName), nn);
	}

	QDir myDir(folderName.c_str());
	QStringList wantedFiles("*.asc");
	QStringList list = myDir.entryList(wantedFiles);

	debugOutput << list.size() << " Files found:\n";

	for (mpcard i = 0; i < (mpcard)list.size(); i++)
	{
		QString name = list[i];
		QStringList nameList = name.split(".");
		QString otherName; 
		if (i+1 < 10)
			otherName = "ScanPos0" + QString(intToStr(i+1).c_str() ) +QString(".DAT");
		else
			otherName = "ScanPos" + QString(intToStr(i+1).c_str() ) +QString(".DAT");


		std::string nameVert =  list[i].toLatin1().constData(); 
		std::string nameTri  =  otherName.toLatin1().constData(); 
		nameVert = folderName + "/" + nameVert;
		nameTri  = folderName + "/" + nameTri;

		debugOutput << "loading: " << nameVert<< " - "<< nameTri << "\n";
		debugOutput.flush();

		std::ifstream in;
		in.clear();
		in.open(nameVert.c_str(),std::ifstream::in);
		if (!in.good()) { 
			warning("importHannover\nError opening file \"" + nameVert + "\"!"); 
			return ;
		};

		std::ifstream inTri;
		inTri.clear();
		inTri.open(nameTri.c_str(),std::ifstream::in);
		if (!inTri.good()) { 
			warning("importHannover\nError opening file \"" + nameTri + "\"!"); 
			return ;
		};


		QString line;

		std::vector<Vector3f> positions;
		std::vector<float32>  intensities;


		while (!in.eof())
		{
			char _buffer[1024];
			in.getline(_buffer, 1024);
			line = QString::fromLatin1(&_buffer[0]);

			// Read vertices
			QStringList strList = line.split( ", ", QString::SkipEmptyParts );
			if ( strList.size() == 4 ){
				Vector3f p = makeVector3f( strList.at( 0 ).toFloat(),
					strList.at( 1 ).toFloat(),
					strList.at( 2 ).toFloat() );
				float32 intens = strList.at( 3 ).toFloat();
				positions.push_back( p );
				intensities.push_back(intens);
			}
		}
		in.close();


		Matrix4f trans;
		mpcard at = 0;

		while (!inTri.eof())
		{
			char _buffer[1024];
			inTri.getline(_buffer, 1024);
			line = QString::fromLatin1(&_buffer[0]);

			// Read faces
			QStringList strList = line.split( " ", QString::SkipEmptyParts );
			if ( !line.isEmpty() ){
				// Only support triangle meshes
				if (  strList.size() == 4)
				{
					int base = 0;

					Vector4f line = makeVector4f( strList.at( base + 0 ).toFloat(), 
						strList.at( base + 1 ).toFloat(),
						strList.at( base + 2 ).toFloat(),
						strList.at( base + 3 ).toFloat()
						);

					trans[at++] = line;
				}
			}
		}

		inTri.close();

		trans = trans.transpose();


		UnstructuredInCorePointCloud* mesh = new UnstructuredInCorePointCloud();
		mesh->clearAndSetup(positions.size());
		checkAttribute(mesh, "intensity", 1, VAD::DATA_FORMAT_FLOAT32);
		VertexArray vert(mesh);
		PointSet* ps = mesh->getPointSet();
		AAT INT = ps->getAAT("intensity");
		for (mpcard p = 0; p < positions.size(); p++)
		{
			vert.setPosition3f(p, positions[p]);
			ps->set1f(p, INT, intensities[p]);
		}

		mesh->setTransformation(trans);

		mesh->setMaterialIndex(0);

		addPointCloud(scene, mesh, targetNodeName + "/" + nameList[0].toLatin1().constData() );
	}
}


void X4::PCCDirectoryImporter::importUrbanChallenge()
{
	// parameters:
	std::string folderName = "C:\\urban\\data\\Marktkirche";
	std::string targetNodeName = "root/tri";

	SceneGraphNode *node = getSceneGraphNode(scene, targetNodeName);
	if (!node) {
		SGRelativeTimeAnimationNode *nn = new SGRelativeTimeAnimationNode();
		nn->setName(extractSGNode(targetNodeName));
		addSceneGraphNode(scene, extractSGPath(targetNodeName), nn);
	}

	QDir myDir(folderName.c_str());
	QStringList wantedFiles("*.asc");
	QStringList list = myDir.entryList(wantedFiles);

	debugOutput << list.size() << " Files found:\n";

	for (mpcard i = 0; i < (mpcard)list.size(); i++)
	{
		QString name = list[i];
		QStringList nameList = name.split(".");
		QString otherName; 
		if (i+1 < 10)
			otherName = "ScanPos0" + QString(intToStr(i+1).c_str() ) +QString(".DAT");
		else
			otherName = "ScanPos" + QString(intToStr(i+1).c_str() ) +QString(".DAT");


		std::string nameVert =  list[i].toLatin1().constData(); 
		std::string nameTri  =  otherName.toLatin1().constData(); 
		nameVert = folderName + "/" + nameVert;
		nameTri  = folderName + "/" + nameTri;

		debugOutput << "loading: " << nameVert<< " - "<< nameTri << "\n";
		debugOutput.flush();

		std::ifstream in;
		in.clear();
		in.open(nameVert.c_str(),std::ifstream::in);
		if (!in.good()) { 
			warning("importHannover\nError opening file \"" + nameVert + "\"!"); 
			return ;
		};

		std::ifstream inTri;
		inTri.clear();
		inTri.open(nameTri.c_str(),std::ifstream::in);
		if (!inTri.good()) { 
			warning("importHannover\nError opening file \"" + nameTri + "\"!"); 
			return ;
		};


		QString line;

		std::vector<Vector3f> positions;
		std::vector<float32>  intensities;


		while (!in.eof())
		{
			char _buffer[1024];
			in.getline(_buffer, 1024);
			line = QString::fromLatin1(&_buffer[0]);

			// Read vertices
			QStringList strList = line.split( ", ", QString::SkipEmptyParts );
			if ( strList.size() == 4 ){
				Vector3f p = makeVector3f( strList.at( 0 ).toFloat(),
					strList.at( 1 ).toFloat(),
					strList.at( 2 ).toFloat() );
				float32 intens = strList.at( 3 ).toFloat();
				positions.push_back( p );
				intensities.push_back(intens);
			}
		}
		in.close();


		Matrix4f trans;
		mpcard at = 0;

		while (!inTri.eof())
		{
			char _buffer[1024];
			inTri.getline(_buffer, 1024);
			line = QString::fromLatin1(&_buffer[0]);

			// Read faces
			QStringList strList = line.split( " ", QString::SkipEmptyParts );
			if ( !line.isEmpty() ){
				// Only support triangle meshes
				if (  strList.size() == 4)
				{
					int base = 0;

					Vector4f line = makeVector4f( strList.at( base + 0 ).toFloat(), 
						strList.at( base + 1 ).toFloat(),
						strList.at( base + 2 ).toFloat(),
						strList.at( base + 3 ).toFloat()
						);

					trans[at++] = line;
				}
			}
		}

		inTri.close();

		trans = trans.transpose();


		UnstructuredInCorePointCloud* mesh = new UnstructuredInCorePointCloud();
		mesh->clearAndSetup(positions.size());
		checkAttribute(mesh, "intensity", 1, VAD::DATA_FORMAT_FLOAT32);
		VertexArray vert(mesh);
		PointSet* ps = mesh->getPointSet();
		AAT INT = ps->getAAT("intensity");
		for (mpcard p = 0; p < positions.size(); p++)
		{
			vert.setPosition3f(p, positions[p]);
			ps->set1f(p, INT, intensities[p]);
		}

		mesh->setTransformation(trans);

		mesh->setMaterialIndex(0);

		addPointCloud(scene, mesh, targetNodeName + "/" + nameList[0].toLatin1().constData() );
	}



}

