#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCImportAnalyze.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "FileNamePropertyEditor.h"
#include "MultiResolutionInCoreVolume.h"
#include "RegularInCoreVolume.h"
#include "RegularVolume.h"
#include <fstream>
#include "Stringhelper.h"
#include "PointCloudTools.h"

//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCCImportAnalyze ,2)
{
	BEGIN_CLASS_INIT( PCCImportAnalyze );
	ADD_STRING_PROP(m_MRIHdr, 0);
	SET_EDITOR(FileNamePropertyEditor::getClass());
	ADD_STRING_PROP(m_MRIImg, 0);
	SET_EDITOR(FileNamePropertyEditor::getClass());
	ADD_STRING_PROP(m_MaskImg, 0);
	SET_EDITOR(FileNamePropertyEditor::getClass());
	ADD_STRING_PROP(m_LabelImg, 0);
	SET_EDITOR(FileNamePropertyEditor::getClass());
	ADD_STRING_PROP(m_PointCloudName,0);
}

PCCImportAnalyze::PCCImportAnalyze(void)
{
	m_PointCloudName	= "volume1";
	m_MRIHdr				= "C:/kerber/bio models/Brain MRI/CUMC12/Heads/m1.hdr";
	m_MRIImg				= "C:/kerber/bio models/Brain MRI/CUMC12/Heads/m1.img";
	m_MaskImg			= "C:/kerber/bio models/Brain MRI/CUMC12/BrainMasks/m1.img";
	m_LabelImg			= "C:/kerber/bio models/Brain MRI/CUMC12/Atlases/m1.img";
}

PCCImportAnalyze::~PCCImportAnalyze(void)
{
}

void PCCImportAnalyze::assign(const Object* obj, X4CopyContext *context)
{
	const PCCImportAnalyze * o = dynamic_cast<const PCCImportAnalyze*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::PCCImportAnalyze::execute( Scene * scene )
{
	RegularVolume * volumeData = new RegularVolume;

	std::ifstream hdrFile( m_MRIHdr.c_str(), ios::binary);
	if( !hdrFile.is_open())
	{
		debugOutput << "Cannot open file: " << m_MRIHdr << "\n";
		return;
	}
	std::ifstream imgFile( m_MRIImg.c_str(), ios::binary);
	if( !imgFile.is_open())
	{
		debugOutput << "Cannot open file: " << m_MRIImg << "\n";
		return;
	}
	std::ifstream maskFile( m_MaskImg.c_str(), ios::binary);
	if( !maskFile.is_open())
	{
		debugOutput << "Cannot open file: " << m_MaskImg << "\n";
		return;
	}
	std::ifstream labelFile( m_LabelImg.c_str(), ios::binary);
	if( !labelFile.is_open())
	{
		debugOutput << "Cannot open file: " << m_LabelImg << "\n";
		return;
	}
	
	std::vector<char> nonsense(1000);
	hdrFile.read( (char*)&nonsense[0], 42 * sizeof(char) );   // Start 0
	std::vector<card16> dims(3);
	hdrFile.read( (char*)&dims[0], 3 * sizeof(card16) ); // Start 42
	card16 dimX = dims[0];
	card16 dimY = dims[1];
	card16 dimZ = dims[2];
	hdrFile.read( (char*)&nonsense[0], 24 * sizeof(char) ); // Start 48
	std::vector<card16> dType(1);
	hdrFile.read( (char*)&dType[0], 1 * sizeof(card16) ); // Start 72
	card16 nBits = dType[0];
	hdrFile.read( (char*)&nonsense[0], 6 * sizeof(char) ); // Start 74
	std::vector<float> spacing(3);
	hdrFile.read( (char*)&spacing[0], 3 * sizeof(float) ); // Start 80
	float dX = spacing[0];
	float dY = spacing[1];
	float dZ = spacing[2];
 	debugOutput << "Dims      = [" <<  dimX  << "," << dimY  << "," << dimZ  << "]\n";
	debugOutput << "Spacing   = [" <<  dX  << "," << dY  << "," << dZ  << "]\n";
	debugOutput << "Precision = " <<  nBits  << "bits\n";
	
	// set up
	VertexDescriptor * vd = new VertexDescriptor;
	vd->pushAttrib( VAD("intensity",1,VAD::DATA_FORMAT_FLOAT32) );
	vd->pushAttrib( VAD("mask",1,VAD::DATA_FORMAT_FLOAT32) );
	vd->pushAttrib( VAD("label",1,VAD::DATA_FORMAT_FLOAT32) );
	PointSet* voxels = new PointSet;
	voxels->clearAndSetup( 1, dimX*dimY*dimZ, vd );

	Vector3f elementSpacing = makeVector3f(dX,dY,dZ);
	BoundingBox3f bb;
	bb.lowerCorner = elementSpacing * -0.5f;
	bb.upperCorner = elementSpacing *  0.5f;
	bb.upperCorner[0] *= dimX;
	bb.upperCorner[1] *= dimY;
	bb.upperCorner[2] *= dimZ;
	bb.lowerCorner[0] *= dimX;
	bb.lowerCorner[1] *= dimY;
	bb.lowerCorner[2] *= dimZ;
	
	AAT intAAT		= voxels->getAAT( "intensity" );
	AAT maskAAT		= voxels->getAAT( "mask" );
	AAT labelAAT	= voxels->getAAT( "label" );
	
	if(dType[0] == 16)
	{
		std::vector<card16> tmpBuffer1(dimX*dimY*dimZ);
		imgFile.read( (char*)&tmpBuffer1[0], dimX*dimY*dimZ * sizeof(card16) );
		std::vector<card16> tmpBuffer2(dimX*dimY*dimZ);
		maskFile.read( (char*)&tmpBuffer2[0], dimX*dimY*dimZ * sizeof(card16) );
		std::vector<card16> tmpBuffer3(dimX*dimY*dimZ);
		labelFile.read( (char*)&tmpBuffer3[0], dimX*dimY*dimZ * sizeof(card16) );
		for(card32 z = 0; z < (card32)dimZ; z++)
		{
			// WICHTIG!!! EVTL BEI ABWEICHENDEN DIMENSIONEN AUCH X UND Y IN DER REIHENFOLGE TAUSCHEN!!!
			for(card32 y = 0; y < (card32)dimY; y++)
			{
				for(card32 x = 0; x < (card32)dimX; x++)
				{
					card32 i = x+dimX*y+dimX*dimY*z;
					card16 intensity = tmpBuffer1[i];
					voxels->set1f(i, intAAT, (float)intensity);
					card16 masked = tmpBuffer2[i];
					if(masked == 1)
						voxels->set1f(i,maskAAT,1.0f);
					else if (masked == 0)
						voxels->set1f(i,maskAAT,0.0f);
					else
						debugOutput << "ERROR in importanalyze!\n";
					card16 label = tmpBuffer3[i];
					voxels->set1f(i,labelAAT,(float)label);
				}
			}
		}
	}	

	RegularVolume* v = new RegularVolume;
	v->setVoxelData(voxels,dimX,dimY,dimZ);
	v->setBoundingBox(bb);
//	v->normalizeChannel_1f_0_1("intensity");
//	v->normalizeChannel_1f_0_1("label");
	
	

	RegularInCoreVolume* volume = new RegularInCoreVolume;
	volume->setVoxels(v);
	addSceneObject(scene,volume,m_PointCloudName);
}

