#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCImportVolumeData.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "FileNamePropertyEditor.h"
#include "MultiResolutionInCoreVolume.h"
#include "RegularInCoreVolume.h"
#include "RegularVolume.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCCImportVolumeData , 3)
{
	BEGIN_CLASS_INIT( PCCImportVolumeData );
	ADD_STRING_PROP(m_FileName, 0);
	SET_EDITOR(FileNamePropertyEditor::getClass());
	ADD_STRING_PROP(m_PointCloudName,0);
	ADD_BOOLEAN_PROP(m_MultiRes, 1);
	ADD_BOOLEAN_PROP(m_CreatePositionChannel, 2);
	ADD_BOOLEAN_PROP(m_is16Bit, 3);
}

PCCImportVolumeData::PCCImportVolumeData(void)
{
	m_PointCloudName = "volume1";
	m_MultiRes = false;
	m_CreatePositionChannel = false;
	m_is16Bit = false;
}

PCCImportVolumeData::~PCCImportVolumeData(void)
{
}

void PCCImportVolumeData::assign(const Object* obj, X4CopyContext *context)
{
	const PCCImportVolumeData * o = dynamic_cast<const PCCImportVolumeData*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::PCCImportVolumeData::execute( Scene * scene )
{
	importAndAddToScene(m_FileName, scene);
}

PointCloud* X4::PCCImportVolumeData::importAndAddToScene( std::string filename, Scene * scene )
{

	RegularVolume * volumeData = new RegularVolume;
	if( !volumeData->loadFromMHDRaw(filename.c_str(),m_CreatePositionChannel,m_is16Bit) )
	{
		debugOutput << "PCCImportVolumeData::importAndAddToScene() - Couldn't load file: " << filename << "\n";
		return NULL;
	}
	if(!m_MultiRes)
	{
		RegularInCoreVolume * volume = new RegularInCoreVolume;
		volume->setVoxels(volumeData);
		addSceneObject(scene,volume,m_PointCloudName);
		volumeData->normalizeChannel_1f_0_1("intensity");
		return volume;
	}
	else
	{
		MultiResolutionInCoreVolume * volume = new MultiResolutionInCoreVolume;
		Vector3i dim = volumeData->getExtends();
		volume->setVoxels(volumeData);
		addSceneObject(scene,volume,m_PointCloudName);
		return volume;
	}
	

	return NULL;
}