#include "StdAfx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PCCImporterEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCCImporter.h"
#include "PointCloudImporterReader.h"
#include "StringHelper.h"
#include "FileDialogs.h"
#include "PointCloudImporterReaderMSDepthBMP.h"
#include "ObjectBrowserWidget.h"
//---------------------------------------------------------------------------
#include <qtCore/qdir>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


IMPLEMENT_CLASS_EDITOR( PCCImporter )

PCCImporterEditorWidget::PCCImporterEditorWidget(QWidget * parent, Qt::WindowFlags f)
:QWidget(parent,f)
{
	m_Object = NULL;

	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	// used Qt auto-connect feature for slots

	// init PointCloudImporterReader combobox
	ui.CBImporterReader->collectClasses("PointCloudImporterReader");
}

PCCImporterEditorWidget::~PCCImporterEditorWidget(void)
{
}

void PCCImporterEditorWidget::setup( PCCImporter * obj )
{
	m_Object = obj;
}

void PCCImporterEditorWidget::updateEditor()
{
	ui.LEFileName->setText(QString(m_Object->getInputFileName().c_str()));
	ui.LETarget->setText(QString(m_Object->getTargetName().c_str()));
	ui.CBImporterReader->setSelection(m_Object->getImporterReader()->getInstanceClass()->getClassName()); 
	ui.GBParametricWrite->setChecked(m_Object->getParamWriteByCoords());
	ui.LEParamCoordsChannel->setText(QString(m_Object->getParamCoordsChannel().c_str()));
	ui.SBParamScaleX->setValue(m_Object->getPCoordsScaleX());
	ui.SBParamScaleY->setValue(m_Object->getPCoordsScaleY());
	ui.SBParamOffsetX->setValue(m_Object->getPCoordsOffsetX());
	ui.SBParamOffsetY->setValue(m_Object->getPCoordsOffsetY());

}



// ---------- user functions ----------

void PCCImporterEditorWidget::switchImporterToFileExtension(std::string ext){

	if (ext==".X4OBJ") {
		// X4 object importer
		ui.CBImporterReader->setEnabled(false);
		ui.PBEdit->setEnabled(false);
		ui.GBParametricWrite->setEnabled(false);

	} else {
		// other importer
		ui.CBImporterReader->setEnabled(true);
		ui.PBEdit->setEnabled(true);
		ui.GBParametricWrite->setEnabled(true);

		if ((ext==".BMP")||(ext==".JPG")||(ext==".PNG"))
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReader2DImages"));
		}
		else if (ext==".WRL")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderVRML"));
		}
		else if (ext==".PLY")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderPLY"));
		}
		else if ((ext==".OBJ")||(ext==".SMF"))
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderSMFOBJ"));
		}
		else if (ext==".TXT")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderCustomASCII"));
		}
		else if (ext==".OFF")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderOFF"));
		}
		else if (ext==".PNT")
		{
			on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderCustom3DScan"));
		}

		ui.CBImporterReader->setSelection(m_Object->getImporterReader()->getInstanceClass()->getClassName());

	};
}



// ---------- slots ----------

void PCCImporterEditorWidget::on_LEFileName_textEdited(QString s){
	m_Object->setInputFileName(qPrintable(s));
}

void PCCImporterEditorWidget::on_LEFileName_editingFinished(){
	switchImporterToFileExtension(strUCase(extractFileExt(m_Object->getInputFileName())));
}

void PCCImporterEditorWidget::on_PBLoadFile_clicked(){
	std::string dir = removeFilenameFromPath(qPrintable(ui.LEFileName->text()));
	std::string s = FileDialogs::getOpenFileName(this, "Choose a file", "All Files (*.*)", NULL, dir, FileDialogs::IMPORT_DIR);
	if (s != ""){ 
		//set filename
		ui.LEFileName->setText(s.c_str());
		m_Object->setInputFileName(s);
		if( !checkAndSetupMSColorDepthBMP( s ) )
			switchImporterToFileExtension(strUCase(extractFileExt(s)));
	}
}


void PCCImporterEditorWidget::on_CBImporterReader_currentIndexChanged(QString v){
	if (m_Object==NULL) return;
	std::string  s = v.toLatin1().constData();
	const MetaClass *newImporterMetaClass = MetaClass::getClassByName(s);
	x4Assert (newImporterMetaClass != NULL );

	m_Object->setImporterReader((PointCloudImporterReader*)(newImporterMetaClass->newInstance()));
}


void PCCImporterEditorWidget::on_PBEdit_clicked(){
	ClassEditor * ce = (ClassEditor*)m_Object->getImporterReader()->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
	ObjectViewsTable::bindView( m_Object->getImporterReader(), ce );
   ObjectBrowserWidget *browser = new ObjectBrowserWidget(m_Object->getImporterReader(), ce, this, Qt::Window);
	QWidget * w = ce->makeEditor( browser->getEditorContainer(), m_Object->getImporterReader() );
   browser->setupEditor(w);
   browser->show();
   
//	w->setWindowFlags( Qt::Window );
//	w->setWindowTitle( objectName.c_str() );
//	w->move( QCursor::pos() );
//	w->setAttribute(Qt::WA_DeleteOnClose);
//	w->show();

}


void PCCImporterEditorWidget::on_LETarget_textEdited(QString s){
	m_Object->setTargetName(qPrintable(s));
}


void  PCCImporterEditorWidget::on_GBParametricWrite_toggled(bool state) {
	m_Object->setParamWriteByCoords(state);
}


void PCCImporterEditorWidget::on_LEParamCoordsChannel_textEdited(QString text) {
	m_Object->setParamCoordsChannel(qPrintable(text));
}


void PCCImporterEditorWidget::on_SBParamScaleX_valueChanged(double value) {
	m_Object->setPCoordsScaleX(value);
}


void PCCImporterEditorWidget::on_SBParamScaleY_valueChanged(double value) {
	m_Object->setPCoordsScaleY(value);
}


void PCCImporterEditorWidget::on_SBParamOffsetX_valueChanged(double value) {
	m_Object->setPCoordsOffsetX(value);
}


void PCCImporterEditorWidget::on_SBParamOffsetY_valueChanged(double value) {
	m_Object->setPCoordsOffsetY(value);
}


#include "PCCImporterEditorWidget_moc.h"

bool PCCImporterEditorWidget::checkAndSetupMSColorDepthBMP(std::string s ) 
{
	std::string filename = X4::removePathFromFilename( s );

	// match "color-camX-fXXX" or "depth-camX-fXXX" ?

	if( filename.length() != 19 )
		return false;		

	std::string prefix = filename.substr(0, 9);
	if( prefix != "color-cam" && prefix != "depth-cam" )
		return false;

	std::string midPart = filename.substr(10,2);
	if( midPart != "-f" )
		return false;

	on_CBImporterReader_currentIndexChanged(QString("PointCloudImporterReaderMSDepthBMP"));
	ui.CBImporterReader->setSelection(m_Object->getImporterReader()->getInstanceClass()->getClassName());
	PointCloudImporterReaderMSDepthBMP * msImporter = dynamic_cast<PointCloudImporterReaderMSDepthBMP*>(m_Object->getImporterReader());
	
	std::string path = X4::removeFilenameFromPath( s );
	path = path.substr(0,path.size()-6);

	QDir dir( path.c_str() );
	QStringList filters;
	filters << "*.txt";
	QStringList list = dir.entryList( filters );
	for( int i=0;i<list.size();i++ )
	{
		QString filename = list[i];
		if( filename.contains( "calib") )
		{
			std::string settingsFilename = path + "/" + filename.toLatin1().constData();
			msImporter->setup( settingsFilename );
			break;
		}
	}

	return true;
}
