//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCCParametricResampling.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
#include "Scene.h"
#include "LinearAlgebra.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "PointCloudTools.h"
#include "SGLegacyPCSceneAdaptor.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* PCCParametricResampling::PCCParametricResamplingMetaClass = NULL;

const MetaClass* PCCParametricResampling::getInstanceClass() const {
   return getClass();
}

Object* PCCParametricResampling::newInstance() {
   return new PCCParametricResampling;
}

const MetaClass* PCCParametricResampling::getClass() {
   if (PCCParametricResamplingMetaClass==NULL) {
      PCCParametricResamplingMetaClass = new MetaClass("PCCParametricResampling", "no description.",
                                  newInstance, sizeof(PCCParametricResampling));
   }
   return PCCParametricResamplingMetaClass;
}

void PCCParametricResampling::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   PCCParametricResamplingMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(PCCParametricResampling)
   ADD_CARD32_PROP(xSamplingRate ,0)
   ADD_CARD32_PROP(ySamplingRate ,0)
}

void PCCParametricResampling::shutdown() {
   delete PCCParametricResamplingMetaClass;
}


// ----

PCCParametricResampling::PCCParametricResampling() {
   xSamplingRate = 2;
   ySamplingRate = 2;
}

void PCCParametricResampling::execute(Scene *scene) {
   SGLegacyPCSceneAdaptor sceneLA(scene);
   std::vector<card32> selection = sceneLA.getSelectedPCs();

   if (selection.size() == 0) {
      //Application->MessageBox("Nothing selected.", "Cannot Execute Command...", MB_OK | MB_ICONERROR);
		 QMessageBox::information( NULL, "Nothing selected" , "Cannot Execute Command..." );
      return;
   }
   const size_t n = selection.size();

   for (card32 sel=0; sel<n; sel++) {
      PointCloud *pc = sceneLA.getPointCloud(selection[sel]);
      PointCloud *tmp= createResampled( pc,xSamplingRate,ySamplingRate );
      sceneLA.replacePointCloud(selection[sel],tmp);
   };

}
