//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PCCPointGenerator.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "BoundingBox.h"
#include "BoundingBox.inline.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra.inline.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "NumericalClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "StringClassProperty.h"
#include "PointSet.h"
#include "UnstructuredInCorePointCloud.h"
#include "DebugOutput.h"
#include "VertexDescriptor.h"
#include "Random.h"
#include "PolarCoordinates.h"
#include "PointCloudIterators.h"
#include "BooleanClassProperty.h"
#include "CommandHistory.h"
#include "ClassMethods.h"
#include "SeparatorClassProperty.h"
#include "VertexArray.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

static std::string describeType(card32 q) {
	if ( q == GENERATE_BOX ) {
		return "BOX";
	} else if ( q == GENERATE_SPHERE ) {
		return "SPHERE";
	} else if ( q == GENERATE_PLANE ) {
		return "PLANE";
	} else if ( q == GENERATE_DOWNWARD_SPIRAL ) {
		return "DOWNWARD_SPIRAL";
	} else if ( q == GENERATE_SINUS ) {
		return "GENERATE_SINUS";
	} else if ( q == GENERATE_PARABEL ) {
		return "PARABEL";
	} else if ( q == GENERATE_CYLINDER ) {
		return "CYLINDER";
	} else if ( q == GENERATE_MERLONS ) {
		return "MERLONS";
	} else {
		return "UNDEFINED";
	}
}

IMPLEMENT_X4_CLASS( PCCPointGenerator, 1 )
{
	BEGIN_CLASS_INIT( PCCPointGenerator )
	ADD_CARD32_PROP( numPoints,0 )
	ADD_CARD32_PROP_UPDATE_DESCR( type, 0, NULL, &describeType, 8, NULL )
	ADD_STRING_PROP( pcName, 0 );

	ADD_SEPARATOR("mesh parameters");
	ADD_CARD32_PROP(m_Mesh_NumSegments,1);
	ADD_VECTOR3F_PROP(m_Mesh_Box,1);

	ADD_NOARGS_METHOD(createCylinderMesh);

}

/** 
 * Create new object, init variables
 */
PCCPointGenerator::PCCPointGenerator()
{
   type = GENERATE_SPHERE;
   numPoints = 2000;
   pcName = "pc1";

   // mesh stuff
   m_Mesh_NumSegments = 20;
   m_Mesh_Box = makeVector3f(1,1,1);
}

/**
 * Run command
 */
void PCCPointGenerator::execute( Scene *scene )
{
   CmdObjGeneratePC* newCmd = new CmdObjGeneratePC();
   newCmd->setup( type, numPoints, pcName );
   newCmd->doAction( scene );
   delete newCmd;
}


void X4::PCCPointGenerator::createCylinderMesh()
{
	mpcard numSegments = m_Mesh_NumSegments;

	UnstructuredInCoreTriangleMesh * mesh = new UnstructuredInCoreTriangleMesh;
	VertexDescriptor * vd = createDefaultVertexDescriptor(true,true,true,true);
	VertexDescriptor vdTri;
	vdTri.pushAttrib(VAD("index",3,VAD::DATA_FORMAT_INT32));
	mesh->clearAndSetup(vd,numSegments*2+numSegments*2+2,&vdTri,numSegments*4);
	delete vd;

	VertexArray va_points(mesh);
	VertexArray va_tri(mesh->getTriangles());

	// cap center points
	mpcard cap_top_center = mesh->getNumPoints()-1;
	mpcard cap_bottom_center = mesh->getNumPoints()-2;
	va_points.setPosition3f(cap_bottom_center,makeVector3f(0,0,-1));
	va_points.setNormal3f(cap_bottom_center,makeVector3f(0,0,-1));
	va_points.setPosition3f(cap_top_center,makeVector3f(0,0,1));
	va_points.setNormal3f(cap_top_center,makeVector3f(0,0,1));
	mpcard cap_top_start = numSegments*2;
	mpcard cap_bottom_start = numSegments*3;

	for( mpcard i=0;i<numSegments;i++ )
	{
		float angle = (float)i / (float)numSegments*2.0f*M_PI;
		float sin_angle = sin(angle);
		float cos_angle = cos(angle);

		// side quads
		va_points.setPosition3f(i*2+0,makeVector3f(sin_angle,cos_angle,-1));
		va_points.setNormal3f(i*2+0,makeVector3f(sin_angle,cos_angle,0));
		va_points.setPosition3f(i*2+1,makeVector3f(sin_angle,cos_angle,1));
		va_points.setNormal3f(i*2+1,makeVector3f(sin_angle,cos_angle,0));

		int32 next_i = (i+1)%numSegments;
		va_tri.setIndex3i(i*2+0,makeVector3i(i*2+1,i*2+0,next_i*2+1));
		va_tri.setIndex3i(i*2+1,makeVector3i(i*2+0,next_i*2+0,next_i*2+1));

		// caps
		va_points.setPosition3f(cap_bottom_start+i,makeVector3f(sin_angle,cos_angle,-1));
		va_points.setNormal3f(cap_bottom_start+i,makeVector3f(0,0,-1));
		va_points.setPosition3f(cap_top_start+i,makeVector3f(sin_angle,cos_angle,1));
		va_points.setNormal3f(cap_top_start+i,makeVector3f(0,0,1));

		va_tri.setIndex3i(2*numSegments+i,makeVector3i(cap_bottom_start+next_i,cap_bottom_start+i,cap_bottom_center));
		va_tri.setIndex3i(3*numSegments+i,makeVector3i(cap_top_start+i,cap_top_start+next_i,cap_top_center));
	}

	for( mpcard i=0;i<va_points.getNumElements();i++ )
	{
		va_points.setPosition3f(i, va_points.getPosition3f(i).componentProduct(m_Mesh_Box));
	}
	mesh->setMaterialIndex(3);

	addPointCloud(getScene(),mesh,"cylinder");
}