//---------------------------------------------------------------------------
#ifndef PCCPointNoiseH
#define PCCPointNoiseH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
#include "PointCloud.h"
#include "Scene.h"
#include "NormalDistribution.h"
//---------------------------------------------------------------------------

namespace X4 {

/// Add noise to an arbitrary attribute
class X3EDITOR_API PCCPointNoise : public PCCommand 
{
	X4_CLASS( PCCPointNoise )

	private:
		/// Attribute to be modified
		std::string attributeID;
		/* Modification coordinate
		* 0 = 1st coordinate
		* 1 = 2nd coordinate
		* 2 = 3rd coordinate
		* 3 = 4th coordinate
		* 4 = all coordinates
		*/
		card32 modificationCoordinate;
		/* Modification type
		* 0 = noise (param0 = strength, param1 = -)
		* 1 = sine x-direction (param0 = frequency, param1 = amplitude, parameterized surface only (xz, height=y))
		* 2 = sine z-direction (param0 = frequency, param1 = amplitude, parameterized surface only (xz, height=y))
		* 3 = sine spherical (param0 = frequency, param1 = amplitude, parameterized surface only (xz, height=y))
		*/
		card32 modificationType;
		float normalVariance;			/// First parameter (see modification type)
		float scale;					/// Second parameter (see modification type)
		bool modifyAllFrames;
		float32 addOutliersRatio;		/// Percentage of outliers

		void modifyPointCloud( PointCloud * pc );
		void addOutliers();

	public:
		PCCPointNoise();
		virtual void execute(Scene *scene);
};


}

#endif
