//---------------------------------------------------------------------------
#ifndef PCCResamplerH
#define PCCResamplerH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
#include "PointCloud.h"
#include "Scene.h"
//---------------------------------------------------------------------------

namespace X4 {

/// Downsampling of point clouds
class X3EDITOR_API PCCResampler : public PCCommand 
{
	X4_CLASS( PCCResampler )
	
	private:
		std::string source;			// Name of source node; DEPRECATED
		card32 numPoints;		// Number of points to end up with; DEPRECATED
		bool animated;			// Flag for animated data; DEPRECATED
		bool selection;			// ?? DEPRECATED
		bool old;			    // DEPRECATED
		float32 spacing;		// Min spacing between points after resampling

		//	PointSet* downSample( PointSet* ps, const card32 numPoints ); // DEPRECATED

		void resampleOCC( PointCloud * pc, SceneObjectIterator * iter );
		void gatherPointCloud( PointCloud * pc, SceneObjectIterator * iter, UnstructuredInCorePointCloud * retPC );
		void selectDeletedPoints(UnstructuredInCorePointCloud * pc);
		void writeModifications(PointCloud * pc, SceneObjectIterator * iter, UnstructuredInCorePointCloud * mpc, card32 & currentPointsIndex );

	public:
		PCCResampler();
		~PCCResampler();
		void execute( Scene *scene );

      static void resamplePC(PointCloud* pc, float spacing, bool changeFlagAttOnly = false);
		
		// function used for resampling animations
		// no progress bar and less checks than resamplePC()
		// well suited for calls with openmp (JK)
		static void resamplePC_ar(PointCloud* pc, float spacing);
      
      float32 getSpacing() const {return spacing;}
      void setSpacing(float32 v) {spacing=v;}

};


}

#endif
