#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCSwapChannels.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCCSwapChannels, 1 )
{
	BEGIN_CLASS_INIT( PCCSwapChannels );
	ADD_STRING_PROP(channel1, 1)
	ADD_STRING_PROP(channel2, 1)
}


// ----


PCCSwapChannels::PCCSwapChannels() {
	channel1 = "position";
	channel2 = "pos2";
}

PCCSwapChannels::~PCCSwapChannels() {
}

void PCCSwapChannels::assign(const Object* obj, X4CopyContext *context) {
	PCCommand::assign(obj,context);
}

void PCCSwapChannels::execute(Scene *scene) {
   std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
   for (unsigned m=0; m<selection.size(); m++) {
      PointCloud *pc = getPointCloud( scene, selection[m] );
		if (pc == NULL) continue;
		if (!pc->providesAttribute(channel1) || !pc->providesAttribute(channel2)) {
			warning("PCCSwapChannels::execute() - one of the selected point clouds did not provide the specified channels.");
			continue;
		}
		VAD vad1 = pc->getDescr()->getAttribute(channel1);
		VAD vad2 = pc->getDescr()->getAttribute(channel2);
		if (vad1.dataFormat != vad2.dataFormat || vad1.numComponents != vad2.numComponents) {
			warning("PCCSwapChannels::execute() - in one of the selected point clouds the specified channel hat a different format.");
			continue;
		}
		AAT aat1 = pc->getAAT(channel1);
		AAT aat2 = pc->getAAT(channel2);
		
		card32 size = vad1.getSize();
		x4Assert(size == vad2.getSize());

		SceneObjectIterator *it = pc->createIterator(SOIT::CAP_BASIC_PC || SOIT::CAP_MODIFYABLE_PC);
		BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(it);
		ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(it);
		
		card8 *vbuffer = new card8[pc->getDescr()->getSize()];
		card8 *buffer1 = new card8[size];
		card8 *buffer2 = new card8[size];
		
		while (!bIt->atEnd()) {
			bIt->getVertex((char*)vbuffer);
			for (card32 i=0; i<size; i++) {
				buffer1[i] = (vbuffer+aat1.getOffset())[i];
				buffer2[i] = (vbuffer+aat2.getOffset())[i];
			}
			for (card32 i=0; i<size; i++) {
				(vbuffer+aat1.getOffset())[i] = buffer2[i];
				(vbuffer+aat2.getOffset())[i] = buffer1[i];
			}
			modIt->setVertex((char*)vbuffer);
			bIt->next();
		}
		
		delete[] vbuffer;
		delete[] buffer1;
		delete[] buffer2;
		
		delete it;
	}		
}
