//---------------------------------------------------------------------------
#ifndef PCCTriangleMeshSamplerH
#define PCCTriangleMeshSamplerH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------

namespace X4 {

class SceneObject;
class UnstructuredInCoreTriangleMesh;
class UnstructuredInCorePointCloud;


/// this method implements the core functionality
/// source must be a triangle mesh, target must be a (dynamic) point cloud
/// expNumPoints is the approximate number of points to be generated (this is the expected value)
X3EDITOR_API void sampleMeshRandomly(Scene * scene, std::string destID, SceneObject *sourceMesh, SceneObject *destPointCloud, card64 numPoints,
                                bool createTangentDiscs, float32 tangentDiscOversampling, bool surfaceNormalSmoothing);


class X3EDITOR_API PCCTriangleMeshSampler : public PCCommand {
 private:
   static MetaClass *PCCTriangleMeshSamplerMetaClass;
   static Object* newInstance();

   std::string sourceNode;
   std::string destNode;
   card64 expNumPoints;
   bool createTangentDiscs;
   float32 tangentDiscOversampling;
   float32 sampleSpacing;
	bool surfaceNormalSmoothing;
	int copyDataFromVertex;

 public:
   PCCTriangleMeshSampler();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   void sampleMeshPoission();
   virtual void execute(Scene *scene);

   /// sample mesh with a specified sample distance (first: per triangle oversampling with jittered grid, second: bilateral poisson resampling)
   /// all attributes are either interpolated or copied from one of the vertices
   static UnstructuredInCorePointCloud * sampleMeshPoisson(UnstructuredInCoreTriangleMesh * mesh, float sampleSpacing, int copyDataFromVertex = -1);
};

}

#endif
 