//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PCICameraController.h"
//---------------------------------------------------------------------------
#include "CameraController.h"
#include "ExaminerCameraController.h"
#include "FreeCameraController.h"
#include "WalkthroughCameraController.h"
#include "CameraPathRecorder.h"
#include "ObjectClassProperty.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "SceneCommandObjects.h"
#include "CommandHistory.h"
#include "PointCloudTools.h"
#include "FileDialogs.h"
#include "ClassMethods.h"
#include "BinaryObjectStreams.h"
#include "InterfaceGlobals.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


                 
MetaClass* PCICameraController::PCICameraControllerMetaClass = NULL;

const MetaClass* PCICameraController::getInstanceClass() const {
   return getClass();
}

Object* PCICameraController::newInstance() {
   return new PCICameraController;
}

const MetaClass* PCICameraController::getClass() {
   if (PCICameraControllerMetaClass==NULL) {
      PCICameraControllerMetaClass = new MetaClass("PCICameraController", "no description.",
                                  newInstance, sizeof(PCICameraController));
   }
   return PCICameraControllerMetaClass;
}

void PCICameraController::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   PCICameraControllerMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT(PCICameraController)
	ADD_OBJECT_PROP_METHOD( currentController,0, &getCamCtrlObj, &setCamCtrlObj, CameraController::getClass(), true )
	ADD_NOARGS_METHOD(loadCameraController);
	ADD_NOARGS_METHOD(saveCameraController);
	ADD_NOARGS_METHOD(setStdExaminer)
	ADD_NOARGS_METHOD(setStdFreeCamera)
	ADD_NOARGS_METHOD(setStdWalkthrough)
	ADD_NOARGS_METHOD(setStdPathRecorder)
	ADD_NOARGS_METHOD_DESC(setVolumeCamera,"Setup FreeCamera with lookat at volume center");

	ADD_NOARGS_METHOD(outputCameraParameters);
}

void PCICameraController::shutdown() {
   delete PCICameraControllerMetaClass;
}


// ----

PCICameraController::PCICameraController() {
   //currentController = new FreeCameraController();
   currentController = new ExaminerCameraController();
   currentController->mouseDown(0, 0, MouseButtons(), Modifiers());
   currentController->keyDown(GeneralKey(Qt::Key_PageDown, Modifiers()));
   currentController->keyDown(GeneralKey(Qt::Key_PageUp, Modifiers()));
   cameraSet = false;
    panning = false;
    lastX = lastY = 0.f;
}

CameraController *PCICameraController::getCamCtrl() {
   return currentController;
}

Object * PCICameraController::getCamCtrlObj() const
{
	return currentController;
}


void PCICameraController::setCamCtrl(CameraController *ctrl) {
   // if (currentController != ctrl) delete currentController;
   //  debugOutput << "setCam" ;
   //      debugOutput.cr();
   currentController = ctrl;
   if (scene != NULL) {
      currentController->bindCamera(&(scene->getRootState()->staticState->camera));
      currentController->mouseDown(0, 0, MouseButtons(), Modifiers());
      currentController->keyDown(GeneralKey(VK_NEXT, Modifiers()));
      currentController->keyDown(GeneralKey(VK_PRIOR, Modifiers()));
   }
   update();
}

void PCICameraController::setCamCtrlObj(Object *ctrl) 
{
	setCamCtrl( static_cast<CameraController*>(ctrl) );
}

void PCICameraController::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {
	if( buttonsState.getMiddle() || (buttonsState.getRight() && modifiersState.getCtrl()))
	{
		/*Vector3f point;
		std::string pcName;
		PointCloud * pc;
		if( getIntersectionFromCursor(x,y, point, pcName,&pc ) )
		{
			Camera * cam = currentController->getCamera();
			currentController->setup(cam->getPosition(), transformVector3f(pc->getTransformation(),point), cam->getUp());
		}*/
		Vector3f point;
		if( guessClickPoint(x,y,5,point) )
		{
			Camera * cam = currentController->getCamera();
			currentController->setup(cam->getPosition(), point, cam->getUp());
		}
    } else if (buttonsState.getRight()) {
        panning = true;
        lastX = x;
        lastY = y;
    }
    else
		currentController->mouseDown(x, y, buttonsState, modifiersState);
   update();
}

void PCICameraController::mouseMoved(int32 x, int32 y) {
    if (panning) {
        Camera* camera = currentController->getCamera();
        const Vector3f cup = normalize( camera->getUp() );
        const Vector3f dir = normalize( camera->getLookAt() - camera->getPosition() );
        const Vector3f lef = cup.crossProduct(dir);
        Matrix3f t3;
        t3[0] = lef; t3[1] = cup; t3[2] = dir;
        const float panSpeed = 1e-2f;
        Vector3f delta = makeVector3f((x - lastX) * panSpeed, (y - lastY) * panSpeed, 0);
        delta = transformVector3f(expand3To4(invertFrame(t3)), delta);
        currentController->setup( camera->getPosition() + delta, camera->getLookAt() + delta, camera->getUp() );  
        lastX = x;
        lastY = y;
    }
    currentController->mouseMoved(x, y);
    update();
}

void PCICameraController::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {
    panning = false;
    currentController->mouseUp(x, y, buttonsState, modifiersState);
    update();
}

void PCICameraController::areaResize(card32 width, card32 height) {
   currentController->areaResize(width, height);
   update();
}


void PCICameraController::keyDown(GeneralKey key) {
	/// Auto align
	if ( key.getKey() == keyF5 ){
		if ( currentController != NULL ){
			 BoundingBox3f bb = getScenePCBBoxVisibleOnly( scene );
			 // Special case: position equals center
			 Camera* camera = currentController->getCamera();
			 Vector3f dir = normalize( camera->getLookAt() - camera->getPosition() );
			 Vector3f newPos = bb.getCenter() - dir * 2.0f*bb.getMaxSideLength();
			 currentController->setup( newPos, bb.getCenter(), camera->getUp() );  
//			 scene->getRootState()->staticState->viewFrustum.setZFarClip( max(10.0 * bb.getMaxSideLength(),
//				 scene->getRootState()->staticState->viewFrustum.getZFarClip()) );
			 scene->getRootState()->staticState->viewFrustum.setZFarClip( 10.0f * bb.getMaxSideLength() );

			 // mb - avoid total loss of zbuffer precision
//			 scene->getRootState()->staticState->viewFrustum.setZNearClip( min(0.02 * bb.getMaxSideLength(), 
//			   scene->getRootState()->staticState->viewFrustum.getZNearClip()) );
			 scene->getRootState()->staticState->viewFrustum.setZNearClip( 0.001f * scene->getRootState()->staticState->viewFrustum.getZFarClip());
		}
	}
	currentController->keyDown(key);
	update();
}

void PCICameraController::keyUp(GeneralKey key) {
   currentController->keyUp(key);
   update();
}


void PCICameraController::connectToSceneImpl(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget, QWidget* toolWidget) {
   cam.assign( &scene->getRootState()->staticState->camera );
   currentController->bindCamera(&cam);
   // currentController->mouseKeyPressed(0, 0, MouseButtons(), Modifiers());
   update();
}

void PCICameraController::activateTool() {
   this->cam = scene->getRootState()->staticState->camera;
   this->vf = scene->getRootState()->staticState->viewFrustum;
   currentController->bindCamera(&cam);
   update();
}

void PCICameraController::update() {
   if (!cameraSet || cam != lastCam
       || vf.getZNearClip() != lastVf.getZNearClip() || vf.getZFarClip() != lastVf.getZFarClip()
       || vf.getVerticalFieldOfView() != lastVf.getVerticalFieldOfView()) {
      cameraSet = true;
      lastCam = cam;
      lastVf = vf;
      CmdObjChangeView *newCmd = new CmdObjChangeView();
      newCmd->setup(vf, cam);
      scene->getCommandHistory()->executeCommand(scene, newCmd, true);
      sceneChanged(PCI_CHANGE_VIEW | PCI_NEW_COMMAND);
   }
}

void PCICameraController::loadCameraController() 
{
	std::string filename = FileDialogs::getOpenFileName(
	   InterfaceGlobals::getMainWindow(), "Load camera controller...", 
      "X4 CameraController Objects (*.camCtrl.x4obj);;X4 Objects (*.x4obj);;All Files (*.*)"
   );
   if (filename != "" && FileExists(filename.c_str())) {
	   BinaryInputObjectStream in(filename.c_str());
      Persistent *instance = NULL;
      in.readObject(instance);
      CameraController *c = dynamic_cast<CameraController*>(instance);
      if (c != NULL) {
         delete currentController;
         currentController = c;
         currentController->bindCamera(&(getScene()->getRootState()->staticState->camera));
      } else {
         delete c;
         throw X4Exception("PCICameraController::loadCameraController() - object is not a camera controller.");
      }
	  update();
   }   
}

void PCICameraController::saveCameraController() 
{
	std::string filename = FileDialogs::getSaveFileName(
	   InterfaceGlobals::getMainWindow(), "Save camera controller...", 
      "X4 CameraController Objects (*.camCtrl.x4obj);;X4 Objects (*.x4obj);;All Files (*.*)"
   );

   if (filename != "") {
	   BinaryOutputObjectStream out(filename.c_str());
      out.writeObject(currentController);
   }   
}

PCICameraController::~PCICameraController() {
   delete currentController;
}

void PCICameraController::setStdExaminer() 
{
   delete currentController;
   currentController = new ExaminerCameraController();
   currentController->bindCamera(&(getScene()->getRootState()->staticState->camera));
   redrawScene();
}

void PCICameraController::setStdFreeCamera() 
{
   delete currentController;
   currentController = new FreeCameraController();
   currentController->bindCamera(&(getScene()->getRootState()->staticState->camera));
   redrawScene();
}

void PCICameraController::setStdWalkthrough() 
{
   delete currentController;
   currentController = new WalkthroughCameraController();
   currentController->bindCamera(&(getScene()->getRootState()->staticState->camera));
   redrawScene();
}

void PCICameraController::setStdPathRecorder() 
{
   delete currentController;
   currentController = new CameraPathRecorder();
   currentController->bindCamera(&(getScene()->getRootState()->staticState->camera));
   redrawScene();
}

void PCICameraController::setVolumeCamera() 
{
	delete currentController;
	currentController = new FreeCameraController();
	getScene()->getRootState()->staticState->camera.setLookAt( makeVector3f(0.5,0.5,0.5));
	getScene()->getRootState()->staticState->camera.setPosition( makeVector3f(0.5,0.5,-1.5));
	getScene()->getRootState()->staticState->camera.setUp( makeVector3f(0,1,0));
	currentController->bindCamera(&(getScene()->getRootState()->staticState->camera));
	redrawScene();
}

void X4::PCICameraController::mouseWheelRotated( int32 rotatedDelta, Modifiers modifiersState )
{
	currentController->mouseWheelRotated( rotatedDelta, modifiersState);
	update();
}

void X4::PCICameraController::outputCameraParameters()
{
	message("camera begin\n");
	debugOutput << "campos " << cam.getPosition()[0] << " " << cam.getPosition()[1] << " " << cam.getPosition()[2] << "\n";
	debugOutput << "camup " << cam.getUp()[0] << " " << cam.getUp()[1] << " " << cam.getUp()[2] << "\n";
	debugOutput << "camlookat " << cam.getLookAt()[0] << " " << cam.getLookAt()[1] << " " << cam.getLookAt()[2] << "\n";
	debugOutput << "camfov " << vf.getVerticalFieldOfView() << "\n";
	debugOutput << "camznear " << vf.getZNearClip() << "\n";
	debugOutput << "camzfar " << vf.getZFarClip() << "\n";
	debugOutput << "camaspect " << vf.getXToYAspectRatio() << "\n";
	debugOutput << "camera end\n\n";
}
