//---------------------------------------------------------------------------
#ifndef PCICameraControllerH
#define PCICameraControllerH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "Camera.h"
#include "ViewFrustum.h"
//---------------------------------------------------------------------------

namespace X4 {

class CameraController;


class X3EDITOR_API PCICameraController : public PCInteractionTool {
 private:
   static MetaClass *PCICameraControllerMetaClass;
   static Object* newInstance();

   CameraController *currentController;
   Camera cam, lastCam;
   ViewFrustum vf, lastVf;
   bool cameraSet;
   bool panning;
   float lastX, lastY;

 protected:
   void update();
   void loadCameraController();
   void saveCameraController();
   void setStdExaminer();
   void setStdFreeCamera();
   void setStdWalkthrough();
   void setStdPathRecorder();
   void setVolumeCamera();
   void outputCameraParameters();

 public:
   PCICameraController();

	Object * getCamCtrlObj() const;
	CameraController *getCamCtrl();
   void setCamCtrl(CameraController *ctrl);
   void setCamCtrlObj(Object *ctrl);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);
   virtual void mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void connectToSceneImpl(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget, QWidget* toolWidget);
   virtual void activateTool();

   CameraController *getCurrentController() {return currentController;}
   inline Camera& getCam(){ return cam; }

   ~PCICameraController();
};

}


#endif
