#ifndef PCICameraManagerH
#define PCICameraManagerH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "Camera.h"
#include "ViewFrustum.h"
//---------------------------------------------------------------------------

namespace X4
{

class SGCameraNode;
class CameraController;
class PCICameraManagerEditorWidget;

/**
	works similar to PCICameraController, but without owning a controller.
	user-input is transfered to a CameraController located in scene graph.
*/
class X3EDITOR_API PCICameraManager : public PCInteractionTool
{
	X4_CLASS( PCICameraManager )

private:

	SGCameraNode * getCameraNode( std::string nodename );
	Camera * getCurrentCamera();
	CameraController * getCurrentCameraController();
	ViewFrustum * getCurrentViewFrustum();
	void update();

	std::string	m_SelectedCamera;	// path in scenegraph
	Camera	m_MainCamera;		// holds camera from scenegraphstate ( saved when activateTool() is called )
	ViewFrustum m_MainFrustum;	// holds frustum from scenegraphstate ( saved when activateTool() is called )

	friend PCICameraManagerEditorWidget;

public:
	PCICameraManager(void);
	~PCICameraManager(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

	virtual void activateTool();
	virtual void deactivateTool();

	void setSelectedCamera( std::string selCam ) {m_SelectedCamera = selCam;}
	std::string getSelectedCamera() {return m_SelectedCamera;}

	void setTime(int time);

	void snapCamera();
};

}

#endif