//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCIMappingTool.h"
//---------------------------------------------------------------------------
#include "ObjectClassProperty.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "DebugOutput.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "NumericalClassProperty.h"
#include "PointSet.h"
#include "SceneCommandObjects.h"
#include "CommandHistory.h"
#include "IteratorAdaptors.h"
#include "GeometricRange.h"
#include "GeometricRangeIteratorAdaptor.h"
#include "SelectionIteratorAdaptor.h"
#include "ParametricInCorePointCloud.h"
#include "CmdObjMappingOnClick.h"
#include "CmdObjSpecialDraw.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "SceneGraphNode.h"
#include "PointCloudTools.h"
#include "LinearAlgebra.inline.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* PCIMappingTool::PCIMappingToolMetaClass = NULL;

const MetaClass* PCIMappingTool::getInstanceClass() const {
   return getClass();
}

Object* PCIMappingTool::newInstance() {
   return new PCIMappingTool;
}

const MetaClass* PCIMappingTool::getClass() {
   if (PCIMappingToolMetaClass == NULL) {
      PCIMappingToolMetaClass = new MetaClass("PCIMappingTool", "no description.",
                                       newInstance, sizeof(PCIMappingTool));
   }
   return PCIMappingToolMetaClass;
}

void PCIMappingTool::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION);
   PCIMappingToolMetaClass->copyParentProperties();

   //PCIMappingToolMetaClass->getGUIDescriptor()->addClassEditor(PCIMappingToolEditor::getClass(), true);
}

void PCIMappingTool::shutdown() {
   delete PCIMappingToolMetaClass;
}


// ----

PCIMappingTool::PCIMappingTool(){
   leftMBdown=false;
   mode=0;
   col=false;
   pos=false;
   pnorm=false;
   load=false;
   soft=true;
   drag=false;

   colTex = new PCTexture();

   scale=0.1f;
   size=50;
   raise=false;
   name="Bitmap";

   localValid=false;

};


void PCIMappingTool::setDispMode(bool m){
   raise=m;
}


void PCIMappingTool::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {
   forget(buttonsState);
   forget(modifiersState);
   leftMBdown=true;

   card32 i=0;
   SGLegacyPCSceneAdaptor sceneLA(scene);
   for (i=0; i<sceneLA.getNumPointClouds(); i++)
   {
      std::string pcName = sceneLA.getNameWithPath( i ); 
      checkAttribute( scene, pcName, "color", 3, VAD::DATA_FORMAT_FLOAT32 );
   };

   paintTex(x,y);
}

void PCIMappingTool::mouseMoved(int32 x, int32 y) {

   mouseMoveX=x;
   mouseMoveY=y;

   if (leftMBdown && drag){
      if (pos){
         paintTexDrag(x,y);}
      else
         paintTex(x,y);
   };
}

void PCIMappingTool::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {
   leftMBdown=false;
   forget(x+y);
   forget(buttonsState);
   forget(modifiersState);
}

void PCIMappingTool::areaResize(card32 width, card32 height) {
   w= width;
   h= height;
   WidthToHeight = ((float)height)/((float)width);
}


void PCIMappingTool::keyDown(GeneralKey key) {
   forget(key);
}

void PCIMappingTool::keyUp(GeneralKey key) {
   forget(key);
}


void PCIMappingTool::loadTexture(std::string filename){
   CmdObjLoadMap *newCmd = new CmdObjLoadMap();
   newCmd->setup(filename);
   scene->getCommandHistory()->executeCommand(scene, newCmd, false);
}

void PCIMappingTool::viewTex3D(){
   if (!(name=="Bitmap")){
     SGLegacyPCSceneAdaptor sceneLA(scene);
	  SceneGraphNode* node=NULL;
	  for (card32 i=0; i<sceneLA.getNumPointClouds(); i++) {
		  if ((sceneLA.getNode(i)->getName() == name)) {
			  node=sceneLA.getNode(i);
		  }
	  }
	 if (node) node->setVisible(!node->visible());
   }

}


QImage* PCIMappingTool::getBMP(){
   if (!(name=="Bitmap")){
     SGLegacyPCSceneAdaptor sceneLA(scene);
     ParametricInCorePointCloud* pc=NULL;

	  for (card32 i=0; i<sceneLA.getNumPointClouds(); i++) {
		  if ((sceneLA.getNode(i)->getName() == name)) {
			  pc=dynamic_cast<ParametricInCorePointCloud*> (sceneLA.getPointCloud(i));
		  }
	  }
     if(pc){
      colTex->setPC(pc);
      return colTex->getBMP();
      }
   }

   return NULL;
}


void PCIMappingTool::setDispScale(float32 s){
   scale=s;
   //colTex->setDisplacementScale(scale); in CmdObj
}

void PCIMappingTool::paintTex(card32 x,card32 y){
   CmdObjMappingOnClick *newCmd = new CmdObjMappingOnClick();
   newCmd->setup(x, y, size, mode, (scene->getRootState()->staticState->camera), (scene->getRootState()->staticState->viewFrustum), w, h, name,soft, col,pos,  pnorm,  raise ,  false, scale);
   scene->getCommandHistory()->executeCommand(scene, newCmd, false);
}

void PCIMappingTool::paintTexDrag(card32 x,card32 y){
   CmdObjMappingOnClick *newCmd = new CmdObjMappingOnClick();
   newCmd->setup(x, y, size, mode, (scene->getRootState()->staticState->camera), (scene->getRootState()->staticState->viewFrustum), w, h, name,soft, col,pos,  pnorm,  raise ,  true, scale);
   scene->getCommandHistory()->executeCommand(scene, newCmd, false);
}

void PCIMappingTool::resetFlagModified(){
   CmdObjSpecialDraw *newCmd = new CmdObjSpecialDraw();
   Vector3f dummy=makeVector3f(0,0,0);
   newCmd->setup(2,dummy, dummy, PF_FLAG_MODIFIED);
   scene->getCommandHistory()->executeCommand(scene, newCmd, false);
}




void PCIMappingTool::drawCircle(float32 x1, float32 y1, float32 r) {
   float32 DEG2RAD = 3.141592654f/180.0f;
   glBegin(GL_LINE_LOOP);
      for (int32 i=0; i < 360; i+=10)
      {
         float32 degInRad = i*DEG2RAD;
         glVertex2f(x1+cos(degInRad)*WidthToHeight*r, y1+sin(degInRad)*r);
      }
   glEnd();
};


void PCIMappingTool::glDrawTool(GLContext *glContext) {
   forget(glContext);


     glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
         glLoadIdentity();
         glMatrixMode(GL_PROJECTION);
         glPushMatrix();
            glLoadIdentity();
            glPushAttrib(GL_ALL_ATTRIB_BITS);
               glDisable(GL_LIGHTING);
               glLineWidth(1.0f);

               float32 posX =   (mouseMoveX-w*0.5f) / (w*0.5f);
               float32 posY = -((mouseMoveY-h*0.5f) / (h*0.5f));


                  float32 radi = size / (h*0.5f);
                  glColor4f(1.0f,0.9f,0.1f,1.0f);
                  drawCircle(posX,posY,radi);


            glPopAttrib();
         glMatrixMode(GL_PROJECTION);
         glPopMatrix();
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();



   if (!localValid) return;

      glPushAttrib(GL_ALL_ATTRIB_BITS);
      glDisable(GL_LIGHTING);
    //  glBlendFunc(GL_DST_COLOR,GL_ZERO);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      glEnable(GL_BLEND);

     // glEnable(GL_DEPTH_TEST);

   /*   glPointSize(8.0f);

      glBegin(GL_POINTS);

         glVertex3fv(mean);
      glEnd();
         */
      // Local Coord System
      Vector3f u,v,n;

      Matrix3f show=eigenVec.transpose();
      u=normalize(show[0]);
      v=normalize(show[1]);
      n=normalize(show[2]);

      u*=sqrt(fabs(eigenVal[0]*1.5f));
      v*=sqrt(fabs(eigenVal[1]*1.5f));

     /*
      Vector3f p; // Point
      Vector3f c; // Erg

      c[0]=(mean-p)* u;
      c[1]=(mean-p)* v;
      c[2]=(mean-p)* n;
      */
      glColor4f(1.f,0.4f,0.4f,0.2f);

      glBegin(GL_QUADS);
                     glTexCoord2f(0,0);
                     glVertex3fv((mean+ v+u).data());
                     glTexCoord2f(0,1);
                     glVertex3fv((mean -v+u).data());
                     glTexCoord2f(1,1);
                     glVertex3fv((mean -v-u).data());
                     glTexCoord2f(1,0);
                     glVertex3fv((mean +v-u).data());
                     glEnd();
      glEnd();
      glPopAttrib();
}


