//---------------------------------------------------------------------------
#ifndef PCIMappingToolH
#define PCIMappingToolH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "ParametricInCorePointCloud.h"
//#include "PointCloudEditTools.h"
#include "GeometricRange.h"
#include "IteratorAdaptors.h"
#include "LinearAlgebra.h"
#include "PCTexture.h"
//---------------------------------------------------------------------------


namespace X4 {


class X3EDITOR_API PCIMappingTool : public PCInteractionTool {
 private:
   static MetaClass *PCIMappingToolMetaClass;
   static Object* newInstance();


   bool leftMBdown;



   card32 w,h;
   Vector3f mean,eigenVal;
   Matrix3f eigenVec;
   bool localValid;
   bool raise;

   PCTexture* colTex;
   PointCloud* tmpC;

   card32 mouseMoveX,mouseMoveY;
   float32 WidthToHeight;

   GeometricRange* getSelectionSphereRangeNotMod(int32 x, int32 y);


 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   PCIMappingTool();
   card32 mode;
   float32 scale;
   float32 size;


   bool col,pos,pnorm,soft,drag;
   std::string fn;
   std::string name;
   bool load;

   void paintTex(card32 x,card32 y);
   void paintTexDrag(card32 x,card32 y);

//   void update();
   void loadTexture(std::string filename);
   void setPC(std::string n){name=n;};
   void viewTex3D();
   void setDispMode(bool);
   bool getDispMode(){return raise;}
   void setDispScale(float32);
   QImage* getBMP(); 
   void resetFlagModified();

   /// Use SVD to find parametric coords
   bool makeLocalCoords(GeometricRange* range, bool notMod ,card32 pcnumber,Vector3f &u,Vector3f &v,Vector3f &w);

   // DEBUG functions -- find maxDist
 //  float32 calcMaxDist(IteratorAdaptor * it, Vector3f u, Vector3f v, AAT POSITION);
   float32 calcMaxVec (IteratorAdaptor * it, Vector3f u, Vector3f v, AAT POSITION);



   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void glDrawTool(GLContext *glContext);
   void drawCircle(float32 x1, float32 y1, float32 r);





};


}

#endif
