//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIPointInfo.h"
//---------------------------------------------------------------------------
//#include "ObjectClassProperty.h"
#include "Scene.h"
#include "VariableArrayClassProperty.h"
#include "StringHelper.h"
#include "SGObjectNode.h"
#include "HierarchicalKNNIterator.h"
#include "SelectionTools.h"
#include "PointCloudTools.h"
#include "ExceptionHandling.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

MetaClass* PCIPointInfo::PCIPointInfoMetaClass = NULL;

const MetaClass* PCIPointInfo::getInstanceClass() const {
	return getClass();
}

Object* PCIPointInfo::newInstance() {
	return new PCIPointInfo;
}

const MetaClass* PCIPointInfo::getClass() {
	if (PCIPointInfoMetaClass == NULL) {
		PCIPointInfoMetaClass = new MetaClass("PCIPointInfo", "no description.",
			newInstance, sizeof(PCIPointInfo));
	}
	return PCIPointInfoMetaClass;
}

void PCIPointInfo::init(const MetaClass *parentClass) {
	((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
	PCIPointInfoMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( PCIPointInfo )
	ADD_STRING_VAR_ARRAY_PROP(PointInfo, 0)
	ADD_STRING_PROP(MouseAndCloudInfo, 0)
}

void PCIPointInfo::shutdown() {
	delete PCIPointInfoMetaClass;
}

// --------------------------------------------------------------------

PCIPointInfo::PCIPointInfo() {
	w = 100;
	h = 100;
	WidthToHeight = 1.0f;
	PointInfo.clear();
	lastPointHadPosition = false;
	lastPointWorldPosition = makeVector3f(0.0f,0.0f,0.0f);
	MouseAndCloudInfo = "";
	previousPoint = false;
}


void PCIPointInfo::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {
	STD_GLOBAL_EXCEPTION_HANDLER_START

	PointInfo.clear();
	//lastPointHadPosition = false;
	//lastPointWorldPosition = makeVector3f(0.0f,0.0f,0.0f);

	// find point which was clicked on
	std::string CloudName = "";
	QMutex m;
	m.lock();
	PointCloudIterator *returnedIt = getClosestPointIteratorFromAllPoints(x, y, w, h, 6, &(scene->getRootState()->staticState->camera), &(scene->getRootState()->staticState->viewFrustum), scene, CloudName);
	m.unlock();
	MouseAndCloudInfo = "mouseXY: " + intToStr(x) + "/" + intToStr(y) + "  cloud: " + CloudName;
	ObjectViewsTable::update(this);
	if (returnedIt==NULL) return;

	PointCloud* PC = getPointCloud(scene, CloudName);
	if (PC==NULL) { delete returnedIt; return; }
	BasicPointCloudIterator *baseIt = dynamic_cast<BasicPointCloudIterator*>(returnedIt);
	IndexedPointCloudIterator *indexIt = dynamic_cast<IndexedPointCloudIterator*>(returnedIt);
	MouseAndCloudInfo += " Point#: " + intToStr(indexIt->getCurrentPointIndex());
	if (baseIt==NULL) { delete returnedIt; return; }
	const VertexDescriptor* VD = PC->getDescr();
	if (VD==NULL) { delete returnedIt; return; }

	for (card32 i=0; i<VD->getNumAttribs(); i++) {
		AAT TMP = VD->getToken(i);
		VAD descr = VD->getAttribute(i);
		std::string tmpString = descr.name + "(" + intToStr(descr.numComponents);

		switch (descr.dataFormat) {

			case VertexAttribDescriptor::DATA_FORMAT_CARD8:
				tmpString += "ub): ";
				switch (descr.numComponents) {
					case 1: { tmpString = tmpString + intToStr(baseIt->get1ub(TMP)); }; break;
					case 2: { Vector2ub v=baseIt->get2ub(TMP); tmpString=tmpString+intToStr(v[0])+", "+intToStr(v[1]); }; break;
					case 3: { Vector3ub v=baseIt->get3ub(TMP); tmpString=tmpString+intToStr(v[0])+", "+intToStr(v[1])+", "+intToStr(v[2]); }; break;
					case 4: { Vector4ub v=baseIt->get4ub(TMP); tmpString=tmpString+intToStr(v[0])+", "+intToStr(v[1])+", "+intToStr(v[2])+", "+intToStr(v[3]); }; break;
					default: // added arbituary number of components - mb
						{
							std::vector<char> vbuffer(VD->getSize());
							baseIt->getVertex(&vbuffer[0]);
							char * ptr = (char*)&vbuffer[0];
							ptr += TMP.getOffset();
							card8 * componentPtr = (card8*)ptr;
							for( unsigned c=0;c<descr.numComponents;c++ )
							{
								if( c )
									tmpString += ",";
								tmpString += intToStr(*componentPtr++);
							}
							
						}
				}
				break;

			case VertexAttribDescriptor::DATA_FORMAT_INT32:
				tmpString += "i): ";
				switch (descr.numComponents) {
					case 1: { tmpString = tmpString + intToStr(baseIt->get1i(TMP)); }; break;
					case 2: { Vector2i v=baseIt->get2i(TMP); tmpString=tmpString+intToStr(v[0])+", "+intToStr(v[1]); }; break;
					case 3: { Vector3i v=baseIt->get3i(TMP); tmpString=tmpString+intToStr(v[0])+", "+intToStr(v[1])+", "+intToStr(v[2]); }; break;
					case 4: { Vector4i v=baseIt->get4i(TMP); tmpString=tmpString+intToStr(v[0])+", "+intToStr(v[1])+", "+intToStr(v[2])+", "+intToStr(v[3]); }; break;
					default: // added arbituary number of components - mb
						{
							std::vector<char> vbuffer(VD->getSize());
							baseIt->getVertex(&vbuffer[0]);
							char * ptr = (char*)&vbuffer[0];
							ptr += TMP.getOffset();
							int32 * componentPtr = (int32*)ptr;
							for( unsigned c=0;c<descr.numComponents;c++ )
							{
								if( c )
									tmpString += ",";
								tmpString += intToStr(*componentPtr++);
							}

						}
				}
				break;

			case VertexAttribDescriptor::DATA_FORMAT_FLOAT32:
				tmpString += "f): ";
				switch (descr.numComponents) {
					case 1: { tmpString = tmpString + floatToStr(baseIt->get1f(TMP)); }; break;
					case 2: { Vector2f v=baseIt->get2f(TMP); tmpString=tmpString+floatToStr(v[0])+", "+floatToStr(v[1]); }; break;
					case 3: { Vector3f v=baseIt->get3f(TMP); tmpString=tmpString+floatToStr(v[0])+", "+floatToStr(v[1])+", "+floatToStr(v[2]); }; break;
					case 4: { Vector4f v=baseIt->get4f(TMP); tmpString=tmpString+floatToStr(v[0])+", "+floatToStr(v[1])+", "+floatToStr(v[2])+", "+floatToStr(v[3]); }; break;
					default: // added arbituary number of components - mb
						{
							std::vector<char> vbuffer(VD->getSize());
							baseIt->getVertex(&vbuffer[0]);
							char * ptr = (char*)&vbuffer[0];
							ptr += TMP.getOffset();
							float32 * componentPtr = (float32*)ptr;
							for( unsigned c=0;c<descr.numComponents;c++ )
							{
								if( c )
									tmpString += ",";
								tmpString += floatToStr(*componentPtr++);
							}

						}
				}
				break;

			default: PointInfo[i] += "error"; break;

		} // end switch data format

		if ((descr.name=="position") && (descr.numComponents==3) && (descr.dataFormat==VertexAttribDescriptor::DATA_FORMAT_FLOAT32)) {
			Matrix4f T = getTransformation(scene,CloudName);
			Vector3f currentPosition = transformVector3f(T,baseIt->get3f(TMP));

			// output distance to closest point
			HierarchicalKNNIterator hIt(PC,32,NULL);
			hIt.setSeekPointAndReset(currentPosition);
			hIt.next();
			AAT posAAT = PC->getAAT("position");
			float distToClosestPoint = norm(currentPosition - hIt.get3f(posAAT));
			PointInfo.insert( PointInfo.begin(), std::string("##DistToClosest: ") + floatToStr(distToClosestPoint) + "\n");

			// output distance to previous point
			if( lastPointHadPosition )
			{
				PointInfo.insert( PointInfo.begin(), std::string("##Distance: ") + floatToStr(norm(lastPointWorldPosition - currentPosition)) + "\n");
				previousPoint = true;
				previousPointPosition = lastPointWorldPosition;
			}
		 	lastPointHadPosition = true;
			lastPointWorldPosition = currentPosition;
		};

		PointInfo.push_back(tmpString);
	} // end for

	delete returnedIt;
	ObjectViewsTable::update(this);
	
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void PCIPointInfo::mouseMoved(int32 x, int32 y) {
}

void PCIPointInfo::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {
}

void PCIPointInfo::areaResize(card32 width, card32 height) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
   h = height;
   w = width;
   if (w==0) { WidthToHeight = 0.0f; return; };
   WidthToHeight = ((float)height)/((float)width);
	STD_GLOBAL_EXCEPTION_HANDLER_END
}



void PCIPointInfo::keyDown(GeneralKey key) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
   ObjectViewsTable::update(this);
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void PCIPointInfo::keyUp(GeneralKey key) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
   ObjectViewsTable::update(this);
	STD_GLOBAL_EXCEPTION_HANDLER_END
}


void PCIPointInfo::glDrawTool(GLContext *glContext) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
   forget(glContext);

	float32 PixelSize = (tan(scene->getRootState()->staticState->viewFrustum.getVerticalFieldOfView()*0.5f*((float32)M_PI/180.0f))) / (h*0.5f);
	float32 distB = norm(scene->getRootState()->staticState->camera.getPosition() - lastPointWorldPosition);
   float32 ArrowSize = (10.0f*distB*PixelSize);

	if (lastPointHadPosition) {
		glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
//         glLoadIdentity();
//         glMultMatrixf(scene->getRootState()->dynamicState->ctm.data());

			glPushAttrib(GL_ALL_ATTRIB_BITS);
            glDisable(GL_LIGHTING);
				glTranslatef(lastPointWorldPosition[0], lastPointWorldPosition[1], lastPointWorldPosition[2]);
            glColor4f(1.0f,0.9f,0.1f,1.0f);
            glLineWidth(1.5f);
            glBegin(GL_LINES);
					// draw 2D-Cross
					glVertex3fv((-(scene->getRootState()->staticState->camera).getOrthoNormRightDirection()*ArrowSize).data());
					glVertex3fv(((scene->getRootState()->staticState->camera).getOrthoNormRightDirection()*ArrowSize).data());
					glVertex3fv((-(scene->getRootState()->staticState->camera).getOrthoNormUpDirection()*ArrowSize).data());
					glVertex3fv(((scene->getRootState()->staticState->camera).getOrthoNormUpDirection()*ArrowSize).data());
/*					
					// draw 3D-Cross
					glVertex3f(-0.05f, 0.0f, 0.0f);
					glVertex3f(0.05f, 0.0f, 0.0f);
					glVertex3f(0.0f, -0.05f, 0.0f);
					glVertex3f(0.0f, 0.05f, 0.0f);
					glVertex3f(0.0f, 0.0f, -0.05f);
					glVertex3f(0.0f, 0.0f, 0.05f);
*/
				glEnd();

//				// draw sphere
//				GLUquadricObj* qo = gluNewQuadric();
//				gluSphere(qo, 0.05f, 16, 8);
			glPopAttrib();

		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();

		if( previousPoint )
		{
			glBegin(GL_LINES);
			glColor3f(0.2f,0.2f,1.0f);
			glVertex3fv(lastPointWorldPosition.data());
			glVertex3fv(previousPointPosition.data());
			glEnd();
		}
	};
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

