//---------------------------------------------------------------------------
#ifndef PCIPointInfoH
#define PCIPointInfoH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
//---------------------------------------------------------------------------

namespace X4 {

	//class PCIPointInfoEditorWidget;

	class X3EDITOR_API PCIPointInfo : public PCInteractionTool {

	private:
		static MetaClass *PCIPointInfoMetaClass;
		static Object* newInstance();

		// parameters
		/// stringlist with informations about last point, every element equals one attribute of the point
		std::vector<std::string> PointInfo;
		/// holds mouse position and the name of the cloud to which the clicked point belongs
		std::string MouseAndCloudInfo;

		// functions for variable std::string array property
		// maybe: IMPLEMENT_VAR_ARRAY_METHODS_STL(PointInfo)
		void setNumPointInfos(mpcard size) {PointInfo.resize(size);}
		void deletePointInfo(mpcard index) {x4Assert(index < PointInfo.size()); PointInfo.erase(PointInfo.begin() + index );}
		char * getPointInfo(mpcard index) {x4Assert(index < PointInfo.size()); return (char*)&PointInfo[index];}
		mpcard getNumPointInfos() {return (mpcard)PointInfo.size();}
		std::string getPointInfoName(mpcard index) {x4Assert(index < PointInfo.size()); return PointInfo[index];}

		// internal tmp
		/// temp var: width and height of window
		card32 w,h;
		/// temp var: factor of width to height
		float32 WidthToHeight;
		/// true if last clicked point had position attribute
		bool lastPointHadPosition;
		/// saves position of last clicked point (including the cloud transformation)
		Vector3f lastPointWorldPosition;

		/// saves previous point to visualize the distance
		bool previousPoint;
		Vector3f previousPointPosition;


 public:
   PCIPointInfo();

   /// gets attribute info of the point the user clicked on
   void getPointInfo();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void glDrawTool(GLContext *glContext);

// friend PCIPointInfoEditorWidget;
};


}

#endif

