//---------------------------------------------------------------------------
#ifndef PCISelectionToolH
#define PCISelectionToolH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
//---------------------------------------------------------------------------

namespace X4 {

class PCISelectionToolEditorWidget;

class X3EDITOR_API PCISelectionTool : public PCInteractionTool {

 private:
   static MetaClass *PCISelectionToolMetaClass;
   static Object* newInstance();

   // parameters
   /// size of the brush in pixels
   card32 brushsize;
   /// tolerance value for color attrib selection
   card32 tolerance;
   /// selection mode which is currently activated. 0=Cloud, 1=Point, 2=CircleBrush, 3=RectangleArea, 4=QuadBrush, 5=CircleArea
   card32 mode;
   /// true while ctrl key is pressed. Used for deselection mode.
   bool ctrlPressed;
   /// true if surface mode is activated. See corresponding X4 help file for details of surface mode.
   bool SurfaceMode;
   /// color for selection by color attrib
   Vector3f SelColor;
   /// angle between neighbour normals for growing selection
   float normalAngleDiff;
   /// use topology when growing
   bool useTopology;

   // internal tmp
   /// temp var: width and height of window
   card32 w,h;
   /// temp var: factor of width to height
   float32 WidthToHeight;
   /// temp var: true while left mouse button pressed
   bool leftMBdown;
   /// temp var: true while right mouse button pressed
   bool rightMBdown;
   /// temp var for mouse x and y when mouse button pressed
   int32 leftMBdownX, leftMBdownY;
   /// temp var for mouse x and y when mouse moved
   int32 leftMBmoveX, leftMBmoveY;


 public:
   PCISelectionTool();

   inline void setMode(card32 m) { mode = m; };
   inline card32 getMode() { return mode; };
   inline void setTolerance(card32 t) { tolerance = t; };
   inline card32 getTolerance() { return tolerance; };
   inline void setBrushSize(card32 bs)  { brushsize = bs; };
   inline card32 getBrushSize() { return brushsize; };
   inline void setSurfaceMode(bool sps) { SurfaceMode = sps; };
   inline bool getSurfaceMode() { return SurfaceMode; };
   inline void setSelColor(Vector3f col) { SelColor = col; };
   inline Vector3f getSelColor() { return SelColor; };
   inline void setDeselect(bool des) { ctrlPressed = des; };
   inline bool getDeselect() { return ctrlPressed; };
   inline void setUseTopology(bool use) { useTopology = use;}
   inline bool getUseTopology() { return useTopology; }
   inline void setNormalAngleDifference(float diff) { normalAngleDiff = diff; }
   inline float getNormalAngleDifference() { return normalAngleDiff; }

   void extendToComponents();
   /// inverts selected points
   void invertSelection();
   /// selects all clouds in scene
   void selectAllClouds();
   /// selects all points in scene
   void selectAllPoints();
   /// clears cloud selection
   void clearCloudSelection();
   /// clears point selection
   void clearPointSelection();
   /// select points by color attrib
   void selectByColor();
   /// grow selection from current points based on normal angle differences
   void growSelectionByNormal();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void glDrawTool(GLContext *glContext);

   /// draws a rectangle in glDrawTool
   void drawRectangle(float32 x1, float32 y1, float32 x2, float32 y2);
   /// draws a filled rectangle in glDrawTool
   void drawFilledRectangle(float32 x1, float32 y1, float32 x2, float32 y2);
   /// draws a circle in glDrawTool
   void drawCircle(float32 x1, float32 y1, float32 r);
   /// draws a filled circle in glDrawTool
   void drawFilledCircle(float32 x1, float32 y1, float32 r);

 friend PCISelectionToolEditorWidget;
};


}

#endif

