//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "PCImageViewers.h"
//---------------------------------------------------------------------------
#include "Tools.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra.inline.h"
#include "FileDialogs.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



static bool containsFlagsFormat(const PointSet *pc) {
   const VertexDescriptor *vd = pc->getDescr();
   bool containsFlags = vd->providesAttribute("flags");
   if (containsFlags) {
      VAD vad = (*vd)[vd->getIndex("flags")];
      containsFlags = (vad.dataFormat == VAD::DATA_FORMAT_INT32) && (vad.numComponents == 1);
   }
   return containsFlags;
}

PCImageViewer::PCImageViewer(QWidget* parent,const PointSet* pc)
   : QWidget(NULL)
{
   this->pc = pc;
   ui.setupUi(this);
   bmp = NULL;
   ui.combo_channel->addItem("color");
   ui.combo_channel->addItem("intensity");
	
	ui.combo_channel->addItem("normal");
	ui.combo_channel->addItem("position");
	

	ui.sb_scaling->setValue(1.0f);
	//scrollArea = new QScrollArea(ui.frame_2);
	ui.scrollArea->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
   ui.scrollArea->setBackgroundRole(QPalette::Dark);
   ui.ImageLabel->setBackgroundRole(QPalette::Dark);
   ui.scrollArea->setWidget(ui.ImageLabel);
	//scrollArea->resize(791,451);
	
   rebuildView();
  
   ui.cb_invalid->setEnabled( containsFlagsFormat(pc));
}
//---------------------------------------------------------------------------
void PCImageViewer::on_sb_zoom_valueChanged()
{
	if (ui.sb_zoom->value() < 0.1) ui.sb_zoom->setValue(0.1) ;
   if (ui.sb_zoom->value() > 10.0) ui.sb_zoom->setValue(10.0) ;
   rebuildView();
   
}
//---------------------------------------------------------------------------

static inline card8 clamp0_255(float32 f) {
   if (f > 255) return 255;
   if (f < 0) return 0;
   return (card8)f;
}

void PCImageViewer::rebuildView() {
   double zoom = ui.sb_zoom->value();
   const bool doswap = ui.cb_xy->checkState();
   const bool markIV = ui.cb_invalid->checkState();
   const bool showLength = ui.cb_3vec->checkState();
   if (zoom < 0.1) zoom = 0.1;
   if (zoom > 10.0) zoom = 10.0;
   const card32 width = pc->getWidth();
   const card32 height = pc->getHeight();
   imgWidth = width * zoom;
   imgHeight = height * zoom;
   if (doswap) swap(imgWidth, imgHeight);
   if (imgWidth > 8192) imgWidth = 8192;
   if (imgHeight > 8192) imgHeight = 8192;
   ui.ImageLabel->resize(imgWidth,imgHeight);
   
   delete bmp;
   bmp=NULL;
      

   std::string attribName = ui.combo_channel->currentText().toLatin1().constData();
   const VertexDescriptor *vd = pc->getDescr();
   bool containsFlags = containsFlagsFormat(pc);
   bool containsField = vd->providesAttribute(attribName);

   if (!containsField) {
	   ui.errorLabel->setText(QString(" - Channel not found - "));
	   ui.errorLabel->setVisible(true);
	   ui.scrollArea->setVisible(false);
	   return;
   }
   
	

   AAT SELECTED_CHANNEL = pc->getAAT(attribName);
   AAT FLAGS;
   if (containsFlags) FLAGS = pc->getAAT("flags");

   card32 numAttribElems = (*vd)[vd->getIndex(attribName)].numComponents;
   card32 dataFormat = (*vd)[vd->getIndex(attribName)].dataFormat;

   if (numAttribElems > 4) {
      ui.errorLabel->setText(QString("-more than 4 channels- "));
	   ui.errorLabel->setVisible(true);
	   ui.scrollArea->setVisible(false);
      return;
   }
   if (!(dataFormat == VAD::DATA_FORMAT_CARD8 || dataFormat == VAD::DATA_FORMAT_INT32 || dataFormat == VAD::DATA_FORMAT_FLOAT32)) {
       ui.errorLabel->setText(QString(" - unsupported format - "));
	   ui.errorLabel->setVisible(true);
	   ui.scrollArea->setVisible(false);
      return;
   }

   ui.errorLabel->setVisible(false);
   ui.scrollArea->setVisible(true);
   bmp=new QImage(imgWidth,imgHeight,QImage::Format_ARGB32);
   
   const float32 s = ui.sb_scaling->value();
   for (card32 y=0; y<imgHeight; y++) {
	   Vector4ub *scanline = (Vector4ub*)(bmp->scanLine(y));
      for (card32 x=0; x<imgWidth; x++) {
         card32 xSource = x/zoom;
         card32 ySource = y/zoom;
         if (doswap) swap(xSource, ySource);
         if (xSource >= width) xSource = width-1;
         if (ySource >= height) ySource = height-1;
         Vector4f col;
         if (dataFormat == VAD::DATA_FORMAT_CARD8) {
            switch (numAttribElems) {
              case 1: {
                card8 v = pc->get<card8, 1>(xSource, ySource, SELECTED_CHANNEL)[0];
                float32 c = v/255.0;
                col = makeVector4f(c*s, 0, 0, 0);
              break;}
              case 2: {
                Vector2ub v = pc->get<card8, 2>(xSource, ySource, SELECTED_CHANNEL);
                col = makeVector4f(v[0]/255.0*s, v[1]/255.0*s, 0, 0);
              break;}
              case 3: {
                Vector3ub v = pc->get<card8, 3>(xSource, ySource, SELECTED_CHANNEL);
                col = makeVector4f(v[0]/255.0*s, v[1]/255.0*s, v[2]/255.0*s, 0);
              break;}
              case 4: {
                Vector4ub v = pc->get<card8, 4>(xSource, ySource, SELECTED_CHANNEL);
                col = makeVector4f(v[0]/255.0*s, v[1]/255.0*s, v[2]/255.0*s, v[3]/255.0*s);
              break;}
            }
         } else if (dataFormat == VAD::DATA_FORMAT_INT32) {
            switch (numAttribElems) {
              case 1: {
                int32 v = pc->get<int32, 1>(xSource, ySource, SELECTED_CHANNEL)[0];
                col = makeVector4f(v*s, 0, 0, 0);
              break;}
              case 2: {
                Vector2i v = pc->get<int32, 2>(xSource, ySource, SELECTED_CHANNEL);
                col = makeVector4f(v[0]*s, v[1]*s, 0, 0);
              break;}
              case 3: {
                Vector3i v = pc->get<int32, 3>(xSource, ySource, SELECTED_CHANNEL);
                col = makeVector4f(v[0]*s, v[1]*s, v[2]*s, 0);
              break;}
              case 4: {
                Vector4i v = pc->get<int32, 4>(xSource, ySource, SELECTED_CHANNEL);
                col = makeVector4f(v[0]*s, v[1]*s, v[2]*s, v[3]*s);
              break;}
            }
         } else if (dataFormat == VAD::DATA_FORMAT_FLOAT32) {
            switch (numAttribElems) {
              case 1: {
                float32 v = pc->get<float32, 1>(xSource, ySource, SELECTED_CHANNEL)[0];
                col = makeVector4f(v*s, 0, 0, 0);
              break;}
              case 2: {
                Vector2f v = pc->get<float32, 2>(xSource, ySource, SELECTED_CHANNEL);
                col = makeVector4f(v[0]*s, v[1]*s, 0, 0);
              break;}
              case 3: {
                Vector3f v = pc->get<float32, 3>(xSource, ySource, SELECTED_CHANNEL);
                col = makeVector4f(v[0]*s, v[1]*s, v[2]*s, 0);
              break;}
              case 4: {
                Vector4f v = pc->get<float32, 4>(xSource, ySource, SELECTED_CHANNEL);
                col = makeVector4f(v[0]*s, v[1]*s, v[2]*s, v[3]*s);
              break;}
            }
         }

			float32 offset = (float32)ui.sbOffset->value();
         col += makeVector4f(offset, offset, offset, 0.0f);
         

         if (showLength) {
            float32 colNorm = norm(col);
            col[0] = colNorm; col[1] = colNorm; col[2] = colNorm; col[3] = colNorm;
         } else if (numAttribElems == 1) {
            col[2] = col[1] = col[0]; 
         }
         col *= 255.0f;
         col[3] = 255.0f;
         if (markIV && containsFlags) {
            if (((pc->get<int32, 1>(xSource, ySource, FLAGS)[0]) & PF_FLAG_INVALID) != 0) {
               col = makeVector4f(128.0f, 0.0f, 0.0f, 255.0f);
            }
         }

         Vector4ub colUB = makeVector4ub(clamp0_255(col[2]),clamp0_255(col[1]),clamp0_255( col[0]), clamp0_255(col[3]));

         *scanline =  colUB;
         scanline++;
      }
	  
   }
  
   PB_paintAreaPaint();
}
void PCImageViewer::on_cb_invalid_clicked()
{
   rebuildView();
   
}
//---------------------------------------------------------------------------

/*void __fastcall TPCImageViewer::CBChannelKeyDown(TObject *Sender,
      WORD &Key, TShiftState Shift)
{
   if (Key == VK_RETURN) {
      rebuildView();
   }
   forget(Shift);
   forget(Sender);
}
//---------------------------------------------------------------------------
*/
void PCImageViewer::PB_paintAreaPaint()
{
   	if (bmp!=NULL)
		ui.ImageLabel->setPixmap(QPixmap::fromImage(*bmp));
	
	ui.ImageLabel->show();
   
}
//---------------------------------------------------------------------------
/*
void PCImageViewer::ScrollBox1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  // PB_paintAreaPaint(Sender);
   forget(X|Y);
   forget(Sender);
   forget(Shift);
}*/
//---------------------------------------------------------------------------

void PCImageViewer::on_combo_channel_activated()
{
   rebuildView();
  
}

void PCImageViewer::on_sb_scaling_valueChanged(){

	rebuildView();
}

void PCImageViewer::on_cb_xy_stateChanged(){

	rebuildView();
}

void PCImageViewer::on_cb_invalid_stateChanged(){

	rebuildView();
}

void PCImageViewer::on_cb_3vec_stateChanged(){

	rebuildView();
}

void PCImageViewer::on_cb_offset_stateChanged(){

	rebuildView();
}

void PCImageViewer::on_saveButton_clicked()
{
	if (bmp!=NULL)
	{
		std::string filename = FileDialogs::getSaveFileName(this, "save Image", "*.png" );
		bmp->save(QString(filename.c_str()),"PNG");
	}

}

//---------------------------------------------------------------------------

#include "PCImageViewers_moc.h"
