//---------------------------------------------------------------------------
#include "stdafx.h"
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------
#include "SelectionTools.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "SceneEditorWidget.h"
#include "ProgressWindow.h"
#include "ProgressFrameWidget.h"
#include "ProgressWindowWidget.h"
#include "Configuration.h"
#include "BinaryObjectStreams.h"
#include <stdio.h>
#include "PointCloudTools.h"
#include "ApplicationDirectories.h"

//---------------------------------------------------------------------------

IMPLEMENT_X4_ABSTRACT_CLASS( PCInteractionToolSceneSettings, 0 )
{
	BEGIN_CLASS_INIT( PCInteractionToolSceneSettings );
	//INIT_PROPERTY_TABLE();
}

IMPLEMENT_X4_ABSTRACT_CLASS( PCInteractionTool, 0 )
{
	BEGIN_CLASS_INIT( PCInteractionTool )
}

PCInteractionTool::PCInteractionTool() 
{

	scene = NULL;
	onChange = NULL;
	screenSizeX = 0;
	screenSizeY = 0;
	toolWidget = NULL;
	sceneEditorWidget = NULL;
	autoSaveAndRestoreToolParameters = false;
	autoTriggerFullLayout = false;
}

PCInteractionToolSceneSettings* PCInteractionTool::getOrCreateSceneSettings(Scene* scene, const std::string& name, const MetaClass* sClass)
{
	PCInteractionToolSceneSettings* settings = NULL;

	sClass = sClass ? sClass : MetaClass::getClassByName(name);

	if (!sClass) return NULL;

	if (scene->getAttachments()->getData(name))
	{
		settings = dynamic_cast<PCInteractionToolSceneSettings*>(scene->getAttachments()->getData(name));
	}else
	{
		settings = dynamic_cast<PCInteractionToolSceneSettings*>(sClass->newInstance());
		settings->setup(name, AttachedData::ADF_PERSISTENT);
		scene->getAttachments()->attachData(settings);
	}

	return settings;
}

void PCInteractionTool::connectToScene(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget, QWidget* toolWidget) 
{
	if (autoSaveAndRestoreToolParameters)
		loadPCISettings();
	
	this->scene = scene;
	this->onChange = onChange;
	this->sceneEditorWidget = sceneEditorWidget;


	// first delete tools progress bar
	if (this->toolWidget && toolProgressWindow)
	{
		toolWidget->layout()->removeWidget(toolProgressWindow->getWidget());
		toolWidget->update();
		delete toolProgressWindow->getWidget();
		delete toolProgressWindow;
	}
	this->toolWidget = toolWidget;

	// if tool's widget is given, then add a progress widget there
	if (toolWidget != NULL)
	{
		toolProgressWindow = new ProgressWindow(new ProgressFrameWidget(toolWidget));
		toolWidget->layout()->addWidget(toolProgressWindow->getWidget());
		toolProgressWindow->getWidget()->hide();
		toolWidget->update();
		toolWidget->updateGeometry();
	}

	connectToSceneImpl(scene, onChange, sceneEditorWidget, toolWidget);

	if (autoTriggerFullLayout)
		dynamic_cast<X4::SceneEditorWidget*>(sceneEditorWidget)->on_action_Auto_Layout_triggered(true);
}

PCInteractionTool::~PCInteractionTool() 
{
   delete onChange;
};

bool PCInteractionTool::forceCamByDefault() 
{
   return false;
}

bool PCInteractionTool::getIntersectionFromCursor( int x, int y, Vector3f &retPoint, std::string &pcName, PointCloud ** pc ) 
{
   PointCloudIterator* it = getClosestPointIteratorFromAllPoints(x, y, screenSizeX, screenSizeY, 4,
			&getScene()->getRootState()->staticState->camera, 
			&getScene()->getRootState()->staticState->viewFrustum,
			getScene(),
			pcName,
			&retPoint
	);

	if( it == NULL )
		return false;

	BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(it);
	*pc = getPointCloud( getScene(), pcName );

	x4Assert( bIt != NULL );
	x4Assert( *pc != NULL );

	retPoint = bIt->get3f( (*pc)->getAAT( "position") );
	delete it;
	return true;
}


std::vector<PointCloud*> X4::PCInteractionTool::getAllPointCloudsFromScene()
{
	std::vector<PointCloud*> ret;

	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			PointCloud * pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				ret.push_back(pc);
		}
		sceneIt->next();
	}

	return ret;
}


std::vector<PointCloud*> X4::PCInteractionTool::getSelectedPointCloudsFromScene()
{
	std::vector<PointCloud*> ret;
	std::vector<std::string> sel = getScene()->getRootState()->staticState->getSelectedNodes();
	for( unsigned i=0;i<sel.size();i++ )
	{
		PointCloud * pc = dynamic_cast<PointCloud*>(
			getPointCloud(getScene(), sel[i]));
		if( pc )
			ret.push_back( pc );
	}

	return ret;
}

bool X4::PCInteractionTool::guessClickPoint( int x, int y, unsigned rad, Vector3f &retPoint )
{
	SceneEditorWidget * se = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
	if( !se )
		return false;
	return se->guessClickPoint(x,y,rad,retPoint);
}


bool PCInteractionTool::estimateOOBBFromScreenArea( const Vector2i &x1, const Vector2i &x2, BoundingBox3f &bb, Matrix4f &worldToBB, float depthOutlierTolerance, Vector3f * firstAxis) 
{
	SceneEditorWidget * se = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
	if( !se )
		return false;
	return se->estimateOOBBFromScreenArea(x1,x2,bb,worldToBB,depthOutlierTolerance,firstAxis);
}


PointCloud * X4::PCInteractionTool::getSingleSelectedPointCloudFromScene()
{
	std::vector<PointCloud*> ret;
	std::vector<std::string> sel = getScene()->getRootState()->staticState->getSelectedNodes();
	if( sel.size() != 1)
		return NULL;
	return dynamic_cast<PointCloud*>(getPointCloud(getScene(), sel.front()));
}

void X4::PCInteractionTool::loadPCISettings()
{
	
	std::string configPath = ApplicationDirectories::getConfigDir();
	std::string filename = configPath + "\\"+  this->getInstanceClass()->getClassName() + ".x4obj";
	debugOutput << " trying to load last settings in " << filename << "\n"; 

	FILE *file = fopen(filename.c_str(),"rb");; 

	if (file) {
		fclose(file);
		PCInteractionTool *s = NULL;
		BinaryInputObjectStream in(filename.c_str());
		in.readAnyObject(s);
		this->assign(s);

	}else
		debugOutput << "  file not found - using default settings.\n";
}

void X4::PCInteractionTool::savePCISettings()
{
	std::string configPath = ApplicationDirectories::getConfigDir();
	std::string filename = configPath + "\\"+  this->getInstanceClass()->getClassName() + ".x4obj";
	BinaryOutputObjectStream out(filename.c_str());
	out.writeObject(this);
	debugOutput << " saved last settings in " << filename << "\n"; 
}

void X4::PCInteractionTool::deactivateTool()
{
	if (autoSaveAndRestoreToolParameters)
	{
		savePCISettings();
	}
}

/// evil hack to permit easy switching between threaded/non-threaded
void X4::PCInteractionTool::execOperatorNoArgs(OperatorNoArgs *op)
{
	(*op)();
}

void PCInteractionTool::mouseDownPassToCam(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->mouseDownPassToCam(x, y, buttonsState, modifiersState);
   }
}

void PCInteractionTool::mouseMovedPassToCam(int32 x, int32 y) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->mouseMovedPassToCam(x, y);
   }
}

void PCInteractionTool::mouseUpPassToCam(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->mouseUpPassToCam(x, y, buttonsState, modifiersState);
   }
}

void PCInteractionTool::areaResizePassToCam(card32 width, card32 height) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->areaResizePassToCam(width, height);
   }
}

void PCInteractionTool::mouseWheelRotatedPassToCam(int32 rotatedDelta, Modifiers modifiersState) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->mouseWheelRotatedPassToCam(rotatedDelta, modifiersState);
   }
}

void PCInteractionTool::keyDownPassToCam(GeneralKey key)
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->keyDownPassToCam(key);
   }
}

void PCInteractionTool::keyUpPassToCam(GeneralKey key)
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->keyUpPassToCam(key);
   }
}

#if 0
ProgressWindow* PCInteractionTool::createProgressWindow()
{
	// if tool's widget is given, then add a progress widget there
	QWidget* widget = getToolWidget();
	if (widget == NULL)
	{
		warning(__FUNCTION__ ": cannot add new progress widget, because there is no widget with the tool specified and hence I have no idea where to add the progress bar");
		return NULL;
	}

	ProgressWindow* prg = new ProgressWindow(new ProgressFrameWidget(widget));
	widget->layout()->addWidget(prg->getWidget());
	widget->update();
	return prg;

    /*SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
    if (w)
    {
        ProgressWindow* prg = new ProgressWindow(new ProgressFrameWidget(w));
        w->addProgressbarWidget(prg->getWidget());
        return prg;
    }
    return NULL;*/
}

void PCInteractionTool::deleteProgressWindow(ProgressWindow* prg)
{
	QWidget* widget = getToolWidget();
	if (widget)
	{
		widget->layout()->removeWidget(prg->getWidget());
		widget->update();
		delete prg->getWidget();
		delete prg;
	}

    /*SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
    if (w)
    {
        w->removeProgressbarWidget(prg->getWidget());
        delete prg->getWidget();
        delete prg;
    }*/
}
#endif 
