/***************************************************************************
*   Copyright (c) 2009   Art Tevs                                         *
*                                                                         *
*   This library is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation; either version 3 of        *
*   the License, or (at your option) any later version.                   *
*                                                                         *
*   This library is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU Lesse General Public License for more details.                    *
*                                                                         *
*   The full license is in LICENSE file included with this distribution.  *
***************************************************************************/

//---------------------------------------------------------------------------
#include "stdafx.h"

#include "PCInteractionToolThreaded.h"
#include "GLContext.h"
#include <QtCore/QTimer.h>
#include <ctime>
#include "Random.h"
using namespace X4;

//---------------------------------------------------------------------------
IMPLEMENT_X4_ABSTRACT_CLASS( PCInteractionToolThreaded, 0 )
{
    BEGIN_CLASS_INIT( PCInteractionToolThreaded );
}

//---------------------------------------------------------------------------
PCInteractionToolThreaded::PCInteractionToolThreaded()
{
    _isActive = false;
    _method = NULL;
	_randomSeed = 1;

    QThread::connect(this, SIGNAL(started()), this, SLOT(threadStarted()));
    QThread::connect(this, SIGNAL(terminated()), this, SLOT(threadTerminated()));
    QThread::connect(this, SIGNAL(finished()), this, SLOT(threadFinished()));
	
	QObject::connect(this, SIGNAL(execOperatorNoArgs(OperatorNoArgs*)), this, SLOT(slotExecCallback(OperatorNoArgs*)));
}

//---------------------------------------------------------------------------
PCInteractionToolThreaded::~PCInteractionToolThreaded()
{
}

//---------------------------------------------------------------------------
void PCInteractionToolThreaded::slotExecCallback(OperatorNoArgs* cb)
{
	(*cb)();
}

//---------------------------------------------------------------------------
void PCInteractionToolThreaded::activateTool()
{
    activateToolImplementation();
}

//---------------------------------------------------------------------------
void PCInteractionToolThreaded::deactivateTool()
{
    QThread::terminate();
    deactivateToolImplementation();
}

//---------------------------------------------------------------------------
void PCInteractionToolThreaded::glDrawTool( GLContext *glContext )
{
    glContext->activate();
    glDrawToolImplementation(glContext);
}

//---------------------------------------------------------------------------
void PCInteractionToolThreaded::dispatchMethod(ClassMethod* method)
{    
    _mutex.lock();
        this->_method = method;
    _mutex.unlock();
    QThread::start();
}

//---------------------------------------------------------------------------
void PCInteractionToolThreaded::run()
{
    // check the current state, in an atomic way
    if (isActive()) return;

	// init random methods with some seed
	srand(_randomSeed);
	qsrand(_randomSeed);

	// call method
    if (dynamic_cast<NoArgsClassMethod*>(_method)) dynamic_cast<NoArgsClassMethod*>(_method)->call(this);
}

//---------------------------------------------------------------------------
void PCInteractionToolThreaded::threadStarted()
{
    _mutex.lock();
        //debugOutput << "Thread started !\n";
        _isActive = true;
		_randomSeed = random32bit();
        onExecutionStart();
    _mutex.unlock();
}

//---------------------------------------------------------------------------
void PCInteractionToolThreaded::threadTerminated()
{
    _mutex.lock();
        //debugOutput << "Thread terminated !\n";
        _isActive = false;
        onExecutionStop();
    _mutex.unlock();
}

//---------------------------------------------------------------------------
void PCInteractionToolThreaded::threadFinished()
{
    _mutex.lock();
         //debugOutput << "Thread finished !\n";
        _isActive = false;
        onExecutionStop();
    _mutex.unlock();
}

#include "PCInteractionToolThreaded_moc.h"

