//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader2DImagesPlusAlpha.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* PointCloudImporterReader2DImagesPlusAlpha::PointCloudImporterReader2DImagesPlusAlphaMetaClass = NULL;

const MetaClass* PointCloudImporterReader2DImagesPlusAlpha::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterReader2DImagesPlusAlpha::newInstance() {
   return new PointCloudImporterReader2DImagesPlusAlpha;
}

const MetaClass* PointCloudImporterReader2DImagesPlusAlpha::getClass() {
   if (PointCloudImporterReader2DImagesPlusAlphaMetaClass == NULL) {
      PointCloudImporterReader2DImagesPlusAlphaMetaClass = new MetaClass("PointCloudImporterReader2DImagesPlusAlpha", "no description.",
                                       newInstance, sizeof(PointCloudImporterReader2DImagesPlusAlpha));
   }
   return PointCloudImporterReader2DImagesPlusAlphaMetaClass;
}

void PointCloudImporterReader2DImagesPlusAlpha::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PointCloudImporterReader2DImagesPlusAlphaMetaClass->copyParentProperties();
}

void PointCloudImporterReader2DImagesPlusAlpha::shutdown() {
   delete PointCloudImporterReader2DImagesPlusAlphaMetaClass;
}

PointCloudImporterReader2DImagesPlusAlpha::PointCloudImporterReader2DImagesPlusAlpha() {
	invWidth = 0;
	invHeight = 0;
   inv255 = 1.0f/255.0f;

   myImage = NULL;
   myAlphaImage = NULL;
	x = 0;
   y = 0;

	myVD = new VertexDescriptor();
	myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("alpha", 1, VAD::DATA_FORMAT_CARD8));
	myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
}

PointCloudImporterReader2DImagesPlusAlpha::~PointCloudImporterReader2DImagesPlusAlpha() {
   delete myAlphaImage;
   delete myImage;
}

void PointCloudImporterReader2DImagesPlusAlpha::assign(const Object* obj, X4CopyContext *context) {
   PointCloudImporterReader::assign(obj,context);

   const PointCloudImporterReader2DImagesPlusAlpha *other = dynamic_cast<const PointCloudImporterReader2DImagesPlusAlpha*>(obj);
   x4Assert(other != NULL);
}


bool PointCloudImporterReader2DImagesPlusAlpha::beginImport(std::string filnam, bool readTrianglesIfPossible) {

	// test if filenames are correct. alpha-filename has be the color-filename + "-maske"
	std::string alphafilnam = filnam;
	alphafilnam.insert(alphafilnam.find_last_of("."), "-maske");
	if (!X4::FileExists(alphafilnam.c_str())) {
		WARNINGMESSAGE("PointCloudImporterReader2DImagesPlusAlpha::beginImport()\nAlpha file does not exist. Alpha-filename has be the color-filename + '-maske'!");
		return false;
	};

	// call beginImport just once
   if (ImportStarted) return false;
	x4Assert(buffer == NULL);

	if ( (!myVD->providesAttribute("position")) || (!myVD->providesAttribute("color")) || (!myVD->providesAttribute("alpha")) || (!myVD->providesAttribute("flags")) ) {
		WARNINGMESSAGE("PointCloudImporterReader2DImagesPlusAlpha::beginImport()\nVertex descriptor doesn't have one or more needed attributes (position,color,alpha,flags)!");
		return false;
	}

	std::string ext = extractFileExt(strUCase(filnam));
   if ( !((ext==".BMP")||(ext==".JPG")||(ext==".PNG")) ) {
		// no valid image
		WARNINGMESSAGE("PointCloudImporterReader2DImagesPlusAlpha::beginImport()\nFile ending is not a valid image type (must be BMP, JPG or PNG)!");
		return false;
	}

   // load color picture
	delete myImage;
	myImage = new QImage(QImage(QString(filnam.c_str())).convertToFormat(QImage::Format_RGB32));
	if (myImage->isNull()) {
		WARNINGMESSAGE("PointCloudImporterReader2DImagesPlusAlpha::beginImport()\nFile \"" + filnam + "\" is not a valid image file!");
		return false;
	};

   // load alpha picture
	delete myAlphaImage;
	myAlphaImage = new QImage(QImage(QString(alphafilnam.c_str())).convertToFormat(QImage::Format_RGB32));
	if (myAlphaImage->isNull()) {
		WARNINGMESSAGE("PointCloudImporterReader2DImagesPlusAlpha::beginImport()\nAlpha-File \"" + alphafilnam + "\" is not a valid image file!");
		return false;
	};

	width = myImage->width();
	IKnowWidth = true;
	height = myImage->height();
	IKnowHeight = true;
	if ((width!=myAlphaImage->width()) || (height != myAlphaImage->height())) {
		WARNINGMESSAGE("PointCloudImporterReader2DImagesPlusAlpha::beginImport()\nAlpha-file resolution does not match Color-file resolution!");
		return false;
	}
	invWidth = 1.0f/((float32)width);
	invHeight = 1.0f/((float32)height);

	x = 0;
	y = 0;

	ImportStarted = false;
	ImportFinished = false;

	delete[] buffer;
	buffer = new card8[myVD->getSize()];

	return true;

}


void PointCloudImporterReader2DImagesPlusAlpha::endImport() {
	x = 0;
	y = 0;
	ImportStarted = false;
	ImportFinished = false;
	delete[] buffer;
	buffer = NULL;
};


mpcard PointCloudImporterReader2DImagesPlusAlpha::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {
      x4Assert(false);
      return 0;
   }
}


mpcard PointCloudImporterReader2DImagesPlusAlpha::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {
      x4Assert(false);
      return 0;
   }
}


mpcard PointCloudImporterReader2DImagesPlusAlpha::getProgressPercentage() {
	return ((y*width+x)*100)/(width*height);
};


const card8* PointCloudImporterReader2DImagesPlusAlpha::getNextPoint() {

   if (ImportFinished) return NULL;
   x4Assert(buffer!=NULL);
   ImportStarted = true;
 
	// color and alpha data
	QRgb pixel = myImage->pixel(x,y);
	Vector3f color = makeVector3f(qRed(pixel),qGreen(pixel),qBlue(pixel))*inv255;

	StaticVector<card8, 1> alpha;
	alpha[0] = qGray(myAlphaImage->pixel(x,y));

   Vector3f pos;
   pos[0] = x * invWidth;
   pos[1] = y * invWidth;
   pos[2] = 0.0f;

   StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("position")))) = pos;
   *((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("color")))) = color;
   *((StaticVector<card8, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("alpha")))) = alpha;
   *((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;

   x++;
   if (x >= width) {
      x = 0;
      y++;
      if (y >= height) ImportFinished = true;
   };

	return buffer;
}


bool PointCloudImporterReader2DImagesPlusAlpha::supportsTriangleImport() {
	return false;
}


const card8* PointCloudImporterReader2DImagesPlusAlpha::getNextTriangle() {
	return NULL;
}
