//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderBSM.h"
//---------------------------------------------------------------------------
#include "X4Exception.h"
//---------------------------------------------------------------------------
#include <fstream>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( PointCloudImporterReaderBSM ,0)
{
	BEGIN_CLASS_INIT( PointCloudImporterReaderBSM );
}

// -----------------------------------------------------------------------------------------------
// ----------------------------------- James Diebel's Import Code --------------------------------
// -----------------------------------------------------------------------------------------------

////////////////////////////
// SimpleMeshVertex Class //
////////////////////////////
 
class JDSimpleMeshVertex {
public:
  // Public Data
  Vector3f x; // position of the vertex
  Vector3f n; // normal of vertex (mean of normals of touching faces)
  Vector3ub c; // 24-bit color
  bool boundary; // flag to indicate if vertex is on boundary of mesh

  // Public Methods
  void writeTo(std::ostream& os); 
  void readFrom(std::istream& is); 
  void print(std::ostream& os) const;
};


// Restores state from stream (binary)
inline void JDSimpleMeshVertex::readFrom(std::istream& is) {
	is.read(reinterpret_cast<char *>(&x),sizeof(Vector3f));
	is.read(reinterpret_cast<char *>(&n),sizeof(Vector3f));
	is.read(reinterpret_cast<char *>(&c),sizeof(Vector3ub));
   is.read(reinterpret_cast<char *>(&boundary),sizeof(bool));
}


//////////////////////////
// SimpleMeshFace Class //
//////////////////////////
 
class JDSimpleMeshFace {
public:
  // Public Data
  Vector3f n; // normal std::vector of face
  Vector3i indices; 

  void readFrom(std::istream& is, JDSimpleMeshVertex* v0); 
};

// Restores state from stream (binary)
inline void JDSimpleMeshFace::readFrom(std::istream& is, JDSimpleMeshVertex* v0) {
	is.read(reinterpret_cast<char *>(&indices),sizeof(Vector3i));
	is.read(reinterpret_cast<char *>(&n),sizeof(Vector3f));
}



//////////////////////
// SimpleMesh Class //
//////////////////////


class JDSimpleMesh {
 public:
  // Protected Data 
  int nv, nf; // numbers of vertices, faces
  JDSimpleMeshVertex* vd; // global vertex list
  JDSimpleMeshFace* fd; // global face list

  // Constructors/Destructor and Related
  JDSimpleMesh() {vd = NULL; fd = NULL;}
  ~JDSimpleMesh() {delete[] vd; delete[] fd;}

  // File I/O
  void readFrom(std::istream& is); 
};

// I/O: Restores state from stream (binary)
inline void JDSimpleMesh::readFrom(std::istream& is) {
	//read number of vertices and faces from stream
	is.read(reinterpret_cast<char *>(&nv),sizeof(int));
	is.read(reinterpret_cast<char *>(&nf),sizeof(int));

	//allocate memory
	vd = new JDSimpleMeshVertex[nv];
	fd = new JDSimpleMeshFace[nf];

	//read all vertices from stream
	for (int i=0;i<nv;i++) vd[i].readFrom(is);

	//read all faces from stream 
	for (int i=0;i<nf;i++) fd[i].readFrom(is,vd);
}



// // I/O: Writes state to stream (binary)
// inline void JDSimpleMesh::writeTo(std::ostream& os) {
//   //write number of vertices and faces to file
//   os.write(reinterpret_cast<char *>(&nv),sizeof(int));
//   os.write(reinterpret_cast<char *>(&nf),sizeof(int));
//  
//   //write all vertices to file
//   for (int i=0;i<nv;i++) v(i).writeTo(os);
//   
//   //write all faces to file 
//   for (int i=0;i<nf;i++) f(i).writeTo(os,vd);
// }

// // Writes state to stream (binary)
// inline void JDSimpleMeshVertex::writeTo(std::ostream& os) {
//   x.writeTo(os);
//   n.writeTo(os);
//   c.writeTo(os);
//   os.write(reinterpret_cast<char *>(&boundary),sizeof(bool));
// }

// // Writes state to stream (binary)
// inline void JDSimpleMeshFace::writeTo(std::ostream& os, 
// 				       JDSimpleMeshVertex* v0) {
//   Vector3i ind(getVertexIndices(v0));
//   ind.writeTo(os);
//   n.writeTo(os);
// }


// -----------------------------------------------------------------------------------------------


PointCloudImporterReaderBSM::PointCloudImporterReaderBSM(void) {
	mesh = NULL;
	pointIndex = 0;
	faceIndex = 0;

   myVD = new VertexDescriptor();
	vadVertexPosition = VAD("position", 3, VAD::DATA_FORMAT_FLOAT32);
	vadVertexNormal = VAD("normal", 3, VAD::DATA_FORMAT_FLOAT32);
	vadVertexColor = VAD("color", 4, VAD::DATA_FORMAT_CARD8);
   
   myVD->pushAttrib(vadVertexPosition);
   myVD->pushAttrib(vadVertexNormal);
   myVD->pushAttrib(vadVertexColor);

	myTD = new VertexDescriptor();
	vadTriangleIndex = VAD("index", 3, VAD::DATA_FORMAT_INT32);
	vadTriangleNormal = VAD("normal", 3, VAD::DATA_FORMAT_FLOAT32);
   myTD->pushAttrib(vadTriangleIndex);
   myTD->pushAttrib(vadTriangleNormal);

	aatVertexPosition = myVD->getToken("position");
	aatVertexNormal = myVD->getToken("normal");
	aatVertexColor = myVD->getToken("color");
	aatTriangleIndex = myTD->getToken("index");
	aatTriangleNormal = myTD->getToken("index");
   buffer = new card8[myVD->getSize()];
	tribuffer = new card8[myTD->getSize()];;
}

PointCloudImporterReaderBSM::~PointCloudImporterReaderBSM(void) {
	delete mesh;
}


bool PointCloudImporterReaderBSM::beginImport(std::string filename, bool readTrianglesIfPossible) {
	this->readTriangles = readTrianglesIfPossible;
	if (QFile::exists(filename.c_str())) {
		mesh = new JDSimpleMesh();
		std::ifstream is(filename.c_str(), ios_base::in | ios_base::binary);
		mesh->readFrom(is);
		pointIndex = 0;
		faceIndex = 0;
		ImportStarted = true;
		
		return true;
	} else {
		//throw X4Exception("PointCloudImporterReaderBSM::beginImport - file not found.");
		return false;
	}
}

const card8* PointCloudImporterReaderBSM::getNextPoint() {
	if ((int)pointIndex >= mesh->nv) return NULL;
	Vector4d v;
	Vector3f pos = mesh->vd[pointIndex].x;
	v = makeVector4d(pos[0], pos[1], pos[2], 0);
	vadVertexPosition.setAnyAttrib(buffer, v, aatVertexPosition);
	Vector3f normal = mesh->vd[pointIndex].n;
	v = makeVector4d(normal[0], normal[1], normal[2], 0);
	vadVertexNormal.setAnyAttrib(buffer, v, aatVertexNormal);
	Vector3ub c = mesh->vd[pointIndex].c;
	v = makeVector4d(c[0], c[1], c[2], 0);
	vadVertexColor.setAnyAttrib(buffer, v, aatVertexColor);
	
	pointIndex++;

	if (readTriangles) {
		if ((int)pointIndex >= mesh->nv && (int)faceIndex >= mesh->nv) ImportFinished = true;
	} else {
		if ((int)pointIndex >= mesh->nv) ImportFinished = true;
	}
	return buffer;
}

const card8* PointCloudImporterReaderBSM::getNextTriangle() {
	if ((int)faceIndex >= mesh->nv) return NULL;
	
	Vector4d v;
	Vector3i i = mesh->fd[pointIndex].indices;
	v = makeVector4d(i[0], i[1], i[2], 0);
	vadTriangleIndex.setAnyAttrib(tribuffer, v, aatTriangleIndex);

	Vector3f n = mesh->fd[pointIndex].n;
	v = makeVector4d(n[0], n[1], n[2], 0);
	vadTriangleNormal.setAnyAttrib(tribuffer, v, aatTriangleNormal);

	if ((int)pointIndex >= mesh->nv && (int)faceIndex >= mesh->nv) ImportFinished = true;
	faceIndex++;
	return tribuffer;
}

void PointCloudImporterReaderBSM::endImport() {
	delete mesh;
	mesh = NULL;
	ImportStarted = false;
}

mpcard PointCloudImporterReaderBSM::getWidth() {
	return 1;
}

mpcard PointCloudImporterReaderBSM::getHeight() {
	if (mesh == NULL) return 0;
	return mesh->nv;
}

mpcard PointCloudImporterReaderBSM::getProgressPercentage() {
	if (mesh == NULL) return 0;
	if (readTriangles) {
		return (card32)(100.0 * ((double)pointIndex+(double)faceIndex) / ((double)mesh->nv + (double)mesh->nf));
	} else {
		return (card32)(100.0 * ((double)pointIndex) / ((double)mesh->nv));
	}
}

bool PointCloudImporterReaderBSM::supportsTriangleImport() {
	return true;
}
