//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderBinary.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ObjectClassProperty.h"
#include "VertexDescriptor.h"
#include "NumericalClassProperty.h"
#include "Tools.h"
#include "LargeBinaryFile.h"
#include <float.h>

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( PointCloudImporterReaderBinary ,0) {
	BEGIN_CLASS_INIT( PointCloudImporterReaderBinary );
	ADD_CARD32_PROP(headerLength, 0)
}

// ----

PointCloudImporterReaderBinary::PointCloudImporterReaderBinary() {
   headerLength = 0;
   myVD = new VertexDescriptor();
   myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
   myVD->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
   myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_CARD8));
   myVD->pushAttrib(mVAD("sampleSpacing", 1, VAD::DATA_FORMAT_FLOAT32));

   // not actually used
	myTD = new VertexDescriptor();
   myTD->pushAttrib(VAD("index", 3, VAD::DATA_FORMAT_INT32));
   myTD->pushAttrib(VAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));

   f = NULL;
   buffer = NULL;
   size = myVD->getSize();
   fileSize = 0;
   bytesRead = 0;
   posAAT = NULL_AAT;
   checkPos = false;
}

void PointCloudImporterReaderBinary::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

PointCloudImporterReaderBinary::~PointCloudImporterReaderBinary() {}




bool PointCloudImporterReaderBinary::beginImport( std::string filename, bool readTrianglesIfPossible ) 
{
   ImportFinished = false;
   if (FileExists(filename.c_str())) {
      f = new LargeBinaryFile(filename.c_str(), true, false);
      fileSize = f->getFileSize();
      bytesRead = 0;
      if ((fileSize-headerLength) % (card64)size != 0) {
         warning("(file length - header size) is not a multiple of vertex descriptor size");
      }
      f->seek(headerLength);
      card32 size = myVD->getSize();
      buffer = new card8[size];
      if (myVD->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
         posAAT = myVD->getToken("position");
         checkPos = true;
      }
      return true;
   } else {
      return false;
   }
}

const card8* PointCloudImporterReaderBinary::getNextPoint() 
{
   if (f) {
      if (fileSize >= bytesRead + (card64)headerLength + size) {
         f->readBuffer(buffer, size);
         bytesRead += size;
         if (checkPos) {
            Vector3f pos = *((Vector3f*)(buffer+posAAT.getOffset()));
            if (!_finite(pos[0]) || _isnan(pos[0])
            ||  !_finite(pos[1]) || _isnan(pos[1])
            ||  !_finite(pos[2]) || _isnan(pos[2]) ) {
               debugOutput << "invalid position detected at byte position " << (long double)bytesRead << " - import aborted";
               throw(X4Exception("PointCloudImporterReaderBinary::getNextPoint() - invalid position detected (see debug console for details)."));
            }
         }
         return buffer;
      } else {
         ImportFinished = true;
         return NULL;
      }
   } else {
      ImportFinished = true;
      return NULL;
   }
}

const card8* PointCloudImporterReaderBinary::getNextTriangle() 
{
   return NULL;
}


void PointCloudImporterReaderBinary::endImport() 
{
   delete f;
   f = NULL;
   delete[] buffer;
}

mpcard PointCloudImporterReaderBinary::getWidth() 
{
   return 1;
}

mpcard PointCloudImporterReaderBinary::getHeight() 
{
   return 0;
}

mpcard PointCloudImporterReaderBinary::getProgressPercentage() 
{
   return 100.0 * (double)bytesRead / (double)max<card64>(fileSize, 1);
}

bool PointCloudImporterReaderBinary::supportsTriangleImport() 
{
   return false;
}
