//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderCustomASCII_standard.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
#include "ImportTools.h"
#include <sstream>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* PointCloudImporterReaderCustomASCII_standard::PointCloudImporterReaderCustomASCII_standardMetaClass = NULL;

const MetaClass* PointCloudImporterReaderCustomASCII_standard::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterReaderCustomASCII_standard::newInstance() {
   return new PointCloudImporterReaderCustomASCII_standard;
}

const MetaClass* PointCloudImporterReaderCustomASCII_standard::getClass() {
   if (PointCloudImporterReaderCustomASCII_standardMetaClass == NULL) {
      PointCloudImporterReaderCustomASCII_standardMetaClass = new MetaClass("PointCloudImporterReaderCustomASCII_standard", "no description.",
                                       newInstance, sizeof(PointCloudImporterReaderCustomASCII_standard));
   }
   return PointCloudImporterReaderCustomASCII_standardMetaClass;
}

void PointCloudImporterReaderCustomASCII_standard::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   PointCloudImporterReaderCustomASCII_standardMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( PointCloudImporterReaderCustomASCII_standard )
	ADD_CARD32_PROP(HeaderType, 0)
	ADD_CARD32_PROP(PresetType, 0)
	ADD_CARD32_PROP(IgnoreFirstLinesNum, 0)
	ADD_STRING_PROP(Separator,1)
}

void PointCloudImporterReaderCustomASCII_standard::shutdown() {
   delete PointCloudImporterReaderCustomASCII_standardMetaClass;
}

PointCloudImporterReaderCustomASCII_standard::PointCloudImporterReaderCustomASCII_standard() {
	tmpFileName = "";
   progress = 0;

   HeaderType = 0;
   PresetType = 0;
   Separator= " ";
   IgnoreFirstLinesNum = 6;
   CommentMarkers.clear();
	IgnoreColumns.clear();

	myVD = new VertexDescriptor();
	myVD->clear();
	myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_CARD32));
	//myVD->pushAttrib(mVAD("origin", 3, VAD::DATA_FORMAT_FLOAT32));
	//myVD->pushAttrib(mVAD("input_class", 1, VAD::DATA_FORMAT_CARD32));
	//myVD->pushAttrib(mVAD("input_object", 1, VAD::DATA_FORMAT_CARD32));
	//myVD->pushAttrib(mVAD("model", 1, VAD::DATA_FORMAT_CARD32));
	//myVD->pushAttrib(mVAD("detect_class", 1, VAD::DATA_FORMAT_CARD32));
	//myVD->pushAttrib(mVAD("detect_object", 1, VAD::DATA_FORMAT_CARD32));
	//myVD->pushAttrib(mVAD("faceidx", 1, VAD::DATA_FORMAT_CARD32));
}

PointCloudImporterReaderCustomASCII_standard::~PointCloudImporterReaderCustomASCII_standard() {
}

void PointCloudImporterReaderCustomASCII_standard::assign(const Object* obj, X4CopyContext *context) {
	PointCloudImporterReader::assign(obj,context);
   const PointCloudImporterReaderCustomASCII_standard *other = dynamic_cast<const PointCloudImporterReaderCustomASCII_standard*>(obj);
   x4Assert(other != NULL);
   this->HeaderType = other->HeaderType;
   this->IgnoreFirstLinesNum = other->IgnoreFirstLinesNum;
   this->CommentMarkers = other->CommentMarkers;
   this->IgnoreColumns = other->IgnoreColumns;
   this->Separator= other->Separator;
}

void PointCloudImporterReaderCustomASCII_standard::write(OutputObjectStream *s) const {
   PointCloudImporterReader::write(s);
   card32 n = (card32)CommentMarkers.size();
   s->write(n);
   for (card32 i=0; i<n; i++) {
      s->writeString(CommentMarkers[i]);
   };
   n = (card32)IgnoreColumns.size();
   s->write(n);
   for (card32 j=0; j<n; j++) {
      s->write(IgnoreColumns[j]);
   };

   s->writeString(Separator);

}

void PointCloudImporterReaderCustomASCII_standard::read(InputObjectStream *s) {
   PointCloudImporterReader::read(s);
   card32 n;
   s->read(n);
   CommentMarkers.resize(n);
   for (card32 i=0; i<n; i++) {
      s->readString(CommentMarkers[i]);
   }
   s->read(n);
   IgnoreColumns.resize(n);
   for (card32 j=0; j<n; j++) {
      s->read(IgnoreColumns[j]);
   }

	if (s->getClassVersion(PointCloudImporterReaderCustomASCII_standard::getClass()) >= MetaClass::FIRST_VERSION + 1) {
		s->readString(Separator);
	}
		
}


bool PointCloudImporterReaderCustomASCII_standard::beginImport(std::string filnam, bool readTrianglesIfPossible) {

	tmpFileName = filnam;

	// call beginImport just once
   if (ImportStarted) return false;
	x4Assert(buffer == NULL);

	{
		QFileInfo qfi(filnam.c_str());
		fileSize = qfi.size();
	}

	in.clear();
	in.open(filnam.c_str(),std::ifstream::in);
	if (!in.good())
	{ 
		if (isExecutedIntrinsic())
			warning("PointCloudImporterReaderCustomASCII_standard::beginImport()\nError opening file \"" + filnam + "\"!"); 
		else
			WARNINGMESSAGE("PointCloudImporterReaderCustomASCII_standard::beginImport()\nError opening file \"" + filnam + "\"!"); 
		return false;
	};

	//in.seekg(-1, ios::end);
	//fileSize = in.tellg();
	//in.seekg(0, ios::beg);

   // skip first # lines
   for (card32 i=0; i<IgnoreFirstLinesNum; i++) skipToEOL(in);

   IKnowWidth = true;
   IKnowHeight = true;
   std::string forgetMe;

   switch (HeaderType)
   {
      case HT_USERDEFINED:  {
                  if ((width==0) || (height==0)) {
                     IKnowWidth = false;
                     IKnowHeight = false;
                  };
               }
               break;
      case HT_WIDTHHEIGHT:  {
                  in >> width;
                  in >> height;
               }
               break;
      case HT_WIDTHWIDTHHEIGHTHEIGHT:  {
                  in >> forgetMe;
                  in >> width;
                  in >> forgetMe;
                  in >> height;
               }
               break;
		case HT_NUMPOINTS: {
						width = 1;
                  in >> height;
               }
               break;
      default: {
                  IKnowWidth = false;
                  IKnowHeight = false;
               }
               break;
   };

   ImportStarted = false;
   ImportFinished = false;
   progress = 0;

   delete[] buffer;
   buffer = new card8[myVD->getSize()];

	return true;
}


void PointCloudImporterReaderCustomASCII_standard::endImport() {
	progress = 0;
	ImportFinished = false;
	ImportStarted = false;
	if (in.is_open()) in.close();
	delete[] buffer;
	buffer = NULL;
};


mpcard PointCloudImporterReaderCustomASCII_standard::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {

      width = 1;
      height = 2;

		while (!isFinished()) {
			getNextPoint();
			height++;
      };

      height -= 2;
      card32 h = height;

      endImport();
      if (!beginImport(tmpFileName)) throw X4Exception("PointCloudImporterReaderCustomASCII_standard::getWidth() - critical error!!");

      width = 1;
      height = h;
      IKnowWidth = true;
      IKnowHeight = true;

      return width;
   };
}


mpcard PointCloudImporterReaderCustomASCII_standard::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {

      width = 1;
      height = 2;

      while (!isFinished()) {
			getNextPoint();
         height++;
      };

      height -= 2;
      card32 h = height;

      endImport();
      if (!beginImport(tmpFileName)) throw X4Exception("PointCloudImporterReaderCustomASCII_standard::getHeight() - critical error!!");

      width = 1;
      height = h;
      IKnowWidth = true;
      IKnowHeight = true;

      return height;
   };
}


mpcard PointCloudImporterReaderCustomASCII_standard::getProgressPercentage() {
	if (IKnowWidth&&IKnowHeight) return (progress*100)/(width*height);
	// hopefully doesn't crash
	card64 sp = in.tellg();
	return (sp*100)/fileSize;
};

void PointCloudImporterReaderCustomASCII_standard::parseHeader()
{
   if (ImportFinished) return;
   x4Assert(buffer!=NULL);
   x4Assert(in.is_open());

	std::string line;


	VertexDescriptor* vd = new VertexDescriptor;

	while(true)
	{
		bool CLFound = true;
		// take just lines which are not beginning with comment marker
		while (CLFound) {
			getline(in, line);
			line = removeDblSpace(line); // remove multiple spaces between values and make them single space
			line = trimStr(line); // remove beginning and ending spaces from line
		
			if (!strcmp(line.c_str(), "#Data:")) { CLFound = true; break; }

			CLFound = false;
		   for (card32 j=0; j<CommentMarkers.size(); j++) {
				if (line.size()<CommentMarkers[j].size()) continue;
				for (card32 k=0; k<CommentMarkers[j].size(); k++)
				{
					if (line[k]!=(CommentMarkers[j])[k]) { CLFound = false; break; }
					CLFound = true;
				};
			};
		};

		// begin of data, then stop
		if (CLFound && strcmp(line.c_str(), "#Data:") == 0) break;

		// parse each line for the type of the attribute
		char namebuf[128]; memset(namebuf, 0, 128);
		char typebuf[128]; memset(typebuf,0,128);
		card32 num;
		sscanf(line.c_str(), "%s %d %s", namebuf, &num, typebuf);
		if (strlen(namebuf) && strlen(typebuf))
		{
			card32 type;
			if (!strcmp(typebuf, "int8")) type = VAD::DATA_FORMAT_INT8;
			else if (!strcmp(typebuf, "int16")) type = VAD::DATA_FORMAT_INT16;
			else if (!strcmp(typebuf, "int32")) type = VAD::DATA_FORMAT_INT32;
			else if (!strcmp(typebuf, "card8")) type = VAD::DATA_FORMAT_CARD8;
			else if (!strcmp(typebuf, "card16")) type = VAD::DATA_FORMAT_CARD16;
			else if (!strcmp(typebuf, "card32")) type = VAD::DATA_FORMAT_CARD32;
			else if (!strcmp(typebuf, "card64")) type = VAD::DATA_FORMAT_CARD64;
			else if (!strcmp(typebuf, "float32")) type = VAD::DATA_FORMAT_FLOAT32;
			else if (!strcmp(typebuf, "float64")) type = VAD::DATA_FORMAT_FLOAT64;
			else continue;
			vd->pushAttrib(mVAD(namebuf, num, type));
		}
	}

	if (myVD) delete myVD;
	myVD = vd;

   delete[] buffer;
   buffer = new card8[myVD->getSize()];

}

const card8* PointCloudImporterReaderCustomASCII_standard::getNextPoint() {

   if (ImportFinished) return NULL;
   x4Assert(buffer!=NULL);
   x4Assert(in.is_open());

	ImportStarted = true;
	

	card32 const nAttribs = myVD->getNumAttribs();
   if (nAttribs == 0) { ImportFinished = true; return NULL; };

	std::string line;
	bool CLFound = true;
	// take just lines which are not beginning with comment marker
	while (CLFound) {
		getline(in, line);
		line = removeDblSpace(line); // remove multiple spaces between values and make them single space
		line = trimStr(line); // remove beginning and ending spaces from line
		
		CLFound = false;
	   for (card32 j=0; j<CommentMarkers.size(); j++) {
			if (line.size()<CommentMarkers[j].size()) continue;
			for (card32 k=0; k<CommentMarkers[j].size(); k++)
			{
				if (line[k]!=(CommentMarkers[j])[k]) { CLFound = false; break; }
				CLFound = true;
			};
		};
	};

	// ignore specified columns
	card32 columncount = 0;
	std::vector<std::string> columns = splitString(line, Separator);
	line = "";
	for (card32 j=0; j<columns.size(); j++) {
		bool found = false;
		for (card32 k=0; k<IgnoreColumns.size(); k++) {
			if (IgnoreColumns[k]==j) found = true;
		}
		if (!found) {
			line = line + columns[j].c_str() + " ";
			columncount++;
		};
	}

//	line = removeDblSpace(line); // remove multiple spaces between values and make them single space

	// test if line is too short and jump over invalid lines
	card32 nAttrColumns = 0;
	for (card32 i=0; i<nAttribs; i++) {
		VAD at = myVD->getAttribute(i);
		nAttrColumns += at.numComponents;
	};
	if (columncount<nAttrColumns) {
		// jump over lines which line don't fit importer settings
		if (in.eof()) ImportFinished = true;
		return NULL;
	};

	card8 *bufferPos = buffer;
	std::istringstream lineStream;
	lineStream.str(line);

	for (card32 i=0; i<nAttribs; i++) {

      VAD at = myVD->getAttribute(i);

		switch (at.dataFormat)
      {
         case VAD::DATA_FORMAT_CARD32:
            for (card32 i=0; i<at.numComponents; i++)
            {
               card32 value;
               lineStream >> value;
               *((card32*)(bufferPos)) = value;
					bufferPos += sizeof(card32);
            }
            break;
         case VAD::DATA_FORMAT_CARD8:
            for (card32 i=0; i<at.numComponents; i++)
            {
               card8 value;
               lineStream >> value;
               *((card8*)(bufferPos)) = value;
					bufferPos += sizeof(card8);
//               ((card8*)bufferPos)++;
            }
            break;
         case VAD::DATA_FORMAT_FLOAT32:
            for (card32 i=0; i<at.numComponents; i++)
            {
               float32 value;
               lineStream >> value;
               *((float32*)(bufferPos)) = value;
					bufferPos += sizeof(float32);
//               ((float32*)bufferPos)++;
            }
            break;
         case VAD::DATA_FORMAT_FLOAT64:
            for (card32 i=0; i<at.numComponents; i++)
            {
               float64 value;
               lineStream >> value;
               *((float64*)(bufferPos)) = value;
					bufferPos += sizeof(float64);
//               ((float32*)bufferPos)++;
            }
            break;
         case VAD::DATA_FORMAT_INT32:
            for (card32 i=0; i<at.numComponents; i++)
            {
               int32 value;
               lineStream >> value;
               *((int32*)(bufferPos)) = value;
					bufferPos += sizeof(int32);
//               ((int32*)bufferPos)++;
            }
            break;
         default: x4Assert(false); break;
      }  // end switch
   }; // end for nAttribs

   progress++;

	if (IKnowWidth&&IKnowHeight) {
		if (progress >= (width*height)) ImportFinished = true;
	}
   if (in.eof()) ImportFinished = true;

	return buffer;
}


bool PointCloudImporterReaderCustomASCII_standard::supportsTriangleImport() {
	return false;
}


const card8* PointCloudImporterReaderCustomASCII_standard::getNextTriangle() {
	return NULL;
}


