//---------------------------------------------------------------------------
#ifndef PointCloudImporterReaderCustomASCII_standardH
#define PointCloudImporterReaderCustomASCII_standardH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
#include <fstream>
//---------------------------------------------------------------------------

namespace X4 {

/// Import Reader for user definable ASCII file formats, where every single line corresponds to a single point in the data set
class X3EDITOR_API PointCloudImporterReaderCustomASCII_standard : public PointCloudImporterReader {

 private:
   static MetaClass *PointCloudImporterReaderCustomASCII_standardMetaClass;
   static Object* newInstance();

	/// import progress
   card32 progress;
	/// input file stream
   std::ifstream in;
	/// input file size
	card64 fileSize;
	/// index of hardcoded preset, used in custom editor
   card32 PresetType;
	/// temp var to save filename
	std::string tmpFileName;



	// propertys:
	/// type of header in ASCII file, use HeaderEnums
   card32 HeaderType;
	/// number of first lines to ignore in file
   card32 IgnoreFirstLinesNum;
	/// markers for comment-lines, e.g. adding "//" will ignore all lines beginning with "//"
   std::vector<std::string> CommentMarkers;
	/// markers for columns to ignore, e.g. adding "0" and "2" will ignore first and third data column
   std::vector<card32> IgnoreColumns;
   ///columns are separated by
   std::string Separator;

 public:
   PointCloudImporterReaderCustomASCII_standard();
   ~PointCloudImporterReaderCustomASCII_standard();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual bool beginImport(std::string filenam, bool readTrianglesIfPossible = false);
   virtual const card8* getNextPoint();
	virtual const card8* getNextTriangle();
   virtual void endImport();

   virtual mpcard getWidth();
   virtual mpcard getHeight();

	virtual mpcard getProgressPercentage();

	virtual bool supportsTriangleImport();
	void parseHeader();


	/// HeaderType explanation: none / use width and height set by user / width height / "width" width "height" height / numberofpoints
   enum HeaderEnum {HT_NONE, HT_USERDEFINED, HT_WIDTHHEIGHT, HT_WIDTHWIDTHHEIGHTHEIGHT, HT_NUMPOINTS};

	// functions for GUI
	inline void setPresetType(card32 t) { PresetType = t; };
   inline card32 getPresetType() { return PresetType; };
	inline void setHeaderType(HeaderEnum t) { HeaderType = t; };
   inline card32 getHeaderType() { return HeaderType; };
   inline void setHeaderWidth(card32 w) { width = w; };
   inline card32 getHeaderWidth() { return width; };
   inline void setHeaderHeight(card32 h) { height = h; };
   inline card32 getHeaderHeight() { return height; };
   inline void setIgnoreFirstLines(card32 ifl) { IgnoreFirstLinesNum = ifl; };
   inline card32 getIgnoreFirstLines() { return IgnoreFirstLinesNum; };
   inline void setCommentMarkers(std::vector<std::string> cm) { CommentMarkers = cm; };
   inline std::vector<std::string> getCommentMarkers() { return CommentMarkers; };
   inline void setIgnoreColumns(std::vector<card32> ic) { IgnoreColumns = ic; };
   inline std::vector<card32> getIgnoreColumns() { return IgnoreColumns; };
   inline VertexDescriptor* getImportVD() { return myVD; };
   inline std::string getSeparator(){ return Separator; };
   inline void setSeparator(std::string sep) { Separator=sep; };
};

}

#endif
