//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderMSDepthBMP.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* PointCloudImporterReaderMSDepthBMP::PointCloudImporterReaderMSDepthBMPMetaClass = NULL;

const MetaClass* PointCloudImporterReaderMSDepthBMP::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterReaderMSDepthBMP::newInstance() {
   return new PointCloudImporterReaderMSDepthBMP;
}

const MetaClass* PointCloudImporterReaderMSDepthBMP::getClass() {
   if (PointCloudImporterReaderMSDepthBMPMetaClass == NULL) {
      PointCloudImporterReaderMSDepthBMPMetaClass = new MetaClass("PointCloudImporterReaderMSDepthBMP", "no description.",
                                       newInstance, sizeof(PointCloudImporterReaderMSDepthBMP));
   }
   return PointCloudImporterReaderMSDepthBMPMetaClass;
}

void PointCloudImporterReaderMSDepthBMP::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   PointCloudImporterReaderMSDepthBMPMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( PointCloudImporterReaderMSDepthBMP )
	ADD_FLOAT32_PROP(MinZ, 0)
	ADD_FLOAT32_PROP(MaxZ, 0)

	ADD_BOOLEAN_PROP( oldImport, 1 );
	ADD_STRING_PROP( m_SettingsFileName, 1 );
}

void PointCloudImporterReaderMSDepthBMP::shutdown() {
   delete PointCloudImporterReaderMSDepthBMPMetaClass;
}

PointCloudImporterReaderMSDepthBMP::PointCloudImporterReaderMSDepthBMP() {
	

	oldImport = false;

	invWidth = 0.0f;
	invHeight = 0.0f;
   inv255 = 1.0f/255.0f;

	// TODO: change hardcoded values!!
	MinZ = 44.0f;
	MaxZ = 120.0f;
   invMinZ = 1.0f/MinZ;
   invMaxZ = 1.0f/MaxZ;

   myImage = NULL;
   myDepthImage = NULL;
	x = 0;
   y = 0;

	myVD = new VertexDescriptor();
	myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));

	for( int i=0;i<m_NumCameras;i++ )
	{
		m_ColorImage[i] = NULL;
		m_DepthImage[i] = NULL;
	}

	m_SettingsFileName = "";
}

PointCloudImporterReaderMSDepthBMP::~PointCloudImporterReaderMSDepthBMP() {
   delete myDepthImage;
   delete myImage;

	for( int i=0;i<m_NumCameras;i++ )
	{
		delete m_ColorImage[i];
		delete m_DepthImage[i];
	}
}

void PointCloudImporterReaderMSDepthBMP::assign(const Object* obj, X4CopyContext *context) {
   PointCloudImporterReader::assign(obj,context);

   const PointCloudImporterReaderMSDepthBMP *other = dynamic_cast<const PointCloudImporterReaderMSDepthBMP*>(obj);
   x4Assert(other != NULL);
	copyObjectProperties(obj,this);
}


bool PointCloudImporterReaderMSDepthBMP::beginImport(std::string filnam, bool readTrianglesIfPossible) {

	// test if filenames are correct. color-filename has to start with "color", depth-filename has to start with "depth".
	if( oldImport )
	{
	std::string tmpfilnam = removePathFromFilename(filnam);
	if (tmpfilnam.substr(0,5)!="color") {
		WARNINGMESSAGE("PointCloudImporterReaderMSDepthBMP::beginImport()\nFilename has to start with 'color', depth-filename has to start with 'depth'.");
		return false;
	};
	std::string depthfilnam = filnam;
	depthfilnam.replace(filnam.size()-tmpfilnam.size(),5,"depth");
	if (!X4::FileExists(depthfilnam.c_str())) {
		WARNINGMESSAGE("PointCloudImporterReaderMSDepthBMP::beginImport()\nDepth file does not exist. Filename has to start with 'color', depth-filename has to start with 'depth'.");
		return false;
	};

	if ( (!myVD->providesAttribute("position")) || (!myVD->providesAttribute("color")) || (!myVD->providesAttribute("flags")) ) {
		WARNINGMESSAGE("PointCloudImporterReaderMSDepthBMP::beginImport()\nVertex descriptor doesn't have one or more needed attributes (position,color,flags).");
		return false;
	}

	// call beginImport just once
   if (ImportStarted) return false;
	x4Assert(buffer == NULL);

	if ( extractFileExt(strUCase(filnam))!=".BMP" ) {
		// no valid image
		WARNINGMESSAGE("PointCloudImporterReaderMSDepthBMP::beginImport()\nFile extension must be .BMP!");
		return false;
	}

   // load color picture
	delete myImage;
	myImage = new QImage(QImage(QString(filnam.c_str())).convertToFormat(QImage::Format_RGB32));
	if (myImage->isNull()) {
		WARNINGMESSAGE("PointCloudImporterReaderMSDepthBMP::beginImport()\nFile \"" + filnam + "\" is not a valid image.");
		return false;
	};

	// load depth picture
	delete myDepthImage;
	myDepthImage = new QImage(QImage(QString(depthfilnam.c_str())).convertToFormat(QImage::Format_RGB32));
	if (myImage->isNull()) {
		WARNINGMESSAGE("PointCloudImporterReaderMSDepthBMP::beginImport()\nDepth-file \"" + filnam + "\" is not a valid image.");
		return false;
	};

	width = myImage->width();
	IKnowWidth = true;
	height = myImage->height();
	IKnowHeight = true;
	if ((width!=myDepthImage->width()) || (height != myDepthImage->height())) {
		WARNINGMESSAGE("PointCloudImporterReaderMSDepthBMP::beginImport()\nDepth-file resolution does not match Color-file resolution!");
		return false;
	}
	invWidth = 1.0f/((float32)width);
	invHeight = 1.0f/((float32)height);

	x = 0;
	y = 0;

	ImportStarted = false;
	ImportFinished = false;

	invMinZ = 1.0f/MinZ;
	invMaxZ = 1.0f/MaxZ;

	delete[] buffer;
	buffer = new card8[myVD->getSize()];
	for( card32 i=0;i<myVD->getSize();i++ )
		buffer[i] = 0;

	positionOffset = myVD->getAttributeOffset(myVD->getIndex("position"));
	colorOffset = myVD->getAttributeOffset(myVD->getIndex("color"));

	return true;

	}
	else	// !oldimport
	{
		// load settings
		if( !readCalibrationFile( (char*)m_SettingsFileName.c_str() ) )
			return false;
		computeProjectionMatrices();

		// load images, assumes the default ms structure:
		// + data_dir
		//             settingsfile.txt
		//					+ cam0
		//								depth-cam0-f000
		//								..
		//								color-cam0-f000
		//								..
		//					+ cam1
		//								depth-cam1-f000
		//								..
		//								color-cam1-f000
		//								..

		std::string file = removePathFromFilename( filnam );
		std::string path = removeFilenameFromPath( filnam );
		
		// risky :-)
		std::string fileNumber = file.substr( file.length() - 7, 3 );

		for( card32 i=0;i<m_NumCameras;i++ )
		{
			m_ColorImage[i] = new QImage(QString( (path + "../cam" + intToStr(i) + "/color-cam" + intToStr(i) + "-f" + fileNumber + ".bmp").c_str() ) );
			m_DepthImage[i] = new QImage(QString( (path + "../cam" + intToStr(i) + "/depth-cam" + intToStr(i) + "-f" + fileNumber + ".bmp").c_str() ) );
		}
		
		delete[] buffer;
		buffer = new card8[myVD->getSize()];
		for( card32 i=0;i<myVD->getSize();i++ )
			buffer[i] = 0;

		ImportStarted = false;
		ImportFinished = false;
		actCamera = 0;
		x = 0;
		y = 0;

		positionOffset = myVD->getAttributeOffset(myVD->getIndex("position"));
		colorOffset = myVD->getAttributeOffset(myVD->getIndex("color"));

		return true;

	}

}


void PointCloudImporterReaderMSDepthBMP::endImport() {
	x = 0;
	y = 0;
	ImportStarted = false;
	ImportFinished = false;
	delete[] buffer;
	buffer = NULL;
};


mpcard PointCloudImporterReaderMSDepthBMP::getWidth() {
	if( !oldImport )
		return m_Width * m_NumCameras;
	else
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {
      x4Assert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderMSDepthBMP::getHeight() {
	if( !oldImport )
		return m_Height;
	else
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {
      x4Assert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderMSDepthBMP::getProgressPercentage() {
	if( oldImport )
		return ((y*width+x)*100)/(width*height);
	else
		return y * 100 / m_Height;
};


const card8* PointCloudImporterReaderMSDepthBMP::getNextPoint() {

   if (ImportFinished) return NULL;
   x4Assert(buffer!=NULL);
   ImportStarted = true;
 
	if( oldImport )
	{
		// color data
		QRgb pixel = myImage->pixel(x,y);
		Vector3f color = makeVector3f(qRed(pixel),qGreen(pixel),qBlue(pixel))*inv255;

		// position data
		pixel = myDepthImage->pixel(x,y);
		float32 depth = qGray(pixel);

		Vector3f pos;
		pos[0] = x * invWidth;
		pos[1] = y * invWidth;
	// todo: this is not the correct calculation
		float32 abc = ( (depth*inv255)*(invMinZ - invMaxZ) + invMaxZ );
		pos[2] = 44.0f / abc * invWidth;

		StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

		*((StaticVector<float32, 3>*)(buffer + positionOffset)) = pos;
		*((StaticVector<float32, 3>*)(buffer + colorOffset )) = color;
//		*((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;

		x++;
		if (x >= width) {
			x = 0;
			y++;
			if (y >= height) ImportFinished = true;
		};

	}
	else
	{
		// color data
		QRgb pixel;
		pixel = m_ColorImage[actCamera]->pixel(x,y);
		Vector3f color = makeVector3f(qRed(pixel),qGreen(pixel),qBlue(pixel)) / 255.0f;
		//Vector3f color = makeVector3f(1,1,1);

		// position data
		pixel = m_DepthImage[actCamera]->pixel(x,y);
		double depth = DepthLevelToZ( qGray(pixel) );

		double u = (double)x;
		double v = (double)y;

		double worldX, worldY;
		projUVZtoXY( m_CalibParams[actCamera].m_ProjMatrix, u,v, depth, &worldX, &worldY );

		//projXYZtoUV( m_CalibParams[4].m_ProjMatrix, worldX, worldY, depth, &v, &u );

		Vector3f pos;
		pos[0] = worldX;
		pos[1] = worldY;
		pos[2] = depth;

		StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

		*((StaticVector<float32, 3>*)(buffer + positionOffset)) = pos;
		*((StaticVector<float32, 3>*)(buffer + colorOffset )) = color;
//		*((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;

		actCamera++;
		if( actCamera >= m_NumCameras )
		{
			actCamera = 0;
			x++;
			if (x >= m_Width) {
				x = 0;
				y++;
				if (y >= m_Height) ImportFinished = true;
			}
		}

	}

	return buffer;
}


bool PointCloudImporterReaderMSDepthBMP::supportsTriangleImport() {
	return false;
}


const card8* PointCloudImporterReaderMSDepthBMP::getNextTriangle() {
	return NULL;
}

bool PointCloudImporterReaderMSDepthBMP::readCalibrationFile(char *fileName)
{
	int i, j, k;
	FILE *pIn;
	double dIn; // dummy variable
	int camIdx;
	
	if(pIn = fopen(fileName, "r"))
	{
		for(k=0; k<m_NumCameras; k++)
		{
			// camera index
			fscanf(pIn, "%d", &camIdx);

			// camera intrinsics
            for (i=0; i<3; i++)
    			fscanf(pIn, "%lf\t%lf\t%lf", &(m_CalibParams[camIdx].m_K[i][0]),
	    			&(m_CalibParams[camIdx].m_K[i][1]), &(m_CalibParams[camIdx].m_K[i][2]));

			// read barrel distortion params (assume 0)
			fscanf(pIn, "%lf", &dIn);
			fscanf(pIn, "%lf", &dIn);
			
			// read extrinsics
			for(i=0;i<3;i++)
			{
				for(j=0;j<3;j++)
				{
					fscanf(pIn, "%lf", &dIn);
					m_CalibParams[camIdx].m_RotMatrix[i][j] = dIn;
				}
				
				fscanf(pIn, "%lf", &dIn);
				m_CalibParams[camIdx].m_Trans[i] = dIn;
			}

		}
		
		fclose(pIn);
	}
	else
		return false;
	return true;
} // readCalibrationFile

void PointCloudImporterReaderMSDepthBMP::computeProjectionMatrices()
{
	int i, j, k, camIdx;
	double (*inMat)[3];
	double exMat[3][4];

	for(camIdx=0; camIdx<m_NumCameras; camIdx++)
	{
		// The intrinsic matrix
        inMat = m_CalibParams[camIdx].m_K;

		// The extrinsic matrix
		for(i=0;i<3;i++)
		{
			for(j=0;j<3;j++)
			{
				exMat[i][j] = m_CalibParams[camIdx].m_RotMatrix[i][j];
			}
		}

		for(i=0;i<3;i++)
		{
			exMat[i][3] = m_CalibParams[camIdx].m_Trans[i];
		}

		// Multiply the intrinsic matrix by the extrinsic matrix to find our projection matrix
		for(i=0;i<3;i++)
		{
			for(j=0;j<4;j++)
			{
				m_CalibParams[camIdx].m_ProjMatrix[i][j] = 0.0;

				for(k=0;k<3;k++)
				{
					m_CalibParams[camIdx].m_ProjMatrix[i][j] += inMat[i][k]*exMat[k][j];
				}
			}
		}

		m_CalibParams[camIdx].m_ProjMatrix[3][0] = 0.0;
		m_CalibParams[camIdx].m_ProjMatrix[3][1] = 0.0;
		m_CalibParams[camIdx].m_ProjMatrix[3][2] = 0.0;
		m_CalibParams[camIdx].m_ProjMatrix[3][3] = 1.0;
	}
}

// d is pixel value in the depth images. 
// d represents depth, but is NOT the actual depth value.
// This function returns actual z (depth)
double PointCloudImporterReaderMSDepthBMP::DepthLevelToZ( unsigned char d )
{
    double z;
    double MinZ = 44.0, MaxZ = 120.0;

    z = 1.0/((d/255.0)*(1.0/MinZ - 1.0/MaxZ) + 1.0/MaxZ);
    return z;
}

void PointCloudImporterReaderMSDepthBMP::projUVZtoXY(double projMatrix[4][4], double u, double v, double z, double *x, double *y)
{
	double c0, c1, c2;

	// image (0,0) is bottom lefthand corner
	v = (double) m_Height - v - 1.0;

	c0 = z*projMatrix[0][2] + projMatrix[0][3];
	c1 = z*projMatrix[1][2] + projMatrix[1][3];
	c2 = z*projMatrix[2][2] + projMatrix[2][3];

	*y = u*(c1*projMatrix[2][0] - projMatrix[1][0]*c2) +
		v*(c2*projMatrix[0][0] - projMatrix[2][0]*c0) +
		projMatrix[1][0]*c0 - c1*projMatrix[0][0];

	*y /= v*(projMatrix[2][0]*projMatrix[0][1] - projMatrix[2][1]*projMatrix[0][0]) +
		u*(projMatrix[1][0]*projMatrix[2][1] - projMatrix[1][1]*projMatrix[2][0]) +
		projMatrix[0][0]*projMatrix[1][1] - projMatrix[1][0]*projMatrix[0][1];
		
	*x = (*y)*(projMatrix[0][1] - projMatrix[2][1]*u) + c0 - c2*u;
	*x /= projMatrix[2][0]*u - projMatrix[0][0];
} // projUVZtoXY


double PointCloudImporterReaderMSDepthBMP::projXYZtoUV(double projMatrix[4][4], double x, double y, double z, double *u, double *v)
{
	double w;

	*u = projMatrix[0][0]*x + 
		 projMatrix[0][1]*y + 
		 projMatrix[0][2]*z + 
		 projMatrix[0][3];

	*v = projMatrix[1][0]*x + 
		 projMatrix[1][1]*y + 
		 projMatrix[1][2]*z + 
		 projMatrix[1][3];

	w = projMatrix[2][0]*x + 
		projMatrix[2][1]*y + 
		projMatrix[2][2]*z + 
		projMatrix[2][3];

	*u /= w;
	*v /= w;

	// image (0,0) is bottom lefthand corner
	*v = (double) m_Height - *v - 1.0;

	return w;

} // projXYZtoUV

