//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderOFF.h"
//---------------------------------------------------------------------------
#include "ProgressWindow.h"
#include "StringHelper.h"
#include <fstream>
#include <sstream>
#include <QtGui/QImage.h>
#include <QtCore/QFile.h>
#include "DebugOutput.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( PointCloudImporterReaderOFF, 1 )
{
	BEGIN_CLASS_INIT( PointCloudImporterReaderOFF )
	ADD_BOOLEAN_PROP( bVertexIndexStartFromZero, 1);

}

/**
 * Constructor
 */
PointCloudImporterReaderOFF::PointCloudImporterReaderOFF() 
{
	bVertexIndexStartFromZero = true;
	pointProgress = 0;
	triangleProgress = 0;
	pointImportFinished = false;
	triangleImportFinished = false;
	Positions.clear();
	Colors.clear();
	Normals.clear();
	Faces.clear();
	TextureCoords.clear();

	delete myVD;
	myVD = new VertexDescriptor();
	myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
}

/**
 * Destructor                                                                     
 **/
PointCloudImporterReaderOFF::~PointCloudImporterReaderOFF() 
{
}


bool PointCloudImporterReaderOFF::beginImport(std::string filnam, bool readTrianglesIfPossible) 
{
	// call beginImport just once
	if (ImportStarted) return false;
	x4Assert(buffer == NULL);
	x4Assert(tribuffer == NULL);
	pointImportFinished = false;
	triangleImportFinished = false;

	if ( (!myVD->providesAttribute("position")) || (!myVD->providesAttribute("color")) || (!myVD->providesAttribute("flags")) || (!myVD->providesAttribute("normal"))) {
		WARNINGMESSAGE("PointCloudImporterReaderOFF::beginImport()\nVertex descriptor doesn't have one or more needed attributes (position,color,flags,normal).");
		return false;
	}

	/*
	art: In Qt 4.5 seems to be broken ???
	QString qfilename = QString::fromStdString( filnam );
	QFileInfo qFileInfo (qfilename);
	if (!qFileInfo.exists())
	{ 
		WARNINGMESSAGE("PointCloudImporterReaderOFF::beginImport()\nError opening file \"" + filnam + "\"!\n"); 
		return false;
	};*/

	std::string ext = extractFileExt(strUCase(filnam));
	if ( ext.compare( ".OFF" ) != 0 ) {
	//if (qFileInfo.suffix().toLower() != QString("off"))
	//{
		// no .off
		WARNINGMESSAGE("PointCloudImporterReaderOFF::beginImport()\nFile ending has to be .off, is " + ext + ".");
		return false;
	};

	// init triangle descriptor if necessary
	if ( readTrianglesIfPossible ) {
		if (myTD==NULL) myTD = new VertexDescriptor();
		if (!myTD->providesAttribute("index")) myTD->pushAttrib(mVAD("index", 3, VAD::DATA_FORMAT_INT32));
		delete[] tribuffer;
		tribuffer = new card8[myTD->getSize()];
	} else {
		triangleImportFinished = true;
	};

	// clear caches
	Positions.clear();
	Colors.clear();
	Normals.clear();
	Faces.clear();
	TextureCoords.clear();

	// open file
	card64 fileSize = 0;//qFileInfo.size();
	//{
		std::ifstream in;
		in.clear();
		in.open(filnam.c_str(),std::ifstream::in);
		if (!in.good()) { 
			WARNINGMESSAGE("PointCloudImporterReaderOFF::beginImport()\nError opening file \"" + filnam + "\"!"); 
			return false;
		};
		in.seekg(-1, ios::end);
		fileSize = in.tellg();
		in.seekg(0, ios::beg);
		//in.close();
	//}

	// init progress window
	progressWindow->pushStep(true, "loading .off");
	progressWindow->progress(0);

	// Read number of vertices
	card32 numVertices = 0;
	card32 numFaces = 0;
	card32 readState = 0;
	QString line;

	Positions.clear();
	Colors.clear();
	Normals.clear();
	Faces.clear();
	TextureCoords.clear();

	//QFile file( qFileInfo.filePath() );
	//file.open( QIODevice::ReadOnly | QIODevice::Text );
	//QTextStream ts( &file );
	
	//while ( !ts.atEnd() )
	while (!in.eof())
	{
		char _buffer[1024];
		in.getline(_buffer, 1024);
		line = QString::fromLatin1(&_buffer[0]);
		//line = ts.readLine();
		if (line.startsWith("#")) continue;
		if( readState == 0 ){
			// Find OFF identifier
			line = line.toUpper();
			if ( line.contains( "OFF" ) ){
				readState++;
			}
		} else if ( readState == 1 ){
			// Read number of vertices / triangles
			if ( !line.isEmpty() ){
				QStringList strList = line.split( " ", QString::SkipEmptyParts );
				if ( strList.size() >= 2 ){
					readState++;
					numVertices = strList.at( 0 ).toInt();
					numFaces = strList.at( 1 ).toInt();
				}
			}
		} else if ( readState == 2 ){
			// Read vertices
			QStringList strList = line.split( " ", QString::SkipEmptyParts );
			if ( strList.size() == 3 ){
				Vector3f p = makeVector3f( strList.at( 0 ).toFloat(),
										   strList.at( 1 ).toFloat(),
										   strList.at( 2 ).toFloat() );
				Positions.push_back( p );
				Normals.push_back( makeVector3f( 0,0,0 ) );
				Colors.push_back( makeVector3f( 0,0,0 ) );
			}
			if( Positions.size() == numVertices ){
				readState++;
			}
		} else if ( readState == 3 ){
			// Read faces
			QStringList strList = line.split( " ", QString::SkipEmptyParts );
			if ( !line.isEmpty() ){
				// Only support triangle meshes
				if ( strList.at( 0 ).toInt() == 3 || strList.size() == 3)
				{
					int base = 1;
					if (strList.size() == 3) base = 0;

					Vector3i face = makeVector3i( strList.at( base + 0 ).toInt(), 
											  strList.at( base + 1 ).toInt(),
											  strList.at( base + 2 ).toInt());

					if (bVertexIndexStartFromZero == false || strList.size() == 3)
					{
						face[0] = face[0] - 1;
						face[1] = face[1] - 1;
						face[2] = face[2] - 1;
					}

					Faces.push_back( face );
				}
			}
		}
	}
	in.close();

	if ( Positions.size() != numVertices ){
		debugOutput << "Incorrect number of vertices read!\n";
		debugOutput.flush();
	}
	if ( Faces.size() != numFaces ){
		debugOutput << "Incorrect number of faces read!\n";
		debugOutput.flush();
	}

	progressWindow->progress(98);

	width = 1;
	height = Positions.size();
	IKnowWidth = true;
	IKnowHeight = true;
	ImportStarted = false;
	ImportFinished = false;
	pointProgress = 0;
	triangleProgress = 0;

	delete[] buffer;
	buffer = new card8[myVD->getSize()];

	// finish progress window
	progressWindow->popSteps();

   for (size_t i=0; i<Faces.size(); i++) {
      Vector3f v0 = Positions[Faces[i][0]];
      Vector3f v1 = Positions[Faces[i][1]];
      Vector3f v2 = Positions[Faces[i][2]];
      Vector3f normal = (v1-v0).crossProduct(v2-v0);
      Normals[Faces[i][0]] += normal;
      Normals[Faces[i][1]] += normal;
      Normals[Faces[i][2]] += normal;
   }
   for (size_t i=0; i<Normals.size(); i++) {
      float32 n = norm(Normals[i]);
      if (n > 0.001) {
         Normals[i] /= n;
      }
   }
	return true;
}


void PointCloudImporterReaderOFF::endImport() 
{
	pointProgress = 0;
	triangleProgress = 0;
	Positions.clear();
	Colors.clear();
	Normals.clear();
	Faces.clear();
	TextureCoords.clear();
	ImportFinished = false;
	ImportStarted = false;
	pointImportFinished = false;
	triangleImportFinished = false;
	delete[] buffer;
	buffer = NULL;
	delete[] tribuffer;
	tribuffer = NULL;
};


mpcard PointCloudImporterReaderOFF::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {
      x4Assert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderOFF::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {
      x4Assert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderOFF::getProgressPercentage() 
{
	size_t sum = Positions.size() + Faces.size();
	if ( sum > 0 ){
		return ((pointProgress+triangleProgress)*100)/sum;
	} else {
		return 0;
	}
};


const card8* PointCloudImporterReaderOFF::getNextPoint() 
{
	if (pointImportFinished||ImportFinished) return NULL;
	x4Assert(buffer!=NULL);

	ImportStarted = true;
	StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("position")))) = Positions[pointProgress];
	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("normal")))) = Normals[pointProgress];
	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("color")))) = Colors[pointProgress];
	*((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;
	if (myVD->providesAttribute("texcoord")) *((StaticVector<float32, 2>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("texcoord")))) = TextureCoords[pointProgress];

	pointProgress++;
	if (pointProgress >= Positions.size()) pointImportFinished = true;
	if ( pointImportFinished && triangleImportFinished ) ImportFinished = true;
	return buffer;	
}


bool PointCloudImporterReaderOFF::supportsTriangleImport() 
{
	return true;
}

const card8* PointCloudImporterReaderOFF::getNextTriangle() 
{
	if (Faces.empty()) { 
		triangleImportFinished = true;
		if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	};
	if (triangleImportFinished||ImportFinished) return NULL;
	x4Assert(tribuffer!=NULL);

	ImportStarted = true;

	*((StaticVector<int32, 3>*)(tribuffer + myTD->getAttributeOffset(myTD->getIndex("index")))) = Faces[triangleProgress];

	triangleProgress++;
	if (triangleProgress >= Faces.size()) triangleImportFinished = true;
	if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	return tribuffer;
}

void PointCloudImporterReaderOFF::assign(const Object* obj, X4CopyContext *context) 
{
	PointCloudImporterReader::assign(obj,context);
	const PointCloudImporterReaderOFF *other = dynamic_cast<const PointCloudImporterReaderOFF*>(obj);
	x4Assert(other != NULL);

	bVertexIndexStartFromZero = other->bVertexIndexStartFromZero;
}