//---------------------------------------------------------------------------
#ifndef PointCloudImporterReaderOFFH
#define PointCloudImporterReaderOFFH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
//---------------------------------------------------------------------------

namespace X4 {

/// Import Reader for OFF (offset) file format
class X3EDITOR_API PointCloudImporterReaderOFF : public PointCloudImporterReader 
{
	X4_CLASS( PointCloudImporterReaderOFF )

	private:
		/// import point progress
		card32 pointProgress;
		/// import triangle progress
		card32 triangleProgress;
		/// true if point import is finished
		bool pointImportFinished;
		/// true if triangle import is finished
		bool triangleImportFinished;
		/// holds all positions
		std::vector<Vector3f> Positions;
		/// holds all colors
		std::vector<Vector3f> Colors;
		/// holds all normals
		std::vector<Vector3f> Normals;
		/// holds all faces
		std::vector<Vector3i> Faces;
		/// holds all texture coordinates
		std::vector<Vector2f> TextureCoords;
		
		bool bVertexIndexStartFromZero;

	public:
		/// Constructor
		PointCloudImporterReaderOFF();
		/// Destructor
		~PointCloudImporterReaderOFF();

		virtual bool beginImport( std::string filenam, bool readTrianglesIfPossible = false );
		virtual const card8* getNextPoint();
		virtual const card8* getNextTriangle();
		virtual void endImport();
		virtual mpcard getWidth();
		virtual mpcard getHeight();
		virtual mpcard getProgressPercentage();
		virtual bool supportsTriangleImport();

		virtual void assign( const Object* obj, X4CopyContext *context = NULL );

};

}

#endif
