//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderPLY_IQmulus_TerraMobilita.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
#include "SeparatorClassProperty.h"
#include "CopyObjectProperties.h"
#include <sstream>
#include <iomanip>
#include <QtGui/QImage.h>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* PointCloudImporterReaderPLY_IQmulus_TerraMobilita::PointCloudImporterReaderPLY_IQmulus_TerraMobilitaMetaClass = NULL;

const MetaClass* PointCloudImporterReaderPLY_IQmulus_TerraMobilita::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterReaderPLY_IQmulus_TerraMobilita::newInstance() {
	return new PointCloudImporterReaderPLY_IQmulus_TerraMobilita;
}

const MetaClass* PointCloudImporterReaderPLY_IQmulus_TerraMobilita::getClass() {
   if (PointCloudImporterReaderPLY_IQmulus_TerraMobilitaMetaClass == NULL) {
      PointCloudImporterReaderPLY_IQmulus_TerraMobilitaMetaClass = new MetaClass("PointCloudImporterReaderPLY_IQmulus_TerraMobilita", "no description.",
		  newInstance, sizeof(PointCloudImporterReaderPLY_IQmulus_TerraMobilita));
   }
   return PointCloudImporterReaderPLY_IQmulus_TerraMobilitaMetaClass;
}

void PointCloudImporterReaderPLY_IQmulus_TerraMobilita::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 2);
   PointCloudImporterReaderPLY_IQmulus_TerraMobilitaMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT(PointCloudImporterReaderPLY_IQmulus_TerraMobilita);
	ADD_BOOLEAN_PROP(useUnambigousTriangulation,1);
	ADD_BOOLEAN_PROP(relativizeCoordinates,2);
	ADD_STRING_PROP(positionPropertyX,2);
	ADD_STRING_PROP(positionPropertyY,2);
	ADD_STRING_PROP(positionPropertyZ,2);
	ADD_STRING_PROP(positionPropertyOriginX, 2);
	ADD_STRING_PROP(positionPropertyOriginY, 2);
	ADD_STRING_PROP(positionPropertyOriginZ, 2);
	ADD_SEPARATOR("manual normalization");
	ADD_BOOLEAN_PROP(useBaseVertexNormalizeFactor, 2);
	ADD_VECTOR3D_PROP(baseVertexNormalizeFactor, 2);
}

void PointCloudImporterReaderPLY_IQmulus_TerraMobilita::shutdown() {
   delete PointCloudImporterReaderPLY_IQmulus_TerraMobilitaMetaClass;
}

PointCloudImporterReaderPLY_IQmulus_TerraMobilita::PointCloudImporterReaderPLY_IQmulus_TerraMobilita() {
	progress = 0;
	binIn = NULL;
	binReadBuff = new card8[8];
	// asciiIn init?
	Elements.clear();
	PlyImportType = PLY_INVALID;
	actuallyImportedElement = 0;
	SumOfElementCounts = 0;
	pointImportFinished = false;
	triangleImportFinished = false;
	ActualTriangles.clear();
	useUnambigousTriangulation = false;
	relativizeCoordinates = false;
	useBaseVertexNormalizeFactor = false;

	baseVertexNormalizeFactor = makeVector3d(1.0, 1.0, 1.0);
	firstPointVertex = makeVector3d(0,0,0);
	firstPointVertexInitialized = false;

	positionPropertyX = "x";
	positionPropertyY = "y";
	positionPropertyZ = "z";
	positionPropertyOriginX = "x_origin";
	positionPropertyOriginY = "y_origin";
	positionPropertyOriginZ = "z_origin";
}

PointCloudImporterReaderPLY_IQmulus_TerraMobilita::~PointCloudImporterReaderPLY_IQmulus_TerraMobilita() {
	delete[] binReadBuff;
}

void PointCloudImporterReaderPLY_IQmulus_TerraMobilita::assign(const Object* obj, X4CopyContext *context) {
	PointCloudImporterReader::assign(obj,context);
	const PointCloudImporterReaderPLY_IQmulus_TerraMobilita *other = dynamic_cast<const PointCloudImporterReaderPLY_IQmulus_TerraMobilita*>(obj);

   copyObjectProperties(other, this);

   x4Assert(other != NULL);
}


bool PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport(std::string filnam, bool readTrianglesIfPossible) {

	// call beginImport just once
   if (ImportStarted) return false;
	x4Assert(buffer == NULL);
	x4Assert(tribuffer == NULL);

	std::string ext = extractFileExt(strUCase(filnam));
   if (ext!=".PLY") {
		WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nFile ending is not .PLY!");
		return false;
	};

	// clear caches
	Elements.clear();
	ActualTriangles.clear();
	pointImportFinished = false;
	triangleImportFinished = false;
	currentFaceID = 1;

	// init triangle descriptor if neccessary
	if (readTrianglesIfPossible)  {
		if (myTD==NULL) myTD = new VertexDescriptor();
		if (!myTD->providesAttribute("index")) myTD->pushAttrib(mVAD("index", 3, VAD::DATA_FORMAT_INT32));
		delete[] tribuffer;
		tribuffer = new card8[myTD->getSize()];
	} else {
		triangleImportFinished = true;
	};

	// read file header
	std::ifstream in;
	in.clear();
	in.open(filnam.c_str(),std::ifstream::in);
	if (!in.good()) { 
		WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nError opening file \"" + filnam + "\"!"); 
		return false;
	};
	in.seekg(0, ios::beg);

	std::string line; // holds actual read line from file
	getline(in, line);
	if (line!="ply") {
		WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nFile \""+ filnam + "\" is not a valid PLY file!"); 
		in.close();
		return false;
	};
	std::string x;
	bool ascii = false;
	bool bigEndian = false;
	bool littleEndian = false;
	bool needsFirstElement = true;
	element actualElement;
	bool headerFinished = false;
   while (!headerFinished) {
		if (in.eof()) {
			WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nend_header not found!");
			in.close();
			return false;
		};
		getline(in, line);
		line = removeDblSpace(line); // remove multiple spaces between values and make them single space
		line = trimStr(line); // remove beginning and ending spaces from line
		if (line.empty()) continue; // jump over empty lines
		std::istringstream lineStream;
		lineStream.str(line);
		x = ""; // has to be done due to problems if lineStream is empty!
		lineStream >> x;

		if (x=="format") {
			// format descriptor
			lineStream >> x;
			float32 y;
			lineStream >> y;
			if (y!=1.0f) continue; // just ply-version 1.0 is supported
			
			if (x=="ascii") {
				ascii = true;
			} else if (x=="binary_little_endian") {
				littleEndian = true;
			} else if (x=="binary_big_endian") {
				bigEndian = true;
			} else {
				// unsupported format
				WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nUnsupported PLY file format \"" + x + "\"!");
				in.close();
				return false;
			};

		} else if (x=="element") {
			// defines next element
			lineStream >> x;
			card32 y;
			lineStream >> y;
			if (!needsFirstElement) Elements.push_back(actualElement);
			actualElement = element(x,y);
			SumOfElementCounts += y;
			needsFirstElement = false;

		} else if (x=="property") {
			// defines next property of actual element
			if (needsFirstElement) { 
				WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nFile is not a valid PLY file, missing element definer!"); 
				in.close();
				return false;
			};
			lineStream >> x;
			if (x=="list") {
				// list property
				lineStream >> x;
				card32 lct = calcPropertyType(x);
				std::string y;
				lineStream >> y;
				card32 pt = calcPropertyType(y);
				std::string z;
				lineStream >> z;
				actualElement.elementProperties.push_back(elementProperty(pt,z,lct));
			} else {
				// scalar property
				card32 t = calcPropertyType(x);
				std::string propname;
				lineStream >> propname;
				card32 vertexPropType = EVT_undefined;
				if( propname == positionPropertyX ) vertexPropType=EVT_x;
				else if( propname == positionPropertyY ) vertexPropType=EVT_y;
				else if( propname == positionPropertyZ ) vertexPropType=EVT_z;
				else if( propname == "nx" ) vertexPropType=EVT_nx;
				else if( propname == "ny" ) vertexPropType=EVT_ny;
				else if( propname == "nz" ) vertexPropType=EVT_nz;
				else if( propname == "red" ) vertexPropType=EVT_red;
				else if( propname == "green" ) vertexPropType=EVT_green;
				else if( propname == "blue" ) vertexPropType=EVT_blue;
				else if (propname == "reflectance") vertexPropType = EVT_reflectance;
				else if (propname == "id"){ 
					vertexPropType = EVT_input_object; 
					propname = "input_object";
				}
				else if (propname == "class"){ 
					vertexPropType = EVT_input_class; 
					propname = "input_class";
				}
				else if (propname == "x_origin"){
					vertexPropType = EVT_originx;
				}
				else if (propname == "y_origin"){
					vertexPropType = EVT_originy;
				}
				else if (propname == "z_origin"){
					vertexPropType = EVT_originz;
				}
				actualElement.elementProperties.push_back(elementProperty(t,propname,PTINVALID,vertexPropType));
			};

		} else if (x=="end_header") {
			// end of header
			Elements.push_back(actualElement);
			headerFinished = true;
			continue;

		} else if (x=="comment") {
			// skip comment lines
			continue;

		} else {
			// unknown command in header
			//x4Assert(false);
			continue;

		}; // end if x

	}; // end while !in.eof

	// end loading of header
	in.close();

	// analyse header data
	delete myVD;
	myVD = new VertexDescriptor();

	card32 positionCount = 0;
	card32 normalCount = 0;
	card32 colorCount = 0;
	card32 VertexCount = 0;
	card32 FaceCount = 0;
	card32 reflectanceCount = 0;
	card32 input_objectCount = 0;
	card32 input_classCount = 0;
	card32 originCount = 0;
	for (card32 i=0; i<Elements.size(); i++) {

		// make attributes for vertex data
		if (Elements[i].elementName=="vertex") {
			// vertex element
			for (card32 j=0; j<(Elements[i].elementProperties.size()); j++) {
				// all position coordinates?
				std::string propName = Elements[i].elementProperties[j].propertyName;
				if (propName==positionPropertyX) positionCount++;
				else if (propName==positionPropertyY) positionCount++;
				else if (propName==positionPropertyZ) positionCount++;
				// all normal coordinates?
				else if (propName=="nx") normalCount++;
				else if (propName=="ny") normalCount++;
				else if (propName=="nz") normalCount++;
				// all colors?
				else if (propName=="red") colorCount++;
				else if (propName=="green") colorCount++;
				else if (propName=="blue") colorCount++;
				// all refletance?
				else if (propName == "reflectance") reflectanceCount++;
				// all object label?
				else if (propName == "input_object") input_objectCount++;
				// all class label?
				else if (propName == "input_class") input_classCount++;
				else if (propName == positionPropertyOriginX) originCount++;
				else if (propName == positionPropertyOriginY) originCount++;
				else if (propName == positionPropertyOriginZ) originCount++;
				else {
					// maybe automatically make VD-Attribute for other Propertys?
				};
			}
			// adapt VertexDescriptor
			if ((positionCount==3) && (!myVD->providesAttribute("position"))) myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
			if ((normalCount==3) && (!myVD->providesAttribute("normal"))) myVD->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
			if ((colorCount==3) && (!myVD->providesAttribute("color"))) myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
			if ((reflectanceCount == 1) && (!myVD->providesAttribute("reflectance"))) myVD->pushAttrib(mVAD("reflectance", 1, VAD::DATA_FORMAT_FLOAT32));
			if ((input_objectCount == 1) && (!myVD->providesAttribute("input_object"))) myVD->pushAttrib(mVAD("input_object", 1, VAD::DATA_FORMAT_CARD32));
			if ((input_classCount == 1) && (!myVD->providesAttribute("input_class"))) myVD->pushAttrib(mVAD("input_class", 1, VAD::DATA_FORMAT_CARD32));
			if ((originCount == 3) && (!myVD->providesAttribute("origin"))) {
				myVD->pushAttrib(mVAD("origin", 3, VAD::DATA_FORMAT_FLOAT32));
			}
			VertexCount = Elements[i].elementCount;

		} else if (Elements[i].elementName=="face") {
			// face element
			for (card32 j=0; j<(Elements[i].elementProperties.size()); j++) {
				std::string propName = Elements[i].elementProperties[j].propertyName;
				if (propName=="vertex_indices") {
					// if face with valid vertex_indices
					FaceCount = Elements[i].elementCount;
				}
			}

		} else if (Elements[i].elementName=="") {
			// maybe others elements like "face" or "egde"

		}; // end if elementName
	}

	if ((!myVD->providesAttribute("position")) || (VertexCount==0)) {
		WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nFile does not define \"vertex x,y,z\" positions! Special ply-formats are not supportet!");
		return false;
	};
	
	//if (!myVD->providesAttribute("flags")) myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));


	if (ascii) {
		// start ascii import
		PlyImportType = PLY_ASCII;
		in.clear();
		//in.close();
		asciiIn.clear();
		asciiIn.open(filnam.c_str(),std::ifstream::in);
		if (!asciiIn.good()) { 
			WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nError opening ascii file \"" + filnam + "\"!"); 
			asciiIn.close();
			return false;
		};
		asciiIn.seekg(0, ios::beg);
		// jump over header
		line = "";
		while (line!="end_header") {
			if (asciiIn.eof()) {
				WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nend_header not found!");
				asciiIn.close();
				return false;
			};
			getline(asciiIn, line);
			line = removeDblSpace(line);
			line = trimStr(line);
		};

	} else {
		// start binary import
		if (littleEndian) PlyImportType = PLY_BINLittleEndian;
		else if (bigEndian) PlyImportType = PLY_BINBigEndian;
		else {
			WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nUnsupported PLY file format!");
			return false;
		}

	   binIn = fopen(filnam.c_str(), "rb");
		clearerr(binIn);
		if (ferror(binIn)){
			WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nError opening binary file \"" + filnam + "\"!");
			fclose(binIn);
			return false;
		}
		// jump over header
	   char hB[1000];
		hB[0]='x';
		while (!((hB[0]=='e')&&(hB[1]=='n')&&(hB[2]=='d')&&(hB[3]=='_')&&(hB[4]=='h')&&(hB[5]=='e')&&(hB[6]=='a')&&(hB[7]=='d')&&(hB[8]=='e')&&(hB[9]=='r'))) {
			// read stringline
			fgets(hB, 1000, binIn);
			if (feof(binIn)) {
				WARNINGMESSAGE("PointCloudImporterReaderPLY_IQmulus_TerraMobilita::beginImport()\nend_header not found!");
				fclose(binIn);
				return false;
			}
		};
	}; // end start binary import

	width = 1;
	IKnowWidth = true;
	height = VertexCount;
	IKnowHeight = true;

	ImportStarted = false;
	ImportFinished = false;
	progress = 0;

	delete[] buffer;
	buffer = new card8[myVD->getSize()];

	actuallyImportedElement = 0;

	// prepare destination vertex format
	m_VertexProvidesNormal = myVD->providesAttribute("normal");
	m_VertexProvidesColor = myVD->providesAttribute("color");
	m_VertexProvidesOrigin = myVD->providesAttribute("origin");
	m_VertexProvidesreflectance = myVD->providesAttribute("reflectance");
	m_VertexProvidesinput_class = myVD->providesAttribute("input_class");
	m_VertexProvidesinput_object = myVD->providesAttribute("input_object");

	m_VertexPosOffset = myVD->getAttributeOffset(myVD->getIndex("position"));
	if( m_VertexProvidesNormal )
	{
		m_VertexNormalOffset = myVD->getAttributeOffset(myVD->getIndex("normal"));
	}
	if( m_VertexProvidesColor )
	{
		m_VertexColorOffset = myVD->getAttributeOffset(myVD->getIndex("color"));
	}
	if (m_VertexProvidesOrigin)
	{
		m_VertexOriginOffset = myVD->getAttributeOffset(myVD->getIndex("origin"));
	}
	if (m_VertexProvidesreflectance){
		m_VertexReflectanceOffset = myVD->getAttributeOffset(myVD->getIndex("reflectance"));
	}
	if (m_VertexProvidesinput_class)
	{
		m_Vertexinput_objectOffset = myVD->getAttributeOffset(myVD->getIndex("input_object"));
	}
	if (m_VertexProvidesinput_object)
	{
		m_Vertexinput_classOffset = myVD->getAttributeOffset(myVD->getIndex("input_class"));
	}
	return true;
}


void PointCloudImporterReaderPLY_IQmulus_TerraMobilita::endImport() {

	switch (PlyImportType) {
		case PLY_ASCII: asciiIn.close(); break;
		case PLY_BINLittleEndian: fclose(binIn); break;
		case PLY_BINBigEndian: fclose(binIn); break;
		default: break;
	}
	Elements.clear();
	actuallyImportedElement = 0;

	pointImportFinished = false;
	triangleImportFinished = false;
	ActualTriangles.clear();

	progress = 0;
	ImportFinished = false;
	ImportStarted = false;
	delete[] buffer;
	buffer = NULL;
};


mpcard PointCloudImporterReaderPLY_IQmulus_TerraMobilita::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {
      x4Assert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderPLY_IQmulus_TerraMobilita::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {
      x4Assert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderPLY_IQmulus_TerraMobilita::getProgressPercentage() {
	card64 totalCount = 0;
	for (card32 i=0; i<actuallyImportedElement; i++) {
		totalCount += Elements[i].elementCount;
	}
	totalCount += progress;
	return (totalCount*100)/SumOfElementCounts;
};


const card8* PointCloudImporterReaderPLY_IQmulus_TerraMobilita::getNextPoint() {

	if (ImportFinished || pointImportFinished || (!ActualTriangles.empty())) return NULL;
	if (pointImportFinished && triangleImportFinished) { 
		ImportFinished = true;
		return NULL;
	};
   x4Assert(buffer!=NULL);

   ImportStarted = true;

	bool bigEndian = (PlyImportType==PLY_BINBigEndian);
	bool importAscii = (PlyImportType==PLY_ASCII);

	std::string actualElementName = Elements[actuallyImportedElement].elementName;
	bool returnNull = false;

	int8 tmpChar = 0;
	card8 tmpUChar = 0;
	int16 tmpShort = 0;
	card16 tmpUShort = 0;
	int32 tmpInt = 0;
	card32 tmpUInt = 0;
	float32 tmpFloat = 0;
	float64 tmpDouble = 0;

	// handle specific elements
	Vector3f tmpPos;
	Vector3f tmpNormal;
	Vector3f tmpColor;
	float32 tmpReflectance;
	card32 tmpinput_object;
	card32 tmpinput_class;
	Vector3f tmpOri;
	if (actualElementName=="vertex") {
		tmpPos = makeVector3f(0.0f,0.0f,0.0f);
		tmpNormal = makeVector3f(0.0f,0.0f,0.0f);
		tmpColor = makeVector3f(0.0f,1.0f,0.0f);
		tmpOri = makeVector3f(0.0f, 0.0f, 0.0f);
	};
	std::vector<card32> tmpTriIndices;

	// begin reading
	for (card32 k=0; k<(Elements[actuallyImportedElement].elementProperties.size()); k++) {
		// for all properties:
		card32 actualElementPropertyDataType = Elements[actuallyImportedElement].elementProperties[k].propertyDataType;
		std::string actualElementPropertyName = Elements[actuallyImportedElement].elementProperties[k].propertyName;
		card32 actualElementVertexPropType = Elements[actuallyImportedElement].elementProperties[k].propertyVertexType;

		if (Elements[actuallyImportedElement].elementProperties[k].propertyListCountDataType==PTINVALID) {
			// scalar property value

			// read property data from file
			if (importAscii) {
				// import data in ascii mode
				switch (actualElementPropertyDataType) {
					case PTCHAR:	asciiIn >> tmpChar; break;
					case PTUCHAR:	asciiIn >> tmpUChar; break;
					case PTSHORT:	asciiIn >> tmpShort; break;
					case PTUSHORT:	asciiIn >> tmpUShort; break;
					case PTINT:		asciiIn >> tmpInt; break;
					case PTUINT:	asciiIn >> tmpUInt; break;
					case PTFLOAT:	asciiIn >> tmpFloat; break;
					case PTDOUBLE:	asciiIn >> tmpDouble; break;
					default: x4Assert(false); continue;
				};

			} else {
				// import data in binary mode
				switch (actualElementPropertyDataType) {
					case PTCHAR:	tmpChar = readChar(); break;
					case PTUCHAR:	tmpUChar = readUChar(); break;
					case PTSHORT:	tmpShort = readShort(bigEndian); break;
					case PTUSHORT:	tmpUShort = readUShort(bigEndian); break;
					case PTINT:		tmpInt = readInt(bigEndian); break;
					case PTUINT:	tmpUInt = readUInt(bigEndian); break;
					case PTFLOAT:	tmpFloat = readFloat(bigEndian); break;
					case PTDOUBLE:	tmpDouble = readDouble(bigEndian); break;
					default: x4Assert(false); continue;
				};
			}

			// std::map data to attributes
			if (actualElementName=="vertex") { 
				// mapping of vertex element data

// 				if ( (actualElementPropertyName=="x")||(actualElementPropertyName=="y")||(actualElementPropertyName=="z")
// 					||(actualElementPropertyName=="nx")||(actualElementPropertyName=="ny")||(actualElementPropertyName=="nz")
// 					||(actualElementPropertyName=="red")||(actualElementPropertyName=="green")||(actualElementPropertyName=="blue")) 
				if( actualElementVertexPropType!=EVT_undefined )
				{

					// if position, normal or color data

					// convert everything to double first, in order for easy normalization if required
					switch (actualElementPropertyDataType) {
						case PTCHAR:	tmpDouble = (float64)tmpChar / 255.0; break;
						case PTUCHAR:	tmpDouble = (float64)tmpUChar / 255.0; break;
						case PTSHORT:	tmpDouble = (float64)tmpShort; break;
						case PTUSHORT:	tmpDouble = (float64)tmpUShort; break;
						case PTINT:		tmpDouble = (float64)tmpInt; break;
						case PTUINT:	tmpDouble = (float64)tmpUInt; break;
						case PTFLOAT:	tmpDouble = (float64)tmpFloat; break;
						case PTDOUBLE:	break;
						default: x4Assert(false); continue;
					};

					// we use first point as our base normalization
					if (relativizeCoordinates)
					{
						if (progress == 0 && !firstPointVertexInitialized)
						{
							std::stringstream conv;
							conv << std::setprecision(20); // No rounding please!
							conv << tmpDouble;
							switch(actualElementVertexPropType)
							{
							case EVT_x:	firstPointVertex[0] = tmpDouble;	debugOutput << "relative to x -> " << conv.str() << "\n"; break;
							case EVT_y:	firstPointVertex[1] = tmpDouble;	debugOutput << "relative to y -> " << conv.str() << "\n"; break;
							case EVT_z:	firstPointVertex[2] = tmpDouble;	debugOutput << "relative to z -> " << conv.str() << "\n"; break;
							default: break;
							}
						}
						else if (progress == 1)
						{
							// After having progressed the first vertex, all three components
							// of firstPointVertex have definitely been initialized.
							// If we process a second point cloud later on, we will
							// not do so again (thereby keeping relative coordinates
							// consistent).
							firstPointVertexInitialized = true;
						}

						switch(actualElementVertexPropType)
						{
						case EVT_x:	tmpDouble -= firstPointVertex[0];	break;
						case EVT_y:	tmpDouble -= firstPointVertex[1];	break;
						case EVT_z:	tmpDouble -= firstPointVertex[2];	break;
						default: break;
						}

					}

					if (useBaseVertexNormalizeFactor)
					{
						// perform normalization
						switch(actualElementVertexPropType)
						{
						case EVT_x:	tmpDouble *= baseVertexNormalizeFactor[0];	break;
						case EVT_y:	tmpDouble *= baseVertexNormalizeFactor[1];	break;
						case EVT_z:	tmpDouble *= baseVertexNormalizeFactor[2];	break;
						default: break;
						}

					}


					// perform simple casting, since here we assume the numbers were normalized as required
					tmpFloat = (float32)tmpDouble;

					switch(actualElementVertexPropType)
					{
					case EVT_x:	tmpPos[0] = tmpFloat;	break;
					case EVT_y:	tmpPos[1] = tmpFloat;	break;
					case EVT_z:	tmpPos[2] = tmpFloat;	break;
					case EVT_nx:	tmpNormal[0] = tmpFloat;	break;
					case EVT_ny:	tmpNormal[1] = tmpFloat;	break;
					case EVT_nz:	tmpNormal[2] = tmpFloat;	break;
					case EVT_red:	tmpColor[0] = tmpFloat;	break;
					case EVT_green:	tmpColor[1] = tmpFloat;	break;
					case EVT_blue:	tmpColor[2] = tmpFloat;	break;
					case EVT_reflectance: tmpReflectance = tmpFloat; break;
					case EVT_input_object: tmpinput_object = tmpUInt; break;
					case EVT_input_class: tmpinput_class = tmpUInt; break;
					case EVT_originx:	tmpOri[0] = tmpFloat;	break;
					case EVT_originy:	tmpOri[1] = tmpFloat;	break;
					case EVT_originz:	tmpOri[2] = tmpFloat;	break;
					}
				
				} else {
					// maybe automatically make VD-Attribute for unknown Properties?

				}; // property name switch


			} else if (actualElementName=="") { 
				// place for mapping of other element data
				returnNull = true;

			} else { 
				// unknown elements
				// maybe automatically std::map VD-Attribute for unknown Elements&Propertys?
				returnNull = true;

			}; // end of std::map data to attributes


		} else {
			// list property value

			// read property components count
			card32 compcount = 0;
			if (importAscii) {
				// import data in ascii mode
				switch (Elements[actuallyImportedElement].elementProperties[k].propertyListCountDataType) {
					case PTCHAR:	asciiIn >> tmpChar; compcount = (card32)tmpChar; break;
					case PTUCHAR:	asciiIn >> tmpUChar; compcount = (card32)tmpUChar; break;
					case PTSHORT:	asciiIn >> tmpShort; compcount = (card32)tmpShort; break;
					case PTUSHORT:	asciiIn >> tmpUShort; compcount = (card32)tmpUShort; break;
					case PTINT:		asciiIn >> tmpInt; compcount = (card32)tmpInt; break;
					case PTUINT:	asciiIn >> tmpUInt; compcount = tmpUInt; break;
					case PTFLOAT:	asciiIn >> tmpFloat; compcount = (card32)tmpFloat; break;
					case PTDOUBLE:	asciiIn >> tmpDouble; compcount = (card32)tmpDouble; break;
					default: x4Assert(false); continue;
				};

			} else {
				// import data in binary mode
				switch (Elements[actuallyImportedElement].elementProperties[k].propertyListCountDataType) {
					case PTCHAR:	tmpChar = readChar(); compcount = (card32)tmpChar; break;
					case PTUCHAR:	tmpUChar = readUChar(); compcount = (card32)tmpUChar; break;
					case PTSHORT:	tmpShort = readShort(bigEndian); compcount = (card32)tmpShort; break;
					case PTUSHORT:	tmpUShort = readUShort(bigEndian); compcount = (card32)tmpUShort; break;
					case PTINT:		tmpInt = readInt(bigEndian); compcount = (card32)tmpInt; break;
					case PTUINT:	tmpUInt = readUInt(bigEndian); compcount = tmpUInt; break;
					case PTFLOAT:	tmpFloat = readFloat(bigEndian); compcount = (card32)tmpFloat; break;
					case PTDOUBLE:	tmpDouble = readDouble(bigEndian); compcount = (card32)tmpDouble; break;
					default: x4Assert(false); continue;
				};
			}

			// read list property data from file
			int32 rootTriIndex = -1; // used for face data
			for (card32 j=0; j<compcount ;j++) {

				if (importAscii) {
					// import data in ascii mode
					switch (actualElementPropertyDataType) {
						case PTCHAR:	asciiIn >> tmpChar; break;
						case PTUCHAR:	asciiIn >> tmpUChar; break;
						case PTSHORT:	asciiIn >> tmpShort; break;
						case PTUSHORT:	asciiIn >> tmpUShort; break;
						case PTINT:		asciiIn >> tmpInt; break;
						case PTUINT:	asciiIn >> tmpUInt; break;
						case PTFLOAT:	asciiIn >> tmpFloat; break;
						case PTDOUBLE:	asciiIn >> tmpDouble; break;
						default: x4Assert(false); continue;
					};

				} else {
					// import data in binary mode
					switch (actualElementPropertyDataType) {
						case PTCHAR:	tmpChar = readChar(); break;
						case PTUCHAR:	tmpUChar = readUChar(); break;
						case PTSHORT:	tmpShort = readShort(bigEndian); break;
						case PTUSHORT:	tmpUShort = readUShort(bigEndian); break;
						case PTINT:		tmpInt = readInt(bigEndian); break;
						case PTUINT:	tmpUInt = readUInt(bigEndian); break;
						case PTFLOAT:	tmpFloat = readFloat(bigEndian); break;
						case PTDOUBLE:	tmpDouble = readDouble(bigEndian); break;
						default: x4Assert(false); continue;
					};
				}

				// std::map list data to attributes
				if (actualElementName=="face") { 
					// mapping of vertex element data

					if (actualElementPropertyName=="vertex_indices") {
						// if valid triangle index

						switch (actualElementPropertyDataType) {
							case PTCHAR:	tmpUInt = (card32)tmpChar; break;
							case PTUCHAR:	tmpUInt = (card32)tmpUChar; break;
							case PTSHORT:	tmpUInt = (card32)tmpShort; break;
							case PTUSHORT:	tmpUInt = (card32)tmpUShort; break;
							case PTINT:		tmpUInt = (card32)tmpInt; break;
							case PTUINT:	break;
							case PTFLOAT:	tmpUInt = (card32)tmpFloat; break;
							case PTDOUBLE:	tmpUInt = (card32)tmpDouble; break;
							default: x4Assert(false); continue;
						};
						tmpTriIndices.push_back(tmpUInt);

					} else {
						// maybe automatically make VD-Attribute for unknown Propertys?

					}; // end of property name switch

				} else if (actualElementName=="") { 
					// place for mapping of other element data
					returnNull = true;

				} else { 
					// unknown elements
					// maybe automatically std::map VD-Attribute for unknown Elements&Propertys?
					returnNull = true;

				}; // end of std::map data to list attributes

			}; // end for compcount


		}; // end of read nolist/list property

	}; // end for all properties


	// finally handle specific elements
	if (actualElementName=="vertex") {
		// vertex: write found data to buffer
/*		*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("position")))) = tmpPos;
		if (myVD->providesAttribute("normal"))
			*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("normal")))) = tmpNormal;
		if (myVD->providesAttribute("color"))
			*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("color")))) = tmpColor;
		StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;
		*((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;*/
		// mb - somewhat faster ...
		*((Vector3f*)(buffer + m_VertexPosOffset)) = tmpPos;
		if( m_VertexProvidesNormal )
			*((Vector3f*)(buffer + m_VertexNormalOffset)) = tmpNormal;
		if( m_VertexProvidesColor )
			*((Vector3f*)(buffer + m_VertexColorOffset)) = tmpColor;
		if (m_VertexProvidesOrigin)
			*((Vector3f*)(buffer + m_VertexOriginOffset)) = tmpOri;
		//*((int32*)(buffer + m_VertexFlagsOffset)) = 0;
		if (m_VertexProvidesreflectance)
		*((float32*)(buffer + m_VertexReflectanceOffset)) = tmpReflectance;
		if (m_VertexProvidesinput_object)
		*((card32*)(buffer + m_Vertexinput_objectOffset)) = tmpinput_object;
		if (m_VertexProvidesinput_class)
		*((card32*)(buffer + m_Vertexinput_classOffset)) = tmpinput_class;

	} else if ((actualElementName=="face") && !triangleImportFinished) {
		// face: add to current triangle list and return null point
      if (tmpTriIndices.size() >= 3) 
		{
			currentFaceID++;
			if( tmpTriIndices.size() > 3 )
				debugOutput << ":";
			if( !useUnambigousTriangulation || tmpTriIndices.size() == 3)
			{
				for (card32 i=0; i<=tmpTriIndices.size()-3; i++) 
				{
					ActualTriangles.push_back(makeVector3i(tmpTriIndices[0], tmpTriIndices[1+i], tmpTriIndices[2+i]));
				}
			}
			else
			{
				// mb - triangulate the polygon by adding a new point to the pointcloud (center of mass).
				// For every polygonedge a triangle is created to this new point.
				// Due to the design of the importer, the new point is added outside.
				// A negative vertex index indicates such a new point.
				debugOutput << ".";
				for (card32 i=0; i<tmpTriIndices.size(); i++) 
				{
					ActualTriangles.push_back(makeVector3i(tmpTriIndices[i], tmpTriIndices[(i+1)%tmpTriIndices.size()], -currentFaceID));
				}

			}
      }
		returnNull = true;

	} else {
		// not used till yet
		returnNull = true;
	};

	progress++;
	if (progress >= Elements[actuallyImportedElement].elementCount) {
		actuallyImportedElement++;
		progress = 0;
	}

	if (actuallyImportedElement>=Elements.size()) pointImportFinished = true;
	if (importAscii) {
		if (asciiIn.eof()) pointImportFinished = true;
	} else {
		if (feof(binIn)) pointImportFinished = true;
	}

	if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	if (returnNull) return NULL;
	else return buffer;

}


bool PointCloudImporterReaderPLY_IQmulus_TerraMobilita::supportsTriangleImport() {
	return true;
}


const card8* PointCloudImporterReaderPLY_IQmulus_TerraMobilita::getNextTriangle() {
	if (ActualTriangles.empty()) {
		if (pointImportFinished) { 
			triangleImportFinished = true;
			ImportFinished = true;
		} else {
			return NULL;
		}
	};
   if (triangleImportFinished||ImportFinished) return NULL;
   x4Assert(tribuffer!=NULL);

   ImportStarted = true;

   *((StaticVector<int32, 3>*)(tribuffer + myTD->getAttributeOffset(myTD->getIndex("index")))) = ActualTriangles[ActualTriangles.size()-1];
	ActualTriangles.pop_back();

	if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	return tribuffer;
}


const card32 PointCloudImporterReaderPLY_IQmulus_TerraMobilita::calcPropertyType(std::string s) {
	if (s=="char") return PTCHAR;
		else if (s=="uchar") return PTUCHAR; 
		else if (s=="uint8") return PTUCHAR; 
		else if (s=="short") return PTSHORT;
		else if (s=="ushort") return PTUSHORT;
		else if (s=="int") return PTINT;
		else if (s=="int32") return PTINT;
		else if (s=="uint") return PTUINT;
		else if (s=="float") return PTFLOAT;
		else if (s=="float32") return PTFLOAT;
		else if (s=="double") return PTDOUBLE;
		else if (s=="float64") return PTDOUBLE;
		else return PTINVALID;
};
