//---------------------------------------------------------------------------
#ifndef PointCloudImporterReaderPLY_Paris_rue_MadameH
#define PointCloudImporterReaderPLY_Paris_rue_MadameH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
#include <stdio.h>
#include <fstream>
//---------------------------------------------------------------------------

namespace X4 {

/// Import Reader for PLY files
/// maps (element:vertex, propertys:x,y,z) to position attribute, (element:vertex, propertys:nx,ny,nz) to normal attribute and (element:vertex, propertys:red,green,blue) to color attribute
/// maps (element:face, property: vertex_indices) to index attribute of triangle mesh triangle-PointSet
/// notice: endian swap and other things not tested!
	class X3EDITOR_API PointCloudImporterReaderPLY_Paris_rue_Madame : public PointCloudImporterReader {


	private:
	static MetaClass *PointCloudImporterReaderPLY_Paris_rue_MadameMetaClass;
	static Object* newInstance();

	enum EVertexType
	{
		EVT_undefined = 0,
		EVT_x = 1,
		EVT_y = 2,
		EVT_z = 3,
		EVT_nx = 4,
		EVT_ny = 5,
		EVT_nz = 6,
		EVT_red = 7,
		EVT_green = 8,
		EVT_blue = 9,
		EVT_reflectance = 10,
		EVT_input_object = 11,
		EVT_input_class = 12,
		EVT_originx = 13,
		EVT_originy = 14,
		EVT_originz = 15
	};

	/// defines an element property of ply file. See ply file specification for details.
	struct elementProperty {
		/// data type of property, see propertyTypeEnum
		card32 propertyDataType;
		/// name of property
		std::string propertyName;
		/// predefined vertex property (EVertexType)
		card32 propertyVertexType;
		/// data type of list component counter, PTINVALID if not a list property, see propertyTypeEnum
		card32 propertyListCountDataType;
		elementProperty(card32 type, std::string name, card32 listcounttype=PTINVALID, card32 vertexType=EVT_undefined) 
		{propertyDataType = type; propertyName = name; propertyListCountDataType = listcounttype;propertyVertexType=vertexType;}
	};

	/// defines an element of ply file. See ply file specification for details.
	struct element {
		/// name of element
		std::string elementName;
		/// number of element "points" in file
		card64 elementCount;
		/// properties of this element, see elementProperty
		std::vector<elementProperty> elementProperties;
		element(std::string name, card64 count) {elementName = name; elementCount = count; elementProperties.clear();}
		element() {elementName = "noName"; elementCount = 0; elementProperties.clear();}
	};

	enum propertyTypeEnum {PTINVALID=0,PTCHAR=1,PTUCHAR=2,PTSHORT=3,PTUSHORT=4,PTINT=5,PTUINT=6,PTFLOAT=7,PTDOUBLE=8};
	enum plyTypeEnum {PLY_INVALID=0, PLY_ASCII=1, PLY_BINLittleEndian=2, PLY_BINBigEndian=3};

	// properties
	bool useUnambigousTriangulation;		// adds a new vertex per polygon (center of mass)
	bool relativizeCoordinates;				// should be coordinates made relative to the first one 
	bool useBaseVertexNormalizeFactor;		// use normalize factor passed in the given std::vector

	Vector3d baseVertexNormalizeFactor;
	Vector3d firstPointVertex;
	bool firstPointVertexInitialized;		// Once this is set, firstPointVertex will not be
											// initialized again. It will not be reset when
											// beginImport() is called, but is preserved
											// over multiple imports as long as the
											// reader instance exists.
	
	std::string positionPropertyX;
	std::string positionPropertyY;
	std::string positionPropertyZ;
	std::string positionPropertyOriginX;
	std::string positionPropertyOriginY;
	std::string positionPropertyOriginZ;

	// user variables:
	int currentFaceID;

	/// import progress
	card32 progress;
	/// file used for binary import
	FILE* binIn;
	/// file used for ascii import
	std::ifstream asciiIn;
	/// buffer for reading binary data
	card8* binReadBuff;
	/// holds the data Elements read from file header
	std::vector<element> Elements;
	/// holds the ply-type which is detected in beginImport()
	card32 PlyImportType;
	/// holds the actual element block which is imported by nextPoint()
	card32 actuallyImportedElement;
	/// holds total sum of element counters, needed for progress percentage
	card64 SumOfElementCounts;
	/// true if point import is finished
	bool pointImportFinished;
	/// true if triangle import is finished
	bool triangleImportFinished;
	/// holds actually found triangles, used for mesh import
	std::vector<Vector3i> ActualTriangles;

	// mb - remove some really really unnecessary bottlenecks (but its just the tip of the iceberg)
	bool m_VertexProvidesNormal;
	bool m_VertexProvidesColor;
	bool m_VertexProvidesOrigin;
	bool m_VertexProvidesreflectance;
	bool m_VertexProvidesinput_class;
	bool m_VertexProvidesinput_object;
	mpcard m_VertexPosOffset; 
	mpcard m_VertexNormalOffset;
	mpcard m_VertexColorOffset;
	mpcard m_VertexFlagsOffset;
	mpcard m_VertexReflectanceOffset;
	mpcard m_Vertexinput_objectOffset;
	mpcard m_Vertexinput_classOffset;
	mpcard m_VertexOriginOffset;

	// user functions:
	/// reads 8bit char from binIn
	inline int8 readChar() {
		fread(binReadBuff, 1, 1, binIn);
		int8* tmp = (int8*)binReadBuff;
		return *tmp;
	}
	/// reads 8bit unsigned char from binIn
	inline card8 readUChar() {
		fread(binReadBuff, 1, 1, binIn);
		card8* tmp = (card8*)binReadBuff;
		return *tmp;
	}
	/// reads 16bit short from binIn
	inline int16 readShort(bool flip) {
		fread(binReadBuff, 2, 1, binIn);
		int16* tmp = (int16*)binReadBuff;
		if (flip) *tmp = (((*tmp>>8)&0xff) | ((*tmp&0xff)<<8)); // endian swap for 2 bytes
		return *tmp;
	}
	/// reads 16bit unsigned short from binIn
	inline card16 readUShort(bool flip) {
		fread(binReadBuff, 2, 1, binIn);
		card16* tmp = (card16*)binReadBuff;
		if (flip) *tmp = (((*tmp>>8)&0xff) | ((*tmp&0xff)<<8)); // endian swap for 2 bytes
		return *tmp;
	}
	/// reads 32bit int from binIn
	inline int32 readInt(bool flip) {
		fread(binReadBuff, 4, 1, binIn);
		int32* tmp = (int32*)binReadBuff;
		if (flip) *tmp = (((*tmp>>24)&0xff) | ((*tmp&0xff)<<24) | ((*tmp>>8)&0xff00) | ((*tmp&0xff00)<<8)); // endian swap for 4 bytes
		return *tmp;
	}
	/// reads 32bit unsigned int from binIn
	inline card32 readUInt(bool flip) {
		fread(binReadBuff, 4, 1, binIn);
		card32* tmp = (card32*)binReadBuff;
		if (flip) *tmp = (((*tmp>>24)&0xff) | ((*tmp&0xff)<<24) | ((*tmp>>8)&0xff00) | ((*tmp&0xff00)<<8)); // endian swap for 4 bytes
		return *tmp;
	}
	/// reads 32bit float from binIn
	inline float32 readFloat(bool flip) {
		fread(binReadBuff, 4, 1, binIn);
		float32* tmp = (float32*)binReadBuff;
		if (flip) {
			int32 *tmp2 = (int32*)tmp;
			*tmp2 = (((*tmp2>>24)&0xff) | ((*tmp2&0xff)<<24) |((*tmp2>>8)&0xff00) | ((*tmp2&0xff00)<<8)); // endian swap for 4 bytes
		};
		return *tmp;
	}
	/// reads 64bit double from binIn
	inline float64 readDouble(bool flip) {
		fread(binReadBuff, 8, 1, binIn);
		float64* tmp = (float64*)binReadBuff;
		if (flip) {	
			// endian swap for 8 bytes
			card8 *tmp2 = (card8*)tmp;
			card8 tmp3;
			tmp3 = *tmp2;
			*tmp2 = *(tmp2+7);
			*(tmp2+7) = tmp3;
			tmp3 = *(tmp2+1);
			*(tmp2+1) = *(tmp2+6);
			*(tmp2+6) = tmp3;
			tmp3 = *(tmp2+2);
			*(tmp2+2) = *(tmp2+5);
			*(tmp2+5) = tmp3;
			tmp3 = *(tmp2+3);
			*(tmp2+3) = *(tmp2+4);
			*(tmp2+4) = tmp3;
		};
		return *tmp;
	}

	/// returns propertyTypeEnum for given property type std::string
	const card32 calcPropertyType(std::string s);



 public:
	 PointCloudImporterReaderPLY_Paris_rue_Madame();
	 ~PointCloudImporterReaderPLY_Paris_rue_Madame();

	virtual const MetaClass *getInstanceClass() const;
	static const MetaClass *getClass();
	static void init(const MetaClass *parentClass);
	static void shutdown();

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual bool beginImport(std::string filenam, bool readTrianglesIfPossible = false);
	virtual const card8* getNextPoint();
	virtual const card8* getNextTriangle();
	virtual void endImport();

	virtual mpcard getWidth();
	virtual mpcard getHeight();

	virtual mpcard getProgressPercentage();

	virtual bool supportsTriangleImport();

};

}

#endif
