//---------------------------------------------------------------------------
#ifndef PointCloudImporterReaderSMFOBJH
#define PointCloudImporterReaderSMFOBJH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
//---------------------------------------------------------------------------

namespace X4 {

/// Import Reader for SimpleModelFormat(SMF) or OBJ files
/// smf: should work with most basic smf-files but doesn't support textures, transformations or grouping
/// obj: should work with simple obj files but doesn't support many features
/// notice: some missing things in vertex normal and vertex color calculation, todo
class X3EDITOR_API PointCloudImporterReaderSMFOBJ : public PointCloudImporterReader {

 private:
   static MetaClass *PointCloudImporterReaderSMFOBJMetaClass;
   static Object* newInstance();

	/// import point progress
   card32 pointProgress;
	/// import triangle progress
   card32 triangleProgress;
	/// true if point import is finished
	bool pointImportFinished;
	/// true if triangle import is finished
	bool triangleImportFinished;
	/// holds all positions
   std::vector<Vector3f> Positions;
	/// holds all colors
   std::vector<Vector3f> Colors;
	/// holds all normals
   std::vector<Vector3f> Normals;
	/// holds all faces
   std::vector<Vector3i> Faces;
   /// polygon ids of face.
   std::vector<card32>	m_FacePolyID;
	/// holds all texture coordinates
   std::vector<Vector2f> TextureCoords;


 public:
   PointCloudImporterReaderSMFOBJ();
   ~PointCloudImporterReaderSMFOBJ();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual bool beginImport(std::string filenam, bool readTrianglesIfPossible = false);
   virtual const card8* getNextPoint();
	virtual const card8* getNextTriangle();
	virtual mpint getCurrentPolygonIndex();
   virtual void endImport();

   virtual mpcard getWidth();
   virtual mpcard getHeight();

	virtual mpcard getProgressPercentage();

	virtual bool supportsTriangleImport();

};

}

#endif
