//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderVelodyne.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ObjectClassProperty.h"
#include "VertexDescriptor.h"
#include "NumericalClassProperty.h"
#include "Tools.h"
#include "LargeBinaryFile.h"
#include <float.h>
#include <iomanip>
#include <iosfwd>
#include <sstream>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( PointCloudImporterReaderVelodyne ,0) {
	BEGIN_CLASS_INIT( PointCloudImporterReaderVelodyne );
	ADD_CARD32_PROP(headerLength, 0)
	ADD_CARD32_PROP(pointSize, 0)
	ADD_CARD32_PROP(positionOffset, 0)
	ADD_BOOLEAN_PROP(positionIsFloat64, 0);
	ADD_BOOLEAN_PROP(relativizePosition, 0);
}

// ----

PointCloudImporterReaderVelodyne::PointCloudImporterReaderVelodyne() {
   headerLength = 0;
   myVD = new VertexDescriptor();
   myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
   positionIsFloat64 = true;
   relativizePosition = true;
   zeroPositionInitialized = false;

   // not actually used
	myTD = new VertexDescriptor();
   myTD->pushAttrib(VAD("index", 3, VAD::DATA_FORMAT_INT32));
   myTD->pushAttrib(VAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));

   f = NULL;
   outBuffer = NULL;
   inBuffer = NULL;
   pointSize = 45;
   positionOffset = 8;
   fileSize = 0;
   bytesRead = 0;
   posAAT = NULL_AAT;
   checkPos = false;
}

void PointCloudImporterReaderVelodyne::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

PointCloudImporterReaderVelodyne::~PointCloudImporterReaderVelodyne() {}




bool PointCloudImporterReaderVelodyne::beginImport( std::string filename, bool readTrianglesIfPossible ) 
{
   ImportFinished = false;
   if (FileExists(filename.c_str())) {
      f = new LargeBinaryFile(filename.c_str(), true, false);
      fileSize = f->getFileSize();
      bytesRead = 0;
      if ((fileSize-headerLength) % (card64)pointSize != 0) {
         warning("(file length - header size) is not a multiple of pointSize");
      }
      f->seek(headerLength);
	  if (pointSize < positionOffset + 3 * (positionIsFloat64 ? sizeof(float64) : sizeof(float32))) {
		  throw(X4Exception("PointCloudImporterReaderBinary::beginImport() - pointSize is too small to contain the position."));
	  }

      if (myVD->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
         posAAT = myVD->getToken("position");
         checkPos = true;
	  } else {
		  throw(X4Exception("PointCloudImporterReaderBinary::beginImport() - No position attribute in output VD. Cannot import."));
	  }
	  inBuffer = new card8[pointSize];
      outBuffer = new card8[myVD->getSize()];
	  memset(outBuffer, 0, myVD->getSize());
      return true;
   } else {
      return false;
   }
}

const card8* PointCloudImporterReaderVelodyne::getNextPoint() 
{
   if (f) {
      if (fileSize >= bytesRead + (card64)headerLength + pointSize) {
         f->readBuffer(inBuffer, pointSize);
         bytesRead += pointSize;

		 // Convert the position
		 Vector3f *posOut = reinterpret_cast<Vector3f*>(outBuffer+posAAT.getOffset());
		 if (positionIsFloat64) {
			 Vector3d posIn = *reinterpret_cast<Vector3d*>(inBuffer+positionOffset);
			 if (relativizePosition && !zeroPositionInitialized) {
				 zeroPosition = posIn;
				 std::stringstream conv;
				 conv << std::setprecision(20); // No rounding please!
				 conv << "Zero position: (" << zeroPosition[0] << ", " << zeroPosition[1] << ", " << zeroPosition[2] << ")";
				 debugOutput << conv.str() << "\n";
				 zeroPositionInitialized = true;
			 }
			 if (relativizePosition) {
				 posIn -= zeroPosition;
			 }
			 (*posOut)[0] = static_cast<float32>(posIn[0]);
			 (*posOut)[1] = static_cast<float32>(posIn[1]);
			 (*posOut)[2] = static_cast<float32>(posIn[2]);
		 } else {
			 Vector3f posIn = *reinterpret_cast<Vector3f*>(inBuffer+positionOffset);
			 *posOut = posIn;
		 }

         if (checkPos) {
            Vector3f pos = *((Vector3f*)(outBuffer+posAAT.getOffset()));
            if (!_finite(pos[0]) || _isnan(pos[0])
            ||  !_finite(pos[1]) || _isnan(pos[1])
            ||  !_finite(pos[2]) || _isnan(pos[2]) ) {
               debugOutput << "invalid position detected at byte position " << (long double)bytesRead << " - import aborted";
               throw(X4Exception("PointCloudImporterReaderBinary::getNextPoint() - invalid position detected (see debug console for details)."));
            }
         }
         return outBuffer;
      } else {
         ImportFinished = true;
         return NULL;
      }
   } else {
      ImportFinished = true;
      return NULL;
   }
}

const card8* PointCloudImporterReaderVelodyne::getNextTriangle() 
{
   return NULL;
}


void PointCloudImporterReaderVelodyne::endImport() 
{
   delete f;
   f = NULL;
   delete[] outBuffer;
   delete[] inBuffer;
}

mpcard PointCloudImporterReaderVelodyne::getWidth() 
{
   return 1;
}

mpcard PointCloudImporterReaderVelodyne::getHeight() 
{
   return 0;
}

mpcard PointCloudImporterReaderVelodyne::getProgressPercentage() 
{
   return 100.0 * (double)bytesRead / (double)max<card64>(fileSize, 1);
}

bool PointCloudImporterReaderVelodyne::supportsTriangleImport() 
{
   return false;
}
