//---------------------------------------------------------------------------
#ifndef SceneCommandObjectsH
#define SceneCommandObjectsH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
#include "PointCloud.h"
#include "ViewFrustum.h"
#include "Camera.h"
//---------------------------------------------------------------------------


namespace X4 {

class Scene;

// CmdObjects List for faster search... ;-)

// CmdObjDeletePCs
// CmdObjRenamePC
// CmdObjPCCommand
// CmdObjChangeView
// CmdObjChangePCFlags
// CmdObjChangeSelection
// CmdObjApplyRegisterTransform


// ----

class X3EDITOR_API CmdObjDeletePCs : public CommandObject {
 private:
   static MetaClass *CmdObjDeletePCsMetaClass;
   static Object* newInstance();

   std::vector<std::string> m_deletedPCs;

   std::string getDPC(mpcard index) const;
   void setDPC(mpcard index, std::string n);
   mpcard getNumDPC() const;
   void setNumDPC(mpcard size);
   void pushDPC();
   void popDPC();                    

	void setNumDeletedPointClouds(mpcard size) {m_deletedPCs.resize(size);}
	void deleteDeletedPointCloud( mpcard index ) {x4Assert(index<m_deletedPCs.size() );m_deletedPCs.erase(m_deletedPCs.begin() + index);}
	char * getDeletedPointCloud( mpcard index ) {x4Assert(index<m_deletedPCs.size() );return (char*)&m_deletedPCs[index];}
	mpcard getNumDeletedPointClouds() {return (mpcard)m_deletedPCs.size();}
	std::string getDeletedPointCloudName(mpcard index ) {return m_deletedPCs[index];}

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   // virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void setup(std::vector<std::string> deletedPCs);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};



class X3EDITOR_API CmdObjRenamePC : public CommandObject {
 private:
   static MetaClass *CmdObjRenamePCMetaClass;
   static Object* newInstance();

   std::string m_oldName;
   std::string m_newName;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   // virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void setup(std::string oldName, std::string newName);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();

};


class PCCommand;

class X3EDITOR_API CmdObjPCCommand : public CommandObject {
 private:
   static MetaClass *CmdObjPCCommandMetaClass;
   static Object* newInstance();

 protected:
    PCCommand *m_cmd;

 public:
   CmdObjPCCommand();

   virtual void setup(const PCCommand *cmd);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   // virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();

   ~CmdObjPCCommand();

};



class X3EDITOR_API CmdObjChangeView : public CommandObject {
 private:
   static MetaClass *CmdObjChangeViewMetaClass;
   static Object* newInstance();

 protected:
   ViewFrustum viewFrustum, old_vf;
   Camera cam, old_cam;
   bool setViewFrustum;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void setup(const ViewFrustum &vf, const Camera &cam, bool setViewFrustum = false);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
   virtual bool mergeCommand(const CommandObject *nextCmd);

};



class X3EDITOR_API CmdObjChangePCFlags : public CommandObject {
 private:
   static MetaClass *CmdObjChangePCFlagsMetaClass;
   static Object* newInstance();

 protected:
   std::string targetName;
   card32 oldFlags, flags;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void setup(card32 flags, std::string targetName);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};


class X3EDITOR_API CmdObjChangeSelection : public CommandObject {
 private:
   static MetaClass *CmdObjChangeSelectionMetaClass;
   static Object* newInstance();

   std::string getSel(mpcard index) const;
   void setSel(mpcard index, std::string n);
   mpcard getNumSel() const;
   void setNumSel(mpcard size);
   void pushSel();
   void popSel();

	void setNumSelections(mpcard size) {m_selection.resize(size);}
	void deleteSelection( mpcard index ) {x4Assert( index < m_selection.size()); m_selection.erase( m_selection.begin() + index );}
	char * getSelection( mpcard index ) {x4Assert( index < m_selection.size()); return (char*)&m_selection[index];}
	mpcard getNumSelections() {return (mpcard)m_selection.size();}
	std::string getSelectionName(mpcard index ) {x4Assert( index < m_selection.size()); return m_selection[index];}


   std::string getOSel(mpcard index) const;
   void setOSel(mpcard index, std::string n);
   mpcard getNumOSel() const;
   void setNumOSel(mpcard size);
   void pushOSel();
   void popOSel();

	void setNumOldSelections(mpcard size) {m_oldSelection.resize(size);}
	void deleteOldSelection( mpcard index ) {x4Assert( index < m_oldSelection.size()); m_oldSelection.erase( m_oldSelection.begin() + index );}
	char * getOldSelection( mpcard index ) {x4Assert( index < m_oldSelection.size()); return (char*)&m_oldSelection[index];}
	mpcard getNumOldSelections() {return (mpcard)m_oldSelection.size();}
	std::string getOldSelectionName(mpcard index ) {x4Assert( index < m_oldSelection.size()); return m_oldSelection[index];}


protected:
   std::vector<std::string> m_selection, m_oldSelection;

   void setSelection(Scene *scene, const std::vector<std::string> &selection);
   std::vector<std::string> getSelection(Scene *scene);

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void setup(std::vector<std::string> selection);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};



class X3EDITOR_API CmdObjApplyRegisterTransform : public CommandObject {
 struct doUndoItem {
      Matrix4f d;
      Matrix4f u;
   };

 private:
   static MetaClass *CmdObjApplyRegisterTransformMetaClass;
   static Object* newInstance();

 protected:
   Matrix4f d,u;
   std::string target;
   int32 gridRes,numIterations;
   bool varRes;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void setup(Matrix4f do_trans,Matrix4f undo_trans,std::string pc,int numIt, int gridRes);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();

   virtual bool mergeCommand(const CommandObject *nextCmd);
};


}

#endif
