//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SceneEditorWidget.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "PCCommand.h"
#include "Configuration.h"
#include "SceneClassEditors.h"
#include "OnlineHelp.h"
#include "CommandHistoryEditorWidget.h"
#include "CmdObjPointCloudOperations.h"
#include "MovieStudioWidget.h"
#include "MacroEditorWidget.h"
#include "PCICameraController.h"
#include "Timer.h"
#include "ImportAssistantWidget.h"
#include "ImportAnimationAssistantWidget.h"
#include "SelectionTools.h"
#include "PointCloudTools.h"
#include "PointCloudExporter.h"
#include "BinaryObjectStreams.h"
#include "SceneInterface.h"
#include <qtoolbar>
#include "AddToToolbarWidget.h"
#include "XGRTEvent.h"
#include "StringHelper.h"
#include "SceneEditorOptions.h"
#include "SceneEditorOptionsEditorWidget.h"
#include "VerticalScrollView.h"
#include "FileDialogs.h"
#include "StringHelper.h"
#include "UserSettings.h"
#include "Configuration.h"
#include "ExceptionHandling.h"
#include "SceneGraphState.h"
#include "EventCounterDisplay.h"
#include "SceneGraphCmdObjs.h"
#include "MeshExporter.h"
#include "PCICameraManager.h"
#include "DebugRenderer.h"
#include "SetGSAOOCCacheSizeDialog.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
#include "LogOutput.h"
#include "PointSet.inline.h"
#include "GLRenderTextureFBO.h"
#include "ConsoleOutput.h"
#include "ShaderEditorWidget.h"
#include "DebugRendererEditorWidget.h"
#include "MemberBrowser.h"
#include "CmdObjMemberMethod.h"
#include "ClassMethods.h"
#include "MaterialAttachment.h"
#include "GLHelper.h"
#include "GLTexture.h"
#include "CameraController.h"
#include "CommandHistory.h"
#include "Statistics.h"
#include "StringHelper.h"
#include "SceneGraphNode.h"
#include "ProgressWidget.h"
#include "VertexArray.h"
#include "PCCResampler.h"
#include "Random.h"
#include "GLDebuggingTools.h"
//---------------------------------------------------------------------------
#include <fstream>
#include <qfiledialog>
#include <QVBoxLayout>
#include <qinputdialog>
#include "RenderingTool.h"
#include "ConsoleOutput.h"
#include <QDesktopWidget>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_CLASS_EDITOR_STANDALONE( Scene )

    static QTime * wait3secondsTimer = NULL;

SceneEditorWidget::SceneEditorWidget(QWidget * parent, Qt::WindowFlags f)
    :QMainWindow(parent,f), methodClickCallback(this)
{
	myObjectEditor = NULL;
	initialized = false;
	importAssistant = NULL;
	importAnimationAssistant = NULL;
	m_OffscreenRenderTarget = NULL;

	m_OffscreenRenderTarget = NULL; scene = NULL; sceneEditor = NULL; sceneEditorControl = NULL; toolsEditor = NULL; toolsEditorControl = NULL; currentTool = NULL;
	luToolMenu = NULL; cmdsEditor = NULL; cmdsEditorControl = NULL; currentCommand = NULL; luCommandMenu = NULL; rendererEditor = NULL; rendererEditorControl = NULL;
	currentRenderer = NULL; staticStateEditor = NULL; staticStateEditorWidget = NULL; dynamicStateEditor = NULL; dynamicStateEditorWidget = NULL; camTool = NULL;
	glContext = NULL; mainGLArea = NULL; debugRendererEditorWidget = NULL; cmdHistoryForm = NULL; movieStudio = NULL; macroScriptEditor = NULL; addToToolbarDlg = NULL;
	animationBarToolbar = NULL; animationBar = NULL; editorOptions = NULL; editorOptionsDlg = NULL; editSceneView = NULL; toolView = NULL; commandView = NULL;
	rendererView = NULL; importAssistant = NULL; importAnimationAssistant = NULL; 

	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	addToToolbarDlg = new AddToToolbarWidget(this);

	initLocalVars();
	this->setAttribute(Qt::WA_DeleteOnClose);


	bool sortTools = true;
	bool sortCommands = true;
	UserSettings * u = dynamic_cast<UserSettings*>( Configuration::getConfiguration()->getData( "USER_SETTINGS" ) );
	if( u )
	{
		sortTools = u->getSortToolsByName();
		sortCommands = u->getSortCommandsByName();
	}


	luToolMenu = new LastUsedMenu(this,5);
	QWidget::connect(luToolMenu,SIGNAL(triggered(QAction*)),this,SLOT(sltLUTool(QAction*)));
	luCommandMenu = new LastUsedMenu(this,5);
	QWidget::connect(luCommandMenu,SIGNAL(triggered(QAction*)),this,SLOT(sltLUCommand(QAction*)));

	ui.toolSelector->collectClasses("PCInteractionTool", sortTools, "PCI");
	ui.commandSelector->collectClasses("PCCommand", sortCommands, "PCC");
	ui.sceneSelector->collectClasses("SceneClassEditor");
	ui.rendererSelector->collectClasses("Renderer");
	QWidget::connect(ui.toolSelector, SIGNAL(currentIndexChanged(QString)), this , SLOT(sltToolChanged(QString)));
	QWidget::connect(ui.commandSelector, SIGNAL(currentIndexChanged(QString)), this , SLOT(sltCommandChanged(QString)));
	QWidget::connect(ui.sceneSelector, SIGNAL(currentIndexChanged(QString)), this , SLOT(sltSceneEditChanged(QString)));
	QWidget::connect(ui.rendererSelector, SIGNAL(currentIndexChanged(QString)), this , SLOT(sltRendererChanged(QString)));

	QWidget::connect(mainGLArea, SIGNAL(sigResize(QSize)), this , SLOT(sltGLResize(QSize)));
	QWidget::connect(mainGLArea, SIGNAL(sigRepaint()), this , SLOT(sltGLRepaint()));
	QWidget::connect(mainGLArea, SIGNAL(sigMouseMove(QMouseEvent*)), this , SLOT(sltGLMouseMove(QMouseEvent*)));
	QWidget::connect(mainGLArea, SIGNAL(sigMousePress(QMouseEvent*)), this , SLOT( sltGLMousePress(QMouseEvent*)));
	QWidget::connect(mainGLArea, SIGNAL(sigMouseRelease(QMouseEvent*)), this , SLOT( sltGLMouseRelease(QMouseEvent*)));
	QWidget::connect(mainGLArea, SIGNAL(sigWheelEvent(QWheelEvent*)), this , SLOT( sltGLWheelEvent(QWheelEvent*)));
	QWidget::connect(mainGLArea, SIGNAL(sigDropEvent( QMimeData*, QPoint)), this , SLOT( sltGLDropEvent(QMimeData *, QPoint )));

	//	QWidget::connect( menuBar(), SIGNAL(triggered(QAction*)),SLOT(sltMenuHandler(QAction*)));

	QWidget::connect(ui.execButton, SIGNAL(clicked()), this, SLOT(sltExecCommand()));


	QVBoxLayout* tl=new QVBoxLayout;
	tl->setMargin(0);
	tl->setSpacing(0);
	ui.toolView->setLayout(tl);
	QVBoxLayout* cl=new QVBoxLayout;
	cl->setMargin(0);
	cl->setSpacing(0);
	ui.commandView->setLayout(cl);
	QVBoxLayout* sl=new QVBoxLayout;
	sl->setMargin(0);
	sl->setSpacing(0);
	ui.sceneView->setLayout(sl);
	QVBoxLayout* rl=new QVBoxLayout;
	rl->setMargin(0);
	rl->setSpacing(0);
	ui.rendererView->setLayout(rl);

	editSceneView = new VerticalScrollView( ui.sceneView );
	toolView = new VerticalScrollView( ui.toolView );

	//QVBoxLayout* tl2=new QVBoxLayout;
	//tl2->setMargin(0);
	//tl2->setSpacing(0);
	//toolView->setLayout(tl2);											 

	commandView = new VerticalScrollView( ui.commandView );
	rendererView = new VerticalScrollView( ui.rendererView );

	ui.sceneView->layout()->addWidget( editSceneView );
	ui.toolView->layout()->addWidget( toolView );
	ui.commandView->layout()->addWidget( commandView );
	ui.rendererView->layout()->addWidget( rendererView );




	//statusLabel = new QLabel(ui.statusbar);
	//statusLabel->setFrameShadow(QFrame::Sunken);
	//statusLabel->setFrameStyle(QFrame::Panel);
	//ui.statusbar->addPermanentWidget(statusLabel, 1);

	/*progressBars = statusLabel->layout();
	if (progressBars) delete progressBars;
	progressBars = new QHBoxLayout(statusLabel);
	progressBars->setAlignment(Qt::AlignBottom);
	statusLabel->setLayout(progressBars);
	*/

	//set icons
	//	ui.tabWidget->setTabIcon(0,QIcon(":/editor/images/Buttons/worlds.png"));
	//	ui.tabWidget->setTabIcon(1,QIcon(":/editor/images/Buttons/tools.PNG"));
	//	ui.tabWidget->setTabIcon(2,QIcon(":/editor/images/Buttons/commands.PNG"));
	//	ui.tabWidget->setTabIcon(3,QIcon(":/editor/images/Buttons/renderer.PNG"));
	//ui.pbAddCommand->setIcon( QIcon((getConfigurationString("IMAGE_PATH")+"/addToToolbar.png").c_str()) );
	//ui.pbAddTool->setIcon( QIcon((getConfigurationString("IMAGE_PATH")+"/addToToolbar.png").c_str()) );

	{
		//put hide/view action for docking widgets in view menu
		//	QAction *toolDockToggleViewAction = ui.toolDockOld->toggleViewAction();
		//	toolDockToggleViewAction->setShortcut(QKeySequence("Ctrl+Alt+T"));
		//	ui.menuView->addAction(toolDockToggleViewAction);
		QAction *action = ui.dockWidgetSceneGraphState->toggleViewAction();
		action->setShortcut(QKeySequence("Ctrl+Alt+S"));
		ui.menuView->addAction(action);
		//		ui.dockWidgetSceneGraphState->setVisible(false);

		action = ui.dockWidgetScene->toggleViewAction();
		ui.menuView->addAction(action);

		action = ui.dockWidgetTools->toggleViewAction();
		ui.menuView->addAction(action);

		action = ui.dockWidgetCommands->toggleViewAction();
		ui.menuView->addAction(action);

		action = ui.dockWidgetRenderer->toggleViewAction();
		ui.menuView->addAction(action);

		action = ui.dockWidgetDebug->toggleViewAction();
		ui.menuView->addAction(action);


		setTabPosition(this->dockWidgetArea(ui.dockWidgetRenderer),QTabWidget::North);	
	}

	staticStateEditor = NULL;
	dynamicStateEditor = NULL;

	buildToolBar();

	animationBarToolbar = new QToolBar("Animation", this);
	animationBar = new AnimationBarWidget(animationBarToolbar);
	animationBarToolbar->setObjectName("animationToolBar");

	//put hide/view action for animationBarToolbar in view menu

	QAction *animationToolbarToggleViewAction = animationBarToolbar->toggleViewAction();
	animationToolbarToggleViewAction->setShortcut(QKeySequence("Ctrl+Alt+A"));
	ui.menuView->addAction(animationToolbarToggleViewAction);

	animationBarToolbar->addWidget(animationBar);
	addToolBar(Qt::BottomToolBarArea, animationBarToolbar );
	animationBarToolbar->setAllowedAreas(Qt::BottomToolBarArea | Qt::TopToolBarArea);
	QWidget::connect(animationBar, SIGNAL(frameChanged(int)), this , SLOT(sltAnimationBarFrameChanged(int)));
	QWidget::connect(animationBar, SIGNAL(traversalModeChanged(SGStaticState::TraversalModeType)), this , SLOT(sltAnimationBarTraversalModeChanged(SGStaticState::TraversalModeType)));

	editorOptionsDlg = NULL;
	editorOptions = dynamic_cast<SceneEditorOptions*>(Configuration::getConfiguration()->getData( "SCENE_EDITOR_OPTIONS" ) );
	if( editorOptions == NULL )
	{
		editorOptions = new SceneEditorOptions;
		editorOptions->setup("SCENE_EDITOR_OPTIONS", AttachedData::ADF_PERSISTENT);
		Configuration::getConfiguration()->attachData( (AttachedData*)editorOptions->copy() );
	}
	else
		editorOptions = (SceneEditorOptions*)editorOptions->copy();

	on_actionRestore_Layout_triggered(true);
	if( !dynamic_cast<BinaryAttachment*>( Configuration::getConfiguration()->getData("DockerState") ) ) {
		// no docker config - reinitialize defaults

		on_actionReset_Layout_triggered(true);
	}


	QWidget::connect(ui.frame, SIGNAL(hasBeenShowen()), this, SLOT(sltStateDockShowing()));

	m_ScreenShotBaseName = "screenshot";
	m_ScreenShotID = 1;
	m_ScreenShotImageFormat = IF_PNG;

	timerID = startTimer(10);
	redrawDelay = editorOptions->getRedrawDelay();

	m_RecordHighQualityScreenShot = false;
	m_SetupTiledImage = false;

	wait3secondsTimer = new QTime;
	wait3secondsTimer->start();

	doubleNum=0;

	m_SimulateLens = false;
	m_SimulateLens_ConvRadius = 1.0f;
	m_SimulateLens_NumSamples = 1;
}

void SceneEditorWidget::sltShowStatusMesage(QString msg)
{
    ui.statusbar->showMessage(msg);
}

void SceneEditorWidget::sltClearStatusMessage()
{
    ui.statusbar->clearMessage();
}

void SceneEditorWidget::addProgressbarWidget(QWidget* widget)
{
    ProgressWidget* wdg = dynamic_cast<ProgressWidget*>(widget);
    if (wdg)
    {
        QObject::connect(wdg, SIGNAL(showToolMessage(QString)), this, SLOT(sltShowStatusMesage(QString)));
    }
    ui.statusbar->addPermanentWidget(widget);
}

void SceneEditorWidget::removeProgressbarWidget(QWidget* widget)
{
    ProgressWidget* wdg = dynamic_cast<ProgressWidget*>(widget);
    if (wdg)
    {
        QObject::disconnect(wdg, SIGNAL(showToolMessage(QString)), this, SLOT(sltShowStatusMesage(QString)));
    }
    ui.statusbar->removeWidget(widget);
}

void SceneEditorWidget::sltStateDockShowing() {					  
    // (mw) reset the allowable with to maximum (in the ui file, a maximum of 200 is set to avoid layouting problems)
    // this is a nasty hack...
    ui.dockWidgetSceneGraphState->setMaximumWidth(16777215);
}


void SceneEditorWidget::addToolBarVec( QToolBar * tb ) {
    QAction *toolButtonsToolbarToggleViewAction = tb->toggleViewAction();
    toolButtonsToolbarToggleViewAction->setShortcut(QKeySequence("Ctrl+Alt+B")); // added shortcut

    ui.menuView->addAction(toolButtonsToolbarToggleViewAction);	//changed to view tb in menu (ab)
    toolBars.push_back(tb);
}

SceneEditorWidget::~SceneEditorWidget(void)
{
    currentTool->deactivateTool();

    delete cmdHistoryForm;
    delete movieStudio;

    // mb - deleting materials may need active context to release gl resources
    glContext->activate();
    debugRenderer->disconnect(glContext);

    //delete m_offlineRenderingForm;
    delete toolsEditor;
    delete cmdsEditor;
    delete rendererEditor;
    delete sceneEditor;

    glContext->deactivate();
    delete glContext;
    delete staticStateEditor;
    delete dynamicStateEditor;

	 if (myObjectEditor) {
		 //ObjectViewsTable::releaseView(scene, myObjectEditor);
		 delete myObjectEditor;
	 }
}

void SceneEditorWidget::setup( Scene * obj )

{
    scene = obj;
    scene->activateMyStatistics();
    logOutput.beginLogOutput(scene);
    mainGLArea->scene = scene;

    movieStudio = new MovieStudioWidget(this);
    macroScriptEditor = new MacroEditorWidget(scene, this);

    cmdHistoryForm = new CommandHistoryEditorWidget(NULL);
    cmdHistoryForm->setup(obj->getCommandHistory());
    cmdHistoryForm->setContext(scene);
    updateMenu();


    //set startup Tools:
    sltToolChanged("PCICameraController");
    std::string preferedTool =  editorOptions->getDefaultPCI()->getClassName();
    if (preferedTool != "PCICameraController")
        //		ui.tabWidget->setCurrentIndex(1);
            ui.dockWidgetTools->raise();
    sltToolChanged(preferedTool.c_str());

    sltCommandChanged(editorOptions->getDefaultPCC()->getClassName().c_str());
    sltSceneEditChanged("SceneGraphEditor");
    sltRendererChanged("LCHierarchyRenderer");

    staticStateEditor = (ClassEditor*)scene->getRootState()->staticState->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
    staticStateEditorWidget = staticStateEditor->makeEditor(ui.frameStaticState, scene->getRootState()->staticState);
    QVBoxLayout *ls = new QVBoxLayout(ui.frameStaticState);
    ls->setMargin(0);
    ls->setSpacing(0);
    ls->addWidget(staticStateEditorWidget);

    dynamicStateEditor = (ClassEditor*)scene->getRootState()->dynamicState->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
    dynamicStateEditorWidget = dynamicStateEditor->makeEditor(ui.frameDynamicState, scene->getRootState()->dynamicState);
    QVBoxLayout *ld = new QVBoxLayout(ui.frameDynamicState);
    ld->setMargin(0);
    ld->setSpacing(0);
    ld->addWidget(dynamicStateEditorWidget);

    animationBar->setTime(scene->getRootState()->dynamicState->time);

	 initialized = true;
}

void SceneEditorWidget::updateEditor()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (!initialized) return;
   animationBar->updateTraversalModeBox(scene->getRootState()->staticState->defaultTraversalMode);
   update();
   mainGLArea->updateGL();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

class SceneEditorWidget_OnSceneChangeCallback : public OnSceneChangeCallback {
public:
    SceneEditorWidget *m_ed;

    SceneEditorWidget_OnSceneChangeCallback(SceneEditorWidget *ed) {m_ed = ed;}

    virtual void changed(PCInteractionToolChanges whatChange) {
        m_ed->updateSceneEditor(whatChange);
    }
};

void SceneEditorWidget::updateSceneEditor( PCInteractionToolChanges whatChange )
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        switch( whatChange )
    {
        case PCI_UPDATE_VIEW_SIGNAL:
            mainGLArea->update();
            break;

        case PCI_CHANGE_ALL:
            mainGLArea->updateGL();
            if( movieStudio->isRecordEveryRepaintChecked() )
                saveFrame();
            break;
        case PCI_CHANGE_VIEW:
            mainGLArea->updateGL();
            if( movieStudio->isRecordEveryRepaintChecked() )
                saveFrame();
            break;
        case PCI_NEW_COMMAND:
            cmdHistoryForm->update();
            break;
        case PCI_WRITE_FRAME:
            if( movieStudio->isRecordPCIFrameEventsChecked() )
            {
                saveFrame();
            }
            break;
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}
void SceneEditorWidget::changeCommand( const MetaClass * newCmdClass, PCCommand * obj)
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string s = newCmdClass->getClassName();
    ui.commandSelector->setSelection(s);

    luCommandMenu->addEntry(s.c_str());

    if (currentCommand != NULL) {
        oldTools.insertObject((Object*)currentCommand);
        currentCommand = NULL;
    }

    if (cmdsEditor != NULL) {
        delete cmdsEditor;
        cmdsEditor = NULL;
    }

    if (cmdsEditorControl != NULL) {
        //ui.toolView->layout()->removeWidget(cmdsEditorControl);
        delete cmdsEditorControl;
    }
    cmdsEditorControl = NULL;

    if (newCmdClass == NULL || newCmdClass->isAbstract()) return;

    if (!oldTools.containsObjectOfClass(newCmdClass)) {
        currentCommand = (PCCommand*)(newCmdClass->newInstance());
        currentCommand->connectScene( scene, new SceneEditorWidget_OnSceneChangeCallback( this ), this );
        oldTools.insertObject(currentCommand);
    } else {
        currentCommand = (PCCommand*)(oldTools.getInstance(newCmdClass));
    }
    currentCommand->setScene(scene);
    if( obj != NULL )
        currentCommand->assign( obj );

    const MetaClass *newCmdEditorClass = newCmdClass->getGUIDescriptor()->getDefaultClassEditor();

    if (newCmdEditorClass == NULL || newCmdEditorClass->isAbstract()) return;

    cmdsEditor = (ClassEditor*)(newCmdEditorClass->newInstance());
    cmdsEditorControl = cmdsEditor->makeEditor(commandView->viewport(), currentCommand);
    commandView->setWidget(cmdsEditorControl  );
    cmdsEditorControl->show();
    //ui.commandView->layout()->addWidget(cmdsEditorControl);
    //cmdsEditor->update();

    MemberBrowser *cmdsEdMemberBrowser = dynamic_cast<MemberBrowser*>(cmdsEditor);
    if (cmdsEdMemberBrowser) {
        MemberBrowser::MethodClickedNotifyEvent ev(&methodClickCallback, &SceneEditorWidget::CallMeBack::cmdMethodCalled);
        cmdsEdMemberBrowser->addOnMethodClickedEvent(ev);
    }

    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::changeTool( const MetaClass * newToolClass, PCInteractionTool * obj)
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string s = newToolClass->getClassName();

    if (newToolClass == NULL ) return;
    ui.toolSelector->setSelection(s);

    luToolMenu->addEntry(s.c_str());

    if (currentTool != NULL) {
        currentTool->deactivateTool();
        oldTools.insertObject(currentTool);
        currentTool = NULL;
    }

    if (toolsEditor != NULL) {
        delete toolsEditor;
        toolsEditor = NULL;
    }

    if (toolsEditorControl != NULL) {
        //ui.toolView->layout()->removeWidget(toolsEditorControl);
        delete toolsEditorControl;
    }
    toolsEditorControl = NULL;

    if (newToolClass == NULL || newToolClass->isAbstract()) return;

    const MetaClass *newToolEditorClass = newToolClass->getGUIDescriptor()->getDefaultClassEditor();
    if (newToolEditorClass == NULL || newToolEditorClass->isAbstract()) return;

    bool connectToScene = false;

    if (!oldTools.containsObjectOfClass(newToolClass)) 
    {
        currentTool = (PCInteractionTool*)(newToolClass->newInstance());
        connectToScene = true;
    } else {
        currentTool = (PCInteractionTool*)(oldTools.getInstance(newToolClass));
    }

    if( obj != NULL )
        currentTool->assign( obj );

    if (connectToScene)
    {
        currentTool->connectToScene(scene, new SceneEditorWidget_OnSceneChangeCallback(this), this, ui.toolView);
        oldTools.insertObject(currentTool);
        if( newToolClass == PCICameraController::getClass() )
        {
            if( camTool == NULL )
                camTool = currentTool;

            CameraController* nextCamCtrl = (CameraController*)editorOptions->getDefaultCameraClass()->newInstance();
            nextCamCtrl->bindCamera(&scene->getRootState()->staticState->camera);
            ((PCICameraController*)currentTool)->setCamCtrl( nextCamCtrl );
        }
    }


    // create editor widget
    toolsEditor = dynamic_cast<ClassEditor*>(newToolEditorClass->newInstance());
    toolsEditorControl = toolsEditor->makeEditor(toolView->viewport(), currentTool);
    toolView->setWidget(toolsEditorControl);
    toolsEditorControl->show();

    currentTool->activateTool(); 
    currentTool->areaResize(ui.mainGLArea->width(), ui.mainGLArea->height());
    currentTool->areaResize_base(ui.mainGLArea->width(), ui.mainGLArea->height());


    bool defaultCam = currentTool->forceCamByDefault();
    if (defaultCam && !force_cam) {
        QKeyEvent e(QEvent::KeyPress, Qt::Key_Space, Qt::KeyboardModifiers());
        processToolbarKeyEvents(&e);
    }

    MemberBrowser *toolsEdMemberBrowser = dynamic_cast<MemberBrowser*> (toolsEditor);
    if (toolsEdMemberBrowser) {
        MemberBrowser::MethodClickedNotifyEvent ev(&methodClickCallback, &SceneEditorWidget::CallMeBack::toolMethodCalled);
        toolsEdMemberBrowser->addOnMethodClickedEvent(ev);
    }

    STD_GLOBAL_EXCEPTION_HANDLER_END
}


void SceneEditorWidget::sltToolChanged( const QString & newToolName ){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string  s = newToolName.toLatin1().constData();
    if( s.find( "PCI" ) != 0 )
        s = std::string( "PCI") + s;
    // std::cout << "\nTool switching to " <<s;
    const MetaClass *newToolClass = MetaClass::getClassByName(s);
    if (newToolClass == NULL ) return;
    changeTool( newToolClass );
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltCommandChanged( const QString & newCommandName ){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string  s = newCommandName.toLatin1().constData();
    if( s.find( "PCC" ) != 0 )
        s = std::string( "PCC") + s;
    // std::cout << "\nCommand switching to " <<s;
    const MetaClass *newCmdClass = MetaClass::getClassByName(s);

    if (newCmdClass == NULL) return;
    changeCommand( newCmdClass );
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltSceneEditChanged( const QString & newCommandName ){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string  s = newCommandName.toLatin1().constData();
    // std::cout << "\nSceneEdit switching to " <<s;
    const MetaClass *newEditor = MetaClass::getClassByName(s);
    if (newEditor == NULL ) return;
    ui.sceneSelector->setSelection(s);

    delete sceneEditor;
    sceneEditor = NULL;
    if (sceneEditorControl != NULL) {
        ui.sceneView->layout()->removeWidget(sceneEditorControl);
        delete sceneEditorControl;
    }
    sceneEditorControl = NULL;

    if (newEditor->isAbstract()) return;

    sceneEditor = (ClassEditor*)(newEditor->newInstance());
    SceneClassEditor * pcSceneEditor = dynamic_cast<SceneClassEditor *>( sceneEditor );
    if( pcSceneEditor )
        pcSceneEditor->setGLContext(glContext); 
    sceneEditorControl = sceneEditor->makeEditor(editSceneView->viewport(), scene);
    editSceneView->setWidget(sceneEditorControl);
    sceneEditorControl->show();
    //ui.sceneView->layout()->addWidget(sceneEditorControl);

    sceneEditor->update();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltRendererChanged( const QString & newToolName ){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string  s = newToolName.toLatin1().constData();
    // std::cout << "\nRenderer switching to \n" <<s;
    const MetaClass *newRendererClass = MetaClass::getClassByName(s);

    if (newRendererClass == NULL ) return;
    ui.rendererSelector->setSelection(s);

    if (currentRenderer != NULL) {
        currentRenderer->disconnect(glContext); 
        oldTools.insertObject(currentRenderer);
        currentRenderer = NULL;
    }

    delete rendererEditor;
    rendererEditor = NULL;
    if (rendererEditorControl != NULL) {
        ui.rendererView->layout()->removeWidget(rendererEditorControl);
        delete rendererEditorControl;
    }
    rendererEditorControl = NULL;

    if (newRendererClass == NULL || newRendererClass->isAbstract()) return;

    if (!oldTools.containsObjectOfClass(newRendererClass)) {
        currentRenderer = (Renderer*)(newRendererClass->newInstance());
        oldTools.insertObject(currentRenderer);
    } else {
        currentRenderer = (Renderer*)(oldTools.getInstance(newRendererClass));
    }

    x4Assert(glContext);
    currentRenderer->connect(scene, glContext);

    const MetaClass *newRendererEditorClass = newRendererClass->getGUIDescriptor()->getDefaultClassEditor();

    if (newRendererEditorClass == NULL || newRendererEditorClass->isAbstract()) return;

    rendererEditor = (ClassEditor*)(newRendererEditorClass->newInstance());
    rendererEditorControl = rendererEditor->makeEditor(rendererView->viewport(), currentRenderer);
    rendererView->setWidget(rendererEditorControl);
    rendererEditorControl->show();
    //ui.rendererView->layout()->addWidget(rendererEditorControl);
    rendererEditor->update();
    sltGLRepaint();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltExecCommand()
{

    STD_GLOBAL_EXCEPTION_HANDLER_START
        if (currentCommand != NULL) {
            CmdObjPCCommand *newCmdObj = new CmdObjPCCommand();
            newCmdObj->setup(currentCommand);
            std::string name = currentCommand->getInstanceClass()->getClassName();
            Timer timer;
            scene->getCommandHistory()->executeCommand(scene, newCmdObj, true);
            UserSettings * u = dynamic_cast<UserSettings*>( Configuration::getConfiguration()->getData( "USER_SETTINGS" ) );
            if(u) {
                if (u->getTimeCommands()) {
                    debugOutput << "Executed command of type <" << name << "> in " << timer.getDeltaValue() << " ms.\n";
                }
            }
            if (cmdHistoryForm->isVisible() ) 
                cmdHistoryForm->updateEditor();
            updateMenu();
            ObjectViewsTable::update(scene);
        }
        mainGLArea->updateGL();  
        STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltGLResize(const QSize){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        //statusBar->Panels->Items[0]->Text = Format("Res.: %d x %d", res, 1);
        QString status = QString(" Res: %1 x %2  ")
        .arg(mainGLArea->width())        
        .arg(mainGLArea->height()) ;   
    //statusLabel->setText(status);
    ui.statusbar->showMessage(status);
    if (mainGLArea->height() > 0) {
        scene->getRootState()->staticState->viewFrustum.setXToYAspectRatio((float32)mainGLArea->width() / (float32)mainGLArea->height());
        ObjectViewsTable::update(&(scene->getRootState()->staticState->viewFrustum));
    }

    if (currentTool != NULL) {
        currentTool->areaResize(ui.mainGLArea->width(), ui.mainGLArea->height());
        currentTool->areaResize_base(ui.mainGLArea->width(), ui.mainGLArea->height());
    }
    if (camTool != NULL) {
        camTool->areaResize(ui.mainGLArea->width(), ui.mainGLArea->height());
        camTool->areaResize_base(ui.mainGLArea->width(), ui.mainGLArea->height());
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltGLRepaint(void){
	STD_GLOBAL_EXCEPTION_HANDLER_START
	X4_GL_DBG
   /// test: try to make frame grabbing work
   glReadBuffer(GL_FRONT);
	X4_GL_DBG
	if (glContext == NULL) {
		glContext = mainGLArea->getContext();
		//m_offlineRenderingForm->setContext(glContext);
		//m_trackingForm->setContext(glContext);
	}
	//   glContext->setGlobalActive(true);
	x4Assert(glContext);

	ShaderEditorWidget::m_DefaultGLContext = glContext;

	if (scene == NULL) return;
	if (!ui.actionRendering_Activated->isChecked()) {
		glContext->activate();
		X4_GL_DBG
		glClearColor(128.0f/255.0f, 30.0f/255.0f, 30.0f/255.0f, 1.0f);
		glClear(GL_COLOR_BUFFER_BIT);
		X4_GL_DBG
		return;
	}

	if (currentRenderer==NULL){
		glContext->activate();
		X4_GL_DBG
		//Vector3f bg = editorOptions->getBackGroundColor();
		Vector4f bg = scene->getRootState()->staticState->backgroundColor;
		glClearColor(bg[0],bg[1],bg[2],bg[3]);
		glClear(GL_COLOR_BUFFER_BIT);
		X4_GL_DBG
		return;
	}

	glContext->activate();

	// create huge offscreen buffer for high quality screenshot
	GLRenderTextureFBO * renderTarget = NULL;
	if( m_RecordHighQualityScreenShot )
	{
		renderTarget = new GLRenderTextureFBO( glContext );
		X4_GL_DBG
		m_RecordHighQualityScreenShot = false;
		if( !renderTarget->create( 4096, 3072, 4, GLRenderTextureFBO::FLAG_DEPTH | GLRenderTextureFBO::FLAG_RECTANGLE ) )
		{
			delete renderTarget;
			renderTarget = NULL;
		} else {
			renderTarget->activate();
		}
		X4_GL_DBG
	}

	glContext->setUseVertexBufferObjects( !editorOptions->getUseSoftwareVertexBuffers() );
	X4_GL_DBG

	if (mainGLArea->width() <= 0) return;

	try {
		//Vector3f bg = editorOptions->getBackGroundColor();
		Vector4f bg = scene->getRootState()->staticState->backgroundColor;
		glClearColor(bg[0],bg[1],bg[2],bg[3]);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
		X4_GL_DBG

		unsigned screenWidth = renderTarget?renderTarget->getWidth():mainGLArea->width();
		unsigned screenHeight = renderTarget?renderTarget->getHeight():mainGLArea->height();

		if( !m_SetupTiledImage)
		{
			glViewport(0,0, screenWidth, screenHeight);
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(scene->getRootState()->staticState->viewFrustum.getVerticalFieldOfView(),
					(float)screenWidth/ (float)screenHeight,
					scene->getRootState()->staticState->viewFrustum.getZNearClip(),
					scene->getRootState()->staticState->viewFrustum.getZFarClip());
			X4_GL_DBG

		}
		else
		{
			glViewport(0,0, screenWidth, screenHeight);
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			X4_GL_DBG

			double clip_near = scene->getRootState()->staticState->viewFrustum.getZNearClip();
			double clip_far = scene->getRootState()->staticState->viewFrustum.getZFarClip();

			double fovy = scene->getRootState()->staticState->viewFrustum.getVerticalFieldOfView()/2.0f;

			double left  = -clip_near*tan(fovy*M_PI/180.0)/ (double)screenHeight * (double)screenWidth;
			double bottom= -clip_near*tan(fovy*M_PI/180.0);
			double top= clip_near*tan(fovy*M_PI/180.0);

			double dx= -2.0*left;
			double dy= -2.0*bottom;

			double x0 = m_TiledImage_X0[0];
			double x1 = m_TiledImage_X1[0];
			double y1 = m_TiledImage_X0[1];
			double y0 = m_TiledImage_X1[1];

			double frustum_right=left+x1*dx;
			double frustum_left=left+x0*dx;

			double frustum_bottom=top-y0*dy;
			double frustum_top=top-y1*dy;



			glFrustum(frustum_left,frustum_right,frustum_bottom,frustum_top,clip_near,clip_far);
			X4_GL_DBG
		}


		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		X4_GL_DBG
		if( m_SimulateLens )
		{
			Vector3f camPos = scene->getRootState()->staticState->camera.getPosition();
			Vector3f camUp = scene->getRootState()->staticState->camera.getUp();
			Vector3f camLookAt = scene->getRootState()->staticState->camera.getLookAt();

			Vector3f dirX = normalize( (camLookAt - camPos).crossProduct(camUp));
			Vector3f dirY = normalize( (camLookAt - camPos).crossProduct(dirX));

			Vector3f newCamPos = camPos + dirX * m_SimulateLens_Temp_SamplePoint[0] + dirY * m_SimulateLens_Temp_SamplePoint[1];

			gluLookAt(newCamPos[0],newCamPos[1],newCamPos[2],
					camLookAt[0],camLookAt[1],camLookAt[2],
					camUp[0],camUp[1],camUp[2]);
			X4_GL_DBG
		} else {
			gluLookAt(scene->getRootState()->staticState->camera.getPosition()[0],
			scene->getRootState()->staticState->camera.getPosition()[1],
			scene->getRootState()->staticState->camera.getPosition()[2],

			scene->getRootState()->staticState->camera.getLookAt()[0],
			scene->getRootState()->staticState->camera.getLookAt()[1],
			scene->getRootState()->staticState->camera.getLookAt()[2],

			scene->getRootState()->staticState->camera.getUp()[0],
			scene->getRootState()->staticState->camera.getUp()[1],
			scene->getRootState()->staticState->camera.getUp()[2]);
		}
		X4_GL_DBG

		bool doDraw = true;
		if (currentTool != NULL) {
			doDraw = currentTool->drawGLSceneRendering();
			X4_GL_DBG
		}

		if (doDraw) {
			// prepare current tool for rendering
			if (currentTool != NULL ) {
				currentTool->beforeGlRendering(glContext);
				X4_GL_DBG
			}
			if (currentRenderer != NULL) {
				if( m_OffscreenRenderTarget ) {
					currentRenderer->setOffscreenRenderTarget(m_OffscreenRenderTarget);
					X4_GL_DBG
				}
				currentRenderer->setUseBBSelection( editorOptions->getUseBBSelection() );
				X4_GL_DBG
				currentRenderer->render(screenWidth, screenHeight);
				X4_GL_DBG
				currentRenderer->setOffscreenRenderTarget(NULL);
				X4_GL_DBG
			}
		}

		// Call current tool draw
		if (currentTool != NULL ) {
			currentTool->glDrawTool(glContext);
			X4_GL_DBG
		}

		// Call current command draw
		if (currentCommand != NULL) {
			currentCommand->glDrawTool(glContext);
			X4_GL_DBG
		}

		// -- Show a texture at mainGLArea for Copyright --
		// get Texture from Attachments -> textureName is hardwired: "CopyrightImage"
		TextureAttachment *tex = dynamic_cast<TextureAttachment*>(
			scene->getAttachments()->getData("CopyrightImage"));

		// If texture is not available -> skip this code
		if ( tex != NULL) {

			glEnable( GL_BLEND );
			glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			X4_GL_DBG
			// Start 2D Modus
			GLHelper::begin2D(screenWidth, screenHeight);
			GLTexture *gltex = tex->getCachedTexture(glContext);
			gltex->bind();
			glEnable( gltex->getTextureTarget() );
			X4_GL_DBG
			glBegin(GL_QUADS);
			glColor4f(1,1,1,0.6f);
			glTexCoord2f(0,0);
			// left top 
			glVertex2f(screenWidth-144, 86); 
			glTexCoord2f(1,0);
			// right top 
			glVertex2f(screenWidth, 86); // rechts oben
			glTexCoord2f(1,1);
			// bottom right 
			glVertex2f(screenWidth, 0);
			glTexCoord2f(0,1);
			// bottom left 
			glVertex2f(screenWidth-144, 0);
			glEnd();
			glDisable( GL_BLEND );
			glDisable( gltex->getTextureTarget() );
			// End 2D Modus
			GLHelper::end2D();
			X4_GL_DBG
		}


		if( currentRenderer->debugRendererAllowed() )
			debugRenderer->draw( glContext, scene );

		if( renderTarget )
		{
			renderTarget->deactivate();
			X4_GL_DBG
			RGBAImage * rgbaImage = renderTarget->getRGBAImage();
			QImage image(rgbaImage->getSizeX(),rgbaImage->getSizeY(),QImage::Format_RGB32);

			for (unsigned x=0;x<(unsigned)image.width();x++) {
				for (unsigned y=0;y<(unsigned)image.height();y++) {
					Vector4ub col = rgbaImage->getPixel(x,image.height() - 1 - y);
					QColor col_qt(col[0],col[1],col[2]);
					image.setPixel(x,y,col_qt.rgb());
				}
			}
			X4_GL_DBG

			QString num = QString::number( m_ScreenShotID++ );
			while( num.length() < 5 )
					num = QString("0") + num;
			std::string filename = movieStudio->getFileNameString() + num.toLatin1().constData() + ".";

			QString qformat = "PNG";
			image.save( (filename + qformat.toLatin1().constData()).c_str(), qformat.toLatin1().constData(), 85 );
			debugOutput << "saved high quality screenshot to: " << filename << "\n";
			delete renderTarget;
			X4_GL_DBG
		}


	} catch (...) {
		ui.actionRendering_Activated->setChecked(false);
		throw;
	}

	/*

	// update slider settings in tracking!
	m_trackingForm->update();
	*/
	if (movieStudio->isRecordEveryRepaintChecked() && !movieStudio->isRecordPCIFrameEventsChecked()) {
		saveFrame();
		X4_GL_DBG
	}


	if (ui.actionDisplay_Hierarchy_Events->isChecked()) {
		EventCounterDisplay::DisplayAndReset evd;
		debugOutput.displayEvent(EventCounterDisplay::getClass(), std::string("fetch nodes"), &evd);
		debugOutput.displayEvent(EventCounterDisplay::getClass(), std::string("write nodes"), &evd);
		debugOutput.displayEvent(EventCounterDisplay::getClass(), std::string("prefetch nodes"), &evd);
		debugOutput.displayEvent(EventCounterDisplay::getClass(), std::string("create nodes"), &evd);
		debugOutput.displayEvent(EventCounterDisplay::getClass(), std::string("delete nodes"), &evd);
	}
	X4_GL_DBG
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltGLMouseMove(QMouseEvent *event){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if (!force_cam)
            currentTool->mouseMoved(event->x(),event->y());
        else
            camTool->mouseMoved(event->x(),event->y());
    if( redrawDelay < 0)
        mainGLArea->updateGL();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltGLMousePress(QMouseEvent *event){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        scene->activateMyStatistics();
    if (currentTool == NULL) return;

    if (!force_cam)
        currentTool->mouseDown(event->x(),event->y(), MouseButtons(event->button()==Qt::LeftButton,event->button()==Qt::RightButton, false, event->button()==Qt::MidButton) , Modifiers(event->modifiers()));
    else
        camTool->mouseDown(event->x(),event->y(), MouseButtons(event->button()==Qt::LeftButton,event->button()==Qt::RightButton, false, event->button()==Qt::MidButton) , Modifiers(event->modifiers()));

    if( redrawDelay < 0)
        mainGLArea->updateGL();
    if (cmdHistoryForm->isVisible() ) 
        cmdHistoryForm->updateEditor();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltGLMouseRelease(QMouseEvent *event){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if (currentTool == NULL) return;
    //currentTool->mouseUp(event->x(),event->y(), MouseButtons(event->button()==Qt::LeftButton,event->button()==Qt::RightButton, event->button()==Qt::MidButton) , Modifiers(event->modifiers())); 
    if (!force_cam)
        currentTool->mouseUp(event->x(),event->y(), MouseButtons(event->button()==Qt::LeftButton,event->button()==Qt::RightButton, event->button()==Qt::MidButton) , Modifiers(event->modifiers())); 
    else
        camTool->mouseUp(event->x(),event->y(), MouseButtons(event->button()==Qt::LeftButton,event->button()==Qt::RightButton, event->button()==Qt::MidButton) , Modifiers(event->modifiers())); 
    if( redrawDelay < 0)
        mainGLArea->updateGL();
    if (cmdHistoryForm->isVisible() ) 
        cmdHistoryForm->updateEditor();
    updateMenu();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void X4::SceneEditorWidget::sltGLWheelEvent( QWheelEvent * event )
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if (currentTool == NULL) return;

    if (!force_cam)
        currentTool->mouseWheelRotated(event->delta(), Modifiers(event->modifiers()));
    else
        camTool->mouseWheelRotated(event->delta(), Modifiers(event->modifiers()));

    if( redrawDelay < 0)
        mainGLArea->updateGL();

    if (cmdHistoryForm->isVisible() ) 
        cmdHistoryForm->updateEditor();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::keyPressEvent ( QKeyEvent * event ){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        scene->activateMyStatistics();
    if (currentTool == NULL) return;

    if (force_cam)
        camTool->keyDown(GeneralKey(event->key(),Modifiers(event->modifiers())));
    else
        currentTool->keyDown(GeneralKey(event->key(),Modifiers(event->modifiers())));

    if( processToolbarKeyEvents( event ) )
        event->accept();
    else
        event->ignore();
    if( redrawDelay < 0)
        mainGLArea->updateGL();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::keyReleaseEvent ( QKeyEvent * event ){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        scene->activateMyStatistics();
    if (currentTool == NULL) return;


    if (force_cam)
        camTool->keyUp(GeneralKey(event->key(),Modifiers(event->modifiers())));
    else
        currentTool->keyUp(GeneralKey(event->key(),Modifiers(event->modifiers())));

    if( redrawDelay < 0)
        mainGLArea->updateGL();
    if (cmdHistoryForm->isVisible() ) 
        cmdHistoryForm->updateEditor();
    updateMenu();
    STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void SceneEditorWidget::focusInEvent( QFocusEvent * e )
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        scene->activateMyStatistics();
    if( getSceneInterface()->getVersion() != m_InterfaceVersion )
        buildToolBar();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::updateMenu(){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CommandHistory *script = scene->getCommandHistory();
    ui.actionUndo->setEnabled(script->canUndo());
    ui.actionRedo->setEnabled(script->canRedo());
    ui.actionRepeat->setEnabled(script->canRepeat());
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

// ----- "file" actions -----


void SceneEditorWidget::on_actionSave_as_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string filename = FileDialogs::getSaveFileName( this, "save file", "*.x4obj" );
    if( filename != "" )
    {
        if( !QString(filename.c_str()).contains( ".x4obj", Qt::CaseInsensitive ) )
            filename += ".x4obj";
        BinaryOutputObjectStream out(filename.c_str());
        out.writeObject((Persistent*)scene);
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}


void SceneEditorWidget::on_actionImport_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if (!importAssistant) importAssistant = new ImportAssistantWidget(this, scene);
    importAssistant->show();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionImport_Animation_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if (!importAnimationAssistant) importAnimationAssistant = new ImportAnimationAssistantWidget(this, scene);
    importAnimationAssistant->show();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionExport_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string filename = FileDialogs::getSaveFileName(this, "Export Scene as Point Cloud (ASCII Format)", "Canonical Ascii Format (*.points)", NULL, "", FileDialogs::EXPORT_DIR);
    if( filename == "" ) return;
    if (extractFileExt(filename) != ".points") {
        filename = filename + std::string(".points");
    }
    std::ofstream f(filename.c_str());

    SGLegacyPCSceneAdaptor sceneLA(scene);
    for (card32 pcnum = 0; pcnum < sceneLA.getNumPointClouds(); ++pcnum) {
        if(!sceneLA.visible(pcnum)) continue;
        PointCloud* pc = sceneLA.getPointCloud(pcnum);
        PointCloudExporter exporter(pc, &f);
        exporter.exportPointCloud();
        f << endl;
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void X4::SceneEditorWidget::on_actionExport_Obj_triggered( bool checked )
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string filename = FileDialogs::getSaveFileName(this, "Export Scene (selected nodes or everything if no selection) as Obj (obj Format)", "OBJ Format (*.obj)", NULL, "", FileDialogs::EXPORT_DIR);
    if( filename == "" ) return;
    if (extractFileExt(filename) != ".obj") {
        //filename = filename + std::string(".obj");
    }

    bool separate = false;
    if (QMessageBox::question(this, "export option", "Do you want to export every mesh in a separate file?", 
        QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes) {
            separate = true;                          
    }

    std::ofstream f((filename + std::string(".obj")).c_str());
    card32 separateCounter = 0;

    std::set<std::string> selection = scene->getRootState()->staticState->selectedNodes;
    if (selection.empty()) {
        SGLegacyPCSceneAdaptor sceneLA(scene);
        for (card32 pcnum = 0; pcnum < sceneLA.getNumPointClouds(); ++pcnum) {
            PointCloud* pc = sceneLA.getPointCloud(pcnum);
            MeshExporter exporter(pc, &f);
            exporter.setExportObjNormals(true);
			exporter.setExportObjSmallTriangles(false);
            //exporter.setExportObjSmallTriangles(true);
            exporter.exportPointCloud();
            f << endl;
            if (separate) {
                f.close();
                f.open((filename+intToStr(++separateCounter)+ std::string(".obj")).c_str());
            }
        }
    } else {
        std::set<std::string>::iterator pos = selection.begin();
        std::set<std::string>::iterator end = selection.end();
        PointCloud *pc = getPointCloud(scene, *pos);
        if (pc != NULL) {
            MeshExporter exporter(pc, &f);
			exporter.setExportObjNormals(true); 
			exporter.setExportObjSmallTriangles(false);
            //exporter.setExportObjSmallTriangles(true);
            exporter.exportPointCloud();
            f << endl;
            if (separate) {
                f.close();
                f.open((filename+intToStr(++separateCounter)).c_str());
            }
        }
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionExport_Mesh_triggered(bool checked){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string filename = FileDialogs::getSaveFileName(this, "Export Scene (selected nodes or everything if no selection) as Mesh (SMF Format)", "SMF Format (*.smf)", NULL, "", FileDialogs::EXPORT_DIR);
    if( filename == "" ) return;
    if (extractFileExt(filename) != ".smf") {
        filename = filename + std::string(".smf");
    }

    bool separate = false;
    if (QMessageBox::question(this, "export option", "Do you want to export every mesh in a separate file?", 
        QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes) {
            separate = true;                          
    }

    std::ofstream f(filename.c_str());
    card32 separateCounter = 0;

    std::set<std::string> selection = scene->getRootState()->staticState->selectedNodes;
    if (selection.empty()) {
        SGLegacyPCSceneAdaptor sceneLA(scene);
        for (card32 pcnum = 0; pcnum < sceneLA.getNumPointClouds(); ++pcnum) {
            PointCloud* pc = sceneLA.getPointCloud(pcnum);
            MeshExporter exporter(pc, &f);
            exporter.exportPointCloud();
            f << endl;
            if (separate) {
                f.close();
                f.open((filename+intToStr(++separateCounter)).c_str());
            }
        }
    } else {
        std::set<std::string>::iterator pos = selection.begin();
        std::set<std::string>::iterator end = selection.end();
        PointCloud *pc = getPointCloud(scene, *pos);
        if (pc != NULL) {
            MeshExporter exporter(pc, &f);
            exporter.exportPointCloud();
            f << endl;
            if (separate) {
                f.close();
                f.open((filename+intToStr(++separateCounter)).c_str());
            }
        }
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionExport_Animation_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string filename = FileDialogs::getSaveFileName(this, "Export all point clouds in file sequence", "Canonical Ascii Format (*.points)", NULL, "", FileDialogs::EXPORT_DIR);

    if( filename == "" ) return;

    SGLegacyPCSceneAdaptor sceneLA(scene);

    for (card32 pcnum = 0; pcnum < sceneLA.getNumPointClouds(); ++pcnum) {
        std::string nextFile = filename + qString2STLString(QString::number(pcnum)) + std::string(".points");

        std::ofstream f(nextFile.c_str());

        PointCloud* pc = sceneLA.getPointCloud(pcnum);
        PointCloudExporter exporter(pc, &f);
        exporter.exportPointCloud();
        f << endl;
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionE_xit_triggered(bool checked) {
    close();
}


// ----- "edit" actions -----


void SceneEditorWidget::on_actionCommandHistory_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        cmdHistoryForm->updateEditor();
    cmdHistoryForm->show();	 
    STD_GLOBAL_EXCEPTION_HANDLER_END
}


void SceneEditorWidget::on_actionUndo_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CommandHistory *script= scene->getCommandHistory();
    script->undo(scene);
    ObjectViewsTable::update(script);
    ObjectViewsTable::update(scene);
    updateMenu();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionRedo_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CommandHistory *script = scene->getCommandHistory();
    script->redo(scene);
    ObjectViewsTable::update(script);
    ObjectViewsTable::update(scene);
    updateMenu();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionRepeat_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CommandHistory *script = scene->getCommandHistory();
    script->repeatLatest(scene);
    ObjectViewsTable::update(script);
    ObjectViewsTable::update(scene);
    updateMenu();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionDelete_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CmdObjPointCloudOperations *newCmd = new CmdObjPointCloudOperations();
    newCmd->setup(0);
    scene->getCommandHistory()->executeCommand(scene, newCmd, true);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionCopy_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CmdObjPointCloudOperations *newCmd = new CmdObjPointCloudOperations();
    newCmd->setup(1);
    scene->getCommandHistory()->executeCommand(scene, newCmd, true);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionSplit_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CmdObjPointCloudOperations *newCmd = new CmdObjPointCloudOperations();
    newCmd->setup(2);
    scene->getCommandHistory()->executeCommand(scene, newCmd, true);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionJoin_Clouds_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CmdObjPointCloudOperations *newCmd = new CmdObjPointCloudOperations();
    newCmd->setup(3);
    scene->getCommandHistory()->executeCommand(scene, newCmd, true);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionDuplicate_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CmdObjPointCloudOperations *newCmd = new CmdObjPointCloudOperations();
    newCmd->setup(4);
    scene->getCommandHistory()->executeCommand(scene, newCmd, true);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionSplit_to_single_cloud_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        CmdObjPointCloudOperations *newCmd = new CmdObjPointCloudOperations();
    newCmd->setup(5);
    scene->getCommandHistory()->executeCommand(scene, newCmd, true);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}


// ----- "view" actions -----

void SceneEditorWidget::on_actionReset_Layout_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START

        ui.dockWidgetSceneGraphState->setVisible(false);
    ui.dockWidgetScene->setVisible(true);
    ui.dockWidgetTools->setVisible(true);
    ui.dockWidgetCommands->setVisible(true);
    ui.dockWidgetRenderer->setVisible(true);
    ui.dockWidgetDebug->setVisible(false);
    tabifyDockWidget( ui.dockWidgetScene, ui.dockWidgetTools );
    tabifyDockWidget( ui.dockWidgetScene, ui.dockWidgetCommands );
    tabifyDockWidget( ui.dockWidgetScene, ui.dockWidgetRenderer );
    ui.dockWidgetScene->raise();

    STD_GLOBAL_EXCEPTION_HANDLER_END
}
void SceneEditorWidget::on_actionSafe_Layout_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        //	QByteArray ba = QWidget::saveGeometry();
        //	editorOptions->setGeometryState( makeDVector( (card8*)ba.begin(), (card8*)ba.end() ) );
        //	ba = QMainWindow::saveState();
        //	editorOptions->setWindowState( makeDVector( (card8*)ba.begin(), (card8*)ba.end() ) );

        QByteArray ba = QWidget::saveGeometry();
    BinaryAttachment* att = new BinaryAttachment;
    att->setup( "GeometryState", BinaryAttachment::ADF_PERSISTENT );
    att->setData( std::string( ba.begin(), ba.end() ) );
    Configuration::getConfiguration()->attachData( att );

    ba = QMainWindow::saveState();
    att = new BinaryAttachment;
    att->setup( "DockerState", BinaryAttachment::ADF_PERSISTENT );
    att->setData( std::string( ba.begin(), ba.end() ) );
    Configuration::getConfiguration()->attachData( att );
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

QByteArray stringToByteArray( std::string s ) {
    if( s.empty() )
        return QByteArray();

    return QByteArray(&*s.begin(), (int) s.size());
}

void SceneEditorWidget::on_actionRestore_Layout_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        BinaryAttachment* att = dynamic_cast<BinaryAttachment*>( Configuration::getConfiguration()->getData("GeometryState") );
    if( att )
        QWidget::restoreGeometry( stringToByteArray( att->getDataAsString() ) );

    att = dynamic_cast<BinaryAttachment*>( Configuration::getConfiguration()->getData("DockerState") );
    if( att )
        QMainWindow::restoreState( stringToByteArray( att->getDataAsString() ) );
    //	if( editorOptions->getWindowState().size() > 0 )
    //		QMainWindow::restoreState( QByteArray(reinterpret_cast<char*>( editorOptions->getWindowState().begin() ), (int)editorOptions->getWindowState().size() ) );
    STD_GLOBAL_EXCEPTION_HANDLER_END
}


// ----- "extras" actions -----

void SceneEditorWidget::on_actionMovieStudio_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        movieStudio->show();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionMacrosAndScripts_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        macroScriptEditor->update();
    macroScriptEditor->show();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_action256_x_256_triggered( bool checked ) 
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        this->changeGlWindowResolution( 256,256 );
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_action400_x_300_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        this->changeGlWindowResolution(400,300);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_action512_x_512_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        this->changeGlWindowResolution(512,512);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_action640_x_480_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        this->changeGlWindowResolution(640,480);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_action1024_x_768_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        this->changeGlWindowResolution(1024,768);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_action1280_x_720_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        this->changeGlWindowResolution(1280,720);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_action1280_x_960_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        this->changeGlWindowResolution(1280,960);
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_action_Auto_Layout_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        QRect screen = QApplication::desktop()->availableGeometry(this);
    move(0,0);
    this->changeGlWindowResolution(1024,768);
    if (screen.width() > screen.height()) {
        resize(screen.width()-400, screen.height()-30);
        x4console.moveConsoleWindow(screen.width()-400, 0);
        x4console.resizeConsoleWindow(400, screen.height()-30);
    } else {
        resize(screen.width()-10, screen.height()-400);
        x4console.moveConsoleWindow(0, screen.height()-440);
        x4console.resizeConsoleWindow(screen.width()-10, 400);
    }
    
    x4console.show();
    this->activateWindow();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}


// ----- "help" actions -----

void SceneEditorWidget::on_actionContext_Help_triggered(bool checked) {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string className = "";
    if( ui.dockWidgetScene->hasFocus() ) {
        //	if (ui.tabWidget->currentIndex() == 0) {
        if (sceneEditor != NULL) {
            className = sceneEditor->getInstanceClass()->getClassName();
        }
    } else if( ui.dockWidgetTools->hasFocus() ) {
        //	} else if (ui.tabWidget->currentIndex() ==  1) {
        if (currentTool != NULL) {
            className = currentTool->getInstanceClass()->getClassName();
        }
    } else if( ui.dockWidgetCommands->hasFocus() ) {
        //	} else if (ui.tabWidget->currentIndex() ==  2) {
        if (currentCommand != NULL) {
            className = currentCommand->getInstanceClass()->getClassName();        
        }
    } else if( ui.dockWidgetRenderer->hasFocus() ) {
        //	} else if (ui.tabWidget->currentIndex() ==  3) {
        if (currentRenderer != NULL) {
            className = currentRenderer->getInstanceClass()->getClassName();
        }
    }

    std::string topic;
    if (className == "") {
        topic = "about.html";
    } else {
        topic = className + std::string(".html");
    }
    OnlineHelp::newTopic(topic);
    OnlineHelp::show();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}


void SceneEditorWidget::changeGlWindowResolution(int32 x, int32 y){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        QSize actualRes= ui.mainGLArea->size();
    QSize newRes= QSize(x,y);
    if (actualRes!=newRes)
    {
        QSize diff= newRes-actualRes;
        QSize newWindowSize= this->size()+ diff;
        this->resize(newWindowSize);
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltSwitchCameraMode() {
    STD_GLOBAL_EXCEPTION_HANDLER_START
        force_cam=!force_cam;

    if (camTool == NULL)
    {
        const MetaClass *newToolClass = PCICameraController::getClass();
        camTool = (PCInteractionTool*)(newToolClass->newInstance());
        camTool->connectToScene(scene, new SceneEditorWidget_OnSceneChangeCallback(this), this, NULL);
        camTool->activateTool(); 
        camTool->areaResize(ui.mainGLArea->width(), ui.mainGLArea->height());
        camTool->areaResize_base(ui.mainGLArea->width(), ui.mainGLArea->height());
        oldTools.insertObject( camTool );
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

/*void SceneEditorWidget::on_viewButton_clicked(){
sltToolChanged("PCICameraController");
ui.tabWidget->setCurrentIndex(1);

}
void SceneEditorWidget::on_selectButton_clicked(){
sltToolChanged("PCISelectionTool");
ui.tabWidget->setCurrentIndex(1);

}
void SceneEditorWidget::on_drawButton_clicked(){

ui.tabWidget->setCurrentIndex(1);
sltToolChanged("PCIDrawingTool");
}

void SceneEditorWidget::on_transButton_clicked(){

ui.tabWidget->setCurrentIndex(1);
sltToolChanged("PCITransfEdit");
}*/

void SceneEditorWidget::sltAnimationBarFrameChanged(int time){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        scene->getRootState()->dynamicState->time = time;
    //std::cout << time <<" ";
    PCICameraController *camCtrl = dynamic_cast<PCICameraController*>(currentTool);
    if (camCtrl != NULL) {
        if (camCtrl->getCurrentController() != NULL) {
            camCtrl->getCurrentController()->setTime(time);
        }
    }
    PCICameraManager * camManagerTool = dynamic_cast<PCICameraManager*>(currentTool);
    if( camManagerTool )
        camManagerTool->setTime(time);
    ui.mainGLArea->getXrtGLWidget()->repaint();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltAnimationBarTraversalModeChanged(SGStaticState::TraversalModeType mode){
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if (mode != scene->getRootState()->staticState->defaultTraversalMode) {
            CmdObjSGChangeDefaultTraversalMode *cmd = new CmdObjSGChangeDefaultTraversalMode(mode);
            scene->getCommandHistory()->executeCommand(scene, cmd);
        };
        updateMenu(); // for edit/undo displayed correctly
        ObjectViewsTable::update(scene->getRootState()->staticState);
        STD_GLOBAL_EXCEPTION_HANDLER_END
}

AnimationBarWidget* SceneEditorWidget::getAnimationBar(){
    return animationBar;
}

void SceneEditorWidget::recordMovieNow(const char* rawFileName, float32 minTime, float32 maxTime, float32 timeStep, bool setSGTime,bool doRecord)
{
    STD_GLOBAL_EXCEPTION_HANDLER_START

        bool doRangeScans=false;


    if ( animationBar->getFramesCount() > 10000) {
        QMessageBox::information(this,  "Export canceled", "Time Settings inconsistent or more than 10000 frames requested.");
        return;
    }
    for (float32 t=minTime; t<= maxTime; t += timeStep) {
        if (setSGTime) animationBar->setFrameNumber(t);//  TBAnimPos->Position = (int)t;
        sltGLRepaint();//   mainGLAreaPaint(this);

        if (doRangeScans){
            if(true){
                PointCloud* pc= getPointCloud(scene, "root/pc1");
                Matrix4f m= pc->getTransformation();
                pc->setTransformation( makeRotY4f(t* M_PI*0.1f) );


            }
            std::string temp="";
            if (t<10) temp="0";

            rangeScan("frame" + temp + intToStr(t));
            inContinuousRenderingLoop = true;
            continue;
        }

        if (doRecord) 
        {
            if( movieStudio->getOversampling() == 0)
            {
                /*EImageFormat format = IF_PNG;
                QString ext = ".png";
                switch( movieStudio->getImageFormat() )
                {
                case 0:
                format = IF_PNG;
                ext = ".png";
                break;
                case 1:
                format = IF_JPG;
                ext = ".jpg";
                break;
                case 2:
                format = IF_BMP;
                ext = ".bmp";
                break;
                }
                saveScreenShot( (QString( rawFileName ) + "_" + QString::number( t ) + ext).toLatin1().constData(),
                format, 85 );*/
                saveTiledScreenshot(1,1);
            }
            else
            {
                switch(movieStudio->getOversampling())
                {
                case 1:
                    saveTiledScreenshot(1,2);
                    break;
                case 2:
                    saveTiledScreenshot(1,4);
                    break;
                case 3:
                    saveTiledScreenshot(1,8);
                    break;
                }

            }
        }
        inContinuousRenderingLoop = true;
    }
    inContinuousRenderingLoop = false;
    STD_GLOBAL_EXCEPTION_HANDLER_END
}


void SceneEditorWidget::saveFrameBMP( const QString& filename )
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        QImage image = mainGLArea->grabFrameBuffer( true );
    image.save( filename, "BMP" );

    /* --- OLD VERSION ---
    Timer bmpWriteTimer;
    card32 numFrames = 1;
    if (movieStudio->cbMatchRealTime->Checked) {
    int32 compTime = (int32)lastRenderingTime;
    //if (inContinuousRenderingLoop) {
    //   compTime -= (int32)lastBMPWriteTime;
    //}
    int32 effCompTime = compTime + missingMilliseconds;
    if (effCompTime < 0) effCompTime = 0;
    numFrames = (effCompTime * movieStudio->ieFrameRate->IntValue) / 1000;
    if ((int)numFrames > movieStudio->ieMaxRepl->IntValue) numFrames = movieStudio->ieMaxRepl->IntValue;
    double timeAccountedForInMS = (double)numFrames * (1000.0 / (double)(movieStudio->ieFrameRate->IntValue));
    missingMilliseconds += compTime;
    missingMilliseconds -= (double)timeAccountedForInMS;
    debugOutput << " " << missingMilliseconds;
    }

    for (card32 frameCount=0; frameCount<numFrames; frameCount++) {
    AnsiString thisFileName;
    do {
    AnsiString path = ExtractFilePath(rawFileName);
    AnsiString filename = ExtractFileName(rawFileName);
    AnsiString extension = ExtractFileExt(rawFileName);
    if (extension.Length() > 0) {
    filename.Delete(filename.Length() - extension.Length() + 1, extension.Length());
    }
    extension = ".bmp";
    filename = path + filename;

    AnsiString number = Format( "%d", ARRAYOFCONST( ((int)exportBMPFilesNumber) ) );
    AnsiString nulls = "";
    for (int j=6; j>number.Length(); j--) {
    nulls += "0";
    }
    exportBMPFilesNumber++;
    thisFileName = (filename + nulls + number + extension);
    } while (FileExists(thisFileName));

    TGLArea *glArea = mainGLArea;
    Graphics::TBitmap *b = new Graphics::TBitmap;
    b->Width = glArea->Width;
    b->Height = glArea->Height;

    // read from back buffer
    glArea->Activate();
    card32 w = glArea->Width;
    card32 h = glArea->Height;
    Vector4ub *pixels = new Vector4ub[w*h];
    glReadPixels(0, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
    glReadBuffer(GL_BACK);
    for (card32 y=0; y<h; y++) {
    for (card32 x=0; x<w; x++) {
    b->Canvas->Pixels[x][h-1-y] = (TColor)RGB((pixels[x+y*w])[0], (pixels[x+y*w])[1], (pixels[x+y*w])[2]);
    }
    }
    delete[] pixels;
    b->SaveToFile(thisFileName);
    delete b;
    }
    lastBMPWriteTime = bmpWriteTimer.getDeltaValue()
    */
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::initLocalVars()
{
    sceneEditor = NULL;
    sceneEditorControl = NULL;

    toolsEditor = NULL;
    toolsEditorControl = NULL;
    currentTool = NULL;

    cmdsEditor = NULL;
    cmdsEditorControl = NULL;
    currentCommand = NULL;

    rendererEditor = NULL;
    rendererEditorControl = NULL;
    currentRenderer = NULL;

    glContext = NULL;
    mainGLArea = ui.mainGLArea->getXrtGLWidget();
    glContext = mainGLArea->getContext();

    inContinuousRenderingLoop = false;
    force_cam = false;
    camTool=NULL;
}

void SceneEditorWidget::sltGLDropEvent( QMimeData * d, QPoint pos )
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        std::string pointCloud = getClickedCloud(pos.x(), pos.y(), mainGLArea->width(), mainGLArea->height(),
        &scene->getRootState()->staticState->camera, 
        &scene->getRootState()->staticState->viewFrustum, scene);

    if( pointCloud.length() > 0 )
    {
        PointCloud * pc = getPointCloud( scene, pointCloud );
        if( pc )
        {
            int * matIndex = (int *)d->data("X4/material").constData();
            if( *matIndex >= 0 )
                pc->setMaterialIndex( *matIndex );
        }
    }
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

SceneInterface * SceneEditorWidget::getSceneInterface()
{
    SceneInterface * sceneInterface = NULL;
    STD_GLOBAL_EXCEPTION_HANDLER_START
        AttachedData * data = Configuration::getConfiguration()->getData( "Configuration" );

    if( data == NULL )
    {
        sceneInterface = new SceneInterface;

        SceneToolbar * main_tb = new SceneToolbar;
        main_tb->setName( "main" );
        sceneInterface->addToolbar( main_tb );

        sceneInterface->setup( "Configuration", AttachedData::ADF_PERSISTENT );
        Configuration::getConfiguration()->attachData( sceneInterface );
    }
    else
        sceneInterface = dynamic_cast<SceneInterface*>(data);

    x4Assert( sceneInterface != NULL );

    STD_GLOBAL_EXCEPTION_HANDLER_END
        return sceneInterface;
}

void SceneEditorWidget::buildToolBar()
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        SceneInterface * sceneInterface = getSceneInterface();
    sceneInterface->createInterface( this );
    sceneInterface->buildKeyMap( this );
    m_InterfaceVersion = sceneInterface->getVersion();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

bool SceneEditorWidget::processToolbarKeyEvents( QKeyEvent * e )
{
    bool ret = false;
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if( toolBarShortCuts.find( e->key() ) != toolBarShortCuts.end() )
        {
            int32 modifiers = 0;
            if( e->modifiers() & Qt::ALT )
                modifiers |= SceneToolbarElement::SCM_Alt;
            if( e->modifiers() & Qt::CTRL )
                modifiers |= SceneToolbarElement::SCM_Ctrl;
            if( e->modifiers() & Qt::SHIFT )
                modifiers |= SceneToolbarElement::SCM_Shift;
            for( unsigned int i=0;i<toolBarShortCuts[e->key()].size();i++ )
            {
                if( toolBarShortCuts[e->key()][i]->getShortCutModifiers() == modifiers )
                {
                    if( ret )
                        debugOutput << "Ambigous short cut: " << toolBarShortCuts[e->key()][i]->getName() << "\n";
                    else
                    {
                        toolBarShortCuts[e->key()][i]->keyEvent( this );
                        ret = true;
                    }
                }
            }
        }
        STD_GLOBAL_EXCEPTION_HANDLER_END
            return ret;
}

void SceneEditorWidget::on_pbAddTool_clicked()
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if( ui.toolSelector->currentText().isEmpty() )
            return;

    std::string className = ui.toolSelector->currentText().toLatin1().constData();
    if( className.find("PCI") != 0 )
        className.insert(0,"PCI");
    addToToolbar( className, true, true );
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_pbAddCommand_clicked()
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if( ui.commandSelector->currentText().isEmpty() )
            return;

    std::string className = ui.commandSelector->currentText().toLatin1().constData();
    if( className.find("PCC") != 0 )
        className.insert(0,"PCC");
    addToToolbar( className, true, true );
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::clearToolBarsVec()
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        for( card32 i=0;i<toolBars.size();i++ )
            delete toolBars[i];
    toolBars.clear();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::addToToolbar( std::string className, bool et_object, bool et_command )
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        const MetaClass * mc = MetaClass::getClassByName(className);

    if( mc->isAbstract() )
        return;

    addToToolbarDlg->setup( className, et_object, et_command, AddToToolbarWidget::Class );
    if( addToToolbarDlg->exec() == QDialog::Accepted )
    {
        SceneInterface * si = getSceneInterface();
        if( addToToolbarDlg->getEventType() == AddToToolbarWidget::Class )
        {
            ((SceneToolbar*)si->getToolbar(0))->addToolButton( className, addToToolbarDlg->getIcon(), addToToolbarDlg->getTooltip(),
                addToToolbarDlg->getShortDesc() );
            ((SceneToolbar*)si->getToolbar(0))->setLastElement_ShortCut( addToToolbarDlg->getShortCutKey(),
                addToToolbarDlg->getShortCutModifer() );
        }
        if( addToToolbarDlg->getEventType() == AddToToolbarWidget::Object )
        {
            const MetaClass * mc = MetaClass::getClassByName( className );
            Object * obj = NULL;
            if( mc->inheritsFrom( PCInteractionTool::getClass() ) )
                obj = currentTool->copy();
            if( mc->inheritsFrom( PCCommand::getClass() ) )
                obj = currentCommand->copy();

            if( obj != NULL )
            {
                ((SceneToolbar*)si->getToolbar(0))->addToolButton( obj, addToToolbarDlg->getIcon(), addToToolbarDlg->getTooltip(),
                    addToToolbarDlg->getShortDesc() );

                ((SceneToolbar*)si->getToolbar(0))->setLastElement_ShortCut( addToToolbarDlg->getShortCutKey(),
                    addToToolbarDlg->getShortCutModifer() );
            }
        }
    }
    buildToolBar();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::processEvent( XGRTEvent * e )
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        // UGLY FIXME HACK DO_SOMETHING

        if( dynamic_cast<ModeChangeEvent*>(e) )
        {
            ModeChangeEvent * modechange = (ModeChangeEvent*)e;
            const MetaClass * mc = MetaClass::getClassByName( modechange->getModeClassName() );
            if( mc )
            {
                x4Assert( !mc->isAbstract() );
                if( mc->inheritsFrom( SceneClassEditor::getClass() ) )
                {
                    sltSceneEditChanged( modechange->getModeClassName().c_str() );
                    //					ui.tabWidget->setCurrentIndex(0);
                    ui.dockWidgetScene->raise();
                }

                if( mc->inheritsFrom( PCInteractionTool::getClass() ) )
                {
                    if( mc != currentTool->getClass() )
                        sltToolChanged( modechange->getModeClassName().c_str() );
                    ui.dockWidgetTools->raise();
                    //					ui.tabWidget->setCurrentIndex(1);
                }

                if( mc->inheritsFrom( PCCommand::getClass() ) )
                {
                    if( mc != currentCommand->getClass() )
                        sltCommandChanged( modechange->getModeClassName().c_str() );
                    ui.dockWidgetCommands->raise();
                    //					ui.tabWidget->setCurrentIndex(2);
                }

            }
        }

        if( dynamic_cast<SpecifiedModeChangeEvent*>(e) )
        {
            SpecifiedModeChangeEvent* modechange = (SpecifiedModeChangeEvent*)e;
            const MetaClass * mc = modechange->getObject()->getInstanceClass();
            if( mc )
            {
                if( mc->inheritsFrom( PCInteractionTool::getClass() ) )
                {
                    changeTool( mc, dynamic_cast<PCInteractionTool*>(modechange->getObject()) );
                    //					ui.tabWidget->setCurrentIndex(1);
                    ui.dockWidgetTools->raise();
                }

                if( mc->inheritsFrom( PCCommand::getClass() ) )
                {
                    changeCommand( mc, dynamic_cast<PCCommand*>(modechange->getObject()) );
                    //					ui.tabWidget->setCurrentIndex(2);
                    ui.dockWidgetCommands->raise();
                }
            }
        }

        if( dynamic_cast<StringEvent*>(e) )
        {
            StringEvent * str = (StringEvent*)e;

            if( str->getString() == "FORCE_CAMERA" )
            {
                sltSwitchCameraMode();
            }
        }
        STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltToolbarHandler( QAction * action )
{
    x4Assert( toolBarEvents.find(action) != toolBarEvents.end() );
    processEvent( toolBarEvents[action] );
}

void SceneEditorWidget::on_actionAdd_Quick_Cam_Checkbox_triggered()
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        SceneInterface * si = getSceneInterface();
    SceneToolbarCheckBox * element = new SceneToolbarCheckBox;
    StringEvent * e = new StringEvent;
    e->setString( "FORCE_CAMERA" );
    element->setup( e, "cam", "enable camera user-input without switching to camera-editor" );
    ((SceneToolbar*)si->getToolbar( 0 ))->addElement( element );
    ((SceneToolbar*)si->getToolbar( 0 ))->setLastElement_ShortCut( Qt::Key_Space );
    buildToolBar();
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::on_actionEditor_Options_triggered()
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        if( editorOptionsDlg == NULL )
        {
            editorOptionsDlg = new QDialog(this);
            SceneEditorOptionsEditor * ce = new SceneEditorOptionsEditor;
            QWidget * w = ce->makeEditor( editorOptionsDlg, editorOptions );

            QHBoxLayout * layout = new QHBoxLayout(editorOptionsDlg);
            layout->addWidget( w );
            layout->setMargin(1);
            layout->setSpacing(1);
        }

        editorOptionsDlg->exec();
        STD_GLOBAL_EXCEPTION_HANDLER_END
}


void SceneEditorWidget::saveScreenShot( std::string filename, EImageFormat format, int quality )
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        QImage image = mainGLArea->grabFrameBuffer();
    QString qformat = "BMP";
    switch( format )
    {
    case IF_PNG:
        qformat = "PNG";
        break;
    case IF_JPG:
        qformat = "JPEG";
        break;
    }
    image.save( filename.c_str(), qformat.toLatin1().constData(), quality );
    debugOutput << "Saved screenshot to " << filename << " .\n";
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltScreenShot()
{
    STD_GLOBAL_EXCEPTION_HANDLER_START
        QString num = QString::number( m_ScreenShotID++ );
    while( num.length() < 5 )
        num = QString("0") + num;
    std::string filename = m_ScreenShotBaseName + num.toLatin1().constData() + ".";
    switch( m_ScreenShotImageFormat )
    {
    case IF_BMP:
        filename += "bmp";
        break;
    case IF_PNG:
        filename += "png";
        break;
    case IF_JPG:
        filename += "jpg";
        break;
    }

    saveScreenShot( filename, m_ScreenShotImageFormat );
    STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneEditorWidget::sltToolbarCheckBoxHandler( bool checked, char * data )
{
    XGRTEvent * e = (XGRTEvent *)data;
    processEvent( e );
}

void SceneEditorWidget::on_actionExport_Bitmap_triggered(bool checked)
{
    saveScreenShot( "c:/test.bmp", IF_BMP );
}


#include "SceneEditorWidget_moc.h"

void SceneEditorWidget::saveFrame() 
{
    if( movieStudio->getOversampling() != 0)
    {
        saveTiledScreenshot(1, movieStudio->getOversampling());
        return;
    }

    m_ScreenShotBaseName = movieStudio->getFileNameString();
    m_ScreenShotImageFormat = IF_PNG;
    switch( movieStudio->getImageFormat() )
    {
    case 0:
        m_ScreenShotImageFormat = IF_PNG;
        break;
    case 1:
        m_ScreenShotImageFormat = IF_JPG;
        break;
    case 2:
        m_ScreenShotImageFormat = IF_BMP;
        break;
    }
    sltScreenShot();
}

void SceneEditorWidget::on_actionScreenshot_triggered() 
{
    std::string tmp = m_ScreenShotBaseName;
    m_ScreenShotBaseName = movieStudio->getFileNameString();
    sltScreenShot();
    m_ScreenShotBaseName = tmp;
}

void SceneEditorWidget::on_actionRangescan_triggered(bool checked) 
{
    rangeScan( "test" );
}

void SceneEditorWidget::rangeScan( std::string pcName ) 
{
    QImage image = mainGLArea->grabFrameBuffer();
    glContext->activate();
    card32 resX = mainGLArea->width();
    card32 resY = mainGLArea->height();
    std::vector<float> zBuffer(resX*resY,0);
    glReadPixels(0,0,resX,resY,GL_DEPTH_COMPONENT,GL_FLOAT,&zBuffer[0]);

    double mv[16], proj[16];
    int vp[4];
    glGetDoublev( GL_MODELVIEW_MATRIX, mv);
    glGetDoublev( GL_PROJECTION_MATRIX, proj );
    glGetIntegerv( GL_VIEWPORT, vp );

    Vector3f right = makeVector3f(1,0,0);
    Vector3f up = makeVector3f(0,1,0);
    Vector3f dir = makeVector3f(0,0,1);

    VertexDescriptor vd;
    vd.pushAttrib( VAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
    vd.pushAttrib( VAD( "flags", 1, VAD::DATA_FORMAT_INT32 ) );
    vd.pushAttrib( VAD( "normal", 3, VAD::DATA_FORMAT_FLOAT32 ) );
    vd.pushAttrib( VAD( "color", 3, VAD::DATA_FORMAT_FLOAT32 ) );
    UnstructuredInCorePointCloud * upc = new UnstructuredInCorePointCloud;
    upc->clearAndSetup( &vd, 0 );
    PointSet * ps = upc->getPointSet();

    std::vector<char> buffer( vd.getSize(),0);
    AAT posAAT = ps->getAAT( "position" );
    AAT colorAAT = ps->getAAT( "color" );

    for( int x=0;x<(int)resX;x++ )
        for( int y=0;y<(int)resY;y++ )
            if( zBuffer[y*resX+x] < 1.0f)
            {

                Vector3f pos;
                double tmpPos[3];
                float z = zBuffer[y*resX+x];
                gluUnProject( x,y, z, mv, proj, vp, &tmpPos[0], &tmpPos[1], &tmpPos[2] );
                pos = makeVector3f( tmpPos[0],tmpPos[1],tmpPos[2] );

                QRgb col = image.pixel(x,image.height() - 1 - y);
                Vector3f col3f = makeVector3f( qRed( col ), qGreen( col ), qBlue( col ) )  / 255.0f;

                ps->addPoint( &buffer[0] );
                ps->set3f( ps->getNumEntries()-1, posAAT, pos );
                ps->set3f( ps->getNumEntries()-1, colorAAT, col3f );
            }


            glContext->deactivate();

            if( ps->getNumEntries() ){
                SceneGraphNode* scannedCloud=addSceneObject( scene, upc, pcName );
                scannedCloud->setVisible(false);
            }
}

void SceneEditorWidget::repaintGLArea() 
{
    mainGLArea->repaint();
}


static bool setglresto640x480  =false;
static bool startbench = false;
void SceneEditorWidget::timerEvent( QTimerEvent * e) 
{
    // HACK! OOC-test
    /*
    if( !setglresto640x480 )
    {
    on_action640_x_480_triggered(true);
    setglresto640x480 = true;	
    }

    if( !startbench && wait3secondsTimer->elapsed() > 5000 )
    {
    startbench = true;
    sltToolChanged("PCIBenchTool");
    repaintGLArea();
    }
    */

    if( redrawDelay != editorOptions->getRedrawDelay() )
    {
        redrawDelay = editorOptions->getRedrawDelay();
        if( editorOptions->getRedrawDelay() != -1)
        {
            killTimer(timerID);
            timerID = startTimer( redrawDelay );
        }

    }

    if( redrawDelay >= 0)
        mainGLArea->updateGL();
}

void SceneEditorWidget::executeCommandClass(const MetaClass *cmd) {
    changeCommand(cmd);
    if (currentCommand->getInstanceClass() == cmd) {
        sltExecCommand();
    } else {
        throw ERangeCheck(std::string("SceneEditorWidget::executeCommandClass() - no command of type \"")
            + cmd->getClassName()+std::string("\" is available."));
    }
}

void SceneEditorWidget::on_actionSet_GSA_OOC_Cache_Size_triggered(bool checked) {
    mpcard v = GenericSwapAllOutOfCorePointCloud::getMaxMem() / (1024*1024);
    if (SetGSAOOCCacheSizeDialog::getValue(this, v)) {
        GenericSwapAllOutOfCorePointCloud::setMaxMem(v*1024*1024);
    }
}

void SceneEditorWidget::on_actionBackgroundBlack_triggered() 
{
    scene->getRootState()->staticState->backgroundColor = makeVector4f(0.0f, 0.0f, 0.0f, 1.0f);
    mainGLArea->updateGL();
}

void SceneEditorWidget::on_actionBackgroundDark_triggered() 
{
    scene->getRootState()->staticState->backgroundColor = makeVector4f(30/255.0f, 40/255.0f, 70/255.0f, 1.0f);
    mainGLArea->updateGL();
}

void SceneEditorWidget::on_actionBackgroundBright_triggered() 
{
    scene->getRootState()->staticState->backgroundColor = makeVector4f(0.6f, 0.6f, 0.9f, 1.0f);
    mainGLArea->updateGL();
}

void SceneEditorWidget::on_actionBackgroundWhite_triggered() 
{
    scene->getRootState()->staticState->backgroundColor = makeVector4f(1.0f, 1.0f, 1.0f, 1.0f);
    mainGLArea->updateGL();
}

void X4::SceneEditorWidget::on_actionScreenshot_4000x4000_triggered()
{
    m_RecordHighQualityScreenShot = true;
    mainGLArea->updateGL();
}


/**
create a high resolution screenshot with oversampling by taking multiple screenshots
*/
void X4::SceneEditorWidget::on_actionScreenshot_Tiles_triggered()
{
    m_RecordHighQualityScreenShot = false;	// just in case "someone" wants to push it 10 times over the limit :-)
    bool ok;
    static unsigned lastNumTiles=1;
    static unsigned lastNumOverSampling=0;

    unsigned numTiles = (unsigned)QInputDialog::getInteger(this,"num tiles", "Number of tiles for output image [1,2,3,4]",lastNumTiles, 1, 8,1,&ok);
    if( !ok )
        return;

    unsigned numOverSampling = (unsigned)QInputDialog::getInteger(this,"oversampling", "Oversampling [0,1,2,3] (0 = 1x1, 3 = 8x8)",lastNumOverSampling, 0, 3,1,&ok);
    if( !ok )
        return;

    lastNumOverSampling = numOverSampling;
    lastNumTiles = numTiles;
    numOverSampling = 1 << numOverSampling;

    /*if( numTiles == 1 && numOverSampling == 1 )
    {
    QMessageBox::information(this,"information","Please use the standard screenshot method.");
    return;
    }*/

    saveTiledScreenshot(numTiles,numOverSampling);
}

void X4::SceneEditorWidget::saveTiledScreenshot( int numTiles, int numOverSampling )
{
    const int oversamplingFrameX = numOverSampling - (mainGLArea->width() %  (numOverSampling>0 ? numOverSampling:1));
    const int oversamplingFrameY = numOverSampling - (mainGLArea->height() % (numOverSampling>0 ? numOverSampling:1));
    int screenWidth = mainGLArea->width()+ oversamplingFrameX % (numOverSampling>0?numOverSampling:1);
    int screenHeight = mainGLArea->height() + oversamplingFrameX % (numOverSampling>0?numOverSampling:1);

    RGBAFloat32Image accImage(screenWidth * numTiles, screenHeight * numTiles);
    accImage.clear( NULL_VECTOR4F );
    float samplePointRadius = m_SimulateLens_ConvRadius * scene->getRootState()->staticState->viewFrustum.getZNearClip();

    // compute sampling pattern
    UnstructuredInCorePointCloud * upcPattern = new UnstructuredInCorePointCloud;
    upcPattern->clearAndSetup(m_SimulateLens_NumSamples*10.0f,true,false,false,true);
    AAT posAAT = upcPattern->getAAT("position");
    for(unsigned i=0;i<upcPattern->getNumPoints();i++ )
    {
        while(true)
        {
            Vector2f p = makeVector2f(rnd01(),rnd01())*2.0f - makeVector2f(1,1);
            if( norm(p) <= 1.0f )
            {
                upcPattern->getPointSet()->set3f(i, posAAT, makeVector3f(p[0],p[1],0));
                break;
            }
        }
    }

    PCCResampler::resamplePC(upcPattern,0.45f * M_PI / sqrt((float)m_SimulateLens_NumSamples));

    std::vector<Vector2f> samplePoints;
    if( m_SimulateLens_NumSamples == 1)
        samplePoints.push_back( NULL_VECTOR2F);
    else
    {
        for( unsigned i=0;i<upcPattern->getNumPoints();i++ )
        {
            samplePoints.push_back( shrink3To2(upcPattern->getPointSet()->get3f(i,posAAT)) );
        }
    }
    delete upcPattern;


    m_SetupTiledImage = true;
    m_SimulateLens = true;
    GLRenderTextureFBO * renderTarget = new GLRenderTextureFBO(glContext);
    if( !renderTarget->create(screenWidth, screenHeight,4,GLRenderTextureFBO::FLAG_DEPTH) )
    {
        message("ERROR: Could not initialize offscreen render target!\n");
        delete renderTarget;
        return;
    }
    m_OffscreenRenderTarget = renderTarget;

    //TODO: this read is just a hack to set the right Opengl states 
    //(otherwise oversampling does not work on some machines and produces a black tile at the beginning).
    RGBAImage * rgba_image = m_OffscreenRenderTarget->getRGBAImage();
    delete rgba_image;

    for( unsigned i=0;i<samplePoints.size();i++ )
    {
        m_SimulateLens_Temp_SamplePoint = samplePoints[i] * samplePointRadius;

        QImage result_image( screenWidth * numTiles, screenHeight * numTiles, QImage::Format_RGB32 );

        for( int tx=0;tx<numTiles*numOverSampling;tx++ )
        {
            for( int ty=0;ty<numTiles*numOverSampling;ty++ )
            {
                m_TiledImage_X0 = makeVector2f(tx,ty)/(numTiles*numOverSampling);
                m_TiledImage_X1 = makeVector2f(tx+1,ty+1)/(numTiles*numOverSampling);

                renderTarget->activate();
                //mainGLArea->updateGL();
                sltGLRepaint();
                renderTarget->deactivate();
                //sltGLRepaint();
                //debugOutput.flush();

                //QImage qimage = mainGLArea->grabFrameBuffer();
                RGBAImage * rgba_image = m_OffscreenRenderTarget->getRGBAImage();

                if( numOverSampling > 1) {
                    RGBAImage * downsampledImage = new RGBAImage; 
                    downsampledImage->resize( screenWidth/numOverSampling, screenHeight/numOverSampling );
                    rgba_image->downsampleAA(numOverSampling, downsampledImage);
                    delete rgba_image;
                    rgba_image = downsampledImage;
                }

                //int startX = tx*screenWidth/numOverSampling;
                //int startY = ty*screenHeight/numOverSampling;
                int startX = tx*rgba_image->getSizeX();
                int startY = ty*rgba_image->getSizeY();
                for( int x=0;x < (int)rgba_image->getSizeX();x++ )
                {
                    for( int y=0;y < (int)rgba_image->getSizeY();y++ )
                    {
                        //result_image.setPixel(x+startX,startY + qimage->height()-1-y,qimage->pixel(x,y));
                        const Vector4ub rgba = rgba_image->getPixel(x,y);
                        result_image.setPixel(startX + x, startY + rgba_image->getSizeY()-1-y, qRgb(rgba[0],rgba[1],rgba[2]));
                    }
                }
                //delete qimage;
            }
        }

        for( int x=0;x<result_image.width();x++ )
        {
            for( int y=0;y<result_image.height();y++ )
            {
                QRgb p = result_image.pixel(x,y);
                Vector4f col = makeVector4f(qRed(p),qGreen(p),qBlue(p),0) / m_SimulateLens_NumSamples;
                accImage.setPixel(x,y, accImage.getPixel(x,y) + col);
            }
        }

    }
    m_SetupTiledImage = false;
    m_SimulateLens = false;
    delete renderTarget;
    m_OffscreenRenderTarget = NULL;
    currentRenderer->customEvent("SCREENSHOT_DONE");


    QString num = QString::number( m_ScreenShotID++ );
    while( num.length() < 5 )
        num = QString("0") + num;
    std::string filename = movieStudio->getFileNameString() + num.toLatin1().constData() + ".";

    QString qformat = "PNG";
    QImage save_image(screenWidth * numTiles, screenHeight * numTiles, QImage::Format_RGB32 );
    for( int x=0;x<save_image.width();x++ )
    {
        for( int y=0;y<save_image.height();y++ )
        {
            Vector4f col = accImage.getPixel(x,y);
            QRgb p = qRgb( col[0],col[1],col[2] );
            save_image.setPixel(x,y,p);
        }
    }
    save_image.save( (filename + qformat.toLatin1().constData()).c_str(), qformat.toLatin1().constData(), 85 );
    debugOutput << "saved high quality screenshot to: " << filename << "\n";

}

//DebugRendererEditorWidget *SceneEditorWidget::debugRendererEditorWidget = NULL;

void X4::SceneEditorWidget::on_actionDebugRenderer_triggered( bool checked )
{
    if( debugRendererEditorWidget == NULL )
    {
        debugRendererEditorWidget = new DebugRendererEditorWidget(this, Qt::Window);
        debugRendererEditorWidget->setup(debugRenderer, scene);
        debugRendererEditorWidget->show();
    }
    else
    {
        debugRendererEditorWidget->hide();
        debugRendererEditorWidget->show();
    }
}

void X4::SceneEditorWidget::on_actionReport_Statistics_triggered( bool checked )
{
    std::string report = scene->getStatistics()->outputAll();
    debugOutput << report;
}

void X4::SceneEditorWidget::on_pbLastUsedTool_clicked()
{
    luToolMenu->exec(QCursor::pos());
}

void X4::SceneEditorWidget::on_pbLastUsedCommand_clicked()
{
    luCommandMenu->exec(QCursor::pos());
}

void X4::SceneEditorWidget::sltLUTool( QAction * action )
{
    sltToolChanged( action->iconText());
}

void X4::SceneEditorWidget::sltLUCommand( QAction * action )
{
    sltCommandChanged( action->iconText());
}

void X4::SceneEditorWidget::on_pbRendererScreenshot_clicked()
{
    int numTiles = ui.cbRendererTiles->currentIndex() + 1;
    int oversampling = ui.cbRendererOversampling->currentIndex();
    if( numTiles == 1 && oversampling == 0 )
    {
        //on_actionScreenshot_triggered();
        saveTiledScreenshot(1,1);
    }
    else
    {
        saveTiledScreenshot(numTiles,1 << oversampling);
    }
}

bool X4::SceneEditorWidget::guessClickPoint( int x, int y, unsigned rad, Vector3f &ret )
{
    // read depth buffer
    glContext->activate();
    card32 resX = mainGLArea->width();
    card32 resY = mainGLArea->height();
    y = resY - 1 - y;
    int minPointX = max(0,x-(int)rad);
    int maxPointX = min((int)resX-1,x+(int)rad);
    int minPointY = max(0,y-(int)rad);
    int maxPointY = min((int)resY-1,y+(int)rad);
    int w = maxPointX - minPointX;
    int h = maxPointY - minPointY;
    if( w <= 0 || h <= 0)
        return false;
    std::vector<float> zBufferPart(w*h,0);
    glReadPixels(minPointX, minPointY,w,h,GL_DEPTH_COMPONENT,GL_FLOAT,&zBufferPart[0]);

    // read necessary matrices
    double mv[16], proj[16];
    int vp[4];
    glGetDoublev( GL_MODELVIEW_MATRIX, mv);
    glGetDoublev( GL_PROJECTION_MATRIX, proj );
    glGetIntegerv( GL_VIEWPORT, vp );
    glContext->deactivate();

    // gather all depth values that are not background
    std::vector<float> depths;
    for( unsigned i=0;i<zBufferPart.size();i++ )
    {
        if(  zBufferPart[i] < 1.0f )
        {
            depths.push_back(zBufferPart[i]);
        }
    }

    // return median as depth
    if( depths.size() == 0)
        return false;
    sort(depths.begin(),depths.end());
    float z = depths[depths.size()/2];
    GLdouble retX,retY,retZ;
    gluUnProject( x,y, z, mv, proj, vp, &retX, &retY, &retZ );
    ret = makeVector3f(retX,retY,retZ);
    return true;


}

bool SceneEditorWidget::estimateOOBBFromScreenArea( Vector2i x1, Vector2i x2, BoundingBox3f &bb, Matrix4f &worldToBB, float depthOutlierTolerance, Vector3f *firstAxis) 
{
    // read depth buffer
    glContext->activate();
    card32 resX = mainGLArea->width();
    card32 resY = mainGLArea->height();
    x1[1] = resY - 1 - x1[1];
    x2[1] = resY - 1 - x2[1];
    int minPointX = max(0,min(x1[0],x2[0]));
    int maxPointX = min((int)resX-1,max(x1[0],x2[0]));
    int minPointY = max(0,min(x1[1],x2[1]));
    int maxPointY = min((int)resY-1,max(x1[1],x2[1]));
    int w = maxPointX - minPointX;
    int h = maxPointY - minPointY;
    if( w <= 0 || h <= 0)
        return false;
    std::vector<float> zBufferPart(w*h,0);
    glReadPixels(minPointX, minPointY,w,h,GL_DEPTH_COMPONENT,GL_FLOAT,&zBufferPart[0]);

    // read necessary matrices
    double mv[16], proj[16];
    int vp[4];
    glGetDoublev( GL_MODELVIEW_MATRIX, mv);
    glGetDoublev( GL_PROJECTION_MATRIX, proj );
    glGetIntegerv( GL_VIEWPORT, vp );
    glContext->deactivate();

    // gather all points & points that are not background
    std::vector<float> depths;
    std::vector<Vector3f> points;
    for( unsigned i=0;i<zBufferPart.size();i++ )
    {
        if(  zBufferPart[i] < 1.0f )
        {
            depths.push_back(zBufferPart[i]);
            points.push_back( makeVector3f(	minPointX + (i%w),
                minPointY+(i/w),
                zBufferPart[i]
            ));
        }
    }

    // compute median depth
    if( depths.size() < 10)
        return false;
    sort(depths.begin(),depths.end());
    float z = depths[depths.size()/2];

    // create array of non-outlier points
    //debugRenderer->beginRenderJob_OneFrame("test rect",1);
    std::vector<Vector3f> pointsWS;
    for( mpcard i=0;i<points.size();i++ )
    {
        const Vector3f p = points[i];

        GLdouble retX,retY,retZ;
        gluUnProject( p[0], p[1], p[2], mv, proj, vp, &retX, &retY, &retZ );
        Vector3f pW = makeVector3f(retX,retY,retZ);

        if( fabs(p[2]-z) < depthOutlierTolerance )
        {
            pointsWS.push_back( pW );
            //debugRenderer->addPoint(pW,makeVector3f(1,0,0));
        }
        //else
        //	debugRenderer->addPoint(pW,makeVector3f(1,1,1)*0.4f);
    }

    if( pointsWS.size() < 10 )
        return false;

    // analyse frame
    if( firstAxis == NULL )
    {
        PCA<float,3> pca;
        for( mpcard i=0;i<pointsWS.size();i++ )
        {
            pca.addPoint(pointsWS[i]);
        }
        Matrix3f M;
        Vector3f v,centroid;
        pca.analyze(v,M,centroid);
        worldToBB = expand3To4(M.transpose())*makeTranslation4f(-centroid);
    }
    else
    {
        // first axis given, so we compute pca of the remaining 2d subspace
        // compute 3d centroid
        Vector3f centroid = NULL_VECTOR3F;
        for( mpcard i=0;i<pointsWS.size();i++ )
        {
            centroid += pointsWS[i];
        }
        centroid /= pointsWS.size();

        // transform points into tangent system & compute 2d pca in local frame
        Matrix3f tangentSystem = calcTangentSystem(*firstAxis);
        Matrix3f invTangentSystem = tangentSystem.transpose();
        PCA<float,2> pca;
        for( mpcard i=0;i<pointsWS.size();i++ )
        {
            Vector2f p2d = shrink3To2( invTangentSystem * (pointsWS[i]-centroid));
            pca.addPoint(p2d);
        }
        Matrix2f eigenVectors;
        Vector2f eigenValues,centroid2d;
        pca.analyze(eigenValues,eigenVectors,centroid2d);

        // compute frame
        Matrix3f frame;
        frame[0] = *firstAxis;
        frame[1] = tangentSystem[0]*eigenVectors[0][0] + tangentSystem[1]*eigenVectors[0][1];
        frame[2] = tangentSystem[0]*eigenVectors[1][0] + tangentSystem[1]*eigenVectors[1][1];
        worldToBB = expand3To4(frame.transpose())*makeTranslation4f(-centroid);
    }

    // compute BB in frame
    bb.lowerCorner = bb.upperCorner = NULL_VECTOR3F;
    for( mpcard i=0;i<pointsWS.size();i++ )
    {
        const Vector3f p_bb = transformVector3f(worldToBB,pointsWS[i]);
        bb.addPoint(p_bb);
    }

    //debugRenderer->addFineBoundingBox( invertFrame(worldToBB), bb, makeVector3f(1,1,1),false);

    //debugRenderer->endRenderJob();
    return true;

}


void X4::SceneEditorWidget::saveHighResFrame()
{
    m_RecordHighQualityScreenShot = true;
    mainGLArea->updateGL();
}

void X4::SceneEditorWidget::setEditorObjForCleanup( ClassEditor *ed )
{
	myObjectEditor = ed;
}

bool SceneEditorWidget::CallMeBack::cmdMethodCalled(std::string name) 
{
    return me->cmdMethodCalled(name);
}

bool SceneEditorWidget::CallMeBack::toolMethodCalled(std::string name) 
{
    return me->toolMethodCalled(name);
}

bool SceneEditorWidget::cmdMethodCalled(std::string name) 
{
    x4Assert(currentCommand);
    if (!currentCommand) return true;
    currentCommand->setScene(scene);
    const ClassMethod *cm = currentCommand->getInstanceClass()->getMethod(name);
    x4Assert(cm);
    if (!cm) return true;
    if (cm->getPackageAsCmdObjHint()) {
        CmdObjMemberMethod *cmdObj = new CmdObjMemberMethod();
        cmdObj->setup(name, currentCommand);
        // this is not always copied correctly...
        ((PCCommand*)(cmdObj->getObject()))->setScene(scene);
        scene->getCommandHistory()->executeCommand(scene, cmdObj);
        update();
        return false;
    } else {
        return true;
    }
}

bool SceneEditorWidget::toolMethodCalled(std::string name) 
{
    x4Assert(currentTool);
    if (!currentTool) return true;
    const ClassMethod *cm = currentTool->getInstanceClass()->getMethod(name);
    x4Assert(cm);
    if (!cm) return true;
    if (cm->getPackageAsCmdObjHint()) {
        CmdObjMemberMethod *cmdObj = new CmdObjMemberMethod();
        cmdObj->setup(name, currentTool);
        ((PCInteractionTool*)(cmdObj->getObject()))->connectToScene(scene, NULL, NULL, NULL);
        scene->getCommandHistory()->executeCommand(scene, cmdObj);
        update();
        return false;
    } else {
        return true;
    }
}

QPoint SceneEditorWidget::mapGLAreaToGlobal(const QPoint &p) {
    return mainGLArea->mapToGlobal(p);
}

void SceneEditorWidget::mouseDownPassToCam(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
    if (camTool) {
        camTool->mouseDown(x, y, buttonsState, modifiersState);
    }
}

void SceneEditorWidget::mouseMovedPassToCam(int32 x, int32 y) 
{
    if (camTool) {
        camTool->mouseMoved(x, y);
    }
}

void SceneEditorWidget::mouseUpPassToCam(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
    if (camTool) {
        camTool->mouseUp(x, y, buttonsState, modifiersState);
    }
}

void SceneEditorWidget::areaResizePassToCam(card32 width, card32 height) 
{
    if (camTool) {
        camTool->areaResize(width, height);
    }
}

void SceneEditorWidget::mouseWheelRotatedPassToCam(int32 rotatedDelta, Modifiers modifiersState) 
{
    if (camTool) {
        camTool->mouseWheelRotated(rotatedDelta, modifiersState);
    }
}

void SceneEditorWidget::keyDownPassToCam(GeneralKey key) 
{
    if (camTool) {
        camTool->keyDown(key);
    }
}

void SceneEditorWidget::keyUpPassToCam(GeneralKey key) 
{
    if (camTool) {
        camTool->keyUp(key);
    }
}

void X4::LastUsedMenu::addEntry( const QString &s )
{
    // search entry and remove
    for( unsigned i=0;i<m_Entries.size();i++ )
    {
        if( m_Entries[i] == s )
        {
            m_Entries.erase(m_Entries.begin() + i);
            break;
        }
    }

    // guarantee maxNumEntries
    while( m_Entries.size() >= m_MaxNumEntries)
    {
        m_Entries.pop_back();
    }

    m_Entries.insert(m_Entries.begin(),s);

    // rebuild menu
    clear();
    for( unsigned i=0;i<m_Entries.size();i++ )
    {
        addAction(m_Entries[i]);
    }

}

X4::LastUsedMenu::LastUsedMenu( QWidget * parent, unsigned maxNumEntries )
{
    m_MaxNumEntries = maxNumEntries;
}
