//---------------------------------------------------------------------------
#ifndef SceneEditorWidgetH 
#define SceneEditorWidgetH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "ClassEditor.h"
#include "x3editor\SceneEditorForm_ui.h"
#include "Renderer.h"
#include "ClassInstanceSet.h"
#include "GLContext.h"
#include "XrtGLWidget.h"
#include "AnimationBarWidget.h"
#include "StandAloneEditorWidget.h"
//---------------------------------------------------------------------------

namespace X4 {
class Scene;
class PCInteractionTool;
class PCCommand;
class CommandHistoryEditorWidget;
class MovieStudioWidget;
class MacroEditorWidget;
class SceneInterface;
class AddToToolbarWidget;
class XGRTEvent;
class SceneToolbarElement;
class SceneEditorOptions;
class VerticalScrollView;
class DebugRendererEditorWidget;
class LastUsedMenu;
class GLRenderTextureFBO;
class ImportAssistantWidget;
class ImportAnimationAssistantWidget;
class Renderer;

class X3EDITOR_API SceneEditorWidget : public QMainWindow, public StandAloneEditorWidget {
	Q_OBJECT


public:
	enum EImageFormat
	{
		IF_BMP,
		IF_PNG,
		IF_JPG
	};

	/// save current gl viewport to disk
	void saveScreenShot( std::string filename, EImageFormat format, int quality = 80 );

	/// reads current frame buffer + zbuffer and creates a "range scan"
	void rangeScan( std::string pcName );
	
 private:

	mpcard doubleNum;

	/// save frame to disk, uses parameters from movieStudio
	void saveFrame();

	std::string	m_ScreenShotBaseName;
	int		m_ScreenShotID;
	EImageFormat	m_ScreenShotImageFormat;
	bool		m_RecordHighQualityScreenShot;
	
	/// rendering lens
	bool		m_SimulateLens;
	float		m_SimulateLens_ConvRadius;
	unsigned	m_SimulateLens_NumSamples;
	Vector2f	m_SimulateLens_Temp_SamplePoint;

	// member variables for tiled screenshots
	bool		m_SetupTiledImage;
	Vector2f	m_TiledImage_X0;
	Vector2f m_TiledImage_X1;
	GLRenderTextureFBO * m_OffscreenRenderTarget;

	Ui_SceneEditorForm ui;

	Scene	*	scene;

   // scene editor panel
   ClassEditor *sceneEditor;
   QWidget *sceneEditorControl;

   // tools editor panel
   ClassEditor *toolsEditor;
   QWidget *toolsEditorControl;
   PCInteractionTool *currentTool;
   LastUsedMenu *luToolMenu;

   // commands editor panel
   ClassEditor *cmdsEditor;
   QWidget *cmdsEditorControl;
   PCCommand *currentCommand;
   LastUsedMenu *luCommandMenu;

   // rendering panel
   ClassEditor *rendererEditor;
   QWidget *rendererEditorControl;
   Renderer *currentRenderer;


   // scene graph state
   ClassEditor *staticStateEditor;
   QWidget *staticStateEditorWidget;
   ClassEditor *dynamicStateEditor;
   QWidget *dynamicStateEditorWidget;

	// debug renderer widget (now a member of this widget)
	DebugRendererEditorWidget *debugRendererEditorWidget;
	//static DebugRendererEditorWidget *debugRendererEditorWidget;

	ClassInstanceSet oldTools;

	bool force_cam;
	PCInteractionTool *camTool;
	
	GLContext *glContext;
	XrtGLWidget* mainGLArea;

	//QLabel* statusLabel;

	CommandHistoryEditorWidget* cmdHistoryForm;

   MovieStudioWidget * movieStudio;
   MacroEditorWidget * macroScriptEditor;

	AddToToolbarWidget	*	addToToolbarDlg;
	QToolBar *animationBarToolbar;
	AnimationBarWidget *animationBar;

    //QLayout* progressBars;

	void initLocalVars();
	void changeGlWindowResolution(int32 x, int32 y);

	// toolbar
	std::vector<QToolBar*> toolBars;
	card32 m_InterfaceVersion;
	std::map<QAction*,XGRTEvent*>	toolBarEvents;

	// editor options
	SceneEditorOptions *	editorOptions;
	QDialog					*	editorOptionsDlg;

	// scroll areas for tabs
	VerticalScrollView * editSceneView;
	VerticalScrollView * toolView;
	VerticalScrollView * commandView;
	VerticalScrollView * rendererView;
	
	// imports
	ImportAssistantWidget* importAssistant;
	ImportAnimationAssistantWidget *importAnimationAssistant;

	// timer id for redraw timer
	int	timerID;
	int	redrawDelay;
	
	class CallMeBack : public Object {
	 public:
	   SceneEditorWidget *me;
	   
	   CallMeBack(SceneEditorWidget *w) {me = w;}
   	// create command objects for clicked methods
	   bool cmdMethodCalled(std::string name);
   	bool toolMethodCalled(std::string name);
   };
   CallMeBack methodClickCallback;
   bool cmdMethodCalled(std::string name);
	bool toolMethodCalled(std::string name);

	// avoid event processing before constructor & setup is completed
	bool initialized;
	// stored for clean-up if used as stand-alone app
	ClassEditor *myObjectEditor;

 public:
	SceneEditorWidget(QWidget * parent = NULL, Qt::WindowFlags f=0);
	~SceneEditorWidget(void);

	void setup( Scene * obj );

	/// update editor content with object data
	void updateEditor();

	Renderer * getCurrentRenderer() {return currentRenderer;}

	AnimationBarWidget* getAnimationBar();
	void recordMovieNow(const char* rawFileName, float32 minTime, float32 maxTime, float32 timeStep, bool setSGTime,
		bool doRecord);
	void saveFrameBMP( const QString& filename );
	void saveHighResFrame();

	bool inContinuousRenderingLoop;

	void clearToolBarsVec();
	void addToolBarVec( QToolBar * tb );
	void clearToolBarEvents() {toolBarEvents.clear();}
	void addEvent( QAction * action, XGRTEvent * e ) {toolBarEvents[action]=e;}

	void processEvent( XGRTEvent * e );
	std::map<int, std::vector< SceneToolbarElement*> >	toolBarShortCuts;

	void updateSceneEditor( PCInteractionToolChanges whatChange );

	PCInteractionTool* getCamTool() const { return camTool; }
	
	void repaintGLArea();
	
   GLContext *getGLContext() {return glContext;}
   XrtGLWidget* getMainGLArea() {return mainGLArea;}

	
	/// advises the SceneEditorWidget to immediately execute the command of the given class
	/// if the class is not found or of wrong type, an exception is thrown
	void executeCommandClass(const MetaClass *cmd);

   QPoint mapGLAreaToGlobal(const QPoint &p);

   /// pass the mouse event on to the camera controller (internal use)
   virtual void mouseDownPassToCam(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   /// pass the mouse event on to the camera controller (internal use)
   virtual void mouseMovedPassToCam(int32 x, int32 y);
   /// pass the mouse event on to the camera controller (internal use)
   virtual void mouseUpPassToCam(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   /// pass the mouse event on to the camera controller (internal use)
   virtual void areaResizePassToCam(card32 width, card32 height);
   /// pass the mouse event on to the camera controller (internal use)
   virtual void mouseWheelRotatedPassToCam(int32 rotatedDelta, Modifiers modifiersState);
   /// pass the keyboard event on to the camera controller (internal use)
   virtual void keyDownPassToCam(GeneralKey key);
   /// pass the keyboard event on to the camera controller (internal use)
   virtual void keyUpPassToCam(GeneralKey key);

   /// add new progress bar to the scene editor widget
   /// scene editor is responsible for placing progress widgets in correct place
   virtual void addProgressbarWidget(QWidget* widget);

   /// remove progress bar widget out of the editor
   virtual void removeProgressbarWidget(QWidget* widget);

   /// guess point in world space that the used clicked, returns false if used clicked in empty space
   bool guessClickPoint(int x, int y, unsigned rad, Vector3f &ret);

	/// compute oobb from screen rectangle (all rendered inlier points will be used for pca...)
	bool estimateOOBBFromScreenArea( Vector2i x1, Vector2i x2, BoundingBox3f &bb, Matrix4f &worldToBB, float depthOutlierTolerance, Vector3f *firstAxis=NULL);

	/// store editor object for cleanup
	virtual void setEditorObjForCleanup(ClassEditor *ed);

 private:
	 void updateMenu();
	 SceneInterface * getSceneInterface();
	 void buildToolBar();
	 void addToToolbar( std::string className, bool et_object, bool et_command );
	 bool processToolbarKeyEvents( QKeyEvent * e ); // returns true if key is accepted

	 void changeCommand( const MetaClass * mc, PCCommand * obj=NULL);
	 void changeTool( const MetaClass * mc, PCInteractionTool * obj=NULL);

	 void saveTiledScreenshot(int numTiles, int numOverSampling);

 public slots:
	void sltGLResize(const QSize);
	void sltGLRepaint(void);
	void sltGLMouseMove(QMouseEvent *event);
	void sltGLMousePress(QMouseEvent *event);
	void sltGLMouseRelease(QMouseEvent *event);
	void sltGLWheelEvent(QWheelEvent * event);
	void sltGLDropEvent( QMimeData * d, QPoint pos );
	void sltScreenShot();


	void on_actionSave_as_triggered(bool checked);
	void on_actionImport_triggered(bool checked);
	void on_actionImport_Animation_triggered(bool checked);
	void on_actionExport_triggered(bool checked);
	void on_actionExport_Mesh_triggered(bool checked);
	void on_actionExport_Obj_triggered(bool checked);
	void on_actionExport_Animation_triggered(bool checked);
	void on_actionE_xit_triggered(bool checked);
	
	void on_actionCommandHistory_triggered(bool checked);
	void on_actionUndo_triggered(bool checked);
	void on_actionRedo_triggered(bool checked);
	void on_actionRepeat_triggered(bool checked);
	void on_actionDelete_triggered(bool checked);
	void on_actionCopy_triggered(bool checked);
	void on_actionSplit_triggered(bool checked);
	void on_actionJoin_Clouds_triggered(bool checked);
	void on_actionDuplicate_triggered(bool checked);
	void on_actionSplit_to_single_cloud_triggered(bool checked);

	void on_actionReset_Layout_triggered(bool checked);
	void on_actionSafe_Layout_triggered(bool checked);
	void on_actionRestore_Layout_triggered(bool checked);

	void on_actionMovieStudio_triggered(bool checked);
	void on_actionMacrosAndScripts_triggered(bool checked);
	void on_action256_x_256_triggered(bool checked);
	void on_action400_x_300_triggered(bool checked);
	void on_action512_x_512_triggered(bool checked);
	void on_action640_x_480_triggered(bool checked);
	void on_action1024_x_768_triggered(bool checked);
	void on_action1280_x_720_triggered(bool checked);
	void on_action1280_x_960_triggered(bool checked);
	void on_action_Auto_Layout_triggered(bool checked);
	
	
	void on_actionContext_Help_triggered(bool checked);
	void on_actionExport_Bitmap_triggered(bool checked);
	void on_actionRangescan_triggered(bool checked);
	void on_actionSet_GSA_OOC_Cache_Size_triggered(bool checked);
	void on_actionDebugRenderer_triggered(bool checked);

    void sltShowStatusMesage(QString msg);
    void sltClearStatusMessage();

	void on_actionReport_Statistics_triggered(bool checked);

 private slots:

	void sltToolChanged( const QString & newToolName );
	void sltCommandChanged( const QString & newToolName );
	void sltSceneEditChanged( const QString & newToolName );
	void sltRendererChanged( const QString & newToolName );
	void sltExecCommand(void);

	void sltToolbarHandler( QAction * action );
	void sltToolbarCheckBoxHandler( bool checked, char * data );

	void sltLUTool( QAction * action );
	void sltLUCommand( QAction * action );

	//void on_viewButton_clicked();
	//void on_selectButton_clicked();
	//void on_drawButton_clicked();
	//void on_transButton_clicked();
	void sltAnimationBarFrameChanged(int);
	void sltAnimationBarTraversalModeChanged(SGStaticState::TraversalModeType);
	void sltSwitchCameraMode();

	void on_pbAddTool_clicked();
	void on_pbAddCommand_clicked();

	void on_pbLastUsedTool_clicked();
	void on_pbLastUsedCommand_clicked();
	
	void on_actionAdd_Quick_Cam_Checkbox_triggered();
	void on_actionEditor_Options_triggered();

	void on_actionScreenshot_triggered();
	
	void on_actionScreenshot_4000x4000_triggered();
	void on_actionScreenshot_Tiles_triggered();

	void on_actionBackgroundBlack_triggered();
	void on_actionBackgroundDark_triggered();
	void on_actionBackgroundBright_triggered();
	void on_actionBackgroundWhite_triggered();

	void on_pbRendererScreenshot_clicked();
	
	void sltStateDockShowing();
	

 protected:
	virtual void keyPressEvent ( QKeyEvent * event );
	virtual void keyReleaseEvent  ( QKeyEvent * event );

	virtual void focusInEvent( QFocusEvent * e );
	virtual void timerEvent( QTimerEvent * e);
};

MAKE_CLASS_EDITOR( X3EDITOR_API , Scene )

class LastUsedMenu : public QMenu
{
private:

	unsigned m_MaxNumEntries;
	std::vector<QString>	m_Entries;

public:
	
	LastUsedMenu(QWidget * parent, unsigned maxNumEntries);
	void addEntry(const QString &s);
};

}

#endif
