//---------------------------------------------------------------------------
#ifndef SceneGraphCmdObjsH
#define SceneGraphCmdObjsH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
//---------------------------------------------------------------------------
#include <string>
#include <vector>
#include <set>
//---------------------------------------------------------------------------


namespace X4 {
class X3EDITOR_API CO_SGAddNode : public CommandObject {
 private:
   static MetaClass *CO_SGAddNodeMetaClass;
   static Object* newInstance();

   card32 action;
   std::string target;
   const MetaClass *newNodeType;
   std::string undo_newNodeName;

 public:
   static const card32 ADD = 0;
   static const card32 INSERT = 1;

   CO_SGAddNode(std::string target = "", const MetaClass *newNodeType = NULL, card32 action = 0);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};



class X3EDITOR_API CO_SGDeleteNode : public CommandObject {
 private:
   static MetaClass *CO_SGDeleteNodeMetaClass;
   static Object* newInstance();

   std::vector<std::string> target;

 public:
   CO_SGDeleteNode() {}
   CO_SGDeleteNode(std::vector<std::string> target);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};



class X3EDITOR_API CO_SGMoveNode : public CommandObject {
 private:
   static MetaClass *CO_SGMoveNodeMetaClass;
   static Object* newInstance();

   std::string target;
   card32 action;

 public:
   static const card32 UP = 1;
   static const card32 DOWN = 2;

   CO_SGMoveNode() {action = 0;}
   CO_SGMoveNode(std::string target, card32 action);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};



class X3EDITOR_API CO_SGMoveSelection : public CommandObject {
 private:
   static MetaClass *CO_SGMoveSelectionMetaClass;
   static Object* newInstance();

   std::string target;
   card32 action;

 public:
   static const card32 MOVE = 1;
   static const card32 LINK = 2;

   CO_SGMoveSelection() {action = 0;}
   CO_SGMoveSelection(std::string target, card32 action);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};



class X3EDITOR_API CO_SGGroupSelection : public CommandObject {
 private:
   static MetaClass *CO_SGGroupSelectionMetaClass;
   static Object* newInstance();

   std::string parent;
   std::vector<std::string> children;

 public:
   CO_SGGroupSelection() {}
   CO_SGGroupSelection(std::string parent, std::vector<std::string> children);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};



class X3EDITOR_API CO_SGUnGroupSelection : public CommandObject {
 private:
   static MetaClass *CO_SGUnGroupSelectionMetaClass;
   static Object* newInstance();

   std::string target;

 public:
   CO_SGUnGroupSelection() {}
   CO_SGUnGroupSelection(std::string target);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};



class X3EDITOR_API CO_SGChangeSelection : public CommandObject {
 private:
   static MetaClass *CO_SGChangeSelectionMetaClass;
   static Object* newInstance();

   set<std::string> prevSelection;
   set<std::string> selection;

 public:
   CO_SGChangeSelection() {}
   CO_SGChangeSelection(set<std::string> selection);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();
   virtual bool mergeCommand(const CommandObject *nextCmd);

   virtual std::string getDescription();
};

class X3EDITOR_API CmdObjConvertType : public CommandObject {
 private:
   static MetaClass *CmdObjConvertTypeMetaClass;
   static Object* newInstance();

   int32 action;
   bool delOriginal;
   std::string filename_depreciated; // not used anylonger
   card32 allocatorIndex; // instead, the allocator is chosen by index from staticState.allocators
   int32 blockSize_depreciated; // not used anylonger
   int32 maxNumNodesInMem;
   int32 maxLeafPts;
   int32 gridSize;

 public:
   CmdObjConvertType(int32 action = 0, bool delOriginal = false, card32 allocatorIndex = 0, 
                     int32 maxNumNodesInMem = 0, int32 maxLeafPts = 0, int32 gridSize = 0);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};


class X3EDITOR_API CmdObjAppendFields : public CommandObject {
 private:
   static MetaClass *CmdObjAppendFieldsMetaClass;
   static Object* newInstance();

   VertexDescriptor *newFields;

 public:
   CmdObjAppendFields(VertexDescriptor *newFields = NULL);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};



class X3EDITOR_API CmdObjSGChangeDefaultTraversalMode : public CommandObject {
 private:
   static MetaClass *CmdObjSGChangeDefaultTraversalModeMetaClass;
   static Object* newInstance();

	SGStaticState::TraversalModeType newMode;
   SGStaticState::TraversalModeType oldMode;

 public:
	 CmdObjSGChangeDefaultTraversalMode(SGStaticState::TraversalModeType mode = SGStaticState::tmAllNodes);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual std::string getDescription();
};

}

#endif
