#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SceneGraphEditorWidget.h"
//---------------------------------------------------------------------------
#include "StandardSceneGraphIterator.h"
#include "SceneGraphNode.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "SGListNode.h"
#include "SGObjectNode.h"
#include "SGTransformationNode.h"
#include "ParametricInCorePointCloud.h"				 
// xxx hierarchy removed: #include "HierarchicalInCorePointCloud.h"
// xxx hierarchy removed: #include "HierarchicalOutOfCorePointCloud.h"
#include "PCImageViewers.h"
//#include "GUITools.h"
#include "ObjectBrowserWidget.h"
#include "SceneGraphCmdObjs.h"
#include "CommandObject.h"
#include "CommandHistory.h"
#include "ClassSelectionDialog.h"
#include "AppendFieldsDialogs.h"
#include "CmdObjPointCloudOperations.h"
//#include "ObjectBrowsers.h"
#include "PointCloudConversionForms.h"
#include "ObjectViewsTable.h"
#include "SGAnimationNode.h"
#include "Configuration.h"
#include "StringHelper.h"
#include "GlobalMacros.h"
#include <qtCore/qStringList>
// xxx hierarchy removed: #include "SGAllocatorList.h"
#include "ExceptionHandling.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
#include "SceneGraphTools.h"
#include "PointCloudTools.h"
#include "GLMaterial.h"
#include "GLShaderMaterialEditorWidget.h"
#include "SceneEditorWidget.h"
#include <QInputDialog>
//----------------------------
using namespace X4;

SGTreeWidgetItem::SGTreeWidgetItem(SGTreeWidgetItem * parent ,const QStringList &s,SceneGraphNode* sg_node)
:QTreeWidgetItem(parent,s)
{
	m_node=sg_node;
}



SGTreeWidgetItem::SGTreeWidgetItem(const QStringList &s,SceneGraphNode* sg_node)
:QTreeWidgetItem(s)
{
	m_node=sg_node;
}
SGTreeWidgetItem::~SGTreeWidgetItem(void){};

SGTreeWidget::SGTreeWidget(QWidget* parent)
:QTreeWidget(parent)
{
	setColumnCount(2);
	QStringList headerNames;
	headerNames << "Name" << "Class";
	setHeaderLabels( headerNames );
	setDragEnabled(true);
	setAcceptDrops(true);
	setSelectionMode(QAbstractItemView::ExtendedSelection);
	
	
}
SGTreeWidget::~SGTreeWidget() {}

void SGTreeWidget::dropEvent(QDropEvent *event)
{
	emit dropped(event);

}

void SGTreeWidget::dragEnterEvent(QDragEnterEvent *event)
{
	
	if (event->source()==this)
	event->accept();
	

}

void SGTreeWidget::mousePressEvent(QMouseEvent *event)
{
	
	emit pressed(event);
	QTreeWidget::mousePressEvent(event);

}






SceneGraphEditorWidget::SceneGraphEditorWidget(QWidget * parent, Scene* scene)
:QWidget(parent)
{
	changingMaterials = false;
   updatingSelection = false;
	this->scene = scene;
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));
	LVSceneGraph= new SGTreeWidget(ui.FrameLV);
	QHBoxLayout * layout2 = new QHBoxLayout(ui.FrameLV );
	layout2->addWidget( LVSceneGraph );
	layout2->setMargin(1);
	layout2->setSpacing(1);
	ui.showPcAttachments->setEnabled(false);
	rebuilding=false;


	LVSceneGraph->show();
	
	
	// init the rest
	QWidget::connect(LVSceneGraph,SIGNAL(itemSelectionChanged()),this,SLOT(LVSceneGraph_itemSelectionChanged()));
	QWidget::connect(LVSceneGraph,SIGNAL(pressed(QMouseEvent*)),this,SLOT(LVSceneGraph_pressed(QMouseEvent*)));
	QWidget::connect(LVSceneGraph,SIGNAL(dropped(QDropEvent*)),this,SLOT(LVSceneGraph_dropped(QDropEvent*)));
	QWidget::connect(LVSceneGraph,SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)),this,SLOT(LVSceneGraph_itemDoubleClicked(QTreeWidgetItem*,int)));




	lastVisible = NULL;
	nodeEditor = NULL;
	nodeEditorWidget = NULL;
	rclickMenu=NULL;
	buildMenu();
	QWidget::connect(rclickMenu,SIGNAL(triggered(QAction*)),this,SLOT(rclickMenu_triggered(QAction*)));

	// todo addNodePopup = makeClassPopup(this, onNewNodeTypeSelection, SceneGraphNode::getClass());
	nodeTypeSelectionAction = 0;

	QHBoxLayout * layout = new QHBoxLayout( ui.PropertyFrame );
	layout->setMargin(0);

//	LVSceneGraph_itemSelectionChanged();
}

SceneGraphEditorWidget::~SceneGraphEditorWidget(void)
{
	delete lastVisible;
}


SceneGraphEditorWidget::RememberVisibleItem::~RememberVisibleItem() {
	std::map<std::string, RememberVisibleItem*>::iterator pos = children.begin();
	std::map<std::string, RememberVisibleItem*>::iterator end = children.end();
	while (pos != end) {
		delete pos->second;
		++pos;
	}
}


void SceneGraphEditorWidget::updateEditor()
{
	//update();
}


void SceneGraphEditorWidget::buildMenu() 
{
	rclickMenu=new QMenu(this);
	QAction* copyObject=new QAction("Make copy",this);
	QAction* imgView=new QAction("Image Viewer",this);
	QAction* convertType=new QAction("Convert type",this);
	QAction* appendFields=new QAction("Append fields",this);
	QAction* removeChannel=new QAction("Remove channel",this);

	QAction* clearPCAttach=new QAction("Clear PC Attachments",this);
	QAction* invalidateHier=new QAction("Invalidate Hierarchy",this);
	QAction* shutdownHier=new QAction("Shutdown Hierarchy",this);

	QAction* copySelected = new QAction("Copy selected clouds", this);
	QAction* toggleInvisible = new QAction("Toggle (in)visible", this);

	rclickMenu->addAction(imgView);
	rclickMenu->addSeparator();
	rclickMenu->addAction(copyObject);
	rclickMenu->addAction(convertType);
	rclickMenu->addAction(appendFields);
	rclickMenu->addAction(removeChannel);
	rclickMenu->addSeparator();
	rclickMenu->addAction(clearPCAttach);
	rclickMenu->addAction(invalidateHier);
	rclickMenu->addAction(shutdownHier);								  	
	rclickMenu->addSeparator();
	rclickMenu->addAction(copySelected);
	rclickMenu->addAction(toggleInvisible);	


}


// ---------- user functions ----------

static int calcImageIndex(SceneGraphNode *node) {
	int result = 0;
	if (dynamic_cast<SGTransformationNode*>(node)) result = 1;
	if (dynamic_cast<SGObjectNode*>(node)) result = 2;
	if (dynamic_cast<SGListNode*>(node)) result = 3;
	if (dynamic_cast<SGAnimationNode*>(node)) result = 5;
	if (node->getNumReferences() > 1) result += 6;
	if (node->getNumReferences() == 0) result = 12;

	std::string className = node->getInstanceClass()->getClassName();
	if( className == "SCGrammar") return 13;
	return result;
}

void SceneGraphEditorWidget::rebuildTree(StandardSceneGraphIterator &it, SGTreeWidgetItem *tviParent, RememberVisibleItem *lastVisible) {
	if (numItems++ > 100000) return;
	SGTreeWidgetItem* newTVI;

	if (tviParent==NULL)
	{	
		newTVI= new SGTreeWidgetItem(QStringList(it.getNode()->getName().c_str()),it.getNode());
		LVSceneGraph->addTopLevelItem(newTVI);
	}
	else
	{
		newTVI = new SGTreeWidgetItem(tviParent, QStringList(it.getNode()->getName().c_str()),it.getNode());
	}
	newTVI->setText(1, it.getNode()->getInstanceClass()->getClassName().c_str() );
	QFont font=newTVI->font(0);
	font.setItalic(!it.getNode()->visible());
	newTVI->setFont(0,font);
	QString fname;
	fname = lookUpIconName(calcImageIndex(it.getNode())).c_str();
	newTVI->setIcon(0,QIcon(fname));
	RememberVisibleItem *next = NULL;
	if (lastVisible != NULL) {
		std::map<std::string, RememberVisibleItem*>::iterator pos = lastVisible->children.find(it.getNode()->getName().c_str());
		if (pos != lastVisible->children.end()) {
			next = pos->second;
			LVSceneGraph->expandItem(newTVI);
		}
	}
	max_platform_card n = it.getNumChildren();
	for (max_platform_card i=0; i<n; i++) {
		it.goToChild(i);
		rebuildTree(it, newTVI, next);
		it.goToParent();
	}
}


int SceneGraphEditorWidget::getNumSelNodes() {
	QList<QTreeWidgetItem*> itemlist = LVSceneGraph->selectedItems();
	return itemlist.count();

}


SceneGraphNode* SceneGraphEditorWidget::getSelNode() {
	QList<QTreeWidgetItem*> itemlist = LVSceneGraph->selectedItems();
	if (itemlist.isEmpty()) return NULL;
	else
		return ((SGTreeWidgetItem*)(itemlist.first()))->getSGNode();	
}



std::vector<SceneGraphNode*> SceneGraphEditorWidget::getSelNodes() {
	std::vector<SceneGraphNode*> result;
	QList<QTreeWidgetItem*> itemlist = LVSceneGraph->selectedItems();

	for (int32 i=0; i<itemlist.count(); i++) {
		result.push_back(((SGTreeWidgetItem*)(itemlist[i]))->getSGNode());
	}

	return result;
}


std::string SceneGraphEditorWidget::lookUpIconName(int index)
{
	switch( index)
	{
	case 1: return ":/editor/images/SceneGraphIcons/Matrix.png";
	case 2: return ":/editor/images/SceneGraphIcons/Object16.png";
	case 3: return ":/editor/images/SceneGraphIcons/Folder16.png";
	case 5: return ":/editor/images/SceneGraphIcons/Anim.png";
	case 7: return ":/editor/images/SceneGraphIcons/MatrixArrow.png";
	case 8: return ":/editor/images/SceneGraphIcons/ObjectArrow16.png";
	case 9: return ":/editor/images/SceneGraphIcons/FolderArrow16.png";
	case 11: return ":/editor/images/SceneGraphIcons/AnimArrow.png";
	case 13: return ":/editor/images/SceneGraphIcons/shapegrammar.png";
	default: return ":/editor/images/SceneGraphIcons/Class.png";
	}

}



void SceneGraphEditorWidget::rebuildLastVisibleRek(SGTreeWidgetItem *node, RememberVisibleItem *current) {

	std::string name=node->text(0).toLatin1().constData();
	name="";
	if (LVSceneGraph->isItemExpanded(node)) {
		RememberVisibleItem* rvi = new RememberVisibleItem();
		current->children.insert(pair<std::string, RememberVisibleItem*>(node->text(0).toLatin1().constData(), rvi));
		int n = node->childCount();
		for (int i=0; i<n; i++) {
			rebuildLastVisibleRek((SGTreeWidgetItem*)node->child(i), rvi);
		}
	}
}

void SceneGraphEditorWidget::rebuildLastVisible() {
	delete lastVisible;
	lastVisible = new RememberVisibleItem();
	card32 n=LVSceneGraph->topLevelItemCount();
	if (n==0) return;
	x4Assert(LVSceneGraph->topLevelItem(0)->parent() == NULL);
	rebuildLastVisibleRek((SGTreeWidgetItem*)LVSceneGraph->topLevelItem(0), lastVisible);
}

void SceneGraphEditorWidget::update() {

	rebuildLastVisible();
	
	rebuilding=true;
	LVSceneGraph->clear();

	if (nodeEditor != NULL) {
		delete nodeEditor; nodeEditor = NULL;
		delete nodeEditorWidget;
		nodeEditorWidget = NULL;
	}
	if (scene->getRootNode() != NULL) {
		StandardSceneGraphIterator it;
		it.clearAndSetup(scene->getRootNode(), scene->getRootState(), SGDynamicState::tmAllNodes, 0, 0);
		numItems = 0;
		rebuildTree(it, NULL, lastVisible);
		//if (numItems > 1000) WARNINGMESSAGE("SceneGraphEditorWidget::update() - more than 1000 nodes (encoded) in scene graph. visualization has been truncated.");
	}
	rebuilding=false;



	delete lastVisible;
	lastVisible = NULL;
	updateSelection();

}


SGTreeWidgetItem* SceneGraphEditorWidget::getSelTTreeNode() {
	x4Assert(getNumSelNodes() == 1);
	QList<QTreeWidgetItem*> itemlist = LVSceneGraph->selectedItems();
	if (itemlist.isEmpty()) return NULL;
	else
		return (SGTreeWidgetItem*)(itemlist.first());

}


std::string SceneGraphEditorWidget::getSelNodeNameWithPath() {
	SGTreeWidgetItem *t = getSelTTreeNode();
	return getNodeNameWithPath(t);
}


std::string SceneGraphEditorWidget::getNodeNameWithPath(SGTreeWidgetItem *node) {
	std::string result;
	while (node != NULL) {
		std::string name = ((SceneGraphNode*)(node->getSGNode()))->getName();
		result = name + result;
		node = (SGTreeWidgetItem*)node->parent();
		if (node != NULL) {
			result = std::string("/") + result;
		}
	}
	return result;
}


bool SceneGraphEditorWidget::allSelParentsSimilar() {
	QList<QTreeWidgetItem*> itemlist = LVSceneGraph->selectedItems();
	QTreeWidgetItem *NONODE = (QTreeWidgetItem*)(-1);
	QTreeWidgetItem *p = NONODE;
	int n = itemlist.count() ;
	for (int i=0; i<n; i++) {
		if (p == NONODE) {
			p = itemlist[i]->parent();
		} else {
			if (itemlist[i]->parent() != p) return false;
		}
	}

	return true;
}

void SceneGraphEditorWidget::applySelection() {
	if (rebuilding)
		return; //itemSelectionChanged occured due to clear(), not user-interaction.

	QList<QTreeWidgetItem*> itemlist = LVSceneGraph->selectedItems();
	set<std::string> sel;
	sel.clear();
	int n=itemlist.count();
	for (int i=0; i<n; i++) {
		sel.insert(getNodeNameWithPath((SGTreeWidgetItem*)itemlist[i]));
	}

	CO_SGChangeSelection *cmd = new CO_SGChangeSelection(sel);
	scene->getCommandHistory()->executeCommand(scene, cmd);
	ObjectViewsTable::update(scene->getRootState()->staticState);
	try {
		ObjectViewsTable::update(scene);
	} catch(...) {
	}
}
SGTreeWidgetItem* SceneGraphEditorWidget::getTWIByPath(std::string path)
{
	std::vector<std::string> splitName=splitString(path,"/");
	int n=(int)splitName.size();
	if (n==0) return NULL;
	SGTreeWidgetItem* twi=(SGTreeWidgetItem*)LVSceneGraph->topLevelItem(0);
	if (n==1) return twi;
	if (twi!=NULL)
	{

		for (int i=1;i<n; i++)
		{
			bool found=false;
			int m=twi->childCount();
			for(int j=0;j<m; j++)
			{
				if (twi->child(j)->text(0).toLatin1().constData()==splitName[i])
				{
					found=true;
					twi=(SGTreeWidgetItem*)twi->child(j);

					break;
				}
			}
			if (!found) return NULL;


		}
	}
	return twi;
}

void SceneGraphEditorWidget::updateSelection() {
	set<std::string> sel = scene->getRootState()->staticState->selectedNodes;
	if (sel.empty()) return;
	set<std::string>::iterator pos=sel.begin();
	set<std::string>::iterator end=sel.end();
   updatingSelection = true;
	while (pos!=end)
	{
		std::string path=*pos;
		LVSceneGraph->setItemSelected(getTWIByPath(path),true);
		++pos;
	}
   updatingSelection = false;
   LVSceneGraph_itemSelectionChanged();
}


void SceneGraphEditorWidget::on_BTUp_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (getNumSelNodes() ==0) return;
	CO_SGMoveNode *cmd = new CO_SGMoveNode(getSelNodeNameWithPath(), CO_SGMoveNode::UP);
	scene->getCommandHistory()->executeCommand(scene, cmd);
	update();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}
//---------------------------------------------------------------------------

void  SceneGraphEditorWidget::on_BTDown_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (getNumSelNodes() ==0) return;
	CO_SGMoveNode *cmd = new CO_SGMoveNode(getSelNodeNameWithPath(), CO_SGMoveNode::DOWN);
	scene->getCommandHistory()->executeCommand(scene, cmd);
	update();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}/*
 //---------------------------------------------------------------------------
 */
void SceneGraphEditorWidget::on_BTAdd_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (getNumSelNodes() != 1) return;
	MetaClass* selectedClass=NULL;
	ClassSelectionDialog	*	m_ClassSelectionDialog=new ClassSelectionDialog(this);
	m_ClassSelectionDialog->init( SceneGraphNode::getClass() , false );
	m_ClassSelectionDialog->move( QCursor::pos() );
	if( m_ClassSelectionDialog->exec() == QDialog::Accepted )
	{
		selectedClass = m_ClassSelectionDialog->getSelectedClass();
	}
	delete m_ClassSelectionDialog;

	if (selectedClass==NULL) return;
	CO_SGAddNode *cmd = new CO_SGAddNode(getSelNodeNameWithPath(), selectedClass, CO_SGAddNode::ADD);
	scene->getCommandHistory()->executeCommand(scene, cmd);
	update();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}
//---------------------------------------------------------------------------
/*

void TSceneGraphEditorFrame::onNewNodeTypeSelection(System::TObject* Sender, const MetaClass *selected) {
if (getNumSelNodes() != 1) return;
if (nodeTypeSelectionAction == NTS_Insert) {
CO_SGAddNode *cmd = new CO_SGAddNode(getSelNodeNameWithPath(), selected, CO_SGAddNode::INSERT);
scene->getCommandHistory()->executeCommand(scene, cmd);
} else {
CO_SGAddNode *cmd = new CO_SGAddNode(getSelNodeNameWithPath(), selected, CO_SGAddNode::ADD);
scene->getCommandHistory()->executeCommand(scene, cmd);
}
update();
addNodePopup->Close();
forget(Sender);
}




void __fastcall TSceneGraphEditorFrame::TV_sceneGraphEdited(
TObject *Sender, TTreeNode *Node, AnsiString &S)
{
((SceneGraphNode*)Node->Data)->setName(S.c_str());
update();
forget(Sender);   
}
//---------------------------------------------------------------------------

void TSceneGraphEditorFrame::onEditSubObject(ClassEditor* sender, Object* newObject, const MetaClass *editor) {
TObjectBrowser *b = new TObjectBrowser(Application);
b->pushObject(newObject, (ClassEditor*)(editor->newInstance()));
b->Show();
forget(sender);
}

void TSceneGraphEditorFrame::onEdObjChange(ClassEditor* sender, bool subObjectDeleted) {
if (sender != NULL) sender->apply();
forget(sender); forget(subObjectDeleted);
}

*/
void SceneGraphEditorWidget::updateMatSel(bool multiSelect, int matIndex) {
	changingMaterials = false;
	ui.cbMaterials->clear();
	if (matIndex != -1 || multiSelect) {
		for (card32 i=0; i<scene->getNumGLMaterials(); i++) {
			std::string name = intToStr(i) + std::string(" - ") + scene->getGLMaterial(i)->getName();
			ui.cbMaterials->addItem(name.c_str());
		}
		if (multiSelect) {
			ui.cbMaterials->addItem("multi-select");
			ui.cbMaterials->setCurrentIndex(ui.cbMaterials->count()-1);
			ui.bpEditMaterial->setEnabled(false);
		} else {
			ui.cbMaterials->setCurrentIndex(matIndex);
			ui.bpEditMaterial->setEnabled(true);
		}
		ui.cbMaterials->setEnabled(true);
		changingMaterials = true;
	} else {
		ui.cbMaterials->addItem("no object selected");
		ui.cbMaterials->setEnabled(false);
		ui.bpEditMaterial->setEnabled(false);
	}
}

void SceneGraphEditorWidget::LVSceneGraph_itemSelectionChanged()
{	
	STD_GLOBAL_EXCEPTION_HANDLER_START
   if (updatingSelection) return;
	int n = getNumSelNodes();
	if (n == 0) {
		ui.BTAdd->setEnabled( false);
		ui.BTDelete->setEnabled( false);
		ui.BTGroup->setEnabled( false);
		ui.BTUngroup->setEnabled( false);
		ui.BTUp->setEnabled( false);
		ui.BTDown->setEnabled( false);
		ui.BTMove->setEnabled( false);
		ui.BTLink->setEnabled( false);
		ui.showPcAttachments->setEnabled(false);
		changingMaterials = false;
		ui.cbMaterials->setEnabled(false);
		ui.bpEditMaterial->setEnabled(false);
		ui.cbMaterials->clear();
		ui.cbMaterials->addItem("no selection");
	} else if (n == 1) {
		ui.BTAdd->setEnabled( true);
		ui.BTDelete->setEnabled( (getSelNode() != scene->getRootNode()));
		ui.BTGroup->setEnabled( false);
		ui.BTUngroup->setEnabled( dynamic_cast<SGListNode*>(getSelNode()) != NULL && (getSelNode() != scene->getRootNode()));
		ui.BTUp->setEnabled( (getSelNode() != scene->getRootNode()));
		ui.BTDown->setEnabled( (getSelNode() != scene->getRootNode()));
		ui.BTMove->setEnabled( true);
		ui.BTLink->setEnabled( true);
		
		changingMaterials = false;
		SGObjectNode *node = dynamic_cast<SGObjectNode*>(getSelNode());
		int matindex = -1;
		if (node && node->getSceneObject()) {
			matindex = node->getSceneObject()->getMaterialIndex();
		}
		updateMatSel(false, matindex);

		ui.showPcAttachments->setEnabled(dynamic_cast<SGObjectNode*>(getSelNode()) != NULL);
	} else {
		ui.BTAdd->setEnabled( false);
		ui.BTDelete->setEnabled( true);
		ui.BTGroup->setEnabled( allSelParentsSimilar());
		ui.BTUngroup->setEnabled( false);
		ui.BTUp->setEnabled( false);
		ui.BTDown->setEnabled( false);
		ui.BTMove->setEnabled( true);
		ui.BTLink->setEnabled( true);
		changingMaterials = false;
		ui.showPcAttachments->setEnabled(false);
		updateMatSel(true, -1);
	}

	if (nodeEditor != NULL) {
		delete nodeEditor; nodeEditor = NULL;
		delete nodeEditorWidget;
		nodeEditorWidget = NULL;
	}


	if (n <= 0) {
		ui.LabelSelectionInfo->setText(QString( "- no selection -"));
	} else if (n > 1) {
      card64 numPoints = 0;
		std::vector<SceneGraphNode*> selection = getSelNodes();
		for (card32 i=0; i<selection.size(); i++) {
			SGObjectNode *objnode = dynamic_cast<SGObjectNode*>(selection[i]);
			if (objnode != NULL) {
				SceneObject* sceneobj = objnode->getSceneObject();
				if (sceneobj != NULL) {
					PointCloud *pc = dynamic_cast<PointCloud*>(sceneobj);
					GenericSwapAllOutOfCorePointCloud * gpc = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(sceneobj);
					if (pc != NULL && !gpc) {
						numPoints += pc->getNumPoints();
					}
				}
			}
		}
		std::string selectionInfo = "- multi-select -";
		selectionInfo += "<br><FONT color=\"#F0B0B0\">";
		std::string numPointsStr = intToStr(numPoints);

		selectionInfo += ( "#Points:" + numPointsStr );
		selectionInfo += "</FONT>";
		ui.LabelSelectionInfo->setText(QString(selectionInfo.c_str()));

	} else {
		std::string selectionInfo = ( getSelNode()->getInstanceClass()->getClassName() + ": " + getSelNodeNameWithPath() );

		SGObjectNode *objnode = dynamic_cast<SGObjectNode*>(getSelNode());
		if (objnode != NULL) {

			SceneObject* sceneobj = objnode->getSceneObject();
			if (sceneobj != NULL) {
				selectionInfo += "<br><FONT color=\"#A0F0A0\">";
				selectionInfo += sceneobj->getInstanceClass()->getClassName();
				selectionInfo += "</FONT>";

				PointCloud *pc = dynamic_cast<PointCloud*>(sceneobj);
				if (pc != NULL) {
					selectionInfo += "<br><FONT color=\"#F0F0A0\">";
					std::vector<std::string> descr = splitString(pc->getDescr()->getDescription(), ";");
					for (card32 i=0; i<descr.size(); i+=4) {
						for (card32 j=0; j<4; j++) {
							if ((i+j)<descr.size()) selectionInfo += descr[i+j];
						}
						selectionInfo += "<br>";
					}
					selectionInfo += "</FONT><FONT color=\"#F0B0B0\">";
					selectionInfo += ( "#Points:" + intToStr(pc->getNumPoints()) );
					selectionInfo += "</FONT>";
				}

			};
		};
		
		ui.LabelSelectionInfo->setText(QString(selectionInfo.c_str()));

		nodeEditor = (ClassEditor*)(getSelNode()->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance());
		nodeEditorWidget = nodeEditor->makeEditor(ui.PropertyFrame, getSelNode());
		nodeEditor->update();
		ui.PropertyFrame->layout()->addWidget( nodeEditorWidget );
		nodeEditorWidget->show();

		
	}
	applySelection();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}
//---------------------------------------------------------------------------

void SceneGraphEditorWidget::on_BTDelete_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (QMessageBox::question (this, "Confirm deletion / Reference Removal.", "Remove selected nodes from scene graph?", QMessageBox::Yes, QMessageBox::No) == QMessageBox::No) return;
	
	//can't work as of yet: selection has to be applied!
	set<std::string> nodes = scene->getRootState()->staticState->selectedNodes;
	set<std::string>::iterator pos = nodes.begin();
	set<std::string>::iterator end = nodes.end();

	std::vector<std::string> target;
	while (pos != end) {
		target.push_back(*pos);
		++pos;
	}

	CO_SGDeleteNode *cmd = new CO_SGDeleteNode(target);
	scene->getCommandHistory()->executeCommand(scene, cmd);
	update();
	//addNodePopup->Close();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}
//---------------------------------------------------------------------------

void SceneGraphEditorWidget::on_BTUpdate_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	update();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

//---------------------------------------------------------------------------

void SceneGraphEditorWidget::on_BTMove_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	bool doIt;
	QString newPath = QInputDialog::getText(this, tr("Move selected node?"),
		tr("New Root:"), QLineEdit::Normal,
		"root/", &doIt);
	if (doIt& !newPath.isEmpty()) {
		CO_SGMoveSelection *cmd = new CO_SGMoveSelection(newPath.toLatin1().constData(), CO_SGMoveSelection::MOVE);
		scene->getCommandHistory()->executeCommand(scene, cmd);
		update();
	}
	STD_GLOBAL_EXCEPTION_HANDLER_END
}
//---------------------------------------------------------------------------

void SceneGraphEditorWidget::on_BTLink_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	bool doIt;
	QString newPath = QInputDialog::getText(this, tr("Create a duplicate reference of selected nodes..."),
		tr("Please specify new ref. root path:"), QLineEdit::Normal,
		"root/", &doIt);

	if (doIt& !newPath.isEmpty()) {
		CO_SGMoveSelection *cmd = new CO_SGMoveSelection(newPath.toLatin1().constData(), CO_SGMoveSelection::LINK);
		scene->getCommandHistory()->executeCommand(scene, cmd);
		update();
	}
	STD_GLOBAL_EXCEPTION_HANDLER_END
}
//---------------------------------------------------------------------------

void SceneGraphEditorWidget::on_BTGroup_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (!allSelParentsSimilar()) return;
	std::vector<std::string> nodes;
	QList<QTreeWidgetItem*> itemlist=LVSceneGraph->selectedItems();
	int n = itemlist.count();
	for (int i=0; i<n; i++) {


		nodes.push_back(itemlist[i]->text(0).toLatin1().constData());

	}
	if (n==0) return;
	if (itemlist[0]->parent() == NULL) return;
	std::string parent = getNodeNameWithPath((SGTreeWidgetItem*)itemlist[0]->parent());
	CO_SGGroupSelection *cmd = new CO_SGGroupSelection(parent, nodes);
	scene->getCommandHistory()->executeCommand(scene, cmd);
	update();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}
//---------------------------------------------------------------------------

void SceneGraphEditorWidget::on_BTUngroup_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (getNumSelNodes() != 1) return;
	CO_SGUnGroupSelection *cmd = new CO_SGUnGroupSelection(getSelNodeNameWithPath());
	scene->getCommandHistory()->executeCommand(scene, cmd);
	update();
	ObjectViewsTable::update(scene);
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

/*
void __fastcall TSceneGraphEditorFrame::Addtonode1Click(TObject *Sender)
{
TPoint p(0, BTAdd->Height);
p = BTAdd->ClientToScreen(p);
nodeTypeSelectionAction = NTS_Add;
addNodePopup->popUp(p.x, p.y, 200,220);
forget(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TSceneGraphEditorFrame::Insertabovenode1Click(
TObject *Sender)
{
TPoint p(0, BTAdd->Height);
p = BTAdd->ClientToScreen(p);
nodeTypeSelectionAction = NTS_Insert;
addNodePopup->popUp(p.x, p.y, 200,220);
forget(Sender);
}
//---------------------------------------------------------------------------


*/

void SceneGraphEditorWidget::LVSceneGraph_itemDoubleClicked(QTreeWidgetItem* twi, int column)
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	bool visible=((SGTreeWidgetItem*)twi)->getSGNode()->visible();
	((SGTreeWidgetItem*)twi)->getSGNode()->setVisible(!visible);
	update();

	ObjectViewsTable::update(scene);
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void  SceneGraphEditorWidget::LVSceneGraph_pressed(QMouseEvent* event)
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (event->button()== Qt::RightButton)
	{
		SGTreeWidgetItem* twi=(SGTreeWidgetItem*)LVSceneGraph->itemAt(event->pos());
		if (twi==NULL) return;
		if (! dynamic_cast<SGObjectNode*> (twi->getSGNode())) return;
		rclickMenu->popup(LVSceneGraph->mapToGlobal(event->pos()));
	}
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void  SceneGraphEditorWidget::LVSceneGraph_dropped(QDropEvent *event)
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	SGTreeWidgetItem* twi=(SGTreeWidgetItem*)LVSceneGraph->itemAt(event->pos());
	std::string target=getNodeNameWithPath(twi);
	if (twi==NULL) return;
	bool noObject=dynamic_cast<SGObjectNode*> (twi->getSGNode());
	if (noObject) return;
	if (event->keyboardModifiers()==Qt::ShiftModifier)
	{//link
		CO_SGMoveSelection *cmd = new CO_SGMoveSelection(target, CO_SGMoveSelection::LINK);
		scene->getCommandHistory()->executeCommand(scene, cmd);
		update();

	}
	else
	{//move
		 {
			CO_SGMoveSelection *cmd = new CO_SGMoveSelection(target, CO_SGMoveSelection::MOVE);
			scene->getCommandHistory()->executeCommand(scene, cmd);
			update();
		}
	}
	STD_GLOBAL_EXCEPTION_HANDLER_END
}


void SceneGraphEditorWidget::showEvent ( QShowEvent * e )
{
	update();
}

void SceneGraphEditorWidget::rclickMenu_triggered(QAction *action) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	QPoint pos=this->rclickMenu->pos();
	pos=LVSceneGraph->mapFromGlobal(pos);
	SGTreeWidgetItem* twi=(SGTreeWidgetItem*)LVSceneGraph->itemAt(pos);
		if (twi==NULL) return;
		
		
		
	if (action->text()=="Image Viewer")
	{
		ParametricInCorePointCloud *pic = dynamic_cast<ParametricInCorePointCloud*>(((SGObjectNode*)twi->getSGNode())->getSceneObject());
		if (pic != NULL) {
			 PointSet *ps = pic->getPointSet();
         if (ps->getHeight() > 1 && ps->getWidth() > 1) {
			   PCImageViewer *iv = new PCImageViewer(this, ps);
			   
				 iv->show();
			} else {
				WARNINGMESSAGE("Parametric point cloud with singular width or height (i.e. <= 1). Image cannot be displayed.");
			}
		} else {
			GenericSwapAllOutOfCorePointCloud *gen=dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(((SGObjectNode*)twi->getSGNode())->getSceneObject());
			if (gen != NULL && gen->supportsIteratorType( SOIT::CAP_PARAMETRIC_PC)) {
				SceneObjectIterator* iter=gen->createIterator(SOIT::CAP_PARAMETRIC_PC );
				BasicPointCloudIterator* basicIt=dynamic_cast<BasicPointCloudIterator*>(iter);
				const PointSet *ps = basicIt->lockBlockRead();
				if (ps->getHeight() > 1 && ps->getWidth() > 1) {
					PCImageViewer *iv = new PCImageViewer(this, ps);
					basicIt->unlock();
					delete iter;

					iv->show();
				} else 
					WARNINGMESSAGE("Parametric point cloud with singular width or height (i.e. <= 1). Image cannot be displayed.");
			}else
				WARNINGMESSAGE("Non-parametric point cloud. Image cannot be displayed.");
		}
	}//Image Vewer
		
	if (action->text()=="Make copy") {
		SGLegacyPCSceneAdaptor sceneLA(scene);
		const std::vector<card32> selection = sceneLA.getSelectedPCs();
		const card32 n = (card32)selection.size();
		if (n == 0) return;
		for (card32 i=0; i<n; i++) {
			const PointCloud *pc = sceneLA.getPointCloud(selection[i]);
			PointCloud* copy = (PointCloud*)pc->copy();
			if (copy)
			{
				SGObjectNode* node = new SGObjectNode;
				node->clearAndSetup(copy);
				node->setName(selection[i] + "_copy");
				scene->getRootNode()->addChildNode(node);
			}
		}
	}

	if (action->text()=="Convert type") {
		SGLegacyPCSceneAdaptor sceneLA(scene);
		const std::vector<card32> selection = sceneLA.getSelectedPCs();
		const card32 n = (card32)selection.size();
		if (n == 0) return;

		const VertexDescriptor *firstVD = sceneLA.getPointCloud(selection[0])->getDescr();
		for (card32 i=1; i<n; i++) {
			const PointCloud *pc = sceneLA.getPointCloud(selection[i]);
			if (!pc->getDescr()->equal(firstVD)) {
				WARNINGMESSAGE("Selection is inhomogeneous (different vertex descriptors).Cannot convert into one point cloud.");
				return;
			}
		}

		int action;
		bool delOriginal;
		card32 allocatorIndex;
		int maxNumNodesInMem;
		int maxLeafPts;
		int gridSize;

		const card32 numAllocatorsInSGState = 0; // xxx hierarchy removed: scene->getRootState()->staticState->allocators->getNumAllocators();

		PointCloudConversionWidget::execute(this, action, delOriginal, allocatorIndex, maxNumNodesInMem, maxLeafPts, gridSize, numAllocatorsInSGState);
		if (action == PointCloudConversionWidget::CANCEL) return;

		CmdObjConvertType *convert = new CmdObjConvertType(action, delOriginal, allocatorIndex, maxNumNodesInMem, maxLeafPts, gridSize);
		scene->getCommandHistory()->executeCommand(scene, convert, true);

	}

	if (action->text()=="Append fields")
	{
		SGLegacyPCSceneAdaptor sceneLA(scene);
		const std::vector<card32> selection = sceneLA.getSelectedPCs();
		const card32 n = (card32)selection.size();
		if (n == 0) return;
		VertexDescriptor *newFields = new VertexDescriptor();
		if (AppendFieldsDialog::execute(this, newFields)) 
		{
			CmdObjAppendFields *app = new CmdObjAppendFields(newFields);
			scene->getCommandHistory()->executeCommand(scene, app, true);
		} else 
		{
			delete newFields;
		}
		ObjectViewsTable::update(scene);

	}

	if (action->text()=="Remove channel")
	{
		SGLegacyPCSceneAdaptor sceneLA(scene);
		const std::vector<card32> selection = sceneLA.getSelectedPCs();
		const card32 n = (card32)selection.size();
		if (n == 0) return;
		VertexDescriptor *newFields = new VertexDescriptor();
		
		if (AppendFieldsDialog::execute(this, newFields,true)) 
		{
			if (newFields->getNumAttributes())
			{
				std::string cName=newFields->getAttribute(0).name;
				CmdObjPointCloudOperations *rem = new CmdObjPointCloudOperations();
				rem->setup(cName);
				scene->getCommandHistory()->executeCommand(scene, rem, true);
			}
		}
		delete newFields;
		ObjectViewsTable::update(scene);

	}

	if (action->text()=="Clear PC Attachments")
	{
		SGLegacyPCSceneAdaptor sceneLA(scene);
		const std::vector<card32> sel = sceneLA.getSelectedPCs();
		for (card32 i=0; i<sel.size(); i++) {
			PointCloud *pc = sceneLA.getPointCloud(sel[i]);
			pc->clearAttachments();
			// Removed - same effect as above call, right?
			//pc->getAttachments()->clear();
		}
   

	}

	if (action->text()=="Invalidate Hierarchy")
	{
	   // xxx hierarchy removed:
		//SGLegacyPCSceneAdaptor sceneLA(scene);
		//const std::vector<card32> sel = sceneLA.getSelectedPCs();
		//for (card32 i=0; i<sel.size(); i++) {
		//	PointCloud *pc = sceneLA.getPointCloud(sel[i]);
		//	HierarchicalInCorePointCloud *hpc = dynamic_cast<HierarchicalInCorePointCloud*>(pc);
		//	if (hpc != NULL) {
		//		hpc->invalidateNodes();
		//	} else {
		//		HierarchicalOutOfCorePointCloud *hpc2 = dynamic_cast<HierarchicalOutOfCorePointCloud*>(pc);
		//		if (hpc2 != NULL) {
		//			hpc2->invalidateNodes();
		//		}
		//	}
		//}

	}

	if (action->text()=="Shutdown Hierarchy")
	{
      // xxx hierarchy removed:
		//SGLegacyPCSceneAdaptor sceneLA(scene);
		//const std::vector<card32> sel = sceneLA.getSelectedPCs();
		//for (card32 i=0; i<sel.size(); i++) {
		//	PointCloud *pc = sceneLA.getPointCloud(sel[i]);
		//	HierarchicalOutOfCorePointCloud *hpc2 = dynamic_cast<HierarchicalOutOfCorePointCloud*>(pc);
		//	if (hpc2 != NULL) {
		//	hpc2->shutdownAllocator();
		//	}
		//}            

	}

	if (action->text()=="Copy selected clouds")
	{

		set<std::string> &sel = scene->getRootState()->staticState->selectedNodes;
		set<std::string>::iterator pos = sel.begin();
		set<std::string>::iterator end = sel.end();

		while (pos != end) {
			SceneGraphNode* n = getSceneGraphNode(scene, *pos);
			
			PointCloud *pc = getPointCloud(n);
			if (pc != NULL) {
				PointCloud *cp = (PointCloud*)pc->copy();
				std::string newName =  *pos + "_copy";
				addPointCloud(scene, cp, newName);
			}
			++pos;
		}

	}


	if (action->text()=="Toggle (in)visible")
	{


		set<std::string> &sel = scene->getRootState()->staticState->selectedNodes;
		set<std::string>::iterator pos = sel.begin();
		set<std::string>::iterator end = sel.end();

		while (pos != end) {
			SceneGraphNode* n = getSceneGraphNode(scene, *pos++);
			n->setVisible(!n->visible());
		}
		          
	}

	update();
	STD_GLOBAL_EXCEPTION_HANDLER_END
};

void SceneGraphEditorWidget::on_showSceneAttachments_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	AttachedDataContainer *par = scene->getAttachments();
    ClassEditor* attachSceneEd =(ClassEditor*)(par->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance());
    ObjectBrowserWidget* newBrowser = new ObjectBrowserWidget(par, attachSceneEd, this);
	QWidget * w = attachSceneEd->makeEditor( newBrowser,par);
	newBrowser->setupEditor(w);
	newBrowser->show();
	ObjectViewsTable::bindView( par, attachSceneEd );
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void SceneGraphEditorWidget::on_showPcAttachments_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (this->getNumSelNodes()==1 && (dynamic_cast<SGObjectNode*>(this->getSelNode())))
	{
		SceneObject* so=((SGObjectNode*)this->getSelNode())->getSceneObject();
		if (so==NULL)
		{	
		WARNINGMESSAGE("ObjectNode doesn't have a SceneObject associated.");
		}
		else
		{
		AttachedDataContainer* par=so->getAttachments();
		ClassEditor * ce = (ClassEditor*)(par->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance());
		ObjectBrowserWidget* newBrowser = new ObjectBrowserWidget(((SGObjectNode*)this->getSelNode())->getSceneObject()->getAttachments(), ce, this);
		QWidget * w = ce->makeEditor( newBrowser,((SGObjectNode*)this->getSelNode())->getSceneObject()->getAttachments());
		newBrowser->setupEditor(w);
		newBrowser->show();
		ObjectViewsTable::bindView( ((SGObjectNode*)this->getSelNode())->getSceneObject()->getAttachments(), ce );
		}
	}
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void X4::SceneGraphEditorWidget::on_cbMaterials_currentIndexChanged( int index )
{
	if (!changingMaterials) return;
	int matIndex = ui.cbMaterials->currentIndex();
	if (matIndex < 0 || matIndex >= (int)scene->getNumGLMaterials()) return;
	std::vector<SceneGraphNode*> sel = getSelNodes();
	for (size_t i=0; i<sel.size(); i++) {
		SGObjectNode *node = dynamic_cast<SGObjectNode*>(sel[i]);
		int matindex = -1;
		if (node && node->getSceneObject()) {
			node->getSceneObject()->setMaterialIndex(matIndex);
		}
	}
	ui.bpEditMaterial->setEnabled(true);
	ObjectViewsTable::update(scene);
}

void X4::SceneGraphEditorWidget::on_bpEditMaterial_clicked()
{
	GLMaterial *mat = NULL;
	int mati = ui.cbMaterials->currentIndex();
	if (mati>=0 && mati < (int)scene->getNumGLMaterials()) {
		mat = scene->getGLMaterial(mati);
	}
   if( mat )
   {
      ClassEditor * classEditor = (ClassEditor *)mat->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
		QWidget* browser= classEditor->makeEditor(dynamic_cast<QWidget*>(parent()),mat);
		browser->setWindowFlags(Qt::Dialog);

      // UGLY  (needed to set gl context)
      GLShaderMaterialEditorWidget * shmatEditor = dynamic_cast<GLShaderMaterialEditorWidget*>(browser);
		if( shmatEditor ){
			// now: very ugly :-)
			if (parent() 
			&& parent()->parent() 
			&& parent()->parent()->parent() 
			&& parent()->parent()->parent()->parent() 
			&& parent()->parent()->parent()->parent()->parent()) {
				QObject * p = parent()->parent()->parent()->parent()->parent()->parent();// sceneeditorwidget
				SceneEditorWidget *sew = dynamic_cast<SceneEditorWidget*>(p);
				if (sew) {
					shmatEditor->setTestContext( sew->getGLContext() );
					shmatEditor->updateEditor();
				}
			}
		}
		browser->setAttribute(Qt::WA_DeleteOnClose);
		browser->show();
		if (browser->x() < 0 || browser->y() < 0) {
		   QPoint me = mapToGlobal(QPoint(0,0));
		   browser->move(me.x()+100, me.y()+10);
		}
   }
}



#include "SceneGraphEditorWidget_moc.h"
