#ifndef SceneGraphEditorWidgetH 
#define SceneGraphEditorWidgetH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "ClassEditor.h"
#include "SceneGraphEditorForm_ui.h"
#include <QTreeWidgetItem>
#include <QTreeWidget>
//---------------------------------------------------------------------------

namespace X4
{
	class SceneGraphNode;
	class StandardSceneGraphIterator;
	class Scene;
	class ClassEditor;
	class ClassSelectionDialog;
	
	class SGTreeWidgetItem: public QTreeWidgetItem
	{
		
		private:

			SceneGraphNode* m_node;
		public:
			SGTreeWidgetItem(SGTreeWidgetItem * parent ,const QStringList &s,SceneGraphNode* sg_node);
			SGTreeWidgetItem(const QStringList &s,SceneGraphNode* sg_node);
			~SGTreeWidgetItem(void);
			inline SceneGraphNode* getSGNode() {return m_node;};

	};

	class SGTreeWidget:public QTreeWidget
	{
	Q_OBJECT
	public:
		SGTreeWidget(QWidget* parent);
		~SGTreeWidget(void);
		
		

	signals:
	void dropped(QDropEvent *event);
	void pressed(QMouseEvent  *event);
	

	protected:

		void dropEvent(QDropEvent *event);
		void dragEnterEvent(QDragEnterEvent* event);
		void mousePressEvent(QMouseEvent *event);

	
	
	
	
	};

	class X3EDITOR_API SceneGraphEditorWidget : public QWidget
	{
		Q_OBJECT

	private:
		///used to access buttons, etc. from QtDesigner
		Ui_SceneGraphEditorForm ui;


		// user declarations
		struct RememberVisibleItem {
			std::map<std::string, RememberVisibleItem*> children;
			~RememberVisibleItem();
		};

		Scene *scene;
		//  todo TPopupWindow *addNodePopup;
		ClassEditor *nodeEditor;
		QWidget *nodeEditorWidget;
		card32 numItems;
		RememberVisibleItem *lastVisible;
		QMenu *rclickMenu;
		bool rebuilding;
		bool updatingSelection; // mw - added to speed up selection updates

		static const card32 NTS_Add = 1;
		static const card32 NTS_Insert = 2;
		card32 nodeTypeSelectionAction;
		SGTreeWidget* LVSceneGraph;
		bool changingMaterials;
		

		int getNumSelNodes();
		SceneGraphNode *getSelNode();
		std::vector<SceneGraphNode*> getSelNodes();

		void rebuildTree(StandardSceneGraphIterator &it, SGTreeWidgetItem *tviParent, RememberVisibleItem *lastVisible);
		void rebuildLastVisibleRek(SGTreeWidgetItem* node,RememberVisibleItem *current);
		void rebuildLastVisible();

		std::string lookUpIconName(int index);

		SGTreeWidgetItem* getSelTTreeNode();
		std::string getSelNodeNameWithPath();
		std::string getNodeNameWithPath(SGTreeWidgetItem *node);
		SGTreeWidgetItem* getTWIByPath(std::string path);
		bool allSelParentsSimilar();
		void applySelection();
		void updateSelection();

		void buildMenu();
		void updateMatSel(bool multiSelect, int matIndex);
		
	public:
		void update();

		SceneGraphEditorWidget(QWidget * parent , Scene*);
		~SceneGraphEditorWidget(void);

		//void setup( SceneGraph * obj );

		/// update editor content with object data
		void updateEditor();

		public slots:
			void on_BTAdd_clicked();
			void on_BTUpdate_clicked();
			void on_BTUp_clicked();
			void on_BTDown_clicked();
			void on_BTDelete_clicked();
			void on_BTMove_clicked();
			void on_BTLink_clicked();
			void on_BTGroup_clicked();
			void on_BTUngroup_clicked();
			void on_bpEditMaterial_clicked();
			void on_cbMaterials_currentIndexChanged(int index);

			void LVSceneGraph_itemSelectionChanged();
			void LVSceneGraph_itemDoubleClicked(QTreeWidgetItem* twi, int column);
			void LVSceneGraph_pressed(QMouseEvent *event);
			void LVSceneGraph_dropped(QDropEvent *event);
			void rclickMenu_triggered(QAction * action);
			void on_showSceneAttachments_clicked();
			void on_showPcAttachments_clicked();

	protected:
			virtual void showEvent ( QShowEvent * e );

	};

}

#endif

