//---------------------------------------------------------------------------
#ifndef SceneInterfaceH
#define SceneInterfaceH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AttachedData.h"
#include <string>
#include <vector>
//---------------------------------------------------------------------------


class QToolBar;

namespace X4
{
class XGRTEvent;
class RGBAImage;
class VCLToolbar;
class TPCSceneEditor;
class SceneEditorWidget;

class X3EDITOR_API SceneToolbarElement : public Persistent
{
private:
   static MetaClass *SceneToolbarElementMetaClass;

	int32		m_ShortCut_Key;
	int32		m_ShortCut_Modifiers;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

	SceneToolbarElement() {m_ShortCut_Key = -1; m_ShortCut_Modifiers = 0; }

   virtual void assign(const Object* obj, X4CopyContext *context = NULL) {}

   virtual std::string getName() = 0;

	enum EShortCutModifier
	{
		SCM_None = 0x0,
		SCM_Ctrl =	0x1,
		SCM_Alt	=	0x2,
		SCM_Shift = 0x4
	};
	int32 getShortCutKey() const {return m_ShortCut_Key;}
	int32 getShortCutModifiers() const {return m_ShortCut_Modifiers;}
	void setShortCut( int32 key, int32 mod ) {m_ShortCut_Key = key; m_ShortCut_Modifiers = mod; }
	std::string getShortCutDesc();


	virtual void createElement( SceneEditorWidget * w,QToolBar * tb ) = 0;
	virtual void keyEvent( SceneEditorWidget * w ) {}

};

/**
   a scene-tool-button is a button in a scene-toolbar. when
   the user clicks the button, event is sent to the specified
   reciever.
*/
class X3EDITOR_API SceneToolButton : public SceneToolbarElement
{
private:
   static MetaClass *SceneToolButtonMetaClass;
   static Object* newInstance();

   XGRTEvent *  m_Event;
   RGBAImage * m_Icon;
   std::string      m_Tooltip;
   std::string      m_ShortDesc;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   SceneToolButton();
   ~SceneToolButton();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   /// setup button, deletes previous data, toolbutton is owner of event & icon !
   void setup( XGRTEvent * event, RGBAImage * icon, std::string tooltip,
      std::string shortDesc = "" );

   virtual std::string getName();

	virtual void createElement( SceneEditorWidget * w,QToolBar * tb );
	virtual void keyEvent( SceneEditorWidget * w );

};

/**
   CheckBox in Toolbar - sends an event with events context set to checked value
   of the CheckBox
*/
class X3EDITOR_API SceneToolbarCheckBox : public SceneToolbarElement
{
private:
   static MetaClass *SceneToolbarCheckBoxMetaClass;
   static Object* newInstance();

   XGRTEvent *  m_Event;
   std::string      m_Tooltip;
   std::string      m_ShortDesc;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   SceneToolbarCheckBox();
   ~SceneToolbarCheckBox();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   /// setup CheckBox, deletes old m_Event and owns event
   void setup( XGRTEvent * event, std::string short_desc, std::string tooltip );

   virtual std::string getName();

	virtual void createElement( SceneEditorWidget * w,QToolBar * tb );
	virtual void keyEvent( SceneEditorWidget * w );



};

class X3EDITOR_API ToolbarSplitter : public SceneToolbarElement
{
private:
   static MetaClass *ToolbarSplitterMetaClass;
   static Object* newInstance();

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual std::string getName() {return "Splitter";}

	virtual void createElement( SceneEditorWidget * w,QToolBar * tb );

};

class X3EDITOR_API SceneToolbar : public Persistent
{
private:
   static MetaClass *SceneToolbarMetaClass;
   static Object* newInstance();

   std::vector<SceneToolbarElement*>  m_Elements;
   std::string                        m_Name;
   card32                        m_Version;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   SceneToolbar() { m_Version = 1; }
   ~SceneToolbar();
   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   inline void setName( std::string name ) { m_Name = name; }
   inline std::string getName() { return m_Name; }

   inline void incVersion() { m_Version++; }
   inline card32 getVersion() { return m_Version; }

   /// creates and adds a tool button with a ModeChangeEvent
   void addToolButton( std::string className, RGBAImage * icon, std::string tooltip,
      std::string short_desc="" );

   /// creates and adds a tool button with a SpecifiedModeChangeEvent
   void addToolButton( Object * obj, RGBAImage * icon, std::string tooltip,
      std::string short_desc="" );

	void setLastElement_ShortCut( int shortCutKey, int shortCutMod=0 );

   /// creates and adds a tool button with a CommandObjectEvent
	//void addToolButtonCommand( Command * cmd, RGBAImage * icon, std::string tooltip,
   //   std::string short_desc="" );

	virtual QToolBar * createToolbar( SceneEditorWidget * w );
	void buildKeyMap( SceneEditorWidget * w  );


   // object list property interface
   void addElement( Object * obj );
   void deleteElements( card32 begin, card32 end );
   Object * getElement( card32 index );
   void moveElement( card32 oldPos, card32 newPos );
   mpcard getNumElements();
   std::string getElementName(card32 index );


};

class X3EDITOR_API SceneInterface : public AttachedData
{
private:
   static MetaClass *SceneInterfaceMetaClass;
   static Object* newInstance();

   std::vector<SceneToolbar*>      m_Toolbars;
   card32                     m_Version;


public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   SceneInterface() {m_Version = 1; }
   ~SceneInterface();
   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   /// returns interface version ( sum of all toolbar version + this version )
   card32 getVersion();

   /// increase interface version
   void incVersion() {m_Version++;}

	virtual void createInterface( SceneEditorWidget * w );
	void buildKeyMap( SceneEditorWidget * w  );

   void addToolbar( Object * obj );
   void deleteToolbars( card32 begin, card32 end );
   Object * getToolbar( card32 index );
   void moveToolbar( card32 oldPos, card32 newPos );
   mpcard getNumToolbars();
   std::string getToolbarName(card32 index );

   SceneToolbar * getToolbarByName( std::string tbname );

};

}

#endif
