//---------------------------------------------------------------------------
#ifndef XGRTEventH
#define XGRTEventH
//---------------------------------------------------------------------------
#include "x3editor.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include <string>
//---------------------------------------------------------------------------


namespace X4
{

class X3EDITOR_API XGRTEvent : public Persistent
{
private:
   static MetaClass *XGRTEventMetaClass;
   static Object* newInstance();

   std::string   m_Source;
   std::string   m_Dest;

   card32   m_Context;  // non-persistent & undefined, must be set before sending the event

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   inline void setSource( std::string source ) { m_Source = source; }
   inline std::string getSource() {return m_Source;}
   inline void setDest( std::string dest ) { m_Dest = dest; }
   inline std::string getDest() {return m_Dest;}

   inline void setContext( card32 context ) { m_Context = context; }
   inline card32 getContext() { return m_Context; }

};

class X3EDITOR_API ModeChangeEvent : public XGRTEvent
{
private:
   static MetaClass *ModeChangeEventMetaClass;
   static Object* newInstance();

   std::string   m_ModeClassName;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   inline std::string getModeClassName() { return m_ModeClassName; }
   inline void setModeClassName( std::string className ) { m_ModeClassName = className; }
};

class X3EDITOR_API SpecifiedModeChangeEvent : public XGRTEvent
{
private:
   static MetaClass *SpecifiedModeChangeEventMetaClass;
   static Object* newInstance();

   Object   *  m_Object;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   SpecifiedModeChangeEvent();
   ~SpecifiedModeChangeEvent();

   /// deletes old object, then sets obj
   void setObject( Object * obj );
   Object * getObject() { return m_Object; }

};

class CommandObject;

class X3EDITOR_API CommandObjectEvent : public XGRTEvent
{

private:
   static MetaClass *CommandObjectEventMetaClass;
   static Object* newInstance();

   CommandObject   *  m_CommandObject;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   CommandObjectEvent();
   ~CommandObjectEvent();

   /// deletes old object, then sets cmdobj
   void setCommandObject( CommandObject * cmd );
   CommandObject * getCommandObject() {return m_CommandObject; }

};

/**
   StringEvent - Simple Event that holds a std::string
*/

class X3EDITOR_API StringEvent : public XGRTEvent
{
private:
   static MetaClass *StringEventMetaClass;
   static Object* newInstance();

   std::string m_String;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   std::string getString() { return m_String; }
   void setString(std::string str)  { m_String = str; }

};

}

#endif
