//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "BucketSplitPointsAddingAdaptor.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
#include "PointSet.h"
#include "ProgressWindow.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( BucketSplitPointsAddingAdaptor ,0)
{
	BEGIN_CLASS_INIT( BucketSplitPointsAddingAdaptor );
}

BucketSplitPointsAddingAdaptor::BucketSplitPointsAddingAdaptor() {
	this->baseIt = NULL;
	this->cubeSideLength = 1;
	this->position3fOffset = 0;
	this->vertexSize = 0;
}


BucketSplitPointsAddingAdaptor::BucketSplitPointsAddingAdaptor(
	DynamicPointCloudIterator *baseIt, 
	float32 cubeSideLength, 
	std::string baseFileName,
	AAT position3fOffset,
	card32 vertexSize)
{
	this->baseIt = baseIt;
	this->cubeSideLength = cubeSideLength;
	this->baseFileName = baseFileName;
	this->position3fOffset = position3fOffset.getOffset();
	this->vertexSize = vertexSize;
}

FILE *BucketSplitPointsAddingAdaptor::getFile(Vector3i bucket) {
	std::map<Key, Bucket>::iterator pos = fileMapper.find(Key(bucket));
	std::map<Key, Bucket>::iterator end = fileMapper.end();
	if (pos == end) {
		std::string filename = baseFileName + std::string("_") + intToStr(bucket[0])
		                               + std::string("_") + intToStr(bucket[1])
		                               + std::string("_") + intToStr(bucket[2]);
		if (FileExists(filename.c_str())) {
			throw X4Exception("BucketSplitPointsAddingAdaptor::getFile() - filename for temporary file already exists; process interrupted for safety reasons.");
		} else {
			FILE *f = fopen(filename.c_str(), "w+b");
			Bucket b;
			b.file = f;
			b.filename = filename;
			fileMapper.insert(pair<Key, Bucket>(Key(bucket), b));
			return f;
		}
	} else {
		return pos->second.file;
	}
}

void BucketSplitPointsAddingAdaptor::add(const char *newPoint) {
	Vector3f pos = *(Vector3f*)(newPoint + position3fOffset);
	Vector3i intPos;
	intPos[0] = (int32)(pos[0] / cubeSideLength);
	intPos[1] = (int32)(pos[1] / cubeSideLength);
	intPos[2] = (int32)(pos[2] / cubeSideLength);	
	FILE *f = getFile(intPos);
   if (fwrite(newPoint, vertexSize, 1, f) != 1) {
      throw EIOException("BucketSplitPointsAddingAdaptor::add() - could not write all data.");
   }
}

void BucketSplitPointsAddingAdaptor::add(const PointSet *newPoints) {
	for (card32 i=0; i<newPoints->getNumEntries(); i++) {
		add(newPoints->getDataPointer(i));
	}
}

void BucketSplitPointsAddingAdaptor::deleteCurrent() {
	throw X4Exception("BucketSplitPointsAddingAdaptor::deleteCurrent() - delete operations are not supported by the bucket splitter (use only for adding points).");
}

#ifndef TARGET_PLATFORM_MSVC2005FF
#include <cstdio>
#endif

BucketSplitPointsAddingAdaptor::~BucketSplitPointsAddingAdaptor(void) {
	char *buf = new char[vertexSize];
	float numBuckets = fileMapper.size();
	std::map<Key, Bucket>::iterator pos = fileMapper.begin();
	std::map<Key, Bucket>::iterator end = fileMapper.end();
	progressWindow->pushStep(true, "adding buckets");
	int i = 0;
	while (pos != end) {
		FILE *f = pos->second.file;
		#if defined(WIN32) || defined(WIN64)
		_fseeki64(f, 0, SEEK_SET);
		#else
		fseek(f, 0, SEEK_SET);
		#endif
		bool atEnd = false;
		do {
			size_t num = fread(buf, vertexSize, 1, f);
			if (num == 1) {
				baseIt->add(buf);
			} else {
				atEnd = true;
			}
			progressWindow->progress((card32)(float(i)/numBuckets*100.0f));
		} while (!atEnd);
		fclose(f);
#ifdef TARGET_PLATFORM_MSVC2005_FF
	   bool success = DeleteFileA(pos->second.filename.c_str());
#else
	   bool success = (remove(pos->second.filename.c_str()) == 0);
#endif
		x4Assert(success);
		i++;
		++pos;
	}
	delete[] buf;
	progressWindow->popSteps();
}

