//---------------------------------------------------------------------------
#ifndef ImportSwissRangerDataH
#define ImportSwissRangerDataH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {

/**
	Import for "Swiss Ranger" 3D Camera Data Format
*/
class X3EDITOR2_API ImportSwissRangerData : public PointCloudImporterReader {
	X4_CLASS( ImportSwissRangerData )

 private:
   struct SRPoint {
      Vector3f pos;
      float32 intensity;
	  float32 confidence;
   };
   static const card32 SIZE_X = 176;//124;
   static const card32 SIZE_Y = 144;//160;
   card32 pos;
   std::vector<SRPoint> points;
   
   AAT POS;
   AAT INTENS;

 public:
	ImportSwissRangerData();
   virtual bool beginImport(std::string filename, bool readTrianglesIfPossible=false);
   virtual const card8* getNextPoint();
	virtual const card8* getNextTriangle();
   virtual void endImport();
   virtual mpcard getWidth();
	virtual mpcard getHeight();
	virtual mpcard getProgressPercentage();
   virtual bool supportsTriangleImport();
	~ImportSwissRangerData();
};

}

#endif



/*class X3EDITOR2_API RangeScanImporterReader : public Persistent {

 private:
   static MetaClass *RangeScanImporterReaderMetaClass;


 protected:
	/// buffer for point data
   card8* buffer;
	/// buffer for triangle data
   card8* tribuffer;

	// propertys:
	/// width of imported set of points, may be unknown (0) before end of import
   card32 width;
	/// true if width is known
   bool IKnowWidth;
	/// height of imported set of points, may be unknown (0) before end of import
	card32 height;
	/// true if height is known
	bool IKnowHeight;
	/// true if import has been started
   bool ImportStarted;
	/// true if import is finished
	bool ImportFinished;
	/// holds the VertexDescriptor used to import point data
   VertexDescriptor* myVD;
	/// holds the Triangle VertexDescriptor used to import point data, standard NULL, please overwrite if triangles are supported
   VertexDescriptor* myTD;


public:
   RangeScanImporterReader();
   ~RangeScanImporterReader();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	/// returns VertexDescriptor used for import
   const VertexDescriptor* getVertexDescriptor();
	/// returns Triangle VertexDescriptor used for import
   const VertexDescriptor* getTriangleDescriptor();


	/// make preparations to begin import, returns true if successfull
   virtual bool beginImport( std::string filename, bool readTrianglesIfPossible=false ) = 0;
	/// returns next point, according to VertexDescriptor
   virtual const card8* getNextPoint() = 0;
	/// returns next triangle for mesh import
	virtual const card8* getNextTriangle() = 0;
	/// returns true if import is finished
   inline bool isFinished() { return ImportFinished; };
	/// end import, should reset all so import can be started again
   virtual void endImport() = 0;

	/// returns width of imported set of points. attention: if unknown then possibly runs through whole file to find out and resets importer
   virtual card32 getWidth() = 0;
	/// returns height of imported set of points. attention: if unknown then possibly runs through whole file to find out and resets importer
	virtual card32 getHeight() = 0;

	/// returns percentage value of how far import has advanced till yet
	virtual card32 getProgressPercentage() = 0;

	/// returns true if import-reader is capable of returning triangle mesh data
   virtual bool supportsTriangleImport() = 0;
};
*/