//---------------------------------------------------------------------------
#ifndef LegacyClassNameTranslationH
#define LegacyClassNameTranslationH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include <map>
#include <string>
//---------------------------------------------------------------------------

namespace X4 {

class MetaClass;
class Persistent;
class InputObjectStream;

/// This class translates the name old classes into new classes in order to read from old streams.
/// Use the versioning mechanism to provide input handling for legacy classes.
/// Register the handler using the static methods of LegacyClassReadHandler.
class X3EDITOR2_API LegacyClassNameTranslation {
 private:
   static std::map<std::string, const MetaClass*> translations;

 public:
   // --- register handlers globally
   /// register a new legacy translator; the object will be subsequently owned by this class.
   static void registerTranslation(std::string legacyClassName, const MetaClass *currentClass);
   /// query for a legacy read handler; returns NULL if class name is not known.
   static const MetaClass *getCurrentClass(std::string legacyClassName);
};


}

#endif
