//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "LocalModelAttachment.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( LocalModelAttachment ,0)
{
	BEGIN_CLASS_INIT( LocalModelAttachment );
}

LocalModelAttachment::LocalModelAttachment() {}

LocalModelAttachment::~LocalModelAttachment() {}

void LocalModelAttachment::assign(const Object* obj, X4CopyContext *context) {
	AttachedData::assign(obj,context);
	const LocalModelAttachment *other = dynamic_cast<const LocalModelAttachment*>(obj);
	if (other == NULL) throw X4Exception("LocalModelAttachment::assign() - incompatible types.");
	houghMaxima = other->houghMaxima;
}

void LocalModelAttachment::write(OutputObjectStream *s) const {
	s->write(quantization);
	s->writeMaxPlatformCard(houghMaxima.size());
	for (card32 i=0; i<houghMaxima.size(); i++) {
		s->writeMaxPlatformCard(houghMaxima[i].size());
		for (card32 j=0; j<houghMaxima[i].size(); j++) {
			s->write(houghMaxima[i][j]);
		}	
	}
}

void LocalModelAttachment::read(InputObjectStream *s) {
	s->read(quantization);
	mpcard listsize;
	s->readMaxPlatformCard(listsize);
	houghMaxima.resize(listsize);
	for (card32 i=0; i<houghMaxima.size(); i++) {
		mpcard entrysize;
		s->readMaxPlatformCard(entrysize);
		houghMaxima[i].resize(entrysize);
		for (card32 j=0; j<houghMaxima[i].size(); j++) {
			s->read(houghMaxima[i][j]);
		}	
	}
}

void LocalModelAttachment::addMaximaList(const std::vector<HTCorner> &list) {
	houghMaxima.push_back(list);
}

card32 LocalModelAttachment::getNumLists() {
	return (card32)houghMaxima.size();
}

card32 LocalModelAttachment::getNumMaxima(card32 i) {
	return (card32)houghMaxima[i].size();
}

HTCorner LocalModelAttachment::getMaxium(card32 list, card32 index) {
	return houghMaxima[list][index];
}

