//---------------------------------------------------------------------------
#ifndef LocalModelAttachmentH
#define LocalModelAttachmentH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "AttachedData.h"
#include "HoughTransformProcessing.h"
//---------------------------------------------------------------------------

namespace X4 {


/**
	Data object to be attached to a point cloud.
	It stores local geometry models (hough transform data) for each point.
*/
class X3EDITOR2_API LocalModelAttachment : public AttachedData {
	X4_CLASS( LocalModelAttachment )

 private:
	std::vector< std::vector<HTCorner> > houghMaxima;
	card32 quantization;

 public:
	LocalModelAttachment();
	
	void setupLocalModel(float32 quantization) {this->quantization = quantization;}
	card32 getQuantization() {return quantization;}

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

	void addMaximaList(const std::vector<HTCorner> &list);
	card32 getNumLists();
	card32 getNumMaxima(card32 i);
	HTCorner getMaxium(card32 list, card32 index);	

	~LocalModelAttachment();
};

}

#endif