//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "MPIScannerImporter.h"
#include "ClassMethods.h"
#include "DebugOutput.h"
#include "SGObjectNode.h"
#include "ProgressWindow.h"
#include <qtcore/qfileinfo.h>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( MPIScannerImporter, 0 )
{
	BEGIN_CLASS_INIT( MPIScannerImporter )
}

/** 
* Constructor
*/
MPIScannerImporter::MPIScannerImporter()
{
}

/** 
* Destructor
*/
MPIScannerImporter::~MPIScannerImporter()
{
}

/** 
 * Import from filename
 */
SGSimpleAnimationNode* MPIScannerImporter::import( const QString& filename )
{
	// Check if filename exists
	QFileInfo fileInfo( filename );
	if ( !fileInfo.exists() ){
		std::cout << "MPIScannerImporter::import: can't find file!" << endl;
		return NULL;
	}
	progressWindow->pushStep( true, "MPI Scanner import" );
	progressWindow->progress( 0 );
	// Open file
	FILE* file;
	fopen_s( &file, filename.toLatin1(), "rb" );
	// Read header
	StructuredLightHeader header;
	fread( &header, 1, sizeof(StructuredLightHeader), file );
	unsigned long numValues = header.width * header.height;
	StructuredLightFrame frame;
	frame.greyValues = new unsigned char[numValues];
	frame.zValues = new float[numValues];
	frame.xValues = new float[numValues];
	frame.yValues = new float[numValues];
	frame.masks = new unsigned char[numValues];
	frame.numValues = numValues;
	frame.width = header.width;
	frame.height = header.height;
	// Create animation node
	SGSimpleAnimationNode* animNode = new SGSimpleAnimationNode();
	animNode->setName( "data" );
	// Read frames
	for ( int n = 0; n < header.timesteps; n++ ){
		// Create a point cloud
		UnstructuredInCorePointCloud* pc = new UnstructuredInCorePointCloud();
		VertexDescriptor vd;
		vd.pushAttrib( mVAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
		vd.pushAttrib( mVAD( "flags", 1, VAD::DATA_FORMAT_INT32 ) );
		vd.pushAttrib( mVAD( "color", 3, VAD::DATA_FORMAT_FLOAT32 ) );
		debugOutput << "Reading frame " << card32( n + 1 ) << "/" << card32( header.timesteps ) << ".\n";
		debugOutput.flush();
		/// Z values
		StructuredLightFrame frame = StructuredLightFrame();
		if ( header.content & DATA_Z ){
			for ( unsigned long i = 0; i < numValues; i++ ){
				fread( &(frame.zValues[i]), sizeof(float), 1, file );
			}
		}
		/// Masks
		if ( header.content & DATA_MASK ){
			for ( unsigned long i = 0; i < numValues; i++){
				fread( &(frame.masks[i]), sizeof(unsigned char), 1, file );
			}
		}
		/// X and Y values
		if ( header.content & DATA_XY ){
			for ( unsigned long i = 0; i < numValues; i++ ){
				fread( &(frame.xValues[i]), sizeof( float ), 1, file );
			}
			for ( unsigned long i = 0; i < numValues; i++ ){
				fread( &(frame.yValues[i]), sizeof( float ), 1, file );
			}
		}
		/// Grey values
		if ( header.content & DATA_GREY ){
			for ( unsigned long i = 0; i < numValues; i++ ){
				fread( &(frame.greyValues[i]), sizeof( unsigned char ), 1, file );
			}
		}
		std::vector<Vector3f> points; points.clear();
		for ( unsigned int pointIndex = 0; pointIndex < numValues; pointIndex++ ){
			if ( !frame.masks[pointIndex] ){
				points.push_back( makeVector3f( frame.xValues[pointIndex], 
					                            frame.yValues[pointIndex], 
												frame.zValues[pointIndex] ) );
			}
		}

		pc->clearAndSetup( &vd, points.size() );
		SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
		BasicPointCloudIterator* basicIt = dynamic_cast<BasicPointCloudIterator*>( newItSO );
		ModifyablePointCloudIterator* modIt = dynamic_cast<ModifyablePointCloudIterator*>( newItSO );
		AAT POSITION = pc->getAAT( "position" );
		AAT COLOR = pc->getAAT( "color" );
		// Set point positions
		for ( card32 pointCounter = 0; ( pointCounter < points.size() ) && !basicIt->atEnd(); pointCounter++ ){
			Vector3f p = points[pointCounter];
			modIt->set3f( POSITION, p );
			modIt->set3f( COLOR, makeVector3f( 1,1,1 ) );
			basicIt->next();
		}
		// Free allocated mem
		delete newItSO;
		// Append point cloud
		SGObjectNode* frameNode = new SGObjectNode();
		frameNode->clearAndSetup( pc );
		char buffer[3];
		std::string frameName = std::string( "frame" ) + itoa( n, buffer, 10 );
		frameNode->setName( frameName );
		animNode->addNode( frameNode );
		progressWindow->progress( card32( float32( n ) * 100.0f / float32( header.timesteps ) ) );
	}
	// Free allocated mem
	delete [] frame.greyValues;
	delete [] frame.zValues;
	delete [] frame.xValues;
	delete [] frame.yValues;
	delete [] frame.masks;
	fclose( file );
	bool dummy;
	progressWindow->popSteps( &dummy );
	return animNode;
}