#ifndef MPIScannerImporterH
#define MPIScannerImporterH

// Project
#include "LinearAlgebra.h"
#include "UnstructuredInCorePointCloud.h"
#include "Persistent.h"
#include "SGSimpleAnimationNode.h"

namespace X4{

/// Header of Scanner format file
struct StructuredLightHeader {
	/// Unknown
	char ident[16];
	/// Width and height of each frame
	int width, height;
	/// Unknown
	int sensors;
	/// Number of frames
	int timesteps;
	/// Content
	int content;
	/// Unused
	int dummy[3];
};

/// Data from one frame
struct StructuredLightFrame {
	StructuredLightFrame()
	{
		zValues = xValues = yValues = NULL;
		greyValues = masks = NULL;
		width = height = numValues = 0;
	}
	float* zValues;
	float* xValues;
	float* yValues;
	unsigned char* greyValues;
	unsigned char* masks;
	unsigned long numValues;
	unsigned long width;
	unsigned long height;
};

/// Contents flag in the header
enum StructuredLightContents {
	/// z-values
	DATA_Z    = 0x00000002, 
	/// masks
	DATA_MASK = 0x00000004, 
	/// xy values
	DATA_XY   = 0x00000008, 
	/// grey
	DATA_GREY = 0x00000020
};

class MPIScannerImporter : public Persistent
{
	X4_CLASS( MPIScannerImporter ) 

	private:
		
	public:
		/// Constructor
		MPIScannerImporter();
		/// Destructor
		~MPIScannerImporter();

		/// Import from filename
		SGSimpleAnimationNode* import( const QString& filename );
};



}

#endif