#ifndef MajorKDTreeH
#define MajorKDTreeH
//---------------------------------------------------------------------------
#include "x3editor2.h"
#include "Plane.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	kd tree for collision detection
	(was not yet promoted to general)
*/
class X3EDITOR2_API MajorKDTree 
{
public:
	struct TNode
	{
		TNode * child[2];
		Plane splitPlane;
		BoundingBox3f bb;
		std::vector<mpcard> indices;
		TNode() {child[0] = child[1] = NULL;}
		~TNode() {delete child[0];delete child[1];}
		bool isLeaf() const {return child[0] == NULL;}
	};
private:
	struct TElement
	{
		BoundingBox3f bb;
	};

	TNode *		m_RootNode;
	std::vector<TElement>	m_Elements;
	mpcard	m_MaxNumElementsPerNode;

	void setupNode(TNode * node);
	void splitNode(TNode * node, unsigned dim, std::vector<mpcard> &child0_indices, std::vector<mpcard> &child1_indices, std::vector<mpcard> &parent_indices);

public:
	MajorKDTree(void);
	~MajorKDTree(void);

	/// add element with bounding box bb and returns index in tree
	mpcard addElement(const BoundingBox3f &bb);

	/// setup kdtree structure (you may also recompute the kdtree if you have added new elements)
	void setupTree(mpcard maxNumElementsPerNode);

	/// root of kdtree
	TNode * getRootNode() {return m_RootNode;}

	/// add all element indices of subtree
	void getSubtreeIndices(TNode * node, std::vector<mpcard> &result);

	struct TPossibleCollision
	{
		mpcard elementIndex;
		std::vector<mpcard> indices;
	};
	
	/// hierarchical collision detection of a tree transformed with T with this tree
	std::vector<TPossibleCollision> computePossibleCollisions( MajorKDTree * tree, const Matrix4f &T);

	/// collision detection of a transformed bounding box with this tree
	std::vector<mpcard> computePossibleCollision(const BoundingBox3f &bb, const Matrix4f &T);

};

}

#endif