#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ManualImageFeaturesEditorWidget.h"
//---------------------------------------------------------------------------
#include "ManualImageFeatures.h"
#include "RGBAImage.h"
#include "FileDialogs.h"
#include "InterfaceGlobals.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_CLASS_EDITOR( ManualImageFeatures )

ManualImageFeaturesEditorWidget::ManualImageFeaturesEditorWidget(QWidget * parent, Qt::WindowFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//scrollArea = new QScrollArea(ui.frame_2);
	ui.scrollArea->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
   ui.scrollArea->setBackgroundRole(QPalette::Dark);
   ui.imgLabel->setBackgroundRole(QPalette::Dark);
   ui.scrollArea->setWidget(ui.imgLabel);
   zoom=1;
   
   	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));

}

ManualImageFeaturesEditorWidget::~ManualImageFeaturesEditorWidget(void)
{
	;
	
}

void ManualImageFeaturesEditorWidget::setup( ManualImageFeatures * obj )
{
	m_Object = obj;
}

void ManualImageFeaturesEditorWidget::updateEditor()
{
	redraw();
}

void ManualImageFeaturesEditorWidget::on_zoomSB_valueChanged(double val)
{
	zoom=val;
	redraw();
	
}

void ManualImageFeaturesEditorWidget::redraw()
{
	if (m_Object->getImage()->getSizeX()==1) return;
	int w=m_Object->getImage()->getSizeX()*zoom;
	int h=m_Object->getImage()->getSizeY()*zoom;
	
	QImage img(QSize(m_Object->getImage()->getSizeX(),m_Object->getImage()->getSizeY()),QImage::Format_RGB32);

	for( card32 y=0;y<m_Object->getImage()->getSizeY();y++ )
	{
		card32 * ptr = (card32 *)img.scanLine(y);  ;
		card32 * srcPtr;
		srcPtr= (card32 *)m_Object->getImage()->getPixels();
		srcPtr += y*m_Object->getImage()->getSizeX();
		for( card32 x=0;x<m_Object->getImage()->getSizeX();x++ )
		{
			ptr[x] = srcPtr[x];
		}
	}


	ui.imgLabel->resize(w,h);
	QPixmap pm;
	pm=pm.fromImage(img.scaled(w,h,Qt::KeepAspectRatio,Qt::SmoothTransformation));
	ui.imgLabel->setPixmap(pm);

	QPainter pnt(&pm);
	pnt.setPen(Qt::blue);
	
	for (int i=0;i<(int)m_Object->imPoints.size();i++)
	{
		
		
		int x=m_Object->imPoints[i][0]*zoom;
		int y=m_Object->imPoints[i][1]*zoom;
		if(false)
		{
			
			pnt.setPen(Qt::red);
			
		}
		if(i==m_Object->imPoints.size()-1)
			pnt.setPen(Qt::blue);
		
		pnt.drawLine(x-4,y-4,x+4,y+4);
		pnt.drawLine(x-4,y+4,x+4,y-4);
		pnt.drawText(x+5,y,QString::number(i)  );
		
	}


	pnt.setPen(Qt::red);
	
	for (int i=0;i<(int)m_Object->imPoints_reProjected.size();i++)
	{
		
		
		int x=m_Object->imPoints_reProjected[i][0]*zoom;
		int y=m_Object->imPoints_reProjected[i][1]*zoom;
		if(false)
		{
			
			pnt.setPen(Qt::red);
			
		}
		if(i==m_Object->imPoints_reProjected.size()-1)
			pnt.setPen(Qt::red);
		
		pnt.drawLine(x-4,y-4,x+4,y+4);
		pnt.drawLine(x-4,y+4,x+4,y-4);
		pnt.drawText(x+5,y,QString::number(i)  );
		
	}


/*	for (int i=0;i<(int)m_Object->estPoints.size();i++)
	{
		
		pnt.setPen(Qt::green);
		int x=m_Object->estPoints[i][0];
		int y=m_Object->estPoints[i][1];
		if(false)
		{
			
			pnt.setPen(Qt::red);
			
		}
		
		pnt.drawLine(x-4,y-4,x+4,y+4);
		pnt.drawLine(x-4,y+4,x+4,y-4);
		pnt.drawText(x+5,y,QString::number(i)  );
		
	}*/

	pnt.end();
	
	ui.imgLabel->setPixmap(pm);
	



}

void ManualImageFeaturesEditorWidget::mousePressEvent(QMouseEvent* e)
{
	double invZoom=1.f/zoom;
	m_draggedFeature=-1;
	Vector2f p;

	QPoint qp=mapToGlobal(e->pos());
	qp=ui.imgLabel->mapFromGlobal(qp);
	p=makeVector2f(qp.x()*invZoom,qp.y()*invZoom);
	
	if  (e->button()==Qt::LeftButton && (e->modifiers()!=Qt::ShiftModifier ))
	{
		if (p[0]<=m_Object->getImage()->getSizeX()
			&&p[1]<=m_Object->getImage()->getSizeY())
		m_Object->imPoints.push_back(p);
	}
	else
	{
		int i_min=-1;
		float32 d_min=200;
		for (int i=0;i<(int)m_Object->imPoints.size();i++)
		{
			Vector2f q=m_Object->imPoints[i];
			float32 d=(p-q).getSqrNorm();
			if (d<100 && d<d_min)
			{	i_min=i; d_min=d;
	
			}
		}
		if (i_min>=0)
		{
			if(e->button()==Qt::RightButton)
				m_Object->deleteimPoints(i_min);
			else if (e->modifiers()==Qt::ShiftModifier)
				m_draggedFeature=i_min;
		}


	}
	redraw();
		
};

void ManualImageFeaturesEditorWidget::mouseReleaseEvent(QMouseEvent* e)
{
	int i;
	double invZoom=1.f/zoom;
	if (m_draggedFeature>=0)
	{
		 i=m_draggedFeature;
		 Vector2f p;
		QPoint qp=mapToGlobal(e->pos());
		qp=ui.imgLabel->mapFromGlobal(qp);
		p=makeVector2f(qp.x()*invZoom,qp.y()*invZoom);
		if (p[0]<=m_Object->getImage()->getSizeX()
			&&p[1]<=m_Object->getImage()->getSizeY())
			m_Object->imPoints[i]=p;
	}
	m_draggedFeature=-1;
	redraw();

}

void ManualImageFeaturesEditorWidget::mouseMoveEvent(QMouseEvent* e)
{
	
	if (e->modifiers()==Qt::ShiftModifier)
	{
		Vector2i p;
		QPoint qp= ((QWidget*)(ui.imgLabel))->mapFrom(this,e->pos());
		p=makeVector2i(qp.x(),qp.y());
		redraw();
		QPainter pnt((QPaintDevice*)ui.imgLabel->pixmap());
		pnt.setPen(Qt::gray);
		
		int x=p[0];
		int y=p[1];
		pnt.drawLine(x-4,y-4,x+4,y+4);
		pnt.drawLine(x-4,y+4,x+4,y-4);
		pnt.end();

	}
		
}

void ManualImageFeaturesEditorWidget::on_loadImageButton_clicked()
{
	std::string filename = FileDialogs::getOpenFileName(InterfaceGlobals::getMainWindow(), "Open Image...",
	                                               "Bitmap Files (*.bmp; *.jpg; *.png)", NULL, "", FileDialogs::IMPORT_DIR);
   if (filename != "") {
      try {
	      m_Object->getImage()->loadFromFile(filename.c_str());
      } catch (...) {
         error("error loading std::map.");
      }
   } 
   redraw();


}

#include "ManualImageFeaturesEditorWidget_moc.h"
