//---------------------------------------------------------------------------
#ifndef MemoryObjectStreamsH
#define MemoryObjectStreamsH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "ObjectStreams.h"
//---------------------------------------------------------------------------

namespace X4 {


/// Write object to a block of memory.
class X3EDITOR2_API MemoryOutputObjectStream : public OutputObjectStream {
 private:
   std::vector<card8> mem;
   max_platform_card currentPos;
   max_platform_card classTablePointerPos;
   bool closed;

   void checkExpand(card32 length);

 protected:
   virtual void writeClassTableHeader();
   virtual void writeClassTable();

 public:
   /// create memory buffer to write object streams to. Arguments: see base class (OutputObjectStream)
   MemoryOutputObjectStream(bool perObjectClassVersion = false, StreamClassTable *classTable = NULL, bool doWriteClassTable = true);

   virtual void writeBuffer(const void* buf, mpcard length);

   /// close stream and get a pointer to the created memory block (existing until the destructor is called). call this function only once.
   virtual const std::vector<card8> *closeAndGetMemoryBuffer();

   virtual ~MemoryOutputObjectStream();
};



/// Read object from a block of memory.
class X3EDITOR2_API MemoryInputObjectStream : public InputObjectStream {
 private:
   const card8 *mem;
   max_platform_card currentPos;
   max_platform_card memSize;

 protected:
   virtual void readClassTable();

 public:
   /// read object from a block of memory. the specified block must exist until the stream is closed by the destructor of this class.
   MemoryInputObjectStream(const card8 *mem, max_platform_card memSize, StreamClassTable *streamClassTable = NULL);
   virtual void readBuffer(void* buf, mpcard length);
   virtual ~MemoryInputObjectStream();
};


}

#endif
