//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "MultiObjectStream.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



std::vector<MultiObjectStream*> MultiObjectStream::activeStreams;

MetaClass* MultiObjectStream::MultiObjectStreamMetaClass = NULL;

const MetaClass* MultiObjectStream::getInstanceClass() const {
   return getClass();
}

const MetaClass* MultiObjectStream::getClass() {
   if (MultiObjectStreamMetaClass==NULL) {
      MultiObjectStreamMetaClass = new MetaClass("MultiObjectStream", "no description.",
                                                 NULL, sizeof(MultiObjectStream));
   }
   return MultiObjectStreamMetaClass;
}

void MultiObjectStream::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   MultiObjectStreamMetaClass->copyParentProperties();
}

void MultiObjectStream::shutdown() {
   delete MultiObjectStreamMetaClass;
}


// ----


MetaClass* FileBasedMultiObjectStream::FileBasedMultiObjectStreamMetaClass = NULL;

const MetaClass* FileBasedMultiObjectStream::getInstanceClass() const {
   return getClass();
}

const MetaClass* FileBasedMultiObjectStream::getClass() {
   if (FileBasedMultiObjectStreamMetaClass==NULL) {
      FileBasedMultiObjectStreamMetaClass = new MetaClass("FileBasedMultiObjectStream", "no description.",
                                                 NULL, sizeof(FileBasedMultiObjectStream));
   }
   return FileBasedMultiObjectStreamMetaClass;
}

void FileBasedMultiObjectStream::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   FileBasedMultiObjectStreamMetaClass->copyParentProperties();
}

void FileBasedMultiObjectStream::shutdown() {
   delete FileBasedMultiObjectStreamMetaClass;
}


// ----

void MultiObjectStream::write(OutputObjectStream *s) const {
	Persistent::write(s);
   s->write<card32>((card32)globalObjectPointers.size());
   for (card32 i=0; i<globalObjectPointers.size(); i++) {
      s->writeObject(globalObjectPointers[i].obj);
      s->write(globalObjectPointers[i].index);
   }
}

void MultiObjectStream::read(InputObjectStream *s) {
	Persistent::read(s);
   card32 n = 0;
   s->read<card32>(n);
   globalObjectPointers.resize(n);
   for (card32 i=0; i<globalObjectPointers.size(); i++) {
      s->readObject(globalObjectPointers[i].obj);
      s->read(globalObjectPointers[i].index);
   }
}

MultiObjectStream *MultiObjectStream::createOOCAwareCopy(std::string idPrefix, bool insertOriginal, std::string idSuffix) {
	return (MultiObjectStream*)copy();
}


void MultiObjectStream::addGlobalObjectPointer(Persistent *obj, card32 index) {
   GOPEntry e;
   e.obj = obj;
   e.index = index;
   globalObjectPointers.push_back(e);
}

mpcard MultiObjectStream::getNumGlobalObjects() {
   return globalObjectPointers.size();
}

bool MultiObjectStream::hasGlobalObjectPointer(Persistent *obj) {
   for (unsigned i=0; i<globalObjectPointers.size(); i++) {
      if (globalObjectPointers[i].obj == obj) return true;
   }
   return false;
}

void MultiObjectStream::removeGlobalObjectPointer(Persistent *obj) {
   for (unsigned i=0; i<globalObjectPointers.size(); i++) {
      if (globalObjectPointers[i].obj == obj) {
         globalObjectPointers.erase(globalObjectPointers.begin() + i);
         return;
      }
   }
   throw EInvalidState("MultiObjectStream::removeGlobalObjectPointer() - object not found.");
}

MultiObjectStream::~MultiObjectStream() {
	std::vector<MultiObjectStream*>::iterator pos = activeStreams.begin();
	std::vector<MultiObjectStream*>::iterator end = activeStreams.end();
	while (pos != end) {
		if (*pos == this) {
			activeStreams.erase(pos);
			break;
		}
		++pos;
	}
}
