//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PCCApplySOTransformation.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS(PCCApplySOTransformation, 0)
{
	BEGIN_CLASS_INIT(PCCApplySOTransformation)
}

//---------------------------------------------------------------------------

PCCApplySOTransformation::PCCApplySOTransformation() {}

PCCApplySOTransformation::~PCCApplySOTransformation() {}

void PCCApplySOTransformation::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj, this);
}

//---------------------------------------------------------------------------

void PCCApplySOTransformation::execute(Scene *scene) 
{
   std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
	for ( unsigned m = 0; m < selection.size(); m++ ){
		PointCloud *pc = getPointCloud( scene, selection[m] );
		if (!pc) continue;
		applyTransformation(pc);
   }   
}

void PCCApplySOTransformation::applyTransformation(PointCloud *pc) 
{
	Matrix4f transf = pc->getTransformation();
   if (!pc->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) return;
   AAT positionAAT = pc->getDescr()->getToken("position", 3, VAD::DATA_FORMAT_FLOAT32);
   AAT normalAAT = NULL_AAT;
   if (pc->getDescr()->providesAttribute("normal", 3, VAD::DATA_FORMAT_FLOAT32)) {
      normalAAT = pc->getDescr()->getToken("normal", 3, VAD::DATA_FORMAT_FLOAT32);
   }
	SceneObjectIterator *it = pc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC);
	if (!it) return;
	BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
	ModifyablePointCloudIterator *mit = dynamic_cast<ModifyablePointCloudIterator*>(it);
   Matrix3f normalTransf;
	try {
	   normalTransf = invertMatrix(shrink4To3(transf).transpose());
	} catch (...) { // singular matrices
	   normalTransf = shrink4To3(transf);
	}
	while (!bit->atEnd()) {
	   Vector3f pos = bit->get3f(positionAAT);
	   mit->set3f(positionAAT, transformVector3f(transf, pos));
	   if (normalAAT != NULL_AAT) {
         Vector3f normal = normalTransf*bit->get3f(normalAAT);
         mit->set3f(normalAAT, normal);
	   }
	   bit->next();
	}
	pc->setTransformation(IDENTITY4F);
	//pc->getAttachments()->deleteDelOnWriteData(AttachedData::CT_POSITION_CHANGED);
	delete it;
}
