//---------------------------------------------------------------------------
#ifndef PCCComputeTopologyH
#define PCCComputeTopologyH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
#include "X4Types.h"
//---------------------------------------------------------------------------

namespace X4 {

class InCorePCTopologyGraph;


class X3EDITOR2_API PCCComputeTopology : public PCCommand {
   X4_CLASS( PCCComputeTopology )
 
 public:
	static const card32 TOPTYPE_RANDOM = 0;
	static const card32 TOPTYPE_KNN = 1;
	static const card32 TOPTYPE_EPS = 2;
	static const card32 TOPTYPE_KNN_AND_EPS = 3;
	static const card32 TOPTYPE_TRIMESH_CONNECTIVITY = 4;

 private:
	card32 topologyType; 
	float32 epsilon;
	card32 k;
	bool attachToOOCData;
	
	bool triangleMeshConnectivityToTopology;
 
 public:
	PCCComputeTopology();
   virtual void execute(Scene *scene);
	card32 getK(){return k;};
	
	/// needed for external use of compute topology functionality (e.g. in PCCPointModeling)
	inline void setup(card32 toptype = TOPTYPE_KNN, float32 eps = 0.1f, card32 ka = 20) {
		topologyType = toptype;
		epsilon = eps;
		k = ka;
	};

	InCorePCTopologyGraph *computeRandomTopology(PointSet *ps);
	InCorePCTopologyGraph *computeEpsKNNTopology(const PointSet *ps, PointCloud *pc,card32 numKNN = 0x7ffffffe);
	InCorePCTopologyGraph *computeEpsTopology(const PointSet *ps, PointCloud *pc);
	InCorePCTopologyGraph *computeKNNTopology(PointSet *ps, PointCloud *pc);
	InCorePCTopologyGraph *computeTopologyFromMesh(PointSet *ps, PointCloud *pc);

	inline InCorePCTopologyGraph *computeTopology(PointSet* ps, PointCloud* pc)
	{
		if (topologyType == TOPTYPE_RANDOM) return computeRandomTopology(ps);
		else if (topologyType == TOPTYPE_KNN) return computeKNNTopology(ps, pc);
		else if (topologyType == TOPTYPE_EPS) return computeEpsTopology(ps, pc);
		else if (topologyType == TOPTYPE_KNN_AND_EPS) return computeEpsKNNTopology(ps, pc, k);
		else if (topologyType == TOPTYPE_TRIMESH_CONNECTIVITY) return computeTopologyFromMesh(ps, pc);
		return NULL;
	}
	// function used for processing animations
	// unsturcturte incore pointclouds only
	// only with KNN setup
	void execute_ar(Scene *scene, UnstructuredInCorePointCloud* upc);
	InCorePCTopologyGraph *computeKNNTopology_ar(PointSet *ps, UnstructuredInCorePointCloud *upc);


	/// gets existing topology from attachment or creates new one
	InCorePCTopologyGraph * getOrCreateTopology(PointCloud* pc);

};

}

#endif