//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCExportTopology.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "TopologyAttachment.h"
#include "InCorePCTopologyGraph.h"
#include "PointCloudTools.h"
#include "FileNamePropertyEditor.h"
//---------------------------------------------------------------------------
#include <fstream>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( PCCExportTopology , 0 ) {
	BEGIN_CLASS_INIT( PCCExportTopology );
	ADD_STRING_PROP(outputFileName, 0)
   SET_EDITOR(FileNamePropertyEditor::getClass());
}

// ----

PCCExportTopology::PCCExportTopology() {
   outputFileName = "topology.txt";
}

PCCExportTopology::~PCCExportTopology() {
}



void PCCExportTopology::execute(Scene *scene)
{
   std::ofstream out(outputFileName.c_str());
   std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
   for (size_t m = 0; m < selection.size(); m++ ){
      // out << "# topology for object number " << m << "; format: index0 [space] index1 [newline]\n";
      PointCloud *pc = getPointCloud( scene, selection[m] );
      AttachedDataContainer *attachments = pc->getAttachments();
      TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
      if (topAtt) {
         InCorePCTopologyGraph *tg = topAtt->getTopology();
         PointSet *edges = tg->getEdges();
         AAT vindexAAT = edges->getAAT("vindex");
         for (mpcard i=0; i<edges->getNumEntries(); i++) {
            Vector2i vind = edges->get2i(i, vindexAAT);
            out << vind[0] << " " << vind[1] << "\n";
         }
      }
   }   
}

