#ifndef PCCGridResamplerH
#define PCCGridResamplerH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------
#include "HashTables.h"
//---------------------------------------------------------------------------


namespace X4
{

/**
	point cloud resampler
*/
class X3EDITOR2_API PCCGridResampler : public PCCommand
{
	X4_CLASS( PCCGridResampler )

private:

	float32	m_AbsoluteSamplingSpace;
	float32	m_RelativeSamplingSpace;	// relative to median point distance
	bool		m_UseRelativeSamplingSpace;

public:
	struct TGridKey
	{
		int x,y,z;
	};

	struct TGridEntry
	{
		std::vector<card32> vertexIndex;
	};

private:
   class gridComp {
     public:
       static bool equals(TGridKey obj1, TGridKey obj2) 
		 { return obj1.x == obj2.x && obj1.y == obj2.y && obj1.z == obj2.z; }
   };

   class MyHashTable : public HashTable<TGridKey, TGridEntry*, gridComp> {
    protected:
      virtual max_platform_card hashFunction(TGridKey key) const;
    public:
      MyHashTable(card32 fillFactor = 1, card32 startSizeIndex = 8)
			:HashTable<TGridKey, TGridEntry*, gridComp>(fillFactor, startSizeIndex) {}

   };



public:
	PCCGridResampler(void);
	~PCCGridResampler(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual void execute(Scene *scene);

	static PointCloud * resamplePointCloud( PointCloud * pc, float samplingSpace );

};

}

#endif