//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCImportBMP2DCloud.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
#include "BooleanClassProperty.h"
#include "PointCloudTools.h"
#include "UnstructuredInCorePointCloud.h"
#include "VertexDescriptor.h"
#include "SGObjectNode.h"
#include "SGListNode.h"
#include "PointCloudIterators.h"
#include "LinearAlgebra.inline.h"
#include "StringHelper.h"
#include "FileNamePropertyEditor.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* PCCImportBMP2DCloud::PCCImportBMP2DCloudMetaClass = NULL;

const MetaClass* PCCImportBMP2DCloud::getInstanceClass() const {
   return getClass();
}

Object* PCCImportBMP2DCloud::newInstance() {
   return new PCCImportBMP2DCloud;
}

const MetaClass* PCCImportBMP2DCloud::getClass() {
   if (PCCImportBMP2DCloudMetaClass==NULL) {
      PCCImportBMP2DCloudMetaClass = new MetaClass("PCCImportBMP2DCloud", "no description.",
                                  newInstance, sizeof(PCCImportBMP2DCloud));
   }
   return PCCImportBMP2DCloudMetaClass;
}

void PCCImportBMP2DCloud::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   PCCImportBMP2DCloudMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT( PCCImportBMP2DCloud )
	ADD_STRING_PROP(filename,0)
   SET_EDITOR(FileNamePropertyEditor::getClass())
	ADD_STRING_PROP(destFolder,0)
	ADD_BOOLEAN_PROP(darkBackground, 0)
	ADD_BOOLEAN_PROP(extrudeZ, 1)
}

void PCCImportBMP2DCloud::shutdown() {
   delete PCCImportBMP2DCloudMetaClass;
}


// ----


PCCImportBMP2DCloud::PCCImportBMP2DCloud() {
   darkBackground = false;
   destFolder = "root";
   extrudeZ = false;
}

struct MyVertex {
	Vector3f position;
	int32 flags;
};

void PCCImportBMP2DCloud::execute(Scene *scene) {
   SceneGraphNode *destNode = getSceneGraphNodeByName(scene, destFolder);
   if (!destNode) throw X4Exception("PCCImportBMP2DCloud::execute() - dest node not found.");
   SGListNode *list = dynamic_cast<SGListNode*>(destNode);
   if (!list) throw X4Exception("PCCImportBMP2DCloud::execute() - dest node is not a valid container");
   
   QImage img(filename.c_str());
   if (img.width() == 0 || img.height() == 0) throw X4Exception("PCCImportBMP2DCloud::execute() - cannot load image (check filename).");
   
   UnstructuredInCorePointCloud *pc = new UnstructuredInCorePointCloud();
   VertexDescriptor vd;
   vd.pushAttrib(VAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
   vd.pushAttrib(VAD("flags", 1, VAD::DATA_FORMAT_INT32));
   pc->clearAndSetup(&vd, 0);
   
   SGObjectNode *objNode = new SGObjectNode();
   objNode->clearAndSetup(pc);
   objNode->setName(removePathFromFilename(filename));
   
   list->addChildNode(objNode);
   
   PointCloudIterator *it = pc->createIterator(SOIT::CAP_DYNAMIC_PC);
   x4Assert(it);
   DynamicPointCloudIterator *dynIt = dynamic_cast<DynamicPointCloudIterator*>(it);
   x4Assert(dynIt);
   
   const card32 w = img.width();
   const card32 h = img.height();
   card32 zmax = 0.0f;
   if (extrudeZ) {
		zmax = 10;
   }
   for (card32 y=0; y<h; y++) {
      for (card32 x=0; x<w; x++) {
         QColor c = img.pixel(x,y);
         bool dark = (c.red() < 127) && (c.green() < 127) && (c.blue() < 127);
         if ( dark^darkBackground ) {
            float32 xf = (float32)x/(float32)w - 0.5f;
            float32 yf = (float32)y/(float32)h - 0.5f;
            for (card32 z=0; z<=zmax; z++) {
					float32 zf = (float32)z*0.1f + extrudeZ?rnd01()*0.1:0.0f;
					MyVertex vert;
					vert.position = makeVector3f(xf, yf, zf);
					vert.flags = 0;
					dynIt->add((const char*)&vert);
				}
         }
      }   
   }
   delete it;
}

