//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCImportGeo.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ImportTools.h"
#include "PointCloudTools.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "UnstructuredInCorePointCloud.h"
#include "MaterialAttachment.h"
#include "values_vs.h"
//---------------------------------------------------------------------------
#include <fstream>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


static std::string descrSingleColorMode(card32 i) {
   if (i==0) return "average";
   if (i==1) return "last valid color";
   if (i==2) return "nearest cam";
   return "undefined";
}

IMPLEMENT_X4_CLASS( PCCImportGeo ,0) {
	BEGIN_CLASS_INIT( PCCImportGeo );
	ADD_STRING_PROP(filename, 0)
	ADD_STRING_PROP(targetNode, 0)
	ADD_BOOLEAN_PROP(importAllCameraColors, 0)
	ADD_CARD32_PROP_UPDATE_DESCR(singleColorMode, 0, NULL, descrSingleColorMode, 3, NULL)
}

// ----

PCCImportGeo::PCCImportGeo() {
	filename = "D:\\mwand\\data\\waegele\\new data\\Sand-Scan-aussen_klein-100k_Pkt.geo";
	targetNode = "root/pc1";
	importAllCameraColors = true;
	singleColorMode = 0;
}

void PCCImportGeo::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

PCCImportGeo::~PCCImportGeo() {
}


// ---

static std::string getNextNumber( std::string currentFileName )
{
     int32 i = (int32)currentFileName.length()-1;
     bool updated = false;
     while(!updated) {

     //48 ...  57  ASCII
     //0  ...  9   numbers
        if( currentFileName[i] >= 48 && currentFileName[i] <= 56 )
        {  // if between 0..8 : +1
           currentFileName[i]++;
           updated = true;
        }
        else if( currentFileName[i] == 57 ) {
           // if == 9 set 0 goto next higher digit
           currentFileName[i] = 48;
           i--;
        }
        else {
           currentFileName.insert(i+1, 1, '1');
           updated = true;
        }
     }
     return currentFileName;
}


void PCCImportGeo::execute(Scene *scene) 
{
   if (importAllCameraColors) {

      // file format description:
      // ---BeginOfFile---
      // Kameras
      // c x1 y1 z1
      // ...
      // c xn yn zn
      // Punkte
      // p1x p1y p1z c1r c1g c1b c2r c2g c3b ... cnr cng cnb
      // ...
      // pmx pmy pmz c1r c1g c1b c2r c2g c3b ... cnr cng cnb
      // ---EOF---


      // init input file and dummy std::string
      std::ifstream in(filename.c_str());
      std::string x;

      // first line must be "Kameras", otherwise it's the wrong file format!
      in >> x;
      if (x != "Kameras") {
         throw X4Exception("PCCImportGeo::execute() - wrong type of input file!");
      }
      skipToEOL(in);


      // temporary floats for vector3f
      float32 v1,v2,v3;


      // make new cloud with attachments
      UnstructuredInCorePointCloud *pc = new UnstructuredInCorePointCloud();
      AttachedDataContainer *pcAttachments = pc->getAttachments();


      // get list of camera positions
      std::vector<Vector3f> camPos;
      std::vector<float32> camDistances;
      std::string camName = "CamPos001";
      card32 camCount = 0;

      while (!in.eof()) {

         in >> x;

         // if next line is "Punkte" then end of camera positions
         if (x == "Punkte") {
           skipToEOL(in);
           break;
         }

         // read camera coords and add to list
         in >> v1; in >> v2; in >> v3;
         camPos.push_back(makeVector3f(v1,v2,v3));

         // add attachment to cloud
         Vector4fAttachment * camPosAt = new Vector4fAttachment();
         camPosAt->setup(camName, AttachedData::ADF_PERSISTENT);
         camPosAt->setVector(makeVector4f(v1,v2,v3,1));
         pcAttachments->attachData(camPosAt);

         // prepare for next line
         skipToEOL(in);
         camName = getNextNumber(camName);
         camCount++;
      }

      // add distance to distlist
      for (card32 cam=1; cam < camCount; cam++){
        camDistances.push_back(norm(camPos[cam]-camPos[cam-1]));
      };

      // get list of points
      std::vector<Vector3f> points;
      std::vector<Vector3f> color;
      Vector3f pointPos;

      while (!in.eof()) {

         // read point coords
         in >> v1; in >> v2; in >> v3;
         pointPos = makeVector3f(v1,v2,v3);

         // read colors and add to color list
         for (card32 cam=0; cam < camCount; cam++){
           in >> v1; in >> v2; in >> v3;
           color.push_back(makeVector3f(v1,v2,v3));
         };

         int32 lastValid = -1;
         int32 nextValid = -1;

         // gehe durch alle bilder
         bool hasToInterpolate = false;
         size_t startAdr = color.size()-camCount;
         for (card32 cam=0; cam < camCount; cam++){

            // falls cam gltig, step forward
            if (((color[startAdr+cam][0]>0) && (color[startAdr+cam][1]>0) && (color[startAdr+cam][2]>0)))
            {
               if (hasToInterpolate)
               {
                  nextValid = (int32)(startAdr+cam);

                  // falls letzter gltiger punkt vorhanden
                  if (lastValid != -1)
                  {
                     // interpolate between lastValid and nextValid

                     // todo
                     x4Assert(lastValid!=nextValid);

                     float32 sumDist = 0.0f;
                     for (int32 i=lastValid; i < nextValid; i++) {
                        sumDist += camDistances[i-startAdr];
                     }

                     for (int32 i=lastValid+1; i < nextValid; i++) {

                        float32 g1=0.0f;
                        for (int32 j1=lastValid; j1 < i; j1++) {
                           g1 += camDistances[j1-startAdr];
                        };
                        g1 /= sumDist;

                        float32 g2=0.0f;
                        for (int32 j2=i; j2 < nextValid; j2++) {
                           g2 += camDistances[j2-startAdr];
                        };
                        g2 /= sumDist;

                        color[i] = ((color[lastValid])*g2 + (color[nextValid])*g1);

                     };

                     lastValid = nextValid;
                     hasToInterpolate = false;
                  }
                  // falls kein letzter gltiger punkt vorhanden
                  else
                  {
                     // alles vorherige mit aktuellem punkt auffllen
                     for (int32 i=(int32)startAdr; i < nextValid; i++){
                        color[i] = color[nextValid];
                     };

                     lastValid = nextValid;
                     hasToInterpolate = false;

                  }
               }
               else // !hasToInterpolate
               {
                  lastValid = nextValid;
                  nextValid = (int32)(startAdr+cam);
               };


            }
            // cam ungltig
            else
            {
               if (!hasToInterpolate) {
                  lastValid = nextValid;
                  hasToInterpolate = true;
               }
               else
               {
               };
            };

         };


         if (hasToInterpolate)
         {
            if ((lastValid==-1) && (nextValid==-1))
            {
               for (card32 cam=0; cam < camCount; cam++)
               {
                  color.pop_back();
               };
            }
            else
            {
               // alles nachfolgende mit letztem aktuellen punkt auffllen
               for (int32 i=lastValid; i < (int32)startAdr+(int32)camCount; i++){
                  color[i] = color[lastValid];
               };
               points.push_back(pointPos);
            };

         }
         else
         {
            points.push_back(pointPos);
         };

         skipToEOL(in);
       }


      // to-do: correct normal calculation
      Vector3f z_normal = makeVector3f(0.0f, 0.0f, 1.0f);


      // make new point set
      PointSet *result = new PointSet();
      VertexDescriptor vd;
      vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
      vd.pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
      vd.pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
      std::string channel = "color001";
      for (card32 i=0; i<camCount; i++)
      {
         vd.pushAttrib(mVAD(channel, 3, VAD::DATA_FORMAT_FLOAT32));
         channel = getNextNumber(channel);
      };
      result->clearAndSetup(1, points.size(), &vd);

      std::vector<AAT> COLORList;
      channel = "color001";
      for (card32 i=0; i<camCount; i++)
      {
         const AAT COLOR = vd.getToken(channel);
         COLORList.push_back(COLOR);
         channel = getNextNumber(channel);
      };

      // finally write points to point set
      const AAT POSITION = vd.getToken("position");
      const AAT FLAGS = vd.getToken("flags");
      const AAT NORMAL = vd.getToken("normal");
      StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

      for (card32 n=0; n<points.size(); ++n)
      {
         result->set<float32, 3>(n , POSITION, points[n]);
         result->set<int32, 1>(n , FLAGS, NULL_FLAG);
         result->set<float32, 3>(n , NORMAL, z_normal);

         for (card32 i=0; i<camCount; i++)
         {
            result->set<float32, 3>(n , COLORList[i], color[n*camCount+i]);
         };

      }

      pc->setPointSet(result);

      PointCloud *oldPC = getPointCloud(scene, targetNode);
      if (oldPC) {
         replacePointCloud(scene, targetNode, pc);
      } else {
         addPointCloud(scene, pc, targetNode);
      }

   } else {

      // file format description:
      // ---BeginOfFile---
      // Kameras
      // c x1 y1 z1
      // ...
      // c xn yn zn
      // Punkte
      // p1x p1y p1z c1r c1g c1b c2r c2g c3b ... cnr cng cnb
      // ...
      // pmx pmy pmz c1r c1g c1b c2r c2g c3b ... cnr cng cnb
      // ---EOF---


      // init input file and dummy std::string
      std::ifstream in(filename.c_str());
      std::string x;

      // first line must be "Kameras", else wrong file format!
      in >> x;
      if (x != "Kameras") {
         throw X4Exception("PCCImportGeo::execute() - wrong type of input file!");
      }
      skipToEOL(in);


      // temporary floats for vector3f
      float32 v1,v2,v3;


      // make new cloud with attachments
      UnstructuredInCorePointCloud *pc = new UnstructuredInCorePointCloud();
      AttachedDataContainer *pcAttachments = pc->getAttachments();


      // get list of camera positions
      std::vector<Vector3f> camPos;
      std::string camName = "CamPos001";
      card32 camCount = 0;

      while (!in.eof()) {

         in >> x;

         // if next line is "Punkte" then end of camera positions
         if (x == "Punkte") {
           skipToEOL(in);
           break;
         }

         // read camera coords and add to list
         in >> v1; in >> v2; in >> v3;
         camPos.push_back(makeVector3f(v1,v2,v3));

         // add attachment to cloud
         Vector4fAttachment * camPosAt = new Vector4fAttachment();
         camPosAt->setup(camName, AttachedData::ADF_PERSISTENT);
         camPosAt->setVector(makeVector4f(v1,v2,v3,1));
         pcAttachments->attachData(camPosAt);

         // prepare for next line
         skipToEOL(in);
         camName = getNextNumber(camName);
         camCount++;
      }


      // get list of points
      std::vector<Vector3f> points;
      std::vector<Vector3f> color;
      std::vector<Vector4f> validColorsForThisPoint;
      Vector3f pointPos;

      while (!in.eof()) {

         // read point coords
         in >> v1; in >> v2; in >> v3;
         pointPos = makeVector3f(v1,v2,v3);

         // read colors and add to valid-colors-for-this-point list
         validColorsForThisPoint.clear();
         for (card32 cam=0; cam < camCount; cam++){
           in >> v1; in >> v2; in >> v3;
           if ((v1 >= 0) && (v2 >= 0) && (v3 >= 0))
              validColorsForThisPoint.push_back(makeVector4f(v1,v2,v3,cam));
         };

         // 0 = average, 1 = last valid color, 2 = nearest cam
         switch (singleColorMode)
         {
            case 1: {
                        float32 n = validColorsForThisPoint.size()-1;
                        if (n<0) break;
                        v1 = validColorsForThisPoint[n][0];
                        v2 = validColorsForThisPoint[n][1];
                        v3 = validColorsForThisPoint[n][2];
                        color.push_back(makeVector3f(v1,v2,v3));
                        points.push_back(pointPos);
                    };
                    break;
            case 2: {
                        int32 nearestCamColorNr = -1;
                        float32 min_length = MAXFLOAT;

                        for (card32 i=0; i<validColorsForThisPoint.size(); i++)
                        {
                           int32 camNum = validColorsForThisPoint[i][3];
                           float32 length = normQuad(pointPos-camPos[camNum]);

                           if (length < min_length) {
                              min_length = length;
                              nearestCamColorNr = i;
                           };
                        };

                        if (nearestCamColorNr!=-1) {
                           v1 = validColorsForThisPoint[nearestCamColorNr][0];
                           v2 = validColorsForThisPoint[nearestCamColorNr][1];
                           v3 = validColorsForThisPoint[nearestCamColorNr][2];
                           color.push_back(makeVector3f(v1,v2,v3));
                           points.push_back(pointPos);
                        };

                    };
                    break;
            default: {
                        if (validColorsForThisPoint.size()<=0) break;
                        Vector3f tempCol = makeVector3f(0,0,0);
                        for (card32 n=0; n < validColorsForThisPoint.size(); n++){
                           v1 = validColorsForThisPoint[n][0];
                           v2 = validColorsForThisPoint[n][1];
                           v3 = validColorsForThisPoint[n][2];
                           tempCol += makeVector3f(v1,v2,v3);
                        };
                        color.push_back(tempCol/validColorsForThisPoint.size());
                        points.push_back(pointPos);
                     };
                     break;
         };

         skipToEOL(in);
       }


      // to-do: correct normal calculation
      Vector3f z_normal = makeVector3f(0.0f, 0.0f, 1.0f);


      // make new point set
      PointSet *result = new PointSet();
      VertexDescriptor vd;
      vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
      vd.pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
      vd.pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
      vd.pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
      result->clearAndSetup(1, points.size(), &vd);

      // finally write points to point set
      const AAT POSITION = vd.getToken("position");
      const AAT FLAGS = vd.getToken("flags");
      const AAT COLOR = vd.getToken("color");
      const AAT NORMAL = vd.getToken("normal");
      StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

      for (card32 n=0; n<points.size(); ++n)
      {
         result->set<float32, 3>(n , POSITION, points[n]);
         result->set<float32, 3>(n , COLOR, color[n]);
         result->set<int32, 1>(n , FLAGS, NULL_FLAG);
         result->set<float32, 3>(n , NORMAL, z_normal);
      }

      pc->setPointSet(result);

      PointCloud *oldPC = getPointCloud(scene, targetNode);
      if (oldPC) {
         replacePointCloud(scene, targetNode, pc);
      } else {
         addPointCloud(scene, pc, targetNode);
      }
   }
   
}
