//---------------------------------------------------------------------------
#ifndef PCCImportMPIStructuredLightH
#define PCCImportMPIStructuredLightH
//---------------------------------------------------------------------------
#include "x3editor2.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------

namespace X4 {

/// Header of Scanner format file
struct StructuredLightHeader {
	/// Unknown
	char ident[16];
	/// Width and height of each frame
	int width, height;
	/// Unknown
	int sensors;
	/// Number of frames
	int timesteps;
	/// Content
	int content;
	/// Unused
	int dummy[3];
};

/// Contents flag in the header
enum StructuredLightContents {
	/// z-values
	DATA_Z    = 0x00000002, 
	/// masks
	DATA_MASK = 0x00000004, 
	/// xy values
	DATA_XY   = 0x00000008, 
	/// grey
	DATA_GREY = 0x00000020
};

/// Data from one frame
struct StructuredLightFrame {
	float* zValues;
	float* xValues;
	float* yValues;
	unsigned char* greyValues;
	unsigned char* masks;
	unsigned long numValues;
	unsigned long width;
	unsigned long height;
};


/// Importer for the MPI SB structured light scanner (O. Schall, Spok Hand)
class X3EDITOR2_API PCCImportMPIStructuredLight : public PCCommand 
{
	X4_CLASS( PCCImportMPIStructuredLight )

	private:
		int32 startFrame;
		int32 endFrame;
		std::string inputFilename;
		bool computeNormals;

	public:
		PCCImportMPIStructuredLight();
		~PCCImportMPIStructuredLight();

		virtual void execute(Scene *scene);
};

}

#endif
