//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCKMeansClustering.h"
//---------------------------------------------------------------------------
#include "PointCloudTools.h"
#include "KMeansClustering.inline.h"
#include "PointCloudIterators.h"
#include "StringClassProperty.h"
#include "NumericalClassProperty.h"
#include "BooleanClassProperty.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCCKMeansClustering , 2)
{
	BEGIN_CLASS_INIT( PCCKMeansClustering );
	ADD_STRING_PROP(input, 0)
	ADD_STRING_PROP(output, 0)
	ADD_CARD32_PROP(numClusters, 0)
	ADD_BOOLEAN_PROP(colorizeInput, 1)
	ADD_CARD32_PROP(numTrials, 2);
}


PCCKMeansClustering::PCCKMeansClustering()
{
	input = "root/testClusters";
	output = "root/target";
	numClusters = 3;
	colorizeInput = true;
	numTrials = 100;
}

static Vector3f getNiceColor(card32 i) {
	if (i>5) return makeVector3f(1,1,1);
	switch (i) {
		case 0: return makeVector3f(1,0,0);
		case 1: return makeVector3f(0,1,0);
		case 2: return makeVector3f(0,0,1);
		case 3: return makeVector3f(1,1,0);
		case 4: return makeVector3f(0,1,1);
		case 5: return makeVector3f(1,0,1);
		default: return makeVector3f(1,1,1);
	}
}

void PCCKMeansClustering::execute(Scene *scene) {
	if (numClusters < 1) throw X4Exception("PCCKMeansClustering::execute() - numClusters must be >= 1.");
   PointCloud *pc_input = getPointCloud(scene, input);
	if (pc_input == NULL) throw X4Exception("PCCKMeansClustering::execute() - input not found.");
   PointCloud *pc_output = getPointCloud(scene, output);
	if (pc_output == NULL) throw X4Exception("PCCKMeansClustering::execute() - output not found.");
	
	pc_output->clear();
	if (!pc_input->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
		throw X4Exception("PCCKMeansClustering::execute() - input does not provide position3f channel.");
	}
	AAT POS_IN = pc_input->getAAT("position");

	if (!pc_output->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
		throw X4Exception("PCCKMeansClustering::execute() - output does not provide position3f channel.");
	}
	AAT POS_OUT = pc_output->getAAT("position");
	VAD POS_OUT_VAD = pc_output->getDescr()->getAttribute("position");

	AAT COL_IN = NULL_AAT;
	if (pc_input->getDescr()->providesAttribute("color", 3, VAD::DATA_FORMAT_FLOAT32)) {
		COL_IN = pc_input->getAAT("color");
	}
	
	SceneObjectIterator *it = pc_input->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC);
	BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
	ModifyablePointCloudIterator *mit = dynamic_cast<ModifyablePointCloudIterator*>(it);

	SceneObjectIterator *it2 = pc_output->createIterator(SOIT::CAP_DYNAMIC_PC);
	DynamicPointCloudIterator *dynit = dynamic_cast<DynamicPointCloudIterator*>(it2);

	if (bit != NULL && dynit != NULL) {
		KMeansClustering<float32> clusterer;

		DVectorF pos(3);
		while (!bit->atEnd()) {
			Vector3f pos_s = bit->get3f(POS_IN);
			pos[0] = pos_s[0]; pos[1] = pos_s[1]; pos[2] = pos_s[2];
			clusterer.add(pos);
			bit->next();
		}
		if (clusterer.getDataSize() < 1) throw X4Exception("PCCKMeansClustering::execute() - input is empty.");

		std::vector< DVectorF > result;
		std::vector<unsigned> indices;
		clusterer.calcEx(numClusters, result, indices, 0.01f, numTrials);
		
		card32 vsize = pc_output->getDescr()->getSize();
		card8 *buffer = new card8[vsize];
		ZeroMemory(buffer, vsize);
		for (unsigned i=0; i<result.size(); i++) {
			Vector3f pos = makeVector3f(result[i][0], result[i][1], result[i][2]);
			POS_OUT_VAD.setAttrib<float32, 3>(buffer, pos, POS_OUT);
			dynit->add((char*)buffer);
		}
		
		if (colorizeInput && mit && COL_IN != NULL_AAT) {
			bit->reset();
			int j = 0;
			while (!bit->atEnd()) {
				mit->set3f(COL_IN, getNiceColor(indices[j]));
				bit->next();
				j++;
			}
		}
		
	} else {
		delete it;
		delete it2;
		throw X4Exception("PCCKMeansClustering::execute() - input is not a \"basic\" point cloud or output is not dynamically modifyable.");
	}	
	delete it;
	delete it2;
}
   
PCCKMeansClustering::~PCCKMeansClustering() {}


