//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCLargeTestScene.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
#include "StringClassProperty.h"
#include "ObjectViewsTable.h"
#include "PointCloudTools.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "ProgressWindow.h"
#include "Timer.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCCLargeTestScene ,0)
{
	BEGIN_CLASS_INIT( PCCLargeTestScene );
	ADD_FLOAT32_PROP(millionPtsToCreate, 0)
	ADD_STRING_PROP(target, 0)
	ADD_BOOLEAN_PROP(showProgress, 0)
}

PCCLargeTestScene::PCCLargeTestScene(void) {
	millionPtsToCreate = 10.0f;
	showProgress = true;
}

void PCCLargeTestScene::execute(Scene *scene) {
	PointCloud *pc = getPointCloud(scene, target);
	if (pc == NULL) {
		throw ECommandFailed("PCCLargeTestScene::execute - target not found.");
	} else {
		SceneObjectIterator *it = pc->createIterator(SOIT::CAP_DYNAMIC_PC);
		DynamicPointCloudIterator *dIt = dynamic_cast<DynamicPointCloudIterator*>(it);
		if (dIt == NULL) {
			delete it;
			throw ECommandFailed("PCCLargeTestScene::execute - cannot create dynamic iterator for selected point cloud.");
		}
		const VertexDescriptor *vd = pc->getDescr();
		const AAT AAT_POS = vd->getToken("position");
		VAD posVad = vd->getAttribute("position");
		const card64 n = (card64)((long double)millionPtsToCreate * (long double)1000000.0);
		char *buffer = new char[vd->getSize()];
		vd->zero(buffer);
		if (showProgress) progressWindow->pushStep(true, "creating points");
		Timer t;
		for (card64 i=0; i<n; i++) {
			long double t = (long double)i / 100000.0;
			Vector3f pos;
			pos[0] = sin(t*M_PI*2) + 0.1*sin(10*t*M_PI*2) + 0.01*sin(100*t*M_PI*2);
			pos[1] = cos(t*M_PI*2) + 0.1*cos(10*t*M_PI*2) + 0.01*cos(100*t*M_PI*2);
			pos[2] = t; 
			posVad.setAttrib<float32, 3>((card8*)buffer, pos, AAT_POS);
			dIt->add(buffer);	  			
			if (showProgress) {
				if (i%10000 == 0) {
					if (!progressWindow->progress((card32)(double(i)/double(n)*100.0f))) break;
				}
			}
		}
		delete it;
		debugOutput << "\nPCCLargeTestScene::execute() - time: " << t.getDeltaValue() / 1000.0 << "\n\n";
		if (showProgress) progressWindow->popSteps();
		delete[] buffer;
	}	
}


PCCLargeTestScene::~PCCLargeTestScene(void) {}

