#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCLevelSetExtractor.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "MarchingCubes.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "VolumeObject.h"
#include "VertexArray.h"
#include "ProgressWindow.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCCLevelSetExtractor ,0)
{
	BEGIN_CLASS_INIT( PCCLevelSetExtractor );

	ADD_FLOAT32_PROP( m_IsoValue ,0 );
}

PCCLevelSetExtractor::PCCLevelSetExtractor(void)
{
	m_IsoValue = 1.0f;
}

PCCLevelSetExtractor::~PCCLevelSetExtractor(void)
{
}

void PCCLevelSetExtractor::assign(const Object* obj, X4CopyContext *context)
{
	PCCommand::assign(obj,context);
	const PCCLevelSetExtractor * o = dynamic_cast<const PCCLevelSetExtractor*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

bool PCCLevelSetExtractor_isempty(void *context, const BoundingBox3f &bb)
{
	return false;
}

float PCCLevelSetExtractor_eval(void *context, float x,float y,float z)
{
	PCCLevelSetExtractor::TEvalContext * ctxt = (PCCLevelSetExtractor::TEvalContext*)context;
	Vector3f coord = (makeVector3f(x,y,z) + ctxt->addPos).componentProduct( ctxt->scalePos );
	ctxt->vIt->goTo(coord[0],coord[1],coord[2]);
	return ctxt->bIt->get1f(ctxt->aat);

	/*Vector3f center = ctxt->vIt->getVolumeRegion().getCenter();
	float size = ctxt->vIt->getVolumeRegion().getMaxSideLength();
	return norm(makeVector3f(x,y,z) - center) - size * 0.5f;*/
}

void X4::PCCLevelSetExtractor::execute(Scene * scene)
{
	// get selected point cloud
	std::vector<std::string> sel = getScene()->getRootState()->staticState->getSelectedNodes();
	std::vector<UnstructuredInCoreTriangleMesh*> meshes;
	std::vector<std::string> mesh_names;
	for( unsigned i=0;i<sel.size();i++ )
	{
		VolumeObject * v = dynamic_cast<VolumeObject*>(
			getPointCloud(getScene(), sel[i]));
		if( v )
		{
			meshes.push_back( createTriangleMesh(v,m_IsoValue) );
			mesh_names.push_back( sel[i]+"_mesh" );
		}
	}
	
	for( unsigned i=0;i<meshes.size();i++ )
		addPointCloud(scene,meshes[i],mesh_names[i]);
}

UnstructuredInCoreTriangleMesh * X4::PCCLevelSetExtractor::createTriangleMesh( VolumeObject * pc, float isoValue )
{
	SceneObjectIterator * iter = pc->createIterator(SOIT::CAP_BASIC_PC);
	VolumetricIterator * vIt = dynamic_cast<VolumetricIterator*>(iter);
	if( iter == NULL )
		X4Exception("Couldn't create a volume iterator!");

	HierarchicalIterator * hIt = dynamic_cast<HierarchicalIterator*>(iter);
	if( hIt )
	{
		while( hIt->goToChild(0) ) {}
	}

	TEvalContext context;
	context.aat = pc->getAAT("intensity");
	context.vIt = vIt;
	context.bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	
	BoundingBox3f bb = context.vIt->getVolumeRegion();
	Vector3i dim = context.vIt->getVolumeSize();
	context.addPos = -bb.lowerCorner;
	context.scalePos = makeVector3f((float)(dim[0]-1) / bb.getSideLength(0),
		(float)(dim[1]-1) / bb.getSideLength(1),
		(float)(dim[2]-1) / bb.getSideLength(2));
	
	float cellsize = bb.getMaxSideLength() / max((float)context.vIt->getVolumeSize()[0],
		max((float)context.vIt->getVolumeSize()[1],(float)context.vIt->getVolumeSize()[2]));

	
	
	MarchingCubes mc;
	mc.setGridSnappingTolerance(0.01f);
	//mc.calculateHierarchical(PCCLevelSetExtractor_eval, &context, PCCLevelSetExtractor_isempty, vIt->getVolumeRegion(), 
	//	vIt->getVolumeSize(), isoValue );
	//mc.calculateVolume(PCCLevelSetExtractor_eval, &context, vIt->getVolumeRegion(), 
	//		vIt->getVolumeSize()-makeVector3i(1,1,1), isoValue );

	Vector3f scaleBB = bb.upperCorner - bb.lowerCorner;
	scaleBB[0] /= (float)dim[0];
	scaleBB[1] /= (float)dim[1];
	scaleBB[2] /= (float)dim[2];
	progressWindow->pushStep();
	for( int x=0;x<dim[0]-1;x++)
	{
		for( int y=0;y<dim[1]-1;y++)
		{
			for( int z=0;z<dim[2]-1;z++)
			{
				float values[8];
				vIt->goTo(x,y,z);
				values[0] = context.bIt->get1f(context.aat);
				vIt->goTo(x+1,y,z);
				values[1] = context.bIt->get1f(context.aat);
				vIt->goTo(x+1,y+1,z);
				values[2] = context.bIt->get1f(context.aat);
				vIt->goTo(x,y+1,z);
				values[3] = context.bIt->get1f(context.aat);
				vIt->goTo(x,y,z+1);
				values[4] = context.bIt->get1f(context.aat);
				vIt->goTo(x+1,y,z+1);
				values[5] = context.bIt->get1f(context.aat);
				vIt->goTo(x+1,y+1,z+1);
				values[6] = context.bIt->get1f(context.aat);
				vIt->goTo(x,y+1,z+1);
				values[7] = context.bIt->get1f(context.aat);
				BoundingBox3f currentBB;
				currentBB.lowerCorner = makeVector3f(x,y,z);
				currentBB.upperCorner = makeVector3f(x+1,y+1,z+1);
				currentBB.lowerCorner = currentBB.lowerCorner.componentProduct(scaleBB) + bb.lowerCorner;
				currentBB.upperCorner = currentBB.upperCorner.componentProduct(scaleBB) + bb.lowerCorner;

				mc.calculateCube(values, currentBB, isoValue );
			}

		}
		progressWindow->progress(x*100/dim[0]);

	}
	progressWindow->popSteps();

	delete iter;

	// create output point cloud
	UnstructuredInCoreTriangleMesh * mesh = new UnstructuredInCoreTriangleMesh;
	mesh->setMaterialIndex( 5 );

	VertexDescriptor vdesc;
	vdesc.pushAttrib( VAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
	vdesc.pushAttrib( VAD( "normal", 3, VAD::DATA_FORMAT_FLOAT32 ) );

	VertexDescriptor idesc;
	idesc.pushAttrib( VAD( "index", 3, VAD::DATA_FORMAT_INT32 ) );

	mesh->clearAndSetup( &vdesc, mc.getNumVertices(), &idesc, mc.getNumTriangles() );
	VertexArray vertices( mesh->getPointSet() );
	VertexArray indices( mesh->getTriangles() );
	for( card32 i=0;i<mc.getNumVertices();i++ )
	{
		vertices.setPosition3f( i, mc.getVertex(i) );
	}
	for( card32 i=0;i<mc.getNumTriangles();i++ )
	{
		indices.setIndex3i( i, mc.getTriangle(i) );
	}

	/*mesh->snapMesh( cellsize * 0.05f );
	mesh->snapMesh( cellsize * 0.05f );
	mesh->snapMesh( cellsize * 0.05f );*/
	mesh->closeMesh();
	mesh->calculateVertexNormals();
	VertexArray vArr2(mesh);
	for( card32 i=0;i<vArr2.getNumElements();i++ )
	{
		vArr2.setNormal3f( i, -vArr2.getNormal3f(i) );
	}
	return mesh;
}