//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCNoisySampling.h"
//---------------------------------------------------------------------------
#include "Random.h"
#include "NormalDistribution.h"
#include "BooleanClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "NumericalClassProperty.h"
#include "PointCloudIterators.h"
#include "UnstructuredInCorePointCloud.h"
#include "VertexDescriptor.h"
#include "PointSet.h"
#include "PointCloud.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


IMPLEMENT_X4_CLASS(PCCNoisySampling, 0)
{
	BEGIN_CLASS_INIT(PCCNoisySampling)
	ADD_CARD32_PROP(expectedNumSamples, 0)
	ADD_CARD32_PROP(numOutliers, 0)
	ADD_BOOLEAN_PROP(noiseXY, 0)
	ADD_BOOLEAN_PROP(noiseZ, 0)
	ADD_FLOAT32_PROP(noiseStdDeviation, 0)
	ADD_STRING_PROP(targetName, 0)
}

PCCNoisySampling::PCCNoisySampling() {
 	expectedNumSamples = 1000;
	numOutliers = 0;
	noiseXY = true;
	noiseZ = false;
	noiseStdDeviation = 0.01f;
	targetName = "someNewNoisyPC";
}

void PCCNoisySampling::createPoint(Vector3f pos, PointSet *ps) {
	if (noiseXY) {
		pos[0] += (float32)NormalDistribution::randomNumber(0.0, noiseStdDeviation);
		pos[1] += (float32)NormalDistribution::randomNumber(0.0, noiseStdDeviation);		
	}
	if (noiseZ) {
		pos[2] += (float32)NormalDistribution::randomNumber(0.0, noiseStdDeviation);
	}
	ps->addPoint((const char*)&pos);
}

void PCCNoisySampling::execute(Scene *scene) {
   std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
	if (selection.size() == 0) return;
	PointCloud *templ = getPointCloud(scene, selection[0]);
	if (templ == 0) return;
	
	UnstructuredInCorePointCloud *result = new UnstructuredInCorePointCloud(); 
	result->clearAndSetup(templ->getDescr(),0);
	PointSet *resultPS = result->getPointSet();
   unsigned int m;
   for ( m = 0; m < selection.size(); m++ ) {
      PointCloud *pc = getPointCloud(scene, selection[m]);
      
      card64 numPoints = pc->getNumPoints();
      float32 pointProb = (float32)expectedNumSamples/(float32)numPoints;

      if (pc && pc->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
         SceneObjectIterator *it = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
			AAT POS = pc->getAAT("position");

         if (it) {
            BasicPointCloudIterator* basicIt = dynamic_cast<BasicPointCloudIterator*>(it);

            if (basicIt) {
               while (!basicIt->atEnd()) {
               
						Vector3f pos = basicIt->get3f(POS);
						float32 pProb = pointProb;
						while (pProb > 1.0f) {
							createPoint(pos, resultPS);
							pProb -= 1.0f;
						}
						if (rnd01() <= pProb) {
							createPoint(pos, resultPS);
						}	
                  basicIt->next();
               }
            }
            delete it;
         }
         
			BoundingBox3f bb = getPCBBox(pc);
			for (card32 i=0; i<numOutliers; i++) {
				Vector3f pos = bb.lowerCorner - (bb.upperCorner - bb.lowerCorner) * 0.5;
				Vector3f off = (bb.upperCorner - bb.lowerCorner) * 2.0f;
				pos[0] += rnd01() * off[0];
				pos[1] += rnd01() * off[1];
				pos[2] += rnd01() * off[2];
				resultPS->addPoint((const char*)&pos);
			}
		}
	}
   
   addPointCloud(scene, result, targetName);
}
