#ifndef PCCNormalToolsH
#define PCCNormalToolsH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	offers some tools for normal orientation etc. 
*/
class X3EDITOR2_API PCCNormalTools : public PCCommand
{
	X4_CLASS( PCCNormalTools )

private:

	float32	m_Sigma;	// used for gaussian filter kernel (sigma = m_Sigma * averagePointDistance)
	mpcard	m_NumNeighbors;	// for knn methods

public:
	PCCNormalTools(void);
	~PCCNormalTools(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
	virtual void execute(Scene *scene);


	/// estimates normals using a pca plane from the topological direct neighbors (uses sigma for gaussian weights in pca)
	/// normals are orientated as before
	void computeMeshNormalsFromTopNeighborsByPCA();

	/// estimates normals using a pca plane (uses sigma for gaussian weights in pca)
	/// normals are orientated to the current scene camera
	void calculateRangeImageNormals();

	/// normals are orientated to the current scene camera
	/// estimates normals using a pca plane
	/// sigma isn't scaled with averagePointDistance!
	void calculateRangeImageNormals( PointCloud * pc, float sigma, Vector3f cameraPosition );

	/// recalculate normals (after registration process)
	void recalculateBadCalculatedNormals();

	void flipNormalsToCamera();
	void flipSelectedNormals();
	void flipSelectedNormalsToCamera();

	/// smooth normals (gaussian)
	void smoothNormals();
	void smoothNormals(PointCloud * pc, float sigma);

	/// compute pca normal and orient by voting
	static void recomputeAndOrientNormalsKNN(PointCloud * pc, mpcard numNeighbors);
	void recomputeAndOrientNormalsKNN();

};

}

#endif