//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCNormalizeAnimation.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "VertexDescriptor.h"
#include "SGListNode.h"
#include "SceneObjectIterator.h"
#include "PointCloudIterators.h"
#include "SGObjectNode.h"
#include "ProgressWindow.h"
#include "PointCloudTools.h"
#include <ClassMethods.h>

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( PCCNormalizeAnimation , 0 ) {
	BEGIN_CLASS_INIT( PCCNormalizeAnimation );
   ADD_STRING_PROP(target, 0)

   ADD_NOARGS_METHOD(unifySizeAndPosition);
}

// ----

PCCNormalizeAnimation::PCCNormalizeAnimation() {
   target = "root/data";
}

void PCCNormalizeAnimation::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

PCCNormalizeAnimation::~PCCNormalizeAnimation() {
}



void PCCNormalizeAnimation::unifySizeAndPosition(SGListNode* targetNode)
{
	if (!targetNode) targetNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(scene, target));

	if (!targetNode) 
	{
		warning("PCCNormalizeAnimation::execute() - target node not found (or not a list node).");
		return;
	}

	mpcard n = targetNode->getNumChildNodes(NULL);
	for (mpcard i=0; i<n; i++)
	{
		SGObjectNode *objNode = dynamic_cast<SGObjectNode*>(targetNode->getChildNode(NULL, i));
		if (!objNode) continue;

		SceneObject *obj = objNode->getSceneObject();
		if (!obj) continue;

		SceneObjectIterator *it = obj->createIterator(SOIT::CAP_HIERARCHICAL);
        BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
		HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(it);
		if (!hit) { delete it; continue; }
		BoundingBox3f bb = hit->getBoundingBox();

		float32 s = 1.0f;
		if (bb.getMaxSideLength() > 0)
			s = 2.0f / bb.getMaxSideLength();

		Matrix4f transformation = makeScale4f(s,s,s) * makeTranslation4f(-bb.getCenter());
		debugOutput << "transformation: " << transformation.transpose() << "\n";

		ModifyablePointCloudIterator *mit = dynamic_cast<ModifyablePointCloudIterator*>(it);
		if (!(bit && mit)) { delete it; continue; }

		PointSet *ps = mit->lockBlockReadWrite();
		if (ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32))
		{
			AAT posAAT = ps->getAAT("position");
			card32 nPts = ps->getNumEntries();
			for (card32 j=0; j<nPts; j++)
				ps->set3f(j, posAAT, transformVector3f(transformation, ps->get3f(j, posAAT)));
		}
		bit->unlock();
		delete it;
	}
}

void PCCNormalizeAnimation::execute(Scene *scene) 
{
   SGListNode *targetNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(scene, target));
   if (!targetNode) {
      warning("PCCNormalizeAnimation::execute() - target node not found (or not a list node).");
      return;
   }
   BoundingBox3f bb;
   bool found = false;
   mpcard n = targetNode->getNumChildNodes(NULL);
	progressWindow->pushStep();
   for (mpcard i=0; i<n; i++) {
      SGObjectNode *objNode = dynamic_cast<SGObjectNode*>(targetNode->getChildNode(NULL, i));
      if (objNode) {
         SceneObject *obj = objNode->getSceneObject();
         if (obj) {
            SceneObjectIterator *it = obj->createIterator(SOIT::CAP_HIERARCHICAL);
            HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(it);
            if (!hit) {
               delete it; continue;
            }
            if (found) {
               bb.addBB(hit->getBoundingBox());
            } else {
               bb = hit->getBoundingBox();
               found = true;
            }
            delete it;
         }
      }
		progressWindow->progress((card32)(50.0*(double)i/(double)n));
   }
   if (found) {
      float32 s = 1.0f;
      if (bb.getMaxSideLength() > 0) {
         s = 2.0f / bb.getMaxSideLength();
      }
      Matrix4f transformation = makeScale4f(s,s,s)*makeTranslation4f(-bb.getCenter());
		debugOutput << "transformation: " << transformation.transpose() << "\n";
      
      for (mpcard i=0; i<n; i++) {
         SGObjectNode *objNode = dynamic_cast<SGObjectNode*>(targetNode->getChildNode(NULL, i));
         if (objNode) {
            SceneObject *obj = objNode->getSceneObject();
            if (obj) {
               SceneObjectIterator *it = obj->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC);
               BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
               ModifyablePointCloudIterator *mit = dynamic_cast<ModifyablePointCloudIterator*>(it);
               if (!(bit && mit)) {
                  delete it; continue;
               }
               PointSet *ps = mit->lockBlockReadWrite();
               if (ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
                  AAT posAAT = ps->getAAT("position");
                  card32 nPts = ps->getNumEntries();
                  for (card32 j=0; j<nPts; j++) {
                     ps->set3f(j, posAAT, transformVector3f(transformation, ps->get3f(j, posAAT)));
                  }
               }
               bit->unlock();
               delete it;
            }
         }
			progressWindow->progress((card32)(50.0*(double)i/(double)n)+50);
      }
   }
	progressWindow->popSteps();
}
