//---------------------------------------------------------------------------
#ifndef PCCParametricDerivatorH
#define PCCParametricDerivatorH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace X4
{

class Scene;
class PointCloud;

class PCCParametricDerivator : public PCCommand
{
private:
   static MetaClass *PCCParametricDerivatorMetaClass;
   static Object* newInstance();


   float32  m_MaxAbsoluteQuadSize;
   float32  m_MaxRelativeQuadSize;  // relative to neighbourquads
   float32  m_ScaleTangents;
   float32  m_MaxQuadRatio;
   float32  m_MaxQuadDiag;
   float32  m_MinTangentAngle;

   card32   m_InvalidateKernelSize;

   bool     m_EstimateNormals;
   bool     m_EstimateTangents;
   bool     m_NormalizeNormals;
   bool     m_NormalizeTangents;

   bool     m_InvalidateOutliers;
   bool     m_CheckAbsolute;
   bool     m_CheckDiag;


protected:

   std::string getAttName_Position() {return "position";}
   std::string getAttName_Normal() {return "normal";}
   std::string getAttName_TangentU() {return "tangent_u";}
   std::string getAttName_TangentV() {return "tangent_v";}

   bool checkNormalTangentsChannels(Scene *scene, card32 pointCloudIndex, PointCloud* &pc);
   void estimate( Scene *scene, card32 pointCloudIndex );

public:
   struct TQuad
   {
      float size;
      Vector3f normal;
      Vector3f tangent_u;
      Vector3f tangent_v;
      bool  valid;
   };

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   PCCParametricDerivator();
   virtual ~PCCParametricDerivator();

   virtual void execute(Scene *scene);

};


}
#endif
 