//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCCParametricPointCloudGenerator.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "PointCloudIterators.h"
#include "NumericalClassProperty.h"
#include "StringClassProperty.h"
#include "BooleanClassProperty.h"
#include "ParametricInCorePointCloud.h"
#include "LinearAlgebra.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "LinearAlgebra.inline.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

// ################################ PERSISTENT REG ###############################################
// ################################ PERSISTENT REG ###############################################
// ################################ PERSISTENT REG ###############################################

MetaClass* PCCParametricPointCloudGenerator::PCCParametricPointCloudGeneratorMetaClass = NULL;

const MetaClass* PCCParametricPointCloudGenerator::getInstanceClass() const {
   return getClass();
}

Object* PCCParametricPointCloudGenerator::newInstance() {
   return new PCCParametricPointCloudGenerator;
}

const MetaClass* PCCParametricPointCloudGenerator::getClass() {
   if (PCCParametricPointCloudGeneratorMetaClass==NULL) {
      PCCParametricPointCloudGeneratorMetaClass = new MetaClass("PCCParametricPointCloudGenerator", "no description.",
                                  newInstance, sizeof(PCCParametricPointCloudGenerator));
   }
   return PCCParametricPointCloudGeneratorMetaClass;
}


void PCCParametricPointCloudGenerator::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass , MetaClass::FIRST_VERSION + 0);
   PCCParametricPointCloudGeneratorMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(PCCParametricPointCloudGenerator)
	ADD_CARD32_PROP(m_Width, 0);
	ADD_CARD32_PROP(m_Height, 0);
	ADD_FLOAT32_PROP(m_SizeX, 0);
	ADD_FLOAT32_PROP(m_SizeY, 0);
	ADD_FLOAT32_PROP(m_NoiseZ, 0);
	ADD_STRING_PROP(m_PCName, 0)
	ADD_BOOLEAN_PROP(m_GenerateFlags, 0)
	ADD_BOOLEAN_PROP(m_RandomColors, 0)
}

void PCCParametricPointCloudGenerator::shutdown() {
   delete PCCParametricPointCloudGeneratorMetaClass;
}

// -------------------------------- PERSISTENT REG -----------------------------------------------
// -------------------------------- PERSISTENT REG -----------------------------------------------
// -------------------------------- PERSISTENT REG -----------------------------------------------

PCCParametricPointCloudGenerator::PCCParametricPointCloudGenerator()
{
   m_Width = 32;
   m_Height = 32;
   m_SizeX = 1.0f;
   m_SizeY = 1.0f;
   m_PCName = "unnamed";
   m_GenerateFlags = false;
   m_NoiseZ = 0.0f;
   m_RandomColors = false;
}

void PCCParametricPointCloudGenerator::execute( Scene * scene )
{
   x4Assert( m_Width > 0 );
   x4Assert( m_Height > 0 );

   // ----------------------------------
   // create vertexdescriptor
   VertexDescriptor vdesc;

   vdesc.pushAttrib( VAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
   if( m_GenerateFlags )
      vdesc.pushAttrib( VAD( "flags", 1, VAD::DATA_FORMAT_INT32 ) );
   vdesc.pushAttrib( VAD( "color", 3, VAD::DATA_FORMAT_FLOAT32 ) );

   AAT token_pos = vdesc.getToken( "position" );
   AAT token_col = vdesc.getToken( "color" );


   // ----------------------------------
   // create cloud
   ParametricInCorePointCloud * pc  = new ParametricInCorePointCloud;
   pc->clearAndSetup( m_Width, m_Height, &vdesc );


   // ----------------------------------
   // calc & set vertices
   PointCloudIterator * iter = pc->createIterator( SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_PARAMETRIC_PC );
   ParametricPointCloudIterator * paramIt = dynamic_cast<ParametricPointCloudIterator*>(iter);
   ModifyablePointCloudIterator * modIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
   x4Assert( paramIt != NULL );
   x4Assert( modIt != NULL );
   for( card32 x=0;x<m_Width;x++ )
      for( card32 y=0;y<m_Height; y++ )
      {
         paramIt->goTo(x,y);
         Vector3f pos = makeVector3f(m_SizeX * ((float)x / (float)m_Width-0.5f),
            (float)rand() / (float)RAND_MAX * m_NoiseZ,
            m_SizeY * ((float)y / (float)m_Height-0.5f));
         modIt->set3f( token_pos, pos );
         if( m_RandomColors )
         {
            Vector3f col = makeVector3f( (float)rand() / (float)RAND_MAX,
               (float)rand() / (float)RAND_MAX,(float)rand() / (float)RAND_MAX );
            modIt->set3f( token_col, col );
         }
      }

   SGLegacyPCSceneAdaptor sceneLA(scene);
   sceneLA.addPointCloud(pc);

}

