//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCCParametricPointSpriteCreator.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
#include "BooleanClassProperty.h"

#include "PointCloudTools.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "PointSet.h"

#include "PointSet.inline.h"

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* PCCParametricPointSpriteCreator::PCCParametricPointSpriteCreatorMetaClass = NULL;

const MetaClass* PCCParametricPointSpriteCreator::getInstanceClass() const {
   return getClass();
}

Object* PCCParametricPointSpriteCreator::newInstance() {
   return new PCCParametricPointSpriteCreator;
}

const MetaClass* PCCParametricPointSpriteCreator::getClass() {
   if (PCCParametricPointSpriteCreatorMetaClass==NULL) {
      PCCParametricPointSpriteCreatorMetaClass = new MetaClass("PCCParametricPointSpriteCreator", "no description.",
                                  newInstance, sizeof(PCCParametricPointSpriteCreator));
   }
   return PCCParametricPointSpriteCreatorMetaClass;
}

void PCCParametricPointSpriteCreator::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   PCCParametricPointSpriteCreatorMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT( PCCParametricPointSpriteCreator )
	ADD_STRING_PROP(source,0)
	ADD_STRING_PROP(dest,0)
	ADD_BOOLEAN_PROP(randomPermutation,0)
}

void PCCParametricPointSpriteCreator::shutdown() {
   delete PCCParametricPointSpriteCreatorMetaClass;
}


// ----


PCCParametricPointSpriteCreator::PCCParametricPointSpriteCreator() {
   randomPermutation = true;
}

void PCCParametricPointSpriteCreator::execute(Scene *scene) {
   SceneGraphNode *sourceNode = getSceneGraphNodeByName(scene, source);
   if (!sourceNode) throw X4Exception("PCCParametricPointSpriteCreator::execute() - source node not found.");
   SceneGraphNode *destNode = getSceneGraphNodeByName(scene, dest);
   if (!destNode) throw X4Exception("PCCParametricPointSpriteCreator::execute() - dest node not found.");

   PointCloud *sourcePC = getPointCloud(sourceNode);
   if (!sourcePC) throw X4Exception("PCCParametricPointSpriteCreator::execute() - source node does not contain a point cloud.");
   PointCloud *destPC = getPointCloud(destNode);
   if (!destPC) throw X4Exception("PCCParametricPointSpriteCreator::execute() - dest node does not contain a point cloud.");

   if (!sourcePC->getDescr()->equal(destPC->getDescr())) throw X4Exception("PCCParametricPointSpriteCreator::execute() - source and dest vertex descriptor (currently) must be equal.");

   SceneObjectIterator *sourceIt = sourcePC->createIterator(SOIT::CAP_BASIC_PC);
   SceneObjectIterator *destIt = destPC->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_PARAMETRIC_PC | SOIT::CAP_DYNAMIC_PARAMETRIC_PC);

   BasicPointCloudIterator *sourceBasicIt = dynamic_cast<BasicPointCloudIterator*>(sourceIt);
   if (!sourceBasicIt)
      throw X4Exception("PCCParametricPointSpriteCreator::execute() - source does not support all caps.");

   ParametricPointCloudIterator *destParamIt = dynamic_cast<ParametricPointCloudIterator*>(destIt);
   DynamicParametricPointCloudIterator *destDynParamIt = dynamic_cast<DynamicParametricPointCloudIterator*>(destIt);
   ModifyablePointCloudIterator *destModIt = dynamic_cast<ModifyablePointCloudIterator*>(destIt);

   if (!(destParamIt && destDynParamIt && destModIt))
      throw X4Exception("PCCParametricPointSpriteCreator::execute() - dest does not support all caps.");

   card64 np64 = sourcePC->getNumPoints();

   if (np64 > 1E8) throw X4Exception("PCCParametricPointSpriteCreator::execute() - source is too large.");

   card32 numPoints = (card32)np64;

   if (numPoints == 0) return;

   card32 sideLength = floor(sqrt((float32)numPoints));

   PointSet *buffer = new PointSet();
   buffer->clearAndSetup(1, 0, sourcePC->getDescr());

   char *pointbuffer = new char[sourcePC->getDescr()->getSize()];

   while (!sourceBasicIt->atEnd()) {
      sourceBasicIt->getVertex(pointbuffer);
      buffer->addPoint(pointbuffer);
      sourceBasicIt->next();
   }

   delete pointbuffer;

   if (randomPermutation) randomlyPermutePointSet(buffer);

   destDynParamIt->setSize(sideLength, sideLength);

   AAT POS = sourcePC->getAAT("position");

   for (card32 y=0; y<sideLength; y++) {
      for (card32 x=0; x<sideLength; x++) {
         destParamIt->goTo(x,y);
         destModIt->setVertex(buffer->getDataPointer(x + y*sideLength));
         Vector3f pos = buffer->get<float32, 3>(x + y*sideLength, POS);
         x4Assert(true);
      }
   }

   delete sourceIt;
   delete destIt;
   delete buffer;
}

