//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCPlayCameraPath.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "NumericalClassProperty.h"
#include "ObjectClassProperty.h"
#include "FileDialogs.h"
#include "CameraController.h"
#include "ClassMethods.h"
#include "BinaryObjectStreams.h"
#include "InterfaceGlobals.h"
#include "LCHierarchyRenderer.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

double X4::GLOBAL_VAR_PCCPlayCameraPath_avPtsInPath;
double X4::GLOBAL_VAR_PCCPlayCameraPath_avNodesInPath;


IMPLEMENT_X4_CLASS( PCCPlayCameraPath , 0 ) {
	BEGIN_CLASS_INIT( PCCPlayCameraPath );
   ADD_OBJECT_PROP(camCtrl, 0, CameraController::getClass(), true)
   ADD_FLOAT32_PROP(startTime, 0)
   ADD_FLOAT32_PROP(endTime, 0)
   ADD_CARD32_PROP(numFrames, 0)
   ADD_BOOLEAN_PROP(outputTiming, 0)

	ADD_NOARGS_METHOD(loadCameraController);
	ADD_NOARGS_METHOD(saveCameraController);
}

// ----

PCCPlayCameraPath::PCCPlayCameraPath() {
   camCtrl = NULL;
   startTime = 0;
   endTime = 50;
   numFrames = 51;
   outputTiming = true;
}

void PCCPlayCameraPath::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

PCCPlayCameraPath::~PCCPlayCameraPath() {
}



void PCCPlayCameraPath::loadCameraController() 
{
	std::string filename = FileDialogs::getOpenFileName(
	   InterfaceGlobals::getMainWindow(), "Load camera controller...", 
      "X4 CameraController Objects (*.camCtrl.object);;X4 Objects (*.object);;All Files (*.*)"
   );
   if (filename != "" && FileExists(filename.c_str())) {
	   BinaryInputObjectStream in(filename.c_str());
      Persistent *instance = NULL;
      in.readObject(instance);
      CameraController *c = dynamic_cast<CameraController*>(instance);
      if (c != NULL) {
         delete camCtrl;
         camCtrl = c;
      } else {
         delete c;
         throw X4Exception("PCCPlayCameraPath::loadCameraController() - object is not a camera controller.");
      }
   }   
}

void PCCPlayCameraPath::saveCameraController() 
{
	std::string filename = FileDialogs::getSaveFileName(
	   InterfaceGlobals::getMainWindow(), "Save camera controller...", 
      "X4 CameraController Objects (*.camCtrl.object);;X4 Objects (*.object);;All Files (*.*)"
   );

   if (filename != "") {
	   BinaryOutputObjectStream out(filename.c_str());
      out.writeObject(camCtrl);
   }   
}

void PCCPlayCameraPath::execute(Scene *scene) 
{
   if (camCtrl) {
      float32 savetime = scene->getRootState()->dynamicState->time;
      Camera saveCam = scene->getRootState()->staticState->camera;
      camCtrl->bindCamera(&(scene->getRootState()->staticState->camera));
		card32 numBoxes = 0, numPoints = 0;
      for (card32 frame=0; frame < numFrames; frame++) {
         float32 t = (float32)frame / (float32)(numFrames-1);
         t *= (endTime-startTime);
         t += startTime;
         camCtrl->setTime(t);
         camCtrl->apply();
         scene->getRootState()->dynamicState->time = t;
         redrawScene();
         numBoxes += GLOBAL_VAR_LCHierarchyRenderer_numNodesRendered;
         numPoints += GLOBAL_VAR_LCHierarchyRenderer_numPointsRendered;
      }
		GLOBAL_VAR_PCCPlayCameraPath_avPtsInPath = (double)numBoxes / (double)numFrames;
	   GLOBAL_VAR_PCCPlayCameraPath_avNodesInPath = (double)numPoints / (double)numFrames;
      scene->getRootState()->dynamicState->time = savetime;
      scene->getRootState()->staticState->camera = saveCam;
   } else {
      throw X4Exception("PCCPlayCameraPath::execute() - no camera controller specified (load one first).");
   } 
}
