//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCPointsToSplats.h"
#include "SeparatorClassProperty.h"
#include "VertexDescriptor.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "PointCloudIterators.h"
#include "LinearAlgebra.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS(PCCPointsToSplats, 0)
{
	BEGIN_CLASS_INIT(PCCPointsToSplats);
	
// 	ADD_SEPARATOR("input/output nodes")
// 	ADD_STRING_PROP(sourcePC, 0)
 	ADD_STRING_PROP(outputPostfix, 0)

	ADD_SEPARATOR("parameters")
	ADD_CARD32_PROP(numVertices, 0);
	ADD_FLOAT32_PROP(scaleFactor, 0);
}

PCCPointsToSplats::PCCPointsToSplats()
{
	outputPostfix = "_splats";
	scaleFactor = 2.0f;
	numVertices = 6;
}

PCCPointsToSplats::~PCCPointsToSplats()
{
}

void PCCPointsToSplats::assign(const Object* obj, X4CopyContext *context)
{
	PCCommand::assign(obj,context);
	const PCCPointsToSplats * o = dynamic_cast<const PCCPointsToSplats*>(obj);
	x4Assert( o != NULL );

}
void PCCPointsToSplats::execute(Scene *scene) 
{
	if (numVertices < 2) {
		warning("numVertices (number boundary vertices) is smaller than 2; set to 2.");
		numVertices = 2;
	}
	std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
	if (selection.size() == 0) warning("nothing selected");

	for (size_t sel=0; sel < selection.size(); sel++ ) {
		string sourcePC = selection[sel];


		PointCloud *sPC = getPointCloud(scene, sourcePC);
		if (!sPC) {
			warning("input point cloud not found.");
			return;
		}
		const VertexDescriptor *vds_orig = sPC->getDescr();

		VertexDescriptor *vds = (VertexDescriptor*)vds_orig->copy();
		vds->pushAttrib(mVAD("radiusParam", 1, VAD::DATA_FORMAT_FLOAT32));
		AAT radiusParamAAT = vds->getToken("radiusParam");

		AAT positionAAT = vds->getToken("position", 3, VAD::DATA_FORMAT_FLOAT32);
		if (positionAAT == NULL_AAT) {
			warning("position channel missing");
			return;
		}
		AAT normalAAT = vds->getToken("normal", 3, VAD::DATA_FORMAT_FLOAT32);
		if (normalAAT == NULL_AAT) {
			warning("normal channel missing");
			return;
		}
		AAT sampleSpacingAAT = vds->getToken("sampleSpacing", 1, VAD::DATA_FORMAT_FLOAT32);
		if (sampleSpacingAAT == NULL_AAT) {
			warning("sampleSpacing channel missing");
			return;
		}

		VertexDescriptor vdt;
		vdt.pushAttrib(VAD("index", 3, VAD::DATA_FORMAT_INT32));
		AAT indexAAT = vdt.getToken("index");

		UnstructuredInCoreTriangleMesh *outputMesh = new UnstructuredInCoreTriangleMesh();
		outputMesh->clearAndSetup( vds, (mpcard)((numVertices+1)*sPC->getNumPoints()), 
										  &vdt, (mpcard)((numVertices)*sPC->getNumPoints()));

		PointCloudIterator *it = (PointCloudIterator*)sPC->createIterator(SOIT::CAP_BASIC_PC);
		BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
		vector<char> buffer;
		buffer.resize(vds->getSize()+42);
		PointSet *meshVertices = outputMesh->getPointSet();
		PointSet *meshFaces = outputMesh->getTriangles();
		mpcard index = 0;
		if (bit) while (!bit->atEnd()) {
			bit->getVertex(&(buffer[0]));
			for (mpcard i=0; i<=(mpcard)numVertices; i++) {
				mpcard vIndex = (numVertices+1)*index+i;
				meshVertices->setVertex(vIndex, &(buffer[0]));
				meshVertices->set1f(vIndex, radiusParamAAT, ((i==0)?0.0f:1.0f));
			}

			Vector3f normal = normalize(bit->get3f(normalAAT));
			Vector3f position = bit->get3f(positionAAT);
			float32 sampleSpacing = bit->get1f(sampleSpacingAAT);
			Vector3f tangent_u = normalize(makeOrthonormalVector(normal));
			Vector3f tangent_v = normal.crossProduct(tangent_u);
			for (mpcard i=1; i<=(mpcard)numVertices; i++) {
				float32 angle = (float32)(i-1) / (float32)(numVertices) * M_PI * 2.0f;
				Vector3f v = position 
							  + tangent_u * (cos(angle)*sampleSpacing*scaleFactor)
							  + tangent_v * (sin(angle)*sampleSpacing*scaleFactor);
				meshVertices->set3f((numVertices+1)*index+i, positionAAT, v);
			}
			for (mpcard i=0; i<(mpcard)(numVertices-1); i++) {
				Vector3i tri = makeVector3i((numVertices+1)*index, (numVertices+1)*index+i+1, (numVertices+1)*index+i+2);
				meshFaces->set3i(numVertices*index+i, indexAAT, tri);
			}
			Vector3i tri = makeVector3i((numVertices+1)*index, (numVertices+1)*index+numVertices, (numVertices+1)*index+1);
			meshFaces->set3i(numVertices*index+(numVertices-1), indexAAT, tri);

			bit->next();
			index++;
		}
		delete it;
		delete vds;
		addPointCloud(scene, outputMesh, sourcePC+outputPostfix);
	}
}

