//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PCCRegisterICP.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "StringHelper.h"
#include "PCCApplySOTransformation.h"
#include "ClassMethods.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS(PCCRegisterICP, 0)
{
	BEGIN_CLASS_INIT(PCCRegisterICP)
   ADD_OBJECT_PROP(icp, 0, FSIcp::getClass(), true);
   ADD_BOOLEAN_PROP(allSelected, 0)
   ADD_STRING_PROP(obj1Path, 0)
   ADD_STRING_PROP(obj2Path, 0)

   ADD_NOARGS_METHOD(registerRochus);
}

//---------------------------------------------------------------------------

PCCRegisterICP::PCCRegisterICP() {
   icp = new FSIcp();
   icp->setupParameters();//(64, 2, 0.1f, 3, 0.20f, 1e20f , -1.0f, 0.001f, true);
   allSelected = true;
   obj1Path = "root/pc1";
   obj2Path = "root/pc2";
}

PCCRegisterICP::~PCCRegisterICP() {
   delete icp;
}

void PCCRegisterICP::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj, this);
}

//---------------------------------------------------------------------------

void PCCRegisterICP::execute(Scene *scene) 
{
   std::vector<std::string> selection;
   if (allSelected) {
      selection = scene->getRootState()->staticState->getSelectedNodes();
   } else {
      selection.resize(2);
      selection[0] = obj1Path;
      selection[1] = obj2Path;
   }

   for (mpcard i = 0; i < selection.size(); i++)
   {
	   debugOutput << selection[i] << "\n";
   }

   PointCloud *firstShape = getPointCloud( scene, selection[0] );
   PCCApplySOTransformation::applyTransformation(firstShape);
   if (!firstShape) {
      throw ECommandFailed("PCCApplySOTransformation::execute() - first object is not a point cloud.");
   }
   SceneObjectIterator *it = firstShape->createIterator(SOIT::CAP_BASIC_PC);
   if (!it) {
      throw ECommandFailed("PCCApplySOTransformation::execute() - cannot create point set for first shape.");
   }
   BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
   const PointSet *firstShapePS = bit->lockBlockRead();
    
	for ( unsigned m = 1; m < selection.size(); m++ ){
		PointCloud *pcOther = getPointCloud( scene, selection[m] );
      PCCApplySOTransformation::applyTransformation(pcOther);
		if (!pcOther) {
		   warning(std::string("PCCApplySOTransformation::execute() - object number ")+intToStr(m+1)+std::string(" is not a point cloud."));
		   continue;
		}

      icp->setupData(const_cast<PointSet*>(firstShapePS), scene);
      
      SceneObjectIterator *itOther = pcOther->createIterator(SOIT::CAP_BASIC_PC);
      if (!itOther) {
         warning(std::string("PCCApplySOTransformation::execute() - object number ")+intToStr(m+1)+std::string(" is not willing to give us a point set."));
      }
      BasicPointCloudIterator *bitOther = dynamic_cast<BasicPointCloudIterator*>(itOther);
      const PointSet *otherShapePS = bitOther->lockBlockRead();

      Matrix4f delta;
      float32 residual = icp->calculateResidualAfterICP(const_cast<PointSet*>(otherShapePS), delta);
      debugOutput << "residual: " << residual << "\n";

      bitOther->unlock();

		Matrix4f transf = pcOther->getTransformation();
		pcOther->setTransformation(delta);
      PCCApplySOTransformation::applyTransformation(pcOther);
      delete itOther;
   }   
   bit->unlock();
   delete it;
}

void X4::PCCRegisterICP::registerRochus()
{
	PointCloud *firstShape = getPointCloud( scene, "root/kap/part_0" );
	PCCApplySOTransformation::applyTransformation(firstShape);
	if (!firstShape) {
		throw ECommandFailed("PCCApplySOTransformation::execute() - first object is not a point cloud.");
	}
	SceneObjectIterator *it = firstShape->createIterator(SOIT::CAP_BASIC_PC);
	if (!it) {
		throw ECommandFailed("PCCApplySOTransformation::execute() - cannot create point set for first shape.");
	}
	BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
	const PointSet *firstShapePS = bit->lockBlockRead();

	for ( unsigned m = 1; m < 12; m++ ){
		PointCloud *pcOther = getPointCloud( scene, "root/kap/part_" + intToStr(m) );
		PCCApplySOTransformation::applyTransformation(pcOther);
		if (!pcOther) {
			warning(std::string("PCCApplySOTransformation::execute() - object number ")+intToStr(m+1)+std::string(" is not a point cloud."));
			continue;
		}

		icp->setupData(const_cast<PointSet*>(firstShapePS), scene);

		SceneObjectIterator *itOther = pcOther->createIterator(SOIT::CAP_BASIC_PC);
		if (!itOther) {
			warning(std::string("PCCApplySOTransformation::execute() - object number ")+intToStr(m+1)+std::string(" is not willing to give us a point set."));
		}
		BasicPointCloudIterator *bitOther = dynamic_cast<BasicPointCloudIterator*>(itOther);
		const PointSet *otherShapePS = bitOther->lockBlockRead();

		Matrix4f delta;
		float32 residual = icp->calculateResidualAfterICP(const_cast<PointSet*>(otherShapePS), delta);
		debugOutput << "residual: " << residual << "\n";

		bitOther->unlock();

		Matrix4f transf = pcOther->getTransformation();
		pcOther->setTransformation(delta);
		PCCApplySOTransformation::applyTransformation(pcOther);
		delete itOther;
	}   
	bit->unlock();
	delete it;
}