//---------------------------------------------------------------------------
#ifndef PCCRemoveOutliersH
#define PCCRemoveOutliersH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
#include "Scene.h"
#include "RenderingTool.h"
#include <TensorVoting.h>
//---------------------------------------------------------------------------

namespace X4 {

#define OUTLIER_REMOVER_TENSOR_VOTING 0

#define OUTLIER_ACTION_COMPUTE 0
#define OUTLIER_ACTION_COLORIZE 1
#define OUTLIER_ACTION_DELETE 2

class X3EDITOR2_API PCCRemoveOutliers : public PCCommand 
{
	X4_CLASS( PCCRemoveOutliers )

	private:
		RenderingTool* rt;			
		TensorVoting* tv;
		std::string pcName;				// Name of point cloud
		card32 technique;			// Removal technique: currently only tensor voting
		float32 tvSigma;			// Influence radius of tensor voting
		float32 tvThreshold;		// Threshold used in outlier classification
		card32 action;				// Action applied to points: compute, colorize, delete
		bool showTVSigma;			// Visualization: show influence radius
		float32 renderScale;		// Visualization: scaling of render objects
		card32 showPointIndex;		// Visualization: Index of point used to visualize influence radius
		bool selectionMode;			// runs outlier removal on all selected point clouds

		// DEPRECATED
		bool batch;
		bool ignoreOutliers;

		// outlier removal
		void processPointCloud( std::string currentPCName, Scene * scene);

	public:
		/// Constructor
		PCCRemoveOutliers();
		/// Destructor
		~PCCRemoveOutliers();

		void glDrawTool( GLContext *glContext );
		void execute( Scene *scene );

		void setAction(card32 newAction){action = newAction;};
		float32 getSigma(){return tvSigma;};
		float32 getThreshold(){return tvThreshold;};

		// function used for processing animations
		void execute_ar( Scene *scene, const std::vector<std::string> selection );
};


}

#endif
 