#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCTestKNNPerformance.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "HierarchicalKNNIterator.h"
#include "SceneGraphState.h"
#include "PointCloud.h"
#include "PointCloudTools.h"
#include "PointCloudIterators.h"
#include "HierarchicalKNNIterator.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "ProgressWindow.h"
#include "ClassMethods.h"
#include "PointSetKNNQuery.h"
#include <time.h>
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCCTestKNNPerformance, 0 )
{
	BEGIN_CLASS_INIT( PCCTestKNNPerformance );
	ADD_BOOLEAN_PROP( range, 0 );
	ADD_INT32_PROP( numDataPoints, 0 );
	ADD_FLOAT32_PROP( epsilon, 0 );
	ADD_INT32_PROP( numKNN, 0 );
}

PCCTestKNNPerformance::PCCTestKNNPerformance(void)
{
	numDataPoints = 100000;
	numRuns = 1000;
	numKNN = 100;
	epsilon = 0.01f;
	range = true;
}

PCCTestKNNPerformance::~PCCTestKNNPerformance(void)
{
}

void PCCTestKNNPerformance::assign(const Object* obj, X4CopyContext *context)
{
	PCCommand::assign(obj,context);
	const PCCTestKNNPerformance * o = dynamic_cast<const PCCTestKNNPerformance*>(obj);
	x4Assert( o != NULL );

}
void PCCTestKNNPerformance::execute(Scene *scene) 
{
	PointSet* ps = new PointSet();
	VertexDescriptor vd;
	vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	ps->clearAndSetup( 1, numDataPoints, &vd ); 
	AAT POSITION = ps->getAAT( "position" );
	for ( int n = 0; n < numDataPoints; n++ ){
		ps->set3f( n, POSITION, makeVector3f( rand()%10000/10000.0f, rand()%10000/10000.0f, rand()%10000/10000.0f ) );
	}

	float32 startTime = clock();

	AttachedIndexedOctree* octree = PointSetKNNQuery::createOctree( ps );
	PointSetKNNQuery knn( ps, octree, makeVector3f( 0,0,0 ), POSITION );
	bool found;
	card32 nIndex;

	float32 buildTreeTime = float32( ( clock() - startTime ) / CLK_TCK );
	float sqrEpsilon = epsilon * epsilon;
	float avgNumRangeKNN = 0;
	startTime = clock();

	for ( int pIndex = 0; pIndex < numDataPoints; pIndex++ ){ 
		Vector3f p = ps->get3f( pIndex, POSITION );
		knn.setTargetPoint( p );
		if ( !range ){
			for ( int n = 0; n < numKNN; n++ ){
				knn.getNextNearestPointIndex( nIndex, found );
			}
		} else {
			Vector3f np = p;
			float sqrD = ( p - np ) * ( p - np );
			int rangeKNN = 0;
			while( sqrD < sqrEpsilon ){
				rangeKNN++;
				knn.getNextNearestPointIndex( nIndex, found );
				np = ps->get3f( nIndex, POSITION );
				sqrD = ( p - np ) * ( p - np );
			}
			avgNumRangeKNN += float( rangeKNN ) / float( numDataPoints );
		}
	}

	float searchTime = float( ( clock() - startTime ) / CLK_TCK );
	debugOutput << "KNN test" << "\n";
	debugOutput << " numDataPoints: " << numDataPoints << "\n";
	if ( !range ){
		debugOutput << " numKNN: " << numKNN << "\n";
	} else {
		debugOutput << " epsilon: " << epsilon << "\n";
		debugOutput << " avgNumRangeKNN: " << avgNumRangeKNN << "\n";
	}
	debugOutput << " build tree time: " << buildTreeTime << " sec." << "\n";
	debugOutput << " search time: " << searchTime << " sec." << "\n";
	debugOutput.flush();

	delete ps;
	delete octree;
}
