#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCTwist.h"
//---------------------------------------------------------------------------
#include "DebugOutput.h"
#include "UnstructuredInCorePointCloud.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;


std::string descrAxis(card32 i) {
	switch (i) {
		case PCCTwist::AXIS_X: return "AXIS_X";
		case PCCTwist::AXIS_Y: return "AXIS_Y";
		case PCCTwist::AXIS_Z: return "AXIS_Z";
		default: return "unknown";
	}
}

IMPLEMENT_X4_CLASS( PCCTwist ,0)
{
	BEGIN_CLASS_INIT( PCCTwist );
	ADD_CARD32_PROP_UPDATE_DESCR(axis, 0, NULL, descrAxis, 3 , NULL);
	ADD_FLOAT32_PROP(angle_per_length, 0);
}

PCCTwist::PCCTwist(void)
{
	angle_per_length = 1.0f;
	axis                   = AXIS_X;
}

PCCTwist::~PCCTwist(void)
{
}

void PCCTwist::assign(const Object* obj, X4CopyContext *context)
{
	PCCommand::assign(obj,context);
	const PCCTwist * o = dynamic_cast<const PCCTwist*>(obj);
	x4Assert( o != NULL );

}

void PCCTwist::execute(Scene *scene)
{
//	float32 x = axis[0];
//	debugOutput << "test: " << x;
	unsigned int m;
	std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();

	if(fabs(angle_per_length) < 1e-6)
		return;

	for(m = 0; m < selection.size(); m++)
	{
		PointCloud	*pc = getPointCloud(scene, selection[m]);

		SceneObjectIterator	*it = pc->createIterator(SOIT::CAP_BASIC_PC|SOIT::CAP_MODIFYABLE_PC|SOIT::CAP_HIERARCHICAL);
		BasicPointCloudIterator	*basicIt = dynamic_cast<BasicPointCloudIterator*>(it);
		ModifyablePointCloudIterator	*mIt = dynamic_cast<ModifyablePointCloudIterator*>(it);
		HierarchicalIterator				*hIt  = dynamic_cast<HierarchicalIterator*>(it);

		AAT		POS = pc->getAAT("position");
		
		Vector3f	 translation = hIt->getBoundingBox().getCenter();

		Matrix3f	 rotation;

		basicIt->reset();
		while(!basicIt->atEnd())
		{
			Vector3f pos = basicIt->get3f(POS);
			pos       -= translation;

			switch(axis)
			{
			case AXIS_X:
				rotation = makeRotX3f(pos[0]/angle_per_length);
				pos       = rotation*pos;
				break;
			case AXIS_Y:
				rotation = makeRotX3f(pos[1]/angle_per_length);
				pos       = rotation*pos;
				break;
			case AXIS_Z:
				rotation = makeRotX3f(pos[2]/angle_per_length);
				pos       = rotation*pos;
				break;
			default: break;
			}      
			pos += translation;
			mIt->set3f(POS, pos);
			basicIt->next();
		}
		
		delete it;
	}
}