//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCCUnifyNormals.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
#include "DebugOutput.h"
#include "PointCloud.h"
#include "PointCloudTools.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "LinearAlgebra.inline.h"
#include <queue> 
#include "NormalUnifier.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( PCCUnifyNormals, 2 )
{
	BEGIN_CLASS_INIT( PCCUnifyNormals )
	ADD_CARD32_PROP( m_numNearestPoints, 0 )
	ADD_BOOLEAN_PROP( recolorUnifiedNormals, 1 )
	ADD_BOOLEAN_PROP( useImprovedAlgorithm, 2 );
}

PCCUnifyNormals::PCCUnifyNormals() 
{
	m_numNearestPoints = 8;
	recolorUnifiedNormals = false;
	useImprovedAlgorithm = false;
}

PCCUnifyNormals::~PCCUnifyNormals() 
{
}

void PCCUnifyNormals::execute(Scene *scene)
{
	unsigned int m;
	std::vector<std::string> selection = scene->getRootState()->staticState->getSelectedNodes();
	PointCloud* pc;
	/// Use all point clouds in selection
	for ( m = 0; m < selection.size(); m++ ){
		/// Check for required attributes
		checkAttribute( scene, selection[m], "position", 3, VAD::DATA_FORMAT_FLOAT32 );
		checkAttribute( scene, selection[m], "normal", 3, VAD::DATA_FORMAT_FLOAT32 );
		checkAttribute( scene, selection[m], "flags", 1, VAD::DATA_FORMAT_INT32 );
		/// Estimate normals
		pc = getPointCloud( scene, selection[m] );
		if ( useImprovedAlgorithm ){
			UnstructuredInCorePointCloud* upc = dynamic_cast<UnstructuredInCorePointCloud*>( pc );
			if ( pc != NULL )
				NormalUnifier::unifyNormalsHoppe( upc->getPointSet(), m_numNearestPoints );
		} else { 
			NormalUnifier::unifyNormals( pc, m_numNearestPoints );
		}
   }
}

