#ifndef PCI2DPointPainterH
#define PCI2DPointPainterH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "CommandObject.h"
//---------------------------------------------------------------------------

namespace X4 {


class X3EDITOR2_API PointPainterCmdObj : public CommandObject {
   X4_CLASS(PointPainterCmdObj)
 private:
	std::string targetNode;
	bool createIfNotFound;
	bool storeSampleSpacing;
	bool storeNoiseCovariance;
	std::vector<Vector3f> points;
	std::vector<float32> sampleSpacings;
	std::vector<Matrix3f> noiseCovariances;
	
	IMPLEMENT_VAR_ARRAY_METHODS_STL(points)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(sampleSpacings)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(noiseCovariances)
	
 public:
	PointPainterCmdObj();

	void setup(std::string targetNode, bool createIfNotFound, bool storeSampleSpacing, bool storeNoiseCovariance, Vector3f point, float32 sampleSpacing, Matrix3f noiseCovariance);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();
   virtual std::string getDescription();
   virtual bool mergeCommand(const CommandObject *nextCmd);
 
};


/**
	Tool that allows painting 2d point cloud curves
*/
class X3EDITOR2_API PCI2DPointPainter : public PCInteractionTool
{
	X4_CLASS( PCI2DPointPainter )

 private:
	// --- persistent state
	std::string targetNode;
	bool createIfNotFound;
	bool storeSampleSpacing;
	bool storeNoiseCovariance;
	float32 zDist;
	float32 randomSigma;

	// --- volatile state
	bool leftDown;
	card32 width;
	card32 height;
	
	Vector3f lastPoint;
	bool lastPointSet, firstPoint;
	
	void createPoint(int32 x, int32 y);

 public:
	PCI2DPointPainter();

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

	~PCI2DPointPainter();
};

}

#endif