//---------------------------------------------------------------------------
#ifndef PCIImageEditingH
#define PCIImageEditingH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "LinearAlgebra.h"
#include "VertexDescriptor.h"
#include "PCImageTools.h"
//---------------------------------------------------------------------------


namespace X4 {

class PCImageTool;
class PCIImageEditingEditorWidget;


class PCIImageEditing : public PCInteractionTool {
private:
	static MetaClass *PCIImageEditingMetaClass;
	static Object* newInstance();

	// target
	std::string targetNode;
	std::string channel;
	PointSet *targetPointSet;
	AAT channelAAT;

	// image display
	float32 zoom;
	Vector2f pan;
	float32 scale;
	float32 offset;
	bool flipY;

	// environment
	card32 width;
	card32 height;

	// editing tool
	PCImageTool *currentTool;

	Vector2f unproject(float32 x, float32 y);

public:
	PCIImageEditing();

	virtual const MetaClass *getInstanceClass() const;
	static const MetaClass *getClass();
	static void init(const MetaClass *parentClass);
	static void shutdown();

	virtual void glDrawTool(GLContext *glContext);
	virtual bool drawGLSceneRendering() {return false;}
	virtual void activateTool();

	virtual void keyDown(GeneralKey key);
	virtual void keyUp(GeneralKey key);

	virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
	virtual void mouseMoved(int32 x, int32 y);
	virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
	virtual void areaResize(card32 width, card32 height);

	void setTargetNode(std::string targetNode);
	void setChannel(std::string channel);

	virtual void redrawScene() {PCInteractionTool::redrawScene();}

	virtual void setTool(PCImageTool *tool);

	~PCIImageEditing();
	friend PCIImageEditingEditorWidget;
};


}


#endif
 