//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCIRegisterPC.h"
//---------------------------------------------------------------------------
#include "CameraController.h"
#include "ExaminerCameraController.h"
#include "ObjectClassProperty.h"
#include "Scene.h"
#include "ClassMethods.h"
#include "SceneGraphState.h"
#include "DebugOutput.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "SelectionIteratorAdaptor.h"     
#include "NumericalClassProperty.h"
#include "Random.h"
#include "SceneCommandObjects.h"
#include "CommandHistory.h"
#include "BooleanClassProperty.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "SceneGraphNode.h"
#include "LinearAlgebra.inline.h"
#include "BoundingBox.inline.h"
#include "PointCloudTools.h"
#include "opencv/cxcore.h"

//---------------------------------------------------------------------------
//#include "cv_include.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



Matrix4f makeRotX(Radian angle) {
   return expand3To4(makeMatrix3f (
              1.0,        0.0,        0.0,

              0.0, cos(angle), -sin(angle),

              0.0,sin(angle), cos(angle)
   ));
}


Matrix4f makeRotY(Radian angle) {
   return expand3To4(makeMatrix3f (
       cos(angle),        0.0, sin(angle),

              0.0,        1.0,        0.0,

      -sin(angle),        0.0, cos(angle)
   ));
}


Matrix4f makeRotZ(Radian angle) {
   return expand3To4(makeMatrix3f (
       cos(angle), -sin(angle),        0.0,

       sin(angle), cos(angle),        0.0,

              0.0,        0.0,        1.0
   ));
}

IMPLEMENT_X4_CLASS(PCIRegisterPC, 0)
{
   BEGIN_CLASS_INIT( PCIRegisterPC )
   
   
   ADD_CARD32_PROP(numIterations,0);
   ADD_CARD32_PROP(nthPoint,0);
   ADD_CARD32_PROP(gridRes,0);
   ADD_FLOAT64_PROP(step,0);
   ADD_FLOAT64_PROP(angle,0);
   ADD_FLOAT64_PROP(q_dist,0);
   ADD_FLOAT32_PROP(eigenAspect,0);
   ADD_BOOLEAN_PROP(ICP,0);
   ADD_OBJECT_PROP( dock, 0, PointCloud::getClass(), true );
   ADD_OBJECT_PROP( dockee, 0, PointCloud::getClass(), true );
   ADD_STRING_PROP(dockName,0);
   ADD_STRING_PROP(dockeeName,0);

   ADD_NOARGS_METHOD_DESC(selectClouds, "selects Clouds based on dockname/dockeename");
   ADD_NOARGS_METHOD_DESC(newton, "do newton optimization with given parameters");
   ADD_NOARGS_METHOD_DESC(descent, "do gradient descent with given parameters");
}


//----------------------------------------------------------------------------
PCIRegisterPC::PCIRegisterPC() {
   bDisplay=false;
   itnum=0;
   scale=1;
   step=1;
   gStep=1;
   numIterations = 1;
   gridRes=1;
   oldPos[0]=0;
   oldPos[1]=0;
   oldPos[2]=0;
   oldPos[3]=0;
   oldPos[4]=0;
   oldPos[5]=0;
   dGrad[0]=0;
   dGrad[1]=0;
   dGrad[2]=0;
   dGrad[3]=0;
   dGrad[4]=0;
   dGrad[5]=0;
   bnewton = g_init = fix_x = fix_y = fix_z = fix_alpha = fix_beta = fix_gamma = false;
   depth_test = true;
   angle = 1.8;
   q_dist =100;
   tex_init = false;
   gauss_alpha=0;
   dock = NULL;
   dockee = NULL;
   eigenAspect=0.0001f;
   r_eigenAsp=0.1f;
   nthPoint=100;
   r_nRP=nthPoint;
   used_cells=1;
   bOptimization=true;
   dockName="invalid";
   dockeeName="invalid";
   ICP=false;
}


void PCIRegisterPC::selectClouds()
{
	
	if (selDockByName(dockName)== "invalid") dock=NULL;
	if (selDockeeByName(dockeeName)=="invalid") dockee=NULL;
	
	


}


std::string PCIRegisterPC::selDockeeByName(std::string name){
   SGLegacyPCSceneAdaptor sceneLA(scene);
   const card32 n = sceneLA.getNumPointClouds();
   for (card32 i=0; i<n; i++) {
      if ((sceneLA.getNode(i)->getName() == name)||(sceneLA.getNameWithPath(i)== name)){
         dockee= sceneLA.getPointCloud(i);
         name=sceneLA.getNameWithPath(i);
         //initRandPoints(nthPoint);
         return name;

         }

   }
   return "invalid";

}

std::string PCIRegisterPC::selDockByName(std::string name){
   SGLegacyPCSceneAdaptor sceneLA(scene);
   const card32 n = sceneLA.getNumPointClouds();
   for (card32 i=0; i<n; i++) {
      if ((sceneLA.getNode(i)->getName() == name)||(sceneLA.getNameWithPath(i)== name)){
         dock=sceneLA.getPointCloud(i);
         name=sceneLA.getNameWithPath(i);
         newDock=true;
         return name;
         }

   }

   return "invalid";

   
}

void PCIRegisterPC::newton()
{
	bOptimization=true;
	bnewton=true;
	execute(false);
}
void PCIRegisterPC::descent()
{
	bOptimization=false;
	bnewton=false;
	execute(false);

}
void PCIRegisterPC::execute(bool adapt)
{
   

   Matrix4f undo=dockee->getTransformation();

   if (bOptimization)
      doOptimization(dock,dockee,3,2,2);
   else if (!bnewton)
      doDescent(dock,dockee,numIterations,adapt);
      else
         doNewton(dock,dockee,numIterations,this->step);
   CmdObjApplyRegisterTransform *newCmd = new CmdObjApplyRegisterTransform();

   // -- quick hack for compatibility with new scene graph framework:
   std::string dockeename;
   SGLegacyPCSceneAdaptor sceneLA(scene);
   for (card32 i=0; i<sceneLA.getNumPointClouds(); i++) {
      if (sceneLA.getPointCloud(i) == dockee) dockeename = sceneLA.getNameWithPath(i);
   }
   // --

   newCmd->setup(dockee->getTransformation(), undo, dockeename, numIterations, gridRes);
   scene->getCommandHistory()->executeCommand(scene, newCmd, true);

}

void PCIRegisterPC::applyMatrix(){
   dockee->setTransformation(result);
   redrawScene();


}


std::string PCIRegisterPC::getFirstSel(){
   SGLegacyPCSceneAdaptor sceneLA(scene);
   std::vector<card32> sel = sceneLA.getSelectedPCs();
   if (sel.size()==0)
      return "None selected";
   else
      return sceneLA.getNameWithPath(sel[0]);
}

void PCIRegisterPC::initGrid (PointCloud* dock, int gRes) {
   //init------------------------
   //bool bSelected=true;


   if (gRes==0) return;

   int gSize=gRes*gRes*gRes;
   cGrid* g=new cGrid;
   g->res=gRes;
   Matrix3f empty;
   empty[0]=makeVector3f(0,0,0);
   empty[1]=makeVector3f(0,0,0);
   empty[2]=makeVector3f(0,0,0);
   for (int i=0; i<8; i++)
   {
      if (gRes==1 && i>0) continue;
      g->grid[i].mean=new Vector3f[gSize];
      g->grid[i].sigma=new Matrix3f[gSize];
      g->grid[i].n=new int[gSize];
      g->grid[i].eigenV=new Matrix3f[gSize];
      g->grid[i].eigenW=new Vector3f[gSize];
      g->grid[i].score=new double[gSize];
      g->grid[i].cnt=new int[gSize];
      for (int j=0; j<gSize;j++)
      {
         g->grid[i].mean[j]=makeVector3f(0,0,0);
         g->grid[i].eigenV[j]=empty;
         g->grid[i].eigenW[j]=makeVector3f(0,0,0);
         g->grid[i].sigma[j]=empty;
         g->grid[i].score[j]=0;
         g->grid[i].cnt[j]=0;
         g->grid[i].n[j]=0;
      }
   }
   //end init------------------------
   if (!dock->providesAttribute("position")) return;
   AAT POSITION = dock->getAAT("position");




   SceneObjectIterator *newIt = dock->createIterator(SOIT(SOIT::CAP_BASIC_PC));
   if (newIt == NULL) return;
   BasicPointCloudIterator *readIt = dynamic_cast<BasicPointCloudIterator*>(newIt);



   //get Dimensions, calculate grid parametrization------------------
   Matrix4f curtransmatr = dock->getTransformation();
   debugOutput <<curtransmatr;

   float maxX,maxY,maxZ,minX,minY,minZ;
   maxX=-99999999999.0f;maxY=-99999999999.0f;maxZ=-99999999999.0f;
   minX=99999999999.0f;minY=99999999999.0f;minZ=99999999999.0f;
   AAT FLAGS = dock->getAAT("flags");

   /*if (bSelected && dock->providesAttribute("flags"))
      do {
                  readIt->next();
            } while (!readIt->atEnd() && (!((readIt->get1i(FLAGS)) & FLAG_SELECTED)))  ;   */
   while (!readIt->atEnd())
   {

      Vector4f hPos = expand3To4(readIt->get3f(POSITION));
      hPos = curtransmatr*hPos;
      float32 w = hPos[3];
      if (fabs(w) > 0.01f) {
         Vector3f pos = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);
         if (pos[0]>maxX) maxX=pos[0];
         if (pos[0]<minX) minX=pos[0];
         if (pos[1]>maxY) maxY=pos[1];
         if (pos[1]<minY) minY=pos[1];
         if (pos[2]>maxZ) maxZ=pos[2];
         if (pos[2]<minZ) minZ=pos[2];
      }
      readIt->next();
      /*if (!bSelected || !dock->providesAttribute("flags"))
         readIt->next();
      else do {
                  readIt->next();
            } while (!readIt->atEnd() && (!((readIt->get1i(FLAGS)) & FLAG_SELECTED)))  ;  */
   }

   readIt->reset();
  /* if (bSelected && dock->providesAttribute("flags"))
      do {
                  readIt->next();
            } while (!readIt->atEnd() && (!((readIt->get1i(FLAGS)) & FLAG_SELECTED)))  ;    */



   Vector3f gOffset=makeVector3f(minX,minY,minZ);
   Vector3f gRange=makeVector3f(maxX-minX,maxY-minY,maxZ-minZ);
   Vector3f gMargin=gRange*0.1f;
   gRange+=gMargin;
   gOffset-=gMargin*0.5;
   g->range=gRange;
   //! Range==0 noch irgendwie abfangen, falls Ebene, Linie oder einzelner Punkt!
   Vector3f gSteps=gRange/gRes;
   g->steps=gSteps;
   Vector3f off=gSteps*0.5;
   g->grid[0].offset=gOffset; //0
   g->grid[1].offset=gOffset+makeVector3f(off[0],0,0); //X
   g->grid[2].offset=gOffset+makeVector3f(0,off[1],0); //Y
   g->grid[3].offset=gOffset+makeVector3f(0,0,off[2]); //Z
   g->grid[4].offset=gOffset+makeVector3f(off[0],off[1],0); //XY
   g->grid[5].offset=gOffset+makeVector3f(off[0],0,off[2]); //XZ
   g->grid[6].offset=gOffset+makeVector3f(0,off[1],off[2]); //YZ
   g->grid[7].offset=gOffset+makeVector3f(off[0],off[1],off[2]); //XYZ

   //end get Dimensions, calculate grid parametrization------------------
   //calculate mean
   while (!readIt->atEnd()) {
      Vector4f hPos = expand3To4(readIt->get3f(POSITION));
      hPos = curtransmatr*hPos;
      float32 w = hPos[3];
      if (fabs(w) > 0.01f) {
         Vector3f pos = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);
         for (int i=0; i<8; i++)
         {
            if (gRes==1 && i>0) continue;
            Vector3f c=(pos-(g->grid[i].offset));
            c[0]/=g->steps[0];
            c[1]/=g->steps[1];
            c[2]/=g->steps[2];
            if (  (c[0]<0)
                  ||(c[0]>=g->res)
                  ||(c[1]<0)
                  ||(c[1]>=g->res)
                  ||(c[2]<0)
                  ||(c[2]>=g->res) )
               continue;

            int cell= int(c[0])+int(c[1])*g->res+int(c[2])*g->res*g->res;
            g->grid[i].mean[cell]+=pos;
            g->grid[i].n[cell]++;
         }
      }
      readIt->next();
   /*   if (!bSelected || !dock->providesAttribute("flags"))
         readIt->next();
      else do {
                  readIt->next();
            } while (!readIt->atEnd() && (!((readIt->get1i(FLAGS)) & FLAG_SELECTED)))  ;        */
   }

   for (int i=0;i<8; i++)
   {
      if (gRes==1 && i>0) continue;
      for (int j=0; j<gSize;j++)
      {
         if (g->grid[i].n[j]!=0)
         {
            g->grid[i].mean[j]=g->grid[i].mean[j]/g->grid[i].n[j];
         }
      }
   }
   //end calculate mean
   //calculate sigma



   readIt->reset();
  /* if (bSelected && dock->providesAttribute("flags"))
      do {
                  readIt->next();
            } while (!readIt->atEnd() && (!((readIt->get1i(FLAGS)) & FLAG_SELECTED)))  ;  */
   int count=0;
   while (!readIt->atEnd())  {
      Vector4f hPos = expand3To4(readIt->get3f(POSITION));
      hPos = curtransmatr*hPos;
      float32 w = hPos[3];
      if (fabs(w) > 0.01f) {
         Vector3f pos = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);

         for (int i=0; i<8; i++)
         {
            if (gRes==1 && i>0) continue;
            Vector3f c=(pos-(g->grid[i].offset));
            c[0]/=g->steps[0];
            c[1]/=g->steps[1];
            c[2]/=g->steps[2];
            if (  (c[0]<0)
                  ||(c[0]>=g->res)
                  ||(c[1]<0)
                  ||(c[1]>=g->res)
                  ||(c[2]<0)
                  ||(c[2]>=g->res) )
               continue;

            int cell= int(c[0])+int(c[1])*g->res+int(c[2])*g->res*g->res;
            float covXX=0;float covXY=0;float covXZ=0;
            float covYY=0;float covYZ=0;float covZZ=0;
            Vector3f mean=g->grid[i].mean[cell];
            covXX=(pos[0]-mean[0])*(pos[0]-mean[0]);
            covXY=(pos[0]-mean[0])*(pos[1]-mean[1]);
            covXZ=(pos[0]-mean[0])*(pos[2]-mean[2]);
            covYY=(pos[1]-mean[1])*(pos[1]-mean[1]);
            covYZ=(pos[1]-mean[1])*(pos[2]-mean[2]);
            covZZ=(pos[2]-mean[2])*(pos[2]-mean[2]);
            g->grid[i].sigma[cell][0]+=makeVector3f(covXX,covXY,covXZ);
            g->grid[i].sigma[cell][1]+=makeVector3f(covXY,covYY,covYZ);
            g->grid[i].sigma[cell][2]+=makeVector3f(covXZ,covYZ,covZZ);
         }



      }
      count++;

      readIt->next();
     /* if (!bSelected || !dock->providesAttribute("flags"))
         readIt->next();
      else do {
                  readIt->next();
            } while (!readIt->atEnd() && (!((readIt->get1i(FLAGS)) & FLAG_SELECTED)))  ; */
   }

   readIt->reset();
   used_cells=0;
   for (int i=0;i<8; i++)
   {
      if (gRes==1 && i>0) continue;
      for (int j=0; j<gSize;j++)
      {
         if (g->grid[i].n[j]>5)
         {
            used_cells++;
            g->grid[i].sigma[j]=g->grid[i].sigma[j]/g->grid[i].n[j];

            double cov[] = { g->grid[i].sigma[j][0][0], g->grid[i].sigma[j][0][1], g->grid[i].sigma[j][0][2],
                  g->grid[i].sigma[j][1][0], g->grid[i].sigma[j][1][1], g->grid[i].sigma[j][1][2],
                  g->grid[i].sigma[j][2][0], g->grid[i].sigma[j][2][1], g->grid[i].sigma[j][2][2]
                  };

            double v[3];
            double m[9];
            CvMat covM,eigenvalueM,eigenvectorM;


            cvInitMatHeader( &covM, 3, 3, CV_64FC1, cov );
            cvInitMatHeader( &eigenvalueM, 3, 1, CV_64FC1,v);
            cvInitMatHeader( &eigenvectorM, 3, 3, CV_64FC1,m);


            cvSVD(&covM,&eigenvalueM,&eigenvectorM);

            Matrix3f eigenVecM,inv;
            Vector3f eigenValV;
            for (int y=0;y<3;y++)
            {  eigenValV[y]=cvGetReal1D( &eigenvalueM, y);
               for (int x=0;x<3;x++)
               {
                 eigenVecM[y][x]=cvGetReal2D( &eigenvectorM, y, x);

               }
            }


            if (eigenValV[0]<0.00001)
               eigenValV[0]=0.00001f;
            if (eigenValV[1]<0.0001f*eigenValV[0])
               eigenValV[1]=0.0001f*eigenValV[0];
            if (eigenValV[2]<eigenAspect*eigenValV[0])
               eigenValV[2]=eigenAspect*eigenValV[0];

            g->grid[i].eigenV[j]=eigenVecM;
            g->grid[i].eigenW[j]=eigenValV;

            Matrix3f inv_singular;
            inv_singular[0][0]=1/eigenValV[0];
            inv_singular[1][1]=1/eigenValV[1];
            inv_singular[2][2]=1/eigenValV[2];


            //Matrix3f lina_inv=invertMatrix(g->grid[i].sigma[j]);


            g->grid[i].sigma[j]=eigenVecM.transpose()*inv_singular*eigenVecM;
            /*if (i==0)
            {
               Matrix3f diff=lina_inv-(g->grid[i].sigma[j]);
               debugOutput<< "Abweichung: " <<expand3To4(diff);
               debugOutput.cr();
            }          */


            //g->grid[i].sigma[j]=invertMatrix(g->grid[i].sigma[j]);
         }

      }
   }

   //Gitter fertig!
   //delete newIt;  adress violation!
  this->grid=g;
  g_init=true;
  r_gridRes=gridRes;
  r_eigenAsp=eigenAspect;
  //this->grid->mean=grid->mean;
  //this->grid->sigma=grid->sigma;
  //this->grid->n=grid->n;
  //delete grid;
}

void PCIRegisterPC::initRandPoints(int nthPoint)
{
   randPoint=getNPoints(dockee,nthPoint);
}

int* PCIRegisterPC::getNPoints(PointCloud* pc, int n){
   SceneObjectIterator *newIt = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
   if (newIt == NULL) return 0;
   BasicPointCloudIterator *readIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
   int i=0;
   while (!readIt->atEnd())  {
         i++;
         readIt->next();

      }
   int nrPoints=i;
   int* result=new int[n];
   for   (int j=0;j<n;j++)
   {
      result[j]=randomNumber(nrPoints+1);
   }

   bool sorted=false;
   int m=n;
   while( !sorted && m>0)
   {
      sorted=true;
      for(i=0;i<m-1;i++)
      {
         if(result[i]>result[i+1])
         {
            int tmp=result[i];
            result[i]=result[i+1];
            result[i+1]=tmp;
            if(i+2<m) sorted=false;
         }
      }
      m--;
   }
   for (i=0;i<n;i++)
      debugOutput <<result[i]<<" ";

   return result;


}

double PCIRegisterPC::calcScore(PointCloud* dockee,Matrix4f changed){

if (ICP)
{
	if (!dockee->providesAttribute("position")) return 0;
   AAT POSITION = dockee->getAAT("position");
   AAT NEAREST= dock->getAAT("position");
   SceneObjectIterator *newIt = dockee->createIterator(SOIT(SOIT::CAP_BASIC_PC));
   if (newIt == NULL) return 0;
   BasicPointCloudIterator *readIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
   double score=0;
   HierarchicalKNNIterator* pcNNIt = new HierarchicalKNNIterator(dock,32,NULL);

   int num=0;
   int j=0;

   while (!readIt->atEnd())  {
   if (j%nthPoint!=0)
   {
      j++;
      readIt->next();
      continue;
   }
   j++;

    Vector4f hPos = expand3To4(readIt->get3f(POSITION));
    hPos = changed*hPos;
    float32 w = hPos[3];
    if (fabs(w) > 0.01f) {
      Vector3f pos = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);
   pcNNIt->setSeekPointAndReset(pos);
   Vector3f nearest=pcNNIt->get3f(NEAREST);
   pos=nearest-pos;
   //if (pos.getSqrNorm()>0.00001)
   score+=pos.getSqrNorm();
   num++;
   readIt->next();
	}
   





   }
   return score/num;





}


if (!dockee->providesAttribute("position")) return 0;
   AAT POSITION = dockee->getAAT("position");
   SceneObjectIterator *newIt = dockee->createIterator(SOIT(SOIT::CAP_BASIC_PC));
   if (newIt == NULL) return 0;
   BasicPointCloudIterator *readIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
   double score=0;
      //Vector3f gRange=grid->range;
      Vector3f gSteps=grid->steps;
      int gRes=grid->res;
      int gSize=gRes*gRes*gRes;
      //calculate score


     int num=0;
     int j=0;

      while (!readIt->atEnd())  {
      if (j%nthPoint!=0)
         {
            j++;
            readIt->next();
            continue;
         }
         j++;



         Vector4f hPos = expand3To4(readIt->get3f(POSITION));
         hPos = changed*hPos;
         float32 w = hPos[3];
         if (fabs(w) > 0.01f) {
            Vector3f pos = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);
            for (int i=0; i<8; i++)
            {
               
               if (gRes==1 && i>0) continue;
               num++;
               Vector3f c=(pos-(grid->grid[i].offset));
               c[0]/=gSteps[0];
               c[1]/=gSteps[1];
               c[2]/=gSteps[2];
               if (  (c[0]<0)
                  ||(c[0]>=grid->res) || ( (i==1 || i==4 || i==5 || i==7) && c[0]>=grid->res-1)
                  ||(c[1]<0)
                  ||(c[1]>=grid->res) || ( (i==2 || i==4 || i==6 || i==7) && c[1]>=grid->res-1)
                  ||(c[2]<0)
                  ||(c[2]>=grid->res) || ( (i==3 || i==5 || i==6 || i==7) && c[1]>=grid->res-1))
               continue;

               int cell= int(c[0])+int(c[1])*gRes+int(c[2])*gRes*gRes;
               if (cell>=0 && cell<gSize)
               if (grid->grid[i].n[cell]>5){
                  Matrix3f sigInv=grid->grid[i].sigma[cell];
                  Vector3f mean=grid->grid[i].mean[cell];
                  Vector3f p=pos-mean;
                  Vector3f q=sigInv*p;
                  double pointScore=exp(-0.5*(p*q));
                  //grid->grid[i].score[cell]+=pointScore;
                  //grid->grid[i].cnt[cell]++;
                  score+=pointScore;

               }
            }
         }
         readIt->next();
      };
   /*
   int nr=0;
   //double score=0;
   for (int i=0; i<8 ; i++)
      for (int j=0; j<gSize;j++)
      {
         if (grid->grid[i].cnt[j]!=0)
         {
         //grid->grid[i].score[j]=grid->grid[i].score[j]/grid->grid[i].cnt[j];
         //normiert durch die anzahl der punkte
         score+=grid->grid[i].score[j];
         grid->grid[i].score[j]=0;
         grid->grid[i].cnt[j]=0;
         nr++;
         }
      }
    if (nr>1)
      score/=nr; */
    if (used_cells >1)
      score=score/used_cells;
    if (num>1)
      score/=num;

return -score;
};

StaticVector<double,6> PCIRegisterPC::calcGradient(PointCloud* dockee,Vector3f rotCenter,double s){
   //letzte nderung : grad-grad2 gendert
   s=gStep;
   StaticVector<double,6> grad;
   Matrix4f changed;
   double dx=s*(grid->range[0]/q_dist);
   double dy=s*(grid->range[1]/q_dist);
   double dz=s*(grid->range[2]/q_dist);
   double pi=3.1415926535897932384626433832795;
   double da=scale*s*angle*pi/180;
   double db=scale*s*angle*pi/180;
   double dc=scale*s*angle*pi/180;
   ctm=dockee->getTransformation();
   double score,ds;
   Matrix4f rc1=makeTranslation4f(-rotCenter);
   Matrix4f rc2=makeTranslation4f(rotCenter);

   if (!fix_x)
   {
   changed=makeTranslation4f(makeVector3f(dx,0,0))*ctm;
   score=calcScore(dockee,changed);
   ds=score-oldScore;

   grad[0]=ds/dx;
   }
   else grad[0]=0;

   if (!fix_y)
   {
   changed=makeTranslation4f(makeVector3f(0,dy,0))*ctm;
   score=calcScore(dockee,changed);
   ds=score-oldScore;

   grad[1]=ds/dy;
   }
   else grad[1]=0;

   if(!fix_z)
   {
   changed=makeTranslation4f(makeVector3f(0,0,dz))*ctm;
   score=calcScore(dockee,changed);
   ds=score-oldScore;

   grad[2]=ds/dz;
   }
   else grad[2]=0;

   if (!fix_alpha)
   {
      changed=rc2*makeRotX4f(da)*rc1*ctm;
      score=calcScore(dockee,changed);
      ds=score-oldScore;

      grad[3]=ds/da;
   }
   else grad[3]= 0;

   if (!fix_beta)
   {
      changed=rc2*makeRotY4f(db)*rc1*ctm;
      score=calcScore(dockee,changed);
      ds=score-oldScore;

      grad[4]=ds/db;
   }
   else grad[4]= 0;

   if (!fix_gamma)
   {
      changed=rc2*makeRotZ4f(dc)*rc1*ctm;
      score=calcScore(dockee,changed);
      ds=score-oldScore;

      grad[5]=ds/dc;
   }
   else grad[5]=0;
  /* double norm=sqrt(grad[0]*grad[0]+grad[1]*grad[1]+grad[2]*grad[2]+grad[3]*grad[3]+grad[4]*grad[4]+grad[5]*grad[5]);
   if (norm>0.0001)
   {
      grad[0]/=norm;
      grad[1]/=norm;
      grad[2]/=norm;
      grad[3]/=norm;
      grad[4]/=norm;
      grad[5]/=norm;
   }

  /* grad[0] /= dx;
   grad[1] /= dy;
   grad[2] /= dz;
   grad[3] /= da;
   grad[4] /= db;
   grad[5] /= dc;    */



   forget(s);

   return  grad;
};

double PCIRegisterPC::doDescent(PointCloud* dock, PointCloud* dockee, int iterations, bool adapt)
{
   double currentStep=step;
	
   if (gridRes!=r_gridRes || eigenAspect!=r_eigenAsp || newDock)
      initGrid(dock,gridRes);
	
   newDock=false;
   oldScore=calcScore(dockee,dockee->getTransformation());
   double startScore=oldScore;
   double newScore=oldScore;
//xxx_missing:   editor->newScore(newScore,0);

   Vector6d deltaP;
   Matrix4f newCtm;

   int i=1;
   while (i<=iterations)
   {
      /*if (adapt)
      {
         if (currentStep<=1) currentStep=1;

      } */
      Vector3f rc=calcRotCenter(dock,dockee);
      deltaP=-calcGradient(dockee,rc,gStep);
      deltaP*=currentStep;
      newCtm=dockee->getTransformation();
      Matrix4f ctm=newCtm;
      newCtm=makeTranslation4f(-rc)*newCtm;
      if (!fix_gamma)
        newCtm= makeRotZ4f(deltaP[5])*newCtm;
      if (!fix_beta)
        newCtm= makeRotY4f(deltaP[4])*newCtm;
      if (!fix_alpha)
        newCtm= makeRotX4f(deltaP[3])*newCtm;
      newCtm=makeTranslation4f(rc)*newCtm;


      newCtm=makeTranslation4f(deltaP[0],
                               deltaP[1],
                               deltaP[2])*newCtm;
      dockee->setTransformation(newCtm);
      newScore=calcScore(dockee,dockee->getTransformation());
      if (adapt)
      {
         //0.000001 is pure guessing based on one test pair
         //TODO:find better, general criteria
         if ((newScore-oldScore)>0.000001)
         {
            currentStep*=0.5;
            dockee->setTransformation(ctm);
            i++;
            continue;

         }
      }
//xxx_missing:        editor->newScore(newScore,i);
      debugOutput.cr();
      debugOutput<< (newScore);
      oldScore=newScore;
      redrawScene();



      i++;
   }
   result=newCtm;

   double endScore=oldScore;
  forget(startScore);

  return endScore;


}


void PCIRegisterPC::calcDerivatives(PointCloud* dockee,Vector3f rotCenter, Matrix6d& hessian, Vector6d& grad, double s ){

   double gStep=s;
   Matrix6d hess;
   double dx=gStep*(grid->range[0]/q_dist);
   double dy=gStep*(grid->range[1]/q_dist);
   double dz=gStep*(grid->range[2]/q_dist);
   double pi=3.1415926535897932384626433832795;
   double da=scale*gStep*(angle*pi/180);
   double db=scale*gStep*(angle*pi/180);
   double dc=scale*gStep*(angle*pi/180);
   Vector6d grad2;
   Matrix4f trans=dockee->getTransformation();
   Matrix4f changed;
   Matrix4f rc1=makeTranslation4f(-rotCenter);
   Matrix4f rc2=makeTranslation4f(rotCenter);
   grad=calcGradient(dockee,rotCenter,gStep);

   changed=makeTranslation4f(makeVector3f(dx,0,0))*trans;
   dockee->setTransformation(changed);
   grad2=calcGradient(dockee,rotCenter,gStep);
   hess[0]=(grad2-grad)/dx;

   changed=makeTranslation4f(makeVector3f(0,dy,0))*trans;
   dockee->setTransformation(changed);
   grad2=calcGradient(dockee,rotCenter,gStep);
   hess[1]=(grad2-grad)/dy;

   changed=makeTranslation4f(makeVector3f(0,0,dz))*trans;
   dockee->setTransformation(changed);
   grad2=calcGradient(dockee,rotCenter,gStep);
   hess[2]=(grad2-grad)/dz;

   changed=rc2*makeRotX4f(da)*rc1*trans;
   dockee->setTransformation(changed);
   grad2=calcGradient(dockee,rotCenter,gStep);
   hess[3]=(grad2-grad)/da;

   changed=rc2*makeRotY4f(db)*rc1*trans;
   dockee->setTransformation(changed);
   grad2=calcGradient(dockee,rotCenter,gStep);
   hess[4]=(grad2-grad)/db;

   changed=rc2*makeRotZ4f(dc)*rc1*trans;
   dockee->setTransformation(changed);
   grad2=calcGradient(dockee,rotCenter,gStep);
   hess[5]=(grad2-grad)/dc;

   dockee->setTransformation(trans);

   /*hess[0]/=dx;
   hess[1]/=dy;
   hess[2]/=dz;
   hess[3]/=da;
   hess[4]/=db;
   hess[5]/=dc;*/

   hessian=hess;
}

void PCIRegisterPC::calcADerivatives(PointCloud* dockee,Vector3f rotCenter, Matrix6d& hessian, Vector6d& grad, double s )
{
   forget(s);
   Vector3f rc=rotCenter;
   //To Do: all in double precision!
   for (int i=0;i<6;i++)
      for (int j=0;j<6;j++)
      {
         grad[i]=0;
         hessian[j][i]=0;
      }
   Vector6d gr;
   Matrix6d hess;
   StaticMatrix<float,6,3> Jac;
   Jac[0]=makeVector3f(1,0,0);
   Jac[1]=makeVector3f(0,1,0);
   Jac[2]=makeVector3f(0,0,1);




   if (!dockee->providesAttribute("position")) return;
   AAT POSITION = dockee->getAAT("position");
   SceneObjectIterator *newIt = dockee->createIterator(SOIT(SOIT::CAP_BASIC_PC));
   if (newIt == NULL) return;
   BasicPointCloudIterator *readIt = dynamic_cast<BasicPointCloudIterator*>(newIt);

   Vector3f gSteps=grid->steps;
   int gRes=grid->res;
   int gSize=gRes*gRes*gRes;

   if (nthPoint!=r_nRP)
   {
      //initRandPoints(nthPoint);
      r_nRP=nthPoint;
   }



   int num=0;
   while (!readIt->atEnd())
   {
     /* if (j<randPoint[k])
      {
         j++;
         readIt->next();
         continue;
      }
      j++;
      k++; */
         if (num%nthPoint!=0)
         {
            num++;
            readIt->next();
            continue;
         }
         num++;
         Vector4f hPos = expand3To4(readIt->get3f(POSITION));
         hPos = (dockee->getTransformation())*hPos;
         float32 w = hPos[3];
         if (fabs(w) > 0.01f)
         {

            Vector3f pos = makeVector3f(hPos[0]/w, hPos[1]/w,hPos[2]/w);
         for (int i=0; i<8; i++)
         {

            if (gRes==1 && i>0)
               continue;

            Vector3f c=(pos-(grid->grid[i].offset));
               c[0]/=gSteps[0];
               c[1]/=gSteps[1];
               c[2]/=gSteps[2];
               if (  (c[0]<0)
                  ||(c[0]>=grid->res) || ( (i==1 || i==4 || i==5 || i==7) && c[0]>=grid->res-1)
                  ||(c[1]<0)
                  ||(c[1]>=grid->res) || ( (i==2 || i==4 || i==6 || i==7) && c[1]>=grid->res-1)
                  ||(c[2]<0)
                  ||(c[2]>=grid->res) || ( (i==3 || i==5 || i==6 || i==7) && c[1]>=grid->res-1))
               continue;

               int cell= int(c[0])+int(c[1])*gRes+int(c[2])*gRes*gRes;
               if (cell>=0 && cell<gSize)
               if (grid->grid[i].n[cell]>5)
               {

                  Matrix3f sigInv=grid->grid[i].sigma[cell];
                  Vector3f mean=grid->grid[i].mean[cell];
                  Vector3f p=pos-mean;
                  Vector3f q=sigInv*p;
                  double pointScore=exp(-0.5*(p*q));

                  Jac[3]=makeVector3f(0,-p[2]-rc[2],p[1]-rc[1]);
                  Jac[4]=makeVector3f(p[2]-rc[2],0,-p[0]-rc[0]);
                  Jac[5]=makeVector3f(-p[1]-rc[1],p[0]-rc[0],0);


                  /*Jac[3]=makeVector3f(0,p[2],-p[1]);
                  Jac[4]=makeVector3f(p[2],0,-p[0]);
                  Jac[5]=makeVector3f(p[1],-p[0],0); */
                  

                  /*//Bug due to different rotation matrizes
                  Jac[3]=makeVector3f(0,-p[2],p[1]);
                  Jac[4]=makeVector3f(p[2],0,-p[0]);
                  Jac[5]=makeVector3f(-p[1],p[0],0);   */






                  for (int i=0;i<6;i++)
                  {

                     Vector3f tmp_i=sigInv*Jac[i];
                     gr[i]=pointScore* (p*tmp_i);
                     for(int j=0;j<6;j++)
                     {
                        Vector3f tmp_j=sigInv*Jac[j];


                        hess[j][i]= pointScore*
                                    ((p*tmp_i)*(-p*tmp_j)+
                                    Jac[j]*tmp_i);
                     }
                  }
                  hessian+=hess;
                  grad+=gr;

               }
            }
         }

         readIt->next();
      };




}



Vector3f PCIRegisterPC::calcRotCenter(PointCloud* dock, PointCloud* dockee){
   BoundingBox3f bb1,bb2;
   SceneObjectIterator *newIt = dock->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_HIERARCHICAL));
   if (newIt == NULL) return makeVector3f(0,0,0);
   HierarchicalIterator *hierIt = dynamic_cast<HierarchicalIterator*>(newIt);
   bb1=hierIt->getBoundingBox();
   bb1=transformBoundingBox(dock->getTransformation(),bb1);

   newIt = dockee->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_HIERARCHICAL));
   if (newIt == NULL) return makeVector3f(0,0,0);
   hierIt = dynamic_cast<HierarchicalIterator*>(newIt);
   bb2=hierIt->getBoundingBox();
   bb2=transformBoundingBox(dockee->getTransformation(),bb2);
   BVIntersection bvi=bb1.intersectBoundingBox(bb2);
   if (bvi != BVQueryDisjoint ){
      bb1.cutTo(bb2);
      return bb1.getCenter();

   }
   else return makeVector3f(0,0,0);
}

Matrix6d PCIRegisterPC::invertHessian(Matrix6d hessian){

            double hess[] = { hessian[0][0], hessian[1][0], hessian[2][0],hessian[3][0],hessian[4][0],hessian[5][0],
                              hessian[0][1], hessian[1][1], hessian[2][1], hessian[3][1], hessian[4][1], hessian[5][1],
                              hessian[0][2], hessian[1][2], hessian[2][2], hessian[3][2], hessian[4][2], hessian[5][2],
                              hessian[0][3], hessian[1][3], hessian[2][3], hessian[3][3], hessian[4][3], hessian[5][3],
                              hessian[0][4], hessian[1][4], hessian[2][4], hessian[3][4], hessian[4][4], hessian[5][4],
                              hessian[0][5], hessian[1][5], hessian[2][5], hessian[3][5], hessian[4][5], hessian[5][5]
                             };

            double v[6];
            double m[36];
            CvMat hessM,hessPD,eigenvalueM,eigenvectorM,invM;


            cvInitMatHeader( &hessM, 6, 6, CV_64FC1, hess );
            cvInitMatHeader( &eigenvalueM, 6, 1, CV_64FC1,v);
            cvInitMatHeader( &eigenvectorM, 6, 6, CV_64FC1,m);
            cvInitMatHeader( &invM,6,6,CV_64FC1,m);

            //cvSVD(&hessM,&eigenvalueM,&eigenvectorM);
            cvEigenVV(&hessM,&eigenvectorM,&eigenvalueM,0.1);
            double uc=cvGetReal1D(&eigenvalueM,5);
            Matrix6d identity;
            for (int i=0;i<6;i++)
               for (int j=0;j<6;j++)
               {
                  if (i==j)
                     identity[i][j]=1;
                  else
                     identity[i][j]=0;
               }


            if (uc<0)
               hessian-=identity*(uc+0.01);

               debugOutput <<uc;

            double hess_pd[] = { hessian[0][0], hessian[1][0], hessian[2][0],hessian[3][0],hessian[4][0],hessian[5][0],
                              hessian[0][1], hessian[1][1], hessian[2][1], hessian[3][1], hessian[4][1], hessian[5][1],
                              hessian[0][2], hessian[1][2], hessian[2][2], hessian[3][2], hessian[4][2], hessian[5][2],
                              hessian[0][3], hessian[1][3], hessian[2][3], hessian[3][3], hessian[4][3], hessian[5][3],
                              hessian[0][4], hessian[1][4], hessian[2][4], hessian[3][4], hessian[4][4], hessian[5][4],
                              hessian[0][5], hessian[1][5], hessian[2][5], hessian[3][5], hessian[4][5], hessian[5][5]
                             };

            cvInitMatHeader( &hessPD,6,6,CV_64FC1,hess_pd);
            cvInvert(&hessPD,&invM,CV_SVD);
            Matrix6d inv;

            for (int y=0;y<6;y++)
            {
               for (int x=0;x<6;x++)
               {
                 inv[x][y]=cvGetReal2D(&invM,y,x);
               }
            }
            //identity=hessian*inv;
            //debugOutput<< identity;
            hessian=inv;

            return hessian;

}

double PCIRegisterPC::doNewton (PointCloud* dock, PointCloud* dockee,int iterations,double step,bool analytic){
   double currentStep=step;


   if (gridRes!=r_gridRes)
      initGrid(dock,gridRes);

   oldScore=calcScore(dockee,dockee->getTransformation());
   //double startScore=oldScore;
   //editor->newScore(startScore,1);
   Matrix6d hessian;
   Vector6d grad;

   int i=1;
   while (i<=iterations)
   {
      Vector3f rc;
      if (!analytic)
      {
         rc=calcRotCenter(dock,dockee);
         calcDerivatives(dockee,rc,hessian,grad,currentStep);
      }
      else
      {
         rc=makeVector3f(0,0,0);
         //rc=calcRotCenter(dock,dockee);
         calcADerivatives(dockee,rc,hessian,grad,currentStep);
      }
      //debugOutput<<hessian;
      //hessian=hessian.transpose();
      Matrix6d inv_hessian=invertHessian(hessian);
      //inv_hessian=inv_hessian.transpose();
      //debugOutput.cr();
      //debugOutput<<inv_hessian;
      //debugOutput.cr();
      //debugOutput<<grad;
      //inv_hessian=inv_hessian.transpose();
      
      Vector6d deltaP=inv_hessian*(-grad);
      //Vector6d deltaP=-grad;
      //necessary for Alg 6.3.5 from Num.Methods
      grad_pk=grad*deltaP;
      //
      deltaP*=currentStep;
      if (analytic)
      {
         for (int i=3;i<6;i++)
            deltaP[i]*=scale;
      //debugOutput.cr();
      //debugOutput<<deltaP;
      debugOutput.cr();
      debugOutput<<"Trans: "<<makeVector3f(deltaP[0],deltaP[1],deltaP[2]);
      debugOutput.cr();
      debugOutput<<"Rot: "<<makeVector3f(deltaP[3],deltaP[4],deltaP[5]);
      debugOutput.cr();

      Matrix4f newCtm=dockee->getTransformation();
      Matrix4f ctm=newCtm;
      newCtm=makeTranslation4f(-rc)*newCtm;
      Matrix4f rotCtm=expand3To4(shrink4To3(newCtm));

         if (!fix_gamma)
           rotCtm= makeRotZ(deltaP[5])*rotCtm;
         if (!fix_beta)
           rotCtm= makeRotY(deltaP[4])*rotCtm;
         if (!fix_alpha)
           rotCtm= makeRotX(deltaP[3])*rotCtm;

      newCtm=makeTranslation4f(newCtm[3][0],
                               newCtm[3][1],
                               newCtm[3][2])
                  *rotCtm;
      newCtm=makeTranslation4f(rc)*newCtm;

         newCtm=makeTranslation4f(deltaP[0],
                                  deltaP[1],
                                  deltaP[2])*newCtm;
      dockee->setTransformation(newCtm);
      }
      else
      {
            debugOutput.cr();
      debugOutput<<"Trans: "<<makeVector3f(deltaP[0],deltaP[1],deltaP[2]);
      debugOutput.cr();
      debugOutput<<"Rot: "<<makeVector3f(deltaP[3],deltaP[4],deltaP[5]);
      debugOutput.cr();

      Matrix4f newCtm=dockee->getTransformation();
      Matrix4f ctm=newCtm;
      newCtm=makeTranslation4f(-rc)*newCtm;


         if (!fix_gamma)
           newCtm= makeRotZ(deltaP[5])*newCtm;
         if (!fix_beta)
           newCtm= makeRotY(deltaP[4])*newCtm;
         if (!fix_alpha)
           newCtm= makeRotX(deltaP[3])*newCtm;

         newCtm=makeTranslation4f(rc)*newCtm;

         newCtm=makeTranslation4f(deltaP[0],
                                  deltaP[1],
                                  deltaP[2])*newCtm;


      dockee->setTransformation(newCtm);
      }

      double newScore=calcScore(dockee,dockee->getTransformation());
      //editor->newScore(newScore,i);
      debugOutput.cr();
      //debugOutput<< (newScore-oldScore);
      oldScore=newScore;
      //redrawScene();



      i++;
   }
  double endScore=oldScore;

  return endScore;

}

void PCIRegisterPC::doOptimization(PointCloud* dock, PointCloud* dockee,int start_res,int gridStep,int snr,int type){

   bDisplay=false;
   if (start_res<1 || gridStep<0 || snr<0 || dock==NULL || dockee==NULL)
      return;

   //gridRes=start_res;
   initGrid(dock,gridRes);
   //int end_res=start_res+snr*gridStep;

   int end_res=gridRes;
   double dScore=0;
   double relScore=9999999;
   double oldScore=0;
   double prevScore=0;
   double newScore=0;
   double stepTol=0.000001;
   //this->gStep=4.0;
   oldScore=calcScore(dockee,dockee->getTransformation());
   double startScore=oldScore;
//xxx_missing:     editor->newScore(startScore,1);

   while ((int)gridRes<=(int)end_res)
   {
      //this->step=1;

      int num=0;
      while (num<(int)numIterations)
      {


        Matrix4f oldTrans=dockee->getTransformation();

        //Alg 6.3.5. from Numerical Methods...
        //double l=0.1 ;
        //double u=0.5 ;
        bool first;
        double lambda=1;
        double lambdaOld=1.1;
        double lambdatmp=1;
        //double minlambda=0.00001;
        //double alpha=0.0001;
        //Alg 6.3.5. from Numerical Methods...
        int n=0;
        first=true;
        do
        {
            n++;
            if(n==8)
            {
               lambda=1;
               lambdaOld=1.1;
               lambdatmp=1;
               scale=-scale;

            }


            lambdaOld=lambda;
            if (lambdaOld<=1)
            {

               if (lambdatmp>0.5*lambda)
                  lambdatmp=0.5*lambda;
               else if (lambdatmp<0.1*lambda)
                  lambdatmp=0.1*lambda;

               lambda=lambdatmp;

            }
            
            dockee->setTransformation(oldTrans);
            if (first)
            {
               first=false;
               newScore=doNewton(dock,dockee,1);
               double grad_pk=this->grad_pk;
               double teiler=(2*(newScore-oldScore-grad_pk));
               lambdatmp=-grad_pk/teiler;
               prevScore=newScore;

            }
            else
            {
               newScore=doNewton(dock,dockee,1,lambda);
               Matrix2f A;
               A[0]=makeVector2f(1/(lambda*lambda),-lambdaOld/(lambda*lambda));
               A[1]=makeVector2f(-1/(lambdaOld*lambdaOld),lambda/(lambdaOld*lambdaOld));
               Vector2f f=makeVector2f(newScore-oldScore-lambda*grad_pk,prevScore-oldScore-lambdaOld*grad_pk);
               Vector2f a=A*f*1/(lambda-lambdaOld);
               if (a[0]>=0.00000001)
                  lambdatmp=-grad_pk/(2*a[1]);
               else
                  lambdatmp=(-a[1]+sqrt(a[1]*a[1]-3*a[0]*grad_pk))/3*a[0];
               prevScore=newScore;
            }




        }while (newScore>oldScore && n<16) ;
        if (n==16)
        {
            //editor->newScore(oldScore,num+1);
            dockee->setTransformation(oldTrans);
            num=numIterations;
            //newScore=doNewton(dock,dockee,1,step,false);
            //newScore=doDescent(dock,dockee,4,false);
            //oldScore=newScore;

        }
        else{

            oldScore=newScore;
            }

        //----
        /*dScore=newScore-oldScore;
        if (fabs(newScore)>stepTol)
            relScore=dScore/fabs(newScore);
        else
            relScore=dScore/stepTol;
        oldScore=newScore;*/
        num++;

//xxx_missing:          editor->newScore(oldScore,num);

        redrawScene();
      }
      break; //!!!!raus!!!
      //gridRes+=gridStep;
      /*for (int i=0;i<gridStep;i+=5)
         stepTol/=10;
      this->gStep*=(double)start_res/ (double)gridRes;
      if ((int) gridRes==end_res)
         this->eigenAspect=0.02;   */
   }
   result=dockee->getTransformation();

   forget(type);
   forget(newScore);
   forget(stepTol+relScore+dScore);
}

//----------------------------------------------------------------------------
void PCIRegisterPC::areaResize(card32 width, card32 height) {
 forget(width+height);
}

void PCIRegisterPC::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {

   //ToDo: variable pcs attained via selection by click
   debugOutput << x << " / " << y;
   debugOutput.cr();
  /* if (buttonsState.getLeft())
   {
      Matrix4f undo=scene->getPointCloud(1)->getTransformation();
      doDescent(scene->getPointCloud(0),scene->getPointCloud(1),numIterations);
      CmdObjApplyRegisterTransform *newCmd = new CmdObjApplyRegisterTransform();
      newCmd->setup(scene->getPointCloud(1)->getTransformation(),undo,scene->getPointCloud(1),numIterations,gridRes);
      scene->getCommandHistory()->executeCommand(newCmd, true);
   }
   else
   {
      Matrix4f undo=scene->getPointCloud(1)->getTransformation();
      doOptimization(scene->getPointCloud(0),scene->getPointCloud(1),0,1,1,13);
      CmdObjApplyRegisterTransform *newCmd = new CmdObjApplyRegisterTransform();
      newCmd->setup(scene->getPointCloud(1)->getTransformation(),undo,scene->getPointCloud(1),numIterations,gridRes);
      scene->getCommandHistory()->executeCommand(newCmd, true);
      //doNewton(scene->getPointCloud(0),scene->getPointCloud(1),numIterations);
   }            */
   forget(x+y);
   forget(buttonsState);
   forget(modifiersState);
}

void PCIRegisterPC::mouseMoved(int32 x, int32 y) {
   forget(x+y);
}

void PCIRegisterPC::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {
   forget(x+y);
   forget(buttonsState);
   forget(modifiersState);
}


/* obsolete
void PCIRegisterPC::keyPressed(char key) {
   SGLegacyPCSceneAdaptor sceneLA(scene);
   debugOutput<<calcScore(sceneLA.getPointCloud(1),ctm);
   debugOutput.cr();

   //debugOutput<<"steps: "<<grid->range<<" offset: "<<grid->offset<<" n: "<<grid->n;
   debugOutput.cr();
   forget(key);
}
*/

void PCIRegisterPC::keyDown(GeneralKey key) {
   forget(key);
}

void PCIRegisterPC::keyUp(GeneralKey key) {
   forget(key);
}

void PCIRegisterPC::glDrawTool(GLContext *glContext) {
   glPushMatrix();

   glMultMatrixf(scene->getRootState()->dynamicState->ctm.data());
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glDisable(GL_LIGHTING);
   if (depth_test)
   glEnable(GL_DEPTH_TEST);
   glPointSize(3.0f);
   glColor4f(1.0f,0.0f,0.0f,1.0f);

   glBegin(GL_POINTS);
   glVertex3f(0.0f,0.0f,0.0f);
   glEnd();
   glPointSize(1.0f);
   glColor4f(0.4f, 0.0f, 1.0f, 0.4f);


   /*glBegin(GL_LINES);
      glVertex3f(oldPos[0],oldPos[1],oldPos[2]);
      glVertex3f(dGrad[0],dGrad[1],dGrad[2]);
   glEnd();

   glPointSize(4.0f);
   glBegin(GL_POINTS);
      glVertex3f(dGrad[0],dGrad[1],dGrad[2]);
   glEnd();
   glPointSize(1.0f);
                   */


   if (g_init)
   {

      int gSize=r_gridRes*r_gridRes*r_gridRes;
      glBegin(GL_LINES);

      float32 scale=3;
      
      Vector3f mean;
      Vector3f ev;
      for (int i=0; i<gSize ; i++)
      {
         if (grid->grid[0].n[i]>5)
         {
            glVertex3fv(grid->grid[0].mean[i].data());
            ev=makeVector3f(grid->grid[0].eigenV[i][0][0],grid->grid[0].eigenV[i][1][0],grid->grid[0].eigenV[i][2][0]);
            ev=normalize(ev);
            ev=ev*sqrt(grid->grid[0].eigenW[i][0]);
            ev=ev+grid->grid[0].mean[i];
            glVertex3fv(ev.data());
            glVertex3fv(grid->grid[0].mean[i].data());
            ev=makeVector3f(grid->grid[0].eigenV[i][0][1],grid->grid[0].eigenV[i][1][1],grid->grid[0].eigenV[i][2][1]);
            ev=normalize(ev);
            ev=ev*sqrt(grid->grid[0].eigenW[i][1]);
            ev=ev+grid->grid[0].mean[i];
            glVertex3fv(ev.data());
            glVertex3fv(grid->grid[0].mean[i].data());
            ev=makeVector3f(grid->grid[0].eigenV[i][0][2],grid->grid[0].eigenV[i][1][2],grid->grid[0].eigenV[i][2][2]);
            ev=normalize(ev);
            ev=ev*sqrt(grid->grid[0].eigenW[i][2]);
            ev=ev+grid->grid[0].mean[i];
            glVertex3fv(ev.data());
         }
      }
      glEnd();

      if (!tex_init)
      {
         const int gauss_w=128;
         const int gauss_h=128;
         Vector3f m = makeVector3f(0.5, 0.5 ,0.5);
         float gauss_map[gauss_w][gauss_h][4];
         Vector3f p;
         for(int k=0; k<16; k++)
         {
            float pz=((float)(k*8) / (float) gauss_w);
            for (int x=0; x<gauss_w; x++)
               for (int y=0; y<gauss_h; y++)
               {
                  float px=((float)x/(float)gauss_w);
                  float py=((float)y/(float)gauss_h);

                  p=makeVector3f(px,py,pz);
                  p-=m;
                  p=p*scale;
                  float val=exp (- p*p*2);
                  gauss_map[x][y][0]= 1;
                  gauss_map[x][y][1]= 1;
                  gauss_map[x][y][2]= 1;
                  gauss_map[x][y][3]= val;

               }
             //glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
             glGenTextures(1,&t_gauss[k]);
             glBindTexture(GL_TEXTURE_2D,t_gauss[k]);
             glTexImage2D(GL_TEXTURE_2D, 0, 4, gauss_w,
             gauss_h, 0, GL_RGBA, GL_FLOAT,
             &gauss_map[0][0][0]);


             glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
             GL_NEAREST);
             glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
             GL_NEAREST);
             glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);



         }

         tex_init=true;
      
      }

      Vector3f e2,e3;
      glEnable(GL_TEXTURE_2D);
      glEnable(GL_BLEND);
      glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

      glDepthMask(0);

      for (int i=0; i<gSize ; i++)
      {
         if (grid->grid[0].n[i]>5)
         {
            mean=grid->grid[0].mean[i];
            ev=makeVector3f(grid->grid[0].eigenV[i][0][0],grid->grid[0].eigenV[i][1][0],grid->grid[0].eigenV[i][2][0]);
            ev=normalize(ev);
            ev=ev*sqrt(grid->grid[0].eigenW[i][0])*scale;
            e2=makeVector3f(grid->grid[0].eigenV[i][0][1],grid->grid[0].eigenV[i][1][1],grid->grid[0].eigenV[i][2][1]);
            e2=normalize(e2);
            e2=e2*sqrt(grid->grid[0].eigenW[i][1])*scale;
            //e2=e2+grid->grid[0].mean[i];
            e3=makeVector3f(grid->grid[0].eigenV[i][0][2],grid->grid[0].eigenV[i][1][2],grid->grid[0].eigenV[i][2][2]);
            e3=normalize(e3);
            e3=e3*sqrt(grid->grid[0].eigenW[i][2])*scale;
            //e3=e3+grid->grid[0].mean[i];



            for (int s=-8; s<8 ; s++)
            {
                  glBindTexture(GL_TEXTURE_2D, t_gauss[s+8] );
                  glBegin(GL_QUADS);
                  glTexCoord2f(0,0);
                  glVertex3fv((mean+e2+ev+e3*s/8).data());
                  glTexCoord2f(0,1);
                  glVertex3fv((mean-e2+ev+e3*s/8).data());
                  glTexCoord2f(1,1);
                  glVertex3fv((mean-e2-ev+e3*s/8).data());
                  glTexCoord2f(1,0);
                  glVertex3fv((mean+e2-ev+e3*s/8).data());
                  glEnd();
            }




                  /*glVertex3fv(mean+e3+ev);
                  glVertex3fv(mean-e3+ev);
                  glVertex3fv(mean-e3-ev);
                  glVertex3fv(mean+e3-ev);

                  glVertex3fv(mean+e2+e3);
                  glVertex3fv(mean-e2+e3);
                  glVertex3fv(mean-e2-e3);
                  glVertex3fv(mean+e2-e3);*/

            






         }
      }
      glDepthMask(1);

      glDisable(GL_TEXTURE_2D);
      glDisable(GL_BLEND);
   }





   glDisable(GL_DEPTH_TEST);
   glPopAttrib();
   glPopMatrix();


   forget(glContext);

}


// ----
