//---------------------------------------------------------------------------
#ifndef PCIRegisterPCH
#define PCIRegisterPCH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "LinearAlgebra.h"
#include "CameraController.h"
#include "ExaminerCameraController.h"
#include "ObjectClassProperty.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "DebugOutput.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "ParametricInCorePointCloud.h"

//---------------------------------------------------------------------------

namespace X4 {


struct dGrid{     //discrete Grid
   Vector3f* mean;
   Matrix3f* sigma;
   Matrix3f* eigenV;
   Vector3f* eigenW;
   int* n;
   double* score; //noetig falls man den Punktestand normieren will
   int* cnt;
   Vector3f offset; //hier unterscheiden sich die einzelnen partGrids vom cGrid

};

struct cGrid{     //"continuous" grid
 dGrid grid[8];
 //enthaelt um cellsize/2 verschobene Gitter in 0,X,Y,Z,XY,XZ,YZ,XYZ-Richtung
 //gemeinsame Attribute aus rGrid hier reinziehen
 Vector3f range, steps;
 int res;
};

typedef StaticVector<double,6> Vector6d;
typedef StaticMatrix<double,6,6> Matrix6d;



class X3EDITOR2_API PCIRegisterPC : public PCInteractionTool {
	X4_CLASS(PCIRegisterPC);

 private:
   cGrid* grid;
   float64 oldScore, angle, q_dist;
   
   float r_eigenAsp;
   card32 r_nRP;
   card32 r_gridRes,used_cells; //gridRes ist die eingestellte, r_gridRes die schon tatsaechlich initialisierte
   Matrix4f ctm;
   Matrix4f result;
   StaticVector<double,6> gradient,oldPos,dGrad;

   bool newDock,tex_init;
   double grad_pk;


   unsigned int t_gauss[16];
   int* randPoint;

   void initRandPoints(int nRP);
   int* getNPoints(PointCloud* pc, int n);
   void initGrid (PointCloud* dock, int g_res);
   double calcScore(PointCloud* dockee,Matrix4f change);
   StaticVector<double,6> calcGradient(PointCloud* dockee,Vector3f rotCenter=makeVector3f(0,0,0),double s=1);
   void calcDerivatives(PointCloud* dockee,Vector3f rotCenter,Matrix6d& hessian, Vector6d& grad,double s);
   void calcADerivatives(PointCloud* dockee,Vector3f rotCenter, Matrix6d& hessian, Vector6d& grad, double s );

   Matrix6d invertHessian(Matrix6d hessian);
   Vector3f calcRotCenter(PointCloud* dock, PointCloud* dockee);
   double doNewton (PointCloud* dock, PointCloud* dockee,int iterations,double step=1,bool analytic=true);
   double doDescent (PointCloud* dock, PointCloud* dockee, int iterations,bool adapt);
   void doOptimization(PointCloud* dock,PointCloud* dockee,int start_res=1,int step=1,int snr=1,int type=0);




 public:
   bool bDisplay,bNormals;
   int itnum;
   bool depth_test,g_init,fix_x, fix_y, fix_z, fix_alpha, fix_beta, fix_gamma;
   double scale;
   bool bnewton,bOptimization;
   float numIdentity,eigenAspect,gauss_alpha;
   double newScore;
   card32 gridRes,numIterations,nthPoint;
   float64 step,gStep;
   PointCloud* dock;
   PointCloud* dockee;
   std::string dockName;
   std::string dockeeName;
   bool ICP;
//   TPCIRegisterPCEditorFrame * editor;


   PCIRegisterPC();
   std::string selDockeeByName(std::string name);
   std::string selDockByName(std::string name);
   void selectClouds();
   std::string getFirstSel();
   void execute(bool adapt);
   void newton();
   void descent();
   void applyMatrix();


   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void glDrawTool(GLContext *glContext);
   //virtual void redrawScene() {if (onChange != NULL) {onChange->changed(PCI_CHANGE_VIEW);}}
};



}

#endif

