//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
//#include "cv_include.h"
//---------------------------------------------------------------------------
#include "PCIRegisterTexture.h"
//---------------------------------------------------------------------------
#include "CameraController.h"
#include "ExaminerCameraController.h"
#include "ObjectClassProperty.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "DebugOutput.h"
#include "PointCloud.h"
#include "ClassMethods.h"
#include "PointCloudIterators.h"
#include "SelectionIteratorAdaptor.h"
#include "NumericalClassProperty.h"
#include "Random.h"
#include "CmdObjSelection.h"
#include "CommandHistory.h"
#include "SelectionIteratorAdaptor.h"
#include "MaterialAttachment.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "PointCloudTools.h"
#include "SelectionTools.h"
#include "LinearAlgebra.inline.h"
#include "RGBAImage.h"

#include "opencv/cxcore.h"

//#include "PCIRegisterTextureWidget.h"
#ifndef _DEBUG
#include "TemplatedImage.inline.h"
#endif
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS(PCIRegisterTexture, 0)
{
	BEGIN_CLASS_INIT(PCIRegisterTexture);

   ADD_OBJECT_PROP(imgFeatures,0,ManualImageFeatures::getClass(),true)
  
   ADD_VAR_ARRAY_PROP( objPoints, 0, 
	new FixedArrayClassProperty("element" , MetaClass::FIRST_VERSION, NULL, 
	                           new Float32ClassProperty("", 0, (pointer_diff)NULL, NULL) , 3, sizeof(float32), 3))

	ADD_NOARGS_METHOD_EX(openFeatureWidget, "Feature Widget...", "Open Feature Widget for Editing", "", true);
   ADD_NOARGS_METHOD_EX(colorPoints, "Color Points...", "color cloud using given image and calculated matrix", "", true);
   ADD_NOARGS_METHOD_EX(calibrate, "Calibrate...", "calibrate, but don't apply", "", true);
   


  // PCIRegisterTextureMetaClass->getGUIDescriptor()->addClassEditor(PCIRegisterTextureEditor::getClass(), true);


}


void PCIRegisterTexture::assign(const Object* obj, X4CopyContext *context)
{
	Persistent::assign(obj,context);
	const  PCIRegisterTexture * o = dynamic_cast<const PCIRegisterTexture*>(obj);
	x4Assert( o != NULL );

}
//----------------------------------------------------------------------------
PCIRegisterTexture::PCIRegisterTexture() {
   current=0;
   camera=NULL;
   rot=NULL;
   trans=NULL;
   calib=false;
   //Image=new RGBAImage();
   
   t=IDENTITY4F;
   //featureWidget=NULL;
   imgFeatures=new ManualImageFeatures();
   imgFeatureWidget=new ManualImageFeaturesEditorWidget;
}

Matrix4f PCIRegisterTexture::calibrateCamera()
{
  /* CvPoint2D32f* imagePoints;
   CvPoint3D32f* objectPoints;
   Matrix4f result;
   int minSize=imPoints.size();
   if (objPoints.size() <minSize)
      minSize=objPoints.size();
   imagePoints=new CvPoint2D32f[minSize];
   objectPoints=new CvPoint3D32f[minSize];

   for (int i=0; i< minSize; i++)
   {
      imagePoints[i].x=imPoints[i][0];
      imagePoints[i].y=imPoints[i][1];

      objectPoints[i].x=objPoints[i][0];
      objectPoints[i].y=objPoints[i][1];
      objectPoints[i].z=objPoints[i][2];



   }
   int numPoints=minSize;
   CvSize imageSize;
   imageSize.width=iw;
   imageSize.height=ih;
   float* distortion=new float[4];
   trans=new float[3];
   camera=new float[9];
   rot=new float[9];

   cvCalibrateCamera( 1, &numPoints, imageSize,
                     imagePoints,objectPoints,
                     distortion, camera,
                     trans,rot,
                     0 );
 
     for (int i=0;i<3;i++)
   {
      debugOutput.cr();

      debugOutput << i <<" : " <<trans[i];
   }
     for (int i=0;i<9;i++)
   {
      debugOutput.cr();

      debugOutput << i <<" : " <<camera[i];
   }

   Matrix4f transM;
   transM=makeTranslation4f(trans[0],trans[1],trans[2]);
   Matrix4f rotM;
   rotM[0][0]=rot[0];
   rotM[0][1]=rot[3];
   rotM[0][2]=rot[6];
   rotM[0][3]=0;
   rotM[1][0]=rot[1];
   rotM[1][1]=rot[4];
   rotM[1][2]=rot[7];
   rotM[1][3]=0;
   rotM[2][0]=rot[2];
   rotM[2][1]=rot[5];
   rotM[2][2]=rot[8];
   rotM[2][3]=0;
   rotM[3][0]=0;
   rotM[3][1]=0;
   rotM[3][2]=0;
   rotM[3][3]=1;

   Matrix4f camM;
   camM[0][0]=camera[0];
   camM[0][1]=0;
   camM[0][2]=0;
   camM[0][3]=0;
   camM[1][0]=0;
   camM[1][1]=camera[4];
   camM[1][2]=0;
   camM[1][3]=0;
   camM[2][0]=0;
   camM[2][1]=0;
   camM[2][2]=0;
   camM[2][3]=0;
   camM[3][0]=camera[2];
   camM[3][1]=camera[5];
   camM[3][2]=0;
   camM[3][3]=1;


   result=camM*rotM*transM;
   Matrix4f result2=camM*rotM*transM;

   for (int i=0; i< minSize; i++)
   {
      debugOutput<< i<<".Bildpunkt:" <<imPoints[i][0]<<" , "<< imPoints[i][1];
      Vector3f p=objPoints[i];
      Vector3f pos=transformVector3f(result,p);
      Vector3f pos2=transformVector3f(result2,p);
      /*if (pos[3]>0.00001f)
         pos/=pos[3];
      debugOutput.cr();
      debugOutput<< i<<".Objpunkt:" <<pos[0]<<" , "<< pos[1];
      debugOutput.cr();
      debugOutput<< i<<".Objpunkt2:" <<pos2[0]<<" , "<< pos2[1];
      debugOutput.cr();


   }*/






   //TO-DO
   /*

      1.)Convert Vector2/3f to cv-Points
      2.)Create OpenCVMatrizes for result
      3.)store result to Matrix4f
      (easy but crappy conversions...)


      Later:
      Use own algorithm from CV-Exercises
      (easy-medium)

      ((Outside this function:
         Apply new Matrix (very easy)
         Project Points and color them accordingly (easy)
   */

            size_t nrPoints=imgFeatures->getFeatures().size();
            if ((int)objPoints.size()<nrPoints)
               nrPoints=objPoints.size();


            CvMat* M=cvCreateMat(12,1,CV_64FC1);
            CvMat* P=cvCreateMat(2*(int)nrPoints,12,CV_64FC1);
            for (unsigned i=0;i<nrPoints; i++)
            {
                double X=objPoints[i][0];
                double Y=objPoints[i][1];
                double Z=objPoints[i][2];
                double x=imgFeatures->getFeatures()[i][0];
                double y=imgFeatures->getFeatures()[i][1];

                //System.out.println("X " +X +" Y "+Y+" Z "+Z+" x "+x+ "y "+y); korrekt
                cvSetReal2D( P, 2*i,0, X);
                cvSetReal2D( P, 2*i,1, Y);
                cvSetReal2D( P,2*i,2,Z   );
                cvSetReal2D( P,2*i,3,1   );
                cvSetReal2D( P,2*i,4,0   );
                cvSetReal2D( P,2*i,5,0   );
                cvSetReal2D( P,2*i,6,0   );
                cvSetReal2D( P,2*i,7,0   );
                cvSetReal2D( P,2*i,8,-x*X   );
                cvSetReal2D( P,2*i,9,-x*Y   );
                cvSetReal2D( P,2*i,10,-x*Z   );
                cvSetReal2D( P,2*i,11,-x   );

                cvSetReal2D( P,2*i+1,0,0   );
                cvSetReal2D( P,2*i+1,1,0   );
                cvSetReal2D( P,2*i+1,2,0   );
                cvSetReal2D( P,2*i+1,3,0   );
                cvSetReal2D( P,2*i+1,4,X   );
                cvSetReal2D( P,2*i+1,5,Y   );
                cvSetReal2D( P,2*i+1,6,Z   );
                cvSetReal2D( P,2*i+1,7,1   );
                cvSetReal2D( P,2*i+1,8,-y*X   );
                cvSetReal2D( P,2*i+1,9,-y*Y   );
                cvSetReal2D( P,2*i+1,10,-y*Z   );
                cvSetReal2D( P,2*i+1,11,-y   );


            }

            int nr2=2*(int)nrPoints;

            double* w=new double[nr2*12];
            double* u=new double[nr2*nr2];
            double* v=new double[12*12];
            CvMat U,V,W;
            cvInitMatHeader( &U, nr2, nr2, CV_64FC1,u);
            cvInitMatHeader( &V, 12, 12, CV_64FC1,v);
            cvInitMatHeader( &W, nr2, 12, CV_64FC1,w);
            cvSVD(P,&W,&U,&V,CV_SVD_V_T);

            /*int minimumI=0;
            double minimumW=w.get(0,0);

            for (int i=0;i<w.rank();i++)
            {
               if(w.get(i,i)<minimumW)
               {
                   minimumW=w.get(i,i);
                   minimumI=i;
               }
            }*/
            //;
            //i-te Spalte von V=i.te Zeile von v', also transpose nicht ntig??
            for (int i=0;i<12;i++)
            {
                cvSetReal2D( M,i,0, cvGetReal2D( &V,11,i));
            }


            double gamma=cvGetReal2D( M,8,0)*cvGetReal2D( M,8,0)
                        +cvGetReal2D( M,9,0)*cvGetReal2D( M,9,0)
                        +cvGetReal2D( M,10,0)*cvGetReal2D( M,10,0);
            gamma=fabsl(1/sqrt(gamma));
            for (int i=0;i<12;i++)
            {
                cvSetReal2D( M,i,0, cvGetReal2D( M,i,0)*gamma);
            }


            Matrix4f Per;
            Per[0][0]=cvGetReal2D(  M, 0,0);
            Per[1][0]=cvGetReal2D(  M, 4,0);
            Per[3][0]=cvGetReal2D(  M, 8,0);
            Per[2][0]=0;
            Per[0][1]=cvGetReal2D(  M, 1,0);
            Per[1][1]=cvGetReal2D(  M, 5,0);
            Per[3][1]=cvGetReal2D(  M, 9,0);
            Per[2][1]=0;
            Per[0][2]=cvGetReal2D(  M, 2,0);
            Per[1][2]=cvGetReal2D(  M, 6,0);
            Per[3][2]=cvGetReal2D(  M, 10,0);
            Per[2][2]=0;
            Per[0][3]=cvGetReal2D(  M, 3,0);
            Per[1][3]=cvGetReal2D(  M, 7,0);
            Per[3][3]=cvGetReal2D(  M, 11,0);
            Per[2][3]=0;

            Per=Per.transpose();

			//Java-Code, if you wondered...
            /*double ox=-M.get(0,0)*M.get(8,0)-M.get(1,0)*M.get(9,0)-M.get(2,0)*M.get(10,0);
            double oy=-M.get(4,0)*M.get(8,0)-M.get(5,0)*M.get(9,0)-M.get(6,0)*M.get(10,0);
            oxCalc=ox;
            oyCalc=oy;
            //soweit korrekt :)
            double focus=Math.sqrt(M.get(0,0)*M.get(0,0)+M.get(1,0)*M.get(1,0)+M.get(2,0)*M.get(2,0)-ox*ox);
            foCalc=focus;


            double sign=1.;
            if (M.get(11,0) <0) sign=-1.;
            Rotcalc.set(0,0, sign*(ox*M.get(8,0)+M.get(0,0))/focus);
            Rotcalc.set(0,1, sign*(ox*M.get(9,0)+M.get(1,0))/focus);
            Rotcalc.set(0,2, sign*(ox*M.get(10,0)+M.get(2,0))/focus);
            Rotcalc.set(1,0, sign*(oy*M.get(8,0)+M.get(4,0))/-focus);//focusy=-focusx
            Rotcalc.set(1,1, sign*(oy*M.get(9,0)+M.get(5,0))/-focus);
            Rotcalc.set(1,2, sign*(oy*M.get(10,0)+M.get(6,0))/-focus);
            Rotcalc.set(2,0, M.get(8,0));
            Rotcalc.set(2,1, M.get(9,0));
            Rotcalc.set(2,2, M.get(10,0));
            

            double rotTx=sign*(M.get(3, 0)+ox*M.get(11,0))/focus;
            double rotTy=sign*(M.get(7, 0)+oy*M.get(11,0))/-focus;
            double rotTz=M.get(11,0);
            Matrix rotT=makeVector(rotTx,rotTy, rotTz);

            Matrix T=Rotcalc.transpose().times(rotT);


            T=T.times(-1);
            TxCalc=T.get(0,0);
            TyCalc=T.get(1,0);
            TzCalc=T.get(2,0);
            */
               float s=0;
               estPoints.clear();
               for (unsigned i=0; i< nrPoints; i++)
               {
                  debugOutput<< i<<".Bildpunkt:" <<imgFeatures->getFeatures()[i][0]<<" , "<< imgFeatures->getFeatures()[i][1];
                  Vector3f p=objPoints[i];
                  Vector3f pos=transformVector3f(Per,p);
                  s+=(float)imgFeatures->getFeatures()[i][0]/(float)pos[0];

                  debugOutput.cr();
                  debugOutput<< i<<".Objpunkt:" <<pos[0]*s<<" , "<< pos[1]*s;
                  debugOutput.cr();

                }
                if (nrPoints)
                  s=s/nrPoints;
                this->s=s;
                  debugOutput<< "s: "<< s <<"gamma: "<< gamma;
               for (unsigned i=0; i< nrPoints; i++)
               {
                  Vector3f p=objPoints[i];
                  Vector3f pos=transformVector3f(Per,p);
                  estPoints.push_back(makeVector2i(pos[0]*s,pos[1]*s));
               }


            Per=makeScale4f(s,s,s)*Per;

            return Per;








}

void PCIRegisterTexture::colorCloud(Matrix4f t,PointCloud* pc)
{

   SceneObjectIterator *newIt = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
   if (newIt == NULL ) return;

   BasicPointCloudIterator *readIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
   ModifyablePointCloudIterator *modIt=dynamic_cast<ModifyablePointCloudIterator*>(newIt);
   if (modIt == NULL ) return;
   if (!pc->providesAttribute("position")) return;
      AAT POSITION = pc->getAAT("position");

   if (!pc->providesAttribute("color")) return;
   AAT COLOR = pc->getAAT("color");
   float scale=1.0f/255.0f;
   iw=imgFeatures->getImage()->getSizeX();
   ih=imgFeatures->getImage()->getSizeY();
   while (!readIt->atEnd())
   {
      Vector3f p=readIt->get3f(POSITION);
      Vector3f pos=transformVector3f(t,p);

      if ((int)pos[0]>0 && (int)pos[0]<iw && (int)pos[1]>0 && (int)pos[1]<ih )
      {
         Vector4ub colora=imgFeatures->getImage()->getPixel((int)pos[0],(int)pos[1]);
         Vector3f color;
         color[0]=scale*(float)colora[2];
         color[1]=scale*(float)colora[1];
         color[2]=scale*(float)colora[0];
         modIt->set3f(COLOR, color);
      }





    readIt->next();
   }
}

void PCIRegisterTexture::calibrate()
{
   t=calibrateCamera();
   calib=true;
   debugOutput<<t;
   imgFeatureWidget->redraw();
   imgFeatureWidget->show();

}

void PCIRegisterTexture::colorPoints()
{
   SGLegacyPCSceneAdaptor sceneLA( scene );
   std::string pcName = sceneLA.getNameWithPath( 0 );
   checkAttribute( scene, pcName, "color", 3, VAD::DATA_FORMAT_FLOAT32 );
   colorCloud(t, sceneLA.getPointCloud(0));
}

void PCIRegisterTexture::openFeatureWidget()
{
	delete imgFeatureWidget;
	imgFeatureWidget=new ManualImageFeaturesEditorWidget();
	imgFeatureWidget->setup(imgFeatures);
	imgFeatureWidget->redraw();
	imgFeatureWidget->show();
	
	
	//open a QDialog with the Widget



}

Matrix4f PCIRegisterTexture::getCameraMatrix()
{
 return t;

}

void  PCIRegisterTexture::setCameraMatrix(Matrix4f cam)
{
 t=cam;
}


std::vector<Vector3f> PCIRegisterTexture::getObjPoints()
{
   return objPoints;

}

void PCIRegisterTexture::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState){


         SGLegacyPCSceneAdaptor sceneLA(scene);

         if (buttonsState.getRight())
         {
            int sel=-1;
            int quad_tol=100;
            int min_qdist=quad_tol;
            //determine nearest Feature to click

            Matrix4f T;
            Matrix4f curtransmatr = sceneLA.getState(0)->dynamicState->ctm;
            Matrix4f projmatr = sceneLA.getState(0)->staticState->viewFrustum.getProjectionMatrix();
            Matrix4f viewmatr = sceneLA.getState(0)->staticState->camera.getViewMatrix();
            T = projmatr*viewmatr*curtransmatr;



            for (int i=0; i<(int)objPoints.size();i++)
            {
               Vector4f homPos = T * (expand3To4(objPoints[i]));
               float32 wmatrix = homPos[3];

               if (fabs(wmatrix) > 0.001f)
               {
               int32 X = w/2+ homPos[0]/wmatrix * w/2;
               int32 Y = h/2+ homPos[1]/wmatrix * h/2;
               //float32 Z=homPos[2]/wmatrix;

                  if (!(X<0 || X>w || Y<0 || Y>h))
                  {
                     debugOutput<<"X: "<<X<<" x: "<<x<<" Y: "<<Y<<" y: "<<y;
                     debugOutput.cr();
                     double quad_dist=(X-x)*(X-x)+(Y-y)*(Y-y);
                     if (quad_dist<=quad_tol && quad_dist<=min_qdist)
                     {
                         min_qdist=quad_dist;
                         sel=i;
                     }
                  }
               }

            }
            //erase Feature if found
            if (sel!=-1 && objPoints.size()>0)
            {
               objPoints.erase(objPoints.begin()+sel);
            }


         }
         else
         {

         CmdObjSelectionByButton *newCmd = new CmdObjSelectionByButton();
         newCmd->setup(4);
         scene->getCommandHistory()->executeCommand(scene, newCmd, true);

         CmdObjChangeSelectionOnClick *newCmd2 = new CmdObjChangeSelectionOnClick();
         newCmd2->setup(x, y, false, 5, 1, (scene->getRootState()->staticState->camera), (scene->getRootState()->staticState->viewFrustum), w, h, true);
         scene->getCommandHistory()->executeCommand(scene, newCmd2, true);

         PointCloud *pc = sceneLA.getPointCloud(0);//To-Do...
         SceneObjectIterator *itSO = pc->createIterator(SOIT::CAP_BASIC_PC);
         PointCloudIterator *it = dynamic_cast<PointCloudIterator*>(itSO);
         x4Assert(itSO);
         //SelectionIteratorAdaptor *sit = new SelectionIteratorAdaptor(it, pc->getDescr());
         IteratorAdaptor *sit = getSelectionIteratorAdaptor(pc, it);

         AAT POSITION = pc->getAAT("position");

            while(!sit->atEnd()) {
                  Vector3f pos=sit->get3f(POSITION);;
                  objPoints.push_back(pos);

                  sit->next();
            };
         current=(int)objPoints.size()-1;
         delete sit;
         delete it;

        }


   forget(modifiersState);
 }
void PCIRegisterTexture::mouseMoved(int32 x, int32 y){

       Matrix4fAttachment * att = (Matrix4fAttachment *)scene->getAttachments()->getData( "tMatrix" );
       if( att == NULL )
       {
         att = new Matrix4fAttachment;
         att->setup( "tMatrix", 0 );
         scene->getAttachments()->attachData( att );
       }

       Matrix4f color=IDENTITY4F;
       color[0][0]=(float)x/1000;
       color[0][1]=(float)y/1000;
       att->setMatrix( color );







forget(x+y); }
void PCIRegisterTexture::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState){forget(x+y);forget(buttonsState); forget(modifiersState); }
void PCIRegisterTexture::areaResize(card32 width, card32 height){
   h = height;
   w = width; }

void PCIRegisterTexture::keyPressed(char key){forget(key); }
void PCIRegisterTexture::keyDown(GeneralKey key){forget(key); }
void PCIRegisterTexture::keyUp(GeneralKey key){forget(key); }

void PCIRegisterTexture::glDrawTool(GLContext *glContext){
/*   if (false)
{
      glMatrixMode(GL_PROJECTION);
      glPushMatrix();
      glLoadIdentity();

      glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
      glLoadIdentity();
      glTranslatef(trans[0],trans[1],trans[2]);
      glMultMatrixf(rot);
      glMultMatrixf(camera);
   }

   */

   glPushMatrix();

   SGLegacyPCSceneAdaptor sceneLA(scene);
   glMultMatrixf(sceneLA.getState(0)->dynamicState->ctm.data());
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glEnable(GL_POINT_SIZE);
   glPointSize(8.0f);
   glBegin(GL_POINTS);

   float modelview[16], projection[16];
         glGetFloatv( GL_MODELVIEW_MATRIX, modelview );
         glGetFloatv( GL_PROJECTION_MATRIX, projection );




   glColor3f(0.0f,0.0f,1.0f);

   for (card32 i=0;i<objPoints.size();i++)
      glVertex3fv(objPoints[i].data());
   

 
   //To-Do:
   //  3.)Numbers


   glEnd();
   glDisable(GL_POINT_SIZE);


   glPopAttrib();
   glPopMatrix();
   /*if (false)
   {
      glPopMatrix();
      glMatrixMode(GL_PROJECTION);
      glPopMatrix();
   } */
   forget(glContext);
 }
   //virtual void PCIRegisterTexture::redrawScene() {if (onChange != NULL) {onChange->changed(PCI_CHANGE_VIEW){ }}}

