//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ClassMethods.h"
#include "PCISurfaceReconstructionIO.h"
#include "UnstructuredInCorePointCloud.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "VertexArray.h"
#include "ProgressWindow.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( PCISurfaceReconstructionIO, 0 )
{
	BEGIN_CLASS_INIT( PCISurfaceReconstructionIO )
	ADD_STRING_PROP ( sourcePC, 0 )
	ADD_STRING_PROP ( destPC, 0 )
	ADD_NOARGS_METHOD ( exportToPWN )
	ADD_NOARGS_METHOD ( exportToPTS )
	ADD_NOARGS_METHOD ( exportToNPTS )
	ADD_NOARGS_METHOD ( exportToPoints )
	ADD_NOARGS_METHOD ( importFromPoly )
}

/**
 * Constructor
 */
PCISurfaceReconstructionIO::PCISurfaceReconstructionIO()
{
	sourcePC = "root/pc1";
	destPC = "root/pc2";
}

/**
 * Destructor
 */
PCISurfaceReconstructionIO::~PCISurfaceReconstructionIO()
{
}
void PCISurfaceReconstructionIO::exportToPWN(){
	//open file dialog
	QFileDialog fd;
	fd.setFileMode(QFileDialog::AnyFile);
	fd.setViewMode(QFileDialog::List);
	fd.setAcceptMode(QFileDialog::AcceptSave);
	fd.setFilter("PWN-Files (*.pwn)");
	fd.setDefaultSuffix("pwn");
	QStringList filelist; 
	if (fd.exec()) filelist = fd.selectedFiles();
	if (filelist.isEmpty()) return;
	QString filename = filelist[0];
	PwnPtsExport(filename,true);
}
void PCISurfaceReconstructionIO::exportToPTS(){
	//open file dialog
	QFileDialog fd;
	fd.setFileMode(QFileDialog::AnyFile);
	fd.setViewMode(QFileDialog::List);
	fd.setAcceptMode(QFileDialog::AcceptSave);
	fd.setFilter("PWN-Files (*.pts)");
	fd.setDefaultSuffix("pts");
	QStringList filelist; 
	if (fd.exec()) filelist = fd.selectedFiles();
	if (filelist.isEmpty()) return;
	QString filename = filelist[0];
	PwnPtsExport(filename,false);
}
void PCISurfaceReconstructionIO::PwnPtsExport(QString filename, bool writeNormals){
	//open file for writing
	FILE* file = fopen(filename.toStdString().c_str(),"w");
	if(!file){
		debugOutput << "Could not open file " << filename.toStdString() << " for writing\n";
		return;
	}
	UnstructuredInCorePointCloud* pc = dynamic_cast<UnstructuredInCorePointCloud*>( getPointCloud( scene, sourcePC ) );
	if(!pc){
		debugOutput << "PCCSurfaceReconstructionIO: Invalid (NULL) PointCloud!\n";
		return;
	}
	PointSet* ps = pc->getPointSet();
	if(!ps->providesAttribute("position") || !ps->providesAttribute("normal")){
		debugOutput << "PCCSurfaceReconstructionIO: PointCloud doesn't provide attribute position or normal\n";
		return;
	}
	//create BasicIterator
	SceneObjectIterator *newIt = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC ) );
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
	AAT POSITION = ps->getAAT("position");
	AAT NORMAL = ps->getAAT("normal");
	//start output
	fprintf (file,"%u\n",ps->getNumEntries());
	while(!basicIt->atEnd()){
		Vector3f pos = basicIt->get3f(POSITION);
		fprintf (file,"%f %f %f\n",pos[0],pos[1],pos[2]);
		basicIt->next();
	}
	//write normals for PWN format?
	if(writeNormals){
		basicIt->reset();
		while(!basicIt->atEnd()){
			Vector3f normal = basicIt->get3f(NORMAL);
			fprintf (file,"%f %f %f\n",normal[0],normal[1],normal[2]);
			basicIt->next();
		}
	}
	fclose(file);
	delete newIt;
}
void PCISurfaceReconstructionIO::importFromPoly(){
	//show file dialog
	QFileDialog fd;
	fd.setFileMode(QFileDialog::ExistingFile);
	fd.setViewMode(QFileDialog::List);
	fd.setAcceptMode(QFileDialog::AcceptOpen);
	fd.setFilter("POLY-Files (*.poly)");
	QStringList filelist; 
	if (fd.exec()) filelist = fd.selectedFiles();
	if (filelist.isEmpty()) return;
	QString filename = filelist[0];

	UnstructuredInCoreTriangleMesh* pc;
	pc = new UnstructuredInCoreTriangleMesh();
	//create descriptors and push attributes
	VertexDescriptor faceDescr;
	faceDescr.pushAttrib(mVAD("index", 3, VAD::DATA_FORMAT_INT32));
	VertexDescriptor* m_vd = new VertexDescriptor;
	m_vd->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	m_vd->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
	m_vd->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	m_vd->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));

	//import
	char s[4096];
	unsigned nVertices = 0;
	unsigned nFaces = 0;
	FILE* file = fopen (filename.toStdString().c_str(),"r");
	if(!feof(file) && fgets(s,4096,file)) sscanf(s,"%u",&nVertices);
	if(!feof(file) && fgets(s,4096,file)) sscanf(s,"%u",&nFaces);
	pc->clearAndSetup(m_vd, nVertices, &faceDescr, nFaces);
	PointSet* ps = pc->getPointSet();
	AAT POSITION = ps->getAAT( "position" );
	unsigned vertexcounter = 0;
	float x,y,z;
	while(!feof(file) && fgets(s,4096,file) && vertexcounter != nVertices){
		if(sscanf(s,"%f %f %f",&x,&y,&z)){
			ps->set3f(vertexcounter,POSITION,makeVector3f(x,y,z));
			vertexcounter++;
		}
	}
	PointSet* tri = pc->getTriangles();
	AAT INDEX = tri->getAAT( "index" );
	unsigned facecounter = 0;
	unsigned a,b,c,d;
	while(!feof(file) && fgets(s,4096,file) && facecounter != nFaces){
		if(sscanf(s,"%d %d %d %d",&d,&a,&b,&c)){
			tri->set3i(facecounter,INDEX,makeVector3i(a,b,c));
			facecounter++;
		}
	}
	addPointCloud(scene,pc,destPC);
}
void PCISurfaceReconstructionIO::exportToNPTS(){
	//open file dialog
	QFileDialog fd;
	fd.setFileMode(QFileDialog::AnyFile);
	fd.setViewMode(QFileDialog::List);
	fd.setAcceptMode(QFileDialog::AcceptSave);
	fd.setFilter("NPTS-Files (*.npts)");
	fd.setDefaultSuffix("npts");
	QStringList filelist; 
	if (fd.exec()) filelist = fd.selectedFiles();
	if (filelist.isEmpty()) return;
	QString filename = filelist[0];
	//open file for writing
	FILE* file = fopen(filename.toStdString().c_str(),"w");
	if(!file){
		debugOutput << "Could not open file " << filename.toStdString() << " for writing\n";
		return;
	}
	UnstructuredInCorePointCloud* pc = dynamic_cast<UnstructuredInCorePointCloud*>( getPointCloud( scene, sourcePC ) );
	if(!pc){
		debugOutput << "PCCSurfaceReconstructionIO: Invalid (NULL) PointCloud!\n";
		return;
	}
	PointSet* ps = pc->getPointSet();
	if(!ps->providesAttribute("position") || !ps->providesAttribute("normal")){
		debugOutput << "PCCSurfaceReconstructionIO: PointCloud doesn't provide attribute position or normal\n";
		return;
	}
	//create BasicIterator
	SceneObjectIterator *newIt = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC ) );
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
	AAT POSITION = ps->getAAT("position");
	AAT NORMAL = ps->getAAT("normal");
	//start output
	fprintf (file,"%u\n",ps->getNumEntries());
	while(!basicIt->atEnd()){
		Vector3f pos = basicIt->get3f(POSITION);
		Vector3f normal = basicIt->get3f(POSITION);
		fprintf (file,"%f %f %f  %f %f %f\n",pos[0],pos[1],pos[2],normal[0],normal[1],normal[2]);
		basicIt->next();
	}
	fclose(file);
	delete newIt;
}
void PCISurfaceReconstructionIO::exportToPoints(){
		//open file dialog
	QFileDialog fd;
	fd.setFileMode(QFileDialog::AnyFile);
	fd.setViewMode(QFileDialog::List);
	fd.setAcceptMode(QFileDialog::AcceptSave);
	fd.setFilter("POINTS-Files (*.points)");
	fd.setDefaultSuffix("points");
	QStringList filelist; 
	if (fd.exec()) filelist = fd.selectedFiles();
	if (filelist.isEmpty()) return;
	QString filename = filelist[0];
	//open file for writing
	FILE* file = fopen(filename.toStdString().c_str(),"w");
	if(!file){
		debugOutput << "Could not open file " << filename.toStdString() << " for writing\n";
		return;
	}
	UnstructuredInCorePointCloud* pc = dynamic_cast<UnstructuredInCorePointCloud*>( getPointCloud( scene, sourcePC ) );
	if(!pc){
		debugOutput << "PCCSurfaceReconstructionIO: Invalid (NULL) PointCloud!\n";
		return;
	}
	PointSet* ps = pc->getPointSet();
	if(!ps->providesAttribute("position")){
		debugOutput << "PCCSurfaceReconstructionIO: PointCloud doesn't provide attribute position\n";
		return;
	}
	//create BasicIterator
	SceneObjectIterator *newIt = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC ) );
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
	AAT POSITION = ps->getAAT("position");
	//start output
	while(!basicIt->atEnd()){
		Vector3f pos = basicIt->get3f(POSITION);
		fprintf (file,"%f %f %f\n",pos[0],pos[1],pos[2]);
		basicIt->next();
	}
	fclose(file);
	delete newIt;
}