#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCITestMLSRenderer.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "GLTexture.h"
#include "GLContext.h"
#include "GLShaderMaterial.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCITestMLSRenderer ,0)
{
	BEGIN_CLASS_INIT( PCITestMLSRenderer );
	ADD_FLOAT32_PROP( m_SplatSize, 0);
}

PCITestMLSRenderer::PCITestMLSRenderer(void)
{
	m_Texture = NULL;
	m_SplatSize = 0.1f;
}

PCITestMLSRenderer::~PCITestMLSRenderer(void)
{
	delete m_Texture;
}

void PCITestMLSRenderer::assign(const Object* obj, X4CopyContext *context)
{
	//PCInteractionTool::assign(obj,context);
	const PCITestMLSRenderer * o = dynamic_cast<const PCITestMLSRenderer*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::PCITestMLSRenderer::glDrawTool( GLContext *glContext )
{

	std::vector<PointCloud*> pointClouds = getAllPointCloudsFromScene();
	if( pointClouds.size() == 0)
		return;
	PointCloud * pc = pointClouds[0];
	UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	if( !upc )
		return;

	AAT posAAT = upc->getAAT("position");
	AAT normalAAT = upc->getAAT("normal");

	if( m_Texture == NULL )
	{

	
		m_Texture = new GLTextureRect(glContext);
		card32 numPoints = 16; 
		std::vector<float> texMem(numPoints*4*2);
		for( unsigned i=0;i<numPoints;i++ )
		{
			Vector3f pos = upc->getPointSet()->get3f(i,posAAT);
			Vector3f normal = upc->getPointSet()->get3f(i,normalAAT);
			texMem[i*4+0] = pos[0];
			texMem[i*4+1] = pos[1];
			texMem[i*4+2] = pos[2];
			texMem[i*4+numPoints*4+0] = normal[0];
			texMem[i*4+numPoints*4+1] = normal[1];
			texMem[i*4+numPoints*4+2] = normal[2];
		}
		m_Texture->createFloatTextureFromMemory(numPoints,2,4, &texMem[0],true);
	}

	GLShaderMaterial * mat = dynamic_cast<GLShaderMaterial*>(getScene()->getGLMaterial(pc->getMaterialIndex()));
	if( mat == NULL )
	{
		return;
	}

	SceneObjectIterator * iter = upc->createIterator(SOIT::CAP_BASIC_PC);
	m_Texture->bind();
	//mat->draw(getScene(),getScene()->getRootState(),NULL,iter,glContext);
	glEnable(GL_DEPTH_TEST);
	mat->bindShaders(glContext);
	glBegin(GL_QUADS);
	for( unsigned i=0;i<upc->getNumPoints() && i < 16;i++ )	
	{
		Vector3f pos = upc->getPointSet()->get3f(i,posAAT);
		Vector3f normal = upc->getPointSet()->get3f(i,normalAAT);

		Matrix3f tmp = calcTangentSystem(normal);
		Vector3f p1 = pos + tmp[0] * m_SplatSize;
		Vector3f p2 = pos + tmp[1] * m_SplatSize;
		Vector3f p3 = pos - tmp[0] * m_SplatSize;
		Vector3f p4 = pos - tmp[1] * m_SplatSize;
		glNormal3fv(normal.data());
		glVertex3fv(p1.data());
		glVertex3fv(p2.data());
		glVertex3fv(p3.data());
		glVertex3fv(p4.data());
	}
	glEnd();

	mat->unbindShaders(glContext);
	m_Texture->unbind();
	delete iter;
}