//---------------------------------------------------------------------------
#ifndef PCITopologyAnalyzerH
#define PCITopologyAnalyzerH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

class QImage;

namespace X4 {

class PCITopologyAnalyzerEditorWidget;
class InCorePCTopologyGraph;
class PointCloud;
class PointSet;


class X3EDITOR2_API PCITopologyAnalyzer : public PCInteractionTool {
	X4_CLASS(PCITopologyAnalyzer)
 private:

	std::string selectedCloud;
	card32 selectedPoint;
	card32 comparePoint;
	card32 quantization;
	float32 epsilonFact;
	float32 imageGain;
	float32 imageOffset;
	float32 sigma;
	card32 expNumPts;
	
	enum LocalModel {lmMeanSquare, lmRobustStat, lmKernel};
	
	LocalModel localModel;

	card32 width;
	card32 height;
	bool leftMouseDown;
	bool rightMouseDown;
	
	void processSelection(int x, int y, bool left);
	void computeViz(QImage &img, bool compCompare);
	void getEnv(PointCloud *&pc, PointSet *&ps, InCorePCTopologyGraph *&tg);
	void updateViz();
	void computeLocalVariance();
	void computeLinkVariance();
	
 public:
	PCITopologyAnalyzer();

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual bool drawGLSceneRendering();
   
  friend PCITopologyAnalyzerEditorWidget;
};

}

#endif