//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCITopologyAnalyzerEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCITopologyAnalyzer.h"
#include "ObjectViewsTable.h"
#include "StringHelper.h"
#include "ExceptionHandling.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_CLASS_EDITOR( PCITopologyAnalyzer )

PCITopologyAnalyzerEditorWidget::PCITopologyAnalyzerEditorWidget(QWidget * parent, Qt::WindowFlags f)
:QWidget(parent,f) 
{
	ui.setupUi(this);
}

PCITopologyAnalyzerEditorWidget::~PCITopologyAnalyzerEditorWidget(void) {}

void PCITopologyAnalyzerEditorWidget::setup( PCITopologyAnalyzer * obj )
{
	m_Object = obj;
}

void PCITopologyAnalyzerEditorWidget::updateEditor() {
	ui.lineEditTarget->setText(m_Object->selectedCloud.c_str());
	ui.spinBoxSelectedPoint->setValue(m_Object->selectedPoint);
	ui.dsbEpsilon->setValue(m_Object->epsilonFact);
	ui.spinBoxQuantization->setValue(m_Object->quantization);
	ui.doubleSpinBoxImageGain->setValue(m_Object->imageGain);
	ui.doubleSpinBoxImageOffset->setValue(m_Object->imageOffset);
	ui.radioButtonMeanSquare->setChecked(m_Object->localModel == PCITopologyAnalyzer::lmMeanSquare);
	ui.radioButtonRobustStat->setChecked(m_Object->localModel == PCITopologyAnalyzer::lmRobustStat);
	ui.radioButtonKernel->setChecked(m_Object->localModel == PCITopologyAnalyzer::lmKernel);
	ui.spinBoxExpNumPts->setValue(m_Object->expNumPts);
	ui.doubleSpinBoxSigma->setValue(m_Object->sigma);
	if (ui.checkBoxContinuousUpdate->isChecked() && m_Object->selectedCloud != "") {
		if (m_Object->leftMouseDown) {
			computeViz(false);
		} else if (m_Object->rightMouseDown) {
			computeViz(true);
		}
	}
}

void PCITopologyAnalyzerEditorWidget::on_pushButtonRecompute_clicked() {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	computeViz(false);
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void PCITopologyAnalyzerEditorWidget::on_pushButtonCompare_clicked() {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	computeViz(true);
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void PCITopologyAnalyzerEditorWidget::on_spinBoxSelectedPoint_valueChanged(int i) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (i < 0) i = 0;
	m_Object->selectedPoint = i;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void PCITopologyAnalyzerEditorWidget::on_doubleSpinBoxImageGain_valueChanged(double d) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_Object->imageGain = d;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}
void PCITopologyAnalyzerEditorWidget::on_doubleSpinBoxImageOffset_valueChanged(double d) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_Object->imageOffset = d;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void PCITopologyAnalyzerEditorWidget::on_spinBoxQuantization_valueChanged(int i) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (i < 4) i = 4;
	m_Object->quantization = i;
	m_Object->redrawScene();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}


void PCITopologyAnalyzerEditorWidget::on_lineEditTarget_textChanged(const QString &text) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_Object->selectedCloud = qString2STLString(text);
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void PCITopologyAnalyzerEditorWidget::on_dsbEpsilon_valueChanged(double d) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_Object->epsilonFact = (float32)d;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}
void PCITopologyAnalyzerEditorWidget::on_radioButtonMeanSquare_clicked(bool checked) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (checked) m_Object->localModel = PCITopologyAnalyzer::lmMeanSquare;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}
void PCITopologyAnalyzerEditorWidget::on_radioButtonRobustStat_clicked(bool checked) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (checked) m_Object->localModel = PCITopologyAnalyzer::lmRobustStat;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}
void PCITopologyAnalyzerEditorWidget::on_radioButtonKernel_clicked(bool checked) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (checked) m_Object->localModel = PCITopologyAnalyzer::lmKernel;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}
void PCITopologyAnalyzerEditorWidget::on_spinBoxExpNumPts_valueChanged(int i) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_Object->expNumPts = i;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}
void PCITopologyAnalyzerEditorWidget::on_doubleSpinBoxSigma_valueChanged(double d) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_Object->sigma = d;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}		

void PCITopologyAnalyzerEditorWidget::on_pushButtonLocalVariance_clicked() {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_Object->computeLocalVariance();
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}		

void PCITopologyAnalyzerEditorWidget::on_pushButtonLinkVariance_clicked() {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_Object->computeLinkVariance();
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}		

void PCITopologyAnalyzerEditorWidget::computeViz(bool compare) {
	QImage img;
	m_Object->computeViz(img, false);
	
	if (compare) {
		QImage img2;
		m_Object->computeViz(img2, true);
		const int w = img.width();
		const int h = img.height();
		double diff = 0;
		for (int y=0; y<h; y++) {
			for (int x=0; x<w; x++) {
				QRgb c1 = img.pixel(x,y);
				QRgb c2 = img2.pixel(x,y);
				QRgb c = qRgb(abs(qRed(c1)-qRed(c2)), abs(qGreen(c1)-qGreen(c2)), abs(qBlue(c1)-qBlue(c2)));
				diff += double(abs(qRed(c1)-qRed(c2)))/255.0f/(double(w)*double(h));
				img.setPixel(x,y,c);
			}
		}
		ui.doubleSpinBoxDifference->setValue(diff);
	}

	QPixmap pix = QPixmap::fromImage(img);
	ui.labelVisualization->setPixmap(pix);
}

void PCITopologyAnalyzerEditorWidget::resizeEvent(QResizeEvent *ev) {
	int w = ui.frame_2->width() - 6;
	ui.labelVisualization->setMinimumHeight(w);
}






#include "PCITopologyAnalyzerEditorWidget_moc.h"
