//---------------------------------------------------------------------------
#ifndef PCITopologyComputationH
#define PCITopologyComputationH
//---------------------------------------------------------------------------
#include "x3editor2.h"
#include "RegularData.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------

namespace X4 {

class InCorePCTopologyGraph;
class LocalGeometryModelStatistics;
class EventViewerWidget;
class PCCComputeTopology;
class HoughTransformCornerDetector;
class LocalGeometryUncertainty;
class TopologyEstimator;


/**
	an interactive tool to compute & visualize the topology of point clouds
*/
class X3EDITOR2_API PCITopologyComputation : public PCInteractionTool {

	X4_CLASS( PCITopologyComputation )

 public:
	// feedbackMode
	static const card32 FM_NONE = 0;
	static const card32 FM_SHOW_LOCAL_MODEL = 1;

 private:
	// --- tool state ---

	// - target
	std::string targetNode;
	card32 selectedPoint;
	card32 comparePoint;

	// - settings
	PCCComputeTopology *baseTopologyEstimator;
	LocalGeometryModelStatistics *localModel;
	HoughTransformCornerDetector *htCornerDetector;
	LocalGeometryUncertainty *uncertModel;
	TopologyEstimator *topologyEstimator;

	bool readStdDevFromPC;
	float32 constStdDev;
	bool readSamplingDensityFromPC;
	float32 constSamplingDensity;
	bool colorPoints;
		

	// - rendering
	bool showCompleteTopology;
	bool showUncertainty;
	bool showLocalModelsVis;
	bool showCurrentLocalModelVis;
	bool showAllLocalModelVis;
	float32 localModelVisRadius;
	float32 localModelVisGamma;
	card32 maxNumMaximaVis;
	card32 feedbackMode;
	
	// - computation
	bool addExtraPosChannel;
	card32 numDiffusionSteps;
	bool useMaxDiffusion;
	
	// --- internal ---
	card32 width;
	card32 height;
	bool leftMouseDown;
	bool rightMouseDown;	
	RegularDataF lastResult;

	bool getEnv(PointCloud *&pc, PointSet *&ps, InCorePCTopologyGraph *&tg);
	void processSelection(int x, int y, bool left); 
	void showLocalModel(bool compare);
	void showFeedbackWindow();
	/// delete render buffers etc. of selected pc (after modifying point set directly)
	void invalidateAttachments();
	
	EventViewerWidget *feedbackWindow;
	
 protected:
	void computeBaseTopology();
	void computeUncertainty();
	void computeUncertainty_internCmdObjCall();
	void computeLocalModelMaxima();
	void computeLocalModelMaxima_internCmdObjCall();
	void diffuseUncertainty();
	void diffuseUncertainty_internCmdObjCall();
	void computeTopologyGreedy();
	void computeTopologyGreedy_internCmdObjCall();
	
 public:

	PCITopologyComputation(void);

   virtual void connectToSceneImpl(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget, QWidget* toolWidget);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key) {}
   virtual void keyUp(GeneralKey key) {}

   virtual bool drawGLSceneRendering();

	~PCITopologyComputation(void);


};


}

#endif