//---------------------------------------------------------------------------
#ifndef PCITriangleMeshH
#define PCITriangleMeshH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "UnstructuredInCoreTriangleMesh.h"
//---------------------------------------------------------------------------

namespace X4 {
class MajorKDTree;
/// Triangle mesh edit tool
class X3EDITOR2_API PCITriangleMesh : public PCInteractionTool 
{
	X4_CLASS( PCITriangleMesh )

	private:
		/// Node name of the mesh
		std::string meshNode;
		/// Parameterization mode
		card32 parameterizationMode;
		/// Repeat texture
		float32 textureRepeat;
		/// Max distance for snapping
		float32 maxSnapDistance;
		/// closeMesh: check normals
		bool checkNormals;
		/// createGroundPlane: up direction
		card32 upDirection;

      /// use PCA alignment for plane mode?
      bool pcaROT;
      /// first tex coord index (either pca or worldcoords, plane mode only)
      card32 coordIndex0;
      /// second tex coord index (either pca or worldcoords, plane mode only)
      card32 coordIndex1;

		card32		m_GroundPlaneResolution;
		float32		m_SampleRadius;
		bool		m_DeleteTriangleMode;
		bool		m_DeleteAllTrianglesWithinPlane;
		bool		m_SnapSelectedPointsOnly;	// restrict vertex snapping

		/// Find mesh in scene graph
		UnstructuredInCoreTriangleMesh* getMesh();

		MajorKDTree	*	m_Temp_KDtree;
		UnstructuredInCoreTriangleMesh * m_Temp_Mesh;
		std::vector<mpint>	m_Temp_DeleteCache;
		std::vector<mpcard>	m_Temp_OrgIndexCache;

	public:
		/// Constructor
		PCITriangleMesh();
		/// Constructor
		~PCITriangleMesh();
		
      virtual void assign(const Object* obj, X4CopyContext *context = NULL);
	  virtual void read(InputObjectStream *s);

	  /// Compute vertex normals based on triangles
	  void computeNormals();
	  // flip triangle indices in case normal orientation does not match current normal
	  void adjustTriangleIndicesToNormals();
		/// Smooth out normals
		void smoothNormals();
		/// Parameterize mesh
		void parameterizeMesh();

		/// Create a ground plane
		void createGroundPlane();

		/// connect equal edges in triangle topology
		void closeMesh();

		void openMesh();

		/// snap nearby vertices and calls closeMesh()
		void snapAndClose();

		/// snap only
		void snap();

		/// subdivide mesh (non-smoothing)
		void subdivide();
		static void subdivideMesh( UnstructuredInCoreTriangleMesh* mesh );

		/// subdivide mesh to a given resolution (don't subdivide triangles smaller than specified edge length)
		static void subdivideMesh( UnstructuredInCoreTriangleMesh* mesh, float maxEdgeLength);
		void constraintSubdivision();

		/// create an upsampled copy of a triangle mesh (main purpose is precomputation of ambient occlusion)
		void upsampleMesh();

		/// remove double triangles and invalid triangles
		void cleanMesh();

		/// returns poisson disk sampled point cloud with attached topology graph
		void pointSampleAndCreateTop();

		///computes cotangent weigts for mesh edges, saved in topology att, channel "weights"
		void computeCoTanWeights();

		/// merge all selected triangle meshes into one
		void mergeSelected();

		// magic
		void tryToRepairTopology();

		// intersect all selected point clouds with triangle mesh
		void pcIntersection();

		virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
		virtual void mouseMoved(int32 x, int32 y){}
		virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState){}
		virtual void areaResize(card32 width, card32 height){}
		virtual void keyDown(GeneralKey key){}
		virtual void keyUp(GeneralKey key){}
		virtual void glDrawTool(GLContext *glContext){}
		virtual void redrawScene() {if (onChange != NULL) {onChange->changed(PCI_CHANGE_VIEW);}}
		virtual void connectToSceneImpl(Scene *scene,OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget, QWidget* toolWidget){ this->scene = scene; }
};

}

#endif

