//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCImageTools.h"
//---------------------------------------------------------------------------
#include "FixedArrayClassProperty.h"
#include "PointSet.h"
#include "VertexDescriptor.h"
#include "NumericalClassProperty.h"
#include "BooleanClassProperty.h"
#include "RegularData.h"
#include "RegularData.inline.h"
#include "RegularDataProcessing.h"
#include "RegularDataProcessing.inline.h"
#include "RegularDataProcessingLD.h"
#include "DebugOutput.h"
#include "LinearAlgebra.inline.h"
//---------------------------------------------------------------------------
#include "math_include.h"
#include <algorithm>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* PCImageTool::PCImageToolMetaClass = NULL;

const MetaClass* PCImageTool::getInstanceClass() const {
   return getClass();
}

const MetaClass* PCImageTool::getClass() {
   if (PCImageToolMetaClass==NULL) {
      PCImageToolMetaClass = new MetaClass("PCImageTool", "no description.",
                                           NULL, sizeof(PCImageTool));
   }
   return PCImageToolMetaClass;
}

void PCImageTool::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   PCImageToolMetaClass->copyParentProperties();
}

void PCImageTool::shutdown() {
   delete PCImageToolMetaClass;
}


// ----


MetaClass* PCITImagePaint::PCITImagePaintMetaClass = NULL;

const MetaClass* PCITImagePaint::getInstanceClass() const {
   return getClass();
}

Object* PCITImagePaint::newInstance() {
   return new PCITImagePaint;
}

const MetaClass* PCITImagePaint::getClass() {
   if (PCITImagePaintMetaClass==NULL) {
      PCITImagePaintMetaClass = new MetaClass("PCITImagePaint", "no description.",
                                  newInstance, sizeof(PCITImagePaint));
   }
   return PCITImagePaintMetaClass;
}

void PCITImagePaint::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   PCITImagePaintMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT( PCITImagePaint )
	ADD_VECTOR4F_PROP(color,0);
	ADD_CARD32_PROP(brushRadius,0)
	ADD_BOOLEAN_PROP(softBrush,0)

}

void PCITImagePaint::shutdown() {
   delete PCITImagePaintMetaClass;
}


// ----


MetaClass* PCIPatchMatchingTest::PCIPatchMatchingTestMetaClass = NULL;

const MetaClass* PCIPatchMatchingTest::getInstanceClass() const {
   return getClass();
}

Object* PCIPatchMatchingTest::newInstance() {
   return new PCIPatchMatchingTest;
}

const MetaClass* PCIPatchMatchingTest::getClass() {
   if (PCIPatchMatchingTestMetaClass==NULL) {
      PCIPatchMatchingTestMetaClass = new MetaClass("PCIPatchMatchingTest", "no description.",
                                  newInstance, sizeof(PCIPatchMatchingTest));
   }
   return PCIPatchMatchingTestMetaClass;
}

static std::string descrEnum(card32 n) {
   if (n == 0) return "MODE_PATCH_SEARCH_NCC";
   if (n == 1) return "MODE_PATCH_SEARCH_SSD";
   if (n == 2) return "MODE_ALIGNED_PATCH_SEARCH_NCC";
   if (n == 3) return "MODE_ALIGNED_PATCH_SEARCH_SSD";
   if (n == 4) return "MODE_POWERSPECTRUM";
   if (n == 5) return "MODE_FFT";
   if (n == 6) return "MODE_INV_FFT";
   if (n == 7) return "MODE_ROT_TEST";
   if (n == 8) return "MODE_TEST_ALIGNED_PATCH";
   if (n == 9) return "FAST_PROC_TEST";
   return "unknown";
}


void PCIPatchMatchingTest::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   PCIPatchMatchingTestMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT( PCIPatchMatchingTest )
	ADD_CARD32_PROP_UPDATE_DESCR(mode,0,NULL,descrEnum, 10, NULL)
	ADD_STRING_PROP(sourceChannel,0)
	ADD_STRING_PROP(targetChannel,0)
	ADD_STRING_PROP(targetChannel2,0)
	ADD_STRING_PROP(invTargetChannel,0)
   ADD_INT32_PROP(patchSize,0)
	ADD_FLOAT32_PROP(angle,0)
	ADD_BOOLEAN_PROP(compositing,0)
	ADD_INT32_PROP(bestN,0)

}


void PCIPatchMatchingTest::shutdown() {
   delete PCIPatchMatchingTestMetaClass;
}


// ----



PCITImagePaint::PCITImagePaint() {
   targetImage = NULL;
   color = NULL_VECTOR4F;
   down = false;
   brushRadius = 10;
   softBrush = true;
}

void PCITImagePaint::setup(PointSet *targetImage, std::string channel) {
   this->targetImage = targetImage;
   this->channel = channel;
}


void PCITImagePaint::paint(float32 x, float32 y) {
   if (targetImage == NULL) return;
   const card32 width = targetImage->getWidth();
   const card32 height = targetImage->getHeight();
   int32 tx = x * width;
   int32 ty = y * height;
   if (tx >= (int32)width || tx < 0 || ty >= (int32)height || ty < 0) return;

   AAT aat = targetImage->getAAT(channel);
   if (aat == NULL_AAT) return;

   const VertexDescriptor *vd = targetImage->getDescr();
   VAD vad = vd->getAttribute(channel);

   Vector4d colord = makeVector4d(color[0], color[1], color[2], color[3]);

   for (int32 by = -(int32)brushRadius; by<(int32)brushRadius; by++) {
      for (int32 bx = -(int32)brushRadius; bx<(int32)brushRadius; bx++) {
         int32 ax = bx+tx;
         int32 ay = by+ty;
         if (bx*bx+by*by > (int32)brushRadius*(int32)brushRadius) continue;
         if (ax<0 || ax>=(int32)width || ay<0 || ay>=(int32)height) continue;
         if (softBrush) {
            Vector4d back = vad.getAnyAttrib((card8*)targetImage->getDataPointer(ax,ay), aat);
            float64 weight = 1.0 - sqrt((float32)(bx*bx+by*by))/(float32)brushRadius;
            Vector4d col = colord*weight + back * (1.0-weight);
            vad.setAnyAttrib((card8*)targetImage->getDataPointer(ax, ay), col, aat);
         } else {
            vad.setAnyAttrib((card8*)targetImage->getDataPointer(ax, ay), colord, aat);
         }

      }
   }
}


void PCITImagePaint::keyDown(GeneralKey key) {
   forget(key);
}

void PCITImagePaint::keyUp(GeneralKey key) {
   forget(key);
}

void PCITImagePaint::mouseDown(float32 x, float32 y, MouseButtons buttonsState, Modifiers modifiersState) {
   if (buttonsState.getLeft()) {
      down = true;
      paint(x, y);
   }
   forget(buttonsState);
   forget(modifiersState);
}

void PCITImagePaint::mouseMoved(float32 x, float32 y) {
   if (down) {
      paint(x, y);
   }
}

void PCITImagePaint::mouseUp(float32 x, float32 y, MouseButtons buttonsState, Modifiers modifiersState) {
   if (buttonsState.getLeft()) {
      down = false;
   }
   forget(x+y);
   forget(buttonsState);
   forget(modifiersState);
}

void PCITImagePaint::areaResize(float32 width, float32 height) {
   forget(width + height);
}


// --


PCIPatchMatchingTest::PCIPatchMatchingTest() {
   mode = 9;
   sourceChannel = "color";
   targetChannel = "test";
   targetChannel2 = "img";
   invTargetChannel = "color";
   patchSize = 5;
   targetImage = NULL;
   angle = 45;
   last_set = false;
   compositing = true;
   bestN = 20;
}

void PCIPatchMatchingTest::setup(PointSet *targetImage, std::string channel) {
   this->targetImage = targetImage;
   forget(channel);
}


void PCIPatchMatchingTest::keyDown(GeneralKey key) {forget(key);}
void PCIPatchMatchingTest::keyUp(GeneralKey key) {forget(key);}

void PCIPatchMatchingTest::mouseDown(float32 x, float32 y, MouseButtons buttonsState, Modifiers modifiersState) {
   if (targetImage == NULL) return;

   if (mode == MODE_ALIGNED_PATCH_SEARCH_NCC || mode == MODE_ALIGNED_PATCH_SEARCH_SSD) {doRGBProcessing(x,y); return;}

   card32 numComponentsSource = targetImage->getDescr()->getAttribute(sourceChannel).numComponents;
   card32 numComponentsTarget = targetImage->getDescr()->getAttribute(targetChannel).numComponents;
   card32 numComponents = min(numComponentsSource, numComponentsTarget);

   RegularDataF img, result, result2;

   for (card32 i=0; i<numComponents; i++) {
      if (mode != MODE_INV_FFT) {
         createRegDataFromPointSet(targetImage, sourceChannel, i, img);

         int32 width = img.getExtends(0);
         int32 height = img.getExtends(1);
         int32 intX = x * width; if (intX<0) intX=0; if (intX >= width) intX = width-1;
         int32 intY = y * height; if (intY<0) intY=0; if (intY >= height) intY = height-1;

         bool useResult2 = false;
         doAction(img, result, result2, intX, intY, useResult2);

         int32 offset = 0;
         if (mode == MODE_PATCH_SEARCH_NCC || mode == MODE_PATCH_SEARCH_SSD) {
            offset = patchSize;
         }
         writeRegDataToPointSet(targetImage, targetChannel, i, result, offset, offset);
         if (useResult2) {
            writeRegDataToPointSet(targetImage, targetChannel2, i, result2, offset, offset);
         }
      } else {
         createRegDataFromPointSet(targetImage, targetChannel,  i, result);
         createRegDataFromPointSet(targetImage, targetChannel2, i, result2);

         int32 width = result.getExtends(0);
         int32 height = result.getExtends(1);

         RegularDataF fftReal, fftImg;
         fourierTransform(&result, &result2, fftReal, fftImg, true);
         fftReal *= 1.0f/sqrt((float)(width*height));
         writeRegDataToPointSet(targetImage, invTargetChannel, i, fftReal);
      }
   }

   forget(buttonsState); forget(modifiersState);
}

void PCIPatchMatchingTest::mouseMoved(float32 x, float32 y) {forget(x+y);}
void PCIPatchMatchingTest::mouseUp(float32 x, float32 y, MouseButtons buttonsState, Modifiers modifiersState) {forget(x+y); forget(buttonsState); forget(modifiersState);}
void PCIPatchMatchingTest::areaResize(float32 width, float32 height) {forget(width); forget(height);}


void PCIPatchMatchingTest::glDrawTool(GLContext *context) {
   if (mode == MODE_TEST_ALIGNED_PATCH) {
      glPushAttrib(GL_ALL_ATTRIB_BITS);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
      glColor3f(1,1,0);
      glDisable(GL_LIGHTING);
      glBegin(GL_LINES);
      glVertex2fv(NULL_VECTOR2F.data());
      glVertex2fv(last_u.data());
      glVertex2fv(NULL_VECTOR2F.data());
      glVertex2fv(last_v.data());

      float32 p = patchSize;
      float32 pi = 1.0f/p;
      Vector2f ps = makeVector2f(p,p);
      glVertex2fv(((last_mean - ps - makeVector2f(1,0)*0.1f)*pi).data());
      glVertex2fv(((last_mean - ps + makeVector2f(1,0)*0.1f)*pi).data());
      glVertex2fv(((last_mean - ps - makeVector2f(0,1)*0.1f)*pi).data());
      glVertex2fv(((last_mean - ps + makeVector2f(0,1)*0.1f)*pi).data());
      glEnd();
      glPopAttrib();
   }
   forget(context);
}


void PCIPatchMatchingTest::doAction(const RegularDataF &img, RegularDataF &result, RegularDataF &result2,
                                    int32 x, int32 y, bool &useResult2)
{
   useResult2 = false;
   last_set = false;

   int32 width = img.getExtends(0);
   int32 height = img.getExtends(1);
   if (width < 2*patchSize + 1) return;
   if (height < 2*patchSize + 1) return;
   int32 patchStartX = x - patchSize;
   if (patchStartX < 0) patchStartX = 0;
   if (patchStartX+2*patchSize+1 >= width) patchStartX = width-2-2*patchSize;
   int32 patchStartY = y - patchSize; if (patchStartY < 0) patchStartY = 0;
   if (patchStartY < 0) patchStartY = 0;
   if (patchStartY+2*patchSize+1 >= height) patchStartY = height-2-2*patchSize;

   DynamicVector<int32> lowerPos(2), regionExtends(2);
   lowerPos[0] = patchStartX;
   lowerPos[1] = patchStartY;
   regionExtends[0] = 2*patchSize+1;
   regionExtends[1] = 2*patchSize+1;

   RegularDataF patch;
   img.getSubDataSet(lowerPos, regionExtends, patch);

   result.setExtends<float32, 2>(makeVector2f(width-2*patchSize, height-2*patchSize));

   switch (mode) {
    case MODE_PATCH_SEARCH_NCC: {
      patch.normalizeMeanNorm();

      float32 *resultPos = result.dataPtr();

      RegularDataF other;
      for (int32 ypos = patchSize; ypos < height - patchSize; ypos++) {
         for (int32 xpos = patchSize; xpos < width - patchSize; xpos++) {
            lowerPos[0] = xpos-patchSize; lowerPos[1] = ypos-patchSize;
            img.getSubDataSet(lowerPos, regionExtends, other);
            other.normalizeMeanNorm();
            float32 c = patch * other;
            *resultPos++ = c;
         }
      }
      break;
    }
    case MODE_TEST_ALIGNED_PATCH: {
      DVectorF mean(2);
      DMatrixF rot(2,2);
      computePCACoordinateSystem(patch, mean, rot);
      debugOutput << "mean: " << mean[0] << " | " << mean[1] << "\n";
      debugOutput << "rot0: " << rot[0][0] << " | " << rot[0][1] << "\n";
      debugOutput << "rot1: " << rot[1][0] << " | " << rot[1][1] << "\n";
      debugOutput.cr();
      last_set = true;
      last_mean[0] = mean[0];
      last_mean[1] = mean[1];
      last_u[0] = rot[0][0];
      last_u[1] = rot[0][1];
      last_v[0] = rot[1][0];
      last_v[1] = rot[1][1];
      break;
    }
    case MODE_ALIGNED_PATCH_SEARCH_NCC: {
      DVectorF mean;
      AffineTransform<float32> transf;
      transf.center.setDim(2);
      transf.center[0] = patchSize;
      transf.center[1] = patchSize;
      computePCACoordinateSystem(patch, mean, transf.transform);
      RegularDataF rotPatch;
      affineTransform(patch, rotPatch, transf);

      sphereClip(rotPatch);
      rotPatch.normalizeMeanNorm();
      result = rotPatch;
                     /*
      float32 *resultPos = result.dataPtr();

      RegularDataF other;
      for (int32 ypos = patchSize; ypos < height - patchSize; ypos++) {
         for (int32 xpos = patchSize; xpos < width - patchSize; xpos++) {
            lowerPos[0] = xpos-patchSize; lowerPos[1] = ypos-patchSize;
            img.getSubDataSet(lowerPos, regionExtends, other);
            other.normalizeMeanNorm();
            float32 c = patch * other;
            *resultPos++ = c;
         }
      } */
     break;
    }
    case MODE_PATCH_SEARCH_SSD: {
      float32 *resultPos = result.dataPtr();

      RegularDataF other;
      for (int32 ypos = patchSize; ypos < height - patchSize; ypos++) {
         for (int32 xpos = patchSize; xpos < width - patchSize; xpos++) {
            lowerPos[0] = xpos-patchSize; lowerPos[1] = ypos-patchSize;
            img.getSubDataSet(lowerPos, regionExtends, other);

            RegularData<float32> diff = patch - other;

            float32 c = 1 - (diff*diff) / (patchSize*patchSize);
            *resultPos++ = c;
         }
      }
      break;
    }
    case MODE_POWERSPECTRUM: {
      RegularDataF fftReal, fftImg;
      fourierTransform(&img, (const RegularDataF*)NULL, fftReal, fftImg);
      // powerspectrum
      fftReal *= 1.0f/sqrt((float)(width*height));
      fftImg *= 1.0f/sqrt((float)(width*height));
      result = fftReal.componentProduct(fftReal) + fftImg.componentProduct(fftImg);
      result.squareRootOfComponents();
      break;
    }
    case MODE_FFT: {
      RegularDataF fftReal, fftImg;
      fourierTransform(&img, (const RegularDataF*)NULL, fftReal, fftImg);
      fftReal *= 1.0f/sqrt((float)(width*height));
      fftImg *= 1.0f/sqrt((float)(width*height));
      result = fftReal;
      result2 = fftImg;
      useResult2 = true;
      break;
    }
    case MODE_ROT_TEST: {
      AffineTransform<float32> rot;
      rot.center.setDim(2);
      rot.center[0] = patch.getExtends(0)/2;
      rot.center[1] = patch.getExtends(1)/2;;
      rot.transform.setDimension(2,2);
      float32 a = angle / 180.0f * M_PI;
      rot.transform[0][0] =  cos(a); rot.transform[1][0] = sin(a);
      rot.transform[0][1] = -sin(a); rot.transform[1][1] = cos(a);
      affineTransform(patch, result, rot);
      break;
    }
    case FAST_PROC_TEST: {
      result.setExtends(patch.getExtends());
      result.clear(0);
      Vector2f center;
      Matrix2f rot;
      // center[0] = patch.getExtends(0)/2;
      // center[1] = patch.getExtends(1)/2;;

      // float32 a = angle / 180.0f * M_PI;
      // rot[0][0] =  cos(a); rot[1][0] = sin(a);
      // rot[0][1] = -sin(a); rot[1][1] = cos(a);
      computePCACoordinateSystem2DF(patch, center, rot);
      rot = rot.transpose();
      affineTransformLinearInterp2DF(patch, result, rot, center);
      cubeSphereClip2DF(result);
      break;
    }
   }
}


static void alignPatches(card32 numComponents, int32 patchSize, const RegularDataF *patches,
                         RegularDataF *rotPatches, RegularDataF *rotPatchesOtherPossibility = NULL)
{
   RegularDataF patchGeoMean = patches[0];
   patchGeoMean.squareOfComponents();
   for (card32 i=1; i<numComponents; i++) {
      RegularDataF t = patches[i];
      t.squareOfComponents();
      patchGeoMean += t;
   }
   patchGeoMean.squareRootOfComponents();

   DVectorF mean;
   AffineTransform<float32> transf;
   transf.center.setDim(2);
   transf.center[0] = patchSize;
   transf.center[1] = patchSize;
   computePCACoordinateSystem(patchGeoMean, mean, transf.transform);
   for (card32 i=0; i<numComponents; i++) {
      affineTransform(patches[i], rotPatches[i], transf);
      sphereClip(rotPatches[i]);
   }
   if (rotPatchesOtherPossibility != NULL) {
      transf.transform *= -1;
      for (card32 i=0; i<numComponents; i++) {
         affineTransform(patches[i], rotPatchesOtherPossibility[i], transf);
         sphereClip(rotPatches[i]);
      }
   }
}

struct APoint {
   Vector2i pos;
   float32 v;
   bool operator<(const APoint &other) const {
      return v < other.v;
   }
};

static float32 SSD(const RegularDataF &a, const RegularDataF &b, float32 patchSize) {
   RegularDataF diff = a - b;
   return (diff*diff) / (patchSize*patchSize);
}

void PCIPatchMatchingTest::doRGBProcessing(float32 xFloat, float32 yFloat) {
   x4Assert(MODE_ALIGNED_PATCH_SEARCH_NCC || MODE_ALIGNED_PATCH_SEARCH_SSD);

   const card32 numComponentsSource = targetImage->getDescr()->getAttribute(sourceChannel).numComponents;
   const card32 numComponentsTarget = targetImage->getDescr()->getAttribute(targetChannel).numComponents;
   const card32 numComponents = min(numComponentsSource, numComponentsTarget);

   RegularDataF *img = new RegularDataF[numComponents];
   RegularDataF *patch = new RegularDataF[numComponents];
   RegularDataF *result = new RegularDataF[numComponents];
   RegularDataF patchGeoMean;

   for (card32 i=0; i<numComponents; i++) {
      createRegDataFromPointSet(targetImage, sourceChannel, i, img[i]);
   }

   int32 width = img[0].getExtends(0);
   int32 height = img[0].getExtends(1);
   int32 intX = xFloat * width;  if (intX<0) intX=0; if (intX >= width) intX = width-1;
   int32 intY = yFloat * height; if (intY<0) intY=0; if (intY >= height) intY = height-1;

   if (width < 2*patchSize + 1) return;
   if (height < 2*patchSize + 1) return;

   int32 patchStartX = intX - patchSize;
   if (patchStartX < 0) patchStartX = 0;
   if (patchStartX+2*patchSize+1 >= width) patchStartX = width-2-2*patchSize;
   int32 patchStartY = intY - patchSize; if (patchStartY < 0) patchStartY = 0;
   if (patchStartY < 0) patchStartY = 0;
   if (patchStartY+2*patchSize+1 >= height) patchStartY = height-2-2*patchSize;

   DynamicVector<int32> lowerPos(2), regionExtends(2);
   lowerPos[0] = patchStartX;
   lowerPos[1] = patchStartY;
   regionExtends[0] = 2*patchSize+1;
   regionExtends[1] = 2*patchSize+1;

   for (card32 i=0; i<numComponents; i++) {
      img[i].getSubDataSet(lowerPos, regionExtends, patch[i]);
      result[i].setExtends<float32, 2>(makeVector2f(width-2*patchSize, height-2*patchSize));
   }

   RegularDataF *rotPatch = new RegularDataF[numComponents];
   RegularDataF *rotPatch2 = new RegularDataF[numComponents];
   alignPatches(numComponents, patchSize, patch, rotPatch, rotPatch2);
   if (mode == MODE_ALIGNED_PATCH_SEARCH_NCC) {
      for (card32 i=0; i<numComponents; i++) {
         rotPatch[i].normalizeMeanNorm();
         rotPatch2[i].normalizeMeanNorm();
      }
   }

   float32 **resultPos = new float32*[numComponents];
   for (card32 i=0; i<numComponents; i++) {
      resultPos[i] = result[i].dataPtr();
   }

   RegularDataF *other = new RegularDataF[numComponents];
   RegularDataF *rotOther = new RegularDataF[numComponents];
   for (int32 ypos = patchSize; ypos < height - patchSize; ypos++) {
      for (int32 xpos = patchSize; xpos < width - patchSize; xpos++) {

         for (card32 i=0; i<numComponents; i++) {
            lowerPos[0] = xpos-patchSize; lowerPos[1] = ypos-patchSize;
            img[i].getSubDataSet(lowerPos, regionExtends, other[i]);
         }

         alignPatches(numComponents, patchSize, other, rotOther);

         float meanDiff;
         if (mode == MODE_ALIGNED_PATCH_SEARCH_NCC) {
            meanDiff = 1;
         } else {
            meanDiff = 0;
         }

         for (card32 i=0; i<numComponents; i++) {
            if (mode == MODE_ALIGNED_PATCH_SEARCH_NCC) {
               rotOther[i].normalizeMeanNorm();
               meanDiff *= max(rotOther[i] * rotPatch[i], rotOther[i] * rotPatch2[i]);
            } else {
               meanDiff += min(SSD(rotOther[i], rotPatch[i], patchSize), SSD(rotOther[i], rotPatch2[i], patchSize));
            }
         }

         if (mode == MODE_ALIGNED_PATCH_SEARCH_SSD) {
            meanDiff = 1.0f - meanDiff;
         }

         for (card32 i=0; i<numComponents; i++) {
            *(resultPos[i])++ = meanDiff;
         }
      }
   }

   if (compositing) {
      const int w = result[0].getExtends(0);
      const int h = result[0].getExtends(1);
      std::vector<APoint> pts;
      int n = w*h;
      pts.resize(n);
      float32 *dataPtr = result[0].dataPtr();
      for (int32 y=0; y<h; y++) {
         for (int32 x=0; x<w; x++) {
            pts[x+y*w].v = *dataPtr++;
            pts[x+y*w].pos = makeVector2i(x,y);
         }
      }

      sort(pts.begin(), pts.end());

      for (card32 i=0; i<numComponents; i++) {
         result[i] = img[i];
         int start = max(n-bestN, 0);
         for (int j=start; j<n; j++) {
            Vector2i pos = pts[j].pos;
            result[i][pos[0]+patchSize + width*(pos[1]+patchSize)] = 2.0f;
         }
      }
   }


   for (card32 i=0; i<numComponents; i++) {
      writeRegDataToPointSet(targetImage, targetChannel, i, result[i], compositing?0:patchSize, compositing?0:patchSize);
   }


   delete[] patch;
   delete[] img;
   delete[] result;
   delete[] rotPatch;
   delete[] rotPatch2;
   delete[] other;
   delete[] rotOther;
   delete[] resultPos;

}



