//---------------------------------------------------------------------------
#ifndef PCImageToolsH
#define PCImageToolsH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "MouseController.h"
#include "KeyboardController.h"
#include "LinearAlgebra.h"
#include "RegularData.h"
//---------------------------------------------------------------------------
#include <string>
//---------------------------------------------------------------------------


namespace X4 {
   
class PointSet;
class GLContext;

/// paint on a channel of a 2d point set. mouse parameters are in image coordinates, scaled to [0..width-1] x [0..height-1].
class PCImageTool : public Persistent, public MouseControllerFloat, public KeyboardController {
 private:
   static MetaClass *PCImageToolMetaClass;

 public:

   virtual void setup(PointSet *targetImage, std::string channel) = 0;
   virtual void glDrawTool(GLContext *context) {}

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

};



class PCITImagePaint : public PCImageTool {
 private:
   static MetaClass *PCITImagePaintMetaClass;
   static Object* newInstance();

   // internal
   PointSet *targetImage;
   std::string channel;
   bool down;

   // persistent properties
   Vector4f color;
   card32 brushRadius;
   bool softBrush;

   void paint(float32 x, float32 y);

 public:
   PCITImagePaint();

   virtual void setup(PointSet *targetImage, std::string channel);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void mouseDown(float32 x, float32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(float32 x, float32 y);
   virtual void mouseUp(float32 x, float32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(float32 width, float32 height);
};


class PCIPatchMatchingTest : public PCImageTool {
 private:
   static MetaClass *PCIPatchMatchingTestMetaClass;
   static Object* newInstance();

   static const card32 MODE_PATCH_SEARCH_NCC = 0;
   static const card32 MODE_PATCH_SEARCH_SSD = 1;
   static const card32 MODE_ALIGNED_PATCH_SEARCH_NCC = 2;
   static const card32 MODE_ALIGNED_PATCH_SEARCH_SSD = 3;
   static const card32 MODE_POWERSPECTRUM = 4;
   static const card32 MODE_FFT = 5;
   static const card32 MODE_INV_FFT = 6;
   static const card32 MODE_ROT_TEST = 7;
   static const card32 MODE_TEST_ALIGNED_PATCH = 8;
   static const card32 FAST_PROC_TEST = 9;

   card32 mode;
   std::string sourceChannel, targetChannel, targetChannel2, invTargetChannel;
   int32 patchSize;
   float32 angle;
   PointSet *targetImage;
   bool compositing;
   int32 bestN;

   // --- intern
   Vector2f last_u, last_v, last_mean;
   bool last_set;

   void doAction(const RegularDataF &img, RegularDataF &result, RegularDataF &result2, int32 x, int32 y, bool &useResult2);
   void doRGBProcessing(float32 xFloat, float32 yFloat);

 public:
   PCIPatchMatchingTest();

   virtual void setup(PointSet *targetImage, std::string channel);
   virtual void glDrawTool(GLContext *context);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void mouseDown(float32 x, float32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(float32 x, float32 y);
   virtual void mouseUp(float32 x, float32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(float32 width, float32 height);
};


}


#endif
