//---------------------------------------------------------------------------
#ifndef S2kitH
#define S2kitH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
//---------------------------------------------------------------------------


namespace X4 {

/// do spherical harmonic transform toolkit
/// WARNING: S2kit is not thread safe, due to shared data used by fftw.
class X3EDITOR2_API S2kit{

 public:
  ///test and example function
  void test();

  /// do spherical harmonic transform  bw is  sampling rate of s2 sphere
  /// -> size of array data* is 4*bw*bw and array coeffs* is bw*bw
  ///dont't use bw > 100 -> this is numerical instable
  ///inverse=true does inverse transform
  void doSHT(double *dataReal, double *dataImg, double *coeffsReal, double *coeffsImg, card32 bw, bool inverse);

  /// First test of faster memory-table versions (seems to work very well :)
  void doForwardMemSHT(double *dataReal, double *dataImg, double *coeffsReal, double *coeffsImg, card32 bw);
  void doInverseMemSHT(double *dataReal, double *dataImg, double *coeffsReal, double *coeffsImg, card32 bw);

  //Tools

  PointCloud* createPointCloudFromS2File(const char* file);
  PointCloud* createPointCloudFromS2DataFile(const char* file);
  PointCloud* createPointCloudFromS2Data(double *rdata, card32 bw);

  /// Returns the index of the coefficient of degree l and order m in the coeffsReal/coeffsImg array returned by doSHT().
  int seanindex(int m, int l, int bw);

  /// Computes the frequency spectrum. dataReal and dataImg must have size 4*bw*bw, and spectrum will have size bw.
  void computeSpectrum(const DVectorD &dataReal, const DVectorD &dataImg, DVectorD *spectrum, card32 bw);

};




} //ns
#endif
