//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SurfaceElementEditingEventDispatcher.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SurfaceElementsControllers.h"
#include "SurfaceElementsViewers.h"
#include "SurfaceElementsInteractionTools.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_ABSTRACT_CLASS( SurfaceElementEditingEventDispatcher ,0)
{
}

SurfaceElementEditingEventDispatcher::SurfaceElementEditingEventDispatcher() {
   mouseDownLeft = false;
   mouseDownRight = false;
   leftClickHandled = false;
   rightClickHandled = false;
}

SurfaceElementEditingEventDispatcher::~SurfaceElementEditingEventDispatcher() {}

void SurfaceElementEditingEventDispatcher::assign(const Object* obj, X4CopyContext *context) {
   copyObjectProperties(obj, this);
}




bool X4::SurfaceElementEditingEventDispatcher::mouseDown( SurfaceElementsInteractionContext *context, int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState )
{
   bool handled = false;
   bool leftClicked = false;
   bool rightClicked = false;
   if (buttonsState.getLeft()) {
      mouseDownLeft  = true;
      for (size_t i=0; i<controllers.size(); i++) {
         bool h = controllers[i]->leftClicked(x, y, modifiersState, context);
         handled |= h;
         if (h) leftClickHandled = true;
         leftClicked |= h;
      }
      if (!handled && surfacePointInfo.hit) {
         handled |= context->currentTool->tryFreeClickPoint(surfacePointInfo.point);
      }
   }
   if (buttonsState.getRight()) {
      mouseDownRight = true;
      for (size_t i=0; i<controllers.size(); i++) {
         bool h = controllers[i]->rightClicked(x, y, modifiersState, context);
         handled |= h;
         if (h) rightClickHandled = true;
         rightClicked |= h;
      }
      if (!handled) context->currentTool->rightClick();
   }
   return handled;
}

bool X4::SurfaceElementEditingEventDispatcher::mouseMoved(SurfaceElementsInteractionContext *context, int32 x, int32 y)
{
   Modifiers mods;
   bool handled = false;
   bool hovered = false;
   bool dragged = false;
   for (size_t i=0; i<controllers.size(); i++) {
      if (mouseDownLeft) {
         bool h = controllers[i]->dragged(x,y,context);
         dragged |= h;
         handled |= leftClickHandled;
      } else {
         bool h = controllers[i]->hover(x,y,context);
         handled |= h;
         hovered |= h;
      }
   }
   handled |= context->currentTool->tryFreeHoverPoint(surfacePointInfo.point, surfacePointInfo.hit);
   return handled;
}

bool X4::SurfaceElementEditingEventDispatcher::mouseUp(SurfaceElementsInteractionContext *context, int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState)
{
   bool handled = false;
   if (buttonsState.getLeft())  {
      mouseDownLeft  = false;
      handled = leftClickHandled;
      leftClickHandled = false;
   }
   if (buttonsState.getRight()) {
      mouseDownRight = false;
      handled = rightClickHandled;
      rightClickHandled = false;
   }
   return handled;
}

bool X4::SurfaceElementEditingEventDispatcher::mouseWheelRotated( SurfaceElementsInteractionContext *context, int32 rotatedDelta, Modifiers modifiersState )
{
   return false;
}

bool X4::SurfaceElementEditingEventDispatcher::keyDown( SurfaceElementsInteractionContext *context, GeneralKey key )
{
   return false;
}

bool X4::SurfaceElementEditingEventDispatcher::keyUp( SurfaceElementsInteractionContext *context, GeneralKey key )
{
   return false;
}

void SurfaceElementEditingEventDispatcher::updateControllers(SurfaceElementViewer *viewerHierarchy)
{
   controllers.clear();
   updateControllersRek(viewerHierarchy);
}

void SurfaceElementEditingEventDispatcher::updateControllersRek(SurfaceElementViewer *viewerHierarchy)
{
   if (viewerHierarchy) {
      if (viewerHierarchy->getController()) {
         controllers.push_back((viewerHierarchy->getController()));
      }
      for (mpcard i=0; i<viewerHierarchy->getNumChildViewers(); i++) {
         updateControllersRek(viewerHierarchy->getChildViewer(i));
      }
   }
}
