//---------------------------------------------------------------------------
#ifndef SurfaceElementEditingEventDispatcherH
#define SurfaceElementEditingEventDispatcherH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "MouseButtons.h"
#include "Modifiers.h"
#include "GeneralKey.h"
#include "SurfaceElementsInteractionContext.h"
//---------------------------------------------------------------------------

namespace X4 {

class SurfaceElementController;
class SurfaceElementViewer;
class Scene;


/**
  dispatches events among controllers and handles communication with tools
*/
class X3EDITOR2_API SurfaceElementEditingEventDispatcher : public Object {
	X4_ABSTRACT_CLASS( SurfaceElementEditingEventDispatcher )
 public:
   struct SurfacePointInfo {
      bool hit;
      int x;
      int y;
      Vector3f point;
   };

 private:
   bool mouseDownLeft;
   bool mouseDownRight;
   bool leftClickHandled;
   bool rightClickHandled;
   SurfacePointInfo surfacePointInfo;
   std::vector<SurfaceElementController*> controllers;
   
   void updateControllersRek(SurfaceElementViewer *viewerHierarchy);
   
 public:
	SurfaceElementEditingEventDispatcher();
	
	/// call before using; update controller hierarchy
   void updateControllers(SurfaceElementViewer *viewerHierarchy);
			
   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   // --- external interface
   // return values: event handled?
   virtual void updateSurfacePointInfo(const SurfacePointInfo &si) {surfacePointInfo=si;}
   virtual bool mouseDown(SurfaceElementsInteractionContext *context, int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual bool mouseMoved(SurfaceElementsInteractionContext *context, int32 x, int32 y);
   virtual bool mouseUp(SurfaceElementsInteractionContext *context, int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual bool mouseWheelRotated(SurfaceElementsInteractionContext *context, int32 rotatedDelta, Modifiers modifiersState);

   virtual bool keyDown(SurfaceElementsInteractionContext *context, GeneralKey key);
   virtual bool keyUp(SurfaceElementsInteractionContext *context, GeneralKey key);

	~SurfaceElementEditingEventDispatcher();
};


}

#endif