//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SurfaceElementStructureAttachment.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SurfaceElementsViewers.h"
#include "SurfaceElements.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


IMPLEMENT_X4_CLASS( SurfaceElementStructureAttachment ,0)
{
	BEGIN_CLASS_INIT( SurfaceElementStructureAttachment );
	ADD_OBJECT_LIST_PROP(points, 0, SurfaceElement::getClass())
   ADD_OBJECT_LIST_PROP(regions, 0, SurfaceElement::getClass())
   ADD_OBJECT_PROP(context, 0, SEVContext::getClass(), true)
   SET_STREAM_PROPERTY(false)
}

SurfaceElementStructureAttachment::SurfaceElementStructureAttachment() {
   viewers = NULL;
   context = NULL;
}

SurfaceElementStructureAttachment::~SurfaceElementStructureAttachment() {
   delete viewers;
   delete context;
}

void SurfaceElementStructureAttachment::assign(const Object* obj, X4CopyContext *context) {
   copyObjectProperties(obj, this);
}



std::string X4::SurfaceElementStructureAttachment::toString()
{
   std::string result = "SurfaceElementStructureAttachment content\n";
   result += "points (centers):\n";
   for (size_t i=0; i<m_points.size(); i++) {
      result += m_points[i]->getCenter().toString();
      result += " ";
   }
   result += "\n";
   result += "regions (centers):\n";
   for (size_t i=0; i<m_points.size(); i++) {
      result += m_regions[i]->getCenter().toString();
      result += " ";
   }
   result += "\n";
   return result;
}

void X4::SurfaceElementStructureAttachment::renderGL( SceneObject *obj )
{
   if (m_points.empty() && m_regions.empty()) return;
   if (!viewers) {
      SurfaceElementViewerList *list = new SurfaceElementViewerList();
      for (size_t i=0; i<m_points.size(); i++) {
         list->addViewer(m_points[i]->createDefaultViewer(false));
      }
      for (size_t i=0; i<m_regions.size(); i++) {
         list->addViewer(m_regions[i]->createDefaultViewer(false));
      }
      viewers = list;
   }
   if (!context) {
      context = new SEVContext();
   }
   context->pushRenderState();
   viewers->drawGL(context);
   context->popRenderState();
}

void X4::SurfaceElementStructureAttachment::deleteRegions( SurfaceElement* region )
{
   for (size_t i=0; i<m_regions.size(); i++) {
      if (m_regions[i] == region) {
         std::vector<SurfaceElement*>::iterator pos = m_regions.begin();
         pos += i;
         m_regions.erase(pos);
         return;
      }
   }
   x4Assert(false);
}
