//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SurfaceElementsControllers.h"
#include "SurfaceElementsViewers.h"
#include "SurfaceElements.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "Scene.h"
#include "surfaceelementsinteractiontools.h"
#include "surfaceelementsinteractioncontext.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_ABSTRACT_CLASS( SurfaceElementController ,0)
{
	BEGIN_CLASS_INIT( SurfaceElementController );
}

SurfaceElementController::SurfaceElementController() {}

SurfaceElementController::~SurfaceElementController() {}

void SurfaceElementController::assign(const Object* obj, X4CopyContext *context) {
   copyObjectProperties(obj, this);
}



SurfaceElementViewer *SurfaceElementController::setup(SurfaceElement *model)
{
   this->model = model;
   viewer = createViewer(model);
   viewer->setup(model, this);
   return viewer;
}




// ----

IMPLEMENT_X4_CLASS( SurfaceElementControllerSelection , 0 )
{
   BEGIN_CLASS_INIT( SurfaceElementControllerSelection );
}

SurfaceElementControllerSelection::SurfaceElementControllerSelection()
{

}

SurfaceElementControllerSelection::~SurfaceElementControllerSelection()
{

}

bool SurfaceElementControllerSelection::hover(int32 x, int32 y, SurfaceElementsInteractionContext *context)
{
   PCISurfaceElementInteractionTool::ClickInfo info;
   bool b = getModel()->mouseInRange(x,y,context,&info.dist,&info.depth, !getViewer()->getLightWeightVis());
   if (b) {
      //info.position = getModel()->getCenter();
      context->currentTool->tryHoverElement(this, getModel(), info);
   }
   getViewer()->setHoverState(false);
   return b;
}

SurfaceElementViewer *SurfaceElementControllerSelection::createViewer(SurfaceElement *model)
{
   SurfaceElementViewerPointAsSphere *result = new SurfaceElementViewerPointAsSphere();
   result->setup(model, this);
   return result;
}


void SurfaceElementControllerSelection::doHover()
{
   getViewer()->setHoverState(true);   
}

bool X4::SurfaceElementControllerSelection::leftClicked( int32 x, int32 y, Modifiers modifiersState, SurfaceElementsInteractionContext *context )
{
   clickModifiers = modifiersState;
   PCISurfaceElementInteractionTool::ClickInfo info;
   bool b = getModel()->mouseInRange(x,y,context,&info.dist,&info.depth, !getViewer()->getLightWeightVis());
   if (b) {
      //info.position = getModel()->getCenter();
      context->currentTool->tryClickElement(this, getModel(), info);
   }
   return b;
}

void X4::SurfaceElementControllerSelection::doLeftClick()
{
   if (clickModifiers.getCtrl()) {
      getViewer()->setSelected(!getViewer()->getSelected());
   } else {
      getViewer()->setSelected(true);   
   }
}

bool X4::SurfaceElementControllerSelection::dragged( int32 x, int32 y, SurfaceElementsInteractionContext *context ) {
   return hover(x,y,context);
}



// ----

IMPLEMENT_X4_CLASS( SurfaceElementControllerPolygonVertices , 0 )
{
   BEGIN_CLASS_INIT( SurfaceElementControllerPolygonVertices );
}

SurfaceElementControllerPolygonVertices::SurfaceElementControllerPolygonVertices() {}

SurfaceElementControllerPolygonVertices::~SurfaceElementControllerPolygonVertices() {}

SurfaceElementViewer *SurfaceElementControllerPolygonVertices::createViewer(SurfaceElement *model)
{
   SurfaceElementPolygon *poly = dynamic_cast<SurfaceElementPolygon*>(model);
   x4Assert(poly);
   if (!poly) return NULL;
   SurfaceElementViewerList *polygonViewerList = new SurfaceElementViewerList();
   // add polygon viewer (does not own model, as this is a secondary viewer, primary is the list)
   SurfaceElementViewer *polyViewer = model->createDefaultViewer(false);
   polygonViewerList->addViewer(polyViewer);
   // add vertex controllers
   for (size_t i=0; i<poly->getPoints().size(); i++) {
      SurfaceElementPolygonPoint *vertex = new SurfaceElementPolygonPoint();
      vertex->setup(poly->getPoints()[i], poly);
      vertex->setViewerHint(polyViewer);
      SurfaceElementControllerSelection *vertexController = new SurfaceElementControllerSelection();
      SurfaceElementViewerPointAsSphere *newVertView = dynamic_cast<SurfaceElementViewerPointAsSphere*>(vertexController->setup(vertex));
      newVertView->setPlainStyleScale(0.7f);
      newVertView->setOwnerOfModel(true);
      polygonViewerList->addViewer(newVertView);
   }
   return polygonViewerList;
}
