//---------------------------------------------------------------------------
#ifndef SurfaceElementControllersH
#define SurfaceElementControllersH
//---------------------------------------------------------------------------
#include "x3editor2.h"
//---------------------------------------------------------------------------
#include "AttachedData.h"
#include "Modifiers.h"
//---------------------------------------------------------------------------

namespace X4 {

class SurfaceElement;
class SurfaceElementViewer;
class SurfaceElementsInteractionContext;

/**
  Interaction controller for a surface element
*/
class X3EDITOR2_API SurfaceElementController : public Object {
	X4_ABSTRACT_CLASS( SurfaceElementController )
 private:
   SurfaceElement *model;
   SurfaceElementViewer *viewer;
   
 protected:
   virtual SurfaceElementViewer *createViewer(SurfaceElement *model) = 0;
   
 public:
	SurfaceElementController();
	
	/// creates viewers for surface element and stores all the links internally
	/// the controller is also attached to the root viewer element that is returned
	/// descendants need to override "createViewer" (at least)
   virtual SurfaceElementViewer *setup(SurfaceElement *model);
		
   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   SurfaceElement *getModel() {return model;}
   SurfaceElementViewer *getViewer() {return viewer;}

   // --- primary events
   /// clicked on element with left mouse button
   virtual bool leftClicked(int32 x, int32 y, Modifiers modifiersState, SurfaceElementsInteractionContext *context) {return false;}
   /// clicked on element with right mouse button
   virtual bool rightClicked(int32 x, int32 y, Modifiers modifiersState, SurfaceElementsInteractionContext *context) {return false;}
   /// mouse over element
   virtual bool hover(int32 x, int32 y, SurfaceElementsInteractionContext *context) {return false;}
   /// element being dragged
   virtual bool dragged(int32 x, int32 y, SurfaceElementsInteractionContext *context) {return false;}

   // --- tool feedback
   /// feedback from tool: actually selected as closest target
   virtual void doHover() {}
   /// feedback from tool: actually selected as closest target
   virtual void doLeftClick() {}

	~SurfaceElementController();
};


/**
   Simple selection controller
*/
class X3EDITOR2_API SurfaceElementControllerSelection : public SurfaceElementController {
   X4_CLASS( SurfaceElementControllerSelection )

 private:
   Modifiers clickModifiers;
   
 protected:
   virtual SurfaceElementViewer *createViewer(SurfaceElement *model);

 public:
   SurfaceElementControllerSelection();

   virtual bool hover(int32 x, int32 y, SurfaceElementsInteractionContext *context);
   virtual bool dragged(int32 x, int32 y, SurfaceElementsInteractionContext *context);
   virtual void doHover();
   virtual bool leftClicked(int32 x, int32 y, Modifiers modifiersState, SurfaceElementsInteractionContext *context);
   virtual void doLeftClick();

   ~SurfaceElementControllerSelection();
};


/**
   Simple selection applied to polygon vertices
*/
class X3EDITOR2_API SurfaceElementControllerPolygonVertices : public SurfaceElementController {
   X4_CLASS( SurfaceElementControllerPolygonVertices )

 private:
   Modifiers clickModifiers;
   
 protected:
   virtual SurfaceElementViewer *createViewer(SurfaceElement *model);

 public:
   SurfaceElementControllerPolygonVertices();

   ~SurfaceElementControllerPolygonVertices();
};



}

#endif