//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SurfaceElementsInteractionContext.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "surfaceelementsinteractiontools.h"
#include "surfaceelementsviewers.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_ABSTRACT_CLASS( SurfaceElementsInteractionContext ,0)
{
	BEGIN_CLASS_INIT( SurfaceElementsInteractionContext );
   ADD_OBJECT_PROP(scene, 0, Scene::getClass(), false)
   ADD_MATRIX4F_PROP(projectionMatrix, 0)
   ADD_OBJECT_PROP(currentTool, 0, PCISurfaceElementInteractionTool::getClass(), false)
   ADD_OBJECT_PROP(viewerHierarchy, 0, SurfaceElementViewer::getClass(), false)
   ADD_FLOAT32_PROP(pickDistance, 0)
   ADD_FLOAT32_PROP(worldSpaceSnap, 0)
}

SurfaceElementsInteractionContext::SurfaceElementsInteractionContext() {
   scene = NULL;
   projectionMatrix = IDENTITY4F;
   currentTool = NULL;
   viewerHierarchy = NULL;
   width = 0;
   height = 0;
   pickDistance = 5;
   worldSpaceSnap = 0.01f;
}

SurfaceElementsInteractionContext::~SurfaceElementsInteractionContext() {}

void SurfaceElementsInteractionContext::assign(const Object* obj, X4CopyContext *context) {
   copyObjectProperties(obj, this);
}



void SurfaceElementsInteractionContext::rayQuery(Vector3f pos, int32 x, int32 y, float32 &depth, float32 &screenDist) 
{
   Vector3f proj = projectiveTransformVector3f(projectionMatrix, pos);
   Vector2f click;
   click[0] = ((float32)x)/(width*0.5f)-1.0f;
   click[1] = ((float32)y)/(height*0.5f)-1.0f;
   Vector2f proj2 = makeVector2f(proj[0], proj[1]);
   Camera camera = scene->getRootState()->staticState->camera;
   depth = (pos-camera.getPosition())*camera.getOrthoNormViewDirection();
   screenDist = norm(click-proj2)*height*0.5f;
}
