//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SurfaceElementsInteractionTools.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SurfaceElementsViewers.h"
#include "SurfaceElementsControllers.h"
#include "SurfaceElementsViewers.h"
#include "SurfaceElements.h"
#include "SurfaceElementStructureAttachment.h"
#include "SurfaceElementsInteractionContext.h"
#include "SurfaceElementEditingEventDispatcher.h"
#include "Object.h"
#include "StringClassProperty.h"
#include "SeparatorClassProperty.h"
#include "PointCloudTools.h"
#include "NumericalClassProperty.h"
#include "ClassMethods.h"
#include "TopologyAttachment.h"
#include "AttachedData.h"
#include "PCCComputeTopology.h"
#include "SurfaceElementsPolyMeshSegmentation.h"
#include "MeshIndexTablesHack.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "DebugRenderer.h"
#include "ColorTools.h"
#include "RaytracingMath.h"
#include "AttachedIndexedOctree.h"
#include "PointSetKNNQuery.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_ABSTRACT_CLASS( PCISurfaceElementInteractionTool ,0)
{
	BEGIN_CLASS_INIT( PCISurfaceElementInteractionTool );
   ADD_SEPARATOR("input")
   ADD_STRING_PROP_METHOD(meshPC, 0, &getMeshPC, &setMeshPC)
   ADD_STRING_PROP_METHOD(continuousPC, 0, &getContinuousPC, &setContinuousPC)
   ADD_SEPARATOR("interface settings")
   ADD_OBJECT_PROP(visContext, 0, SEVContext::getClass(), true)
   ADD_OBJECT_PROP(interactionContext, 0, SurfaceElementsInteractionContext::getClass(), true)
}

PCISurfaceElementInteractionTool::PCISurfaceElementInteractionTool() {
   width = 0;
   height = 0;
   meshPC = "root/pc1";
   continuousPC = "root/pc1";
   visContext = new SEVContext();
   dispatcher = new SurfaceElementEditingEventDispatcher();
   interactionContext = new SurfaceElementsInteractionContext();
   resultsAttachment = NULL;
   viewerHierarchy = NULL;
   leftDown = false;
   camMode = false;
}

PCISurfaceElementInteractionTool::~PCISurfaceElementInteractionTool() {
    delete dispatcher;
    delete visContext;
    delete viewerHierarchy;
    delete interactionContext;
    for (size_t i=0; i<helperSurfaceElements.size(); i++) delete helperSurfaceElements[i];
    for (size_t i=0; i<meshSurfaceElements.size(); i++) delete meshSurfaceElements[i];
    for (size_t i=0; i<tempSurfaceElements.size(); i++) delete tempSurfaceElements[i];
}

void PCISurfaceElementInteractionTool::assign(const Object* obj, X4CopyContext *context) {
   copyObjectProperties(obj, this);
}




SurfaceElementViewer *PCISurfaceElementInteractionTool::createViewerControllers(
   std::vector<SurfaceElement*> &elementList, bool ownTheElements,
   bool lightWeightVis) const
{
   SurfaceElementViewerList *result = new SurfaceElementViewerList();
   result->setup(NULL, NULL);
   for (size_t i=0; i<elementList.size(); i++) {
      SurfaceElementController *controller = createLocalControler(elementList[i]);
      SurfaceElementViewer     *viewer     = controller->setup(elementList[i]);
      viewer->setOwnerOfModel(ownTheElements);
      viewer->setLightWeightVis(lightWeightVis);
      result->addViewer(viewer);
   }
   return result;
}

void PCISurfaceElementInteractionTool::addResultsViewer(SurfaceElement *element)
{
   SurfaceElementController *controller = createLocalControler(element);
   if (controller) {
      SurfaceElementViewer *viewer = controller->setup(element);
      viewer->setOwnerOfModel(false);
      viewerHierarchy->addResultsViewer(viewer);
   }
   dispatcher->updateControllers(viewerHierarchy);
}


void X4::PCISurfaceElementInteractionTool::addHelperElementWithViewer( SurfaceElement *element )
{
   helperSurfaceElements.push_back(element);
   SurfaceElementController *controller = createLocalControler(element);
   if (controller) {
      SurfaceElementViewer *viewer = controller->setup(element);
      viewer->setOwnerOfModel(false);
      if (dynamic_cast<SurfaceElementViewerPointAsSphere*>(viewer))
         ((SurfaceElementViewerPointAsSphere*)viewer)->setColor(makeVector4f(1,1,0,1));
      viewerHierarchy->addMeshViewer(viewer);
      viewer->setLightWeightVis(true);
   }
}


void X4::PCISurfaceElementInteractionTool::updateAllControllers()
{
   dispatcher->updateControllers(viewerHierarchy);
}


void PCISurfaceElementInteractionTool::addTempViewer(SurfaceElement *element)
{
   SurfaceElementController *controller = createLocalControler(element);
   if (controller) {
      SurfaceElementViewer *viewer = controller->setup(element);
      viewer->setOwnerOfModel(false);
      viewerHierarchy->addTempViewer(viewer);
   }
   dispatcher->updateControllers(viewerHierarchy);
}

void PCISurfaceElementInteractionTool::addTempViewerOnly(SurfaceElementViewer *viewer)
{
   viewerHierarchy->addTempViewer(viewer);
}

void X4::PCISurfaceElementInteractionTool::newClicks()
{
   clickBuffer.clear();
}


void PCISurfaceElementInteractionTool::doGuessClickPoint(int32 x, int32 y) {
   SurfaceElementEditingEventDispatcher::SurfacePointInfo info;
   info.x = x;
   info.y = y;
   info.hit = guessClickPoint(x,y, interactionContext->pickDistance, info.point);
   dispatcher->updateSurfacePointInfo(info);
}


void X4::PCISurfaceElementInteractionTool::addClick( const Click &c )
{
   clickBuffer.push_back(c);
}

void X4::PCISurfaceElementInteractionTool::keyDown( GeneralKey key )
{
   if (width==0 || !viewerHierarchy) return;
   updateIntContext();
   if (!dispatcher->keyDown(interactionContext, key)) keyDownPassToCam(key);
}

void X4::PCISurfaceElementInteractionTool::keyUp( GeneralKey key )
{
   if (width==0 || !viewerHierarchy) return;
   updateIntContext();
   if (!dispatcher->keyUp(interactionContext, key)) keyUpPassToCam(key);
}

void X4::PCISurfaceElementInteractionTool::mouseDown( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState )
{
   if (buttonsState.getLeft()) leftDown = true;
   startInternalEvent(modifiersState, TYPE_DOWN);
   if (width==0 || !viewerHierarchy) return;
   doGuessClickPoint(x,y);
   updateIntContext();
   if (!dispatcher->mouseDown(interactionContext, x,y,buttonsState,modifiersState)) {
      mouseDownPassToCam(x,y,buttonsState,modifiersState);
      if (leftDown) camMode = true;
   }
   endInternalEvent(TYPE_DOWN);
}

void X4::PCISurfaceElementInteractionTool::mouseMoved( int32 x, int32 y )
{
   if (!camMode) {
      Modifiers mods;
      startInternalEvent(mods, TYPE_MOVE);
      if (width==0 || !viewerHierarchy) return;
      doGuessClickPoint(x,y);
      updateIntContext();
      if (!dispatcher->mouseMoved(interactionContext, x,y)) mouseMovedPassToCam(x,y);
      endInternalEvent(TYPE_MOVE);
   } else {
      mouseMovedPassToCam(x,y);
   }
}

void X4::PCISurfaceElementInteractionTool::mouseUp( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState )
{
   if (buttonsState.getLeft()) {
      leftDown = false;
   }
   if (!camMode) {
      startInternalEvent(modifiersState, TYPE_UP);
      if (width==0 || !viewerHierarchy) return;
      doGuessClickPoint(x,y);
      updateIntContext();
      if (!dispatcher->mouseUp(interactionContext, x,y,buttonsState,modifiersState)) mouseUpPassToCam(x,y,buttonsState,modifiersState);
      endInternalEvent(TYPE_UP);
   } else {
      camMode = false;
      mouseUpPassToCam(x,y,buttonsState,modifiersState);
   }
}

void X4::PCISurfaceElementInteractionTool::mouseWheelRotated( int32 rotatedDelta, Modifiers modifiersState )
{
   mouseWheelRotatedPassToCam(rotatedDelta, modifiersState);
}

void X4::PCISurfaceElementInteractionTool::areaResize( card32 width, card32 height )
{
   this->width = width;
   this->height = height;
   if (width==0 || !viewerHierarchy) return;
   updateIntContext();
}

void X4::PCISurfaceElementInteractionTool::glDrawTool( GLContext *glContext )
{
   visContext->setupUpdate(width, height, glContext);
   if (viewerHierarchy) {
      visContext->pushRenderState();
      viewerHierarchy->drawGL(visContext);
      visContext->popRenderState();
   }
}

void X4::PCISurfaceElementInteractionTool::tryInit()
{
   UnstructuredInCorePointCloud *meshUPC = dynamic_cast<UnstructuredInCorePointCloud*>(getPointCloud(getScene(),meshPC));
   if (!meshUPC) return;

   meshSurfaceElements.clear();
   tempSurfaceElements.clear();
   PointSet *ps = meshUPC->getPointSet();
   mpcard n = ps->getNumEntries();
   if (n >= 100000) visContext->quick = true;
   for (mpcard i=0; i<n; i++) {
      SurfaceElementVertex *newSE = new SurfaceElementVertex();
      newSE->setup(meshUPC, i);
      meshSurfaceElements.push_back(newSE);
   }
   resultsAttachment = 
      dynamic_cast<SurfaceElementStructureAttachment*>(meshUPC->getAttachments()->getData("surfaceElements"));
   if (resultsAttachment == NULL) {
      resultsAttachment = new SurfaceElementStructureAttachment();
      resultsAttachment->setup("surfaceElements", AttachedData::ADF_PERSISTENT, NULL);
      meshUPC->getAttachments()->attachData(resultsAttachment);
   }
   updateViewerHierarchy();
}

void PCISurfaceElementInteractionTool::updateViewerHierarchy() {
   delete viewerHierarchy;
   viewerHierarchy = new MyRootViewer();
   viewerHierarchy->addMeshViewer(createViewerControllers(meshSurfaceElements, true, true));
   viewerHierarchy->addMeshViewer(createViewerControllers(helperSurfaceElements, true, true));
   viewerHierarchy->addResultsViewer(createViewerControllers(resultsAttachment->getRegions(), false));
   viewerHierarchy->addResultsViewer(createViewerControllers(resultsAttachment->getPoints(), false));
   updateIntContext();
   dispatcher->updateControllers(viewerHierarchy);
}

void X4::PCISurfaceElementInteractionTool::updateIntContext()
{
   interactionContext->scene = scene;
   interactionContext->currentTool = this;
   interactionContext->projectionMatrix = 
      scene->getRootState()->staticState->viewFrustum.getProjectionMatrix() *
      scene->getRootState()->staticState->camera.getViewMatrix();
   interactionContext->viewerHierarchy = viewerHierarchy;
   interactionContext->width = width;
   interactionContext->height = height;

}

void X4::PCISurfaceElementInteractionTool::clearTempViewers()
{
   viewerHierarchy->clearTempViewers();
   dispatcher->updateControllers(viewerHierarchy);
}

PCISurfaceElementInteractionTool::Click *PCISurfaceElementInteractionTool::getTopClick()
{
   if (clickBuffer.empty()) return NULL;
   sort(clickBuffer.begin(), clickBuffer.end());
   return &(clickBuffer[0]);
}


// ----


IMPLEMENT_X4_ABSTRACT_CLASS( PCISESelection , 0 )
{
   BEGIN_CLASS_INIT( PCISESelection );
}


SurfaceElementController *PCISESelection::createLocalControler(SurfaceElement *element) const
{
   if (dynamic_cast<SurfaceElementPoint*>(element)) {
      return new SurfaceElementControllerSelection();
   } else if (dynamic_cast<SurfaceElementPolygon*>(element)) {
      return new SurfaceElementControllerPolygonVertices();
   } else {
      return NULL;
   }
}

void PCISESelection::tryHoverElement( SurfaceElementController *issuer, SurfaceElement *element, ClickInfo &info )
{
   Click c;
   c.issuer = issuer;
   c.element = element;
   c.info = info;
   //c.position = issuer->getModel()->getPosition();
   addClick(c);
}


void PCISESelection::tryClickElement( SurfaceElementController *issuer, SurfaceElement *element, ClickInfo &info )
{
   Click c;
   c.issuer = issuer;
   c.element = element;
   c.info = info;
   //c.position = issuer->getModel()->getPosition();
   addClick(c);
   clicked = true;
}

void PCISESelection::glDrawTool( GLContext *glContext )
{
   PCISurfaceElementInteractionTool::glDrawTool(glContext);
   if (surfaceHit && !hoverHit) {
      getVisContext()->pushRenderState();
      Vector4f col = SurfaceElementViewer::SURFACE_HOVER_COLOR;
      glColor4fv(col.data());
      if (getVisContext()->quick) {
         glPointSize(getVisContext()->visScale);
         glBegin(GL_POINTS);
         glVertex3fv(lastSurfacePoint.data());
         glEnd();
      } else {
         glMatrixMode(GL_MODELVIEW);
         glPushMatrix();
         glTranslatef(lastSurfacePoint[0], lastSurfacePoint[1], lastSurfacePoint[2]);
         gluSphere(getVisContext()->sphereQuadric, getVisContext()->visScale * getVisContext()->worldSpaceScale, 5,5);
         glPopMatrix();
      }
      getVisContext()->popRenderState();
   }
}

bool X4::PCISESelection::tryFreeHoverPoint(Vector3f surfPoint, bool hit)
{
   lastSurfacePoint = surfPoint;
   surfaceHit = hit;
   return false;
}

void X4::PCISESelection::startInternalEvent(Modifiers modifiers, card32 type)
{
   freePointClicked = false;
   hoverHit = false;
   clicked = false;
   this->modifiers = modifiers;
   newClicks();
}

void X4::PCISESelection::endInternalEvent(card32 type)
{
   if (clicked) {
      Click *topClick = getTopClick();
      if (topClick) {
         if (type == TYPE_DOWN) processClickPoint(topClick); 
      }
      newClicks();  
   } else if (freePointClicked) {
      if (type == TYPE_DOWN) processFreeClickPoint(lastSurfacePoint);
   } else {
      hoverHit = false;
      Click *topClick = getTopClick();
      if (topClick) {
         if (getLeftDown()) {
            if (type == TYPE_MOVE) processDragPoint(topClick->issuer->getModel()->getCenter());
         } else {
            if (type == TYPE_MOVE) processHoverPoint(topClick);
         }
         hoverHit = true;
      }
      newClicks();
   }
}

bool X4::PCISESelection::tryFreeClickPoint( Vector3f surfPoint )
{
   lastSurfacePoint = surfPoint;
   surfaceHit = true;
   freePointClicked = true;
   return true;
}

void X4::PCISESelection::rightClick()
{
   canceled();
}

PCISESelection::PCISESelection() 
{
   lastSurfacePoint = NULL_VECTOR3F;
   surfaceHit = false;
   hoverHit = false;
   clicked = false;   
}


// ----


IMPLEMENT_X4_CLASS( PCISEPointSelection , 0 )
{
   BEGIN_CLASS_INIT( PCISEPointSelection );
}


PCISEPointSelection::PCISEPointSelection() {}

void PCISEPointSelection::processHoverPoint(Click *click)
{
   click->issuer->doHover();
}

void PCISEPointSelection::processClickPoint(Click *click)
{
   click->issuer->doLeftClick();
}

void PCISEPointSelection::processFreeClickPoint(Vector3f point)
{

}


// ----

static std::string descrMode(card32 m) {
   if (m==0) return "CREATE_POLYS";
   if (m==1) return "EDIT_POLYS";
   return "undefined";
}

IMPLEMENT_X4_CLASS( PCISEPolygonRegionTool , 0 )
{
   BEGIN_CLASS_INIT( PCISEPolygonRegionTool );
   ADD_SEPARATOR("PolygonRegionTool")
   ADD_CARD32_PROP_UPDATE_DESCR(mode, 0, &changeMode, descrMode, 2, NULL)
   ADD_INT32_PROP_UPDATE(regionID, 0, &regionIDChanged)
   ADD_INT32_PROP(autoColorNumIDs, 0)
   ADD_VECTOR4F_PROP(regionColor, 0)
   ADD_SEPARATOR("Symmetry")
   ADD_VECTOR3F_PROP(upDirection, 0)
   ADD_FLOAT32_PROP(symSearchRadius, 0)
   ADD_FLOAT32_PROP(symAngleThreshold, 0)
   ADD_FLOAT32_PROP(lineFeatureCosAngle, 0)
   ADD_SEPARATOR("Segmentation")
   ADD_OBJECT_PROP(segmentation, 0, SurfaceElementsPolyMeshSegmentation::getClass(), true)
   ADD_STRING_PROP(cutOutMesh, 0)
   ADD_FLOAT32_PROP(lineFeatureSamplingFact, 0)
   ADD_FLOAT32_PROP(lfDistFactor, 0)
   ADD_BOOLEAN_PROP(noOverlapTest, 0)
   ADD_FLOAT32_PROP(minCoverageFact, 0)
   ADD_FLOAT32_PROP(lineTestDownSample, 0)
   ADD_FLOAT32_PROP(diagTemplateBorderFact, 0)
   
   ADD_SEPARATOR_METHOD("input")
   ADD_NOARGS_METHOD(addIntersections)
   ADD_SEPARATOR_METHOD("polygon editing")
   ADD_NOARGS_METHOD(changeRegionID)
   ADD_NOARGS_METHOD(deletePoly)
   ADD_NOARGS_METHOD(subdivideEdge)
   ADD_NOARGS_METHOD(subdivideEdgeAllRegions)
   ADD_NOARGS_METHOD(removeVertex)
   ADD_NOARGS_METHOD(removeVertexAllRegions)
   ADD_SEPARATOR_METHOD("symmetry")
   ADD_NOARGS_METHOD(findAllSimilar)
   ADD_NOARGS_METHOD(matchLineFeatures)
   ADD_SEPARATOR_METHOD("output")
   ADD_NOARGS_METHOD(cutRegions)
}


PCISEPolygonRegionTool::PCISEPolygonRegionTool() {
   mode = 0;
   regionID = 0;
   autoColorNumIDs = -1;
   regionColor = makeVector4f(0.5f,0.0f,0.0f,0.0f);
   currentPoly = NULL;
   selectedPoly = NULL;
   selectedPoint = NULL;
   selectedViewer = NULL;
   upDirection = YAXIS_VECTOR3F;
   symSearchRadius = 0.01f;
   symAngleThreshold = 0.0314f;
   lineFeatureCosAngle = 0.9f; // ~ 25?
   segmentation = new SurfaceElementsPolyMeshSegmentation();
   cutOutMesh = "root/pc1";
   lineFeatureSamplingFact = 1;
   lfDistFactor = 0.1f;
   noOverlapTest = false;
   minCoverageFact = 0.95f;
   lineTestDownSample = 10; 
   diagTemplateBorderFact = 0.05f;
}

PCISEPolygonRegionTool::~PCISEPolygonRegionTool() {
   delete segmentation;
}

void PCISEPolygonRegionTool::processHoverPoint(Click *click)
{
   click->issuer->doHover();
}

void PCISEPolygonRegionTool::processClickPoint(Click *click)
{
   if (mode == 0) {
      if (!currentPoly) {
         currentPoly = new SurfaceElementPolygonAnnotated();
      }
      Vector3f pos = click->element->getCenter();
      if (!getModifiers().getCtrl()) {
         if (!getModifiers().getShift()) {
            if (!currentPoly->getPoints().empty()) {
               if (pos == currentPoly->getPoints()[0]) {
                  currentPoly->getPoints().push_back(pos);
                  currentPoly->setId(regionID);
                  currentPoly->setColor(regionColor);
                  getResultsAttachment()->getRegions().push_back(currentPoly);
                  addResultsViewer(currentPoly);
                  clearTempViewers();               
                  currentPoly = NULL;
                  return;
               }
            }
            currentPoly->getPoints().push_back(pos);   
            clearTempViewers();
            addTempViewer(currentPoly);
         } else {
            currentPoly->getAnnotations().push_back(pos);
            clearTempViewers();
            addTempViewer(currentPoly);
         }
      } else {
         for (size_t i=0; i<currentPoly->getPoints().size(); i++) {
            if (currentPoly->getPoints()[i] == pos) {
               std::vector<Vector3f>::iterator pos = currentPoly->getPoints().begin();
               pos += i;
               currentPoly->getPoints().erase(pos);
               if (currentPoly->getPoints().empty()) {
                  delete currentPoly;
                  currentPoly = NULL;
                  clearTempViewers();
               }
               return;
            }
         }
         for (size_t i=0; i<currentPoly->getAnnotations().size(); i++) {
            if (currentPoly->getAnnotations()[i] == pos) {
               std::vector<Vector3f>::iterator pos = currentPoly->getAnnotations().begin();
               pos += i;
               currentPoly->getAnnotations().erase(pos);
               clearTempViewers();
               addTempViewer(currentPoly);
               return;
            }
         }
      }
   } else { // polygon editing mode
      SurfaceElementPolygonPoint *polyPoint = dynamic_cast<SurfaceElementPolygonPoint*>(click->element);
      if (polyPoint) {
         if (selectedPoint != polyPoint) {
            if (selectedViewer) selectedViewer->setSelected(false);
            if (selectedPoint) selectedPoint->getViewerHint()->setSelected(false);
                        
            selectedPoly = polyPoint->getParent();
            selectedPoint = polyPoint;
            SurfaceElementViewer *parentViewer = polyPoint->getViewerHint();
            selectedViewer = parentViewer;
            if (selectedViewer) selectedViewer->setSelected(true);
            SurfaceElementPolygonAnnotated *anno = dynamic_cast<SurfaceElementPolygonAnnotated*>(selectedPoly);
            if (anno) {
               regionID = anno->getId();
               regionColor = anno->getColor();
               ObjectViewsTable::update(this);
            }
         }
      } else {
         selectedPoly = NULL;
         selectedPoint = NULL;
         selectedViewer = NULL;
         updateViewerHierarchy();
      }
   }
}

void PCISEPolygonRegionTool::processFreeClickPoint(Vector3f point)
{
   if (mode != 0) return;
   if (!currentPoly) {
      currentPoly = new SurfaceElementPolygonAnnotated();
   }
   if (!getModifiers().getCtrl()) {
      if (!getModifiers().getShift()) {
         if (!currentPoly->getPoints().empty()) {
            if (norm(point - currentPoly->getPoints().front()) < getInteractionContext()->worldSpaceSnap) {
               currentPoly->getPoints().push_back(point);
               currentPoly->setId(regionID);
               currentPoly->setColor(regionColor);
               getResultsAttachment()->getRegions().push_back(currentPoly);
               addResultsViewer(currentPoly);
               clearTempViewers();
               currentPoly = NULL;
               return;
            }
         }
         currentPoly->getPoints().push_back(point);
         clearTempViewers();
         addTempViewer(currentPoly);
      } else {
         currentPoly->getAnnotations().push_back(point);
         clearTempViewers();
         addTempViewer(currentPoly);
      }
   } else {
      for (size_t i=0; i<currentPoly->getPoints().size(); i++) {
         if (norm(point - currentPoly->getPoints()[i]) < getInteractionContext()->worldSpaceSnap) {
            std::vector<Vector3f>::iterator pos = currentPoly->getPoints().begin();
            pos += i;
            currentPoly->getPoints().erase(pos);
            if (currentPoly->getPoints().empty()) {
               delete currentPoly;
               currentPoly = NULL;
            }
            clearTempViewers();
            return;
         }
      }
      for (size_t i=0; i<currentPoly->getAnnotations().size(); i++) {
         if (norm(point - currentPoly->getAnnotations()[i]) < getInteractionContext()->worldSpaceSnap) {
            std::vector<Vector3f>::iterator pos = currentPoly->getAnnotations().begin();
            pos += i;
            currentPoly->getAnnotations().erase(pos);
            clearTempViewers();
            addTempViewer(currentPoly);
            return;
         }
      }
   }
}

void PCISEPolygonRegionTool::glDrawTool( GLContext *glContext )
{
   PCISESelection::glDrawTool(glContext);
   //if (currentPoly && !currentPoly->getPoints().empty()) {
   //   SurfaceElementViewer *viewer = currentPoly->createDefaultViewer();
   //   viewer->drawGL(getVisContext());
   //   delete viewer;
   //}
}

void X4::PCISEPolygonRegionTool::canceled()
{
   delete currentPoly;
   currentPoly = NULL;
   clearTempViewers();
}

void X4::PCISEPolygonRegionTool::changeMode()
{
   if (selectedViewer) selectedViewer->setSelected(false);
   selectedPoly = NULL;
   selectedPoint = NULL;
   selectedViewer = NULL;
   delete currentPoly;
   currentPoly = NULL;
   clearTempViewers();
}

void X4::PCISEPolygonRegionTool::changeRegionID()
{
   if (mode == 1 && selectedPoly) {
      SurfaceElementPolygonAnnotated *anno = dynamic_cast<SurfaceElementPolygonAnnotated *>(selectedPoly);
      if (NULL != anno)
      {
      	anno->setColor(regionColor);
      	anno->setId(regionID);
         updateViewerHierarchy();
      }
   }
}

void X4::PCISEPolygonRegionTool::deletePoly()
{
   if (mode == 1 && selectedPoly) {
      getResultsAttachment()->deleteRegions(selectedPoly);
      updateViewerHierarchy();
   }
}

void X4::PCISEPolygonRegionTool::keyDown( GeneralKey key )
{
   char k = key.getkeyAlphaNumKey();
   if (k == 'r' || k == 'R') {
      changeRegionID();
   } if (k == 'd' || k == 'D') {
      deletePoly();
   } if (k == 'e' || k == 'E') {
      mode = 1;
      changeMode();
      ObjectViewsTable::update(this);
   } if (k == 'c' || k == 'C') {
      mode = 0;
      changeMode();
      ObjectViewsTable::update(this);
   }else {
      PCISurfaceElementInteractionTool::keyDown(key);   
   }   
}

void X4::PCISEPolygonRegionTool::subdivideEdge()
{
   if (mode == 1 && selectedPoly && selectedPoint) {
      mpint i = selectedPoly->getIndex(selectedPoint->getPoint());
      if (i==-1) return;
      selectedPoly->subdivideEdge(i);
      selectedPoint = NULL;
      updateViewerHierarchy();
   }
}

void X4::PCISEPolygonRegionTool::subdivideEdgeAllRegions()
{
   if (mode == 1 && selectedPoly && selectedPoint) {
      mpint index = selectedPoly->getIndex(selectedPoint->getPoint());
      if (index==-1) return;
      mpint selectedPolyID = -1;
      SurfaceElementPolygonAnnotated *anno = dynamic_cast<SurfaceElementPolygonAnnotated*>(selectedPoly);
      if (anno) {
         selectedPolyID = anno->getId();
      }      
      for (size_t i = 0; i<getResultsAttachment()->getRegions().size(); i++) {
         SurfaceElementPolygon *poly = dynamic_cast<SurfaceElementPolygon*>(getResultsAttachment()->getRegions()[i]);
         SurfaceElementPolygonAnnotated *annoOther = dynamic_cast<SurfaceElementPolygonAnnotated*>(poly);
         if (annoOther && anno) {
            if (selectedPolyID != annoOther->getId()) continue;
         }
         if (poly) poly->subdivideEdge(index);      
      }
      selectedPoint = NULL;
      updateViewerHierarchy();
   }
}

void X4::PCISEPolygonRegionTool::removeVertex()
{
   if (mode == 1 && selectedPoly && selectedPoint) {
      mpint i = selectedPoly->getIndex(selectedPoint->getPoint());
      if (i==-1) return;
      selectedPoly->deleteVertex(i);
      selectedPoint = NULL;
      updateViewerHierarchy();
   }
}

void X4::PCISEPolygonRegionTool::removeVertexAllRegions()
{
   if (mode == 1 && selectedPoly && selectedPoint) {
      mpint index = selectedPoly->getIndex(selectedPoint->getPoint());
      if (index==-1) return;
      mpint selectedPolyID = -1;
      SurfaceElementPolygonAnnotated *anno = dynamic_cast<SurfaceElementPolygonAnnotated*>(selectedPoly);
      if (anno) {
         selectedPolyID = anno->getId();
      }      
      for (size_t i = 0; i<getResultsAttachment()->getRegions().size(); i++) {
         SurfaceElementPolygon *poly = dynamic_cast<SurfaceElementPolygon*>(getResultsAttachment()->getRegions()[i]);
         SurfaceElementPolygonAnnotated *annoOther = dynamic_cast<SurfaceElementPolygonAnnotated*>(poly);
         if (annoOther && anno) {
            if (selectedPolyID != annoOther->getId()) continue;
         }
         if (poly) poly->deleteVertex(index);
      }
      selectedPoint = NULL;
      updateViewerHierarchy();
   }
}

void X4::PCISEPolygonRegionTool::processDragPoint( Vector3f where )
{
   if (mode == 1 && selectedPoint) {
      mpint index = selectedPoint->getParent()->getIndex(selectedPoint->getPoint());
      if (index != -1) {
         selectedPoint->setup(where, selectedPoint->getParent());
         selectedPoint->getParent()->getPoints()[index] = where;
      }
   }
}

static InCorePCTopologyGraph *getTopology(PointCloud *pc) {
   AttachedDataContainer *attachments = pc->getAttachments();
   TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
   if (topAtt) {
      return topAtt->getTopology();
   } else {
      return NULL;
   }
}   

static void attachTopology(PointCloud *pc, InCorePCTopologyGraph *tg, bool visible) {
   AttachedDataContainer *attachments = pc->getAttachments();
   TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
   if (topAtt == NULL) {
      topAtt = new TopologyAttachment();
      topAtt->setup("TopologyAtt", AttachedData::ADF_PERSISTENT);
      topAtt->setVisible(visible);
      attachments->attachData(topAtt);
   }
   topAtt->setTopology(tg);
}


void X4::PCISEPolygonRegionTool::cutRegions()
{
   UnstructuredInCorePointCloud *cutMeshUPC = dynamic_cast<UnstructuredInCorePointCloud*>(getPointCloud(getScene(),cutOutMesh));
   if (!cutMeshUPC) {
      warning("cutOutMesh not found, or not a triangle mesh.");
      return;
   }
   InCorePCTopologyGraph *tg = getTopology(cutMeshUPC);
   if (!tg) {
      message("computing topology\n");
      PCCComputeTopology topComp;
      UnstructuredInCoreTriangleMesh *cutMesh = dynamic_cast<UnstructuredInCoreTriangleMesh*>(cutMeshUPC);
      if (cutMesh) {
         tg = topComp.computeTopologyFromMesh(cutMeshUPC->getPointSet(), cutMeshUPC);
      } else {
         tg = topComp.computeKNNTopology(cutMeshUPC->getPointSet(), cutMeshUPC);      
      }
      attachTopology(cutMeshUPC, tg, false);
   }
   clearTempViewers();
   
   SurfaceElementStructureAttachment *cutOutAttachment = 
      dynamic_cast<SurfaceElementStructureAttachment*>(cutMeshUPC->getAttachments()->getData("surfaceElements"));
   if (cutOutAttachment == NULL) {
      cutOutAttachment = new SurfaceElementStructureAttachment();
      cutOutAttachment->setup("surfaceElements", AttachedData::ADF_PERSISTENT, NULL);
      cutMeshUPC->getAttachments()->attachData(cutOutAttachment);
   }

   
   for (size_t r=0; r<getResultsAttachment()->getRegions().size(); r++) {
      SurfaceElementPolygonAnnotated *poly = dynamic_cast<SurfaceElementPolygonAnnotated*>(getResultsAttachment()->getRegions()[r]);
      if (poly) {
         SurfaceElementMeshPolygon *result = segmentation->cutMesh(cutMeshUPC, tg, poly);
         //SurfaceElementViewer *viewer = result->createDefaultViewer(true);
         //SurfaceElementViewerPolyLine *pv = dynamic_cast<SurfaceElementViewerPolyLine*>(viewer);
         //if (pv) {
         //   pv->setColor(poly->getColor()*0.5f+makeVector4f(1,1,1,1)*0.5f);
         //   pv->setShowVertices(false);
         //}
         cutOutAttachment->getRegions().push_back(result);
      }
   }
}

//void X4::PCISEPolygonRegionTool::findAllSimilar()
//{
//   DescriptorContext ctxt(upDirection, symAngleThreshold);
//   if (mode == 1 && selectedPoly && selectedPoint) {
//      UnstructuredInCoreTriangleMesh *meshUPC = dynamic_cast<UnstructuredInCoreTriangleMesh*>(getPointCloud(getScene(),getMeshPC()));
//      if (!meshUPC) return;
//      MeshIndexTablesHack table(meshUPC);
//      PointSet *ps = meshUPC->getPointSet();
//      AAT positionAAT = ps->getAAT("position");
//      DescriptorSet descrSet(ctxt, meshUPC);
//      Vector3f selPos = selectedPoint->getPoint();
//      mpcard N = ps->getNumEntries();
//      mpint selIndex = -1;
//      for (mpcard i=0; i<N; i++) {
//         if (ps->get3f(i, positionAAT) == selPos) {
//            selIndex = i;
//            break;
//         }
//      }
//      if (selIndex == -1) return;
//      
//      Descriptor descr = descrSet.getDescriptor(selIndex);
//      for (mpcard i=0; i<descrSet.getNumDescriptors(); i++) {
//         if (descr.equal(ctxt, descrSet.getDescriptor(i))) {
//            SurfaceElementFreePoint *el = new SurfaceElementFreePoint();
//            el->setup(ps->get3f(i, positionAAT));
//            SurfaceElementViewer *viewer = el->createDefaultViewer(true);
//            viewer->setSelected(true);
//            addTempViewerOnly(viewer);
//         }
//      }      
//   }
//}

static bool noOverlap(std::vector<SurfaceElement*> elements, SurfaceElement *newElement) {
   BoundingBox3f compBB = newElement->getBoundingBox();
   float32 bbDiameter = compBB.getDiagonalLength();
   compBB.addBorder(bbDiameter/5);
   for (size_t i=0; i<elements.size(); i++) {
      BoundingBox3f bb = elements[i]->getBoundingBox();
      if (bb.intersectBoundingBox(compBB) != BVQueryDisjoint) {
         BoundingBox3f intersect = bb.computeIntersection(compBB);
         if (intersect.getDiagonalLength() > bbDiameter / 2) {
            return false;
         }
      }
   }
   return true;
}

void X4::PCISEPolygonRegionTool::findAllSimilar()
{
   DescriptorContext ctxt(upDirection, symAngleThreshold, lineFeatureCosAngle);
   if (mode == 1 && selectedPoly && selectedPoint) {
      UnstructuredInCoreTriangleMesh *meshUPC = dynamic_cast<UnstructuredInCoreTriangleMesh*>(getPointCloud(getScene(),getMeshPC()));
      if (!meshUPC) return;
      MeshIndexTablesHack table(meshUPC);
      PointSet *ps = meshUPC->getPointSet();
      AAT positionAAT = ps->getAAT("position");
      DescriptorSet descrSet(ctxt, meshUPC, &table);
      Vector3f selPos = selectedPoint->getPoint();
      mpcard N = ps->getNumEntries();
      mpint selIndex = -1;
      for (mpcard i=0; i<N; i++) {
         if (ps->get3f(i, positionAAT) == selPos) {
            selIndex = i;
            break;
         }
      }
      if (selIndex == -1) return;

      Matrix3f selCoords;
      selCoords[1] = ctxt.upDirection;
      selCoords[2] = normalize(table.getVertexNormal(selIndex));
      selCoords[0] = selCoords[1].crossProduct(selCoords[2]);
      selCoords[2] = selCoords[1].crossProduct(selCoords[0]);


      Descriptor descr = descrSet.getDescriptor(selIndex);
      for (mpcard i=0; i<descrSet.getNumDescriptors(); i++) {
         if (descr.equal(ctxt, descrSet.getDescriptor(i))) {
            Vector3f matchPos = ps->get3f(i, positionAAT);
            if (matchPos == selPos) continue;
            Matrix3f matchCoords;
            matchCoords[1] = ctxt.upDirection;
            matchCoords[2] = normalize(table.getVertexNormal(i));
            matchCoords[0] = matchCoords[1].crossProduct(matchCoords[2]);
            matchCoords[2] = matchCoords[1].crossProduct(matchCoords[0]);
            
            Matrix4f transformSelToCurrent = 
                     makeTranslation4f(matchPos)*
                         expand3To4(matchCoords)*
               expand3To4(selCoords.transpose())*
                      makeTranslation4f(-selPos);
            
            //Matrix4f idCheck = transformSelToCurrent - IDENTITY4F;
            //            
            //if (idCheck[0]*idCheck[0] + idCheck[1]*idCheck[1] + idCheck[2]*idCheck[2] + idCheck[3]*idCheck[3] <= 1e-4) continue;
            
            bool matchAllFine = true;
            //debugRenderer->beginRenderJob_OneFrame("dbSymSel", i);
            SimplePolygon smppoly;
            for (size_t j=0; j<selectedPoly->getNumPoints(); j++) {
               Vector3f addMatchPoint = transformVector3f(transformSelToCurrent, selectedPoly->getPoint(j));
               smppoly.addVertex(PolyVector3f::create(addMatchPoint));
               Descriptor descr;
               bool foundIt = false;
               for (mpcard k=0; k<ps->getNumEntries(); k++) {
                  if (ps->get3f(k, positionAAT) == selectedPoly->getPoint(j)) {
                     descr = descrSet.getDescriptor(k);
                     foundIt = true;
                     break;
                  }
               }
               if (!foundIt) continue;
               
               bool matchOk = false;
               for (mpcard k=0; k<ps->getNumEntries(); k++) {
                  if (normQuad(ps->get3f(k, positionAAT) - addMatchPoint) < symSearchRadius*symSearchRadius) {
                     if (descrSet.getDescriptor(k).equal(ctxt, descr)) {
                        matchOk = true;
                        break;
                     }
                  }
               }
               if (!matchOk) {   
                  matchAllFine = false;
                  break;
               }
            }
            
            if (matchAllFine) {
               SurfaceElementPolygon *trCopy = selectedPoly->createTransformedCopy(transformSelToCurrent);
               if (noOverlap(getResultsAttachment()->getRegions(), trCopy)) {
                  getResultsAttachment()->getRegions().push_back(trCopy);
               } else {
                  delete trCopy;
               }
               //debugRenderer->addPolygonOutline(smppoly, makeVector3f(0,0.6f,0), 8);
            } //else {
               //debugRenderer->addPolygonOutline(smppoly, makeVector3f(0.6f,0,0), 8);
            //}
            //debugRenderer->endRenderJob();
         }
      }
      updateViewerHierarchy();
   }
}

void X4::PCISEPolygonRegionTool::regionIDChanged()
{
   if (autoColorNumIDs >0) {
      regionColor = rainbowGradient(regionID / (float32)(autoColorNumIDs-1))*0.5f;
      ObjectViewsTable::update(this);
   }
}

struct Triangle {
   Vector3f vertex[3];
   float32 getMinX() const {
      return min(vertex[0][0],min(vertex[1][0],vertex[2][0]));
   }
   float32 getMaxX() const {
      return max(vertex[0][0],max(vertex[1][0],vertex[2][0]));
   }
   bool operator<(Triangle &other) const {
      return getMaxX() < other.getMaxX();
   }
};

static size_t findFirstOverlap(const std::vector<Triangle> &tris, Triangle &search, size_t s, size_t e) {
   if (e-s < 2) {
      //if (tris[s].getMinX() < search.getMaxX() && s>0) return s-1;
      if (s>0) return s-1;
      return s;
   } else {
      size_t m = (s+e)/2;
      if (tris[m].getMaxX() < search.getMinX()) {
         return findFirstOverlap(tris, search, m,e);
      } else {
         return findFirstOverlap(tris, search, s,m);      
      }
   }
}

static bool intersect(const Triangle &tri, const Vector3f &x, const Vector3f &y, Vector3f &intersectionPoint) {
   Ray3f ray;
   ray.origin = x;
   ray.direction = y-x;
   float hitParam;
   bool hit = intersectTriangle3f(ray, tri.vertex[0], tri.vertex[1], tri.vertex[2], hitParam);
   if (hit && hitParam>= 0 && hitParam <= 1) {
      float32 maxEdge = max(norm(tri.vertex[1]-tri.vertex[0]), 
                            max(norm(tri.vertex[2]-tri.vertex[0]), norm(tri.vertex[2]-tri.vertex[1])));
      if (maxEdge < 1e-7) return false;
      intersectionPoint = ray.origin + ray.direction * hitParam;
      if (norm(tri.vertex[0]-intersectionPoint) / maxEdge < 0.01) return false;
      if (norm(tri.vertex[1]-intersectionPoint) / maxEdge < 0.01) return false;
      if (norm(tri.vertex[2]-intersectionPoint) / maxEdge < 0.01) return false;
      return true;
   }
   return false;
}

void X4::PCISEPolygonRegionTool::addIntersections()
{
   UnstructuredInCoreTriangleMesh *meshUPC = dynamic_cast<UnstructuredInCoreTriangleMesh*>(getPointCloud(getScene(),getMeshPC()));
   if (!meshUPC) return;
   std::vector<Triangle> triangles;
   PointSet *ps = meshUPC->getPointSet();
   AAT positionAAT = ps->getAAT("position");
   PointSet *psT = meshUPC->getTriangles();
   AAT indexAAT = psT->getAAT("index");
   for (mpcard i=0; i<meshUPC->getNumTriangles(); i++) {
      Vector3i ind = psT->get3i(i, indexAAT);
      Triangle t;
      t.vertex[0] = ps->get3f(ind[0], positionAAT);
      t.vertex[1] = ps->get3f(ind[1], positionAAT);
      t.vertex[2] = ps->get3f(ind[2], positionAAT);
      triangles.push_back(t);
   }
   sort(triangles.begin(),triangles.end());
   
   for (size_t compTri=0; compTri<triangles.size(); compTri++) {
      //for (size_t compEdges=0; compEdges<triangles.size(); compEdges++) {
      size_t startComp = findFirstOverlap(triangles, triangles[compTri], 0, triangles.size()-1);
      for (size_t compEdges=startComp; compEdges<triangles.size(); compEdges++) {
         if (triangles[compEdges].getMinX() > triangles[compTri].getMaxX()) break;
         Vector3f result;
         if (intersect(triangles[compTri], triangles[compEdges].vertex[0], triangles[compEdges].vertex[1], result)) {
            SurfaceElementFreePoint *el = new SurfaceElementFreePoint();
            el->setup(result);
            addHelperElementWithViewer(el);
         }
         if (intersect(triangles[compTri], triangles[compEdges].vertex[1], triangles[compEdges].vertex[2], result)) {
            SurfaceElementFreePoint *el = new SurfaceElementFreePoint();
            el->setup(result);
            addHelperElementWithViewer(el);
         }
         if (intersect(triangles[compTri], triangles[compEdges].vertex[0], triangles[compEdges].vertex[2], result)) {
            SurfaceElementFreePoint *el = new SurfaceElementFreePoint();
            el->setup(result);
            addHelperElementWithViewer(el);
         }
      }
   }
   updateAllControllers();
}




X4::PCISurfaceElementInteractionTool::MyRootViewer::MyRootViewer()
{
   this->meshViewer = new SurfaceElementViewerList();
   resultsViewer = new SurfaceElementViewerList();
   tempViewer = new SurfaceElementViewerList();
}

X4::mpcard X4::PCISurfaceElementInteractionTool::MyRootViewer::getNumChildViewers()
{
   return 3;
}

SurfaceElementViewer * X4::PCISurfaceElementInteractionTool::MyRootViewer::getChildViewer( mpcard index )
{
   if (index==0) return meshViewer;
   if (index==1) return resultsViewer;
   if (index==2) return tempViewer;
   return NULL;
}

void X4::PCISurfaceElementInteractionTool::MyRootViewer::drawGL( SEVContext *context )
{
   meshViewer->drawGL(context);
   resultsViewer->drawGL(context);
   tempViewer->drawGL(context);
}

void X4::PCISurfaceElementInteractionTool::MyRootViewer::addMeshViewer( SurfaceElementViewer *v )
{
   meshViewer->addViewer(v);
}

void X4::PCISurfaceElementInteractionTool::MyRootViewer::addResultsViewer( SurfaceElementViewer *v )
{
   resultsViewer->addViewer(v);
}

void X4::PCISurfaceElementInteractionTool::MyRootViewer::addTempViewer( SurfaceElementViewer *v )
{
   tempViewer->addViewer(v);
}

void X4::PCISurfaceElementInteractionTool::MyRootViewer::clearTempViewers()
{
   delete tempViewer;
   tempViewer = new SurfaceElementViewerList();
}

X4::PCISurfaceElementInteractionTool::MyRootViewer::~MyRootViewer()
{
   delete meshViewer;
   delete resultsViewer;
   delete tempViewer;
}


X4::PCISEPolygonRegionTool::Descriptor::Descriptor( const DescriptorContext &ctxt, MeshIndexTablesHack *table, mpcard vIndex )
{
   Matrix3f localCoords;
   localCoords[1] = normalize(ctxt.upDirection);
   localCoords[2] = normalize(table->getVertexNormal(vIndex));
   localCoords[0] = localCoords[1].crossProduct(localCoords[2]);
   Matrix3f toLocal = localCoords.transpose();
   std::vector<mpcard> adjacentEdges = table->getVertexEdges(vIndex);
   Vector3f vertexPosition = table->getVertexPosition(vIndex);
   for (size_t i=0; i<adjacentEdges.size(); i++) {
      mpcard nf = table->getEdgeNumFaces(adjacentEdges[i]);
      if (nf == 2) {
         Vector3f n0 = table->getFaceNormal(table->getEdgeFace(adjacentEdges[i], 0));
         Vector3f n1 = table->getFaceNormal(table->getEdgeFace(adjacentEdges[i], 1));
         if (normalize(n0)*normalize(n1) >= ctxt.lineFeatureCosAngle) continue;
      }
      
      mpcard otherIndex = table->getEdgeVertex0(adjacentEdges[i]);
      if (otherIndex == vIndex) {
         otherIndex = table->getEdgeVertex1(adjacentEdges[i]);
         x4Assert(otherIndex != vIndex);
      }
      Vector3f edgeEnd = table->getVertexPosition(otherIndex);
      Vector3f edgeVec = edgeEnd-vertexPosition;
      float32 l = norm(edgeVec);
      if (l<1e-5) continue;
      edgeVec /= l;
      Vector3f localEdgeVec = toLocal*edgeVec;
      DEntry entry;
      entry.angleUX = atan2(localEdgeVec[1], localEdgeVec[0]);
      entry.angleUN = atan2(localEdgeVec[1], localEdgeVec[2]);
      entries.push_back(entry);
   }
}

bool X4::PCISEPolygonRegionTool::Descriptor::eq( const DescriptorContext &ctxt, const Descriptor &other ) const
{
   bool overallOk = true;
   for (size_t i=0; i<entries.size(); i++) {
      float32 aux = entries[i].angleUX;
      float32 aun = entries[i].angleUN;
      bool ok = false;
      for (size_t j=0; j<other.entries.size(); j++) {
         if (fabs(aux - other.entries[j].angleUX) < ctxt.angleThreshold
         &&  fabs(aun - other.entries[j].angleUN) < ctxt.angleThreshold) {
            ok = true;
         }
      }
      overallOk &= ok;
   }
   return overallOk;
}

bool X4::PCISEPolygonRegionTool::Descriptor::equal( const DescriptorContext &ctxt, const Descriptor &other ) const
{
   return eq(ctxt, other) && other.eq(ctxt, *this);
}

//void X4::PCISEPolygonRegionTool::Descriptor::growEdge( DescriptorContext &ctxt, UnstructuredInCoreTriangleMesh *mesh, MeshIndexTablesHack *table, mpcard edgeIndex, float32 &angle, float32 &length )
//{
//   
//}

X4::PCISEPolygonRegionTool::DescriptorSet::DescriptorSet( DescriptorContext &ctxt, UnstructuredInCoreTriangleMesh *mesh, MeshIndexTablesHack *table )
{
   for (mpcard i=0; i<mesh->getPointSet()->getNumEntries(); i++) {
      Descriptor descr(ctxt, table, i);
      descriptors.push_back(descr);
   }
}




struct MyEdge {
   Vector3f v1,v2;
};

struct Base {
   Vector3f pos;
   Matrix3f coords;
};

static Base createBase(Vector3f pos, Vector3f up, Vector3f edge) {
   Base result;
   result.pos = pos;
   result.coords[1] = normalize(up);
   result.coords[2] = normalize(up.crossProduct(edge));
   result.coords[0] = normalize(up.crossProduct(result.coords[2]));
   return result;
}

void X4::PCISEPolygonRegionTool::matchLineFeatures()
{
   if (mode == 1 && selectedPoly) {
      UnstructuredInCoreTriangleMesh *snapped = NULL;
      UnstructuredInCoreTriangleMesh *meshUPC = dynamic_cast<UnstructuredInCoreTriangleMesh*>(getPointCloud(getScene(),getMeshPC()));
      if (!meshUPC) return;
      snapped = (UnstructuredInCoreTriangleMesh*)meshUPC->copy();
      AttachedIndexedOctree *meshOctree = PointSetKNNQuery::createOctree(meshUPC->getPointSet());
      AAT meshNormalAAT = meshUPC->getAAT("normal");
      PointSet *meshPS = meshUPC->getPointSet();
      PointSetKNNQuery meshKnn(meshUPC->getPointSet(),meshOctree, NULL_VECTOR3F, meshUPC->getAAT("position"));
      
      snapped->snapMesh(getVisContext()->worldSpaceScale/10);
      MeshIndexTablesHack table(snapped);
      mpcard numEdges = table.getNumEdges();
      std::vector<MyEdge> sharpEdges;
      debugRenderer->beginRenderJob("sharpEdges");
      for (mpcard ei=0; ei<numEdges; ei++) {
         mpcard nf = table.getEdgeNumFaces(ei);
         if (nf == 2) {
            mpcard f1 = table.getEdgeFace(ei, 0);
            mpcard f2 = table.getEdgeFace(ei, 1);
            Vector3f n1 = normalize(table.getFaceNormal(f1));
            Vector3f n2 = normalize(table.getFaceNormal(f2));
            float32 sprod = n1*n2;
            if (sprod <= 0.9) {
               MyEdge edge;
               edge.v1 = table.getVertexPosition(table.getEdgeVertex0(ei));
               edge.v2 = table.getVertexPosition(table.getEdgeVertex1(ei));
               sharpEdges.push_back(edge);
               debugRenderer->addLine(sharpEdges.back().v1,sharpEdges.back().v2,makeVector4f(1,1,1,1),makeVector4f(1,1,1,1), 2);
            }
         }
      }
      
      std::vector<MyEdge> templateLines;
      BoundingBox3f templateBB = selectedPoly->getBoundingBox();
      templateBB.addBorder(templateBB.getDiagonalLength()*diagTemplateBorderFact);
      for (mpcard i=0; i<sharpEdges.size(); i++) {
         if (templateBB.inBoundingBox(sharpEdges[i].v1) && templateBB.inBoundingBox(sharpEdges[i].v2)) {
            templateLines.push_back(sharpEdges[i]);
            debugRenderer->addLine(sharpEdges[i].v1, sharpEdges[i].v2, makeVector3f(0,0,0.5),makeVector3f(0,0,0.5),3);
         }
      }
      
      
      PointSet *iclPoints = new PointSet();
      VertexDescriptor vd;
      vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
      iclPoints->clearAndSetup(1,0,&vd);
      AAT iclPositionAAT = vd.getToken("position");
      
      const float32 lineSampleRes = (getVisContext()->worldSpaceScale*getVisContext()->visScale*lineFeatureSamplingFact);
      
      for (size_t i=0; i<sharpEdges.size(); i++) {
         float32 len = norm(sharpEdges[i].v2 - sharpEdges[i].v1);
         int numPts = ceil(len/lineSampleRes);
         float32 step = len / numPts;
         for (int j=0; j<=numPts; j++) {
            Vector3f newPoint = sharpEdges[i].v1 +(sharpEdges[i].v2-sharpEdges[i].v1)*((float)j/ (float)numPts);
            //debugRenderer->addPoint(newPoint, makeVector3f(0,0,0.5));
            iclPoints->addPoint((char*)&newPoint);
         }
      }

      AttachedIndexedOctree *octree = PointSetKNNQuery::createOctree(iclPoints);
      PointSetKNNQuery knn(iclPoints, octree, NULL_VECTOR3F, iclPoints->getAAT("position"));
      
      int selectedAngle = -1;
      Vector3f edge;
      for (mpcard i=0; i<selectedPoly->getNumPoints()-1; i++) {
         edge = selectedPoly->getPoint(i+1)-selectedPoly->getPoint(i);
         float32 n = norm(edge);
         if (n>=1e-5) {
            edge /=n;
            if (edge*upDirection <= 0.9) {
               selectedAngle = i;
               break;
            }
         }      
      }
      if (selectedAngle != -1) {
         Base edgeBase = createBase(selectedPoly->getPoint(selectedAngle), upDirection, edge);
         float32 baseEdgeOrient = 0;
         {
            meshKnn.setTargetPoint(edgeBase.pos);
            bool found;
            card32 index;
            meshKnn.getNextNearestPointIndex(index, found);
            Vector3f n = meshPS->get3f(index, meshNormalAAT);
            Vector3f ref = edgeBase.coords[1];
            baseEdgeOrient = n*ref;
         }
         for (size_t i=0; i<sharpEdges.size(); i++) {
            Vector3f dir = sharpEdges[i].v2-sharpEdges[i].v1;
            float32 ndir = norm(dir);
            if (ndir <= 1e-5) continue;
            dir /= ndir;
            if (upDirection*dir > 0.9) continue;
            Vector3f matchEdge[2] = {dir, -dir};
            Vector3f matchPos[2] = {sharpEdges[i].v1, sharpEdges[i].v2};
            for (int d=0; d<=1; d++) {                           
               Base matchBase = createBase(matchPos[d], upDirection, matchEdge[d]);
               {
                  meshKnn.setTargetPoint(matchBase.pos);
                  bool found;
                  card32 index;
                  meshKnn.getNextNearestPointIndex(index, found);
                  Vector3f n = meshPS->get3f(index, meshNormalAAT);
                  Vector3f ref = matchBase.coords[1];
                  float32 matchBaseOrient = n*ref;
                  if (baseEdgeOrient*matchBaseOrient < 0) continue;
               }


               Matrix4f transformSelToCurrent = 
                  makeTranslation4f(matchPos[d])*
                  expand3To4(matchBase.coords)*
                  expand3To4(edgeBase.coords.transpose())*
                  makeTranslation4f(-edgeBase.pos);
               
               if (determinant(matchBase.coords*edgeBase.coords.transpose()) < 0) continue;
                  
               int coverage = 0;
               int weight = 0;
               for (size_t j=0; j<templateLines.size(); j++) {
                  Vector3f lineStart = transformVector3f(transformSelToCurrent, templateLines[j].v1);
                  Vector3f lineEnd = transformVector3f(transformSelToCurrent, templateLines[j].v2);
                     
                  float32 len = norm(lineEnd - lineStart);
                  int numPts = ceil(len/(lineSampleRes * lineTestDownSample));
                  for (int p=0; p<=numPts; p++) {
                     weight ++;
                     Vector3f newPoint = lineStart +(lineEnd-lineStart)*((float)p/ (float)numPts);
                     knn.setTargetPoint(newPoint);
                     bool found;
                     card32 index;
                     knn.getNextNearestPointIndex(index, found);
                     if (found) {
                        Vector3f match = iclPoints->get3f(index, iclPositionAAT);
                        float32 dist = norm(match-newPoint);
                        if (dist < lfDistFactor*lineSampleRes) {
                           //debugOutput << "(" << dist << "/" << lfDistFactor*lineSampleRes << ") ";
                           coverage++;
                           debugRenderer->addPoint(newPoint, makeVector3f(1,1,0.5f));
                        } else {
                           debugRenderer->addPoint(newPoint, makeVector3f(0,0,0.5f));
                        }
                     }
                  }
                  float32 relCov = (float)coverage/(float)weight;
                  if (relCov > minCoverageFact) {
                     debugOutput << relCov;
                     //for (size_t j=0; j<selectedPoly->getNumPoints()-1; j++) {
                     //   Vector3f lineStart = selectedPoly->getPoint(j);
                     //   Vector3f lineEnd = selectedPoly->getPoint(j+1);
                     //   debugRenderer->addLine(
                     //      transformVector3f(transformSelToCurrent,lineStart),
                     //      transformVector3f(transformSelToCurrent,lineEnd),
                     //      makeVector3f(0.7f,0.5f,0.3f),makeVector3f(0.7f,0.5f,0.3f),1);
                     //}
                     SurfaceElementPolygon *trCopy = selectedPoly->createTransformedCopy(transformSelToCurrent);
                     if (noOverlapTest || noOverlap(getResultsAttachment()->getRegions(), trCopy)) {
                        getResultsAttachment()->getRegions().push_back(trCopy);
                        debugOutput << " accepted\n";
                     } else {
                        debugOutput << " overlaps\n";
                        delete trCopy;
                     }
                     
                  }
               }
            }
         }
         
      
      } else {
         warning("all edges align with upward direction!");
      }
      
      delete meshOctree;
      delete octree;
      delete iclPoints;
      debugRenderer->endRenderJob();
      delete snapped;
   }
}