//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SurfaceElementsPolyMeshSegmentation.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SurfaceElements.h"
#include "GeometricTools.h"
#include "InCorePCTopologyGraph.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_ABSTRACT_CLASS( SurfaceElementsPolyMeshSegmentation ,0)
{
	BEGIN_CLASS_INIT( SurfaceElementsPolyMeshSegmentation );
   ADD_BOOLEAN_PROP(curvatureAware,0)
   ADD_FLOAT32_PROP(nonCurvatureBias,0)
}

SurfaceElementsPolyMeshSegmentation::SurfaceElementsPolyMeshSegmentation() {
   curvatureAware = false;
   nonCurvatureBias = 0.1f;
}

SurfaceElementsPolyMeshSegmentation::~SurfaceElementsPolyMeshSegmentation() {}

void SurfaceElementsPolyMeshSegmentation::assign(const Object* obj, X4CopyContext *context) {
   copyObjectProperties(obj, this);
}


class CustomDistanceComp {
   PointSet *ps;
   AAT positionAAT;
   AAT curv2AAT;
   float nonCurvatureBias;

public:
   void setup(PointSet *ps, float bias) {
      this->ps = ps; 
      nonCurvatureBias=bias;
      positionAAT = ps->getAAT("position");
      curv2AAT = ps->getAAT("curv2");
   }
   static float computeDistance(int i1, int i2, void *context);   
};

float CustomDistanceComp::computeDistance(int i1, int i2, void *context)
{
   CustomDistanceComp *cThis = (CustomDistanceComp*)(context);
   Vector3f p1 = cThis->ps->get3f(i1, cThis->positionAAT);
   Vector3f p2 = cThis->ps->get3f(i2, cThis->positionAAT);
   float c1 = cThis->ps->get2f(i1, cThis->curv2AAT)[0];
   float c2 = cThis->ps->get2f(i2, cThis->curv2AAT)[0];
   float dist = norm(p1-p2);
   dist /= cThis->nonCurvatureBias + (fabs(c1+c2))/2;
   return dist;
}


SurfaceElementMeshPolygon * X4::SurfaceElementsPolyMeshSegmentation::cutMesh( 
   UnstructuredInCorePointCloud *mesh, 
   InCorePCTopologyGraph *tg, 
   SurfaceElementPolygon *poly )
{
   CustomDistanceComp dcomp;
   if (curvatureAware) {
      if (!mesh->providesAttribute("curv2")) throw EInvalidState("curv2 channel required (use PCCGaussianCurvature first)");
      dcomp.setup(mesh->getPointSet(), nonCurvatureBias);
   }
   SurfaceElementMeshPolygon *result = new SurfaceElementMeshPolygon();
   result->setup(mesh);
   ComputeGeodeticDistanceShortRange comp(mesh, tg);
   for (mpint i=0; i<((mpint)poly->getNumPoints())-1; i++) {
      Vector3f start = poly->getPoint(i+1);
      Vector3f end = poly->getPoint(i);

      bool flip = false;
      if (start[0] > end[0]) {
         flip = true;
         if (start[0] == end[0]) {
            if (start[1] > end[1]) {
               flip = true;
               if (start[1] == end[1]) {
                  if (start[2] > end[2]) {
                     flip = true;
                  }
               }
            }
         }
      }
      if (flip) {
         swap(start, end);
      }
      
      std::vector<Vector3f> endPoints;
      endPoints.push_back(end);
      std::vector< std::vector<int32> > paths;
      comp.compute(start, endPoints, &paths, curvatureAware?&dcomp.computeDistance:NULL, (void*)&dcomp );
      if (!paths.empty()) {
         if (flip) {
            reverse(paths[0].begin(),paths[0].end());
         }
         result->addIndices(paths.front());
      }
   }
   return result;
}
