#include "StdAfx.h"
#include "FastDMatrix.h"

using namespace X4;

FastDMatrix::FastDMatrix(void)
{
	data = NULL;
	numColumns = 0;
	numRows = 0;
}

FastDMatrix::~FastDMatrix(void)
{
}

void FastDMatrix::setZero()
{
	memset(data, 0, numColumns*numRows * sizeof(float) );
}

void FastDMatrix::setDimension( size_t columns, size_t rows )
{
	if (numColumns == columns && numRows == rows)
		return;
	delete[] data;
	numColumns = columns;
	numRows    = rows;
	data = new float[columns*rows];
}

float FastDMatrix::biggestValue() const
{
	const unsigned numEnries = getNumEntries();
	float biggestNumber = 0;
	for (unsigned i = 0; i < numEnries; ++i)
	{
		if (data[i] > biggestNumber)
		{
			biggestNumber = data[i];
		}
	}
	return biggestNumber;
}

size_t FastDMatrix::countNonZeros() const
{
	const unsigned numEnries = getNumEntries();
	size_t numNonZeros = 0;
	for (unsigned i = 0; i < numEnries; ++i)
	{
		if (data[i] != 0.0f)
			++numNonZeros;
	}
	return numNonZeros;
}

size_t FastDMatrix::getNumEntries() const
{
	return numColumns*numRows;
}

float FastDMatrix::sumAllValues() const
{
	const unsigned numEnries = getNumEntries();
	float sumOfValues = 0;
	for (unsigned i = 0; i < numEnries; ++i)
	{
		sumOfValues += data[i];
	}
	return sumOfValues;
}
