#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "FastSparseMatrix.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SparseLinearAlgebra.inline.h"
#include "Tools.h"
#include "IterativeSolvers.h"
#include "IterativeSolvers.inline.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( FastSparseMatrix ,0)
{
	BEGIN_CLASS_INIT( FastSparseMatrix );
}

FastSparseMatrix::FastSparseMatrix(void)
{
	m_MaxNumSolveIterations = 5000;
	stopRes = 0.0001f;
}

FastSparseMatrix::~FastSparseMatrix(void)
{
}


void FastSparseMatrix::solveLinearSystem(DVectorF &x, const DVectorF &b){
	solveWithIntel(x, b);
}

void FastSparseMatrix::assign(const Object* obj, X4CopyContext *context)
{
	// Persistent::assign(obj,context);
	const FastSparseMatrix * o = dynamic_cast<const FastSparseMatrix*>(obj);
	pAssert( o != NULL );

	copyObjectProperties(obj,this);
}



X4::int32 X4::FastSparseMatrix::solveWithIntel( DVectorF &x_f, const DVectorF&b_f )
{
	CGSolver<float32> solver;
	solver.printDebugOutput = true;
    solver.maxNumIterations = m_MaxNumSolveIterations;
	solver.stopRes = stopRes;
    solver.solve(matrix, x_f, b_f);

	return 0;
}

void X4::FastSparseMatrix::solveWithCuda( DVectorF &x, const DVectorF&b )
{
	notImplemented();
}

void X4::FastSparseMatrix::initUpperHalfMatrixFromFullSymmetricSparseMatrixF( const SparseMatrixF &input )
{
	matrix = input;

	//refill lower half - warning: has to be empty!
	SparseMatrixF lowerHalf = input.transpose();
	for(mpcard i = 0; i < lowerHalf.getNumRows(); i++){
		lowerHalf[i].setEntry(i, 0.0f);
	}
	matrix += lowerHalf;

}

X4::int32 X4::FastSparseMatrix::solveWithIntelPlusPreConditioner( DVectorF &x_f, const DVectorF &b_f )
{
	notImplemented();
	return -1;

}

X4::int32 X4::FastSparseMatrix::solveWithIntelPlusJacobiPreConditioner( DVectorF &x_f, const DVectorF &b_f )
{
	notImplemented();
	return -1;
}