#ifndef FastSparseMatrixH
#define FastSparseMatrixH
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "SparseLinearAlgebra.h"
#include "x3fastSparseMath.h"
//---------------------------------------------------------------------------

namespace X4
{


class X3FASTSPARSEMATH_API FastSparseMatrix : public Persistent
{
	X4_CLASS( FastSparseMatrix )

private:
	std::vector<double> dataVec;
	std::vector<int> columns;
	std::vector<int> rowIndex;

	mpcard m_MaxNumSolveIterations;
	float32 stopRes;
	SparseMatrixF matrix;

public:
	FastSparseMatrix(void);
	~FastSparseMatrix(void);

	void setMaxNumSolveIterations(mpcard n) {m_MaxNumSolveIterations = n;}
	void setStopResidual(float32 st) {stopRes = st;}

	void solveLinearSystem(DVectorF &x, const DVectorF &b);

	void initUpperHalfMatrixFromFullSymmetricSparseMatrixF(const SparseMatrixF &input);
	int32 solveWithIntel(DVectorF &x, const DVectorF &b);
	int32 solveWithIntelPlusPreConditioner(DVectorF &x, const DVectorF &b);
	int32 solveWithIntelPlusJacobiPreConditioner(DVectorF &x_f, const DVectorF &b_f);
	int32 solveWithIntelDirectSolver( DVectorF &x_f, const DVectorF &b_f );

	void solveWithCuda(DVectorF &x_f, const DVectorF&b_f);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

};

}

#endif