//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "AbstractGrid.h"
#include "ClassMethods.h"
#include "ObjectClassProperty.h"
#include "SeparatorClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "NumericalClassProperty.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_ABSTRACT_CLASS( AbstractGrid, 2 )
{
	BEGIN_CLASS_INIT( AbstractGrid )
	ADD_SEPARATOR( "Abstract grid" )
	ADD_VECTOR3F_PROP( bbox.lowerCorner, 1 )
	ADD_VECTOR3F_PROP( bbox.upperCorner, 1 )
	ADD_VECTOR3F_PROP( normalizationT, 2 )
	ADD_FLOAT32_PROP( normalizationS, 2 )
}

AbstractGrid::AbstractGrid()
{
	bbox = BoundingBox3f( makeVector3f( 0,0,0 ) );
	normalizationT = NULL_VECTOR3F;
	normalizationS = 1.0f;
}

AbstractGrid::~AbstractGrid()
{
}

Matrix4f AbstractGrid::getNormalization() const
{
	Matrix4f scale;
	Matrix4f translate;
	for ( unsigned int i = 0; i < 3; i++ ){
		scale[i][i] = normalizationS;
	}
	for ( unsigned int i = 0; i < 3; i++ ){
		translate[3][i] = -normalizationT[i];
	}
	return scale * translate;
}

Matrix4f AbstractGrid::getInvNormalization() const
{
	Matrix4f scale;
	Matrix4f translate;
	for ( unsigned int i = 0; i < 3; i++ ){
		scale[i][i] = 1.0f / normalizationS;
	}
	for ( unsigned int i = 0; i < 3; i++ ){
		translate[3][i] = normalizationT[i];
	}
	return translate * scale;
}

bool AbstractGrid::contains( const Vector3f& p ) const
{
	for ( unsigned int i = 0; i < 3; i++ ){
		if ( ( p[i] < bbox.lowerCorner[i] ) || ( p[i] > bbox.upperCorner[i] ) ) return false;
	}
	return true;
}