#ifndef AbstractGridH
#define AbstractGridH

#include <assert.h>
#include "AbstractNDTCell.h"
#include "Persistent.h"

namespace X4{

typedef pair <long, long> LongPair;

#define NDT_CELL 0
#define COLOR_NDT_CELL 1
#define SIXD_NDT_CELL 2
#define INVALID_CELL_TYPE 999

#define CREATE_GRID_FROM_TAU 0
#define CREATE_GRID_FROM_RES 1

class X3FOUNDATION_PRIVATE_API AbstractGrid : public Persistent
{
	X4_ABSTRACT_CLASS( AbstractGrid )

	protected:
		long numCells;
		BoundingBox3f bbox;
		Vector3f normalizationT;
		float32 normalizationS;
		card32 cellType;
		
	public:
		AbstractGrid();
		~AbstractGrid();

		Matrix4f getNormalization() const;
		Matrix4f getInvNormalization() const;

		card32 getCellType() const { return cellType; }
		void setCellType(card32 t) { this->cellType=t; }
		inline long getNumCells() const { return numCells; }
		bool contains( const Vector3f& p ) const;

		virtual void clear() = 0;
		virtual void setup( 
			const BoundingBox3f& bbox, 
			const float32 normalizationS, 
			const Vector3f& normalizationT, 
			const card32 setupMode, 
			const float32 setupTau, 
			const card32 setupRes ) = 0;
		virtual Vector3i getIndex3D( const Vector3f& p ) const = 0;
		virtual long getIndex1D( const Vector3i& index3D ) const = 0;
		virtual AbstractNDTCell* getCell( const Vector3f& p ) = 0;
		virtual AbstractNDTCell* getCell( const Vector3i& index3D ) = 0;
		virtual AbstractNDTCell* getCell( const long index1D ) = 0;
		virtual void setCell( const long index1D, AbstractNDTCell* cell ) = 0;
		virtual void printStats() = 0;
		virtual long getNumFilledCells() const = 0;
		virtual AbstractNDTCell* getFilledCell( const long filledCellIndex ) = 0;
		virtual BoundingBox3f computeCellBBox( const long index1D ) = 0;
		virtual BoundingBox3f computeCellBBox( const Vector3i& index3D ) = 0;
		virtual Vector3i getIndex3D( const long index1D ) = 0;

		/*** MEMBER ACCESS ***/
		inline BoundingBox3f getBBox() const { return bbox; }
		inline void setBBox( const BoundingBox3f& bbox ){ this->bbox = bbox; }
		inline void setNormalizationT( const Vector3f normalizationT ){ this->normalizationT = normalizationT; }
		inline Vector3f getNormalizationT() const {  return normalizationT; }
		inline void setNormalizationS( float32 normalizationS ){ this->normalizationS = normalizationS; }
		inline float32 getNormalizationS() const { return normalizationS; }
};

}

#endif