#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "AbstractMatrix.h"
//---------------------------------------------------------------------------
#include "opencv/cv.h"
#include "dynamiclinearalgebra.inline.h"
//---------------------------------------------------------------------------
using namespace X4;

AbstractMatrix::AbstractMatrix(void)
{
}

AbstractMatrix::~AbstractMatrix(void)
{
}

void X4::AbstractMatrix::setupIndentity( unsigned numRowsAndColumns)
{
	setup(numRowsAndColumns,numRowsAndColumns);
	for( unsigned i=0;i<numRowsAndColumns;i++ )
	{
		setEntry(i,i,1.0);
	}
}

void X4::DenseAbstractMatrix::setup( unsigned numColumns, unsigned numRows )
{
	m_Matrix.setDimension(numColumns, numRows, false);
}

void X4::DenseAbstractMatrix::setEntry( unsigned column, unsigned row, double value )
{
	m_Matrix[column][row] = value;
}

double X4::DenseAbstractMatrix::getEntry( unsigned column, unsigned row )
{
	return m_Matrix[column][row];
}

void X4::DenseAbstractMatrix::computeEigenStructure()
{
	m_EigenVectors.resize(m_Matrix.getColsDim()* m_Matrix.getRowsDim());
	m_EigenValues.setDim(m_Matrix.getColsDim());

	std::vector<double> input(m_EigenVectors.size());
	m_Matrix.copyDataToArray(&input[0]);

	// Prepare OpenCV Datasructure
	CvMat cvCov, cvEigenvalue, cvEigenvector;
	cvInitMatHeader( &cvCov, m_Matrix.getColsDim(), m_Matrix.getRowsDim(), CV_64FC1, &input[0]);
	cvInitMatHeader( &cvEigenvalue, m_Matrix.getColsDim(), 1, CV_64FC1, m_EigenValues.data());
	cvInitMatHeader( &cvEigenvector, m_Matrix.getColsDim(), m_Matrix.getRowsDim(), CV_64FC1, &m_EigenVectors[0]);

	// Do SVD
	cvSVD(&cvCov, &cvEigenvalue, &cvEigenvector, 0, CV_SVD_MODIFY_A | (CV_SVD_U_T));

}

X4::DVectorD X4::DenseAbstractMatrix::getEigenVector( unsigned i )
{
	DVectorD ret;
	ret.setDim(m_EigenValues.getDim());
	for( unsigned x=0;x<ret.getDim();x++ )
	{
		ret[x] = m_EigenVectors[i*ret.getDim() + x];
	}
	return ret;
}

double X4::DenseAbstractMatrix::getEigenValue( unsigned i )
{
	return m_EigenValues[i];
}