#ifndef AbstractMatrixH
#define AbstractMatrixH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	abstraction layer for matrix computations (e.g. eigenvectors)
*/
class X3FOUNDATION_PRIVATE_API AbstractMatrix
{
public:
	AbstractMatrix(void);
	~AbstractMatrix(void);

	/// setup matrix with zeros 
	virtual void setup(unsigned numColumns, unsigned numRows) = 0;

	/// setup identity matrix 
	void setupIndentity(unsigned numRowsAndColumns);

	/// set matrix entry[column,row] to value
	virtual void setEntry( unsigned column, unsigned row, double value) = 0;

	/// get matrix entry[column,row]
	virtual double getEntry(unsigned column, unsigned row) = 0;

	/// computes eigenvectors
	virtual void computeEigenStructure() = 0;

	/// get n th eigenvector (use computeEigenStructure before using this method!)
	virtual DVectorD getEigenVector( unsigned i) = 0;

	/// get n th eigenvalue (use computeEigenStructure before using this method!)
	virtual double getEigenValue( unsigned i) = 0;
};

/**
	dense matrix (funny name, I know)
*/
class X3FOUNDATION_PRIVATE_API DenseAbstractMatrix : public AbstractMatrix
{
private:

	DMatrixD		m_Matrix;
	std::vector<double>		m_EigenVectors;
	DVectorD		m_EigenValues;
public:

	/// setup matrix with zeros 
	virtual void setup(unsigned numColumns, unsigned numRows);

	/// set matrix entry[column,row] to value
	virtual void setEntry( unsigned column, unsigned row, double value);

	/// get matrix entry[column,row]
	virtual double getEntry(unsigned column, unsigned row);

	/// computes eigenvectors
	virtual void computeEigenStructure();

	/// get n th eigenvector (use computeEigenStructure before using this method!)
	virtual DVectorD getEigenVector( unsigned i);

	/// get n th eigenvalue (use computeEigenStructure before using this method!)
	virtual double getEigenValue( unsigned i);


};

}

#endif