//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "AnnSearch.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

AnnSearch::AnnSearch(float distEps){
   kdTree = NULL;
	nnIdx1 = new ANNidx[1];	
	dists1 = new ANNdist[1];
	numPoints=0;
	distanceEpsilon = distEps;
}

AnnSearch::~AnnSearch(){
   delete kdTree;
	delete [] nnIdx1;							// clean things up
	delete [] dists1;
	//annClose();		//TODO: should only be called, if all ANN trees are deleted
}

void AnnSearch::buildSearchStructure(card32 number, card32 dimension, ANNpointArray data){
	if (kdTree)
		delete kdTree;
	
	kdTree = new ANNkd_tree(
					data,					         // the data points
					number,						   // number of points
					dimension);						// dimension of space

	numPoints=number;

}

card32 AnnSearch::getNearestNeighbor(ANNpoint x)const{
	card32 nearestIndex=0;
	ANNidxArray			nnIdx = new ANNidx[1];					// near neighbor indices
	ANNdistArray		dists = new ANNdist[1];					// near neighbor distances


		kdTree->annkSearch(						// search
				x,						// query point
				1,								// number of near neighbors
				nnIdx,							// nearest neighbors (returned)
				dists,							// distance (returned)
				0.5);							// error bound
   nearestIndex= nnIdx[0];

   delete [] nnIdx;							// clean things up
   delete [] dists;

   return  nearestIndex;

}

/*
Test:

   std::vector<ANNpoint> ptData;
   card32 dim=2;
   ANNpoint a = new ANNcoord[dim];
   a[0]= 1;
   a[1]= 2;
   ANNpoint b = new ANNcoord[dim];
   b[0]= 5;
   b[1]= 4;
   ANNpoint c = new ANNcoord[dim];
   c[0]= -2;
   c[1]= 0;

   ptData.push_back(a);
   ptData.push_back(b);
   ptData.push_back(c);

   ANNpointArray annPtData= &ptData[0];


   
   AnnSearch search;
   search.buildSearchStructure(3,dim,annPtData);
   int erg=0;
   erg = search.getNearestNeighbor(b);
   erg = search.getNearestNeighbor(a);
   erg = search.getNearestNeighbor(c);



*/

X4::card32 X4::AnnSearch::getNearestNeighborAndDistance( const ANNpoint &searchPoint,float &distance )const
{
	kdTree->annkSearch(					
		searchPoint,						// query point
		1,								// number of near neighbors
		nnIdx1,							// nearest neighbors (returned)
		dists1,							// distance (returned)
		0.2f						// error bound
		);							

	distance=dists1[0];
	return  nnIdx1[0];
}

void X4::AnnSearch::getkNearestNeighbors( const ANNpoint &searchPoint,const card32 &k, float *distances, int32 *indexes )const
{
	x4Assert(k<=numPoints);
	kdTree->annkSearch(					
		searchPoint,						// query point
		k,								// number of near neighbors
		indexes,							// nearest neighbors (returned)
		distances,							// distance (returned)
		distanceEpsilon);							// error bound
}

int X4::AnnSearch::getkNearestNeighborsInRadius( const ANNpoint &searchPoint,const card32 &k, const float32 &radius, float *distances, int32 *indexes )const
{
	x4Assert(k<=numPoints);
	return kdTree->annkFRSearch(					
		searchPoint,						// query point
		radius,
		k,
		indexes,							// nearest neighbors (returned)
		distances,							// distance (returned)
		distanceEpsilon);							// error bound
}

void X4::AnnSearch::reset()
{
	delete kdTree;
	kdTree = NULL;
	numPoints=0;
}