#include "StdAfx.h"
#include "AttachedFloatSignatures.h"
#include "CopyObjectProperties.h"

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


IMPLEMENT_X4_CLASS( AttachedFloatSignatures, 1 )
{
	BEGIN_CLASS_INIT( AttachedFloatSignatures );

}

AttachedFloatSignatures::AttachedFloatSignatures(void)
{
}

AttachedFloatSignatures::~AttachedFloatSignatures(void)
{
}

void X4::AttachedFloatSignatures::assign( const Object* obj, X4CopyContext *context /*= NULL*/ )
{
	const AttachedFloatSignatures * o = dynamic_cast<const AttachedFloatSignatures*>(obj);
	x4Assert( o != NULL );
	this->descriptors = o->descriptors;
	this->distances = o->distances;
	copyObjectProperties(obj,this);
}

void X4::AttachedFloatSignatures::write( OutputObjectStream *s ) const
{
	Persistent::write(s);
	card64 size = descriptors.size();
	s->writeBuffer(&size, sizeof(card64));
	for (mpcard i = 0; i < descriptors.size(); i++)
		descriptors[i].write(s);

	size = distances.size();
	s->writeBuffer(&size, sizeof(card64));
	for (mpcard i = 0; i < distances.size(); i++)
		distances[i].write(s);

	
}

void X4::AttachedFloatSignatures::read( InputObjectStream *s )
{
	Persistent::read(s);
	card64 numDescr = 0;
	s->readBuffer(&numDescr, sizeof(card64));
	descriptors.resize(numDescr);
	for (mpcard i = 0; i < numDescr; i++)
		descriptors[i].read(s);

	if (s->getClassVersion(AttachedFloatSignatures::getClass()) >= MetaClass::FIRST_VERSION + 1) {
		card64 numDist = 0;
		s->readBuffer(&numDist, sizeof(numDist));
		distances.resize(numDist);
		for (mpcard i = 0; i < numDist; i++)
			distances[i].read(s);
	}
}

void X4::AttachedFloatSignatures::computeDistanceTable()
{
	const mpcard numDescriptors = descriptors.size();	
	debugOutput << "precomputing " << (numDescriptors*numDescriptors)/2 <<" pairwise distances\n";
	debugOutput.flush();


	distances.resize(descriptors.size());
	for (mpcard i = 0; i < numDescriptors; i++)
	{
		distances[i].setDim(i);
		for (mpcard j = 0; j < i; j++)
		{
			distances[i][j] = norm(descriptors[i]-descriptors[j]);
		}
	}
	debugOutput << "precomputing finished!\n";
	debugOutput.flush();

	
}

X4::float32 X4::AttachedFloatSignatures::getDistance( const mpcard &a, const mpcard &b ) const
{
	if (a==b)
		return 0.0f;
	
	if (a > b)
		return distances[a][b];
	else
		return distances[b][a];

}


//-----------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( AttachedFloatVector, 0 )
{
	BEGIN_CLASS_INIT( AttachedFloatVector );

}

AttachedFloatVector::AttachedFloatVector(void)
{
}

AttachedFloatVector::~AttachedFloatVector(void)
{
}

void X4::AttachedFloatVector::assign( const Object* obj, X4CopyContext *context /*= NULL*/ )
{
	const AttachedFloatVector * o = dynamic_cast<const AttachedFloatVector*>(obj);
	x4Assert( o != NULL );
	this->data = o->data;
	copyObjectProperties(obj,this);
}

void X4::AttachedFloatVector::write( OutputObjectStream *s ) const
{
	Persistent::write(s);
	card64 size = data.size();
	s->writeBuffer(&size, sizeof(card64));
	for (mpcard i = 0; i < data.size(); i++)
	{
		float f = data[i];
		s->writeBuffer(&f, sizeof(float));
	}
}

void X4::AttachedFloatVector::read( InputObjectStream *s )
{
	Persistent::read(s);
	card64 numDescr = 0;
	s->readBuffer(&numDescr, sizeof(card64));
	data.resize(numDescr);
	for (mpcard i = 0; i < numDescr; i++)
	{
		float f = 0;
		s->readBuffer(&f, sizeof(float));
		data[i] = f;
	}
}


//-----------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( AttachedCard32Vector, 0 )
{
	BEGIN_CLASS_INIT( AttachedCard32Vector );

}

AttachedCard32Vector::AttachedCard32Vector(void)
{
}

AttachedCard32Vector::~AttachedCard32Vector(void)
{
}

void X4::AttachedCard32Vector::assign( const Object* obj, X4CopyContext *context /*= NULL*/ )
{
	const AttachedCard32Vector * o = dynamic_cast<const AttachedCard32Vector*>(obj);
	x4Assert( o != NULL );
	this->data = o->data;
	copyObjectProperties(obj,this);
}

void X4::AttachedCard32Vector::write( OutputObjectStream *s ) const
{
	Persistent::write(s);
	card64 size = data.size();
	s->writeBuffer(&size, sizeof(card64));
	for (mpcard i = 0; i < data.size(); i++)
	{
		card32 f = data[i];
		s->writeBuffer(&f, sizeof(card32));
	}
}

void X4::AttachedCard32Vector::read( InputObjectStream *s )
{
	Persistent::read(s);
	card64 numDescr = 0;
	s->readBuffer(&numDescr, sizeof(card64));
	data.resize(numDescr);
	for (mpcard i = 0; i < numDescr; i++)
	{
	    card32 f = 0;
		s->readBuffer(&f, sizeof(card32));
		data[i] = f;
	}
}

//-----------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( AttachedInt32Vector, 0 )
{
	BEGIN_CLASS_INIT( AttachedInt32Vector );

}

AttachedInt32Vector::AttachedInt32Vector(void)
{
}

AttachedInt32Vector::~AttachedInt32Vector(void)
{
}

void X4::AttachedInt32Vector::assign( const Object* obj, X4CopyContext *context /*= NULL*/ )
{
	const AttachedInt32Vector * o = dynamic_cast<const AttachedInt32Vector*>(obj);
	x4Assert( o != NULL );
	this->data = o->data;
	copyObjectProperties(obj,this);
}

void X4::AttachedInt32Vector::write( OutputObjectStream *s ) const
{
	Persistent::write(s);
	card64 size = data.size();
	s->writeBuffer(&size, sizeof(card64));
	for (mpcard i = 0; i < data.size(); i++)
	{
		int32 f = data[i];
		s->writeBuffer(&f, sizeof(card32));
	}
}

void X4::AttachedInt32Vector::read( InputObjectStream *s )
{
	Persistent::read(s);
	card64 numDescr = 0;
	s->readBuffer(&numDescr, sizeof(card64));
	data.resize(numDescr);
	for (mpcard i = 0; i < numDescr; i++)
	{
		int32 f = 0;
		s->readBuffer(&f, sizeof(card32));
		data[i] = f;
	}
}
