//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "BRDFAttachment.h"
#include "ObjectClassProperty.h"
#include "SeparatorClassProperty.h"
#include "PropertyTableProperty.h"
#include "CopyObjectProperties.h"

//#include <glTools.h>
#include <GLTexture.h>

#include <time.h>
#include "scene.h"
#include <opencv\cxcore.h>

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( BRDFContainer, 0 )
{
	//BEGIN_CLASS_INIT( BRDFContainer); // comment out by CL @ 2014/04/17
	//INIT_PROPERTY_TABLE(); // comment out by CL @ 2014/04/17

	ADD_OBJECT_LIST_PROP(BRDF, 0, BRDFAttachment::getClass());
}

//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( BRDFAttachment, 0 )
{
	BEGIN_CLASS_INIT( BRDFAttachment );
	//NIT_PROPERTY_TABLE(); // comment out by CL @ 2014/04/17

	ADD_VECTOR3F_PROP(m_Normal, 0);
	ADD_CARD16_PROP(m_outPhiDomain, 0);
	ADD_CARD16_PROP(m_outThetaDomain, 0);
	ADD_CARD16_PROP(m_inPhiDomain, 0);
	ADD_CARD16_PROP(m_inThetaDomain, 0);
	ADD_CARD8_PROP(m_numChannels, 0);

	ADD_SEPARATOR("PCA");
	ADD_BOOLEAN_PROP(m_usePCAprojection, 0);

	ADD_SEPARATOR("Texture");
	ADD_BOOLEAN_PROP(m_use16bit, 0);
	ADD_BOOLEAN_PROP(m_isotrop, 0);
	ADD_CARD16_PROP(m_isotropOutPhi, 0);
}


//---------------------------------------------------------------------------
BRDFAttachment::BRDFAttachment()
{
	m_use16bit = true;
	m_isotrop = false;
	m_isotropOutPhi = 0;
	m_BRDF = NULL;
	m_usePCAprojection = false;
	m_Normal = makeVector3f(0,0,1);
	setDiscretization(1,1,1,1,1);
}

//---------------------------------------------------------------------------
BRDFAttachment::~BRDFAttachment()
{

}

//---------------------------------------------------------------------------
void BRDFAttachment::setDiscretization(card16 outPhi, card16 outTheta, card16 inPhi, card16 inTheta, card8 numChannels)
{
	if (m_BRDF) delete [] m_BRDF;
	m_BRDF = new float[outPhi*outTheta*inPhi*inTheta*numChannels];
	m_outPhiDomain = outPhi;
	m_outThetaDomain = outTheta;
	m_inPhiDomain = inPhi;
	m_inThetaDomain = inTheta;
	m_numChannels = numChannels;
}

//---------------------------------------------------------------------------
void BRDFAttachment::assign(const Object* obj, X4CopyContext *context)
{
	TextureAttachment::assign(obj,context);
	const BRDFAttachment* o = dynamic_cast<const BRDFAttachment*>(obj);
	copyObjectProperties(obj, this);

	setDiscretization(o->m_outPhiDomain, o->m_outThetaDomain, o->m_inPhiDomain, o->m_inThetaDomain, m_numChannels);
	memcpy(m_BRDF, o->m_BRDF, getMemorySize());
}

//---------------------------------------------------------------------------
void BRDFAttachment::write(OutputObjectStream *s) const
{
	AttachedData::write(s);
	s->writeBuffer(m_BRDF, getMemorySize());
}

//---------------------------------------------------------------------------
void BRDFAttachment::read(InputObjectStream *s)
{
	AttachedData::read(s);
	setDiscretization(m_outPhiDomain, m_outThetaDomain, m_inPhiDomain, m_inThetaDomain, m_numChannels);
	s->readBuffer(m_BRDF, getMemorySize());
}

//---------------------------------------------------------------------------
BRDFAttachment& BRDFAttachment::operator+=(const BRDFAttachment& b)
{
	x4Assert(!(m_inPhiDomain != b.m_inPhiDomain || m_inThetaDomain != b.m_inThetaDomain || m_outPhiDomain != b.m_outPhiDomain || m_outThetaDomain != b.m_outThetaDomain));
	for (size_t i=0; i < getNumEntries(); i++)
		m_BRDF[i] += b.m_BRDF[i];
	return *this;
}

//---------------------------------------------------------------------------
BRDFAttachment& BRDFAttachment::operator-=(const BRDFAttachment& b)
{
	x4Assert(!(m_inPhiDomain != b.m_inPhiDomain || m_inThetaDomain != b.m_inThetaDomain || m_outPhiDomain != b.m_outPhiDomain || m_outThetaDomain != b.m_outThetaDomain));
	for (size_t i=0; i < getNumEntries(); i++)
		m_BRDF[i] -= b.m_BRDF[i];
	return *this;
}

//---------------------------------------------------------------------------
BRDFAttachment& BRDFAttachment::operator*=(float v)
{
	for (size_t i=0; i < getNumEntries(); i++)
		m_BRDF[i] *= v;
	return *this;
}

//---------------------------------------------------------------------------
float BRDFAttachment::operator*=(const BRDFAttachment& b) const
{
	x4Assert(!(m_inPhiDomain != b.m_inPhiDomain || m_inThetaDomain != b.m_inThetaDomain || m_outPhiDomain != b.m_outPhiDomain || m_outThetaDomain != b.m_outThetaDomain));

	float sum = 0;
	for (size_t i=0; i < getNumEntries(); i++)
		sum += m_BRDF[i] * b.m_BRDF[i];
	return sum;
}

//---------------------------------------------------------------------------
float& BRDFAttachment::at(card16 outphi, card16 outtheta, card16 inphi, card16 intheta, card8 channel)
{
	return m_BRDF[(outtheta * m_outPhiDomain * m_inPhiDomain * m_inThetaDomain + outphi * m_inPhiDomain * m_inThetaDomain + intheta*m_inPhiDomain + inphi) * m_numChannels + channel];
}

//---------------------------------------------------------------------------
BRDFAttachment::DiscreteUniformPolarCoordinates BRDFAttachment::getCoordinatesForOutputDirection(Vector3f _dir)
{
	// normalize direction std::vector
	Vector3f dir = _dir;
	norm(dir);

	// convert given direction into polar coordinates (here assume our polar axis is 0,0,1)
	float phi = acos(dir[2]);
	float theta = dir[0] > 0 ? M_PI*0.5 : -M_PI*0.5;
	if (fabs(dir[0]) > 1e-10) 
		theta = atan(dir[1] / dir[0]);


	return DiscreteUniformPolarCoordinates((card16)phi, (card16)theta);
}

//---------------------------------------------------------------------------
GLTexture3D* BRDFAttachment::createBRDFTexture(GLContext* glContext, bool makeIsotrop, card16 isotropOutPhi, bool use16bit)
{ 
	if (m_usePCAprojection) return NULL;
	if (m_numChannels > 4)
		error(__FUNCTION__ ": do only support BRDF textures with 0,1,2,3 or 4 channel");

	unsigned size = m_numChannels * m_outThetaDomain * m_inPhiDomain * m_inThetaDomain;
	if (makeIsotrop == false) size *= m_outPhiDomain;

	// create temporary data, holding the full dense BRDF
	float* data = new float[size];
	memset(data, 0, sizeof(float) * size);

	// happy indexing ;)
	float* ptr = data;
	for (size_t i=0; i < getNumEntries(); i++)
	{
		card32 inphi = i % m_inPhiDomain;
		card32 intheta = (card32)i / m_inPhiDomain;
		card32 outphi = (card32)i / (m_inPhiDomain * m_inThetaDomain);
		card32 outtheta = (card32)i / (m_inPhiDomain * m_inThetaDomain * m_outPhiDomain);
		if (!makeIsotrop) isotropOutPhi = outphi;
		if (outphi == isotropOutPhi)
			*ptr++ = m_BRDF[i];
	}

	// create texture and upload data to GPU
	GLTexture3D* tex = new GLTexture3D(glContext);
	tex->createFloatTextureFromMemory(m_inPhiDomain, m_inThetaDomain, makeIsotrop ? m_outThetaDomain : m_outThetaDomain * m_outPhiDomain, m_numChannels, (char*)data, use16bit, false);
	delete data;
	return tex;
}

//---------------------------------------------------------------------------
bool BRDFAttachment::reconstructFromPCAbasis(const std::vector<BRDFAttachment*>& pcaBasis)
{
	if (m_usePCAprojection == false || pcaBasis.size() == 0 || m_pcaBasisScalar.size() != pcaBasis.size())
		return false;

	setDiscretization(pcaBasis[0]->m_outPhiDomain, pcaBasis[0]->m_outThetaDomain, pcaBasis[0]->m_inPhiDomain, pcaBasis[0]->m_inThetaDomain, pcaBasis[0]->getNumChannels());
	for (unsigned i=0; i < pcaBasis.size(); i++)
		*this += (*pcaBasis[i]) * m_pcaBasisScalar[i];
	return true;
}

//---------------------------------------------------------------------------
void BRDFAttachment::reduceToPCAbasis(const std::vector<BRDFAttachment*>& pcaBasis)
{
	if (m_usePCAprojection == true)
		error(__FUNCTION__ ": currently cannot reproject BRDF to another PCA space. Need to do this in the future!");

	m_pcaBasisScalar.resize(pcaBasis.size(), 0);

	// project BRDF to every pca basis BRDF (just a dot-product)
	for (unsigned i=0; i < pcaBasis.size(); i++)
	{
		m_pcaBasisScalar[i] = (*pcaBasis[i]) * (*this);
	}

	if (m_BRDF) delete [] m_BRDF;
	m_BRDF = NULL;
	m_usePCAprojection = true;
}

//---------------------------------------------------------------------------
card32 BRDFAttachment::pcaAnalysis(const std::vector<BRDFAttachment*>& brdfs, std::vector<BRDFAttachment*>& result, card32 maxNumBRDFS, float eigenValueThreshold /*= 1e-5f*/)
{
	unsigned dimX, dimY, dimZ, dimW, numChannels;

	// check that all BRDFs has the same dimensionality
	for (unsigned i=0; i < brdfs.size(); i++)
	{
		if (i==0)
		{
			dimX = brdfs[i]->getInPhiDomainSize();
			dimY = brdfs[i]->getInThetaDomainSize();
			dimZ = brdfs[i]->getOutPhiDomainSize();
			dimW = brdfs[i]->getOutThetaDomainSize();
			numChannels = brdfs[i]->getNumChannels();

			continue;
		}

		if (brdfs[i]->getInPhiDomainSize() != dimX) return false;
		if (brdfs[i]->getInThetaDomainSize() != dimY) return false;
		if (brdfs[i]->getOutPhiDomainSize() != dimZ) return false;
		if (brdfs[i]->getOutThetaDomainSize() != dimW) return false;
		if (brdfs[i]->getNumChannels() != numChannels) return false;
	}

	// currently only works if number of given BRDFS is smaller than BRDF dimension
	if (brdfs.size() > dimX*dimY*dimZ*dimW*numChannels) error(__FUNCTION__ ": currently only implemented if number of BRDFs is less then its dimension");
	result.clear();

	// TODO: precompute amount of memory required to run the algorithm

	// compute centroid 
	BRDFAttachment* centroid = new BRDFAttachment();
	centroid->setDiscretization(dimZ, dimW, dimX, dimY, numChannels);
	for (unsigned i=0; i < brdfs.size(); i++)
		*centroid += *brdfs[i];
	*centroid *= 1.0 / float(brdfs.size());
	
	unsigned dim = dimX*dimY*dimZ*dimW*numChannels;

	// compute M matrix, this contains all the BRDfs
	CvMat* M = cvCreateMat(dim, (int)brdfs.size(), CV_32FC1);
	for (unsigned i=0; i < brdfs.size(); i++)
		for (unsigned j=0; j < dim; j++)
			cvmSet(M, j, i, brdfs[i]->m_BRDF[j] - centroid->m_BRDF[j]);
	delete centroid;

	// compute SVD (here compute for the part which gives less dimensions)
	//{
		CvMat* mat = cvCreateMat((int)brdfs.size(), (int)brdfs.size(), CV_32FC1);
		cvMulTransposed(M, mat, 1);

		CvMat* D  = cvCreateMat((int)brdfs.size(), 1, CV_32FC1);
		CvMat* V  = cvCreateMat((int)brdfs.size(), (int)brdfs.size(), CV_32FC1);

		cvSVD(mat, D, NULL, V, /*CV_SVD_V_T|*/CV_SVD_MODIFY_A /*|CV_SVD_U_T|*/ ); // A = U D V^T
		cvReleaseMat(&mat);

		// scale V by computed inverted eigenvalues
		unsigned numValidEigenvalues = 0;
		for (unsigned i=0; i < brdfs.size(); i++)
		{
			// truncate eigenvalues
			float eigenvalue = sqrt(cvmGet(D, i, 0));
			float scale = 0;
			if (eigenvalue < eigenValueThreshold) 
				eigenvalue = 0;
			else
			{
				scale = 1.0 / eigenvalue;
				numValidEigenvalues++;
			}
			for (unsigned j=0; j < brdfs.size(); j++)
				cvmSet(V, i, j, cvmGet(V, i, j) * scale);
		}

		// compute U matrix, which represents the eigenvectors
		CvMat* U = cvCreateMat(dim, (int)brdfs.size(), CV_32FC1);
		cvMatMul(M, V, U);
		cvReleaseMat(&M);
		cvReleaseMat(&V);
		
		// extract only required/valid amount of principal components
		unsigned numResult = std::min<unsigned>(numValidEigenvalues, maxNumBRDFS);
		result.resize(numResult, NULL);
		for (unsigned i=0; i < numResult; i++)
		{
			result[i] = new BRDFAttachment();
			result[i]->setDiscretization(dimZ, dimW, dimX, dimY, numChannels);
			for (unsigned j=0; j < dim; j++)
				result[i]->m_BRDF[j] = cvmGet(U, j, i);
		}

		// free remaining memory
		cvReleaseMat(&U);
		cvReleaseMat(&D);
//	}

	return numResult;
}


