#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "BRDFShaderMaterial.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "VertexDescriptor.h"
#include "Shader.h"
#include "GLShader.h"
#include "DebugOutput.h"
#include "PointCloudIterators.h"
#include "GLAttachments.h"
#include "SceneObject.h"
#include "GLTexture.h"
#include "PointCloud.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( BRDFShaderMaterial , 0)
{
	BEGIN_CLASS_INIT( BRDFShaderMaterial );

}

//---------------------------------------------------------------------------
BRDFShaderMaterial::BRDFShaderMaterial(void)
{

}

//---------------------------------------------------------------------------
BRDFShaderMaterial::~BRDFShaderMaterial(void)
{
}

//---------------------------------------------------------------------------
void BRDFShaderMaterial::assign(const Object* obj, X4CopyContext *context)
{
	GLShaderMaterial::assign(obj,context);
	const BRDFShaderMaterial * o = dynamic_cast<const BRDFShaderMaterial*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

//---------------------------------------------------------------------------
void BRDFShaderMaterial::setShaderParameters(Scene * scene, SceneObject * obj, SceneObjectIterator *iter, GLContext * context, MemoryManager * mm)
{
	GLShaderMaterial::setShaderParameters(scene,obj,iter,context,mm);
	BRDFContainer* container = dynamic_cast<BRDFContainer*>(obj->getAttachments()->getData(BRDFContainer::getDefaultName()));
	PointCloud* pc = dynamic_cast<PointCloud*>(obj);
	if (container == NULL || pc == NULL) return;

	// setup attributes accordingly
	{
		// clear all attributes which has something to do, with the brdf
		VertexDescriptor* vd = (VertexDescriptor*)getVertexDescriptor()->copy();
		getVertexDescriptor()->clear();
		for (unsigned i=0; i < vd->getNumAttributes(); i++)
		{
			if (strstr(vd->getAttribute(i).name.c_str(), "brdf_") == NULL)
			{
				getVertexDescriptor()->pushAttrib(vd->getAttribute(i));
			}
		}
		delete vd;

		// copy all pc's attributes, which begins with brdf to the shader attribute list
		const VertexDescriptor* pcvd = pc->getDescr();
		for (unsigned i=0; i < pcvd->getNumAttributes(); i++)
		{
			// TODO: this should begin with the same substd::string as in HighQualityRenderer where this will be generated
			if (strstr(pcvd->getAttribute(i).name.c_str(), "brdf_") != NULL)
			{
				if (!getVertexDescriptor()->providesAttribute(pcvd->getAttribute(i).name))
				{
					getVertexDescriptor()->pushAttrib( pcvd->getAttribute(i));		
				}
			}
		}

	}


}
