#ifndef BOUNDED_FIFO_STACK_H
#define BOUNDED_FIFO_STACK_H

// System
#include <assert.h>

namespace X4{

/**
 * Provides a stack of arbitrary items
 */
template <class ItemType, unsigned int Bound>
class  BoundedFifoStack
{
	private:
      /// Index of top element
      long top;
	  /// Index of bottom element
	  long bottom;
      /// Item container
      ItemType items[Bound];

	public:
		/// Constructor
		BoundedFifoStack();
		/// Destructor
		~BoundedFifoStack();
		/// Empty stack
		void clear();
		/// Put new item in top of stack
		void push( ItemType item );
		/// Pop lowest item from stack
		ItemType pop();
		/// Test if stack is empty
		inline bool empty() const { return ( bottom > top ); }
		/// Debugging purpose
		void printDebugInfo() const;
		/// Return number of items on stack
		inline int getNumItems() const { return ( top - bottom + 1 ); }

		/// Special function for convex hull
		inline ItemType getTop(){ return items[top]; }
		/// Special function for convex hull
		inline ItemType getSecond(){ return items[top-1]; }

};

/**
 * Create new object
 */
template<class ItemType, unsigned int Bound>
BoundedFifoStack<ItemType,Bound>::BoundedFifoStack()
{
	clear();
}

/**
 * Delete object
 */
template<class ItemType, unsigned int Bound>
BoundedFifoStack<ItemType,Bound>::~BoundedFifoStack()
{
}

/**
 * Empty stack
 */
template<class ItemType, unsigned int Bound>
void BoundedFifoStack<ItemType,Bound>::clear()
{
   top = -1;
   bottom = 0;
}

/**
 * Put new item on top of stack
 */
template<class ItemType, unsigned int Bound>
void BoundedFifoStack<ItemType,Bound>::push( ItemType item )
{
   top++;
   items[top] = item;
}

/**
 * Remove the lowest element in the stack and return item
 */
template<class ItemType, unsigned int Bound>
ItemType BoundedFifoStack<ItemType,Bound>::pop()
{
   bottom++;
   return items[bottom-1];
}

/**
 * Print information about the state of the stack
 */
template<class ItemType, unsigned int Bound>
void BoundedFifoStack<ItemType,Bound>::printDebugInfo() const
{	
	std::cout << "* BoundedFifoStack state * " << endl;
	std::cout << "  Top element:    " << top << endl;
	std::cout << "  Bottom element:    " << bottom << endl;
	std::cout << "  Number of items: " << getNumItems() << endl;
	std::cout << "  Items: ";
	for ( long n = bottom; n <= top; n++ ){
		std::cout << items[n] << " ";
	}
	std::cout << endl;
}

}

#endif
