#ifndef BoundedFiloStackH
#define BoundedFiloStackH

// System
#include <assert.h>

namespace X4 {

/**
 * Provides a stack of arbitrary items
 */
template <class ItemType, unsigned int Bound>
class X3FOUNDATION_PRIVATE_API BoundedFiloStack
{
	private:
      /// Index of top element
      long top;
      /// Item container
      ItemType items[Bound];

	public:
		/// Constructor
		BoundedFiloStack();
		/// Destructor
		~BoundedFiloStack();
		/// Empty stack
		void clear();
		/// Put new item in top of stack
		void push( ItemType item );
		/// Pop lowest item from stack
		ItemType pop();
		/// Test if stack is empty
		inline bool empty() const { return ( top < 0 ); }
		/// Debugging purpose
		void printDebugInfo() const;
		/// Return number of items on stack
		inline int getNumItems() const { return ( top + 1 ); }

		/// Special function for convex hull
		inline ItemType getTop(){ return items[top]; }
		/// Special function for convex hull
		inline ItemType getSecond(){ return items[top-1]; }

};

/**
 * Create new object
 */
template<class ItemType, unsigned int Bound>
BoundedFiloStack<ItemType,Bound>::BoundedFiloStack()
{
	clear();
}

/**
 * Delete object
 */
template<class ItemType, unsigned int Bound>
BoundedFiloStack<ItemType,Bound>::~BoundedFiloStack()
{
}

/**
 * Empty stack
 */
template<class ItemType, unsigned int Bound>
void BoundedFiloStack<ItemType,Bound>::clear()
{
   top = -1;
}

/**
 * Put new item on top of stack
 */
template<class ItemType, unsigned int Bound>
void BoundedFiloStack<ItemType,Bound>::push( ItemType item )
{
   top++;
   items[top] = item;
}

/**
 * Remove the lowest element in the stack and return item
 */
template<class ItemType, unsigned int Bound>
ItemType BoundedFiloStack<ItemType,Bound>::pop()
{
	assert( top >= 0 );
	top--;
	return items[top+1];
}

/**
 * Print information about the state of the stack
 */
template<class ItemType, unsigned int Bound>
void BoundedFiloStack<ItemType,Bound>::printDebugInfo() const
{	
	std::cout << "* BoundedFiloStack state * " << endl;
	std::cout << "  Top element:    " << top << endl;
	std::cout << "  Number of items: " << getNumItems() << endl;
	std::cout << "  Items: ";
	for ( long n = 0; n <= top; n++ ){
		std::cout << items[n] << " ";
	}
	std::cout << endl;
}

}

#endif
