#ifndef BOUNDED_SET_H
#define BOUNDED_SET_H

// System
#include <assert.h>

// Provides a unsorted set with a maximum of 'MaxSize' items.
template <class ElementType, int MaxSize>
class BoundedSet
{
	private:
		/// Container for the elements
		ElementType elements[MaxSize];
		/// Number of elements in the set
		unsigned int numElements;

	public:
		/// Constructor
		BoundedSet();
		/// Destructor
		~BoundedSet();
		/// Empty the set
		void clear();
		/// Insert item into set
		bool insert( ElementType item );
		/// Return number of elements in the set
		inline unsigned int getNumElements(){ return numElements; };
      /// Access
      inline ElementType operator[]( unsigned int index ){ return elements[index]; }
		/// Test if element is in set
		bool contains( ElementType element );
		/// Create stream from bounded set
		friend ostream& operator<<( ostream& os, const BoundedSet<ElementType,MaxSize>& set )
		{
			os << "( "; 
			for ( unsigned int n = 0; n < set.numElements; n++ ) 
				os << set.elements[n] << " "; 
			os << ")";
			return os;
		}
};

/**
 * Create new object
 */
template<class ElementType, int MaxSize>
BoundedSet<ElementType,MaxSize>::BoundedSet()
{
	clear();
}

/**
 * Delete object
 */
template<class ElementType, int MaxSize>
BoundedSet<ElementType,MaxSize>::~BoundedSet()
{
}

/**
 * Set nummer of elements to zero
 */
template<class ElementType, int MaxSize>
void BoundedSet<ElementType,MaxSize>::clear()
{
	numElements = 0;
}

/**
 * Return true, if possible
 */
template<class ElementType, int MaxSize>
bool BoundedSet<ElementType,MaxSize>::insert( ElementType element )
{
	if ( ( numElements >= MaxSize ) || contains( element ) )
		return false;

	elements[numElements] = element;
	numElements++;
	return true;
}

/**
 * Return true, if element is already within the set
 */
template<class ElementType, int MaxSize>
bool BoundedSet<ElementType,MaxSize>::contains( ElementType element )
{
	for ( unsigned int n = 0; n < numElements; n++ )
		if ( elements[n] == element )
			return true;
	return false;
}

#endif