#ifndef BOUNDED_SORTED_SET_H
#define BOUNDED_SORTED_SET_H

// System
#include <assert.h>

// Provides a unsorted set with a maximum of 'MaxSize' items.
template <class ElementType, int MaxSize>
class BoundedSortedSet
{
	private:
		/// Container for the elements
		ElementType elements[MaxSize];
		/// Number of elements in the set
		unsigned int numElements;

	public:
		/// Constructor
		BoundedSortedSet();
		/// Destructor
		~BoundedSortedSet();
		/// Empty the set
		void clear();
		/// Insert item into set
		bool insert( ElementType item );
      /// Remove item from set
		bool remove( ElementType item );
		/// Return number of elements in the set
		inline unsigned int getNumElements(){ return numElements; };
      /// Access
      inline ElementType operator[]( unsigned int index ) const { return elements[index]; }
		ElementType & operator[]( unsigned int index ){ return elements[index]; }
		/// Test if element is in set
		bool contains( ElementType element );
};

/**
 * Create new object
 */
template<class ElementType, int MaxSize>
BoundedSortedSet<ElementType,MaxSize>::BoundedSortedSet()
{
	clear();
}

/**
 * Delete object
 */
template<class ElementType, int MaxSize>
BoundedSortedSet<ElementType,MaxSize>::~BoundedSortedSet()
{
}

/**
 * Set nummer of elements to zero
 */
template<class ElementType, int MaxSize>
void BoundedSortedSet<ElementType,MaxSize>::clear()
{
	numElements = 0;
}

/**
 * Return true, if possible
 */
template<class ElementType, int MaxSize>
bool BoundedSortedSet<ElementType,MaxSize>::insert( ElementType element )
{
	if ( ( numElements >= MaxSize ) || contains( element ) )
		return false;

   for ( unsigned int n = 0; n < numElements; n++ ){
      if ( element < elements[n] ){
         // Shift all element
         for ( unsigned int m = numElements; m > n; m-- ){
            elements[m] = elements[m-1];
         }
         // Insert new element
         elements[n] = element;
	      numElements++;
         return true;
      }
   }
   // Append element at end of set
	elements[numElements] = element;
	numElements++;
	return true;
}

/**
 * Return true, if possible
 */
template<class ElementType, int MaxSize>
bool BoundedSortedSet<ElementType,MaxSize>::remove( ElementType element )
{
   for ( unsigned int n = 0; n < numElements; n++ ){
      if ( element == elements[n] ){
         // Shift all element
         for ( unsigned int m = n; m > numElements-1; m++ ){
            elements[m] = elements[m+1];
         }
         numElements--;
         return true;
      }
   }
	return true;
}

/**
 * Return true, if element is already within the set
 */
template<class ElementType, int MaxSize>
bool BoundedSortedSet<ElementType,MaxSize>::contains( ElementType element )
{
	for ( unsigned int n = 0; n < numElements; n++ )
		if ( elements[n] == element )
			return true;
	return false;
}

#endif