#ifndef Circle3DH
#define Circle3DH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

namespace X4
{

/**
	circle in 3d space
*/
class X3FOUNDATION_PRIVATE_API Circle3D
{
public:

	Circle3D();
	
	/// try to construct a circle through 3 points
	/// returns true if succeded
	bool constructFromPoints(const Vector3f &p0,const Vector3f &p1,const Vector3f &p2);

	/// compute distance to circle 
	float distance(const Vector3f &p) const;

	/// returns true if circle is valid (all construction methods set radius to -1 when failing)
	inline bool valid() const {return m_Radius >= 0;}

	/// compute projection on circle (returns absolut angle 0..2pi, basically atan2)
	float computeProjection(const Vector3f &p) const;

	/// compute point on circle in world space (angle or from computeProjection)
	Vector3f computeCirclePoint(float angle) const;

	/// compute circle motion from 0 to angle
	Matrix4f computeT( float angle) const;

	/// draw with debug renderer
	void drawDR(const Vector3f &color, mpcard numSegments=15);

	/// flip orientation direction
	void flipOrientation() {m_Frame[2] *= -1; m_Frame[0] *=-1;}

	// interface
	Vector3f getCenter() const {return m_Center;}
	float getRadius() const {return m_Radius;}
	Matrix3f getFrame() const {return m_Frame;}
	void setup( const Matrix3f &frame, const Vector3f &center, float radius);


private:

	Matrix3f m_Frame, m_FrameT;	// circle lives in x,y dimension of frame (frameT = transposed)
	Vector3f	m_Center;
	float m_Radius;

};

}

#endif