#include "StdAfx.h"
//---------------------------------------------------------------------------
//#include <windows.h>
//#include <mmsystem.h>
#include "EyeCandy.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "GLMultipleRenderTexture.h"
#include "GLHelper.h"
#include "GLShaderMaterial.h"
#include "Configuration.h"
#include "Random.h"
#include "ObjectClassProperty.h"
#include "GLRenderTextureFBO.h"
#include "StandardSceneGraphIterator.h"
#include "Scene.h"
#include "SceneGraphNode.h"
#include "SGObjectNode.h"
#include "SGTransformationNode.h"
#include "SceneObject.h"
#include "MaterialAttachment.h"
#include "GLTexture.h"
#include "ClassMethods.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "TriangleBBIntersection.h"
#include "PointSet.h"
#include "PointCloudTools.h"
#include "ECPhotonTracer.h"
#include "VertexArray.h"
#include "DebugRenderer.h"
#include "MaterialParameter.h"
#include "MPContainer.h"
#include "RGBAImage.h"
#include "Timer.h"
#include "GLBuffer.h"
#include "GLAttachments.h"
#include "SeparatorClassProperty.h"
#include "QInputDialog"
#include "PCCResampler.h"
#include "ApplicationDirectories.h"

//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( EyeCandy ,0)
{
	BEGIN_CLASS_INIT( EyeCandy );
	ADD_OBJECT_PROP(m_Mat_RenderPosNormalColor,0,GLShaderMaterial::getClass(),true);
	ADD_OBJECT_PROP(m_Mat_RenderTextureToScreen,0,GLShaderMaterial::getClass(),true);
	ADD_OBJECT_PROP(m_Mat_RenderSSAO,0,GLShaderMaterial::getClass(),true);
	ADD_OBJECT_PROP(m_Mat_BlurSSAO,0,GLShaderMaterial::getClass(),true);
	ADD_OBJECT_PROP(m_Mat_FinalGather,0,GLShaderMaterial::getClass(),true);
	ADD_OBJECT_PROP(m_Mat_FinalGatherDepth,0,GLShaderMaterial::getClass(),true);
	ADD_OBJECT_PROP(m_Mat_SplatLight,0,GLShaderMaterial::getClass(),true);
	ADD_OBJECT_PROP(m_Mat_Combine,0,GLShaderMaterial::getClass(),true);

	ADD_SEPARATOR("light source");
	ADD_BOOLEAN_PROP(m_HemisphereLight,0);
	ADD_FLOAT32_PROP(m_LightStrength,0);
	ADD_BOOLEAN_PROP(m_SelectedPointsAsLightSource,0);
	ADD_BOOLEAN_PROP(m_IndirectLight,0);

	ADD_SEPARATOR("precision");
	ADD_FLOAT32_PROP(m_NumSamples,0);
	ADD_FLOAT32_PROP(m_NumPhotons,0);
	ADD_FLOAT32_PROP(m_Offset,0);

	ADD_SEPARATOR("view");
	ADD_FLOAT32_PROP(m_Gamma,0);
	ADD_FLOAT32_PROP(m_Brightness,0);
	ADD_BOOLEAN_PROP(m_AllowDebugRenderer,0);

	ADD_NOARGS_METHOD(sampleScene);
	ADD_NOARGS_METHOD(setupSSAOSamples);
}

EyeCandy::EyeCandy(void)
{
	m_AllowDebugRenderer = false;
	m_RenderTexture = NULL;
	m_RT_SSAO = NULL;
	m_RT_SSAO_Blurred = NULL;
	m_RT_FinalGather = NULL;
	m_RT_SplatLight = NULL;
	
	// setup materials
	m_Mat_RenderPosNormalColor = GLShaderMaterial::createFromShaderFile(ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/EC_RenderPosNormalColor.txt");
	m_Mat_RenderPosNormalColor->getVertexDescriptor()->pushAttrib(VAD("normal",3,VAD::DATA_FORMAT_FLOAT32));
	m_Mat_RenderPosNormalColor->getVertexDescriptor()->pushAttrib(VAD("color",3,VAD::DATA_FORMAT_FLOAT32));
	m_Mat_RenderPosNormalColor->setForceTriangleNormal(true);
	m_Mat_RenderPosNormalColor->setBackfaceCulling(true);
	m_Mat_RenderTextureToScreen = GLShaderMaterial::createFromShaderFile(ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/EC_DrawRTToScreen.txt");
	m_Mat_RenderSSAO = GLShaderMaterial::createFromShaderFile(ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/EC_SSAO.txt");
	m_Mat_BlurSSAO = GLShaderMaterial::createFromShaderFile(ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/EC_SSAO_blur.txt");
	m_Mat_FinalGather = GLShaderMaterial::createFromShaderFile(ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/EC_final_gather.txt");
	m_Mat_FinalGather->getVertexDescriptor()->pushAttrib(VAD("normal",3,VAD::DATA_FORMAT_FLOAT32));
	m_Mat_FinalGather->getVertexDescriptor()->pushAttrib(VAD("color",3,VAD::DATA_FORMAT_FLOAT32));
	m_Mat_FinalGather->getVertexDescriptor()->pushAttrib(VAD("screen_pos",2,VAD::DATA_FORMAT_FLOAT32));
	m_Mat_FinalGatherDepth = GLShaderMaterial::createFromShaderFile(ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/EC_final_gather_depth.txt");
	m_Mat_FinalGatherDepth->setPointSize(4);
	m_Mat_FinalGatherDepth_ScaleParm = new MPVector4;
	m_Mat_FinalGatherDepth_ScaleParm->setName("scaleZ");
	m_Mat_FinalGatherDepth->getParameters()->addParm(m_Mat_FinalGatherDepth_ScaleParm);
	
	m_Mat_Combine = GLShaderMaterial::createFromShaderFile(ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/EC_Combine.txt");
	m_Mat_SplatLight = GLShaderMaterial::createFromShaderFile(ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/EC_SplatLight.txt");
	m_Mat_SplatLight->getVertexDescriptor()->pushAttrib( VAD("normal",3,VAD::DATA_FORMAT_FLOAT32));
	m_Mat_SplatLight->getVertexDescriptor()->pushAttrib( VAD("color",3,VAD::DATA_FORMAT_FLOAT32));
	m_Mat_SplatLight->getVertexDescriptor()->pushAttrib( VAD("pcoord",1,VAD::DATA_FORMAT_FLOAT32));
	m_Mat_SplatLight->getVertexDescriptor()->pushAttrib( VAD("sampleSize",1,VAD::DATA_FORMAT_FLOAT32));

	m_RandomTexture = new RandomTextureAttachment;
	m_CubeShadowMap = NULL;

	m_NumSamples = 0;
	m_NumPhotons = 0;
	m_Offset = 0;
	m_LightStrength = 1.0f;
	m_HemisphereLight = true;
	m_SelectedPointsAsLightSource = false;
	m_IndirectLight = true;
	m_Gamma = 1.0f;
	m_Brightness = 1.0f;
}

EyeCandy::~EyeCandy(void)
{
	delete m_RenderTexture;
	delete m_Mat_RenderPosNormalColor;
	delete m_Mat_RenderTextureToScreen;
	delete m_Mat_RenderSSAO;
	delete m_RT_SSAO;
	delete m_RandomTexture;
	delete m_RT_SSAO_Blurred;
	delete m_Mat_BlurSSAO;
	delete m_CubeShadowMap;
	delete m_Mat_FinalGather;
	delete m_Mat_FinalGatherDepth;
	delete m_RT_FinalGather;
	delete m_Mat_Combine;
	delete m_Mat_SplatLight;
	delete m_RT_SplatLight;
}

void EyeCandy::assign(const Object* obj, X4CopyContext *context)
{
	Renderer::assign(obj,context);
	const EyeCandy * o = dynamic_cast<const EyeCandy*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void EyeCandy::connect(Scene *scene, GLContext* ctxt) 
{
	Renderer::connect(scene,ctxt);
}

// some debug stuff
void EC_drawQuad( const Vector3f &p1, const Vector3f &p2, const Vector3f &p3, const Vector3f &p4, const Vector3f &color )
{
	Vector3f normal = normalize( (p2-p1).crossProduct(p3-p1));
	glBegin(GL_QUADS);
		glColor3fv(color.data());
		glNormal3fv(normal.data());
		glVertex3fv(p1.data());
		glVertex3fv(p2.data());
		glVertex3fv(p3.data());
		glVertex3fv(p4.data());
	glEnd();
}

void EC_drawCube( const Vector3f &pos, const Matrix3f &orientation, const Vector3f &scale, const Vector3f &color )
{
	Vector3f c[8];
	c[0] = makeVector3f(0,0,0);
	c[1] = makeVector3f(1,0,0);
	c[2] = makeVector3f(1,0,1);
	c[3] = makeVector3f(0,0,1);
	c[4] = makeVector3f(0,1,0);
	c[5] = makeVector3f(1,1,0);
	c[6] = makeVector3f(1,1,1);
	c[7] = makeVector3f(0,1,1);

	for(int i=0;i<8;i++)
	{
		c[i] = orientation * (c[i].componentProduct(scale)) + pos;
	}

	EC_drawQuad( c[0],c[1],c[2],c[3], color );
	EC_drawQuad( c[7],c[6],c[5],c[4], color );
	EC_drawQuad( c[0],c[4],c[5],c[1], color );
	EC_drawQuad( c[3],c[2],c[6],c[7], color );
	EC_drawQuad( c[2],c[1],c[5],c[6], color );
	EC_drawQuad( c[0],c[3],c[7],c[4], color );
}

struct EC_TCube
{
	Vector3f pos;
	Matrix3f M;
	Vector3f color;
	Vector3f scale;
};

void EyeCandy::render(card32 viewportWidth, card32 viewportHeight) 
{
	if( !setupRenderTexture(viewportWidth,viewportHeight) )
	{
		glClearColor(1,0,0,1);
		glClear(GL_COLOR_BUFFER_BIT);
		return;
	}

	// ----------------------------------------------------------
	// render scene into buffers (mrt)
	m_RenderTexture->activate();
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	//m_Mat_RenderPosNormalColor->bindShaders( getGLContext() );

	StandardSceneGraphIterator sceneIt;
	sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, 0, 0);
	renderSceneRecursive(&sceneIt, IDENTITY4F, m_Mat_RenderPosNormalColor);
	//m_Mat_RenderPosNormalColor->unbindShaders( getGLContext() );
	glPopAttrib();
	m_RenderTexture->deactivate();


	// compute screen space ambient occlusion
	m_RT_SSAO->activate();
	m_Mat_RenderSSAO->bindShaders(getGLContext());
	m_RenderTexture->bind(3,0);
	GLTexture * tex = m_RandomTexture->getCachedTexture(getGLContext());
	tex->bind(1);
	drawScreenQuad();
	m_RenderTexture->unbindAll();
	m_Mat_RenderSSAO->unbindShaders(getGLContext());
	m_RT_SSAO->deactivate();

	// blur ssao
	m_RT_SSAO_Blurred->activate();
	m_Mat_BlurSSAO->bindShaders(getGLContext());
	m_RenderTexture->bind(1,0);	// bind normal texture
	m_RT_SSAO->bind(1);
	drawScreenQuad();
	m_RenderTexture->unbindAll();
	m_RT_SSAO->unbind();
	m_Mat_BlurSSAO->unbindShaders(getGLContext());
	m_RT_SSAO_Blurred->deactivate();


	// copy result to screen
	m_Mat_RenderTextureToScreen->bindShaders( getGLContext() );
	m_RT_SSAO_Blurred->bind();
	m_RenderTexture->bind(1,1);	// bind normal texture
	m_RenderTexture->bind(2,2);	// bind color texture
	drawScreenQuad();
	m_RT_SSAO_Blurred->unbind();
	m_RenderTexture->unbindAll();
	m_Mat_RenderTextureToScreen->unbindShaders( getGLContext() );

	return;

	// ----------------------------------------------------------
	// render light splats
	UnstructuredInCorePointCloud * lightSamples = dynamic_cast<UnstructuredInCorePointCloud*>
		( getPointCloud(getScene(),"root/lightSamples") );
	if( lightSamples )
	{
		m_RT_SplatLight->activate();
		glPushAttrib(GL_ALL_ATTRIB_BITS);
		glClearColor(0,0,0,0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glEnable(GL_DEPTH_TEST);

		// depth pass
		glColorMask(0,0,0,0);
		sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, 0, 0);
		renderSceneRecursive(&sceneIt, IDENTITY4F, (GLShaderMaterial*)getScene()->getGLMaterial(1));
		glColorMask(1,1,1,1);

		glEnable(GL_BLEND);
		glBlendFunc(GL_ONE,GL_ONE);
		glDepthMask(0);
		//m_Mat_SplatLight->setDefaultParameter("screenSize",makeVector4f(m_RenderTexture->getWidth(),m_RenderTexture->getHeight(),0,0));
		m_Mat_SplatLight->bindShaders(getGLContext());
		m_Mat_SplatLight->setParameter4fv(getGLContext(),"screenSize",makeVector4f(m_RenderTexture->getWidth(),m_RenderTexture->getHeight(),0,0).data());

		m_RenderTexture->bind(0,0);	// bind color texture
		m_RenderTexture->bind(1,1);	// bind normal texture

		SharedVertexBufferAttachment * vbuffer = getSplatVertexBuffer(lightSamples);
		vbuffer->getVertexBuffer()->bind();
		vbuffer->getVertexBuffer()->drawArrays( GL_QUADS );

		vbuffer->getVertexBuffer()->unbind();
		m_RenderTexture->unbindAll();
		m_Mat_SplatLight->unbindShaders(getGLContext());
		glDepthMask(1);
		glPopAttrib();
		m_RT_SplatLight->deactivate();
	}

	// ----------------------------------------------------------
	// copy result to screen
	m_Mat_Combine->bindShaders( getGLContext() );
	m_RT_SplatLight->bind(0);
	drawScreenQuad();
	m_RT_SplatLight->unbind();
	m_Mat_Combine->unbindShaders( getGLContext() );


	return;

	Matrix4f shadowMat = makeRotX4f(45.0f / 180.0f * (float)M_PI);
	renderShadowCube(m_CubeShadowMap,shadowMat);

	m_CubeShadowMap->outputViaImageDisplayer("cube");
	return;
	
	{
		glClearColor(0,0,0,1);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		m_CubeShadowMap->bind();
		StandardSceneGraphIterator sceneIt;
		sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, 0, 0);
		
		Matrix4f invFrame = invertFrame(shadowMat);
//		m_Mat_FinalGather->setMatrix4fv(getGLContext(), "matShadowMap", invFrame.data());
		
		renderSceneRecursive(&sceneIt, IDENTITY4F, m_Mat_FinalGather);
		m_CubeShadowMap->unbind();
	}

	return;

	// create random scene
	static std::vector<EC_TCube> scene;
	if(scene.size() == 0)
	{
		for( unsigned i=0;i<20;i++ )
		{
			EC_TCube cube;
			cube.pos = makeVector3f(rnd01(),0,rnd01() );
			cube.M = //makeRotX3f(rnd01() * M_PI * 2.0f) * makeRotY3f(rnd01() * M_PI * 2.0f) * makeRotZ3f(rnd01() * M_PI * 2.0f);
				makeRotY3f(rnd01() * M_PI * 2.0f);
			cube.scale = makeVector3f(0.1f,0.1f + rnd01()*.1f,0.1f);//makeVector3f(0.05f + rnd01() * 0.15f,0.05f + rnd01() * 0.15f,0.05f + rnd01() * 0.15f);
			cube.color = makeVector3f(rnd01(),rnd01(),rnd01() );
			scene.push_back(cube);
		}
		EC_TCube basecube;
		basecube.pos = makeVector3f(-2,-0.1f,-2);
		basecube.scale = makeVector3f(4,0.1f,4);
		basecube.M = IDENTITY3F;
		basecube.color = makeVector3f(1,1,1);
		scene.push_back(basecube);
	}

	// render scene into buffers
	m_RenderTexture->activate();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	m_Mat_RenderPosNormalColor->bindShaders( getGLContext() );
	if(0)
	for( unsigned i=0;i<scene.size();i++ )
	{
		EC_TCube c = scene[i];
		EC_drawCube(c.pos,c.M,c.scale,c.color);
	}

	sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, 0, 0);
	renderSceneRecursive(&sceneIt, IDENTITY4F, m_Mat_RenderPosNormalColor);


	m_Mat_RenderPosNormalColor->unbindShaders( getGLContext() );
	m_RenderTexture->deactivate();

// 	// compute screen space ambient occlusion
// 	m_RT_SSAO->activate();
// 	m_Mat_RenderSSAO->bindShaders(getGLContext());
// 	m_RenderTexture->bind(3,0);
// 	GLTexture * tex = m_RandomTexture->getCachedTexture(getGLContext());
// 	tex->bind(1);
// 	drawScreenQuad();
// 	m_RenderTexture->unbindAll();
// 	m_Mat_RenderSSAO->unbindShaders(getGLContext());
// 	m_RT_SSAO->deactivate();
// 
// 	// blur ssao
// 	m_RT_SSAO_Blurred->activate();
// 	m_Mat_BlurSSAO->bindShaders(getGLContext());
// 	m_RenderTexture->bind(1,0);	// bind normal texture
// 	m_RT_SSAO->bind(1);
// 	drawScreenQuad();
// 	m_RenderTexture->unbindAll();
// 	m_RT_SSAO->unbind();
// 	m_Mat_BlurSSAO->unbindShaders(getGLContext());
// 	m_RT_SSAO_Blurred->deactivate();
// 
// 
// 	// copy result to screen
// 	m_Mat_RenderTextureToScreen->bindShaders( getGLContext() );
// 	m_RT_SSAO_Blurred->bind();
// 	m_RenderTexture->bind(1,1);	// bind normal texture
// 	m_RenderTexture->bind(2,2);	// bind color texture
// 	drawScreenQuad();
// 	m_RT_SSAO_Blurred->unbind();
// 	m_RenderTexture->unbindAll();
// 	m_Mat_RenderTextureToScreen->unbindShaders( getGLContext() );

}

void EyeCandy::disconnect(GLContext* ctxt) 
{
	Renderer::disconnect(ctxt);
}

bool EyeCandy::setupRenderTexture( unsigned width, unsigned height ) 
{
// 	if( m_CubeShadowMap == NULL )
// 	{
// 		m_CubeShadowMap = new GLRenderTextureCube(getGLContext());
// 		if( !m_CubeShadowMap->createDepthCube(1024) )
// 		{
// 			return false;
// 			delete m_CubeShadowMap;
// 			m_CubeShadowMap = NULL;
// 		}
// 	}

	if( !m_RenderTexture || m_RenderTexture->getWidth() != width || m_RenderTexture->getHeight() != height )
	{
		delete m_RenderTexture;
		bool success = true;
		m_RenderTexture = new GLMultipleRenderTexture(getGLContext());
		m_RenderTexture->init(width,height,true,false);
		m_RenderTexture->addRenderTexture(3,true,true);	// position channel
		m_RenderTexture->addRenderTexture(3,true,true); // normal channel
		m_RenderTexture->addRenderTexture(3,true,true); // color channel
		m_RenderTexture->addRenderTexture(3,true,true); // extra channel (zbuffer,..,..)
		success &= m_RenderTexture->create();

		delete m_RT_SSAO;
		m_RT_SSAO = new GLRenderTextureFBO(getGLContext());
		success &= m_RT_SSAO->create( width, height,3,GLRenderTextureFBO::FLAG_RECTANGLE | GLRenderTextureFBO::FLAG_FLOAT16);
	
		delete m_RT_SSAO_Blurred;
		m_RT_SSAO_Blurred = new GLRenderTextureFBO(getGLContext());
		success &= m_RT_SSAO_Blurred->create( width, height,3,GLRenderTextureFBO::FLAG_RECTANGLE | GLRenderTextureFBO::FLAG_FLOAT16);

		delete m_RT_SplatLight;
		m_RT_SplatLight = new GLRenderTextureFBO(getGLContext());
		success &= m_RT_SplatLight->create( width, height,4,GLRenderTextureFBO::FLAG_RECTANGLE 
						| GLRenderTextureFBO::FLAG_FLOAT16
						| GLRenderTextureFBO::FLAG_DEPTH);

		if( !success )
		{
			delete m_RenderTexture; m_RenderTexture = NULL;
			delete m_RT_SSAO; m_RT_SSAO = NULL;
			delete m_RT_SSAO_Blurred;	m_RT_SSAO_Blurred = NULL;
			delete m_RT_SplatLight; m_RT_SplatLight = NULL;
			return false;
		}
	}
	return true;
}

void EyeCandy::drawScreenQuad() 
{
	glPushAttrib( GL_ALL_ATTRIB_BITS);
	GLHelper::begin2D(1,1);
	glDisable( GL_DEPTH_TEST );

	float x0 = 0;
	float x1 = 1;
	float y0 = 0;
	float y1 = 1;
	glBegin( GL_QUADS );
	glColor3f(1,1,1);
	glTexCoord2f( 0,0 );
	glVertex2f(x0,y0);

	glTexCoord2f( m_RenderTexture->getWidth(),0 );
	glVertex2f(x1,y0);

	glTexCoord2f( m_RenderTexture->getWidth(),m_RenderTexture->getHeight() );
	glVertex2f(x1,y1);

	glTexCoord2f( 0,m_RenderTexture->getHeight() );
	glVertex2f(x0,y1);

	glEnd();
	GLHelper::end2D();
	glPopAttrib();
}

void EyeCandy::renderSceneRecursive( StandardSceneGraphIterator * sceneIter, const Matrix4f &modelMatrix, GLShaderMaterial * mat )
{
	if (!sceneIter->getNode()->visible()) return;

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glMultMatrixf(sceneIter->getState()->dynamicState->ctm.data());
	Matrix4f currentModelMatrix = modelMatrix;
	SGTransformationNode * tnode = dynamic_cast<SGTransformationNode*>(sceneIter->getNode());
	if( tnode )
	{
		currentModelMatrix = currentModelMatrix * tnode->getTransformation();
	}

	SGObjectNode *objNode = dynamic_cast<SGObjectNode*>(sceneIter->getNode());
	if (objNode != NULL) {
		SceneObject* obj = objNode->getSceneObject();

		if (obj != NULL && objNode->visible()) {

			SceneGraphState *state = sceneIter->getState();
			SceneObjectIterator *it = obj->createIterator(SOIT(SOIT::CAP_HIERARCHICAL | SOIT::CAP_ATTACH_SECONDARY));

			if( mat != NULL )
				mat->draw( getScene(), state, obj, it, getGLContext());
			else
				getScene()->getGLMaterial(2)->draw( getScene(), state, obj, it, getGLContext());
			
			delete it;
		}
	}

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	unsigned numChildren = sceneIter->getNumChildren();
	for( unsigned i=0;i<numChildren;i++ )
	{
		sceneIter->goToChild(i);
		renderSceneRecursive(sceneIter,currentModelMatrix,mat);
		sceneIter->goToParent();
	}

}

SharedVertexBufferAttachment * EyeCandy::getSplatVertexBuffer( UnstructuredInCorePointCloud * lightSamples )
{
	SharedVertexBufferAttachment * vbuffer = dynamic_cast<SharedVertexBufferAttachment*>(
		lightSamples->getAttachments()->getData( "lightSampleVB" ) );
	if( !vbuffer )
	{
		vbuffer = new SharedVertexBufferAttachment;

		AAT posAAT = lightSamples->getAAT("position");
		AAT normalAAT = lightSamples->getAAT("normal");
		AAT colorAAT = lightSamples->getAAT("color");
		AAT sampleSizeAAT = lightSamples->getAAT("sampleSize");

		PointSet tmpPS;
		tmpPS.clearAndSetup(1,4 * lightSamples->getNumPoints(),m_Mat_SplatLight->getVertexDescriptor());
		AAT dst_posAAT = tmpPS.getAAT("position");
		AAT dst_normalAAT = tmpPS.getAAT("normal");
		AAT dst_colorAAT = tmpPS.getAAT("color");
		AAT dst_pcoordAAT = tmpPS.getAAT("pcoord");
		AAT dst_sampleSizeAAT = tmpPS.getAAT("sampleSize");
		for( unsigned i=0;i<lightSamples->getNumPoints();i++ )		 
		{

			for( unsigned j=0;j<4;j++ )
			{
				unsigned index = i*4+j;
				tmpPS.set3f(index,dst_posAAT, lightSamples->getPointSet()->get3f(i,posAAT));
				tmpPS.set3f(index,dst_colorAAT, lightSamples->getPointSet()->get3f(i,colorAAT));
				tmpPS.set3f(index,dst_normalAAT, lightSamples->getPointSet()->get3f(i,normalAAT));
				tmpPS.set1f(index,dst_sampleSizeAAT, lightSamples->getPointSet()->get1f(i,sampleSizeAAT));
				float pcoord;
				switch(j)
				{
				case 0:
					pcoord = -1;
					break;
				case 1:
					pcoord =  1;
					break;
				case 2:
					pcoord =  2;
					break;
				case 3:
					pcoord = -2;
					break;
				}
				tmpPS.set1f(index,dst_pcoordAAT,pcoord);
			}
		}

		GLVertexBuffer * glvb = new GLVertexBuffer(getGLContext(), *m_Mat_SplatLight->getVertexDescriptor());
		glvb->init(tmpPS.getNumEntries());
		glvb->writeToBuffer( tmpPS.getDataPointer(),0, tmpPS.getNumEntries());
		vbuffer->setupVB(glvb, "lightSampleVB",NULL,true,0,0,NULL_VECTOR3F, IDENTITY4F);
		vbuffer->setup("lightSampleVB",AttachedData::ADF_DELETEONUPDATE);
		lightSamples->getAttachments()->attachData(vbuffer);
	}
	return vbuffer;
}
void gatherPoints( StandardSceneGraphIterator * sceneIter, const Matrix4f &modelMatrix, ECResampleOctree* result)
{
	if (!sceneIter->getNode()->visible()) return;

	Matrix4f currentModelMatrix = modelMatrix;
	SGTransformationNode * tnode = dynamic_cast<SGTransformationNode*>(sceneIter->getNode());
	if( tnode )
	{
		currentModelMatrix = currentModelMatrix * tnode->getTransformation();
	}

	SGObjectNode *objNode = dynamic_cast<SGObjectNode*>(sceneIter->getNode());
	if (objNode != NULL) {
		SceneObject* obj = objNode->getSceneObject();

		if (obj != NULL && objNode->visible()) 
		{
			UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(obj);
			UnstructuredInCoreTriangleMesh * mesh = dynamic_cast<UnstructuredInCoreTriangleMesh*>(obj);
			Matrix4f tmpM = currentModelMatrix * obj->getTransformation();
			if( upc && !mesh)
			{
				AAT posAAT = upc->getAAT("position");
				AAT flagsAAT = upc->getAAT("flags");
				AAT normalAAT = upc->getAAT("normal");
				bool providesColor = upc->providesAttribute("color");
				AAT colorAAT;
				if( providesColor )
					colorAAT = upc->getAAT("color");
				Matrix3f tmpM3 = shrink4To3(tmpM);
				Vector3f color = makeVector3f(1,1,1);
				for( unsigned i=0;i<upc->getNumPoints();i++ )
				{

					Vector3f pos = transformVector3f(tmpM, upc->getPointSet()->get3f(i,posAAT));
					if( providesColor )
						color = upc->getPointSet()->get3f(i,colorAAT);
					Vector3f normal = tmpM3 * upc->getPointSet()->get3f(i,normalAAT);
					result->addPoint(pos,normal,color);
				}
			}
			if( mesh )
			{
				AAT posAAT = mesh->getAAT("position");
				//AAT normalAAT = mesh->getAAT("normal");
				bool providesColor = mesh->providesAttribute("color");
				AAT colorAAT;
				if( providesColor )
					colorAAT = mesh->getAAT("color");
				AAT triAAT = mesh->getTriangles()->getAAT("index");
				for( unsigned i=0;i<mesh->getNumTriangles();i++ )
				{
					Vector3i tri = mesh->getTriangles()->get3i(i,triAAT);
					Vector3f p0 = transformVector3f(tmpM,mesh->getPointSet()->get3f(tri[0],posAAT));
					Vector3f p1 = transformVector3f(tmpM,mesh->getPointSet()->get3f(tri[1],posAAT));
					Vector3f p2 = transformVector3f(tmpM,mesh->getPointSet()->get3f(tri[2],posAAT));
					if( providesColor )
					{
						Vector3f col = (mesh->getPointSet()->get3f(tri[0],colorAAT)+
							mesh->getPointSet()->get3f(tri[1],colorAAT)+
							mesh->getPointSet()->get3f(tri[2],colorAAT)/3.0f);
						result->addTriangle(p0,p1,p2,col);
					}
					else
						result->addTriangle(p0,p1,p2,makeVector3f(1,1,1));
				}
			}
		}
	}

	unsigned numChildren = sceneIter->getNumChildren();
	for( unsigned i=0;i<numChildren;i++ )
	{
		sceneIter->goToChild(i);
		gatherPoints(sceneIter,currentModelMatrix, result);
		sceneIter->goToParent();
	}

}


void X4::EyeCandy::sampleScene()
{
	deletePointCloud(getScene(),"root/lightSamples");
	BoundingBox3f sceneBB = getScenePCBBox(getScene());
	const float samplePrec = 0.001f * m_NumSamples + 0.005f * (1.0f - m_NumSamples);
	debugOutput << "sample prec:" << samplePrec << "\n";
	float initialSampleSpacing = samplePrec * sceneBB.getDiagonalLength();
	
	StandardSceneGraphIterator sceneIt;
	sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, 0, 0);

	// gather points
	debugOutput << "gathering samples from scene...";
	debugOutput.flush();
	ECResampleOctree resample_octree(sceneBB,initialSampleSpacing);
	gatherPoints(&sceneIt, IDENTITY4F, &resample_octree);
	UnstructuredInCorePointCloud * sampledScene = resample_octree.getResult();
	//addPointCloud(getScene(),sampledScene,"sampledScene");

	// photons tracing
	debugOutput << "done\nStarting photon tracing...";
	debugOutput.flush();
	ECPhotonTracer ptracer;
	ptracer.setup(sampledScene);
	ptracer.addWhiteHemisphere(makeVector3f(0,0,1), 10000);
	sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, 0, 0);
	float sampleSize = sceneBB.getDiagonalLength() * 0.005f;
	//ptracer.addSelectedPointsAsLightSource(&sceneIt, 30000, sampleSize );

	//debugRenderer->beginRenderJob("photons");
	std::vector<ECPhoton> photons = ptracer.compute();

	// store in pc
	BoundingBox3f bbPhotons(photons[0].m_Position);
	for( unsigned i=0;i<photons.size();i++ )
		bbPhotons.addPoint(photons[i].m_Position);
	ECResampleOctree lightResampler(bbPhotons,initialSampleSpacing * 16.0f);
	for( unsigned i=0;i<photons.size();i++ )
	{
		lightResampler.addPoint(photons[i].m_Position,photons[i].m_SurfaceDir,photons[i].m_Color / photons.size());
	}
	UnstructuredInCorePointCloud * photonPC = lightResampler.getResult();
//	debugOutput << "num resampled " << photonPC->getNumPoints()<<"\n";
	//addPointCloud(getScene(),photonPC,"samplePC");
	//return;
	//delete sampledScene;
	debugOutput << "done\nFinal Gathering...\n";
	debugOutput.flush();


	// final gathering
	finalGathering( photonPC, sampledScene);
	
	SGObjectNode * objNode = new SGObjectNode;
	objNode->setName("lightSamples");
	objNode->setVisible(false);
	objNode->clearAndSetup(sampledScene);
	getScene()->getRootNode()->addChildNode(objNode);
	debugOutput << "\ndone\n";
	debugOutput.flush();

}

void X4::EyeCandy::renderShadowCube( GLRenderTextureCube * cubemap, const Matrix4f &frame )
{
	//m_Mat_FinalGatherDepth_ScaleParm->setDefaultValue(makeVector4f(1.0f / getScene()->getRootState()->staticState->viewFrustum.getZFarClip(),0,0,0));
	m_Mat_FinalGatherDepth->setDefaultParameter("scaleZ",makeVector4f(1.0f / getScene()->getRootState()->staticState->viewFrustum.getZFarClip(),0,0,0));
	m_Mat_FinalGatherDepth->setDefaultParameter("camPos",frame[3]);
	for( unsigned i=0;i<4;i++ )
	{
		// setup camera
		Vector3f camPos = shrink4To3(frame[3]);
		Matrix3f M = shrink4To3(frame);
		Vector3f camDir;
		Vector3f camUp;
		switch (i)
		{
		case 0:	// pos-x
			cubemap->activate(0);
			camDir = M[0];
			camUp =  -M[1];
			break;
		case 1:	// pos-y
			cubemap->activate(2);
			camDir = M[1];
			camUp =  M[2];
			break;
		case 2:	// pos-z
			cubemap->activate(4);
			camDir = M[2];
			camUp =  -M[1];
			break;
		case 3:	// neg-z
			cubemap->activate(5);
			camDir = -M[2];
			camUp =  -M[1];
			break;

		}
		Vector3f lookAt = camPos + camDir;

		GLHelper::pushMVP();
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(90.0f, 1.0f,
				getScene()->getRootState()->staticState->viewFrustum.getZNearClip(),
				getScene()->getRootState()->staticState->viewFrustum.getZFarClip());
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(camPos[0], camPos[1], camPos[2],
			lookAt[0], lookAt[1], lookAt[2],
			camUp[0], camUp[1], camUp[2]);

		// render scene into depth buffer
		glClearColor(1,1,1,1);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glEnable(GL_DEPTH_TEST);
		//glColorMask(false,false,false,false);
		GLShaderMaterial * mat = m_Mat_FinalGatherDepth;
		StandardSceneGraphIterator sceneIt;
		sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, 0, 0);
		renderSceneRecursive(&sceneIt, IDENTITY4F, mat);
		
		//glColorMask(true,true,true,true);
		GLHelper::popMVP();
		cubemap->deactivate();
	}
}

void X4::EyeCandy::finalGathering( UnstructuredInCorePointCloud * photons, UnstructuredInCorePointCloud * dest )
{
	// setup
	const card64 maxRenderTextureSize = 2048;
	if( m_RT_FinalGather == NULL )
	{
		m_RT_FinalGather = new GLRenderTextureFBO(getGLContext());
		if( !m_RT_FinalGather->create(maxRenderTextureSize,maxRenderTextureSize,4, GLRenderTextureFBO::FLAG_RECTANGLE | GLRenderTextureFBO::FLAG_FLOAT32) )
		{
			message("final gathering failed! could not create rendertarget!");
			delete m_RT_FinalGather;
			m_RT_FinalGather = NULL;
			return;
		}
	}

	// split dest point cloud into blocks and processes blocks
	card64 maxNumPointsPerBlock = maxRenderTextureSize*maxRenderTextureSize;
	SceneObjectIterator * iter = dest->createIterator(SOIT::CAP_BASIC_PC 
		| SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_RANDOM_ACCESS_PC);
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
	RandomAccessPointCloudIterator * rIt = dynamic_cast<RandomAccessPointCloudIterator*>(iter);

	card32 numBlocks = (dest->getNumPoints()+maxNumPointsPerBlock-1 )/ maxNumPointsPerBlock;
	STATUSREPORT_DEFINE(finalGathering);
	for( unsigned b=0;b<numBlocks;b++ )
	{
		// create a copy of the current block
		UnstructuredInCorePointCloud tmpUPC;
		VertexDescriptor * vd = createDefaultVertexDescriptor(true,true,true,false);
		vd->pushAttrib(VAD("screen_pos",2,VAD::DATA_FORMAT_FLOAT32));
		unsigned startPointIndex = b*maxNumPointsPerBlock;
		unsigned lastPointIndex = min(dest->getNumPoints(),(b+1)*maxNumPointsPerBlock);
		tmpUPC.clearAndSetup(vd,lastPointIndex - startPointIndex);
		AAT srcPosAAT = dest->getAAT("position");
		AAT srcColorAAT = dest->getAAT("color");
		AAT srcNormalAAT = dest->getAAT("normal");
		AAT destPosAAT = tmpUPC.getAAT("position");
		AAT destColorAAT = tmpUPC.getAAT("color");
		AAT destNormalAAT = tmpUPC.getAAT("normal");
		AAT destScreenPosAAT = tmpUPC.getAAT("screen_pos");
		for(unsigned j=0;j<tmpUPC.getNumPoints();j++ )
		{
			rIt->goToIndex(startPointIndex + j);
			Vector3f pos = bIt->get3f(srcPosAAT);
			Vector3f color = bIt->get3f(srcColorAAT);
			Vector3f normal = bIt->get3f(srcNormalAAT);
			Vector2f screenPos = makeVector2f(j % maxRenderTextureSize, j / maxRenderTextureSize+0.5f) / maxRenderTextureSize * 2.0f
				+ makeVector2f(-1 ,-1);
			tmpUPC.getPointSet()->set3f(j,destPosAAT,pos);
			tmpUPC.getPointSet()->set3f(j,destColorAAT,color);
			tmpUPC.getPointSet()->set3f(j,destNormalAAT,normal);
			tmpUPC.getPointSet()->set2f(j,destScreenPosAAT,screenPos);
		}

		// light block
		SceneObjectIterator * block_iter = tmpUPC.createIterator(SOIT::CAP_BASIC_PC);
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(block_iter);
		VertexArray va_photons(photons);
		for( unsigned i=0;i<va_photons.getNumElements();i++ )
		{

			// setup photon
 			Vector3f lightPos = va_photons.getPosition3f(i);
 			Vector3f lightNormal = va_photons.getNormal3f(i);
 			Vector3f lightColor = va_photons.getColor3f(i);
			BoundingBox3f bbscene = getScenePCBBox(getScene());
//			Vector3f lightPos = (bbscene.lowerCorner + bbscene.getCenter())*.5f;
								//bbscene.getCenter();
//			Vector3f lightNormal = normalize(bbscene.getCenter() - lightPos);
//			Vector3f lightColor = makeVector3f(1,1,0);

			// setup matrix with the following properties: z-axis perpendicular to lightNormal, angle (x-axis,lightNormal) = 45, same for y-axis
			Matrix3f tmpM = calcTangentSystem(lightNormal);
			Matrix3f lMat; lMat[0] = tmpM[2]; lMat[1] = tmpM[0]; lMat[2] = tmpM[1];
			Matrix4f shadowmapMatrix = makeTranslation4f(lightPos) * expand3To4(lMat) * makeRotZ4f(45.0f / 180.0f * (float)M_PI);
			Matrix4f shadowmapMatrixInv = invertFrame(shadowmapMatrix);

			// render depth cube
			renderShadowCube(m_CubeShadowMap, shadowmapMatrix);
			//m_CubeShadowMap->outputViaImageDisplayer("shadowcube");

			m_RT_FinalGather->activate();

			// blend light
			glPushAttrib( GL_ALL_ATTRIB_BITS );
			glDisable(GL_DEPTH_TEST);
			glDepthMask(0);
			m_Mat_FinalGather->setDepthTest(false);
			// clear
			if( i==0 ) // first pass?
			//if(1)
			{
				glClearColor(1,0,1,0);
				glClear( GL_COLOR_BUFFER_BIT);
				m_Mat_FinalGather->setBlending(false);
			}
			else
			{
				m_Mat_FinalGather->setBlending(true);
				m_Mat_FinalGather->setDestBlendFunc(GLShaderMaterial::BM_ONE);
				m_Mat_FinalGather->setSourceBlendFunc(GLShaderMaterial::BM_ONE);
			}

			m_Mat_FinalGather->setDefaultParameter("matShadowMap",shadowmapMatrixInv);
			m_Mat_FinalGather->setDefaultParameter("lightPos",expand3To4(lightPos));
			m_Mat_FinalGather->setDefaultParameter("lightColor",expand3To4(lightColor));
			m_Mat_FinalGather->setDefaultParameter("scaleDepth",makeVector4f(1.0f / getScene()->getRootState()->staticState->viewFrustum.getZFarClip(),0,0,0));
			m_Mat_FinalGather->setDefaultParameter("shadowOffset",makeVector4f(getScene()->getRootState()->staticState->viewFrustum.getZFarClip() / 10000.0f,0,0,0));
			m_Mat_FinalGather->setDefaultParameter("surfaceLightDir",expand3To4(lightNormal));

			// render
			bIt->reset();
			m_CubeShadowMap->bind();
			m_Mat_FinalGather->draw( getScene(), getScene()->getRootState(), NULL, block_iter, getGLContext());
			m_CubeShadowMap->unbind();

			glDepthMask(1);
			glPopAttrib();
			m_RT_FinalGather->deactivate();

			float p = (float)i / (float)photons->getNumPoints()*100.0f;
			STATUSREPORT_REPORT(finalGathering,10000,std::string("process block ") + floatToStr(p) + "\n");
		}
		delete block_iter;
		RGBAFloat32Image * image = m_RT_FinalGather->getRGBAFloat32Image();
		// compute lighting for block
		//RGBAFloat32Image * image = precomputeLight_Block(&tmpUPC,pc->getTransformation());


		// store results in original point cloud
		for(unsigned j=0;j<tmpUPC.getNumPoints();j++ )
		{
			rIt->goToIndex(startPointIndex + j);
			Vector3f color = shrink4To3(image->getPixel(j % maxRenderTextureSize, j / maxRenderTextureSize));
			mIt->set3f(srcColorAAT,color);
		}
		delete image;
		//m_Temp_NumPointsProcessed += tmpUPC.getNumPoints();
	}

	delete iter;


}

void X4::EyeCandy::setupSSAOSamples()
{
	// ask for max sample dist
	bool ok;
	double maxSampleDist = QInputDialog::getDouble(NULL,"maximum sample distance", "please specify the minimal distances between"
		" two samples on the unit hemisphere.",.15,0.0001,1,3,&ok);
	if( !ok)
		return;

	// create point cloud with much more samples than necessary

	float areaHemisphere = 2.0f * (float)M_PI;
	float sampleArea = sqr(maxSampleDist/2.0)*M_PI;
	float minIdealSamples = areaHemisphere / sampleArea;
	mpcard numSamples = 10.0f * minIdealSamples;
	UnstructuredInCorePointCloud upc;
	upc.clearAndSetup(numSamples,true,false,false,true);
	VertexArray points(&upc);
	for( mpcard i=0;i<numSamples;i++ )
	{
		float phi = acos(rnd01());
		float theta = rnd01() * 2.0f * M_PI;
		Vector3f p = makeVector3f(sin(theta)*sin(phi),cos(theta)*sin(phi),cos(phi));
		points.setPosition3f(i,p);
	}
	addPointCloud(getScene(),(PointCloud*)upc.copy(),"initial samples");

	// resample
	PCCResampler::resamplePC(&upc,maxSampleDist);

	addPointCloud(getScene(),(PointCloud*)upc.copy(),"resampled");
	
	// compute total weight
	float summedWeight = 0;
	VertexArray points_new(&upc);
	for( mpcard i=0;i<upc.getNumPoints();i++ )
	{
		Vector3f p = points_new.getPosition3f(i);
		summedWeight += p[2];
	}	
	// output via debug renderer
	debugOutput << "\tconst int numSamples = " << upc.getNumPoints() << "\n";
	debugOutput << "\tconst float4 samples[numSamples] = \n\t{\t\n";
	for( mpcard i=0;i<upc.getNumPoints();i++ )
	{
		Vector3f p = points_new.getPosition3f(i);
		float w = p[2] / summedWeight;
		debugOutput << "\t\t" << (i>0?",":"") <<  "float4(" << p[0] << ", " << p[1] << ", " << p[2] << ", " << w << ")\n";
	}
	debugOutput << "};";
}

X4::ECResampleOctree::TNode::TNode()
{
	for( unsigned i=0;i<8;i++ )
		child[i] = NULL;
	reprPointIndex = -1;
}

X4::ECResampleOctree::TNode::~TNode()
{
	for( unsigned i=0;i<8;i++ )
		delete child[i];
}

X4::ECResampleOctree::ECResampleOctree( const BoundingBox3f &sceneBB, float sampleSpacing )
{
	m_UPC = new UnstructuredInCorePointCloud;
	m_UPC->clearAndSetup(0);
	VertexDescriptor * vd = (VertexDescriptor*)m_UPC->getDescr();
	vd->pushAttrib(VAD("sampleSize",1,VAD::DATA_FORMAT_FLOAT32));
	m_PosAAT = m_UPC->getAAT("position");
	m_NormalAAT = m_UPC->getAAT("normal");
	m_ColorAAT = m_UPC->getAAT("color");
	m_SampleSizeAAT= m_UPC->getAAT("sampleSize");
	m_SampleSpacing = sampleSpacing;
	m_RootNode = new TNode;
	m_RootNode->bb = sceneBB;
	m_RootNode->bb.makeBoundingCube();
}

X4::ECResampleOctree::~ECResampleOctree()
{
	delete m_RootNode;
}

void X4::ECResampleOctree::addTriangle( const Vector3f &p0,const Vector3f &p1,const Vector3f &p2, const Vector3f &color )
{
	// first: check for bad triangles
	float l0 = norm(p0-p1);
	float l1 = norm(p0-p2);
	float l2 = norm(p2-p1);
	float minl = min(min(l0,l1),l2);
	float maxl = max(max(l0,l1),l2);

	if( minl / maxl < 0.00001f )
		return;
	if( fabs(normalize(p1-p0)*normalize(p2-p0)) > cos(M_PI/180.0f) )
		return;

	BoundingBox3f tbb(p0); tbb.addPoint(p1);tbb.addPoint(p2);
	Vector3f normal = normalize((p1-p0).crossProduct(p2-p0));
	Plane3f plane; // changed from Plane by Chris @ 2014/02/17
	plane.setNormal(normal);
	plane.setPoint(p0);
	const float threshold = cos(30.0f / 180.0f * M_PI);

	if( !triangleIntersectsBB(p0,p1,p2,m_RootNode->bb) )
		return;

	std::vector<TNode*> nodes; nodes.reserve(100); nodes.push_back(m_RootNode);
	while(!nodes.empty())
	{
		TNode * node = nodes.back();
		nodes.pop_back();

		Vector3f bbCenter = node->bb.getCenter();
		if( node->bb.getSideLength(0) < m_SampleSpacing )
		{
			// leaf case
			bool foundExistingPlane = false;
			for( unsigned i=0;i<node->planeNormals.size();i++ )
			{
				if( node->planeNormals[i]*normal > threshold )
				{
					foundExistingPlane = true;
					break;
				}
			}
			
			//if( node->planeNormals.size() == 0)
			if( !foundExistingPlane)
			{
				// project random point onto plane
				Vector3f projPoint;
				Vector3f rPoint = bbCenter + makeVector3f(rnd01()-.5f,rnd01()-.5f,rnd01()-.5f) * node->bb.getSideLength(0);
				projectPointPlane(plane,rPoint,projPoint);
				unsigned pID = m_UPC->getNumPoints();
				m_UPC->getPointSet()->addPoint(NULL);
				m_UPC->getPointSet()->set3f(pID,m_PosAAT,projPoint);
				m_UPC->getPointSet()->set3f(pID,m_NormalAAT,normal);
				m_UPC->getPointSet()->set3f(pID,m_ColorAAT,color);
				m_UPC->getPointSet()->set1f(pID,m_SampleSizeAAT,m_SampleSpacing);
				node->planeNormals.push_back(normal);
			}
			continue;
		}

		// node case:
		for( unsigned i=0;i<8;i++ )
		{
			BoundingBox3f bbchild;
			if( node->child[i] == NULL )
			{
				bbchild = node->bb;
				for( unsigned j=0;j<3;j++ )
				{
					if( i&(1 << j) )
						bbchild.lowerCorner[j] = bbCenter[j];
					else
						bbchild.upperCorner[j] = bbCenter[j];
				}
			}
			else
				bbchild = node->child[i]->bb;

			if( bbchild.intersectBoundingBox(tbb) && triangleIntersectsBB(p0,p1,p2,bbchild) )
			{
				if( node->child[i] == NULL )
				{
					node->child[i] = new TNode;
					node->child[i]->bb = bbchild;
				}
				nodes.push_back(node->child[i]);
			}
		}
	}
}

void X4::ECResampleOctree::addPoint( const Vector3f &p, const Vector3f &n, const Vector3f &col )
{
	if( !m_RootNode->bb.inBoundingBox(p))
		return;

	std::vector<TNode*> nodes; nodes.reserve(100); nodes.push_back(m_RootNode);
	static unsigned maxlevel = 0;
	while(!nodes.empty())
	{
		TNode * node = nodes.back();
		nodes.pop_back();

		Vector3f bbCenter = node->bb.getCenter();
		if( node->bb.getSideLength(0) < m_SampleSpacing )
		{
			// leaf case
			if( node->reprPointIndex >= 0 )
			{
				m_UPC->getPointSet()->set3f(node->reprPointIndex,m_ColorAAT,col + m_UPC->getPointSet()->get3f(node->reprPointIndex,m_ColorAAT));
			}
			else
			{
				unsigned pID = m_UPC->getNumPoints();
				m_UPC->getPointSet()->addPoint(NULL);
				m_UPC->getPointSet()->set3f(pID,m_PosAAT,p);
				m_UPC->getPointSet()->set3f(pID,m_NormalAAT,n);
				m_UPC->getPointSet()->set3f(pID,m_ColorAAT,col);
				m_UPC->getPointSet()->set1f(pID,m_SampleSizeAAT,m_SampleSpacing);
				node->reprPointIndex = pID;
			}
			continue;
		}

		// node case:
		unsigned childIndex = ((p[0] < bbCenter[0])?0:1)
			+ ((p[1] < bbCenter[1])?0:2)
			+ ((p[2] < bbCenter[2])?0:4);
		if( node->child[childIndex] == NULL )
		{
			node->child[childIndex] = new TNode;
			node->child[childIndex]->bb = node->bb; 
			for( unsigned j=0;j<3;j++ )
			{
				if( childIndex&(1 << j) )
					node->child[childIndex]->bb.lowerCorner[j] = bbCenter[j];
				else
					node->child[childIndex]->bb.upperCorner[j] = bbCenter[j];
			}
		}
		nodes.push_back(node->child[childIndex]);
		
	}

	SharedVertexBufferAttachment * getSplatVertexBuffer( SharedVertexBufferAttachment * vbuffer, UnstructuredInCorePointCloud * lightSamples );
}