#ifndef EyeCandyH
#define EyeCandyH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Renderer.h"
//---------------------------------------------------------------------------

namespace X4
{
class GLMultipleRenderTexture;
class GLRenderTextureFBO;
class GLShaderMaterial;
class StandardSceneGraphIterator;
class RandomTextureAttachment;
class ECResampleOctree;
class UnstructuredInCorePointCloud;
class GLRenderTextureCube;
class MPMatrix4x4;
class MPVector4;
class SharedVertexBufferAttachment;

/**
	renders stuff
*/
class X3FOUNDATION_PRIVATE_API EyeCandy : public Renderer
{
	X4_CLASS( EyeCandy )

private:

	bool	m_AllowDebugRenderer;

	GLMultipleRenderTexture	*	m_RenderTexture;
	GLRenderTextureFBO		*	m_RT_SSAO;
	GLRenderTextureFBO		*	m_RT_SSAO_Blurred;
	GLRenderTextureFBO		*	m_RT_SplatLight;
	GLRenderTextureCube		*	m_CubeShadowMap;
	GLRenderTextureFBO	*		m_RT_FinalGather;
	RandomTextureAttachment	*	m_RandomTexture;
	bool setupRenderTexture( unsigned width, unsigned height );
	void drawScreenQuad();
	void renderSceneRecursive( StandardSceneGraphIterator * sceneIter, const Matrix4f &modelMatrix, GLShaderMaterial * mat );
	void renderShadowCube(GLRenderTextureCube * cubemap, const Matrix4f &frame);
	void finalGathering(UnstructuredInCorePointCloud * photons, UnstructuredInCorePointCloud * dest);
	SharedVertexBufferAttachment * getSplatVertexBuffer( UnstructuredInCorePointCloud * lightSamples );

	GLShaderMaterial	*	m_Mat_RenderPosNormalColor;
	GLShaderMaterial	*	m_Mat_RenderTextureToScreen;
	GLShaderMaterial	*	m_Mat_RenderSSAO;
	GLShaderMaterial	*	m_Mat_BlurSSAO;
	GLShaderMaterial	*	m_Mat_FinalGather;
	GLShaderMaterial	*	m_Mat_FinalGatherDepth;
	MPVector4			*	m_Mat_FinalGatherDepth_ScaleParm;
	GLShaderMaterial	*	m_Mat_SplatLight;
	GLShaderMaterial	*	m_Mat_Combine;

	float32		m_LightStrength;
	bool		m_HemisphereLight;
	bool		m_SelectedPointsAsLightSource;
	bool		m_IndirectLight;
	float32		m_NumSamples;
	float32		m_NumPhotons;
	float32		m_Offset;
	float32		m_Gamma;
	float32		m_Brightness;

public:
	EyeCandy(void);
	~EyeCandy(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void connect(Scene *scene, GLContext* ctxt);
   virtual void render(card32 viewportWidth, card32 viewportHeight);
   virtual void disconnect(GLContext* ctxt);

   void sampleScene();
   void setupSSAOSamples();

	/// returns true if renderer allows debugrenderer
	virtual bool debugRendererAllowed() {return m_AllowDebugRenderer;}


	//  ----------- access methods ---------------
	float32 getLightStrength() const {return m_LightStrength;}
	void setLightStrength(float32 v) {m_LightStrength=v;}
	bool getHemisphereLight() const {return m_HemisphereLight;}
	void setHemisphereLight(bool v) {m_HemisphereLight=v;}
	bool getSelectedPointsAsLightSource() const {return m_SelectedPointsAsLightSource;}
	void setSelectedPointsAsLightSource(bool v) {m_SelectedPointsAsLightSource=v;}
	bool getIndirectLight() const {return m_IndirectLight;}
	void setIndirectLight(bool v) {m_IndirectLight=v;}
	float32 getNumSamples() const {return m_NumSamples;}
	void setNumSamples(float32 v) {m_NumSamples=v;}
	float32 getNumPhotons() const {return m_NumPhotons;}
	void setNumPhotons(float32 v) {m_NumPhotons=v;}
	float32 getOffset() const {return m_Offset;}
	void setOffset(float32 v) {m_Offset=v;}
	float32 getGamma() const {return m_Gamma;}
	void setGamma(float32 v) {m_Gamma=v;}
	float32 getBrightness() const {return m_Brightness;}
	void setBrightness(float32 v) {m_Brightness=v;}



};

class ECResampleOctree
{
private:
	UnstructuredInCorePointCloud * m_UPC;
	AAT	m_PosAAT;
	AAT m_NormalAAT;
	AAT m_ColorAAT;
	AAT m_SampleSizeAAT;


	struct TNode
	{

		TNode();
		~TNode();
		TNode * child[8];
		BoundingBox3f bb;
		std::vector<Vector3f> planeNormals;
		int reprPointIndex;
	};

	TNode * m_RootNode;
	float	m_SampleSpacing;

public:
	
	ECResampleOctree(const BoundingBox3f &sceneBB, float sampleSpacing);
	~ECResampleOctree();
	
	void addTriangle(const Vector3f &p0,const Vector3f &p1,const Vector3f &p2, const Vector3f &color);
	void addPoint(const Vector3f &p, const Vector3f &n, const Vector3f &col);

	UnstructuredInCorePointCloud * getResult() {return m_UPC;}
};

}

#endif