#ifndef FSIcpH
#define FSIcpH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{

	class PointSetANNQuery;
	class Scene;
	class HierarchicalKNNIterator;
	class PointCloud;

/**
	improves transformations using point to plane icp
*/
class X3FOUNDATION_PRIVATE_API FSIcp : public Persistent
{
	X4_CLASS( FSIcp )

private:

private:

	card32  maxICPSteps;
	float32 allowedNormalDiff;
	float32 minPercentInliers;
	float32 m_MedPointDistance;
	float32 outlierDistanceFactor;
	float32 icpAllowedMoveDist;
	float32 icpMaxResidual;
	float32 icpConvergenceDiff;
	bool    visualization;
	bool		useANN;


	PointSet* targetPS;
	Scene* scene;
	PointSetANNQuery* annQuery;
	HierarchicalKNNIterator* hIt;
	AAT POSITION;
	AAT NORMAL;
	Matrix4f betterTransformation;
	Vector3f startPScenter;
	Vector3f startPScenterOrgTransformed;


public:
	FSIcp(void);
	~FSIcp(void);

 
   void setupParameters( card32 maxICPSteps = 64, float32 allowedNormalDiff = 2, float32 m_MedPointDistance = 0.0f, float32 outlierDistanceFactor = 30, float32 minPercentInliers =0.20f, float32 icpAllowedMoveDist= 1e20f , float32 icpMaxResidual= -1.0f, float32 icpConvergenceDiff = 0.001f, bool visualization= true);

   void setupData(PointSet* ps, Scene* scene);
	void setupData(PointCloud* pc, Scene* scene);
   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
   void setMedianPointDistance(float32 d);
   float32 getMedianPointDistance()const ;

   ///returns remaining residual after ICP Alignment
   /// returns 1e20 if: moved too far or max steps reached and not converged
   /// if converged, overwrites parameter Matrix4f transformation 
   float32 calculateResidualAfterICP(PointSet* startPS, Matrix4f&	transformation);
   float32 calculateResidualAfterStandardICP(PointSet* startPS, Matrix4f& transformation);

   float32 calculateResidualAfterICP(const std::vector<Vector3f> &positions, Matrix4f& transformation);
   float32 calculateResidualAfterStandardICP(const std::vector<Vector3f> &positions, Matrix4f& transformation);



   void getClosestPointAndNormal( Vector3f transformedPos, Vector3f &closestPoint, Vector3f &closestPointNormal ) const;
private:
	bool checkAllowedMovement(const Matrix4f &trans);
	float32 getMedianPointDistance(PointSet* ps);

};

}

#endif