#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "FastSpatialQuery.h"
//---------------------------------------------------------------------------
#include "TriangleBBIntersection.h"
#include "AttachedIndexedOctree.h"
#include "DebugRenderer.h"
#include "OOBBox3f.h"
#include "BoundingBox.h"
//---------------------------------------------------------------------------
using namespace X4;

FastSpatialQuery::FastSpatialQuery(PointCloud * pc)
{
	m_NumPoints = 0;
	m_IndexCache = std::vector<mpcard>(50);
	m_SceneIter = pc->createIterator(SOIT::CAP_ATTACH_SECONDARY|SOIT::CAP_BASIC_PC); 
	m_SDS = dynamic_cast<SecondaryDataStructureIterator*>(m_SceneIter);
	m_BasicIt = dynamic_cast<BasicPointCloudIterator*>(m_SceneIter);

	AttachedDataContainer *attachments = m_SDS->lockAttachments();

	m_PS = (PointSet*)m_BasicIt->lockBlockRead();

	m_Octree = dynamic_cast<AttachedIndexedOctree*>(attachments->getData("SimpleOctree"));
	if(m_Octree==NULL){
		m_Octree=new AttachedIndexedOctree();
		m_Octree->setup("SimpleOctree", AttachedData::ADF_QUERY_DELETE_ON_UPDATE,  NULL);
		m_Octree->octreesetup(m_PS, 32);
		attachments->attachData(m_Octree);
	}

}

FastSpatialQuery::~FastSpatialQuery(void)
{
	if( m_BasicIt )
		m_BasicIt->unlock();
	if( m_SDS )
		m_SDS->unlockAttachments();
	delete m_SceneIter;
}

float distanceSqrToLine(const Vector3f &linepos, const Vector3f &dir_normalized, const Vector3f &queryPos)
{
	Vector3f tmpDiff = linepos + dir_normalized * (dir_normalized*(queryPos-linepos))-queryPos;
	return	tmpDiff[0]*tmpDiff[0] + tmpDiff[1]*tmpDiff[1] + tmpDiff[2]*tmpDiff[2];
}

bool thickLineIntersectsBoundingBox(const Vector3f &pos, const Vector3f &dir_normalized, float maxdist, const BoundingBox3f &bb)
{
//	return true;
	Vector3f bb_center = bb.getCenter();
	float distToLine = norm(pos + dir_normalized * (dir_normalized*(bb_center-pos)) - bb_center);
	return distToLine <= maxdist + bb.getDiagonalLength()*0.5f;
//return distToLine <= maxdist + bb.getSideLength(0)/2.0f*sqrtf(3.0f);	// isn't cubic :-(
// 
// 	if( rayIntersectsBoundingBox(pos,dir_normalized,bb) )
// 		return true;
// 	BoundingBox3f bb_outer = bb;
// 	float maxDist = sqrt(maxdist_2);
// 	bb_outer.addBorder(maxDist);
// 	if( !rayIntersectsBoundingBox(pos,dir_normalized,bb) )
// 		return false;
// 	
// 	// here we need an accurate test
// 	return true; 

//	float distToCenter_2 = distanceSqrToLine(pos,dir_normalized,bb.getCenter());
//	float rad_2 = 
}

void FastSpatialQuery::lineQuery( const Vector3f &p, const Vector3f &dir, float maxDistance, mpcard ** indices, mpcard &numPoints) 
{
	const float maxdist_2 = sqr(maxDistance);
	
	AIOctreeNode * rootNode = m_Octree->getOctree();
	m_NodeStack.clear();
	m_NodeStack.push_back(rootNode);
	const Vector3f &dir_n = normalize(dir);

	m_NumPoints = 0;
	const PointSet * ps = m_PS;
	AAT posAAT = ps->getAAT("position");

	static int frame = 20;
	bool show = false;//frame < 400 && rand() % 40 == 0;
	if( show)
	{
		debugRenderer->beginRenderJob_OneFrame("narfnarf",frame++);
		debugRenderer->addLine(p + dir * 100, p - dir*100);
	}

	if( thickLineIntersectsBoundingBox(p,dir_n, maxDistance,rootNode->getContent().m_bb) )
	while( !m_NodeStack.empty() )
	{
		AIOctreeNode * node = m_NodeStack.pop_back();
		//node_queue.pop_back();

		IndexedNode & content = node->getContent();
		if( show )
			debugRenderer->addBoundingBox(content.m_bb,makeVector3f(0,0,1),1);
		if( node->isLeaf() )
		{
			const size_t numPointsInNode = content.m_index.size();
			if( numPointsInNode )
			{
				card32 * tmpIndices = &content.m_index[0];	// TODO_64BIT ATTENTION! Has to be changed for 64 bit! See definition of IndexedNode.
				for( unsigned i=0;i<numPointsInNode;i++ )
				{
					const Vector3f &tmpPos = ps->get3f(*tmpIndices,posAAT);
					Vector3f tmpDiff = p + dir_n * (dir_n*(tmpPos-p))-tmpPos;
					float dist2 = //normQuad(tmpDiff);
						tmpDiff[0]*tmpDiff[0] + tmpDiff[1]*tmpDiff[1] + tmpDiff[2]*tmpDiff[2];
					if( dist2 <= maxdist_2 )
					{
						addIndexToCache(*tmpIndices);

					}
					tmpIndices++;
				}
			}
		}
		else
		{
			// add children that intersect cylinder (not optimized)
			for( unsigned i=0;i<8;i++ )
			{
				AIOctreeNode * childNode = node->getChild(i);
				if( childNode && thickLineIntersectsBoundingBox(p,dir_n, maxDistance,childNode->getContent().m_bb) )
				{
					m_NodeStack.push_back(childNode);
				}
			}


		}
	}
	if( show )
		debugRenderer->endRenderJob();
	numPoints = m_NumPoints;
	if(numPoints > 0)
		*indices = &m_IndexCache[0];
	else
		*indices = NULL;
}

bool  thickPlaneIntersectsBoundingBox(const Plane &plane, const float maxDistance,const BoundingBox3f &bb)
{
	float radius = bb.getDiagonalLength()*0.5f;
	float distToCenter = plane.calculateSignedDistance(bb.getCenter());
	return (fabs(distToCenter)-maxDistance  <= radius);
}

void FastSpatialQuery::planeQuery( const Plane &plane, float maxDistance ,mpcard ** indices, mpcard &numPoints) 
{
	const float maxdist_2 = sqr(maxDistance);
	
	AIOctreeNode * rootNode = m_Octree->getOctree();
	m_NodeStack.clear();
	m_NodeStack.push_back(rootNode);

	m_NumPoints = 0;
	const PointSet * ps = m_PS;
	AAT posAAT = ps->getAAT("position");

	const float plane_n_dot_p = plane.getNormal() * plane.getPoint();

	if( thickPlaneIntersectsBoundingBox(plane, maxDistance,rootNode->getContent().m_bb) )
	while( !m_NodeStack.empty() )
	{
		AIOctreeNode * node = m_NodeStack.pop_back();
		//node_queue.pop_back();

		IndexedNode & content = node->getContent();
		if( node->isLeaf() )
		{
			const size_t numPointsInNode = content.m_index.size();
			if( numPointsInNode )
			{
				card32 * tmpIndices = &content.m_index[0];	// TODO_64BIT ATTENTION! Has to be changed for 64 bit! See definition of IndexedNode.
				for( unsigned i=0;i<numPointsInNode;i++ )
				{
					const Vector3f &tmpPos = ps->get3f(*tmpIndices,posAAT);
					float dist = tmpPos*plane.getNormal() - plane_n_dot_p;
					if( fabs(dist) <= maxDistance )
					{
						addIndexToCache(*tmpIndices);

					}
					tmpIndices++;
				}
			}
		}
		else
		{
			
			// add children that intersect thick plane (not optimized)
			for( unsigned i=0;i<8;i++ )
			{
				AIOctreeNode * childNode = node->getChild(i);
				if( childNode && thickPlaneIntersectsBoundingBox(plane, maxDistance,childNode->getContent().m_bb) )
				{
					m_NodeStack.push_back(childNode);
				}
			}


		}
	}
	numPoints = m_NumPoints;
	if(numPoints > 0)
		*indices = &m_IndexCache[0];
	else
		*indices = NULL;
}

bool FSQ_sphereIntersectsBoundingBox(const Vector3f &center, float sqr_radius, const BoundingBox3f &bb)
{
	float s, d = 0; 

	//find the square of the distance
	//from the sphere to the box
	for( unsigned int i=0 ; i<3 ; i++ ) {
		if( center[i] < bb.lowerCorner[i] ) {
			s = center[i] - bb.lowerCorner[i];
			d += s*s; 
		} else if( center[i] > bb.upperCorner[i] ) {
			s = center[i] - bb.upperCorner[i];
			d += s*s; 
		}
	}
	return d <= sqr_radius;
}


void FastSpatialQuery::planeAndSphereQuery( const Plane &plane, float maxDistance, const Vector3f &sphere_center, float sphere_radius, mpcard ** indices, mpcard &numPoints) 
{
	const float maxdist_2 = sqr(maxDistance);
	const float rad_2 = sphere_radius * sphere_radius;
	
	AIOctreeNode * rootNode = m_Octree->getOctree();
	m_NodeStack.clear();
	m_NodeStack.push_back(rootNode);

	m_NumPoints = 0;
	const PointSet * ps = m_PS;
	AAT posAAT = ps->getAAT("position");

	const float plane_n_dot_p = plane.getNormal() * plane.getPoint();

	if( thickPlaneIntersectsBoundingBox(plane, maxDistance,rootNode->getContent().m_bb) )
	if( FSQ_sphereIntersectsBoundingBox(sphere_center,rad_2,rootNode->getContent().m_bb) )
	while( !m_NodeStack.empty() )
	{
		AIOctreeNode * node = m_NodeStack.pop_back();
		//node_queue.pop_back();

		IndexedNode & content = node->getContent();
		if( node->isLeaf() )
		{
			const size_t numPointsInNode = content.m_index.size();
			if( numPointsInNode )
			{
				card32 * tmpIndices = &content.m_index[0];	// TODO_64BIT ATTENTION! Has to be changed for 64 bit! See definition of IndexedNode.
				for( unsigned i=0;i<numPointsInNode;i++ )
				{
					const Vector3f &tmpPos = ps->get3f(*tmpIndices,posAAT);
					float dist = tmpPos*plane.getNormal() - plane_n_dot_p;
					if( fabs(dist) <= maxDistance )
					{
						Vector3f tmpDiff = tmpPos - sphere_center;
						float dist2 = //normQuad(tmpDiff);
							tmpDiff[0]*tmpDiff[0] + tmpDiff[1]*tmpDiff[1] + tmpDiff[2]*tmpDiff[2];
						if( dist2 <= rad_2 )
							addIndexToCache(*tmpIndices);

					}
					tmpIndices++;
				}
			}
		}
		else
		{
			
			// add children that intersect thick plane (not optimized)
			for( unsigned i=0;i<8;i++ )
			{
				AIOctreeNode * childNode = node->getChild(i);
				if( childNode && thickPlaneIntersectsBoundingBox(plane, maxDistance,childNode->getContent().m_bb) 
					&& FSQ_sphereIntersectsBoundingBox(sphere_center,rad_2,childNode->getContent().m_bb))
				{
					m_NodeStack.push_back(childNode);
				}
			}


		}
	}
	numPoints = m_NumPoints;
	if(numPoints > 0)
		*indices = &m_IndexCache[0];
	else
		*indices = NULL;
}


bool torusIntersectsBoundingCube(const Circle3D & circle, float tubeRadius, const BoundingBox3f &cube)
{
	Vector3f cube_center = cube.getCenter();
	float dist = circle.distance(cube_center);
	return ( dist <= tubeRadius + cube.getSideLength(0)/2.0f );
}

void X4::FastSpatialQuery::torusQuery( const Circle3D & circle, float tubeRadius, mpcard ** indices, mpcard &numPoints )
{

	AIOctreeNode * rootNode = m_Octree->getOctree();
	m_NodeStack.clear();
	m_NodeStack.push_back(rootNode);

	m_NumPoints = 0;
	const PointSet * ps = m_PS;
	AAT posAAT = ps->getAAT("position");

	if( torusIntersectsBoundingCube(circle,tubeRadius,rootNode->getContent().m_bb) )
		while( !m_NodeStack.empty() )
		{
			AIOctreeNode * node = m_NodeStack.pop_back();
			//node_queue.pop_back();

			IndexedNode & content = node->getContent();
			if( node->isLeaf() )
			{
				const size_t numPointsInNode = content.m_index.size();
				if( numPointsInNode )
				{
					card32 * tmpIndices = &content.m_index[0];	// TODO_64BIT ATTENTION! Has to be changed for 64 bit! See definition of IndexedNode.
					for( unsigned i=0;i<numPointsInNode;i++ )
					{
						const Vector3f &tmpPos = ps->get3f(*tmpIndices,posAAT);
						if( circle.distance(tmpPos) <= tubeRadius )
						{
							addIndexToCache(*tmpIndices);

						}
						tmpIndices++;
					}
				}
			}
			else
			{
				// add children that intersect torus (not optimized)
				for( unsigned i=0;i<8;i++ )
				{
					AIOctreeNode * childNode = node->getChild(i);
					if( childNode && torusIntersectsBoundingCube(circle,tubeRadius,childNode->getContent().m_bb) )
					{
						m_NodeStack.push_back(childNode);
					}
				}


			}
		}
		numPoints = m_NumPoints;
		if(numPoints > 0)
			*indices = &m_IndexCache[0];
		else
			*indices = NULL;
}

bool boundingBoxIntersectsHalfSpace( const BoundingBox3f &bb, const Plane & plane, float maxDistance )
{
	for( mpcard i=0;i<8;i++ )
	{
		Vector3f corner = bb.getCorner(i);
		if( plane.calculateSignedDistance(corner) > -maxDistance )
			return true;
	}
	return false;
}

void FastSpatialQuery::triangleQuery( const Vector3f p0,const Vector3f p1,const Vector3f p2, float maxDistance,mpcard ** indices, mpcard &numPoints) 
{
	const float maxdist_2 = sqr(maxDistance);
	
	AIOctreeNode * rootNode = m_Octree->getOctree();
	m_NodeStack.clear();
	m_NodeStack.push_back(rootNode);

	m_NumPoints = 0;
	const PointSet * ps = m_PS;
	AAT posAAT = ps->getAAT("position");

	// setup triangle plane
	Plane plane;
	plane.setPoint(p0);
	Vector3f normal = (p1-p0).crossProduct(p2-p0);
	if( norm(normal) < 1e-15f )
	{
		numPoints = 0;
		*indices = NULL;
	}
	plane.setNormal( normalize(normal) );

	// setup boundary planes (normal points inwards) & boundary lines
	Vector3f points[3];
	points[0] = p0;
	points[1] = p1;
	points[2] = p2;
	Plane boundary[3];
	Line boundaryLines[3];
	for( mpcard i=0;i<3;i++ )
	{
		Vector3f bnormal = normalize(normal.crossProduct(points[(i+1)%3] - points[i]));
		boundary[i].setPoint(points[i]);
		boundary[i].setNormal(bnormal);
		float distToThirdPoint = boundary[i].calculateSignedDistance(points[(i+2)%3]);
		if( fabs(distToThirdPoint) < 1e-15f )
		{
			numPoints = 0;
			*indices = NULL;
		}
		if( distToThirdPoint < 0 )
			boundary[i].setNormal(-bnormal);

		boundaryLines[i].setPoint(points[i]);
		boundaryLines[i].setDirection(points[(i+1)%3] - points[i]);
	}

	const float plane_n_dot_p = plane.getNormal() * plane.getPoint();

	if( thickPlaneIntersectsBoundingBox(plane, maxDistance,rootNode->getContent().m_bb) )
	while( !m_NodeStack.empty() )
	{
		AIOctreeNode * node = m_NodeStack.pop_back();
		//node_queue.pop_back();

		IndexedNode & content = node->getContent();
		if( node->isLeaf() )
		{
			const size_t numPointsInNode = content.m_index.size();
			if( numPointsInNode )
			{
				card32 * tmpIndices = &content.m_index[0];	// TODO_64BIT ATTENTION! Has to be changed for 64 bit! See definition of IndexedNode.
				for( unsigned i=0;i<numPointsInNode;i++ )
				{
					const Vector3f &tmpPos = ps->get3f(*tmpIndices,posAAT);
					float dist = tmpPos*plane.getNormal() - plane_n_dot_p;
					if( fabs(dist) <= maxDistance )
					{
						float sd[3];
						bool skip = false;
						for( mpcard j=0;j<3;j++ )
						{
							sd[j]=boundary[0].calculateSignedDistance(tmpPos);
							if( sd[j] < -maxDistance )
							{
								skip=true;
								break;
							}
						}
						if( skip )
							continue;
						if( sd[0] >= 0 && sd[1] >= 0 && sd[2] >= 0 )
						{
							addIndexToCache(*tmpIndices);
							continue;
						}

						for( mpcard j=0;j<3;j++ )
						{
							float lineProj = boundaryLines[j].computeProjection(tmpPos);
							if( (lineProj >= 0 && lineProj <= 1.0f && boundaryLines[j].computeDistance(tmpPos) <= maxDistance )
								|| norm(tmpPos-points[j]) <= maxDistance )
							{
								addIndexToCache(*tmpIndices);
								break;
							}
						}
					}
					tmpIndices++;
				}
			}
		}
		else
		{
			
			// add children that intersect thick plane truncated by triangle halfspaces (not optimized)
			for( unsigned i=0;i<8;i++ )
			{
				AIOctreeNode * childNode = node->getChild(i);
				if( childNode && thickPlaneIntersectsBoundingBox(plane, maxDistance,childNode->getContent().m_bb) )
				{
					if(		boundingBoxIntersectsHalfSpace(childNode->getContent().m_bb, boundary[0],maxDistance)
						&&	boundingBoxIntersectsHalfSpace(childNode->getContent().m_bb, boundary[1],maxDistance)
						&&	boundingBoxIntersectsHalfSpace(childNode->getContent().m_bb, boundary[2],maxDistance) )
					m_NodeStack.push_back(childNode);
				}
			}


		}
	}
	numPoints = m_NumPoints;
	if(numPoints > 0)
		*indices = &m_IndexCache[0];
	else
		*indices = NULL;
}



void X4::FastSpatialQuery::oobbQuery( const OOBBox3f &oobb,mpcard ** indices, mpcard &numPoints) 
{
	AIOctreeNode * rootNode = m_Octree->getOctree();
	m_NodeStack.clear();
	m_NodeStack.push_back(rootNode);

	m_NumPoints = 0;
	const PointSet * ps = m_PS;
	AAT posAAT = ps->getAAT("position");

	if( oobb.intersect(rootNode->getContent().m_bb) )
	while( !m_NodeStack.empty() )
	{
		AIOctreeNode * node = m_NodeStack.pop_back();
		//node_queue.pop_back();

		IndexedNode & content = node->getContent();
		if( node->isLeaf() )
		{
			const size_t numPointsInNode = content.m_index.size();
			if( numPointsInNode )
			{
				card32 * tmpIndices = &content.m_index[0];	// TODO_64BIT ATTENTION! Has to be changed for 64 bit! See definition of IndexedNode.
				for( unsigned i=0;i<numPointsInNode;i++ )
				{
					const Vector3f &tmpPos = ps->get3f(*tmpIndices,posAAT);
					if( oobb.inBoundingBox(tmpPos) )
					{
						addIndexToCache(*tmpIndices);

					}
					tmpIndices++;
				}
			}
		}
		else
		{
			
			// add children that intersect thick plane (not optimized)
			for( unsigned i=0;i<8;i++ )
			{
				AIOctreeNode * childNode = node->getChild(i);
				if( childNode && oobb.intersect(childNode->getContent().m_bb) )
				{
					m_NodeStack.push_back(childNode);
				}
			}


		}
	}
	numPoints = m_NumPoints;
	if(numPoints > 0)
		*indices = &m_IndexCache[0];
	else
		*indices = NULL;
}
