#include "StdAfx.h"
#include <windows.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#include "FeatureClustering.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "DynamicLinearAlgebraTools.inline.h"
#include "GeometricFeature.h"
#include "LocalDescriptor.h"
#include "UnstructuredInCorePointCloud.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( FeatureClustering ,0)
{
	BEGIN_CLASS_INIT( FeatureClustering );
}

FeatureClustering::FeatureClustering(void)
{
}

FeatureClustering::~FeatureClustering(void)
{
}

void FeatureClustering::assign(const Object* obj, X4CopyContext *context)
{
	// Persistent::assign(obj,context);
	const FeatureClustering * o = dynamic_cast<const FeatureClustering*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

PointCloud * X4::FeatureClustering::pcaTransform3D( PointCloud * pc, FeatureSet * fs /*= NULL */ )
{
	std::vector<DVectorF> points;
	pcaTransform(pc, 3, points, fs);

	UnstructuredInCorePointCloud * ret = new UnstructuredInCorePointCloud;
	VertexDescriptor * vd = createDefaultVertexDescriptor(true,false,false);
	ret->clearAndSetup(vd,points.size());
	PointSet * ps = ret->getPointSet();
	AAT posAAT = ret->getAAT("position");
	for( unsigned i=0;i<points.size();i++ )
	{
		Vector3f v = makeVector3f(points[i][0],points[i][1],points[i][2]);
		ps->set3f(i,posAAT,v);
	}

	delete vd;
	return ret;
}

void X4::FeatureClustering::pcaTransform( PointCloud * pc, card32 numDimensions, std::vector<DVectorF> &transformedSet, FeatureSet * fs /*= NULL */ )
{
	if( fs == NULL)
	{
		fs = dynamic_cast<FeatureSet*>(pc->getAttachments()->getData(FeatureSet::getDefaultName()));
	}
	if( fs == NULL )
	{
		throw X4Exception("FeatureClustering::pcaTransform - pointcloud contains no feature set!");
	}
	if( fs->getNumFeatures() == 0)
		return;

	// setup input for pca
	std::vector<DVectorF> signatures(fs->getNumFeatures());
	GeometricFeature * f = fs->m_Features[0];
	unsigned sigIndex = (unsigned)f->m_Signatures.size();
	for( unsigned i=0;i<f->m_Signatures.size();i++ )
	{
		if( dynamic_cast<FloatVectorSignature*>(f->m_Signatures[i]) )
		{
			sigIndex = i;
			break;
		}
	}
	if( sigIndex == f->m_Signatures.size())
		return;
	for( unsigned i=0;i<fs->getNumFeatures();i++ )
	{
		FloatVectorSignature * sig = (FloatVectorSignature *)fs->m_Features[i]->m_Signatures[sigIndex];
		signatures[i] = sig->getVector();
	}

	DynamicPCA<float> dpca(&signatures[0],(card32)signatures.size());
	DVectorF eigenValues = dpca.calcEigenValues(true);
	DMatrixF eigenVectors_T = dpca.getEigenVectors();
	DVectorF mean = dpca.getCentroid();

	float energy = 0;
	if( eigenValues[0] < 0)
		eigenValues[0] = -eigenValues[0];
	for( unsigned i=0;i<eigenValues.getDim();i++ )
		energy += eigenValues[i];

	debugOutput << "eigenValues:\n";
	float summedEnergy = 0;
	for( unsigned i=0;i<eigenValues.getDim();i++ )
	{
		summedEnergy += eigenValues[i];
		debugOutput << eigenValues[i] <<"(" << summedEnergy / energy << ")\n";
		if( summedEnergy / energy > 0.99f )
			break;
	}

	transformedSet.resize(signatures.size());
	for( unsigned i=0;i<signatures.size();i++ )	
	{
		DVectorF transf = eigenVectors_T * (signatures[i] - mean);
		transformedSet[i].setDim(numDimensions);
		for( unsigned j=0;j<numDimensions;j++ )
		{
			transformedSet[i][j] = transf[j];
		}
	}

}