#include "StdAfx.h"
#include <windows.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#include "FeatureCorrespondence.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ObjectListProperty.h"
#include "ObjectClassProperty.h"

#include <SeparatorClassProperty.h>
#include <HierarchicalKNNIterator.h>
#include <UnstructuredInCorePointCloud.h>
#include <PointCloudTools.h>
#include "GeometricTools.h"
#include "GeometricFeature.h"
//#include <opencv\cxcore.h>
#include "opencv\cxcore.h"

//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( FeatureCorrespondence , 3)
{
	BEGIN_CLASS_INIT( FeatureCorrespondence );

	ADD_VECTOR3F_PROP( m_FeaturePoint[FRAME0], 0);
	ADD_VECTOR3F_PROP( m_FeaturePoint[FRAME1], 0);
	ADD_CARD32_PROP( m_FeatureIndex[FRAME0], 1);
	ADD_CARD32_PROP( m_FeatureIndex[FRAME1], 1);
	ADD_FLOAT32_PROP(m_Distance,2);
    ADD_BOOLEAN_PROP(m_Valid, 3);
}

IMPLEMENT_X4_CLASS( FeatureCorrespondenceSet ,1)
{
	BEGIN_CLASS_INIT( FeatureCorrespondenceSet );
	ADD_OBJECT_LIST_PROP( Correspondence,0,FeatureCorrespondence::getClass() );
   
   ADD_SEPARATOR("Rendering");
   ADD_BOOLEAN_PROP(m_Visible, 1);   
}

FeatureCorrespondence::FeatureCorrespondence(void)
{
	m_Distance = 0;
	m_FeatureIndex[0] = 0;
   m_FeatureIndex[1] = 0;
   m_FeaturePoint[0] = NULL_VECTOR3F;
   m_FeaturePoint[1] = NULL_VECTOR3F;
   m_Valid = true;
}

FeatureCorrespondence::~FeatureCorrespondence(void)
{
}

void FeatureCorrespondence::assign(const Object* obj, X4CopyContext *context)
{
	const FeatureCorrespondence * o = dynamic_cast<const FeatureCorrespondence*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}


//---------------------------------------------------------------------------
X4::FeatureCorrespondenceSet::FeatureCorrespondenceSet() : X4::AttachedData(), X4::RenderableObject()
{
   m_Visible = false;
   for (unsigned i=0; i < 2; i++)
   {
      m_TemporalFeature_hIt[i] = NULL;
      m_TemporalFeaturePC[i] = NULL;
   }
}

X4::FeatureCorrespondenceSet::~FeatureCorrespondenceSet()
{
   deleteCorrespondences(0, (mpcard)m_Correspondences.size());

   for (unsigned i=0; i < 2; i++)
   {
      if (m_TemporalFeature_hIt[i]) delete m_TemporalFeature_hIt[i];
      if (m_TemporalFeaturePC[i]) delete m_TemporalFeaturePC[i];
   }
}

void X4::FeatureCorrespondenceSet::deleteCorrespondences( mpcard begin, mpcard end )
{
	for( unsigned i=begin;i<end;i++ )
		delete m_Correspondences[i];
	m_Correspondences.erase( m_Correspondences.begin() + begin,m_Correspondences.begin() + end);
}

X4::Matrix4f X4::FeatureCorrespondenceSet::computeTransformationMatrix()
{
	Vector3f centroidA = NULL_VECTOR3F;
	Vector3f centroidB = NULL_VECTOR3F;
	for( unsigned i=0;i<m_Correspondences.size();i++ )
	{
		centroidA += m_Correspondences[i]->m_FeaturePoint[0];
		centroidB += m_Correspondences[i]->m_FeaturePoint[1];
	}

	centroidA /= m_Correspondences.size();
	centroidB /= m_Correspondences.size();


	Matrix3f C = IDENTITY3F * 0;
	for( unsigned i=0;i<m_Correspondences.size();i++ )
	{
		C += outerProduct(m_Correspondences[i]->m_FeaturePoint[0] - centroidA,m_Correspondences[i]->m_FeaturePoint[1] - centroidB);
	}

	C /= m_Correspondences.size();

	// Prepare OpenCV Datastructure CvMat
	Matrix3f U,V;
	Vector3f eigenValues;
	CvMat cvMatInput, cvMat_U, cvMat_W, cvMat_Vt;
	cvInitMatHeader( &cvMatInput, 3, 3, CV_32FC1, C.data());
	cvInitMatHeader( &cvMat_W, 3, 1, CV_32FC1, eigenValues.data() );
	cvInitMatHeader( &cvMat_U, 3, 3, CV_32FC1, U.data() );
	cvInitMatHeader( &cvMat_Vt, 3, 3, CV_32FC1, V.data() );

	// Do SVD
	cvSVD(&cvMatInput, &cvMat_W, &cvMat_U, &cvMat_Vt, CV_SVD_MODIFY_A | CV_SVD_U_T );

	Matrix3f rotation = U*V;
	Matrix4f ret = makeTranslation4f(centroidA) * expand3To4(rotation) * makeTranslation4f(-centroidB);
	return ret;
}


void X4::FeatureCorrespondenceSet::sort()
{
	std::vector<TSortEntry> entries;
	for( unsigned i=0;i<m_Correspondences.size();i++ )
	{
		TSortEntry e;
		e.index = i;
		e.dist = m_Correspondences[i]->m_Distance;
	}

	::sort(entries.begin(),entries.end());

	std::vector<FeatureCorrespondence*> corrNew;
	for( unsigned i=0;i<entries.size();i++ )
	{
		corrNew.push_back( m_Correspondences[ entries[i].index]);
	}
}

void X4::FeatureCorrespondenceSet::setupTemporalPC( FeatureCorrespondence::Frame f )
{
   delete m_TemporalFeature_hIt[f];
   delete m_TemporalFeaturePC[f];
   
   m_TemporalFeaturePC[f] = new UnstructuredInCorePointCloud;
   m_TemporalFeaturePC[f]->clearAndSetup(createDefaultVertexDescriptor(true,false,false,false),
   m_Correspondences.size());
   AAT posAAT = m_TemporalFeaturePC[f]->getAAT("position");
   for( unsigned i=0;i<m_Correspondences.size();i++ )
   {
      m_TemporalFeaturePC[f]->getPointSet()->set3f(i,posAAT,m_Correspondences[i]->m_FeaturePoint[f]);
   }
   m_TemporalFeature_hIt[f] = new HierarchicalKNNIterator(m_TemporalFeaturePC[f],32,NULL);

}

X4::Vector3f X4::FeatureCorrespondenceSet::interpolateCorrespondence( const Vector3f& point, float sigma, 
																								 FeatureCorrespondence::Frame frameFrom,
																								 bool * succededPtr, float radius)
{
	if( m_TemporalFeaturePC[frameFrom] == NULL )
		setupTemporalPC(frameFrom);

	FeatureCorrespondence::Frame frameTo = (frameFrom == FeatureCorrespondence::FRAME0) ?
		FeatureCorrespondence::FRAME1 : FeatureCorrespondence::FRAME0;
	const float inv_sigma_2 = 1.0f / sigma / sigma;

	float rad_2 = sigma * sigma * 4.0f;
	if (radius > 0) rad_2 = radius * radius;

	HierarchicalKNNIterator * hIt = m_TemporalFeature_hIt[frameFrom];
	hIt->setSeekPointAndReset(point);
	FeatureCorrespondence * corr = m_Correspondences[hIt->getCurrentPointIndex()];
	if( normQuad(corr->m_FeaturePoint[frameFrom] - point) > rad_2 )
	{
		if( succededPtr)
			*succededPtr = false;
		return corr->m_FeaturePoint[frameTo] - corr->m_FeaturePoint[frameFrom] + point;
	}
	if( succededPtr)
		*succededPtr = true;

	Vector3f ret = NULL_VECTOR3F;
	float summedWeight = 0;
	while( !hIt->atEnd())
	{
		FeatureCorrespondence * corr = m_Correspondences[hIt->getCurrentPointIndex()];
		float dist_2 = normQuad(corr->m_FeaturePoint[frameFrom] - point);
		if( dist_2 > rad_2 )
			break;
		
		float weight = exp(-dist_2 * inv_sigma_2);
		ret += (corr->m_FeaturePoint[frameTo] - corr->m_FeaturePoint[frameFrom]) * weight;
		summedWeight += weight;

		hIt->next();
	}

	return ret / summedWeight + point;
}


//---------------------------------------------------------------------------
float X4::FeatureCorrespondenceSet::compareTo( FeatureCorrespondenceSet* fcs, float sigma )
{
   return compareTo(fcs, sigma, NULL, 0);
}

//---------------------------------------------------------------------------
float FeatureCorrespondenceSet::compareTo(FeatureCorrespondenceSet* fcs, float sigma, UnstructuredInCorePointCloud* framePC[FeatureCorrespondence::FRAME_COUNT], float projectSigma)
{
   FeatureSet* fs[FeatureCorrespondence::FRAME_COUNT];
   memset(fs, 0, sizeof(FeatureSet*) * FeatureCorrespondence::FRAME_COUNT);
   return compareTo(fcs,sigma, framePC, projectSigma, fs);
}

//---------------------------------------------------------------------------
float FeatureCorrespondenceSet::compareTo(FeatureCorrespondenceSet* fcs, float sigma, UnstructuredInCorePointCloud* framePC[FeatureCorrespondence::FRAME_COUNT], float projectSigma, X4::FeatureSet* fs[FeatureCorrespondence::FRAME_COUNT])
{
   float score = 0;
   int numpseudo = 0;
   
   FeatureCorrespondence::Frame fr[2] = {FeatureCorrespondence::FRAME0, FeatureCorrespondence::FRAME1};
   
   for( unsigned f=0;f<2;f++)
   {
      unsigned fn = (f+1) % 2;
      if (fs[f] == NULL || framePC[f] == NULL)
         continue;
         
      /*FeatureCorrespondence::Frame f1,f2;
      if( f == 0 ) 
      {
         f1 = FeatureCorrespondence::FRAME0;
         f2 = FeatureCorrespondence::FRAME1;
      }
      else
      {
         f2 = FeatureCorrespondence::FRAME0;
         f1 = FeatureCorrespondence::FRAME1;
      }*/

      for( unsigned i=0;i<m_Correspondences.size();i++ )
      {
         FeatureCorrespondence * corr = m_Correspondences[i];
         //if (fs[f]->m_Features[corr->m_FeatureIndex[f]]->getIsPseudoFeature())
         //{
         //   numpseudo ++;
         //   continue;
         //}
            
         bool succeeded;
         Vector3f interpolatedCorrespondence = fcs->interpolateCorrespondence(corr->m_FeaturePoint[f], sigma, fr[f], &succeeded);
         if( succeeded )
         {
            // project interpolated correspondence on the given pc
            if (framePC[fn] && projectSigma > 0)
            {
               Vector3f newPos, newNorm;
               projectPointOntoPatch(interpolatedCorrespondence, projectSigma, framePC[fn], newPos, newNorm);
               interpolatedCorrespondence = newPos;
            }
            
            float dist_2 = normQuad(corr->m_FeaturePoint[fn] - interpolatedCorrespondence);
            score += 1.0 / exp(dist_2 / sigma / sigma);
         }
      }

   }
   score /= (float)(m_Correspondences.size()*2 - numpseudo);

   return score;

}

//---------------------------------------------------------------------------
void FeatureCorrespondenceSet::renderGL(X4::SceneObject *obj)
{
   if (!m_Visible) return;

   glPushAttrib(GL_ALL_ATTRIB_BITS);

   glDisable(GL_LIGHTING);
   glDisable(GL_DEPTH_TEST);
   glEnable(GL_BLEND);
   //glBlendFunc(GL_SRC_ALPHA, GL_ONE);
   glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glColor4f(0.5f, 1.0f, 1.0f, 0.5f);
   glBegin(GL_LINES);
   glEnable(GL_LINE_SMOOTH);
   glHint (GL_LINE_SMOOTH_HINT, GL_NICEST);
   glLineWidth (1.0);
   
   // for each feature correspondency do
   for (unsigned i=0; i < m_Correspondences.size(); i++)
   {
      FeatureCorrespondence* fc = m_Correspondences[i];
      glVertex3fv(fc->m_FeaturePoint[0].data());
      glVertex3fv(fc->m_FeaturePoint[1].data());
   }
   
   glDisable(GL_BLEND);
   glEnd();
   
   glPopAttrib();
   
}
