#ifndef FeatureCorrespondenceH
#define FeatureCorrespondenceH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AttachedData.h"
#include "RenderableObject.h"
#include "RenderingTool.h"
//---------------------------------------------------------------------------

namespace X4
{

   class UnstructuredInCorePointCloud;
   class HierarchicalKNNIterator;
   class FeatureSet;
   
/**
	correspondence between two features
*/
class X3FOUNDATION_PRIVATE_API FeatureCorrespondence : public Persistent
{
	X4_CLASS( FeatureCorrespondence )


public:
	FeatureCorrespondence(void);
	virtual ~FeatureCorrespondence(void);

   enum Frame
   {
      FRAME0 = 0, 
      FRAME1 = 1, 
      FRAME_COUNT = 2
   };
   
   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	Vector3f		m_FeaturePoint[FRAME_COUNT];
	card32		m_FeatureIndex[FRAME_COUNT];
	float			m_Distance;
    bool        m_Valid;
};

class X3FOUNDATION_PRIVATE_API FeatureCorrespondenceSet : public AttachedData, public RenderableObject 
{
	X4_CLASS( FeatureCorrespondenceSet )
public:
   FeatureCorrespondenceSet();
	virtual ~FeatureCorrespondenceSet();

	std::vector<FeatureCorrespondence*>	m_Correspondences;
	
	/// default for attachment name
	static std::string getDefaultName() {return "FeatureCorrespondenceSet";}

	Matrix4f computeTransformationMatrix();
	void sort();

	void addCorrespondence( Object * obj ) {m_Correspondences.push_back((FeatureCorrespondence*)obj);}
	void deleteCorrespondences( mpcard begin, mpcard end );
	Object * getCorrespondence( mpcard index ) {return m_Correspondences[index];}
	void moveCorrespondence( mpcard oldPos, mpcard newPos ) {throw "ToDo";} // not necessary}
	mpcard getNumCorrespondences() {return (mpcard)m_Correspondences.size();}
	std::string getCorrespondenceName(mpcard index ) {return "-";}

   // Render object interface
   virtual void renderGL(SceneObject *obj);

   inline bool getVisible() const { return m_Visible; }
   inline void setVisible(bool v) { m_Visible = v; }
   
   // Interpolate point matching based on feature matching
	// when point is to far away from a valid correspondence, then succeded is set to false
   Vector3f interpolateCorrespondence(const Vector3f& point, float sigma, FeatureCorrespondence::Frame frameFrom,
		bool * succeded = NULL, float radius = 0);
   
   // compare this to another correspondence set (0 nothing is equal, 1 100% match)
	// sigma controls how much a correspondence can differ (it's also used as support
	// sigma for the interpolation)
   float compareTo(FeatureCorrespondenceSet* fcs, float sigma);
   
   // same compariosn however use the given pc to project interpolated features on it
   float compareTo(FeatureCorrespondenceSet* fcs, float sigma, UnstructuredInCorePointCloud* framePC[FeatureCorrespondence::FRAME_COUNT], float projectSigma);
   
   // use the same compare method, however it do not compare features which are marked as pseudo features. For this feature set has to be provided
   float compareTo(FeatureCorrespondenceSet* fcs, float sigma, UnstructuredInCorePointCloud* framePC[FeatureCorrespondence::FRAME_COUNT], float projectSigma, X4::FeatureSet* frameFS[FeatureCorrespondence::FRAME_COUNT]);
      
private:
   bool  m_Visible;
   
	struct TSortEntry
	{
		unsigned index;
		float dist;
		bool operator < (const TSortEntry &e) {return dist < e.dist;}
	};
	
	UnstructuredInCorePointCloud * m_TemporalFeaturePC[2];
	HierarchicalKNNIterator * m_TemporalFeature_hIt[2];
	
	void setupTemporalPC(FeatureCorrespondence::Frame f);


};

}

#endif