#ifndef FeatureMatchingH
#define FeatureMatchingH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{
class FeaturePointDetector;
class FeatureCorrespondence;
class FeatureCorrespondenceSet;
class SpectralCorrespondenceValidation;
class FeatureSet;
class PointCloud;
/**
	combines all modules for feature matching
*/
class X3FOUNDATION_PRIVATE_API FeatureMatching : public Persistent
{
	X4_CLASS( FeatureMatching )

public:
	FeatureMatching(void);
	~FeatureMatching(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   /// Compute correspondance set. If given detector is NULL then the default one is used.
	FeatureCorrespondenceSet * computeInitialCorrespondences(FeatureSet * f1, FeatureSet * f2, FeaturePointDetector* fd = NULL);
	FeatureCorrespondenceSet * computeValidatedCorrespondences(FeatureSet * f1, FeatureSet * f2, FeaturePointDetector* fd = NULL);
	FeatureCorrespondenceSet * computeValidatedCorrespondences(FeatureSet * f1, FeatureSet * f2,FeatureCorrespondenceSet * initialSet, FeaturePointDetector* fd = NULL);

	FeatureCorrespondenceSet * computeInitialCorrespondences(PointCloud * pc1,PointCloud * pc2, FeaturePointDetector* fd = NULL);
	FeatureCorrespondenceSet * computeValidatedCorrespondences(PointCloud * pc1,PointCloud * pc2, FeaturePointDetector* fd = NULL);
	
	FeaturePointDetector * getFeatureDetector() {return m_FeatureDetector;}

	card32 getMaxNumCorrespondingFeatures() const {return m_MaxNumCorrespondingFeatures;}
	void setMaxNumCorrespondingFeatures(card32 v) {m_MaxNumCorrespondingFeatures=v;}

	card32 getMaxNumInitialCorrespondences() const {return m_MaxNumInitialCorrespondences;}
	void setMaxNumInitialCorrespondences(card32 v) {m_MaxNumInitialCorrespondences=v;}

private:

	FeaturePointDetector*	m_FeatureDetector;
	SpectralCorrespondenceValidation*	m_SpectralCV;
	card32		m_MaxNumCorrespondingFeatures;	// for removing ambigous features for rigid registration
	card32		m_MaxNumInitialCorrespondences;






};

}

#endif