#include "StdAfx.h"
#include <windows.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#include "FeaturePointDetector.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ObjectListProperty.h"
#include "LocalDescriptor.h"
#include "PointCloudTools.h"
#include "GeometricFeature.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_ABSTRACT_CLASS( FeaturePointDetector ,0)
{
	BEGIN_CLASS_INIT( FeaturePointDetector );
	ADD_OBJECT_LIST_PROP( Descriptor, 0, LocalDescriptor::getClass());
	ADD_FLOAT32_PROP(m_DescriptorRadius,0);
}

FeaturePointDetector::FeaturePointDetector(void)
{
	progress = NULL;
	m_DescriptorRadius = 15.0f;
}

FeaturePointDetector::~FeaturePointDetector(void)
{
	deleteDescriptors(0, (card32)m_Descriptors.size());
}

void FeaturePointDetector::assign(const Object* obj, X4CopyContext *context)
{
	const FeaturePointDetector * o = dynamic_cast<const FeaturePointDetector*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::FeaturePointDetector::recomputeFeatureSignatures( UnstructuredInCorePointCloud * upc, std::string attachmentName/*="" */ )
{
	if( attachmentName == "" )
		attachmentName = FeatureSet::getDefaultName();
	FeatureSet * fs = (FeatureSet*)upc->getAttachments()->getData(attachmentName);

	if( fs == NULL )
	{
		debugOutput << "no featureset present!\n";
		return;
	}

	for( unsigned i=0;i<m_Descriptors.size();i++ )
	{
		LocalDescriptor * d = m_Descriptors[i];
		if( !d->isRecomputeable() )
			continue;

		d->beginSignatureComputations(upc);
		
		for( unsigned j=0;j<fs->m_Features.size();j++ )
		{
			GeometricFeature * f = fs->m_Features[j];
			LocalSignature * sig = d->computeSignature(f->m_Position, f->m_Normal, m_MedianPointDistance * m_DescriptorRadius,
				upc);
			if (sig == NULL) continue;
			if( f->m_Signatures.size() > i)
			{
				delete f->m_Signatures[i];
				f->m_Signatures[i] = sig;
			}
			else
				f->m_Signatures.push_back(sig);

		}

		d->endSignatureComputations();
	}
}

void X4::FeaturePointDetector::recomputeFeatureSignatures( UnstructuredInCorePointCloud * upc, FeatureSet * fs )
{
	for( unsigned i=0;i<m_Descriptors.size();i++ )
	{
		LocalDescriptor * d = m_Descriptors[i];
		if( !d->isRecomputeable() )
			continue;

		d->beginSignatureComputations(upc);

		for( unsigned j=0;j<fs->m_Features.size();j++ )
		{
			GeometricFeature * f = fs->m_Features[j];
			LocalSignature * sig = d->computeSignature(f->m_Position, f->m_Normal, m_MedianPointDistance * m_DescriptorRadius,
				upc);
			if (sig == NULL) continue;
			if( f->m_Signatures.size() > i)
			{
				delete f->m_Signatures[i];
				f->m_Signatures[i] = sig;
			}
			else
				f->m_Signatures.push_back(sig);

		}

		d->endSignatureComputations();
	}
}

void X4::FeaturePointDetector::deleteDescriptors( card32 begin, card32 end )
{
	for( unsigned i=begin;i<end;i++)
		delete m_Descriptors[i];
	m_Descriptors.erase(m_Descriptors.begin() + begin, m_Descriptors.begin() + end);
}

Object * X4::FeaturePointDetector::getDescriptor( card32 index )
{
	return m_Descriptors[index];
}

void X4::FeaturePointDetector::begin( const std::vector<UnstructuredInCorePointCloud*>& clouds )
{
	m_MedianPointDistance = getMedianPointDistance(clouds[0]);
	beginDetection(clouds);
}