//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "GLImageCombiner.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
#include "GLShaderMaterial.h"
#include "Configuration.h"
#include "StringHelper.h"
#include "GLRenderTextureCache.h"
#include "GLImage.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra.inline.h"
#include "GLHelper.h"
#include "ApplicationDirectories.h"

//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;


MetaClass* GLImageCombiner::GLImageCombinerMetaClass = NULL;

const MetaClass* GLImageCombiner::getInstanceClass() const {
   return getClass();
}

Object* GLImageCombiner::newInstance() {
   return new GLImageCombiner;
}

const MetaClass* GLImageCombiner::getClass() {
   if (GLImageCombinerMetaClass==NULL) {
      GLImageCombinerMetaClass = new MetaClass("GLImageCombiner", "no description.",
                                  newInstance, sizeof(GLImageCombiner));
   }
   return GLImageCombinerMetaClass;
}

void GLImageCombiner::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   GLImageCombinerMetaClass->copyParentProperties();

   GLImageCombinerMetaClass->addProperty(
      new StringClassProperty( "combiner op", MetaClass::FIRST_VERSION + 0,
         (GetStringPropMethod)&getCombineOp,
         (SetStringPropMethod)&setCombineOp  ) );
}

void GLImageCombiner::shutdown() {
   delete GLImageCombinerMetaClass;
}


// ----

GLImageCombiner::GLImageCombiner()
{
   m_CombineOp = "O = I0;";

   loadTemplates();
}

GLImageCombiner::~GLImageCombiner()
{
}

void GLImageCombiner::assign(const Object* obj, X4CopyContext *context)
{
   GLImageMaterialOperator::assign( obj, context );
   const GLImageCombiner * ic = dynamic_cast<const GLImageCombiner*>(obj);
   x4Assert( ic );

   m_CombineOp = ic->m_CombineOp;
}

#ifdef _WIN32
#define X4GetProcAddress2(PROC_TYPE, PROC_NAME, PROC_STR) PROC_NAME = (PROC_TYPE)wglGetProcAddress(PROC_STR)
#endif

GLImage * GLImageCombiner::execute(GLRenderTextureCache * cache)
{
   x4Assert( m_InputImages.size() != 0 );

   // get output rendertexture from cache
   GLImage * ret = new GLImage;
   bool isFloat = false;
   bool isFloat32 = false;
   switch( getOutputFormat() )
   {
   case OF_Undefined:
      // take inputformat
      isFloat = m_InputImages[0]->isFloat();
      isFloat32 = m_InputImages[0]->isFloat32();
   break;
   case OF_Float32:
      isFloat32 = true;
   case OF_Float16:
      isFloat = true;
   break;
   }

   card32 width = m_InputImages[0]->getWidth();
   card32 height = m_InputImages[0]->getHeight();
   ret->setup( cache, width , height,
      m_InputImages[0]->getNumComponents(), m_InputImages[0]->isRectangle(),
         isFloat, isFloat32 );

	if( glVertexAttrib2fARB == NULL )
	{
		X4GetProcAddress2( PFNGLVERTEXATTRIB2FARBPROC, glVertexAttrib2fARB, "glVertexAttrib2fARB" );
	}


   // #####################################
   // --------- render to texture --------------
   ret->activate();
   glPushAttrib( GL_ALL_ATTRIB_BITS );

   GLHelper::begin2D( width, height );
   bindMaterial( cache->getContext() );

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
       m_InputImages[i]->bind(i);
   }

   glBegin( GL_QUADS );

   std::vector<Vector2f> coords(m_InputImages.size()*2);
   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      coords[i*2+0] = makeVector2f(0,0);
      if( m_InputImages[i]->isRectangle() )
         coords[i*2+1] = makeVector2f(m_InputImages[i]->getWidth(),
                     m_InputImages[i]->getHeight());
      else
         coords[i*2+1] = makeVector2f(1,1);
   }

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      glVertexAttrib2fARB( i+1,     coords[i*2+0][0],coords[i*2+0][1] );
   }
   glVertexAttrib2fARB(0, 0,0);

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      glVertexAttrib2fARB( i+1,     coords[i*2+1][0],coords[i*2+0][1] );
   }
   glVertexAttrib2fARB(0, width, 0);

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      glVertexAttrib2fARB( i+1,     coords[i*2+1][0],coords[i*2+1][1] );
   }
   glVertexAttrib2fARB(0, width, height);

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      glVertexAttrib2fARB( i+1,     coords[i*2+0][0],coords[i*2+1][1] );
   }
   glVertexAttrib2fARB(0, 0, height);

   glEnd();


   // #####################################
   // ----------- clean up ----------------

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
       m_InputImages[i]->unbind();
   }

   GLHelper::end2D();
   glPopAttrib();

   ret->deactivate();
   unbindMaterial( cache->getContext() );


   return ret;
}

void GLImageCombiner::loadTemplates()
{
   m_VSTemplate = readTextFile( ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/combine_vs.txt" );
   m_PSTemplate = readTextFile( ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/combine_ps.txt" );
}

void GLImageCombiner::modify( std::string &vsTemplate, std::string &psTemplate )
{
   forget(vsTemplate);
   psTemplate = replaceSubStrings( psTemplate, "$COMBINE_OP", m_CombineOp );
}


