//---------------------------------------------------------------------------
#include "stdafx.h"
#include <windows.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#include "GLImageOperator.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
#include "NumericalClassProperty.inline.h"
#include <fstream>
#include "GLRenderTextureFBO.h"
#include "GLShaderMaterial.h"
#include "Shader.h"
#include "GLImage.h"
#include "StringHelper.h"
#include "debugOutput.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;

MetaClass* GLImageOperator::GLImageOperatorMetaClass = NULL;

const MetaClass* GLImageOperator::getInstanceClass() const {
   return getClass();
}


const MetaClass* GLImageOperator::getClass() {
   if (GLImageOperatorMetaClass==NULL) {
      GLImageOperatorMetaClass = new MetaClass("GLImageOperator", "no description.",
                                  NULL, sizeof(GLImageOperator));
   }
   return GLImageOperatorMetaClass;
}


std::string descOF( card32 of )
{
   switch(of)
   {
   case GLImageOperator::OF_Undefined:
      return "undefined";
   case GLImageOperator::OF_Byte:
      return "byte";
   case GLImageOperator::OF_Float16:
      return "float16";
   case GLImageOperator::OF_Float32:
      return "float32";
   }
   return "bad input";
}

void GLImageOperator::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   GLImageOperatorMetaClass->copyParentProperties();

	ADD_CARD32_PROP_METHOD(numInputImages, 0, &getNumInputImages, &setOutputFormat)
   ADD_CARD32_PROP_METHOD_DESCR(format, 0, &getOutputFormat , &setOutputFormat, &descOF, 4, NULL)
/*
   GLImageOperatorMetaClass->addProperty( new Card32ClassProperty( "num input images",
      MetaClass::FIRST_VERSION + 0,(GetCard32PropMethod)getNumInputImages,
                              (SetCard32PropMethod)setNumInputImages) );
   GLImageOperatorMetaClass->addProperty( new Card32ClassProperty( "output format",
      MetaClass::FIRST_VERSION + 0,(GetCard32PropMethod)getOutputFormat,
                              (SetCard32PropMethod)setOutputFormat,
                              (Card32ClassProperty::GetEnumDescriptionMethod)descOF) );

*/
}

void GLImageOperator::shutdown() {
   delete GLImageOperatorMetaClass;
}


// ----

GLImageOperator::GLImageOperator()
{
   m_OutputFormat = OF_Undefined;
}

GLImageOperator::~GLImageOperator()
{
}

void GLImageOperator::assign(const Object* obj, X4CopyContext *context)
{
   forget(context);
   const GLImageOperator * op = dynamic_cast<const GLImageOperator*>(obj);
   x4Assert( op );

   m_InputImages.resize( op->m_InputImages.size() );
}

void GLImageOperator::setNumInputImages(card32 num)
{
   m_InputImages.resize( num );
   for( card32 i=0;i<m_InputImages.size();i++ )
      m_InputImages[i] = NULL;
   numInputImagesChanged();
}

void GLImageOperator::setInputImage( card32 i, GLImage * img )
{
   x4Assert( i < m_InputImages.size() );
   m_InputImages[i] = img;
}


std::string GLImageOperator::readTextFile( std::string filename)
{
   fstream input( filename.c_str() );
   if( !input.is_open() ) {
		X4::warning(string("GLImageOperator::readTextFile() -- File \"")+filename+string("\" not found."));
      return "";
	}

   std::string ret;
   std::string line;
   while( getline( input, line ) )
      ret += line + "\n";

   input.close();
   return ret;
}

GLImageMaterialOperator::GLImageMaterialOperator()
{
   m_Modified = true;
   m_CompilerError = false;
}

GLImageMaterialOperator::~GLImageMaterialOperator()
{
   clearMaterials();
}

void GLImageMaterialOperator::clearMaterials()
{
   for( std::map<std::string,GLShaderMaterial *>::iterator iter = m_Materials.begin();
      iter != m_Materials.end(); ++iter )
   {
      delete iter->second;
   }
}

void GLImageMaterialOperator::assign(const Object* obj, X4CopyContext *context)
{
   GLImageOperator::assign( obj, context );
   const GLImageMaterialOperator * op = dynamic_cast<const GLImageMaterialOperator*>(obj);
   x4Assert( op != NULL );

   clearMaterials();

   m_VSTemplate = op->m_VSTemplate;
   m_PSTemplate = op->m_PSTemplate;
   m_Modified = true;

}

void GLImageMaterialOperator::bindMaterial( GLContext * context )
{
   if( m_Modified )
   {
       clearMaterials();
       m_Modified = false;
   }

   std::string desc = getDesc( );
   if( m_Materials.find( desc ) == m_Materials.end() )
   {
      GLShaderMaterial * mat = new GLShaderMaterial;
      mat->getVertexShader()->setShaderType( Shader::VERTEX );
      mat->getVertexShader()->setShaderClass( Shader::CLASS_CG );
      mat->getFragmentShader()->setShaderClass( Shader::CLASS_CG );

      m_Materials[desc] = mat;

      std::string vsInput, vsOutput, vsProgram, psInput, psReadTex, psParam;

      for( card32 i=0;i<m_InputImages.size();i++ )
      {
         char c = (char)('0' + i);
         char c_plus_1 = (char)('0' + i + 1);
         bool isRect = m_InputImages[i]->getRenderTexture()->isRectangle();
         std::string attributes;
         attributes += "\tfloat2 tex";
         attributes += c;
         attributes += "\t: TEXCOORD";
         attributes += c;
         attributes += ";\n";
         vsOutput += attributes;
         psInput += attributes;

         vsInput += "\tfloat2 tex";
         vsInput += c;
         vsInput += "\t: ATTR";
         vsInput += c_plus_1;
         vsInput += ";\n";


         vsProgram += std::string("OUT.tex") + c + " = IN.tex" + c + ";\n";

         psParam += ", uniform sampler";
         psParam += (isRect) ? "RECT":"2D";
         psParam += " texture";
         psParam += c;
         psParam += "\n";

         // I? = tex?( texture?, IN.tex? )
         psReadTex += std::string("float4 I") + c + " = tex";
         psReadTex += (isRect) ? "RECT":"2D";
         psReadTex += std::string("( texture") + c + ", IN.tex" + c + " );\n";

      }
      std::string vshader = m_VSTemplate;
      std::string pshader = m_PSTemplate;

      vshader = replaceSubStrings( vshader, "$VS_INPUT", vsInput );
      vshader = replaceSubStrings( vshader, "$VS_OUTPUT", vsOutput );
      vshader = replaceSubStrings( vshader, "$VS_PROGRAM", vsProgram );
      pshader = replaceSubStrings( pshader, "$PS_INPUT", psInput );
      pshader = replaceSubStrings( pshader, "$PS_PARAM", psParam );
      pshader = replaceSubStrings( pshader, "$PS_READ_TEX", psReadTex );

      modify( vshader, pshader );

//      debugOutput << " - - - - - -   vertex shader - - - - - - -"
         //<< vshader << " - - - - - -   fragment shader - - - - - - -"
         //<< pshader ;

      mat->getVertexShader()->setSource( vshader );
      mat->getFragmentShader()->setSource( pshader );

   }


   m_Materials[desc]->bindShaders( context );

}

void GLImageMaterialOperator::unbindMaterial( GLContext * context )
{
   x4Assert( m_Materials.find(getDesc()) != m_Materials.end() );
   m_Materials[getDesc()]->unbindShaders( context );
}



std::string GLImageMaterialOperator::getDesc()
{
   std::string ret;
   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      if( m_InputImages[i]->isRectangle() )
         ret += "R";
      else
         ret += "2D";
   }
   return ret;
}

void GLImageMaterialOperator::numInputImagesChanged()
{
   clearMaterials();
}


