//---------------------------------------------------------------------------
#ifndef GLImageOperatorH
#define GLImageOperatorH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------
#include <string>
#include <map>

namespace X4
{

class GLImage;
class GLRenderTextureCache;
class GLShaderMaterial;
class GLContext;

class X3FOUNDATION_PRIVATE_API GLImageOperator : public Persistent
{
private:
   static MetaClass *GLImageOperatorMetaClass;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

protected:

   virtual void numInputImagesChanged() {}

   // returns "" if file doesn't exist
   std::string readTextFile( std::string filename );

   card32            m_OutputFormat;

   // non-properties
   std::vector<GLImage*>  m_InputImages;


public:

   GLImageOperator();
   virtual ~GLImageOperator();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   /// process images and stores the result in a new GLImage
   /// GLRenderTextureCache is a context-bound set of images to handle multiple
   /// contexts and cache render textures
   virtual GLImage * execute(GLRenderTextureCache * cache) = 0;

   /// process images and stores result in outputImage at position [x,y]
   virtual void execute(GLRenderTextureCache * cache, GLImage * outputImage, card32 x, card32 y ) = 0 ;

   mpcard getNumInputImages() {return m_InputImages.size(); }
   void setNumInputImages(card32 num);
   void setInputImage( card32 i, GLImage * img );

   enum OutputFormat
   {
      OF_Undefined,
      OF_Byte,
      OF_Float16,
      OF_Float32
   };

   void setOutputFormat( card32 format ) { m_OutputFormat = format; }
   card32 getOutputFormat() { return m_OutputFormat ; }


};


class X3FOUNDATION_PRIVATE_API GLImageMaterialOperator : public GLImageOperator
{
protected:

   virtual void numInputImagesChanged();

   /// generates material(if neccessary) and binds it
   void bindMaterial( GLContext * context );

   /// unbind material
   void unbindMaterial( GLContext * context );


   /// creates a description ( depence on inputtextures )
   std::string getDesc();

   /// delete all materials
   void clearMaterials();

   /// special replacements in derivates
   virtual void modify( std::string &vsTemplate, std::string &psTemplate ) {}

   std::map<std::string,GLShaderMaterial *>   m_Materials;
   std::string               m_VSTemplate;
   std::string               m_PSTemplate;
   bool                 m_Modified;
   bool                 m_CompilerError;


public:
   GLImageMaterialOperator();
   virtual ~GLImageMaterialOperator();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   bool getCompilerError() const {return m_CompilerError;}

};


}

#endif
 