//---------------------------------------------------------------------------
#include "stdafx.h"
#include <windows.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#include "GLImageReducer.h"
#include "GLImage.h"
#include "GLHelper.h"
#include "GLRenderTextureCache.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra.inline.h"
#include "StringHelper.h"
#include "Configuration.h"
#include "ApplicationDirectories.h"

//---------------------------------------------------------------------------
#include "StringClassProperty.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;

MetaClass* GLImageReducer::GLImageReducerMetaClass = NULL;

const MetaClass* GLImageReducer::getInstanceClass() const {
   return getClass();
}

Object* GLImageReducer::newInstance() {
   return new GLImageReducer;
}

const MetaClass* GLImageReducer::getClass() {
   if (GLImageReducerMetaClass==NULL) {
      GLImageReducerMetaClass = new MetaClass("GLImageReducer", "no description.",
                                  newInstance, sizeof(GLImageReducer));
   }
   return GLImageReducerMetaClass;
}

void GLImageReducer::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   GLImageReducerMetaClass->copyParentProperties();

   GLImageReducerMetaClass->addProperty(
      new StringClassProperty( "reduce op", MetaClass::FIRST_VERSION + 0,
         (GetStringPropMethod)&getReduceOp,
         (SetStringPropMethod)&setReduceOp  ) );

}

void GLImageReducer::shutdown() {
   delete GLImageReducerMetaClass;
}


// ----

GLImageReducer::GLImageReducer()
{
   m_ReduceOp = "O = I0 + I1 + I2 + I3;";

   loadTemplates();
}

GLImageReducer::~GLImageReducer()
{
}

void GLImageReducer::assign(const Object* obj, X4CopyContext *context)
{
   const GLImageReducer * r = dynamic_cast<const GLImageReducer*>(obj);
   x4Assert( r != NULL );

   GLImageMaterialOperator::assign( obj, context );
   m_ReduceOp = r->m_ReduceOp;
}

GLImage * GLImageReducer::execute(GLRenderTextureCache * cache)
{
   x4Assert( m_InputImages.size() != 0 );
   x4Assert( m_InputImages[0]->isRectangle() );

   // get output rendertexture from cache
   GLImage * ret = new GLImage;
   bool isFloat = false;
   bool isFloat32 = false;
   switch( getOutputFormat() )
   {
   case OF_Undefined:
      // take inputformat
      isFloat = m_InputImages[0]->isFloat();
      isFloat32 = m_InputImages[0]->isFloat32();
   break;
   case OF_Float32:
      isFloat32 = true;
   case OF_Float16:
      isFloat = true;
   break;
   }

   card32 size = 1;

   while( (size < m_InputImages[0]->getWidth()) || (size < m_InputImages[0]->getHeight()) )
   {
      size *= 2;
   }
   size /= 2;

   ret->setup( cache, size , size,
      m_InputImages[0]->getNumComponents(), true,
         isFloat, isFloat32 );

   execute( cache, ret, 0,0 );

   return ret;

}

void GLImageReducer::execute(GLRenderTextureCache * cache, GLImage * outputImage, card32 x, card32 y )
{
   x4Assert( m_InputImages.size() != 0 );
   x4Assert( m_InputImages[0]->isRectangle() );

   card32 size = 1;

   while( (size < m_InputImages[0]->getWidth()) || (size < m_InputImages[0]->getHeight()) )
   {
      size *= 2;
   }
   size /= 2;

   // #####################################
   // --------- render to texture --------------
   outputImage->activate();
   glPushAttrib( GL_ALL_ATTRIB_BITS );

   GLHelper::begin2D( outputImage->getWidth(), outputImage->getHeight() );
   bindMaterial( cache->getContext() );


   for( card32 i=0;i<m_InputImages.size();i++ )
   {
       m_InputImages[i]->bind(i);
   }

   glBegin( GL_QUADS );

   std::vector<Vector2f> coords(m_InputImages.size()*2);
   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      coords[i*2+0] = makeVector2f(-0.5f,-0.5f);
      coords[i*2+1] = makeVector2f(m_InputImages[i]->getWidth()-0.5f,
                  m_InputImages[i]->getHeight()-0.5f);
   }

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
	   GLHelper::gl_vertexAttrib2f( i+1,     coords[i*2+0][0],coords[i*2+0][1] );
   }
   GLHelper::gl_vertexAttrib2f(0, x,y);

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      GLHelper::gl_vertexAttrib2f( i+1,     coords[i*2+1][0],coords[i*2+0][1] );
   }
   GLHelper::gl_vertexAttrib2f(0, x+size, y);

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      GLHelper::gl_vertexAttrib2f( i+1,     coords[i*2+1][0],coords[i*2+1][1] );
   }
   GLHelper::gl_vertexAttrib2f(0, x+size, y+size);

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
      GLHelper::gl_vertexAttrib2f( i+1,     coords[i*2+0][0],coords[i*2+1][1] );
   }
   GLHelper::gl_vertexAttrib2f(0, x, y+size);

   glEnd();


   // #####################################
   // ----------- clean up ----------------

   for( card32 i=0;i<m_InputImages.size();i++ )
   {
       m_InputImages[i]->unbind();
   }

   GLHelper::end2D();
   glPopAttrib();

   outputImage->deactivate();
   unbindMaterial( cache->getContext() );

}


void GLImageReducer::loadTemplates()
{
   m_VSTemplate = readTextFile( ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/reduce2x2_vs.txt" );
   m_PSTemplate = readTextFile( ApplicationDirectories::getResourcesDir() + "shaders/hqRenderer/reduce2x2_ps.txt" );
}

void GLImageReducer::modify( std::string &vsTemplate, std::string &psTemplate )
{
   forget(vsTemplate);
   psTemplate = replaceSubStrings( psTemplate, "$REDUCE_OP", m_ReduceOp );
}


