//---------------------------------------------------------------------------
#ifndef GLImageWarperH
#define GLImageWarperH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "GLImageOperator.h"
#include <map>
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace X4
{

class GLTextureRect;
class GLContext;

/**
   warps an image based on a warping texture ( RECT )
*/
class X3FOUNDATION_PRIVATE_API GLImageWarper : public GLImageMaterialOperator
{
private:
   static MetaClass *GLImageWarperMetaClass;

   std::string            m_WarpOp;
   std::map<card32,GLTextureRect *>   m_WarpTextures;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   GLImageWarper();
   ~GLImageWarper();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual GLImage * execute(GLRenderTextureCache * cache);
   virtual void execute(GLRenderTextureCache * cache, GLImage * outputImage, card32 x, card32 y ){x4Assert(false);}

   std::string getWarpOp() const { return m_WarpOp; }
   void setWarpOp( std::string op ) {m_WarpOp = op; m_Modified = true; }

   /// force rebuild of warp texture
   void updateWarpTexture();


protected:


   void clearWarpTextures();

   void loadTemplates();
   virtual void modify( std::string &vsTemplate, std::string &psTemplate );

   /// calculate warp coordinates, x is the coordinate in the destination image,
   /// dim holds the image dimension, returns source-coordinates with repsect to x and dim
   /// imagewarper operates the following way: newImage[x] = oldImage[ calculateWarpCoords(x) ]
   virtual Vector2f calculateWarpCoords( const Vector2f x, const Vector2i dim ) = 0;

   /// builds warp texture with calculateWarpCoords()
   GLTextureRect * buildWarpTexture( GLContext * context );

};










}

#endif
 