//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GridBasedPointSetKNNQuery.h"
//---------------------------------------------------------------------------
#include "GridQuery.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



GridBasedPointSetKNNQuery::GridBasedPointSetKNNQuery(const PointSet *ps, float32 maximumDistance) 
: internalQuery(ps, maximumDistance)
{
   index = 0;
   this->ps = ps;
   positionAAT = ps->getAAT("position", 3, VAD::DATA_FORMAT_FLOAT32);
}

void GridBasedPointSetKNNQuery::setTargetPoint(const Vector3f &targetPoint) {
   sorted.clear();
   std::vector<int32> qResult;
   internalQuery.getIndices(targetPoint, qResult);
   sorted.resize(qResult.size());
   for (card32 i=0; i<qResult.size(); i++) {
      sorted[i].index = qResult[i];
      sorted[i].distanceQuad = normQuad(targetPoint - ps->get3f(qResult[i], positionAAT));
   }
   sort(sorted.begin(), sorted.end());
   index = 0;
}

void GridBasedPointSetKNNQuery::getNextNearestPointIndex(card32 &nextIndex, bool &found) {
   if (index < sorted.size()) {
      nextIndex = sorted[index].index;
      index++;
      found = true;
      return;
   } else {
      nextIndex = -1;
      found = false;
      return;
   }
}



/*
static inline float32 clamp0(const float32 x) {return x>=0?x:0;}
static inline float32 minSphereDist(const Sphere3f &s, const Vector3f &p) {
   return clamp0(norm(p-s.center)-s.radius);
}

GridBasedPointSetKNNQuery::GridBasedPointSetKNNQuery(const PointSet *ps, const Vector3f &targetPoint, float32 cellSize)
: query(cellSize, ps->getBoundingBox().lowerCorner)
 {
	if (ps == NULL) throw X4Exception("GridBasedPointSetKNNQuery - ps must not be NULL.");
	this->ps = ps;
   overallBB = ps->getBoundingBox();
   this->targetPoint = targetPoint;
   this->positionAAT= ps->getAAT("position", 3, VAD::DATA_FORMAT_FLOAT32);
   Vector3i startCell = query.getCell(targetPoint);
   Sphere3f startBS = query.getBoundingBox(startCell).getBoundingSphere();
   pqBoxes.push( BoxEntry(startCell, startBS, minSphereDist(startBS, targetPoint)) );
   
   const card32 numPts = ps->getNumEntries();
   for (card32 i=0; i<numPts; i++) {
      Vector3f pos = ps->get3f(i, positionAAT);
      query.addPoint(query.getCell(pos), i);
   }
}

void GridBasedPointSetKNNQuery::setTargetPoint(const Vector3f& targetPoint) {
   this->targetPoint = targetPoint;

   static const std::priority_queue<PointEntry> EMPTY_POINTS;
   static const std::priority_queue<BoxEntry> EMPTY_BOXES;

   pqBoxes = EMPTY_BOXES;
   pqPoints = EMPTY_POINTS;

   Vector3i startCell = query.getCell(targetPoint);
   Sphere3f startBS = query.getBoundingBox(startCell).getBoundingSphere();
   pqBoxes.push( BoxEntry(startCell, startBS, minSphereDist(startBS, targetPoint)) );
   visited.clear();
}

void GridBasedPointSetKNNQuery::getNextNearestPointIndex(card32 &nextIndex, bool &found) {
   // check for end of points
   if (pqBoxes.empty() && pqPoints.empty()) {   // no more points
      nextIndex = 0xFFFFFFFF;
      found = false;
      return;
   }
   
   while (true) {
      if (!pqPoints.empty()) {
         if (pqBoxes.empty() || pqPoints.top().dist <= pqBoxes.top().dist) { // point nearer than any box
            card32 i = pqPoints.top().index;
            pqPoints.pop();
				nextIndex = i;
				found = true;
            return;
         }
      }

      // step into node
      if (pqBoxes.empty()) {
         nextIndex = 0xFFFFFFFF;
         found = false; 
         return;
      }
      
      Vector3i nextBox = pqBoxes.top().box;
      const std::vector<int32> *newPoints = query.query(nextBox);
      pqBoxes.pop();
      if (newPoints) {
         const unsigned size = newPoints->size();
         for(unsigned i=0; i<size; i++) {
            card32 index = (*newPoints)[i];
            Vector3f pos = ps->get3f(index, positionAAT);
            PointEntry p(pos, index, norm(targetPoint-pos));
            pqPoints.push(p);
         }
      }
      addNextBoxes(nextBox);
   } 
}




void GridBasedPointSetKNNQuery::addNextBoxes(const Vector3i &cell) 
{
   visited.setObject(cell, true);
   for (int32 z=-1; z<1; z++) {
      for (int32 y=-1; y<1; y++) {
         for (int32 x=-1; x<1; x++) {
            Vector3i pos = cell+makeVector3i(x,y,z);
            bool forget;
            if (!visited.getObject(pos, forget)) {
               visited.setObject(pos, true);
               BoundingBox3f bb = query.getBoundingBox(pos);
               if (bb.intersectBoundingBox(overallBB) == BVQueryDisjoint) continue;
               Sphere3f bs = bb.getBoundingSphere();
               pqBoxes.push( BoxEntry(pos, bs, minSphereDist(bs, targetPoint)) );
            }
         }
      }
   }
}
*/