//---------------------------------------------------------------------------
#ifndef GridBasedPointSetKNNQueryH
#define GridBasedPointSetKNNQueryH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
//#include "VertexDescriptor.h"
//#include "SparseGrid.h"
//#include "GridQuery.h"
#include "GridBasedSphericalRangeQuery.h"
//---------------------------------------------------------------------------

namespace X4 {


/// compute k-nearest neighbors efficiently using a hierarchical data structure
class X3FOUNDATION_PRIVATE_API GridBasedPointSetKNNQuery {
 private:
   struct Entry {
      float32 distanceQuad;
      int32 index;
      bool operator<(const Entry &other) const {return distanceQuad < other.distanceQuad;}
   };

   GridBasedSphericalRangeQuery internalQuery;
   std::vector< Entry > sorted;
   AAT positionAAT;
   const PointSet *ps;
   unsigned index;
   
 public:   
   GridBasedPointSetKNNQuery(const PointSet *ps, float32 maximumDistance);
	/// reset position in space where search starts
   void setTargetPoint(const Vector3f &targetPoint);
   /// get next nearest point. found is true if another point has been found. if found is false, nextIndex is undefined.
   void getNextNearestPointIndex(card32 &nextIndex, bool &found);
};



/*  --- buggy code

class PointSet;
class GridQuery3f;

///
///   computes k-nearest neighbors within a limited neighborhood (maximum distance)
///   using a grid data structure (more efficient for localized queries with a limited number of points)
///
class DEFORMATION_API GridBasedPointSetKNNQuery {
 private:
   class NULL_CLASS {};
 
   GridQuery<float32, int32, 3, int32> query;
   const PointSet *ps;
   AAT positionAAT;
   Vector3f targetPoint;
   Vector3i startCell;
   SparseGrid<3, bool, NULL_CLASS> visited;
   BoundingBox3f overallBB;
   
   struct PointEntry {
      Vector3f point;
      card32 index;
      float32 dist;

      PointEntry() {}
      PointEntry(Vector3f point, card32 index, float32 dist) {
         this->point = point; this->index = index; this->dist = dist;
      }
      inline bool operator<(const PointEntry &other) const {return dist > other.dist;} // minHeap
   };
   struct BoxEntry {
      Vector3i box;
      Sphere3f bs;
      float32 dist;
      
      BoxEntry() {}
      BoxEntry(Vector3i box, Sphere3f bs, float32 dist) {
         this->box = box; this->bs = bs; this->dist = dist;
      }
      inline bool operator<(const BoxEntry &other) const {return dist > other.dist;} // minHeap
   };
   
   std::priority_queue<PointEntry> pqPoints;
   std::priority_queue<BoxEntry> pqBoxes;

   inline void putPointOnQueue(const PointEntry &indexOfPoint);
   void addNextBoxes(const Vector3i &cell);

 public:
	GridBasedPointSetKNNQuery(const PointSet *ps, const Vector3f &targetPoint, float32 cellSize);
   void setTargetPoint(const Vector3f &targetPoint);
   void getNextNearestPointIndex(card32 &nextIndex, bool &found);
};    */

}

#endif