#ifndef HDRLightprobeSamplingH
#define HDRLightprobeSamplingH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "RGBAImage.h"
#include "BoundingBox.h"
//---------------------------------------------------------------------------


namespace X4
{

class HDRSummedAreaTable;
class HDRLightprobeSamplingEditorWidget;

/**
	samples a high dynamic range lightprobe image using debevecs median cut
	resulting samples are distributed over a unit sphere/halfsphere (z is "up")
	if no hdr image is specified, then a white image is used
*/
class X3FOUNDATION_PRIVATE_API HDRLightprobeSampling : public Persistent
{
	X4_CLASS( HDRLightprobeSampling )

private:

	RGBAFloat32Image	*	m_LightImage;
	bool						m_HalfSphere;

	card32					m_NumSamples;
	float32					m_InterpToGray;


	// non-persistent
	HDRSummedAreaTable	* m_SummedAreaTable;
	card32						m_Version;


public:
	HDRLightprobeSampling(void);
	~HDRLightprobeSampling(void);

	struct TLightSample
	{
		Vector3f		m_Position;
		Vector3f			m_Color;
		Vector2f			m_PixelPosition;
		BoundingBox2f	m_Bounds;
	};

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);
	virtual void read(InputObjectStream *s);

	void setup() {setupSamples(m_NumSamples);}
	void setupSamples( card32 numSamples );
	TLightSample getSample( card32 i ) {return m_Samples[i];}
	mpcard getNumSamples() {return m_Samples.size();}

	void setNumSamples(card32 v) {m_NumSamples=v;}

	// becomes owner of image!
	void setImage( RGBAFloat32Image * image );
	RGBAFloat32Image * getImage() {return m_LightImage;}
	void resetWhiteImage();
	bool loadFromPFM(const char * str);

	card32 getVersion() const {return m_Version;}
	void setVersion(card32 v) {m_Version=v;}

	bool getSampleHalfSphereFlag() const {return m_HalfSphere;}
	void setSampleHalfSphereFlag(bool v) {m_HalfSphere=v;}

	void setInterpToGray(float v) {m_InterpToGray = v;}
	float getInterpToGray() const {return m_InterpToGray;}



private:
	std::vector<TLightSample>	m_Samples;

	friend HDRLightprobeSamplingEditorWidget;


};

class HDRSummedAreaTable
{
public:
	HDRSummedAreaTable * m_ChildA, *m_ChildB;
	float		m_SummedEnergy;
	int		m_X1, m_Y1, m_X2, m_Y2;


	HDRSummedAreaTable(RGBAFloat32Image * image, int x1, int y1, int x2, int y2);
	~HDRSummedAreaTable();

	float getEnergy( int x1, int y1, int x2, int y2 );

};

class MedianCutLightNode
{
public:

	MedianCutLightNode();
	~MedianCutLightNode();

	bool operator < (const MedianCutLightNode& n ) const {return m_SummedEnergy < n.m_SummedEnergy;}

	Vector2f			m_SamplePoint;
	Vector3f			m_Color;
	float32			m_SummedEnergy;
	int				m_X1, m_Y1, m_X2, m_Y2;
};

}

#endif