#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "HDRLightprobeSamplingEditorWidget.h"
//---------------------------------------------------------------------------
#include "HDRLightprobeSampling.h"
#include "StringHelper.h"
#include <fstream>
#include <qtGUI/qfiledialog>
#include <qtGUI/qpainter>
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_CLASS_EDITOR( HDRLightprobeSampling )

HDRLightprobeSamplingEditorWidget::HDRLightprobeSamplingEditorWidget(QWidget * parent, Qt::WindowFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));
	
	QVBoxLayout * l = new QVBoxLayout(ui.imageWidget);
	l->setMargin(0);
	l->setSpacing(0);
	l->setAlignment(Qt::AlignTop);

	m_ImageLabel = new MouseEmittingLabel(ui.imageWidget);
	l->addWidget(m_ImageLabel);
	l->addStretch(0);

	QObject::connect(m_ImageLabel,SIGNAL(sigMouseMove(QMouseEvent*)),this,SLOT(sltMouseMove(QMouseEvent*)));
	QObject::connect(m_ImageLabel,SIGNAL(sigMouseRelease(QMouseEvent*)),this,SLOT(sltMouseRelease(QMouseEvent*)));
	m_PaintColor = QColor(255,255,255);
	m_Changed = false;
}

HDRLightprobeSamplingEditorWidget::~HDRLightprobeSamplingEditorWidget(void)
{
}

void HDRLightprobeSamplingEditorWidget::setup( HDRLightprobeSampling * obj )
{
	m_Object = obj;
}

void HDRLightprobeSamplingEditorWidget::updateEditor()
{
	RGBAFloat32Image * image = m_Object->getImage();
   QImage img(QSize(image->getSizeX(),image->getSizeY()),QImage::Format_RGB32);
	float exposure = ui.dExposure->value();
   
   for( card32 y=0;y<image->getSizeY();y++ )
   {
      for( card32 x=0;x<image->getSizeX();x++ )
      {
			card32 col = 0;
			Vector4f fcol = image->getPixel(x,y);
			for( card32 i=0;i<3;i++ )
			{
				int icol = min(255.0f, max( 0.0f, fcol[2-i] * 255.0f * exposure ));
				col |= (card32)icol << (i*8);
			}
			img.setPixel(x,y, col );
      }
   }

	QPixmap pixmap = QPixmap::fromImage(img);
	QPainter painter(&pixmap);

	for( card32 i=0;i<m_Object->getNumSamples();i++ )
	{
		HDRLightprobeSampling::TLightSample sample = m_Object->getSample(i);
		
		painter.setPen( QColor(255,0,0));
		painter.drawEllipse(sample.m_PixelPosition[0]-2,sample.m_PixelPosition[1]-2,5,5);
		painter.setPen( QColor(0,255,0));
		Vector2f size = sample.m_Bounds.upperCorner - sample.m_Bounds.lowerCorner;
		painter.drawRect( sample.m_Bounds.lowerCorner[0],sample.m_Bounds.lowerCorner[1],
			size[0], size[1] );

	}
	painter.end();

   m_ImageLabel->setPixmap(pixmap);
	ui.bSampleFullSphere->setChecked( !m_Object->getSampleHalfSphereFlag() );
	ui.iNumSamples->setValue(m_Object->getNumSamples() );
	ui.dInterpGray->setValue(m_Object->getInterpToGray());

	ui.pbPaintColor->setPalette(QPalette(m_PaintColor));
	ui.pbPaintColor->setAutoFillBackground(true);

}

#include "HDRLightprobeSamplingEditorWidget_moc.h"

// RGBAFloat32Image * importHDRImageFromPFM( const char * str )
// {
// 	fstream input( str );
// 	if( !input.is_open() )
// 		return NULL;
// 
// 	std::string line;
// 	getline( input, line );
// 	int width,height;
// 	float aspect;
// 	input >> width;
// 	input >> height;
// 	input >> aspect;
// 	input.close();
// 
// 	std::ifstream bin_input( str, ios::binary );
// 	bin_input.seekg( 0, ios::end );
// 	int fSize = bin_input.tellg();
// 	int newPos = fSize - width*height*4*3;
// 	bin_input.seekg( newPos, ios::beg );
// 
// 	RGBAFloat32Image * ret = new RGBAFloat32Image;
// 	ret->resize(width,height);
// 	std::vector<char> data(width * height *4 * 3);
// 	char * ptr = &data[0];
// 	bin_input.read( (char*)ptr, width * height *4 * 3);
// 	bin_input.close();
// 
// 	// big/little endian?
// 	if( aspect > 0 )
// 	{
// 		for( int i=0;i<width * height;i++ )
// 		{
// 			char tmp = ptr[0];
// 			ptr[0] = ptr[3];
// 			ptr[3] = tmp;
// 			tmp = ptr[1];
// 			ptr[1] = ptr[2];
// 			ptr[2] = tmp;
// 
// 			ptr += 4;
// 		}
// 	}
// 
// 	Vector4f * retPtr = ret->getPixels();
// 	Vector3f * srcPtr = (Vector3f*)&data[0];
// 	for( card32 x=0;x<ret->getSizeX();x++ )
// 		for( card32 y=0;y<ret->getSizeY();y++ )
// 		{
// 			Vector3f col = srcPtr[(ret->getSizeY()-1-y)*ret->getSizeX() + x ];
// 			retPtr[y*ret->getSizeX()+x] = makeVector4f( col[0],col[1],col[2],1.0f);
// 		}
// 
// 	return ret;
// }

void HDRLightprobeSamplingEditorWidget::on_pbLoadImage_clicked() 
{
	QString filename = QFileDialog::getOpenFileName(this,"load hdr image","","*.pfm" );
	if( filename.isEmpty() )
		return;

	//RGBAFloat32Image * image = importHDRImageFromPFM(filename.toLatin1().constData());
	//m_Object->setImage(image);
	m_Object->loadFromPFM(filename.toLatin1().constData());
	updateEditor();

}

void HDRLightprobeSamplingEditorWidget::on_dExposure_valueChanged(double d) 
{
	updateEditor();
}

void HDRLightprobeSamplingEditorWidget::on_iNumSamples_valueChanged(int i) 
{
	m_Object->setNumSamples(i);
}

void HDRLightprobeSamplingEditorWidget::on_pbSample_clicked() 
{
   if (ui.iNumSamples->value() <=0 ) return;
	m_Object->setupSamples( ui.iNumSamples->value() );
	updateEditor();
}

void HDRLightprobeSamplingEditorWidget::on_bSampleFullSphere_clicked() 
{
	m_Object->setSampleHalfSphereFlag( !ui.bSampleFullSphere->isChecked() );
	emit sigObjectChanged();
}

void X4::HDRLightprobeSamplingEditorWidget::on_pbResetLightprobe_clicked()
{
	m_Object->resetWhiteImage();
	updateEditor();
	emit sigObjectChanged();
}

void X4::HDRLightprobeSamplingEditorWidget::on_dInterpGray_valueChanged( double d)
{
	m_Object->setInterpToGray(d);
	emit sigObjectChanged();
}

void X4::HDRLightprobeSamplingEditorWidget::sltMouseMove( QMouseEvent *event )
{
	QPoint pos = event->pos();
	const int brushSize = 5;
	const float inv_sigma_2 = 1.0f / pow(brushSize*0.5f,2.0f);
	if( event->buttons() == Qt::LeftButton )
	{
		const int sizeX = (int)m_Object->m_LightImage->getSizeX();
		const int sizeY = (int)m_Object->m_LightImage->getSizeY();
		RGBAFloat32Image * img = m_Object->m_LightImage;
		if( pos.x() >= -brushSize && pos.y() >= -brushSize && pos.x() < sizeX+brushSize && pos.y() < sizeY+brushSize)
		{
			Vector4f color = makeVector4f(m_PaintColor.red(),m_PaintColor.green(),m_PaintColor.blue(),0)*ui.dPaintIntensity->value()/255.0f;
			for( int x=-brushSize;x<=brushSize;x++)
			{
				int imgX = x + pos.x();
				if( imgX < 0 )					
					continue;
				if( imgX > sizeX)
					break;
				for( int y=-brushSize;y<=brushSize;y++)
				{
					int imgY = y + pos.y();
					if( imgY >= 0 && imgY < sizeY)
					{
						float dist_2 = x*x+y*y;
						float weight = exp(-dist_2 * inv_sigma_2);
						img->setPixel(imgX,imgY,blend(color,img->getPixel(imgX,imgY),weight));
					}
					
				}
			}
			m_Changed = true;
			updateEditor();
			
		}
	}
}

void X4::HDRLightprobeSamplingEditorWidget::on_pbPaintColor_clicked()
{
	QColor newCol = QColorDialog::getColor(m_PaintColor,this); //,"choose paint color"); // Miguel: title not supported in Qt 4.4
	if( newCol.isValid())
	{
		m_PaintColor = newCol;
		ui.pbPaintColor->setPalette(QPalette(m_PaintColor));
		ui.pbPaintColor->setAutoFillBackground(true);
	}
}

void X4::HDRLightprobeSamplingEditorWidget::sltMouseRelease( QMouseEvent *event )
{
	if( m_Changed )
	{
		m_Changed = false;
		emit sigObjectChanged(); 
	}
}
X4::MouseEmittingLabel::MouseEmittingLabel( QWidget * parent /*= NULL*/ )
:QLabel(parent)
{
	setMouseTracking(true);
}

void X4::MouseEmittingLabel::mouseMoveEvent( QMouseEvent *event )
{
	emit sigMouseMove(event);
}

void X4::MouseEmittingLabel::mouseReleaseEvent( QMouseEvent *event )
{
	emit sigMouseRelease(event);
}