#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "HighQualityRenderer.h"
//---------------------------------------------------------------------------
#include "GLShaderMaterial.h"
#include "Shader.h"
#include "Configuration.h"
#include "DebugOutput.h"
#include "MaterialParameter.h"
#include "MPContainer.h"
#include "VertexDescriptor.h"
#include "RGBAImage.h"
#include "ObjectClassProperty.h"
#include "NumericalClassProperty.h"
#include "GLHelper.h"
#include "GLExtensions.h"
#include "CubeMap.h"
#include "FixedArrayClassProperty.h"
#include "MathTools.h"
#include "SeparatorClassProperty.h"
#include "BooleanClassProperty.h"
#include "ClassMethods.h"
#include "HQRLight.h"
#include "CopyObjectProperties.h"
#include "Random.h"
#include "DebugOutput.h"
#include "StandardSceneGraphIterator.h"
#include "SceneGraphNode.h"
#include "SGObjectNode.h"
#include "SceneObject.h"
#include "SGCameraNode.h"
#include "SGIndexedAndByNameSceneGraphIteratorAdaptor.h"
#include "CameraController.h"
#include "HDRLightprobeSampling.h"
#include "PointCloudTools.h"
#include "RenderingTool.h"
#include "GLImage.h"
#include "GLImageFilter.h"
#include "GLImageCombiner.h"
#include "FilterMask.h"
#include "ObjectListProperty.h"
#include "GLRenderTextureCache.h"
#include "GLRenderTextureFBO.h"
#include "ImageDisplayer.h"
#include "ProgressWindow.h"
#include "RGBATextureAttachment.h"
#include "GLTexture.h"
#include "FileDialogs.h"
#include "BinaryObjectStreams.h"
#include "DebugRenderer.h"
#include "ECPhotonTracer.h"
#include "PCCResampler.h"
#include "VertexArray.h"
//#include "BRDFAttachment.h"
#include "SGListNode.h"
#include "SceneGraphTools.h"
#include "ApplicationDirectories.h"
#include "GLDebuggingTools.h"

//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( HighQualityRenderer ,1)
{
	BEGIN_CLASS_INIT( HighQualityRenderer );

	ADD_BOOLEAN_PROP( m_RenderScene, 0 );
	ADD_BOOLEAN_PROP( m_RenderLightSamples, 0 );
	ADD_BOOLEAN_PROP( m_RenderSpotLightSamples, 0 );
	//ADD_BOOLEAN_PROP( m_UseScreenSpaceAmbientOcclusion, 1);
	ADD_OBJECT_PROP( m_Settings, 0, HQRSettings::getClass(), true );

	//ADD_NOARGS_METHOD( updateLightSamples );
	ADD_NOARGS_METHOD( precomputeLight );
	ADD_NOARGS_METHOD( loadSettings );
	ADD_NOARGS_METHOD( saveSettings );
	//ADD_NOARGS_METHOD( debug_SetLightSampleCam );
}

IMPLEMENT_X4_CLASS( HQRSettings , 12)
{
	BEGIN_CLASS_INIT( HQRSettings );

	ADD_SEPARATOR( "Shadowmapping" );
	ADD_OBJECT_PROP( m_Mat_GenerateShadowmap, 0, GLShaderMaterial::getClass(), true );
//	SET_USER_EDIT(false);
	ADD_OBJECT_PROP( m_Mat_RenderWithShadowmap, 0, GLShaderMaterial::getClass(), true );
	ADD_OBJECT_PROP( m_Mat_PrecomputeLight, 0, GLShaderMaterial::getClass(), true );
//	SET_USER_EDIT(false);
	//ADD_OBJECT_PROP( m_Mat_ScreenSpaceAmbientOcclusion, 2, GLShaderMaterial::getClass(), true );
	ADD_FLOAT32_PROP( m_ShadowMapOffset, 0 );
	ADD_CARD32_PROP(m_ShadowMapSize,4);
	ADD_SEPARATOR( "Ambient" );
	ADD_OBJECT_PROP( m_AmbientLightSampling,0,HDRLightprobeSampling::getClass(),true);
	ADD_FLOAT32_PROP( m_LightStrength, 0 );
	ADD_FLOAT32_PROP( m_RotateAmbientLight, 6);
	ADD_FLOAT32_PROP( m_RotateAmbientLightPhi, 7);
	ADD_FLOAT32_PROP( m_RotateAmbientLightPsi, 7);
	ADD_BOOLEAN_PROP( m_ShowAmbientLightSamples, 6);
	ADD_SEPARATOR( "Specular" );
	ADD_OBJECT_PROP( m_Mat_SpecularEnviroment, 0,  GLShaderMaterial::getClass(), true );
	ADD_SEPARATOR( "Options" );
	ADD_BOOLEAN_PROP( m_UseSpecularEnviromentShader, 0 );
	SET_USER_EDIT(false);
	ADD_VECTOR4F_PROP( m_BackgroundColor, 0 );
	ADD_OBJECT_PROP(m_BackgroundImage,0,RGBATextureAttachment::getClass(),true);
	ADD_BOOLEAN_PROP( m_UpVectorIsZ, 0 );	// obsolete
	SET_USER_EDIT(false);
	ADD_BOOLEAN_PROP( m_Bloom,1);
	ADD_OBJECT_LIST_PROP( HQRLight, 0, HQRLight::getClass() );
	ADD_BOOLEAN_PROP( m_AllowDebugRenderer,3);
	ADD_BOOLEAN_PROP( m_Use32Bit, 5);
	ADD_BOOLEAN_PROP( m_InstanciateDebugRendererCurrentFrame, 10);
	ADD_SEPARATOR("additional render pass (own material)")
	ADD_BOOLEAN_PROP( finalPassWithObjMaterial, 12 )
	ADD_STRING_PROP( finalPassPathWithWildcards, 12 )


// 	ADD_SEPARATOR("BRDF Rendering");
// 	ADD_BOOLEAN_PROP(m_brdfEnable, 9);
// 	ADD_CARD16_PROP(m_brdfNumLightSamples, 9);
// 	ADD_CARD16_PROP(m_brdfMaxNumBRDFs, 9);
// 	ADD_OBJECT_PROP(m_Mat_brdfGenerateLightDirectionmap, 9, GLShaderMaterial::getClass(), true);
}

HighQualityRenderer::HighQualityRenderer(void)
{
	m_doImageOperations = true;

	//m_RenderBRDF = false;

	m_RenderScene = false;
	m_RenderSpotLightSamples = false;
	m_RenderLightSamples = false;
	//m_UseScreenSpaceAmbientOcclusion = false;

	m_Settings = new HQRSettings;

	m_LightSamplesVersion = 0xffff;
	m_MainRenderTarget = NULL;

	m_MainRenderTargetName = "RT_Combine";
}

HighQualityRenderer::~HighQualityRenderer(void)
{
	delete m_Settings;
}

void HighQualityRenderer::assign(const Object* obj, X4CopyContext *context)
{
//	MultiPassRenderer::assign(obj,context);
	const HighQualityRenderer * o = dynamic_cast<const HighQualityRenderer*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties( obj, this );
}



void HighQualityRenderer::renderMultiPass()
{
	//float32 minPointSpacing;
	//hi->getNodeInfo(HierarchicalIterator::niPointSpacing, NULL, &minPointSpacing);
	//m_Mat_RenderWithShadowmap->setParameter1f(getGLContext(), "minPointSpacing", minPointSpacing);
	float32 verticalFieldOfView = getScene()->getRootState()->staticState->viewFrustum.getVerticalFieldOfView();
	float32 tanOfHalfVerticalViewAngle = tan(verticalFieldOfView / 180.0f * M_PI / 2.0f);
	m_Settings->m_Mat_RenderWithShadowmap->setParameter1f(getGLContext(), "vertViewAngle", verticalFieldOfView);
	m_Settings->m_Mat_RenderWithShadowmap->setParameter1f(getGLContext(), "tanOfHalfVerticalViewAngle", tanOfHalfVerticalViewAngle);
	m_Settings->m_Mat_RenderWithShadowmap->setParameter1f(getGLContext(), "vertResolution", getScreenHeight());

	if( m_Settings->m_InstanciateDebugRendererCurrentFrame)
	{
		deleteSceneGraphNode(getScene(),"root/hqr_inst");
		SGListNode * node = new SGListNode;
		node->setName("hqr_inst");
		addSceneGraphNode(getScene(),"root/",node);
		debugRenderer->instantiateAllMeshesInFrame(getScene(), getScene()->getRootState()->dynamicState->time,3,node);
	}


	getScene()->getRootState()->dynamicState->setStateVariable("SC_globalColor",makeVector4f(1,1,1,1));
	if( !m_RenderScene || (!m_RenderLightSamples && !m_RenderSpotLightSamples))
	{
		resetMaterial();
		renderScene();
		if( m_Settings->m_InstanciateDebugRendererCurrentFrame)
		{
			deleteSceneGraphNode(getScene(),"root/hqr_inst");
		}
		return;
	}


//	if( !m_RenderScene )
	{
		resetMaterial();
		renderScene();
		//return;
	}


	if( m_LightSamplesVersion != m_Settings->m_AmbientLightSampling->getVersion() )
	{
		createLightSamples();
		m_LightSamplesVersion = m_Settings->m_AmbientLightSampling->getVersion();
	}
	
	// compute total number of lightsamples
	card32 numSamples = m_RenderLightSamples?(card32)m_LightSamples.size():0;
	if(m_RenderSpotLightSamples)
	{
		for( card32 i=0;i<m_Settings->m_SpotLights.size();i++ )
			numSamples += m_Settings->m_SpotLights[i]->getNumSamples();
	}
	m_Temp_ConstW = 1.0f / (float)numSamples;
	//debugOutput << "num samples: " << numSamples << "\n";


	if( m_RenderLightSamples )
	{
		for( card32 i=0;i<m_LightSamples.size();i++ )
		{
			float32 verticalFieldOfView = m_LightSamples[i].vf.getVerticalFieldOfView();
			float32 tanOfHalfVerticalViewAngle = tan(verticalFieldOfView / 180.0f * M_PI / 2.0f);
			m_Settings->m_Mat_GenerateShadowmap->setParameter1f(getGLContext(), "vertViewAngle", verticalFieldOfView);
			m_Settings->m_Mat_GenerateShadowmap->setParameter1f(getGLContext(), "tanOfHalfVerticalViewAngle", tanOfHalfVerticalViewAngle);
			m_Settings->m_Mat_GenerateShadowmap->setParameter1f(getGLContext(), "vertResolution", m_Settings->getShadowMapSize());

			renderLightSample( m_LightSamples[i], i == 0 );
		}
	}

	
	BoundingBox3f bb = getScenePCBBox( getScene() );
	float sceneSize = norm( bb.upperCorner - bb.lowerCorner);

	if(m_RenderSpotLightSamples)
	{
		for( card32 i=0;i<m_Settings->m_SpotLights.size();i++ )
		{
			StandardSceneGraphIterator sceneIt;
			sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, SceneGraphNode::FLAG_VISIBLE, 0);
			SGIndexedAndByNameSceneGraphIteratorAdaptor nameIter;
			nameIter.clearAndSetup( &sceneIt );
			if( !nameIter.goToName( m_Settings->m_SpotLights[i]->getCameraName() ) )
				continue;

			SGCameraNode * node = dynamic_cast<SGCameraNode*>(nameIter.getNode());
			if( node != NULL )
			{
				TLightSource ls;
				ls.color = m_Settings->m_SpotLights[i]->getColor() * m_Settings->m_SpotLights[i]->getStrength() / m_Settings->m_SpotLights[i]->getNumSamples();
				ls.spot = true;
				node->getCameraController()->setTime( getScene()->getRootState()->dynamicState->time );
				ls.vf.setAll( sceneSize * 0.01f, sceneSize * 2.0f, 45.0f, 1.0f );//*node->getViewFrustum();
				std::vector<Vector3f> dist = getSpotLightDistribution(m_Settings->m_SpotLights[i]->getNumSamples());
				for( card32 j=0;j<m_Settings->m_SpotLights[i]->getNumSamples();j++ )
				{
					ls.cam = *node->getCamera();
					ls.cam.setPosition( ls.cam.getPosition() + dist[j] * m_Settings->m_SpotLights[i]->getSize() );
					renderLightSample( ls, !m_RenderLightSamples && i==0 && j==0 );
				}
			}
		}
	}

	performPostprocessing();

	//if( m_UseScreenSpaceAmbientOcclusion )
	//{
	//	renderScreenSpaceAmbientOcclusion();
	//}
	
	if( !m_RenderLightSamples && !m_RenderSpotLightSamples )
	{
		X4_GL_FB_DBG
		setRenderTarget( m_MainRenderTargetName, RT_Depth | (m_Settings->m_Use32Bit?RT_RGBA32:RT_RGBA16), getScreenWidth(), getScreenHeight() );
		X4_GL_FB_DBG
		clearScreen();
	}

	if( m_Settings->m_Mat_SpecularEnviroment != NULL && m_Settings->m_UseSpecularEnviromentShader)
	{
		setMaterial( m_Settings->m_Mat_SpecularEnviroment );
		m_Settings->m_Mat_SpecularEnviroment->setBlending(true);
		m_Settings->m_Mat_SpecularEnviroment->setDepthTest(true);
		m_Settings->m_Mat_SpecularEnviroment->setDepthWrite(false);
		glDepthFunc( GL_LEQUAL );
		renderScene();
	}

	if (m_Settings->finalPassWithObjMaterial) {
		renderNodesDefaultMat();
	}

//	if( !m_MainRenderTarget ) {
	X4_GL_FB_DBG
	setDefaultRenderTarget();
	X4_GL_FB_DBG
//	} else
//		m_MainRenderTarget->activate();

	static GLRenderTextureCache * cache = NULL;
	static GLImageCombiner * toneMapping = NULL;
	static GLImageCombiner * divideByW = NULL;
	static GLImageFilter * verticalImageFilter = NULL;
	static GLImageFilter * horizontalImageFilter = NULL;
	GLImage * combineRT = NULL;
	combineRT = new GLImage;
	X4_GL_FB_DBG
	combineRT->setup( getRenderTargetFBO( m_MainRenderTargetName ), false );
	X4_GL_FB_DBG
	if( cache == NULL && m_doImageOperations )
	{
		cache = new GLRenderTextureCache( getGLContext() );

		toneMapping = new GLImageCombiner;
		toneMapping->setCombineOp( "O = saturate(I0 - 0.9);" );
		toneMapping->setOutputFormat( GLImageOperator::OF_Float16 );
		toneMapping->setNumInputImages(1);
		toneMapping->setInputImage( 0, combineRT );

		divideByW = new GLImageCombiner;
		divideByW->setCombineOp(" O = I0 / I0.w;");
		//divideByW->setCombineOp(" O = I0.w*float4(1,0.5,0.25,0.1);");
		//divideByW->setCombineOp(" O = float4(1,0,0,0.5);");
		divideByW->setOutputFormat( GLImageOperator::OF_Byte );
		divideByW->setNumInputImages(1);
		divideByW->setInputImage( 0, combineRT );
		
		DynamicFilterMask *verticalMask = new DynamicFilterMask, *horizontalMask = new DynamicFilterMask;
		verticalMask->setup( 21 );
		verticalMask->makeVerticalGauss( 10.0f );
		horizontalMask->setup( 21 );
		horizontalMask->makeHorizontalGauss( 10.0f );
		
		verticalImageFilter = new GLImageFilter;
		verticalImageFilter->setNumInputImages(1);
		verticalImageFilter->setInputImage( 0, combineRT );
		verticalImageFilter->setOutputFormat( GLImageOperator::OF_Float16 );
		verticalImageFilter->setNormalizeMask(true);
		verticalImageFilter->setFilterMask( verticalMask );

		horizontalImageFilter = new GLImageFilter;
		horizontalImageFilter->setNumInputImages(1);
		horizontalImageFilter->setInputImage( 0, combineRT );
		horizontalImageFilter->setOutputFormat( GLImageOperator::OF_Byte );
		horizontalImageFilter->setNormalizeMask(true);
		horizontalImageFilter->setFilterMask( horizontalMask );


	}


	GLImage * filteredImage2  = NULL;
	GLImage * toneMappedImage = NULL;
	GLImage * filteredImage1 = NULL;
	GLImage * normalizedImage = NULL;
	if( m_Settings->m_Bloom && m_doImageOperations)
	{
		toneMapping->setInputImage( 0, combineRT );
		toneMappedImage = toneMapping->execute( cache );

		verticalImageFilter->setInputImage( 0, toneMappedImage );
		filteredImage1 = verticalImageFilter->execute( cache );

		horizontalImageFilter->setInputImage( 0, filteredImage1 );
		filteredImage2 = horizontalImageFilter->execute( cache );

	}

	if (m_doImageOperations)
	{
		divideByW->setInputImage(0,combineRT);
		normalizedImage = divideByW->execute(cache);
	}

	if( m_MainRenderTarget ) {
		m_MainRenderTarget->activate();
		X4_GL_FB_DBG
	}
	if( getOffscreenRenderTarget() ) {
		getOffscreenRenderTarget()->activate();
		X4_GL_FB_DBG
	}

	glDepthMask(0);
	if (m_doImageOperations && normalizedImage)
		normalizedImage->drawToScreen();
	else
		combineRT->drawToScreen();
	glDepthMask(1);
	
	if( m_Settings->m_Bloom && m_doImageOperations)
	{
		glEnable( GL_BLEND );
		glBlendFunc( GL_ONE, GL_ONE );
		filteredImage2->drawToScreen();
		glDisable( GL_BLEND );
	}

	if( m_MainRenderTarget ) {
		m_MainRenderTarget->deactivate();
		X4_GL_FB_DBG
	}
	if( getOffscreenRenderTarget() ) {
		getOffscreenRenderTarget()->deactivate();
		X4_GL_FB_DBG
	}

	delete filteredImage2;
	delete normalizedImage;
	delete toneMappedImage;
	delete filteredImage1;
	delete combineRT;

	if( m_Settings->m_InstanciateDebugRendererCurrentFrame)
	{
		deleteSceneGraphNode(getScene(),"root/hqr_inst");
	}

/*	glPointSize(5.0f);
	glBegin( GL_POINTS);
	glColor3f(0,0,1);
	for( unsigned i=0;i<m_LightSamples.size();i++ )
	{
		Vector3f camPos = m_LightSamples[i].cam.getPosition();
		glVertex3fv( camPos.data() );
	}
	glEnd();
*/	
}


void HighQualityRenderer::createLightSamples() 
{
	BoundingBox3f bb = getScenePCBBox( getScene() );
	Vector3f sceneCenter = bb.getCenter();

	float sceneSize = norm( bb.upperCorner - bb.lowerCorner);
	UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(getPointCloud(getScene(),"root/pc1"));
	m_LightSamples.clear();
	debugRenderer->clearDebugData("show ambient samples");
	debugRenderer->beginRenderJob("show ambient samples");

	float angle = m_Settings->m_RotateAmbientLight*M_PI / 180.0f;
//	Matrix3f rot1 = makeRotZ3f(angle);
	float anglePhi = m_Settings->m_RotateAmbientLightPhi*M_PI / 180.0f;
//	rot1 = rot1*makeRotX3f(anglePhi);
//	Vector3f upDirection = rot1*makeVector3f(0,0,1);
	Vector3f upDirection = makeVector3f(sin(anglePhi)*sin(angle),cos(anglePhi)*sin(angle),cos(angle));
	Matrix3f tangentFrame = calcTangentSystem(upDirection);
	float anglePsi = m_Settings->m_RotateAmbientLightPsi*M_PI / 180.0f;
	Matrix3f rot = tangentFrame * makeRotZ3f(anglePsi);


	// only for debug visualization of samples
	float scaleColor = 1.0f/m_Settings->m_AmbientLightSampling->getNumSamples();
	for( card32 i=0;i<m_Settings->m_AmbientLightSampling->getNumSamples();i++ )
	{
		HDRLightprobeSampling::TLightSample sample = m_Settings->m_AmbientLightSampling->getSample(i);
		float cn = norm(sample.m_Color);
		if( cn > scaleColor )
			scaleColor = cn;
	}
	scaleColor = 3.0f / scaleColor;

	if( m_Settings->m_ShowAmbientLightSamples )
	{
		Vector3f upDir = makeVector3f(0,m_Settings->m_UpVectorIsZ?0:sceneSize,!m_Settings->m_UpVectorIsZ?0:sceneSize);
		debugRenderer->addFineArrow(sceneCenter,sceneCenter + rot*upDir,makeVector3f(0,0,1),sceneSize*0.02f,false);
	}


	for( card32 i=0;i<m_Settings->m_AmbientLightSampling->getNumSamples();i++ )
	{
		HDRLightprobeSampling::TLightSample sample = m_Settings->m_AmbientLightSampling->getSample(i);
		Vector3f pos = sample.m_Position * sceneSize;
		


		pos = sceneCenter + rot*pos;

		if( !m_Settings->m_UpVectorIsZ )
		{
			float tmp = pos[1];
			pos[1] = pos[2];
			pos[2] = tmp;
		}

		Camera cam;
		cam.setLookAt( sceneCenter );
		cam.setPosition( pos );
		cam.setUp( makeVector3f(0,0,1 ) );

		ViewFrustum vf;
		vf.setAll( sceneSize * 0.01f, sceneSize * 2.0f, 45.0f, 1.0f );
		
		TLightSource light;

		light.color = sample.m_Color;
		light.cam = cam;
		light.vf = vf;
		light.spot = false;

		m_LightSamples.push_back( light );

		if( m_Settings->m_ShowAmbientLightSamples )
		{
			debugRenderer->addPoint(pos,light.color*scaleColor);
		}
	}
	debugRenderer->endRenderJob();
}

void HighQualityRenderer::updateLightSamples() 
{
	m_Settings->m_AmbientLightSampling->setup();
	createLightSamples();
}

void HighQualityRenderer::renderLightSample( TLightSource ls, bool firstPass) 
{
	glDepthFunc( GL_LESS );
	glDepthMask(1);
	// ---------------------------------------
	// generate shadowmap pass
	setRenderTarget( "RT_ShadowMap", RT_Depth, m_Settings->getShadowMapSize(),m_Settings->getShadowMapSize());
	X4_GL_FB_DBG

	// clear
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// setup
	//setCamera( "root/camera1" );
	setCamera( ls.cam, ls.vf );
	//setOrthoCamera( ls.cam, ls.vf );
	setMaterial( m_Settings->m_Mat_GenerateShadowmap );
	//setMaterialFromScene( 1 );

	// render
	renderScene();
//		debugRenderer->renderJobs(getGLContext(),getScene(),(GLShaderMaterial *)getScene()->getGLMaterial(1));


	getScene()->getRootState()->dynamicState->setStateVariable("SC_globalColor",makeVector4f(1,1,1,1));

	// ---------------------------------------
	// final render pass
	setDefaultRenderTarget();
	X4_GL_FB_DBG
	setRenderTarget( m_MainRenderTargetName, RT_Depth | (m_Settings->m_Use32Bit?RT_RGBA32:RT_RGBA16), getScreenWidth(), getScreenHeight() );
	X4_GL_FB_DBG
	glPushAttrib( GL_ALL_ATTRIB_BITS );
	// clear
	if( firstPass )
	{
		clearScreen();
		m_Settings->m_Mat_RenderWithShadowmap->setBlending(false);
		m_Settings->m_Mat_RenderWithShadowmap->setDepthTest(true);
		m_Settings->m_Mat_RenderWithShadowmap->setDepthWrite(true);
		m_Settings->m_Mat_RenderWithShadowmap->setDestBlendFunc(0);
		m_Settings->m_Mat_RenderWithShadowmap->setSourceBlendFunc(0);
		glDepthFunc( GL_LEQUAL );
	}
	else
	{
		m_Settings->m_Mat_RenderWithShadowmap->setBlending(true);
		m_Settings->m_Mat_RenderWithShadowmap->setDepthTest(true);
		m_Settings->m_Mat_RenderWithShadowmap->setDepthWrite(false);
		glDepthFunc( GL_LEQUAL );
	}


	// setup
	setDefaultCamera();

	setMaterial( m_Settings->m_Mat_RenderWithShadowmap );

	bindTarget( "RT_ShadowMap", "HQ_ShadowMap" );
	//setOrthoCameraAsParameter( ls.cam, ls.vf, "HQ_ShadowCameraMatrix" );
	setCameraAsParameter( ls.cam, ls.vf, "HQ_ShadowCameraMatrix" );
	setMaterialParameter1f( "offset", m_Settings->m_ShadowMapOffset );
	Vector3f lightCol = ls.color * (ls.spot ? 1.0f:m_Settings->m_LightStrength);
	setMaterialParameter3f( "lightColor", lightCol[0] ,lightCol[1],lightCol[2] );
	setMaterialParameter1f( "const_w", m_Temp_ConstW);
	setMaterialParameter1f( "shadowmapSize", m_Settings->getShadowMapSize());

	// render
	renderScene();

		

	glPopAttrib();
}


void HighQualityRenderer::renderNodesDefaultMat() 
{
	// ---------------------------------------
	// final render pass
	setDefaultRenderTarget();
	X4_GL_FB_DBG
	setRenderTarget( m_MainRenderTargetName, RT_Depth | (m_Settings->m_Use32Bit?RT_RGBA32:RT_RGBA16), getScreenWidth(), getScreenHeight() );
	X4_GL_FB_DBG
	glPushAttrib( GL_ALL_ATTRIB_BITS );
	glDepthFunc( GL_LEQUAL );
	setDefaultCamera();
	//setMaterialParameter1f( "const_w", m_Temp_ConstW);

	renderScene(true, true, m_Settings->finalPassPathWithWildcards);

	glPopAttrib();
}



std::vector<Vector3f> HighQualityRenderer::getSpotLightDistribution( card32 numSamples ) 
{
	if( m_SpotLightDistribution.find( numSamples ) != m_SpotLightDistribution.end() )
	{
		return m_SpotLightDistribution[numSamples];
	}
	else
	{
		std::vector<Vector3f> ret;
		for( card32 i=0;i<numSamples;i++ )
		{
			ret.push_back( makeVector3f( rnd01(), rnd01(), rnd01() ) * 2.0f - makeVector3f(1,1,1)  );
		}
		m_SpotLightDistribution[numSamples] = ret;
		return ret;
	}
}

void HighQualityRenderer::connect(Scene* scene, GLContext* ctxt) 
{
	MultiPassRenderer::connect(scene,ctxt);
	createLightSamples();
}

// void X4::HighQualityRenderer::prepareBRDF()
// {
// 	debugOutput << "Start BRDF analysis\n";
// 	std::vector<PointCloud*> clouds;
// 	const std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
// 	m_Temp_NumTotalPoints = 0;
// 	for( card32 i=0;i<selection.size();i++ )
// 	{
// 		PointCloud * pc = getPointCloud( getScene(), selection[i] );
// 		if( !pc || !pc->providesAttribute("brdf") || !dynamic_cast<BRDFContainer*>(pc->getAttachments()->getData(BRDFContainer::getDefaultName())))
// 			continue;
// 
// 		m_Temp_NumTotalPoints += pc->getNumPoints();
// 
// 		clouds.push_back(pc);
// 	}
// 
// 	debugOutput << "I've found " << clouds.size() << " point clouds from " << selection.size() <<" selected which provides \"brdf\" attribute and has BRDFContainer attached.\n";
// 
// 	m_Temp_NumPointsProcessed = 0;
// 	for( unsigned i=0;i<clouds.size();i++ )
// 	{	
// 		progressWindow->pushStep(true,"analyse");
// 		progressWindow->pushStep(true,"project");
// 	}
// 
// 	for( unsigned i=0;i<clouds.size();i++ )
// 	{
// 		PointCloud * pc = clouds[i];
// 		SceneObjectIterator * iter = pc->createIterator(SOIT::CAP_BASIC_PC 
// 			| SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_RANDOM_ACCESS_PC);
// 		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
// 		RandomAccessPointCloudIterator * rIt = dynamic_cast<RandomAccessPointCloudIterator*>(iter);
// 		ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
// 
// 		BRDFContainer* container = dynamic_cast<BRDFContainer*>(pc->getAttachments()->getData(BRDFContainer::getDefaultName()));
// 
// 		AAT brdfAAT = pc->getAAT("brdf");
// 
// 		std::vector<BRDFAttachment*> brdfs(min<size_t>(m_Settings->m_brdfMaxNumBRDFsToConsiderForAnalysis, pc->getNumPoints()), NULL);
// 		debugOutput << "Sample randomly BRDFs from " << brdfs.size() << " points\n"; debugOutput.flush();
// 		for (unsigned j=0; j < brdfs.size(); j++)
// 		{
// 			// TODO: do something more intelligent rather than simple random sampling
// 			rIt->goToIndex(random64bit(pc->getNumPoints()));
// 			brdfs[j] = dynamic_cast<BRDFAttachment*>(container->getBRDF(bIt->get1i(brdfAAT)));
// 		}
// 
// 		debugOutput << "Perform BRDF - PCA analysis, so compute at most " << m_Settings->m_brdfMaxNumBRDFs << " basis vectors\n"; debugOutput.flush();
// 		std::vector<BRDFAttachment*> pcaBRDF;
// 		if (BRDFAttachment::pcaAnalysis(brdfs, pcaBRDF, m_Settings->m_brdfMaxNumBRDFs))
// 		{
// 			debugOutput << "Project point's BRDFs to the PCA space and store this information. We have now " << pcaBRDF.size() << " main vectors\n"; 
// 			debugOutput.flush();
// 
// 			// attach basis BRDFs to the point cloud.
// 			BRDFContainer* pcaContainer = new BRDFContainer;
// 			pcaContainer->setup(BRDFContainer::getDefaultName(), AttachedData::ADF_PERSISTENT);
// 
// 			// check BRDF attributes of the point cloud
// 			std::vector<AAT> pcaAAT(pcaBRDF.size());
// 			for (unsigned j=0; j < pcaBRDF.size(); j++)
// 			{
// 				checkAttribute(pc, std::string("brdf_") + intToStr(j), 1, VAD::DATA_FORMAT_FLOAT32);
// 				pcaAAT[j] = pc->getAAT(std::string("brdf_") + intToStr(j));
// 				pcaContainer->addBRDF(pcaBRDF[j]);
// 			}
// 
// 			// now we project every point's BRDF to the PCA basis, and so compute PCA parametrization
// 			bIt->reset();
// 			//for (size_t j=0; j < pc->getNumPoints(); j++)
// 			unsigned j=0;
// 			while(!bIt->atEnd())
// 			{
// 				//rIt->goToIndex(j);
// 				unsigned bid = bIt->get1i(brdfAAT);
// 				BRDFAttachment* brdf = dynamic_cast<BRDFAttachment*>(container->getBRDF(bid));
// 				if (!brdf->isProjectedToPCA())
// 					brdf->reduceToPCAbasis(pcaBRDF);
// 
// 				// store pca parameters
// 				const std::vector<float>& pcaSpace = brdf->getPCAParameters();
// 				for (unsigned k=0; k < pcaBRDF.size(); k++)
// 					mIt->set1f(pcaAAT[k], pcaSpace[k]);
// 
// 				progressWindow->progressf(float(j) / float(pc->getNumPoints()));
// 				bIt->next();
// 			}
// 			delete container;
// 			pc->getAttachments()->attachData(pcaContainer);
// 		}
// 		progressWindow->popStep();
// 
// 		delete iter;
// 
// 		progressWindow->progressf(float(i) / float(clouds.size()));
// 	}
// 	progressWindow->popStep();
// 
// }

void X4::HighQualityRenderer::precomputeLight()
{
	updateLightSamples();

	std::vector<PointCloud*> clouds;
	const std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	m_Temp_NumTotalPoints = 0;
	for( card32 i=0;i<selection.size();i++ )
	{
		PointCloud * pc = getPointCloud( getScene(), selection[i] );
		if( !pc )
			continue;

		m_Temp_NumTotalPoints += pc->getNumPoints();
	
		clouds.push_back(pc);
	}

	card32 maxTextureSize = MAX_RENDER_TEXTURE_SIZE;
	card32 maxNumPointsPerBlock = maxTextureSize*maxTextureSize;
	progressWindow->pushStep(true,"precompute");
	m_Temp_NumPointsProcessed = 0;
	for( unsigned i=0;i<clouds.size();i++ )
	{
		PointCloud * pc = clouds[i];
		SceneObjectIterator * iter = pc->createIterator(SOIT::CAP_BASIC_PC 
			| SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_RANDOM_ACCESS_PC);
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
		RandomAccessPointCloudIterator * rIt = dynamic_cast<RandomAccessPointCloudIterator*>(iter);

// 		// if brdf rendering is activated, then we store light input directions for lights which are visible 
// 		if (m_Settings->m_brdfEnable)
// 		{
// 			for (unsigned j=0; j < m_Settings->m_brdfNumLightSamples; j++)
// 			{
// 				checkAttribute(pc, std::string("brdf_") + intToStr(j), 4, VAD::DATA_FORMAT_FLOAT32);
// 			}
// 		}

		card32 numBlocks = (pc->getNumPoints()+maxNumPointsPerBlock-1 )/ maxNumPointsPerBlock;
		for( unsigned b=0;b<numBlocks;b++ )
		{
			// create a copy of the current block
			UnstructuredInCorePointCloud tmpUPC;
			VertexDescriptor * vd = createDefaultVertexDescriptor(true,true,true,true);
			vd->pushAttrib(VAD("screen_pos",2,VAD::DATA_FORMAT_FLOAT32));
			mpcard startPointIndex = b*maxNumPointsPerBlock;
			mpcard lastPointIndex = min( pc->getNumPoints(), (card64)(b+1) * maxNumPointsPerBlock);
			tmpUPC.clearAndSetup(vd,lastPointIndex - startPointIndex);
			AAT srcPosAAT = pc->getAAT("position");
			AAT srcColorAAT = pc->getAAT("color");
			AAT srcNormalAAT = pc->getAAT("normal");
			AAT destPosAAT = tmpUPC.getAAT("position");
			AAT destColorAAT = tmpUPC.getAAT("color");
			AAT destNormalAAT = tmpUPC.getAAT("normal");
			AAT destScreenPosAAT = tmpUPC.getAAT("screen_pos");


			for(unsigned j=0;j<tmpUPC.getNumPoints();j++ )
			{
				rIt->goToIndex(startPointIndex + j);
				Vector3f pos = bIt->get3f(srcPosAAT);
				Vector3f color = NULL_VECTOR3F;
				Vector3f normal = bIt->get3f(srcNormalAAT);
				Vector2f screenPos = makeVector2f(j % maxTextureSize, j / maxTextureSize + 0.5f) / maxTextureSize * 2.0f
					 + makeVector2f(-1,-1);
				tmpUPC.getPointSet()->set3f(j,destPosAAT,pos);
				tmpUPC.getPointSet()->set3f(j,destColorAAT,color);
				tmpUPC.getPointSet()->set3f(j,destNormalAAT,normal);
				tmpUPC.getPointSet()->set2f(j,destScreenPosAAT,screenPos);
			}

			// compute lighting for block
			//std::vector<RGBAFloat32Image*> brdfInLightSamples;
			RGBAFloat32Image * image = precomputeLight_Block(&tmpUPC,pc->getTransformation());//, m_Settings->m_brdfEnable ? &brdfInLightSamples : NULL);
			//
			//std::vector<AAT> brdfInLightAAT(brdfInLightSamples.size());
			//for (unsigned j=0; j < brdfInLightSamples.size(); j++)
			//	brdfInLightAAT[j] = pc->getAAT(std::string("brdf_") + intToStr(j));

			// store results in original point cloud
			for(unsigned j=0;j<tmpUPC.getNumPoints();j++ )
			{
				rIt->goToIndex(startPointIndex + j);
				Vector3f color = shrink4To3(image->getPixel(j % maxTextureSize, j / maxTextureSize));
				mIt->set3f(srcColorAAT,color);

				// store brdf light direction [0:2] and index[3]
				//for (unsigned k=0; k < brdfInLightAAT.size(); k++)
				//	mIt->set4f(brdfInLightAAT[k], brdfInLightSamples[k]->getPixel(j % maxTextureSize, j / maxTextureSize));
			}
			delete image;
			m_Temp_NumPointsProcessed += tmpUPC.getNumPoints();
		}

		delete iter;
	}

	progressWindow->popSteps();

}

RGBAFloat32Image * X4::HighQualityRenderer::precomputeLight_Block( UnstructuredInCorePointCloud * upc, const Matrix4f& modelMatrix)//, std::vector<RGBAFloat32Image*>* brdfInLightSamples /* = NULL */ )
{
	if( m_LightSamplesVersion != m_Settings->m_AmbientLightSampling->getVersion() )
	{
		createLightSamples();
		m_LightSamplesVersion = m_Settings->m_AmbientLightSampling->getVersion();
	}


	//bool brdfPingPong = false;

	SceneObjectIterator * iter = upc->createIterator(SOIT::CAP_BASIC_PC);
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	for( unsigned i=0;i<m_LightSamples.size();i++ )
	{
		progressWindow->progress(((float)m_Temp_NumPointsProcessed*m_LightSamples.size() + i*upc->getNumPoints()) * 100.0f / (m_Temp_NumTotalPoints*m_LightSamples.size()));
		TLightSource ls = m_LightSamples[i];
		// ---------------------------------------
		// generate shadowmap pass
		setRenderTarget( "RT_ShadowMap", RT_Depth, m_Settings->m_ShadowMapSize, m_Settings->m_ShadowMapSize);
		X4_GL_FB_DBG

		// clear & setup
		glClear( GL_DEPTH_BUFFER_BIT );
		glPushAttrib( GL_ALL_ATTRIB_BITS );
		glEnable(GL_DEPTH_TEST);
		glDepthFunc( GL_LESS );
		glDepthMask(1);

		// setup
		//setCamera( "root/camera1" );
		setCamera( ls.cam, ls.vf );
		setMaterial( m_Settings->m_Mat_GenerateShadowmap );
		//setMaterialFromScene( 1 );

		// render
		renderScene();

		glPopAttrib();


		// ---------------------------------------
		// render to 16bit render texture
		setRenderTarget( "RT_Precompute", RT_RGBA16, MAX_RENDER_TEXTURE_SIZE, MAX_RENDER_TEXTURE_SIZE);
		X4_GL_FB_DBG
		glPushAttrib( GL_ALL_ATTRIB_BITS );
		glDisable(GL_DEPTH_TEST);
		glDepthMask(0);
		// clear
		if( i==0 ) // first pass?
		{
			glClearColor(0,0,0,0);
			glClear( GL_COLOR_BUFFER_BIT);
			m_Settings->m_Mat_PrecomputeLight->setBlending(false);
		}
		else
		{
			m_Settings->m_Mat_PrecomputeLight->setBlending(true);
			m_Settings->m_Mat_PrecomputeLight->setDestBlendFunc(GLShaderMaterial::BM_ONE);
			m_Settings->m_Mat_PrecomputeLight->setSourceBlendFunc(GLShaderMaterial::BM_ONE);
		}


		// setup
		setDefaultCamera();
		setMaterial( m_Settings->m_Mat_PrecomputeLight );
		//GLHelper::begin2D(MAX_RENDER_TEXTURE_SIZE,MAX_RENDER_TEXTURE_SIZE);

		bindTarget( "RT_ShadowMap", "HQ_ShadowMap" );
		//setCameraAsParameter( "root/camera1", "HQ_ShadowCameraMatrix" );
		setCameraAsParameter( ls.cam, ls.vf, "HQ_ShadowCameraMatrix" );
		setMatrix4f( "ModelMatrix", modelMatrix );
		setMaterialParameter1f( "offset", m_Settings->m_ShadowMapOffset );
		Vector3f lightCol = ls.color * (ls.spot ? 1.0f:m_Settings->m_LightStrength);
		setMaterialParameter3f( "lightColor", lightCol[0] ,lightCol[1],lightCol[2] );
		setMaterialParameter1f( "shadowmapSize", m_Settings->getShadowMapSize());

		// render
		bIt->reset();
		
		m_Settings->m_Mat_PrecomputeLight->draw( getScene(), getScene()->getRootState(), NULL, iter, getGLContext());

		glPopAttrib();

		// ---------------------------------------
// 		// update input light direction and light index for the point (need for BRDF sampling)
// 		if (m_Settings->m_brdfEnable)
// 		{
// 			// on first iteration we clear both gathered light textures
// 			if (i == 0)
// 			{
// 				setRenderTarget( "RT_GatheredLightMap_A", RT_RGBA16, MAX_RENDER_TEXTURE_SIZE, MAX_RENDER_TEXTURE_SIZE);
// 				X4_GL_FB_DBG
// 				glClearColor(0,0,0,0);
// 				glClear( GL_COLOR_BUFFER_BIT);
// 				setRenderTarget( "RT_GatheredLightMap_B", RT_RGBA16, MAX_RENDER_TEXTURE_SIZE, MAX_RENDER_TEXTURE_SIZE);
// 				X4_GL_FB_DBG
// 				glClearColor(0,0,0,0);
// 				glClear( GL_COLOR_BUFFER_BIT);
// 			}
// 
// 			if (brdfPingPong) {
// 				setRenderTarget( "RT_GatheredLightMap_A", RT_RGBA16, MAX_RENDER_TEXTURE_SIZE, MAX_RENDER_TEXTURE_SIZE);
// 				X4_GL_FB_DBG
// 			} else {
// 				setRenderTarget( "RT_GatheredLightMap_B", RT_RGBA16, MAX_RENDER_TEXTURE_SIZE, MAX_RENDER_TEXTURE_SIZE);
// 				X4_GL_FB_DBG
// 			}
// 
// 			glPushAttrib( GL_ALL_ATTRIB_BITS );
// 			glDisable(GL_DEPTH_TEST);
// 			glDepthMask(0);
// 
// 			// setup
// 			setDefaultCamera();
// 			setMaterial( m_Settings->m_Mat_brdfGenerateLightDirectionmap );
// 
// 			bindTarget( "RT_ShadowMap", "ShadowMap" );
// 			if (brdfPingPong)
// 				bindTarget( "RT_GatheredLightMap_B", "GatheredLightMap" );
// 			else
// 				bindTarget( "RT_GatheredLightMap_A", "GatheredLightMap" );
// 
// 			setCameraAsParameter( ls.cam, ls.vf, "ShadowCameraMatrix" );
// 			setMatrix4f( "ModelMatrix", modelMatrix );
// 			setMaterialParameter1f( "offset", m_Settings->m_ShadowMapOffset );
// 			setMaterialParameter1f( "lightIndex", i );
// 
// 			// render
// 			bIt->reset();
// 			m_Settings->m_Mat_brdfGenerateLightDirectionmap->draw( getScene(), getScene()->getRootState(), NULL, iter, getGLContext());
// 
// 			brdfPingPong = !brdfPingPong;
// 
// 			glPopAttrib();
// 		}

	}
	delete iter;

	glDepthFunc( GL_LEQUAL );
	glDepthMask(1);

	X4_GL_FB_DBG
	GLRenderTextureFBO * rt = getRenderTargetFBO("RT_Precompute");
	X4_GL_FB_DBG
	RGBAFloat32Image * ret = rt->getRGBAFloat32Image();

	/*
	if (m_Settings->m_brdfEnable)
	{
		GLRenderTextureFBO * brdfrt = getRenderTargetFBO(brdfPingPong ? "RT_GatheredLightMap_B" : "RT_GatheredLightMap_A");
		RGBAFloat32Image * brdfret = brdfrt->getRGBAFloat32Image();
		
		ImageDisplayer::DisplayRGBAFloat32Image ev(brdfret);
		debugOutput.displayEvent(ImageDisplayer::getClass(), "brdf_gather", &ev);
	}*/

	//ImageDisplayer::DisplayRGBAFloat32Image ev(ret);
	//debugOutput.displayEvent(ImageDisplayer::getClass(), "precomp", &ev);

	return ret;
}

void X4::HighQualityRenderer::debug_SetLightSampleCam()
{
	static int currentIndex = 0;
	if( m_LightSamples.size() )
	{
		getScene()->getRootState()->staticState->camera = m_LightSamples[currentIndex % m_LightSamples.size()].cam;
		getScene()->getRootState()->staticState->viewFrustum = m_LightSamples[currentIndex % m_LightSamples.size()].vf;
	}
	currentIndex++;
}

void X4::HighQualityRenderer::clearScreen()
{
	glClearColor(m_Settings->m_BackgroundColor[0],m_Settings->m_BackgroundColor[1],m_Settings->m_BackgroundColor[2],m_Settings->m_BackgroundColor[3]);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	if( m_Settings->m_BackgroundImage )
	{
		glPushAttrib(GL_ALL_ATTRIB_BITS);
		GLTexture * tex = m_Settings->m_BackgroundImage->getCachedTexture(getGLContext());
		tex->bind(0);

		GLHelper::begin2D(1.0f,1.0f);
		glDisable(GL_DEPTH_TEST);
		glEnable(tex->getTextureTarget());
		glDepthMask(0);
		
		glBegin(GL_QUADS);
		float w = m_Settings->m_BackgroundImage->getImage()->getSizeX();
		float h = m_Settings->m_BackgroundImage->getImage()->getSizeY();

		glColor3f(1,1,1);
		glTexCoord2f(0,1);
		glVertex2f(0,0);
		glTexCoord2f(1,1);
		glVertex2f(1,0);
		glTexCoord2f(1,0);
		glVertex2f(1,1);
		glTexCoord2f(0,0);
		glVertex2f(0,1);
		
		glEnd();

		glDepthMask(1);
		GLHelper::end2D();
		tex->unbind(0);
		glPopAttrib();
	}
}

void X4::HighQualityRenderer::saveSettings()
{
	std::string filename = FileDialogs::getSaveFileName(NULL, "save settings to file", "*.hqr.x4obj");
	if (!filename.empty()) {
		BinaryOutputObjectStream out(filename.c_str());
		out.writeObject(m_Settings);
	}	
	
}

void X4::HighQualityRenderer::loadSettings()
{
	std::string filename = FileDialogs::getOpenFileName(NULL, "load settings", "*.hqr.x4obj");
	if (!filename.empty()) {
		BinaryInputObjectStream in(filename.c_str());
		HQRSettings * s = NULL;
		in.readAnyObject(s);
		m_Settings->assign(s);
		delete s;
	}	
}

//void X4::HighQualityRenderer::renderScreenSpaceAmbientOcclusion()
//{
//	glDepthFunc( GL_LESS );
//	glDepthMask(1);
//	// ---------------------------------------
//	// generate zbuffer (we could reuse existing zbuffer ...)
//	setRenderTarget( "RT_SSAO_ZBUFFER", RT_Depth | RT_RGBA16, getScreenWidth(), getScreenHeight() );
//	X4_GL_FB_DBG
//
//	// clear
//	glClear( GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
//
//	// setup
//	//setMaterialFromScene( 1 );
//	setMaterial( m_Settings->m_Mat_GenerateShadowmap );
//
//	DebugGL( getGLContext() );
//
//	// render
//	renderScene();
//	//		debugRenderer->renderJobs(getGLContext(),getScene(),(GLShaderMaterial *)getScene()->getGLMaterial(1));
//
//
//	// ---------------------------------------
//	// screen space pass
//	setDefaultRenderTarget();
//	X4_GL_FB_DBG
//	setRenderTarget( m_MainRenderTargetName, RT_Depth | (m_Settings->m_Use32Bit?RT_RGBA32:RT_RGBA16), getScreenWidth(), getScreenHeight() );
//	X4_GL_FB_DBG
//	glPushAttrib( GL_ALL_ATTRIB_BITS );
//	m_Settings->m_Mat_ScreenSpaceAmbientOcclusion->setBlending(true);
//	m_Settings->m_Mat_ScreenSpaceAmbientOcclusion->setDepthTest(false);
//	m_Settings->m_Mat_ScreenSpaceAmbientOcclusion->setDestBlendFunc(0);
//	m_Settings->m_Mat_ScreenSpaceAmbientOcclusion->setSourceBlendFunc(0);
//	glDisable( GL_DEPTH_TEST );
//	glEnable(GL_BLEND);
//	glBlendFunc(GL_DST_COLOR,GL_ZERO);
//	
//
//	// draw screen quad
//	unsigned width = getScreenWidth();
//	unsigned height = getScreenHeight();
//
//	GLHelper::begin2D(width,height);
//	m_Settings->m_Mat_ScreenSpaceAmbientOcclusion->bindShaders(getGLContext());
//	bindTarget( "RT_SSAO_ZBUFFER",0);
//
//
//	glBegin( GL_QUADS );
//	glColor3f(0.5,1,.5);
//	glTexCoord2f( 0,0 );
//	glVertex2f(0,0);
//	glTexCoord2f( width,0 );
//	glVertex2f(width,0);
//	glTexCoord2f( width,height );
//	glVertex2f(width,height);
//	glTexCoord2f( 0,height );
//	glVertex2f(0,height);
//	glEnd();
//	
//	GLHelper::end2D();
//	
//	m_Settings->m_Mat_ScreenSpaceAmbientOcclusion->unbindShaders(getGLContext());
//
//	glPopAttrib();
//
//
//}


bool X4::HighQualityRenderer::debugRendererAllowed()
{
	return m_Settings->m_AllowDebugRenderer;
}
X4::HQRSettings::HQRSettings()
{
	m_BackgroundImage = NULL;
	m_UpVectorIsZ = true;	// obsolete, will be ignored
	m_ShadowMapOffset = 0.0001f;
	m_Mat_SpecularEnviroment = NULL;
	m_AmbientLightSampling = new HDRLightprobeSampling;
	m_AmbientLightSampling->setupSamples(16);
	m_LightStrength = 1.0f;
	m_UseSpecularEnviromentShader = true;
	m_BackgroundColor = makeVector4f(1,1,1,1);
	m_ShadowMapSize = 512;
	m_Bloom = false;
	m_RotateAmbientLight = 0;
	m_RotateAmbientLightPhi = 0;
	m_RotateAmbientLightPsi = 0;
	m_ShowAmbientLightSamples = false;
	m_InstanciateDebugRendererCurrentFrame = false;
	finalPassWithObjMaterial = false;
	finalPassPathWithWildcards = "root/*";

	//// init screen space ambient occlusion material
	//m_Mat_ScreenSpaceAmbientOcclusion = new GLShaderMaterial;
	//if( !m_Mat_ScreenSpaceAmbientOcclusion->importFromTextFile( ApplicationDirectories::getResourcesDir()
	//	+ "shaders/hqRenderer/HQ_ScreenSpaceAmbientOcclusion.txt" ) )
	//{
	//	debugOutput << "Couldn't find " + ApplicationDirectories::getResourcesDir()
 //           + "shaders/hqRenderer/HQ_ScreenSpaceAmbientOcclusion.txt" + "\n";
	//}	
	//m_Mat_ScreenSpaceAmbientOcclusion->getVertexShader()->setShaderType( Shader::VERTEX );
	//m_Mat_ScreenSpaceAmbientOcclusion->getVertexShader()->setShaderClass( Shader::CLASS_CG );
	//m_Mat_ScreenSpaceAmbientOcclusion->getFragmentShader()->setShaderClass( Shader::CLASS_CG );
	//MPTexture * ssao_zbuffer = new MPTexture;
	//ssao_zbuffer->setName( "HQ_SSAO_ZBUFFER" ); // default stage=0
	//m_Mat_ScreenSpaceAmbientOcclusion->getParameters()->addParm( ssao_zbuffer );


	// init generateshadowmap material
	m_Mat_GenerateShadowmap = new GLShaderMaterial;
	if( !m_Mat_GenerateShadowmap->importFromTextFile( ApplicationDirectories::getResourcesDir()
		+ "shaders/hqRenderer/HQ_GenerateShadowMap.txt" ) )
	{
		debugOutput << "Couldn't find " + ApplicationDirectories::getResourcesDir()
			+ "shaders/hqRenderer/HQ_GenerateShadowMap.txt" + "\n";
	}
	m_Mat_GenerateShadowmap->getVertexShader()->setShaderType( Shader::VERTEX );
	m_Mat_GenerateShadowmap->getVertexShader()->setShaderClass( Shader::CLASS_CG );
	m_Mat_GenerateShadowmap->getFragmentShader()->setShaderClass( Shader::CLASS_CG );


	// init renderwithshadowmap material
	m_Mat_RenderWithShadowmap = new GLShaderMaterial;
	if( !m_Mat_RenderWithShadowmap->importFromTextFile( ApplicationDirectories::getResourcesDir()
		+ "shaders/hqRenderer/HQ_RenderWithShadowmapFiltered.txt" ) )
	{
		debugOutput << "Couldn't find " + ApplicationDirectories::getResourcesDir()
            + "shaders/hqRenderer/HQ_RenderWithShadowmapFiltered.txt" + "\n";
	}

	m_Mat_PrecomputeLight = new GLShaderMaterial;
	if( !m_Mat_PrecomputeLight->importFromTextFile( ApplicationDirectories::getResourcesDir()
        + "shaders/hqRenderer/HQ_PrecomputeAC.txt" ) )
	{
		debugOutput << "Couldn't find " + ApplicationDirectories::getResourcesDir()
            + "shaders/hqRenderer/HQ_PrecomputeAC.txt" + "\n";
	}
	m_Mat_PrecomputeLight->addVectorParameter("shadowmapSize");

	MPTexture * parm = new MPTexture;
	parm->setName( "HQ_ShadowMap" ); // default stage=0
	m_Mat_RenderWithShadowmap->getParameters()->addParm( parm );

	m_Mat_RenderWithShadowmap->addMatrixParameter("HQ_ShadowCameraMatrix");
	m_Mat_RenderWithShadowmap->addMatrixParameter("HQ_ShadowCameraMatrixinv");
	m_Mat_RenderWithShadowmap->addMatrixParameter("ModelMatrix");
	m_Mat_RenderWithShadowmap->addVectorParameter("HQ_ShadowCameraMatrixCamPos");
	m_Mat_RenderWithShadowmap->addVectorParameter("offset");
	m_Mat_RenderWithShadowmap->addVectorParameter("lightColor");  
	m_Mat_RenderWithShadowmap->addVectorParameter("const_w");
	m_Mat_RenderWithShadowmap->addVectorParameter("shadowmapSize");

	VertexDescriptor * tmpVD = m_Mat_RenderWithShadowmap->getVertexDescriptor();
	tmpVD->pushAttrib( VAD( "color",3,VAD::DATA_FORMAT_FLOAT32 ) );
	tmpVD->pushAttrib( VAD( "normal",3,VAD::DATA_FORMAT_FLOAT32 ) );

	MPTexture * parm2 = new MPTexture;
	parm2->setName( "HQ_ShadowMap" ); // default stage=0
	m_Mat_PrecomputeLight->getParameters()->addParm( parm2 );

	m_Mat_PrecomputeLight->addMatrixParameter("HQ_ShadowCameraMatrix");
	m_Mat_PrecomputeLight->addMatrixParameter("ModelMatrix");
	m_Mat_PrecomputeLight->addVectorParameter("HQ_ShadowCameraMatrixCamPos");
	m_Mat_PrecomputeLight->addVectorParameter("offset");
	m_Mat_PrecomputeLight->addVectorParameter("lightColor");  

	VertexDescriptor * tmpVD2 = createDefaultVertexDescriptor(true,false,true,false);
	tmpVD2->pushAttrib(VAD("screen_pos",2,VAD::DATA_FORMAT_FLOAT32));
	m_Mat_PrecomputeLight->getVertexDescriptor()->assign(tmpVD2);
	delete tmpVD2;

// 	// ---------------------
// 	//  BRDF
// 	// ---------------------
// 	{
// 		m_brdfEnable = false;
// 		m_brdfNumLightSamples = 1;
// 		m_brdfMaxNumBRDFs = 8;
// 		m_brdfMaxNumBRDFsToConsiderForAnalysis = 1000;
// 
// 		m_Mat_brdfGenerateLightDirectionmap = new GLShaderMaterial;
// 		if( !m_Mat_brdfGenerateLightDirectionmap->importFromTextFile( ApplicationDirectories::getResourcesDir() 
// 			+ "shaders/hqRenderer/BRDF_sampleLightDir.txt" ) )
// 		{
// 			debugOutput << "Couldn't find " + ApplicationDirectories::getResourcesDir() 
// 				+ "shaders/hqRenderer/BRDF_sampleLightDir.txt" + "\n";
// 		}
// 
// 		// shadow std::map as tex unit 0
// 		MPTexture * shadowMap = new MPTexture;
// 		shadowMap->setName( "ShadowMap" );
// 		m_Mat_brdfGenerateLightDirectionmap->getParameters()->addParm( shadowMap );
// 
// 		// gathered light info as tex unit 1
// 		MPTexture * gatheredLightMap_A = new MPTexture;
// 		gatheredLightMap_A->setName( "GatheredLightMap_A" ); 
// 		m_Mat_brdfGenerateLightDirectionmap->getParameters()->addParm( gatheredLightMap_A );
// 
// 		// for ping-pong usage of gathered light std::map texture
// 		MPTexture * gatheredLightMap_B = new MPTexture;
// 		gatheredLightMap_B->setName( "GatheredLightMap_B" ); 
// 		m_Mat_brdfGenerateLightDirectionmap->getParameters()->addParm( gatheredLightMap_B );
// 
// 		m_Mat_brdfGenerateLightDirectionmap->addMatrixParameter("ShadowCameraMatrix");
// 		m_Mat_brdfGenerateLightDirectionmap->addMatrixParameter("ModelMatrix");
// 		m_Mat_brdfGenerateLightDirectionmap->addVectorParameter("ShadowCameraMatrixCamPos");
// 		m_Mat_brdfGenerateLightDirectionmap->addVectorParameter("offset");
// 		m_Mat_brdfGenerateLightDirectionmap->addVectorParameter("lightIndex");  
// 
// 		VertexDescriptor * tmpVDspos = createDefaultVertexDescriptor(true,false,true,false);
// 		tmpVDspos->pushAttrib(VAD("screen_pos",2,VAD::DATA_FORMAT_FLOAT32));
// 		m_Mat_brdfGenerateLightDirectionmap->getVertexDescriptor()->assign(tmpVDspos);
// 		delete tmpVDspos;
// 	}
// 
}

X4::HQRSettings::~HQRSettings()
{
	delete m_Mat_RenderWithShadowmap;
	delete m_Mat_GenerateShadowmap;
	delete m_Mat_SpecularEnviroment;
	delete m_AmbientLightSampling;
	delete m_Mat_PrecomputeLight;
	//delete m_Mat_brdfGenerateLightDirectionmap;
}

void X4::HQRSettings::assign( const Object* obj, X4CopyContext *context /* = NULL */ )
{
	copyObjectProperties(obj, this);
}

void HQRSettings::addHQRLight( Object * obj ) 
{
	m_SpotLights.push_back( (HQRLight*)obj );
}

void HQRSettings::deleteHQRLights( card32 begin, card32 end ) 
{
	for( card32 i=begin;i<end;i++ )
		delete m_SpotLights[i];
	m_SpotLights.erase( m_SpotLights.begin() + begin, m_SpotLights.begin() + end );
}

Object *HQRSettings::getHQRLight( card32 index ) 
{
	return m_SpotLights[index];
}

void HQRSettings::moveHQRLight( card32 oldPos, card32 newPos ) 
{
}

mpcard HQRSettings::getNumHQRLights() 
{
	return m_SpotLights.size();
}


std::string HQRSettings::getHQRLightName(card32 index ) 
{
	std::string ret = std::string("Light[") + intToStr(index) + "] - " + 
		m_SpotLights[index]->getCameraName();
	return ret;
}

void HighQualityRenderer::customEvent( std::string e ) 
{
	if( e == "SCREENSHOT_DONE" )
	{
		if( m_Settings->m_TurnOffLiveRenderingAfterScreenshot )
		{
			m_RenderScene = false;
			m_RenderLightSamples = false;
		}
	}
}
