#ifndef HighQualityRendererH
#define HighQualityRendererH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "MultiPassRenderer.h"
//---------------------------------------------------------------------------

namespace X4
{

class GLShaderMaterial;
class HQRLight;
class HDRLightprobeSampling;
class RGBAFloat32Image;
class UnstructuredInCorePointCloud;
class RGBATextureAttachment;
class HQRSettings;
class GLImage;
class HighQualityRendererEditorWidget;

/**
	offers high quality rendering (surface splatting, shadowmaps, area light sources)
*/
class X3FOUNDATION_PRIVATE_API HighQualityRenderer : public MultiPassRenderer
{
	X4_CLASS( HighQualityRenderer )

protected:

	//bool						m_RenderBRDF;
	bool						m_RenderLightSamples;
	bool						m_RenderSpotLightSamples;
	bool						m_RenderScene;
	//bool						m_UseScreenSpaceAmbientOcclusion;
	card32						m_LightSamplesVersion;
	HQRSettings		*			m_Settings;

	struct TLightSource
	{
		Camera cam;
		ViewFrustum vf;
		Vector3f color;
		bool spot;
	};
	std::vector<TLightSource> m_LightSamples;
	std::map<card32,std::vector<Vector3f> >		m_SpotLightDistribution;

	// temporal members
	card32		m_Temp_NumTotalPoints;
	card32		m_Temp_NumPointsProcessed;
	float		m_Temp_ConstW;
	GLImage	*	m_MainRenderTarget;
	std::string      m_MainRenderTargetName;
	bool        m_doImageOperations;

	//void renderScreenSpaceAmbientOcclusion();
	void createLightSamples();
	virtual void renderLightSample( TLightSource ls, bool firstPass);	
	void renderNodesDefaultMat();
	std::vector<Vector3f> getSpotLightDistribution( card32 numSamples );

	RGBAFloat32Image * precomputeLight_Block( UnstructuredInCorePointCloud * upc, const Matrix4f& modelMatrix);//, std::vector<RGBAFloat32Image*>* brdfInLightSamples = NULL );
	static const unsigned int MAX_RENDER_TEXTURE_SIZE = 1024;

	void clearScreen();

public:
	HighQualityRenderer(void);
	virtual ~HighQualityRenderer(void);

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);
	virtual void connect(Scene* scene, GLContext* ctxt);

	void updateLightSamples();
	void precomputeLight();
	void debug_SetLightSampleCam();
	void saveSettings();
	void loadSettings();
	//virtual void prepareBRDF();
	void setMainRenderTarget(GLImage * rt) {m_MainRenderTarget=rt;}
	virtual void customEvent( std::string e );


	virtual bool debugRendererAllowed();

protected:

	virtual void renderMultiPass();
	virtual void performPostprocessing() {}

	friend	HighQualityRendererEditorWidget;
};


class X3FOUNDATION_PRIVATE_API HQRSettings : public Persistent
{
	X4_CLASS( HQRSettings );
public:

	HQRSettings();
	~HQRSettings();

	void assign(const Object* obj, X4CopyContext *context  = NULL );

	void addHQRLight( Object * obj );
	void deleteHQRLights( card32 begin, card32 end );
	Object * getHQRLight( card32 index );
	void moveHQRLight( card32 oldPos, card32 newPos );
	mpcard getNumHQRLights();
	std::string getHQRLightName(card32 index );
	card32 getShadowMapSize() const {return m_ShadowMapSize;}

	GLShaderMaterial	*	m_Mat_GenerateShadowmap;
	GLShaderMaterial	*	m_Mat_RenderWithShadowmap;
	GLShaderMaterial	*	m_Mat_SpecularEnviroment;
	GLShaderMaterial	*	m_Mat_PrecomputeLight;
	//GLShaderMaterial	*	m_Mat_ScreenSpaceAmbientOcclusion;
	float						m_ShadowMapOffset;
	HDRLightprobeSampling * m_AmbientLightSampling;
	float						m_LightStrength;
	bool						m_UseSpecularEnviromentShader;
	Vector4f					m_BackgroundColor;
	bool						m_UpVectorIsZ;	// obsolete
	RGBATextureAttachment	*			m_BackgroundImage;
	std::vector<HQRLight*>		m_SpotLights;
	bool					m_Bloom;
	bool					m_TurnOffLiveRenderingAfterScreenshot;
	bool					m_AllowDebugRenderer;
	card32					m_ShadowMapSize;
	bool					m_Use32Bit;
	float32					m_RotateAmbientLight;
	float32					m_RotateAmbientLightPhi;
	float32					m_RotateAmbientLightPsi;
	bool					m_ShowAmbientLightSamples;
	bool					m_InstanciateDebugRendererCurrentFrame;
	bool              finalPassWithObjMaterial;
	string				finalPassPathWithWildcards; // nodes in the scene graph to be rendered with their default material, in a second pass (supports wildcards)

	friend	HighQualityRendererEditorWidget;

// 	// BRDF settings
// 	bool           m_brdfEnable;
// 	card16         m_brdfNumLightSamples;
// 	float          m_brdfMaxNumBRDFsToConsiderForAnalysis;
// 	GLShaderMaterial	*	m_Mat_brdfGenerateLightDirectionmap;
// 	card16 m_brdfMaxNumBRDFs;
};

}

#endif