#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "HighQualityRendererEditorWidget.h"
//---------------------------------------------------------------------------
#include "HighQualityRenderer.h"
#include "Configuration.h"
#include "HDRLightprobeSampling.h"
#include "Scene.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "PointCloud.h"
#include "PointCloudTools.h"
#include "SelectionTools.h"
#include "ApplicationDirectories.h"

//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_CLASS_EDITOR( HighQualityRenderer )

const float HighQualityRendererEditorWidget::minShadowMapOffset = 0.0001f;
const float HighQualityRendererEditorWidget::maxExponent = 3.0f;


HighQualityRendererEditorWidget::HighQualityRendererEditorWidget(QWidget * parent, Qt::WindowFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));

	m_EnvMapWidget = NULL;
	m_AdvancedWidget = NULL;
	std::string envMapPath = ApplicationDirectories::getResourcesDir();
	envMapPath += "envMaps";
	QDir dir(envMapPath.c_str(),"*.pfm");
	QStringList availableFiles = dir.entryList();
	m_PresetFileNames.push_back("no preset");

	ui.comboPreset->addItem("no preset");
	for( mpcard i=0;i<(mpcard)availableFiles.size();i++ )
	{
		m_PresetFileNames.push_back(envMapPath + "\\" + qString2STLString(availableFiles.at(i)));
		ui.comboPreset->addItem(availableFiles.at(i));				
	}

	m_EnvMapClassEditor = NULL;
	m_AdvancedClassEditor = NULL;

}

HighQualityRendererEditorWidget::~HighQualityRendererEditorWidget(void)
{
	delete m_AdvancedWidget;
	delete m_AdvancedClassEditor;
	delete m_EnvMapWidget;
	delete m_EnvMapClassEditor;
}

void HighQualityRendererEditorWidget::setup( HighQualityRenderer * obj )
{
	m_Object = obj;
}

void setLabelText(QLabel * l, const QString &baseText, int v)
{
	QString vString; vString.setNum(v);
	QString text = baseText + " [" + vString + "]";
	l->setText(text);
}

void setLabelTextF(QLabel * l, const QString &baseText, float f)
{
	QString text = baseText + " [" + floatToStr(f).c_str() + "]";
	l->setText(text);
}


void HighQualityRendererEditorWidget::updateEditor()
{
	ui.cbLiveRendering->setChecked( m_Object->m_RenderScene && m_Object->m_RenderLightSamples );
	ui.cbShowOrientation->setChecked(m_Object->m_Settings->m_ShowAmbientLightSamples);

	ui.sliderLightStrength->setValue((m_Object->m_Settings->m_LightStrength-0.1f)/2.9f*(float)ui.sliderLightStrength->maximum());
	setLabelTextF(ui.labelLightStrength,"Light strength",m_Object->m_Settings->m_LightStrength);

	ui.sliderNumSamples->setValue(m_Object->m_Settings->m_AmbientLightSampling->getNumSamples());
	setLabelText(ui.labelNumSamples,"Num samples",m_Object->m_Settings->m_AmbientLightSampling->getNumSamples());

	ui.sliderTheta->setValue(m_Object->m_Settings->m_RotateAmbientLight);
	ui.sliderPhi->setValue(m_Object->m_Settings->m_RotateAmbientLightPhi);
	setLabelText(ui.labelTheta,"Theta",m_Object->m_Settings->m_RotateAmbientLight);
	setLabelText(ui.labelPhi,"Phi",m_Object->m_Settings->m_RotateAmbientLightPhi);
	ui.cbBloom->setCheckState(m_Object->m_Settings->m_Bloom?Qt::Checked:Qt::Unchecked);
	ui.cbTurnOffLiveRenderingAfterScreenshot->setCheckState(m_Object->m_Settings->m_TurnOffLiveRenderingAfterScreenshot?Qt::Checked:Qt::Unchecked);
	ui.cbUse32bit->setCheckState(m_Object->m_Settings->m_Use32Bit?Qt::Checked:Qt::Unchecked);
	ui.cbFullSphere->setCheckState(!m_Object->m_Settings->m_AmbientLightSampling->getSampleHalfSphereFlag()?Qt::Checked:Qt::Unchecked);

	setLabelTextF(ui.labelShadowmapOffset,"Offset",m_Object->m_Settings->m_ShadowMapOffset);
	ui.sliderOffset->setValue(offsetToSlider(m_Object->m_Settings->m_ShadowMapOffset));

	Vector4f col = m_Object->m_Settings->m_BackgroundColor;
	QColor color(col[0]*255.0f, col[1]*255.0f, col[2]*255.0f);

	ui.pbBGColor->setPalette(QPalette(color));
	ui.pbBGColor->setAutoFillBackground(true);



	if( m_Object->m_Settings->m_ShadowMapSize <= 512)
		ui.comboShadowmapSize->setCurrentIndex(0);
	else
	if( m_Object->m_Settings->m_ShadowMapSize <= 1024)
		ui.comboShadowmapSize->setCurrentIndex(1);
	else
	if( m_Object->m_Settings->m_ShadowMapSize <= 2048)
		ui.comboShadowmapSize->setCurrentIndex(2);
	else
		ui.comboShadowmapSize->setCurrentIndex(3);
}

#include "HighQualityRendererEditorWidget_moc.h"

void HighQualityRendererEditorWidget::on_pbPrecompute_clicked() 
{
	m_Object->precomputeLight();
}

void HighQualityRendererEditorWidget::on_pbCopyChannelColorToColor2_clicked() 
{
	std::vector<PointCloud*> sel = getSelectedPointClouds(m_Object->getScene());
	for( int i1 = 0; i1 < (int)sel.size(); ++ i1 ) {
		//	StandardSceneGraphIterator sgit;
		//	sgit.clearAndSetup(m_Object->getScene()->getRootNode(), m_Object->getScene()->getRootState());
		//
		//	while( !sgit.atEnd() ) {
		//		SceneGraphNode* sgn = sgit.getNode();
		//		sgit.next();
		//
		//		SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( sgn );
		//
		//		if( !sgon )
		//			continue;
		//
		//		PointCloud* pc = dynamic_cast<PointCloud*>( sgon->getSceneObject() );
		PointCloud* pc = sel[i1];
		if( !pc )
			continue;

		//		// adding another color channel breaks on certain PointClouds, so only consider UnstructuredInCorePointClouds by now
		//		UnstructuredInCorePointCloud* pc2 = dynamic_cast<UnstructuredInCorePointCloud*>( pc );
		//		if( !pc2 )
		//			continue;

		if( !pc->providesAttribute("color")) continue;
		checkAttribute(pc, "color2", 3, VAD::DATA_FORMAT_FLOAT32);


		SceneObjectIterator *itSO = pc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL);
		//		PointCloudIterator *it = dynamic_cast<PointCloudIterator*>(itSO);
		BasicPointCloudIterator* bIt = dynamic_cast<BasicPointCloudIterator*>(itSO);
		ModifyablePointCloudIterator* mIt = dynamic_cast<ModifyablePointCloudIterator*>(itSO);
		if (bIt==NULL || mIt == NULL) { delete itSO; continue; };


		AAT colorAAT = pc->getAAT("color");
		AAT color2AAT = pc->getAAT("color2");

		while(!bIt->atEnd()) {
			mIt->set3f(color2AAT, bIt->get3f(colorAAT));
			bIt->next();
		};
		delete itSO;
	}
}


void HighQualityRendererEditorWidget::on_pbCopyChannelColor2ToColor_clicked() 
{
	std::vector<PointCloud*> sel = getSelectedPointClouds(m_Object->getScene());
	for( int i1 = 0; i1 < (int)sel.size(); ++ i1 ) {
		//	StandardSceneGraphIterator sgit;
		//	sgit.clearAndSetup(m_Object->getScene()->getRootNode(), m_Object->getScene()->getRootState());
		//
		//	while( !sgit.atEnd() ) {
		//		SceneGraphNode* sgn = sgit.getNode();
		//		sgit.next();
		//
		//		SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( sgn );
		//
		//		if( !sgon )
		//			continue;
		//
		//		PointCloud* pc = dynamic_cast<PointCloud*>( sgon->getSceneObject() );
		PointCloud* pc = sel[i1];
		if( !pc )
			continue;

		//		// adding another color channel breaks on certain PointClouds, so only consider UnstructuredInCorePointClouds by now
		//		UnstructuredInCorePointCloud* pc2 = dynamic_cast<UnstructuredInCorePointCloud*>( pc );
		//		if( !pc2 )
		//			continue;

		if( !pc->providesAttribute("color2")) continue;
		checkAttribute(pc, "color", 3, VAD::DATA_FORMAT_FLOAT32);


		SceneObjectIterator *itSO = pc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL);
		//		PointCloudIterator *it = dynamic_cast<PointCloudIterator*>(itSO);
		BasicPointCloudIterator* bIt = dynamic_cast<BasicPointCloudIterator*>(itSO);
		ModifyablePointCloudIterator* mIt = dynamic_cast<ModifyablePointCloudIterator*>(itSO);
		if (bIt==NULL || mIt == NULL) { delete itSO; continue; };


		AAT colorAAT = pc->getAAT("color");
		AAT color2AAT = pc->getAAT("color2");

		while(!bIt->atEnd()) {
			mIt->set3f(colorAAT, bIt->get3f(color2AAT));
			bIt->next();
		};
		delete itSO;
	}
}


void HighQualityRendererEditorWidget::on_cbLiveRendering_clicked() 
{
	bool checked = ui.cbLiveRendering->isChecked();
	m_Object->m_RenderLightSamples = checked;
	m_Object->m_RenderScene = checked;
}

void HighQualityRendererEditorWidget::on_comboPreset_activated(int i) 
{
	if( i > 0)
	{
		m_Object->m_Settings->m_AmbientLightSampling->loadFromPFM(m_PresetFileNames[i].c_str());
		m_Object->m_Settings->m_AmbientLightSampling->setup();
		m_Object->createLightSamples();
		ObjectViewsTable::update(m_Object->m_Settings->m_AmbientLightSampling);
		ObjectViewsTable::update(m_Object->getScene());
	}
}

void HighQualityRendererEditorWidget::on_pbEnvMap_clicked() 
{
	delete m_EnvMapWidget;
	delete m_EnvMapClassEditor;
	Object * envmap = (Object*)m_Object->m_Settings->m_AmbientLightSampling;
	if( !envmap)
		return;
	m_EnvMapClassEditor = (ClassEditor *)envmap->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
	m_EnvMapWidget = m_EnvMapClassEditor->makeEditor(dynamic_cast<QWidget*>(this),envmap);
	m_EnvMapWidget->setWindowFlags(Qt::Dialog);
	//m_EnvMapWidget->setAttribute(Qt::WA_DeleteOnClose);
	QObject::connect(m_EnvMapWidget,SIGNAL(sigObjectChanged()),this,SLOT(sltAmbientChanged()));
	m_EnvMapWidget->show();
}

void HighQualityRendererEditorWidget::on_cbShowOrientation_clicked() 
{
	m_Object->m_Settings->m_ShowAmbientLightSamples = ui.cbShowOrientation->isChecked();
	m_Object->createLightSamples();
	if( m_Object->m_Settings->m_ShowAmbientLightSamples )
	{
		ObjectViewsTable::update(m_Object->getScene());
	}
}

void HighQualityRendererEditorWidget::on_sliderTheta_sliderMoved(int theta) 
{
	m_Object->m_Settings->m_RotateAmbientLight = theta;
	setLabelText(ui.labelTheta,"Theta",m_Object->m_Settings->m_RotateAmbientLight);
	m_Object->createLightSamples();
	if( m_Object->m_Settings->m_ShowAmbientLightSamples )
	{
		ObjectViewsTable::update(m_Object->getScene());
	}
}

void HighQualityRendererEditorWidget::on_sliderPhi_sliderMoved(int phi) 
{
	m_Object->m_Settings->m_RotateAmbientLightPhi = phi;
	setLabelText(ui.labelPhi,"Phi",m_Object->m_Settings->m_RotateAmbientLightPhi);
	m_Object->createLightSamples();
	if( m_Object->m_Settings->m_ShowAmbientLightSamples )
	{
		ObjectViewsTable::update(m_Object->getScene());
	}
}

void X4::HighQualityRendererEditorWidget::on_sliderPsi_sliderMoved( int psi)
{
	m_Object->m_Settings->m_RotateAmbientLightPsi = psi;
	setLabelText(ui.labelPsi,"Psi",m_Object->m_Settings->m_RotateAmbientLightPsi);
	m_Object->createLightSamples();
	if( m_Object->m_Settings->m_ShowAmbientLightSamples )
	{
		ObjectViewsTable::update(m_Object->getScene());
	}

}

void X4::HighQualityRendererEditorWidget::on_cbFullSphere_clicked()
{
	m_Object->m_Settings->m_AmbientLightSampling->setSampleHalfSphereFlag(!ui.cbFullSphere->isChecked());
	m_Object->m_Settings->m_AmbientLightSampling->setup();
	m_Object->createLightSamples();
	if( m_Object->m_Settings->m_ShowAmbientLightSamples )
	{
		ObjectViewsTable::update(m_Object->getScene());
	}
}

void X4::HighQualityRendererEditorWidget::on_sliderNumSamples_sliderMoved( int n)
{
	m_Object->m_Settings->m_AmbientLightSampling->setupSamples(n);
	setLabelText(ui.labelNumSamples,"Num samples",m_Object->m_Settings->m_AmbientLightSampling->getNumSamples());
	m_Object->createLightSamples();
	if( m_Object->m_Settings->m_ShowAmbientLightSamples )
	{
		ObjectViewsTable::update(m_Object->getScene());
	}

}

void HighQualityRendererEditorWidget::on_sliderOffset_sliderMoved(int offset) 
{
	float fOffset = sliderToOffset(offset);
	m_Object->m_Settings->m_ShadowMapOffset = fOffset;
	setLabelTextF(ui.labelShadowmapOffset,"Offset",m_Object->m_Settings->m_ShadowMapOffset);
}

void HighQualityRendererEditorWidget::on_comboShadowmapSize_activated(int i) 
{
	switch (i)
	{
	default:
	case 0:
		m_Object->m_Settings->m_ShadowMapSize = 512;
		break;
	case 1:
		m_Object->m_Settings->m_ShadowMapSize = 1024;
		break;
	case 2:
		m_Object->m_Settings->m_ShadowMapSize = 2048;
		break;
	case 3:
		m_Object->m_Settings->m_ShadowMapSize = 4096;
		break;
	}
}

void HighQualityRendererEditorWidget::on_pbBGColor_clicked() 
{
	Vector4f bgcol = m_Object->m_Settings->m_BackgroundColor;
	QColor col(bgcol[0]*255.0f,bgcol[1]*255.0f,bgcol[2]*255.0f);
	QColor newCol = QColorDialog::getColor(col, this); //,"select back ground color"); // Miguel: title not supported in Qt 4.3
	if( newCol.isValid())
	{
		m_Object->m_Settings->m_BackgroundColor = makeVector4f((float)newCol.red() / 255.0f,(float)newCol.green() / 255.0f,(float)newCol.blue() / 255.0f,1.0f);
		ui.pbBGColor->setPalette(QPalette(newCol));
		ui.pbBGColor->setAutoFillBackground(true);
	}
}

void HighQualityRendererEditorWidget::on_cbUse32bit_clicked() 
{
	m_Object->m_Settings->m_Use32Bit = ui.cbUse32bit->isChecked();
}

void HighQualityRendererEditorWidget::on_cbBloom_clicked() 
{
	m_Object->m_Settings->m_Bloom = ui.cbBloom->isChecked();
}

void HighQualityRendererEditorWidget::on_pbAdvanced_clicked() 
{
	delete m_AdvancedWidget;
	delete m_AdvancedClassEditor;
	Object * settings = (Object*)m_Object->m_Settings;
	if( !settings)
		return;
	m_AdvancedClassEditor = (ClassEditor *)settings->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
	m_AdvancedWidget = m_AdvancedClassEditor->makeEditor(dynamic_cast<QWidget*>(this),settings);
	m_AdvancedWidget->setWindowFlags(Qt::Dialog);
	//m_AdvancedWidget->setAttribute(Qt::WA_DeleteOnClose);
	m_AdvancedWidget->show();
	
}

void HighQualityRendererEditorWidget::on_pbLoadSettings_clicked() 
{
	m_Object->loadSettings();
	updateEditor();
}

void HighQualityRendererEditorWidget::on_pbSaveSettings_clicked() 
{
	m_Object->saveSettings();
}



float HighQualityRendererEditorWidget::sliderToOffset(int s) const 
{
	float sf = (float)s / (float)ui.sliderOffset->maximum();
	float offset = minShadowMapOffset * pow(0.1f,sf*maxExponent);
	debugOutput << "sliderToOffset(" << sf << ") = " << offset << "\n";
	return offset;
}

int HighQualityRendererEditorWidget::offsetToSlider( float offset) const 
{
	float sf = offset / minShadowMapOffset;
	sf = log(sf) / log(0.1f)/ maxExponent;
	return sf  * (float)ui.sliderOffset->maximum();
}

void X4::HighQualityRendererEditorWidget::on_sliderLightStrength_sliderMoved( int v)
{
	float ls = (float)v / (float)ui.sliderLightStrength->maximum();
	m_Object->m_Settings->m_LightStrength = ls*2.9f + 0.1f;
	setLabelTextF(ui.labelLightStrength,"Light strength",m_Object->m_Settings->m_LightStrength);
}

void X4::HighQualityRendererEditorWidget::sltAmbientChanged()
{
	m_Object->m_Settings->m_AmbientLightSampling->setup();
	m_Object->createLightSamples();
	if( m_Object->m_Settings->m_ShowAmbientLightSamples )
	{
		ObjectViewsTable::update(m_Object->getScene());
	}

}

// void HighQualityRendererEditorWidget::on_pbBRDFprepare_clicked() 
// {
// 	// ask user if he really want to prepare BRDF
// 	if (QMessageBox::question(this, "Do you really want to prepare BRDF for rendering?", "Please note that BRDF prepare will analysis all attached BRDF of every selected point cloud. This method is pretty time consuming. Do you really want to take this opportunity to burn your time?\n\nWarning: All BRDF materials will be replaced by the reduced set.", QMessageBox::Yes | QMessageBox::No)  == QMessageBox::Yes)
// 		m_Object->prepareBRDF();
// }
// 
// void HighQualityRendererEditorWidget::on_cbBRDFactive_clicked() 
// {
// 	bool checked = ui.cbBRDFactive->isChecked();
// 	m_Object->m_RenderBRDF = checked;
// }

void HighQualityRendererEditorWidget::on_cbTurnOffLiveRenderingAfterScreenshot_clicked() 
{
	m_Object->m_Settings->m_TurnOffLiveRenderingAfterScreenshot = ui.cbTurnOffLiveRenderingAfterScreenshot->isChecked();
}
