#ifndef HighQualityRendererEditorWidgetH 
#define HighQualityRendererEditorWidgetH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "ClassEditor.h"
#include "HighQualityRendererEditorForm_ui.h"
//---------------------------------------------------------------------------

namespace X4
{

class HighQualityRenderer;

class X3FOUNDATION_PRIVATE_API HighQualityRendererEditorWidget : public QWidget
{
	Q_OBJECT

protected:

	HighQualityRenderer	*	m_Object;

	///used to access buttons, etc. from QtDesigner
	Ui_HighQualityRendererEditorForm ui;

	QWidget *	m_EnvMapWidget;
	QWidget	*	m_AdvancedWidget;
	ClassEditor	*	m_EnvMapClassEditor, * m_AdvancedClassEditor;

	float sliderToOffset(int s) const;
	int offsetToSlider( float offset) const;
	static const float minShadowMapOffset;
	static const float maxExponent;
	std::vector<std::string>	m_PresetFileNames;


public:
	HighQualityRendererEditorWidget(QWidget * parent = NULL, Qt::WindowFlags f=0);
	virtual ~HighQualityRendererEditorWidget(void);

	virtual void setup( HighQualityRenderer * obj );

	/// update editor content with object data
	virtual void updateEditor();

	HighQualityRenderer* getRenderer() { return m_Object; }

public slots:

	void on_pbPrecompute_clicked();
	void on_pbCopyChannelColorToColor2_clicked();
	void on_pbCopyChannelColor2ToColor_clicked();
	void on_cbLiveRendering_clicked();
	void on_sliderLightStrength_sliderMoved(int);
	void on_sliderNumSamples_sliderMoved(int);

	void on_comboPreset_activated(int);
	void on_pbEnvMap_clicked();
	void on_cbShowOrientation_clicked();
	void on_sliderTheta_sliderMoved(int);
	void on_sliderPhi_sliderMoved(int);
	void on_sliderPsi_sliderMoved(int);


	void on_sliderOffset_sliderMoved(int);
	void on_comboShadowmapSize_activated(int);

	void on_pbBGColor_clicked();
	void on_cbUse32bit_clicked();
	void on_cbBloom_clicked();
	void on_cbTurnOffLiveRenderingAfterScreenshot_clicked();
	void on_cbFullSphere_clicked();
	void on_pbAdvanced_clicked();

	void on_pbLoadSettings_clicked();
	void on_pbSaveSettings_clicked();

	void sltAmbientChanged();

// 	void on_pbBRDFprepare_clicked();
// 	void on_cbBRDFactive_clicked();

};

MAKE_CLASS_EDITOR( X3FOUNDATION_PRIVATE_API , HighQualityRenderer )


}

#endif