#ifndef HoppeMesherH
#define HoppeMesherH

#include "x3foundation_private.h"

#include "Persistent.h"
#include "PointCloud.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "PointSetKNNQuery.h"

namespace X4
{

#define DISTANCE_CLOSEST_POINT 0
#define DISTANCE_EPSILON 1

/*
 * Mesh a point cloud/set using Hoppe's method [HDD*92]
 *
 * Required: 
 * - position (3f)
 * - normal   (3f)   NormalEstimator, NormalUnifier
 *
 * Output:
 * UnstructuredInCoreTriangleMesh
*/
class X3FOUNDATION_PRIVATE_API HoppeMesher : public Persistent
{	
	private:

		static float32 eval( 
			PointSet* ps, 
			const Vector3f& x, 
			PointSetKNNQuery& knn,
			const float32 tau,
			const float32 epsilon = -1,
			const card32 distanceMode = DISTANCE_CLOSEST_POINT );

	public:

		/// Create triangle mesh from point set
		static UnstructuredInCoreTriangleMesh* triangulate( 
			PointSet* ps, 
			const card32 res, 
			const float32 epsilon = -1,								/// distanceMode == DISTANCE_EPSILON
			const card32 distanceMode = DISTANCE_CLOSEST_POINT );

		/// Create triangle mesh from point cloud
		static UnstructuredInCoreTriangleMesh* triangulate(
			PointCloud* pc, 
			const card32 res, 
			const float32 epsilon = -1,								/// distanceMode == DISTANCE_EPSILON
			const card32 distanceMode = DISTANCE_CLOSEST_POINT );

};

}

#endif