#ifndef ICPH
#define ICPH

// Project
#include "LinearAlgebra.h"
//#include "EventViewerWidget.h" // comment out by CL @ 2014/02/17
#include "Persistent.h"
#include "Transformation.h"
#include "Registerizer.h"
#include "ICPOctree.h"
#include "ICPKNNQuery.h"

// System
#include <vector>

namespace X4{

#define FLAG_REGISTER_CANDIDATE 16	// Bit set: point used in registration process

/// Implementation of the ICP algorithm for registration of two point clouds
class X3FOUNDATION_PRIVATE_API ICP : public Registerizer
{
	X4_CLASS( ICP ) 
	private:
		ICPOctree* octree;				// Only created if required!
		ICPKNNQuery* knnBase;			// Only created if required!
		bool ownOctree;
		Vector3f normalizationT;		// Normalization to [-1;1]^3
		float32 normalizationS;			// Normalization to [-1;1]^3

		Matrix4f getNormalization() const;
		Matrix4f getInvNormalization() const;
		
	public:
		/// Constructor
		ICP();
		/// Destructor
		~ICP();

		void selectRegisterCandidates();

		/// Setup all required members
		void setup( PointSet* basePS, const Matrix3f& baseR = IDENTITY3F, const Vector3f& baseT = NULL_VECTOR3F );
		void setupWithOctree( PointSet* basePS, ICPOctree* octree );
		float32 computeGradient( StaticVector<float32,7>& gradient );
		float32 computeHessian( StaticMatrix<float32,7,7>& hessian, StaticVector<float32,7>& gradient );
		float32 computeScore(Matrix3f rot, Vector3f t) { return -1.0f; };
		inline bool supportsGradientDescent(){ return true; };
		inline bool supportsNewton(){ return false; };
		Matrix4f getTransformation() const;
		
		/*** MEMBER ACCESS ***/
		inline ICPOctree* getOctree() const { return octree; }
};



}

#endif