//---------------------------------------------------------------------------
#ifndef ICPKNNQueryH
#define ICPKNNQueryH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
#include "HierarchicalKNNIterator.h"
#include "ICPOctree.h"
#include <queue>
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {

   struct ICPPointEntry {
	   card32 index;
	   float32 quDist;
	   ICPPointEntry(const card32& i, const float32& q) {quDist = q; index = i;}
	   inline bool operator<(const ICPPointEntry &other) const {
		   return quDist > other.quDist;
	   }
   };

/// helper structure, used in KNNQuery
struct ICPBoxEntry {
	ICPOctreeNode* node;
	float32 quDist;
	ICPBoxEntry( ICPOctreeNode *n, const float32& q) {quDist = q; node = n;}
	inline bool operator<(const ICPBoxEntry &other) const {
		return quDist > other.quDist;
	}
};


/// compute k-nearest neighbors efficiently using a hierarchical data structure
class X3FOUNDATION_PRIVATE_API ICPKNNQuery {
	private:
		ICPOctree* octree;
		const PointSet *ps;
		AAT POSITION;
		Vector3f targetPoint;
		std::priority_queue<ICPPointEntry> pqPoints;
		std::priority_queue<ICPBoxEntry> pqBoxes;
		ICPOctreeNode* rootNode;

		void putBoxOnQueue(ICPOctreeNode* box);
		void putPointOnQueue(const card32 &indexOfPoint);

	public:   
		ICPKNNQuery(const PointSet *ps, ICPOctree *octree, const Vector3f &targetPoint, const AAT &positionAAT);
		/// reset position in space where search starts
		void setTargetPoint(const Vector3f &targetPoint);
		/// get next nearest point. found is true if another point has been found. if found is false, nextIndex is undefined.
		void getNextNearestPointIndex(card32 &nextIndex, bool &found);
};



}

#endif