//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "ICPOctree.h"
#include "BoundingBox.inline.h"
#include "Persistent.h"
#include "Tools.h"
#include "DebugOutput.h"
#include "PointSet.inline.h"
#include "linearalgebra.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( ICPOctree, 0 ) 
{
	BEGIN_CLASS_INIT( ICPOctree );
}

card64 ICPOctree::m_globalAge = 0;

void ICPOctree::octreesetup( const PointSet* ps, card32 max_points, const Matrix3f& R, const Vector3f& t )
{
	this->R = R;
	this->t = t;
	m_octree.deleteChildren();
	
	m_ps = ps;
	m_age = m_globalAge;
	const VertexDescriptor* desc = ps->getDescr();
	const AAT POSITION = desc->getToken( "position" );
	//const AAT FLAGS = desc->getToken( "flags" );
	card32 numProcessed = 0;

	// Compute bounding box
	BoundingBox3f bbox;
	bool first = true;
	for ( unsigned int pointIndex = 0; pointIndex < ps->getNumEntries(); pointIndex++ ){
		//if ( isValidPoint( pointIndex, FLAGS, ps ) )
		Vector3f pos = R * ps->get3f( pointIndex, POSITION ) + t;
		if ( first ){
			bbox = BoundingBox3f( pos );
			first = false;
		} else {
			bbox.addPoint( pos );
		}
	}
	m_octree.setBBox( bbox );

	for( card32 i = 0; i < ps->getNumEntries(); i++ ){
		ICPOctreeNode* node=&m_octree;
		Vector3f pos = R * ps->get<float, 3>(i, POSITION) + t;
		int depth=0;
		int used_child=0;
		forget(used_child);
		while(!node->isLeaf()){
			BoundingBox3f pbb=node->getBBox();
			Vector3f center=pbb.getCenter();
			int child=0;
			if(center[0]<pos[0]) child|=1;
			if(center[1]<pos[1]) child|=2;
			if(center[2]<pos[2]) child|=4;
			node=node->getChild(child);
			used_child=child;
			depth++;
		}

		// too many points in the octree node -> subdivide
		while(node->getPointIndices().size()+1>=max_points){
			BoundingBox3f pbb=node->getBBox();
			Vector3f center=pbb.getCenter();

			// create children and update bounding boxes
			node->addChildren();
			for(int c=0; c<8; c++){
				BoundingBox3f bb = node->getBBox();
				if(c & 1) bb.lowerCorner[0]=center[0];
				else      bb.upperCorner[0]=center[0];
				if(c & 2) bb.lowerCorner[1]=center[1];
				else      bb.upperCorner[1]=center[1];
				if(c & 4) bb.lowerCorner[2]=center[2];
				else      bb.upperCorner[2]=center[2];
				node->getChild( c )->setBBox( bb );
			}
			// copy indizes to corresponding subnodes
			const size_t s=node->getPointIndices().size();
			if (s == 0) continue;
			bool ptsDifferent = false;
			card32 firstIndex = node->getPointIndex( 0 );
			Vector3f firstPoint = R * ps->get<float, 3>(firstIndex, POSITION) + t; 
			for(card32 j=0; j<s; j++) {
				card32 index=node->getPointIndex( j );
				Vector3f p = R * ps->get<float, 3>(index, POSITION) + t;
				if (firstPoint != p) ptsDifferent = true;
				int child=0;
				if(center[0]<p[0]) child|=1;
				if(center[1]<p[1]) child|=2;
				if(center[2]<p[2]) child|=4;
				node->getChild(child)->addPointIndex( index );
			}

			// avoid endless recursion if all points are the same
			if (!ptsDifferent) break;
			// clear index-table
			node->clearPointIndices();
			// find corresponding node (from new children)
			int child=0;
			if(center[0]<pos[0]) child|=1;
			if(center[1]<pos[1]) child|=2;
			if(center[2]<pos[2]) child|=4;
			node=node->getChild(child);
			used_child=child;
		}
		// add index of point to node

		node->addPointIndex( i );
		x4Assert(node->getBBox().inBoundingBox(pos));
		numProcessed++;
	}
}