//---------------------------------------------------------------------------
#ifndef ICPOctreeH
#define ICPOctreeH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "BoundingBox.h"
#include "PointSet.h"
#include "Persistent.h"
#include "ICPOctreeNode.h"
#include <queue>
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {
   

class X3FOUNDATION_PRIVATE_API ICPOctree : public Persistent
{
	X4_CLASS( ICPOctree )

	private:
		static card64 m_globalAge;
		card64 m_age;
		ICPOctreeNode m_octree;
		const PointSet* m_ps;
		Matrix3f R;
		Vector3f t;

	public:
		ICPOctree(){}

		/// Build octree
		void octreesetup(
			const PointSet* ps,
			card32 max_points,
			const Matrix3f& R = IDENTITY3F,
			const Vector3f& t = NULL_VECTOR3F
		);
		inline ICPOctreeNode* findNode(const Vector3f& pos){
			if( !m_octree.getBBox().inBoundingBox(pos) ) return(NULL);
			ICPOctreeNode* node=&m_octree;
			while(!node->isLeaf()){
				for(card32 c=0; c<8; c++){
					if(node->getChild(c)->getBBox().inBoundingBox(pos)){
						node=node->getChild(c);
						break;
					}
				}
			}
			return(node);
		}
		static void goOlder(void){ m_globalAge++; };
		inline bool isOld(void){ return(m_globalAge>m_age); };

		/*** MEMBER ACCESS ***/
		inline ICPOctreeNode* getOctree(void){ return(&m_octree); };
		inline const PointSet* getPointSet(void){ return(m_ps); };
		inline void setPointSet( PointSet* ps ){ this->m_ps = ps; }
		inline void setAge( card64 age ){ this->m_age = age; }
		inline card64 getGlobalAge(){ return m_globalAge; }
		inline Matrix3f getRotation() const { return R; }
		inline Vector3f getTranslation() const { return t; }
};

}

#endif
