//---------------------------------------------------------------------------
#ifndef ICPOctreeNodeH
#define ICPOctreeNodeH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "BoundingBox.h"
#include "PointSet.h"
#include "Persistent.h"
//---------------------------------------------------------------------------
#include <queue>
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {
   

/// index means:
///     6---7
///    /   /|
///   2---3 5
///   |   |/
///   0---1

class X3FOUNDATION_PRIVATE_API ICPOctreeNode : public Persistent 
{
	X4_CLASS( ICPOctreeNode )

	private:
		ICPOctreeNode* m_children;
		std::vector<card32> pointIndices;
		BoundingBox3f m_bb;
		int32 m_index;					// Child index from parent perspective
		ICPOctreeNode* m_parent;		// Parent node

	public:
		ICPOctreeNode();
		~ICPOctreeNode();

		void addChildren();
		void deleteChildren();
		ICPOctreeNode* getChild(const int index) const;
		inline bool isLeaf(void) const { return(m_children==NULL); }

		/*** MEMBER ACCESS ***/
		inline int32 getIndex() const { return m_index; }
		inline ICPOctreeNode* getParent() const { return m_parent; }
		inline BoundingBox3f getBBox(){ return m_bb; }
		inline void setBBox( const BoundingBox3f& bbox ){ m_bb = bbox; }
		inline std::vector<card32> getPointIndices() { return pointIndices; }
		inline card32 getPointIndex( const card32 index ){ return pointIndices[index]; }
		inline void addPointIndex( const card32 pointIndex ){ pointIndices.push_back( pointIndex ); } 
		inline void clearPointIndices(){ pointIndices.clear(); }
};

}

#endif
