#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "InteractiveHandleLength.h"
//---------------------------------------------------------------------------
#include "DebugRenderer.h"
//---------------------------------------------------------------------------
using namespace X4;


InteractiveHandleLength::InteractiveHandleLength(const Vector3f &p0, const Vector3f p1, std::string name)
{
	setName(name);
	setup(p0,p1);
}

InteractiveHandleLength::~InteractiveHandleLength(void)
{
}

void X4::InteractiveHandleLength::setup( const Vector3f &p0, const Vector3f p1 )
{
	Vector3f dir = p1 - p0;
	Matrix3f frame = calcTangentSystem(dir);
	m_Frame = expand3To4(frame);
	m_Length = norm(dir);
	resetSourcePoint(p0);
}

void X4::InteractiveHandleLength::draw( HandleSetInformation * info, bool selected )
{
	Vector3f p0 = transformVector3f(m_Frame,NULL_VECTOR3F);
	Vector3f p1 = transformVector3f(m_Frame,makeVector3f(0,0,m_Length));
	Vector3f middle = (p0+p1)/2;
	float handlesize = computeHandleSize_worldSpace(middle,info);
	float rad = min(handlesize,norm(p1-p0)*0.2f);
	Vector3f color0 = getColor() * 0.8f * (selected?1.2f:1);
	Vector3f color1 = getColor() * 1.0f * (selected?1.2f:1);
	Vector3f tmp_p0 = p0 + shrink4To3(m_Frame[2])*rad;
	Vector3f tmp_p1 = p1 - shrink4To3(m_Frame[2])*rad;
	debugRenderer->addFastCylinder(tmp_p0,tmp_p1,rad*0.5f,color0,false,true);
	debugRenderer->addFastCone(tmp_p0,-shrink4To3(m_Frame[2]),rad,rad*1.25f,color1,true);
	debugRenderer->addFastCone(tmp_p1,shrink4To3(m_Frame[2]),rad,rad*1.25f,color1,true);
}

bool X4::InteractiveHandleLength::selectHandle( float x, float y, const Vector3f &worldSpacePoint,HandleSetInformation * info, bool isSelected )
{
	float r = computeHandleSize_worldSpace(worldSpacePoint,info)*1.5f;
	Matrix4f invFrame = invertFrame(m_Frame);
	Vector3f tmp = transformVector3f(invFrame,worldSpacePoint);
	m_MinSize = computeHandleSize_worldSpace(worldSpacePoint,info)*1.5f;
	if( sqr(tmp[0])+sqr(tmp[1]) < sqr(r) )
	{
		if( tmp[2]>=-r && tmp[2] <= r )
		{
			m_Point0Selected=true;
			return true;
		}
		if( tmp[2]>=m_Length-r && tmp[2] <= m_Length+r)
		{
			m_Point0Selected=false;
			return true;
		}
	}
	return false;
}

void X4::InteractiveHandleLength::moveHandle( float dx, float dy, const Vector3f &worldSpacePointInc,HandleSetInformation * info )
{
	float inc = shrink4To3(m_Frame[2]) * worldSpacePointInc;
	if( m_Point0Selected)
	{
		Vector3f p1 = transformVector3f(m_Frame,makeVector3f(0,0,m_Length));
		m_Length = max(m_MinSize,m_Length-inc);
		m_Frame[3] = expand3To4(p1) - m_Frame[2] * m_Length;
	}
	else
	{
		m_Length = max(m_MinSize,m_Length+inc);
	}
}

void X4::InteractiveHandleLength::hover( float x, float y, const Vector3f &worldSpacePoint,HandleSetInformation * info )
{

}
