#ifndef InteractiveHandleLengthH
#define InteractiveHandleLengthH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "InteractiveHandleSet.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	controls a single float parameter (distance between two points)
*/
class X3FOUNDATION_PRIVATE_API InteractiveHandleLength : public InteractiveHandle
{

public:
	InteractiveHandleLength(const Vector3f &p0, const Vector3f p1, std::string name);
	~InteractiveHandleLength(void);

	virtual void draw(HandleSetInformation * info, bool selected);

	virtual bool selectHandle(float x, float y, const Vector3f &worldSpacePoint,HandleSetInformation * info, bool isSelected);
	virtual void moveHandle(float dx, float dy, const Vector3f &worldSpacePointInc,HandleSetInformation * info);
	virtual void hover(float x, float y, const Vector3f &worldSpacePoint,HandleSetInformation * info);


	void setup(const Vector3f &p0, const Vector3f p1);
	void setup(const Matrix4f &frame, float length) {m_Frame = frame; m_Length = length;}
	void resetSourcePoint(const Vector3f &point) {m_Frame[3] = expand3To4(point);}

	float32 getLength() const {return m_Length;}
	void setLength(float32 v) {m_Length=v;}

private:

	float32		m_Length;
	Matrix4f	m_Frame;	// z-axis defines length
	
	bool		m_Point0Selected;
	float32		m_MinSize;

};

}

#endif