#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "IntrinsicWaveDescriptor.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "HierarchicalKNNIterator.h"
#include "UnstructuredInCorePointCloud.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "PCIMeshGeodesics.h"

#include "X4Exception.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( IntrinsicWaveDescriptor ,0)
{
	BEGIN_CLASS_INIT( IntrinsicWaveDescriptor );
}

IntrinsicWaveDescriptor::IntrinsicWaveDescriptor(void)
{
	knnIt = NULL;
}

IntrinsicWaveDescriptor::~IntrinsicWaveDescriptor(void)
{
}

void IntrinsicWaveDescriptor::assign(const Object* obj, X4CopyContext *context)
{
	LocalDescriptor::assign(obj,context);
	const IntrinsicWaveDescriptor * o = dynamic_cast<const IntrinsicWaveDescriptor*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::IntrinsicWaveDescriptor::beginSignatureComputations( UnstructuredInCorePointCloud * upc )
{
	tri = dynamic_cast<UnstructuredInCoreTriangleMesh*>(upc);
	if (!tri)
	{
		throw X4Exception("IntrinsicWaveDescriptor: works with mesh only (up to now)");
	}
	knnIt = new HierarchicalKNNIterator( upc, 32, NULL);
}

LocalSignature * X4::IntrinsicWaveDescriptor::computeSignature( const Vector3f &point, const Vector3f &normal, float size, UnstructuredInCorePointCloud * upc )
{
	knnIt->setSeekPointAndReset(point);
	int32 meshVertexInd = knnIt->getCurrentPointIndex();

	PCIMeshGeodesics geod;
	DVectorF featureVec = geod.computeIntrinsicWaveSequenceDescriptorFast(tri, meshVertexInd, size);
	FloatVectorSignature* sig = new FloatVectorSignature;
	sig->setVector(featureVec);
	return sig;

}


DVectorF X4::IntrinsicWaveDescriptor::getWaveSignature( mpcard pointIndex, float size, UnstructuredInCoreTriangleMesh * tri )
{
	DVectorF featureVec = PCIMeshGeodesics::computeIntrinsicWaveSequenceDescriptorFast(tri, pointIndex, size);
	return featureVec;
}


void X4::IntrinsicWaveDescriptor::endSignatureComputations()
{
	delete knnIt;
	knnIt = NULL;
}


std::vector<DVectorF> X4::IntrinsicWaveDescriptor::getWaveSignaturesForAll( float size, UnstructuredInCoreTriangleMesh * upc, const std::vector<card32>& selected)
{
	return PCIMeshGeodesics::computeIntrinsicWaveSequenceDescriptorForAll(upc, size, selected);
}