//---------------------------------------------------------------------------
#ifndef InverseDistanceWeightingH
#define InverseDistanceWeightingH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include <DynamicLinearAlgebra.h>

namespace X4
{

   class X3FOUNDATION_PRIVATE_API InverseDistanceWeighting
   {

   public:

      InverseDistanceWeighting(unsigned dim);
      virtual ~InverseDistanceWeighting();

      /**
       * add a sample point which is used for later interpolation
       * @param sample Sample which will be used for interpolatioon
       * @param distance distance of the sample to the interpolated point
       **/
      inline void addSample(DVectorF sample, float distance)
      {
         if (sample.getDim() == m_dim)
            m_samples.push_back(std::pair<DVectorF,float>(sample, distance));
         else
            throw EMath("InverseDistanceWeighting: parameter with wrong dimensions given");
      }
      
      /**
       * Use the added points to interpolate them. Return result of the interpolation.
       **/
      virtual DVectorF interpolateUseGauss(float sigma);
      
   private:
      typedef std::vector<std::pair<DVectorF, float> > SampleList;
      unsigned m_dim;
      
      SampleList m_samples;
      
   };

};

#endif
