#ifndef X4JoingHistogram_H
#define X4JoingHistogram_H
//--------------------------------------------------------------------------
#include "x3foundation_private.h"
//--------------------------------------------------------------------------
#include <ParametricInCorePointCloud.h>

//--------------------------------------------------------------------------
namespace X4
{
    /**
    * joint histogram class definition. used to compute entropy and NMI.
    * @see http://www.cs.cornell.edu/~rdz/Papers/PZ-JMS99.pdf
    * or search for: "Comparing Images Using Joint Histograms"
    *
    * This is an implementation joining only 2 histogram
    **/
    class X3FOUNDATION_PRIVATE_API JointHistogram : public Persistent
    {
        X4_CLASS( JointHistogram );

    private:
        card32 m_count;
        float32 m_min[2];
        float32 m_max[2];
        card32 m_binSize[2];
        std::vector<float32> m_jointHist;
        std::vector<float32> m_hist;

        IMPLEMENT_VAR_ARRAY_METHODS_STL(m_jointHist);
        IMPLEMENT_VAR_ARRAY_METHODS_STL(m_hist);
    public:
        /// create histogram, specify the number of bins used
        JointHistogram();

        /// set number of bins, clear also all previous data
        void setup(card32 binSize1, card32 binSize2);

        /// destruct 
        virtual ~JointHistogram();

        /// add measurements (value and bin index)
        void add(float32 value1, float32 value2);

        /// setup maximas to define the borders of the bins
        void setMinMax(float min, float max, card32 binIndex);

        /// distance operator, based on center distance
        float32 computeNMI();

        /// compute joint entropy of all bins
        float32 computeJointEntropy();

        /// compute entropy in the bin
        float32 computeEntropy(card32 binIndex);
    };

}
#endif