#ifndef LBaseH
#define LBaseH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{
class LineFeature;
class AnnSearch;
/**
	defines a pair of non-congruent line features
*/
class X3FOUNDATION_PRIVATE_API LBase : public Persistent
{
	X4_CLASS( LBase )

private:

	LineFeature * m_F1;
	LineFeature * m_F2;
	card32	m_Index1;
	card32	m_Index2;
	Vector3f	m_BasePoint;
	
	enum EBaseType
	{
		BT_NoRotation = 0,
		BT_F1Rotates = 1,
		BT_F2Rotates = 2
	};

	card32	m_BaseType;

	Matrix4f computeTransformationForRotationalBases(LineFeature * fLine, LineFeature * fRot,
		LineFeature * fLine2, LineFeature * fRot2, Vector3f basePoint2);

	// temporal list of transformations
	std::vector<unsigned> m_TransformationList;

public:
	LBase(void);
	~LBase(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	/// returns true if both bases are similar
	bool isSimilar( LBase * b, float maxSurfaceCurvDiff, float maxSlippageCurvatureDiff, float maxAngleDiff );

	/// returns signature std::vector
	DVectorF getSignature();

	/// setup - creates copies of both line features
	void setup( LineFeature * f1, LineFeature * f2, card32 index1, card32 index2, float rotThres);

	/// computes transformation from this base to base b
	Matrix4f computeTransformation( LBase * b );

	/// get coordinate frame (x aligned with f1)
	Matrix4f getCoordinateFrame() const;

	/// visualize base with debug renderer
	void drawWithDebugRenderer(const Vector3f &color = makeVector3f(1,1,1), float radius = 0.05f, float length = -1.f);

	/// visualize base as coordinate frame with debug renderer
	void drawFrameWithDebugRenderer();

	/// compute base point
	Vector3f getBasePoint() {return m_BasePoint;}
	void setBasePoint(const Vector3f &v) {m_BasePoint = v;}

	LineFeature * getF1() {return m_F1;}
	LineFeature * getF2() {return m_F2;}
	card32 getIndex1() {return m_Index1;}
	card32 getIndex2() {return m_Index2;}
	bool baseContainsRotation() {return m_BaseType != BT_NoRotation;}

	/// did we already tested this transformation (successfully)
	bool searchTestedTransformation( const Matrix4f transf, std::vector<Matrix4f> &transformations, float translationTolerance);

	/// add successful transformation to list
	void addTestedTransformation( unsigned transformationIndex ) {m_TransformationList.push_back(transformationIndex);}

	/// clear list of transformations
	void clearTestedTransformations() {m_TransformationList.clear();}

	/// visualize transformations
	void drawTransformationsWithDR(std::vector<Matrix4f> &transformations);

};

class X3FOUNDATION_PRIVATE_API LBaseQuerry
{
private:
	AnnSearch *		m_ANN;
	std::vector<float>	m_Signatures;
	std::vector<float*> m_SignaturePointers;
	DVectorF			m_ScaleSignature;

	std::vector<int> m_Indices;
	int			m_NumValidIndices;

public:
	LBaseQuerry( LBase ** base_array, const card32 numBases, DVectorF scaleSignature);
	~LBaseQuerry();

	/// querries nearest neighbours in radius using ANN
	/// returns number of neighbours within the given radius and a pointer to the index array
	int neighbourQuerry(DVectorF signature, float radius, int *& indices);

	DVectorF getScaleSignature() const {return m_ScaleSignature;}
	


};

}

#endif