#ifndef LDCentroidDescriptorH
#define LDCentroidDescriptorH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "LocalDescriptor.h"
//---------------------------------------------------------------------------

namespace X4
{
class HierarchicalKNNIterator;
class LDCentroidSignature;
/**
	centroid descriptor
*/
class X3FOUNDATION_PRIVATE_API LDCentroidDescriptor : public LocalDescriptor
{
	X4_CLASS( LDCentroidDescriptor )

private:

	float32	m_SpatialSigma;
	card32	m_NumRings;
	
	std::string	m_Attribute;
	float32	m_AttributeSigma;
	card32	m_NumAttributeSamples;

	float32	m_MaxHistogramDistance;

	// ------------members----------
	HierarchicalKNNIterator		*		m_hIter;
	AAT			m_PosAAT, m_NormalAAT, m_AttributeAAT;


public:
	LDCentroidDescriptor(void);
	~LDCentroidDescriptor(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	/// setup descriptor
	virtual void beginSignatureComputations(UnstructuredInCorePointCloud * upc);

	/// compute local signature for point
	virtual LocalSignature * computeSignature(const Vector3f &point, const Vector3f &normal, float size, UnstructuredInCorePointCloud * upc);

	/// cleanup
	virtual void endSignatureComputations();

	friend LDCentroidSignature;

};

class X3FOUNDATION_PRIVATE_API LDCentroidSignature : public LocalSignature
{
	X4_CLASS(LDCentroidSignature)
public:

	virtual float compare( LocalSignature * other, LocalDescriptor * myDescriptor );
	virtual float compare( LocalSignature * other ) { return compare(other, NULL); }

	float computeHistogramDistance( LDCentroidSignature * sig ) const;
	float compareCentroidDistances( LDCentroidSignature * sig ) const;
	Matrix4f calculateTransformation(LDCentroidSignature * sourceSignature, bool usePCANormal);
	
	// draw centroids (begin a renderjob first!)
	void debug_draw();

	std::vector<Vector3f>	m_Centroids;
	std::vector<float32>	m_CentroidWeights;
	Vector3f				m_PCANormal;	// not normalized!!!

	IMPLEMENT_VAR_ARRAY_METHODS_STL(m_Centroids);
	IMPLEMENT_VAR_ARRAY_METHODS_STL(m_CentroidWeights);

};

}

#endif