#ifndef LDHistogramH
#define LDHistogramH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "LocalDescriptor.h"
//---------------------------------------------------------------------------

namespace X4
{
class HierarchicalKNNIterator;
class TopologicalRangeQuerry;
/**
	ring histogram descriptor
*/
class X3FOUNDATION_PRIVATE_API LDHistogram : public LocalDescriptor
{
	X4_CLASS( LDHistogram )

private:

	card32	m_NumBins;			// number of bins per histogram
	card32	m_NumHistograms;	// num rings per signatures
	std::string	m_Attribute;		// histogram attribute
	card32	m_AttributeCompNumber;
	float32	m_MaxValue;			// maximal attribute value (upper histogram bound)
	float32	m_MinValue;			// minimal attribute value (lower histogram bound)
	bool	m_UseTopology;		// uses topological range queries

	// non-property-members
	HierarchicalKNNIterator	*	m_hIt;
	TopologicalRangeQuerry	*	m_TopQuery;

public:
	LDHistogram(void);
	~LDHistogram(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	/// setup descriptor
	virtual void beginSignatureComputations(UnstructuredInCorePointCloud * upc);

	/// compute local signature for point
	virtual LocalSignature * computeSignature(const Vector3f &point, const Vector3f &normal, float size, UnstructuredInCorePointCloud * upc);

	/// cleanup
	virtual void endSignatureComputations();

   card32 getNumBins() const {return m_NumBins;}
   void setNumBins(card32 v) {m_NumBins=v;}

   card32 getNumHistograms() const {return m_NumHistograms;}
   void setNumHistograms(card32 v) {m_NumHistograms=v;}

   std::string getAttribute() const {return m_Attribute;}
   void setAttribute(std::string v) {m_Attribute=v;}

   card32 getAttributeCompNumber() const {return m_AttributeCompNumber;}
   void setAttributeCompNumber(card32 v) {m_AttributeCompNumber=v;}

   float32 getMaxValue() const {return m_MaxValue;}
   void setMaxValue(float32 v) {m_MaxValue=v;}

   float32 getMinValue() const {return m_MinValue;}
   void setMinValue(float32 v) {m_MinValue=v;}

   bool getUseTopology() const {return m_UseTopology;}
   void setUseTopology(bool v) {m_UseTopology=v;}
	

};

}

#endif