#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "LandmarkCoordinate.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "TopologyAttachment.h"
//---------------------------------------------------------------------------
using namespace X4;

//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( LandmarkCoordinate, 0 )
{
	BEGIN_CLASS_INIT( LandmarkCoordinate );
}

//---------------------------------------------------------------------------
LandmarkCoordinate::LandmarkCoordinate()
{
	//m_CoordinateMap.set_empty_key(0xBAADFEED);
}

//---------------------------------------------------------------------------
LandmarkCoordinate::~LandmarkCoordinate()
{

}

//---------------------------------------------------------------------------
void LandmarkCoordinate::assign(const Object* obj, X4CopyContext *context )
{
	// Persistent::assign(obj,context);
	const LandmarkCoordinate * o = dynamic_cast<const LandmarkCoordinate*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj, this);

	// copy std::map
	m_CoordinateMap = o->m_CoordinateMap;
}

//---------------------------------------------------------------------------
float32 LandmarkCoordinate::get(card32 globalTrackIndex) const
{
	//return (*this)[globalTrackIndex];
	LandmarkCoordinateMap::const_iterator it = m_CoordinateMap.find(globalTrackIndex);
	if (it == m_CoordinateMap.end()) return 1e30f;
	return it->second;
}

//---------------------------------------------------------------------------
float LandmarkCoordinate::mixtureModel(const LandmarkCoordinate* obj, float sigma, float lambda, unsigned kNearestLandmarks, unsigned minLandmarksToCount)
{
//#define USE_NEW_LANDMARK_MODEL

	double invz = 1.0 / (sigma * sqrt(M_PI * 2.0));
	unsigned scored = 0;

	double sigma2 = 2.0 * sigma * sigma;
	double lambdaC = (1.0 - lambda);// * 1e-5f;

//#ifdef USE_NEW_LANDMARK_MODEL

	// collect all differences into a list
	std::vector<pair<float,float> > dists;
	for (LandmarkCoordinateMap::const_iterator it = m_CoordinateMap.begin(); it != m_CoordinateMap.end(); it++)
	{
		float aDiff = it->second;
		float bDiff = obj->get(it->first);

		if (aDiff > 1e10f || bDiff > 1e10f)
			continue;

		dists.push_back(pair<float,float>(aDiff, bDiff));
	}

	struct _SortPredicate
	{
		bool operator()(const pair<float,float>& a, const pair<float,float>& b)
		{
			return a.first < b.first;
		}
	}SortPredicate;

	// sort list to get with the smallest distance first
	if (kNearestLandmarks > 0)
		std::sort(dists.begin(), dists.end(), SortPredicate);
	else
		kNearestLandmarks = (unsigned)dists.size();

	float exponentMult = invz / float(min<unsigned>((unsigned)dists.size(), kNearestLandmarks));//log(invz * lambda);

	// compute score from at least 5 scores
	for (unsigned i=0; i < dists.size() && i < kNearestLandmarks; i++)
	{
		float diff = abs(dists[i].first - dists[i].second);
		double e = lambda * exp(- double(diff * diff / sigma2));

		//exponentMult += log( e + lambdaC);
		exponentMult *= (e + lambdaC);
		//exponentMult +=  (diff * diff) / sigma2 ;
		scored ++;
	}
/*
#else
	// compare landmark coordinates on this side (A) and on the other side (B)
	for (LandmarkCoordinateMap::const_iterator it = m_CoordinateMap.begin(); it != m_CoordinateMap.end(); it++)
	{
		float aDiff = it->second;
		float bDiff = obj->get(it->first);

		if (aDiff > 1e10f || bDiff > 1e10f)
		{
			//exponentMult *= lambdaC;
			continue;
		}

		float diff = abs(aDiff - bDiff);

		exponentMult *= ( invz * lambda * exp(- (diff * diff) / sigma2 ) + lambdaC );
		//exponentSum += ( diff*diff / sigma2 );

		scored++;
	}
#endif
*/

	if (scored < minLandmarksToCount) return 0;

	float res = exponentMult;
	//float res = exp(-exponentSum) / (sqrt(pow(float(2.0f * M_PI), float(scored)) * pow(sigma, float(scored)) ));

	return res;
}

//---------------------------------------------------------------------------
std::pair<unsigned,float> LandmarkCoordinate::isEqualTo(const LandmarkCoordinate* obj, float epsilon, std::vector<std::pair<unsigned,float> >* results)
{
	std::pair<unsigned,float> result(0,0.0f);

	if (results) results->clear();

	for (LandmarkCoordinateMap::const_iterator it = m_CoordinateMap.begin(); it != m_CoordinateMap.end(); it++)
	{
		float32 dist = obj->get(it->first);
		if (dist > -1)
		{
			float diff = fabs(dist - it->second);
			if (diff < epsilon)
			{
				result.first++;
				result.second += diff;
			}
			if (results) results->push_back(std::pair<unsigned,float>(it->first, diff));
		}
	}

	return result;
}

#if 0
//---------------------------------------------------------------------------
void LandmarkCoordinate::updateStability()
{
	// compute stability score of the point
	CvMat* A  = cvCreateMat(3, m_CoordinateMap.size(),CV_32FC1);
	CvMat* U  = cvCreateMat(3,3,CV_32FC1);
	CvMat* D  = cvCreateMat(3, m_CoordinateMap.size(),CV_32FC1);
	CvMat* V  = cvCreateMat( m_CoordinateMap.size(), m_CoordinateMap.size(),CV_32FC1);

	LandmarkCoordinateMap::const_iterator it = m_CoordinateMap.begin();
	for (unsigned j=0; it != m_CoordinateMap.end(); j++,it++)
	{
		cvmSet(A, 0, j, (*it)[0]);
		cvmSet(A, 1, j, m_CoordinateMap[j][1]);
		cvmSet(A, 2, j, m_CoordinateMap[j][2]);
	}

	cvSVD(A, D, U, V, CV_SVD_U_T|CV_SVD_V_T); // A = U D V^T

	// get out the d values
	float d1 = fabs(cvmGet(D, 0, 0));
	float d2 = fabs(cvmGet(D, 1, 1));
	float stability = min<float>(d1,d2) / max<float>(d1,d2);

	cvReleaseMat(&A);
	cvReleaseMat(&U);
	cvReleaseMat(&D);
	cvReleaseMat(&V);

	m_Stability = stability;

}
#endif


//---------------------------------------------------------------------------
void LandmarkCoordinate::write(OutputObjectStream *s) const
{
	Persistent::write(s);

	s->writeMaxPlatformCard(m_CoordinateMap.size());
	LandmarkCoordinateMap::const_iterator it = m_CoordinateMap.begin();
	for (; it != m_CoordinateMap.end(); it++)
	{
		float32 val = it->second;
		s->writeMaxPlatformCard(it->first);
		s->writeBuffer(&val, sizeof(float32));
	}

}

//---------------------------------------------------------------------------
void LandmarkCoordinate::read(InputObjectStream *s)
{
	Persistent::read(s);

	mpcard size = 0;
	s->readMaxPlatformCard(size);
	if (size == 0) return;

	m_CoordinateMap.clear();
	for (card32 i=0; i < size; i++)
	{
		float32 val = 0;
		mpcard index = 0;
		s->readMaxPlatformCard(index);
		s->readBuffer(&val, sizeof(float32));

		m_CoordinateMap[index] = val;
	}

}

