#ifndef LandmarkCoordinateH
#define LandmarkCoordinateH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------
#include "PointSet.inline.h"
#include <google/dense_hash_map> 

namespace X4
{

	/**
	* Landmark coordinate std::map which can be used by every data point.
	**/
	class X3FOUNDATION_PRIVATE_API  LandmarkCoordinate : public Persistent
	{
		X4_CLASS (LandmarkCoordinate)

	public:

		/// Data type
		typedef float32 MyData;
		typedef card32 MyKey;

#if 0
		/// Compare keys function
		struct MyHashEql
		{
			bool operator()(const MyKey& obj1, const MyKey& obj2) const
			{
				return obj1 == obj2;
			}
		};

		/// Hash function 
		struct MyHashFunction
		{
			size_t operator()(const MyKey& obj) const
			{
				register card32 a = obj;
				a = (a+0x7ed55d16) + (a<<12);
				a = (a^0xc761c23c) ^ (a>>19);
				a = (a+0x165667b1) + (a<<5);
				a = (a+0xd3a2646c) ^ (a<<9);
				a = (a+0xfd7046c5) + (a<<3);
				a = (a^0xb55a4f09) ^ (a>>16);
				return a;
			}
		};

		// Hash definition used in this class
		typedef google::dense_hash_map<MyKey, MyData, MyHashFunction, MyHashEql> MySparseHashTable;
		typedef MySparseHashTable LandmarkCoordinateMap;
#endif
		typedef std::map<MyKey, MyData> LandmarkCoordinateMap;

	protected:
		LandmarkCoordinateMap m_CoordinateMap;

	public:
		LandmarkCoordinate();
		virtual ~LandmarkCoordinate();

		/// check if this coordinate is equal to another one (equality is in certain epsilon range)
		/// returns a pair of first=number of landmark supported by, second=overall support error
		std::pair<unsigned,float> isEqualTo(const LandmarkCoordinate* obj, float epsilon, std::vector<std::pair<unsigned,float> >* results = NULL);
		float mixtureModel(const LandmarkCoordinate* obj, float sigma, float lambda, unsigned kNearestLandmarks = 5, unsigned minLandmarksToCount = 4);

		// get coordinate for some track (return 1e30f if not such exists)
		inline size_t size() const { return m_CoordinateMap.size(); }
		inline float32& operator[](card32 coord)
		{
			return m_CoordinateMap[coord];
		}
		float32 get(card32 globalTrackIndex) const;
		
		inline const LandmarkCoordinateMap& getMap() const { return m_CoordinateMap; }

		inline void clearCoordinateMap() { m_CoordinateMap.clear(); }
		inline void setCoordinate(const std::vector<float>& coordinates)
		{
			clearCoordinateMap();
			for (unsigned i=0; i < coordinates.size(); i++) m_CoordinateMap[i] = coordinates[i];
		}

		// --------------- Persistent --------------------------
		virtual void write(OutputObjectStream *s) const;
		virtual void read(InputObjectStream *s);
		virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	};
};

#endif 
