#ifndef LinearProgrammingSolverH
#define LinearProgrammingSolverH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	abstract base class for linear programming, quadratic programming,
	and mixed-integer programming
*/
class X3FOUNDATION_PRIVATE_API LinearProgrammingSolver
{

public:


	/// minimize:		0.5 * x^T Q x + p x + c
	/// subject to:		Ax >= b
	///					Ex = d
	///	optional:		specify integer variables (x_i is integer if integer_variables[i]=true)
	virtual bool solveQuadraticProblem(	const SparseMatrixD &Q, const DVectorD &p, double c,
										const SparseMatrixD &A, const DVectorD &b,
										const SparseMatrixD &E, const DVectorD &d, 
										DVectorD &solution, bool * integer_variables=NULL) {return false;}

	/// minimize:		0.5 * x^T Q x + p x + c
	/// subject to:		Ax >= b
	///					Ex = d
	///	optional:		specify integer variables (x_i is integer if integer_variables[i]=true)
	virtual bool solveQuadraticProblem(	const DMatrixD &Q, const DVectorD &p, double c,
		const DMatrixD &A, const DVectorD &b,
		const DMatrixD &E, const DVectorD &d, 
		DVectorD &solution, bool * integer_variables=NULL) {return false;}


	/// turn on/off solver output via debugOutput console
	void setReportSolverOutput(bool v) {m_ReportSolverOutput=v;}
	bool getReportSolverOutput() const {return m_ReportSolverOutput;}

protected:


	bool	m_ReportSolverOutput;


};

}

#endif