#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "LocalDescriptor.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_ABSTRACT_CLASS( LocalDescriptor ,0)
{
	BEGIN_CLASS_INIT( LocalDescriptor );
	ADD_FLOAT32_PROP(m_DifferenceScaleFactor,0)
}

void LocalDescriptor::assign(const Object* obj, X4CopyContext *context)
{
	const LocalDescriptor * o = dynamic_cast<const LocalDescriptor*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

IMPLEMENT_X4_ABSTRACT_CLASS( LocalSignature ,0)
{
	BEGIN_CLASS_INIT( LocalSignature );
}

void LocalSignature::assign(const Object* obj, X4CopyContext *context)
{
	const LocalSignature * o = dynamic_cast<const LocalSignature*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

IMPLEMENT_X4_CLASS( FloatVectorSignature ,0)
{
	BEGIN_CLASS_INIT( LocalSignature );

	ADD_FLOAT32_VAR_ARRAY_PROP( Data, 0);
}

void FloatVectorSignature::assign(const Object* obj, X4CopyContext *context)
{
	const FloatVectorSignature * o = dynamic_cast<const FloatVectorSignature*>(obj);
	x4Assert( o != NULL );

	m_Data = o->m_Data;
}

float FloatVectorSignature::compare( LocalSignature * other, LocalDescriptor * myDescriptor )
{
	const FloatVectorSignature * o = dynamic_cast<const FloatVectorSignature*>(other);
	return norm(m_Data - o->m_Data) * (myDescriptor ? myDescriptor->getDifferenceScaleFactor() : 1.0);
}


void X4::FloatVectorSignature::deleteData( card32 index )
{
	DVectorF tmp = m_Data;
	m_Data.setDim(tmp.getDim()-1);
	for( unsigned i=0;i<index;i++ )
		m_Data[i] = tmp[i];
	for( unsigned i=index+1;i<tmp.getDim();i++ )
		m_Data[i-1] = tmp[i];
}

float X4::FloatVectorSignature::compare( FloatVectorSignature * other )
{
	return norm(m_Data - other->m_Data) *15.0f;
}

std::string X4::FloatVectorSignature::printToString() const
{
	std::string ret;
	for( unsigned i=0;i<m_Data.getDim();i++ )
	{
		if( i>0 )
			ret += " ";
		ret += X4::floatToStr(m_Data[i]);
	}
	return ret;
}