#include "stdafx.h"
//---------------------------------------------------------------------------
#include "MLSSmooth.h"
#include "DebugOutput.h"
#include "HierarchicalKNNIterator.h"
#include "ProgressWindow.h"
#include "PointSetKNNQuery.h"
#include "PointSet.inline.h"
#include <time.h>
#include "NumericalClassProperty.h"
#include "MathTools.h"
#include "FifoStack.h"
#include "PointSetKNNQuery.h"
#include "IndexGraph.h"
#include "PriorityQueue.h"
#include "PCA.h"
#include "LocalSurfaceFunction.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

void MLSSmooth::smooth( PointSet* ps, const float32 h )
{
	if ( ps == NULL ){
		debugOutput << "smooth: invalid point set.\n"; debugOutput.flush();
		return;
	}

	if ( !ps->providesAttribute( "position" ) || !ps->providesAttribute( "coeffs" ) ||
		!ps->providesAttribute( "normal" ) || !ps->providesAttribute( "tangent_u" ) ||
		!ps->providesAttribute( "tangent_v" ) || !ps->providesAttribute( "polyScale" ) ){
		debugOutput << "smooth: channel missing.\n"; debugOutput.flush();
		return;
	}

	const float32 startTime = clock();

	AAT POSITION = ps->getAAT( "position" );
	AAT NORMAL = ps->getAAT( "normal" );
	AAT TANGENT_U = ps->getAAT( "tangent_u" );
	AAT TANGENT_V = ps->getAAT( "tangent_v" );
	AAT COEFFS = ps->getAAT( "coeffs" );
	AAT POLY_SCALE = ps->getAAT( "polyScale" );

	progressWindow->pushStep( true, "MLS Smooth" );
	for ( unsigned int pIndex = 0; pIndex < ps->getNumEntries(); pIndex++ ){
		const Vector3f& p = ps->get3f( pIndex, POSITION );
		const Vector3f& normal = ps->get3f( pIndex, NORMAL );
		const Vector3f& tangent_u = ps->get3f( pIndex, TANGENT_U );
		const Vector3f& tangent_v = ps->get3f( pIndex, TANGENT_V );
		const float32 polyScale = ps->get1f( pIndex, POLY_SCALE );
		Matrix3f T;
		T[0] = tangent_u;
		T[1] = tangent_v;
		T[2] = normal;
		StaticVector<float32,MLS_NUM_BASIS_FUNCTIONS> coeffs = ps->get<float32,MLS_NUM_BASIS_FUNCTIONS>( pIndex, COEFFS );
		Vector3f p_local = makeVector3f( 0, 0, LocalSurfaceFunction<MLS_NUM_BASIS_FUNCTIONS>::f( coeffs, 0, 0 ) );
		Vector3f pProj = T * p_local * polyScale + p;
		ps->set3f( pIndex, POSITION, pProj );
		progressWindow->progress( float32( pIndex * 100 ) / float( ps->getNumEntries( ) ) );	
	}

	debugOutput << "Time required (MLS smooth): " << float32( ( clock() - startTime ) / CLK_TCK ) << "\n"; debugOutput.flush(); 
	progressWindow->popSteps();
}

void MLSSmooth::smooth( PointCloud* pc, const float32 h )
{
	debugOutput << "smooth: not implemented for point cloud.\n"; debugOutput.flush();
	return;
}