#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ManifoldCorrespondences.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "VariableArrayClassProperty.h"
#include "ObjectClassProperty.h"
#include "ObjectListProperty.h"
#include "FixedArrayClassProperty.h"

//---------------------------------------------------------------------------
using namespace X4;

//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( ManifoldCorrespondence ,0)
{
	BEGIN_CLASS_INIT( ManifoldCorrespondence );

	ADD_OBJECT_LIST_PROP(Point, 0, ManifoldPoint::getClass());
}

//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( ManifoldCorrespondenceSet ,0)
{
	BEGIN_CLASS_INIT( ManifoldCorrespondenceSet );
	ADD_FLOAT32_PROP(m_sampleSpacing,0);
	ADD_OBJECT_LIST_PROP(Correspondence, 0, ManifoldCorrespondence::getClass());
}

//---------------------------------------------------------------------------
ManifoldCorrespondenceSet::ManifoldCorrespondenceSet()
{
	m_sampleSpacing = 0;
}

//---------------------------------------------------------------------------
ManifoldCorrespondenceSet::ManifoldCorrespondenceSet(const std::vector<ManifoldCorrespondence*>& corrs, float32 sampleSpacing)
{
	m_Correspondence = corrs;
	m_sampleSpacing = sampleSpacing;
}

//---------------------------------------------------------------------------
ManifoldCorrespondenceSet::~ManifoldCorrespondenceSet()
{
	deleteCorrespondences(0, (card32)m_Correspondence.size());
}

//---------------------------------------------------------------------------
void ManifoldCorrespondenceSet::addCorrespondenceSet(ManifoldCorrespondenceSet* mcs)
{
	if (mcs == NULL) return;
	for (unsigned i=0; i < mcs->getNumCorrespondences(); i++)
		addCorrespondence(mcs->getCorrespondence(i)->copy());
}

//---------------------------------------------------------------------------
void ManifoldCorrespondenceSet::assign(const Object* obj, X4CopyContext *context)
{
	const ManifoldCorrespondenceSet * o = dynamic_cast<const ManifoldCorrespondenceSet*>(obj);
	x4Assert( o != NULL );

	m_sampleSpacing = o->m_sampleSpacing;
	deleteCorrespondences(0, (card32)m_Correspondence.size());
	m_Correspondence.resize(o->m_Correspondence.size());

	for (size_t i=0; i < o->m_Correspondence.size(); i++)
	{
		m_Correspondence[i] = (ManifoldCorrespondence*)(o->m_Correspondence[i]->copy());
	}
}

