#ifndef AttachedManifoldCorrespondencesH
#define AttachedManifoldCorrespondencesH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "AttachedData.h"
#include "Manifold.h"

//---------------------------------------------------------------------------

namespace X4
{

	/**
	* A pair of manifold points representing a correspondence
	**/
	class X3FOUNDATION_PRIVATE_API ManifoldCorrespondence : public Persistent
	{
		X4_CLASS(ManifoldCorrespondence);
	protected:
		std::vector<ManifoldPoint*> m_Point;

	public:
		IMPLEMENT_OBJECT_LIST_METHODS_STL(Point, ManifoldPoint);

		ManifoldCorrespondence() {}
		ManifoldCorrespondence(ManifoldPoint* p1, ManifoldPoint* p2) {setCorrepondence(p1,p2);}
		virtual ~ManifoldCorrespondence() {}

		//! Set both manifold points which are in correspondence (ownership not changed)
		void setCorrepondence(ManifoldPoint* p1, ManifoldPoint* p2)
		{
			m_Point.clear();
			m_Point.push_back(p1);
			m_Point.push_back(p2);
		}

		//! get correspondnece pair
		pair<ManifoldPoint*,ManifoldPoint*> getCorrespondence() const { return pair<ManifoldPoint*,ManifoldPoint*>(m_Point[0], m_Point[1]); }
	};

	/**
	* Manifold correspondencies are represented just as a list of correspondence pairs.
	*/
	class X3FOUNDATION_PRIVATE_API ManifoldCorrespondenceSet : public AttachedData
	{
		X4_CLASS( ManifoldCorrespondenceSet )

	protected:
		std::vector<ManifoldCorrespondence*> m_Correspondence;
		float32 m_sampleSpacing;

	public:
		ManifoldCorrespondenceSet(void);
		ManifoldCorrespondenceSet(const std::vector<ManifoldCorrespondence*>& corrs, float32 sampleSpacing);
		virtual ~ManifoldCorrespondenceSet(void);

		void setSampleSpacing(float spacing) { m_sampleSpacing = spacing;}
		float getSampleSpacing() const { return m_sampleSpacing; }
		ManifoldCorrespondence* getManifoldCorrespondence(mpcard index) const { return m_Correspondence[index]; }

		virtual void assign(const Object* obj, X4CopyContext *context = NULL);

		static const std::string getDefaultName() { return "ManifoldCorrespondenceSet"; }

		void addCorrespondenceSet(ManifoldCorrespondenceSet* mcs);

		IMPLEMENT_OBJECT_LIST_METHODS_STL(Correspondence, ManifoldCorrespondence);
	};

}

#endif