#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MeshGeodesics.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "geodesic_algorithm_exact.h"
#include "geodesic_algorithm_dijkstra.h"
#include "geodesic_algorithm_subdivision.h"

#include "UnstructuredInCoreTriangleMesh.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( MeshGeodesics ,1)
{
	BEGIN_CLASS_INIT( MeshGeodesics );

	ADD_INT32_PROP(m_SubdivisionLevel, 1);
}

MeshGeodesics::MeshGeodesics(void)
{
	m_SubdivisionLevel = 0;
}

MeshGeodesics::~MeshGeodesics(void)
{
}

void MeshGeodesics::assign(const Object* obj, X4CopyContext *context)
{
	// Persistent::assign(obj,context);
	const MeshGeodesics * o = dynamic_cast<const MeshGeodesics*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::MeshGeodesics::testGeodesics()
{
// 	int argc = 2;
// 	char **argv;

	std::vector<double> points;	
	std::vector<unsigned> faces;

	bool success = geodesic::read_mesh_from_file("C:/svn/X4/trunk/xrt/private/foundation_private/meshGeodesic/flat_triangular_mesh.txt"  ,points,faces);
	if(!success)
	{
		std::cout << "something is wrong with the input file" << std::endl;
		return ;
	}

	geodesic::Mesh mesh;
	mesh.initialize_mesh_data(points, faces);		//create internal mesh data structure including edges

	geodesic::GeodesicAlgorithmExact algorithm(&mesh);	//create exact algorithm for the mesh

	unsigned source_vertex_index = 0 ;

	geodesic::SurfacePoint source(&mesh.vertices()[source_vertex_index]);		//create source 
	std::vector<geodesic::SurfacePoint> all_sources(1,source);					//in general, there could be multiple sources, but now we have only one

// 	if(argc > 3)	//target vertex specified, compute single path
// 	{
// 		unsigned target_vertex_index = atol(argv[3]);
// 		geodesic::SurfacePoint target(&mesh.vertices()[target_vertex_index]);		//create source 
// 
// 		std::vector<geodesic::SurfacePoint> path;	//geodesic path is a sequence of SurfacePoints
// 
// 		bool const lazy_people_flag = false;		//there are two ways to do exactly the same
// 		if(lazy_people_flag)
// 		{
// 			algorithm.geodesic(source, target, path); //find a single source-target path
// 		}
// 		else		//doing the same thing explicitly for educational reasons
// 		{
// 			double const distance_limit = geodesic::GEODESIC_INF;			// no limit for propagation
// 			std::vector<geodesic::SurfacePoint> stop_points(1, target);	//stop propagation when the target is covered
// 			algorithm.propagate(all_sources, distance_limit, &stop_points);	//"propagate(all_sources)" is also fine, but take more time because covers the whole mesh
// 
// 			algorithm.trace_back(target, path);		//trace back a single path 
// 		}
// 
// 		print_info_about_path(path);
// 		for(unsigned i = 0; i<path.size(); ++i)
// 		{
// 			geodesic::SurfacePoint& s = path[i];
// 			std::cout << s.x() << "\t" << s.y() << "\t" << s.z() << std::endl;
// 		}
// 	}
// 	else		//target vertex is not specified, print distances to all vertices
// 	{
		algorithm.propagate(all_sources);	//cover the whole mesh

		for(unsigned i=0; i<mesh.vertices().size(); ++i)
		{
			geodesic::SurfacePoint p(&mesh.vertices()[i]);		

			double distance;
			unsigned best_source = algorithm.best_source(p,distance);		//for a given surface point, find closets source and distance to this source

			std::cout << distance << " ";		//print geodesic distance for every vertex
		}
		std::cout << std::endl;
//	}

	return ;
}

void X4::MeshGeodesics::initializeFromUnstructuredInCoreTriangleMesh( UnstructuredInCoreTriangleMesh * mesh )
{
	std::vector<double> points;	
	std::vector<unsigned> faces;
	PointSet* ps = mesh->getPointSet();
	AAT POSITION = ps->getAAT("position");
	mpcard numVert = ps->getNumEntries();
	points.resize(numVert*3);

	for (mpcard i = 0; i < numVert; i++)
	{
		Vector3f pos = ps->get3f(i, POSITION);
		points[i*3+0] = pos [0];
		points[i*3+1] = pos [1];
		points[i*3+2] = pos [2];
	}

	PointSet* triangles = mesh->getTriangles();
	mpcard numTris = mesh->getNumTriangles();
	faces.resize(numTris*3);
	AAT IND = triangles->getAAT("index");

	for (mpcard i = 0; i< numTris; i++)
	{
		Vector3i ind = triangles->get3i(i, IND);
		faces[i*3+0] = ind [0];
		faces[i*3+1] = ind [1];
		faces[i*3+2] = ind [2];
	}

	
	myMesh.initialize_mesh_data(points, faces);		//create internal mesh data structure including edges

	/*
	geodesic::GeodesicAlgorithmExact algorithm(&myMesh);	//create exact algorithm for the mesh
	
		unsigned source_vertex_index = 0 ;
	
		geodesic::SurfacePoint source(&myMesh.vertices()[source_vertex_index]);		//create source 
		std::vector<geodesic::SurfacePoint> all_sources(1,source);					//in general, there could be multiple sources, but now we have only one
	
		algorithm.propagate(all_sources);	//cover the whole mesh
	
		for(unsigned i=0; i<myMesh.vertices().size(); ++i)
		{
			geodesic::SurfacePoint p(&myMesh.vertices()[i]);		
	
			double distance;
			unsigned best_source = algorithm.best_source(p,distance);		//for a given surface point, find closets source and distance to this source
	
			std::cout << distance << " ";		//print geodesic distance for every vertex
		}
		std::cout << std::endl;
	*/
	
}


std::vector<X4::float32> X4::MeshGeodesics::getGeodesicDistances(mpcard pointA, const std::vector<mpcard>& pointB, std::vector<std::vector<Vector3f> >* path3f)
{
	// source point
	std::vector<geodesic::SurfacePoint> all_sources(1, &myMesh.vertices()[pointA]);

	// all target points
	std::vector<geodesic::SurfacePoint> all_targets(pointB.size());
	for (size_t i=0; i < pointB.size(); i++)
		all_targets[i] = (&myMesh.vertices()[pointB[i]]);

	// compute geodesics
	std::vector<std::vector<geodesic::SurfacePoint> > paths;
	geodesic::GeodesicAlgorithmBase* algorithm = NULL;
	if (m_SubdivisionLevel < 0)
		algorithm = new geodesic::GeodesicAlgorithmExact(&myMesh);
	else if (m_SubdivisionLevel == 0)
		algorithm = new geodesic::GeodesicAlgorithmDijkstra(&myMesh);
	else 
		algorithm = new geodesic::GeodesicAlgorithmSubdivision(&myMesh, m_SubdivisionLevel);

	algorithm->geodesic(all_sources, all_targets, paths);

	// copy paths
	if (path3f)
	{
		path3f->clear();
		path3f->resize(pointB.size());

		for (size_t p=0; p < path3f->size(); p++)
		{
			for(unsigned i = 0; i < paths[p].size(); ++i)
			{
				geodesic::SurfacePoint& s = paths[p][i];
				Vector3f pos = makeVector3f(s.x(), s.y(), s.z());
				path3f->at(p).push_back(pos);
			}	
		}
	}

	// set distances
	std::vector<float32> dist(pointB.size(), 1e30f);
	for (size_t i=0; i < pointB.size(); i++)
	{
		dist[i] = length(paths[i]);
/*
		debugOutput << pointA << "->" << pointB[i] << " number of the points in the path = " << paths[i].size()
			<< ", length of the path = " << length(paths[i]) 
			<< "\n";
*/

	}

	delete algorithm;

	return dist;
}

X4::float32 X4::MeshGeodesics::getGeodesicDistance( mpcard pointA, mpcard pointB, std::vector<Vector3f>* path3f )
{
	std::vector<std::vector<Vector3f> > paths/*(1, NULL)*/;

	std::vector<float> dist = getGeodesicDistances(pointA, std::vector<mpcard>(1,pointB), path3f!=NULL ? &paths : NULL);
	if (path3f)
	{
		*path3f = paths[0];
	}

	return dist[0];
#if 0
	unsigned target_vertex_index = pointB;
	geodesic::SurfacePoint target(&myMesh.vertices()[target_vertex_index]);		//create source 
	unsigned source_vertex_index = pointA;
	geodesic::SurfacePoint source(&myMesh.vertices()[source_vertex_index]);
	std::vector<geodesic::SurfacePoint> all_sources(1,source);					//in general, there could be multiple sources, but now we have only one

	std::vector<geodesic::SurfacePoint> path;	//geodesic path is a sequence of SurfacePoints
	geodesic::GeodesicAlgorithmExact algorithm(&myMesh);	//create exact algorithm for the mesh


	bool const lazy_people_flag = false;		//there are two ways to do exactly the same
	if(lazy_people_flag)
	{
		algorithm.geodesic(source, target, path); //find a single source-target path
	}
	else		//doing the same thing explicitly for educational reasons
	{
		double const distance_limit = geodesic::GEODESIC_INF;			// no limit for propagation
		std::vector<geodesic::SurfacePoint> stop_points(1, target);	//stop propagation when the target is covered
		algorithm.propagate(all_sources, distance_limit, &stop_points);	//"propagate(all_sources)" is also fine, but take more time because covers the whole mesh

		algorithm.trace_back(target, path);		//trace back a single path 
	}

	//print_info_about_path(path);
	if (path3f)
	{
		for(unsigned i = 0; i<path.size(); ++i)
		{
			geodesic::SurfacePoint& s = path[i];
			//std::cout << s.x() << "\t" << s.y() << "\t" << s.z() << std::endl;
			Vector3f p = makeVector3f(s.x(), s.y(), s.z());
			path3f->push_back(p);
		}	
	}
	return length(path);
#endif 
}