//---------------------------------------------------------------------------
#ifndef MiniBallH
#define MiniBallH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace X4 {


template <class FloatType>
class MiniBall {
 public:
   StaticVector<FloatType, 3> center;
   FloatType  radius;

	MiniBall<FloatType>();
	MiniBall<FloatType>(const MiniBall<FloatType> &X);
	
	// Center and radius (not squared)
	MiniBall<FloatType>(
		const StaticVector<FloatType, 3> &O, 
		FloatType R);   
	
	// Sphere through two points
	MiniBall<FloatType>(
		const StaticVector<FloatType, 3> &O, 
		const StaticVector<FloatType, 3> &A,
		const FloatType radiusEpsilon);  

	// Sphere through three points
	MiniBall<FloatType>(
		const StaticVector<FloatType, 3> &O, 
		const StaticVector<FloatType, 3> &A,
		const StaticVector<FloatType, 3> &B,
		const FloatType radiusEpsilon); 

	// Sphere through four points
	MiniBall<FloatType>(
		const StaticVector<FloatType, 3> &O,
		const StaticVector<FloatType, 3> &A, 
		const StaticVector<FloatType, 3> &B,
		const StaticVector<FloatType, 3> &C,
		const FloatType radiusEpsilon);   

	MiniBall<FloatType> &operator=(const MiniBall &ball);

	// Distance from p to boundary of the Sphere
	FloatType d(const StaticVector<FloatType, 3> &position) const;  
	// Square distance from p to boundary of the Sphere
	FloatType d2(const StaticVector<FloatType, 3> &position) const;  
	
	// Distance from p to boundary of the Sphere
	static FloatType d(const MiniBall &ball, const StaticVector<FloatType, 3> &positon); 
	// Distance from p to boundary of the Sphere
	static FloatType d(const StaticVector<FloatType, 3> &position, const MiniBall &ball);  

	// Square distance from p to boundary of the Sphere
	static FloatType d2(const MiniBall &ball, const StaticVector<FloatType, 3> &position);  
	// Square distance from p to boundary of the Sphere
	static FloatType d2(const StaticVector<FloatType, 3> &position, const MiniBall &ball);  

	// Smallest enclosing sphere
	static MiniBall<FloatType> miniBall(
		StaticVector<FloatType, 3> Positions[], 
		card32 numOfPointsInside, 
		const FloatType radiusEpsilon);  
	
	// Enclosing sphere approximation
	static MiniBall<FloatType> smallBall(
		StaticVector<FloatType, 3> Positions[], 
		card32 numOfPointsInside, 
		const FloatType radiusEpsilon);  

	// Smallest enclosing sphere
	static MiniBall<FloatType> miniBall(
		std::vector<StaticVector<FloatType, 3> > *positions, 
		card32 numOfPointsInside, 
		const FloatType radiusEpsilon);  

	// Enclosing sphere approximation
	static MiniBall<FloatType> smallBall(
		std::vector<StaticVector<FloatType, 3> > *positions, 
		card32 numOfPointsInside, 
		const FloatType radiusEpsilon);  

private:
	static MiniBall<FloatType> recurseMini(
		StaticVector<FloatType, 3>  *Positions[], 
		card32 numOfPointsInside, 
		card32 numOfPointsOnBoundary = 0,
		const FloatType radiusEpsilon = 1e-4f);
};

}

#include "MiniBall.inline.h"

#endif