//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ClassMethods.h"
#include "Monomial2D.h"
#include "MathTools.h"
#include "PointSet.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

Monomial2D::Monomial2D()
{
}

Monomial2D::~Monomial2D()
{
}

void Monomial2D::computeCoeffs(
	const std::vector<Vector2f> &points, 
	StaticVector<float32, 3> &coeffs, 
	float32 gaussianWindowVariance )
{
	if ( points.size() < MIN_NUM_POINTS ){
		coeffs[0] = coeffs[1] = coeffs[2] = 0;
		return;
	}
	const size_t NUM_NEIGHBORS = points.size();
	StaticVector<float32,NUM_BASIS_FUNS> cd;
	StaticMatrix<float32,NUM_BASIS_FUNS,NUM_BASIS_FUNS> dm;
	for (card32 i=0; i<NUM_BASIS_FUNS;i++) {
		coeffs[i] = 0.0f;
		for (card32 j=0; j<NUM_BASIS_FUNS;j++) {
			dm[i][j] = 0.0f;
		}
	}

	for (card32 pIndex=0; pIndex<NUM_NEIGHBORS; pIndex++){
		const float32 u = points[pIndex][0];
		const float32 n = points[pIndex][1];
		const float32 w = ( (gaussianWindowVariance>0) ? sqr(exp(-(u*u)/gaussianWindowVariance)) : 1) / (float32)NUM_NEIGHBORS;
		// c[j] += b[j](u(i)) * n(i) * w(u(i));
		cd[0] =       w;
		cd[1] = u   * w;
		cd[2] = u*u * w;
		coeffs[0] += cd[0] * n;
		coeffs[1] += cd[1] * n;
		coeffs[2] += cd[2] * n;
		// 1 u uu
		dm[0][0] += w;     dm[0][1] += u*w;     dm[0][2] += u*u*w;     
		dm[1][0] += u*w;   dm[1][1] += u*u*w;   dm[1][2] += u*u*u*w;   
		dm[2][0] += u*u*w; dm[2][1] += u*u*u*w; dm[2][2] += u*u*u*u*w; 
	}

	StaticMatrix<float32,NUM_BASIS_FUNS,NUM_BASIS_FUNS> invDm;
	try {
		invDm = invertMatrix( dm );
	} catch (...) {
		for (card32 i=0; i<NUM_BASIS_FUNS; i++) {
			for (card32 j=0; j<NUM_BASIS_FUNS; j++) {
				invDm[i][j] = 0.0f;
			}
		}
	}
	coeffs = invDm * coeffs;
}

float32 Monomial2D::evaluate( const StaticVector<float32,3> &coeffs, const float32& u )
{
	return coeffs[0] + coeffs[1] * u + coeffs[2] * u * u;
}