#include "stdafx.h"
//---------------------------------------------------------------------------
#include "MonomialFitter.h"
#include "NumericalClassProperty.h"
#include "DebugOutput.h"
#include "HierarchicalKNNIterator.h"
#include "ProgressWindow.h"
#include "PointSetKNNQuery.h"
#include "PointSet.inline.h"
#include <time.h>
#include "HierarchicalRangeQueryIterator.h"
#include "MathTools.h"
#include "Monomials2ndOrder.h"
#include "LocalSurfaceFunction.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


void MonomialFitter::fitMonomial( PointSet* ps, const float32 epsilon )
{
	if ( ps == NULL ){
		debugOutput << "fitMonomial: invalid point set\n"; debugOutput.flush();
		return;
	}

	if ( !ps->providesAttribute( "position" ) || !ps->providesAttribute( "normal" ) ||
		!ps->providesAttribute( "tangent_u" ) || !ps->providesAttribute( "tangent_v" ) ||
		!ps->providesAttribute( "coeffs" ) || !ps->providesAttribute( "polyScale" ) ){
		debugOutput << "fitMonomial: channel missing\n"; debugOutput.flush();
		return;
	}

	AAT POSITION = ps->getAAT( "position" );
	AAT NORMAL = ps->getAAT( "normal" );
	AAT TANGENT_U = ps->getAAT( "tangent_u" );
	AAT TANGENT_V = ps->getAAT( "tangent_v" );
	AAT COEFFS = ps->getAAT( "coeffs" );
	AAT POLY_SCALE = ps->getAAT( "polyScale" );

	HierarchicalRangeQueryIterator* rangeIt = NULL;
	AttachedIndexedOctree* octree = PointSetKNNQuery::createOctree( ps, 32 );
	PointSetKNNQuery knn( ps, octree, makeVector3f( 0,0,0 ), POSITION );

	progressWindow->pushStep( true, "Curvatures" );
	float32 start = clock();

	float32 scale = 1.0f;
	float32 sqrEpsilon = sqr( epsilon );
	StaticVector<float32,6> coeffs;
	std::vector<Vector3f> points;

	for ( unsigned int pointIndex = 0; pointIndex < ps->getNumEntries(); pointIndex++ ){
		const Vector3f& p = ps->get3f( pointIndex, POSITION );
		scale = epsilon * 2.0f;
		points.clear();

		knn.setTargetPoint( p );
		float32 sqrD = 0;
		card32 nIndex = pointIndex;
		bool found = true;
		while( ( sqrD < sqrEpsilon ) && found ){
			const Vector3f np = ps->get3f( nIndex, POSITION );
			sqrD = ( p - np ) * ( p - np );
			points.push_back( np );
			knn.getNextNearestPointIndex( nIndex, found );
		}

		Matrix3f T;
		T[0] = ps->get3f( pointIndex, TANGENT_U );
		T[1] = ps->get3f( pointIndex, TANGENT_V );
		T[2] = ps->get3f( pointIndex, NORMAL );
		T = T.transpose();
		for ( unsigned int n = 0; n < points.size(); n++ ) points[n] = ( T * ( points[n] - p ) ) / scale;
		Monomials2ndOrder::computeCoeffs( points, coeffs );
		ps->set<float32,6>( pointIndex, COEFFS, coeffs );
		ps->set1f( pointIndex, POLY_SCALE, scale );

		progressWindow->progress( card32( float( pointIndex ) * 100 / float32( ps->getNumEntries() ) ) );
	}

	if ( rangeIt != NULL ) delete rangeIt;

	float32 time = float32( ( clock() - start ) / CLK_TCK );
	progressWindow->popSteps(  );
	delete octree;

	//debugOutput << "time required: " << time << ".\n"; debugOutput.flush();
}

void MonomialFitter::fitMonomial( PointCloud* pc, const float32 epsilon )
{
	debugOutput << "not implemented yet\n"; debugOutput.flush();
}