#ifndef Monomials3rdOrderH
#define Monomials3rdOrderH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	computes least squares fit of 3rd order 1d function to a set of data points
	polynomial: f(x) = c0 + c1 x + c2 x^2 + c3 x^3
*/
class X3FOUNDATION_PRIVATE_API Monomials3rdOrder1D
{

public:

	/// computes least squares fit of 3rd order 1d function to a set of data points
	/// y components are ignored
	static Vector4f computeCoeffs( Vector3f * dataPoints , float * weights, const unsigned numPoints);

	/// evaluates polynomial at point x
	static float evalate(const Vector4f & coeffs, float x);

	/// compute curvature maximum/minimum (returns true if polynomial has max/min)
	static bool computeCurvatureMinMax(const Vector4f &coeffs, float &result);

	/// evaluate first derivate at x
	static float evaluateD1( const Vector4f & coeffs, float x);

	/// evaluate second derivate at x
	static float evaluateD2( const Vector4f & coeffs, float x);

	/// evalute curvature at x
	static float evaluateCurvature( const Vector4f & coeffs, float x);


};

/**
computes least squares fit of 4th order 1d function to a set of data points
polynomial: f(x) = c0 + c1 x + c2 x^2 + c3 x^3 + c4 x^4
*/
class X3FOUNDATION_PRIVATE_API Monomials4thOrder1D
{

public:

	/// computes least squares fit of 3rd order 1d function to a set of data points
	/// y components are ignored
	static StaticVector<float,5> computeCoeffs( Vector3f * dataPoints , float * weights, const unsigned numPoints);

	/// evaluates polynomial at point x
	static float evalate(const StaticVector<float,5> & coeffs, float x);

	/// compute curvature maximum/minimum (returns true if polynomial has max/min)
	static bool computeCurvatureMinMax(const StaticVector<float,5> &coeffs, float &result);

};


}

#endif