#ifndef MultiResolutionPCH
#define MultiResolutionPCH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{
class PointCloud;
/**
	holds multiple resolution of a point cloud.
	position is resampled by poisson disk sampler.
*/
class X3FOUNDATION_PRIVATE_API MultiResolutionPC : public Persistent
{
	X4_CLASS( MultiResolutionPC )

private:

	struct TPointCloud
	{
		PointCloud		*pc;
		float32			averagePointDistance;
	};

	std::vector<TPointCloud>	m_PointClouds;


public:
	MultiResolutionPC(void);
	~MultiResolutionPC(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	/// creates multi resolution point clouds. stops when desired minNumberOfPoints
	/// was reached. resolution is decreased by multiplying the average point distance
	/// with distanceFactor (distanceFactor > 1)
   /// TODO:[attributes are filtered with a gaussian kernel with h = sigma * averagePointDistance^2]
	void setup( PointCloud * pc, card32 minNumberOfPoints, float distanceFactor=2.0f,
		float sigma = 1.0f);

	mpcard getNumberOfPointClouds() const {return m_PointClouds.size();}

	PointCloud * getPointCloud( card32 i );

	/// returns average point distance from point cloud i
	float getPCAveragePointDistance( card32 i );

	/// returns closest point cloud which pc.averagePointDistance < averagePointDistance
	PointCloud * getClosestPointCloud( float averagePointDistance );


};

}

#endif