#ifndef MultiScaleFeatureDetectorH
#define MultiScaleFeatureDetectorH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "FeaturePointDetector.h"
#include "LocalDescriptor.h"
//---------------------------------------------------------------------------

namespace X4
{
class UnstructuredInCoreTriangleMesh;
///
///	detects geometric features using extreme points in scale-space differences
///
class X3FOUNDATION_PRIVATE_API MultiScaleFeatureDetector : public FeaturePointDetector
{
	X4_CLASS( MultiScaleFeatureDetector )

private:

	// properties
	
	card32	m_NumScales;	// number of smoothed versions
	float32	m_H0;				// start-scale
	float32	m_F;				// h_i = h_0 * F^i
	bool	m_UseNormalOrientation;
	card32	m_SmoothingMethod;
	bool	m_DebugOutputScales;
	bool	m_UseTopology;
	float32	m_TargetMedPointDistance;

	enum ESmoothingMethod
	{
		SM_Plane = 0,
		SM_QuadPatch = 1,
		SM_MLS = 2
	};

	// members
	std::vector<float32>	m_ScaleDifferences;


	void computeScaleSpaceDifferences( UnstructuredInCorePointCloud * upc );
	void extractFeatures( UnstructuredInCorePointCloud * upc, FeatureSet * fs);

public:
	MultiScaleFeatureDetector(void);
	~MultiScaleFeatureDetector(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual void beginDetection( const std::vector<UnstructuredInCorePointCloud*>& clouds );
	virtual FeatureSet * detect( UnstructuredInCorePointCloud * upc, std::string attachmentName="" );

	void setupSHREC();


};

class ScaleSpaceSignature;

/**
		descriptor for scale space signatures
*/
class ScaleSpaceDescriptor : public LocalDescriptor
{
	X4_CLASS( ScaleSpaceDescriptor )
public:

	ScaleSpaceDescriptor();

	virtual LocalSignature * computeSignature(const Vector3f &point, const Vector3f &normal, float size, UnstructuredInCorePointCloud * upc)
	{return NULL;}

	virtual bool isRecomputeable() const {return false;}

private:

	float32	m_MinScaleSpaceDifference;

	friend ScaleSpaceSignature;
	friend MultiScaleFeatureDetector;
};

/**
		signature for scale space features
*/
class ScaleSpaceSignature : public LocalSignature
{
	X4_CLASS( ScaleSpaceSignature )

public:
		
	ScaleSpaceSignature();

	virtual bool badFeature(LocalDescriptor * myDescriptor) const;
	virtual float compare( LocalSignature * other, LocalDescriptor * myDescriptor );
	virtual float compare( LocalSignature * other ) { return compare(other, NULL) ; }

	float32	m_ScaleSpaceDifference;
	bool		m_IsMaxima;		// when normal direction is not used, all features are maxima
	card32	m_ScaleIndex;

};

}
      
#endif