#ifndef NDTH
#define NDTH

#include "x3foundation_private.h"

// Project
#include "LinearAlgebra.h"
#include "Persistent.h"
#include "Registerizer.h"
#include "RegularGrid.h"
#include "NDTCell.h"
#include "AbstractGrid.h"
#include "GridBuilder.h"

namespace X4 {

#define NDT_GRID_REGULAR 0
#define NDT_GRID_REGULAR_OVERLAPPING 1

/// Implementation of the Normal Distributions Transform for registration of two point clouds
class X3FOUNDATION_PRIVATE_API NDT : public Registerizer
{
	X4_CLASS( NDT ) 

	private:
		std::vector<AbstractGrid*> grids;			// Grid
		card32 numGrids;						// Number of grids; should equal grids.size()
		card32 setupGridMode;
		float32 setupGridTau;
		card32 setupGridRes;
		GridBuilder* gridBuilder;
		card32 gridType;
		bool ownGrids;

		void getDeriv( Vector3f& df_dp, const card32 component, const StaticMatrix<float32,7,3>& jacobian );
		void get2ndDeriv( Vector3f& d2f_dpdp, const card32 comp1, const card32 comp2, const StaticMatrix<float32,7,3>& jacobian );
		bool contains( BoundingBox3f bbox, Vector3f p );
		void printScore();
		void clear();
		void createGrids( const card32 type );
		void fillGrids();

		/// Container descriptor
		void setNumgridss(card32 size) { grids.resize( size ); }
		void deletegrids( card32 index ){ x4Assert( index < grids.size()); grids.erase( grids.begin()+index ); numGrids--; }
		char* getgrids( card32 index ){ x4Assert( index < grids.size()); return (char*)&(grids[index]);}		
		mpcard getNumgridss(){ return grids.size(); }
		std::string getgridsName( card32 index ){ 
			x4Assert( index < grids.size()); 
			return "Grid " + QString::number( index ).toStdString();
		}
		
	public:
		NDT();
		~NDT();

		void setup( PointSet* basePS, const Matrix3f& baseR = IDENTITY3F, const Vector3f& baseT = NULL_VECTOR3F );
		void setupWithGrid( PointSet* basePS, AbstractGrid* grid, const Matrix3f& baseR = IDENTITY3F, const Vector3f& baseT = NULL_VECTOR3F );
		float32 computeGradient( StaticVector<float32,7>& gradient );
		float32 computeHessian( StaticMatrix<float32,7,7>& hessian, StaticVector<float32,7>& gradient );
		inline bool supportsGradientDescent(){ return true; }
		inline bool supportsNewton(){ return true; }
		Matrix4f getTransformation() const;

		float32 computeScore(Matrix3f rot = IDENTITY3F, Vector3f t = NULL_VECTOR3F);
		void printStats() const;

		/*** MEMBER ACCESS ***/
		inline AbstractGrid* getGrid( const card32 index ) const { if ( index < grids.size() ) return grids[index]; else return NULL; }
		inline PointSet* getRegisterPS() const { return registerPS; }
		inline card32 getNumGrids() const { return numGrids; }
};
}

#endif