//---------------------------------------------------------------------------
#ifndef OctreeH
#define OctreeH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------

namespace X4 {

template<int dim>
class OctreeNode
{
	public:
		BoundingBox<float32,dim> bbox;
		OctreeNode<dim>* *children;

		OctreeNode(){
			children = NULL;
			for ( unsigned int n = 0; n < dim; n++ ){
				bbox.lowerCorner[n] = -1;
				bbox.upperCorner[n] = 1;
			}
		}
		~OctreeNode(){
			if ( children != NULL ){
				for ( unsigned int n = 0; n < getNumChildren(); n++ ){
					delete children[n];
					delete [] children;
					children = NULL;
				}
			}
		}

		bool isLeaf(){ return ( children == NULL ); }
		virtual OctreeNode<dim>* getChild( const card32 index ){ assert( children != NULL ); return children[index]; }
		inline card32 getNumChildren() const { assert( ( dim == 2 ) || ( dim == 3 ) ); return (dim==2)?4:8; }
};

template<int dim>
class Octree
{
	protected:
		OctreeNode<dim>* root;

		void deleteNode( OctreeNode<dim>* node );

	public:
		Octree();
		~Octree();

		void clear();
		virtual OctreeNode<dim>* getRootNode(){ return root; }
};

template<int dim>
Octree<dim>::Octree()
{
	root = new OctreeNode<dim>;
}

template<int dim>
Octree<dim>::~Octree()
{
	clear();
	delete root;
}

template<int dim>
void Octree<dim>::clear()
{
	deleteNode( root );
	for ( unsigned int n = 0; n < dim; n++ ){
		root->bbox.lowerCorner[n] = -1;
		root->bbox.upperCorner[n] = 1;
	}
}

template<int dim>
void Octree<dim>::deleteNode( OctreeNode<dim>* node )
{
	if ( !node->isLeaf() ){
		card32 size = pow( 2.0f, dim );
		for ( unsigned int n = 0; n < size; n++ ){
			deleteNode( node->children[n] );
			delete node->children[n];
		}
		delete [] node->children;
		node->children = NULL;
	}
}

}

#endif