//---------------------------------------------------------------------------
#ifndef OctreeIteratorH
#define OctreeIteratorH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
#include "Octree.h"
#include "FifoStack.h"
//---------------------------------------------------------------------------

namespace X4 {

template<int dim>
class PROJECTS_PJ_API OctreeIterator
{
	protected: 
		Octree<dim>* octree;
		FifoStack< OctreeNode<dim>* > stack;

		// Find next node to be returned in iterator
		virtual void findNextNode();

	public:
		OctreeIterator();
		void setup( Octree<dim>* octree ){ this->octree = octree; reset(); }
		void reset();
		bool atEnd();
		virtual OctreeNode<dim>* next();
};

template<int dim>
OctreeIterator<dim>::OctreeIterator()
{
	octree = NULL;
	stack.clear();
}

template<int dim>
void OctreeIterator<dim>::reset()
{
	stack.clear();
	if ( octree == NULL ) return;
	stack.push( octree->getRootNode() );
	findNextNode();
}

template<int dim>
bool OctreeIterator<dim>::atEnd()
{
	return ( stack.getNumItems() == 0 );
}

template<int dim>
void OctreeIterator<dim>::findNextNode()
{
	while ( ( stack.getNumItems() > 0 ) && !stack.first()->isLeaf() ){
		OctreeNode<dim>* node = stack.pop();
		for ( unsigned int childIndex = 0; childIndex < node->getNumChildren(); childIndex++ ){
			stack.push( node->getChild( childIndex ) );
		}
	}
}

template<int dim>
OctreeNode<dim>* OctreeIterator<dim>::next()
{
	OctreeNode<dim>* node = stack.pop();
	findNextNode();
	return node;
}

}

#endif