#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCAGeneral.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SVD.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCAGeneral ,0)
{
	BEGIN_CLASS_INIT( PCAGeneral );
}

PCAGeneral::PCAGeneral(void)
{
}

PCAGeneral::~PCAGeneral(void)
{
}

void PCAGeneral::assign(const Object* obj, X4CopyContext *context)
{
	// Persistent::assign(obj,context);
	const PCAGeneral * o = dynamic_cast<const PCAGeneral*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
	eigenValues = o->getEigenValues();
	meanVec = o->meanVec;
	eigenVecs = o->eigenVecs;
	parametersForInputData = o->parametersForInputData;

}


void X4::PCAGeneral::calculatePCA( const std::vector<DVectorF > &rawDataVecs, float normEigValCut, int maxDimension )
{
	if (rawDataVecs.size()<2){

		mpcard dataLen = rawDataVecs[0].getDim();

		meanVec.setDim(dataLen);

		for( unsigned i=0;i<rawDataVecs.size();i++ )
			meanVec=rawDataVecs[i];
		
	
		//calc parameters for input data
		parametersForInputData.clear();
		parametersForInputData.resize(rawDataVecs.size());
		DVectorF dummy(0);
		for( unsigned i=0;i<rawDataVecs.size();i++ )
		{
			parametersForInputData[i] = dummy; 
		}

		return;
	}

	
	mpcard dataLen = rawDataVecs[0].getDim();



	meanVec.setDim(dataLen);
	meanVec.setZero();

	for( unsigned i=0;i<rawDataVecs.size();i++ )
	{
		meanVec+=rawDataVecs[i];
	}
	meanVec /= (float)rawDataVecs.size();


	DMatrixD data;
	data.setDimension((card32)rawDataVecs.size(), rawDataVecs[0].getDim());

	for( unsigned i=0;i<rawDataVecs.size();i++ )
	{
		for (unsigned k=0; k < rawDataVecs[0].getDim(); k++)
			data[i][k] = rawDataVecs[i][k] - meanVec[k];
	}

	DVectorD retEigenValues ; DMatrixD U, Vt;
	SVD<double> svd;
	svd.computeSVD(data, U, retEigenValues, Vt);

	eigenVecs.clear();
	this->eigenValues.setDim(min(retEigenValues.getDim(), dataLen));
	for(mpcard i=0; i<min(retEigenValues.getDim(), dataLen); i++){
		DVectorF eigvec; 
		eigvec.setDim(eigenValues.getDim());
		for (unsigned k=0; k < eigenValues.getDim(); k++)
			eigvec[k] = U[i][k];

		eigenVecs.push_back(eigvec);
		//eigenVecs.push_back(U[i]);
		eigenVecs.back() /= eigenVecs.back().getNorm();
		eigenValues[i] = retEigenValues[i]/(sqrtf((float32)(rawDataVecs.size()-1)));
	}

	//this->eigenValues = retEigenValues/(sqrtf((float32)(rawDataVecs.size()-1)));

	/*debugOutput << "\npca eingenvals\n";
	for(mpcard e=0; e < eigenValues.size(); e++)
		debugOutput << eigenValues[e] << "\t";
	debugOutput << "\n\n";
	*/

	reasonabelCutOffEigenVectors( normEigValCut, maxDimension );

	//calc parameters for input data
	parametersForInputData.clear();
	parametersForInputData.resize(rawDataVecs.size());
	for( unsigned i=0;i<rawDataVecs.size();i++ )
	{
		parametersForInputData[i] = projectData(rawDataVecs[i]); 
	}

}

void X4::PCAGeneral::cutOffEigenVectors( mpint fromHere )
{
	//debugOutput << "cutOffEigenVectors: normalized EigVals before: " << getNormalizedEigenValues() << "\n";	
	
	mpcard numBefore = eigenVecs.size();
	for(mpint i=eigenVecs.size()-1; i>=fromHere; i--)
		eigenVecs.erase(eigenVecs.begin() + i);

// 	DVectorF backup = eigenValues;
// 	eigenValues.setDim(fromHere);
// 	for (mpint i=0; i<fromHere; ++i)
// 	{
// 		eigenValues[i]=backup[i];
// 	}
	eigenValues.cutOffDim(fromHere);

	for( unsigned i=0;i<parametersForInputData.size();i++ )
	{
		if (parametersForInputData[i].getDim() > 0)
			parametersForInputData[i].cutOffDim(fromHere); 
	}

	//debugOutput << "cut Eigenvectors from " << numBefore << " to " << eigenVecs.size() << "\n";
}

const X4::DVectorF X4::PCAGeneral::projectData( const DVectorF &testVec ) const
{
	DVectorF res;
	res.setDim((card32)eigenVecs.size());
	DVectorF centeredVec = testVec - meanVec;

	for(mpcard i=0; i<eigenVecs.size(); i++){
		res[i] = centeredVec * eigenVecs[i];
	}
	return res;
}

/*
Partial projections: Calculating lambda for some eigenvector works as follows:
1. set unknown components to 0 in both the eigenvector as well as the centered test std::vector
2. calculate the norm of the new eigenvector (=conversion factor), normalize it
3. calculate lambda using the changed eigenvector, divide lambda by conversion factor
(note: conversion factor works since all eigenvectors are normalized)

TODO: Recheck if this is correct / proof...
*/
const X4::DVectorF X4::PCAGeneral::projectPartialData(const DVectorF& testVec, const std::vector<bool>& knownComponents) const
{
	if (knownComponents.size() != testVec.getDim() || testVec.getDim() != meanVec.getDim())
		throw EInvalidState("Dimension mismatch in projectPartialData");

	DVectorF res;
	res.setDim((card32)eigenVecs.size());
	DVectorF centeredVec = testVec - meanVec;

	for (size_t i2 = 0; i2 < knownComponents.size(); ++i2)
		if (!knownComponents[i2])
			centeredVec[i2] = 0.0f;

	for(mpcard i=0; i<eigenVecs.size(); i++) {
		DVectorF eigenVector = eigenVecs[i];
		for (size_t i2 = 0; i2 < knownComponents.size(); ++i2)
			if (!knownComponents[i2])
				eigenVector[i2] = 0.0f;

		float32 conversionFactor = norm(eigenVector);
		eigenVector /= conversionFactor;

		res[i] = centeredVec * eigenVector;
		res[i] /= conversionFactor;
	}
	return res;
}

X4::float32 X4::PCAGeneral::calculatePartialError(const DVectorF &testVec, const DVectorF& lambdas, const std::vector<bool>& knownComponents)
{
	if (knownComponents.size() != testVec.getDim() || testVec.getDim() != meanVec.getDim())
		throw EInvalidState("Dimension mismatch in projectPartialData");

	float32 sigmaError = 0;

	DVectorF sigmaDist = getSigmaDistance(lambdas);
	if (!isWithin3Sigma(sigmaDist))
		return 1e30f;

	sigmaError = (sigmaDist * sigmaDist);
	DVectorF centeredVec = testVec - meanVec;
	DVectorF d(testVec.getDim());
	d.setZero();

	for(mpcard i=0; i<eigenVecs.size(); i++){
		d += eigenVecs[i] * lambdas[i];
	}

	DVectorF diff = centeredVec - d;
	float32 recError = 0;
	for (size_t i = 0; i < knownComponents.size(); ++i)
		if (knownComponents[i])
			recError += diff[i] * diff[i];

	// TODO: Scale depending on number of known components?

	return sqrtf(recError);
}


X4::float32 X4::PCAGeneral::calculateError( const DVectorF &testVec, const DVectorF& lambdas )
{
	float32 sigmaError = 0;

	DVectorF sigmaDist = getSigmaDistance(lambdas);
	if (!isWithin3Sigma(sigmaDist))
	{
		return 1e30f;
	}


	sigmaError = (sigmaDist * sigmaDist);
	DVectorF centeredVec = testVec - meanVec;
	DVectorF d(testVec.getDim());
	d.setZero();

	for(mpcard i=0; i<eigenVecs.size(); i++){
		d += eigenVecs[i] * lambdas[i];
	}

	DVectorF diff = centeredVec - d;
	float32 recError = ( diff *diff );


    //float32 sqrFac = 1/(eigenValues[0] * eigenValues[0]);
	//debugOutput << "sigmaErr: " << sqrtf(sigmaError) << " recErr: " << sqrtf(sqrFac*recError)  << " = " << sqrtf(sigmaError + sqrFac*recError) <<" factor:" << eigenValues[0] <<"\n";
	

	//return sqrtf(sigmaError + sqrFac*recError);
	return sqrtf(recError);
}

void X4::PCAGeneral::write( OutputObjectStream *s ) const
{
	Persistent::write(s);
	eigenValues.write(s);
	meanVec.write(s);
	card64 numEigenVecs = eigenVecs.size();
	s->write(numEigenVecs);
	for(mpcard i=0; i<eigenVecs.size(); i++){
		eigenVecs[i].write(s);
	}

	card64 numParameters = parametersForInputData.size();
	s->write(numParameters);
	for(mpcard i = 0; i< parametersForInputData.size(); i++){
		parametersForInputData[i].write(s);
	}
}


void X4::PCAGeneral::read( InputObjectStream *s )
{
	Persistent::read(s);
	eigenValues.read(s);
	meanVec.read(s);
	card64 numEigenVecs=0;
	s->read(numEigenVecs);
	eigenVecs.resize(numEigenVecs);
	for(mpcard i=0; i<numEigenVecs; i++){
		eigenVecs[i].read(s);
	}

	card64 numParameters=0;
	s->read(numParameters);
	parametersForInputData.resize(numParameters);
	for(mpcard i=0; i<numParameters; i++){
		parametersForInputData[i].read(s);
	}


}

const DVectorF X4::PCAGeneral::getSigmaDistance( const DVectorF &lambdas ) const
{
	DVectorF res;
	res.setDim(lambdas.getDim());
	for (mpcard i = 0; i < lambdas.getDim(); ++i)
	{
		res[i] = lambdas[i] / eigenValues[i];
	}
	return  res;
}

const DVectorF X4::PCAGeneral::applyLamdas( const DVectorF &lambdas ) const
{
	DVectorF result = meanVec;

	for(mpcard i=0; i<std::min<mpcard>(lambdas.size(), eigenVecs.size()); i++){
		result += eigenVecs[i] * lambdas[i];
	}
	return result;
}

bool X4::PCAGeneral::isWithin3Sigma( const DVectorF &sigmaDistance ) const
{
	for (mpcard i = 0; i < sigmaDistance.getDim(); ++i)
	{
		if (fabs(sigmaDistance[i])>3.0f)
			return false;
	}
	return true;
}

const DVectorF X4::PCAGeneral::getNormalizedEigenValues() const
{
	DVectorF result;
	if (eigenValues.size()<1 || eigenValues[0] == 0.0f)
	{
		return result;
	}
	result.setDim(eigenValues.getDim());
	float32 multiplier = 1.0f / eigenValues[0];
	for (mpcard i = 0; i < eigenValues.size(); i++)
	{
		result[i] = eigenValues[i] * multiplier;
	}

	return result;


}

void X4::PCAGeneral::reasonabelCutOffEigenVectors( float normEigValCut /*=0.05f*/, int maxDimension )
{
	DVectorF normEigVals = getNormalizedEigenValues();
	mpcard cutHere = 0;
	for (; cutHere < normEigVals.getDim(); cutHere++ )
	{
		if (normEigValCut > 0 && (normEigVals[cutHere] < normEigValCut || eigenValues[cutHere] < 0.000001f)) // relative + absolute test
		{
			break;
		}
	}
	cutOffEigenVectors(min((int)cutHere, maxDimension));

}


std::vector<Vector3f> X4::PCAGeneral::getMeanVecAsVec3f()
{

	x4Assert(meanVec.getDim() % 3 == 0);
	std::vector<Vector3f> result(meanVec.getDim() / 3);

	mpcard vecPos=0;
	for (mpcard i = 0; i < result.size(); i++ ){
		result[i] = makeVector3f(meanVec[vecPos], meanVec[vecPos+1], meanVec[vecPos+2]);
		vecPos+=3;
	}

	return result;

}

const std::vector<Vector3f> X4::PCAGeneral::convertDVtoVec3f( const DVectorF &dVec )
{
	const mpcard dim = dVec.getDim();
	x4Assert(dim % 3 == 0);
	std::vector<Vector3f> result(dim / 3);

	for (mpcard i = 0; i < dim/3; i++)
	{
		const Vector3f vec = makeVector3f(dVec[3 * i], dVec[3 * i + 1], dVec[3 * i + 2]);
		result[i] = vec;
	}

	return result;

}

const DVectorF X4::PCAGeneral::convertVec3fToDVec( const std::vector<Vector3f> &vec3f )
{
	DVectorF result(vec3f.size() * 3);


	for (mpcard p = 0; p < vec3f.size(); p++)
	{
		result[p*3+0] =  vec3f[p][0];
		result[p*3+1] =  vec3f[p][1];
		result[p*3+2] =  vec3f[p][2];
	}

	return result;

}

const DMatrixF X4::PCAGeneral::getEigenVecs() const
{
	DMatrixF result;
	if (eigenVecs.empty())
		return result;
	
	result.setDimension(eigenVecs.size(), eigenVecs[0].getDim(), false);
	for (mpcard i = 0; i < eigenVecs.size(); i++)
		result[i] = eigenVecs[i];
	return result;
}

void X4::PCAGeneral::resampleMe( const std::vector<bool> &pointsToDelete, mpcard targetNumDimensions )
{
	DVectorF oldMeanVec = meanVec;
	std::vector<DVectorF> oldEigenVecs(eigenVecs);
	for (mpcard i = 0; i < eigenVecs.size(); i++)
	{
		oldEigenVecs[i] = eigenVecs[i];
		eigenVecs[i].setDim(targetNumDimensions);
	}
	meanVec.setDim(targetNumDimensions);

	mpcard currentTargetIndex = 0;
	for (mpcard i = 0; i < pointsToDelete.size(); i++)
	{
		if (!pointsToDelete[i])
		{
			meanVec[currentTargetIndex] = oldMeanVec[i];
			for (mpcard e = 0; e < eigenVecs.size(); e++)
				eigenVecs[e][currentTargetIndex] = oldEigenVecs[e][i];
			currentTargetIndex++;
		}
	}

}
