#ifndef PCAGeneralH
#define PCAGeneralH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{
	class PCAMeshUrshape;

/**
	plz comment your class
*/
class X3FOUNDATION_PRIVATE_API PCAGeneral : public Persistent
{
	X4_CLASS( PCAGeneral )

private:
	DVectorF meanVec;
	std::vector<DVectorF> eigenVecs;
	DVectorF eigenValues;

	std::vector<DVectorF> parametersForInputData;


public:
	PCAGeneral(void);
	~PCAGeneral(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   void calculatePCA(const std::vector<DVectorF > &rawDataVecs, float normEigValCut = 0.05f, int maxDimension = 1000 );
   void cutOffEigenVectors( mpint fromHere );
   void reasonabelCutOffEigenVectors(float normEigValCut = 0.05f, int maxDimension = 1000 );
   const DVectorF projectData(const DVectorF &testVec) const;
   const DVectorF projectPartialData(const DVectorF &testVec, const std::vector<bool>& knownComponents) const;
   const DVectorF getSigmaDistance(const DVectorF &lambdas) const;
   bool isWithin3Sigma(const DVectorF &sigmaDistance) const;
   float32 calculateError(const DVectorF &testVec, const DVectorF& lambdas);
   float32 calculatePartialError(const DVectorF &testVec, const DVectorF& lambdas, const std::vector<bool>& knownComponents);
  
   const DVectorF& getMeanVec() const{return meanVec;}
   const DVectorF& getEigenValues() const {return eigenValues;}
   const DVectorF  getNormalizedEigenValues() const;
   const DMatrixF  getEigenVecs() const;

   const DVectorF applyLamdas(const DVectorF &lambdas) const;
   const DVectorF& getParametersForInputInstance(mpcard instInd) const {return parametersForInputData[instInd];}
   mpcard   getNumInputInstances() const{return parametersForInputData.size();}
   const std::vector<DVectorF>& getPCAWeightsForAllInputInstances() const {return parametersForInputData;}

   void resampleMe(const std::vector<bool> &pointsToDelete, mpcard targetNumDimensions);

   static const std::vector<Vector3f> convertDVtoVec3f(const DVectorF &dVec);
   static const DVectorF convertVec3fToDVec(const std::vector<Vector3f> &vec3f);

   std::vector<Vector3f> getMeanVecAsVec3f();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   friend PCAMeshUrshape;

};

}

#endif