#ifndef PCCCacheOptimizerH
#define PCCCacheOptimizerH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------

namespace X4
{

class UnstructuredInCorePointCloud;

/**
	optimizes unstructured in-core point clouds by reordering
*/
class X3FOUNDATION_PRIVATE_API PCCCacheOptimizer : public PCCommand
{
	X4_CLASS( PCCCacheOptimizer )

private:

	float32		m_Radius;
	bool			m_UseUniformResampler;

public:
	PCCCacheOptimizer(void);
	~PCCCacheOptimizer(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual void execute(Scene *scene);

	/// optimizes for a given cache radius (good value: 2.7)
	/// use it in the following way:
	///  - specify a minimum radius rad_min for your point cloud querries and a cache radius rad_cache
	///  - optimize with rad_cache
	///  - make a range querry with radius: rad_min + rad_cache
	/// don't use the uniform resampler, there was no real speed up with it!
	static UnstructuredInCorePointCloud * optimize( UnstructuredInCorePointCloud * upc, float radius,
		bool useUniformResampler=false);

	void optimizeForOctree();
	
	static void optimizeForOctree(UnstructuredInCorePointCloud * upc);

	/// asks for directory and counts all code lines in cpp/h files
	void countLinesInFiles();

	/// search for lines in text file that contain a std::string (order of charaters must be preserved)
	void specialSearchInFile();

};

}

#endif