#include "StdAfx.h"
//---------------------------------------------------------------------------
#include <windows.h>
#include <mmsystem.h>
#include "PCCImportGothic.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "FileNamePropertyEditor.h"
#include "RGBAImage.h"
#include "UnstructuredInCorePointCloud.h"
#include "PointCloudTools.h"
#include "ClassMethods.h"
#include "SceneGraphState.h"
#include "UnstructuredInCorePointCloud.h"
#include "ObjectViewsTable.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCCImportGothic ,0)
{
	BEGIN_CLASS_INIT( PCCImportGothic );
	ADD_STRING_PROP(m_Filename, 0 );
	SET_EDITOR( FileNamePropertyEditor::getClass());
	ADD_FLOAT32_PROP( m_Height, 0);
	ADD_FLOAT32_PROP( m_Radius, 0);
	ADD_FLOAT32_PROP( m_Perc_One, 0);

	ADD_NOARGS_METHOD( createScene1 );
	ADD_NOARGS_METHOD( importLKVK );
}

PCCImportGothic::PCCImportGothic(void)
{
	m_Height = 5;
	m_Radius = 5;
	m_Perc_One = 0.25f;
}

PCCImportGothic::~PCCImportGothic(void)
{
}

void PCCImportGothic::assign(const Object* obj, X4CopyContext *context)
{
	PCCommand::assign(obj,context);
	const PCCImportGothic * o = dynamic_cast<const PCCImportGothic*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}


void X4::PCCImportGothic::execute( Scene *scene )
{
	// load image
	RGBAImage img;
	img.loadFromFile(m_Filename.c_str());

	// extract height field (non-gray colors)
	RGBAImage heightfield(img.getSizeX(),img.getSizeY());
	for( int x=0;x<(int)img.getSizeX();x++ )
	{
		for( int y=0;y<(int)img.getSizeY();y++ )
		{
			Vector4ub p = img.getPixel(x,y);
			if( p[1] == p[2] )
				heightfield.setPixel(x,y, makeVector4ub(0,0,0,0));
			else
			{
				img.setPixel(x,y,makeVector4ub(0,0,0,0));
				heightfield.setPixel(x,y,p);
			}
			
		}
	}

	// create dest pc
	UnstructuredInCorePointCloud * pc = new UnstructuredInCorePointCloud;
	VertexDescriptor * vd = createDefaultVertexDescriptor(true,true,true,true);
	pc->clearAndSetup(vd,img.getSizeX()*img.getSizeY());
	PointSet * ps = pc->getPointSet();

	// create a minimum filtered image
	RGBAImage tmpImg(img.getSizeX(),img.getSizeY());
	int iRadius = ceil(m_Radius);
	for( int x=0;x<(int)img.getSizeX();x++ )
	{
		for( int y=0;y<(int)img.getSizeY();y++ )
		{
			int tmp = img.getPixel(x,y)[0];
			int dmin = max(x-iRadius,0);
			int dmax = min(x+iRadius,(int)(img.getSizeX()-1));
			for( int d = dmin;d<=dmax; d++ )
			{
				tmp = min(tmp,(int)img.getPixel(d,y)[0]);
			}
			tmpImg.setPixel(x,y,makeVector4ub(tmp,0,0,0));
		}
	}

	RGBAImage tmpImg2(img.getSizeX(),img.getSizeY());
	for( int x=0;x<(int)img.getSizeX();x++ )
	{
		for( int y=0;y<(int)img.getSizeY();y++ )
		{
			int tmp = tmpImg.getPixel(x,y)[0];
			int dmin = max(y-iRadius,0);
			int dmax = min(y+iRadius, (int)img.getSizeY()-1);
			for( int d = dmin;d<=dmax; d++ )
			{
				tmp = min(tmp,(int)tmpImg.getPixel(x,d)[0]);
			}
			tmpImg2.setPixel(x,y,makeVector4ub(tmp,0,0,0));
		}
	}

	// copy data and apply gothic style filter
	AAT posAAT = ps->getAAT("position");
	AAT normalAAT = ps->getAAT("normal");
	AAT colorAAT = ps->getAAT("color");
	for( int x=0;x<(int)img.getSizeX();x++ )
	{
		for( int y=0;y<(int)img.getSizeY();y++ )
		{
			int heightfieldHeight = heightfield.getPixel(x,y)[2];
			if( heightfieldHeight != 0)
			{
				float h = (float)heightfieldHeight / 255.0f * m_Height * 2.0f;
				Vector3f pos = makeVector3f(x,y,h + m_Height * 0.9f);
				Vector3f normal = makeVector3f(0,0,1);
				Vector3f color = makeVector3f(1,1,1);
				ps->set3f(y*img.getSizeX() + x,posAAT,pos);
				ps->set3f(y*img.getSizeX() + x,normalAAT,normal);
				ps->set3f(y*img.getSizeX() + x,colorAAT,color);
				continue;
			}

			float height = (float)img.getPixel(x,y)[0]/ 255.0f;
			Vector3f pos = makeVector3f(x,y,(1.0f - height) * m_Height);
			Vector3f normal = makeVector3f(0,0,1);
			Vector3f color = makeVector3f(1,1,1);

			// skip points near image border
			if( x>iRadius && y>iRadius && x<(int)img.getSizeX() -2-iRadius
				&& y<(int)img.getSizeY() -2-iRadius && tmpImg2.getPixel(x,y)[0] < 240)
			{
				// find nearest black pixel
				float distToNearest = 1e20f;
				Vector2f nearestDir;
				for( int sx=-iRadius;sx<=iRadius;sx++)
					for( int sy=-iRadius;sy<=iRadius;sy++)
					{
						float nHeight = (float)img.getPixel(x+sx,y+sy)[0] / 255.0f;
						if( nHeight < 0.25f )
						{
							float tmpDist = sqrtf((float)sx*sx+sy*sy) + nHeight;
							if( tmpDist < distToNearest)
							{
								distToNearest = tmpDist;
								nearestDir = makeVector2f(sx,sy);
							}
						}
					}

					pos[2] = arcFunction(distToNearest);
					if( norm(nearestDir) > 0 )
					{
						normal = arcFunctionNormal(distToNearest,nearestDir);
					}
					color = makeVector3f(0,distToNearest / m_Radius,0);
				
			}
			ps->set3f(y*img.getSizeX() + x,posAAT,pos);
			ps->set3f(y*img.getSizeX() + x,normalAAT,normal);
			ps->set3f(y*img.getSizeX() + x,colorAAT,color);
		}
	}

	addPointCloud(scene,pc,"pc");
	
}

float X4::PCCImportGothic::arcFunction( float x )
{
	x = x/m_Radius;
	if( x >= 1.0f )
		return 0;

	if( x < m_Perc_One )
	{
		return m_Height;
	}
	x = (x-m_Perc_One)/(1.0f - m_Perc_One);

	float maxX = 0.8f;
	float scaleY = 1.0f / (1.0f - sqrtf(1.0f - maxX*maxX));
	
	x = (1.0f - x) * maxX;
	return (1.0f - sqrtf(1.0f - x*x)) * scaleY * m_Height;
}

X4::Vector3f X4::PCCImportGothic::arcFunctionNormal( float x, Vector2f dir )
{
	x = x/m_Radius;
	//if( x >= 1.0f )
		return makeVector3f(0,0,1);

	/*float maxX = 0.8f;
	float scaleY = 1.0f / (1.0f - sqrtf(1.0f - maxX*maxX));

	x = (1.0f - x) * maxX;
	return (1.0f - sqrtf(1.0f - x*x)) * scaleY * m_Height;*/
	
}

void X4::PCCImportGothic::createScene1()
{
	// get point cloud
	std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() != 1 )
		return;
	PointCloud * pc = getPointCloud(getScene(),selection[0]);
	if( !pc )
		return;
	AAT posAAT = pc->getAAT("position");
	AAT normalAAT = pc->getAAT("normal");

	// create dest point cloud
	UnstructuredInCorePointCloud * destPC = new UnstructuredInCorePointCloud;
	destPC->clearAndSetup( pc->getDescr(), pc->getNumPoints() * 4 );
	
	/*
		segments:

						222222222223333333333333
						1
				      1
				     1
				    1
			    1
		   111
	  1111
	  0
	  0
	  0
	  0
	  0
	  0
	  0
	  0
	  0


	*/

	BoundingBox3f bb = getPCBBox(pc);
	float medPointDistance = getMedianPointDistance(pc);
	float towerRadius = 2.0f / (float)M_PI;
	PointSet * destPS = destPC->getPointSet();
	card32 pindex = 0;
	for( unsigned segment=0;segment<4;segment++ )
	{
		Matrix4f T = IDENTITY4F;
		switch( segment )
		{
		case 0:
			T = makeTranslation4f(0,0,towerRadius + 1.0f) * makeRotY4f((float)M_PI/2.0f);
			break;
		case 2:
			T = makeTranslation4f(towerRadius,0,0);
			break;
		case 3:
			T = makeTranslation4f(towerRadius+1.0f,0,0);
			break;
		default:
			break;
		}

		std::vector<char> vbuffer(pc->getDescr()->getSize());
		SceneObjectIterator * iter = pc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC);
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);

		while( !bIt->atEnd())
		{
			Vector3f pos = bIt->get3f(posAAT) / (bb.getSideLength(0)+medPointDistance);
			Vector3f normal = bIt->get3f(normalAAT);

			if( segment == 1 )
			{
				T = makeRotY4f((pos[0])*M_PI_2)*makeTranslation4f(-pos[0],0,towerRadius);
			}

			pos = transformVector3f(T,pos);
			normal = shrink4To3(T) * normal;
			
			// copy vertex
			bIt->getVertex(&vbuffer[0]);
			destPS->setVertex(pindex,&vbuffer[0]);

			destPS->set3f(pindex,posAAT,pos);
			destPS->set3f(pindex,normalAAT,normal);
			pindex++;

			bIt->next();
		}

		delete iter;

	}

	addPointCloud(getScene(),destPC, "castle");

}

struct VFormat {
	Vector3f position;
	Vector3f color;
	float value;
	int32 flags;
};

void X4::PCCImportGothic::importLKVK()
{
	VertexDescriptor vd;
	vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(mVAD("color",    3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(mVAD("value",    1, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(mVAD("flags",    1, VAD::DATA_FORMAT_INT32));

	UnstructuredInCorePointCloud *pc = new UnstructuredInCorePointCloud();
	pc->clearAndSetup(&vd, 0);
	addPointCloud(scene, pc, "lkvk");
	
	SceneObjectIterator *it = pc->createIterator(SOIT::CAP_DYNAMIC_PC);
	if (!it) throw X4Exception("targetPC is not dynamic");
	DynamicPointCloudIterator *dit = dynamic_cast<DynamicPointCloudIterator*>(it);

	std::string filename = "I:/aberner/ludwigskirche.pts";

	mpcard numParts = 0;

	const mpcard maxParts = 999999;
	BoundingBox3d bb;
	bool bbset= false;
	{
		//std::ifstream in(filename.c_str());
		QFile inFile(filename.c_str());
		if( !inFile.open(QIODevice::ReadOnly) ) {
			throw EIOException("file not found.");
		}
		QTextStream inStream(&inFile);
		inStream.readLine();

		card64 numRead = 0;
		while (!inStream.atEnd()) {
			if (numParts > maxParts)
				break;
			
			QString line = inStream.readLine();
			std::string test = line.toStdString();
			QStringList list=line.split(" ");
			if (list.size()!=7){
				debugOutput << numParts <<". part\n"; debugOutput.flush();
				numParts++;
				continue;
			}

			Vector3d p;
			p[0]= list[0].toDouble();
			p[1]= list[1].toDouble();
			p[2]= list[2].toDouble();

			if (bbset) {
				bb.addPoint(p);
			} else {
				bb.upperCorner = p;
				bb.lowerCorner = p;
				bbset = true;
			}
			if ((numRead % 1000000) == 0) {
				debugOutput << "p: " << p.toString() << "\n";
				debugOutput << numRead << "\n";
				debugOutput.flush();
			}

			numRead++;
		}
	}

	debugOutput << "next\n";
	debugOutput.flush();

	Vector3d center = bb.getCenter();
	double scale = 2.0/bb.getMaxSideLength();

	numParts = 0;

	{
		//std::ifstream in(filename.c_str());
		QFile inFile(filename.c_str());
		if( !inFile.open(QIODevice::ReadOnly) ) {
			throw EIOException("file not found.");
		}
		QTextStream inStream(&inFile);
		inStream.readLine();

		card64 numRead = 0;
		while (!inStream.atEnd()) {
			if (numParts > maxParts)
				break;
			QString line = inStream.readLine();
			std::string test = line.toStdString();
			QStringList list=line.split(" ");
			if (list.size()!=7){
				debugOutput << numParts <<". part\n"; debugOutput.flush();
				numParts++;

				pc = new UnstructuredInCorePointCloud();
				pc->clearAndSetup(&vd, 0);
				addPointCloud(scene, pc, "lkvk");

				delete it;

				ObjectViewsTable::update(scene);
				
				it = pc->createIterator(SOIT::CAP_DYNAMIC_PC);

				dit = dynamic_cast<DynamicPointCloudIterator*>(it);


				continue;
			}

			Vector3d p;
			p[0]= list[0].toDouble();
			p[1]= list[1].toDouble();
			p[2]= list[2].toDouble();
			Vector3f c;
			c[0]=list[4].toInt();
			c[1]=list[5].toInt();
			c[2]=list[6].toInt();

			p -= center;
			p *= scale;
			c /= 255.0f;

			VFormat vertex;
			vertex.position[0] = (float32)p[0];
			vertex.position[1] = (float32)p[1];
			vertex.position[2] = (float32)p[2];
			vertex.color[0] = c[0];
			vertex.color[1] = c[1];
			vertex.color[2] = c[2];
			vertex.value = (float32)numParts;  // list[3].toInt();;
			vertex.flags = 0;

			dit->add((const char*)&vertex);

			if ((numRead % 1000000) == 0) {
				debugOutput << "p: " << p.toString() << "\n";
				debugOutput << numRead << "\n";
				debugOutput.flush();
			}


			numRead++;
		}
		debugOutput << "num read: " << numRead << "\n";
	}



}