//---------------------------------------------------------------------------
#ifndef PCCResampleByCurvatureH
#define PCCResampleByCurvatureH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
#include "PointCloud.h"
#include "UnstructuredInCorePointCloud.h"
#include "Scene.h"
//---------------------------------------------------------------------------

namespace X4 {

   /// Downsampling of point clouds by curvature (can be used for pc simplification)
   class X3FOUNDATION_PRIVATE_API PCCResampleByCurvature : public PCCommand 
   {
      X4_CLASS( PCCResampleByCurvature )

   private:
      float32 minSpacing;		// Min spacing between points for curvature 1
      float32 maxSpacing;     // max spacing of points for curvature 0
      
      //int curvatureChannel;   // Channel in curvature attachment to get the curvature value from
      float32 mTopologyReconnectMaxRadius;
      
	  bool useTopologicalRangeQuery;

   public:
      PCCResampleByCurvature();
      ~PCCResampleByCurvature();
      void execute( Scene *scene );


      /**
       * Resample the given point cloud. If topology graph is connected, then it will be changed too
       * according to the existing connections.
       **/
	  void resamplePC(UnstructuredInCorePointCloud* pc, bool writeToFlagsOldIndex = false, bool onlyWriteFlagsToDelete = false) {resamplePC(pc,minSpacing,maxSpacing,mTopologyReconnectMaxRadius,useTopologicalRangeQuery,writeToFlagsOldIndex,onlyWriteFlagsToDelete);}
	  static void resamplePC(UnstructuredInCorePointCloud* pc, float minSpacing, float maxSpacing, float topologyReconnect, bool useTopologicalRangeQuery, bool writeToFlagsOldIndex = false, bool onlyWriteFlagsToDelete = false);

      float32 getMinSpacing() const {return minSpacing;}
      void setMinSpacing(float32 v) {minSpacing=v;}

      float32 getMaxSpacing() const {return maxSpacing;}
      void setMaxSpacing(float32 v) {maxSpacing=v;}

      //int getCurvatureChannel() const { return curvatureChannel; }
      //void setCurvatureChannel(int v) {curvatureChannel=v;}
      
      float32 getTopologyReconnectMaxRadius() const { return mTopologyReconnectMaxRadius; }
      void setTopologyReconnectMaxRadius(float32 f) { mTopologyReconnectMaxRadius = f;}

   };


}

#endif
