#ifndef PCCSurfaceToolsH
#define PCCSurfaceToolsH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------

namespace X4
{

	class PointCloud;

/**
	provides needful things for surface analysis.
*/
class X3FOUNDATION_PRIVATE_API PCCSurfaceTools : public PCCommand
{
	X4_CLASS( PCCSurfaceTools )

private:

	// properties
	float32	m_KernelSigma;
	bool		m_UseNormalOrientation;
	float32 m_MlsMaxOffset;
	std::string	m_DestChannelName;
	bool		m_ScaleCurvatures;
	float32	m_ScaleCurvatureBorder;
	float32	m_CacheRadius;
	bool		m_UseMedianFilter;
	bool		m_UseTopogicalRangeQuerry;

	// members
	std::vector<float>	m_Curv1, m_Curv2, m_Curv3;
	
	std::vector<PointCloud*> getSelectedPointClouds();

public:
	PCCSurfaceTools(void);
	~PCCSurfaceTools(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual void execute(Scene *scene) {}
	
	void estimateCurvature();
	void calculateAveragePointDistance();
	void calculateMedianPointDistance();
	void computeCurvatureHistogram();
	void colorizePointClouds();
	void resample();
	void importHeightfield();
	void selectBorderPoints();
	void eraseBadPoints();

	static void selectBorderPoints( PointCloud * pc, float radius, float minAngle, unsigned flag = PF_FLAG_SELECTED);
	void deleteInvalidPoints();
	void colorizeFromOtherPC();
	void mlsProjectPC();
	void alignBoundingBoxes();

   void estimateCurvature( PointCloud * pc);
   void scaleCurvature( PointCloud * pc, const Vector4f &minCurv, const Vector4f &maxCurv);

	/// creates a smoothed version of pc using mls projection
	/// if dataPC is specified, all points from pc are projected onto the surface defined by dataPC
	/// if dataPC is NULL, pc itself is used as input
   static PointCloud * mlsProjectPC( PointCloud * pc, float sigma, PointCloud * dataPC = NULL );

	static std::vector<Vector3f> getRandomColorTable(card32 numColors);

	// resamples pc using grid resampling and mls projection (also smoothes normals)
	static PointCloud * resamplePC( PointCloud * pc, float sigma);
	
   float32 getKernelSigma() const {return m_KernelSigma;}
   void setKernelSigma(float32 v) {m_KernelSigma=v;}

   bool getUseNormalOrientation() const {return m_UseNormalOrientation;}
   void setUseNormalOrientation(bool v) {m_UseNormalOrientation=v;}

   std::string getDestChannelName() const {return m_DestChannelName;}
   void setDestChannelName(std::string v) {m_DestChannelName=v;}

   bool getScaleCurvatures() const {return m_ScaleCurvatures;}
   void setScaleCurvatures(bool v) {m_ScaleCurvatures=v;}

   float32 getScaleCurvatureBorder() const {return m_ScaleCurvatureBorder;}
   void setScaleCurvatureBorder(float32 v) {m_ScaleCurvatureBorder=v;}

   float32 getCacheRadius() const {return m_CacheRadius;}
   void setCacheRadius(float32 v) {m_CacheRadius=v;}

   bool getUseMedianFilter() const {return m_UseMedianFilter;}
   void setUseMedianFilter(bool v) {m_UseMedianFilter=v;}

   bool getUseTopogicalRangeQuerry() const {return m_UseTopogicalRangeQuerry;}
   void setUseTopogicalRangeQuerry(bool v) {m_UseTopogicalRangeQuerry=v;}


};

}

#endif
