#ifndef PCHOCDescriptorH
#define PCHOCDescriptorH

#include "x3foundation_private.h"

#include "Persistent.h"



namespace X4 {

class FastSphereQuerry;
class AttachedIndexedOctree;
class UnstructuredInCorePointCloud;


/// mw - Nov 12th 2011
/// Histograms of oriented principal curvature directions
class X3FOUNDATION_PRIVATE_API PCHOCDescriptor : public Persistent {
	X4_CLASS( PCHOCDescriptor )
 private:
   // -- parameters
 
   /// point +/- radius in u,v direction is covered by descriptor
   float32 radius;
   /// subdivisions of the [0?..180] range
   card32 numOrientationBins;
   /// subdivisions of the [-radius..+radius] range
   card32 numSpatialBins;
   /// relative smoothing of Gaussian filters employed (default = 1.0)
   float32 smoothness;
   /// threshold for determining unique orientation 
   /// (factor by which non-dominant bins need to be surpassed, default = 0.9)
   float32 orientationUniquenessThreshold;
   /// normalize to mean of curvature magnitudes (default = true)
   bool normalizeDescriptor;

   Vector3f m_up;
   bool useUp;

   // -- temporary
   FastSphereQuerry *query;
   UnstructuredInCorePointCloud *upc;
   PointSet *ps;
   AAT positionAAT;
   AAT curv2AAT;
   AAT tangentVAAT;
   AAT normalAAT;
   bool singleWarning;

 public:
   PCHOCDescriptor();
   PCHOCDescriptor(float32 rad, mpcard orientationBins, mpcard spatialBins, 
	               float32 OrientationUniqueness = 0.9f, float32 smoothnessThreshold = 1.0f, bool normalize = true);

   void assign(const Object* obj, X4CopyContext *context);
   
   void beginComputation(UnstructuredInCorePointCloud *upc);
   /// returns true if descriptor is "valid" and false if it is "fishy"
   bool getDescriptor(mpcard index, float32 radius, DVectorF &result, bool debugVis = false);
   bool getDescriptor(const Vector3f& pos, const Vector3f& n, float32 radius, DVectorF &result, bool debugVis = false);
   void endComputation();


   void setUpDir(Vector3f& up);
   void useUpDir(bool use=true);

   ~PCHOCDescriptor();

};


}

#endif