#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIMeshGeodesics.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "MeshGeodesics.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "PointCloudTools.h"
#include "ClassMethods.h"
#include "Random.h"
//#include <Windows.h>
#include "DebugRenderer.h"
#include "ColorTools.h"
#include "GeometricFeature.h"
#include "LocalDescriptor.h"
#include "SGObjectNode.h"
#include "SceneGraphState.h"
#include "InCorePCTopologyGraph.h"
#include "ObjectViewsTable.h"
#include "ColorTools.h"
#include "TopologyAttachment.h"
#include "PickFromPointSet.h"
#include "ProgressWindow.h"
#include "BinaryObjectStreams.h"
#include "AttachedFloatSignatures.h"
#include "IntrinsicWaveDescriptor.h"
#include "VertexArray.h"
#include "PCAGeneral.h"
#include "SGRelativeTimeAnimationNode.h"
#include "SceneGraphTools.h"
#include "SceneEditorWidget.h"
#include "FastSphereQuerry.h"
#include "UnstructuredInCoreTriangleMeshManifold.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCIMeshGeodesics ,0)
{
	BEGIN_CLASS_INIT( PCIMeshGeodesics );
	//	ADD_NOARGS_METHOD(initGeo);
	//	ADD_NOARGS_METHOD(testGeo);

	// 	ADD_NOARGS_METHOD(calcBronsteinFeatures);
	// 	ADD_NOARGS_METHOD(testGeodesicDescriptor);
	// 	ADD_NOARGS_METHOD(testExactGeodesicDescriptor);
	// 	ADD_NOARGS_METHOD(calculateWaveSequenceDescriptor);
	ADD_NOARGS_METHOD(testShowDescriptor);
	//ADD_NOARGS_METHOD(theOtherTest);
	ADD_NOARGS_METHOD(descriptorPCA);

	ADD_NOARGS_METHOD(importTOSCA);
	ADD_NOARGS_METHOD(importHannover);
	ADD_NOARGS_METHOD(maxCurv);

	ADD_NOARGS_METHOD(filterOpt);
	ADD_NOARGS_METHOD(precomputeAllDescriptors);

	ADD_FLOAT32_PROP(maxGeoDist,0);

	ADD_CARD32_PROP(minFaces, 0);
	ADD_FLOAT32_PROP(angleTol,0)
		ADD_FLOAT32_PROP(weightSumThreshold,0)
		ADD_FLOAT32_PROP(curvPower,0)
		ADD_FLOAT32_PROP(anglePower,0)
		ADD_FLOAT32_PROP(absoluteCurvThreshold,0)
}

PCIMeshGeodesics::PCIMeshGeodesics(void)
{
	mg = NULL;
	minFaces = 10;

	maxGeoDist = 0.07f;
	useFeature = 0;
	checkPoint = false;

	selectedVertex = -1;

	//selectedVertex = 40361;


	angleTol = 0.7f;
	weightSumThreshold = 0.54f;
	curvPower = 3.0f;
	anglePower = 4.0f;
	absoluteCurvThreshold = 0.2f;




}

PCIMeshGeodesics::~PCIMeshGeodesics(void)
{
	delete mg;
}

void PCIMeshGeodesics::assign(const Object* obj, X4CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCIMeshGeodesics * o = dynamic_cast<const PCIMeshGeodesics*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::PCIMeshGeodesics::testGeo()
{

	if (mg)
	{
		lastPath.clear();
		mg->getGeodesicDistance(random(numPoints), random(numPoints), &lastPath);

		debugRenderer->beginRenderJob_OneFrame("line_", 0);
		for (mpcard p = 0; p < lastPath.size(); p++)
		{
			Vector3f col = shrink4To3(rainbowGradient(float(p)/float(lastPath.size())));
			debugRenderer->addPoint(lastPath[p], col);
			if(p < (lastPath.size()-1))
				debugRenderer->addLine(lastPath[p], lastPath[p+1], col, col,3);
		}
		debugRenderer->endRenderJob();
	}

}

void X4::PCIMeshGeodesics::initGeo()
{

	UnstructuredInCoreTriangleMesh* tri = dynamic_cast<UnstructuredInCoreTriangleMesh*> (getPointCloud(getScene(), "root/pc1"));
	if (!tri)
		return;
	numPoints = tri->getNumPoints()-1;

	if (mg)
	{
		delete mg;
		mg = NULL;
	}

	mg = new MeshGeodesics;
	mg->initializeFromUnstructuredInCoreTriangleMesh(tri);
	lastPath.clear();

}

void X4::PCIMeshGeodesics::glDrawTool( GLContext *glContext )
{

}

void X4::PCIMeshGeodesics::initBronstein()
{
	std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();

	for (unsigned sel=0; sel < selection.size(); sel++)
	{
		SGObjectNode* node = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), selection[sel]));

		UnstructuredInCoreTriangleMesh* tri = dynamic_cast<UnstructuredInCoreTriangleMesh*> (node->getSceneObject());
		if (!tri) continue;

		std::vector<card32> numfaces(tri->getNumPoints(), 0);

		PointSet* tris = tri->getTriangles();
		AAT IND = tris->getAAT("index");
		for (mpcard i = 0; i < tris->getNumEntries(); i++)
		{
			Vector3i ti = tris->get3i(i, IND);
			for (mpcard j = 0; j < 3; j++)
			{
				numfaces[ti[j]]++;
			}
		}


		checkAttribute(tri, "numFaces", 1, VAD::DATA_FORMAT_INT32);
		PointSet* ps = tri->getPointSet();
		AAT NUMF = ps->getAAT("numFaces");


		for (mpcard i = 0; i < ps->getNumEntries(); i++)
		{
			ps->set1i(i, NUMF, numfaces[i]);
		}

	}
}

void X4::PCIMeshGeodesics::calcBronsteinFeatures()
{
	initBronstein();
	std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();

	for (unsigned sel=0; sel < selection.size();  sel++)
	{
		SGObjectNode* node = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), selection[sel]));

		UnstructuredInCoreTriangleMesh* tri = dynamic_cast<UnstructuredInCoreTriangleMesh*> (node->getSceneObject());
		if (!tri) continue;


		PointSet* ps = tri->getPointSet();
		AAT NUMF = ps->getAAT("numFaces");
		AAT POS = ps->getAAT("position");
		AAT NORMAL = ps->getAAT("normal");

		debugRenderer->clearDebugData("feat");
		debugRenderer->beginRenderJob("feat");

		card32 maxNum = 0;
		card32 minNum = 1e10;

		for (mpcard i = 0; i < ps->getNumEntries(); i++)
		{
			card32 numf = ps->get1i(i, NUMF);
			if (numf<minFaces)
				continue;

			if (numf > maxNum)
				maxNum = numf;
			if (numf < minNum)
				minNum = numf;
		}


		FeatureSet * fs = new FeatureSet;

		for (mpcard i = 0; i < ps->getNumEntries(); i++)
		{
			card32 numf = ps->get1i(i, NUMF);
			if (numf<minFaces)
				continue;


			float32 floatVal = float32(numf-minNum)/float32(maxNum-minNum);
			debugRenderer->addFastSphere(ps->get3f(i,POS), 0.002f, shrink4To3(rainbowGradient(floatVal)), false);
			debugOutput << numf << "\n";

			GeometricFeature* f = new GeometricFeature;
			f->m_Position = ps->get3f(i,POS);
			f->m_Normal   = ps->get3f(i, NORMAL);
			f->nearestPointIndex = i;

			FloatVectorSignature* sig = new FloatVectorSignature;

			DVectorF x; x.setDim(1); x[0] = numf; 

			sig->setVector(x);

			f->addSignature(sig);

			fs->m_Features.push_back(f);

		}

		fs->setup( FeatureSet::getDefaultName(), AttachedData::ADF_PERSISTENT );
		tri->getAttachments()->attachData(fs);

		debugRenderer->endRenderJob();
	}


	paths.clear();
}

class  DistanceQueuePoint
{
public:

	DistanceQueuePoint(int32 i, int32 cameFrom, float32 ld){index=i; myGeoDist=ld; cameFromIndex=cameFrom;}
	int32 index;
	int32 cameFromIndex;
	float32 myGeoDist;
	inline bool operator<(const DistanceQueuePoint &other) const {
		return myGeoDist > other.myGeoDist;
	}


};

class BorderEdge{
public:
	BorderEdge(int32 outsideIn, int32 cameFrom, float32 d){outIndex=outsideIn; cameFromIndex=cameFrom; outsideDistance=d;}
	int32 outIndex;
	int32 cameFromIndex;
	float32 outsideDistance;

	Vector3f exactBorderPosition;


	bool operator<(const BorderEdge &other) const {
		return outIndex < other.outIndex;
	}
	bool operator==(const BorderEdge &other) const {
		return outIndex==other.outIndex;
	}
};

class MyPair{
public:
	MyPair(int32 aa, int32 bb){a=aa; b=bb; }
	int32 a;
	int32 b;

	bool operator<(const MyPair &other) const {
		if (a==other.a)
			return b< other.b;
		else
			return a< other.a;
	}
	bool operator==(const MyPair &other) const {
		return a==other.a && b == other.b;
	}
};

class RingNeighbour{
public:
	int32 myIndex;
	Vector3f myExactBorderPosition;
	int32 prevIndex;
	int32 nextIndex;
	RingNeighbour(int32 my, Vector3f pos, int32 a, int32 b){
		myIndex = my; myExactBorderPosition=pos; prevIndex = a; nextIndex = b;
	};
	void swapPrevNext(){
		int32 temp = prevIndex;
		prevIndex = nextIndex;
		nextIndex = temp;
	}

};

void X4::PCIMeshGeodesics::testGeodesicDescriptor()
{
	//debugOutput << "testGeodesicDescriptor\n";
	std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();

	debugRenderer->clearDebugData("dijk");
	debugRenderer->beginRenderJob("dijk");

	for (unsigned sel=0; sel < selection.size(); sel++)
	{
		SGObjectNode* node = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), selection[sel]));

		UnstructuredInCoreTriangleMesh* tri = dynamic_cast<UnstructuredInCoreTriangleMesh*> (node->getSceneObject());
		if (!tri) continue;


		if(!dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt")))
			tri->computeTopologyAttachment();


		FeatureSet* fSet = dynamic_cast<FeatureSet*>(tri->getAttachments()->getData(FeatureSet::getDefaultName()));
		if (!fSet ) continue;

		if (fSet->m_Features[0]->nearestPointIndex == -1)
		{
			HierarchicalKNNIterator knn( tri, 32, NULL);
			for (mpcard featureInd = 0; featureInd < fSet->m_Features.size(); featureInd++)
			{
				knn.setSeekPointAndReset(fSet->m_Features[featureInd]->m_Position);
				int32 meshVertexInd = knn.getCurrentPointIndex();
				fSet->m_Features[featureInd]->nearestPointIndex = meshVertexInd;
			}
		}

		for (mpcard featureInd = 0; featureInd < fSet->m_Features.size(); featureInd++)
		{

			if (featureInd != useFeature)
				continue;

			int32 startIndex = fSet->m_Features[featureInd]->nearestPointIndex;
			float32 res = computeIntrinsicWaveDescriptor(tri, startIndex, maxGeoDist);
			debugOutput << res << "\n";
		}

		debugRenderer->endRenderJob();
		ObjectViewsTable::update(getScene());

	}
}

void X4::PCIMeshGeodesics::testExactGeodesicDescriptor()
{
	debugOutput << "testExactGeodesicDescriptor\n";
	std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();

	debugRenderer->clearDebugData("exact");
	debugRenderer->beginRenderJob("exact");

	for (unsigned sel=0; sel < selection.size(); sel++)
	{
		SGObjectNode* node = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), selection[sel]));

		UnstructuredInCoreTriangleMesh* tri = dynamic_cast<UnstructuredInCoreTriangleMesh*> (node->getSceneObject());
		if (!tri) continue;


		if(!dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt")))
			tri->computeTopologyAttachment();

		PointSet* ps = tri->getPointSet();
		AAT POS = ps->getAAT("position");

		FeatureSet* fSet = dynamic_cast<FeatureSet*>(tri->getAttachments()->getData(FeatureSet::getDefaultName()));
		TopologyAttachment* graphAtt = dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt"));
		if (!fSet || ! graphAtt) continue;
		InCorePCTopologyGraph* graph = graphAtt->getTopology();

		PointSet* edges = graph->getEdges();
		AAT EDIST = edges->getAAT("edgeDistance");

		MeshGeodesics mGeos;
		mGeos.initializeFromUnstructuredInCoreTriangleMesh(tri);
		mGeos.setSubdivisionLevel(-1);

		std::vector<float32> upperBounds;

		float32 lengthInPlane = (2*M_PI*maxGeoDist);
		//debugOutput << "lengthInPlane: "<<lengthInPlane <<"\n";

		float32 maxGeoDistTimesFac = maxGeoDist *1.5f;

		for (mpcard featureInd = 0; featureInd < fSet->m_Features.size(); featureInd++)
		{

			if (featureInd != useFeature)
			{
				continue;
			}

			int32 startIndex = fSet->m_Features[featureInd]->nearestPointIndex;

			std::vector<float> distanceField(ps->getNumEntries(), 1e30f);
			distanceField[startIndex] = 0;

			priority_queue<DistanceQueuePoint> growQueue;
			growQueue.push(DistanceQueuePoint(startIndex, startIndex, 0));

			std::vector<BorderEdge> outsidePoints;
			outsidePoints.reserve(50);

			while (!growQueue.empty() && growQueue.top().myGeoDist <= (maxGeoDistTimesFac))
			{
				DistanceQueuePoint testPoint=growQueue.top();
				growQueue.pop();

				for (mpcard n = 0; n < graph->getNumAdjacentVertices(testPoint.index); n++)
				{
					int32 neighborInd = graph->getAdjacentVertex(testPoint.index, n);
					float32 dist = edges->get1f(graph->getEdgeIndex(testPoint.index, n), EDIST) + testPoint.myGeoDist;
					if (dist<distanceField[neighborInd])
					{
						distanceField[neighborInd] = dist;
						growQueue.push(DistanceQueuePoint(neighborInd, testPoint.index ,dist));

						if (dist>maxGeoDistTimesFac && testPoint.myGeoDist < maxGeoDistTimesFac){
							outsidePoints.push_back(BorderEdge(neighborInd, testPoint.index, dist));
						}
					}
				}
			}


			sort(outsidePoints.begin(), outsidePoints.end());
			outsidePoints.erase(  unique(outsidePoints.begin(), outsidePoints.end()), outsidePoints.end());

			for (mpint op = outsidePoints.size()-1; op >= 0; op--)
			{
				if (distanceField[outsidePoints[op].outIndex] < maxGeoDistTimesFac)
				{
					debugRenderer->addSphere(ps->get3f(outsidePoints[op].outIndex, POS), 0.015f, makeVector3f(1,1,0));
					outsidePoints.erase(outsidePoints.begin() + op);
				}
				else
				{
					Vector3f endPos = ps->get3f(outsidePoints[op].outIndex, POS);				
					debugRenderer->addSphere(endPos, 0.012f, makeVector3f(1,0,0));
				}
			}


			std::vector<std::vector<int32> > borderNeighbors;
			borderNeighbors.resize(outsidePoints.size());
			for (mpint op = outsidePoints.size()-1; op >= 0; op--)
			{
				int32 oInd = outsidePoints[op].outIndex;
				for (mpcard n = 0; n < graph->getNumAdjacentVertices(oInd); n++)
				{
					int32 neighborInd = graph->getAdjacentVertex(oInd, n);
					BorderEdge dummy(neighborInd,-1,0);
					if (binary_search(outsidePoints.begin(), outsidePoints.end(), dummy))
					{
						borderNeighbors[op].push_back(neighborInd);
					}
				}
			}


			//get rid of wrong edges
			std::vector<MyPair> unallowedEdges;
			for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
			{
				Vector3f col = makeVector3f(0,1,0);
				float w =3;
				if (borderNeighbors[bn].size()!=2)
				{
					col = makeVector3f(1,0,0);
					w=5;

					std::vector<int32> myNeighbours = borderNeighbors[bn];
					for (mpcard cn = 0; cn < borderNeighbors.size(); cn++)
					{
						if (borderNeighbors[cn].size()!=2)
						{
							for (mpcard ne = 0; ne < myNeighbours.size(); ne++)
							{
								if (myNeighbours[ne]==outsidePoints[cn].outIndex)
								{
									debugRenderer->addLine(ps->get3f(outsidePoints[bn].outIndex, POS), ps->get3f(borderNeighbors[bn][ne], POS), makeVector3f(1,0.5f,0.5f), makeVector3f(1,1,0), w*2);
									//debugRenderer->addSphere(ps->get3f(outsidePoints[bn].outIndex, POS), 0.02f, makeVector3f(1,0,1));

									unallowedEdges.push_back(MyPair(outsidePoints[bn].outIndex, borderNeighbors[bn][ne]));
								}

							}
						}
					}
				}
			}

			sort(unallowedEdges.begin(), unallowedEdges.end());
			unallowedEdges.erase(  unique(unallowedEdges.begin(), unallowedEdges.end()), unallowedEdges.end());
			sort(unallowedEdges.begin(), unallowedEdges.end());

			// 			for (mpcard p = 0; p< unallowedEdges.size(); p++)
			// 			{
			// 				debugRenderer->addLine(ps->get3f(unallowedEdges[p].a, POS), ps->get3f(unallowedEdges[p].b, POS), makeVector3f(1,1,0), makeVector3f(1,1,0), 8);
			// 			}


			for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
			{
				if (borderNeighbors[bn].size()!=2)
				{
					for (mpint cn = borderNeighbors[bn].size()-1; cn >=0 ; cn--){
						MyPair test(outsidePoints[bn].outIndex, borderNeighbors[bn][cn]);
						if (binary_search(unallowedEdges.begin(), unallowedEdges.end(), test ) )
						{
							borderNeighbors[bn].erase(borderNeighbors[bn].begin() + cn);
						}

					}
				}
			}


			/*
			std::vector<BorderEdge> orderedOutsidePoints;
			orderedOutsidePoints.push_back(outsidePoints[0]);
			int32 lastIndex = orderedOutsidePoints[orderedOutsidePoints.size()-1].outIndex;
			int32 nextTargetInd = borderMap[lastIndex];
			int32 nextTarget = borderNeighbors[nextTargetInd][0];
			int32 nextOldIndex = borderMap[nextTarget];

			for (mpcard i = 1; i < outsidePoints.size(); i++)
			{

			BorderEdge connectedEdge = outsidePoints[nextOldIndex];
			std::vector<int32> theTwoNeigbours = borderNeighbors[nextOldIndex];
			if (theTwoNeigbours[0]==lastIndex)
			nextTarget = theTwoNeigbours[1];
			else
			nextTarget = theTwoNeigbours[0];		

			orderedOutsidePoints.push_back(connectedEdge);


			}
			*/






			float32 lengthUpperBound = 0;
			for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
			{

				for (mpcard ne = 0; ne < borderNeighbors[bn].size(); ne++)
				{
					Vector3f col = makeVector3f(1,0.3f,0.3f);
					debugRenderer->addLine(ps->get3f(outsidePoints[bn].outIndex, POS), ps->get3f(borderNeighbors[bn][ne], POS), col, col, 12);
					lengthUpperBound += norm(ps->get3f(outsidePoints[bn].outIndex, POS)- ps->get3f(borderNeighbors[bn][ne], POS));
				}
			}

			lengthUpperBound *= 0.5f; // every part counted 2 times before!

			upperBounds.push_back(lengthUpperBound/ lengthInPlane);

			//debugOutput << lengthUpperBound/ lengthInPlane <<"\n";



			// test exact method
			std::vector<mpcard> targets;
			for (mpcard i = 0; i < outsidePoints.size(); ++i)
			{
				targets.push_back(outsidePoints[i].outIndex);
			}

			std::vector<float32> exactLengths = mGeos.getGeodesicDistances(startIndex, targets, &paths);

			//debugOutput << "exactLengths:\n";
			//debugRenderer->setPointSize(5);
			for (mpcard i = 0; i < targets.size(); ++i)
			{
				//debugOutput << exactLengths[i] << "\n";
				Vector3f col = rainbowGradient3f((float)i / (float32)targets.size());
				for (mpcard j = 0; j < paths[i].size()-1; ++j)
				{
					debugRenderer->addLine(paths[i][j], paths[i][j+1], col, col, 1);
					//debugRenderer->addPoint(paths[i][j], makeVector3f(1,0,0));
				}
				//debugRenderer->addPoint(paths[i][paths[i].size()-1], makeVector3f(1,0,0));


				Vector3f searchPos = getPosOnTheWay(maxGeoDist, paths[i]);
				debugRenderer->addSphere(searchPos, 0.02f, makeVector3f(1,1,0));
				outsidePoints[i].exactBorderPosition = searchPos;
			}





			/*
			float32 realCircleLen = 0;
			for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
			{

			for (mpcard ne = 0; ne < borderNeighbors[bn].size(); ne++)
			{
			Vector3f col = makeVector3f(1,0,0);
			//debugRenderer->addLine(ps->get3f(outsidePoints[bn].outIndex, POS), ps->get3f(borderNeighbors[bn][ne], POS), col, col, 12);
			lengthUpperBound += norm(ps->get3f(outsidePoints[bn].outIndex, POS)- ps->get3f(borderNeighbors[bn][ne], POS));
			}
			}

			lengthUpperBound *= 0.5f; // every part counted 2 times before!

			upperBounds.push_back(lengthUpperBound/ lengthInPlane);

			debugOutput << lengthUpperBound/ lengthInPlane <<"\n";*/





			std::vector<int32> borderMap(ps->getNumEntries(), -1);
			for (mpcard i = 0; i<borderNeighbors.size(); i++)
				borderMap[outsidePoints[i].outIndex] = i;


			std::vector<RingNeighbour> ringPoints;
			for (mpcard i = 0; i< outsidePoints.size(); i++)
			{
				if(borderNeighbors[i].size() == 2) //TODO: hack for non manifolds!!
					ringPoints.push_back(RingNeighbour(outsidePoints[i].outIndex, outsidePoints[i].exactBorderPosition, borderNeighbors[i][0], borderNeighbors[i][1] ));
			}

			std::vector<RingNeighbour> orderedRingPoints;
			orderedRingPoints.reserve(ringPoints.size());
			orderedRingPoints.push_back(ringPoints[0]);
			for (mpcard i = 1; i < ringPoints.size(); i++)
			{
				int32 nextRealPoint = orderedRingPoints[orderedRingPoints.size()-1].nextIndex;
				int32 nextArrayIndex = borderMap[nextRealPoint];
				RingNeighbour nextNeighborPoint = ringPoints[nextArrayIndex];
				if (nextNeighborPoint.prevIndex!=orderedRingPoints[orderedRingPoints.size()-1].myIndex)
				{
					nextNeighborPoint.swapPrevNext();
				}
				orderedRingPoints.push_back(nextNeighborPoint);
			}

			float32 circleRingRadius = 0;
			for (mpcard i = 0; i < orderedRingPoints.size()-1; i++)
			{
				Vector3f a = orderedRingPoints[i].myExactBorderPosition;
				Vector3f b = orderedRingPoints[i+1].myExactBorderPosition;
				circleRingRadius += norm(b-a);
				debugRenderer->addLine(a,b, makeVector3f(1,0,0), makeVector3f(1,0,0),8);
			}

			if (orderedRingPoints[orderedRingPoints.size()-1].nextIndex == orderedRingPoints[0].myIndex)
			{
				Vector3f a = orderedRingPoints[orderedRingPoints.size()-1].myExactBorderPosition;
				Vector3f b = orderedRingPoints[0].myExactBorderPosition;
				circleRingRadius += norm(b-a);
				debugRenderer->addLine(a,b, makeVector3f(1,0,0), makeVector3f(1,0,0),8);
			}

			//debugOutput << "The exact circle: " << circleRingRadius << "\n";
			debugOutput << /*"The normalized circle: " <<*/ circleRingRadius/lengthInPlane << "\n";



		}



		debugRenderer->endRenderJob();
		ObjectViewsTable::update(getScene());


	}
}

X4::Vector3f X4::PCIMeshGeodesics::getPosOnTheWay( float32 howFar, const std::vector<Vector3f> &way )
{
	mpint at=way.size()-1;
	float32 wayGone = 0;
	while(at>0 && wayGone < howFar){
		wayGone += norm(way[at]-way[at-1]);
		at--;
	}

	x4Assert(at>=0);

	x4Assert(howFar < wayGone);

	Vector3f startPos = way[at+1];
	Vector3f endPos = way[at];
	Vector3f diffAB = endPos-startPos;
	Vector3f direction = normalize( diffAB) ; 
	float32 wayTooFar = wayGone - howFar;
	Vector3f goVec = direction * wayTooFar;
	Vector3f finalPos = endPos - goVec;

	return finalPos;



}

bool X4::PCIMeshGeodesics::pickPoint( int32 x, int32 y, PointCloud *pc, mpcard &index )
{
	UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	if (!upc) return false;
	PointSet *ps = upc->getPointSet();
	float32 dist = computeMinOnScreenDistancePoint(
		x,y,ps, 
		getScene()->getRootState()->staticState->viewFrustum,
		getScene()->getRootState()->staticState->camera,
		screenSizeX, screenSizeY, 
		index);
	if (dist <= 12.0f) return true;
	return false;
}

void X4::PCIMeshGeodesics::mouseDown( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState )
{
	UnstructuredInCoreTriangleMesh* umf = (UnstructuredInCoreTriangleMesh* 	) getPointCloud(getScene(), "root/pc1");
	mpcard index = 0;
	if (pickPoint(x,y,umf, index))
	{
		debugOutput << "clicked on point " << index << "\n"; 

	}else{
		debugOutput << "pickPoint didn't work\n";
		return;
	}

//////////////////////////////////////////////////////////////////////////
	if (selectedVertex == -1)
	{
		selectedVertex = index;
		return;
	}else{
		// show geo path between selectedVertex and index
		//calc geo path
		MeshGeodesics mg;
		mg.setSubdivisionLevel(-1);
		mg.initializeFromUnstructuredInCoreTriangleMesh(umf);
		std::vector<Vector3f> path;
		mg.getGeodesicDistance(selectedVertex, index, &path);

		debugRenderer->beginRenderJob_OneFrame("line_", 0);
		for (mpcard p = 0; p < path.size(); p++)
		{
			Vector3f col = shrink4To3(rainbowGradient(float(p)/float(path.size())));
			debugRenderer->addPoint(path[p], col);
			if(p < (path.size()-1))
				debugRenderer->addLine(path[p], path[p+1], col, col,3);
		}
		debugRenderer->endRenderJob();
		ObjectViewsTable::update(scene);


		selectedVertex = index;
		return;
	}
//////////////////////////////////////////////////////////////////////////


	selectedVertex = index;

	AttachedFloatSignatures* sigs = dynamic_cast<AttachedFloatSignatures*>(umf->getAttachments()->getData("AttachedFloatSignatures"));
	if (!sigs)
	{
		mpcard index = 0;
		if (pickPoint(x,y,umf, index))
		{
			debugOutput << "clicked on point " << index << "\n"; 
			debugRenderer->beginRenderJob_OneFrame("d",0);
			computeIntrinsicWaveDescriptor(umf, index, maxGeoDist);
			debugRenderer->endRenderJob();
			return;
		}else
			debugOutput << "clicked on point didn't work... \n"; 
	}






	/*

	UnstructuredInCoreTriangleMeshManifold* manA = dynamic_cast<UnstructuredInCoreTriangleMeshManifold*>(getSceneObject(scene, "root/Manifolds/mf1")) ;
	if (!manA)
	throw X4Exception( "PCIEntropyMatching::descriptorDistance: Manifold not found/dynamic_cast didn't work...\n");

	UnstructuredInCoreTriangleMesh* meshA = manA->getMesh() ;*/


	UnstructuredInCoreTriangleMesh* meshA = (UnstructuredInCoreTriangleMesh* 	) getPointCloud(getScene(), "root/pc1");






	if (selectedVertex < 0 )
		return;

	debugRenderer->beginRenderJob_OneFrame("iwd", 0);
	VertexArray verta(meshA);
	debugRenderer->addFastSphere(verta.getPosition3f(selectedVertex), 0.003f,  makeVector3f(0.333f,0.333f,1));
	debugRenderer->endRenderJob();





	if (cachedDescriptors.empty())
	{

		AttachedFloatSignatures* sigs = dynamic_cast<AttachedFloatSignatures*>(meshA->getAttachments()->getData("AttachedFloatSignatures"));
		if (sigs)
		{
			cachedDescriptors = sigs->descriptors;
			debugOutput << "loaded precomputed descriptors..\n";
			debugOutput.flush();
		}else
			throw X4Exception("Please precompute descriptors first.\n");
	}

	const DVectorF testDiscr = cachedDescriptors[selectedVertex];

	for (mpcard i = 0; i < testDiscr.getDim(); i++)
	{
		debugOutput << testDiscr[i] << "\t";
	}
	debugOutput << "\n";

	VertexArray vert(meshA);

	float32 maxDist = 0;
	std::vector<float32> dists(cachedDescriptors.size());
	int32 best = 0;
	float32 minVal = 1e30f;

	for (mpcard i = 0;  i < cachedDescriptors.size(); i++)
	{
		float32 diff = norm(testDiscr-cachedDescriptors[i]);
		if(diff > maxDist)
			maxDist = diff;
		dists[i] = diff;
		if (i== selectedVertex)
			continue;

		if (diff < minVal)
		{
			minVal = diff;
			best = i;
		}
	}

	for (mpcard i = 0;  i < cachedDescriptors.size(); i++)
	{
		vert.setColor3f(i, rainbowGradient3f(1.0f - (dists[i]/maxDist)));
	}

	debugRenderer->clearDebugData("clickedOn");
	debugRenderer->beginRenderJob("clickedOn");	
	debugRenderer->addFastSphere(vert.getPosition3f(selectedVertex), 0.005f, makeVector3f(1,0.333f,1), false);
	debugRenderer->addFastSphere(vert.getPosition3f(best), 0.005f, makeVector3f(1,0.333f,1), false);
	debugRenderer->endRenderJob();

	scene->rebuildRenderObjects();
	ObjectViewsTable::update(scene);


	return;


	if (checkPoint)
	{

		UnstructuredInCorePointCloud* umf = (UnstructuredInCorePointCloud* 	) getPointCloud(getScene(), "root/pc1");
		FeatureSet* fs = (FeatureSet*)umf->getAttachments()->getData(FeatureSet::getDefaultName());

		mpcard index = 0;
		int nearestFeature = -1;
		if (pickPoint(x,y,umf, index))
		{
			debugOutput << "clicked on point " << index << "\n"; 
			Vector3f pos = umf->getPointSet()->get3f(index, umf->getAAT("position"));
			float nearestDist = FLT_MAX;

			// find feature point which is the nearest to the chosen one
			for (unsigned i=0; i < fs->getNumFeatures(); i++)
			{
				GeometricFeature* gf = fs->m_Features[i];
				if (norm(gf->m_Position - pos) < nearestDist)
				{
					nearestFeature = i;
					nearestDist = norm(gf->m_Position - pos);
				}
			}
			if (nearestFeature < 0) return;
		}

		debugOutput << "using feature " << nearestFeature << "\n"; 

		useFeature = nearestFeature;

		//find best matches



		std::vector<float32> diffs(fs->m_Features.size());
		GeometricFeature* cf = fs->m_Features[nearestFeature];

		float32 maxVal = 0;
		for (mpcard featureInd = 0; featureInd < fs->m_Features.size(); featureInd++){
			diffs[featureInd] = cf->compare( fs->m_Features[featureInd], NULL );

			if (diffs[featureInd] > maxVal)
				maxVal = diffs[featureInd];
		}
		if (maxVal==0)
		{
			maxVal=0.1f;
		}

		std::vector<float32> sorted = diffs;
		sort(sorted.begin(), sorted.end());


		debugRenderer->clearDebugData("test");
		debugRenderer->beginRenderJob("test");
		for (mpcard featureInd = 0; featureInd < fs->m_Features.size(); featureInd++){
			debugRenderer->addSphere( fs->m_Features[featureInd]->m_Position, 0.008f, rainbowGradient3f(diffs[featureInd]/maxVal)) ;
		}
		debugRenderer->endRenderJob();
		ObjectViewsTable::update(getScene());
	}
	else


	{
		UnstructuredInCorePointCloud* pc1 = (UnstructuredInCorePointCloud* 	) getPointCloud(getScene(), "root/pc1");
		FeatureSet* fs1 = (FeatureSet*)pc1->getAttachments()->getData(FeatureSet::getDefaultName());

		UnstructuredInCorePointCloud* pc2 = pc1;//(UnstructuredInCorePointCloud* 	) getPointCloud(getScene(), "root/pc2");
		FeatureSet* fs2 = (FeatureSet*)pc2->getAttachments()->getData(FeatureSet::getDefaultName());

		Matrix4f trans2 = pc2->getTransformation();




		mpcard index = 0;
		int nearestFeature = -1;
		if (pickPoint(x, y ,pc1, index))
		{
			debugOutput << "clicked on point " << index << "\n"; 
			Vector3f pos = pc1->getPointSet()->get3f(index, pc1->getAAT("position"));
			float nearestDist = FLT_MAX;

			// find feature point which is the nearest to the chosen one
			for (unsigned i=0; i < fs1->getNumFeatures(); i++)
			{
				GeometricFeature* gf = fs1->m_Features[i];
				if (norm(gf->m_Position - pos) < nearestDist)
				{
					nearestFeature = i;
					nearestDist = norm(gf->m_Position - pos);
				}
			}
			if (nearestFeature < 0) return;
		}

		debugOutput << "using feature " << nearestFeature << "\n"; 

		if (nearestFeature < 0) return;
		useFeature = nearestFeature;


		//find best matches
		std::vector<float32> diffs(fs2->m_Features.size());
		GeometricFeature* cf = fs1->m_Features[nearestFeature];

		float32 maxVal = 0;
		int32 best = 0;
		float32 minVal = 1e30f;

		for (mpcard featureInd = 0; featureInd < fs2->m_Features.size(); featureInd++){
			diffs[featureInd] = cf->compare( fs2->m_Features[featureInd], NULL );

			if (diffs[featureInd] > maxVal)
				maxVal = diffs[featureInd];

			if (diffs[featureInd] < minVal)
			{
				minVal = diffs[featureInd];
				best = featureInd;
			}
		}
		if (maxVal==0)
		{
			maxVal=0.1f;
		}

		std::vector<float32> sorted = diffs;
		sort(sorted.begin(), sorted.end());
		maxVal = sorted[3000];


		debugRenderer->clearDebugData("test");
		debugRenderer->beginRenderJob("test");
		for (mpcard featureInd = 0; featureInd < fs2->m_Features.size(); featureInd++){
			//debugRenderer->addSphere( transformVector3f(trans2, fs2->m_Features[featureInd]->m_Position), 0.008f, rainbowGradient3f(diffs[featureInd]/maxVal)) ;
			debugRenderer->addSphere(  fs2->m_Features[featureInd]->m_Position, 0.008f, rainbowGradient3f(diffs[featureInd]/maxVal)) ;
		}
		//debugRenderer->addSphere( cf->m_Position, 0.008f, makeVector3f(1,1,0)) ;		
		//debugRenderer->addLine(cf->m_Position, transformVector3f(trans2, fs2->m_Features[best]->m_Position), makeVector3f(1,1,0), makeVector3f(0,0,1), 2);
		//debugRenderer->addSphere( transformVector3f(trans2, fs2->m_Features[best]->m_Position), 0.012f, rainbowGradient3f(diffs[best]/maxVal)) ;	
		debugRenderer->endRenderJob();
		ObjectViewsTable::update(getScene());
	}


}

void X4::PCIMeshGeodesics::calculateWaveSequenceDescriptor()
{



	std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	for (unsigned sel=0; sel < selection.size(); sel++)
	{
		SGObjectNode* node = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), selection[sel]));

		UnstructuredInCoreTriangleMesh* tri = dynamic_cast<UnstructuredInCoreTriangleMesh*> (node->getSceneObject());
		if (!tri) continue;


		if(!dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt")))
			tri->computeTopologyAttachment();
		//this TopologyAttachment has to be here


		FeatureSet* fSet = dynamic_cast<FeatureSet*>(tri->getAttachments()->getData(FeatureSet::getDefaultName()));
		if (!fSet ) continue;

		if (fSet->m_Features[0]->nearestPointIndex == -1)
		{
			HierarchicalKNNIterator knn( tri, 32, NULL);
			for (mpcard featureInd = 0; featureInd < fSet->m_Features.size(); featureInd++)
			{
				knn.setSeekPointAndReset(fSet->m_Features[featureInd]->m_Position);
				int32 meshVertexInd = knn.getCurrentPointIndex();
				fSet->m_Features[featureInd]->nearestPointIndex = meshVertexInd;
			}
		}

		for (mpcard featureInd = 0; featureInd < fSet->m_Features.size(); featureInd++)
		{

			debugOutput << featureInd << "\n"; debugOutput.flush(); 

			int32 vertexIndexInMesh = fSet->m_Features[featureInd]->nearestPointIndex; //nearestPointIndex must be set in feature detector (if not, e. g. take nearest mesh vertex here)

			DVectorF featureVec = computeIntrinsicWaveSequenceDescriptor(tri, vertexIndexInMesh);
			FloatVectorSignature* sig = new FloatVectorSignature;
			sig->setVector(featureVec);

			GeometricFeature * f = fSet->m_Features[featureInd];
			f->deleteSignatures(0, f->getNumSignatures()); // remove old signatures
			f->addSignature(sig);	//add new signature

		}
	}

}

void X4::PCIMeshGeodesics::initGeo(UnstructuredInCoreTriangleMesh* tri)
{
	if (mg) delete mg;
	mg = new MeshGeodesics;
	mg->initializeFromUnstructuredInCoreTriangleMesh(tri);
	mg->setSubdivisionLevel(-1);
}

float32 X4::PCIMeshGeodesics::computeDistance (card32 ptA, card32 ptB, std::vector<Vector3f>* path)
{
	return mg->getGeodesicDistance(ptA, ptB, path);
}


X4::float32 X4::PCIMeshGeodesics::computeDistanceB (UnstructuredInCoreTriangleMesh* tri, card32 ptA, card32 ptB)
{
	MeshGeodesics* geod = new MeshGeodesics;
	geod->initializeFromUnstructuredInCoreTriangleMesh(tri);
	geod->setSubdivisionLevel(-1);
	float32 dist = geod->getGeodesicDistance(ptA, ptB);
	delete geod;

	return dist;
}

X4::float32 X4::PCIMeshGeodesics::computeIntrinsicWaveDescriptor( UnstructuredInCoreTriangleMesh* tri, int32 startIndex, float32 intrinsicRadius )
{
	PointSet* ps = tri->getPointSet();
	AAT POS = ps->getAAT("position");

	if(!dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt")))
		tri->computeTopologyAttachment();


	TopologyAttachment* graphAtt = dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt"));
	InCorePCTopologyGraph* graph = graphAtt->getTopology();

	if (!graph->getEdges()->getDescr()->providesAttribute("edgeDistance"))
	{
		tri->computeTopologyAttachment();
		graphAtt = dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt"));
		graph = graphAtt->getTopology();
	}


	PointSet* edges = graph->getEdges();
	AAT EDIST = edges->getAAT("edgeDistance");

	float32 lengthInPlane = (2*M_PI*intrinsicRadius);

	std::vector<float> distanceField(ps->getNumEntries(), 1e30f);
	distanceField[startIndex] = 0;

	priority_queue<DistanceQueuePoint> growQueue;
	growQueue.push(DistanceQueuePoint(startIndex, startIndex, 0));

	std::vector<BorderEdge> outsidePoints;
	outsidePoints.reserve(50);

	while (!growQueue.empty() && growQueue.top().myGeoDist <= intrinsicRadius)
	{
		DistanceQueuePoint testPoint=growQueue.top();
		growQueue.pop();


		for (mpcard n = 0; n < graph->getNumAdjacentVertices(testPoint.index); n++)
		{
			int32 neighborInd = graph->getAdjacentVertex(testPoint.index, n);
			float32 dist = edges->get1f(graph->getEdgeIndex(testPoint.index, n), EDIST) + testPoint.myGeoDist;
			if (dist<distanceField[neighborInd])
			{
				distanceField[neighborInd] = dist;
				growQueue.push(DistanceQueuePoint(neighborInd, testPoint.index ,dist));

				if (dist>intrinsicRadius && testPoint.myGeoDist < intrinsicRadius){
					outsidePoints.push_back(BorderEdge(neighborInd, testPoint.index, dist));
				}
			}
		}
	}


	sort(outsidePoints.begin(), outsidePoints.end());
	outsidePoints.erase(  unique(outsidePoints.begin(), outsidePoints.end()), outsidePoints.end());

	for (mpint op = outsidePoints.size()-1; op >= 0; op--)
	{
		if (distanceField[outsidePoints[op].outIndex]<intrinsicRadius)
		{
			//debugRenderer->addSphere(ps->get3f(outsidePoints[op].outIndex, POS), 0.015f, makeVector3f(0,1,0));
			outsidePoints.erase(outsidePoints.begin() + op);
		}else
		{
			Vector3f endPos = ps->get3f(outsidePoints[op].outIndex, POS);
			Vector3f startPos = ps->get3f(outsidePoints[op].cameFromIndex, POS);
			Vector3f direction = normalize( endPos - startPos ) ;
			float32 wayTooFar = outsidePoints[op].outsideDistance - intrinsicRadius;
			Vector3f finalPos = endPos - direction * wayTooFar;
			//debugRenderer->addSphere(endPos, 0.012f, makeVector3f(1,0,0));
			//debugRenderer->addSphere(finalPos, 0.01f, makeVector3f(0,0,1));
			//debugRenderer->addLine(endPos, finalPos, makeVector3f(1,0,0),  makeVector3f(0,0,1),2);
			outsidePoints[op].exactBorderPosition = finalPos;
		}
	}

	std::vector<std::vector<int32> > borderNeighbors;
	borderNeighbors.resize(outsidePoints.size());
	for (mpint op = outsidePoints.size()-1; op >= 0; op--)
	{
		int32 oInd = outsidePoints[op].outIndex;
		for (mpcard n = 0; n < graph->getNumAdjacentVertices(oInd); n++)
		{
			int32 neighborInd = graph->getAdjacentVertex(oInd, n);
			BorderEdge dummy(neighborInd,-1,0);
			if (binary_search(outsidePoints.begin(), outsidePoints.end(), dummy))
			{
				borderNeighbors[op].push_back(neighborInd);
			}
		}
	}


	//get rid of wrong edges
	std::vector<MyPair> unallowedEdges;
	for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
	{
		Vector3f col = makeVector3f(0,1,0);
		float w =3;
		if (borderNeighbors[bn].size()!=2)
		{
			col = makeVector3f(1,0,0);
			w=5;

			std::vector<int32> myNeighbours = borderNeighbors[bn];
			for (mpcard cn = 0; cn < borderNeighbors.size(); cn++)
			{
				if (borderNeighbors[cn].size()!=2)
				{
					for (mpcard ne = 0; ne < myNeighbours.size(); ne++)
					{
						if (myNeighbours[ne]==outsidePoints[cn].outIndex)
						{
							//debugRenderer->addLine(ps->get3f(outsidePoints[bn].outIndex, POS), ps->get3f(borderNeighbors[bn][ne], POS), makeVector3f(1,1,0), makeVector3f(1,1,0), w*2);
							//debugRenderer->addSphere(ps->get3f(outsidePoints[bn].outIndex, POS), 0.02f, makeVector3f(1,0,1));

							unallowedEdges.push_back(MyPair(outsidePoints[bn].outIndex, borderNeighbors[bn][ne]));
						}

					}
				}
			}
		}
	}

	sort(unallowedEdges.begin(), unallowedEdges.end());
	unallowedEdges.erase(  unique(unallowedEdges.begin(), unallowedEdges.end()), unallowedEdges.end());
	sort(unallowedEdges.begin(), unallowedEdges.end());

	for (mpcard p = 0; p< unallowedEdges.size(); p++)
	{
		//debugRenderer->addLine(ps->get3f(unallowedEdges[p].a, POS), ps->get3f(unallowedEdges[p].b, POS), makeVector3f(1,1,0), makeVector3f(1,1,0), 7);
	}


	for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
	{
		if (borderNeighbors[bn].size()!=2)
		{
			for (mpint cn = borderNeighbors[bn].size()-1; cn >=0 ; cn--){
				MyPair test(outsidePoints[bn].outIndex, borderNeighbors[bn][cn]);
				if (binary_search(unallowedEdges.begin(), unallowedEdges.end(), test ) )
				{
					borderNeighbors[bn].erase(borderNeighbors[bn].begin() + cn);
				}
			}
		}
	}



	std::vector<int32> borderMap(ps->getNumEntries(), -1);
	for (mpcard i = 0; i<borderNeighbors.size(); i++)
		borderMap[outsidePoints[i].outIndex] = i;


	//float32 lengthUpperBound = 0;
	float32 lengthLowerBound = 0;
	for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
	{

		for (mpcard ne = 0; ne < borderNeighbors[bn].size(); ne++)
		{
			Vector3f col = makeVector3f(1,0,0);
			// 			debugRenderer->addLine(ps->get3f(outsidePoints[bn].outIndex, POS), ps->get3f(borderNeighbors[bn][ne], POS), col, col, 6);
			// 			lengthUpperBound += norm(ps->get3f(outsidePoints[bn].outIndex, POS)- ps->get3f(borderNeighbors[bn][ne], POS));

			col = makeVector3f(0.333f,0.333f,1);
			debugRenderer->addLine(outsidePoints[bn].exactBorderPosition, outsidePoints[borderMap[borderNeighbors[bn][ne]]].exactBorderPosition, col, col, 12);
			lengthLowerBound += norm(outsidePoints[bn].exactBorderPosition - outsidePoints[borderMap[borderNeighbors[bn][ne]]].exactBorderPosition);

		}
	}

	//lengthUpperBound *= 0.5f; // every part counted 2 times before!
	lengthLowerBound *= 0.5f; // every part counted 2 times before!

	//debugOutput << lengthUpperBound/ lengthInPlane <<"\t";
	//debugOutput << lengthLowerBound/ lengthInPlane <<"\n";

	return lengthLowerBound / lengthInPlane;
}

X4::DVectorF X4::PCIMeshGeodesics::computeIntrinsicWaveSequenceDescriptor( UnstructuredInCoreTriangleMesh* tri, int32 meshVertexIndex , float scale )
{




	std::vector<float32> rawValues(160, -1.0);

	for (mpcard i = 0; i < 160; i++) //160
	{
		float32 maxGeoWaveDist = (0.05f + (float32)i * 0.006f) * scale;//0.00125f;
		bool showThisStep = false; //(i%10 ==0);

		if (showThisStep)
		{
			debugRenderer->clearDebugData("irad");
			debugRenderer->beginRenderJob("irad");
		}
		float32 res = computeIntrinsicWaveDescriptor(tri, meshVertexIndex, maxGeoWaveDist);

		//debugOutput << res << "\n";
		rawValues[i]=res;

		if (showThisStep)
		{
			debugRenderer->endRenderJob();
			ObjectViewsTable::update(getScene());
			doScreenShot();
		}
	}

	//filter result
	float32 first = 0;
	for (mpcard i=0 ; i < 10; i++)
		first+= rawValues[i];
	first/=10.0f;
	DVectorF featureVec(15);
	featureVec[0] = first;

	for (mpcard i = 1; i < 15; i++)
	{
		float32 av = 0;
		for (mpcard j = 10*i-5; j < 10*i+5; j++)
			av += rawValues[j];
		av /= 10.0f;
		featureVec[i] = av;
	}
	// 		debugOutput << "\nfiltered\n";
	// 		for (mpcard i = 0; i < 15; i++)
	// 		{
	// 			debugOutput << featureVec[i] <<"\n";
	// 		}

	return featureVec;
}

X4::DVectorF X4::PCIMeshGeodesics::computeIntrinsicWaveSequenceDescriptorFast( UnstructuredInCoreTriangleMesh* tri, int32 startIndex , float scale /*= 1.0f*/ )
{

	std::vector<float32> rawValues(160);




	PointSet* ps = tri->getPointSet();
	AAT POS = ps->getAAT("position");

	if(!dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt")))
		tri->computeTopologyAttachment();


	TopologyAttachment* graphAtt = dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt"));
	InCorePCTopologyGraph* graph = graphAtt->getTopology();

	if (!graph->getEdges()->getDescr()->providesAttribute("edgeDistance"))
	{
		tri->computeTopologyAttachment();
		graphAtt = dynamic_cast<TopologyAttachment*>(tri->getAttachments()->getData("TopologyAtt"));
		graph = graphAtt->getTopology();
	}

	PointSet* edges = graph->getEdges();
	AAT EDIST = edges->getAAT("edgeDistance");
	std::vector<BorderEdge> outsidePoints;
	outsidePoints.reserve(50);		

	std::vector<float> distanceField(ps->getNumEntries(), 1e30f);

	for (mpcard circleNum = 0; circleNum < 160; circleNum++)
	{
		float32 intrinsicRadius = (0.05f + (float32)circleNum * 0.006f) * scale;//0.00125f;

		float32 lengthInPlane = (2*M_PI*intrinsicRadius);

		distanceField.clear();//!!!!
		distanceField.resize(ps->getNumEntries(), 1e30f);
		distanceField[startIndex] = 0;


		priority_queue<DistanceQueuePoint> growQueue;
		growQueue.push(DistanceQueuePoint(startIndex, startIndex, 0));

		outsidePoints.clear();

		while (!growQueue.empty() && growQueue.top().myGeoDist <= intrinsicRadius)
		{
			DistanceQueuePoint testPoint=growQueue.top();
			growQueue.pop();


			for (mpcard n = 0; n < graph->getNumAdjacentVertices(testPoint.index); n++)
			{
				// untouched neighbor vertex
				int32 neighborInd = graph->getAdjacentVertex(testPoint.index, n);
				//dist = last dist (of test point) + edge(testoint->neighbour) dist
				float32 dist = edges->get1f(graph->getEdgeIndex(testPoint.index, n), EDIST) + testPoint.myGeoDist;

				// dijkstra if smaller, then set and push
				if (dist < distanceField[neighborInd])
				{
					distanceField[neighborInd] = dist;
					growQueue.push(DistanceQueuePoint(neighborInd, testPoint.index ,dist));

					// if line on circle edge
					if (dist > intrinsicRadius && testPoint.myGeoDist < intrinsicRadius){
						outsidePoints.push_back(BorderEdge(neighborInd, testPoint.index, dist));
					}
				}
			}
		}


		sort(outsidePoints.begin(), outsidePoints.end());
		outsidePoints.erase(  unique(outsidePoints.begin(), outsidePoints.end()), outsidePoints.end());

		for (mpint op = outsidePoints.size()-1; op >= 0; op--)
		{
			if (distanceField[outsidePoints[op].outIndex]<intrinsicRadius)
			{
				outsidePoints.erase(outsidePoints.begin() + op);
			}else
			{
				Vector3f endPos = ps->get3f(outsidePoints[op].outIndex, POS);
				Vector3f startPos = ps->get3f(outsidePoints[op].cameFromIndex, POS);
				Vector3f direction = normalize( endPos - startPos ) ;
				float32 wayTooFar = outsidePoints[op].outsideDistance - intrinsicRadius;
				Vector3f finalPos = endPos - direction * wayTooFar;
				outsidePoints[op].exactBorderPosition = finalPos;
			}
		}

		std::vector<std::vector<int32> > borderNeighbors;
		borderNeighbors.resize(outsidePoints.size());
		for (mpint op = outsidePoints.size()-1; op >= 0; op--)
		{
			int32 oInd = outsidePoints[op].outIndex;
			for (mpcard n = 0; n < graph->getNumAdjacentVertices(oInd); n++)
			{
				int32 neighborInd = graph->getAdjacentVertex(oInd, n);
				BorderEdge dummy(neighborInd,-1,0);
				if (binary_search(outsidePoints.begin(), outsidePoints.end(), dummy))
				{
					borderNeighbors[op].push_back(neighborInd);
				}
			}
		}


		//get rid of wrong edges
		std::vector<MyPair> unallowedEdges;
		for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
		{
			Vector3f col = makeVector3f(0,1,0);
			float w =3;
			if (borderNeighbors[bn].size()!=2)
			{
				col = makeVector3f(1,0,0);
				w=5;

				std::vector<int32> myNeighbours = borderNeighbors[bn];
				for (mpcard cn = 0; cn < borderNeighbors.size(); cn++)
				{
					if (borderNeighbors[cn].size()!=2)
					{
						for (mpcard ne = 0; ne < myNeighbours.size(); ne++)
						{
							if (myNeighbours[ne]==outsidePoints[cn].outIndex)
							{
								unallowedEdges.push_back(MyPair(outsidePoints[bn].outIndex, borderNeighbors[bn][ne]));
							}
						}
					}
				}
			}
		}

		sort(unallowedEdges.begin(), unallowedEdges.end());
		unallowedEdges.erase(  unique(unallowedEdges.begin(), unallowedEdges.end()), unallowedEdges.end());
		sort(unallowedEdges.begin(), unallowedEdges.end());



		for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
		{
			if (borderNeighbors[bn].size()!=2)
			{
				for (mpint cn = borderNeighbors[bn].size()-1; cn >=0 ; cn--){
					MyPair test(outsidePoints[bn].outIndex, borderNeighbors[bn][cn]);
					if (binary_search(unallowedEdges.begin(), unallowedEdges.end(), test ) )
					{
						borderNeighbors[bn].erase(borderNeighbors[bn].begin() + cn);
					}
				}
			}
		}

		std::vector<int32> borderMap(ps->getNumEntries(), -1);
		for (mpcard i = 0; i<borderNeighbors.size(); i++)
			borderMap[outsidePoints[i].outIndex] = i;


		float32 lengthLowerBound = 0;
		for (mpcard bn = 0; bn < borderNeighbors.size(); bn++)
		{

			for (mpcard ne = 0; ne < borderNeighbors[bn].size(); ne++)
			{
				/*
				Vector3f col = makeVector3f(1,0,0);
				col = makeVector3f(0.333f,0.333f,1);
				debugRenderer->addLine(outsidePoints[bn].exactBorderPosition, outsidePoints[borderMap[borderNeighbors[bn][ne]]].exactBorderPosition, col, col, 4);*/

				lengthLowerBound += norm(outsidePoints[bn].exactBorderPosition - outsidePoints[borderMap[borderNeighbors[bn][ne]]].exactBorderPosition);

			}
		}

		lengthLowerBound *= 0.5f; // every part counted 2 times before!




		float32 res = lengthLowerBound / lengthInPlane;
		rawValues[circleNum]=res;

		// 		float32 nextIntrinsicRadius = (0.05f + (float32)(circleNum+1) * 0.006f) * scale;//0.00125f;
		// 		bool allInside = true;
		// 		for (mpcard k = 0; k < outsidePoints.size(); k++)
		// 		{
		// 			if ((outsidePoints[k].outsideDistance < nextIntrinsicRadius))
		// 				allInside = false;
		// 		}
		// 		debugOutput << allInside <<"\n";


	}

	//filter result
	float32 first = 0;
	for (mpcard i=0 ; i < 10; i++)
		first+= rawValues[i];
	first/=10.0f;
	DVectorF featureVec(15);
	featureVec[0] = first;

	for (mpcard i = 1; i < 15; i++)
	{
		float32 av = 0;
		for (mpcard j = 10*i-5; j < 10*i+5; j++)
			av += rawValues[j];
		av /= 10.0f;
		featureVec[i] = av;
	}
	// 		debugOutput << "\nfiltered\n";
	// 		for (mpcard i = 0; i < 15; i++)
	// 		{
	// 			debugOutput << featureVec[i] <<"\n";
	// 		}

	return featureVec;
}


std::vector<DVectorF> X4::PCIMeshGeodesics::computeIntrinsicWaveSequenceDescriptorForAll( UnstructuredInCoreTriangleMesh* tri, float scale /*= 1.0f*/, const std::vector<card32>& _selected)
{
	std::vector<DVectorF> allDescriptors(tri->getNumPoints());

	std::vector<card32> selected = _selected;

	// if empty selected array given, then compute for all
	if (selected.size() == 0)
	{
		selected.resize(tri->getNumPoints());
		for (mpcard i=0; i < tri->getNumPoints(); i++) selected[i] = i;
	}

	// we also only interested in points we have subsampled before

	progressWindow->pushStep(true, std::string("Find ") + intToStr(selected.size()) + std::string(" descriptors..."));

	for (mpcard meshVertexIndex = 0; meshVertexIndex < selected.size(); meshVertexIndex++)
	{
		if (meshVertexIndex%100 ==0)
		{
			progressWindow->progressf(float(meshVertexIndex) / float( selected.size() ) );
			debugOutput << meshVertexIndex << "\n";
			debugOutput.flush();
		}


		allDescriptors[selected[meshVertexIndex]]= computeIntrinsicWaveSequenceDescriptorFast(tri, selected[meshVertexIndex], scale) ;
	}
	progressWindow->popSteps();

	return allDescriptors;
}

void X4::PCIMeshGeodesics::testShowDescriptor()
{
	UnstructuredInCoreTriangleMesh* umf = dynamic_cast< UnstructuredInCoreTriangleMesh*> ( getPointCloud(getScene(), "root/tri/pc1") );

	if (selectedVertex < 0 || !umf)
		return;

	debugRenderer->beginRenderJob_OneFrame("iwd", 0);
	VertexArray vert(umf);
	debugRenderer->addFastSphere(vert.getPosition3f(selectedVertex), 0.003f,  makeVector3f(0.333f,0.333f,1));
	debugRenderer->endRenderJob();


	DVectorF result = computeIntrinsicWaveSequenceDescriptor(umf, selectedVertex, maxGeoDist);
	//DVectorF result(1); result[0] = computeIntrinsicWaveDescriptor(umf, selectedVertex, maxGeoDist);



	debugOutput << result << " = computeIntrinsicWaveDescriptor\n";





}

void X4::PCIMeshGeodesics::theOtherTest()
{
	UnstructuredInCoreTriangleMesh* umf = dynamic_cast< UnstructuredInCoreTriangleMesh*> ( getPointCloud(getScene(), "root/pc1") );



	std::vector<DVectorF> descriptors = IntrinsicWaveDescriptor::getWaveSignaturesForAll(0.123f, umf );
	// 	std::vector<DVectorF> descriptors(1);
	// 	descriptors[0] = computeIntrinsicWaveSequenceDescriptorFast(umf, 0, 0.123f);



	AttachedFloatSignatures att;
	att.descriptors = descriptors;

	AttachedFloatSignatures oA = att;
	BinaryOutputObjectStream* out = new BinaryOutputObjectStream("c:/a/temp.tmp");
	oA.write(out);
	delete out;
	BinaryInputObjectStream in("c:/a/temp.tmp");
	AttachedFloatSignatures readA;
	readA.read(&in);

	descriptors = readA.descriptors;

	DVectorF waaa = computeIntrinsicWaveSequenceDescriptor(umf, (int32)descriptors.size()-1, 0.123f);


	for (mpcard k = 0; k < descriptors.size(); k++)
		debugOutput << descriptors[k] << "\n";

	debugOutput << waaa << "\n";
	debugOutput << norm(descriptors[descriptors.size()-1]-waaa) << "\n";


}

void X4::PCIMeshGeodesics::descriptorPCA()
{
	UnstructuredInCoreTriangleMesh* meshA = dynamic_cast<UnstructuredInCoreTriangleMesh*>(getPointCloud(scene, "root/tri/pc1")) ;
	if (!meshA)
		throw X4Exception( "PCIEntropyMatching::descriptorDistance: mesh not found/dynamic_cast didn't work...\n");


	if (cachedDescriptors.empty())
	{

		AttachedFloatSignatures* sigs = dynamic_cast<AttachedFloatSignatures*>(meshA->getAttachments()->getData("AttachedFloatSignatures"));
		if (sigs)
		{
			cachedDescriptors = sigs->descriptors;
			debugOutput << "loaded precomputed descriptors..\n";
			debugOutput.flush();
		}else
			throw X4Exception("Please precompute descriptors first.\n");
	}

	PCAGeneral pca;
	pca.calculatePCA(cachedDescriptors);
	cachedDescriptors = pca.getPCAWeightsForAllInputInstances();
	debugOutput << "reduced to " << cachedDescriptors[0].getDim() << "-dim.\n";

}

void X4::PCIMeshGeodesics::importTOSCA()
{
	// parameters:
	std::string folderName = "c:/toscaHi";
	std::string targetNodeName = "root/tri";




	SceneGraphNode *node = getSceneGraphNode(scene, targetNodeName);
	if (!node) {
		SGRelativeTimeAnimationNode *nn = new SGRelativeTimeAnimationNode();
		nn->setName(extractSGNode(targetNodeName));
		addSceneGraphNode(scene, extractSGPath(targetNodeName), nn);
	}

	QDir myDir(folderName.c_str());
	QStringList wantedFiles("*.vert");
	QStringList list = myDir.entryList(wantedFiles);

	debugOutput << list.size() << " Files found:\n";

	for (mpcard i = 0; i < (mpcard)list.size(); i++)
	{
		QString name = list[i];
		QStringList nameList = name.split(".");
		QString otherName = nameList[0] + QString(".tri");

		std::string nameVert =  list[i].toLatin1().constData(); 
		std::string nameTri  =  otherName.toLatin1().constData(); 
		nameVert = folderName + "/" + nameVert;
		nameTri  = folderName + "/" + nameTri;

		debugOutput << "loading: " << nameVert<< " - "<< nameTri << "\n";
		debugOutput.flush();

		std::ifstream in;
		in.clear();
		in.open(nameVert.c_str(),std::ifstream::in);
		if (!in.good()) { 
			warning("importTOSCA\nError opening file \"" + nameVert + "\"!"); 
			return ;
		};

		std::ifstream inTri;
		inTri.clear();
		inTri.open(nameTri.c_str(),std::ifstream::in);
		if (!inTri.good()) { 
			warning("importTOSCA\nError opening file \"" + nameTri + "\"!"); 
			return ;
		};


		QString line;

		std::vector<Vector3f> positions;
		std::vector<Vector3i> faces;


		while (!in.eof())
		{
			char _buffer[1024];
			in.getline(_buffer, 1024);
			line = QString::fromLatin1(&_buffer[0]);

			// Read vertices
			QStringList strList = line.split( " ", QString::SkipEmptyParts );
			if ( strList.size() == 3 ){
				Vector3f p = makeVector3f( strList.at( 0 ).toFloat(),
					strList.at( 1 ).toFloat(),
					strList.at( 2 ).toFloat() );
				positions.push_back( p );
			}
		}
		in.close();



		while (!inTri.eof())
		{
			char _buffer[1024];
			inTri.getline(_buffer, 1024);
			line = QString::fromLatin1(&_buffer[0]);

			// Read faces
			QStringList strList = line.split( " ", QString::SkipEmptyParts );
			if ( !line.isEmpty() ){
				// Only support triangle meshes
				if ( strList.at( 0 ).toInt() == 3 || strList.size() == 3)
				{
					int base = 1;
					if (strList.size() == 3) base = 0;

					Vector3i face = makeVector3i( strList.at( base + 0 ).toInt(), 
						strList.at( base + 1 ).toInt(),
						strList.at( base + 2 ).toInt());


					face[0] = face[0] - 1;
					face[1] = face[1] - 1;
					face[2] = face[2] - 1;


					faces.push_back( face );
				}
			}
		}

		inTri.close();


		UnstructuredInCoreTriangleMesh* mesh = new UnstructuredInCoreTriangleMesh();
		mesh->clearAndSetupDefault(positions.size(), faces.size());
		VertexArray vert(mesh);
		for (mpcard p = 0; p < positions.size(); p++)
		{
			vert.setPosition3f(p, positions[p]);
		}
		VertexArray tri(mesh->getTriangles());
		for (mpcard t = 0; t < faces.size(); t++)
		{
			tri.setIndex3i(t, faces[t]);
		}

		mesh->setMaterialIndex(3);

		addPointCloud(scene, mesh, targetNodeName + "/" + nameList[0].toLatin1().constData() );
	}







}

struct QueueEntry 
{
	mpcard index, lastIndex;
};

void X4::PCIMeshGeodesics::fixNormals( mpcard startPoint, UnstructuredInCoreTriangleMesh* mesh )
{

	TopologyAttachment* tatt = dynamic_cast<TopologyAttachment*>(mesh->getAttachments()->getData(TopologyAttachment::getDefaultName()));
	InCorePCTopologyGraph* tpg = tatt->getTopology();
	std::vector<bool> visited(mesh->getNumPoints(), false);

	VertexArray vert(mesh);

	queue<QueueEntry> neighbours;

	QueueEntry start;
	start.index=startPoint; start.lastIndex = startPoint;

	neighbours.push(start);

	while(!neighbours.empty()){
		QueueEntry currentVertex = neighbours.front();
		neighbours.pop();
		if (visited[currentVertex.index])
			continue;
		visited[currentVertex.index] = true;
		if (vert.getNormal3f(currentVertex.index) * vert.getNormal3f(currentVertex.lastIndex) > 0.5f)
		{
			vert.setFlags(currentVertex.index, PF_FLAG_SELECTED);
			mpcard numNN= tpg->getNumAdjacentVertices(currentVertex.index);
			for (mpcard i = 0; i < numNN; i++)
			{
				mpcard nInd = tpg->getAdjacentVertex(currentVertex.index, i);
				if (visited[nInd])
					continue;
				QueueEntry next;
				next.index = nInd;
				next.lastIndex = currentVertex.index;
				neighbours.push(next);
			}
		}
	}



	scene->rebuildRenderObjects();
	ObjectViewsTable::update(scene);


}

void X4::PCIMeshGeodesics::doScreenShot()
{
	SceneEditorWidget* sceneEditorW = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
	sceneEditorW->saveHighResFrame();
}

void X4::PCIMeshGeodesics::importHannover()
{
	// parameters:
	std::string folderName = "I:\\aberner\\kirche\\Marktkirche";
	std::string targetNodeName = "root/tri";




	SceneGraphNode *node = getSceneGraphNode(scene, targetNodeName);
	if (!node) {
		SGRelativeTimeAnimationNode *nn = new SGRelativeTimeAnimationNode();
		nn->setName(extractSGNode(targetNodeName));
		addSceneGraphNode(scene, extractSGPath(targetNodeName), nn);
	}

	QDir myDir(folderName.c_str());
	QStringList wantedFiles("*.asc");
	QStringList list = myDir.entryList(wantedFiles);

	debugOutput << list.size() << " Files found:\n";

	for (mpcard i = 0; i < (mpcard)list.size(); i++)
	{
		QString name = list[i];
		QStringList nameList = name.split(".");
		QString otherName; 
		if (i+1 < 10)
			otherName = "ScanPos0" + QString(intToStr(i+1).c_str() ) +QString(".DAT");
		else
			otherName = "ScanPos" + QString(intToStr(i+1).c_str() ) +QString(".DAT");


		std::string nameVert =  list[i].toLatin1().constData(); 
		std::string nameTri  =  otherName.toLatin1().constData(); 
		nameVert = folderName + "/" + nameVert;
		nameTri  = folderName + "/" + nameTri;

		debugOutput << "loading: " << nameVert<< " - "<< nameTri << "\n";
		debugOutput.flush();

		std::ifstream in;
		in.clear();
		in.open(nameVert.c_str(),std::ifstream::in);
		if (!in.good()) { 
			warning("importHannover\nError opening file \"" + nameVert + "\"!"); 
			return ;
		};

		std::ifstream inTri;
		inTri.clear();
		inTri.open(nameTri.c_str(),std::ifstream::in);
		if (!inTri.good()) { 
			warning("importHannover\nError opening file \"" + nameTri + "\"!"); 
			return ;
		};


		QString line;

		std::vector<Vector3f> positions;
		std::vector<float32>  intensities;


		while (!in.eof())
		{
			char _buffer[1024];
			in.getline(_buffer, 1024);
			line = QString::fromLatin1(&_buffer[0]);

			// Read vertices
			QStringList strList = line.split( ", ", QString::SkipEmptyParts );
			if ( strList.size() == 4 ){
				Vector3f p = makeVector3f( strList.at( 0 ).toFloat(),
					strList.at( 1 ).toFloat(),
					strList.at( 2 ).toFloat() );
				float32 intens = strList.at( 3 ).toFloat();
				positions.push_back( p );
				intensities.push_back(intens);
			}
		}
		in.close();


		Matrix4f trans;
		mpcard at = 0;

		while (!inTri.eof())
		{
			char _buffer[1024];
			inTri.getline(_buffer, 1024);
			line = QString::fromLatin1(&_buffer[0]);

			// Read faces
			QStringList strList = line.split( " ", QString::SkipEmptyParts );
			if ( !line.isEmpty() ){
				// Only support triangle meshes
				if (  strList.size() == 4)
				{
					int base = 0;

					Vector4f line = makeVector4f( strList.at( base + 0 ).toFloat(), 
						strList.at( base + 1 ).toFloat(),
						strList.at( base + 2 ).toFloat(),
						strList.at( base + 3 ).toFloat()
						);

					trans[at++] = line;
				}
			}
		}

		inTri.close();

		trans = trans.transpose();


		UnstructuredInCorePointCloud* mesh = new UnstructuredInCorePointCloud();
		mesh->clearAndSetup(positions.size());
		checkAttribute(mesh, "intensity", 1, VAD::DATA_FORMAT_FLOAT32);
		VertexArray vert(mesh);
		PointSet* ps = mesh->getPointSet();
		AAT INT = ps->getAAT("intensity");
		for (mpcard p = 0; p < positions.size(); p++)
		{
			vert.setPosition3f(p, positions[p]);
			ps->set1f(p, INT, intensities[p]);
		}

		mesh->setTransformation(trans);

		mesh->setMaterialIndex(0);

		addPointCloud(scene, mesh, targetNodeName + "/" + nameList[0].toLatin1().constData() );
	}



}

void X4::PCIMeshGeodesics::maxCurv()
{
	UnstructuredInCorePointCloud* pc = (UnstructuredInCorePointCloud*)getPointCloud(scene, "root/pc1");

	FastSphereQuerry query(pc);
	PointSet* ps = pc->getPointSet();

	mpcard * indices;
	mpcard numPoints;
	AAT POS = ps->getAAT("position");
	AAT CURV = ps->getAAT("curv2");
	AAT TANGENT = ps->getAAT("tangent_u");
	AAT TANGENTV = ps->getAAT("tangent_v");

	debugRenderer->clearDebugData("df");
	debugRenderer->beginRenderJob("df");

	UnstructuredInCorePointCloud * newPC = new UnstructuredInCorePointCloud();
	newPC->clearAndSetup(pc->getDescr(), 0);


	signed int curvSign = 1;	
signSwitch:

	for(mpcard i = 0; i< pc->getNumPoints(); i++){
		if (curvSign * ps->get2f(i, CURV)[0] < 0.0f)
			continue;

		float32 max = fabs(ps->get2f(i, CURV)[0]);
		if (max < absoluteCurvThreshold)
			continue;

		query.querry(ps->get3f(i, POS), maxGeoDist, &indices, numPoints);

		/*mpcard bestIndex = i;
		for (mpcard j = 0; j < numPoints; j++)
		{
		float32 myCurv = fabs(ps->get2f(indices[j], CURV)[0]);
		if (curvSign * ps->get2f(indices[j], CURV)[0] < 0.0f)
		continue;
		const float32 angleThreshold = 0.7;
		const float32 angleMatch = fabs(normalize(ps->get3f(i, TANGENT)) * normalize(ps->get3f(indices[j], TANGENT)));
		if (myCurv > max && angleMatch > angleThreshold)
		{
		max = myCurv;
		bestIndex = indices[j];
		}
		}*/
		Vector3f bestPosition;
		Vector3f optTangent;
		bestPosition.setZero();
		optTangent.setZero();
		float32 weightSum = 0.0f;
		for (mpcard j = 0; j < numPoints; j++)
		{
			float32 myCurv = fabs(ps->get2f(indices[j], CURV)[0]);
			if (curvSign * ps->get2f(indices[j], CURV)[0] < 0.0f)
				continue;
			const float32 angle = fabs(normalize(ps->get3f(i, TANGENTV)) * normalize(ps->get3f(indices[j], TANGENTV)));
			weightSum += pow(myCurv, curvPower) * pow(angle, anglePower);
			bestPosition += ps->get3f(indices[j], POS) * pow(myCurv, curvPower) * pow(angle, anglePower);
			optTangent += ps->get3f(indices[j], TANGENTV) * pow(myCurv, curvPower) * pow(angle, anglePower);
		}

		bestPosition /= weightSum;
		optTangent /= weightSum;

		if (weightSum > weightSumThreshold)
		{
			//Vector3f translation = ps->get3f(i, POS) - ps->get3f(bestIndex, POS);
			Vector3f translation = ps->get3f(i, POS) - bestPosition;
			Vector3f tangent_u = normalize(ps->get3f(i, TANGENT));
			translation = tangent_u * (translation * tangent_u);

			Vector3f optimizedPosition = ps->get3f(i, POS) - translation;

			newPC->getPointSet()->addPoint(ps->getDataPointer(i));
			//newPC->getPointSet()->addPoint(ps->getDataPointer(bestIndex));
			newPC->getPointSet()->set3f(newPC->getNumPoints()-1, POS, optimizedPosition);
			newPC->getPointSet()->set3f(newPC->getNumPoints()-1, TANGENTV, optTangent);


			Vector3f color;
			if (curvSign > 0.0f)
				color = makeVector3f(0,1,0);
			else
				color = makeVector3f(1,0,0);
			//debugRenderer->addFastSphere(optimizedPosition, 0.03f, color, false);
			//debugRenderer->addLine(optimizedPosition - normalize(ps->get3f(bestIndex, TANGENTV)) * 0.08f, optimizedPosition + normalize(ps->get3f(bestIndex, TANGENTV)) * 0.08f, color, color, 1);
		}

	}

	if (curvSign > 0)
	{
		curvSign *= -1;
		goto signSwitch;
	}

	addOrReplacePointCloud(scene, "root/optPC", newPC);

	debugRenderer->endRenderJob();


}

void X4::PCIMeshGeodesics::filterOpt()
{
	UnstructuredInCorePointCloud* pc = (UnstructuredInCorePointCloud*)getPointCloud(scene, "root/optPC");

	FastSphereQuerry query(pc);
	PointSet* ps = pc->getPointSet();

	mpcard * indices;
	mpcard numPoints;
	AAT POS = ps->getAAT("position");
	AAT CURV = ps->getAAT("curv2");
	AAT TANGENT = ps->getAAT("tangent_u");
	AAT TANGENTV = ps->getAAT("tangent_v");

	UnstructuredInCorePointCloud * newPC = new UnstructuredInCorePointCloud();
	newPC->clearAndSetup(pc->getDescr(), 0);


	signed int curvSign = 1;	
signSwitch:

	for(mpcard i = 0; i< pc->getNumPoints(); i++){
		if (curvSign * ps->get2f(i, CURV)[0] < 0.0f)
			continue;

		query.querry(ps->get3f(i, POS), maxGeoDist, &indices, numPoints);

		unsigned int matchCount = 0;
		for (mpcard j = 0; j < numPoints; j++)
		{

			if (curvSign * ps->get2f(indices[j], CURV)[0] < 0.0f)
				continue;

			float32 match = fabs(normalize(ps->get3f(i, TANGENT)) * normalize(ps->get3f(indices[j], TANGENT)));
			if (match > angleTol)
				matchCount++;
		}


		if (matchCount > minFaces)
			newPC->getPointSet()->addPoint(ps->getDataPointer(i));
	}

	if (curvSign > 0)
	{
		curvSign *= -1;
		goto signSwitch;
	}


	newPC->setMaterialIndex(7);
	addOrReplacePointCloud(scene, "root/filteredOptPC", newPC);

	ObjectViewsTable::update(scene);
}

void X4::PCIMeshGeodesics::precomputeAllDescriptors()
{
	UnstructuredInCoreTriangleMesh* mesh = (UnstructuredInCoreTriangleMesh* 	) getPointCloud(getScene(), "root/pc1");


	mpcard numVert = mesh->getNumPoints();

	std::vector<DVectorF> descriptors = IntrinsicWaveDescriptor::getWaveSignaturesForAll(maxGeoDist, mesh);


	AttachedFloatSignatures* sigs = new AttachedFloatSignatures();
	sigs->descriptors = descriptors;


	sigs->setup("AttachedFloatSignatures", AttachedData::ADF_PERSISTENT);

	mesh->getAttachments()->attachData(sigs);
}