#ifndef PCIMeshGeodesicsH
#define PCIMeshGeodesicsH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------

namespace X4
{
class MeshGeodesics;
class UnstructuredInCoreTriangleMesh;
/**
	plz comment your class
*/
class X3FOUNDATION_PRIVATE_API PCIMeshGeodesics : public PCInteractionTool
{
	X4_CLASS( PCIMeshGeodesics )

private:
	mpcard numPoints;
	MeshGeodesics* mg;
	std::vector<Vector3f> lastPath;
	card32 minFaces;
	float32 maxGeoDist;
	bool checkPoint;
	std::vector<std::vector<Vector3f> > paths;

	float32 angleTol;
	float32 anglePower;
	float32 weightSumThreshold;
	float32 curvPower;
	float32 absoluteCurvThreshold;

	mpcard useFeature;
	mpint selectedVertex;

	Vector3f getPosOnTheWay(float32 wherePos, const std::vector<Vector3f> &way);
	bool pickPoint(int32 x, int32 y, PointCloud *pc, mpcard &index);


	std::vector<DVectorF> cachedDescriptors;

	void fixNormals(mpcard startPoint, UnstructuredInCoreTriangleMesh* mesh);
	void doScreenShot();


public:
	PCIMeshGeodesics(void);
	~PCIMeshGeodesics(void);



   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y){}
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState){}
   virtual void areaResize(card32 width, card32 height){}

   virtual void keyDown(GeneralKey key){}
   virtual void keyUp(GeneralKey key){}


   virtual void glDrawTool(GLContext *glContext);

   void testGeo();
   void initGeo();
   void initBronstein();
   void calcBronsteinFeatures();
   void testGeodesicDescriptor();

   void initGeo(UnstructuredInCoreTriangleMesh* tri);
   float32 computeDistance (card32 ptA, card32 ptB, std::vector<Vector3f>* path = NULL);
   static float32 computeIntrinsicWaveDescriptor( UnstructuredInCoreTriangleMesh* tri, int32 meshVertexIndex, float32 intrinsicRadius );
   DVectorF computeIntrinsicWaveSequenceDescriptor( UnstructuredInCoreTriangleMesh* tri, int32 meshVertexIndex , float scale = 1.0f);
   static DVectorF computeIntrinsicWaveSequenceDescriptorFast( UnstructuredInCoreTriangleMesh* tri, int32 meshVertexIndex , float scale = 1.0f);
   static std::vector<DVectorF> computeIntrinsicWaveSequenceDescriptorForAll( UnstructuredInCoreTriangleMesh* tri, float scale = 1.0f, const std::vector<card32>& selected = std::vector<card32>());
   void testExactGeodesicDescriptor();
   void calculateWaveSequenceDescriptor();

   static float32 computeDistanceB (UnstructuredInCoreTriangleMesh* tri, card32 ptA, card32 ptB);

   void testShowDescriptor();
   void theOtherTest();
   void descriptorPCA();
   void importTOSCA();
   void importHannover();
   void maxCurv();
   void filterOpt();
   void precomputeAllDescriptors();


};

}

#endif