//---------------------------------------------------------------------------
#ifndef PCIPointCloudProcessingH
#define PCIPointCloudProcessingH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "IteratorAdaptors.h"
#include "LinearAlgebra.h"
#include "DebugOutput.h"
#include "PointCloudProcessingRenderer.h"
#include "PointCloudAttributes.h"
//---------------------------------------------------------------------------

namespace X4 {

class X3FOUNDATION_PRIVATE_API PCIPointCloudProcessing : public PCInteractionTool {

	X4_CLASS( PCIPointCloudProcessing )

	private:
		std::string pcName;
		bool usePointSet;
		PointCloudProcessingRenderer* renderer;
		PointCloudAttributes* checkAtt;

		// Influence settings
		card32 knn;
		float32 epsilon; 
		float32 timesSamplingSpacing;
		card32 mode;

		// MeshVertex
		card32 mcRes;
		card32 meshDistanceMode;

		// Tensor Voting
		bool saveTensors;

		void samplingSpacing();
		void normalsPCA();
		void unifyNormals();
		void unifyNormalsHoppe();
		void estimateTangents();
		void fitMonomial();
		void estimateCurvature();
		void mlsSmooth();
		void hoppeTriangulate();
		void tensorVoteInit();
		void tensorVote();
		
		void connectToSceneImpl(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget, QWidget* toolWidget);
	public:
		PCIPointCloudProcessing();
		~PCIPointCloudProcessing();

		/// PCInteractable
		void glDrawTool( GLContext *glContext );
		void redraw(){ redrawScene(); }
		void mouseDown( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState ){}
		void mouseMoved( int32 x, int32 y ){}
		void mouseUp( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState ){}
		void areaResize( card32 width, card32 height ){}
		void keyDown( GeneralKey key ){}
		void keyUp( GeneralKey key );
};
}

#endif

