#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIPoissonReconstruction.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ClassMethods.h"
#include "UnstructuredInCorePointCloud.h"
#include "PointCloudTools.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "VertexArray.h"
#include "HierarchicalKNNIterator.h"
#include "ObjectViewsTable.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCIPoissonReconstruction ,0)
{
	BEGIN_CLASS_INIT( PCIPoissonReconstruction );
	ADD_STRING_PROP(dataPC, 0)
	ADD_STRING_PROP(reconstructedPC, 0)
	ADD_FLOAT32_PROP(maxDistToData, 0)
	ADD_NOARGS_METHOD(removeUnsupportetPoints)
}

PCIPoissonReconstruction::PCIPoissonReconstruction(void)
{
	dataPC = "root/pc1";
	reconstructedPC = "root/pc2";
	maxDistToData = 0.006f;
}

PCIPoissonReconstruction::~PCIPoissonReconstruction(void)
{
}

void PCIPoissonReconstruction::assign(const Object* obj, X4CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCIPoissonReconstruction * o = dynamic_cast<const PCIPoissonReconstruction*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::PCIPoissonReconstruction::removeUnsupportetPoints()
{
	UnstructuredInCorePointCloud* dataCloud = dynamic_cast<UnstructuredInCorePointCloud*> (getPointCloud(scene, dataPC));
	UnstructuredInCoreTriangleMesh* recMesh = dynamic_cast<UnstructuredInCoreTriangleMesh*> (getPointCloud(scene, reconstructedPC));

	if (!dataCloud || ! recMesh)
	{
		debugOutput << "PCIPoissonReconstruction: pc not found\n";
		return;
	}

	VertexArray* recArray = new VertexArray(recMesh);
	mpcard numRecPts = recMesh->getNumPoints();
	HierarchicalKNNIterator dataKnn(dataCloud, 32, NULL);
	AAT dataPOS = dataCloud->getAAT("position");


	for (mpcard i = 0; i < numRecPts; i++)
	{
		Vector3f recPos = recArray->getPosition3f(i);
		dataKnn.setSeekPointAndReset(recPos);
		Vector3f nearestData = dataKnn.get3f(dataPOS);
		if (norm(recPos - nearestData) > maxDistToData)
			recArray->setFlags(i, PF_FLAG_SELECTED);
		else
			recArray->setFlags(i, 0);	
	}

	delete recArray;
	scene->rebuildRenderObjects();
	ObjectViewsTable::update(scene);

}