#include "StdAfx.h"
#include <windows.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#include "PCIRigidRegistration.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "MultiScaleFeatureDetector.h"
#include "GeometricFeature.h"
#include "ObjectClassProperty.h"
#include "scene.h"
#include "DebugRenderer.h"
#include "PointCloudTools.h"
#include "ClassMethods.h"
#include "SceneGraphState.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "ProgressWindow.h"
#include "FeatureMatching.h"
#include "FeatureCorrespondence.h"
#include "SGAnimationNode.h"
#include "SeparatorClassProperty.h"
#include "RenderingTool.h"
#include "LDCentroidDescriptor.h"
#include "LocalICP.h"
#include "Random.h"
#include "GeometricTools.h"
#include "LogOutput.h"
#include "SlippageFeatureDetector.h"
#include "StringHelper.h"
#include "Quaternion.h"
#include "LDCorrelation.h"
#include "CachedRangeQuerry.h"
#include "FeatureClustering.h"
#include "LDHistogram.h"
#include "VertexArray.h"
#include <QtGui/QFileDialog>
#include "StringHelper.h"
//---------------------------------------------------------------------------
using namespace X4;


std::map<unsigned,unsigned> colortable;

std::map<unsigned,Vector3f> betterPos;
unsigned currentColorIndex = 0;

IMPLEMENT_X4_CLASS( PCIRigidRegistration ,0)
{
	BEGIN_CLASS_INIT( PCIRigidRegistration );

	ADD_OBJECT_PROP( m_FeatureMatching, 0, FeatureMatching::getClass(), true);
	ADD_BOOLEAN_PROP(m_ValidateCorrespondences,0);
	SET_USER_EDIT(false);
	ADD_OBJECT_PROP( m_icp, 0, LocalICP::getClass(), true);
	SET_USER_EDIT(false);
	ADD_BOOLEAN_PROP(m_TestICPClustering,0);
	SET_USER_EDIT(false);
	ADD_FLOAT32_PROP(m_MaxICPResidual,0);
	SET_USER_EDIT(false);
	ADD_BOOLEAN_PROP(m_DeleteOnBurn,0);
	SET_USER_EDIT(false);
	ADD_OBJECT_PROP( m_LastCorrSet,0, FeatureCorrespondenceSet::getClass(), true );
	SET_USER_EDIT(false);

	ADD_NOARGS_METHOD(computeFeaturesOneFrame);
//	ADD_NOARGS_METHOD(computeCorrespondencesOneFrame);
//	ADD_NOARGS_METHOD(computeDeformCorresp);
	//ADD_NOARGS_METHOD(filterFeaturesOneFrame);
	//ADD_NOARGS_METHOD(resetTransformations);
	//ADD_NOARGS_METHOD(icp);
	//ADD_NOARGS_METHOD(icp_setCloud0);
	//ADD_NOARGS_METHOD(setTransformationFromString);
	//ADD_NOARGS_METHOD(testFeatureClustering);
//	ADD_NOARGS_METHOD(normalizeScene);
//	ADD_NOARGS_METHOD(removeOverlappingFeatures);

	ADD_SEPARATOR("rendering");
	ADD_BOOLEAN_PROP(m_ShowFeaturePoints,0);
	ADD_FLOAT32_PROP(m_FeatureSize,0);
	ADD_BOOLEAN_PROP(m_VisualizeICP,0);
	SET_USER_EDIT(false);

//	ADD_NOARGS_METHOD(precomputeFeatures);
//	ADD_NOARGS_METHOD(recomputeSignatures);
//	ADD_NOARGS_METHOD(featureBurning);
//	ADD_NOARGS_METHOD(orientateFeatureNormals);
//	ADD_NOARGS_METHOD(computeCorrespondences);
//	ADD_NOARGS_METHOD(recomputeGeodeticDistances);
//	ADD_NOARGS_METHOD(alignClouds);
//	ADD_NOARGS_METHOD(automaticAlignment);
//	ADD_NOARGS_METHOD(testDetection);
	
//	ADD_NOARGS_METHOD(testSmoothing);
	ADD_NOARGS_METHOD(exportFeaturePoints);
}

PCIRigidRegistration::PCIRigidRegistration(void)
{
	m_MaxICPResidual = 3.0f;
	m_FeatureMatching = new FeatureMatching;
	m_ValidateCorrespondences = false;
	m_ShowFeaturePoints = true;
	m_FeatureSize = 1.5f;
	m_RenderingTool = new RenderingTool;
	m_RenderingTool->init();
	m_SelectedFeaturePoint1 = NULL_VECTOR3F;
	m_icp = new LocalICP;
	m_TestICPClustering = false;
	m_VisualizeICP = false;
	m_DeleteOnBurn = false;
	m_LastCorrSet = new FeatureCorrespondenceSet;
}

PCIRigidRegistration::~PCIRigidRegistration(void)
{
	delete m_FeatureMatching;
	delete m_RenderingTool;
	delete m_icp;
	delete m_LastCorrSet;
}

void PCIRigidRegistration::assign(const Object* obj, X4CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCIRigidRegistration * o = dynamic_cast<const PCIRigidRegistration*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::PCIRigidRegistration::computeFeaturesOneFrame()
{
	std::vector<UnstructuredInCorePointCloud*> pcs = getAllPointClouds();
	if( pcs.size() == 0 )
		return;
	PointCloud * pc = pcs[0];
	if( pc == NULL )
		return;


	std::vector<UnstructuredInCorePointCloud*> clouds;
	clouds.push_back( (UnstructuredInCorePointCloud*)pc);
	FeaturePointDetector * detector = m_FeatureMatching->getFeatureDetector();
	SlippageFeatureDetector * slippage = dynamic_cast<SlippageFeatureDetector*>(detector);
	if( slippage )
		slippage->sceneHack = getScene();

	detector->begin(clouds);
	FeatureSet * fs = detector->detect((UnstructuredInCorePointCloud*)pc);
	detector->end();

	float radius = getMedianPointDistance(pc)*1.5f;
	debugRenderer->clearDebugData("features");
	debugRenderer->beginRenderJob("features",false);
	for( unsigned i=0;i<fs->getNumFeatures();i++ )
	{
		debugRenderer->addFastSphere(fs->m_Features[i]->m_Position, radius, makeVector3f(1,0.2f,0),false);
	}
	debugRenderer->endRenderJob();

}

void X4::PCIRigidRegistration::computeCorrespondencesOneFrame()
{
	std::vector<PointCloud*> clouds;
	const std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	for( card32 i=0;i<selection.size();i++ )
	{
		PointCloud * pc = getPointCloud( getScene(), selection[i] );
		if( !pc )
			continue;

		if( pc->getAttachments()->getData(FeatureSet::getDefaultName()) == NULL )
			return;

		clouds.push_back(pc);
	}

	if( clouds.size() != 2 )
		return;

	FeatureCorrespondenceSet * set = computeFeatureCorrespondences( clouds[0],clouds[1]);

	debugRenderer->clearDebugData("correspondence-test");
	debugRenderer->beginRenderJob("correspondence-test");
	
	for( unsigned i=0;i<set->m_Correspondences.size();i++ )
	{
		debugRenderer->addLine( transformVector3f(clouds[0]->getTransformation(), set->m_Correspondences[i]->m_FeaturePoint[0]),
			transformVector3f(clouds[1]->getTransformation(), set->m_Correspondences[i]->m_FeaturePoint[1]), makeVector3f(0.8f,0.3f,0),makeVector3f(0.9f,0.2f,0),1.0f);
	}

	delete set;

	debugRenderer->endRenderJob();
}

PointCloud * X4::PCIRigidRegistration::getSelectedPointCloud()
{
	const std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	for( card32 i=0;i<selection.size();i++ )
	{
		PointCloud * pc = getPointCloud( getScene(), selection[i] );
		if( pc )
			return pc;
	}
	return NULL;
}

void X4::PCIRigidRegistration::glDrawTool( GLContext * context )
{
	Vector3f color = makeVector3f(255,174,0) / 255.0f;
	static std::vector<Vector3f> colors;
	if( colors.size() == 0)
		for( unsigned i=0;i<100;i++ )
			colors.push_back(makeVector3f(rnd01(),rnd01(),rnd01()));

//	debugRenderer->renderJob(context, getScene(), "features" );
	StandardSceneGraphIterator sIt;
	sIt.clearAndSetup( getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering);
	while( !sIt.atEnd())
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sIt.getNode());
		if( objNode )
		{
			SceneObject * so = objNode->getSceneObject();
			Matrix4f T = so->getTransformation();
			if( m_ShowFeaturePoints )
			{
				FeatureSet * fs = dynamic_cast<FeatureSet*>(so->getAttachments()->getData(FeatureSet::getDefaultName()) );
				if( fs )
				{
					m_RenderingTool->setRenderColor(0.9f,0.5f,0.1f);
					UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(so);
					float medPointDistance = getCachedMedianPointDistance(upc);
					for( unsigned i=0;i<fs->m_Features.size();i++ )
					{
						GeometricFeature * f = fs->m_Features[i];
						SlippageSignature * slipSig = dynamic_cast<SlippageSignature*>(f->m_Signatures[0]);
						if( slipSig )
						{
							/*if( slipSig->m_FeatureWeight < 1.5f )
								m_RenderingTool->setRenderColor(0,0.5f,1);
							else
								if( slipSig->m_FeatureWeight < 2.5f)
									m_RenderingTool->setRenderColor(0,1.0f,0.5f);
								else
									if( slipSig->m_FeatureWeight < 3.5f)
										m_RenderingTool->setRenderColor(1,0.5f,0.0f);
									else
										m_RenderingTool->setRenderColor(1,1,0.2f);*/
						}
						bool badFeature = false;
						for( unsigned j=0;j<f->m_Signatures.size() && j<m_FeatureMatching->getFeatureDetector()->getNumLocalDescriptors();j++ )
							badFeature |= f->m_Signatures[j]->badFeature( m_FeatureMatching->getFeatureDetector()->getLocalDescriptor(j) );
						if( !badFeature)
						{
							Vector3f fPos = transformVector3f(T, f->m_Position);
							if( colortable.find(i) == colortable.end() )
							{
								m_RenderingTool->setRenderColor( color );
								m_RenderingTool->renderSphere(fPos, medPointDistance * m_FeatureSize);
							}
							else
							{
								m_RenderingTool->setRenderColor( colors[colortable[i]]);
								if( betterPos.find(i) == betterPos.end())
									m_RenderingTool->renderSphere(fPos, medPointDistance * m_FeatureSize);
								else
									m_RenderingTool->renderSphere(betterPos[i], medPointDistance * m_FeatureSize);
							}

							//if(0)
							if( slipSig )
							{
								Matrix3f frame = shrink4To3(T) * calcTangentSystem(f->m_Normal);
								unsigned numSeg = 20;
								float angle1 = 0;
								glBegin(GL_LINE_STRIP);
								glColor3fv( color.data() );
								for( unsigned x=0;x<numSeg;x++ )
								{
									float angle = (float)x / (float)(numSeg-1) * 2.0f * M_PI;
									Vector3f p1 = fPos + frame[0]*slipSig->m_Sigma * sin(angle)
										+ frame[1]*slipSig->m_Sigma * cos(angle);
									
									glVertex3fv(p1.data());

								}
								glEnd();
							}

						}
					}
				}
			}
		}
		sIt.next();
	}
	

}

void X4::PCIRigidRegistration::precomputeFeatures()
{
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();
	FeaturePointDetector * detector = m_FeatureMatching->getFeatureDetector();

	detector->begin(clouds);
	progressWindow->pushStep(false,"features");
	for( unsigned i=0;i<clouds.size();i++ )
	{
		progressWindow->progress((float)i/(float)clouds.size()*100.0f);
		detector->detect(clouds[i]);
	}
	progressWindow->popSteps();

	detector->end();
}

void X4::PCIRigidRegistration::recomputeSignatures()
{
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();
	FeaturePointDetector * detector = m_FeatureMatching->getFeatureDetector();

	detector->begin(clouds);
	for( unsigned i=0;i<clouds.size();i++ )
		detector->recomputeFeatureSignatures(clouds[i]);
	detector->end();
}


std::vector<UnstructuredInCorePointCloud*> X4::PCIRigidRegistration::getAllPointClouds()
{
	std::vector<UnstructuredInCorePointCloud*> ret;
	std::vector<std::string> sel = getScene()->getRootState()->staticState->getSelectedNodes();
	if( sel.size() )
	{
		for( unsigned i=0;i<sel.size();i++ )
		{
			UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(
				getPointCloud(getScene(), sel[i]));
			if( upc )
				ret.push_back( upc );
		}

		if( ret.size() )
			return ret;
	}
	

	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			UnstructuredInCorePointCloud * pc = dynamic_cast<UnstructuredInCorePointCloud*>(objNode->getSceneObject());
			if( pc)
				ret.push_back(pc);
		}
		sceneIt->next();
	}

	return ret;
}

void X4::PCIRigidRegistration::filterFeaturesOneFrame()
{
	PointCloud * pc = getSelectedPointCloud();
	if( pc == NULL )
		return;

	FeatureSet *fs = (FeatureSet*)pc->getAttachments()->getData(FeatureSet::getDefaultName());
	if( fs == NULL )
		return;
	
	float radius = getMedianPointDistance(pc)*1.5f;
	debugRenderer->clearDebugData("features");
	debugRenderer->beginRenderJob("features",false);

	for( unsigned i=0;i<fs->getNumFeatures();i++ )
	{
		GeometricFeature *f = fs->m_Features[i];
		bool badFeature = false;
		for( unsigned j=0;j<f->m_Signatures.size();j++ )
			badFeature |= f->m_Signatures[j]->badFeature( m_FeatureMatching->getFeatureDetector()->getLocalDescriptor(j) );
		debugRenderer->addFastSphere(fs->m_Features[i]->m_Position, radius, 
			badFeature ? makeVector3f(0.5f,0.5f,0.5f):makeVector3f(1,0.2f,0),false);
	}
	debugRenderer->endRenderJob();

}

void X4::PCIRigidRegistration::computeCorrespondences()
{
	StandardSceneGraphIterator sIt;
	sIt.clearAndSetup( getScene()->getRootNode(),getScene()->getRootState(),
		SGDynamicState::tmRendering);

	while( !sIt.atEnd())
	{
		SGAnimationNode * an = dynamic_cast<SGAnimationNode*>(sIt.getNode());
		if( an && an->getNumChildNodes(NULL) > 0)
		{
			for( int i=0;i<=int(an->getNumChildNodes(NULL)-1);i++ )						
			{
				SGObjectNode * objNode1 = dynamic_cast<SGObjectNode*>(an->getChildNode(NULL,i));
				SGObjectNode * objNode2 = dynamic_cast<SGObjectNode*>(an->getChildNode(NULL,i+1));
				if( objNode1 && objNode2 )
				{
					PointCloud * pc1 = dynamic_cast<PointCloud*>(objNode1->getSceneObject());
					PointCloud * pc2 = dynamic_cast<PointCloud*>(objNode2->getSceneObject());

					FeatureSet *fs1 = (FeatureSet*)pc1->getAttachments()->getData(FeatureSet::getDefaultName());
					FeatureSet *fs2 = (FeatureSet*)pc2->getAttachments()->getData(FeatureSet::getDefaultName());

					if( fs1 && fs2)
					{
						FeatureCorrespondenceSet * set;
						if( m_ValidateCorrespondences )
							set = m_FeatureMatching->computeValidatedCorrespondences(fs1,fs2);
						else
							set = m_FeatureMatching->computeInitialCorrespondences(fs1,fs2);
						set->setup(FeatureCorrespondenceSet::getDefaultName(), AttachedData::ADF_PERSISTENT);
						pc1->getAttachments()->attachData( set);
					}
					
				}
			}
		}
		sIt.next();
	}
}

void X4::PCIRigidRegistration::recomputeGeodeticDistances()
{
/*	StandardSceneGraphIterator sIt;
	sIt.clearAndSetup( getScene()->getRootNode(),getScene()->getRootState(),
		SGDynamicState::tmRendering);

	while( !sIt.atEnd())
	{
		SGAnimationNode * an = dynamic_cast<SGAnimationNode*>(sIt.getNode());
		if( an && an->getNumChildNodes(NULL) > 0)
		{
			for( int i=0;i<=int(an->getNumChildNodes(NULL));i++ )						
			{
				SGObjectNode * objNode1 = dynamic_cast<SGObjectNode*>(an->getChildNode(NULL,i));
				if( objNode1 )
				{
					PointCloud * pc1 = dynamic_cast<PointCloud*>(objNode1->getSceneObject());
					FeatureSet *fs1 = (FeatureSet*)pc1->getAttachments()->getData(FeatureSet::getDefaultName());

					if( fs1)
					{
						fs1->computeGeodeticDistances((UnstructuredInCorePointCloud*)pc1);
					}

				}
			}
		}
		sIt.next();
	}
*/
	std::vector<UnstructuredInCorePointCloud*> pcs = getAllPointClouds();
	for( unsigned i=0;i<pcs.size();i++ )
	{
		FeatureSet *fs1 = (FeatureSet*)pcs[i]->getAttachments()->getData(FeatureSet::getDefaultName());

		if( fs1)
			fs1->computeGeodeticDistances(pcs[i]);
	}
}

void X4::PCIRigidRegistration::alignClouds()
{
	std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() < 2 )
		return;

	//alignTwoClouds(selection[0],selection[1]);
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();
	
	
	if( 0 )
	{
	
		FeatureCorrespondenceSet * set = computeFeatureCorrespondences(clouds[0],clouds[1]);
	
		Matrix4f transf = set->computeTransformationMatrix();

		clouds[1]->setTransformation( clouds[0]->getTransformation()*transf );
		Matrix4f fineTransf = computeICPAlignment(clouds[1],clouds[0]);
		clouds[1]->setTransformation( fineTransf );
		delete set;
	}
	else
	{
		Matrix4f fineTransf = computeICPAlignment(clouds[1],clouds[0]);
		clouds[1]->setTransformation( fineTransf );
	}


}

struct TFeatureAndDistance
{
	float dist;
	GeometricFeature * f;
	bool operator < (const TFeatureAndDistance &o ) const {return dist < o.dist;}
};

void X4::PCIRigidRegistration::alignTwoClouds( std::string pc1, std::string pc2 )
{
	// ----------------------------------------------
	// get point clouds and feature sets
	UnstructuredInCorePointCloud * upc1 = dynamic_cast<UnstructuredInCorePointCloud*>
		(getPointCloud(getScene(), pc1));
	UnstructuredInCorePointCloud * upc2 = dynamic_cast<UnstructuredInCorePointCloud*>
		(getPointCloud(getScene(), pc2));

	if( !upc1 || !upc2)
	{
		debugOutput << "Select at least two point clouds!\n";
		return;
	}

	FeatureSet *fs1 = (FeatureSet*)upc1->getAttachments()->getData(FeatureSet::getDefaultName());
	FeatureSet *fs2 = (FeatureSet*)upc2->getAttachments()->getData(FeatureSet::getDefaultName());

	if( !fs1 || !fs2 )
	{
		debugOutput << "compute features first!\n";
		return;
	}

	// ----------------------------------------------
	// find centroid descriptor/ prepare ransac
	FeaturePointDetector * fd = m_FeatureMatching->getFeatureDetector();
	unsigned cdID = fd->getNumLocalDescriptors();
	for(unsigned i=0;i<fd->getNumLocalDescriptors();i++ )
	{
		if( dynamic_cast<LDCentroidDescriptor*>(fd->getLocalDescriptor(i)))
		{
			cdID = i;
			break;
		}
	}
	if( cdID == fd->getNumLocalDescriptors() )
		return;
	LDCentroidDescriptor * centroidDescriptor = (LDCentroidDescriptor*)fd->getLocalDescriptor(cdID);

	unsigned numRandomSamples = 1000;
	Matrix4f bestTransformation = IDENTITY4F;
	
	// ----------------------------------------------
	// test random samples
	for( unsigned r=0;r<numRandomSamples;r++ )
	{
		unsigned sample1 = rand() % fs1->m_Features.size();
		GeometricFeature * f1 = fs1->m_Features[sample1];
		LDCentroidSignature * sig1 = (LDCentroidSignature*)f1->m_Signatures[cdID];

		// find good matches
		std::vector<TFeatureAndDistance> bestMatches;
		for( unsigned i=0;i<fs2->m_Features.size();i++ )
		{
			TFeatureAndDistance f;
			f.f = fs2->m_Features[i];
			LDCentroidSignature * sig2 = (LDCentroidSignature*)f.f->m_Signatures[cdID];
			f.dist = sig1->compare( sig2, centroidDescriptor);
			if( f.dist < 1e10f )
				bestMatches.push_back(f);
		}
		sort( bestMatches.begin(), bestMatches.end());

		// test best n
		for( unsigned i=0;i<10;i++ )
		{
			LDCentroidSignature * sig2 = (LDCentroidSignature*)bestMatches[i].f->m_Signatures[cdID];
			Matrix4f transf = sig2->calculateTransformation(sig1,false);

			float score = 0;
			for( unsigned j=0;j<fs1->m_Features.size();j++ )
			{
				Vector3f transformedFeaturePoint = shrink4To3( transf * expand3To4(fs1->m_Features[j]->m_Position));
				float bestDist = 1e20f;
				for( unsigned k=0;k<fs2->m_Features.size();k++ )
				{
										
				}
			}
		}


		
	}

}

float X4::PCIRigidRegistration::computeScore( UnstructuredInCorePointCloud * pc1, UnstructuredInCorePointCloud * pc2 )
{
	PointSet * ps = pc1->getPointSet();
	AAT posAAT = ps->getAAT("position");
	AAT normalAAT = ps->getAAT("normal");

	float scoreSigma = getMedianPointDistance(pc1) * 2.0f;
	float inv_scoreSigma_2 = 1.0f / pow( scoreSigma, 2.0f);

	HierarchicalKNNIterator hIt(pc2,32,NULL);
	Matrix4f mat_pc1ToPc2 = invertFrame(pc2->getTransformation()) * pc1->getTransformation();

	float ret = 0;
	for( unsigned i=0;i<ps->getNumEntries();i++ )
	{
		if( i % 10 != 0)
			continue;

		Vector3f pos = ps->get3f(i,posAAT);	
		pos = transformVector3f(mat_pc1ToPc2,pos);
		hIt.setSeekPointAndReset(pos);

		Vector3f closestPoint = hIt.get3f(posAAT);
		Vector3f cp_normal = hIt.get3f(normalAAT);
		Vector3f projectedPoint = pos - cp_normal * 
			(cp_normal * (pos - closestPoint));
		float w1 = exp(-normQuad(pos-closestPoint) * inv_scoreSigma_2);
		float w2 = exp(-normQuad(projectedPoint-closestPoint) * inv_scoreSigma_2);
		ret += w1*w2;
	}

	return ret;
}

static PCIRigidRegistration * PCIRigidRegistration_currentObject = NULL;

void PCIRigidRegistration_redraw()
{
	if( PCIRigidRegistration_currentObject )
		PCIRigidRegistration_currentObject->debug_redraw();
}


void X4::PCIRigidRegistration::mouseDown( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState )
{
	PCIRigidRegistration_currentObject = this;
	static GeometricFeature * targetFeature = NULL;
	static PointCloud * targetPC = NULL;
	Vector3f coordSystemColor1 = makeVector3f(255,174,0) / 255.0f;
	Vector3f coordSystemColor2 = makeVector3f(255,25,0) / 255.0f;
	if( buttonsState.getLeft() )
	{
		PointCloud * pc;
		std::string pcName;
		Vector3f point;
		if( getIntersectionFromCursor(x,y, point, pcName, &pc) )
		{
			FeatureSet *fs = (FeatureSet*)pc->getAttachments()->getData(FeatureSet::getDefaultName());
			targetFeature = fs->getNearestFeature(point);
			targetPC = pc;
			float medPointDistance = getCachedMedianPointDistance(pc);
			debugRenderer->clearDebugData("drawtargetfeature");
			debugRenderer->beginRenderJob("drawtargetfeature");

			Vector3f tPos = transformVector3f(pc->getTransformation(),targetFeature->m_Position);

			// draw feature support radius
			float radius = medPointDistance * m_FeatureMatching->getFeatureDetector()->getDescriptorRadius();
			/*debugRenderer->addPlanarEllipse( tPos, makeVector3f(1,0,0)*radius,makeVector3f(0,1,0)*radius,makeVector3f(1,1,0),2);
			debugRenderer->addPlanarEllipse( tPos, makeVector3f(0,0,1)*radius,makeVector3f(0,1,0)*radius,makeVector3f(1,1,0),2);
			debugRenderer->addPlanarEllipse( tPos, makeVector3f(1,0,0)*radius,makeVector3f(0,0,1)*radius,makeVector3f(1,1,0),2);*/
			debugRenderer->endRenderJob();
		}
		else
		{
			targetFeature = NULL;
			targetPC = NULL;
			debugRenderer->clearDebugData("drawtargetfeature");
		}
		colortable.clear();
		betterPos.clear();
		currentColorIndex = 0;
	}

	if( buttonsState.getRight() )
	{
		PointCloud * pc;
		std::string pcName;
		Vector3f point;
		if( getIntersectionFromCursor(x,y, point, pcName, &pc) )
		{
			/*HierarchicalKNNIterator hIt(pc,32,NULL);
			hIt.setSeekPointAndReset(point);
			AAT normalAAT = pc->getAAT("normal");
			Vector3f normal = hIt.get3f(normalAAT);
			float medPointDistance = getMedianPointDistance(pc);
			float size = medPointDistance * 10.0f;

			LDCorrelation corr;
			corr.beginSignatureComputations((UnstructuredInCorePointCloud*)pc);
			static LDCorrelationSignature * sig1 = NULL;
			LDCorrelationSignature * sig2 = NULL;
			static Vector3f p1;
			if( sig1 )
				sig2 = (LDCorrelationSignature*)corr.computeSignature(point, normal, size, (UnstructuredInCorePointCloud*)pc);
			else
			{
				sig1 = (LDCorrelationSignature*)corr.computeSignature(point, normal, size, (UnstructuredInCorePointCloud*)pc);
				p1 = point;
			}
			corr.endSignatureComputations();

			if( sig2 )
			{
				std::vector<Matrix3f> dest_frames;
				sig1->computeAlignment(sig2, &corr, dest_frames);

				debugRenderer->clearDebugData("align");
				debugRenderer->beginRenderJob("align");
				debugRenderer->addFineCoordinateFrame(p1, sig1->m_LocalFrame,size*3.0f, makeVector3f(1.0f,0.75f,0));
				debugRenderer->addFineCoordinateFrame(point, dest_frames[0],size*3.0f, makeVector3f(1.0f,0.75f,0));

				PointSet ps;
				ps.clearAndSetup(1, 0, pc->getDescr());
				hIt.setSeekPointAndReset(p1);
				AAT posAAT = pc->getAAT("position");
				char buffer[2000];
				while( !hIt.atEnd())
				{
					Vector3f pos = hIt.get3f(posAAT);
					float dist_2 = normQuad(pos-p1);
					if( dist_2 > size*size)
						break;

					hIt.getVertex(buffer);
					ps.addPoint(buffer);

					hIt.next();
				}
				m_icp->beginICP(pc,pc);
				Matrix4f frame1 = invertFrame(makeTranslation4f(p1)*expand3To4(sig1->m_LocalFrame));
				Matrix4f frame2 = invertFrame(makeTranslation4f(point)*expand3To4(dest_frames[0]));
				m_icp->computeICP(&ps, frame1, frame2);
				frame1 = invertFrame(frame1);
				debugRenderer->addFineCoordinateFrame(shrink4To3(frame1[3]), shrink4To3(frame1),size*3.0f, makeVector3f(1.0f,0.75f,0));
				m_icp->endICP();

				delete sig1;
				delete sig2;
				sig1 = NULL;


				debugRenderer->endRenderJob();
			}*/

			FeatureSet *fs = (FeatureSet*)pc->getAttachments()->getData(FeatureSet::getDefaultName());
			GeometricFeature * f = fs->getNearestFeature(point);
			float medPointDistance = getCachedMedianPointDistance(pc);

			debugRenderer->clearDebugData("testClustering");
			debugRenderer->beginRenderJob("testClustering");

			Vector3f tPos = transformVector3f(pc->getTransformation(),f->m_Position);

			// draw feature support radius
			float radius = medPointDistance * m_FeatureMatching->getFeatureDetector()->getDescriptorRadius();
			/*debugRenderer->addPlanarEllipse( tPos, makeVector3f(1,0,0)*radius,makeVector3f(0,1,0)*radius,makeVector3f(1,0,0),2);
			debugRenderer->addPlanarEllipse( tPos, makeVector3f(0,0,1)*radius,makeVector3f(0,1,0)*radius,makeVector3f(1,0,0),2);
			debugRenderer->addPlanarEllipse( tPos, makeVector3f(1,0,0)*radius,makeVector3f(0,0,1)*radius,makeVector3f(1,0,0),2);*/

			
			FeaturePointDetector * det = m_FeatureMatching->getFeatureDetector();

			PointSet *ps = NULL;
			CachedRangeQuerry querry(pc);
			Camera sceneCam = getScene()->getRootState()->staticState->camera;
			ViewFrustum sceneVF = getScene()->getRootState()->staticState->viewFrustum;
			if( m_TestICPClustering )
			{
				if( targetPC != pc && targetPC != NULL)
					m_icp->beginICP(pc,targetPC);
				else
					m_icp->beginICP(pc,pc);

				if( m_VisualizeICP )
				{
					m_icp->setRedrawMethod(PCIRigidRegistration_redraw);
					getScene()->getRootState()->staticState->camera.setLookAt( makeVector3f(0,0,0) );
					getScene()->getRootState()->staticState->camera.setUp( makeVector3f(0,0,1) );
					getScene()->getRootState()->staticState->camera.setPosition( makeVector3f(0,3,4)*medPointDistance*det->getDescriptorRadius() );
					getScene()->getRootState()->staticState->viewFrustum.setZNearClip(medPointDistance*det->getDescriptorRadius()*0.001f);
					getScene()->getRootState()->staticState->viewFrustum.setZFarClip(medPointDistance*det->getDescriptorRadius()*10);
				}

				// gather points around selected feature for icp
				ps = querry.getSpherePoints(f->m_Position,medPointDistance*det->getDescriptorRadius(),0);
			}
			
			// find LDCorrelation descriptor
			unsigned correlationIndex = 0;
			LDCorrelation * corr = NULL;
			for( unsigned i=0;i<det->getNumLocalDescriptors();i++ )
			{
				corr = dynamic_cast<LDCorrelation*>(det->getLocalDescriptor(i));
				if( corr != NULL )
				{
					correlationIndex = i;
					break;
				}
			}

			if( m_TestICPClustering && !corr )
			{
				throw X4Exception("Correlation signature not found!");
			}

			LDCorrelationSignature * corrSigRef = NULL;
			Matrix4f refFrame;
			if( m_TestICPClustering )
			{
				corrSigRef = (LDCorrelationSignature *)f->m_Signatures[correlationIndex];
				refFrame = makeTranslation4f(f->m_Position) * expand3To4(corrSigRef->m_LocalFrame);
				//debugRenderer->addFineCoordinateFrame(pc->getTransformation() * refFrame, medPointDistance*det->getDescriptorRadius(), coordSystemColor1,false);
				refFrame = invertFrame(refFrame);
			}
			else
			{
				corrSigRef = (LDCorrelationSignature *)f->m_Signatures[correlationIndex];
				refFrame = makeTranslation4f(f->m_Position) * expand3To4(corrSigRef->m_LocalFrame);
				//debugRenderer->addFineCoordinateFrame(pc->getTransformation() * refFrame, medPointDistance*det->getDescriptorRadius(), coordSystemColor1,false);
			}

			for( unsigned i=0;i<fs->m_Features.size();i++ )
			{
				GeometricFeature * f2 = fs->m_Features[i];
				if( i > 0 && targetFeature )
					break;
				if( targetFeature )
					f2 = targetFeature;

				if( f == f2 )
				{
					colortable[i] = currentColorIndex;
					continue;
				}
				if( f2->badFeature(m_FeatureMatching->getFeatureDetector()) )
					continue;

				if( !f->compare(f2, m_FeatureMatching->getFeatureDetector()) && targetFeature==NULL)
					continue;

				if( colortable.find(i) != colortable.end() )
					continue;

				if( !m_TestICPClustering )
				{
					LDCorrelationSignature * corrSigDest = (LDCorrelationSignature *)f2->m_Signatures[correlationIndex];
					Matrix4f destFrame = makeTranslation4f(f2->m_Position) * expand3To4(corrSigDest->m_LocalFrame);
					//debugRenderer->addFineCoordinateFrame(targetPC->getTransformation() * destFrame, medPointDistance*det->getDescriptorRadius(), coordSystemColor1,false);

				}

				if( m_TestICPClustering )
				{
					LDCorrelationSignature * corrSigDest = (LDCorrelationSignature *)f2->m_Signatures[correlationIndex];
					
					/*std::vector<Matrix3f> dest_frames;
					corrSigRef->computeAlignment( corrSigDest, corr, dest_frames, true);

					//debugRenderer->addFineCoordinateFrame(makeTranslation4f(f2->m_Position) * expand3To4(dest_frames[0]), medPointDistance * det->getDescriptorRadius()
					//	, makeVector3f(1.0f,0.0f,0.0f),false);

					float bestResidual = 1e20f;
					Matrix4f bestFrame = IDENTITY4F;
					Matrix4f bestInitialFrame = IDENTITY4F;
										
					for( unsigned df=0;df<dest_frames.size();df++ )
					{
						Matrix4f tmpRefFrame = refFrame;
						Matrix4f destFrame = invertFrame(makeTranslation4f(f2->m_Position) * expand3To4(dest_frames[df]));
						float residual = m_icp->computeFastICP(ps,tmpRefFrame, destFrame, medPointDistance*det->getDescriptorRadius() *0.5f );

						if( residual < bestResidual )
						{
							Matrix4f dest0 = invertFrame(destFrame);
							Matrix4f r0 = invertFrame(refFrame);
							Matrix4f r1 = invertFrame(tmpRefFrame);
							Matrix4f T = invertFrame(r1)*r0;
							bestInitialFrame = dest0;
							bestFrame = dest0 * T;
							bestResidual = residual;
						}
					}
					*/

					corrSigRef->beginICPCompare(m_icp,pc,pc,corr);
					float32 bestResidual = corrSigRef->computeICPDistance(f->m_Position, f2->m_Position, corrSigDest, medPointDistance * det->getDescriptorRadius());
					corrSigRef->endICPCompare();

					debugOutput << "best residual: " << bestResidual << "\n";
					if( bestResidual < m_MaxICPResidual)
					{
						/*debugRenderer->addFineCoordinateFrame(targetPC->getTransformation()*bestFrame, medPointDistance * det->getDescriptorRadius()
							, coordSystemColor1,false);
						debugRenderer->addFineCoordinateFrame(targetPC->getTransformation()*bestInitialFrame, medPointDistance * det->getDescriptorRadius()
							, coordSystemColor2,false);*/
						colortable[i] = currentColorIndex;
						betterPos[i] = f2->m_Position;//shrink4To3(pc->getTransformation()*bestFrame[3]);
					}
				}
				else
				{
					colortable[i] = currentColorIndex;
				}
			}

			if( m_TestICPClustering )
			{
				m_icp->endICP();
				m_icp->setRedrawMethod(NULL);
			}

			currentColorIndex++;

			debugRenderer->endRenderJob();
			getScene()->getRootState()->staticState->viewFrustum = sceneVF;
			getScene()->getRootState()->staticState->camera = sceneCam;


			if(0)
			if( norm(m_SelectedFeaturePoint1) < 1e-10f )
			{
				m_SelectedFeaturePoint1 = point;
				m_SelectedPC1 = pc;
			}
			else
			{
				m_SelectedFeaturePoint2 = point;
				FeatureSet *fs1 = (FeatureSet*)m_SelectedPC1->getAttachments()->getData(FeatureSet::getDefaultName());
				FeatureSet *fs2 = (FeatureSet*)pc->getAttachments()->getData(FeatureSet::getDefaultName());
				GeometricFeature * f1 = fs1->getNearestFeature(m_SelectedFeaturePoint1);
				GeometricFeature * f2 = fs2->getNearestFeature(m_SelectedFeaturePoint2);

				FeaturePointDetector * fd = m_FeatureMatching->getFeatureDetector();
				
				debugOutput << "descriptor distance: "
					<< f1->compare(f2,fd ) << "\n";

				/*LDCentroidSignature * cd1 = dynamic_cast<LDCentroidSignature*>(f1->m_Signatures[1]);
				LDCentroidSignature * cd2 = dynamic_cast<LDCentroidSignature*>(f2->m_Signatures[1]);
				Matrix4f transf = cd2->calculateTransformation(cd1,true);

				m_SelectedPC1->setTransformation(transf);
				debugRenderer->clearDebugData("drawCentroids");
				debugRenderer->beginRenderJob("drawCentroids");
				cd1->debug_draw();
				cd2->debug_draw();
				debugRenderer->endRenderJob();*/


				m_SelectedFeaturePoint1 = NULL_VECTOR3F;
			}
		}
	}
}

void X4::PCIRigidRegistration::resetTransformations()
{
	StandardSceneGraphIterator sIt;
	sIt.clearAndSetup( getScene()->getRootNode(),getScene()->getRootState(),
		SGDynamicState::tmAllNodes);

	while( !sIt.atEnd())
	{
		SGObjectNode * objNode1 = dynamic_cast<SGObjectNode*>(sIt.getNode());
		if( objNode1 )
		{
			objNode1->getSceneObject()->setTransformation(IDENTITY4F);
		}
		sIt.next();
	}
}

void X4::PCIRigidRegistration::icp()
{
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();
	if( clouds.size() != 1)
		return;

	clouds.insert( clouds.begin(), (UnstructuredInCorePointCloud*)m_ICP_PointCloud0);


	m_icp->beginICP( clouds[0],clouds[1] );
	Matrix4f m1 = clouds[0]->getTransformation();
	Matrix4f m2 = clouds[1]->getTransformation();

	// select a random subset
	PointSet * ps = new PointSet;
	VertexDescriptor * vdCopy = (VertexDescriptor*)clouds[0]->getDescr()->copy();
	std::vector<char> buffer(vdCopy->getSize());
	ps->clearAndSetup(1,0,vdCopy);
	PointSet * srcPS = clouds[0]->getPointSet();
	for( unsigned i=0;i<600;i++ )
	{
		mpcard index = random32bit(clouds[0]->getNumPoints());
		srcPS->getVertex(index, &buffer[0]);
		ps->addPoint(&buffer[0]);
	}
	
	m_icp->computeICP(ps, m1, m2);

	clouds[0]->setTransformation( m1 );

	delete ps;
	m_icp->endICP();
}

void X4::PCIRigidRegistration::testSmoothing()
{
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();
	if( clouds.size() == 0)
		return;

	UnstructuredInCorePointCloud * pc = clouds[0];
	float medPointDistance = getMedianPointDistance(pc);
	float sigma_0 = medPointDistance * 1.5f;

	for( unsigned i=0;i<8;i++ )
	{
		float sigma = sigma_0 * pow(2,(float)i / 2.0f);
		UnstructuredInCorePointCloud * pcCopy = (UnstructuredInCorePointCloud*)pc->copy();
		
		smoothNormals( pc, pcCopy, "normal", sigma );
		addPointCloud(getScene(), pcCopy, "smoothed");
	}
}

void X4::PCIRigidRegistration::icp_setCloud0()
{
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();
	if( clouds.size() != 1)
		return;

	m_ICP_PointCloud0 = clouds[0];
}

void X4::PCIRigidRegistration::setTransformationFromString()
{
	std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() != 1)
	{
		debugOutput << "PCCTestSceneCD::setupScanAlignment() - no point cloud selected\n";
		return;
	}

	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	SGObjectNode * node = NULL;
	while( !sceneIt->atEnd() )
	{
		if( sceneIt->getNameWithPath() == selection[0] )
		{
			node = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
			break;
		}
		sceneIt->next();
	}

	if( node == NULL )
	{
		debugOutput << "PCCTestSceneCD::setupScanAlignment() - Couldn't find scenegraphnode \"" << selection[0] << "\"\n";
		return;
	}


	QString aligmentText = QInputDialog::getText(NULL, "input", "Alignment std::string (tx ty tz r1 r2 r3 r4):");
	if( !aligmentText.isEmpty())
	{
		std::string tmp = aligmentText.toLatin1().constData();
		std::vector<std::string> entrys = splitString(tmp, " \t");
		if( entrys.size() != 7 )
			return;

		Vector3f position;
		Quaternion rot;
		bool fine = true;
		fine &= strToFloat(entrys[0], position[0]);
		fine &= strToFloat(entrys[1], position[1]);
		fine &= strToFloat(entrys[2], position[2]);
		fine &= strToFloat(entrys[3], rot[3]);
		fine &= strToFloat(entrys[4], rot[2]);
		fine &= strToFloat(entrys[5], rot[1]);
		fine &= strToFloat(entrys[6], rot[0]);

		if( fine )
		{
			position = rot.getRotMat().transpose() * position;
			Matrix4f transformation = expand3To4(rot.getRotMat())*makeTranslation4f( position );
			node->getSceneObject()->setTransformation( transformation);
		}
		else
		{
			debugOutput << "Somethings wrong with input std::string.\n";
		}
	}
}

void X4::PCIRigidRegistration::debug_redraw()
{
	redrawScene();
	debugOutput.flush();
}

void X4::PCIRigidRegistration::testFeatureClustering()
{
	PointCloud * inputPC = getSelectedPointCloud();
	if( inputPC == NULL )
		return;

	FeatureClustering fc;
	PointCloud * pc = fc.pcaTransform3D(inputPC);
	addPointCloud(getScene(), pc, "features");
}

void X4::PCIRigidRegistration::featureBurning()
{
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();
	for( unsigned i=0;i<clouds.size();i++ )
	{
		PointCloud * pc = clouds[i];

		FeatureSet *fs = (FeatureSet*)pc->getAttachments()->getData(FeatureSet::getDefaultName());
		float medPointDistance = getCachedMedianPointDistance(pc);

		debugRenderer->clearDebugData("burnFeatures");
		debugRenderer->beginRenderJob("burnFeatures");

		FeaturePointDetector * det = m_FeatureMatching->getFeatureDetector();

		CachedRangeQuerry querry(pc);
		m_icp->beginICP(pc,pc);
		std::vector<bool> burnList(fs->m_Features.size(),false);


		progressWindow->pushStep();
		for( unsigned i=0;i<(unsigned)fs->m_Features.size();i++ )
		{
			if( i % 10 == 0)
				progressWindow->progress(i*100/(unsigned)fs->m_Features.size());
			GeometricFeature * f = fs->m_Features[i];

			if( f->badFeature(m_FeatureMatching->getFeatureDetector()) )
				continue;

			card32 numCorrespondingFeatures = 0;
			for( unsigned j=0;j<fs->m_Features.size();j++ )
			{
				if( i == j )
					continue;
				GeometricFeature * f2 = fs->m_Features[j];
				if( f->compare(f2, m_FeatureMatching->getFeatureDetector()))
					numCorrespondingFeatures++;
			}

			burnList[i] = numCorrespondingFeatures > m_FeatureMatching->getMaxNumCorrespondingFeatures();

			if( !m_DeleteOnBurn && !burnList[i])
				debugRenderer->addFastSphere(f->m_Position, medPointDistance * m_FeatureSize * 2.0f, makeVector3f(0,1,0),false);

		}
		progressWindow->popSteps();
		m_icp->endICP();
		debugRenderer->endRenderJob();

		if( m_DeleteOnBurn )
		{
			for( int i=fs->getNumFeatures()-1;i>=0;i--)
			{
				if( burnList[i] || fs->m_Features[i]->badFeature(det) )
					fs->deleteFeatures(i,i+1);
			}

		}

	}
}

struct TCorrespondenceCounter
{
	card32 numCorrespondingFeatures;
	card32 featureID;
	bool operator < (const TCorrespondenceCounter &corr)
	{
		return numCorrespondingFeatures < corr.numCorrespondingFeatures;
	}
};

struct TCorrespondenceDistance
{
	unsigned featureID;
	float dist;
	bool operator < (const TCorrespondenceDistance &o)
	{
		return dist < o.dist;
	}
};

FeatureCorrespondenceSet * X4::PCIRigidRegistration::computeFeatureCorrespondences( PointCloud * pc1, PointCloud * pc2 )
{
	FeatureSet *fs2 = (FeatureSet*)pc1->getAttachments()->getData(FeatureSet::getDefaultName());
	FeatureSet *fs1 = (FeatureSet*)pc2->getAttachments()->getData(FeatureSet::getDefaultName());


	FeatureCorrespondenceSet * set = new FeatureCorrespondenceSet;

	// find LDCorrelation descriptor
	unsigned correlationIndex = 0;
	unsigned histogramIndex = 0;
	LDCorrelation * corr = NULL;
	LDHistogram  * hisDesc = NULL;
	FeaturePointDetector * det = m_FeatureMatching->getFeatureDetector();
	for( unsigned i=0;i<det->getNumLocalDescriptors();i++ )
	{
		LDCorrelation * tmp_corr = dynamic_cast<LDCorrelation*>(det->getLocalDescriptor(i));
		if( tmp_corr != NULL )
		{
			corr = tmp_corr;
			correlationIndex = i;
		}

		LDHistogram * tmp_hisDesc = dynamic_cast<LDHistogram*>(det->getLocalDescriptor(i));
		if( tmp_hisDesc )
		{
			hisDesc = tmp_hisDesc;
			histogramIndex = i;
		}
	}

	float medPointDistance = getCachedMedianPointDistance(pc1);
	std::vector<TCorrespondenceCounter> correspondenceCounter(fs1->m_Features.size());

	for( unsigned i=0;i<fs1->m_Features.size();i++ )
	{
		correspondenceCounter[i].numCorrespondingFeatures = 0;
		correspondenceCounter[i].featureID = i;

		GeometricFeature * f1 = fs1->m_Features[i];
		if( f1->badFeature(det) )
			continue;


		LDCorrelationSignature * corrSigRef = (LDCorrelationSignature *)f1->m_Signatures[correlationIndex];
		PointSet *ps = NULL;
		//CachedRangeQuerry querry(pc1);
		Camera sceneCam = getScene()->getRootState()->staticState->camera;
		ViewFrustum sceneVF = getScene()->getRootState()->staticState->viewFrustum;
		Matrix4f refFrame;

		card32 numCorrespondingFeatures = 0;
		float bestDist = 1e20f;
		//TCorrespondenceDistance maxDist;
		std::vector<TCorrespondenceDistance> distances;

		for( unsigned j=0;j<fs2->m_Features.size();j++ )
		{
			GeometricFeature * f2 = fs2->m_Features[j];
			if( f2->badFeature(det) )
				continue;

			float dist = f1->m_Signatures[histogramIndex]->compare(f2->m_Signatures[histogramIndex],hisDesc);
			if( dist < 1.0f)
			{
				bestDist = min(bestDist,dist);
				TCorrespondenceDistance d;
				d.dist = dist;
				d.featureID = j;
				distances.push_back(d);
			}
			
		}

		sort( distances.begin(), distances.end());

		for( unsigned j=0;j<distances.size() && j<m_FeatureMatching->getMaxNumCorrespondingFeatures();j++ )
		{
			if( distances[j].dist > distances[0].dist*1.2f )
			{
				break;
			}
			FeatureCorrespondence * corr = new FeatureCorrespondence;
			corr->m_FeatureIndex[1] = i;
			corr->m_FeatureIndex[0] = distances[j].featureID;
			corr->m_FeaturePoint[1] = f1->m_Position;
			corr->m_FeaturePoint[0] = fs2->m_Features[distances[j].featureID]->m_Position;
			corr->m_Distance = distances[j].dist;
			set->m_Correspondences.push_back(corr);
			numCorrespondingFeatures++;
		}
		correspondenceCounter[i].numCorrespondingFeatures = numCorrespondingFeatures;

	}

	if( m_TestICPClustering )
		m_icp->endICP();

	//sort(correspondenceCounter.begin(), correspondenceCounter.end());
	std::vector<bool> keepFeature(fs1->getNumFeatures(),false);

	card32 maxNumCorrespondences = m_FeatureMatching->getMaxNumInitialCorrespondences();
	card32 numCorr = 0;
	for( unsigned i=0;i<correspondenceCounter.size();i++ )
	{
		numCorr += correspondenceCounter[i].numCorrespondingFeatures;
		keepFeature[correspondenceCounter[i].featureID] = correspondenceCounter[i].numCorrespondingFeatures <= m_FeatureMatching->getMaxNumCorrespondingFeatures();
	}

	FeatureCorrespondenceSet * prunedSet = new FeatureCorrespondenceSet;
	for( unsigned i=0;i<set->m_Correspondences.size();i++ )
	{
		FeatureCorrespondence * corr = set->m_Correspondences[i];
		if( keepFeature[corr->m_FeatureIndex[1]] )
		{
			prunedSet->m_Correspondences.push_back( (FeatureCorrespondence*)corr->copy());
		}
	}

	delete set;
	set = prunedSet;

	debugOutput << "num initial corr: " << set->m_Correspondences.size() << "\n";

	if( set->m_Correspondences.size() > maxNumCorrespondences )
	{
		set->sort();
		set->deleteCorrespondences(maxNumCorrespondences, (card32)set->m_Correspondences.size());
	}


	if( m_TestICPClustering )
	{
		PCIRigidRegistration_currentObject = this;
		Camera sceneCam = getScene()->getRootState()->staticState->camera;
		ViewFrustum sceneVF = getScene()->getRootState()->staticState->viewFrustum;
		if( m_VisualizeICP )
		{
			m_icp->setRedrawMethod(PCIRigidRegistration_redraw);
			getScene()->getRootState()->staticState->camera.setLookAt( makeVector3f(0,0,0) );
			getScene()->getRootState()->staticState->camera.setUp( makeVector3f(0,0,1) );
			getScene()->getRootState()->staticState->camera.setPosition( makeVector3f(0,3,4)*medPointDistance*det->getDescriptorRadius() );
			getScene()->getRootState()->staticState->viewFrustum.setZNearClip(medPointDistance*det->getDescriptorRadius()*0.001f);
			getScene()->getRootState()->staticState->viewFrustum.setZFarClip(medPointDistance*det->getDescriptorRadius()*10);
		}

		// find LDCorrelation descriptor
		unsigned correlationIndex = 0;
		LDCorrelation * corr = NULL;
		for( unsigned i=0;i<det->getNumLocalDescriptors();i++ )
		{
			corr = dynamic_cast<LDCorrelation*>(det->getLocalDescriptor(i));
			if( corr != NULL )
			{
				correlationIndex = i;
				break;
			}
		}

		if( !corr )
		{
			throw X4Exception("Correlation signature not found!");
		}

		CachedRangeQuerry querry(pc1);
		m_icp->beginICP(pc1,pc2);
		for( unsigned i=0;i<set->m_Correspondences.size();i++ )
		{
			FeatureCorrespondence * correspondence = set->m_Correspondences[i];
			GeometricFeature * f1 = fs2->m_Features[correspondence->m_FeatureIndex[0]];
			GeometricFeature * f2 = fs1->m_Features[correspondence->m_FeatureIndex[1]];

			LDCorrelationSignature * corrSigRef = (LDCorrelationSignature *)f1->m_Signatures[correlationIndex];
			Matrix4f refFrame = makeTranslation4f(f1->m_Position) * expand3To4(corrSigRef->m_LocalFrame);
			refFrame = invertFrame(refFrame);

			// gather points around selected feature for icp
			PointSet * ps = querry.getSpherePoints(f1->m_Position,medPointDistance*det->getDescriptorRadius(),0);

			LDCorrelationSignature * corrSigDest = (LDCorrelationSignature *)f2->m_Signatures[correlationIndex];

			std::vector<Matrix3f> dest_frames;
			corrSigRef->computeAlignment( corrSigDest, corr, dest_frames, false);

			//debugRenderer->addFineCoordinateFrame(makeTranslation4f(f2->m_Position) * expand3To4(dest_frames[0]), medPointDistance * det->getDescriptorRadius()
			//	, makeVector3f(1.0f,0.0f,0.0f),false);

			float bestResidual = 1e20f;
			Matrix4f bestFrame = IDENTITY4F;

			for( unsigned i=0;i<dest_frames.size();i++ )
			{
				Matrix4f tmpRefFrame = refFrame;
				Matrix4f destFrame = invertFrame(makeTranslation4f(f2->m_Position) * expand3To4(dest_frames[i]));
				debugRenderer->clearDebugData("icptest");
				debugRenderer->beginRenderJob("icptest");
				float residual = m_icp->computeFastICP(ps,tmpRefFrame, destFrame, medPointDistance*det->getDescriptorRadius() *0.5f );

				if( residual < bestResidual )
				{
					Matrix4f dest0 = invertFrame(destFrame);
					Matrix4f r0 = invertFrame(refFrame);
					Matrix4f r1 = invertFrame(tmpRefFrame);
					Matrix4f T = invertFrame(r1)*r0;
					bestFrame = dest0 * T;
					bestResidual = residual;
				}
				debugRenderer->endRenderJob();
			}
			//debugOutput << "best residual: " << bestResidual << "\n";

			if( bestResidual > m_MaxICPResidual	)
			{
				correspondence->m_FeaturePoint[0] = correspondence->m_FeaturePoint[1] = makeVector3f(0,0,0);
			}

		}
		m_icp->endICP();
		m_icp->setRedrawMethod(NULL);
		getScene()->getRootState()->staticState->viewFrustum = sceneVF;
		getScene()->getRootState()->staticState->camera = sceneCam;
		PCIRigidRegistration_currentObject = NULL;
	}

	for( unsigned i=(unsigned)set->m_Correspondences.size();i>0;i-- )
	{
		FeatureCorrespondence * corr = set->m_Correspondences[i-1];
		if( norm(corr->m_FeaturePoint[0]) < 1e-20f )
			set->deleteCorrespondences(i-1,i);
	}
	
	if( m_ValidateCorrespondences )
	{
		/*if( set->m_Correspondences.size() < 50 )
		{
			delete set;
			return new FeatureCorrespondenceSet;
		}*/
		Timer timer;
		FeatureCorrespondenceSet * ret = m_FeatureMatching->computeValidatedCorrespondences(fs2,fs1,set);
		debugOutput << "timing spectral val: " << convertTimeToString(timer.getDeltaValue()) << "\n";
		delete set;
		return ret;
	}
	else
		return set;
}

X4::Matrix4f X4::PCIRigidRegistration::computeICPAlignment( PointCloud * pc1, PointCloud * pc2 )
{
	card32 numSurfels = pc1->getNumPoints() / 50;
	card32 blockSize = pc1->getNumPoints() / numSurfels;
	PointSet * surfels = new PointSet;
	surfels->clearAndSetup(1, numSurfels, pc1->getDescr());
	std::vector<char> buffer(pc1->getDescr()->getSize());
	PointSet * src = ((UnstructuredInCorePointCloud*)pc1)->getPointSet();
	for( unsigned i=0;i<numSurfels;i++ )
	{
		unsigned index = rand() % blockSize;
		index += blockSize * i;
		src->getVertex(index,&buffer[0]);
		surfels->setVertex(i,&buffer[0]);
	}

	m_icp->beginICP(pc1,pc2);
	Matrix4f retMat = pc1->getTransformation();
	m_icp->computeFastICP(surfels, retMat, pc2->getTransformation(), 1e20f);
	m_icp->endICP();


	delete surfels;
	return retMat;
}

void X4::PCIRigidRegistration::automaticAlignment()
{
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();	
	if( clouds.size() < 2 )
		return;

	std::vector<UnstructuredInCorePointCloud*> aligned;
	UnstructuredInCorePointCloud * resPC = (UnstructuredInCorePointCloud *)clouds[0]->copy();
	aligned.push_back(clouds[0]);
	clouds.erase( clouds.begin());

	while( clouds.size() )
	{
		std::vector<float> score(clouds.size(),0);
		float maxScore = 0;
		for( unsigned i=0;i<clouds.size();i++ )
		{
			debugOutput << "Align cloud[" << i << "]\n";
			FeatureCorrespondenceSet * set = computeFeatureCorrespondences(resPC,clouds[i]);
			if( set->m_Correspondences.size() <= 3 )
			{
				debugOutput << " --> not enough correspondeces\n";
				continue;
			}

			Matrix4f transf = set->computeTransformationMatrix();

			// check for mirrored coordinate frame
			Matrix3f checkT = shrink4To3(transf);
			if( checkT[0].crossProduct(checkT[1])*checkT[2] < 0)
			{
				continue;
			}


			clouds[i]->setTransformation( resPC->getTransformation()*transf );
			Matrix4f fineTransf = computeICPAlignment(clouds[i],resPC);
			clouds[i]->setTransformation( fineTransf );
			float tmpScore = computeScore(resPC,clouds[i]);
			score[i] = tmpScore;
			maxScore = max(maxScore,tmpScore);
			debugOutput << " --> score = " << tmpScore << "\n";

			delete set;
		}

		if( maxScore == 0.0f)
			break;

		for( int i=(int)score.size()-1;i>=0;i-- )
		{
			if( score[i] < maxScore * 0.75f )
			{
				clouds[i]->setTransformation( IDENTITY4F );
			}
			else
			{
				merge( resPC, clouds[i]);
				mergeFeatures( resPC, clouds[i] );
				aligned.push_back(clouds[i]);
				clouds.erase(clouds.begin()+i);
				debugOutput << "merged: " << i << "\n";
			}
		}

	}

	addPointCloud(getScene(),resPC,"res");

	//delete resPC;
	
}

void X4::PCIRigidRegistration::mergeFeatures( PointCloud * pc1, PointCloud * pc2 )
{
	Matrix4f mat_2to1 = invertFrame(pc1->getTransformation()) * pc2->getTransformation();

	FeatureSet *fs1 = (FeatureSet*)pc1->getAttachments()->getData(FeatureSet::getDefaultName());
	FeatureSet *fs2 = (FeatureSet*)pc2->getAttachments()->getData(FeatureSet::getDefaultName());

	float medPointDist = getCachedMedianPointDistance(pc2);
	float minDiff = medPointDist * 3.0f;

	for( unsigned i=0;i<fs2->m_Features.size();i++ )
	{
		GeometricFeature * f2 = fs2->m_Features[i];
		Vector3f tPos = transformVector3f(mat_2to1,f2->m_Position);

		GeometricFeature * f1 = fs1->getNearestFeature( tPos );
		if( norm(tPos - f1->m_Position) < minDiff )
			continue;

		GeometricFeature * fAdd = (GeometricFeature *)f2->copy();
		fAdd->transformFeature(mat_2to1);
		fs1->addFeature(fAdd);
	}

}

void X4::PCIRigidRegistration::orientateFeatureNormals()
{
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();
	for( unsigned i=0;i<clouds.size();i++ )
	{
		FeatureSet *fs = (FeatureSet*)clouds[i]->getAttachments()->getData(FeatureSet::getDefaultName());
		fs->orientateNormals(clouds[i], getMedianPointDistance(clouds[i])*15.0f);
	}
}

void X4::PCIRigidRegistration::normalizeScene()
{
	std::vector<UnstructuredInCorePointCloud*> clouds = getAllPointClouds();
	BoundingBox3f bb;
	
	for( unsigned i=0;i<clouds.size();i++ )
	{
		AAT posAAT = clouds[i]->getAAT("position");
		SceneObjectIterator * iter = clouds[i]->createIterator(SOIT::CAP_BASIC_PC|SOIT::CAP_MODIFYABLE_PC);
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
		if( i == 0)
			bb.lowerCorner = bb.upperCorner = bIt->get3f(posAAT);

		while( !bIt->atEnd())
		{
			bb.addPoint(bIt->get3f(posAAT));
			bIt->next();
		}
	}

	Matrix4f scale = makeScale4f( makeVector3f(1,1,1) / bb.getMaxSideLength()) * makeTranslation4f(-bb.getCenter());

	for( unsigned i=0;i<clouds.size();i++ )
	{
		AAT posAAT = clouds[i]->getAAT("position");
		SceneObjectIterator * iter = clouds[i]->createIterator(SOIT::CAP_BASIC_PC|SOIT::CAP_MODIFYABLE_PC);
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
		while( !bIt->atEnd())
		{
			Vector3f pos = bIt->get3f(posAAT);
			mIt->set3f(posAAT, transformVector3f(scale,pos));
			bIt->next();
		}

		FeatureSet * fs = dynamic_cast<FeatureSet*>(clouds[i]->getAttachments()->getData(FeatureSet::getDefaultName()));
		if( fs )
		{
			for( unsigned i=0;i<fs->m_Features.size();i++ )
			{
				GeometricFeature * f = fs->m_Features[i];
				f->m_Position = transformVector3f( scale, f->m_Position);
			}
		}

		delete iter;
	}
	
}

void X4::PCIRigidRegistration::testDetection()
{
	// test different sampling conditions
}

void X4::PCIRigidRegistration::computeDeformCorresp()
{
	std::vector<UnstructuredInCorePointCloud*> pcs = getAllPointClouds();
	if( pcs.size() < 2)
		return;
	PointCloud * pc1 = pcs[0];
	PointCloud * pc2 = pcs[1];

	FeatureSet *fs2 = (FeatureSet*)pc1->getAttachments()->getData(FeatureSet::getDefaultName());
	FeatureSet *fs1 = (FeatureSet*)pc2->getAttachments()->getData(FeatureSet::getDefaultName());


	FeatureCorrespondenceSet * set = new FeatureCorrespondenceSet;

	// find LDCorrelation descriptor
	unsigned correlationIndex = 0;
	unsigned histogramIndex = 0;
	LDCorrelation * corr = NULL;
	LDHistogram  * hisDesc = NULL;
	FeaturePointDetector * det = m_FeatureMatching->getFeatureDetector();
	for( unsigned i=0;i<det->getNumLocalDescriptors();i++ )
	{
		LDCorrelation * tmp_corr = dynamic_cast<LDCorrelation*>(det->getLocalDescriptor(i));
		if( tmp_corr != NULL )
		{
			corr = tmp_corr;
			correlationIndex = i;
		}

		LDHistogram * tmp_hisDesc = dynamic_cast<LDHistogram*>(det->getLocalDescriptor(i));
		if( tmp_hisDesc )
		{
			hisDesc = tmp_hisDesc;
			histogramIndex = i;
		}
	}

	float medPointDistance = getCachedMedianPointDistance(pc1);
	std::vector<TCorrespondenceCounter> correspondenceCounter(fs1->m_Features.size());

	for( unsigned i=0;i<fs1->m_Features.size();i++ )
	{
		correspondenceCounter[i].numCorrespondingFeatures = 0;
		correspondenceCounter[i].featureID = i;

		GeometricFeature * f1 = fs1->m_Features[i];
		if( f1->badFeature(det) )
			continue;


		LDCorrelationSignature * corrSigRef = (LDCorrelationSignature *)f1->m_Signatures[correlationIndex];
		PointSet *ps = NULL;
		//CachedRangeQuerry querry(pc1);
		Camera sceneCam = getScene()->getRootState()->staticState->camera;
		ViewFrustum sceneVF = getScene()->getRootState()->staticState->viewFrustum;
		Matrix4f refFrame;

		card32 numCorrespondingFeatures = 0;
		float bestDist = 1e20f;
		//TCorrespondenceDistance maxDist;
		std::vector<TCorrespondenceDistance> distances;

		for( unsigned j=0;j<fs2->m_Features.size();j++ )
		{
			GeometricFeature * f2 = fs2->m_Features[j];
			if( f2->badFeature(det) )
				continue;

			float dist = f1->m_Signatures[histogramIndex]->compare(f2->m_Signatures[histogramIndex],hisDesc);
			if( dist < 1.0f)
			{
				bestDist = min(bestDist,dist);
				TCorrespondenceDistance d;
				d.dist = dist;
				d.featureID = j;
				distances.push_back(d);
			}

		}

		sort( distances.begin(), distances.end());

		for( unsigned j=0;j<distances.size() && j<m_FeatureMatching->getMaxNumCorrespondingFeatures();j++ )
		{
			if( distances[j].dist > distances[0].dist*1.2f )
			{
				break;
			}
			FeatureCorrespondence * corr = new FeatureCorrespondence;
			corr->m_FeatureIndex[1] = i;
			corr->m_FeatureIndex[0] = distances[j].featureID;
			corr->m_FeaturePoint[1] = f1->m_Position;
			corr->m_FeaturePoint[0] = fs2->m_Features[distances[j].featureID]->m_Position;
			corr->m_Distance = distances[j].dist;
			set->m_Correspondences.push_back(corr);
			numCorrespondingFeatures++;
		}
		correspondenceCounter[i].numCorrespondingFeatures = numCorrespondingFeatures;

	}

	if( m_TestICPClustering )
		m_icp->endICP();

	//sort(correspondenceCounter.begin(), correspondenceCounter.end());
	std::vector<bool> keepFeature(fs1->getNumFeatures(),false);

	card32 maxNumCorrespondences = m_FeatureMatching->getMaxNumInitialCorrespondences();
	card32 numCorr = 0;
	for( unsigned i=0;i<correspondenceCounter.size();i++ )
	{
		numCorr += correspondenceCounter[i].numCorrespondingFeatures;
		keepFeature[correspondenceCounter[i].featureID] = correspondenceCounter[i].numCorrespondingFeatures <= m_FeatureMatching->getMaxNumCorrespondingFeatures();
	}

	FeatureCorrespondenceSet * prunedSet = new FeatureCorrespondenceSet;
	for( unsigned i=0;i<set->m_Correspondences.size();i++ )
	{
		FeatureCorrespondence * corr = set->m_Correspondences[i];
		if( keepFeature[corr->m_FeatureIndex[1]] )
		{
			prunedSet->m_Correspondences.push_back( (FeatureCorrespondence*)corr->copy());
		}
	}

	delete set;
	set = prunedSet;

	debugOutput << "num initial corr: " << set->m_Correspondences.size() << "\n";

	if( set->m_Correspondences.size() > maxNumCorrespondences )
	{
		set->sort();
		set->deleteCorrespondences(maxNumCorrespondences, (int)set->m_Correspondences.size());
	}


	if( m_TestICPClustering )
	{
		PCIRigidRegistration_currentObject = this;
		Camera sceneCam = getScene()->getRootState()->staticState->camera;
		ViewFrustum sceneVF = getScene()->getRootState()->staticState->viewFrustum;
		if( m_VisualizeICP )
		{
			m_icp->setRedrawMethod(PCIRigidRegistration_redraw);
			getScene()->getRootState()->staticState->camera.setLookAt( makeVector3f(0,0,0) );
			getScene()->getRootState()->staticState->camera.setUp( makeVector3f(0,0,1) );
			getScene()->getRootState()->staticState->camera.setPosition( makeVector3f(0,3,4)*medPointDistance*det->getDescriptorRadius() );
			getScene()->getRootState()->staticState->viewFrustum.setZNearClip(medPointDistance*det->getDescriptorRadius()*0.001f);
			getScene()->getRootState()->staticState->viewFrustum.setZFarClip(medPointDistance*det->getDescriptorRadius()*10);
		}

		// find LDCorrelation descriptor
		unsigned correlationIndex = 0;
		LDCorrelation * corr = NULL;
		for( unsigned i=0;i<det->getNumLocalDescriptors();i++ )
		{
			corr = dynamic_cast<LDCorrelation*>(det->getLocalDescriptor(i));
			if( corr != NULL )
			{
				correlationIndex = i;
				break;
			}
		}

		if( !corr )
		{
			throw X4Exception("Correlation signature not found!");
		}

		CachedRangeQuerry querry(pc1);
		m_icp->beginICP(pc1,pc2);
		for( unsigned i=0;i<set->m_Correspondences.size();i++ )
		{
			FeatureCorrespondence * correspondence = set->m_Correspondences[i];
			GeometricFeature * f1 = fs2->m_Features[correspondence->m_FeatureIndex[0]];
			GeometricFeature * f2 = fs1->m_Features[correspondence->m_FeatureIndex[1]];

			LDCorrelationSignature * corrSigRef = (LDCorrelationSignature *)f1->m_Signatures[correlationIndex];
			Matrix4f refFrame = makeTranslation4f(f1->m_Position) * expand3To4(corrSigRef->m_LocalFrame);
			refFrame = invertFrame(refFrame);

			// gather points around selected feature for icp
			PointSet * ps = querry.getSpherePoints(f1->m_Position,medPointDistance*det->getDescriptorRadius(),0);

			LDCorrelationSignature * corrSigDest = (LDCorrelationSignature *)f2->m_Signatures[correlationIndex];

			std::vector<Matrix3f> dest_frames;
			corrSigRef->computeAlignment( corrSigDest, corr, dest_frames, false);

			//debugRenderer->addFineCoordinateFrame(makeTranslation4f(f2->m_Position) * expand3To4(dest_frames[0]), medPointDistance * det->getDescriptorRadius()
			//	, makeVector3f(1.0f,0.0f,0.0f),false);

			float bestResidual = 1e20f;
			Matrix4f bestFrame = IDENTITY4F;

			for( unsigned i=0;i<dest_frames.size();i++ )
			{
				Matrix4f tmpRefFrame = refFrame;
				Matrix4f destFrame = invertFrame(makeTranslation4f(f2->m_Position) * expand3To4(dest_frames[i]));
				debugRenderer->clearDebugData("icptest");
				debugRenderer->beginRenderJob("icptest");
				float residual = m_icp->computeFastICP(ps,tmpRefFrame, destFrame, medPointDistance*det->getDescriptorRadius() *0.5f );

				if( residual < bestResidual )
				{
					Matrix4f dest0 = invertFrame(destFrame);
					Matrix4f r0 = invertFrame(refFrame);
					Matrix4f r1 = invertFrame(tmpRefFrame);
					Matrix4f T = invertFrame(r1)*r0;
					bestFrame = dest0 * T;
					bestResidual = residual;
				}
				debugRenderer->endRenderJob();
			}
			//debugOutput << "best residual: " << bestResidual << "\n";

			if( bestResidual > m_MaxICPResidual	)
			{
				correspondence->m_FeaturePoint[0] = correspondence->m_FeaturePoint[1] = makeVector3f(0,0,0);
			}
			else
			{
				correspondence->m_FeaturePoint[1] = shrink4To3(bestFrame[3]);
			}

		}
		m_icp->endICP();
		m_icp->setRedrawMethod(NULL);
		getScene()->getRootState()->staticState->viewFrustum = sceneVF;
		getScene()->getRootState()->staticState->camera = sceneCam;
		PCIRigidRegistration_currentObject = NULL;
	}

	for( unsigned i=(unsigned)set->m_Correspondences.size();i>0;i-- )
	{
		FeatureCorrespondence * corr = set->m_Correspondences[i-1];
		if( norm(corr->m_FeaturePoint[0]) < 1e-20f )
			set->deleteCorrespondences(i-1,i);
	}

	/*FeatureSet * fset1 = new FeatureSet;
	FeatureSet * fset2 = new FeatureSet;
	std::map<unsigned,unsigned> fsetMap1;

	for( unsigned i=0;i<set->m_Correspondences.size();i++ )
	{
		FeatureCorrespondence * corr = set->m_Correspondences[i];
		if( fsetMap1[corr->m_FeatureIndex[0]] == fsetMap1.end() )
		{
			fset1->m_Features.push_back( (GeometricFeature*)fs1->getFeature(corr->m_FeatureIndex[0])->copy() );
			fsetMap1[corr->m_FeatureIndex[0]] = fset1->m_Features.size()-1;
		}
		corr->m_FeatureIndex[0] = fsetMap1[corr->m_FeatureIndex[0]];
		fset2->m_Features.push_back() (GeometricFeature*)fs2->getFeature(corr->m_FeatureIndex[1])->copy() );
	}*/


	if( m_ValidateCorrespondences )
	{
		/*if( set->m_Correspondences.size() < 50 )
		{
		delete set;
		return new FeatureCorrespondenceSet;
		}*/
		Timer timer;
		FeatureCorrespondenceSet * ret = m_FeatureMatching->computeValidatedCorrespondences(fs2,fs1,set);
		debugOutput << "timing spectral val: " << convertTimeToString(timer.getDeltaValue()) << "\n";
		delete set;
		set = ret;
	}


	debugRenderer->clearDebugData("deform");
	debugRenderer->beginRenderJob("deform");
	for( unsigned i=0;i<set->m_Correspondences.size();i++ )
	{
		Vector3f p1 = transformVector3f(pc1->getTransformation(),set->m_Correspondences[i]->m_FeaturePoint[0]);
		Vector3f p2 = transformVector3f(pc2->getTransformation(),set->m_Correspondences[i]->m_FeaturePoint[1]);
		debugRenderer->addFastCylinder( p1, normalize(p2-p1), norm(p2-p1), medPointDistance*m_FeatureSize, makeVector3f(255,174,0)/255,true);
	}
	debugRenderer->endRenderJob();

	delete m_LastCorrSet;
	m_LastCorrSet = set;

}

void X4::PCIRigidRegistration::removeOverlappingFeatures()
{
	std::vector<UnstructuredInCorePointCloud*> pcs = getAllPointClouds();
	for( unsigned i=0;i<pcs.size();i++ )
	{
		FeatureSet * fs = dynamic_cast<FeatureSet*>(pcs[i]->getAttachments()->getData(FeatureSet::getDefaultName()));
		if( !fs )
			continue;

		card32 numRemoved = 0;
		float medPointDistance = getMedianPointDistance(pcs[i]);
		for( unsigned f1=(unsigned)fs->m_Features.size();f1>0;f1-- )
		{
			unsigned i1 = f1-1;
			GeometricFeature * f = fs->m_Features[i1];
			SlippageSignature * slipSig1 = NULL;
			if( f->m_Signatures.size())
				slipSig1 = dynamic_cast<SlippageSignature*>(f->m_Signatures[0]);

			float bestDist = 1e20f;
			unsigned bi = 0;
			for( unsigned f2=f1-1;f2>0;f2-- )
			{
				unsigned i2 = f2-1;
				float dist = norm(f->m_Position - fs->m_Features[i2]->m_Position);
				if( dist < bestDist )
				{
					bi = i2;
					bestDist = min( bestDist, dist);
				}
			}

			if( bestDist < medPointDistance * 6.0f )
			{
				if( slipSig1 )
				{
					// switch features if feature i is less slippable
					SlippageSignature * slipSig2 = (SlippageSignature*)fs->m_Features[bi]->m_Signatures[0];
					if( slipSig1->m_Unslippability > slipSig2->m_Unslippability)
					{
						fs->m_Features[i1] = fs->m_Features[bi];
						fs->m_Features[bi] = f;
					}
				}

				fs->deleteFeatures(i1,i1+1);
				numRemoved++;
			}
		}

		debugOutput << "removed " << numRemoved << " overlapping features\n";
	}
}

void X4::PCIRigidRegistration::exportFeaturePoints()
{
	std::vector<UnstructuredInCorePointCloud*> pcs = getAllPointClouds();
	if( pcs.size() == 0)
		return;

	for( unsigned i=0;i<pcs.size();i++ )
	{
		FeatureSet * fs = dynamic_cast<FeatureSet*>(pcs[i]->getAttachments()->getData(FeatureSet::getDefaultName()));
		if( !fs )
			continue;

		QString filename = QFileDialog::getSaveFileName(NULL,"save features");
		if( filename.isEmpty() )
			return;

		QFile f(filename);
		if (!f.open(QIODevice::WriteOnly | QIODevice::Text))
			return;
		QTextStream out(&f);

		for( unsigned j=0;j<fs->m_Features.size();j++ )
		{
			QString line;
			GeometricFeature * gf = fs->m_Features[j];
			SlippageSignature * slipSig = dynamic_cast<SlippageSignature*>(gf->m_Signatures[0]);
			if( slipSig )
				if( slipSig->badFeature(m_FeatureMatching->getFeatureDetector()->getLocalDescriptor(0)))
					continue;

			for( unsigned k=0;k<3;k++ )
			{
				line += QString::number(gf->m_Position[k]) + " ";
			}
			/*if( slipSig )
			{
				line += QString::number(slipSig->m_Scale) + " ";
				line += QString::number(slipSig->m_Unslippability);
			}*/
			for( unsigned k=0;k<m_FeatureMatching->getFeatureDetector()->getNumLocalDescriptors();k++ )
			{
				
				if( dynamic_cast<LDHistogram*>(m_FeatureMatching->getFeatureDetector()->getLocalDescriptor(k)))
				{
					FloatVectorSignature * sig = dynamic_cast<FloatVectorSignature*>(gf->m_Signatures[k]);
					line += sig->printToString().c_str();
				}
			}

			line += "\n";
			out << line;
			
		}

		break;
	}	
}