#ifndef PCIRigidRegistrationH
#define PCIRigidRegistrationH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------

namespace X4
{
class FeaturePointDetector;
class PointCloud;
class UnstructuredInCorePointCloud;
class FeatureMatching;
class RenderingTool;
class LocalICP;
class FeatureCorrespondenceSet;
/**
	provides tools for rigid registration
*/
class X3FOUNDATION_PRIVATE_API PCIRigidRegistration : public PCInteractionTool
{
	X4_CLASS( PCIRigidRegistration )

private:

	FeatureMatching	*		m_FeatureMatching;
	bool							m_ValidateCorrespondences;
	RenderingTool *			m_RenderingTool;
	bool							m_ShowFeaturePoints;
	float32						m_FeatureSize;
	LocalICP	*					m_icp;
	bool							m_VisualizeICP;
	bool							m_TestICPClustering;
	float32						m_MaxICPResidual;
	bool							m_DeleteOnBurn;
	FeatureCorrespondenceSet * m_LastCorrSet;

	// test members
	Vector3f						m_SelectedFeaturePoint1;
	Vector3f						m_SelectedFeaturePoint2;
	PointCloud*					m_SelectedPC1;
	PointCloud*					m_ICP_PointCloud0;


	PointCloud * getSelectedPointCloud();
	std::vector<UnstructuredInCorePointCloud*> getAllPointClouds();
	void alignTwoClouds( std::string pc1, std::string pc2 );
	float computeScore( UnstructuredInCorePointCloud * pc1, UnstructuredInCorePointCloud * pc2);

	FeatureCorrespondenceSet * computeFeatureCorrespondences( PointCloud * pc1, PointCloud * pc2 );
	Matrix4f computeICPAlignment( PointCloud * pc1, PointCloud * pc2);
	void mergeFeatures( PointCloud * pc1, PointCloud * pc2);

public:
	PCIRigidRegistration(void);
	~PCIRigidRegistration(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual void keyDown(GeneralKey key) {}
	virtual void keyUp(GeneralKey key) {}

	virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
	virtual void mouseMoved(int32 x, int32 y) {}
	virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {}
	virtual void areaResize(card32 width, card32 height) {}

	virtual void glDrawTool(GLContext * context);

	void computeFeaturesOneFrame();
	void computeCorrespondencesOneFrame();
	void computeDeformCorresp();

	void filterFeaturesOneFrame();
	void alignClouds();
	void resetTransformations();
	void icp();
	void icp_setCloud0();
	void setTransformationFromString();
	void testFeatureClustering();
	void featureBurning();
	void orientateFeatureNormals();
	void normalizeScene();
	void removeOverlappingFeatures();
	void testDetection();


	void testSmoothing();
	void precomputeFeatures();
	void recomputeSignatures();
	void recomputeGeodeticDistances();
	void computeCorrespondences();
	void automaticAlignment();
	void exportFeaturePoints();



	// debug methods
	void debug_redraw();
};

}

#endif