#include "StdAfx.h"
#include <windows.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#include "PCITestGeometricFeatures.h"
//---------------------------------------------------------------------------
#include "MultiScaleFeatureDetector.h"
#include "CopyObjectProperties.h"
#include "ObjectClassProperty.h"
#include "StandardSceneGraphIterator.h"
#include "scene.h"
#include "PointCloudTools.h"
#include "SGObjectNode.h"
#include "GeometricFeature.h"
#include "GeometricTools.h"
#include "ClassMethods.h"
#include "TableOutputTools.h"
#include "StringHelper.h"
#include "VertexArray.h"
#include "DebugRenderer.h"
#include "Random.h"
#include "LocalICP.h"
#include "NormalEstimator.h"
#include "SlippageFeatureDetector.h"
#include "PCCCacheOptimizer.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( PCITestGeometricFeatures ,0)
{
	BEGIN_CLASS_INIT( PCITestGeometricFeatures );

	ADD_OBJECT_PROP( m_Detector ,0,FeaturePointDetector::getClass(),true);
	ADD_OBJECT_PROP( m_icp, 0, LocalICP::getClass(),true);
	ADD_CARD32_PROP(m_NumTestsPerAmplitude,0);
	ADD_BOOLEAN_PROP( m_ViewTests, 0);
	ADD_NOARGS_METHOD( noiseFeatureTest );
	ADD_NOARGS_METHOD( noiseFeatureTest_MultipleScales );
	ADD_NOARGS_METHOD(geodeticTest);
	ADD_NOARGS_METHOD(computeSlipage);
	ADD_NOARGS_METHOD(noiseFeatureTest_Guskov);
}

PCITestGeometricFeatures::PCITestGeometricFeatures(void)
{
	m_Detector = new SlippageFeatureDetector;
	m_icp = new LocalICP;
	m_NumTestsPerAmplitude = 2;
}

PCITestGeometricFeatures::~PCITestGeometricFeatures(void)
{
	delete m_Detector;
	delete m_icp;
}

void PCITestGeometricFeatures::assign(const Object* obj, X4CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCITestGeometricFeatures * o = dynamic_cast<const PCITestGeometricFeatures*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

PointCloud * generateNoisyTestPointCloud(PointCloud * pc, float noiseAmpl)
{
	PointCloud * ret = createNoisyCopy( pc, noiseAmpl, 200.0f*noiseAmpl, true );
	/*float medPointDistance = getMedianPointDistance(pc,5,5000);
	float normalSigma = noiseAmpl * 5.0f;
	if( normalSigma < medPointDistance * 5.0f )
	{
		normalSigma = medPointDistance * 5.0f;
	}
	normalSigma = medPointDistance*2.5f + noiseAmpl;// medPointDistance * 5.0f + noiseAmpl*3.0f;
	NormalEstimator::estimateNormals(ret, 20, normalSigma, -1, ENVIRONMENT_EPSILON);
	VertexArray va1(pc);
	VertexArray va2(ret);
	for( unsigned i=0;i<va1.getNumElements();i++ )
	{
		if( va1.getNormal3f(i)*va2.getNormal3f(i) < 0 )
			va2.setNormal3f(i,-va2.getNormal3f(i));

	}*/
	VertexArray va1(ret);
	for( unsigned i=0;i<va1.getNumElements();i++ )
	{
		Vector3f normal = va1.getNormal3f(i);
		normal += (makeVector3f(rnd01(),rnd01(),rnd01())*2 - makeVector3f(1,1,1)) * 0.05f;
		normal = normalize(normal);
		va1.setNormal3f(i,normal);
	}
	return ret;
}

void X4::PCITestGeometricFeatures::noiseFeatureTest()
{
	PointCloud * pc = getPC();
	if( !pc )
		return;

	float medPointDistance = getMedianPointDistance(pc);
	std::vector<UnstructuredInCorePointCloud*> clouds;
	clouds.push_back( (UnstructuredInCorePointCloud*)pc);
	BoundingBox3f bb = getPCBBox(pc);

	/*PointCloud * test = generateNoisyTestPointCloud(pc, bb.getMaxSideLength() * 0.0025f);
	addPointCloud(getScene(),test,"noisy");
	return;*/

	m_Detector->begin( clouds );
	
	std::vector<float> testAmplitudes;
	//testAmplitudes.push_back(0.0025f);
	testAmplitudes.push_back(0.005f);
	//testAmplitudes.push_back(0.0075f);
	//testAmplitudes.push_back(0.01f);

	std::string tableName = "feature-stability with noisy point clouds";
	tableOutputTool->clear();
	tableOutputTool->setColumnName( tableName, 0, "noise amplitude" );
	tableOutputTool->setColumnName( tableName, 1, "num features" );
	tableOutputTool->setColumnName( tableName, 2, "0.5" );
	tableOutputTool->setColumnName( tableName, 3, "1.0" );
	tableOutputTool->setColumnName( tableName, 4, "1.5" );

	for( unsigned i=0;i<testAmplitudes.size();i++ )
	{
		float noiseAmpl = testAmplitudes[i] * bb.getMaxSideLength();

		// compute reference set
		PointCloud * refPC = generateNoisyTestPointCloud(pc, noiseAmpl);
		FeatureSet * referenceSet = m_Detector->detect( (UnstructuredInCorePointCloud*)refPC, "test");

		if( m_ViewTests )
		{
			debugDrawPointCloud(refPC, "refPC");
		}

		float numFeatures_05 = 0;
		float numFeatures_10 = 0;
		float numFeatures_20 = 0;
		float numFeatures = 0;

		tableOutputTool->setRowName(tableName,i,std::string("test #") + intToStr(i));
		tableOutputTool->setData(tableName,i,0,testAmplitudes[i]);

		for( unsigned tests = 0;tests<m_NumTestsPerAmplitude;tests++)
		{
			PointCloud * noisyPC = generateNoisyTestPointCloud(pc, noiseAmpl);
			FeatureSet * testSet = m_Detector->detect( (UnstructuredInCorePointCloud*)noisyPC, "test" );
			if( m_ViewTests )
			{
				debugDrawPointCloud(noisyPC, "testPC");
			}
			numFeatures += referenceSet->getNumFeatures();


			for( unsigned j=0;j<referenceSet->getNumFeatures();j++ )
			{
				float bestdist = 1e20f;
				for( unsigned k=0;k<testSet->getNumFeatures();k++ )
				{
					float dist = norm( referenceSet->m_Features[j]->m_Position - 
						testSet->m_Features[k]->m_Position );

					bestdist = min(dist,bestdist);
				}

				numFeatures_05 += ((bestdist < medPointDistance * 0.5f) ? 1.0f : 0.0);
				numFeatures_10 += ((bestdist < medPointDistance * 1.0f) ? 1.0f : 0.0);
				numFeatures_20 += ((bestdist < medPointDistance * 2.0f) ? 1.0f : 0.0);
			}

			if( m_ViewTests )
			{
				deletePointCloud(getScene(), "root/testPC");
			}
			else
				delete noisyPC;
		}

		tableOutputTool->addData(tableName,i,1,numFeatures/ (float)m_NumTestsPerAmplitude);
		tableOutputTool->addData(tableName,i,2,numFeatures_05 / (float)m_NumTestsPerAmplitude);
		tableOutputTool->addData(tableName,i,3,numFeatures_10 / (float)m_NumTestsPerAmplitude);
		tableOutputTool->addData(tableName,i,4,numFeatures_20 / (float)m_NumTestsPerAmplitude);

		if( m_ViewTests )
			deletePointCloud( getScene(), "root/refPC");
		else
			delete refPC;
	}


	debugOutput << tableOutputTool->outputSimpleTable(tableName, TableOutputTools::DF_FloatPoint);

	m_Detector->end();
}

PointCloud * X4::PCITestGeometricFeatures::getPC()
{
	std::vector<PointCloud*> ret;
	const std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	for( card32 i=0;i<selection.size();i++ )
	{
		PointCloud * pc = getPointCloud( getScene(), selection[i] );
		if( pc )
			return pc;
	}

	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	return pc;
}

void X4::PCITestGeometricFeatures::geodeticTest()
{
	PointCloud * pc = getPC();
	UnstructuredInCorePointCloud * upc= dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	if( upc == NULL )
		return;

	// pick a set of random points
	VertexArray va(upc->getPointSet());
	std::vector<Vector3f> points;
	for( unsigned i=0;i<40;i++ )
	{
		points.push_back(va.getPosition3f(random32bit(upc->getNumPoints())));
	}

	
	FixedKNNTopologyIterator topologyIter( upc->getPointSet(), upc->getAttachments(), upc, 8 );
	float medPointDistance = getMedianPointDistance(upc);

	
	std::vector< std::vector<Vector3f> > paths;
	std::vector<float> distances = computeGeodeticDistances( pc, &topologyIter, points[0], points, &paths );

	debugRenderer->clearDebugData("geodetic");
	debugRenderer->beginRenderJob("geodetic");
	for( unsigned i=0;i<paths.size();i++ )
	{
		for( unsigned j=0;j<paths[i].size()-1;j++ )
		{
			float linInterp = (float)j / (float)(paths[i].size()-1);
			Vector3f color = makeVector3f(1,0,0) * linInterp + makeVector3f(1,1,.1f)*(1.0f - linInterp);
			debugRenderer->addLine( paths[i][j],paths[i][j+1],color, color, 2);
		}
	}
	debugRenderer->endRenderJob();

	// compare geodetic distances with euklidians
	float averageDifference = 0;
	float averageDiffFactor = 0;
	for( unsigned i=0;i<points.size();i++ )
	{
		float euklidDist = norm(points[0]-points[i]);
		debugOutput << "euklid Dist: " << euklidDist / medPointDistance
			<< "\t difference: " << (distances[i] - euklidDist)/ medPointDistance
		<< "\t difference factor: ";
		if( euklidDist > 1e-10f )
			debugOutput << (distances[i] - euklidDist) / euklidDist << "\n";
		else
			debugOutput << "#\n";
		averageDifference += (distances[i] - euklidDist)/ medPointDistance / (float)(points.size()-1);
		if( euklidDist > 1e-10f )
			averageDiffFactor += (distances[i] - euklidDist)/ euklidDist / (float)(points.size()-2);
	}
	debugOutput << "averageDifference: " << averageDifference << "\naverageDiffFactor: " << averageDiffFactor << "\n";
}

void X4::PCITestGeometricFeatures::mouseDown( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState )
{
	static Vector3f p1;
	static int clicks = 0;
	static PointCloud * refPC = NULL;

	PointCloud * pc;
	Vector3f point;
	std::string name;
	if( getIntersectionFromCursor(x,y, point, name, &pc) )
	{
		if( buttonsState.getLeft() )
		{
			refPC = pc;
			p1 = point;
		}
		
		if( buttonsState.getRight() && refPC)
		{
			debugRenderer->clearDebugData("asdf2");
			debugRenderer->beginRenderJob("asdf2");
			float med = getMedianPointDistance(refPC);
			m_icp;
			m_icp->beginICP(refPC, pc);
			Matrix3f rot = IDENTITY3F;
			Vector3f oldPoint = point;
			if( m_icp->computeICP(p1, point, rot) )
			{
			}
			m_icp->endICP();
			debugRenderer->endRenderJob();
		}
		clicks ++;
	}
}

void X4::PCITestGeometricFeatures::computeSlipage()
{
	PointCloud * pc = getPC();
	UnstructuredInCorePointCloud * upc= dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	if( upc == NULL )
		return;

	checkAttribute(upc,"color",3,VAD::DATA_FORMAT_FLOAT32);
	checkAttribute(upc,"rcolor",3,VAD::DATA_FORMAT_FLOAT32);

	float startSigma = getMedianPointDistance(pc) * 2.5f;
	
	for( unsigned tests = 0;tests<6;tests++)
	{
		m_icp->setSigma(startSigma);
		startSigma *= pow(2.0f,0.25f);
	
	VertexArray arr(upc->getPointSet());
	float maxEigen5 = 0;
	
	m_icp->beginICP(pc,pc);
	for( unsigned i=0;i<arr.getNumElements();i++ )
	{
		Vector3f pos = arr.getPosition3f(i);
		Vector3f normal = arr.getNormal3f(i);

		Matrix6f covariance;
		Vector6f eigenValues;
		m_icp->computeSlippage6d(pos, covariance, eigenValues);
		maxEigen5 = max(maxEigen5, min(eigenValues[5],1.0f));
		arr.setColor3f(i,makeVector3f(eigenValues[5],eigenValues[4],eigenValues[3]));

	}
	m_icp->endICP();
	debugOutput << "max eigen: " << maxEigen5 << "\n";
	for( unsigned i=0;i<arr.getNumElements();i++ )
	{
		arr.setColor3f(i, arr.getColor3f(i) / maxEigen5);
	}
	//upc->getAttachments()->deleteDelOnWriteData(AttachedData::CT_EVERYTHING_CHANGED);
	upc->clearAttachments();
	redrawScene();
	
	return;

	float medPointDistance = getMedianPointDistance(upc);
	float ms_h = 5.0f * medPointDistance;
	AAT posAAT = upc->getAAT("position");
	AAT colorAAT = upc->getAAT("color");
	float radius_2 = pow( 2.0f * ms_h,2.0f);
	float inv_sigma_2 = 1.0f / pow(ms_h,2.0f);
	debugRenderer->clearDebugData("meanshift");
	debugRenderer->beginRenderJob("meanshift");
	HierarchicalKNNIterator hIt(upc,32,NULL);
	FixedKNNTopologyIterator topIt(upc->getPointSet(), upc->getAttachments(), pc, 10);
	std::map<unsigned,std::vector<unsigned> > edgeMap;
	std::vector<Vector3f> bestPos(arr.getNumElements());
	std::vector<float> summedWeights(arr.getNumElements());
	for( unsigned i=0;i<arr.getNumElements();i++ )
	{
		//if( rand() % 100 != 0)
		//	continue;

		Vector3f pos = arr.getPosition3f(i);
		Vector3f initialPos = pos;
		std::vector<Vector3f> topNeighbours;
		std::vector<unsigned> topNeighbourIndices;

		topIt.goToIndex(i);
		for( unsigned j=0;j<topIt.getNumNeighbours();j++)
		{
			unsigned n = topIt.getNeighbour(j);
			topNeighbourIndices.push_back(n);
			topNeighbours.push_back(arr.getPosition3f(n));
		}

		float bestNeighbourDistance = 1e20f;
		unsigned bestNeighbourIndex = topNeighbourIndices[0];

		for( unsigned j=0;j<30;j++ )
		{
			hIt.setSeekPointAndReset(pos);
			float summed_weight = 0;
			Vector3f nextPos = NULL_VECTOR3F;
			while( !hIt.atEnd())
			{
				Vector3f x = hIt.get3f(posAAT);
				Vector3f c = hIt.get3f(colorAAT);
				float dist_2 = normQuad(pos - x);
				if( dist_2 > radius_2)
					break;
			
				float weight = exp( -dist_2 * inv_sigma_2) * c[0];
				summed_weight += weight;
				nextPos += x * weight;

				hIt.next();
			}
			nextPos /= summed_weight;
			
			{
				/*Vector3f col = makeVector3f(1,0,0);
				if( norm(nextPos - pos) < medPointDistance * 0.5f )
					col = makeVector3f(0,100,0);
				debugRenderer->addLine(pos,nextPos, col *j / 14,col *(j+1) / 14,1);*/
			}

			for( unsigned n=0;n<topNeighbours.size();n++ )
			{
				float dist = norm(nextPos - topNeighbours[n]);
				if( dist < bestNeighbourDistance )
				{
					bestNeighbourDistance = dist;
					bestNeighbourIndex = topNeighbourIndices[n];
				}
			}

			pos = nextPos;
			if( norm(initialPos - pos) > medPointDistance * 0.9f || j == 29)
			{
				//debugRenderer->addLine(initialPos, arr.getPosition3f(bestNeighbourIndex), makeVector3f(1,0,0),makeVector3f(1,1,0),1);
				summedWeights[i] = summed_weight;
				bestPos[i] = pos;
				edgeMap[bestNeighbourIndex].push_back(i);
				edgeMap[i].push_back(bestNeighbourIndex);
				break;
			}
		}
	}
	
	unsigned regionID = 0;
	std::vector<unsigned> region( arr.getNumElements(), 0);
	AAT regionColorAAT = upc->getAAT("rcolor");
	for( unsigned i=0;i<arr.getNumElements();i++ )
	{
		if( region[i] != 0)
			continue;

		regionID++;
		Vector3f rndColor = makeVector3f(rnd01(),rnd01(),rnd01());
		Vector3f bestPoint;
		float bestWeight=0;

		list<unsigned> queue;
		queue.push_back(i);

		while( !queue.empty())
		{
			unsigned index = queue.front();
			queue.pop_front();

			if( region[index] != 0)
				continue;

			region[index] = regionID;
			if( summedWeights[index] > bestWeight )
			{
				bestWeight = summedWeights[index];
				bestPoint = bestPos[index];
			}
			//arr.set(index,regionColorAAT, rndColor);
			arr.setColor3f(index,rndColor);
			for( unsigned j=0;j<edgeMap[index].size();j++ )
			{
				queue.push_back(edgeMap[index][j]);
			}

		}

		debugRenderer->addSphere(bestPoint, medPointDistance, makeVector3f(1,0,0));
		
	}

//	upc->getAttachments()->deleteDelOnWriteData(AttachedData::CT_EVERYTHING_CHANGED);
	upc->clearAttachments();
	redrawScene();
	}
	debugRenderer->endRenderJob();


}

void X4::PCITestGeometricFeatures::noiseFeatureTest_MultipleScales()
{
	unsigned numScales = 4;
	for( unsigned i=0;i<numScales;i++  )
	{
		SlippageFeatureDetector * slippage = dynamic_cast<SlippageFeatureDetector*>(m_Detector);
		if( !slippage)
			return;

		float sigma = slippage->getSigma();
		debugOutput << "Noisetest: sigma=" << sigma << "\n";
		noiseFeatureTest();
		debugOutput.flush();
		slippage->setSigma( sigma * sqrt(2.0f));
	}
}

void X4::PCITestGeometricFeatures::debugDrawPointCloud( PointCloud * pc, std::string name )
{
	addPointCloud( getScene(), pc, name);
//	redrawScene(); // draw with default material

	unsigned matInd = pc->getMaterialIndex();
	pc->setMaterialIndex(2);
//	redrawScene(); // draw with color material
	BoundingBox3f bb = getPCBBox(pc);

	FeatureSet * fs = (FeatureSet*)pc->getAttachments()->getData("test");
	debugRenderer->clearDebugData("drawf");
	debugRenderer->beginRenderJob("drawf");
	for( unsigned f=0;f<fs->getNumFeatures();f++ )
	{
		debugRenderer->addSphere(fs->m_Features[f]->m_Position, bb.getMaxSideLength() * 0.01f, makeVector3f(1,0.2f,0.1f));
	}
	redrawScene(); // render with feature set
	debugRenderer->endRenderJob();
	debugRenderer->clearDebugData("drawf");
	pc->setMaterialIndex(1000); // make cloud invisible	
}

void X4::PCITestGeometricFeatures::multipleScanTest()
{
	std::vector<PointCloud*> clouds = getSelectedPointClouds();
	if( clouds.size() == 0)
		return;


}


std::vector<PointCloud*> X4::PCITestGeometricFeatures::getSelectedPointClouds()
{
	std::vector<PointCloud*> ret;
	const std::vector<std::string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	for( card32 i=0;i<selection.size();i++ )
	{
		PointCloud * pc = getPointCloud( getScene(), selection[i] );
		if( pc )
			ret.push_back(pc);
	}
	return ret;
}

void X4::PCITestGeometricFeatures::noiseFeatureTest_Guskov()
{
	PointCloud * pc = getPC();
	if( !pc )
		return;

	float medPointDistance = getMedianPointDistance(pc);
	std::vector<UnstructuredInCorePointCloud*> clouds;
	clouds.push_back( (UnstructuredInCorePointCloud*)pc);
	BoundingBox3f bb = getPCBBox(pc);

	/*PointCloud * test = generateNoisyTestPointCloud(pc, bb.getMaxSideLength() * 0.0025f);
	addPointCloud(getScene(),test,"noisy");
	return;*/

	m_Detector->begin( clouds );

	std::vector<float> testAmplitudes;
	//testAmplitudes.push_back(0.0025f);
	testAmplitudes.push_back(0.005f);
	//testAmplitudes.push_back(0.0075f);
	//testAmplitudes.push_back(0.01f);

	std::string tableName = "feature-stability with noisy point clouds";
	tableOutputTool->clear();
	tableOutputTool->setColumnName( tableName, 0, "noise amplitude" );
	tableOutputTool->setColumnName( tableName, 1, "num features" );
	tableOutputTool->setColumnName( tableName, 2, "num corresponding features" );

	SlippageFeatureDetector * det = dynamic_cast<SlippageFeatureDetector*>(m_Detector);
	card32 numScales = 0;
	if( det )
		numScales = det->getNumScales()+1;

	for( unsigned i=0;i<numScales;i++ )
	{
		tableOutputTool->setColumnName( tableName, i+3, std::string("#f") + intToStr(i) );
		//tableOutputTool->setColumnName( tableName, i+3+numScales, std::string("#corr") + intToStr(i) );
	}

	for( unsigned i=0;i<testAmplitudes.size();i++ )
	{
		float noiseAmpl = testAmplitudes[i] * bb.getMaxSideLength();

		// compute reference set
		PointCloud * refPC = (PointCloud*)pc->copy();//generateNoisyTestPointCloud(pc, noiseAmpl);
		FeatureSet * referenceSet = m_Detector->detect( (UnstructuredInCorePointCloud*)refPC, "test");

		if( m_ViewTests )
		{
			debugDrawPointCloud(refPC, "refPC");
		}

		float numCorrFeatures = 0;
		float numFeatures = 0;
		std::vector<float> numFeaturesScale(numScales,0);
		std::vector<float> numCorrFeaturesScale(numScales,0);

		tableOutputTool->setRowName(tableName,i,std::string("test #") + intToStr(i));
		tableOutputTool->setData(tableName,i,0,testAmplitudes[i]);

		for( unsigned tests = 0;tests<m_NumTestsPerAmplitude;tests++)
		{
			PointCloud * noisyPC = generateNoisyTestPointCloud(pc, noiseAmpl);
			PCCCacheOptimizer::optimize((UnstructuredInCorePointCloud*)noisyPC, 2.7f * getMedianPointDistance(noisyPC));
			FeatureSet * testSet = m_Detector->detect( (UnstructuredInCorePointCloud*)noisyPC, "test" );
			if( m_ViewTests )
			{
				debugDrawPointCloud(noisyPC, "testPC");
			}
			numFeatures += referenceSet->getNumFeatures();


			for( unsigned j=0;j<referenceSet->getNumFeatures();j++ )
			{
				bool corresponding = false;
				GeometricFeature * f = referenceSet->m_Features[j];
				SlippageSignature * slipSig = dynamic_cast<SlippageSignature*>(f->m_Signatures[0]);
				if( !slipSig )
					throw X4Exception("Use this test for slippage features only");

				for( unsigned k=0;k<testSet->getNumFeatures();k++ )
				{
					GeometricFeature * f2 = testSet->m_Features[k];
					SlippageSignature * slipSig2 = dynamic_cast<SlippageSignature*>(f2->m_Signatures[0]);
					float dist = norm( f->m_Position - f2->m_Position );

					if( dist < slipSig->m_Sigma * 0.5f && fabs(slipSig->m_Scale - slipSig2->m_Scale) < 1.0f )
					{
						corresponding = true;
						break;
					}
				}


				//bool corresponding = (bestdist < medPointDistance*1.5f);
				numCorrFeatures += ( corresponding? 1.0f : 0.0);
				if( corresponding)
					numCorrFeaturesScale[slipSig->m_Scale] += 1.0f;
				numFeaturesScale[slipSig->m_Scale] += 1.0f;
			}

			if( m_ViewTests )
			{
				deletePointCloud(getScene(), "root/testPC");
			}
			else
				delete noisyPC;
		}

		tableOutputTool->addData(tableName,i*2,1,numFeatures/ (float)m_NumTestsPerAmplitude);
		tableOutputTool->addData(tableName,i*2,2,numCorrFeatures / (float)m_NumTestsPerAmplitude);
		tableOutputTool->addData(tableName,i*2+1,1,numFeatures/ (float)m_NumTestsPerAmplitude);
		tableOutputTool->addData(tableName,i*2+1,2,numCorrFeatures / (float)m_NumTestsPerAmplitude);
		for( unsigned s=0;s<numScales;s++ )
		{
			tableOutputTool->addData(tableName,i*2,s+3,numFeaturesScale[s] / (float)m_NumTestsPerAmplitude);
			tableOutputTool->addData(tableName,i*2+1,s+3,numCorrFeaturesScale[s] / (float)m_NumTestsPerAmplitude);
		}

		if( m_ViewTests )
			deletePointCloud( getScene(), "root/refPC");
		else
			delete refPC;
	}


	debugOutput << tableOutputTool->outputSimpleTable(tableName, TableOutputTools::DF_FloatComma);

	m_Detector->end();
}

void X4::PCITestGeometricFeatures::testApproximateAlignment()
{

}

void X4::PCITestGeometricFeatures::testWithAlignedScans()
{

}