//---------------------------------------------------------------------------
#ifndef PointCloudProcessingRendererH
#define PointCloudProcessingRendererH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
#include "PointCloud.h"
#include "IteratorAdaptors.h"
#include "LinearAlgebra.h"
#include "DebugOutput.h"
#include "RenderingTool.h"
#include "Scene.h"
//---------------------------------------------------------------------------

namespace X4 {

/// Debug rendering functionality for the point cloud processing operations */
class X3FOUNDATION_PRIVATE_API PointCloudProcessingRenderer : public Persistent {

	X4_CLASS( PointCloudProcessingRenderer )

	private:
			RenderingTool* rt;
			int32 monitorPoint;
			float32 renderScale;
			Scene* scene;
			std::string* pcName;

			bool showTensor;
			bool showNormals;
			bool showTangents;
			bool showEpsilon;
			bool showMonomials;
			bool showMeanCurvature;
			bool showGaussianCurvature;

			// Color code
			void colorCode();
			float32 curvMin;
			float32 curvMax;
		
	public:
		PointCloudProcessingRenderer();
		~PointCloudProcessingRenderer();

		void render(
			const float32 epsilon, 
			const card32 knn,
			PointCloud* pc );

		inline void incMonitorPoint(){ monitorPoint++; }
		inline void decMonitorPoint(){ monitorPoint--; }

		inline void setup( Scene* scene, std::string* pcName ){ this->scene = scene; this->pcName = pcName; }
};
}

#endif

