//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PointSetANNQuery.h"
//---------------------------------------------------------------------------
#include "BoundingBox.inline.h"
#include "Persistent.h"
#include "Tools.h"
#include "DebugOutput.h"
#include "PointSet.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



PointSetANNQuery::PointSetANNQuery(const PointSet *ps, const card32 &nk)
{

	k=nk;
	numPoints=ps->getNumEntries();
	distances=new float32[k];
	indexes  =new int32[k];
	data = new ANNpoint[numPoints];
	for (card32 i=0; i<numPoints; i++)
	{
		data[i]=(ANNpoint)ps->getDataPointer(i);
	}

	
	kdTree = new ANNkd_tree(
		data,					         // the data points
		numPoints,						   // number of points
		3);						// dimension of space

	position=0;
}

PointSetANNQuery::~PointSetANNQuery(void)
{
	delete kdTree;
	delete[] data;
	delete[] distances;
	delete[] indexes;
}

void X4::PointSetANNQuery::setTargetPoint( const Vector3f &targetPoint )
{
	x4Assert(k<=numPoints);
	Vector3f temp=targetPoint;
	kdTree->annkSearch(					
		temp.data(),						// query point
		k,								// number of near neighbors
		indexes,							// nearest neighbors (returned)
		distances,							// distance (returned)
		0.0f);							// error bound
	position=0;

}

void X4::PointSetANNQuery::getNextNearestPointIndex( card32 &nextIndex, bool &found )
{
	x4Assert(position<k);
	nextIndex=indexes[position];
	position++;
	found=(nextIndex!= ANN_NULL_IDX);
}

int32 * X4::PointSetANNQuery::getAllIndices( card32 &returnedNumOfIndices )
{
	returnedNumOfIndices=k;
	return indexes;
}

X4::int32 X4::PointSetANNQuery::getNearestPointIndex( const Vector3f &targetPosition ) const
{
	x4Assert(k>=1);
	Vector3f temp(targetPosition);
	kdTree->annkSearch(					
		temp.data(),						// query point
		1,								// number of near neighbors
		indexes,							// nearest neighbors (returned)
		distances,							// distance 
		0.0f);							// error bound
	//position=0;
	return indexes[0];
}

X4::int32 X4::PointSetANNQuery::getNearestPointIndexAndSqrDistance( const Vector3f &targetPosition, float32 &returnedDistance )
{
	x4Assert(k>=1);
	Vector3f temp(targetPosition);
	kdTree->annkSearch(					
		temp.data(),						// query point
		1,								// number of near neighbors
		indexes,							// nearest neighbors (returned)
		distances,							// distance (returned)
		0.0f);							// error bound
	position=0;
	returnedDistance=distances[0];
	return indexes[0];
}

//=====================

X4::PointSetRangeQuery::PointSetRangeQuery( const PointSet *ps, const float32& sqrRad1 )
{
	k=1;
	this->sqrRad=sqrRad1;
	numPoints=ps->getNumEntries();
	distances=new float32[numPoints];
	indexes  =new int32[numPoints];
	data = new ANNpoint[numPoints];
	for (card32 i=0; i<numPoints; i++)
	{
		data[i]=(ANNpoint)ps->getDataPointer(i);
	}

	kdTree = new ANNkd_tree(
		data,					         // the data points
		numPoints,						   // number of points
		3);						// dimension of space

	position=0;
	lastK=1;
}


void X4::PointSetRangeQuery::setTargetPoint( const Vector3f &targetPoint )
{
	Vector3f temp=targetPoint;
	k= kdTree->annkFRSearch(					
		temp.data(),						// query point
		sqrRad,								
		0);						


	k= kdTree->annkFRSearch(					
			temp.data(),						// query point
			sqrRad,								// number of near neighbors
			k,
			indexes,							// nearest neighbors (returned)
			distances,							// distance (returned)
			0.0f);							// error bound
	

	position=0;
}

void X4::PointSetRangeQuery::getNextNearestPointIndex( card32 &nextIndex, bool &found )
{
	x4Assert(position<=k);
	nextIndex=indexes[position];
	found=(position<k);
	position++;
}

void X4::PointSetRangeQuery::setRangeSqrRad( const float32 &sqrRa )
{
	this->sqrRad=sqrRa;
}

void X4::PointSetRangeQuery::setTargetPointRadius( const Vector3f &targetPoint, const float32 &tempSqrRad )
{
	Vector3f temp=targetPoint;
	k= kdTree->annkFRSearch(					
		temp.data(),						// query point
		tempSqrRad,								
		0);						

	k= kdTree->annkFRSearch(					
		temp.data(),						// query point
		tempSqrRad,							
		k,
		indexes,							// nearest neighbors (returned)
		distances,							// distance (returned)
		0.0f);							// error bound


	position=0;
}