//---------------------------------------------------------------------------
#ifndef QuaternionH
#define QuaternionH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------

namespace X4 {

/// Quaternion representation
class X3FOUNDATION_PRIVATE_API Quaternion
{
	private:
		float32 v[4];

	public:
		/// Constructors
		Quaternion();
		Quaternion( float32 q0, float32 q1, float32 q2, float32 q3 );

		/// Quaternion operations
		Quaternion operator*( const Quaternion& q ) const;
		Quaternion operator+( const Quaternion& q ) const;
		void operator+=( const Quaternion& q );
		void operator/=( const float32 s );
		Quaternion operator*( const float32& s ) const;
		float32& operator[]( const card32 i ) { return v[i]; }
		float32 operator[]( const card32 i ) const { return v[i]; }
		/// Rotate point/std::vector via quaternion; return rotated point/std::vector
		Vector3f rotate( const Vector3f& p ) const;
		/// Compute inverse of quaternion
		Quaternion inverse() const;
		/// Create copy of quaternion
		Quaternion copy() const;
		/// Get rotation matrix from quaternion
		Matrix3f getRotMat() const;
		/// Print to console
		void println() const;
		/// Convert quaternion representation from rotation matrix
		static Quaternion fromRotMat(Matrix3f R);
		/// Normalize
		void normalize();
};

}

#endif