//---------------------------------------------------------------------------
#include "stdafx.h"
#include <windows.h>
#include <mmsystem.h>
#include "Ransac.h"
//#include "ImportanceSampling.h"
#include "GeometricFeature.h"
#include "FixedArrayClassProperty.h"
#include "ObjectClassProperty.h"
#include "GeometricFeature.h"
#include "FeatureCorrespondence.h"
#include "SeparatorClassProperty.h"
#include "DiscreteManifold.h"
#include "ProgressWindow.h"
#include "Random.h"
#include <opencv\cxcore.h>

//---------------------------------------------------------------------------
using namespace X4;

//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( RansacSolution, 0 )
{
	BEGIN_CLASS_INIT( RansacSolution );

	ADD_FLOAT32_PROP (error, 0);
	ADD_FLOAT32_PROP (geodeticsLength, 0);
	ADD_CARD32_PROP (numGeodetics,0);

	ADD_OBJECT_PROP (fcs, 0, FeatureCorrespondenceSet::getClass(), false);
}


IMPLEMENT_X4_CLASS(RansacRegistration, 0)
{
	BEGIN_CLASS_INIT(RansacRegistration);

	//ADD_OBJECT_FIXED_ARRAY_PROP (m_FeatureSet, 0, 2, FeatureSet::getClass(), false);
	//ADD_OBJECT_FIXED_ARRAY_PROP (m_Manifold, 0, 2, Manifold::getClass(), false);
	//ADD_OBJECT_PROP(m_ImportanceSampler, 0, ImportanceSampler::getClass(), true);

	ADD_SEPARATOR("Matching");
	ADD_FLOAT32_PROP(m_Sigma, 0);
	ADD_FLOAT32_PROP(m_maxSupportDifference, 0);
	ADD_FLOAT32_PROP(m_descriptorScoreFactor, 0);
	ADD_CARD32_PROP(m_initialMatchCount, 0);
	ADD_BOOLEAN_PROP(m_bSortByNumberOfCorrespondencies, 0);
	ADD_FLOAT32_PROP(m_UncertaintyEpsilonFactor, 0);

	ADD_SEPARATOR("Iteration");
	ADD_CARD32_PROP(m_maxIterations, 0);
	ADD_CARD32_PROP(m_minNumberOfMatches, 0);
	ADD_FLOAT32_PROP(m_minMatchError, 0);
	ADD_BOOLEAN_PROP(m_bUseBestMatchWhenRansac, 0);

	ADD_SEPARATOR("Settings");
	ADD_INT32_PROP(m_useSupportedOfNumGeodesics, 0);
	ADD_FLOAT32_PROP(m_supportByAtLeastPercent, 0);
	ADD_BOOLEAN_PROP(m_bCheckMirroring, 0);
	ADD_BOOLEAN_PROP(m_ignoreSameInitialCorrespondence, 0);
	SET_USER_EDIT(false);
	ADD_BOOLEAN_PROP(m_putInitialMatchIntoSolution, 0);
	ADD_BOOLEAN_PROP(m_bLookForGoodFeatureWhenRansac, 0);
}


//---------------------------------------------------------------------------
RansacRegistration::RansacRegistration()
{
	setManifold(NULL, NULL, NULL, NULL);
	//m_ImportanceSampler = new ImportanceSampler;

	m_Sigma = 0.2f;
	m_UncertaintyEpsilonFactor = 1.0f;

	m_maxIterations = 5000;
	m_minNumberOfMatches = -1;
	m_minMatchError = 0.01f;
	m_initialMatchCount = 3;
	m_maxSupportDifference = 0.05f;
	m_descriptorScoreFactor = 0.5f;

	m_supportByAtLeastPercent = 0.75f;
	m_useSupportedOfNumGeodesics = 4;

	m_ignoreSameInitialCorrespondence = true;
	m_putInitialMatchIntoSolution = true;
	m_bUseBestMatchWhenRansac = true;
	m_bSortByNumberOfCorrespondencies = true;
	m_bCheckMirroring = true;
	m_bLookForGoodFeatureWhenRansac = true;
}

//---------------------------------------------------------------------------
RansacRegistration::~RansacRegistration()
{
	//for (unsigned i=0; i < 2; i++)
	//	if (m_ManifoldPointList[i]) delete m_ManifoldPointList[i];
}
#if 0
//---------------------------------------------------------------------------
void RansacRegistration::setImportanceSampler(ImportanceSampler* sampler)
{
	if (m_ImportanceSampler) delete m_ImportanceSampler;
	m_ImportanceSampler = dynamic_cast<ImportanceSampler*>(sampler->copy());
}
#endif
//---------------------------------------------------------------------------
std::vector<FeatureCorrespondence*> RansacRegistration::InitialCallback::operator()(DiscreteManifold* mfSide, FeatureSet* fsSide, FeatureCorrespondenceSet* fcs, unsigned number)
{
	// variables
	std::vector<FeatureCorrespondence*> resultList;
	std::vector<FeatureCorrespondence*> fcList = fcs->m_Correspondences;

	// just some checks
	if (number > fcs->getNumCorrespondences()) return fcs->m_Correspondences;
	if (number == 0) return resultList;
	if (fcs->getNumCorrespondences() <= 1) return fcs->m_Correspondences;

	float sigma2 = _sigma * _sigma;

	for (unsigned i=0; i < fcList.size(); i++)
	{         
		float32 prob = exp(-(fcList[i]->m_Distance*fcList[i]->m_Distance) / sigma2);
		ImportanceSample* sample = new ImportanceSample(prob);
		sample->setUserData(fcList[i]);
		sampler.addSample(sample);
	}

	while(resultList.size() < number && fcList.size())
	{   
		if (sampler.getNumSamples())
		{      
			ImportanceSample* sample = sampler.sample();
			FeatureCorrespondence* fc = static_cast<FeatureCorrespondence*>(sample->getUserData());

			// check if this match is positive
			bool ok = true;
			for (unsigned i=0; i < resultList.size(); i++)
				if (fc->m_FeatureIndex[0] == resultList[i]->m_FeatureIndex[0] || fc->m_FeatureIndex[1] == resultList[i]->m_FeatureIndex[1])
				{ok = false; break; }

			if (ok) resultList.push_back(fc);
			else
			{
				// remove that one, so that it can not be sampled next time
				std::vector<FeatureCorrespondence*>::iterator it = fcList.begin();
				for (; it != fcList.end(); it++)
					if (*it == fc) {fcList.erase(it); break;}
			}

			sampler.removeSample(sample->getSampleID());
			delete sample;

		}else
			number = 0;

	}
	for (unsigned i=0; i < sampler.getNumSamples(); i++) delete sampler.getSample(i);
	sampler.clearSamples();  


	return resultList;

}

//---------------------------------------------------------------------------
class InterestingInitCallback : public RansacRegistration::InitialCallback
{
public:
	InterestingInitCallback(float sigma) : RansacRegistration::InitialCallback(sigma) {}
	std::vector<FeatureCorrespondence*> operator()(Manifold* mfSide, FeatureSet* fsSide, FeatureCorrespondenceSet* fcs, unsigned number)
	{
		// variables
		std::vector<FeatureCorrespondence*> resultList;
		std::vector<FeatureCorrespondence*> fcList = fcs->m_Correspondences;

		// just some checks
		if (number > fcs->getNumCorrespondences()) return fcs->m_Correspondences;
		if (number == 0) return resultList;
		if (fcs->getNumCorrespondences() <= 1) return fcs->m_Correspondences;

		float sigma2 = _sigma * _sigma;

		// get random feature points 
		std::vector<unsigned> featurePoints;
		while (featurePoints.size() < number)
		{
			unsigned index = random(fsSide->getNumFeatures());
			if (std::find(featurePoints.begin(), featurePoints.end(), index) == featurePoints.end())
				featurePoints.push_back(index);
			else 
				continue;

			// put all correspondences of this feature point to the importance sampler
			for (unsigned i=0; i < fcList.size(); i++)
			{
				if (fcList[i]->m_FeatureIndex[0] == index)
				{
					float64 prob = exp(-float64((fcList[i]->m_Distance * fcList[i]->m_Distance) / sigma2));
					ImportanceSample* sample = new ImportanceSample(prob);
					sample->setUserData(fcList[i]);
					sampler.addSample(sample);
				}
			}

			// sample one and put into solution list
			ImportanceSample* sample = sampler.sample();
			FeatureCorrespondence* fc = static_cast<FeatureCorrespondence*>(sample->getUserData());
			resultList.push_back(fc);

			// remove all correspondences in the list which has both chosen feature points in common
			for (std::vector<FeatureCorrespondence*>::iterator it = fcList.begin(); it != fcList.end(); )
			{
				if ((*it)->m_FeatureIndex[0] == fc->m_FeatureIndex[0] || (*it)->m_FeatureIndex[1] == fc->m_FeatureIndex[1])
					it = fcList.erase(it);
				else 
					it ++;
			}

			// prepare sampler
			for (unsigned i=0; i < sampler.getNumSamples(); i++) delete sampler.getSample(i);
			sampler.clearSamples();
		}

		return resultList;
	}
};

//---------------------------------------------------------------------------
float RansacRegistration::computeCorrespondenceScore(const std::vector<X4::FeatureCorrespondence*>& inSet, FeatureCorrespondence* fc,
													 float* geodeticsLength, unsigned* supportCount)
{
	//std::vector<unsigned> support1, support2;
	//std::vector<float> diffs (inSet.size());
	//unsigned diffCount = 0;
	unsigned supportSize = 0;
	std::vector<float> diffs(inSet.size());

	float score = 1.0;
	if (*geodeticsLength) *geodeticsLength = 0.0f;
	if (*supportCount) *supportCount = 0;

	// convert feature poisition into manifold space (here just R3 euclidian space)
	DVectorF pos1 = makeDVector( fc->m_FeaturePoint[0] );
	DVectorF pos2 = makeDVectorL(3, fc->m_FeaturePoint[1].data());
	ManifoldPoint* pointA = getFeatureManifoldPoint(m_Manifold[0], m_FeatureSet[0], fc->m_FeatureIndex[0]);
	ManifoldPoint* pointB = getFeatureManifoldPoint(m_Manifold[1], m_FeatureSet[1], fc->m_FeatureIndex[1]);

	for (unsigned i=0; i < inSet.size(); i++)
	{
		// if on of the candidate's vertices is already in the inSet, then reject the candidate
		if ( (fc->m_FeatureIndex[0] == inSet[i]->m_FeatureIndex[0] || fc->m_FeatureIndex[1] == inSet[i]->m_FeatureIndex[1]) )
			return 0;
		else 
		{
			DiscreteManifoldPoint* distA = getFeatureManifoldPoint(m_Manifold[0], m_FeatureSet[0], inSet[i]->m_FeatureIndex[0]);
			DiscreteManifoldPoint* distB = getFeatureManifoldPoint(m_Manifold[1], m_FeatureSet[1], inSet[i]->m_FeatureIndex[1]);

			float32 dist1 = m_Manifold[0]->getGeodeticDistance(pointA, distA);
			float32 dist2 = m_Manifold[1]->getGeodeticDistance(pointB, distB);

			if (geodeticsLength)
			{
				*geodeticsLength += dist1;
				*geodeticsLength += dist2;         
			}
			float fd = fabs(dist1-dist2);
			if (dist1 > 1e20 || dist2 > 1e20) fd = 100.0f; 

			if (fd < m_maxSupportDifference)
			{
				supportSize ++;
				//support1.push_back(inSet[i]->m_FeatureIndex[0]);
				//support2.push_back(inSet[i]->m_FeatureIndex[1]);
				if (supportCount) *supportCount = *supportCount + 1;
			}

			//diffs.push_back(fabs(min<float>(dist1,dist2)/max<float>(dist1,dist2)));
			//diffs.push_back(fd);
			diffs[i] = fd;
		}
	}

	// check if the correspondence is not an outlier
	//if ( (float(support1.size()) / float(diffs.size())) < m_supportByAtLeastPercent)
	if ( (float(supportSize) / float(inSet.size())) < m_supportByAtLeastPercent)
		return 0;

	// sort the differences in ascending order, so that only the best survives which is better when handling with outliers
	std::sort(diffs.begin(), diffs.end());

	for (int i=0; i < (int)supportSize && (m_useSupportedOfNumGeodesics > 0 ? i < m_useSupportedOfNumGeodesics : true); i++)
	{
		score *= computeScore(diffs[i] + fabs(float(m_descriptorScoreFactor * fc->m_Distance)));
	}

	if (score < 0.001) score = 0;

	// check mirroring
	if(m_bCheckMirroring && inSet.size() >= 2)
	{
		DiscreteManifold* dfA = dynamic_cast<DiscreteManifold*>(m_Manifold[0]);
		DiscreteManifold* dfB = dynamic_cast<DiscreteManifold*>(m_Manifold[1]);

		if (dfA == NULL || dfB == NULL)
		{
			debugOutput << "Cannot check mirroring, because only discrete manifolds are supported!\n";
		}else
		{
			// create list of manifold points of the inSet
			std::vector<const ManifoldPoint*> inSetPointsA(inSet.size());
			std::vector<const ManifoldPoint*> inSetPointsB(inSet.size());

			const ManifoldPoint* pA = dynamic_cast<const ManifoldPoint*>(m_FeatureSet[0]->m_Features[fc->m_FeatureIndex[0]]->getUserData());
			const ManifoldPoint* pB = dynamic_cast<const ManifoldPoint*>(m_FeatureSet[1]->m_Features[fc->m_FeatureIndex[1]]->getUserData());
			Vector3f normalA = normalize(makeVector3f(pA->getNormal()[0], pA->getNormal()[1], pA->getNormal()[2]));
			Vector3f normalB = normalize(makeVector3f(pB->getNormal()[0], pB->getNormal()[1], pB->getNormal()[2]));

			for (unsigned i=0; i < inSet.size(); i++)
			{
				inSetPointsA[i] = dynamic_cast<const ManifoldPoint*>(m_FeatureSet[0]->m_Features[inSet[i]->m_FeatureIndex[0]]->getUserData());
				inSetPointsB[i] = dynamic_cast<const ManifoldPoint*>(m_FeatureSet[1]->m_Features[inSet[i]->m_FeatureIndex[1]]->getUserData());
			}

			// get paths from the new correspondence to the list of this points
			const std::vector<const std::vector<DVectorF>*>& pathA = dfA->getGeodeticPaths(pA, inSetPointsA);
			const std::vector<const std::vector<DVectorF>*>& pathB = dfB->getGeodeticPaths(pB, inSetPointsB);

			// check correspondence against each gradient and look if normal orientation changes
			for (unsigned i=0; i < inSet.size(); i++)
			{
				for (unsigned j=0; j < inSet.size(); j++)
				{         
					if (pathA[i]->size() <= 2 || pathA[j]->size() <= 2 || pathB[i]->size() <= 2 || pathB[j]->size() <= 2) continue;

					DVectorF dgrad1A = (pathA[i])->at(0) - (pathA[i])->at(2);
					DVectorF dgrad2A = (pathA[j])->at(0) - (pathA[j])->at(2);
					Vector3f grad1A = makeVector3f(dgrad1A[0], dgrad1A[1], dgrad1A[2]);
					Vector3f grad2A = makeVector3f(dgrad2A[0], dgrad2A[1], dgrad2A[2]);
					Vector3f nA = normalize(grad1A.crossProduct(grad2A));
					bool sA = (nA * normalA) < 0;

					DVectorF dgrad1B = (pathB[i])->at(0) - (pathB[i])->at(2);
					DVectorF dgrad2B = (pathB[j])->at(0) - (pathB[j])->at(2);
					Vector3f grad1B = makeVector3f(dgrad1B[0], dgrad1B[1], dgrad1B[2]);
					Vector3f grad2B = makeVector3f(dgrad2B[0], dgrad2B[1], dgrad2B[2]);
					Vector3f nB = normalize(grad1B.crossProduct(grad2B));
					bool sB = (nB * normalB) < 0;

					if (sA != sB) 
					{
						//debugOutput << "Mirroring found\n"; debugOutput.flush();
						return 0;
					}
				}
			}

		}
	}

	return score;
}

//---------------------------------------------------------------------------
bool RansacSolution::SortSolutionPredicate(const RansacSolution& a, const RansacSolution& b)
{
	return a.fcs->m_Correspondences.size() * a.numGeodetics > b.fcs->m_Correspondences.size() * b.numGeodetics;
}

#if 0
//---------------------------------------------------------------------------
void RansacRegistration::setupManifolds()
{
	// first setup manifold point list, so that we get faster access
	for (unsigned i=0; i < 2; i++)
	{
		if (m_Manifold[i] == NULL) continue;
		if (dynamic_cast<DiscreteManifold*>(m_Manifold[i]) == NULL) continue;
		DiscreteManifold* df = dynamic_cast<DiscreteManifold*>(m_Manifold[i]);

		ManifoldPointList* list = dynamic_cast<ManifoldPointList*>(m_Manifold[i]->getAttachments()->getData(ManifoldPointList::getDefaultName()));
		if (list && list->getNumManifoldPoints() != df->getNumPoints())
		{
			m_Manifold[i]->deleteAttachment(ManifoldPointList::getDefaultName());
			list = NULL;
		}
		if (list == NULL)
		{
			list = new ManifoldPointList;
			list->setup(ManifoldPointList::getDefaultName(), AttachedData::ADF_PERSISTENT);
			m_Manifold[i]->getAttachments()->attachData(list);
		}
		list->deleteManifoldPoints(0, (card32)list->getNumManifoldPoints());


		debugOutput << "Generate PointList for " << df->getNumPoints() << " points\n"; debugOutput.flush();

		// generate manifold for each point
		for (unsigned j=0; j < df->getNumPoints(); j++)
		{
			if ((j % 5000) == 0)debugOutput << j << "\n"; debugOutput.flush();
			list->addManifoldPoint(df->newPoint(j));
		}

		m_ManifoldPointList[i] = list;
	}

}
#endif 

//---------------------------------------------------------------------------
DiscreteManifoldPoint* RansacRegistration::getFeatureManifoldPoint(DiscreteManifold* manifold, FeatureSet* featureSet, mpcard featureIndex)
{
	GeometricFeature* feature = featureSet->m_Features[featureIndex];

	// check if it was stored as user data
	if (feature->mUserData)
	{
		return (DiscreteManifoldPoint*)(feature->mUserData);
	}

	// if not, then create it and put as user data
	DiscreteManifoldPoint* point = new DiscreteManifoldPoint(featureIndex, makeDVectorL(3, feature->m_Position.data()), makeDVectorL(3, feature->m_Normal.data()));
	feature->mUserData = point;
	return point;
}

//---------------------------------------------------------------------------
int RansacRegistration::findBestFeaturePoint(const std::vector<X4::FeatureCorrespondence*>& previousSet, int side)
{
	if (side != 0 && side != 1) return -1;

	// collect all feature points which are not in the previous set 
	ImportanceSampler sampler;

	std::vector<const ManifoldPoint*> targetPoints(previousSet.size());
	for (unsigned j=0; j < previousSet.size(); j++)
		targetPoints[j] = getFeatureManifoldPoint(m_Manifold[side], m_FeatureSet[side], previousSet[j]->m_FeatureIndex[side]);

	std::vector<int> indices(m_FeatureSet[side]->getNumFeatures());
	for (unsigned i=0; i < m_FeatureSet[side]->getNumFeatures(); i++)
	{
		indices[i] = i;

		// check if this one already exists in the previous set
		bool found = false;
		for (unsigned j=0; j < previousSet.size(); j++)
			if (previousSet[j]->m_FeatureIndex[side] == i) 
				{found = true; break;}
		if (found) continue;

		// get all paths from the feature point to all other points in the set
		DiscreteManifoldPoint* point = getFeatureManifoldPoint(m_Manifold[side], m_FeatureSet[side], i);
		std::vector<const std::vector<DVectorF>*> paths = m_Manifold[side]->getGeodeticPaths(point, targetPoints);
		//std::vector<float> dists = m_Manifold[side]->getGeodeticDistances(point, targetPoints);

		// compute stability for a given manifold point in respect to the other
		ImportanceSample* sample = new ImportanceSample(computeStability(point, paths));
		sampler.addSample(sample);
		sample->setUserData((void*)&indices[i]);
	}

	if (sampler.getNumSamples() == 0) return -1;

	ImportanceSample* sample = sampler.sample();
	int index = *((int*)sample->getUserData());

	for (unsigned i=0; i < sampler.getNumSamples(); i++)
		delete sampler.getSample(i);

	return index;
}

//---------------------------------------------------------------------------
float RansacRegistration::computeStability(DiscreteManifoldPoint* point, const std::vector<const std::vector<DVectorF>*> paths)
{
	// create list of valid gradients
	std::vector<Vector3f> grad;
	for (unsigned i=0; i < paths.size(); i++)
	{
		if (paths[i]->size() >= 2)
		{
			DVectorF dgrad = (paths[i])->at(1) - (paths[i])->at(0);
			Vector3f sgrad = makeVector3f(dgrad[0], dgrad[1], dgrad[2]);
			normalize(sgrad);
			grad.push_back(sgrad);
		}
	}

	// compute stability score of the feature point
	CvMat* A  = cvCreateMat(3, (int)grad.size(),CV_32FC1);
	CvMat* U  = cvCreateMat(3,3,CV_32FC1);
	CvMat* D  = cvCreateMat(3, (int)grad.size(),CV_32FC1);
	CvMat* V  = cvCreateMat( (int)grad.size(), (int)grad.size(),CV_32FC1);

	for (unsigned j=0; j < grad.size(); j++)
	{
		cvmSet(A, 0, j, grad[j][0]);
		cvmSet(A, 1, j, grad[j][1]);
		cvmSet(A, 2, j, grad[j][2]);
	}

	cvSVD(A, D, U, V, CV_SVD_U_T|CV_SVD_V_T); // A = U D V^T

	// get out the d values
	float d1 = fabs(cvmGet(D, 0, 0));
	float d2 = fabs(cvmGet(D, 1, 1));
	float stability = min<float>(d1,d2) / max<float>(d1,d2);

	cvReleaseMat(&A);
	cvReleaseMat(&U);
	cvReleaseMat(&D);
	cvReleaseMat(&V);

	return stability;
}

//---------------------------------------------------------------------------
RansacRegistration::SolutionList RansacRegistration::computeMatching(FeatureCorrespondenceSet* fcs, RansacRegistration::InitialCallback* callback, SolutionFoundCallback* foundCallback)
{
	SolutionList solutions;

	if (m_FeatureSet[0] == NULL || m_FeatureSet[1] == NULL || m_Manifold[0] == NULL || m_Manifold[1] == NULL) 
		return solutions;

	// current implementation only works on discrete manifolds
	//if (dynamic_cast<DiscreteManifold*>(m_Manifold[0]) == NULL || dynamic_cast<DiscreteManifold*>(m_Manifold[1]) == NULL) return solutions;
	//setupManifolds();
	//return solutions;

	InitialCallback* initCallback = callback;
	if (callback == NULL) initCallback = new InterestingInitCallback(m_Sigma);

	// this std::vector do store all previous initial results, to be able to compare
	std::vector<std::vector<FeatureCorrespondence*> > previousInitial;
	float maxError = 0;
	int thatWasWrongDecission = m_maxIterations;
	unsigned oldRansacIteration = m_maxIterations;

	progressWindow->pushStep(true, "Ransac");

	for (unsigned ransac=0; ransac < m_maxIterations && thatWasWrongDecission > 0; ransac++)
	{
		progressWindow->progress(100.0f * float(ransac) / float(m_maxIterations));

		debugOutput << "init\n";
		// this list do contain all the correspondencies, which we want to handle
		std::vector<FeatureCorrespondence*> fcList = fcs->m_Correspondences;
		std::vector<FeatureCorrespondence*> fcResultList = (*initCallback)(m_Manifold[0], m_FeatureSet[0], fcs, m_initialMatchCount);
		std::vector<char> markInitialMatches(fcResultList.size(), 1);

#if 0
		// check first if there was already the same initial match, if yes, then go to next iteration
		if (m_ignoreSameInitialCorrespondence == false)
		{
			std::vector<bool> foundsame(previousInitial.size() * m_initialMatchCount, false);
			for (unsigned k=0; k < fcResultList.size(); k++)
			{
				for (unsigned i=0; i < previousInitial.size(); i++)
				{
					if (std::find(previousInitial[i].begin(), previousInitial[i].end(), fcResultList[k]) != previousInitial[i].end())
						foundsame[i * m_initialMatchCount + k] = true;
				}
			}

			// now just check if there is any line containing only trues
			bool foundone = false;
			for (unsigned i=0; i < previousInitial.size(); i++)
			{
				bool exists = true;
				for (unsigned k=0; k < m_initialMatchCount; k++)
					if (foundsame[i * m_initialMatchCount + k] == false)
					{
						exists = false;
						break;
					}
					if (exists)
					{
						foundone = true;
						break;
					}
			}

			if (foundone)
			{
				thatWasWrongDecission--;
				m_maxIterations++;
				continue;
			}
			thatWasWrongDecission = oldRansacIteration;

			previousInitial.push_back(fcResultList);
			// TODO: implement comparison so that it is the same to have 1,2,3 or 3,1,2 or 2,1,3 or ...
		}      
#endif 

		unsigned supportingGeodetics = 0;
		float geodeticsLength = 0;

		debugOutput << "search\n";

		// just run until no correspondencies left
		while (true)
		{
			// first find best point on the source side
			// if no such point could be found, then stop this iteration
			int bestFeaturePoint = -1;
			if (m_bLookForGoodFeatureWhenRansac && fcResultList.size() > 2)
			{
				bestFeaturePoint = findBestFeaturePoint(fcResultList, 0);
				if (bestFeaturePoint < 0) break;
			}

			ImportanceSampler sampler;
			std::vector<unsigned> listOfSupportingGeodetics(fcList.size(), 0);
			std::vector<int> listOfCorrespondenceIds(fcList.size(), -1);
			std::vector<float> listOfGeodeticsLength(fcList.size(), 0);

			for (unsigned i=0; i < fcList.size(); i++)
			{
				if (fcList[i] == NULL || fcList[i]->m_Valid == false) continue;

				// if correspondence is not of the best feature point, then ignore it
				if (bestFeaturePoint != -1 && fcList[i]->m_FeatureIndex[0] != bestFeaturePoint) continue;

				card32 supportCount = 0;
				float glength = 0;
				float fcScore = computeCorrespondenceScore(fcResultList, fcList[i], &glength, &supportCount);
				if (fcScore > 0 && supportCount > 0)
				{
					ImportanceSample* sample = new ImportanceSample(fcScore);
					sample->setUserData(&(listOfCorrespondenceIds[i]));
					sampler.addSample(sample);
				}

				// just for easier access
				listOfSupportingGeodetics[i] = supportCount;
				listOfGeodeticsLength[i] = glength;
				listOfCorrespondenceIds[i] = i;
			}

			if (sampler.getNumSamples() == 0)
				break;

			int bestFc = *(static_cast<int*>(sampler.sample(m_bUseBestMatchWhenRansac)->getUserData()));

			supportingGeodetics += listOfSupportingGeodetics[bestFc];
			geodeticsLength += listOfGeodeticsLength[bestFc];

			fcResultList.push_back(fcList[bestFc]);
			markInitialMatches.push_back(0);

			fcList[bestFc] = NULL;

			for (unsigned i=0; i < sampler.getNumSamples(); i++) delete sampler.getSample(i);
		}
		//debugOutput.flush();


		// create the correspondence set and compute the geodetics error
		FeatureCorrespondenceSet* result = new FeatureCorrespondenceSet();
		for (unsigned i = 0; i < fcResultList.size(); i++)
		{  
			if (m_putInitialMatchIntoSolution == false && markInitialMatches[i] == 1) continue;

			FeatureCorrespondence* fcNext  = (FeatureCorrespondence*)(fcResultList[i]);

			FeatureCorrespondence* fc = new FeatureCorrespondence;
			fc->m_FeatureIndex[0] = fcNext->m_FeatureIndex[0];
			fc->m_FeatureIndex[1] = fcNext->m_FeatureIndex[1];
			fc->m_FeaturePoint[0] = fcNext->m_FeaturePoint[0];
			fc->m_FeaturePoint[1] = fcNext->m_FeaturePoint[1];
			fc->m_Distance = fcNext->m_Distance;
			if (markInitialMatches[i]) fc->m_Distance *= -1;

			result->addCorrespondence(fc);         
		}

		// error      
		float error = computeCorrespondenceError(result); 
		if (error > maxError) maxError = error;

		// now we store the so computed graph to compare it later
		if (result)
		{
			RansacSolution s(error, supportingGeodetics, geodeticsLength, result);

			if (m_minNumberOfMatches > 0 && s.fcs->getNumCorrespondences() < m_minNumberOfMatches)
				continue;

			solutions.push_back(s);
			if (foundCallback) foundCallback->foundSolution(&s);

			// if good score then stop iterating
			if (m_minNumberOfMatches > 0 || m_minMatchError > 0)
			{
				bool donotuseMinError = m_minMatchError < 0;
				bool donotuseMinNumber = m_minNumberOfMatches < 0;

				if ((donotuseMinNumber || m_minNumberOfMatches <= s.fcs->getNumCorrespondences())
					&& (donotuseMinError || s.error < m_minMatchError + 1e-10f))
				break;
			}
		}	

	} // end ransac iterations
	progressWindow->popSteps();

	sortSolutions(solutions, m_bSortByNumberOfCorrespondencies);

	if (callback == NULL) delete initCallback;
	return solutions;
}

//---------------------------------------------------------------------------
void RansacRegistration::sortSolutions(SolutionList& solution, bool sortByNumberOfCorrespondenciesToo)
{
	if (solution.size() <= 1) return;

	SolutionList::iterator itbegin = solution.begin();
	SolutionList::iterator itend   = solution.end();

	if (sortByNumberOfCorrespondenciesToo)
	{   
		std::sort(itbegin, itend, RansacSolution::SortSolutionPredicate);

		itbegin = solution.begin();
		itend = solution.begin()+1;

		for (unsigned i=0; i < solution.size() && itend != solution.end(); i++)
		{
			if (itbegin->fcs->getNumCorrespondences() != itend->fcs->getNumCorrespondences())
			{
				std::sort(itbegin, itend);
				itbegin = itend;
			}
			itend++;
		}

	}

	std::sort(itbegin, itend);
}

//---------------------------------------------------------------------------
float RansacRegistration::computeCorrespondenceError(FeatureCorrespondenceSet* fcs)
{
	float error = 0;
	int count = 0;

	for (unsigned i = 0; i < fcs->m_Correspondences.size(); i++)
	{   
		FeatureCorrespondence* fc  = fcs->m_Correspondences[i];

		for (unsigned j = 0; j < fcs->m_Correspondences.size(); j++)
		{
			FeatureCorrespondence* fc2 = fcs->m_Correspondences[j];

			ManifoldPoint* cur0 = getFeatureManifoldPoint(m_Manifold[0], m_FeatureSet[0], fc->m_FeatureIndex[0]);
			ManifoldPoint* cur1 = getFeatureManifoldPoint(m_Manifold[0], m_FeatureSet[0], fc2->m_FeatureIndex[0]);
			ManifoldPoint* ref0 = getFeatureManifoldPoint(m_Manifold[1], m_FeatureSet[1], fc->m_FeatureIndex[1]);
			ManifoldPoint* ref1 = getFeatureManifoldPoint(m_Manifold[1], m_FeatureSet[1], fc2->m_FeatureIndex[1]);

			float cur = m_Manifold[0]->getGeodeticDistance(cur0, cur1);
			float ref = m_Manifold[1]->getGeodeticDistance(ref0, ref1);

			if (cur < 1e10f && ref < 1e10f)
			{
				error += (cur - ref)*(cur - ref);
				count ++;
			}
		}

	}

	if(count == 0) return FLT_MAX;

	return sqrt(error / float(count));
}
