//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include <windows.h>
#include <mmsystem.h>
#include "RegistrationDebug.h"
#include "ClassMethods.h"
#include "ObjectClassProperty.h"
#include "SeparatorClassProperty.h"
#include "ICP.h"
#include "NDT.h"
#include "GLShaderMaterial.h"
#include "RegistrationRenderer.h"
#include "NumericalClassProperty.h"
#include "BooleanClassProperty.h"
#include "FixedArrayClassProperty.h"
#
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS( RegistrationDebug, 4 )
{
	BEGIN_CLASS_INIT( RegistrationDebug )
	ADD_SEPARATOR( "General" )
	ADD_FLOAT32_PROP( renderScale, 0 )
	ADD_BOOLEAN_PROP( showDataStructure, 0 )
	ADD_BOOLEAN_PROP( showBBox, 3 )
	ADD_CARD32_PROP( monitorGrid, 2 )	
	ADD_VECTOR4F_PROP( dataStructureColor, 4 )
}

RegistrationDebug::RegistrationDebug()
{
	registerizer = NULL;
	rt = new RenderingTool();
	rt->init();
	renderScale = 1.0f;
	monitorGrid = -1;
	showDataStructure = true;
	showBBox = false;
	rr = new RegistrationRenderer( rt );
	dataStructureColor = makeVector4f( 0.5f, 0.5f, 0.5f, 0.3f );
}

RegistrationDebug::~RegistrationDebug()
{
	delete rt;
	delete rr;
}

void RegistrationDebug::renderGL()
{
	rt->enableLighting();
	if ( registerizer == NULL ){
		return;
	}

	ICP* icp = dynamic_cast<ICP*>( registerizer );
	NDT* ndt = dynamic_cast<NDT*>( registerizer );

	// ICP
	if ( icp != NULL ){
		// Octree
		if ( showDataStructure ){
			ICPOctree* octree = icp->getOctree();
			if ( octree != NULL ){
				rt->setRenderColor( dataStructureColor );
				glDisable( GL_LIGHTING );
				glPushAttrib( GL_ALL_ATTRIB_BITS );
				glEnable( GL_BLEND );
				glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
				rr->renderOctreeNode( octree->getOctree() );
				glPopAttrib();
			}
		} 
		// Bounding box
		if ( showBBox ){
			rt->setRenderColor( 1,1,1 );
			ICPOctree* octree = icp->getOctree();
			if ( octree != NULL ){
				rt->renderBox( octree->getOctree()->getBBox() );
			}
		}
	} 
	
	/// NDT
	if ( ndt != NULL ){
		// Grid + ndts
		if ( showDataStructure ){
			for ( unsigned int gridIndex = 0; gridIndex < ndt->getNumGrids(); gridIndex++ ){
				if ( ( monitorGrid == -1 ) || ( gridIndex == monitorGrid ) ){
					AbstractGrid* grid = ndt->getGrid( gridIndex );
					float32 c = 0.25 + ( gridIndex+1 ) / float32( 2.0f * ndt->getNumGrids() );
					rr->renderGrid( 
							grid, 
							makeVector4f( c,c,c, 1.0f ), 
							renderScale );
				}
			}
		}
		// Bounding box
		if ( showBBox ){
			AbstractGrid* grid = ndt->getGrid( 0 );
			if ( grid != NULL ){
				BoundingBox3f bbox = grid->getBBox();
				bbox.lowerCorner = bbox.lowerCorner /*/ ndt->getT()->getNormalizationS() + ndt->getT()->getNormalizationT()*/;
				bbox.upperCorner = bbox.upperCorner /*/ ndt->getT()->getNormalizationS() + ndt->getT()->getNormalizationT()*/;
				rt->setRenderColor( 1,1,1 );
				rt->renderBox( bbox );
			}
		}
	}
}

/*
void RegistrationDebug::getNodes( std::vector<ColorRegistrationNode*>& nodes, card32 colorRegistrationLevel, ColorRegistrationNode* node )
{
	if ( colorRegistrationLevel > 0 ){
		if ( !node->isLeaf() ){
			for ( unsigned int childIndex = 0; childIndex < 8; childIndex++ ){
				getNodes( nodes, colorRegistrationLevel-1, node->children[childIndex] );
			}
		} else {
			// Node does not have any children
			nodes.push_back( node );
		}
	} else {
		// Level 0 reached
		nodes.push_back( node );
	}
}
*/

/*
void RegistrationDebug::renderTreeNode( const ColorRegistrationNode* node )
{
	rt->setRenderColor( 1,1,1,1 );
	rt->renderBox( node->bbox );
	if ( !node->isLeaf() ){
		for ( unsigned int childIndex = 0; childIndex < 8; childIndex++ ){
			renderTreeNode( node->children[childIndex] );
		}
	}
}
*/

/*
void RegistrationDebug::renderGridLevel( MultiResolutionGrid<NDTCell>* mrGrid, const card32 level )
{
if ( ( mrGrid == NULL ) || ( level >= mrGrid->getNumLevels() ) ){
return;
}
card32 res = mrGrid->getRes( level );
AAT POSITION = mrGrid->getPS()->getAAT( "position" );
AAT COLOR = mrGrid->getPS()->getAAT( "color" );
for ( unsigned int i = 0; i < res; i++ ){
for ( unsigned int j = 0; j < res; j++ ){
for ( unsigned int k = 0; k < res; k++ ){
NDTCell cell;
unsigned long index = mrGrid->getIndex( i,j,k,level );
if ( mrGrid->getCell( cell, index, level ) ){
BoundingBox3f bbox;
bbox.lowerCorner = mrGrid->getBBox().lowerCorner + makeVector3f( 
i * mrGrid->getDelta( level ),
j * mrGrid->getDelta( level ),
k * mrGrid->getDelta( level ) );
bbox.upperCorner = mrGrid->getBBox().lowerCorner + makeVector3f( 
(i+1) * mrGrid->getDelta( level ),
(j+1) * mrGrid->getDelta( level ),
(k+1) * mrGrid->getDelta( level ) );
renderNDTCell( cell, bbox, makeVector3f( 0,0,0 ), 1.0f, cell.getColor() );
}
}
}
}
}
*/

/*
void RegistrationDebug::renderGridLevel( MultiResolutionGrid<ColorRegistrationCell>* mrGrid, const card32 level )
{
if ( ( mrGrid == NULL ) || ( level >= mrGrid->getNumLevels() ) ){
return;
}
card32 res = mrGrid->getRes( level );
AAT POSITION = mrGrid->getPS()->getAAT( "position" );
AAT COLOR = mrGrid->getPS()->getAAT( "color" );
for ( unsigned int i = 0; i < res; i++ ){
for ( unsigned int j = 0; j < res; j++ ){
for ( unsigned int k = 0; k < res; k++ ){
ColorRegistrationCell cell;
unsigned long index = mrGrid->getIndex( i,j,k,level );	
if ( mrGrid->getCell( cell, index, level ) ){
BoundingBox3f bbox;
bbox.lowerCorner = mrGrid->getBBox().lowerCorner + makeVector3f( 
i * mrGrid->getDelta( level ),
j * mrGrid->getDelta( level ),
k * mrGrid->getDelta( level ) );
bbox.upperCorner = mrGrid->getBBox().lowerCorner + makeVector3f( 
(i+1) * mrGrid->getDelta( level ),
(j+1) * mrGrid->getDelta( level ),
(k+1) * mrGrid->getDelta( level ) );
rt->disableLighting();
rt->setRenderColor( 1,1,1,1 );
rt->renderBox( bbox );
rt->enableLighting();
for ( unsigned int n = 0; n < cell.samples.size(); n++ ){
card32 pointIndex = cell.samples[n];
Vector3f pos = mrGrid->getPS()->get3f( pointIndex, POSITION );
Vector3f color = mrGrid->getPS()->get3f( pointIndex, COLOR );
rt->setRenderColor( color );
rt->renderSphere( pos, 0.02 * renderScale );
}
}
}
}
}
}
*/