//---------------------------------------------------------------------------
#include "stdafx.h"
#include <windows.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#include "RegistrationRenderer.h"
#include "NDTCell.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

RegistrationRenderer::RegistrationRenderer()
{
	rt = NULL;
}

RegistrationRenderer::RegistrationRenderer( RenderingTool* rt )
{
	this->rt = rt;
}

RegistrationRenderer::~RegistrationRenderer()
{
}

void RegistrationRenderer::renderGrid( 
	   AbstractGrid* grid, 
	   const Vector4f& color, 
	   const float32 renderScale ) const
{
	if ( rt == NULL ) return;

	// Show grid bounding box
	rt->enableBlend();
	rt->setRenderColor( 1, 1, 1, 0.3f );
	BoundingBox3f transformedBBox = grid->getBBox();
	transformedBBox.lowerCorner = transformedBBox.lowerCorner / grid->getNormalizationS() + grid->getNormalizationT();
	transformedBBox.upperCorner = transformedBBox.upperCorner / grid->getNormalizationS() + grid->getNormalizationT();
	rt->renderBox( transformedBBox );

	rt->setRenderColor( color );
	// Compute ndt scale
	float32 ndtScale = 0.0f;
	float32 tau = 0;
	for ( long filledCellIndex = 0; filledCellIndex < grid->getNumFilledCells(); filledCellIndex++ ){
		NDTCell* cell = (NDTCell*) grid->getFilledCell( filledCellIndex );
		if ( cell->getEigenValues()[0] > ndtScale ) ndtScale = cell->getEigenValues()[0];
		tau = cell->getBBox().getMaxSideLength();
	}
	if ( ndtScale > 1e-9 ) ndtScale = 0.5f * tau / ndtScale;
	for ( long filledCellIndex = 0; filledCellIndex < grid->getNumFilledCells(); filledCellIndex++ ){
		NDTCell* cell = (NDTCell*) grid->getFilledCell( filledCellIndex );
		if ( cell != NULL ){
			renderNDTCell( cell, ndtScale * renderScale, color, grid->getNormalizationS(), grid->getNormalizationT() );
		}
	}
	rt->disableBlend();
}

void RegistrationRenderer::renderNDTCell( 
	  AbstractNDTCell* aCell, 
	  const float32 scale,
	  const Vector4f& color,
	  const float32 s,
	  const Vector3f& t ) const
{
	if ( rt == NULL ) return;
	if ( aCell == NULL ) return;
	if ( aCell->getValid() ){
		rt->setRenderColor( color );
	} else {
		rt->setRenderColor( 1, 1, 1, 0.05f );
	}

	// typecast cell
	NDTCell * cell = (NDTCell*) aCell;

	rt->disableLighting();
	BoundingBox3f transformedBBox = cell->getBBox();
	transformedBBox.lowerCorner = transformedBBox.lowerCorner / s + t;
	transformedBBox.upperCorner = transformedBBox.upperCorner / s + t;
	rt->renderBox( transformedBBox );

	if ( cell->getValid() ){
		Vector3f tMean = cell->getMean() / s + t;
		Vector3f tEigenValues = cell->getEigenValues() / s;
		rt->setRenderColor( cell->getColor() );
		rt->enableLighting();
		rt->renderEllipse( tMean, cell->getEigenVectors(), tEigenValues * scale );
	}
}

void RegistrationRenderer::renderOctreeNode( ICPOctreeNode* node ) const
{
	if ( rt == NULL ) return;
	if ( node->isLeaf() ){
		rt->renderBox( node->getBBox() );
	} else {
		for ( unsigned int childIndex = 0; childIndex < 8; childIndex++ ){
			renderOctreeNode( node->getChild( childIndex ) );
		}
	}
}