#ifndef RegularGridH
#define RegularGridH

#include <assert.h>
#include "AbstractGrid.h"
#include "AbstractNDTCell.h"
#include <qtcore/qstring.h>

namespace X4{

class X3FOUNDATION_PRIVATE_API RegularGrid : public AbstractGrid
{
	X4_CLASS( RegularGrid )

	private:
		float tau;						/// Cell spacing
		card32 res;						/// Resolution of cells
		std::vector<AbstractNDTCell*> cells;			/// Container for all cells, only filled cells are saved
		std::map<long,long> cellMapping;		/// Mapping from (i/j/k)-index in grid to cell container (only filled cells)

		/// Cell list descriptor (required in the editor)
		void setNumcellss(card32 size) { cells.resize( size ); }
		void deletecells( card32 index ){x4Assert( index < cells.size()); cells.erase( cells.begin()+index );}
		char* getcells( card32 index ){x4Assert( index < cells.size()); return (char*)&cells[index];}
		mpcard getNumcellss(){return cells.size();}
		std::string getcellsName( card32 index ){ 
			x4Assert( index < cells.size()); 
			return "Cell " + QString::number( index ).toStdString();
		}
		
	public:
		RegularGrid();
		~RegularGrid();

		void clear();
		void setup(
			const BoundingBox3f& bbox, 
			const float32 normalizationS,
			const Vector3f& normalizationT,
			const card32 setupMode, 
			const float32 setupTau, 
			const card32 setupRes );
		Vector3i getIndex3D( const Vector3f& p ) const;
		long getIndex1D( const Vector3i& index3D ) const;
		AbstractNDTCell* getCell( const Vector3f& p );
		AbstractNDTCell* getCell( const long index1D );
		AbstractNDTCell* getCell( const Vector3i& index3D );
		void setCell( const long index1D, AbstractNDTCell* cell );
		void printStats();
		long getNumFilledCells() const { return (long)cells.size(); }
		AbstractNDTCell* getFilledCell( const long filledCellIndex ) { return cells[filledCellIndex]; }
		BoundingBox3f computeCellBBox( const long index1D );
		BoundingBox3f computeCellBBox( const Vector3i& index3D );
		Vector3i getIndex3D( const long index1D );

		/*** MEMBER ACCESS ***/
		inline float32 getTau() const { return tau; }
		inline float32 getRes( const card32 i ) const { return res; }
		inline void setRes( const card32 gridRes ){ this->res = gridRes; }
		inline card32 getRes() const { return res; }
};

}

#endif