#include "stdafx.h"
//---------------------------------------------------------------------------
#include "Resampling.h"
#include "DebugOutput.h"
#include "PointSetKNNQuery.h"
#include "PointSet.inline.h"
#include <time.h>
#include "NumericalClassProperty.h"
#include "CachedRangeQuerry.h"
#include "MathTools.h"
#include "TangentsEstimator.h"
#include "TopologyRangeSearch.h"
#include "SamplingSpacingEstimator.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( Resampling, 0 )
{
	BEGIN_CLASS_INIT( Resampling )
}

bool Resampling::downsample( 
   PointSet* ps,
   InCorePCTopologyGraph* topology,
   const float32 samplingSpacing )
{
	if ( ps == NULL ) return false;
	if ( topology == NULL ) return false;
	AAT POSITION = ps->getAAT( "position" );
	AAT FLAGS = ps->getAAT( "flags" );
	AttachedIndexedOctree* octree = PointSetKNNQuery::createOctree( ps, 32 );
	PointSetKNNQuery knn( ps, octree, makeVector3f( 0,0,0 ), POSITION );
	for ( unsigned int pIndex = 0; pIndex < ps->getNumEntries(); pIndex++ ) ps->set1i( pIndex, FLAGS, 0 );
	card32 nIndex;
	bool found;

	// flags == 0: unvisited
	// flags == 1: keep
	// flags == 2: discard
	card32 newNumPoints = 0;
	for ( unsigned int pIndex = 0; pIndex < ps->getNumEntries(); pIndex++ ){
		if ( ps->get1i( pIndex, FLAGS ) == 0 ){
			newNumPoints++;
			const Vector3f p = ps->get3f( pIndex, POSITION );
			ps->set1i( pIndex, FLAGS, 1 );
			knn.setTargetPoint( p );
			float32 sqrD = 0;
			Vector3f np = p;
			nIndex = pIndex;
			found = true;
			while( found && ( sqrD < sqr( samplingSpacing ) ) ){
				if ( ps->get1i( nIndex, FLAGS ) == 0 ){
					ps->set1i( nIndex, FLAGS, 2 );
				}
				knn.getNextNearestPointIndex( nIndex, found );
				np = ps->get3f( nIndex, POSITION );
				sqrD = ( np - p ) * ( np - p );
			}
		}
	} 

	// Write back resampled point set
	card32 newPIndex = 0;
	std::vector<char> vbuffer( ps->getDescr()->getSize() );
	for ( unsigned int pIndex = 0; pIndex < ps->getNumEntries(); pIndex++ ){
		if ( ps->get1i( pIndex, FLAGS ) == 1 ){
			ps->getVertex( pIndex, &vbuffer[0] );
			ps->setVertex( newPIndex, &vbuffer[0] );
			newPIndex++;
		}
	}
	ps->changeHeight( newNumPoints );

	// Update topology (create new)
	topology->clearAndSetup( topology->getDescr() );
	delete octree;
	octree = PointSetKNNQuery::createOctree( ps, 32 );
	knn = PointSetKNNQuery( ps, octree, makeVector3f( 0,0,0 ), POSITION );
	for ( unsigned int pIndex = 0; pIndex < ps->getNumEntries(); pIndex++ ){
		const Vector3f p = ps->get3f( pIndex, POSITION );
		knn.setTargetPoint( p );
		found = true;
		nIndex = pIndex;
		Vector3f np = ps->get3f( nIndex, POSITION );
		float32 sqrD = 0;
		while( found && ( sqrD < sqr( samplingSpacing * 3 ) ) ){
			if ( nIndex != pIndex ){
				topology->addEdge( pIndex, nIndex );
			}
			knn.getNextNearestPointIndex( nIndex, found );
			if ( found ){
				np = ps->get3f( nIndex, POSITION );
				sqrD = ( np - p ) * ( np - p );
			}
		}
	}

	delete octree;
	return true;
}

bool Resampling::upsample( 
	 PointSet* ps, 
	 InCorePCTopologyGraph* topology, 
	 float32 oldSamplingSpacing )
{
	if ( ps == NULL ) return false;
	if ( topology == NULL ) return false;
	AAT POSITION = ps->getAAT( "position" );
	AAT NORMAL = ps->getAAT( "normal" );
	AAT FLAGS = ps->getAAT( "flags" );

	card32 oldNumPoints = ps->getNumEntries();

	ps->changeHeight( oldNumPoints * 4 );

	/*
	InCorePCTopologyGraph* backuptopology = dynamic_cast<InCorePCTopologyGraph*>( topology->copy() );
	topology->
	*/

	float32 alpha = 120.0f / 180.0f * float32( M_PI );
	Matrix2f rot60 = makeMatrix2f(
		cos( alpha ), sin( alpha ),
		-sin( alpha ), cos( alpha ) );

	for ( unsigned int pIndex = 0; pIndex < oldNumPoints; pIndex++ ){

		const Vector3f p = ps->get3f( pIndex, POSITION );
		const Vector3f normal = ps->get3f( pIndex, NORMAL );
		Vector3f tangent_u, tangent_v;
		TangentsEstimator::computeTangents( normal, tangent_u, tangent_v );
		Matrix3f T;
		T[0] = tangent_u;
		T[1] = tangent_v;
		T[2] = normal;
		// 0
		float32 randAlpha = 0; //rand()%2000/1000.0f*M_PI;
		Matrix2f rotRand = makeMatrix2f(
			cos( randAlpha ), sin( randAlpha ),
			-sin( randAlpha ), cos( randAlpha ) );
		Vector2f v = rotRand * makeVector2f( 1, 0 );
		Vector3f x = p + T * makeVector3f( v[0], v[1], 0 ) * oldSamplingSpacing * 0.5f;
		ps->setVertex( oldNumPoints + pIndex*3, ps->getDataPointer( pIndex ) );
		ps->set3f( oldNumPoints + pIndex*3, POSITION, x );
		// 1
		v = rot60 * v;
		x = p + T * makeVector3f( v[0], v[1], 0 ) * oldSamplingSpacing * 0.5f;
		ps->setVertex( oldNumPoints + pIndex*3+1, ps->getDataPointer( pIndex ) );
		ps->set3f( oldNumPoints + pIndex*3+1, POSITION, x );
		// 2
		v = rot60 * v;
		x = p + T * makeVector3f( v[0], v[1], 0 ) * oldSamplingSpacing * 0.5f;
		ps->setVertex( oldNumPoints + pIndex*3+2, ps->getDataPointer( pIndex ) );
		ps->set3f( oldNumPoints + pIndex*3+2, POSITION, x );
		topology->addEdge( pIndex, oldNumPoints + pIndex*3 );
		topology->addEdge( pIndex, oldNumPoints + pIndex*3+1 );
		topology->addEdge( pIndex, oldNumPoints + pIndex*3+2 );
	}

	// Resampling topology
	const float32 newSamplingSpacing = oldSamplingSpacing / 2.0f;
	const float32 sqrInfluence = sqr( newSamplingSpacing * 3.0f );
	TopNeighborhoodSphereSearch topoSphereSearch;
	for ( unsigned int pIndex = 0; pIndex < ps->getNumEntries(); pIndex++ ){
		const Vector3f p = ps->get3f( pIndex, POSITION );
		topoSphereSearch.setup( 
			topology, 
			ps, 
			pIndex, 
			p, 
			oldSamplingSpacing * 3.0f, 
			POSITION );
		while( !topoSphereSearch.atEnd() ){
			const card32 nIndex = topoSphereSearch.getCurrentIndex();
			const Vector3f np = ps->get3f( nIndex, POSITION );
			if ( ( np - p ) * ( np - p ) < sqrInfluence ){
				topology->addEdge( pIndex, nIndex );
			}
			topoSphereSearch.next();
		}
		ps->set1i( pIndex, FLAGS, 0 );
	}
	// Prune long connections
	for ( unsigned int pIndex = 0; pIndex < ps->getNumEntries(); pIndex++ ){
		const Vector3f p = ps->get3f( pIndex, POSITION );
		for ( int32 n = 0; n < int32( topology->getNumAdjacentVertices( pIndex ) ); n++ ){
			const card32 nIndex = topology->getAdjacentVertex( pIndex, n );
			const Vector3f np = ps->get3f( nIndex, POSITION );
			const float32 sqrD = ( np - p ) * ( np - p );
			if ( sqrD > sqr( newSamplingSpacing * 3 ) ){
				topology->deleteEdge( pIndex, n );
				n--;
			}
		}
	}

	return true;
}

bool Resampling::upsample2( 
  PointSet* ps, 
  InCorePCTopologyGraph* topology, 
  const float32 oldSamplingSpacing )
{
	if ( ps == NULL ) return false;
	if ( topology == NULL ) return false;

	card32 numPoints = ps->getNumEntries();
	AAT POSITION = ps->getAAT( "position" );
	AAT NORMAL = ps->getAAT( "normal" );
	AAT FLAGS = ps->getAAT( "flags" );
	for ( unsigned int pIndex = 0; pIndex < numPoints; pIndex++ ){
		const Vector3f p = ps->get3f( pIndex, POSITION );
		const Vector3f normal = ps->get3f( pIndex, NORMAL );
		card32 numNeighbors = topology->getNumAdjacentVertices( pIndex );
		for ( unsigned int n = 0; n < numNeighbors; n++ ){
			const card32 nIndex = topology->getAdjacentVertex( pIndex, n );
			if ( nIndex > pIndex ){
				const Vector3f np = ps->get3f( nIndex, POSITION );
				const Vector3f nNormal = ps->get3f( nIndex, NORMAL );
				Vector3f x = ( p + np ) / 2.0f;
				Vector3f xNormal = normal + nNormal;
				xNormal.normalize();
				ps->changeHeight( ps->getNumEntries() + 1 );
				ps->setVertex( ps->getNumEntries() - 1, ps->getDataPointer( pIndex ) );	// Copy all vertex attributes
				ps->set3f( ps->getNumEntries() - 1, POSITION, x );
				ps->set3f( ps->getNumEntries() - 1, NORMAL, xNormal );
				topology->addEdge( pIndex, ps->getNumEntries() - 1 );		
			}
		}
	}

	// Resampling topology
	const float32 sqrInfluence = sqr( oldSamplingSpacing / 2.0f * 3.0f );
	TopNeighborhoodSphereSearch topoSphereSearch;
	for ( unsigned int pIndex = 0; pIndex < ps->getNumEntries(); pIndex++ ){
		const Vector3f p = ps->get3f( pIndex, POSITION );
		topoSphereSearch.setup( 
			topology, 
			ps, 
			pIndex, 
			p, 
			oldSamplingSpacing * 3.0f, 
			POSITION );
		while( !topoSphereSearch.atEnd() ){
			const card32 nIndex = topoSphereSearch.getCurrentIndex();
			const Vector3f np = ps->get3f( nIndex, POSITION );
			if ( ( np - p ) * ( np - p ) < sqrInfluence ){
				topology->addEdge( pIndex, nIndex );
			}
			topoSphereSearch.next();
		}
		ps->set1i( pIndex, FLAGS, 0 );
	}
	// Prune long connections
	for ( unsigned int pIndex = 0; pIndex < ps->getNumEntries(); pIndex++ ){
		const Vector3f p = ps->get3f( pIndex, POSITION );
		for ( int32 n = 0; n < int32( topology->getNumAdjacentVertices( pIndex ) ); n++ ){
			const card32 nIndex = topology->getAdjacentVertex( pIndex, n );
			const Vector3f np = ps->get3f( nIndex, POSITION );
			const float32 sqrD = ( np - p ) * ( np - p );
			if ( sqrD > sqr( oldSamplingSpacing / 2.0f * 2.0f ) ){
				topology->deleteEdge( pIndex, n );
				n--;
			}
		}
	}

	return true;
}