//---------------------------------------------------------------------------
#ifndef RigidBodyMotion3dH
#define RigidBodyMotion3dH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace X4 {


template <class FloatType>
class RigidBodyMotion3{
 public:
   // ---- data fields
   StaticMatrix<FloatType, 3, 3> rotation;
   StaticVector<FloatType, 3>	 translation;

   // ---- methods
   RigidBodyMotion3() {}
   RigidBodyMotion3(StaticMatrix<FloatType, 3, 3> rotation, StaticVector<FloatType, 3> translation);

   inline StaticMatrix<FloatType, 3, 3> getRotation() const;
   inline StaticVector<FloatType, 3> getTranslation() const;
	inline void	setRotation(StaticMatrix<FloatType, 3, 3> &rot);
	inline void	setTranslation(StaticVector<FloatType, 3> &trans);

   inline RigidBodyMotion3<FloatType> operator*(const RigidBodyMotion3<FloatType> &left) const;

   inline void	MultiplyOnLeft(StaticVector<FloatType, 3> C, StaticVector<FloatType, 3> BarC);
   inline void  MultiplyOnLeft(StaticVector<FloatType, 3> C, StaticVector<FloatType, 3> BarC,
	   StaticVector<FloatType, 3> center);
};

typedef RigidBodyMotion3<float32> RigidBodyMotion3f;
typedef RigidBodyMotion3<float64> RigidBodyMotion3d;

template <class FloatType, unsigned dim>
inline bool operator==(const RigidBodyMotion3<FloatType> &motion1, const RigidBodyMotion3<FloatType> &motion2);

                                   
}

#include "RigidBodyMotion3d.inline.h"

#endif
