//---------------------------------------------------------------------------
#ifndef RigidBodyMotion3d_templCodeH
#define RigidBodyMotion3d_templCodeH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "BoundingBox.h"
#include "LinearAlgebra.inline.h"
#include "Sphere.inline.h"
#include "math_include.h"
//---------------------------------------------------------------------------

namespace X4 {


template <class FloatType>
RigidBodyMotion3<FloatType>::RigidBodyMotion3(StaticMatrix<FloatType, 3, 3> rotation, StaticVector<FloatType, 3> translation) {
   this->rotation = rotation;
   this->translation = translation;
}

template <class FloatType>
inline StaticMatrix<FloatType, 3, 3> RigidBodyMotion3<FloatType>::getRotation() const {
	return rotation;
}

template <class FloatType>
inline StaticVector<FloatType, 3> RigidBodyMotion3<FloatType>::getTranslation() const {
   return translation;
}

template <class FloatType>
inline void RigidBodyMotion3<FloatType>::setRotation(StaticMatrix<FloatType, 3, 3>& rot)
{
	rotation = rot;
}

template <class FloatType>
inline void RigidBodyMotion3<FloatType>::setTranslation( StaticVector<FloatType, 3>& trans)
{
	translation = trans;
}

template <class FloatType>
inline RigidBodyMotion3<FloatType> RigidBodyMotion3<FloatType>::operator*(const RigidBodyMotion3<FloatType> &left) const 
{
   RigidBodyMotion3<FloatType> result = *this;
   result.rotation	  = left.rotation*rotation;
   result.translation = left.rotation*translation + translation;
   return result;
}

template <class FloatType>
inline void RigidBodyMotion3<FloatType>::MultiplyOnLeft(StaticVector<FloatType, 3> C, StaticVector<FloatType, 3> BarC)
{
	FloatType	norm = sqrt(C.getSqrNorm()); 
	C /= norm;
	FloatType	cosa = cos(norm), sina= sin(norm);
	FloatType	coef1, coef2;
	if(norm > 1e-8)
	{
		coef1 = sina/norm;
		FloatType temp = sin(norm/2)/norm;
		coef2 = 2*temp*temp;
	}
	else
	{
		coef1 = 1.0f;
		coef2 = 0.5f;
	}
	StaticMatrix<FloatType, 3,3>	rot;
	rot[0][0] = cosa		+ (1-cosa)*C[0]*C[0];
	rot[1][0] = -sina*C[2]	+ (1-cosa)*C[0]*C[1];
	rot[2][0] = sina*C[1]	+ (1-cosa)*C[0]*C[2];

	rot[0][1] = sina*C[2]	+ (1-cosa)*C[1]*C[0];
	rot[1][1] = cosa		+ (1-cosa)*C[1]*C[1];
	rot[2][1] = -sina*C[0]	+ (1-cosa)*C[1]*C[2];
	
	rot[0][2] = -sina*C[1]	+ (1-cosa)*C[2]*C[0];
	rot[1][2] = sina*C[0]	+ (1-cosa)*C[2]*C[1];
	rot[2][2] = cosa		+ (1-cosa)*C[2]*C[2];
	
//	StaticVector<FloatType, 3> trans= BarC*coef1 + C*(1-coef1)*(C*BarC) + C.crossProduct(BarC)*coef2;
	StaticVector<FloatType, 3> trans= BarC;
	

	StaticMatrix<FloatType, 3,3>	rot_buf;
	StaticVector<FloatType, 3>		trans_buf;
	for(int32 i = 0; i < 3; i++)
	{
		trans_buf[i] = rot[0][i]*translation[0]+rot[1][i]*translation[1]+rot[2][i]*translation[2]+trans[i]; 
		for(int j = 0; j < 3; j++)
		{
			rot_buf[j][i] = rot[0][i]*rotation[j][0]+rot[1][i]*rotation[j][1]+rot[2][i]*rotation[j][2];
		}
	}
	rot_buf[0].normalize();
	rot_buf[1] -= rot_buf[0]*(rot_buf[0]*rot_buf[1]);
	rot_buf[1].normalize();
	rot_buf[2] -= rot_buf[1]*(rot_buf[1]*rot_buf[2])+ rot_buf[0]*(rot_buf[0]*rot_buf[2]);
	rot_buf[2].normalize();

	for(int32 i = 0; i < 3; i++)
	{
		translation[i] = trans_buf[i]; 
		for(int j = 0; j < 3; j++)
		{
			rotation[j][i] = rot_buf[j][i];
		}
	}

//	translation = rot*translation + trans;
//	rotation		= rot*rotation;
}

template <class FloatType>
inline void RigidBodyMotion3<FloatType>::MultiplyOnLeft(StaticVector<FloatType, 3> C, StaticVector<FloatType, 3> BarC,
														StaticVector<FloatType, 3> center)
{
	FloatType	norm = sqrt(C.getSqrNorm()); C.normalize();
	FloatType	cosa = cos(norm), sina= sin(norm);
	FloatType	coef1, coef2;
	if(norm > 1e-8)
	{
		coef1 = sina/norm;
		FloatType temp = sin(norm/2)/norm;
		coef2 = 2*temp*temp;
	}
	else
	{
		coef1 = 1.0f;
		coef2 = 0.5f;
	}
	StaticMatrix<FloatType, 3,3>	rot;
	rot[0][0] = cosa		+ (1-cosa)*C[0]*C[0];
	rot[1][0] = -sina*C[2]	+ (1-cosa)*C[0]*C[1];
	rot[2][0] = sina*C[1]	+ (1-cosa)*C[0]*C[2];

	rot[0][1] = sina*C[2]	+ (1-cosa)*C[1]*C[0];
	rot[1][1] = cosa		+ (1-cosa)*C[1]*C[1];
	rot[2][1] = -sina*C[0]	+ (1-cosa)*C[1]*C[2];

	rot[0][2] = -sina*C[1]	+ (1-cosa)*C[2]*C[0];
	rot[1][2] = sina*C[0]	+ (1-cosa)*C[2]*C[1];
	rot[2][2] = cosa		+ (1-cosa)*C[2]*C[2];

	//	StaticVector<FloatType, 3> trans= BarC*coef1 + C*(1-coef1)*(C*BarC) + C.crossProduct(BarC)*coef2;
	StaticVector<FloatType, 3> trans= BarC + center - rot*center;

	translation = rot*translation + trans;
	rotation		= rot*rotation;
}

template <class FloatType>
inline bool operator==(const RigidBodyMotion3<FloatType> &motion1, const RigidBodyMotion3<FloatType> &motion2) {
   return motion1.rotation == motion2.rotation  &&  motion1.translation == motion2.translation;
}


}

#endif
