//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "RotationVector.h"
#include "DebugOutput.h"
#include "MathTools.h"
#include "Quaternion.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

RotationVector::RotationVector()
{
	v[0] = 1;
	v[1] = 0;
	v[2] = 0;	
}

RotationVector::RotationVector( float32 v0, float32 v1, float32 v2 )
{
	this->v[0] = v0;
	this->v[1] = v1;
	this->v[2] = v2;
}

RotationVector RotationVector::fromRotMat(Matrix3f R)
{		
	/// convert to Quaternion and then to Rotation Vector
	Quaternion q = Quaternion::fromRotMat(R);
	float32 a = (float32) (2.0*acos((double) q[0])/sqrt(1.0-q[0]*q[0]));
	RotationVector v;
	v[0] = a * q[1];
	v[1] = a * q[2];
	v[2] = a * q[3];

	return v;
}

Matrix3f RotationVector::getRotMat() const
{
	/// convert to Quaternion and use Quaternion's conversion method
	float32 norm_v = float32(sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]));
	float32 s = float32(sin(norm_v/2.0))/norm_v;
	Quaternion q( cos(norm_v/2.0), v[0]*s, v[1]*s, v[2]*s );
	return q.getRotMat();
}

/// Print to console
void RotationVector::println() const
{
	debugOutput << "(" << v[0] << " " << v[1] << " " << v[2] << ")\n"; 
	debugOutput.flush();
}