//---------------------------------------------------------------------------
#include "stdafx.h"
#include <windows.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#include <ClassMethods.h>
#include <StringHelper.h>

#include <SampledLandmarkDetector.h>
#include <PCCResampler.h>
#include <GeometricTools.h>
#include <PointCloudTools.h>
#include <GeometricFeature.h>

//---------------------------------------------------------------------------
using namespace X4;

//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS (SampledLandmarkDetector, 0)
{
    BEGIN_CLASS_INIT (SampledLandmarkDetector);
    ADD_FLOAT32_PROP (m_MedianPointDistance, 0);
	SET_EDIT_READONLY(true);
}

//---------------------------------------------------------------------------
SampledLandmarkDetector::SampledLandmarkDetector()
{
    setDescriptorRadius(5.0f);
    m_MedianPointDistance = 0;
}

//---------------------------------------------------------------------------
SampledLandmarkDetector::~SampledLandmarkDetector()
{

}


//---------------------------------------------------------------------------
void SampledLandmarkDetector::beginDetection(const std::vector<UnstructuredInCorePointCloud*>& clouds)
{
    if (clouds.size() == 0) return;

    m_MedianPointDistance = 0;
    for (unsigned i=0; i < clouds.size(); i++)
    {
        m_MedianPointDistance += getMedianPointDistance(clouds[i]);
    }
    m_MedianPointDistance /= clouds.size();

    //if (getDescriptorRadius() < 0)
    //    setDescriptorRadius(m_MedianPointDistance * 4.0f);

    if (getNumDescriptors() == 0 || getDescriptor(0) == NULL)
    {
        addDescriptor(new SampledLandmarkDescriptor());
        dynamic_cast<LocalDescriptor*>(getDescriptor(0))->setDifferenceScaleFactor(1.0f);
    }
}

//---------------------------------------------------------------------------
FeatureSet * SampledLandmarkDetector::detect( UnstructuredInCorePointCloud * upc, std::string attachmentName)
{
    if (upc == NULL) return NULL;

    // add feature set as attachment, remove the old one if such exists
    if(attachmentName == "")
        attachmentName = FeatureSet::getDefaultName();
    FeatureSet * fs = dynamic_cast<FeatureSet*>(upc->getAttachments()->getData(attachmentName));
    delete fs;
    fs = new FeatureSet;
    fs->setup( attachmentName, AttachedData::ADF_PERSISTENT );
    upc->getAttachments()->attachData(fs);

    // create temporary copy of the point cloud and perform resampling on it
    UnstructuredInCorePointCloud* tempUpc = copyToUnstructuredInCorePointCloud(upc);
    PCCResampler cmd;
    cmd.resamplePC(tempUpc, getDescriptorRadius() * m_MedianPointDistance);
    PointSet* ps = tempUpc->getPointSet();

    LocalDescriptor* descr = dynamic_cast<LocalDescriptor*>(getDescriptor(0));
    if (descr == NULL) exception( __FUNCTION__ " - there must be a descriptor specified!");

    descr->beginSignatureComputations(upc);

    for (unsigned i = 0; i < ps->getNumEntries(); i++)
    {
        GeometricFeature* feature = new GeometricFeature();
        feature->m_Position = ps->get3f(i, ps->getAAT("position"));
        feature->m_Normal = ps->get3f(i, ps->getAAT("normal"));

        feature->addSignature(descr->computeSignature(feature->m_Position, feature->m_Normal, getDescriptorRadius(), upc));

        fs->addFeature(feature);
    }

    descr->endSignatureComputations();

    delete tempUpc;

    return fs;
}

//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS (SampledLandmarkDescriptor, 0)
{
    BEGIN_CLASS_INIT (SampledLandmarkDescriptor);
}

//---------------------------------------------------------------------------
SampledLandmarkDescriptor::SampledLandmarkDescriptor()
{

}

//---------------------------------------------------------------------------
SampledLandmarkDescriptor::~SampledLandmarkDescriptor()
{

}

//---------------------------------------------------------------------------
LocalSignature * SampledLandmarkDescriptor::computeSignature(const Vector3f &point, const Vector3f &normal, float size, UnstructuredInCorePointCloud * upc)
{
    if (upc == NULL) return NULL;

    FloatVectorSignature* signature = new FloatVectorSignature();
    DVectorF data;
    data.setDim(1);
    data.setZero();
    signature->setVector(data);

    return signature;
}

