#ifndef UniformlySampledLandmarks_H
#define UniformlySampledLandmarks_H
//--------------------------------------------------------------------------
#include "x3foundation_private.h"
//--------------------------------------------------------------------------
#include <LocalDescriptor.h>
#include <FeaturePointDetector.h>

//--------------------------------------------------------------------------
namespace X4
{
    class SampledLandmarkDescriptor;
    class SampledLandmarkDetector;

    /**
    * Simple class to detect feature points. Detected points are not
    * real features, just a uniformly distributed points. So don't think points are distributed
    * over highly featured parts. No, this is just a workaround to convert resampled
    * point clouds to feature points. Sometimes we need this.
    * Uses descriptor radius to set the minimum distance between two feature points (default is 4*median point distance)
    **/
    class X3FOUNDATION_PRIVATE_API SampledLandmarkDetector : public FeaturePointDetector
    {
        X4_CLASS (SampledLandmarkDetector);
    private:

        /// virtual version of "begin"
        virtual void beginDetection(const std::vector<UnstructuredInCorePointCloud*>& clouds);

    public:
        SampledLandmarkDetector();
        virtual ~SampledLandmarkDetector();

        FeatureSet * detect( UnstructuredInCorePointCloud * upc, std::string attachmentName="" );

        virtual void end() {}
    };

    /**
    * Descriptor for the sampled landmarks. It gives useless signatures back (FloatVecSignature)
    **/
    class X3FOUNDATION_PRIVATE_API SampledLandmarkDescriptor : public LocalDescriptor
    {
        X4_CLASS( SampledLandmarkDescriptor );
    private:

    public:
        /// construct from scratch
        SampledLandmarkDescriptor();

        /// destruct 
        virtual ~SampledLandmarkDescriptor();

        /// setup descriptor
        virtual void beginSignatureComputations(UnstructuredInCorePointCloud * upc) {}

        /// compute local signature for point
        virtual LocalSignature * computeSignature(const Vector3f &point, const Vector3f &normal, float size, UnstructuredInCorePointCloud * upc);

        /// cleanup
        virtual void endSignatureComputations() {}

        /// returns true if signatures can be recomputed (e.g. false for all detector-based signatures)
        bool isRecomputeable() const {return true;}
    };

}
#endif