//---------------------------------------------------------------------------
#ifndef SparseGridH
#define SparseGridH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "HashTables.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "X4Assert.h"
#include "BoundingBox.h"
//---------------------------------------------------------------------------

namespace X4 {

class OutputObjectStream;
class InputObjectStream;


template <int Dimension, typename ObjectType, class BaseClass>
class SparseGrid : public BaseClass {
 private:
   typedef StaticVector<int32, Dimension> KeyType;

   struct KeyComp {
      static bool equals(KeyType pos1, KeyType pos2) { return pos1 == pos2; }
   };

   class SparseGridHashTable : public HashTable<KeyType, ObjectType, KeyComp> {
    protected:
      virtual max_platform_card hashFunction(KeyType key) const {
		  mpcard ret = 0;
		  for( int i=0;i<Dimension;i++ )
			  ret = ret * 1031 + key[Dimension-1-i]+521;
		  return ret;
      };
    public:
   };

   SparseGridHashTable grid;

 public:
   void clear();
   inline bool getObject(StaticVector<int32, Dimension> index, ObjectType &obj) const;
   inline void setObject(StaticVector<int32, Dimension> index, ObjectType obj);
   inline void deleteObject(StaticVector<int32, Dimension> index);

   template <typename FloatType>
   inline static StaticVector<int32, Dimension> calcIndex(StaticVector<FloatType, Dimension> pos,
                                                        StaticVector<FloatType, Dimension> origin,
														FloatType gridSize);
   template <typename FloatType>
   BoundingBox<FloatType,Dimension> calcBoundingBox(	StaticVector<int32,Dimension> &gridPos,
														StaticVector<FloatType, Dimension> origin,
														FloatType gridSize) const;
   void operator=(const SparseGrid<Dimension, ObjectType, BaseClass> &other);
   void write(OutputObjectStream *s) const;
   void read(InputObjectStream *s);

   inline max_platform_card getNumElements()           const {return grid.count();}
   inline ObjectType getEntry(max_platform_card index) const {return grid.getData(index);}
   inline ObjectType& getEntryRef(max_platform_card index) {return grid.getDataRef(index);}  // commented by CL @ 2014/02/17
   inline KeyType getKey(max_platform_card index)      const {return grid.getKey(index);}
};


}

#endif
