//---------------------------------------------------------------------------
#ifndef SparseGrid_templCodeH
#define SparseGrid_templCodeH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "SparseGrid.h"
//---------------------------------------------------------------------------
#include "HashTables.inline.h"
#include "linearalgebra.inline.h"
#include "X4Assert.h"
#include "ObjectStreams.h"
#include "BoundingBox.inline.h"
//---------------------------------------------------------------------------

namespace X4 {


template <int Dimension, typename ObjectType, class BaseClass>
void SparseGrid<Dimension, ObjectType, BaseClass>
//-------
::clear()
//-------
{
   grid = SparseGridHashTable();
}

template <int Dimension, typename ObjectType, class BaseClass>
inline bool SparseGrid<Dimension, ObjectType, BaseClass>
//--------------------------------------------------------------
::getObject(StaticVector<int32, Dimension> index, ObjectType &obj) const
//--------------------------------------------------------------
{
   return grid.getData(index, obj);
}

template <int Dimension, typename ObjectType, class BaseClass>
inline void SparseGrid<Dimension, ObjectType, BaseClass>
//-------------------------------------------------------------
::setObject(StaticVector<int32, Dimension> index, ObjectType obj)
//-------------------------------------------------------------
{
   grid.addElement(index, obj);
}

template <int Dimension, typename ObjectType, class BaseClass>
inline void SparseGrid<Dimension, ObjectType, BaseClass>
//------------------------------------------------
::deleteObject(StaticVector<int32, Dimension> index)
//------------------------------------------------
{
   grid.deleteElement(index);
}



template <int Dimension, typename ObjectType, class BaseClass>
template <typename FloatType>
inline StaticVector<int32, Dimension> SparseGrid<Dimension, ObjectType, BaseClass>
//------------------------------------------------------------------------------------------------------------
::calcIndex(StaticVector<FloatType, Dimension> pos, StaticVector<FloatType, Dimension> origin, FloatType gridSize)
//------------------------------------------------------------------------------------------------------------
{
   pos -= origin;
   pos *= (FloatType)1.0/gridSize;
   StaticVector<int32, Dimension> result;
   for (int i=0; i<Dimension; i++) {
      result[i] = (int32)pos[i];
   }
   return result;
}

template <int Dimension, typename ObjectType, class BaseClass>
template <typename FloatType>
BoundingBox<FloatType,Dimension> SparseGrid<Dimension, ObjectType, BaseClass>
//------------------------------------------------------------------------------------------------------------
::calcBoundingBox(		StaticVector<int32,Dimension> &gridPos,
						StaticVector<FloatType, Dimension> origin,
						FloatType gridSize) const
//------------------------------------------------------------------------------------------------------------
{
	BoundingBox<FloatType,Dimension> bb;
	bb.lowerCorner = origin;
	for( unsigned i=0;i<Dimension;i++ )
	{
		bb.lowerCorner[i] += (FloatType)gridPos[i]*gridSize;
		bb.upperCorner[i] = bb.lowerCorner[i] + gridSize;			
	}
	return bb;
}


template <int Dimension, typename ObjectType, class BaseClass>
inline void SparseGrid<Dimension, ObjectType, BaseClass>
//--------------------------------------------------------------------
::operator=(const SparseGrid<Dimension, ObjectType, BaseClass> &other)
//--------------------------------------------------------------------
{
   grid = other.grid;
}


template <int Dimension, typename ObjectType, class BaseClass>
inline void SparseGrid<Dimension, ObjectType, BaseClass>
//----------------------------------
::write(OutputObjectStream *s) const
//----------------------------------
{
/*   SparseGridHashTable::EntryPtr pos = grid.getFirstEntry();
   SparseGridHashTable::EntryPtr end = grid.getEndEntry();
   s->write(grid.count());
   while (pos != end) {
      s->write(pos.getKey());
      s->write(pos.getData());
      pos.next();
   }*/
   card32 n = (card32)grid.getNumElements();
   s->write(n);
   for (card32 i=0; i<n; i++) {
      s->write(grid.getKey(i));
      s->write(grid.getData(i));
   }
}


template <int Dimension, typename ObjectType, class BaseClass>
inline void SparseGrid<Dimension, ObjectType, BaseClass>
//--------------------------
::read(InputObjectStream *s)
//--------------------------
{
   card32 numObjs;
   s->read(numObjs);
   for (card32 i=0; i<numObjs; i++) {
      KeyType key;
      ObjectType obj;
      s->read(key);
      s->read(obj);
      grid.addElement(key,obj);
   }

   x4Assert(grid.count() == numObjs);
}





}

#endif
