#ifndef TransformationH
#define TransformationH

// Project
#include "Persistent.h"
#include "Quaternion.h"
#include "RotationVector.h"

namespace X4{

#define TRANSFORMATION_EULER_ANGLES 0
#define TRANSFORMATION_QUATERNIONS 1
#define TRANSFORMATION_ROTATION_VECTOR 2
#define TRANSFORMATION_ROTATION_VECTOR_EXACT 3

/// Use the Normal Distributions Transform for registration of two point clouds
class X3FOUNDATION_PRIVATE_API Transformation : public Persistent
{
	X4_CLASS( Transformation ) 

	private:
		card32 mode;

		/*** INTERNAL ***/
		StaticVector<float32,7> state;	// internal rotation representation
		Matrix3f eulerAnglesR() const;

	public:
		Transformation();
		~Transformation();

		static void jacobianEulerAngles( StaticMatrix<float32,7,3>& jacobian, const Vector3f& pos );
		static void jacobianQuaternion( StaticMatrix<float32,7,3>& jacobian, const Vector3f& pos, const Quaternion& q );
		static void jacobianRotationVector( StaticMatrix<float32,7,3>& jacobian, const Vector3f& pos, bool approx, const Vector3f = NULL_VECTOR3F );
		void jacobian( StaticMatrix<float32,7,3>& jacobian, const Vector3f& pos ) const;
		//void hessian( Vector3f &H, const card32 i, const card32 j, const Vector3f &pos);
		void hessianQuaternion( Vector3f &H, const card32 i, const card32 j, const Vector3f &pos);
		void hessianRotationVector( Vector3f &H, const card32 i, const card32 j, const Vector3f &pos);
		void add( const StaticVector<float32,7> step );
		void reset();
		void cleanup();		// Put state in valid state if required
		void println();

		Matrix4f getTransformation() const;

		/*** MEMBER ACCESS ***/
		inline void setMode( const card32 mode ){ this->mode = mode; reset(); }
		inline card32 getMode() const { return mode; }
		inline StaticVector<float32,7> getState() const { return state; }
		inline void setState( const StaticVector<float32,7>& state ){ this->state = state; }
		void setTranslation( Vector3f translation );
		void setQuaternionRotation( const Quaternion& quaternion ); 
		Quaternion getQuaternion() const;
		Vector3f getTranslation() const { return makeVector3f( state[0], state[1], state[2] ); }
		Matrix3f getRotation() const;

		inline void hessian( Vector3f &H, const card32 i, const card32 j, const Vector3f &pos)
{
	if (i<3 || j<3)
	{
		// translation is linear
		H[0] = 0.0f; H[1] = 0.0f; H[2] = 0.0f;
		return;
	}

	if ( mode == TRANSFORMATION_EULER_ANGLES )
	{
		// not implemented
	} 
	else if ( mode == TRANSFORMATION_QUATERNIONS )
	{		
		hessianQuaternion( H, i, j, pos);
	}
	else if ( mode == TRANSFORMATION_ROTATION_VECTOR_EXACT)
	{		
		hessianRotationVector( H, i, j, pos);
	}
	else if ( mode == TRANSFORMATION_ROTATION_VECTOR)
	{
		// not implemented
	}
}
};



}

#endif