#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "TransformationGrid.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( TransformationGrid ,0)
{
	BEGIN_CLASS_INIT( TransformationGrid );
}

TransformationGrid::TransformationGrid(void)
{
}

TransformationGrid::~TransformationGrid(void)
{
}

void TransformationGrid::assign(const Object* obj, X4CopyContext *context)
{
	const TransformationGrid * o = dynamic_cast<const TransformationGrid*>(obj);
	x4Assert( o != NULL );

	m_AngleTolerance = o->m_AngleTolerance;
	m_InvAngleTolerance = o->m_InvAngleTolerance;
	m_SpatialTolerance = o->m_SpatialTolerance;
	m_InvSpatialTolerance = o->m_InvSpatialTolerance;
	m_Transformations = o->m_Transformations;
}


bool X4::TransformationGrid::transformationExists( const Matrix4f &T )
{
	Vec7i descr = computeMatrixDescription(T);
	Matrix4f tmpM;
	return m_Transformations.getElement(descr,tmpM);
}

void X4::TransformationGrid::addTransformation( const Matrix4f &T )
{
	Vec7i descr = computeMatrixDescription(T);
	m_Transformations.addElement(descr,T);	
}

void X4::TransformationGrid::setup( float angleTolerance, float spatialTolerance )
{
	m_AngleTolerance = angleTolerance;
	m_SpatialTolerance = spatialTolerance;
	m_InvAngleTolerance = 1.0f / angleTolerance;
	m_InvSpatialTolerance = 1.0f / spatialTolerance;
}

X4::TransformationGrid::Vec7i X4::TransformationGrid::computeMatrixDescription( const Matrix4f &T ) const
{
	Vec7i ret;
	ret[0] = T[3][0] * m_InvSpatialTolerance;
	ret[1] = T[3][1] * m_InvSpatialTolerance;
	ret[2] = T[3][2] * m_InvSpatialTolerance;

	// remove scale factor (assume uniform scaling)
	Matrix3f mat = shrink4To3(T);
	float scale = norm(mat[0]);
	mat /= scale;

	// http://de.wikipedia.org/wiki/Roll-Pitch-Yaw-Winkel
	float beta = atan2( -mat[2][0], sqrt(pow(mat[0][0],2.0f)+pow(mat[1][0],2.0f)) );
	float alpha, gamma;
	if(fabs(beta - M_PI/2.0f) < 1e-10f)
	{
		alpha = 0;
		gamma = atan2(mat[0][1],mat[1][1]);
	}
	else
	{
		if(fabs(beta + M_PI/2.0f) < 1e-10f)
		{
			alpha = 0;
			gamma = -atan2(mat[0][1],mat[1][1]);
		}
		else
		{
			float cos_beta = cos(beta);
			alpha = atan2(mat[1][0] / cos_beta,mat[0][0] / cos_beta);
			gamma = atan2(mat[2][1] / cos_beta,mat[2][2] / cos_beta);
		}
	}
	
	alpha = alpha<0?alpha+2.0f*M_PI:alpha;
	beta = beta<0?beta+2.0f*M_PI:alpha;
	gamma = gamma<0?gamma+2.0f*M_PI:alpha;

	ret[3] = alpha * m_InvAngleTolerance;
	ret[4] = beta * m_InvAngleTolerance;
	ret[5] = gamma * m_InvAngleTolerance;


	ret[6] = scale * 50.0f;
	return ret;
}

X4::max_platform_card X4::TransformationGrid::MyHashTable::hashFunction( Vec7i key ) const
{
	mpcard ret = 0;
	ret += (abs(key[0]))*1019 + (key[0] < 0) ? 1:0;
	ret += (abs(key[1]))*8081 + (key[1] < 0) ? 17:0;
	ret += (abs(key[2]))*24473 + (key[2] < 0) ? 31:0;
	ret += (abs(key[3]))*31039;
	ret += (abs(key[4]))*956119;
	ret += (abs(key[5]))*2499997;
	ret += (abs(key[6]))*7389197;

	return ret;
}

X4::TransformationGrid::MyHashTable::MyHashTable( card32 fillFactor /*= 1*/, card32 startSizeIndex /*= 8*/ )
:HashTable(fillFactor,startSizeIndex)
{

}