#ifndef TransformationGridH
#define TransformationGridH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	stores transformations (hash grid in transformation space)
*/
class X3FOUNDATION_PRIVATE_API TransformationGrid : public Persistent
{
	X4_CLASS( TransformationGrid )

public:
	TransformationGrid(void);
	~TransformationGrid(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   void setup(float angleTolerance, float spatialTolerance);   

   /// add transformation to list and returns transformation id
   void addTransformation(const Matrix4f &T);

   /// returns true if transformation is already present
   bool transformationExists(const Matrix4f &T);

private:
	typedef StaticVector<int,7> Vec7i;

	Vec7i computeMatrixDescription(const Matrix4f &T) const;
	
	class Vec7iComp {
	public:
		static bool equals( Vec7i v1, Vec7i v2) { return v1 == v2; }
	};

	class MyHashTable : public HashTable<Vec7i, Matrix4f, Vec7iComp> {
	protected:
		virtual max_platform_card hashFunction(Vec7i key) const;
	public:
		MyHashTable(card32 fillFactor = 1, card32 startSizeIndex = 8);

	};

	MyHashTable m_Transformations;

	float32		m_InvSpatialTolerance;
	float32		m_InvAngleTolerance;
	float32		m_SpatialTolerance;
	float32		m_AngleTolerance;



};


}

#endif