#ifndef TriangleMeshHashGridH
#define TriangleMeshHashGridH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "SparseGrid.h"

namespace X4
{
class UnstructuredInCoreTriangleMesh;
/**
	computes a hash grid from a triangle mesh
	(each triangle will be rasterized into voxels and stored in a hash grid)
	zero area triangles will be skipped
*/
class X3FOUNDATION_PRIVATE_API TriangleMeshHashGrid
{

public:	
	class GridCell
	{
	public:
		std::vector<unsigned> m_Indices;
	};

private:
	class NullClass {};
	SparseGrid<3, GridCell*, NullClass>	m_Grid;
	BoundingBox3f			m_BoundingBox;
	float32					m_CellSize;

	void clear();

public:
	TriangleMeshHashGrid(void);
	~TriangleMeshHashGrid(void);

	void setup(UnstructuredInCoreTriangleMesh * mesh, float cellSize);

	// returns the corresponding cell or NULL if cell does not exist
	// if not NULL the center of the cell is reported in cellCenter
	GridCell * getCell( const Vector3f &p);

	float32 getCellSize() const {return m_CellSize;}
	BoundingBox3f getCellBoundingBox3f( const Vector3f &p) const;
	Vector3f getNearestCellCenter( const Vector3f &p) const;


	/// draw non-empty grid cells (you need an active debugrenderer renderjob!)
	void drawWithDR(const Vector3f &color) const;

	// ----------------------------
	// sequential interface

	/// get total number of non-empty cells
	mpcard getNumCells() const;	

	/// get cell #i
	GridCell * getCell(mpcard i);

	/// get cell #i's bounding box
	BoundingBox3f getCellBoundingBox3f( mpcard i) const;


};

}

#endif