#include "StdAfx.h"
//---------------------------------------------------------------------------
//#include <windows.h>
//#include <mmsystem.h>
#include "TronRenderer.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "GLMultipleRenderTexture.h"
#include "GLHelper.h"
#include "GLShaderMaterial.h"
#include "Configuration.h"
#include "Random.h"
#include "ObjectClassProperty.h"
#include "GLRenderTextureFBO.h"
#include "StandardSceneGraphIterator.h"
#include "Scene.h"
#include "SceneGraphNode.h"
#include "SGObjectNode.h"
#include "SGTransformationNode.h"
#include "SceneObject.h"
#include "MaterialAttachment.h"
#include "GLTexture.h"
#include "ClassMethods.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "TriangleBBIntersection.h"
#include "PointSet.h"
#include "PointCloudTools.h"
#include "ECPhotonTracer.h"
#include "VertexArray.h"
#include "DebugRenderer.h"
#include "MaterialParameter.h"
#include "MPContainer.h"
#include "RGBAImage.h"
#include "Timer.h"
#include "GLBuffer.h"
#include "GLAttachments.h"
#include "SeparatorClassProperty.h"
#include "qtGUI/QInputDialog"
#include "PCCResampler.h"
#include "RaytracingMath.h"
#include "LCHierarchyRenderer.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS(TronRenderer, 1)
{
	BEGIN_CLASS_INIT(TronRenderer);
	ADD_BOOLEAN_PROP(postProcessing, 0)
   ADD_VECTOR3F_PROP(groundPlaneNormal, 0)
   ADD_VECTOR3F_PROP(groundPlaneBase, 0)
   ADD_FLOAT32_PROP(fresnelExp, 0)
   ADD_VECTOR4F_PROP(planeColor, 0)
   ADD_FLOAT32_PROP(reflectivity, 0)
   ADD_FLOAT32_PROP(blurPerUnit, 0)
	ADD_OBJECT_PROP(subRenderer, 1, Renderer::getClass(), true)
}

TronRenderer::TronRenderer()
{
   postProcessing = false;
   groundPlaneBase = makeVector3f(0,0,0);
   groundPlaneNormal = makeVector3f(0,1,0);
   fresnelExp = 1;
   planeColor = makeVector4f(0.3f,0.3f,0.3f,1.0f);
   reflectivity = 0.5f;
   blurPerUnit = 3.0f;
   subRenderer = new LCHierarchyRenderer();
}

TronRenderer::~TronRenderer()
{
	delete subRenderer;
}

void TronRenderer::assign(const Object* obj, X4CopyContext *context)
{
	Renderer::assign(obj,context);
	const TronRenderer * o = dynamic_cast<const TronRenderer*>(obj);
	x4Assert( o != NULL );
   if (o != NULL)	copyObjectProperties(obj,this);
}

void TronRenderer::connect(Scene *scene, GLContext* ctxt) 
{
	Renderer::connect(scene,ctxt);
	subRenderer->connect(scene, ctxt);
}

void TronRenderer::readGLBuffers(Vector4ub *pixels, Vector3f *pos, int width, int height)
{
   std::vector<float> depth(width*height);
   glReadPixels(0, 0, width, height, GL_DEPTH_COMPONENT, GL_FLOAT, &depth[0]);
   glReadPixels(0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, &pixels[0]);

   // read necessary matrices
   double mv[16], proj[16];
   int vp[4];
   glGetDoublev( GL_MODELVIEW_MATRIX, mv);
   glGetDoublev( GL_PROJECTION_MATRIX, proj );
   glGetIntegerv( GL_VIEWPORT, vp );

   //for (int y=0; y<height/2; y++) {
   //   for (int x=0; x<width; x++) {
   //      swap(pixels[x+width*y], pixels[x+width*(height-y-1)]);
   //      swap(depth[x+width*y], depth[x+width*(height-y-1)]);
   //   }
   //}

   Vector3f cam = getScene()->getRootState()->staticState->camera.getPosition();
   Vector3f dir = normalize(getScene()->getRootState()->staticState->camera.getLookAt() - cam);
   for (int y=0; y<height; y++) {
      for (int x=0; x<width; x++) {
         double tmpPos[3];
         float z = depth[y*width+x];
         gluUnProject(x, height-y-1, z, mv, proj, vp, &tmpPos[0], &tmpPos[1], &tmpPos[2] );
         *pos = makeVector3f( tmpPos[0],tmpPos[1],tmpPos[2] );
         pos++;
      }
   }
}


void TronRenderer::smoothGaussian(RGBAImage *img, Vector4ub bg, Vector3f *points, int width) {
   static const int MAX_KERNEL = 16;
   Vector4ub *pixels = img->getPixels();
	// setup 1d gaussian
	std::vector< std::vector<float> > kernels;
	for (int kernelSize=1; kernelSize<=MAX_KERNEL; kernelSize++) {
	   std::vector<float> kernel(2*kernelSize+1);
	   float32 sigma = kernelSize/2.0f;
	   float summedWeight = 0;
	   for( int x=-kernelSize;x<=kernelSize;x++ )
	   {
		   kernel[x+kernelSize] = exp(-(float)x*x / sigma / sigma);
		   summedWeight += kernel[x+kernelSize];
	   }

	   for( unsigned j=0;j<kernel.size();j++ )
		   kernel[j] /= summedWeight;
	   kernels.push_back(kernel);
	}

	// filter image horizontally
	const int iSizeX = img->getSizeX();
	const int iSizeY = img->getSizeY();
	Vector4ub * tmp = new Vector4ub[iSizeX*iSizeY];
	for( int x=0;x<iSizeX;x++ )
	{
		for( int y=0;y<iSizeY;y++ )
		{
		   Vector3f point = points[x+y*width];
		   int kernelSize;
		   if (normQuad(convertVector<float32, card8>(img->getPixel(x,y)) - convertVector<float32, card8>(bg)) <= 5) {
		      kernelSize = 3;
		   } else {
		      kernelSize = fabs((point-groundPlaneBase)*groundPlaneNormal)*blurPerUnit;
		      if (kernelSize >= MAX_KERNEL) kernelSize = MAX_KERNEL-1;
		      if (kernelSize < 0) kernelSize = 0;
		   }
		   int minI = -kernelSize;
		   if( x+minI < 0 )
			   minI = -x;
		   int maxI = kernelSize;
		   if( x+maxI >= iSizeX )
			   maxI = iSizeX-x-1;
			Vector4f p = NULL_VECTOR4F;
			Vector4ub *ptr = &pixels[y*iSizeX+x+minI];
			float *kernelPtr = &kernels[kernelSize][minI + kernelSize];
			for( int i=minI;i<=maxI;i++ )
			{
			   Vector4f val = makeVector4f((*ptr)[0],(*ptr)[1],(*ptr)[2],(*ptr)[3]);
				p += val * (*kernelPtr);
				ptr++;
				kernelPtr++;
			}
			tmp[y*iSizeX+x] = makeVector4ub(p[0],p[1],p[2],p[3]);
		}
	}

	// filter image vertically
	for( int y=0;y<iSizeY;y++ )
	{
		for( int x=0;x<iSizeX;x++ )
		{
		   Vector3f point = points[x+y*width];
		   int kernelSize;
		   if (normQuad(convertVector<float32, card8>(img->getPixel(x,y)) - convertVector<float32, card8>(bg)) <= 5) {
		      kernelSize = 3;
		   } else {
		      kernelSize = fabs((point-groundPlaneBase)*groundPlaneNormal)*blurPerUnit;
		      if (kernelSize >= MAX_KERNEL) kernelSize = MAX_KERNEL-1;
		      if (kernelSize < 1) kernelSize = 1;
		   }
		   int minI = -kernelSize;
		   if( y+minI < 0 )
			   minI = -y;
		   int maxI = kernelSize;
		   if( y+maxI >= iSizeY )
			   maxI = iSizeY-y-1;
			Vector4f p = NULL_VECTOR4F;
			Vector4ub * ptr = &tmp[(y+minI)*iSizeX+x];
			float * kernelPtr = &kernels[kernelSize][minI + kernelSize];
			for( int i=minI;i<=maxI;i++ )
			{
			   Vector4f val = makeVector4f((*ptr)[0],(*ptr)[1],(*ptr)[2],(*ptr)[3]);
				p += val * (*kernelPtr);
				kernelPtr++;
				ptr += iSizeX;
			}
			img->setPixel(x,y,makeVector4ub(p[0],p[1],p[2],p[3]));
		}
	}

	delete tmp;

}



void TronRenderer::render(card32 viewportWidth, card32 viewportHeight) 
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
   if (norm(groundPlaneNormal) <= 1e-5) {
      debugOutput << "TronRenderer::render - normal is zero.\n";
   }
   Vector3f normalNormed = normalize(groundPlaneNormal);
   float32 planeDist = normalNormed*groundPlaneBase;
	Matrix3f refl = IDENTITY3F - outerProduct(normalNormed,normalNormed)*2;
	Matrix4f refl4 = expand3To4(refl); refl4[3][3] = 1;
	Matrix4f mirroring = makeTranslation4f(-groundPlaneBase)*refl4*makeTranslation4f(-groundPlaneBase);
	
	Vector4f bg = getScene()->getRootState()->staticState->backgroundColor;
	glClearColor(bg[0], bg[1], bg[2], bg[3]);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);

   //Camera &sceneCam = getScene()->getRootState()->staticState->camera;
   //Camera camSave = sceneCam;
   //Vector3f position = sceneCam.getPosition();
   //Vector3f lookAt = sceneCam.getLookAt();
   //Vector3f up = sceneCam.getUp();
   //StandardSceneGraphIterator sceneIt;

	glPushAttrib(GL_ALL_ATTRIB_BITS);
   {
	   glMatrixMode(GL_MODELVIEW);
	   glPushMatrix();
	   glMultMatrixf(refl4.data());

      double planeEq[4] = {-normalNormed[0], -normalNormed[1], -normalNormed[2], planeDist};
      glEnable(GL_CLIP_PLANE0);
      glClipPlane(0, planeEq);

	   //sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, 0, 0);
	   //renderSceneRecursive(&sceneIt, IDENTITY4F, NULL);
	   subRenderer->render(viewportWidth, viewportHeight);
	   glPopMatrix();
   }
	glPopAttrib();
	glFlush();
	
	if (postProcessing) {
	   RGBAImage img(viewportWidth, viewportHeight);
	   std::vector<Vector3f> pos(viewportWidth*viewportHeight);
	   readGLBuffers(img.getPixels(), &(pos[0]), viewportWidth, viewportHeight);
	   Vector4f bgf = getScene()->getRootState()->staticState->backgroundColor;
	   Vector4ub bg = makeVector4ub(bgf[0]*255, bgf[1]*255, bgf[2]*255, bgf[3]*255);
      smoothGaussian(&img, bg, &(pos[0]), viewportWidth);
      blendPlane(img, bgf);
	   glDisable(GL_DEPTH_TEST);
      glDrawPixels(viewportWidth, viewportHeight, GL_RGBA, GL_UNSIGNED_BYTE, img.getPixels());
	   glFlush();
	   glEnable(GL_DEPTH_TEST);
	}

	glPushAttrib(GL_ALL_ATTRIB_BITS);
   {
      double planeEq[4] = {normalNormed[0], normalNormed[1], normalNormed[2], planeDist};
      glEnable(GL_CLIP_PLANE0);
      glClipPlane(0, planeEq);

	   //sceneIt.clearAndSetup(getScene()->getRootNode(), getScene()->getRootState(), SGDynamicState::tmRendering, 0, 0);
	   //renderSceneRecursive(&sceneIt, IDENTITY4F, NULL);
		subRenderer->render(viewportWidth, viewportHeight);
   }
	glPopAttrib();

	glPopAttrib();
}

void TronRenderer::disconnect(GLContext* ctxt) 
{
	Renderer::disconnect(ctxt);
}


void TronRenderer::renderSceneRecursive( StandardSceneGraphIterator * sceneIter, const Matrix4f &modelMatrix, GLShaderMaterial * mat )
{
	if (!sceneIter->getNode()->visible()) return;

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glMultMatrixf(sceneIter->getState()->dynamicState->ctm.data());
	Matrix4f currentModelMatrix = modelMatrix;
	SGTransformationNode * tnode = dynamic_cast<SGTransformationNode*>(sceneIter->getNode());
	if( tnode )
	{
		currentModelMatrix = currentModelMatrix * tnode->getTransformation();
	}

	SGObjectNode *objNode = dynamic_cast<SGObjectNode*>(sceneIter->getNode());
	if (objNode != NULL) {
		SceneObject* obj = objNode->getSceneObject();

		if (obj != NULL && objNode->visible()) {

			SceneGraphState *state = sceneIter->getState();
			SceneObjectIterator *it = obj->createIterator(SOIT(SOIT::CAP_HIERARCHICAL | SOIT::CAP_ATTACH_SECONDARY));

			if( mat != NULL ) {
				mat->draw( getScene(), state, obj, it, getGLContext());
			} else {
			   GLMaterial *ownMat = getScene()->getGLMaterial(obj->getMaterialIndex());
			   if (ownMat)	ownMat->draw( getScene(), state, obj, it, getGLContext());
			}
			
			delete it;
		}
	}

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	unsigned numChildren = sceneIter->getNumChildren();
	for( unsigned i=0;i<numChildren;i++ )
	{
		sceneIter->goToChild(i);
		renderSceneRecursive(sceneIter,currentModelMatrix,mat);
		sceneIter->goToParent();
	}
}

void TronRenderer::setupCam(card32 viewportWidth, card32 viewportHeight)
{
   if (viewportHeight <= 0) return;
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(getScene()->getRootState()->staticState->viewFrustum.getVerticalFieldOfView(),
                  (float)viewportWidth / (float)viewportHeight,
                  getScene()->getRootState()->staticState->viewFrustum.getZNearClip(),
                  getScene()->getRootState()->staticState->viewFrustum.getZFarClip());
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   gluLookAt(getScene()->getRootState()->staticState->camera.getPosition()[0],
             getScene()->getRootState()->staticState->camera.getPosition()[1],
             getScene()->getRootState()->staticState->camera.getPosition()[2],

             getScene()->getRootState()->staticState->camera.getLookAt()[0],
             getScene()->getRootState()->staticState->camera.getLookAt()[1],
             getScene()->getRootState()->staticState->camera.getLookAt()[2],

             getScene()->getRootState()->staticState->camera.getUp()[0],
             getScene()->getRootState()->staticState->camera.getUp()[1],
             getScene()->getRootState()->staticState->camera.getUp()[2]);

}

void X4::TronRenderer::blendPlane( RGBAImage &img, Vector4f backgroundColor )
{
   float32 ngp = norm(groundPlaneNormal);
   if (ngp <1e-3) return;
   Vector3f gpn = normalize(groundPlaneNormal);
   Vector4ub bg = makeVector4ub(backgroundColor[0]*255, backgroundColor[1]*255, backgroundColor[2]*255, backgroundColor[3]*255);
   int w = img.getSizeX();
   int h = img.getSizeY();

   // projection matrices
   double mv[16], proj[16];
   int vp[4];
   glGetDoublev( GL_MODELVIEW_MATRIX, mv);
   glGetDoublev( GL_PROJECTION_MATRIX, proj );
   glGetIntegerv( GL_VIEWPORT, vp );

   Vector3f cam = getScene()->getRootState()->staticState->camera.getPosition();
   Vector3f dir = normalize(getScene()->getRootState()->staticState->camera.getLookAt() - cam);
   for (int y=0; y<h; y++) {
      for (int x=0; x<w; x++) {
         double tmpPos[3];
         gluUnProject(x, y, 0, mv, proj, vp, &tmpPos[0], &tmpPos[1], &tmpPos[2] );
         Vector3f pos0 = makeVector3f( tmpPos[0],tmpPos[1],tmpPos[2] );
         gluUnProject(x, y, 1, mv, proj, vp, &tmpPos[0], &tmpPos[1], &tmpPos[2] );
         Vector3f pos1 = makeVector3f( tmpPos[0],tmpPos[1],tmpPos[2] );
         Vector3f rayDirection = normalize(pos1-pos0);
         float32 cosI = -rayDirection*gpn;
         if (cosI <= 0) {
            img.setPixel(x,y,bg);
         } else {
            float32 blend = pow(fabs(1-cosI), fresnelExp)*reflectivity;
            Vector4ub colub = img.getPixel(x,y);
            Vector4f colf = makeVector4f(colub[0],colub[1],colub[2],colub[3]);
            Vector4f c = colf*blend + (planeColor*255)*(1-blend);
            img.setPixel(x,y,makeVector4ub(c[0],c[1],c[2],c[3]));
         }
      }
   }
}
