//---------------------------------------------------------------------------
#ifndef TronRendererH
#define TronRendererH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Renderer.h"
//---------------------------------------------------------------------------


namespace X4 {

class GLMultipleRenderTexture;
class GLRenderTextureFBO;
class GLShaderMaterial;
class StandardSceneGraphIterator;
class RandomTextureAttachment;
class ECResampleOctree;
class UnstructuredInCorePointCloud;
class GLRenderTextureCube;
class MPMatrix4x4;
class MPVector4;
class SharedVertexBufferAttachment;

/**
	renders stuff w/ blurry mirroring plane
*/
class X3FOUNDATION_PRIVATE_API TronRenderer : public Renderer {
	X4_CLASS( TronRenderer )
 private:
   bool postProcessing;
   Vector3f groundPlaneNormal;
   Vector3f groundPlaneBase;
   float32  fresnelExp;
   Vector4f planeColor;
   float32 reflectivity;
   float32 blurPerUnit;
   Renderer *subRenderer;

	void renderSceneRecursive( StandardSceneGraphIterator * sceneIter, const Matrix4f &modelMatrix, GLShaderMaterial * mat );
   void readGLBuffers(Vector4ub *pixels, Vector3f *pos, int width, int height);
   void setupCam(card32 viewportWidth, card32 viewportHeight);
   void blendPlane(RGBAImage &img, Vector4f backgroundColor);
   void smoothGaussian(RGBAImage *img, Vector4ub bg, Vector3f *points, int width);

 public:
	TronRenderer();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void connect(Scene *scene, GLContext* ctxt);
   virtual void render(card32 viewportWidth, card32 viewportHeight);
   virtual void disconnect(GLContext* ctxt);

	~TronRenderer();
};


}

#endif