/***************************************************************************
*   Copyright (c) 2010   Art Tevs                                         *
*                                                                         *
*   This library is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation; either version 3 of        *
*   the License, or (at your option) any later version.                   *
*                                                                         *
*   This library is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU Lesse General Public License for more details.                    *
*                                                                         *
*   The full license is in LICENSE file included with this distribution.  *
***************************************************************************/

//---------------------------------------------------------------------------
#ifndef UNSTRUCTUREDINCOREPC_MANIFOLD__H
#define UNSTRUCTUREDINCOREPC_MANIFOLD__H

#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "DiscreteManifold.h"
#include "UnstructuredInCorePointCloud.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "HierarchicalKNNIterator.h"

//---------------------------------------------------------------------------
namespace X4
{
	/**
	* Representing a incore point cloud as manifold.
	**/
	class X3FOUNDATION_PRIVATE_API UnstructuredInCorePointCloudManifold : public DiscreteManifold
	{
		X4_CLASS(UnstructuredInCorePointCloudManifold)
	protected:

		HierarchicalKNNIterator* m_hIt;
		InCorePCTopologyGraph* m_Tpg;
		UnstructuredInCorePointCloud* m_PointCloud;
		UnstructuredInCorePointCloud* m_Subsampled;

		virtual float computeDistanceAndFillHash(mpcard i, mpcard j, std::vector<DVectorF>* path);
		virtual std::vector<float> computeDistancesAndFillHash(mpcard i, const std::vector<mpcard>& j, std::vector<std::vector<DVectorF>>* paths);

	public:
		UnstructuredInCorePointCloudManifold();
		virtual ~UnstructuredInCorePointCloudManifold();

		//! Just set, not a copy. Ownership is not changed (a topology graph should be attached to the pc)
		virtual void setPointCloud(UnstructuredInCorePointCloud* pc);
		UnstructuredInCorePointCloud* getPointCloud() { return m_PointCloud; }
		InCorePCTopologyGraph* getTopology() { return m_Tpg; }

		virtual float getMedianPointDistance();

		virtual mpcard getNumPoints();
		virtual mpcard findNearestPoint(const DVectorF& point);
		//virtual DiscreteManifoldPoint getPoint(mpcard index);
		virtual ManifoldPoint* createPoint(const DVectorF& coordinate);
		virtual ManifoldPoint* createPoint(mpcard index, const DVectorF& coordinate = DVectorF());
		virtual bool supportSubdivisionLevels() { return false; }

		void subsamplePointCloud(float epsilonFactor);
		UnstructuredInCorePointCloud* getSubsamplePointCloud() { return m_Subsampled; }
	};

	/**
	* Create a discrete manifold from given point cloud. The point cloud is copied into the manifold, so it can be deleted after this call.
	**/
	extern X3FOUNDATION_PRIVATE_API DiscreteManifold* createDiscreteManifold(UnstructuredInCorePointCloud* pc);
	extern X3FOUNDATION_PRIVATE_API DiscreteManifold* createDiscreteManifold(UnstructuredInCoreTriangleMesh* pc);

};

#endif 
