#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "UnstructuredInCoreTriangleMeshManifold.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "MeshGeodesics.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( UnstructuredInCoreTriangleMeshManifold ,0)
{
	BEGIN_CLASS_INIT( UnstructuredInCoreTriangleMeshManifold );
}

UnstructuredInCoreTriangleMeshManifold::UnstructuredInCoreTriangleMeshManifold(void)
{
	m_hIt = NULL;
	mGeo  = NULL;
}

UnstructuredInCoreTriangleMeshManifold::~UnstructuredInCoreTriangleMeshManifold(void)
{
	if (mGeo) delete mGeo;
}

#if 0
void UnstructuredInCoreTriangleMeshManifold::assign(const Object* obj, X4CopyContext *context)
{
	notImplemented(); //todo
	
	//Manifold::assign(obj,context);
	const UnstructuredInCoreTriangleMeshManifold * o = dynamic_cast<const UnstructuredInCoreTriangleMeshManifold*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}
#endif 

float UnstructuredInCoreTriangleMeshManifold::computeDistanceAndFillHash(mpcard i, mpcard j, std::vector<DVectorF>* path)
{
	if (m_PointCloud == NULL) return -1;

	if (m_bDirtyGeodesics || mGeo == NULL)
	{
		if (mGeo) delete mGeo;
		mGeo = new MeshGeodesics;
		mGeo->initializeFromUnstructuredInCoreTriangleMesh(dynamic_cast<UnstructuredInCoreTriangleMesh*>(m_PointCloud));
	}

	mGeo->setSubdivisionLevel(getSubdivisionLevel());

	std::vector<Vector3f> geoPath;
	float dist = mGeo->getGeodesicDistance(i, j, &geoPath);
	if (path) path->resize(geoPath.size());

	// convert path to dvectorf path
	std::vector<DVectorF> myPath;
	myPath.resize(geoPath.size());
	for (unsigned k=0; k < geoPath.size(); k++)
	{
		myPath[k] = makeDVectorL(3, geoPath[k].data());
		if (path) path->at(k) = myPath[k];
	}

	if (getCachePaths() == false) myPath.clear();

	setHashData(i, j, HashData(dist,myPath));
	//setHashData(j, i, HashData(dist,myPath));

	return dist;
}

std::vector<float> UnstructuredInCoreTriangleMeshManifold::computeDistancesAndFillHash(mpcard i, const std::vector<mpcard>& j, std::vector<std::vector<DVectorF>>* paths)
{
	if (m_PointCloud == NULL || j.size() == 0) return std::vector<float>(j.size(), -1);
	
	if (m_bDirtyGeodesics || mGeo == NULL)
	{
		if (mGeo) delete mGeo;
		mGeo = new MeshGeodesics;
		mGeo->initializeFromUnstructuredInCoreTriangleMesh(dynamic_cast<UnstructuredInCoreTriangleMesh*>(m_PointCloud));
	}

	mGeo->setSubdivisionLevel(getSubdivisionLevel());
	std::vector<std::vector<Vector3f> > geoPath;
	std::vector<float> dist = mGeo->getGeodesicDistances(i, j, &geoPath);
	if (paths) paths->resize(geoPath.size());

	for (size_t k=0; k < j.size(); k++)
	{
		// convert path to dvectorf path
		std::vector<DVectorF> myPath(geoPath[k].size());
		for (unsigned n=0; n < geoPath[k].size(); n++)
			myPath[n] = makeDVectorL(3, geoPath[k][n].data());
		if (paths) paths->at(k) = myPath;

		if (getCachePaths() == false) myPath.clear();

		setHashData(i, j[k], HashData(dist[k], myPath));
		//setHashData(j[k], i, HashData(dist[k], myPath));
	}

	return dist;
}


UnstructuredInCoreTriangleMesh* X4::UnstructuredInCoreTriangleMeshManifold::getMesh()
{
	return dynamic_cast<UnstructuredInCoreTriangleMesh*>(m_PointCloud);
}

const UnstructuredInCoreTriangleMesh* X4::UnstructuredInCoreTriangleMeshManifold::getMesh() const
{
	return dynamic_cast<UnstructuredInCoreTriangleMesh*>(m_PointCloud);
}