//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "VSTreeIterators.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

void VSTreeIterator::setup( VSTree* tree )
{
	this->tree = tree;
	reset();
}

void VSTreeIterator::reset()
{
	stack.clear();
	if ( tree == NULL ) return;
	if ( tree->getRoot() != NULL )
		stack.push( tree->getRoot() );
}

bool VSTreeIterator::atEnd()
{
	return stack.empty();
}

VSTreeNode* VSTreeIterator::next()
{
	VSTreeNode* node = stack.pop();
	VSTreeVNode* vNode = dynamic_cast<VSTreeVNode*>( node );
	VSTreeTNode* tNode = dynamic_cast<VSTreeTNode*>( node );
	VSTreeSNode* sNode = dynamic_cast<VSTreeSNode*>( node );
	if ( vNode != NULL ){
		// V-Node
		if ( vNode->hasChildren() ){
			for ( unsigned int cIndex = 0; cIndex < 8; cIndex++ ){
				VSTreeNode* node  = vNode->getChild( cIndex );
				if ( node != NULL ){
					stack.push( node );
				} else {
					debugOutput << "trying to push NULL node; should not happen.\n"; debugOutput.flush();
				}
			}
		}
		return vNode;
	} else if ( tNode != NULL ){
		// T-Node
		VSTreeNode* node = tNode->getChild();
		if ( node != NULL ){
			stack.push( node );
		} else {
			debugOutput << "trying to push NULL node; should not happen.\n"; debugOutput.flush();
		}
		return tNode;
	} else if ( sNode != NULL ){
		// S-Node
		if ( sNode->hasChildren() ){
			for ( unsigned int cIndex = 0; cIndex < 4; cIndex++ ){
				VSTreeNode* node = sNode->getChild( cIndex );
				if ( node != NULL ){
					stack.push( node );
				} else {
					debugOutput << "trying to push NULL node; should not happen.\n"; debugOutput.flush();
				}
			}
		}
		return sNode;
	} else { 
		debugOutput << "returning NULL node; should not happen.\n"; debugOutput.flush();
		return NULL;
	}
}