//---------------------------------------------------------------------------
#ifndef VSTreeOperationsH
#define VSTreeOperationsH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
#include "Persistent.h"
#include "VSTree.h"
#include "VSTreeVNode.h"
#include "VSTreeTNode.h"
#include "VSTreeSNode.h"
//---------------------------------------------------------------------------

namespace X4 {

/** Various operations on a VS-Tree, including build **/
class X3FOUNDATION_PRIVATE_API VSTreeOperations : public Persistent
{
	X4_CLASS( VSTreeOperations )

	private:
		card32 splitSize;		// Build octree split size for 
		card32 maxDepth;		// Max depth of tree structure
		float32 maxTAngle;		// Max angle between normals below T-node

		card32 currentID;

		// Build a node (recursively with children) for a set of indices; requires normals!
		VSTreeNode* buildVSTreeVNode( const card32& depth, std::vector<card32> indices, PointSet* ps, BoundingBox3f& bbox );
		VSTreeNode* buildVSTreeTNode( const card32& depth, std::vector<card32> indices, PointSet* ps );
		VSTreeSNode* buildVSTreeSNode( 
			const card32& depth, 
			std::vector<card32> indices, 
			PointSet* ps,
			VSTreeTNode* parentTNode,
			const BoundingBox2f bbox );

	public:
		VSTreeOperations();
		~VSTreeOperations();

		// Build a VS-Tree for a point set; parameters: splitSize, maxDepth, maxTAngle
		bool buildVSTree( VSTree* tree, PointSet* ps ); 
		// Count number of nodes
		void getNumNodes( VSTree* tree, card32& numNodes, card32& numVNodes, card32& numTNodes, card32& numSNodes );

};
}

#endif