//---------------------------------------------------------------------------
#ifndef VSTreeRendererH
#define VSTreeRendererH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
#include "Persistent.h"
#include "VSTree.h"
#include "VSTreeVNode.h"
#include "VSTreeTNode.h"
#include "VSTreeSNode.h"
#include "RenderingTool.h"
//---------------------------------------------------------------------------

namespace X4 {

#define COLORIZE_CONSTANT 0
#define COLORIZE_NODE_COLOR 1

/** Display functionality of a VS-Tree and its nodes **/
class X3FOUNDATION_PRIVATE_API VSTreeRenderer : public Persistent
{
	X4_CLASS( VSTreeRenderer )

	private:
		RenderingTool rt;
		bool showVNodes;			/// Show all v-nodes
		bool showTNodes;			/// Show all t-nodes
		bool showSNodes;			/// Show all s-nodes
		card32 nodeColorization;	/// Colorization of nodes (constant, node-color)
		Vector4f vNodeColor;		/// Constant color of v-nodes
		Vector4f tNodeColor;		/// Constant color of t-nodes
		Vector4f sNodeColor;		/// Constant color of s-nodes
		bool showNodeDataPoints;	/// Display points contained in node

		void renderNode( VSTreeNode* node, PointSet* ps );
		void renderVNode( VSTreeVNode* vNode, PointSet* ps );
		void renderTNode( VSTreeTNode* tNode, PointSet* ps );
		void renderSNode( VSTreeSNode* sNode, PointSet* ps );

		inline Vector4f getColor( VSTreeVNode* vNode ) const { return ( nodeColorization == COLORIZE_CONSTANT )? vNodeColor : vNode->getColor(); }
		inline Vector4f getColor( VSTreeTNode* tNode ) const { return ( nodeColorization == COLORIZE_CONSTANT )? tNodeColor : tNode->getColor(); }
		inline Vector4f getColor( VSTreeSNode* sNode ) const { return ( nodeColorization == COLORIZE_CONSTANT )? sNodeColor : sNode->getColor(); }

	public:
		VSTreeRenderer();
		~VSTreeRenderer();

		// Render a VS-Tree
		void renderTree( VSTree* tree, PointSet* ps );
		

};
}

#endif