//---------------------------------------------------------------------------
#ifndef VSTreeSNodeH
#define VSTreeSNodeH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
#include "Persistent.h"
#include "VSTreeNode.h"
//---------------------------------------------------------------------------

namespace X4 {

/** Implementation of the s-node (surface) in a VS-Tree **/
class X3FOUNDATION_PRIVATE_API VSTreeSNode : public VSTreeNode
{
	X4_CLASS( VSTreeSNode )

	private: 
		VSTreeSNode* sNodes[4];		// Children
		std::vector<card32> indices;		// Pointer to indices
		BoundingBox2f bbox;			// Bounding box (in local coordinate frame from parent T-Node)
		VSTreeNode* parentTNode;	// Pointer to corresponding parent T-node

		void setNumindicess( const card32 n ){ indices.resize( n ); }
		void deleteindices( const card32 index ){ indices.erase( indices.begin()+index ); }
		char* getindices( const card32 index ){x4Assert( index < indices.size()); return (char*)&(indices[index]);}		
		mpcard getNumindicess() const { return indices.size(); }
		std::string getindicesName( const card32 index ){ return QString::number( indices[index] ).toStdString(); }

	public:
		VSTreeSNode();
		~VSTreeSNode();

		void clear();

		Matrix3f getT() const;
		Vector3f getCenter() const;

		/** MEMBER ACCESS **/
		inline bool hasChildren() const { return sNodes[0] != NULL; }
		inline void setChild( const card32 index, VSTreeSNode* child ){ this->sNodes[index] = child; }
		inline VSTreeSNode* getChild( const card32 index ){ return sNodes[index]; }
		inline void setIndices( std::vector<card32> indices ){ this->indices = indices; }
		inline mpcard getNumIndices() const { return indices.size(); }
		inline card32 getIndex( const card32 index ) const { return indices[index]; }
		inline void setBBox( const BoundingBox2f& bbox ){ this->bbox = bbox; }
		inline BoundingBox2f getBBox() const { return bbox; }
		inline void setParentTNode( VSTreeNode* tNode ){ this->parentTNode = tNode; }
		inline VSTreeNode* getParentTNode(){ return parentTNode; }

	};
}

#endif