//---------------------------------------------------------------------------
#ifndef VSTreeVNodeH
#define VSTreeVNodeH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
#include "Persistent.h"
#include "VSTreeNode.h"
//---------------------------------------------------------------------------

namespace X4 {

/** Implementation of the s-node (volume, equals octree implementation) in a VS-Tree **/
class X3FOUNDATION_PRIVATE_API VSTreeVNode : public VSTreeNode
{
	X4_CLASS( VSTreeVNode )

	private: 
		VSTreeNode* children[8];		// Child nodes, NULL if leaf
		BoundingBox3f bbox;				// Bounding box of octree cell
		std::vector<card32> indices;			// Indices of points in associated point set

		void setNumindicess( const card32 n ){ indices.resize( n ); }
		void deleteindices( const card32 index ){ indices.erase( indices.begin()+index ); }
		char* getindices( const card32 index ){x4Assert( index < indices.size()); return (char*)&(indices[index]);}		
		mpcard getNumindicess() const { return indices.size(); }
		std::string getindicesName( const card32 index ){ return QString::number( indices[index] ).toStdString(); }

	public:
		VSTreeVNode();
		~VSTreeVNode();

		void clear();
		bool hasChildren() const { return ( children[0] != NULL ); }

		inline VSTreeNode* getChild( const card32 index ){ return children[index]; }
		inline void setChild( const card32 childIndex, VSTreeNode* child ){ children[childIndex] = child; }
		inline BoundingBox3f getBBox() const { return bbox; }
		inline void setBBox( BoundingBox3f bbox ){ this->bbox = bbox; }
		inline void setIndices( std::vector<card32> indices ){ this->indices = indices; }
		inline mpcard getNumIndices() const { return indices.size(); }
		inline card32 getIndex( const card32 index ){ return indices[index]; }
	};
}

#endif