//----------------------------------------------------------------------
// File:			kd_pr_search.h
// Programmer:		Sunil Arya and David Mount and Nils Hasler
// Description:		Priority kd-tree search
// Last modified:	12/06/11 (Version 1.2)
//----------------------------------------------------------------------
// Copyright (c) 1997-2011 University of Maryland and Sunil Arya and
// David Mount and Nils Hasler.  All Rights Reserved.
// 
// This software and related documentation is part of the Approximate
// Nearest Neighbor Library (ANN).  This software is provided under
// the provisions of the Lesser GNU Public License (LGPL).  See the
// file ../ReadMe.txt for further information.
// 
// The University of Maryland (U.M.) and the authors make no
// representations about the suitability or fitness of this software for
// any purpose.  It is provided "as is" without express or implied
// warranty.
//----------------------------------------------------------------------
// History:
//	Revision 0.1  03/04/98
//		Initial release
//  Revision 1.2  12/06/11
//		Made ANN thread safe
//----------------------------------------------------------------------

#ifndef ANN_kd_pr_search_H
#define ANN_kd_pr_search_H

#include "kd_tree.h"					// kd-tree declarations
#include "kd_util.h"					// kd-tree utilities
#include "pr_queue.h"					// priority queue declarations
#include "pr_queue_k.h"					// k-element priority queue

#include <ANNperf.h>				// performance evaluation

//----------------------------------------------------------------------
//	Search Data
//		Stores data that is valid for the duration of a query.
//		The structure is instantiated by the top-level search function.
//		This allows several queries to be performed in parallel.
//----------------------------------------------------------------------

struct ANNprData {
	double			ANNprEps;				// the error bound
	int				ANNprDim;				// dimension of space
	ANNpoint		ANNprQ;					// query point
	double			ANNprMaxErr;			// max tolerable squared error
	ANNpointArray	ANNprPts;				// the points
	ANNpr_queue		*ANNprBoxPQ;			// priority queue for boxes
	ANNmin_k		*ANNprPointMK;			// set of k closest points
	int				ANNptsVisited;			// total points visited
};

#endif
