//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "AttachedIndexedOctree.h"
//---------------------------------------------------------------------------
#include "BoundingBox.inline.h"
#include "Persistent.h"
#include "Tools.h"
//#include "EventCounterDisplay.h"
#include "DebugOutput.h"
#include "PointSet.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( AttachedIndexedOctree, 4 ) 
{
	BEGIN_CLASS_INIT( AttachedIndexedOctree );
}

// ----

card64  AttachedIndexedOctree::m_globalAge=0;

AttachedIndexedOctree::AttachedIndexedOctree() {
   m_age = 0;
   m_ps = 0;
}

void AttachedIndexedOctree::assign(const Object* obj, X4CopyContext *context) {
   const AttachedIndexedOctree *other = dynamic_cast<const AttachedIndexedOctree*>(obj);
   if (!other) {
      throw ETypeMismatch("AttachedIndexedOctree::assign() - wrong obj type.");
   }
   m_ps = other->m_ps;
   m_age = other->m_age;
   m_octree.assign(other->m_octree);
}


void AttachedIndexedOctree::octreesetup	(
	const PointSet* ps,			// Point set
	card32 max_points			// Max points per leaf
	)
{
   m_octree.deleteChildren();

   m_octree.getContent().m_bb = ps->getBoundingBox();m_ps=ps;
   m_age=m_globalAge;

   const VertexDescriptor* desc=ps->getDescr();
   const AAT POSITION = desc->getToken("position");

   card32 numProcessed=0;

   for(card32 i=0; i<ps->getNumEntries(); i++){
      AIOctreeNode* node=&m_octree;
      Vector3f pos = ps->get<float, 3>(i, POSITION);

      // find node for corresponding position
      int depth=0;
      //int used_child=0;
      //forget(used_child);
      while(!node->isLeaf()){
         const BoundingBox3f& pbb=node->getContent().m_bb;
         Vector3f center=pbb.getCenter();

         int child=0;
         if(center[0]<pos[0]) child|=1;
         if(center[1]<pos[1]) child|=2;
         if(center[2]<pos[2]) child|=4;
         node=node->getChild(child);
         //used_child=child;

         depth++;
      }

      // too many points in the octree node -> subdivide
      while(node->getContent().m_index.size()+1>=max_points){
         const BoundingBox3f& pbb=node->getContent().m_bb;
         Vector3f center=pbb.getCenter();

         // create children and update bounding boxes
         node->addChildren();
         for(int c=0; c<8; c++){
            BoundingBox3f* bb=&(node->getChild(c)->getContent().m_bb);
            *bb=node->getContent().m_bb;

            if(c & 1) bb->lowerCorner[0]=center[0];
            else      bb->upperCorner[0]=center[0];

            if(c & 2) bb->lowerCorner[1]=center[1];
            else      bb->upperCorner[1]=center[1];

            if(c & 4) bb->lowerCorner[2]=center[2];
            else      bb->upperCorner[2]=center[2];
         }

         // copy indizes to corresponding subnodes
         const size_t s=node->getContent().m_index.size();
         if (s == 0) continue;

         bool ptsDifferent = false;
         card32 firstIndex = node->getContent().m_index[0];
         Vector3f firstPoint = ps->get<float, 3>(firstIndex, POSITION);

         for(card32 j=0; j<s; j++) {
            card32 index=node->getContent().m_index[j];
            Vector3f p = ps->get<float, 3>(index, POSITION);
            if (firstPoint != p) ptsDifferent = true;
            int child=0;
            if(center[0]<p[0]) child|=1;
            if(center[1]<p[1]) child|=2;
            if(center[2]<p[2]) child|=4;
            node->getChild(child)->getContent().m_index.push_back(index);
         }

         // avoid endless recursion if all points are the same
         if (!ptsDifferent) break;

         // clear index-table
         node->getContent().m_index.clear();

         // find corresponding node (from new children)
         int child=0;
         if(center[0]<pos[0]) child|=1;
         if(center[1]<pos[1]) child|=2;
         if(center[2]<pos[2]) child|=4;
         node=node->getChild(child);
         //used_child=child;
      }

      // add index of point to node

      node->getContent().m_index.push_back(i);
      x4Assert(node->getContent().m_bb.inBoundingBox(pos));

      numProcessed++;
      if (numProcessed % 1000 == 0) {
//            EventCounterDisplay::NewValueStepOnChange e;
//            e.value = numProcessed;
//            debugOutput.displayEvent(EventCounterDisplay::getClass(), std::string("AITree: pts"), &e);
      }


   }

	computeBoundingSpheres();
}

void AttachedIndexedOctree::octreesetupWithBBox(
										const PointSet* ps,			// Point set
										card32 max_points,			// Max points per leaf
										BoundingBox3f bbox
										)
{
	m_octree.deleteChildren();

	m_octree.getContent().m_bb = bbox;			// This is the only difference to the 
	m_ps=ps;
	m_age=m_globalAge;

	const VertexDescriptor* desc=ps->getDescr();
	const AAT POSITION = desc->getToken("position");

	card32 numProcessed=0;

	for(card32 i=0; i<ps->getNumEntries(); i++){
		AIOctreeNode* node=&m_octree;
		Vector3f pos = ps->get<float, 3>(i, POSITION);

		// find node for corresponding position
		int depth=0;
		int used_child=0;
		forget(used_child);
		while(!node->isLeaf()){
			BoundingBox3f pbb=node->getContent().m_bb;
			Vector3f center=pbb.getCenter();

			int child=0;
			if(center[0]<pos[0]) child|=1;
			if(center[1]<pos[1]) child|=2;
			if(center[2]<pos[2]) child|=4;
			node=node->getChild(child);
			used_child=child;

			depth++;
		}

		// too many points in the octree node -> subdivide
		while(node->getContent().m_index.size()+1>=max_points){
			BoundingBox3f pbb=node->getContent().m_bb;
			Vector3f center=pbb.getCenter();

			// create children and update bounding boxes
			node->addChildren();
			for(int c=0; c<8; c++){
				BoundingBox3f* bb=&(node->getChild(c)->getContent().m_bb);
				*bb=node->getContent().m_bb;

				if(c & 1) bb->lowerCorner[0]=center[0];
				else      bb->upperCorner[0]=center[0];

				if(c & 2) bb->lowerCorner[1]=center[1];
				else      bb->upperCorner[1]=center[1];

				if(c & 4) bb->lowerCorner[2]=center[2];
				else      bb->upperCorner[2]=center[2];
			}

			// copy indices to corresponding sub-nodes
			const size_t s=node->getContent().m_index.size();
			if (s == 0) continue;

			bool ptsDifferent = false;
			card32 firstIndex = node->getContent().m_index[0];
			Vector3f firstPoint = ps->get<float, 3>(firstIndex, POSITION);

			for(card32 j=0; j<s; j++) {
				card32 index=node->getContent().m_index[j];
				Vector3f p = ps->get<float, 3>(index, POSITION);
				if (firstPoint != p) ptsDifferent = true;
				int child=0;
				if(center[0]<p[0]) child|=1;
				if(center[1]<p[1]) child|=2;
				if(center[2]<p[2]) child|=4;
				node->getChild(child)->getContent().m_index.push_back(index);
			}

			// avoid endless recursion if all points are the same
			if (!ptsDifferent) break;

			// clear index-table
			node->getContent().m_index.clear();

			// find corresponding node (from new children)
			int child=0;
			if(center[0]<pos[0]) child|=1;
			if(center[1]<pos[1]) child|=2;
			if(center[2]<pos[2]) child|=4;
			node=node->getChild(child);
			used_child=child;
		}

		// add index of point to node

		node->getContent().m_index.push_back(i);
		x4Assert(node->getContent().m_bb.inBoundingBox(pos));

		numProcessed++;
	}

	computeBoundingSpheres();
}

void AttachedIndexedOctree::insertPointIndex( 
	const card32 index,
	const card32 max_points )
{
	const VertexDescriptor* desc = m_ps->getDescr();
	const AAT POSITION = desc->getToken( "position" );

	AIOctreeNode* node=&m_octree;
	Vector3f pos = m_ps->get<float, 3>( index, POSITION );

	// find node for corresponding position
	int depth=0;
	int used_child=0;
	forget(used_child);
	while(!node->isLeaf()){
		BoundingBox3f pbb=node->getContent().m_bb;
		Vector3f center=pbb.getCenter();

		int child=0;
		if(center[0]<pos[0]) child|=1;
		if(center[1]<pos[1]) child|=2;
		if(center[2]<pos[2]) child|=4;
		node=node->getChild(child);
		used_child=child;

		depth++;
	}

	// too many points in the octree node -> subdivide
	while(node->getContent().m_index.size()+1>=max_points){
		BoundingBox3f pbb=node->getContent().m_bb;
		Vector3f center=pbb.getCenter();

		// create children and update bounding boxes
		node->addChildren();
		for(int c=0; c<8; c++){
			BoundingBox3f* bb=&(node->getChild(c)->getContent().m_bb);
			*bb=node->getContent().m_bb;

			if(c & 1) bb->lowerCorner[0]=center[0];
			else      bb->upperCorner[0]=center[0];

			if(c & 2) bb->lowerCorner[1]=center[1];
			else      bb->upperCorner[1]=center[1];

			if(c & 4) bb->lowerCorner[2]=center[2];
			else      bb->upperCorner[2]=center[2];
		}

		// copy indidces to corresponding subnodes
		const size_t s=node->getContent().m_index.size();
		if (s == 0) continue;

		bool ptsDifferent = false;
		card32 firstIndex = node->getContent().m_index[0];
		Vector3f firstPoint = m_ps->get<float, 3>(firstIndex, POSITION);

		for(card32 j=0; j<s; j++) {
			card32 index=node->getContent().m_index[j];
			Vector3f p = m_ps->get<float, 3>(index, POSITION);
			if (firstPoint != p) ptsDifferent = true;
			int child=0;
			if(center[0]<p[0]) child|=1;
			if(center[1]<p[1]) child|=2;
			if(center[2]<p[2]) child|=4;
			node->getChild(child)->getContent().m_index.push_back(index);
		}

		// avoid endless recursion if all points are the same
		if (!ptsDifferent) break;

		// clear index-table
		node->getContent().m_index.clear();

		// find corresponding node (from new children)
		int child=0;
		if(center[0]<pos[0]) child|=1;
		if(center[1]<pos[1]) child|=2;
		if(center[2]<pos[2]) child|=4;
		node=node->getChild(child);
		used_child=child;
	}

	// add index of point to node

	node->getContent().m_index.push_back( index );
	x4Assert(node->getContent().m_bb.inBoundingBox(pos));
}

bool AttachedIndexedOctree::queryDelete(ChangeType whatChanged) {
   if (whatChanged == CT_ATTRIBUTE_CHANGED) {
      return false;
   } else {
      return true;     // xxx todo true !!!!
   }
}


//==============================================================================


KNNQuery::KNNQuery(SceneObjectIterator* pcIt, const Vector3f& sp, const card32 &subTreeSize, MemoryManager *attachedOctreeMemManager, const bool &modifyable){
   seekPoint= sp;
   
   setup(pcIt, subTreeSize, attachedOctreeMemManager,modifyable);
}

KNNQuery::KNNQuery(SceneObjectIterator* pcIt, const card32 &subTreeSize, MemoryManager *attachedOctreeMemManager, const bool &modifyable){
   seekPoint= makeVector3f(0,0,0); //Dummy point

   setup(pcIt, subTreeSize, attachedOctreeMemManager,modifyable);
}

KNNQuery::~KNNQuery(){
   m_basic->unlock();

	// ooc unlock
   m_sds->unlockAttachments();
}

void KNNQuery::setup(SceneObjectIterator* pcIt, const card32 &subTreeSize, MemoryManager *attachedOctreeMemManager, const bool &modifyable){

	maxDistanceToSeekPoint_2 = 1e35f;
   m_octree=NULL;
   pntSet_rw=NULL;
	m_sds=NULL;

   m_basic = dynamic_cast<BasicPointCloudIterator*>(pcIt);

   if (modifyable){
      ModifyablePointCloudIterator* mod= dynamic_cast<ModifyablePointCloudIterator*>(pcIt);
		pntSet_rw= mod->lockBlockReadWrite(ModifyablePointCloudIterator::CHANGE_NOFLAG_ATTRIBS_ONLY);
      pntSet=    pntSet_rw;
   }
   else
      pntSet=m_basic->lockBlockRead();

   POSITION = pntSet->getDescr()->getToken("position");

   //test the Octree
   #ifdef _DEBUG
   HierarchicalIterator* m_hpi=dynamic_cast<HierarchicalIterator*>(pcIt);
   x4Assert(m_hpi->isLeaf());
   #endif

   m_sds = dynamic_cast<SecondaryDataStructureIterator*>(pcIt);
	x4Assert(m_sds);
 
	// ooc lock
	pcIt->lockObject();
   AttachedDataContainer *attachments = m_sds->lockAttachments();

   m_octree = dynamic_cast<AttachedIndexedOctree*>(attachments->getData("SimpleOctree"));
   if(m_octree==NULL){
      m_octree=new AttachedIndexedOctree();
      m_octree->setup("SimpleOctree", AttachedData::ADF_QUERY_DELETE_ON_UPDATE,  attachedOctreeMemManager); // attachedOctreeMemManager !!
      m_octree->octreesetup(pntSet, subTreeSize);
      attachments->attachData(m_octree);
   }
	pcIt->unlockObject();

   rootNode= m_octree->getOctree();
   putBoxOnQueue(rootNode);

   atPoint=0;
   nextPointReady=false;
}


void KNNQuery::setSeekPoint(const Vector3f& sp){
   atPoint=0;
   nextPointReady=false;
   seekPoint= sp;

   std::priority_queue<PointEntry> emptyPoints;
   std::priority_queue<BoxEntry> emptyBoxes;

   //pqBoxes=emptyBoxes;
   //pqPoints=emptyPoints;
	pqBoxes.clear();
	pqPoints.clear();

   putBoxOnQueue(rootNode);
   // ...
}

inline float computeSquaredDistanceToBox( const Vector3f &pos, const BoundingBox3f &bb)
{
	// Code from http://www.gamasutra.com/features/19991018/Gomez_4.htm
	// Author: Miguel Gomez / Gamasutra / October 18, 1999

	float s, d = 0; 

	//find the square of the distance
	//from the sphere to the box
	for( long i=0 ; i<3 ; i++ )
	{

		if( pos[i] < bb.lowerCorner[i] )
		{

			s = pos[i] - bb.lowerCorner[i];
			d += s*s; 

		}

		else if( pos[i] > bb.upperCorner[i] )
		{

			s = pos[i] - bb.upperCorner[i];
			d += s*s; 

		}

	}
	return d;

}

void KNNQuery::putBoxOnQueue(AIOctreeNode* box){

   // calc box-point distance
   float32 qDistance;

   /*const Sphere3f sp=//box->getContent().m_bb.getBoundingSphere();
		box->getContent().m_bsphere;
   qDistance=normQuad(sp.center-seekPoint);
   if(qDistance<sp.radius*sp.radius){
      qDistance=-1.0f;
   }else{
      qDistance=sqrt(qDistance);
      qDistance-=sp.radius;
      qDistance*=qDistance;
   }*/

	qDistance = computeSquaredDistanceToBox(seekPoint, box->getContent().m_bb);
	if( qDistance > maxDistanceToSeekPoint_2 )
		return;

   BoxEntry bEntry(box, qDistance);
   pqBoxes.push(bEntry);
}
void KNNQuery::putPointOnQueue(const card32 &indexOfPoint){

	if (indexOfPoint >= pntSet->getNumEntries()) return; // saftety measure - this should not happen, but it sometimes does in some code pieces (there is still a bug somewhere)
   // calc point-point distance
   Vector3f pos=pntSet->get<float32,3>( indexOfPoint, POSITION);

   float32 qDistance = normQuad(pos-seekPoint);
   if( qDistance > maxDistanceToSeekPoint_2 )
	   return;

   PointEntry pnt(indexOfPoint, qDistance);
   pqPoints.push(pnt);
}

bool KNNQuery::seekNextPoint(){
   getNextPoint();
   return nextPointReady;
}

card32 KNNQuery::getNextPoint(){


   //check for points
   if (pqBoxes.empty() && pqPoints.empty()){   // No more Points in Cloud
      nextPointReady=false;
      return 0; // TODO: send Error code  ?
   }

   while (true){
      if (!pqPoints.empty()){
         if(pqBoxes.empty() || pqPoints.top().quDist <= pqBoxes.top().quDist) //point nearer than any box
         {
			 const PointEntry &pEntry = pqPoints.top();
            card32 i=pEntry.index;
			m_SquaredDistanceToCurrentPoint = pEntry.quDist;
            pqPoints.pop();
            atPoint=i;
            nextPointReady=true;
            return i;
         }
      }

		if( pqBoxes.empty() )
		{
			nextPointReady = false;
			return 0;
		}	

      // go into Box
      AIOctreeNode *thisBox = pqBoxes.top().node;
      pqBoxes.pop();

      if(thisBox->isLeaf()){  //Put Boxes Points on Queue
         size_t size = thisBox->getContent().m_index.size();
         for(card32 i=0; i<size; i++){
            putPointOnQueue(thisBox->getContent().m_index[i]);
         }

      }else{ //Put Box Children on Queue
         for(card32 i=0; i<8; i++){
            putBoxOnQueue(thisBox->getChild(i));
         }
      }

   } //while

}



//==============================================================================







//==============================================================================
/*
MetaClass* AIHierarchicalOctreeIterator::AIHierarchicalOctreeIteratorMetaClass = NULL;

const MetaClass* AIHierarchicalOctreeIterator::getInstanceClass() const {
   return getClass();
}

Object* AIHierarchicalOctreeIterator::newInstance() {
   return new AIHierarchicalOctreeIterator;
}

const MetaClass* AIHierarchicalOctreeIterator::getClass() {
   if (AIHierarchicalOctreeIteratorMetaClass==NULL) {
      AIHierarchicalOctreeIteratorMetaClass = new MetaClass("AIHierarchicalOctreeIterator", "no description.",
                                  newInstance, sizeof(AIHierarchicalOctreeIterator));
   }
   return AIHierarchicalOctreeIteratorMetaClass;
}

void AIHierarchicalOctreeIterator::init(const MetaClass *parentClass, card32 uniqueIdentifier) {
   ((MetaClass*)getClass())->init(uniqueIdentifier, parentClass);
   AIHierarchicalOctreeIteratorMetaClass->copyParentProperties();
}

void AIHierarchicalOctreeIterator::shutdown() {
   delete AIHierarchicalOctreeIteratorMetaClass;
}


// ----

AIHierarchicalOctreeIterator::AIHierarchicalOctreeIterator(){
   m_inOctree=false;
   m_octree=NULL;
   m_node=NULL;
   m_currentIndex=0;
   m_lockedPS=NULL;
   m_locked=false;
}


void AIHierarchicalOctreeIterator::assign(const Object* obj,  X4CopyContext *context){
   const AIHierarchicalOctreeIterator* that=dynamic_cast<const AIHierarchicalOctreeIterator*>(obj);
   x4Assert(that);
   clearAndSetup(NULL, dynamic_cast<PointCloudIterator*>(that->m_pi->copy(context)));
   m_inOctree=that->m_inOctree;
   m_octree=that->m_octree;
   m_node=that->m_node;
   m_currentIndex=that->m_currentIndex;
}

void AIHierarchicalOctreeIterator::checkForOctree(void){
   if(m_hpi->isLeaf()){
      if(m_inOctree==false){
         m_inOctree=true;
			/// not ooc safe! - code deprecated
			AttachedDataContainer *attachments = m_sds->lockAttachments();
         AttachedIndexedOctree* d = dynamic_cast<AttachedIndexedOctree*>(attachments->getData("SimpleOctree"));
         if(d){
            if(d->isOld()){
               attachments->deleteData("SimpleOctree");
               d=NULL;
            }
         }
         if(d==NULL){
            d=new AttachedIndexedOctree();
            d->setup("SimpleOctree", AttachedData::ADF_QUERY_DELETE_ON_UPDATE);
            d->octreesetup(m_bpci->lockBlockRead(), 32);
            m_bpci->unlock();
            attachments->attachData(d);
         }
         m_node=d->getOctree();
         m_octree=d;
         /// not ooc safe! - code deprecated
         m_sds->unlockAttachments();
      }
   }else{
      m_inOctree=false;
      m_node=NULL;
      m_currentIndex=0;
      m_octree=NULL;
   }
}


void AIHierarchicalOctreeIterator::clearAndSetup(PointCloud*, SceneObjectIterator* mpi, MemoryManager *aiMemManage)
{
   m_pi = mpi;
   m_hpi=dynamic_cast<HierarchicalIterator*>(mpi);
   m_sds=dynamic_cast<SecondaryDataStructureIterator*>(mpi);
   m_bpci=dynamic_cast<BasicPointCloudIterator*>(mpi);
   m_aiMemManage = aiMemManage;
   x4Assert(m_hpi!=NULL || m_sds!=NULL || m_bpci!=NULL);
}


AIHierarchicalOctreeIterator::~AIHierarchicalOctreeIterator(){
   delete m_pi;
}

void AIHierarchicalOctreeIterator::reset(){
   if(m_inOctree){
      m_currentIndex=0;
   }else{
      m_bpci->reset();
   }
}

void AIHierarchicalOctreeIterator::next(){
   if(m_inOctree){
      m_currentIndex++;
   }else{
      m_bpci->next();
   }
}

bool AIHierarchicalOctreeIterator::atEnd(){
   if(m_inOctree){
      return(m_currentIndex>=m_node->getContent().m_index.size());
   }
   return(m_bpci->atEnd());
}











/// get attributes in different formats
card8     AIHierarchicalOctreeIterator::get1ub(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<card8, 1>(m_node->getContent().m_index[m_currentIndex], token)[0]);
   }else{
      return(m_bpci->get1ub(token));
   }
}

Vector2ub AIHierarchicalOctreeIterator::get2ub(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<card8, 2>(m_node->getContent().m_index[m_currentIndex], token));
   }else{
      return(m_bpci->get2ub(token));
   }
}

Vector3ub AIHierarchicalOctreeIterator::get3ub(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<card8, 3>(m_node->getContent().m_index[m_currentIndex], token));
   }else{
      return(m_bpci->get3ub(token));
   }
}

Vector4ub AIHierarchicalOctreeIterator::get4ub(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<card8, 4>(m_node->getContent().m_index[m_currentIndex], token));
   }else{
      return(m_bpci->get4ub(token));
   }
}

int32    AIHierarchicalOctreeIterator::get1i(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<int32, 1>(m_node->getContent().m_index[m_currentIndex], token)[0]);
   }else{
      return(m_bpci->get1i(token));
   }
}

Vector2i AIHierarchicalOctreeIterator::get2i(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<int32, 2>(m_node->getContent().m_index[m_currentIndex], token));
   }else{
      return(m_bpci->get2i(token));
   }
}

Vector3i AIHierarchicalOctreeIterator::get3i(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<int32, 3>(m_node->getContent().m_index[m_currentIndex], token));
   }else{
      return(m_bpci->get3i(token));
   }
}

Vector4i AIHierarchicalOctreeIterator::get4i(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<int32, 4>(m_node->getContent().m_index[m_currentIndex], token));
   }else{
      return(m_bpci->get4i(token));
   }
}

float32  AIHierarchicalOctreeIterator::get1f(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<float, 1>(m_node->getContent().m_index[m_currentIndex], token)[0]);
   }else{
      return(m_bpci->get1f(token));
   }
}

Vector2f AIHierarchicalOctreeIterator::get2f(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<float, 2>(m_node->getContent().m_index[m_currentIndex], token));
   }else{
      return(m_bpci->get2f(token));
   }
}

Vector3f AIHierarchicalOctreeIterator::get3f(const AAT &token) const {
   if(m_inOctree){
      int i=m_node->getContent().m_index[m_currentIndex];
//      int j=m_octree->getPointSet()->getNumEntries();
      return(m_octree->getPointSet()->get<float, 3>(i, token));
   }else{
      return(m_bpci->get3f(token));
   }
}

Vector4f AIHierarchicalOctreeIterator::get4f(const AAT &token) const {
   if(m_inOctree){
      return(m_octree->getPointSet()->get<float, 4>(m_node->getContent().m_index[m_currentIndex], token));
   }else{
      return(m_bpci->get4f(token));
   }
}

void AIHierarchicalOctreeIterator::getVertex(char *buffer) const{
   if(m_inOctree){
      m_octree->getPointSet()->getVertex(m_node->getContent().m_index[m_currentIndex], buffer);
   }else{
      m_bpci->getVertex(buffer);
   }
}


bool AIHierarchicalOctreeIterator::nextBlock(){
   bool v=m_bpci->nextBlock();
   m_currentIndex=0;
   checkForOctree();
   return(v);
}

const PointSet * AIHierarchicalOctreeIterator::lockBlockRead(){
   if(m_inOctree){
      m_locked=true;
//      x4Assert(m_lockedPS!=NULL);
      if(!m_node->isLeaf() || m_octree->getPointSet()==NULL){
         m_lockedPS=NULL;
         return(NULL);
      }

      m_lockedPS=new PointSet();
      m_lockedPS->clearAndSetup(1, m_node->getContent().m_index.size(), m_octree->getPointSet()->getDescr());
      char* data=new char[m_octree->getPointSet()->getDescr()->getSize()];
      for(int i=0; i<(int)m_node->getContent().m_index.size(); i++){
         m_octree->getPointSet()->getVertex(m_node->getContent().m_index[i], data);
         m_lockedPS->setVertex(i, data);
      }
      delete data;
      return(m_lockedPS);
   }else{
      m_locked=false;
      return(m_bpci->lockBlockRead());
   }
}

void AIHierarchicalOctreeIterator::unlock(){
   if(m_locked){
      if(m_lockedPS){
         delete m_lockedPS;
      }
      m_lockedPS=NULL;
      m_locked=false;
   }else{
      m_bpci->unlock();
      checkForOctree();
   }
}

card32 AIHierarchicalOctreeIterator::getNumChildren(){
   checkForOctree();
   if(m_inOctree){
      if(m_node->isLeaf())
         return(0);
      else
         return(8);
   }else{
      return(m_hpi->getNumChildren());
   }
}

bool AIHierarchicalOctreeIterator::goToChild(card32 index){
   if(m_inOctree){
      if(m_node->isLeaf())
         return(false);

      x4Assert(index<8);
      m_node=m_node->getChild(index);
      m_currentIndex=0;
      return(true);
   }else{
      bool v=m_hpi->goToChild(index);
      m_currentIndex=0;
      checkForOctree();
      return(v);
   }

}

bool AIHierarchicalOctreeIterator::goToParent(){
   if(m_inOctree){

      x4Assert(false);
      //m_node=m_node->getParent();


      m_currentIndex=0;
      if(m_node!=NULL)
         return(true);
   }
   m_inOctree=false;
   bool v=m_hpi->goToParent();
   checkForOctree();
   return(v);
}

BoundingBox3f AIHierarchicalOctreeIterator::getBoundingBox(){
   if(m_inOctree){
      return(m_node->getContent().m_bb);
   }else{
      return(m_hpi->getBoundingBox());
   }
}

bool AIHierarchicalOctreeIterator::isLeaf() const {
   const_cast<AIHierarchicalOctreeIterator*>(this)->checkForOctree();
   if(m_inOctree){
      return(m_node->isLeaf());
   }else{
      return(m_hpi->isLeaf());
   }
}

bool AIHierarchicalOctreeIterator::getNodeInfo(NodeInfoType whichInfo, const void *parameter, void *data){
   checkForOctree();
   if (whichInfo == niPointSpacing) {
      if(m_inOctree){
         if(!m_node->isLeaf()){
            *((float*)data) = MAX_POINT_SPACING;
         }else
            *((float*)data) = 0;
      }
      return true;
   } else {
      return(m_hpi->getNodeInfo(whichInfo, parameter, data));
   }
}

AttachedDataContainer *AIHierarchicalOctreeIterator::lockAttachments() {
	return m_sds->lockAttachments();
}

void AIHierarchicalOctreeIterator::unlockAttachments() {
	m_sds->unlockAttachments();
}


void AIHierarchicalOctreeIterator::invalidateOctrees(void){
   AttachedIndexedOctree::goOlder();
}


*/

X4::card32 X4::KNNQuery::getNearestPointIndexAndSqrDistance( const Vector3f &targetPosition, float32 &returnedDistance )
{
	setSeekPoint(targetPosition);
	//check for points
	if (pqBoxes.empty() && pqPoints.empty()){   // No more Points in Cloud
		nextPointReady=false;
		return 0; // TODO: send Error code  ?
	}

	while (true){
		if (!pqPoints.empty()){
			if(pqBoxes.empty() || pqPoints.top().quDist <= pqBoxes.top().quDist) //point nearer than any box
			{
				card32 i=pqPoints.top().index;
				returnedDistance=pqPoints.top().quDist;
				pqPoints.pop();
				atPoint=i;
				nextPointReady=true;
				return i;
			}
		}

		if( pqBoxes.empty() )
		{
			nextPointReady = false;
			return 0;
		}	

		// go into Box
		AIOctreeNode *thisBox = pqBoxes.top().node;
		pqBoxes.pop();

		if(thisBox->isLeaf()){  //Put Boxes Points on Queue
			size_t size = thisBox->getContent().m_index.size();
			for(card32 i=0; i<size; i++){
				putPointOnQueue(thisBox->getContent().m_index[i]);
			}

		}else{ //Put Box Children on Queue
			for(card32 i=0; i<8; i++){
				putBoxOnQueue(thisBox->getChild(i));
			}
		}

	} //while

}

void X4::AttachedIndexedOctree::computeBoundingSpheres()
{
	std::vector<AIOctreeNode*> stack;
	stack.push_back(&m_octree);

	while(!stack.empty())
	{
		AIOctreeNode * currentNode = stack.back();
		stack.pop_back();
		
		currentNode->getContent().m_bsphere = currentNode->getContent().m_bb.getBoundingSphere();
		if( !currentNode->isLeaf() )
		for( unsigned i=0;i<8;i++ )
		{
			AIOctreeNode * c = currentNode->getChild(i);
			if( c )
				stack.push_back(c);
		}
	}
}