//---------------------------------------------------------------------------
#ifndef AttachedIndexedOctreeGeometricIteratorH
#define AttachedIndexedOctreeGeometricIteratorH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "SimpleOctree.h"
#include "AttachedIndexedOctree.h"
#include "GeometricRange.h"
#include "PointCloudIterators.h"
#include "PointSet.inline.h"
//---------------------------------------------------------------------------

namespace X4 {

class X3GEOMETRY_API AttachedIndexedOctreeGeometricIterator: public BasicPointCloudIterator,  // INTERFACE
																				  public ModifyablePointCloudIterator,  // INTERFACE
                                                              public IndexedPointCloudIterator // INTERFACE
{
 private:
   GeometricRange *m_range;

   AIOctreeNode* rootNode;
   std::vector<AIOctreeNode*> nodeStack;

   std::vector<card32>* currentBox;
   card32 currentBoxPosition;

   PointSet* pntSet;
   AAT POSITION;

   card32 atPoint;
   bool noMorePoints;
   bool pullBoxFromStack();
   bool getNextPointIndex(card32 &nextPoint );

 public:
   void setup(SceneObjectIterator* , const card32& ptsInNode, MemoryManager *,GeometricRange *m_range);

   virtual void reset();
   /// go to next point
   virtual void next() ;
   /// true if end-of-set
   virtual bool atEnd() ;

   virtual card64 getNumPointsInBlock() const; 
   virtual card64 getCurrentPointIndex() const;

   template <class Type, int dim>
   inline StaticVector<Type, dim> genericGet(AAT token) const {
        return pntSet->get<Type, dim>(atPoint,  token) ;
   }

   template <class Type>
   inline Type genericGetS(const AAT& token) const {
        const StaticVector<Type, 1> v = pntSet->get<Type, 1>(atPoint, token);
        return v[0];
   }


   /// get attributes in different formats
   virtual card8     get1ub(const AAT &token) const ;
   virtual Vector2ub get2ub(const AAT &token) const ;
   virtual Vector3ub get3ub(const AAT &token) const ;
   virtual Vector4ub get4ub(const AAT &token) const ;

   virtual int32    get1i(const AAT &token) const ;
   virtual Vector2i get2i(const AAT &token) const ;
   virtual Vector3i get3i(const AAT &token) const ;
   virtual Vector4i get4i(const AAT &token) const ;

   virtual float32  get1f(const AAT &token) const ;
   virtual Vector2f get2f(const AAT &token) const ;
   virtual Vector3f get3f(const AAT &token) const ;
   virtual Vector4f get4f(const AAT &token) const ;

   /// ask VertexDescriptor for buffer size
   virtual void     getVertex(char *buffer) const ;

   // --- blockwise access NOT useable here!
   virtual bool nextBlock(){x4Assert(false); return false;}
   virtual const PointSet * lockBlockRead(){x4Assert(false); return NULL;}
   virtual void unlock(){x4Assert(false);}


   template <class Type, int dim>
   inline void genericSet(const AAT& token, const StaticVector<Type, dim> &v) const {
         pntSet->set<Type, dim>(atPoint, token, v);

   }

   template <class Type>
   inline void genericSetS(const AAT& token, const Type &v) const {
        StaticVector<Type, 1> v1; v1[0] = v;
       pntSet->set<Type, 1>(atPoint, token, v1);

   }

   /// set attributes in different formats
   virtual void set1ub(const AAT &token, const card8 &x)     const ;
   virtual void set2ub(const AAT &token, const Vector2ub &v) const ;
   virtual void set3ub(const AAT &token, const Vector3ub &v) const ;
   virtual void set4ub(const AAT &token, const Vector4ub &v) const ;

   virtual void set1i(const AAT &token, const int32 &x)     const ;
   virtual void set2i(const AAT &token, const Vector2i &v) const ;
   virtual void set3i(const AAT &token, const Vector3i &v) const ;
   virtual void set4i(const AAT &token, const Vector4i &v) const ;

   virtual void set1f(const AAT &token, const float32 &x)  const ;
   virtual void set2f(const AAT &token, const Vector2f &v) const ;
   virtual void set3f(const AAT &token, const Vector3f &v) const ;
   virtual void set4f(const AAT &token, const Vector4f &v) const ;

   /// ask VertexDescriptor for size
   virtual void setVertex(const char *data) const ;


   // --- blockwise access NOT used here!
   virtual PointSet *lockBlockReadWrite(){x4Assert(false); return NULL;}




};


}//ns

#endif
