//---------------------------------------------------------------------------
#ifndef DebugOutputImagesH
#define DebugOutputImagesH
//---------------------------------------------------------------------------
#include "x3geometry.h"
#include "ImageDisplayer.h"
#include "DynamicLinearAlgebra.inline.h"
#include "RGBAImage.h"
//---------------------------------------------------------------------------


namespace X4 {


// -------- RGBAImageFloat32

class DBOImgFloat32 {
 public:
   std::string name;
   const RGBAFloat32Image *img;
   DBOImgFloat32(std::string name, const RGBAFloat32Image &img) {
      this->name = name;
      this->img = &img;
   }
};

/// output operator for RGBAImageFloat32 to debug output
inline DebugOutput &operator<<(DebugOutput &dbo, const RGBAFloat32Image &output) {
   ImageDisplayer::DisplayRGBAFloat32Image ev(&output);
   debugOutput.displayEvent(ImageDisplayer::getClass(), "image", &ev);
   return dbo;
}
/// output operator for RGBAImageFloat32 to debug output (with name)
inline DebugOutput &operator<<(DebugOutput &dbo, const DBOImgFloat32 &output) {
   ImageDisplayer::DisplayRGBAFloat32Image ev(output.img);
   debugOutput.displayEvent(ImageDisplayer::getClass(), output.name, &ev);
   return dbo;
}


// -------- RegularDataF


class DBOImgRegDataF {
public:
   std::string name;
   const RegularDataF *img;
   DBOImgRegDataF(std::string name, const RegularDataF &img) {
      this->name = name;
      this->img = &img;
   }
};

/// output operator for RegularDataF to debug output
inline DebugOutput &operator<<(DebugOutput &dbo, const RegularDataF &output) {
   ImageDisplayer::Display2DRegularData ev(&output);
   debugOutput.displayEvent(ImageDisplayer::getClass(), "image", &ev);
   return dbo;
}
/// output operator for RegularDataF to debug output (with name)
inline DebugOutput &operator<<(DebugOutput &dbo, const DBOImgRegDataF &output) {
   ImageDisplayer::Display2DRegularData ev(output.img);
   debugOutput.displayEvent(ImageDisplayer::getClass(), output.name, &ev);
   return dbo;
}


// -------- QImage


class DBOQImage {
public:
   std::string name;
   const QImage *img;
   DBOQImage(std::string name, const QImage &img) {
      this->name = name;
      this->img = &img;
   }
};

/// output operator for QImages to debug output
inline DebugOutput &operator<<(DebugOutput &dbo, const QImage &output) {
   ImageDisplayer::DisplayQImage ev(&output);
   debugOutput.displayEvent(ImageDisplayer::getClass(), "image", &ev);
   return dbo;
}
/// output operator for QImages to debug output (with name)
inline DebugOutput &operator<<(DebugOutput &dbo, const DBOQImage &output) {
   ImageDisplayer::DisplayQImage ev(output.img);
   debugOutput.displayEvent(ImageDisplayer::getClass(), output.name, &ev);
	return dbo;
}


// -------- QImage


class DBOPointSet {
public:
   std::string name;
   const PointSet *img;
   DBOPointSet(std::string name, const PointSet *img) {
      this->name = name;
      this->img = img;
   }
};

/// output operator for PointSets to debug output
inline DebugOutput &operator<<(DebugOutput &dbo, const PointSet *output) {
   ImageDisplayer::DisplayPointSet ev(output);
   debugOutput.displayEvent(ImageDisplayer::getClass(), "image", &ev);
	return dbo;
}
/// output operator for PointSets to debug output (with name)
inline DebugOutput &operator<<(DebugOutput &dbo, const DBOPointSet &output) {
   ImageDisplayer::DisplayPointSet ev(output.img);
   debugOutput.displayEvent(ImageDisplayer::getClass(), output.name, &ev);
   return dbo;
}


// -------- RGBAImageFloat32


// /// works for DynamicMatrix, StaticMatrix, SparseMatrix
// template<typename MatrixType>
// class DBODMatrix {
// public:
//    std::string name;
//    const MatrixType *img;
//    DBODMatrix(std::string name, const MatrixType &img) {
//       this->name = name;
//       this->img = &img;
//    }
// };
// 
// /// output operator for matrix types to debug output (with name)
// template<typename MatrixType>
// inline DebugOutput &operator<<(DebugOutput &dbo, const DBODMatrix<MatrixType> &output) {
//    RGBAFloat32Image tmp(output.img->getColumns(), output.img->getRows());
//    const card32 w = output.img->getColumns();
//    const card32 h = output.img->getRows();
//    for (card32 y=0; y<h; y++) {
//       for (card32 x=0; x<w; x++) {
//          tmp.setPixel(x,y, (*(output.img))[x][y]);
//       }
//    }
//    ImageDisplayer::DisplayRGBAFloat32Image ev(&tmp);
//    debugOutput.displayEvent(ImageDisplayer::getClass(), output.name, &ev);
// }
// /// output operator for matrix types to debug output
// template<typename MatrixType>
// inline DebugOutput &operator<< (DebugOutput &dbo, const MatrixType &output) {
//    debugOutput << DBODMatrix<MatrixType>("image", output);
// }

}


#endif