#ifndef FastSphereQuerryH
#define FastSphereQuerryH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AttachedIndexedOctree.h"
//---------------------------------------------------------------------------

namespace X4
{

class AttachedIndexedOctree;
class PointCloud;
class SecondaryDataStructureIterator;
class SceneObjectIterator;
class BasicPointCloudIterator;

template<class T> 
class FSQ_Stack
{
	std::vector<T> data;
	unsigned mySize;

public:
	FSQ_Stack() {mySize = 0;}

	void clear()
	{
		mySize = 0;
	}

	inline void push_back(const T& v)
	{
		if( data.size() == mySize )
			data.push_back(v);

		data[mySize] = v;
		mySize++;
	}

	inline T pop_back()
	{
		mySize--;
		return data[mySize];
	}

	inline bool empty() const 
	{
		return mySize == 0;
	}

};

/**
	fast sphere querry - operates on AttachedIndexedOctree
	no ordering is performed
*/
class X3GEOMETRY_API FastSphereQuerry
{

private:

	std::vector<mpcard> m_IndexCache;
	std::vector<float>	m_SquaredDistanceCache;
	mpcard			m_NumPoints;
	
	AttachedIndexedOctree	* m_Octree;
	SecondaryDataStructureIterator	* m_SDS;
	SceneObjectIterator * m_SceneIter;
	BasicPointCloudIterator * m_BasicIt;
	PointSet *		m_PS;

	// temp members
	FSQ_Stack<AIOctreeNode*> node_queue;

public:
	FastSphereQuerry(PointCloud * pc);
	FastSphereQuerry(AttachedIndexedOctree * octree, PointSet * ps);
	~FastSphereQuerry(void);

	typedef void (Object::* ProcessPointMethod)( const Vector3f &, float, mpcard );

	/// querries all points in a sphere
	/// indices are valid until next call or destruction of this object
	void querry( const Vector3f &pos, const float radius, mpcard ** indices, mpcard &numPoints );

	/// querries all points in a sphere shell
	/// indices are valid until next call or destruction of this object
	void querry( const Vector3f &pos, const float minRadius, const float maxRadius, mpcard ** indices, mpcard &numPoints );

	/// get squared distance from point indices[i]
	/// querry must be called before using this method!
	inline float getSquaredDistance(unsigned i) const {return m_SquaredDistanceCache[i];}

	/// processes all points in a sphere
	void process( const Vector3f &pos, const float radius, Object * obj, ProcessPointMethod method);

};

}

#endif