//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GenericSwapAllOutOfCorePointCloud.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointCloudIterators.h"
#include "SceneObjectIterator.h"
#include "BinaryObjectStreams.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


// --- custom meta class init

MetaClass* GenericSwapAllOutOfCorePointCloud::__MetaClass = NULL;

const MetaClass* GenericSwapAllOutOfCorePointCloud::getInstanceClass() const {
   return getClass();
}

Object* GenericSwapAllOutOfCorePointCloud::newInstance() {
	return new GenericSwapAllOutOfCorePointCloud;
}

const MetaClass* GenericSwapAllOutOfCorePointCloud::getClass() {
   if (__MetaClass==NULL) {
		__MetaClass = new MetaClass( "GenericSwapAllOutOfCorePointCloud" , "no description.",
                                  newInstance, sizeof( GenericSwapAllOutOfCorePointCloud ));
   }
   return __MetaClass;
}

void GenericSwapAllOutOfCorePointCloud::finalShutdownCache() {
   if (pcPool != NULL) {
		delete pcPool;
		pcPool = NULL;
   }
}

void GenericSwapAllOutOfCorePointCloud::shutdown() {
   delete __MetaClass;
	// finalShutdownCache(); // now in preShutdownEvents
}

void GenericSwapAllOutOfCorePointCloud::init(const MetaClass *parentClass) 
{
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0 );
   __MetaClass->copyParentProperties();
	initMetaClass();
	pcPool = new GSAPointCloudPool();
}

void GenericSwapAllOutOfCorePointCloud::initMetaClass() {
	BEGIN_CLASS_INIT( GenericSwapAllOutOfCorePointCloud );
	ADD_STRING_PROP(filename, 0)
	ADD_BOOLEAN_PROP(relativePath, 0)
}

IMPLEMENT_X4_CLASS( GSAOOCPointCloudIterator, 0) {
	BEGIN_CLASS_INIT( GSAOOCPointCloudIterator );
}



// --- GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool



GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::GSAPointCloudPool(mpcard maxMem) : mutex(QMutex::Recursive)
{
	this->maxMem = maxMem;
	memUsed = 0;
}

void GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::checkSwap() {
	list<std::string>::iterator pos = lruQueue.begin();
	list<std::string>::iterator end = lruQueue.end();
	
	while (memUsed > maxMem && pos != end) {
		std::string next = *pos;
		MapType::iterator it = inCore.find(next);
		if (it == inCore.end()) {
			x4Assert(false);
			list<std::string>::iterator nPos = pos; ++nPos;
			lruQueue.erase(pos);
			pos = nPos;
			continue;
		}
		if (it->second.lockCounter > 0) {
			++pos;
			continue;
		}
		memUsed -= it->second.size;
		x4Assert(memUsed >= 0);
		if (it->second.dirty) {
			BinaryOutputObjectStream out(it->first.c_str());
			out.writeObject(it->second.pc);
		}
		delete it->second.pc;
		inCore.erase(it);
		list<std::string>::iterator nPos = pos; ++nPos;
		lruQueue.erase(pos);
		pos = nPos;
	}
}

GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::~GSAPointCloudPool() {
	sync();
	MapType::iterator pos = inCore.begin();
	MapType::iterator end = inCore.end();
	while (pos != end) {
		delete pos->second.pc;
		++pos;
	}
	inCore.clear();
	lruQueue.clear();
}

void GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::sync() {
	MapType::iterator pos = inCore.begin();
	MapType::iterator end = inCore.end();
	while (pos != end) {
		if (pos->second.dirty) {
			BinaryOutputObjectStream out(pos->first.c_str());
			out.writeObject(pos->second.pc);
			pos->second.dirty = false;
		}
		++pos;
	}
}

mpcard GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::computeSize(PointCloud *pc) {
	return (mpcard)(pc->getNumPoints()) * (mpcard)(pc->getDescr()->getSize()) + pc->getAttachments()->getSizeInMemory();
}

PointCloud *GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::fetchPC(std::string filename) {
	mutex.lock();
	checkSwap();
	MapType::iterator pos = inCore.find(filename);
	if (pos != inCore.end()) {
		lruQueue.erase(pos->second.lruEntry);
		lruQueue.push_back(filename);
		pos->second.lruEntry = lruQueue.end();
		--(pos->second.lruEntry);
		updateSize(pos->second);
		mutex.unlock();
		return pos->second.pc;
	} else {
		BinaryInputObjectStream in(filename.c_str());
		EntryType e;
		in.readAnyObject(e.pc);
		e.dirty = false;
		e.lockCounter = 0;
		lruQueue.push_back(filename);
		e.lruEntry = lruQueue.end();
		--(e.lruEntry);
		e.size = computeSize(e.pc);
		memUsed += e.size;
		inCore[filename] = e;
		mutex.unlock();
		return e.pc;
	}
}

void GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::update(std::string filename, card32 lockCounter, SetDirtyFlag dirty) {
	mutex.lock();
	fetchPC(filename);
	MapType::iterator pos = inCore.find(filename);
	if (pos != inCore.end()) {
		pos->second.lockCounter = lockCounter;
		if (dirty == setDirty) {
			pos->second.dirty = true;
		} else if (dirty == setClean) {
			pos->second.dirty = false;
		}
		updateSize(pos->second);
	} else {
		mutex.unlock();
		throw EInvalidState(std::string("GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::update() - filename \"")+filename+std::string("\" not found."));
	}
	mutex.unlock();
}

void GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::setMaxMem(mpcard maxMem) {
	this->maxMem = maxMem;
	checkSwap();
}

void GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool::updateSize(EntryType &e) {
	mpint oldSize = e.size;
	e.size = computeSize(e.pc);
	mpint newSize = e.size;
	memUsed += (newSize - oldSize);
}



// --- GenericSwapAllOutOfCorePointCloud



GenericSwapAllOutOfCorePointCloud::GSAPointCloudPool *GenericSwapAllOutOfCorePointCloud::pcPool = NULL;

GenericSwapAllOutOfCorePointCloud::GenericSwapAllOutOfCorePointCloud() {
	vertexDescriptorCopy = NULL;
	relativePath = true;
	lockCounter = 0;
}

void GenericSwapAllOutOfCorePointCloud::clearAndSetup(const PointCloud *pc, std::string filename, bool relativePath) 
{
	this->filename = filename;
	this->relativePath = relativePath;
	lockCounter = 0;
	BinaryOutputObjectStream out(filename.c_str());
	out.writeObject(pc);	
}

void GenericSwapAllOutOfCorePointCloud::assign(const Object* obj, X4CopyContext *context) {
	if( dynamic_cast<const GenericSwapAllOutOfCorePointCloud*>(obj) ) 
	{
		copyObjectProperties(obj, this);
	}
	else
	{
		fetchPC()->assign(obj,context);
	}
	setDirty();
}

SceneObject *GenericSwapAllOutOfCorePointCloud::createOOCAwareCopy(std::string idPrefix, bool insertOriginal, std::string idSuffix) {
	GenericSwapAllOutOfCorePointCloud *result = (GenericSwapAllOutOfCorePointCloud*)newInstance();
	std::string newFilename = removeFilenameFromPath(filename) + idPrefix + (insertOriginal?removePathFromFilename(filename):string("")) + idSuffix;
	result->clearAndSetup(pcPool->fetchPC(filename), newFilename, relativePath);
	return result;
}


void GenericSwapAllOutOfCorePointCloud::truncatePathRelativeTo(std::string basePath, std::string filename, std::string &result, bool &matches) {
	if (basePath.size() >= filename.size()) {
		matches = false;
	} else {
		for (unsigned i=0; i<basePath.size(); i++) {
			if (toupper(basePath[i]) != toupper(filename[i])) {
				matches = false;
				return;
			}
		}
		std::string resultToReturn = filename.substr(basePath.size(), filename.size() - basePath.size());
		result = resultToReturn;
		matches = true;
	}
}

void GenericSwapAllOutOfCorePointCloud::read(InputObjectStream *s) {
	PointCloud::read(s);

	bool matches = false;
	s->readBool(matches);
	if (relativePath && matches) {
		filename = removeFilenameFromPath(s->getFileNameIfFileStream()) + filename;
	}
}

void GenericSwapAllOutOfCorePointCloud::write(OutputObjectStream *s) const {
	std::string saveFileName = filename;
	bool matches = false;
	if (relativePath) {
		std::string result = allDirSeperatorsToDos(filename);
		
		truncatePathRelativeTo(
			allDirSeperatorsToDos(removeFilenameFromPath(s->getFileNameIfFileStream())),
			allDirSeperatorsToDos(filename), 
			result, 
			matches
		);
		if (matches) {
			const_cast<GenericSwapAllOutOfCorePointCloud*>(this)->filename = result;
		}
		#ifndef WIN32
		this needs to be fixed for unix (dos directory separators)
		#endif
	}
	PointCloud::write(s);
	s->writeBool(matches);
	const_cast<GenericSwapAllOutOfCorePointCloud*>(this)->filename = saveFileName;
}


GenericSwapAllOutOfCorePointCloud::~GenericSwapAllOutOfCorePointCloud() {
	if (vertexDescriptorCopy) delete vertexDescriptorCopy;
}



void GenericSwapAllOutOfCorePointCloud::incrementLockCounter() const {
	lockCounter++;
	if (lockCounter == 1) {
		pcPool->update(filename, 1, GSAPointCloudPool::noChange);
	}
}

void GenericSwapAllOutOfCorePointCloud::decrementLockCounter() const {
	lockCounter--;
	if (lockCounter == 0) {
		pcPool->update(filename, 0, GSAPointCloudPool::noChange);
	}
}

void GenericSwapAllOutOfCorePointCloud::updatePC() {
	pcPool->update(filename, 0, GSAPointCloudPool::noChange);
}

void GenericSwapAllOutOfCorePointCloud::setDirty() {
	pcPool->update(filename, (lockCounter>0)?1:0, GSAPointCloudPool::setDirty);
}


const VertexDescriptor *GenericSwapAllOutOfCorePointCloud::getDescr() const {
	if (vertexDescriptorCopy) delete vertexDescriptorCopy;
	incrementLockCounter();
	vertexDescriptorCopy = static_cast<VertexDescriptor*>(const_cast<GenericSwapAllOutOfCorePointCloud*>(this)->fetchPC()->getDescr()->copy());
	decrementLockCounter();
	return vertexDescriptorCopy;
}

PointCloud *GenericSwapAllOutOfCorePointCloud::appendFields(const VertexDescriptor *newAttribs) const {
	incrementLockCounter();
	PointCloud *newPc = const_cast<GenericSwapAllOutOfCorePointCloud*>(this)->fetchPC()->appendFields(newAttribs);
	decrementLockCounter();
	return newPc;
}

card64 GenericSwapAllOutOfCorePointCloud::getNumPoints() const {
	incrementLockCounter();
	GenericSwapAllOutOfCorePointCloud * consthack = (GenericSwapAllOutOfCorePointCloud *)this;
	card64 numPoints = consthack->fetchPC()->getNumPoints();
	decrementLockCounter();
	return numPoints;	
}

void GenericSwapAllOutOfCorePointCloud::clear() {
	incrementLockCounter();
	fetchPC()->clear();
	decrementLockCounter();
}

SceneObjectIterator *GenericSwapAllOutOfCorePointCloud::createIterator(SceneObjectIteratorType type) {
	if (!supportsIteratorType(type)) {
		return NULL;
	} else {
		incrementLockCounter();
		GSAOOCPointCloudIterator *newGSAIt = new GSAOOCPointCloudIterator(this, fetchPC()->createIterator(type));
		decrementLockCounter();
		return newGSAIt;
	}
}

bool GenericSwapAllOutOfCorePointCloud::supportsIteratorType(SceneObjectIteratorType cap) {
   static SOIT SUPPORTED_CAPS = SOIT(
		SOIT::CAP_ATTACH_SECONDARY | SOIT::CAP_HIERARCHICAL | SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC |
		SOIT::CAP_DYNAMIC_PC | SOIT::CAP_PARAMETRIC_PC | SOIT::CAP_DYNAMIC_PARAMETRIC_PC | SOIT::CAP_TRIANGLE_MESH |
		SOIT::CAP_REGULAR_VOLUME | SOIT::CAP_INDEXED_PC | SOIT::CAP_RANDOM_ACCESS_PC | SOIT::CAP_TOPOLOGY | SOIT::CAP_VOLUMETRIC);
	if (SUPPORTED_CAPS.providesCapsFor(cap)) {
		incrementLockCounter();
		bool supported = fetchPC()->supportsIteratorType(cap);
		decrementLockCounter();
		return supported;
	} else {
		return false;
	}
}

PointCloud *GenericSwapAllOutOfCorePointCloud::fetchPC() {
	// We must never fetch a PC when it's not locked. No matter what operation we perform on it, in
	// a multi-threaded environment there's always the chance that it is evicted before we could get
	// a chance at performing that operation.
	//x4Assert(lockCounter > 0); 
	return pcPool->fetchPC(filename);
}

mpcard GenericSwapAllOutOfCorePointCloud::getMaxMem() {
	return pcPool->getMaxMem();
}
void GenericSwapAllOutOfCorePointCloud::setMaxMem(mpcard mm) {
	pcPool->setMaxMem(mm);
}



// --- 	 GSAOOCPointCloudIterator

GSAOOCPointCloudIterator::GSAOOCPointCloudIterator() {
	myPC = NULL;
	it = NULL;
	interfaceSecondaryDataStructureIterator = NULL;
	interfaceHierarchicalIterator = NULL;
	interfaceBasicPointCloudIterator = NULL;
	interfaceIndexedPointCloudIterator = NULL;
	interfaceRandomAccessPointCloudIterator = NULL;
	interfaceModifyablePointCloudIterator = NULL;
	interfaceDynamicPointCloudIterator = NULL;
	interfaceDynamicParametricPointCloudIterator = NULL;
	interfaceParametricPointCloudIterator = NULL;
	interfaceTopologyIterator = NULL;
	interfaceVolumetricIterator = NULL;
}

void GSAOOCPointCloudIterator::assign(const Object* obj, X4CopyContext *context) {
	const GSAOOCPointCloudIterator *other = dynamic_cast<const GSAOOCPointCloudIterator*>(obj);
	x4Assert(obj);
	if (!obj) return;
	if (obj == this) return;
	delete it;
	myPC = other->myPC;
	if (other->it == NULL) {
		it = NULL;
		interfaceSecondaryDataStructureIterator = NULL;
		interfaceHierarchicalIterator = NULL;
		interfaceBasicPointCloudIterator = NULL;
		interfaceIndexedPointCloudIterator = NULL;
		interfaceRandomAccessPointCloudIterator = NULL;
		interfaceModifyablePointCloudIterator = NULL;
		interfaceDynamicPointCloudIterator = NULL;
		interfaceDynamicParametricPointCloudIterator = NULL;
		interfaceParametricPointCloudIterator = NULL;
		interfaceTopologyIterator = NULL;
		interfaceVolumetricIterator = NULL;
	} else {
		it = (SceneObjectIterator*)(other->it->copy());
		interfaceSecondaryDataStructureIterator = dynamic_cast<SecondaryDataStructureIterator*>(it);
		interfaceHierarchicalIterator = dynamic_cast<HierarchicalIterator*>(it);
		interfaceBasicPointCloudIterator = dynamic_cast<BasicPointCloudIterator*>(it);
		interfaceIndexedPointCloudIterator = dynamic_cast<IndexedPointCloudIterator*>(it);
		interfaceRandomAccessPointCloudIterator = dynamic_cast<RandomAccessPointCloudIterator*>(it);
		interfaceModifyablePointCloudIterator = dynamic_cast<ModifyablePointCloudIterator*>(it);
		interfaceDynamicPointCloudIterator = dynamic_cast<DynamicPointCloudIterator*>(it);
		interfaceDynamicParametricPointCloudIterator = dynamic_cast<DynamicParametricPointCloudIterator*>(it);
		interfaceParametricPointCloudIterator = dynamic_cast<ParametricPointCloudIterator*>(it);
		interfaceTopologyIterator = dynamic_cast<TopologyIterator*>(it);
		interfaceVolumetricIterator = dynamic_cast<VolumetricIterator*>(it);
		myPC->incrementLockCounter();
	}
}



GSAOOCPointCloudIterator::GSAOOCPointCloudIterator(GenericSwapAllOutOfCorePointCloud *myPC, SceneObjectIterator *it) 
{
	this->myPC = myPC;
	this->it = it;
	interfaceSecondaryDataStructureIterator = dynamic_cast<SecondaryDataStructureIterator*>(it);
	interfaceHierarchicalIterator = dynamic_cast<HierarchicalIterator*>(it);
	interfaceBasicPointCloudIterator = dynamic_cast<BasicPointCloudIterator*>(it);
	interfaceIndexedPointCloudIterator = dynamic_cast<IndexedPointCloudIterator*>(it);
	interfaceRandomAccessPointCloudIterator = dynamic_cast<RandomAccessPointCloudIterator*>(it);
	interfaceModifyablePointCloudIterator = dynamic_cast<ModifyablePointCloudIterator*>(it);
	interfaceDynamicPointCloudIterator = dynamic_cast<DynamicPointCloudIterator*>(it);
	interfaceDynamicParametricPointCloudIterator = dynamic_cast<DynamicParametricPointCloudIterator*>(it);
	interfaceParametricPointCloudIterator = dynamic_cast<ParametricPointCloudIterator*>(it);
	interfaceTopologyIterator = dynamic_cast<TopologyIterator*>(it);
	interfaceVolumetricIterator = dynamic_cast<VolumetricIterator*>(it);
	myPC->incrementLockCounter();
}

GSAOOCPointCloudIterator::~GSAOOCPointCloudIterator() 
{
	myPC->updatePC();
	myPC->decrementLockCounter();
	delete it;
}

AttachedDataContainer *GSAOOCPointCloudIterator::lockAttachments()
{
	x4Assert(interfaceSecondaryDataStructureIterator);
	myPC->incrementLockCounter();
	myPC->setDirty();
	return interfaceSecondaryDataStructureIterator->lockAttachments();
}
void GSAOOCPointCloudIterator::unlockAttachments() {
	x4Assert(interfaceSecondaryDataStructureIterator);
	myPC->decrementLockCounter();
	interfaceSecondaryDataStructureIterator->unlockAttachments();
}

card32 GSAOOCPointCloudIterator::getNumChildren() {
	x4Assert(interfaceHierarchicalIterator);
	return interfaceHierarchicalIterator->getNumChildren();
}
bool GSAOOCPointCloudIterator::goToChild(card32 index) {
	x4Assert(interfaceHierarchicalIterator);
	return interfaceHierarchicalIterator->goToChild(index);
}
bool GSAOOCPointCloudIterator::goToParent() {
	x4Assert(interfaceHierarchicalIterator);
	return interfaceHierarchicalIterator->goToParent();
}
BoundingBox3f GSAOOCPointCloudIterator::getBoundingBox() {
	x4Assert(interfaceHierarchicalIterator);
	return interfaceHierarchicalIterator->getBoundingBox();
}
bool GSAOOCPointCloudIterator::isLeaf() const {
	x4Assert(interfaceHierarchicalIterator);
	return interfaceHierarchicalIterator->isLeaf();
}
bool GSAOOCPointCloudIterator::getNodeInfo(HierarchicalIterator::NodeInfoType whichInfo, const void *parameter, void *data) {
	x4Assert(interfaceHierarchicalIterator);
	return interfaceHierarchicalIterator->getNodeInfo(whichInfo, parameter, data);
}


void GSAOOCPointCloudIterator::reset() {
	x4Assert(interfaceBasicPointCloudIterator);
	interfaceBasicPointCloudIterator->reset();
}
void GSAOOCPointCloudIterator::next() {
	x4Assert(interfaceBasicPointCloudIterator);
	interfaceBasicPointCloudIterator->next();
}
bool GSAOOCPointCloudIterator::atEnd() {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->atEnd();
}
card8     GSAOOCPointCloudIterator::get1ub(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get1ub(token);
}
Vector2ub GSAOOCPointCloudIterator::get2ub(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get2ub(token);
}
Vector3ub GSAOOCPointCloudIterator::get3ub(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get3ub(token);
}
Vector4ub GSAOOCPointCloudIterator::get4ub(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get4ub(token);
}
int32    GSAOOCPointCloudIterator::get1i(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get1i(token);
}
Vector2i GSAOOCPointCloudIterator::get2i(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get2i(token);
}
Vector3i GSAOOCPointCloudIterator::get3i(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get3i(token);
}
Vector4i GSAOOCPointCloudIterator::get4i(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get4i(token);
}
card32    GSAOOCPointCloudIterator::get1u(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get1u(token);
}
Vector2u GSAOOCPointCloudIterator::get2u(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get2u(token);
}
Vector3u GSAOOCPointCloudIterator::get3u(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get3u(token);
}
Vector4u GSAOOCPointCloudIterator::get4u(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get4u(token);
}
float32  GSAOOCPointCloudIterator::get1f(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get1f(token);
}
Vector2f GSAOOCPointCloudIterator::get2f(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get2f(token);
}
Vector3f GSAOOCPointCloudIterator::get3f(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get3f(token);
}
Vector4f GSAOOCPointCloudIterator::get4f(const AAT &token) const {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->get4f(token);
}
void     GSAOOCPointCloudIterator::getVertex(char *buffer) const {
	x4Assert(interfaceBasicPointCloudIterator);
	interfaceBasicPointCloudIterator->getVertex(buffer);
}
bool GSAOOCPointCloudIterator::nextBlock() {
	x4Assert(interfaceBasicPointCloudIterator);
	return interfaceBasicPointCloudIterator->nextBlock();
}
const PointSet* GSAOOCPointCloudIterator::lockBlockRead() {
	x4Assert(interfaceBasicPointCloudIterator);
	myPC->incrementLockCounter();
	return interfaceBasicPointCloudIterator->lockBlockRead();
}
void GSAOOCPointCloudIterator::unlock() {
	x4Assert(interfaceBasicPointCloudIterator);
	myPC->decrementLockCounter();
	return interfaceBasicPointCloudIterator->unlock();
}

card64 GSAOOCPointCloudIterator::getNumPointsInBlock() const {
	x4Assert(interfaceIndexedPointCloudIterator);
	return interfaceIndexedPointCloudIterator->getNumPointsInBlock();
}
card64 GSAOOCPointCloudIterator::getCurrentPointIndex() const {
	x4Assert(interfaceIndexedPointCloudIterator);
	return interfaceIndexedPointCloudIterator->getCurrentPointIndex();
}

void GSAOOCPointCloudIterator::goToIndex(card64 i) {
	x4Assert(interfaceRandomAccessPointCloudIterator);
	interfaceRandomAccessPointCloudIterator->goToIndex(i);
}

void GSAOOCPointCloudIterator::set1ub(const AAT &token, const card8 &x)     const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set1ub(token, x);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set2ub(const AAT &token, const Vector2ub &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set2ub(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set3ub(const AAT &token, const Vector3ub &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set3ub(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set4ub(const AAT &token, const Vector4ub &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set4ub(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set1i(const AAT &token, const int32 &x)     const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set1i(token, x);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set2i(const AAT &token, const Vector2i &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set2i(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set3i(const AAT &token, const Vector3i &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set3i(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set4i(const AAT &token, const Vector4i &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set4i(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set1u(const AAT &token, const card32 &x)     const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set1u(token, x);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set2u(const AAT &token, const Vector2u &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set2u(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set3u(const AAT &token, const Vector3u &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set3u(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set4u(const AAT &token, const Vector4u &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set4u(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set1f(const AAT &token, const float32 &x)  const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set1f(token, x);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set2f(const AAT &token, const Vector2f &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set2f(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set3f(const AAT &token, const Vector3f &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set3f(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::set4f(const AAT &token, const Vector4f &v) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->set4f(token, v);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::setVertex(const char *data) const {
	x4Assert(interfaceModifyablePointCloudIterator);
	interfaceModifyablePointCloudIterator->setVertex(data);
	myPC->setDirty();
}
PointSet* GSAOOCPointCloudIterator::lockBlockReadWrite(ModifyablePointCloudIterator::LockRWHint hint) {
	x4Assert(interfaceModifyablePointCloudIterator);
	myPC->incrementLockCounter();
	myPC->setDirty();
	return interfaceModifyablePointCloudIterator->lockBlockReadWrite(hint);
}


void GSAOOCPointCloudIterator::add(const char *newPoint) {
	x4Assert(interfaceDynamicPointCloudIterator);
	interfaceDynamicPointCloudIterator->add(newPoint);
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::add(const PointSet *newPoints) {
	x4Assert(interfaceDynamicPointCloudIterator);
	interfaceDynamicPointCloudIterator->add(newPoints);	
	myPC->setDirty();
}
void GSAOOCPointCloudIterator::deleteCurrent() {
	x4Assert(interfaceDynamicPointCloudIterator);
	interfaceDynamicPointCloudIterator->deleteCurrent();
	myPC->setDirty();
}

void GSAOOCPointCloudIterator::setSize(card32 width, card32 height) {
	x4Assert(interfaceDynamicParametricPointCloudIterator);
	interfaceDynamicParametricPointCloudIterator->setSize(width, height);
	myPC->setDirty();
}

void GSAOOCPointCloudIterator::goTo(int32 x, int32 y) {
	x4Assert(interfaceParametricPointCloudIterator);
	interfaceParametricPointCloudIterator->goTo(x, y);
}
card32 GSAOOCPointCloudIterator::getWidth() {
	x4Assert(interfaceParametricPointCloudIterator);
	return interfaceParametricPointCloudIterator->getWidth();
}
card32 GSAOOCPointCloudIterator::getHeight() {
	x4Assert(interfaceParametricPointCloudIterator);
	return interfaceParametricPointCloudIterator->getHeight();
}


card32 GSAOOCPointCloudIterator::getNumNeighbours() {
	x4Assert(interfaceTopologyIterator);
	return interfaceTopologyIterator->getNumNeighbours();
}
PointIndex GSAOOCPointCloudIterator::getNeighbour( card32 nIndex ) {
	x4Assert(interfaceTopologyIterator);
	return interfaceTopologyIterator->getNeighbour(nIndex);
}


void GSAOOCPointCloudIterator::goTo(card32 x, card32 y, card32 z) {
	x4Assert(interfaceVolumetricIterator);
	interfaceVolumetricIterator->goTo(x,y,z);
}
Vector3i GSAOOCPointCloudIterator::getVolumeSize() {
	x4Assert(interfaceVolumetricIterator);
	return interfaceVolumetricIterator->getVolumeSize();
}
BoundingBox3f GSAOOCPointCloudIterator::getVolumeRegion() {
	x4Assert(interfaceVolumetricIterator);
	return interfaceVolumetricIterator->getVolumeRegion();
}

const RegularVolume *GSAOOCPointCloudIterator::lockVolumeBlockRead() {
	x4Assert(interfaceVolumetricIterator);
	return interfaceVolumetricIterator->lockVolumeBlockRead();
}

RegularVolume *GSAOOCPointCloudIterator::lockVolumeBlockReadWrite() {
	x4Assert(interfaceVolumetricIterator);
	return interfaceVolumetricIterator->lockVolumeBlockReadWrite();
}

void GSAOOCPointCloudIterator::unlockVolume() {
	x4Assert(interfaceVolumetricIterator);
	interfaceVolumetricIterator->unlockVolume();
}
