//---------------------------------------------------------------------------
#ifndef HierarchicalIteratorAdaptorH
#define HierarchicalIteratorAdaptorH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "PointCloudIterators.h"
#include "GeometricRange.h"
#include "PointFlags.h"
#include "IteratorAdaptors.h"
//---------------------------------------------------------------------------

namespace X4 {





///Abstract IteratorAdaptor Base Class
class X3GEOMETRY_API HierarchicalIteratorAdaptor :  public IteratorAdaptor

{
 protected:

   /// go to next leaf node in tree, returns true if an additional node is found
   bool searchNextNode();


   std::vector<card32> nextChildIndexStack;
   bool allNodesFinished;

   HierarchicalIterator* m_hr;
   virtual bool pointMeetsCond() {return true;}

   /// true if actual Node meets Condition of Adaptor
   virtual bool nodeMeetsCond()  = 0;

 public:

   HierarchicalIteratorAdaptor(PointCloudIterator* native, const VertexDescriptor *vd);
   ~HierarchicalIteratorAdaptor();

   /// true if end-of-set
   virtual bool atEnd();
   /// go to first point in Range
   virtual void reset();
   /// go to next point in Range
   virtual void next();



   // --- blockwise access (faster)
   /// step foreward to next data block, false if end-of-set (affects the same position pointer as "next()")
   virtual bool nextBlock();
   /// access raw data of current data block; call unlock to free;
   /// one data block is guaranteed to easily fit into main memory
   virtual const PointSet *lockBlockRead();
   /// free locked PointSet again
   /// & write back chainged points
   void unlock();
   /// access raw data of current data block for read-write access;
   /// call unlock to free locked PointSet
   virtual PointSet *lockBlockReadWrite(LockRWHint hint = CHANGE_EVERYTHING);

};


class X3GEOMETRY_API MHAllPointsIteratorAdaptor : public HierarchicalIteratorAdaptor
{
 protected:

   protected:
   AAT FLAGS;


   virtual bool pointMeetsCond()
   {
      return true;
   }

   virtual bool nodeMeetsCond()
   {
      return true;
   }


 public:
   MHAllPointsIteratorAdaptor(PointCloudIterator* native, const VertexDescriptor *vd)
      :HierarchicalIteratorAdaptor(native, vd){
         this->reset();
      }

   ~MHAllPointsIteratorAdaptor(){};

};



}//ns
#endif
