//---------------------------------------------------------------------------
#ifndef HierarchicalRangeQueryIteratorH
#define HierarchicalRangeQueryIteratorH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "PointCloudIterators.h"
#include "PointCloud.h"
#include "AttachedIndexedOctree.h"
//---------------------------------------------------------------------------
namespace X4 {

class X3GEOMETRY_API HierarchicalRangeQueryIterator: public PointCloudIterator, public BasicPointCloudIterator {

	protected:
		PointCloud* m_pc;
		const char* m_buffer;
		Vector3f m_seekPoint;
		bool nextPointReady;
		bool hierachyMode;
		BasicPointCloudIterator* m_basic;
		const PointSet* pntSet;
		SecondaryDataStructureIterator* m_sds;
		AttachedIndexedOctree* m_octree;
		AAT POSITION;
		std::vector<AIOctreeNode*> q;
		int32 atPoint;		// Current point index in point set
		int32 atIndex;		// Current point index in octree node
		float32 m_sqrEpsilon;

		// Iterators
		SceneObjectIterator *native;
		HierarchicalIterator* m_hr;

		void seekNextPoint();

	public:
		/// Use setSeekPointAndReset after construction
		HierarchicalRangeQueryIterator( PointCloud*, card32 subTreeSize, MemoryManager *attachedOctreeMemManager );
		~HierarchicalRangeQueryIterator();

		void setSeekPointAndReset( const Vector3f& seedPoint, const float32 epsilon);
		void reset();
		void next();
		bool atEnd();

		template <class Type, int dim>
		inline StaticVector<Type, dim> genericGet(AAT token) const {
			const char *pos =  m_buffer + token.getOffset();
			return *((StaticVector<Type, dim>*)(pos));
		}

		template <class Type>
		inline Type genericGetS(const AAT& token) const {
			const char *pos =  m_buffer + token.getOffset();
			const StaticVector<Type, 1> v = *((StaticVector<Type, 1>*)(pos));
			return v[0];
		}

		virtual card8     get1ub(const AAT &token) const;
		virtual Vector2ub get2ub(const AAT &token) const;
		virtual Vector3ub get3ub(const AAT &token) const;
		virtual Vector4ub get4ub(const AAT &token) const;
		virtual int32    get1i(const AAT &token) const;
		virtual Vector2i get2i(const AAT &token) const;
		virtual Vector3i get3i(const AAT &token) const;
		virtual Vector4i get4i(const AAT &token) const;
		virtual card32    get1u(const AAT &token) const;
		virtual Vector2u get2u(const AAT &token) const;
		virtual Vector3u get3u(const AAT &token) const;
		virtual Vector4u get4u(const AAT &token) const;
		virtual float32  get1f(const AAT &token) const;
		virtual Vector2f get2f(const AAT &token) const;
		virtual Vector3f get3f(const AAT &token) const;
		virtual Vector4f get4f(const AAT &token) const;
		virtual void     getVertex(char *buffer) const;
		virtual bool nextBlock();
		virtual const PointSet * lockBlockRead();
		virtual void unlock();

		inline card32 getCurrentPointIndex() const { return atPoint; }
};

} //namespace X4

#endif
