//---------------------------------------------------------------------------
#ifndef ImageDisplayerH
#define ImageDisplayerH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "EventDisplayer.h"
#include "RegularData.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------

class QImage;

namespace X4 {

class PointSet;
class ImageDisplayerWidget;
class RGBAFloat32Image;

/**
	EventDisplayer that shows images
*/
class X3GEOMETRY_API ImageDisplayer : public EventDisplayer
{
	X4_CLASS( ImageDisplayer )

 private:
	card32 imgNumber;
	
 protected:
	virtual QWidget *createSpecialWidget(QWidget *parent);

	ImageDisplayerWidget *widget;

 public:
	class DisplayPointSet : public EventDisplayerEvent {
	 public:
		/// point set to be displayed
		const PointSet *ps;
		/// set to "" for no changes
		std::string showChannel;
		DisplayPointSet(const PointSet *ps, std::string showChannel = "") {this->ps = ps; this->showChannel = showChannel;}
	};
	class DisplayQImage : public EventDisplayerEvent {
	 public:
		const QImage *image;
		DisplayQImage(const QImage *image) {this->image = image;}
	};
	class Display2DRegularData : public EventDisplayerEvent {
	 public:
		const RegularDataF *image;
		Display2DRegularData(const RegularDataF *image) {this->image = image;}
	};
	class DisplayRGBAFloat32Image : public EventDisplayerEvent {
	 public:
		const RGBAFloat32Image *image;
		DisplayRGBAFloat32Image(const RGBAFloat32Image *image) {this->image = image;}
	};
	class ChangeZoom : public EventDisplayerEvent {
	 public:
		double zoomFactor;
		ChangeZoom(double zoomFactor) {this->zoomFactor = zoomFactor;}
	};
	class ChangeMapping : public EventDisplayerEvent {
	 public:
		double offset, scale, gamma;
		ChangeMapping(double offset, double scale, double gamma) {this->offset = offset; this->scale = scale; this->gamma = gamma;}
	};
 
 
	ImageDisplayer();

	virtual void displayEvent(const EventDisplayerEvent *e, const char *logfile);
	
	~ImageDisplayer();
};

class X3GEOMETRY_API DOutput_TalkToPointCloudProject_Impl : public DOutput_TalkToPointCloudProject
{
public:
	void displayRGBAFloatImage(RGBAFloat32Image * image, std::string name);
	void displayQImage(QImage * image, std::string name);
};

}

#endif