//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "InCorePCTopologyGraph.h"
#include "VertexDescriptor.h"
#include "PointSetIterator.h"
//---------------------------------------------------------------------------
#include "PointSet.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( InCorePCTopologyGraph, 1 ) 
{
	BEGIN_CLASS_INIT( InCorePCTopologyGraph );
}

          
// ---
          
          
InCorePCTopologyGraph::InCorePCTopologyGraph() {
   edges = new PointSet();
	aatVIndex = NULL_AAT;
	tmpBuffer = NULL;
	tmpBuffer2 = NULL;
	tmpBufferSize = 0;
}

void InCorePCTopologyGraph::assign(const Object* obj, X4CopyContext *context) {
	const InCorePCTopologyGraph *other = dynamic_cast<const InCorePCTopologyGraph*>(obj);
	x4Assert(other);
	delete edges;
	edges = (PointSet*)(other->edges->copy());
	edgeTable = other->edgeTable;
	aatVIndex = other->aatVIndex;	
	tmpBufferSize = other->tmpBufferSize;
	delete[] tmpBuffer;
	delete[] tmpBuffer2;
	tmpBuffer = new char[tmpBufferSize];
	tmpBuffer2 = new char[tmpBufferSize];
}

void InCorePCTopologyGraph::write(OutputObjectStream *s) const {
	s->writeObject(edges);
	mpcard numEdges = edgeTable.size();
	s->writeMaxPlatformCard(numEdges);
	for (mpcard i=0; i<numEdges; i++) {
		mpcard numNeighbors = edgeTable[i].size();
		s->writeMaxPlatformCard(numNeighbors);
		for (mpcard j=0; j<numNeighbors; j++) {
			s->write(edgeTable[i][j]);
		}
	}
}

void InCorePCTopologyGraph::read(InputObjectStream *s) {
	//if (s->getClassVersion(getClass()) >= MetaClass::FIRST_VERSION + 1) 
	{
		s->readAnyObject(edges);
		mpcard numEdges;
		if (edges != NULL) {
			aatVIndex = edges->getDescr()->getToken("vindex");	
			tmpBufferSize = edges->getDescr()->getSize();
			delete[] tmpBuffer;
			delete[] tmpBuffer2;
			tmpBuffer = new char[tmpBufferSize];
			tmpBuffer2 = new char[tmpBufferSize];
		}
		s->readMaxPlatformCard(numEdges);
		//edgeTable.resize(numEdges);	// resize allocates memory forever ...
		edgeTable = std::vector< std::vector<EdgeIndex> >(numEdges);
		for (mpcard i=0; i<numEdges; i++) {
			mpcard numNeighbors;
			s->readMaxPlatformCard(numNeighbors);
			//edgeTable[i].resize(numNeighbors);
			edgeTable[i] = std::vector<EdgeIndex>(numNeighbors);
			for (mpcard j=0; j<numNeighbors; j++) {
				s->read(edgeTable[i][j]);
			}
		}
	}
}


void InCorePCTopologyGraph::clearAndSetup(const VertexDescriptor *vd) {
	if (!vd->providesAttribute("vindex", 2, VAD::DATA_FORMAT_INT32)) {
		throw X4Exception("InCorePCTopologyGraph::clearAndSetup - VertexDescriptor must contain attribute int32 vindex[2].");
	}
	aatVIndex = vd->getToken("vindex");
	edges->clearAndSetup(1,0,vd);
	edgeTable.clear();
	tmpBufferSize = vd->getSize();
	delete[] tmpBuffer;
	delete[] tmpBuffer2;
	tmpBuffer = new char[tmpBufferSize];
	tmpBuffer2 = new char[tmpBufferSize];
}

void InCorePCTopologyGraph::addEdge(VertexIndex v1, VertexIndex v2) 
{
	memset(tmpBuffer, 0, tmpBufferSize);
	Vector2i indices = makeVector2i(v1, v2);
	aatVIndex.setAttrib<int32, 2>(tmpBuffer, indices);
	addEdge(tmpBuffer);
}

/**
 * Remove connection 'index' from the vertex in the graph
 */
void InCorePCTopologyGraph::deleteEdge( VertexIndex v, const card32 index )
{
	if ( ( v >= edgeTable.size() ) || ( index >= edgeTable[v].size() ) )
	{
		error(__FUNCTION__ ": invalid parameters.");
	}else
	{
		deleteEdge(edgeTable[v][index]);
		//edgeTable[v].erase( edgeTable[v].begin() + index );
	}
}

void InCorePCTopologyGraph::deleteEdge(EdgeIndex edge)
{
	Vector2i vindex = edges->get<int32, 2>(edge, aatVIndex);

	// delete edge from the point set 
	x4Assert(edges->getWidth() == 1);
	x4Assert(edges->getNumEntries() > 0);
	x4Assert(edge < edges->getNumEntries());

	bool indexChanged = false;
	if (edge < edges->getHeight()-1)
	{
		edges->setVertex(edge, edges->getDataPointer(edges->getHeight()-1));
		indexChanged = true;
	}
	card32 oldIndex = edges->getHeight() - 1;
	edges->changeHeight(edges->getHeight()-1);

	// go through all vertices and change edge indices correspondingly
	for (size_t v=0; v < edgeTable.size(); v++)
	{
		for (std::vector<EdgeIndex>::iterator it = edgeTable[v].begin(); it != edgeTable[v].end(); )
		{
			if (*it == edge)
			{
				it = edgeTable[v].erase(it);
				continue;
			}else if (*it == oldIndex)
			{
				*it = edge;
			}

			it++;
		}
	}
}

void InCorePCTopologyGraph::deleteEdge(const std::vector<EdgeIndex>& edges)
{
	if (edges.size() == 0) return;

	std::vector<EdgeIndex> edg = edges;
	std::sort(edg.begin(), edg.end());

	// remove equal elements
	EdgeIndex currentIndex = 0xFFFFFFFF;
	for (std::vector<EdgeIndex>::iterator it = edg.begin(); it != edg.end(); )
	{
		if (*it != currentIndex)
		{
			currentIndex = *it;
			it++;
		}
		else it = edg.erase(it);
	}

	for (std::vector<EdgeIndex>::const_reverse_iterator it = edg.rbegin(); it != edg.rend(); it++)
		deleteEdge(*it);
}

void InCorePCTopologyGraph::deleteVertex( const std::vector<VertexIndex>& vert )
{
#if 0
	std::vector<VertexIndex> points = vert;
	std::sort(points.begin(), points.end());

	// vertex indices
	std::vector<int> newIndices(edgeTable.size());
	for (size_t v=0; v < edgeTable.size(); v++) newIndices[v] = (int)v;

	std::vector<int> lastVertices(edgeTable.size(), -1);
	int lastValidVIndex = (int)edgeTable.size() - 1;
	size_t newVSize = edgeTable.size() - vert.size();
	for (std::vector<VertexIndex>::const_reverse_iterator it = points.rbegin(); it != points.rend(); it++)
	{
		if (*it < newVSize)
		{
			for (; (size_t)lastValidVIndex >= newVSize && newIndices[lastValidVIndex] == -1; lastValidVIndex--){}
			newIndices[*it] = lastValidVIndex;
			lastVertices[lastValidVIndex] = *it;
			newIndices[lastValidVIndex] = -1;
		}else
		{
			x4Assert(newIndices[*it] >= 0);
			newIndices[*it] = -1;
		}
	}

	// collect all edges which has to be deleted (mark those to delete and then collect them)
	std::vector<int> newEdgeIndices(edges->getHeight(), 0);
	for (std::vector<VertexIndex>::const_reverse_iterator it = points.rbegin(); it != points.rend(); it++)
	{
		for (size_t e=0; e < edgeTable[*it].size(); e++)
		{
			newEdgeIndices[edgeTable[*it][e]] = -1;
			//edgesToDelete.push_back(edgeTable[*it][e]);
		}
	}
	std::vector<EdgeIndex> edgesToDelete;
	for (size_t e=0; e < newEdgeIndices.size(); e++)
		if (newEdgeIndices[e] < 0) edgesToDelete.push_back((EdgeIndex)e); 
	std::sort(edgesToDelete.begin(), edgesToDelete.end());


	// change vertices tables
	for (size_t v=0; v < newVSize; v++)
	{
		x4Assert(newIndices[v] >= 0);
		if (v == newIndices[v]) continue;
		edgeTable[v] = edgeTable[newIndices[v]];
	}
	edgeTable.resize(newVSize);


	// resetup edge indices
	for (size_t e=0; e < newEdgeIndices.size(); e++) newEdgeIndices[e] = (int)e;

	std::vector<int> lastEdges(edges->getHeight(), -1);
	int lastValidEIndex = edges->getHeight() - 1;
	size_t newESize = edges->getHeight() - edgesToDelete.size();
	for (std::vector<EdgeIndex>::const_reverse_iterator it = edgesToDelete.rbegin(); it != edgesToDelete.rend(); it++)
	{
		if (*it < newESize)
		{
			for (; (size_t)lastValidEIndex >= newESize && newEdgeIndices[lastValidEIndex] == -1; lastValidEIndex--){}
			newEdgeIndices[*it] = lastValidEIndex;
			lastEdges[lastValidEIndex] = *it;
			newEdgeIndices[lastValidEIndex] = -1;
		}else
		{
			x4Assert(newEdgeIndices[*it] >= 0);
			newEdgeIndices[*it] = -1;
		}
	}

	// resize edges points set
	for (unsigned e=0; e < newESize; e++)
	{
		x4Assert(newEdgeIndices[e] >= 0);
		if (e == newEdgeIndices[e]) continue;
		edges->setVertex(e, edges->getDataPointer(newEdgeIndices[e]));	
	}
	edges->changeHeight(newESize);

	// change vertices table
	for (size_t v=0; v < edgeTable.size(); v++)
	{
		for (size_t e=0; e < edgeTable[v].size(); e++)
		{
			if (lastEdges[edgeTable[v][e]] >= 0)
				edgeTable[v][e] = lastEdges[edgeTable[v][e]];
		}
	}

	// change edges table
	for (unsigned e=0; e < edges->getHeight(); e++)
	{
		Vector2i index = edges->get2i(e, aatVIndex);
		if (lastVertices[index[0]] >= 0) index[0] = lastVertices[index[0]];
		if (lastVertices[index[1]] >= 0) index[1] = lastVertices[index[1]];
		edges->set2i(e, aatVIndex, index);

		x4Assert(index[0] >= 0 && index[0] < newVSize);
		x4Assert(index[1] >= 0 && index[1] < newVSize);
	}

#endif
}

void InCorePCTopologyGraph::deleteVertex( VertexIndex v )
{
	if ( v >= edgeTable.size() )
	{
		error (__FUNCTION__ ": invalid vertex index.\n");
	} else
	{
		clearVertex(v);
		deleteEdge(edgeTable[v]);

		size_t oldIndex = edgeTable.size()-1;
		edgeTable[v] = edgeTable[oldIndex];
		edgeTable.resize(edgeTable.size() - 1);
	}
}

/**
 * Remove all connection for a vertex in the graph
 */
void InCorePCTopologyGraph::clearVertex( VertexIndex v )
{
	if ( v >= edgeTable.size() )
	{
		error (__FUNCTION__ ": invalid vertex index.\n");
	} else
	{
		deleteEdge(edgeTable[v]);
		edgeTable[v].clear();
	}
}

void InCorePCTopologyGraph::addEdge(const char *data) {
   Vector2i indices = aatVIndex.getAttrib<int32,2>(data);
   //Vector2i indices = *((Vector2i*)data+aatVIndex.getOffset());

   //if ((int32)edgeTable.size() <= indices[0]) edgeTable.resize(indices[0]+1);
   //if ((int32)edgeTable.size() <= indices[1]) edgeTable.resize(indices[1]+1);
	unsigned largestIndex = max(indices[0],indices[1]);
	while( edgeTable.size() <= largestIndex )
		edgeTable.push_back( std::vector<EdgeIndex>() );

	VertexIndex n = (VertexIndex)(edgeTable[indices[0]].size());

	for (VertexIndex i=0; i<n; i++) {
		//edges->getVertex(edgeTable[indices[0]][i], tmpBuffer2);
      //Vector2i edgeIndices = aatVIndex.getAttrib<int32,2>(tmpBuffer2);	
		const char *ptr = edges->getDataPointer(edgeTable[indices[0]][i]);
		Vector2i edgeIndices = *((Vector2i*)ptr+aatVIndex.getOffset());	
		if (   (edgeIndices[0] == indices[0] && edgeIndices[1] == indices[1])
		    || (edgeIndices[1] == indices[0] && edgeIndices[0] == indices[1])) {
			// first case: edge already present - update data
			edges->setVertex(edgeTable[indices[0]][i], data);
			return;
		}
	}
	
	// second case: edge not found - create new one
	edges->addPoint(data);
	EdgeIndex e = edges->getNumEntries() - 1;
	edgeTable[indices[0]].push_back(e);
	edgeTable[indices[1]].push_back(e);
	return;
}

/** 
* Create copy of topology graph
*/
InCorePCTopologyGraph* InCorePCTopologyGraph::copy() const
{
	InCorePCTopologyGraph* tg = new InCorePCTopologyGraph();
	//delete tg->edges;
	tg->edges = (PointSet*)(edges->copy());
	tg->edgeTable = edgeTable;
	tg->aatVIndex = aatVIndex;

	// Added by pj
	tg->tmpBufferSize = tmpBufferSize;
	tg->tmpBuffer = new char[tmpBufferSize];
	tg->tmpBuffer2 = new char[tmpBufferSize];
	
	return tg;
}



std::vector<InCorePCTopologyGraph::EdgeIndex> InCorePCTopologyGraph::getEdgeIndices(VertexIndex v) const {
	if ( v >= edgeTable.size() ) {
		return std::vector<EdgeIndex>();
	} else {
		return edgeTable[v];
	}
}

InCorePCTopologyGraph::EdgeIndex InCorePCTopologyGraph::getEdgeIndex(VertexIndex v, VertexIndex i) const {
	x4Assert(edgeTable.size() > v);
	x4Assert(edgeTable[v].size() > i);
	return edgeTable[v][i];
}

InCorePCTopologyGraph::EdgeIndex InCorePCTopologyGraph::getNumEdges() const {
	return edges->getNumEntries();
}


InCorePCTopologyGraph::VertexIndex InCorePCTopologyGraph::getNumAdjacentVertices(VertexIndex v) const {
	if (v >= edgeTable.size()) {
		return 0;
	} else {
		return (VertexIndex)(edgeTable[v].size());
	}
}

InCorePCTopologyGraph::VertexIndex InCorePCTopologyGraph::getAdjacentVertex(VertexIndex v, VertexIndex i, EdgeIndex* edgeIndex) const {
	x4Assert(v < edgeTable.size());
	x4Assert(i < edgeTable[v].size());
	EdgeIndex e = edgeTable[v][i];
	if (edgeIndex != NULL) *edgeIndex = e;
	Vector2i vindex = edges->get<int32, 2>(e, aatVIndex);
	if (vindex[0] == v) return vindex[1]; else return vindex[0];
}

	
PointCloudIterator *InCorePCTopologyGraph::createIterator(PointSet *ps) {
   InCorePCTopologyGraphPointSetIterator *result = new InCorePCTopologyGraphPointSetIterator(
      ps, NULL, this );

   return result;
}

void InCorePCTopologyGraph::appendFields(const VertexDescriptor *newAttribs) {
	PointSet *newPs = edges->appendFields(newAttribs);
	if (newPs != NULL) {
		delete edges;
		edges = newPs;
      aatVIndex = edges->getDescr()->getToken("vindex");	
	}
}



InCorePCTopologyGraph::~InCorePCTopologyGraph() {
   delete edges;
   delete[] tmpBuffer;
   delete[] tmpBuffer2;
}


// --



InCorePCTopologyGraphPointSetIterator::InCorePCTopologyGraphPointSetIterator(PointSet *ps, AttachedDataContainer *attData, 
	InCorePCTopologyGraph * graph)
	: AttachedDataPointSetIterator(ps, attData)
{
	m_Graph = graph;
}

InCorePCTopologyGraphPointSetIterator::~InCorePCTopologyGraphPointSetIterator()
{
}

card32 InCorePCTopologyGraphPointSetIterator::getNumNeighbours()
{
	return m_Graph->getNumAdjacentVertices( getCurrentPointIndex() );
}

/// returns an index of a neighbour point
PointIndex InCorePCTopologyGraphPointSetIterator::getNeighbour( card32 nIndex )
{
	return m_Graph->getAdjacentVertex( getCurrentPointIndex(), nIndex );
}
