//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "IteratorAdaptors.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "PointSetIterator.h"
//#include "PointCloudEditTools.h"
#include "DebugOutput.h"
#include "LinearAlgebra.inline.h"
#include "X4Exception.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


///Abstract IteratorAdaptor Base Class
//==================================================================================

IteratorAdaptor::IteratorAdaptor
        ( PointCloudIterator* native, const VertexDescriptor *vd)
{
   m_base=dynamic_cast<BasicPointCloudIterator*>(native);
   x4Assert(  m_base!=NULL);
   m_vd=vd;
   locked=false;
   next_block=false;

   m_mod_base=dynamic_cast<ModifyablePointCloudIterator*>(native);
   m_indexed = dynamic_cast<IndexedPointCloudIterator*>(native);
   m_native=native;
   m_native_remember_write=NULL;

   write=false;
}

IteratorAdaptor::~IteratorAdaptor(){
//   debugOutput << "IteratorAdaptor: Destruktor" ;
//   debugOutput.cr();

// TODO
// delete native; //???

}


bool IteratorAdaptor::atEnd() {
   while (!m_base->atEnd() && !pointMeetsCond() ){
      m_base->next();
   };
  return m_base->atEnd();
}

/// go to first point
void IteratorAdaptor::reset(){
   m_base->reset();

   while (!m_base->atEnd() && !pointMeetsCond() ){
      m_base->next();
   };
}

/// go to next point
void IteratorAdaptor::next() {
   m_base->next();
  /*
   do {
        m_base->next();
      } while (!m_base->atEnd() && !pointMeetsCond() );
      */
}

card64 IteratorAdaptor::getNumPointsInBlock() const {
	if (m_indexed) {
		return m_indexed->getNumPointsInBlock();
	} else {
		throw X4Exception("IteratorAdaptor::getNumPointsInBlock() - base iterator does not support indexing.");
	}
}

card64 IteratorAdaptor::getCurrentPointIndex() const {
	if (m_indexed) {
		return m_indexed->getCurrentPointIndex();
	} else {
		throw X4Exception("IteratorAdaptor::getCurrentPointIndex() - base iterator does not support indexing.");
	}
}


/// get attributes in different formats
card8     IteratorAdaptor::get1ub(const AAT &token) const{return m_base->get1ub(token);}
Vector2ub IteratorAdaptor::get2ub(const AAT &token) const{return m_base->get2ub(token);  }
Vector3ub IteratorAdaptor::get3ub(const AAT &token) const{return m_base->get3ub(token);  }
Vector4ub IteratorAdaptor::get4ub(const AAT &token) const{return m_base->get4ub(token);  }

int32    IteratorAdaptor::get1i(const AAT &token) const{return m_base->get1i(token);  }
Vector2i IteratorAdaptor::get2i(const AAT &token) const{return m_base->get2i(token);  }
Vector3i IteratorAdaptor::get3i(const AAT &token) const{return m_base->get3i(token);  }
Vector4i IteratorAdaptor::get4i(const AAT &token) const{return m_base->get4i(token);  }

card32   IteratorAdaptor::get1u(const AAT &token) const{return m_base->get1u(token);  }
Vector2u IteratorAdaptor::get2u(const AAT &token) const{return m_base->get2u(token);  }
Vector3u IteratorAdaptor::get3u(const AAT &token) const{return m_base->get3u(token);  }
Vector4u IteratorAdaptor::get4u(const AAT &token) const{return m_base->get4u(token);  }

float32  IteratorAdaptor::get1f(const AAT &token) const{return m_base->get1f(token);  }
Vector2f IteratorAdaptor::get2f(const AAT &token) const{return m_base->get2f(token);  }
Vector3f IteratorAdaptor::get3f(const AAT &token) const{return m_base->get3f(token);  }
Vector4f IteratorAdaptor::get4f(const AAT &token) const{return m_base->get4f(token);  }

/// ask VertexDescriptor for buffer size
void     IteratorAdaptor::getVertex(char *buffer) const{
        m_base->getVertex(buffer);
}



// --- blockwise access (faster)
//==================================================================

/// step foreward to next data block, false if end-of-set (affects the same position pointer as "next()")
bool     IteratorAdaptor::nextBlock(){
        return next_block;
  }

  /// access raw data of current data block{  } call unlock to free{  }
/// one data block is guaranteed to easily fit into main memory

const PointSet*  IteratorAdaptor::lockBlockRead(){
   x4Assert(!locked); // first unlock() last block
   locked=true;



   tmp_ps= new PointSet();// ( w, h        , descr)
   tmp_ps->clearAndSetup     ( 1, maxBlockNumPoints, m_vd);

   PointSetIterator *tmpIt=new PointSetIterator(tmp_ps);

   char *buffer=new char[m_vd->getSize()];
   int numEntries=0;


   //goto first _selected_ Point
   reset();


   if (!m_base->atEnd()){
      do{
          m_base->getVertex(buffer);
          tmpIt->setVertex(buffer);
          numEntries++;

          this->next();
          tmpIt->next();
      }while(!m_base->atEnd() && (numEntries< maxBlockNumPoints));
   }

   delete tmpIt;
   delete[] buffer;

   //Resize if smaller than maxBlockNumPoints
   if (numEntries< maxBlockNumPoints)
   {
       // YYY tmp_ps->resize(1, numEntries);
       tmp_ps->changeHeight(numEntries);
       next_block=false;
   }else{    //we need more than one block
     next_block=true;
   }

   return tmp_ps;

 }



/// free locked PointSet again
void IteratorAdaptor::unlock(){
   if (write){
      PointSetIterator *fromIt=new PointSetIterator(tmp_ps);   //changed PointSet
      fromIt->reset();
      char *buffer=new char[m_vd->getSize()];

      BasicPointCloudIterator* m_base_remember_read = dynamic_cast<BasicPointCloudIterator*>(m_native_remember_write);
      ModifyablePointCloudIterator* m_base_remember_write= dynamic_cast<ModifyablePointCloudIterator*>(m_native_remember_write);

      if (!fromIt->atEnd()){
         do{
             // write Vertex from changed_PointSet to m_base
             fromIt->getVertex(buffer);
             m_base_remember_write->setVertex(buffer);
             //go to next point in both sets
             fromIt->next();
             do { //goto next selected point
               m_base_remember_read->next();            //change only selected Points
             } while (!m_base_remember_read->atEnd() && !pointMeetsCond())  ;
         }while(!m_base_remember_read->atEnd() && !fromIt->atEnd());
      }

      delete fromIt;
      delete[] buffer;

      write=false;
   }

   delete tmp_ps;
   tmp_ps=NULL;
   locked=false;


}

/// access raw data of current data block for read-write access
/// call unlock to free locked PointSet
PointSet* IteratorAdaptor::lockBlockReadWrite(LockRWHint hint){

   write=true;

   x4Assert(!locked); // first unlock() last block
   locked=true;
   tmp_ps= new PointSet();// ( w, h        , descr)
   tmp_ps->clearAndSetup     ( 1, maxBlockNumPoints, m_vd);

   PointSetIterator *tmpIt=new PointSetIterator(tmp_ps);

   char *buffer=new char[m_vd->getSize()];
   int numEntries=0;

   //goto first _selected_ Point
   reset();

   m_native_remember_write = (PointCloudIterator*) m_native->copy();
   // remember Position, for write back

   if (!m_base->atEnd()){
      do{
          m_base->getVertex(buffer);
          tmpIt->setVertex(buffer);
          numEntries++;
          
          tmpIt->next();
          this->next();
      }while(!m_base->atEnd() && (numEntries< maxBlockNumPoints));
   }

   delete tmpIt;
   delete[] buffer;

   //Resize if smaller than maxBlockNumPoints
   if (numEntries< maxBlockNumPoints)
   {
       // YYY tmp_ps->resize(1, numEntries);
       tmp_ps->changeHeight(numEntries);
       next_block=false;
   }else{    //we need more than one block
     next_block=true;
   }

   return tmp_ps;


}

/// get attributes in different formats
void IteratorAdaptor::set1ub(const AAT &token, const card8 &x)     const{m_mod_base->set1ub(token,x);  }
void IteratorAdaptor::set2ub(const AAT &token, const Vector2ub &v) const{m_mod_base->set2ub(token,v);  }
void IteratorAdaptor::set3ub(const AAT &token, const Vector3ub &v) const{m_mod_base->set3ub(token,v);  }
void IteratorAdaptor::set4ub(const AAT &token, const Vector4ub &v) const{m_mod_base->set4ub(token,v);  }

void IteratorAdaptor::set1i(const AAT &token, const int32 &x)    const{m_mod_base->set1i(token,x);  }
void IteratorAdaptor::set2i(const AAT &token, const Vector2i &v) const{m_mod_base->set2i(token,v);  }
void IteratorAdaptor::set3i(const AAT &token, const Vector3i &v) const{m_mod_base->set3i(token,v);  }
void IteratorAdaptor::set4i(const AAT &token, const Vector4i &v) const{m_mod_base->set4i(token,v);  }

void IteratorAdaptor::set1u(const AAT &token, const card32 &x)    const{m_mod_base->set1u(token,x);  }
void IteratorAdaptor::set2u(const AAT &token, const Vector2u &v) const{m_mod_base->set2u(token,v);  }
void IteratorAdaptor::set3u(const AAT &token, const Vector3u &v) const{m_mod_base->set3u(token,v);  }
void IteratorAdaptor::set4u(const AAT &token, const Vector4u &v) const{m_mod_base->set4u(token,v);  }

void IteratorAdaptor::set1f(const AAT &token, const float32 &x)  const{m_mod_base->set1f(token,x);  }
void IteratorAdaptor::set2f(const AAT &token, const Vector2f &v) const{m_mod_base->set2f(token,v);  }
void IteratorAdaptor::set3f(const AAT &token, const Vector3f &v) const{m_mod_base->set3f(token,v);  }
void IteratorAdaptor::set4f(const AAT &token, const Vector4f &v) const{m_mod_base->set4f(token,v);  }

/// ask VertexDescriptor for size
void IteratorAdaptor::setVertex(const char *data) const{m_mod_base->setVertex(data);  }


//====================================================================================



NotModGeometricRangeIteratorAdaptor::NotModGeometricRangeIteratorAdaptor
        ( PointCloudIterator* native, const VertexDescriptor *vd,GeometricRange *r)
        :IteratorAdaptor(native,vd)
{
   m_range=r;
   POSITION = m_vd->getToken("position");
   FLAGS    = m_vd->getToken("flags");
   //goto first Point in Range
   reset();

}

NotModGeometricRangeIteratorAdaptor::~NotModGeometricRangeIteratorAdaptor(){
}


//===================================================
NormalIteratorAdaptor::NormalIteratorAdaptor
        ( PointCloudIterator* native, const VertexDescriptor *vd,Vector3f n)
        :IteratorAdaptor(native,vd)
{
   myNormal=n;
   NORMAL  = m_vd->getToken("normal");

   //goto first Point in Range
   reset();

}

//===================================================
AllPointsIteratorAdaptor::AllPointsIteratorAdaptor
        ( PointCloudIterator* native, const VertexDescriptor *vd)
        :IteratorAdaptor(native,vd)
{
   //goto first Point in Range
   reset();

}














