//---------------------------------------------------------------------------
#include "StdAfx.h"
#include "Manifold.h"
#include "SeparatorClassProperty.h"
#include "VariableArrayClassProperty.h"
#include "EmbeddedObjectClassProperty.h"
#include "CopyObjectProperties.h"

//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS(ManifoldPoint, 0)
{
	BEGIN_CLASS_INIT(ManifoldPoint);

	//ADD_DVECTORF_PROP (m_Coordinate, 0);
}

//---------------------------------------------------------------------------
ManifoldPoint::ManifoldPoint()
{

}
//---------------------------------------------------------------------------
ManifoldPoint::~ManifoldPoint()
{
}

//---------------------------------------------------------------------------
ManifoldPoint::ManifoldPoint(const ManifoldPoint& point)
{
	copyObjectProperties(&point, this);
	m_Coordinate = point.m_Coordinate;
	m_Normal = point.m_Normal;
}

//---------------------------------------------------------------------------
ManifoldPoint::ManifoldPoint(const ManifoldPoint* point)
{
	copyObjectProperties(point, this);
	m_Coordinate = point->m_Coordinate;
	m_Normal = point->m_Normal;
}

//---------------------------------------------------------------------------
void ManifoldPoint::assign(const Object* obj, X4CopyContext *context)
{
	const ManifoldPoint* p = dynamic_cast<const ManifoldPoint*>(obj);
	copyObjectProperties(p, this);
	m_Coordinate = p->m_Coordinate;
	m_Normal = p->m_Normal;
}

//---------------------------------------------------------------------------
void ManifoldPoint::write(OutputObjectStream *s) const
{
	Persistent::write(s);
	m_Coordinate.write(s);
	m_Normal.write(s);
}

//---------------------------------------------------------------------------
void ManifoldPoint::read(InputObjectStream *s)
{
	Persistent::read(s);
	m_Coordinate.read(s);
	m_Normal.read(s);
}

//---------------------------------------------------------------------------
IMPLEMENT_X4_CLASS(ManifoldPointList, 0)
{
	BEGIN_CLASS_INIT(ManifoldPointList);

	ADD_OBJECT_LIST_PROP(ManifoldPoint, 0, ManifoldPoint::getClass());
}

//---------------------------------------------------------------------------
ManifoldPointList::ManifoldPointList()
{

}
//---------------------------------------------------------------------------
ManifoldPointList::~ManifoldPointList()
{
	deleteManifoldPoints(0, (card32)m_ManifoldPoint.size());
}

//---------------------------------------------------------------------------
void ManifoldPointList::addManifoldPointList(ManifoldPointList* list)
{
	if (list == NULL) return;
	for (unsigned i=0; i < list->getNumManifoldPoints(); i++)
		addManifoldPoint(list->getManifoldPoint(i)->copy());
}

//---------------------------------------------------------------------------
IMPLEMENT_X4_ABSTRACT_CLASS(Manifold, 0)
{
	BEGIN_CLASS_INIT(Manifold);

	ADD_SEPARATOR("Manifold");
	ADD_CARD32_PROP(m_InterpolationAlgorithm, 0);
	ADD_BOOLEAN_PROP(m_bDirtyGeodesics, 0);
}

//---------------------------------------------------------------------------
Manifold::Manifold()
{
	m_bDirtyGeodesics = true;
	m_InterpolationAlgorithm = NEAREST;
}

//---------------------------------------------------------------------------
Manifold::~Manifold()
{
}

//---------------------------------------------------------------------------
SceneObjectIterator *Manifold::createIterator(SceneObjectIteratorType type)
{
	if (!supportsIteratorType(type)) {
		return NULL;
	} else {
		return new SceneObjectIterator();
	}
}

//---------------------------------------------------------------------------
bool Manifold::supportsIteratorType(SceneObjectIteratorType cap)
{
	SOIT possibleCaps(  SOIT::CAP_MANIFOLD ) ;
	if( !possibleCaps.providesCapsFor(cap))
		return false;
	else
		return true;
}
