//---------------------------------------------------------------------------
#ifndef MultiScaleInCorePointCloudH
#define MultiScaleInCorePointCloudH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "MultiScalePointCloud.h"
#include "PointCloudIterators.h"
#include "ObjectListProperty.h"
#include "PointSetIterator.h"
#include "ObjectListProperty.h"
//---------------------------------------------------------------------------

namespace X4 {

class PointCloudSmoother;


class X3GEOMETRY_API MultiScaleInCorePointCloud : public MultiScalePointCloud {
   X4_CLASS(MultiScaleInCorePointCloud)

 private:
   VertexDescriptor *vertDescr;
   MultiScalePCParams *params;
   std::vector<PointSet*> levels;
   std::vector<AttachedDataContainer*> hierarchyAttachments;
   
   IMPLEMENT_OBJECT_LIST_METHODS_STL_NO_PREFIX_M( levels, PointSet )
   IMPLEMENT_OBJECT_LIST_METHODS_STL_NO_PREFIX_M( hierarchyAttachments, AttachedDataContainer )
   
 public:
 
   MultiScaleInCorePointCloud();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   // --- new methods
   virtual void clearAndSetup(const VertexDescriptor *vertDescr);

   virtual void addPoint(const card8 *pt);
   virtual void addPointSet(const PointSet *ps);

   virtual void build(const MultiScalePCParams *params, const PointCloudSmoother *interp);
   void build(const MultiScalePCParams *params, std::deque<PointSet*> psvec);
   void build(const MultiScalePCParams *params);

   virtual const MultiScalePCParams *getMultiScaleParameters() const;
   
   virtual UnstructuredInCorePointCloud *createExcerptDiscrScale(const BoundingBox3f &region, card32 level);

   virtual UnstructuredInCorePointCloud *createExcerptContScale(
      const BoundingBox3f &region, 
      float32 gridSpacing, 
      const PointCloudSmoother *interp);

	card32 getNumLevels() {return (card32)levels.size();}
	const PointSet *getLevel(card32 level) const;
	PointSet *getLevel(card32 level);

   bool isBuilt() const { return levels.size() > 1; }
   bool isBeingSetup() const { return levels.size() == 1; }
   // ---
   
   virtual const VertexDescriptor *getDescr() const {return vertDescr;}
   virtual PointCloud *appendFields(const VertexDescriptor *newAttribs) const;
   virtual card64 getNumPoints() const;
   virtual void clear();

   virtual SceneObjectIterator *createIterator(SceneObjectIteratorType type);
   virtual bool supportsIteratorType(SceneObjectIteratorType cap);

   void attachData(AttachedData* att, unsigned level);
   AttachedData* getAttachedData(unsigned level, std::string name);
   virtual ~MultiScaleInCorePointCloud() { clear(); }
};


class X3GEOMETRY_API MultiScaleInCorePointCloudIterator : public AttachedDataPointSetIterator /*, public MultiScalePCIterator*/ {
      X4_CLASS( MultiScaleInCorePointCloudIterator )
 private:
   std::vector<PointSet*> *levels;
   std::vector<AttachedDataContainer*> *attData;
   MultiScalePCParams *params;
   card32 currentLevel;
   bool lockedReadWrite;

   MultiScaleInCorePointCloudIterator() {levels = NULL; attData = NULL; currentLevel = 0; lockedReadWrite = false;}

 public:
   MultiScaleInCorePointCloudIterator(std::vector<PointSet*> *levels, std::vector<AttachedDataContainer*> *attData, MultiScalePCParams *params);

   /// SecondaryDataStructureIterator
   virtual AttachedDataContainer *lockAttachments();
   virtual void unlockAttachments();

   /// HierarchicalIterator
   virtual card32 getNumChildren();
   virtual bool goToChild(card32 index);
   virtual card32 getCurrentLevel(void);
   virtual bool goToParent();
   virtual BoundingBox3f getBoundingBox();
   virtual bool isLeaf() const;
   virtual bool getNodeInfo(NodeInfoType whichInfo, const void *parameter, void *data);

   virtual ~MultiScaleInCorePointCloudIterator();
};


}

#endif
