//---------------------------------------------------------------------------
#ifndef MultiScalePointCloudH
#define MultiScalePointCloudH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "MultiScalePointCloudParameters.h"
#include "BoundingBox.h"
//---------------------------------------------------------------------------

namespace X4 {

class PointCloudSmoother;
class UnstructuredInCorePointCloud;


class X3GEOMETRY_API MultiScalePointCloud : public PointCloud {
   X4_ABSTRACT_CLASS(MultiScalePointCloud)

 public:
   // --- new methods
   virtual void clearAndSetup(const VertexDescriptor *vertDescr) = 0;

   /// add a single point during initial build phase (after clearAndSetup, before build)
   /// must match vertDescr
   virtual void addPoint(const card8 *pt) = 0;
   /// add a set of points (after clearAndSetup, before build)
   /// must exactly match vertDescr
   virtual void addPointSet(const PointSet *ps) = 0;

   /// build data structure for use
   virtual void build(const MultiScalePCParams *params, const PointCloudSmoother *interp) = 0;

   /// get the parameters; call only after build
   virtual const MultiScalePCParams *getMultiScaleParameters() const = 0;
   
   /// access the data. Will create new point cloud (you have to delete it after using)
   virtual UnstructuredInCorePointCloud *createExcerptDiscrScale(const BoundingBox3f &region, card32 level) = 0;

   /// access the data. Will create new point cloud (you have to delete it after using)
   /// the float parameter corresponds to a factional level; will be interpolated
   virtual UnstructuredInCorePointCloud *createExcerptContScale(
      const BoundingBox3f &region, 
      float32 gridSpacing, 
      const PointCloudSmoother *interp) = 0;

};



}

#endif
