//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCNearestNeighborIterator.h"
//---------------------------------------------------------------------------
#include "VertexDescriptor.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "DebugOutput.h"
#include "LinearAlgebra.inline.h"
#include "BoundingBox.inline.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

/*

MetaClass* PCNearestNeighborIterator::PCNearestNeighborIteratorMetaClass = NULL;

const MetaClass* PCNearestNeighborIterator::getInstanceClass() const {
   return getClass();
}

Object* PCNearestNeighborIterator::newInstance() {
   return new PCNearestNeighborIterator;
}
const MetaClass* PCNearestNeighborIterator::getClass() {
   if (PCNearestNeighborIteratorMetaClass==NULL) {
      PCNearestNeighborIteratorMetaClass = new MetaClass("PCNearestNeighborIterator", "no description.",
                                  newInstance, sizeof(PCNearestNeighborIterator));
   }
   return PCNearestNeighborIteratorMetaClass;
}

void PCNearestNeighborIterator::init(const MetaClass *parentClass, card32 uniqueIdentifier) {
   ((MetaClass*)getClass())->init(uniqueIdentifier, parentClass);
   PCNearestNeighborIteratorMetaClass->copyParentProperties();
}

void PCNearestNeighborIterator::shutdown() {
   delete PCNearestNeighborIteratorMetaClass;
}


// ----
PCNearestNeighborIterator::PCNearestNeighborIterator():
  PointCloudIterator(),
  BasicPointCloudIterator(),
  m_pc(NULL),
  m_ps(NULL),
  m_startHIt(NULL),
  m_currentHIt(NULL),
  m_buffer(NULL)
{
}

PCNearestNeighborIterator::~PCNearestNeighborIterator(){
   reset();

   while(!m_pqLeafs.empty()){
      delete m_pqLeafs.top().hit;
      m_pqLeafs.pop();
   }

   if(m_currentHIt) delete dynamic_cast<SceneObjectIterator*>(m_currentHIt);
   if(m_startHIt) delete dynamic_cast<SceneObjectIterator*>(m_startHIt);
   if(m_ps) delete m_ps;
   delete[] m_buffer;
}

void PCNearestNeighborIterator::nextBox(void) {
   bool resume;
   do{
   while((m_pqLeafs.empty() && !m_pqHierarchy.empty()) ||
    (!m_pqLeafs.empty() && !m_pqHierarchy.empty() && m_pqHierarchy.top().quaddist<=m_pqLeafs.top().quaddist)){

      PQHierarchyNode node=m_pqHierarchy.top();
      m_pqHierarchy.pop();

//      HierarchicalIterator* h=node.hit;
      AIHierarchicalOctreeIterator* h=node.hit;
      bool isLeaf=true;
      for(card32 i=0; i<h->getNumChildren(); i++){
         PQHierarchyNode hn;
//         hn.hit=dynamic_cast<HierarchicalIterator*>(dynamic_cast<SceneObjectIterator*>(h)->copy());
         hn.hit=dynamic_cast<AIHierarchicalOctreeIterator*>(dynamic_cast<SceneObjectIterator*>(h)->copy());

         if(hn.hit->goToChild(i)){
            isLeaf=false;

            BoundingBox3f bb=hn.hit->getBoundingBox();
            if(bb.inBoundingBox(m_seekPoint)){
               hn.quaddist=-2.0f;
            }else{
               Sphere3f sp=bb.getBoundingSphere();
               hn.quaddist=(sp.center-m_seekPoint)*(sp.center-m_seekPoint);
               if(hn.quaddist<sp.radius*sp.radius){
                  hn.quaddist=-1.0f;
               }else{
                  hn.quaddist=sqrt(hn.quaddist);
                  hn.quaddist-=sp.radius;
                  hn.quaddist*=hn.quaddist;
               }
            }
            m_pqHierarchy.push(hn);
         }else{
            delete dynamic_cast<SceneObjectIterator*>(hn.hit);
         }
      }
      if(isLeaf){
         m_pqLeafs.push(node);
      }else{
         delete dynamic_cast<SceneObjectIterator*>(h);
      }
   }

   if(!m_pqLeafs.empty()){
      const VertexDescriptor* desc=m_pc->getDescr();
      const AAT POSITION = desc->getToken("position");
      NeighborPoint np;
      while(!m_pqLeafs.empty() && (m_neighbors.empty() || m_pqLeafs.top().quaddist<m_neighbors.top().quaddist)){
         // HierarchicalIterator* h=m_pqLeafs.top().hit;
         AIHierarchicalOctreeIterator* h=m_pqLeafs.top().hit;
         BasicPointCloudIterator* bit=dynamic_cast<BasicPointCloudIterator*>(h);

         bit->reset();
         while(!bit->atEnd()){
            Vector3f pos=bit->get3f(POSITION);
            np.quaddist=(pos-m_seekPoint)*(pos-m_seekPoint);

            np.index=m_lastIndex;
            bit->getVertex(m_buffer);
            m_ps->addPoint(m_buffer);
            m_neighbors.push(np);
            m_lastIndex++;
            bit->next();
         }
         delete dynamic_cast<SceneObjectIterator*>(h);
         m_pqLeafs.pop();
      }
   }

   resume=false;
   if(!m_neighbors.empty() && !m_pqHierarchy.empty() && m_pqHierarchy.top().quaddist<=m_neighbors.top().quaddist)
      resume=true;

   if(!m_pqLeafs.empty() && !m_pqHierarchy.empty() && m_pqHierarchy.top().quaddist<=m_pqLeafs.top().quaddist)
      resume=true;
   }while(resume);
}

void PCNearestNeighborIterator::findNeighbors(void) {
   const VertexDescriptor* vd=m_pc->getDescr();

   if(m_ps==NULL){
      m_ps=new PointSet();
   }
   m_ps->clearAndSetup(1,0,vd);

   SceneObjectIterator *it = m_pc->createIterator(SOIT(SOIT::CAP_HIERARCHICAL));
   //   HierarchicalIterator* hit=dynamic_cast<HierarchicalIterator*>(it);
   AIHierarchicalOctreeIterator* hit=new AIHierarchicalOctreeIterator();
   hit->clearAndSetup(m_pc, it, m_attachedOctreeMemManager);

   PQHierarchyNode hn;
   // hn.hit=dynamic_cast<HierarchicalIterator*>(dynamic_cast<SceneObjectIterator*>(hit)->copy());
   hn.hit=dynamic_cast<AIHierarchicalOctreeIterator*>(dynamic_cast<SceneObjectIterator*>(hit)->copy());

   hn.quaddist=0;
   m_pqHierarchy.push(hn);
   delete hit;
   m_lastIndex=0;
   nextBox();
}

bool PCNearestNeighborIterator::clearAndSetup(PointCloud* pc, const Vector3f& sp, float32, MemoryManager *attachedOctreeMemManager){
   m_attachedOctreeMemManager = attachedOctreeMemManager;
   m_seekPoint=sp;
   m_pc=pc;
   const VertexDescriptor* vd=dynamic_cast<const VertexDescriptor*>(m_pc->getDescr());
   delete[] m_buffer;
   m_buffer=new char[vd->getSize()];

   reset();
   findNeighbors();
   return(m_startHIt!=NULL);
}

float olddist=0.0;
int oldindex=0;

/// go to first point
void PCNearestNeighborIterator::reset(){
   while(!m_neighbors.empty())
      m_neighbors.pop();
   while(!m_pqLeafs.empty()){
      delete dynamic_cast<SceneObjectIterator*>(m_pqLeafs.top().hit);
      m_pqLeafs.pop();
   }
   while(!m_pqHierarchy.empty()){
      delete dynamic_cast<SceneObjectIterator*>(m_pqHierarchy.top().hit);
      m_pqHierarchy.pop();
   }


   if(m_startHIt) delete dynamic_cast<SceneObjectIterator*>(m_startHIt);
   if(m_currentHIt) delete dynamic_cast<SceneObjectIterator*>(m_currentHIt);
   SceneObjectIterator *it = m_pc->createIterator(SOIT(SOIT::CAP_HIERARCHICAL | SOIT::CAP_ATTACH_SECONDARY));
   HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(it);
   m_startHIt=hit;
   m_currentHIt=dynamic_cast<HierarchicalIterator*>(it->copy());

   olddist=0.0f;
   oldindex=0;
}


/// go to next point
void PCNearestNeighborIterator::next(){
// DEBUG
//   float a=m_neighbors.top().quaddist;

   if(m_neighbors.size()>0)
      m_neighbors.pop();

   nextBox();



}
/// true if end-of-set
bool PCNearestNeighborIterator::atEnd(){
   if(m_neighbors.size()==0){
      nextBox();
   }
   return(m_neighbors.size()==0);
}

card64 PCNearestNeighborIterator::getNumPointsInBlock() const {
	x4Assert(m_ps);
	return m_ps->getNumEntries();
}

card64 PCNearestNeighborIterator::getCurrentPointIndex() const {
	x4Assert(!m_neighbors.empty());
	return m_neighbors.top().index;
}


/// get attributes in different formats
card8     PCNearestNeighborIterator::get1ub(const AAT &token) const {
   return(m_ps->get<card8, 1>(0,m_neighbors.top().index, token)[0]);
}

Vector2ub PCNearestNeighborIterator::get2ub(const AAT &token) const {
   return(m_ps->get<card8, 2>(0, m_neighbors.top().index, token));
}

Vector3ub PCNearestNeighborIterator::get3ub(const AAT &token) const {
   return(m_ps->get<card8, 3>(0, m_neighbors.top().index, token));
}

Vector4ub PCNearestNeighborIterator::get4ub(const AAT &token) const {
   return(m_ps->get<card8, 4>(0, m_neighbors.top().index, token));
}

int32    PCNearestNeighborIterator::get1i(const AAT &token) const {
   return(m_ps->get<int32, 1>(0, m_neighbors.top().index, token)[0]);
}

Vector2i PCNearestNeighborIterator::get2i(const AAT &token) const {
   return(m_ps->get<int, 2>(0, m_neighbors.top().index, token));
}

Vector3i PCNearestNeighborIterator::get3i(const AAT &token) const {
   return(m_ps->get<int, 3>(0, m_neighbors.top().index, token));
}

Vector4i PCNearestNeighborIterator::get4i(const AAT &token) const {
   return(m_ps->get<int, 4>(0, m_neighbors.top().index, token));
}

float32  PCNearestNeighborIterator::get1f(const AAT &token) const {
   return(m_ps->get<float32, 1>(0, m_neighbors.top().index, token)[0]);
}

Vector2f PCNearestNeighborIterator::get2f(const AAT &token) const {
   return(m_ps->get<float32, 2>(0, m_neighbors.top().index, token));
}

Vector3f PCNearestNeighborIterator::get3f(const AAT &token) const {
   return(m_ps->get<float32, 3>(0, m_neighbors.top().index, token));
}

Vector4f PCNearestNeighborIterator::get4f(const AAT &token) const {
   return(m_ps->get<float32, 4>(0, m_neighbors.top().index, token));
}

/// ask VertexDescriptor for buffer size
// DO NOT USE!
void     PCNearestNeighborIterator::getVertex(char *buffer) const {
   m_ps->getVertex(m_neighbors.top().index, buffer);
}

// --- blockwise access (faster) NOT IMPLEMENTED!!!!!!!!!!!
// DO NOT USE...
/// step foreward to next data block, false if end-of-set (affects the same position pointer as "next()")
/// no effect for hierarchal iterators (guaranteed to yield only one block per node)
bool PCNearestNeighborIterator::nextBlock(){ return(false); };
/// access raw data of current data block; call unlock to free;
/// one data block is guaranteed to easily fit into main memory
/// for hierarchical iterators: locks current node only
const PointSet * PCNearestNeighborIterator::lockBlockRead(){ return(NULL); };
/// free locked PointSet again
void PCNearestNeighborIterator::unlock(){ };
*/
