//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCTopologyNeighborhoodIterator.h"
//---------------------------------------------------------------------------
#include "InCorePCTopologyGraph.h"
#include "PointSet.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS(PCTopologyNeighborhoodIterator, 0)
{
	BEGIN_CLASS_INIT( PCTopologyNeighborhoodIterator );
}


PCTopologyNeighborhoodIterator::PCTopologyNeighborhoodIterator() {
	tg = NULL;
	ps = NULL;
	currentPoint = 0;
	currentEdge = 0;
	numPoints = 0;
}

PCTopologyNeighborhoodIterator::PCTopologyNeighborhoodIterator(InCorePCTopologyGraph *tg, PointSet *ps) {
	this->ps = ps;
	this->tg = tg;
	x4Assert(ps);
	x4Assert(tg);
	currentPoint = 0;
	currentEdge = 0;
	numPoints = ps->getNumEntries();
}

void PCTopologyNeighborhoodIterator::goToVertex(card32 v) {
	if (v >= numPoints) v = numPoints-1;
	currentPoint = v;
	currentEdge = 0;
}

card32 PCTopologyNeighborhoodIterator::getCurrentVertex() const {
	return currentPoint;
}

void PCTopologyNeighborhoodIterator::nextVertex() {
	currentPoint++;
}

bool PCTopologyNeighborhoodIterator::atEndOfVertices() const {
	return currentPoint >= numPoints;
}


void PCTopologyNeighborhoodIterator::reset() {
	currentEdge = 0;
}

void PCTopologyNeighborhoodIterator::next() {
	currentEdge++;
}

bool PCTopologyNeighborhoodIterator::atEnd() {
	if (currentPoint >= numPoints) return true;
	return currentEdge >= tg->getNumAdjacentVertices(currentPoint);
}


#define CALC_INDEX \
	x4Assert(!atEndOfVertices());\
	x4Assert(!((PCTopologyNeighborhoodIterator*)this)->atEnd()); \
	card32 index = tg->getAdjacentVertex(currentPoint, currentEdge);


card32 PCTopologyNeighborhoodIterator::getCurrentAdjacentVertex() const {
	CALC_INDEX
	return index;
}

// -- adjacent vertices attribute access
card8     PCTopologyNeighborhoodIterator::get1ub(const AAT &token) const {
	CALC_INDEX
	return ps->get<card8,1>(index, token)[0];
}

Vector2ub PCTopologyNeighborhoodIterator::get2ub(const AAT &token) const {
	CALC_INDEX
	return ps->get<card8,2>(index, token);
}

Vector3ub PCTopologyNeighborhoodIterator::get3ub(const AAT &token) const {
	CALC_INDEX
	return ps->get<card8,3>(index, token);
}

Vector4ub PCTopologyNeighborhoodIterator::get4ub(const AAT &token) const {
	CALC_INDEX
	return ps->get<card8,4>(index, token);
}


int32    PCTopologyNeighborhoodIterator::get1i(const AAT &token) const {
	CALC_INDEX
	return ps->get<int32,1>(index, token)[0];
}

Vector2i PCTopologyNeighborhoodIterator::get2i(const AAT &token) const {
	CALC_INDEX
	return ps->get<int32,2>(index, token);
}

Vector3i PCTopologyNeighborhoodIterator::get3i(const AAT &token) const {
	CALC_INDEX
	return ps->get<int32,3>(index, token);
}

Vector4i PCTopologyNeighborhoodIterator::get4i(const AAT &token) const {
	CALC_INDEX
	return ps->get<int32,4>(index, token);
}

card32    PCTopologyNeighborhoodIterator::get1u(const AAT &token) const {
	CALC_INDEX
	return ps->get<card32,1>(index, token)[0];
}

Vector2u PCTopologyNeighborhoodIterator::get2u(const AAT &token) const {
	CALC_INDEX
	return ps->get<card32,2>(index, token);
}

Vector3u PCTopologyNeighborhoodIterator::get3u(const AAT &token) const {
	CALC_INDEX
	return ps->get<card32,3>(index, token);
}

Vector4u PCTopologyNeighborhoodIterator::get4u(const AAT &token) const {
	CALC_INDEX
	return ps->get<card32,4>(index, token);
}

float32  PCTopologyNeighborhoodIterator::get1f(const AAT &token) const {
	CALC_INDEX
	return ps->get<float32,1>(index, token)[0];
}

Vector2f PCTopologyNeighborhoodIterator::get2f(const AAT &token) const {
	CALC_INDEX
	return ps->get<float32,2>(index, token);
}

Vector3f PCTopologyNeighborhoodIterator::get3f(const AAT &token) const {
	CALC_INDEX
	return ps->get<float32,3>(index, token);
}

Vector4f PCTopologyNeighborhoodIterator::get4f(const AAT &token) const {
	CALC_INDEX
	return ps->get<float32,4>(index, token);
}


void     PCTopologyNeighborhoodIterator::getVertex(char *buffer) const {
	CALC_INDEX
	ps->getVertex(index, buffer);
}


bool PCTopologyNeighborhoodIterator::nextBlock() {
	return false;
}

const PointSet * PCTopologyNeighborhoodIterator::lockBlockRead() {
	return ps;
}

void PCTopologyNeighborhoodIterator::unlock() {}


card64 PCTopologyNeighborhoodIterator::getNumPointsInBlock() const {
	return ps->getNumEntries();
}
 
card64 PCTopologyNeighborhoodIterator::getCurrentPointIndex() const {
	return currentPoint;
}

void PCTopologyNeighborhoodIterator::goToIndex(card64 i) {
	goToVertex((card32)i);	
}

