//---------------------------------------------------------------------------
#ifndef ParametricInCorePointCloudH
#define ParametricInCorePointCloudH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "PointSetIterator.h"
#include "PointSet.h"
//---------------------------------------------------------------------------


namespace X4 {

class AttachedData;
class AttachedDataContainer;
class BasicParametricInCorePointCloudIterator;


class X3GEOMETRY_API ParametricInCorePointCloud : public PointCloud {
 private:
   static MetaClass *ParametricInCorePointCloudMetaClass;
   static Object* newInstance();

   PointSet *ps;
   AttachedDataContainer *hierarchyAttachedDSs;

 public:
   ParametricInCorePointCloud();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual const VertexDescriptor *getDescr() const;
   virtual PointCloud             *appendFields(const VertexDescriptor *newAttribs) const;
   virtual PointCloudIterator     *createIterator(SceneObjectIteratorType type);
   virtual bool                    supportsIteratorType(SceneObjectIteratorType cap);
   virtual card64                  getNumPoints() const;
   virtual void                    clear();

   virtual void clearAttachments();

   // --- new methods
   virtual void clearAndSetup(card32 width, card32 height, const VertexDescriptor *vertDescr);
   virtual const PointSet *getPointSet() const {return ps;}
   virtual void setPointSet(PointSet *ps) {delete this->ps; this->ps = ps;}
   virtual PointSet *getPointSet() {return ps;}
   virtual ParametricInCorePointCloud *createCopyWithNewPointSet(PointSet *ps) const; // ownership of ps will be transfered to this
   // ---

	template<class FloatType,unsigned dim>
	StaticVector<FloatType,dim> getBilinearInterpolatedAttribute( float x, float y, AAT aat) const;

	Vector3f getBilinearInterpolatedAttribute3f( float x, float y, AAT aat) const
	{
		return getBilinearInterpolatedAttribute<float,3>(x,y,aat);
	}


	virtual void invalidateSelectedPoints();

   ~ParametricInCorePointCloud();
 friend BasicParametricInCorePointCloudIterator;
};




class X3GEOMETRY_API BasicParametricInCorePointCloudIterator : public AttachedDataPointSetIterator {
 private:
   static MetaClass *BasicParametricInCorePointCloudIteratorMetaClass;
   static Object* newInstance();

   ParametricInCorePointCloud *pointCloud;

   BasicParametricInCorePointCloudIterator() {}

 public:
   BasicParametricInCorePointCloudIterator(PointSet *ps, AttachedDataContainer *attData);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();
};


template <class FloatType, unsigned dim>
StaticVector<FloatType,dim> ParametricInCorePointCloud::getBilinearInterpolatedAttribute( float x, float y, AAT aat) const
{
	const mpint sizeX = ps->getWidth();
	const mpint sizeY = ps->getHeight();

	x = max(0.0f,min(x,(float)sizeX-0.01f));
	y = max(0.0f,min(y,(float)sizeY-0.01f));

	int ix = (int)x;
	int iy = (int)y;
	float fx = x - ix;
	float fy = y - iy;

	

	if( ix + 1 < sizeX && iy + 1 < sizeY )
	{
		return (ps->get<FloatType,dim>(ix,iy,aat) * (1.0f - fx)
			    + ps->get<FloatType,dim>(ix+1,iy,aat) * fx ) * (1.0f - fy)
			+   (ps->get<FloatType,dim>(ix,iy+1,aat) * (1.0f - fx)
				 + ps->get<FloatType,dim>(ix+1,iy+1,aat) * fx ) * fy;
		//return (pixels[ix+sizeX*iy] * (1.0f - fx) + pixels[ix+1+sizeX*iy] * fx) * (1.0f - fy)
		//	+ (pixels[ix+sizeX*(iy+1)] * (1.0f - fx) + pixels[ix+1+sizeX*(iy+1)] * fx) * fy;
	}

	if( ix + 1 < sizeX )
	{
		return ps->get<FloatType,dim>(ix,iy,aat) * (1.0f - fx)
			+ ps->get<FloatType,dim>(ix+1,iy,aat) * fx;
		//return (pixels[ix+sizeX*iy] * (1.0f - fx) + pixels[ix+1+sizeX*iy] * fx);
	}

	if( iy + 1 < sizeY )
	{
		return ps->get<FloatType,dim>(ix,iy,aat) * (1.0f - fy)
			+ ps->get<FloatType,dim>(ix,iy+1,aat) * fy;
		//return pixels[ix+sizeX*iy] * (1.0f - fy) + pixels[ix+sizeX*(iy+1)] * fy;
	}
	return ps->get<FloatType,dim>(ix,iy,aat);
	//return pixels[ix+sizeX*iy];

}

}


#endif
