//---------------------------------------------------------------------------
#ifndef PointCloudH
#define PointCloudH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "VertexDescriptor.h"
#include "SceneObject.h"
#include "PointCloudIterators.h"
//---------------------------------------------------------------------------


namespace X4 {

    /// A collection of points with attributes in 3-space.
    /// Data access via iterators (createIterator).
    class X3GEOMETRY_API PointCloud : public SceneObject {
        X4_ABSTRACT_CLASS(UnstructuredInCoreTriangleMesh);

    public:
        PointCloud();

        // new methods (abstract)
        /// retrieve the per-point attribute description
        virtual const VertexDescriptor *getDescr() const = 0;
        /// append additional attributes to the point - attention, this method creates a _copy_.
        virtual PointCloud             *appendFields(const VertexDescriptor *newAttribs) const = 0;
        /// get number of points in cloud
        virtual card64                  getNumPoints() const = 0;
        /// throw away all points, keep settings
        virtual void                    clear() = 0;

        // macros
        /// macro - get attribute access token by name
        inline AttributeAccessToken getAAT(std::string name) const {
            return getDescr()->getToken(name);
        }
        /// macro - check for attribute by name
        inline bool providesAttribute(std::string name) const {
            return getDescr()->providesAttribute(name);
        }
        /// macro - are there any points in here?
        inline virtual bool isEmpty() {return getNumPoints() == 0;}

        virtual ~PointCloud();
    };


}

#endif
