//---------------------------------------------------------------------------
#ifndef PointCloudTopologyGraphH
#define PointCloudTopologyGraphH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "Persistent.h"
#include "PointCloudIterators.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {

class PointSet;
class VertexDescriptor;



class X3GEOMETRY_API PointCloudTopologyGraph : public Persistent {
	X4_ABSTRACT_CLASS( PointCloudTopologyGraph )

 public:
   typedef card32 VertexIndex;
   typedef card32 EdgeIndex;
   
	/// setup attribute structure
	/// ironically, the per edge attributes are specified by an (ill named) VertexDescriptor...
	/// vd must contain the attribute channel int32 vindex[2]
	/// further attributes may be added ad lib
	virtual void clearAndSetup(const VertexDescriptor *vd) = 0;

	/// add a new edge; all attributes will be mem-zeroed
	virtual void addEdge(VertexIndex v1, VertexIndex v2) = 0;

	/// add a new edge with attributes; member attributes 
	/// index1, index2 will be extracted to determine the topology
	virtual void addEdge(const char *data) = 0;
	
	/// appends new attribute channels to the edges (in place). duplicates are avoided: if an attribute name already exists, no new data field is created.
   virtual void appendFields(const VertexDescriptor *newAttribs)  = 0;

	/// access data structure; argument is a reference to
	/// the point set for which the topology is defined
	virtual PointCloudIterator *createIterator(PointSet *ps) = 0;
};




}

#endif