//---------------------------------------------------------------------------
#ifndef PointSetKNNQueryH
#define PointSetKNNQueryH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "HierarchicalKNNIterator.h"
//---------------------------------------------------------------------------
#include <queue>
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {

/// compute k-nearest neighbors efficiently using a hierarchical data structure
class X3GEOMETRY_API PointSetKNNQuery {
 private:
   AttachedIndexedOctree *octree;
   const PointSet *ps;
   AAT POSITION;
   Vector3f targetPoint;

   std::priority_queue<PointEntry> pqPoints;
   std::priority_queue<BoxEntry> pqBoxes;
   AIOctreeNode* rootNode;

   void putBoxOnQueue(AIOctreeNode* box);
   void putPointOnQueue(const card32 &indexOfPoint);

 public:   
	static AttachedIndexedOctree *createOctree(
		const PointSet *ps,					// Point set
		card32 maxPointsPerNode = 20		// Number of points per node
		);

   PointSetKNNQuery(const PointSet *ps, AttachedIndexedOctree *octree, const Vector3f &targetPoint, const AAT &positionAAT);

	/// reset position in space where search starts
   void setTargetPoint(const Vector3f &targetPoint);
   /// get next nearest point. found is true if another point has been found. if found is false, nextIndex is undefined.
   void getNextNearestPointIndex(card32 &nextIndex, bool &found);
};



}

#endif