//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PointSetSphericalQuery.h"
//---------------------------------------------------------------------------
#include "PointSetKNNQuery.h"
#include "PointSet.h"
#include "PointSet.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



PointSetSphericalQuery::PointSetSphericalQuery(const PointSet *ps, AttachedIndexedOctree *octree, const Vector3f &targetPoint, float32 radius)
: knnQuery(ps, octree, targetPoint, ps->getAAT("position", 3, VAD::DATA_FORMAT_FLOAT32))
{
	this->targetPoint = targetPoint;
	this->radius = radius;
	this->ps = ps;
	positionAAT = ps->getAAT("position");
}

PointSetSphericalQuery::~PointSetSphericalQuery() {}




void PointSetSphericalQuery::setTargetSphere(const Vector3f &targetPoint, float32 radius) 
{
	this->radius = radius;
	this->targetPoint = targetPoint;
	knnQuery.setTargetPoint(targetPoint);
}

void PointSetSphericalQuery::getNextPointIndex(card32 &nextIndex, bool &found) 
{
	knnQuery.getNextNearestPointIndex(nextIndex, found);
	if (found) {
		Vector3f pos = ps->get3f(nextIndex, positionAAT);
		if (norm(pos-targetPoint) <= radius) {
			found = true;
		} else {
			found = false;
		}
	}
}

AttachedIndexedOctree *PointSetSphericalQuery::createOctree(const PointSet *ps, card32 maxPointsPerNode) {
	return PointSetKNNQuery::createOctree(ps, maxPointsPerNode);
}
