//---------------------------------------------------------------------------
#ifndef PointSetSphericalQueryH
#define PointSetSphericalQueryH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "VertexDescriptor.h"
#include "PointSetKNNQuery.h"
//---------------------------------------------------------------------------

namespace X4 {

class PointSetKNNQuery;
class PointSet;
class AttachedIndexedOctree;

///
///   computes all points within a spherical neighborhood of a given spatial position
///   for a point set
///
class X3GEOMETRY_API PointSetSphericalQuery {
 private:
	/// evil hack - should be replaced by native range query (more efficient)
	PointSetKNNQuery knnQuery;
	const PointSet *ps;
	Vector3f targetPoint;
	float32 radius;
	AAT positionAAT;
	
 public:
	static AttachedIndexedOctree *createOctree(
		const PointSet *ps,					// Point set
		card32 maxPointsPerNode = 20		// Number of points per node
	);

	PointSetSphericalQuery(const PointSet *ps, AttachedIndexedOctree *octree, const Vector3f &targetPoint, float32 radius);

	/// reset range
   void setTargetSphere(const Vector3f &targetPoint, float32 radius);
   /// get next point from range. found is true if another point has been found. if found is false, nextIndex is undefined.
   void getNextPointIndex(card32 &nextIndex, bool &found);

   ~PointSetSphericalQuery();
};

}

#endif