#ifndef PrecomputedDistanceFieldH
#define PrecomputedDistanceFieldH
//---------------------------------------------------------------------------
#include "x3geometry.h"
#include "BoundingBox.inline.h"
//---------------------------------------------------------------------------
#include "Plane.h"
#include "opencv\cxcore.h"
#include "X4Types.h"

namespace X4
{
class PointCloud;
class PrecomputedDistanceField;
class SceneObjectIterator;
class BasicPointCloudIterator;
class RandomAccessPointCloudIterator;
/**
	provides precomputed distances, accurate only in near-field of the surface
	input point cloud requires normal vectors
*/
class X3GEOMETRY_API PrecomputedDistanceField 
{

private:

	struct TGrid
	{
		TGrid( unsigned dim, bool isLeaf) 
		{
			data = NULL;
			subgrids = NULL;
			dimension = dim;
			//data = new unsigned[dim*dim*dim];
			//data[0] = 0x7fffffff;	// mark as new

			if( !isLeaf )
			{
				subgrids = new TGrid*[dim*dim*dim];
				for( unsigned i=0;i<dim*dim*dim;i++ )
					subgrids[i] = NULL;
			}
		}

		~TGrid()
		{
			if( subgrids != NULL )
			{
				for( unsigned i=0;i<dimension*dimension*dimension;i++ )
					delete subgrids[i];
				delete[] subgrids;
			}
			else
				delete[] data;
		}

		static const int NodeDimension = 8;
		static const int MaxRootNodeDimension = 64;

		// setup grid
		void insert( PrecomputedDistanceField * df, const Vector3f &p, mpcard index, float nearfield, int level);
		inline Vector3f computeLocalVector( const Vector3f &p) const;
		void getBBs( std::vector<BoundingBox3f> &bbs);
		unsigned getClosestPoint( const Vector3f &p);
		
		unsigned dimension;
		BoundingBox3f bb;
		TGrid ** subgrids;
		unsigned * data;	// indices
		std::vector<unsigned> points;
	};

	TGrid		*	m_Grid;
	PointCloud	* m_PC;
	AAT			m_PosAAT;
	AAT			m_NormalAAT;
	SceneObjectIterator * m_PCIter;
	BasicPointCloudIterator * m_PC_bIt;
	RandomAccessPointCloudIterator * m_PC_rIt;

public:

	/// creates distance field with grid size gridPrecision, guarantees to be accurate in nearfield
	PrecomputedDistanceField( PointCloud * pc, float gridPrecision, float nearfield );
	~PrecomputedDistanceField(void);
 
	/// returns index of closest point
	unsigned getClosestPointIndex( const Vector3f & pos);
	
	/// returns closest point's plane
	Plane getClosestPointPlane( const Vector3f &pos);

	/// returns all non-empty bounding boxes (for debug)
	std::vector<BoundingBox3f> getBBs();

	/// returns position of point #index
	Vector3f getPointPosition( mpcard index) const;

};

}

#endif