//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "RegularInCoreVolume.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "RegularVolume.h"
#include "AttachedData.h"
#include "BoundingBox.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( RegularInCoreVolume, 0 ) {
	BEGIN_CLASS_INIT( RegularInCoreVolume );
	ADD_OBJECT_PROP(voxels, 0, RegularVolume::getClass(), true);
	ADD_OBJECT_PROP(hierarchyAttachedDSs, 0, AttachedDataContainer::getClass(), true);
}


IMPLEMENT_X4_CLASS( RegularInCoreVolumeIterator, 0 ) {
	BEGIN_CLASS_INIT( RegularInCoreVolumeIterator );
}


// ----


RegularInCoreVolume::RegularInCoreVolume() 
{
   voxels = NULL;
   hierarchyAttachedDSs = new AttachedDataContainer();
}


void RegularInCoreVolume::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

RegularInCoreVolume::~RegularInCoreVolume() 
{
   delete voxels;
   delete hierarchyAttachedDSs;
}



card64 RegularInCoreVolume::getNumPoints() const
{
   return voxels->getWidth() * voxels->getHeight() * voxels->getDepth();
}


const VertexDescriptor *RegularInCoreVolume::getDescr() const 
{
   if (voxels == NULL) return NULL;
   return voxels->getDescr();
}

PointCloud *RegularInCoreVolume::appendFields(const VertexDescriptor *newAttribs) const 
{
   RegularInCoreVolume *result = (RegularInCoreVolume*)this->copy();
   result->setVoxels(voxels->appendFields(newAttribs));
   return result;
}

void RegularInCoreVolume::clear() 
{
   delete voxels;
   voxels = NULL;
}

void RegularInCoreVolume::clearAndSetup(RegularVolume *vol) 
{
   setVoxels(vol);
}

void RegularInCoreVolume::clearAndSetup(card32 width, card32 height, card32 depth, const VertexDescriptor *descr) 
{
   clearAndSetup(width, height, depth, descr, BoundingBox3f(NULL_VECTOR3F, makeVector3f(1,1,1)));
}

void RegularInCoreVolume::clearAndSetup(card32 width, card32 height, card32 depth, const VertexDescriptor *descr, const BoundingBox3f &bb) 
{
   clear();
   voxels = new RegularVolume();
   voxels->clearAndSetup(width, height, depth, descr, bb);
}

void RegularInCoreVolume::setVoxels(RegularVolume *v) {
   delete voxels; 
   voxels = v;
}

SceneObjectIterator *RegularInCoreVolume::createIterator(SceneObjectIteratorType type) 
{
   if (!supportsIteratorType(type)) {
      return NULL;
   } else {
		return new RegularInCoreVolumeIterator(voxels, hierarchyAttachedDSs);
   }
}

bool RegularInCoreVolume::supportsIteratorType(SceneObjectIteratorType cap) 
{
   SOIT possibleCaps(  SOIT::CAP_BASIC_PC 
							| SOIT::CAP_INDEXED_PC 
							| SOIT::CAP_RANDOM_ACCESS_PC 
							| SOIT::CAP_MODIFYABLE_PC
							| SOIT::CAP_HIERARCHICAL  
							| SOIT::CAP_ATTACH_SECONDARY
							//| SOIT::CAP_REGULAR_VOLUME
							| SOIT::CAP_VOLUMETRIC );
   if( !possibleCaps.providesCapsFor(cap))
		return false;
	else
		return true;
}


// --


RegularInCoreVolumeIterator::RegularInCoreVolumeIterator(RegularVolume *voxels, AttachedDataContainer *attData) 
: AttachedDataPointSetIterator(voxels->getVoxelData(), attData)
{
   this->voxels = voxels;
}

BoundingBox3f RegularInCoreVolumeIterator::getBoundingBox() {
   return voxels->getBoundingBox();
}

BoundingBox3f RegularInCoreVolumeIterator::getVolumeRegion() {
   return voxels->getBoundingBox();
}

void RegularInCoreVolumeIterator::goTo(card32 x, card32 y, card32 z)
{
	goToIndex( (z*voxels->getHeight()  + y)*voxels->getWidth() + x  );
}

Vector3i RegularInCoreVolumeIterator::getVolumeSize()
{
	return makeVector3i(voxels->getWidth(), voxels->getHeight(), voxels->getDepth());
}

const RegularVolume *RegularInCoreVolumeIterator::lockVolumeBlockRead() {
	return voxels;
}

RegularVolume *RegularInCoreVolumeIterator::lockVolumeBlockReadWrite() {
   attachedDataLargeChange = true;
	return voxels;
}

void RegularInCoreVolumeIterator::unlockVolume() {}
