//---------------------------------------------------------------------------
#ifndef RegularInCoreVolumeH
#define RegularInCoreVolumeH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "InCoreVolume.h"
#include "PointSetIterator.h"
//---------------------------------------------------------------------------

namespace X4 {
   
class RegularVolume;
class AttachedDataContainer;
   

///
///   Regularly sampled, multi-channel volume object (array of voxels)
///   This is a scene object, indirectly inheriting from PointCloud
///
class X3GEOMETRY_API RegularInCoreVolume : public InCoreVolume {
	X4_CLASS( RegularInCoreVolume )
 private:
   RegularVolume *voxels;
   AttachedDataContainer *hierarchyAttachedDSs;

 public:
	RegularInCoreVolume();

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual SceneObjectIterator *createIterator(SceneObjectIteratorType type);
   virtual bool                 supportsIteratorType(SceneObjectIteratorType cap);

   virtual const VertexDescriptor *getDescr() const;
   virtual PointCloud             *appendFields(const VertexDescriptor *newAttribs) const;
   virtual card64                  getNumPoints() const;
   virtual void                    clear();

   /// takes ownership
   virtual void clearAndSetup(RegularVolume *vol);
   virtual void clearAndSetup(card32 width, card32 height, card32 depth, const VertexDescriptor *descr);
   virtual void clearAndSetup(card32 width, card32 height, card32 depth, const VertexDescriptor *descr, const BoundingBox3f &bb);
   virtual const RegularVolume *getVoxels() const {return voxels;}
   virtual void setVoxels(RegularVolume *v);
   virtual RegularVolume *getVoxels() {return voxels;}

   ~RegularInCoreVolume();
};


class X3GEOMETRY_API RegularInCoreVolumeIterator : public AttachedDataPointSetIterator,
                                                    public VolumetricIterator {
   X4_CLASS( RegularInCoreVolumeIterator )
   
 private:
   RegularVolume *voxels;

   RegularInCoreVolumeIterator() {}

 public:
   RegularInCoreVolumeIterator(RegularVolume *voxels, AttachedDataContainer *attData);

   /// HierarchicalIterator
   virtual BoundingBox3f getBoundingBox();

   /// RegularVolumeIterator
   virtual BoundingBox3f getVolumeRegion();
	virtual void goTo(card32 x, card32 y, card32 z);
	virtual Vector3i getVolumeSize();
	virtual const RegularVolume *lockVolumeBlockRead();
	virtual RegularVolume *lockVolumeBlockReadWrite();
	virtual void unlockVolume();
};


}

#endif