//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SceneObject.h"
//---------------------------------------------------------------------------
#include "ObjectClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "NumericalClassProperty.h"
#include "PointCloud.h"
#include "LinearAlgebra.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_ABSTRACT_CLASS(SceneObject, 0)
{
    BEGIN_CLASS_INIT(SceneObject);
    ADD_CARD32_PROP( materialIndex, 0);
    ADD_MATRIX4F_PROP( transformation,0 );
    ADD_OBJECT_PROP( attachments,0,AttachedDataContainer::getClass(), true );
}

// ----

SceneObject::SceneObject() {
    materialIndex  = 0;
    transformation = IDENTITY4F;
    attachments = new AttachedDataContainer;
}

void SceneObject::assignSettings(const SceneObject *assignFrom) {
    x4Assert(assignFrom != NULL);
    this->materialIndex = assignFrom->materialIndex;
    this->transformation = assignFrom->transformation;
    attachments->assign(assignFrom->attachments);
    attachments->deleteDelOnWriteData(AttachedData::CT_EVERYTHING_CHANGED);
}

void SceneObject::assign(const Object* obj, X4CopyContext *context) {
    const SceneObject *so = dynamic_cast<const SceneObject*>(obj);
    assignSettings(so);
    forget(context);
}

SceneObject *SceneObject::createOOCAwareCopy(std::string idPrefix, bool insertOriginal, std::string idSuffix) {
    forget(idPrefix);
    forget(insertOriginal);
    forget(idSuffix);
    return (SceneObject*)copy();
}


void SceneObject::read(InputObjectStream *s) {
    //bool oldStyle = s->getClassVersion(PointCloud::getClass()) < MetaClass::FIRST_VERSION + 2;
    //if (oldStyle) {
    //    /// legacy parameter
    //    std::string name;
    //    s->readString(name);
    //    forget(name);

    //    card32 flags;
    //    s->read(flags);
    //    forget(flags);
    //}
    Persistent::read(s);
}

void SceneObject::clearAttachments() {
    attachments->clear();
}

void SceneObject::deleteAttachment(const std::string &name){
    attachments->deleteData(name);
}

SceneObject::~SceneObject() {
    delete attachments;
}
