//---------------------------------------------------------------------------
#ifndef SimpleOctreeH
#define SimpleOctreeH
//---------------------------------------------------------------------------
#include "x3geometry.h"
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4 {

    template <typename T>
    class SimpleOctree {
    private:
        T m_container;
        SimpleOctree* m_children;
        SimpleOctree* m_parent;

    public:
        SimpleOctree(void) :m_children(NULL), m_parent(NULL){
        }
        ~SimpleOctree(void)
        {
            deleteChildren();
        }

        void addChildren(void)
        {
            //if(m_children) deleteChildren();
            m_children[i] = new SimpleOctree<T>[8];
            for (int i = 0; i < 8; i++){
                m_children[i].m_parent = this;
            }
        }
        void deleteChildren(void)
        {
            if (m_children != NULL){
                delete[] m_children;
                m_children = NULL;
            }
        }

        /// returns child
        /// index means:
        ///     6---7
        ///    /   /|
        ///   2---3 5
        ///   |   |/
        ///   0---1
        ///
        SimpleOctree* getChild(const int index) const {
            if (m_children == NULL) return(NULL);
            return(&(m_children[index]));
        }
        SimpleOctree* getParent(void) const {
            return(m_parent);
        }
        bool isLeaf(void) const {
            return(m_children == NULL);
        }
        T& getContent(void){ return(m_container); };
    };


    template <typename T>
    class FastSimpleOctree
    {
    private:
        T m_container;
        // Parent node
        FastSimpleOctree* m_parent;
        FastSimpleOctree* m_children;
        // Child index from parent perspective
        int32 m_index;

    public:
        FastSimpleOctree(void) : m_children(NULL), m_index(-1), m_parent(NULL) {}

        void assign(const FastSimpleOctree<T>& other)
        {
            m_container = other.m_container;
            deleteChildren();
            if (other.m_children) {
                addChildren();
                for (unsigned i = 0; i < 8; i++) {
                    m_children[i].assign(other.m_children[i]);
                }
            } else {
                m_children = NULL;
            }
        }

        ~FastSimpleOctree(void)
        {
            deleteChildren();
            m_parent = NULL;
        }

        inline void addChildren(void)
        {
            m_children = new FastSimpleOctree<T>[8];
            for (unsigned index = 0; index < 8; index++){
                m_children[index].m_index = index;
                m_children[index].m_parent = this;
            }
        }

        inline void deleteChildren(void)
        {
            if (m_children != NULL) {
                delete [] m_children;
                m_children = NULL;
            }
        }

        /// returns child
        /// index means:
        ///     6---7
        ///    /   /|
        ///   2---3 5
        ///   |   |/
        ///   0---1
        ///s
        inline FastSimpleOctree* getChild(const int& index) const {
            if (m_children == NULL) return (NULL);
            return (&(m_children[index]));
        }

        inline bool isLeaf(void) const {
            return (m_children == NULL);
        }

        inline T& getContent(void) { return (m_container); }
        inline const T& getContent(void) const { return (m_container); }
        inline int32 getIndex() const { return m_index; }
        inline FastSimpleOctree* getParent() const { return m_parent; }
    };



}

#endif
