#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SplatPullPushInterpolation.h"
//---------------------------------------------------------------------------
#include "PointSet.h"
#include "PointSet.inline.h"
#include "BoundingBox.h"
#include "VertexDescriptor.h"
#include "DebugOutput.h"
#include <qtGUI/qimage>
#include "RGBAImage.h"
#include "TemplatedImage.inline.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( SplatPullPushInterpolation ,0)
{
	BEGIN_CLASS_INIT( SplatPullPushInterpolation );
}

SplatPullPushInterpolation::SplatPullPushInterpolation(void)
{
	m_Sigma = 2.0f;
}

SplatPullPushInterpolation::~SplatPullPushInterpolation(void)
{
}

void SplatPullPushInterpolation::assign(const Object* obj, X4CopyContext *context)
{
	ScatteredDataInterpolation::assign(obj,context);
	const SplatPullPushInterpolation * o = dynamic_cast<const SplatPullPushInterpolation*>(obj);
	x4Assert( o != NULL );

}

void saveRGBAImageToTheDiskAndMakeSureThatThereIsNoNameConflict( RGBAImage * image, const char * filename )
{
	QImage qimage(image->getSizeX(), image->getSizeY(), QImage::Format_RGB32 );
	for( card32 x=0;x<image->getSizeX();x++ )
	{
		for( card32 y=0;y<image->getSizeY();y++ )
		{
			Vector4ub col = image->getPixel(x,y);
			card32 qcol =  (col[2] << 0)+
								(col[1] << 8)+
								(col[0] << 16);
			qimage.setPixel(x,y, qcol );

		}
	}
	qimage.save( filename, "PNG" );
}

template<class FloatType,unsigned dim>
void saveFloatImagePyramid( std::vector< TemplatedImage< StaticVector<FloatType,dim+1> >* > &images, std::string filename )
{
	card32 width = images[0]->getSizeX();
	card32 height = 0;
	for( card32 i=0;i<images.size();i++ )
		height += 2 + images[i]->getSizeY(); 
	RGBAImage result( width, height ), resultAlpha( width, height);
	result.clear( makeVector4ub(255,255,255,255) );
	resultAlpha.clear(makeVector4ub(255,255,255,255) );
	
	card32 currentY = 0;
	for( card32 i=0;i<images.size();i++ )
	{
		TemplatedImage< StaticVector<FloatType,dim+1> >* image = images[ images.size()-1-i ];
		for( card32 x=0;x<image->getSizeX();x++ )
			for( card32 y=0;y<image->getSizeY();y++ )
			{
				StaticVector<FloatType,dim+1> pixel = image->getPixel(x,y);
				Vector4ub col4ub = makeVector4ub(0,0,0,255);
				Vector4ub alpha4ub = makeVector4ub(0,0,0,255);
				for( card32 k=0;k<3 && k<dim;k++ )
					col4ub[k] = min( 255.0f, max( 0.0f, pixel[k] * 255.0f ) );
				float alpha = min( 255.0f, max( 0.0f, pixel[dim] * 255.0f ) );
				alpha4ub += makeVector4ub(1,1,1,0) * alpha;
				result.setPixel(x, currentY+y, col4ub );
				resultAlpha.setPixel(x, currentY+y, alpha4ub );

			}
		currentY += image->getSizeY() + 2;
	}

	saveRGBAImageToTheDiskAndMakeSureThatThereIsNoNameConflict( &result, (filename + ".png").c_str() );
	saveRGBAImageToTheDiskAndMakeSureThatThereIsNoNameConflict( &resultAlpha, (filename + "_alpha.png").c_str() );
}

template<class FloatType, unsigned dim>
TemplatedImage< StaticVector<FloatType,dim> > * interpolateImage( card32 imageSize, std::vector<SplatPullPushInterpolation::TWorkingVertex> &data, bool scaled, float sigma=2.0f) 
{
	// ------------------------------------------------
	// transform into [0,1]x[0,1]
	
	if (scaled)
	{
		float maxDistance = 0.0f; // distance from 0 (1D)
		for( card32 i=0;i<data.size();i++ )
		{
			maxDistance = max( maxDistance, fabs(data[i].paramPos[0]) );
			maxDistance = max( maxDistance, fabs(data[i].paramPos[1]) );
		}


		float scale = 0.5f / maxDistance;
		for( card32 i=0;i<data.size();i++ )
		{
			data[i].paramPos = data[i].paramPos * scale + makeVector2f(0.5f,0.5f);
		}
	}
	


	// ------------------------------------------------
	// create image pyramid
	std::vector< TemplatedImage< StaticVector<FloatType,dim+1> >* > images;
	card32 tmpImageSize = imageSize;
	
	while( tmpImageSize > 0 )
	{
		images.push_back( createSplatImage<FloatType,dim+1>( tmpImageSize, sigma, data ) );
		tmpImageSize /= 2;
	
	}

	//saveFloatImagePyramid<float,dim>( images, "splat_pull" );

	// ------------------------------------------------
	// push information from lower to higher resolutions
	for( int i=(int)images.size() - 2;i>=0;i-- )
	{
		for( card32 x=0;x<images[i]->getSizeX();x++ )
		{
			for( card32 y=0;y<images[i]->getSizeY();y++ )
			{
				StaticVector<FloatType,dim+1 > pixel = images[i]->getPixel(x,y);
				if( pixel[dim] <1.0f ) // need information from lower resolution?
				{
					// bilinear interpolation from lower res
					StaticVector<FloatType,dim+1 > highRes = images[i+1]->getBilinearInterpolatedPixel( (float)x / 2.0f, (float)y / 2.0f );
					
					for( card32 c=0;c<dim;c++ )
					{
						pixel[c] *= pixel[dim];
					
						pixel[c] += highRes[c]*(1-pixel[dim]);
					}
					      pixel[dim]+=highRes[dim]*(1-pixel[dim]);
					//pixel += highRes;
					//pixel[0] /= pixel[1];
					
					pixel[dim] = min(pixel[dim],1.0f);

					images[i]->setPixel(x,y,pixel);

				}

			}
		}


	}

	//saveFloatImagePyramid<float,dim>( images, "push" );

	// ------------------------------------------------
	// return highest resolution
	TemplatedImage< StaticVector<FloatType,dim> > * ret = new TemplatedImage< StaticVector<FloatType,dim> >(imageSize,imageSize);
	for( card32 x=0;x<imageSize;x++ )
	{
		for( card32 y=0;y<imageSize;y++ )
		{
			StaticVector<FloatType,dim> dest;
			StaticVector<FloatType,dim+1> source = images[0]->getPixel(x,y);
			for( card32 c=0;c<dim;c++ )
			{
				dest[c] = source[c];
	
			}			
			ret->setPixel(x,y, dest);
		
		}
	}

	// cleanup image pyramid
	for( card32 i=0;i<images.size();i++ )
		delete images[i];

	return ret;	

}


template<class FloatType, unsigned dim>
TemplatedImage< StaticVector<FloatType,dim> > * interpolateImage( card32 imageSize, PointSet * ps, 
																				  std::string src_channel, card32 src_componentNumber, bool scaled, float sigma) 
{
	x4Assert( ps->providesAttribute( "param_pos" ) );
	x4Assert( ps->providesAttribute( src_channel ) );

	// ------------------------------------------------
	// get working data
	std::vector<SplatPullPushInterpolation::TWorkingVertex> data(ps->getNumEntries());
	AAT paramposAAT = ps->getAAT( "param_pos" );
	AAT srcAAT = ps->getAAT( src_channel );
	VAD srcVAD = ps->getDescr()->getAttribute( src_channel );
	x4Assert( srcVAD.dataFormat == VAD::DATA_FORMAT_FLOAT32 );
	//x4Assert( srcVAD.numComponents > src_componentNumber );

	for( card32 i=0;i<ps->getNumEntries();i++ )
	{
		data[i].paramPos = ps->get<float,2>(i, paramposAAT);

		switch( srcVAD.numComponents )
		{
		case 1:
			data[i].attribute[0] = ps->get<float,1>(i,srcAAT)[src_componentNumber];
			break;
		case 2: 
			for( card32 c=0;c<2;c++ )
				data[i].attribute[c] = ps->get<float,2>(i,srcAAT)[src_componentNumber+c];
			break;
		case 3:
			for( card32 c=0;c<3;c++ )
				data[i].attribute[c] = ps->get<float,3>(i,srcAAT)[src_componentNumber+c];
			break;
		case 4:
			for( card32 c=0;c<4;c++ )
				data[i].attribute[c] = ps->get<float,4>(i,srcAAT)[src_componentNumber+c];
			break;
		}
	}

	return interpolateImage<FloatType,dim>( imageSize, data, scaled,sigma );
}



TemplatedImage<Vector3f> *SplatPullPushInterpolation::createImage3f( card32 imageSize, std::vector<TWorkingVertex> &data, bool scaled) 
{
	return interpolateImage<float,3>( imageSize, data, scaled, m_Sigma );
}

TemplatedImage< StaticVector<float,1> > * SplatPullPushInterpolation::createImage1f( card32 imageSize, std::vector<TWorkingVertex> &data, bool scaled)
{
	return interpolateImage<float,1>( imageSize, data, scaled, m_Sigma );
}


template<class FloatType, unsigned dim>
TemplatedImage< StaticVector<FloatType,dim> > * createSplatImage( card32 imageSize, float sigma, 
																					  std::vector<SplatPullPushInterpolation::TWorkingVertex> &data ) 
{
	// paramPos in data is normalized to [0,1]x[0,1]
	// define a minimum weight to prevent too small weights:
	float minWeight = 0.01f;


	// create an image filled with zeros
	TemplatedImage< StaticVector<FloatType,dim> > * ret = new TemplatedImage< StaticVector<FloatType,dim> >( imageSize, imageSize );
	StaticVector<FloatType,dim> zeroVec;
	for( card32 i=0;i<dim;i++ )
		zeroVec[i] = 0;
	ret->clear( zeroVec );

	// determine splat size
	int splatSize = 1;
	while(SplatPullPushInterpolation::basisFunction(splatSize, sigma) > minWeight)
		splatSize++;
	
	
	// iterate over every data point and draw splat
	for( card32 i=0;i<data.size();i++ )
	{
		Vector2f point = data[i].paramPos * (imageSize-1) + makeVector2f(0.5f,0.5f);
		Vector4f	attribute = data[i].attribute;

		// clamp splat to image
		int startX = max(point[0] - splatSize,0.0f);
		int endX = min(point[0] + splatSize, imageSize-1.0f);
		int startY = max(point[1] - splatSize,0.0f);
		int endY = min(point[1] + splatSize, imageSize-1.0f);
		for( int x=startX;x<=endX;x++ )
		{
			for( int y=startY;y<=endY;y++ )
			{
				float weight = SplatPullPushInterpolation::basisFunction( norm( point - makeVector2f(x,y)), sigma );
		
				if( weight < minWeight )
					continue;

				//Vector2f pix = makeVector2f( attribute * weight, weight );
				StaticVector<FloatType,dim> pix;
				for( card32 j=0;j<dim-1;j++ )
					pix[j] = attribute[j] * weight;
				pix[dim-1] = weight;

				pix += ret->getPixel(x,y);
				ret->setPixel(x,y, pix );
			}
		}
	}

	// normalize image and clamp weight to [0,1]
	
	StaticVector<FloatType,dim> * pixPtr = ret->getPixels();
	
	for( card32 i=0;i<imageSize*imageSize;i++ )
	{
		if( pixPtr[i][dim-1] > 0.0f )
		{
			for( card32 j=0;j<dim-1;j++ )
			{
				pixPtr[i][j] /= pixPtr[i][dim-1];	// normalize
				
			}
			
			pixPtr[i][dim-1] = min( 1.0f, pixPtr[i][dim-1] );
		}
	}
	

	return ret;
}

float SplatPullPushInterpolation::basisFunction( float x, float sigma ) 
{
	return exp( -x*x/(sigma*sigma)); // unscaled gauss
	//return exp( -x*x/(sigma*sigma)) / sigma  /sqrt(2.0f * M_PI); // scaled gauss
}

TemplatedImage<Vector3f> *SplatPullPushInterpolation::createImage3f( card32 imageSize, PointSet * ps, std::string channel, card32 componentNumber, bool scaled) 
{
	return interpolateImage<float,3>(imageSize,ps,channel,componentNumber,scaled,m_Sigma);
}

TemplatedImage<float> *SplatPullPushInterpolation::createImage( card32 imageSize, PointSet * ps, std::string channel, card32 componentNumber, bool scaled) 
{
	//TemplatedImage< StaticVector<float,1> >  tmp = interpolateImage<float,2>(imageSize,ps,channel,componentNumber,scaled);
	return (TemplatedImage<float> *) interpolateImage<float,1>(imageSize,ps,channel,componentNumber,scaled,m_Sigma);

}

TemplatedImage<Vector2f> *SplatPullPushInterpolation::createImage2f( card32 imageSize, PointSet * ps, std::string channel, card32 componentNumber, bool scaled) 
{
	return interpolateImage<float,2>(imageSize,ps,channel,componentNumber,scaled,m_Sigma);
}
