#ifndef SplatPullPushInterpolationH
#define SplatPullPushInterpolationH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "ScatteredDataInterpolation.h"
//---------------------------------------------------------------------------

namespace X4
{

/**
	uses a simplified version of the splat-pull-push algorithm to create an
	image from scattered sample points.
*/
class X3GEOMETRY_API SplatPullPushInterpolation : public ScatteredDataInterpolation
{
	X4_CLASS( SplatPullPushInterpolation )

private:

	/// create a splat image from samples, splatsize is defined by sigma (gauss distribution)
	/// returns a renormalized image in channel 0 and the corresponding weights in channel 1
	/// weight channel is clamped to [0,1]
	/// imageSize must be a power of two
	//TemplatedImage<Vector2f> * createSplatImage( card32 imageSize, float sigma, std::vector<TWorkingVertex> &data );
	float m_Sigma;


public:
	SplatPullPushInterpolation(void);
	~SplatPullPushInterpolation(void);

	struct TWorkingVertex
	{
			Vector2f	paramPos;
			Vector4f	attribute;
	};

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual TemplatedImage<float> * createImage( card32 imageSize, PointSet * ps, std::string channel, card32 componentNumber)
   { return createImage( imageSize, ps, channel, componentNumber, true);}

   virtual TemplatedImage<float> * createImage( card32 imageSize, PointSet * ps, std::string channel, card32 componentNumber, bool scaled);

	TemplatedImage<Vector2f> * createImage2f( card32 imageSize, PointSet * ps, std::string channel, card32 componentNumber, bool scaled);
	TemplatedImage<Vector3f> * createImage3f( card32 imageSize, PointSet * ps, std::string channel, card32 componentNumber, bool scaled);

	TemplatedImage< StaticVector<float,1> > * createImage1f( card32 imageSize, std::vector<TWorkingVertex> &data, bool scaled);
	TemplatedImage<Vector3f> * createImage3f( card32 imageSize, std::vector<TWorkingVertex> &data, bool scaled);
	
	static float basisFunction( float x, float sigma );

	float getSigma() const {return m_Sigma;}
	void setSigma(float v) {m_Sigma=v;}
};

}

#endif