#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SuperIterator.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
//---------------------------------------------------------------------------
using namespace X4;

SuperIterator::SuperIterator(SceneObjectIterator * iter, const VertexDescriptor * vd)
:m_VD(vd)
{
	m_Iter = iter;
	m_Buffer.resize(m_VD->getSize());
	m_BufferPtr = &m_Buffer[0];
	
	bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	iIt = dynamic_cast<IndexedPointCloudIterator*>(iter);
	rIt = dynamic_cast<RandomAccessPointCloudIterator*>(iter);
	mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
	dIt = dynamic_cast<DynamicPointCloudIterator*>(iter);
	tIt = dynamic_cast<TopologyIterator*>(iter);
	pIt = dynamic_cast<ParametricPointCloudIterator*>(iter);
	vIt = dynamic_cast<VolumetricIterator*>(iter);

	bIt->getVertex( m_BufferPtr );
}

SuperIterator::~SuperIterator(void)
{
	delete m_Iter;
}


void SuperIterator::flush() 
{
	mIt->setVertex( m_BufferPtr );
}

void SuperIterator::reset() 
{
	bIt->reset();
	if( !bIt->atEnd() )
		bIt->getVertex( m_BufferPtr );
}

void SuperIterator::next() 
{
	bIt->next();
	if( !bIt->atEnd() )
		bIt->getVertex( m_BufferPtr );
}

bool SuperIterator::atEnd() 
{
	return bIt->atEnd();
}

void SuperIterator::getVertex(char *buffer) const 
{
	bIt->getVertex(m_BufferPtr);
}

float SuperIterator::gget1( const AAT &token, const VAD &vad ) const 
{
	return 0;
}

void SuperIterator::gset1( const AAT &token, const VAD &vad, const float &d) 
{
}

card64 SuperIterator::getCurrentPointIndex() const 
{
	return iIt->getCurrentPointIndex();
}

void SuperIterator::goToIndex(card64 i) 
{
	rIt->goToIndex(i);
	if( !bIt->atEnd() )
		bIt->getVertex(m_BufferPtr);
}

void SuperIterator::add(const char *newPoint) 
{
	dIt->add(newPoint);
}

void SuperIterator::add(const PointSet *newPoints) 
{
	dIt->add(newPoints);
}

void SuperIterator::deleteCurrent() 
{
	dIt->deleteCurrent();
	if( !bIt->atEnd() )
		bIt->getVertex(m_BufferPtr);
}

void SuperIterator::goTo(int32 x, int32 y) 
{
	pIt->goTo(x,y);
	if( !bIt->atEnd() )
		bIt->getVertex(m_BufferPtr);
}

card32 SuperIterator::getWidth() 
{
	return pIt->getWidth();
}

card32 SuperIterator::getHeight() 
{
	return pIt->getHeight();
}

card32 SuperIterator::getNumNeighbours() 
{
	return tIt->getNumNeighbours();
}

PointIndex SuperIterator::getNeighbour( card32 nIndex ) 
{
	return tIt->getNeighbour(nIndex);
}

void SuperIterator::goTo(card32 x, card32 y, card32 z) 
{
	vIt->goTo(x,y,z);
}

Vector3i SuperIterator::getVolumeSize() 
{
	return vIt->getVolumeSize();
}
