//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "TopologyAttachment.h"
#include "InCorePCTopologyGraph.h"
#include "PointSetIterator.h"
#include "ObjectClassProperty.h"
#include "UnstructuredInCorePointCloud.h"
#include "ParametricInCorePointCloud.h"
#include "DebugOutput.h"
#include "BooleanClassProperty.h"
#include "FixedArrayClassProperty.h"
//---------------------------------------------------------------------------
#include "PointSet.h"
#include <GL\gl.h>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( TopologyAttachment, 6 ) 
{
    BEGIN_CLASS_INIT( TopologyAttachment );
    ADD_OBJECT_PROP( topology, 0, InCorePCTopologyGraph::getClass(), true );
    ADD_BOOLEAN_PROP( visible, 1 );
    ADD_FLOAT32_PROP( colorVisAmplification, 2 );
    ADD_FLOAT32_PROP( colorVisOffset, 3 );
    ADD_BOOLEAN_PROP( blending, 4 );	
    ADD_BOOLEAN_PROP(displayColor, 5);
    ADD_FLOAT32_PROP(lineWidth, 6);
    ADD_VECTOR4F_PROP(lineColor, 6);
}



TopologyAttachment::TopologyAttachment() {
    topology = NULL;
    visible = true;
    colorVisAmplification = 1.0f;
    colorVisOffset = 0.02f;
    displayColor = false;
    /**
    blending = true;
    lineWidth = 3.0f;
    lineColor = makeVector4f(1,1,1, 0.05f);
    /**/
    blending = true;
    lineWidth = 3;
    lineColor = makeVector4f(0.1f,0.1f,0.1f, 0.1f);
    /**
    blending = false;
    lineWidth = 1.0f;
    lineColor = makeVector4f(1,1,1, 0.05f);
    /**/
}

void TopologyAttachment::assign(const Object* obj, X4CopyContext *context) {
    AttachedData::assign(obj, context);
    const TopologyAttachment *other = dynamic_cast<const TopologyAttachment*>(obj);
    x4Assert(other != NULL);
    if (other != NULL) {
        if (other != this) {
            delete topology;
            topology = (InCorePCTopologyGraph*)(other->topology->copy());
        }
    }
}

void TopologyAttachment::setTopology(InCorePCTopologyGraph *topology) {
    if (this->topology != topology) {
        delete this->topology;
        this->topology = topology;
    }
}

void TopologyAttachment::swapTopology(InCorePCTopologyGraph *topology) {
    this->topology = topology;
}

void TopologyAttachment::renderGL(SceneObject *obj) {

    if (!visible) return;
    PointSet *ps = NULL;
    PointSet *edges = topology->getEdges();
    if (edges == NULL) return;
    if (dynamic_cast<UnstructuredInCorePointCloud*>(obj) != NULL) {
        ps = dynamic_cast<UnstructuredInCorePointCloud*>(obj)->getPointSet();
    }
    if (dynamic_cast<ParametricInCorePointCloud*>(obj) != NULL) {
        ps = dynamic_cast<ParametricInCorePointCloud*>(obj)->getPointSet();
    }
    if (ps == NULL) return;

    if (!ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) return;

    AAT POS = ps->getAAT("position");
    AAT EDGE_COL = NULL_AAT;
    bool colInThere = false;
    if (edges->getDescr()->providesAttribute("color", 4, VAD::DATA_FORMAT_FLOAT32)) {
        EDGE_COL = edges->getAAT("color");
        colInThere = true;
    }


    glPushAttrib(GL_ALL_ATTRIB_BITS);
    try {
        glDisable(GL_LIGHTING);
        if (blending) {
            glDisable(GL_DEPTH_TEST);
            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        }
        glLineWidth(lineWidth);
        glColor4fv(lineColor.data());//.0f, 1.0f, 1.0f, 0.05f);
        glBegin(GL_LINES);
        card32 n = ps->getNumEntries();
        for (card32 v=0; v<n; v++) {
            InCorePCTopologyGraph::VertexIndex ne = topology->getNumAdjacentVertices(v);
            for (InCorePCTopologyGraph::VertexIndex i=0; i<ne; i++) {
                int32 v2 = topology->getAdjacentVertex(v, i);
                Vector3f pos1 = ps->get3f(v, POS);
                Vector3f pos2 = ps->get3f(v2, POS);
                if (colInThere && displayColor) {
                    card32 e = topology->getEdgeIndex(v, i);
                    Vector4f c = edges->get<float,4>(e, EDGE_COL);
                    c *= colorVisAmplification;
                    c[0] += colorVisOffset;
                    c[1] += colorVisOffset;
                    c[2] += colorVisOffset;
                    if (blending)
                    {
                        float alpha = c[0];
                        glColor4f(lineColor[0],lineColor[1],lineColor[2],alpha);
                    } else {
                        glColor4fv(c.data());
                    }
                }
                glVertex3fv(pos1.data());
                glVertex3fv(pos2.data());
            }
        }
        glEnd();
    } catch (...) {
        warning("exception");
        glPopAttrib();
        throw;
    }
    glPopAttrib();
}


TopologyAttachment::~TopologyAttachment() {
    delete topology;
}

X4::card32 X4::TopologyAttachment::getSize() const
{
    PointSet * top = topology->getEdges();
    return top->getDescr()->getSize() * top->getNumEntries();
}