//---------------------------------------------------------------------------
#ifndef TopologyAttachmentH
#define TopologyAttachmentH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "AttachedData.h"
#include "RenderableObject.h"
//---------------------------------------------------------------------------


namespace X4 {

    class InCorePCTopologyGraph;
    class PointCloud;


    class X3GEOMETRY_API TopologyAttachment : public AttachedData, public RenderableObject {
        X4_CLASS( TopologyAttachment )

    private:
        InCorePCTopologyGraph *topology;
        bool visible;
        float32 colorVisAmplification;
        float32 colorVisOffset;
        bool blending;
        bool displayColor;
        float32 lineWidth;
        Vector4f lineColor;

    public:
        TopologyAttachment();

        virtual void assign(const Object* obj, X4CopyContext *context = NULL);

        // default name for attachment      
        static std::string getDefaultName() { return "TopologyAtt"; }

        void setTopology(InCorePCTopologyGraph *topology);
        InCorePCTopologyGraph *getTopology() {return topology;}
        /// like setTopology, but does not delete old object
        void swapTopology(InCorePCTopologyGraph *topology);

        bool getVisible() const {return visible;}
        void setVisible(bool v) {visible=v;}
        float32 getColorVisAmplification() const {return colorVisAmplification;}
        void setColorVisAmplification(float32 v) {colorVisAmplification=v;}
        float32 getColorVisOffset() const {return colorVisOffset;}
        void setColorVisOffset(float32 v) {colorVisOffset=v;}
        bool getBlending() const {return blending;}
        void setBlending(bool v) {blending=v;}

        void setLineColor(Vector4f col) { lineColor = col; }

        virtual void renderGL(SceneObject *obj);
        virtual card32 getSize() const;


        ~TopologyAttachment();
    };


}

#endif