//---------------------------------------------------------------------------
#ifndef TriangleMeshToolsH
#define TriangleMeshToolsH
//---------------------------------------------------------------------------
#include "x3geometry.h"
#include "UnstructuredInCoreTriangleMesh.h"
//---------------------------------------------------------------------------

namespace X4 {

	#define MESH_PARAMETERIZATION_MODE_SPHERE 0
	#define MESH_PARAMETERIZATION_MODE_PLANE 1
	#define MESH_PARAMETERIZATION_MODE_CYLINDER 2

	/// Compute normals for the mesh
	void X3GEOMETRY_API computeTriangleMeshNormals( UnstructuredInCoreTriangleMesh* mesh );
	/// Compute normals for a mesh with fitting pca to the vertices
	void X3GEOMETRY_API computeTriangleMeshNormalsByPCA( UnstructuredInCoreTriangleMesh* mesh );
	void X3GEOMETRY_API adjustTriangleIndicesToNormals( UnstructuredInCoreTriangleMesh* mesh );

	/// Smooth out normals
	void X3GEOMETRY_API smoothTriangleMeshNormals( UnstructuredInCoreTriangleMesh* mesh, float32 sigma );
	/// Snap close vertices (attention naive implementation O(n^2))
	void X3GEOMETRY_API snapTriangleMeshMesh( UnstructuredInCoreTriangleMesh* mesh, const float32 maxDist );
	/// Parameterize a triangle mesh
	void X3GEOMETRY_API parameterizeTriangleMesh( 
		UnstructuredInCoreTriangleMesh* mesh, 
		const card32 mode, 
		const float32 textureRepeat = 1.0f, 
		bool pcaROT = true, 
		card32 coordIndex0 = 0, 
		card32 coordIndex1 = 1 );

}

#endif
