//---------------------------------------------------------------------------
#ifndef UnstructuredInCorePointCloudH
#define UnstructuredInCorePointCloudH
//---------------------------------------------------------------------------
#include "x3geometry.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "PointSetIterator.h"
#include "PointSet.h"
//#include "PointSetOctree.h"
//---------------------------------------------------------------------------

namespace X4 {

class AttachedData;
class AttachedDataContainer;


class BasicUnstructeredInCorePointCloudIterator;

class X3GEOMETRY_API UnstructuredInCorePointCloud : public PointCloud {
	X4_CLASS(UnstructuredInCorePointCloud)
 protected:
   PointSet *ps;
   AttachedDataContainer *hierarchyAttachedDSs;

 protected:
   AttachedDataContainer *getHierarchyAttachedDSs() {return hierarchyAttachedDSs;}

 public:
   UnstructuredInCorePointCloud();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual const VertexDescriptor *getDescr() const;
   virtual PointCloud             *appendFields(const VertexDescriptor *newAttribs) const;
   virtual PointCloudIterator     *createIterator(SceneObjectIteratorType type);
   virtual bool                    supportsIteratorType(SceneObjectIteratorType cap);
   virtual card64                  getNumPoints() const;
   virtual void                    clear();


   virtual void clearAttachments();
   virtual void deleteDelOnWriteAttachments(AttachedData::ChangeType change = AttachedData::CT_EVERYTHING_CHANGED);
   virtual void deleteAttachment(const std::string &name);

   // --- new methods
   virtual void clearAndSetup(const VertexDescriptor *vertDescr, mpcard height);

   /// shortcut for one of the most commonly used setup
   void clearAndSetup(mpcard numPoints, bool hasPosition=true, bool hasNormal=true, bool hasColor=true, bool hasFlags=true);

   virtual const PointSet *getPointSet() const {return ps;}
   virtual void setPointSet(PointSet *ps) {delete this->ps; this->ps = ps;}
   virtual PointSet *getPointSet() {return ps;}
   virtual UnstructuredInCorePointCloud *createCopyWithNewPointSet(PointSet *ps) const; // ownership of ps will be transfered to this

   // ---

   virtual Vector3f computeCenterOfMass() const;

   ~UnstructuredInCorePointCloud();
   friend class BasicUnstructuredInCorePointCloudIterator;
};

class X3GEOMETRY_API BasicUnstructuredInCorePointCloudIterator : public AttachedDataPointSetIterator,
                                                          public DynamicPointCloudIterator {
	X4_CLASS(BasicUnstructuredInCorePointCloudIterator)
 private:
   BasicUnstructuredInCorePointCloudIterator() {}

 public:
   BasicUnstructuredInCorePointCloudIterator(PointSet *ps, AttachedDataContainer *attData);

   // DynamicPointIterator
   virtual void add(const char *newPoint);
   virtual void add(const PointSet *newPoints);
   virtual void deleteCurrent();

};


}

#endif
