//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "GLDebuggingTools.h"
#include "GL\glew.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

// define this symbol to enable diagnostic messages
#undef GL_CHECK_OUTPUT

void X4::x4glErrorCheck(string filename, string line) {
	if (filename=="") filename = "(not specified)";
	if (line=="") line = "(not specified)";

	GLuint error = glGetError();
	if( error != GL_NO_ERROR ) {
		std::string errorTypeMsg = "(unknown gl error)";
		switch( error )
		{
			case GL_INVALID_ENUM:
				errorTypeMsg = "invalid enum";
				break;
			case GL_INVALID_VALUE:
				errorTypeMsg = "invalid value";
				break;
			case GL_INVALID_OPERATION:
				errorTypeMsg = "invalid operation";
				break;
			case GL_STACK_OVERFLOW:
				errorTypeMsg = "stack overflow";
				break;
			case GL_STACK_UNDERFLOW:
				errorTypeMsg = "stack underflow";
				break;
			case GL_OUT_OF_MEMORY:
				errorTypeMsg = "out of memory";
				break;
		}
		#ifdef GL_CHECK_OUTPUT
		X4::error(string("x4glErrorCheck found a problem in file ")
			   + filename
				+ string(" line ")
				+ line
				+ string("; the error code is: ")
				+ intToStr(error)
				+ string(" which translates to ")
				+ errorTypeMsg);
		#endif
	}
}


void X4::x4glCheckFramebuffer(string filename, string line) {
	if (filename=="") filename = "(not specified)";
	if (line=="") line = "(not specified)";

	GLenum error = glCheckFramebufferStatus(GL_FRAMEBUFFER);;
	if( error != GL_FRAMEBUFFER_COMPLETE ) {
		std::string errorTypeMsg = "(unknown gl error)";
		switch( error )
		{
		case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT:
			errorTypeMsg = "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT";
			break;
		case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT:
			errorTypeMsg = "GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT";
			break;
		case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT:
			errorTypeMsg = "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT";
			break;
		case GL_FRAMEBUFFER_UNSUPPORTED:
			errorTypeMsg = "GL_FRAMEBUFFER_UNSUPPORTED";
			break;
		}
		X4::error(string("x4glCheckFramebuffer found a problem in file ")
			+ filename
			+ string(" line ")
			+ line
			+ string("; the error code is: ")
			+ intToStr(error)
			+ string(" which translates to ")
			+ errorTypeMsg);
	}
}

