//---------------------------------------------------------------------------
#ifndef GLDebuggingToolsH
#define GLDebuggingToolsH
//---------------------------------------------------------------------------
#include "x3opengl.h"
//---------------------------------------------------------------------------


namespace X4 {
	/// This function checks for openGL errors and outputs any error codes to the console;
	/// errors are written to the console as error msgs.
	void X3OPENGL_API x4glErrorCheck(string filename, string line);

	/// This function checks for openGL problems with the framebuffer setup and outputs any error codes to the console
	/// (it calls function "glCheckFramebufferStatus()");
	/// errors are written to the console as error msgs.
	void X3OPENGL_API x4glCheckFramebuffer(string filename, string line);
}

#if defined(X4_DEBUG)

	#define X4_GLDBG_STRINGIFY(x) #x
	#define X4_GLDBG_TOSTRING(x) X4_GLDBG_STRINGIFY(x)

	/// this macro reports OpenGL errors with line nr and file, but only in (fast-)debug mode
	#define X4_GL_DBG X4::x4glDbgExt(__FILE__, X4ASSERT_TOSTRING(__LINE__));

	namespace X4 {
		inline void X3OPENGL_API x4glDbgExt(const char* filename, const char* line)
		{
			X4::x4glErrorCheck(filename, line);
		}
	}

	/// this macro reports problems with the framebuffer setup (glCheckFramebufferStatus), but only in (fast-)debug mode
	#define X4_GL_FB_DBG X4::x4glFbDbgExt(__FILE__, X4ASSERT_TOSTRING(__LINE__));

	namespace X4 {
		inline void X3OPENGL_API x4glFbDbgExt(const char* filename, const char* line)
		{
			X4::x4glCheckFramebuffer(filename, line);
		}
	}

#else
	#define X4_GL_DBG
	#define X4_GL_FB_DBG
#endif


#endif
