//---------------------------------------------------------------------------
#ifndef GLEventDisplayerH
#define GLEventDisplayerH
//---------------------------------------------------------------------------
#include "x3openGL.h"
//---------------------------------------------------------------------------
#include "EventDisplayer.h"
#include "BoundingBox.h"
#include "PopupWindow.h"
#include "Object.h"
//---------------------------------------------------------------------------

class QGLWidget;


namespace X4 {

class GLEventDisplayerWidget;
class GLEventObject;
class Camera;

class X3OPENGL_API GLEventConfiguration : public PopupWindow {
   Q_OBJECT
 protected:
	GLEventObject *obj;
 public:
	GLEventConfiguration(QWidget *parent, GLEventObject *obj) : PopupWindow(parent) {this->obj = obj;}
 signals:
	void changed();
};


/** subclasses of this type can be displayed by the GLEventDisplayer */
class X3OPENGL_API GLEventObject : public Object {
	X4_ABSTRACT_CLASS(GLEventObject)
 public:
	/// specify bounding box (for navigation)
	virtual BoundingBox3f getBoundingBox() = 0;
	/// draw gl commands
	virtual void drawGL(QGLWidget *widget, Camera &cam) = 0;
	/// create a window to configure the event displayer, may return NULL
	virtual GLEventConfiguration *createConfigWindow(QWidget *parent) = 0;
	/// updates only the data to be displayed
	virtual void updateData(GLEventObject *other) = 0;
	
	virtual ~GLEventObject() {}
};


/**
	Event displayer that uses a QGLWidget to visualize 3d data
*/
class X3OPENGL_API GLEventDisplayer : public EventDisplayer {
	X4_CLASS( GLEventDisplayer )

 private:

 protected:
	GLEventDisplayerWidget *widget;
	virtual QWidget *createSpecialWidget(QWidget *parent);

 public:
	class SetNewGLEventObject : public EventDisplayerEvent {
	 public:
		/// new self-drawing object; ownership will be transferred to GLEventDisplayer
		/// if updateData is true, changes in the settings are ignored, only data is updated
		GLEventObject *glevo;
		bool updateDataOnly;
		SetNewGLEventObject(GLEventObject *o, bool updateData = true) {glevo = o; updateDataOnly = updateData;}
	};
 
	GLEventDisplayer();

	virtual void displayEvent(const EventDisplayerEvent *e, const char *logfile);

	~GLEventDisplayer();
};


}

#endif
