//---------------------------------------------------------------------------
#ifndef GLGraphDisplayerH
#define GLGraphDisplayerH
//---------------------------------------------------------------------------
#include "x3openGL.h"
//---------------------------------------------------------------------------
#include "GLEventDisplayer.h"
#include "Object.h"
//---------------------------------------------------------------------------
#include "x3openGL/GLGraphDisplayerConfig_ui.h"
//---------------------------------------------------------------------------


namespace X4 {

class GLGraphDisplayerConfig;


class X3OPENGL_API GLGraphDisplayer : public GLEventObject {
	X4_CLASS(GLGraphDisplayer)
 private:
   std::vector< std::vector<Vector3f> > graphs;
   std::vector< Vector4ub > colors;
   bool showZAxis;
   bool showAxis;
	
 public:
	GLGraphDisplayer();
	GLGraphDisplayer(const std::vector< std::vector<Vector3f> > &graphs, const std::vector< Vector4ub > &colors);
	GLGraphDisplayer(const std::vector< std::vector<Vector2f> > &graphs, const std::vector< Vector4ub > &colors);
	GLGraphDisplayer(const std::vector< std::vector<float32> > &graphs, const std::vector< Vector4ub > &colors);

   void addGraph(const std::vector<float32> &graph, const Vector4ub &color);
   void addGraph(const std::vector<Vector2f> &graph, const Vector4ub &color);
   void addGraph(const std::vector<Vector3f> &graph, const Vector4ub &color);
	
	virtual BoundingBox3f getBoundingBox();
	virtual void drawGL(QGLWidget *widget, Camera &cam);
	virtual GLEventConfiguration *createConfigWindow(QWidget *parent);
	virtual void updateData(GLEventObject *other);

	virtual ~GLGraphDisplayer();
	friend class GLGraphDisplayerConfig;
};

class X3OPENGL_API GLGraphDisplayerConfig : public GLEventConfiguration {
	Q_OBJECT
 private:
	Ui_GLGraphDisplayerConfig ui;
 private slots:
   void on_cbShowCoords_stateChanged(int);
 public:
	GLGraphDisplayerConfig(QWidget *parent, GLEventObject *obj);
};

void X3OPENGL_API debugOutputGraphs(std::string name, const std::vector< std::vector<float32> > &graphs, const std::vector<Vector4ub> &colors);
void X3OPENGL_API debugOutputGraphs(std::string name, const std::vector< std::vector<Vector2f> > &graphs, const std::vector<Vector4ub> &colors);
void X3OPENGL_API debugOutputGraphs(std::string name, const std::vector< std::vector<Vector3f> > &graphs, const std::vector<Vector4ub> &colors);
void X3OPENGL_API debugOutputGraph(std::string name, const std::vector<float32> &graph, const Vector4ub color);
void X3OPENGL_API debugOutputGraph(std::string name, const std::vector<Vector2f> &graph, const Vector4ub color);
void X3OPENGL_API debugOutputGraph(std::string name, const std::vector<Vector3f> &graph, const Vector4ub color);

}

#endif
