#ifndef GLMultipleRenderTextureH
#define GLMultipleRenderTextureH
//---------------------------------------------------------------------------
#include "x3openGL.h"
//---------------------------------------------------------------------------
#include "GLObject.h"
//---------------------------------------------------------------------------

namespace X4
{
	class GLTexture;
	class GLObject;
/**
	handles multiple rendertargets 

	Usage:

	GLMultipleRenderTexture * rt = new GLMultipleRenderTexture( glContext );

	rt->init(1280,720,true,false);		// init hd720p sized rendertarget with depthbuffer and without stencil
	rt->addRenderTexture(3,true,false);	// add first render texture with 3 16-bit floats 
	rt->addRenderTexture(4,false,false);// add second render texture with 4 8-bit bytes
	if( !rt->create() )
	{
		// rt could not be created
		...
	}

	rt->activate();	// activate all rendertargets
	...
	rt->deactivate();

	rt->bindAll(1);	// bind as texture (texture stage 1 -> texture 0, texture stage 2 -> texture 1)
	...
	rt->unbindAll();
*/
class X3OPENGL_API GLMultipleRenderTexture : public GLObject
{
private:


	std::vector<GLTexture *>	m_RenderTextures;
	bool		m_HasDepth;
	bool		m_HasStencil;


	enum EStatus
	{
	  None,
	  Active,
	  Bound,
	  PartiallyBound
	};

	EStatus			m_Status;

	struct TBoundTexture
	{
		unsigned id;
		unsigned stage;
	};
	std::vector<TBoundTexture>	m_BoundTextures;

   bool checkStatus();

   GLuint         m_FrameBufferID;
   GLuint         m_DepthRenderBufferID;
   GLuint         m_StencilRenderBufferID;
   card32         m_LastStage;

   card32         m_Width;
   card32		  m_Height;

   GLint          m_OldViewport[4];


public:
	GLMultipleRenderTexture(GLContext * context);
	~GLMultipleRenderTexture(void);

	/// setup rt dimension
	void init(unsigned width, unsigned height, bool hasDepth, bool hasStencil);	

	/// add render texture
	void addRenderTexture(unsigned numComponents, bool isFloat, bool isFloat32=false);

	/// try to create rt
	bool create();

	/// activate all rendertargets
	void activate();

	/// deactivate all rendertargets
	void deactivate();

	/// bind all render textures 
	void bindAll(unsigned firstStage=0);

	/// bind single texture
	void bind(unsigned texid, unsigned stage);

	/// unbind all render textures
	void unbindAll();

	void outputViaImageDisplayer();

	card32 getWidth() const {return m_Width;}
	card32 getHeight() const {return m_Height;}

};

}

#endif