//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "GLRenderTextureCache.h"
//---------------------------------------------------------------------------
#include "X4Assert.h"
#include "GLRenderTextureFBO.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;


GLRenderTextureCache::GLRenderTextureCache( GLContext * context )
{
   m_MyContext = context;
}

GLRenderTextureCache::~GLRenderTextureCache()
{
   clearAll();
}

GLRenderTextureFBO * GLRenderTextureCache::getRenderTexture( card32 width, card32 height, card32 numComponents,
       bool isRect, bool isFloat , bool isFloat32 , bool hasDepth ,
       bool hasStencil  )
{

   card32 desc = getDesc( width, height, numComponents, isRect, isFloat, isFloat32, hasDepth, hasStencil );
   for( RTListIterator iter = m_Unused.begin(); iter != m_Unused.end(); ++iter )
   {
      if( iter->desc == desc )
      {
         x4Assert( iter->rt->isRectangle() == isRect );
         if( isFloat )
         {
            if( isFloat32 )
               x4Assert( iter->rt->isFloat32() );
            else
               x4Assert( iter->rt->isFloat16() );
         }
         else
         {
            x4Assert( !iter->rt->isFloat16() );
            x4Assert( !iter->rt->isFloat32() );
         }

         TRenderTexture tmpRT = *iter;
         m_Unused.erase( iter );
         m_Used.push_back( tmpRT );
         m_IndexMap[tmpRT.rt] = (--m_Used.end());

         return tmpRT.rt;
      }

   }

   // no unused existing rt found -> create new

   TRenderTexture tmpRT;
   tmpRT.desc = desc;
   tmpRT.rt = new GLRenderTextureFBO( m_MyContext );
   card32 flags = 0;
   flags |= isRect ? GLRenderTextureFBO::FLAG_RECTANGLE : 0;
   flags |= (isFloat && !isFloat32)? GLRenderTextureFBO::FLAG_FLOAT16 : 0;
   flags |= (isFloat && isFloat32 )? GLRenderTextureFBO::FLAG_FLOAT32 : 0;
   flags |= hasDepth ? GLRenderTextureFBO::FLAG_DEPTH : 0;
   flags |= hasStencil ? GLRenderTextureFBO::FLAG_STENCIL : 0;

   if( !tmpRT.rt->create( width, height, numComponents, flags ) )
   {
      delete tmpRT.rt;
      return NULL;
   }

   m_Used.push_back( tmpRT );

   //debugOutput << "num cached render textures: " << (mpcard)(m_Used.size() + m_Unused.size()) << "\n";

   m_IndexMap[tmpRT.rt] = (--m_Used.end());
   return tmpRT.rt;
}

void GLRenderTextureCache::freeRenderTexture( GLRenderTextureFBO * rt )
{
   x4Assert( m_IndexMap.find( rt ) != m_IndexMap.end() );

   RTListIterator iter = m_IndexMap[rt];
   TRenderTexture tmpRT = *iter;
   m_Used.erase( iter );
   m_IndexMap.erase( m_IndexMap.find( rt ) );
   m_Unused.push_back( tmpRT );
}

void GLRenderTextureCache::clearUnused()
{
   for( RTListIterator iter = m_Unused.begin(); iter != m_Unused.end(); ++iter )
   {
      delete iter->rt;
   }
   m_Unused.clear();
}

void GLRenderTextureCache::clearAll()
{
   for( RTListIterator iter = m_Used.begin(); iter != m_Used.end(); ++iter )
   {
      delete iter->rt;
   }
   m_Used.clear();
   clearUnused();
}


card32 GLRenderTextureCache::getDesc( card32 width, card32 height, card32 numComponents,
       bool isRect , bool isFloat , bool isFloat32 ,
       bool hasDepth , bool hasStencil )
{
   x4Assert( numComponents <= 4 );
   x4Assert( numComponents > 0 );
   x4Assert( width <= 2048 );
   x4Assert( height <= 2048 );
   // RFFD SNN_ wwww wwww wwww hhhh hhhh hhhh
   card32 ret = 0;
   ret |= isRect ? (1 << 0) : 0;
   ret |= isFloat ? (1 << 1) : 0;
   ret |= isFloat32 ? (1 << 2) : 0;
   ret |= hasDepth ? (1 << 3) : 0;
   ret |= hasStencil ? (1 << 4) : 0;

   ret |= (numComponents-1) << 5;
   ret |= (width - 1) << 8;
   ret |= (height - 1) << 20;

   return ret;
}



void X4::GLRenderTextureCache::deleteRenderTexture( GLRenderTextureFBO * rt )
{
	x4Assert( m_IndexMap.find( rt ) != m_IndexMap.end() );

	RTListIterator iter = m_IndexMap[rt];
	TRenderTexture tmpRT = *iter;
	m_Used.erase( iter );
	m_IndexMap.erase( m_IndexMap.find( rt ) );
	delete rt;
}