//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "GLRenderTextureFBO.h"
//---------------------------------------------------------------------------
#include "GLExtensions.h"
#include "GLTexture.h"
#include "GLContext.h"
#include "ImageDisplayer.h"
#include "Random.h"
#include "RGBAImage.h"
//---------------------------------------------------------------------------
 

using namespace X4;

GLRenderTextureFBO::GLRenderTextureFBO( GLContext * context )
 : GLObject( context )
{
   m_Texture = NULL;
   m_FrameBufferID = 0;
   m_Status = None;
   m_DepthRenderBufferID = 0;
   m_StencilRenderBufferID = 0;
	m_PureDepthTexture = false;
	m_useMipmapping = false;
	m_LastStage = 0;
}

GLRenderTextureFBO::~GLRenderTextureFBO()
{
   delete m_Texture;

   if( isValid() )
      if( m_FrameBufferID )
         glDeleteFramebuffersEXT( 1, &m_FrameBufferID );
}

bool GLRenderTextureFBO::create( card32 width, card32 height, card32 numComponents, card32 flags )
{
   // check everything
   x4Assert( m_Texture == NULL );
   x4Assert( m_FrameBufferID == 0 );

#if 0
	// NOTE: nowadays 2D texture can be of any size, not only power of two
   if( (flags & FLAG_RECTANGLE) == 0 )
   {
      card32 testw = width;
      while( testw > 1 )
      {
         x4Assert( (testw & 1) == 0 );
         testw >>= 1;
      }
      card32 testh = height;
      while( testh > 1 )
      {
         x4Assert( (testh & 1) == 0 );
         testh >>= 1;
      }
   }
#endif

   m_Width = width;
   m_Height = height;
   m_NumComponents = numComponents;
   m_Flags = flags;


   // generate texture
   if( flags & FLAG_RECTANGLE )
   {
      GLTextureRect* tex = new GLTextureRect( getContext() );

	  if( (flags & FLAG_FLOAT16) || (flags & FLAG_FLOAT32))
		  tex->createFloatTextureFromMemory( width, height, numComponents, NULL, (flags & FLAG_FLOAT32 ) ? true : false);
	  else
		  tex->createFromMemory( width, height, numComponents, NULL);

	  m_Texture = tex;
	  //glTexParameteri( m_Texture->getTextureTarget(), GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
   }else
   {
	   GLTexture2D* tex = new GLTexture2D(getContext());

	   if( (flags & FLAG_FLOAT16) || (flags & FLAG_FLOAT32))
		   tex->createFromFloatMemory( width, height, numComponents, NULL, (flags & FLAG_FLOAT32 ) ? true : false, flags & FLAG_MIPMAP );
	   else
		   tex->createFromMemory( width, height, numComponents, NULL, flags & FLAG_MIPMAP );

	   m_Texture = tex;
   }


	if (flags & FLAG_MIPMAP)
	{
		m_useMipmapping = true;
		glGenerateMipmap(m_Texture->getTextureTarget());
	}

   glGenFramebuffersEXT( 1, &m_FrameBufferID );
   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, m_FrameBufferID );

   // attach texture to buffer
   glFramebufferTexture2DEXT(    GL_FRAMEBUFFER_EXT,
                                 GL_COLOR_ATTACHMENT0_EXT,
                                 m_Texture->getTextureTarget(),
                                 m_Texture->getTextureID(), 0);

   // attach depth buffer
   if( flags & FLAG_DEPTH )
   {
      glGenRenderbuffersEXT( 1, &m_DepthRenderBufferID );
      glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, m_DepthRenderBufferID);
      glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,
                               GL_DEPTH_COMPONENT24, width, height);
      glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT,
                                    GL_DEPTH_ATTACHMENT_EXT,
                                    GL_RENDERBUFFER_EXT,
                                    m_DepthRenderBufferID);

   }

   // attach stencil buffer
   if( flags & FLAG_STENCIL )
   {
      glGenRenderbuffersEXT( 1, &m_StencilRenderBufferID );
      glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, m_StencilRenderBufferID);
      glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,
                               GL_STENCIL_INDEX, width, height);
      glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT,
                                    GL_STENCIL_ATTACHMENT_EXT,
                                    GL_RENDERBUFFER_EXT,
                                    m_StencilRenderBufferID);

   }

   if( !checkStatus() )
      return false;

   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

   return true;
}


bool GLRenderTextureFBO::createDepthTexture( card32 width, card32 height ) 
{
	m_Flags = 0;
	m_PureDepthTexture = true;
   card32 testw = width;
   while( testw > 1 )
   {
      if( (testw & 1) != 0 )
			return false;
      testw /= 2;
   }
   card32 testh = height;
   while( testh > 1 )
   {
      if( (testh & 1) != 0 )
			return false;
      testh /= 2;
   }

   m_Width = width;
   m_Height = height;
   m_NumComponents = 0;
   m_Flags = FLAG_DEPTH;

	m_Texture = new GLTexture2D( getContext() );
	m_Texture->bind();
	glTexImage2D( m_Texture->getTextureTarget(), 0, GL_DEPTH_COMPONENT, width, height, 0, GL_DEPTH_COMPONENT,
		GL_FLOAT, NULL );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
	m_Texture->unbind();

	glGenFramebuffersEXT( 1, &m_FrameBufferID );
   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, m_FrameBufferID );

   // attach texture to buffer
   glFramebufferTexture2DEXT(    GL_FRAMEBUFFER_EXT,
                                 GL_DEPTH_ATTACHMENT_EXT,
                                 m_Texture->getTextureTarget(),
                                 m_Texture->getTextureID(), 0);
	glDrawBuffer(GL_NONE);
	glReadBuffer(GL_NONE);

   if( !checkStatus() )
      return false;

   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

   return true;
}




bool GLRenderTextureFBO::checkStatus()
{
    GLenum status;
    status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
    if( status ==  GL_FRAMEBUFFER_COMPLETE_EXT || status == 0 /*ATI, dont ask!*/ )
         return true;

    return false;
}

void GLRenderTextureFBO::activate()
{
   x4Assert( m_Status == None );
   x4Assert( m_Texture != NULL );

   glGetIntegerv( GL_VIEWPORT, m_OldViewport );

   m_Status = Active;
   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, m_FrameBufferID );

   glViewport(0,0, m_Width, m_Height);

}

void GLRenderTextureFBO::deactivate()
{
   x4Assert( m_Status == Active );
   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

   // automatically generate mipmaps
   if (m_useMipmapping)
   {
	   m_Texture->bind(m_LastStage);
	   //glHint(GL_GENERATE_MIPMAP_HINT, GL_NICEST);
	   glGenerateMipmapEXT(m_Texture->getTextureTarget());
	   m_Texture->unbind( m_LastStage );
   }

   m_Status = None;
   glViewport(m_OldViewport[0], m_OldViewport[1], m_OldViewport[2], m_OldViewport[3]);
}

void GLRenderTextureFBO::bind( card32 stage )
{
   x4Assert( m_Status == None );
   x4Assert( m_Texture != NULL );

   m_LastStage = stage;
   m_Texture->bind( stage );
   m_Status = Bound;
}

void GLRenderTextureFBO::unbind()
{
   x4Assert( m_Status == Bound );
   m_Texture->unbind( m_LastStage );
   m_Status = None;
}

GLenum GLRenderTextureFBO::getTextureTarget()
{
   x4Assert( m_Texture != NULL );
   return m_Texture->getTextureTarget();
}

RGBAImage * GLRenderTextureFBO::getRGBAImage()
{
	bool wasActive = isActive();
	bool wasBound = isBound();

	if( isActive() )
		deactivate();

	RGBAImage * ret = new RGBAImage;
	ret->resize( getWidth(), getHeight() );
	if( !isBound())
		bind(0);
	glGetTexImage( getTextureTarget(),0, GL_RGBA, GL_UNSIGNED_BYTE,
		ret->getPixels() );
	if( !wasBound )
		unbind();

	if( wasActive )
		activate();
	
	return ret;
}

RGBAFloat32Image * X4::GLRenderTextureFBO::getRGBAFloat32Image()
{
	bool wasActive = isActive();
	bool wasBound = isBound();

	if( isActive() )
		deactivate();

	RGBAFloat32Image * ret = new RGBAFloat32Image;
	ret->resize( getWidth(), getHeight() );
	if( !isBound())
		bind(0);
	glGetTexImage( getTextureTarget(),0, GL_RGBA, GL_FLOAT,
		ret->getPixels() );
	if( !wasBound )
		unbind();

	if( wasActive )
		activate();

	return ret;
}
void GLRenderTextureFBO::outputViaImageDisplayer(const std::string &name) 
{
	RGBAFloat32Image * img = getRGBAFloat32Image();
	ImageDisplayer::DisplayRGBAFloat32Image ev(img);
	debugOutput.displayEvent(ImageDisplayer::getClass(), name, &ev);
	
	delete img;
}

bool X4::GLRenderTextureCube::checkStatus()
{
	GLenum status;
	status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
	if( status ==  GL_FRAMEBUFFER_COMPLETE_EXT )
		return true;

	return false;
}

X4::GLRenderTextureCube::GLRenderTextureCube( GLContext * context )
:GLObject(context)
{
	m_FrameBufferID = 0;
	m_Status = None;
}

X4::GLRenderTextureCube::~GLRenderTextureCube()
{
	if( isValid())
		if( m_FrameBufferID )
			glDeleteFramebuffersEXT( 1, &m_FrameBufferID );
}

bool X4::GLRenderTextureCube::createDepthCube( card32 size )
{
	m_Size = size;
	glGenFramebuffersEXT (1, &m_FrameBufferID);
	glBindFramebufferEXT (GL_FRAMEBUFFER_EXT, m_FrameBufferID);

	//glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,	GL_DEPTH_COMPONENT, size, size);
	//glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT,
	//	GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, m_FrameBufferID);

	glGenTextures(1, &m_TextureID);
	glBindTexture(GL_TEXTURE_CUBE_MAP, m_TextureID);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	for( unsigned side=0;side<6;side++ )
	{
		std::vector<float> test(size*size);
		for( unsigned j=0;j<test.size();j++ )
			test[j] = rnd01();
		glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X+side, 0, 0x8818,
			size, size, 0,
			GL_LUMINANCE, GL_FLOAT, &test[0]);
	}

	for( unsigned side=0;side<6;side++ )
	{
		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
			GL_TEXTURE_CUBE_MAP_POSITIVE_X+side, m_TextureID, 0);

		GLuint depthrb;
		glGenRenderbuffersEXT( 1, &depthrb );
		glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, depthrb);
		glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,
							   GL_DEPTH_COMPONENT, size, size);
		glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT,
									GL_DEPTH_ATTACHMENT_EXT,
									GL_RENDERBUFFER_EXT,
									depthrb);
	}


	//glDrawBuffer(GL_NONE);
	//glReadBuffer(GL_NONE);

	glBindTexture(GL_TEXTURE_CUBE_MAP,0);

	if( !checkStatus())
		return false;

	glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
	return true;
}

void X4::GLRenderTextureCube::bind( unsigned stage/*=0*/ )
{
	if( m_Status != None)
		throw X4Exception("GLRenderTextureCube::bind() - invalid state");

	m_LastStage = stage;
	m_Status = Bound;
	glActiveTextureARB(GL_TEXTURE0_ARB + stage);
	glBindTexture( GL_TEXTURE_CUBE_MAP, m_TextureID);
}

void X4::GLRenderTextureCube::unbind()
{
	if( m_Status != Bound)
		throw X4Exception("GLRenderTextureCube::unbind() - invalid state");

	m_Status = None;

	glActiveTextureARB(GL_TEXTURE0_ARB + m_LastStage);
	glBindTexture(GL_TEXTURE_CUBE_MAP,0);
}

void X4::GLRenderTextureCube::activate( unsigned side )
{
	if( m_Status != None)
		throw X4Exception("GLRenderTextureCube::activate() - invalid state");

	m_Status = Active;
	glGetIntegerv( GL_VIEWPORT, m_OldViewport );

	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_FrameBufferID);
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
		GL_TEXTURE_CUBE_MAP_POSITIVE_X+side, m_TextureID, 0);
	glViewport(0,0,m_Size,m_Size);

}

void X4::GLRenderTextureCube::deactivate()
{
	if( m_Status != Active)
		throw X4Exception("GLRenderTextureCube::deactivate() - invalid state");

	m_Status = None;

	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	glViewport(m_OldViewport[0],m_OldViewport[1],m_OldViewport[2],m_OldViewport[3]);

}

void X4::GLRenderTextureCube::outputViaImageDisplayer( const std::string &name )
{
	for( unsigned i=0;i<6;i++ )
	{
		RGBAFloat32Image * img = getRGBAFloat32Image(i);
		ImageDisplayer::DisplayRGBAFloat32Image ev(img);
		debugOutput.displayEvent(ImageDisplayer::getClass(), name + intToStr(i), &ev);
		delete img;
	}


}

RGBAFloat32Image * X4::GLRenderTextureCube::getRGBAFloat32Image( unsigned side )
{
	std::vector<float> depthImage(m_Size *  m_Size);
	bind(0);
	glGetTexImage( GL_TEXTURE_CUBE_MAP_POSITIVE_X+side ,0, GL_LUMINANCE, GL_FLOAT,
		&depthImage[0] );
	unbind();

	RGBAFloat32Image * ret = new RGBAFloat32Image;
	ret->resize( m_Size, m_Size );
	Vector4f * retPtr = ret->getPixels();
	for( unsigned i=0;i<depthImage.size();i++ )
		retPtr[i] = makeVector4f(1,1,1,1) * depthImage[i];
	return ret;	
}

void GLRenderTextureFBO::getRGBAFloat32Image(RGBAFloat32Image * img) 
{
	bool wasActive = isActive();
	bool wasBound = isBound();

	if( isActive() )
		deactivate();

	if( img->getSizeX() != getWidth() || img->getSizeY() != getHeight() )
	{
		img->resize( getWidth(), getHeight() );
	}
	if( !isBound())
		bind(0);
	glGetTexImage( getTextureTarget(),0, GL_RGBA, GL_FLOAT,
		img->getPixels() );
	if( !wasBound )
		unbind();

	if( wasActive )
		activate();

}
