//---------------------------------------------------------------------------
#ifndef GLSimpleObjectsH
#define GLSimpleObjectsH
//---------------------------------------------------------------------------
#include "x3openGL.h"
//---------------------------------------------------------------------------
#include <vector>
#include "LinearAlgebra.h"
#include "BoundingBox.h"
#include "Polygon.h"
//---------------------------------------------------------------------------

namespace X4
{

class GLContext;
class GLSimpleObjectContainer;

class X3OPENGL_API GLSimpleObject
{
private:
   card32   m_ClassID;  // 0 = not specified
public:

   GLSimpleObject() {m_ClassID = 0;}
   virtual ~GLSimpleObject() {}

   virtual void draw( GLContext * context, GLSimpleObjectContainer * container ) = 0;

   card32 getClassID() const {return m_ClassID;}
   void setClassID( card32 id ) {m_ClassID = id;}

};

class X3OPENGL_API GLSimpleObjectContainer
{
private:
   std::vector<GLSimpleObject*>    m_Objects;

public:

   ~GLSimpleObjectContainer();

   /// draws every object with classID ( classID == 0 => draw every object with every id)
   void drawObjects( GLContext * context, card32 classID = 0 );

   /// deletes all objects with classID ( or every object if classID == 0 )
   void clear(card32 classID=0);

   /// add object
   void addObject( GLSimpleObject * obj ) {m_Objects.push_back( obj );}

   /// add simple box
   void addBox( const BoundingBox3f &bb, card32 id = 0 );

   // add triangle
   void addTriangle( const Vector3f &v1,const Vector3f &v2,const Vector3f &v3, card32 id = 0, bool useNormalAsColor=false );

   // add line
   void addLine( const Vector3f &v1, const Vector3f &v2, card32 id = 0 );
};

// ---------------------------------------------------------------------------------------------
//          SimpleObject Derivates
// ---------------------------------------------------------------------------------------------
class X3OPENGL_API GLSimpleBox : public GLSimpleObject
{
private:
   BoundingBox3f  m_Box;
public:

   void setup( const Vector3f &lowerCorner, const Vector3f &upperCorner );
   void setup( const BoundingBox3f &bb );

   virtual void draw( GLContext * context, GLSimpleObjectContainer * container );
};

class X3OPENGL_API GLSimpleLines : public GLSimpleObject
{
private:
   std::vector<Vector3f> m_Points;
public:

   void addLine( const Vector3f v1,const Vector3f v2 );
   virtual void draw( GLContext * context, GLSimpleObjectContainer * container );
};

class X3OPENGL_API GLSimpleMesh : public GLSimpleObject
{
private:
   std::vector< SimplePolygon >    m_Mesh;
   bool                       m_PolygonMode;
   SimplePolygon              m_ActPoly;
   bool                       m_UseNormalAsColor;
public:

   GLSimpleMesh() {m_PolygonMode = false;m_UseNormalAsColor = false;}

   virtual void draw( GLContext * context, GLSimpleObjectContainer * container );

   void clear() {m_Mesh.clear();}
   void addPolygon( const SimplePolygon &p ) {m_Mesh.push_back(p);}
   void addTriangle( const Vector3f &v1,const Vector3f &v2,const Vector3f &v3 );
   void addQuad( const Vector3f &v1,const Vector3f &v2,const Vector3f &v3,const Vector3f &v4 );

   void setPolygonMode( bool b ) {m_PolygonMode = b;}
   void setUseNormalAsColor( bool b ) {m_UseNormalAsColor = b;}
   bool getUseNormalAsColor() {return m_UseNormalAsColor;}


   void beginPoly();
   void addPolyVertex( const Vector3f &v );
   void endPoly();
};

}

#endif
