#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "Tex2dVolumeRenderer.h"
//---------------------------------------------------------------------------
#include "RegularVolume.h"
#include "RegularVolume.inline.h"
#include "MathTools.h"
#include "MathTools.inline.h"
#include "X4Exception.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------
#ifndef GL_RGBA32F_ARB
   #define GL_RGBA32F_ARB 0x8814
#endif


IMPLEMENT_X4_CLASS( Tex2dVolumeRenderer ,0) {
	BEGIN_CLASS_INIT( Tex2dVolumeRenderer );
}

// --

static inline StaticVector<float32, 1> tvc(float32 v) {
   StaticVector<float32, 1> r;
   r[0] = v;
   return r;
}

Vector4ub Tex2dVolumeRenderer::mapping2(void* ctxt, const StaticVector<float32, 2> &in) {
   Tex2dVolumeRenderer *cThis = (Tex2dVolumeRenderer*)ctxt;
   return makeVector4ub(cThis->mappingTmp(cThis->mappingContextTmp, tvc(in[0]))[3],
                        cThis->mappingTmp(cThis->mappingContextTmp, tvc(in[1]))[3],
                        0, 
                        16);
}
Vector4ub Tex2dVolumeRenderer::mapping3(void* ctxt, const StaticVector<float32, 3> &in) {
   Tex2dVolumeRenderer *cThis = (Tex2dVolumeRenderer*)ctxt;
   return makeVector4ub(cThis->mappingTmp(cThis->mappingContextTmp, tvc(in[0]))[3],
                        cThis->mappingTmp(cThis->mappingContextTmp, tvc(in[1]))[3],
                        cThis->mappingTmp(cThis->mappingContextTmp, tvc(in[2]))[3],
                        16);
}
Vector4ub Tex2dVolumeRenderer::mapping4(void* ctxt, const StaticVector<float32, 4> &in) {
   Tex2dVolumeRenderer *cThis = (Tex2dVolumeRenderer*)ctxt;
   return makeVector4ub(cThis->mappingTmp(cThis->mappingContextTmp, tvc(in[0]))[3],
                        cThis->mappingTmp(cThis->mappingContextTmp, tvc(in[1]))[3],
                        cThis->mappingTmp(cThis->mappingContextTmp, tvc(in[2]))[3],
                        cThis->mappingTmp(cThis->mappingContextTmp, tvc(in[3]))[3]);
}


Tex2dVolumeRenderer::Tex2dVolumeRenderer() {
	mappingContextTmp = NULL;
	mappingTmp = NULL;
}

void Tex2dVolumeRenderer::computeOrder(const Vector3f &viewDirection, card32 &dir, int32 &sign) {
	float32 maxDirVal = fabs(viewDirection[0]);
	dir = 0;
	if (fabs(viewDirection[1]) > maxDirVal) {
		maxDirVal = fabs(viewDirection[1]);
		dir = 1;
	}
	if (fabs(viewDirection[2]) > maxDirVal) {
		maxDirVal = fabs(viewDirection[2]);
		dir = 2;
	}
	if (viewDirection[dir] < 0) sign = -1; else sign = 1;
}

void Tex2dVolumeRenderer::prepareTextures(
	RegularVolume *vol, 
	Vector3f viewDirection, 
	const std::vector<std::string> &channels, 
	TransferFunction mapping, void* mappingContext) 
{
   std::string channel = channels[0];
	disposeTextures();
	x4Assert(vol != NULL);

	mappingContextTmp = mappingContext;
	mappingTmp = mapping;
	
	for (unsigned ch=0; ch<channels.size(); ch++) {
      channelTextures.push_back( std::vector<GLuint>() );
      texOffsets.push_back( std::vector<mpcard>() );
	
	   std::string channel = channels[ch];
   	
	   for (card32 dir=0; dir<3; dir++) {
		   texOffsets[ch].push_back(channelTextures[ch].size());
   		
		   card32 dir_u, dir_v;
   		
		   if (dir ==0) {
			   dir_u = 1;
			   dir_v = 2;
		   } else if (dir == 1) {
			   dir_u = 0;
			   dir_v = 2;
		   } else if (dir == 2) {
			   dir_u = 0;
			   dir_v = 1;
		   }
		   card32 sign;
		   if (viewDirection[dir] < 0) sign = -1; else sign = 1;
   		
		   card32 textureSize_u_used = vol->getExtends()[dir_u];
		   card32 textureSize_v_used = vol->getExtends()[dir_v];
		   card32 textureSlices_n = vol->getExtends()[dir];
		   x4Assert(textureSize_u_used >= 1);
		   x4Assert(textureSize_v_used >= 1);
		   if (textureSize_u_used < 1 || textureSize_v_used < 1) return;
		   card32 textureSize_u_allocated = 2 << simpleLog2(textureSize_u_used-1);
		   card32 textureSize_v_allocated = 2 << simpleLog2(textureSize_v_used-1);

		   x4Assert((float)textureSize_u_allocated / (float) textureSize_u_used <= 2.0f);
		   x4Assert((float)textureSize_v_allocated / (float) textureSize_v_used <= 2.0f);
		   x4Assert(textureSize_u_allocated >= textureSize_u_used);
		   x4Assert(textureSize_v_allocated >= textureSize_v_used);

		   std::vector<Vector4ub> data(textureSize_u_allocated * textureSize_v_allocated, NULL_VECTOR4UB);
		   std::vector<Vector4ub> colorData(textureSize_u_used*textureSize_v_used);
   		
		   VAD channelVAD = vol->getDescr()->getAttribute(channel);
		   if (channelVAD.dataFormat != VAD::DATA_FORMAT_FLOAT32) {
		      throw X4Exception("Tex2dVolumeRenderer::prepareTextures - only float channels are supported.");
		   }
		   card32 dim = channelVAD.numComponents;
		   if (dim ==0 || dim > 4) {
		      throw X4Exception("Tex2dVolumeRenderer::prepareTextures - only channel dimension 1..4 is supported.");
		   }
   		
		   for (card32 i=0; i<textureSlices_n; i++) {
            switch (dim) {
			    case 1: {
	            if (dir == 0) {
		            vol->mapChannel<float32, 1, card8, 4>(
			            colorData, channel, mapping, mappingContext, 
			            i, i, 
			            0xFFFFFFFF, 0xFFFFFFFF, 
			            0xFFFFFFFF, 0xFFFFFFFF);
	            } else if (dir == 1) {
		            vol->mapChannel<float32, 1, card8, 4>(
			            colorData, channel, mapping, mappingContext, 
			            0xFFFFFFFF, 0xFFFFFFFF, 
			            i, i, 
			            0xFFFFFFFF, 0xFFFFFFFF);
	            } else if (dir == 2) {
		            vol->mapChannel<float32, 1, card8, 4>(
			            colorData, channel, mapping, mappingContext, 
			            0xFFFFFFFF, 0xFFFFFFFF, 
			            0xFFFFFFFF, 0xFFFFFFFF,
			            i, i);
	            } else {
		            x4Assert(false);
	            }
		         break;
		       }
			    case 2: {
			      if (dir == 0) {
				      vol->mapChannel<float32, 2, card8, 4>(
					      colorData, channel, mapping2, this, 
					      i, i, 
					      0xFFFFFFFF, 0xFFFFFFFF, 
					      0xFFFFFFFF, 0xFFFFFFFF);
			      } else if (dir == 1) {
				      vol->mapChannel<float32, 2, card8, 4>(
					      colorData, channel, mapping2, this, 
					      0xFFFFFFFF, 0xFFFFFFFF, 
					      i, i, 
					      0xFFFFFFFF, 0xFFFFFFFF);
			      } else if (dir == 2) {
				      vol->mapChannel<float32, 2, card8, 4>(
					      colorData, channel, mapping2, this, 
					      0xFFFFFFFF, 0xFFFFFFFF, 
					      0xFFFFFFFF, 0xFFFFFFFF,
					      i, i);
			      } else {
				      x4Assert(false);
			      }
			      break;
			    }
			    case 3: {
			      if (dir == 0) {
				      vol->mapChannel<float32, 3, card8, 4>(
					      colorData, channel, mapping3, this, 
					      i, i, 
					      0xFFFFFFFF, 0xFFFFFFFF, 
					      0xFFFFFFFF, 0xFFFFFFFF);
			      } else if (dir == 1) {
				      vol->mapChannel<float32, 3, card8, 4>(
					      colorData, channel, mapping3, this, 
					      0xFFFFFFFF, 0xFFFFFFFF, 
					      i, i, 
					      0xFFFFFFFF, 0xFFFFFFFF);
			      } else if (dir == 2) {
				      vol->mapChannel<float32, 3, card8, 4>(
					      colorData, channel, mapping3, this, 
					      0xFFFFFFFF, 0xFFFFFFFF, 
					      0xFFFFFFFF, 0xFFFFFFFF,
					      i, i);
			      } else {
				      x4Assert(false);
			      }
			      break;
			    }
			    case 4: {
			      if (dir == 0) {
				      vol->mapChannel<float32, 4, card8, 4>(
					      colorData, channel, mapping4, this, 
					      i, i, 
					      0xFFFFFFFF, 0xFFFFFFFF, 
					      0xFFFFFFFF, 0xFFFFFFFF);
			      } else if (dir == 1) {
				      vol->mapChannel<float32, 4, card8, 4>(
					      colorData, channel, mapping4, this, 
					      0xFFFFFFFF, 0xFFFFFFFF, 
					      i, i, 
					      0xFFFFFFFF, 0xFFFFFFFF);
			      } else if (dir == 4) {
				      vol->mapChannel<float32, 4, card8, 4>(
					      colorData, channel, mapping4, this, 
					      0xFFFFFFFF, 0xFFFFFFFF, 
					      0xFFFFFFFF, 0xFFFFFFFF,
					      i, i);
			      } else {
				      x4Assert(false);
			      }
			      break;
			    }
			    default: {x4Assert(false);}
			   }
			   for (card32 v=0; v<textureSize_v_used; v++) {
				   for (card32 u=0; u<textureSize_u_used; u++) {
				      Vector4ub vec = colorData[u+v*textureSize_u_used];
				      data[u+v*textureSize_u_allocated] = vec;
				   }
			   }

			   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
			   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
			   glTexImage2D( GL_TEXTURE_2D, 0, 4, textureSize_u_allocated, textureSize_v_allocated, 0, GL_RGBA, GL_UNSIGNED_BYTE, (&(data[0])) );
			   channelTextures[ch].push_back(0);
			   glGenTextures(1, &(channelTextures[ch].back()));
			   glBindTexture(GL_TEXTURE_2D, channelTextures[ch].back());
		   }
	   }
	}
   mappingContextTmp = NULL;
   mappingTmp = NULL;
}

void Tex2dVolumeRenderer::drawVolume(RegularVolume *vol, Vector3f viewDirection, 
                                     std::string channel, TransferFunction mapping, void *mappingContext) 
{
   std::vector<std::string> cList;
   cList.push_back(channel);
   drawVolume(vol, viewDirection, cList, mapping, mappingContext);
}

void Tex2dVolumeRenderer::drawVolume(RegularVolume *vol, Vector3f viewDirection, 
                                     const std::vector<std::string> &channels, TransferFunction mapping, void *mappingContext) 
{
	if (vol == NULL) return;
	if (vol->getWidth() <= 0 || vol->getHeight() <= 0 || vol->getDepth() <= 0) return;
	float32 maxDirVal = fabs(viewDirection[0]);

	card32 dir = 0;
	card32 dir_u = 1;
	card32 dir_v = 2;
	if (fabs(viewDirection[1]) > maxDirVal) {
		maxDirVal = fabs(viewDirection[1]);
		dir = 1;
		dir_u = 0;
		dir_v = 2;
	}
	if (fabs(viewDirection[2]) > maxDirVal) {
		maxDirVal = fabs(viewDirection[2]);
		dir = 2;
		dir_u = 0;
		dir_v = 1;
	}
	card32 sign;
	if (viewDirection[dir] < 0) sign = -1; else sign = 1;
	
	card32 textureSize_u_used = vol->getExtends()[dir_u];
	card32 textureSize_v_used = vol->getExtends()[dir_v];
	card32 textureSlices_n = vol->getExtends()[dir];
	x4Assert(textureSize_u_used >= 1);
	x4Assert(textureSize_v_used >= 1);
	if (textureSize_u_used < 1 || textureSize_v_used < 1) return;
	card32 textureSize_u_allocated = 2 << simpleLog2(textureSize_u_used-1);
	card32 textureSize_v_allocated = 2 << simpleLog2(textureSize_v_used-1);

	x4Assert((float)textureSize_u_allocated / (float) textureSize_u_used <= 2.0f);
	x4Assert((float)textureSize_v_allocated / (float) textureSize_v_used <= 2.0f);
	x4Assert(textureSize_u_allocated >= textureSize_u_used);
	x4Assert(textureSize_v_allocated >= textureSize_v_used);

	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	//glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glDepthMask(0);
	card32 newDir;
	int32 newSign;
	computeOrder(viewDirection, newDir, newSign);
	if (channelTextures.size() != channels.size() || dir != newDir || newSign != sign) {
		prepareTextures(vol, viewDirection, channels, mapping, mappingContext);
	}

	BoundingBox3f bb = vol->getBoundingBox();

	const card32 n = vol->getExtends()[dir];
	const float32 MU = (float32)textureSize_u_used / (float32)textureSize_u_allocated;
	const float32 MV = (float32)textureSize_v_used / (float32)textureSize_v_allocated;
	for (card32 i=0; i<n; i++) {
		card32 i_use;
		if (sign == 1) {
			i_use = n-i-1;
		} else {
			i_use = i;
		}
		float32 ifl = (float32)i_use/(float32)n;
		
		for (unsigned ch=0; ch<channelTextures.size(); ch++) {
         glActiveTextureARB(GL_TEXTURE0_ARB + ch);
		   glBindTexture(GL_TEXTURE_2D, i_use+texOffsets[ch][dir]);
		   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
      }
      glActiveTextureARB(GL_TEXTURE0_ARB);
      
      
		if (dir == 0) {
			glBegin(GL_QUADS);
			glTexCoord2f(0,0);
			glVertex3f(bb.lowerCorner[0] + bb.getSideLength(0)*ifl, bb.lowerCorner[1], bb.lowerCorner[2]);
			glTexCoord2f(0,MV);
			glVertex3f(bb.lowerCorner[0] + bb.getSideLength(0)*ifl, bb.lowerCorner[1], bb.upperCorner[2]);
			glTexCoord2f(MU,MV);
			glVertex3f(bb.lowerCorner[0] + bb.getSideLength(0)*ifl, bb.upperCorner[1], bb.upperCorner[2]);
			glTexCoord2f(MU,0);
			glVertex3f(bb.lowerCorner[0] + bb.getSideLength(0)*ifl, bb.upperCorner[1], bb.lowerCorner[2]);
			glEnd();
		} else if (dir == 1) {
			glBegin(GL_QUADS);
			glTexCoord2f(0,0);
			glVertex3f(bb.lowerCorner[0], bb.lowerCorner[1] + bb.getSideLength(1)*ifl, bb.lowerCorner[2]);
			glTexCoord2f(0,MV);
			glVertex3f(bb.lowerCorner[0], bb.lowerCorner[1] + bb.getSideLength(1)*ifl, bb.upperCorner[2]);
			glTexCoord2f(MU,MV);
			glVertex3f(bb.upperCorner[0], bb.lowerCorner[1] + bb.getSideLength(1)*ifl, bb.upperCorner[2]);
			glTexCoord2f(MU,0);																		
			glVertex3f(bb.upperCorner[0], bb.lowerCorner[1] + bb.getSideLength(1)*ifl, bb.lowerCorner[2]);
			glEnd();
		} else if (dir == 2) {
			glBegin(GL_QUADS);
			glTexCoord2f(0,0);
			glVertex3f(bb.lowerCorner[0], bb.lowerCorner[1], bb.lowerCorner[2] + bb.getSideLength(2)*ifl);
			glTexCoord2f(0,MV);
			glVertex3f(bb.lowerCorner[0], bb.upperCorner[1], bb.lowerCorner[2] + bb.getSideLength(2)*ifl);
			glTexCoord2f(MU,MV);
			glVertex3f(bb.upperCorner[0], bb.upperCorner[1], bb.lowerCorner[2] + bb.getSideLength(2)*ifl);
			glTexCoord2f(MU,0);
			glVertex3f(bb.upperCorner[0], bb.lowerCorner[1], bb.lowerCorner[2] + bb.getSideLength(2)*ifl);
			glEnd();
		}
	}
	glPopAttrib();
}

void Tex2dVolumeRenderer::disposeTextures() {
   for (unsigned ch=0; ch<channelTextures.size(); ch++) {
	   if (channelTextures[ch].size() > 0) {
		   glDeleteTextures((GLsizei)channelTextures[ch].size(), &(channelTextures[ch][0]));
	   }
   }
	channelTextures.clear();
	texOffsets.clear();
}																  

Tex2dVolumeRenderer::~Tex2dVolumeRenderer() {}

