//---------------------------------------------------------------------------
#include "stdafx.h"
 //---------------------------------------------------------------------------
#include "CubeTextureAttachment.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"

#include "ObjectClassProperty.h"
#include "BooleanClassProperty.h"
#include "GLContext.h"
#include "GLTexture.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra.inline.h"
#include "CubeMap.h"
#include "CubeMap.inline.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* CubeTextureAttachment::CubeTextureAttachmentMetaClass = NULL;

const MetaClass* CubeTextureAttachment::getInstanceClass() const {
   return getClass();
}

Object* CubeTextureAttachment::newInstance() {
   return new CubeTextureAttachment;
}

const MetaClass* CubeTextureAttachment::getClass() {
   if (CubeTextureAttachmentMetaClass==NULL) {
      CubeTextureAttachmentMetaClass = new MetaClass("CubeTextureAttachment", "no description.",
                                  newInstance, sizeof(CubeTextureAttachment));
   }
   return CubeTextureAttachmentMetaClass;
}

void CubeTextureAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   CubeTextureAttachmentMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(CubeTextureAttachment);
   ADD_OBJECT_PROP(m_CubeMap,0, RGBACubeMap::getClass(), true )
   ADD_BOOLEAN_PROP(m_MagFilterLinear,0)
   ADD_BOOLEAN_PROP(m_MinFilterLinear,0)
}

void CubeTextureAttachment::shutdown() {
   delete CubeTextureAttachmentMetaClass;
}


// ----


MetaClass* CubeFunction3toN::CubeFunction3toNMetaClass = NULL;

const MetaClass* CubeFunction3toN::getInstanceClass() const {
   return getClass();
}

const MetaClass* CubeFunction3toN::getClass() {
   if (CubeFunction3toNMetaClass==NULL) {
      CubeFunction3toNMetaClass = new MetaClass("CubeFunction3toN", "no description.",
                                  NULL, sizeof(CubeFunction3toN));
   }
   return CubeFunction3toNMetaClass;
}

void CubeFunction3toN::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   CubeFunction3toNMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT(CubeFunction3toN);
   ADD_CARD32_PROP(m_ImageDimension,0);

   Card32ClassProperty * prop=(Card32ClassProperty*)(CubeFunction3toNMetaClass->getProperty(CubeFunction3toNMetaClass->getNumProperties()-1));
   prop->setMinimumValue( 1 );
   prop->setMaximumValue( 4 );
}

void CubeFunction3toN::shutdown() {
   delete CubeFunction3toNMetaClass;
}


// ----

MetaClass* CubeTextureFunctionAttachment::CubeTextureFunctionAttachmentMetaClass = NULL;

const MetaClass* CubeTextureFunctionAttachment::getInstanceClass() const {
   return getClass();
}

Object* CubeTextureFunctionAttachment::newInstance() {
   return new CubeTextureFunctionAttachment;
}

const MetaClass* CubeTextureFunctionAttachment::getClass() {
   if (CubeTextureFunctionAttachmentMetaClass==NULL) {
      CubeTextureFunctionAttachmentMetaClass = new MetaClass("CubeTextureFunctionAttachment", "no description.",
                                  newInstance, sizeof(CubeTextureFunctionAttachment));
   }
   return CubeTextureFunctionAttachmentMetaClass;
}

void CubeTextureFunctionAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   CubeTextureFunctionAttachmentMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT(CubeTextureFunctionAttachment);
   ADD_OBJECT_PROP(m_Function,0, CubeFunction3toN::getClass(), true )
   ADD_CARD32_PROP(m_Size,0);

  
}

void CubeTextureFunctionAttachment::shutdown() {
   delete CubeTextureFunctionAttachmentMetaClass;
}


// ----

MetaClass* CubeFunctionIdentity::CubeFunctionIdentityMetaClass = NULL;

const MetaClass* CubeFunctionIdentity::getInstanceClass() const {
   return getClass();
}

Object* CubeFunctionIdentity::newInstance() {
   return new CubeFunctionIdentity;
}

const MetaClass* CubeFunctionIdentity::getClass() {
   if (CubeFunctionIdentityMetaClass==NULL) {
      CubeFunctionIdentityMetaClass = new MetaClass("CubeFunctionIdentity", "no description.",
                                  newInstance, sizeof(CubeFunctionIdentity));
   }
   return CubeFunctionIdentityMetaClass;
}

void CubeFunctionIdentity::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   CubeFunctionIdentityMetaClass->copyParentProperties();
}

void CubeFunctionIdentity::shutdown() {
   delete CubeFunctionIdentityMetaClass;
}


// ----

CubeTextureAttachment::CubeTextureAttachment()
{
   m_CubeMap = new RGBACubeMap;
//   m_NeedsUpdate = true;
   m_MinFilterLinear = true;
   m_MagFilterLinear = true;
}

CubeTextureAttachment::~CubeTextureAttachment()
{
   delete m_CubeMap;
}

void CubeTextureAttachment::assign( const Object * obj, X4CopyContext * context )
{
   TextureAttachment::assign( obj, context );
   CubeTextureAttachment * cta = (CubeTextureAttachment*)obj;
   m_CubeMap->assign( cta->m_CubeMap );
   m_MinFilterLinear = cta->m_MinFilterLinear;
   m_MagFilterLinear = cta->m_MagFilterLinear;
}

GLTexture * CubeTextureAttachment::createGLTexture(GLContext * context)
{
   GLTextureCube * ret = new GLTextureCube( context );
   card32 size = m_CubeMap->getSide(0)->getSizeX();
   if( size == 0 )
   {
      ret->create(1, 4 );
   }
   else
   {
      ret->create( size, 4 );

      glTexParameteri( ret->getTextureTarget(), GL_TEXTURE_MIN_FILTER,
         m_MinFilterLinear ? GL_LINEAR : GL_NEAREST );
      glTexParameteri( ret->getTextureTarget(), GL_TEXTURE_MAG_FILTER,
         m_MagFilterLinear ? GL_LINEAR : GL_NEAREST );

      ret->setPositiveX( (char*)m_CubeMap->getSide( CubeMap<Vector4ub>::CMS_POSITIVE_X )->getPixels(),true);
      ret->setPositiveY( (char*)m_CubeMap->getSide( CubeMap<Vector4ub>::CMS_NEGATIVE_Y )->getPixels(),true);
      ret->setPositiveZ( (char*)m_CubeMap->getSide( CubeMap<Vector4ub>::CMS_POSITIVE_Z )->getPixels(),true);
      ret->setNegativeX( (char*)m_CubeMap->getSide( CubeMap<Vector4ub>::CMS_NEGATIVE_X )->getPixels(),true);
      ret->setNegativeY( (char*)m_CubeMap->getSide( CubeMap<Vector4ub>::CMS_POSITIVE_Y )->getPixels(),true);

      TemplatedImage<Vector4ub> *negZ = m_CubeMap->getSide( CubeMap<Vector4ub>::CMS_NEGATIVE_Z );

      // mirror negZ for openGL cubemaps
      std::vector<card32> tmpNegZ(negZ->getSizeX()*negZ->getSizeY());
      card32 * ptr = (card32 *)negZ->getPixels();
      size_t sOffset = tmpNegZ.size()-1;
      for( card32 i=0;i<tmpNegZ.size();i++ )
      {
         tmpNegZ[i] = ptr[ sOffset-- ];
      }

      ret->setNegativeZ( (char*)&tmpNegZ[0],true);

   }

   return ret;
}


CubeTextureFunctionAttachment::CubeTextureFunctionAttachment()
{
   m_Function = new CubeFunctionIdentity;
   m_Size = 128;
   m_ActSize = 0;
   m_ActFunctionVersion = 0;
}

CubeTextureFunctionAttachment::~CubeTextureFunctionAttachment()
{
   delete m_Function;
}

void CubeTextureFunctionAttachment::assign( const Object * obj, X4CopyContext * context )
{
   TextureAttachment::assign( obj, context );
   CubeTextureFunctionAttachment * att = (CubeTextureFunctionAttachment*)obj;
   m_Function->assign( att->m_Function );
   m_Size = att->m_Size;
   m_ActSize = 0;
   m_ActFunctionVersion = 0;
}

bool CubeTextureFunctionAttachment::needsUpdate()
{
   return (m_ActSize != m_Size) || (m_ActFunctionVersion != m_Function->getVersion());
}

GLTexture * CubeTextureFunctionAttachment::createGLTexture( GLContext * context )
{
   x4Assert( m_Function != NULL );

   if( (m_ActSize != m_Size) ||
      (m_ActFunctionVersion != m_Function->getVersion()) )
      calcCubemap();

   GLTextureCube * ret = new GLTextureCube( context );
   ret->create( m_Size,m_Function->getImageDimension() );
   ret->setPositiveX( (char*)&m_CubemapFace[ FACE_POSX ][0] );
   ret->setPositiveY( (char*)&m_CubemapFace[ FACE_POSY ][0] );
   ret->setPositiveZ( (char*)&m_CubemapFace[ FACE_POSZ ][0] );
   ret->setNegativeX( (char*)&m_CubemapFace[ FACE_NEGX ][0] );
   ret->setNegativeY( (char*)&m_CubemapFace[ FACE_NEGY ][0] );
   ret->setNegativeZ( (char*)&m_CubemapFace[ FACE_NEGZ ][0] );


   return ret;
}

void CubeTextureFunctionAttachment::calcCubemap()
{
   if( m_Size*m_Size*m_Function->getImageDimension() != m_CubemapFace[0].size() )
   {
      for( card32 i=0;i<6;i++ )
         m_CubemapFace[i] = std::vector<card8>( m_Size*m_Size*m_Function->getImageDimension() );
      m_ActSize = m_Size;
   }

   m_ActFunctionVersion = m_Function->getVersion();

    card32 numComp = m_Function->getImageDimension();
    int size = (int)m_Size;
    float funcRet[4];
    card8 * tmp = &m_CubemapFace[ FACE_POSX ][0];
	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(0.5f,  -yf, -xf );
				vec = normalize( vec );
            m_Function->calculate( vec[0], vec[1], vec[2], funcRet );
            for( card32 i=0;i<numComp;i++ )
            {
               float clamped = max(0.0f, min(1.0f,funcRet[i]) ) * 127.0f + 128.0f;
               tmp[(y*size+x)*numComp + i] = (card8)clamped;
            }
		  }

	 tmp = &m_CubemapFace[ FACE_POSZ ][0];
	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(xf, -yf,0.5f );
				vec = normalize( vec );
            m_Function->calculate( vec[0], vec[1], vec[2], funcRet );
            for( card32 i=0;i<numComp;i++ )
            {
               float clamped = max(0.0f, min(1.0f,funcRet[i]) ) * 127.0f + 128.0f;
               tmp[(y*size+x)*numComp + i] = (card8)clamped;
            }
		  }

	 tmp = &m_CubemapFace[ FACE_NEGX ][0];
	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(-0.5f, -yf,xf );
				vec = normalize( vec );
            m_Function->calculate( vec[0], vec[1], vec[2], funcRet );
            for( card32 i=0;i<numComp;i++ )
            {
               float clamped = max(0.0f, min(1.0f,funcRet[i]) ) * 127.0f + 128.0f;
               tmp[(y*size+x)*numComp + i] = (card8)clamped;
            }
		  }

	 tmp = &m_CubemapFace[ FACE_NEGZ ][0];
	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(-xf, -yf,-0.5f );
				vec = normalize( vec );
            m_Function->calculate( vec[0], vec[1], vec[2], funcRet );
            for( card32 i=0;i<numComp;i++ )
            {
               float clamped = max(0.0f, min(1.0f,funcRet[i]) ) * 127.0f + 128.0f;
               tmp[(y*size+x)*numComp + i] = (card8)clamped;
            }
		  }

    tmp = &m_CubemapFace[ FACE_NEGY ][0];
	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(xf, -0.5f,-yf );
				vec = normalize( vec );
            m_Function->calculate( vec[0], vec[1], vec[2], funcRet );
            for( card32 i=0;i<numComp;i++ )
            {
               float clamped = max(0.0f, min(1.0f,funcRet[i]) ) * 127.0f + 128.0f;
               tmp[(y*size+x)*numComp + i] = (card8)clamped;
            }
		  }

	 tmp = &m_CubemapFace[ FACE_POSY ][0];
	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				Vector3f vec = makeVector3f(xf, 0.5f,yf );
				vec = normalize( vec );
            m_Function->calculate( vec[0], vec[1], vec[2], funcRet );
            for( card32 i=0;i<numComp;i++ )
            {
               float clamped = max(0.0f, min(1.0f,funcRet[i]) ) * 127.0f + 128.0f;
               tmp[(y*size+x)*numComp + i] = (card8)clamped;
            }
		  }

/*    for( int i=0;i<6;i++ )
    for( int y=0;y<m_Size;y++ )
      for( int x=0;x<m_Size;x++ )
      {
         card8 col = (float)x / (float)m_Size * 255.0f;
         for( int c=0;c<numComp;c++ )
            m_CubemapFace[i][(y*m_Size+x)*numComp + c] = col;
      }*/

}
