#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "CurvatureDirectionsMaterial.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointCloudIterators.h"
#include "PointSet.h"
#include "SeparatorClassProperty.h"
#include "BooleanClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "PointSet.inline.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( CurvatureDirectionsMaterial, 2)
{
	BEGIN_CLASS_INIT( CurvatureDirectionsMaterial );
	ADD_STRING_PROP(curv2Channel, 0)
	ADD_VECTOR4F_PROP(positiveColor, 0)
	ADD_VECTOR4F_PROP(negativeColor, 0)
	ADD_FLOAT32_PROP(lineWidth, 0)
	ADD_BOOLEAN_PROP(rotateAroundNormal, 1)
	ADD_FLOAT32_PROP(maxLen, 2)
}

CurvatureDirectionsMaterial::CurvatureDirectionsMaterial(void)
{
	rotateAroundNormal = false;
	dir1Channel = "tangent_v";
	dir1Color = makeVector4f(0.7f, 0.7f, 1.0f, 0.4f);
	dir2Channel = "";
	dir3Channel = "";


	scaling = 0.04f;
	positionChannel = "position";
	normalizeDir = false;
	alphaBlending = true;
	cullBelow= 0.1f;
	setName("CurvatureDirectionsMaterial");
	maxLen = 0;

	curv2Channel="curv2";

	positiveColor = makeVector4f(0.7f, 1.0f, 0.7f, 0.4f);
	negativeColor = makeVector4f(1.0f, 0.7f, 0.7f, 0.4f);

	lineWidth = 2;
}

CurvatureDirectionsMaterial::~CurvatureDirectionsMaterial(void)
{
}

void CurvatureDirectionsMaterial::assign(const Object* obj, X4CopyContext *context)
{
	DirectionsMaterial::assign(obj,context);
	const CurvatureDirectionsMaterial * o = dynamic_cast<const CurvatureDirectionsMaterial*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::CurvatureDirectionsMaterial::draw( Scene *scene, SceneGraphState *state, SceneObject *obj, SceneObjectIterator *iter, GLContext *context, MemoryManager * mm /*= NULL*/, const int32 includeFlags /*= 0*/, const int32 excludeFlags /*= PF_FLAG_INVALID */ )
{
	float32 cullBelowQuad=cullBelow*cullBelow; 
	BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(iter);
	if (bit) {
		const PointSet *ps = bit->lockBlockRead();
		if (positionChannel != "" && ps->providesAttribute(positionChannel, 3, VAD::DATA_FORMAT_FLOAT32)) {
			AAT positionAAT = ps->getAAT(positionChannel);
			AAT dir1AAT = NULL_AAT;
			AAT dir2AAT = NULL_AAT;
			AAT dir3AAT = NULL_AAT;
			AAT dir4AAT = NULL_AAT;
			AAT curv2AAT = NULL_AAT;
			AAT normalAAT = NULL_AAT;
			if (rotateAroundNormal) normalAAT = ps->getAAT("normal");
			if (dir1Channel != "" && ps->providesAttribute(dir1Channel, 3, VAD::DATA_FORMAT_FLOAT32)) {
				dir1AAT = ps->getAAT(dir1Channel);
			}
			if (dir2Channel != "" && ps->providesAttribute(dir2Channel, 3, VAD::DATA_FORMAT_FLOAT32)) {
				dir2AAT = ps->getAAT(dir2Channel);
			}
			if (dir3Channel != "" && ps->providesAttribute(dir3Channel, 3, VAD::DATA_FORMAT_FLOAT32)) {
				dir3AAT = ps->getAAT(dir3Channel);
			}
			if (dir4Channel != "" && ps->providesAttribute(dir4Channel, 3, VAD::DATA_FORMAT_FLOAT32)) {
				dir4AAT = ps->getAAT(dir4Channel);
			}
			if (curv2Channel != "" && ps->providesAttribute(curv2Channel, 2, VAD::DATA_FORMAT_FLOAT32)) {
				curv2AAT = ps->getAAT(curv2Channel);
			}
			glPushAttrib(GL_ALL_ATTRIB_BITS);
			glEnable(GL_DEPTH_TEST);
			if (depthOffsetConst != 0 || depthOffsetFactor != 0) {
				glEnable(GL_POLYGON_OFFSET_LINE);
				glPolygonOffset(depthOffsetFactor, depthOffsetConst);
			}
			if (alphaBlending) {
				glEnable(GL_BLEND); 
				glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			} else {
				glDisable(GL_BLEND);
			}
			glDisable(GL_LIGHTING);

			glLineWidth(lineWidth);

			card32 n = ps->getNumEntries();
			glBegin(GL_LINES);
			for (card32 i=0; i<n; i++) {

				Vector3f position = ps->get3f(i, positionAAT);
				if (dir1AAT != NULL_AAT) {
					Vector3f d = ps->get3f(i, dir1AAT);					
					if (rotateAroundNormal) d = d.crossProduct(normalize(ps->get3f(i,normalAAT)));
					if (curv2AAT != NULL_AAT)
					{
						if (ps->get2f(i, curv2AAT)[0]<0)
							glColor4fv(negativeColor.data());
						else
							glColor4fv(positiveColor.data());
						d*=ps->get2f(i, curv2AAT)[0];
					}else
						glColor4fv(dir1Color.data());


					if (normQuad(d)>cullBelowQuad)
					{
						if (maxLen <= 0 || norm(d) < maxLen) {
							if (normalizeDir) d = normalize(d);
							d *= (0.5f*scaling);
							glVertex3fv((position-d).data());
							glVertex3fv((position+d).data());
						}
					}
				}

// 				if (dir2AAT != NULL_AAT) {
// 					glColor4fv(dir2Color.data());
// 					Vector3f d = ps->get3f(i, dir2AAT);
// 					if (normalizeDir) d = normalize(d);
// 					d *= scaling;
// 					glVertex3fv(position.data());
// 					glVertex3fv((position+d).data());
// 				}
// 				if (dir3AAT != NULL_AAT) {
// 					glColor4fv(dir3Color.data());
// 					Vector3f d = ps->get3f(i, dir3AAT);
// 					if (normalizeDir) d = normalize(d);
// 					d *= scaling;
// 					glVertex3fv(position.data());
// 					glVertex3fv((position+d).data());
// 				}
// 				if (dir4AAT != NULL_AAT) {
// 					glColor4fv(dir4Color.data());
// 					Vector3f d = ps->get3f(i, dir4AAT);
// 					if (normalizeDir) d = normalize(d);
// 					d *= scaling;
// 					glVertex3fv(position.data());
// 					glVertex3fv((position+d).data());
// 				}
			}
			glEnd();

			glPopAttrib();
		}
		bit->unlock();
	}
}