//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "DirectionsMaterial.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointCloudIterators.h"
#include "PointSet.h"
#include "SeparatorClassProperty.h"
#include "BooleanClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "PointSet.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( DirectionsMaterial , 3 ) {
	BEGIN_CLASS_INIT( DirectionsMaterial );
	ADD_SEPARATOR("channels")
	ADD_STRING_PROP(dir1Channel, 0)
	ADD_VECTOR4F_PROP(dir1Color, 0)
	ADD_STRING_PROP(dir2Channel, 0)
	ADD_VECTOR4F_PROP(dir2Color, 0)
	ADD_STRING_PROP(dir3Channel, 0)
	ADD_VECTOR4F_PROP(dir3Color, 0)
	ADD_STRING_PROP(dir4Channel, 0)
	ADD_VECTOR4F_PROP(dir4Color, 0)
	ADD_SEPARATOR("options")
	ADD_FLOAT32_PROP(scaling, 0)
	ADD_FLOAT32_PROP(lineWidthDM, 3)
	ADD_BOOLEAN_PROP(normalizeDir, 0)
	ADD_BOOLEAN_PROP(alphaBlending, 0)
	ADD_FLOAT32_PROP(cullBelow, 1)
	ADD_FLOAT32_PROP(depthOffsetFactor, 2)
	ADD_FLOAT32_PROP(depthOffsetConst, 2)
}

// ----

DirectionsMaterial::DirectionsMaterial() {
   dir1Channel = "tangent_u";
   dir1Color = makeVector4f(1.0f, 0.5f, 0.5f, 1.0f);
   dir2Channel = "tangent_v";
   dir2Color = makeVector4f(0.5f, 1.0f, 0.5f, 1.0f);
   dir3Channel = "normal";
   dir3Color = makeVector4f(0.5f, 0.5f, 1.0f, 1.0f);
   dir4Color = makeVector4f(1.0f, 1.0f, 1.0f, 1.0f);
   scaling = 1.0f;
   positionChannel = "position";
   normalizeDir = false;
   alphaBlending = false;
   cullBelow= 0;
	depthOffsetFactor=0;
	depthOffsetConst=0;
	lineWidthDM = 1;
   setName("DirectionsMaterial");
}

void DirectionsMaterial::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

DirectionsMaterial::~DirectionsMaterial() {
}



void DirectionsMaterial::draw(
	Scene *scene, SceneGraphState *state, SceneObject *obj,
   SceneObjectIterator *iter, GLContext *context, MemoryManager * mm,
   const int32 includeFlags, const int32 excludeFlags)
{
	float32 cullBelowQuad=cullBelow*cullBelow; 
	BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(iter);
	if (bit) {
		const PointSet *ps = bit->lockBlockRead();
	   if (positionChannel != "" && ps->providesAttribute(positionChannel, 3, VAD::DATA_FORMAT_FLOAT32)) {
		   AAT positionAAT = ps->getAAT(positionChannel);
		   AAT dir1AAT = NULL_AAT;
		   AAT dir2AAT = NULL_AAT;
		   AAT dir3AAT = NULL_AAT;
		   AAT dir4AAT = NULL_AAT;
		   if (dir1Channel != "" && ps->providesAttribute(dir1Channel, 3, VAD::DATA_FORMAT_FLOAT32)) {
		      dir1AAT = ps->getAAT(dir1Channel);
		   }
		   if (dir2Channel != "" && ps->providesAttribute(dir2Channel, 3, VAD::DATA_FORMAT_FLOAT32)) {
		      dir2AAT = ps->getAAT(dir2Channel);
		   }
		   if (dir3Channel != "" && ps->providesAttribute(dir3Channel, 3, VAD::DATA_FORMAT_FLOAT32)) {
		      dir3AAT = ps->getAAT(dir3Channel);
		   }
		   if (dir4Channel != "" && ps->providesAttribute(dir4Channel, 3, VAD::DATA_FORMAT_FLOAT32)) {
		      dir4AAT = ps->getAAT(dir4Channel);
		   }
			glPushAttrib(GL_ALL_ATTRIB_BITS);
			glLineWidth(lineWidthDM);
			glEnable(GL_DEPTH_TEST);
			if (depthOffsetConst != 0 || depthOffsetFactor != 0) {
				glEnable(GL_POLYGON_OFFSET_LINE);
				glPolygonOffset(depthOffsetFactor, depthOffsetConst);
			}
			if (alphaBlending) {
			   glEnable(GL_BLEND); 
			   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		   } else {
			   glDisable(GL_BLEND);
			}
			glDisable(GL_LIGHTING);

			card32 n = ps->getNumEntries();
			glBegin(GL_LINES);
			for (card32 i=0; i<n; i++) {
				Vector3f position = ps->get3f(i, positionAAT);
				if (dir1AAT != NULL_AAT) {
					glColor4fv(dir1Color.data());
					Vector3f d = ps->get3f(i, dir1AAT);
					
					if (normQuad(d)>cullBelowQuad)
					{
						if (normalizeDir) d = normalize(d);
						d *= scaling;
						glVertex3fv(position.data());
						glVertex3fv((position+d).data());
					}
				}
				if (dir2AAT != NULL_AAT) {
					glColor4fv(dir2Color.data());
					Vector3f d = ps->get3f(i, dir2AAT);
					if (normalizeDir) d = normalize(d);
					d *= scaling;
					glVertex3fv(position.data());
					glVertex3fv((position+d).data());
				}
				if (dir3AAT != NULL_AAT) {
					glColor4fv(dir3Color.data());
					Vector3f d = ps->get3f(i, dir3AAT);
					if (normalizeDir) d = normalize(d);
					d *= scaling;
					glVertex3fv(position.data());
					glVertex3fv((position+d).data());
				}
				if (dir4AAT != NULL_AAT) {
					glColor4fv(dir4Color.data());
					Vector3f d = ps->get3f(i, dir4AAT);
					if (normalizeDir) d = normalize(d);
					d *= scaling;
					glVertex3fv(position.data());
					glVertex3fv((position+d).data());
				}
			}
			glEnd();

         glPopAttrib();
      }
		bit->unlock();
	}
}
