//---------------------------------------------------------------------------
#ifndef DirectionsMaterialH
#define DirectionsMaterialH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "GLMaterial.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace X4 {

///
///   A simple immediate mode material that draws points in point clouds as spheres
///
class X3RENDERING_API DirectionsMaterial : public GLMaterial {
	X4_CLASS( DirectionsMaterial )
 protected:
	std::string dir1Channel;
	Vector4f dir1Color;
	std::string dir2Channel;
	Vector4f dir2Color;
	std::string dir3Channel;
	Vector4f dir3Color;
	std::string dir4Channel;
	Vector4f dir4Color;
	std::string positionChannel;
	float32 scaling;
	float32  lineWidthDM; // postfix for compatibility with curvature directions material, which unfortunately introduces a property of the same name
	bool normalizeDir;
	bool alphaBlending;
	/// culls line if length below this value
	float32 cullBelow;
	float32 depthOffsetFactor; // for shifting the lines in depth direction (for overlays; just the GL parameters)
	float32 depthOffsetConst;  // for shifting the lines in depth direction (for overlays; just the GL parameters)


 public:
	DirectionsMaterial();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void draw(Scene *scene, SceneGraphState *state, SceneObject *obj,
      SceneObjectIterator *iter, GLContext *context, MemoryManager * mm = NULL,
      const int32 includeFlags = 0, const int32 excludeFlags = PF_FLAG_INVALID );

	std::string getdir1Channel() const {return dir1Channel;}
	void setdir1Channel(std::string v) {dir1Channel=v;}

	Vector4f getdir1Color() const {return dir1Color;}
	void setdir1Color(Vector4f v) {dir1Color=v;}

	std::string getdir2Channel() const {return dir2Channel;}
	void setdir2Channel(std::string v) {dir2Channel=v;}

	Vector4f getdir2Color() const {return dir2Color;}
	void setdir2Color(Vector4f v) {dir2Color=v;}

	std::string getdir3Channel() const {return dir3Channel;}
	void setdir3Channel(std::string v) {dir3Channel=v;}

	Vector4f getdir3Color() const {return dir3Color;}
	void setdir3Color(Vector4f v) {dir3Color=v;}

	std::string getdir4Channel() const {return dir4Channel;}
	void setdir4Channel(std::string v) {dir4Channel=v;}

	Vector4f getdir4Color() const {return dir4Color;}
	void setdir4Color(Vector4f v) {dir4Color=v;}

	std::string getpositionChannel() const {return positionChannel;}
	void setpositionChannel(std::string v) {positionChannel=v;}

	float32 getscaling() const {return scaling;}
	void setscaling(float32 v) {scaling=v;}

	bool getnormalize() const {return normalizeDir;}
	void setnormalize(bool v) {normalizeDir=v;}

	bool getalphaBlending() const {return alphaBlending;}
	void setalphaBlending(bool v) {alphaBlending=v;}

   ~DirectionsMaterial();
};

}

#endif