#ifndef ECRayCasterH
#define ECRayCasterH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "RaytracingMath.h"
//---------------------------------------------------------------------------

namespace X4
{
class AttachedIndexedOctree;
class UnstructuredInCorePointCloud;

/**
	hierarchical ray casting of point clouds (assumes surfel point cloud) 
*/
class X3RENDERING_API ECRayCaster
{

private:

	bool		m_RayHitSurfel;
	mpcard		m_LastHittedSurfelIndex;
	Vector3f	m_LastHittedSurfelPosition;
	Vector3f	m_LastHittedSurfelNormal;

	UnstructuredInCorePointCloud	*	m_UPC;
	AAT			m_PosAAT;
	AAT			m_NormalAAT;
	AttachedIndexedOctree * m_Octree;
	
	


public:
	ECRayCaster(void);
	~ECRayCaster(void);

	/// setup ray caster (attached index octree will be used for ray casting, call setup again if point cloud is modified)
	void setup(UnstructuredInCorePointCloud* pc);
	
	/// returns true if ray hits a surfel (optionally ray parameter t of hit point will be returned)
	/// position, normal & index of hit will be available via getLast... methods
	/// ATTENTION! getLast... methods are only valid directly after the castRay method
	bool castRay( const Ray3f &r, float surfelSize, float  * t = NULL);

	/// returns intersection point of last hit 
	Vector3f getLastHitPosition() const {return m_LastHittedSurfelPosition;}

	/// returns index of intersection point of last hit 
	mpcard getLastHitPointIndex() const {return m_LastHittedSurfelIndex;}

	/// returns normal of last hit 
	Vector3f getLastHitNormal() const {return m_LastHittedSurfelNormal;}

};

}

#endif