//---------------------------------------------------------------------------
#ifndef GLAttachmentsH
#define GLAttachmentsH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "MemoryManagedAttachment.h"
#include "GLBuffer.h"
//---------------------------------------------------------------------------

namespace X4 {

class GLVertexBuffer;
class GLTexture;


class X3RENDERING_API SharedVertexBufferAttachment : public MemoryManagedAttachment
{
private:

   GLVertexBuffer *  m_VertexBuffer;

protected:
   virtual card32 getManagedSize();

   // force using setupVB
   virtual void setupMMA( const std::string &name, MemoryManager * mm, bool deleteOnUpdate=true);

public:

	/// hints for sorting
	Vector3f lastDirection;
	/// hints for sorting
	Matrix4f lastCTM;

   SharedVertexBufferAttachment();
   virtual ~SharedVertexBufferAttachment();

   GLVertexBuffer * getVertexBuffer() const {return m_VertexBuffer;}

   /// setup
   /// SharedVertexBuffer becomes owner of vbuffer!
   /// uses generateID to determine attachment - name
   void setupVB( GLVertexBuffer * vbuffer, const std::string &addCustomID, MemoryManager * mm,
      bool deleteOnUpdate, const int32 includeFlags, const int32 excludeFlags, 
      Vector3f lastDirection, Matrix4f lastCTM );

   /// generates ID-String with
   /// vdesc->getDescription() + intToStr( (int)context ) + addCustomID
   static std::string generateID( const VertexDescriptor * vdesc, GLContext * context,
      const std::string &addCustomID, const int32 includeFlags, const int32 excludeFlags );

};

class UnstructuredInCoreTriangleMesh;

class X3RENDERING_API GLIndexAttachment : public MemoryManagedAttachment
{
public:
	GLIndexAttachment();
	~GLIndexAttachment();

	/// hints for sorting
	Vector3f lastDirection;
	/// hints for sorting
	Matrix4f lastCTM;

	/// create buffer and init attachment
	void setupIndexBuffer(const std::string &addCustomID, GLContext * context, MemoryManager * mm,
		bool deleteOnUpdate, const int32 includeFlags, const int32 excludeFlags,
		UnstructuredInCoreTriangleMesh * mesh, 
		Vector3f lastDirection, Matrix4f lastCTM );

	/// generate id
	static std::string generateID(const std::string &addCustomID, GLContext * context, MemoryManager * mm,
		const int32 includeFlags, const int32 excludeFlags);

	virtual card32 getManagedSize();

	GLIndexBuffer32 * getBuffer() const {return m_Buffer;}

private:
	
	GLIndexBuffer32 * m_Buffer;
};

class X3RENDERING_API GLTextureAttachment : public MemoryManagedAttachment
{
private:
   GLTexture   *  m_Texture;
	std::string			m_TextureDescription;

protected:

   virtual card32 getManagedSize();

public:

   GLTextureAttachment();
   ~GLTextureAttachment();

   /// init attachment
   /// name = generateID( texture->gerRootContext(), addCustomID )
   void setupTA( GLTexture * texture, const std::string &addCustomID, MemoryManager * memManager, bool deleteOnUpdate=true );

   /// generates ID-String with
   /// intToStr( (int)context ) + addCustomID
   static std::string generateID( GLContext * context, const std::string &addCustomID = "" );

   GLTexture   *  getTexture() {return m_Texture;}

	std::string getTextureDescription() const {return m_TextureDescription;}
	void setTextureDescription(std::string v) {m_TextureDescription=v;}


};


class X3RENDERING_API GLTextureMapAttachment : public MemoryManagedAttachment
{
private:
	std::map<int, GLTexture*>  m_Texture;
	std::string			m_TextureDescription;

protected:

	virtual card32 getManagedSize();

public:

	GLTextureMapAttachment();
	~GLTextureMapAttachment();

	/// Init texture attachment
	void setupTA( GLTexture * texture, int index, MemoryManager * memManager, bool deleteOnUpdate=true );

	/// return texture on specified index, if no such found, then return NULL
	GLTexture*  getTexture(int index);

	std::string getTextureDescription() const {return m_TextureDescription;}
	void setTextureDescription(std::string v) {m_TextureDescription=v;}
};



class X3RENDERING_API GLMultiTextureAttachment : public MemoryManagedAttachment
{
private:
	std::vector<GLTexture*>  m_Textures;
	std::string		m_TextureDescription;

protected:

	virtual card32 getManagedSize();

public:

	GLMultiTextureAttachment(){};
	~GLMultiTextureAttachment(){};

	/// init attachment
	/// name = generateID( texture->gerRootContext(), addCustomID )
//	void setupTA(const std::string &addCustomID, MemoryManager * memManager, bool deleteOnUpdate=true );

	void addTA( GLTexture * texture){m_Textures.push_back(texture);};

	/// generates ID-String with
	/// intToStr( (int)context ) + addCustomID
	static std::string generateID( GLContext * context, const std::string &addCustomID = "" );

	GLTexture*  getTexture(const int i) {return m_Textures[i];}
	std::vector<GLTexture*>  getTextures() {return m_Textures;}

	std::string getTextureDescription() const {return m_TextureDescription;}
	void setTextureDescription(std::string v) {m_TextureDescription=v;}

};




}






#endif
