#ifndef GLShaderMaterialEditorWidgetH 
#define GLShaderMaterialEditorWidgetH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "ClassEditor.h"
#include "x3rendering/GLShaderMaterialEditorForm_ui.h"
#include "QButtonGroup"
#include "Shader.h"
#include "ShaderEditorWidget.h"

//---------------------------------------------------------------------------

namespace X4
{

class GLShaderMaterial;
class Shader;
class ShaderEditor;
class ShaderEditorWidget;
class VertexDescriptorEditor;
class VertexDescriptorEditorWidget;
class GLContext;
class ClassEditor;

class X3RENDERING_API GLShaderMaterialEditorWidget : public QWidget
{
	Q_OBJECT

private:

	GLShaderMaterial  * m_ShaderMaterial;
	GLContext     * m_TestContext;
	ShaderEditor     * m_ShaderEditorVS,
                     * m_ShaderEditorPS;
	VertexDescriptorEditor * m_VertexDescriptorEditor;
	ShaderEditorWidget * m_ShaderEditorWidgetVS,
                           * m_ShaderEditorWidgetPS;
	VertexDescriptorEditorWidget * m_VertexDescriptorEditorWidget;
	ClassEditor       *  m_ParameterEditor;
	QWidget          *  m_ParameterEditorWidget;
	

	///used to access buttons, etc. from QtDesigner
	Ui_GLShaderMaterialEditorForm ui;

   QButtonGroup* BG_source,* BG_dest;

   void createVSEditor( Shader *);
   void freeVSEditor();
   void createPSEditor( Shader *);
   void freePSEditor();

public:
	GLShaderMaterialEditorWidget(QWidget * parent = NULL, Qt::WindowFlags f=0);
	~GLShaderMaterialEditorWidget(void);

	void setup( GLShaderMaterial * obj );

	/// update editor content with object data
	void updateEditor();

	void setTestContext( GLContext * context );

public slots:
	void MatNameChange();
	void RenderModeChange();
	void Button_RebuildObjectsClick();
	void ButtonLoadVSClick( );
	void ButtonSaveVSClick( );
	void ButtonLoadPSClick( );
	void ButtonSavePSClick( );
	void Scale_EditValueChange( );
	void FE_pointSizeValueChange( );
	void CB_blendingClick( );
	void cb_vspointsizeClick( );
	void CB_depthTestClick( );
	//void CB_depthWriteClick( );
	void CB_depthWrite_activated(int);
	void RG_sourceBlendClick(int );
	void RG_destBlendClick(int );
	void ImportShadersClick( );
	void ForceTriangleNormalClick( );
	void DrawWireMeshClick( );
	void ColorButtonClick( );
	void on_bDontChangeState_clicked();
	void on_bPointSpriteMode_clicked();
	void cbSortPoints_clicked();
	void cbFrontToBack_clicked();
	void on_cbSphPoints_clicked();
	void on_cbBackfaceCulling_clicked();

protected:

	void resizeEvent( QResizeEvent *e);


};

MAKE_CLASS_EDITOR( X3RENDERING_API , GLShaderMaterial )


}

#endif