//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GLSimpleVolumeMaterial.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "RegularInCoreVolume.h"
#include "RegularVolume.h"
#include "GLVolumeRenderer2DSlices.h"
#include "Tex2dVolumeRenderer.h"
#include "SceneGraphState.h"
#include "BooleanClassProperty.h"
#include "ObjectClassProperty.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS(GLSimpleVolumeMaterial, 0) {
	BEGIN_CLASS_INIT(GLSimpleVolumeMaterial );
	ADD_OBJECT_PROP(renderer, 0, GLVolumeRenderer2DSlices::getClass(), true);
}



IMPLEMENT_X4_CLASS(TextureStackAttachment, 0) {
}
// ----

GLSimpleVolumeMaterial::GLSimpleVolumeMaterial() {
   renderer = new GLVolumeRenderer2DSlices();
   renderer->setChannel("intensity");
   renderer->setShowCoords(false);
}

void GLSimpleVolumeMaterial::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

GLSimpleVolumeMaterial::~GLSimpleVolumeMaterial() {
   delete renderer;
}




void GLSimpleVolumeMaterial::draw(
   Scene *scene, SceneGraphState *state, SceneObject *obj, SceneObjectIterator *iter, 
   GLContext *context, MemoryManager *mm, 
   const int32 includeFlags, const int32 excludeFlags)
{
   if (dynamic_cast<RegularInCoreVolume*>(obj)) {
      RegularInCoreVolume *vol = dynamic_cast<RegularInCoreVolume*>(obj);
      RegularVolume *rvol = vol->getVoxels();
      
      SecondaryDataStructureIterator *dataIter = dynamic_cast<SecondaryDataStructureIterator*>(iter);
      x4Assert(dataIter);
      if (!dataIter) return;

		AttachedDataContainer *attachments = dataIter->lockAttachments();
      AttachedData *data = attachments->getData( "attachedTextureStack" );
      TextureStackAttachment *tStack = dynamic_cast<TextureStackAttachment*>(data);

      if (!tStack) {
         tStack = new TextureStackAttachment();
         tStack->setup("attachedTextureStack", AttachedData::ADF_DELETEONUPDATE);
         tStack->texStack = new Tex2dVolumeRenderer();
         tStack->channel = renderer->getChannel();
         if (rvol->getDescr()->providesAttribute(renderer->getChannel())) {
            rvol->computeMinMax1f(renderer->getChannel(), tStack->minVal, tStack->maxVal);
         } else {
            tStack->minVal = 0;
            tStack->maxVal = 1;
         }
         attachments->attachData( tStack );
      }
      if (tStack->channel != renderer->getChannel()) {
         if (rvol->getVoxelData()->getDescr()->providesAttribute(renderer->getChannel())) {
            rvol->computeMinMax1f(renderer->getChannel(), tStack->minVal, tStack->maxVal);
            tStack->channel = renderer->getChannel();
         } else {
            warning("GLSimpleVolumeMaterial::draw() - channel not found");
            return;
         }
      }
   
      Camera cam = scene->getRootState()->staticState->camera;

      renderer->drawGL(rvol, tStack->texStack, cam, tStack->minVal, tStack->maxVal);
   }   
}


