//---------------------------------------------------------------------------
#ifndef GLVolumeDisplayerH
#define GLVolumeDisplayerH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "GLEventDisplayer.h"
#include "RegularData.h"
#include "Object.h"
//---------------------------------------------------------------------------
#include "x3rendering\GLVolumeDisplayerConfig_ui.h"
//---------------------------------------------------------------------------


namespace X4 {

class GLVolumeDisplayerConfig;
class RegularVolume;
class GLVolumeRenderer2DSlices;
class Tex2dVolumeRenderer;


class X3RENDERING_API GLVolumeDisplayer : public GLEventObject {
	X4_CLASS(GLVolumeDisplayer)
 private:
	RegularVolume *volume;
   GLVolumeRenderer2DSlices *renderer;
	Tex2dVolumeRenderer *textureStack;

	std::string xAxisLabel;
	std::string yAxisLabel;
	std::string zAxisLabel;

	float32 maxValue;
	float32 minValue;

	void initialSetup();
		
 public:
	/// ownership of the RegularVolume object is transfered to the event object
	GLVolumeDisplayer(RegularVolume *vol = NULL, 
	                  std::string xAxisLabel = "unnamed", std::string yAxisLabel = "unnamed", std::string zAxisLabel = "unnamed");
	/// creates a copy of the provided regular data object
	/// regular data must be 3-dimensional
	GLVolumeDisplayer(const RegularDataF *vol, 
	                  std::string xAxisLabel = "unnamed", std::string yAxisLabel = "unnamed", std::string zAxisLabel = "unnamed");
	/// creates copy of point set
	GLVolumeDisplayer(const PointSet *ps, card32 width, card32 height, card32 depth, 
	                  std::string xAxisLabel = "unnamed", std::string yAxisLabel = "unnamed", std::string zAxisLabel = "unnamed");
	
	RegularVolume *getVolume() {return volume;}

	virtual BoundingBox3f getBoundingBox();
	virtual void drawGL(QGLWidget *widget, Camera &cam);
	virtual GLEventConfiguration *createConfigWindow(QWidget *parent);
	virtual void updateData(GLEventObject *other);

	virtual ~GLVolumeDisplayer();
 friend GLVolumeDisplayerConfig;
};


class X3RENDERING_API GLVolumeDisplayerConfig : public GLEventConfiguration {
	Q_OBJECT
 private:
	Ui_GLVolumeDisplayerConfig ui;
	bool deactivateChannelChoice;

 private slots:
   void on_cbShowCoords_stateChanged(int);
   void on_cbChannel_currentIndexChanged(const QString &text);
   void on_rbOpacityWhite_clicked(bool);
   void on_rbOpacityIntensity_clicked(bool);
   void on_rbOpacityBlueRed_clicked(bool);
   void on_dsbOffset_valueChanged(double);
   void on_dsbScale_valueChanged(double);
   void on_dsbGamma_valueChanged(double);
   void on_cbAutomatic_clicked(bool);

 public:
	GLVolumeDisplayerConfig(QWidget *parent, GLEventObject *obj, QString xAxisLabel, QString yAxisLabel, QString zAxisLabel);
};


void X3RENDERING_API glEventDisplayer_unitTest();


}

#endif