//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "GLVolumeMaterialStandard.h"
//---------------------------------------------------------------------------
#include "GLShader.h"
#include "InCoreVolume.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
#include "CopyObjectProperties.h"
#include "RegularVolume.h"
#include "GLAttachments.h"
#include "Random.h"
#include "GLTexture.h"
//---------------------------------------------------------------------------
using namespace X4;
using namespace std;
//---------------------------------------------------------------------------


IMPLEMENT_X4_CLASS(GLVolumeMaterialStandard, 0) {
	BEGIN_CLASS_INIT(GLVolumeMaterialStandard );
}

GLVolumeMaterialStandard::GLVolumeMaterialStandard()
{
	setName( "unnamed VolumeMaterialStandard" );
}

GLVolumeMaterialStandard::~GLVolumeMaterialStandard()
{
}

void GLVolumeMaterialStandard::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj, this);
}

void GLVolumeMaterialStandard::draw( Scene * scene, SceneGraphState *state, SceneObject * obj, SceneObjectIterator *iter,
	GLContext * context, MemoryManager * mm, const int32 includeFlags, const int32 excludeFlags)
{
	SecondaryDataStructureIterator *attIt = dynamic_cast<SecondaryDataStructureIterator*>(iter);
	if (!attIt)
		return;
	AttachedDataContainer *attachments = attIt->lockAttachments();
	GLTextureAttachment *gltexAttachment = dynamic_cast<GLTextureAttachment*>(
		attachments->getData( GLTextureAttachment::generateID(context, "Vol_Tex_3f") )
		);

	GLTexture3D *texture = NULL;
	if(!gltexAttachment)
	{
		gltexAttachment = new GLTextureAttachment();
		texture = new GLTexture3D(context);
		float texels[64];
		for (int i = 0; i < 64; i++)
		{
			texels[i] = rnd01();
		}
		texture->createFloatTextureFromMemory(4,4,4,1,(char*)texels,false,false);
		gltexAttachment->setupTA(texture,"Vol_Tex_3f", mm, true);
		attachments->attachData(gltexAttachment);
	}
	else
	{
		texture = dynamic_cast<GLTexture3D*>(gltexAttachment->getTexture());
		if (!texture)
			return;
	}

	VolumetricIterator * vIter = dynamic_cast<VolumetricIterator *>( iter );
	if (!vIter) return;

	const RegularVolume *volume = vIter->lockVolumeBlockRead();

	BoundingBox3f bb = volume->getBoundingBox();

	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glEnable(GL_TEXTURE_3D);
	GLTextureSettings settings;

	texture->bind(settings, 0);
	//   glMatrixMode(GL_TEXTURE);
	//   glActiveTextureARB(GL_TEXTURE0_ARB);

	glPointSize(5.0);

	glBegin(GL_POINTS);
	for (card32 w = 0; w < volume->getWidth(); w++)
	{
		for (card32 h = 0; h < volume->getHeight(); h++)
		{
			for (card32 d = 0; d < volume->getDepth(); d++)
			{
				float32 wf = (float32)w/(float32)volume->getWidth();
				float32 hf = (float32)h/(float32)volume->getHeight();
				float32 df = (float32)d/(float32)volume->getDepth();
				glTexCoord3f(wf, hf, df);
				Vector3f v = makeVector3f(wf, hf, df);
				v = v.componentProduct(bb.upperCorner-bb.lowerCorner) + bb.lowerCorner;
				glVertex3fv(v.data());
			}
		}
	}
	glEnd();

	glPopAttrib();


	texture->unbind(0);
	vIter->unlockVolume();
	attIt->unlockAttachments();
}
