#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GLVolumeRaycastMaterial.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "VertexDescriptor.h"
#include "Shader.h"
#include "GLShader.h"
#include "DebugOutput.h"
#include "PointCloudIterators.h"
#include "GLAttachments.h"
#include "RegularVolume.h"
#include "SceneObject.h"
#include "InterpFunctionTextureAttachment.h"
#include "InterpFunctionTextureAttachmentRGB.h"
#include "GLTexture.h"
#include "Configuration.h"
#include "ApplicationDirectories.h"

//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( GLVolumeRaycastMaterial ,1)
{
	BEGIN_CLASS_INIT( GLVolumeRaycastMaterial );
	ADD_OBJECT_PROP( m_ElementDescriptor,1,VertexDescriptor::getClass(),true );
	ADD_OBJECT_PROP( m_VertexShader,1,Shader::getClass(),true );
	ADD_OBJECT_PROP( m_PixelShader,1,Shader::getClass(),true );
	ADD_OBJECT_PROP( m_TransferFunctionAlpha,1,InterpFunctionTextureAttachment::getClass(),true );
	ADD_OBJECT_PROP( m_TransferFunctionColor,1,InterpFunctionTextureAttachmentRGB::getClass(),true );
		
}

GLVolumeRaycastMaterial::GLVolumeRaycastMaterial(void)
{
	m_ElementDescriptor = new VertexDescriptor;
	m_ElementDescriptor->pushAttrib(VAD("intensity",1,VAD::DATA_FORMAT_FLOAT32));

	m_VertexShader = new Shader;
	m_PixelShader = new Shader;
	m_VertexShader->setShaderType( Shader::VERTEX );
	m_PixelShader->setShaderType( Shader::FRAGMENT );
	m_VertexShader->setShaderClass( Shader::CLASS_CG);
	m_PixelShader->setShaderClass( Shader::CLASS_CG);

	/*
	std::string shaderPath = ApplicationDirectories::getResourcesDir();
	std::string vshaderName = shaderPath + "\\shaders\\VolumeRaycastingVS.txt";
	std::string pshaderName = shaderPath + "\\shaders\\VolumeRaycastingPS.txt";
	if( !m_VertexShader->loadFromFile(vshaderName))
	{
		message( std::string("GLVolumeRaycastMaterial::GLVolumeRaycastMaterial() - Could not find shader: ") +   vshaderName + "\n");
	}
	if( !m_PixelShader->loadFromFile(pshaderName))
	{
		message( std::string("GLVolumeRaycastMaterial::GLVolumeRaycastMaterial() - Could not find shader: ") + pshaderName + "\n");
	}
	*/

	m_TransferFunctionAlpha = new InterpFunctionTextureAttachment;
	m_TransferFunctionColor = new InterpFunctionTextureAttachmentRGB;
}

GLVolumeRaycastMaterial::~GLVolumeRaycastMaterial(void)
{
	delete m_TransferFunctionAlpha;
	delete m_TransferFunctionColor;
	delete m_ElementDescriptor;
	while( !m_glPixelShaders.empty())
	{
		delete m_glPixelShaders.begin()->second;
		m_glPixelShaders.erase(m_glPixelShaders.begin());
	}
	while( !m_glVertexShaders.empty())
	{
		delete m_glVertexShaders.begin()->second;
		m_glVertexShaders.erase(m_glVertexShaders.begin());
	}
	while( !m_glTransferFunctionTextures.empty())
	{
		delete m_glTransferFunctionTextures.begin()->second;
		m_glTransferFunctionTextures.erase(m_glTransferFunctionTextures.begin());
	}
}

void GLVolumeRaycastMaterial::assign(const Object* obj, X4CopyContext *context)
{
	GLMaterial::assign(obj,context);
	const GLVolumeRaycastMaterial * o = dynamic_cast<const GLVolumeRaycastMaterial*>(obj);
	x4Assert( o != NULL );

	copyObjectProperties(obj,this);
}

void X4::GLVolumeRaycastMaterial::draw( Scene *scene, SceneGraphState *state, SceneObject *obj, SceneObjectIterator *iter, GLContext *context, 
													  MemoryManager * mm /*= NULL*/, const int32 includeFlags /*= 0*/, const int32 excludeFlags /*= PF_FLAG_INVALID */ )
{
	if( !obj )
		return;

	std::vector<GLTexture*> tex3d = getVolumeTexture(context,iter,mm);
	if( !tex3d.empty())
	{
		if( bindShaders(context) )
		{
			// setup matrix world to texture space
			VolumetricIterator * vIter = dynamic_cast<VolumetricIterator *>( iter );
			BoundingBox3f bb = vIter->getVolumeRegion();
			Vector3f scale = makeVector3f(1.0f / bb.getSideLength(0),1.0f / bb.getSideLength(1),1.0f / bb.getSideLength(2));
			Matrix4f worldToTexture = makeScale4f(scale) * makeTranslation4f(-bb.lowerCorner);// * invertFrame(obj->getTransformation());
			m_glLastPixelShader->setMatrix4f( "worldToTexture",0,worldToTexture.data());
			

			// set step size (1 / maxTextureResolution)
			Vector3i volResolution = vIter->getVolumeSize();
			((GLShaderCG*)m_glLastPixelShader)->setParameter1f( "stepSize", 1.0f / max(volResolution[0],max(volResolution[1],volResolution[2])));

			// bind volume texture
			for(int i = 0; i < (int)tex3d.size(); i++)
			{
				tex3d[i]->bind(i);
//				glTexParameteri( GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
//				glTexParameteri( GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
			}
// 			glTexParameteri( GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
// 			glTexParameteri( GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );

			// bind transfer function texture
			bindTransferFunctionTexture(context, (unsigned)tex3d.size());

			// draw a box with front-face culling (so we always see the backside)
			glPushAttrib(GL_ALL_ATTRIB_BITS);
			glEnable(GL_CULL_FACE);
			glFrontFace(GL_BACK);

			drawBox( iter );

			glFrontFace(GL_FRONT);
			glPopAttrib();
			for(int i = 0; i < (int)tex3d.size(); i++)
			{
				tex3d[i]->unbind(i);
			}
		}
		unbindShaders(context);
	}

}


std::vector<GLTexture*> X4::GLVolumeRaycastMaterial::getVolumeTexture( GLContext * context, SceneObjectIterator *iter, MemoryManager * mem )
{
	std::vector<GLTexture*> result;
	result.clear();
	VolumetricIterator * vIter = dynamic_cast<VolumetricIterator *>( iter );
	if (!vIter)
		return result;
	SecondaryDataStructureIterator *attIt = dynamic_cast<SecondaryDataStructureIterator*>(iter);
	if (!attIt)
		return result;

	// check if descriptor is valid
	bool validDesc = true;
	int componentCounter = 0;
	for( unsigned i=0;i<m_ElementDescriptor->getNumAttributes();i++ )
	{
		validDesc &= m_ElementDescriptor->getAttribute(i).dataFormat == VAD::DATA_FORMAT_FLOAT32;
		componentCounter += m_ElementDescriptor->getAttribute(i).numComponents;
	}
	if(!validDesc)
		return result;
	if(componentCounter == 0)
		return result;

	// generate attachment name
	std::string attName = "vol";
	card64 contextID = context->getRootContext()->getContextNumber();
	attName += intToStr(contextID) + " " + intToStr(componentCounter);
	
	// try to get attachment
	AttachedDataContainer * attachments = attIt->lockAttachments();
	GLMultiTextureAttachment * texAtt = dynamic_cast<GLMultiTextureAttachment*>(attachments->getData(attName));

	// compare descriptors (if attachment exists)
	std::string elementDesc = m_ElementDescriptor->getDescription();
	if( texAtt && (texAtt->getTextureDescription() != elementDesc) )
	{
		delete texAtt;
		texAtt = NULL;
	}
	
	if( !texAtt )
	{
		// texture attachment doesn't exist, so we need to create it
		const RegularVolume *volume = vIter->lockVolumeBlockRead();
		if(!volume || volume->getExtends()*makeVector3i(1,1,1) == 0 )
			return result;

		// setup attachment and texture
		texAtt = new GLMultiTextureAttachment;
		texAtt->setTextureDescription(elementDesc);

		// create texture data in cpu memory & copy to gpu
		std::vector<card8> data;	
		m_ElementDescriptor->convertVertices(volume->getVoxelData(), data);
		float* fData = (float*) &data[0];
		for(int i = 0; i <= (componentCounter-1)/4; i++)
		{
			int elementPosOffset = i*4;
			int numComponents = min(4,componentCounter-elementPosOffset);
			std::vector<float32> texData(numComponents*volume->getNumVoxels());
			for(int j = 0; j < volume->getNumVoxels(); j++)
			{	
				for(int k = 0; k < numComponents; k++)
				{	
					texData[(j*numComponents)+k] = fData[(j*componentCounter)+elementPosOffset+k];
				}
			}
			GLTexture3D * tex3d = new GLTexture3D(context);
			Vector3i vDimensions = volume->getExtends();
			texAtt->addTA(tex3d);
			tex3d->createFloatTextureFromMemory( vDimensions[0], vDimensions[1], vDimensions[2], numComponents, (char*)&texData[0],true, false);
		}
//		texAtt->setupTA(attName, mem, true);
		texAtt->setup(attName,AttachedData::ADF_DELETEONUPDATE,mem);	// overwrite attachment name
		attachments->attachData(texAtt);

		/*
		// while we are still here and with all the volume stuff lying around, let's compute a histogram
		HistogramAttachment * hisAtt = dynamic_cast<HistogramAttachment*>(attachments->getData(HistogramAttachment::defaultName()));
		if( hisAtt == NULL && volume->getVoxelData()->providesAttribute("intensity"))
		{
			float maxValue = 0;
			const PointSet * voxels = volume->getVoxelData();
			AAT intensityAAT = voxels->getAAT("intensity");
			for( unsigned i=0;i<voxels->getNumEntries();i++ )
			{
				maxValue = max(maxValue,voxels->get1f(i,intensityAAT));
			}
			unsigned numBins = 128;
			hisAtt = new HistogramAttachment;
			hisAtt->setup(HistogramAttachment::defaultName(),AttachedData::ADF_DELETEONUPDATE,0);
			hisAtt->m_Data = std::vector<float>(numBins,0);
			hisAtt->m_Range = makeVector2f(0,maxValue);
			for( unsigned i=0;i<voxels->getNumEntries();i++ )
			{
				hisAtt->m_Data[min(voxels->get1f(i,intensityAAT) / maxValue * numBins,numBins-1)]++;
			}
			attachments->attachData(hisAtt);
			m_TransferFunctionAlpha->setupEditorHistogram(hisAtt->m_Data);
		}
		*/

		// unlock volume 
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		bIt->unlock();

		result = texAtt->getTextures();
		return result;
	}
	else
	{
		// try to get histogram attachment for editor
		/*
		HistogramAttachment * hisAtt = dynamic_cast<HistogramAttachment*>(attachments->getData(HistogramAttachment::defaultName()));
		if( hisAtt )
		{
			m_TransferFunctionAlpha->setupEditorHistogram(hisAtt->m_Data);
		}
		*/

		result = texAtt->getTextures();
		return result;
	}
}

void X4::GLVolumeRaycastMaterial::drawBox( SceneObjectIterator *iter )
{
	VolumetricIterator * vIter = dynamic_cast<VolumetricIterator *>( iter );

	BoundingBox3f bb = vIter->getVolumeRegion();

	std::vector<Vector3f> bb_corners;
	bb_corners.push_back( makeVector3f( bb.lowerCorner[0],bb.lowerCorner[1],bb.lowerCorner[2] ));
	bb_corners.push_back( makeVector3f( bb.upperCorner[0],bb.lowerCorner[1],bb.lowerCorner[2] ));
	bb_corners.push_back( makeVector3f( bb.lowerCorner[0],bb.upperCorner[1],bb.lowerCorner[2] ));
	bb_corners.push_back( makeVector3f( bb.upperCorner[0],bb.upperCorner[1],bb.lowerCorner[2] ));
	bb_corners.push_back( makeVector3f( bb.lowerCorner[0],bb.lowerCorner[1],bb.upperCorner[2] ));
	bb_corners.push_back( makeVector3f( bb.upperCorner[0],bb.lowerCorner[1],bb.upperCorner[2] ));
	bb_corners.push_back( makeVector3f( bb.lowerCorner[0],bb.upperCorner[1],bb.upperCorner[2] ));
	bb_corners.push_back( makeVector3f( bb.upperCorner[0],bb.upperCorner[1],bb.upperCorner[2] ));

	// ... and 6 faces
	std::vector<Vector3f> points;
	points.push_back( bb_corners[0] );
	points.push_back( bb_corners[1] );
	points.push_back( bb_corners[3] );
	points.push_back( bb_corners[2] );

	points.push_back( bb_corners[5] );
	points.push_back( bb_corners[4] );
	points.push_back( bb_corners[6] );
	points.push_back( bb_corners[7] );

	points.push_back( bb_corners[1] );
	points.push_back( bb_corners[5] );
	points.push_back( bb_corners[7] );
	points.push_back( bb_corners[3] );

	points.push_back( bb_corners[4] );
	points.push_back( bb_corners[0] );
	points.push_back( bb_corners[2] );
	points.push_back( bb_corners[6] );

	points.push_back( bb_corners[1] );
	points.push_back( bb_corners[0] );
	points.push_back( bb_corners[4] );
	points.push_back( bb_corners[5] );

	points.push_back( bb_corners[2] );
	points.push_back( bb_corners[3] );
	points.push_back( bb_corners[7] );
	points.push_back( bb_corners[6] );

	glBegin( GL_QUADS );
	for(card32 i = 0; i < points.size(); i++)
	{
		Vector3f point = points[i];
		glVertex3fv( point.data());
	}
	glEnd();


}

bool X4::GLVolumeRaycastMaterial::bindShaders( GLContext * context )
{
	if( bindShader(m_VertexShader,context,m_glVertexShaders)
		&& bindShader(m_PixelShader,context,m_glPixelShaders) )
	{
		m_glLastVertexShader = m_glVertexShaders[context];
		m_glLastPixelShader = m_glPixelShaders[context];
		return true;
	}
	return false;	
}

void X4::GLVolumeRaycastMaterial::unbindShaders( GLContext * context )
{
	unbindShader(context,m_glVertexShaders);
	unbindShader(context,m_glPixelShaders);
}

bool X4::GLVolumeRaycastMaterial::bindShader( Shader * shader, GLContext * context, TShaderMap &shaderMap )
{
	if( shaderMap.find(context) == shaderMap.end() )
	{
		GLShader * vshader = shader->createShaderInstance(context);
		if( vshader )
		{
			if( !vshader->compile(shader->getSource()))
			{
				delete vshader;
				return false;
			}
		}
		shaderMap[context]	= vshader;
		vshader->bind();
		return true;
	}
	else
	{
		if( shader->wasChanged())
		{
			delete shaderMap[context];
			GLShader * vshader = shader->createShaderInstance(context);
			if( vshader )
			{
				if( !vshader->compile(shader->getSource()))
				{
					delete vshader;
					return false;
				}
			}
			shaderMap[context]	= vshader;
			vshader->bind();
			return true;
		}
	}
	
	shaderMap[context]->bind();
	return true;

}

void X4::GLVolumeRaycastMaterial::unbindShader( GLContext * context, TShaderMap &shaderMap )
{
	if( shaderMap.find(context) != shaderMap.end() )
		shaderMap[context]->unbind();
}

void X4::GLVolumeRaycastMaterial::bindTransferFunctionTexture(GLContext * context, unsigned stage)
{
	// try to get existing texture
	if( m_glTransferFunctionTextures.find(context) != m_glTransferFunctionTextures.end() )
	{
		GLTexture * tex = m_glTransferFunctionTextures[context];
		// TODO: dirty flag is not context dependent...
		if( m_TransferFunctionAlpha->isDirty() || m_TransferFunctionColor->isDirty() )
		{
			delete tex;
			m_glTransferFunctionTextures[context] = NULL;
		}
		else
		{
			tex->bind(stage);
			return;
		}
	
	}

	// create transfer function texture
	m_TransferFunctionColor->setDirty(false);
	m_TransferFunctionAlpha->setDirty(false);
	card32 dimX = 512;
	card32 dimY = 16;
	std::vector<card32> colorTextureImage = m_TransferFunctionColor->createImage(dimX,dimY);
	std::vector<card32> alphaTextureImage = m_TransferFunctionAlpha->createImage(dimX,dimY);
	for( unsigned i=0;i<colorTextureImage.size();i++ )
	{
		card32 tmp = alphaTextureImage[i] & 0xff;
		colorTextureImage[i] |= tmp << 24;
	}

	GLTexture2D * tex2D = new GLTexture2D( context );
	tex2D->createFromMemory( dimX, dimY, 4, (char*)&colorTextureImage[0], false );
	tex2D->bind(stage);
	glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
	glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );

	m_glTransferFunctionTextures[context] = tex2D;
}