#ifndef GLVolumeRaycastMaterialH
#define GLVolumeRaycastMaterialH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "GLMaterial.h"
//---------------------------------------------------------------------------

namespace X4
{

	class GLVolumeRaycastMaterialEditorWidget;
	class GLShader;
	class GLContext;
	class GLTexture;
	class VertexDescriptor;
	class Shader;
	class InterpFunctionTextureAttachment;
	class InterpFunctionTextureAttachmentRGB;

/**
	material class for volume data sets - performs raycasting in pixelshader
*/
class X3RENDERING_API GLVolumeRaycastMaterial : public GLMaterial
{
	X4_CLASS( GLVolumeRaycastMaterial )

private:

	friend	GLVolumeRaycastMaterialEditorWidget;

	VertexDescriptor		*		m_ElementDescriptor;
	Shader					*		m_VertexShader;
	Shader					*		m_PixelShader;
	InterpFunctionTextureAttachment	* m_TransferFunctionAlpha;
	InterpFunctionTextureAttachmentRGB	* m_TransferFunctionColor;

	GLShader * m_glLastVertexShader;		// stores shader from last bindShaders() call
	GLShader * m_glLastPixelShader;

	typedef std::map<GLContext*,GLShader*> TShaderMap;

	TShaderMap   m_glVertexShaders;
	TShaderMap   m_glPixelShaders;

	std::map<GLContext*,GLTexture*> m_glTransferFunctionTextures;



protected:

	bool bindShader(Shader * shader, GLContext * context, TShaderMap &shaderMap );
	void unbindShader(GLContext * context, TShaderMap &shaderMap );
	bool bindShaders( GLContext * context );
	void unbindShaders( GLContext * context );
	std::vector<GLTexture*> getVolumeTexture( GLContext * context, SceneObjectIterator *iter, MemoryManager * mem );
	void drawBox(SceneObjectIterator *iter);
	void bindTransferFunctionTexture(GLContext * context,unsigned stage);

public:
	GLVolumeRaycastMaterial(void);
	~GLVolumeRaycastMaterial(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual void draw(Scene *scene, SceneGraphState *state, SceneObject *obj,
		SceneObjectIterator *iter, GLContext *context, MemoryManager * mm = NULL,
		const int32 includeFlags = 0, const int32 excludeFlags = PF_FLAG_INVALID );


};

}

#endif