//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GLVolumeRenderer2DSlices.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ObjectViewsTable.h"
#include "Camera.h"
#include "Tex2dVolumeRenderer.h"
#include "DebugOutput.h"
#include "GLHelper.h"
#include "RegularVolume.h"
#include "Tex2dVolumeRenderer.h"
#include "BooleanClassProperty.h"
#include "NumericalClassProperty.h"
#include "StringClassProperty.h"
#include "Shader.h"
#include "GLShaderMaterial.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



static std::string descrTransf(card32 i) {
   if (i==0) return "whiteOpacityMapping";
   if (i==1) return "intensityOpacityMapping";
   if (i==2) return "blueRedOpacityMapping";
   if (i==3) return "noMapping";
   return "undefined";   
}

static std::string descrAuto(card32 i) {
   if (i==0) return "false";
   return "true";   
}


IMPLEMENT_X4_CLASS( GLVolumeRenderer2DSlices, 2) {
	BEGIN_CLASS_INIT( GLVolumeRenderer2DSlices );

	ADD_BOOLEAN_PROP(showCoords, 0)
	ADD_CARD32_PROP_UPDATE_DESCR(automatic, 0, &invalidate, descrAuto, 2, NULL)

	ADD_STRING_VAR_ARRAY_PROP(channels, 2)
	ADD_CARD32_PROP_UPDATE_DESCR(transferFunction, 0, &invalidate, descrTransf, 4, NULL)
		
	ADD_FLOAT32_PROP_UPDATE(scale, 0, &invalidate)
	ADD_FLOAT32_PROP_UPDATE(offset, 0, &invalidate)
	ADD_FLOAT32_PROP_UPDATE(gamma, 0, &invalidate)

	ADD_OBJECT_PROP(vertexShader, 1, Shader::getClass(), true)
	ADD_OBJECT_PROP(pixelShader, 1, Shader::getClass(), true)
}

// ----


GLVolumeRenderer2DSlices::GLVolumeRenderer2DSlices() {
	needRebuild = true;
	showCoords = true;
	automatic = 0;
	transferFunction = 0;
	scale = 1;
	offset = 0;
	gamma = 1;

   static const std::string VSHADER =
      std::string("struct TIn\n") +
      std::string("{\n") +
      std::string("   float4 pos : ATTR0;\n") +
      std::string("   float3 texcoords : ATTR1;\n") +
      std::string("};\n") +
      std::string("\n") +
      std::string("struct TOut\n") +
      std::string("{\n") +
      std::string("   float4 pos : POSITION;\n") +
      std::string("   float2 texcoords : TEX_COORD0;\n") +
      std::string("};\n") +
      std::string("\n") +
      std::string("TOut main( TIn IN, uniform float4x4 matMVP)\n") +
      std::string("{\n") +
      std::string("   TOut OUT;\n") +
      std::string("   OUT.pos = mul( matMVP, IN.pos );\n") +
      std::string("   OUT.texcoords = IN.texcoords;\n") +
      std::string("   return OUT;\n") +
      std::string("}\n");

   static const std::string PSHADER =
      std::string("struct TIn\n") +
      std::string("{\n") +
      std::string("	  float2 texcoords : TEX_COORD0;\n") +
      std::string("};\n") +
      std::string("\n") +
      std::string("float4 main(TIn IN) : COLOR\n") +
      std::string("{\n") +
      std::string("	  return float4(IN.texcoords,0,0.1);\n") +
      std::string("}\n");

   GLShaderMaterial *result = new GLShaderMaterial;

   VertexDescriptor *vd = result->getVertexDescriptor();
   vd->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));

   vertexShader = result->getVertexShader();
   vertexShader->setShaderType(Shader::VERTEX);
   vertexShader->setShaderClass(Shader::CLASS_CG);
   vertexShader->setSource(VSHADER);

   pixelShader = result->getFragmentShader();
   pixelShader->setShaderType(Shader::FRAGMENT);
   pixelShader->setShaderClass(Shader::CLASS_CG);
   pixelShader->setSource(PSHADER);
}

void GLVolumeRenderer2DSlices::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

GLVolumeRenderer2DSlices::~GLVolumeRenderer2DSlices() {
   delete vertexShader;
   delete pixelShader;
}



inline int32 GLVolumeRenderer2DSlices::mapValue(float32 v) {
	v += offset;
	v *= scale;
	if (v<0) v=0;
	if (v>1) v=1;
	v = pow(v, 1.0f/gamma);
	return (int32)(v*255.0f);
}

Vector4ub GLVolumeRenderer2DSlices::whiteOpacityMapping(void *context, const StaticVector<float32, 1> &in) {
	int32 v = ((GLVolumeRenderer2DSlices*)context)->mapValue(in[0]);
	if (v > 255) v = 255;
	if (v < 0) v = 0;
	return makeVector4ub(255,255,255, v);
}

Vector4ub GLVolumeRenderer2DSlices::intensityOpacityMapping(void *context, const StaticVector<float32, 1> &in) {
	int32 v = ((GLVolumeRenderer2DSlices*)context)->mapValue(in[0]);
	if (v > 255) v = 255;
	if (v < 0) v = 0;
	return makeVector4ub(v,v,v,v);
}

Vector4ub GLVolumeRenderer2DSlices::blueRedOpacityMapping(void *context, const StaticVector<float32, 1> &in) {
	int32 v = ((GLVolumeRenderer2DSlices*)context)->mapValue(in[0]);
	if (v > 255) v = 255;
	if (v < 0) v = 0;
	return makeVector4ub(v,0,255-v,v);
}


void GLVolumeRenderer2DSlices::drawGL(RegularVolume *volume, Tex2dVolumeRenderer *&textureStack, Camera &cam, float32 minValue, float32 maxValue) {
	if (volume == NULL) return;
	if (channels.empty()) return;
   for (unsigned ch=0; ch<channels.size(); ch++) {
      if (!volume->getDescr()->providesAttribute(channels[ch])) {
         warning("GLVolumeRenderer2DSlices::drawGL() - not all channels found.");
         return;
      }
	   VAD vad = volume->getVoxelData()->getDescr()->getAttribute(channels[ch]);
	   if (vad.dataFormat != VAD::DATA_FORMAT_FLOAT32) {
		   warning("GLVolumeDisplayer::drawGL() - can only display float channels.");
		   return;
	   }
	   if (vad.numComponents > 4 || vad.numComponents == 0) {
		   warning("GLVolumeDisplayer::drawGL() - can only display 1-4 dim channels.");	   
		   return;
	   }
   }

	if (showCoords) {
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		BoundingBox3f bb = volume->getBoundingBox();
		glTranslatef(-bb.lowerCorner[0]-bb.getSideLength(0)*0.1, -bb.lowerCorner[1]-bb.getSideLength(1)*0.1, -bb.lowerCorner[2]-bb.getSideLength(2)*0.1);
		glScalef(bb.getSideLength(0), bb.getSideLength(1), bb.getSideLength(2));
		GLHelper::drawCoords();
		glPopMatrix();
	}
	if (volume != NULL) {
		if (automatic) {
			double delta = maxValue - minValue;
			if (delta < 1e-37) delta = 1e-37;
			scale = 1.0f/delta;
			offset = -minValue/delta;
			ObjectViewsTable::update(this);
		}
		Tex2dVolumeRenderer::TransferFunction mapping = NULL;
		if (transferFunction == 0) mapping = whiteOpacityMapping;
		if (transferFunction == 1) mapping = intensityOpacityMapping;
		if (transferFunction == 2) mapping = blueRedOpacityMapping;
		if (needRebuild) {
			textureStack->disposeTextures();
			needRebuild = false;
		}
		
      textureStack->drawVolume(volume, cam.getOrthoNormViewDirection(), channels, mapping, this);
		
	}
}


void GLVolumeRenderer2DSlices::invalidate() 
{
   needRebuild=true;
}


std::string GLVolumeRenderer2DSlices::getChannel() const {
   if (channels.empty()) return "";
   return channels[0];
}

void GLVolumeRenderer2DSlices::setChannel(std::string s) {
   channels.clear();
   channels.resize(1);
   channels[0] = s;
   invalidate();
} 

