//---------------------------------------------------------------------------
#ifndef InterpFunctionTextureAttachmentH
#define InterpFunctionTextureAttachmentH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "MaterialAttachment.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "VariableArrayClassProperty.h"
//---------------------------------------------------------------------------


namespace X4
{
class InterpFunctionTextureAttachmentEditorWidget;

class X3RENDERING_API InterpFunctionTextureAttachment : public TextureAttachment
{
private:

   static MetaClass *InterpFunctionTextureAttachmentMetaClass;
   static Object* newInstance();

   friend InterpFunctionTextureAttachmentEditorWidget;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   void write( OutputObjectStream * s ) const;
   void read( InputObjectStream * s );

   virtual card32 getTextureType() const  {return TYPE_2D; }
   virtual GLTexture * createGLTexture(GLContext * context);

   virtual void assign( const Object * obj,  X4CopyContext *context = NULL );

   InterpFunctionTextureAttachment();
   ~InterpFunctionTextureAttachment();

   struct TPoint
   {
      Vector2f p;
      float weight;
   };

   mpcard getNumPoints() const { return m_Points.size(); }
   TPoint getPoint( card32 i ) {return m_Points[i]; }
	
   void addPoint( TPoint point );
   void deletePoint( float x );
   void movePoint( card32 index, Vector2f dir );
   card32 getPointIndex( float x, float y );

   void setupEditorHistogram( const std::vector<float> &histogram );
   std::vector<float> & getHistogram() {return m_Histogram;}

   std::vector<card32> createImage( card32 dimX, card32 dimY );

	bool isDirty() const {return m_Dirty;}
	void setDirty(bool b) {m_Dirty = b;}

private:

   card32 findPoint( float x );

   std::vector<TPoint>    m_Points;
   card32            m_TextureResolution;    // default 128
	
   std::vector<float>	m_Histogram;
	bool				m_Dirty;

};

/*
	stores a histogram
*/

class X3RENDERING_API HistogramAttachment : public AttachedData
{
	X4_CLASS( HistogramAttachment )
public:

	HistogramAttachment();

	static std::string defaultName() {return "I am a histogram";}

	std::vector<float>	m_Data;
	Vector2f			m_Range;

	IMPLEMENT_VAR_ARRAY_METHODS_STL( m_Data );

};


}

#endif
 