#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "InterpFunctionTextureAttachmentEditorWidget.h"
//---------------------------------------------------------------------------
#include "InterpFunctionTextureAttachment.h"
#include <qtGUI/QPainter>
#include <QWidget>
#include <qtGui/QMouseEvent>
#include "RGBAImage.h"
#include "RGBATextureAttachment.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_CLASS_EDITOR( InterpFunctionTextureAttachment )

InterpFunctionTextureAttachmentEditorWidget::InterpFunctionTextureAttachmentEditorWidget(QWidget * parent, Qt::WindowFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));

	m_Zoom.lowerCorner = makeVector2f(-0.1f,-0.1f);
	m_Zoom.upperCorner = makeVector2f(1.1f,1.1f);
}

InterpFunctionTextureAttachmentEditorWidget::~InterpFunctionTextureAttachmentEditorWidget(void)
{
}

void InterpFunctionTextureAttachmentEditorWidget::setup( InterpFunctionTextureAttachment * obj )
{
	m_Object = obj;
	float maxBin = 0.00001f;
	//for( card32 i=0;i<obj->m_Histogram.size();i++)
	//	maxBin = max( maxBin, obj->m_Histogram[i]);
	std::vector<float> tmp = obj->m_Histogram;
	sort( tmp.begin(),tmp.end());
	if( tmp.size() > 1)
		maxBin = max(maxBin,tmp[tmp.size()-2]);

	m_Image = QImage((int32)obj->m_Histogram.size(),40,QImage::Format_RGB32);
	for( card32 i=0;i<obj->m_Histogram.size();i++ )
	{
		for( int32 j=0;j<m_Image.height();j++ )
		{
			QColor col = ((float)j/(float)m_Image.height() < obj->m_Histogram[i] / maxBin) ? QColor(96,96,96):QColor(192,192,192);
			m_Image.setPixel(i,m_Image.height()-1-j, col.rgb());
		}
	}
}

void InterpFunctionTextureAttachmentEditorWidget::updateEditor()
{
}

#include "InterpFunctionTextureAttachmentEditorWidget_moc.h"
	
void InterpFunctionTextureAttachmentEditorWidget::mouseMoveEvent ( QMouseEvent * event ) 
{
	Vector2f dir = viewToObject(event->pos()) - viewToObject(m_LastPos);
	if( m_Mode == Mode_Move && m_SelectedPoint < m_Object->getNumPoints())
	{
		m_Object->movePoint( m_SelectedPoint, dir );
		m_Object->clearAttachments();
		update();
	}
	else
		m_SelectedPoint = 0xffff;


	m_LastPos = event->pos();
}

void InterpFunctionTextureAttachmentEditorWidget::mousePressEvent ( QMouseEvent * event ) 
{
	m_LastPos = event->pos();
	m_Mode = Mode_None;
	if( event->button() == Qt::LeftButton )
	{
		InterpFunctionTextureAttachment::TPoint p;
		p.weight = 1;
		p.p = viewToObject(event->pos());
		m_Object->addPoint( p );
		m_Object->clearAttachments();
		update();
	}
	if( event->button() == Qt::RightButton )
	{
		Vector2f point = viewToObject(event->pos());
		m_SelectedPoint = m_Object->getPointIndex( point[0],point[1] );
		InterpFunctionTextureAttachment::TPoint p;
		p = m_Object->getPoint(m_SelectedPoint);
		m_Mode = Mode_Move;

	}

}

void InterpFunctionTextureAttachmentEditorWidget::mouseReleaseEvent ( QMouseEvent * event ) 
{
}

void InterpFunctionTextureAttachmentEditorWidget::paintEvent ( QPaintEvent * event ) 
{
	QPainter painter(this);
	float imageDimX = width();
	float imageDimY = height();
	QRect tmpRect;
	tmpRect.setBottomLeft( objectToView(makeVector2f(0,0)) );
	tmpRect.setTopRight( objectToView(makeVector2f(1,1)) );

	painter.drawImage( tmpRect, m_Image);

	painter.setPen( QColor(96,96,255));
	painter.setBrush( QColor(96,96,255));
	card32 gridDim = 10;
	for( card32 i=0;i<=gridDim;i++ )
	{
		Vector2f startX = makeVector2f( (float)i/(float)gridDim,0);
		Vector2f endX = makeVector2f(startX[0],1);
		Vector2f startY = makeVector2f( 0,(float)i/(float)gridDim);
		Vector2f endY = makeVector2f(1,startY[1]);

		painter.drawLine( objectToView(startX),objectToView(endX));
		painter.drawLine( objectToView(startY),objectToView(endY));
	}

	painter.setPen( Qt::black );
	painter.setBrush( Qt::black );
	for( card32 i=0;i<m_Object->getNumPoints()-1;i++ )
	{
		QPoint p1 = objectToView(m_Object->getPoint( i ).p);
		QPoint p2 = objectToView(m_Object->getPoint( i+1 ).p);
		painter.drawLine( p1, p2 );
	}

	painter.setPen( Qt::red );
	painter.setBrush( Qt::red );
	for( card32 i=0;i<m_Object->getNumPoints();i++ )
	{
		QPoint p1 = objectToView(m_Object->getPoint( i ).p);
		painter.drawEllipse( p1.x()-2, p1.y()-2, 5, 5 );
	}

}

void InterpFunctionTextureAttachmentEditorWidget::resizeEvent ( QResizeEvent * event ) 
{
}

Vector2f InterpFunctionTextureAttachmentEditorWidget::viewToObject( const QPoint &p) const 
{
	return makeVector2f( (float)p.x()/(float)width()*(m_Zoom.upperCorner[0]-m_Zoom.lowerCorner[0])+m_Zoom.lowerCorner[0],
		1.0f - (float)p.y()/(float)height()*(m_Zoom.upperCorner[1]-m_Zoom.lowerCorner[1])-m_Zoom.lowerCorner[1] );
}

QPoint InterpFunctionTextureAttachmentEditorWidget::objectToView( const Vector2f &p) const 
{
	return QPoint( (p[0] - m_Zoom.lowerCorner[0])/(m_Zoom.upperCorner[0]-m_Zoom.lowerCorner[0]) * (float)width(),
		(1.0f - p[1] -  m_Zoom.lowerCorner[1])/(m_Zoom.upperCorner[1]-m_Zoom.lowerCorner[1]) * (float)height() );
}
