//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "MaterialAttachment.h"
//---------------------------------------------------------------------------
#include "FixedArrayClassProperty.h"
#include "GLTexture.h"
#include "LinearAlgebra.inline.h"
#include "GLAttachments.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* Matrix4fAttachment::Matrix4fAttachmentMetaClass = NULL;

const MetaClass* Matrix4fAttachment::getInstanceClass() const {
   return getClass();
}

Object* Matrix4fAttachment::newInstance() {
   return new Matrix4fAttachment;
}

const MetaClass* Matrix4fAttachment::getClass() {
   if (Matrix4fAttachmentMetaClass==NULL) {
      Matrix4fAttachmentMetaClass = new MetaClass("Matrix4fAttachment", "no description.",
                                  newInstance, sizeof(Matrix4fAttachment));
   }
   return Matrix4fAttachmentMetaClass;
}

void Matrix4fAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   Matrix4fAttachmentMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(Matrix4fAttachment)
	ADD_FLOAT32_FIXED_ARRAY_PROP(m_Matrix, 0, 16, 4)

}

void Matrix4fAttachment::shutdown() {
   delete Matrix4fAttachmentMetaClass;
}


// ----

IMPLEMENT_X4_CLASS(Vector4fAttachment, 1)
{
	BEGIN_CLASS_INIT(Vector4fAttachment);
	ADD_FLOAT32_FIXED_ARRAY_PROP(m_Vector, 0, 4, 4)
	ADD_CARD32_PROP( m_NumComponentsUsed, 1 );
}

IMPLEMENT_X4_CLASS(Vector4iAttachment, 1)
{
	BEGIN_CLASS_INIT(Vector4iAttachment);
	ADD_INT32_FIXED_ARRAY_PROP(m_Vector, 0, 4, 4)
	ADD_CARD32_PROP( m_NumComponentsUsed, 1 );
}

// ----

MetaClass* TextureAttachment::TextureAttachmentMetaClass = NULL;

const MetaClass* TextureAttachment::getInstanceClass() const {
   return getClass();
}

const MetaClass* TextureAttachment::getClass() {
   if (TextureAttachmentMetaClass==NULL) {
      TextureAttachmentMetaClass = new MetaClass("TextureAttachment", "no description.",
                                  NULL, sizeof(TextureAttachment));
   }
   return TextureAttachmentMetaClass;
}

void TextureAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   TextureAttachmentMetaClass->copyParentProperties();
}

void TextureAttachment::shutdown() {
   delete TextureAttachmentMetaClass;
}


// ----

void Matrix4fAttachment::assign(const Object* obj, X4CopyContext *context) {
   AttachedData::assign( obj, context );
   const Matrix4fAttachment *other = dynamic_cast<const Matrix4fAttachment*>(obj);
   x4Assert(other);
   this->m_Matrix = other->m_Matrix;
}

template<class FloatType, unsigned dim>
void VectorAttachmentBase<FloatType, dim>::assign(const Object* obj, X4CopyContext *context)
{
   AttachedData::assign( obj, context );
   const VectorAttachmentBase<FloatType, dim>*other = dynamic_cast<const VectorAttachmentBase<FloatType, dim>*>(obj);
   x4Assert(other);
   this->m_Vector = other->m_Vector;
}


void TextureAttachment::assign( const Object * obj,  X4CopyContext *context )
{
   AttachedData::assign( obj, context );
   TextureAttachment * ta = (TextureAttachment*)obj;
   m_Container->assign( ta->m_Container );
}

GLTexture * TestTextureAttachment::createGLTexture( GLContext * context )
{
   int size = 64;

   std::vector<card8> lightTab;
   lightTab.push_back(20);
   lightTab.push_back(60);
   lightTab.push_back(70);
   lightTab.push_back(80);
   lightTab.push_back(140);
   lightTab.push_back(200);
   lightTab.push_back(210);
   lightTab.push_back(255);

   std::vector<card8> tmp(size*size*4);
   for( int i=0;i<size;i++ )
   {
      float f = (float)i/(float)size;

      card8 col = lightTab[ f * lightTab.size() ];

      for( int j=0;j<size;j++ )
      {
         tmp[(j*size+i)*4+0] = col;
         tmp[(j*size+i)*4+1] = col;
         tmp[(j*size+i)*4+2] = col;
         tmp[(j*size+i)*4+3] = col;
      }
   }

   GLTexture2D * ret = new GLTexture2D( context );
   ret->createFromMemory( size, size, 4, (char*)&tmp[0], true );
   return ret;
}

IMPLEMENT_X4_CLASS(RandomTextureAttachment,0)
{
}

GLTexture * X4::RandomTextureAttachment::createGLTexture(GLContext * context)
{
   int size = 256;

   std::vector<card8> tmp(size*size*4);
   for( int i=0;i<size;i++ )
   {
		for( int j=0;j<size;j++ )
		{
			tmp[(j*size+i)*4+0] = (card8)rand();
			tmp[(j*size+i)*4+1] = (card8)rand();
			tmp[(j*size+i)*4+2] = (card8)rand();
			tmp[(j*size+i)*4+3] = (card8)rand();
		}
   }

   GLTexture2D * ret = new GLTexture2D( context );
   ret->createFromMemory( size, size, 4, (char*)&tmp[0], true );
   return ret;
}


GLTexture * X4::TextureAttachment::getCachedTexture(GLContext * glContext)
{
	std::string texID = GLTextureAttachment::generateID( glContext->getRootContext(), "" );
	GLTextureAttachment * glTexAtt = dynamic_cast<GLTextureAttachment*>(fetchData(texID));
	if( !glTexAtt )
	{
		glTexAtt = new GLTextureAttachment;

		GLTexture * texture = createGLTexture( glContext );
		glTexAtt->setupTA( texture, "", NULL, true );
		attachData( glTexAtt );
	}

	return glTexAtt->getTexture();
}