//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "PCTexture.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.inline.h"
#include "PointSet.inline.h"
#include <QtGui/QImage>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



PCTexture::PCTexture()
{
   texture = NULL;

   pcIt = NULL;
   basicIt = NULL;
   paramIt = NULL;

   h=2;
   w=2;
   makeEmptyTex(2,2);
}

void PCTexture::setup(){
   delete pcIt;

   pcIt = texture->createIterator(SOIT(SOIT::CAP_PARAMETRIC_PC));
   basicIt = dynamic_cast<BasicPointCloudIterator*>(pcIt);
   paramIt = dynamic_cast<ParametricPointCloudIterator*>(pcIt);
   x4Assert (basicIt != NULL && paramIt != NULL);

   //Input Dimensions
   w=paramIt->getWidth();
   h=paramIt->getHeight();

   //To calc Normals we need x/y distace
   left=1.0f/w;
   down=1.0f/h;

   rad=1.0f;
   raise=true;
   dScale=0.2f;

   // Point Cloud Channels
   COLOR = texture->getAAT("color");
   POSITION = texture->getAAT("position");
   NORMAL = texture->getAAT("normal");

}



void PCTexture::makeEmptyTex(card32 iw, card32 ih){
   PointSet *result = new PointSet();

   VertexDescriptor vd;
   vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
   vd.pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
   vd.pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
   vd.pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));

   const AAT POSITION = vd.getToken("position");
   const AAT FLAGS = vd.getToken("flags");
   const AAT TCOLOR = vd.getToken("color");
   const AAT NORMAL = vd.getToken("normal");

   result->clearAndSetup(iw, ih, &vd);

   StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;
   card32 i=0;


   Vector3f color= makeVector3f(0.6f, 0.6f, 0.6f);
   Vector3f z_normal = makeVector3f(0.0f, 0.0f, -1.0f);
   normalize(z_normal);

   for (card32 x=0; x<iw; ++x)
      for (card32 y=0; y<ih; ++y)

      { // store points

         Vector3f pos;
         pos[0]=(float)x/(float)iw;
         pos[1]=(float)y/(float)ih;
         pos[2]=0.0f;

         result->set<float32, 3>(x, y, POSITION, pos);
         result->set<float32, 3>(x, y, TCOLOR, color);
         result->set<float32, 3>(x, y, NORMAL, z_normal);
         result->set<int32, 1>(x, y, FLAGS, NULL_FLAG);
         i++;
      }

   delete texture;
   texture = new ParametricInCorePointCloud();
   texture->setPointSet(result);

   setup();
   calcHeightMap();
   calcNormalMap();
}



PCTexture::PCTexture(std::string filename)
{
   texture = NULL;
      pcIt = NULL;
   basicIt = NULL;
   paramIt = NULL;

   loadBMP(filename);
}

PCTexture::~PCTexture()
{
   delete pcIt;
   delete texture;
}

void PCTexture::calcHeightMap(){

  ModifyablePointCloudIterator* modIt = dynamic_cast<ModifyablePointCloudIterator*>(pcIt);
  basicIt->reset();

  while(!basicIt->atEnd()) {
     Vector3f c = basicIt->get3f(COLOR);
     float32 hight=(c[0]+c[1]+c[2])/(3.0f);

     if (!raise) hight = -hight;

     Vector3f oldPos= basicIt->get3f(POSITION);
     oldPos[2]= dScale * hight;

     modIt->set3f(POSITION, oldPos);

     basicIt->next();
  }
  basicIt->reset();

}

void PCTexture::calcNormalMap(){

  ModifyablePointCloudIterator* modIt = dynamic_cast<ModifyablePointCloudIterator*>(pcIt);
  basicIt->reset();

  bool oldStyle=false;

  if(oldStyle){
   
   for (card32 x=0; x<(w-1); ++x)
      for (card32 y=0; y<(h-1); ++y){

         lookUp(x+1,y);
         float32 hx=getZ();
         lookUp(x,y+1);
         float32 hy=getZ();
         lookUp(x,y);                    
         float32 hh=getZ();

         float32 dZx=hx-hh;
         float32 dZy=hy-hh;

         Vector3f vDx= makeVector3f(left, 0.0f , dZx);
         Vector3f vDy= makeVector3f(0.0f, down , dZy);
         Vector3f normal;

         normal=vDy.crossProduct(vDx);

         if(normQuad(normal) < 0.0000000000001f || (dZx==0.0f && dZy == 0.0f))
            normal= makeVector3f(0.0f, 0.0f , -1.0f);
         else
            hx=hx;

         normal= normalize(normal);



     
         modIt->set3f(NORMAL, normal);
      }


   }else{

   int32 w_int = (int32) w;
   int32 h_int = (int32) h;

   for (int32 y=0; y<h_int; ++y)
      for (int32 x=0; x<w_int; ++x){
         // Randbehandlung:
         int32 xa = x-1;
         int32 xb = x+1;
         int32 ya = y-1;
         int32 yb = y+1;
         if (xa < 0)  xa = x;
         if (xb >= w_int)  xb = x;
         if (ya < 0)  ya = y;
         if (yb >= h_int)  yb = y;

         // calc tangent Vector vDx
         lookUp(xb,y);
         Vector3f xB = basicIt->get3f(POSITION);
         lookUp(xa,y);
         Vector3f xA = basicIt->get3f(POSITION);
         Vector3f vDx= xB-xA;
                 
         // calc tangent Vector vDy
         lookUp(x,yb);
         Vector3f yB = basicIt->get3f(POSITION);
         lookUp(x,ya);
         Vector3f yA = basicIt->get3f(POSITION);
         Vector3f vDy= yB-yA;

         vDy[1]= -vDy[1]; //fixes orientation

         //calc normal by crossProduct
         Vector3f normal=vDy.crossProduct(vDx);

         // handle zero length
         if((normQuad(normal) < 0.0000000000001f))
            normal= makeVector3f(0.0f, 0.0f , 1.0f);

         normal= normalize(normal);

         // write calculated normal
         lookUp(x,y);
         modIt->set3f(NORMAL, normal);
  }

   }//end if








   /*

   //=======================
	for (int y=0; y<h; y++)
		for (int x=0; x<w; x++){
         // am Rand anders
         int xa = x-1;
         int xb = x+1;
         int ya = y-1;
         int yb = y+1;
         if (xa<0)  xa = x;
         else if (xb>=m_ResolutionX)  xb = x;
         if (ya<0)  ya = y;
         else if (yb>=m_ResolutionY)  yb = y;

         // calc vx,vy
         Vector3f vx = (m_Particles[xb+y*m_ResolutionX]->Position) - (m_Particles[xa+y*m_ResolutionX]->Position);
         Vector3D vy = (m_Particles[x+yb*m_ResolutionX]->Position) - (m_Particles[x+ya*m_ResolutionX]->Position);

         // return your calculated normal here
         m_Particles[x+y*m_ResolutionX]->Normal = (vy.Cross(vx)).Normalized();
  }
      */

}


void PCTexture::loadBMP(std::string filename){
   delete texture;
   texture = loadFromFile(filename);
   setup();
   calcHeightMap();
   calcNormalMap();

}



ParametricInCorePointCloud* PCTexture::loadFromFile(std::string filename){
	QImage bitmap(filename.c_str());

	int h=bitmap.height();// ();//  img->Height;
	int w=bitmap.width();

	PointSet *result = new PointSet();

	VertexDescriptor vd;
	vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
	vd.pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));


	const AAT POSITION = vd.getToken("position");
	const AAT FLAGS = vd.getToken("flags");
	const AAT COLOR = vd.getToken("color");
	const AAT NORMAL = vd.getToken("normal");

	result->clearAndSetup(w, h, &vd);

	Vector3f z_normal = makeVector3f(0.0f, 0.0f, 1.0f);
	normalize(z_normal);

	float32   inv255 = 1.0f/255.0f;



	StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;
	card32 i=0;
	for (int x=0; x<w; ++x)
		for (int y=0; y<h; ++y)

		{ // store points

			QRgb pixel = bitmap.pixel(x,y);
			Vector3f color = makeVector3f(qRed(pixel),qGreen(pixel),qBlue(pixel))*inv255;

			Vector3f pos;
			pos[0]=(float)x/(float)w;
			pos[1]=(float)y/(float)h;
			pos[2]=0.0f;

			result->set<float32, 3>(x, y, POSITION, pos);
			result->set<float32, 3>(x, y, COLOR, color);
			result->set<int32, 1>(x, y, FLAGS, NULL_FLAG);
			result->set<float32, 3>(x, y, NORMAL, z_normal);

			i++;

		}

		ParametricInCorePointCloud *res=new ParametricInCorePointCloud;
		res->setPointSet(result);
		return res;
}

/*
void PCTexture::lookUp(const card32& x, const card32& y){
   paramIt->goTo(x,y);
}
*/

void PCTexture::setRadius(float32 r){
   rad=r*2.0f;

   left=rad/w;
   down=rad/h;
   calcNormalMap();

}

void PCTexture::setDisplacementScale(float32 s){

   dScale=s;
   calcHeightMap();
   calcNormalMap();
}


void PCTexture::setDisplacementMode(bool r){
   if(r==raise) return; //nothing to do
   raise=r;
   calcHeightMap();
   calcNormalMap();
}


void PCTexture::transformLookUp(const float32& fx, const  float32& fy){
  // fx*= (float)w;
  // fy*= (float)h;

   int32 x=(int)((fx/rad)*(float)w);
   int32 y=(int)((fy/rad)*(float)h);

   x+=w/2;
   y+=h/2;

   x=abs(x);
   y=abs(y);

   x=x % w;
   y=y % h;

   x4Assert (x>=0 && y>=0 && x<(int)w && y<(int)h);

   lookUp(x,y);
}

Vector3f PCTexture::getCol(){
   return basicIt->get3f(COLOR);
}

Vector3f PCTexture::getNormal(){
   return basicIt->get3f(NORMAL);
}

float32  PCTexture::getZ() const{
   return (basicIt->get3f(POSITION))[2];
}


QImage* PCTexture::getBMP(){
   QImage *bmp;
   bmp = new QImage(w,h,QImage::Format_RGB32);


   for (card32 y=0; y<h; y++) {
      Vector4ub *scanline = (Vector4ub*)(bmp->scanLine(y));
      for (card32 x=0; x<w; x++) {

         Vector4f col;
         paramIt->goTo(x,y);

         basicIt->get3f(COLOR);
         Vector3f c=basicIt->get3f(COLOR);
         col = makeVector4f(c[0], c[1], c[2], 0);
         col *= 255.0f;
         col[3] = 255.0f;
         Vector4ub colUB = makeVector4ub(clamp(col[2]), clamp(col[1]), clamp(col[0]), clamp(col[3]));

         *scanline = colUB;
         scanline++;
      }
   }



   return bmp;
}
