//---------------------------------------------------------------------------
#ifndef PCTextureH
#define PCTextureH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "ParametricInCorePointCloud.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace X4 {


class X3RENDERING_API PCTexture
{

   ParametricInCorePointCloud* texture;
   
   PointCloudIterator           *pcIt;
   BasicPointCloudIterator      *basicIt;
   ParametricPointCloudIterator *paramIt;

   AAT COLOR;
   AAT NORMAL;
   AAT POSITION;

   float32 rad,dScale;
   bool raise;


   float32 left;
   float32 down;

   static inline card8 clamp(float32 f) {
      if (f > 255) return 255;
      if (f < 0) return 0;
      return (card8)f;
   }


 public:
   card32 w,h;

   PCTexture();
   ~PCTexture();
   PCTexture(std::string filename);

   void loadBMP(std::string filename);
   void setup();

   void makeEmptyTex(card32 w, card32 h);
   void calcHeightMap();
   void calcNormalMap();

   void setDisplacementMode(bool raise);

   void setRadius(float32 r);
   void setDisplacementScale(float32);

   inline void lookUp(const card32& x,const  card32& y){paramIt->goTo(x ,(h-1)-y);} //fixes orientation
   void transformLookUp(const float32& x, const float32& y);

	static ParametricInCorePointCloud* loadFromFile(std::string filename);



   Vector3f getCol();
   Vector3f getNormal();
   float32  getZ() const;

   PointCloud* getPC(){return texture;};
   void setPC(ParametricInCorePointCloud* pc){ texture=pc;setup();};
   QImage*  getBMP();

};










}
#endif
 