//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PointsAsSpheresMaterial.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointCloudIterators.h"
#include "PointSet.h"
#include "SeparatorClassProperty.h"
#include "BooleanClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "RenderingTool.h"
#include "PointSet.inline.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( PointsAsSpheresMaterial , 2 ) {
	BEGIN_CLASS_INIT( PointsAsSpheresMaterial );
	ADD_SEPARATOR("position")
	ADD_STRING_PROP(positionChannel, 0)
	ADD_SEPARATOR("radius")
	ADD_BOOLEAN_PROP(useRadiusChannel, 0)
	ADD_STRING_PROP(radiusChannel, 0)
	ADD_FLOAT32_PROP(defaultRadius, 0)
	ADD_FLOAT32_PROP(radiusScale, 1)
	ADD_SEPARATOR("color")
	ADD_BOOLEAN_PROP(useColorChannel, 0)
	ADD_STRING_PROP(colorChannel, 0)
	ADD_VECTOR3F_PROP(defaultColor, 0)
	ADD_SEPARATOR("axes")
	ADD_BOOLEAN_PROP(useAxesChannel, 2)
	ADD_STRING_PROP(axesChannel, 2)
	ADD_MATRIX3F_PROP(defaultAxes, 2)
	ADD_VECTOR3F_PROP(axesScale, 2)
}

// ----

PointsAsSpheresMaterial::PointsAsSpheresMaterial() {
	positionChannel = "position";
	useRadiusChannel = true;
	radiusChannel = "sampleSpacing";
	defaultRadius = 0.01f;
	radiusScale = 1.0f;
	useColorChannel = true;
	colorChannel = "color";
	defaultColor = makeVector3f(0.9f, 0.2f, 0.1f);
	useAxesChannel = false;//complementary to useRadiusChannnel
	axesChannel = "axes";
	axesScale = makeVector3f(1.0f, 2.0f, 1.0f);
	defaultAxes = makeMatrix3f(0.1f,0.0f,0.0f,  0.0f,0.2f,0.0f,  0.0f,0.0f,0.15f);
	axesScaleChannel = "axesScale";
	setName("PointsAsSpheresMaterial");
}

void PointsAsSpheresMaterial::assign(const Object* obj, X4CopyContext *context) {
	copyObjectProperties(obj, this);
}

PointsAsSpheresMaterial::~PointsAsSpheresMaterial() {
}



void PointsAsSpheresMaterial::draw(
	Scene *scene, SceneGraphState *state, SceneObject *obj,
   SceneObjectIterator *iter, GLContext *context, MemoryManager * mm,
   const int32 includeFlags, const int32 excludeFlags)
{
	BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(iter);
	if (bit) {
		const PointSet *ps = bit->lockBlockRead();
		AAT positionAAT = NULL_AAT;
		AAT flagsAAT = NULL_AAT;
		AAT radiusAAT  = NULL_AAT;
		AAT colorAAT = NULL_AAT;
		
		AAT axesAAT = NULL_AAT;
		AAT axesScaleAAT = NULL_AAT;
		float32 radius = 0.01f; // defaultRadius; // not sure why, but default radius do not work
		Vector3f color = defaultColor;

		Matrix3f ellipsoidAxes = defaultAxes;
		
		if (ps->getDescr()->providesAttribute(positionChannel, 3, VAD::DATA_FORMAT_FLOAT32)) {
			positionAAT = ps->getAAT(positionChannel);
			if (useColorChannel && ps->providesAttribute(colorChannel, 3, VAD::DATA_FORMAT_FLOAT32)) {
				colorAAT = ps->getAAT(colorChannel);
			}
			if (useRadiusChannel && ps->providesAttribute(radiusChannel, 1, VAD::DATA_FORMAT_FLOAT32)) {
				radiusAAT = ps->getAAT(radiusChannel);
			}
			if (useAxesChannel && ps->providesAttribute(axesChannel, 9, VAD::DATA_FORMAT_FLOAT32)) {
				axesAAT = ps->getAAT(axesChannel);				
			}
			if (useAxesChannel && ps->providesAttribute(axesScaleChannel, 3, VAD::DATA_FORMAT_FLOAT32)) {
				axesScaleAAT = ps->getAAT(axesChannel);				
			}
			if (ps->providesAttribute("flags", 1, VAD::DATA_FORMAT_INT32)) {
				flagsAAT = ps->getAAT("flags");
			}
			
			glPushAttrib(GL_ALL_ATTRIB_BITS);
			glEnable(GL_DEPTH_TEST);
			glDisable(GL_BLEND);
			//glEnable(GL_LIGHTING); // turn off lighting, for having a sprite look
			glEnable(GL_COLOR_MATERIAL);
			//glEnable(GL_LIGHT0);

			float ambient[] = { 0.1f, 0.1f, 0.1f, 1.0 };
			float specular[] = { 1.0, 1.0, 1.0, 1.0 };
			float shininess[] = { 20.0 };
			float light_pos[] = { 1.0, 1.0, 1.0, 0.0 };

			glMaterialfv( GL_FRONT, GL_AMBIENT, ambient );
			glMaterialfv( GL_FRONT, GL_SPECULAR, specular );
			glMaterialfv( GL_FRONT, GL_SHININESS, shininess );
			glLightfv( GL_LIGHT0, GL_POSITION, light_pos );

			if (useRadiusChannel){
				GLUquadricObj *q = gluNewQuadric();
				glMatrixMode(GL_MODELVIEW);
				card32 n = ps->getNumEntries();
				for (card32 i=0; i<n; i++) {
					Vector3f position = ps->get3f(i, positionAAT);
					if (colorAAT != NULL_AAT) {
						color = ps->get3f(i, colorAAT);
					}
					if (radiusAAT != NULL_AAT) {
						radius = ps->get1f(i, radiusAAT);
					}
					if (flagsAAT != NULL_AAT) {
						int32 flags = ps->get1i(i, flagsAAT);
						bool valid = ((flags & includeFlags) == includeFlags) && ((flags & excludeFlags) == 0);
						if (!valid) continue;
					}
					glPushMatrix();
					glTranslatef(position[0], position[1], position[2]);
					glColor3fv(color.data());
					radius *= radiusScale;
					gluSphere(q, radius, 10,10);
					glPopMatrix();
				}
				gluDeleteQuadric(q);
				glPopAttrib();
			}
			if (useAxesChannel && axesAAT != NULL_AAT){
				GLUquadricObj *q = gluNewQuadric();
				glMatrixMode(GL_MODELVIEW);
				card32 n = ps->getNumEntries();
				for (card32 i=0; i<n; i++) {
					Vector3f position = ps->get3f(i, positionAAT);
					if (colorAAT != NULL_AAT) {
						color = ps->get3f(i, colorAAT);
					}
					if (axesAAT != NULL_AAT) {
						ellipsoidAxes = ps->get9f(i, axesAAT);
					}
					if (axesScaleAAT != NULL_AAT) {
						axesScale = ps->get3f(i, axesScaleAAT);
					}
					if (flagsAAT != NULL_AAT) {
						int32 flags = ps->get1i(i, flagsAAT);
						bool valid = ((flags & includeFlags) == includeFlags) && ((flags & excludeFlags) == 0);
						if (!valid) continue;
					}
				   float32 rotationMatrix[] = { ellipsoidAxes[0][0], ellipsoidAxes[0][1], ellipsoidAxes[0][2], 0,
						ellipsoidAxes[1][0], ellipsoidAxes[1][1], ellipsoidAxes[1][2], 0,
						ellipsoidAxes[2][0], ellipsoidAxes[2][1], ellipsoidAxes[2][2], 0,
						0,				    0,				   0,	              1 };
					glPushMatrix();
					glTranslatef( position[0], position[1], position[2] );
					glMultMatrixf( rotationMatrix );
					glScalef( axesScale[0], axesScale[1], axesScale[2] );
					glColor3fv(color.data());
					gluSphere( q, 1.0, 10, 10 );
					glPopMatrix();
					
				}
				gluDeleteQuadric(q);
				glPopAttrib();
			}
			}
		
		bit->unlock();
	}
}
