//---------------------------------------------------------------------------
#ifndef PointsAsSpheresMaterialH
#define PointsAsSpheresMaterialH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "GLMaterial.h"
//---------------------------------------------------------------------------

namespace X4 {

///
///   A simple immediate mode material that draws points in point clouds as spheres
///
class X3RENDERING_API PointsAsSpheresMaterial : public GLMaterial {
	X4_CLASS( PointsAsSpheresMaterial )
 private:
	std::string positionChannel;
	bool useRadiusChannel;
	std::string radiusChannel;
	float32 defaultRadius;
	float32 radiusScale;
	bool useColorChannel;
	std::string colorChannel;
	Vector3f defaultColor;
	bool useAxesChannel;
	std::string axesChannel;
	Matrix3f defaultAxes;//to remove
	std::string axesScaleChannel;
	Vector3f axesScale;
	
 public:
	PointsAsSpheresMaterial();

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

   virtual void draw(Scene *scene, SceneGraphState *state, SceneObject *obj,
      SceneObjectIterator *iter, GLContext *context, MemoryManager * mm = NULL,
      const int32 includeFlags = 0, const int32 excludeFlags = PF_FLAG_INVALID );

   std::string getPositionChannel() const {return positionChannel;}
   void setPositionChannel(std::string v) {positionChannel=v;}

   bool getUseRadiusChannel() const {return useRadiusChannel;}
   void setUseRadiusChannel(bool v) {useRadiusChannel=v;}

   std::string getRadiusChannel() const {return radiusChannel;}
   void setRadiusChannel(std::string v) {radiusChannel=v;}

   float32 getDefaultRadius() const {return defaultRadius;}
   void setDefaultRadius(float32 v) {defaultRadius=v;}

   float32 getRadiusScale() const {return radiusScale;}
   void setRadiusScale(float32 v) {radiusScale=v;}

   bool getUseColorChannel() const {return useColorChannel;}
   void setUseColorChannel(bool v) {useColorChannel=v;}

   std::string getColorChannel() const {return colorChannel;}
   void setColorChannel(std::string v) {colorChannel=v;}

   Vector3f getDefaultColor() const {return defaultColor;}
   void setDefaultColor(Vector3f v) {defaultColor=v;}

   ~PointsAsSpheresMaterial();
};

}

#endif