//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "RGBATextureAttachment.h"
//---------------------------------------------------------------------------
//#include "TextureAttachmentEditorFrames.h"
#include "GLContext.h"
#include "GLTexture.h"
#include "RGBAImage.h"
#include "ObjectClassProperty.h"
#include "BooleanClassProperty.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* RGBATextureAttachment::RGBATextureAttachmentMetaClass = NULL;

const MetaClass* RGBATextureAttachment::getInstanceClass() const {
   return getClass();
}

Object* RGBATextureAttachment::newInstance() {
   return new RGBATextureAttachment;
}

const MetaClass* RGBATextureAttachment::getClass() {
   if (RGBATextureAttachmentMetaClass==NULL) {
      RGBATextureAttachmentMetaClass = new MetaClass("RGBATextureAttachment", "no description.",
                                  newInstance, sizeof(RGBATextureAttachment));
   }
   return RGBATextureAttachmentMetaClass;
}

void RGBATextureAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   RGBATextureAttachmentMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(RGBATextureAttachment)
	ADD_OBJECT_PROP(m_Image,0, RGBAImage::getClass(), true )
	ADD_OBJECT_PROP(m_Settings,0, GLTextureSettings::getClass(), true )
}

void RGBATextureAttachment::shutdown() {
   delete RGBATextureAttachmentMetaClass;
}


// ----


RGBATextureAttachment::RGBATextureAttachment()
{
   m_Image = new RGBAImage;
	m_Image->resize(1,1);
   m_Settings = new GLTextureSettings;
}

RGBATextureAttachment::~RGBATextureAttachment()
{
   delete m_Image;
   delete m_Settings;
}

GLTexture * RGBATextureAttachment::createGLTexture(GLContext * context)
{
   GLTexture2D * ret = new GLTexture2D( context );
   ret->createFromImage( *m_Image, true );
   ret->bind( *m_Settings, 0 );
   return ret;
}

void RGBATextureAttachment::assign(const Object* obj, X4CopyContext *context)
{
   TextureAttachment::assign( obj, context );
   RGBATextureAttachment * ta =  (RGBATextureAttachment*)obj;
   m_Settings->assign( ta->m_Settings );
   m_Image->assign( ta->m_Image );
}


void RGBATextureAttachment::createFromImage( RGBAImage * image ) 
{
	m_Image->assign( image );
	clearAttachments();
}





// ----

IMPLEMENT_X4_CLASS(RGBAFloat32TextureAttachment, 2)
{
	BEGIN_CLASS_INIT(RGBAFloat32TextureAttachment);
	ADD_OBJECT_PROP(m_Image,0, RGBAImage::getClass(), true );
	ADD_OBJECT_PROP(m_Settings,0, GLTextureSettings::getClass(), true );

	ADD_BOOLEAN_PROP(useHalfFloat, 1);
}


// ----


RGBAFloat32TextureAttachment::RGBAFloat32TextureAttachment()
{
	m_Image = new RGBAFloat32Image;
	m_Image->resize(1,1);
	m_Settings = new GLTextureSettings;
	useHalfFloat = false;
}

RGBAFloat32TextureAttachment::~RGBAFloat32TextureAttachment()
{
	delete m_Image;
	delete m_Settings;
}

GLTexture * RGBAFloat32TextureAttachment::createGLTexture(GLContext * context)
{
	GLTexture2D * ret = new GLTexture2D( context );
	ret->createFromImage( *m_Image, !useHalfFloat );
	ret->bind( *m_Settings, 0 );
	return ret;
}

void RGBAFloat32TextureAttachment::assign(const Object* obj, X4CopyContext *context)
{
	TextureAttachment::assign( obj, context );
	RGBAFloat32TextureAttachment * ta =  (RGBAFloat32TextureAttachment*)obj;
	m_Settings->assign( ta->m_Settings );
	m_Image->assign( ta->m_Image );
}


void RGBAFloat32TextureAttachment::createFromImage( RGBAFloat32Image * image ) 
{
	m_Image->assign( image );
	clearAttachments();
}
