//---------------------------------------------------------------------------
#ifndef TextureAttachmentH
#define TextureAttachmentH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "MaterialAttachment.h"
#include "GLTexture.h"
//---------------------------------------------------------------------------

namespace X4 {

class RGBAImage;
class RGBAFloat32Image;
class GLTextureSettings;


class X3RENDERING_API RGBATextureAttachment : public TextureAttachment
{
private:
   static MetaClass *RGBATextureAttachmentMetaClass;
   static Object* newInstance();

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   RGBATextureAttachment();
   virtual ~RGBATextureAttachment();

   virtual card32 getTextureType() const {return TYPE_2D; }
   virtual GLTexture * createGLTexture(GLContext * context);
//   virtual bool needsUpdate() {return false;}

   void createFromImage( RGBAImage * image );
	RGBAImage * getImage() {return m_Image;}
   GLTextureSettings * getSettings() {return m_Settings;}

   virtual void assign(const Object* obj, X4CopyContext *context);

private:

   RGBAImage * m_Image;
   GLTextureSettings * m_Settings;

// TODO!
//   card32 m_MagFilter;


};


class X3RENDERING_API RGBAFloat32TextureAttachment : public TextureAttachment
{
	X4_CLASS(RGBAFloat32TextureAttachment);

public:

	RGBAFloat32TextureAttachment();
	virtual ~RGBAFloat32TextureAttachment();

	virtual card32 getTextureType() const {return TYPE_2D; }
	virtual GLTexture * createGLTexture(GLContext * context);

	void createFromImage( RGBAFloat32Image * image );
	RGBAFloat32Image * getImage() {return m_Image;}
	GLTextureSettings * getSettings() {return m_Settings;}

	virtual void assign(const Object* obj, X4CopyContext *context);

	// this will use 16bit floats within the gl texture
	void setUseHalfFloat(bool use) { useHalfFloat = use; }
	bool getUseHalfFloat() const { return useHalfFloat; }

private:

	RGBAFloat32Image * m_Image;
	GLTextureSettings * m_Settings;
	bool useHalfFloat;

};


}
#endif
 