//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "RenderingRoutines.h"
//---------------------------------------------------------------------------
#include "GLRenderTextureFBO.h"
#include "Renderer.h"
#include "RGBAImage.h"
#include "Scene.h"
#include "SceneGraphState.h"
#ifndef _DEBUG
#include "LinearAlgebra.inline.h"
#endif
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



RGBAImage *X4::createImage(GLRenderTextureFBO *rt, Scene *scene, Renderer *renderer, bool disconnectRenderer) {
   RGBAImage *result = NULL;
   try {
      const int32 width = rt->getWidth();
      const int32 height = rt->getHeight();
      rt->activate();

      Vector4f c = scene->getRootState()->staticState->backgroundColor;
      glClearColor(c[0], c[1], c[2], c[3]);
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

      glViewport(0,0, width, height);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(scene->getRootState()->staticState->viewFrustum.getVerticalFieldOfView(),
                     (float)width / (float)height,
                     scene->getRootState()->staticState->viewFrustum.getZNearClip(),
                     scene->getRootState()->staticState->viewFrustum.getZFarClip());
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
      gluLookAt(scene->getRootState()->staticState->camera.getPosition()[0],
                scene->getRootState()->staticState->camera.getPosition()[1],
                scene->getRootState()->staticState->camera.getPosition()[2],

                scene->getRootState()->staticState->camera.getLookAt()[0],
                scene->getRootState()->staticState->camera.getLookAt()[1],
                scene->getRootState()->staticState->camera.getLookAt()[2],

                scene->getRootState()->staticState->camera.getUp()[0],
                scene->getRootState()->staticState->camera.getUp()[1],
                scene->getRootState()->staticState->camera.getUp()[2]);

      renderer->connect(scene, rt->getContext());
      renderer->render(width, height);
      if (disconnectRenderer) renderer->disconnect(rt->getContext());
		result = rt->getRGBAImage();
   } catch(...) {
      rt->deactivate();
		throw;
   }
   rt->deactivate();
   return result;
}


RGBAImage *X4::createImage(GLContext *mainContext, Scene *scene, Renderer *renderer,
                                 card32 width, card32 height)
{
   if (height == 0) return NULL;
   mainContext->activate();
   GLRenderTextureFBO rt(NULL);
	if (!rt.create(width, height, 4, GLRenderTextureFBO::FLAG_DEPTH | GLRenderTextureFBO::FLAG_DEPTH | GLRenderTextureFBO::FLAG_STENCIL)) {
      return NULL;
   }
   mainContext->deactivate();

   return createImage(&rt, scene, renderer);
}
