//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "SceneCreator.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "Tools.h"
#include "StandardMaterials.h"
#include "GLShaderMaterial.h"
#include "GLMaterial.h"
#include "SimplePCMaterial.h"
#include "SimpleGLTriMeshMaterial.h"
#include "PointsAsSpheresMaterial.h"
#include "Configuration.h"
#include "DebugOutput.h"
#include "BinaryObjectStreams.h"
#include "ApplicationDirectories.h"
#include "CurvatureDirectionsMaterial.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* SceneCreator::SceneCreatorMetaClass = NULL;

const MetaClass* SceneCreator::getInstanceClass() const {
   return getClass();
}

Object* SceneCreator::newInstance() {
   return new SceneCreator;
}

const MetaClass* SceneCreator::getClass() {
   if (SceneCreatorMetaClass==NULL) {
      SceneCreatorMetaClass = new MetaClass("SceneCreator", "no description.",
                                  newInstance, sizeof(SceneCreator));
   }
   return SceneCreatorMetaClass;
}

void SceneCreator::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   SceneCreatorMetaClass->copyParentProperties();
//   SceneCreatorMetaClass->getGUIDescriptor()->addClassEditor(SceneCreatorEditor::getClass(), true);
}

void SceneCreator::shutdown() {
   delete SceneCreatorMetaClass;
}


// ----

SceneCreator::SceneCreator() {
	createStdMaterials = true;
}

static void tryAddMaterial(Scene *scene, std::string path) {
	try {
		std::string materialPath = ApplicationDirectories::getResourcesDir() + path;
		debugOutput << std::string("trying to load ")+materialPath+std::string(".\n");
		BinaryInputObjectStream in(materialPath.c_str());
		Persistent *p;
		GLMaterial *mat = NULL;
		try {
			in.readObject(p);
			mat = dynamic_cast<GLMaterial*>(p);
		} catch (...) {
			debugOutput << "could not load this material - io error.\n";
			return;
		}
		if (!mat) {
			delete p;
			debugOutput << "wrong type - not loaded\n";
		} else {
			scene->addGLMaterial(mat);
			debugOutput << "material loaded successfully.\n";
		}	
	} catch (...) {
		warning("could not add default material.");
	}
}

Object *SceneCreator::makeObject() {
   Scene *scene = new Scene();
   scene->addGLMaterial(new SimplePCMaterial());
   if (createStdMaterials) {
      scene->addGLMaterial(makeWhitePointsMaterial());
      scene->addGLMaterial(makeColoredPointsMaterial());
      scene->addGLMaterial(new SimpleGLTriMeshMaterial);
      scene->addGLMaterial(new PointsAsSpheresMaterial);
		tryAddMaterial(scene, "shaders/BlinnPhong.material.x4obj");
		tryAddMaterial(scene, "shaders/Blinn-Phong Color Two-Sided.material.x4obj");
		tryAddMaterial(scene, "shaders/CookTorranceDarkBlue.material.x4obj");
		tryAddMaterial(scene, "shaders/GaussianVolumeSplatter.material.x4obj");
		//tryAddMaterial(scene, "shaders/Blinn-Phong Color Two-Sided.material.x4obj");
		//tryAddMaterial(scene, "shaders/Blinn-Phong Alpha.material.x4obj");
		//tryAddMaterial(scene, "shaders/Blinn-Phong Color Alpha.material.x4obj");
		//tryAddMaterial(scene, "shaders/Blinn-Phong Color.material.x4obj");
      scene->addGLMaterial(new CurvatureDirectionsMaterial);
      
      //scene->addGLMaterial(makeNormalPointsMaterial());
      //scene->addGLMaterial(makeColorNormalMaterial());
   }
   return scene;
}


