//---------------------------------------------------------------------------
#ifndef ShaderH
#define ShaderH
//---------------------------------------------------------------------------
#include "x3rendering.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4 {

class GLShader;
class GLContext;


class X3RENDERING_API Shader : public Persistent
{
 private:
   static MetaClass *ShaderMetaClass;
   static Object* newInstance();

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   Shader();
   virtual void assign(const Object* obj, X4CopyContext *context=NULL );

   // creates a gl shader from this source
	GLShader * createShader( GLContext *context );
	// creates an empty gl shader
	GLShader * createShaderInstance( GLContext * context );
   bool compile( GLContext *testcontext);
   std::string getError() {return m_LastError;}

   bool isValid();

   static const card32 INVALID = 0xffff;
   static const card32 VERTEX = 1;
   static const card32 FRAGMENT = 2;
   static const card32 CLASS_ARB = 1;
   static const card32 CLASS_CG = 2;

   card32 getShaderType() const {return m_ShaderType;}
   void setShaderType(card32 stype );
   card32 getShaderClass() const {return m_ShaderClass;}
   void setShaderClass(card32 stype );

   void setShaderProfile(const std::string& profile) { m_ShaderProfile = profile; }
   const std::string& getShaderProfile() const { return m_ShaderProfile; }

   std::string getSource() const;
   void setSource(std::string);

   bool wasChanged() const {return m_Changed;}
   void setChanged() {m_Changed = true; }

	// load shader from text file
	bool loadFromFile( const std::string & filename);

 protected:

   bool     m_Changed;

   // persistent members:
   std::string m_FileName;
   std::string   m_ShaderSource;
   card32   m_ShaderType;
   card32   m_ShaderClass;

    // nonpersistent members:
    std::string  m_LastError;
	std::string  m_ShaderProfile;

 //   GLContext * m_TestContext;
};

};
#endif
