//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "SimpleGLTriMeshMaterial.h"
//---------------------------------------------------------------------------
#include "UnstructuredInCoreTriangleMesh.h"
#include "BooleanClassProperty.h"
#include "NumericalClassProperty.h"

#include "LinearAlgebra.inline.h"
#include "PointSet.inline.h"
#include "VertexArray.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
#include "SceneGraphState.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* SimpleGLTriMeshMaterial::SimpleGLTriMeshMaterialMetaClass = NULL;

//const MetaClass* SimpleGLTriMeshMaterial::getInstanceClass() const {
//	return getClass();
//}
//
//Object* SimpleGLTriMeshMaterial::newInstance() {
//   return new SimpleGLTriMeshMaterial;
//}
//
//const MetaClass* SimpleGLTriMeshMaterial::getClass() {
//   if (SimpleGLTriMeshMaterialMetaClass==NULL) {
//      SimpleGLTriMeshMaterialMetaClass = new MetaClass("SimpleGLTriMeshMaterial", "no description.",
//                                  newInstance, sizeof(SimpleGLTriMeshMaterial));
//   }
//   return SimpleGLTriMeshMaterialMetaClass;
//}

IMPLEMENT_X4_CLASS(SimpleGLTriMeshMaterial, 10)
{
	BEGIN_CLASS_INIT(SimpleGLTriMeshMaterial);
	ADD_BOOLEAN_PROP(wireframe, 0)
		ADD_BOOLEAN_PROP(showNormals, 0)
		ADD_FLOAT32_PROP(normalLength, 0)
		ADD_BOOLEAN_PROP(colorizeNormal, 0)
		ADD_BOOLEAN_PROP(useVertexNormals, 0)
		ADD_BOOLEAN_PROP(showWireFrameOnly, 0)
		ADD_BOOLEAN_PROP(showWireFrameOnly_depthTest, 0)
		ADD_BOOLEAN_PROP(blackWireFrame, 0)
		ADD_BOOLEAN_PROP(showBorder, 7)
		ADD_BOOLEAN_PROP(cullBackFace, 8)
		ADD_BOOLEAN_PROP(intensityPerFace, 9)
		ADD_FLOAT32_PROP(intensityThreshold, 9)
}
//
//void SimpleGLTriMeshMaterial::init(const MetaClass *parentClass) {
//   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 9);
//   SimpleGLTriMeshMaterialMetaClass->copyParentProperties();
//   BEGIN_CLASS_INIT(SimpleGLTriMeshMaterial)
//	ADD_BOOLEAN_PROP(wireframe ,0)
//	ADD_BOOLEAN_PROP(showNormals ,0)
//	ADD_FLOAT32_PROP(normalLength ,0)
//	ADD_BOOLEAN_PROP(colorizeNormal ,0)
//	ADD_BOOLEAN_PROP(useVertexNormals ,0)
//	ADD_BOOLEAN_PROP(showWireFrameOnly ,0)
//	ADD_BOOLEAN_PROP(showWireFrameOnly_depthTest ,0)
//	ADD_BOOLEAN_PROP(blackWireFrame ,0)
//	ADD_BOOLEAN_PROP(showBorder,7)
//	ADD_BOOLEAN_PROP(cullBackFace,8)               
//   ADD_BOOLEAN_PROP(intensityPerFace, 9)
//   ADD_FLOAT32_PROP(intensityThreshold, 9)
//}

//void SimpleGLTriMeshMaterial::shutdown() {
//   delete SimpleGLTriMeshMaterialMetaClass;
//}


// ----


SimpleGLTriMeshMaterial::SimpleGLTriMeshMaterial() {
   wireframe = false;
   showNormals = false;
   normalLength = 0.1f;
   colorizeNormal = true;
   useVertexNormals = true;
   showWireFrameOnly = false;
   showWireFrameOnly_depthTest = false;
   blackWireFrame = true;
	showBorder = false;
	cullBackFace = false;
	setName("SimpleGLTriMeshMaterial");
	intensityPerFace = false;
	intensityThreshold = 0.9f;
}


void SimpleGLTriMeshMaterial::draw( Scene * scene, SceneGraphState *state, SceneObject * cloud,
      SceneObjectIterator *iter, GLContext * context, MemoryManager * mm,
      const int32 includeFlags, const int32 excludeFlags)
{
   forget(includeFlags);
   forget(excludeFlags);
   forget(mm);
   forget(context);
   forget(iter);
   forget(scene);

   UnstructuredInCoreTriangleMesh *dest = dynamic_cast<UnstructuredInCoreTriangleMesh*>(cloud);

   if ( dest == NULL ){
	   GenericSwapAllOutOfCorePointCloud* gen = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>( cloud );
	   if ( gen != NULL ){
		   dest = dynamic_cast<UnstructuredInCoreTriangleMesh*>( gen->fetchPC() );
	   }
   }

   Matrix3f T3f = shrink4To3(state->dynamicState->ctm);
   if (dest != NULL) {
      PointSet *pts = dest->getPointSet();

      if (!pts->providesAttribute("position")) return;
      AAT POS = pts->getAAT("position");

      AAT COL;
      bool hasCol = pts->providesAttribute("color");
      if (hasCol) COL = pts->getAAT("color");

      AAT NRM;
      bool hasNormal = pts->providesAttribute("normal");
      if (hasNormal) NRM = pts->getAAT("normal");

      PointSet *idx = dest->getTriangles();
      if (!idx->providesAttribute("index")) return;
      AAT IDX = idx->getAAT("index");
      
      AAT INTENSITY;
      if (idx->providesAttribute("intensity", 1, VAD::DATA_FORMAT_FLOAT32)) {
         INTENSITY = idx->getAAT("intensity");
      }

      glPushAttrib(GL_ENABLE_BIT);

      if( showWireFrameOnly && showWireFrameOnly_depthTest )
      {
         glEnable(GL_DEPTH_TEST);
			if( cullBackFace )
		      glEnable(GL_CULL_FACE);
			else
			   glDisable(GL_CULL_FACE);
         glEnable( GL_POLYGON_OFFSET_FILL );
         glPolygonOffset( 1.2f, 4.0f );

         const card32 numTri = idx->getNumEntries();
         glBegin(GL_TRIANGLES);
         glColor3f(1,1,1);
         for (card32 i=0; i<numTri; i++) {
            Vector3i tind = idx->get<int32, 3>(i, IDX);
            Vector3f pos[3];
            pos[0] = pts->get<float32, 3>(tind[0], POS);
            pos[1] = pts->get<float32, 3>(tind[1], POS);
            pos[2] = pts->get<float32, 3>(tind[2], POS);


            glVertex3fv(pos[0].data());
            glVertex3fv(pos[1].data());
            glVertex3fv(pos[2].data());
         }
         glEnd();
      }
      else
      if( !showWireFrameOnly )
      if( colorizeNormal )
      {
         glEnable(GL_DEPTH_TEST);
			if( cullBackFace )
		      glEnable(GL_CULL_FACE);
			else
			   glDisable(GL_CULL_FACE);

         const card32 numTri = idx->getNumEntries();
         glBegin(GL_TRIANGLES);
         for (card32 i=0; i<numTri; i++) {
            Vector3i tind = idx->get<int32, 3>(i, IDX);
            Vector3f pos[3];
            Vector3f color[3];
            Vector3f normal[3];
            pos[0] = pts->get<float32, 3>(tind[0], POS);
            pos[1] = pts->get<float32, 3>(tind[1], POS);
            pos[2] = pts->get<float32, 3>(tind[2], POS);

            if (!hasNormal || !useVertexNormals) {
               Vector3f tnormal = (pos[2]-pos[0]).crossProduct(pos[1]-pos[0]);
               float32 n = norm(tnormal);
               if (n > 1e-7)
                  tnormal /= n;
               else
                  continue;
			   tnormal = T3f * tnormal;
               normal[0] = tnormal;
               normal[1] = tnormal;
               normal[2] = tnormal;
            }

            if (hasNormal && useVertexNormals) {
               normal[0] = T3f * pts->get<float32, 3>(tind[0], NRM);
               normal[1] = T3f * pts->get<float32, 3>(tind[1], NRM);
               normal[2] = T3f * pts->get<float32, 3>(tind[2], NRM);
            }

            for( card32 j=0; j<3; j++ ) {
               color[j] = (normal[j] +  makeVector3f(1,1,1))* 0.5f;
            }
            
            if (intensityPerFace) {
               if (INTENSITY != NULL_AAT) {
                  float32 intensity = idx->get1f(i, INTENSITY);
                  if (intensity < intensityThreshold) continue;
                  for (card32 i2=0; i2<3; i2++) {
                     color[i2][0] = intensity;
                     color[i2][1] = intensity;
                     color[i2][2] = intensity;
                  }
               }

            }

            glColor3fv(color[0].data());
            glVertex3fv(pos[0].data());
            glColor3fv(color[1].data());
            glVertex3fv(pos[1].data());
            glColor3fv(color[2].data());
            glVertex3fv(pos[2].data());
         }
         glEnd();
      }
      else {

         glEnable(GL_DEPTH_TEST);
         glEnable(GL_LIGHTING);
         glEnable(GL_NORMALIZE);
         glEnable(GL_COLOR_MATERIAL);
         glDisable(GL_CULL_FACE);
         glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
         glEnable(GL_LIGHT0);
         glColor4f(1,1,1,1);
         float lightPos[] = {100.0f,1000.0f,100.0f,1.0f};
         glLightfv( GL_LIGHT0, GL_POSITION, lightPos );

         const card32 numTri = idx->getNumEntries();
         glBegin(GL_TRIANGLES);
         for (card32 i=0; i<numTri; i++) {
            Vector3i tind = idx->get<int32, 3>(i, IDX);
            Vector3f pos[3];
            Vector3f color[3];
            Vector3f normal[3];

            pos[0] = pts->get<float32, 3>(tind[0], POS);
            pos[1] = pts->get<float32, 3>(tind[1], POS);
            pos[2] = pts->get<float32, 3>(tind[2], POS);

            if (!hasNormal || !useVertexNormals) {
               Vector3f tnormal = (pos[2]-pos[0]).crossProduct(pos[1]-pos[0]);
               float32 n = norm(tnormal);
               if (n > 1e-20) tnormal /= n;
               normal[0] = tnormal;
               normal[1] = tnormal;
               normal[2] = tnormal;
            }

            if (hasCol) {
               color[0] = pts->get<float32, 3>(tind[0], COL);
               color[1] = pts->get<float32, 3>(tind[1], COL);
               color[2] = pts->get<float32, 3>(tind[2], COL);
            }
            if (hasNormal && useVertexNormals) {
               normal[0] = pts->get<float32, 3>(tind[0], NRM);
               normal[1] = pts->get<float32, 3>(tind[1], NRM);
               normal[2] = pts->get<float32, 3>(tind[2], NRM);
            }

            if (hasCol) glColor3fv(color[0].data());
            glNormal3fv(normal[0].data());
            glVertex3fv(pos[0].data());
            if (hasCol) glColor3fv(color[1].data());
            glNormal3fv(normal[1].data());
            glVertex3fv(pos[1].data());
            if (hasCol) glColor3fv(color[2].data());
            glNormal3fv(normal[2].data());
            glVertex3fv(pos[2].data());
         }
         glEnd();
      }
      glPopAttrib();

      glPushAttrib(GL_ENABLE_BIT);

       if (wireframe || showWireFrameOnly) {
         if( (showWireFrameOnly_depthTest && showWireFrameOnly) || intensityPerFace)
            glEnable(GL_DEPTH_TEST);
         else
            glDisable(GL_DEPTH_TEST);
         glDisable(GL_LIGHTING);
         glDisable(GL_COLOR_MATERIAL);
         if( !blackWireFrame )
            glColor4f(1,1,1,0.2f);
         else
            glColor4f(0,0,0,0.2f);
         //glEnable(GL_BLEND);
         glBlendFunc(GL_SRC_ALPHA, GL_ONE);
         glDepthMask(false);
         glEnable(GL_LINE_SMOOTH);
         glDisable( GL_CULL_FACE );

         const card32 numTri = idx->getNumEntries();
         glBegin(GL_LINES);
         for (card32 i=0; i<numTri; i++) {
            Vector3i tind = idx->get<int32, 3>(i, IDX);
            Vector3f pos[3];
            pos[0] = pts->get<float32, 3>(tind[0], POS);
            pos[1] = pts->get<float32, 3>(tind[1], POS);
            pos[2] = pts->get<float32, 3>(tind[2], POS);

            glVertex3fv(pos[0].data());
            glVertex3fv(pos[1].data());

            glVertex3fv(pos[1].data());
            glVertex3fv(pos[2].data());

            glVertex3fv(pos[2].data());
            glVertex3fv(pos[0].data());
         }
         glEnd();

         glDepthMask(true);
      }

      glPopAttrib();

      glPushAttrib(GL_ENABLE_BIT);
      if( showNormals && hasNormal)
      {
         glBegin( GL_LINES );
         glColor3f( 1,0,0);
         for( card32 i=0;i<pts->getNumEntries();i++ )
         {
            Vector3f pos = pts->get<float32,3>(i, POS );
            Vector3f normal = pts->get<float32,3>(i, NRM );
            Vector3f p2 = pos + normal * normalLength;
            glVertex3fv( &pos[0] );
            glVertex3fv( &p2[0] );
         }
         glEnd();
      }

      glPopAttrib();
   }

	if( dest && showBorder )
	{
		std::map<int64,int64> edgeMap;
		VertexArray tri(dest->getTriangles() );
		VertexArray vert( dest->getPointSet() );

		for( card32 i=0;i<tri.getNumElements();i++ )
		{
			for( card32 e=0;e<3;e++ )
			{
				int64 i1 = tri.getIndex3i(i)[e];
				int64 i2 = tri.getIndex3i(i)[(e+1)%3];
				int64 index = min(i1,i2) * dest->getNumPoints() + max(i1,i2);
				if( edgeMap.find(index) == edgeMap.end() )
					edgeMap[index] = index;
				else
					edgeMap.erase( edgeMap.find(index) );
			}
		}

		glPushAttrib( GL_ALL_ATTRIB_BITS );
		glDisable( GL_DEPTH_TEST );

		glBegin( GL_LINES );
		glColor3f(1,0,0);

		for( std::map<int64,int64>::iterator iter = edgeMap.begin(); iter != edgeMap.end(); ++iter )
		{
			int64 i1 = iter->second / (int64)dest->getNumPoints();
			int64 i2 = iter->second % (int64)dest->getNumPoints();

			Vector3f v1 = vert.getPosition3f( i1 );
			Vector3f v2 = vert.getPosition3f( i2 );
			glVertex3fv( &v1[0] );
			glVertex3fv( &v2[0] );
		}
		glEnd();

		glPopAttrib();
	}
}



