//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SimpleLinesMaterial.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointCloudIterators.h"
#include "PointSet.h"
#include "SeparatorClassProperty.h"
#include "BooleanClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "RenderingTool.h"
#include "PointSet.inline.h"
#include "SeparatorClassProperty.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS( SimpleLinesMaterial , 0 )
{
	BEGIN_CLASS_INIT( SimpleLinesMaterial );
   ADD_SEPARATOR("lines")
   ADD_FLOAT32_PROP(lineWidth, 0)
   ADD_VECTOR4F_PROP(color, 0)
   ADD_BOOLEAN_PROP(blending, 0)
   ADD_SEPARATOR("stippeling")
   ADD_BOOLEAN_PROP(stipple, 0)
   ADD_FLOAT32_PROP(stipplePointSize, 0)
   ADD_VECTOR4F_PROP(stippleColor, 0)
}

SimpleLinesMaterial::SimpleLinesMaterial()
{
   color = makeVector4f(0.5f,0.1f,0.025f,0.25f);
   lineWidth = 3;
   blending = true;
   stipple = true;
   stipplePointSize = 3;
   stippleColor = makeVector4f(1,1,1,1);   
}

SimpleLinesMaterial::~SimpleLinesMaterial()
{
}

void SimpleLinesMaterial::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj,this);
}

void SimpleLinesMaterial::draw(
	Scene *scene, SceneGraphState *state, SceneObject *obj,
   SceneObjectIterator *iter, GLContext *context, MemoryManager * mm,
   const int32 includeFlags, const int32 excludeFlags)
{
	BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(iter);
	if (bit) {
		const PointSet *ps = bit->lockBlockRead();
	   glPushAttrib(GL_ALL_ATTRIB_BITS);
	   glLineWidth(lineWidth);
	   if (blending) {
	      glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	      glEnable(GL_BLEND);
	      glDepthMask(0);
	      glEnable(GL_LINE_SMOOTH);
	   }
	   glColor4fv(color.data());
		if (ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
         AAT positionAAT = ps->getAAT("position");
         for (card32 y=0; y<ps->getHeight(); y++) {
            glBegin(GL_LINE_STRIP);
            for (card32 x=0; x<ps->getWidth(); x++) {
               glVertex3fv(ps->get3f(x,y,positionAAT).data());
            }
            glEnd();
         }
      }
      glPopAttrib();
      if (stipple) {
	      glPushAttrib(GL_ALL_ATTRIB_BITS);
	      glPointSize(stipplePointSize);
	      glColor4fv(stippleColor.data());
		   if (ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
            AAT positionAAT = ps->getAAT("position");
            glBegin(GL_POINTS);
            for (card32 y=0; y<ps->getHeight(); y++) {
               for (card32 x=0; x<ps->getWidth(); x++) {
                  glVertex3fv(ps->get3f(x,y,positionAAT).data());
               }
            }
            glEnd();
         }
         glPopAttrib();
      }
      bit->unlock();
   }   
}
