//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "SimplePCMaterial.h"
//---------------------------------------------------------------------------
#include "DebugOutput.h"
#include "FixedArrayClassProperty.h"
#include "StringClassProperty.h"
#include "BooleanClassProperty.h"
#include "NumericalClassProperty.h"

#include "X4Assert.h"
#include "PointCloudIterators.h"
#include "PointCloud.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "GLBuffer.h"
#include "GLContext.h"
#include "GLShader.h"
//#include "GLShaderMaterialEditor.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra.inline.h"
#include "VertexDescriptor.h"
#include "Shader.h"
#include "MPContainer.h"
#include "GLAttachments.h"
#include "MaterialParameter.h"
#include "StandardMaterials.h"
#include "GLShaderMaterial.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* SimplePCMaterial::SimplePCMaterialMetaClass = NULL;

const MetaClass* SimplePCMaterial::getInstanceClass() const {
   return getClass();
}

Object* SimplePCMaterial::newInstance() {
   return new SimplePCMaterial;
}

const MetaClass* SimplePCMaterial::getClass() {
   if (SimplePCMaterialMetaClass==NULL) {
      SimplePCMaterialMetaClass = new MetaClass("SimplePCMaterial", "no description.",
                                  newInstance, sizeof(SimplePCMaterial));
   }
   return SimplePCMaterialMetaClass;
}

void SimplePCMaterial::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 3);
   SimplePCMaterialMetaClass->copyParentProperties();
//IMPLEMENT_X4_CLASS( SimplePCMaterial, 1 )
   BEGIN_CLASS_INIT(SimplePCMaterial)
	ADD_BOOLEAN_PROP(immediateMode ,0)
	ADD_BOOLEAN_PROP(showColors ,0)
	ADD_BOOLEAN_PROP(showNormals ,0)
	ADD_BOOLEAN_PROP(drawNormals ,0)
	ADD_BOOLEAN_PROP( drawTangents, 1 )
	ADD_FLOAT32_PROP(normalLength ,0)
	ADD_CARD32_PROP(pointSize ,0)
	ADD_BOOLEAN_PROP(drawQuads ,0)
	ADD_BOOLEAN_PROP(drawSphericalPoints, 2);
	ADD_VECTOR3F_PROP(defaultColor,3);
}

void SimplePCMaterial::shutdown() {
   delete SimplePCMaterialMetaClass;
}


// ----


MetaClass* GLImModePtsRenderObject::GLImModePtsRenderObjectMetaClass = NULL;

const MetaClass* GLImModePtsRenderObject::getInstanceClass() const {
   return getClass();
}

Object* GLImModePtsRenderObject::newInstance() {
 //  return new GLImModePtsRenderObject;
 return NULL;
}

const MetaClass* GLImModePtsRenderObject::getClass() {
   if (GLImModePtsRenderObjectMetaClass==NULL) {
      GLImModePtsRenderObjectMetaClass = new MetaClass("GLImModePtsRenderObject", "no description.",
                                  NULL, sizeof(GLImModePtsRenderObject));
   }
   return GLImModePtsRenderObjectMetaClass;
}

void GLImModePtsRenderObject::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   GLImModePtsRenderObjectMetaClass->copyParentProperties();
}

void GLImModePtsRenderObject::shutdown() {
   delete GLImModePtsRenderObjectMetaClass;
}


// ----


GLImModePtsRenderObject::GLImModePtsRenderObject() {
   ps = NULL;
}

GLImModePtsRenderObject::GLImModePtsRenderObject(PointSet *ps) {
   this->ps = (PointSet*)ps->copy();
}

void GLImModePtsRenderObject::setPointSet(PointSet *ps) {
   this->ps = ps;
}

GLImModePtsRenderObject::~GLImModePtsRenderObject() {
   delete ps;
}


// --


SimplePCMaterial::SimplePCMaterial() {
   immediateMode = false; // currently: no effect
   showColors = true;
   showNormals = false; //TEST
   pointSize = 1;
   drawQuads = false;			
   drawNormals = false;
   drawTangents = false;
   normalLength = 0.2f;
   drawSphericalPoints=true;
   setName("SimplePCMaterial");
	defaultColor = makeVector3f(1,1,1);
}

void SimplePCMaterial::draw( Scene * scene, SceneGraphState *state, SceneObject * cloud,
      SceneObjectIterator *iter, GLContext * context, MemoryManager * mm,
      const int32 includeFlags, const int32 excludeFlags )
{
   BasicPointCloudIterator * bIter = dynamic_cast<BasicPointCloudIterator *>( iter );
   if (bIter == NULL) return;

   const PointSet * ps = bIter->lockBlockRead();

   if (ps == NULL) {
      bIter->unlock();
      return;
   }

   glPushAttrib(GL_ENABLE_BIT);
   glEnable(GL_DEPTH_TEST);
   if (showNormals||drawTangents) {

      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);

		float ambient0[] = { 0.1f, 0.1f, 0.1f, 1.0f };
		float diffuse0[] = { 0.8f, 0.8f, 0.8f, 1.0f };
		glLightfv( GL_LIGHT0, GL_AMBIENT, ambient0 );
		glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuse0 );

      glEnable(GL_COLOR_MATERIAL);
      glEnable(GL_NORMALIZE);
      glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
   }
   else
   {
      glDisable(GL_LIGHTING);
      glDisable(GL_COLOR_MATERIAL);
   }
   glPointSize(pointSize);

   if(drawSphericalPoints)
	   glEnable (GL_POINT_SMOOTH);

   intern_drawImmediateMode( context, ps, includeFlags, excludeFlags );

   if(drawNormals) {
      AAT POSITION = ps->getAAT("position");
      AAT NORMAL;

      glPushAttrib(GL_ALL_ATTRIB_BITS);
      glDisable(GL_LIGHTING);
      glColor4f(0.9f,1,0.8f,1);
      glEnable(GL_DEPTH_TEST);
      if (ps->providesAttribute("normal")) {
         if ((*(ps->getDescr()))["normal"].dataFormat == VAD::DATA_FORMAT_FLOAT32
          && (*(ps->getDescr()))["normal"].numComponents == 3) {
            NORMAL = ps->getAAT("normal");

            glBegin(GL_LINES);
            const card32 n = ps->getNumEntries();
            for (card32 i=0; i<n; i++){
               Vector3f pos = ps->get<float32, 3>(i, POSITION);
               Vector3f normal = ps->get<float32, 3>(i, NORMAL);

               glVertex3fv(pos.data());
               glVertex3fv((pos + normal*normalLength).data());
            }
			glEnd();
          }
	  }
	  glPopAttrib();
   }
   
   if(drawTangents) {
	   AAT POSITION = ps->getAAT("position");
	   AAT TANGENT_U;
	   AAT TANGENT_V;
	   glPushAttrib(GL_ALL_ATTRIB_BITS);
	   glDisable(GL_LIGHTING);
	   glColor4f(0.2f,0.3f,0.8f,1);
	   glEnable(GL_DEPTH_TEST);
	   if ( ps->providesAttribute("tangent_u") && ps->providesAttribute("tangent_v") ) {
		  if ( ( (*(ps->getDescr()))["tangent_u"].dataFormat == VAD::DATA_FORMAT_FLOAT32 ) &&
			   ( (*(ps->getDescr()))["tangent_u"].numComponents == 3 ) &&
			   ( (*(ps->getDescr()))["tangent_v"].dataFormat == VAD::DATA_FORMAT_FLOAT32 ) &&
			   ( (*(ps->getDescr()))["tangent_v"].numComponents == 3 ) )
		  {
			  TANGENT_U = ps->getAAT("tangent_u");
			  TANGENT_V = ps->getAAT("tangent_v");

			  glBegin(GL_LINES);
			  const card32 n = ps->getNumEntries();
			  for (card32 i=0; i<n; i++){
				  Vector3f pos = ps->get<float32, 3>(i, POSITION);
				  Vector3f tangent_u = ps->get<float32, 3>(i, TANGENT_U);
				  Vector3f tangent_v = ps->get<float32, 3>(i, TANGENT_V);
				  glVertex3fv(pos.data());
				  glVertex3fv((pos + tangent_u*normalLength).data());
				  glVertex3fv(pos.data());
				  glVertex3fv((pos + tangent_v*normalLength).data());
			  }
			  glEnd();
		  }
	  }
	  
	  glPopAttrib();
   }

   if(drawNormals){

      glBegin(GL_TRIANGLES);
      //impts->drawNormals(glctxt, ps );
      intern_drawNormals( context, ps, includeFlags, excludeFlags );
      glEnd();

   }
   // jsux: done
   glPopAttrib();

   bIter->unlock();

   forget(scene);
   forget(cloud);
   forget(mm);
}
/*
//   }
//   else
//   {

      glPushAttrib(GL_ALL_ATTRIB_BITS);
      glEnable(GL_DEPTH_TEST);
      glDepthMask(true);
      try {
         whiteMaterial->draw( scene, cloud, iter, context, mm, includeFlags, excludeFlags );
      } catch (...) {
         immediateMode = true;
      }
      glPopAttrib();

      -----

      glPushAttrib(GL_ENABLE_BIT);
      glEnable(GL_DEPTH_TEST);

      SecondaryDataStructureIterator *dataIter = dynamic_cast<SecondaryDataStructureIterator *>
         (iter);
      x4Assert(dataIter);

      VertexDescriptor simpleVD;
      simpleVD.pushAttrib( VAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
      const VertexDescriptor * vPS = cloud->getDescr();
      if( vPS->checkName( "intensity" ) )
         simpleVD.pushAttrib( VAD( "intensity", 3 , VAD::DATA_FORMAT_FLOAT32 ) );
      else
         if( vPS->checkName( "color" ) )
      simpleVD.pushAttrib( VAD( "color", 3 , VAD::DATA_FORMAT_FLOAT32 ) );

      std::string dataName = SharedVertexBufferAttachment::generateID( &simpleVD, context->getRootContext(), "", includeFlags, excludeFlags );

      // try to find a existing compatible vertexbuffer
      AttachedData * data = dataIter->fetchData( dataName );
      SharedVertexBufferAttachment * vbuffer = NULL;
      if( data )
      {
         vbuffer = dynamic_cast<SharedVertexBufferAttachment*>(data);
      }

      if( vbuffer )
         vbuffer->touched();  // <- informs memorymanaged


      // generate shared vertex buffer & attach
      if( vbuffer == NULL )
      {
         vbuffer = new SharedVertexBufferAttachment;

         std::vector<card8> renderData;
         simpleVD.convertVertices( ps, renderData, includeFlags, excludeFlags );

         if( simpleVD.checkName( "intensity" ) )
         {
            int offset = simpleVD.getAttributeOffset( simpleVD.getIndex("intensity") );
            float maxInt = 0.0f;
            for( card32 i=0;i<renderData.size()/simpleVD.getSize();i++ )
            {
               float * ptr = (float *)&renderData[i*simpleVD.getSize() + offset];
               maxInt = maxInt < ptr[0] ? ptr[0] : maxInt;
            }
            for( card32 i=0;i<renderData.size()/simpleVD.getSize();i++ )
            {
               float * ptr = (float *)&renderData[i*simpleVD.getSize() + offset];
               ptr[0] *= 1.0f / maxInt*2.0f;
               ptr[1] = ptr[0];
               ptr[2] = ptr[0];
            }
         }

         // generate gl vertex buffer
         GLVertexBuffer * glVertexBuffer = new GLVertexBuffer( context, simpleVD );
         card32 numElements =  renderData.size() / simpleVD.getSize();
         glVertexBuffer->init( numElements );
         glVertexBuffer->writeToBuffer( &renderData[0], 0, numElements );
         glVertexBuffer->setMappingFlag(true);   // <- use std gl array-bindings ( glVertexPointer, glNormalPointer , ... )

         vbuffer->setupVB( glVertexBuffer, "" , mm, true, includeFlags, excludeFlags );
         dataIter->attachData( vbuffer );
      }

      if( vbuffer )
      {
         GLVertexBuffer * glbuffer = vbuffer->getVertexBuffer();
         glbuffer->bind();
         glbuffer->drawArrays(GL_POINTS);
         glbuffer->unbind();
      }

      glPopAttrib();

   }
*/

void SimplePCMaterial::intern_drawImmediateMode( GLContext * ctxt, const PointSet * ps,
                                                 const int32 includeFlags, const int32 excludeFlags )
{
   AAT POSITION;
   AAT COLOR;
   AAT NORMAL;
   AAT FLAGS;
   bool bShowNormals = showNormals;
   bool bShowColors = showColors;

   bool flags = ps->providesAttribute("flags");
   if (flags) {
      if ((*(ps->getDescr()))["flags"].dataFormat != VAD::DATA_FORMAT_INT32
       || (*(ps->getDescr()))["flags"].numComponents != 1) {
         flags = false;
      } else {
         FLAGS = ps->getAAT("flags");
      }
   }

   if (!ps->providesAttribute("position")) return;
   POSITION = ps->getAAT("position");

   if (!ps->providesAttribute("color")) {
      bShowColors = false;
   } else {
      if ((*(ps->getDescr()))["color"].dataFormat != VAD::DATA_FORMAT_FLOAT32
       || (*(ps->getDescr()))["color"].numComponents != 3) {
         bShowColors = false;
      } else {
         COLOR = ps->getAAT("color");
      }
   }
   if (!ps->providesAttribute("normal")) {
      bShowNormals = false;
   } else {
      if ((*(ps->getDescr()))["normal"].dataFormat != VAD::DATA_FORMAT_FLOAT32
       || (*(ps->getDescr()))["normal"].numComponents != 3) {
         bShowNormals = false;
      } else {
         NORMAL = ps->getAAT("normal");
      }
   }

   bool canDrawQuads = drawQuads && ps->providesAttribute("tangent_u")
                                      && ps->providesAttribute("tangent_v");

   AAT TANGENT_U;
   AAT TANGENT_V;
   if (canDrawQuads) {
      if ((*(ps->getDescr()))["tangent_u"].dataFormat != VAD::DATA_FORMAT_FLOAT32
       || (*(ps->getDescr()))["tangent_u"].numComponents != 3) {
         canDrawQuads = false;
      } else {
         TANGENT_U = ps->getAAT("tangent_u");
      }
      if ((*(ps->getDescr()))["tangent_v"].dataFormat != VAD::DATA_FORMAT_FLOAT32
       || (*(ps->getDescr()))["tangent_v"].numComponents != 3) {
         canDrawQuads = false;
      } else {
         TANGENT_V = ps->getAAT("tangent_v");
      }
   }

   if (canDrawQuads) {
      glBegin(GL_QUADS);
   } else {
      glBegin(GL_POINTS);
   }
	if( !bShowColors )
	{
		glColor3fv(defaultColor.data());
	}
   const card32 n = ps->getNumEntries();

   for (card32 i=0; i<n; i++){
      if (flags) {
         int32 flag = ps->get<int32, 1>(i, FLAGS)[0];
         bool valid = ((flag & includeFlags) == includeFlags) && ((flag & excludeFlags) == 0);
         if (!valid) continue;

         //Here would be a good place to draw FLAG_SELECTED
         //like
         //if (flag & FLAG_SELECTED) glColor3f(1.f,0,0);
      }
      if (bShowColors) glColor3fv(ps->get<float32, 3>(i, COLOR).data());
      if (bShowNormals) glNormal3fv(ps->get<float32, 3>(i, NORMAL).data());
      if (canDrawQuads) {
         Vector3f pos = ps->get<float32, 3>(i, POSITION);
         Vector3f du = ps->get<float32, 3>(i, TANGENT_U);
         Vector3f dv = ps->get<float32, 3>(i, TANGENT_V);
         glVertex3fv((pos-du-dv).data());
         glVertex3fv((pos+du-dv).data());
         glVertex3fv((pos+du+dv).data());
         glVertex3fv((pos-du+dv).data());
      } else {
         glVertex3fv(ps->get<float32, 3>(i, POSITION).data());
      }
         //glColor3f(1.f,1.f,1.f);
   }
   glEnd();
   forget(ctxt);
}

void SimplePCMaterial::intern_drawNormals(GLContext *ctxt, const PointSet * ps, const int32 includeFlags, const int32 excludeFlags) {
   AAT POSITION;
   AAT COLOR;
   AAT NORMAL;
   AAT FLAGS;
   //float normalLength = normalLength;
   bool flags = ps->providesAttribute("flags");
   if (flags) {
      if ((*(ps->getDescr()))["flags"].dataFormat != VAD::DATA_FORMAT_INT32
       || (*(ps->getDescr()))["flags"].numComponents != 1) {
         flags = false;
      } else {
         FLAGS = ps->getAAT("flags");
      }
   }

   if (!ps->providesAttribute("position")) return;
   POSITION = ps->getAAT("position");

   if (!ps->providesAttribute("color")) {
      showColors = false;
   } else {
      if ((*(ps->getDescr()))["color"].dataFormat != VAD::DATA_FORMAT_FLOAT32
       || (*(ps->getDescr()))["color"].numComponents != 3) {
         showColors = false;
      } else {
         COLOR = ps->getAAT("color");
      }
   }
   if (!ps->providesAttribute("normal")) {
      showNormals = false;
   } else {
      if ((*(ps->getDescr()))["normal"].dataFormat != VAD::DATA_FORMAT_FLOAT32
       || (*(ps->getDescr()))["normal"].numComponents != 3) {
         showNormals = false;
      } else {
         NORMAL = ps->getAAT("normal");
      }
   }

   const card32 n = ps->getNumEntries();
   for (card32 i=0; i<n; i++){
      if (flags) {
         int32 flag = ps->get<int32, 1>(i, FLAGS)[0];
         bool valid = ((flag & includeFlags) == includeFlags) && ((flag & excludeFlags) == 0);
         if (!valid) continue;
      }
      if (showColors) glColor3fv(ps->get<float32, 3>(i, COLOR).data());
      if (showNormals){
//      glNormal3fv(ps->get<float32, 3>(i, NORMAL));
//      glVertex3fv(ps->get<float32, 3>(i, POSITION));
         Vector3f pos=ps->get<float32, 3>(i, POSITION);
         Vector3f normal=ps->get<float32, 3>(i, NORMAL);

         Vector3f x;
         x[0]=1.0f;
         x[1]=0.0f;
         x[2]=0.0f;
         if(pos==x){
            x[0]=0.0f;
            x[1]=1.0f;
         }
         Vector3f e1=normalize(pos.crossProduct(x))*normalLength/20.0f;
         Vector3f e2=normalize(x.crossProduct(e1))*normalLength/20.0f;

         Vector3f a=pos+(normal*normalLength);
         Vector3f b=pos+e1;
         Vector3f c=pos-e1;
         glNormal3fv(normalize(e2).data());
         glVertex3fv(a.data());
         glVertex3fv(b.data());
         glVertex3fv(c.data());
         b=pos+e2;
         c=pos-e2;
         glNormal3fv(normalize(e1).data());
         glVertex3fv(a.data());
         glVertex3fv(b.data());
         glVertex3fv(c.data());
      }
   }
   forget(ctxt);
}

