//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "GLMaterial.h"
//---------------------------------------------------------------------------
#include "DebugOutput.h"
#include "FixedArrayClassProperty.h"
#include "StringClassProperty.h"
#include "BooleanClassProperty.h"
#include "NumericalClassProperty.h"
#include "X4Assert.h"
#include "PointCloudIterators.h"
#include "PointCloud.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "GLBuffer.h"
#include "GLContext.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra.inline.h"
#include "VertexDescriptor.h"
#include "Tools.h"
#include "Object.h"
#include "X4Exception.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_ABSTRACT_CLASS(GLMaterial,0) {
	BEGIN_CLASS_INIT(GLMaterial)
   ADD_STRING_PROP_METHOD( name, 0, &getName, &setName );
   SET_USER_EDIT( false );
}


// ----

IMPLEMENT_X4_ABSTRACT_CLASS(GLRenderObject, 0) {
	BEGIN_CLASS_INIT(GLRenderObject)
}

// ----

IMPLEMENT_X4_ABSTRACT_CLASS(BufferedRenderObject, 0) {
	BEGIN_CLASS_INIT(BufferedRenderObject)
}

// ----

GLMaterial::GLMaterial() {
   name = "unnamed material";
//   m_Container = new AttachedDataContainer;
}

GLMaterial::~GLMaterial()
{
//   delete m_Container;
}


void GLMaterial::assign(const Object* obj, X4CopyContext *context )
{
   GLMaterial * mat = (GLMaterial*)obj;
   name = mat->name;
   forget(context);
}


BufferedRenderObject::BufferedRenderObject()
{
   m_VertexBuffer = NULL;
   m_RenderMode = RENDERMODE_POINTS;
}

BufferedRenderObject::~BufferedRenderObject()
{
   if( m_VertexBuffer )
      delete m_VertexBuffer;
}

void BufferedRenderObject::setup( GLContext * context, const VertexDescriptor &vdesc)
{
   if( m_VertexBuffer )
      throw X4Exception("BufferedRenderObject::setup - vertex buffer already exists! multiple use of setup is not allowed!");

   GLRenderObject::setup(context);
   m_VertexBuffer = new GLVertexBuffer( context, vdesc );
}

void BufferedRenderObject::setBufferData( void * data, card32 offset, card32 numVertices )
{
   if( m_VertexBuffer == NULL )
      throw X4Exception("BufferedRenderObject::setBufferData - vertex buffer doesn't exists! use setup before using setBufferData");

   m_VertexBuffer->init( numVertices );
   m_VertexBuffer->writeToBuffer( data, offset, numVertices );
}

void BufferedRenderObject::setMappingFlag( bool f )
{
   if( m_VertexBuffer )
      m_VertexBuffer->setMappingFlag(f);
}




