//---------------------------------------------------------------------------
#ifndef GLMaterialH
#define GLMaterialH
//---------------------------------------------------------------------------
#include "x3sceneGraph.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "PointFlags.h"
//---------------------------------------------------------------------------
#include <map>
//---------------------------------------------------------------------------

namespace X4 {

class PointSet;
class Shader;
class VertexDescriptor;
class Scene;
class SceneObject;
class SceneObjectIterator;
class MemoryManager;
class GLContext;
class GLVertexBuffer;
class GLShader;
class SceneGraphState;


class X3SCENEGRAPH_API GLMaterial : public Persistent {
	X4_ABSTRACT_CLASS(GLMaterial)
 private:
   std::string name;

 public:
   GLMaterial();

   virtual void assign(const Object* obj, X4CopyContext *context=NULL );

   // new methods
   std::string getName() const {return name;}
   void setName(std::string n) {name = n;}

   /// Draw point cloud defined by "iter", beeing part of "cloud", which itself beeing a part of "scene".
   /// GL buffer objects are created automatically and attached to the input parameters.
   /// A memory manager can be specified for limiting memory usage (as well as for final cleanup).
   /// Flagged point clouds: points not including "includeFlags" or including "excludeFlags" are ignored.
   virtual void draw(Scene *scene, SceneGraphState *state, SceneObject *obj,
      SceneObjectIterator *iter, GLContext *context, MemoryManager * mm = NULL,
      const int32 includeFlags = 0, const int32 excludeFlags = PF_FLAG_INVALID ) = 0;

   virtual void connect( Scene * scene, GLContext * context ) {}

   virtual void rebuildRenderObjects(){}

   virtual ~GLMaterial();
};



class X3SCENEGRAPH_API GLRenderObject : public Object {
 private:
	 X4_ABSTRACT_CLASS(GLRenderObject)

   GLContext *ctxt;

 public:
   virtual ~GLRenderObject(){};

   void setup(GLContext *ctxt) {this->ctxt = ctxt;}
   virtual bool isCompatible(GLContext *ctxt) = 0;
};



class X3SCENEGRAPH_API BufferedRenderObject : public GLRenderObject
{
 private:
	X4_ABSTRACT_CLASS(BufferedRenderObject)

   GLVertexBuffer * m_VertexBuffer;
   card32               m_RenderMode;
   card64               m_Version;

 public:
   BufferedRenderObject();
   virtual ~BufferedRenderObject();

   // rendermode
   static const card32 RENDERMODE_POINTS = 1;
   static const card32 RENDERMODE_QUADS = 2;
   static const card32 RENDERMODE_LINES = 3;
   void setRenderMode( card32 rm ) {m_RenderMode = rm;}
   card32 getRenderMode() const {return m_RenderMode;}

   // version
   card64 getVersion() const {return m_Version;}

   // creation
   void setVersion( card64 version ) {m_Version = version;}
   void setup( GLContext * context, const VertexDescriptor &vdesc);
   void setBufferData( void * data, card32 offset, card32 numVertices );
   void setMappingFlag( bool f);

   // draw
   // for special use
   GLVertexBuffer * getGLVertexBuffer() const {return m_VertexBuffer;}


};




}

#endif

