#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "LogOutput.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "AttachedData.h"
#include <sstream>
#include <iostream>
#include "scene.h"
#include "AttachedData.h"
#include "SparseLinearAlgebra.inline.h"
#include <opencv\cxcore.h>
#include "..\..\foundation\x4interface\source\DebugOutput.h"
//---------------------------------------------------------------------------
using namespace X4;

LogOutput X4::logOutput;

LogOutput::LogOutput(void)
{
	m_Scene = NULL;
}

LogOutput::~LogOutput(void)
{
}


LogOutput& LogOutput::operator<<(bool b) 
{
	std::ostringstream s;
	s << (b?"true":"false");
	getString() += s.str();
	debugOutput << s.str();
	return *this;
}

LogOutput& LogOutput::operator<<(int i) 
{
	std::ostringstream s;
	s << i;
	getString() += s.str();
	debugOutput << i;
	return *this;
}

LogOutput& LogOutput::operator<<(unsigned int i) 
{
	std::ostringstream s;
	s << i;
	getString() += s.str();
	debugOutput << i;
	return *this;
}

LogOutput& LogOutput::operator<<(X4::int64 i) 
{
	std::ostringstream s;
	s << i;
	getString() += s.str();
	debugOutput << i;
	return *this;
}

LogOutput& LogOutput::operator<<(card64 i) 
{
	std::ostringstream s;
	s << i;
	getString() += s.str();
	debugOutput << i;
	return *this;
}

LogOutput& LogOutput::operator<<(long double d) 
{
	std::ostringstream s;
	s << d;
	getString() += s.str();
	debugOutput << d;
	return *this;
}

LogOutput& LogOutput::operator<<(float f) 
{
	std::ostringstream s;
	s << f;
	getString() += s.str();
	debugOutput << f;
	return *this;
}

LogOutput& LogOutput::operator<<(double d) 
{
	std::ostringstream s;
	s << d;
	getString() += s.str();
	debugOutput << d;
	return *this;
}

LogOutput& LogOutput::operator<<(std::string s) 
{
	std::ostringstream os;
	os << s;
	getString() += os.str();
	debugOutput << s;
	return *this;
}

LogOutput& LogOutput::operator<<(char* s) 
{
	std::ostringstream os;
	os << s;
	getString() += os.str();
	debugOutput << s;
	return *this;

}

LogOutput& LogOutput::operator<<(const Vector2f &v) 
{
	return (*this) << "(" << (long double)v[0] << " / " << (long double)v[1] << ")";
}

LogOutput& LogOutput::operator<<(const Vector3f &v) 
{
	return (*this) << "(" << (long double)v[0] << " / " << (long double)v[1] << " / " << (long double)v[2]<< ")";
}

LogOutput& LogOutput::operator<<(const Vector4f &v) 
{
	return (*this) << "(" << (long double)v[0] << " / " << (long double)v[1] << " / " << (long double)v[2] << " / " << (long double)v[3] << ")";
}

LogOutput& LogOutput::operator<<(const Vector6f &v) 
{
	return (*this) << "(" << (long double)v[0] << " / " << (long double)v[1] << " / " << (long double)v[2] << " / " << (long double)v[3]
		<< " / " << (long double)v[4] << " / " << (long double)v[5] << ")";
}

LogOutput& LogOutput::operator<<(const Vector3i &v) 
{
	return	(*this) << "(" << int32(v[0]) << " / " << (int32)v[1] << " / " << (int32)v[2] << ")";

}

LogOutput& LogOutput::operator<<(const Matrix4f &m) 
{
	for( unsigned i=0;i<4;i++ )
	{
		for( unsigned j=0;j<4;j++ )
		{
			(*this) << " \t" << m[j][i] << "";
		}
		(*this) << "\n";
	}
	return *this;
}

LogOutput& LogOutput::operator<<(const Matrix3f &m) 
{
	for( unsigned i=0;i<3;i++ )
	{
		for( unsigned j=0;j<3;j++ )
		{
			(*this) << " \t" << m[j][i] << "";
		}
		(*this) << "\n";
	}
	return *this;
}

LogOutput& LogOutput::operator<<(const BoundingBox3f &bbox) 
{
	(*this) << "(" << bbox.lowerCorner[0] << "," << bbox.lowerCorner[1] << "," << bbox.lowerCorner[2] << ")/";
	(*this) << "(" << bbox.upperCorner[0] << "," << bbox.upperCorner[1] << "," << bbox.upperCorner[2] << ")";
	return *this;
}

LogOutput& LogOutput::operator<<(const StaticVector<float32,7> &v) 
{
	(*this) << "(" << v[0] << "," << v[1] << "," << v[2] << "," << v[3] << "," << v[4] << "," << v[5] << "," << v[6] << ")";
	return *this;
}

LogOutput& LogOutput::operator<<(const DynamicVector<float> &v) 
{
	(*this) << "(";
	for( unsigned i=0;i<v.size();i++ )
	{
		if( i )
			(*this) << ",";
		(*this) << v[i];
	}
	(*this) << ")";

	return *this;	
}

LogOutput& LogOutput::operator<<(const DynamicMatrix<float> &m) 
{
	for ( unsigned int r = 0; r < m.getRows(); r++ ){
		for ( unsigned int c = 0; c < m.getColumns(); c++ ){
			(*this) << m[c][r] << " ";
		}
		(*this) << "\n";
	}
	return *this;
}

LogOutput& LogOutput::operator<<(const CvMat &m) 
{
	for (int32 i=0;i<m.rows;i++ )
	{
		(*this) << "\n";
		for (int32 j=0;j<m.cols;j++)
		{
			if( j )
				(*this) << ",   ";
			(*this) << cvmGet(&m,i,j);
		}
	}
	(*this) << "\n";

	return *this;
}

LogOutput& LogOutput::operator<<(SparseMatrixF &A) 
{
	float32 value;
	mpcard index;
	for ( mpcard r = 0; r < A.getRows(); r++ ){
		SparseVectorF& row = A[r];
		for ( mpcard i = 0; i < row.getNumSparseEntries(); i++ ){
			row.getSparseEntry( i, index, value );
			(*this) << index << ": " << value << ", ";
		}
		(*this) << "\n";
	}
	return *this;
}

void X4::LogOutput::beginLogOutput( Scene * scene )
{
	m_Scene = scene;
}


std::string & X4::LogOutput::getString()
{
	if( m_Scene == NULL )
	{
		throw X4Exception( "LogOutput - Can't write to log." );
	}
	StringAttachment * att = dynamic_cast<StringAttachment*>(m_Scene->getAttachments()->getData("log"));
	if( att == NULL )
	{
		att = new StringAttachment;
		att->setup("log", AttachedData::ADF_PERSISTENT);
		m_Scene->getAttachments()->attachData(att);
	}

	return att->m_String;
}