//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudTools.h"
#include "ParametricInCorePointCloud.h"

#include "Random.h"
#include "DebugOutput.h"

#include "SGIndexedAndByNameSceneGraphIteratorAdaptor.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "SGListNode.h"
#include "StringHelper.h"
#include "PointSet.h"
#include "PointSet.inline.h"
#include "NormalDistribution.h"
#include "VertexArray.h"
#include "ProgressWindow.h"
#include <float.h>
#include "SimpleAttachment.h"
#include "PointSetTools.h"
#include "TopologyAttachment.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//--------------------------------------------------------------------------


SGObjectNode* X4::getSceneGraphObject( Scene* scene, std::string pcID ) {
	if ( scene == NULL ) return NULL;

//	std::auto_ptr<StandardSceneGraphIterator> it( new StandardSceneGraphIterator() );
	StandardSceneGraphIterator it;
	it.clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it.atEnd() ){
		if ( it.getNameWithPath() == pcID ){
			SGObjectNode* sgo = dynamic_cast<SGObjectNode*>( it.getNode() );
			if (sgo != NULL) {
				return sgo;
			}
		}
		it.next();
	}
	return NULL;
}


/**
* Find point cloud in scene
*/
PointCloud* X4::getPointCloud( Scene* scene, std::string pcID )
{
	SGObjectNode* sgo = getSceneGraphObject(scene, pcID);

	return sgo == NULL ? NULL : dynamic_cast<PointCloud*>( sgo->getSceneObject() );
}

/**
* Find any scene object
*/
SceneObject* X4::getSceneObject( Scene* scene, std::string pcID ) {
	SGObjectNode* sgo = getSceneGraphObject(scene, pcID);

	return sgo == NULL ? NULL : sgo->getSceneObject();
}

/**
* Replace point cloud 'pcID' with 'newPC'
*/
void X4::replacePointCloud( Scene* scene, std::string pcID, PointCloud* newPC )
{
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		if ( it->getNameWithPath() == pcID ){
			SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
			sgon->clearAndSetup( newPC );
			return;
		}
		it->next();
	}
	delete it;
}

X3SCENEGRAPH_API void X4::addOrReplacePointCloud( Scene* scene, std::string pcID, PointCloud* newPC )
{
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		if ( it->getNameWithPath() == pcID ){
			SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
			sgon->clearAndSetup( newPC );
			return;
		}
		it->next();
	}
	delete it;

	addSceneObject(scene, newPC, pcID);
}

SceneGraphNode* X4::addSceneObject( Scene * scene, SceneObject *so, std::string name )
{
	SGObjectNode *n = new SGObjectNode();
	n->clearAndSetup( so );
	if( ((int)name.find( "/" ))<0  )
	{
		n->setName( name );
		SGListNode *rootNode = dynamic_cast<SGListNode*>(scene->getRootNode());
		rootNode->addNode( n );
	}
	else
	{
		std::vector<std::string> words = splitString( name, "/" );
		SGListNode *node = dynamic_cast<SGListNode*>(scene->getRootNode());
		if (node->getName() != words[0]) {
			// root name missing
			words.insert(words.begin(), node->getName());
		}
		x4Assert( node->getName() == words[0] );

		bool found = false;

		for( card32 i=1;i<words.size()-1;i++ )
		{

			for( card32 j=0;j<node->getNumChildNodes( NULL );j++ )
			{
				SGListNode * tmpNode = dynamic_cast<SGListNode*>(node->getChildNode( NULL, j ));
				if( tmpNode )
					if( tmpNode->getName() == words[i] )
					{
						found = true;
						node = tmpNode;
						break;
					}
			} 
		}

		//x4Assert( found ); // maybe create listnode here ?
		if (!found&&words.size()>2)
		{
			SGListNode * newListNode = new SGListNode();
			newListNode->setName(words[1]);
			dynamic_cast<SGListNode*>(scene->getRootNode())->addNode(newListNode);
			n->setName( words.back() );
			newListNode->addNode(n);
			return n;
		}


		n->setName( words.back() );
		node->addNode( n );
	}
	return n;
}


/**
* Add point cloud 'pc' as a child of the root node; set node name to 'name'.
*/
SceneGraphNode* X4::addPointCloud( Scene* scene, PointCloud* pc, std::string name )
{
	return addSceneObject(scene,pc,name);
}


/**
* Add point cloud 'pc' as a child of the root node; set node name to 'name'; returns really set name because this can differ from the requested name. always use this returned name afterwards!
*/

std::string X4::addPointCloudAndReturnNodeName( Scene* scene, PointCloud* pc, std::string name )
{
	SGObjectNode *n = new SGObjectNode();
	n->clearAndSetup( pc );
	n->setName( name );
	SGListNode *rootNode = dynamic_cast<SGListNode*>(scene->getRootNode());
	rootNode->addNode( n );
	return n->getName();
}


// Check if PointCloud has attribute (attrib, numComponents, dataType), if not add it
void X4::checkAttribute( Scene* scene, std::string id, std::string attrib, card32 numComponents, card8 dataType )
{
	// Find node in sg;
	PointCloud *pc = NULL;
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		//debugOutput << it->getNameWithPath() << " vs. " << id << "\n";
		//debugOutput.flush();
		if ( it->getNameWithPath() == id ){
			SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
			pc = dynamic_cast<PointCloud*>( sgon->getSceneObject() );
			if ( pc == NULL ){
				debugOutput << "X4::checkAttribute: invalid point cloud: " << id << "!\n";
				debugOutput.flush();
				return;
			}
			// Create new node with attribute
			if ( !pc->providesAttribute( attrib ) ){
					checkAttribute(pc, attrib, numComponents, dataType);
// 				VertexDescriptor *newDescr = (VertexDescriptor*)(pc->getDescr()->copy());
// 				newDescr->pushAttrib( mVAD( attrib, numComponents, dataType ) );
// 				// Replace old node
// 				newPC = pc->appendFields( newDescr );
// 				sgon->clearAndSetup( newPC );

			} else {
				//debugOutput << "Point cloud " << id << " already has attribute " << attrib << "!\n";
				//debugOutput.flush();
			}
			return;
		}
		it->next();
	}

	debugOutput << "X4::checkAttribute: can find point cloud: " << id << "!\n";
	debugOutput.flush();
}

void X4::checkAttribute(PointCloud * pc, std::string attrib, card32 numComponents, card8 dataType)
{
	if( !pc->providesAttribute( attrib ) )
	{
		VertexDescriptor vd;
		vd.pushAttrib( VAD( attrib, numComponents, dataType ) );
		UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
		if( upc )
		{
			PointSet * psNew = upc->getPointSet()->appendFields(&vd);
			upc->setPointSet(psNew);
		}
		else
		{
			PointCloud * pcCopy = pc->appendFields( &vd );
			pc->assign( pcCopy );
			delete pcCopy;
		}
	}
}

/**
* CheckAttribute: Check for (position, normal, flags, color)
*/
void X4::checkSceneAllAttributes(Scene* scene){
	SGLegacyPCSceneAdaptor sceneLA(scene);
	for ( card32 i=0; i<sceneLA.getNumPointClouds(); i++ ){
		std::string pcName = sceneLA.getNameWithPath( i );
		checkAttribute( scene, pcName, "color", 3, VAD::DATA_FORMAT_FLOAT32 );
		checkAttribute( scene, pcName, "flags", 1, VAD::DATA_FORMAT_INT32 );
		checkAttribute( scene, pcName, "normal", 3, VAD::DATA_FORMAT_FLOAT32 );
		checkAttribute( scene, pcName, "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	}
}

/**
* Converts input PointCloud to UnstructuredInCorePointCloud. Returns NULL on Error.
* Input PointCloud is not deleted!!!
*/
UnstructuredInCorePointCloud* X4::convertToUnstructuredInCorePointCloud(PointCloud* inPC) {
	// input
	SceneObjectIterator *inIt = inPC->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	if (inIt == NULL) return NULL;
	BasicPointCloudIterator *inBasicIt = dynamic_cast<BasicPointCloudIterator*>(inIt);
	if (inBasicIt == NULL) {delete inIt; return NULL;}

	//output
	UnstructuredInCorePointCloud* outPC = new UnstructuredInCorePointCloud();
	outPC->clearAndSetup(inPC->getDescr(), 0);

	outPC->assignSettings(inPC);

	PointCloudIterator *outIt = outPC->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	if (outIt == NULL) { delete inIt; delete outPC; return NULL; };
	DynamicPointCloudIterator *outDynIt = dynamic_cast<DynamicPointCloudIterator*>(outIt);
	if (outDynIt==NULL) {delete inIt; delete outIt; delete outPC; return NULL; }

	char *buffer = new char[inPC->getDescr()->getSize()];

	while(!inBasicIt->atEnd()) {
		inBasicIt->getVertex(buffer);
		outDynIt->add(buffer);
		inBasicIt->next();
	};

	delete inIt;
	delete outIt;

	return outPC;
};

/// Return the bounding box of a point cloud
BoundingBox3f  X4::getPCBBox( PointCloud* pc )
{
	BoundingBox3f bb;
	if ( !pc->providesAttribute( "position" ) ) return bb;

	SceneObjectIterator *newIt = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC ) );
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
	AAT POSITION = pc->getAAT( "position" );
	Matrix4f transformation = pc->getTransformation();
	basicIt->reset();
	bool bbIsEmpty = true;
	while( !basicIt->atEnd() ){
		if ( bbIsEmpty ){
			bb = BoundingBox3f( transformVector3f(transformation, basicIt->get3f( POSITION )) );
			bbIsEmpty = false;
		} else
			bb.addPoint( transformVector3f(transformation, basicIt->get3f( POSITION ) ));

		basicIt->next();
	}
	delete newIt;
	return bb;
}

/// Return the bounding box of all scene point clouds
BoundingBox3f X4::getScenePCBBox( Scene* scene )
{
	BoundingBox3f bb;
	PointCloud* pc = NULL;
	// Search for point cloud
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	bool containsPointClouds = false;
	while ( !it->atEnd() ){
		SGObjectNode* sgo = dynamic_cast<SGObjectNode*>( it->getNode() );
		if ( sgo != NULL ){
			pc = dynamic_cast<PointCloud*>( sgo->getSceneObject() );
			if ( pc != NULL && pc->providesAttribute("position") && pc->getNumPoints() > 0){
				if ( !containsPointClouds )
					bb = getPCBBox( pc );
				else
					bb.addBB( getPCBBox( pc ) );
				containsPointClouds = true;
			}
		}
		it->next();
	}
	delete it;
	if ( !containsPointClouds )
		return BoundingBox3f( makeVector3f( -1, -1, -1 ), makeVector3f( 1,1,1 ) );
	else 
		return bb;
}

/// Return the bounding box of all scene point clouds that are visible
BoundingBox3f X4::getScenePCBBoxVisibleOnly( Scene* scene )
{
	BoundingBox3f bb;
	PointCloud* pc = NULL;
	// Search for point cloud
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState(), SGDynamicState::tmRendering );
	bool containsPointClouds = false;
	while ( !it->atEnd() ){
		SGObjectNode* sgo = dynamic_cast<SGObjectNode*>( it->getNode() );
		if ( sgo != NULL ){
			pc = dynamic_cast<PointCloud*>( sgo->getSceneObject() );
			if ( pc != NULL && pc->providesAttribute("position") && pc->getNumPoints() > 0){

				BoundingBox3f pc_bb;
				// special case: hierarchical point cloud
				SceneObjectIterator * iter = pc->createIterator(SOIT::CAP_HIERARCHICAL);
				HierarchicalIterator * hIt = dynamic_cast<HierarchicalIterator *>(iter);
				if( iter && hIt->getNumChildren() > 0 )
				{
					pc_bb = hIt->getBoundingBox();
				}
				else
					pc_bb = getPCBBox( pc );
				delete iter;

				if ( !containsPointClouds )
					bb = pc_bb;
				else
					bb.addBB( pc_bb );
				containsPointClouds = true;
			}
		}
		it->next();
	}
	delete it;
	if ( !containsPointClouds )
		return BoundingBox3f( makeVector3f( -1, -1, -1 ), makeVector3f( 1,1,1 ) );
	else 
		return bb;
}



/**
* Return transformation that automatically aligns the point cloud
*/
Matrix4f X4::getAutoAlignMatrix(PointCloud* pc)
{
	// calc bb
	BoundingBox3f bb;
	bool posset = false;
	AAT POSITION = pc->getAAT("position");
	SceneObjectIterator *newIt = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
	while( !basicIt->atEnd() ) {
		Vector3f pos = basicIt->get3f(POSITION);
		if (posset) {
			bb.addPoint(pos);
		} else {
			bb = BoundingBox3f(pos);
			posset = true;
		}
		basicIt->next();
	}
	delete newIt;
	// compute transform
	Matrix4f transform = IDENTITY4F;
	Vector3f mean = bb.getCenter();
	transform = makeTranslation4f(-mean);
	float32 diameter = bb.getMaxSideLength();
	if (diameter > 1E-10f) {
		float32 s = 2.0f/diameter;
		transform = makeScale4f(s,s,s) * transform;
	}
	return transform;
}

/**
* Returns resampled point cloud
*/
PointCloud* X4::createResampled( PointCloud *input, card32 x, card32 y ) {

	if (0 == x  || 0 == y ){
		warning("PointCloudTools::createResampled()\nParameters may not be zero!");
		return input;
	}

	// Input Iterator
	SceneObjectIterator *readIt = input->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_PARAMETRIC_PC));
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(readIt);
	ParametricPointCloudIterator *paramIt = dynamic_cast<ParametricPointCloudIterator*>(readIt);
	if (basicIt == NULL || paramIt == NULL){
		warning("PointCloudTools::createResampled()\nInput not parametric!");
		delete readIt;
		return input;
	}

	//Input Dimensions
	card32 w=paramIt->getWidth();
	card32 h=paramIt->getHeight();
	if (w < x  || h< y ){
		warning("PointCloudTools::createResampled()\nInput not parametric, wrong input Dimensions!");
		delete readIt;
		return input;
	}

	// Create Output
	PointCloud *output = NULL;

	if (dynamic_cast<ParametricInCorePointCloud*>(input) != NULL) {
		output = (ParametricInCorePointCloud*)input->copy();
		((ParametricInCorePointCloud*)output)->clearAndSetup(w/x, h/y, input->getDescr());
	} else if (dynamic_cast<UnstructuredInCorePointCloud*>(input) != NULL) {
		output = (UnstructuredInCorePointCloud*)input->copy();
		((UnstructuredInCorePointCloud*)output)->clearAndSetup(input->getDescr(), (w/x)*(h/y));
	} else {
		warning("PointCloudTools::createResampled()\nUnrecognized input type!");
		delete readIt;
		return input;
	}

	// Output Iterator
	SceneObjectIterator *writeIt = output->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_PARAMETRIC_PC | SOIT::CAP_MODIFYABLE_PC);
	ModifyablePointCloudIterator *mody2It = dynamic_cast<ModifyablePointCloudIterator*>(writeIt);
	ParametricPointCloudIterator *param2It = dynamic_cast<ParametricPointCloudIterator*>(writeIt);
	if (mody2It == NULL || param2It == NULL){
		warning("PointCloudTools::createResampled()\nOutput Iterator could not be created!");
		delete readIt;
		delete output;
		delete writeIt;
		return input;
	}

	// Copy every xth, yth Point from Input to Output
	char *buffer=new char[input->getDescr()->getSize()];
	for (card32 i=0; i<=w-x; i+=x)
		for(card32 j=0; j<=h-y; j+=y)
		{
			paramIt->goTo(i,j);
			basicIt->getVertex(buffer);
			param2It->goTo(i/x,j/y);
			mody2It->setVertex(buffer);
		}

		// clean up
		delete basicIt;
		delete writeIt;
		delete[] buffer;
		return output;
};

/**
* Return node from name
*/
SceneGraphNode *X4::getSceneGraphNodeByName(Scene *scene, std::string name) {
	StandardSceneGraphIterator sceneIterator;
	sceneIterator.clearAndSetup(scene->getRootNode(), scene->getRootState());
	SGIndexedAndByNameSceneGraphIteratorAdaptor indexingAdaptor;
	indexingAdaptor.clearAndSetup(&sceneIterator);
	if (!indexingAdaptor.goToName(name)) {
		return NULL;
	} else {
		return indexingAdaptor.getNode();
	}
}
SceneGraphNode *X4::getSceneGraphNodeByName(SceneGraphNode *root, std::string name, const SceneGraphState* state) {
    StandardSceneGraphIterator sceneIterator;
    sceneIterator.clearAndSetup(root, state);
    SGIndexedAndByNameSceneGraphIteratorAdaptor indexingAdaptor;
    indexingAdaptor.clearAndSetup(&sceneIterator);
    if (!indexingAdaptor.goToName(name)) {
        return NULL;
    } else {
        return indexingAdaptor.getNode();
    }
}

/**
* Return point cloud from scene graph node
*/
PointCloud *X4::getPointCloud(SceneGraphNode *node) {
	SGObjectNode *objn = dynamic_cast<SGObjectNode*>(node);
	if (objn == NULL) return NULL;
	return dynamic_cast<PointCloud*>(objn->getSceneObject());
}

/**
* Permute points in set
*/
void X4::randomlyPermutePointSet(PointSet *ps) {
	size_t size = ps->getNumEntries();
	for (mpcard i=0; i<size; i++) {
		swapPSPoints(ps, i, i+random64bit(size-i-1)+1);
	}
}

/**
* Swap two points in point set
*/
void X4::swapPSPoints(PointSet *ps, mpcard i1, mpcard i2) {
	mpcard n = ps->getElementSize();
	char *p1 = ps->getDataPointer(i1);
	char *p2 = ps->getDataPointer(i2);
	for (card32 i=0; i<n; i++) {
		swap(*p1, *p2);
		++p1;
		++p2;
	}
}

bool X4::deletePointCloud( Scene *scene, std::string pcID )
{
   return deleteSceneGraphNode(scene, pcID);
}


/**
* Delete point cloud 'id' from scene. Returns true on success.
*/
bool X4::deleteSceneGraphNode( Scene* scene, std::string id) {
	StandardSceneGraphIterator it;
	it.clearAndSetup(scene->getRootNode(), scene->getRootState());
	SGIndexedAndByNameSceneGraphIteratorAdaptor byName;
	byName.clearAndSetup(&it);
	if (byName.goToName(id)) {
		if (byName.getParent() != NULL) {
			byName.getParent()->removeChildNode(byName.getNode()->getName());
			return true;
		}
	}
	return false;
}


/**
* Compute the regression plane from closest points
*/
void X4::getLocalPlane( RegressionPlane3f& plane, Vector3f pos, PointCloud* pc, HierarchicalKNNIterator* pcNNIt )
{
	AttributeAccessToken POSITION = pc->getAAT( "position" );
	// Extract region of closest neighbor
	plane.clear();
	pcNNIt->setSeekPointAndReset( pos );
	for ( unsigned int n = 0; n < NUM_NEIGHBORS_LOCAL_CONTROL_PLANE; n++ ){
		plane.addPoint( pcNNIt->get3f( POSITION ) );
		pcNNIt->next();
	}
	plane.computePlane();
}


/**
* Compute the regression plane from closest points
*/
void X4::getLocalPlane( RegressionPlane3f& plane, Vector3f pos, float32 lfs, PointCloud* pc, HierarchicalKNNIterator* pcNNIt )
{
	AttributeAccessToken POSITION = pc->getAAT( "position" );
	Vector3f nPos;
	// Extract region of closest neighbor
	plane.clear();
	pcNNIt->setSeekPointAndReset( pos );
	while( !pcNNIt->atEnd() ){
		nPos = pcNNIt->get3f( POSITION );
		if ( fabs( ( nPos - pos ) * ( nPos - pos ) ) > lfs ){
			break;
		}
		plane.addPoint( nPos );
		pcNNIt->next();
	}
	plane.computePlane();
}

Matrix4f X4::getTransformation(Scene *scene, std::string node) {
	StandardSceneGraphIterator sceneIterator;
	sceneIterator.clearAndSetup(scene->getRootNode(), scene->getRootState());
	SGIndexedAndByNameSceneGraphIteratorAdaptor indexingAdaptor;
	indexingAdaptor.clearAndSetup(&sceneIterator);
	bool found = indexingAdaptor.goToName(node);	
	x4Assert(found);
	if (found) {
		return indexingAdaptor.getState()->dynamicState->ctm;
	} else {
		return IDENTITY4F;
	} 
}

float X4::getAveragePointDistance( PointCloud *pc, card32 numDirectNeighbours ) 
{
	SceneObjectIterator *iter = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	HierarchicalKNNIterator * hIt = new HierarchicalKNNIterator(pc, 32, NULL);
	AAT posAAT = pc->getAAT( "position" );

	BoundingBox3f bb = getPCBBox(pc);

	float averageDistance = 0;
	card32 numSamples = 0;

	while( !bIt->atEnd() )
	{
		Vector3f samplePos = bIt->get3f(posAAT);
		hIt->setSeekPointAndReset( samplePos );

		// calc average distance to direct neighbours
		float averageSampleDistance = 0.0f;
		for( card32 i=0;i<numDirectNeighbours;i++ )
		{
			hIt->next();
			float dist = norm( samplePos - hIt->get3f( posAAT ) );
			averageSampleDistance += dist;
		}

		averageSampleDistance /= (float)numDirectNeighbours;
		averageDistance += averageSampleDistance;

		bIt->next();
		numSamples++;
	}

	delete hIt;
	delete iter;

	return averageDistance / (float)numSamples;
}

float X4::getMedianPointDistance( PointCloud *pc, card32 numDirectNeighbours, card32 numSamples )
{
	if( pc->getNumPoints() < 2)
	{
		throw X4Exception(std::string("getMedianPointDistance() - Pointcloud has ") + intToStr(pc->getNumPoints()) + " points!");
	}
	//X4_TIMER_START(getMedianPointDistance);
	SceneObjectIterator *iter = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	RandomAccessPointCloudIterator *rIt = dynamic_cast<RandomAccessPointCloudIterator*>(iter);
	HierarchicalKNNIterator * hIt = new HierarchicalKNNIterator(pc, 32, NULL);
	AAT posAAT = pc->getAAT( "position" );
	std::vector<float> distances;
	if( numSamples == 0)
		distances.resize((card32)pc->getNumPoints() * numDirectNeighbours );
	else
		distances.resize( min( (card32)pc->getNumPoints(), numSamples) * numDirectNeighbours );

	card32 index = 0;
	if( numSamples == 0 || numSamples >= pc->getNumPoints() )
	{
		while( !bIt->atEnd() )
		{
			Vector3f samplePos = bIt->get3f(posAAT);
			hIt->setSeekPointAndReset( samplePos );

			for( card32 i=0;i<numDirectNeighbours;i++ )
			{
				distances[index++] = norm( samplePos - hIt->get3f( posAAT ) );
				hIt->next();
			}

			bIt->next();
		}
	}
	else
	{
		for( card32 i=0;i<numSamples;i++ )
		{
			rIt->goToIndex( random64bit(pc->getNumPoints()) );
			Vector3f samplePos = bIt->get3f(posAAT);
			hIt->setSeekPointAndReset( samplePos );

			for( card32 j=0;j<numDirectNeighbours;j++ )
			{
				distances[index++] = norm( samplePos - hIt->get3f( posAAT ) );
				hIt->next();
			}
		}
	
	}
	sort( distances.begin(), distances.end() );
	delete iter;
	delete hIt;
	//X4_TIMER_STOP(getMedianPointDistance);
	//debugOutput << "getMedianPointDistance: " << distances[ distances.size() / 2] <<"\n";

	return distances[ distances.size() / 2];
}

float X4::getCachedMedianPointDistance( PointCloud * pc, card32 numDirectNeighbours, card32 numSamples )
{
	const std::string attName = "medianPointDistance";
	AttachedDataContainer * att = pc->getAttachments();
	if( SimpleAttachment::exists(att,attName) )
		return SimpleAttachment::get1f(att,attName);
	else
	{
		float32 medPointDistance = getMedianPointDistance(pc,numDirectNeighbours,numSamples);
		SimpleAttachment::set1f(att,attName,medPointDistance);
		return medPointDistance;
	}
}

/// smoothes attribute "inputChannel" and stores the (normalized?) result in "outputChannel"
/// uses a gaussian filter kernel exp( -distance^2 / h^2), distance will be scaled by 1/averagePointDistance
/// averagePointDistance will be estimated when not specified 
/// this function doesn't perform smoothing in a two step way, therefore: inputChannel != outputChannel
/// (otherwise there will be different results)
template<typename FloatType,int dim> void smoothAttribute( PointCloud * pc, std::string inputChannel, 
														  std::string outputChannel, float h,bool normalize=false,
														  float averagePointDistance = 0.0f )
{
	if( !pc->providesAttribute( inputChannel ) || !pc->providesAttribute( outputChannel ) )
		throw "Error! Missing channel!\n";

	if( averagePointDistance == 0.0f )
	{
		averagePointDistance = getAveragePointDistance( pc );
	}
	
	float scaleKernel = 1.0f / pow(h * averagePointDistance,2.0f);

	UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	x4Assert( upc != NULL );
	VertexArray va( upc->getPointSet() );

	HierarchicalKNNIterator * hIt = new HierarchicalKNNIterator(pc, 32, NULL );

	AAT posAAT = pc->getAAT( "position" );
	AAT inputAAT = pc->getAAT( inputChannel );
	AAT outputAAT = pc->getAAT( outputChannel );

	
	progressWindow->pushStep( true, "smoothing" );
	for( card32 i=0;i<va.getNumElements();i++ )
	{
		Vector3f pos = va.getPosition3f(i);
		hIt->setSeekPointAndReset( pos );

		float summed_weight = 1.0f;
		StaticVector<FloatType,dim> attribute = hIt->genericGet<FloatType,dim>(inputAAT);
		hIt->next();

		while(!hIt->atEnd() )
		{
			Vector3f nPos = hIt->get3f( posAAT );
			float weight = exp( - normQuad( pos - nPos ) * scaleKernel );
			attribute += hIt->genericGet<FloatType,dim>(inputAAT)*weight;
			summed_weight += weight;

			if( weight < 0.01f )
				break;

			hIt->next();
		}

		attribute /= summed_weight;
		if( normalize )
			attribute.normalize();
		va.set<FloatType,dim>(i, outputAAT, attribute);
		if( !progressWindow->progress((card32)((float)i/(float)va.getNumElements() * 100.0f )) )
			break;
	}

	progressWindow->popSteps();
	

}


void X4::smoothAttribute1f( PointCloud * pc, std::string inputChannel, std::string outputChannel, float h,
							  bool normalize, float averagePointDistance)
{
	smoothAttribute<float,1>( pc, inputChannel, outputChannel, h, normalize, averagePointDistance );
}

void X4::smoothAttribute2f( PointCloud * pc, std::string inputChannel, std::string outputChannel, float h,
							  bool normalize, float averagePointDistance)
{
	smoothAttribute<float,2>( pc, inputChannel, outputChannel, h, normalize, averagePointDistance );
}

void X4::smoothAttribute3f( PointCloud * pc, std::string inputChannel, std::string outputChannel, float h,
							  bool normalize, float averagePointDistance)
{
	smoothAttribute<float,3>( pc, inputChannel, outputChannel, h, normalize, averagePointDistance );
}

void X4::smoothAttribute4f( PointCloud * pc, std::string inputChannel, std::string outputChannel, float h,
							  bool normalize, float averagePointDistance)
{
	smoothAttribute<float,4>( pc, inputChannel, outputChannel, h, normalize, averagePointDistance );
}

void X4::getSGNodeTransformation(Scene *scene, std::string nodeID, Matrix4f &transformation, SceneGraphNode *&node) 
{
	node = NULL;
	if ( scene == NULL )	return;
	StandardSceneGraphIterator it;
	it.clearAndSetup( scene->getRootNode(), scene->getRootState() );
	SGIndexedAndByNameSceneGraphIteratorAdaptor byName;
	byName.clearAndSetup(&it);
	if (byName.goToName(nodeID)) {
		node = byName.getNode();
		transformation = byName.getState()->dynamicState->ctm;
	}
}

VertexDescriptor * X4::createDefaultVertexDescriptor( bool position, bool color, bool normal, bool flags/*=true*/, bool tangents/*=false */ )
{
	VertexDescriptor * vd = new VertexDescriptor;
	if( position )
		vd->pushAttrib(VAD("position",3,VAD::DATA_FORMAT_FLOAT32));
	if( color )
		vd->pushAttrib(VAD("color",3,VAD::DATA_FORMAT_FLOAT32));
	if( normal )
		vd->pushAttrib(VAD("normal",3,VAD::DATA_FORMAT_FLOAT32));
	if( flags )
		vd->pushAttrib(VAD("flags",1,VAD::DATA_FORMAT_INT32));
	if( tangents )
	{
		vd->pushAttrib(VAD("tangent_u",3,VAD::DATA_FORMAT_FLOAT32));
		vd->pushAttrib(VAD("tangent_v",3,VAD::DATA_FORMAT_FLOAT32));
	}

	return vd;
}

X3SCENEGRAPH_API void X4::merge( PointCloud * pc1, const PointCloud * pc2 )
{
	if( *(pc1->getDescr()) != (VertexDescriptor*)(pc2->getDescr()))
		throw "merge(point clouds) - descriptors don't match!";

	SceneObjectIterator * iter = pc1->createIterator( SOIT::CAP_DYNAMIC_PC );
	DynamicPointCloudIterator * dIt = dynamic_cast<DynamicPointCloudIterator*>(iter);
	SceneObjectIterator * iter2 = ((PointCloud*)pc2)->createIterator( SOIT::CAP_BASIC_PC );
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter2);
	std::vector<char> buffer(pc1->getDescr()->getSize());

	Vector3f * pos = NULL;
	if( pc1->providesAttribute("position"))
	{
		AAT posAAT = pc1->getAAT("position");
		pos = (Vector3f*)(&buffer[0] + posAAT.getOffset());
	}
	Vector3f * normal = NULL;
	if( pc1->providesAttribute("normal"))
	{
		AAT normalAAT = pc1->getAAT("normal");
		normal = (Vector3f*)(&buffer[0] + normalAAT.getOffset());
	}

	Matrix4f mat_2to1 = invertFrame(pc1->getTransformation()) * pc2->getTransformation();
	Matrix3f mat_2to1_3f = shrink4To3(mat_2to1);

	while( !bIt->atEnd())
	{
		bIt->getVertex(&buffer[0]);

		if( pos )
			*pos = transformVector3f(mat_2to1, *pos);
		if( normal )
			*normal = mat_2to1_3f* *normal;

		dIt->add(&buffer[0]);
		bIt->next();
	}


	delete iter;
	delete iter2;
}


// get absolute point cloud positions
X3SCENEGRAPH_API void X4::getPCAbsolutPositions(PointCloud* pc, std::vector<Vector3f>& ret) {
/*	ret.clear();
	Matrix4f instanceTransformation = pc->getTransformation();
	mpcard numPts = pc->getNumPoints();
	ret.resize(numPts);
	SceneObjectIterator *iter = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	AAT posAAT = pc->getAAT( "position" );

	while( !bIt->atEnd() ) {
		ret.push_back(transformVector3f(instanceTransformation, bIt->get3f(posAAT)));

		bIt->next();
	}

	delete iter;
*/




	ret.reserve(pc->getNumPoints());
	ret.clear();
	if ( !pc->providesAttribute( "position" ) ) {
		ret.reserve(0);
		return;
	}

	SceneObjectIterator *newIt = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC ) );
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
	AAT POSITION = pc->getAAT( "position" );
	Matrix4f transformation = pc->getTransformation();
	basicIt->reset();
	while( !basicIt->atEnd() ){
		ret.push_back( transformVector3f(transformation, basicIt->get3f( POSITION ) ) );

		basicIt->next();
	}
	delete newIt;
}


X3SCENEGRAPH_API void X4::removePoints( UnstructuredInCorePointCloud* pc, const std::vector<bool>& keep ) {
	X4::PointSetTools::removePoints( pc->getPointSet(), keep );
}

X3SCENEGRAPH_API void X4::removePointsAdjustTriangles( UnstructuredInCoreTriangleMesh* mesh, const std::vector<bool>& keep ) {
	std::vector<int> newIdx;
	newIdx.resize( keep.size(), -1 );

	int i2 = 0;
	for( int i1 = 0; i1 < (int)keep.size(); ++ i1 ) {
		if( keep[i1] ) {
			newIdx[i1] = i2;
			++ i2;
		} else {
		}
	}

	removePoints( mesh, keep );


	PointSet* ps = mesh->getTriangles();
	AAT INDEX = ps->getAAT("index");
	std::vector<bool> keepTri;
	keepTri.resize( ps->getNumEntries(), true);
	for( int i1 = 0; i1 < (int)ps->getNumEntries(); ++ i1 ) {
		Vector3i i = ps->get3i(i1, INDEX );
		i = makeVector3i( newIdx[i[0]], newIdx[i[1]], newIdx[i[2]] );

		if( i[0] < 0 || i[1] < 0 || i[2] < 0 ) {
			keepTri[i1] = false;
		} else {
			ps->set3i(i1, INDEX, i);
		}
	}
	X4::PointSetTools::removePoints(ps, keepTri);

	mesh->getAttachments()->deleteData(TopologyAttachment::getDefaultName());
//	mesh->computeTopologyAttachment();
}

X3SCENEGRAPH_API Vector3f X4::getPCCenterOfGravity( PointCloud* pc )
{
	Vector3f ret = NULL_VECTOR3F;
	if ( !pc->providesAttribute( "position" ) ) {
		return ret;
	}

	SceneObjectIterator *newIt = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC ) );
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
	AAT POSITION = pc->getAAT( "position" );
	basicIt->reset();
	int count = 0;
	while( !basicIt->atEnd() ){
		ret += basicIt->get3f( POSITION );
		++ count;

		basicIt->next();
	}
	delete newIt;

	ret /= count;
	ret = transformVector3f( pc->getTransformation(), ret );

	return ret;
}
