#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SGCameraNode.h"
//---------------------------------------------------------------------------
#include "Camera.h"
#include "PanoramicViewFrustum.h"
#include "CameraController.h"
#include "ObjectClassProperty.h"
#include "ViewFrustum.h"
//---------------------------------------------------------------------------
using namespace X4;

IMPLEMENT_X4_CLASS( SGCameraNode ,0)
{
	BEGIN_CLASS_INIT( SGCameraNode );
	ADD_OBJECT_PROP( m_Camera, 0, Camera::getClass(), true );
	ADD_OBJECT_PROP( m_CameraController, 0, CameraController::getClass(), true );
	ADD_OBJECT_PROP( m_ViewFrustum, 0, ViewFrustum::getClass(), true );
	
}

SGCameraNode::SGCameraNode(void)
{
	m_Camera = new Camera;
	m_Camera->setup( makeVector3f(0,0,0), makeVector3f(0,0,1), makeVector3f(0,1,0));
	m_CameraController = NULL;

	m_ViewFrustum = new ViewFrustum;
	m_ViewFrustum->setAll( 0.001f, 1000.0f, 45.0f, 1.0f );
}

SGCameraNode::SGCameraNode(card32 Type)
{
	m_Camera = new Camera;
	m_Camera->setup( makeVector3f(0,0,0), makeVector3f(0,0,1), makeVector3f(0,1,0));
	m_CameraController = NULL;

	m_ViewFrustum = NULL;
	switch (Type)
	{
	case 0: m_ViewFrustum = new ViewFrustum;
			break;
	case 1: m_ViewFrustum = (ViewFrustum*) new PanoramicViewFrustum(); 
			break;
	}
}


SGCameraNode::~SGCameraNode(void)
{
	delete m_Camera;
	delete m_CameraController;
	delete m_ViewFrustum;
}

void SGCameraNode::assign(const Object* obj, X4CopyContext *context)
{
	SceneGraphNode::assign(obj,context);
	const SGCameraNode * o = dynamic_cast<const SGCameraNode*>(obj);
	x4Assert( o != NULL );

}
void SGCameraNode::setCamera(Camera * cam) 
{
	delete m_Camera;
	m_Camera = cam;
	if( m_CameraController )
		m_CameraController->bindCamera( cam );
}

void SGCameraNode::setViewFrustum(ViewFrustum * vf) 
{
	delete m_ViewFrustum;
	m_ViewFrustum = vf;
}

void SGCameraNode::setCameraController(CameraController * controller) 
{
	delete m_CameraController;
	m_CameraController = controller;
	m_CameraController->bindCamera( m_Camera );
}

CameraController* SGCameraNode::getCameraController() 
{
	m_CameraController->bindCamera( m_Camera );
	return m_CameraController;
}
