//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "SGIndexedAndByNameSceneGraphIteratorAdaptor.h"
//---------------------------------------------------------------------------
#include "SceneGraphState.h"
//---------------------------------------------------------------------------
#include <utility>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



MetaClass* SGIndexedAndByNameSceneGraphIteratorAdaptor::SGIndexedAndByNameSceneGraphIteratorAdaptorMetaClass = NULL;

const MetaClass* SGIndexedAndByNameSceneGraphIteratorAdaptor::getInstanceClass() const {
   return getClass();
}

Object* SGIndexedAndByNameSceneGraphIteratorAdaptor::newInstance() {
   return new SGIndexedAndByNameSceneGraphIteratorAdaptor;
}

const MetaClass* SGIndexedAndByNameSceneGraphIteratorAdaptor::getClass() {
   if (SGIndexedAndByNameSceneGraphIteratorAdaptorMetaClass==NULL) {
      SGIndexedAndByNameSceneGraphIteratorAdaptorMetaClass = new MetaClass("SGIndexedAndByNameSceneGraphIteratorAdaptor", "no description.",
                                  newInstance, sizeof(SGIndexedAndByNameSceneGraphIteratorAdaptor));
   }
   return SGIndexedAndByNameSceneGraphIteratorAdaptorMetaClass;
}

void SGIndexedAndByNameSceneGraphIteratorAdaptor::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   SGIndexedAndByNameSceneGraphIteratorAdaptorMetaClass->copyParentProperties();
}

void SGIndexedAndByNameSceneGraphIteratorAdaptor::shutdown() {
   delete SGIndexedAndByNameSceneGraphIteratorAdaptorMetaClass;
}


// ----


SGIndexedAndByNameSceneGraphIteratorAdaptor::SGIndexedAndByNameSceneGraphIteratorAdaptor() {
   position = 0;
}

void SGIndexedAndByNameSceneGraphIteratorAdaptor::clearAndSetup(SceneGraphIterator *baseIt) {
   card32 i=0;

   SGBasicNodesIterator *basicIt = dynamic_cast<SGBasicNodesIterator*>(baseIt);
   x4Assert(basicIt != NULL);

   if (basicIt == NULL) return;

   while (!basicIt->atEnd()) {
      Entry e;
      e.node = basicIt->getNode();
      e.parent = basicIt->getParent();
      e.name = basicIt->getNameWithPath();
      e.state = const_cast<SceneGraphState*>(basicIt->getState())->createTraversalState();
      entries.push_back(e);
      byName.insert(pair<std::string, card32>(basicIt->getNameWithPath(), i));
      i++;
      basicIt->next();
   }
}

void SGIndexedAndByNameSceneGraphIteratorAdaptor::next() {
   if (position + 1 < entries.size()) {
      position++;
   }
}

bool SGIndexedAndByNameSceneGraphIteratorAdaptor::atEnd() {
   return position >= entries.size()-1;
}

SceneGraphNode *SGIndexedAndByNameSceneGraphIteratorAdaptor::getNode() {
   return entries[position].node;
}

SceneGraphNode *SGIndexedAndByNameSceneGraphIteratorAdaptor::getParent() {
   return entries[position].parent;
}

SceneGraphState *SGIndexedAndByNameSceneGraphIteratorAdaptor::getState() {
   return entries[position].state;
}


std::string SGIndexedAndByNameSceneGraphIteratorAdaptor::getNameWithPath() {
   return entries[position].name;
}


max_platform_card SGIndexedAndByNameSceneGraphIteratorAdaptor::getNumNodes() {
   return entries.size();
}

void SGIndexedAndByNameSceneGraphIteratorAdaptor::goTo(max_platform_card nodeIndex) {
   x4Assert(nodeIndex < entries.size());
   position = nodeIndex;
}

max_platform_card SGIndexedAndByNameSceneGraphIteratorAdaptor::getCurrentIndex() {
   return position;
}


bool SGIndexedAndByNameSceneGraphIteratorAdaptor::goToName(const std::string name) {
	std::string cName = name;
	if (cName.size() > 0 && cName[cName.size()-1] == '/') {
		cName = cName.substr(0, cName.size()-1);
	}
   std::map<std::string, card32>::iterator pos = byName.find(cName);
   if (pos == byName.end()) return false;
   position = pos->second;
   x4Assert(position < entries.size());
   return true;
}

SGIndexedAndByNameSceneGraphIteratorAdaptor::~SGIndexedAndByNameSceneGraphIteratorAdaptor() {
   const size_t n = entries.size();
   for (card32 i=0; i<n; i++) {
      delete entries[i].state;
   }
}
