//---------------------------------------------------------------------------
#ifndef SGIndexedAndByNameSceneGraphIteratorAdaptorH
#define SGIndexedAndByNameSceneGraphIteratorAdaptorH
//---------------------------------------------------------------------------
#include "x3sceneGraph.h"
//---------------------------------------------------------------------------
#include "SceneGraphIterators.h"
//---------------------------------------------------------------------------
#include <vector>
#include <map>
//---------------------------------------------------------------------------

namespace X4 {

/// scene graph nodes iterator adaptor - creates indexed and efficient by name access
/// for arbitrary SGBasicNodeIterators
class X3SCENEGRAPH_API SGIndexedAndByNameSceneGraphIteratorAdaptor : public SceneGraphIterator,
                                                            public SGBasicNodesIterator,
                                                            public SGIndexedNodesIterator,
                                                            public SGByNameNodesIterator
{
 private:
   static MetaClass *SGIndexedAndByNameSceneGraphIteratorAdaptorMetaClass;
   static Object* newInstance();

   struct Entry {
      SceneGraphNode* node;
      SceneGraphNode* parent;
      SceneGraphState* state;
      std::string name;
   };

   std::vector<Entry> entries;
   std::map<std::string, card32> byName;
   mpcard position;

 public:
   SGIndexedAndByNameSceneGraphIteratorAdaptor();

   /// setup iterator adaptor, baseIt must implement interface SGBasicNodesIterator
   /// baseIt evaluation takes place at the time of execution of this method
   virtual void clearAndSetup(SceneGraphIterator *baseIt);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void next();
   virtual bool atEnd();
   virtual SceneGraphNode *getNode();
   virtual SceneGraphNode *getParent();
   virtual SceneGraphState *getState();
   virtual std::string getNameWithPath();

   virtual max_platform_card getNumNodes();
   virtual void goTo(max_platform_card nodeIndex);
   virtual max_platform_card getCurrentIndex();

   virtual bool goToName(const std::string name);

   virtual ~SGIndexedAndByNameSceneGraphIteratorAdaptor();
};


}

#endif
 