//---------------------------------------------------------------------------
#ifndef SGLegacyPCSceneAdaptorH
#define SGLegacyPCSceneAdaptorH
//---------------------------------------------------------------------------
#include "x3sceneGraph.h"
//---------------------------------------------------------------------------
#include "X4Types.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace X4 {

class Scene;
class PointCloud;
class CommandHistory;
class VisitSGNodesOnceIterator;
class StandardSceneGraphIterator;
class SGSceneObjectFilterAdaptor;
class SGIndexedAndByNameSceneGraphIteratorAdaptor;
class SceneGraphState;
class GLMaterial;
class AttachedDataContainer;
class CommandHistory;
class SceneGraphNode;


/// Adaptor class that emulates the "PointCloudScene" interface of previous versions of X4
/// Allows to reuse existing code with scene graph without neither great deal of modifications
/// in existing code nor severe restrictions in scene graph usage.
class X3SCENEGRAPH_API SGLegacyPCSceneAdaptor {
 private:
   Scene *scene;
   StandardSceneGraphIterator *sceneIterator;
   SGSceneObjectFilterAdaptor *pointCloudFilter;
   SGIndexedAndByNameSceneGraphIteratorAdaptor *indexingAdaptor;

   /// recreates all iterators in order to reflect recent changes to scene graph 
   void rebuildIndex();

 public:
   // --- constructor

   /// assumes a scene object with a SGListNode as root object
   /// if root is of another (non-descendand of SGListNode) type,
   /// a SGListNode is inserted automatically
   SGLegacyPCSceneAdaptor(Scene *scene);


   // --- access point clouds within scene graph
   mpcard getNumPointClouds();
   PointCloud *getPointCloud(card32 index);
   PointCloud *getPointCloud(std::string name);
   SceneGraphNode *getNode(card32 index);
   std::string getNameWithPath(card32 index);
   SceneGraphNode *getNode(std::string name);
   /// returns -1 if no point cloud is found
   mpint getPointCloudIndex(std::string name);
   bool visible(card32 index);


   // --- add, remove, replace point clouds
   void addPointCloud(PointCloud* newPC, std::string name = "");
   void addNode(SceneGraphNode *node);
   void deletePointClouds(const std::vector<card32> pcs);
   void setSelectedPCs(const std::vector<card32> &sel);
   void replacePointCloud(card32 i, PointCloud *pc);
   void clear();

   // --- state handling
   virtual SceneGraphState *getRootState();
   // the returned object is deleted when this iterator is deleted
   virtual const SceneGraphState *getState(card32 pcNum);


   // --- materials
   void addMaterial(GLMaterial *mat);
   GLMaterial *getMaterial(card32 i);
   GLMaterial *getMaterial(std::string name);
   int32 getMaterialIndex(std::string name);
   /// returns false if name already exists
   bool renameMat(card32 index, std::string newName);


   // --- attachments
   inline AttachedDataContainer *getAttachments();

   // --- selection
   std::vector<card32> getSelectedPCs();

   // --- command history
   virtual CommandHistory *getCommandHistory();

   ~SGLegacyPCSceneAdaptor();
};

}

#endif
