//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SGRelativeTimeAnimationNode.h"
//---------------------------------------------------------------------------
#include "SGTimeStateChanger.h"
#include "VariableArrayClassProperty.h"
#include "SceneGraphState.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------



IMPLEMENT_X4_CLASS( SGRelativeTimeAnimationNode , 2 ) {
	BEGIN_CLASS_INIT( SGRelativeTimeAnimationNode );
	ADD_FLOAT32_VAR_ARRAY_PROP(timePerFrame, 0);
	SET_STREAM_PROPERTY(false)
	ADD_FLOAT32_PROP_UPDATE(globalTimeMultiplier,1,&computeStartTimes);
	SET_STREAM_PROPERTY(false)
	ADD_BOOLEAN_PROP(noAnim, 2)
}

// ----

SGRelativeTimeAnimationNode::SGRelativeTimeAnimationNode() {
	startTimesValid = false;
	globalTimeMultiplier = 1.0f;
	noAnim = false;
}


SGRelativeTimeAnimationNode::~SGRelativeTimeAnimationNode() {}


void SGRelativeTimeAnimationNode::setDuration(card32 frameIndex, float32 frameTime) 
{
	if (frameIndex > getNumChildNodes(NULL)) {
		throw ERangeCheck("SGRelativeTimeAnimationNode::setDuration() - frameIndex out of range");
	}
	x4Assert(timePerFrame.size() > frameIndex);
	timePerFrame[frameIndex] = frameTime;
	startTimesValid = false;
}

float32 SGRelativeTimeAnimationNode::getFrameTime(card32 frameIndex) 
{
	if (frameIndex > getNumChildNodes(NULL)) {
		throw ERangeCheck("SGRelativeTimeAnimationNode::setDuration() - frameIndex out of range");
	}
	x4Assert(timePerFrame.size() > frameIndex);
	return timePerFrame[frameIndex];
}

void SGRelativeTimeAnimationNode::write(OutputObjectStream *s) const 
{
	SGAnimationNode::write(s);
	mpcard numFrames = timePerFrame.size();
	x4Assert(timePerFrame.size() == getNumChildNodes(NULL));
	s->writeMaxPlatformCard(numFrames);
	for (mpcard i=0; i<numFrames; i++) {
		s->write<float32>(timePerFrame[i]);
	}
	s->write(globalTimeMultiplier);
}

void SGRelativeTimeAnimationNode::read(InputObjectStream *s) 
{
	SGAnimationNode::read(s);
	mpcard numFrames;
	s->readMaxPlatformCard(numFrames);
	timePerFrame.resize(numFrames, 1);
	for (mpcard i=0; i<numFrames; i++) {
		s->read<float32>(timePerFrame[i]);
	}
	startTimesValid = false;
	x4Assert(timePerFrame.size() == getNumChildNodes(NULL));
	if (s->getClassVersion(SGRelativeTimeAnimationNode::getClass()) >= MetaClass::FIRST_VERSION + 1)
		s->read(globalTimeMultiplier);
}

max_platform_card SGRelativeTimeAnimationNode::getNumChildNodes(const SceneGraphState *state) const 
{
   if (state == NULL || state->dynamicState->traversalMode != SGDynamicState::tmRendering || noAnim) {
      return SGListNode::getNumChildNodes(state);
   } else {
      if (SGListNode::getNumChildNodes(state) > 0) {
         return 1;
      } else {
         return 0;
      }
   }
}

SceneGraphNode *SGRelativeTimeAnimationNode::getChildNode(const SceneGraphState *state, max_platform_card i) 
{
	if (SGListNode::getNumChildNodes(state) <= 0) {
		throw EInvalidState("SGRelativeTimeAnimationNode::getChildNode() - no child nodes present.");
	}
	if (!startTimesValid && state != NULL) {
		computeStartTimes();
	}
   if (state == NULL || state->dynamicState->traversalMode != SGDynamicState::tmRendering || noAnim) {
      return SGListNode::getChildNode(state, i);
   } else {
      x4Assert(SGListNode::getNumChildNodes(state) > 0);
      float32 time = state->dynamicState->time; 
      const mpcard n = SGListNode::getNumChildNodes(state);
      x4Assert(n>=1);
      mpcard index = 0;
      if (time > 0) {
			index = n-1;
			x4Assert(n == timePerFrame.size());
			for (mpcard i=0; i<n-1; i++) {
				if (startTimes[i+1] > time) {
					index = i;
					break;
				}
			}
      }
		x4Assert(index >= 0 && index < n);
		
      return SGListNode::getChildNode(state, index);
   }
}

SGStateChanger *SGRelativeTimeAnimationNode::getChildNodeEntryStateChange(const SceneGraphState *state, max_platform_card i)
{
	x4Assert(timePerFrame.size() == getNumChildNodes(NULL));
	if (SGListNode::getNumChildNodes(state) <= 0) {
		throw EInvalidState("SGRelativeTimeAnimationNode::getChildNode() - no child nodes present.");
	}
	if (!startTimesValid) {
		computeStartTimes();
	}
   if (state == NULL || state->dynamicState->traversalMode != SGDynamicState::tmRendering || noAnim) {
      return NULL;
   } else {
      x4Assert(SGListNode::getNumChildNodes(state) > 0);
      float32 time = state->dynamicState->time; 
      float32 startTime = 0;
      const mpcard n = SGListNode::getNumChildNodes(state);
      x4Assert(n>=1);
      mpcard index = 0;
      if (time > 0) {
			index = n-1;
			startTime = startTimes.back();
			x4Assert(n == timePerFrame.size());
			for (mpcard i=0; i<n-1; i++) {
				if (startTimes[i+1] > time) {
					startTime = startTimes[i];
					index = i;
					break;
				}
			}
      }
		x4Assert(index >= 0 && index < n);
		SGTimeStateChanger *change = new SGTimeStateChanger();
		change->setup(time - startTime);
      return change;
   }
}

void SGRelativeTimeAnimationNode::removeChildNode(std::string name) 
{
	max_platform_int which = getChildIndex(name);
	removeNode((card32)which);
	startTimesValid = false;
	x4Assert(timePerFrame.size() == getNumChildNodes(NULL));
}

bool SGRelativeTimeAnimationNode::addChildNode(SceneGraphNode *node) 
{
	addNode(node);
	return true;
}

void SGRelativeTimeAnimationNode::swapChildren(card32 index1, card32 index2) 
{
	SGAnimationNode::swapChildren(index1, index2);
	swap(timePerFrame[index1], timePerFrame[index2]);
	startTimesValid = false;
	x4Assert(timePerFrame.size() == getNumChildNodes(NULL));
}

void SGRelativeTimeAnimationNode::addNode(SceneGraphNode *n) 
{
	SGAnimationNode::addNode(n);
	timePerFrame.push_back(1);
	startTimesValid = false;
	x4Assert(timePerFrame.size() == getNumChildNodes(NULL));
}

void SGRelativeTimeAnimationNode::removeNode(card32 index) 
{
	SGAnimationNode::removeNode(index);
	timePerFrame.erase(timePerFrame.begin()+index);
	startTimesValid = false;
	x4Assert(timePerFrame.size() == getNumChildNodes(NULL));
}

void SGRelativeTimeAnimationNode::clear() 
{
   SGAnimationNode::clear();
	timePerFrame.clear();	
	startTimes.clear();
	startTimesValid = false;
	x4Assert(timePerFrame.size() == getNumChildNodes(NULL));
}

void SGRelativeTimeAnimationNode::computeStartTimes() 
{
	if(timePerFrame.size() == 0 )
		return;
	startTimes.resize(timePerFrame.size(), 0);
	startTimesValid = true;
	startTimes[0] = 0;
	for (unsigned i=1; i<timePerFrame.size(); i++) {
		startTimes[i] = startTimes[i-1] + timePerFrame[i-1]*globalTimeMultiplier;
	}
	x4Assert(timePerFrame.size() == getNumChildNodes(NULL));
}



void SGRelativeTimeAnimationNode::setNumtimePerFrames(card32 size)
{
	startTimesValid = false;
	throw X4Exception("SGRelativeTimeAnimationNode::setNumtimePerFrames() - changing the size of this array via class properties is  not allowed.");
	//timePerFrame.resize(size);
}
void SGRelativeTimeAnimationNode::deletetimePerFrame(card32 index)
{
	startTimesValid = false;
	throw X4Exception("SGRelativeTimeAnimationNode::setNumtimePerFrames() - changing the size of this array via class properties is  not allowed.");
	//x4Assert( index < timePerFrame.size());
	//timePerFrame.erase( startTimes.begin()+index );
}
char *SGRelativeTimeAnimationNode::gettimePerFrame(card32 index)
{
	x4Assert( index < timePerFrame.size()); 
	startTimesValid = false;
	return (char*)&timePerFrame[index];
}
mpcard SGRelativeTimeAnimationNode::getNumtimePerFrames()
{
	return timePerFrame.size();
}

std::string SGRelativeTimeAnimationNode::gettimePerFrameName(card32 index)
{
	x4Assert( index < timePerFrame.size()); 
	return std::string("Frame ")+intToStr(index);
}
