//---------------------------------------------------------------------------
#ifndef SGRelativeTimeAnimationNodeH
#define SGRelativeTimeAnimationNodeH
//---------------------------------------------------------------------------
#include "x3sceneGraph.h"
//---------------------------------------------------------------------------
#include "SGAnimationNode.h"
//---------------------------------------------------------------------------

namespace X4 {

///
///   In contrast to the simple animation node, this node supports a separate time
///   interval for each child node. In addition, the nodes can be cascaded hierarchically;
///   the animation node changes the dynamic time in the scene graph state to
///   time - (start of the current frame)
///
class X3SCENEGRAPH_API SGRelativeTimeAnimationNode : public SGAnimationNode {
	X4_CLASS( SGRelativeTimeAnimationNode )
 private:
	std::vector<float32> timePerFrame;
	std::vector<float32> startTimes;
	bool startTimesValid;
	float32 globalTimeMultiplier;
	/// if set to true, the node acts just as a list node. default=false
	bool noAnim;
	void computeStartTimes();
	
	void setNumtimePerFrames(card32 size);
	void deletetimePerFrame( card32 index );
	char *gettimePerFrame( card32 index );
	mpcard getNumtimePerFrames();
	std::string gettimePerFrameName(card32 index );


 public:
	SGRelativeTimeAnimationNode();

	/// set duration of a frame - standard value is one time unit
   void setDuration(card32 frameIndex, float32 frameTime);
   float32 getFrameTime(card32 frameIndex);
	
	bool getNoAnim() const { return noAnim; }
	/// if set to true, the node acts just as a list node. default=false
	void setNoAnim(bool val) { noAnim = val; }

   void write(OutputObjectStream *s) const;
   void read(InputObjectStream *s);

   virtual max_platform_card getNumChildNodes(const SceneGraphState *state) const;
   virtual SceneGraphNode *getChildNode(const SceneGraphState *state, max_platform_card i);
   virtual SGStateChanger *getChildNodeEntryStateChange(const SceneGraphState *state, max_platform_card i);

   virtual void removeChildNode(std::string name);
   virtual bool addChildNode(SceneGraphNode *node);
   virtual void swapChildren(card32 index1, card32 index2);

   virtual void addNode(SceneGraphNode *n);
   virtual void removeNode(card32 index);
   virtual void clear();

   ~SGRelativeTimeAnimationNode();
};

}

#endif