//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "SGSimpleAnimationNode.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"

#include "SceneGraphState.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


MetaClass* SGSimpleAnimationNode::SGSimpleAnimationNodeMetaClass = NULL;

const MetaClass* SGSimpleAnimationNode::getInstanceClass() const {
   return getClass();
}

Object* SGSimpleAnimationNode::newInstance() {
   return new SGSimpleAnimationNode;
}

const MetaClass* SGSimpleAnimationNode::getClass() {
   if (SGSimpleAnimationNodeMetaClass==NULL) {
      SGSimpleAnimationNodeMetaClass = new MetaClass("SGSimpleAnimationNode", "no description.",
                                                     newInstance, sizeof(SGSimpleAnimationNode));
   }
   return SGSimpleAnimationNodeMetaClass;
}

void SGSimpleAnimationNode::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   SGSimpleAnimationNodeMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT( SGSimpleAnimationNode );
   ADD_FLOAT32_PROP( timePerFrame, 0 ); // 0 = version

}

void SGSimpleAnimationNode::shutdown() {
   delete SGSimpleAnimationNodeMetaClass;
}


// ----


SGSimpleAnimationNode::SGSimpleAnimationNode() {
   timePerFrame = 1;
}

max_platform_card SGSimpleAnimationNode::getNumChildNodes(const SceneGraphState *state) const {
   if (state == NULL || state->dynamicState->traversalMode != SGDynamicState::tmRendering) {
      return SGListNode::getNumChildNodes(state);
   } else {
      if (SGListNode::getNumChildNodes(state) > 0) {
         return 1;
      } else {
         return 0;
      }
   }
}

SceneGraphNode *SGSimpleAnimationNode::getChildNode(const SceneGraphState *state, max_platform_card i) {
   if (state == NULL || state->dynamicState->traversalMode != SGDynamicState::tmRendering) {
      return SGListNode::getChildNode(state, i);
   } else {
      x4Assert(SGListNode::getNumChildNodes(state) > 0);
      float32 time = state->dynamicState->time;
      mpint index = (mpint)(time / timePerFrame);
      if (index < 0) index = 0;
      const mpcard n = SGListNode::getNumChildNodes(state);
      if (index >= (mpint)n) {
         index = (mpint)n-1;
      }
      return SGListNode::getChildNode(state, index);
   }
}

void SGSimpleAnimationNode::setTimePerFrame(float32 timePerFrame) {
   this->timePerFrame = timePerFrame;
}

float32 SGSimpleAnimationNode::getTimePerFrame() {
   return timePerFrame;
}

