//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "SceneGraphTools.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "SceneGraphNode.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


std::string X4::extractSGPath(const std::string &nodeWPath) {
   const int n = (int)nodeWPath.size();
   int result = -1;
   for (int i=n-1; i>=0; i--) {
      if (nodeWPath[i] == '/') {
         result = i;
         break;
      }
   }
   if (result != -1) {
      return nodeWPath.substr(0, result+1);
   } else {
      return nodeWPath;
   }
}


std::string X4::extractSGNode(const std::string &nodeWPath) {
   const int n = (int)nodeWPath.size();
   int result = -1;
   for (int i=n-1; i>=0; i--) {
      if (nodeWPath[i] == '/') {
         result = i;
         break;
      }
   }
   if (result != -1) {
      return nodeWPath.substr(result+1, n-result-1);
   } else {
      return std::string("");
   }
}


std::vector<std::string> X4::parseSGPath(const std::string &path) {
   std::vector<std::string> result;
   std::string currentString;
   const size_t n=path.size();
   for (size_t i=0; i<n; i++) {
      if (path[i] == '/') {
         result.push_back(currentString);
         currentString = "";
      } else {
         currentString.push_back(path[i]);
      }
   }
   if (currentString.size() > 0) {
      result.push_back(currentString);
   }
   return result;
}


std::string X4::endPathWithSlash(const std::string &str) {
	std::string result = str;
	if (result.empty()) {
      x4Assert(false);
		return "";
	}
	if (result[result.size()-1] != '/') {
		result.append(std::string("/"));
	}
   return result;
}

SceneGraphNode *X4::getSceneGraphNode(Scene *scene, std::string path) 
{
   std::vector<std::string> pathParsed = parseSGPath(path);
   return getSceneGraphNode(scene, pathParsed);
}

SceneGraphNode *X4::getSceneGraphNode(Scene *scene, std::vector<std::string> pathComponents) 
{
   SceneGraphNode *currentNode = scene->getRootNode();
   if (currentNode->getName() != pathComponents[0]) return NULL;
   for (size_t i=1; i<pathComponents.size(); i++) {
      mpcard n = currentNode->getNumChildNodes(NULL);
      SceneGraphNode *nextNode = NULL;
      for (mpcard j=0; j<n; j++) {
         if (currentNode->getChildNode(NULL, j)->getName() == pathComponents[i]) {
            nextNode = currentNode->getChildNode(NULL, j);
            break;
         }
      }
      if (nextNode == NULL) {
         return NULL;
      } else {
         currentNode = nextNode;
      }
   }
   return currentNode;
}

void X4::addSceneGraphNode(Scene *scene, std::string path, SceneGraphNode *addThis) 
{
   SceneGraphNode *addWhere = getSceneGraphNode(scene, path);
   if (!addWhere) {
      throw EInvalidState("SceneGraphTools::addSceneGraphNode() - path not found.");
   }
   addWhere->addChildNode(addThis);
}

X3SCENEGRAPH_API std::string X4::makePathUnique( Scene *scene, std::string path )
{
	int i = 0;
	
	while( getSceneGraphNode(scene, path + intToStr(i,3)) )
		++ i;

	return path + intToStr(i,3);
}
