//---------------------------------------------------------------------------
#ifndef StandardSceneGraphIteratorH
#define StandardSceneGraphIteratorH
//---------------------------------------------------------------------------
#include "x3sceneGraph.h"
//---------------------------------------------------------------------------
#include "SceneGraphIterators.h"
#include "SceneGraphState.h"
//---------------------------------------------------------------------------

namespace X4 {

class SGStateChanger;

/// standard scene graph iterator: enumerates all nodes (with corresponding state) in depth-first order
class X3SCENEGRAPH_API StandardSceneGraphIterator : public SceneGraphIterator,
                                           public SGBasicNodesIterator,
                                           public SGHierarchicalIterator {
 private:
   static MetaClass *StandardSceneGraphIteratorMetaClass;
   static Object* newInstance();

   struct Entry {
      /// stack of nodes, top: current node
      SceneGraphNode *node;
      /// stack of state changes (NULL on stack for "no change")
      SGStateChanger *childEntryStateChange;
      /// stack of state changes (NULL on stack for "no change")
      SGStateChanger *nodeEntryStateChange;
      /// stack of node indices to be visited next
      card32 subnodeIndex;
   };

   std::vector<Entry> entriesStack;

   /// original scene graph state (used for reset)
   const SceneGraphState *originalState;
   /// current state (copy, changing during traversal)
   SceneGraphState *currentState;

   /// no more nodes found
   bool endOfTraversal;

   // nodes must include these flags to be reported
   card32 includeNodeFlags;
   // nodes must not include these flags to be reported (both conditions are combined by logical and)
   card32 excludeNodeFlags;

   void clear();
   void applyStateChanges(SceneGraphState *state, Entry &e);
   void undoStateChanges(SceneGraphState *state, Entry &e);
   bool internProcessNode(SceneGraphNode *n);

 protected:
   /// filter function: descide whether to continue traversal at a certain node
   /// override this method to stop traversal at certain nodes
   virtual bool processNode(SceneGraphNode *n) {return true;}


 public:
   StandardSceneGraphIterator();

   virtual void clearAndSetup(SceneGraphNode *rootNode, const SceneGraphState *rootState,
                              // traversal mode - set in scene graph state before starting traversal
                              SGDynamicState::TraversalModeType traversalMode = SGDynamicState::tmDefault,
                              // nodes must include these flags to be reported
                              card32 includeNodeFlags = 0,
                              // nodes must not include these flags to be reported (both conditions are combined by logical and)
                              card32 excludeNodeFlags = 0);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void next();
   virtual bool atEnd();

   virtual SceneGraphNode *getNode();
   virtual SceneGraphNode *getParent();
   virtual SceneGraphState *getState();
   virtual std::string getNameWithPath();

   virtual max_platform_card getNumChildren();
   virtual bool goToChild(max_platform_card i);
   virtual bool goToParent();

   ~StandardSceneGraphIterator();
};

}

#endif
 