#ifndef AR4DiscreteManifoldH
#define AR4DiscreteManifoldH

#include "threeParameterMatchingGMOD14.h"

#include "X4Types.h"
#include "AR4Manifold.h"



namespace X4 {


/// This is an abstract manifold. Once again.
/// It describes a manifold that is embedded in a Euclidean space and discretized into sample points.
template <class ManifoldTraits>
class THREE_PARAMETER_MATCHING_GMOD14_API AR4DiscreteManifold : virtual public AR4Manifold<ManifoldTraits> {
 public: 
   /// the surface is discretized into sample points (query number)
   virtual mpcard getNumDiscrPoints() = 0;
   /// the surface is discretized into sample points (get the point)
   virtual VectorType getDiscrPoint(mpcard index) = 0;
   
   /// scale parameter: sample spacing of the data
   virtual FloatType getSampleSpacing(const mpcard index) const = 0;

   /// precomputed / provided neighborhood graph (number of neighbors per point)
   virtual mpcard getNumNeighbors(mpcard point) = 0;
   /// precomputed / provided neighborhood graph (get concrete neighboring point)
   virtual mpcard getNeighborIndex(mpcard point, mpcard index) = 0;
   
   /// uses Euclidean space
   virtual void getKNearestNeighborsEucl(VectorType point, mpcard k, vector<mpcard> &neighbors) = 0;
   /// uses neighborhood graphs (index version)
   virtual void getSphericalNeighborsNG(mpcard index, FloatType radius, vector<mpcard> &neighbors) = 0;
   /// uses neighborhood graphs (continuous version)
   virtual void getSphericalNeighborsNG(VectorType point, FloatType radius, vector<mpcard> &neighbors) = 0;

   virtual ~AR4DiscreteManifold() {}
};


}

#endif
