#include "AR4FuncOnManifoldInChannel.h"

namespace X4 {


template<class DiscreteManifoldType, typename OutputType>
AR4FuncOnManifoldInChannel<DiscreteManifoldType, OutputType>
::AR4FuncOnManifoldInChannel(DiscreteManifold *manifold, PointSet *representation, VAD channel)
: AR4FuncOnManifold(manifold)
{
   ps = representation;
   channelVAD = channel;
   if (!ps->providesAttribute(channel.name, channel.numComponents, channel.dataFormat)) {
      VertexDescriptor vd;
      vd.pushAttrib(channel);
      ps->appendFields(vd);
   }
   channelAAT = ps->getAAT(channel.name, channel.numComponents, channel.dataFormat);
}


template<class DiscreteManifoldType, typename OutputType>
void AR4FuncOnManifoldInChannel<DiscreteManifoldType, OutputType>::initialize() {
   vector<OutputType> funcValues;
   precompute(getManifold(), funcValues);
   if (!funcValues.size() == representation.getNumEntries()) {
      throw PException("AR4FuncOnManifoldInChannel - init failed, precompute returned wrong number of points.");
   }
   for (size_t i=0; i<funcValues.size; i++) {
      char *dataPtr = ps->getDataPointer(i);
      dataPtr += channelAAT.getOffset();
      *((OutputType*)dataPtr) = funcValues[i];
   }
}

template<class DiscreteManifoldType, typename OutputType>
inline OutputType AR4FuncOnManifoldInChannel<DiscreteManifoldType, OutputType>::evaluate(mpcard discrPoint) const {
   const char *dataPtr = ps->getDataPointer(discrPoint);
   dataPtr += channelAAT.getOffset();
   return *((OutputType*)dataPtr);
}

}