#ifndef AR4MLSManifoldInlineH
#define AR4MLSManifoldInlineH

#include "threeParameterMatchingGMOD14.h"

#include "basics/DeformationTools.h"

#include "AR4MLSManifold.h"



namespace X4 {


	template<class T1, class T2>
	class PairFirstCompare
	{
	public:
	   inline bool operator()(const pair<T1, T2>& p1, const pair<T1, T2>& p2) const
	   {
		   return (p1.first < p2.first);
	   }
	};


template <class ManifoldTraits>
void AR4MLSManifold<ManifoldTraits>::resetup(UnstructuredInCorePointCloud *pc)
{
   this->pc = pc;
   ps = pc->getPointSet();
   tg = X4::getTopology(pc);
   if (!tg) throw EInvalidState("AR4MLSManifold::setup - need neighborhood (\"topology\") graph");

   if (!ps) throw ETypeMismatch("AR4MLSManifold::setup - NULL point set");

   if (ps->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
      positionAAT = ps->getAAT("position");
   } else {
      throw ETypeMismatch("AR4MLSManifold::setup - no position channel found");
   }
   if (ps->providesAttribute("normal", 3, VAD::DATA_FORMAT_FLOAT32)) {
      normalAAT = ps->getAAT("normal");
   } else {
      throw ETypeMismatch("AR4MLSManifold::setup - no normal channel found (required for IMLS)");
   }
   if (ps->providesAttribute("sampleSpacing", 1, VAD::DATA_FORMAT_FLOAT32)) {
      sampleSpacingAAT = ps->getAAT("sampleSpacing");
   } else {
      throw ETypeMismatch("AR4MLSManifold::setup - no sampleSpacing channel found");
   }
}

template <class ManifoldTraits>
AR4MLSManifold<ManifoldTraits>::AR4MLSManifold(UnstructuredInCorePointCloud *pc) {
   ps = NULL;
   tg = NULL;
   sampleSpacingAAT = NULL_AAT;
   positionAAT = NULL_AAT;
   m_sampleEpsilon = (FloatType)1;
   m_avgPointArea = 0;
   m_bAreasDirty = true;
   AR4MLSManifold<ManifoldTraits>::resetup(pc);
}

template <class ManifoldTraits>
AR4MLSManifold<ManifoldTraits>::~AR4MLSManifold() {}


template <class ManifoldTraits>
mpcard AR4MLSManifold<ManifoldTraits>::getNumDiscrPoints()
{
   return ps->getNumEntries();
}

template <class ManifoldTraits>
typename AR4MLSManifold<ManifoldTraits>::VectorType AR4MLSManifold<ManifoldTraits>::getDiscrPoint(mpcard index)
{
   return *((VectorType*)(ps->getDataPointer(index)+positionAAT.getOffset()));
}

template <class ManifoldTraits>
mpcard AR4MLSManifold<ManifoldTraits>::getNumNeighbors(mpcard point)
{
   return tg->getNumAdjacentVertices(point);
}

template <class ManifoldTraits>
mpcard AR4MLSManifold<ManifoldTraits>::getNeighborIndex(mpcard point, mpcard index)
{
   return tg->getAdjacentVertex(point, index);
}



bool AR4MLSManifold3f::project(const Vector3f &startPos, Vector3f &surfacePoint)
{
   Matrix3f refFrame; 
   Vector3f curvTensor_u_uv_v;
   return project(startPos, surfacePoint, refFrame[0], refFrame[1], refFrame[2], curvTensor_u_uv_v);
}

bool AR4MLSManifold3f::project(const Vector3f &startPos, Vector3f &surfacePoint, Vector3f &normal)
{
   Vector3f curvTensor_u_uv_v;
   Vector3f tangentU, tangentV;
   return project(startPos, surfacePoint, tangentU, tangentV, normal, curvTensor_u_uv_v);
}


bool AR4MLSManifold3f::project(const Vector3f &startPos, Vector3f &surfacePoint, Vector3f &normal, StaticMatrix<float32, 2, 3> &tangentSpace)
{
   Vector3f curvTensor_u_uv_v;
   return project(startPos, surfacePoint, tangentSpace[0], tangentSpace[1], normal, curvTensor_u_uv_v);
}

template <class ManifoldTraits>
typename AR4MLSManifold<ManifoldTraits>::FloatType AR4MLSManifold<ManifoldTraits>::getAvgEdgeLength()
{
	mpcard i, j, k, n, m, numEdges;

	VectorType cur, nbr;
	FloatType avgEdge = (FloatType)0;

	n = pc->getNumPoints();

	numEdges = 0;
	for (i = 0; i < n; i++)
	{
		cur = ps->get3f(i, positionAAT);
		m = getNumNeighbors(i);
		for (j = 0; j < m; j++)
		{
			k = getNeighborIndex(i, j);
			nbr = ps->get3f(k, positionAAT);
			avgEdge += (nbr - cur).getNorm();
			numEdges++;
		}
	}

	avgEdge /= (FloatType)numEdges;

	return avgEdge;
}



}

#endif
