#ifndef AR4ManifoldTraitsH
#define AR4ManifoldTraitsH

#include "threeParameterMatchingGMOD14.h"

#include "X4Types.h"
#include "LinearAlgebra.inline.h"
#include "BoundingBox.inline.h"

namespace X4 {

class AR4ManifoldTraits {
   // must have:
   // typename FloatType    - scalars
   // typename VectorType   - a vector type (Static/Dynamic vector) compatible with FloatType
   // typename TangentSpace - a matrix type (Static/Dynamic) compatible with FloatType; the normal is always the last entry
   // typename BoundingBoxType - a bounding box type, compatible with FloatType
};                              


/// traits for the AR4 Manifold with static vectors and flexible dimension
template <unsigned dim, typename TheFloatType>
class AR4ManifoldTraitsStaticVec {
 public:
   /// scalars
   typedef TheFloatType FloatType;
   /// points and vectors
   typedef StaticVector<float32, dim> VectorType;
   /// tangent space
   typedef StaticMatrix<float32, dim-1, dim> TangentSpace;
   /// axis aligned bounding boxes
   typedef BoundingBox<TheFloatType, dim> BoundingBoxType;
};


typedef AR4ManifoldTraitsStaticVec<3, float32> ManifoldTraits3f;

}


#endif
