//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "AR4PartialMatch.h"
#include "AR4PartialMatch.inline.h"
#include "AR4MLSManifold.h"
#include "ProgressWindow.h"
#include "TopologyAttachment.h"
//---------------------------------------------------------------------------
#include <omp.h>
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------


static void zeroAllFlags(UnstructuredInCorePointCloud* pc)
{
	if (pc == NULL)
		throw ERangeCheck("zeroAllFlags(pc): pc == NULL");

	mpcard i, n = pc->getNumPoints();
	AAT flags = pc->getAAT("flags");
	PointSet* ps = pc->getPointSet();

	for (i = 0; i < n; i++)
	{
		ps->set1i(i, flags, 0);
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::initGeodesics()
{
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::updateGeodesics(mpcard numNewRings/* = 2*/)
{
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::breadthFirstLocalPatch(UnstructuredInCorePointCloud* pc, PointCloudIterator* pc_it, vector<mpcard>& indices, mpcard ringsToAdd, int32 flagMask, FloatType* pAvgEdgeLength /*= NULL*/)
{
//	Timer timer;

	mpcard i, j, k, d;
	mpcard istart, inbr, numnbrs;
	vector<mpcard> depth;
	TopologyIterator* top_it = NULL;
	RandomAccessPointCloudIterator* rand_it = NULL;
	AAT flagsAAT, posAAT;
	int32 flags;
	FloatType avgEdgeLength = 0;
	mpcard numEdges = 0;
	VectorType vcur, vnbr;

	PointSet* ps = pc->getPointSet();

	top_it = dynamic_cast<TopologyIterator*>(pc_it);
	rand_it = dynamic_cast<RandomAccessPointCloudIterator*>(pc_it);

	flagsAAT = pc->getAAT("flags");
	posAAT = pc->getAAT("position");

	for (k = 0; k < indices.size(); k++)
	{
		i = indices[k];
		flags = ps->get1i(i, flagsAAT);
		flags |= flagMask;
		ps->set1i(i, flagsAAT, flags);
	}

	d = 0;
	for (i = 0; i < indices.size(); i++)
		depth.push_back(d);

	k = 0;
	while (k < indices.size())
	{
		i = indices[k];
		d = depth[k];

		if (d < ringsToAdd)
		{
			rand_it->goToIndex(i);
			numnbrs = top_it->getNumNeighbours();
			d++;

			vcur = ps->get3f(i, posAAT);

			for (j = 0; j < numnbrs; j++)
			{
				inbr = top_it->getNeighbour(j);
				flags = ps->get1i(inbr, flagsAAT);
				if ((flags & flagMask) == 0)
				{
					vnbr = ps->get3f(inbr, posAAT);
					avgEdgeLength += (vnbr - vcur).getNorm();
					numEdges++;
					flags |= flagMask;
					indices.push_back(inbr);
					depth.push_back(d);
					ps->set1i(inbr, flagsAAT, flags);
				}
			}
		}

		k++;
	}

	if (numEdges > 0)
		avgEdgeLength /= (FloatType)numEdges;

	if (pAvgEdgeLength != NULL)
		*pAvgEdgeLength = avgEdgeLength;
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::breadthFirstLocalPatch(UnstructuredInCorePointCloud* pc, PointCloudIterator* pc_it, const vector<mpcard>& indicesStart, mpcard ringsToAdd, int32 flagMask, vector<mpcard>& indicesNew, bool bResetFlagsAtEnd/* = false*/, FloatType* pAvgEdgeLength/* = NULL*/)
{


	mpcard i, j, k, d;
	mpcard istart, inbr, numnbrs;
	vector<mpcard> depth;
	TopologyIterator* top_it = NULL;
	RandomAccessPointCloudIterator* rand_it = NULL;
	AAT flagsAAT, posAAT;
	int32 flags;
	FloatType avgEdgeLength = 0;
	mpcard numEdges = 0;
	VectorType vcur, vnbr;

	int32 flagInvert = ~flagMask;

	PointSet* ps = pc->getPointSet();

	top_it = dynamic_cast<TopologyIterator*>(pc_it);
	rand_it = dynamic_cast<RandomAccessPointCloudIterator*>(pc_it);

	flagsAAT = pc->getAAT("flags");
	posAAT = pc->getAAT("position");

	for (k = 0; k < indicesStart.size(); k++)
	{
		i = indicesStart[k];
		flags = ps->get1i(i, flagsAAT);
		flags |= flagMask;
		ps->set1i(i, flagsAAT, flags);
	}

	d = 0;
	for (i = 0; i < indicesStart.size(); i++)
		depth.push_back(d);

	indicesNew.clear();

	k = 0;
	while (k < depth.size())
	{
		if (k < indicesStart.size())
			i = indicesStart[k];
		else
			i = indicesNew[k - indicesStart.size()];
		d = depth[k];

		if (d < ringsToAdd)
		{
			rand_it->goToIndex(i);
			numnbrs = top_it->getNumNeighbours();
			d++;

			vcur = ps->get3f(i, posAAT);

			for (j = 0; j < numnbrs; j++)
			{
				inbr = top_it->getNeighbour(j);
				flags = ps->get1i(inbr, flagsAAT);
				if ((flags & flagMask) == 0)
				{
					vnbr = ps->get3f(inbr, posAAT);
					avgEdgeLength += (vnbr - vcur).getNorm();
					numEdges++;
					flags |= flagMask;
					indicesNew.push_back(inbr);
					depth.push_back(d);
					ps->set1i(inbr, flagsAAT, flags);
				}
			}
		}

		k++;
	}

	if (numEdges > 0)
		avgEdgeLength /= (FloatType)numEdges;

	if (bResetFlagsAtEnd)
	{
		for (k = 0; k < indicesStart.size(); k++)
		{
			flags = ps->get1i(indicesStart[k], flagsAAT);
			flags &= flagInvert;
			ps->set1i(indicesStart[k], flagsAAT, flags);
		}
		for (k = 0; k < indicesNew.size(); k++)
		{
			flags = ps->get1i(indicesNew[k], flagsAAT);
			flags &= flagInvert;
			ps->set1i(indicesNew[k], flagsAAT, flags);
		}
	}

	if (pAvgEdgeLength != NULL)
		*pAvgEdgeLength = avgEdgeLength;

}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::precomputeSourceGeodesics(mpcard nTopologies, string strAttachBaseName)
{
	Timer timer;

	mpcard i, j, k, l, m, n;
	mpcard destInd;
	TopologyAttachment** ppTopoAtt = NULL;
	InCorePCTopologyGraph** ppGraph = NULL;
	PointSet* pEdgeSet = NULL;

	AR4MLSManifold<ManifoldTraits>* pmlsSource = dynamic_cast<AR4MLSManifold<ManifoldTraits>*>(source);
	AR4MLSManifold<ManifoldTraits>* pmlsTarget = dynamic_cast<AR4MLSManifold<ManifoldTraits>*>(target);
	AR4MLSManifold3f* pmls3Source = dynamic_cast<AR4MLSManifold3f*>(pmlsSource);
	AR4MLSManifold3f* pmls3Target = dynamic_cast<AR4MLSManifold3f*>(pmlsTarget);

	if (pmlsSource == NULL)
		throw X4::EInvalidState("AR4PartialMatch<ManifoldTraits>::initGeodesics(): source is not an MLSManifold.");
	if (pmlsTarget == NULL)
		throw X4::EInvalidState("AR4PartialMatch<ManifoldTraits>::initGeodesics(): target is not an MLSManifold.");

	UnstructuredInCorePointCloud*	upcSource = pmlsSource->getPointCloud();
	UnstructuredInCorePointCloud*	upcTarget = pmlsTarget->getPointCloud();

	PointSet*						psSource = upcSource->getPointSet();
	AAT								pos = upcSource->getAAT("position");

	string strLevelName;

	vector<mpcard> destIndices;
	set<mpcard> destSet;
	set<mpcard>::iterator destIt;
	
	InCorePCTopologyGraph::EdgeIndex ei;
	AAT weightAAT;
	AAT geoidxAAT;

	Vector3f vertpos;

	float dist;
	Vector3f grad;

	vector<float> dists;
	vector<Vector3f> grads;
	vector<vector<Vector3f>> paths;

	ppTopoAtt = new TopologyAttachment*[nTopologies];
	ppGraph = new InCorePCTopologyGraph*[nTopologies];

	m_nSampleLevels = nTopologies + 1;
	m_ppTopoGraphSource = new InCorePCTopologyGraph*[m_nSampleLevels];
	m_ppIterSource = new PointCloudIterator*[m_nSampleLevels];
	m_ppTopoGraphSource[0] = pmlsSource->getTopology();
	m_ppIterSource[0] = m_ppTopoGraphSource[0]->createIterator(pmlsSource->getPointSet());

	m_pHknnSource = new HierarchicalKNNIterator(upcSource, 32, NULL);
	m_pHknnTarget = new HierarchicalKNNIterator(upcTarget, 32, NULL);

	m_geodSigma = pmlsSource->getAvgEdgeLength();

	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMLocal:
		m_pLocalSourceGeodesics->setSourceIndexString(m_strIndex);
		m_pLocalSourceGeodesics->setSourcePC(upcSource);
		break;
	case eAR4PMGMSparse:
		m_pSparseSourceGeodesics->setManifold(pmls3Source);
		m_pSparseTargetGeodesics->setManifold(pmls3Target);
		break;
	}
		
	for (i = 0; i < nTopologies; i++)
	{
		strLevelName = strAttachBaseName + "_s_" + intToStr(i);

		ppTopoAtt[i] = dynamic_cast<TopologyAttachment*>(upcSource->getAttachments()->getData(strLevelName));
		if (ppTopoAtt[i] == NULL)
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::precomputeSourceGeodesics(...): did not find sub-sampled topology attachment: " + strLevelName);

		ppGraph[i] = ppTopoAtt[i]->getTopology();

		m_ppTopoGraphSource[i + 1] = ppGraph[i];
		m_ppIterSource[i + 1] = m_ppTopoGraphSource[i + 1]->createIterator(pmlsSource->getPointSet());
	}

	vector<mpcard> numEdges(m_nSampleLevels, 0);

	n = upcSource->getNumPoints();

	mpcard numSourcesComputed = 0;
	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMLocal:
		for (i = 0; i < n; i++)
		{
			destIndices.clear();
			destSet.clear();

			if (ppGraph[0]->getNumAdjacentVertices(i) == 0)
				continue;
			numSourcesComputed++;

			for (j = 0; j < nTopologies; j++)
			{
				pEdgeSet = ppGraph[j]->getEdges();
				weightAAT = pEdgeSet->getAAT("weight");

				m = ppGraph[j]->getNumAdjacentVertices(i);
				for (l = 0; l < m; l++)
				{
					k = ppGraph[j]->getAdjacentVertex(i, l, &ei);
						destSet.insert(k);
				}
			}

			for (destIt = destSet.begin(); destIt != destSet.end(); destIt++)
				destIndices.push_back(*destIt);

			if (destIndices.size() > 0)
				m_pLocalSourceGeodesics->cacheGeodesics(i, destIndices);

			for (j = 0; j < nTopologies; j++)
			{
				pEdgeSet = ppGraph[j]->getEdges();
				weightAAT = pEdgeSet->getAAT("weight");

				m = ppGraph[j]->getNumAdjacentVertices(i);
				for (l = 0; l < m; l++)
				{
					k = ppGraph[j]->getAdjacentVertex(i, l, &ei);
					
					m_pLocalSourceGeodesics->getCachedGeodesic(i, k, dist, grad);
					pEdgeSet->set1f(ei, weightAAT, dist);
					
				}
			}
		}
		break;
	case eAR4PMGMSparse:
		m_topoSampleSpacings.resize(m_nSampleLevels);
		m_topoSampleSpacings[0] = computeAvgSourceEdgeLength();
		for (j = 1; j < m_nSampleLevels; j++)
			m_topoSampleSpacings[j] = 0.f;

		//in case we are using path relaxation
		m_pSparseSourceGeodesics->setRelaxationSoftConstraint(0.01f);

		for (i = 0; i < n; i++)
		{
			destIndices.clear();
			destSet.clear();

			if (ppGraph[0]->getNumAdjacentVertices(i) == 0)
				continue;
			numSourcesComputed++;

			for (j = 0; j < nTopologies; j++)
			{
				vertpos = psSource->get3f(i, pos);

				pEdgeSet = ppGraph[j]->getEdges();
				weightAAT = pEdgeSet->getAAT("weight");

				m = ppGraph[j]->getNumAdjacentVertices(i);
				for (l = 0; l < m; l++)
				{
					k = ppGraph[j]->getAdjacentVertex(i, l, &ei);
					destSet.insert(k);

					m_topoSampleSpacings[j + 1] += (psSource->get3f(k, pos) - vertpos).getNorm();
					numEdges[j + 1]++;
				}
			}

			for (destIt = destSet.begin(); destIt != destSet.end(); destIt++)
				destIndices.push_back(*destIt);

			if (destIndices.size() > 0)
			{
				m_pSparseSourceGeodesics->computeGeodesicsEarlyStop(m_geodSigma, i, destIndices, dists, grads, &paths);
				vector<Vector3f> dirsAtSource;
				if (m_bUsePathRelaxation)
					m_pSparseSourceGeodesics->relaxPaths(m_nRelaxIters, paths, dists, grads, dirsAtSource);
				m_pSparseSourceGeodesics->storeGeodesicData(i, destIndices, dists, grads, &paths);
			}

			for (j = 0; j < nTopologies; j++)
			{
				pEdgeSet = ppGraph[j]->getEdges();
				weightAAT = pEdgeSet->getAAT("weight");

				m = ppGraph[j]->getNumAdjacentVertices(i);
				for (l = 0; l < m; l++)
				{
					k = ppGraph[j]->getAdjacentVertex(i, l, &ei);
					
					Vector3f dirsAtSource;
					destInd = m_pSparseSourceGeodesics->getGeodesicData(i, k, dist, grad, dirsAtSource);
					pEdgeSet->set1f(ei, weightAAT, dist);
					
				}
			}
		}

		debugOutput << "AR4PartialMatch::precomputeSourceGeodesics(...) - number of edges at topology level:\n";
		for (j = 1; j < m_nSampleLevels; j++)
		{
			m_topoSampleSpacings[j] /= (float)numEdges[j];
			debugOutput << "\t" << j << " " << numEdges[j] << "\n";
		}
		break;
	}

	m_nSourcePointsMatchingLevel = numSourcesComputed;
	debugOutput << "numSourcesComputed = " << numSourcesComputed << "\n";

	delete [] ppTopoAtt;
	delete [] ppGraph;

	m_bSourceGeodesicsPrecomputed = true;

	debugOutput << "AR4PartialMatch<ManifoldTraits>::precomputeSourceGeodesics(...) - time: " << convertTimeToString(timer.getDeltaValue()) << "\n";
}

template <class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::computeCorrespondenceError()
{
	mpcard i, j, k;
	FloatType cur, ref, diff;
	FloatType error = 0;
	VectorType vsrc1, vsrc2, vdst1, vdst2;

	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];
		vsrc1 = source->getDiscrPoint(k);
		vdst1 = matches[k].targetPos;

		for (j = 0; j < matchedSourceIndices.size(); j++)
		{
			if (j == i)
				continue;

			k = matchedSourceIndices[j];
			vsrc2 = source->getDiscrPoint(k);
			vdst2 = matches[k].targetPos;

			ref = sourceGeodesic(vsrc1, vsrc2);
			cur = targetGeodesic(vdst1, vdst2);

			if (cur < 1e10 && ref < 1e10)
			{
				diff = cur - ref;
				error += diff*diff;
			}
		}
	}

	return error;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::computeCorrespondenceError(const std::pair<Vector3f, Vector3f>& corr)
{
	mpcard i, k;
	FloatType cur, ref, diff;
	FloatType error = 0;
	VectorType vsrc1, vdst1;

	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];

		if (m_eGeodesicsMode == eAR4PMGMLocal && !m_pLocalSourceGeodesics->isCached(k))
			continue;

		vsrc1 = source->getDiscrPoint(k);
		vdst1 = matches[k].targetPos;

		ref = sourceGeodesic(vsrc1, corr.first);
		cur = targetGeodesic(vdst1, corr.second);

		if (cur < 1e10 && ref < 1e10)
		{
			diff = cur - ref;
			error += diff*diff;
		}
	}

	return error;
}

//concrete class
void AR4PartialMatch3f::initGeodesics()
{
	AR4PartialMatch<ManifoldTraits3f>::initGeodesics();
}

void AR4PartialMatch3f::updateGeodesics(X4::mpcard numNewRings/* = 2*/)
{
	AR4PartialMatch<ManifoldTraits3f>::updateGeodesics(numNewRings);
}

void AR4PartialMatch3f::breadthFirstLocalPatch(UnstructuredInCorePointCloud* pc, PointCloudIterator* pc_it, vector<mpcard>& indices, mpcard ringsToAdd, int32 flagMask, FloatType* pAvgEdgeLength /*= NULL*/)
{
	AR4PartialMatch<ManifoldTraits3f>::breadthFirstLocalPatch(pc, pc_it, indices, ringsToAdd, flagMask, pAvgEdgeLength);
}

void AR4PartialMatch3f::breadthFirstLocalPatch(UnstructuredInCorePointCloud* pc, PointCloudIterator* pc_it, const vector<mpcard>& indicesStart, mpcard ringsToAdd, int32 flagMask, vector<mpcard>& indicesNew, bool bResetFlagsAtEnd/* = false*/, FloatType* pAvgEdgeLength /*= NULL*/)
{
	AR4PartialMatch<ManifoldTraits3f>::breadthFirstLocalPatch(pc, pc_it, indicesStart, ringsToAdd, flagMask, indicesNew, bResetFlagsAtEnd, pAvgEdgeLength);
}

ManifoldTraits3f::FloatType AR4PartialMatch3f::computeCorrespondenceError()
{
	return AR4PartialMatch<ManifoldTraits3f>::computeCorrespondenceError();
}

ManifoldTraits3f::FloatType AR4PartialMatch3f::computeCorrespondenceError(const std::pair<Vector3f, Vector3f>& corr)
{
	return AR4PartialMatch<ManifoldTraits3f>::computeCorrespondenceError(corr);
}

void AR4PartialMatch3f::precomputeSourceGeodesics(mpcard nTopologies, string strAttachBaseName)
{
	AR4PartialMatch<ManifoldTraits3f>::precomputeSourceGeodesics(nTopologies, strAttachBaseName);
}

void AR4PartialMatch3f::copyGeodesicAndTopologyData(AR4PartialMatch3f* pOtherMatch)
{
	mpcard i;

	m_pSparseSourceGeodesics = pOtherMatch->m_pSparseSourceGeodesics;
	m_pSparseTargetGeodesics = pOtherMatch->m_pSparseTargetGeodesics;

	m_geodSigma = pOtherMatch->m_geodSigma;

	m_topoSampleSpacings = pOtherMatch->m_topoSampleSpacings;

	m_bUsePathRelaxation = pOtherMatch->m_bUsePathRelaxation;

	AR4MLSManifold<ManifoldTraits3f>* pmlsSource = dynamic_cast<AR4MLSManifold<ManifoldTraits3f>*>(source);
	AR4MLSManifold<ManifoldTraits3f>* pmlsTarget = dynamic_cast<AR4MLSManifold<ManifoldTraits3f>*>(target);

	if (pmlsSource == NULL)
		throw X4::EInvalidState("AR4PartialMatch<ManifoldTraits>::copyGeodesicAndTopologyData(): source is not an MLSManifold.");
	if (pmlsTarget == NULL)
		throw X4::EInvalidState("AR4PartialMatch<ManifoldTraits>::copyGeodesicAndTopologyData(): target is not an MLSManifold.");

	m_nSampleLevels = pOtherMatch->m_nSampleLevels;

	m_ppTopoGraphSource = new InCorePCTopologyGraph*[m_nSampleLevels];
	m_ppIterSource = new PointCloudIterator*[m_nSampleLevels];

	for (i = 0; i < m_nSampleLevels; i++)
	{
		m_ppTopoGraphSource[i] = pOtherMatch->m_ppTopoGraphSource[i];
		m_ppIterSource[i] = m_ppTopoGraphSource[i]->createIterator(pmlsSource->getPointSet());
	}

	m_bOwnsGeodesicsAndTopology = false;
	
}

void AR4PartialMatch3f::initThroughPointCloud(UnstructuredInCorePointCloud* pc)
{
	AR4PartialMatch<ManifoldTraits3f>::initThroughPointCloud(pc);
}

void AR4PartialMatch3f::interpolateFullRes(const card32 nIterations)
{
	const mpcard numSourcePoints = source->getNumDiscrPoints();

	mpcard i, j, k;
	card32 iter;

	vector<mpcard> indicesNew;
	vector<Vector3f> targetPointsNew;

	AR4MLSManifold3f* pTargetMLS = dynamic_cast<AR4MLSManifold3f*>(target);

	const mpcard numTargPts = pTargetMLS->getNumDiscrPoints();

	vector<float> currentDistTrans(numTargPts);
	vector<float> sumDistTrans(numTargPts, 0.f);

	for (iter = 0; iter < nIterations; iter++)
	{
		for (i = 0; i < numSourcePoints; i++)
		{
			if (!matches[i].matched)
			{
				vector<mpcard> neighbors;
				getSourceNeighborhood(i, 2, neighbors);

				vector<mpcard> sourceRefs;
				vector<float> sourceDists;
				vector<Vector3f> sourceGrads;
				vector<Vector3f> targetRefs;

				const mpcard numNbrs = neighbors.size();
				for (j = 0; j < numNbrs; j++)
				{
					const mpcard nbr = neighbors[j];
					if (matches[nbr].matched)
					{
						sourceRefs.push_back(nbr);
						targetRefs.push_back(getMatch(nbr));
					}
				}

				const mpcard numRefs = sourceRefs.size();
				if (numRefs < 3)
					continue;

				m_pSparseSourceGeodesics->computeGeodesicsEarlyStop(m_geodSigma * 0.1f, i, sourceRefs, sourceDists, sourceGrads);

				fill(sumDistTrans.begin(), sumDistTrans.end(), 0.f);

				float distMaxSum = 0.f;

				vector<Vector3f> hintSet;
				vector<Vector3f> zeroSet(1);
				for (j = 0; j < numRefs; j++)
				{
					const float distRef = sourceDists[j];
					const float distTrunc = distRef * 3.f;
					distMaxSum += distTrunc;

					zeroSet[0] = targetRefs[j];
					m_pSparseTargetGeodesics->distanceTransform(zeroSet, distTrunc);
					m_pSparseTargetGeodesics->copyDistanceField(currentDistTrans);
					hintSet.push_back(targetRefs[j]);

					for (k = 0; k < numTargPts; k++)
						sumDistTrans[k] += fabs(currentDistTrans[k] - distRef);
				}

				Vector3f distSumMinPoint;
				float distSumMin;
				pTargetMLS->findScalarFieldMinimum(sumDistTrans, hintSet, distSumMinPoint, distSumMin);

				if (distSumMin < distMaxSum && (distSumMin / (float)numRefs) < m_stretchThresh)
				{
					indicesNew.push_back(i);
					targetPointsNew.push_back(distSumMinPoint);
				}
			}
		}

		const mpcard numNew = indicesNew.size();
		if (numNew == 0)
			break;

		for (i = 0; i < numNew; i++)
		{
			setMatch(indicesNew[i], targetPointsNew[i]);
		}

		checkForNewGeodesicConnections();
		validatePaths();
		removeOutliersLastCohort();
		finalizeMatchCohort();
	}
}


