#ifndef AR4PartialMatchH
#define AR4PartialMatchH

#include "threeParameterMatchingGMOD14.h"

#include "AR4DiscreteManifold.h"
#include "AR4ManifoldTraits.h"
#include "AR4SmoothManifold.h"
#include "ApproximatedGeodesicsGM14.h"
#include "AR4CachedLocalGeodesics.h"
#include "AR4SparseGeodesics.h"
#include "PointSetKNNQuery.h"

#include "AR4Profile.h"


namespace X4 {


template <class ManifoldTraits>
class AR4PartialMatch {
 public:
   typedef typename ManifoldTraits::FloatType  FloatType;
   typedef typename ManifoldTraits::VectorType VectorType;
   typedef typename ManifoldTraits AR4DiscreteManifoldTraits;
   typedef typename ManifoldTraits::TangentSpace TangentSpace;
   typedef typename ManifoldTraits::BoundingBoxType BoundingBoxType;

   enum EAR4PartialMatchGeodesicsMode
   {
	   eAR4PMGMLocal = 0,
	   eAR4PMGMGlobal,
	   eAR4PMGMSparse,
	   eAR4PMGMNumModes
   };

   const static mpcard					m_PatchVisitMask = 1;
   const static mpcard					m_InvalidIndex = 0xFFFFFFFF;

protected:
   AR4DiscreteManifold<ManifoldTraits> *source;
   AR4SmoothManifold<ManifoldTraits>   *target;

   atrec::ApproximatedGeodesicsGM14*		m_pSourceGeodesics;
   atrec::ApproximatedGeodesicsGM14*		m_pTargetGeodesics;
   bool									m_bGeodesicsDirty;
   FloatType							m_geodSigma;

   AR4CachedLocalGeodesics*				m_pLocalSourceGeodesics;
   AR4CachedLocalGeodesics*				m_pLocalTargetGeodesics;

   AR4SparseGeodesics*					m_pSparseSourceGeodesics;
   AR4SparseGeodesics*					m_pSparseTargetGeodesics;

   EAR4PartialMatchGeodesicsMode		m_eGeodesicsMode;

   mpcard								m_patchRings;
   vector<mpcard>						m_sourcePatchIndices;
   vector<mpcard>						m_targetPatchIndices;

   vector<mpcard>						m_matchCohortCounts;

   HierarchicalKNNIterator*				m_pHknnSource;
   HierarchicalKNNIterator*				m_pHknnTarget;

   mpcard								m_nSampleLevels;
   InCorePCTopologyGraph**				m_ppTopoGraphSource;
   InCorePCTopologyGraph**				m_ppTopoGraphTarget;
   PointCloudIterator**					m_ppIterSource;
   PointCloudIterator**					m_ppIterTarget;
   TopologicalKNNIterator**				m_ppTopoknnSource;
   TopologicalKNNIterator**				m_ppTopoknnTarget;
   mpcard								m_nGeodesicsNN;
   string								m_strIndex;

   mpcard								m_nSourcePointsMatchingLevel;

   UnstructuredInCorePointCloud*		m_pTargetMatchCloud;
   HierarchicalKNNIterator*				m_pHknnTargetMatch;
   VertexDescriptor						m_vdMatch;
   string								m_strSourceIndexName;

   UnstructuredInCorePointCloud*		m_pSourceMatchCloud;
   HierarchicalKNNIterator*				m_pHknnSourceMatch;

   PointSetKNNQuery*					m_pKNNMatchSource;
   PointSetKNNQuery*					m_pKNNMatchTarget;
   AttachedIndexedOctree*				m_pOctreeMatchSource;
   AttachedIndexedOctree*				m_pOctreeMatchTarget;

   UnstructuredInCorePointCloud*		m_pInitializerPointCloud;

   bool									m_bSourceGeodesicsPrecomputed;

   vector<FloatType>					m_topoSampleSpacings;

   FloatType							m_stretchThresh;
   FloatType							m_avgEdgeWeight;

   bool									m_bUsePathRelaxation;
   mpcard								m_nRelaxIters;

   bool									m_bIsFullRes;

   bool									m_bMimick;
   bool									m_bOwnsGeodesicsAndTopology;

   AR4Profile							m_profStretchMeasure;
   AR4Profile							m_profValidateSource;
   AR4Profile							m_profValidateTarget;
   AR4Profile							m_profFullValidate;
   AR4Profile							m_profRemoveOutliers;
   AR4Profile							m_profWalkIsometry;

   VectorType							m_startSourceDir;
   VectorType							m_startTargetDir;

   struct Match {
      VectorType targetPos;
      bool matched;
	  bool fixed;
	  mpcard matchOrder;
	  mpcard backupLookup;
	  FloatType stretch;
	  FloatType sourceDistToStart;
	  Match()
	  {
		  matched=false;
		  fixed=false;
		  targetPos=NULL_VECTOR3F;
		  matchOrder=m_InvalidIndex;
		  backupLookup=m_InvalidIndex;
		  stretch=(FloatType)-1;
		  sourceDistToStart=AR4SparseGeodesics::m_HugeDistance;
	  }
	  Match& operator=(const Match& m)
	  {
		  matched = m.matched;
		  fixed = m.fixed;
		  matchOrder = m.matchOrder;
		  backupLookup = m.backupLookup;
		  targetPos = m.targetPos;
		  stretch = m.stretch;
		  return *this;
	  }
   };
   vector<Match> matches;
   vector<mpcard> matchedSourceIndices;

   vector<VectorType> matchPosBackup;
   vector<bool> matchPosDirty;

   set<mpcard> matchesForGeodesicUpdate;

   class RejectedOutlier
   {
   public:
	   mpcard			m_sourceIndex;
	   VectorType		m_targetPos;
	   float			m_stretch;
	   set<mpcard>		m_rejectors;
	   RejectedOutlier()
	   {
		   m_sourceIndex = m_InvalidIndex;
		   m_targetPos = NULL_VECTOR3F;
		   m_stretch = -1.f;
	   }
	   inline RejectedOutlier& operator=(const RejectedOutlier& ro)
	   {
		   m_sourceIndex = ro.m_sourceIndex;
		   m_targetPos = ro.m_targetPos;
		   m_stretch = ro.m_stretch;
		   m_rejectors = ro.m_rejectors;
		   return *this;
	   }
	   inline bool operator<(const RejectedOutlier& ro) const
	   {
		   return (m_sourceIndex < ro.m_sourceIndex);
	   }
   };

   set<RejectedOutlier>								m_rejectedOutliers;
   bool												m_bRecordRejectedOutliers;

	inline void dirtyMatchBackup(mpcard sourceIndex)
	{
		mpcard ibu;
		if ((ibu = matches[sourceIndex].backupLookup) != m_InvalidIndex)
			matchPosDirty[ibu] = true;
	}
	inline void backupMatch(mpcard sourceIndex)
	{
		mpcard ibu;

		if ((ibu = matches[sourceIndex].backupLookup) != m_InvalidIndex && matchPosDirty[ibu])
		{
			matchPosBackup[ibu] = matches[sourceIndex].targetPos;
			matchPosDirty[ibu] = false;
		}
		else
		{
			ibu = matchPosBackup.size();
			matchPosBackup.push_back(matches[sourceIndex].targetPos);
			matchPosDirty.push_back(false);
			matches[sourceIndex].backupLookup = ibu;
		}
	}
	inline void rollbackMatch(mpcard sourceIndex)
	{
		mpcard ibu;

		if ((ibu = matches[sourceIndex].backupLookup) != m_InvalidIndex && matchPosDirty[ibu])
		{
			matches[sourceIndex].targetPos = matchPosBackup[ibu];
			matchPosDirty[ibu] = false;
		}
	}

   class PairFirstCompare
   {
   public:
	   inline bool operator()(const pair<mpcard, mpcard>& p1, const pair<mpcard, mpcard>& p2) const
	   {
		   return (p1.first < p2.first);
	   }
   };

   vector<set<pair<mpcard, mpcard>, PairFirstCompare>>	m_geodesicMatches;

   void addGeodesicMatch(mpcard sourceStart, mpcard sourceEnd, mpcard targGeo)
   {
	   pair<mpcard, mpcard> ins;
	   if (sourceStart > m_geodesicMatches.size())
		   throw ERangeCheck("AR4PartialMatch::addGeodesicMatch(...): sourceStart out of range");
	   ins.first = sourceEnd;
	   ins.second = targGeo;
	   m_geodesicMatches[sourceStart].insert(ins);
   }

   mpcard lookupGeodesicMatch(mpcard sourceStart, mpcard sourceEnd)
   {
		pair<mpcard, mpcard> temp;
		set<pair<mpcard, mpcard>, PairFirstCompare>::iterator resIt;
		temp.first = sourceEnd;
		resIt = m_geodesicMatches[sourceStart].find(temp);
		if (resIt == m_geodesicMatches[sourceStart].end())
			return AR4SparseGeodesics::m_InvalidIndex;
		return (*resIt).second;
   }

#define AR4PM_MATCH_NEAREST_HKNNIT			0

   void forceResetTargetMatchKNNQuery()
   {
	   if (m_pTargetMatchCloud != NULL)
	   {
#if AR4PM_MATCH_NEAREST_HKNNIT
		   if (m_pHknnTargetMatch == NULL)
			   m_pHknnTargetMatch = new HierarchicalKNNIterator(m_pTargetMatchCloud, 32, NULL);

		   PointCloudIterator* pcIt = m_pTargetMatchCloud->createIterator(SOIT(SOIT::CAP_HIERARCHICAL));
		   SecondaryDataStructureIterator* sdsIt = dynamic_cast<SecondaryDataStructureIterator*>(pcIt);
		   if (sdsIt == NULL)
			   throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::forceResetTargetMatchKNNQuery(): cannot get SecondaryDataStructureIterator");
		   AttachedIndexedOctree* poctree = dynamic_cast<AttachedIndexedOctree*>(sdsIt->lockAttachments()->getData("SimpleOctree"));
		   if (poctree != NULL)
		   {
			   poctree->octreesetup(m_pTargetMatchCloud->getPointSet(), 32);
		   }
		   sdsIt->unlockAttachments();
		   delete pcIt;
#else
		   delete m_pKNNMatchTarget;
		   delete m_pOctreeMatchTarget;
		   m_pOctreeMatchTarget = PointSetKNNQuery::createOctree(m_pTargetMatchCloud->getPointSet());
		   m_pKNNMatchTarget = new PointSetKNNQuery(m_pTargetMatchCloud->getPointSet(), m_pOctreeMatchTarget, NULL_VECTOR3F, m_pTargetMatchCloud->getAAT("position"));
#endif //AR4PM_MATCH_NEAREST_HKNNIT
	   }
   }
   void forceResetSourceMatchKNNQuery()
   {
	   if (m_pSourceMatchCloud != NULL)
	   {
#if AR4PM_MATCH_NEAREST_HKNNIT
		   if (m_pHknnSourceMatch == NULL)
			   m_pHknnSourceMatch = new HierarchicalKNNIterator(m_pSourceMatchCloud, 32, NULL);

		   PointCloudIterator* pcIt = m_pSourceMatchCloud->createIterator(SOIT(SOIT::CAP_HIERARCHICAL));
		   SecondaryDataStructureIterator* sdsIt = dynamic_cast<SecondaryDataStructureIterator*>(pcIt);
		   if (sdsIt == NULL)
			   throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::forceResetSourceMatchKNNQuery(): cannot get SecondaryDataStructureIterator");
		   AttachedIndexedOctree* poctree = dynamic_cast<AttachedIndexedOctree*>(sdsIt->lockAttachments()->getData("SimpleOctree"));
		   if (poctree != NULL)
		   {
			   poctree->octreesetup(m_pSourceMatchCloud->getPointSet(), 32);
//			   poctree->insertPointIndex(m_pSourceMatchCloud->getNumPoints() - 1, 32);
		   }
		   sdsIt->unlockAttachments();
		   delete pcIt;
#else
		   delete m_pKNNMatchSource;
		   delete m_pOctreeMatchSource;
		   m_pOctreeMatchSource = PointSetKNNQuery::createOctree(m_pSourceMatchCloud->getPointSet());
		   m_pKNNMatchSource = new PointSetKNNQuery(m_pSourceMatchCloud->getPointSet(), m_pOctreeMatchSource, NULL_VECTOR3F, m_pSourceMatchCloud->getAAT("position"));
#endif //AR4PM_MATCH_NEAREST_HKNNIT
	   }
   }

   VectorType getNearestSourceMatchPoint(const VectorType& v)
   {
#if AR4PM_MATCH_NEAREST_HKNNIT
		AAT pos = m_pSourceMatchCloud->getAAT("position");
		m_pHknnSourceMatch->setSeekPointAndReset(v);
		if (!m_pHknnSourceMatch->atEnd())
			return m_pHknnSourceMatch->get3f(pos);
		return NULL_VECTOR3F;
#else
		AAT pos = m_pSourceMatchCloud->getAAT("position");
		card32 index;
		bool bfound;
		m_pKNNMatchSource->setTargetPoint(v);
		m_pKNNMatchSource->getNextNearestPointIndex(index, bfound);
		if (bfound)
			return m_pSourceMatchCloud->getPointSet()->get3f(index, pos);
		return NULL_VECTOR3F;
#endif //AR4PM_MATCH_NEAREST_HKNNIT
   }
   VectorType getNearestTargetMatchPoint(const VectorType& v)
   {
#if AR4PM_MATCH_NEAREST_HKNNIT
		AAT pos = m_pTargetMatchCloud->getAAT("position");
		m_pHknnTargetMatch->setSeekPointAndReset(v);
		if (!m_pHknnTargetMatch->atEnd())
			return m_pHknnTargetMatch->get3f(pos);
		return NULL_VECTOR3F;
#else
		AAT pos = m_pSourceMatchCloud->getAAT("position");
		card32 index;
		bool bfound;
		m_pKNNMatchTarget->setTargetPoint(v);
		m_pKNNMatchTarget->getNextNearestPointIndex(index, bfound);
		if (bfound)
			return m_pTargetMatchCloud->getPointSet()->get3f(index, pos);
		return NULL_VECTOR3F;
#endif //AR4PM_MATCH_NEAREST_HKNNIT
   }

   card32 getNextNearestSourceMatchIndex(const VectorType& v)
   {
#if AR4PM_MATCH_NEAREST_HKNNIT
		card32 index;
		m_pHknnSourceMatch->setSeekPointAndReset(v);
		if (!m_pHknnSourceMatch->atEnd())
		{
			index = m_pHknnSourceMatch->getPointSetPointNumber();
			m_pHknnSourceMatch->next();
			return index;
		}
		return m_InvalidIndex;
#else
		card32 index;
		bool bfound;
		m_pKNNMatchSource->setTargetPoint(v);
		m_pKNNMatchSource->getNextNearestPointIndex(index, bfound);
		if (bfound)
			return index;
		return m_InvalidIndex;
#endif //AR4PM_MATCH_NEAREST_HKNNIT
   }
   card32 getNextNearestSourceMatchIndex()
   {
#if AR4PM_MATCH_NEAREST_HKNNIT
		card32 index;
		if (!m_pHknnSourceMatch->atEnd())
		{
			index = m_pHknnSourceMatch->getPointSetPointNumber();
			m_pHknnSourceMatch->next();
			return index;
		}
		return m_InvalidIndex;
#else
		card32 index;
		bool bfound;
		m_pKNNMatchSource->getNextNearestPointIndex(index, bfound);
		if (bfound)
			return index;
		return m_InvalidIndex;
#endif //AR4PM_MATCH_NEAREST_HKNNIT
   }

   card32 getNextNearestTargetMatchIndex(const VectorType& v)
   {
#if AR4PM_MATCH_NEAREST_HKNNIT
		card32 index;
		m_pHknnTargetMatch->setSeekPointAndReset(v);
		if (!m_pHknnTargetMatch->atEnd())
		{
			index = m_pHknnTargetMatch->getPointSetPointNumber();
			m_pHknnTargetMatch->next();
			return index;
		}
		return m_InvalidIndex;
#else
		card32 index;
		bool bfound;
		m_pKNNMatchTarget->setTargetPoint(v);
		m_pKNNMatchTarget->getNextNearestPointIndex(index, bfound);
		if (bfound)
			return index;
		return m_InvalidIndex;
#endif //AR4PM_MATCH_NEAREST_HKNNIT
   }
   card32 getNextNearestTargetMatchIndex()
   {
#if AR4PM_MATCH_NEAREST_HKNNIT
		card32 index;
		if (!m_pHknnTargetMatch->atEnd())
		{
			index = m_pHknnTargetMatch->getPointSetPointNumber();
			m_pHknnTargetMatch->next();
			return index;
		}
		return m_InvalidIndex;
#else
		card32 index;
		bool bfound;
		m_pKNNMatchTarget->getNextNearestPointIndex(index, bfound);
		if (bfound)
			return index;
		return m_InvalidIndex;
#endif //AR4PM_MATCH_NEAREST_HKNNIT
   }
 
 public:
	AR4PartialMatch(AR4DiscreteManifold<ManifoldTraits> *source, AR4SmoothManifold<ManifoldTraits> *target);
	~AR4PartialMatch();

	// --- access to source and destination surfaces
	AR4DiscreteManifold<ManifoldTraits>* getSourceManifold()	{ return source; }
	AR4SmoothManifold<ManifoldTraits>* getTargetManifold()		{ return target; }

	// --- access to source and destination geodesics
	AR4SparseGeodesics* getSparseSourceGeodesics()				{ return m_pSparseSourceGeodesics; }
	AR4SparseGeodesics* getSparseTargetGeodesics()				{ return m_pSparseTargetGeodesics; }

	EAR4PartialMatchGeodesicsMode getGeodesicsMode()			{ return m_eGeodesicsMode; }
	void setGeodesicsMode(EAR4PartialMatchGeodesicsMode eMode)	{ m_eGeodesicsMode = eMode; }

	bool areGeodesicsDirty()									{ return m_bGeodesicsDirty; }

	FloatType getGeodesicsSigma()								{ return m_geodSigma; }
	void setGeodesicsSigma(FloatType sigma)						{ m_geodSigma = sigma; }

	void setTopologyHierarchy(mpcard numLevels, InCorePCTopologyGraph** ppTopoSource, InCorePCTopologyGraph** ppTopoTarget);
	mpcard getNumTopologyLevels()								{ return m_nSampleLevels; }

	mpcard getNumSourcePointsMatchingLevel()					{ return m_nSourcePointsMatchingLevel; }

	void setIndexAttrString(string strIndex)					{ m_strIndex = strIndex; }
	string getIndexAttrString()									{ return m_strIndex; }

	void setStretchThreshold(FloatType thresh)					{ m_stretchThresh = thresh; }
	FloatType getStretchThreshold()								{ return m_stretchThresh; }

	void addTopologySampleSpacing(float spacing)				{ m_topoSampleSpacings.push_back(spacing); }

	void setUsePathRelaxation(bool bRelax)						{ m_bUsePathRelaxation = bRelax; }
	bool isPathRelaxationUsed()									{ return m_bUsePathRelaxation; }

	UnstructuredInCorePointCloud* getMatchingCloud()			{ return m_pTargetMatchCloud; }
	UnstructuredInCorePointCloud* getMatchingCloudOnSource()	{ return m_pSourceMatchCloud; }

	UnstructuredInCorePointCloud* getInitializerCloud()			{ return m_pInitializerPointCloud; }

	void setRecordRejectedOutliers(bool bRecord)				{ m_bRecordRejectedOutliers = bRecord; }
	bool areRejectedOutliersRecorded()							{ return m_bRecordRejectedOutliers; }

	mpcard getNumRejectedOutliers()								{ return m_rejectedOutliers.size(); }
	void getRejectInfo(mpcard sourceIndex, VectorType& targetPoint, FloatType& stretch, vector<mpcard>& sourceMeasurePoints);

	FloatType sourceGeodesic(const VectorType& from, const VectorType& to, VectorType& derivative);
	FloatType targetGeodesic(const VectorType& from, const VectorType& to, VectorType& derivative);
	FloatType sourceGeodesic(const VectorType& from, const VectorType& to);
	FloatType targetGeodesic(const VectorType& from, const VectorType& to);

	FloatType sourceGeodesic(const mpcard start, const mpcard end, VectorType& derivative);
	FloatType sourceGeodesic(const mpcard start, const mpcard end);
	FloatType sourceGeodesic(const mpcard start, const mpcard end, vector<VectorType>& path);
	FloatType targetGeodesic(const mpcard assocSourceStart, const mpcard assocSourceEnd, const VectorType& targetStart, const VectorType& targetEnd, VectorType& derivative);
	FloatType targetGeodesic(const mpcard assocSourceStart, const mpcard assocSourceEnd, const VectorType& targetStart, const VectorType& targetEnd, vector<VectorType>& path);
	FloatType targetGeodesic(const mpcard geodesicEntry, VectorType& derivative);

	FloatType sourceGeodesicDiameter();
	FloatType targetGeodesicDiameter();

	// --- validate geodesic paths: make sure they do not go outside the matched region
	bool validateSourcePath(const vector<mpcard>& path);
	bool validateSourcePath(const vector<Vector3f>& path);
	bool validateTargetPath(const vector<mpcard>& path);
	bool validateTargetPath(const vector<Vector3f>& path);

	// --- whether or not the match is computed on the full resolution of the point cloud underlying the manifold
	bool isFullResMatch()					{ return m_bIsFullRes; }
	void setFullResMatch(bool bFullRes)		{ m_bIsFullRes = bFullRes; }

	// --- return the "matching level" of the partial match
	mpcard getMatchingLevel()
	{
		if (m_nSampleLevels > 1 && !m_bIsFullRes)
			return 1;
		return 0;
	}

	// -- check if a point index has neighbors at the "matching" level of the topology hierarchy
	bool isAtMatchingLevel(const mpcard ptidx)
	{
		if (m_nSampleLevels < 1)
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::isAtMatchingLevel(...): no topology hierarchy");
		const mpcard mlevel = getMatchingLevel();
		return m_ppTopoGraphSource[mlevel]->getNumAdjacentVertices(ptidx) > 0;
	}
	bool isAtLevel(const mpcard sourceIndex, const mpcard level)
	{
		if (level >= m_nSampleLevels)
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::isAtLevel(...): invalid level");
		return (m_ppTopoGraphSource[level]->getNumAdjacentVertices(sourceIndex) > 0);
	}
	mpcard findNearestSourcePointAtMatchingLevel(const mpcard ptidx)
	{
		mpcard ptres;
		if (m_nSampleLevels < 1)
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::findNearestAtMatchingLevel(...): no topology hierarchy");
		if (m_pHknnSource == NULL)
			throw EInvalidState("AR4PartialMatch<ManifoldTraits>::findNearestAtMatchingLevel(...): k-NN iterator not initialized");

		m_pHknnSource->setSeekPointAndReset(source->getDiscrPoint(ptidx));
		if (m_nSampleLevels < 2)
		{
			while (!m_pHknnSource->atEnd())
			{
				ptres = m_pHknnSource->getPointSetPointNumber();
				if (m_ppTopoGraphSource[0]->getNumAdjacentVertices(ptres) > 0)
					return ptres;
				m_pHknnSource->next();
			}
			return AR4SparseGeodesics::m_InvalidIndex;
		}
		while (!m_pHknnSource->atEnd())
		{
			ptres = m_pHknnSource->getPointSetPointNumber();
			if (m_ppTopoGraphSource[1]->getNumAdjacentVertices(ptres) > 0)
				return ptres;
			m_pHknnSource->next();
		}
		return AR4SparseGeodesics::m_InvalidIndex;
	}
	mpcard findNearestSourcePointAtMatchingLevel(const VectorType& p)
	{
		mpcard ptres;
		if (m_nSampleLevels < 1)
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::findNearestAtMatchingLevel(...): no topology hierarchy");
		if (m_pHknnSource == NULL)
			throw EInvalidState("AR4PartialMatch<ManifoldTraits>::findNearestAtMatchingLevel(...): k-NN iterator not initialized");

		m_pHknnSource->setSeekPointAndReset(p);
		if (m_nSampleLevels < 2)
		{
			while (!m_pHknnSource->atEnd())
			{
				ptres = m_pHknnSource->getPointSetPointNumber();
				if (m_ppTopoGraphSource[0]->getNumAdjacentVertices(ptres) > 0)
					return ptres;
				m_pHknnSource->next();
			}
			return AR4SparseGeodesics::m_InvalidIndex;
		}
		while (!m_pHknnSource->atEnd())
		{
			ptres = m_pHknnSource->getPointSetPointNumber();
			if (m_ppTopoGraphSource[1]->getNumAdjacentVertices(ptres) > 0)
				return ptres;
			m_pHknnSource->next();
		}
		return AR4SparseGeodesics::m_InvalidIndex;
	}

	// --- find nearest point on target surface and return index
	mpcard findNearestTargetPoint(const VectorType& p)
	{
		if (m_pHknnTarget == NULL)
			throw EInvalidState("AR4PartialMatch<ManifoldTraits>::findNearestTargetPoint(...): NN queries on target surface not initialized");
		m_pHknnTarget->setSeekPointAndReset(p);
		return (mpcard)m_pHknnTarget->getPointSetPointNumber();
	}

	// --- find the nearest match on the target surface and return the point index of the source point
	mpcard findNearestMatchOnTarget(const VectorType& p)
	{
		if (m_pHknnTargetMatch == NULL)
			throw EInvalidState("AR4PartialMatch<ManifoldTraits>::findNearestMatchOnTarget(...): NN queries on correspondences not initialized");
		m_pHknnTargetMatch->setSeekPointAndReset(p);
		return matchedSourceIndices[m_pHknnTargetMatch->getPointSetPointNumber()];
	}

	mpcard getNumSourceNeighborsMatchingLevel(mpcard sourceIndex)
	{
		if (m_nSampleLevels < 1)
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getNumSourceNeighborsMatchingLevel(...): no topology hierarchy");
		const mpcard mlevel = getMatchingLevel();
		return m_ppTopoGraphSource[mlevel]->getNumAdjacentVertices(sourceIndex);
	}
	mpcard getSourceNeighborMatchingLevel(mpcard sourceIndex, mpcard nbrIndex)
	{
		if (m_nSampleLevels < 1)
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getSourceNeighborMatchingLevel(...): no topology hierarchy");
		const mpcard mlevel = getMatchingLevel();
		if (nbrIndex > m_ppTopoGraphSource[mlevel]->getNumAdjacentVertices(sourceIndex))
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getSourceNeighborMatchingLevel(...): invalid neighbor index");
		return m_ppTopoGraphSource[mlevel]->getAdjacentVertex(sourceIndex, nbrIndex);
	}

	void getSourceNeighborhood(const mpcard sourceIndex, const mpcard numRings, vector<mpcard>& neighbors);
	void getSourceMatchingLevelNeighborhood(mpcard sourceIndex, mpcard rings, vector<mpcard>& neighbors);
	void getSourceNeighborhood(const mpcard sourceIndex, const mpcard level, const mpcard numRings, vector<mpcard>& neighbors);

	bool walkIsometry(const mpcard sourceStart, const mpcard sourceEnd, const vector<VectorType>& sourcePath, const VectorType& targetPoint, const VectorType& targetDir, VectorType& targetEndPoint);
	void initMatchesFromPointAndDir(const mpcard sourceIndex, const VectorType& sourceDir, const VectorType& targetPoint, const VectorType& targetDir, mpcard numMatchesToInit = 1);

	VectorType getStartSourceDir() const		{ return m_startSourceDir; }
	VectorType getStartTargetDir() const		{ return m_startTargetDir; }

	// --- source indices that are matched (only matched points)
	mpcard getNumMatchedIndices() const;
	mpcard getMatchedSourceIndex(mpcard matchIndex) const;

	// --- access to match information by source index (global over all points)
	bool isMatched(mpcard sourceIndex) const;
	bool isFixed(mpcard sourceIndex) const;
	VectorType getMatch(mpcard sourceIndex) const;
	FloatType getStretch(mpcard sourceIndex) const;
	FloatType getSourceDistToStart(mpcard sourceIndex) const;

	void setMatch(mpcard sourceIndex, const VectorType &positionOnTarget);
	void setMatchLimitedUpdate(mpcard sourceIndex, const VectorType &positionOnTarget, const vector<mpcard>& sourceDests, const vector<mpcard>& targGeosAsStart, const vector<mpcard>& targGeosAsEnd);
	void setMatchLimitedUpdate(mpcard sourceIndex, const VectorType& positionOnTarget);
	void setMatchWithGeoData(mpcard sourceIndex, const VectorType &positionOnTarget, const vector<mpcard>& connectIndices, const vector<mpcard>& toTargGeos, const vector<mpcard>& fromTargGeos);

	// --- retreive the total stretch summed over all matched source indices
	FloatType getTotalStretch()
	{
		mpcard i, k;
		const mpcard numMatched = matchedSourceIndices.size();
		FloatType totalStretch = (FloatType)0;
		for (i = 0; i < numMatched; i++)
		{
			k = matchedSourceIndices[i];
			totalStretch += matches[k].stretch;
		}
		return totalStretch;
	}

	// --- update geodesic information on the target surface, where points may have moved
	void updateTargetGeodesics();
	void updateTargetGeodesicsLimited();

	// --- validate paths: determine whether or not they stay within the matched region
	void validatePaths();

	void checkForNewGeodesicConnections();

	void removeOutliersLastCohort();
	void resetMatchCloud();

	void setFixed(mpcard sourceIndex, bool bfixed);
	void removeMatch(mpcard sourceIndex);
	void removeLastMatch();
	void fixAllMatches();
	void unfixAllMatches();

	void backupNonFixedMatches();
	void backupNonFixedMatchesAndGeodesics();
	void backupMatches();
	void backupMatchesAndGeodesics();
	void rollbackMatches();
	void rollbackMatchesAndGeodesics();

	// --- mimick an existing match; this is useful when we want to create a temporary match to test new correspondences for geodesic error
	// --- avoids creating new geodesics and neighborhood data structures
	void mimick(AR4PartialMatch<ManifoldTraits>* pmatch);

	void getSourceNeighborhood(mpcard sourceIndex, vector<mpcard>& neighbors);
	void getSourceMatchedNeighborhood(mpcard sourceIndex, vector<mpcard>& Neighbors);
	void getSourceMatchedCachedNeighborhood(mpcard sourceIndex, vector<mpcard>& neighbors);
	void getSourceMatchedCachedNeighborhood(mpcard sourceIndex, vector<mpcard>& neighbors, vector<mpcard>& neighborsCorrIndex);
	void getSourceMatchedFixedCachedNeighborhood(mpcard sourceIndex, vector<mpcard>& neighbors);
	void getSourceMatchedFixedCachedNeighborhood(mpcard sourceIndex, mpcard level, mpcard numMaxNbrs, vector<mpcard>& neighbors);
	void getSourceRecentMatchedNeighborhood(mpcard sourceIndex, mpcard numCohortsBack, vector<mpcard>& Neighbors);

	void getSourceGeodesicsIndices(mpcard sourceStart, vector<mpcard>& sourceDests);
	void getMatchedGeodesics(mpcard sourceStart, vector<mpcard>& sourceDests, vector<float>& sourceDists, vector<float>& targetDists, vector<Vector3f>& targetGrads);
	void getValidMatchedGeodesics(mpcard sourceStart, vector<mpcard>& sourceDests, vector<float>& sourceDists, vector<float>& targetDists, vector<Vector3f>& targetGrads);
	void getValidMatchedGeodesicsIndices(mpcard sourceStart, vector<mpcard>& sourceDests, vector<mpcard>& targetGeoEntries);
	void getValidMatchedGeodesicsIndices(mpcard sourceStart, vector<mpcard>& sourceDests, vector<mpcard>& corrIndices, vector<mpcard>& targetGeoEntries);
	void getValidMatchedGeodesicsIndicesLevel(mpcard sourceStart, mpcard level, vector<mpcard>& sourceDests, vector<mpcard>& corrIndices, vector<mpcard>& targetGeoEntries);
	void getMatchedGeodesicsIndicesMostLocal(mpcard sourceStart, vector<mpcard>& sourceDests, vector<mpcard>& targetGeosOut, vector<mpcard>& targetGeosIn);

	void collectAllMatchedGeodesics(mpcard sourceIndex, vector<mpcard>& connectedIndices, vector<mpcard>& toTargetGeos, vector<mpcard>& fromTargetGeos);

	void findMatchedValidIndirectPath(mpcard sourceIndex, mpcard endIndex, float& dist, vector<mpcard>& path, vector<mpcard>& targetGeos);

	bool canBeTriangulated(mpcard sourceIndex)
	{
		mpcard mlevel, i, n;
		mpcard matchedNbrs = 0;
		if (m_nSampleLevels < 2)
			mlevel = 0;
		else
			mlevel = 1;
		n = m_ppTopoGraphSource[mlevel]->getNumAdjacentVertices(sourceIndex);
		for (i = 0; i < n; i++)
		{
			if (matches[m_ppTopoGraphSource[mlevel]->getAdjacentVertex(sourceIndex, i)].matched)
				matchedNbrs++;
		}
		return (matchedNbrs >= 2);
	}

	FloatType computeAvgSourceEdgeLength();
	FloatType computeAvgSourceMatchLevelEdgeLength();
	FloatType computeAvgSourceMatchLevelEdgeWeight();
	
	FloatType getAvgSourceMatchLevelEdgeWeight()
	{
		if (m_avgEdgeWeight == (FloatType)0)
			m_avgEdgeWeight = computeAvgSourceMatchLevelEdgeWeight();
		return m_avgEdgeWeight;
	}

	// --- match cohort count interface: gives which matches were added together and in what sequence
	void finalizeMatchCohort()				
	{
		m_matchCohortCounts.push_back(getNumMatchedIndices());
		resetMatchCloud();
	}
	mpcard getNumMatchCohorts()				{ return m_matchCohortCounts.size(); }
	mpcard getLastMatchCohortCount()
	{
	   if (m_matchCohortCounts.size() > 0)
		   return m_matchCohortCounts.back();
	   return 0;
	}
	mpcard getMatchCohortCount(mpcard cohort)
	{
	   if (cohort < m_matchCohortCounts.size())
		   return m_matchCohortCounts[cohort];
	   if (m_matchCohortCounts.size() > 0)
		   return m_matchCohortCounts.back();
	   return 0;
	}

	void clear();

	virtual void precomputeSourceGeodesics(mpcard nTopologies, string strAttachBaseName);
	virtual void initGeodesics();
	virtual void breadthFirstLocalPatch(UnstructuredInCorePointCloud* pc, PointCloudIterator* pc_it, vector<mpcard>& indices, mpcard ringsToAdd, int32 flagMask, FloatType* pAvgEdgeLength = NULL);
	virtual void breadthFirstLocalPatch(UnstructuredInCorePointCloud* pc, PointCloudIterator* pc_it, const vector<mpcard>& indicesStart, mpcard ringsToAdd, int32 flagMask, vector<mpcard>& indicesNew, bool bResetFlagsAtEnd = false, FloatType* pAvgEdgeLength = NULL);
	virtual void updateGeodesics(mpcard numNewRings = 2);

	FloatType computeErrorSingleMatch(mpcard sourceIndex);
	FloatType computeErrorSingleMatch(mpcard sourceIndex, set<mpcard>& sourceMeasurePoints);

	//compute error of geodesic distances for all pairs of correspondences
	virtual FloatType computeCorrespondenceError();

	//compute error of geodesic distance from all existing correspondences to a single candidate correspondence pair
	virtual FloatType computeCorrespondenceError(const std::pair<Vector3f, Vector3f>& corr);

	//these functions are so that we can save many partial matches to the scene graph structure and leverage out-of-core point cloud
	//infrastructure alread in-place in case the memory requirements are too high
	UnstructuredInCorePointCloud* exportAsFullPointCloud();
	UnstructuredInCorePointCloud* exportAsCompactPointCloud();



	//this function initializes the internal structures from a previous exported point cloud
	void initThroughPointCloud(UnstructuredInCorePointCloud* pc);

	//these functions give the attribute names for accessing match information from the exported point clouds
	static inline string getMatchAttNameTargetPos()			{ return "targetPos"; }
	static inline string getMatchAttNameMatched()			{ return "matched"; }
	static inline string getMatchAttNameMatchOrder()		{ return "matchOrder"; }
	static inline string getMatchAttNameStretch()			{ return "stretch"; }
	static inline string getMatchAttNameIndex()				{ return "index"; }
	static inline string getMatchAttNameDistToStart()		{ return "distToStart"; }

	void reportAllProfileResults();
};


//typedef AR4PartialMatch<ManifoldTraits3f> AR4PartialMatch3f;
class AR4PartialMatch3f : public AR4PartialMatch<ManifoldTraits3f>
{
public:
	AR4PartialMatch3f(AR4DiscreteManifold<ManifoldTraits3f> *source, AR4SmoothManifold<ManifoldTraits3f> *target): AR4PartialMatch<ManifoldTraits3f>(source, target)
	{
	}

	virtual void initGeodesics();
	virtual void breadthFirstLocalPatch(UnstructuredInCorePointCloud* pc, PointCloudIterator* pc_it, vector<mpcard>& indices, mpcard ringsToAdd, int32 flagMask, FloatType* pAvgEdgeLength = NULL);
	virtual void breadthFirstLocalPatch(UnstructuredInCorePointCloud* pc, PointCloudIterator* pc_it, const vector<mpcard>& indicesStart, mpcard ringsToAdd, int32 flagMask, vector<mpcard>& indicesNew, bool bResetFlagsAtEnd = false, FloatType* pAvgEdgeLength = NULL);
	virtual void updateGeodesics(mpcard numNewRings = 2);

	virtual ManifoldTraits3f::FloatType computeCorrespondenceError();
	virtual ManifoldTraits3f::FloatType computeCorrespondenceError(const std::pair<Vector3f, Vector3f>& corr);

	void precomputeSourceGeodesics(mpcard nTopologies, string strAttachBaseName);
	void copyGeodesicAndTopologyData(AR4PartialMatch3f* pOtherMatch);

	void initThroughPointCloud(UnstructuredInCorePointCloud* pc);

	void interpolateFullRes(const card32 nIterations);
};


//class for computing distance/dissimilarity of two partial matches
template <class ManifoldTraits>
class AR4PartialMatchDistance
{
public:
   typedef typename ManifoldTraits::FloatType  FloatType;
   typedef typename ManifoldTraits::VectorType VectorType;
   typedef typename ManifoldTraits AR4DiscreteManifoldTraits;
   typedef typename ManifoldTraits::TangentSpace TangentSpace;
   typedef typename ManifoldTraits::BoundingBoxType BoundingBoxType;

protected:

	AR4PartialMatch<ManifoldTraits>*		m_pMatch1;
	AR4PartialMatch<ManifoldTraits>*		m_pMatch2;

	mpcard									m_compareLevel;

	mpcard									m_nSharedSourcePoints;
	mpcard									m_nSharedTargetPoints;
	FloatType								m_overlap;

	PointSet*								m_pSecondTargPoints;
	AAT										m_aatSecTarg;

	PointSet*								m_pFirstTargPoints;
	AAT										m_aatFirstTarg;

	AttachedIndexedOctree*					m_pOctree2;
	PointSetKNNQuery*						m_pSecondTargetKNN;

	AttachedIndexedOctree*					m_pOctree1;
	PointSetKNNQuery*						m_pFirstTargetKNN;


	void validate()
	{
		if (m_pMatch1 == NULL || m_pMatch2 == NULL)
			throw EInvalidState("AR4PartialMatchDistance -- invalid! -- NULL match");
		if (m_pMatch1->getSourceManifold() != m_pMatch2->getSourceManifold() || m_pMatch1->getTargetManifold() != m_pMatch2->getTargetManifold())
			throw EInvalidState("AR4PartialMatchDistance -- invalid! -- manifolds are not the same");
	}

	void initKNNForSymm()
	{
		delete m_pSecondTargetKNN;
		delete m_pOctree2;
		delete m_pFirstTargetKNN;
		delete m_pOctree1;

		const mpcard numPts = m_pMatch2->getNumMatchedIndices();

		VertexDescriptor vd;
		vd.pushAttrib(VAD("position", 3, VAD::DATA_FORMAT_FLOAT32));

		m_pSecondTargPoints = new PointSet();
		m_pSecondTargPoints->clearAndSetup(1, numPts, &vd);
		m_aatSecTarg = m_pSecondTargPoints->getAAT("position");
		for (mpcard i = 0; i < numPts; i++)
		{
			m_pSecondTargPoints->set3f(i, m_aatSecTarg, m_pMatch2->getMatch(m_pMatch2->getMatchedSourceIndex(i)));
		}

		m_pOctree2 = PointSetKNNQuery::createOctree(m_pSecondTargPoints);
		m_pSecondTargetKNN = new PointSetKNNQuery(m_pSecondTargPoints, m_pOctree2, NULL_VECTOR3F, m_aatSecTarg);

		const mpcard numPtsFirst = m_pMatch1->getNumMatchedIndices();
		
		VertexDescriptor vd1;
		vd1.pushAttrib(VAD("position", 3, VAD::DATA_FORMAT_FLOAT32));

		m_pFirstTargPoints = new PointSet();
		m_pFirstTargPoints->clearAndSetup(1, numPtsFirst, &vd1);
		m_aatFirstTarg = m_pFirstTargPoints->getAAT("position");
		for (mpcard i = 0; i < numPtsFirst; i++)
		{
			m_pFirstTargPoints->set3f(i, m_aatFirstTarg, m_pMatch1->getMatch(m_pMatch1->getMatchedSourceIndex(i)));
		}

		m_pOctree1 = PointSetKNNQuery::createOctree(m_pFirstTargPoints);
		m_pFirstTargetKNN = new PointSetKNNQuery(m_pFirstTargPoints, m_pOctree1, NULL_VECTOR3F, m_aatFirstTarg);
	}

	void sphericalQuerySecondTarget(const VectorType& queryPoint, const FloatType epsilon, vector<mpcard>& neighbors, vector<FloatType>& distances)
	{
		card32 nextNear;
		bool bfound;
		VectorType targPos;
		FloatType targDist;

		neighbors.clear();
		distances.clear();

		m_pSecondTargetKNN->setTargetPoint(queryPoint);
		m_pSecondTargetKNN->getNextNearestPointIndex(nextNear, bfound);
		if (bfound)
		{
			targPos = m_pSecondTargPoints->get3f(nextNear, m_aatSecTarg);
			targDist = m_pMatch2->targetGeodesic(queryPoint, targPos);
		}
		while (bfound && targDist <= epsilon)
		{
			neighbors.push_back((mpcard)nextNear);
			distances.push_back(targDist);
			m_pSecondTargetKNN->getNextNearestPointIndex(nextNear, bfound);
			if (bfound)
			{
				targPos = m_pSecondTargPoints->get3f(nextNear, m_aatSecTarg);
				targDist = m_pMatch2->targetGeodesic(queryPoint, targPos);
			}
		}
	}

	void sphericalQueryFirstTarget(const VectorType& queryPoint, const FloatType epsilon, vector<mpcard>& neighbors, vector<FloatType>& distances)
	{
		card32 nextNear;
		bool bfound;
		VectorType targPos;
		FloatType targDist;

		neighbors.clear();
		distances.clear();

		m_pFirstTargetKNN->setTargetPoint(queryPoint);
		m_pFirstTargetKNN->getNextNearestPointIndex(nextNear, bfound);
		if (bfound)
		{
			targPos = m_pFirstTargPoints->get3f(nextNear, m_aatFirstTarg);
			targDist = m_pMatch1->targetGeodesic(queryPoint, targPos);
		}
		while (bfound && targDist <= epsilon)
		{
			neighbors.push_back((mpcard)nextNear);
			distances.push_back(targDist);
			m_pFirstTargetKNN->getNextNearestPointIndex(nextNear, bfound);
			if (bfound)
			{
				targPos = m_pFirstTargPoints->get3f(nextNear, m_aatFirstTarg);
				targDist = m_pMatch1->targetGeodesic(queryPoint, targPos);
			}
		}
	}


public:

	AR4PartialMatchDistance(AR4PartialMatch<ManifoldTraits>* pMatch1, AR4PartialMatch<ManifoldTraits>* pMatch2)
	{
		m_pMatch1 = pMatch1;
		m_pMatch2 = pMatch2;
		m_nSharedSourcePoints = 0;
		m_nSharedTargetPoints = 0;
		m_overlap = (FloatType)0;

		m_compareLevel = 1;

		m_pSecondTargPoints = NULL;
		m_aatSecTarg = NULL_AAT;
		m_pOctree2 = NULL;
		m_pSecondTargetKNN = NULL;

		m_pFirstTargPoints = NULL;
		m_aatFirstTarg = NULL_AAT;
		m_pOctree1 = NULL;
		m_pFirstTargetKNN = NULL;

		validate();
		initKNNForSymm();
	}
	~AR4PartialMatchDistance()
	{
		delete m_pSecondTargetKNN;
		delete m_pOctree2;
		delete m_pSecondTargPoints;

		delete m_pFirstTargetKNN;
		delete m_pOctree1;
		delete m_pFirstTargPoints;
	}

	virtual FloatType evaluate();
	FloatType getOverlap() const			{ return m_overlap; }

	mpcard getNumSharedSourcePoints() const	{ return m_nSharedSourcePoints; }
	FloatType getOverlapFraction1() const	{ return (FloatType)m_nSharedSourcePoints / (FloatType)m_pMatch1->getNumMatchedIndices(); }
	FloatType getOverlapFraction2() const	{ return (FloatType)m_nSharedSourcePoints / (FloatType)m_pMatch2->getNumMatchedIndices(); }
};


class AR4PartialMatchDistance3f : public AR4PartialMatchDistance<ManifoldTraits3f>
{
public:

	AR4PartialMatchDistance3f(AR4PartialMatch3f* pMatch1, AR4PartialMatch3f* pMatch2): AR4PartialMatchDistance<ManifoldTraits3f>(pMatch1, pMatch2)
	{
	}

	float evaluate()						{ return AR4PartialMatchDistance<ManifoldTraits3f>::evaluate(); }
};


template <class ManifoldTraits>
class AR4SymmetricPartialMatchDistance : public AR4PartialMatchDistance<ManifoldTraits>
{
public:

	AR4SymmetricPartialMatchDistance(AR4PartialMatch<ManifoldTraits>* pMatch1, AR4PartialMatch<ManifoldTraits>* pMatch2): AR4PartialMatchDistance(pMatch1, pMatch2)
	{
	}

	FloatType evaluate();
};


class AR4SymmetricPartialMatchDistance3f : public AR4SymmetricPartialMatchDistance<ManifoldTraits3f>
{
public:

	AR4SymmetricPartialMatchDistance3f(AR4PartialMatch3f* pMatch1, AR4PartialMatch3f* pMatch2): AR4SymmetricPartialMatchDistance<ManifoldTraits3f>(pMatch1, pMatch2)
	{
	}

	float evaluate()						{ return AR4SymmetricPartialMatchDistance<ManifoldTraits3f>::evaluate(); }
};


}

#endif
