#ifndef AR4PartialMatchInlineH
#define AR4PartialMatchInlineH

#include "threeParameterMatchingGMOD14.h"
#include "LinearAlgebra.inline.h"

#include "AR4PartialMatch.h"


#define AR4_PARTIALMATCH_DEBUG_WALKISO		0
#define AR4PM_PROFILE						1
#define AR4_DEBUG_OUTPUT					0

#if AR4PM_PROFILE
#define AR4PM_START_PROFILE(profile) profile.start();
#define AR4PM_STOP_PROFILE(profile)	profile.stop();
#else
#define AR4PM_START_PROFILE(profile)
#define AR4PM_STOP_PROFILE(profile)
#endif //AR4PM_PROFILE


namespace X4 {



template <class ManifoldTraits>
AR4PartialMatch<ManifoldTraits>::AR4PartialMatch( AR4DiscreteManifold<ManifoldTraits> *source, AR4SmoothManifold<ManifoldTraits> *target )
{
   this->source = source;
   this->target = target;

   m_pSourceGeodesics = NULL;
   m_pTargetGeodesics = NULL;
   m_pLocalSourceGeodesics = NULL;
   m_pLocalTargetGeodesics = NULL;
   m_pSparseSourceGeodesics = new AR4StaticSparseGeodesics();
   m_pSparseTargetGeodesics = new AR4DynamicSparseGeodesics();

   m_pHknnSource = NULL;
   m_pHknnTarget = NULL;

   m_nSampleLevels = 1;
   m_ppIterSource = NULL;
   m_ppIterTarget = NULL;
   m_ppTopoknnSource = NULL;
   m_ppTopoknnTarget = NULL;
   m_ppTopoGraphSource = NULL;
   m_ppTopoGraphTarget = NULL;

   m_nSourcePointsMatchingLevel = 0;

   m_pTargetMatchCloud = new UnstructuredInCorePointCloud();
   m_pHknnTargetMatch = NULL;
   m_strSourceIndexName = "sourceIndex";
   m_vdMatch.pushAttrib(VAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
   m_vdMatch.pushAttrib(VAD(m_strSourceIndexName, 1, VAD::DATA_FORMAT_CARD32));
   m_vdMatch.pushAttrib(VAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
   m_pSourceMatchCloud = new UnstructuredInCorePointCloud();
   m_pHknnSourceMatch = NULL;

   m_pKNNMatchSource = NULL;
   m_pKNNMatchTarget = NULL;
   m_pOctreeMatchSource = NULL;
   m_pOctreeMatchTarget = NULL;

   m_pInitializerPointCloud = NULL;

   m_bGeodesicsDirty = true;
   m_eGeodesicsMode = eAR4PMGMSparse;
   m_geodSigma = 10.0;
   m_patchRings = 3;
   m_nGeodesicsNN = 10;

   m_stretchThresh = (FloatType)2;
   m_avgEdgeWeight = (FloatType)0;

   m_bUsePathRelaxation = false;
   m_nRelaxIters = 2;

   m_bSourceGeodesicsPrecomputed = false;

   m_bIsFullRes = false;

   m_bMimick = false;
   m_bOwnsGeodesicsAndTopology = true;

   m_bRecordRejectedOutliers = false;

   m_profStretchMeasure.setName("AR4PartialMatch::computeErrorSingleMatch");
   m_profValidateSource.setName("AR4PartialMatch::validateSourcePath");
   m_profValidateTarget.setName("AR4PartialMatch::validateTargetPath");
   m_profFullValidate.setName("AR4PartialMatch::validatePaths");
   m_profRemoveOutliers.setName("AR4PartialMatch::removeOutliersLastCohort");
   m_profWalkIsometry.setName("AR4PartialMatch::walkIsometry");

   clear();
}

template <class ManifoldTraits>
AR4PartialMatch<ManifoldTraits>::~AR4PartialMatch()
{
	mpcard i;

	if (!m_bMimick)
	{
		if (m_bOwnsGeodesicsAndTopology)
		{
			delete m_pSourceGeodesics;
			delete m_pTargetGeodesics;
			delete m_pLocalSourceGeodesics;
			delete m_pLocalTargetGeodesics;
			delete m_pSparseSourceGeodesics;
			delete m_pSparseTargetGeodesics;

			if (m_ppIterSource != NULL)
			{
				for (i = 0; i < m_nSampleLevels; i++)
					delete m_ppIterSource[i];
			}
		}

		delete [] m_ppIterSource;
		delete [] m_ppTopoGraphSource;

		delete m_pHknnSource;
		delete m_pHknnTarget;
		delete m_pHknnTargetMatch;
		delete m_pTargetMatchCloud;
		delete m_pHknnSourceMatch;
		delete m_pSourceMatchCloud;

		delete m_pKNNMatchSource;
		delete m_pKNNMatchTarget;
		delete m_pOctreeMatchSource;
		delete m_pOctreeMatchTarget;

		m_pInitializerPointCloud = NULL;

		if (m_ppTopoknnSource != NULL)
		{
			for (i = 0; i < m_nSampleLevels; i++)
				delete m_ppTopoknnSource[i];
		}
		delete [] m_ppTopoknnSource;
		if (m_ppTopoknnTarget != NULL)
		{
			for (i = 0; i < m_nSampleLevels; i++)
				delete m_ppTopoknnTarget[i];
		}
		delete [] m_ppTopoknnTarget;
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::clear()
{
   matches.clear();
   Match m;
   matches.resize(source->getNumDiscrPoints(), m);
   matchedSourceIndices.clear();
   m_geodesicMatches.clear();
   m_geodesicMatches.resize(source->getNumDiscrPoints());

   m_rejectedOutliers.clear();

   m_pTargetMatchCloud->clear();
   delete m_pHknnTargetMatch;
   m_pHknnTargetMatch = NULL;
   m_pSourceMatchCloud->clear();
   delete m_pHknnSourceMatch;
   m_pHknnSourceMatch = NULL;

	delete m_pKNNMatchSource;
	m_pKNNMatchSource = NULL;
	delete m_pKNNMatchTarget;
	m_pKNNMatchTarget = NULL;
	delete m_pOctreeMatchSource;
	m_pOctreeMatchSource = NULL;
	delete m_pOctreeMatchTarget;
	m_pOctreeMatchTarget = NULL;
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::mimick(AR4PartialMatch<ManifoldTraits>* pmatch)
{
	mpcard i;

	if (pmatch != NULL && source == pmatch->source && target == pmatch->target)
	{
		clear();
		if (!m_bMimick)
		{
			delete m_pSourceGeodesics;
			delete m_pTargetGeodesics;
			delete m_pLocalSourceGeodesics;
			delete m_pLocalTargetGeodesics;
			delete m_pSparseSourceGeodesics;
			delete m_pSparseTargetGeodesics;
			delete m_pHknnSource;
			delete m_pHknnTarget;
			delete m_pHknnTargetMatch;
			delete m_pTargetMatchCloud;
			delete m_pHknnSourceMatch;
			delete m_pSourceMatchCloud;
			delete [] m_ppTopoGraphSource;
			delete [] m_ppIterSource;
		}

		m_eGeodesicsMode = pmatch->m_eGeodesicsMode;

		m_pSourceGeodesics = pmatch->m_pSourceGeodesics;
		m_pTargetGeodesics = pmatch->m_pTargetGeodesics;

		m_pLocalSourceGeodesics = pmatch->m_pLocalSourceGeodesics;
		m_pLocalTargetGeodesics = pmatch->m_pLocalTargetGeodesics;

		m_pSparseSourceGeodesics = pmatch->m_pSparseSourceGeodesics;
		m_pSparseTargetGeodesics = pmatch->m_pSparseTargetGeodesics;

		m_geodesicMatches = pmatch->m_geodesicMatches;

		m_geodSigma = pmatch->m_geodSigma;

		m_pHknnSource = pmatch->m_pHknnSource;
		m_pHknnTarget = pmatch->m_pHknnTarget;

		m_nSampleLevels = pmatch->m_nSampleLevels;
		m_ppTopoGraphSource = pmatch->m_ppTopoGraphSource;
		m_ppIterSource = pmatch->m_ppIterSource;
		m_ppTopoknnSource = pmatch->m_ppTopoknnSource;
		m_ppTopoknnTarget = pmatch->m_ppTopoknnTarget;

		m_pTargetMatchCloud = pmatch->m_pTargetMatchCloud;
		m_pHknnTargetMatch = pmatch->m_pHknnTargetMatch;

		m_pSourceMatchCloud = pmatch->m_pSourceMatchCloud;
		m_pHknnSourceMatch = pmatch->m_pHknnSourceMatch;

		m_pKNNMatchSource = pmatch->m_pKNNMatchSource;
		m_pKNNMatchTarget = pmatch->m_pKNNMatchTarget;
		m_pOctreeMatchSource = pmatch->m_pOctreeMatchSource;
		m_pOctreeMatchTarget = pmatch->m_pOctreeMatchTarget;

		m_topoSampleSpacings = pmatch->m_topoSampleSpacings;

		m_bUsePathRelaxation = pmatch->m_bUsePathRelaxation;

		//add matches
		matches = pmatch->matches;
		matchedSourceIndices = pmatch->matchedSourceIndices;

		matchPosBackup = pmatch->matchPosBackup;
		matchPosDirty = pmatch->matchPosDirty;

		m_matchCohortCounts.resize(pmatch->m_matchCohortCounts.size());
		for (i = 0; i < m_matchCohortCounts.size(); i++)
			m_matchCohortCounts[i] = pmatch->m_matchCohortCounts[i];

		m_bMimick = true;
		m_bGeodesicsDirty = pmatch->m_bGeodesicsDirty;
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::setTopologyHierarchy(mpcard numLevels, InCorePCTopologyGraph** ppTopoSource, InCorePCTopologyGraph** ppTopoTarget)
{
	mpcard i;

	m_nSampleLevels = numLevels + 1;

	AR4MLSManifold<ManifoldTraits>* pmlsSource = dynamic_cast<AR4MLSManifold<ManifoldTraits>*>(source);
	AR4MLSManifold<ManifoldTraits>* pmlsTarget = dynamic_cast<AR4MLSManifold<ManifoldTraits>*>(target);

	m_ppTopoGraphSource = new InCorePCTopologyGraph*[m_nSampleLevels];
	m_ppTopoGraphTarget = new InCorePCTopologyGraph*[m_nSampleLevels];

	m_ppTopoGraphSource[0] = pmlsSource->getTopology();
	m_ppTopoGraphTarget[0] = pmlsTarget->getTopology();

	for (i = 1; i < m_nSampleLevels; i++)
	{
		m_ppTopoGraphSource[i] = ppTopoSource[i - 1];
		m_ppTopoGraphTarget[i] = ppTopoTarget[i - 1];
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceNeighborhood(mpcard sourceIndex, vector<mpcard>& neighbors)
{
	VectorType qpoint;
	mpcard currentIndex;

	if (m_pHknnSource == NULL)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getSourceNeighborhood(...): hierarchical k-nn query structure is null");

	qpoint = source->getDiscrPoint(sourceIndex);

	neighbors.clear();

	m_pHknnSource->setSeekPointAndReset(qpoint);
	m_pHknnSource->next();
	while (!m_pHknnSource->atEnd())
	{
		currentIndex = m_pHknnSource->getPointSetPointNumber();
		neighbors.push_back(currentIndex);
		m_pHknnSource->next();
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceMatchedNeighborhood(mpcard sourceIndex, vector<mpcard>& neighbors)
{
	VectorType qpoint;
	mpcard currentIndex;

	if (m_pHknnSource == NULL)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getSourceNeighborhood(...): hierarchical k-nn query structure is null");

	qpoint = source->getDiscrPoint(sourceIndex);

	neighbors.clear();

	m_pHknnSource->setSeekPointAndReset(qpoint);
	m_pHknnSource->next();
	while (!m_pHknnSource->atEnd())
	{
		currentIndex = m_pHknnSource->getPointSetPointNumber();
		if (isMatched(currentIndex))
			neighbors.push_back(currentIndex);
		m_pHknnSource->next();
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceMatchedCachedNeighborhood(mpcard sourceIndex, vector<mpcard>& neighbors)
{
	warning("AR4PartialMatch<ManifoldTraits>::getSourceMatchedCachedNeighborhood(...): deprecated and removed");
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceMatchedCachedNeighborhood(mpcard sourceIndex, vector<mpcard>& neighbors, vector<mpcard>& neighborsCorrIndex)
{
	warning("AR4PartialMatch<ManifoldTraits>::getSourceMatchedCachedNeighborhood(...): deprecated and removed");
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceMatchedFixedCachedNeighborhood(mpcard sourceIndex, vector<mpcard>& neighbors)
{
	warning("AR4PartialMatch<ManifoldTraits>::getSourceMatchedFixedCachedNeighborhood(...): deprecated and removed");
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceMatchedFixedCachedNeighborhood(mpcard sourceIndex, mpcard level, mpcard numMaxNbrs, vector<mpcard>& neighbors)
{
	warning("AR4PartialMatch<ManifoldTraits>::getSourceMatchedFixedCachedNeighborhood(...): deprecated and removed");
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceRecentMatchedNeighborhood(mpcard sourceIndex, mpcard numCohortsBack, vector<mpcard>& neighbors)
{
	warning("AR4PartialMatch<ManifoldTraits>::getSourceRecentMatchedNeighborhood(...): deprecated");

	VectorType qpoint;
	mpcard currentIndex;
	mpcard i, icoho, icohostart;

	if (m_pHknnSource == NULL)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getSourceNeighborhood(...): hierarchical k-nn query structure is null");

	qpoint = source->getDiscrPoint(sourceIndex);

	neighbors.clear();

	m_pHknnSource->setSeekPointAndReset(qpoint);
	m_pHknnSource->next();
	while (!m_pHknnSource->atEnd())
	{
		currentIndex = m_pHknnSource->getPointSetPointNumber();
		if (isMatched(currentIndex))
		{
			icoho = getNumMatchCohorts();
			if (icoho > numCohortsBack)
				icoho -= numCohortsBack;
			else
				icoho = 0;
			if (icoho > 0)
				icohostart = getMatchCohortCount(icoho - 1);
			else
				icohostart = 0;

			for (i = icohostart; i < getNumMatchedIndices(); i++)
			{
				if (matchedSourceIndices[i] == currentIndex)
				{
					neighbors.push_back(currentIndex);
					break;
				}
			}
		}
		m_pHknnSource->next();
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceGeodesicsIndices(mpcard sourceStart, vector<mpcard>& sourceDests)
{
	vector<float> tempSourceDists;
	vector<Vector3f> tempSourceGrads;

	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getSourceGeodesicsIndices(...): only valid in sparse geodesic mode");
	if (!matches[sourceStart].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getSourceGeodesicsIndices(...): sourceStart not matched");

	sourceDests.clear();

	m_pSparseSourceGeodesics->getGeodesicData(sourceStart, sourceDests, tempSourceDists, tempSourceGrads);
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getMatchedGeodesics(mpcard sourceStart, vector<mpcard>& sourceDests, vector<float>& sourceDists, vector<float>& targetDists, vector<Vector3f>& targetGrads)
{
	mpcard i;
	vector<mpcard> tempSourceDests;
	vector<float> tempSourceDists;
	vector<Vector3f> tempSourceGrads;
	vector<Vector3f> targpts(1);
	vector<float> targdists(1);
	vector<Vector3f> targgrads(1);
	mpcard targGeo;
	float targDist;
	Vector3f targGrad;

	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getMatchedGeodesics(...): only valid in sparse geodesic mode");
	if (!matches[sourceStart].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getMatchedGeodesics(...): sourceStart not matched");

	sourceDests.clear();
	sourceDists.clear();
	targetDists.clear();
	targetGrads.clear();

	m_pSparseSourceGeodesics->getGeodesicData(sourceStart, tempSourceDests, tempSourceDists, tempSourceGrads);

	for (i = 0; i < tempSourceDests.size(); i++)
	{
		if (isMatched(tempSourceDests[i]))
		{
			sourceDests.push_back(tempSourceDests[i]);
			sourceDists.push_back(tempSourceDists[i]);

			//get corresponding target information
			targGeo = lookupGeodesicMatch(sourceStart, tempSourceDests[i]);
			if (targGeo != AR4SparseGeodesics::m_InvalidIndex)
			{
				m_pSparseTargetGeodesics->getGeodesicData(targGeo, targDist, targGrad);
				targetDists.push_back(targDist);
				targetGrads.push_back(targGrad);
			}
			else
			{
				targpts[0] = matches[tempSourceDests[i]].targetPos;
				m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, matches[sourceStart].targetPos, targpts, targdists, targgrads);
				targGeo = m_pSparseTargetGeodesics->storeGeodesicData(matches[sourceStart].targetPos, targpts[0], targdists[0], targgrads[0]);
				addGeodesicMatch(sourceStart, tempSourceDests[i], targGeo);
			}
		}
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesics(mpcard sourceStart, vector<mpcard>& sourceDests, vector<float>& sourceDists, vector<float>& targetDists, vector<Vector3f>& targetGrads)
{
	mpcard i;
	vector<mpcard> tempSourceDests;
	vector<float> tempSourceDists;
	vector<Vector3f> tempSourceGrads;
	mpcard targGeo;
	float targDist;
	Vector3f targGrad;

	vector<Vector3f> endpts(1);
	vector<float> dists(1);
	vector<Vector3f> endgrads(1);
	vector<vector<Vector3f>> paths(1);
	bool bvalid;

	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesics(...): only valid in sparse geodesic mode");
	if (!matches[sourceStart].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesics(...): sourceStart not matched");

	sourceDests.clear();
	sourceDists.clear();
	targetDists.clear();
	targetGrads.clear();

	//retrieve all available source geodesic information from given start point (index)
	m_pSparseSourceGeodesics->getGeodesicData(sourceStart, tempSourceDests, tempSourceDists, tempSourceGrads);

	//from this, find which paths are between matched points and are valid on both source and target surfaces
	for (i = 0; i < tempSourceDests.size(); i++)
	{
		bvalid = false;
		if (isMatched(tempSourceDests[i]) && m_pSparseSourceGeodesics->isPathValid(sourceStart, tempSourceDests[i]))
		{
			//we have verified that this destination is matched and that the path is valid
			//get corresponding target information, and check for validity
			targGeo = lookupGeodesicMatch(sourceStart, tempSourceDests[i]);
			if (targGeo != AR4SparseGeodesics::m_InvalidIndex)
			{
				//geodesic on target surface has already been computed
				//check validity, and if valid retrieve distance and gradient
				bvalid = m_pSparseTargetGeodesics->isPathValid(targGeo);
				if (bvalid)
					m_pSparseTargetGeodesics->getGeodesicData(targGeo, targDist, targGrad);
			}
			else
			{
				//target geodesic not yet computed
				//compute now
				endpts[0] = matches[tempSourceDests[i]].targetPos;
				m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, matches[sourceStart].targetPos, endpts, dists, endgrads, &paths);
				targGeo = m_pSparseTargetGeodesics->storeGeodesicData(matches[sourceStart].targetPos, endpts[0], dists[0], endgrads[0], &paths[0]);
				targDist = dists[0];
				targGrad = endgrads[0];

				//add this index to geodesic match look-up
				addGeodesicMatch(sourceStart, tempSourceDests[i], targGeo);
			}
		}
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndices(mpcard sourceStart, vector<mpcard>& sourceDests, vector<mpcard>& targetGeoEntries)
{
	mpcard i;
	mpcard targGeo;
	vector<mpcard> tempSourceDests;
	vector<float> tempSourceDists;
	vector<Vector3f> tempSourceGrads;

	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndices(...): only valid in sparse geodesic mode");
	if (!matches[sourceStart].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndices(...): sourceStart not matched");

	sourceDests.clear();
	targetGeoEntries.clear();

	//retrieve all available source geodesic information from given start point (index)
	m_pSparseSourceGeodesics->getGeodesicData(sourceStart, tempSourceDests, tempSourceDists, tempSourceGrads);

	for (i = 0; i < tempSourceDests.size(); i++)
	{
		if (isMatched(tempSourceDests[i]) && m_pSparseSourceGeodesics->isPathValid(sourceStart, tempSourceDests[i]))
		{
			targGeo = lookupGeodesicMatch(sourceStart, tempSourceDests[i]);
			if (targGeo != AR4SparseGeodesics::m_InvalidIndex && m_pSparseTargetGeodesics->isPathValid(targGeo))
			{
				sourceDests.push_back(tempSourceDests[i]);
				targetGeoEntries.push_back(targGeo);
			}
		}
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getMatchedGeodesicsIndicesMostLocal(mpcard sourceStart, vector<mpcard>& sourceDests, vector<mpcard>& targetGeosOut, vector<mpcard>& targetGeosIn)
{
	mpcard i, k, n;
	mpcard targGeo;
	mpcard matchLevel;

	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getMatchedGeodesicsIndicesMostLocal(...): only valid in sparse geodesic mode");
	if (!matches[sourceStart].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getMatchedGeodesicsIndicesMostLocal(...): source index " + intToStr(sourceStart) + " not matched");

	if (m_nSampleLevels < 2)
		matchLevel = 0;
	else
		matchLevel = 1;

	sourceDests.clear();
	targetGeosOut.clear();
	targetGeosIn.clear();

	n = m_ppTopoGraphSource[matchLevel]->getNumAdjacentVertices(sourceStart);
	for (i = 0; i < n; i++)
	{
		k = m_ppTopoGraphSource[matchLevel]->getAdjacentVertex(sourceStart, i);
		if (matches[k].matched)
		{
			targGeo = lookupGeodesicMatch(sourceStart, k);
			if (targGeo != AR4SparseGeodesics::m_InvalidIndex)
			{
				sourceDests.push_back(k);
				targetGeosOut.push_back(targGeo);
			}

			targGeo = lookupGeodesicMatch(k, sourceStart);
			if (targGeo != AR4SparseGeodesics::m_InvalidIndex)
				targetGeosIn.push_back(targGeo);
		}
	}

	if (targetGeosIn.size() != targetGeosOut.size())
		warning("AR4PartialMatch<ManifoldTraits>::getMatchedGeodesicsIndicesMostLocal(...): asymmetry in target geodesic information");
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndices(mpcard sourceStart, vector<mpcard>& sourceDests, vector<mpcard>& corrIndices, vector<mpcard>& targetGeoEntries)
{
	mpcard i;
	mpcard targGeo;
	vector<mpcard> tempSourceDests;
	vector<float> tempSourceDists;
	vector<Vector3f> tempSourceGrads;

	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndices(...): only valid in sparse geodesic mode");
	if (!matches[sourceStart].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndices(...): sourceStart not matched");

	sourceDests.clear();
	corrIndices.clear();
	targetGeoEntries.clear();

	//retrieve all available source geodesic information from given start point (index)
	m_pSparseSourceGeodesics->getGeodesicData(sourceStart, tempSourceDests, tempSourceDists, tempSourceGrads);

	for (i = 0; i < tempSourceDests.size(); i++)
	{
		if (isMatched(tempSourceDests[i]) && m_pSparseSourceGeodesics->isPathValid(sourceStart, tempSourceDests[i]))
		{
			targGeo = lookupGeodesicMatch(sourceStart, tempSourceDests[i]);
			if (targGeo != AR4SparseGeodesics::m_InvalidIndex && m_pSparseTargetGeodesics->isPathValid(targGeo))
			{
				sourceDests.push_back(tempSourceDests[i]);
				targetGeoEntries.push_back(targGeo);
				corrIndices.push_back(matches[tempSourceDests[i]].matchOrder);
			}
		}
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndicesLevel(mpcard sourceStart, mpcard level, vector<mpcard>& sourceDests, vector<mpcard>& corrIndices, vector<mpcard>& targetGeoEntries)
{
	mpcard i;
	mpcard targGeo;
	vector<mpcard> tempSourceDests;
	vector<float> tempSourceDists;
	vector<Vector3f> tempSourceGrads;

	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndicesLevel(...): only valid in sparse geodesic mode");
	if (!matches[sourceStart].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndicesLevel(...): sourceStart not matched");
	if (level >= m_nSampleLevels)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getValidMatchedGeodesicsIndicesLevel(...): invalid topology level");

	sourceDests.clear();
	corrIndices.clear();
	targetGeoEntries.clear();

	if (!isAtLevel(sourceStart, level))
		return;

	//retrieve all available source geodesic information from given start point (index)
	m_pSparseSourceGeodesics->getGeodesicData(sourceStart, tempSourceDests, tempSourceDists, tempSourceGrads);

	for (i = 0; i < tempSourceDests.size(); i++)
	{
		if (isMatched(tempSourceDests[i]) && isAtLevel(tempSourceDests[i], level) && m_pSparseSourceGeodesics->isPathValid(sourceStart, tempSourceDests[i]))
		{
			targGeo = lookupGeodesicMatch(sourceStart, tempSourceDests[i]);
			if (targGeo != AR4SparseGeodesics::m_InvalidIndex && m_pSparseTargetGeodesics->isPathValid(targGeo))
			{
				sourceDests.push_back(tempSourceDests[i]);
				targetGeoEntries.push_back(targGeo);
				corrIndices.push_back(matches[tempSourceDests[i]].matchOrder);
			}
		}
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::collectAllMatchedGeodesics(mpcard sourceIndex, vector<mpcard>& connectedIndices, vector<mpcard>& toTargetGeos, vector<mpcard>& fromTargetGeos)
{
	mpcard i, j, targGeo;
	vector<mpcard> sourceDests;
	vector<float> dists;
	vector<Vector3f> grads;

	if (!matches[sourceIndex].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::collectAllMatchedGeodesics(...): source index " + intToStr(sourceIndex) + " is not matched");
	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::collectAllMatchedGeodesics(...): only valid in sparse geodesic mode");

	connectedIndices.clear();
	toTargetGeos.clear();
	fromTargetGeos.clear();

	m_pSparseSourceGeodesics->getGeodesicData(sourceIndex, sourceDests, dists, grads);

	for (i = 0; i < sourceDests.size(); i++)
	{
		targGeo = lookupGeodesicMatch(sourceIndex, sourceDests[i]);
		if (isMatched(sourceDests[i]) && targGeo != AR4SparseGeodesics::m_InvalidIndex)
		{
			connectedIndices.push_back(sourceDests[i]);
			toTargetGeos.push_back(targGeo);
			targGeo = lookupGeodesicMatch(sourceDests[i], sourceIndex);
			if (targGeo != AR4SparseGeodesics::m_InvalidIndex)
				fromTargetGeos.push_back(targGeo);
			else
				warning("AR4PartialMatch<ManifoldTraits>::collectAllMatchedGeodesics(...) - geodesic is matched in one direction (" + intToStr(sourceIndex) + " to " + intToStr(sourceDests[i]) + ") but not the other - this should not happen");
		}
	}
}

template<class ManifoldTraits>
bool AR4PartialMatch<ManifoldTraits>::validateSourcePath(const vector<mpcard>& path)
{
	bool bvalid = true;

	return bvalid;
}

template<class ManifoldTraits>
bool AR4PartialMatch<ManifoldTraits>::validateSourcePath(const vector<Vector3f>& path)
{
	AR4PM_START_PROFILE(m_profValidateSource)

	bool bvalid = true;
	mpcard i;
	AAT pos = m_pSourceMatchCloud->getAAT("position");
	Vector3f matchPos;

	for (i = 0; i < path.size() && bvalid; i++)
	{
		bvalid = false;
		matchPos = getNearestSourceMatchPoint(path[i]);
		if ((matchPos - path[i]).getNorm() <= m_topoSampleSpacings[1])
		{
			bvalid = true;
			break;
		}
	}

	AR4PM_STOP_PROFILE(m_profValidateSource)

	return bvalid;
}

template<class ManifoldTraits>
bool AR4PartialMatch<ManifoldTraits>::validateTargetPath(const vector<mpcard>& path)
{
	bool bvalid = true;

	return bvalid;
}

template<class ManifoldTraits>
bool AR4PartialMatch<ManifoldTraits>::validateTargetPath(const vector<Vector3f>& path)
{
	AR4PM_START_PROFILE(m_profValidateTarget)

	bool bvalid = true;
	mpcard i;
	AAT pos = m_pTargetMatchCloud->getAAT("position");
	Vector3f matchPos;

	for (i = 0; i < path.size() && bvalid; i++)
	{
		bvalid = false;
		matchPos = getNearestTargetMatchPoint(path[i]);
		if ((matchPos - path[i]).getNorm() <= m_topoSampleSpacings[1])
		{
			bvalid = true;
			break;
		}
	}

	AR4PM_STOP_PROFILE(m_profValidateTarget)

	return bvalid;
}


template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::computeAvgSourceEdgeLength()
{
	mpcard i, j, k, n, nedges = 0;
	FloatType avgEdge = (FloatType)0;
	VectorType v1, v2;

	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];
		v1 = source->getDiscrPoint(k);
		n = source->getNumNeighbors(k);
		for (j = 0; j < n; j++)
		{
			v2 = source->getDiscrPoint(source->getNeighborIndex(k, j));
			avgEdge += (v2 - v1).getNorm();
			nedges++;
		}
	}

	avgEdge /= (FloatType)nedges;

	return avgEdge;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::computeAvgSourceMatchLevelEdgeLength()
{
	mpcard i, j, k, n, nedges = 0;
	FloatType avgEdge = (FloatType)0;
	VectorType v1, v2;

	if (m_nSampleLevels < 2)
		return computeAvgSourceEdgeLength();

	for (i = 0; i < source->getNumDiscrPoints(); i++)
	{
		v1 = source->getDiscrPoint(i);
		n = m_ppTopoGraphSource[1]->getNumAdjacentVertices(i);
		for (j = 0; j < n; j++)
		{
			k = m_ppTopoGraphSource[1]->getAdjacentVertex(i, j);
			v2 = source->getDiscrPoint(k);
			avgEdge += (v2 - v1).getNorm();
			nedges++;
		}
	}

	avgEdge /= (FloatType)nedges;

	return avgEdge;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::computeAvgSourceMatchLevelEdgeWeight()
{
	mpcard i, j, k, n, nedges = 0;
	FloatType avgEdge = (FloatType)0;
	PointSet* pEdges;
	InCorePCTopologyGraph::EdgeIndex ei;
	AAT waat;

	if (m_nSampleLevels < 2)
		return computeAvgSourceEdgeLength();

	pEdges = m_ppTopoGraphSource[1]->getEdges();
	waat = pEdges->getAAT("weight");

	for (i = 0; i < source->getNumDiscrPoints(); i++)
	{
		n = m_ppTopoGraphSource[1]->getNumAdjacentVertices(i);
		for (j = 0; j < n; j++)
		{
			k = m_ppTopoGraphSource[1]->getAdjacentVertex(i, j);
			ei = m_ppTopoGraphSource[1]->getEdgeIndex(i, j);
			avgEdge += pEdges->get1f(ei, waat);
			nedges++;
		}
	}

	avgEdge /= (FloatType)nedges;

	return avgEdge;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::sourceGeodesic(const VectorType& from, const VectorType& to, VectorType& derivative)
{
	FloatType geodist = 0;

	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMLocal:
		geodist = m_pLocalSourceGeodesics->geodesic(from, to, m_geodSigma, derivative);
		break;
	case eAR4PMGMGlobal:
		geodist = m_pSourceGeodesics->interpolate(from, to, m_geodSigma, derivative);
		break;
	}

	return geodist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::targetGeodesic(const VectorType& from, const VectorType& to, VectorType& derivative)
{
	FloatType geodist = 0;

	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMLocal:
		geodist = m_pLocalTargetGeodesics->geodesic(from, to, m_geodSigma, derivative);
		break;
	case eAR4PMGMGlobal:
		geodist = m_pTargetGeodesics->interpolate(from, to, m_geodSigma, derivative);
		break;
	}

	return geodist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::sourceGeodesic(const VectorType& from, const VectorType& to)
{
	FloatType geodist = 0;

	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMLocal:
		geodist = m_pLocalSourceGeodesics->geodesic(from, to, m_geodSigma);
		break;
	case eAR4PMGMGlobal:
		geodist = m_pSourceGeodesics->interpolate(from, to, m_geodSigma);
		break;
	}

	return geodist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::targetGeodesic(const VectorType& from, const VectorType& to)
{
	FloatType geodist = 0;
	Vector3f grad;

	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMLocal:
		geodist = m_pLocalTargetGeodesics->geodesic(from, to, m_geodSigma);
		break;
	case eAR4PMGMGlobal:
		geodist = m_pTargetGeodesics->interpolate(from, to, m_geodSigma);
		break;
	case eAR4PMGMSparse:
		m_pSparseTargetGeodesics->computeGeodesicSingleDest(m_geodSigma, from, to, geodist, grad);
		break;
	}

	return geodist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::sourceGeodesic(const mpcard start, const mpcard end, VectorType& derivative)
{
	FloatType geodist = 0;

	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMSparse:
		if (m_pSparseSourceGeodesics->getGeodesicData(start, end, geodist, derivative) == AR4SparseGeodesics::m_InvalidIndex)
			geodist = (FloatType)-1;
		break;
	}

	return geodist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::sourceGeodesic(const mpcard start, const mpcard end)
{
	FloatType geodist = 0;
	VectorType derivative, sourceAtDir;

	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMSparse:
		if (m_pSparseSourceGeodesics->getGeodesicData(start, end, geodist, derivative, sourceAtDir) == AR4SparseGeodesics::m_InvalidIndex)
			geodist = (FloatType)-1;
		break;
	}

	return geodist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::sourceGeodesic(const mpcard start, const mpcard end, vector<VectorType>& path)
{
	FloatType geodist = 0;
	VectorType derivative, dirAtSource;

	if (m_eGeodesicsMode == eAR4PMGMSparse)
	{
		if (m_pSparseSourceGeodesics->getGeodesicData(start, end, geodist, derivative, dirAtSource, &path) == AR4SparseGeodesics::m_InvalidIndex)
			geodist = (FloatType)-1;
	}

	return geodist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::targetGeodesic(const mpcard assocSourceStart, const mpcard assocSourceEnd, const VectorType& targetStart, const VectorType& targetEnd, VectorType& derivative)
{


	FloatType geodist = 0;
	mpcard targGeo;
	vector<VectorType> path;

	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMSparse:
		targGeo = lookupGeodesicMatch(assocSourceStart, assocSourceEnd);
		if (targGeo == AR4SparseGeodesics::m_InvalidIndex)
		{
			//not yet computed, compute and associate with these source indices
			m_pSparseTargetGeodesics->computeGeodesicSingleDest(m_geodSigma, targetStart, targetEnd, geodist, derivative, path);
			targGeo = m_pSparseTargetGeodesics->storeGeodesicData(targetStart, targetEnd, geodist, derivative, path);
			addGeodesicMatch(assocSourceStart, assocSourceEnd, targGeo);
		}
		else
		{
			//has been previously computed, simply retrieve information
			m_pSparseTargetGeodesics->getGeodesicData(targGeo, geodist, derivative);
		
		}
		break;
	}

	return geodist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::targetGeodesic(const mpcard assocSourceStart, const mpcard assocSourceEnd, const VectorType& targetStart, const VectorType& targetEnd, vector<VectorType>& path)
{
	FloatType geodist = 0;
	mpcard targGeo;
	VectorType derivative;

	switch (m_eGeodesicsMode)
	{
	case eAR4PMGMSparse:
		targGeo = lookupGeodesicMatch(assocSourceStart, assocSourceEnd);
		if (targGeo == AR4SparseGeodesics::m_InvalidIndex)
		{
			//not yet computed, compute and associate with these source indices
			m_pSparseTargetGeodesics->computeGeodesicSingleDest(m_geodSigma, targetStart, targetEnd, geodist, derivative, &path);
			targGeo = m_pSparseTargetGeodesics->storeGeodesicData(targetStart, targetEnd, geodist, derivative, &path);
			addGeodesicMatch(assocSourceStart, assocSourceEnd, targGeo);
		}
		else
		{
			//has been previously computed, simply retrieve information
			m_pSparseTargetGeodesics->getGeodesicData(targGeo, geodist, derivative, &path);
		}
		break;
	}

	return geodist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::targetGeodesic(const mpcard geodesicEntry, VectorType& derivative)
{
	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::targetGeodesic(...): must be using sparse geodesics");

	FloatType dist = (FloatType)0;

	m_pSparseTargetGeodesics->getGeodesicData(geodesicEntry, dist, derivative);

	return dist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::sourceGeodesicDiameter()
{
	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::sourceGeodesicDiameter(): must be using sparse geodesics");

	FloatType dist, distMax = 0.f;
	mpcard i, n = source->getNumDiscrPoints();

	for (i = 0; i < n; i++)
	{
		dist = m_pSparseSourceGeodesics->computeEccentricity(i);
		if (dist > distMax)
			distMax = dist;
	}

	return dist;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::targetGeodesicDiameter()
{
	if (m_eGeodesicsMode != eAR4PMGMSparse)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::targetGeodesicDiameter(): must be using sparse geodesics");

	AR4MLSManifold<ManifoldTraits>* pTargetMLS = dynamic_cast<AR4MLSManifold<ManifoldTraits>*>(target);
	FloatType dist, distMax = 0.f;
	mpcard i, n = pTargetMLS->getNumDiscrPoints();

	for (i = 0; i < n; i++)
	{
		dist = m_pSparseTargetGeodesics->computeEccentricity(i);
		if (dist > distMax)
			distMax = dist;
	}

	return dist;
}

template <class ManifoldTraits>
bool AR4PartialMatch<ManifoldTraits>::walkIsometry(const mpcard sourceStart, const mpcard sourceEnd, const vector<VectorType>& sourcePath, const VectorType& targetPoint, const VectorType& targetDir, VectorType& targetEndPoint)
{
	const mpcard numMaxSteps = 1000;

	AR4PM_START_PROFILE(m_profWalkIsometry)

	mpcard i;
	VectorType sdir, tdir, sdirNext;
	VectorType sstep, tstep;
	VectorType sstart, tstart;
	VectorType scur, tcur, snext, tnext, sprev, tprev;
	FloatType distSoFar;

	mpcard idirPtSrc;
	VectorType srcDirPtFound, srcDisplace, targDisplace;
	VectorType snorm, scross, tnorm, tcross;
	FloatType distToWalk, localu, localv;

	AR4MLSManifold<ManifoldTraits>* pmlsSource = dynamic_cast<AR4MLSManifold<ManifoldTraits>*>(source);
	AR4MLSManifold<ManifoldTraits>* pmlsTarget = dynamic_cast<AR4MLSManifold<ManifoldTraits>*>(target);

	//ideally this should be set based on curvature (inversely proportional or something)
	const FloatType stepSize = (FloatType)1;//(FloatType)0.1;
	FloatType stepSizeTemp;
	mpcard stepCount;
	bool bprjs, bprjt;

	FloatType pathLenS = (FloatType)0;
	FloatType pathLenT = (FloatType)0;

	tdir = targetDir;
	tdir.normalize();

	sprev = scur = sourcePath[0];
	tprev = tcur = targetPoint;

#if AR4_PARTIALMATCH_DEBUG_WALKISO
	mpcard dbgFrameCount = 0;
	debugRenderer->beginRenderJob_oneFrame("AR4PartialMatch::walkIsometry", dbgFrameCount);
	debugRenderer->endRenderJob();
#endif //AR4_PARTIALMATCH_DEBUG_WALKISO

	for (i = 1; i < sourcePath.size(); i++)
	{
		sdir = sourcePath[i] - sourcePath[i - 1];
		distToWalk = sdir.getNorm();
		sdir.normalize();

		pathLenS += distToWalk;

		stepCount = 0;
		distSoFar = (FloatType)0;
		stepSizeTemp = stepSize * distToWalk;
		sstep = sdir * stepSizeTemp;
		tstep = tdir * stepSizeTemp;
		while (distSoFar < distToWalk && stepCount < numMaxSteps)
		{
			tnext = tcur + tstep;
			stepCount++;
			bprjt = pmlsTarget->project(tnext, tcur, tnorm);
			if (!bprjt)
			{
				tcur = tprev;
				stepSizeTemp *= (FloatType)0.5;
				tstep = tdir * stepSizeTemp;
				continue;
			}

			distSoFar += (tcur - tprev).getNorm();
			stepSizeTemp = min(stepSizeTemp, distToWalk - distSoFar);

			tcross = tnorm.crossProduct(tdir);
			tcross.normalize();
			tdir = tcross.crossProduct(tnorm);
			tdir.normalize();

#if AR4_PARTIALMATCH_DEBUG_WALKISO
			dbgFrameCount++;
			debugRenderer->beginRenderJob_oneFrame("AR4PartialMatch::walkIsometry", dbgFrameCount);
			debugRenderer->endRenderJob();
#endif //AR4_PARTIALMATCH_DEBUG_WALKISO

			tstep = tdir * stepSizeTemp;

			tprev = tcur;
		}

		pathLenT += distSoFar;

		//if we failed to reach destination in prescribed number of steps
		if (stepCount >= numMaxSteps)
		{
			AR4PM_STOP_PROFILE(m_profWalkIsometry);
			return false;
		}

		if (i < sourcePath.size() - 1)
		{
			//compute change in path, and change direction accordingly on target
			bprjs = pmlsSource->project(sourcePath[i], scur, snorm);
			if(!bprjs)
				debugOutput << "walkIsometry::project on source failed\n";
			bprjt = pmlsTarget->project(tnext, tcur, tnorm);
			if(!bprjt)
				debugOutput << "walkIsometry::project on target failed\n";

			scross = snorm.crossProduct(sdir);
			scross.normalize();
			sdir = scross.crossProduct(snorm);
			sdir.normalize();

			tcross = tnorm.crossProduct(tdir);
			tcross.normalize();
			tdir = tcross.crossProduct(tnorm);
			tdir.normalize();

			sdirNext = sourcePath[i + 1] - sourcePath[i];
			sdirNext.normalize();
			localu = sdirNext * sdir;
			localv = sdirNext * scross;

			tdir = tdir * localu + tcross * localv;
			tdir.normalize();
		}
	}


	targetEndPoint = tcur;

	AR4PM_STOP_PROFILE(m_profWalkIsometry)

	return true;
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::initMatchesFromPointAndDir(const mpcard sourceIndex, const VectorType& sourceDir, const VectorType& targetPoint, const VectorType& targetDir, mpcard numMatchesToInit/* = 1*/)
{
	Timer timer;

	mpcard i;

	numMatchesToInit = max((int) numMatchesToInit, 1);

	FloatType angleStepSize = (FloatType)(2.0 * M_PI / (double)numMatchesToInit);
	FloatType angle;
	FloatType u, v;

	VectorType spt, tpt;
	VectorType sdir, snorm, scross, tdir, tnorm, tcross;
	VectorType sdirWalk, tdirWalk;

	vector<mpcard> sourceNeighbors;
	mpcard sourceDirIndex;
	VectorType targetDirPoint;

	vector<FloatType> sourceDists;
	vector<VectorType> sourceGrads, dirsAtSource;
	vector<vector<VectorType>> sourcePaths;

	AR4MLSManifold<ManifoldTraits>* pmlsSource = dynamic_cast<AR4MLSManifold<ManifoldTraits>*>(source);
	AR4MLSManifold<ManifoldTraits>* pmlsTarget = dynamic_cast<AR4MLSManifold<ManifoldTraits>*>(target);

	getSourceMatchingLevelNeighborhood(sourceIndex, 1, sourceNeighbors);
	sourceDists.resize(sourceNeighbors.size());
	sourceGrads.resize(sourceNeighbors.size());
	sourcePaths.resize(sourceNeighbors.size());
	dirsAtSource.resize(sourceNeighbors.size());

	for (i = 0; i < sourceNeighbors.size(); i++)
	{
		m_pSparseSourceGeodesics->getGeodesicData(sourceIndex, sourceNeighbors[i], sourceDists[i], sourceGrads[i], dirsAtSource[i], &sourcePaths[i]);
	}

	//relax paths for accurate directions
	m_pSparseSourceGeodesics->setRelaxationSoftConstraint(0.01f);
	m_pSparseSourceGeodesics->relaxPaths(4, sourcePaths, sourceDists, sourceGrads, dirsAtSource);

	//set the given point
	setMatch(sourceIndex, targetPoint);

	sdir = sourceDir;
	sdir.normalize();
	tdir = targetDir;
	tdir.normalize();

	//save the starting directions on both source and target
	m_startSourceDir = sdir;
	m_startTargetDir = tdir;

	pmlsSource->project(pmlsSource->getDiscrPoint(sourceIndex), spt, snorm);
	bool PrSuccess = pmlsTarget->project(targetPoint, tpt, tnorm);
	if(!PrSuccess)
		debugOutput << "initMatchesFromPointAndDir:: project failed!\n";

	//probably completely unnecessary, but whatever, lets be overcautious
	snorm.normalize();
	tnorm.normalize();

	//now orthogonal vectors in tangent plane
	scross = snorm.crossProduct(sdir);
	scross.normalize();
	sdir = scross.crossProduct(snorm);
	sdir.normalize();
	tcross = tnorm.crossProduct(tdir);
	tcross.normalize();
	tdir = tcross.crossProduct(tnorm);
	tdir.normalize();

	//enumerate neighbors and compute directions projected into tangent plane of current point
	for (i = 0; i < sourceNeighbors.size(); i++)
	{
		sdirWalk = sourcePaths[i][1] - sourcePaths[i][0];
		sdirWalk.normalize();

		u = sdirWalk * sdir;
		v = sdirWalk * scross;
		tdirWalk = tdir * u + tcross * v;
		tdirWalk.normalize();

		if (walkIsometry(sourceIndex, sourceNeighbors[i], sourcePaths[i], targetPoint, tdirWalk, targetDirPoint))
		{
			setMatch(sourceNeighbors[i], targetDirPoint);
		}
	}

	//measure stretch of initial point set... may end up just debug, or not
	FloatType stretch = 0.f;
	FloatType totalSourceDist = 0.f;
	mpcard numMeasurements = 0;
	for (i = 0; i < sourceNeighbors.size(); i++)
	{
		if (isMatched(sourceNeighbors[i]))
		{
			mpcard j;
			for (j = 0; j < sourceNeighbors.size(); j++)
			{
				if (j == i)	continue;
				if (!isMatched(sourceNeighbors[j]))	continue;

				FloatType distSource, distTarget;
				VectorType gradSource, gradTarget, dirAtSource;
				vector<VectorType> path;
				mpcard tg;

				tg = lookupGeodesicMatch(sourceNeighbors[i], sourceNeighbors[j]);
				if (tg == AR4SparseGeodesics::m_InvalidIndex)
					continue;

				m_pSparseSourceGeodesics->getGeodesicData(sourceNeighbors[i], sourceNeighbors[j], distSource, gradSource, dirAtSource, &path);

				m_pSparseTargetGeodesics->getGeodesicData(tg, distTarget, gradTarget, &path);

				stretch += fabs(distTarget - distSource);
				totalSourceDist += distSource;
				numMeasurements++;
			}
		}
	}

}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::setMatch( mpcard sourceIndex, const VectorType &positionOnTarget )
{
	mpcard i, targGeo;
	vector<mpcard> sourceDests, matchedSourceDests, targetEntries;
	vector<float> dists, singleDist(1);
	vector<Vector3f> grads, singleGrad(1), dirsAtSource;
	vector<Vector3f> targetDests, singleDest(1);
	vector<vector<Vector3f>> paths, singlePath(1);	
	Vector3f grad, Dir;
	float dist;
	
	if (!matches[sourceIndex].matched)
	{
		//indicate that the point is matched and record location and order
		matches[sourceIndex].matched = true;
		matches[sourceIndex].targetPos = positionOnTarget;
		matches[sourceIndex].matchOrder = matchedSourceIndices.size();

		if (m_eGeodesicsMode == eAR4PMGMSparse)
		{
			m_pSparseSourceGeodesics->getGeodesicData(sourceIndex, sourceDests, dists, grads, &paths);
			FloatType distToStartMin = AR4SparseGeodesics::m_HugeDistance;
			for (i = 0; i < sourceDests.size(); i++)
			{
				if (isMatched(sourceDests[i]))
				{
					targetDests.push_back(matches[sourceDests[i]].targetPos);
					matchedSourceDests.push_back(sourceDests[i]);
					const FloatType distThruPoint = matches[sourceDests[i]].sourceDistToStart + dists[i];
					if (distThruPoint < distToStartMin)
						distToStartMin = distThruPoint;
				}
			}
			
			if (matchedSourceIndices.size() == 0)
				matches[sourceIndex].sourceDistToStart = (FloatType)0;
			else
				matches[sourceIndex].sourceDistToStart = distToStartMin;

			singleDest[0] = positionOnTarget;
			m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, positionOnTarget, targetDests, dists, grads, &paths);
			if (m_bUsePathRelaxation)
			{
				m_pSparseTargetGeodesics->setRelaxationSoftConstraint(0.01f);		
				m_pSparseTargetGeodesics->relaxPaths(m_nRelaxIters, paths, dists, grads, dirsAtSource);
			}
			for (i = 0; i < targetDests.size(); i++)
			{
				targGeo = m_pSparseTargetGeodesics->storeGeodesicData(positionOnTarget, targetDests[i], dists[i], grads[i], &paths[i]);
				addGeodesicMatch(sourceIndex, matchedSourceDests[i], targGeo);

				//the following is fucking inefficient, but it's not clear how to avoid it because of how gradient is computed
				m_pSparseTargetGeodesics->computeGeodesicSingleDest(m_geodSigma, targetDests[i], positionOnTarget, dist, grad, &singlePath[0]);
				if (m_bUsePathRelaxation)
					m_pSparseTargetGeodesics->relaxPath(m_nRelaxIters, singlePath[0], dist, grad, Dir);
				targGeo = m_pSparseTargetGeodesics->storeGeodesicData(targetDests[i], positionOnTarget, dist, grad, &singlePath[0]);
				addGeodesicMatch(matchedSourceDests[i], sourceIndex, targGeo);
			}
		}

		matchedSourceIndices.push_back(sourceIndex);


	}
	else //already matched
	{
		if (matches[sourceIndex].fixed)
			warning("AR4PartialMatch<ManifoldTraits>::setMatch(...): setting a match that is fixed (" + intToStr(sourceIndex) + ")");

		//update target position
		matches[sourceIndex].targetPos = positionOnTarget;

		//indicate that target geodesics to&from this match will need to be updated
		matchesForGeodesicUpdate.insert(sourceIndex);

		//mark that this point has been changed in case of back-up
		dirtyMatchBackup(sourceIndex);
	}


}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::setMatchLimitedUpdate(mpcard sourceIndex, const VectorType &positionOnTarget, const vector<mpcard>& sourceDests, const vector<mpcard>& targGeosAsStart, const vector<mpcard>& targGeosAsEnd)
{
	mpcard i, j, tg;
	float dist;
	Vector3f grad;
	vector<Vector3f> path;
	vector<float> dists, singleDist(1);
	vector<Vector3f> grads, singleGrad(1);
	vector<Vector3f> targetDests, singleDest(1);
	vector<vector<Vector3f>> paths, singlePath(1);	

	if (!matches[sourceIndex].matched)
		throw ERangeCheck("(...): only valid to update an existing match; source index " + intToStr(sourceIndex) + " is not matched yet");

	if (matches[sourceIndex].fixed)
		warning("AR4PartialMatch<ManifoldTraits>::setMatchLimitedUpdate(...): setting a match that is fixed (" + intToStr(sourceIndex) + ")");

	matches[sourceIndex].targetPos = positionOnTarget;

	dirtyMatchBackup(sourceIndex);

	//collect end points from pre-existing paths to run Dijkstra's only once
	for (i = 0; i < targGeosAsStart.size(); i++)
	{
		if (sourceDests[i] == sourceIndex)
			continue;
		targetDests.push_back(matches[sourceDests[i]].targetPos);
	}

	//in case we are using relaxation
	m_pSparseTargetGeodesics->setRelaxationSoftConstraint(0.01f);

	//Dijkstra
	m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, positionOnTarget, targetDests, dists, grads, &paths);
	if (m_bUsePathRelaxation)
		m_pSparseTargetGeodesics->relaxPaths(paths);

	//store again
	j = 0;
	for (i = 0; i < targGeosAsStart.size(); i++)
	{
		if (sourceDests[i] == sourceIndex)
			continue;
		tg = targGeosAsStart[i];
		m_pSparseTargetGeodesics->updateGeodesicData(tg, positionOnTarget, targetDests[j], dists[j], grads[j], &paths[j]);
		j++;
	}

	//now for other (reverse/incoming) directions, have to compute individually
	singleDest[0] = positionOnTarget;
	for (i = 0; i < targGeosAsEnd.size(); i++)
	{
		if (sourceDests[i] == sourceIndex)
			continue;
		tg = targGeosAsEnd[i];
		m_pSparseTargetGeodesics->computeGeodesicSingleDest(m_geodSigma, matches[sourceDests[i]].targetPos, positionOnTarget, singleDist[0], singleGrad[0], &singlePath[0]);
		if (m_bUsePathRelaxation)
			m_pSparseTargetGeodesics->relaxPath(singlePath);
		m_pSparseTargetGeodesics->updateGeodesicData(tg, matches[sourceDests[i]].targetPos, positionOnTarget, singleDist[0], singleGrad[0], &singlePath[0]);
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::setMatchLimitedUpdate(mpcard sourceIndex, const VectorType& positionOnTarget)
{
	mpcard i, j, n;
	mpcard mlevel;
	mpcard tg;
	vector<mpcard> targGeosOut, targGeosIn;
	vector<Vector3f> targDests, grads, singleDest(1);
	vector<float> dists;
	vector<vector<Vector3f>> paths;

	if (!matches[sourceIndex].matched)
		throw ERangeCheck("(...): only valid to update an existing match; source index " + intToStr(sourceIndex) + " is not matched yet");

	if (matches[sourceIndex].fixed)
		warning("AR4PartialMatch<ManifoldTraits>::setMatchLimitedUpdate(...): setting a match that is fixed (" + intToStr(sourceIndex) + ")");

	if (m_nSampleLevels < 2)
		mlevel = 0;
	else
		mlevel = 1;

	matches[sourceIndex].targetPos = positionOnTarget;

	dirtyMatchBackup(sourceIndex);

	//collect matched neighbors and geodesic entry indices
	n = m_ppTopoGraphSource[mlevel]->getNumAdjacentVertices(sourceIndex);
	for (i = 0; i < n; i++)
	{
		j = m_ppTopoGraphSource[mlevel]->getAdjacentVertex(sourceIndex, i);
		if (matches[j].matched)
		{
			tg = lookupGeodesicMatch(sourceIndex, j);
			if (tg != AR4SparseGeodesics::m_InvalidIndex)
			{
				targDests.push_back(matches[j].targetPos);
				targGeosOut.push_back(tg);
			}

			tg = lookupGeodesicMatch(j, sourceIndex);
			if (tg != AR4SparseGeodesics::m_InvalidIndex)
			{
				targGeosIn.push_back(tg);
			}
		}
	}

	if (targGeosIn.size() != targGeosOut.size())
		warning("AR4PartialMatch<ManifoldTraits>::setMatchLimitedUpdate(...): target surface geodesics are not symmetric");

	//in case we are using path relaxation
	m_pSparseTargetGeodesics->setRelaxationSoftConstraint(0.01f);

	//re-compute and re-store outgoing goedesics (from this point to neighbors)
	m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, positionOnTarget, targDests, dists, grads, &paths);
	if (m_bUsePathRelaxation)
		m_pSparseTargetGeodesics->relaxPaths(paths, dists, grads);
	for (i = 0; i < targDests.size(); i++)
	{
		tg = targGeosOut[i];
		m_pSparseTargetGeodesics->updateGeodesicData(tg, positionOnTarget, targDests[i], dists[i], grads[i], &paths[i]);
	}

	//re-compute and re-store incoming geodesics (from neighbors to this point)
	//have to be computed individually
	singleDest[0] = positionOnTarget;
	dists.resize(1);
	grads.resize(1);
	paths.resize(1);
	for (i = 0; i < targDests.size(); i++)
	{
		tg = targGeosIn[i];
//		m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, targDests[i], singleDest, dists, grads, &paths);
		m_pSparseTargetGeodesics->computeGeodesicSingleDest(m_geodSigma, targDests[i], positionOnTarget, dists[0], grads[0], &paths[0]);
		if (m_bUsePathRelaxation)
			m_pSparseTargetGeodesics->relaxPath(paths[0], dists[0], grads[0]);
		m_pSparseTargetGeodesics->updateGeodesicData(tg, targDests[i], positionOnTarget, dists[0], grads[0], &paths[0]);
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::setMatchWithGeoData(mpcard sourceIndex, const VectorType &positionOnTarget, const vector<mpcard>& connectIndices, const vector<mpcard>& toTargGeos, const vector<mpcard>& fromTargGeos)
{

	mpcard i, targGeo;
	vector<mpcard> sourceDests, matchedSourceDests;
	vector<float> dists;
	vector<Vector3f> grads, targetDests, singleDest(1);
	vector<vector<Vector3f>> paths;

	if (toTargGeos.size() != connectIndices.size() || fromTargGeos.size() != connectIndices.size())
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::setMatchWithGeoData(...): conncetIndices, toTargGeos and fromTargGeos must all have the same size");


	
	if (!matches[sourceIndex].matched)
	{
		matches[sourceIndex].matched = true;
		matches[sourceIndex].matchOrder = matchedSourceIndices.size();
		matchedSourceIndices.push_back(sourceIndex);
	}



	if (matches[sourceIndex].fixed)
		warning("AR4PartialMatch<ManifoldTraits>::setMatchWithGeoData(...): setting a match that is fixed (" + intToStr(sourceIndex) + ")");
	

	matches[sourceIndex].targetPos = positionOnTarget;


	FloatType distToStartMin = AR4SparseGeodesics::m_HugeDistance;

	for (i = 0; i < connectIndices.size(); i++)
	{
		if (!isMatched(connectIndices[i]))
		{
			warning("AR4PartialMatch<ManifoldTraits>::setMatchWithGeoData(...): inconsistency between point-wise matches and pair-wise matches");
			continue;
		}

		FloatType connectDist;
		VectorType connectGrad, dirAtSource;
		m_pSparseSourceGeodesics->getGeodesicData(sourceIndex, connectIndices[i], connectDist, connectGrad, dirAtSource);

		const FloatType distThruPoint = matches[connectIndices[i]].sourceDistToStart + connectDist;
		if (distThruPoint < distToStartMin)
			distToStartMin = distThruPoint;

		targGeo = lookupGeodesicMatch(sourceIndex, connectIndices[i]);
		if (targGeo == AR4SparseGeodesics::m_InvalidIndex)
			addGeodesicMatch(sourceIndex, connectIndices[i], toTargGeos[i]);
		else if (targGeo != toTargGeos[i])
			warning("AR4PartialMatch<ManifoldTraits>::setMatchWithGeoData(...): target geodesic between matches of " + intToStr(sourceIndex) + " and " + intToStr(connectIndices[i]) + " redundantly computed");

		targGeo = lookupGeodesicMatch(connectIndices[i], sourceIndex);
		if (targGeo == AR4SparseGeodesics::m_InvalidIndex)
			addGeodesicMatch(connectIndices[i], sourceIndex, fromTargGeos[i]);
		else if (targGeo != fromTargGeos[i])
			warning("AR4PartialMatch<ManifoldTraits>::setMatchWithGeoData(...): target geodesic between matches of " + intToStr(connectIndices[i]) + " and " + intToStr(sourceIndex) + " redundantly computed");
	}

	matches[sourceIndex].sourceDistToStart = distToStartMin;

	
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::updateTargetGeodesics()
{
	mpcard j, tgi, tgo;
	set<mpcard>::iterator mIt;
	mpcard sourceIndex;
	vector<mpcard> sourceDests, targGeosOut, singleTargGeos;
	vector<float> dists, singleDist(1);
	vector<Vector3f> grads, targetDests, singleDest(1), singleGrad(1);
	vector<vector<Vector3f>> paths, singlePath(1);
	vector<Vector3f> singleStarts, singleDests;

	set<mpcard> updatedGeos;

	//in case we are using path relaxation
	m_pSparseTargetGeodesics->setRelaxationSoftConstraint(0.01f);

	for (mIt = matchesForGeodesicUpdate.begin(); mIt != matchesForGeodesicUpdate.end(); mIt++)
	{
		sourceIndex = *mIt;

		targetDests.clear();
		targGeosOut.clear();

		m_pSparseSourceGeodesics->getGeodesicData(sourceIndex, sourceDests, dists, grads);
		for (j = 0; j < sourceDests.size(); j++)
		{
			//for the available geodesic destinations, check that
			if (matches[sourceDests[j]].matched) //the point is matched
			{
				if ((tgo = lookupGeodesicMatch(sourceIndex, sourceDests[j])) != AR4SparseGeodesics::m_InvalidIndex && //the geodesic on the target has been computed
					updatedGeos.find(tgo) == updatedGeos.end()) //the geodesic has not already been updated
				{
					targetDests.push_back(matches[sourceDests[j]].targetPos);
					targGeosOut.push_back(tgo);
				}

				if ((tgi = lookupGeodesicMatch(sourceDests[j], sourceIndex)) != AR4SparseGeodesics::m_InvalidIndex && //the reverse geodesic on the target has been computed
					updatedGeos.find(tgi) == updatedGeos.end()) //but has not been updated yet
				{
					singleTargGeos.push_back(tgi);
					singleStarts.push_back(matches[sourceDests[j]].targetPos);
					singleDests.push_back(matches[sourceIndex].targetPos);
				}
			}
		}

		if (m_bUsePathRelaxation)
		{
			for (j = 0; j < targetDests.size(); j++)
			{
				//update the path by moving the end points and relaxing the intermediate points
				m_pSparseTargetGeodesics->updateEndPointsAndRelax(m_nRelaxIters, targGeosOut[j], matches[sourceIndex].targetPos, targetDests[j]);

				//and make a note of it
				updatedGeos.insert(targGeosOut[j]);
			}
		}
		else
		{
			//run Dijkstra from current target point
			m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, matches[sourceIndex].targetPos, targetDests, dists, grads, &paths);

			for (j = 0; j < targetDests.size(); j++)
			{
				//update each of the geodesic on the target from current point to others
				m_pSparseTargetGeodesics->updateGeodesicData(targGeosOut[j], matches[sourceIndex].targetPos, targetDests[j], dists[j], grads[j], &paths[j]);

				//and make a note of it
				updatedGeos.insert(targGeosOut[j]);
			}
		}
	}

	//update any left over singletons that were not covered above
	for (j = 0; j < singleTargGeos.size(); j++)
	{
		tgi = singleTargGeos[j];
		if (updatedGeos.find(tgi) == updatedGeos.end())
		{
			//this geodesic did not get updated above, do it as singleton
			singleDest[0] = singleDests[j];
			if (m_bUsePathRelaxation)
			{
				m_pSparseTargetGeodesics->updateEndPointsAndRelax(m_nRelaxIters, tgi, singleStarts[j], singleDests[j]);
			}
			else
			{
//				m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, singleStarts[j], singleDest, singleDist, singleGrad, &singlePath);
				m_pSparseTargetGeodesics->computeGeodesicSingleDest(m_geodSigma, singleStarts[j], singleDest[0], singleDist[0], singleGrad[0], &singlePath[0]);
				m_pSparseTargetGeodesics->updateGeodesicData(tgi, singleStarts[j], singleDest[0], singleDist[0], singleGrad[0], &singlePath[0]);
			}

			//now make a note that it has been update (probably unnecessary)
			updatedGeos.insert(tgi);
		}
	}

	//now clear
	matchesForGeodesicUpdate.clear();
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::updateTargetGeodesicsLimited()
{
	mpcard j, k, n, tgi, tgo;
	set<mpcard>::iterator mIt;
	mpcard sourceIndex;
	vector<mpcard> targGeosOut, singleTargGeos;
	vector<float> dists, singleDist(1);
	vector<Vector3f> grads, targetDests, singleDest(1), singleGrad(1);
	vector<vector<Vector3f>> paths, singlePath(1);
	vector<Vector3f> singleStarts, singleDests;

	set<mpcard> updatedGeos;

	mpcard mlevel;
	if (m_nSampleLevels < 2)
		mlevel = 0;
	else
		mlevel = 1;

	//in case we are using path relaxation
	m_pSparseTargetGeodesics->setRelaxationSoftConstraint(0.01f);

	for (mIt = matchesForGeodesicUpdate.begin(); mIt != matchesForGeodesicUpdate.end(); mIt++)
	{
		sourceIndex = *mIt;

		targetDests.clear();
		targGeosOut.clear();

		n = m_ppTopoGraphSource[mlevel]->getNumAdjacentVertices(sourceIndex);
		for (j = 0; j < n; j++)
		{
			k = m_ppTopoGraphSource[mlevel]->getAdjacentVertex(sourceIndex, j);

			//for the available geodesic destinations, check that
			if (matches[k].matched) //the point is matched
			{
				if ((tgo = lookupGeodesicMatch(sourceIndex, k)) != AR4SparseGeodesics::m_InvalidIndex && //the geodesic on the target has been computed
					updatedGeos.find(tgo) == updatedGeos.end()) //the geodesic has not already been updated
				{
					targetDests.push_back(matches[k].targetPos);
					targGeosOut.push_back(tgo);
				}

				if ((tgi = lookupGeodesicMatch(k, sourceIndex)) != AR4SparseGeodesics::m_InvalidIndex && //the reverse geodesic on the target has been computed
					updatedGeos.find(tgi) == updatedGeos.end()) //but has not been updated yet
				{
					singleTargGeos.push_back(tgi);
					singleStarts.push_back(matches[k].targetPos);
					singleDests.push_back(matches[sourceIndex].targetPos);
				}
			}
		}

		if (m_bUsePathRelaxation)
		{
			for (j = 0; j < targetDests.size(); j++)
			{
				//update path and distance via relaxation
				m_pSparseTargetGeodesics->updateEndPointsAndRelax(m_nRelaxIters, targGeosOut[j], matches[sourceIndex].targetPos, targetDests[j]);

				//and make a note of it
				updatedGeos.insert(targGeosOut[j]);
			}
		}
		else
		{
			//run Dijkstra from current target point
			m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, matches[sourceIndex].targetPos, targetDests, dists, grads, &paths);

			for (j = 0; j < targetDests.size(); j++)
			{
				//update each of the geodesic on the target from current point to others
				m_pSparseTargetGeodesics->updateGeodesicData(targGeosOut[j], matches[sourceIndex].targetPos, targetDests[j], dists[j], grads[j], &paths[j]);

				//and make a note of it
				updatedGeos.insert(targGeosOut[j]);
			}
		}
	}

	//update any left over singletons that were not covered above
	for (j = 0; j < singleTargGeos.size(); j++)
	{
		tgi = singleTargGeos[j];
		if (updatedGeos.find(tgi) == updatedGeos.end())
		{
			//this geodesic did not get updated above, do it as singleton
			singleDest[0] = singleDests[j];
			if (m_bUsePathRelaxation)
			{
				m_pSparseTargetGeodesics->updateEndPointsAndRelax(m_nRelaxIters, tgi, singleStarts[j], singleDests[j]);
			}
			else
			{
				m_pSparseTargetGeodesics->computeGeodesicSingleDest(m_geodSigma, singleStarts[j], singleDest[0], singleDist[0], singleGrad[0], &singlePath[0]);
				m_pSparseTargetGeodesics->updateGeodesicData(tgi, singleStarts[j], singleDest[0], singleDist[0], singleGrad[0], &singlePath[0]);
			}

			//now make a note that it has been update (probably unnecessary)
			updatedGeos.insert(tgi);
		}
	}

	//now clear
	matchesForGeodesicUpdate.clear();
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::validatePaths()
{
	AR4PM_START_PROFILE(m_profFullValidate)

	mpcard i, j, k, tg;
	mpcard startMatch;
	vector<mpcard> sourceDests, targetGeos, targetGeosReverse;
	vector<float> dists;
	vector<Vector3f> grads, targetDests;
	vector<vector<Vector3f>> paths;
	float dist;
	Vector3f grad, vmatch;
	bool bvalid;
	AAT pos, idx, col;
	PointSet* pMatchPS;
	PointSet* pMatchSrcPS;

	set<mpcard> testedTargGeos;

	//update target match point cloud and re-setup knn query for validation on target
	startMatch = getLastMatchCohortCount();
	if (m_pTargetMatchCloud->getNumPoints() == 0)
	{
		//create and initially populate point clouds
		m_pTargetMatchCloud->clearAndSetup(&m_vdMatch, matchedSourceIndices.size());
		m_pSourceMatchCloud->clearAndSetup(&m_vdMatch, matchedSourceIndices.size());

		pMatchPS = m_pTargetMatchCloud->getPointSet();
		pMatchSrcPS = m_pSourceMatchCloud->getPointSet();

		pos = m_pTargetMatchCloud->getAAT("position");
		idx = m_pTargetMatchCloud->getAAT(m_strSourceIndexName);
		col = m_pTargetMatchCloud->getAAT("color");
		for (i = 0; i < matchedSourceIndices.size(); i++)
		{
			k = matchedSourceIndices[i];
			vmatch = matches[k].targetPos;

			pMatchPS->set3f(i, pos, vmatch);
			pMatchPS->set1u(i, idx, (card32)k);
			pMatchPS->set3f(i, col, makeVector3f(0.f, 0.f, 0.f));

			pMatchSrcPS->set3f(i, pos, source->getDiscrPoint(k));
			pMatchSrcPS->set1u(i, idx, (card32)k);
			pMatchSrcPS->set3f(i, col, makeVector3f(0.f, 0.f, 0.f));
		}
	}
	else
	{
		char* pbuffer = new char[m_vdMatch.getSize()];

		pMatchPS = m_pTargetMatchCloud->getPointSet();
		pMatchSrcPS = m_pSourceMatchCloud->getPointSet();

		//add new matches to point cloud
		for (i = startMatch; i < matchedSourceIndices.size(); i++)
		{
			k = matchedSourceIndices[i];
			vmatch = matches[k].targetPos;

			*((Vector3f*)(pbuffer + m_vdMatch.getAttributeOffset(m_vdMatch.getIndex("position")))) = vmatch;
			*((card32*)(pbuffer + m_vdMatch.getAttributeOffset(m_vdMatch.getIndex(m_strSourceIndexName)))) = k;
			*((Vector3f*)(pbuffer + m_vdMatch.getAttributeOffset(m_vdMatch.getIndex("color")))) = makeVector3f(0.f, 0.f, 0.f);
			pMatchPS->addPoint(pbuffer);

			*((Vector3f*)(pbuffer + m_vdMatch.getAttributeOffset(m_vdMatch.getIndex("position")))) = source->getDiscrPoint(k);
			*((card32*)(pbuffer + m_vdMatch.getAttributeOffset(m_vdMatch.getIndex(m_strSourceIndexName)))) = k;
			*((Vector3f*)(pbuffer + m_vdMatch.getAttributeOffset(m_vdMatch.getIndex("color")))) = makeVector3f(0.f, 0.f, 0.f);
			pMatchSrcPS->addPoint(pbuffer);
		}

		delete [] pbuffer;
	}
	
	//reset knn query (re-build octree)
	forceResetTargetMatchKNNQuery();
	forceResetSourceMatchKNNQuery();

	//now validate all paths
	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		//retrieve matched source point
		k = matchedSourceIndices[i];

		//retrieve available geodesics
		m_pSparseSourceGeodesics->getGeodesicData(k, sourceDests, dists, grads, &paths);
		for (j = 0; j < sourceDests.size(); j++)
		{
			if (matches[sourceDests[j]].matched)
			{
				//only interested in paths to other matches
				//retrieve look-up index for target path and reverse target path, and remember both
				tg = lookupGeodesicMatch(k, sourceDests[j]);
				if (tg != AR4SparseGeodesics::m_InvalidIndex)
				{
					if (testedTargGeos.find(tg) == testedTargGeos.end())
						targetGeos.push_back(tg);
					tg = lookupGeodesicMatch(sourceDests[j], k);
					if (testedTargGeos.find(tg) == testedTargGeos.end())
						targetGeosReverse.push_back(tg);
				}

				if (!m_pSparseSourceGeodesics->isPathValid(k, sourceDests[j]))
				{
					//once a source path becomes valid, it will remain valid, because source points do not move
					bvalid = validateSourcePath(paths[j]);
					m_pSparseSourceGeodesics->setPathValidity(k, sourceDests[j], bvalid);
					m_pSparseSourceGeodesics->setPathValidity(sourceDests[j], k, bvalid);//validity is symmetric
				}
			}
		}

		//now for the target surface
		paths.resize(1);//target paths are processed individually
		for (j = 0; j < targetGeos.size(); j++)
		{
			//geodesic look-up index
			tg = targetGeos[j];

			//retrieve path
			m_pSparseTargetGeodesics->getGeodesicData(tg, dist, grad, &paths[0]);

			if (!m_pSparseTargetGeodesics->isPathValid(tg))
			{
				//in general, a target path may go from valid to invalid if the point moves a lot, but this is very unlikely (I think)
				//so for now assume we only have to validate target paths once
				bvalid = validateTargetPath(paths[0]);
				m_pSparseTargetGeodesics->setPathValidity(tg, bvalid);
				tg = targetGeosReverse[j];
				m_pSparseTargetGeodesics->setPathValidity(tg, bvalid);//validity is symmetric
			}

			testedTargGeos.insert(targetGeos[j]);
			testedTargGeos.insert(targetGeosReverse[j]);
		}
	}

	AR4PM_STOP_PROFILE(m_profFullValidate)
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::checkForNewGeodesicConnections()
{
	mpcard i, j, k, tg, matchStart;
	vector<mpcard> sourceDests, matchedSourceDests;
	vector<float> dists;
	vector<Vector3f> grads, targetDests, singleDest(1);
	vector<vector<Vector3f>> paths;
	vector<pair<mpcard, mpcard>> singletonGeos;

	matchStart = getLastMatchCohortCount();
	
	for (i = matchStart; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];

		targetDests.clear();
		matchedSourceDests.clear();

		m_pSparseSourceGeodesics->getGeodesicData(k, sourceDests, dists, grads);
		for (j = 0; j < sourceDests.size(); j++)
		{
			if (matches[sourceDests[j]].matched)
			{
				tg = lookupGeodesicMatch(k, sourceDests[j]);
				if (tg == AR4SparseGeodesics::m_InvalidIndex)
				{
					targetDests.push_back(matches[sourceDests[j]].targetPos);
					matchedSourceDests.push_back(sourceDests[j]);
				}

				tg = lookupGeodesicMatch(sourceDests[j], k);
				if (tg == AR4SparseGeodesics::m_InvalidIndex)
					singletonGeos.push_back(pair<mpcard, mpcard>(sourceDests[j], k));
			}
		}

		m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, matches[k].targetPos, targetDests, dists, grads, &paths);
		for (j = 0; j < targetDests.size(); j++)
		{
			tg = m_pSparseTargetGeodesics->storeGeodesicData(matches[k].targetPos, targetDests[j], dists[j], grads[j], &paths[j]);
			addGeodesicMatch(k, matchedSourceDests[j], tg);
		}
	}

	//now process any singletons we have left over
	dists.resize(1);
	grads.resize(1);
	paths.resize(1);
	for (i = 0; i < singletonGeos.size(); i++)
	{
		//check to see if we didn't already compute it above
		tg = lookupGeodesicMatch(singletonGeos[i].first, singletonGeos[i].second);
		if (tg == AR4SparseGeodesics::m_InvalidIndex)
		{
			singleDest[0] = matches[singletonGeos[i].second].targetPos;
			m_pSparseTargetGeodesics->computeGeodesicSingleDest(m_geodSigma, matches[singletonGeos[i].first].targetPos, singleDest[0], dists[0], grads[0], &paths[0]);
			tg = m_pSparseTargetGeodesics->storeGeodesicData(matches[singletonGeos[i].first].targetPos, singleDest[0], dists[0], grads[0], &paths[0]);
			addGeodesicMatch(singletonGeos[i].first, singletonGeos[i].second, tg);
		}
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::findMatchedValidIndirectPath(mpcard sourceIndex, mpcard endIndex, float& dist, vector<mpcard>& path, vector<mpcard>& targetGeos)
{
	mpcard level, nedges, tg;
	InCorePCTopologyGraph::EdgeIndex ei;
	PointSet* pEdges;
	AAT weightAAT, vindexAAT;
	vector<float> savedWeights;
	float weight;
	Vector2i vidx;

	for (level = m_nSampleLevels; level >= 1; level--)
	{
		pEdges = m_ppTopoGraphSource[level]->getEdges();
		nedges = pEdges->getNumEntries();
		weightAAT = pEdges->getAAT("weight");
		vindexAAT = pEdges->getAAT("vindex");

		savedWeights.resize(nedges);

		for (ei = 0; ei < nedges; ei++)
		{
			weight = pEdges->get1f(ei, weightAAT);
			vidx = pEdges->get2i(ei, vindexAAT);

			//save weights, since we don't want to permanently modify them
			savedWeights[ei] = weight;

			//add huge value to paths that:
			// - have un-matched end points
			// - do not have a corresponding path computed on target surface
			// - are not valid on either source or target surface
			if (!isMatched(vidx[0]) || !isMatched(vidx[1]) || 
				(tg = lookupGeodesicMatch(vidx[0], vidx[1])) == AR4SparseGeodesics::m_InvalidIndex || 
				m_pSparseSourceGeodesics->isPathValid(vidx[0], vidx[1]) || m_pSparseTargetGeodesics->isPathValid(tg))
			{
				weight += AR4SparseGeodesics::m_HugeDistance;
				pEdges->set1f(ei, weightAAT, weight);
			}
		}

		//compute minimum weight path
		m_pSparseSourceGeodesics->computeMinWeightPath(m_ppTopoGraphSource[level], sourceIndex, endIndex, dist, path);

		//reset edge weights
		for (ei = 0; ei < nedges; ei++)
		{
			pEdges->set1f(ei, weightAAT, savedWeights[ei]);
		}

		//if we have found a path without "infinite" weight (essentially, if we found a path)
		//then return it with the corresponding path-entries on the target surface
		if (dist < AR4SparseGeodesics::m_HugeDistance)
		{
			mpcard j;
			targetGeos.resize(path.size() - 1);
			for (j = 1; j < path.size(); j++)
				targetGeos[j - 1] = lookupGeodesicMatch(path[j - 1], path[j]);
			return;
		}
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::removeOutliersLastCohort()
{
	AR4PM_START_PROFILE(m_profRemoveOutliers)

	mpcard i, j, k, m, costart, remtotal;
	FloatType err, threshAbs;

	set<mpcard> measureSourceSet;

	if (m_avgEdgeWeight == (FloatType)0)
		m_avgEdgeWeight = computeAvgSourceMatchLevelEdgeWeight();
	threshAbs = m_stretchThresh * m_avgEdgeWeight;

	costart = getLastMatchCohortCount();
	remtotal = 0;
	if (m_bRecordRejectedOutliers)
	{
		for (i = costart; i < matchedSourceIndices.size(); i++)
		{
			k = matchedSourceIndices[i];
			measureSourceSet.clear();
			err = computeErrorSingleMatch(k, measureSourceSet);
			if (err > threshAbs)
			{
				//un-flag match
				matches[k].matched = false;
				matches[k].fixed = false;

				//increment number of times we have to pop
				remtotal++;

				RejectedOutlier outlier;
				pair<set<RejectedOutlier>::iterator, bool> retval;
				outlier.m_sourceIndex = k;
				outlier.m_targetPos = matches[k].targetPos;
				outlier.m_stretch = err;
				outlier.m_rejectors = measureSourceSet;
				retval = m_rejectedOutliers.insert(outlier);
				if (!retval.second)
				{
					// mw - this is a totally evil hack. 
					// The code initial code was wrong in terms of the C++ type system (it did not compile on my VS 2010);
					// I fix it in-situ to keep the semantics. I think it should work, because the elment ordering is not
					// altered by changing additional fields (the reference returned by the iterator should be immutable).
					// original code:
					// retval.first->m_targetPos = matches[k].targetPos;
					// retval.first->m_stretch = err;
					// retval.first->m_rejectors = measureSourceSet;
					// new version:
					RejectedOutlier *entryPtr = const_cast<RejectedOutlier*>(&(*(retval.first)));
					entryPtr->m_targetPos = matches[k].targetPos;
					entryPtr->m_stretch = err;
					entryPtr->m_rejectors = measureSourceSet;
				}
			}
			else
			{
				//down shift by the total to be removed so far
				j = i - remtotal;
				matches[k].matchOrder = j;
				matchedSourceIndices[j] = k;
			}
		}
	}
	else
	{
		for (i = costart; i < matchedSourceIndices.size(); i++)
		{
			k = matchedSourceIndices[i];
			err = computeErrorSingleMatch(k);
			if (err > threshAbs)
			{
				//un-flag match
				matches[k].matched = false;
				matches[k].fixed = false;

				//increment number of times we have to pop
				remtotal++;
			}
			else
			{
				//down shift by the total to be removed so far
				j = i - remtotal;
				matches[k].matchOrder = j;
				matchedSourceIndices[j] = k;
			}
		}
	}

	//shrink vector
	for (j = 0; j < remtotal; j++)
		matchedSourceIndices.pop_back();

	AR4PM_STOP_PROFILE(m_profRemoveOutliers)

#if AR4_DEBUG_OUTPUT
	debugOutput << "removed " << remtotal << " outliers\n";
	debugOutput << "using threshold " << threshAbs << "\n";
#endif //AR4_DEBUG_OUTPUT

}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getRejectInfo(mpcard sourceIndex, VectorType& targetPoint, FloatType& stretch, vector<mpcard>& sourceMeasurePoints)
{
	RejectedOutlier roQuery;
	set<RejectedOutlier>::iterator roIt;
	set<mpcard>::iterator it;

	sourceMeasurePoints.clear();

	if (m_bRecordRejectedOutliers)
	{
		roQuery.m_sourceIndex = sourceIndex;
		roIt = m_rejectedOutliers.find(roQuery);
		if (roIt != m_rejectedOutliers.end())
		{
			targetPoint = roIt->m_targetPos;
			stretch = roIt->m_stretch;
			for (it = roIt->m_rejectors.begin(); it != roIt->m_rejectors.end(); it++)
				sourceMeasurePoints.push_back(*it);
		}
		else
		{
			stretch = -1.f;
		}
	}
	else
	{
		stretch = -1.f;
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::resetMatchCloud()
{
	mpcard i, k;

	//this condition ensures that we only do this if outliers have been removed
	if (matchedSourceIndices.size() < m_pTargetMatchCloud->getNumPoints())
	{
		m_pTargetMatchCloud->clearAndSetup(&m_vdMatch, matchedSourceIndices.size());
		m_pSourceMatchCloud->clearAndSetup(&m_vdMatch, matchedSourceIndices.size());

		PointSet* pMatchPS = m_pTargetMatchCloud->getPointSet();
		PointSet* pMatchSrcPS = m_pSourceMatchCloud->getPointSet();

		AAT pos = m_pTargetMatchCloud->getAAT("position");
		AAT idx = m_pTargetMatchCloud->getAAT(m_strSourceIndexName);
		AAT col = m_pTargetMatchCloud->getAAT("color");

		for (i = 0; i < matchedSourceIndices.size(); i++)
		{
			k = matchedSourceIndices[i];

			pMatchPS->set3f(i, pos, matches[k].targetPos);
			pMatchPS->set1u(i, idx, (card32)k);
			pMatchPS->set3f(i, col, makeVector3f(0.f, 0.f, 0.f));

			pMatchSrcPS->set3f(i, pos, source->getDiscrPoint(k));
			pMatchSrcPS->set1u(i, idx, (card32)k);
			pMatchSrcPS->set3f(i, col, makeVector3f(0.f, 0.f, 0.f));
		}

		//reset knn query (re-build octree)
		forceResetTargetMatchKNNQuery();
		forceResetSourceMatchKNNQuery();
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceNeighborhood(const mpcard sourceIndex, const mpcard numRings, vector<mpcard>& neighbors)
{
	mpcard i, vert, numNbrs, nbr;
	pair<mpcard, mpcard> nbrdepth;
	queue<pair<mpcard, mpcard>> nbrsToVisit;
	vector<bool> visited(source->getNumDiscrPoints(), false);

	neighbors.clear();

	nbrdepth.first = sourceIndex;
	nbrdepth.second = 0;
	nbrsToVisit.push(nbrdepth);
	visited[sourceIndex] = true;
	while (!nbrsToVisit.empty())
	{
		nbrdepth = nbrsToVisit.front();
		nbrsToVisit.pop();

		vert = nbrdepth.first;
		if (vert != sourceIndex)
			neighbors.push_back(vert);

		if (nbrdepth.second < numRings)
		{
			nbrdepth.second++;
			numNbrs = m_ppTopoGraphSource[0]->getNumAdjacentVertices(vert);
			for (i = 0; i < numNbrs; i++)
			{
				nbr = m_ppTopoGraphSource[0]->getAdjacentVertex(vert, i);
				if (!visited[nbr])
				{
					nbrdepth.first = nbr;
					visited[nbr] = true;
					nbrsToVisit.push(nbrdepth);
				}
			}
		}
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceNeighborhood(const mpcard sourceIndex, const mpcard level, const mpcard numRings, vector<mpcard>& neighbors)
{
	mpcard i, vert, numNbrs, nbr;
	pair<mpcard, mpcard> nbrdepth;
	queue<pair<mpcard, mpcard>> nbrsToVisit;
	vector<bool> visited(source->getNumDiscrPoints(), false);

	neighbors.clear();

	if (level >= m_nSampleLevels)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getSourceNeighborhood(...): invalid topology level");

	nbrdepth.first = sourceIndex;
	nbrdepth.second = 0;
	nbrsToVisit.push(nbrdepth);
	visited[sourceIndex] = true;
	while (!nbrsToVisit.empty())
	{
		nbrdepth = nbrsToVisit.front();
		nbrsToVisit.pop();

		vert = nbrdepth.first;
		if (vert != sourceIndex)
			neighbors.push_back(vert);

		if (nbrdepth.second < numRings)
		{
			nbrdepth.second++;
			numNbrs = m_ppTopoGraphSource[level]->getNumAdjacentVertices(vert);
			for (i = 0; i < numNbrs; i++)
			{
				nbr = m_ppTopoGraphSource[level]->getAdjacentVertex(vert, i);
				if (!visited[nbr])
				{
					nbrdepth.first = nbr;
					visited[nbr] = true;
					nbrsToVisit.push(nbrdepth);
				}
			}
		}
	}
}

template<class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::getSourceMatchingLevelNeighborhood(mpcard sourceIndex, mpcard rings, vector<mpcard>& neighbors)
{
	mpcard i, vert, numNbrs, nbr;
	pair<mpcard, mpcard> nbrdepth;
	queue<pair<mpcard, mpcard>> nbrsToVisit;
	vector<bool> visited(source->getNumDiscrPoints(), false);

	mpcard mlevel;
	if (m_nSampleLevels < 2)
		mlevel = 0;
	else
		mlevel = 1;

	neighbors.clear();

	nbrdepth.first = sourceIndex;
	nbrdepth.second = 0;
	nbrsToVisit.push(nbrdepth);
	while (!nbrsToVisit.empty())
	{
		nbrdepth = nbrsToVisit.front();
		nbrsToVisit.pop();

		vert = nbrdepth.first;
		if (vert != sourceIndex)
			neighbors.push_back(vert);

		if (nbrdepth.second < rings)
		{
			nbrdepth.second++;
			numNbrs = m_ppTopoGraphSource[mlevel]->getNumAdjacentVertices(vert);
			for (i = 0; i < numNbrs; i++)
			{
				nbr = m_ppTopoGraphSource[mlevel]->getAdjacentVertex(vert, i);
				if (!visited[nbr])
				{
					nbrdepth.first = nbr;
					visited[nbr] = true;
					nbrsToVisit.push(nbrdepth);
				}
			}
		}
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::removeMatch(mpcard sourceIndex)
{
	mpcard i;
	mpint j;

	if (!matches[sourceIndex].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::removeMatch(mpcard sourceIndex) - sourceIndex is not matched");

	
	matches[sourceIndex].matched = false;
	for (j = (mpint)(matchedSourceIndices.size() - 1); j >= 0; j--)
	{
		if (matchedSourceIndices[j] == sourceIndex)
			break;
	}
	if (j >= 0)
	{
		for (i = (mpcard)j + 1; i < matchedSourceIndices.size(); i++)
			matchedSourceIndices[i - 1] = matchedSourceIndices[i];
		matchedSourceIndices.pop_back();
	}
	else
		warning("AR4PartialMatch<ManifoldTraits>::removeMatch(...): sourceIndex was not found in list of matched indices -- inconsistency in matching information!");
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::removeLastMatch()
{
	if (matchedSourceIndices.size() == 0)
		return;

	
	mpcard sourceIndex = matchedSourceIndices.back();

	matches[sourceIndex].matched = false;
	matchedSourceIndices.pop_back();
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::setFixed(mpcard sourceIndex, bool bfixed)
{
	if (!matches[sourceIndex].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::setFixed() - index is not matched.");
	matches[sourceIndex].fixed = bfixed;
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::fixAllMatches()
{
	mpcard i, k;
	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];
		if (matches[k].matched)
			matches[k].fixed = true;
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::unfixAllMatches()
{
	mpcard i;
	for (i = 0; i < matches.size(); i++)
	{
		if (matches[i].matched)
			matches[i].fixed = false;
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::backupNonFixedMatches()
{
	mpcard i, k;
	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];
		if (matches[k].fixed)	continue;
		backupMatch(k);
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::backupNonFixedMatchesAndGeodesics()
{
	mpcard i, j, k, tg;
	vector<mpcard> sourceDests;
	vector<float> dists;
	vector<Vector3f> grads;

	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];
		if (matches[k].fixed)	continue;
		backupMatch(k);

		m_pSparseSourceGeodesics->getGeodesicData(k, sourceDests, dists, grads);

		for (j = 0; j < sourceDests.size(); j++)
		{
			tg = lookupGeodesicMatch(k, sourceDests[j]);
			if (tg != AR4SparseGeodesics::m_InvalidIndex)
				m_pSparseTargetGeodesics->backupGeodesicData(tg);

			tg = lookupGeodesicMatch(sourceDests[j], k);
			if (tg != AR4SparseGeodesics::m_InvalidIndex)
				m_pSparseTargetGeodesics->backupGeodesicData(tg);
		}
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::backupMatches()
{
	mpcard i, k;
	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];
		backupMatch(k);
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::backupMatchesAndGeodesics()
{
	mpcard i, j, k, tg;
	vector<mpcard> sourceDests;
	vector<float> dists;
	vector<Vector3f> grads;

	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];
		backupMatch(k);

		m_pSparseSourceGeodesics->getGeodesicData(k, sourceDests, dists, grads);

		for (j = 0; j < sourceDests.size(); j++)
		{
			tg = lookupGeodesicMatch(k, sourceDests[j]);
			if (tg != AR4SparseGeodesics::m_InvalidIndex)
				m_pSparseTargetGeodesics->backupGeodesicData(tg);

			tg = lookupGeodesicMatch(sourceDests[j], k);
			if (tg != AR4SparseGeodesics::m_InvalidIndex)
				m_pSparseTargetGeodesics->backupGeodesicData(tg);
		}
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::rollbackMatches()
{
	mpcard i, k;

	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];
		rollbackMatch(k);
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::rollbackMatchesAndGeodesics()
{
	mpcard i, j, k, tg;
	vector<mpcard> sourceDests;
	vector<float> dists;
	vector<Vector3f> grads;

	for (i = 0; i < matchedSourceIndices.size(); i++)
	{
		k = matchedSourceIndices[i];
		rollbackMatch(k);

		m_pSparseSourceGeodesics->getGeodesicData(k, sourceDests, dists, grads);

		for (j = 0; j < sourceDests.size(); j++)
		{
			tg = lookupGeodesicMatch(k, sourceDests[j]);
			if (tg != AR4SparseGeodesics::m_InvalidIndex)
				m_pSparseTargetGeodesics->rollbackGeodesicData(tg);

			tg = lookupGeodesicMatch(sourceDests[j], k);
			if (tg != AR4SparseGeodesics::m_InvalidIndex)
				m_pSparseTargetGeodesics->rollbackGeodesicData(tg);
		}
	}
}

template <class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::VectorType AR4PartialMatch<ManifoldTraits>::getMatch( mpcard sourceIndex ) const
{
   if (!matches[sourceIndex].matched) {
      throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::getMatch() - index is not matched.");
   }
   return matches[sourceIndex].targetPos;
}

template <class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::getStretch(mpcard sourceIndex) const
{
	return matches[sourceIndex].stretch;
}

template<class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::getSourceDistToStart(mpcard sourceIndex) const
{
	return matches[sourceIndex].sourceDistToStart;
}

template <class ManifoldTraits>
bool AR4PartialMatch<ManifoldTraits>::isMatched( mpcard sourceIndex ) const
{
   return matches[sourceIndex].matched;
}

template <class ManifoldTraits>
bool AR4PartialMatch<ManifoldTraits>::isFixed(mpcard sourceIndex) const
{
	if (!matches[sourceIndex].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::isFixed() - index is not matched.");
	return matches[sourceIndex].fixed;
}

template <class ManifoldTraits>
mpcard AR4PartialMatch<ManifoldTraits>::getMatchedSourceIndex( mpcard matchIndex ) const
{
   return matchedSourceIndices[matchIndex];
}

template <class ManifoldTraits>
mpcard AR4PartialMatch<ManifoldTraits>::getNumMatchedIndices() const
{
   return matchedSourceIndices.size();
}

template <class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::computeErrorSingleMatch(mpcard sourceIndex)
{
	AR4PM_START_PROFILE(m_profStretchMeasure)

	mpcard i, k, tg;
	FloatType error = (FloatType)0;
	FloatType dsource, dtarget, diff;
	VectorType grad;

	if (!matches[sourceIndex].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::computeErrorSingleMatch(...): index " + intToStr(sourceIndex) + " not matched");

	vector<mpcard> availDests, measureDests;
	vector<mpcard> matchedDests, prematchedDests;
	vector<float> dists, tempSourceDists, tempTargetDists, measureSourceDists, measureTargetDists;
	vector<Vector3f> grads;
	vector<Vector3f> targetDests, measureTargetDests;
	vector<vector<Vector3f>> paths;
	float dist;
	mpcard mlevel;

	bool bAllowNonValidPaths = false;//this could be a parameter
	bool bTestSymmetricError = true;//this could also be a parameter

	float distMinThresh, distMaxThresh;

	if (m_nSampleLevels < 2)
		mlevel = 0;
	else
		mlevel = 1;

	if (m_avgEdgeWeight == (FloatType)0)
		m_avgEdgeWeight = computeAvgSourceMatchLevelEdgeWeight();

	distMinThresh = 2.f * m_avgEdgeWeight;
	distMaxThresh = 5.f * m_avgEdgeWeight;

	getSourceMatchingLevelNeighborhood(sourceIndex, 7, availDests);
	for (i = 0; i < availDests.size(); i++)
	{
		if (matches[availDests[i]].matched)
			matchedDests.push_back(availDests[i]);
	}
	
	m_pSparseSourceGeodesics->computeGeodesicsEarlyStop(m_geodSigma, sourceIndex, matchedDests, dists, grads, &paths);
	for (i = 0; i < matchedDests.size(); i++)
	{
		if (dists[i] >= distMinThresh && dists[i] <= distMaxThresh && (bAllowNonValidPaths || validateSourcePath(paths[i])))
		{
			measureDests.push_back(matchedDests[i]);
			tempSourceDists.push_back(dists[i]);
			targetDests.push_back(matches[matchedDests[i]].targetPos);
		}
	}

	m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, matches[sourceIndex].targetPos, targetDests, dists, grads, &paths);
	for (i = 0; i < targetDests.size(); i++)
	{
		if (bAllowNonValidPaths || validateTargetPath(paths[i]))
		{
			measureSourceDists.push_back(tempSourceDists[i]);
			measureTargetDists.push_back(dists[i]);
		}
	}

	for (i = 0; i < measureSourceDists.size(); i++)
	{
		dsource = measureSourceDists[i];
		dtarget = measureTargetDists[i];
		diff = fabs(dtarget - dsource);
		error += diff;
	}
	if (measureSourceDists.size() > 0)
		error /= (float)measureSourceDists.size();
	if (measureSourceDists.size() < 3)
		warning("Small number of points for triangulation/stretch computation for source index " + intToStr(sourceIndex));

	if (bTestSymmetricError)
	{
		card32 index;
		AAT pos, idx;
		Vector3f otherPos;
		card32 otherSource;
		FloatType euclidDist;
		FloatType symmErr = (FloatType)0;

		availDests.clear();
		targetDests.clear();
		measureDests.clear();
		measureSourceDists.clear();
		measureTargetDists.clear();

		pos = m_pTargetMatchCloud->getAAT("position");
		idx = m_pTargetMatchCloud->getAAT(m_strSourceIndexName);
		index = getNextNearestTargetMatchIndex(matches[sourceIndex].targetPos);
		while (index != m_InvalidIndex)
		{
			otherPos = m_pTargetMatchCloud->getPointSet()->get3f(index, pos);
			otherSource = m_pTargetMatchCloud->getPointSet()->get1u(index, idx);

			index = getNextNearestTargetMatchIndex();

			euclidDist = (otherPos - matches[sourceIndex].targetPos).getNorm();
			if (euclidDist > distMaxThresh)
				break;

			availDests.push_back(otherSource);
			targetDests.push_back(otherPos);
		}

		m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, matches[sourceIndex].targetPos, targetDests, dists, grads, &paths);
		for (i = 0; i < targetDests.size(); i++)
		{
			if (dists[i] >= distMinThresh && dists[i] <= distMaxThresh && (bAllowNonValidPaths || validateTargetPath(paths[i])))
			{
				measureDests.push_back(availDests[i]);
				measureTargetDests.push_back(targetDests[i]);
				tempTargetDists.push_back(dists[i]);
			}
		}

		m_pSparseSourceGeodesics->computeGeodesicsEarlyStop(m_geodSigma, sourceIndex, measureDests, dists, grads, &paths);
		for (i = 0; i < measureDests.size(); i++)
		{
			if (bAllowNonValidPaths || validateSourcePath(paths[i]))
			{
				measureSourceDists.push_back(dists[i]);
				measureTargetDists.push_back(tempTargetDists[i]);
			}
		}

		for (i = 0; i < measureSourceDists.size(); i++)
		{
			dsource = measureSourceDists[i];
			dtarget = measureTargetDists[i];
			diff = fabs(dtarget - dsource);
			symmErr += diff;
		}
		if (measureSourceDists.size() > 0)
			symmErr /= (float)measureSourceDists.size();
		if (measureSourceDists.size() < 3)
			warning("Small number of points for triangulation/stretch computation for source index " + intToStr(sourceIndex));

		if (symmErr > error)
			error = symmErr;
	}

	matches[sourceIndex].stretch = error;

	AR4PM_STOP_PROFILE(m_profStretchMeasure)

	return error;
}

template <class ManifoldTraits>
typename AR4PartialMatch<ManifoldTraits>::FloatType AR4PartialMatch<ManifoldTraits>::computeErrorSingleMatch(mpcard sourceIndex, set<mpcard>& sourceMeasurePoints)
{
	AR4PM_START_PROFILE(m_profStretchMeasure)

	mpcard i, k, tg;
	FloatType error = (FloatType)0;
	FloatType dsource, dtarget, diff;
	VectorType grad;

	if (!matches[sourceIndex].matched)
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::computeErrorSingleMatch(...): index " + intToStr(sourceIndex) + " not matched");

	vector<mpcard> availDests, measureDests;
	vector<mpcard> matchedDests, prematchedDests;
	vector<float> dists, tempSourceDists, tempTargetDists, measureSourceDists, measureTargetDists;
	vector<Vector3f> grads;
	vector<Vector3f> targetDests, measureTargetDests;
	vector<vector<Vector3f>> paths;
	float dist;
	mpcard mlevel;

	bool bAllowNonValidPaths = false;
	bool bTestSymmetricError = true;
	bool bMeasureWRTNewMatches = false;

	float distMinThresh, distMaxThresh;

	if (m_nSampleLevels < 2)
		mlevel = 0;
	else
		mlevel = 1;

	mpcard lastCohoCount = getLastMatchCohortCount();

	if (m_avgEdgeWeight == (FloatType)0)
		m_avgEdgeWeight = computeAvgSourceMatchLevelEdgeWeight();

	distMinThresh = 2.f * m_avgEdgeWeight;
	distMaxThresh = 5.f * m_avgEdgeWeight;

	getSourceMatchingLevelNeighborhood(sourceIndex, 7, availDests);
	for (i = 0; i < availDests.size(); i++)
	{
		if (matches[availDests[i]].matched && (bMeasureWRTNewMatches || matches[availDests[i]].matchOrder < lastCohoCount))
			matchedDests.push_back(availDests[i]);
	}
	
	m_pSparseSourceGeodesics->computeGeodesicsEarlyStop(m_geodSigma, sourceIndex, matchedDests, dists, grads, &paths);
	for (i = 0; i < matchedDests.size(); i++)
	{
		if (dists[i] >= distMinThresh && dists[i] <= distMaxThresh && (bAllowNonValidPaths || validateSourcePath(paths[i])))
		{
			measureDests.push_back(matchedDests[i]);
			tempSourceDists.push_back(dists[i]);
			targetDests.push_back(matches[matchedDests[i]].targetPos);
		}
	}

	m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, matches[sourceIndex].targetPos, targetDests, dists, grads, &paths);
	for (i = 0; i < targetDests.size(); i++)
	{
		if (bAllowNonValidPaths || validateTargetPath(paths[i]))
		{
			measureSourceDists.push_back(tempSourceDists[i]);
			measureTargetDists.push_back(dists[i]);

			sourceMeasurePoints.insert(measureDests[i]);
		}
	}

	for (i = 0; i < measureSourceDists.size(); i++)
	{
		dsource = measureSourceDists[i];
		dtarget = measureTargetDists[i];
		diff = fabs(dtarget - dsource);
		error += diff;
	}
	if (measureSourceDists.size() > 0)
		error /= (float)measureSourceDists.size();
	if (measureSourceDists.size() < 3)
		warning("Small number of points for triangulation/stretch computation for source index " + intToStr(sourceIndex));

	if (bTestSymmetricError)
	{
		AAT pos, idx;
		Vector3f otherPos;
		card32 otherSource;
		FloatType euclidDist;
		FloatType symmErr = (FloatType)0;

		availDests.clear();
		targetDests.clear();
		measureDests.clear();
		measureSourceDists.clear();
		measureTargetDists.clear();

		pos = m_pTargetMatchCloud->getAAT("position");
		idx = m_pTargetMatchCloud->getAAT(m_strSourceIndexName);
		m_pHknnTargetMatch->setSeekPointAndReset(matches[sourceIndex].targetPos);
		while (!m_pHknnTargetMatch->atEnd())
		{
			otherPos = m_pHknnTargetMatch->get3f(pos);
			otherSource = m_pHknnTargetMatch->get1u(idx);

			m_pHknnTargetMatch->next();

			euclidDist = (otherPos - matches[sourceIndex].targetPos).getNorm();
			if (euclidDist > distMaxThresh)
				break;

			availDests.push_back(otherSource);
			targetDests.push_back(otherPos);
		}

		m_pSparseTargetGeodesics->computeGeodesicsEarlyStop(m_geodSigma, matches[sourceIndex].targetPos, targetDests, dists, grads, &paths);
		for (i = 0; i < targetDests.size(); i++)
		{
			if (dists[i] >= distMinThresh && dists[i] <= distMaxThresh && (bAllowNonValidPaths || validateTargetPath(paths[i])))
			{
				measureDests.push_back(availDests[i]);
				measureTargetDests.push_back(targetDests[i]);
				tempTargetDists.push_back(dists[i]);
			}
		}

		m_pSparseSourceGeodesics->computeGeodesicsEarlyStop(m_geodSigma, sourceIndex, measureDests, dists, grads, &paths);
		for (i = 0; i < measureDests.size(); i++)
		{
			if (bAllowNonValidPaths || validateSourcePath(paths[i]) && (bMeasureWRTNewMatches || matches[measureDests[i]].matchOrder < lastCohoCount))
			{
				measureSourceDists.push_back(dists[i]);
				measureTargetDists.push_back(tempTargetDists[i]);

				sourceMeasurePoints.insert(measureDests[i]);
			}
		}

		for (i = 0; i < measureSourceDists.size(); i++)
		{
			dsource = measureSourceDists[i];
			dtarget = measureTargetDists[i];
			diff = fabs(dtarget - dsource);
			symmErr += diff;
		}
		if (measureSourceDists.size() > 0)
			symmErr /= (float)measureSourceDists.size();
		if (measureSourceDists.size() < 3)
			warning("Small number of points for triangulation/stretch computation for source index " + intToStr(sourceIndex));

		if (symmErr > error)
			error = symmErr;
	}

	matches[sourceIndex].stretch = error;

	AR4PM_STOP_PROFILE(m_profStretchMeasure)

	return error;
}

template <class ManifoldTraits>
UnstructuredInCorePointCloud* AR4PartialMatch<ManifoldTraits>::exportAsFullPointCloud()
{
	UnstructuredInCorePointCloud* pExport = new UnstructuredInCorePointCloud();

	mpcard i;

	const mpcard numPointExport = source->getNumDiscrPoints();
	VertexDescriptor vd;

	vd.pushAttrib(VAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(VAD(getMatchAttNameMatched(), 1, VAD::DATA_FORMAT_CARD32));
	vd.pushAttrib(VAD(getMatchAttNameMatchOrder(), 1, VAD::DATA_FORMAT_CARD32));
	vd.pushAttrib(VAD(getMatchAttNameTargetPos(), 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(VAD(getMatchAttNameStretch(), 1, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(VAD(getMatchAttNameDistToStart(), 1, VAD::DATA_FORMAT_FLOAT32));

	pExport->clearAndSetup(&vd, numPointExport);

	AAT pos = pExport->getAAT("position");
	AAT matched = pExport->getAAT(getMatchAttNameMatched());
	AAT matchOrder = pExport->getAAT(getMatchAttNameMatchOrder());
	AAT targPos = pExport->getAAT(getMatchAttNameTargetPos());
	AAT stretch = pExport->getAAT(getMatchAttNameStretch());
	AAT distStart = pExport->getAAT(getMatchAttNameDistToStart());

	PointSet* ps = pExport->getPointSet();

	for (i = 0; i < numPointExport; i++)
	{
		ps->set3f(i, pos, source->getDiscrPoint(i));
		ps->set1u(i, matched, (card32)matches[i].matched);
		ps->set1u(i, matchOrder, matches[i].matchOrder);
		ps->set3f(i, targPos, matches[i].targetPos);
		ps->set1f(i, stretch, matches[i].stretch);
		ps->set1f(i, distStart, matches[i].sourceDistToStart);
	}

	return pExport;
}

template <class ManifoldTraits>
UnstructuredInCorePointCloud* AR4PartialMatch<ManifoldTraits>::exportAsCompactPointCloud()
{
	UnstructuredInCorePointCloud* pExport = new UnstructuredInCorePointCloud();

	mpcard i, k;

	const mpcard numPointExport = getNumMatchedIndices();
	VertexDescriptor vd;

	vd.pushAttrib(VAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(VAD(getMatchAttNameIndex(), 1, VAD::DATA_FORMAT_CARD32));
	vd.pushAttrib(VAD(getMatchAttNameTargetPos(), 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(VAD(getMatchAttNameStretch(), 1, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(VAD(getMatchAttNameDistToStart(), 1, VAD::DATA_FORMAT_FLOAT32));

	pExport->clearAndSetup(&vd, numPointExport);

	AAT pos = pExport->getAAT("position");
	AAT index = pExport->getAAT(getMatchAttNameIndex());
	AAT targPos = pExport->getAAT(getMatchAttNameTargetPos());
	AAT stretch = pExport->getAAT(getMatchAttNameStretch());
	AAT distStart = pExport->getAAT(getMatchAttNameDistToStart());

	PointSet* ps = pExport->getPointSet();


	for (i = 0; i < numPointExport; i++)
	{

		k = matchedSourceIndices[i];
		ps->set3f(i, pos, source->getDiscrPoint(k));
		ps->set1u(i, index, k);

		Vector3f sourcePos = source->getDiscrPoint(k);
		Vector3f targetPos = matches[k].targetPos;
		


		ps->set3f(i, targPos, matches[k].targetPos);
	
		ps->set1f(i, stretch, matches[k].stretch);
		ps->set1f(i, distStart, matches[k].sourceDistToStart);
	}

	return pExport;
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::initThroughPointCloud(UnstructuredInCorePointCloud* pc)
{
	if (pc == NULL)
		throw EInvalidState("AR4PartialMatch<ManifoldTraits>::initThroughPointCloud(...): cannot init from a NULL point cloud, stupid.");

	if (!pc->providesAttribute(getMatchAttNameTargetPos()))
		throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::initThroughPointCloud(...): point cloud does not provides attributes necessary to reconstruct partial match");

	bool bCompact;
	if (!pc->providesAttribute(getMatchAttNameMatched()))
	{
		if (!pc->providesAttribute(getMatchAttNameIndex()))
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::initThroughPointCloud(...): point cloud does not provides attributes necessary to reconstruct partial match");

		bCompact = true;
	}
	else
		bCompact = false;

	mpcard i, k;
	const mpcard numPoints = pc->getNumPoints();

	Vector3f targetPos;

	m_pInitializerPointCloud = pc;

	PointSet* ps = pc->getPointSet();

	if (bCompact)
	{
		AAT index = pc->getAAT(getMatchAttNameIndex());
		AAT targPos = pc->getAAT(getMatchAttNameTargetPos());
		AAT stretch = pc->getAAT(getMatchAttNameStretch());
		AAT distStart = pc->getAAT(getMatchAttNameDistToStart());

		matchedSourceIndices.resize(numPoints);

		for (i = 0; i < numPoints; i++)
		{
			k = ps->get1u(i, index);
			targetPos = ps->get3f(i, targPos);

			matches[k].matched = true;
			matches[k].targetPos = targetPos;
			matches[k].stretch = ps->get1f(i, stretch);
			matches[k].matchOrder = i;
			matches[k].sourceDistToStart = ps->get1f(i, distStart);

			matchedSourceIndices[i] = k;
		}
	}
	else
	{
		if (numPoints != source->getNumDiscrPoints())
			throw ERangeCheck("AR4PartialMatch<ManifoldTraits>::initThroughPointCloud(...): number of points in full match point cloud does not match the number of points on source manifold");

		AAT matched = pc->getAAT(getMatchAttNameMatched());
		AAT matchOrder = pc->getAAT(getMatchAttNameMatchOrder());
		AAT targPos = pc->getAAT(getMatchAttNameTargetPos());
		AAT stretch = pc->getAAT(getMatchAttNameStretch());
		AAT distStart = pc->getAAT(getMatchAttNameDistToStart());

		mpcard matchCount = 0;
		for (i = 0; i < numPoints; i++)
		{
			if (ps->get1u(i, matched) != 0)
			{
				matches[i].matched = true;
				matches[i].matchOrder = ps->get1u(i, matchOrder);
				matches[i].targetPos = ps->get3f(i, targPos);
				matches[i].stretch = ps->get1f(i, stretch);
				matches[i].sourceDistToStart = ps->get1f(i, distStart);
				matchCount++;
			}
		}

		matchedSourceIndices.resize(matchCount);
		for (i = 0; i < numPoints; i++)
		{
			if (matches[i].matched)
			{
				matchedSourceIndices[matches[i].matchOrder] = i;
			}
		}
	}
}

template <class ManifoldTraits>
void AR4PartialMatch<ManifoldTraits>::reportAllProfileResults()
{
#if AR4PM_PROFILE
	m_profStretchMeasure.reportStats();
	m_profValidateSource.reportStats();
	m_profValidateTarget.reportStats();
	m_profFullValidate.reportStats();
	m_profRemoveOutliers.reportStats();
	m_profWalkIsometry.reportStats();
	debugOutput << "Source geodesic profiling:\n";
	m_pSparseSourceGeodesics->reportProfiling();
	debugOutput << "Target geodesic profiling:\n";
	m_pSparseTargetGeodesics->reportProfiling();
#endif //AR4PM_PROFILE
}


template <class ManifoldTraits>
typename AR4PartialMatchDistance<ManifoldTraits>::FloatType AR4PartialMatchDistance<ManifoldTraits>::evaluate()
{
	FloatType distanceSource = (FloatType)0;
	FloatType distanceTarget = (FloatType)0;
	FloatType distanceCombined;

	mpcard i, j, k, m;
	mpcard num1 = m_pMatch1->getNumMatchedIndices();
	mpcard num2 = m_pMatch2->getNumMatchedIndices();

	FloatType dist1, dist2;
	VectorType dest1, dest2, grad1, grad2;
	vector<VectorType> path1, path2;

	VectorType projPoint, projNorm;
	TangentSpace ts1, ts2;

	m_nSharedSourcePoints = 0;
	m_nSharedTargetPoints = 0;

	for (i = 0; i < num1; i++)
	{
		k = m_pMatch1->getMatchedSourceIndex(i);

		vector<mpcard> compLevelNbrs;
		m_pMatch1->getSourceNeighborhood(k, m_compareLevel, 1, compLevelNbrs);
		if (compLevelNbrs.size() == 0)
			continue;

		VectorType targ1 = m_pMatch1->getMatch(k);

		//project to get tangent space
		m_pMatch1->getTargetManifold()->project(targ1, projPoint, projNorm, ts1);

		if (m_pMatch2->isMatched(k))
		{
			VectorType targ2 = m_pMatch2->getMatch(k);

			//evaluate distance between matches of the same source point
			FloatType targDist = m_pMatch2->targetGeodesic(targ1, targ2);

			//add to total
			distanceSource += targDist;

			m_nSharedSourcePoints++;
		}

		vector<mpcard> nbrIndices;
		vector<FloatType> nbrDistances;
		sphericalQuerySecondTarget(targ1, m_pMatch2->getAvgSourceMatchLevelEdgeWeight(), nbrIndices, nbrDistances);

		const mpcard numNbrs = nbrIndices.size();
		for (mpcard nbr = 0; nbr < numNbrs; nbr++)
		{
			const FloatType targDist = nbrDistances[nbr];

			mpcard srcIdx = m_pMatch2->getMatchedSourceIndex(nbrIndices[nbr]);
			FloatType srcDist = m_pMatch2->sourceGeodesic(k, srcIdx);
			if (srcDist < (FloatType)0)
			{
				VectorType srcGrad;
				m_pMatch2->getSparseSourceGeodesics()->computeGeodesicSingleDest(m_pMatch2->getGeodesicsSigma() * 0.01f, k, srcIdx, srcDist, srcGrad);
			}

			//add to total
			distanceTarget += fabs(srcDist - targDist);

			//consider this a "shared point"
			m_nSharedTargetPoints++;
		}
	}

	//the following should make the distance measure symmetric
	//but should also add to the cost
#if 1
	for (i = 0; i < num2; i++)
	{
		k = m_pMatch2->getMatchedSourceIndex(i);

		vector<mpcard> compLevelNbrs;
		m_pMatch2->getSourceNeighborhood(k, m_compareLevel, 1, compLevelNbrs);
		if (compLevelNbrs.size() == 0)
			continue;

		VectorType targ2 = m_pMatch2->getMatch(k);

		vector<mpcard> nbrIndices;
		vector<FloatType> nbrDistances;
		sphericalQueryFirstTarget(targ2, m_pMatch1->getAvgSourceMatchLevelEdgeWeight(), nbrIndices, nbrDistances);

		const mpcard numNbrs = nbrIndices.size();
		for (mpcard nbr = 0; nbr < numNbrs; nbr++)
		{
			const FloatType targDist = nbrDistances[nbr];

			mpcard srcIdx = m_pMatch1->getMatchedSourceIndex(nbrIndices[nbr]);
			FloatType srcDist = m_pMatch1->sourceGeodesic(k, srcIdx);
			if (srcDist < (FloatType)0)
			{
				VectorType srcGrad;
				m_pMatch1->getSparseSourceGeodesics()->computeGeodesicSingleDest(m_pMatch1->getGeodesicsSigma() * 0.01f, k, srcIdx, srcDist, srcGrad);
			}

			//add to total
			distanceTarget += fabs(srcDist - targDist);

			//consider this a "shared point"
			m_nSharedTargetPoints++;
		}
	}
#endif

	if (m_nSharedSourcePoints > 0)
		distanceSource /= (FloatType)m_nSharedSourcePoints;

	if (m_nSharedTargetPoints > 0)
		distanceTarget /= (FloatType)m_nSharedTargetPoints;

	if (m_nSharedSourcePoints == 0)
	{
		distanceCombined = distanceTarget;
		m_overlap = (FloatType)m_nSharedTargetPoints;
	}
	else if (m_nSharedTargetPoints == 0)
	{
		distanceCombined = distanceSource;
		m_overlap = (FloatType)m_nSharedSourcePoints;
	}
	else
	{
		distanceCombined = (distanceSource + distanceTarget) * (FloatType)0.5;
		m_overlap = (FloatType)(m_nSharedSourcePoints + m_nSharedTargetPoints) * (FloatType)0.5;
	}

	return distanceCombined;
}


template <class ManifoldTraits>
typename AR4PartialMatchDistance<ManifoldTraits>::FloatType AR4SymmetricPartialMatchDistance<ManifoldTraits>::evaluate()
{
	FloatType distance12 = (FloatType)0;
	FloatType distance21 = (FloatType)0;
	FloatType distanceCombined;

	mpcard i, j, k, m;
	const mpcard num1 = m_pMatch1->getNumMatchedIndices();
	const mpcard num2 = m_pMatch2->getNumMatchedIndices();

	FloatType dist1, dist2;
	VectorType dest1, dest2, grad1, grad2;

	vector<mpcard> dests2(num2);
	vector<mpcard> dests1(num1);

	for (i = 0; i < num1; i++)
		dests1[i] = m_pMatch1->getMatchedSourceIndex(i);

	for (i = 0; i < num2; i++)
		dests2[i] = m_pMatch2->getMatchedSourceIndex(i);

	AR4SparseGeodesics* pGeoS = m_pMatch1->getSparseSourceGeodesics();

	vector<mpcard> nearestNeighbors;
	pGeoS->geodesicVoronoi(dests2, nearestNeighbors);

	mpcard num1actual = 0;
	for (i = 0; i < num1; i++)
	{
		k = m_pMatch1->getMatchedSourceIndex(i);

		vector<mpcard> compLevelNbrs;
		m_pMatch1->getSourceNeighborhood(k, m_compareLevel, 1, compLevelNbrs);
		if (compLevelNbrs.size() == 0)
			continue;

		num1actual++;

		const VectorType targ1 = m_pMatch1->getMatch(k);

		if (m_pMatch2->isMatched(k))
		{
			VectorType targ2 = m_pMatch2->getMatch(k);

			//evaluate distance between matches of the same source point
			FloatType targDist = m_pMatch2->targetGeodesic(targ1, targ2);

			//add to total
			distance12 += targDist*targDist;
		}
		else
		{
			float distSrc;
			m = nearestNeighbors[k];
			distSrc = pGeoS->getDistanceField(k);

			VectorType targ2 = m_pMatch2->getMatch(m);

			FloatType targDist = m_pMatch2->targetGeodesic(targ1, targ2);

			distance12 += sqr(targDist - distSrc);
		}
	}

	pGeoS->geodesicVoronoi(dests1, nearestNeighbors);

	mpcard num2actual = 0;
	for (i = 0; i < num2; i++)
	{
		k = m_pMatch2->getMatchedSourceIndex(i);

		vector<mpcard> compLevelNbrs;
		m_pMatch2->getSourceNeighborhood(k, m_compareLevel, 1, compLevelNbrs);
		if (compLevelNbrs.size() == 0)
			continue;

		num2actual++;

		const VectorType targ2 = m_pMatch2->getMatch(k);

		if (m_pMatch1->isMatched(k))
		{
			VectorType targ1 = m_pMatch1->getMatch(k);

			//evaluate distance between matches of the same source point
			FloatType targDist = m_pMatch1->targetGeodesic(targ1, targ2);

			//add to total
			distance21 += targDist*targDist;
		}
		else
		{
			float distSrc;
			m = nearestNeighbors[k];
			distSrc = pGeoS->getDistanceField(k);

			VectorType targ1 = m_pMatch1->getMatch(m);

			FloatType targDist = m_pMatch1->targetGeodesic(targ1, targ2);

			distance21 += sqr(targDist - distSrc);
		}
	}

	distanceCombined = (distance12 / (FloatType)num1actual + distance21 / (FloatType)num2actual) * 0.5;

	return distanceCombined;
}



}

#endif
