///////////////////////////////////////////////////////////////////////////////
//
//	AR4PartialMatchMaterial.h
//
//	Header file for the AR4PartialMatchMaterial class
//	subclass of GLMaterial
//	for visualizing a partial match (shape correspondence)
//
//	Alan Brunton, MPII, March 2013
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __AR4PARTIALMATCHMATERIAL_H__
#define __AR4PARTIALMATCHMATERIAL_H__


//---------------------------------------------------------------------------
#include "threeParameterMatchingGMOD14.h"
//---------------------------------------------------------------------------
#include "GLMaterial.h"
#include "LinearAlgebra.inline.h"
#include <string>
//---------------------------------------------------------------------------


namespace X4
{
	//material to display corresponding points on two surfaces
	class THREE_PARAMETER_MATCHING_GMOD14_API AR4PartialMatchMaterial : public GLMaterial
	{
		X4_CLASS(AR4PartialMatchMaterial)

	protected:

		string								m_strPositionChannel;
		string								m_strTargetPosChannel;

		string								m_strRadiusChannel;
		bool								m_bUseRadiusChannel;
		float32								m_radiusScale;

		bool								m_bUseColorChannel;
		string								m_strColorChannel;
		Vector3f							m_defaultColorRepeatPeriod;
		bool								m_bOnlyHighFreqPattern;

	public:

		AR4PartialMatchMaterial();
		~AR4PartialMatchMaterial();

		virtual void assign(const Object* obj, X4CopyContext *context = NULL);

		virtual void draw(	Scene *scene, SceneGraphState *state, SceneObject *obj,
							SceneObjectIterator *iter, GLContext *context, MemoryManager * mm = NULL,
							const int32 includeFlags = 0, const int32 excludeFlags = PF_FLAG_INVALID );

		string getPositionChannel()			{ return m_strPositionChannel; }
		void setPositionChannel(string str)	{ m_strPositionChannel = str; }

		string getTargetPosChannel()		{ return m_strTargetPosChannel; }
		void setTargetPosChannel(string str){ m_strTargetPosChannel = str; }

		string getRadiusChannel()			{ return m_strRadiusChannel; }
		void setRadiusChannel(string str)	{ m_strRadiusChannel = str; }

		bool useRadiusChannel()				{ return m_bUseRadiusChannel; }
		void setUseRadiusChannel(bool b)	{ m_bUseRadiusChannel = b; }

		bool useColorChannel()				{ return m_bUseColorChannel; }
		void setUseColorChannel(bool b)		{ m_bUseColorChannel = b; }

		string getColorChannel()			{ return m_strColorChannel; }
		void setColorChannel(string str)	{ m_strColorChannel = str; }

		Vector3f getDefaultColorRepeatPeriod()				{ return m_defaultColorRepeatPeriod; }
		void setDefaultColorRepeatPeriod(const Vector3f& v)	{ m_defaultColorRepeatPeriod = v; }
	};
}


#endif //__AR4PARTIALMATCHMATERIAL_H__


