///////////////////////////////////////////////////////////////////////////////
//
//	AR4PartialMatchMerger.h
//
//	Header file for AR4PartialMatchMerger class
//	to merge partial isometric matches
//
//	Alan Brunton, MPII, March 2013
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __AR4PARTIALMATCHMERGER_H__
#define __AR4PARTIALMATCHMERGER_H__


#include "threeParameterMatchingGMOD14.h"
#include "AR4PartialMatch.h"

#include "GenericSwapAllOutOfCorePointCloud.h"


#define AR4PMM_BP_PAIRWISE_POINT_DISTANCES	0


namespace X4
{

template<class ManifoldTraits>
class AR4PartialMatchMerger
{
public:
	typedef typename ManifoldTraits::FloatType  FloatType;
	typedef typename ManifoldTraits::VectorType VectorType;
	typedef typename ManifoldTraits AR4DiscreteManifoldTraits;
	typedef typename ManifoldTraits::TangentSpace TangentSpace;
	typedef typename ManifoldTraits::BoundingBoxType BoundingBoxType;

protected:

public:

	AR4PartialMatchMerger()
	{
	}
	virtual ~AR4PartialMatchMerger()
	{
	}

	virtual void run(vector<GenericSwapAllOutOfCorePointCloud*>& partialMatches, const vector<mpcard>& matchIndices, AR4PartialMatch<ManifoldTraits>* pMergeMatch) = 0;
};


template<class ManifoldTraits>
class AR4PartialMatchWeightedAverageMerger : public AR4PartialMatchMerger<ManifoldTraits>
{
protected:

	typedef pair<FloatType, VectorType>		TWeightedPos;
	typedef pair<FloatType, card32>			TIndexRating;

public:

	AR4PartialMatchWeightedAverageMerger(): AR4PartialMatchMerger()
	{
	}
	~AR4PartialMatchWeightedAverageMerger()
	{
	}

	virtual void run(vector<GenericSwapAllOutOfCorePointCloud*>& partialMatches, const vector<mpcard>& matchIndices, AR4PartialMatch<ManifoldTraits>* pMergeMatch) { }
};


class AR4PartialMatchWeightedAverageMerger3f : public AR4PartialMatchWeightedAverageMerger<ManifoldTraits3f>
{
protected:

public:

	AR4PartialMatchWeightedAverageMerger3f(): AR4PartialMatchWeightedAverageMerger<ManifoldTraits3f>()
	{
	}
	~AR4PartialMatchWeightedAverageMerger3f()
	{
	}

	virtual void run(vector<GenericSwapAllOutOfCorePointCloud*>& partialMatches, const vector<mpcard>& matchIndices, AR4PartialMatch3f* pMergeMatch);
};


template<class ManifoldTraits>
class AR4PartialMatchMetricPreservingMerger : public AR4PartialMatchMerger<ManifoldTraits>
{
protected:

	typedef pair<FloatType, VectorType>		TWeightedPos;
	typedef pair<FloatType, card32>			TIndexRating;

public:

	AR4PartialMatchMetricPreservingMerger()
	{
	}
	~AR4PartialMatchMetricPreservingMerger()
	{
	}

	virtual void run(vector<GenericSwapAllOutOfCorePointCloud*>& partialMatches, const vector<mpcard>& matchIndices, AR4PartialMatch<ManifoldTraits>* pMergeMatch) { }
};


class AR4PartialMatchMetricPreservingMerger3f : public AR4PartialMatchMetricPreservingMerger<ManifoldTraits3f>
{
protected:

public:

	AR4PartialMatchMetricPreservingMerger3f(): AR4PartialMatchMetricPreservingMerger<ManifoldTraits3f>()
	{
	}
	~AR4PartialMatchMetricPreservingMerger3f()
	{
	}

	virtual void run(vector<GenericSwapAllOutOfCorePointCloud*>& partialMatches, const vector<mpcard>& matchIndices, AR4PartialMatch3f* pMergeMatch);
};


template<class ManifoldTraits>
class AR4PartialMatchBPDirectMerger : public AR4PartialMatchMerger<ManifoldTraits>
{
protected:

public:

	AR4PartialMatchBPDirectMerger()
	{
	}
	virtual ~AR4PartialMatchBPDirectMerger()
	{
	}

	virtual void run(vector<GenericSwapAllOutOfCorePointCloud*>& partialMatches, const vector<mpcard>& matchIndices, AR4PartialMatch<ManifoldTraits>* pMergeMatch) { }
};


class AR4PartialMatchBPDirectMerger3f : public AR4PartialMatchBPDirectMerger<ManifoldTraits3f>
{
protected:

	class PartialMapping
	{
	public:

		mpcard								m_iPartial;
		Vector3f							m_targetPos;

		PartialMapping(const mpcard iPartial, const Vector3f& targetPos)
		{
			m_iPartial = iPartial;
			m_targetPos = targetPos;
		}

		PartialMapping& operator=(const PartialMapping& pm)
		{
			m_iPartial = pm.m_iPartial;
			m_targetPos = pm.m_targetPos;
			return *this;
		}
	};

	class BPNode
	{
	public:
		vector<BPNode*>						m_neighbors;
		vector<vector<float>>				m_messages;
		vector<mpcard>						m_nbrMsgLU;

		mpcard addNeighbor(BPNode* pNode)
		{
			const mpcard numCurrNbrs = m_neighbors.size();
			mpcard i;
			for (i = 0; i < numCurrNbrs; i++)
			{
				if (m_neighbors[i] == pNode)
					return i;
			}
			m_neighbors.push_back(pNode);
			mpcard otherDir = pNode->addNeighbor(this);
			m_nbrMsgLU.push_back(otherDir);
			return i;
		}

		virtual void computeMessages() = 0;
		virtual void allocateMessages() = 0;

		virtual void init()
		{
			allocateMessages();
		}

		void normalizeMessages()
		{
			const mpcard numNbrs = m_neighbors.size();
			mpcard i, j;
			for (i = 0; i < numNbrs; i++)
			{
				const mpcard nLabels = m_messages[i].size();
				float val, valMin = 1e30f;
				for (j = 0; j < nLabels; j++)
				{
					val = m_messages[i][j];
					if (val < valMin)
						valMin = val;
				}
				for (j = 0; j < nLabels; j++)
					m_messages[i][j] -= valMin;
			}
		}
	};

	class BPVariableNode : public BPNode
	{
	public:
		mpcard								m_nLabels;
		mpcard								m_appDataLU;
		vector<mpcard>						m_labelMap;
		vector<float>						m_beliefs;
		mpcard								m_MAPLabel;

		BPVariableNode(mpcard nLabels, mpcard appDataLU): m_nLabels(nLabels), m_appDataLU(appDataLU) 
		{
			m_labelMap.resize(m_nLabels);
			for (mpcard i = 0; i < m_nLabels; i++)
				m_labelMap[i] = i;
			m_MAPLabel = (mpcard)-1;
		}
		BPVariableNode(mpcard nLabels, mpcard appDataLU, const vector<mpcard>& labelMap): m_nLabels(nLabels), m_appDataLU(appDataLU) 
		{
			m_labelMap.resize(m_nLabels);
			if (labelMap.size() < m_nLabels)
				throw ERangeCheck("AR4PartialMatchBPDirectMerger3f::BPVarNode::BPVarNode(...): label map wrong size");
			for (mpcard i = 0; i < m_nLabels; i++)
				m_labelMap[i] = labelMap[i];
			m_MAPLabel = (mpcard)-1;
		}

		void computeMessages()
		{
			const mpcard numNbrs = m_neighbors.size();
			const mpcard nLabels = m_nLabels;
			mpcard i, j, k;
			for (i = 0; i < numNbrs; i++)
			{
				for (j = 0; j < numNbrs; j++)
				{
					if (j == i)	continue;
					const mpcard otherDir = m_nbrMsgLU[j];

					for (k = 0; k < nLabels; k++)
						m_messages[i][k] += m_neighbors[j]->m_messages[otherDir][k];
				}
			}
		}

		void computeBeliefs()
		{
			const mpcard numNbrs = m_neighbors.size();
			const mpcard nLabels = m_nLabels;
			mpcard i, k;

			for (k = 0; k < nLabels; k++)
				m_beliefs[k] = 0.f;

			for (i = 0; i < numNbrs; i++)
			{
				const mpcard oidx = m_nbrMsgLU[i];

				for (k = 0; k < nLabels; k++)
					m_beliefs[k] += m_neighbors[i]->m_messages[oidx][k];
			}

			float belMin = 1e30f;
			for (k = 0; k < nLabels; k++)
			{
				if (m_beliefs[k] < belMin)
				{
					m_MAPLabel = k;
					belMin = m_beliefs[k];
				}
			}
		}

		virtual void allocateMessages()
		{
			const mpcard numNbrs = m_neighbors.size();
			const mpcard nLabels = m_nLabels;
			mpcard i;

			m_messages.resize(numNbrs);
			for (i = 0; i < numNbrs; i++)
			{
				m_messages[i].resize(nLabels, 0.f);
			}

			m_beliefs.resize(m_nLabels, 0.f);
		}
	};

	class BPFactorNode : public BPNode
	{
	public:

		virtual void allocateMessages()
		{
			const mpcard numNbrs = m_neighbors.size();
			mpcard i;

			m_messages.resize(numNbrs);
			for (i = 0; i < numNbrs; i++)
			{
				const mpcard otherDir = m_nbrMsgLU[i];
				const mpcard nLabels = m_neighbors[i]->m_messages[otherDir].size();
				m_messages[i].resize(nLabels, 0.f);
			}
		}
	};

	class BPGeodesicConstraintNode : public BPFactorNode
	{
	public:

		vector<float>*						m_pSourceDistFields;
		vector<float>*						m_pTargetDistFields;
		mpcard*								m_pSourceTargetOffset;
		mpcard								m_nSourceFieldSize;
		mpcard								m_nTargetFieldSize;
		mpcard								m_nDistFieldsPerMatch;

		BPGeodesicConstraintNode(	vector<float>* pSourceDistFields, vector<float>* pTargetDistFields, 
									mpcard* pSourceTargetOffset, mpcard nSourceFieldSize, 
									mpcard nTargetFieldSize, mpcard nDistFieldsPerMatch)
		{
			m_pSourceDistFields = pSourceDistFields;
			m_pTargetDistFields = pTargetDistFields;
			m_pSourceTargetOffset = pSourceTargetOffset;
			m_nSourceFieldSize = nSourceFieldSize;
			m_nTargetFieldSize = nTargetFieldSize;
			m_nDistFieldsPerMatch = nDistFieldsPerMatch;
		}
	};

	class BPPairwiseGeodesicConstraintNode : public BPGeodesicConstraintNode
	{
	public:

#if AR4PMM_BP_PAIRWISE_POINT_DISTANCES
		vector<vector<float>>				m_pairwiseErrors;
#endif //AR4PMM_BP_PAIRWISE_POINT_DISTANCES

		BPPairwiseGeodesicConstraintNode(	vector<float>* pSourceDistFields, vector<float>* pTargetDistFields, 
											mpcard* pSourceTargetOffset, mpcard nSourceFieldSize, 
											mpcard nTargetFieldSize, mpcard nDistFieldsPerMatch): BPGeodesicConstraintNode(	pSourceDistFields, pTargetDistFields, 
																															pSourceTargetOffset, nSourceFieldSize, 
																															nTargetFieldSize, nDistFieldsPerMatch)
		{
		}

#if AR4PMM_BP_PAIRWISE_POINT_DISTANCES
		void computeMessages()
		{
			const mpcard numNbrs = m_neighbors.size();
			mpcard i, j, k, m, n;
			BPVariableNode* pDestVar;

			i = 0;
			j = 1;
			mpcard otherDirOut = m_nbrMsgLU[i];
			mpcard nOutLabels = m_messages[i].size();
			pDestVar = dynamic_cast<BPVariableNode*>(m_neighbors[i]);

			for (k = 0; k < nOutLabels; k++)
			{
				float valMin = 1e30f;

				const mpcard otherDirIn = m_nbrMsgLU[j];
				const mpcard nInLabels = m_neighbors[j]->m_messages[otherDirIn].size();
				for (m = 0; m < nInLabels; m++)
				{
					float val = m_pairwiseErrors[k][m];
					val += m_neighbors[j]->m_messages[otherDirIn][m];

					if (val < valMin)
						valMin = val;
				}

				m_messages[i][k] = valMin;
			}

			i = 1;
			j = 0;
			otherDirOut = m_nbrMsgLU[i];
			nOutLabels = m_messages[i].size();
			pDestVar = dynamic_cast<BPVariableNode*>(m_neighbors[i]);

			for (k = 0; k < nOutLabels; k++)
			{
				float valMin = 1e30f;

				const mpcard otherDirIn = m_nbrMsgLU[j];
				const mpcard nInLabels = m_neighbors[j]->m_messages[otherDirIn].size();
				for (m = 0; m < nInLabels; m++)
				{
					float val = m_pairwiseErrors[m][k];
					val += m_neighbors[j]->m_messages[otherDirIn][m];

					if (val < valMin)
						valMin = val;
				}

				m_messages[i][k] = valMin;
			}
		}
#else
		void computeMessages()
		{
			const mpcard numNbrs = m_neighbors.size();
			mpcard i, j, k, m, n;

			for (i = 0; i < numNbrs; i++)
			{
				const mpcard otherDirOut = m_nbrMsgLU[i];
				const mpcard nOutLabels = m_messages[i].size();
				BPVariableNode* pDestVar = dynamic_cast<BPVariableNode*>(m_neighbors[i]);
				const mpcard sourceOffset = pDestVar->m_appDataLU;
				const mpcard targetOffset = m_pSourceTargetOffset[sourceOffset];

				for (k = 0; k < nOutLabels; k++)
				{
					float valMin = 1e30f;
					float val = 0.f;

					const mpcard outMatch = pDestVar->m_labelMap[k];

					j = 1 - i;
					const mpcard otherDirIn = m_nbrMsgLU[j];
					const mpcard nInLabels = m_neighbors[j]->m_messages[otherDirIn].size();

					BPVariableNode* pInVar = dynamic_cast<BPVariableNode*>(m_neighbors[j]);
					const mpcard sourceOffsetIn = pInVar->m_appDataLU;
					const mpcard targetOffsetIn = m_pSourceTargetOffset[sourceOffsetIn];

					for (m = 0; m < nInLabels; m++)
					{
						const mpcard inMatch = pInVar->m_labelMap[m];
						mpcard inMatchField = inMatch * m_nDistFieldsPerMatch;

						for (n = 0; n < m_nDistFieldsPerMatch; n++)
						{
							const float sourceDist = m_pSourceDistFields[inMatchField][sourceOffset];
							const float targetDist = m_pTargetDistFields[inMatchField][targetOffset + k];
							inMatchField++;

							const float distDiff = targetDist - sourceDist;
							val += distDiff*distDiff;
						}

						mpcard outMatchField = outMatch * m_nDistFieldsPerMatch;
						for (n = 0; n < m_nDistFieldsPerMatch; n++)
						{
							const float sourceDist = m_pSourceDistFields[outMatchField][sourceOffsetIn];
							const float targetDist = m_pTargetDistFields[outMatchField][targetOffsetIn + m];
							outMatchField++;

							const float distDiff = targetDist - sourceDist;
							val += distDiff*distDiff;
						}

						val += m_neighbors[j]->m_messages[otherDirIn][m];

						if (val < valMin)
						{
							valMin = val;
						}
					}

					m_messages[i][k] = valMin;
				}
			}
		}
#endif //AR4PMM_BP_PAIRWISE_POINT_DISTANCES
	};

	typedef BPGeodesicConstraintNode			GCN;
	typedef BPPairwiseGeodesicConstraintNode	PGCN;


	mpcard									m_nSourcePoints;
	mpcard									m_nTargetPoints;

	AR4MLSManifold3f*						m_pSourceManifold;
	AR4MLSManifold3f*						m_pTargetManifold;
	AR4SparseGeodesics*						m_pSourceGeos;
	AR4SparseGeodesics*						m_pTargetGeos;

	vector<vector<float>>					m_sourceDistFields;
	vector<vector<float>>					m_targetDistFields;
	mpcard									m_nFieldsPerPartial;

	vector<mpcard>							m_sourceIndices;
	vector<mpcard>							m_sourceMatchOffsets;

	map<mpcard, vector<PartialMapping>>		m_possibleMatches;
	map<mpcard, Vector3f>					m_finalMatches;

	vector<BPVariableNode*>					m_variables;
	vector<BPFactorNode*>					m_factors;


	void initDistanceFields(vector<GenericSwapAllOutOfCorePointCloud*>& partialMatches);
	void initPairwiseDistances(vector<GenericSwapAllOutOfCorePointCloud*>& partialMatches);

	void clearFactorGraph()
	{
		mpcard i, numVars = m_variables.size(), numFactors = m_factors.size();
		for (i = 0; i < numVars; i++)
			delete m_variables[i];
		m_variables.clear();
		for (i = 0; i < numFactors; i++)
			delete m_factors[i];
		m_factors.clear();
	}

	void initFactorGraph();
	void iterateFactorGraph();
	void computeMAPSolution(AR4PartialMatch3f* pMAPMatch);

public:

	AR4PartialMatchBPDirectMerger3f()
	{
		m_nSourcePoints = 0;
		m_nTargetPoints = 0;
		m_pSourceManifold = NULL;
		m_pTargetManifold = NULL;
		m_pSourceGeos = NULL;
		m_pTargetGeos = NULL;
		m_nFieldsPerPartial = 10;
	}
	~AR4PartialMatchBPDirectMerger3f()
	{
	}

	virtual void run(vector<GenericSwapAllOutOfCorePointCloud*>& partialMatches, const vector<mpcard>& matchIndices, AR4PartialMatch3f* pMergeMatch);
};


}


#endif //__AR4PARTIALMATCHMERGER_H__


