//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "AR4Settings.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

static string descExtResType(card32 type)
{
	if (type == AR4Settings::extResBIM)		return "BIM";
	if (type == AR4Settings::extResAvinash)	return "Avinash";
	if (type == AR4Settings::extResAurela)	return "Aurela";
	if (type == AR4Settings::extResArt)		return "Art";
	return "undefined";
}

static string descMergeMode(card32 mode)
{
	if (mode == AR4Settings::m_PCIARMMClusterGDL)		return "cluster - GDL";
	if (mode == AR4Settings::m_PCIARMMClusterAP)		return "cluster - AP";
	if (mode == AR4Settings::m_PCIARMMClusterSpectral)	return "cluster - spectral";
	if (mode == AR4Settings::m_PCIARMMDirectBP)			return "direct - BP";
	return "undefined";
}

static string descFeatureMatchSource(card32 source)
{
	if (source == AR4Settings::fmSrcInternal)			return "internal";
	if (source == AR4Settings::fmSrcExternal)			return "external";
	return "undefined";
}

IMPLEMENT_X4_CLASS(AR4Settings, 12)
{
	BEGIN_CLASS_INIT(AR4Settings)
	ADD_SEPARATOR("surface settings")
	ADD_CARD32_PROP(numNN, 0)
	ADD_FLOAT32_PROP(maxSampleSpacing, 0)
	ADD_SEPARATOR("feature settings")
	ADD_CARD32_PROP_UPDATE_DESCR(featureMatchSource, 2, NULL, descFeatureMatchSource, 2, NULL)
	ADD_STRING_PROP(strExtSparseCorrFile, 2)
	ADD_SEPARATOR("growing settings")
	ADD_FLOAT32_PROP(stretchThreshold, 7)
	ADD_CARD32_PROP(growingKillSteps, 7)
	ADD_CARD32_PROP(numFeatureTries, 12)
	ADD_SEPARATOR("clustering settings")
	ADD_MAX_PLATFORM_CARD_PROP(numClustersMin, 0)
	ADD_FLOAT32_PROP(clusterMergeThreshold, 8)
	ADD_CARD32_PROP(clusteringNumNN, 12)
	ADD_CARD32_PROP_UPDATE_DESCR(mergeMode, 1, NULL, descMergeMode, m_PCIARMNumModes, NULL)
	ADD_SEPARATOR("visualization settings")
	ADD_CARD32_PROP(numInterpIters, 1)
	ADD_CARD32_PROP(numCheckerBins, 0)
	ADD_FLOAT32_PROP(texKernelFactor, 0)
	ADD_VECTOR3F_PROP(checkerColor1, 0)
	ADD_VECTOR3F_PROP(checkerColor2, 0)
	ADD_BOOLEAN_PROP(bHighResViz, 11)
	ADD_SEPARATOR("comparison settings")
	ADD_CARD32_PROP_UPDATE_DESCR(extResType, 0, NULL, descExtResType, extResNumTypes, NULL)
	ADD_STRING_PROP(strCorrFile, 0)
	ADD_BOOLEAN_PROP(bEvalExternalCorrs, 0)
	ADD_STRING_PROP(strFirstSubfolder, 12)
	ADD_CARD32_PROP(numSubfolders, 12)
	ADD_STRING_PROP(strCumulativeFilename, 12)
	ADD_STRING_PROP(strCoverageFilename, 12)
	
	
}

//---------------------------------------------------------------------------

AR4Settings::AR4Settings() 
{
   numNN = 8;
   maxSampleSpacing = 0.1f;

   featureMatchSource = fmSrcInternal;
   strExtSparseCorrFile = "";

   stretchThreshold = 0.25f;
   growingKillSteps = 300;
   numFeatureTries = 200;

   numClustersMin = 0;
   clusterMergeThreshold = 0.f;
   mergeMode = m_PCIARMMClusterGDL;
   clusteringNumNN = 10;

   numInterpIters = 16;
   numCheckerBins = 16;
   texKernelFactor = 0.25f;
   checkerColor1 = makeVector3f(1.f, 1.f, 1.f);
   checkerColor2 = makeVector3f(0.3f, 0.3f, 1.0f);
   bHighResViz = false;
   ColorPeriod = 0.2f;

   extResType = extResBIM;
   strCorrFile = "corrs.txt";
   bEvalExternalCorrs = false;

   strFirstSubfolder = "scape_000_001";
   numSubfolders = 70;
   strCumulativeFilename = "ar4MatchErrCumulativeDistribution0000.txt";
   strCoverageFilename = "ar4MatchCoverage0000.txt";

   
  
  
  
   
}

void AR4Settings::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj, this);
}

AR4Settings::~AR4Settings() {}

//---------------------------------------------------------------------------
