//---------------------------------------------------------------------------
#ifndef AR4SettingsH
#define AR4SettingsH
//---------------------------------------------------------------------------
#include "threeParameterMatchingGMOD14.h"
//---------------------------------------------------------------------------
#include "X4Types.h"
#include "Persistent.h"
#include "SeparatorClassProperty.h"
#include "LinearAlgebra.inline.h"
//---------------------------------------------------------------------------


namespace X4 {



class THREE_PARAMETER_MATCHING_GMOD14_API AR4Settings : public Persistent {
	X4_CLASS( AR4Settings )
 public:

	static const card32					extResBIM					= 0;
	static const card32					extResAvinash				= 1;
	static const card32					extResAurela				= 2;
	static const card32					extResArt					= 3;
	static const card32					extResNumTypes				= 4;

	const static card32					m_PCIARMMClusterGDL			= 0;
	const static card32					m_PCIARMMClusterAP			= 1;
	const static card32					m_PCIARMMClusterSpectral	= 2;
	const static card32					m_PCIARMMDirectBP			= 3;
	const static card32					m_PCIARMNumModes			= 4;

	const static card32					fmSrcInternal				= 0;
	const static card32					fmSrcExternal		= 1;



   //surface representation parameters
   /// number of nearest neighbors for building point cloud connectivity
   card32 numNN;
   /// maximum distance between nearest neighbors
   float32 maxSampleSpacing;

   //geodesic computation parameters
   bool bUsePathRelaxation;//use Laplacian relaxation of geodesic paths for higher accuracy; default: false

   //feature matching parameters
   card32 featureMatchSource;
   string strExtSparseCorrFile;

   //growing matcher parameters
   float32 stretchThreshold;//threshold for rejection as outlier based on intrinsic stretching; default: 0.25
   card32 growingKillSteps;//number of steps of growing matcher before stopping; default: 300
   card32 numFeatureTries;//number of feature matches to try to initialize grower; default: 200

   //clustering parameters
   card32 mergeMode;
   mpcard numClustersMin;
   float32 clusterMergeThreshold;
   mpcard  clusteringNumNN;

   //matching visualizaton parameters
   //number of iterations of correspondence interpolation to do
   card32 numInterpIters;
   //number of bins along bounding box's maximum dimension
   card32 numCheckerBins;
   //factor of sample spacing for blending correspondence colors
   float32 texKernelFactor;
   //colors for the checkerbin pattern
   Vector3f checkerColor1, checkerColor2;
   //whether or not to use high-resolution point cloud sampling for visualization
   bool bHighResViz;
   float32 ColorPeriod;
   //stuff for visualizing and evaluating other methods
   card32 extResType;
   string strCorrFile;

   //whether or not to evaluate ground truth correspondences
   bool bEvalExternalCorrs;

   //for collecting evaluation over a benchmark
   string strFirstSubfolder;
   card32 numSubfolders;
   string strCumulativeFilename;
   string strCoverageFilename;

   
	AR4Settings();
	virtual void assign(const Object* obj, X4CopyContext *context = NULL);
    virtual ~AR4Settings();
  
};


}

#endif
