//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PCCGaussianCurvature.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "UnstructuredInCorePointCloud.h"
#include "PCCTriangleMeshSampler.h"
#include "PointCloudTools.h"
#include "InCorePCTopologyGraph.h"
#include "GeometricTools.h"
#include "HierarchicalKNNIterator.h"
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

IMPLEMENT_X4_CLASS(PCCGaussianCurvature, 0)
{
	BEGIN_CLASS_INIT(PCCGaussianCurvature)
   ADD_FLOAT32_PROP(sampleSpacing, 0)
   ADD_FLOAT32_PROP(relativeFittingRadius, 0)
   ADD_BOOLEAN_PROP(useFittingReAlignment, 0)
   ADD_BOOLEAN_PROP(outputResampledMeshes, 0)
}

//---------------------------------------------------------------------------

PCCGaussianCurvature::PCCGaussianCurvature() 
{
   sampleSpacing = 0.1f;
   relativeFittingRadius = 3.0f;
   useFittingReAlignment = true;
   outputResampledMeshes = false;
}

void PCCGaussianCurvature::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj, this);
}

void PCCGaussianCurvature::execute(Scene *scene)
{
   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
   for (size_t i = 0; i<selection.size(); i++) {
      PointCloud *pc = getPointCloud(scene, selection[i]);
      if (pc) {
         UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
         UnstructuredInCorePointCloud *resampledMesh = NULL;
         if (upc) {
            UnstructuredInCoreTriangleMesh *mesh = dynamic_cast<UnstructuredInCoreTriangleMesh*>(upc);
            if (mesh) {
               resampledMesh = PCCTriangleMeshSampler::sampleMeshPoisson(mesh, sampleSpacing);
               if (outputResampledMeshes) {
                  addPointCloud(scene, resampledMesh, "resampledMesh");
               }
            }
            UnstructuredInCorePointCloud *integralPC = resampledMesh?resampledMesh:upc;
            UnstructuredInCorePointCloud *computePC = upc;
            
            checkAttribute(computePC, "gaussMeanCurv", 2, VAD::DATA_FORMAT_FLOAT32) ;
            checkAttribute(computePC, "principalDir1", 3, VAD::DATA_FORMAT_FLOAT32) ;
            checkAttribute(computePC, "principalDir2", 3, VAD::DATA_FORMAT_FLOAT32) ;
            
            AAT positionAAT = computePC->getAAT("position");
            if (positionAAT == NULL_AAT) {
               warning("invalid point cloud");
               continue;
            }
            AAT gaussMeanCurvAAT = computePC->getAAT("gaussMeanCurv"); 
            AAT principalDir1AAT = computePC->getAAT("principalDir1"); 
            AAT principalDir2AAT = computePC->getAAT("principalDir2"); 
           
            HierarchicalKNNIterator hIt(integralPC, 32, NULL);
            
            const float32 sigma = sampleSpacing*relativeFittingRadius;
            const mpcard numPoints = computePC->getNumPoints();
            
            for (mpcard j=0; j<numPoints; j++) {
               Vector3f point = computePC->getPointSet()->get3f(j, positionAAT);

               float meanCurvature;
               float gaussianCurvature;
               Vector3f principalDir1; 
               Vector3f principalDir2;
            
               if (useFittingReAlignment) {
                  computeCurvature_PatchFittingBoxed( 
                     &hIt, integralPC, point, sigma, 
                     meanCurvature, gaussianCurvature, principalDir1, principalDir2 );
               } else {
                  computeCurvature_PatchFitting( 
                     integralPC, point, sigma, 
                     meanCurvature, gaussianCurvature, principalDir1, principalDir2 );
               }
               
               computePC->getPointSet()->set2f(j, gaussMeanCurvAAT, makeVector2f(gaussianCurvature, meanCurvature));
               computePC->getPointSet()->set3f(j, principalDir1AAT, principalDir1);
               computePC->getPointSet()->set3f(j, principalDir2AAT, principalDir2);
            }
            
            computePC->deleteDelOnWriteAttachments();
            
            if (!outputResampledMeshes) {
               delete resampledMesh;
            }            
         }
      }
   }


}


PCCGaussianCurvature::~PCCGaussianCurvature() 
{
}

//---------------------------------------------------------------------------
