//---------------------------------------------------------------------------
//PCCTopologyAndFeaturePreservingResampling.h
//Header file for the PCCTopologyAndFeaturePreservingResampling class
//This class is basically a copy of PCCTopologyPreservingResampling, with
//an added check for a "feature flag" so that vertices marked as features
//do not get removed
//Alan Brunton, MPII, February 2013
//---------------------------------------------------------------------------
#ifndef PCCTopologyAndFeaturePreservingResamplingH
#define PCCTopologyAndFeaturePreservingResamplingH
//---------------------------------------------------------------------------
#include "threeParameterMatchingGMOD14.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
#include "UnstructuredInCorePointCloud.h"
#include "Scene.h"

//---------------------------------------------------------------------------

namespace X4 {

   class THREE_PARAMETER_MATCHING_GMOD14_API PCCTopologyAndFeaturePreservingResampling : public PCCommand 
   {
      X4_CLASS( PCCTopologyAndFeaturePreservingResampling )

   private:
      float32 m_Spacing;
      float32 m_ReconnectFactor;

	  card32	m_featureLevel;


	  //static member variables
	  static string	m_strFeatureFlag;
	     
   public:
      PCCTopologyAndFeaturePreservingResampling();
      ~PCCTopologyAndFeaturePreservingResampling();
      void execute( Scene *scene );

      void resamplePC(UnstructuredInCorePointCloud* pc);

      float32 getSpacing() const			{return m_Spacing;}
      void setSpacing(float32 newSpacing)	{ m_Spacing = newSpacing;}
      
      float32 getReconnectFactor() const	{ return m_ReconnectFactor; }
      void setReconnectFactor(float32 f)	{ m_ReconnectFactor = f; }

	  card32 getFeatureLevel() const		{ return m_featureLevel; }
	  void setFeatureLevel(card32 level)	{ m_featureLevel = max(1, (int) level); }

	  static string getFeatureFlagString()	{ return m_strFeatureFlag; }

	  
   };


}

#endif //PCCTopologyAndFeaturePreservingResamplingH
