/*  Time-stamp: <@(#)PCIGMOD14Features.h   12/9/2012 4:48 PM   Tino Weinkauf>
 *
 *********************************************************************
 *  File    : PCIGMOD14Features.h
 *
 *  Project : X4 Animation Reconstruction IV
 *
 *  Package : animRec4
 *
 *  Company : MPII
 *
 *  Author  : Tino Weinkauf                            Date: 12/9/2012
 *
 *  Purpose : Declaration of class PCIGMOD14Features
 *
 *********************************************************************
 * Version History:
 *
 * V 0.10  12/9/2012 16:48:56  TW : First Revision
 *
 *********************************************************************
 */

#ifndef PCIGMOD14FeaturesH
#define PCIGMOD14FeaturesH

#include "threeParameterMatchingGMOD14.h"
#include "X4Types.h"
#include "PCInteractionTool.h"

#include "AR4MLSManifold.h"


namespace X4
{

class AR4Settings;
class IntrinsicWavesCompute;

/** A tool to find some good starting points for the growing matcher.

	Based on intrinsic waves, or better to say: based on the characteristics
	of isocurves of the geodesic distance function. These isocurves change
	their length/circumference in a very descriptive manner. This is used here to find
	the most discriminative vertices/points on a surface.

	@see AR4IntrinsicWavesCompute

@author Tino Weinkauf
*/
class THREE_PARAMETER_MATCHING_GMOD14_API PCIGMOD14Features : public PCInteractionTool
{
	X4_CLASS( PCIGMOD14Features )

//Friends
//Types
public:

//Construction / Deconstruction
public:
	PCIGMOD14Features();
	virtual ~PCIGMOD14Features();

//Methods
public:
	

	virtual void keyDown(GeneralKey key) {forget(key);}
	virtual void keyUp(GeneralKey key) {forget(key);}
	virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {return;}
	virtual void mouseMoved(int32 x, int32 y) {return;}
	virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {return;}
	virtual void areaResize(card32 width, card32 height) {return;}

	///Precomputes descriptors for later seed finding
	void PrecomputeSeedFindingDescriptors();

	///Unwraps the feature points of the selected mesh and puts it into the scene graph for rendering
	void UnwrapFeaturePoints();

	///Finds good starting points for the growing matcher
	void FindSeeds();

	///Compute geodesic distances from start index and save in channel.
	/// Useful to determine good parameters for the actual seed finding.
	void ShowGeodesicArea();

	///Fills color channel with boundary info. Unbelievable...
	void ShowBoundary();

	///Export the lengths of the circles - of the last object that had been computed.
	void ExportCircleLengths();

	//Detect the boundaries based on number of triangles connected to the edges (waqar)
	void DetectBoundaries();

protected:
	void SetupICompute(IntrinsicWavesCompute& iComp, AR4MLSManifold3f* pManifold);

//Attributes
public:

	AR4Settings *settings;

	vector<AR4MLSManifold3f*>	manifolds;

	AR4MLSManifold3f* checkForManifold(PointCloud* pc)
	{
		for (card32 m = 0; m < manifolds.size(); m++)
		{
			if (manifolds[m]->getPointCloud() == pc)
				return manifolds[m];
		}
		return NULL;
	}

	///Scale of the intrinsic radius
	float32 IntrinsicRadiusScale;

	///scale of the models
	float32 modelScale;

	///Minimal radius to start with
	float32 MinRadius;

	///Maximal radius for growing the curves
	float32 MaxRadius;

	///Radius for clustering - everything in this radius will be absorbed as well as everything in the same radius of the absorbed.
	float32 ClusterRadius;

	///Number of isocurves to be computed between MinRadius and MaxRadius.
	card32 NumCircles;

	///Number of levels of isocurves to be computed
	card32 NumLevels;

	///level of sample spacing to be ensured -- 2^SamplingLevel * avg edge length will be the sampling density ensured
	card32 SamplingLevel;

	///Number of secondary features per primary feature (max)
	card32 NumSecondaryFeatures;

	///Twice this number of adjacent isocurves will be averaged. 
	card32 AveragingHalfWindow;

	///Number of OpenMP Threads for parallel wave computation.
	int32 NumThreads;

	///Number of significant points (points with highest descriptor difference) to be saved in the object.
	int32 NumSignificantPoints;

	///Whether or not to compute entropy as a simple diff or a more involved statistics measure
	bool bEntropyAsDiff;

	///Whether or not to visualize the descriptor difference using color
	bool bVisualizeEntropy;

	///Debug: Start only from one index
	int32 StartIndex;

protected:
	///Result of the last computation.
	vector< vector< float32 > > AllRawCircleLengths;
};

}

#endif
