//---------------------------------------------------------------------------
#ifndef PCIGMOD14ShapeMatchingH
#define PCIGMOD14ShapeMatchingH
//---------------------------------------------------------------------------
#include "threeParameterMatchingGMOD14.h"
//---------------------------------------------------------------------------
#include "X4Types.h"
#include "PCInteractionTool.h"
#include "AR4MLSManifold.h"
#include "AR4DiscreteManifold.h"
#include "AR4FuncOnManifold.h"
#include "AR4SmoothManifold.h"
//---------------------------------------------------------------------------


//types of distance measures between feature descriptors
#define AR4SM_FEATURE_DIST_L2				0
#define AR4SM_FEATURE_DIST_L1				1
#define AR4SM_FEATURE_DIST_L1_DT			2

#define AR4SM_FEATURE_DIST_TYPE				AR4SM_FEATURE_DIST_L2

//how a feature match score is interpreted -- as a priority (larger better) or a cost (smaller better)
#define AR4SM_FEATUREMATCH_SCORE_PRIORITY	0
#define AR4SM_FEATUREMATCH_SCORE_COST		1

#define AR4SM_FEATUREMATCH_SCORE_TYPE		AR4SM_FEATUREMATCH_SCORE_PRIORITY


namespace X4 {

class InCorePCTopologyGraph;

class AR4Settings;
class AR4MLSManifold3f;
class AR4PartialMatch3f;
class AR4GrowingMatcher3f;
class AR4SparseBundle3f;



class THREE_PARAMETER_MATCHING_GMOD14_API PCIGMOD14ShapeMatching :  public PCInteractionTool {
	X4_CLASS( PCIGMOD14ShapeMatching )
 private:

	class AR4SMFeatureMatch
	{
	public:
		mpcard			m_iSourceFeature;
		mpcard			m_iTargetFeature;
		mpcard			m_iSourcePoint;
		mpcard			m_iTargetPoint;
		float			m_targetDir;
		float			m_score;
		card32			m_dirOrder;

		AR4SMFeatureMatch(): m_iSourceFeature(0), m_iTargetFeature(0), m_iSourcePoint(0), m_iTargetPoint(0), m_targetDir(0.f), m_score(0.f), m_dirOrder(0) 
		{
		}
		AR4SMFeatureMatch(mpcard isource, mpcard itarget, float score):	m_iSourceFeature(isource), m_iTargetFeature(itarget), 
																		m_iSourcePoint(0), m_iTargetPoint(0), m_targetDir(0.f), m_score(score), m_dirOrder(0) 
		{
		}

		inline AR4SMFeatureMatch& operator=(const AR4SMFeatureMatch& fm)
		{
			m_iSourceFeature = fm.m_iSourceFeature;
			m_iTargetFeature = fm.m_iTargetFeature;
			m_iSourcePoint = fm.m_iSourcePoint;
			m_iTargetPoint = fm.m_iTargetPoint;
			m_targetDir = fm.m_targetDir;
			m_score = fm.m_score;
			m_dirOrder = fm.m_dirOrder;
			return *this;
		}
		inline bool operator<(const AR4SMFeatureMatch& fm) const
		{
			return (m_score < fm.m_score);
		}
		inline bool operator>(const AR4SMFeatureMatch& fm) const
		{
			return (m_score > fm.m_score);
		}

		inline void adjustScore(const float adjustment)
		{
			m_score *= adjustment;
		}

		static inline float computeScore(const float cost, const float parameter)
		{
			return exp(-cost * parameter);
		}
		static inline float adjustScore(const float score, const float adjustment)
		{
			return score * adjustment;
		}
		static inline float invertScore(const float score)
		{
			return 1.f / score;
		}
		static inline bool isScoreBetterThan(const float score, const float refScore)
		{
			return (score > refScore);
		}
		static inline float reallyBadScore()
		{
			return 1e-9f;
		}
		static inline float bestScore()
		{
			return 1.f;
		}

		//comparison classes for sorting and set operations on feature matches
		//test equality of both source and target point indices
		class PointsEqual
		{
		public:
			inline bool operator()(const AR4SMFeatureMatch& fm1, const AR4SMFeatureMatch& fm2) const
			{
				return (fm1.m_iSourcePoint == fm2.m_iSourcePoint && fm1.m_iTargetPoint == fm2.m_iTargetPoint);
			}
		};

		//test that point indices of first argument are less than second
		//(order of precedence: source, target)
		class PointsLess
		{
		public:
			inline bool operator()(const AR4SMFeatureMatch& fm1, const AR4SMFeatureMatch& fm2) const
			{
				if (fm1.m_iSourcePoint == fm2.m_iSourcePoint)
					return (fm1.m_iTargetPoint < fm2.m_iTargetPoint);
				return (fm1.m_iSourcePoint < fm2.m_iSourcePoint);
			}
		};

		//test that point indices and direction order of first argument are less than second
		//(order of precedence: source points, target points, direction orders)
		class PointsDirOrdersLess
		{
		public:
			inline bool operator()(const AR4SMFeatureMatch& fm1, const AR4SMFeatureMatch& fm2) const
			{
				if (fm1.m_iSourcePoint == fm2.m_iSourcePoint)
				{
					if (fm1.m_iTargetPoint == fm2.m_iTargetPoint)
						return (fm1.m_dirOrder < fm2.m_dirOrder);
					return (fm1.m_iTargetPoint < fm2.m_iTargetPoint);
				}
				return (fm1.m_iSourcePoint < fm2.m_iSourcePoint);
			}
		};

		//test that targetDir of first argument less than that of second
		class TargetDirLess
		{
		public:
			inline bool operator()(const AR4SMFeatureMatch& fm1, const AR4SMFeatureMatch& fm2) const
			{
				return (fm1.m_targetDir < fm2.m_targetDir);
			}
		};

		static inline void sortBestFirst(vector<AR4SMFeatureMatch>::iterator start, vector<AR4SMFeatureMatch>::iterator stop)
		{
			sort(start, stop, greater<AR4SMFeatureMatch>());
		}
	};

	typedef set<AR4SMFeatureMatch, AR4SMFeatureMatch::PointsDirOrdersLess> TExistFMSet;

	typedef pair<mpcard, float>				TIndexCost;

	AR4Settings *settings;
	string target;
	card32 mode;
	mpint selPoint;
	float32 exDetRadius;

	//added by abrunton
	mpint									m_sourceSelPoint;
	mpint									m_sourceDirPoint;
	mpint									m_targetDirPoint;
	Vector3f									m_sourceDir;
	Vector3f									m_targetDir;

	//added by abrunton
	string source;

	bool leftDown;
	AR4MLSManifold3f						*manifold;

	//added by abrunton
	AR4MLSManifold3f*						m_pSourceManifold;
	AR4MLSManifold3f*						m_pTargetManifold;
	AR4PartialMatch3f*						m_pMatch;
	AR4GrowingMatcher3f*					m_pGrower;
	AR4SparseBundle3f*						m_pGlobalBundle;

	InCorePCTopologyGraph**					m_ppSubSampleSource;
	InCorePCTopologyGraph**					m_ppSubSampleTarget;
	mpcard									m_nSubSampleLevels;
	mpcard									m_nGeodesicsNN;

	vector<AR4PartialMatch3f*>				m_partialMatches;

	string									m_strRenderJobName;
	string									m_strBaseIndexAttribName;
	string									m_strSparseTopoName;

	float									m_avgDenseEdgeLength;

	bool									m_bDataInit;

	bool									m_bUseGlobalBundle;
	mpcard									m_currentGrowingStep;//this counter is now used to determine the current visualization frame of debug renderer
	card32									m_growingMode;

	//features
	//basic storage, dimensions, etc.
	mpcard									m_nSourceFeatures;
	mpcard									m_nTargetFeatures;
	mpcard									m_nDescriptorDim;
	mpcard									m_nDescriptorLevels;
	UnstructuredInCorePointCloud*			m_pSourceFeatures;
	UnstructuredInCorePointCloud*			m_pTargetFeatures;

	//Cartesian product of descriptor distances
	vector<float>							m_featureDistance;

	//order in which feature matches should be used to initialize growing matcher
	//store as a vector and sort, because we have to periodically update
	vector<AR4SMFeatureMatch>				m_featureMatchQueue;
	mpcard									m_nextFeatureMatchIndex;
	vector<bool>							m_featureMatchDirty;

	//parameter for descriptor distance
	float									m_matchSigma;

	//secondary feature data
	vector<vector<float>>					m_sourceSecondaryFeatureDesc;
	vector<vector<float>>					m_targetSecondaryFeatureDesc;
	vector<vector<card32>>					m_sourceSecondaryFeatureIndex;
	vector<vector<card32>>					m_targetSecondaryFeatureIndex;
	vector<vector<float>>					m_sourceSecondaryFeatureEntropy;
	vector<vector<float>>					m_targetSecondaryFeatureEntropy;

	//set of feature matches already tried
	TExistFMSet								m_processedFeatures;

	bool									m_bFeatureQueueReady;

	float									m_geoDiameter;
	bool									m_bErrorNormByArea;
	float									m_surfaceArea;

	//information for storing matching results
	string									m_strResultFolder;
	string									m_strPartialSequence;
	mpcard									m_partialMatchCount;
	string									m_strClusterSequence;
	string									m_strDirSwapPath;

	//information for clustering and merging partial matches
	vector<vector<mpcard>>					m_partialMatchClusters;

	card32									m_rejectToVisualize;

	//error and stretch visualization
	vector<float>							m_matchesStretch;
	bool									m_bStretchDirty;
	vector<float>							m_matchesError;
	bool									m_bErrorDirty;


	//private/protected member functions
	AR4MLSManifold3f *createM3f();

	//overloaded version added by abrunton
	AR4MLSManifold3f* createM3f(std::string strPC);

	bool select(int32 x, int32 y);
	void calcDist();

	void initVisualization();
	void visualizeMatching(AR4PartialMatch3f* pMatch);
	void visualizePointAndDirMatch(const Vector3f& sourcePoint, const Vector3f& sourceDir, const Vector3f& targetPoint, const Vector3f& targetDir);

	void computeTargetSurfaceArea();

	void evalGroundTruthErrorInternal(AR4PartialMatch3f* pMatch, mpcard counter);
	void colorTargetByCorrespondenceInternal(UnstructuredInCorePointCloud* pMatchPC);
	void colorTargetByCorrespondenceInternalHighRes(AR4PartialMatch3f* pMatch);

	void computeSourceAlignment(Vector3f& centroid, Matrix3f& rotation, Vector3f& scaling);

	void checkForFeatures();
	void initFeatureMatching();
	void collectSecondariesFromPrimaries();
	void matchFeatures();
	void primaryFeatureDescCartProdStratified();
	void primaryFeatureDescCartProdMultiScale();
	void matchFeaturesBestStratified(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue);
	void matchFeaturesBestMultiScale(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue);
	void matchFeaturesPriorityStratified(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue);
	void matchFeaturesPriorityMultiScale(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue);
	void matchFeaturesClusterStratified(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue);
	void matchFeaturesClusterMultiScale(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue);
	void matchFeaturesForwardStratified(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue, vector<pair<mpcard, mpcard>>* pRefMatches = NULL);

	void featureDescCartProdStratified(const vector<float>& descriptors1, const vector<float>& descriptors2, vector<float>& cartProdDist);
	void featureDescCartProdMultiScale(const vector<float>& descriptors1, const vector<float>& descriptors2, vector<float>& cartProdDist);
	bool getBestFeatureMatch(vector<AR4SMFeatureMatch>& featureQueue, mpcard& nextFeatureMatchIndex, TExistFMSet& existingMatches, AR4SMFeatureMatch& fm);

	bool getBestFeatureMatch(AR4SMFeatureMatch& fm);
	void getMatchDirections(const AR4SMFeatureMatch& fm, Vector3f& sourceDir, vector<Vector3f>& targetDirs);
	void getMatchDirections(vector<AR4SMFeatureMatch>& featureMatchQueue, const mpcard posInQueue);

	void savePartialMatch(AR4PartialMatch3f* pMatch);
	inline float computeFeatureMatchCoverage(AR4PartialMatch3f* pMatch, const AR4SMFeatureMatch& fm);
	void updateFeatureMatchQueue(AR4PartialMatch3f* pMatch);

	void visualizeFeatureMatchSequence();
	void visualizeFeatureMatchSequenceWithDirections();

	static inline float descriptorDistance(const mpcard descLen, const float* pDesc1, const float* pDesc2)
	{
		float dist = 0.f;
#if AR4SM_FEATURE_DIST_TYPE == AR4SM_FEATURE_DIST_L2
		float diff;
		for (mpcard i = 0; i < descLen; i++)
		{
			diff = pDesc1[i] - pDesc2[i];
			dist += diff*diff;
		}
#elif AR4SM_FEATURE_DIST_TYPE == AR4SM_FEATURE_DIST_L1
		float diff;
		for (mpcard i = 0; i < descLen; i++)
		{
			diff = pDesc1[i] - pDesc2[i];
			dist += fabs(diff);
		}
#elif AR4SM_FEATURE_DIST_TYPE == AR4SM_FEATURE_DIST_L1_DT
		float diff;
		vector<float> distField(descLen);
		for (mpcard i = 0; i < descLen; i++)
		{
			diff = pDesc1[i] - pDesc2[i];
			distField[i] = fabs(diff);
		}
		for (mpcard i = 1; i < descLen; i++)
		{
			distField[i] = min(distField[i], distField[i - 1] + 1.f);
		}
		dist = distField[descLen - 1];
		for (mpint i = descLen - 2; i >= 0; i--)
		{
			distField[i] = min(distField[i], distField[i + 1] + 1.f);
			dist += distField[i];
		}
#endif //AR4SM_FEATURE_DIST_TYPE
		return dist;
	}

	static inline float descriptorDistanceMultiScale(const mpcard levels, const mpcard descLen, const float* pDesc1, const float* pDesc2)
	{
		mpcard i, offset = 0;
		float distLevel;
		float distSum = 0.f;
		mpcard levelCount = 0;
		for (i = 0; i < levels; i++)
		{
			if (pDesc1[offset + descLen - 1] < 0.f || pDesc2[offset + descLen - 1] < 0.f)
				break;

			distLevel = descriptorDistance(descLen, pDesc1 + offset, pDesc2 + offset);
			distSum += distLevel;
			levelCount++;
			offset += descLen;
		}
		return distSum / (float)levelCount;
	}

	inline bool wasFeatureMatchAlreadyTried(const AR4SMFeatureMatch& fm, const TExistFMSet& processedFeatures)
	{
		return (processedFeatures.find(fm) != processedFeatures.end());
	}


protected:

 public:

	//added by abrunton
	const static card32					m_PCIARSMSourcePt1	= 0;
	const static card32					m_PCIARSMSourcePt2	= 1;
	const static card32					m_PCIARSMTargetPt1	= 2;
	const static card32					m_PCIARSMTargetPt2	= 3;
	const static card32					m_PCIARSRejectFind	= 4;
	const static card32					m_PCIARSNumModes	= 5;

	PCIGMOD14ShapeMatching();

	virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	virtual void keyDown(GeneralKey key);
	virtual void keyUp(GeneralKey key);

	virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
	virtual void mouseMoved(int32 x, int32 y);
	virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
	virtual void areaResize(card32 width, card32 height);
	virtual void mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState);



	//main functions that computes correspondence
	void computeCorrespondence();

	//initialize scene graph directories that we expect
	void initSceneStructure();

	//initialize data - essentially construct manifolds from point clouds
	void initData();

	//initialize matching objects and algorithms
	void initMatching();
	void initMatch(AR4PartialMatch3f* pMatch, const mpcard sourceIndex, const Vector3f& sourceDir, const Vector3f& targetPoint, const Vector3f& targetDir);

	//generate sub-sampled hierarchy of point clouds
	void generateSubSampleHierarchy();

	//pre-compute geodesics on source manifold -- on target they must be computed on the fly
	void precomputeSourceGeodesics();

	//run the growing algorithm (local fitting + occasional global fitting)
	void runGrowingMatcher();
	void runGrowingMatcherOneStep();
	void runGrowingMatcherNextFeature();
	void runFullMatcher();

	
	//embed/cluster/visualize partial matches
	void clusterPartialMatches();

	//merge partial matches
	void mergePartialMatches();

	//cluster individual correspondences
	void clusterPointMatches();

	//merger directly from partials using belief propagation
	void mergePartialsDirectBP();

	//reset/initialize the feature match queue
	void resetFeatureMatchQueue();

	//if we have TOSCA (eg) models with same point indices, we can evaluate ground truth error
	void evalGroundTruthError();
	void collectBenchmarkError();

	//save correspondences as a point cloud in scene graph
	void saveCorrespondences();

	//load correspondences computed from a previous X4 session
	void loadCorrespondences();

	//set the color of matching points to their error relative to ground truth
	void colorMatchesByError();

	//set the color the matching points to their stretch factor measured against the other matches
	void colorMatchesByStretch();

	//set the color of the matching points to the 3D position on the source surface
	void colorMatchesByPosition();

	//interpolate correspondence to get result at full point cloud resolution
	void interpolateMatching();

	//upsample point clouds (using MLS) for nice visualization
	void upsamplePointClouds();

	//set the color channel of the surfaces themselves according to the computed correspondences
	void colorSourceByCheckerVoxels();
	void colorTargetByCorrespondence();

	//load correspondences compute externally by another method to evaluate
	void loadMatchesBIM();
	void loadMatchesAurela();
	void loadMatchesAvinash();
	void loadMatchesArt();
	void loadExternalResult();

	//visualize the points rejected as outliers in reverse order
	void visualizeReject();

	virtual ~PCIGMOD14ShapeMatching();

};


}

#endif
