//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PCIGMOD14ShapeMatching.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "AR4Settings.h"
#include "AR4MLSManifold.h"
#include "basics/DeformationTools.h"
#include "AR4MLSManifold.h"
#include "InCorePCTopologyGraph.h"
#include "TopologyAttachment.h"
#include "AR4DijkstraDistPC3f.h"
#include "ColorTools.h"
#include "PCCComputeTopology.h"
#include "PCCEstimateSampleSpacing.h"
#include "PCCTopologyPreservingResampling.h"
#include "PCCTopologyAndFeaturePreservingResampling.h"
#include <ObjectAttachment.h>
#include <AttachedFloatSignatures.h>
#include <SimpleAttachment.h>
#include "ObjectViewsTable.h"
#include "SGListNode.h"
#include "GeometricTools.h"
#include "PCCSurfaceTools.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
#include "SGRelativeTimeAnimationNode.h"
#include "AR4PartialMatch.h"
#include "AR4PartialMatch.inline.h"
#include "AR4IncrementalMatcher.h"
#include "AR4IncrementalMatcher.inline.h"
#include "AR4GDLCluster.h"
#include "AR4PartialMatchClustering.h"
#include "AR4PartialMatchMerger.h"
#include "AR4PartialMatchMaterial.h"
#include "TopologyTools.h"
#include "DebugRenderer.h"
#include "Timer.h"
#include "FileDialogs.h"
#include "Configuration.h"
#include "BinaryObjectStreams.h"
#include "HashTables.h"
#include "userInterface/PCCBoundaryDetector.h"
#include <fstream>


//---------------------------------------------------------------------------
#define AR4_DEBUG_RENDER					0
#define AR4_INIT_GENERAL					1
#define AR4_RECORD_REJECTED_OUTLIERS		0
#define AR4_DEFAULT_POSITION_COLOR_FREQ		4
#define AR4_MATCH_FEATURES_STRATAFIED		1
#define AR4_MATCH_FEATURES_CLUSTER			1
#define AR4_DEFAULT_NUM_ERR_THRESH			10000


//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
using namespace X4;
//---------------------------------------------------------------------------

static string descrMode(card32 i)
{
	//modified by abrunton
	if (i == PCIGMOD14ShapeMatching::m_PCIARSMSourcePt1)	return "source point 1";
	if (i == PCIGMOD14ShapeMatching::m_PCIARSMSourcePt2)	return "source point 2";
	if (i == PCIGMOD14ShapeMatching::m_PCIARSMTargetPt1)	return "target point 1";
	if (i == PCIGMOD14ShapeMatching::m_PCIARSMTargetPt2)	return "target point 2";
	if (i == PCIGMOD14ShapeMatching::m_PCIARSRejectFind)	return "find rejected outlier";
	return "undefined";
}

static string descrGrowingMode(card32 mode)
{
	if (mode == (card32)AR4GrowingMatcher3f::eGMRigidProjection)		return "rigid + projection";
	if (mode == (card32)AR4GrowingMatcher3f::eGMGeodesicTriangulation)	return "geodesic triangulation";
	if (mode == (card32)AR4GrowingMatcher3f::eGMParallelTransport)		return "parallel transport";
	return "undefined";
}

IMPLEMENT_X4_CLASS(PCIGMOD14ShapeMatching, 1)
{
	BEGIN_CLASS_INIT(PCIGMOD14ShapeMatching)
	ADD_OBJECT_PROP(settings, 0, AR4Settings::getClass(), true)
	ADD_STRING_PROP(source, 0)//abrunton
	ADD_STRING_PROP(target, 0)
	ADD_CARD32_PROP_UPDATE_DESCR(mode, 0, NULL, descrMode, m_PCIARSNumModes, NULL)
	ADD_MAX_PLATFORM_INT_PROP(m_sourceSelPoint, 0)
	ADD_MAX_PLATFORM_INT_PROP(m_sourceDirPoint, 0)
	ADD_MAX_PLATFORM_INT_PROP(selPoint, 0)
	ADD_MAX_PLATFORM_INT_PROP(m_targetDirPoint, 0)
	ADD_MAX_PLATFORM_CARD_PROP(m_nSubSampleLevels, 0)
	ADD_FLOAT32_PROP(m_matchSigma, 0)
	ADD_BOOLEAN_PROP(m_bUseGlobalBundle, 0)
	ADD_CARD32_PROP_UPDATE_DESCR(m_growingMode, 0, NULL, descrGrowingMode, 3, NULL)
	ADD_STRING_PROP(m_strDirSwapPath, 0)
	ADD_NOARGS_METHOD(computeCorrespondence)
	ADD_NOARGS_METHOD(initData)
	ADD_NOARGS_METHOD(resetFeatureMatchQueue)
	ADD_NOARGS_METHOD(runGrowingMatcher)
	ADD_NOARGS_METHOD(runGrowingMatcherNextFeature)
	ADD_NOARGS_METHOD(runFullMatcher)
	ADD_NOARGS_METHOD(clusterPartialMatches)
	ADD_NOARGS_METHOD(mergePartialMatches)
	ADD_NOARGS_METHOD(evalGroundTruthError)
	ADD_NOARGS_METHOD(collectBenchmarkError)
	ADD_NOARGS_METHOD(saveCorrespondences)
	ADD_NOARGS_METHOD(loadCorrespondences)
	ADD_NOARGS_METHOD(interpolateMatching)
	ADD_NOARGS_METHOD(upsamplePointClouds)
	ADD_NOARGS_METHOD(colorSourceByCheckerVoxels)
	ADD_NOARGS_METHOD(colorTargetByCorrespondence)
	ADD_NOARGS_METHOD(loadExternalResult)
}

//---------------------------------------------------------------------------

PCIGMOD14ShapeMatching::PCIGMOD14ShapeMatching() 
{
   settings = new AR4Settings();
   manifold = NULL;
   source = "root/pc0";//abrunton
   target = "root/pc1";
   mode = m_PCIARSMSourcePt1;

   //the following indices are temporary for easy testing purposes
   m_sourceSelPoint = 3205;
   m_sourceDirPoint = 3221;
   selPoint = 3205;
   m_targetDirPoint = 3221;

   m_avgDenseEdgeLength = 1.f;

   m_pSourceManifold = NULL;
   m_pTargetManifold = NULL;
   m_pMatch = NULL;
   m_pGrower = NULL;
   m_pGlobalBundle = NULL;
   leftDown = false;
   exDetRadius = 0.03f;
   m_nSubSampleLevels = 4;
   m_ppSubSampleSource = NULL;
   m_ppSubSampleTarget = NULL;
   m_nGeodesicsNN = 10;
   m_strRenderJobName = "GMOD14ShapeMatching";
   m_strBaseIndexAttribName = "ar4_base_index";
   m_strSparseTopoName = "ar4_sparse_topology";

   m_bUseGlobalBundle = true;
   m_currentGrowingStep = 0;

   m_growingMode = (card32)AR4GrowingMatcher3f::eGMParallelTransport;

   m_nSourceFeatures = 0;
   m_nTargetFeatures = 0;
   m_nDescriptorDim = 0;
   m_nDescriptorLevels = 1;
   m_pSourceFeatures = NULL;
   m_pTargetFeatures = NULL;
   m_matchSigma = 0.05f;
   m_bFeatureQueueReady = false;

   m_nextFeatureMatchIndex = 0;

   m_partialMatchCount = 0;

   m_geoDiameter = 0.f;
   m_bErrorNormByArea = true;
   m_surfaceArea = 0.f;

   m_rejectToVisualize = 0;

   m_strResultFolder = "ar4result";
   m_strPartialSequence = "partialMatches";
   m_strClusterSequence = "clusteredMatches";

   StandardDirectoriesConfig *att = dynamic_cast<StandardDirectoriesConfig*>(Configuration::getConfiguration()->getData("StandardDirectories"));
	if (att) {m_strDirSwapPath = att->openDir;}

   m_bStretchDirty = false;
   m_bErrorDirty = false;

   m_bDataInit = false;
}

void PCIGMOD14ShapeMatching::assign(const Object* obj, X4CopyContext *context)
{
	copyObjectProperties(obj, this);
}


AR4MLSManifold3f *PCIGMOD14ShapeMatching::createM3f()
{
   PointCloud *pc = getPointCloud(scene, target);
   if (!pc) {
      throw ERangeCheck("PCIGMOD14ShapeMatching::createM3f() - target is not a point cloud.");
   }
   UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
   if (!upc) {
      throw ERangeCheck("PCIGMOD14ShapeMatching::createM3f() - target must currently be unstructured PC.");
   }
   InCorePCTopologyGraph *tg = getTopology(pc);
   if (!tg) {
      PCCComputeTopology ctop;
      ctop.setup(PCCComputeTopology::TOPTYPE_KNN, 0.1f, settings->numNN);
      tg = ctop.computeKNNTopology(upc->getPointSet(), upc);
      attachTopology(upc, tg);
   }
   
   if (!upc->getPointSet()->providesAttribute("sampleSpacing")) {
      PCCEstimateSampleSpacing::estimateSampleSpacing(upc, settings->numNN);
   }
      
   return new AR4MLSManifold3f(upc);
}

AR4MLSManifold3f* PCIGMOD14ShapeMatching::createM3f(std::string strPC)
{
   PointCloud *pc = getPointCloud(scene, strPC);
   checkAttribute(pc, PCCTopologyAndFeaturePreservingResampling::getFeatureFlagString(), 1, VAD::DATA_FORMAT_CARD32);
   return AR4MLSManifold3f::createFromPointCloud(pc, settings->numNN, 0.1f);

}

PCIGMOD14ShapeMatching::~PCIGMOD14ShapeMatching() 
{
   delete settings;
   delete manifold;
   delete m_pSourceManifold;
   delete m_pTargetManifold;
   delete m_pMatch;
   delete m_pGrower;
   delete m_pGlobalBundle;

}

//---------------------------------------------------------------------------

bool PCIGMOD14ShapeMatching::select(int32 x, int32 y) {
   Vector3f retPoint; 
   bool hit = guessClickPoint(x,y, 5, retPoint);
   float32 oldDist = 1e20f;

   mpcard i, n;
   mpcard idirsel;
   Vector3f p, d;
   float32 distq;
   std::vector<Vector3f> destpoints;
   std::vector<std::vector<int32>> paths;
   std::vector<mpcard> nearest;
   PointCloudIterator* pciter = NULL;
   float dist;
   Vector3f grad;
   vector<mpcard> path;

   mpcard srcidx;
   float stretch;

   if (m_pMatch == NULL)
	   throw EInvalidState("PCIGMOD14ShapeMatching::select(...): data and matching must be initialized first");

   switch (mode)
   {
   case m_PCIARSMSourcePt1:
	   //initialize the visualization here, will be clear once the matcher is started
	   initVisualization();

	   m_sourceSelPoint = -1;
	   if (hit && m_pSourceManifold != NULL)
	   {
		   n = m_pSourceManifold->getNumDiscrPoints();
		   for (i = 0; i < n; i++)
		   {
			   p = m_pSourceManifold->getDiscrPoint(i);
			   distq = normQuad(p - retPoint);
			   if (distq < oldDist && m_pMatch->isAtMatchingLevel(i))
			   {
				   m_sourceSelPoint = i;
				   oldDist = distq;
			   }
		   }
#if AR4_DEBUG_RENDER
		   debugRenderer->addSphere(m_pSourceManifold->getDiscrPoint(m_sourceSelPoint), m_avgDenseEdgeLength, makeVector3f(0.f, 0.f, 1.f));
		   debugRenderer->endRenderJob();
#endif //AR4_DEBUG_RENDER
		   mode = m_PCIARSMSourcePt2;
		   return true;
	   }
	   break;
   case m_PCIARSMSourcePt2:
	   idirsel = -1;
	   if (hit && m_pSourceManifold != NULL)
	   {
		   n = m_pSourceManifold->getNumDiscrPoints();
		   for (i = 0; i < n; i++)
		   {
			   p = m_pSourceManifold->getDiscrPoint(i);
			   distq = normQuad(p - retPoint);
			   if (distq < oldDist)
			   {
				   idirsel = i;
				   oldDist = distq;
			   }
		   }

		   //compute geodesic path and set source direction
		   pciter = m_pSourceManifold->getTopology()->createIterator(m_pSourceManifold->getPointSet());
		   if (pciter == NULL)
		   {
			   throw ERangeCheck("PCIGMOD14ShapeMatching::select(...): m_pSourceManifold->getTopology()->createIterator(m_pSourceManifold->getPointSet()) returned NULL");
		   }
		   destpoints.clear();
		   destpoints.push_back(m_pSourceManifold->getDiscrPoint(idirsel));
		   m_pMatch->getSparseSourceGeodesics()->computeGeodesicSingleDest(0.1f, m_sourceSelPoint, idirsel, dist, grad, &path);

		   if (path.size() > 1)
		   {
#if AR4_DEBUG_RENDER
			   debugRenderer->beginRenderJob(m_strRenderJobName);
			   float t, rcpSize = 1.f / (float)path.size();
			   for (i = 0; i < path.size(); i++)
			   {
				   t = (float)i * rcpSize;
				   debugRenderer->addPoint(m_pSourceManifold->getDiscrPoint(path[i]), makeVector3f(1.f, 0.f, 0.f) * t + makeVector3f(0.f, 1.f, 0.f) * (1.f - t));
			   }
#endif //AR4_DEBUG_RENDER

#if AR4_INIT_GENERAL
			   m_sourceDirPoint = path[1];
#else
			   for (i = 0; i < path.size(); i++)
			   {
				   m_sourceDirPoint = m_pMatch->findNearestSourcePointAtMatchingLevel(path[i]);
				   if (m_sourceDirPoint != m_sourceSelPoint)
					   break;
			   }
#endif //AR4_INIT_GENERAL

#if AR4_DEBUG_RENDER
			   debugRenderer->addSphere(m_pSourceManifold->getDiscrPoint(m_sourceDirPoint), 0.5f * m_avgDenseEdgeLength, makeVector3f(0.f, 1.f, 0.f));
			   debugRenderer->endRenderJob();
#endif //AR4_DEBUG_RENDER

#if AR4_DEBUG_OUTPUT
			   debugOutput << "point selected: " << idirsel << "; direction point: " << m_sourceDirPoint;
			   debugOutput.cr();
#endif //AR4_DEBUG_OUTPUT

			   m_sourceDir = m_pSourceManifold->getDiscrPoint(m_sourceDirPoint) - m_pSourceManifold->getDiscrPoint(m_sourceSelPoint);

#if AR4_DEBUG_OUTPUT
			   debugOutput << "source dir: " << m_sourceDir;
			   debugOutput.cr();
#endif //AR4_DEBUG_OUTPUT
		   }
		   else
		   {
			   throw ERangeCheck("PCIGMOD14ShapeMatching::select(...): no path found on source manifold");
		   }

		   delete pciter;

		   //update mode
		   mode = m_PCIARSMTargetPt1;
		   return true;
	   }
	   break;
   case m_PCIARSMTargetPt1:
	   selPoint = -1;
	   if (hit && m_pTargetManifold != NULL)
	   {
		   n = m_pTargetManifold->getNumDiscrPoints();
		   for (i = 0; i < n; i++)
		   {
			   p = m_pTargetManifold->getDiscrPoint(i);
			   distq = normQuad(p - retPoint);
			   if (distq < oldDist)
			   {
				   selPoint = i;
				   oldDist = distq;
			   }
		   }
#if AR4_DEBUG_RENDER
		   debugRenderer->beginRenderJob(m_strRenderJobName);
		   debugRenderer->addSphere(m_pTargetManifold->getDiscrPoint(selPoint), m_avgDenseEdgeLength, makeVector3f(0.f, 0.f, 1.f));
		   debugRenderer->endRenderJob();
#endif //AR4_DEBUG_RENDER
		   mode = m_PCIARSMTargetPt2;
		   return true;
	   }
	   break;
   case m_PCIARSMTargetPt2:
	   idirsel = -1;
	   if (hit && m_pTargetManifold != NULL)
	   {
		   n = m_pTargetManifold->getNumDiscrPoints();
		   for (i = 0; i < n; i++)
		   {
			   p = m_pTargetManifold->getDiscrPoint(i);
			   distq = normQuad(p - retPoint);
			   if (distq < oldDist)
			   {
				   idirsel = i;
				   oldDist = distq;
			   }
		   }

		   //compute geodesic path and set source direction
		   pciter = m_pTargetManifold->getTopology()->createIterator(m_pTargetManifold->getPointSet());
		   if (pciter == NULL)
		   {
			   throw ERangeCheck("PCIGMOD14ShapeMatching::select(...): m_pTargetManifold->getTopology()->createIterator(m_pTargetManifold->getPointSet()) returned NULL");
		   }
		   destpoints.clear();
		   destpoints.push_back(m_pTargetManifold->getDiscrPoint(idirsel));
		   m_pMatch->getSparseTargetGeodesics()->computeGeodesicSingleDest(0.1f, selPoint, idirsel, dist, grad, &path);

		   if (path.size() > 1)
		   {
#if AR4_DEBUG_RENDER
			   debugRenderer->beginRenderJob(m_strRenderJobName);
			   float t, rcpSize = 1.f / (float)path.size();
			   for (i = 0; i < path.size(); i++)
			   {
				   t = (float)i * rcpSize;
				   debugRenderer->addPoint(m_pTargetManifold->getDiscrPoint(path[i]), makeVector3f(1.f, 0.f, 0.f) * t + makeVector3f(0.f, 1.f, 0.f) * (1.f - t));
			   }
#endif //AR4_DEBUG_RENDER

			   m_targetDirPoint = path[1];
#if AR4_INIT_GENERAL
#else
			   d = (m_pTargetManifold->getDiscrPoint(m_targetDirPoint) - m_pTargetManifold->getDiscrPoint(selPoint));
			   d.normalize();
			   d *= m_sourceDir.getNorm();
			   m_pTargetManifold->project(m_pTargetManifold->getDiscrPoint(selPoint) + d, p);
			   m_pTargetManifold->getKNearestNeighborsEucl(p, 1, nearest);
			   m_targetDirPoint = nearest[0];
#endif //AR4_INIT_GENERAL

#if AR4_DEBUG_RENDER
			   debugRenderer->addSphere(m_pTargetManifold->getDiscrPoint(m_targetDirPoint), 0.5f * m_avgDenseEdgeLength, makeVector3f(0.f, 1.f, 0.f));
			   debugRenderer->endRenderJob();
#endif //AR4_DEBUG_RENDER

#if AR4_DEBUG_OUTPUT
			   debugOutput << "point selected: " << idirsel << "; direction point: " << m_targetDirPoint;
			   debugOutput.cr();
#endif //AR4_DEBUG_OUTPUT

			   m_targetDir = m_pTargetManifold->getDiscrPoint(m_targetDirPoint) - m_pTargetManifold->getDiscrPoint(selPoint);
			   m_targetDir.normalize();
			   m_sourceDir.normalize();

#if AR4_DEBUG_OUTPUT
			   debugOutput << "target dir: " << m_targetDir;
			   debugOutput.cr();
#endif //AR4_DEBUG_OUTPUT
		   }
		   else
		   {
			   throw ERangeCheck("PCIGMOD14ShapeMatching::select(...): no path found on target manifold");
		   }

		   delete pciter;

		   mode = m_PCIARSMSourcePt1;
		   return true;
	   }
	   break;
   case m_PCIARSRejectFind:
	   srcidx = m_pMatch->findNearestSourcePointAtMatchingLevel(retPoint);
	   if (!m_pMatch->isMatched(srcidx))
	   {
		   stretch = m_pMatch->getStretch(srcidx);
		   if (stretch >= 0.f)
		   {
			   m_rejectToVisualize = srcidx;
			   visualizeReject();
		   }
	   }
	   break;
   }

   return false;
}

void PCIGMOD14ShapeMatching::mouseDown( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState )
{
	if (buttonsState.getLeft())
	{
		leftDown = true;
		select(x, y);
		mouseDownPassToCam(x,y,buttonsState,modifiersState);
	}
	else
	{
		mouseDownPassToCam(x,y,buttonsState,modifiersState);
	}

	ObjectViewsTable::update(this);
}

void PCIGMOD14ShapeMatching::mouseMoved(int32 x, int32 y) 
{
   bool handled = false;
   if (leftDown) {
      if (mode == 1 && selPoint != -1) {
         if (!manifold) manifold = createM3f();
         if (manifold && select(x, y)) {
            calcDist();
            handled = true;
         }
      }
   } 
   if (!handled) mouseMovedPassToCam(x,y);

   ObjectViewsTable::update(this);
}
void PCIGMOD14ShapeMatching::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
   leftDown = false;
   mouseUpPassToCam(x,y,buttonsState,modifiersState);
}


void X4::PCIGMOD14ShapeMatching::calcDist()
{
   if (manifold && selPoint > -1) {
      mpcard n = manifold->getNumDiscrPoints();
      if (selPoint < (mpint)n) {
         AR4DijkstraDistPC3f dist(manifold);
         dist.initialize();
         dist.compute(selPoint);
         UnstructuredInCorePointCloud *pc = manifold->getPointCloud();
         
         checkAttribute(pc, "color", 3, VAD::DATA_FORMAT_FLOAT32);
         manifold->resetup(pc); // reassign pointers
         AAT distColAAT = pc->getAAT("color");
         for (mpcard i=0; i<n; i++) {
            Vector3f col = rainbowGradient3f(dist.evaluate(i) / dist.getMaxValue());
            pc->getPointSet()->set3f(i, distColAAT, col);
            pc->deleteDelOnWriteAttachments();
         }
      } else {
         message("point out of range");
      }
   } else {
      message("not initialized (press setup) / no point selected (click on model)");
   }
}

void PCIGMOD14ShapeMatching::initVisualization()
{
#if AR4_DEBUG_RENDER
	debugRenderer->clearDebugData(m_strRenderJobName);
	debugRenderer->beginRenderJob(m_strRenderJobName);
#endif //AR4_DEBUG_RENDER
}

void PCIGMOD14ShapeMatching::visualizeMatching(AR4PartialMatch3f* pMatch)
{
#if AR4_DEBUG_RENDER
	mpcard i, k, n;
	Vector3f vstart, vsm, vtm;
	float32 t, rcpn;
	Vector3f color, red, green, blue;

	float colorFreq = (float)AR4_DEFAULT_POSITION_COLOR_FREQ;
	float colorPeriod = 1.f / colorFreq;

	red.setZero();
	green.setZero();
	blue.setZero();
	red[0] = 1.f;
	green[1] = 1.f;
	blue[2] = 1.f;

	vstart = m_pSourceManifold->getDiscrPoint(m_sourceSelPoint);
	n = pMatch->getNumMatchedIndices();
	rcpn = 1.f / (float32)n;

	debugRenderer->beginRenderJob_OneFrame(m_strRenderJobName, m_currentGrowingStep);

	float smallrad = m_avgDenseEdgeLength * 1.f, bigrad = m_avgDenseEdgeLength * 1.5f;
	float midrad = 0.5f * (smallrad + bigrad);
	for (i = 0; i < n; i++)
	{
		k = pMatch->getMatchedSourceIndex(i);
		vsm = m_pSourceManifold->getDiscrPoint(k);
		vtm = pMatch->getMatch(k);

		color = vsm - m_pSourceManifold->getBoundingBox().getCenter();
		color[0] /= m_pSourceManifold->getBoundingBox().getSideLength(0);
		color[1] /= m_pSourceManifold->getBoundingBox().getSideLength(1);
		color[2] /= m_pSourceManifold->getBoundingBox().getSideLength(2);
		color[0] += 0.5f;
		color[1] += 0.5f;
		color[2] += 0.5f;
		color[0] = fmod(color[0], colorPeriod) * colorFreq;
		color[1] = fmod(color[1], colorPeriod) * colorFreq;
		color[2] = fmod(color[2], colorPeriod) * colorFreq;

		debugRenderer->addSphere(vsm, midrad, color);
		debugRenderer->addSphere(vtm, midrad, color);
	}

	debugRenderer->endRenderJob();
	redrawScene();

	m_currentGrowingStep++;
#endif //AR4_DEBUG_RENDER
}

void PCIGMOD14ShapeMatching::visualizePointAndDirMatch(const Vector3f& sourcePoint, const Vector3f& sourceDir, const Vector3f& targetPoint, const Vector3f& targetDir)
{
#if AR4_DEBUG_RENDER
	debugRenderer->beginRenderJob_OneFrame(m_strRenderJobName, m_currentGrowingStep);

	debugRenderer->addSphere(sourcePoint, m_avgDenseEdgeLength * 1.5f, makeVector3f(0.f, 0.f, 1.f));
	debugRenderer->addFineArrow(sourcePoint, sourcePoint + sourceDir * m_avgDenseEdgeLength * 6.f, makeVector3f(0.f, 0.f, 0.f), m_avgDenseEdgeLength * 0.25f);
	debugRenderer->addSphere(targetPoint, m_avgDenseEdgeLength * 1.5f, makeVector3f(0.f, 0.f, 1.f));
	debugRenderer->addFineArrow(targetPoint, targetPoint + targetDir * m_avgDenseEdgeLength * 6.f, makeVector3f(0.f, 0.f, 0.f), m_avgDenseEdgeLength * 0.25f);

	debugRenderer->endRenderJob();
	redrawScene();

	m_currentGrowingStep++;
#endif //AR4_DEBUG_RENDER
}

void PCIGMOD14ShapeMatching::computeTargetSurfaceArea()
{
	PointCloud* pc = m_pTargetManifold->getPointCloud();
	UnstructuredInCoreTriangleMesh* ptm = dynamic_cast<UnstructuredInCoreTriangleMesh*>(pc);
	if (ptm != NULL)
	{
		//we have a triangle mesh, compute the area by summing the area of the triangles
		mpcard t, numtri = ptm->getNumTriangles();
		PointSet* ptris = ptm->getTriangles();
		PointSet* ppts = ptm->getPointSet();
		const VertexDescriptor* ptrivd = ptm->getTriangleDescr();
		AAT IND = ptris->getAAT("index");
		AAT POS = ppts->getAAT("position");
		float areatri;

		m_surfaceArea = 0.f;
		for (t = 0; t < numtri; t++)
		{
			Vector3i indices = ptris->get3i(t, IND);
			Vector3f v1 = ppts->get3f(indices[0], POS);
			Vector3f v2 = ppts->get3f(indices[1], POS);
			Vector3f v3 = ppts->get3f(indices[2], POS);
			areatri = (v3 - v1).crossProduct(v2 - v1).getNorm() * 0.5f;
			m_surfaceArea += areatri;
		}

		debugOutput << "target surface area = " << m_surfaceArea << "\n";
	}
	else
	{
		warning("PCIGMOD14ShapeMatching::computeTargetSurfaceArea(): surface area computation for manifolds other than triangle meshes not implemented yet.");
	}
}

void PCIGMOD14ShapeMatching::evalGroundTruthErrorInternal(AR4PartialMatch3f* pMatch, mpcard counter)
{
	mpcard i, k, n;
	Vector3f matchPos, gtVertPos;

	float diameter, sqrtArea, factor, normFactor;
	float distToTruth, diff;
	float errSum = 0.f, errMin = 1e30f, errMax = 0.f;
	float errMean, errStd;

	float thresholds[4] = { 0.01f, 0.05f, 0.1f, 0.2f };
	mpcard countAbove[4] = { 0 };

	if (m_bErrorNormByArea)
	{
		//compute surface area of target to scale errors
		if (m_surfaceArea == 0.f)
			computeTargetSurfaceArea();
		sqrtArea = sqrt(m_surfaceArea);
		factor = sqrtArea;
		normFactor = 1.f / sqrtArea;
	}
	else
	{
		//compute geodesic diameter to scale errors
		if (m_geoDiameter == 0.f)
			m_geoDiameter = pMatch->targetGeodesicDiameter();
		diameter = m_geoDiameter;
		factor = diameter;
		normFactor = 1.f / diameter;
	}

	//loop over matches and compute distance to same index point on target
	n = pMatch->getNumMatchedIndices();
	m_matchesError.resize(n);
	for (i = 0; i < n; i++)
	{
		k = pMatch->getMatchedSourceIndex(i);
		matchPos = pMatch->getMatch(k);
		gtVertPos = m_pTargetManifold->getDiscrPoint(k);

		distToTruth = pMatch->targetGeodesic(matchPos, gtVertPos);

		m_matchesError[i] = distToTruth;
		errSum += distToTruth;
		if (distToTruth < errMin)	errMin = distToTruth;
		if (distToTruth > errMax)	errMax = distToTruth;

		if (distToTruth > thresholds[0] * factor)
			countAbove[0]++;
		if (distToTruth > thresholds[1] * factor)
			countAbove[1]++;
		if (distToTruth > thresholds[2] * factor)
			countAbove[2]++;
		if (distToTruth > thresholds[3] * factor)
			countAbove[3]++;
	}

	errMean = errSum / (float)n;

	errStd = 0.f;
	for (i = 0; i < n; i++)
	{
		diff = m_matchesError[i] - errMean;
		errStd += diff*diff;
	}

	errStd /= (float)n;
	errStd = sqrt(errStd);

	errSum *= normFactor;
	errMean *= normFactor;
	errStd *= normFactor;
	errMin *= normFactor;
	errMax *= normFactor;

	string strFactor;
	debugOutput << "Matching error measured by geodesic distance between match and ground-truth:\n";
	debugOutput << "Number of matches: " << n << "\n";
	if (m_bErrorNormByArea)
	{
		strFactor = "sqrt(A)";
		debugOutput << "square root of surface area of target (A): " << sqrtArea << "\n";
	}
	else
	{
		strFactor = "D";
		debugOutput << "geodesic diameter of target (D): " << diameter << "\n";
	}
	debugOutput << "Following statistics given as a fraction of " << strFactor << ":\n";
	debugOutput << "\tsum = " << errSum << "\n";
	debugOutput << "\tavg = " << errMean << "\n";
	debugOutput << "\tstd = " << errStd << "\n";
	debugOutput << "\tmin = " << errMin << "\n";
	debugOutput << "\tmax = " << errMax << "\n";
	debugOutput << "\tnumber of matches with error above " << thresholds[0] << strFactor << " = " << countAbove[0] << " (" << 100.f * (float)countAbove[0] / (float)n << " %)\n";
	debugOutput << "\tnumber of matches with error above " << thresholds[1] << strFactor << " = " << countAbove[1] << " (" << 100.f * (float)countAbove[1] / (float)n << " %)\n";
	debugOutput << "\tnumber of matches with error above " << thresholds[2] << strFactor << " = " << countAbove[2] << " (" << 100.f * (float)countAbove[2] / (float)n << " %)\n";
	debugOutput << "\tnumber of matches with error above " << thresholds[3] << strFactor << " = " << countAbove[3] << " (" << 100.f * (float)countAbove[3] / (float)n << " %)\n";
	
	debugOutput.flush();

	std::ofstream fileCumulativeDist;
	fileCumulativeDist.open((m_strDirSwapPath + "ar4MatchErrCumulativeDistribution" + intToStr(counter, 4) + ".txt").c_str(), std::ios_base::out);

	const mpcard numErrThresh = AR4_DEFAULT_NUM_ERR_THRESH;
	const float rcpNumErrThresh = 1.f / (float)numErrThresh;
	const float errStep = factor * rcpNumErrThresh;
	float errT, errFrac, pct;
	mpcard countLess;

	fileCumulativeDist << "0 0\n";
	for (i = 1; i <= numErrThresh; i++)
	{
		errT = errStep * (float)i;
		errFrac = rcpNumErrThresh * (float)i;
		countLess = 0;
		for (k = 0; k < n; k++)
		{
			if (m_matchesError[k] < errT)
				countLess++;
		}

		pct = ((float)countLess / (float)n) * 100.f;

		fileCumulativeDist << errFrac << " " << pct << "\n";
	}

	fileCumulativeDist.close();

	//for computing target coverage
	//collect all matches on target
	//compute distance field for entire target surface (at vertices)
	debugOutput << "collecting matches\n"; debugOutput.flush();
	const mpcard numMatches = pMatch->getNumMatchedIndices();
	vector<Vector3f> zeroSet(numMatches);
	for (i = 0; i < numMatches; i++)
		zeroSet[i] = pMatch->getMatch(pMatch->getMatchedSourceIndex(i));
	
	debugOutput << "distance field\n"; debugOutput.flush();
	pMatch->getSparseTargetGeodesics()->distanceTransform(zeroSet);

	debugOutput << "threshold\n"; debugOutput.flush();
	const float32 coverThresh = pMatch->getAvgSourceMatchLevelEdgeWeight();

	debugOutput << "coverage\n"; debugOutput.flush();
	const mpcard numTargetPoints = m_pTargetManifold->getNumDiscrPoints();
	mpcard numCover = 0;
	for (i = 0; i < numTargetPoints; i++)
	{
		if (pMatch->getSparseTargetGeodesics()->getDistanceField(i) <= coverThresh)
			numCover++;
	}

	const float32 fracCover = (float32)numCover / (float32)numTargetPoints;

	std::ofstream fileCoverage;
	fileCoverage.open((m_strDirSwapPath + "ar4MatchCoverage" + intToStr(counter, 4) + ".txt").c_str(), std::ios_base::out);

	fileCoverage << pMatch->getNumMatchedIndices() << "\n";
	fileCoverage << errMean << "\n";
	fileCoverage << errStd << "\n";
	fileCoverage << errMin << "\n";
	fileCoverage << errMax << "\n";
	fileCoverage << fracCover << "\n";

	fileCoverage.close();

	m_bErrorDirty = false;
}

void PCIGMOD14ShapeMatching::checkForFeatures()
{
	mpcard i;
	AAT ffaat;
	bool bSourceHasFeatures = false, bTargetHasFeatures = false;

	//reset feature flags
	ffaat = m_pSourceManifold->getPointCloud()->getAAT(PCCTopologyAndFeaturePreservingResampling::getFeatureFlagString());
	for (i = 0; i < m_pSourceManifold->getNumDiscrPoints(); i++)
	{
		m_pSourceManifold->getPointSet()->set1u(i, ffaat, 0);
	}
	ffaat = m_pTargetManifold->getPointCloud()->getAAT(PCCTopologyAndFeaturePreservingResampling::getFeatureFlagString());
	for (i = 0; i < m_pTargetManifold->getNumDiscrPoints(); i++)
	{
		m_pTargetManifold->getPointSet()->set1u(i, ffaat, 0);
	}

	if (settings->featureMatchSource == AR4Settings::fmSrcExternal)
	{
		ifstream fmfile((m_strDirSwapPath + settings->strExtSparseCorrFile).c_str());

		const mpcard numSourcePoints = m_pSourceManifold->getNumDiscrPoints();
		const mpcard numTargetPoints = m_pTargetManifold->getNumDiscrPoints();

		map<mpcard, mpcard> sourcePointToFeature, targetPointToFeature;

		AAT ffaats = m_pSourceManifold->getPointCloud()->getAAT(PCCTopologyAndFeaturePreservingResampling::getFeatureFlagString());
		AAT ffaatt = m_pTargetManifold->getPointCloud()->getAAT(PCCTopologyAndFeaturePreservingResampling::getFeatureFlagString());

		card32 numLinesFile, numColsFile;
		fmfile >> numLinesFile;
		fmfile >> numColsFile;

		m_nSourceFeatures = 0;
		m_nTargetFeatures = 0;
		mpcard iSrcFeat = 0;
		for (i = 0; i < numTargetPoints && fmfile.good(); i++)
		{
			int32 idx, icorr;
			fmfile >> idx;
			fmfile >> icorr;
			if (icorr != -1 && idx != -1)
			{
				
				iSrcFeat = m_nSourceFeatures;
				m_nSourceFeatures++;
				

				m_pSourceManifold->getPointSet()->set1u((mpcard)icorr, ffaats, m_nSubSampleLevels);
				m_pTargetManifold->getPointSet()->set1u((mpcard)i, ffaatt, m_nSubSampleLevels);

				AR4SMFeatureMatch fm;
				fm.m_iSourceFeature = iSrcFeat;
				fm.m_iTargetFeature = m_nTargetFeatures;
				fm.m_iSourcePoint = icorr;
				fm.m_iTargetPoint = i;
				fm.m_score = AR4SMFeatureMatch::bestScore();

				m_featureMatchQueue.push_back(fm);

				m_nTargetFeatures++;
			}
		}

		if (fmfile.bad())
			throw EInvalidState("PCIGMOD14ShapeMatching::checkForFeatures(): error reading external feature match file");

		return;
	}

	card32 nSourceDescDim, nTargetDescDim, nSourceDescLevels, nTargetDescLevels;

	//check source for features
	UnstructuredInCorePointCloud* pPointCloud = m_pSourceManifold->getPointCloud();
	ObjectAttachment* pAttachment = dynamic_cast< ObjectAttachment* >(pPointCloud->getAttachments()->getData("FeaturePoints"));
	bSourceHasFeatures = (pAttachment != NULL);
	if (bSourceHasFeatures)
	{
		//read source features
		UnstructuredInCorePointCloud* pFeaturePoints = dynamic_cast< UnstructuredInCorePointCloud* >(pAttachment->getObject());
		if (pFeaturePoints != NULL)
		{
			AAT idxAAT = pFeaturePoints->getAAT("index");
			ffaat = m_pSourceManifold->getPointCloud()->getAAT(PCCTopologyAndFeaturePreservingResampling::getFeatureFlagString());
			m_nSourceFeatures = pFeaturePoints->getNumPoints();
			for (i = 0; i < m_nSourceFeatures; i++)
			{
				m_pSourceManifold->getPointSet()->set1u(pFeaturePoints->getPointSet()->get1u(i, idxAAT), ffaat, m_nSubSampleLevels);
			}
		}

		m_pSourceFeatures = pFeaturePoints;

		//check feature parameters
		AttachedCard32Vector* pFeatureParams = dynamic_cast< AttachedCard32Vector* >(pFeaturePoints->getAttachments()->getData("FeatureParameters"));
		if (pFeatureParams != NULL && pFeatureParams->data.size() >= 2)
		{
			nSourceDescDim = pFeatureParams->data[0];
			nSourceDescLevels = pFeatureParams->data[1];
		}
		else
		{
			//set to "null" settings
			nSourceDescDim = 0;
			nSourceDescLevels = 0;
		}
	}

	//check target for features
	pPointCloud = m_pTargetManifold->getPointCloud();
	pAttachment = dynamic_cast< ObjectAttachment* >(pPointCloud->getAttachments()->getData("FeaturePoints"));
	bTargetHasFeatures = (pAttachment != NULL);
	if (bTargetHasFeatures)
	{
		//read target features
		UnstructuredInCorePointCloud* pFeaturePoints = dynamic_cast< UnstructuredInCorePointCloud* >(pAttachment->getObject());
		if (pFeaturePoints != NULL)
		{
			AAT idxAAT = pFeaturePoints->getAAT("index");
			ffaat = m_pTargetManifold->getPointCloud()->getAAT(PCCTopologyAndFeaturePreservingResampling::getFeatureFlagString());
			m_nTargetFeatures = pFeaturePoints->getNumPoints();
			for (i = 0; i < m_nTargetFeatures; i++)
			{
				m_pTargetManifold->getPointSet()->set1u(pFeaturePoints->getPointSet()->get1u(i, idxAAT), ffaat, m_nSubSampleLevels);
			}
		}

		m_pTargetFeatures = pFeaturePoints;

		//check feature parameters
		AttachedCard32Vector* pFeatureParams = dynamic_cast< AttachedCard32Vector* >(pFeaturePoints->getAttachments()->getData("FeatureParameters"));
		if (pFeatureParams != NULL && pFeatureParams->data.size() >= 2)
		{
			nTargetDescDim = pFeatureParams->data[0];
			nTargetDescLevels = pFeatureParams->data[1];
		}
		else
		{
			//set to "null" settings
			nTargetDescDim = 0;
			nTargetDescLevels = 0;
		}
	}

	if (bSourceHasFeatures && bTargetHasFeatures)
	{
		if (nTargetDescDim != nSourceDescDim || nTargetDescLevels != nSourceDescLevels)
		{
			m_nDescriptorDim = 0;
			m_nDescriptorLevels = 1;
			warning("PCIGMOD14ShapeMatching::checkForFeatures(): feature descriptor parameters on source and target do not match");
		}
		else
		{
			m_nDescriptorDim = nSourceDescDim;
			m_nDescriptorLevels = nSourceDescLevels;
			debugOutput << "PCIGMOD14ShapeMatching::checkForFeatures(): features are of dimension " << m_nDescriptorDim << " at " << m_nDescriptorLevels << " levels\n";
		}
	}
}

void PCIGMOD14ShapeMatching::initFeatureMatching()
{
	mpcard i, j, k, m, n, o;
	mpcard soff, toff;

	if (m_pSourceFeatures == NULL || m_pTargetFeatures == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::initFeatureMatching(): cannot match features, null feature pointer");

	AttachedFloatVector* pSourceDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pSourceFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));
	AttachedFloatVector* pTargetDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pTargetFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));

	AttachedCard32Vector* pSourceSecondCountAtt = dynamic_cast< AttachedCard32Vector* >(m_pSourceFeatures->getAttachments()->getData("SecondaryFeatureCount"));
	AttachedCard32Vector* pTargetSecondCountAtt = dynamic_cast< AttachedCard32Vector* >(m_pTargetFeatures->getAttachments()->getData("SecondaryFeatureCount"));

	AttachedFloatVector* pSourceSecondDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pSourceFeatures->getAttachments()->getData("IntrinsicWavesDescriptor_secondary"));
	AttachedFloatVector* pTargetSecondDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pTargetFeatures->getAttachments()->getData("IntrinsicWavesDescriptor_secondary"));

	UnstructuredInCorePointCloud* pSourceSecondFeatures = NULL;
	UnstructuredInCorePointCloud* pTargetSecondFeatures = NULL;

	ObjectAttachment* pAttachment = dynamic_cast< ObjectAttachment* >(m_pSourceManifold->getPointCloud()->getAttachments()->getData("SecondaryFeaturePoints"));
	if (pAttachment != NULL)
		pSourceSecondFeatures = dynamic_cast< UnstructuredInCorePointCloud* >(pAttachment->getObject());

	pAttachment = dynamic_cast< ObjectAttachment* >(m_pTargetManifold->getPointCloud()->getAttachments()->getData("SecondaryFeaturePoints"));
	if (pAttachment != NULL)
		pTargetSecondFeatures = dynamic_cast< UnstructuredInCorePointCloud* >(pAttachment->getObject());


	if (pSourceDescrAtt == NULL || pTargetDescrAtt == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::initFeatureMatching(): cannot match features, missing descriptors");

	bool bSeconds = true;
	if (pSourceSecondFeatures == NULL || pTargetSecondFeatures == NULL || 
		pSourceSecondCountAtt == NULL || pTargetSecondCountAtt == NULL || 
		pSourceSecondDescrAtt == NULL || pTargetSecondDescrAtt == NULL)
		bSeconds = false;

	if (m_nDescriptorDim == 0)
	{
		m_nDescriptorDim = pSourceDescrAtt->data.size() / m_nSourceFeatures;
		if ((pTargetDescrAtt->data.size() / m_nTargetFeatures) != m_nDescriptorDim)
			throw ERangeCheck("PCIGMOD14ShapeMatching::initFeatureMatching(): features have difference dimensionality on source and target");
	}

	m_sourceSecondaryFeatureDesc.resize(m_nSourceFeatures);
	m_sourceSecondaryFeatureIndex.resize(m_nSourceFeatures);
	m_sourceSecondaryFeatureEntropy.resize(m_nSourceFeatures);
	m_targetSecondaryFeatureDesc.resize(m_nTargetFeatures);
	m_targetSecondaryFeatureIndex.resize(m_nTargetFeatures);
	m_targetSecondaryFeatureEntropy.resize(m_nTargetFeatures);

	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
#if AR4_MATCH_FEATURES_STRATAFIED
	const mpcard targetFeatStep = m_nTargetFeatures * m_nDescriptorLevels;
#else
	const mpcard targetFeatStep = m_nTargetFeatures;
#endif //AR4_MATCH_FEATURES_STRATAFIED

#if AR4_MATCH_FEATURES_STRATAFIED
	primaryFeatureDescCartProdStratified();
#else
	primaryFeatureDescCartProdMultiScale();
#endif

	if (bSeconds)
	{
		
		//collect secondary feature descriptors
		AAT idxs2AAT = pSourceSecondFeatures->getAAT("index");
		AAT idxt2AAT = pTargetSecondFeatures->getAAT("index");
		AAT ents2AAT = pSourceSecondFeatures->getAAT("entropy");
		AAT entt2AAT = pTargetSecondFeatures->getAAT("entropy");

		AAT sffaat = m_pSourceManifold->getPointCloud()->getAAT(PCCTopologyAndFeaturePreservingResampling::getFeatureFlagString());
		AAT tffaat = m_pTargetManifold->getPointCloud()->getAAT(PCCTopologyAndFeaturePreservingResampling::getFeatureFlagString());

		n = 0;
		k = 0;
		for (i = 0; i < m_nSourceFeatures; i++)
		{
			m_sourceSecondaryFeatureIndex[i].resize(pSourceSecondCountAtt->data[i]);
			m_sourceSecondaryFeatureEntropy[i].resize(pSourceSecondCountAtt->data[i]);
			m_sourceSecondaryFeatureDesc[i].resize(pSourceSecondCountAtt->data[i] * descriptorStep);
			for (m = 0; m < m_sourceSecondaryFeatureIndex[i].size(); m++)
			{
				m_sourceSecondaryFeatureIndex[i][m] = pSourceSecondFeatures->getPointSet()->get1u(n, idxs2AAT);

				card32 featFlag = m_pSourceManifold->getPointSet()->get1u(m_sourceSecondaryFeatureIndex[i][m], sffaat);
				featFlag = max((int) featFlag, 1);
	
				m_sourceSecondaryFeatureEntropy[i][m] = pSourceSecondFeatures->getPointSet()->get1f(n, ents2AAT);
				for (j = 0; j < descriptorStep; j++)
				{
					m_sourceSecondaryFeatureDesc[i][m * descriptorStep + j] = pSourceSecondDescrAtt->data[k];
					k++;
				}
				n++;
			}
		}

		n = 0;
		k = 0;
		for (i = 0; i < m_nTargetFeatures; i++)
		{
			m_targetSecondaryFeatureIndex[i].resize(pTargetSecondCountAtt->data[i]);
			m_targetSecondaryFeatureEntropy[i].resize(pTargetSecondCountAtt->data[i]);
			m_targetSecondaryFeatureDesc[i].resize(pTargetSecondCountAtt->data[i] * descriptorStep);
			for (m = 0; m < m_targetSecondaryFeatureIndex[i].size(); m++)
			{
				m_targetSecondaryFeatureIndex[i][m] = pTargetSecondFeatures->getPointSet()->get1u(n, idxt2AAT);

				card32 featFlag = m_pTargetManifold->getPointSet()->get1u(m_targetSecondaryFeatureIndex[i][m], tffaat);
				featFlag = max((int) featFlag, 1);
	
				m_targetSecondaryFeatureEntropy[i][m] = pTargetSecondFeatures->getPointSet()->get1f(n, entt2AAT);
				for (j = 0; j < descriptorStep; j++)
				{
					m_targetSecondaryFeatureDesc[i][m * descriptorStep + j] = pTargetSecondDescrAtt->data[k];
					k++;
				}
				n++;
			}
		}
	}
}

void PCIGMOD14ShapeMatching::matchFeatures()
{
	mpcard i, j, k, m, n, o;
	mpcard soff, toff;
	float diff, descDist;

	if (m_pSourceFeatures == NULL || m_pTargetFeatures == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::matchFeatures(): cannot match features, null feature pointer");

	AttachedFloatVector* pSourceDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pSourceFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));
	AttachedFloatVector* pTargetDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pTargetFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));

	if (pSourceDescrAtt == NULL || pTargetDescrAtt == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::matchFeatures(): cannot match features, missing descriptors");

	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
#if AR4_MATCH_FEATURES_STRATAFIED
	const mpcard targetFeatStep = m_nTargetFeatures * m_nDescriptorLevels;
#else
	const mpcard targetFeatStep = m_nTargetFeatures;
#endif //AR4_MATCH_FEATURES_STRATAFIED

	//here is where we initialize the priority queue
	AAT entropyAAT = m_pSourceFeatures->getAAT("entropy");
	AAT idxsAAT = m_pSourceFeatures->getAAT("index");
	AAT idxtAAT = m_pTargetFeatures->getAAT("index");

	vector<float> sourceEntropies(m_nSourceFeatures);
	vector<mpcard> sourceIndices(m_nSourceFeatures);
	vector<mpcard> targetIndices(m_nTargetFeatures);

	for (i = 0; i < m_nSourceFeatures; i++)
	{
		sourceIndices[i] = m_pSourceFeatures->getPointSet()->get1u(i, idxsAAT);
		sourceEntropies[i] = m_pSourceFeatures->getPointSet()->get1f(i, entropyAAT);
	}

	for (i = 0; i < m_nTargetFeatures; i++)
	{
		targetIndices[i] = m_pTargetFeatures->getPointSet()->get1u(i, idxtAAT);
	}

	X4_TIMER_START(matchFeatures);
	matchFeaturesForwardStratified(m_nSourceFeatures, m_nTargetFeatures, sourceIndices, targetIndices, sourceEntropies, m_featureDistance, m_featureMatchQueue);
	X4_TIMER_STOP(matchFeatures);
	visualizeFeatureMatchSequenceWithDirections();
}

void PCIGMOD14ShapeMatching::collectSecondariesFromPrimaries()
{
	mpcard i, j;

	if (settings->featureMatchSource != AR4Settings::fmSrcInternal)
	{
		set<mpcard> sourceFeaturesEncountered, targetFeaturesEncountered;

		m_sourceSecondaryFeatureIndex.resize(m_nSourceFeatures);
		m_targetSecondaryFeatureIndex.resize(m_nTargetFeatures);

		const mpcard nQueueLen = m_featureMatchQueue.size();

		for (i = 0; i < nQueueLen; i++)
		{
			AR4SMFeatureMatch fm = m_featureMatchQueue[i];

			const mpcard sourceFeature = fm.m_iSourceFeature;
			const mpcard sourceIdx = fm.m_iSourcePoint;
			const mpcard targetFeature = fm.m_iTargetFeature;

			bool bSourceNew = sourceFeaturesEncountered.find(sourceFeature) == sourceFeaturesEncountered.end();
			if (bSourceNew)
				targetFeaturesEncountered.insert(sourceFeature);
			bool bTargetNew = targetFeaturesEncountered.find(targetFeature) == targetFeaturesEncountered.end();
			if (bTargetNew)
				targetFeaturesEncountered.insert(targetFeature);

			if (bSourceNew || bTargetNew)
			{
				vector<mpcard> sourceDests;
				vector<float> sourceDists;
				vector<Vector3f> sourceGrads;
				m_pMatch->getSparseSourceGeodesics()->getGeodesicData(sourceIdx, sourceDests, sourceDists, sourceGrads);

			
				const mpcard numDests = sourceDests.size();
				mpcard k;
				for (j = 0; j < numDests; j++)
				{
					for (k = 0; k < nQueueLen; k++)
					{
						if (k == i)	continue;

						AR4SMFeatureMatch fmOther = m_featureMatchQueue[k];
						if (fmOther.m_iSourcePoint == sourceDests[j])
						{
							if (bSourceNew)
								m_sourceSecondaryFeatureIndex[sourceFeature].push_back(sourceDests[j]);
							if (bTargetNew)
								m_targetSecondaryFeatureIndex[targetFeature].push_back(fmOther.m_iTargetPoint);
						}
					}
				}
			}
		}

		return;
	}

	vector<mpcard> sourceIndices(m_nSourceFeatures);
	vector<mpcard> targetIndices(m_nTargetFeatures);
	vector<float> sourceEntropies(m_nSourceFeatures);

	map<mpcard, mpcard> sourcePointToFeature;
	map<mpcard, mpcard> targetPointToFeature;

	AAT entropyAAT = m_pSourceFeatures->getAAT("entropy");
	AAT idxsAAT = m_pSourceFeatures->getAAT("index");
	AAT idxtAAT = m_pTargetFeatures->getAAT("index");

	mpcard totalSeconds = 0;
	for (i = 0; i < m_nSourceFeatures; i++)
	{
		sourceIndices[i] = m_pSourceFeatures->getPointSet()->get1u(i, idxsAAT);
		sourcePointToFeature[sourceIndices[i]] = i;

		sourceEntropies[i] = m_pSourceFeatures->getPointSet()->get1f(i, entropyAAT);

		totalSeconds += m_sourceSecondaryFeatureIndex[i].size();
	}
	for (i = 0; i < m_nTargetFeatures; i++)
	{
		targetIndices[i] = m_pTargetFeatures->getPointSet()->get1u(i, idxtAAT);
		targetPointToFeature[targetIndices[i]] = i;

		totalSeconds += m_targetSecondaryFeatureIndex[i].size();
	}

	if (totalSeconds > 0)
	{
		debugOutput << "PCIGMOD14ShapeMatching::collectSecondariesFromPrimaries(): secondary features provided externally.\n";
		return;
	}

	AttachedFloatVector* pSourceDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pSourceFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));
	AttachedFloatVector* pTargetDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pTargetFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));

	if (pSourceDescrAtt == NULL || pTargetDescrAtt == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::matchFeatures(): cannot match features, missing descriptors");

	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;

	AR4SparseGeodesics* pSrcGeos = m_pMatch->getSparseSourceGeodesics();
	for (i = 0; i < m_nSourceFeatures; i++)
	{
		vector<mpcard> sourceDests;
		vector<float> sourceDists;
		vector<Vector3f> sourceGrads;

		pSrcGeos->getGeodesicData(sourceIndices[i], sourceDests, sourceDists, sourceGrads);

		const mpcard numDests = sourceDests.size();
		for (j = 0; j < numDests; j++)
		{
			if (sourcePointToFeature.find(sourceDests[j]) != sourcePointToFeature.end())
			{
				m_sourceSecondaryFeatureIndex[i].push_back(sourceDests[j]);
			}
		}

		const mpcard numSeconds = m_sourceSecondaryFeatureIndex[i].size();

		m_sourceSecondaryFeatureDesc[i].resize(numSeconds * descriptorStep);
		m_sourceSecondaryFeatureEntropy[i].resize(numSeconds);

		vector<float>::iterator sdIt = m_sourceSecondaryFeatureDesc[i].begin();
		for (j = 0; j < numSeconds; j++)
		{
			const mpcard ptidx = m_sourceSecondaryFeatureIndex[i][j];
			const mpcard featidx = sourcePointToFeature[ptidx];
			m_sourceSecondaryFeatureEntropy[i][j] = sourceEntropies[featidx];
			sdIt = std::copy(	pSourceDescrAtt->data.begin() + featidx * descriptorStep, 
								pSourceDescrAtt->data.begin() + (featidx + 1) * descriptorStep, sdIt);
		}
	}

	const float radius = m_avgDenseEdgeLength * (float)(1 << m_nSubSampleLevels) * 2.f;

	AR4SparseGeodesics* pTrgGeos = m_pMatch->getSparseTargetGeodesics();
	for (i = 0; i < m_nTargetFeatures; i++)
	{
		vector<mpcard> zeroSet(1, targetIndices[i]);
		pTrgGeos->distanceTransform(zeroSet);

		for (j = 0; j < m_nTargetFeatures; j++)
		{
			if (j == i)	continue;

			if (pTrgGeos->getDistanceField(targetIndices[j]) < radius)
			{
				m_targetSecondaryFeatureIndex[i].push_back(targetIndices[j]);
			}
		}

		const mpcard numSeconds = m_targetSecondaryFeatureIndex[i].size();

		m_targetSecondaryFeatureDesc[i].resize(numSeconds * descriptorStep);
		m_targetSecondaryFeatureEntropy[i].resize(numSeconds);

		vector<float>::iterator sdIt = m_targetSecondaryFeatureDesc[i].begin();
		for (j = 0; j < numSeconds; j++)
		{
			const mpcard ptidx = m_targetSecondaryFeatureIndex[i][j];
			const mpcard featidx = targetPointToFeature[ptidx];
			sdIt = std::copy(	pTargetDescrAtt->data.begin() + featidx * descriptorStep, 
								pTargetDescrAtt->data.begin() + (featidx + 1) * descriptorStep, sdIt);
		}
	}
}

void PCIGMOD14ShapeMatching::primaryFeatureDescCartProdStratified()
{
	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
	const mpcard targetFeatStep = m_nTargetFeatures * m_nDescriptorLevels;

	mpcard i, j, level, soff, toff;

	m_featureDistance.resize(m_nSourceFeatures * targetFeatStep);
	m_featureMatchDirty.resize(m_nSourceFeatures * targetFeatStep, false);

	AttachedFloatVector* pSourceDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pSourceFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));
	AttachedFloatVector* pTargetDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pTargetFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));

	soff = 0;
	for (i = 0; i < m_nSourceFeatures; i++)
	{
		toff = 0;
		for (j = 0; j < m_nTargetFeatures; j++)
		{
			mpcard level;
			for (level = 0; level < m_nDescriptorLevels; level++)
			{
				float featdist;
				if (pSourceDescrAtt->data[soff + (level + 1) * m_nDescriptorDim - 1] < 0.f || pTargetDescrAtt->data[toff + (level + 1) * m_nDescriptorDim - 1] < 0.f)
					featdist = numeric_limits< float32 >::quiet_NaN();
				else
					featdist = descriptorDistance(m_nDescriptorDim, &(pSourceDescrAtt->data[soff + level * m_nDescriptorDim]), &(pTargetDescrAtt->data[toff + level * m_nDescriptorDim]));
				m_featureDistance[i * targetFeatStep + j * m_nDescriptorLevels + level] = featdist;
			}

			//increment target counter
			toff += descriptorStep;
		}

		//increment source counter
		soff += descriptorStep;
	}
}

void PCIGMOD14ShapeMatching::primaryFeatureDescCartProdMultiScale()
{
	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
	const mpcard targetFeatStep = m_nTargetFeatures;

	mpcard i, j, level, soff, toff;

	m_featureDistance.resize(m_nSourceFeatures * targetFeatStep);
	m_featureMatchDirty.resize(m_nSourceFeatures * targetFeatStep, false);

	AttachedFloatVector* pSourceDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pSourceFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));
	AttachedFloatVector* pTargetDescrAtt = dynamic_cast< AttachedFloatVector* >(m_pTargetFeatures->getAttachments()->getData("IntrinsicWavesDescriptor"));

	soff = 0;
	for (i = 0; i < m_nSourceFeatures; i++)
	{
		toff = 0;
		for (j = 0; j < m_nTargetFeatures; j++)
		{
			//compute distance between these descriptors at multiple scales
			m_featureDistance[i * m_nTargetFeatures + j] = descriptorDistanceMultiScale(m_nDescriptorLevels, m_nDescriptorDim, 
																						&(pSourceDescrAtt->data[soff]), &(pTargetDescrAtt->data[toff]));

			//increment target counter
			toff += descriptorStep;
		}

		//increment source counter
		soff += descriptorStep;
	}
}

void PCIGMOD14ShapeMatching::matchFeaturesBestStratified(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue)
{
	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
	const mpcard targetFeatStep = nTargetFeatures * m_nDescriptorLevels;

	mpcard i, j, k;
	mpcard idxs, idxt;

	const float rcp2MatchSigma2 = 1.f / (2.f * m_matchSigma*m_matchSigma);
	float entropyMax, rcpEntropyMax;
	float startPriority;

	vector<AR4SMFeatureMatch> featureMatchesInit(nSourceFeatures * m_nDescriptorLevels);

	k = 0;
	for (i = 0; i < nSourceFeatures; i++)
	{
		idxs = sourceIndices[i];

		const mpcard kSourceStart = k;

		float entropyPriority = sourceEntropies[i];
		if (i == 0)
		{
			entropyMax = entropyPriority;
			rcpEntropyMax = 1.f / entropyMax;
		}
		entropyPriority *= rcpEntropyMax;

		mpcard level;
		for (level = 0; level < m_nDescriptorLevels; level++)
		{
			float descDistLevelMin = 1e30f;
			mpcard iLevelBestPoint = m_pTargetManifold->getNumDiscrPoints();
			mpcard iLevelBestFeature = nTargetFeatures;

			for (j = 0; j < nTargetFeatures; j++)
			{
				idxt = targetIndices[j];

				const card32 idist = i * targetFeatStep + j * m_nDescriptorLevels + level;
				float matchPriority;

				if (_finite(cartProdDist[idist]))
				{
					if (cartProdDist[idist] < descDistLevelMin)
					{
						descDistLevelMin = cartProdDist[idist];
						iLevelBestPoint = idxt;
						iLevelBestFeature = j;
					}
				}
			}//for (j...

			if (iLevelBestPoint < m_pTargetManifold->getNumDiscrPoints())
			{
				featureMatchesInit[k].m_iSourceFeature = i;
				featureMatchesInit[k].m_iTargetFeature = iLevelBestFeature;
				featureMatchesInit[k].m_iSourcePoint = idxs;
				featureMatchesInit[k].m_iTargetPoint = iLevelBestPoint;
				featureMatchesInit[k].m_score = entropyPriority * exp(-descDistLevelMin * rcp2MatchSigma2);
			}
			else
			{
				featureMatchesInit[k].m_score = 0.f;
			}

			k++;
		}//for (level...
	}

	
	featureMatchQueue = featureMatchesInit;
	AR4SMFeatureMatch::sortBestFirst(featureMatchQueue.begin(), featureMatchQueue.end());
}

void PCIGMOD14ShapeMatching::matchFeaturesBestMultiScale(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue)
{
	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
	const mpcard targetFeatStep = nTargetFeatures;

	mpcard i, j, k;
	mpcard idxs, idxt;

	const float rcp2MatchSigma2 = 1.f / (2.f * m_matchSigma*m_matchSigma);
	float entropyMax, rcpEntropyMax;
	float startPriority;

	vector<AR4SMFeatureMatch> featureMatchesInit(cartProdDist.size());

	k = 0;
	for (i = 0; i < nSourceFeatures; i++)
	{
		idxs = sourceIndices[i];

		const mpcard kSourceStart = k;

		float entropyPriority = sourceEntropies[i];
		if (i == 0)
		{
			entropyMax = entropyPriority;
			rcpEntropyMax = 1.f / entropyMax;
		}
		entropyPriority *= rcpEntropyMax;

		float descDistLevelMin = 1e30f;
		mpcard iLevelBestPoint = m_pTargetManifold->getNumDiscrPoints();
		mpcard iLevelBestFeature = nTargetFeatures;

		for (j = 0; j < nTargetFeatures; j++)
		{
			idxt = targetIndices[j];

			const card32 idist = i * targetFeatStep + j;

			if (_finite(cartProdDist[idist]))
			{
				if (cartProdDist[idist] < descDistLevelMin)
				{
					descDistLevelMin = cartProdDist[idist];
					iLevelBestPoint = idxt;
					iLevelBestFeature = j;
				}
			}
		}

		if (iLevelBestPoint < m_pTargetManifold->getNumDiscrPoints())
		{
			featureMatchesInit[k].m_iSourceFeature = i;
			featureMatchesInit[k].m_iTargetFeature = iLevelBestFeature;
			featureMatchesInit[k].m_iSourcePoint = idxs;
			featureMatchesInit[k].m_iTargetPoint = iLevelBestPoint;
			featureMatchesInit[k].m_score = entropyPriority * exp(-descDistLevelMin * rcp2MatchSigma2);
		}
		else
		{
			featureMatchesInit[k].m_score = 0.f;
		}

		k++;
	}

	featureMatchQueue = featureMatchesInit;
	AR4SMFeatureMatch::sortBestFirst(featureMatchQueue.begin(), featureMatchQueue.end());
}

void PCIGMOD14ShapeMatching::matchFeaturesPriorityStratified(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue)
{
	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
	const mpcard targetFeatStep = nTargetFeatures * m_nDescriptorLevels;

	mpcard i, j, k;
	mpcard idxs, idxt;

	const float rcp2MatchSigma2 = 1.f / (2.f * m_matchSigma*m_matchSigma);
	float entropyMax, rcpEntropyMax;
	float startPriority;

	vector<AR4SMFeatureMatch> featureMatchesInit(cartProdDist.size());

	vector<float> rcp2MatchSigma2Level(m_nDescriptorLevels);
	for (i = 0; i < m_nDescriptorLevels; i++)
	{
		float levelSigma = m_matchSigma * (i + 1);
		rcp2MatchSigma2Level[i] = 1.f / (2.f * levelSigma*levelSigma);
	}

	k = 0;
	for (i = 0; i < nSourceFeatures; i++)
	{
		
		float entropyPriority = sourceEntropies[i];
		if (i == 0)
		{
			entropyMax = entropyPriority;
			rcpEntropyMax = 1.f / entropyMax;
		}
		entropyPriority *= rcpEntropyMax;

		idxs = sourceIndices[i];

		mpcard level;
		for (level = 0; level < m_nDescriptorLevels; level++)
		{
			for (j = 0; j < nTargetFeatures; j++)
			{
				idxt = targetIndices[j];

				card32 idist = i * targetFeatStep + j * m_nDescriptorLevels + level;
				float matchPriority;

				if (!_finite(cartProdDist[idist]))
					matchPriority = 0.f;
				else
					matchPriority = exp(-cartProdDist[idist] * rcp2MatchSigma2Level[level]);

				startPriority = entropyPriority * matchPriority;

				featureMatchesInit[k].m_iSourceFeature = i;
				featureMatchesInit[k].m_iTargetFeature = j;
				featureMatchesInit[k].m_iSourcePoint = idxs;
				featureMatchesInit[k].m_iTargetPoint = idxt;
				featureMatchesInit[k].m_score = startPriority;

				k++;
			}
		}
	}

	featureMatchQueue = featureMatchesInit;
	AR4SMFeatureMatch::sortBestFirst(featureMatchQueue.begin(), featureMatchQueue.end());
}

void PCIGMOD14ShapeMatching::matchFeaturesPriorityMultiScale(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue)
{
	mpcard i, j, k;
	mpcard idxs, idxt;

	const float rcp2MatchSigma2 = 1.f / (2.f * m_matchSigma*m_matchSigma);
	float entropyMax, rcpEntropyMax;
	float startPriority;

	vector<AR4SMFeatureMatch> featureMatchesInit(cartProdDist.size());

	k = 0;
	for (i = 0; i < nSourceFeatures; i++)
	{
		float entropyPriority = sourceEntropies[i];
		if (i == 0)
		{
			entropyMax = entropyPriority;
			rcpEntropyMax = 1.f / entropyMax;
		}
		entropyPriority *= rcpEntropyMax;

		idxs = sourceIndices[i];

		for (j = 0; j < nTargetFeatures; j++)
		{
			idxt = targetIndices[j];

			float matchPriority = exp(-cartProdDist[k] * rcp2MatchSigma2);

			startPriority = entropyPriority * matchPriority;

			featureMatchesInit[k].m_iSourceFeature = i;
			featureMatchesInit[k].m_iTargetFeature = j;
			featureMatchesInit[k].m_iSourcePoint = idxs;
			featureMatchesInit[k].m_iTargetPoint = idxt;
			featureMatchesInit[k].m_score = startPriority;

			k++;
		}
	}

	featureMatchQueue = featureMatchesInit;
	AR4SMFeatureMatch::sortBestFirst(featureMatchQueue.begin(), featureMatchQueue.end());
}

void PCIGMOD14ShapeMatching::matchFeaturesClusterStratified(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue)
{
	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
	const mpcard targetFeatStep = nTargetFeatures * m_nDescriptorLevels;

	mpcard i, j, k;

	card32 idxs, idxt;

	const mpcard numFeatDescMatch = m_nDescriptorLevels;
	const mpcard numFeatNN = 20;
	AR4GDLCluster gdl;
	vector<vector<mpcard>> corrClusters;
	mpcard numInlierClusters;

	vector<vector<mpcard>> neighbors;
	vector<vector<float32>> distances;
	vector<vector<card32>> incompatibility;

	vector<AR4SMFeatureMatch> featureMatchesInit(cartProdDist.size());
	vector<vector<mpcard>> targetMatches(nTargetFeatures);

	vector<mpcard> sourceMatchIndexStart(nSourceFeatures);
	map<mpcard, mpcard> pointToFeatureSource, pointToFeatureTarget;

	map<pair<mpcard, mpcard>, float> targetGeodesicLookup;
	map<pair<mpcard, mpcard>, float>::iterator tglIt;

	for (i = 0; i < nSourceFeatures; i++)
	{
		idxs = sourceIndices[i];
		pointToFeatureSource[idxs] = i;
	}
	for (i = 0; i < nTargetFeatures; i++)
	{
		idxt = targetIndices[i];
		pointToFeatureTarget[idxt] = i;
		targetMatches[i].clear();
	}

	const float rcp2MatchSigma2 = 1.f / (2.f * m_matchSigma*m_matchSigma);

	vector<float> rcp2MatchSigma2Level(m_nDescriptorLevels);
	for (i = 0; i < m_nDescriptorLevels; i++)
	{
		float levelSigma = m_matchSigma * (i + 1);
		rcp2MatchSigma2Level[i] = 1.f / (2.f * levelSigma*levelSigma);
	}

	k = 0;
	for (i = 0; i < nSourceFeatures; i++)
	{
		idxs = sourceIndices[i];

		const mpcard kSourceStart = k;

		mpcard level;
		for (level = 0; level < m_nDescriptorLevels; level++)
		{
			float descDistLevelMin = 1e30f;
			mpcard iLevelBestMatch = m_pTargetManifold->getNumDiscrPoints();

			for (j = 0; j < nTargetFeatures; j++)
			{
				idxt = targetIndices[j];

				card32 idist = i * targetFeatStep + j * m_nDescriptorLevels + level;
				float matchPriority;

				if (_finite(cartProdDist[idist]))
				{
					featureMatchesInit[k].m_iSourceFeature = i;
					featureMatchesInit[k].m_iTargetFeature = j;
					featureMatchesInit[k].m_iSourcePoint = idxs;
					featureMatchesInit[k].m_iTargetPoint = idxt;
					featureMatchesInit[k].m_score = cartProdDist[idist] * rcp2MatchSigma2;

					if (cartProdDist[idist] < descDistLevelMin)
					{
						descDistLevelMin = cartProdDist[idist];
						iLevelBestMatch = idxt;
					}

					k++;
				}
			}//for (j...
		}//for (level...

		//remove duplicates
		//sort according to indices
		AR4SMFeatureMatch::PointsLess compLess;
		sort(featureMatchesInit.begin() + kSourceStart, featureMatchesInit.begin() + k, compLess);

		//make unique (don't need to erase, we just adjust k and overwrite with matches for next source feature)
		AR4SMFeatureMatch::PointsEqual compEqual;
		vector<AR4SMFeatureMatch>::iterator itUnique = unique(featureMatchesInit.begin() + kSourceStart, featureMatchesInit.begin() + k, compEqual);

		//adjust counter to account for elements removed
		k  = (mpcard)(itUnique - featureMatchesInit.begin());

		//sort by matching distance (< operator for AR4SMFeatureMatch)
		sort(featureMatchesInit.begin() + kSourceStart, featureMatchesInit.begin() + k);

		//only keep the best numFeatDescMatch
		k = min(k, kSourceStart + numFeatDescMatch);

		for (j = kSourceStart; j < k; j++)
		{
			targetMatches[featureMatchesInit[j].m_iTargetFeature].push_back(j);
		}

		//record starting position of matches for this source feature
		sourceMatchIndexStart[i] = kSourceStart;
	}

	const mpcard numFeatureMatchesToCluster = k;

	sourceMatchIndexStart.push_back(numFeatureMatchesToCluster);

	neighbors.resize(numFeatureMatchesToCluster);
	distances.resize(numFeatureMatchesToCluster);
	incompatibility.resize(numFeatureMatchesToCluster);

	X4_TIMER_START(feature_cluster_geo_nbrs);

	float avgTotalDists = 0.f;
	float avgMeasureDists = 0.f;
	float avgNbrs = 0.f;
	float avgIncomp = 0.f;
	for (i = 0; i < nSourceFeatures; i++)
	{
		vector<mpcard> endIndices, measureIndices;
		vector<float> sourceDists, measureDists;
		vector<Vector3f> sourceGrads;
		float targetDist;
		Vector3f targetGrad;

		const mpcard kstart = sourceMatchIndexStart[i];
		const mpcard kstop = sourceMatchIndexStart[i + 1];

		//for this match, collect source geodesic information
		m_pMatch->getSparseSourceGeodesics()->getGeodesicData(featureMatchesInit[kstart].m_iSourcePoint, endIndices, sourceDists, sourceGrads);

		//find other features to which there is a computed geodesic from this one
		for (j = 0; j < endIndices.size(); j++)
		{
			if (pointToFeatureSource.find(endIndices[j]) != pointToFeatureSource.end())
			{
				measureIndices.push_back(endIndices[j]);
				measureDists.push_back(sourceDists[j]);
			}
		}

		avgTotalDists += (float)endIndices.size();

		const mpcard numMeasurements = measureIndices.size();

		for (k = kstart; k < kstop; k++)
		{
			neighbors[k].clear();
			distances[k].clear();
			incompatibility[k].clear();

			//indicate other feature matches which share incompatible with this one
			for (mpcard m = kstart; m < k; m++)
				incompatibility[k].push_back((card32)m);
			for (mpcard m = k + 1; m < kstop; m++)
				incompatibility[k].push_back((card32)m);
			const mpcard itarg = featureMatchesInit[k].m_iTargetFeature;
			const mpcard numTargMatch = targetMatches[itarg].size();
			for (mpcard m = 0; m < numTargMatch; m++)
			{
				if (targetMatches[itarg][m] != k)
					incompatibility[k].push_back(targetMatches[itarg][m]);
			}

			avgIncomp += (float)incompatibility[k].size();
			avgMeasureDists += (float)numMeasurements;

			//compute geodesic between matched features on target
			for (j = 0; j < numMeasurements; j++)
			{
				//for each source feature connected by sampling topology, check that it is NOT matched against the same target feature
				const mpcard ifeat = pointToFeatureSource[measureIndices[j]];
				const mpcard featStart = sourceMatchIndexStart[ifeat];
				const mpcard featStop = sourceMatchIndexStart[ifeat + 1];

				if (ifeat == i)	continue;

				for (mpcard m = featStart; m < featStop; m++)
				{
					if (featureMatchesInit[m].m_iTargetPoint != featureMatchesInit[k].m_iTargetPoint)
					{
						pair<mpcard, mpcard> targGeoPair(featureMatchesInit[k].m_iTargetPoint, featureMatchesInit[m].m_iTargetPoint);
						pair<mpcard, mpcard> targGeoPairRev(featureMatchesInit[m].m_iTargetPoint, featureMatchesInit[k].m_iTargetPoint);

						tglIt = targetGeodesicLookup.find(targGeoPair);
						if (tglIt == targetGeodesicLookup.end())
						{
							//compute geodesic on target between these two features
							m_pMatch->getSparseTargetGeodesics()->computeGeodesicSingleDest(m_avgDenseEdgeLength * 0.1f, 
																							featureMatchesInit[k].m_iTargetPoint, featureMatchesInit[m].m_iTargetPoint, 
																							targetDist, targetGrad);

							targetGeodesicLookup[targGeoPair] = targetDist;
							targetGeodesicLookup[targGeoPairRev] = targetDist;
						}
						else
							targetDist = tglIt->second;

						const float corrDist = fabs(targetDist - measureDists[j]);

						neighbors[k].push_back(m);
						distances[k].push_back(corrDist);
					}
				}
			}

			avgNbrs += (float)neighbors[k].size();
		}
	}

	avgIncomp /= (float)numFeatureMatchesToCluster;
	avgNbrs /= (float)numFeatureMatchesToCluster;
	avgMeasureDists /= (float)numFeatureMatchesToCluster;
	avgTotalDists /= (float)nSourceFeatures;

	mpcard kCorrDists[5] = { 2, 5, 10, 15, 20 };
	float avgCorrDists[5] = { 0 };
	avgNbrs = 0.f;
	for (i = 0; i < numFeatureMatchesToCluster; i++)
	{
		//sort neighbors by distance and take the top K
		const mpcard numInitNbrs = neighbors[i].size();
		vector<pair<float, mpcard>> distNbrs(numInitNbrs);
		for (j = 0; j < numInitNbrs; j++)
		{
			distNbrs[j].first = distances[i][j];
			distNbrs[j].second = neighbors[i][j];
		}

		sort(distNbrs.begin(), distNbrs.end());

		const mpcard numNbrsMin = min(numInitNbrs, numFeatNN);
		for (j = 0; j < numNbrsMin; j++)
		{
			neighbors[i][j] = distNbrs[j].second;
			distances[i][j] = distNbrs[j].first;
			if (j < kCorrDists[0])	avgCorrDists[0] += distances[i][j];
			if (j < kCorrDists[1])	avgCorrDists[1] += distances[i][j];
			if (j < kCorrDists[2])	avgCorrDists[2] += distances[i][j];
			if (j < kCorrDists[3])	avgCorrDists[3] += distances[i][j];
			if (j < kCorrDists[4])	avgCorrDists[4] += distances[i][j];
		}

		neighbors[i].erase(neighbors[i].begin() + numNbrsMin, neighbors[i].end());
		distances[i].erase(distances[i].begin() + numNbrsMin, distances[i].end());

		avgNbrs += (float)numNbrsMin;
	}
	avgNbrs /= (float)numFeatureMatchesToCluster;
	avgCorrDists[0] /= (float)numFeatureMatchesToCluster;
	avgCorrDists[1] /= (float)numFeatureMatchesToCluster;
	avgCorrDists[2] /= (float)numFeatureMatchesToCluster;
	avgCorrDists[3] /= (float)numFeatureMatchesToCluster;
	avgCorrDists[4] /= (float)numFeatureMatchesToCluster;

	X4_TIMER_STOP(feature_cluster_geo_nbrs);

	debugOutput << "PCIGMOD14ShapeMatching::matchFeaturesClusterStratified():\n";
	debugOutput << "\tnumber of feature matches to cluster = " << numFeatureMatchesToCluster << "\n";
	debugOutput << "\taverage source geodesic connections = " << avgTotalDists << "\n";
	debugOutput << "\taverage number of distance measurements = " << avgMeasureDists << "\n";
	debugOutput << "\taverage number of neighbors = " << avgNbrs << "\n";
	debugOutput << "\taverage number of incompatible matches " << avgIncomp << "\n";
	debugOutput << "\taverage correspondence distance of first " << kCorrDists[0] << " neighbors = " << avgCorrDists[0] << "\n";
	debugOutput << "\taverage correspondence distance of first " << kCorrDists[1] << " neighbors = " << avgCorrDists[1] << "\n";
	debugOutput << "\taverage correspondence distance of first " << kCorrDists[2] << " neighbors = " << avgCorrDists[2] << "\n";
	debugOutput << "\taverage correspondence distance of first " << kCorrDists[3] << " neighbors = " << avgCorrDists[3] << "\n";
	debugOutput << "\taverage correspondence distance of first " << kCorrDists[4] << " neighbors = " << avgCorrDists[4] << "\n";

	const mpcard clipK = 1;

	avgNbrs = 0.f;
	for (i = 0; i < numFeatureMatchesToCluster; i++)
	{
		const mpcard numNbrs = neighbors[i].size();

		for (j = 0; j < numNbrs; j++)
		{
			if (distances[i][j] > avgCorrDists[clipK])
				break;
		}

		neighbors[i].erase(neighbors[i].begin() + j, neighbors[i].end());
		distances[i].erase(distances[i].begin() + j, distances[i].end());

		avgNbrs += (float)j;
	}
	avgNbrs /= (float)numFeatureMatchesToCluster;
	debugOutput << "\taverage number of neighbors within " << avgCorrDists[clipK] << " = " << avgNbrs << "\n";

	//now cluster correspondences using GDL
	debugOutput << "\tinitializing GDL\n";
	X4_TIMER_START(gdl_init);
	gdl.init(numFeatureMatchesToCluster, numFeatNN, neighbors, distances, incompatibility);
	X4_TIMER_STOP(gdl_init);
	debugOutput << "\trunning GDL\n";
	X4_TIMER_START(gdl_cluster);
	gdl.cluster(30, corrClusters, numInlierClusters);
	X4_TIMER_STOP(gdl_cluster);

	//print out some info
	debugOutput << "\tnumber of match clusters = " << corrClusters.size() << "\n";
	debugOutput << "\tnumber of inlier clusters = " << numInlierClusters << "\n";

	//now add matches from clusters (in order) to priority queue
	float lastClusterMax = 0.f;
	for (i = 0; i < corrClusters.size(); i++)
	{
		debugOutput << "\t\tcluster " << i << " size = " << corrClusters[i].size() << "\n";
		float currentClusterMax = 0.f;
		if (i < numInlierClusters)
		{
			for (j = 0; j < corrClusters[i].size(); j++)
			{
				k = corrClusters[i][j];
				featureMatchesInit[k].m_score += lastClusterMax;
				if (featureMatchesInit[k].m_score > currentClusterMax)
					currentClusterMax = featureMatchesInit[k].m_score;
				featureMatchesInit[k].m_score = exp(-featureMatchesInit[k].m_score);
			}
		}
		else
		{
			for (j = 0; j < corrClusters[i].size(); j++)
			{
				k = corrClusters[i][j];
				featureMatchesInit[k].m_score += lastClusterMax;
				if (featureMatchesInit[k].m_score > currentClusterMax)
					currentClusterMax = featureMatchesInit[k].m_score;
				featureMatchesInit[k].m_score = exp(-featureMatchesInit[k].m_score);
			}
		}

		lastClusterMax = currentClusterMax;
	}


	featureMatchQueue.resize(numFeatureMatchesToCluster);
	std::copy(featureMatchesInit.begin(), featureMatchesInit.begin() + numFeatureMatchesToCluster, featureMatchQueue.begin());
	AR4SMFeatureMatch::sortBestFirst(featureMatchQueue.begin(), featureMatchQueue.end());
}

void PCIGMOD14ShapeMatching::matchFeaturesClusterMultiScale(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue)
{
}

void PCIGMOD14ShapeMatching::matchFeaturesForwardStratified(const mpcard nSourceFeatures, const mpcard nTargetFeatures, const vector<mpcard>& sourceIndices, const vector<mpcard>& targetIndices, const vector<float>& sourceEntropies, const vector<float>& cartProdDist, vector<AR4SMFeatureMatch>& featureMatchQueue, vector<pair<mpcard, mpcard>>* pRefMatches/* = NULL*/)
{
	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
	const mpcard targetFeatStep = nTargetFeatures * m_nDescriptorLevels;

	mpcard i, j, k;
	mpcard idxs, idxt;

	const mpcard numDescMatchKeep = 10;

	//parameter for descriptor distances
	const float rcp2MatchSigma2 = 1.f / (2.f * m_matchSigma*m_matchSigma);

	//parameter for distance deviation
	const float distSigma = 2.f * m_avgDenseEdgeLength;
	const float rcp2DistSigma2 = 1.f / (2.f * distSigma*distSigma);

	float entropyMax, rcpEntropyMax;
	float startPriority;

	map<mpcard, mpcard> pointToFeatureSource, pointToFeatureTarget;

	map<pair<mpcard, mpcard>, float> targetGeodesicLookup;
	map<pair<mpcard, mpcard>, float>::iterator tglIt;

	vector<mpcard> sourceFeatureMatchStart(nSourceFeatures + 1);

	vector<AR4SMFeatureMatch> featureMatchesInit(cartProdDist.size());
	vector<AR4SMFeatureMatch> featureMatchesIntermed;

	//in case geodesic path relaxation is being used
	m_pMatch->getSparseTargetGeodesics()->setRelaxationSoftConstraint(0.01f);

	mpcard numRefTemp = 0;
	if (pRefMatches != NULL)
		numRefTemp = pRefMatches->size();
	const mpcard numRef = numRefTemp;

	vector<float> cartProdRefDist;
	if (numRef > 0)
		cartProdRefDist.resize(nSourceFeatures * nTargetFeatures, 0.f);

	for (i = 0; i < nSourceFeatures; i++)
	{
		idxs = sourceIndices[i];
		pointToFeatureSource[idxs] = i;
	}
	for (i = 0; i < nTargetFeatures; i++)
	{
		idxt = targetIndices[i];
		pointToFeatureTarget[idxt] = i;
	}

	vector<float> rcp2MatchSigma2Level(m_nDescriptorLevels);
	for (i = 0; i < m_nDescriptorLevels; i++)
	{
		float levelSigma = m_matchSigma * (i + 1);
		rcp2MatchSigma2Level[i] = 1.f / (2.f * levelSigma*levelSigma);
	}

	if (numRef > 0)
	{
		k = 0;
		for (i = 0; i < nSourceFeatures; i++)
		{
			idxs = sourceIndices[i];

			for (j = 0; j < nTargetFeatures; j++)
			{
				idxt = targetIndices[j];

				for (mpcard r = 0; r < numRef; r++)
				{
					const mpcard rs = (*pRefMatches)[r].first;
					const mpcard rt = (*pRefMatches)[r].second;

					float sourceDist, targetDist;
					Vector3f sourceGrad, targetGrad, dirAtSource;

					if (m_pMatch->getSparseSourceGeodesics()->getGeodesicData(rs, idxs, sourceDist, sourceGrad,dirAtSource) == AR4SparseGeodesics::m_InvalidIndex)
					{
						cartProdRefDist[k] += distSigma*distSigma;
					}
					else
					{
						m_pMatch->getSparseTargetGeodesics()->computeGeodesicSingleDest(m_avgDenseEdgeLength * 0.1f, rt, idxt, targetDist, targetGrad);

						const float distDiff = targetDist - sourceDist;
						const float distCost = distDiff*distDiff;
						cartProdRefDist[k] += distCost;
					}
				}

				cartProdRefDist[k] = exp(-cartProdRefDist[k] * rcp2DistSigma2);

				k++;
			}
		}
	}

	k = 0;
	for (i = 0; i < nSourceFeatures; i++)
	{
		const mpcard kSourceStart = k;
		sourceFeatureMatchStart[i] = kSourceStart;

		float entropyPriority = sourceEntropies[i];
		if (i == 0)
		{
			entropyMax = entropyPriority;
			rcpEntropyMax = 1.f / entropyMax;
		}
		entropyPriority *= rcpEntropyMax;

		idxs = sourceIndices[i];

		mpcard level;
		for (level = 0; level < m_nDescriptorLevels; level++)
		{
			for (j = 0; j < nTargetFeatures; j++)
			{
				idxt = targetIndices[j];

				card32 idist = i * targetFeatStep + j * m_nDescriptorLevels + level;
				float matchPriority;

				if (!_finite(cartProdDist[idist]))
					matchPriority = 0.f;
				else
					matchPriority = exp(-cartProdDist[idist] * rcp2MatchSigma2);

				startPriority = entropyPriority * matchPriority;

				if (numRef > 0)
					startPriority *= cartProdRefDist[i * nTargetFeatures + j];

				featureMatchesInit[k].m_iSourceFeature = i;
				featureMatchesInit[k].m_iTargetFeature = j;
				featureMatchesInit[k].m_iSourcePoint = idxs;
				featureMatchesInit[k].m_iTargetPoint = idxt;
				featureMatchesInit[k].m_score = startPriority;

				k++;
			}
		}

		AR4SMFeatureMatch::sortBestFirst(featureMatchesInit.begin() + kSourceStart, featureMatchesInit.begin() + k);

		//remove duplicates
		vector<bool> targetFeatureTaken(nTargetFeatures, false);
		mpcard numRemoved = 0;
		for (j = kSourceStart; j < k; j++)
		{
			if (targetFeatureTaken[featureMatchesInit[j].m_iTargetFeature])
			{
				numRemoved++;
			}
			else
			{
				targetFeatureTaken[featureMatchesInit[j].m_iTargetFeature] = true;
				if (numRemoved > 0)
					featureMatchesInit[j - numRemoved] = featureMatchesInit[j];
			}
		}

		k -= numRemoved;

		k = min(k, kSourceStart + numDescMatchKeep);
	}

	const mpcard numInitMatches = k;
	sourceFeatureMatchStart[i] = numInitMatches;
	
	const float scoreGroupTermThresh = 1e-5f;

	vector<bool> matchGrouped(numInitMatches, false);
	vector<vector<mpcard>> matchGroups;

	mpcard numMatchesGrouped = 0;
	float lastGroupLastPriority = 1.f;

	while (numMatchesGrouped < numInitMatches)
	{
		//find max priority among matches not yet grouped
		float bestScore = 0.f;
		mpcard iBestScore = 0;
		for (i = 0; i < numInitMatches; i++)
		{
			if (matchGrouped[i])	continue;
			if (AR4SMFeatureMatch::isScoreBetterThan(featureMatchesInit[i].m_score, bestScore))
			{
				bestScore = featureMatchesInit[i].m_score;
				iBestScore = i;
			}
		}

		//keep track of where in "intermed" list of matches this group started
		mpcard iGroupIntermedStart = featureMatchesIntermed.size();

		//add the "group leader" to the intermediate list
		featureMatchesIntermed.push_back(featureMatchesInit[iBestScore]);

		//vector of matches belonging to current group, and set of indices on the source surface (of matches in this group)
		vector<mpcard> currentMatchGroup;
		set<mpcard> groupSources;

		//start the new group with the max-priority match
		mpcard currentGroupLastSize = 0;
		currentMatchGroup.push_back(iBestScore);
		matchGrouped[iBestScore] = true;
		groupSources.insert(featureMatchesInit[iBestScore].m_iSourcePoint);

		while (currentMatchGroup.size() > currentGroupLastSize)
		{
			currentGroupLastSize = currentMatchGroup.size();
			const mpcard currentGroupSize = currentMatchGroup.size();

			map<mpcard, mpcard> allSourcesConsidered;
			map<mpcard, mpcard>::iterator ascIt;
			pair<map<mpcard, mpcard>::iterator, bool> insRet;

			map<mpcard, vector<float>> allSourceDistMeasures;

			for (i = 0; i < currentGroupSize; i++)
			{
				vector<mpcard> sourceDests;
				vector<float> sourceDists;
				vector<Vector3f> sourceGrads;

				vector<mpcard> sourceTests;
				vector<float> sourceTestDists;

				k = currentMatchGroup[i];

				m_pMatch->getSparseSourceGeodesics()->getGeodesicData(featureMatchesInit[k].m_iSourcePoint, sourceDests, sourceDists, sourceGrads);

				const mpcard numSourceDests = sourceDests.size();
				for (j = 0; j < numSourceDests; j++)
				{
					if (pointToFeatureSource.find(sourceDests[j]) != pointToFeatureSource.end() && groupSources.find(sourceDests[j]) == groupSources.end())
					{
						sourceTests.push_back(sourceDests[j]);
						sourceTestDists.push_back(sourceDists[j]);
						insRet = allSourcesConsidered.insert(pair<mpcard, mpcard>(sourceDests[j], 0));
						if (insRet.second)
						{
							insRet.first->second = 1;
							allSourceDistMeasures.insert(pair<mpcard, vector<float>>(sourceDests[j], vector<float>(numDescMatchKeep, 0)));
						}
						else
							insRet.first->second++;
					}
				}

				mpcard m;
				float targetDist;
				Vector3f targetGrad, Dir;

				const mpcard numSourceTests = sourceTests.size();
				for (j = 0; j < numSourceTests; j++)
				{
					const mpcard iSrcFeat = pointToFeatureSource[sourceTests[j]];
					const mpcard mstart = sourceFeatureMatchStart[iSrcFeat];
					const mpcard mstop = sourceFeatureMatchStart[iSrcFeat + 1];

					const float refDist = sourceTestDists[j];

					for (m = mstart; m < mstop; m++)
					{

						pair<mpcard, mpcard> targGeo(featureMatchesInit[k].m_iTargetPoint, featureMatchesInit[m].m_iTargetPoint);
						pair<mpcard, mpcard> targGeoRev(featureMatchesInit[m].m_iTargetPoint, featureMatchesInit[k].m_iTargetPoint);

						tglIt = targetGeodesicLookup.find(targGeo);
						if (tglIt == targetGeodesicLookup.end())
						{
							if (settings->bUsePathRelaxation)
							{
								vector<Vector3f> targetPath;
								m_pMatch->getSparseTargetGeodesics()->computeGeodesicSingleDest(m_avgDenseEdgeLength * 0.1f, 
																								featureMatchesInit[k].m_iTargetPoint, featureMatchesInit[m].m_iTargetPoint, 
																								targetDist, targetGrad, targetPath);
								m_pMatch->getSparseTargetGeodesics()->relaxPath(2, targetPath, targetDist, targetGrad, Dir);
							}
							else
							{
								m_pMatch->getSparseTargetGeodesics()->computeGeodesicSingleDest(m_avgDenseEdgeLength * 0.1f, 
																								featureMatchesInit[k].m_iTargetPoint, featureMatchesInit[m].m_iTargetPoint, 
																								targetDist, targetGrad);
							}

							targetGeodesicLookup.insert(pair<pair<mpcard, mpcard>, float>(targGeo, targetDist));
							targetGeodesicLookup.insert(pair<pair<mpcard, mpcard>, float>(targGeoRev, targetDist));
						}
						else
						{
							targetDist = tglIt->second;
						}

						const float distDiff = targetDist - refDist;
						const float distCost = distDiff*distDiff;

						allSourceDistMeasures[sourceTests[j]][m - mstart] += distCost;
					}
				}
			}

			const mpcard numMinMeasureConsider = min((int) currentGroupSize, 3);

			mpcard iChosenSource = 0;
			bestScore = 0.f;
			iBestScore = 0;
			for (ascIt = allSourcesConsidered.begin(); ascIt != allSourcesConsidered.end(); ascIt++)
			{
				const mpcard iSrcFeat = pointToFeatureSource[ascIt->first];
				const mpcard kstart = sourceFeatureMatchStart[iSrcFeat];
				const mpcard kstop = sourceFeatureMatchStart[iSrcFeat + 1];

				const float rcpDistMeasCount = 1.f / (float)ascIt->second;

				if (ascIt->second < numMinMeasureConsider)	continue;

				for (k = kstart; k < kstop; k++)
				{

					const float distCostAvg = allSourceDistMeasures[ascIt->first][k - kstart];// * rcpDistMeasCount;
					const float distPriority = exp(-distCostAvg * rcp2DistSigma2);

					const float combinedPriority = AR4SMFeatureMatch::adjustScore(featureMatchesInit[k].m_score, distPriority);

					if (AR4SMFeatureMatch::isScoreBetterThan(combinedPriority, bestScore))
					{
						bestScore = combinedPriority;
						iBestScore = k;
						iChosenSource = ascIt->first;
					}
				}
			}

			if (AR4SMFeatureMatch::isScoreBetterThan(bestScore, scoreGroupTermThresh))
			{
				currentMatchGroup.push_back(iBestScore);
				featureMatchesIntermed.push_back(featureMatchesInit[iBestScore]);
				featureMatchesIntermed.back().m_score = bestScore;
				groupSources.insert(iChosenSource);
			}
		}

		const float firstPriority = featureMatchesInit[currentMatchGroup.front()].m_score;
		const float invFirstPriority = AR4SMFeatureMatch::invertScore(firstPriority);
		const float adjustment = invFirstPriority;

		lastGroupLastPriority = 1.f;
		for (i = 0; i < currentMatchGroup.size(); i++)
		{
			featureMatchesIntermed[iGroupIntermedStart + i].adjustScore(adjustment);
			if (AR4SMFeatureMatch::isScoreBetterThan(lastGroupLastPriority, featureMatchesIntermed[iGroupIntermedStart + i].m_score))
				lastGroupLastPriority = featureMatchesIntermed[iGroupIntermedStart + i].m_score;
		}

		matchGroups.push_back(currentMatchGroup);
		numMatchesGrouped += currentMatchGroup.size();
	}

	//sort groups by size (decreasing)
	const mpcard numGroups = matchGroups.size();
	vector<pair<mpcard, mpcard>> groupSizes(numGroups);
	for (i = 0; i < numGroups; i++)
	{
		groupSizes[i].first = matchGroups[i].size();
		groupSizes[i].second = i;
	}

	sort(groupSizes.begin(), groupSizes.end(), greater<pair<mpcard, mpcard>>());

	const float badScore = AR4SMFeatureMatch::reallyBadScore();

	lastGroupLastPriority = 1.f;
	k = 0;
	for (i = 0; i < numGroups; i++)
	{
		const mpcard ig = groupSizes[i].second;
		const mpcard groupSize = matchGroups[ig].size();

		const float invGroupFirst = AR4SMFeatureMatch::invertScore(featureMatchesInit[matchGroups[ig][0]].m_score);
		const float adjustment = AR4SMFeatureMatch::adjustScore(lastGroupLastPriority, invGroupFirst);
		for (j = 0; j < groupSize; j++)
		{
			featureMatchesIntermed[k].adjustScore(adjustment);
			if (AR4SMFeatureMatch::isScoreBetterThan(lastGroupLastPriority, featureMatchesIntermed[k].m_score))
				lastGroupLastPriority = featureMatchesIntermed[k].m_score;
			k++;
		}

		if (AR4SMFeatureMatch::isScoreBetterThan(badScore, lastGroupLastPriority))
			lastGroupLastPriority = badScore;
	}

	for (i = 0; i < numInitMatches; i++)
	{
		if (!matchGrouped[i])
			featureMatchesInit[i].m_score = 0.f;
	}

	const mpcard numIntermedMatches = featureMatchesIntermed.size();
	featureMatchQueue.resize(numIntermedMatches);
	std::copy(featureMatchesIntermed.begin(), featureMatchesIntermed.begin() + numIntermedMatches, featureMatchQueue.begin());
	AR4SMFeatureMatch::sortBestFirst(featureMatchQueue.begin(), featureMatchQueue.end());
}

void PCIGMOD14ShapeMatching::featureDescCartProdStratified(const vector<float>& descriptors1, const vector<float>& descriptors2, vector<float>& cartProdDist)
{
	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
	const mpcard nSourceFeatures = descriptors1.size() / descriptorStep;
	const mpcard nTargetFeatures = descriptors2.size() / descriptorStep;
	const mpcard targetFeatStep = nTargetFeatures * m_nDescriptorLevels;

	mpcard i, j, level, soff, toff;

	cartProdDist.resize(nSourceFeatures * targetFeatStep);

	soff = 0;
	for (i = 0; i < nSourceFeatures; i++)
	{
		toff = 0;
		for (j = 0; j < nTargetFeatures; j++)
		{
			mpcard level;
			for (level = 0; level < m_nDescriptorLevels; level++)
			{
				float featdist;
				if (descriptors1[soff + (level + 1) * m_nDescriptorDim - 1] < 0.f || descriptors2[toff + (level + 1) * m_nDescriptorDim - 1] < 0.f)
					featdist = numeric_limits< float32 >::quiet_NaN();
				else
					featdist = descriptorDistance(m_nDescriptorDim, &(descriptors1[soff + level * m_nDescriptorDim]), &(descriptors2[toff + level * m_nDescriptorDim]));
				cartProdDist[i * targetFeatStep + j * m_nDescriptorLevels + level] = featdist;
			}

			//increment target counter
			toff += descriptorStep;
		}

		//increment source counter
		soff += descriptorStep;
	}
}

void PCIGMOD14ShapeMatching::featureDescCartProdMultiScale(const vector<float>& descriptors1, const vector<float>& descriptors2, vector<float>& cartProdDist)
{
	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;
	const mpcard nSourceFeatures = descriptors1.size() / descriptorStep;
	const mpcard nTargetFeatures = descriptors2.size() / descriptorStep;
	const mpcard targetFeatStep = nTargetFeatures;

	mpcard i, j, level, soff, toff;

	cartProdDist.resize(nSourceFeatures * targetFeatStep);

	soff = 0;
	for (i = 0; i < nSourceFeatures; i++)
	{
		toff = 0;
		for (j = 0; j < nTargetFeatures; j++)
		{
			//compute distance between these descriptors at multiple scales
			cartProdDist[i * targetFeatStep + j] = descriptorDistanceMultiScale(m_nDescriptorLevels, m_nDescriptorDim, 
																						&(descriptors1[soff]), &(descriptors2[toff]));

			//increment target counter
			toff += descriptorStep;
		}

		//increment source counter
		soff += descriptorStep;
	}
}

bool PCIGMOD14ShapeMatching::getBestFeatureMatch(vector<AR4SMFeatureMatch>& featureQueue, mpcard& nextFeatureMatchIndex, TExistFMSet& existingMatches, AR4SMFeatureMatch& fm)
{
	bool btried;

	if (nextFeatureMatchIndex == featureQueue.size())
		return false;

	fm = featureQueue[nextFeatureMatchIndex];
	btried = wasFeatureMatchAlreadyTried(fm, existingMatches);
	if (!btried && fm.m_dirOrder == 0)
	{
		getMatchDirections(featureQueue, nextFeatureMatchIndex);
		fm = featureQueue[nextFeatureMatchIndex];
	}
	nextFeatureMatchIndex++;

	//check if it was already matched; if yes discard and take the next
	while (btried && nextFeatureMatchIndex < featureQueue.size())
	{
		fm = featureQueue[nextFeatureMatchIndex];
		btried = wasFeatureMatchAlreadyTried(fm, existingMatches);
		if (!btried && fm.m_dirOrder == 0)
		{
			getMatchDirections(featureQueue, nextFeatureMatchIndex);
			fm = featureQueue[nextFeatureMatchIndex];
		}
		nextFeatureMatchIndex++;
	}

	existingMatches.insert(fm);
	return !btried;
}

bool PCIGMOD14ShapeMatching::getBestFeatureMatch(AR4SMFeatureMatch& fm)
{
	return getBestFeatureMatch(m_featureMatchQueue, m_nextFeatureMatchIndex, m_processedFeatures, fm);
}

typedef pair<float, mpcard> TAngleIdx;
typedef pair<float, float> TCostAngle;

void PCIGMOD14ShapeMatching::getMatchDirections(const AR4SMFeatureMatch& fm, Vector3f& sourceDir, vector<Vector3f>& targetDirs)
{
	const double TWO_PI	= 2.0 * M_PI;

	mpcard i, j;
	Vector3f sourcePoint, targetPoint, projPoint, sourceNorm, targetNorm;
	AR4MLSManifold3f::TangentSpace sourceTS, targetTS;

	sourcePoint = m_pSourceManifold->getDiscrPoint(fm.m_iSourcePoint);
	m_pSourceManifold->project(sourcePoint, projPoint, sourceNorm, sourceTS);

	targetPoint = m_pTargetManifold->getDiscrPoint(fm.m_iTargetPoint);
	m_pTargetManifold->project(targetPoint, projPoint, targetNorm, targetTS);

	const mpcard nSourceSeconds = m_sourceSecondaryFeatureIndex[fm.m_iSourceFeature].size();
	const mpcard nTargetSeconds = m_targetSecondaryFeatureIndex[fm.m_iTargetFeature].size();

	targetDirs.clear();

	if (nSourceSeconds == 0 || nTargetSeconds == 0)
		return;

	//arbitrary source direction
	sourceDir = sourceTS[0];

	//////////
	//now collect directions of secondary features on source relative to sourceDir (i.e. in tangent space)
	vector<Vector3f> sourceTangDirs(nSourceSeconds);
	vector<TAngleIdx> sourceAngle(nSourceSeconds);
	vector<mpcard> sourceDests(nSourceSeconds);
	vector<float> sourceDists;
	vector<Vector3f> sourceGrads, dirsAtSource;
	vector<vector<Vector3f>> sourcePaths;

	//compute geodesic all at once
	for (i = 0; i < nSourceSeconds; i++)
		sourceDests[i] = m_sourceSecondaryFeatureIndex[fm.m_iSourceFeature][i];
	m_pMatch->getSparseSourceGeodesics()->computeGeodesicsEarlyStop(m_pMatch->getGeodesicsSigma(), fm.m_iSourcePoint, sourceDests, sourceDists, sourceGrads, &sourcePaths);

	//relax paths for accuracy
	m_pMatch->getSparseSourceGeodesics()->setRelaxationSoftConstraint(0.01f);
	m_pMatch->getSparseSourceGeodesics()->relaxPaths(4, sourcePaths, sourceDists, sourceGrads, dirsAtSource);

	//now collect dirs
	for (i = 0; i < nSourceSeconds; i++)
	{
		//take direction of first edge of this path
		Vector3f dir = sourcePaths[i][1] - sourcePaths[i][0];
		dir.normalize();

		//project into tangent space
		Vector2f dir2d = makeVector2f(dir * sourceTS[0], dir * sourceTS[1]);
		dir = sourceTS * dir2d;

		//save
		sourceTangDirs[i] = dir;
		float32 theta = atan2(dir2d[1], dir2d[0]);
		if (theta < 0.f)
			theta += TWO_PI;
		sourceAngle[i] = TAngleIdx(theta, i);
	}

	//////////
	//collect directions of secondary features on target in tangent space of target feature
	vector<Vector3f> targetTangDirs(nTargetSeconds);
	vector<TAngleIdx> targetAngle(nTargetSeconds);
	vector<mpcard> targetDests(nTargetSeconds);
	vector<float> targetDists;
	vector<Vector3f> targetGrads;
	vector<vector<Vector3f>> targetPaths;

	//compute geodesics all at once
	for (i = 0; i < nTargetSeconds; i++)
		targetDests[i] = m_targetSecondaryFeatureIndex[fm.m_iTargetFeature][i];
	m_pMatch->getSparseTargetGeodesics()->computeGeodesicsEarlyStop(m_pMatch->getGeodesicsSigma(), fm.m_iTargetPoint, targetDests, targetDists, targetGrads, &targetPaths);

	//relax paths for accuracy
	m_pMatch->getSparseTargetGeodesics()->setRelaxationSoftConstraint(0.01f);
	m_pMatch->getSparseTargetGeodesics()->relaxPaths(4, targetPaths, targetDists, targetGrads, dirsAtSource);

	//now collect dirs
	for (i = 0; i < nTargetSeconds; i++)
	{
		//take direction of first edge of this path
		Vector3f dir = targetPaths[i][1] - targetPaths[i][0];
		dir.normalize();

		//project into tangent space
		Vector2f dir2d = makeVector2f(dir * targetTS[0], dir * targetTS[1]);
		dir = targetTS * dir2d;

		//save
		targetTangDirs[i] = dir;
		float32 theta = atan2(dir2d[1], dir2d[0]);
		if (theta < 0.f)
			theta += TWO_PI;
		targetAngle[i] = TAngleIdx(theta, i);
	}

	sort(sourceAngle.begin(), sourceAngle.end(), less<TAngleIdx>());
	sort(targetAngle.begin(), targetAngle.end(), less<TAngleIdx>());

	//get direct references to source and target secondary feature descriptors
	vector<float>& sourceDescriptors(m_sourceSecondaryFeatureDesc[fm.m_iSourceFeature]);
	vector<float>& targetDescriptors(m_targetSecondaryFeatureDesc[fm.m_iTargetFeature]);

	const float rcp2MatchSigma2 = (1.f / (2.f * m_matchSigma*m_matchSigma)) * 0.01f;
	const float angleWeight = 0.f;//(float)(1.0 / M_PI);
	const float distWeight = 1.f / (m_avgDenseEdgeLength * 5.f);
	const float angleSepWeight = (float)(1.0 / M_PI);

	const mpcard descriptorStep = m_nDescriptorDim * m_nDescriptorLevels;

	vector<TCostAngle> rankedAngles;

	//now that we have the tangent space directions, we try to find one match that has the most support (or multiple matches with at least some support)
	for (i = 0; i < nSourceSeconds; i++)
	{
		mpcard idxs = sourceAngle[i].second;
		float sa = sourceAngle[i].first;
		Vector3f sdir = sourceTangDirs[idxs];

		for (j = 0; j < nTargetSeconds; j++)
		{
			mpcard idxt = targetAngle[j].second;
			float ta = targetAngle[j].first;
			Vector3f tdir = targetTangDirs[idxt];

			float deltaAngle = ta - sa;

			float dirCost = 0.f;

			float partCostAngle = 0.f;
			float partCostDist = 0.f;
			float partCostDesc = 0.f;

			mpcard k;
			for (k = 1; k < nSourceSeconds; k++)//start at 1 because we have already fixed i, so there are nSourceSeconds - 1 left to test
			{
				float angleDiff;
				mpcard m = (i + k) % nSourceSeconds;
				mpcard idxs2 = sourceAngle[m].second;
				float sa2 = sourceAngle[m].first;
				sa2 += deltaAngle;
				if (sa2 < 0.f)
					sa2 += TWO_PI;
				else if (sa2 >= TWO_PI)
					sa2 -= TWO_PI;

				//we want to find the direction on the target surface with the closest angle to the offset source angle
				TAngleIdx query(sa2, 0);
				TAngleIdx match;
				pair<vector<TAngleIdx>::iterator, vector<TAngleIdx>::iterator> range = equal_range(targetAngle.begin(), targetAngle.end(), query);
				if (range.first == targetAngle.end() && range.second == targetAngle.end())
				{
					//this happens if sa2 greater than all angles in targetAngle
					//since the angles are cyclical, we test the first and the last
					float ang1 = sa2 - (targetAngle.end() - 1)->first;
					float ang2 = targetAngle.begin()->first + (TWO_PI - sa2);
					if (ang2 < ang1)
					{
						match = *targetAngle.begin();
						angleDiff = ang2;
					}
					else
					{
						match = *(range.second - 1);
						angleDiff = ang1;
					}
				}
				else if (range.first == range.second)
				{
					//first element "greater" than sa2 is the same as the first element "not less" than sa2
					//so there was no element equal
					//if it is not the first element, then also check the element before it, and take the one that is closer
					//if it is the first element, then, since angles are cyclical, we also test the last
					float ang1, ang2;
					if (range.first != targetAngle.begin())
					{
						ang1 = sa2 - (range.first - 1)->first;
						ang2 = range.first->first - sa2;
						if (ang1 < ang2)
						{
							match = *(range.first - 1);
							angleDiff = ang1;
						}
						else
						{
							match = *range.first;
							angleDiff = ang2;
						}
					}
					else
					{
						ang1 = sa2 + (TWO_PI - (targetAngle.end() - 1)->first);
						ang2 = range.first->first - sa2;
						if (ang1 < ang2)
						{
							match = *(targetAngle.end() - 1);
							angleDiff = ang1;
						}
						else
						{
							match = *range.first;
							angleDiff = ang2;
						}
					}
				}
				else
				{
					match = *range.first;
					angleDiff = 0.f;
				}

				//angle penalty is the difference in angles 
				float anglePenalty = angleDiff;

				//from the index compare distances and descriptors
				//distance penalty 
				float distDiff = targetDists[match.second] - sourceDists[idxs2];
				float distPenalty = fabs(distDiff);//distDiff*distDiff;

				//descriptor penalty
				float descDist = descriptorDistanceMultiScale(m_nDescriptorLevels, m_nDescriptorDim, &sourceDescriptors[idxs2 * descriptorStep], &targetDescriptors[match.second * descriptorStep]);

				
				dirCost += anglePenalty * angleWeight + distPenalty * distWeight + descDist * rcp2MatchSigma2;

				partCostAngle += anglePenalty * angleWeight;
				partCostDist += distPenalty * distWeight;
				partCostDesc += descDist * rcp2MatchSigma2;
			}

			//add this direction with its associated cost
			rankedAngles.push_back(TCostAngle(dirCost, deltaAngle));
		}
	}

	sort(rankedAngles.begin(), rankedAngles.end(), less<TCostAngle>());

	//encourage angles to be separated, so that we don't try multiple virtually identical angles
	for (i = 1; i < rankedAngles.size(); i++)
	{
		const float currAngle = rankedAngles[i].second;

		float angleDiff, angleDiffSum = 0.f;
		for (j = 0; j < i; j++)
		{
			if (currAngle > rankedAngles[j].second)
				angleDiff = min((float) currAngle - rankedAngles[j].second, (float) TWO_PI - currAngle + rankedAngles[j].second);
			else
				angleDiff = min((float) rankedAngles[j].second - currAngle, (float) TWO_PI - rankedAngles[j].second + currAngle);
			angleDiffSum += 1.f / (1.f + angleDiff * angleSepWeight);
		}

		rankedAngles[i].first += angleDiffSum / (float)i;
	}

	sort(rankedAngles.begin(), rankedAngles.end(), less<TCostAngle>());

	for (i = 0; i < rankedAngles.size(); i++)
	{
		Vector2f dir2d = makeVector2f(cos(rankedAngles[i].second), sin(rankedAngles[i].second));
		Vector3f dir = targetTS * dir2d;

		targetDirs.push_back(dir);
	}
}

void PCIGMOD14ShapeMatching::getMatchDirections(vector<AR4SMFeatureMatch>& featureMatchQueue, const mpcard posInQueue)
{
	const double TWO_PI	= 2.0 * M_PI;

	AR4SMFeatureMatch fm = featureMatchQueue[posInQueue];

	mpcard i, j;
	Vector3f sourcePoint, targetPoint, projPoint, sourceNorm, targetNorm;
	AR4MLSManifold3f::TangentSpace sourceTS, targetTS;

	sourcePoint = m_pSourceManifold->getDiscrPoint(fm.m_iSourcePoint);
	m_pSourceManifold->project(sourcePoint, projPoint, sourceNorm, sourceTS);

	targetPoint = m_pTargetManifold->getDiscrPoint(fm.m_iTargetPoint);
	m_pTargetManifold->project(targetPoint, projPoint, targetNorm, targetTS);

	const mpcard nSourceSeconds = m_sourceSecondaryFeatureIndex[fm.m_iSourceFeature].size();
	const mpcard nTargetSeconds = m_targetSecondaryFeatureIndex[fm.m_iTargetFeature].size();

	if (nSourceSeconds == 0 || nTargetSeconds == 0)
		return;

	if (settings->featureMatchSource != AR4Settings::fmSrcInternal)
	{
		vector<mpcard> sourceDests(nSourceSeconds);
		vector<float> sourceDists;
		vector<Vector3f> sourceGrads, dirsAtSource;
		vector<vector<Vector3f>> sourcePaths;
		vector<mpcard> targetDests(nTargetSeconds);
		vector<float> targetDists;
		vector<Vector3f> targetGrads;
		vector<vector<Vector3f>> targetPaths;

		//compute geodesic all at once
		for (i = 0; i < nSourceSeconds; i++)
			sourceDests[i] = m_sourceSecondaryFeatureIndex[fm.m_iSourceFeature][i];
		m_pMatch->getSparseSourceGeodesics()->computeGeodesicsEarlyStop(m_avgDenseEdgeLength * 0.1f, fm.m_iSourcePoint, sourceDests, sourceDists, sourceGrads, &sourcePaths);

		//relax paths for accuracy
		m_pMatch->getSparseSourceGeodesics()->setRelaxationSoftConstraint(0.01f);
		m_pMatch->getSparseSourceGeodesics()->relaxPaths(4, sourcePaths, sourceDists, sourceGrads, dirsAtSource);

		//compute geodesics all at once
		for (i = 0; i < nTargetSeconds; i++)
			targetDests[i] = m_targetSecondaryFeatureIndex[fm.m_iTargetFeature][i];
		m_pMatch->getSparseTargetGeodesics()->computeGeodesicsEarlyStop(m_avgDenseEdgeLength * 0.1f, fm.m_iTargetPoint, targetDests, targetDists, targetGrads, &targetPaths);

		//relax paths for accuracy
		m_pMatch->getSparseTargetGeodesics()->setRelaxationSoftConstraint(0.01f);
		m_pMatch->getSparseTargetGeodesics()->relaxPaths(4, targetPaths, targetDists, targetGrads, dirsAtSource);

		const mpcard numLowerSecond = min(nSourceSeconds, nTargetSeconds);
		for (i = 0; i < nSourceSeconds; i++)
		{
			Vector3f sdir = sourcePaths[i][1] - sourcePaths[i][0];
			sdir.normalize();

			Vector3f tdir = targetPaths[i][1] - targetPaths[i][0];
			tdir.normalize();

			Vector2f sdir2 = makeVector2f(sdir * sourceTS[0], sdir * sourceTS[1]);
			Vector2f tdir2 = makeVector2f(tdir * targetTS[0], tdir * targetTS[1]);

			float32 alpha = atan2(sdir2[1], sdir2[0]);
			if (alpha < 0.f)
				alpha += TWO_PI;
			float32 beta = atan2(tdir2[1], tdir2[0]);
			if (beta < 0.f)
				beta += TWO_PI;

			//delta is the angle to align the tangent space of the target with the tangent space of the source
			float32 delta = beta - alpha;
			if (delta < 0.f)
				delta += TWO_PI;

			if (i == 0)
			{
				featureMatchQueue[posInQueue].m_targetDir = delta;
			}
			else
			{
				AR4SMFeatureMatch fmNew;
				fmNew.m_iSourceFeature = fm.m_iSourceFeature;
				fmNew.m_iSourcePoint = fm.m_iSourcePoint;
				fmNew.m_iTargetFeature = fm.m_iTargetFeature;
				fmNew.m_iTargetPoint = fm.m_iTargetPoint;
				fmNew.m_targetDir = delta;
				fmNew.m_dirOrder = i;
				fmNew.m_score = AR4SMFeatureMatch::bestScore();
				featureMatchQueue.push_back(fmNew);
			}
		}

		AR4SMFeatureMatch::sortBestFirst(featureMatchQueue.begin() + posInQueue + 1, featureMatchQueue.end());
		return;
	}

	vector<float> secondaryDescCartProd(nSourceSeconds * nTargetSeconds * m_nDescriptorLevels);
	vector<AR4SMFeatureMatch> secondaryMatchQueue;
	
	//compute Cartesian product of feature descriptor distances
#if AR4_MATCH_FEATURES_STRATAFIED
	featureDescCartProdStratified(m_sourceSecondaryFeatureDesc[fm.m_iSourceFeature], m_targetSecondaryFeatureDesc[fm.m_iTargetFeature], secondaryDescCartProd);
#else
	featureDescCartProdMultiScale(m_sourceSecondaryFeatureDesc[fm.m_iSourceFeature], m_targetSecondaryFeatureDesc[fm.m_iTargetFeature], secondaryDescCartProd);
#endif //AR4_MATCH_FEATURES_STRATAFIED

	vector<mpcard> sourceSecondsIndices(nSourceSeconds), targetSecondsIndices(nTargetSeconds);
	for (i = 0; i < nSourceSeconds; i++)
		sourceSecondsIndices[i] = m_sourceSecondaryFeatureIndex[fm.m_iSourceFeature][i];
	for (i = 0; i < nTargetSeconds; i++)
		targetSecondsIndices[i] = m_targetSecondaryFeatureIndex[fm.m_iTargetFeature][i];

	vector<pair<mpcard, mpcard>> refMatch(1);
	refMatch[0].first = fm.m_iSourcePoint;
	refMatch[0].second = fm.m_iTargetPoint;

	//find feature matches using forward matching scheme
	matchFeaturesForwardStratified(	nSourceSeconds, nTargetSeconds, 
									sourceSecondsIndices, targetSecondsIndices, 
									m_sourceSecondaryFeatureEntropy[fm.m_iSourceFeature], secondaryDescCartProd, secondaryMatchQueue, 
									&refMatch);

	//now from matched secondary features, get directions
	const mpcard numSecondaryMatches = secondaryMatchQueue.size();

	//compute geodesics from source primary to source secondaries
	vector<Vector3f> sourceTangDirs(nSourceSeconds);
	vector<TAngleIdx> sourceAngle(nSourceSeconds);
	vector<mpcard> sourceDests(nSourceSeconds);
	vector<float> sourceDists;
	vector<Vector3f> sourceGrads, dirsAtSource;
	vector<vector<Vector3f>> sourcePaths;

	//compute geodesic all at once
	for (i = 0; i < nSourceSeconds; i++)
		sourceDests[i] = m_sourceSecondaryFeatureIndex[fm.m_iSourceFeature][i];
	m_pMatch->getSparseSourceGeodesics()->computeGeodesicsEarlyStop(m_avgDenseEdgeLength * 0.1f, fm.m_iSourcePoint, sourceDests, sourceDists, sourceGrads, &sourcePaths);

	//relax paths for accuracy
	m_pMatch->getSparseSourceGeodesics()->setRelaxationSoftConstraint(0.01f);
	m_pMatch->getSparseSourceGeodesics()->relaxPaths(4, sourcePaths, sourceDists, sourceGrads, dirsAtSource);

	//compute geodesics from target primary to target secondaries
	vector<Vector3f> targetTangDirs(nTargetSeconds);
	vector<TAngleIdx> targetAngle(nTargetSeconds);
	vector<mpcard> targetDests(nTargetSeconds);
	vector<float> targetDists;
	vector<Vector3f> targetGrads;
	vector<vector<Vector3f>> targetPaths;

	//compute geodesics all at once
	for (i = 0; i < nTargetSeconds; i++)
		targetDests[i] = m_targetSecondaryFeatureIndex[fm.m_iTargetFeature][i];
	m_pMatch->getSparseTargetGeodesics()->computeGeodesicsEarlyStop(m_avgDenseEdgeLength * 0.1f, fm.m_iTargetPoint, targetDests, targetDists, targetGrads, &targetPaths);

	//relax paths for accuracy
	m_pMatch->getSparseTargetGeodesics()->setRelaxationSoftConstraint(0.01f);
	m_pMatch->getSparseTargetGeodesics()->relaxPaths(4, targetPaths, targetDists, targetGrads, dirsAtSource);

	float32 scoreStart, invScoreStart;

	const mpcard numStartMatches = featureMatchQueue.size();

	//now compute directions for secondary feature matches
	for (i = 0; i < numSecondaryMatches; i++)
	{
		const mpcard isrcsec = secondaryMatchQueue[i].m_iSourceFeature;
		const mpcard itrgsec = secondaryMatchQueue[i].m_iTargetFeature;

		Vector3f sdir = sourcePaths[isrcsec][1] - sourcePaths[isrcsec][0];
		sdir.normalize();

		Vector3f tdir = targetPaths[itrgsec][1] - targetPaths[itrgsec][0];
		tdir.normalize();

		Vector2f sdir2 = makeVector2f(sdir * sourceTS[0], sdir * sourceTS[1]);
		Vector2f tdir2 = makeVector2f(tdir * targetTS[0], tdir * targetTS[1]);

		float32 alpha = atan2(sdir2[1], sdir2[0]);
		if (alpha < 0.f)
			alpha += TWO_PI;
		float32 beta = atan2(tdir2[1], tdir2[0]);
		if (beta < 0.f)
			beta += TWO_PI;

		float32 delta = beta - alpha;
		if (delta < 0.f)
			delta += TWO_PI;

		if (i == 0)
		{
			featureMatchQueue[posInQueue].m_targetDir = delta;
			scoreStart = secondaryMatchQueue[0].m_score;
			invScoreStart = AR4SMFeatureMatch::invertScore(scoreStart);
		}
		else
		{
			secondaryMatchQueue[i].adjustScore(invScoreStart);

			AR4SMFeatureMatch fmNew;
			fmNew.m_iSourceFeature = fm.m_iSourceFeature;
			fmNew.m_iSourcePoint = fm.m_iSourcePoint;
			fmNew.m_iTargetFeature = fm.m_iTargetFeature;
			fmNew.m_iTargetPoint = fm.m_iTargetPoint;
			fmNew.m_targetDir = delta;
			fmNew.m_dirOrder = i;
			fmNew.m_score = secondaryMatchQueue[i].m_score;
			fmNew.adjustScore(fm.m_score);
			featureMatchQueue.push_back(fmNew);
		}
	}

	AR4SMFeatureMatch::sortBestFirst(featureMatchQueue.begin() + posInQueue + 1, featureMatchQueue.end());
}

void PCIGMOD14ShapeMatching::savePartialMatch(AR4PartialMatch3f* pMatch)
{
	AR4PartialMatchMaterial pmmat;
	string strMatName = pmmat.getName();

	SceneGraphNode* pNode = getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strPartialSequence);
	SGRelativeTimeAnimationNode* pPMSequence = dynamic_cast<SGRelativeTimeAnimationNode*>(pNode);
	if (pPMSequence == NULL)
	{
		throw EInvalidState("PCIGMOD14ShapeMatching::savePartialMatch(...): partial match sequence node is not available to save partial match in");
	}

	UnstructuredInCorePointCloud* pExpCloud = pMatch->exportAsCompactPointCloud();
	GenericSwapAllOutOfCorePointCloud* pSwap = new GenericSwapAllOutOfCorePointCloud();
	pSwap->clearAndSetup(pExpCloud, m_strDirSwapPath + string("partialMatch") + intToStr(m_partialMatchCount, 4));

	pExpCloud->setMaterialIndex(getScene()->getGLMaterialIndex(strMatName));
	pSwap->setMaterialIndex(getScene()->getGLMaterialIndex(strMatName));

	string nodeName = string("partialMatch") + intToStr(m_partialMatchCount, 4);

	SGObjectNode* pSwapNode = NULL;
	pNode = getSceneGraphNodeByName(getScene(), nodeName);
	mpint ichild = pPMSequence->getChildIndex(nodeName);
	if (ichild == -1)
	{
		pSwapNode = new SGObjectNode(pSwap);
		pPMSequence->addChildNode(pSwapNode);
	}
	else
	{
		pNode = pPMSequence->getChildNode(getScene()->getRootState(), (mpcard)ichild);
		pSwapNode = dynamic_cast<SGObjectNode*>(pNode);
		if (pSwapNode == NULL)
		{
			pPMSequence->removeChildNode(nodeName);
			pSwapNode = new SGObjectNode(pSwap);
			pPMSequence->addChildNode(pSwapNode);
		}
		else
			pSwapNode->clearAndSetup(pSwap);
	}
	
	pSwapNode->setName(nodeName);

	m_partialMatchCount++;
}

float PCIGMOD14ShapeMatching::computeFeatureMatchCoverage(AR4PartialMatch3f* pMatch, const AR4SMFeatureMatch& fm)
{
	if (pMatch->isMatched(fm.m_iSourcePoint))
	{
		const float agreeMu = pMatch->getAvgSourceMatchLevelEdgeWeight();
		const float agreeLambda = 1.f / agreeMu;
		const float distStartMu = pMatch->getAvgSourceMatchLevelEdgeWeight() * 5.f;
		const float distStartLambda = 1.f / distStartMu;

		const Vector3f partial = pMatch->getMatch(fm.m_iSourcePoint);
		const Vector3f feature = m_pTargetManifold->getDiscrPoint(fm.m_iTargetPoint);
		const float distToStart = pMatch->getSourceDistToStart(fm.m_iSourcePoint);
		Vector3f grad;
		float dist;
		pMatch->getSparseTargetGeodesics()->computeGeodesicSingleDest(0.1f * m_avgDenseEdgeLength, partial, feature, dist, grad);
		return 1.f - exp(-(dist * agreeLambda + distToStart * distStartLambda));
	}
	return 1.f;
}

void PCIGMOD14ShapeMatching::updateFeatureMatchQueue(AR4PartialMatch3f* pMatch)
{
	const mpcard numMatches = m_featureMatchQueue.size();

	mpcard i;

	for (i = m_nextFeatureMatchIndex; i < numMatches; i++)
	{
		if (m_featureMatchQueue[i].m_iSourcePoint >= m_pSourceManifold->getNumDiscrPoints())
		{
			debugOutput << "yeah, we about to crash " << i << " " << m_featureMatchQueue[i].m_iSourcePoint << "\n";
			debugOutput.flush();
		}
		const float coverage = computeFeatureMatchCoverage(pMatch, m_featureMatchQueue[i]);
		m_featureMatchQueue[i].adjustScore(coverage);
	}

	AR4SMFeatureMatch::sortBestFirst(m_featureMatchQueue.begin() + m_nextFeatureMatchIndex, m_featureMatchQueue.end());
}

void PCIGMOD14ShapeMatching::visualizeFeatureMatchSequence()
{
	mpcard i, j, k, m;
	AR4SMFeatureMatch fm;

	const float priorityMax = m_featureMatchQueue.front().m_score;
	const float invPriorityMax = AR4SMFeatureMatch::invertScore(priorityMax);

	TExistFMSet processFeatMatches;

	debugOutput << "feature match queue size = " << m_featureMatchQueue.size() << "\n";

	i = 0;
	m = 0;
	while (i < m_featureMatchQueue.size() && m < 500)
	{
		fm = m_featureMatchQueue[i];

		if (processFeatMatches.find(fm) != processFeatMatches.end())
		{
			i++;
			continue;
		}

		processFeatMatches.insert(fm);

		k = fm.m_iSourceFeature * m_nTargetFeatures + fm.m_iTargetFeature;
		const float pointRadius = 2.f;
		Vector3f featPos, featColor = rainbowGradient3f(AR4SMFeatureMatch::adjustScore(fm.m_score, invPriorityMax));
		debugRenderer->beginRenderJob_OneFrame("GMOD14ShapeMatching_FeatureMatch", m);

		featPos = m_pSourceManifold->getDiscrPoint(fm.m_iSourcePoint);
		debugRenderer->addSphere(featPos, m_avgDenseEdgeLength * pointRadius, featColor);
#if 1
		for (j = 0; j < m_sourceSecondaryFeatureIndex[fm.m_iSourceFeature].size(); j++)
		{
			featPos = m_pSourceManifold->getDiscrPoint(m_sourceSecondaryFeatureIndex[fm.m_iSourceFeature][j]);
			debugRenderer->addSphere(featPos, m_avgDenseEdgeLength * pointRadius * 0.5f, featColor * 0.5f);
		}
#endif

		featPos = m_pTargetManifold->getDiscrPoint(fm.m_iTargetPoint);
		debugRenderer->addSphere(featPos, m_avgDenseEdgeLength * pointRadius, featColor);
#if 1
		for (j = 0; j < m_targetSecondaryFeatureIndex[fm.m_iTargetFeature].size(); j++)
		{
			featPos = m_pTargetManifold->getDiscrPoint(m_targetSecondaryFeatureIndex[fm.m_iTargetFeature][j]);
			debugRenderer->addSphere(featPos, m_avgDenseEdgeLength * pointRadius * 0.5f, featColor * 0.5f);
		}
#endif

		debugRenderer->endRenderJob();
		i++;
		m++;
	}
}

void PCIGMOD14ShapeMatching::visualizeFeatureMatchSequenceWithDirections()
{
	vector<AR4SMFeatureMatch> tempQueue = m_featureMatchQueue;
	TExistFMSet processFeatMatches;

	AR4SMFeatureMatch fm;
	mpcard ifeat = 0;
	while (getBestFeatureMatch(m_featureMatchQueue, ifeat, m_processedFeatures, fm) && ifeat < 500)
	{
		Vector3f sourcePoint = m_pSourceManifold->getDiscrPoint(fm.m_iSourcePoint);
		Vector3f targetPoint = m_pTargetManifold->getDiscrPoint(fm.m_iTargetPoint);
		
		Vector3f sourceDir, targetDir;

		Vector3f proj, norm;
		StaticMatrix<float32, 2, 3> ts;

		m_pSourceManifold->project(sourcePoint, proj, norm, ts);
		sourceDir = ts[0];

		m_pTargetManifold->project(targetPoint, proj, norm, ts);
		targetDir = ts[0] * cos(fm.m_targetDir) + ts[1] * sin(fm.m_targetDir);

		visualizePointAndDirMatch(sourcePoint, sourceDir, targetPoint, targetDir);
	}
}

void PCIGMOD14ShapeMatching::areaResize(card32 width, card32 height)
{
   areaResizePassToCam(width, height);
}

void PCIGMOD14ShapeMatching::mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState)
{
   mouseWheelRotatedPassToCam(rotatedDelta, modifiersState);
}


void PCIGMOD14ShapeMatching::keyDown(GeneralKey key) 
{
   keyDownPassToCam(key);
}

void PCIGMOD14ShapeMatching::keyUp(GeneralKey key) 
{
   keyUpPassToCam(key);
}

void PCIGMOD14ShapeMatching::initSceneStructure()
{
	SceneGraphNode* pPlaceHolder = NULL;
	SGListNode* plistresult = NULL;

	pPlaceHolder = getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder);
	if (pPlaceHolder == NULL)
	{
		plistresult = new SGListNode();
		plistresult->setName(m_strResultFolder);
		getScene()->getRootNode()->addChildNode(plistresult);
	}
	else
	{
		plistresult = dynamic_cast<SGListNode*>(pPlaceHolder);
		if (plistresult == NULL)
		{
			getScene()->getRootNode()->removeChildNode(m_strResultFolder);
			plistresult = new SGListNode();
			plistresult->setName(m_strResultFolder);
			getScene()->getRootNode()->addChildNode(plistresult);
		}
	}

	SGRelativeTimeAnimationNode* pPartialMatchNode = NULL;

	pPlaceHolder = getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strPartialSequence);
	if (pPlaceHolder == NULL)
	{
		pPartialMatchNode = new SGRelativeTimeAnimationNode();
		pPartialMatchNode->setName(m_strPartialSequence);
		plistresult->addChildNode(pPartialMatchNode);
	}
	else
	{
		pPartialMatchNode = dynamic_cast<SGRelativeTimeAnimationNode*>(pPlaceHolder);
		if (pPartialMatchNode == NULL)
		{
			plistresult->removeChildNode(m_strPartialSequence);
			pPartialMatchNode = new SGRelativeTimeAnimationNode();
			pPartialMatchNode->setName(m_strPartialSequence);
			plistresult->addChildNode(pPartialMatchNode);
		}
	}

	SGRelativeTimeAnimationNode* pCMSequenceNode = NULL;

	pPlaceHolder = getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strClusterSequence);
	if (pPlaceHolder == NULL)
	{
		pCMSequenceNode = new SGRelativeTimeAnimationNode();
		pCMSequenceNode->setName(m_strClusterSequence);
		plistresult->addChildNode(pCMSequenceNode);
	}
	else
	{
		pCMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(pPlaceHolder);
		if (pCMSequenceNode == NULL)
		{
			plistresult->removeChildNode(m_strClusterSequence);
			pCMSequenceNode = new SGRelativeTimeAnimationNode();
			pCMSequenceNode->setName(m_strClusterSequence);
			plistresult->addChildNode(pCMSequenceNode);
		}
	}
}

void PCIGMOD14ShapeMatching::initData()
{
	initSceneStructure();

	delete m_pSourceManifold;
	m_pSourceManifold = createM3f(source);
	delete m_pTargetManifold;
	m_pTargetManifold = createM3f(target);


	checkForFeatures();
	if (m_nSourceFeatures > 0 && m_nTargetFeatures > 0)
	{
		debugOutput << m_nSourceFeatures << " features available on source\n";
		debugOutput << m_nTargetFeatures << " features available on target\n";
		if (settings->featureMatchSource == AR4Settings::fmSrcInternal)
			initFeatureMatching();
	}

	delete m_pMatch;
	m_pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);

	generateSubSampleHierarchy();

	if (m_nSourceFeatures > 0 && m_nTargetFeatures > 0)
	{
		collectSecondariesFromPrimaries();
	}

	m_bDataInit = true;

	AR4PartialMatchMaterial mat;
	string strMatName = mat.getName();
	if (getScene()->getGLMaterialIndex(strMatName) > getScene()->getNumGLMaterials())
		getScene()->addGLMaterial(new AR4PartialMatchMaterial);

	redrawScene();
	ObjectViewsTable::update(this);
}

void PCIGMOD14ShapeMatching::generateSubSampleHierarchy()
{
	mpcard i, j, k, numnbrs, numedges;
	float avgedge;
	
	UnstructuredInCorePointCloud* pcsource = m_pSourceManifold->getPointCloud();
	UnstructuredInCorePointCloud* pctarget = m_pTargetManifold->getPointCloud();
	PointSet* pssource = pcsource->getPointSet();
	PointSet* pstarget = pctarget->getPointSet();
	AAT pos;
	Vector3f v1, v2;

	UnstructuredInCorePointCloud* pclast = NULL;
	UnstructuredInCorePointCloud* pcnext = NULL;

	m_ppSubSampleSource = new InCorePCTopologyGraph*[m_nSubSampleLevels];
	for (i = 0; i < m_nSubSampleLevels; i++)
		m_ppSubSampleSource[i] = NULL;

	m_ppSubSampleTarget = new InCorePCTopologyGraph*[m_nSubSampleLevels];
	for (i = 0; i < m_nSubSampleLevels; i++)
		m_ppSubSampleTarget[i] = NULL;

	Vector3f colors[6];
	colors[0].setZero();
	colors[0][0] = 1.f;
	colors[1].setZero();
	colors[1][1] = 1.f;
	colors[2].setZero();
	colors[2][2] = 1.f;
	colors[3].setZero();
	colors[3][0] = 1.f;
	colors[3][1] = 1.f;
	colors[4].setZero();
	colors[4][1] = 1.f;
	colors[4][2] = 1.f;
	colors[5].setZero();
	colors[5][0] = 1.f;
	colors[5][2] = 1.f;

	PCCTopologyPreservingResampling tpr;
	PCCTopologyAndFeaturePreservingResampling tafpr;

	InCorePCTopologyGraph* tpg = NULL;
	InCorePCTopologyGraph* tpgCopy = NULL;
	InCorePCTopologyGraph* tpgSubSample = NULL;
	TopologyAttachment* tatt = dynamic_cast<TopologyAttachment*>(pcsource->getAttachments()->getData(TopologyAttachment::getDefaultName()));
	tpg = tatt->getTopology();
	PointCloudIterator* pc_it = tpg->createIterator(pssource);
	TopologyIterator* top_it = dynamic_cast<TopologyIterator*>(pc_it);
	RandomAccessPointCloudIterator* rand_it = dynamic_cast<RandomAccessPointCloudIterator*>(pc_it);

	string strLevelTopoName;

	TopologyAttachment* pNewTopoAttach;

	pos = pcsource->getAAT("position");
	avgedge = 0.f;
	numedges = 0;
	for (i = 0; i < pcsource->getNumPoints(); i++)
	{
		v1 = pssource->get3f(i, pos);
		rand_it->goToIndex(i);
		numnbrs = top_it->getNumNeighbours();
		for (j = 0; j < numnbrs; j++)
		{
			k = top_it->getNeighbour(j);
			v2 = pssource->get3f(k, pos);
			avgedge += (v1 - v2).getNorm();
			numedges++;
		}
	}
	avgedge /= (float)numedges;

	m_avgDenseEdgeLength = avgedge;

#if AR4_DEBUG_OUTPUT
	debugOutput << "source point cloud, num points: " << pcsource->getNumPoints() << "\n";
	debugOutput << "source point cloud, avg edge length: " << avgedge << "\n";
#endif //AR4_DEBUG_OUTPUT

	delete pc_it;

	//add index attribute, so we can keep track of vertices over different hierarchy levels
	checkAttribute(pcsource, m_strBaseIndexAttribName, 1, VAD::DATA_FORMAT_CARD32);
	m_pSourceManifold->resetup(pcsource);

	mpcard numPoints = pcsource->getNumPoints();
	pssource = pcsource->getPointSet();
	AAT bi = pcsource->getAAT(m_strBaseIndexAttribName);
	for (i = 0; i < numPoints; i++)
		pssource->set1u(i, bi, (card32)i);

	pclast = pcsource;
	pcnext = NULL;

#if AR4_DEBUG_RENDER
	debugRenderer->beginRenderJob(m_strRenderJobName + "_TopologyHierarchy");
#endif //AR4_DEBUG_RENDER

	mpcard numNewTopos = 0;
	for (i = 0; i < m_nSubSampleLevels; i++)
	{
		strLevelTopoName = m_strSparseTopoName + "_s_" + intToStr(i);
		if (pcsource->getAttachments()->getData(strLevelTopoName) != NULL)
		{
			debugOutput << "PCIGMOD14ShapeMatching::generateSubSampleHierarchy(): level " << i << " topology already attached to source\n";
			continue;
		}

		pcnext = copyToUnstructuredInCorePointCloud(pclast);

		tatt = dynamic_cast<TopologyAttachment*>(pclast->getAttachments()->getData(TopologyAttachment::getDefaultName()));
		tpg = tatt->getTopology();
		tpgCopy = tpg->copy();
		tatt = new TopologyAttachment();
		tatt->setup(TopologyAttachment::getDefaultName(), AttachedData::ADF_PERSISTENT);
		tatt->setTopology(tpgCopy);            
		pcnext->getAttachments()->attachData(tatt);

		tafpr.setSpacing(avgedge * (float)(1 << (i + 1)));
		tafpr.resamplePC(pcnext);

		//for next level, increase the required feature level
		tafpr.setFeatureLevel(tafpr.getFeatureLevel() + 1);

		pcnext->deleteDelOnWriteAttachments();

#if AR4_DEBUG_OUTPUT
		debugOutput << "level " << i << " num points: " << pcnext->getNumPoints() << "\n";
#endif //AR4_DEBUG_OUTPUT

		tatt = dynamic_cast<TopologyAttachment*>(pcnext->getAttachments()->getData(TopologyAttachment::getDefaultName()));
		tpgCopy = tatt->getTopology();

		pc_it = tpgCopy->createIterator(pcnext->getPointSet());
		top_it = dynamic_cast<TopologyIterator*>(pc_it);
		rand_it = dynamic_cast<RandomAccessPointCloudIterator*>(pc_it);

		VertexDescriptor topoVertDesc;
		topoVertDesc.pushAttrib(VAD("vindex", 2, VAD::DATA_FORMAT_INT32));
		topoVertDesc.pushAttrib(VAD("weight", 1, VAD::DATA_FORMAT_FLOAT32));
		tpgSubSample = new InCorePCTopologyGraph();
		tpgSubSample->clearAndSetup(&topoVertDesc);

		AAT weightAAT = tpgSubSample->getEdges()->getAAT("weight");

		bi = pcnext->getAAT(m_strBaseIndexAttribName);

		pos = pcnext->getAAT("position");
		for (j = 0; j < pcnext->getNumPoints(); j++)
		{
#if AR4_DEBUG_RENDER
			debugRenderer->addPoint(pcnext->getPointSet()->get3f(j, pos), colors[i]);
			debugRenderer->addSphere(pcnext->getPointSet()->get3f(j, pos), 0.25f * (i + 1), colors[i]);
			debugRenderer->addSphere(pcnext->getPointSet()->get3f(j, pos), 0.5f * m_avgDenseEdgeLength * (float)(i + 1), colors[i]);
#endif //AR4_DEBUG_RENDER

			PointCloudTopologyGraph::VertexIndex i1, i2;
			i1 = (PointCloudTopologyGraph::VertexIndex)pcnext->getPointSet()->get1u(j, bi);
			rand_it->goToIndex(j);
			numnbrs = top_it->getNumNeighbours();
			for (k = 0; k < numnbrs; k++)
			{
				i2 = (PointCloudTopologyGraph::VertexIndex)pcnext->getPointSet()->get1u(top_it->getNeighbour((card32)k), bi);
				tpgSubSample->addEdge(i1, i2);
				tpgSubSample->getEdges()->set1f(tpgSubSample->getEdgeIndex(i1, k), weightAAT, -1.f);
			}
		}

		m_ppSubSampleSource[i] = tpgSubSample;

		//attach sparser topology to point cloud so that it can be saved
		pNewTopoAttach = new TopologyAttachment();
		pNewTopoAttach->setup(strLevelTopoName, 1);
		pNewTopoAttach->setTopology(m_ppSubSampleSource[i]);
		pcsource->getAttachments()->attachData(pNewTopoAttach);

		pcsource->deleteDelOnWriteAttachments();
		
		numNewTopos++;

		if (pclast != pcsource)
			delete pclast;
		pclast = pcnext;

		delete pc_it;
	}

	delete pcnext;

	pcsource->deleteDelOnWriteAttachments();

#if AR4_DEBUG_RENDER
	debugRenderer->endRenderJob();
#endif //AR4_DEBUG_RENDER

	m_pMatch->addTopologySampleSpacing(avgedge);
	for (i = 0; i < m_nSubSampleLevels; i++)
		m_pMatch->addTopologySampleSpacing(avgedge * (float)(1 << (i + 1)));

	precomputeSourceGeodesics();

	redrawScene();
}

void PCIGMOD14ShapeMatching::precomputeSourceGeodesics()
{
	if (m_pMatch == NULL)
		m_pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);

	if (m_pMatch == NULL)
		throw ERangeCheck("PCIGMOD14ShapeMatching::precomputeSourceGeodesics(): unable to allocate m_pMatch");

	m_pMatch->setIndexAttrString(m_strBaseIndexAttribName);
	m_pMatch->setUsePathRelaxation(settings->bUsePathRelaxation);
	m_pMatch->precomputeSourceGeodesics(m_nSubSampleLevels, m_strSparseTopoName);
}

void PCIGMOD14ShapeMatching::initMatching()
{
	///////////////////////////////////
	//initialize partial match
	if (m_pMatch == NULL)
		m_pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);

	if (m_pMatch == NULL)
		throw ERangeCheck("PCIGMOD14ShapeMatching::initMatching(): unable to allocate m_pMatch");

	if (m_sourceSelPoint >= 0 && m_sourceDirPoint >= 0 && selPoint >= 0 && m_targetDirPoint >= 0)
	{
#if AR4_DEBUG_OUTPUT
		debugOutput << "source: " << m_sourceSelPoint << "; target: " << selPoint << "\n";
		debugOutput << "source: " << m_sourceDirPoint << "; target: " << m_targetDirPoint << "\n";
#endif //AR4_DEBUG_OUTPUT

		m_sourceDir = (m_pSourceManifold->getDiscrPoint(m_sourceDirPoint) - m_pSourceManifold->getDiscrPoint(m_sourceSelPoint));
		m_sourceDir.normalize();
		m_targetDir = (m_pTargetManifold->getDiscrPoint(m_targetDirPoint) - m_pTargetManifold->getDiscrPoint(selPoint));
		m_targetDir.normalize();

#if AR4_DEBUG_OUTPUT
		debugOutput << "source dir: " << m_sourceDir << "\n";
		debugOutput << "target dir: " << m_targetDir << "\n";
#endif //AR4_DEBUG_OUTPUT
	}

	m_pMatch->setStretchThreshold(settings->stretchThreshold);

	///////////////////////////////////
	//initialize growing matcher
	if (m_pGrower == NULL)
		m_pGrower = new AR4GrowingMatcher3f(m_pSourceManifold, m_pTargetManifold);

	if (m_pGrower == NULL)
		throw ERangeCheck("PCIGMOD14ShapeMatching::initMatching(): NULL growing matcher");

	m_pGrower->setGrowingMode((AR4GrowingMatcher3f::EGrowingMode)m_growingMode);

	///////////////////////////////////
	//initialize global bundle
	if (m_pGlobalBundle == NULL)
		m_pGlobalBundle = new AR4SparseBundle3f(m_pSourceManifold, m_pTargetManifold);

	if (m_pGlobalBundle == NULL)
		throw ERangeCheck("PCIGMOD14ShapeMatching::initMatching(): NULL global bundle");
}

void PCIGMOD14ShapeMatching::initMatch(AR4PartialMatch3f* pMatch, const mpcard sourceIndex, const Vector3f& sourceDir, const Vector3f& targetPoint, const Vector3f& targetDir)
{
	pMatch->setStretchThreshold(settings->stretchThreshold);
#if AR4_RECORD_REJECTED_OUTLIERS
	pMatch->setRecordRejectedOutliers(true);
#endif //AR4_RECORD_REJECTED_OUTLIERS

	m_pGrower->reset();

	pMatch->initMatchesFromPointAndDir(sourceIndex, sourceDir, targetPoint, targetDir, 4);

	pMatch->finalizeMatchCohort();
	pMatch->updateTargetGeodesics();

	visualizeMatching(pMatch);
}


void PCIGMOD14ShapeMatching::runGrowingMatcher()
{
	Timer timer;

	if (!m_bDataInit)
		initData();

	if (m_pSourceManifold == NULL)
		throw ERangeCheck("PCIGMOD14ShapeMatching::runGrowingMatcher(): NULL source manifold");
	if (m_pTargetManifold == NULL)
		throw ERangeCheck("PCIGMOD14ShapeMatching::runGrowingMatcher(): NULL target manifold");

	initMatching();

	mpcard n = m_pMatch->getNumMatchedIndices();

	if (n == 0)
	{
		visualizePointAndDirMatch(m_pSourceManifold->getDiscrPoint(m_sourceSelPoint), m_sourceDir, m_pTargetManifold->getDiscrPoint(selPoint), m_targetDir);
		initMatch(m_pMatch, m_sourceSelPoint, m_sourceDir, m_pTargetManifold->getDiscrPoint(selPoint), m_targetDir);
	}

	n = m_pMatch->getNumMatchedIndices();

	m_pGrower->setDebugOutput(false);

#if AR4_DEBUG_OUTPUT
	debugOutput.flush();
#endif //AR4_DEBUG_OUTPUT

	mpcard step = 0;
	while (step < settings->growingKillSteps)
	{
		m_pGrower->improveMatch(m_pMatch);
		if (m_pMatch->getNumMatchedIndices() == n)
			break;

		initVisualization();

		if (m_bUseGlobalBundle && m_pMatch->getNumMatchedIndices() > 2 * m_pGlobalBundle->getNumMatchesLastRun())
		{
			m_pGlobalBundle->setStepScale(m_avgDenseEdgeLength * 0.5f);
			m_pGlobalBundle->improveMatch(m_pMatch);
		}

		n = m_pMatch->getNumMatchedIndices();

		step++;

		debugOutput.flush();
		visualizeMatching(m_pMatch);
		redrawScene();
	}

	if (m_bUseGlobalBundle && m_pMatch->getNumMatchedIndices() > m_pGlobalBundle->getNumMatchesLastRun())
	{
		m_pGlobalBundle->setStepScale(m_avgDenseEdgeLength * 0.5f);
		m_pGlobalBundle->improveMatch(m_pMatch);
	}

	debugOutput.flush();
	visualizeMatching(m_pMatch);
	redrawScene();

	m_bErrorDirty = true;
	m_bStretchDirty = true;

	debugOutput << "PCIGMOD14ShapeMatching::rowGrowingMatcher():\n";
	debugOutput << "\tsteps: " << step << "\n";
	debugOutput << "\ttime: " << convertTimeToString(timer.getDeltaValue()) << "\n";
	debugOutput << "\tnumber of matches: " << m_pMatch->getNumMatchedIndices() << "\n";

	m_pMatch->reportAllProfileResults();
	m_pGrower->reportProfiling();
}

void PCIGMOD14ShapeMatching::runGrowingMatcherOneStep()
{
	ObjectViewsTable::update(this);

	if (!m_bDataInit)
		initData();

	if (m_pSourceManifold == NULL)
		throw ERangeCheck("PCIGMOD14ShapeMatching::runGrowingMatcherOneStep(): NULL source manifold");
	if (m_pTargetManifold == NULL)
		throw ERangeCheck("PCIGMOD14ShapeMatching::runGrowingMatcherOneStep(): NULL target manifold");

	initMatching();
	initMatch(m_pMatch, m_sourceSelPoint, m_sourceDir, m_pTargetManifold->getDiscrPoint(selPoint), m_targetDir);

	mpcard i, k, n = m_pMatch->getNumMatchedIndices();
#if AR4_DEBUG_OUTPUT
	debugOutput << "num matches (start): " << n << "\n";
	debugOutput.flush();
#endif //AR4_DEBUG_OUTPUT

	initVisualization();

	m_pGrower->improveMatch(m_pMatch);

	if (m_bUseGlobalBundle && m_pMatch->getNumMatchedIndices() > 2 * m_pGlobalBundle->getNumMatchesLastRun())
	{
		m_pGlobalBundle->setStepScale(m_avgDenseEdgeLength * 0.5f);
		m_pGlobalBundle->improveMatch(m_pMatch);
	}

	n = m_pMatch->getNumMatchedIndices();
#if AR4_DEBUG_OUTPUT
	debugOutput << "num matches (end): " << n << "\n";
	debugOutput.flush();
#endif //AR4_DEBUG_OUTPUT

	m_currentGrowingStep++;

	visualizeMatching(m_pMatch);

	ObjectViewsTable::update(this);
	redrawScene();

	m_bErrorDirty = true;
	m_bStretchDirty = true;

	m_pMatch->reportAllProfileResults();
	m_pGrower->reportProfiling();
}

void PCIGMOD14ShapeMatching::runGrowingMatcherNextFeature()
{
	const card32 max_dirs_try = 4;

	mpcard i, n;

	if (!m_bFeatureQueueReady)
		resetFeatureMatchQueue();

	//get the next best feature match according to our criterion
	AR4SMFeatureMatch fm;
	if (!getBestFeatureMatch(fm))
	{
		debugOutput << "PCIGMOD14ShapeMatching::runGrowingMatcherNextFeature(): No more feature matches to grow from; stopping.\n";
		return;
	}

	Vector3f sourceDir, targetDir;
	Vector3f sourcePoint, targetPoint;
	Vector3f projPoint,  projNorm;
	AR4PartialMatch3f::TangentSpace tangSpace;
	
	sourcePoint = m_pSourceManifold->getDiscrPoint(fm.m_iSourcePoint);
	targetPoint = m_pTargetManifold->getDiscrPoint(fm.m_iTargetPoint);

	m_pSourceManifold->project(sourcePoint, projPoint, projNorm, tangSpace);
	sourceDir = tangSpace[0];

	m_pTargetManifold->project(targetPoint, projPoint, projNorm, tangSpace);
	targetDir = tangSpace[0] * cos(fm.m_targetDir) + tangSpace[1] * sin(fm.m_targetDir);

	visualizePointAndDirMatch(sourcePoint, sourceDir, targetPoint, targetDir);

	//initialize data structures and algorithms needed for partial matching
	initMatching();

	m_pGrower->setDebugOutput(false);

	Timer timer;

	//create new partial match and initialize it
	AR4PartialMatch3f* pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
	pMatch->copyGeodesicAndTopologyData(m_pMatch);
	initMatch(pMatch, fm.m_iSourcePoint, sourceDir, targetPoint, targetDir);

#if AR4_DEBUG_OUTPUT
	debugOutput.flush();
#endif //AR4_DEBUG_OUTPUT

	m_pGlobalBundle->improveMatch(pMatch);

	m_pGlobalBundle->resetNumMatchesLastRun();
	n = pMatch->getNumMatchedIndices();

	//proceed with growing matcher
	mpcard step = 0;
	while (step < settings->growingKillSteps)
	{
		m_pGrower->improveMatch(pMatch);
		if (pMatch->getNumMatchedIndices() == n)
			break;

		initVisualization();

		if (m_bUseGlobalBundle && pMatch->getNumMatchedIndices() > 2 * m_pGlobalBundle->getNumMatchesLastRun())
		{
			m_pGlobalBundle->setStepScale(m_avgDenseEdgeLength * 0.5f);
			m_pGlobalBundle->improveMatch(pMatch);
		}

		n = pMatch->getNumMatchedIndices();

		step++;

		debugOutput.flush();
		visualizeMatching(pMatch);
	}

	if (m_bUseGlobalBundle && pMatch->getNumMatchedIndices() > m_pGlobalBundle->getNumMatchesLastRun())
	{
		m_pGlobalBundle->setStepScale(m_avgDenseEdgeLength * 0.5f);
		m_pGlobalBundle->improveMatch(pMatch);
	}

	if (pMatch->getNumMatchedIndices() > 20)//this will maybe be yet another parameter (YAP)
	{
		//re-order queue if this match has informed us about other features
		updateFeatureMatchQueue(pMatch);
		
		//save this match for later clustering and merging
		savePartialMatch(pMatch);
	}

	debugOutput.flush();
	visualizeMatching(pMatch);

	m_bErrorDirty = true;
	m_bStretchDirty = true;

	debugOutput << "PCIGMOD14ShapeMatching::runGrowingMatcherNextFeature():\n";
	debugOutput << "\tsteps: " << step << "\n";
	debugOutput << "\ttime: " << convertTimeToString(timer.getDeltaValue()) << "\n";
	debugOutput << "\tnumber of matches: " << pMatch->getNumMatchedIndices() << "\n";

	pMatch->reportAllProfileResults();
	m_pGrower->reportProfiling();

	delete pMatch;
}

void PCIGMOD14ShapeMatching::runFullMatcher()
{
	

	const card32 max_feat_try = settings->numFeatureTries;

	mpcard i, n;
	card32 featureTry = 0;

	//initialize data structures and algorithms needed for partial matching
	initMatching();

	m_pGrower->setDebugOutput(false);

	while (featureTry < max_feat_try)
	{
		//get the next best feature match according to our criterion
		AR4SMFeatureMatch fm;
		if (!getBestFeatureMatch(fm))
		{
			debugOutput << "PCIGMOD14ShapeMatching::runFullMatcher(): No more feature matches to grow from; stopping.\n";
			break;
		}

		Vector3f sourceDir, targetDir;
		Vector3f sourcePoint, targetPoint;
		Vector3f projPoint,  projNorm;
		AR4PartialMatch3f::TangentSpace tangSpace;

		sourcePoint = m_pSourceManifold->getDiscrPoint(fm.m_iSourcePoint);
		targetPoint = m_pTargetManifold->getDiscrPoint(fm.m_iTargetPoint);

		m_pSourceManifold->project(sourcePoint, projPoint, projNorm, tangSpace);
		sourceDir = tangSpace[0];

		m_pTargetManifold->project(targetPoint, projPoint, projNorm, tangSpace);
		targetDir = tangSpace[0] * cos(fm.m_targetDir) + tangSpace[1] * sin(fm.m_targetDir);

		visualizePointAndDirMatch(sourcePoint, sourceDir, targetPoint, targetDir);

		Timer timer;

		//create new partial match and initialize it
		AR4PartialMatch3f* pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
		pMatch->copyGeodesicAndTopologyData(m_pMatch);
		initMatch(pMatch, fm.m_iSourcePoint, sourceDir, targetPoint, targetDir);

#if AR4_DEBUG_OUTPUT
		debugOutput.flush();
#endif //AR4_DEBUG_OUTPUT

		m_pGlobalBundle->improveMatch(pMatch);

		m_pGlobalBundle->resetNumMatchesLastRun();
		n = pMatch->getNumMatchedIndices();

		//proceed with growing matcher
		mpcard step = 0;
		while (step < settings->growingKillSteps)
		{
			m_pGrower->improveMatch(pMatch);
			if (pMatch->getNumMatchedIndices() == n)
				break;

			debugOutput << "exitted grower\n";

			initVisualization();

			if (m_bUseGlobalBundle && pMatch->getNumMatchedIndices() > 2 * m_pGlobalBundle->getNumMatchesLastRun())
			{
				m_pGlobalBundle->setStepScale(m_avgDenseEdgeLength * 0.5f);
				m_pGlobalBundle->improveMatch(pMatch);
			}

			n = pMatch->getNumMatchedIndices();

			step++;

			debugOutput << "completed " << step << " steps\n";
			debugOutput.flush();
			visualizeMatching(pMatch);
		}

		if (m_bUseGlobalBundle && pMatch->getNumMatchedIndices() > m_pGlobalBundle->getNumMatchesLastRun())
		{
			m_pGlobalBundle->setStepScale(m_avgDenseEdgeLength * 0.5f);
			m_pGlobalBundle->improveMatch(pMatch);
		}

		if (pMatch->getNumMatchedIndices() > 20)//this will maybe be yet another parameter (YAP)
		{
			//re-order queue if this match has informed us about other features
			updateFeatureMatchQueue(pMatch);
			
			//save this match for later clustering and merging
			savePartialMatch(pMatch);
		}

		featureTry++;

		debugOutput.flush();
		visualizeMatching(pMatch);

		m_bErrorDirty = true;
		m_bStretchDirty = true;

		debugOutput << "PCIGMOD14ShapeMatching::runFullMatcher() - " << featureTry << ":\n";
		debugOutput << "\tsteps: " << step << "\n";
		debugOutput << "\ttime: " << convertTimeToString(timer.getDeltaValue()) << "\n";
		debugOutput << "\tnumber of matches: " << pMatch->getNumMatchedIndices() << "\n";

		pMatch->reportAllProfileResults();
		m_pGrower->reportProfiling();

		delete pMatch;
	}

}

void PCIGMOD14ShapeMatching::computeCorrespondence()
{


	X4_TIMER_START(computeCorrespondence);

	initData();
	resetFeatureMatchQueue();

	X4_TIMER_START(growing);
	runFullMatcher();
	X4_TIMER_STOP(growing);

	//save here
	BinaryOutputObjectStream* pOutputPartials = new BinaryOutputObjectStream((m_strDirSwapPath + "partials_matches.object").c_str());
	pOutputPartials->writeObject(getScene());
	delete pOutputPartials;

	//the following may be removed if we change to not use clustering before merging
	X4_TIMER_START(clustering);
	clusterPartialMatches();
	X4_TIMER_STOP(clustering);

	//save here
	BinaryOutputObjectStream* pOutputClusters = new BinaryOutputObjectStream((m_strDirSwapPath + "partials_matches_clusters.object").c_str());
	pOutputClusters->writeObject(getScene());
	delete pOutputClusters;

	X4_TIMER_START(merging);
	mergePartialMatches();
	X4_TIMER_STOP(merging);

	//save here
	BinaryOutputObjectStream* pOutputMerged = new BinaryOutputObjectStream((m_strDirSwapPath + "partials_matches_clusters_merged.object").c_str());
	pOutputMerged->writeObject(getScene());
	delete pOutputMerged;

	X4_TIMER_STOP(computeCorrespondence);
}

void PCIGMOD14ShapeMatching::clusterPartialMatches()
{
	X4_TIMER_START(clusterPartialMatches);

	mpcard i, j, k;

	AR4PartialMatchClusteringMDS3f mdsEmbedCluster(settings);
	AR4PartialMatchClusteringAP3f apCluster(settings);
	AR4PartialMatchSpectralClustering3f spectralCluster(settings);

	vector<vector<mpcard>> clusters;

	vector<GenericSwapAllOutOfCorePointCloud*> partialMatches;

	SGRelativeTimeAnimationNode* pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strPartialSequence));
	if (pPMSequenceNode == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::embedPartialMatches(): partial match list not available");

	const mpcard numPartials = pPMSequenceNode->getNumChildNodes(getScene()->getRootState());

	debugOutput << "PCIGMOD14ShapeMatching::embedPartialMatches()\n\tnumber of partial matches = " << numPartials << "\n";

	partialMatches.resize(numPartials);
	for (i = 0; i < numPartials; i++)
	{
		SceneGraphNode* pNode = pPMSequenceNode->getChildNode(getScene()->getRootState(), i);
		SGObjectNode* pObjNode = dynamic_cast<SGObjectNode*>(pNode);
		partialMatches[i] = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pObjNode->getSceneObject());
	}

	debugOutput << "\trunning clustering/embedding\n";

	UnstructuredInCorePointCloud* pEmbedPC = NULL;

	if (settings->mergeMode == AR4Settings::m_PCIARMMClusterGDL)
	{
		X4_TIMER_START(mdsEmbedCluster);
		mdsEmbedCluster.setPartialDistThresh(settings->clusterMergeThreshold);
		mdsEmbedCluster.setNumMinClusters(settings->numClustersMin);
		mdsEmbedCluster.run(m_pMatch, partialMatches, clusters);
		X4_TIMER_STOP(mdsEmbedCluster);

		//get embedding as point cloud and add it to the scene for visualization
		pEmbedPC = mdsEmbedCluster.getEmbeddedPointCloud();
	}
	else if (settings->mergeMode == AR4Settings::m_PCIARMMClusterAP)
	{
		X4_TIMER_START(apCluster);
		apCluster.setNumMinClusters(settings->numClustersMin);
		apCluster.run(m_pMatch, partialMatches, clusters);
		X4_TIMER_STOP(apCluster);
	}
	else if (settings->mergeMode == AR4Settings::m_PCIARMMClusterSpectral)
	{
		X4_TIMER_START(spectralCluster);
		spectralCluster.run(m_pMatch, partialMatches, clusters);
		X4_TIMER_STOP(spectralCluster);
	}

	m_partialMatchClusters = clusters;
	debugOutput << "\tnumber of clusters = " << m_partialMatchClusters.size() << "\n";

	debugOutput << "\treporting clusters:\n";
	for (i = 0; i < m_partialMatchClusters.size(); i++)
	{
		debugOutput << "\t\tcluster " << i << ": ";
		for (j = 0; j < m_partialMatchClusters[i].size(); j++)
			debugOutput << m_partialMatchClusters[i][j] << " ";
		debugOutput << "\n";
	}

	//scene graph nodes for the results folder and the point cloud itself
	SceneGraphNode* pResultFolder;
	SGObjectNode* pEmbedNode;

	//create the results folder if it does not already exist
	pResultFolder = getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder);
	if (pResultFolder == NULL)
		initSceneStructure();
	pResultFolder = getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder);
	if (pResultFolder == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::saveCorrespondences(): unable to create results folder, saving in root");

	//create the object node for the embedding if it does not already exist
	pEmbedNode = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/partial_match_embedding"));
	if (pEmbedNode == NULL)
		pEmbedNode = new SGObjectNode();

	if (pEmbedPC != NULL)
	{
		//add the embedding to the scene
		pEmbedNode->clearAndSetup(pEmbedPC);
		pEmbedNode->setName("partial_match_embedding");
		pResultFolder->addChildNode(pEmbedNode);

		//TODO: also want to visualize clusters... maybe as topology on embedding point cloud? Alt: assign color per-cluster
		checkAttribute(pEmbedPC, "color", 3, VAD::DATA_FORMAT_FLOAT32);
		AAT colAAT = pEmbedPC->getAAT("color");

		for (i = 0; i < m_partialMatchClusters.size(); i++)
		{
			const Vector3f clusterColor = makeVector3f(rnd01(), rnd01(), rnd01());
			for (j = 0; j < m_partialMatchClusters[i].size(); j++)
			{
				pEmbedPC->getPointSet()->set3f(m_partialMatchClusters[i][j], colAAT, clusterColor);
			}
		}
	}

	//save cluster indices as scene attachment
	//actually save as two attachments:
	//(1) stores the number of partial matches in each cluster
	//(2) stores the indices of partial matches in the order of the clusters
	AttachedCard32Vector* pAttClusterSizes = new AttachedCard32Vector();
	AttachedCard32Vector* pAttMatchIndices = new AttachedCard32Vector();

	pAttClusterSizes->setup("partialMatch_cluster_sizes", AttachedData::ADF_PERSISTENT);
	pAttMatchIndices->setup("partialMatch_indices", AttachedData::ADF_PERSISTENT);

	card32 countMatches = 0;
	pAttClusterSizes->data.resize(m_partialMatchClusters.size());
	for (i = 0; i < m_partialMatchClusters.size(); i++)
	{
		pAttClusterSizes->data[i] = (card32)m_partialMatchClusters[i].size();
		countMatches += (card32)m_partialMatchClusters[i].size();
	}

	pAttMatchIndices->data.resize(countMatches);
	k = 0;
	for (i = 0; i < m_partialMatchClusters.size(); i++)
	{
		for (j = 0; j < m_partialMatchClusters[i].size(); j++)
		{
			pAttMatchIndices->data[k] = (card32)m_partialMatchClusters[i][j];
			k++;
		}
	}

	getScene()->getAttachments()->attachData(pAttClusterSizes);
	getScene()->getAttachments()->attachData(pAttMatchIndices);

	X4_TIMER_STOP(clusterPartialMatches);
}

void PCIGMOD14ShapeMatching::mergePartialMatches()
{
	mpcard i, j, k;

	AR4PartialMatchMaterial pmmat;
	string strMatName = pmmat.getName();

	if (settings->mergeMode == AR4Settings::m_PCIARMMDirectBP)
	{
		mergePartialsDirectBP();
		return;
	}

	if (m_partialMatchClusters.size() == 0)
	{
		//load from scene
		AttachedCard32Vector* pAttClusterSizes = dynamic_cast<AttachedCard32Vector*>(getScene()->getAttachments()->getData("partialMatch_cluster_sizes"));
		AttachedCard32Vector* pAttMatchIndices = dynamic_cast<AttachedCard32Vector*>(getScene()->getAttachments()->getData("partialMatch_indices"));

		if (pAttClusterSizes == NULL || pAttMatchIndices == NULL)
		{
			clusterPartialMatches();
		}
		else
		{
			const card32 numTempClusters = (card32)pAttClusterSizes->data.size();
			m_partialMatchClusters.resize(numTempClusters);
			for (i = 0; i < numTempClusters; i++)
			{
				m_partialMatchClusters[i].resize(pAttClusterSizes->data[i]);
			}

			k = 0;
			for (i = 0; i < numTempClusters; i++)
			{
				const card32 tempClusterSize = (card32)m_partialMatchClusters[i].size();
				for (j = 0; j < tempClusterSize; j++)
				{
					m_partialMatchClusters[i][j] = pAttMatchIndices->data[k];
					k++;
				}
			}
		}
	}
	const mpcard numClusters = m_partialMatchClusters.size();

	SGRelativeTimeAnimationNode* pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strPartialSequence));
	if (pPMSequenceNode == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::mergePartialMatches(): partial match list not available");

	//create vector containing partial match point clouds
	vector<GenericSwapAllOutOfCorePointCloud*> partialMatchClouds;
	const mpcard numPartialMatches = (mpcard)pPMSequenceNode->getNumChildNodes(getScene()->getRootState());
	for (i = 0; i < numPartialMatches; i++)
	{
		SGObjectNode* pMatchNode = dynamic_cast<SGObjectNode*>(pPMSequenceNode->getChildNode(getScene()->getRootState(), i));
		if (pMatchNode == NULL)
			throw EInvalidState("PCIGMOD14ShapeMatching::mergePartialMatches(): unable to retrieve partial node");
		GenericSwapAllOutOfCorePointCloud* pSwap = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pMatchNode->getSceneObject());
		if (pSwap == NULL)
			throw EInvalidState("PCIGMOD14ShapeMatching::mergePartialMatches(): unabel to retrieve partial match point cloud");
		partialMatchClouds.push_back(pSwap);
	}

	AR4PartialMatchWeightedAverageMerger3f merger;
	AR4PartialMatchMetricPreservingMerger3f metricMerger;

	vector<GenericSwapAllOutOfCorePointCloud*> clusterClouds;

	for (i = 0; i < numClusters; i++)
	{
		AR4PartialMatch3f* pClusterMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
		pClusterMatch->setStretchThreshold(settings->stretchThreshold);
		pClusterMatch->copyGeodesicAndTopologyData(m_pMatch);

		debugOutput << "cluster " << i << ": " << m_partialMatchClusters[i].size() << " partial matches\n";

		X4_TIMER_START(mergePartialMatches);
		//merge the partial matches in this cluster
		merger.run(partialMatchClouds, m_partialMatchClusters[i], pClusterMatch);
		X4_TIMER_STOP(mergePartialMatches);

		//export point cloud and store out-or-core (potentially)
		UnstructuredInCorePointCloud* pClusterCloud = pClusterMatch->exportAsCompactPointCloud();

		GenericSwapAllOutOfCorePointCloud* pSwap = new GenericSwapAllOutOfCorePointCloud();
		pSwap->clearAndSetup(pClusterCloud, m_strDirSwapPath + "clusterMatch" + intToStr(i, 4));

		pSwap->setMaterialIndex(getScene()->getGLMaterialIndex(strMatName));
		clusterClouds.push_back(pSwap);

		delete pClusterMatch;
	}

	SGRelativeTimeAnimationNode* pCMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strClusterSequence));
	if (pCMSequenceNode == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::mergePartialMatches(): clustered partial match list not available");

	for (i = 0; i < numClusters; i++)
	{
		SGObjectNode* pCMNode = new SGObjectNode(clusterClouds[i]);
		pCMNode->setName("clusterMatch" + intToStr(i, 4));
		pCMSequenceNode->addChildNode(pCMNode);
	}
}

void PCIGMOD14ShapeMatching::clusterPointMatches()
{
	X4_TIMER_START(clusterPointMatches);

	mpcard i, j;

	AR4PartialMatchMaterial pmmat;
	string strMatName = pmmat.getName();

	AR4PartialMatchDirectCorrClustering3f corrClustering(settings);

	vector<vector<mpcard>> clusters;

	vector<GenericSwapAllOutOfCorePointCloud*> partialMatches;

	SGRelativeTimeAnimationNode* pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strPartialSequence));
	if (pPMSequenceNode == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::clusterPointMatches(): partial match list not available");

	const mpcard numPartials = pPMSequenceNode->getNumChildNodes(getScene()->getRootState());

	debugOutput << "PCIGMOD14ShapeMatching::clusterPointMatches()\n\tnumber of partial matches = " << numPartials << "\n";

	partialMatches.resize(numPartials);
	for (i = 0; i < numPartials; i++)
	{
		SceneGraphNode* pNode = pPMSequenceNode->getChildNode(getScene()->getRootState(), i);
		SGObjectNode* pObjNode = dynamic_cast<SGObjectNode*>(pNode);
		partialMatches[i] = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pObjNode->getSceneObject());
	}

	debugOutput << "\trunning clustering\n";
	corrClustering.setNumMinClusters(settings->numClustersMin);
	corrClustering.run(m_pMatch, partialMatches, clusters);

	m_partialMatchClusters = clusters;
	const mpcard numClusters = clusters.size();
	debugOutput << "\tnumber of clusters = " << numClusters << "\n";

	vector<GenericSwapAllOutOfCorePointCloud*> clusterClouds;

	//construct correspondence clusters as new partial maps
	for (i = 0; i < numClusters; i++)
	{
		AR4PartialMatch3f* pClusterMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);

		const mpcard numMatches = clusters[i].size();
		for (j = 0; j < numMatches; j++)
		{
			mpcard sourceIndex;
			Vector3f targetPoint;
			corrClustering.getPointMatch(clusters[i][j], sourceIndex, targetPoint);
			pClusterMatch->setMatch(sourceIndex, targetPoint);
		}

		//export point cloud and store out-or-core  
		UnstructuredInCorePointCloud* pClusterCloud = pClusterMatch->exportAsCompactPointCloud();

		GenericSwapAllOutOfCorePointCloud* pSwap = new GenericSwapAllOutOfCorePointCloud();
		pSwap->clearAndSetup(pClusterCloud, m_strDirSwapPath + "clusterMatch" + intToStr(i, 4));

		pSwap->setMaterialIndex(getScene()->getGLMaterialIndex(strMatName));
		clusterClouds.push_back(pSwap);

		delete pClusterMatch;
	}

	SGRelativeTimeAnimationNode* pCMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strClusterSequence));
	if (pCMSequenceNode == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::mergePartialMatches(): clustered partial match list not available");

	for (i = 0; i < numClusters; i++)
	{
		SGObjectNode* pCMNode = new SGObjectNode(clusterClouds[i]);
		pCMNode->setName("clusterMatch" + intToStr(i, 4));
		pCMSequenceNode->addChildNode(pCMNode);
	}

	X4_TIMER_STOP(clusterPointMatches);
}

void PCIGMOD14ShapeMatching::mergePartialsDirectBP()
{
	X4_TIMER_START(mergePartialsDirectBP);

	mpcard i, j;

	AR4PartialMatchMaterial pmmat;
	string strMatName = pmmat.getName();

	AR4PartialMatchBPDirectMerger3f bpMerge;

	vector<GenericSwapAllOutOfCorePointCloud*> partialMatches;

	SGRelativeTimeAnimationNode* pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strPartialSequence));
	if (pPMSequenceNode == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::mergePartialsDirectBP(): partial match list not available");

	const mpcard numPartials = pPMSequenceNode->getNumChildNodes(getScene()->getRootState());

	debugOutput << "PCIGMOD14ShapeMatching::mergePartialsDirectBP()\n\tnumber of partial matches = " << numPartials << "\n";

	partialMatches.resize(numPartials);
	for (i = 0; i < numPartials; i++)
	{
		SceneGraphNode* pNode = pPMSequenceNode->getChildNode(getScene()->getRootState(), i);
		SGObjectNode* pObjNode = dynamic_cast<SGObjectNode*>(pNode);
		partialMatches[i] = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pObjNode->getSceneObject());
	}

	AR4PartialMatch3f* pMergeMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
	pMergeMatch->copyGeodesicAndTopologyData(m_pMatch);
	pMergeMatch->setStretchThreshold(settings->stretchThreshold);

	bpMerge.run(partialMatches, vector<mpcard>(1, 0), pMergeMatch);

	UnstructuredInCorePointCloud* pMergeMatchPC = pMergeMatch->exportAsCompactPointCloud();

	SGListNode* pResultNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder));

	mpcard imat = getScene()->getGLMaterialIndex(strMatName);
	pMergeMatchPC->setMaterialIndex(imat);

	SGObjectNode* pMergeMatchNode = new SGObjectNode(pMergeMatchPC);
	pMergeMatchNode->setName("global_merge_match");

	pResultNode->addChildNode(pMergeMatchNode);

	X4_TIMER_STOP(mergePartialsDirectBP);
}

void PCIGMOD14ShapeMatching::resetFeatureMatchQueue()
{
	if (!m_bDataInit)
		initData();

	m_nextFeatureMatchIndex = 0;
	if (settings->featureMatchSource == AR4Settings::fmSrcInternal)
		matchFeatures();

	m_processedFeatures.clear();

	m_bFeatureQueueReady = true;
}

void PCIGMOD14ShapeMatching::evalGroundTruthError()
{
	if (m_pMatch->getNumMatchedIndices() > 0)
		evalGroundTruthErrorInternal(m_pMatch, 0);
	else
	{
		const mpcard numClusters = m_partialMatchClusters.size();

		mpcard i;

		SGRelativeTimeAnimationNode* pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strClusterSequence));
		if (pPMSequenceNode == NULL || pPMSequenceNode->getNumChildNodes(getScene()->getRootState()) == 0)
		{
			//if there are no clusters yet, evaluate the partials
			pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strPartialSequence));
			if (pPMSequenceNode == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::evalGroundTruthError(): partial match list not available");
		}

		//create vector containing partial match point clouds
		vector<GenericSwapAllOutOfCorePointCloud*> partialMatchClouds;
		const mpcard numPartialMatches = (mpcard)pPMSequenceNode->getNumChildNodes(getScene()->getRootState());
		for (i = 0; i < numPartialMatches; i++)
		{
			SGObjectNode* pMatchNode = dynamic_cast<SGObjectNode*>(pPMSequenceNode->getChildNode(getScene()->getRootState(), i));
			if (pMatchNode == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::evalGroundTruthError(): unable to retrieve partial node");
			GenericSwapAllOutOfCorePointCloud* pSwap = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pMatchNode->getSceneObject());
			if (pSwap == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::evalGroundTruthError(): unable to retrieve partial match point cloud");
			partialMatchClouds.push_back(pSwap);
		}

		for (i = 0; i < numPartialMatches; i++)
		{
			AR4PartialMatch3f* pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
			pMatch->copyGeodesicAndTopologyData(m_pMatch);

			UnstructuredInCorePointCloud* pUPC = dynamic_cast<UnstructuredInCorePointCloud*>(partialMatchClouds[i]->fetchPC());
			if (pUPC == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::evalGroundTruthError(): partial match point cloud is not UnstructuredInCorePointCloud");
			pMatch->initThroughPointCloud(pUPC);

			debugOutput << "evaluating ground truth error for cluster " << i << "...\n";
			evalGroundTruthErrorInternal(pMatch, i);

			delete pMatch;
		}
	}

	ObjectViewsTable::update(this);
}

void PCIGMOD14ShapeMatching::collectBenchmarkError()
{
	string strFilename;

	const card32 numBench = settings->numSubfolders;

	card32 i;

	const card32 firstDirLen = (card32)settings->strFirstSubfolder.length();
	for (i = firstDirLen - 1; i >= 0; i--)
	{
		const char c = settings->strFirstSubfolder[i];
		if (!isdigit(c))
			break;
	}

	const card32 benchNumStart = i + 1;
	const card32 benchNumWidth = firstDirLen - benchNumStart;

	char* szNum = new char[benchNumWidth + 1];

	const card32 coverageFilenameLen = (card32)settings->strCoverageFilename.length();
	i = (card32)settings->strCoverageFilename.rfind('.');
	for (i--; i >= 0; i--)
	{
		const char c = settings->strCoverageFilename[i];
		if (!isdigit(c))
			break;
	}

	const card32 filenameNumStart = i + 1;

	const mpcard numErrThresh = AR4_DEFAULT_NUM_ERR_THRESH;
	vector<float32> globalCumulative(numErrThresh + 1, 0);
	vector<float32> errLevels(numErrThresh + 1);

	mpcard totalCount = 0;

	ofstream fileErrMeanGlobal((m_strDirSwapPath + "/benchmark_mean_err" + settings->strCoverageFilename.substr(filenameNumStart)).c_str());
	ofstream fileCoverageGlobal((m_strDirSwapPath + "/benchmark_coverage" + settings->strCoverageFilename.substr(filenameNumStart)).c_str());

	for (i = 1; i <= numBench; i++)
	{
		string strFolder = settings->strFirstSubfolder.substr(0, benchNumStart);
		strFolder += intToStr((int64)i, benchNumWidth);

		strFilename = m_strDirSwapPath + strFolder + "/" + settings->strCoverageFilename;

		ifstream fileCoverage(strFilename.c_str());

		if (!fileCoverage)
		{
			debugOutput << "could not open " << strFilename << "\n";
			continue;
		}

		card32 numIndices;
		float32 errMean, errStd, errMin, errMax, fracTargCover;
		fileCoverage >> numIndices;
		fileCoverage >> errMean;
		fileCoverage >> errStd;
		fileCoverage >> errMin;
		fileCoverage >> errMax;
		fileCoverage >> fracTargCover;

		fileCoverage.close();

		debugOutput << "summary for " << i << ": " << numIndices << " " << errMean << " " << errStd << " " << errMin << " " << errMax << "\n";

		fileErrMeanGlobal << i << " " << errMean << " " << errStd << "\n";
		fileCoverageGlobal << i << " " << fracTargCover << "\n";

		strFilename = m_strDirSwapPath + "/" + strFolder + "/" + settings->strCumulativeFilename;

		ifstream fileCumulative(strFilename.c_str());

		if (fileCumulative.bad())
		{
			debugOutput << "could not open " << strFilename << "\n";
			continue;
		}

		totalCount += (mpcard)numIndices;

		mpcard threshIdx = 0;
		float32 thresh, percent, fraction, count;
		while (fileCumulative.good() && !fileCumulative.eof())
		{
			fileCumulative >> thresh >> percent;
			if (fileCumulative.eof())
				break;

			fraction = percent * 0.01f;
			count = fraction * (float32)numIndices;

			globalCumulative[threshIdx] += count;
			errLevels[threshIdx] = thresh;

			threshIdx++;
		}

		if (fileCumulative.bad())
			throw EInvalidState("Error reading from cumulative distribution file");

		fileCumulative.close();
	}

	fileErrMeanGlobal.close();
	fileCoverageGlobal.close();

	ofstream fileGlobalCumulative((m_strDirSwapPath + "/benchmark_cumulativeErr" + settings->strCoverageFilename.substr(filenameNumStart)).c_str());

	for (i = 0; i < (card32)numErrThresh; i++)
	{
		globalCumulative[i] /= (float32)totalCount;

		fileGlobalCumulative << errLevels[i] << " " << (globalCumulative[i] * 100.f) << "\n";
	}

	fileGlobalCumulative.close();

	delete szNum;
}

void PCIGMOD14ShapeMatching::saveCorrespondences()
{
	if (m_pMatch == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::saveCorrespondences(): null partial match");

	//first for the points on the target surface
	UnstructuredInCorePointCloud* pMatchPC = m_pMatch->getMatchingCloud();

	SceneGraphNode* pResultFolder;
	SGObjectNode* pMatchNode;

	pResultFolder = getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder);
	if (pResultFolder == NULL)
		initSceneStructure();
	pResultFolder = getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder);
	if (pResultFolder == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::saveCorrespondences(): unable to create results folder, saving in root");

	pMatchNode = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/matching_points"));
	if (pMatchNode == NULL)
		pMatchNode = new SGObjectNode();

	pMatchNode->clearAndSetup(pMatchPC);
	pMatchNode->setName("matching_points");

	pResultFolder->addChildNode(pMatchNode);

	//and the points on the source surface
	pMatchPC = m_pMatch->getMatchingCloudOnSource();

	pMatchNode = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/source_points"));
	if (pMatchNode == NULL)
		pMatchNode = new SGObjectNode();

	pMatchNode->clearAndSetup(pMatchPC);
	pMatchNode->setName("source_points");

	pResultFolder->addChildNode(pMatchNode);

	debugOutput << "PCIGMOD14ShapeMatching::saveCorrespondences(): correspondences saved to scene -- don't forget to save the scene!\n";
}

void PCIGMOD14ShapeMatching::loadCorrespondences()
{
	warning("PCIGMOD14ShapeMatching::loadCorrespondences(): not implemented yet");
}

void PCIGMOD14ShapeMatching::colorMatchesByError()
{
	if (m_pMatch == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::colorMatchesByError(): null partial match");

	UnstructuredInCorePointCloud* pMatchPC = m_pMatch->getMatchingCloud();
	UnstructuredInCorePointCloud* pMatchPCSrc = m_pMatch->getMatchingCloudOnSource();
	mpcard i, k, n = pMatchPC->getNumPoints();
	Vector3f matchPos, gtVertPos, errColor;
	AAT colAAT = pMatchPC->getAAT("color");

	float distToTruth, errMax = 0.f, rcpErrMax;

	if (m_bErrorDirty)
	{
		if (m_pMatch->getNumMatchedIndices() != n)
			throw ERangeCheck("PCIGMOD14ShapeMatching::colorMatchesByError(): Match cloud has different number of points that number of matches");

		m_matchesError.resize(n);
		for (i = 0; i < n; i++)
		{
			k = m_pMatch->getMatchedSourceIndex(i);
			matchPos = m_pMatch->getMatch(k);
			gtVertPos = m_pTargetManifold->getDiscrPoint(k);

			distToTruth = m_pMatch->targetGeodesic(matchPos, gtVertPos);

			m_matchesError[i] = distToTruth;
			if (distToTruth > errMax)	errMax = distToTruth;
		}

		rcpErrMax = 1.f / errMax;
		for (i = 0; i < n; i++)
			m_matchesError[i] *= rcpErrMax;
	}
	else
	{
		for (i = 0; i < n; i++)
		{
			if (m_matchesError[i] > errMax)
				errMax = m_matchesError[i];
		}
		rcpErrMax = 1.f / errMax;
		for (i = 0; i < n; i++)
			m_matchesError[i] *= rcpErrMax;
	}

	for (i = 0; i < n; i++)
	{
		errColor = rainbowGradient3f(m_matchesError[i]);
		pMatchPC->getPointSet()->set3f(i, colAAT, errColor);
		pMatchPCSrc->getPointSet()->set3f(i, colAAT, errColor);
	}

	m_bErrorDirty = false;
	redrawScene();
}

void PCIGMOD14ShapeMatching::colorMatchesByStretch()
{
	if (m_pMatch == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::colorMatchesByStretch(): null partial match");

	UnstructuredInCorePointCloud* pMatchPC = m_pMatch->getMatchingCloud();
	UnstructuredInCorePointCloud* pMatchPCSrc = m_pMatch->getMatchingCloudOnSource();
	mpcard i, k, n = pMatchPC->getNumPoints();
	Vector3f stretchColor;
	AAT colAAT = pMatchPC->getAAT("color");

	float stretch, stretchMax = 0.f, rcpStretchMax;

	if (m_bStretchDirty)
	{
		if (m_pMatch->getNumMatchedIndices() != n)
			throw ERangeCheck("PCIGMOD14ShapeMatching::colorMatchesByStretch(): WTF? Match cloud has different number of points that number of matches");

		m_matchesStretch.resize(n);
		for (i = 0; i < n; i++)
		{
			k = m_pMatch->getMatchedSourceIndex(i);
			stretch = m_pMatch->getStretch(k);

			m_matchesStretch[i] = stretch;
			if (stretch > stretchMax)	stretchMax = stretch;
		}

		rcpStretchMax = 1.f / stretchMax;
		for (i = 0; i < n; i++)
			m_matchesStretch[i] *= rcpStretchMax;
	}

	for (i = 0; i < n; i++)
	{
		stretchColor = rainbowGradient3f(m_matchesStretch[i]);
		pMatchPC->getPointSet()->set3f(i, colAAT, stretchColor);
		pMatchPCSrc->getPointSet()->set3f(i, colAAT, stretchColor);
	}

	m_bStretchDirty = false;
	redrawScene();
}

void PCIGMOD14ShapeMatching::colorMatchesByPosition()
{
	if (m_pMatch == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::colorMatchesByPosition(): null partial match");

	UnstructuredInCorePointCloud* pMatchPC = m_pMatch->getMatchingCloud();
	UnstructuredInCorePointCloud* pMatchPCSrc = m_pMatch->getMatchingCloudOnSource();
	mpcard i, k, n = pMatchPC->getNumPoints();
	Vector3f matchPos, color;
	AAT colAAT = pMatchPC->getAAT("color");

	float colorFreq = (float)AR4_DEFAULT_POSITION_COLOR_FREQ;
	float colorPeriod = 1.f / colorFreq;

	if (m_pMatch->getNumMatchedIndices() != n)
		throw ERangeCheck("PCIGMOD14ShapeMatching::colorMatchesByPosition(): Match cloud has different number of points that number of matches");
	for (i = 0; i < n; i++)
	{
		k = m_pMatch->getMatchedSourceIndex(i);
		matchPos = m_pSourceManifold->getDiscrPoint(k);

		color = matchPos - m_pSourceManifold->getBoundingBox().getCenter();
		color[0] /= m_pSourceManifold->getBoundingBox().getSideLength(0);
		color[1] /= m_pSourceManifold->getBoundingBox().getSideLength(1);
		color[2] /= m_pSourceManifold->getBoundingBox().getSideLength(2);
		color[0] += 0.5f;
		color[1] += 0.5f;
		color[2] += 0.5f;
		color[0] = fmod(color[0], colorPeriod) * colorFreq;
		color[1] = fmod(color[1], colorPeriod) * colorFreq;
		color[2] = fmod(color[2], colorPeriod) * colorFreq;

		pMatchPC->getPointSet()->set3f(i, colAAT, color);
		pMatchPCSrc->getPointSet()->set3f(i, colAAT, color);
	}

	redrawScene();
}

void PCIGMOD14ShapeMatching::interpolateMatching()
{
	X4_TIMER_START(interpolateMatching);

	SGRelativeTimeAnimationNode* pPMSequenceNode = NULL;
	SGObjectNode* pMatchNode = NULL;
	GenericSwapAllOutOfCorePointCloud* pSwap = NULL;
	UnstructuredInCorePointCloud* pMatchPC = NULL;
	string inputNodeName = "";

	initMatching();

	PointCloud* pSelCloud = getSingleSelectedPointCloudFromScene();
	if (pSelCloud != NULL)
	{
		pMatchPC = dynamic_cast<UnstructuredInCorePointCloud*>(pSelCloud);
		if (pMatchPC == NULL)
			throw EInvalidState("match cloud must be unstructured in-core point cloud");
	}
	else
	{
		pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strClusterSequence));
		if (pPMSequenceNode != NULL && pPMSequenceNode->getNumChildNodes(getScene()->getRootState()) > 0)
		{
			//we assume integer time steps for now
			const mpcard currentFrame = (mpcard)getScene()->getRootState()->dynamicState->time;

			pMatchNode = dynamic_cast<SGObjectNode*>(pPMSequenceNode->getChildNode(getScene()->getRootState(), currentFrame));
			if (pMatchNode == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial node");
			inputNodeName = pMatchNode->getName();

			pSwap = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pMatchNode->getSceneObject());
			if (pSwap == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial match point cloud");

			pMatchPC = dynamic_cast<UnstructuredInCorePointCloud*>(pSwap->fetchPC());
			if (pMatchPC == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial match point cloud");
		}
		else
		{
			pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strPartialSequence));
			if (pPMSequenceNode != NULL && pPMSequenceNode->getNumChildNodes(getScene()->getRootState()) > 0)
			{
				//we assume integer time steps for now
				const mpcard currentFrame = (mpcard)getScene()->getRootState()->dynamicState->time;

				pMatchNode = dynamic_cast<SGObjectNode*>(pPMSequenceNode->getChildNode(getScene()->getRootState(), currentFrame));
				if (pMatchNode == NULL)
					throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial node");
				inputNodeName = pMatchNode->getName();

				pSwap = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pMatchNode->getSceneObject());
				if (pSwap == NULL)
					throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial match point cloud");

				pMatchPC = dynamic_cast<UnstructuredInCorePointCloud*>(pSwap->fetchPC());
				if (pMatchPC == NULL)
					throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial match point cloud");
			}
			else
			{
				if (m_pMatch->getNumMatchedIndices() == 0)
					throw EInvalidState("PCIARShapeMatching::colorSurfaceByCorrespondence(): no correspondences with which to color surface");

				const mpcard numStart = m_pMatch->getNumMatchedIndices();
				debugOutput << "starting from " << numStart << " matches\n";

				AR4MatchInterpolator3f* pInterp = new AR4MatchInterpolator3f(m_pSourceManifold, m_pTargetManifold);
				pInterp->setGrowingMode((AR4GrowingMatcher3f::EGrowingMode)m_growingMode);
				pInterp->setDebugOutput(false);
				m_pGlobalBundle->resetNumMatchesLastRun();
				for (mpcard iter = 0; iter < settings->numInterpIters; iter++)
				{
					pInterp->improveMatch(m_pMatch);
					if (m_pMatch->getNumMatchedIndices() > 2 * m_pGlobalBundle->getNumMatchesLastRun())
					{
						m_pGlobalBundle->setStepScale(m_pMatch->computeAvgSourceEdgeLength() * 0.5f);
						m_pGlobalBundle->improveMatch(m_pMatch);
					}
				}
				m_pGlobalBundle->setStepScale(m_pMatch->computeAvgSourceEdgeLength() * 0.5f);
				m_pGlobalBundle->improveMatch(m_pMatch);

				const mpcard numDense = m_pMatch->getNumMatchedIndices();
				debugOutput << "now " << numDense << " matches\n";

				pInterp->reportProfiling();
			}
		}
	}

	if (pMatchPC != NULL)
	{
		AR4PartialMatch3f* pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
		pMatch->copyGeodesicAndTopologyData(m_pMatch);
		pMatch->setStretchThreshold(settings->stretchThreshold);
		pMatch->initThroughPointCloud(pMatchPC);
		pMatch->checkForNewGeodesicConnections();
		pMatch->validatePaths();
		pMatch->finalizeMatchCohort();

		const mpcard numStart = pMatch->getNumMatchedIndices();
		debugOutput << "starting from " << numStart << " matches\n";

		AR4MatchInterpolator3f* pInterp = new AR4MatchInterpolator3f(m_pSourceManifold, m_pTargetManifold);
		pInterp->setGrowingMode((AR4GrowingMatcher3f::EGrowingMode)m_growingMode);
		pInterp->setDebugOutput(false);
		m_pGlobalBundle->resetNumMatchesLastRun();
		mpcard numMatchesLast = numStart;
		for (mpcard iter = 0; iter < settings->numInterpIters; iter++)
		{
			pInterp->improveMatch(pMatch);
			if (pMatch->getNumMatchedIndices() == numMatchesLast)
				break;
			numMatchesLast = pMatch->getNumMatchedIndices();
		}

		const mpcard numDense = pMatch->getNumMatchedIndices();
		debugOutput << "now " << numDense << " matches\n";

		pInterp->reportProfiling();

		if (numDense > numStart)
		{
			string strDenseName;
			if (inputNodeName.length() > 0)
				strDenseName = inputNodeName + ".denser";
			else
				strDenseName = "dense_match";

			UnstructuredInCorePointCloud* pMatchDensePC = pMatch->exportAsCompactPointCloud();

			if (pPMSequenceNode != NULL)
			{
				string strSwapName = m_strDirSwapPath + strDenseName;
				pSwap = new GenericSwapAllOutOfCorePointCloud();
				pSwap->clearAndSetup(pMatchDensePC, strDenseName);
				pMatchNode = new SGObjectNode(pSwap);
				pMatchNode->setName(strDenseName);
				pPMSequenceNode->addChildNode(pMatchNode);
			}
			else
			{
				pMatchNode = new SGObjectNode(pMatchDensePC);
				pMatchNode->setName(strDenseName);

				SGListNode* pResultFolder = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder));
				pResultFolder->addChildNode(pMatchNode);
			}
		}

		delete pMatch;
	}

	X4_TIMER_STOP(interpolateMatching);
}

void PCIGMOD14ShapeMatching::upsamplePointClouds()
{
	const mpcard upsample_factor = 100;
	const float32 tangent_plane_extent = m_avgDenseEdgeLength * 1.5f;

	mpcard i, j;
	SceneGraphNode* pNode;
	SGObjectNode* pHighResNode;
	UnstructuredInCorePointCloud* pHighResSource;
	UnstructuredInCorePointCloud* pHighResTarget;

	//retrieve high-res source point cloud from scene
	pNode = getSceneGraphNodeByName(getScene(), "root/source_highres");
	if (pNode != NULL)
	{
		warning("root/source_highres already exists");
	}
	else
	{
		pHighResSource = new UnstructuredInCorePointCloud();
		pHighResSource->clearAndSetup(0);

		AAT hrColAAT = pHighResSource->getAAT("color");
		AAT hrPosAAT = pHighResSource->getAAT("position");
		AAT hrNormAAT = pHighResSource->getAAT("normal");
		AAT hrFlagsAAT = pHighResSource->getAAT("flags");

		const card32 posoff = hrPosAAT.getOffset();
		const card32 normoff = hrNormAAT.getOffset();
		const card32 coloff = hrColAAT.getOffset();
		const card32 flagsoff = hrFlagsAAT.getOffset();

		char* pbuffer = new char[pHighResSource->getDescr()->getSize()];

		const mpcard numSource = m_pSourceManifold->getNumDiscrPoints();
		for (i = 0; i < numSource; i++)
		{
			const Vector3f origPoint = m_pSourceManifold->getDiscrPoint(i);
			Vector3f projPoint, normal;
			StaticMatrix<float32, 2, 3> tangSpace;

			m_pSourceManifold->project(origPoint, projPoint, normal, tangSpace);

			for (j = 0; j < upsample_factor; j++)
			{
				const Vector3f tangPoint = tangSpace[0] * (rnd01() - 0.5f) * tangent_plane_extent + tangSpace[1] * (rnd01() - 0.5f) * tangent_plane_extent;
				m_pSourceManifold->project(origPoint + tangPoint, projPoint, normal);

				memcpy(pbuffer + posoff, &projPoint, sizeof(Vector3f));
				memcpy(pbuffer + normoff, &normal, sizeof(Vector3f));
				memset(pbuffer + coloff, 0, sizeof(Vector3f));
				memset(pbuffer + flagsoff, 0, sizeof(card32));

				pHighResSource->getPointSet()->addPoint(pbuffer);
			}
		}

		delete [] pbuffer;

		addPointCloud(getScene(), pHighResSource, "source_highres");
	}

	//retrieve high-res target point cloud from scene
	pNode = getSceneGraphNodeByName(getScene(), "root/target_highres");
	if (pNode != NULL)
	{
		warning("root/target_highres already exists");
	}
	else
	{
		pHighResTarget = new UnstructuredInCorePointCloud();
		pHighResTarget->clearAndSetup(0);

		AAT hrColAAT = pHighResTarget->getAAT("color");
		AAT hrPosAAT = pHighResTarget->getAAT("position");
		AAT hrNormAAT = pHighResTarget->getAAT("normal");
		AAT hrFlagsAAT = pHighResTarget->getAAT("flags");

		const card32 posoff = hrPosAAT.getOffset();
		const card32 normoff = hrNormAAT.getOffset();
		const card32 coloff = hrColAAT.getOffset();
		const card32 flagsoff = hrFlagsAAT.getOffset();

		char* pbuffer = new char[pHighResTarget->getDescr()->getSize()];

		const mpcard numTarget = m_pTargetManifold->getNumDiscrPoints();
		for (i = 0; i < numTarget; i++)
		{
			const Vector3f origPoint = m_pTargetManifold->getDiscrPoint(i);
			Vector3f projPoint, normal;
			StaticMatrix<float32, 2, 3> tangSpace;

			m_pTargetManifold->project(origPoint, projPoint, normal, tangSpace);

			for (j = 0; j < upsample_factor; j++)
			{
				const Vector3f tangPoint = tangSpace[0] * (rnd01() - 0.5f) * tangent_plane_extent + tangSpace[1] * (rnd01() - 0.5f) * tangent_plane_extent;
				m_pTargetManifold->project(origPoint + tangPoint, projPoint, normal);

				memcpy(pbuffer + posoff, &projPoint, sizeof(Vector3f));
				memcpy(pbuffer + normoff, &normal, sizeof(Vector3f));
				memset(pbuffer + coloff, 0, sizeof(Vector3f));
				memset(pbuffer + flagsoff, 0, sizeof(card32));

				pHighResTarget->getPointSet()->addPoint(pbuffer);
			}
		}

		delete [] pbuffer;

		addPointCloud(getScene(), pHighResTarget, "target_highres");
	}
}

void PCIGMOD14ShapeMatching::colorSourceByCheckerVoxels()
{
	const BoundingBox3f bbox = m_pSourceManifold->getBoundingBox();
	const mpcard numBins = settings->numCheckerBins;
	const float32 binSize = bbox.getMaxSideLength() / (float32)numBins;
	const float32 binFreq = 1.f / binSize;
	const Vector3f origin = bbox.lowerCorner - makeVector3f(binSize * 0.5f, binSize * 0.5f, binSize * 0.5f);

	const Vector3f color1 = settings->checkerColor1;
	const Vector3f color2 = settings->checkerColor2;

	const mpcard numSourcePoints = m_pSourceManifold->getNumDiscrPoints();

	mpcard i, j;

	AAT sourceColAAT = m_pSourceManifold->getPointCloud()->getAAT("color");

	for (i = 0; i < numSourcePoints; i++)
	{
		const Vector3f sourcePoint = m_pSourceManifold->getDiscrPoint(i);
		Vector3u sourceBin;

		sourceBin[0] = (card32)((sourcePoint[0] - origin[0]) * binFreq);
		sourceBin[1] = (card32)((sourcePoint[1] - origin[1]) * binFreq);
		sourceBin[2] = (card32)((sourcePoint[2] - origin[2]) * binFreq);

		if ((sourceBin[2] & 1) == 0)
		{
			if ((sourceBin[1] & 1) == 0)
			{
				if ((sourceBin[0] & 1) == 0)
					m_pSourceManifold->getPointSet()->set3f(i, sourceColAAT, color1);//matchColors[i] = color1;
				else
					m_pSourceManifold->getPointSet()->set3f(i, sourceColAAT, color2);//matchColors[i] = color2;
			}
			else
			{
				if ((sourceBin[0] & 1) == 0)
					m_pSourceManifold->getPointSet()->set3f(i, sourceColAAT, color2);//matchColors[i] = color2;
				else
					m_pSourceManifold->getPointSet()->set3f(i, sourceColAAT, color1);//matchColors[i] = color1;
			}
		}
		else
		{
			if ((sourceBin[1] & 1) == 0)
			{
				if ((sourceBin[0] & 1) == 0)
					m_pSourceManifold->getPointSet()->set3f(i, sourceColAAT, color2);//matchColors[i] = color2;
				else
					m_pSourceManifold->getPointSet()->set3f(i, sourceColAAT, color1);//matchColors[i] = color1;
			}
			else
			{
				if ((sourceBin[0] & 1) == 0)
					m_pSourceManifold->getPointSet()->set3f(i, sourceColAAT, color1);//matchColors[i] = color1;
				else
					m_pSourceManifold->getPointSet()->set3f(i, sourceColAAT, color2);//matchColors[i] = color2;
			}
		}
	}

	m_pSourceManifold->getPointCloud()->deleteDelOnWriteAttachments();
	redrawScene();
}

void PCIGMOD14ShapeMatching::colorTargetByCorrespondence()
{
	SGRelativeTimeAnimationNode* pPMSequenceNode = NULL;
	SGObjectNode* pMatchNode = NULL;
	GenericSwapAllOutOfCorePointCloud* pSwap = NULL;
	UnstructuredInCorePointCloud* pMatchPC = NULL;
	AR4PartialMatch3f* pMatch = NULL;

	PointCloud* pSelCloud = getSingleSelectedPointCloudFromScene();
	if (pSelCloud != NULL)
	{
		pMatchPC = dynamic_cast<UnstructuredInCorePointCloud*>(pSelCloud);
		if (pMatchPC == NULL)
			throw EInvalidState("match cloud must be unstructured in-core point cloud");
	}
	else
	{
		pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strClusterSequence));
		if (pPMSequenceNode != NULL && pPMSequenceNode->getNumChildNodes(getScene()->getRootState()) > 0)
		{
			//we assume integer time steps for now
			const mpcard currentFrame = (mpcard)getScene()->getRootState()->dynamicState->time;

			pMatchNode = dynamic_cast<SGObjectNode*>(pPMSequenceNode->getChildNode(getScene()->getRootState(), currentFrame));
			if (pMatchNode == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial node");
			pSwap = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pMatchNode->getSceneObject());
			if (pSwap == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial match point cloud");

			pMatchPC = dynamic_cast<UnstructuredInCorePointCloud*>(pSwap->fetchPC());
			if (pMatchPC == NULL)
				throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial match point cloud");
		}
		else
		{
			pPMSequenceNode = dynamic_cast<SGRelativeTimeAnimationNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder + "/" + m_strPartialSequence));
			if (pPMSequenceNode != NULL && pPMSequenceNode->getNumChildNodes(getScene()->getRootState()) > 0)
			{
				//we assume integer time steps for now
				const mpcard currentFrame = (mpcard)getScene()->getRootState()->dynamicState->time;

				pMatchNode = dynamic_cast<SGObjectNode*>(pPMSequenceNode->getChildNode(getScene()->getRootState(), currentFrame));
				if (pMatchNode == NULL)
					throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial node");
				pSwap = dynamic_cast<GenericSwapAllOutOfCorePointCloud*>(pMatchNode->getSceneObject());
				if (pSwap == NULL)
					throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial match point cloud");

				pMatchPC = dynamic_cast<UnstructuredInCorePointCloud*>(pSwap->fetchPC());
				if (pMatchPC == NULL)
					throw EInvalidState("PCIGMOD14ShapeMatching::colorSurfaceByCorrespondence(): unable to retrieve partial match point cloud");
			}
			else
			{
				if (m_pMatch->getNumMatchedIndices() == 0)
					throw EInvalidState("PCIARShapeMatching::colorSurfaceByCorrespondence(): no correspondences with which to color surface");

				pMatchPC = m_pMatch->exportAsCompactPointCloud();
				pMatch = m_pMatch;

				string strId = "root/" + m_strResultFolder + "/matches";
				SceneGraphNode* pDummyNode = getSceneGraphNodeByName(getScene(), strId);
				if (pDummyNode != NULL)
				{
					pMatchNode = dynamic_cast<SGObjectNode*>(pDummyNode);
					if (pMatchNode == NULL)
					{
						deleteSceneGraphNode(getScene(), strId);
						pMatchNode = new SGObjectNode(pMatchPC);
						pMatchNode->setName("matches");
						SGListNode* pResults = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder));
						pResults->addChildNode(pMatchNode);
					}
					else
						pMatchNode->clearAndSetup(pMatchPC);
				}
				else
				{
					pMatchNode = new SGObjectNode(pMatchPC);
					pMatchNode->setName("matches");
					SGListNode* pResults = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(getScene(), "root/" + m_strResultFolder));
					pResults->addChildNode(pMatchNode);
				}
			}
		}
	}

	X4_TIMER_START(transferColor);

	if (settings->bHighResViz)
	{
		if (pMatch == NULL)
		{
			pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
			pMatch->copyGeodesicAndTopologyData(m_pMatch);
			pMatch->initThroughPointCloud(pMatchPC);
		}

		debugOutput << "transferring high-res color by correspondence\n";
		colorTargetByCorrespondenceInternalHighRes(pMatch);

		if (pMatch != m_pMatch)
			delete pMatch;
	}
	else
		colorTargetByCorrespondenceInternal(pMatchPC);

	X4_TIMER_STOP(transferColor);

	redrawScene();
}

void PCIGMOD14ShapeMatching::colorTargetByCorrespondenceInternal(UnstructuredInCorePointCloud* pMatchPC)
{
	const mpcard numSourcePoints = m_pSourceManifold->getNumDiscrPoints();
	const mpcard numTargetPoints = m_pTargetManifold->getNumDiscrPoints();

	mpcard i, j;

	AAT sourceColAAT = m_pSourceManifold->getPointCloud()->getAAT("color");
	AAT targetColAAT = m_pTargetManifold->getPointCloud()->getAAT("color");

	//first reset all colors
	for (i = 0; i < numTargetPoints; i++)
		m_pTargetManifold->getPointSet()->set3f(i, targetColAAT, makeVector3f(0.f, 0.f, 0.f));

	AAT indexAAT = pMatchPC->getAAT(AR4PartialMatch3f::getMatchAttNameIndex());
	AAT targetAAT = pMatchPC->getAAT(AR4PartialMatch3f::getMatchAttNameTargetPos());

	const mpcard numMatches = pMatchPC->getNumPoints();

	vector<float32> targetWeights(numTargetPoints, 0.f);
	vector<Vector3f> targetColors(numTargetPoints, NULL_VECTOR3F);

	const float32 matchSampleSpacing = m_pMatch->getAvgSourceMatchLevelEdgeWeight();
	const float32 kernelSigma = matchSampleSpacing * settings->texKernelFactor;
	const float32 radius = matchSampleSpacing * 2.f;
	const float32 sourceRadius = matchSampleSpacing * 0.5f;
	const float32 rcp2Sigma2 = 1.f / (2.f * kernelSigma*kernelSigma);

	//transfer color from source, compute weighted sum
	for (i = 0; i < numMatches; i++)
	{
		const card32 index = pMatchPC->getPointSet()->get1u(i, indexAAT);
		const Vector3f sourcePoint = m_pSourceManifold->getDiscrPoint(index);
		const Vector3f targetPoint = pMatchPC->getPointSet()->get3f(i, targetAAT);
		const Vector3f sourceColor = m_pSourceManifold->getPointSet()->get3f(index, sourceColAAT);

		vector<mpcard> neighbors;

		m_pTargetManifold->getSphericalNeighborsNG(targetPoint, radius, neighbors);

		for (j = 0; j < neighbors.size(); j++)
		{
			const mpcard nbr = neighbors[j];
			const Vector3f pos = m_pTargetManifold->getDiscrPoint(nbr);
			const float32 weight = exp(-normQuad(pos - targetPoint) * rcp2Sigma2);

			targetColors[nbr] += sourceColor * weight;
			targetWeights[nbr] += weight;
		}
	}

	//set target colors to weighted average
	for (i = 0; i < numTargetPoints; i++)
	{
		if (targetWeights[i] > 1e-5f)
			targetColors[i] /= targetWeights[i];
		m_pTargetManifold->getPointSet()->set3f(i, targetColAAT, targetColors[i]);
	}

	m_pSourceManifold->getPointCloud()->deleteDelOnWriteAttachments();
	m_pTargetManifold->getPointCloud()->deleteDelOnWriteAttachments();
}

void PCIGMOD14ShapeMatching::colorTargetByCorrespondenceInternalHighRes(AR4PartialMatch3f* pMatch)
{
	const mpcard numSourcePoints = m_pSourceManifold->getNumDiscrPoints();
	const mpcard numTargetPoints = m_pTargetManifold->getNumDiscrPoints();

	mpcard i, j, k;

	//retrieve high-res source point cloud from scene
	SceneGraphNode* pNode = getSceneGraphNodeByName(getScene(), "root/source_highres");
	if (pNode == NULL)
		throw EInvalidState("no high-res source point cloud to transfer color from");

	SGObjectNode* pHighResNode = dynamic_cast<SGObjectNode*>(pNode);
	if (pHighResNode == NULL)
		throw EInvalidState("root/source_highres is not an object node");

	UnstructuredInCorePointCloud* pHighResSource = dynamic_cast<UnstructuredInCorePointCloud*>(pHighResNode->getSceneObject());
	if (pHighResSource == NULL)
		throw EInvalidState("root/source_highres does not contain an unstructured in-core point cloud");

	//retrieve high-res target point cloud from scene
	pNode = getSceneGraphNodeByName(getScene(), "root/target_highres");
	if (pNode == NULL)
		throw EInvalidState("no high-res target point cloud to transfer color from");

	pHighResNode = dynamic_cast<SGObjectNode*>(pNode);
	if (pHighResNode == NULL)
		throw EInvalidState("root/target_highres is not an object node");

	UnstructuredInCorePointCloud* pHighResTarget = dynamic_cast<UnstructuredInCorePointCloud*>(pHighResNode->getSceneObject());
	if (pHighResTarget == NULL)
		throw EInvalidState("root/target_highres does not contain an unstructured in-core point cloud");

	AAT sourceColAAT = pHighResSource->getAAT("color");
	AAT hrPosAAT = pHighResSource->getAAT("position");

	AAT targetColAAT = pHighResTarget->getAAT("color");
	AAT hrTargPosAAT = pHighResTarget->getAAT("position");

	//detect boundaries on high-res point clouds
	PCCBoundaryDetector bd;
	bd.detectBoundaries(pHighResTarget);

	AAT boundaryAAT = pHighResTarget->getAAT("boundary");

	//compute transformation from high-res pcs to source and target (in case high-res ones are not produced by sampling source and target)
	const BoundingBox3f sourceBox = m_pSourceManifold->getBoundingBox();
	const BoundingBox3f targetBox = m_pTargetManifold->getBoundingBox();

	const float sourceMaxSide = sourceBox.getMaxSideLength();
	const float targetMaxSide = targetBox.getMaxSideLength();

	const BoundingBox3f highResSourceBox = pHighResSource->getPointSet()->getBoundingBox();
	const BoundingBox3f highResTargetBox = pHighResTarget->getPointSet()->getBoundingBox();

	float sourceScale = 1.f, targetScale = 1.f;
	Vector3f sourceShift = NULL_VECTOR3F, targetShift = NULL_VECTOR3F;

	//scale and shift from high-res target to target
	targetScale = targetMaxSide / highResTargetBox.getMaxSideLength();
	targetShift = targetBox.getCenter() - highResTargetBox.getCenter() * targetScale;

	//scale and shift from source to high-res source
	sourceScale = highResSourceBox.getMaxSideLength() / sourceMaxSide;
	sourceShift = highResSourceBox.getCenter() - sourceBox.getCenter() * sourceScale;

	debugOutput << "source scale: " << sourceScale << "\n";
	debugOutput << "source shift: " << sourceShift << "\n";
	debugOutput << "target scale: " << targetScale << "\n";
	debugOutput << "target shift: " << targetShift << "\n";

	//create point cloud to represent inverse mapping
	UnstructuredInCorePointCloud* pInvMatch = new UnstructuredInCorePointCloud();
	VertexDescriptor vd;
	vd.pushAttrib(VAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(VAD("sourceIndex", 1, VAD::DATA_FORMAT_CARD32));

	const mpcard numMatches = pMatch->getNumMatchedIndices();

	pInvMatch->clearAndSetup(&vd, numMatches);
	AAT matchPosAAT = pInvMatch->getAAT("position");
	AAT matchIdxAAT = pInvMatch->getAAT("sourceIndex");

	for (i = 0; i < numMatches; i++)
	{
		const mpcard idx = pMatch->getMatchedSourceIndex(i);
		pInvMatch->getPointSet()->set3f(i, matchPosAAT, pMatch->getMatch(idx));
		pInvMatch->getPointSet()->set1u(i, matchIdxAAT, idx);
	}

	//create knn for inverse mapping
	AttachedIndexedOctree* pMatchOctree = PointSetKNNQuery::createOctree(pInvMatch->getPointSet());
	PointSetKNNQuery* pMatchKNN = new PointSetKNNQuery(pInvMatch->getPointSet(), pMatchOctree, NULL_VECTOR3F, matchPosAAT);

	AttachedIndexedOctree* pHRSourceOctree = PointSetKNNQuery::createOctree(pHighResSource->getPointSet());
	PointSetKNNQuery* pHRSourceKNN = new PointSetKNNQuery(pHighResSource->getPointSet(), pHRSourceOctree, NULL_VECTOR3F, hrPosAAT);

	AttachedIndexedOctree* pHRTargetOctree = PointSetKNNQuery::createOctree(pHighResTarget->getPointSet());
	PointSetKNNQuery* pHRTargetKNN = new PointSetKNNQuery(pHighResTarget->getPointSet(), pHRTargetOctree, NULL_VECTOR3F, hrTargPosAAT);

	vector<float32> targetWeights(numTargetPoints, 0.f);
	vector<Vector3f> targetColors(numTargetPoints, NULL_VECTOR3F);

	const float32 matchSampleSpacing = m_pMatch->getAvgSourceMatchLevelEdgeWeight();
	const float32 kernelSigma = matchSampleSpacing * settings->texKernelFactor;
	const float32 radius = matchSampleSpacing * 1.f;//2.f;
	const float32 sourceRadius = matchSampleSpacing * 0.5f;
	const float32 rcp2Sigma2 = 1.f / (2.f * kernelSigma*kernelSigma);

	const mpcard numHRSourcePoints = pHighResSource->getNumPoints();
	const mpcard numHRTargetPoints = pHighResTarget->getNumPoints();

	//first set all target points to "not mapped to color"
	debugOutput << "resetting target color\n";
	debugOutput.flush();
	for (i = 0; i < numHRTargetPoints; i++)
	{
		pHighResTarget->getPointSet()->set3f(i, targetColAAT, makeVector3f(1.f, 0.f, 0.f));
	}

	//compute PCA of source to get 2D checkerboard pattern
	debugOutput << "compute PCA of source cloud for alignment\n";
	Vector3f sourceCentroid, sourceScaling;
	Matrix3f sourceRotation;
	computeSourceAlignment(sourceCentroid, sourceRotation, sourceScaling);

	debugOutput << sourceCentroid << "\n";
	debugOutput << sourceRotation[0] << "\n";
	debugOutput << sourceRotation[1] << "\n";
	debugOutput << sourceRotation[2] << "\n";
	debugOutput << sourceScaling << "\n";

	const mpcard numBins = settings->numCheckerBins;
	const float32 stdDevCount = 3.f;
	const float32 scaleu = 1.f / sourceScaling[0];
	const float32 scalev = 1.f / sourceScaling[1];
	const float32 binSize = 2.f * stdDevCount * sourceScaling[0] / (float32)numBins;
	const float32 binFreq = 1.f / binSize;
	const float32 u_off = -stdDevCount * sourceScaling[0];
	const float32 v_off = -stdDevCount * sourceScaling[1];

	debugOutput << "bin size: " << binSize << "\nbin frequency: " << binFreq << "\n";
	debugOutput << "offset: " << u_off << " " << v_off << "\n";

	for (i = 0; i < numHRSourcePoints; i++)
	{
		const Vector3f hrpos = pHighResSource->getPointSet()->get3f(i, hrPosAAT);

		const Vector3f hrposShifted = hrpos - sourceCentroid;
		const float32 u = (hrposShifted * sourceRotation[0]);
		const float32 v = (hrposShifted * sourceRotation[1]);

		const card32 ubin = (card32)(fabs(u - u_off) * binFreq);
		const card32 vbin = (card32)(fabs(v - v_off) * binFreq);

		const Vector3f srccol = pHighResSource->getPointSet()->get3f(i, sourceColAAT);
		if ((vbin & 1) == 0)
		{
			if ((ubin & 1) == 0)
				pHighResSource->getPointSet()->set3f(i, sourceColAAT, srccol * 0.5f);
		}
		else
		{
			if ((ubin & 1) != 0)
				pHighResSource->getPointSet()->set3f(i, sourceColAAT, srccol * 0.5f);
		}
	}

	vector<mpcard> sourceMatchIndices(numMatches), sourceVoronoi, targetVoronoi;
	vector<Vector3f> targetMatchPoints(numMatches), matchColor(numMatches);
	vector<Vector3f> sourceColor(numSourcePoints), targetColor(numTargetPoints);

	debugOutput << "number of matches: " << numMatches << "\n";
	for (i = 0; i < numMatches; i++)
	{
		sourceMatchIndices[i] = pMatch->getMatchedSourceIndex(i);
		targetMatchPoints[i] = pMatch->getMatch(sourceMatchIndices[i]);
	}

	pMatch->getSparseSourceGeodesics()->geodesicVoronoi(sourceMatchIndices, sourceVoronoi);
	pMatch->getSparseTargetGeodesics()->geodesicVoronoi(targetMatchPoints, targetVoronoi);

	for (i = 0; i < numMatches; i++)
	{
		card32 nearest;
		bool bfound;
		pHRSourceKNN->setTargetPoint(m_pSourceManifold->getDiscrPoint(sourceMatchIndices[i]));
		pHRSourceKNN->getNextNearestPointIndex(nearest, bfound);
		if (bfound)
		{
			matchColor[i] = pHighResSource->getPointSet()->get3f((mpcard)nearest, sourceColAAT);
		}
	}

	for (i = 0; i < numTargetPoints; i++)
	{
		targetColor[i] = matchColor[targetVoronoi[i]];
	}

	for (i = 0; i < numHRTargetPoints; i++)
	{
		const Vector3f hrpos = pHighResTarget->getPointSet()->get3f(i, hrTargPosAAT) * targetScale + targetShift;
		
		//project to get tangent plane
		Vector3f projPoint, projNorm;
		StaticMatrix<float32, 2, 3> tang;
		m_pTargetManifold->project(hrpos, projPoint, projNorm, tang);

		//points projected into tangent plane
		vector<Vector2f> tangPoints;

		//compute weights wrt to target matches
		float32 weightSum = 0.f;
		bool bfound;
		card32 next;
		pMatchKNN->setTargetPoint(hrpos);
		
		Vector3f mappedPoint = NULL_VECTOR3F;
		pMatchKNN->getNextNearestPointIndex(next, bfound);
		while (bfound)
		{
			const Vector3f matchPos = targetMatchPoints[next];
			const float32 dist = norm(hrpos - matchPos);
			if (dist > radius)
				break;

			tangPoints.push_back(makeVector2f(tang[0] * (matchPos - hrpos), tang[1] * (matchPos - hrpos)));

			const float32 weight = exp(-(dist*dist * rcp2Sigma2));

			mappedPoint += m_pSourceManifold->getDiscrPoint(sourceMatchIndices[next]) * weight;
			weightSum += weight;

			pMatchKNN->getNextNearestPointIndex(next, bfound);
		}

		if (weightSum > 0.f)
		{
			const mpcard numSlices = 24;
			const float32 dslice = (float32) (2.0 * M_PI / numSlices);
			const mpcard numTangPoints = tangPoints.size();
			float32 dpMaxMin = 2.f;

			//only check the angular distribution if point is not on a boundary
			if (pHighResTarget->getPointSet()->get1f(i, boundaryAAT) < 0.5f)
			{
				for (k = 0; k < numSlices; k++)
				{
					const float32 theta = dslice * (float32)k;
					const Vector2f sliceDir = makeVector2f(cos(theta), sin(theta));
					float32 dpMax = -2.f;
					for (j = 0; j < numTangPoints; j++)
					{
						const Vector2f tangDir = normalize(tangPoints[j]);
						const float32 dp = sliceDir * tangDir;
						if (dp > dpMax)
							dpMax = dp;
					}
					if (dpMax < dpMaxMin)
						dpMaxMin = dpMax;
				}
			}

			if (dpMaxMin >= 0.f)
			{
				mappedPoint /= weightSum;
				mappedPoint *= sourceScale;
				mappedPoint += sourceShift;

				pHRSourceKNN->setTargetPoint(mappedPoint);

				pHRSourceKNN->getNextNearestPointIndex(next, bfound);
				if (bfound)
				{
					const Vector3f mappedColor = pHighResSource->getPointSet()->get3f((mpcard)next, sourceColAAT);
					pHighResTarget->getPointSet()->set3f(i, targetColAAT, mappedColor);
				}
			}
		}
	}

	//clean-up
	m_pSourceManifold->getPointCloud()->deleteDelOnWriteAttachments();
	m_pTargetManifold->getPointCloud()->deleteDelOnWriteAttachments();
	pHighResSource->deleteDelOnWriteAttachments();
	pHighResTarget->deleteDelOnWriteAttachments();

	delete pMatchKNN;
	delete pMatchOctree;
	delete pInvMatch;
	delete pHRSourceOctree;
	delete pHRSourceKNN;
	delete pHRTargetOctree;
	delete pHRTargetKNN;
}

void PCIGMOD14ShapeMatching::computeSourceAlignment(Vector3f& centroid, Matrix3f& rotation, Vector3f& scaling)
{
	vector<Vector3f> sourcePoints;
	Matrix3f eigenvecs;
	Vector3f eigenvals;

	const mpcard numSourcePoints = m_pSourceManifold->getNumDiscrPoints();
	sourcePoints.resize(numSourcePoints);
	for (mpcard i = 0; i < numSourcePoints; i++)
		sourcePoints[i] = m_pSourceManifold->getDiscrPoint(i);

	X4::computePCA(sourcePoints, centroid, eigenvecs, eigenvals);

	rotation[0] = eigenvecs[2];
	rotation[1] = eigenvecs[1];
	rotation[2] = eigenvecs[0];
	scaling[0] = sqrt(eigenvals[2]);
	scaling[1] = sqrt(eigenvals[1]);
	scaling[2] = sqrt(eigenvals[0]);
}

void PCIGMOD14ShapeMatching::loadMatchesBIM()
{
	//according to Blended Intrinsic Maps documentation
	//correspondences are stored as follows
	//source index given by file line number; triangle index, barycentric coordinates

	const string strFilename = m_strDirSwapPath + settings->strCorrFile;
	
	const mpcard numSourcePoints = m_pSourceManifold->getNumDiscrPoints();

	mpcard i;

	vector<mpcard> triIndex(numSourcePoints);
	vector<Vector3f> baryCoords(numSourcePoints);

	debugOutput << "reading file\n";
	ifstream corrFile(strFilename.c_str());

	for (i = 0; i < numSourcePoints && corrFile.good(); i++)
	{
		mpcard tri;
		float bary1, bary2, bary3;
		corrFile >> tri;
		corrFile >> bary1;
		corrFile >> bary2;
		corrFile >> bary3;

		triIndex[i] = tri;
		baryCoords[i][0] = bary1;
		baryCoords[i][1] = bary2;
		baryCoords[i][2] = bary3;
	}

	if (corrFile.bad())
		throw EInvalidState("PCIGMOD14ShapeMatching::loadMatchesBIM(): error reading file " + strFilename);

	corrFile.close();

	UnstructuredInCoreTriangleMesh* pMesh = dynamic_cast<UnstructuredInCoreTriangleMesh*>(m_pTargetManifold->getPointCloud());
	if (pMesh == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::loadMatchesBIM(): error target is not a triangle mesh");

	PointSet* ptris = pMesh->getTriangles();
	AAT idxaat = ptris->getAAT("index");

	AR4PartialMatch3f* pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
	pMatch->copyGeodesicAndTopologyData(m_pMatch);

	debugOutput << "computing points from barycentric coords\n";
	for (i = 0; i < numSourcePoints; i++)
	{
		const mpcard tri = triIndex[i];
		const Vector3i vidx = ptris->get3i(tri, idxaat);
		const Vector3f barys = baryCoords[i];
		Vector3f targetPoint;
		targetPoint =	m_pTargetManifold->getDiscrPoint(vidx[0]) * barys[0] + 
						m_pTargetManifold->getDiscrPoint(vidx[1]) * barys[1] + 
						m_pTargetManifold->getDiscrPoint(vidx[2]) * barys[2];

		pMatch->setMatch(i, targetPoint);
	}

	UnstructuredInCorePointCloud* pMatchPC = pMatch->exportAsCompactPointCloud();

	debugOutput << "visualizing\n";
	//visualize
	if (settings->bHighResViz)
	{
		debugOutput << "transferring high-res color by correspondence\n";
		colorTargetByCorrespondenceInternalHighRes(pMatch);
	}
	else
		colorTargetByCorrespondenceInternal(pMatchPC);

	debugOutput << "maybe evaluating error\n";
	//optionally evaluate error
	if (settings->bEvalExternalCorrs)
		evalGroundTruthErrorInternal(pMatch, 9999);

	delete pMatch;
	delete pMatchPC;
}

void PCIGMOD14ShapeMatching::loadMatchesAurela()
{
	const string strFilename = m_strDirSwapPath + settings->strCorrFile;
	
	const mpcard numSourcePoints = m_pSourceManifold->getNumDiscrPoints();

	mpcard i;

	vector<mpcard> vertIndex(numSourcePoints);

	debugOutput << "reading file\n";
	ifstream corrFile(strFilename.c_str());

	for (i = 0; i < numSourcePoints && corrFile.good(); i++)
	{
		mpcard vert;
		corrFile >> vert;

		vertIndex[i] = vert;
	}

	if (corrFile.bad())
		throw EInvalidState("PCIGMOD14ShapeMatching::loadMatchesAurela(): error reading file " + strFilename);

	corrFile.close();

	AR4PartialMatch3f* pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
	pMatch->copyGeodesicAndTopologyData(m_pMatch);

	debugOutput << "setting matches\n";
	for (i = 0; i < numSourcePoints; i++)
	{
		const mpcard itarg = vertIndex[i];
		if (itarg == -1)	continue;

		const Vector3f targetPoint = m_pTargetManifold->getDiscrPoint(itarg);

		pMatch->setMatch(i, targetPoint);
	}

	debugOutput << "exporting point cloud\n";
	UnstructuredInCorePointCloud* pMatchPC = pMatch->exportAsCompactPointCloud();

	debugOutput << "visualizing\n";
	//visualize
	if (settings->bHighResViz)
	{
		debugOutput << "transferring high-res color by correspondence\n";
		colorTargetByCorrespondenceInternalHighRes(pMatch);
	}
	else
		colorTargetByCorrespondenceInternal(pMatchPC);

	debugOutput << "maybe evaluating error\n";
	//optionally evaluate error
	if (settings->bEvalExternalCorrs)
		evalGroundTruthErrorInternal(pMatch, 9998);

	delete pMatch;
	delete pMatchPC;
}

void PCIGMOD14ShapeMatching::loadMatchesAvinash()
{
	const string strFilename = m_strDirSwapPath + settings->strCorrFile;
	
	const mpcard numSourcePoints = m_pSourceManifold->getNumDiscrPoints();
	const mpcard numTargetPoints = m_pTargetManifold->getNumDiscrPoints();

	mpcard i;

	vector<mpcard> vertIndex(numSourcePoints);

	debugOutput << "reading file\n";
	ifstream corrFile(strFilename.c_str());

	mpcard numPtsFromFile, columns;
	corrFile >> numPtsFromFile;
	corrFile >> columns;


	for (i = 0; i < numTargetPoints && corrFile.good(); i++)
	{
		mpcard vertSource, vert;
		corrFile >> vert;
		corrFile >> vertSource;

		if (vertSource != -1 && vertSource < numSourcePoints)
			vertIndex[vertSource] = vert;
	}

	if (corrFile.bad())
		throw EInvalidState("PCIGMOD14ShapeMatching::loadMatchesAvinash(): error reading file " + strFilename);

	corrFile.close();

	AR4PartialMatch3f* pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
	pMatch->copyGeodesicAndTopologyData(m_pMatch);

	debugOutput << "setting matches\n";
	for (i = 0; i < numSourcePoints; i++)
	{
		const mpcard itarg = vertIndex[i];
		if (itarg == -1)	continue;

		const Vector3f targetPoint = m_pTargetManifold->getDiscrPoint(itarg);

		pMatch->setMatch(i, targetPoint);
	}

	UnstructuredInCorePointCloud* pMatchPC = pMatch->exportAsCompactPointCloud();

	debugOutput << "visualizing\n";
	//visualize
	if (settings->bHighResViz)
	{
		debugOutput << "transferring high-res color by correspondence\n";
		colorTargetByCorrespondenceInternalHighRes(pMatch);
	}
	else
		colorTargetByCorrespondenceInternal(pMatchPC);

	debugOutput << "maybe evaluating error\n";
	//optionally evaluate error
	if (settings->bEvalExternalCorrs)
		evalGroundTruthErrorInternal(pMatch, 9997);

	delete pMatch;
	delete pMatchPC;
}

void PCIGMOD14ShapeMatching::loadMatchesArt()
{
	const string strFilename = m_strDirSwapPath + settings->strCorrFile;
	
	const mpcard numSourcePoints = m_pSourceManifold->getNumDiscrPoints();
	const mpcard numTargetPoints = m_pTargetManifold->getNumDiscrPoints();

	const BoundingBox3f sourceBox = m_pSourceManifold->getBoundingBox();
	const BoundingBox3f targetBox = m_pTargetManifold->getBoundingBox();

	const float sourceMaxSide = sourceBox.getMaxSideLength();
	const float targetMaxSide = targetBox.getMaxSideLength();

	mpcard i, corrMax, corrIdx = 0;

	debugOutput << "reading file " << strFilename << "\n";
	ifstream corrFile(strFilename.c_str());

	corrFile >> corrMax;
	debugOutput << "maximum correspondence index " << corrMax << "\n";

	vector<Vector3f> sourcePoints, targetPoints;

	while (!corrFile.eof() && corrFile.good())
	{
		float x, y, z;

		corrFile >> corrIdx;
	
		corrFile >> x; corrFile >> y; corrFile >> z;
		sourcePoints.push_back(makeVector3f(x, y, z));

		corrFile >> x; corrFile >> y; corrFile >> z;
		targetPoints.push_back(makeVector3f(x, y, z));
	}

	const mpcard numCorrs = sourcePoints.size();
	if (corrFile.bad() || targetPoints.size() != numCorrs)
		warning("problem reading Art's correspondence file");

	debugOutput << "number of correspondences: " << numCorrs << "\n";

	SGObjectNode* pObjNode = NULL;
	UnstructuredInCorePointCloud* pSourceScaled = NULL;
	UnstructuredInCorePointCloud* pTargetScaled = NULL;

	BoundingBox3f sourceBoxScaled, targetBoxScaled;

	float sourceScale = 1.f, targetScale = 1.f;
	Vector3f sourceShift = NULL_VECTOR3F, targetShift = NULL_VECTOR3F;

	SceneGraphNode* pNode = getSceneGraphNodeByName(getScene(), "root/source_scaled");
	if (pNode != NULL)
	{
		pObjNode = dynamic_cast<SGObjectNode*>(pNode);
		if (pObjNode == NULL)
			throw EInvalidState("node is not an object node");

		pSourceScaled = dynamic_cast<UnstructuredInCorePointCloud*>(pObjNode->getSceneObject());
		if (pSourceScaled == NULL)
			throw EInvalidState("problem with scaled source surface");

		sourceBoxScaled = pSourceScaled->getPointSet()->getBoundingBox();

		sourceScale = sourceMaxSide / sourceBoxScaled.getMaxSideLength();
		sourceShift = sourceBox.getCenter() - sourceBoxScaled.getCenter() * sourceScale;
	}

	debugOutput << "source scale: " << sourceScale << "\n";
	debugOutput << "source shift: " << sourceShift << "\n";

	pNode = getSceneGraphNodeByName(getScene(), "root/target_scaled");
	if (pNode != NULL)
	{
		pObjNode = dynamic_cast<SGObjectNode*>(pNode);
		if (pObjNode == NULL)
			throw EInvalidState("node is not an object node");

		pSourceScaled = dynamic_cast<UnstructuredInCorePointCloud*>(pObjNode->getSceneObject());
		if (pSourceScaled == NULL)
			throw EInvalidState("problem with scaled target surface");

		targetBoxScaled = pSourceScaled->getPointSet()->getBoundingBox();

		targetScale = targetMaxSide / targetBoxScaled.getMaxSideLength();
		targetShift = targetBox.getCenter() - targetBoxScaled.getCenter() * targetScale;
	}

	debugOutput << "target scale: " << targetScale << "\n";
	debugOutput << "target shift: " << targetShift << "\n";

	for (i = 0; i < numCorrs; i++)
	{
		sourcePoints[i] *= sourceScale;
		sourcePoints[i] += sourceShift;

		targetPoints[i] *= targetScale;
		targetPoints[i] += targetShift;
	}



	AR4PartialMatch3f* pMatch = new AR4PartialMatch3f(m_pSourceManifold, m_pTargetManifold);
	pMatch->copyGeodesicAndTopologyData(m_pMatch);

	debugOutput << "setting matches\n";
	for (i = 0; i < numCorrs; i++)
	{
		vector<mpcard> neighbors;

		m_pSourceManifold->getKNearestNeighborsEucl(sourcePoints[i], 1, neighbors);
		mpcard iSrc = neighbors[0];

		const Vector3f targetPoint = targetPoints[i];

		pMatch->setMatch(iSrc, targetPoint);
	}

	UnstructuredInCorePointCloud* pMatchPC = pMatch->exportAsCompactPointCloud();

	debugOutput << "visualizing\n";
	//visualize
	if (settings->bHighResViz)
	{
		debugOutput << "transferring high-res color by correspondence\n";
		colorTargetByCorrespondenceInternalHighRes(pMatch);
	}
	else
		colorTargetByCorrespondenceInternal(pMatchPC);

	debugOutput << "maybe evaluating error\n";
	//optionally evaluate error
	if (settings->bEvalExternalCorrs)
		evalGroundTruthErrorInternal(pMatch, 9996);

	delete pMatch;
	delete pMatchPC;
}

void PCIGMOD14ShapeMatching::loadExternalResult()
{
	if (settings->extResType == AR4Settings::extResBIM)
		loadMatchesBIM();
	else if (settings->extResType == AR4Settings::extResAvinash)
		loadMatchesAvinash();
	else if (settings->extResType == AR4Settings::extResAurela)
		loadMatchesAurela();
	else if (settings->extResType == AR4Settings::extResArt)
		loadMatchesArt();

	redrawScene();
}

void PCIGMOD14ShapeMatching::visualizeReject()
{
	if (m_pMatch == NULL)
		throw EInvalidState("PCIGMOD14ShapeMatching::colorMatchesByPosition(): null partial match");

	const float threshMulFactor = 1.f;
	const float factor = 1.f / (threshMulFactor * m_pMatch->computeAvgSourceMatchLevelEdgeWeight() * settings->stretchThreshold);
	
	card32 i, j;
	Vector3f targetPoint;
	float stretch;
	vector<mpcard> sourceMeasurePoints;

	float distS, distT, pathStretch;
	Vector3f gradS, gradT;
	vector<Vector3f> pathS, pathT;
	m_pMatch->getRejectInfo(m_rejectToVisualize, targetPoint, stretch, sourceMeasurePoints);
	if (stretch < 0.f)
	{
		debugOutput << "PCIGMOD14ShapeMatching::visualizeReject(): no outlier rejection info found for source index " << m_rejectToVisualize << "\n";
		return;
	}

	//visualize
	debugRenderer->clearDebugData("ARShapeMatching::visualizeReject");
	debugRenderer->beginRenderJob("ARShapeMatching::visualizeReject");
	debugRenderer->addSphere(m_pSourceManifold->getDiscrPoint(m_rejectToVisualize), m_avgDenseEdgeLength, NULL_VECTOR3F);
	debugRenderer->addSphere(targetPoint, m_avgDenseEdgeLength, NULL_VECTOR3F);
	for (i = 0; i < sourceMeasurePoints.size(); i++)
	{

		m_pMatch->getSparseSourceGeodesics()->computeGeodesicSingleDest(m_pMatch->getGeodesicsSigma(), m_rejectToVisualize, sourceMeasurePoints[i], distS, gradS, pathS);
		m_pMatch->getSparseTargetGeodesics()->computeGeodesicSingleDest(m_pMatch->getGeodesicsSigma(), targetPoint, m_pMatch->getMatch(sourceMeasurePoints[i]), distT, gradT, &pathT);
		pathStretch = fabs(distT - distS);
		for (j = 1; j < pathS.size(); j++)
			debugRenderer->addFastCylinder(pathS[j - 1], pathS[j], m_avgDenseEdgeLength * 0.25f, rainbowGradient3f(pathStretch * factor));
		for (j = 1; j < pathT.size(); j++)
			debugRenderer->addFastCylinder(pathT[j - 1], pathT[j], m_avgDenseEdgeLength * 0.25f, rainbowGradient3f(pathStretch * factor));
	}
	debugRenderer->endRenderJob();

	//print
	debugOutput << "Rejected outlier: " << m_rejectToVisualize << " -> " << targetPoint << "\n";
	debugOutput << "measured stretch " << stretch << " wrt the following points ";
	for (i = 0; i < sourceMeasurePoints.size(); i++)
		debugOutput << sourceMeasurePoints[i] << " ";
	debugOutput << "\n";

	redrawScene();
}



	
