#ifndef MLSProjectorH
#define MLSProjectorH
//---------------------------------------------------------------------------
#include "x3foundation_private.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace X4
{

class CachedRangeQuerry;
class PointCloud;

/**
	implements several types of mls projections efficiently
*/
class X3FOUNDATION_PRIVATE_API MLSProjector : public Persistent
{
	X4_CLASS( MLSProjector )

private:

	CachedRangeQuerry	*	m_RangeQuery;
	PointCloud			*	m_PC;
	AAT						m_NormalAAT;
	AAT						m_PositionAAT;
	AAT						m_WeightAAT;
	bool					m_ProvideWeightChannel;
	
	float32					m_MedPointDistance;
	float32					m_Sigma;
	float32					m_SigmaN;
	float32					m_SigmaR;
	card32					m_MinRefittingIterations;
	card32					m_MaxRefittingIterations;
	card32                  m_minNumberOfSamples;

	bool RIMLS_computePotentialAndGradient(const Vector3f &x, Vector3f &gradient, float &potential);

	// temporal variables
	bool			m_LastPointOK;
	float32			m_LastSummedWeight;


public:
	MLSProjector(void);
	~MLSProjector(void);

   virtual void assign(const Object* obj, X4CopyContext *context = NULL);

	void begin(PointCloud * pc, float medPointDistance = 0);

	/// project a single points (exploit spatial coherence whenever possible!)
	Vector3f project(const Vector3f &pos, Vector3f *projDerivate);

	bool lastProjectionValid() const {return m_LastPointOK;}
	float32 lastSummedWeight() const {return m_LastSummedWeight;}

	Vector3f project_SimpleMLS(const Vector3f &pos, Vector3f *projDerivate,  bool* valid = NULL, float* weight = NULL);
	Vector3f project_RIMLS(const Vector3f &pos, Vector3f *projDerivate, bool* valid = NULL);

	float getSigma() const {return m_Sigma;}
	void setSigma( float s ) {m_Sigma = s;}

	void setMedianPointDistance(float s) {m_MedPointDistance = s;}
	void setMinNumberOfSamples(card32 minnum) {m_minNumberOfSamples = minnum;}
};

}

#endif